/*agentMsgs.h,v 1.1 1994/03/21 13:14:54 uchenick Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		CHANNEL ACCESS AGENT MESSAGES			*/
/*								*/
/*	This file defines the messages that are used in the	*/
/*	agent architecture for channel access.			*/
/*								*/
/*	Author:	Gordon Uchenick					*/
/*								*/
/****************************************************************/

/****************************************************************/
/*								*/
/*			MESSAGE STRUCTURE			*/
/*								*/
/*	All messages start with a short. This short declares	*/
/*	the message type. The type of structure that follows	*/
/*	depends on the message type. In some cases, there may	*/
/*	be either variable length fields or a variable number	*/
/*	of instances of other data. The description of each	*/
/*	message type will describe the actual format.		*/
/*								*/
/*	NOTE: All of the structures defined here are padded	*/
/*	out to a multiple of 32 bits. That is done because	*/
/*	some C compilers have the nasty habit of rounding up	*/
/*	the sizeof structures that are an odd number of 16	*/
/*	bit words, just in case you might make an array of	*/
/*	them. Thanks, guys.					*/
/*								*/
/****************************************************************/

#ifndef	agentmsgsH
#define	agentmsgsH

#ifdef	WIN32
#    include <winsock.h>
#else
#    include <netinet/in.h>
#endif	WIN32
#include "tsDefs.h"

/******
	Base numbers for each group
******/

#define	UNIVERSAL_BASE	0x0	/* Messages everyone uses */
#define	SOCKMAINT_BASE	0x20	/* Socket Creation and Maintenance */
#define	CHANCNTRL_BASE	0x40	/* Channel Acquisition and Control */
#define	NAMESERVR_BASE	0x80	/* Name Server */


/******
	MESSAGES THAT APPLY TO EVERYONE
******/

#define	BOOMERANG	(UNIVERSAL_BASE + 0x00)	/* Boomerang for keep alive */
#define DUMP		(UNIVERSAL_BASE + 0x01)	/* Dump your control tables */
#define DEBUG		(UNIVERSAL_BASE + 0x02) /* Toggle your debugging flag */


/******
	SOCKET CREATION AND MAINTENANCE MESSAGES
******/

#define SOCK_WRITE	(SOCKMAINT_BASE + 0x00)	/* Data to Write from CAC */
#define	SOCK_FLUSH	(SOCKMAINT_BASE + 0x01)	/* Flush directive */
#define	SOCK_PARM	(SOCKMAINT_BASE + 0x02)	/* Tuning Param Request */
#define	SOCK_PARM_REPLY	(SOCKMAINT_BASE + 0x03)	/* Tuning Param Reply */
#define SOCK_TUNE	(SOCKMAINT_BASE + 0x04)	/* Socket Parameter Tuning */
#define SOCK_CONN	(SOCKMAINT_BASE + 0x05)	/* Request for TCP Connection */
#define SOCK_CONN_REPLY	(SOCKMAINT_BASE + 0x06)	/* Request for Connection Reply */
#define SOCK_READ	(SOCKMAINT_BASE + 0x07)	/* Data to Read from IOC */
#define SOCK_ALQCONN	(SOCKMAINT_BASE + 0x08) /* Request alarm queue redund conn */
#define SOCK_DPAREG	(SOCKMAINT_BASE + 0x09) /* Register DPA */
#define SOCK_DPAREQ	(SOCKMAINT_BASE + 0x0A)	/* Request DPA by name */
#define SOCK_DPACONN	(SOCKMAINT_BASE + 0x0B)	/* Ask DPA to request to host */


/******
	CHANNEL ACQUISITION AND CONTROL MESSAGES
******/

#define CHAN_IOCUP	(CHANCNTRL_BASE + 0x00)	/* IOC Up */
#define CHAN_IOCDN	(CHANCNTRL_BASE + 0x01)	/* IOC Down */
#define CHAN_RGST	(CHANCNTRL_BASE + 0x02)	/* SCM Registration */
#define CHAN_SESS_SET	(CHANCNTRL_BASE + 0x03)	/* Channel Session Set */
#define CHAN_SESS_INQ	(CHANCNTRL_BASE + 0x04)	/* Channel Session Inquire */
#define CHAN_SESS_REPLY	(CHANCNTRL_BASE + 0x05)	/* Channel Session Response */
#define CHAN_BUILD	(CHANCNTRL_BASE + 0x06)	/* Channel Build and Connect */
#define CHAN_TYPE	(CHANCNTRL_BASE + 0x07)	/* Channel Data Type Change */
#define CHAN_NOTFOUND	(CHANCNTRL_BASE + 0x08)	/* Channel Not Found */
#define CHAN_DIRREAD	(CHANCNTRL_BASE + 0x09)	/* Directed Read */
#define CHAN_DIRPUT	(CHANCNTRL_BASE + 0x0A)	/* Directed Put */
#define CHAN_MONADD	(CHANCNTRL_BASE + 0x0B) /* Add Monitor */
#define CHAN_MONREM	(CHANCNTRL_BASE + 0x0C)	/* Remove Monitor */
#define CHAN_CLOSE	(CHANCNTRL_BASE + 0x0D)	/* Close Channel */
#define CHAN_BLDACK	(CHANCNTRL_BASE + 0x0E)	/* Build Acknowledgement */
#define CHAN_DRRDATA	(CHANCNTRL_BASE + 0x0F) /* Direct Read Data */
#define CHAN_MONDATA	(CHANCNTRL_BASE + 0x10)	/* Monitored Data */
#define CHAN_RESMODE	(CHANCNTRL_BASE + 0x11)	/* Resource Mode Change */
#define CHAN_CONNEVNT	(CHANCNTRL_BASE + 0x12)	/* Channel Connected */
#define CHAN_DISCEVNT	(CHANCNTRL_BASE + 0x13) /* Channel Connection Broken */
#define CHAN_CLOSEACK	(CHANCNTRL_BASE + 0x14) /* Channel Close Acknowledgement */
#define CHAN_MONREQ	(CHANCNTRL_BASE + 0x15) /* Request for immediate monitor */
#define CHAN_SINGMON	(CHANCNTRL_BASE + 0x16) /* Immediate response monitor */
#define CHAN_RESINQ	(CHANCNTRL_BASE + 0x17) /* Resource mode inquiry */
#define CHAN_NEEDNMSRV	(CHANCNTRL_BASE + 0x18)	/* Need connection to nameSrv */
#define CHAN_GOTNMSRV	(CHANCNTRL_BASE + 0x19)	/* Found name server address */
#define CHAN_BLDRDMON	(CHANCNTRL_BASE + 0x1A) /* Channel Build, Read, and Monitor */
#define CHAN_HANDLERS	(CHANCNTRL_BASE + 0x1B)	/* Set alarm & trend handler status */
#define CHAN_GETPARAMS	(CHANCNTRL_BASE + 0x1C)	/* Get channel parameters */
#define CHAN_PARMREPLY	(CHANCNTRL_BASE + 0x1D)	/* Channel Parameter Reply */
#define CHAN_TRDATA	(CHANCNTRL_BASE + 0x1E)	/* Unsolicited Trend data */
#define CHAN_ALARM	(CHANCNTRL_BASE + 0x1F) /* Unsolicited Alarm */
#define CHAN_ALARM_RECV	(CHANCNTRL_BASE + 0x20)	/* Wkstn Acks Receipt of Alarm to IOC */
#define CHAN_ALARM_CTL	(CHANCNTRL_BASE + 0x21)	/* Alarm enable/disable */
#define CHAN_ALARM_ACK	(CHANCNTRL_BASE + 0x22) /* Alarm ACK by operator */
#define CHAN_ALQCONN	(CHANCNTRL_BASE + 0x23) /* Redundant alarm connection made */
#define CHAN_QBUILD	(CHANCNTRL_BASE + 0x24) /* Quick Build */
#define CHAN_QBLDACK	(CHANCNTRL_BASE + 0x25) /* Quick Build Acknowledgement */


/******
	NAME SERVER MESSAGES
******/

#define NAME_SRCH	(NAMESERVR_BASE + 0x00)	/* Find Request */
#define NAME_SRCH_REPLY	(NAMESERVR_BASE + 0x01)	/* Find Reply */
#define NAME_RSRQ	(NAMESERVR_BASE + 0x02)	/* Resource List Request */
#define NAME_RSRQ_REPLY	(NAMESERVR_BASE + 0x03)	/* Resource List Reply */
#define NAME_IOCL	(NAMESERVR_BASE + 0x04)	/* IOC List Request */
#define NAME_IOCL_REPLY	(NAMESERVR_BASE + 0x05)	/* IOC List Reply */
#define NAME_PROBE	(NAMESERVR_BASE + 0x06)	/* IOC search for nameSrv */
#define NAME_DPAL	(NAMESERVR_BASE + 0x07)	/* DPA List Request */
#define NAME_DPAL_REPLY (NAMESERVR_BASE + 0x08)	/* DPA List Reply */
#define NAME_FILEMOD	(NAMESERVR_BASE + 0x09) /* Database File Mod Notification */


/*****************************************************************/
/*****************************************************************/
/**								**/
/**		MESSAGES THAT APPLY TO EVERYONE			**/
/**								**/
/*****************************************************************/
/*****************************************************************/


/*		       *************				*/
/*********************** BOOMERANG ******************************/
/*		       *************				*/

/******
	The Boomerang. This message is used for heartbeats so
	that we know if TCP/IP links are still alive. We use our
	own mechanism because we can tune it for every link and
	we can also change our minds without having to remake
	kernels.

	This message has no following structure. Whoever receives
	this message code it should just send it back immediately.
******/

/*			********				*/
/************************ DUMP **********************************/
/*			********				*/

/******
	The Dump. Any process that receives this message is to dump
	all of its internal control tables to a data file in its
	current directory.

	This message has no following structure.
******/


/*****************************************************************/
/*****************************************************************/
/**								**/
/**		SOCKET CREATION AND MAINTENANCE MESSAGES	**/
/**								**/
/*****************************************************************/
/*****************************************************************/


/*			**************				*/
/************************ SOCK_WRITE ****************************/
/*			**************				*/

/******
	This is data to be written to a socket. The header contains
	the destination Inet Address and the length.
******/

typedef struct
    {
    struct in_addr smInet;		/* Destination Inet Address */
    short	smLength;	/* Message length */
    short	smFill;		/* Pad to 32 bit boundary */
    } SM_WRITE;

/******
	MESSAGE TRAILER:
		This message is followed by the data to be written.
		The number of bytes is in smLength. A zero length
		message is illegal. Socket Creation and Maintenance
		puts no interpretation on the data.
******/


/*			**************				*/
/************************ SOCK_FLUSH ****************************/
/*			**************				*/

/******
	This message causes Socket Creation and Maintenance to flush
	all of its buffers.

	This message has no following structure.
******/


/*			*************				*/
/************************ SOCK_PARM *****************************/
/*			*************				*/

/******
	This message is a request for Socket Creation and Maint-
	enance to send its complete list of socket tuning para-
	meters back to the requester.

	This message has no following structure.
******/


/*			*******************			*/
/************************ SOCK_PARM_REPLY ***********************/
/*			*******************			*/

/******
	This message consists of a header that declares the number
	of sockets that will be reported.
******/

typedef struct
    {
    short	smNumSockets;
    short	smFill;		/* Pad to 32 bit boundary */
    } SM_PARM_REPLY;

/******
	MESSAGE TRAILER:
		The header is followed by smNumSockets instances
		of struct sockTune. See <sockTune.h> for details of that
		structure.
******/


/*			*************				*/
/************************ SOCK_TUNE *****************************/
/*			*************				*/

/******
	This message is used to send revised tuning parameters to
	Socket Creation and Maintenance. The header declares the
	number of sockets that are to be tuned.
******/

typedef struct
    {
    short	smNumSockets;
    short	smFill;		/* Pad to 32 bit boundary */
    } SM_TUNE;

/******
	MESSAGE TRAILER:
		The header is followed by smNumSockets instances
		of struct sockTune. See <sockTune.h> for details of that
		structure.
******/


/*			*************				*/
/************************ SOCK_CONN *****************************/
/*			*************				*/

/******
	This message is broadcast by an IOC when it first comes to
	life. It is used so that we don't have to wait for the
	workstation's Socket Creation and Maintenance to get around
	to initiating the connection on a timed basis.

	This message has no following structure.
******/


/*			*******************			*/
/************************ SOCK_CONN_REPLY ***********************/
/*			*******************			*/

/******
	Socket creation and maintenance sends this message in reply
	to a SOCK_CONN message. Also, we will periodically send this
	message to an IOC that isn't currently connected in an
	attempt to get him to connect to us.

	This message has no following structure.
******/


/*			*************				*/
/************************ SOCK_READ *****************************/
/*			*************				*/

/******
	This message contains data that was transmitted from the
	IOC to be interpreted by Channel Acquisiton and Control.
	By having it read by Socket Creation and Maintenance we
	give up an extra context switch for each message, but we
	get capacity for more channels because we use an independent
	program's file table as well as a simpler, more reliable
	programming environment.

	Note that the IOC does not fill in his own Inet number.
	Socket Creation and Maintenance fills that in when he
	forwards the data to Channel Acquisition and Control.
******/

typedef struct
    {
    struct in_addr smInet;		/* Inet addr, not filled in by IOC */
    short	smLength;	/* Length of following message */
    short	smFill;		/* Pad to 32 bit boundary */
    } SM_READ;

/******
	MESSAGE TRAILER:
		This message is followed by the number of bytes
		declared in smLength. This data is to be treated
		transparently.
******/


/*			****************			*/
/************************ SOCK_ALQCONN **************************/
/*			****************			*/

/******
	This message is broadcast from sockMnt in one IOC to rsrvTask
	in redundant partner IOCs. It indicates that the sending IOC
	needs a redundant alarm queue connection.

	This message has no following structure.
******/


/*			***************				*/
/************************ SOCK_DPAREG ***************************/
/*			***************				*/

/******
	This message is sent from a DPA's rsrv child to sockMnt.
	It informs sockMnt that a virtual Inet address is now
	online.
******/

typedef struct
    {
    short	dpaPrefixLen;	/* Including the NULL! */
    short	filler;
    } SM_DPA;

/*******
	MESSAGE TRAILER:
		This message is followed by the DPA prefix. The
		length of the name in the header includes the
		null.
******/


/*			***************				*/
/************************ SOCK_DPAREQ ***************************/
/*			***************				*/

/******
	This message is sent from one sockMnt to another, requesting
	that the receiving sockMnt have the DPA application connect
	to the sending sockMnt. This message is sent via UDP.
******/

typedef struct
    {
    char	dpaPrefix[16];
    } SM_DPAREQ;


/*			****************			*/
/************************ SOCK_DPACONN **************************/
/*			****************			*/

/******
	This message is sent from sockMnt to a DPA on its local node.
	It requests that the DPA register with sockMnt on the given
	node. The Internet address in the message is always in
	network byte order. This message is sent by TCP.
******/

typedef struct
    {
    struct in_addr dpaInet;
    } SM_DPACONN;


/*****************************************************************/
/*****************************************************************/
/**								**/
/**		CHANNEL ACQUISTION AND CONTROL MESSAGES		**/
/**								**/
/*****************************************************************/
/*****************************************************************/


/*			**************				*/
/************************ CHAN_IOCUP ****************************/
/*			**************				*/

/******
	Socket Creation and Maintenance transmits this message to
	Channel Acquisition and Control to inform him that a new
	socket has just been established with an IOC.
******/

typedef struct
    {
    struct in_addr chInet;
    } CH_IOCUP;


/*			**************				*/
/************************ CHAN_IOCDN ****************************/
/*			**************				*/

/******
	Socket Creation and Maintenance transmits this message to
	Channel Acquisition and Control to inform him that the
	socket to an IOC has just failed and has been closed.
******/

typedef struct
    {
    struct in_addr chInet;
    } CH_IOCDN;


/*			*************				*/
/************************ CHAN_RGST *****************************/
/*			*************				*/

/******
	Socket Creation and Maintenance transmits this message to
	Channel Acquisition and Control to inform him that he is
	up and ready to start I/O services.

	This message has no following structure.
******/


/*			*****************			*/
/************************ CHAN_SESS_SET *************************/
/*			*****************			*/

/******
	Channel Acquisition and Control transmits this message
	to the IOC to set the session number. This session number
	is used to recover from broken sockets without having to
	remake every channel every time. If the IOC receives this
	message from a workstation with a different session number
	than it had previously, then it is to drop all channel
	connections with that workstation.
******/

typedef struct
    {
    int		chSession;
    } CH_SESS_SET;


/*			*****************			*/
/************************ CHAN_SESS_INQ *************************/
/*			*****************			*/

/******
	Channel Acquisition and Control transmits this message to
	the IOC to get the session number. If they match, we can
	pick up where we left off. If not, then every channel has
	to be dumped and rebuilt.
******/

typedef struct
    {
    int		chSession;
    } CH_SESS_INQ;


/*			*******************			*/
/************************ CHAN_SESS_REPLY ***********************/
/*			*******************			*/

/******
	When asked by the workstation, the IOC responds with this
	message to report his session ID. If the numbers match with
	the workstation's session ID, then we can pick up where we
	left off. If not, then we have to trash all previous channels
	we may have had with this IOC/workstation and start over.
******/

typedef struct
    {
    int		chSession;
    } CH_SESS_REPLY;


/*			**************				*/
/************************ CHAN_BUILD ****************************/
/*			**************				*/

/******
	An application sends this message to Channel Acquisition and
	Control to open a channel.
******/

typedef struct
    {
    int		chHandle;
    short	chType;
    short	chCount;
    short	chNameLen;
    short	shFill;		/* Pad to 32 bit boundary */
    } CH_BUILD;


/******
	MESSAGE TRAILER:
		This message is followed by the channel name. The number
		of characters (including the null!) is contained in
		chNameLen.
******/


/*			*************				*/
/************************ CHAN_TYPE *****************************/
/*			*************				*/

/******
	Channel Acquisition and Control sends this message to
	the IOC to change the data type, count, and mask of a channel.
******/

typedef struct
    {
    int		chHandle;
    short	chType;
    short	chCount;
    short	chMask;
    short	chFill;		/* Pad to 32 bit boundary */
    } CH_TYPE;


/*			*****************			*/
/************************ CHAN_NOTFOUND *************************/
/*			*****************			*/

/******
	Channel Acquisition and Control sends this message to
	an application to inform him that the name server doesn't
	know about a channel that was requested.
******/

typedef struct
    {
    int		chHandle;
    } CH_NOTFOUND;


/*			****************			*/
/************************ CHAN_DIRREAD **************************/
/*			****************			*/

/******
	Channel Acquisition and Control sends this message to the
	IOC to have it read a channel immediately and return the
	appropriate data. An application also sends this data to
	Channel Acquisition and Control. The chCaHandle element
	is only used for workstation -> IOC transmissions.
******/

typedef struct
    {
    int		chHandle;
    short	chDataType;
    short	chNumElem;
    int		chCaHandle;
    int		chAppHandle;
    } CH_DIRREAD;


/*			***************				*/
/************************ CHAN_DIRPUT ***************************/
/*			***************				*/

/******
	Channel Acquisition and Control sends this message to the
	IOC to cause it to write data to a channel. An application
	also sends this message to Channel Acquisition and Control.
******/

typedef struct
    {
    int		chHandle;
    short	chDataType;
    short	chDataSize;
    int		chCaHandle;
    } CH_DIRPUT;

/******
	MESSAGE TRAILER:
		This message is followed by the data to write. The
		number of bytes is chDataSize.
******/


/*			***************				*/
/************************ CHAN_MONADD ***************************/
/*			***************				*/

/******
	Channel Acquisition and Control sends this message to the
	IOC to add a monitor to a channel. An application also sends
	this message to Channel Acquisiton and Control.
******/

typedef struct
    {
    int		chHandle;
    short	chType;
    short	chCount;
    short	mask;
    short	filler;
    } CH_MONADD;


/*			***************				*/
/************************ CHAN_MONREM ***************************/
/*			***************				*/

/******
	Channel Acquisition and Control sends this message to the
	IOC to remove a monitor from a channel. An application also
	sends this message to Channel Acquisition and Control.
******/

typedef struct
    {
    int		chHandle;
    } CH_MONREM;


/*			**************				*/
/************************ CHAN_CLOSE ****************************/
/*			**************				*/

/******
	Channel Acquisition and Control sends this message to the
	IOC to have it delete a channel. An application also sends
	this message to Channel Acquisition and Control. Both
	handles are included just in case the application decides
	to close the channel before it is completely built.
******/

typedef struct
    {
    int		chHandle;
    int		chAppHandle;
    } CH_CLOSE;


/*			***************				*/
/************************ CHAN_BLDACK ***************************/
/*			***************				*/

/******
	This message is sent from the IOC to Channel Acquisiton and
	Control. It is the response to a CHAN_BUILD message. It
	contains the fundamental data type and element count. It
	also contains the state of that channels resource.
******/

typedef struct
    {
    int		chHandle;		/* Workstation's handle */
    int		chIocHandle;		/* IOC's handle */
    short	currState;		/* Current resource state */
    short	chDataType;		/* Data type, old codes */
    short	chElementCount;		/* Element count */
    short	filler;
    } CH_BLDACK;


/*			****************			*/
/************************ CHAN_DRRDATA **************************/
/*			****************			*/

/******
	This message is sent from the IOC to Channel Acquisition and
	Control. It is the response to a CHAN_DIRREAD message.
******/

typedef struct
    {
    int		chHandle;
    int		chCaHandle;
    short	chDataType;
    short	chDataLen;	/* 0 => I don't have the resource */
    short	chCount;
    short	filler;
    } CH_DRRDATA;

/******
	MESSAGE TRAILER:
		This message is followed by the requested data. Its length
		is chDataLen.
******/


/*			****************			*/
/************************ CHAN_MONDATA **************************/
/*			****************			*/

/******
	This message is sent from the IOC to Channel Acquisition and
	Control. It contains data that is reported as the result of
	montors that have been put on the channel.
******/

typedef struct
    {
    int		chHandle;
    short	chDataType;
    short	chDataLen;
    short	chMask;
    short	chCount;
    } CH_MONDATA;

/******
	MESSAGE TRAILER:
		This message is followed by the monitored data. It's length
		is chDataLen.
******/


/*			****************			*/
/************************ CHAN_RESMODE **************************/
/*			****************			*/

/******
	This message is sent from the IOC to Channel Acquisition and
	Control. It informs the workstation about a change of state
	for a resource in the IOC.
******/

typedef struct
    {
    char	chResName[18];
    short	chResStatus;
    } CH_RESMODE;


/*			*****************			*/
/************************ CHAN_CONNEVNT *************************/
/*			*****************			*/

/******
	Channel Acquisition and Control sends this message to an
	application to inform it that it has made a connection
	with an IOC.
******/

typedef struct
    {
    int		chHandle;
    } CH_CONNEVNT;


/*			*****************			*/
/************************ CHAN_DISCEVNT *************************/
/*			*****************			*/

/******
	Channel Acquisition and Control sends this message to an
	application to inform it that it has lost the connection
	with an IOC.
******/

typedef struct
    {
    int		chHandle;
    } CH_DISCEVNT;


/*			*****************			*/
/************************ CHAN_CLOSEACK *************************/
/*			*****************			*/

/******
	The IOC sends this message to Channel Acquisition and Control
	to inform him that he has closed a channel and freed any
	resources in the IOC that were associated with it.
******/

typedef struct
    {
    int		chHandle;
    } CH_CLOSEACK;


/*			***************				*/
/************************ CHAN_MONREQ ***************************/
/*			***************				*/

/******
	Channel Acquisition and Control sends this message to the IOC
	to have it create an immediate monitor for the point. This is
	done when an application first places a monitor on a channel.
******/

typedef struct
    {
    int		chHandle;
    int		chCaHandle;
    short	chType;
    short	chCount;
    short	mask;
    short	filler;
    } CH_MONREQ;


/*			****************			*/
/************************ CHAN_SINGMON **************************/
/*			****************			*/

/******
	This is the proper response to a CHAN_MONREQ. This is the
	same thing as a normal monitor except that we identify a
	unique application. The normal monitor applies to all
	applications on a channel.
******/

typedef struct
    {
    int		chHandle;
    int		chCaHandle;
    short	chDataType;
    short	chDataLen;
    short	chMask;
    short	chCount;
    } CH_SINGMON;


/*			***************				*/
/************************ CHAN_RESINQ ***************************/
/*			***************				*/

/******
	Channel Acquisition and Control sends this message to an IOC
	when a connection is established and the session numbers match.
	When the IOC receives this message, it generates a CHAN_RESMODE
	message for each resource that it contains. The reason for this
	transaction is to inform the workstation about mode changes
	that may have happened while the connection was disrupted.

	This message has no following structure.
******/


/*			******************			*/
/************************ CHAN_NEEDNMSRV ************************/
/*			******************			*/

/******
	Channel Acquisition and Control sends this message to Socket
	Creation and Maintenance to inform him that he needs a connection
	to a name server.

	This message has no following structure.
******/


/*			*****************			*/
/************************ CHAN_GOTNMSRV *************************/
/*			*****************			*/

/******
	Socket Creation and Maintenance sends this message to Channel
	Acquisition and Control to let him know that a name server
	has been found. Note that the Inet address will be sent in
	network byte order! Bytes should not be flipped on little
	endian machines!
******/

typedef struct
    {
    struct in_addr chInet;
    } CH_GOTNMSRV;


/*			*****************			*/
/************************ CHAN_BLDRDMON *************************/
/*			*****************			*/

/******
	The application sends this message to chanAcq who then
	forwards it onto the IOC to build a channel, do a read
	for the control variant of the native data type, and then
	also establish a monitor for the status variant of the
	native data type. The IOC responds with three messages:
	First a CHAN_BLDACK, then a CHAN_DRRDATA, and finally
	a CHAN_SINGMON.
******/

typedef struct
    {
    int		chHandle;	/* chanAcq's CHB address */
    int		chCaHandle;	/* chanAcq's CAB address */
    int		chAppHandle;	/* chanAcq's ACB address */
    short	chNameLen;	/* Length of Channel Name */
    short	filler;
    } CH_BLDRDMON;

/******
	MESSAGE TRAILER:
		This message is followed by the channel name. The number
		of characters (including the null!) is contained in
		chNameLen.
******/


/*			*****************			*/
/************************ CHAN_HANDLERS *************************/
/*			*****************			*/

/******
	chanAcq sends this message to the IOC to give him the
	state of the trend and alarm handler flags for his
	workstation. These flags are boolean, so there is no
	need to worry about endian issues. Applications also
	send this message to chanAcq to indicate that they are
	a trend and/or alarm handler.
******/

typedef struct
    {
    short	chTrendHandler;	/* Workstation handles trends */
    short	chAlarmHandler;	/* Workstation handles alarms */
    } CH_HANDLERS;


/*			******************			*/
/************************ CHAN_GETPARAMS ************************/
/*			******************			*/

/******
	chanAcq sends this message to the IOC to get the parameters
	for a channel. In redundant configurations, it is sent only
	to the master IOC.
******/

typedef struct
    {
    short	chParmType;	/* Type desired */
    short	chNameLen;	/* Length of the name */
    int		chArgument;	/* Argument (end user's handle) */
    int		chACB;		/* ACB pointer for return */
    } CH_GETPARAMS;

/******
	MESSAGE TRAILER:
		This message is followed by the channel name. The number
		of characters (including the null!) is contained in
		chNameLen.
******/


/*			******************			*/
/************************ CHAN_PARMREPLY ************************/
/*			******************			*/

/******
	The IOC sends this message back to chanAcq to give him the
	parameters that he requested.
******/

typedef struct
    {
    short	chParmType;	/* data type */
    short	chDataLen;	/* Data Lenth */
    int		chArgument;	/* Argument (end user's handle) */
    int		chACB;		/* ACB pointer for return */
    } CH_PARMREPLY;

/******
	MESSAGE TRAILER:
		This message is followed by the stream of data that
		is necessary to make up the dbr_xxx_param structures.
		The stream is designed to keep the traffic as small
		as possible. It is parsed by the routines in libca.a.
******/


/*			***************				*/
/************************ CHAN_TRDATA ***************************/
/*			***************				*/

/*******
	This message contains data that is to sent to the trend
	handler as unsolicited data.
******/

typedef struct
    {
    short	chNameLen;	/* Channel name length */
    short	chDataLen;	/* Length of data */
    short	chDataType;	/* Data type, agent encoding */
    short	filler;
    } CH_TRDATA;

/******
	MESSAGE TRAILER:
		This message is followed by the channel name (null
		terminated, # chars including the null in chNameLen).
		It is also followed by a data stream that is the timed
		variant of the native data type and count.
******/


/*			**************				*/
/************************ CHAN_ALARM ****************************/
/*			**************				*/

/******
	This message contains an alarm message that is to be sent
	to all registered alarm handlers as unsolicited data.
******/

typedef struct
    {
    struct
	{
	int	secPastEpoch;
	int	nsec;
	}	alarmTime;
    int		alarmSerial;
    int		alarmHandle;
    short	alarmStat;
    short	alarmSevr;
    short	alarmAck;
    short	alarmEnabled;
    short	alarmOnscan;
    short	alarmDataLen;
    } CH_ALARM;

/******
	MESSAGE TRAILER:
		This message is followed by a data stream which is
		alarmDataLen bytes long. This data stream consists
		of the null terminated point name followed by the
		null terminated descriptor followed by the null
		terminated value followed by the null terminated
		EGU tag.
******/


/*			******************			*/
/************************ CHAN_ALARM_ACK ************************/
/*			******************			*/

/******
	This message is from a workstation and tells the remote
	Channel Access server that a human operator has ack'd the
	an alarm. This message only serves to clear the ACK field
	in the record. It does NOT communicate the human ack to
	other workstations in the system. That is the work station
	alarm handler's task.
******/

typedef struct
    {
    struct
	{
	int	secPastEpoch;
	int	nsec;
	}	alarmTime;	/* Time of alarm being ack'd */
    short	alarmNameLen;	/* Length of name, including null */
    short	filler;
    } CH_ALARM_ACK;

/******
	MESSAGE TRAILER:
		This message is followed by the name of the point.
		The length of the name, including the null, is
		in alarmNameLen.
******/


/*			******************			*/
/************************ CHAN_ALARM_CTL ************************/
/*			******************			*/

/******
	This message is used to enable or disable alarming for
	a single point.
******/

typedef struct
    {
    short	alarmCtlFunc;	/* 0 = alarm disable, else enable */
    short	alarmNameLen;	/* Length of name, including null */
    } CH_ALARM_CTL;

/******
	MESSAGE TRAILER:
		This message is followed by the name of the point.
		The length of the name, including the null, is in
		alarmNameLen.
******/


/*			*******************			*/
/************************ CHAN_ALARM_RECV ***********************/
/*			*******************			*/

/******
	This message is sent from the workstation alarm handler
	to the remote Channel Access server to indicate that it
	has received and written an alarm to the log file. It does
	NOT indicate in any way that a human operator has seen
	this alarm!
******/

typedef struct
    {
    int		alarmSerial;
    int		alarmHandle;
    struct
	{
	int	secPastEpoch;
	int	nsec;
	}	alarmTime;
    short	alarmNameLen;
    short	filler;
    } CH_ALARM_RECV;


/******
	MESSAGE TRAILER:
		This message is followed by the name of the point.
		The length of the name, including the null, is in
		alarmNameLen.
******/

/*			****************			*/
/************************ CHAN_ALQCONN **************************/
/*			****************			*/

/******
	This message is sent from one rsrvTask to its redundant
	partner. Only the end that performs the connect sends the
	message.

	This message has no following structure.
******/


/*			*************				*/
/************************ CHAN_QBLD *****************************/
/*			*************				*/

/******
	This message is sent from ChanAcq to the IOC to create a
	channel. The only response that is expected other than the
	IOC's handle is the status and severity of the point, the
	resource state and the current value in the native type of
	the field.
******/

typedef struct
    {
    int		chHandle;	/* chanAcq's CHB address */
    int		chCaHandle;	/* chanAcq's CAB address */
    short	chNameLen;	/* Length of name */
    short	filler;		/* Boundary alignment */
    } CH_QBLD;

/******
	MESSAGE TRAILER:
		This message is followed by the name of the point.
		The length of the name, including the null, is in
		chNameLen.
******/


/*			****************			*/
/************************ CHAN_QBLDACK **************************/
/*			****************			*/

/******
	This message is the response to a CHAN_QBLD. It is sent
	from both the master and the slave. The slave does not
	respond with any data in the trailer.
******/

typedef struct
    {
    int		chHandle;	/* chanAcq's CHB address */
    int		chCAHandle;	/* chanAcq's CAB address */
    int		chIocHandle;	/* IOC's handle */
    short	currState;	/* Current resource state */
    short	chDataLen;	/* Length of following data */
    } CH_QBLDACK;

/******
	MESSAGE TRAILER:
		This message is followed by the status, severity,
		and the value of the point. The value is in the
		native type for the field. Only the master IOC
		responds with data. The slave sets chDataLen to
		zero.
******/




/*****************************************************************/
/*****************************************************************/
/**								**/
/**			NAME SERVER MESSAGES			**/
/**								**/
/*****************************************************************/
/*****************************************************************/


/*			*************				*/
/************************ NAME_SRCH *****************************/
/*			*************				*/

/******
	Name Search. The user of this service sends a channel name
	and a handle. The name must be null terminated. The channel
	name can have a field suffix which will be silently ignored.
	The server responds with a NAME_REPLY message.
******/

typedef struct
    {
    int		nsHandle;	/* Opaque to the Name Server */
    short	nsNameLen;	/* Length of name, including null */
    short	nsFill;		/* Pad to 32 bit boundary */
    } NS_SRCH;

/******
	MESSAGE TRAILER:
		This message is followed by the channel name. The number
		of characters (including the null!) is contained in
		nsNameLen.
******/


/*			*******************			*/
/************************ NAME_SRCH_REPLY ***********************/
/*			*******************			*/

/******
	Name Search Reply. Name Server sends back the handle that
	was contained in the request so that the caller can figure
	out what he got. The reply message contains the number of
	nodes that can possibly contain the channel, 0 if the
	channel was not found in the database. The resource name
	is also part of the reply, transmitted as a trailer.
******/

typedef struct
    {
    int			nsHandle;	/* Returned to originator */
    unsigned int	nsCreationTime;	/* Record creation time */
    unsigned short	nsChecksum;	/* Record CRC */
    unsigned short	nsPointType;	/* Point type index */
    short		nsNumNodes;	/* # possible nodes */
    short		nsResNameLen;	/* Length of resource name, including null */
    } NS_SRCH_REPLY;

/******
	MESSAGE TRAILER:
		This message is followed by the number of 32 bit integers
		indicated in nsNumNodes. Each of these integers is an
		Internet address. The resource name follows the Internet
		addresses. Its length is nsResNameLen, including the null.
		If nsNumNodes is zero, then no Inet addresses are transmitted
		and no resource name is transmitted either, regardless
		of what may be in nsResNameLen.
******/


/*			*************				*/
/************************ NAME_RSRQ *****************************/
/*			*************				*/

/******
	Resource List Request. This message is transmitted to the
	Name Server to ask it to send back the list of resource
	names.

	This message has no following structure.
******/


/*			*******************			*/
/************************ NAME_RSRQ_REPLY ***********************/
/*			*******************			*/

/******
	Resource List Reply. Name server sends back the resource
	names. The header contains the number of names and the
	size of each name. Note that if nsNumNames is zero then
	the resource list couldn't be accessed and that no names
	are transmitted.
******/

typedef struct
    {
    short	nsNumNames;		/* Number of Names */
    short	nsMaxNameLen;		/* Max name length, including the null */
    } NS_RSRQ_REPLY;

/******
	MESSAGE TRAILER:
		The name list follows the header structure. Each name
		is null terminated and occupies nsMaxNameLen bytes.
		Each name is null terminated. We don't compress this
		particular transmission because each node will only
		use this one time. Each name is then followed by the
		two Internet addresses of the IOCs that contain that
		resource. A zero address indicates that the address
		is not applicable. The total number of bytes that is
		transmitted is nsNumNames * (nsMaxNameLen + 2 * sizeof(int)).
******/


/*			*************				*/
/************************ NAME_IOCL *****************************/
/*			*************				*/

/******
	IOC List Request. This is transmitted to the Name Server to
	ask it to send back the list of IOCs known to the system.

	This message has no following structure.
******/


/*			*******************			*/
/************************ NAME_IOCL_REPLY ***********************/
/*			*******************			*/

/******
	IOC List Reply. Name server sends back the list of IOCs
	known to the system. The header contains the number of
	names. Note that if the number of nodes is zero, then the
	IOC list can't be accessed and no Internet addresses are
	transmitted.
******/

typedef struct
    {
    short	nsNumNodes;
    short	nsFill;		/* Pad to 32 bit boundary */
    } NS_IOCL_REPLY;

/******
	MESSAGE TRAILER:
		The list of Internet addresses for each node
		follow the header. Each address is a 32 bit
		integer.
******/


/*			**************				*/
/************************ NAME_PROBE ****************************/
/*			**************				*/

/******
	Name Probe. When sockMnt is running in the IOC, it sends
	this message to all of its potential name servers via the
	UDP port. The name server returns this message to let the
	IOC know that it has found a name server and should try to
	connect to it. sockMnt also sends this message to chanAcq
	when a connection to nameSrv is established so that he will
	know to retry all the messages in his queue.

	This message has no following structure.
******/

/*			*************				*/
/************************ NAME_DPAL *****************************/
/*			*************				*/

/******
	DPA (Data Provider Application) List Request. This message
	is transmitted to the name server to get the list of DPA
	prefixes and the virtual Inet addresses that have been 
	assigned to each of them.

	This message has no following structure.
******/


/*			*******************			*/
/************************ NAME_DPAL_REPLY ***********************/
/*			*******************			*/

/******
	DPA List Request Reply. The name server sends back the list
	of DPAs known to the sytsem. The header declares the number
	of DPAs and the number of bytes in each name.
******/

typedef struct
    {
    short	nsNumDPAs;
    short	nsMaxNameLen;
    } NS_DPAL_REPLY;

/******
	MESSAGE TRAILER:
		The name list follows the header structure. Each name
		is null terminated and occupies nsMaxNameLen bytes.
		Each name is null terminated. We don't compress this
		particular transmission because each node will only
		use this one time. The two virtual Inet addresses
		follow the name. A zero indicates that the corresponding
		Inet address is not used.  The total number of bytes that 
		is transmitted is nsNumNames * (nsMaxNameLen + 2 * sizeof(int)).
******/

/*			****************			*/
/************************ NAME_FILEMOD **************************/
/*			****************			*/

/******
	Database File Mod Notification. This message is sent to the name
	server by anyone who modifies the database form, namely a mod
	to RESLIST, IOCLIST, or adds or deletes to PVMAP. The mods to
	PVMAP should not have to be notified, but problems with the
	Codebase library indicate that we should close and reopen the
	file when a significant number of records have been either
	added or deleted. There is no telling what a significant number
	is, so we'll do this every time a record is added or deleted.
******/

typedef struct
    {
    short	nsFileResList;		/* Reslist modified */
    short	nsFileIocList;		/* Ioclist modified */
    short	nsFilePvMap;		/* PvMap modified */
    } NS_FILE_MOD;

#endif	agentmsgsH
