/**********
*
*	File:	ahdlcstats.h
*	Author:	William H. Szostak
*	Date:	June 12, 1995
*
*	This file contains the structures for the ahdlc communication statistics
*	which can be accessed via TIS4000 database points.
*
**********/

typedef struct
{
	unsigned int	line_active;	/* Is the line active / enabled */
	unsigned int	line_responding;	/* Is someone on this line responding */
	unsigned int	master_slave;	/* 0 = slave, 1 = master */
	unsigned int	partner_alive;	/* 0 = false, 1 = true */
	unsigned int	compression;	/* 0 = disabled, 1 = enabled */
	unsigned int	ipackets;	/* number of packets received */
	unsigned int	opackets;	/* number of packets transmitted */
	unsigned int	ierrors;	/* number of receive errors */
	unsigned int	oerrors;	/* number of transmit errors */
	unsigned int	txbufs;		/* number of transmit buffers allocated */
	unsigned int	rxbufs;		/* number of receive buffers allocated */
	unsigned int	txbufs_avail;	/* number of transmit buffers available */
	unsigned int	rxbufs_avail;	/* number of receive buffers available */
	unsigned int	txbuf_size;	/* size of transmit buffers */
	unsigned int	rxbuf_size;	/* size of receive buffers */
	unsigned int	txframes;	/* number of hdlc frames transmitted */
	unsigned int	txtimeouts;	/* number of hdlc frames with transmit timeout */
	unsigned int	rxframes;	/* number of hdlc frames received */
	unsigned int	rxtimeouts;	/* number of hdlc frames with receive timeout */
	unsigned int	rxhwerrors;	/* number of hdlc frames with receive hardware errors */


} ahLineStats;

typedef struct
{
	unsigned int	online;
	unsigned long	inet_addr;
	unsigned int	online_count;
	unsigned int	crc_errors;
	unsigned int	frame_errors;
	unsigned int	format_errors;
	unsigned int	rxhw_errors;
	unsigned int	ipackets;
	unsigned int	opackets;
	unsigned int	ierrors;
	unsigned int	oerrors;
	unsigned int	txframes;	/* number of hdlc frames transmitted */
	unsigned int	txtimeouts;	/* number of hdlc frames with transmit timeout */
	unsigned int	rxframes;	/* number of hdlc frames received */
	unsigned int	rxtimeouts;	/* number of hdlc frames with receive timeout */
	unsigned int	rxhwerrors;	/* number of hdlc frames with receive hardware errors */


} ahUnitStats;

#ifndef MAXLINES
#define MAXLINES 64
#endif

typedef struct
{
	unsigned int	data_valid [MAXLINES];
	ahLineStats	linestats [MAXLINES];
	ahUnitStats	unitstats [MAXLINES] [32];
} ahDevStats;
