/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************************
*
* Description:	Header file for AVL tree module.
*
****************************************************************************/

#ifndef	_AVL_H_
#define	_AVL_H_

typedef	int	bool;
#define	false	0
#define	true	1

/*---------------------- Macros and type definitions ----------------------*/

typedef struct AVL_BUCKET
{
  struct AVL_BUCKET	*left;	/* Pointer to left subtree */
  struct AVL_BUCKET	*right;	/* Pointer to right subtree */
  short			bal;	/* Balance factor */
  char			pad[6];	/* pad bytes to force alignment on alphas GCT 30-May-96 */
} AVL_BUCKET;

typedef struct {
	int		count;		/* Number of elements currently in tree	*/
	int		(*cmp)(void*,void*);	/* Compare two nodes */
	AVL_BUCKET	*root;		/* Pointer to root node of AVL tree */
} AVL_TREE;

/* The three traversal orders supported	*/

#define	PREORDER	0
#define	INORDER		1
#define	POSTORDER	2
#define	MAXLEVEL	64		/* Maximum levels for avl_print */

/* The three balance factors stored in each subtree */

#define	LH		0		/* Left subtree is larger */
#define	BAL		1		/* Subtree is balanced */
#define	RH		2		/* Right subtree is balanced */

/* Return a pointer to the user space given the address of the header of a node. */

#define	AVL_USERSPACE(h)	((void*)((AVL_BUCKET*)(h) + 1))

/* Return a pointer to the header of a node, given the address of the user space. */

#define	AVL_HEADER(n)		((AVL_BUCKET*)(n) - 1)

#define	AVL_EMPTY(t)		((t)->count == 0)

/*-------------------------- Function Prototypes --------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

void	*avl_newnode (int size);
void	avl_freenode (void *node);
AVL_TREE *avl_init (int (*cmp_function) (void *, void *));
void	avl_empty (AVL_TREE *t, void (*freeNode) (void *));
void	*avl_insert (AVL_TREE *tree, void *node);
void	*avl_delete (AVL_TREE *tree, void *node);
void	avl_traverse (AVL_TREE *tree, int order, void (*visit) (void *, void*), void *params);
void	avl_print (FILE *out, AVL_TREE *tree, void (*prnt) (FILE *, void*), bool graph_chars);
void	*avl_findsym (AVL_TREE *tree, void *node);
void	avl_range (AVL_TREE *tree, void *lower, void *upper, 
		   void (*visit) (void *, void*), void *params);
void	*avl_delmin (AVL_TREE *tree);
void	*avl_delmax (AVL_TREE *tree);

#ifdef __cplusplus
}
#endif

#endif
