/*caContext.h,v 1.1 1994/03/21 13:16:52 uchenick Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		TASK CONTEXT FOR CHANNEL ACCESS			*/
/*								*/
/*	This structure keeps track of all the information	*/
/*	that is unique to any single task that uses channel	*/
/*	access. In Unix environments, there is only one		*/
/*	instance of this structure per program. In vxWorks,	*/
/*	there is one instance per task that uses channel	*/
/*	access.							*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifndef INCLcacontext
#define INCLcacontext

#ifdef vxWorks
#include <vxWorks.h>
#include <semLib.h>
#endif
#ifdef WIN32
#include <sys/types.h>
#include <winsock.h>
#else
#include <types.h>
#endif WIN32
#include <cadef.h>

typedef struct catx
    {
    int		caInitialized;			/* This task initialized? */
    int		caOutputQueued;			/* # Bytes in caOutputBuff */
    int		caSocketFd;			/* Our connection with chanAcq */
    fd_set	caSocketMask;			/* Select mask for caSocketFd */
    int		caRxBuff;			/* Curr socket rx buffer size */
    int		caTxBuff;			/* Curr socket tx buffer size */
    int		caRxTopped;			/* Rx buffer hit the max? */
    int		caTxTopped;			/* Tx buffer hit the max? */
    chid	pFirstChid;			/* Pointer to first chid */
    void	(*regFunc)();			/* Function for registration */
    void	*regFuncArg;			/* Argument for that function */
    int		firstSelDone;			/* First select already done? */
    int		nchib;				/* # bytes in caDataBuff */
    int		bufOutPos;			/* Next avail byte in caDataBuff */
#ifdef vxWorks
    int		taskId;				/* Task ID of this task */
    struct catx	*nextTask;			/* Next task in list */
    SEM_ID	sockMeSem;			/* ME semaphore to protect socket & buffers */
    struct catx	*importedContext;		/* Master task's context */
#else
    void	(*trendEventFunc)();		/* Trend event handler function */
    void	(*trendConnFunc)();		/* Trend connection handler function */
    void	(*alarmEventFunc)();		/* Alarm event handler function */
    void	(*alarmConnFunc)();		/* Alarm connection handler function */
    void	(*paramDataFunc)();		/* Connectionless parameter data function */
#endif vxWorks
    char	caOutputBuff[CA_TX_BUFF_SIZE];	/* Output buffer */
    char	caDataBuff[CA_RX_BUFF_SIZE];	/* Input buffer */
    char	caDbrBuff[CA_RX_BUFF_SIZE];	/* Auxiliary Input buffer */
    } CATX;

CATX	*getContext();

#define	CATXDEFINED

/* routime in msgProc.c */
int caValidChid (chid pChid, CATX *ctx);

#endif INCLcacontext

