/* choice.h				*/
/* share/epicsH @(#)choice.h	1.3     6/7/91 */

/*
 *      Author:          Marty Kraimer
 *      Date:            11-7-90
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  mm-dd-yy        iii     Comment
 * .02  03-29-94	ems	GET_CHOICE no longer explicitly checks for
 *				< 0 since it is unsigned
 *				same for GET_PARR_CHOICE_SET
 */


#ifndef INCchoiceh
#define INCchoiceh 1
struct choiceSet { /* This defines one set of choices*/
	long	number;		/*number of choices 			*/
	char	**papChoice;	/*ptr to arr of ptr to choice string	*/
	};
struct arrChoiceSet{ /*An array of choice sets for particular record type*/
	long		number;		/*number of choice sets 	*/
	struct choiceSet **papChoiceSet	;/*ptr to arr of ptr to choiceSet*/
	};
struct choiceRec{ /*define choices for each record type*/
	long		number;		   /*number of arrChoiceSet	*/
	struct arrChoiceSet **papArrChoiceSet;
		/*ptr to arr of ptr to arrChoiceSet*/
	};
extern struct choiceSet		*choiceCvt;
extern struct arrChoiceSet	*choiceGbl;
extern struct choiceRec		*choiceRec;
/* device choices					*/
struct devChoice{
	long	link_type;	/*link type for this device*/
	char	*pchoice;	/*ptr to choice string	*/
	};
struct devChoiceSet { /* This defines one set of device choices*/
	long	number;		/*number of choices 			*/
	struct devChoice	**papDevChoice;
		/*ptr to arr of ptr to devChoice	*/
	};
struct devChoiceRec{ /*define device choices for each record type*/
	long		number;		   /*number of devChoiceSet	*/
	struct devChoiceSet **papDevChoiceSet;
		/*ptr to arr of ptr to devChoiceSet*/
	};
extern struct devChoiceRec	*choiceDev;
/*NOTE: choiceCvt is initialized when cvtTable is loaded		*/
/*      choiceDev is initialized when devSup is loaded			*/

/************************************************************************
 * Except that entries could be null the following are valid references
 *   choiceCvt->papChoice[i]
 *   choiceGbl->papChoiceSet[i]->papChoice[j]
 *   choiceRec->papArrChoiceSet[i]->papChoiceSet[j]->papChoice[k]
 *   choiceDev->papDevChoiceSet[i]->papDevChoice[j]->pchoice->"<value>"
 *
 * The memory layout is
 *
 *   choiceCvt->choiceSet
 *		   number
 *		   papChoice->[0]
 *			      [1]->"<choice string>"
 *
 *   choiceGbl->arrChoiceSet
 *		   number
 *		   papChoiceSet->[0]
 *                               [1]->choiceSet
 *		  	     		number
 *		  			papChoice->[0]
 *						   [1]->"<choice string>"
 *   choiceRec->choiceRec
 *		   number
 *		   papArrChoiceSet->[0]
 * 				    [1]->arrChoiceSet
 *					  number
 *					  papChoiceSet->[0]
 *                                                      [1]->choiceSet
 *						              number
 *						              papChoice->[0]
 *								         [1]->
 *
 *   choiceDev->devChoiceRec
 *                 number
 *                 papDevChoiceSet->[0]
 *                                  [1]->devChoiceSet
 *                                          number
 *                                          papDevChoice->[0]
 *                                                        [1]->devChoice
 *                                                                link_type
 *                                                                pchoice->"val"
 *****************************************************************************/

/*************************************************************************
 *The following macro returns NULL or a ptr to a string		
 * A typical usage is:
 *
 *	char *pchoice;
 *	if(!(pchoice=GET_CHOICE(pchoiceSet,ind))){ <no string found>}
 ***************************************************************************/

#ifdef WIN32
#define GET_CHOICE(PCHOICE_SET,IND_CHOICE)\
(\
    (PCHOICE_SET)\
    ?(\
	( (((int)(IND_CHOICE))<0) ||\
		(((int)(IND_CHOICE))>=((struct choiceSet*)(PCHOICE_SET))->number) )\
	? NULL\
	: ((PCHOICE_SET)->papChoice[((int)(IND_CHOICE))])\
    )\
    : NULL\
)
#else
#define GET_CHOICE(PCHOICE_SET,IND_CHOICE)\
(\
    (PCHOICE_SET)\
    ?(\
	((unsigned)(IND_CHOICE)>=((struct choiceSet*)(PCHOICE_SET))->number) \
	? NULL\
	: ((PCHOICE_SET)->papChoice[(IND_CHOICE)])\
    )\
    : NULL\
)
#endif WIN32

/*****************************************************************************
 * The following macro returns NULL or a ptr to a choiceSet structure
 * A typical usage is:
 *
 *	struct choiceSet *pchoiceSet;
 *	if(!(pchoiceSet=GET_PCHOICE_SET(parrChoiceSet,ind))){<null>}
 ***************************************************************************/
#define GET_PCHOICE_SET(PARR_CHOICE_SET,IND_ARR)\
(\
    (PARR_CHOICE_SET)\
    ?(\
	( ((IND_ARR)<0) ||\
	       ((IND_ARR)>=((struct arrChoiceSet*)(PARR_CHOICE_SET))->number) )\
	? NULL\
	: ((PARR_CHOICE_SET)->papChoiceSet[(IND_ARR)])\
    )\
    : NULL\
)

/*****************************************************************************
 * The following macro returns NULL or a ptr to a arrChoiceSet structure
 * A typical usage is:
 *
 *	struct arrChoiceSet *parrChoiceSet;
 *	if(!(parrChoiceSet=GET_PARR_CHOICE_SET(pchoiceRec,ind))){<null>}
 ***************************************************************************/

#ifdef WIN32
#define GET_PARR_CHOICE_SET(PCHOICE_REC,IND_REC)\
(\
    (PCHOICE_REC)\
    ?(\
	( (((int)(IND_REC))<0) ||\
		(((int)(IND_REC))>=((struct choiceRec*)(PCHOICE_REC))->number) )\
	? NULL\
	: ((PCHOICE_REC)->papArrChoiceSet[((int)(IND_REC))])\
    )\
    : NULL\
)
#else
#define GET_PARR_CHOICE_SET(PCHOICE_REC,IND_REC)\
(\
    (PCHOICE_REC)\
    ?(\
	((unsigned)(IND_REC)>=((struct choiceRec*)(PCHOICE_REC))->number) \
	? NULL\
	: ((PCHOICE_REC)->papArrChoiceSet[(IND_REC)])\
    )\
    : NULL\
)
#endif WIN32


/*************************************************************************
 *The following macro returns NULL or a ptr to a devChoice structure
 * A typical usage is:
 *
 *	struct devChoice *pdevChoice;
 *	if(!(pdevChoice=GET_DEV_CHOICE(pdevChoiceSet,ind))){ <not found>}
 ***************************************************************************/
#define GET_DEV_CHOICE(PDEV_CHOICE_SET,IND_CHOICE)\
(\
    (PDEV_CHOICE_SET)\
    ?(\
	( ((IND_CHOICE)<0) ||\
 	    ((IND_CHOICE)>=((struct devChoiceSet*)(PDEV_CHOICE_SET))->number) )\
	? NULL\
	: ((PDEV_CHOICE_SET)->papDevChoice[(IND_CHOICE)])\
    )\
    : NULL\
)

/*****************************************************************************
 * The following macro returns NULL or a ptr to a devChoiceSet structure
 * A typical usage is:
 *
 *	struct devChoiceSet *pdevChoiceSet;
 *	if(!(pdevChoiceSet=GET_PDEV_CHOICE_SET(pdevChoiceRec,ind))){<null>}
 ***************************************************************************/
#define GET_PDEV_CHOICE_SET(PDEV_CHOICE_REC,IND_REC)\
(\
    (PDEV_CHOICE_REC)\
    ?(\
	( ((IND_REC)<0) ||\
	      ((IND_REC)>=((struct devChoiceRec*)(PDEV_CHOICE_REC))->number) )\
	? NULL\
	: ((PDEV_CHOICE_REC)->papDevChoiceSet[(IND_REC)])\
    )\
    : NULL\
)
#endif
