/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/* This file is a template for a codebase database converter
**
**	To use this file, you must define several symbols:
**		DBNAME is the name of the database you wish to convert
**			(if you don't define it, code will be generated
**			 to use command line arg 1, or convertDB arg)
**
**		TIS4000DB defines the DB directory (default /tis4000db)
**		DBCNVRT defines the save directory for the pre convert DB
**			(default is DBConversion)
**		TRANSLATE is the name of a routine that maps old DB field
**			names onto new DB field names. (default is that
**			the fields have the same name in the new DB)
**		POSTPROCESS is the name of a routine that initializes or
**			modifies the record content of the new file on a
**			per record basis (default does not change record)
**		FIELDINFO is the symbol for an array defining the new db
**			structure (MUST BE DEFINED)
**		TAGINFO is the symbol for an array defining the new db
**			tags (MUST BE DEFINED)
**		MEMOFILE indicates that the files have memo fields
**		cb	must be defined as the name given your CODE4 structure
**
**	The program can be stand-alone or used as a subroutine named convertDB()
**	depending on the symbol CONVERTSUB. When used as a subroutine, the program
**	assumes that CodeBase is opened with CODE4 named cb.
**
*/

#include	<stdio.h>
#ifndef AXP
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>
#include	<d4all.h>

#ifndef CONVERTSUB
static CODE4		cb;
#endif CONVERTSUB
static char		cnvrt_pathbuff[128];
static char		cnvrt_sysbuff[256];
static struct stat	cnvrt_statbuff;

static int create_subdir (void);
static int convert_database (char *dbname);

/* predefine TIS4000DB to use a different db directory */
#ifndef TIS4000DB
#ifdef WIN32
#define	TIS4000DB	"\\tis4000db\\"
#else
#define	TIS4000DB	"/tis4000db/"
#endif WIN32
#endif

/* predefine DBCNVRT to use a different save directory */
#ifndef DBCNVRT
#define	DBCNVRT		"DBConversion"
#endif

/* Predefine a field name translation routine if necessary */
#ifndef TRANSLATE
#define TRANSLATE(str)	str
#endif

/* Predefine a record postprocess routine if necessary */
#ifndef POSTPROCESS
#define POSTPROCESS(db)	;
#endif

#ifdef CONVERTSUB
int convertDB (char *dbname)
{
#else
int main()
{
    d4init (&cb);
    cb.go_error = 0;
    cb.open_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
#endif CONVERTSUB

    fprintf (stdout, "Starting database conversion\n");
    if (create_subdir ())
	return (1);
#ifdef CONVERTSUB
    if (convert_database (dbname))
#elif defined(DBNAME)
    if (convert_database (DBNAME))
#else
    if (argc >= 2 && convert_database (arg[1]))
#endif DBNAME
	return (1);
#ifndef CONVERTSUB
    d4close_all (&cb);
#endif CONVERTSUB
    fprintf (stdout, "\nDatabase conversion finished\n");
    return (0);
}


static int create_subdir (void)
{
    sprintf (cnvrt_pathbuff, "%s%s", TIS4000DB, DBCNVRT);
    if (stat (cnvrt_pathbuff, &cnvrt_statbuff))
    {
	if (errno == ENOENT)
	{
	    sprintf (cnvrt_sysbuff, "mkdir %s", cnvrt_pathbuff);
	    system (cnvrt_sysbuff);
	    if (stat (cnvrt_pathbuff, &cnvrt_statbuff))
	    {
		fprintf (stdout, "Database conversion err: failed to create %s\n", cnvrt_pathbuff);
		return (1);
	    }
	    else
	    {
		fprintf (stdout, "Database conversion: %s created\n", cnvrt_pathbuff);
		return (0);
	    }
	}
	else
	{
	    fprintf (stdout, "Database conversion err: stat problem (%d) on %s\n", errno, cnvrt_pathbuff);
	    return (1);
	}
    }
    return (0);
}


static int convert_database (char *dbname)
{
    DATA4	*old_database, *new_database;
    FIELD4	*old_field;
    char	*old_fieldname, *new_fieldname;
    int		i;

    fprintf (stdout, "...converting %s.dbf\n", dbname);

    sprintf (cnvrt_pathbuff, "%s%s.dbf", TIS4000DB, dbname);
    /* if there is no existing DB, quit */
    if (stat (cnvrt_pathbuff, &cnvrt_statbuff) != 0)
    {
	if (errno == ENOENT)
	    return (0);
	else
	{
	    fprintf (stdout, "Database conversion err: stat problem (%d) on %s\n", errno, cnvrt_pathbuff);
	    return (1);
	}
    }
#ifdef WIN32
    sprintf (cnvrt_pathbuff, "%s%s\\%s", TIS4000DB, DBCNVRT, dbname);
#else
    sprintf (cnvrt_pathbuff, "%s%s/%s", TIS4000DB, DBCNVRT, dbname);
#endif WIN32

    /* save the old dbase files */
#ifdef WIN32
    sprintf (cnvrt_sysbuff, "move %s%s.dbf %s%s\\%s.dbf", 
#else
    sprintf (cnvrt_sysbuff, "mv %s%s.dbf %s/%s/%s.dbf", 
#endif WIN32
			   TIS4000DB, dbname, TIS4000DB, DBCNVRT, dbname);
    system (cnvrt_sysbuff);
#ifdef WIN32
    sprintf (cnvrt_sysbuff, "move %s%s.mdx %s%s\\%s.mdx", 
#else
    sprintf (cnvrt_sysbuff, "mv %s%s.mdx %s%s/%s.mdx", 
#endif WIN32
			   TIS4000DB, dbname, TIS4000DB, DBCNVRT, dbname);
    system (cnvrt_sysbuff);
#ifdef MEMOFILE
    /* memo database if present! */
#ifdef WIN32
    sprintf (cnvrt_sysbuff, "move %s%s.dbt %s%s\\%s.dbt", 
#else
    sprintf (cnvrt_sysbuff, "mv %s%s.dbt %s%s/%s.dbt", 
#endif WIN32
			   TIS4000DB, dbname, TIS4000DB, DBCNVRT, dbname);
    system (cnvrt_sysbuff);
#endif MEMOFILE

    if (!(old_database = d4open (&cb, cnvrt_pathbuff)))
    {
	fprintf (stdout, "Database conversion err: open error on %s\n", cnvrt_pathbuff);
	return (1);
    }
    d4alias_set (old_database, "old_database");

    /* create the new dbase file */
    sprintf (cnvrt_pathbuff, "%s%s", TIS4000DB, dbname);
    if (!(new_database = d4create (&cb, cnvrt_pathbuff, FIELDINFO, TAGINFO)))
    {
	fprintf (stdout, "Database conversion err: failed to create %s\n", cnvrt_pathbuff);
	return (1);
    }

    for (d4top (old_database); !d4eof (old_database); d4skip (old_database, 1))
    {
	d4append_start (new_database, 0);
	d4blank (new_database);
	for (i = 1; i <= d4num_fields (old_database); i++)
	{
	    old_field = d4field_j (old_database, i);
	    old_fieldname = (char *)f4name(old_field);
	    new_fieldname = TRANSLATE (old_fieldname);
	    if (f4type (old_field) == 'M')
		f4memoAssign (d4field(new_database, new_fieldname), f4memoPtr (old_field));
	    else
		f4assignField (d4field(new_database, new_fieldname), old_field);
	}
	/* post-process here */
	POSTPROCESS (new_database);
	d4append (new_database);
	d4flush (new_database);
    }

    d4close (old_database);
    d4close (new_database);

    fprintf (stdout, "...finished database.dbf\n");
    return (0);
}

