/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	    MEMORY LAYOUT FOR CURRENT VALUE OF A POINT		*/
/*								*/
/*	These structes define the layout of the current value	*/
/*	of a point as returned to the user when he tries to	*/
/*	get data out of the tree. It is NOT the actual pointer	*/
/*	stored in the tree.					*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

/******
	Common information.
		All entries contain this regardless of type.
******/

typedef struct
    {
    struct timeval	lastUpdate;	/* Last time updated */
    unsigned short	nativeType;	/* Type of data. See dataTypes.h */
    unsigned short	nativeCount;	/* Array count. */
    unsigned short	stat;		/* Point status */
    unsigned short	sevr;		/* Severity */
    unsigned short	ack;		/* Alarm ack required */
    char		fillStuff[6];	/* Round up to multiple of 8 */
    } VALUE_COMMON;

/******
	String data types, AGNT_STRING
******/

typedef struct
    {
    VALUE_COMMON	common;
    char		value[MAX_STRING_SIZE + 44];	/* Allow for "decorations" in status strings */
    } VALUE_STRING;

/******
	16 bit integer data types, AGNT_INT or AGNT_SHORT
******/

typedef struct
    {
    VALUE_COMMON	common;
    short		value;
    } VALUE_SHORT;

/******
	32 bit float data type, AGNT_FLOAT
******/

typedef struct
    {
    VALUE_COMMON	common;
    float		value;
    } VALUE_FLOAT;

/******
	16 bit enumerated data types, AGNT_ENUM
******/

typedef struct
    {
    VALUE_COMMON	common;
    short		value;
    } VALUE_ENUM;

/******
	Character data types, AGNT_CHAR
******/

typedef struct
    {
    VALUE_COMMON	common;
    char		value;
    } VALUE_CHAR;

/******
	32 bit integer data types, AGNT_LONG
******/

typedef struct
    {
    VALUE_COMMON	common;
    int			value;
    } VALUE_LONG;

/******
	64 bit float data types, AGNT_DOUBLE
******/

typedef struct
    {
    VALUE_COMMON	common;
    double		value;
    } VALUE_DOUBLE;

