/* DB_EVENT.H */

/* routines for scheduling events to lower priority tasks via the RT kernel */

#define EVENTQUESIZE	EVENTENTRIES  *32 
#define EVENTENTRIES	16	/* the number of que entries for each event */
#define EVENTQEMPTY	((struct event_block *)NULL)

/*
 * really a ring buffer
 */
typedef struct event_que{	
  	struct event_block	*evque[EVENTQUESIZE];
  	db_field_log		valque[EVENTQUESIZE];
  	unsigned short	putix;
  	unsigned short	getix;
  	unsigned short	quota;		/* the number of assigned entries*/

  	/* lock writers to the ring buffer only */
  	/* readers must never slow up writers */
  	FAST_LOCK		writelock;

  	struct event_que	*nextque;	/* in case que quota exceeded */
  	struct event_user	*evuser;	/* event user parent struct */
} EVENT_QUE;

typedef struct event_block{
  	NODE			node;
  	struct db_addr		*paddr;
  	void			(*user_sub)();
  	void			*user_arg;
  	struct event_que	*ev_que;
  	unsigned char		select;
  	char			valque;
  	unsigned short		npend;	/* n times this event is on the que */
} EVENT_BLOCK;

typedef struct event_user{
  	int			taskid;		/* event handler task id */
  	int			taskpri;	/* event handler task pri */

  	char			pendlck;	/* Only one task can pend */
  	SEM_ID			ppendsem;	/* Wait while empty */
  	unsigned char		pendexit;	/* exit pend task */

  	unsigned short	queovr;			/* event que overflow count */
  	void			(*overflow_sub)();/* called overflow detect */
  	void			*overflow_arg;	/* parameter to above 	*/

  	EVENT_QUE		firstque;	/* the first event que */
} EVENT_USER;

int		db_event_list (char *name);
EVENT_USER	*db_init_events (void);
int		db_close_events (EVENT_USER *evuser);
int		db_sizeof_event_block (void);
int		db_add_event (EVENT_USER *evuser,
			      DB_ADDR *paddr,
			      void (*user_sub)(void),
			      void *user_arg,
			      unsigned int sel,
			      EVENT_BLOCK *pevent); /* ptr to event blk (not required) */
int		db_cancel_event (EVENT_BLOCK *pevent);
int		db_add_overflow_event (EVENT_USER *evuser,
				void (*overflow_sub)(void),
				void *overflow_arg);
int		db_post_single_event (EVENT_BLOCK *pevent);
/* this routine is located in .../share/src/rsrv/postEvents.c instead of dnEvents.c! */
int		db_post_events (struct dbCommon	*precord, caddr_t pfield, int mask);
int		db_start_events (EVENT_USER *evuser,
			 	char *taskname,	/* defaulted if NULL */
			 	void (*init_func)(void),
			 	int init_func_arg);

