/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	METADATA DEFINITIONS FOR CHANNEL ACCESS CLIENTS		*/
/*								*/
/*	The purpose of this set of control tables is to tell	*/
/*	the software how to fill out control structures for	*/
/*	all of the fields in a record. We have default		*/
/*	display high, display low, control high, and		*/
/*	control low for numeric fields. We also have a list	*/
/*	of the fields that are exceptions to these defaults.	*/
/*	For enumerated fields that do NOT get their values	*/
/*	from the SDR, we also have a list of where to get	*/
/*	the strings that correspond to the various values.	*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		NUMERIC LIMIT/PRECISION  STRUCTURE		*/
/*								*/
/*	This structure defines a limit or precision for a	*/
/*	field. The value can be a reference to another field	*/
/*	in the same record, a constant, or a special routine	*/
/*	Special routines are defined by their special number.	*/
/*								*/
/****************************************************************/

typedef struct
    {
    short		limitType;		/* 0 = constant, 1 = reference, 2 = Back Door */
    char		reference[4];		/* Reference into SDR, where to get data */
    double		hardConst;		/* For hard coded numbers, like PID gains */
    short		tableSlot;		/* Where to put it in the numeric table */
    short		backDoor;		/* For sleaze */
    } limitStruct;


/****************************************************************/
/*								*/
/*		NUMERIC EXCEPTION FIELD STRUCTURE		*/
/*								*/
/*	This stucture declares the parameters for a numeric	*/
/*	field that does not follow the defaults. If any one	*/
/*	of the pointers is NULL, then the default is to be	*/
/*	used for that particular parameter.			*/
/*								*/
/****************************************************************/

typedef struct numField	numericField;

struct numField
    {
    numericField	*nextException;		/* Forward link */
    char		fieldName[4];		/* Which field we are talking about */
    limitStruct		*displayHigh;		/* 0 = use default, else how to find limit */
    limitStruct 	*displayLow;		/* 0 = use default, else how to find limit */
    limitStruct		*controlHigh;		/* 0 = use default, else how to find limit */
    limitStruct		*controlLow;		/* 0 = use default, else how to find limit */
    limitStruct		*precision;		/* 0 = use default, else how to find limit */
    };


/****************************************************************/
/*								*/
/*		ENUMERATED FIELD STRUCTURE			*/
/*								*/
/*	This structure is used to identify the enumerated	*/
/*	fields that are NOT defined in the SDR. An example	*/
/*	of such a field is VAL in a bi.				*/
/*								*/
/****************************************************************/

typedef struct enumFld	enumField;

struct enumFld
    {
    enumField		*nextEnum;		/* Forward link */
    char		fieldName[4];		/* Which field we are talking about */
    short		numStrings;		/* The largest number of strings */
    char		refField[16][4];	/* List of field names */
    short		backDoor;
    };


/****************************************************************/
/*								*/
/*		POINT TYPE DESCRIPTOR STRUCTURE			*/
/*								*/
/*	This structure contains the default parameters for the	*/
/*	point type and pointers to the first numeric exception	*/
/*	and the first enumeration. The size of the double	*/
/*	table is also included to make it easier for the	*/
/*	database traversal program that builds the tree. This	*/
/*	structure is actually used in an array that is indexed	*/
/*	by point type.						*/
/*								*/
/****************************************************************/


typedef struct
    {
    int			tableSize;		/* Size of the double table for this point type */
    limitStruct		*defaultDisplayHigh;
    limitStruct		*defaultDisplayLow;
    limitStruct		*defaultControlHigh;
    limitStruct		*defaultControlLow;
    limitStruct		*defaultPrec;
    numericField	*exceptions;
    enumField		*enumList;
    } typeDesc;
