/*============================================================================*/
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Edit:			Doug Dotson				      */
/*	Date:			14-Jan-1993				      */
/*									      */
/*	Module Name:		msmain.h				      */
/*	Module Description:						      */
/*									      */
/*	Modification History:	First time 14-Jan-1993			      */
/*									      */
/*============================================================================*/

#include	<sys/types.h>
#include	<time.h>
#include	<sys/socket.h>
#include	<netinet/in.h>
#ifndef AXP
#include	<sys/sockio.h>
#endif
#include	<sys/time.h>
#include	<sys/file.h>
#include	<netdb.h>
#include	<netinet/tcp.h>

#define	LISTEN_PORT		8599

/* dbLibrary file names */

#ifdef WIN32
#define	DEFAULT_DIRECTORY	"\\tis4000db\\"
#else
#define	DEFAULT_DIRECTORY	"/tis4000db/"
#endif WIN32
#define	RESLIST_FILE		"reslist"
#define	FLD_IOC1		"PRIMARYIOC"
#define	FLD_IOC2		"SECNDRYIOC"

/* size parameters */

#define	MAXPATHLENGTH	100
#define	MAXHOSTNAMELENGTH	16
#define	RESNAMESIZE		16
#define	APP_NAME_LEN		32
#define	FIRST_FD		3
#define	MAX_APPS		256	/* application table entries (sparse array) */
#define	HEARTRATE		2
#define	PARTNER_TIMEOUT		7
#define	DISCONNECT_PEND_TIME	10
#define	MAX_MSG_LEN		1000
#define	BUFFER_SIZE		10000
#define	BACKUP_BUFFER_SIZE	1000
#ifndef	ERROR
#define	ERROR (-1)
#endif

/* A portable way of saying 32 bit unsigned integer */

#define	uint32	unsigned int

/* State codes. */

typedef enum {
	SLAVE = 1,
	MASTER,
	MASTER_NOPARTNER,
	UNKNOWN
} WS_STATE;

/* Communications buffers */

typedef struct _combuf {
    int		fd;	/* For debug purposes */
    char	*buffer;
    int		bufSize;
    int		curSize;
} ComBuf;

/* Application table entries */

typedef	struct _msappnode {
    char	name [APP_NAME_LEN];		/* name of application from 'appRegister' */
    int		fd;				/* return address socket to app */
    ComBuf	rxBuffer;			/* Receive buffer for application messages */
    ComBuf	txBuffer;			/* Transmit buffer for application messages */
} AppNode;

/* Message types */

#define	PARTNER_STATE	1
#define	APP_REGISTER	2
#define	APP_DATA	3
#define	APP_STATE	4


#define	MSG_MAGIC_NUMBER	0x12345678

/* Messages */

typedef struct _msgheader {
    uint32	magic;
    uint32	size;
    uint32	type;
} MsgHeader;

typedef struct _statemsg {
    uint32	magic;
    uint32	size;
    uint32	type;
    uint32	state;
    uint32	upTime;
} StateMsg;

typedef struct _aprregistermsg {
    uint32	magic;
    uint32	size;
    uint32	type;
    char	name [APP_NAME_LEN];
} AppRegisterMsg;

typedef struct _appdatamsg {
    uint32	magic;
    uint32	size;
    uint32	type;
    char	name [APP_NAME_LEN];
    char	data [MAX_MSG_LEN];
} AppDataMsg;

typedef struct _appstatemsg {
    uint32	magic;
    uint32	size;
    uint32	type;
    uint32	state;
} AppStateMsg;

typedef union _msg {
    MsgHeader		msgHeader;
    StateMsg		stateMsg;
    AppRegisterMsg	appRegisterMsg;
    AppDataMsg		appDataMsg;
    AppStateMsg		appStateMsg;
} Msg;

#ifdef MIPSEL
#define	LOG_OPTIONS	LOG_SALERT
#else
#define	LOG_OPTIONS	LOG_ALERT|LOG_LOCAL1
#endif

