#ifndef INC_pfoPlotDefs_h
#define INC_pfoPlotDefs_h
/*	@(#)pfoPlotDefs.h	1.1 6/19/91
 *	Author:	Roger A. Cole
 *	Date:	03-04-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .00	03-04-91	rac	initial version
 * .01	06-19-91	rac	installed in SCCS
 *
 * make options
 *	For user programs, there are no interesting options.  For private
 *	options relevent only to pfoPlot, see pfoPlot.c .
 */
/*+/mod***********************************************************************
* TITLE	pfoPlotDefs.h - plotting definitions for "plotting for OPI" routines
*
* DESCRIPTION
*
*-***************************************************************************/

typedef enum {
    PFO_ATTR_FG1,
    PFO_ATTR_FG2,
} PFO_ATTR;

/*/subhead PFO_CHAN------------------------------------------------------------
* PFO_CHAN
*----------------------------------------------------------------------------*/
#if !defined PFO_PRIVATE && !defined PFO_CHAN_DEF
  typedef struct pfoChan PFO_CHAN;
#else
  typedef struct pfoChan {
    struct gsd_sync_data *pGsd;	/* pointer to sync data buf */
    SYD_CHAN	*pSchan;	/* pointer to sync data channel struct */
    union db_access_val *pDbrGr;/* pointer to graphics info */
    chtype	dbrGrType;	/* type of graphics info buffer */
  } PFO_CHAN;
#endif

/*/subhead PFO_AXIS------------------------------------------------------------
* PFO_AXIS
*----------------------------------------------------------------------------*/
#ifndef PFO_PRIVATE
  typedef struct pfoAxis PFO_AXIS;
#else
  typedef struct pfoAxis {
    struct pfoAxis *pNext;	/* pointer to next axis structure */
    struct pfoAxis *pPrev;	/* pointer to previous axis structure */
    struct pfoWin *pPfo;	/* pointer to pfo win struct */
    PFO_CHAN	pfoChan;	/* pfo channel struct for data */
    SYD_PL_SLAVE *pSlave;	/* pointer to sydPlot slave struct */
    short	isAnX;		/* 1 if this is an x axis */
  } PFO_AXIS;
#endif

/*/subhead PFO_WIN-------------------------------------------------------------
* PFO_WIN
*----------------------------------------------------------------------------*/
#ifndef PFO_PRIVATE
  typedef struct pfoAxis PFO_WIN;
#else
  typedef struct pfoWin {
    SYD_SPEC	*pSspec;	/* pointer to sync data spec struct */
    SYD_PL_MSTR	*pMstr;		/* pointer to master plot struct */
    SYD_PL_MSTR	master;		/* master plot struct */
    GC		gc;		/* gc to use in drawing */
    unsigned long fg;		/* foreground pixel value */
    unsigned long bg;		/* background pixel value */
    Display	*pDisp;		/* X display */
    Window	window;		/* X window */
    short	nSamples;	/* dimension of circular buffers */
    PFO_AXIS	*pXAxisHead;	/* ptr to first axis struct for horiz axis */
    PFO_AXIS	*pXAxisTail;	/* ptr to last axis struct for horiz axis */
    PFO_AXIS	*pYAxisHead;	/* ptr to first axis struct for vert axis */
    PFO_AXIS	*pYAxisTail;	/* ptr to last axis struct for vert axis */
    short	binNum;		/* bin number, for plotting scalars */
    char	title[120];	/* top title for plot */
    char	yTitle[120];	/* left title for plot */
    char	xTitle[120];	/* bottom title for plot */
    enum dl_plot_style style;	/* plot style */
    int		axis;		/* axis type */
  } PFO_WIN;
#endif

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
void pfoAxisAutoRange(PFO_AXIS *pAxis);
void pfoAxisClose(PFO_AXIS *pAxis);
PFO_AXIS *pfoAxisOpen(PFO_WIN *pPfo, int isX, int fill, unsigned long fg,
		      struct gsd_sync_data *pGsd, union db_access_val *pDbrGr,
		      chtype dbrGrType);
void pfoWinAutoRange(PFO_WIN *pPfo);
void pfoWinClose(PFO_WIN *pPfo);
PFO_WIN *pfoWinOpen(Display *pDisp, Window window, unsigned long fg, unsigned long bg,
		    int nSamples, char *title, char *xTitle, char *yTitle);
void pfoWinPlotSample(PFO_WIN *pPfo);
void pfoWinRepaint(PFO_WIN *pPfo, enum dl_plot_style style, int	axis);
void pfoWinRestart(PFO_WIN *pPfo, enum dl_plot_style style, int axis);
void pfoWinSetAttr(PFO_WIN *pPfo, PFO_ATTR attr, int value, void *pArg);
void pfoWinWritePSFile(PFO_WIN *pPfo, char *filePath);

#endif INC_pfoPlotDefs_h
