#ifndef INC_pprPlotDefs_h
#define INC_pprPlotDefs_h
/*	@(#)pprPlotDefs.h	1.3 9/23/91
 *	Author:	Roger A. Cole
 *	Date:	12-04-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .00	12-04-90	rac	initial version
 * .01	06-19-91	rac	installed in SCCS
 * .02	09-04-91	rac	remove PRIVATE from structures; add
 *				PixToFrac macros; add support for Erase;
 *				set to always be for X11
 * .03	09-23-91	rac	undefine XWINDOWS from `make'
 *
 * make options
 *	For user programs, there are no interesting options.  For private
 *	options relevant only to pprPlot, see pprPlot.c .
 */
/*+/mod***********************************************************************
* TITLE	pprPlotDefs.h - definitions for portable plotting routines
*
* DESCRIPTION
*
*-***************************************************************************/

#undef XWINDOWS
#define XWINDOWS

#ifdef PPR_PRIVATE

/*-----------------------------------------------------------------------------
* PPR_BTN_CLOSE is parameterized here.  If you change its definition, change
*	the documentation in pprWinLoop.
*----------------------------------------------------------------------------*/
#  ifdef SUNVIEW
#     include <suntool/sunview.h>
#     include <suntool/canvas.h>
#     define PPR_BTN_CLOSE MS_RIGHT
# elif defined XWINDOWS
#     include <X11/Xlib.h>
#     include <X11/Xutil.h>
#     define PPR_BTN_CLOSE Button3
#  endif

#  define PprLineThick(pWin, thick) \
	( (thick != pWin->lineThick) ? pprLineThick(pWin, thick) : 0 )

#endif

#  define PprAbs(v1) ((v1)>=0. ? (v1) : (-(v1)))
#  define PprMin(v1, v2) ((v1)<(v2) ? (v1) : (v2))
#  define PprMax(v1, v2) ((v1)<(v2) ? (v2) : (v1))
#  define PprDfltCharHt(winYbot, winYtop) \
		PprMin(PprAbs(.042 * ((winYtop) - (winYbot))), .014)
#  define PprYFracToXFrac(pWin, yfrac) \
		((yfrac)*pWin->height/pWin->width)
#  define PprPixXToXFrac(pWin, xpix) \
	((double)((xpix) - pWin->x) / (double)(pWin->width))
/* this assumes that 0,0 is northwest corner, as in X11 and SunView */
#  define PprPixYToYFrac(pWin, ypix) \
	((double)(pWin->height - ((ypix) - pWin->y)) / (double)(pWin->height))

typedef enum {
    PPR_WIN_SCREEN,
    PPR_WIN_POSTSCRIPT,
    PPR_WIN_EPS			/* encapsulated postscript */
} PPR_WIN_TY;

typedef enum {
    PPR_TXT_CEN,
    PPR_TXT_RJ,
    PPR_TXT_LJ
} PPR_TXT_JUST;

typedef enum {
    PPR_ATTR_KEYNUM,
    PPR_ATTR_LINE_THICK,
    PPR_ATTR_COLORNUM,
    PPR_ATTR_PATT_ARRAY,
    PPR_ATTR_CLIP,
    PPR_ATTR_NOCLIP,
    PPR_ATTR_BG,
    PPR_ATTR_FG,
} PPR_ATTR_CODE;

#define PPR_NKEYS 11


/*-----------------------------------------------------------------------------
* pattern
*	distances are in pixels
*
* thickness
*	thickness is in thousandths of window height.  For example, 1 is
*	1/1000 of the window height, 15 is 15/1000, etc.
*
*	For SunView, PostScript, and EPS, the line thickness items in the
*	plot window are used, ignoring those in the plot area.  The opposite
*	is true for X.
*----------------------------------------------------------------------------*/
typedef struct {
	unsigned int myGC : 1;	/* GC is mine; must be released */
	unsigned int clip : 1;	/* clip lines at edge of data area */
	unsigned int bgGC : 1;	/* a "background GC" exists; must be released */
	double	rem;		/* length remaining in current part of patt */
	short	*pPatt;		/* pointer to pattern array, or NULL */
	short	sub;		/* current subscript in pattern array */
	short	pen;		/* 0,1 for pen up,down */
	short	lineThick;	/* line thickness, thousandths of
				win ht; used for PostScript, EPS, and
				SunView */
	short	ltPix;		/* line thickness in pixels */
	short	ltCurr;		/* current line thickness, win ht /1000 */
# ifdef XWINDOWS
	GC	gc;		/* graphics context */
	GC	gcBG;		/* background color graphics context */
# endif
} PPR_ATTR;

#ifdef PPR_PRIVATE
    static short p0[]={4,4,0};
    static short p1[]={8,4,0};
    static short p2[]={16,4,0};
    static short p3[]={24,4,0};
    static short p4[]={32,4,0};
    static short p5[]={4,4,32,4,0};
    static short p6[]={8,4,32,4,0};
    static short p7[]={16,4,32,4,0};
    static short p8[]={4,4,4,4,32,4,0};
    static short p9[]={8,4,8,4,32,4,0};
    static short p10[]={16,4,16,4,32,4,0};
    static short *pglPprPat[]={p0,p1,p2,p3,p4,p5,p6,p7,p8,p9,p10};
/*-----------------------------------------------------------------------------
* colors
*----------------------------------------------------------------------------*/
#define PPR_NCOLORS 12

# ifdef XWINDOWS
    static char	*pglPprColor[]={
	"red", "blue", "magenta", "green", "cyan", "gold", "orange",
	"yellow green", "medium turquoise", "gray", "maroon", "violet"
    };
# endif
#endif
/*-----------------------------------------------------------------------------
* plot marks
*	There are small and medium sized marks.  The latter are commented
*	out because the small ones turned out to be legible on crt's.
*----------------------------------------------------------------------------*/
#define PPR_NMARKS 11

#ifdef PPR_PRIVATE
static short
    ms0[]={-2,-2,0, 2,4,1, 2,-4,1, -4,0,2},		/* triangle */
    ms1[]={-2,-2,0, 0,4,1, 4,0,1, 0,-4,1, -4,0,2},	/* square */
    ms2[]={-1,-2,0, -1,2,1, 1,2,1, 2,0,1, 1,-2,1, -1,-2,1, -2,0,2},/* hex */
    ms3[]={-2,-2,0, 2,4,1, 2,-4,1, -4,3,1, 4,0,1, -4,-3,2},/* star */
    ms4[]={0,-2,0, -2,2,1, 2,2,1, 2,-2,1, -2,-2,2},	/* diamond */
    ms5[]={-2,-2,0, 4,4,1, -4,0,1, 4,-4,1, -4,0,2},	/* hourglass */
    ms6[]={0,-2,0, 0,4,1, -2,-2,0, 4,0,2},		/* plus */
    ms7[]={-2,-2,0, 2,2,1, 0,2,1, 0,-2,0, 2,-2,2},	/* inverted y */
    ms8[]={-2,-2,0, 4,0,1, -2,0,0, 0,4,1, -2,0,0, 4,0,2},/* I-beam */
    ms9[]={-1,-2,0, 2,4,1, -2,0,0, 2,-4,1, -3,2,0, 4,0,2},/* snowflake */
    ms10[]={-1,-2,0, 1,0,1, 0,4,1, 1,0,1, -3,-1,0, 0,-1,1, 4,0,1,
							0,-1,2},/* swastika */
    *pglPprMarkS[]={ms0,ms1,ms2,ms4,ms3,ms5,ms6,ms7,ms8,ms9,ms10};
#if 0
static short
    mm0[]={-3,-3,0, 3,6,1, 3,-6,1, -6,0,2},
    mm1[]={-3,-3,0, 0,6,1, 6,0,1, 0,-6,1, -6,0,2},
    mm2[]={-2,-3,0, -1,3,1, 1,3,1, 4,0,1, 1,-3,1, -1,-3,1, -4,0,2},
    mm3[]={-2,-3,0, 2,6,1, 2,-6,1, -5,4,1, 6,0,1, -5,-4,2},
    mm4[]={0,-3,0, -3,3,1, 3,3,1, 3,-3,1, -3,-3,2},
    mm5[]={-3,-3,0, 6,6,1, -6,0,1, 6,-6,1, -6,0,2},
    mm6[]={0,-3,0, 0,6,1, -3,-3,0, 6,0,2},
    mm7[]={-3,-3,0, 3,3,1, 0,3,1, 0,-3,0, 3,-3,2},
    mm8[]={-3,-3,0, 6,0,1, -3,0,0, 0,6,1, -3,0,0, 6,0,2},
    mm9[]={-2,-3,0, 4,6,1, -4,0,0, 4,-6,1, -5,3,0, 6,0,2},
    mm10[]={-2,-3,0, 2,0,1, 0,6,1, 2,0,1,- 5,-1,0, 0,-2,1, 6,0,1, 0,-2,2},
    *pglPprMarkM[]={mm0,mm1,mm2,mm4,mm3,mm5,mm6,mm7,mm8,mm9,mm10};
#endif

#endif

/*/subhead PPR_AREA------------------------------------------------------------
* PPR_AREA
*
*	most pixel values are relative to the southwest corner of the plot
*	window.  xPix and yPix are relative to the `natural' corner for the
*	type of windowing platform--lower left for PostScript, upper left
*	for X and SunView.
*----------------------------------------------------------------------------*/

typedef struct pprArea {
    struct pprWin *pWin;	/* pointer to window structure */
    struct pprArea *pNext;	/* pointer to next area structure */
    struct pprArea *pPrev;	/* pointer to previous area structure */
    int		oldWinHt;	/* height of plot win at last rescale */
    double	xPix[2];	/* x coord, in pixels */
    double	yPix[2];	/* y coord, in pixels */
    double	xScale;		/* scale factor for x data to pixels */
    double	xPixLeft;	/* pix from win left to data area left */
    double	xPixRight;	/* pix from win left to data area right */
    double	xLeft;		/* left x data value, for conv. */
    double	xRight;		/* right x data value, for conv. */
    double	xFracLeft;	/* window fraction at left x data value */
    double	xFracRight;	/* window fraction at right x data value */
    double	xInterval;	/* x interval size, in x data units */
    int		xNint;		/* number of x intervals */
    double	yScale;		/* scale factor for y data to pixels */
    double	yPixBot;	/* pix from win bottom to data area bottom */
    double	yPixTop;	/* pix from win bottom to data area top */
    double	yBot;		/* bottom y data value, for conv. */
    double	yTop;		/* top y data value, for conv. */
    double	yFracBot;	/* window fraction at bottom y data value */
    double	yFracTop;	/* window fraction at top y data value */
    double	yInterval;	/* y interval size, in y data units */
    int		yNint;		/* number of y intervals */
    double	tickHt;		/* default height of tick mark, in pixels */
    double	charHt;		/* default height of characters, in pixels */
    PPR_ATTR	attr;		/* area attributes */
  } PPR_AREA;

/*/subhead PPR_WIN-------------------------------------------------------------
* PPR_WIN
*	Structure for the overall plot window, which may be an actual window,
*	or it may be a PostScript or EPS file.
*----------------------------------------------------------------------------*/
typedef struct pprWin {
    PPR_AREA	*pAreaHead;
    PPR_AREA	*pAreaTail;
    void	(*drawFun)();	/* function to draw the plot */
    void	*pDrawArg;	/* pointer to pass to the draw function */
    PPR_WIN_TY	winType;	/* type of window: PPR_WIN_xxx */
    char	winDispName[120];/* name of "display" or file for window */
    char	title[80];	/* title for window and icon */
# ifdef SUNVIEW
    Frame	frame;		/* SunView frame for plotting */
    Canvas	canvas;		/* SunView canvas for plotting */
    Pixwin	*pw;		/* SunView pixwin for plotting */
    struct pr_brush brush;	/* SunView brush structure for line thick */
# elif defined XWINDOWS
    Display     *pDisp;         /* pointer to X server connection */
    Window      plotWindow;     /* the plot window on the display */
# endif
    FILE	*file;		/* file pointer for PostScript and EPS */
    int		x;		/* x pos of window on the display, in pixels */
    int		y;		/* y pos of window on the display, in pixels */
    int		width;		/* width of the window, in pixels */
    int		height;		/* height of the window, in pixels */
    PPR_ATTR	attr;		/* window attributes */
  } PPR_WIN;

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
void pprArcD(), pprArcD_ac();
void pprAnnotX(), pprAnnotX_wc();
void pprAnnotY(), pprAnnotY_wc();
void pprAnnotYMark();
void pprAreaClose();
void pprAreaErase();
PPR_AREA *pprAreaOpen();
PPR_AREA *pprAreaOpenDflt();
void pprAreaRescale();
long pprAreaSetAttr();
void pprAutoEnds();
void pprAutoInterval();
void pprAutoRangeD();
void pprAutoRangeF();
void pprAutoRangeL();
void pprAutoRangeS();
void pprChar();
double pprCos_deg();
void pprCvtDblToTxt();
void pprGrid();
void pprGridErase();
void pprGridLabel();
void pprLineF(), pprLineEraseF();
void pprLineD(), pprLineEraseD();
void pprLineS(), pprLineEraseS();
void pprLineL(), pprLineEraseL();
void pprLineSegD(), pprLineSegEraseD(), pprLineSegD_ac(), pprLineSegD_wc();
void pprLineSegL(), pprLineSegEraseL(), pprLineSegL_ac(), pprLineSegL_wc();
void pprMarkD(), pprMarkEraseD();
void pprMarkL(), pprMarkEraseL();
void pprMoveD(), pprMoveEraseD(), pprMoveD_ac(), pprMoveD_wc();
void pprPerim();
void pprPerimErase();
void pprPerimLabel();
void pprPointD(), pprPointEraseD();
void pprPointL(), pprPointEraseL();
void pprRegionErase();
double pprSin_deg();
void pprText(), pprTextErase(), pprText_wc();
void pprText1();
void pprTextPS();
void pprWinClose();
void pprWinErase();
void pprWinEvHandler();
void pprWinInfo();
int pprWinIsMono();
long pprWinLoop();
long pprWinMap();
PPR_WIN *pprWinOpen();
PPR_WIN *pprWinOpenUW();
void pprWinReplot();
void pprWinRescale();
void pprWinWrapup();
double pprYFracToXFrac();
void pprSleep();

#endif
