/************************************************************************/
/*									*/
/*			INTER-IOC REDUNDANCY MESSAGES			*/
/*									*/
/*	The following messages are sent between two cooperating		*/
/*	dbServer tasks. Each of the messages starts with a short	*/
/*	that declares the message type. A unique structure then		*/
/*	follows. All fields in all messages will always be in		*/
/*	network byte order.						*/
/*									*/
/************************************************************************/

/****** MOD LOG
01/08/92	GMU	410-581-0422	uchenick@tate.com
		Original entry

02/15/92	GCT	410-581-0422	tomalesk@tate.com
		Added dbClient/dbServer messages.
******/

#ifndef	IncrdmsgsH
#define	IncrdmsgsH

#include "dbDefs.h"

/************************************************************************/
/*									*/
/*		"WELL KNOWN" UDP PORT FOR DBSERVER			*/
/*									*/
/************************************************************************/

#define DBSERVER_UDP_PORT	8585

/************************************************************************/
/************************************************************************/

#define	DBC_RESNAMESIZE		(16)
#define	DBC_MAXCRCPERREQ	(32)
#define	DBC_PTNAMESIZE		(48)
#ifndef	OLD_SDR
#define	DBC_TRANSMIT_SIZE	(16384)		/* 16Kbytes /message MAX */
#else
#define	DBC_TRANSMIT_SIZE	(512)		/* 16Kbytes /message MAX */
#endif
/************************************************************************/
/*									*/
/*			VALUES FOR MESSAGE TYPE				*/
/*									*/
/************************************************************************/

#define RED_KEEPALIVE	0	/* Used only when line is idle */
#define RED_CONNREQ	1	/* See typedef for RED_CONNREQ_HDR */
#define RED_CONNREPLY	2	/* See typedef for RED_CONNREPLY_HDR */
#define RED_IOCDBREQ	3	/* See typedef for RED_IOCLINKREQ */
#define RED_UPDATE	4	/* See typedef for RED_DBUPDATE */
#define RED_CRCFAIL	5	/* See typedef for RED_PROMPTREQ */
#define RED_UPDREQ	6	/* See typedef for RED_ASKUPDT */

/************************************************************************/
/*									*/
/*		    VALUES FOR dbClient MESSAGE TYPES. Messages marked	*/
/*	as "NO BODY" contain ONLY the message type.			*/
/*									*/
/************************************************************************/

#define	DBC_MSG_WAKEUP		64	/* IOC Wakeup call from dbClient C->S...NO BODY! */
#define	DBC_MSG_SYMREQUEST	65	/* Symbol table request, C->S */
#define	DBC_MSG_SYMRESPONSE	66	/* Symbol table response, S->C */
#define	DBC_MSG_AWAKE		67	/* IOC response to Wakeup call S->C...NO BODY */
#define	DBC_MSG_SYMPURGE	68	/* Symbol table purge order, S->C */
#define	DBC_MSG_CRCRESPONSE	69	/* checksum response from IOC S->C 8-Jan-97 timestamp added */

/*
	There is no structure for the DBC_MSG_CRCRESPONSE because it is
	dangerous. After the message type, there is a two byte count of
	the number of sums in the message. The message stream for each sum
	will be the four byte key followed by the two byte CRC. If you make
	a struct like that on the IOC, it will work just fine. If you make a
	struct like that on the Sun, it will put two slack bytes between the
	key and the CRC. Therefore, let the coder beware.

	8-jan-97
	Body of message is as follows:

	numofSums(2 bytes)+[Key(4 bytes)+TSTP(4 bytes)+RCRC(2 bytes)...]

	30-Apr-1999

	Disregard the above comment! The CRC response will consist of a well-padded
	structure containing the CRC/Symbol pairs and the time stamp for the given point.
*/

#define	DBC_MSG_DWNLDREQ	70	/* request for database records S->C */
#define	DBC_MSG_DWNLDRESP	71	/* response for database download C->S */
#define	DBC_MSG_SDRREQUEST	72	/* request for SDR download S->C...NO BODY */
#define	DBC_MSG_SDRRESPONSE	73	/* SDR download response C->S */
#define	DBC_MSG_DBMODREQ	74	/* Point modification request S->C 8-Jan-97 timestamp added */
#define DBC_MSG_RECREQUEST	75	/* Record upload request, C->S */
#define DBC_MSG_OLDNOTIFY	76	/* Online download notify dataVu ->S...NO BODY */
#define DBC_MSG_OLDREQ		77	/* Online download request S->C
						Same structure as DBC_MSG_DWNLDREQ */
#define DBC_MSG_OLDRESP		78	/* Online download response C->S
						Same structure as DBC_MSG_DWNLDRESP */
#define DBC_MSG_NEWSYM		79	/* New record symbol S-> C
						Same structure as DBC_MSG_SYMRESPONSE */
#define DBC_MSG_BOOMERANG	80	/* Heartbeat between client and server */
#define	DBC_MSG_OLDELETE	81	/* On-line point delete dataVu -> S */
#define	DBC_MSG_OLDELRESP	82	/* On-line point delete S -> C */
#define	DBC_MSG_DELSTATUS	83	/* Server point deletion status S -> dataVu  */
#define	DBC_MSG_RESETNOTIFY	84	/* IOC reset message dataVu -> S... NO BODY */
#define	DBC_MSG_RESETACK	85	/* IOC reset acknowledge message S -> dataVu... NO BODY */
#define	DBC_MSG_PARTIALSDR	86	/* C -> S host sent partial SDR to dbserver */
#define	DBC_MSG_PARTSDRREQ	87	/* S -> C dbserver request partial SDR from client...NO BODY */
#define	DBC_MSG_OKTOUNMARK	88	/* S -> C IOC says it's OK to un-mark recently downloaded points */
#define DBC_MSG_FWDDBMOD	89	/* H(M) -> H(S) forwarded DBMODREQ message from master host to slave host */
/*
	Inter dbClient message types. Also used for dbClient <-> Utility communications
*/
#define	VI_MSG_VERIFYREQUEST	100    	/* sent to all other hosts */
#define	VI_MSG_VERIFYREPLY     	101    	/* received from hosts */
#define	VI_MSG_RECORDREQUEST	102    	/* sent to host with good rec */
#define	VI_MSG_RECORDREPLY     	103    	/* send record to requesting host */
#define	VI_MSG_IHAVENEWREC     	104    	/* sent to other hosts telling them to ask me for new record */
/*
	Utility comm msg types
*/
#define	UTIL_MSG_VERIFYNOW     	200    	/* utility tells dbClient to send verify request NOW */
#define	UTIL_MSG_TOGGLEVERBOSE	201    	/* toggle verbose flag */
#define	UTIL_MSG_BYEBYE	       	202    	/* close local fd */
#define	UTIL_MSG_JOURNAL_ON	203    	/* 28-Mar-96 Turn journaling on */
#define	UTIL_MSG_JOURNAL_OFF	204    	/* 28-Mar-96 Turn journaling off */
#define UTIL_MSG_RESETNOTIFY	205	/* 5-May-1999 from dataVu to IOC via dbClient see DBC_MSG_RESETNOTIFY */
#define UTIL_MSG_OLDNOTIFY	206	/* 5-May-1999 from dataVu to IOC via dbClient see DBC_MSG_OLDNOTIFY */
#define UTIL_MSG_OLDELETE	207	/* 5-May-1999 from dataVu to IOC via dbClient see DBC_MSG_OLDELETE */
/************************************************************************/
/*									*/
/*		    STATUS VALUES FOR DOWNLOAD OPERATIONS		*/
/*									*/
/************************************************************************/

typedef enum
	{
	DSTAT_OK,		/* All is OK */
	DSTAT_DONE,		/* Download is complete */
	DSTAT_FULL,		/* IOC is full...send no more */
	DSTAT_ERROR		/* Download error... */
	} DBC_DWNLD_STATUS;

/************************************************************************/
/*									*/
/*			CONNECTION REQUEST				*/
/*									*/
/*	The dbServer tasks transmits this message to advertise the	*/
/*	names of the resources that he has. It always contains as	*/
/*	many resources as will fit within the maximum message size.	*/
/*	This message is the only one that is broadcast. The broadcast	*/
/*	is constrained to be on the same network as the originating	*/
/*	node.								*/
/*									*/
/************************************************************************/

typedef struct
	{
	short	redNumRequests;	/* Number of following request structs */
	} RED_CONNREQ_HDR;

typedef struct
	{
	char	redReqName[16];	/* Resource name, null terminated */
	short	redReqPort;	/* Driver TCP port in requesting node */
	} RED_CONNREQ_REQ;

/************************************************************************/
/*									*/
/*			CONNECTION REPLY				*/
/*									*/
/*	This message is transmitted in reply to the connection		*/
/*	request. Its purpose is to tell the requestor about the		*/
/*	list of resources that we have that are ready to link up	*/
/*	with him. This message is always directed to the node that	*/
/*	made the request. It is never broadcast.			*/
/*									*/
/************************************************************************/

typedef struct
	{
	short	redNumReplies;	/* Number of following reply structs */
	} RED_CONNREPLY_HDR;

typedef struct
	{
	char	redReplyName[16];	/* Resource name, null terminated */
	short	redReplyPort;		/* Driver TCP port in replying node */
	} RED_CONNREPLY_REPLY;

/************************************************************************/
/*									*/
/*			REQUEST FOR REDUNDANT UPDATE			*/
/*									*/
/*	This message is sent by the slave IOC to the master when	*/
/*	there has not been a redundant update for a record in the	*/
/*	last 5 minutes. The purpose of this mechanism is to make	*/
/*	sure that we are up to date for passive points (like output	*/
/*	points in supervisory mode) and for other types of points	*/
/*	where the inputs are not changing.				*/
/*									*/
/************************************************************************/

typedef struct
	{
	char	redPointName[PVNAME_SZ];	/* Point name */
	} RED_ASKUPDT;

/************************************************************************/
/*									*/
/*			DBSERVER CONNECTION REQUEST			*/
/*									*/
/*	After they have exchanged lists of resources, dbServers then	*/
/*	exchange TCP port numbers so they can set up their own stream	*/
/*	for communicating databse updates. Regardless of the number	*/
/*	of resources that are shared between two IOCs, or the number	*/
/*	of TCP streams that may be used to connect the drivers in	*/
/*	them, only one stream is used between the two dbServers.	*/
/*									*/
/************************************************************************/

typedef struct
	{
	short	redDbServerPort;	/* TCP port number for dbServer */
	} RED_IOCLINKREQ;

/************************************************************************/
/*									*/
/*			INTER-IOC REDUNDANT UPDATE			*/
/*									*/
/*	This message type is sent from the IOC that controls a		*/
/*	resource to its slave. It is used to update the fields		*/
/*	in the slave unit to match the ones in the master.		*/
/*	It is immediately followed in the output stream by the		*/
/*	point name.							*/
/*									*/
/************************************************************************/

typedef struct
	{
	short		nameLen;	/* Point name */
	short		crc;		/* Checksum */
	unsigned char	ack;		/* Alarm ack flag */
	unsigned char	promptMod;	/* Prompted field modified flag */
	short		numFields;	/* Number of fields in this msg */
	} RED_DBUPDATE;

/******
	The body of an inter-IOC redundant update consists of the following
	things for each field that we are updating:

		1.	short:	length of data
		2.	short:	offset of data
		3.	string:	new value. Length is given in the first field.

	If the high order bit of the length is set (0x8000), then that is a special
	indication that we are dealing with an update to the ROC buffer. In
	that case, we are dealing with the following layout:

		1.	short:	length of ROCI field (low order 14 bits)
		2.	short:	offset of ROCB field
		3.	short:	offset of VAL field.
		4.	short:	length of VAL field
		5.	short:  offset of ROCI field
		6.	string:	new value for ROCI. Length is given in the
			first field.

	dbServer will first update the ROCI field as normal, and then will
	perform the action ROCB[ROCI] = VAL. The VAL field can be either floats
	or doubles (length = 4 or 8, respectively). The ROC buffer is always floats.
	This can eventually grow into a generalized array handler if we ever need it.

	If the next to high order bit of the length is set (0x4000), then that
	is another special indication. This time, the field that has been modified
	requires special processing. In this case, we are dealing with the following
	layout:

		1.	short:	length of data (low order 14 bits)
		2.	short:	special processing number extracted from fldDes.
		3.	short:	offset of data
		4.	string: new value. Length is given in the first field.

	At the current time, having both upper bits set (0xC000) is illegal. That
	state can be used later, if necessary.

	Here is a mask and a set of values that can be used for comparing this stuff.
******/

#define	SPECIAL_MASK	0xC000	/* Both bits */
#define ROCI_UPDATE	0x8000	/* ROCI field being changed */
#define SPECIAL_PROC	0x4000	/* Field requires special processing */

/************************************************************************/
/*									*/
/*		REQUEST FOR UPDATE OF ALL PROMPTED FIELDS		*/
/*									*/
/*	This message is sent from the slave IOC to the master to	*/
/*	indicate that his record CRC does not match the masters.	*/
/*	The master is to respond by performing an update of all		*/
/*	prompted fields.						*/
/*									*/
/************************************************************************/

typedef struct
	{
	char	pointName[28];	/* Request point's name */
	} RED_PROMPTREQ;

/************************************************************************/
/*									*/
/*	The following message structures are for dbClient/dbServer	*/
/*	communications. The message type numbers are defined above	*/
/*	along with the redundant IOC messages types.			*/
/*									*/
/************************************************************************/


/************************************************************************/
/*	This mesage is sent by the IOC when it wants to get a database.	*/
/*	'msgLastName' is the name of					*/
/*	the last PV received or NULL if this is the first request. The	*/
/*	'msgStatus' field is used to communicate the status of the	*/
/*	download process.						*/ 
/************************************************************************/

typedef struct
	{
	char			msgLastName[DBC_PTNAMESIZE];
	DBC_DWNLD_STATUS	msgStatus;
	} DBC_DWNLD_REQUEST, *DBC_DWNLD_REQUEST_PTR;

#define	DBC_REQSIZE sizeof(DBC_DWNLD_REQUEST)

/************************************************************************/
/*	This message contains the records requested in the DBC_DWNLDREQ	*/
/*	message. This is actually a message header because of the	*/
/*	variable length nature of the database records.	Each database	*/
/*	record sent is preceeded by the point type index.		*/
/************************************************************************/

typedef	struct
	{
	short		msgNumberSent;
	} DBC_DWNLD_RESP, *DBC_DWNLD_RESP_PTR;

#define	DBC_RESPSIZE sizeof(DBC_DWNLD_RESP)
/****************************************************************************/
/*	This message contains a 4 byte status value which is not used...yet and */
/*	is used to signal dbClient that it is OK to un-mark recently downloaded */
/*	points.                                                                 */
/****************************************************************************/
typedef	struct
	{
	int	ackStatus;
	} DBC_DWNLD_OK, *DBC_DWNLD_OK_PTR;
#define	DBC_DWNLDOKSIZE	sizeof(DBC_DWNLD_OK)

/************************************************************************/
/*	This message contains the sdrHeader structure followed by the	*/
/*	sdrHeader data.							*/
/************************************************************************/

typedef	struct
	{
	unsigned int		msgStatus;	/* EOF or next offset in file */
	}DBC_SDRRESP,*DBC_SDRRESP_PTR;

/************************************************************************/
/*	This structure is the header for the DBC_MSG_DBMODREQ message	*/
/*									*/
/*	Field modification messages are in the following order:		*/
/*									*/
/*	DBC_DBMODHDR							*/
/*		|							*/
/*		DBC_FIELD #1						*/
/*		value	  #1						*/
/*		DBC_FIELD #2						*/
/*		value     #2						*/
/*		.							*/
/*		.							*/
/*		.							*/
/*		DBC_FIELD #n	n is dbmodNFields			*/
/*		value     #n						*/
/************************************************************************/
typedef	struct	dbmodhdr
{
  char		dbmodName[PVNAME_SZ];		/* point name to be modified */
  short		dbmodType;	       		/* 10-May-1994 point type field */
  short		dbmodNFields;			/* number of fields to modify */
  unsigned short	dbmodCRC;      		/* CRC after modification */
  short		dbmodSize;	       		/* size of field + value stuff that follows this header. 15-Sep-1994 GCT */
  int		dbmodPointId;			/* point ID for symbol table update */
  int		dbmodTimeStamp;			/* time stamp from IOC dbCommon */
}DBC_DBMODHDR, *DBC_DBMODHDR_PTR;

#define	DBC_MODHDRSIZE	sizeof(DBC_DBMODHDR)

typedef	struct	dbmodfield
	{
	char	fieldName[4];			/* EPICS field name */
						/* NOT NULL TERMINATED! BE CAREFUL! */
	}DBC_DBFIELD,*DBC_DBFIELD_PTR;
#define	DBC_FIELDSIZE	sizeof(DBC_DBFIELD)

/************************************************************************/
/*									*/
/*	      Forward of DBC_MSG_DBMODREQ				*/
/*									*/
/*	This header contains the IOC name from the originating IOC	*/
/*  and the size of the original message. After this header is the	*/
/*  origianal message.							*/
/*									*/
/*	GCT 13-Jan-97							*/
/************************************************************************/
typedef struct dbfwdhdr
{
  char   fwdName[DBC_IOCNAMESIZE];		/* name of originating IOC */
  short  fwdSize;				/* size of following message */
  char   pad[6];				/* padd to next 8 byte boundary */
} DBC_FWDHDR, *DBC_FWDHDR_PTR;
#define DBC_FWDHDRSIZE sizeof(DBC_FWDHDR)	/* size of above structure */

/************************************************************************/
/*									*/
/*			REQUEST FOR SYMBOL TABLE			*/
/*									*/
/*	This message is transmitted from the workstation to the IOC.	*/
/*	It requests that the IOC send its symbol table.			*/
/*									*/
/************************************************************************/

typedef struct
	{
	char	symLastName[PVNAME_SZ];		/* Last name he got, null on first msg */
	short	numSyms;			/* Number of symbols he wants */
	}DBC_SYMREQ, *DBC_SYMREQ_PTR;

/************************************************************************/
/*									*/
/*			SYMBOL TABLE RESPONSE				*/
/*									*/
/*	This message consists of a header, containing the number of	*/
/*	symbols in response. The header is followed by a number of	*/
/*	structures, each structure containing a name and its four	*/
/*	byte translation.						*/
/*									*/
/************************************************************************/

typedef struct
	{
	short	numSymbols;			/* Number symbols to follow */
	}DBC_SYMRESPHDR, *DBC_SYMRESPHDR_PTR;

typedef struct
	{
	char	symName[PVNAME_SZ];		/* The name */
	int	symTrans;			/* Four byte translation */
	}DBC_SYMRESPNAME, *DBC_SYMRESPNAME_PTR;


/************************************************************************/
/*									*/
/*			UPLOAD REQUEST MESSAGE				*/
/*									*/
/*	This message is sent from the workstation to the IOC to		*/
/*	request a message upload. The IOC responds with an uplink	*/
/*	field mod message for all prompted fields if it owns the	*/
/*	resource for the point. It ignores the message if it doesn't	*/
/*	own the resource.						*/
/*									*/
/************************************************************************/

typedef struct
	{
	char	requestName[PVNAME_SZ];
	}DBC_UPLINKREQ, *DBC_UPLINKREQ_PTR;


/************************************************************************/
/*									*/
/*			ONLINE DELETION MESSAGES			*/
/*									*/
/*	DBC_OLDELETE:	Message code DBC_MSG_OLDELETE			*/
/*	DBC_DELSTATUS:	Message code DBC_MSG_DELSTATUS			*/
/*	DBC_DELRESP:	Message code DBC_MSG_DELRESP			*/
/*									*/
/************************************************************************/

typedef struct
{
  char nameToDelete[PVNAME_SZ];
  int originatorFd;			/* 6-May-1999 GCT added to echoed in reply message */
} DBC_OLDELETE, *DBC_OLDELETE_PTR;

typedef struct
{
  short	deletionStatus;
  short numRefs;			/* 6-May-1999 GCT number of points referencing this one */
  int originatorFd;			/* 6-May-1999 GCT fd of originating dataVu */
#define	DELETION_OK	0
#define DELETION_ONSCAN	1
#define DELETION_OPEN	2
#define	DELETION_NOREC	3
#define	DELETION_REF	4

/*
  If the record is referenced, the message is followed by that number of names.
  Each name is PVNAME_SZ bytes long, null terminated.
*/

} DBC_DELSTATUS, *DBC_DELSTATUS_PTR;

typedef struct
{
  char	deletedName[PVNAME_SZ];
  int	deletedSym;
} DBC_DELRESP, *DBC_DELRESP_PTR;


/*
  30-Apr-1999 GCT

  This is the CRC information structure. It is sent along with 31 others to
  dbClient on the Workstation. Since the structure is padded it should look
  the same as the previous incarnation with the addition of 2 pad bytes. This
  structure immdeatly follows the message type and the CRC count.
*/

typedef struct
	{
	short	numCrcs;			/* Number symbols to follow */
	}DBC_CRCRESPHDR, *DBC_CRCRESPHDR_PTR;

typedef struct
{
  int crcKey;			/* Actually the address of this record in the PVD */
  int crcTimeStamp;		/* time stamp of record. */
  unsigned short crcCRC;       	/* CRC from this record. */
  short fill;			/* padding to fill-out structure */
} DBC_CRCINFO,*DBC_CRCINFO_PTR;

/*
  4-May-1999 GCT

  This is the new IOC/RTU reset message. It is sent after the receipt of a message
  from dataVu. The old message consisted of only the message type. This message now
  contains the file descriptor of the dataVu that originated the message. This is
  done so that when a reset acknowledge message is recieved, dbClient will know which
  dataVu to forward the ack message to. This message will be re-used for the reset ack
  message.
*/
typedef struct
{
  int originatorFd;
}DBC_RESETREQ,*DBC_RESETREQ_PTR;

#endif	IncrdmsgsH
