/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/* serhwLib.h - definitions for Serial Hardware Interface Library */

#ifndef __SERHWLIB__
#define __SERHWLIB__
#include <vxWorks.h>

/* some data type aliases - to simplify casting */

#define UCP(x) (UCHAR*)(x)
#define USP(x) (USHORT*)(x)
#define ULP(x) (ULONG*)(x)
#define IN2IV(i) INUM_TO_IVEC(i)
#define V_IN2IV(i) ((VOIDFUNCPTR*)IN2IV(i))

/* control structure for i8250 uart */

struct hw8250
{
  UCHAR volatile *rb;	/*rcv buf*/
  UCHAR volatile *thr;	/*xmt hld reg*/
  UCHAR volatile *ier;	/*int ena reg*/
  UCHAR volatile *iir;	/*int id reg*/
  UCHAR volatile *lcr;	/*line ctl reg*/
  UCHAR volatile *mcr;	/*mdm ctl reg*/
  UCHAR volatile *lsr;	/*line stat reg*/
  UCHAR volatile *msr;	/*mdm stat reg*/
  UCHAR _ier;	/*shadow int ena reg*/
  UCHAR _mcr;	/*shadow mdm ctl reg*/
  UCHAR _rxm_;	/*rx char mask*/
};

/* control structure for z8530 dusart */

struct hw8530
{
  UCHAR volatile *cr;	/*cmd reg*/
  UCHAR volatile *dr;	/*data reg*/
  UCHAR volatile *crA;	/*chan A cmd reg*/
  UCHAR volatile *crB;	/*chan B cmd reg*/
  UCHAR _wr1;	/*shadow wr1 reg*/
  UCHAR _wr3;	/*shadow wr3 reg*/
  UCHAR _wr5;	/*shadow wr5 reg*/
  UCHAR _rxm_;	/*rx char mask*/
  USHORT _VEC;	/*shadow vector num*/
};

/* control structure for z85230 dusart on IP */

struct hw85230ip
{
  UCHAR volatile *cr;	/*cmd reg*/
  UCHAR volatile *dr;	/*data reg*/
  UCHAR volatile *crA;	/*chan A cmd reg*/
  UCHAR volatile *crB;	/*chan B cmd reg*/
  UCHAR _wr1;	/*shadow wr1 reg*/
  UCHAR _wr3;	/*shadow wr3 reg*/
  UCHAR _wr5;	/*shadow wr5 reg*/
  UCHAR _wr10;	/*shadow wr10 reg*/
  UCHAR _rxm_;	/*rx char mask*/
  USHORT _VEC;	/*shadow vector num*/
};

/* control structure for m68681 dual uart */

struct hw68681
{
  UCHAR volatile *mr1;	/*mode1 reg*/
  UCHAR volatile *mr2;	/*mode2 reg*/
  UCHAR volatile *csr;	/*clk sel reg*/
  UCHAR volatile *acr;	/*aux ctrl reg*/
  UCHAR volatile *cr;	/*cmd reg*/
  UCHAR volatile *sr;	/*stat reg*/
  UCHAR volatile *rb;	/*rcv buf*/
  UCHAR volatile *tb;	/*xmt buf*/
  UCHAR volatile *ipr;	/*inp port reg*/
  UCHAR volatile *ipcr;	/*inp port chg reg*/
  UCHAR volatile *opcr;	/*out port cfg reg*/
  UCHAR volatile *sopr;	/*set out port reg*/
  UCHAR volatile *ropr;	/*rst out port reg*/
  UCHAR volatile *isr;	/*int stat reg*/
  UCHAR volatile *imr;	/*int mask reg*/
  UCHAR volatile *ivr;	/*int vec reg*/
  UCHAR volatile *_imr;	/*-> shared imr shadow reg*/
  UCHAR volatile *_acr;	/*-> shared acr shadow reg*/
  UCHAR _iRX;	/*rx int ena mask*/
  UCHAR _iTX;	/*tx int ena mask*/
  UCHAR mDTR;	/*DTR stat mask*/
  UCHAR mDSR;	/*DSR stat mask*/
  UCHAR mRTS;	/*RTS stat mask*/
  UCHAR mCTS;	/*CTS stat mask*/
  UCHAR mDCD;	/*DCD stat mask*/
  UCHAR _rxm_;	/*rx char mask*/
  USHORT _VEC;	/*shadow vector num*/
};

/* control structure for scc2698b octal uart */

struct hw2698
{
  UCHAR volatile *mr1;	/*mode1 reg*/
  UCHAR volatile *mr2;	/*mode2 reg*/
  UCHAR volatile *sr;	/*stat reg*/
  UCHAR volatile *csr;	/*clk sel reg*/
  UCHAR volatile *cr;	/*cmd reg*/
  UCHAR volatile *rhr;	/*rcv hld reg*/
  UCHAR volatile *thr;	/*xmt hld reg*/
  UCHAR volatile *ipcr;	/*inp port chg reg*/
  UCHAR volatile *acr;	/*aux ctrl reg*/
  UCHAR volatile *isr;	/*int stat reg*/
  UCHAR volatile *imr;	/*int mask reg*/
  UCHAR volatile *ipr;	/*inp port reg*/
  UCHAR volatile *opcr;	/*out port cfg reg*/
  UCHAR volatile *_imr;	/*-> shared imr shadow reg*/
  UCHAR volatile *_acr;	/*-> shared acr shadow reg*/
  UCHAR _iRX;	/*rx int ena mask*/
  UCHAR _iTX;	/*tx int ena mask*/
  UCHAR mCTS;	/*CTS stat mask*/
  UCHAR _rxm_;	/*rx char mask*/
};

/* control structure for ALPHI ICC8/30 */

struct hwicc830
{
  UCHAR	chan_idx;	/* channel index */
  UINT	brd_num;	/* Board number.. 0, 1, 2, or 3 */
  UCHAR _rxm_;		/* rx char mask */
  USHORT _VEC;		/* shadow VME vector num */
  UCHAR	*stshead;
  UCHAR	*ststail;
  UCHAR	*rcvhead;
  UCHAR	*rcvtail;
  UCHAR	*txhead;
  UCHAR	*txtail;
};

/*
  control structure for higher level access to
  standard hardware routines of any characteristic module
*/

struct _hwifc
{
  FUNCPTR     hwOpen;
  VOIDFUNCPTR hwClose;
  VOIDFUNCPTR hwConfig;
  VOIDFUNCPTR hwModem;
  VOIDFUNCPTR hwWrite;
  FUNCPTR     hwRead;
  FUNCPTR     hwRxStatus;
  FUNCPTR     hwRxReady;
  FUNCPTR     hwTxEmpty;
  VOIDFUNCPTR hwRxIntCtl;
  VOIDFUNCPTR hwTxIntCtl;
  VOIDFUNCPTR hwStIntCtl;
  VOIDFUNCPTR hwTxUndCtl;
  VOIDFUNCPTR hwMrkFlgCtl;
  VOIDFUNCPTR hwSetAddr;
  VOIDFUNCPTR hwISR;
  void        *hwblk;
};

/* enumerated constants to identify a particular characteristic module */

enum _dvrtypes
{
  dvrNONE,
  dvr8530,
  dvr68681,
  dvr2698,
  dvr8250,
  dvr85230ip,
  dvrPT330,
  dvricc830,
  dvrMAX
};

/*
  control structure for mapping the physical channels of a given board
  to logical channels accessible through the serial interface library
*/

struct _hwline
{
  enum _dvrtypes dvrType;
  int chanBase;		/*logical channel id of 1st physical channel*/
  int chanId;		/*logical channel id of current physical channel*/
  UCHAR *vmeBaseAddr;	/*base address of card(chip) in VME bus address space*/
  UCHAR vmeVecNum;	/*assigned vector number*/
  UCHAR sysIntLvl;	/*int level to unmask -not used by all interfaces*/
  struct _hwifc *ifc;	/*-> hardware interface dispatch table*/
  UCHAR *_imr;		/* array of shared int mask reg readbacks if needed */
  UCHAR *_acr;		/* array of shared aux ctl reg readbacks if needed */
};

/*
  enumerated mask values and shift counts used by low-level routines
  for packing and unpacking serial characteristics selected by applications
*/

enum
{
  mskBAUD=0xf000, shfBAUD=12,
  mskBITS=0x0f00, shfBITS=8,
  mskPAR=0x00f0,  shfPAR=4,
  mskSTOP=0x000f, shfSTOP=0
};
 
/*
  enumerated constants for use by upper level applications to select
  serial line characteristics -
  also used as <zero-rel array indices> by middle level routines
  and used in packed form by lower level routines
*/

enum { b57400, b38400, b19200, b9600, b4800, b2400, b1200, b300, bMAX };/*4bits*/
enum { bits8, bits7, bits6, bits5, bitsMAX };                   /*2bits*/
enum { parNONE, parODD, parEVEN, parMAX };                      /*2bits*/
enum { stop1, stop2, stopMAX };                                 /*1bit*/

#define MKCNFG(bd,cs,pr,st) \
((((bd)<<shfBAUD)&mskBAUD)|(((cs)<<shfBITS)&mskBITS)|(((pr)<<shfPAR)&mskPAR)|(((st)<<shfSTOP)&mskSTOP))

/* enumerated constants for standardized access to modem control/status bits */

enum _mdmbits
{
  mdmDTR=0x8000, mdmDSR=0x4000,
  mdmRTS=0x2000, mdmCTS=0x1000,
  mdmDCD=0x800,  mdmRI=0x400,
  errFRAME=0x80, errPARITY=0x40, errOVRRN=0x20, errCRC=0x10,
  stsTXEMPTY=0x08, stsRESCD1=0x04, stsRESCD0=0x02, stsENDFRAME=0x01
};

/* prototypes of published routines */

STATUS serhwLibInit( int );
STATUS serhwOpen( int );
void serhwClose( int );
void serhwWrite( int, UCHAR );
void serhwForce( int, UCHAR );
UCHAR serhwRead( int );
int serhwFlush( int );
void serhwMapShow( void );

#endif /*__SERHWLIB__*/
