/*serverControl.h,v 1.1.1.1 1999/08/03 14:41:29 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	CHANNEL ACCESS SERVER CONTROL STRUCTURES		*/
/*								*/
/*	These structures are used by the IOC channel access	*/
/*	server to keep track of what it is doing. The basic	*/
/*	concepts are similar to the control blocks outlined	*/
/*	in channelControl.h, but are simpler here.		*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifndef TX_BUFF_SIZE

#ifdef	vxWorks
#include "dbAccess.h"
#endif	vxWorks

#include "dbAddr.h"

#define TX_BUFF_SIZE		4096	/* Transmit buffer, each client */


/****************************************************************/
/*								*/
/*		CLIENT CONTROL BLOCK (CCB)			*/
/*								*/
/*	There is one channel control block for each client.	*/
/*	In the agent architecure, a client is a node like	*/
/*	a workstation or another IOC.				*/
/*								*/
/****************************************************************/

typedef struct cCl
    {
    struct in_addr clientInet;		/* Inet address */
    int		clientSession;		/* Session handle */
    int		clientFd;		/* TCP socket FD */
    int		txBuffSize;		/* Curr buffer size */
    int		clientTxBuff;		/* Current Tx buffer size */
    int		clientRxBuff;		/* Current Rx buffer size */
    unsigned long lastRxTime;		/* Time we got the last message */
    short	clientRxTopped;		/* Rx buffer size maxout flag */
    short	handlerMsgRcvd;		/* Handler message received? */
    short	trendHandler;		/* Is the node a trend handler? */
    short	alarmHandler;		/* Is the node an alarm handler? */
    short	alarmTravPending;	/* Is an alarm traversal pending? */
    short	trendTravPending;	/* Is a trend traversal pending? */
    struct cCl	*nextClient;		/* Next in chain */
    struct mCb	*firstChan;		/* First channel this client */
    char	txBuff[TX_BUFF_SIZE];	/* Transmit buffer */
    } CCB;


/****************************************************************/
/*								*/
/*		FIELD CONTROL BLOCK (FCB)			*/
/*								*/
/*	There is an FCB for every field that has a channel	*/
/*	attached to it, even if the channels have different	*/
/*	data types. The FCB chains to another FCB when channels	*/
/*	are open on more than one field in a record. Each FCB	*/
/*	points to the head of a list of MCBs that define	*/
/*	each channel open on that particular field.		*/
/*								*/
/****************************************************************/

typedef struct fCb
    {
    struct dbAddr	dbAddr;		/* see dbAccess.h */
    struct fCb		*nextFCB;	/* next field, this record */
    struct fCb		*prevFCB;	/* prev field, this record */
    struct mCb		*firstChan;	/* first chan, this field */
    short		currMask;	/* Overall event mask */
    } FCB;


/****************************************************************/
/*								*/
/*		MONITOR CONTROL BLOCK				*/
/*								*/
/*	There is a monitor control block for each channel	*/
/*	opened by a client.					*/
/*								*/
/****************************************************************/

typedef struct mCb
    {
    short		chanMask;	/* Event mask this channel */
    short		dataType;	/* Data type */
    int			numElem;	/* Array size */
    CCB			*client;	/* Client for this channel */
    struct mCb		*nextMCB;	/* Next MCB, this field */
    struct mCb		*nextClient;	/* Next MCB, this client */
    struct mCb		*prevClient;	/* Prev MCB, this client */
    FCB			*fldCntl;	/* FCB at start of chain */
    int			handle;		/* Client's handle */
    } MCB;


/****************************************************************/
/*								*/
/*			FUNCTION PROTOTYPES			*/
/*								*/
/****************************************************************/

FCB	*newFCB(void);
void	deleteFCB(FCB *oldFCB);
MCB	*newMCB(void);
void	deleteMCB(MCB *oldMCB);
#endif
