/* tankfarm.h */
/* share/epicsH  @(#)tankfarm.h   1.00 3/24/93 */
  
/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************************
 *
 *  FILE    :   tankfarm.h
 *  DATE    :   22 Mar 93
 *  AUTHOR  :   emsteel
 *  PURPOSE :   defines and external defs for the tank farm product 
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : ems
 *  REVISION DATE          : 11 nov 93
 *  REVISION:  deleted EPICS_CLOCK_RATE, will use vxTicksPerSecond as 
 *             defined in .../share/epicsH/dbDefs.h
 *
 *  REVISED BY             : ems
 *  REVISION DATE          : 14 dec 93
 *  REVISION:  added #define TF_SPECIAL_DIGITAL_CMD
 *             added FirstSpecialDigitalCMd to struct tfcmd
 *             added #define TF_INIT_CMD, TF_UNDEF_CMD
 *
 *  REVISED BY             : ems
 *  REVISION DATE          : 19 May 94
 *  REVISION:  make FindAPITable and FindStrappingTable return long not void
 *
****************************************************************************/


/****************************************************************************
*
* Programmer's Notes:
*
*   ComputeTFFlowRate () - see tankfarm.c
*   ComputeTFVolume () - see tankfarm.c
*   ComputeTFLevel () - see tankfarm.c
*   FindAPIcf () - see tankfarm.c
*   FindAPITable () - see tankfarm.c
*   FindStrappingTable () - see tankfarm.c
*   TankFarmPostAlarm () - see tankfarm.c
*   tanktbls.h - defines the strapping table, api table ... entries
*
*
****************************************************************************/


#include <tanktbls.h>

#ifndef INCTankFarmH
#define INCTankFarmH 1


/* DEFINE THE STRUCTURE FOR PROCESSING A TANKFARM COMMAND
      used as arg to GetTFCommand () and ResetTFCommand ()
      see this file and tankfarm.c
*/

struct tfcmd
{
   short          NumCmds;                  /* number of commands for record */
   short          *FirstDigitalCmd;         /* ptr to first digital command  */
   short          *FirstSpecialDigitalCmd;  /* ptr to 1st special dig cmd */
   char           *FirstCmdString;          /* ptr to first command string */
   char           *CmdString;               /* ptr to cmd string buffer */
};

/* DEFINE RETURNS FOR GetTFCommand ()
   Note: positive numbers are the commands as defined in
         each records .c file and returned by GetTFCommand ()
*/

#define  NO_TF_COMMAND           0
#define  ILLEGAL_TF_STRING_CMD   -1
#define  TF_NO_CMD_TYPE          0
#define  TF_STRING_CMD           1
#define  TF_DIGITAL_CMD          2
#define  TF_SPECIAL_DIGITAL_CMD  3	/* set by other points */

/* defines to be used for the PCMD (previous command) field */

#define  TF_INIT_CMD	"none"
#define  TF_UNDEF_CMD	"undefined"


/* DEFINE GENERAL FUNCTIONS FOR A TANK FARM */

/*-------------------------------------------------------------------------*/

   /* compute a flow rate

      long ComputeTFFlowRate (char *recname, char *vol_field_name,
                              char *flow_field_name, ushort vol_units,
                              ushort flow_units, double curr_volume,
                              double prev_volume, unsigned long *curr_time,
                              unsigned long *last_time, double *flow_rate)

         recname;           name of record 
         vol_field_name;    name of the volume units field
         flow_field_name;   name of the flow rate units field 
         vol_units;         index to volume units list 
         flow_units;        index to flow rate units list 
         curr_volume;     
         prev_volume;
         curr_time;         in system ticks
         last_time;         in system ticks
         flow_rate;         return the flow rate 

   */

long ComputeTFFlowRate (char *recname, char *vol_field_name, char *flow_field_name, 
			       unsigned short vol_units, unsigned short flow_units, double curr_volume,
			       double prev_volume, unsigned long *curr_ticks, unsigned long  *last_ticks, 
			       double *flow_rate);
      
/*-------------------------------------------------------------------------*/

   /* compute a level from a strapping table

      void ComputeTFLevel (caddr_t pstr, int num_entries, double *curr_level,
                                                       double curr_volume)

           pstr - ptr to the strapping table structure
           num_entries - number of entries in table
           curr_level - this is returned
           curr_volume - current volume from sensor
   */

void ComputeTFLevel (caddr_t pstr, int num_entries, double *curr_level,
			    double curr_volume);

/*-------------------------------------------------------------------------*/

   /* compute a volume from a strapping table

      void ComputeTFVolume (caddr_t pstr, int num_entries, double curr_sval,
                                                       double *curr_volume)

           pstr - ptr to the strapping table structure
           num_entries - number of entries in table
           curr_sval - current sensor value
           curr_volume - this is returned
   */

void ComputeTFVolume (caddr_t pstr, int num_entries, double curr_sval,
			     double *curr_volume);

/*-------------------------------------------------------------------------*/

   /* find the API thermal correction factor

      FindAPIcf (char *api_tbl, int num_entries, double ave_temp,
                 double sp_grav, double *texp, char *recname,
                 char *temp_field_name, short temp_units)

         api_tbl     - ptr to API table
         num_entries - number of entries in table
         ave_temp    - average temperature of product
         sp_grav     - specific gravity of product
         texp        - thermal correction factor returned
         recname     - ptr to record name
         temp_field_name = ptr to field name
         temp_units  - current choice in choice_set
   */

long FindAPIcf (caddr_t api_tbl, int num_entries, double ave_temp, double sp_grav,
		       double *texp, char *recname, char *temp_field_name,
		       short temp_units);

/*-------------------------------------------------------------------------*/

   /* given the name of an API table, return a ptr to it and the
      number of entries in the table

      long FindAPITable (char *apit, caddr_t papi, int *num_entries)

   	   apit - ptr to name of the API table 
	      papi - init this ptr in the record to the address of the
                API table
         num_entries - return number of table entries here
   */

long FindAPITable (char *apit, caddr_t *papi, int *num_entries);

/*-------------------------------------------------------------------------*/

   /* find a named strapping table and initialize ptr to it in a record
      and the number of table entries

      long FindStrappingTable (char *sttn, caddr_t pstr, int *num_entries)

           sttn - ptr to name of strapping table to find
           pstr - init this ptr in the record to the address of the named
                  strapping table
           num_entries - return number of table entries here
   */

long FindStrappingTable (char *tablename, caddr_t *pstr, int *num_entries);

/*-------------------------------------------------------------------------*/

   /* get a tanfarm command

      int GetTFCommand (struct tfcmd *ptfcmd, int *cmd_type)

         ptfcmd - ptr to tankfarm command structure as defined this file
         cmd_type - digital or string command

   */

int GetTFCommand (struct tfcmd *ptfcmd, int *cmd_type);

/*-------------------------------------------------------------------------*/

   /* reset a tankfarm command

      void ResetTFCommand (struct tfcmd *ptfcmd)

         ptfcmd - ptr to tankfarm command structure as defined this file

   */

void ResetTFCommand (struct tfcmd *ptfcmd);

/*-------------------------------------------------------------------------*/

   /* post alarms for tank farm commands

      void TankFarmPostAlarm (struct dbCommon *precord,
                              unsigned short stat, unsigned short sevr)

           precord - ptr to tank farm record, actually ptr to common data
                found at start of all record types
           stat - the alarm status to be posted
           sevr - the alarm severity to be posted
   */

void TankFarmPostAlarm (struct dbCommon *precord,
			     unsigned short stat, unsigned short sevr);

/*-------------------------------------------------------------------------*/

   /* round a value
      if < 0.25 round down to nearest integral
      if < 0.75 round down to 0.5
      else round up to nearest integral

      double TankFarmRound (double value)

   */

double TankFarmRound (double value);

/*-------------------------------------------------------------------------*/

   /* compute the differnece in seconds between to system tick values 

      double TFTicksDiffInSeconds (unsigned long *pT1, unsigned long *pT2)

         NOTE: diff = T1 - T2
   */

double TFTicksDiffInSeconds (unsigned long *pT1, unsigned long *pT2);


/*-------------------------------------------------------------------------*/

   /* convert system time ticks into seconds and return it as a double

      double TFTicksToSeconds (unsigned long *pTicks)
   */

double TFTicksToSeconds (unsigned long *pTicks);


/* DEFINES FOR TANKFARM ERRORS, see errMdef.h
   Note: value must be odd
*/

#define S_tank_StTbl_Err         (M_tank | 1)   /* no access to strapping_table */
#define S_tank_TankValve_Err     (M_tank | 3)	/* err defining dbAddr for a tank valve */
#define S_tank_APITbl_Err        (M_tank | 5)   /* no access to API table */
#define S_tank_ROL_Err           (M_tank | 7)   /* unknown ROL value */
#define S_tank_Line_Err          (M_tank | 9)   /* illegal line mode configuration */
#define S_tank_ValveFault        (M_tank | 11)  /* mbbi fault pv returns fault for valve */
#define S_tank_PotDropReady      (M_tank | 13)  /* pot has dropped ready state */
#define S_tank_TankLevel_Err     (M_tank | 15)  /* failed to get level from levelpv */


/* DEFINES FOR TANKFARM VALVES states */

#define VALVE_OPENED            0
#define VALVE_CLOSED            1
#define VALVE_OPENING           2
#define VALVE_CLOSING           3
#define VALVE_JOG_OPEN          4
#define VALVE_JOG_CLOSE         5
#define VALVE_POSITION_OPENING  6
#define VALVE_POSITION_CLOSING  7
#define VALVE_FAULT             8
#define VALVE_UNKNOWN           9
#define VALVE_POSITIONED        10
#define NO_VALVE                (VALVE_POSITIONED + 1)

/* defines for pump states */

#define	PUMP_RUNNING		0
#define	PUMP_STOPPED		1
#define	PUMP_STARTING		2
#define	PUMP_STOPPING		3
#define	PUMP_FAULT		4
#define	PUMP_MIN_OFF		5

/* defines for the REMOTE-LOCAL switch */

#define	RL_LOCAL		0
#define	RL_REMOTE		1

/* DEFINES FOR TANKFARM TANK 
   Note: these must match what is in choiceTank.h in 
	 .../share/ascii
*/

#define REC_TANK_SHAPE        0
#define REC_TANK_ROOF         1
#define REC_TANK_LEVEL        2
#define REC_TANK_MASS         3
#define REC_TANK_DENSITY      4
#define REC_TANK_VALVE        5
#define REC_TANK_MEAS         6

/* DEFINES FOR TANKFARM VALVE
   Note: these must match the spelling and value in
	 .../share/ascii/choiceRec.ascii and
	 .../share/ascii/choiceValve.h
*/

#define REC_VALVE_ES          0



/* DEFINES FOR TANKFARM TANK VALVES 
   Note: these values must match those in choiceRec.ascii
	 under REC_TANK_VALVE
*/

#define TANK_VALVE_CLOSED       0
#define TANK_VALVE_OPENED       1
#define TANK_VALVE_UNSPECIFIED  2


/* DEFINES FOR MODE 
   Note: these definitions must match those in
	 choiceGbl.ascii
*/

#define TF_AUTOMATIC    0
#define TF_MANUAL       1
#define TF_LOCKED       2

#endif INCTankFarmH
