/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*
 * Modification Log:
 * -----------------
 * .01  mm-dd-yy        iii     Comment
 * .01	01-11-96	ems	added usd.zcmf
*/

#include <dbDefs.h>


#ifndef	UDBDEFS
#define	UDBDEFS

#define	UDB_NUM_INPUTS		8
#define	UDB_NUM_CMDS		8
#define	UDB_NUM_OUTPUTS		8
#define	UDB_NUM_CHKBXS		8
#define	UDB_NUM_NUMERIC_FLOAT	12
#define UDB_NUM_NUMERIC_DOUBLE	6
#define	UDB_NUM_NUMERIC_ULONG	2
#define	UDB_NUM_TIMERS		12
#define	UDB_NUM_TEMP_FLOAT	16
#define	UDB_NUM_TEMP_DOUBLE	4
#define	UDB_NUM_TEMP_ULONG	8
#define	UDB_NUM_FLAGS		16
#define	UDB_NUM_STATUS		12
#define	UDB_NUM_MODES		8
#define	UDB_NUM_FUNCS		4

/* NOTE: this must agree with what is in .../share/ascii/udbRecord.ascii */
#define	UDB_MSG_SIZE		32

/* defines used to determine whether a function is to
   copy data to the local stack variable from the 
   record or to the record from the local stack
*/
#define	UDB_COPY_TO_LOCAL	1
#define	UDB_COPY_TO_REC		2

/* defines for time delay */
#define	UDB_TD_LE		1	/* leading edge		*/
#define	UDB_TD_TE		2	/* trailing edge	*/

/* defines for timer status	*/
#define	UDB_TIMER_NOT_ACTIVE	0
#define	UDB_TIMER_ACTIVE	1
#define	UDB_TIMER_HELD		2

/* defines for some enum strings */
#define	UDB_LAST_MODE		"LAST MODE"
#define	UDB_LAST_FUNC		"LAST FUNCTION"

/* defines for the control and graphic limits */
#define	UDB_UPPER_CTRL_LIMIT	100
#define	UDB_LOWER_CTRL_LIMIT	0
#define	UDB_UPPER_GRAPHIC_LIMIT	100
#define	UDB_LOWER_GRAPHIC_LIMIT	0
#define	UDB_DEFAULT_PRECISION	1

/* define for UDB_get_inputs() and UDB_get_outputs, 
   the ok flag in the record is set to this if the
   dbNameToAddr fails, or the dbGetLink fails or the
   dbGetField fails
*/
#define	UDB_IO_GET_FAILURE	0xffff

/* define for UDB_get_inputs and UDB_get_outputs,
   the ok flag is set to this if there is no input
   or output entered
*/
#define	UDB_NO_IO_POINT		0xfff

/* defines for the UDB_get_field and UDB_put_field */
#define	UDB_INPUT_FIELD		1
#define UDB_OUTPUT_FIELD	2

/* defines for the type of field returned by UDB_get_field()
   and UDB_put_field, 
   NOTE: these values must agree with what is in .../share/epicsH/dbFldTypes.h
*/
#define	UDB_DBR_STRING	0
#define	UDB_DBR_CHAR	1
#define	UDB_DBR_UCHAR	2
#define	UDB_DBR_SHORT	3
#define	UDB_DBR_USHORT	4
#define	UDB_DBR_LONG	5
#define	UDB_DBR_ULONG	6
#define	UDB_DBR_FLOAT	7
#define	UDB_DBR_DOUBLE	8
#define	UDB_DBR_ENUM	9
#define	UDB_NTYPES	(UDB_DBR_ENUM + 1)

/* defines for error returns
*/
#define	UDB_NO_ERROR		0
#define S_udb_RangeError	(M_udb | 1)	/* field value is out of range	*/
#define S_udb_NoUdbLib		(M_udb | 3)	/* udbLib not loaded		*/
#define S_udb_NoInitFlag	(M_udb | 5)	/* UDB_xxx_initialize_flag not defined*/
#define S_udb_NoInit		(M_udb | 7)	/* UDB_xxx_initialize not defined */
#define S_udb_InitFailed	(M_udb | 9)	/* UDB_xxx_initialize failed	*/
#define S_udb_NoIni2		(M_udb | 11)	/* UDB_xxx_ini2 not defined	*/
#define S_udb_NoProcess		(M_udb | 13)	/* UDB_xxx_process not defined	*/

/* define a pointer to a function with a void pointer 
   as an arg and returns an int 
*/
typedef	int	(*UDBPTR)(void*);


/* structure for users data, IOC rec data is copied to this
   on the users stack
*/


struct userudb_data
{
    char		udb_name[PVNAME_SZ + FLDNAME_SZ + 2]; /* point name	*/  
    short		ini2;				/* secondary init flag	*/  
    unsigned short	nsta;				/* new alarm status	*/  
    unsigned short	nsev;				/* new alarm severity	*/  
    unsigned short	stat;				/* current alarm status	*/  
    unsigned short	sevr;				/* current alarm severity */  
    unsigned short	tenb;				/* trend enable		*/
    double		inv[UDB_NUM_INPUTS];		/* input values		*/  
    unsigned short	iok[UDB_NUM_INPUTS];		/* input status		*/  
    unsigned short	inpf;				/* input flag		*/  
    unsigned short	ook[UDB_NUM_OUTPUTS];		/* output status	*/  
    unsigned short	outf;				/* output flag		*/  
    double		ovl[UDB_NUM_OUTPUTS];		/* output values	*/  
    unsigned short	mode;				/* current mode		*/  
    unsigned short	ncmd;				/* new command		*/  
    unsigned short	trnc;				/* NCMD trend flag	*/
    short		cmm[UDB_NUM_CMDS];		/* manual command flags	*/  
    short		cma[UDB_NUM_CMDS];		/* automatic command flags */  
    unsigned short	lcmd;				/* last command		*/  
    unsigned short	func;				/* current function	*/  
    short		ffl[UDB_NUM_FUNCS];		/* function flags	*/  
    unsigned short	stts;				/* current status	*/  
    unsigned short	chk[UDB_NUM_CHKBXS];		/* check boxes		*/  
    float		n[UDB_NUM_NUMERIC_FLOAT];	/* numeric floats	*/  
    double		d[UDB_NUM_NUMERIC_DOUBLE];	/* numeric doubles	*/  
    unsigned long	u[UDB_NUM_NUMERIC_ULONG];	/* numeric unsigned long */  
    unsigned long	npfl;				/* numeric prompt flag	*/  
    unsigned long	nrfl;				/* numeric redundant flag */  
    unsigned long	cetm;				/* current epics time (secs) */  
    short		year;				/* current year as 1995	*/  
    short		leap;				/* leap year,               
    						   	   0 isnt, 1 is		*/  
    short		doy;				/* current day of year,     
    						   	   1 is jan 1st		*/  
    short		mon;				/* current month, 	    
    						   	   1 is jan		*/  
    short		dom;				/* current day of month,    
    						   	   1 is 1st day of month */  
    short		dow;				/* current day of week,     
    						   	   1 is sunday, 7 is sat */  
    short		hour;				/* current hour of day	*/  
    short		min;				/* current minute	*/  
    short		sec;				/* current seconds	*/  
    unsigned long	ctck;				/* current clock ticks	*/  
    unsigned long	dctk;				/* delta clock ticks	*/  
    unsigned long	tmr[UDB_NUM_TIMERS];		/* timers		*/  
    unsigned short	tf[UDB_NUM_TIMERS];		/* timer flags		*/  
    float		fp[UDB_NUM_TEMP_FLOAT];		/* temp float values	*/  
    unsigned short	trfl;				/* temp float trend flag */
    unsigned long	ul[UDB_NUM_TEMP_ULONG];		/* temp ulong values	*/  
    double		db[UDB_NUM_TEMP_DOUBLE];	/* temp double values	*/  
    unsigned short	bl[UDB_NUM_FLAGS];		/* boolean flags	*/  
    unsigned short	trbl;				/* bool trend flag	*/
    char		msgf[UDB_MSG_SIZE];		/* message field	*/  
    unsigned long	tmfl;				/* temp/msgf bit flag	*/  
    short		fpas;				/* first process pass flag */  
    long		error;				/* error flag		*/  
    int			debug;				/* debug switch		*/
    short		zcmf;				/* zero cmds flag	*/
};



/* PROTOTYPE DEFS */

void UDB_data_transfer (void *, struct userudb_data *, int);
void UDB_new_alarm (unsigned short, unsigned short, unsigned short *, \
		    unsigned short *, int, long *, char *);
long UDB_stamp_to_text (unsigned long *, char *);
long UDB_text_to_stamp (char *, unsigned long *);
long UDB_get_field (void *, int, char *, void *, int, int);
long UDB_put_field (void *, int, char *, void *, int, int);
long UDB_master (void *, int, int, long *);

#endif

