/*drvBailey.h,v 1.1.1.1 1999/08/03 14:41:54 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
**  File:  drvBailey.h							       	
**  Edit:  03/15/95     WEP: Created
**
**	This file contains the data structures used by the Bailey CIU
**	I/O driver.                                                            	       	
**                                                                             	      	
*/

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/*
**      There are about 2500 Function Blocks in the Bailey database now. Only a non-    
**      contiguous subset of these function blocks will be referenced by this driver.   
**      These blocks translate into multiple binary and analog points in the TIS-4000   
**      database points. Because of the non-orthogonal mapping of Function Blocks to    
**      TIS-4000 points and the large Loop/PCU/Module/Block address range over which    
**      the function blocks are located in the Bailey system, a dynamically allocated   
**      non-contiguous data structure scheme will be employed to store and reference    
**      the data.                                                                       
**                                                                                      
**      The IOC assumes that the configuration of the Bailey INFI-90 System is unknown. 
**      Therefore, incoming driver requests to read and write data will result in the   
**      dynamic allocation and configuration of the driver's data storage area.         
**                                                                                      
**      Furthermore, the IOC will utilize the exception reporting capabilities of the   
**      CIU to minimize the communication traffic volume and loop delays which would    
**      result from direct polling.                                                     
**                                                                                      
*/

/************************************************/
/*	number of lines of Bailey INFI-90 CIU 	*/
/************************************************/
#define	NUMLINES         7
#define HIGHLINE         6

#define MAXLINES         4      /* modify for IOC pair cross-driver redundancy */

/************************************************/
/*	number of Loops per Bailey INFI-90 Loop */
/************************************************/
#define	LOWLOOP          1 /* 1 - 9 */
#define	HIGHLOOP         9 /* 1 - 9 */


/***********************************************/
/*	number of PCUs per Bailey INFI-90 Loop */
/***********************************************/
#define	LOWPCU           1
#define	HIGHPCU         63


/*****************************************/
/*	number of Modules per Bailey PCU */
/*****************************************/
#define	LOWMODULE         0
#define	HIGHMODULE        9

#define	LOWBLOCKMODULE    2
#define	HIGHBLOCKMODULE   9

 
/*******************************************/
/*	number of Blocks per Bailey Module */
/*******************************************/
#define	LOWBLOCK         1
#define	HIGHBLOCK     9899

#define PSEUDO        9900   /* Card number for ioc comm stats i/o        */
#define CIUPSEUDO     9901   /* Card number for bailey ciu stats i/o      */
#define LOOPPSEUDO    9902   /* Card number for bailey loop stats i/o     */
#define PCUPSEUDO     9903   /* Card number for bailey pcu stats i/o      */
#define MODULEPSEUDO  9904   /* Card number for bailey module stats i/o   */
#define MODULESTATUS  9905   /* Card number for module level data.        */
#define DATETIME      9906   /* Card number for ciu level date/time data  */

#define RESETSTATISTICS 0  /* unique field/element value which used to issue reset to statistics */

#define MAXINDEX     10000 /* 40K of memory/line - fixed allocation */

#define BITBLOCK         8 /* size of digital groups */

/*******************************************/
/*     number of Function Types            */
/*******************************************/
#define NUMTYPES         6

/*****************************/
/* Function Type Definitions */
/*****************************/
#define AER_TYPE         1                      /* Analog Exception Report Function Blocks    */
#define DER_TYPE         2                      /* Digital Exception Report Function Blocks   */
#define RCM_TYPE         3                      /* Remote Control Memory Function Blocks      */
#define RMSC_TYPE        4                      /* Remote Manual Set Constant Function Blocks */
#define CS_TYPE          5                      /* Control Station Function Blocks            */
#define DD_TYPE          6                      /* Device Driver Function Blocks              */

/*************************/
/* Tate Type Definitions */
/*************************/
#define ANALOG           1                      /* Analog Exception Report Function Blocks    */
#define BINARY           2                      /* Digital Exception Report Function Blocks   */

/* not used in call to driver. only used  */
/* to identify the index allocation type. */
#define MODULE_TYPE      7                      /* Module Status Information                  */

/* Function Element Definitions */
#define AER_BINUM        8
#define AER_AINUM        1

#define DER_BINUM        8

#define RCM_BINUM       16
#define RCM_BONUM        8

#define RMSC_BINUM       8
#define RMSC_BONUM       8
#define RMSC_AINUM       1
#define RMSC_AONUM       2

#define CS_BINUM        16
#define CS_AINUM         4
#define CS_AONUM         3
#define CS_BONUM         8


#define DD_BINUM        16
#define DD_BONUM         8

#define MODULE_BINUM   128

#define TIME_BINUM       8
#define TIME_AINUM       9

/* CIU Point Types (used to establish points) */
#define CPT_ANALOGREAD              5
#define CPT_DIGITALREAD             7
#define CPT_STATIONMODE            11
#define CPT_MODULESTATUS           14
#define CPT_RCMREAD                15
#define CPT_SISTATIONREAD          17
#define CPT_RMSCREAD               19


typedef struct   {
    unsigned short  address;
    unsigned short  card;
    unsigned short  signal;
    unsigned short  loop;
    unsigned short  pcu;
    unsigned short  module;
    unsigned short  block;
    unsigned char   type;
    unsigned short  element;

    unsigned char   was_requested;
    unsigned char   established;
    unsigned char   connected;
    unsigned short  index;
    unsigned char   data_valid;

    unsigned char   CIUpoint;

                 } COMMON;

typedef struct   {
    COMMON          control;
    unsigned long   data_read[10];
                 } COMMONALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   status_read;
    unsigned char   raw_read[3];    /* only stored for debug purposes. converted to long float. */
    float           data_read;
                 } AERALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   status_read;
                 } DERALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   status_read[5]; /* only two bytes actually used at time of design. */
    unsigned char   status_write;
                 } RCMALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   status_read;
    unsigned char   status_write;
    float           data_read;
    float           data_write;
    unsigned char   raw_read[3];
    unsigned char   raw_write[3];
                 } RMSCALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   status_read[2];
    float           PV_read;
    float           SP_read;
    float           CO_read;
    float           RI_read;
    float           SP_write;
    float           RI_write;
    float           CO_write;
    unsigned char   PVraw_read[3];
    unsigned char   SPraw_read[3];
    unsigned char   COraw_read[3];
    unsigned char   RIraw_read[3];
    unsigned char   SPraw_write[3];
    unsigned char   RIraw_write[3];
    unsigned char   COraw_write[3];
    unsigned char   status_write;
                 } CSALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   status_read[5]; /* only two bytes actually used at time of design. */
    unsigned char   status_write;
                 } DDALLOC;


/* Number of Blocks in a Block Allocation Cluster */
#define NUMBLOCKALLOC  100

typedef struct   {
    COMMON          control;
    void           *blocks[NUMBLOCKALLOC];
                 } BLOCK100ALLOC;

typedef struct   {
    COMMON          control;
    unsigned char   is_busy;
    unsigned char   is_responding;
    unsigned char   holdflag;               /*   The Module data is in hold   */
    unsigned char   status[5];
    unsigned char   detailstatus[11];
    unsigned char   modulemode;
    unsigned char   module_badmsg;          /* 100 */
    unsigned char   module_busy;            /* 101 */
    unsigned char   module_cmmismatch;      /* 102 */
    unsigned char   module_rangeerr;        /* 103 */
    unsigned char   module_blkerr;          /* 104 */
    unsigned char   module_blkundef;        /* 105 */
    unsigned char   module_blknoread;       /* 106 */
    unsigned char   module_funerr;          /* 107 */
    unsigned char   module_funblkerr;       /* 108 */
    unsigned char   module_memerr;          /* 109 */
    unsigned char   module_dead;            /* 110 */
    BLOCK100ALLOC  *blocks100[((HIGHBLOCK/NUMBLOCKALLOC)+1)];
                 } MODULEALLOC;


typedef struct   {
    COMMON          control;
    unsigned char   is_responding;
    unsigned char   node_busy;
    unsigned char   node_offline;
    unsigned char   node_looptimeout;
    MODULEALLOC    *modules[HIGHMODULE+1];
                 } PCUALLOC;


typedef struct   {
    COMMON          control;
    unsigned char   is_responding;
    unsigned short  ring_busy;          /*  41 */
    unsigned short  ring_offline;       /*  39 */
    PCUALLOC       *pcus[HIGHPCU+1];
		} LOOPALLOC;



typedef struct   {
    LOOPALLOC      *ploop;
    PCUALLOC       *ppcu;
    MODULEALLOC    *pmodule;
    void           *pblock;

    unsigned char   status_valid;
    unsigned char   status_read[5];     /* CIU module status uses all five bytes.       */
    unsigned char   ciumode;            /* CIU mode field from status_read[0].          */
    unsigned short  is_responding;	/* 1  The CIU is responding 		        */
    unsigned short  holdflag;		/*    The CIU data is in hold 		        */
    unsigned short  no_dataout_msgs;	/* 5  Count Output Msgs 			*/
    unsigned short  no_datain_msgs;	/* 6  Count Data Requests 		        */
    unsigned short  no_rxnoresp;	/* 7  Count No Responses 			*/
    unsigned short  no_txnoresp;	/* 8  Count No Responses 		        */
    unsigned short  no_dataerr;		/* 9  Count Message Echo Errors 	        */
    unsigned short  no_retries;		/* 10 Count Retries 			        */
    unsigned short  no_rxhwerrs;	/* 11 Count Comm Hardware Errors 	        */
    unsigned short  no_txhwerrs;	/* 12 Count Comm Hardware Errors 	        */
    unsigned short  no_chksumerrs;	/* 13 Count RX Checksum Errors 		        */
    unsigned short  no_offlines;	/* 14 Count Off Line Transitions	        */
    unsigned short  cmd_reset_stat;	/* 15 Command to reset stats		        */
    unsigned short  cmd_force_offline;	/* 16 Command to Force Offline 		        */
    unsigned short  badoutcnt;		/* 17 Count Failed output commands 	        */

    COMMON          control;
    unsigned short  reply;              /* response to last command (if checksum was good) */
    unsigned short  replyclass;         /* classification of reply to last command (good)  */

    unsigned short  ciu_noerrors;       /*   0 */

    unsigned short  ciu_loopwait;       /*   1 */
    unsigned short  ciu_repeatrestart;  /*   9 */
    unsigned short  ciu_needrestart;    /*  18 */
    unsigned short  ciu_specslost;      /*  22 */
    unsigned short  ciu_emptyqueue;     /*  23 */
    unsigned short  ciu_indexinuse;     /*  28 */

    unsigned short  ciu_btmbad;         /*   7 */
    unsigned short  ciu_commerror;      /*  12 */
    unsigned short  ciu_commtimeout;    /*  27 */
    unsigned short  ciu_checksum;       /*  31 */
    unsigned short  ciu_busy;           /*  35 */
    unsigned short  node_busy;          /*  40 */
    unsigned short  ring_busy;          /*  41 */
    unsigned short  module_busy;        /* 101 */

    unsigned short  ciu_btmdead;        /*  13 */
    unsigned short  ciu_looptimeout;    /*  15 */
    unsigned short  node_offline;       /*  32 */
    unsigned short  ciu_error;          /*  34 */
    unsigned short  ciu_btmoffline;     /*  36 */
    unsigned short  ring_offline;       /*  39 */
    unsigned short  module_memerr;      /* 109 */
    unsigned short  module_dead;        /* 110 */

    unsigned short  ciu_eierr;          /*   8 */
    unsigned short  ciu_memerr;         /*  11 */
    unsigned short  ciu_ieerr;          /*  14 */
    unsigned short  ciu_rangeerr;       /*  16 */
    unsigned short  module_badmsg;      /* 100 */
    unsigned short  module_rangeerr;    /* 103 */
    unsigned short  module_blkerr;      /* 104 */
    unsigned short  module_blkundef;    /* 105 */
    unsigned short  module_blknoread;   /* 106 */
    unsigned short  module_funerr;      /* 107 */
    unsigned short  module_funblkerr;   /* 108 */

    unsigned short  ciu_format;         /*   2 */
    unsigned short  ciu_illegal;        /*   3 */
    unsigned short  ciu_reindex;        /*   4 */
    unsigned short  ciu_reblock;        /*   5 */
    unsigned short  ciu_toolong;        /*   6 */
    unsigned short  ciu_noindex;        /*  10 */
    unsigned short  ciu_badkey;         /*  17 */
    unsigned short  ciu_mierr;          /*  19 */
    unsigned short  ciu_msgactive;      /*  20 */
    unsigned short  ciu_iomerr;         /*  21 */
    unsigned short  ciu_bigreply;       /*  24 */
    unsigned short  ciu_illegalsmc;     /*  25 */
    unsigned short  ciu_reporterr;      /*  26 */
    unsigned short  ciu_pointbad;       /*  29 */
    unsigned short  ciu_watchdog;       /*  30 */
    unsigned short  ciu_needcallup;     /*  33 */
    unsigned short  ciu_mmconflict;     /*  37 */
    unsigned short  ciu_pointtype;      /*  38 */
    unsigned short  module_cmmismatch;  /* 102 */

    unsigned short  ciu_unknown;        /* ??? */

    unsigned char   time_requested;
    unsigned char   time_valid;
    unsigned char   syncbyte;
    unsigned short  year;
    unsigned short  month;
    unsigned short  day;
    unsigned short  day_of_week;
    unsigned short  hour;
    unsigned short  minute;
    unsigned short  second;
    unsigned short  sample_number;
    unsigned short  time_sync_master;

    unsigned short  LOOPnumber;
    unsigned short  PCUnumber;

    LOOPALLOC       *loops[HIGHLOOP+1];
    unsigned short  lastconnected;
    unsigned short  nextindex;                  /* assign indexes sequentially               */
    void           *indextable[MAXINDEX];       /* holds void pointer to function block data */
                                                /* structure assigned to this CIU index.     */
		} CIUALLOC;

/*
**	Conversion types for analog inputs	*/
/*
*/
#define AICNVINTEGER	10
#define AICNVDOUBLEINT	11
#define AICNVFLOAT	12
#define AICNVDOUBLEFLT	13
#define AICNVASCIIFLT	14
#define AICNVSTATUS	15

#define AICNVTIME	16

#define MAXCONV		20

/* The following must match the definition in recChoice.ascii */
#define NOCONV 0
#define LINEAR 1
#define ASCFLT 2


#define WATCHDOGTIMEOUT 10*60      /* watchdog monitor timeout.  */


#define LOOPCNT 50                 /* max number of each point type we will attempt to */
                                   /* acquire on each call to scan ciu.                */

/*
**	Communications I/O Structures
**
*/

#define	RXSEM            0
#define	TXSEM            1
/*#define MAXRXMSG     512     ** worst case command used is actually 499 on old CIU.  */
/*#define MAXTXMSG     512     ** worst case command used is actually  12 on old CIU.  */
#define	MAXRXMSG      3074     /* the FORTRAN code declared both TX and RX as 3072.    */
#define	MAXTXMSG       512     /* we are using much less now but leave it here anyway. */

typedef	struct   {
    unsigned short  rxstate;
    unsigned short  rxmsglenexpected;
    unsigned short  rxmsglen;
    unsigned short  rxix;
    SEMAPHORE      *semid;
    unsigned short *status;
    unsigned char   rxbuf[MAXRXMSG];
		 } RXLINE;

typedef	struct   {
    unsigned short  protocol;
    unsigned short  txdisable;
    unsigned short  txtocnt;
    unsigned short  txhwerrcnt;
    unsigned short  txmsglen;
    unsigned short  txix;
    SEMAPHORE      *semid;
    unsigned short *status;
    unsigned char   txbuf[MAXTXMSG];
                 } TXLINE;

/*
**	Communications I/O Constants
**
*/

#define RXGOOD            1
#define	RXBAD             2
#define	NORMAL_TIMEOUT   60           /* 60 seconds for a normal RX response (16 ms ticks)         */
#define	RESTART_TIMEOUT 300           /* 5 seconds for a Restart Command RX response (16 ms ticks) */
#define	ACTIVITYTIME    300           /* must see activity in 5 seconds 	                   */

#define	VARRESP       FALSE
#define	FIXEDRESP      TRUE
#define	ONERETRY     0x8000           /* OR'ed into line number for xfermsg 	*/

#define	RXIGNORE          0
#define	RXRECEIVE         1


#define	TXGOOD            1
#define	TXBAD             2
#define	IO_TIMEOUT        3           /* Code returned by timer 		*/

#define	TXTIMEOUT        60           /* 16 ms ticks: one second              */

#define	MAXRETRYS         3           /* retransmission trys 			*/

#define	MAXCFGCK          8           /* Try to Bring online each cycle 	*/
#define	BRINGBACKCNT      4           /* Poll loops per cycle 		*/

#define	XFERGOOD         1
#define	XFERBAD	         2

#define	RESNAMELEN      16           /* length of a resource name 		*/
#define	SLWWINDOW      900           /* # of ticks to allow slave to try 	*/
/*
** Output Queue Management Structures for Bailey Host-CIU Protocol
**
*/

#define OUTQSIZE       128


typedef struct   {
    unsigned short  otype;
    unsigned short  blocktype;
    unsigned short  tatetype;
    unsigned short  index;
    unsigned short  link;
    unsigned short  adapter;
    unsigned short  card;
    unsigned short  signal;
    unsigned long   mask;
    unsigned long   bovalue;
    unsigned short  block;
    unsigned short  conversion;
             short  aovalue;
            double  dblaovalue;
    LOOPALLOC      *ploop;
    PCUALLOC       *ppcu;
    MODULEALLOC    *pmodule;
    void           *pblock;
		 } OUTQENT;


typedef struct   {
    unsigned short   ooutix;
    unsigned short   oinix;
    OUTQENT          outque[OUTQSIZE];
                 } OUTPUTQUE;

/*
** Bailey Host-CIU Protocol command codes
*/

#define ESTABLISHPOINT          1   /* ** Enter a point in the CIU point table.                      */
#define CONNECTPOINTLIST        4   /* ** Connect established input routes in the CIU.               */
#define READEXCEPTIONS          8   /* ** Get current status and values for all points in exception. */
/* #define REREADREPLY             9   ** Request CIU to re-transmit the previous reply.             */
#define OUTPUTVALUE             10  /* ** Used to output a value to an RMSC block.                   */
#define OUTPUTSTATUS            11  /* ** Used to output a value to an RCM block.                    */
#define CIURESTART              19  /* ** Causes CIU to clear its index list.                        */

#define READMISCSTATUSEXCEPTS   22  /* ** Read Misc Status Exceptions for Stations, RCMs, Modules    */
/* #define READPROBLEMREPORT      26   ** Read Module Problem Reports. This command, along with #27, */
                                       /*  are the only commands used for which replycode >= 100.    */
/* #define DEMANDMODULESTATUS     27   ** Read Module Status. This command, along with #26, are      */
                                       /*  the only commands used for which replycode >= 100.        */

/* #define CIULOOPBACK             30  ** illegal command used as loopback. verify benign effect.    */

/* #define SETCIUSTATUS            33  ** Set fifth status byte of CIU module status.                */
/* #define READWORKFLAG            34  ** Find out what work there is to do, if any (get exceptions) */
#define READSYSTEMDATETIME      43  /* ** Read CIU System Time and Date.                             */
/* #define CIUONLINE               44  ** Set CIU as Primary.                                        */
/* #define CIUOFFLINE              44  ** Set CIU as Secondary.                                      */
/* #define OUTPUTSTATUSGROUP       47  ** Output to RCM/DD point.                                    */
/* #define ESTABLISHANDCONNECT     49  ** Establish and Connect Point                                */ 
#define READSTATIONEXCEPTS      53  /* ** Read Station Exceptions (#22 does Stations, RCMs, Modules) */
#define OUTPUTSTATIONVALUE      56  /* ** Output Station Value. Set single value: SP, RI, or CO      */

/* #define READDATAEXCEPTS         63  ** No Documentation                                           */
/* #define CIU69                   69  ** Causes CIU to clear its index list? More? Details unknown. */
/* #define REPEATLASTCOMMAND       97  ** No Documentation                                           */
/* #define EXIT                    99  ** No Documentation                                           */


/*
** Bailey Host-CIU Protocol command lengths
*/
#define ESTABLISHPOINTLEN        11 /* 1  Enter a point in the CIU point table.                      */
#define CONNECTPOINTLISTLEN       6 /* 4  Connect established input routes in the CIU.               */
#define READEXCEPTIONSLEN         3 /* 8  Get current status and values for all points in exception. */
#define REREADREPLYLEN            2 /* 9  Request CIU to re-transmit the previous reply.             */
#define OUTPUTVALUELEN            8 /* 10 Used to output a value to an RMSC block.                   */
#define OUTPUTSTATUSLEN           5 /* 11 Used to output a value to an RCM block.                    */
#define CIURESTARTLEN             6 /* 19 Cause CIU to clear its index list.                         */
/* #define CIURESTARTLEN            5 ** 19 Cause CIU to clear its index list.                       */

#define READMISCSTATUSEXCEPTSLEN  3 /* 22 Read Misc Status Exceptions for Stations, RCMs, Modules    */
#define READPROBLEMREPORTLEN      8 /* 26 Read Module Problem Reports. This command, along with #27, */
                                    /*     are the only commands used for which replycode >= 100.    */
#define DEMANDMODULESTATUSLEN     6 /* 27 Read Module Status. This command, along with #26, are      */
                                    /*     the only commands used for which replycode >= 100.        */

#define CIULOOPBACKLEN            1 /* 30 illegal command used as loopback. verify benign effect.    */

#define SETCIUSTATUSLEN           4 /* 33 Set fifth status byte of CIU module status.                */
#define READWORKFLAGLEN           4 /* 34 Find out what work there is to do, if any (get exceptions) */
#define READSYSTEMDATETIMELEN     2 /* 43 Read CIU System Time and Date.                             */
#define CIUONLINELEN              3 /* 44 Set CIU as Primary.                                        */
#define CIUOFFLINELEN             3 /* 44 Set CIU as Secondary.                                      */
#define OUTPUTSTATUSGROUPLEN      5 /* 47 Output to RCM/DD point.                                    */
#define ESTABLISHANDCONNECTLEN   12 /* 49 Establish and Connect Point                                */
#define READSTATIONEXCEPTSLEN     3 /* 53 Read Station Exceptions (#22 does Stations, RCMs, Modules) */
#define OUTPUTSTATIONVALUELEN     8 /* 56 Output Station Value. Set single value: SP, RI, or CO      */

#define READDATAEXCEPTSLEN        1 /* 63 No Documentation                                           */
#define CIU69LEN                  2 /* 69 Determined by line monitoring of PC software.              */
#define REPEATLASTCOMMANDLEN      1 /* 97 No Documentation                                           */
#define EXITLEN                   1 /* 99 No Documentation                                           */


/*  Bailey Host-CIU Protocol reply code definitions */

/* note: for now, we havn't bothered to name them all, just the */
/*       ones which have special meaning within the replyclass. */
#define R_NOERRORS              0
#define R_SPECSLOST            22
#define R_EMPTYQUEUE           23
#define R_NOERRORS              0
#define R_NOERRORS              0
#define R_REPEATRESTART         9


/* Bailey Host-CIU Protocol reply class definitions */

#define RC_NOERROR      0
#define RC_INIT         1
#define RC_PROBLEM      2
#define RC_CIU_ERR      3
#define RC_PCU_ERR      4
#define RC_RING_ERR     5
#define RC_MODULE_ERR   6
#define RC_CONF_ERR     7
#define RC_MCONF_ERR    8
#define RC_LOGIC_ERR    9
#define RC_MLOGIC_ERR  10
#define RC_UNKNOWN_ERR 11

/* Bailey Host-CIU Protocol msgxfer resturned class definitions */

#define CMDNADA          0
#define CMDDONE          1
#define CMDINIT          2
#define CMDNACK          3
#define CMDFAIL          4


/* Bailey Driver RCM output definitions */

#define SUSTAINRESET     1
#define SUSTAINSET       2
#define PULSERESET       5
#define PULSESET         6


/* Bailey Driver task definitions */

#define BAILEY_STACK		8000   /* Stack size              */
#define	BAILEY_PRIORITY		152    /* default priority        */
#define	BAILEY_TASK_OPTIONS	8      /* Floating point          */

/*
** Mutual-Exclusion Control for diagnostic/data radio channel sharing */
/*
*/
#define		MUXTIMEOUT	60*30	/* 16.67 ms ticks (60Hz): 30 seconds 	*/


/* Other Structure Definitions */

typedef struct   {
    unsigned long   TXintcount;
    unsigned long   RXintcount;
    unsigned long   msgxfercount;
    unsigned long   startingtick;
                 }  LINESTATS;

/* Trace Bit Definitions */

#define ALLTRAFFICTRACE      0x0001
#define CIU08TRAFFIC         0x0002    /* trace non-zero responses to 'read exceptions' commands.*/
#define CIU22TRAFFIC         0x0004    /* trace non-zero responses to 'read misc status except'  */
#define CIU53TRAFFIC         0x0008    /* trace non-zero responses to 'read station exceptions'  */

#define CONFIGURATIONTRACE   0x0010    /* trace configuration errors in driver calls.            */
#define RUNTIMEERRORTRACE    0x0020    /* trace run time errors in driver calls.                 */
#define CALLREJECTTRACE      0x0040    /* trace non-error based driver call rejections.          */

#define CIUOKTRACE           0x0080    /* trace 'no error' response to CIU commands.             */
#define CIUSTATETRACE        0x0100    /* trace other 'normal' responses to CIU commands.        */
#define CIUERRORTRACE        0x0200    /* trace 'runtime error' responses to CIU commands.       */
#define CIUCONFIGTRACE       0x0400    /* trace 'configuration error' responses to CIU commands. */
#define CIUUNKNOWNTRACE      0x0800    /* trace 'unknown error' responses to CIU commands.       */

#define CIUALLOCTRACE        0x1000    /* trace error and non-error allocations.                 */
#define CIUALLOCERRORTRACE   0x2000    /* trace allocation errors.                               */
#define INDEXERRORTRACE      0x4000    /* trace index maintenance related errors.                */

/*	End of File:	drvBailey.h */
