/*devBendix.c,v 1.1.1.1 1999/08/03 14:41:54 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : devBendix.c 
 *  AUTHOR                 : emsteel
 *  DATE                   : 21 Dec 93
 *
 *
 * DESCRIPTION: device support for Bendix
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 30 Mar 94
 *  REVISION:  make forward func def agree with actual func def,
 *             remove arm_df, not used
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 1 June 94
 *  REVISION:	ai_init_record,	do conversions like read_bendix_ai
 *		read_ao,	do not set READ_ALARM
 *		bi_init_record,	set rval if driver call is ok
 *		mbbi_init_record, set rval if driver call ok
 *
****************************************************************************/

/****************************************************************************
 *
 * Programmer's Notes:
 *    This is a compendium of all dev**Bendix.c files in .../share/src/dev
 *    Where noted, change appropriately to create a new device support
 *    module
 *
 *
****************************************************************************/


#include   <vxWorks.h>
#include   <types.h>
#include   <stdioLib.h>
#include   <math.h>

#include   <alarm.h>
#include   <cvtTable.h>
#include   <dbAccess.h>
#include   <dbDefs.h>
#include   <devSup.h>
#include   <link.h>
#include   <module_types.h>
#include   <special.h>
#include   <cvnTypes.h>   /* defines for conversion types */
#include   <aiRecord.h>
#include   <aoRecord.h>
#include   <biRecord.h>
#include   <boRecord.h>
#include   <mbbiRecord.h>
#include   <mbboRecord.h>
#include   <stringinRecord.h>
#include   <stringoutRecord.h>
#include   <tdRecord.h>
#include   <waveformRecord.h>

#define  IO_TYPE     SBDX_IO              /* NOTE: change here */
#define  AIDRIVER    lmp_aidriver         /* NOTE: change here */
#define  AODRIVER    lmp_aodriver         /* NOTE: change here */
#define  AOREAD      lmp_aoread           /* NOTE: change here */
#define  BIDRIVER    lmp_bidriver         /* NOTE: change here */
#define  BODRIVER    lmp_bodriver         /* NOTE: change here */
#define  BOREAD      lmp_boread           /* NOTE: change here */
#define  SIDRIVER    lmp_sidriver         /* NOTE: change here */
#define  SODRIVER    lmp_sodriver         /* NOTE: change here */
#define  WFDRIVER    lmp_wfdriver         /* NOTE: change here */

static char *io_name = "Bendix";         /* NOTE: change here */

static double atof ();

/* Create the dset for devAiBendix */

static long ai_init_record ();
static long read_bendix_ai ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_bendix_ai;
   DEVSUPFUN   special_linconv;
} devAiBendix = {                      /* NOTE: change here */
   6,
   NULL,
   NULL,
   ai_init_record,
   NULL,
   read_bendix_ai,
   NULL};

/* Create the dset for devAoBendix */

static long ao_init_record();
static int read_ao ();
static long write_ao();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_ao;
   DEVSUPFUN   special_linconv;
} devAoBendix = {                      /* NOTE: change here */
   6,
   NULL,
   NULL,
   ao_init_record,
   NULL,
   write_ao,
   NULL};

/* Create the dset for devBiBendix */

static long bi_init_record ();
static long read_bi ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_bi;
} devBiBendix = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   bi_init_record,
   NULL,
   read_bi};

/* Create the dset for devBoBendix */

static long bo_init_record ();
static long write_bo ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_bo;
} devBoBendix = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   bo_init_record,
   NULL,
   write_bo};

/* Create the dset for devMbbiBendix */

static long mbbi_init_record ();
static long read_mbbi ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_mbbi;
} devMbbiBendix = {                    /* NOTE: change here */
   5,
   NULL,
   NULL,
   mbbi_init_record,
   NULL,
   read_mbbi};

/* Create the dset for devMbboBendix */

static long mbbo_init_record ();
static long write_mbbo ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_mbbo;
} devMbboBendix = {                    /* NOTE: change here */
   5,
   NULL,
   NULL,
   mbbo_init_record,
   NULL,
   write_mbbo};

/* Create the dset for devSiSoft */

static long si_init_record();
static long read_stringin();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_stringin;
} devSiBendix = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   si_init_record,
   NULL,
   read_stringin};

/* Create the dset for devSoSoft */

static long write_stringout();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_stringout;
} devSoBendix = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   NULL,
   NULL,
   write_stringout};

/* Create the dset for devTdBendix */

static long td_init_record ();
static long write_td ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_td;
} devTdBendix = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   td_init_record,
   NULL,
   write_td};

/* Create the dset for devWfBendix */ 

static long wf_init_record();
static long read_wf();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_wf;
} devWfBendix = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   wf_init_record,
   NULL,
   read_wf};

static long masks [] = {
   0x00000001, 0x00000002, 0x00000004, 0x00000008,
   0x00000010, 0x00000020, 0x00000040, 0x00000080,
   0x00000100, 0x00000200, 0x00000400, 0x00000800,
   0x00001000, 0x00002000, 0x00004000, 0x00008000,
   0x00010000, 0x00020000, 0x00040000, 0x00080000,
   0x00100000, 0x00200000, 0x00400000, 0x00800000,
   0x01000000, 0x02000000, 0x04000000, 0x08000000,
   0x10000000, 0x20000000, 0x40000000, 0x80000000,
};



/********************************* AI ***********************************/

static long ai_init_record (pai)
   struct   aiRecord   *pai;
{
   char              message[128];
   unsigned short    value, explen;
   struct   tateio   *ptateio;
   int               status;
   short             i, decdig, intdig;
   unsigned char     digit;
   double	     val, decplace;

   union u_value {
      short    ivalue;
      long     lvalue;
      float    fvalue;
      double   dvalue;
      char     afvalue[100];
   } uval;

   switch (pai->inp.type)
   {

      case (IO_TYPE):
         break;

      default:
         strcpy (message, pai->name);
         strcat (message, ": devAi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }


      /* call driver so that it configures card */

   if ((pai->linr == CNVASCIIFLT) && (pai->desc[0] == '%'))
   {
         /* format must follow % - ##.## */
      intdig = ((pai -> desc[1] & 0x0f) * 10) + (pai -> desc[2] & 0x0f);
      decdig =  ((pai -> desc[4] & 0x0f) * 10) + (pai -> desc[5] & 0x0f);
      if ((intdig + decdig) > 20)
         return (-1);
      explen = (intdig + decdig);
   }
   else
      (explen = 0);

   ptateio = (struct tateio *) &(pai->inp.value);
   status = AIDRIVER (pai->name, ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, &uval, pai->linr,explen, pai->resn);
   if (status == 0)
   {
      switch(pai->linr)
      {
         case NOCONV:
         case LINEAR:
         case (CNVINTEGER):
            pai->val = uval.ivalue;
            break;

         case (CNVDOUBLEINT):
            pai->val = uval.lvalue;
            break;

         case (CNVFLOAT):
            pai->val = uval.fvalue;
            break;

         case (CNVDOUBLEFLT):
            pai->val = uval.dvalue;
            break;

         case (CNVASCIIFLT):
            if (pai->desc[0] == '%')
            {
               val = 0;
               for (i=0;(i<intdig);i++)
               {
                  digit = uval.afvalue[i];
                  if (digit == ' ')
                     digit = '0';
                  if ((digit < '0') || (digit > '9'))
                     return (-1);
                  digit -= '0';
                  val = (val * 10) + digit;
               }
               decplace = .1;
               for (i = 0; (i < decdig);i ++)
               {
                  digit = uval.afvalue[intdig+i];
                  if (digit == ' ')
                     digit = '0';
                  if ((digit < '0') || (digit > '9'))
                     return (-1);
                  digit -= '0';
                  val = (val + (digit * decplace));
                  decplace = decplace / 10;
               }
               pai->val = val;
               break;
            }
            else
            {
               uval.afvalue[12] = 0; /* Null Termination */         
               pai->val = atof(uval.afvalue);
               break;
            }

         default:
            pai->rval = uval.ivalue;
            break;
      }
   }

   return (status);
}

static long read_bendix_ai (pai)
   struct aiRecord   *pai;
{
   struct   tateio   *ptateio;
   int               status;
   double            val, decplace;
   short             i, decdig, intdig;
   unsigned char     digit;
   unsigned short    explen;

   union u_value {
      short    ivalue;
      long     lvalue;
      float    fvalue;
      double   dvalue;
      char     afvalue[100];
   } uval;


   ptateio = (struct tateio *) &(pai->inp.value);
   if ((pai->linr == CNVASCIIFLT) && (pai->desc[0] == '%'))
   {
         /* format must follow % - ##.## */
      intdig = ((pai -> desc[1] & 0x0f) * 10) + (pai -> desc[2] & 0x0f);
      decdig =  ((pai -> desc[4] & 0x0f) * 10) + (pai -> desc[5] & 0x0f);
      if ((intdig + decdig) > 20)
         return (-1);
      explen = (intdig + decdig);
   }
   else
      explen = 0;

   status = AIDRIVER (pai->name, ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, &uval, pai->linr,explen, 0);

   /* returning non 0 status tells ai's process() not to do a conversion */

   if (status == 0)
   {
      switch(pai->linr)
      {
         case NOCONV:
         case LINEAR:
         case (CNVINTEGER):
            pai->val = uval.ivalue;
            status = 2;
            break;

         case (CNVDOUBLEINT):
            pai->val = uval.lvalue;
            status = 2;
            break;

         case (CNVFLOAT):
            pai->val = uval.fvalue;
            status = 2;
            break;

         case (CNVDOUBLEFLT):
            pai->val = uval.dvalue;
            status = 2;
            break;

         case (CNVASCIIFLT):
            if (pai->desc[0] == '%')
            {
               val = 0;
               for (i=0;(i<intdig);i++)
               {
                  digit = uval.afvalue[i];
                  if (digit == ' ')
                     digit = '0';
                  if ((digit < '0') || (digit > '9'))
                     return (-1);
                  digit -= '0';
                  val = (val * 10) + digit;
               }
               decplace = .1;
               for (i = 0; (i < decdig);i ++)
               {
                  digit = uval.afvalue[intdig+i];
                  if (digit == ' ')
                     digit = '0';
                  if ((digit < '0') || (digit > '9'))
                     return (-1);
                  digit -= '0';
                  val = (val + (digit * decplace));
                  decplace = decplace / 10;
               }
               pai->val = val;
               status = 2;
               break;
            }
            else
            {
               uval.afvalue[12] = 0; /* Null Termination */         
               pai->val = atof(uval.afvalue);
               status = 2;
               break;
            }

         default:
            pai->rval = uval.ivalue;
            break;
      }
   }
   if ( status == -2)
   {
      pai->rval = uval.ivalue;
   }
   if (status == -1)
   {
      if ((pai -> aenb != ALARM_DISABLED) && (pai->nsev < VALID_ALARM) )
      {
         pai->nsta = READ_ALARM;
         pai->nsev = VALID_ALARM;
      }
      status = 2;         /* dont convert */
   }
   return (status);
}


static double atof(s)
char   *s;
{
   double   val;
   double   power;
   double   exp = 0;
   int      i;
   int      sign;
   int      gotexp = 0;

   for (i = 0; s[i] == ' ' || s[i] == '\n' || s[i] == '\t'; i++)
      ;   /* skip white space */
   sign = 1;
   if (s[i] == '+' || s[i] == '-')      /* sign */
      sign = (s[i++] == '+') ? 1 : -1;
   for (val = 0; s[i] >= '0' && s[i] <= '9'; i++)
      val = 10 * val + s[i] - '0';
   if (s[i] == '.')
      i++;
   for (power = 1; s[i] >= '0' && s[i] <= '9'; i++)
   {
      val = 10 * val + s[i] - '0';
      power *= 10;
   }
   if (s[i] == '+' || s[i] == '-')
   {
      gotexp = 1;
      exp = atof(s + i);
   }
   return (sign * val / power * ((gotexp) ? pow((double) 10.0, exp) : 1));
}


/*********************************** AO ***********************************/

static long ao_init_record (pao)
   struct   aoRecord   *pao;
{
   char              message[128];
   short             value;
   struct   tateio   *ptateio;

   switch (pao->out.type)
   {

      case (IO_TYPE):
         break;

      default:
         strcpy (message, pao->name);
         strcat (message, ": devAo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField,message);
         return (S_db_badField);
   }

      /* call driver so that it configures card */

   return (read_ao (pao));
}

static long write_ao (pao)
   struct aoRecord   *pao;
{
   struct   tateio   *ptateio;
   int               status;
   int               value;
   double            dvalue;

   ptateio = (struct tateio *) &(pao->out.value);
   dvalue = pao->oval;
   value = pao->rval;
   status = AODRIVER (pao->name, ptateio->link, ptateio->adapter, ptateio->card,
		      ptateio->signal,dvalue, value, 0);
   if(status == -1)
   {
      if ((pao -> aenb != ALARM_DISABLED) && (pao->nsev < VALID_ALARM))
      {
         pao->nsta = WRITE_ALARM;
         pao->nsev = VALID_ALARM;
      }
   }
   else if (status == -2)
   {
      status = 0;
      if (pao->nsev < VALID_ALARM )
      {
         pao->nsta = HW_LIMIT_ALARM;
         pao->nsev = VALID_ALARM;
      }
   }
   return (status);
}


static int read_ao (pao)
   struct aoRecord   *pao;
{
   int               status;
   unsigned int      value;
   struct   tateio   *ptateio = &pao->out.value.tateio;

      /* get the value from the ao driver */

   status = AOREAD (pao->name,ptateio->link, ptateio->adapter, ptateio->card,
                    ptateio->signal, &value, &pao->resn [0]);

   if (!status)
   {
      pao->rval = value;
   }

   return (status);
}


/*********************************** BI ***********************************/

static long bi_init_record (pbi)
   struct biRecord   *pbi;
{
   struct   tateio   *ptateio;
   char              message[128];
   int               status = 0;
   unsigned long     value;

   switch (pbi->inp.type)
   {
      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pbi->inp.value);
         pbi->mask = masks[((ptateio->signal) % 32)];
         status = BIDRIVER (pbi->name, ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, pbi->mask, &value,
                            &pbi->resn [0]);
	 if (!status)
	     pbi -> rval = value;

         break;
   
      default :
         strcpy (message, pbi->name);
         strcat (message, ": devBi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static long read_bi(pbi)
   struct   biRecord *pbi;
{
   struct   tateio   *ptateio;
   int               status;
   long              value;

   ptateio = (struct tateio *) &(pbi->inp.value);
   status = BIDRIVER (pbi->name, ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, pbi->mask, &value, 0);
   if (status == 0)
   {
      pbi->rval = value;
      return (status);
   }
   else
   {
      if ((pbi -> aenb != ALARM_DISABLED) && (pbi->nsev < VALID_ALARM))
      {
         pbi->nsta = READ_ALARM;
         pbi->nsev = VALID_ALARM;
      }
      return (2);
   }
}


/*********************************** BO ************************************/

static long bo_init_record (pbo)
   struct boRecord   *pbo;
{
   struct   tateio   *ptateio;
   char              message[128];
   unsigned long     value;
   int               status = 0;

   switch (pbo->out.type)
   {

      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pbo->out.value);

            /* read the value via the Bendix LMP driver */

         pbo->mask = masks [((ptateio->signal) % 32)];
         status = BOREAD (pbo->name,ptateio->link, ptateio->adapter, 
                          ptateio->card, ptateio->signal, &value, pbo->mask, 
                          &pbo->resn [0]);
         if(status == 0)
         {
            pbo->rval = value;
         }
         break;

      default :
         strcpy (message, pbo->name);
         strcat (message, ": devBo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static long write_bo (pbo)
   struct   boRecord   *pbo;
{
   struct   tateio   *ptateio;
   int               status;
   char              message[128];

   ptateio = (struct tateio *) &(pbo->out.value);

      /* Format the message String */

   message[0]= 0;
   if (pbo->desc[0] == '+')
   {
      strcpy (message, &(pbo -> desc[1]));
   }
   if (pbo->rval == 0)
   {
      strcat (message, pbo->znam);
   }
   else
   {
      strcat (message, pbo->onam);
   }
   if (pbo->desc[0] == '-')
   {
      strcat (message, &(pbo -> desc[1]));
   }

   status = BODRIVER (pbo->name, ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, message, pbo->rval, pbo->mask, 0);
   if (status != 0)
   {
      if ((pbo -> aenb != ALARM_DISABLED) && (pbo->nsev < VALID_ALARM))
      {
         pbo->nsta = WRITE_ALARM;
         pbo->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/********************************* MBBI **********************************/

static   long mbbi_init_record (pmbbi)
   struct   mbbiRecord   *pmbbi;
{
   struct   tateio   *ptateio;
   char              message[128];
   int               status = 0;
   unsigned long     value;

   switch (pmbbi->inp.type)
   {
      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pmbbi->inp.value);
         pmbbi->shft = ((pmbbi->inp.value.tateio.signal) % 32);
         pmbbi->mask <<= pmbbi->shft;
         status = BIDRIVER (pmbbi->name, ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, pmbbi->mask, &value,
                            &pmbbi->resn [0]);
	 if (!status)
	     pmbbi -> rval = value;

         break;

      default :
         strcpy (message, pmbbi->name);
         strcat (message, ": devMbbi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static   long   read_mbbi (pmbbi)
   struct   mbbiRecord   *pmbbi;
{
   struct   tateio   *ptateio;
   int               status = 0;
   unsigned long     value;


   ptateio = (struct tateio *) &(pmbbi->inp.value);
   status = BIDRIVER (pmbbi->name, ptateio->link, ptateio->adapter, 
                      ptateio->card, ptateio->signal, pmbbi->mask, &value, 0);
   if (status == 0)
   {
      pmbbi->rval = value;
   }
   else
   {
      if ((pmbbi -> aenb != ALARM_DISABLED) && (pmbbi->nsev < VALID_ALARM))
      {
         pmbbi->nsta = READ_ALARM;
         pmbbi->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/*********************************** MBBO *********************************/
 
static long mbbo_init_record   (pmbbo)
   struct   mbboRecord   *pmbbo;
{
   struct   tateio   *ptateio;
   char              message[128];
   unsigned long     value;
   int               status = 0;

   switch (pmbbo->out.type)
   {

      case (IO_TYPE) :
         ptateio = &(pmbbo->out.value.tateio);
         pmbbo->shft = ((ptateio->signal) % 32);
         pmbbo->mask <<= pmbbo->shft;
         status = BOREAD (pmbbo->name,ptateio->link, ptateio->adapter, 
                          ptateio->card, ptateio->signal, &value, pmbbo->mask,
                          &pmbbo->resn [0]);
         if (status == 0)
         {
            pmbbo->rval = value;
         }
         break;

      default :
         strcpy (message, pmbbo->name);
         strcat (message, ": devMbbo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static long write_mbbo(pmbbo)
    struct mbboRecord   *pmbbo;
{
   struct   tateio   *ptateio;
   int               status, i;
   unsigned long     value;
   char              message[128];
   char              *psource;

   ptateio = &(pmbbo->out.value.tateio);

      /* Format the message */

   message[0]= 0;
   if (pmbbo->desc[0] == '+')
   {
      strcpy (message, &(pmbbo -> desc[1]));
   }
   if (pmbbo->val > 15)
   {
      strcat (message, "Illegal Value");
   }
   else
   {
       i = (pmbbo->val * sizeof(pmbbo -> zrst));
       psource = pmbbo->zrst;
       strcat (message, &psource[i]);
   }
   if (pmbbo->desc[0] == '-')
   {
      strcat (message, &(pmbbo -> desc[1]));
   }


   status = BODRIVER (pmbbo->name, ptateio->link, ptateio->adapter, 
                      ptateio->card, ptateio->signal, message, pmbbo->rval, 
                      pmbbo->mask, 0);
   if(status != 0)
   {
      if ((pmbbo -> aenb != ALARM_DISABLED) && (pmbbo->nsev < VALID_ALARM))
      {
         pmbbo->nsta = WRITE_ALARM;
         pmbbo->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/********************************* SI *************************************/

static long si_init_record(pstringin)
   struct stringinRecord   *pstringin;
{
   struct   tateio   *ptateio;
   char              message[128];
   long              status = 0;

   switch (pstringin->inp.type)
   {

      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pstringin->inp.value);
         status = SIDRIVER (pstringin->name, ptateio->link, ptateio->adapter, 
			    ptateio->card, ptateio->signal, message, 
                            pstringin->rdsz, pstringin->resn);
	 if (!status)
	 {
	    strncpy (pstringin -> val, message, sizeof (pstringin -> val) );   
            pstringin->udf = FALSE;
	 }

         break;

      default :
         strcpy (message, pstringin->name);
         strcat (message, ": devSi:");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
    }
   return(status);
}

static long read_stringin(pstringin)
   struct stringinRecord   *pstringin;
{
   struct   tateio   *ptateio;
   int               status;
   char              message[128];


   ptateio = (struct tateio *) &(pstringin->inp.value);
   status = SIDRIVER (pstringin->name, ptateio->link, ptateio->adapter, 
		      ptateio->card, ptateio->signal, message, 
		      pstringin->rdsz, 0);

   if(status!=0)
   {                 
      if((pstringin -> aenb != ALARM_DISABLED) &&
         (pstringin -> nsev < VALID_ALARM))
      {
         pstringin->nsev = VALID_ALARM;
         pstringin->nsta = READ_ALARM;
      }
   }
   else
   {
      pstringin->udf = FALSE;
      strncpy (pstringin->val, message, sizeof (pstringin -> val));
   }
  
   return(status);
}


/*********************************** SO ************************************/

static long write_stringout(pstringout)
   struct stringoutRecord   *pstringout;
{
   struct   tateio   *ptateio;
   char              message[128];
   long              status;

   switch (pstringout->out.type)
   {
      case (IO_TYPE) :

            /* Format the message String */

         message[0]= 0;
         if (pstringout->desc[0] == '+')
         {
            strcpy (message, &(pstringout -> desc[1]));
         }
         strcat (message, pstringout->val);
         if (pstringout->desc[0] == '-')
         {
            strcat (message, &(pstringout-> desc[1]));
         }
         ptateio = (struct tateio *) &(pstringout->out.value);
         status = SODRIVER (pstringout->name, ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, message, 0, 0);
         break;

      default :
         if ((pstringout -> aenb != ALARM_DISABLED) &&
             (pstringout -> nsev < VALID_ALARM))
         {
            pstringout->nsev = VALID_ALARM;
            pstringout->nsta = SOFT_ALARM;
            if(pstringout->stat!=SOFT_ALARM)
            {
               strcpy (message, pstringout->name);
               strcat (message, ": devSo");
               strcat (message, io_name);
               strcat (message, " (write_stringout) Illegal OUT field");
               errMessage(S_db_badField,message);
            }
         }
         status = (-1);
   }
   return(status);
}


/************************************* TD *********************************/

static long td_init_record (ptd)
   struct tdRecord   *ptd;
{
   struct   tateio   *ptateio;
   char              message[128];
   unsigned long     value;
   int               status = 0;

   switch (ptd->out.type)
   {

      case (IO_TYPE) :
         ptateio = (struct tateio *) &(ptd->out.value);

            /* read the value via the Bendix LMP driver */

         ptd->mask = masks [((ptateio->signal) % 32)];
         status = BOREAD (ptd->name, ptateio->link, ptateio->adapter, 
                          ptateio->card, ptateio->signal, &value, ptd->mask, 
                          &ptd->resn [0]);
         if(status == 0)
         {
            ptd->rval = value;
         }
         break;

      default :
         strcpy (message, ptd->name);
         strcat (message, ": devTd");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
   }
   return (status);
}

static long write_td (ptd)
   struct   tdRecord   *ptd;
{
   struct   tateio   *ptateio;
   int               status;
   char              message[128];

   ptateio = (struct tateio *) &(ptd->out.value);

      /* Format the message String */

   message[0]= 0;
   if (ptd->desc[0] == '+')
   {
      strcpy (message, &(ptd -> desc[1]));
   }
   if (ptd->rval == 0)
   {
      strcat (message, ptd->znam);
   }
   else
   {
      strcat (message, ptd->onam);
   }
   if (ptd->desc[0] == '-')
   {
      strcat (message, &(ptd -> desc[1]));
   }

   status = BODRIVER (ptd->name, ptateio->link, ptateio->adapter, 
                      ptateio->card, ptateio->signal, message, ptd->rval, 
                      ptd->mask, 0);
   if (status != 0)
   {
      if ((ptd -> aenb != ALARM_DISABLED) && (ptd->nsev < VALID_ALARM))
      {
         ptd->nsta = WRITE_ALARM;
         ptd->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/*********************************** WF ***********************************/

static long wf_init_record (pwf, process)
   struct waveformRecord   *pwf;
   void (*process)();
{
   char              message[128];
   struct   callback *pcallback;
   long		     status = 0;

   switch (pwf->inp.type)
   {
      case (IO_TYPE) :
         break;

      default :
         strcpy (message, pwf->name);
         strcat (message, ": devWf");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static long read_wf (pwf)
   struct waveformRecord   *pwf;
{
 
   struct   tateio   *ptateio;
   int               status;

   ptateio = (struct tateio *) &(pwf->inp.value);
   status = WFDRIVER (pwf->name, ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, pwf->bptr, pwf->nelm, &pwf->nord );

   return(status);
}

