/*drvBently.h,v 1.1.1.1 1999/08/03 14:41:55 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
**  File: drvBently.h
**  Edit: 02/03/98      WEP: created based on drvMultilin.h
**
*/

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/*	number of lines of bently nevada      */
#define		BNLINES	        32
/*	number of bently nevada plcs per line */
#define		NUMBNPLCS	32

/*      number of groups which can be defined in the PCU/LGU */
#define         MAXGROUPNUM     16

#define         RES_LOCKED       0

/*
**	The IOC will assume that the data to be exchanged with each PLC will           	
**	come from a fixed number of coils, digital inputs, analog inputs               	
**	or registers. It will read these in groups to minimize the amount of           	
**	communications overhead. It will also try to build as large a fetch            	
**	message as possible & required by remembering which consecutive groups         	
**	have been requested by the SCAN task.                                          	
*/

#define MDINS		1			/* Card number for digital inputs 	*/
#define MAINS		3			/* Card number for analog inputs 	*/
#define MREGS		4			/* Card number for registers 		*/
#define WAVEFORM	5			/* Card number for waveform 		*/
#define MPSEUDO		33			/* Card number for pseudo i/o 		*/
#define NUMSLOTS       12
#define NUMPPLS        16

/* Number of Groups of items */
#define NUMBNDIS	5        /*     36 */   /*    5 groups of 8 digital inputs      */
#define NUMBNAIS	9        /*     36 */   /*    9 groups of 4 analogs		*/
#define NUMREGISTERS	9        /*     36 */   /*    9 groups of 4 registers		*/
#define NUMBNWFS        NUMSLOTS*NUMPPLS        /*  196 groups of waveforms             */

#define BITBLOCK        8			/* size of digital input groups         */
#define REGBLOCK	4			/* size of analog/register groups	*/
#define WFBLOCK        40

/* Number of items */
#define MINDINO         0                       /* min number of individual digitals 	*/
#define MINAINO         0                       /* min number of individual analogs     */
#define MINREGNO        0                       /* min number of individual registers	*/
#define MINSLOTNO       1                       /* min number of individual slots    	*/
#define MINPPLNO        1                       /* min number of individual PLLs     	*/

#define WAVEFORMBASE  310
#define WAVEFORMLENGTH WFBLOCK

#define MAXDINO         36                      /* max number of individual digitals 	*/
#define MAXAINO         NUMBNAIS*REGBLOCK       /* max number of analogs 		*/
#define MAXREGNO        NUMREGISTERS*REGBLOCK   /* max number of registers 		*/
#define MAXSLOTNO       12                      /* min number of individual slots    	*/
#define MAXPPLNO        16                      /* min number of individual PLLs     	*/

#define MAXDIGBLKSIZE	64			/* Maximum number of digital groups 	*/
						/*  to request in one packet        	*/
#define MAXREGBLKSIZE	16			/* Maximum number of digital groups 	*/
						/*  to request in one packet        	*/

/*
**	Conversion types for analog inputs	*/
/*
*/
#define		AICNVINTEGER	10
#define		AICNVDOUBLEINT	11
#define		AICNVFLOAT	12
#define		AICNVDOUBLEFLT	13
#define		AICNVASCIIFLT	14
#define		AICNVSTATUS	15
#define		AICNVTIME	16
#define		AICNVASCII_2	17
#define		AICNVASCII_4	18
#define		AICNVASCII_12	19
#define		AICNVASCII_20	20
#define		MAXCONV		20

/* The following must match the definition in recChoice.ascii */
#define NOCONV 0
#define LINEAR 1
#define ASCFLT 2


#define    WATCHDOGTIMEOUT       10*60 /* watchdog monitor timeout.  */


/*
**	Communications I/O Structures
**
*/

#define		RXSEM		0
#define		TXSEM		1
#define		MAXRXMSG	256
#define		MAXTXMSG	256


typedef	struct {
	unsigned short		rxstate;
	unsigned short		fixedlen;
	unsigned short		rxmsglenexpected;
        unsigned short          rxmsglen;
       	unsigned short		rxix;
	SEMAPHORE		*semid;
	unsigned short		*status;
        unsigned char           txaddress;
        unsigned char           txfunction;
       	unsigned char		rxbuf[MAXRXMSG];
		} 	RXLINE;


typedef	struct {
	unsigned short		protocol;
	unsigned short		txdisable;
	unsigned short		txtocnt;
	unsigned short		txhwerrcnt;
	unsigned short		txmsglen;
	unsigned short		txix;
	SEMAPHORE		*semid;
	unsigned short		*status;
	unsigned char		txbuf[MAXTXMSG];
		} 	TXLINE;
/*
**	Communications I/O Constants
**
*/

#define		RXGOOD			1
#define		RXBAD			2
#define		RXTIMEOUT		60	/* 16 ms ticks 				*/
#define		ACTIVITYTIME		300	/* must see activity in 5 seconds 	*/

#define		VARRESP			FALSE
#define		FIXEDRESP		TRUE
#define		ONERETRY		0x8000	/* OR'ed into line number for xfermsg 	*/
#define		RXIGNORE		0
#define		RXADRST			1
#define		RXFNCST			2
#define		RXLENST			3
#define		RXDATAST		4

#define		TXGOOD			1
#define		TXBAD			2
#define		IO_TIMEOUT		3	/* Code returned by timer 		*/


#define		TXTIMEOUT		60	/* 16 ms ticks: one second              */

#define		MAXRETRYS		3	/* retransmission trys 			*/

#define		MAXCFGCK		8	/* Try to Bring online each cycle 	*/
#define		BRINGBACKCNT		4	/* Poll loops per cycle 		*/

#define		XFERGOOD		1
#define		XFERBAD			2

#define		RESNAMELEN		16	/* length of a resource name 		*/
#define		SLWWINDOW		900	/* # of ticks to allow slave to try 	*/


/*******************/
#define VALIDDATA   0
#define INVALIDDATA 1
#define BADADDRESS  2




/*
** Output Queue Management Structures for Bently/Modbus Protocol
**
*/

#define         AOUTTYPE        1

#define         OUTQSIZE        128
#define		STRINGMSGLEN	80

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	short           aovalue;
	double		dblaovalue;
	unsigned short	conversion;
		} AOUT, *AOUTPTR;

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	long            bomask;
	long            bovalue;
		} BOUT, *BOUTPTR;


typedef struct {
		unsigned short  otype;
		union   {
			AOUT    aonode;
			BOUT    bonode;
			}out;
	       } OUTQENT;

typedef struct {
		unsigned short	ooutix;
		unsigned short	oinix;
		OUTQENT         outque[OUTQSIZE];
		} OUTPUTQUE;
/*
**	Internal IOC Structure for Bently PLC
**
*/
typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		last_read;
		}	BNDIS;

typedef struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		ioregs[REGBLOCK];
	unsigned short		data_togo[REGBLOCK];
	unsigned short		regs_togo[REGBLOCK];

		}	REGISTERS;

typedef struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	short			aivals[REGBLOCK];
		}	BNAIS;

typedef struct {
        unsigned short	        was_requested;
        unsigned short          data_valid;
        unsigned short          data[WFBLOCK];
        unsigned short          slotnumber;
        unsigned short          pplnumber;
		}        BNWFS;


typedef	struct	{
	unsigned short  was_requested;		/* 0  The scan task has accesses this RTU 	*/
	unsigned short  is_responding;		/* 1  The RTU is responding 			*/
	unsigned short  holdflag;		/*    The RTU data is in hold 			*/
	unsigned short  no_dataout_msgs;	/* 5  Count Output Msgs 			*/
	unsigned short  no_datain_msgs;		/* 6  Count Data Requests 			*/
	unsigned short  no_rxnoresp;		/* 7  Count No Responses 			*/
	unsigned short  no_txnoresp;		/* 8  Count No Responses 			*/
	unsigned short  no_dataerr;		/* 9  Count Message Echo Errors 		*/
	unsigned short  no_retries;		/* 10 Count Retries 				*/
	unsigned short  no_rxhwerrs;		/* 11 Count Comm Hardware Errors 		*/
	unsigned short  no_txhwerrs;		/* 12 Count Comm Hardware Errors 		*/
	unsigned short  no_crcerrs;		/* 13 Count CRC Errors 				*/
	unsigned short  no_offlines;		/* 14 Count Off Line Transitions		*/
	unsigned short  cmd_reset_stat;		/* 15 Comand to reset stats			*/
	unsigned short  cmd_force_offline;	/* 16 Command to Force Offline 			*/
	unsigned short  badoutcnt;		/* 17 Count Failed output commands 		*/

        unsigned short  tscanpass;		/* Total number of scanpasses. incremented at   */
                                                /*  the start of each plc scan.                 */
        unsigned short  msgs;			/* Count of messages which got good responses.  */
                                                /*  incremented on each exit from 'good' msgxfer*/
        unsigned long   responsetime;		/* Total time(ticks) waiting for good responses.*/
                                                /* accumu (exit-entry) time for 'good' msgxfers */

        BNDIS           dis [NUMBNDIS];
	BNAIS           ais [NUMBNAIS];
	REGISTERS       regs[NUMREGISTERS];
	BNWFS           wf;
	}	BNPLC;
/*
** MODBUS Protocol function codes
*/

/*#define MODCOILREAD		0x01  ** READ COIL STATUS: (ON/OFF) read a group of logic coils.     */
#define MODDIREAD		0x02  /* READ INPUT STATUS:(ON/OFF) of a group of discrete inputs.   */
#define MODREGREAD		0x03  /* READ HOLDING REGISTERS:    get current binary value in one  */
                                      /*                            or more holding registers.       */
#define MODAIREAD		0x04  /* READ INPUT REGISTERS:      get current binary value in one  */
                                      /*                            or more input registers.         */

#define MODCOILWRITE		0x05  /* FORCE SINGLE COIL:         force logic coil to ON/OFF state.*/
#define MODREGWRITE		0x06  /* PRESET SINGLE REGISTER:    place a specific binary value    */
                                      /*                            into a holding register.         */

/*#define MODEXCEPTREAD           0x07  ** READ EXCEPTION STATUS:     get the 8 predefined exception   */
                                      /*                            status bits.             .       */
#define MODLOOPBACK		0x08  /* LOOPBACK DIAGNOSTIC TEST                                    */

/*#define MODMCOILWRITE		0x0F  ** FORCE MULTIPLE COILS:      force series of consecutive      */
                                      /*                            logic coils to ON/OFF states.    */
#define MODMREGWRITE		0x10  /* PRESET MULTIPLE REGISTERS: put specific binary values into  */
                                      /*                            consecutive series of holding    */
                                      /*                            registers.                       */

#define FNCERRBIT		0x80  /* Modbus error function code */


/* Various message lengths used in message transmission and response validation */

#define MODWRITEHDRLEN		0x06	/* wep: not currently used */
#define MODREADMSGLEN		0x06
#define MODSETCOILLEN		0x06
#define MODSETREGLEN		0x06
#define MODSET1REGLEN		0x09
#define MODSET2REGLEN		0x0b
#define MODSET3REGLEN		0x0d
#define MODSET4REGLEN		0x0f

#define MODEXCEPTLEN            0x02	/* wep: specially added for Bently */
#define MODEXCEPTRESPLEN        0x05    /* Including CRC */

#define MODMSETCOILLEN		0x07	/* wep: not currently used */
#define MODMSETCOILRESPLEN	0x06
#define MODCRCLEN		0x02
#define MODLOOPBACKLEN		0x06
#define MODLOOPBACKRESPLEN	0x08	/* Including CRC */
#define MODWRITERESPLEN		0x08	/* Including CRC */
#define MODSETMREGRESPLEN	0x06
#define NAKMSGLEN		0x03	/* bytes left get */

/*
** MODBUS Exception Codes
*/

#define MODEXCEPTFNC		0x01	/* wep: not used */
#define MODEXCEPTADR		0x02	/* "    "   "    */
#define MODEXCEPTDATA		0x03	/* "    "   "    */
#define MODEXCEPSLAVE		0x04	/* "    "   "    */
#define MODEXCEPTSACK		0x05	/* "    "   "    */
#define MODEXCEPTBUSY		0x06	/* "    "   "    */

/* Command Results */

#define CMDDONE                 0x00     /* the command was sent and the response was error free.   */
#define CMDFAIL                 0x01     /* there were errors in sending or receiving the command.  */
#define CMDNACK                 0x02     /* response was error free, but command was not completed. */
#define CMDREJECT               0x03     /* driver rejected command due to unsupported data type.   */
#define CMDBUSY			0x05	 /* The command was sent, but the device is busy */

/* task definitions */

#define BN_STACK                8000   /* Stack size              */
#define	BN_PRIORITY             152    /* default priority        */
#define	BN_TASK_OPTIONS	8              /* Floating point          */



/*
** Mutual-Exclusion Control for diagnostic/data radio channel sharing */
/*
*/

#define		MUXTIMEOUT	60*30	/* 16.67 ms ticks (60Hz): 30 seconds 	*/


typedef struct {
        unsigned long   TXintcount;
        unsigned long   RXintcount;
        unsigned long   RXbadcount;
        unsigned long   msgxfercount;
        unsigned long   startingtick;
               }        LINESTATS;

/* Trace Bit Definitions */

#define ALLTRAFFICTRACE      0x0001
#define MODBUSNACKTRACE      0x0002        /* MODBUS NACK errors.                               */
#define CONFIGURATIONTRACE   0x0010        /* trace configuration errors in driver calls.       */
#define RUNTIMEERRORTRACE    0x0020        /* trace run time errors in driver calls.            */
#define CALLREJECTTRACE      0x0040        /* trace non-error based driver call rejections.     */


/*	End of File:	drvBently.h */
