
/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
/*  File: drvInf7.c
/*  Edit:  12/2/97      WEP: created driver                                                         /*
*/

#include <ctype.h>
#include <vxWorks.h>
#include <sysLib.h>
#include <semLib.h>
#include <wdLib.h>
#include <rngLib.h>
#include <task_params.h>
#include <stdioLib.h>
#include <dbDefs.h>
#include <drvSup.h>
#include <module_types.h>
#include <drvInf7.h>			  /* NOTE: change here */
#include <drvInf7_proto.h> 		  /* NOTE: change here */
#include <restable.h>

struct
{
	long		number;
	DRVSUPFUN	report;
	DRVSUPFUN	init;
}
drvTankway =
{
	2,
	NULL,
	init
};

/*
/*      External Line Level Data Structures (declared in drvcommon.c)
/*
*/

/* array of resource name strings.		*/

extern	unsigned char	resnames [INF7LINES] [RESNAMELEN];

/* array of pointers to resource tables 	*/

extern	RES_TABLE	*prestable [INF7LINES];	

/* array of resource response flags. 		*/
/* False=not active resource(slave),		*/
/* True= active resource(master) 		*/

extern	unsigned short	resresponse [INF7LINES];

/* true/false line active flag. calculated	*/
/* using activity_time below. 			*/

extern	unsigned short	line_activity [INF7LINES];

/* assigned value of line_activity prior to	*/
/* it's recalculation.   			*/

extern	unsigned short	lastactivity [INF7LINES];

/* controls output of communication error log	*/
/* messages (true/false) 			*/

extern	unsigned short	modtrace [INF7LINES];

/* controls output of certain resource related	*/
/* log messages (level) 			*/

extern	unsigned short	mltrace [INF7LINES];

/* controls output of MASTER/SLAVE arbitration	*/
/* log messages (binary) 			*/

extern	unsigned short	modrestrace [INF7LINES];

/* 'new' semaphore logic. the old SEMAPHORE is	*/
/* still used. 					*/

extern	SEM_ID		semids [INF7LINES] [2];

/* holds value passed in task spawn. used for	*/
/* each msgxfer attempt 			*/

extern	unsigned short	intermsgdelay [INF7LINES];

/* holds post RTS delay before TX. 		*/

extern	unsigned short	pretxdelay [INF7LINES];

/* holds post TX delay before dropping RTS. 	*/

extern	unsigned short	posttxdelay [INF7LINES];
/* holds timetick of last activity. used to	*/
/* set line_activity flag  			*/

extern	long		activity_time [INF7LINES];

/* store half/full duplex line status           */

extern  unsigned short  halfduplex [INF7LINES];

/* holds max time in seconds that driver will   */
/* wait to obtain partner state on first        */
/* attempting to decide if it should be master. */

extern  unsigned short  pstatewait [INF7LINES];

/*
/*      Storage for Infinity 7 Comm Line Structures
/*
*/

/* array of pointers to dynamically           */
/* allocated comm controls/buffers            */

static RXLINE          *rxlinetab [INF7LINES] =
{
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL
};

/* array of pointers to dynamically           */
/* allocated comm controls/buffers            */

static TXLINE          *txlinetab [INF7LINES] =
{
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL, NULL, NULL, NULL,
	NULL, NULL
};

/*
/*      Storage for Infinity 7 PLC Structures
/*
*/

/* 0 = redundant mode,   !0=private mode */

static unsigned short   pryvate [INF7LINES];

static unsigned int     tick [INF7LINES][5];

/* holds timestamp of beginning of major loop. */

static unsigned int     lastscan [INF7LINES];

/* holds actual time taken to scan units. */

static unsigned int     scantime [INF7LINES];

static unsigned int     loopoutmsgs [INF7LINES];

static unsigned int     loopmsgs [INF7LINES];

/* array of line level statistics. */

static LINESTATS	linestats [INF7LINES];

/* two dimensional array of pointers to the     */
/* fixed length dynamically allocated Unit      */
/* control and data structures.                 */

static INF7		*units [INF7LINES] [NUMUNITS];

/* controls output of system error messages: */
/* bit mapped controls */

static  unsigned short  trace [INF7LINES];

static  unsigned char   traceadr [INF7LINES];

static  unsigned char   tracetype [INF7LINES];

/* holds id and password of the user that       */
/* spawned the driver task required by host to  */
/* check file read privileges on PLC load files */

static char		user [40], password [40];

/* store half/full duplex line status */

static  unsigned short  options [INF7LINES];

static char		*bools [] =
{
	"TRUE ",
	"FALSE"
};

static char		send_outputs [INF7LINES] =
{
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0
};

static int		taskId [INF7LINES];

static char		*tasknames [INF7LINES] =
{
	"Inf7 Ch0", "Inf7 Ch1", "Inf7 Ch2", "Inf7 Ch3",
	"Inf7 Ch4", "Inf7 Ch5", "Inf7 Ch6", "Inf7 Ch7",
	"Inf7 Ch8", "Inf7 Ch9", "Inf7 Ch10","Inf7 Ch11",
	"Inf7 Ch12","Inf7 Ch13","Inf7 Ch14","Inf7 Ch15",
	"Inf7 Ch16","Inf7 Ch17","Inf7 Ch18","Inf7 Ch19",
	"Inf7 Ch20","Inf7 Ch21","Inf7 Ch22","Inf7 Ch23",
	"Inf7 Ch24","Inf7 Ch25","Inf7 Ch26","Inf7 Ch27",
	"Inf7 Ch28","Inf7 Ch29","Inf7 Ch30","Inf7 Ch31"
};

/**********
*
*	This routine is required by iocinit in the EPICS system.
*
**********/
static long init ()
{
	return (0);
}

/********
/*
/*	This routine outputs information to help with debugging.
/*
********/
void inf7help (void)
{
    printf ("Starting and Stopping the Infinity 7 Driver:\n");
    printf ("\n");
    printf (" startinf7(N, C, t1, f, t2, t3, P)\n");
    printf ("                     N:  start the driver on line N.\n");
    printf ("                     C:  communications parameter string (comma delimited ascii)\n"); 
    printf ("                     t1: intermessage delay (in ticks)\n");
    printf ("                     f:  half duplex flag. 1 = half duplex, 0 = full duplex\n");
    printf ("                     t2: post RTS assert, pre TX delay (in ticks)\n");
    printf ("                     t3: post TX, pre RTS drop delay (in ticks)\n");
    printf ("                     P:  Private flag.  0 = Redundant, 1 = Non-Redundant\n");
    printf ("\n");
    printf (" Example: startinf7(2, \"9600,N,8,1\", 2, 1, 4, 0)\n");
    printf ("\n");
    printf (" stopinf7(N)\n");
    printf ("                     N = line number\n");
    printf ("\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" Infinity 7 Driver Control & Display Functions (* = default)\n");
    printf ("\n");
    printf (" inf7traceon(N)         Enable the following debug messages for line N:\n");
    printf ("                          1) Major Scan loop Info:\n");
    printf ("                            scan time, total output messages, and total messages.\n"); 
    printf ("                          2) Output Queue Overflow.\n");
    printf ("                          3) All communication errors encountered in msgxfer.\n");
    printf ("*inf7traceoff(N)        Disable all of the inf7traceon debug messages for line N.\n");
    printf ("\n");
    printf (" inf7restrace(N,X)      Resource Trace Level: 0 = off. 1 = state change; 2 = delay.\n");
    printf ("*inf7restraceon(N)      Enable display of master/slave state changes.\n");
    printf (" inf7restraceoff(N)     Disable display of master/slave state changes.\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" inf7contraceon(N)      Enable display of configuration errors in driver calls.\n");
    printf ("*inf7contraceoff(N)     Disable display of configuration errors in driver calls.\n");
    printf (" inf7runtraceon(N)      Enable display of runtime errors (queue overflow).\n");
    printf ("*inf7runtraceoff(N)     Disable display of runtime errors (queue overflow).\n");
    printf (" inf7rejecttraceon(N)   Enable display of driver call rejections in driver calls.\n");
    printf ("*inf7rejecttraceoff(N)  Disable display of driver call rejections in driver calls.\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" inf7trlv(N,X)          Enable the following message for line N:\n");
    printf ("                          X=0,  display none of the messages below.\n");
    printf ("                          X=1,  beginning of unit scan loop, bringing units online.\n");
    printf ("                          X=2,  beginning of unit scan.\n");
    printf ("                          X=3 to 7,  reserved.\n");
    printf ("                          X=8,  AIDRIVER, AOREAD: normal call return.\n");
    printf ("                          X=9,  BIDRIVER, BOREAD: normal call return.\n");
    printf ("                          X=10 to 254,  reserved.\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" inf7trafficon(N)       Enable traffic tracing for all unit addresses\n");
    printf ("                          and all Infinity 7 functions on line N.\n");
    printf ("*inf7trafficoff(N)      Disable all traffic tracing on line N.\n");
    printf (" inf7traffic(N,X,Y)     Modifies line N traffic tracing as follows:\n");
    printf ("                          X=0   disables tracing of all addresses.\n");
    printf ("                          X=255 enables tracing of all addresses.\n");
    printf ("                          0 < X < 255 enables tracing of address X traffic.\n");
    printf ("                         Y=0   disables tracing of all Infinity 7 functions.\n");
    printf ("                         Y=255 enables tracing of all Infinity 7 functions.\n");
    printf ("                          0 < Y < 255 enables tracing of function Y traffic.\n");
    printf ("\n");
    printf (" inf7defaulttrace(N)    Restores default setting for all traces.\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" inf7info(N)            Display startup and current state information.\n");
    printf (" inf7comm(N,t1,f,t2,t3) Set communication timing parameters for line N as follows:\n");
    printf ("                          t1: intermessage delay (in ticks).\n");
    printf ("                           f: 0 = full duplex, 1 = half duplex.\n");
    printf ("                          t2: post RTS assert, pre TX delay (in ticks).\n");
    printf ("                          t3: post TX, pre RTS drop delay (in ticks).\n");
    printf ("\n");
    printf (" inf7dumpmsgcnts(N)     Display line N communication statistics.\n");
    printf (" inf7clearmsgcnts(N)    Display and clear line N communication statistics.\n");
    printf (" inf7dumpstats(N,X)     Display line N, unit X, communication statistics.\n");
    printf (" inf7clearstats(N,X)    Display and clear line N, unit X, xfers, status & errors.\n");
    printf (" inf7table(N,X)         Display line N, unit X, xfers, status, errors and input data.\n");
}


/********
/*
/*	This routine enables logging of comm trace messages
/*
********/
void inf7traceon (unsigned short line)
{
	modtrace [line] = TRUE;
}

/********
/*
/*	This routine disables logging of comm trace messages
/*
********/
void inf7traceoff (unsigned short line)
{
	modtrace [line] = FALSE;
}

/********
/*
/*	This routine enables detailed display of normal communications traffic
/*
********/
void inf7traffic (unsigned short line, unsigned char add, unsigned type)
{
	trace [line] = trace [line] | ALLTRAFFICTRACE;
        traceadr [line] = add;
        tracetype [line] = type;
}

/********
/*
/*	This routine enables detailed display of normal communications traffic
/*
********/
void inf7trafficon (unsigned short line)
{
	trace [line] = trace [line] | ALLTRAFFICTRACE;
        traceadr [line] = 255;  /* every address */
        tracetype [line] = 255; /* every message type */
}

/********
/*
/*	This routine disables detailed display of normal
/*	communications traffic
/*
********/
void inf7trafficoff (unsigned short line)
{
	trace [line] = trace [line] & ~ALLTRAFFICTRACE;
}

/********
/*
/*	This routine enables logging of resource trace messages
/*
********/
void inf7restrlv (unsigned short line, unsigned short level)
{
	modrestrace [line] = level;
}

/********
/*
/*	This routine enables logging of resource trace messages
/*
********/
void inf7restraceon (unsigned short line)
{
	modrestrace [line] = 1;
}

/********
/*
/*	This routine disables logging of resource trace messages
/*
********/
void inf7restraceoff (unsigned short line)
{
	modrestrace [line] = 0;
}

/********
/*
/*	This routine controls logging of trace messages
/*
********/
void inf7trlv (unsigned short line, unsigned short level)
{
	mltrace [line] = level;
}

/********
/*
/*	These routines enable and disable tracing of database
/*	configuration errors in driver calls
/*
********/
void inf7contraceon (unsigned short line)
{
	trace[line] = trace[line] | CONFIGURATIONTRACE;
}

void inf7contraceoff (unsigned short line)
{
	trace[line] = trace[line] & ~CONFIGURATIONTRACE;
}

/********
/*
/*	These routines enable and disable tracing of runtime
/*	errors in driver calls
/*
********/
void inf7runtraceon (unsigned short line)
{
	trace[line] = trace[line] | RUNTIMEERRORTRACE;
}

void inf7runtraceoff (unsigned short line)
{
	trace[line] = trace[line] & ~RUNTIMEERRORTRACE;
}

/********
/*
/*	These routines enable and disable tracing of call rejection
/*	errors in driver calls
/*
********/
void inf7rejecttraceon (unsigned short line)
{
	trace[line] = trace[line] | CALLREJECTTRACE;
}

void inf7rejecttraceoff (unsigned short line)
{
	trace[line] = trace[line] & ~CALLREJECTTRACE;
}


/********
/*
/*	This routine restores default setting for all traces.
/*
********/
void inf7defaulttrace (unsigned short line)
{
        /* debug structures */

	modtrace [line] = FALSE;
	mltrace [line] = 0;
	modrestrace [line] = 1;
        trace [line] = 0;
        traceadr [line] = 255;  /* every address      */
        tracetype [line] = 255; /* every message type */
}
/********
/*
/*	This routine turns on all error tracing.
/*
********/
void inf7errortrace (unsigned short line)
{
        /* debug structures */

	modtrace [line] = TRUE;
	modrestrace [line] = 2;
	trace[line] = MODBUSNACKTRACE |
                      CONFIGURATIONTRACE |
                      RUNTIMEERRORTRACE |
                      CALLREJECTTRACE |
                      0;
        traceadr [line] = 255;  /* every address      */
        tracetype [line] = 255; /* every message type */
}


int stopinf7 (short ln)
{
	if (comm_disconnect (ln) != -1)
	{
	    taskDelete (taskId [ln]);
	    return (1);
	}
	else
	{
	    return (-1);
	}
}

int startinf7 (unsigned short line,      /* the line number of this driver.                      */
                 char *mode,               /* UART initializations string (comma delimited ascii). */
                 unsigned short delay,     /* intermessage delay value, expressed in ticks.        */
                 unsigned short hdflag,    /* half duplex flag. 1=half duplex mode, 1!=full duplex */
                 unsigned short delay1,    /* half duplex post RTS assert, pre TX delay.           */
                 unsigned short delay2,    /* half duplex post TX, pre RTS drop delay.             */
                 unsigned short private)   /* private flag. if 0, normal. if !0, no partner driver.*/

{
	if (line < INF7LINES)
	{
	    taskId [line] = taskSpawn (tasknames [line],
                                     INF7_PRIORITY,
                                     INF7_TASK_OPTIONS,
                                     INF7_STACK,
                                     inf7_driver,
                                     line,
                                     mode,
                                     delay,
                                     hdflag,
                                     delay1,
                                     delay2,
                                     private);

	    logMsg ("Started the Inf7 driver on line %d\n", tasknames [line],
			line);

	    remCurIdGet (user, password);
	    return (1);
	}
	else
	{
	    logMsg ("Invalid line number %d\n", line);
	    return (-1);
	}
}

/********
/*
/*	Set communication logic time delays to handle half/full duplex,
/*	radio delays, etc */ 
/*
********/
void inf7comm (unsigned short line,
              unsigned short tickcount1,
              unsigned short hdflag,
              unsigned short tickcount2,
              unsigned short tickcount3)
{
	intermsgdelay [line] = tickcount1;  /* at beginning new message/response sequence. */
	halfduplex [line] = hdflag;
	pretxdelay [line] = tickcount2;  /* after asserting RTS but before beginning transmission. */
	posttxdelay [line] = tickcount3;  /* after sending last character but before dropping RTS. */
}

/********
/*
/* Show Driver Info and Current State Information */ 
/*
********/
void inf7info (unsigned short line)
{
    printf ("intermsgdelay [%d]  = %d\n", line, intermsgdelay [line]);
    printf ("halfduplex [%d]     = %d\n", line, halfduplex [line]);
    printf ("pretxdelay [%d]     = %d\n", line, pretxdelay [line]);
    printf ("posttxdelay [%d]    = %d\n", line, posttxdelay [line]);
    printf ("pryvate [%d]        = %d\n", line, pryvate [line]);
    printf ("resresponse [%d]    = %d\n", line, resresponse [line]);
    printf ("activity_time [%d]  = %d\n", line, activity_time [line]);
    printf ("line_activity [%d]  = %d\n", line, line_activity [line]);
    printf ("lastactivity [%d]   = %d\n", line, lastactivity [line]);
    printf ("modtrace [%d]       = %d\n", line, modtrace [line]);
    printf ("mltrace [%d]        = %d\n", line, mltrace [line]);
    printf ("modrestrace [%d]    = %d\n", line, modrestrace [line]);
    printf ("trace [%d]          = %d\n", line, trace [line]);
}


/**********
*
*      inf7_driver
*
*	The purpose of this task is to scan the Infinity 7 unit lines. It
*	will maintain a configuration list supporting online additions
*	and replacement of modules, and maintain a table of most recently
*	scanned values.                                                    
*
**********/

/* not called from within this driver 	*/

void inf7_driver (unsigned short line, unsigned char *mode, unsigned short delay,
			unsigned short hdflag, unsigned short delay1,
			unsigned short delay2, unsigned short private)
{
	unsigned short	unit, cfgchk, cnt, unitspolled, onlinefreq;
	unsigned short	activity, pstate, lstate, execute, command, i;
	long		this_time, last_time;
	INF7		*punit;
	RXLINE		*rxp;
        unsigned int	temptick, tempdiff;
        int	temp;

	/* Initialize the data structures */

	intermsgdelay [line] = delay;
	halfduplex [line] = hdflag;
	pretxdelay [line] = delay1;
	posttxdelay [line] = delay2;
	pryvate [line] = private;
	resresponse [line] = FALSE;
	resnames [line] [0] = 0;
	prestable [line] = NULL;

	activity_time [line] = tickGet ();
	lastactivity [line] = line_activity [line] = FALSE;

	/* will wait up to one minute for partner to become valid */

	pstatewait [line] = 60;

	/* debug structures */

/*	inf7debugtrace (line); */
	inf7defaulttrace (line);

	tick [line] [0] = tick [line] [1] = tick [line] [2] = tick [line] [3] = 0;

	linestats [line].TXintcount = 0;
	linestats [line].RXintcount = 0;
	linestats [line].RXbadcount = 0;
	linestats [line].msgxfercount = 0;
	linestats [line].startingtick = tickGet ();

	/* communication initialization */

	rxlinetab [line] = (RXLINE *) calloc (1, (sizeof (RXLINE)));
	if (!rxlinetab [line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: rx)\n",line);
	    return;
	}
	txlinetab [line] = (TXLINE *) calloc (1, (sizeof (TXLINE)));
	if (!txlinetab [line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: tx)\n",line);
	    return;
	}

	/* make terminal display easier to read */

	taskLock ();
	comm_init (line, mode, &tx_int_proc, &rx_int_proc);
	clear_rts (line);
	taskUnlock ();

	/* causes on_line calls to begin on first pass if master */

	onlinefreq = 1;

	/* Initialize the Rx ints to receive once for activity check */
	/* enable the receiver interrupt after set to ignore */

	rxp = rxlinetab [line];
	rxp->rxstate = RXIGNORE;
	enable_rx_ints (line);

	/* Initialize the line communication semaphores */

	semids [line] [0] = semBCreate (SEM_Q_PRIORITY, SEM_EMPTY);
	semids [line] [1] = semBCreate (SEM_Q_PRIORITY, SEM_EMPTY);

	/* Wait for resource to be defined via a call to 'register_resource' */

	while ((resnames [line] [0] == 0) || (prestable [line] == NULL))
	{
	    if (modrestrace [line] > 1)
	    {
	    	logMsg ( "No Resource Defined for line - %d \n", line);
	    }
	    taskDelay (600);
	}

	if (pryvate [line])
	{
	    /*****************************/
	    /* Immediately become master */
	    /*****************************/

	    resSetCaAnswer (prestable [line]);
	    resPutLocalState (prestable [line], RES_MSTR);

	    lastscan [line] = tickGet ();
	    scantime [line] = 0;

	    lstate = RES_MSTR;
	    if (modrestrace [line])
	    {
	    	logMsg ("0] Private: Initialize as RES_MSTR\n");
	    }
	}
	else /* this is the 'normal' path */
	{
	    /**********************/
	    /* Start out as slave */

	    resClearCaAnswer (prestable [line]);	/* start quiet */
	    resPutLocalState (prestable [line], RES_SLV);

	    if (modrestrace [line])
	    {
		logMsg ("1] Initialized as RES_SLV\n");
	    }

	    /* wait finite time for pstate to be resolved */
	    logMsg ("Begin wait for pstate [%d] to become valid.\n",
			pstatewait [line]);
	    for (i = 0; i <= pstatewait [line]; i++)
	    {
		resGetStates (prestable [line], &lstate, &pstate);
		if (pstate != 0)
		{
		    break;
		}
		taskDelay (60);
	    }
	    if (pstate == 0)
	    {
		logMsg ("Timeout of pstate wait.\n");
	    }
	    else
	    {
		logMsg ("Completion of pstate wait.\n");
	    }
	    for (i = 0; i < ACTIVITYTIME/10; i++) ;  /* ten seconds */
	    {
		taskDelay (10);
	    }
	    activity = check_line_activity (line, 0);
	    resGetStates (prestable [line], &lstate, &pstate);

	    /******************************************/
	    /* if the partner is not there, be master */

	    if ((pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity))
	    {
		resSetCaAnswer (prestable [line]);
		resPutLocalState (prestable [line], RES_MSTR);
		lastscan [line] = tickGet ();
		scantime [line] = 0;

		lstate = RES_MSTR;
		if (modrestrace [line])
		{
		    logMsg ("2] Became RES_MSTR   P= %x A= %d \n",
				pstate, activity);
		}
	    }
	}
	cfgchk = 0;		/* Init the new config pointer */
	last_time = tickGet (); /* Initialize the time  */

	while (TRUE)
	{
	    activity_time [line] = tickGet ();
	    lastactivity [line] = line_activity [line] = FALSE;

/**********
*
*	we arrive here because we are a slave.
*
*	we stay until:
*	asked to be master by partner. if we can reach Units, we become master.
*	the partner is not master and there is no activity on the line.
*
**********/

	    while (lstate == RES_SLV)
	    {
		if (pstate != RES_SLVWANT)
		{
		    /* make sure line is left for master */
		    clear_rts (line);
		}

 		/* make sure driver is not responding */

		resresponse [line] = FALSE;

 		/* clear out commands */

		resPutCommand (prestable [line], 0);

 		/* wait 10 seconds before checking */

		activity = check_line_activity (line, 10);
		resGetStates (prestable [line], &lstate, &pstate);

		/**********
		*
		*	see if needed by partner to be master.
		*	if yes, do it and break out of slave state.
		*
		**********/

		if (pstate == RES_SLVWANTCA)
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;

                    lastscan [line] = tickGet ();
                    scantime [line] = 0;

		    if (modrestrace [line])
		    {
			logMsg ("3] Became RES_MSTR  P= %x A= %d \n",
				pstate, activity);
		    }
		    break;
		}

	/***************************************************************/
	/* If partner is there and wants to give it up, try to take it */

		if (pstate == RES_SLVWANT)
		{
		    execute = FALSE;
		    taskDelay (2);	/* Time for switch to occur */

		    /* See if anyone is out there and able to respond to a loopback command */
		    for (unit = 0; (unit <= NUMUNITS); unit++)
		    {
			punit = units [line] [unit];
			if ((punit) &&		/* verify pointer validity prior to use */
			    (punit->was_requested) &&	/* data points have been requested */
                            (!punit->cmd_force_offline)) /* user has NOT forced unit offline */
			{
			    on_line (line, unit);

                            /* check cmd_force_offline flag before
				AND after due to asynchronous tasks */

			    if ((punit->is_responding) &&
				(!punit->cmd_force_offline))
			    {
				execute = TRUE;
				break;
			    }
			}
		    }

		    /* If someone answered, become master and make sure
			partner really gave it up */

		    if (execute)
		    {
			resPutLocalState (prestable [line], RES_MSTR);
			lstate = RES_MSTR;

			lastscan [line] = tickGet ();
			scantime [line] = 0;

			if (modrestrace [line])
			{
			    logMsg ("4] Became RES_MSTR  P= %x A= %d \n",
					pstate, activity);
			}
			execute = FALSE;
			activity_time [line] = tickGet ();
			lastactivity [line] = line_activity [line] = FALSE;
			for (i = 0; (i < 60); i++)
			{
			    activity = check_line_activity (line, 2); /* pause before testing */
			    resGetStates (prestable [line], &lstate, &pstate);
			    if ((pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity) )
			    {
				execute = TRUE; /* Partner Gave It Up so take CA */
				break;
			    }
			}
			if (!execute)
			{   /* Partner did not give it up go back to slave */
			    resPutLocalState (prestable [line], RES_SLV);
			    lstate = RES_SLV;
			    if (modrestrace [line])
			    {
			    	logMsg ("5] Became RES_SLV  P= %x A= %d \n",
					pstate, activity);
			    }
			}
			else
			{ /* Assume control by taking the CA responsibilities */ 
			    resSetCaAnswer (prestable [line]);
			    prepresponses (line);
			    break;
			}
		    }
		}

		/********************************************/
		/* if the partner is not talking, be master */

		if ((pstate != RES_MSTR) &&  (pstate != RES_SLVWANT) && (!activity))
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;

		    lastscan [line] = tickGet ();
		    scantime [line] = 0;

		    if (modrestrace [line])
		    {
		    	logMsg ("6] Became RES_MSTR P= %x A= %d \n",
				pstate, activity);
		    }
		    prepresponses (line);
		}
	    }

            /**********
	    *
	    * to arrive here, we used to be master,
	    * and had been told to relinquish master status.
	    * stay in this loop until the partner takes over,
	    * or the command is canceled
	    *
	    **********/

	    while (lstate == RES_SLVWANTCA)
	    {
		taskDelay (2);                   /* Give OS a chance...  */
		clear_rts (line); 		 /* Make sure line is left for master    */
		resresponse [line] = FALSE; 	 /* Make Sure driver is not responding   */

		resGetStates (prestable [line], &lstate, &pstate);
		if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT))
		{
		    resClearCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_SLV);
		    lstate = RES_SLV;
		    if (modrestrace [line])
		    {
		    	logMsg ("7] Became RES_SLV  P= %x A= %d \n", pstate, activity);
		    }
		    break;
		}
		resGetCommand (prestable [line], &command);
		resPutCommand (prestable [line], 0); /* clear out commands */
		if (command == RES_CANCELREL)
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;
		    if (modrestrace [line])
		    {
		    	logMsg ("8] Became RES_MSTR  P= %x A= %d \n", pstate, activity);
		    }
		    prepresponses (line);
		}
	    }

	    /**********
	    *
	    * in normal operation this code should never execute.
	    * it has, however, been seen to execute when the database has been
	    * inadvertently configured with the same resource assigned to two
	    * different drivers in the same IOC. The state does funny things.
	    *
	    **********/

	    while (lstate == RES_SLVWANT)
	    {
		if ((modrestrace [line]) || (trace[line] & CONFIGURATIONTRACE))
		{

		    /* condition should NEVER occur. print error message every 5 seconds. */

		    logMsg ("Illegal Local Resource State.\n");
		    if (trace[line] & CONFIGURATIONTRACE)
		    {
			logMsg ("  Possible Resource Configuration Error:\n");
			logMsg ("   Check for multiple drivers sharing single Resource.\n");
		    }
		    taskDelay (600);
		}
		taskDelay (2);
	    }
	    /**********
	    *
	    * we come here because we have gone to, or are looping in,
	    * the master state.
	    * we stay in this loop until:
	    *	we have been told to give master state to the partner.
	    *	we break out to RES_SLVWANTCA.
	    *	we have successfully handed master state to partner.
	    *
	    * While master, this task will process all modules
	    * sending outputs and getting inputs.
	    * It will then try to configure requested non-responding modules.
	    *
	    **********/

	    last_time = tickGet ();		/* initialize gap */
	    while (lstate == RES_MSTR)
	    {
		if (mltrace [line] == 1)
		{
		    logMsg ( "Top of Master Loop... \n");
	        }
		/* initial stuff */
		resresponse [line] = TRUE;	/* Make sure driver is responding. driver won't */
						/* issue commands if not set. */

		resGetCommand (prestable [line], &command);
		resPutCommand (prestable [line], 0);	/* clear out commands */

		/************************************************************************/
/* if relinquish command sent, change state and break out to outer loop */

		if (command == RES_RELINQUISH)
		{
		    resClearCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_SLVWANTCA);
		    lstate = RES_SLVWANTCA;
		    if (modrestrace [line])
		    {
			logMsg ("9] Became RES_SLVWANTCA  P= %x A= %d \n", pstate, activity);
		    }
		    break;
		}

		/* measure time between scans of the entire set of units. */
		/* display if requested. */
		temptick = tickGet ();
		tempdiff = temptick - lastscan [line];

		if (modtrace [line])
		{
		    logMsg ("ScanTime: =%4d, # OutputMsgs =%5d, # TotalMsgs =%3d\n",
				scantime[line], loopoutmsgs[line],
				loopmsgs[line]);
		}

		loopoutmsgs [line] = 0;
		loopmsgs [line] = 0;
		lastscan [line] = temptick;

		/* scan units for data i/o, if allowed */
		unitspolled = 0;

		/* call 'scan_unit' for each PLC device, unless user has */
		/* taken it off line or it is not responding. */

		for (unit = 0; (unit < NUMUNITS); unit++)
		{
		    punit = units [line] [unit];	/* shorthand pointer */
		    if (punit)				/* verify pointer validity prior to use */
		    {
			if (punit->cmd_force_offline)
			{ 
			    /* if user has turned this unit offline via pseudo commands, */
			    /* mark it as not responding. */

 			    punit->is_responding = FALSE;
			}
			else if (punit->is_responding)
			{
			    /* local/partner lockstep recovery */

			    resGetStates (prestable [line], &lstate, &pstate);
			    if ((pstate == RES_MSTR) && (!pryvate[line]))
			    {
				resClearCaAnswer (prestable [line]);
				resPutLocalState (prestable [line], RES_SLV);
				lstate = RES_SLV;
				clear_rts (line); /* Make sure line is left for master */
				if (modrestrace [line])
				{
				    logMsg ("13] Became RES_SLV  P= %x A= %d \n",
						pstate, activity);
				}
				temp = ((tickGet () % 8) + 5); /* random 5 to 12 second delay */
				for ( i = 0; i < temp; i++)
				{
				   taskDelay(60);
				}
				break;
			    }
			    unitspolled++;

			    scan_unit (punit, line, unit); /* I/O done here! */

			    if (!punit->is_responding)
			    {
				mark_not_valid (punit);
                            }
			}
		    }
		    /* end of single unit action */

		    /* if told to relinquish our master state, stop unit scanning immediately. */
		    /* don't trash the command, so that we can still read it on the next loop */
		    /* through this while statement.                                          */

		    resGetCommand (prestable [line], &command);
		    if (command == RES_RELINQUISH)
		    {
			break;
		    }
		} /* end of scan loop */

		/* test for lockstep detection. */

		if (lstate != RES_MSTR)
		{
		    break;
		}
		/* measure how long the scan took. to be displayed at beginning of next loop. */
		temptick = tickGet ();
		scantime [line] = temptick - lastscan [line];

		/* try and bring all nonresponding units on line, but */
 		/* don't bring in more than a specified number.      */

		cnt = 0;
		if (--onlinefreq == 0) 		/* do we try to bring any in on this pass? */
		{ 
		    onlinefreq = BRINGBACKCNT;
		    for (unit = 0; (unit < NUMUNITS); unit++)
		    {
			cfgchk++;		/* manage unit index */
			if (cfgchk == NUMUNITS)
			{
			    cfgchk = 0;	/* wrap unit index */
			}
			punit = units [line] [cfgchk];
			if ((punit) &&			/* verify pointer validity prior to use*/
			    (!punit->is_responding) &&
			    (punit->was_requested) &&
			    (!punit->cmd_force_offline))
			{
			    on_line (line, cfgchk);	/* loopback testing done here! 	*/

			    unitspolled++;
			    cnt++;
			    if (cnt == MAXCFGCK)
			    {
				break;
			    }
			}
		    }
		}

		/* see if anyone on the line has been responding. */
		/* if not, try to get partner to become slave.    */
		/* Check for no one responding */

		if (!any_responding (line) && (!pryvate[line]))
		{
		    /* See if it is time to let partner try */

		    this_time = tickGet ();
		    if ((this_time - last_time) > (3 * SLWWINDOW))
		    {   /* #ticks for slave to try */

			/* Switch States to let partner know it's its time to try */
			resPutLocalState (prestable [line], RES_SLVWANT);
			lstate = RES_SLVWANT;
			if (modrestrace [line])
			{
			    logMsg ("10] Became RES_SLVWANT  P= %x A= %d \n", pstate, activity);
			}
			/* Give partner access to the hardware */
			clear_rts (line);

			/* Remember the time of this try */
			last_time = tickGet ();
			lastactivity [line] = line_activity [line] = FALSE;
			for (i = 0; (i < SLWWINDOW); i += 10)
			{       		/* SLWINDOW is the # of ticks for slave to try */
			    taskDelay (10);
		    				/* If partner picked up let it have control */
			    activity = check_line_activity (line, 0);  /* check immediately */
			    resGetStates (prestable [line], &lstate, &pstate);
			    if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT) || (activity))
			    {
				resClearCaAnswer (prestable [line]);
				resPutLocalState (prestable [line], RES_SLV);
				lstate = RES_SLV;
				if (modrestrace [line])
				{
				    logMsg ("11] Became RES_SLV  P= %x A= %d \n",
						pstate, activity);
				}
				taskDelay(60);
				taskDelay(60);
				taskDelay(60);
				break;
			    }
			}
			/* Go Back to being the Master and take control of the hardware */
			if (lstate != RES_SLV)
			{
			    resSetCaAnswer (prestable [line]);
			    resPutLocalState (prestable [line], RES_MSTR);
			    lstate = RES_MSTR;
			    if (modrestrace [line])
			    {
			    	logMsg ("12] Became RES_MSTR  P= %x A= %d \n", pstate, activity);
			    }
			}
		    }
		}

		/* take units off hold status.
			the unit's data are not to be held off since now master. */

		for (unit = 0; (unit < NUMUNITS); unit++ )
		{
		    punit = units [line] [unit];
		    if (punit)			/* verify pointer validity prior to use */
		    {        
			punit->holdflag = FALSE;
		    }
		}
		if (unitspolled == 0)
		{
		   taskDelay(6);
		}
		else
		{
		    taskDelay(2);
		}
	    }
	} /* end infinite task loop */
}

/**********
*
*	Try to bring a PLC on line
*
**********/

static void on_line (unsigned short line, unsigned short unit)
{
	INF7	       *punit;
	unsigned	short i, j;

	punit = units [line] [unit];
	if (punit)			/* verify pointer prior to use */
	{
	    if (mltrace [line] == 1)
	    {
		logMsg ( "Bring Online PLC %d\n", unit );
	    }
            if (!inf7loopback (ONERETRY + line, unit, punit))
	    {
                return;			/* if no response, get out */
	    }
	    mark_not_valid (punit);	/* a PLC that has just been brought online must have */
                                        /* it's data marked as being not valid. */
	    punit->is_responding = TRUE;
        }
}

/**********
*
*	Mark all unit data elements as not valid
*                                             		
**********/                                             	

static void mark_not_valid (INF7 *punit)
{
	unsigned short		i;

	for (i = 0; i < MAXINF7AI; i++)
	{
	    punit->aival [i].data_valid = FALSE;
	}
	for (i = 0; i < MAXINF7AO; i++)
	{
	    punit->aoval [i].data_valid = FALSE;
	}
	for (i = 0; i < MAXINF7BI; i++)
	{
	    punit->bival [i].data_valid = FALSE;
	}
	for (i = 0; i < MAXINF7BO; i++)
	{
	    punit->boval [i].data_valid = FALSE;
	}
}

/**********
*
*	check_outputs
*
*	check each unit to see if ther are any outputs to send
*
**********/

static	void check_outputs (unsigned short line)
{
	unsigned short	i;
	unsigned short	j;
	INF7		*punit;

	for (i = 0; i < NUMUNITS; i++)
	{
	    punit = units [line] [i];
	    if (punit == 0)
	    {
		continue;
	    }
	    if (punit->aoval [AITOTAL].output_ready == TRUE)
	    {
		writerawtotal (line, i, punit, punit->aoval [AITOTAL].output_val);
	    }
	}
}

/**********
*
*	Scan a Infinity 7 Unit - get inputs as required
*                                                 
**********/

static	void scan_unit (INF7 *punit, unsigned short line, unsigned short unit)
{
	unsigned short  i, temp, cmd;

	if (mltrace [line] == 2)
	{
            logMsg ( "Scanning Infinity 7 Unit %d.\n", unit);
	}

	punit->tscanpass++;

	if (readmode (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readdisplaymode (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readblocka (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readblockb (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readblockc (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readblockd (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readmeastime (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readrawbatchcnt (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readrawbatchno (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readrawtotal (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	if (readrawtime (line, unit, punit) == FALSE)
	{
	    return;
	}
	if (send_outputs [line] == TRUE)
	{
	    send_outputs [line] = FALSE;
	    check_outputs (line);
	}
	return;
}

/**********
*                       
*	inf7_aidriver
*
*	This routines main purpose is to return the analog input value for the
*	specified unit. A (-1) will be returned if there is no valid data ready.
*
**********/

int inf7_aidriver (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal, float *pvalue,
			unsigned short conversion, unsigned char *resname)
{
	INF7         *punit;
	unsigned short   blkix;
	unsigned short  *pdata;

	if (link > INF7LINES)
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg("AIDRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (adapter > NUMUNITS)
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg("AIDRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if ((card != INF7AI) && (card != INF7PSEUDO))
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg("AIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}

	punit = units [link] [adapter];
	if (punit == NULL)
	{
	    if (!allocate_unit (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    punit = units [link] [adapter];
	}
	if (card == INF7PSEUDO)
	{
	    switch (signal)
	    {
	    case 0: /* Were the Unit values requested */
		*pvalue = punit->was_requested;
		break;

	    case 1: /* Is the Unit responding */
		*pvalue = punit->is_responding;
		break;

	    case 5: /* Output Messages */
		*pvalue = punit->no_dataout_msgs & 0x7fff;
		break;

	    case 6: /* Input Messages */
		*pvalue = punit->no_datain_msgs & 0x7fff;
		break;

	    case 7: /* No Receiver Responses */
		*pvalue = punit->no_rxnoresp & 0x7fff;
		break;

	    case 8: /* No Transmitter Response */
		*pvalue = punit->no_txnoresp & 0x7fff;
		break;

	    case 9: /* Data Echo Errors */
		*pvalue = punit->no_dataerr & 0x7fff;
		break;

	    case 10: /* Message Retries */
 		*pvalue = punit->no_retries & 0x7fff;
		break;

	    case 11: /* RX Hardware errors */
		*pvalue = punit->no_rxhwerrs & 0x7fff;
		break;

	    case 12: /* TX Hardware errors */
		*pvalue = punit->no_txhwerrs & 0x7fff;
		break;

	    case 13: /* CRC errors */
		*pvalue = punit->no_crcerrs & 0x7fff;
		break;

	    case 14: /* Offline transitions */
		*pvalue = punit->no_offlines & 0x7fff;
		break;

	    case 15: /* reset statistics */
		*pvalue = punit->cmd_reset_stat;
		break;

	    case 16: /* Force the Unit offline */
 		*pvalue = punit->cmd_force_offline;
		break;

	   default:
		if (trace[link] & CONFIGURATIONTRACE)
		{
		    logMsg("AIDRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            pointname, link, adapter, card, signal);
		}
		return (-1);
	    }
	    return (0);
	}

	if (signal > MAXINF7AI)
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg("AIDRIVER: Illegal signal(0):[%s] L=%d, A=%d, C=%d, S=%d : max=%d\n",
			pointname, link, adapter, card, signal, MAXINF7AI);
	    }
	    return (-1);
	}
	if (resname != 0)
	{ 
	    register_resource (link, punit, resname, pointname);
	}

	punit->was_requested = TRUE;

	/************************************************************/
	/* verify conversion type - MUST BE NO CONVERSION or LINEAR */
	/************************************************************/

	if ((conversion != NOCONV) &&
	    (conversion != LINEAR))
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
                logMsg("AIDRIVER: Illegal Conversion Type:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg("                                 :[%s] conversion type = %d\n",
			pointname, conversion);
	    }
	    return (-1);
	}

	punit->aival [signal].was_requested = TRUE;
	if (!resresponse[link])
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AIDRIVER: Driver Offline(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->holdflag)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
 		logMsg("AIDRIVER: On Hold(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-2);
	}
	if (!punit->is_responding)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AIDRIVER: Not Responding(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (!punit->aival [signal].data_valid)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AIDRIVER: Data Not Valid(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	*pvalue = punit->aival [signal].last_read;

	if (mltrace [link] == 8)
	{
	    logMsg("AIDRIVER: Data Returned(1):[%s] L=%d, A=%d, C=%d, S=%d, V=%f\n",
			pointname, link, adapter, card, signal, *pvalue);
	}
        return(0);
}

/**********
*
*	inf7_aodriver
*
*	This routines main purpose is to set requested analog outputs to the
*	value on the requested pseudo points.
*
**********/

int inf7_aodriver (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal,double dvalue,
			unsigned short value, unsigned short conversion,
			unsigned char *resname)
{
	INF7			*punit;
	unsigned short		blkix;

	if (link > INF7LINES)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg("AODRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
	    }
	    return (-1);
	}
	if (adapter > NUMUNITS)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("AODRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
	    }
	    return (-1);
	}
	if ((card != INF7PSEUDO) && (card != INF7AO))
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg ("AODRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
	    }
	    return (-1);
	}

	punit = units [link] [adapter];
	if (punit == NULL)
	{
	    if (!allocate_unit (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    punit = units [link] [adapter];
	}
	if (card == INF7PSEUDO)
	{
	    switch (signal)
	    {
	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
		{
		    initstats(punit);
		}
		break;

	    case 16: /* Force the PLC online/offline */
		punit->cmd_force_offline = (value != 0);
		if (value != 0)
		{
		    /* only required when asserting cmd_force_offline */
		    punit->is_responding = FALSE;
		}
		break;

	    default:
		if (trace [link] & CONFIGURATIONTRACE)
		{
		    logMsg("AODRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
		}
		return (-1);
	    }
	    return (0);
	}
	switch (signal)
	{
	case AITOTAL:
	    break;

	default:
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg("AODRIVER: Illegal signal(0):[%s] L=%d, A=%d, C=%d, S=%d : max=%d\n",
			pointname, link, adapter, card, signal, MAXINF7AO);
	    }
	    return (-1);
	}
	if (resname != 0)
	{ 
	    register_resource (link, punit, resname, pointname);
	}

	punit->was_requested = TRUE;

	/************************************************************/
	/* verify conversion type - MUST BE NO CONVERSION or LINEAR */
	/************************************************************/

	if ((conversion != NOCONV) &&
	    (conversion != LINEAR))
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
                logMsg("AODRIVER: Illegal Conversion Type:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg("                                 :[%s] conversion type = %d\n",
			pointname, conversion);
	    }
	    return (-1);
	}

	punit->aoval [signal].was_requested = TRUE;
	if (!resresponse[link])
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AODRIVER: Driver Offline(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->holdflag)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
 		logMsg("AODRIVER: On Hold(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-2);
	}
	if (!punit->is_responding)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AODRIVER: Not Responding(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (!punit->aoval [signal].data_valid)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AODRIVER: Data Not Valid(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	punit->aoval [signal].output_val = value;
	punit->aoval [signal].output_ready = TRUE;
	punit->outputs_to_go = TRUE;
	send_outputs [link] = TRUE;
	return (0);
}

/**********
*
*	inf7_aoread
*
*	This routines main purpose is to return the value last value written
*	to the specified point. This is used to initialize analog outputs.
*
**********/

int inf7_aoread (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal, unsigned short *pvalue,
			unsigned short conversion, unsigned char *resname)
{
	INF7			*punit;
	unsigned short		blkix;
	unsigned short		*pdata;

	if (link > INF7LINES)
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg ("AOREAD: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (adapter > NUMUNITS)
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg ("AOREAD: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if ((card != INF7AO) && (card != INF7PSEUDO))
	{
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg ("AOREAD: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}

	punit = units [link] [adapter];
	if (punit == NULL)
	{
	    if (!allocate_unit (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    punit = units [link] [adapter];
	}

	if (card == INF7PSEUDO)
	{
	    switch (signal)
	    {
	    case 15: /* read statistics command value */
		*pvalue = punit->cmd_reset_stat;
		break;

	    case 16: /* Read the current PLC offline command status */
		*pvalue = punit->cmd_force_offline;
		break;

	    default:
		if (trace[link] & CONFIGURATIONTRACE)
		{
		    logMsg("AOREAD: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
				pointname, link, adapter, card, signal);
		}
		return (-1);
	    }
	    return (0);
	}

	switch (signal)
	{
	case AITOTAL:
	    break;

	default:
	    if (trace[link] & CONFIGURATIONTRACE)
	    {
		logMsg("AOREAD: Illegal signal(0):[%s] L=%d, A=%d, C=%d, S=%d : max=%d\n",
			pointname, link, adapter, card, signal, MAXINF7AO);
	    }
	    return (-1);
	}
	if (resname != 0)
	{ 
	    register_resource (link, punit, resname, pointname);
	}

	punit->was_requested = TRUE;

	/************************************************************/
	/* verify conversion type - MUST BE NO CONVERSION or LINEAR */
	/************************************************************/

	if ((conversion != NOCONV) &&
	    (conversion != LINEAR))
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
                logMsg("AOREAD: Illegal Conversion Type:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg("                                 :[%s] conversion type = %d\n",
			pointname, conversion);
	    }
	    return (-1);
	}

	punit->aoval [signal].was_requested = TRUE;
	if (!resresponse[link])
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AOREAD: Driver Offline(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->holdflag)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
 		logMsg("AOREAD: On Hold(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-2);
	}
	if (!punit->is_responding)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AOREAD: Not Responding(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (!punit->aoval [signal].data_valid)
	{
	    if (trace[link] & CALLREJECTTRACE)
	    {
		logMsg("AOREAD: Data Not Valid(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	*pvalue = punit->aoval [signal].last_read;

	if (mltrace [link] == 8)
	{
	    logMsg("AOREAD: Data Returned(1):[%s] L=%d, A=%d, C=%d, S=%d, V=%f\n",
			pointname, link, adapter, card, signal, *pvalue);
	}
	return (0);
}

/**********
*
*	inf7_bidriver
*
*	The main purpose of this routine is to return the request digital input
*	value(s) from the requested unit. This routine will return a (-1) if the
*	requested module is the wrong type or the data is not yet valid.
*
**********/

int inf7_bidriver (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal, unsigned long mask,
			unsigned long *pvalue, unsigned char *resname)
{
	INF7			*punit;
	unsigned short		blkix;
	unsigned long		tvalue;

	if (link > INF7LINES)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BIDRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (adapter > NUMUNITS)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BIDRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if ((card != INF7BI) && (card != INF7PSEUDO))
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}

	punit = units [link] [adapter];
	if (punit == NULL)
	{
	    if (!allocate_unit (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    punit = units [link] [adapter];
	}

	punit->was_requested = TRUE;

	if (card == INF7PSEUDO)
	{
	    switch (signal)
	    {
	    case 0: /* Were the PLC values requested */
		*pvalue = punit->was_requested;
		break;

	    case 1: /* Is the PLC responding */
		*pvalue = punit->is_responding;
		break;

	    case 15: /* reset statistics */
		*pvalue = punit->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = punit->cmd_force_offline;
		break;

	    default:
		if (trace [link] & CONFIGURATIONTRACE)
		{
		    logMsg ("BIDRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
				pointname, link, adapter, card, signal);
		}
		return (-1);
	    }
	    return (0);
	}

	tvalue = 0;

	if (signal > MAXINF7BI)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BIDRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (resname != 0)
	{ 
	    register_resource (link, punit, resname, pointname);
	}

	punit->was_requested = TRUE;

	blkix = signal / 32;

	punit->bival [blkix].was_requested = TRUE;
	if (!resresponse [link])
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->holdflag)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-2);
	}
	if (!punit->is_responding)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (!punit->bival [blkix].data_valid)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	tvalue = punit->bival [blkix].last_read & mask;
	*pvalue = tvalue & mask;
	if (mltrace [link] == 9)
	{
	    logMsg ("BIDRIVER: Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
			pointname, link, adapter, card, signal + 1, pvalue);
	}
	return (0);
}

/**********
*
*	inf7_bodriver
*
*	The main purpose of this routine is to set the request digital output
*	value(s) on the requested pseudo points.
*
**********/

int inf7_bodriver (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal, unsigned long value,
			unsigned long mask, unsigned char *resname)
{
	INF7			*punit;
	unsigned short		blkix;

	if (link > INF7LINES)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BODRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
	    }
	    return (-1);
	}
	if (adapter > NUMUNITS)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BODRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
	    }
	    return (-1);
	}
	if ((card != INF7PSEUDO) && (card != INF7BO))
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BODRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
			pointname, link, adapter, card, signal, value);
	    }
	    return (-1);
	}

	punit = units [link] [adapter];
	if (punit == NULL)
	{
	    if (!allocate_unit (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	     punit = units [link] [adapter];
	}

	if (card == INF7PSEUDO)
	{
	    switch (signal & 0x1f)
	    {
	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
		{
		    initstats (punit);
		}
		break;

	    case 16: /* Force the PLC offline */
		punit->cmd_force_offline = (value != 0);
		if (value != 0)
		{
		    /* only required when asserting cmd_force_offline */
		    punit->is_responding = FALSE;
		}
		break;

	    default:
		if (trace [link] & CONFIGURATIONTRACE)
		{
		    logMsg("BODRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
				pointname, link, adapter, card, signal, value);
		}
		return (-1);
	    }
	    return (0);
	}

	blkix = signal / 32;

	punit->boval [blkix].was_requested = TRUE;
	if (!resresponse [link])
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BODRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->holdflag)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BODRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-2);
	}
	if (!punit->is_responding)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BODRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (!punit->boval [blkix].data_valid)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BODRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->boval [blkix].output_ready)
	{
	    logMsg ("BODRIVER: Output overflow:[%s] L=%d, A-%d, S=%d, V=0x%x\n",
			pointname, link, adapter, card, signal, value);
	}
	else
	{
	    punit->boval [blkix].output_val = value;
	    punit->boval [blkix].output_mask = mask;
	    punit->boval [blkix].output_ready = TRUE;
	    punit->outputs_to_go = TRUE;
	    send_outputs [link] = TRUE;
	}
	if (mltrace [link] == 9)
	{
	    logMsg ("BODRIVER: Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
			pointname, link, adapter, card, signal, value);
	}
	return (0);
}

/**********
*
*	inf7_boread
*
*	This routines main purpose is to return the value last value written
*	to the specified point. This is used to initialize digital outputs.
*
**********/

int inf7_boread (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal,
			unsigned long *pvalue, unsigned long mask,
			unsigned char *resname)

{
	INF7			*punit;
	unsigned short		blkix;
	unsigned long		tvalue;

	if (link > INF7LINES)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BOREAD: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (adapter > NUMUNITS)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg ("BOREAD: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (card != INF7PSEUDO)
	{
	    if (trace [link] & CONFIGURATIONTRACE)
	    {
		logMsg("BOREAD: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	punit = units [link] [adapter];
	if (punit == NULL)
	{
	    if (!allocate_unit (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    punit = units [link] [adapter];
	}

	if (card == INF7PSEUDO)
	{
	    switch (signal)
	    {
	    case 15: /* reset statistics */
		*pvalue = punit->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = punit->cmd_force_offline;
		break;

	    default:
		if (trace [link] & CONFIGURATIONTRACE)
		{
		    logMsg("BOREAD: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
				pointname, link, adapter, card, signal);
		}
		return (-1);
	    }
	    return (0);
	}

	blkix = signal / 32;

	punit->boval [blkix].was_requested = TRUE;
	if (!resresponse [link])
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (punit->holdflag)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-2);
	}
	if (!punit->is_responding)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	if (!punit->boval [blkix].data_valid)
	{
	    if (trace [link] & CALLREJECTTRACE)
	    {
		logMsg ("BOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
			pointname, link, adapter, card, signal);
	    }
	    return (-1);
	}
	tvalue = punit->boval [blkix].last_read & mask;
	*pvalue = tvalue & mask;
	if (mltrace [link] == 9)
	{
	    logMsg ("BOREAD: Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
			pointname, link, adapter, card, signal, pvalue);
	}
	return (0);
}

/**********
*
*	register_resource
*
*	This routine is used to initialize the resource pointers
*	If this is the first time the resource for this line is being
*	referenced the pointer to the resource table entry will be set
*	and the resource name will be saved. If the resource name has
*	already been set, this routine will verify that all other points
*	for this driver are using the same resource name.
*
**********/                                                                                    	

static void  register_resource (unsigned short link, INF7 *punit,
                                unsigned char *resname, char *pointname)
{
	if (resnames [link] [0] == 0) /* This is the first reference */
	{
	    if (resGetByName (resname, &prestable [link]) != RES_OK)
	    {
		logMsg ("Resource not found in table.(Point Name = %s, Resource Name = %s)\n",
			pointname, resname );
	    }
	    else
	    {
		strcpy (resnames [link], resname);
            }
	}
	else
	{
	    if (strcmp (resnames [link], resname) != 0)
	    {
		logMsg ("Resource name[%s] conflict.(Point Name = %s, Resource Name = %s)\n",
			resnames[link], pointname, resname );
	    }
	}
}


/**********
*                                                                              
*	allocate_unit                                                           
*                                                                              
*	The purpose of this routine is to allocate the memory for a single      
*	unit data structure. If not successful, a console error message is      
*	printed and an error flag is returned to the caller.                    
*	If successful, the newly allocated data structures are initialized.     
*                                                                             	
**********/

static int allocate_unit (unsigned short line, unsigned short unit )
{
	units [line] [unit] = (INF7 *) calloc (1, (sizeof (INF7)));
	if (!units [line] [unit])
	{
	    logMsg ("Unable to allocate memory");
	    printf (" for INF7 data structure on line %d, unit %d\n",line ,unit);
	    return (FALSE);
	}
	else
	{
	    logMsg ("Allocated memory for INF7 data structure on line %d, unit %d\n",
			line, unit);
	    init_unit_tab (line, unit);	     /*	initialize the data structures. */
	}
	return (TRUE);
}

/**********
*
*	start_rx
*
*	The purpose of this routine is to initialize a line table
*	for receive and to enable the receiver.
*
**********/

static	void start_rx (unsigned short line, SEMAPHORE *semid, unsigned short *pstatus )
{
	RXLINE	*rxp;

	rxp = rxlinetab [line];
	rxp->rxix = 0;
	rxp->semid = semid;
	rxp->status = pstatus;
	rxp->rxstate = RXBYTE1;
	rxp->rxmsglen = rxp->rxmsglenexpected;
}

/**********
*
*	stop_rx
*
*	The purpose of this routine is to initialize a line table
*	for receive and to disable the receiver.
*
**********/

static	void stop_rx (unsigned short line )
{
	RXLINE	*rxp;

	rxp = rxlinetab [line];
	rxp->rxstate = RXIGNORE;
}

/**********
*
*	rx_int_proc
*
*	The purpose of this routine is to process a receive character interrupt
*	for a given line.
*
**********/

static	void rx_int_proc (unsigned short line )
{
	unsigned char	rxchar;
	unsigned short	rxstat;
	unsigned short	rx_err_mask;
	RXLINE		*rxp;

	read_rx_hw (line, &rxchar, &rxstat, &rx_err_mask);

	/* only execute if no comm error */

	if ((rxstat & rx_err_mask) == 0)
	{
	    /* Something is sending in data. */

	    line_activity [line] = TRUE;
	    linestats [line].RXintcount++;
	}
	else
	{
	    linestats [line].RXbadcount++;
	}

	rxp = rxlinetab [line];

	if (rxp->rxstate == RXIGNORE)
	{
	    return;
	}

	/* no matter what the state, errors cause a logical reset of the message processing */

	if ((rxstat & rx_err_mask) != 0)
	{
					/* and go wait for the good stuff. */ 
	    rxp->rxix = 0;		/* reset the buffer index/counter */
	    return;
	}

	/* Check for leading NULL in the response */

	if (rxchar == 0)
	{
	    return;
	}

	/* store the character in the buffer and bump the index.*/

	rxp->rxbuf [rxp->rxix++] = (rxchar & 0x7f); 

	if ((rxchar & 0x7f) == CR)
	{
	    *(rxp->status) = RXGOOD;
	    rxp->rxstate = RXIGNORE;
	    rxp->rxbuf [rxp->rxix] = 0; 
	    semGive (rxp->semid);
	    return;
	}
}

/**********
*
*	check_line_activity
*
**********/

static int check_line_activity (unsigned short line, unsigned short predelay )
{
	RXLINE	*rxp;
	long	thistime;
	short	status;

	if (predelay != 0)
	{
	    taskDelay (predelay);
	}
	thistime = tickGet ();
	if ((thistime - activity_time [line]) < ACTIVITYTIME)
	{
	    return (lastactivity [line] || (line_activity [line]));
	}
	lastactivity [line] = status = line_activity [line];
	line_activity [line] = FALSE;
	activity_time [line] = thistime;
	rxp = rxlinetab [line];
	rxp->rxstate = RXIGNORE;
	return (status);
}

/**********
*
*	start_tx
*
*	The purpose of this routine is to initialize a line table
*	for send and to enable the transmitter.
*
**********/

static void start_tx (unsigned short line,SEMAPHORE *semid, unsigned short *pstatus)
{
	TXLINE        *txp;

	disable_tx_ints (line);
	txp = txlinetab [line];
	txp->txix = 0;
	txp->semid = semid;
	txp->status = pstatus;
	txp->txdisable = FALSE;
	enable_tx_ints (line);
}


/**********
*
*	stop_tx
*
*	The purpose of this routine is to initialize a line table
*	for receive and to disable the receiver.
*
**********/

static void stop_tx (unsigned short line )
{
	TXLINE        *txp;

	disable_tx_ints (line);
	txp = txlinetab [line];
	txp->txdisable = TRUE;
}

/**********
*
*	tx_int_proc
*
*	The purpose of this routine is to process a transmit character interrupt
*	for a given line.
*
**********/

static void tx_int_proc (unsigned short line )
{
	char		txchar;
	unsigned short	txstat;
	unsigned short	tx_err_mask;
	TXLINE		*txp;

	linestats [line].TXintcount++;

	txp = txlinetab [line];
	if (txp->txdisable)
	{
	    return;
	}
	if (txp->txix == txp->txmsglen)
	{
	    txp->txdisable = TRUE;
	    disable_tx_ints (line);
	    *(txp->status) = TXGOOD;
	    semGive (txp->semid);
	    return;
	}
	else
	{
	    txchar = txp->txbuf [txp->txix++];
	    write_tx_hw (line, txchar, &txstat, &tx_err_mask);
	    if ((txstat & tx_err_mask) != 0 )
	    {
		txp->txdisable = TRUE;
		disable_tx_ints (line);
		*(txp->status) = TXBAD;
		semGive (txp->semid);
		return;
	    }
	}
	if (txp->txix >= MAXTXMSG)
	{
	    txp->txdisable = TRUE;
	    disable_tx_ints (line);
	    *(txp->status) = TXBAD;
	    semGive (txp->semid);
	    return;
	}
}

/**********
*
*	msgxfer
*
*	The purpose of this routine is to transmit a buffer to the
*	analog devices I/O and to await a response. The transmit and receive
*	buffers are in their respective line tables. This routine will retry
*	the message up to a maximum number of times or until a response
*	is received with a proper checksum.
*		returns		XFERGOOD if response received
*				XFERBAD  if no good response is received
*
**********/

static unsigned short  msgxfer (unsigned short fline, INF7 *punit)
{
	unsigned short	retry, txstatus, rxstatus, txlen, rxlen, line, ix, i;
	RXLINE		*rxp;
	TXLINE		*txp;
	unsigned char	csums [2];
	unsigned short	oldrxix;
	unsigned int	tick1, tick2, tick3, tick4;
	unsigned int	delta40, delta01, delta12, delta23, delta34;
	unsigned long	msgstart;
	unsigned long	errorcode;
	unsigned long	commandcode;

	msgstart = tickGet();

	line = (fline & (~ONERETRY));          /* strip off the no retry flag*/

	delta40 = delta01 = delta12 = delta23 = delta34 = 0;

	enable_rx_ints (line);

	rxp = rxlinetab [line];
	txp = txlinetab [line];

	for (retry = ((fline == line) ? 0 : MAXRETRYS - 1);
			(retry < MAXRETRYS); retry++)
	{
	    linestats [line].msgxfercount++;
	    loopmsgs [line]++;

	    /* time since end of last msgxfer */

	    tick [line] [0] = tickGet ();
	    delta40 = (tick [line] [0] - tick [line] [4]);

	    if (intermsgdelay [line] != 0)
	    {
		taskDelay (intermsgdelay [line]);
	    }
	    set_rts (line);

	    if (pretxdelay [line] != 0)
	    {
		taskDelay (pretxdelay [line]);
	    }

	    txstatus = IO_TIMEOUT;
	    rxstatus = IO_TIMEOUT;
	    semTake (semids [line] [TXSEM], NO_WAIT);
	    semTake (semids [line] [RXSEM], NO_WAIT);

	    txlen = txp->txmsglen;

	    if (trace [line] & ALLTRAFFICTRACE)
	    {
		printf ("TX: %s\n", txp->txbuf);
	    }

	    /* time from entering msgxfer to starting transmission. (including delays & diags) */ 

	    tick [line] [1] = tickGet ();
	    delta01 = (tick [line] [1] - tick [line] [0]);

	    if (halfduplex [line] == FALSE)
	    {
		start_rx (line, semids [line] [RXSEM], &rxstatus);
	    }
	    /* this fires off the transmission. */

	    start_tx (line, semids [line] [TXSEM], &txstatus); 

	    if ((semTake (semids [line] [TXSEM], TXTIMEOUT)) == ERROR)
	    {
		txstatus = IO_TIMEOUT;
	    }

	    /* time to transmit message. */

	    tick [line] [2] = tickGet ();
	    delta12 = (tick [line] [2] - tick [line] [1]);

	    stop_tx (line);

	    if (posttxdelay [line] != 0)
	    {
		taskDelay (posttxdelay [line]);
	    }

	    if (halfduplex [line] == TRUE)
	    {
		clear_rts (line);
		start_rx (line, semids [line] [RXSEM], &rxstatus);
	    }

	    /* intermessage wait time. */

	    tick [line] [3] = tickGet ();
	    delta23 = (tick [line] [3] - tick [line] [2]);

	    if (txstatus == TXGOOD)
	    {
		oldrxix = -1;    /* make it pass through timeout loop at least once */
		while ((semTake (semids [line] [RXSEM], (int) (RXTIMEOUT))) == ERROR)
		{
		    if (oldrxix == rxp->rxix)
		    {
			if (modtrace [line])
			{
			    logMsg("RX timeout\n");
			}
			rxstatus = IO_TIMEOUT;
			break;
		    }
		    else
		    {
			if ((modtrace [line]) || (mltrace[line] == 12))
			{
			    logMsg("waiting...");
			}
		    }
                    oldrxix = rxp->rxix;
		}
		stop_rx (line);

		/* time to receive message. */

		tick [line] [4] = tickGet();
		delta34 = (tick [line] [4] - tick [line] [3]);

		rxlen = rxp->rxix;
		if (rxstatus == RXGOOD)
		{
		    if (trace [line] & ALLTRAFFICTRACE)
		    {
			printf("RX: %s\n", rxp->rxbuf);
			printf("DELTA: %3d %3d %3d %3d %3d\n",
				delta40, delta01, delta12, delta23, delta34);
		    }
		    if (rxlen < 2)
		    {
			if (modtrace [line])
			{
			    logMsg("Invalid rxlen... %d\n", rxlen);
			}
		    }
		    else
		    {
			punit->msgs++;
			punit->responsetime += (tickGet() - msgstart);
			disable_rx_ints (line);
			if (rxp->rxbuf [2] == '?')
			{
			    asciihex (&errorcode, &rxp->rxbuf [3], 2);
			    asciihex (&commandcode, &txp->txbuf [4], 2);
			    switch (errorcode)
			    {
			    case 0x43:
				logMsg ("Command Code 0x%X, Error Code 0x43... Command Error\n", commandcode);
				break;

			    case 0x46:
				logMsg ("Command Code 0x%X, Error Code 0x46... Format Error\n", commandcode);
				break;

			    case 0x48:
				logMsg ("Command Code 0x%X, Error Code 0x48... Checksum Error\n", commandcode);
				break;

			    case 0x50:
				logMsg ("Command Code 0x%X, Error Code 0x50... Parity Error\n", commandcode);
				break;

			    case 0x4c:
				logMsg ("Command Code 0x%X, Error Code 0x4C... Calibration/Write Lockout Error\n", commandcode);
				break;

			    case 0x45:
				logMsg ("Command Code 0x%X, Error Code 0x45... EEPROM Write Lockout Error\n, commandcode");
				break;

			    case 0x56:
				logMsg ("Command Code 0x%X, Error Code 0x56... Serial Recognition Error\n", commandcode);
				break;

			    default:
				logMsg ("Command Code 0x%X, Unexpected Error Code 0x%0X\n", commandcode, errorcode);
			    }
			    return (XFERBAD);
			}
			else
			{
			    return (XFERGOOD);
			}
		    }
		}
		else if (rxstatus == IO_TIMEOUT)
		{
		    if (modtrace [line])
		    {
			logMsg ("Receive Timeout... Length: %d\n", rxlen);
		    }
		    if ((trace [line] & ALLTRAFFICTRACE) &&
			((traceadr [line] == txp->txbuf [0]) || (traceadr [line] == 255)) &&
			((tracetype [line] == txp->txbuf [1]) || (tracetype [line] == 255)))
		    {
			printf("DELTA: %3d %3d %3d %3d %3d\n",
				delta40, delta01, delta12, delta23, delta34);
		    }
		    punit->no_rxnoresp++;
		}
		else
		{
		    if (modtrace [line])
		    {
			logMsg ("Receive Error\n");
		    }
		    punit->no_rxhwerrs++;
		}
	    }
	    else if (txstatus == IO_TIMEOUT)
	    {
		if (modtrace [line])
		{
		    logMsg ("Transmit Timeout\n");
		}
		if (trace [line] & ALLTRAFFICTRACE)
		{
		    printf("DELTA: %3d %3d %3d %3d %3d\n",
			delta40, delta01, delta12, delta23, delta34);
		}
		punit->no_txnoresp++;
	    }
	    else
	    {
		if (modtrace [line])
		{
		    logMsg ("Transmit Error\n");
		}
		punit->no_txhwerrs++;
	    }
	    if (retry != MAXRETRYS)
	    {
		punit->no_retries++;
	    }
	}
	if (modtrace [line])
	{
            logMsg ("Xfer Bad\n");
	}
	disable_rx_ints (line);
	return (XFERBAD);
}

/**********
*
*	init_unit_tab
*
*	The purpose of this routine is to initialize the line tables for the
*	analog devices I/O
*
**********/

static void init_unit_tab (unsigned short line, unsigned short unit)
{
	INF7	*punit;

	punit = units [line] [unit];
	if (!punit)
	{
	    return;
	}
	punit->is_responding = FALSE;
	punit->holdflag = FALSE;
	punit->was_requested = FALSE;
	punit->cmd_force_offline = FALSE;

	initunit (punit);
	initstats (punit);
}

/**********
*
* Cold Init a PLC
*
**********/

static void initunit (INF7 *punit)
{
	unsigned short  i;

	for (i = 0; i < MAXINF7AI; i++)
	{
	    punit->aival [i].data_valid = FALSE;
	    punit->aival [i].was_requested = FALSE;
	    punit->aival [i].last_read = 0;
	}
	for (i = 0; i < MAXINF7AO; i++)
	{
	    punit->aoval [i].data_valid = FALSE;
	    punit->aoval [i].was_requested = FALSE;
	    punit->aoval [i].last_read = 0;
	}
	for (i = 0; i < MAXINF7BI; i++)
	{
	    punit->bival [i].data_valid = FALSE;
	    punit->bival [i].was_requested = FALSE;
	    punit->bival [i].last_read = 0;
	}
	for (i = 0; i < MAXINF7BO; i++)
	{
	    punit->boval [i].data_valid = FALSE;
	    punit->boval [i].was_requested = FALSE;
	    punit->boval [i].last_read = 0;
	}
}

/**********
*
* initialize the statistics for an Infinity 7
*
**********/

static    void initstats (INF7 *punit)
{
	punit->no_dataout_msgs = 0;  /* 5  Count Output Msgs              */
	punit->no_datain_msgs  = 0;  /* 6  Count Data Requests            */
	punit->no_rxnoresp     = 0;  /* 7  Count No Responses             */
	punit->no_txnoresp     = 0;  /* 8  Count No Responses             */
	punit->no_dataerr      = 0;  /* 9  Count Message Echo Errors      */
	punit->no_retries      = 0;  /* 10 Count Retries                  */
	punit->no_rxhwerrs     = 0;  /* 11 Count Comm Hardware Errors     */
	punit->no_txhwerrs     = 0;  /* 12 Count Comm Hardware Errors     */
	punit->no_crcerrs      = 0;  /* 13 Count CRC Errors               */
	punit->no_offlines     = 0;  /* 14 Count Off Line Transitions     */
	punit->badoutcnt       = 0;  /* 17 Count failed output commands   */

	punit->tscanpass       = 0;  /* unit scan pass counter            */
	punit->msgs            = 0;  /* good response message counts      */
	punit->responsetime    = 0;  /* total wait time for good messages */
}

/**********
*
* inf7loopback
*
**********/

static unsigned short inf7loopback (unsigned short line, unsigned short unit,
					INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Set the meter response delay to 300 ms */
	/* Since I don't know if the checksum is enabled */
	/* I will send this command without the checksum... */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'W';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = '0';
	txp->txbuf [6] = '0';
	txp->txbuf [7] = '2';
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	msgxfer (line, punit);

	/* Reset the meter to activate the delay */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'Z';
	txp->txbuf [4] = '0';
	txp->txbuf [5] = '4';
	txp->txbuf [6] = CR;
	txp->txbuf [7] = 0;
	txp->txmsglen = 7;

	msgxfer (line, punit);

	/* Delay for 500 ms */

	taskDelay (30);

	/* Set the meter response delay to 300 ms */
	/* And then I will send with the checksum */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'W';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = '0';
	txp->txbuf [6] = '0';
	txp->txbuf [7] = '2';
	makecsum (txp->txbuf, 8);
	txp->txbuf [10] = CR;
	txp->txbuf [11] = 0;
	txp->txmsglen = 11;

	msgxfer (line, punit);

	/* Reset the meter to activate the delay */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'Z';
	txp->txbuf [4] = '0';
	txp->txbuf [5] = '4';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	msgxfer (line, punit);

	/* Delay for 500 ms */

	taskDelay (30);

	/* Format the output message */

	txp->txbuf [0] = '^';
	txp->txbuf [1] = 'A';
	txp->txbuf [2] = 'E';
	hexascii (unit, &txp->txbuf [3]);
	txp->txbuf [5] = CR;
	txp->txbuf [6] = 0;
	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	asciihex (&value, &rxp->rxbuf [0], 8);
	punit->recog_char = (char) ((value & 0xff000000) >> 24);
	punit->ser_addr = (char) ((value & 0x00ff0000) >> 16);
	punit->bival [BIBUSFORMAT].last_read = (char) ((value & 0x0000ff00) >> 8);
	punit->bival [BISERIALCONFIG].last_read = (char) (value & 0x000000ff);

	/* Configure the Bus Format */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'P';
	txp->txbuf [4] = '1';
	txp->txbuf [5] = 'C';
	txp->txbuf [6] = '5';
	txp->txbuf [7] = 'D';
	if (punit->bival [BIBUSFORMAT].last_read & CSUM_REQUIRED)
	{
	    makecsum (txp->txbuf, 8);
	    txp->txbuf [10] = CR;
	    txp->txbuf [11] = 0;
	    txp->txmsglen = 11;
	}
	else
	{
	    txp->txbuf [8] = CR;
	    txp->txbuf [9] = 0;
	    txp->txmsglen = 9;
	}

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->is_responding = FALSE;
	    return (FALSE);
	}

	/* Set L1 CNF to unlocked */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'W';
	txp->txbuf [4] = '0';
	txp->txbuf [5] = '6';
	txp->txbuf [6] = '0';
	txp->txbuf [7] = '0';
	makecsum (txp->txbuf, 8);
	txp->txbuf [10] = CR;
	txp->txbuf [11] = 0;
	txp->txmsglen = 11;

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->is_responding = FALSE;
	    return (FALSE);
	}

	/* Set L2 CNF to unlocked */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'W';
	txp->txbuf [4] = '0';
	txp->txbuf [5] = '7';
	txp->txbuf [6] = '0';
	txp->txbuf [7] = '0';
	makecsum (txp->txbuf, 8);
	txp->txbuf [10] = CR;
	txp->txbuf [11] = 0;
	txp->txmsglen = 11;

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->is_responding = FALSE;
	    return (FALSE);
	}

	/* Set L3 CNF to unlocked */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'W';
	txp->txbuf [4] = '0';
	txp->txbuf [5] = '8';
	txp->txbuf [6] = '0';
	txp->txbuf [7] = '0';
	makecsum (txp->txbuf, 8);
	txp->txbuf [10] = CR;
	txp->txbuf [11] = 0;
	txp->txmsglen = 11;

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	return (TRUE);
}

/**********
*
*	prepresponses
*
*	The purpose of this routine is to mark the PLCs as on hold and the
*	data as not valid until read again from the PLC
*
**********/

static void prepresponses (unsigned short line)
{
	unsigned short	unit;
	INF7		*punit;

	for (unit = 1; (unit <= NUMUNITS); unit++)
	{
	    punit = units [line] [unit];
	    if (punit)	/* verify pointer validity prior to use */
	    {
		punit->holdflag = TRUE;
		mark_not_valid (punit);
	    }
	}
}

/**********
*
*	any_responding
*
*	The purpose of this routine is to see if any units are
*	responding on this line
*
**********/

static int any_responding (unsigned short line)
{
	unsigned short	unit;
	INF7		*punit;

	for (unit = 0; (unit < NUMUNITS); unit++)
	{
	    punit = units [line] [unit];
	    /* verify pointer validity prior to use */
	    if (punit)  
	    {
		/* check cmd_force_offline flag because it is set in an asynchronous task */
		if ((punit->is_responding) && (!punit->cmd_force_offline))
		{
		    return (TRUE);
		}
	    }
	}
	return (FALSE);
}


/**********
*
*	Read Block A
*
**********/

static unsigned short readblocka (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '4';
	txp->txbuf [5] = '0';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	switch (punit->mode)
	{
	case BATCH_MODE:
	    if (asciihex (&value, &rxp->rxbuf [5], 6) == FALSE) /* Total Scale */
	    {
		punit->aival [AITOTALSCALE].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AITOTALSCALE].last_read = (value & 0xfffff);
		punit->aival [AITOTALSCALE].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [11], 6) == FALSE) /* Total Offset */
	    {
		punit->aival [AITOTALOFFSET].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AITOTALOFFSET].last_read = (value & 0xfffff);
		punit->aival [AITOTALOFFSET].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [17], 2) == FALSE) /* Total Decimal Point */
	    {
		punit->aival [AITOTALDP].data_valid = 0;
	    }
	    else
	    {
		switch (value)
		{
		case 0:
		    punit->aival [AITOTALDP].last_read = 1.0;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 1:
		    punit->aival [AITOTALDP].last_read = 1.0;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 2:
		    punit->aival [AITOTALDP].last_read = 0.1;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 3:
		    punit->aival [AITOTALDP].last_read = 0.01;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 4:
		    punit->aival [AITOTALDP].last_read = 0.001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 5:
		    punit->aival [AITOTALDP].last_read = 0.0001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 6:
		    punit->aival [AITOTALDP].last_read = 0.00001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 7:
		    punit->aival [AITOTALDP].last_read = 0.000001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		default:
		    punit->aival [AITOTALDP].last_read = 0;
		    punit->aival [AITOTALDP].data_valid = 0;
		}
	    }
	    if (asciihex (&value, &rxp->rxbuf [19], 6) == FALSE) /* Batch Scale */
	    {
		punit->aival [AIBATCHSCALE].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIBATCHSCALE].last_read = value;
		punit->aival [AIBATCHSCALE].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [25], 6) == FALSE) /* Batch Load */
	    {
		punit->aival [AIBATCHLOAD].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIBATCHLOAD].last_read = value;
		punit->aival [AIBATCHLOAD].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [31], 2) == FALSE) /* Batch Decimal Point */
	    {
		punit->aival [AIBATCHDP].data_valid = 0;
	    }
	    else
	    {
		switch (value)
		{
		case 0:
		    punit->aival [AIBATCHDP].last_read = 0.0;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 1:
		    punit->aival [AIBATCHDP].last_read = 0.0;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 2:
		    punit->aival [AIBATCHDP].last_read = 0.1;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 3:
		    punit->aival [AIBATCHDP].last_read = 0.01;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 4:
		    punit->aival [AIBATCHDP].last_read = 0.001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 5:
		    punit->aival [AIBATCHDP].last_read = 0.0001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 6:
		    punit->aival [AIBATCHDP].last_read = 0.00001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 7:
		    punit->aival [AIBATCHDP].last_read = 0.000001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		default:
		    punit->aival [AIBATCHDP].last_read = 0;
		    punit->aival [AIBATCHDP].data_valid = 0;
		}
	    }
	    if (asciihex (&value, &rxp->rxbuf [33], 2) == FALSE) /* Configuration 4 */
	    {
		punit->bival [BICINFIG4].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG4].last_read = value;
	        punit->bival [BICINFIG4].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [35], 6) == FALSE) /* Configuration 3 */
	    {
		punit->bival [BICINFIG3].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG3].last_read = value;
	        punit->bival [BICINFIG3].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [41], 2) == FALSE) /* Configuration 2 */
	    {
		punit->bival [BICINFIG2].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG2].last_read = value;
	        punit->bival [BICINFIG2].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [43], 2) == FALSE) /* Configuration 1 */
	    {
		punit->bival [BICINFIG1].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG1].last_read = value;
	        punit->bival [BICINFIG1].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [45], 2) == FALSE) /* Lockout 3 */
	    {
		punit->bival [BILOCKOUT3].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BILOCKOUT3].last_read = value;
	        punit->bival [BILOCKOUT3].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [47], 2) == FALSE) /* Lockout 2 */
	    {
		punit->bival [BILOCKOUT2].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BILOCKOUT2].last_read = value;
	        punit->bival [BILOCKOUT2].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [49], 2) == FALSE) /* Lockout 1 */
	    {
		punit->bival [BILOCKOUT1].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BILOCKOUT1].last_read = value;
	        punit->bival [BILOCKOUT1].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [51], 6) == FALSE) /* Setpoint 5 value */
	    {
		punit->aival [AISETPOINT5].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT5].last_read = value;
	        punit->aival [AISETPOINT5].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [57], 6) == FALSE) /* Setpoint 4 value */
	    {
		punit->aival [AISETPOINT4].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT4].last_read = value;
	        punit->aival [AISETPOINT4].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [63], 6) == FALSE) /* Setpoint 3 value */
	    {
		punit->aival [AISETPOINT3].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT3].last_read = value;
	        punit->aival [AISETPOINT3].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [69], 6) == FALSE) /* Setpoint 2 value */
	    {
		punit->aival [AISETPOINT2].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT2].last_read = value;
	        punit->aival [AISETPOINT2].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [75], 6) == FALSE) /* Setpoint 1 value */
	    {
		punit->aival [AISETPOINT1].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT1].last_read = value;
	        punit->aival [AISETPOINT1].data_valid = 1;
	    }
	    break;

	case RATE_MODE:
	case SQRT_MODE:
	    if (asciihex (&value, &rxp->rxbuf [5], 6) == FALSE) /* Total Scale */
	    {
		punit->aival [AITOTALSCALE].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AITOTALSCALE].last_read = (value & 0xfffff);
		punit->aival [AITOTALSCALE].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [11], 6) == FALSE) /* Total Offset */
	    {
		punit->aival [AITOTALOFFSET].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AITOTALOFFSET].last_read = (value & 0xfffff);
		punit->aival [AITOTALOFFSET].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [17], 2) == FALSE) /* Total Decimal Point */
	    {
		punit->aival [AITOTALDP].data_valid = 0;
	    }
	    else
	    {
		switch (value)
		{
		case 0:
		    punit->aival [AITOTALDP].last_read = 1.0;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 1:
		    punit->aival [AITOTALDP].last_read = 1.0;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 2:
		    punit->aival [AITOTALDP].last_read = 0.1;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 3:
		    punit->aival [AITOTALDP].last_read = 0.01;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 4:
		    punit->aival [AITOTALDP].last_read = 0.001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 5:
		    punit->aival [AITOTALDP].last_read = 0.0001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 6:
		    punit->aival [AITOTALDP].last_read = 0.00001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		case 7:
		    punit->aival [AITOTALDP].last_read = 0.000001;
		    punit->aival [AITOTALDP].data_valid = 1;
		    break;

		default:
		    punit->aival [AITOTALDP].last_read = 0;
		    punit->aival [AITOTALDP].data_valid = 0;
		}
	    }
	    if (asciihex (&value, &rxp->rxbuf [19], 6) == FALSE) /* Rate Scale */
	    {
		punit->aival [AIBATCHSCALE].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIBATCHSCALE].last_read = (value & 0xfffff);
		punit->aival [AIBATCHSCALE].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [25], 6) == FALSE) /* Rate Offset */
	    {
		punit->aival [AIBATCHLOAD].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIBATCHLOAD].last_read = (value & 0xfffff);
		punit->aival [AIBATCHLOAD].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [31], 2) == FALSE) /* Rate Decimal Point */
	    {
		punit->aival [AIBATCHDP].data_valid = 0;
	    }
	    else
	    {
		switch (value)
		{
		case 0:
		    punit->aival [AIBATCHDP].last_read = 1.0;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 1:
		    punit->aival [AIBATCHDP].last_read = 1.0;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 2:
		    punit->aival [AIBATCHDP].last_read = 0.1;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 3:
		    punit->aival [AIBATCHDP].last_read = 0.01;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 4:
		    punit->aival [AIBATCHDP].last_read = 0.001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 5:
		    punit->aival [AIBATCHDP].last_read = 0.0001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 6:
		    punit->aival [AIBATCHDP].last_read = 0.00001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		case 7:
		    punit->aival [AIBATCHDP].last_read = 0.000001;
		    punit->aival [AIBATCHDP].data_valid = 1;
		    break;

		default:
		    punit->aival [AIBATCHDP].last_read = 0;
		    punit->aival [AIBATCHDP].data_valid = 0;
		}
	    }
	    if (asciihex (&value, &rxp->rxbuf [33], 2) == FALSE) /* Avg Configuration */
	    {
		punit->aival [AIAVGCONFIG].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIAVGCONFIG].last_read = value;
		punit->aival [AIAVGCONFIG].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [35], 2) == FALSE) /* Configuration 4 */
	    {
		punit->bival [BICINFIG4].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG4].last_read = value;
		punit->bival [BICINFIG4].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [37], 6) == FALSE) /* Configuration 3 */
	    {
		punit->bival [BICINFIG3].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG3].last_read = value;
		punit->bival [BICINFIG3].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [43], 2) == FALSE) /* Configuration 2 */
	    {
		punit->bival [BICINFIG2].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG2].last_read = value;
		punit->bival [BICINFIG2].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [45], 2) == FALSE) /* Configuration 1 */
	    {
		punit->bival [BICINFIG1].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BICINFIG1].last_read = value;
		punit->bival [BICINFIG1].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [47], 2) == FALSE) /* Lockout 3 */
	    {
		punit->bival [BILOCKOUT3].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BILOCKOUT3].last_read = value;
		punit->bival [BILOCKOUT3].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [49], 2) == FALSE) /* Lockout 2 */
	    {
		punit->bival [BILOCKOUT2].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BILOCKOUT2].last_read = value;
		punit->bival [BILOCKOUT2].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [51], 2) == FALSE) /* Lockout 1 */
	    {
		punit->bival [BILOCKOUT1].data_valid = 0;
	    }
	    else
	    {
		punit->bival [BILOCKOUT1].last_read = value;
		punit->bival [BILOCKOUT1].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [53], 6) == FALSE) /* Setpoint 5 value */
	    {
		punit->aival [AISETPOINT5].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT5].last_read = (value & 0xfffff);
		punit->aival [AISETPOINT5].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [59], 6) == FALSE) /* Setpoint 4 value */
	    {
		punit->aival [AISETPOINT4].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT4].last_read = (value & 0xfffff);
		punit->aival [AISETPOINT4].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [65], 6) == FALSE) /* Setpoint 3 value */
	    {
		punit->aival [AISETPOINT3].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT3].last_read = (value & 0xfffff);
		punit->aival [AISETPOINT3].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [71], 6) == FALSE) /* Setpoint 2 value */
	    {
		punit->aival [AISETPOINT2].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT2].last_read = (value & 0xfffff);
		punit->aival [AISETPOINT2].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [77], 6) == FALSE) /* Setpoint 1 value */
	    {
		punit->aival [AISETPOINT1].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AISETPOINT1].last_read = (value & 0xfffff);
		punit->aival [AISETPOINT1].data_valid = 1;
	    }
	    break;
	}
	return (TRUE);
}

/**********
*
*	Read Block B
*
**********/

static unsigned short readblockb (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;
	unsigned long	temp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'R';
	txp->txbuf [4] = '4';
	txp->txbuf [5] = '1';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	if (asciihex (&value, &rxp->rxbuf [5], 6) == FALSE) /* Output Offset */
	{
	    punit->aival [AIOUTOFFSET].data_valid = 0;
	}
	else
	{
	    temp = ((value & 0x700000) >> 20);
	    if (value && 0x800000)
	    {
		value &= 0xfffff;
		value |= 0x80000000;
	    }
	    else
	    {
		value &= 0xfffff;
	    }
	    punit->aival [AIOUTOFFSET].last_read = (value * (10 ^ temp));
	    punit->aival [AIOUTOFFSET].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [11], 6) == FALSE) /* Output Scale */
	{
	    punit->aival [AIOUTSCALE].data_valid = 0;
	}
	else
	{
	    temp = ((value & 0xf00000) >> 20);
	    if (value && 0x80000)
	    {
		value &= 0x7fffff;
		value |= 0x80000000;
	    }
	    else
	    {
		value &= 0x7fffff;
	    }
	    punit->aival [AIOUTSCALE].last_read = (value * (10 ^ temp));
	    punit->aival [AIOUTSCALE].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [17], 2) == FALSE) /* Serial Delay */
	{
	    punit->aival [AISERIALDELAY].data_valid = 0;
	}
	else
	{
	    punit->aival [AISERIALDELAY].last_read = value;
	    punit->aival [AISERIALDELAY].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [19], 2) == FALSE) /* Recognition Char */
	{
	    punit->aival [AIRECOGCHAR].data_valid = 0;
	}
	else
	{
	    punit->aival [AIRECOGCHAR].last_read = value;
	    punit->aival [AIRECOGCHAR].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [21], 6) == FALSE) /* Input Offset */
	{
	    punit->aival [AIINPUTOFFSET].data_valid = 0;
	}
	else
	{
	    temp = ((value & 0x700000) >> 20);
	    value &= 0xfffff;
	    punit->aival [AIINPUTOFFSET].last_read = (value ^ temp);
	    punit->aival [AIINPUTOFFSET].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [27], 6) == FALSE) /* Input Scale */
	{
	    punit->aival [AIINPUTSCALE].data_valid = 0;
	}
	else
	{
	    punit->aival [AIINPUTSCALE].last_read = value;
	    punit->aival [AIINPUTSCALE].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [33], 2) == FALSE) /* Scale Operator */
	{
	    punit->aival [AISCALEOPERATOR].data_valid = 0;
	}
	else
	{
	    temp = ((value & 0x700000) >> 20);
	    value &= 0xfffff;
	    punit->aival [AISCALEOPERATOR].last_read = (value ^ temp);
	    punit->aival [AISCALEOPERATOR].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [35], 2) == FALSE) /* Pulsed Alarm # */
	{
	    punit->aival [AIPULSEDALARMNO].data_valid = 0;
	}
	else
	{
	    punit->aival [AIPULSEDALARMNO].last_read = value;
	    punit->aival [AIPULSEDALARMNO].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [37], 4) == FALSE) /* Serial Time */
	{
	    punit->aival [AISERIALTIME].data_valid = 0;
	}
	else
	{
	    punit->aival [AISERIALTIME].last_read = value;
	    punit->aival [AISERIALTIME].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [41], 2) == FALSE) /* Serial Address */
	{
	    punit->aival [AISERIALADDR].data_valid = 0;
	}
	else
	{
	    punit->aival [AISERIALADDR].last_read = value;
	    punit->aival [AISERIALADDR].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [43], 2) == FALSE) /* Bus Format */
	{
	    punit->bival [BIBUSFORMAT].data_valid = 0;
	}
	else
	{
	    punit->bival [BIBUSFORMAT].last_read = value;
	    punit->bival [BIBUSFORMAT].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [45], 2) == FALSE) /* Data Format */
	{
	    punit->bival [BIDATAFORMAT].data_valid = 0;
	}
	else
	{
	    punit->bival [BIDATAFORMAT].last_read = value;
	    punit->bival [BIDATAFORMAT].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [47], 2) == FALSE) /* Serial Configuration */
	{
	    punit->bival [BISERIALCONFIG].data_valid = 0;
	}
	else
	{
	    punit->bival [BISERIALCONFIG].last_read = value;
	    punit->bival [BISERIALCONFIG].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [49], 2) == FALSE) /* Baud Rate */
	{
	    punit->bival [BIBAUDRATE].data_valid = 0;
	}
	else
	{
	    punit->bival [BIBAUDRATE].last_read = value;
	    punit->bival [BIBAUDRATE].data_valid = 1;
	}
	switch (punit->mode)
	{

	case RATE_MODE:
	    if (asciihex (&value, &rxp->rxbuf [51], 2) == FALSE) /* Gate Time */
	    {
		punit->aival [AIGATETIME].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIGATETIME].last_read = value;
		punit->aival [AIGATETIME].data_valid = 1;
	    }
	    if (asciihex (&value, &rxp->rxbuf [53], 4) == FALSE) /* Pulsed Alarm Time */
	    {
		punit->aival [AIPULSEDALARMTM].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIPULSEDALARMTM].last_read = value;
		punit->aival [AIPULSEDALARMTM].data_valid = 1;
	    }
	    break;

	case BATCH_MODE:
	case SQRT_MODE:
	    if (asciihex (&value, &rxp->rxbuf [51], 4) == FALSE) /* Pulsed Alarm Time */
	    {
		punit->aival [AIPULSEDALARMTM].data_valid = 0;
	    }
	    else
	    {
		punit->aival [AIPULSEDALARMTM].last_read = value;
		punit->aival [AIPULSEDALARMTM].data_valid = 1;
	    }
	    break;
	}
	return (TRUE);
}

/**********
*
*	Read Block C
*
**********/

static unsigned short readblockc (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	if (punit->mode != BATCH_MODE)
	{
	    return (TRUE);
	}
	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '4';
	txp->txbuf [5] = '2';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	if (asciihex (&value, &rxp->rxbuf [5], 6) == FALSE) /* Number of Batches */
	{
	    punit->aival [AINUMBERBATCHES].data_valid = 0;
	}
	else
	{
	    punit->aival [AINUMBERBATCHES].last_read = value;
	    punit->aival [AINUMBERBATCHES].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [11], 8) == FALSE) /* Batche Value */
	{
	    punit->aival [AIBATCHVALUE].data_valid = 0;
	}
	else
	{
	    punit->aival [AIBATCHVALUE].last_read = value;
	    punit->aival [AIBATCHVALUE].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [19], 8) == FALSE) /* Total */
	{
	    punit->aival [AITOTAL].data_valid = 0;
	}
	else
	{
	    punit->aival [AITOTAL].last_read = value;
	    punit->aival [AITOTAL].data_valid = 1;
	}
	return (TRUE);
}

/**********
*
*	Read Block D
*
**********/

static unsigned short readblockd (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'R';
	txp->txbuf [4] = '4';
	txp->txbuf [5] = '3';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	if (asciihex (&value, &rxp->rxbuf [5], 6) == FALSE) /* Unit of Measure 2 */
	{
	    punit->bival [BIUNITMEASURE2].data_valid = 0;
	}
	else
	{
	    punit->bival [BIUNITMEASURE2].last_read = value;
	    punit->bival [BIUNITMEASURE2].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [11], 6) == FALSE) /* Unit of Measure 1 */
	{
	    punit->bival [BIUNITMEASURE1].data_valid = 0;
	}
	else
	{
	    punit->bival [BIUNITMEASURE1].last_read = value;
	    punit->bival [BIUNITMEASURE1].data_valid = 1;
	}
	return (TRUE);
}

/**********
*
*	Read the instrument Mode
*
**********/
static unsigned short readmode (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'R';
	txp->txbuf [4] = '0';
	txp->txbuf [5] = '9';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->is_responding = FALSE;
	    return (FALSE);
	}

	asciihex (&value, &rxp->rxbuf [5], 2);

	switch (value)
	{
	case 0:
	    punit->mode = BATCH_MODE;
	    break;

	case 1:
	    punit->mode = RATE_MODE;
	    break;

	case 2:
	    punit->mode = SQRT_MODE;
	    break;
	}
	return (TRUE);
}

/**********
*
* 	Read Display Mode
*
**********/

static unsigned short readdisplaymode (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '3';
	txp->txbuf [5] = '3';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}

	asciihex (&value, &rxp->rxbuf [5], 2);

	switch (punit->mode)
	{
	case BATCH_MODE:
	    switch (value & DISPMODEMASK)
	    {
	    case 1:
		punit->disp_mode = DISP_BATCH;
		break;

	    case 2:
		punit->disp_mode = DISP_NO_BATCHES;
		break;

	    case 3:
		punit->disp_mode = DISP_TOTAL;
		break;

	    case 4:
		punit->disp_mode = DISP_TIME;
		break;

	    default:
		punit->disp_mode = DISPMODEFAULT;
	    }
	    break;

	case RATE_MODE:
	case SQRT_MODE:
	    switch (value & DISPMODEMASK)
	    {
	    case 1:
		punit->disp_mode = DISP_RATE;
		break;

	    case 2:
		punit->disp_mode = DISP_AVGRT;
		break;

	    case 3:
		punit->disp_mode = DISP_TOTAL;
		break;

	    case 4:
		punit->disp_mode = DISP_TIME;
		break;

	    default:
		punit->disp_mode = DISPMODEFAULT;
	    }
	    break;
	}
	switch (value & RUNMODEMASK)
	{
	case 0:
	    punit->run_mode = RUN_MODE;
	    break;

	case 8:
	    punit->run_mode = STOP_MODE;
	    break;
	}
	return (TRUE);
}

/**********
*
* 	Read Measurement Time (RATE & SQRT mode only)
*
**********/

static unsigned short readmeastime (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	if (punit->mode == BATCH_MODE)
	{
	    return (TRUE);
	}
	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '1';
	txp->txbuf [5] = '8';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	asciihex (&value, &rxp->rxbuf [5], 2); /* Measurement Time */
	return (TRUE);
}

/**********
*
* 	Read Raw Batch Count
*
**********/

static unsigned short readrawbatchcnt (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	if (punit->mode != BATCH_MODE)
	{
	    return (TRUE);
	}
	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = 'A';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	asciihex (&value, &rxp->rxbuf [5], 8); /* Batch Count */
	return (TRUE);
}

/**********
*
* 	Read Raw Batch Number
*
**********/

static unsigned short readrawbatchno (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	if (punit->mode != BATCH_MODE)
	{
	    return (TRUE);
	}
	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = 'B';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	asciihex (&value, &rxp->rxbuf [5], 8); /* Batch Number */
	return (TRUE);
}

/**********
*
* 	Read Raw Total
*
**********/

static unsigned short readrawtotal (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = '9';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	if (asciihex (&value, &rxp->rxbuf [5], 8) == FALSE) /* Total */
	{
	    punit->aival [AITOTAL].data_valid = 0;
	    punit->aoval [AITOTAL].data_valid = 0;
	}
	else
	{
	    punit->aival [AITOTAL].last_read = value;
	    punit->aival [AITOTAL].data_valid = 1;
	    punit->aoval [AITOTAL].last_read = value;
	    punit->aoval [AITOTAL].data_valid = 1;
	}
	return (TRUE);
}

/**********
*
* 	Read Raw Time
*
**********/

static unsigned short readrawtime (unsigned short line, unsigned short unit, INF7 *punit)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	value;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	punit->no_datain_msgs++;

	/* Build the read block message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'G';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = '8';
	makecsum (txp->txbuf, 6);
	txp->txbuf [8] = CR;
	txp->txbuf [9] = 0;
	txp->txmsglen = 9;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	if (asciihex (&value, &rxp->rxbuf [5], 2) == FALSE) /* Time */
	{
	    punit->aival [AITIMEHOUR].data_valid = 0;
	}
	else
	{
	    punit->aival [AITIMEHOUR].last_read = value;
	    punit->aival [AITIMEHOUR].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [7], 2) == FALSE) /* Time */
	{
	    punit->aival [AITIMEMIN].data_valid = 0;
	}
	else
	{
	    punit->aival [AITIMEMIN].last_read = value;
	    punit->aival [AITIMEMIN].data_valid = 1;
	}
	if (asciihex (&value, &rxp->rxbuf [9], 2) == FALSE) /* Time */
	{
	    punit->aival [AITIMESEC].data_valid = 0;
	}
	else
	{
	    punit->aival [AITIMESEC].last_read = value;
	    punit->aival [AITIMESEC].data_valid = 1;
	}
	return (TRUE);
}

/**********
*
* 	Write Raw Total
*
**********/

static unsigned short writerawtotal (unsigned short line, unsigned short unit, INF7 *punit,
					unsigned long value)
{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned long	tvalue;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data write messages */

	punit->no_dataout_msgs++;

	/* Build the write Raw Total message */

	txp->txbuf [0] = '*';
	hexascii (unit, &txp->txbuf [1]);
	txp->txbuf [3] = 'P';
	txp->txbuf [4] = '2';
	txp->txbuf [5] = '9';
	tvalue = ((value & 0xff000000) >> 24);
	hexascii (tvalue, &txp->txbuf [6]);
	tvalue = ((value & 0xff0000) >> 16);
	hexascii (tvalue, &txp->txbuf [8]);
	tvalue = ((value & 0xff00) >> 8);
	hexascii (tvalue, &txp->txbuf [10]);
	tvalue = (value & 0xff);
	hexascii (tvalue, &txp->txbuf [12]);
	makecsum (txp->txbuf, 14);
	txp->txbuf [16] = CR;
	txp->txbuf [17] = 0;
	txp->txmsglen = 17;

	/* Send out the message */

	if (msgxfer (line, punit) == XFERBAD)
	{
	    punit->no_offlines++;
	    punit->holdflag = TRUE;
	    punit->is_responding = FALSE;
	    return (FALSE);
	}
	punit->aoval [AITOTAL].output_ready = FALSE;
	return (TRUE);
}

/**********
*      inf7dumpmsgcnts
*
*      The purpose of this routine is to dump the line level statistics for this driver. 
*
**********/

void inf7dumpmsgcnts (unsigned short line)
{
    unsigned long    temp, t, x, y, e, z;

    temp = tickGet ();
    t = (temp - linestats [line].startingtick);

    x = linestats [line].TXintcount;
    y = linestats [line].RXintcount;
    e = linestats [line].RXbadcount;
    z = linestats [line].msgxfercount;

    if (z == 0)
    {
        printf ("TX Interrupt Count    = %10d\n", x);
        printf ("RX Interrupt Count    = %10d\n", y);
        printf ("BadRX Intrpt Count    = %10d\n", e);
        printf (" ******  No Messages Transferred ****** \n");
        return;
    }

    if (t == 0)
    {
        printf ("TX Interrupt Count    = %10d\n", x);
        printf ("RX Interrupt Count    = %10d\n", y);
        printf ("BadRX Intrpt Count    = %10d\n", e);
        printf (" ******  Timer Failure ****** \n");
        return;
    }

     printf ("\n");
     printf ("Measurement Interval: clock ticks = %d, seconds = %d.%1d \n", t, t/60, (t%60)/6);
     printf ("\n");
     printf ("TX Interrupt Count    = %10d   Avg TX cnts/sec        = %d \n", x, (60*x)/t);
     printf ("RX Interrupt Count    = %10d   Avg RX cnts/sec        = %d \n", y, (60*y)/t);
     printf ("                                     Avg RX cnts/Msgxfer    = %d.\n", x/z);
     printf ("                                     Avg TX cnts/Msgxfer    = %d.\n", y/z);
     printf ("BadRX Intrpt Count    = %10d\n", e);
     printf ("\n");
     printf ("RX+TX Interrupt Count = %10d   Avg RX+TX cnts/sec     = %d \n",(x+y),(60*(x+y))/t);
     printf ("                                     Avg RX+TX cnts/Msgxfer = %d \n",(x+y)/z);
     printf ("\n");
     printf ("Msgxfer Count         = %10d   Avg Msgxfers/sec       = %d \n", z,(60*z)/t);
     printf ("                                     Avg msec/Msgxfer       = %d \n",(t*1000)/(z*60));
     printf ("\n");
     return;
}

/**********
*
*      inf7clearmsgcnts
*
*      The purpose of this routine is to dump and clear the line level statistics
*      for this driver.
*
**********/

void inf7clearmsgcnts (unsigned short line)
{
    linestats [line].TXintcount = 0;
    linestats [line].RXintcount = 0;
    linestats [line].RXbadcount = 0;
    linestats [line].msgxfercount = 0;

    linestats [line].startingtick = tickGet ();
    printf (" ******  All Counts Cleared  ****** \n", linestats [line].msgxfercount);
    printf ("\n");
        return;
}


/**********
*
*      inf7dumpstats
*
*      The purpose of this routine is to dump the unit level statistics for this driver.
*
**********/

void inf7dumpstats (unsigned short line, unsigned short maddr)
{
    INF7        *punit;
    unsigned long   microsecs;

    punit = units [line] [maddr];
    if (!punit)
    {
        logMsg ("Line / Address Not Defined \n");
        return;
    }

    printf (" Was requested = %d \n",       punit->was_requested);
    printf (" Is responding = %d \n",       punit->is_responding);
    printf (" # Output messages = %d \n",   punit->no_dataout_msgs & 0x7fff);
    printf (" # Data in messages = %d \n",  punit->no_datain_msgs & 0x7fff);
    printf (" # No Rx Response = %d \n",     punit->no_rxnoresp & 0x7fff);
    printf (" # No Tx Responses = %d \n",    punit->no_txnoresp & 0x7fff);
    printf (" # Data Errors = %d \n",        punit->no_dataerr & 0x7fff);
    printf (" # Retries = %d \n",            punit->no_retries & 0x7fff);
    printf (" # Rx Hw Errors = %d \n",       punit->no_rxhwerrs & 0x7fff);
    printf (" # Tx Hw Errors = %d \n",       punit->no_txhwerrs & 0x7fff);
    printf (" # Crc Errors = %d \n",         punit->no_crcerrs & 0x7fff);
    printf (" # Off Lines = %d \n",          punit->no_offlines & 0x7fff);
    printf (" Reset Statistics = %d \n",    punit->cmd_reset_stat);
    printf (" Force Offline = %d \n",       punit->cmd_force_offline);
    printf (" # Failed Outputs = %d \n",     punit->badoutcnt & 0x7fff);

    printf ("\n");
    printf (" # of Messages %d \n",         punit->msgs);
    printf (" # Total response time (ticks) = %lu \n", punit->responsetime);
    if (punit->msgs)
    {
        printf ("   Average response time (ticks) = %d \n", (punit->responsetime/punit->msgs));
        microsecs = (16667*punit->responsetime)/punit->msgs;
        printf ("   Average response time (msec) = %d.%1d \n", microsecs/1000, (microsecs%1000)/100);
    }
    printf (" # of scan passes = %d \n",    punit->tscanpass);
    printf ("\n");

    printf (" Resource Responding = %d \n", resresponse[line]);
    printf (" Hold flag  = %d \n",          punit->holdflag );
        return;
}

/**********
*
*      inf7clearstats
*
*      The purpose of this routine is to clear the unit level statistics for this driver.
*
**********/

void inf7clearstats (unsigned short line, unsigned short maddr)
{
    INF7        *punit;

    punit = units [line] [maddr];
    if (!punit)
    {
        logMsg ("Line / Address Not Defined \n");
        return;
    }
    initstats (punit); /* use existing procedure */
    return;
}


/**********
*
*      inf7table
*
*      The purpose of this routine is twofold:                                 
*         1) dump the unit level statistics for the Infinity driver.
*         2) dump the data for the specified Infinity 7 unit.
*                                                                              
**********/

void inf7table (unsigned short line, unsigned short maddr)
{
	INF7		*punit;
	unsigned short	i;
	unsigned char	*unit_mode [] =
			{
				"BATCH_MODE",
				"RATE_MODE",
				"SQRT_MODE"
			};

	unsigned char	*bdisplay_mode [] =
			{
				"DISP_ILLEGAL",
				"DISP_BATCH",
				"DISP_NO_BATCHES",
				"DISP_TOTAL",
				"DISP_TIME"
			};

	unsigned char	*rsdisplay_mode [] =
			{
				"DISP_ILLEGAL",
				"DISP_RATE",
				"DISP_AVGRT",
				"DISP_TOTAL",
				"DISP_TIME"
			};

	unsigned char	*run_stop_mode [] =
			{
				"RUN_MODE",
				"STOP_MODE"
			};

	punit = units [line] [maddr];
	if (!punit)
	{
	    logMsg("Line/Address Not Defined \n");
	    return;
	}
	printf ("Unit %d Information\n\n", maddr);
	printf ("Unit Mode = %s (%d)\n", unit_mode [punit->mode], punit->mode);
	switch (punit->mode)
	{
	case BATCH_MODE:
	    printf ("Display Mode = %s (%d)\n", bdisplay_mode [punit->disp_mode],
			punit->disp_mode);
	    break;

	case RATE_MODE:
	case SQRT_MODE:
	    printf ("Display Mode = %s (%d)\n", rsdisplay_mode [punit->disp_mode],
			punit->disp_mode);
	    break;
	}
	printf ("Run Mode = %s (%d)\n\n", run_stop_mode [punit->run_mode],
			punit->run_mode);
	printf ("Total Scale = %f, Valid = %d\n",
			punit->aival [AITOTALSCALE].last_read,
			punit->aival [AITOTALSCALE].data_valid);
	printf ("Total Offset = %f, Valid = %d\n",
			punit->aival [AITOTALOFFSET].last_read,
			punit->aival [AITOTALOFFSET].data_valid);
	printf ("Total DP = %f, Valid = %d\n",
			punit->aival [AITOTALDP].last_read,
			punit->aival [AITOTALDP].data_valid);
	printf ("Batch Scale = %f, Valid = %d\n",
			punit->aival [AIBATCHSCALE].last_read,
			punit->aival [AIBATCHSCALE].data_valid);
	printf ("Batch Load = %f, Valid = %d\n",
			punit->aival [AIBATCHLOAD].last_read,
			punit->aival [AIBATCHLOAD].data_valid);
	printf ("Batch DP = %f, Valid = %d\n",
			punit->aival [AIBATCHDP].last_read,
			punit->aival [AIBATCHDP].data_valid);
	printf ("Average Configuration = %f, Valid = %d\n",
			punit->aival [AIAVGCONFIG].last_read,
			punit->aival [AIAVGCONFIG].data_valid);
	printf ("Setpoint 5 = %f, Valid = %d\n",
			punit->aival [AISETPOINT5].last_read,
			punit->aival [AISETPOINT5].data_valid);
	printf ("Setpoint 4 = %f, Valid = %d\n",
			punit->aival [AISETPOINT4].last_read,
			punit->aival [AISETPOINT4].data_valid);
	printf ("Setpoint 3 = %f, Valid = %d\n",
			punit->aival [AISETPOINT3].last_read,
			punit->aival [AISETPOINT3].data_valid);
	printf ("Setpoint 2 = %f, Valid = %d\n",
			punit->aival [AISETPOINT2].last_read,
			punit->aival [AISETPOINT2].data_valid);
	printf ("Setpoint 1 = %f, Valid = %d\n",
			punit->aival [AISETPOINT1].last_read,
			punit->aival [AISETPOINT1].data_valid);
	printf ("Output Offset = %f, Valid = %d\n",
			punit->aival [AIOUTOFFSET].last_read,
			punit->aival [AIOUTOFFSET].data_valid);
	printf ("Output Scale = %f, Valid = %d\n",
			punit->aival [AIOUTSCALE].last_read,
			punit->aival [AIOUTSCALE].data_valid);
	printf ("Serial Delay = %f, Valid = %d\n",
			punit->aival [AISERIALDELAY].last_read,
			punit->aival [AISERIALDELAY].data_valid);
	printf ("Recognition Character = %f, Valid = %d\n",
			punit->aival [AIRECOGCHAR].last_read,
			punit->aival [AIRECOGCHAR].data_valid);
	printf ("Input Offset = %f, Valid = %d\n",
			punit->aival [AIINPUTOFFSET].last_read,
			punit->aival [AIINPUTOFFSET].data_valid);
	printf ("Input Scale = %f, Valid = %d\n",
			punit->aival [AIINPUTSCALE].last_read,
			punit->aival [AIINPUTSCALE].data_valid);
	printf ("Scale Operator = %f, Valid = %d\n",
			punit->aival [AISCALEOPERATOR].last_read,
			punit->aival [AISCALEOPERATOR].data_valid);
	printf ("Pulsed Alarm Number = %f, Valid = %d\n",
			punit->aival [AIPULSEDALARMNO].last_read,
			punit->aival [AIPULSEDALARMNO].data_valid);
	printf ("Serial Time = %f, Valid = %d\n",
			punit->aival [AISERIALTIME].last_read,
			punit->aival [AISERIALTIME].data_valid);
	printf ("Pulsed Alarm Time = %f, Valid = %d\n",
			punit->aival [AIPULSEDALARMTM].last_read,
			punit->aival [AIPULSEDALARMTM].data_valid);
	printf ("Gate Time = %f, Valid = %d\n",
			punit->aival [AIGATETIME].last_read,
			punit->aival [AIGATETIME].data_valid);
	printf ("Number of Batches = %f, Valid = %d\n",
			punit->aival [AINUMBERBATCHES].last_read,
			punit->aival [AINUMBERBATCHES].data_valid);
	printf ("Batch Value = %f, Valid = %d\n",
			punit->aival [AIBATCHVALUE].last_read,
			punit->aival [AIBATCHVALUE].data_valid);
	printf ("Total = %f, Valid = %d\n",
			punit->aival [AITOTAL].last_read,
			punit->aival [AITOTAL].data_valid);
	printf ("Time = %f, Valid = %d\n",
			punit->aival [AITIMEHOUR].last_read,
			punit->aival [AITIMEHOUR].data_valid);
	printf ("Time = %f, Valid = %d\n",
			punit->aival [AITIMEMIN].last_read,
			punit->aival [AITIMEMIN].data_valid);
	printf ("Time = %f, Valid = %d\n",
			punit->aival [AITIMESEC].last_read,
			punit->aival [AITIMESEC].data_valid);
	printf ("Configuration 4 = %x, Valid = %d\n",
			punit->bival [BICINFIG4].last_read,
			punit->bival [BICINFIG4].data_valid);
	printf ("Configuration 3 = %x, Valid = %d\n",
			punit->bival [BICINFIG3].last_read,
			punit->bival [BICINFIG3].data_valid);
	printf ("Configuration 2 = %x, Valid = %d\n",
			punit->bival [BICINFIG2].last_read,
			punit->bival [BICINFIG2].data_valid);
	printf ("Configuration 1 = %x, Valid = %d\n",
			punit->bival [BICINFIG1].last_read,
			punit->bival [BICINFIG1].data_valid);
	printf ("Lockout 3 = %x, Valid = %d\n",
			punit->bival [BILOCKOUT3].last_read,
			punit->bival [BILOCKOUT3].data_valid);
	printf ("Lockout 2 = %x, Valid = %d\n",
			punit->bival [BILOCKOUT2].last_read,
			punit->bival [BILOCKOUT2].data_valid);
	printf ("Lockout 1 = %x, Valid = %d\n",
			punit->bival [BILOCKOUT1].last_read,
			punit->bival [BILOCKOUT1].data_valid);
	printf ("Bus Format = %x, Valid = %d\n",
			punit->bival [BIBUSFORMAT].last_read,
			punit->bival [BIBUSFORMAT].data_valid);
	printf ("Data Format = %x, Valid = %d\n",
			punit->bival [BIDATAFORMAT].last_read,
			punit->bival [BIDATAFORMAT].data_valid);
	printf ("Serial Configuration = %x, Valid = %d\n",
			punit->bival [BISERIALCONFIG].last_read,
			punit->bival [BISERIALCONFIG].data_valid);
	printf ("Baud Rate = %x, Valid = %d\n",
			punit->bival [BIBAUDRATE].last_read,
			punit->bival [BIBAUDRATE].data_valid);
	printf ("Unit of Measure 2 = %x, Valid = %d\n",
			punit->bival [BIUNITMEASURE2].last_read,
			punit->bival [BIUNITMEASURE2].data_valid);
	printf ("Unit of Measure 1 = %x, Valid = %d\n",
			punit->bival [BIUNITMEASURE1].last_read,
			punit->bival [BIUNITMEASURE1].data_valid);
}

/**********
*
*    The purpose of this routine is to display the contents
*    of the static variables contained within this driver.
*
**********/

inf7showlineinfo ()
{
	short		i;
	char		buffer [132];
	char		*resrsp;
	char		*lineact;


	printf ("                      Infinity 7 Line Information Table          \n");
	printf ("\n");
	printf (" Line        Pointer to        Resource      Resource");
	printf ("       Line        Delay Times:       Activity\n");
	printf ("Number      INF7 Table         Name        Response");
	printf ("     Activity      IM  .TX  TX.         Time\n");
	printf ("\n");

	for (i = 0; i < INF7LINES; i++)
	{
	    if (units [i] == NULL)
	    {
		continue;
	    }
	    if (resresponse [i] == 0)
	    {
		resrsp = "FALSE";
	    }
	    else
	    {
		resrsp = "TRUE";
	    }
	    if (line_activity [i] == 0)
	    {
		lineact = "FALSE";
	    }
	    else
	    {
		lineact = "TRUE";
	    }
	    sprintf (buffer,
	"  %2d         %4x   %16s        %5s        %5s        %2d   %2d   %2d        %4d\n",
			i, units [i], &resnames [i],
			&resrsp [0],
			&lineact [0],
			intermsgdelay [i],
			pretxdelay [i],
			posttxdelay [i],
			activity_time [i]);
	    printf ( "%s\n", buffer);
	}
	return (0);
}

/**********
*
*	makecsum (char *buf, int len)
*
*	This routine calculates the checksum for the message and puts int in
*	the buffer.
*
**********/
static void makecsum (unsigned char *buf, int len)
{
	int	i;
	char	csum = 0;

	for (i = 0; i < len; i++)
	{
	    csum += *buf++;
	}
	hexascii (csum , buf);
}


/**********
*                                                                        	
*      hexascii
*
*      The purpose of this routine is to convert the hex byte to 2 ascii
*      characters and to place them into the buffer.
*
**********/
static	void hexascii (unsigned short val, char *buf)

{
	char    hi, lo;

	hi = ((val & 0x0f0) >> 4);
	lo = (val & 0x0f);

	if (hi <= 9)
	{
	    hi+='0';
	}
	else
	{
	    hi += ('A' - 10);
	}
	if (lo <= 9)
	{
	    lo += '0';
	}
	else
	{
	    lo += ('A' - 10);
	}
	buf [0] = hi;
	buf [1] = lo;

}

/**********
*
*      The purpose of this routine is to convert up to 8 ascii         	
*      characters in the buffer to hex and return them in val. It         	
*      will return TRUE if good conversion of FALSE if bad.               	
*
**********/

static	short asciihex (unsigned long *val, char *buf, unsigned short len)
{
	unsigned short	i;

	*val = 0;
	for (i = 0; i < len; i++)
	{
	    if ((buf [i] < '0') || (buf [i] > 'F'))
	    {
		return (FALSE);
	    }
	    if (buf [i] <= '9')
	    {
	    *val += ((buf [i] & 0x0f) << ((len - (i + 1)) * 4));
	    }
	    else
	    {
		if ((buf [i] < 'A') || (buf [i] > 'F'))
		{
		    return (FALSE);
		}
		else
		{
		    *val += ((buf [i] - 'A' + 10) << ((len - (i + 1)) * 4));
		}
	    }
	}
	return (TRUE);
}

/*    End of File:    drvInf7.c */
