/*drvLimitorque.h,v 1.1.1.1 1999/08/03 14:43:13 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
**  File: drvLimitorque.h
**  Edit: 03/03/97      WEP: created based on drvModicon.h                                           
**        09/04/97	WEP: added support for Resource Group Lock function
**
*/

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/*	number of lines of limitorque 	*/
#define		LIMLINES	32
/*	number of limitorque plcs per line */
#define		NUMLIMPLCS	20

/*      number of groups which can be defined in the PCU/LGU */
#define         MAXGROUPNUM     16

#define         RES_LOCKED       0

/*
**	The IOC will assume that the data to be exchanged with each PLC will           	
**	come from a fixed number of coils, digital inputs, analog inputs               	
**	or registers. It will read these in groups to minimize the amount of           	
**	communications overhead. It will also try to build as large a fetch            	
**	message as possible & required by remembering which consecutive groups         	
**	have been requested by the SCAN task.                                          	
*/

#define MCOILS		0                       /* Card number for coils 		     */
#define MDINS		1                       /* Card number for digital inputs 	     */
#define MAINS		3                       /* Card number for analog inputs 	     */
#define MREGS		4                       /* Card number for registers 		     */
#define MVALVE         10                       /* Card number for open/close valve control  */
#define MMOVETO        11                       /* Card number for positioning valve control */
#define MPSEUDO	       33                       /* Card number for pseudo i/o 		     */

/* Limitorque: Number of Groups of items */
#define NUMLIMCOILS     1       /*   8 */      /* 1 group of 8 coils		             */
#define NUMLIMDIS     (10+2+16) /*  80 */      /* 10 groups of 8 digitals		     */
#define NUMLIMAIS      57       /* 227 */      /* 57 groups of 4 analogs		     */
#define NUMLIMREGS     57       /* 227 */      /* 57 groups of 4 registers	 	     */

/* Rotork: Number of Groups of items */
#define NUMROTCOILS     1       /*   8 */      /* 1 group of 8 coils		             */
#define NUMROTDIS       1       /*  24 */      /* 3 groups of 8 digitals		     */
#define NUMROTAIS       2       /*   6 */      /* 2 groups of 4 analogs	                     */
#define NUMROTREGS      1       /*   2 */      /* 1 group of 4 registers	 	     */


/* Limitorque AND Rotork: Max of Number of Groups of items */
#define NUMALLCOILS   1 /* ((NUMLIMCOILS > NUMROTCOILS)? NUMLIMCOILS = NUMROTCOILS)*/
#define NUMALLDIS    28 /* ((NUMLIMDIS   > NUMROTDIS)  ? NUMLIMDIS   = NUMROTDIS)  */
#define NUMALLAIS    57 /* ((NUMLIMAIS   > NUMROTAIS)  ? NUMLIMAIS   = NUMROTAIS)  */
#define NUMALLREGS   57 /* ((NUMLIMREGS  > NUMROTREGS) ? NUMLIMREGS  = NUMROTREGS) */


#define BITBLOCK        8                      /* size of coil/digital groups 	             */
#define REGBLOCK	4                      /* size of analog/register groups	     */


#define VALVEPOSITIONREGISTER       8          /* Limitorque Register Holding Value Position */
#define VALVEMOVETOCOMMAND       6656          /* Limitorque 'Move To Location' Command      */


/* Number of items */
#define LIMMINCOILNO       1                     /* min number of individual coil 	        */
#define LIMMINDINO         129                   /* max number of individual digital            */
#define LIMMINAINO         1                     /* max number of individual analog             */
#define LIMMINREGNO        1                     /* max number of individual register           */

/* Number of items */
#define LIMMAXCOILNO       LIMMINCOILNO + (NUMLIMCOILS*BITBLOCK)  /* max coil number               */
#define LIMMAXDINO	      (129+80)                            /* max digitals                  */
#define LIMMAXAINO         LIMMINAINO   + (NUMLIMAIS*REGBLOCK)    /* max num analogs               */
#define LIMMAXREGNO        LIMMINREGNO  + (NUMLIMREGS*REGBLOCK) /* max num registers             */


/* Number of items */
#define ROTMINCOILNO       1                     /* min number of individual coil 	        */
#define ROTMINDINO         1                     /* max number of individual digital            */
#define ROTMINAINO         1                     /* max number of individual analog             */
#define ROTMINREGNO        1                     /* max number of individual register           */

/* Number of items */
#define ROTMAXCOILNO    ROTMINCOILNO + (NUMROTCOILS*BITBLOCK) -1  /* max coil number               */
#define ROTMAXDINO	ROTMINDINO   + (NUMROTDIS*BITBLOCK)   -1  /* max digitals                  */
#define ROTMAXAINO      ROTMINAINO   + (NUMROTAIS*REGBLOCK)   -1  /* max num analogs               */
#define ROTMAXREGNO     ROTMINREGNO  + (NUMROTREGS*REGBLOCK)  -1  /* max num registers             */


#define MAXDIGBLKSIZE	64			/* Maximum number of digital groups 	     */
						/*  to request in one packet        	     */
#define MAXREGBLKSIZE	16			/* Maximum number of registers               */
						/*  to request in one packet        	     */

/*
**	Conversion types for analog inputs	*/
/*
*/
#define		AICNVINTEGER	10
#define		AICNVDOUBLEINT	11
#define		AICNVFLOAT	12
#define		AICNVDOUBLEFLT	13
#define		AICNVASCIIFLT	14
#define		AICNVSTATUS	15
#define		AICNVTIME	16
#define		AICNVASCII_2	17
#define		AICNVASCII_4	18
#define		AICNVASCII_12	19
#define		AICNVASCII_20	20
#define		MAXCONV		20

/* The following must match the definition in recChoice.ascii */
#define NOCONV 0
#define LINEAR 1
#define ASCFLT 2


#define    WATCHDOGTIMEOUT       10*60 /* watchdog monitor timeout.  */


/*
**	Communications I/O Structures
**
*/

#define		RXSEM		0
#define		TXSEM		1
#define		MAXRXMSG	256
#define		MAXTXMSG	256


typedef	struct {
	unsigned short		rxstate;
	unsigned short		fixedlen;
	unsigned short		rxmsglenexpected;
        unsigned short          rxmsglen;
       	unsigned short		rxix;
	SEMAPHORE		*semid;
	unsigned short		*status;
        unsigned char           txaddress;
        unsigned char           txfunction;
       	unsigned char		rxbuf[MAXRXMSG];
		} 	RXLINE;


typedef	struct {
	unsigned short		protocol;
	unsigned short		txdisable;
	unsigned short		txtocnt;
	unsigned short		txhwerrcnt;
	unsigned short		txmsglen;
	unsigned short		txix;
	SEMAPHORE		*semid;
	unsigned short		*status;
	unsigned char		txbuf[MAXTXMSG];
		} 	TXLINE;

/*
**	Communications I/O Constants
**
*/

#define		RXGOOD			1
#define		RXBAD			2
#define		RXTIMEOUT		60	/* 16 ms ticks 				*/
#define		ACTIVITYTIME		300	/* must see activity in 5 seconds 	*/

#define		VARRESP			FALSE
#define		FIXEDRESP		TRUE
#define		ONERETRY		0x8000	/* OR'ed into line number for xfermsg 	*/
#define		RXIGNORE		0
#define		RXADRST			1
#define		RXFNCST			2
#define		RXLENST			3
#define		RXDATAST		4

#define		TXGOOD			1
#define		TXBAD			2
#define		IO_TIMEOUT		3	/* Code returned by timer 		*/


#define		TXTIMEOUT		60	/* 16 ms ticks: one second              */

#define		MAXRETRYS		3	/* retransmission trys 			*/

#define		MAXCFGCK		8	/* Try to Bring online each cycle 	*/
#define		BRINGBACKCNT		4	/* Poll loops per cycle 		*/

#define		XFERGOOD		1
#define		XFERBAD			2

#define		RESNAMELEN		16	/* length of a resource name 		*/
#define		SLWWINDOW		900	/* # of ticks to allow slave to try 	*/


/*
** Output Queue Management Structures for Limitorque/Modbus Protocol
**
*/

#define         AOUTTYPE        1
#define         BOUTTYPE        2

#define         OUTQSIZE        128

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	short           aovalue;
	double		dblaovalue;
	unsigned short	conversion;
		} AOUT, *AOUTPTR;

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	long            bomask;
	long            bovalue;
		} BOUT, *BOUTPTR;

typedef struct {
		unsigned short  otype;
		union   {
			AOUT    aonode;
			BOUT    bonode;
			}out;
	       } OUTQENT;

typedef struct {
		unsigned short	ooutix;
		unsigned short	oinix;
		OUTQENT         outque[OUTQSIZE];
		} OUTPUTQUE;

/*
**	Internal IOC Structure for Limitorque PLC
**
*/
typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		last_read;
		}	LIMCOILS;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		last_read;
		}	LIMDIS;

typedef struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		ioregs[REGBLOCK];

		}	REGISTERS;

typedef struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		aivals[REGBLOCK];
		}	LIMAIS;


typedef	struct	{
	unsigned short  was_requested;		/* 0  The scan task has accesses this RTU 	*/
	unsigned short  is_responding;		/* 1  The RTU is responding 			*/
	unsigned short  holdflag;		/*    The RTU data is in hold 			*/
	unsigned short  no_dataout_msgs;	/* 5  Count Output Msgs 			*/
	unsigned short  no_datain_msgs;		/* 6  Count Data Requests 			*/
	unsigned short  no_rxnoresp;		/* 7  Count No Responses 			*/
	unsigned short  no_txnoresp;		/* 8  Count No Responses 			*/
	unsigned short  no_dataerr;		/* 9  Count Message Echo Errors 		*/
	unsigned short  no_retries;		/* 10 Count Retries 				*/
	unsigned short  no_rxhwerrs;		/* 11 Count Comm Hardware Errors 		*/
	unsigned short  no_txhwerrs;		/* 12 Count Comm Hardware Errors 		*/
	unsigned short  no_crcerrs;		/* 13 Count CRC Errors 				*/
	unsigned short  no_offlines;		/* 14 Count Off Line Transitions		*/
	unsigned short  cmd_reset_stat;		/* 15 Comand to reset stats			*/
	unsigned short  cmd_force_offline;	/* 16 Command to Force Offline 			*/
	unsigned short  badoutcnt;		/* 17 Count Failed output commands 		*/
	unsigned short  activechannel;		/* 19 Active Channel 0= Primary,  1 = Secondary	*/

        unsigned short  tscanpass;		/* Total number of scanpasses. incremented at   */
                                                /*  the start of each plc scan.                 */
        unsigned short  msgs;			/* Count of messages which got good responses.  */
                                                /*  incremented on each exit from 'good' msgxfer*/
        unsigned long   responsetime;		/* Total time(ticks) waiting for good responses.*/
                                                /* accumu (exit-entry) time for 'good' msgxfers */

	LIMCOILS	coils[NUMALLCOILS];
	LIMDIS		dis  [NUMALLDIS];
	LIMAIS		ais  [NUMALLAIS];
	REGISTERS	regs [NUMALLREGS];
	}	LIMPLC;


/*
** MODBUS Protocol function codes
*/

#define MODCOILREAD		0x01  /* READ COIL STATUS: (ON/OFF) read a group of logic coils.     */
#define MODDIREAD		0x02  /* READ INPUT STATUS:(ON/OFF) of a group of discrete inputs.   */
#define MODREGREAD		0x03  /* READ HOLDING REGISTERS:    get current binary value in one  */
                                      /*                            or more holding registers.       */
#define MODAIREAD		0x04  /* READ INPUT REGISTERS:      get current binary value in one  */
                                      /*                            or more input registers.         */

#define MODCOILWRITE		0x05  /* FORCE SINGLE COIL:         force logic coil to ON/OFF state.*/
#define MODREGWRITE		0x06  /* PRESET SINGLE REGISTER:    place a specific binary value    */
                                      /*                            into a holding register.         */

#define MODLOOPBACK		0x08  /* LOOPBACK DIAGNOSTIC TEST                                    */

#define FNCERRBIT		0x80  /* Modbus error function code */


/* Various message lengths used in message transmission and response validation */
#define MODREADMSGLEN		0x06
#define MODSETCOILLEN		0x06
#define MODSETREGLEN		0x06
#define MODCRCLEN		0x02
#define MODLOOPBACKLEN		0x06
#define MODLOOPBACKRESPLEN	0x08	/* Including CRC */
#define MODWRITERESPLEN		0x08	/* Including CRC */
#define NAKMSGLEN		0x03	/* bytes left get */

/*
** MODBUS Exception Codes
*/
#define MODEXCEPTBUSY		0x06	/* "    "   "    */

/* Command Results */

#define CMDDONE                 0x00     /* the command was sent and the response was error free.   */
#define CMDFAIL                 0x01     /* there were errors in sending or receiving the command.  */
#define CMDNACK                 0x02     /* response was error free, but command was not completed. */
#define CMDREJECT               0x03     /* driver rejected command due to unsupported data type.   */
#define CMDBUSY			0x05	 /* The command was sent, but the device is busy */

/* task definitions */

#define LIM_STACK		8000   /* Stack size              */
#define	LIM_PRIORITY		152    /* default priority        */
#define	LIM_TASK_OPTIONS	8      /* Floating point          */


/*
** Mutual-Exclusion Control for diagnostic/data radio channel sharing */
/*
*/

#define		MUXTIMEOUT	60*30	/* 16.67 ms ticks (60Hz): 30 seconds 	*/

typedef struct {
        unsigned long   TXintcount;
        unsigned long   RXintcount;
        unsigned long   RXbadcount;
        unsigned long   msgxfercount;
        unsigned long   startingtick;
               }        LINESTATS;

/* Trace Bit Definitions */

#define ALLTRAFFICTRACE      0x0001
#define MODBUSNACKTRACE      0x0002        /* MODBUS NACK errors.                               */
#define VALVEOUTPUTTRACE     0x0004        /* trace special value output points.                */
#define CONFIGURATIONTRACE   0x0010        /* trace configuration errors in driver calls.       */
#define RUNTIMEERRORTRACE    0x0020        /* trace run time errors in driver calls.            */
#define CALLREJECTTRACE      0x0040        /* trace non-error based driver call rejections.     */


/*	End of File:	drvLimitorque.h */
