/****************************************************************/
/*                                                              */
/*      This software is the proprietary property of Tate       */
/*      Integrated Systems, L.P.  Patents and copyrights are    */
/*      pending.                                                */
/*                                                              */
/*      This software is furnished under a license and may      */
/*      be used and copied only in accordance with the terms    */
/*      of such license and with the inclusion of the above     */
/*      notice. This software or any other copies thereof       */
/*      may not be provided or otherwise made available to      */
/*      any other person. No title to and ownership of the      */
/*      software is hereby transferred.                         */
/*                                                              */
/*      The information in this software is subject to          */
/*      change without notice and should not be construed       */
/*      as a commitment by Tate Integrated Systems L.P. or      */
/*      its third party suppliers.                              */
/*                                                              */
/*      Tate Integrated Systems, L.P. (TIS) and its third       */
/*      party suppliers assume no responsibility for the        */
/*      use or inability to use any of its software. TIS        */
/*      software is provided "as is" without warranty of        */
/*      any kind and TIS expressly disclaims all implied        */
/*      warranties, including but not limited to the implied    */
/*      warranties of merchantability and fitness for a         */
/*      particular purpose.                                     */
/*                                                              */
/*      Notice: Notwithstanding any other lease or license      */
/*      that may pertain to or accompany the delivery of this   */
/*      computer software, the rights of the Government         */
/*      regarding its use, reproduction, and disclosure are     */
/*      as set forth in Section 52.227-19 of the FARS           */
/*      Computer Software-Restricted Rights clause.             */
/*                                                              */
/****************************************************************/
/*
/*      File:   drvMds.h                                                                
/*      Auth:   Walter E. Palmer                                                        
/*      Edit:   03/11/94        WEP: based on smod_driver.h                             
/*
/*      This file contains the data structures used by the MDS Radio I/O driver.
/*
*/
#define TRUE    1
#define FALSE   0
#define NULLP   0
#define CR      0x0d

/* Debug Tracing Bit Definitions for mdstrace() */
/* note: this information must match definitions in TRACEINFO in drvMds.c */

#define MASTERCOMMTRACE     0x0001 /* Communications:    Master Radio Only related messages    */
#define REMOTECOMMTRACE     0x0002 /* Communications:    Remote Radio related messages         */
#define GENERALCOMMTRACE    0x0004 /* Communications:    All communiations errors              */
#define MASTERPOLLTRACE     0x0010 /* Communications:    Beginning of master radio poll        */
#define REMOTEPOLLTRACE     0x0020 /* Communications:    Beginning of remote radio poll        */
#define DEMANDACTIONTRACE   0x0040 /* Communications:    All outputs and forced loopback tests */
#define MUTEXTRACE          0x0100 /* Mutual Exclusion:  Semaphore activity                    */
#define RESOURCESTATETRACE  0x0200 /* Resource State:    Master/Slave activity                 */
#define RESOURCEERRORTRACE  0x0400 /* Resource Error:    Resource not allocated                */
#define RADIOMAPTRACE       0x0800 /* Radio Map:         Initialization and running details    */
#define OUTPUTQUEUETRACE    0x1000 /* Output Queue:      Full indication                       */
#define CALLOCTRACE         0x2000 /* Memory Allocation: Noted only (errors ALWAYS reported)   */



#define TICKSPERSEC 60

/*      maximum number of lines in the system           */
#define         IOCLINES        20
/*      number of lines of MDS Master Radios            */
#define         MDSLINES        20
/*      number of MDS Remote Radios per line            */
#define         NUMMDSRADIOS    100

/*                                                                                      
/*      The IOC will assume that the data to be exchanged with each Radio will          
/*      come from a fixed number of coils, digital inputs, analog inputs                
/*      or registers, and will use the predefine maximum values to command validation   
/*      on driver calls for data. However, it will translate these data I/O requests    
/*      into appropriate MDS commands. These commands are of a format such that there   
/*      is not a one-to-one correspondence between the data value request and the       
/*      command which must be issued to obtain the data. In general, the issuance of a  
/*      a single MDS Radio command will result in the loading of many of the database   
/*      points.                                                                         
*/

#define MASTERHOLDOFFTIME 0*TICKSPERSEC  /* # of ticks for which the MDS master radio   */
					 /* will disable the radio data channel after   */
					 /* beginning a master radio diagnostic access. */

#define REMOTEHOLDOFFTIME 12*TICKSPERSEC /* # of ticks for which the MDS master radio   */
					 /* will disable the radio data channel after   */
					 /* beginning a remote radio diagnostic access. */

#define MAINTENANCETIME 12*TICKSPERSEC    /* # of ticks for which the MDS master radio   */
					 /* will disable the radio data channel after   */
					 /* beginning a remote radio maintenance access.*/

#define MASTERADDRESS   0               /* logical address of Master Radio      */
#define FIRSTRADIOADDRESS 1             /* logical address of First Remote Radio*/
#define LASTRADIOADDRESS NUMMDSRADIOS   /* logical address of Last Remote Radio */

#define MCOILS     0                    /* Card number for coils                */
#define MDINS      0                    /* Card number for digital inputs       */
#define MAINS      0                    /* Card number for analog inputs        */
#define MREGS      0                    /* Card number for registers            */
#define MPSEUDO    33                   /* Card number for pseudo i/o           */

#define CARDNOCOILS     0               /* Card number for coils                */
#define CARDNODINS      0               /* Card number for digital inputs       */
#define CARDNOAINS      0               /* Card number for analog inputs        */
#define CARDNOREGS      0               /* Card number for registers            */
#define CARDNOPSEUDO    33              /* Card number for pseudo i/o           */

#define NUMMDSBISXXX       8            /* 8 groups of 8 digitals               */
#define NUMMDSAISXXX      30            /* 30 groups of analogs                 */

/* Number of items */
#define MAXBINO         40              /* max number of digital input bits  */
#define MAXAINO         27              /* max number of analogs inputs      */
#define MAXBONO          9              /* max number of digital output bits */
#define MAXAONO          6              /* max number of analog outputs      */


/*
/*      Communications I/O Structures
/*
*/

#define         RXSEM           0
#define         TXSEM           1
#define         MAXRXMSG        256
#define         MAXTXMSG        256


typedef struct {
	unsigned short          protocol;
	unsigned short          rxstate;
	unsigned short          rxtocnt;
	unsigned short          rxhwerrcnt;
	unsigned short          rxmsglen;
	unsigned short          rxix;
	int                     rxtimeout;
	SEMAPHORE               *semid;
	unsigned short          *status;
	unsigned char           rxbuf[MAXRXMSG];
		}       RXLINE;


typedef struct {
	unsigned short          protocol;
	unsigned short          txdisable;
	unsigned short          txtocnt;
	unsigned short          txhwerrcnt;
	unsigned short          txmsglen;
	unsigned short          txix;
	int                     txtimeout;
	SEMAPHORE               *semid;
	unsigned short          *status;
	unsigned char           txbuf[MAXTXMSG];
		}       TXLINE;
/*
/*      Communications I/O Constants
/*
*/

#define         RXGOOD                  1
#define         RXBAD                   2

#define         RXTIMEOUT               1*TICKSPERSEC  /* default value = one second     */

#define         ACTIVITYTIME            5*TICKSPERSEC  /* must see activity in 5 seconds */

#define         VARRESP                 FALSE
#define         FIXEDRESP               TRUE
#define         ONERETRY                0x8000  /* OR'ed into line number for xfermsg    */
#define         RXIGNORE                0
#define         RXADRST                 1
#define         RXFNCST                 2
#define         RXLENST                 3
#define         RXDATAST                4

#define         TXGOOD                  1
#define         TXBAD                   2

#define         TXTIMEOUT               10*TICKSPERSEC  /* default timeout - 10 seconds */ 

#define         IO_TIMEOUT              3       /* Code returned by timer               */

#define         MAXRETRYS               1       /* transmission attempts/message        */

#define         FAILRETRYCOUNT          2       /* # of message trys per radio before   */
						/* marking the radio as offline.        */

#define         MAXCFGCK                8       /* Try to Bring online each cycle       */
#define         BRINGBACKCNT            4       /* Poll loops per cycle                 */

#define         XFERGOOD                1
#define         XFERBAD                 2

#define         RESNAMELEN              16      /* length of a resource name            */
#define         SLWWINDOW               15*TICKSPERSEC /* time to allow slave to try    */

/*
/* Output Queue Management Structures for MDS Radio Protocol
/*
*/

#define         AOUTTYPE                        1
#define         BOUTTYPE                        2
#define         FORCEREMOTELOOPBACKTYPE         3
#define         OUTQSIZE                        128

typedef struct
{
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	short           aovalue;
	short           spare;
} AOUT, *AOUTPTR;

typedef struct
{
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	short           bomask;
	short           bovalue;
} BOUT, *BOUTPTR;

typedef struct
{
    unsigned short  otype;
    union
    {
	    AOUT    aonode;
	    BOUT    bonode;
    } out;
} OUTQENT;

typedef struct
{
    unsigned short  ooutix;
    unsigned short  oinix;
    OUTQENT         outque[OUTQSIZE];
} OUTPUTQUE;



/*
/*      Internal IOC Structure for Microwave Data Systems Remote Radio
/*
*/

#define DIG1 0            /* Master Transmitter A Status          */
#define DIG2 1            /* Master Transmitter B Status          */
#define DIG3 2            /* Master Operation Status              */
#define DIG4 3            /* Master Remote Status                 */
#define DIG8 4            /* Master Mains Status                  */
#define DIG6 6            /* Response Status - no associated data */
#define DIG5 5            /* not used                             */
#define DIG7 7            /* not used                             */

typedef struct {
	unsigned short          was_requested;
	unsigned short          data_valid;
	unsigned short           last_read;
		}       XXXAIS;

#define   AN1A  0         /*  0 Master Secondary Supply Voltage A         */
#define   AN1B  1         /*  1 Master Secondary Supply Voltage B         */
#define   AN2A  2         /*  2 Master Primary Supply A                   */
#define   AN2B  3         /*  3 Master Primary Supply B                   */
#define   AN3A  4         /*  4 Master Supply Current A                   */
#define   AN3B  5         /*  5 Master Supply Current B                   */
#define   AN4A  6         /*  6 Master TX RF Power A                      */
#define   AN4B  7         /*  7 Master TX RF Power B                      */
#define   AN5A  8         /*  8 Master Signal Strength A                  */
#define   AN5B  9         /*  9 Master Signal Strength B                  */
#define   AN6A 10         /* 10 Master Frequency Offset A                 */
#define   AN6B 11         /* 11 Master Frequency Offset B                 */
#define   AN7A 12         /* 12 Master Deviation A                        */
#define   AN7B 13         /* 13 Master Deviation B                        */
#define   AN8  14         /* 14 Remote Signal Strength                    */
#define   AN9  15         /* 15 Remote TX RF Power                        */
#define   AN10 16         /* 16 Remote Supply Voltage                     */
#define   AN11 17         /* 17 Remote Voltage Regulator                  */
#define   AN12 18         /* 18 Remote VCO Lock Voltage                   */
#define   AN13 19         /* 19 Remote Temperature                        */
#define   AN14 20         /* 20 Remote Spare                              */
#define   AN15 21         /* 21 Master Aux Voltage Regulator              */
#define   AN16 22         /* 22 Master Temperature                        */
#define   AN17 23         /* 23 Master Spare                              */
#define   EEP1 24         /* 24 Remote TX RF Power EEPOT Setting          */
#define   EEP2 25         /* 25 Remote Frequency EEPOT Setting            */
#define   EEP3 26         /* 26 Remote Deviation EEPOT Setting            */
#define   REV1 27         /* 27 Master Firmware Revision Level            */
#define   SPR1 28         /* 28 Remote Spare 1                            */
#define   SPR2 29         /* 29 Remote Spare 2                            */


typedef struct {
	unsigned short          was_requested;
	unsigned short          data_valid;
	unsigned long           last_read;
		}       XXXBIS;

typedef struct
{
	unsigned short  was_requested;          /* 0  The scan task accessed this Radio */
	unsigned short  is_responding;          /* 1  The Radio is responding           */
	unsigned short  retry_count;            /* 2  The current retry counter value   */
	unsigned short  offline;                /* 3  The Radio not in normal poll loop */
	unsigned short  holdflag;               /* 4  The Radio data is on hold         */
	unsigned short  no_dataout_msgs;        /* 5  Count Output Msgs                 */
	unsigned short  no_datain_msgs;         /* 6  Count Data Requests               */
	unsigned short  no_rxnoresp;            /* 7  Count No Responses                */
	unsigned short  no_txnoresp;            /* 8  Count No Responses                */
	unsigned short  no_dataerrs;            /* 9  Count Message Echo Errors         */
	unsigned short  no_rxlenerrs;           /* 10 Count Message Length Errors       */
	unsigned short  no_retries;             /* 11 Count Retries                     */
	unsigned short  no_rxhwerrs;            /* 12 Count Comm Hardware Errors        */
	unsigned short  no_txhwerrs;            /* 13 Count Comm Hardware Errors        */
	unsigned short  no_crcerrs;             /* 14 Count CRC Errors                  */
	unsigned short  no_chksumerrs;          /* 15 Count Checksum Errors             */
	unsigned short  no_offlines;            /* 16 Count Off Line Transitions        */
	unsigned short  cmd_reset_stat;         /* 17 Comand to reset stats             */
	unsigned short  cmd_force_offline;      /* 18 Command to Force Offline          */
	unsigned short  badoutcnt;              /* 19 Count Failed output commands      */
	XXXBIS          bismds[NUMMDSBISXXX];
	XXXAIS          aismds[NUMMDSAISXXX];
} MDSRADIO;

/* was_requested;     This flag indicates that the scan has task accessed this Radio
		      at least once. Until this flag has been set, the radio will       
		      not be polled on a scheduled basis. This flag is cleared when the 
		      IOC driver becomes the Master.                                   */

/* is_responding;     This flag indicates that the last communication to the radio
		      was successful. With the Mds radios, this could be logically be
		      implemented identically to the data_valid flag. However, it
		      will remain valid during transitory communication errors through
		      the use of retry_count below. once the max count has been reached
		      the is_responding flag will be marked false, and the radio will
		      be taken out of the 'normal' polling list.*/

/* retry_count;       This counter is used to determine when a non-responsive radio
		      has exausted its right to a position in the 'normal' polling
		      loop, and must be put into a low frequency polling mode.
		      Each time the radio fails to respond, the counter is incremented.
		      Once a predefined count has been reached the offline flag
		      will be marked true, thereby effectively taking the radio out of
		      the 'normal' polling list.*/

/* offline;           This flag indicates that the radio has been taken out of the
		      'normal' polling loop, and will be retried at a much lower
		      duty-cycle. */

/* holdflag;          This flag indicates to the radio and its data are not available
		      for input/output via the driver call program. The flag is set
		      when 1) going to master state having been asked to by our partner
		      IOC, 2) when going to master state because partner isn't there,
		      3) returning to master state after being told to give up master
		      state and, having gone to slavewait state and waiting, having
		      been commanded to go back to master state. It is cleared 1) at the
		      end of every major master mode loop, 2) when a PLC is first brought
		      online(init_plc_tab), and 3) whenever a communications error occurs.
		      (comment: it really doesn't seem to do much anymore. instead, its
		      function seems to have been taken over by the resresponse[] flag.

/* no_dataout_msgs;   Count Output Msgs                 */
/* no_datain_msgs;    Count Data Requests               */
/* no_rxnoresp;       Count No Responses                */
/* no_txnoresp;       Count No Responses                */
/* no_dataerr;        Count Message Echo Errors         */
/* no_retries;        Count Retries                     */
/* no_rxhwerrs;       Count Comm Hardware Errors        */
/* no_txhwerrs;       Count Comm Hardware Errors        */
/* no_crcerrs;        Count CRC Errors                  */
/* no_offlines;       Count Off Line Transitions        */
/* cmd_reset_stat;    Comand to reset stats             */
/* cmd_force_offline; Command to Force Offline          */
/* badoutcnt;         Count Failed output commands      */
/*
/* Microwave Data Systems Radio function codes
*/

#define MDSMSGSTARTBYTE         0x96    /* MDS Message Start Byte                       */
#define MDSMSGENDBYTE           0x99    /* MDS Message End Byte                         */
#define MDSMSGSPARE             0x00    /* MDS Message Spare Byte                       */


#define REQUESTALARMSTATUS      0x31    /* R1  REQUEST ALARM STATUS:                    */

#define REMOTECONTROLMASTER     0x32    /* R2  REMOTE CONTROL of MASTER:                */
#define         SWITCHTXA       0x40    /*     SWITCH TX TO A SIDE:                     */
#define         SWITCHTXB       0x41    /*     SWITCH TX TO B SIDE:                     */
#define         SWITCHTXAUTO    0x42    /*     SWITCH TX TO AUTO:                       */
#define         SWITCHRXA       0x43    /*     SWITCH RX TO A SIDE:                     */
#define         SWITCHRXB       0x44    /*     SWITCH RX TO B SIDE:                     */
#define         SWITCHRXAUTO    0x45    /*     SWITCH RX TO AUTO:                       */

#define REQUESTTXDATA           0x33    /* R3  REQUEST TX DATA:                         */

#define REQUESTRXDATA           0x34    /* R4  REQUEST RX DATA:                         */

#define REMOTEDIAGNOSTICS       0x36    /* R6  REMOTE ADVANCED LOOPBACK DIAGNOSTICS:    */

#define MASTERSTATUS            0x39    /* R9  MASTER STATUS INFORMATION:               */

#define MASTERCONTROL           0x3A    /* R10 MASTER CONTROL:                          */
#define         ENABLEDIAGS     0X53    /*     ENABLE MASTER DIAGNOSTICS:               */
#define         DISABLEDIAGS    0x63    /*     DISABLE MASTER DIAGNOSTICS:              */


#define REMOTEACTIVATE          0x3B    /* R11 REMOTE MAINTENANCE ACTIVATE:             */

#define REMOTECONTROL           0x50    /* R15 REMOTE MAINTENANCE OPERATING PARAM CNTL: */
#define         CLOSECHANNEL    0x01    /*     CLOSE REMOTE MAINTENANCE CHANNEL:        */
#define         INCRFTXPOWER    0x10    /*     INCREMENT RF TX POWER BY VALUE:          */
#define         DECRFTXPOWER    0x11    /*     DECREMENT RF TX POWER BY VALUE:          */
#define         INCRFFREQUENCY  0x12    /*     INCREMENT RF FREQUENCY BY VALUE:         */
#define         DECRFFREQUENCY  0x13    /*     DECREMENT RF FREQUENCY BY VALUE:         */
#define         INCRFDEVIATION  0x14    /*     INCREMENT RF DEVIATION BY VALUE:         */
#define         DECRFDEVIATION  0x15    /*     DECREMENT RF DEVIATION BY VALUE:         */

#define         STOREEEPOTS     0x20    /*     STORE EEPOT SETTINGS:                    */
#define         DISABLEAFC      0x50    /*     DISABLE AUTOMATIC FREQ CNTL (tx & rx):   */
#define         ENABLETXAFC     0x51    /*     ENABLE TX AFC:                           */
#define         ENABLERXAFC     0x52    /*     ENABLE RX AFC:                           */

#define REPEATLASTMSG           0x51    /* R16 REPEAT LAST MESSAGE:                     */


/* Various message lengths used in MDS radio message transmission and response validation */

#define REQUESTALARMSTATUSLEN   0x04    /* R1  REQUEST ALARM STATUS:                    */
#define REMOTECONTROLMASTERLEN  0x0B    /* R2  REMOTE CONTROL of MASTER:                */
#define REQUESTTXDATALEN        0x04    /* R3  REQUEST TX DATA:                         */
#define REQUESTRXDATALEN        0x04    /* R4  REQUEST RX DATA:                         */
#define REMOTEDIAGNOSTICSLEN    0x08    /* R6  REMOTE ADVANCED LOOPBACK DIAGNOSTICS:    */
#define MASTERSTATUSLEN         0x04    /* R9  MASTER STATUS INFORMATION:               */
#define MASTERCONTROLLEN        0x0B    /* R10 MASTER CONTROL:                          */
#define REMOTEACTIVATELEN       0x0C    /* R11 REMOTE MAINTENANCE ACTIVATE:             */
#define REMOTECONTROLLEN        0x0C    /* R15 REMOTE MAINTENANCE OPERATING PARAM CNTL: */
#define REPEATLASTMSGLEN        0x04    /* R16 REPEAT LAST MESSAGE:                     */

#define T1LEN                    7      /* T1  RESPONSE TO COMMANDS CODE 31H & 32H      */
#define T2LEN                   10      /* T2  RESPONSE TO COMMANDS CODE 33H            */
#define T3LEN                   10      /* T3  RESPONSE TO COMMANDS CODE 34H            */
#define T4LEN                   20      /* T4  ADVANCED DIAGNOSTICS (REMOTE RESPONSE)   */
#define T7LEN                   15      /* T7  MASTER STATION STATUS                    */
#define T8LEN                    9      /* T8  MASTER STATION DIAGNOSTICS STATUS        */
#define T9LEN                   24      /* T9  REMOTE MAINTENANCE DIAGNOSTICS CONFIG.   */

/* Various message timeout values used in MDS radio message transmission and response   */

#define R1TIMEOUT      1*TICKSPERSEC    /* R1  REQUEST ALARM STATUS:                    */
#define R2TIMEOUT      1*TICKSPERSEC    /* R2  REMOTE CONTROL of MASTER:                */
#define R3TIMEOUT      1*TICKSPERSEC    /* R3  REQUEST TX DATA:                         */
#define R4TIMEOUT      1*TICKSPERSEC    /* R4  REQUEST RX DATA:                         */
#define R6TIMEOUT      1*TICKSPERSEC    /* R6  REMOTE ADVANCED LOOPBACK DIAGNOSTICS:    */
#define R9TIMEOUT      1*TICKSPERSEC    /* R9  MASTER STATUS INFORMATION:               */
#define R10TIMEOUT     1*TICKSPERSEC    /* R10 MASTER CONTROL:                          */
#define R11TIMEOUT     1*TICKSPERSEC    /* R11 REMOTE MAINTENANCE ACTIVATE:             */
#define R15TIMEOUT     1*TICKSPERSEC    /* R15 REMOTE MAINTENANCE OPERATING PARAM CNTL: */
#define R16TIMEOUT     1*TICKSPERSEC    /* R16 REPEAT LAST MESSAGE:                     */

#define T1TIMEOUT      1*TICKSPERSEC    /* T1  RESPONSE TO COMMANDS CODE 31H & 32H      */
#define T2TIMEOUT      1*TICKSPERSEC    /* T2  RESPONSE TO COMMANDS CODE 33H            */
#define T3TIMEOUT      1*TICKSPERSEC    /* T3  RESPONSE TO COMMANDS CODE 34H            */
#define T4TIMEOUT      6*TICKSPERSEC    /* T4  ADVANCED DIAGNOSTICS (REMOTE RESPONSE)   */
#define T7TIMEOUT      1*TICKSPERSEC    /* T7  MASTER STATION STATUS                    */
#define T8TIMEOUT      2*TICKSPERSEC    /* T8  MASTER STATION DIAGNOSTICS STATUS        */
#define T9TIMEOUT      6*TICKSPERSEC    /* T9  REMOTE MAINTENANCE DIAGNOSTICS CONFIG.   */


/*
/* MDS Command Status Codes
*/

#define NOCHECKSUM            FALSE    /* no checksum to be calculated for the message */
#define CHECKSUM              TRUE     /* checksum to be calculated for the message    */
				       


/* task definitions */

#define MDS_STACK               8000   /* Stack size              */
#define MDS_PRIORITY            152    /* default priority        */
#define MDS_TASK_OPTIONS        8      /* Floating point          */


/*
/* Mutual-Exclusion Control for diagnostic/data radio channel sharing */
/*
*/

#define         MUXTIMEOUT      30*TICKSPERSEC   /*  30 seconds    */


/* File Loading Termination Error Codes */

#define LOADOK                  0x00
#define CANNOTOPENFILE          0x01
#define SYNTAXERROR             0x02
#define NOMASTERADDRESS         0x03
#define BADRADIOADDRESS         0x04


/* Map File Parsing State Name Assignments */

#define  START                  0x00
#define  FIRSTPARAM             0x01
#define  SECONDPARAM            0x02
#define  STOREVALUE             0x03
#define  WRAPUP                 0x04                 


/*      Structure Definitions */
typedef struct
{
	unsigned short valid_flag;
	unsigned short  hardware;
}       MAP;

typedef struct
{
	unsigned short  line;
	unsigned short  loading;
	unsigned short  map_load_status;
	unsigned short  map_load_code;
	FILE            *mapfile;
	long            fileaddr;
	unsigned char   filename[30];
	MAP             address[NUMMDSRADIOS+1];
		}       LOADMAP;
/*      End of File:    drvMds.h */
