/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/




/****************************************************************************
 *
 *  MODULE NAME            : devQ984.c
 *  AUTHOR                 : John L. Dalesio
 *  DATE                   : 12 Aug 1996
 *
 *
 * DESCRIPTION: device support for Modicon Quantum I/O
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : 
 *  REVISION DATE          : 
 *  REVISION:
 *
****************************************************************************/

/****************************************************************************
 *
 * Programmer's Notes:
 *
****************************************************************************/


#include   <vxWorks.h>
#include   <types.h>
#include   <stdioLib.h>
#include   <semLib.h>
#include   <sysLib.h>
#include   <task_params.h>

#include   <alarm.h>
#include   <cvtTable.h>
#include   <dbAccess.h>
#include   <dbDefs.h>
#include   <devSup.h>
#include   <link.h>
#include   <devType.h>
/*#include   <module_types.h>*/
#include   <special.h>
#include   <cvnTypes.h>   /* defines for conversion types */
#include   <rec/aiRecord.h>
#include   <rec/aoRecord.h>
#include   <rec/biRecord.h>
#include   <rec/boRecord.h>
#include   <rec/mbbiRecord.h>
#include   <rec/mbboRecord.h>
#include   <rec/patRecord.h>
#include   <rec/ptoRecord.h>
#include   <rec/tdRecord.h>
#include   <drvQ984.h>
#include <logLib.h>
#include <string.h>
#include < error.h>

#define  IO_TYPE     V984_IO               /* NOTE: change here */
#define  AIDRIVER    vQtm_aidriver       /* NOTE: change here */
#define  AODRIVER    vQtm_aodriver       /* NOTE: change here */
#define  AOREAD      vQtm_aoread         /* NOTE: change here */
#define  BIDRIVER    vQtm_bidriver       /* NOTE: change here */
#define  BODRIVER    vQtm_bodriver       /* NOTE: change here */
#define  BOREAD      vQtm_boread         /* NOTE: change here */

static char *io_name = "V984";         /* NOTE: change here */

/* Create the dset for devAivQtm */

static long ai_init_record();
static long read_ai();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_ai;
   DEVSUPFUN   special_linconv;
} devAiV984 = {                        /* NOTE: change here */
   6,
   NULL,
   NULL,
   ai_init_record,
   NULL,
   read_ai,
   NULL};

/* Create the dset for devAoV984 */

static long ao_init_record();
static int read_ao ();
static long write_ao();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_ao;
   DEVSUPFUN   special_linconv;
   DEVSUPFUN   readback_ao;
} devAoV984 = {                        /* NOTE: change here */
   7,
   NULL,
   NULL,
   ao_init_record,
   NULL,
   write_ao,
   NULL,
   NULL};

/* Create the dset for devBiV984 */

static long bi_init_record ();
static long read_bi ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_bi;
} devBiV984 = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   bi_init_record,
   NULL,
   read_bi};

/* Create the dset for devBoV984 */

static long bo_init_record ();
static long write_bo ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_bo;
   DEVSUPFUN   readback_bo;
} devBoV984 = {                        /* NOTE: change here */
   6,
   NULL,
   NULL,
   bo_init_record,
   NULL,
   write_bo,
   NULL};

/* Create the dset for devMbbiV984 */

static long mbbi_init_record ();
static long read_mbbi ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_mbbi;
} devMbbiV984 = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   mbbi_init_record,
   NULL,
   read_mbbi};

/* Create the dset for devMbboV984 */

static long mbbo_init_record ();
static long write_mbbo ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_mbbo;
   DEVSUPFUN   readback_mbbo;
} devMbboV984 = {                      /* NOTE: change here */
   6,
   NULL,
   NULL,
   mbbo_init_record,
   NULL,
   write_mbbo,
   NULL};

/* Create the dset for devPtoV984 */

static long pto_init_record ();
static long write_pto ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_pto;
} devPtoV984 = {                       /* NOTE: change here */
   5,
   NULL,
   NULL,
   pto_init_record,
   NULL,
   write_pto};

/* Create the dset for devTdV984 */

static long td_init_record ();
static long write_td ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_td;
} devTdV984 = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   td_init_record,
   NULL,
   write_td};

/* Create the dsets for V984 pat block */

static long pat_init_record ();
static long write_pat ();
static long read_pat ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_pat;
   DEVSUPFUN   special_lincon;
} devPatAiV984 = {                     /* NOTE: change here */
   6,
   NULL,
   NULL,
   pat_init_record,
   NULL,
   read_pat,
   NULL};

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_pat;
} devPatBiV984 = {                     /* NOTE: change here */
   5,
   NULL,
   NULL,
   pat_init_record,
   NULL,
   read_pat};

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_pat;
} devPatBoV984 = {                     /* NOTE: change here */
   5,
   NULL,
   NULL,
   pat_init_record,
   NULL,
   write_pat};

static long masks [] = {
   0x00000001, 0x00000002, 0x00000004, 0x00000008,
   0x00000010, 0x00000020, 0x00000040, 0x00000080,
   0x00000100, 0x00000200, 0x00000400, 0x00000800,
   0x00001000, 0x00002000, 0x00004000, 0x00008000,
   0x00010000, 0x00020000, 0x00040000, 0x00080000,
   0x00100000, 0x00200000, 0x00400000, 0x00800000,
   0x01000000, 0x02000000, 0x04000000, 0x08000000,
   0x10000000, 0x20000000, 0x40000000, 0x80000000,
};

static long rmasks [] = {
   0x80000000, 0x40000000, 0x20000000, 0x10000000, 
   0x08000000, 0x04000000, 0x02000000, 0x01000000,   
   0x00800000, 0x00400000, 0x00200000, 0x00100000,   
   0x00080000, 0x00040000, 0x00020000, 0x00010000,   
   0x00008000, 0x00004000, 0x00002000, 0x00001000,   
   0x00000800, 0x00000400, 0x00000200, 0x00000100,  
   0x00000080, 0x00000040, 0x00000020, 0x00000010,  
   0x00000008, 0x00000004, 0x00000002, 0x00000001
};

#define	AIMTYPE		0
#define	AOMTYPE		1
#define	BIMTYPE		2
#define	BOMTYPE		3
#define	MBBIMTYPE	4
#define	MBBOMTYPE	5
#define	PTOMTYPE	6
#define	TDMTYPE		7
#define	PATMTYPE	8

#define	AISEED		0
#define	AOSEED		4
#define	BISEED		6
#define	BOSEED		12
#define	MBBISEED	18
#define	MBBOSEED	24
#define	PTOSEED		30
#define	TDSEED		36
#define	PATSEED		42

MODMAP	QtmModMap[] = {	{AIMTYPE,ai_ACI_030_AI,0x102}, /* 0 */
			{AIMTYPE,ai_AVI_030_AI,0x106}, /* 1 */
			{AIMTYPE,ai_ATI_030_AI,0x107}, /* 2 */
			{AIMTYPE,ai_ARI_030_AI,0x10a}, /* 3 */
			{AOMTYPE,ao_ACO_020_AO,0x103}, /* 4 */
			{AOMTYPE,ao_AVO_020_AO,0x104}, /* 5 */
			{BIMTYPE,bi_DAI_540_BI,0x00f}, /* 6 */
			{BIMTYPE,bi_DAI_553_BI,0x003}, /* 7 */
			{BIMTYPE,bi_DAI_740_BI,0x00b}, /* 8 */
			{BIMTYPE,bi_DDI_353_BI,0x006}, /* 9 */
			{BIMTYPE,bi_DDI_841_BI,0x012}, /* 10 */
			{BIMTYPE,bi_DDI_853_BI,0x011}, /* 11 */
			{BOMTYPE,bo_DAO_840_BO,0x002}, /* 12 */
			{BOMTYPE,bo_DAO_842_BO,0x005}, /* 13 */
			{BOMTYPE,bo_DDO_353_BO,0x004}, /* 14 */
			{BOMTYPE,bo_DDO_843_BO,0x010}, /* 15 */
			{BOMTYPE,bo_DRA_840_BO,0x00c}, /* 16 */
			{BOMTYPE,bo_DRC_830_BO,0x008}, /* 17 */
			{MBBIMTYPE,mbbi_DAI_540_MBBI,0x00f}, /* 18 */
			{MBBIMTYPE,mbbi_DAI_553_MBBI,0x003}, /* 19 */
			{MBBIMTYPE,mbbi_DAI_740_MBBI,0x00b}, /* 20 */
			{MBBIMTYPE,mbbi_DDI_353_MBBI,0x006}, /* 21 */
			{MBBIMTYPE,mbbi_DDI_841_MBBI,0x012}, /* 22 */
			{MBBIMTYPE,mbbi_DDI_853_MBBI,0x011}, /* 23 */
			{MBBOMTYPE,mbbo_DAO_840_MBBO,0x002}, /* 24 */
			{MBBOMTYPE,mbbo_DAO_842_MBBO,0x005}, /* 25 */
			{MBBOMTYPE,mbbo_DDO_353_MBBO,0x004}, /* 26 */
			{MBBOMTYPE,mbbo_DDO_843_MBBO,0x010}, /* 27 */
			{MBBOMTYPE,mbbo_DRA_840_MBBO,0x00c}, /* 28 */
			{MBBOMTYPE,mbbo_DRC_830_MBBO,0x008}, /* 29 */
			{PTOMTYPE,pto_DAO_840_BO,0x002}, /* 30 */
			{PTOMTYPE,pto_DAO_842_BO,0x005}, /* 31 */
			{PTOMTYPE,pto_DDO_353_BO,0x004}, /* 32 */
			{PTOMTYPE,pto_DDO_843_BO,0x010}, /* 33 */
			{PTOMTYPE,pto_DRA_840_BO,0x00c}, /* 34 */
			{PTOMTYPE,pto_DRC_830_BO,0x008}, /* 35 */
			{TDMTYPE,td_DAO_840_BO,0x002}, /* 36 */
			{TDMTYPE,td_DAO_842_BO,0x005}, /* 37 */
			{TDMTYPE,td_DDO_353_BO,0x004}, /* 38 */
			{TDMTYPE,td_DDO_843_BO,0x010}, /* 39 */
			{TDMTYPE,td_DRA_840_BO,0x00c}, /* 40 */
			{TDMTYPE,td_DRC_830_BO,0x008}, /* 41 */
			{PATMTYPE,pat_DAO_840_BO,0x002}, /* 42 */
			{PATMTYPE,pat_DAO_842_BO,0x005}, /* 43 */
			{PATMTYPE,pat_DDO_353_BO,0x004}, /* 44 */
			{PATMTYPE,pat_DDO_843_BO,0x010}, /* 45 */
			{PATMTYPE,pat_DRA_840_BO,0x00c}, /* 46 */
			{PATMTYPE,pat_DRC_830_BO,0x008}, /* 47 */
			{PATMTYPE,pat_DAI_540_BI,0x00f}, /* 48 */
			{PATMTYPE,pat_DAI_553_BI,0x003}, /* 49 */
			{PATMTYPE,pat_DAI_740_BI,0x00b}, /* 50 */
			{PATMTYPE,pat_DDI_353_BI,0x006}, /* 51 */
			{PATMTYPE,pat_DDI_841_BI,0x012}, /* 52 */
			{PATMTYPE,pat_DDI_853_BI,0x011}, /* 53 */
			{PATMTYPE,pat_ACI_030_AI,0x102}, /* 54 */
			{PATMTYPE,pat_AVI_030_AI,0x106}, /* 55 */
			{PATMTYPE,pat_ATI_030_AI,0x107}, /* 56 */
			{PATMTYPE,pat_ARI_030_AI,0x10a}, /* 57 */
			{-1,0,0} };

QtmBdtypeToId(unsigned short ptype, unsigned short devtype,
		  unsigned short seed,  unsigned short i);




/********************************** AI ************************************/

static long ai_init_record (pai)
   struct   aiRecord *pai;
{
   struct	tateio	*ptateio;
   char			message[128];
   unsigned	short	value;
   int			bdid,status;

   union u_value {
      short		ivalue;
      long		lvalue;
      float		fvalue;
      double		dvalue;
      char		afvalue[100];
  }  uval;

   switch (pai->inp.type)
   {

      case (IO_TYPE):
         break;

      default:
         strcpy (message, pai->name);
         strcat (message, ": devAi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }

      /* call driver so that it configures card */

   ptateio = (struct tateio *) &(pai->inp.value);

   bdid = QtmBdtypeToId(AIMTYPE, pai->dtyp, AISEED, 0xffff);
 
   status = AIDRIVER (pai->name, ptateio->link, ptateio->adapter,
			ptateio->card, ptateio->signal, &uval,
			pai->linr, pai->gain, bdid, pai->resn);

   if (status == 0)
   {
      switch(pai->linr)
      {
         case (CNVINTEGER):
            pai->val = uval.ivalue;
            break;

         case (CNVDOUBLEINT):
            pai->val = uval.lvalue;
            break;

         case (CNVFLOAT):
            pai->val = uval.fvalue;
            break;

         case (CNVDOUBLEFLT):
            pai->val = uval.dvalue;
            break;

         case (CNVASCIIFLT):
            status = -1;

         case NOCONV:
         case LINEAR:
	 case CNVSQRROOT:
         default:
            pai->rval = uval.ivalue;
            break;
       }
   }

   return (status);
}

static long read_ai (pai)
   struct aiRecord   *pai;
{
   struct	tateio	*ptateio;
   int			status;
   unsigned	short	value,bdid;

   union u_value {
      short		ivalue;
      long		lvalue;
      float		fvalue;
      double	dvalue;
      char		afvalue [100];
  }  uval;

   ptateio = (struct tateio *) &(pai->inp.value);

   bdid = QtmBdtypeToId(AIMTYPE, pai->dtyp, AISEED, 0xffff);

   status = AIDRIVER (pai->name,ptateio->link, ptateio->adapter,
			ptateio->card, ptateio->signal, &uval, pai->linr,
			pai->gain, bdid, 0);

   /* setting status to 2 tells ai's process() not to convert value */

   if (status == 0)
   {
      switch(pai->linr)
      {
         case (CNVINTEGER):
            pai->val = uval.ivalue;
            status = 2;
         break;

         case (CNVDOUBLEINT):
            status = -1;
            break;

         case (CNVFLOAT):
            status = -1;
            break;

         case (CNVDOUBLEFLT):
            status = -1;
            break;

         case (CNVASCIIFLT):
            status = -1;

         case NOCONV:
         case LINEAR:
	 case CNVSQRROOT:
         default:
            pai->rval = uval.ivalue;
            break;
       }
   }

   if (status == -1)
   {
      if ((pai->aenb != ALARM_DISABLED) && (pai->nsev < VALID_ALARM))
      {
         pai->nsta = READ_ALARM;
         pai->nsev = VALID_ALARM;
      }
      status = 2;         /*dont convert*/
   }
   return (status);
}


/*********************************** AO **********************************/

static long ao_init_record (pao)
   struct   aoRecord *pao;
{
   struct	tateio	*ptateio;
   char			message[128];
   short		value;

   switch (pao->out.type)
   {
      case (IO_TYPE):
         break;

      default:
         strcpy (message, pao->name);
         strcat (message, ": devAo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField,message);
         return (S_db_badField);
   }

      /* call driver so that it configures card */

   return (read_ao (pao));
}

static long write_ao (pao)
   struct aoRecord   *pao;
{
   struct   tateio   *ptateio;
   int               status;
   int               value,bdid;
   double            dvalue;


   ptateio = (struct tateio *) &(pao->out.value);
   dvalue = pao->oval;
   value = pao->rval;

   bdid = QtmBdtypeToId(AOMTYPE, pao->dtyp, AOSEED, 0xffff);

   status = AODRIVER (pao->name, ptateio->link, ptateio->adapter,ptateio->card,
                      ptateio->signal, dvalue, value, pao->linr, bdid, 0);
   if (status == -1)
   {
      if((pao->aenb != ALARM_DISABLED) && (pao->nsev < VALID_ALARM))
      {
         pao->nsta = WRITE_ALARM;
         pao->nsev = VALID_ALARM;
      }
   }
   else if (status == -2)
   {
      status = 0;
      if (pao->nsev < VALID_ALARM )
      {
         pao->nsta = HW_LIMIT_ALARM;
         pao->nsev = VALID_ALARM;
      }
   }
   return (status);
}


static int read_ao (pao)
   struct   aoRecord *pao;
{
   struct   tateio   *ptateio = &pao->out.value.tateio;
   int               status,bdid;

   union u_value {
      short    ivalue;
      long     lvalue;
      float    fvalue;
      double   dvalue;
      char     afvalue[100];
   }  uval;

      /* get the value from the ao driver */
   bdid = QtmBdtypeToId(AOMTYPE, pao->dtyp, AOSEED, 0xffff);

   status = AOREAD (pao->name, ptateio->link, ptateio->adapter, ptateio->card,
                    ptateio->signal, &uval, pao->linr, bdid, pao->resn);

   if (status == 0)
   {
      switch (pao->linr)
      {
         case ASCFLT:   /* ascii float conversion */
            pao -> val = uval.fvalue;
            break;

         case NOCONV:   /* no conversion */
         case LINEAR:   /* linear conversion */
         default:
            pao -> rval = uval.ivalue;
            break;
      }
   }
   return (status);
}

/*********************************** BI ***********************************/

static long bi_init_record (pbi)
   struct   biRecord *pbi;
{
   struct   tateio   *ptateio;
   char              message[128];
   int               bdid,status = 0;
   unsigned long     value;

   switch (pbi->inp.type)
   {
      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pbi->inp.value);
         pbi->mask = rmasks[((ptateio->signal - 1) % 32)];

	   bdid = QtmBdtypeToId(BIMTYPE, pbi->dtyp, BISEED, 0xffff);

         status = BIDRIVER (pbi->name, ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, pbi->mask, &value,
                            bdid, &pbi->resn [0]);
	 if (!status)
	    pbi -> rval = value;

         break;

      default :
         strcpy (message, pbi->name);
         strcat (message, ": devBi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static long read_bi(pbi)
   struct   biRecord *pbi;
{
   struct   tateio   *ptateio;
   int               status,bdid;
   long              value;

   ptateio = (struct tateio *) &(pbi->inp.value);

   bdid = QtmBdtypeToId(BIMTYPE, pbi->dtyp, BISEED, 0xffff);

   status = BIDRIVER (pbi->name,ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, pbi->mask, &value, bdid, 0);
   if (status == 0)
   {
      pbi->rval = value;
      return (status);
   }
   else if (status == -1)
   {
      if ((pbi->aenb != ALARM_DISABLED) && (pbi->nsev < VALID_ALARM))
      {
         pbi->nsta = READ_ALARM;
         pbi->nsev = VALID_ALARM;
      }
      return (2);
   }
}


/*********************************** BO ***********************************/

static long bo_init_record (pbo)
   struct   boRecord *pbo;
{
   struct   tateio   *ptateio;
   char              message[128];
   unsigned long     value;
   int               status = 0;
   int		     bdid;

   switch (pbo->out.type)
   {
      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pbo->out.value);

           /* read the value via the modbus driver */

	 bdid = QtmBdtypeToId(BOMTYPE, pbo->dtyp, BOSEED, 0xffff);
         pbo->mask = rmasks [((ptateio->signal - 1) % 32)];
         status = BOREAD (pbo->name,ptateio->link, ptateio->adapter, 
                          ptateio->card, ptateio->signal, &value, pbo->mask, 
                          bdid, &pbo->resn [0]);
         if(status == 0)
         {
            pbo->rval = value;
         }
         break;

      default :
         strcpy (message, pbo->name);
         strcat (message, ": devBo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static long write_bo (pbo)
   struct   boRecord *pbo;
{
   struct   tateio   *ptateio;
   int               status,bdid;

   ptateio = (struct tateio *) &(pbo->out.value);

   bdid = QtmBdtypeToId(BOMTYPE, pbo->dtyp, BOSEED, 0xffff);

   status = BODRIVER (pbo->name,ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, pbo->rval, pbo->mask, bdid, 0);

   if (status != 0)
   {
      if ((pbo -> aenb != ALARM_DISABLED) && (pbo -> nsev < VALID_ALARM))
      {
         pbo->nsta = WRITE_ALARM;
         pbo->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/*********************************** MBBI **********************************/

static long mbbi_init_record (pmbbi)
   struct   mbbiRecord  *pmbbi;
{
   struct   tateio   *ptateio;
   char              message[128];
   int               bdid,status = 0;
   unsigned long     value;
   unsigned short	ib,ob,i;
   unsigned long	nvalue;


   switch (pmbbi->inp.type)
   {
      case (IO_TYPE) :
         ptateio = (struct tateio *) &(pmbbi->inp.value);
         pmbbi->shft = ((pmbbi->inp.value.tateio.signal-1) % 32);
         pmbbi->mask <<= ((32-pmbbi->shft)-pmbbi->nobt);

	   bdid = QtmBdtypeToId(MBBIMTYPE, pmbbi->dtyp, MBBISEED, 0xffff);

         status = BIDRIVER (pmbbi->name,ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, pmbbi->mask, &value,
                            bdid, &pmbbi->resn [0]);
	 if (!status)
             /* reverse the bits */
	     nvalue = 0;
	     ib = 32-pmbbi->shft-1;
	     ob = ib-(pmbbi->nobt-1);
	     for (i=0;(i<pmbbi->nobt);i++) {
	         if (value & masks[ib--]) {
		     nvalue |= masks[ob++];
		 } else {
		    ob++;
		 }
	     }
      	     /* Adjust the value to the should have been bit position */
      	     nvalue >>= ((32-pmbbi->shft)-pmbbi->nobt);
      	     nvalue <<= pmbbi->shft;
	     pmbbi -> rval = nvalue;

         break;

      default :
         strcpy (message, pmbbi->name);
         strcat (message, ": devMbbi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}

static   long   read_mbbi (pmbbi)
   struct   mbbiRecord  *pmbbi;
{
   struct   tateio   *ptateio;
   int               bdid,status = 0;
   unsigned long     value;
   unsigned short	ib,ob,i;
   unsigned long	nvalue;


   ptateio = (struct tateio *) &(pmbbi->inp.value);

   bdid = QtmBdtypeToId(MBBIMTYPE, pmbbi->dtyp, MBBISEED, 0xffff);

   status = BIDRIVER (pmbbi->name,ptateio->link, ptateio->adapter, 
                      ptateio->card, ptateio->signal, pmbbi->mask, &value,
		          bdid, 0);
   if (status == 0)
   {
      /* reverse the bits */
      nvalue = 0;
      ib = 32-pmbbi->shft-1;
      ob = ib-(pmbbi->nobt-1);
      for (i=0;(i<pmbbi->nobt);i++) {
          if (value & masks[ib--]) {
 	     nvalue |= masks[ob++];
	  } else {
	     ob++;
	  }
      }
      /* Adjust the value to the should have been bit position */
      nvalue >>= ((32-pmbbi->shft)-pmbbi->nobt);
      nvalue <<= pmbbi->shft;
      pmbbi->rval = nvalue;
   }
   else
   {
      if ((pmbbi -> aenb != ALARM_DISABLED) && (pmbbi->nsev < VALID_ALARM))
      {
         pmbbi->nsta = READ_ALARM;
         pmbbi->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/********************************** MBBO *********************************/

static long mbbo_init_record   (pmbbo)
   struct   mbboRecord  *pmbbo;
{
   char			message[128];
   unsigned   long      value;
   struct   tateio      *ptateio;
   int			bdid,status = 0;
   unsigned short	ib,ob,i;
   unsigned long	nvalue;

   switch (pmbbo->out.type)
   {
      case (IO_TYPE) :
         ptateio = &(pmbbo->out.value.tateio);
         pmbbo->shft = ((ptateio->signal-1) % 32);
         pmbbo->mask <<= ((32-pmbbo->shft)-pmbbo->nobt);

	   bdid = QtmBdtypeToId(MBBOMTYPE, pmbbo->dtyp, MBBOSEED, 0xffff);

         status = BOREAD (pmbbo->name,ptateio->link, ptateio->adapter, 
                          ptateio->card, ptateio->signal, &value, pmbbo->mask,
                          bdid, &pmbbo->resn [0]);
         if (status == 0)
	    /* reverse the bits */
	    nvalue = 0;
            ib = 32-pmbbo->shft-1;
            ob = ib-(pmbbo->nobt-1);
	    for (i=0;(i<pmbbo->nobt);i++) {
	        if (value & masks[ib--]) {
		    nvalue |= masks[ob++];
	  	} else {
		    ob++;
		}
	    }
	    /* Adjust the value to the should have been bit position */
	    nvalue >>= ((32-pmbbo->shft)-pmbbo->nobt);
	    nvalue <<= pmbbo->shft;
            pmbbo->rval = nvalue;
         break;

      default :
         strcpy (message, pmbbo->name);
         strcat (message, ": devMbbo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return (status);
}


static long write_mbbo(pmbbo)
   struct   mbboRecord  *pmbbo;
{
   struct   tateio   *ptateio;
   int               bdid,status;
   unsigned long     value;
   unsigned short	ib,ob,i;
   unsigned long	nvalue;


   ptateio = &(pmbbo->out.value.tateio);

   bdid = QtmBdtypeToId(MBBOMTYPE, pmbbo->dtyp, MBBOSEED, 0xffff);

   value = pmbbo->rval;

   /* reverse the bits */
   nvalue = 0;
   ib = pmbbo->shft;
   ob = pmbbo->shft+pmbbo->nobt-1;
   for (i=0;(i<pmbbo->nobt);i++) {
       if (value & masks[ib++]) {
	   nvalue |= masks[ob--];
       } else {
	   ob--;
       }
   }

   /* Adjust the value to the should have been bit position */
   nvalue >>= pmbbo->shft;
   nvalue <<= ((32-pmbbo->shft)-pmbbo->nobt);


   status = BODRIVER (pmbbo->name,ptateio->link, ptateio->adapter, 
                      ptateio->card, ptateio->signal, nvalue, 
                      pmbbo->mask, bdid, 0);
   if(status == -1)
   {
      if ((pmbbo->aenb != ALARM_DISABLED) && (pmbbo -> nsev < VALID_ALARM))
      {
         pmbbo->nsta = WRITE_ALARM;
         pmbbo->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/************************************ PTO *********************************/

static long pto_init_record (ppto, bit_no)
   struct   ptoRecord   *ppto;
   unsigned int   bit_no;
{
   struct   tateio   *ptateio;
   char              message[128];
   unsigned long     value;
   int               bdid,status = 0;

   switch (bit_no)
   {

      case 0:
         switch (ppto->pout.type)
         {
            case (IO_TYPE) :
               ptateio = (struct tateio *) &(ppto->pout.value);

                  /* read the value via the modbus driver */

               ppto->pmsk = rmasks [((ptateio->signal - 1) % 32)];

		   bdid = QtmBdtypeToId(PTOMTYPE, ppto->dtyp, PTOSEED, 0xffff);

               status = BOREAD (ppto->name,ptateio->link, ptateio->adapter, 
                                ptateio->card, ptateio->signal, &value, 
                                ppto->pmsk,  bdid, &ppto->resn [0]);
	       if (!status)
		   ppto -> rbv = value;
               break;

            default :
               strcpy (message, ppto->name);
               strcat (message, ": devPto");
               strcat (message, io_name);
               strcat (message, " (init_record) Illegal POS OUT field");
               errMessage (S_db_badField, message);
               status = S_db_badField;
	       break;
         }
	 break;

      case 1:
         switch (ppto->nout.type)
         {
            case (IO_TYPE) :
               ptateio = (struct tateio *) &(ppto->nout.value);

                  /* read the value via the Allen Bradley driver */

               ppto->nmsk = rmasks [((ptateio->signal - 1) % 32)];

		   bdid = QtmBdtypeToId(PTOMTYPE, ppto->dtyp, PTOSEED, 0xffff);

               status = BOREAD (ppto->name,ptateio->link, ptateio->adapter, 
                                ptateio->card, ptateio->signal, &value, 
                                ppto->nmsk,  bdid, &ppto->resn [0]);
	       if (!status)
		   ppto -> rbv = value;
               break;

            default :
               strcpy (message, ppto->name);
               strcat (message, ": devPto");
               strcat (message, io_name);
               strcat (message, " (init_record) Illegal NEG OUT field");
               errMessage (S_db_badField, message);
               status = S_db_badField;
	       break;
         }
	 break;

      default:
         strcpy (message, ppto->name);
         strcat (message, ": devPto");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal bit_no");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;

   } /* end switch bit_no */

   return (status);
}

static long write_pto (ppto, bit_no)
   struct   ptoRecord   *ppto;
   unsigned int         bit_no;
{
   struct   tateio   *ptateio;
   int               bdid,status;
   char              message[128];

   switch (bit_no)
   {
      case 0:
         ptateio = (struct tateio *) &(ppto->pout.value);

	   bdid = QtmBdtypeToId(PTOMTYPE, ppto->dtyp, PTOSEED, 0xffff);

         status = BODRIVER (ppto->name,ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, ppto->rbv, 
                            ppto->pmsk, bdid, 0);
          break;

      case 1:
         ptateio = (struct tateio *) &(ppto->nout.value);

	   bdid = QtmBdtypeToId(PTOMTYPE, ppto->dtyp, PTOSEED, 0xffff);

         status = BODRIVER (ppto->name,ptateio->link, ptateio->adapter, 
                            ptateio->card, ptateio->signal, ppto->rbv, 
                            ppto->nmsk,  bdid, 0);
         break;

      default:
         strcpy (message, ppto->name);
         strcat (message, ": devPto");
         strcat (message, io_name);
         strcat (message, " (write_pto) Illegal bit_no");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }

   if (status == -1)
   {
      if ((ppto->aenb != ALARM_DISABLED) && (ppto->nsev < VALID_ALARM))
      {
         ppto->nsta = WRITE_ALARM;
         ppto->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/************************************ TD **********************************/

static long td_init_record (ptd)
   struct   tdRecord *ptd;
{
   struct   tateio   *ptateio;
   char              message[128];
   unsigned long     value;
   int               bdid,status = 0;

   switch (ptd->out.type)
   {
      case (IO_TYPE) :
         ptateio = (struct tateio *) &(ptd->out.value);

           /* read the value via the modbus driver */

         ptd->mask = rmasks [((ptateio->signal - 1) % 32)];

	   bdid = QtmBdtypeToId(TDMTYPE, ptd->dtyp, TDSEED, 0xffff);

         status = BOREAD (ptd->name,ptateio->link, ptateio->adapter, 
                          ptateio->card, ptateio->signal, &value, ptd->mask, 
                          bdid, &ptd->resn [0]);
         if(status == 0)
         {
            ptd->rval = value;
         }
         break;

      default :
         strcpy (message, ptd->name);
         strcat (message, ": devTd");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
   }
   return (status);
}

static long write_td (ptd)
   struct   tdRecord *ptd;
{
   struct   tateio   *ptateio;
   int               bdid,status;

   ptateio = (struct tateio *) &(ptd->out.value);

   bdid = QtmBdtypeToId(TDMTYPE, ptd->dtyp, TDSEED, 0xffff);

   status = BODRIVER (ptd->name,ptateio->link, ptateio->adapter, ptateio->card,
                      ptateio->signal, ptd->rval, ptd->mask,  bdid, 0);
   if (status == -1)
   {
      if ((ptd->aenb != ALARM_DISABLED) && (ptd->nsev < VALID_ALARM))
      {
         ptd->nsta = WRITE_ALARM;
         ptd->nsev = VALID_ALARM;
      }
   }
   return (status);
}


/********************************** PAT ************************************/

static long pat_init_record (ppat, which_one)
    struct patRecord	*ppat;
    int			which_one;
{
    char		message[128];
    unsigned long	value;
    unsigned short	inp_value;
    struct tateio	*ptateio;
    int			bdid,status = 0;

    switch (which_one)
    {
	case 0:
	    switch (ppat->pout.type)
	    {
		case (IO_TYPE) :
		    ptateio = (struct tateio *) &(ppat->pout.value);

		    /* read the value via the driver */

		    ppat->pmsk = rmasks [((ptateio->signal - 1) % 32)];

		    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyp, PATSEED, 0xffff);

		    status = BOREAD (ppat->name, ptateio->link, ptateio->adapter,
			             ptateio->card, ptateio->signal, &value,
			             ppat->pmsk,  bdid, &ppat->resn [0]);
		    break;

		default :
		    strcpy (message, ppat->name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, " (pat_init_record) Illegal POS OUT field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
	    }
	    break;

	case 1:
	    switch (ppat->nout.type)
	    {
		case (IO_TYPE) :
		    ptateio = (struct tateio *) &(ppat->nout.value);

		    /* read the value via the driver */

		    ppat->nmsk = rmasks [((ptateio->signal - 1) % 32)];

		    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyp, PATSEED, 0xffff);

		    status = BOREAD (ppat->name, ptateio->link, ptateio->adapter,
			             ptateio->card, ptateio->signal, &value,
			             ppat->nmsk, bdid, &ppat->resn [0]);
		    break;

		default :
		    strcpy (message, ppat->name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, " (pat_init_record) Illegal NEG OUT field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
	    }
	    break;

	case 2:
	    switch (ppat -> rlin.type)
	    {
		case (IO_TYPE) :
		    ptateio = (struct tateio *) &(ppat->rlin.value);
		    ppat->rmsk = rmasks[((ptateio->signal - 1) % 32)];

		    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyr, PATSEED, 0xffff);

		    status = BIDRIVER (ppat->name, ptateio->link, ptateio->adapter, 
				       ptateio->card, ptateio->signal, ppat->rmsk, 
				       &value, bdid, &ppat->resn [0]);
		    if (!status)
			ppat -> rlrv = value;
		    break;

		default :
		    strcpy (message, ppat->name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, " (pat_init_record) Illegal RLIN field");
		    errMessage (S_db_badField, message);
		    status =  S_db_badField;
		    break;
	    }
	    break;

	case 3:
	    switch (ppat->inp.type)
	    {
		case (IO_TYPE):
		    /* call driver so that it configures card */

		    ptateio = (struct tateio *) &(ppat->inp.value);

		    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyf, PATSEED, 0xffff);

		    status = AIDRIVER (ppat->name, ptateio->link, ptateio->adapter, 
				       ptateio->card, ptateio->signal, &inp_value, 
				       0, 0, bdid, ppat->resn);
		    if (!status)
			ppat -> irvl = inp_value;
		    break;

		default:
		    strcpy (message, ppat->name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, " (pat_init_record) Illegal INP field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }

	    break;

	default:
	    strcpy (message, ppat->name);
	    strcat (message, ": devPat");
	    strcat (message, io_name);
	    strcat (message, " (pat_init_record) Illegal type");
	    errMessage (S_db_badField, message);
	    status = S_db_badField;

    } /* end switch bit_no */

     return (status);
}

static long write_pat (ppat, which_one)
    struct patRecord	*ppat;
    int			which_one;
{
    struct tateio	*ptateio;
    int			bdid,status = 0;
    char		message[128];

    switch (which_one)
    {
	case 0:
	    ptateio = (struct tateio *) &(ppat->pout.value);

	    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyp, PATSEED, 0xffff);

	    status = BODRIVER (ppat->name, ptateio->link, ptateio->adapter,
			       ptateio->card, ptateio->signal, ppat->rbv,
			       ppat->pmsk, bdid , 0);
	    break;

	case 1:
	    ptateio = (struct tateio *) &(ppat->nout.value);

	    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyp, PATSEED, 0xffff);

	    status = BODRIVER (ppat->name, ptateio->link, ptateio->adapter,
			       ptateio->card, ptateio->signal, ppat->rbv,
			       ppat->nmsk, bdid , 0);
	    break;

	default:
	    strcpy (message, ppat->name);
	    strcat (message, ": devPat");
	    strcat (message, io_name);
	    strcat (message, " (write_pat) Illegal which_one");
	    errMessage (S_db_badField, message);
	    status = S_db_badField;
	    break;
    }

    if (status != 0)
    {
	if (ppat -> nsev < VALID_ALARM)
	{
	    ppat->nsta = WRITE_ALARM;
	    ppat->nsev = VALID_ALARM;
	}
    }

   return (status);
}

static long read_pat (ppat, which_one)
    struct patRecord	*ppat;
    int			which_one;
{
    struct tateio	*ptateio;
    int			bdid,status;
    char		message[128];
    unsigned long	value;
    unsigned short	inp_value;

    switch (which_one)
    {
	case 2:
	    ptateio = (struct tateio *) &(ppat->rlin.value);

	    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyr, PATSEED, 0xffff);

	    status = BIDRIVER (ppat->name, ptateio->link, ptateio->adapter,
			       ptateio->card, ptateio->signal, ppat->rmsk, 
			       &value, bdid, 0);
	    if (status == 0)
		ppat->rlrv = value;
	    else
	    {
		if (ppat -> nsev < VALID_ALARM)
		{
		    ppat -> nsev = VALID_ALARM;
		    ppat -> nsev = READ_ALARM;
		}
	    }
	    break;

	case 3:
	    ptateio = (struct tateio *) &(ppat->inp.value);

	    bdid = QtmBdtypeToId(PATMTYPE, ppat->dtyf, PATSEED, 0xffff);

	    status = AIDRIVER (ppat->name, ptateio->link, ptateio->adapter, 
			       ptateio->card, ptateio->signal, &inp_value, 
			       0,0, bdid, 0);
	    if ((status == 0) || (status == -2))
	    {
		ppat->irvl = inp_value;
	    }
	    if (status == -1)
	    {
		if (ppat -> nsev < VALID_ALARM)
		{
		    ppat -> nsev = VALID_ALARM;
		    ppat -> nsev = READ_ALARM;
		}
	    }
	    break;

	default:
	    strcpy (message, ppat->name);
	    strcat (message, ": devPat");
	    strcat (message, io_name);
	    strcat (message, " (read_pat) Illegal which_one");
	    errMessage (S_db_badField, message);
	    status = S_db_badField;
	    break;
    }

    return (status);
}

/*
**
** QtmBdtypeToId
**	maps the device type in the record to the Qtm board Id
**
*/
QtmBdtypeToId(unsigned short ptype, unsigned short devtype,
		  unsigned short seed,  unsigned short i)
    {

	unsigned short	j;

	/* Search for the point type and device type to map to board params */
	j = seed;
	while (QtmModMap[j].ptype != -1) {
	    if ((QtmModMap[j].ptype == ptype) && (QtmModMap[j].ptbdtype == devtype))
		    break;
	    j++;
	}
	if (QtmModMap[j].ptype == -1) {
	    logMsg ("Invalid point device type in pg structure %s,%d, at index %d\n",
		ptype,
		devtype,
		i );
	    return(-1);
	}
	return (QtmModMap[j].modbdtype);
    }
