/*drvQ984.h,v 1.1.1.1 1999/08/03 14:44:14 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*
**	File:	      drvQ984.h
**	Auth:	      John L. Dalesio
**	Create: 08/25/95  JLD
**
**      Edit:
**              09/04/97  wep : added support for Resource Group Lock function
**
**	This file contains the data structures used by the AB SCANNER
**	I/O driver.
**
*/

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR		0x0d

/*      number of groups which can be defined in the PCU/LGU */
#define         MAXGROUPNUM     16

#define         RES_LOCKED       0


/* S908 network limits */
#define MAX_DROPS       32      /* drop numbers are 1 to 32				*/
#define MAX_RACKS	5       /* rack numbers are 1 to 5				*/
#define MAX_QRACKS	4       /* rack numbers are 1 to 4 for quantum			*/
#define MAX_SLOTS	11      /* slot numbers are 1 to 11				*/
#define MAX_QSLOTS      16      /* Quantum drop can have up to 16 modules		*/
#define HEALTH_LENGTH   10      /* there are 10 bytes of HEALTH data per drop		*/

#define MAX_OFFSET      0xff00  /* maximum value for data offsets			*/



/*	number of possible scanner cards */
#define		LINES	4	
/*	number of possible adapters per card */
#define		NUMDROPS	MAX_DROPS
#define		NUMRACKS	MAX_RACKS 
#define		NUMSLOTS	MAX_SLOTS
#define		MAXDROP		NUMDROPS
#define		MAXRACK		NUMRACKS 
#define		MAXCARD		NUMSLOTS
#define		NUMCARDS	NUMSLOTS


/*
**	The I/O is address by 
**	Line 	(scanner 0-3)
**	Address	(Drop 0-31)
**	Card	(Rack 0-5 *100)+ Slot (1-16)
**	Signal (1-32)
**
**	The inputs and outputs are memory mapped
**
*/

#define QTMSCANNER	9999		/* Address for accessing scanner status */
#define QTMADAPTER	9998		/* Address for accessing drop status */
#define QTMCARD		9997		/* Address for accessing module status */


/* The number of 100 ms ticks that the outputs will hold value without comm */
/* This should not matter for Quantum I/O */
#define HOLDUPTIME	100
#define FAILTIME	10
#define RESETTIME	10800		/* 3 minutes */


/*
**	Communications I/O Structures
**
*/


#define		MAXRXMSG	256

typedef	struct {
	unsigned short		protocol;
	unsigned short		rxstate;
	unsigned short		rxtocnt;
	unsigned short		rxhwerrcnt;
	unsigned short		fixedlen;
	unsigned short		rxmsglen;
	unsigned short		rxix;
	SEMAPHORE		*semid;
	unsigned short		*status;
	unsigned char		rxbuf[MAXRXMSG];
		} 	RXLINE;


#define		MAXTXMSG	256	

typedef	struct {
	unsigned short		protocol;
	unsigned short		txdisable;
	unsigned short		txtocnt;
	unsigned short		txhwerrcnt;
	unsigned short		txmsglen;
	unsigned short		txix;
	SEMAPHORE		*semid;
	unsigned short		*status;
	unsigned char		txbuf[MAXTXMSG];
		} 	TXLINE;

/*
**	Communications I/O Constants
**
*/

#define		RXGOOD			1
#define		RXBAD				2
#define		RXTIMEOUT			10	/* 16 ms ticks */
#define		ACTIVITYTIME		60	/* must see activity in 1 second */
#define		OUTPUTTIME			3	/* output ckeck 20 times per second */
#define		DROPSTATFETCHTIME		900
#define		REDUNDOUTUPDATETIME	60

#define		VARRESP			FALSE
#define		FIXEDRESP			TRUE
#define		ONERETRY			0x8000	/* ored into line number for xfermsg */

#define		TXGOOD			1
#define		TXBAD				2

#define		TXTIMEOUT			2	/* 16 ms ticks */

#define		IO_TIMEOUT			3	/* Code returned by timer */

#define		MAXRETRYS			3	/* retransmission trys */

#define		MAXCFGCK			8	/* Try to Bring online each cycle */
#define		BRINGBACKCNT		4	/* Poll loops per cycle */

#define		XFERGOOD			1
#define		XFERBAD			2

#define		RESNAMELEN			16	/* length of a resource name */
#define		SLWWINDOW			600	/* # of ticks to allow slave to try */



/*
* Output Que Management Structures for QTMSCANNER Protocol
*/
#define         AOUTTYPE        		1
#define         BOUTTYPE        		2
#define         OUTQSIZE        		128

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	short           aovalue;
		} AOUT, *AOUTPTR;

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	long            bomask;
	long            bovalue;
		} BOUT, *BOUTPTR;

typedef struct {
		unsigned short  otype;
		union   {
			AOUT    aonode;
			BOUT    bonode;
			}out;
	       } OUTQENT;

typedef struct {
		unsigned short	ooutix;
		unsigned short	oinix;
		OUTQENT         	outque[OUTQSIZE];
		} OUTPUTQUE;


#define MAX_QUANTUM_PTS		2000

typedef struct
    {
    char	*pointType;	/* "ai", "bi", etc. */
    short	devType;	/* from records dtyp field */
    short	line;
    short	addr;
    short	card;
    short	signal;
    short	gain;		/* Only meaningful for "ai" */
    } GAIN_TABLE;


/* drop status bits		*/
#define ONLINE		0x01	/* set by QRIO when packets are detected from this drop */
#define BADCONFIG		0x02	/* set by QRIO when configure command does not match network config */
#define DATA_CHANGE     0x04	/* set by QRIO when the PLC changes this REAL drop's outputs (cleared by host) */
#define CFG_VALID       0x08	/* set by QRIO to indicate config has been verified with the PLC */
                            	/* or set by host to force operation without verifying config with PLC */
#define TIMEOUT         0x10	/* set by QRIO when drop->activity counter reaches zero */
#define ISQUANTUM       0x20	/* set by QRIO if this drop is a QUANTUM RIO drop (cleared for 800 IO) */
#define STAT_CHANGE     0x40	/* set by QRIO when this REAL drop's status changes */
#define TIMEOUT_IRQ     0x80	/* set by QRIO when this drop's TIMEOUT state changes, if IRQ is enabled. */

/* drop control bits */
#define VIRTUAL         0x01    /* set by host if QRIO is to emulate this drop */
#define CHG_IRQ         0x02    /* set by host if an interrupt is to be generated when this REAL drop's */
                                /* outputs have been changed by the PLC */
#define DATA_CHANGE 0x04        /* set by host after changing data for this drop */
#define CFG_VALID   0x08        /* set by host to force use of an old configuration offsets */
#define STAT_CHANGE 0x40        /* set by host after changing status for this drop */
#define TIMEOUT_IRQ 0x80        /* set by host to enable interrupt on drop timeout/return */

/* ()'s indicate identical definitions. IE: There is a DATA_CHANGE bit in Drop Status, one in */
/*                                          Drop Control and one in Card Status */

/* card command byte values */
#define READY		0x00	/* card is waiting for a command */
#define BUSY		0x01	/* card is busy executing a command */
#define STARTUP         0x02	/* start emulating virtual drops, reading data for real drops */
#define SHUTDOWN	0x03	/* cease activity */
#define RESTART         0x04	/* re-initialize card structure, cease transmission */
#define IRQ_HOST        0x05	/* cause an interrupt to the host */
#define TXTEST          0x06	/* Transmit continually (diagnostic) to drop arg[0], arg[1] is data offset */
#define ADD_DROP        0x07	/* arg[0]=Drop number, arg[1]=# of modules to define - enter modules before going online */
#define CONFIG_DROP     0x08	/* arg[0]=Drop number. Tells QRIO to process config, and generate offsets for that drop */
#define ID_REQUEST      0x09	/* tells QRIOM to scan all drops, and fill in module ID words for everything it finds */

/* card status bits */
#define ONLINE	        0x01	/* set/cleared by QRIO to show if network interface is currently enabled */
#define BADCONFIG	0x02	/* set by QRIO if configure command does not match network config */
#define DATA_CHANGE     0x04	/* set by QRIO if any drop(s) have caused a DATA_CHANGE interrupt (cleared by host) */
#define BADCOMMAND      0x08     /* set by QRIO if an invalid command is issued */
#define TIMEOUT         0x10	/* set/cleared by QRIO if any drop's TIMEOUT bit has been set */

#define QRIOM_PRESENT   0x6d51
#define QRIO_PRESENT    0x7351

/* in_cfg and out_cfg bits */
#define CFG_COIL        0x80    /* True for coil reference, false for register */
#define CFG_BCD         0x40    /* True if data is in BCD format */
#define CFG_SIMPLE      0x20    /* True if this is "984/Quantum simple module" */
#define CFG_HOLD        0x10    /* True if this OUTPUT is to hold it's last state */
                                /* when the PLC vanishes */
/* If you force the state to be exception it will reprogram the drop */
/* drop state values */
#define PUP_STATE       0x00    /* power up state */
#define STANDBY_STATE   0x01    /* standby - powerup is complete */
#define RST_STATE       0x02    /* reset - drop needs to be configured */
#define RW_STATE        0x03    /* normal read/write running state */
#define XEP_STATE       0x05    /* exception state */

/* eos_delay MSB is WAIT flag - indicates whether QRIOM is currently waiting or not */
#define EOS_WAIT_EN     0x80    /* tell QRIOM to wait at the end of each scan */
#define EOS_WAITING     0x40    /* QRIOM is currently waiting. Set by QRIOM, cleared by host */

/* Motorola Byte ordering via 8-bit accesses macro */
#define getword( x ) (unsigned short)((((unsigned char *)(x))[ 1 ] << 8 ) | ( (unsigned char *)(x))[ 0 ])
#define putword( x , y ) { ((unsigned char *)(x))[ 1 ] = ((y)>>8) ; ((unsigned char *)(x))[ 0 ] = ((y)&0xff); }



/* QO indicates Quantum Only variables */

struct module_str
{
    unsigned char rack;           /* 0x0000 rack number for this module */
    unsigned char slot;           /* 0x0001 slot number for this module */
    unsigned short w_id;          /* 0x0002 module ID word for this module */
    unsigned char in_length;      /* 0x0004 # of bytes of input data */
    unsigned char in_cfg;         /* 0x0005 input config bits (QO) */
    unsigned short w_in_ref;      /* 0x0006 input starting reference number (QO) */
    unsigned char out_length;     /* 0x0008 # of bytes of output data */
    unsigned char out_cfg;        /* 0x0009 output config bits (QO) */
    unsigned short w_out_ref;     /* 0x000a output starting reference number (QO) */
    unsigned short w_tout_length; /* 0x000c # of bytes of timeout state data */
    unsigned short w_in_offset;   /* 0x000e shared RAM offset to data (set by QRIO, do not change!) */
    unsigned short w_out_offset;  /* 0x0010 shared RAM offset to data (set by QRIO, do not change!) */
    unsigned short w_tout_offset; /* 0x0012 shared RAM offset to timeout state data */
    unsigned char unused[ 2 ];    /* 0x0014 */
}; /* 0x16 BYTES */

struct drop_str
{
    unsigned char control;      /* 0x0030, 0x0060, 0x0090,... drop control bits, set by HOST. */
    unsigned char status;       /* 0x0031 */
    unsigned short w_activity;    /* 0x0032 set to holdup by QRIO every time drop communicates. Decremented every 100 ms */
    unsigned char last_seq;     /* 0x0034 last sequence number to this drop (set by QRIO, do not change!) */
    unsigned char state;        /* 0x0035 QRIO state variable, do not change */
    unsigned short w_retries;     /* 0x0036 # of retries to this drop (incremented by QRIO) */
    unsigned char health[ HEALTH_LENGTH ];  /* 0x0038 module health bits (read by plc)*/
    unsigned short w_holdup_time; /* 0x0042 timeout time in 100ms ticks */
    unsigned char asciiport;    /* 0x0044 Reserved (QO) */
    unsigned char headslot;     /* 0x0045 Reserved (QO) */
    unsigned short w_status_ref;  /* 0x0046 plc status reference (QO)            (set by plc) */
    unsigned short w_module_offset;/* 0x0048 Shared RAM offset to slots for this drop */
    unsigned short w_module_length;/* 0x004a number of modules */
    unsigned short w_offset;      /* 0x004c starting drop data offset (set by QRIO, do not change!) */
    unsigned char in_bytes;     /* 0x004e total drop input bytes (set by QRIO during config, do not change!) */
    unsigned char out_bytes;    /* 0x004f total drop output bytes (set by QRIO during config, do not change!) */
    unsigned short w_plc_status_offset;    /* 0x0050 plc drop status location (QO) */
    unsigned short w_plc_status_length;    /* 0x0052 plc drop status length (QO) (read by plc) */
    unsigned char unused[ 12 ]; /* reserved */
}; /* 0x30 BYTES */

typedef struct
{
    char version[ 12 ];         /* 0x0000 "QRIOM-x.xx" string */
    unsigned short w_present;     /* 0x000c existence bytes */
    unsigned char command;      /* 0x000e command register, set by HOST reset to READY by QRIO */
    unsigned char status;       /* 0x000f card status bits, set by QRIO */
    unsigned short w_arg[ 2 ];    /* 0x0010 arguments for commands */
    unsigned short w_rx_count;    /* 0x0014 Network reception counter, incremented by QRIO */
    unsigned short w_tx_count;    /* 0x0016 Network transmision counter, incremented by QRIO */
    unsigned short w_err_count;   /* 0x0018 Network error counter, incremented by QRIO */
    unsigned short w_timer;       /* 0x001a Proof that QRIO is alive - this value is incremented every 100ms when QRIO is running */
    unsigned short w_mem_used;    /* 0x001c amount of memory used */
    unsigned char eos_irq_mult; /* 0x001e QRIOM only - generate interrupt every EOS_IRQ_mult scans */
    unsigned char eos_control;  /* 0x001f QRIOM only - end of scan control */
    unsigned char unused[ 16 ];
    struct drop_str drop[ MAX_DROPS ];
} QTM_REGION; /* 0x30 bytes + MAX_DROPS * sizeof drop_str = 0x0630 bytes */

/* cntl_stat bit definitions */
#define	QFAIL		0x01	/* 0 turns on the fail led */
#define	QPASS		0x02	/* 1 turns on the Pass led */
#define	QINT_ENB	0x04	/* 0 Clears a pending interrupt, 1 Enables interrupts */
#define	QINT_REQ	0x04	/* 1 means interrupt pending */
#define	QINT_V40	0x08	/* 1 interrupts the card processor */
#define	QRUN_V40	0x40	/* 0 holds the card in reset, 1 releases the procesor */
#define	QSEM		0x80	/* 1 indicates card in use by another (multimaster) */
/*lbase bit definitions */
#define	SRUN		0x01	/* 1 = memory write enable */

typedef struct   /* Layout of short address space on 5136 Board */
{
    unsigned char ctl_stat;	  /* Control and status register */
    unsigned char not1;	  	  /* Not used */

    unsigned char lbase;	  /* set standard base address here */
    unsigned char not2;	  	  /* Not used */

    unsigned char intid;	  /* Set interrupt vector id here */
    unsigned char not3;	  	  /* Not used */

} QTM_SREGION; /* 3 16 bit entities only 1 byte active each */
typedef struct {
		short		ptype;
		short		ptbdtype;
		short		modbdtype;
		} MODMAP;

#define	DI8		1
#define	DI16		2
#define	DI32		3
#define	DO8		4
#define	DO16		5
#define	DO32		6
#define	AO		7
#define	AICUR		8
#define	AIVOLT	9
#define	AITC		10
#define	AIRTD		11

#define	SAI		0
#define	SAO		1
#define	SBI		2
#define	SBO		3

typedef struct {
		short		id;
		short		qtmflg;
		short		incfg;
		short		outcfg;
		short		inbyte;
		short		outbyte;
		short		toutlen;
		short		sigtype;
		short		maxsig;
		short		type;
		} MODPARAMS;

#define	MAXAOS		4
#define	MAXGAINS	8

typedef struct {
		char			type;
		MODPARAMS		*mp;
		unsigned char		*indata;
		unsigned char		*outdata;
		struct module_str	*qmod;
		unsigned char		*ptout;
		unsigned char		*status;
		unsigned char		gains[MAXGAINS];
 		} MODULE;

typedef struct {
		MODULE	*mods[MAX_QSLOTS];
 		} RACK;
 
typedef struct {
		short			was_defined;
		short			is_quantum;
		short			nummods;
		RACK			racks[MAX_RACKS];
 		} DROP;

typedef struct {
		unsigned char		drop;
		unsigned char		rack;
		unsigned char		card;
		unsigned short		type;
		unsigned char		data[8];
		} OUTUPD;

#define		MAXUPDS		50

typedef struct {
		short			line;
		short			mastercnt;
		unsigned char		seqs[MAX_DROPS];
		unsigned char		state[MAX_DROPS];
		unsigned char		status[MAX_DROPS];
		short			outcount;
		OUTUPD			outs[MAXUPDS];
		} QTMSREDUPD;

typedef	struct	{
	unsigned short	was_requested;		/* 0  The scan task has accesses this RTU */
	unsigned short	init_complete;		/*    The driver has initialized the scanner board */
	unsigned short	is_responding;		/* 1  The RTU is responding	*/
	unsigned short	holdflag;			/*    The RTU data is in hold */
	unsigned short	no_dataout_msgs;		/* 5  Count Output Msgs */
	unsigned short	no_offlines;		/* 14 Count Off Line Transitions */
	unsigned short	cmd_reset_stat;		/* 15 Comand to reset stats*/
	unsigned short	cmd_force_offline;	/* 16 Command to Force Offline */
	unsigned long	scanrate;			/* Last scan time delay*/

	unsigned short	partnerresp;		/* 23 Partner is responding */
	unsigned short	partnercount;		/* 24 Counter received from partner's registers */
	unsigned short	mycount;			/* 28 Counter sent to partner's registers */

	unsigned short	partnerstate;		/* 29 Register which reflects the partners state */


	unsigned short	ammaster;			/* I am master */
	
	unsigned long	lastdropmask;		/* Last set of drops on line */

	unsigned long	lastscantime;		/* Last time we executed a scan cycle */
	unsigned long	tickchecktime;		/* Last time the ticker was checked */
	unsigned short	lasttimervalue;		/* Last timer value read from board */
	unsigned long	redundmsgtime;		/* Last time master sent output table was copied to */
	unsigned short	newoutput;			/* New outputs were sent need redundant update */

	unsigned char	firmwareinfo[80];	/* Firmware identification string */

	QTMSREDUPD	redund;			/* Redundant message area */

	/* I/O Configuration Global Parameters */
	unsigned short	hold_last;		/* Flag for output control */
	unsigned short	extcj;		/* Flag for External Cold Junction */
	unsigned short	enbcj;		/* flag to enable cold junction compensation */
	unsigned short	enboc;		/* flag to enable open circuit detection */
	DROP		drops[MAX_DROPS];		/* storeage for data structures */

	}	QTMLINK;

/* Quantum I/O Board Configuration Parameters */
#define	QAVI_RIGHTJ	0x01
#define	QAVO_PARAMS	0x80
#define	QAVO_RIGHTJ	0x01
#define	QAVO_HOLD2	0x55
#define	QAVO_DROP2	0x00
#define	QAVO_LOSSH	0x00
#define	QAVO_LOSSL	0x00
#define	QATC_EXTCJ	0x08
#define	QATC_ENBCJ	0x20
#define	QATC_ENBOC	0x20
#define	QATC_TENTH	0x02


/* task definitions */

#define 	QTMS_STACK		8000   /* Stack size              */
#define	QTMS_PRIORITY		108    /* default proirity        */
#define	QTMS_TASK_OPTIONS	8      /* Floating point	  */


/* QTM-SCANNER board dual port RAM base addresses */
/* Short Address Space */
#define	QTMSL1SBASE	0xe000
#define	QTMSL2SBASE	0xe400
#define	QTMSL3SBASE	0xe800
#define	QTMSL4SBASE	0xec00

#define	QTMSL1LBASE	0xe00000
#define	QTMSL2LBASE	0xe40000
#define	QTMSL3LBASE	0xe80000
#define	QTMSL4LBASE	0xec0000

#define	QTMSL1VEC	0xE0
#define	QTMSL2VEC	0xE4
#define	QTMSL3VEC	0xE8
#define	QTMSL4VEC	0xEC

#define	QTMSINTLEV	2

#define	CMDTIMEOUT	6	/* .1 Seconds */
#define	MINTICKPERSCAN  2	/* 80 ms. */
#define	CARDRETRIES	3
/* Trace Structure */
typedef	struct {
	unsigned short		path;
	unsigned short		cmd;
	unsigned long		data;
		}	QTMSTRACE;
#define	QTMSTRACELEN	64

typedef struct {
	unsigned short		nextptr;
	QTMSTRACE			tdata[QTMSTRACELEN];
		} QTMSTRACEQUE;
