/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/**********
*
*	File:	drvRtp.h
*
*	This file contains the data structures used by the Dap
*	I/O driver.                                                            	       	*                                                                             	      	**********/

#ifndef	INCDRVRTP
#define	INCDRVRTP

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/*	Define the card types for the RTP IO Rack	*/

#define	RTP_EMPTY	0xffff		/* No RTP Device defined */
#define	SLOT_EMPTY	0x00		/* No card assigned to slot */
#define	RTP_AIN		0x01		/* Analog ADC Card */
#define	RTP_GATE_4	0x02		/* Analog Gate Card */
#define	RTP_GATE_8	0x03		/* Analog Gate Card */
#define	RTP_GATE_16	0x04		/* Analog Gate Card */
#define	RTP_AOT_1	0x05		/* Analog Output Card */
#define	RTP_AOT_4	0x06		/* Analog Output Card */
#define	RTP_AOT_8	0x07		/* Analog Output Card */
#define	RTP_DIN_8	0x08		/* Digital Input Card */
#define	RTP_DIN_12	0x09		/* Digital Input Card */
#define	RTP_DIN_16	0x0A		/* Digital Input Card */
#define	RTP_DOT_8	0x0B		/* Digital Output Card */
#define	RTP_DOT_12	0x0C		/* Digital Output Card */
#define	RTP_DOT_16	0x0D		/* Digital Output Card */
#define	PSEUDO90	90		/* Pseudo points for stats & controls */
#define	PSEUDO91	91		/* Pseudo points for stats & controls */
#define	PSEUDO92	92		/* Pseudo points for stats & controls */

#define	RTP_RESPONDING	0x01		/* The RTP I/O board is responding */
#define	RTP_NORESPONSE	0x00		/* The RTP I/O board is not responding */
#define	NUM_IOBC	4		/* Number of IOBC's */
#define	NUM_RTP		8		/* Max number RTP's per IOBC */

#define MAX_RTP_ADDR	32
#define	MAX_RTP_AIN	256		/* Max AIN per RTP ADC card */
#define	MAX_RTP_AOT_1	8		/* Max AOT per RTP AOT card */
#define	MAX_RTP_AOT_4	4		/* Max AOT per RTP AOT card */
#define	MAX_RTP_AOT_8	8		/* Max AOT per RTP AOT card */
#define	MAX_RTP_DIN_8	32		/* Max DIN per RTP DIN card */
#define	MAX_RTP_DIN_12	12		/* Max DIN per RTP DIN card */
#define	MAX_RTP_DIN_16	16		/* Max DIN per RTP DIN card */
#define	MAX_RTP_DIN_32	32		/* Max DIN per RTP DIN card */
#define	MAX_RTP_DOT_8	32		/* Max DOT per RTP DOT card */
#define	MAX_RTP_DOT_12	12		/* Max DOT per RTP DOT card */
#define	MAX_RTP_DOT_16	16		/* Max DOT per RTP DOT card */
#define	MAX_RTP_DOT_32	32		/* Max DOT per RTP DOT card */
#define	MAX_GATE_ADDR	16		/* Max GATE / SLOT Address per Rack */
#define	MAX_GATE_SIG_4	16		/* Max AIN per RTP GATE card */
#define	MAX_GATE_SIG_8	8		/* Max AIN per RTP GATE card */
#define	MAX_GATE_SIG_16	16		/* Max AIN per RTP GATE card */

/*	Define redundancy constants	*/

#define	MAXCFGCK		8	/* Try to Bring online each cycle */
#define	BRINGBACKCNT		4	/* Poll loops per cycle */

#define	ACTIVITYTIME		300	/* must see activity in 5 seconds */	
#define	RESNAMELEN		16	/* length of a resource name */
#define	SLWWINDOW		900	/* # of ticks to allow slave to try */

/*	Define the resource group constants for group failover control */
/*      number of groups which can be defined in the PCU/LGU */

#define         MAXGROUPNUM     16

#define         RES_LOCKED       0


/**********
*
*	Internal IOC Structures for RTP IO
*
**********/

typedef struct
{
	unsigned char	was_requested;
	unsigned char	is_responding;
	unsigned short	devtype;
	long		scanrate;
	long		next_scan;
	int		gain;
	unsigned short	value;
	unsigned short	value_1;
	unsigned short	value_2;
	unsigned short	value_3;

} RTP_AIN_IO;

typedef struct
{
	unsigned char	was_requested;
	unsigned char	is_responding;
	long		scanrate;
	long		next_scan;
	unsigned char	out_requested;
	unsigned short	out_value;
	unsigned short	value;

} RTP_AOT_IO;

typedef struct
{
	unsigned char	was_requested;
	unsigned char	is_responding;
	long		scanrate;
	long		next_scan;
	unsigned long	value;

} RTP_DIN_IO;

typedef struct
{
	unsigned char	was_requested;
	unsigned char	is_responding;
	long		scanrate;
	long		next_scan;
	unsigned char	out_requested;
	unsigned long	out_value;
	unsigned long	value;

} RTP_DOT_IO;

typedef union
{
	RTP_AIN_IO	*ainio;
	RTP_AOT_IO	*aotio;
	RTP_DIN_IO	*dinio;
	RTP_DOT_IO	*dotio;

} RTPIO_PTR;

typedef struct
{
	unsigned short	rtp_type;
	unsigned short	status;
	unsigned short	tmouts;
	RTPIO_PTR	pio;

} RTPIO_CARD;

typedef struct
{
	unsigned short	*rtp_cmd;
	unsigned short	*rtp_data;
	RTPIO_CARD	io [MAX_GATE_ADDR];

} RTPIO_RACK;

typedef struct
{
	unsigned char	time [40];
	unsigned char	pointname [40];
	unsigned int	mask;
	unsigned int	value;
	unsigned int	previous_mask;
	unsigned int	current_mask;
	unsigned int	card;
	unsigned int	rack;
} BO_RECORDER;

/* Message definitions for redundant updates */

#define		REDUNDOUTUPDATETIME	60


typedef struct
{
		unsigned char		rack;
		unsigned char		card;
		unsigned short		type;
		unsigned short		data;
} OUTUPD;

#define		MAXUPDS		50

typedef struct
{
		short			line;
		short			mastercnt;
		short			outcount;
		OUTUPD			outs [MAXUPDS];
} RTPSREDUPD;
	
typedef	struct
{
	unsigned short  was_requested;		/* 0  The scan task has accesses this RTU 	*/
	unsigned short  is_responding;		/* 1  The RTU is responding */
	unsigned short  holdflag;		/*    The RTU data is in hold */
	unsigned short	badoutcnt;		/* 10 Count Bad / Dropped outputs */
	unsigned short  cmd_reset_stat;		/* 15 Comand to reset stats */
	unsigned short  cmd_force_offline;	/* 16 Command to Force Offline */
	unsigned short	out_requested;		/* set to send outputs */
	unsigned short	rack_tmouts;
	RTPIO_RACK	rtp;

} RTPIO;

/*	Define the analog input command and data word constants and masks */

/*	Command Word	*/

#define	RTP_AIN_INT_MSK	0x0c00
#define	RTP_AIN_INT_DIS	0x0400
#define	RTP_AIN_INT_ENA	0x0800
#define	RTP_AIN_INT_DEF	0x0100

#define	RTP_AIN_CVT_MSK	0x000f

/*	Data Word	*/

#define	RTP_AIN_OTD_MSK	0x4000
#define	RTP_AIN_OTD_DIS	0x0000
#define	RTP_AIN_OTD_ENA	0x4000

#define	RTP_AIN_INP_MSK	0x3000
#define	RTP_AIN_INP_DIF	0x0000
#define	RTP_AIN_INP_SEE	0x2000
#define	RTP_AIN_INP_SEO	0x3000

#define	RTP_AIN_GAT_MSK	0x0780
#define	RTP_AIN_GAT_SFT	0x0007

#define	RTP_AIN_SIG_MSK	0x0070
#define	RTP_AIN_SIG_SFT	0x0004

#define	RTP_AIN_GAN_MSK	0x000f
#define RTP_AIN_GAIN_1	0x0001
#define RTP_AIN_GAIN_2	0x0002
#define	RTP_AIN_GAIN_4	0x0004
#define	RTP_AIN_GAIN_8	0x0008

/*	Define the calibration gain/voltage selection */

#define	RTP_AIN_CAL_ZERO	0x0000
#define	RTP_AIN_CAL_PLUS	0x0004
#define	RTP_AIN_CAL_MINUS	0x0008
#define	RTP_AIN_CAL_SYSMON	0x000c

#define	RTP_AIN_CAL_FULL	0x8000
#define	RTP_AIN_CAL_HALF	0x0000
#define	RTP_AIN_CAL_GAIN1	0x0000
#define	RTP_AIN_CAL_GAIN2	0x0001
#define	RTP_AIN_CAL_GAIN4	0x0002
#define	RTP_AIN_CAL_GAIN8	0x0003

/*	Define the Rtp Digital Input command word	*/

#define	RTP_DIN_INT_MSK	0x0c00
#define	RTP_DIN_INT_ENA	0x0800
#define	RTP_DIN_INT_DIS	0x0400

#define	RTP_DIN_GAT_MSK	0x000f
#define	RTP_DIN_LO_BANK	0x0000
#define RTP_DIN_HI_BANK	0x0010

/*	Define the Rtp Digital Output command word	*/

#define	RTP_DOT_INT_MSK	0x0c00
#define	RTP_DOT_INT_ENA	0x0800
#define	RTP_DOT_INT_DIS	0x0400

#define	RTP_DOT_GAT_MSK	0x000f
#define	RTP_DOT_LO_BANK	0x0000
#define RTP_DOT_HI_BANK	0x0010

/*	Define the Rtp Analog Output command word	*/

#define	RTP_AOT_MOD_RAN	0x0100
#define	RTP_AOT_MOD_SEQ	0x0200
#define	RTP_AOT_MOD_MSK	0x0300

#define	RTP_AOT_GAT_MSK	0x000f

/*	Define the RTP Analog Output Data word	*/

#define	RTP_AOT_OUT_MSK	0x0fff
#define	RTP_AOT_LOC_MSK	0xe000

/*	Define the RTP Analog Output channel select values	*/

#define	RTP_AOT_CHAN_0	0x0000
#define	RTP_AOT_CHAN_1	0x4000
#define	RTP_AOT_CHAN_2	0x8000
#define	RTP_AOT_CHAN_3	0xc000
#define	RTP_AOT_CHAN_4	0x2000
#define	RTP_AOT_CHAN_5	0x6000
#define	RTP_AOT_CHAN_6	0xa000
#define	RTP_AOT_CHAN_7	0xe000

/*	Define the Test Input values	*/

#define	RTP_DEV_SEL_MSK	0x8000
#define	RTP_DEV_RDY_MSK	0x4000
#define	RTP_BUS_DIS_MSK	0x1000

#define	RTP_MAX_TST_CNT	0x0005

/*	Define the BUS Switch Data Output words */

#define	RTP_BUS_MASTER	0x10
#define	RTP_BUS_CLEAR	0x08

/* task definitions */

#define RTP_STACK		8000   /* Stack size              */
#define	RTP_PRIORITY		152    /* default priority        */
#define	RTP_TASK_OPTIONS	8      /* Floating point          */

#endif

/*	End of File:	drvRtp.h */
