/*drvrtu.c,v 1.1.1.1 1999/08/03 14:44:32 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
/*  File: drvRtu.c
*/

#define  MAIN        rtu_driver
#define  AIDRIVER    rtu_aidriver
#define  AODRIVER    rtu_aodriver
#define  AOREAD      rtu_aoread
#define  BIDRIVER    rtu_bidriver
#define  BODRIVER    rtu_bodriver
#define  BOREAD      rtu_boread

#define  TRACEON     rtuton
#define  TRACEOFF    rtutoff
#define  RTRACEON    rturestraceon
#define  RTRACEOFF   rturestraceoff
#define  TRACELEVEL  rtutrlv

#define  STOPDRIVER  stoprtu
#define  STARTDRIVER1 startrtu
#define  TABLEINFO   rtu_dmt
#define  LINEINFO    rtu_sli


#include <ctype.h>

#include <vxWorks.h>
#include <sysLib.h>
#include <semLib.h>
#include <wdLib.h>
#include <rngLib.h>
#include <task_params.h>
#include <stdioLib.h>

#include <dbDefs.h>
#include <drvSup.h>
#include <module_types.h>
#include <drvRtu.h>
#include <drvRtu_proto.h>
#include <restable.h>


/* static long	init(); */

struct	{
		long		number;
		DRVSUPFUN	report;
		DRVSUPFUN	init;
	}
	drvRtu={ 2,	NULL, init};

float	rtu_filter;		/* This value is the percent change that should be
				   ignored when reading the analog inputs */


/*
/*      Storage for RTU I/O  Structures
/*
*/

static CELL		*plcs[RTULINES][MAXCELLS+1];	/* two dimensional array of pointers to the fixed length   		*/
							/* dynamically allocated PLC control and data structures.  		*/
							/* extra pointer is for Query structure used independently 		*/
							/* of the others. it also allows for the array index to    		*/
							/* match the mod value.                                    		*/

static unsigned char	resnames[RTULINES][RESNAMELEN];	/* array of resource name strings. 					*/

static RES_TABLE	*prestable[RTULINES];		/* array of pointers to resource tables 				*/

static unsigned short	resresponse[RTULINES];		/* array of resource response flags. 					*/
							/* False=not active resource(slave), True= active resource(master) 	*/

static unsigned short	rtusim[RTULINES] = {FALSE};	/* I/O SIMULATION MODE 							*/
static unsigned short	rtusimaival = 0;		/* Analog input simulation value 					*/
static unsigned short	rtusimdival = 0;		/* Digital input simulation value 					*/
static unsigned short	rtusimaimin = 0;		/* Analog input simulation value 					*/
static unsigned short	rtusimaimax = 0;		/* Analog input simulation value 					*/
static unsigned long	rtusimtime = 0;			/* last time rtusimulated value was changes				*/
static unsigned short	rtusimdelta = 300;		/* Analog input simulation time delta 					*/
static unsigned short	rtusimaidelta = 1;		/* Analog input simulation value delta 					*/
static unsigned short	rtusimdidelta = 0;		/* Digital input simulation value delta 				*/
static unsigned short	rtusimmemwrite = 0;		/* Digital input simulation value delta 				*/

static unsigned short	line_activity[RTULINES];	/* true/false line active flag. calculated using activity_time below. 	*/
static unsigned short	lastactivity[RTULINES];		/* assigned value of line_activity prior to it's recalculation.   	*/

static unsigned short	mltrace[RTULINES];		/* controls output of certain resource related log messages (level) 	*/
static unsigned short	modrestrace[RTULINES];		/* controls output of MASTER/SLAVE arbitration log messages (binary) 	*/

static OUTPUTQUE	*oq[RTULINES];			/* array of ptrs to the line output queues and their control structures */

static SEM_ID		semids[RTULINES][2];		/* 'new' semaphore logic. the old SEMAPHORE is still used. 		*/

static short		linetimers[RTULINES];		/* except on init (init_line_tabs), all references commented out 	*/

static SEM_ID		linesems[RTULINES];		/* not used. no references in the code at all 				*/
                                                	
static unsigned short	intermsgdelay[RTULINES];	/* holds value passed in task spawn. used  for each msgxfer attempt 	*/
static unsigned short	pretxdelay[RTULINES];		/* holds post RTS delay before TX. 					*/
static unsigned short	posttxdelay[RTULINES];		/* holds post TX delay before dropping RTS. 				*/

static long		activity_time[RTULINES];	/* holds timetick of last activity. used to set line_activity flag  	*/

static char		user[40],password[40];		/* holds id and password of the user that spawned the driver task	*/
							/* required by host to check file read privileges on PLC load files 	*/
static char	*bdaddr[SLOTSPERCELL] = {
	SLOT1,SLOT2,SLOT3,SLOT4,
	SLOT5,SLOT6,SLOT7,SLOT8
    };



/*
/*      Storage for RTU Comm Line Structures
/*
*/

static RXLINE          *rxlinetab[RTULINES] = 
			{NULL} ; /* array of pointers to dynamically allocated comm controls/buffers 	*/

static TXLINE          *txlinetab[RTULINES] = 
			{NULL} ; /* array of pointers to dynamically allocated comm controls/buffers */


static char    *bools[] = {
	"TRUE ",
	"FALSE"
    };
static int		taskId [RTULINES];

static char *tasknames[] = {"RTU Ch0","RTU Ch1","RTU Ch2","RTU Ch3","RTU Ch4","RTU Ch5",
			    "RTU Ch6","RTU Ch7","RTU Ch8","RTU Ch9","RTU Ch10","RTU Ch11",
			    "RTU Ch12","RTU Ch13","RTU Ch14","RTU Ch15","RTU Ch16","RTU Ch17",
			    "RTU Ch18","RTU Ch19","RTU Ch20"
			    };

/**********
*
*	This routine is required by iocinit in the EPICS system.
*
**********/
static long init ()
    {
	return (0);
    }
/********
/*
/*	This routine enables RTU I/O simulation mode
/*
********/
void rtusimon (unsigned short min, unsigned short max, unsigned short time, unsigned short deltaav,
	       unsigned short deltadv, unsigned short memwrite)
    {
	rtusimaival = rtusimaimin = min;

	if (deltaav != 0)
	    rtusimaidelta = deltaav;

	if (deltadv != 0)
	    rtusimdidelta = deltadv;
	else
	    rtusimdidelta = 0;

	if (rtusimaimax == 0)
		rtusimaimax = 16250;
	else
		rtusimaimax = max;

	if (time == 0)
		rtusimdelta = 300;
	else
		rtusimdelta = time*60;

	rtusim[0] = TRUE;

	rtusimmemwrite = memwrite;

    }

/********
/*
/*	This routine enables RTU I/O simulation mode
/*
********/
void rtusimoff ()
    {
	rtusim[0] = FALSE;
    }


/********
/*
/*	This routine enables logging of comm trace messages
/*
********/
void TRACEON (unsigned short line)
    {
	mltrace[line] = 1;
    }

/********
/*
/*	This routine disables logging of comm trace messages
/*
********/
void TRACEOFF (unsigned short line)
    {
	mltrace[line] = 0;
    }

/********
/*
/*	This routine enables logging of resource trace messages
/*
********/
void RTRACEON (unsigned short line)
    {
	modrestrace[line] = TRUE;
    }

/********
/*
/*	This routine disables logging of resource trace messages
/*
********/
void RTRACEOFF (unsigned short line)
    {
	modrestrace[line] = FALSE;
    }

/********
/*
/*	This routine controls logging of trace messages
/*
********/
void TRACELEVEL (unsigned short line, unsigned short level)
    {
	mltrace[line] = level;
    }


int STOPDRIVER (short ln)
{
	if (comm_disconnect (ln) != -1)
	{
	    taskDelete (taskId [ln]);
	    return (1);
	}
	else
	{
	    return (-1);
	}
}

int STARTDRIVER1 (unsigned short ln, char *mode,unsigned short delay, float filter)
{
	if ( ln < RTULINES )
	{
	    if (filter == 0.0)
	    {
		rtu_filter = 0.1;
	    }
	    else
	    {
		if ((filter < 0.0) || (filter > 100.0))
		{
		    logMsg ("Analog filter out of range\n");
		    rtu_filter = 0.1;
		}
		else
		{
		    rtu_filter = (filter / 100);
		}
	    }
	    taskId [ln] = taskSpawn (tasknames[ln],RTU_PRIORITY,RTU_TASK_OPTIONS,RTU_STACK,MAIN,ln, mode, delay);

	    logMsg ("Started the RTU driver on line %d\n",ln);

	    remCurIdGet (user,password);
	    return (1);
	}
	else
	{

	    logMsg ("Invalid line number %d\n", ln);
	    return (-1);
	}
}


/*											
/*      RTU_driver                                                                      
/*                                                                                     	
/*      The purpose of this task is to scan the local CELL I/O of this Card                             	
/*      lines. It will maintain a configuration list supporting online additions       	
/*      and replacement of modules, and maintain a table of most recently              	
/*      scanned values.                                                                	
/*                                                                                     	
*/                                                                                     	
/* not called from within this driver 						      */

void MAIN (unsigned short line, unsigned char *mode, unsigned short delay)
    {
	unsigned short  mod, cfgchk, cnt, modspolled, onlinefreq;
	unsigned short  activity, pstate, lstate, execute, command,i;
	long	        this_time, last_time, onlinetime;
	CELL          *pmod;
	RXLINE          *rxp;

/* Initialize the data structures */

	intermsgdelay[line] = delay;

	resresponse[line] = FALSE;
	resnames[line][0] = 0;
	prestable[line] = NULL;

	mltrace[line] = 0;
	modrestrace[line] = TRUE;
	onlinefreq = 1;		/* causes on_line calls to begin on first pass if master */
	onlinetime = 0;

	activity_time[line] = tickGet ();
	lastactivity[line] = line_activity[line] = FALSE;

	/* output queue initialization */
	oq[line] = (OUTPUTQUE *)calloc(1,(sizeof(OUTPUTQUE)));
	if (!oq[line])
	{
	    logMsg ("Not enough memory for line %d data structure (data output queue)\n",line);
	    return;
	}

	/* communication initialization */
	rxlinetab[line] = (RXLINE *)calloc(1,(sizeof(RXLINE)));
	if (!rxlinetab[line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: rx)\n",line);
	    return;
	}
	txlinetab[line] = (TXLINE *)calloc(1,(sizeof(TXLINE)));
	if (!txlinetab[line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: tx)\n",line);
	    return;
	}


	while ( (resnames[line][0] == 0) || (prestable[line] == NULL) )
	{
	    if (mltrace[line] == 1)
	    	logMsg ( "No Resource Defined for line - %d \n", line);
	    taskDelay (120);
	}


	/* Wait for resource to be defined via an initial call to our own 'register_resource' from someone higher up */
	while ( (mltrace[line] == 0) )
	{
	    logMsg ( "Waiting for rtuton \n");
	    taskDelay (300);
	}

	/* Start out as slave */
	resClearCaAnswer(prestable[line]);		/* start quiet */
	resPutLocalState(prestable[line],RES_SLV);
	if (modrestrace[line])
		logMsg ("1] Became RES_SLV \n");
	taskDelay(300);
	resGetStates(prestable[line],&lstate,&pstate);
	activity = check_line_activity(line,ACTIVITYTIME);


	/* if the partner is not there, be master */
	if ((pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity))
	{
	    resSetCaAnswer(prestable[line]);
	    resPutLocalState(prestable[line],RES_MSTR);
	    lstate = RES_MSTR;
	    if (modrestrace[line])
	    	logMsg ("2] Became RES_MSTR \n");
	}


	cfgchk = 0;     /* Init the new config pointer */
	last_time = tickGet(); /* Initialize the time  */

	while (TRUE)
	{
	    activity_time[line] = tickGet();
	    lastactivity[line] = line_activity[line] = FALSE;

	    /* we arrive here because we are a slave. 									  */
	    /* we stay here until:    asked to be master by partner. if we can reach PLCs, we breakout to 'master' state. */
	    /*                        the partner is not master and there is no activity on the line. 			  */

	    while (lstate == RES_SLV)
	    {

		resresponse[line] = FALSE; 			/* make sure driver is not responding 	*/
		resPutCommand(prestable[line],0); 		/* clear out commands 			*/
		activity = check_line_activity(line,10); 	/* wait 10 seconds before checking 	*/
		resGetStates(prestable[line],&lstate,&pstate);


		/* see if needed by partner to be master. if yes, do it and break out of this slave state */
		if (pstate == RES_SLVWANTCA)
		{
		    resSetCaAnswer(prestable[line]);
		    resPutLocalState(prestable[line],RES_MSTR);
		    lstate = RES_MSTR;
		    if (modrestrace[line]) logMsg ("3] Became RES_MSTR  P= %x A= %d \n",pstate,activity);
		    break;
		}

		/* If partner is there and wants to give it up, try to take it */
		if (pstate == RES_SLVWANT)
		{
		    execute = FALSE;

		    taskDelay(2); /* Time for switch to occur */

		    /* See if anyone is out there and able to respond to a loopback command */
		    for ( mod=0; (mod <= MAXCELLS); mod++)
		    {
			pmod = plcs[line][mod];
			if ((pmod) &&			/* verify pointer validity prior to use */
			    (pmod->was_requested))
			{
			    on_line(line,mod);	
			    if (pmod->is_responding)
			    {
				execute = TRUE;
				break;
			    }
			}
		    }
		    /* If someone answered, become master and make sure partner really gave it up */
		    if (execute)
		    {
			resPutLocalState(prestable[line],RES_MSTR);
			lstate = RES_MSTR;
			if (modrestrace[line])
				logMsg ("4] Became RES_MSTR  P= %x A= %d \n",pstate,activity);
			execute = FALSE;
			activity_time[line] = tickGet();
			lastactivity[line] = line_activity[line] = FALSE;
			for (i=0;(i<60);i++)
			{
			    activity = check_line_activity(line,2);    /* wait two seconds before testing */
			    resGetStates(prestable[line],&lstate,&pstate);
			    if ( (pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity) )
			    {
				execute = TRUE; /* Partner Gave It Up so take CA */
				break;
			    }
			}
			if (!execute)
			{   /* Partner did not give it up go back to slave */
			    resPutLocalState(prestable[line],RES_SLV);
			    lstate = RES_SLV;
			    if (modrestrace[line])
			    	logMsg ("5] Became RES_SLV  P= %x A= %d \n",pstate,activity);
			}
			else
			{ /* Assume control by taking the CA responsibilities */ 
			    resSetCaAnswer(prestable[line]);
			    prepresponses(line);
			    break;
			}
		    }
		}

		/* if the partner is not talking, be master */
		if ((pstate != RES_MSTR) &&  (pstate != RES_SLVWANT) && (!activity))
		{
		    resSetCaAnswer(prestable[line]);
		    resPutLocalState(prestable[line],RES_MSTR);
		    lstate = RES_MSTR;
		    if (modrestrace[line])
		    	logMsg ("6] Became RES_MSTR P= %x A= %d \n",pstate,activity);
		    prepresponses(line);
		}
	    }

            /* to arrive here, we used to be master, and had been told to relinquish master status. */
	    /* stay in this loop until the partner takes over, or the command is canceled */
	    while (lstate == RES_SLVWANTCA)
	    {
		resresponse[line] = FALSE; 	/* Make Sure driver is not responding */

		resGetStates(prestable[line],&lstate,&pstate);
		if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT))
		{
		    resClearCaAnswer(prestable[line]);
		    resPutLocalState(prestable[line],RES_SLV);
		    lstate = RES_SLV;
		    if (modrestrace[line])
		    	logMsg ("7] Became RES_SLV  P= %x A= %d \n",pstate,activity);
		    break;
		}
		resGetCommand(prestable[line],&command);
		resPutCommand(prestable[line],0); /* clear out commands */
		if (command == RES_CANCELREL)
		{
		    resSetCaAnswer(prestable[line]);
		    resPutLocalState(prestable[line],RES_MSTR);
		    lstate = RES_MSTR;
		    if (modrestrace[line])
		    	logMsg ("8] Became RES_MSTR  P= %x A= %d \n",pstate,activity);
		    prepresponses(line);
		}
	    }

            /* we come here because we have gone to, or are looping in, the master state.          */
	    /* we stay in this loop until:    we have been told to give master state to the partner.*/
	    /*                                we break out to RES_SLVWANTCA.                        */
	    /*                                we have successfully handed master state to partner.  */
	    /* While master, this task will process all modules sending outputs and getting inputs. */
	    /* It will then try to configure requested non-responding modules.                      */

	    /* it doesn't appear that we ever come here in a RES_SLVWANT lstate mode. it happens    */
	    /*  within this loop, but is never seen by the while loop conditional evaluation.       */

	    last_time = tickGet ();		/* initialize gap */
	    while ((lstate == RES_MSTR) || (lstate == RES_SLVWANT))
	    {
		/* initial stuff */
		resresponse[line] = TRUE; /* Make sure driver is responding. driver won't issue commands if not set. */

		resGetCommand(prestable[line],&command);
		resPutCommand(prestable[line],0);        /* clear out commands */


		/* if relinquish command sent, change state and break out to outer loop */
		if (command == RES_RELINQUISH)
		{
		    resClearCaAnswer(prestable[line]);
		    resPutLocalState(prestable[line],RES_SLVWANTCA);
		    lstate = RES_SLVWANTCA;
		    if (modrestrace[line]) logMsg ("9] Became RES_SLVWANTCA  P= %x A= %d \n",pstate,activity);
		    break;
		}


		/*  scan plcs for data i/o, if allowed */
		modspolled = 0;
		/* call 'scan_plc' for each PLC device, unless user has taken it off line or it is not responding */
		for ( mod=0; (mod <= MAXCELLS); mod++ )
		{
		    pmod = plcs[line][mod];			/* shorthand pointer 			*/
		    if (pmod)					/* verify pointer validity prior to use */
		    {
			if (pmod->cmd_force_offline)
			{ 
				/* if user has turned this plc offline via pseudo commands, mark it as not responding */
 				pmod->is_responding = FALSE;
	    		}
			else if (pmod->is_responding)
			{
				modspolled++;
				scan_plc(pmod,line,mod);      	/* I/O done here! 			*/
				if (!pmod->is_responding)
				{
					mark_not_valid(pmod);
                               	}
			}
		    }
		    /* end of single module action */
		    /* if told to relinquish our master state, stop plc scanning immediately. */
		    /* don't trash the command, so that we can still read it on the next loop through this while statement. */
		    resGetCommand(prestable[line],&command);
		    if (command == RES_RELINQUISH) break;

		} /* end of scan loop */

		if (command == RES_RELINQUISH) break;

		/* try and bring all nonresponding plcs on line, but don't bring in more than a specified number. */

		cnt = 0;

	        this_time = tickGet ();
		if ((this_time - onlinetime) > BRINGBACKTIME)		/* do we try to bring any in on now */
		{ 

		    onlinetime = this_time;
		    for ( mod=0; (mod <= MAXCELLS); mod++)
		    {
			cfgchk++;		/* manage plc index */
			if (cfgchk==MAXCELLS+1)
			{
				cfgchk=0;	/* wrap plc index */
                        }
			pmod = plcs[line][cfgchk];
			if ((pmod) &&		/* verify pointer validity prior to use*/
			    (!pmod->is_responding) &&
			    (pmod->was_requested) &&
			    (!pmod->cmd_force_offline))
		    	{
				on_line(line,cfgchk);	/* loopback testing done here! 			*/

				modspolled++;
				cnt++;
                                if (cnt == MAXCFGCK) break;
			}
		    }
		}

		/* see if anyone on the line has been responding. if not, try to get partner to become slave. */
		/* Check for no one responding */
		if (!any_responding(line))
		{
		    /* See if it is time to let partner try */
		    this_time = tickGet();
		    if ((this_time - last_time) > (3*SLWWINDOW))
		    {   /* #ticks for slave to try */

			/* Switch States to let partner know it's its time to try */



			resPutLocalState(prestable[line],RES_SLVWANT);
			lstate = RES_SLVWANT;
			if (modrestrace[line])
				logMsg ("10] Became RES_SLVWANT  P= %x A= %d \n",pstate,activity);

			/* Give partner access to the hardware */
/*			clear_rts(line);
*/
			/* Remember the time of this try */
			last_time = tickGet();
			lastactivity[line] = line_activity[line] = FALSE;
			for (i=0; (i<SLWWINDOW); i+=10)
			{       		/* SLWINDOW is the # of ticks for slave to try */
			    taskDelay(10);
		    				/* If partner picked up let it have control */
			    activity = check_line_activity(line,0);  /* check immediately */
			    resGetStates(prestable[line],&lstate,&pstate);
			    if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT) || (activity))
			    {
				resClearCaAnswer(prestable[line]);
				resPutLocalState(prestable[line],RES_SLV);
				lstate = RES_SLV;
				if (modrestrace[line])
					logMsg ("11] Became RES_SLV  P= %x A= %d \n",pstate,activity);
				taskDelay(240);
				break;
			    }
			}
			/* Go Back to being the Master and take control of the hardware */
			if (lstate != RES_SLV)
			{
			    resPutLocalState(prestable[line],RES_MSTR);
			    lstate = RES_MSTR;
			    if (modrestrace[line])
			    	logMsg ("12] Became RES_MSTR  P= %x A= %d \n",pstate,activity);
			}
		    }
		}

		/* take plcs off of hold status. the plc's data is not to be held off since we are now master. */
		for ( mod=0; (mod <= MAXCELLS); mod++ )
		{
		    pmod = plcs[line][mod];
		    if (pmod)			/* verify pointer validity prior to use */
		    {        
			pmod->holdflag = FALSE;
                    }
		}
		if (modspolled==0)
	    	    taskDelay(60);
	    	else taskDelay(1);
	    } /* end the master state loop */
	} /* end infinite task loop */
     }

/*
/* Try to bring a CELL on line
/*
*/

/* this routine only called from MAIN */


static void on_line (unsigned short line,unsigned short mod)
    {
	CELL  *pmod;
	unsigned short i,j;

	pmod = plcs[line][mod];	
	if (pmod)			/* verify pointer prior to use */
	{
		if (mltrace[line] == 1)
		{
			logMsg ( "Bring Online line - %d  mod - %d \n",
				line, mod );
	        }

		/* Local I/O is Alwaws online */
		if (mod == 0) {
		    pmod->is_responding = TRUE;
		    return;
		}

		/* Use the loop back function to see if the unit is there and responding */

		if (!cellloopback(line,mod,pmod))
			return; /* if no response, get out */

		mark_not_valid(pmod);	/* a PLC that has just been brought online must have it's data marked not valid */
		pmod->is_responding = TRUE;
        }
    }



/*							
/* Mark all plc data elements as not valid     		
/*                                             		
*/                                             	
/* called from on_line and prepresponses      		*/    	


static void mark_not_valid (CELL  *pmod)
    {
	unsigned short i,j;

	/* There is only one flag for the entire cell */
	pmod->data_valid = FALSE;

    }
/*                                                           	
/* Scan a RTU PLC -  get inputs and set outputs as required	
/*                                                            	
*/
/* this routine only called from MAIN 		*/


static	void scan_plc (CELL *pmod, unsigned short line,unsigned short mod)
    {
	unsigned short i, j, tval, thisslot, tbit;
	unsigned long	thistime,lval;
	volatile unsigned char *bdbase;
	UPDATEMSG	upmsg;

	checkoutputs(line);

	if (mltrace[line] == 3)
		logMsg ( "Scan  line - %d  mod - %d \n",
				line,mod );

	thistime = tickGet ();

	if (rtusim[line]) {
	    if (thistime < (rtusimtime+rtusimdelta) )
		return;
	    rtusimtime = thistime;
	    rtusimaival+=rtusimaidelta;
	    rtusimdival+=rtusimdidelta;
	    if (rtusimaival > rtusimaimax)
		rtusimaival = rtusimaimin;
	    if (rtusimmemwrite != 0) {
		bdbase = bdaddr[0];
		for (i=0; (i < 0x300); i++) {
		    bdbase[i] = 0xff;
		}
	    }
	    return;
	}

	if (mod == 0) {
	    /* Local I/O Processing Here */
	    /* Read the DIs */
	    for (i=0;(i<SLOTSPERCELL);i++) {
		bdbase = bdaddr[i];
		if (pmod->sl[i].card_type == CDTYPEDI8) {
		    pmod->sl[i].divals = ((unsigned long) bdbase[DI_LVALUE] & 0xff);
{int n;for(n=0;n<8;++n) fpnl_write_led((i<<3)+(n+1),((pmod->sl[i].divals&(1<<n))?1:0));}
		} else {
		    if (pmod->sl[i].card_type == CDTYPEDI32) {
		        lval = (unsigned long) bdbase[DI_HVALUE];
		        pmod->sl[i].divals = ( ( (lval << 8) + (unsigned long) bdbase[DI_LVALUE] ) & 0xffff);
		    }
		}
	    }
	    /* Read the AIs */
	    for (i=0;(i<SLOTSPERCELL);i++) {

		bdbase = bdaddr[i];
		if ( (pmod->sl[i].card_type == CDTYPEAI60HZ) ||
		     (pmod->sl[i].card_type == CDTYPEAI50HZ) ) {

		    /* Check the ready timers */
		    for (j=0;(j<AISPERSLOT);j++) {
			if ((pmod->sl[i].ais[j].ai_read_time +
			     pmod->sl[i].ais[j].ai_scan_rate) < thistime) {
			         pmod->sl[i].ais[j].ai_need_read = TRUE;
		    	}
		    }
		    /* Process the current scan state */
		    switch (pmod->sl[i].aistate) {
		        case RTUAIRESET:
			    /* Start a conversion for ZERO point */
			    bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq +
					        AICTL_START_ZOFF_CNV;
			    pmod->sl[i].aistate = RTUAICONVERT;
			    pmod->sl[i].aipoint = 0;
			    pmod->sl[i].aitime = thistime;
			    break;
		        case RTUAIIDLESTATE:
			    /* See if there is another point ready to start and start it */
			    nextaipoint(pmod,i);
			    break;
		        case RTUAISETTLE:
			    if ((pmod->sl[i].aitime + AISETTLETIME) > thistime) {
			        break;
			    }
			    /* Start the conversion process */
			    bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq +
					        AICTL_START_CHAN_CNV;
			    pmod->sl[i].aistate = RTUAICONVERT;
			    pmod->sl[i].aitime = thistime;
			    break;
		        case RTUAICONVERT:
			    /* Set up to Read the Status */
			    if (pmod->sl[i].aipoint == 0) {
			        bdbase[AI_BRD_CTL] = AICTL_CONT_ZOFF_CNV;
			    } else {
			        bdbase[AI_BRD_CTL] = AICTL_CONT_CHAN_CNV;
			    }
			    /* Is the conversion done yet */
			    if ((bdbase[AI_BRD_STAT] & AISTAT_CONVDONE) == 0) {
			        /* Have we timed out the conversion */
			        if ((pmod->sl[i].aitime + MAXAICVTTIME) < thistime) {
				    logMsg("AI conversion time out mod = %d slot = %d point = %d \n",
					    mod,
					    i,
					    pmod->sl[i].aipoint);
				    nextaipoint(pmod,i);
			        }
			        break; /* No */
			    }
			    j = pmod->sl[i].aipoint;
			    if (j == 0) {
			        /* Read the result */
			        bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq |
						    AICTL_SELZOFF |
						    AICTL_READ_LSB;
			        tval = bdbase[AI_VALUE];
			        bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq |
						    AICTL_SELZOFF |
						    AICTL_READ_MSB;
			        tval = tval | ((unsigned short) bdbase[AI_VALUE] << 8);
			        /* Save the result set data valid and reschedule time*/
				if (pmod->sl[i].ais[j].ai_data_valid)
				{
				    if (!pmod->sl[i].ais[j].ai_read_again)
				    {
					if ((((pmod->sl[i].ais[j].aival * rtu_filter) +
					    pmod->sl[i].ais[j].aival) < tval) ||
					    ((pmod->sl[i].ais[j].aival - (pmod->sl[i].ais[j].aival * rtu_filter)) > tval))
					{
					    pmod->sl[i].ais[j].ai_read_again = TRUE;
					}
					else
					{
			        	    pmod->sl[i].ais[j].aival = tval;
			        	    pmod->sl[i].ais[j].ai_data_valid = TRUE;
			        	    pmod->sl[i].ais[j].ai_read_time = thistime;
			        	    pmod->sl[i].ais[j].ai_need_read = FALSE;
					}
				    }
				    else
				    {
			        	pmod->sl[i].ais[j].aival = tval;
			        	pmod->sl[i].ais[j].ai_data_valid = TRUE;
			        	pmod->sl[i].ais[j].ai_read_time = thistime;
			        	pmod->sl[i].ais[j].ai_need_read = FALSE;
					pmod->sl[i].ais[j].ai_read_again = FALSE;
				    }
				}
				else
				{
			            pmod->sl[i].ais[j].aival = tval;
			            pmod->sl[i].ais[j].ai_data_valid = TRUE;
			            pmod->sl[i].ais[j].ai_read_time = thistime;
			            pmod->sl[i].ais[j].ai_need_read = FALSE;
				}
			        /* Start the next operation */
			        nextaipoint(pmod,i);
			    } else {
			        /* Read the result */
				bdbase[AI_DISCON] = 0;
			        bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq |
						    AICTL_SELVSRC |
						    AICTL_READ_LSB;
			        tval = bdbase[AI_VALUE];
			        bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq |
						    AICTL_SELVSRC |
						    AICTL_READ_MSB;
			        tval = tval | ((unsigned short) bdbase[AI_VALUE] << 8);
			        /* Save the result set data valid and reschedule time*/
				if (pmod->sl[i].ais[j].ai_data_valid)
				{
				    if (!pmod->sl[i].ais[j].ai_read_again)
				    {
					if ((((pmod->sl[i].ais[j].aival * rtu_filter) +
					    pmod->sl[i].ais[j].aival) < (tval - pmod->sl[i].ais[0].aival)) ||
					    ((pmod->sl[i].ais[j].aival - (pmod->sl[i].ais[j].aival * rtu_filter)) > (tval - pmod->sl[i].ais[0].aival)))
					{
					    pmod->sl[i].ais[j].ai_read_again = TRUE;
					}
					else
					{
			    	    	    pmod->sl[i].ais[j].aival = tval - 
							   pmod->sl[i].ais[0].aival;
			        	    pmod->sl[i].ais[j].ai_data_valid = TRUE;
			        	    pmod->sl[i].ais[j].ai_read_time = thistime;
			        	    pmod->sl[i].ais[j].ai_need_read = FALSE;
					}
				    }
				    else
				    {
			    	        pmod->sl[i].ais[j].aival = tval - 
							   pmod->sl[i].ais[0].aival;
			        	pmod->sl[i].ais[j].ai_data_valid = TRUE;
			        	pmod->sl[i].ais[j].ai_read_time = thistime;
			        	pmod->sl[i].ais[j].ai_need_read = FALSE;
					pmod->sl[i].ais[j].ai_read_again = FALSE;
				    }
				}
				else
				{
			    	    pmod->sl[i].ais[j].aival = tval - 
							   pmod->sl[i].ais[0].aival;
			            pmod->sl[i].ais[j].ai_data_valid = TRUE;
			            pmod->sl[i].ais[j].ai_read_time = thistime;
			            pmod->sl[i].ais[j].ai_need_read = FALSE;
				}

			        /* Start the next operation */

			        nextaipoint(pmod,i);
			    }
		    }

	        }

	    }
	    pmod->data_valid = TRUE;

	} else {
	    char *work;

	    /* else process remote base fetches */
	    pmod->no_datain_msgs++;
	    if (arc_UpdateALL(mod,&upmsg) != OK) {
		pmod->no_rxnoresp++;
		pmod->no_offlines++;
	    	pmod->holdflag = TRUE;
	    	pmod->is_responding = FALSE;
		return;
	    }
	    /* store away the data returned and mark it all valid*/
	    work = (char*) &upmsg.slot;
	    for (i=0;(i<upmsg.nslots);i++) {
		pmod->data_valid = TRUE;
		/* all members start with their slot number */
		thisslot = ((AISLOT*) work)->slot-1;
		switch (pmod->sl[thisslot].card_type) {
		    case CDTYPEAO:
			for(j=0;(j<AOSPERSLOT);j++) {
			    pmod->sl[thisslot].aovals[j] = 
				((AOSLOT*) work)->aovals[j];
			}
			work += sizeof(AOSLOT);
			break;
		    case CDTYPEAI60HZ:
		    case CDTYPEAI50HZ:
			for(j=0;(j<AISPERSLOT);j++) {
			    pmod->sl[thisslot].ais[j].aival = 
				((AISLOT*) work)->aivals[j];
			    pmod->sl[thisslot].ais[j].ai_data_valid = TRUE; 
			}
			work += sizeof(AISLOT);
			break;
		    case CDTYPEDO:
			tval = 0;
			tbit = 1;
			for(j=0;(j<NUMDOCHANS);j++) {
			    if (((DOSLOT*) work)->dovals[j] != 0) {
				tval = tval | tbit;
			    }
			    tbit = tbit << 1;
			}
			pmod->sl[thisslot].dovals = tval;
			work += sizeof(DOSLOT);
			break; 
		    case CDTYPEDI8:
			tval = 0;
			tbit = 1;
			for(j=0;(j<8);j++) {
			    if (((DI8SLOT*) work)->divals[j] != 0) {
				tval = tval | tbit;
			    }
			    tbit = tbit << 1;
			}
			pmod->sl[thisslot].divals = tval;
			work += sizeof(DI8SLOT);
			break; 
		    case CDTYPEDI32:
			tval = 0;
			tbit = 1;
			for(j=0;(j<16);j++) {
			    if (((DI16SLOT*) work)->divals[j] != 0) {
				tval = tval | tbit;
			    }
			    tbit = tbit << 1;
			}
			pmod->sl[thisslot].divals = tval;
			work += sizeof(DI16SLOT);
			break; 
		}
	    }
	}
    }

/*
/* nextaipoint
/*	Decide which ai point is to be read next
/*	if there is one, start the conversion process and
/*	set the next state. Otherwise set the scan state 
/*	to idle.
*/
static void nextaipoint( CELL *pmod, unsigned short slot)
    {
	unsigned short i, j, lastpoint;
	unsigned long	thistime;
	volatile unsigned char *bdbase;


	thistime = tickGet ();
	bdbase = bdaddr[slot];
	lastpoint = pmod->sl[slot].aipoint;
	for (i=0;(i<AISPERSLOT);i++) {
	    j = ((lastpoint + i) % AISPERSLOT);
	    if (pmod->sl[slot].ais[j].ai_need_read) {
		/* Set the point number */
		pmod->sl[slot].aipoint = j;
		/* Set the start time */
		pmod->sl[slot].aitime = thistime;
		/* See if I am starting a converion on point 0 */
		if (j == 0) {
		    bdbase[AI_BRD_CTL] = pmod->sl[slot].ai_freq +
				        AICTL_START_ZOFF_CNV;
		    pmod->sl[slot].aistate = RTUAICONVERT;
		    return;
		} else {
		    /* Start Conversion on non zero channel */
		    bdbase[AI_CHAN_SELECT] = (1 << (j-1));
		    pmod->sl[slot].aistate = RTUAISETTLE;
		    return;
		}
	    }
	}
	/* None ready yet go idle */
	pmod->sl[slot].aistate = RTUAIIDLESTATE;
    }


/*												
/* checkoutputs											
/*      checks output que and returns if empty.							
/*      if there is something on the queue it attempts to process all                           
/*      pending outputs, doing immediate retries. If attempts fail the request is dropped.      
/*                                                                                              
*/                                                                                              
/* this routine only called from scan_plc */

static	void checkoutputs (unsigned short line)
    {
	OUTQENT *optr;
	unsigned short  regaddr,regval,link,adapter,coiladdr,conv,card,cell;
	unsigned long   coilmask,coilval;
	CELL  *pmod;
	double 		dblaovalue;

	while ((optr=ogetnextp(line)) != NULLP)
	{
	    if ((optr->otype)==AOUTTYPE)
	    {
		link = optr->out.aonode.link;
		adapter = optr->out.aonode.adapter;

		cell = (adapter/SLOTSPERCELL);

		pmod = plcs[link][cell];
		if (pmod == NULL)
		{
			if (!allocate_plc(link, adapter))
				return; /* if not allocated, try. if can't allocate, get out. */
			pmod = plcs[link][cell];
		}

		regaddr = optr->out.aonode.aosignal;
		regval = optr->out.aonode.aovalue;
		dblaovalue = optr->out.aonode.dblaovalue;
		conv = optr->out.aonode.conversion;
		card = optr->out.aonode.card;

		if (mltrace[line] == 2) 
			logMsg ( "Analog Output to line %d, address %d, signal  - %d\n",
				link, adapter, regaddr );

		if ((!pmod->cmd_force_offline) && (resresponse[link]))
		{
		    if (!(writeao(link,adapter,card,pmod,regaddr,regval,dblaovalue,conv )))
		    {
		        pmod->badoutcnt++;		    
		    }
		}
	    }
	    else if ((optr->otype)==BOUTTYPE)
	    {
		link = optr->out.bonode.link;
		adapter = optr->out.bonode.adapter;

		cell = (adapter/SLOTSPERCELL);
		pmod = plcs[link][cell];
		if (pmod == NULL)
		{
			if (!allocate_plc(link, adapter))
				return; /* if not allocated, try. if can't allocate, get out. */
			pmod = plcs[link][cell];
		}

		coiladdr = optr->out.bonode.bosignal;
		coilval = optr->out.bonode.bovalue;
		coilmask = optr->out.bonode.bomask;

		if (mltrace[line] == 2) 
			logMsg ( "Binary Output to line %d, address %d, signal  - %d\n",
				link, adapter, coiladdr );

		if ((!pmod->cmd_force_offline) && (resresponse[link]))
		{
		    if (!(writebo (link,adapter,pmod,coiladdr,coilmask,coilval)))
		    {
			pmod->badoutcnt++;
		    }
		}
	    }
	    oreleasefull(line);     /* release the output que entry */
	}
    }

/*
/* writeao(link,adapter,pmod,regaddr,regval,dblaovalue,conv )
/*
/*	write an analog value to the microsol I/O
/*
*/
static void writeao(unsigned short link,unsigned short adapter,
		    unsigned short card, CELL *pmod,unsigned short regaddr,
		    unsigned short regval, double dblaovalue,
		    unsigned short conv )
    {
	unsigned short slot,cell,cval;
	volatile unsigned char *bdbase;
	AOCMD			aomsg;
	AIRATECMD		ratemsg;
	unsigned long		sec,msec;

	slot = (adapter % SLOTSPERCELL);
	cell = (adapter/SLOTSPERCELL);
	bdbase = bdaddr[slot];
	pmod->no_dataout_msgs++;

	if (card == RTUAIRATE) {
	    pmod->sl[slot].ais[regaddr].ai_scan_rate = regval;         /*ktb:'ai_freq' -> 'ai_scan_rate'*/
	    if (cell == 0) {
		return;
	    }
	    sec = pmod->sl[slot].ais[regaddr].ai_scan_rate/60;         /*ktb:'ai_freq' -> 'ai_scan_rate'*/
	    msec = (17*(pmod->sl[slot].ais[regaddr].ai_scan_rate%60)); /*ktb:'ai_freq' -> 'ai_scan_rate'*/
logMsg("writeao-SetRateAI(%d,%d,%d) s:%u ms:%u\n",cell,slot+1,regaddr,sec,msec);
	    if (arc_SetRateAI (cell,slot+1,regaddr,sec,msec,&ratemsg) != OK) {
		pmod->badoutcnt++;
		pmod->no_rxnoresp++;
		pmod->holdflag = TRUE;
		pmod->is_responding = FALSE;
		return;
	    }
	    return;
	}

	/* save the value in memory */
	pmod->sl[slot].aovals[regaddr] = regval;

	
	if ( cell == 0) {
	    /* Process Local I/O */
	    /* write the 12 bit value to the card */
	    bdbase[(regaddr << 2)+1] = (regval & 0xff);
	    bdbase[(regaddr << 2)+3] = ( (regval >> 8) & 0xf);
 	    /* hit the activate control by reading it */
	    cval = bdbase[AO_ACTIVATE];
	} else {
	    /* send the write bank DO command */
	    if (arc_WriteAO (cell,slot+1,regaddr+1,
				 pmod->sl[slot].aovals[regaddr],
				 &aomsg) != OK) {
		pmod->badoutcnt++;
		pmod->no_rxnoresp++;
		pmod->holdflag = TRUE;
		pmod->is_responding = FALSE;
		return;
	    }
	}

    }

/*
/* writebo(link,adapter,pmod,coiladdr,coilmask,coilval)
/*
/*	write an analog value to the microsol I/O
/*
*/
static void writebo(unsigned short link,unsigned short adapter,
		    CELL *pmod,unsigned short coiladdr,
		    unsigned long coilmask, unsigned long coilval)
    {
	unsigned short 		slot,cell,cval;
	volatile unsigned char 	*bdbase;
	DOCMD			domsg;

	slot = (adapter % SLOTSPERCELL);
	cell = (adapter/SLOTSPERCELL);
	bdbase = bdaddr[slot];

	pmod->no_dataout_msgs++;

	/* save the value in memory */
	pmod->sl[slot].dovals = (pmod->sl[slot].dovals) & (~coilmask);
	pmod->sl[slot].dovals = (pmod->sl[slot].dovals) | (coilval);

	if ( cell == 0) {
	    /* Process Local I/O */
	    /* write the 12 bit value to the card */
	    bdbase[DO_VALUE] = pmod->sl[slot].dovals;
	    fpnl_write_led( (slot<<3) + (coiladdr+1), (coilval?1:0) );
	} else {
	    /* send the write bank DO command */
	    if (arc_WriteDOBank (cell,slot+1,pmod->sl[slot].dovals,&domsg) != OK) {
		pmod->badoutcnt++;
		pmod->no_rxnoresp++;
		pmod->holdflag = TRUE;
		pmod->is_responding = FALSE;
		return;
	    }
	}
    }

/*                                                                                    	
/*      AIDRIVER
/*                                                                              	
/*      This routines main purpose is to return the analog input value for the         	
/*      specified module. For a module value                                           	
/*      a (-1) will be returned if there is no valid data ready.                       	
/*                                                                                     	
*/
/* this routine not called from within this driver */

int AIDRIVER (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, short *pvalue, unsigned short conversion,
		unsigned char *resname)
    {
	CELL  *pmod;
	unsigned short	*pdata;
	unsigned short  slot,cell;

	adapter--;
	slot = (adapter % SLOTSPERCELL);
	cell = (adapter / SLOTSPERCELL);

	if (link > RTULINES)
		return (-1);
	if (adapter > (MAXCELLS*SLOTSPERCELL))
		return (-1);
	if ((card != RTUAI) && (card != MPSEUDO))
		return (-1);
	if (conversion != conversion)
		return (-1); /* No warnings */

	signal--; /* Signals are defined as 1 relative */

	if ((card == RTUAI) && (signal > AISPERSLOT))
		return (-1);

	pmod = plcs[link][cell];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, cell))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
		pmod = plcs[link][cell];
	}

	pmod->was_requested = TRUE;


	if (card == MPSEUDO)
	{
	    signal++;
	    if (!resresponse[link])
	    	return (-2);
	    switch (signal)
	    {
	    case 0: /* Were the PLC values requested */
		*pvalue = pmod->was_requested;
		break;

	    case 1: /* Is the PLC responding */
		*pvalue = pmod->is_responding;
		break;

	    case 5: /* Output Messages */
		*pvalue = pmod->no_dataout_msgs & 0x7ffff;
		break;

	    case 6: /* Input Messages */
		*pvalue = pmod->no_datain_msgs & 0x7ffff;
		break;

	    case 7: /* No Receiver Responses */
		*pvalue = pmod->no_rxnoresp & 0x7ffff;
		break;

	    case 8: /* No Transmitter Response */
		*pvalue = pmod->no_txnoresp & 0x7ffff;
		break;

	    case 9: /* Data Echo Errors */
		*pvalue = pmod->no_dataerr & 0x7ffff;
		break;

	    case 10: /* Message Retries */
		*pvalue = pmod->no_retries & 0x7ffff;
		break;

	    case 11: /* RX Hardware errors */
		*pvalue = pmod->no_rxhwerrs & 0x7ffff;
		break;

	    case 12: /* TX Hardware errors */
		*pvalue = pmod->no_txhwerrs & 0x7ffff;
		break;

	    case 13: /* CRC errors */
		*pvalue = pmod->no_crcerrs & 0x7ffff;
		break;

	    case 14: /* Offline transitions */
		*pvalue = pmod->no_offlines & 0x7ffff;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat & 0x7ffff;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline & 0x7ffff;
		break;

	    case 17: /* Failed output commands */
		*pvalue = pmod->badoutcnt & 0x7ffff;
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	if (resname != 0)
	{ 
	    register_resource(link,pmod,resname);
	}

	if (!resresponse[link])
	    return (-1);
	if (pmod->holdflag)
	    return (-2);
	if (!pmod->is_responding)
	    return (-1);

	/* Make Allowance for Simulated I/O mode */
	if (rtusim[link]) {
		pvalue[0] = rtusimaival;
		return(0);
	}

	/* Does the slot have the right hardware */
	if ((pmod->sl[slot].card_type != CDTYPEAI60HZ) &&
	    (pmod->sl[slot].card_type != CDTYPEAI50HZ) ) {
		return(-1);
	}

	/* fixup to go around zero offset signal */

	if (signal == 8) {
	    signal = 0;
	} else {
	    signal++;
	}

	if (!pmod->sl[slot].ais[signal].ai_data_valid)
	    return(-1);

	pdata = ((unsigned short*) & (pmod->sl[slot].ais[signal].aival));

	/* NOTE !! MULTIPLE PRECISION VALUES DO NOT WORK */

	/* Select the Size and Type of Response based on Conversion */
	switch (conversion)
	{
	    case(AICNVDOUBLEINT):
		return(-1);
		break;
	    case(AICNVFLOAT):
		return(-1);
		break;
	    case(AICNVDOUBLEFLT):
		return(-1);
		break;
	    case(AICNVINTEGER):
	    default:
		*pvalue = *pdata;
		break;

 	}
	return(0);

    }

/*  										
/*      AODRIVER
/*                                                                            	
/*      This routines main purpose is to set requested analog outputs to the  	
/*      specified module.                                                     	
*/                                                                            	
/* this routine not called from within this driver */


int AODRIVER (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
	    unsigned short signal,double dvalue, unsigned short value,
	    unsigned short conversion, unsigned char *resname)

    {
	CELL  *pmod;
	unsigned short  blkix;
	OUTQENT *oqp;
	unsigned short  slot,cell;

	adapter--;

	slot = (adapter % SLOTSPERCELL);
	cell = (adapter / SLOTSPERCELL);

	signal--; /* Signals are defined as 1 relative */

	if (link > RTULINES)
		return (-1);

	if (cell > MAXCELLS)
		return (-1);

	if ((card != RTUAO) && (card != MPSEUDO) && (card != RTUAIRATE))
		return -1;

	pmod = plcs[link][cell];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, cell))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
		pmod = plcs[link][cell];
	}


	if (card == MPSEUDO)
	{
	    signal++;

	    switch (signal)
	    {

	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
			initstats(pmod);
		break;

	    case 16: /* Force the PLC online/offline */
		pmod->cmd_force_offline = (value != 0);
		pmod->is_responding = (value != 0);
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	if (resname != 0)
	{ 
	    register_resource(link,pmod,resname);
	}

	if ((signal > AOSPERSLOT) && (card == RTUAO))
	{
	    return (-1);
	}
	if ((signal > AISPERSLOT) && (card == RTUAIRATE))
	{
	    return (-1);
	}

	if ((rtusim[link]) && (card != RTUAIRATE)) {
	    pmod->sl[slot].aovals[signal] = value;
	    return(0);
	}

	/* Does the slot have the right hardware */
	if ((card == RTUAO) && (pmod->sl[slot].card_type != CDTYPEAO)) {
		return(-1);
	}

	/* Does the slot have the right hardware */
	if ( (card == RTUAIRATE) && 
	     ( (pmod->sl[slot].card_type != CDTYPEAI60HZ) &&
	       (pmod->sl[slot].card_type != CDTYPEAI50HZ) )  ) {
		return(-1);
	}

	pmod->was_requested = TRUE;

	if (!resresponse[link])
		return (-1);

	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}
	oqp = ogetemptyp (link);
	if (oqp == NULLP)
	{
	    return (-1);		/* output que is full */
	}
	oqp->otype = AOUTTYPE;
	oqp->out.aonode.link = link;
	oqp->out.aonode.adapter = adapter;
	oqp->out.aonode.card = card;
	oqp->out.aonode.aosignal = signal;
	oqp->out.aonode.aovalue = value;
	oqp->out.aonode.dblaovalue = dvalue;
	oqp->out.aonode.conversion = conversion;

	omarkfull (link);		/* Mark this entry as ready for output */

	return (0);
    }


/*
/*      AOREAD
/*                                                                          	
/*      This routines main purpose is to return the value last read back     	
/*      from the analog output hardware. This is used to initialize analog   	
/*      outputs.                                                             	
*/
/* this routine not called from within this driver */


int AOREAD (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
	  unsigned short signal, unsigned short *pvalue,
	  unsigned short conv, unsigned char *resname)

    {
	CELL  *pmod;
	unsigned short  blkix, *pdata;
	unsigned short  slot,cell;

	adapter--;
	slot = (adapter % SLOTSPERCELL);
	cell = (adapter / SLOTSPERCELL);
	
	signal--; /* Signals are defined as 1 relative */

	if (link > RTULINES)
		return (-1);
	if (cell > MAXCELLS)
		return (-1);
	if ((card != RTUAO) && (card != MPSEUDO))
		return -1;

	pmod = plcs[link][cell];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, cell)) return(-1); /* if not allocated, try. if can't allocate, get out. */
		pmod = plcs[link][cell];
	}

	if (card == MPSEUDO)
	{
	    signal++;
	    switch (signal)
	    {

	    case 15: /* read statistics command value */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Read the current PLC offline command status */
		*pvalue = pmod->cmd_force_offline;
		break;


	    default:
		return(-1);
	    }
	    return (0);
	}

	if (resname != 0)
	{ 
	    register_resource(link,pmod,resname);
	}

	if (signal > AOSPERSLOT)
	{
	    return (-1);
	}

	if (rtusim[link]) {
	    *pvalue = 0;
	    return(0);
	}

	/* Does the slot have the right hardware */
	if (pmod->sl[slot].card_type != CDTYPEAO) {
		return(-1);
	}

	pmod->was_requested = TRUE;

	if (!resresponse[link])
		return (-1);

	if (pmod->holdflag)
		return (-2);

	if (!pmod->is_responding)
	{
	    return (-1);
	}

	if (!pmod->data_valid)
	{
	    return (-1);
	}

	pdata = &(pmod->sl[slot].aovals[signal]);

	/* NOTE !! MULTIPLE PRECISION VALUES MUST RESIDE IN A SINGLE BLOCK */

	/* Select the Size and Type of Response based on Conversion */
	switch (conv)
	{
	    case(ASCFLT):
		pvalue[0] = pdata[1];
		pvalue[1] = pdata[0];
		break;
	    default:
		*pvalue = *pdata;
		break;
 	}
	return (0);
    }

/*
/*      BIDRIVER
/*                                                                               	
/*      The main purpose of this routine is to return the request digital input  	
/*      value(s) from the requested card. This routine will return a (-1) if the  	
/*      requested module is the wrong type or the data is not yet valid.          	
/*                                                                                	
*/
/* this routine not called from within this driver */



int BIDRIVER (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long mask,unsigned long *pvalue,
		unsigned char *resname)

    {
	CELL  *pmod;
	unsigned short blkix;
	unsigned long tvalue;
	unsigned short  slot,cell;

	adapter--;
	slot = (adapter % SLOTSPERCELL);
	cell = (adapter / SLOTSPERCELL);

	signal--; /* Signals are defined as 1 relative */

	if (link > RTULINES)
		return (-1);
	if (cell > MAXCELLS)
		return (-1);

	pmod = plcs[link][cell];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, cell))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
		pmod = plcs[link][cell];
	}

	pmod->was_requested = TRUE;

	if (card == MPSEUDO)
	{
	    signal++;
	    switch (signal)
	    {
		    case 0: /* Were the PLC values requested */
			*pvalue = pmod->was_requested;
			break;

		    case 1: /* Is the PLC responding */
			*pvalue = pmod->is_responding;
			break;

		    case 15: /* reset statistics */
			*pvalue = pmod->cmd_reset_stat;
			break;

		    case 16: /* Force the PLC offline */
			*pvalue = pmod->cmd_force_offline;
			break;

		    default:
			return (-1);
	    }
	    return (0);
	}

	if (card != RTUDI) {
	    return(-1);
	}

	if (resname != 0)
	{ 
	    register_resource(link,pmod,resname);
	}

	if (rtusim[link]) {
		pvalue[0] = (rtusimdival & mask);
		return(0);
	}

	/* Does the slot have the right hardware */
	if ((pmod->sl[slot].card_type != CDTYPEDI8) &&
	    (pmod->sl[slot].card_type != CDTYPEDI32) ) {
		return(-1);
	}

	if ( (pmod->sl[slot].card_type != CDTYPEDI8) &&
	     (signal >= 8))  {
		return (-1);
	} else {
	    if (signal >= 16) {
		return(-1);
	    }
	}

	if (!resresponse[link])
		return (-1);
	if (pmod->holdflag)
		return (-2);
	if (!pmod->is_responding)
		return (-1);
	if (!pmod->data_valid)
	    return (-1);

	*pvalue = pmod->sl[slot].divals & mask;

	return (0);

    }
/*                                                                                	
/*      BODRIVER
/*                                                                                 	
/*      The main purpose of this routine is to set the request digital output      	
/*      value(s) on the requested card. This routine will return a (-1) if the     	
/*      requested module is the wrong type or not configured yet.                  	
/*                                                                                 	
*/
/* this routine not called from within this driver */


int BODRIVER (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long value, unsigned long mask,
		unsigned char *resname)


    {
	CELL  *pmod;
	OUTQENT *oqp;
	unsigned short  slot,cell;

	adapter--;
	slot = (adapter % SLOTSPERCELL);
	cell = (adapter / SLOTSPERCELL);

	signal--; /* Signals are defined as 1 relative */

	if (link > RTULINES)
		return (-1);
	if (cell > MAXCELLS)
		return (-1);
	if ((card != RTUDO) && (card != MPSEUDO))
		return (-1);

	pmod = plcs[link][cell];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, cell))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
		pmod = plcs[link][cell];
	}

	pmod->was_requested = TRUE;

	if (card == MPSEUDO)
	{
	    signal++;
	    switch (signal & 0x1f)
	    {

	    case 15: /* Reset the accumulated statistics */
		if (value != 0) initstats(pmod);
		break;

	    case 16: /* Force the PLC offline */
		pmod->cmd_force_offline = (value != 0);
		pmod->is_responding = (value != 0);
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	if (resname != 0)
	{ 
	    register_resource(link,pmod,resname);
	}

	if (signal > NUMDOCHANS)
	{
	    return (-1);
	}

	if (rtusim[link]) {
		/* save the value in memory */
		pmod->sl[slot].dovals = (pmod->sl[slot].dovals) & (~mask);
		pmod->sl[slot].dovals = (pmod->sl[slot].dovals) | (value);
		if (cell == 0)
		    fpnl_write_led( (slot<<3) + (signal+1), (value?1:0) );
		return(0);
	}


	/* Does the slot have the right hardware */
	if (pmod->sl[slot].card_type != CDTYPEDO) {
		return(-1);
	}

	if (!resresponse[link])
		return (-1);

	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}
	oqp = ogetemptyp (link);
	if (oqp == NULLP)
	{
	    return (-1);		/* output que is full */
	}
	oqp->otype = BOUTTYPE;
	oqp->out.bonode.link = link;
	oqp->out.bonode.adapter = adapter;
	oqp->out.bonode.card = card;
	oqp->out.bonode.bosignal = signal;
	oqp->out.bonode.bomask = mask;
	oqp->out.bonode.bovalue = value;

	omarkfull (link);		/* Mark this entry as ready for output */

	return (0);
    }
/*
/*      BOREAD
/*                                                                           	
/*      This routines main purpose is to return the value last read back      	
/*      from the digital output hardware. This is used to initialize digital  	
/*      outputs. This will return a (-1) if the module is the wrong type or    	
/*      the data is not yet valid                                              	
*/
/* this routine not called from within this driver */


int BOREAD (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long *pvalue, unsigned long mask,
		unsigned char *resname)

    {
	CELL  *pmod;
	unsigned long	tvalue;
	unsigned short  slot,cell;

	adapter--;
	slot = (adapter % SLOTSPERCELL);
	cell = (adapter / SLOTSPERCELL);

	signal--; /* Signals are defined as 1 relative */

	if (link > RTULINES)
		return (-1);
	if (cell > MAXCELLS)
		return (-1);
	if ((card != RTUDO) && (card != MPSEUDO))
		return (-1);

						/* masks aligned on 32 bits */

	pmod = plcs[link][cell];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, cell))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
		pmod = plcs[link][cell];
	}


	pmod->was_requested = TRUE;

	if (card == MPSEUDO)
	{
	    signal++;
	    switch (signal)
	    {

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	if (resname != 0)
	{
	    register_resource(link,pmod,resname);  /* under what condition(s) could this ever test false? */
	}

	if ((signal) > NUMDOCHANS)
	{
	    return (-1);
	}

	if (rtusim[link]) {
	    *pvalue = 0;
	    return(0);
	}

	/* Does the slot have the right hardware */
	if (pmod->sl[slot].card_type != CDTYPEDO) {
		return(-1);
	}

	if (!resresponse[link])
		return (-1);
	if (pmod->holdflag)
		return (-2);
	if (!pmod->is_responding)
		return (-1);
	if (!pmod->data_valid)
	    return (-1);

	*pvalue = pmod->sl[slot].dovals & mask;

	return (0);
    }

/*
/* register_resource                                                              	
/*                                                                                	
/*	This routine is used to initialize the resource pointers                  	
/*  If this is the first time the resource for this line is being                  	
/*  referenced the pointer to the resource table entry will be set                  	
/*  and the resource name will be saved. If the resouce name has                    	
/*  already been set, this routine will verify that all other points                 	
/*  for this driver are using the same resource name.                                	
/*                                                                                   	
*/                                                                                    	
/* this routine called from AIDRIVER, AODRIVER, AOREAD, BOREAD,				
			    BIDRIVER, and BODRIVER */                               	

static void  register_resource(unsigned short link,CELL *pmod,unsigned char *resname)
    {
	if (resnames[link][0] == 0) /* This is the first reference */
	{
	    if (resGetByName( resname,&prestable[link]) != RES_OK)
	    {
		logMsg ( "Resource not found in table - %s \n",
				resname );
	    }
	    else strcpy(resnames[link],resname);
	}
	else
	{
	    if (strcmp(resnames[link],resname) != 0)
	    {
		logMsg ("Resource name conflict - %s,  %s \n",
				resnames[link],
				resname );
	    }
	}
    }
/*                                                                  		
/*      Initialize the Queue                                        		
*/                                                                        	
/* not called from within this driver */

static	void outqueinit (unsigned short line)
    {
	oq[line]->ooutix = oq[line]->oinix = 0;
    }


/*                                                                     		
/*  ogetempty - get the next available slot on the output list         		
*/
/*    called from AODRIVER and BODRIVER   */

static	OUTQENT *ogetemptyp (unsigned short line)
    {
	if (((oq[line]->oinix + 1) % OUTQSIZE) != oq[line]->ooutix)
	{
	    return (&oq[line]->outque[oq[line]->oinix]);
	}
	if (mltrace[line] == 1) 
		logMsg ( "Output Que Full line - %d\n",line);
	return (NULLP);
    }


/*                                                                    		
/* omarkfull                                                          		
/*      mark the next block as full                                     	
*/
/*  called from AODRIVER and BODRIVER   */

static	void omarkfull (unsigned short line)
	{
	    oq[line]->oinix = ((oq[line]->oinix + 1) % OUTQSIZE);
	}


/*                                                                      	
/*  ogetnextp - get the next full slot on the output list               	
*/                                                                      	
/* only called from checkoutputs */

static	OUTQENT *ogetnextp (unsigned short line)
    {
	if (oq[line]->ooutix != oq[line]->oinix)
	{
	    return (&oq[line]->outque[oq[line]->ooutix]);
	}
	else return (NULLP);
    }


/*                                                       	               	
/* oreleasefull - release the current full output block    	              	
*/                                                                            	
/* only called from checkoutputs */

static	void oreleasefull (unsigned short line)
    {
	if (oq[line]->ooutix == oq[line]->oinix)
		return;
	oq[line]->ooutix = ((oq[line]->ooutix + 1) % OUTQSIZE);
    }
/*                                                                             	
/*                                                                              
/*	allocate_plc                                                            
/*                                                                              
/*	The purpose of this routine is to allocate the memory for a single      
/*	plc data structure. If not successful, a console error message is  	
/*	printed and an error flag is returned to the caller.                    
/*	If successful, the newly allocated data structures are initialized.     
/*                                                                             	
*/

static int allocate_plc (unsigned short line, unsigned short plc )
   {
	plcs[line][plc] = (CELL *)calloc(1,(sizeof(CELL)));
	if (!plcs[line][plc])
	{
		logMsg ("Unable to allocate memory for CELL data structure on line %d, plc %d\n",line ,plc);
		 return(FALSE);
	}
	else
	{
		logMsg ("Allocated memory for CELL data structure on line %d, plc %d\n",line ,plc);
		init_plc_tab(line, plc);	     /*	initialize the data structures. */
	}

	return(TRUE);
   }

/*                                                                           	
/*      check_line_activity
/*                                                                             	
/*      The purpose of this routine is to process a receive character interrupt	
/*      for a given line.                                                      	
/*                                                                             	
*/
/* only called from MAIN  */


static	int check_line_activity (unsigned short line, unsigned short predelay )

    {
	RXLINE          *rxp;
	long		thistime;
	short		status;

	if (predelay != 0)
		taskDelay(predelay);
	thistime = tickGet();
	if ((thistime-activity_time[line]) < ACTIVITYTIME)
		return (lastactivity[line] || (line_activity[line]));
	lastactivity[line] = status = line_activity[line];
	line_activity[line] = FALSE;
	activity_time[line] = thistime;
	rxp = rxlinetab[line];
	rxp->rxstate = RXIGNORE;

	return (status);
    }


/*
/*      init_plc_tab
/*
/*      The purpose of this routine is to initialize the line tables for the
/*      analog devices I/O
/*
*/
/* only called from allocate_plc  */


static	void init_plc_tab (unsigned short line, unsigned short mod)

    {
	CELL  *pmod;

	pmod = plcs[line][mod];
	if (!pmod)
		return;

	pmod->is_responding = FALSE;
	pmod->data_valid = FALSE;
	pmod->holdflag = FALSE;
	pmod->was_requested = FALSE;
	pmod->cmd_force_offline = FALSE;

	initplc(pmod,mod);
	initstats(pmod);

	/* Initialize this tasks semaphore*/
	semids[line][0]=semBCreate(SEM_Q_PRIORITY, SEM_EMPTY);
	semids[line][1]=semBCreate(SEM_Q_PRIORITY, SEM_EMPTY);
    }





/*
/* Cold Init a PLC
/*
*/
/* only called from init_line_tabs  */


static void initplc (CELL  *pmod,unsigned short mod)

    {

	unsigned short i,j;
	volatile unsigned char *bdbase;
	
	for (i=0;(i<SLOTSPERCELL);i++) {
	    pmod->sl[i].is_initialized = FALSE;
	    pmod->sl[i].aipoint = 0;
	    pmod->sl[i].aistate = RTUAIRESET;
	    pmod->sl[i].aitime = 0;
	    for (j=0;(j<AISPERSLOT);j++) {
		pmod->sl[i].ais[j].aival = 0;
		pmod->sl[i].ais[j].ai_data_valid = FALSE;
/*logMsg("init ai (%d,%d,%d) = %u\n",mod,i+1,j,*/
		pmod->sl[i].ais[j].ai_scan_rate = RTUDEFAULTAIRATE/*)*/;
		pmod->sl[i].ais[j].ai_read_time = 0;
		pmod->sl[i].ais[j].ai_need_read = FALSE;
	    }
	    for (j=0;(j<AOSPERSLOT);j++) {
		pmod->sl[i].aovals[j] = 0;
	    }
	    pmod->sl[i].dovals = 0;
	    pmod->sl[i].divals = 0;

	    if (mod == 0) {

	        /* The rest of this initialization is for local I/O boards only */

	        /* Read in the card type and save it */
	        bdbase = bdaddr[i];
	        bdbase[SLOT_ID] = 0xff;
	        pmod->sl[i].card_type = bdbase[SLOT_ID];
	        if (pmod->sl[i].card_type == CDTYPEAI60HZ) {
		    /* Clear the Channel Select */
		    /* Set the control register to the correct freq */
		    bdbase[AI_CHAN_SELECT] = 0;
		    bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq = AICTL_RATE60HZ;
	        }
	        if (pmod->sl[i].card_type == CDTYPEAI50HZ) {
		    /* Clear the Channel Select */
		    bdbase[AI_CHAN_SELECT] = 0;
		    /* Set the control register to the correct freq */
		    bdbase[AI_BRD_CTL] = pmod->sl[i].ai_freq = AICTL_RATE50HZ;
	        }
	        if (pmod->sl[i].card_type == CDTYPEAO) {
		    /* Clear all of the output values */
		    for (j=0;(j<NUMAOCHANS);j++) {
		        bdbase[(j<<2)+1] = 0;
		        bdbase[(j<<2)+3] = 0;
		    }
		    /* Activate the converter by reading this location*/
		    j = bdbase[AO_ACTIVATE];
		    /* Set all of the relays */
		    bdbase[AO_RLY_CTL] = 0x0f;
		    /* read back and verify all of the relays */
		    if (bdbase[AO_RLY_STAT] != 0x0f) {
		        logMsg ("AO relay status read back failure mod = %d  slot = %d \n",
			     mod,
			     i);
		    }
	        }
	        if (pmod->sl[i].card_type == CDTYPEDO) {
		    /* Clear all of the output values */
		    bdbase[DO_VALUE] = 0;
	        }
	    }
	}
    }

/*
/*                                                              	
/* initialize the statistics for a Microsol RTU
/*                                                              	
*/                                                               	
/* called from AODRIVER, BODRIVER, and init_line_tabs  */


static	void initstats (CELL *pmod)

    {
	pmod->no_dataout_msgs = 0;  /* 5  Count Output Msgs 		*/
	pmod->no_datain_msgs = 0;   /* 6  Count Data Requests 		*/
	pmod->no_rxnoresp = 0;      /* 7  Count No Responses 		*/
	pmod->no_txnoresp = 0;      /* 8  Count No Responses 		*/
	pmod->no_dataerr = 0;       /* 9  Count Message Echo Errors 	*/
	pmod->no_retries = 0;       /* 10 Count Retries 		*/          	
	pmod->no_rxhwerrs = 0;      /* 11 Count Comm Hardware Errors	*/
	pmod->no_txhwerrs = 0;      /* 12 Count Comm Hardware Errors 	*/
	pmod->no_crcerrs = 0;       /* 13 Count CRC Errors 		*/
	pmod->no_offlines = 0;      /* 14 Count Off Line Transitions 	*/
	pmod->badoutcnt = 0;	    /* 17 Count failed output commands 	*/
    }

/*                                                                       
/* cellloopback                                                            
/*                                                                        
*/                                                                          	
/* only called from on_line */


static	unsigned short cellloopback (unsigned short line, unsigned short mod,
			 CELL *pmod)

    {

	IOCFGMSG        cfgmsg;
	AIRATECMD	ratemsg;	
	unsigned long	sec,msec;
	unsigned short	i,j;

	if (mod == 0)
	    return (TRUE);
	/* request slot configurations for this cell */
	pmod->no_datain_msgs++;
	if (arc_GetConfig (mod,&cfgmsg) != OK) {
	    pmod->no_rxnoresp++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return(FALSE);
	}

	/* Save the board types */
	for (i=0;(i<SLOTSPERCELL);i++) {
	    pmod->sl[i].card_type = cfgmsg.cfg[i].physid;
	}
	/* Set the Scan Rates for all Ai Boards */
	for (i=0;(i<SLOTSPERCELL);i++) {
	    if ( (pmod->sl[i].card_type == CDTYPEAI60HZ) ||
		 (pmod->sl[i].card_type == CDTYPEAI50HZ) ) {

		for (j=0;(j<AISPERSLOT);j++) {
		    sec = pmod->sl[i].ais[j].ai_scan_rate/60;
		    msec = (17*(pmod->sl[i].ais[j].ai_scan_rate % 60));
/*logMsg("cellloopback-SetRateAI(%d,%d,%d) s:%u ms:%u\n",mod,i+1,j,sec,msec);*/
		    if (arc_SetRateAI (mod,i+1,j,sec,msec,&ratemsg) != OK) {
			pmod->no_rxnoresp++;
			pmod->holdflag = TRUE;
			pmod->is_responding = FALSE;
			return(FALSE);
		    }
		}
	    }
	}
	return(TRUE);
    }


/*                                                                           	
/*     prepresponses                                                            
/*                                                                             	
/*      The purpose of this routine is to mark the PLCs as on hold and the    	
/*	data as not valid until read again from the PLC                       	
/*                                                                           	
*/
/* only called from MAIN */

static	void prepresponses (unsigned short line)

    {
	unsigned short  mod;
	CELL  *pmod;

	for (mod = 0; (mod <= MAXCELLS); mod++)
	{
            pmod = plcs[line][mod];
	    if (pmod)				/* verify pointer validity prior to use */
	    {
	    	pmod->holdflag = TRUE;
		mark_not_valid(pmod);
            }
	}
    }



/*                                                                  	
/*      any_responding                                               	
/*                                                                   	
/*      The purpose of this routine is to see if any units are       	
/*	responding on this line                                      	
/*                                                                   	
*/
/* only called from MAIN */

static	int any_responding (unsigned short line)

    {
	unsigned short  mod;
	CELL  *pmod;

	pmod = plcs[line][0];
	if ( (pmod) && (pmod->was_requested) && (pmod->is_responding) ) {
	    return(TRUE);
	}
	for (mod = 1; (mod <= MAXCELLS); mod++)
	{
	    pmod = plcs[line][mod];
	    if (pmod)			/* verify pointer validity prior to use */
	    {
		if (pmod->is_responding)
			 return(TRUE);
	    }
	}
	return(FALSE);
    }


/*                                                                          	
/*      dump_mod_table                                                       	
/*                                                                           	
/*      The purpose of this routine is to dump the module description        	
/*      portion of the analog devices driver tables for the selected line.   	
/*      It will print out two lines for each module which is responding.     	
/*      The first line will give the general module header, the second line 	
/*      will give module type specific info.                                	
*/
/* not called from within this driver */

void TABLEINFO (unsigned short line, unsigned short maddr)

    {
	CELL  *pmod;
	short           ix;
	unsigned	short	i, n, j; 
	unsigned	short req, val, value, addr, firstaiinblk, firstreginblk, linecnt;
	unsigned	long	bigvalue;
	unsigned	char	buffer[200];


	pmod = plcs[line][maddr];
	if (!pmod)
	{
	    logMsg("Line/Cell Address Not Defined \n");
	    return;
	}

	printf (" Was requested = %d \n", pmod->was_requested);
	printf (" Is responding = %d \n", pmod->is_responding);
	printf (" Data Is Valid = %d \n", pmod->data_valid);
	printf (" # Output messages = %d \n", pmod->no_dataout_msgs);
	printf (" # Data in messages = %d \n", pmod->no_datain_msgs);
	printf (" #No Rx Response = %d \n", pmod->no_rxnoresp);
	printf (" #No Tx Responses = %d \n", pmod->no_txnoresp);
	printf (" #Data Errors = %d \n", pmod->no_dataerr);
	printf (" #Retries = %d \n", pmod->no_retries);
	printf (" #Rx Hw Errors = %d \n", pmod->no_rxhwerrs);
	printf (" #Tx Hw Errors = %d \n", pmod->no_txhwerrs);
	printf (" #Crc Errors = %d \n", pmod->no_crcerrs);
	printf (" #Off Lines = %d \n", pmod->no_offlines);
	printf (" #Failed Outputs = %d \n", pmod->badoutcnt);


	printf (" Resource Responding = %d \n", resresponse[line]);
	printf (" Module Is Responding = %d \n", pmod->is_responding );
	printf (" Hold flag  = %d \n", pmod->holdflag );

	for (i=0;(i<SLOTSPERCELL);i++) {
	    switch (pmod->sl[i].card_type) {
		case CDTYPEAO:
		    printf("Cell = %d \tSlot = %d \t AO Board \tpt1 = %x, \tpt2 = %x, \tpt3 = %x, \tpt4 = %x, \t\n",
				maddr,
				i,
				pmod->sl[i].aovals[0],
				pmod->sl[i].aovals[1],
				pmod->sl[i].aovals[2],
				pmod->sl[i].aovals[3]
			   );
		    break;
		case CDTYPEAI60HZ:
		    printf("Cell = %d \tSlot = %d \t AI60HZ Board\n",
				maddr,
				i
			  );
		    for (j=0;(j<AISPERSLOT);j++) {
			printf("\t\tAI Data Valid %x \tAI Need Read %x \tAI Read Time %xl \tAI Scan Rate %dl \tAI Freq %x \tAI Value %x\n",
				pmod->sl[i].ais[j].ai_data_valid,
				pmod->sl[i].ais[j].ai_need_read,
				pmod->sl[i].ais[j].ai_read_time,
				pmod->sl[i].ais[j].ai_scan_rate,
				pmod->sl[i].ais[j].ai_freq,
				pmod->sl[i].ais[j].aival
			      );
		    }
		    break;
		case CDTYPEAI50HZ:
		    printf("Cell = %d \tSlot = %d \t AI50HZ Board\n",
				maddr,
				i
			  );
		    for (j=0;(j<AISPERSLOT);j++) {
			printf("\t\tAI Data Valid %x \tAI Need Read %x \tAI Read Time %xl \tAI Scan Rate %dl \tAI Freq %x \tAI Value %x\n",
				pmod->sl[i].ais[j].ai_data_valid,
				pmod->sl[i].ais[j].ai_need_read,
				pmod->sl[i].ais[j].ai_read_time,
				pmod->sl[i].ais[j].ai_scan_rate,
				pmod->sl[i].ais[j].ai_freq,
				pmod->sl[i].ais[j].aival
			      );
		    }
		    break;
		case CDTYPEDO:
		    printf("Cell = %d \tSlot = %d \t DO Board Value = %xl\n",
				maddr,
				i,
				pmod->sl[i].dovals
			   );
		    break;
		case CDTYPEDI8:
		    printf("Cell = %d \tSlot = %d \t DI8 Board Value = %xl\n",
				maddr,
				i,
				pmod->sl[i].divals
			   );
		    break;
		case CDTYPEDI32:
		    printf("Cell = %d \tSlot = %d \t DI32 Board Value = %xl\n",
				maddr,
				i,
				pmod->sl[i].divals
			   );
		    break;
		default:
		    printf("Cell = %d \tSlot = %d \t Board Type Not Defined \n",
				maddr,
				i
			   );
		    break;

	    }
	}
    }


/*                                                               		
**                                                                	 	
**	The purpose of this routine is to display the contents            	
**	of the static variables contained within this driver.             	
**                                                                        	
*/

/* this routine not called from within this driver */


LINEINFO ()
{
	short	i;
	char	buffer[132];
	char	*resrsp;
	char	*lineact;


	printf("                                 Modbus Line Information Table         \n");
	printf("\n");
       printf(" Line        Pointer to        Resource      Resource       Line        Delay Times:       Activity\n");
       printf("Number      CELL Table         Name        Response     Activity      IM  .TX  TX.         Time\n");
        /*     01234567891         01234567893         01234567895         0123456789 
			 01234567892         01234567894         01234567896
	*/
	printf("\n");

	for (i=0; i < RTULINES; i++)
	{
		if(plcs[i] == NULL)
		{
			continue;
		}
		if (resresponse[i] == 0)
		{
			resrsp = "FALSE";
		}
		else
		{
			resrsp = "TRUE";
		}
		if (line_activity[i] == 0)
		{
			lineact = "FALSE";
		}
		else
		{
			lineact = "TRUE";
		}
		sprintf(buffer, "  %2d         %4x   %16s        %5s        %5s        %2d   %2d   %2d        %4d\n",
			i,plcs[i],&resnames[i],
			&resrsp[0],
			&lineact[0],
			intermsgdelay[i],
			pretxdelay[i],
			posttxdelay[i],
			activity_time[i]);
		printf( "%s\n", buffer);
	}
	return 0;
}

/*	End of File:	drvRtu.c */

