/*drvStcs.c,v 1.1.1.1 1999/08/03 14:44:48 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*
/*  File: drvStcs.c
/*  Edit: 10/13/92 12:33 JLD
	03-30-94	ems	starttcs(), remove test ln >= 0
				it is unsigned
/*
*/
#ifdef TEST
/* #include "testmode.h" */
#include "bioscomm.h"
#include "dummyos.h"
#include  "tcs.h"
#include  "link.h"
#include "tcsproto.h"
#define logMsg printf
#endif
#ifndef TEST
#include <ctype.h>
#include "vxWorks.h"
#include <sysLib.h>
#include "semLib.h"
#include <wdLib.h>
#include <rngLib.h>
#include <task_params.h>
#include <stdioLib.h>

#include <dbDefs.h>
#include <drvSup.h>
#include <module_types.h>
#include "drvStcs.h"
#include "drvStcs_proto.h"
#endif


static long	init();

struct {
	long		number;
	DRVSUPFUN	report;
	DRVSUPFUN	init;
} drvStcs={
	2,
	NULL,
	init};

/*
/*      Storage for TCS Controller Structures
/*
*/

static 	TCSCNTRL        *plcs[TCSLINES];
static	OUTPUTQUE       oq[TCSLINES];
static	SEMAPHORE       *semids[TCSLINES];
static	short           linetimers[TCSLINES];
static	SEMAPHORE       linesems[TCSLINES];
static	short		backup_mode[TCSLINES];
static	short		channel_select[TCSLINES];

/*
/*      Storage for TCS Controller Comm Line Structures
/*
*/

static	RXLINE          rxlinetab[TCSLINES];
static	TXLINE          txlinetab[TCSLINES];

static	int	taskId[TCSLINES];
static  char    *bools[] = {
	"TRUE ",
	"FALSE",
    };

static  char *tasknames[] = {"TCSCNTRL Ch0","TCSCNTRL Ch1","TCSCNTRL Ch2","TCSCNTRL Ch3","TCSCNTRL Ch4","TCSCNTRL Ch5"};

/**********
*
*	This routine is required by iocinit in the EPICS system.
*
**********/
static long init () {
	return (0);
}


#ifndef TEST
int stoptcs (short ln)
{
	if (comm_disconnect (ln) != -1) {
	    taskDelete (taskId [ln]);
	    return (1);
	} else {
	    return (-1);
	}
}

int starttcs(unsigned short ln, unsigned char *mode)
{

	if ( ln <= 5 )
	{
	    plcs[ln] = (TCSCNTRL *)calloc(1,(sizeof(TCSCNTRL)*NUMTCSCNTRLS));

	    if (!plcs[ln]) {
		logMsg ("Not enough memory for Controller line %d \n",ln);
		return (-1);
	    }

	    comm_init(ln, mode, &tx_int_proc, &rx_int_proc);
	
	    taskId [ln] = taskSpawn(tasknames[ln],GEN_PRIORITY,GEN_TASK_OPTIONS,GEN_STACK,tcs_driver,ln);

	    logMsg("Started the TCSCNTRL driver on line %d\n",ln);

	    return (1);
	} else {

	    logMsg ("Invalid line number %d\n", ln);
	    return (-1);
	}
}
#endif



/*
/*      tcs_driver
/*
/*      The purpose of this task is to scan the TCS Controller
/*      lines. It will maintain a configuration list supporting online additions
/*      and repplacement of modules, and  maintain a table of most recently
/*      scanned values.
/*
*/
void tcs_driver (unsigned short line)


    {


	unsigned short  mod, cfgchk, cnt, modspolled, onlinefreq;
	TCSCNTRL          *pmod;

	/* Initialize the data structures */
	init_line_tabs (line);
	onlinefreq = 1;

#ifdef  TEST
	scansome(0);
#endif

	/* Task will process all modules sending outputs and getting inputs */
	/* It will then try to configure requested non responding modules   */

	cfgchk = 0;     /* Init the new config pointer */

	while (TRUE) {

	    modspolled = 0;

	    for ( mod=1; (mod < NUMTCSCNTRLS); mod++ ) {
		pmod = plcs[line]+mod;  /* shorthand pointer */
		if ((pmod->is_responding) && (!pmod->cmd_force_offline)) {
		    modspolled++;
		    scan_plc(pmod,line,mod);
		} /* end of module */
	    } /* end of scan loop */

	    cnt = 0;  /* count the number to bring on in one pass */

	    if (onlinefreq-- == 0) {
		onlinefreq = BRINGBACKCNT;

		for ( mod=1; (mod<NUMTCSCNTRLS); mod++) {
		    cfgchk++;
		    if (cfgchk==NUMTCSCNTRLS) cfgchk=1;
		    pmod = plcs[line]+cfgchk;
		    if ((!pmod->is_responding) &&
			(pmod->was_requested) &&
			(!pmod->cmd_force_offline)) {

			on_line(line,cfgchk);
			modspolled++;
			cnt++;
			if (cnt == MAXCFGCK) break;
		    }
		}
	    }
#ifndef TEST
	    if (modspolled==0)  taskDelay(60);
	    else taskDelay(10);
#endif
#ifdef  TEST
	    scansome();
#endif

	} /* end task loop */

     }

/*
/* Try to bring a PLC on line
/*
*/
static  void    on_line(unsigned short line,unsigned short mod)

    {
	TCSCNTRL  *pmod;
	unsigned short i,j;

	pmod = plcs[line]+mod;

/*
/*      EVERY CONTROLLER MUST HAVE A PARAMETER 0
/*      & IF I CAN READ IT THE CONTROLLER IS RESPONDING
/*
*/
	if (!readparams (ONERETRY+line,mod,pmod,0,1)) return;

/*
/*      INITIALIZE ALL VALUES AS
/*              - NOT REQUESTED
/*              - DATA NOT VALID
/*
*/

	for (i=0; (i<NUMTCSPARAM); i++) {
	    pmod->params[i].was_requested = FALSE;
	    pmod->params[i].data_valid = FALSE;
	}

	pmod->scanpass = 0;
	pmod->is_responding = TRUE;

/*
/*	MUST ATTEMP TO READ ALL OF THE PARAMETERS TO HAVE VALID DATA
/*	FOR OUTPUTS (BITS FOR WRITING DIGITALS AND FORMATS FOR ALL
/*	OUTPUT VALUES.
*/
	if (!readparams (ONERETRY+line,mod,pmod,0,NUMTCSPARAM)) {
		pmod->is_responding = FALSE;
		return;
	}

    }

/*
/* Scan a TCS Controller -  get inputs an set outputs as required
/*
*/

static  void    scan_plc(TCSCNTRL *pmod, unsigned short line,unsigned short mod)

    {
	unsigned short i, j, val, maxints, numgroups;
	unsigned long   bitmask;

	pmod->scanpass++;
	checkoutputs(line);

	/* Use Enquiry poll for fast scan of important parameters */

	if ((pmod->scanpass % FULLSCAN) != 0) {
		readenqparams(line,mod,pmod);
		return;
	}

	/* Periodically fetch all of the requested paramters */
	/* read the requested parameters values to be scanned */

	for (i=0; (i<NUMTCSPARAM); i++) {
	    if (pmod->params[i].was_requested) {
		for (j=1; ((j<TCSMSGPARAMS) && ( (i+j)<NUMTCSPARAM )); j++) {
		    if (!pmod->params[i+j].was_requested) break;
		}
		numgroups = j;
		if (!readparams (line,mod,pmod,i,numgroups)) return;
		checkoutputs(line);
		i=(i+j)-1;
	    }
	}

    }

/*
/* checkoutputs
/*      checks output que and returns if empty.
/*      if there is something on the que it attempts to process all
/*      pending outputs doing imediate retries. If attempts fail the request is dropped
/*
*/
static  void checkoutputs(unsigned short line)

    {
	OUTQENT *optr;
	unsigned short  signal,value,link,adapter, card;
	unsigned long   bomask,boval;
	TCSCNTRL  *pmod;

	while ((optr=ogetnextp(line)) != NULLP) {

	    if ((optr->otype)==AOUTTYPE) {
		link = optr->out.aonode.link;
		adapter = optr->out.aonode.adapter;
		pmod = plcs[link]+adapter;
		card = optr->out.aonode.card;
		signal = optr->out.aonode.aosignal;
		value = optr->out.aonode.aovalue;

		if (!setaouts(link,adapter,pmod,card,signal,value))
		    pmod->badoutcnt++;

	    }

	    else if ((optr->otype)==BOUTTYPE) {
		link = optr->out.bonode.link;
		adapter = optr->out.bonode.adapter;
		pmod = plcs[link]+adapter;
		card = optr->out.bonode.card;
		signal = optr->out.bonode.bosignal;
		boval = optr->out.bonode.bovalue;
		bomask = optr->out.bonode.bomask;

		if (!setbouts (line,adapter,pmod,card,signal,bomask,boval))
		    pmod->badoutcnt++;

	    }

	    oreleasefull(line);     /* release the output que entry */

	}

    }


/*
/*      aidriver
/*
/*      This routines main purpose is to return the analog input value for the
/*      specified module. For a module value
/*      a (-1) will be returned if there is no valid data ready.
/*
*/
int tcs_aidriver (unsigned short link, unsigned short adapter, unsigned short card,
	     unsigned short signal, short *pvalue, unsigned short conversion)

    {
	TCSCNTRL  *pmod;
	unsigned short  blkix,elemix,subelement;

	if (!plcs[link]) return (-1);

	if (!rangeok(link,adapter,card,signal)) return (-1);

	if (conversion != conversion) return (-1);        /* No warnings */

	pmod = plcs[link]+adapter;

	if (card == PSEUDO) {
	    switch (signal) {

	    case 0: /* Were the PLC values requested */
		*pvalue = pmod->was_requested;
		break;

	    case 1: /* Is the PLC responding */
		*pvalue = pmod->is_responding;
		break;

	    case 2: /* The program Load request */
		*pvalue = pmod->prog_load_stat;
		break;

	    case 3: /* PLC Program Load Block Number */
		*pvalue = pmod->prog_load_page;
		break;

	    case 4: /* Set the program number to Load */
		*pvalue = pmod->prog_load_number;
		break;

	    case 5: /* Output Messages */
		*pvalue = pmod->no_dataout_msgs;
		break;

	    case 6: /* Input Messages */
		*pvalue = pmod->no_datain_msgs;
		break;

	    case 7: /* No Receiver Responses */
		*pvalue = pmod->no_rxnoresp;
		break;

	    case 8: /* No Transmitter Response */
		*pvalue = pmod->no_txnoresp;;
		break;

	    case 9: /* Data Echo Errors */
		*pvalue = pmod->no_dataerr;
		break;

	    case 10: /* Message Retries */
		*pvalue = pmod->no_retries;
		break;

	    case 11: /* RX Hardware errors */
		*pvalue = pmod->no_rxhwerrs;
		break;

	    case 12: /* TX Hardware errors */
		*pvalue = pmod->no_txhwerrs;
		break;

	    case 13: /* CRC errors */
		*pvalue = pmod->no_crcerrs;
		break;

	    case 14: /* Offline transitions */
		*pvalue = pmod->no_offlines;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 17: /* Failed output commands */
		*pvalue = pmod->badoutcnt;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode[link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select[link];
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	pmod->was_requested = TRUE;

	if (!pmod->is_responding) return (-1);

	pmod->params[card].was_requested = TRUE;

	if (!pmod->params[card].data_valid) return (-1);
	*pvalue = pmod->params[card].value;
	return (0);

    }


/*
/*      tcs_aodriver
/*
/*      This routines main purpose is to set requested analog outputs to the
/*      specified module.
*/
int tcs_aodriver(unsigned short link, unsigned short adapter, unsigned short card,
	    unsigned short signal, unsigned short value)

    {
	TCSCNTRL  *pmod;
	unsigned short  blkix;
	OUTQENT *oqp;


	if (!plcs[link]) return (-1);

	if (!rangeok(link,adapter,card,signal)) return (-1);

	pmod = plcs[link]+adapter;

	if (card == PSEUDO) {
	    switch (signal) {

	    case 2: /* Set the program Load request */
		if (value != 0)
		    pmod->prog_load_stat = PLC_LOAD_REQUESTED;
		break;

	    case 4: /* Set the program number to Load */
		pmod->prog_load_number = value;
		break;

	    case 15: /* Reset the accumulated statistics */
		if (value != 0) initstats(pmod);
		break;

	    case 16: /* Force the PLC offline */
		pmod->cmd_force_offline = (value != 0);
		break;

	    case 18: /* What is the backup mode for the line*/
		backup_mode[link] = (value != 0);
		break;

	    case 19: /* What is the selected channel for the line */
		channel_select[link] = (value != 0);
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	pmod->was_requested = TRUE;

	if (!pmod->is_responding) return (-1);

	pmod->params[card].was_requested = TRUE;

	/* Must have initial read for format type */
	if (!pmod->params[card].data_valid) return (-1);

	oqp=ogetemptyp(link);
	if (oqp == NULLP) return (-1); /* output que is full */

	oqp->otype = AOUTTYPE;
	oqp->out.aonode.link = link;
	oqp->out.aonode.adapter = adapter;
	oqp->out.aonode.card = card;
	oqp->out.aonode.aosignal = signal;
	oqp->out.aonode.aovalue = value;

	omarkfull(link); /* Mark this entry as ready for output */

	return (0);


    }



/*
/*      tcs_aoread
/*
/*      This routines main purpose is to return the value last read back
/*      from the analog output hardware. This is used to initialize analog
/*      outputs.
*/
int tcs_aoread(unsigned short link, unsigned short adapter, unsigned short card,
	  unsigned short signal, unsigned short *pvalue)

    {
	TCSCNTRL  *pmod;
	unsigned short  blkix,elemix,subelement;

	if (!plcs[link]) return (-1);

	if (!rangeok(link,adapter,card,signal)) return (-1);

	pmod = plcs[link]+adapter;

	if (card == PSEUDO) {
	    switch (signal) {

	    case 2: /* Set the program Load request */
		*pvalue = pmod->prog_load_stat;
		break;

	    case 4: /* Set the program number to Load */
		*pvalue = pmod->prog_load_number;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode[link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select[link];
		break;


	    default:
		return(-1);
	    }
	    return (0);
	}

	pmod->was_requested = TRUE;

	if (!pmod->is_responding) return (-1);

	pmod->params[card].was_requested = TRUE;

	if (!pmod->params[card].data_valid) return (-1);
	*pvalue = pmod->params[card].value;
	return (0);

    }


/*
/*      tcs_bidriver
/*
/*      The main purpose of this routine is to return the request digital input
/*      value(s) from the requested card. This routine will return a (-1) if the
/*      requested module is the wrong type or the data is not yet valid.
/*
*/

int tcs_bidriver(unsigned short link, unsigned short adapter, unsigned short card,
	    unsigned short signal, unsigned long mask, unsigned long *pvalue)

    {
	TCSCNTRL  *pmod;
	unsigned short blkix,elemix,subelement,tvallow,tvalhigh;
	long	retval;


	if (!plcs[link]) return (-1);

	if ((card != PSEUDO) && (signal > 15)) return (-1);
	if ((mask & 0xFFFF0000) != 0) return (-1);

	if (!rangeok(link,adapter,card,signal)) return (-1);

	pmod = plcs[link]+adapter;

	if (card == PSEUDO) {
	    switch (signal) {

	    case 0: /* Were the PLC values requested */
		*pvalue = pmod->was_requested;
		break;

	    case 1: /* Is the PLC responding */
		*pvalue = pmod->is_responding;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode[link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select[link];
		break;

	    default:
		return (-1);
	    }

	    return (0);
	}


	pmod->was_requested = TRUE;

	if (!pmod->is_responding) return (-1);

	pmod->params[card].was_requested = TRUE;

	if (!pmod->params[card].data_valid) return (-1);
	*pvalue = (pmod->params[card].value & mask);
	return (0);

    }



/*
/*      tcs_bodriver
/*
/*      The main purpose of this routine is to set the request digital output
/*      value(s) on the requested card. This routine will return a (-1) if the
/*      requested module is the wrong type or not configured yet.
/*
*/
int tcs_bodriver(unsigned short link, unsigned short adapter, unsigned short card,
	    unsigned short signal, unsigned long value, unsigned long mask)


    {
	TCSCNTRL  *pmod;
	OUTQENT *oqp;
	unsigned short blkix;
	unsigned short  tsignal;

	if (!plcs[link]) return (-1);

	if ((card != PSEUDO) && (signal > 15)) return (-1);
	if ((mask & 0xFFFF0000) != 0) return (-1);

	if (!rangeok(link,adapter,card,signal)) return (-1);

	pmod = plcs[link]+adapter;

	if (card == PSEUDO) {
	    switch (signal & 0x1f) {

	    case 2: /* Set the program Load request */
		if (value != 0)
		    pmod->prog_load_stat = PLC_LOAD_REQUESTED;
		break;

	    case 15: /* Reset the accumulated statistics */
		if (value != 0) initstats(pmod);
		break;

	    case 16: /* Force the PLC offline */
		pmod->cmd_force_offline = (value != 0);
		break;

	    case 18: /* What is the backup mode for the line*/
		backup_mode[link] = (value != 0);
		break;

	    case 19: /* What is the selected channel for the line */
		channel_select[link] = (value != 0);
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	pmod->was_requested = TRUE;

	if (!pmod->is_responding) return (-1);

	pmod->params[card].was_requested = TRUE;

	oqp=ogetemptyp(link);

	if (oqp == NULLP) return (-1); /* output que is full */

	oqp->otype = BOUTTYPE;
	oqp->out.bonode.link = link;
	oqp->out.bonode.adapter = adapter;
	oqp->out.bonode.card = card;
	oqp->out.bonode.bosignal = signal;
	oqp->out.bonode.bomask = mask;
	oqp->out.bonode.bovalue = value;

	omarkfull(link); /* Mark this entry as ready for output */

	return (0);

    }



/*
/*      tcs_boread
/*
/*      This routines main purpose is to return the value last read back
/*      from the digital output hardware. This is used to initialize digital
/*      outputs. This will return a (-1) if the module is the wrong type or
/*      the data is not yet valid
*/
int tcs_boread(unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long *pvalue, unsigned long mask)

    {
	TCSCNTRL  *pmod;
	unsigned short blkix,elemix,subelement,tvallow,tvalhigh;
	unsigned short  tsignal;
	long	retval;


	if (!plcs[link]) return (-1);

	if ((card != PSEUDO) && (signal > 15)) return (-1);
	if ((mask & 0xFFFF0000) != 0) return (-1);

	if (!rangeok(link,adapter,card,tsignal)) return (-1);

	pmod = plcs[link]+adapter;

	if (card == PSEUDO) {
	    switch (signal) {

	    case 2: /* Set the program Load request */
		*pvalue = 0;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode[link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select[link];
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	pmod->was_requested = TRUE;

	if (!pmod->is_responding) return (-1);

	pmod->params[card].was_requested = TRUE;

	if (!pmod->params[card].data_valid) return (-1);
	*pvalue = (pmod->params[card].value & mask);
	return (0);

    }

/*
/* rangeok
/*	routine to check the boundaries for link adapter card and signal
/*
*/
static  int	rangeok(unsigned short link,unsigned short adapter,
		unsigned short card,unsigned short signal)

    {

	if (link >= TCSLINES) return (FALSE);
	if (adapter >= NUMTCSCNTRLS) return (FALSE);
	if ((card > NUMTCSPARAM) && (card != PSEUDO) )
		 return (FALSE);
	if (signal != signal) return (FALSE);
	return (TRUE);

    }


/*
/*      Initialize the Queue
*/
static  void outqueinit (unsigned short line)

    {
	oq[line].ooutix = oq[line].oinix = 0;

    }



/*
/*  ogetempty - get the next available slot on the output list
*/
static  OUTQENT *ogetemptyp (unsigned short line)

    {

	if (((oq[line].oinix + 1) % OUTQSIZE) != oq[line].ooutix) {
	    return (&oq[line].outque[oq[line].oinix]);
	}
	else return (NULLP);

    }



/*
/* omarkfull
/*      mark the next block as full
*/
static  void omarkfull(unsigned short line)
	{
	    oq[line].oinix = ((oq[line].oinix + 1) % OUTQSIZE);
	}


/*
/*  ogetfullp - get the next full slot on the output list
*/
static  OUTQENT *ogetnextp (unsigned short line)

    {

	if (oq[line].ooutix != oq[line].oinix) {
	    return (&oq[line].outque[oq[line].ooutix]);
	}
	else return (NULLP);

    }


/*
/* oreleasefull - release the current full output block
*/
static  void oreleasefull(unsigned short line)

    {

	if (oq[line].ooutix == oq[line].oinix) return;
	oq[line].ooutix = ((oq[line].ooutix + 1) % OUTQSIZE);

    }



/*
/*      link_status
/*
/*      The purpose of this routine is to print out the state of each of the
/*      analog devices lines and the statistics gathered about them.
/*
*/
static  void link_status(unsigned short line)

    {
    if (line > TCSLINES-1) return;
    }


/*
/*      startrx
/*
/*      The purpose of this routine is to initialize a line table
/*      for receive and to enable the receiver.
/*
*/
static  void start_rx( unsigned short line,SEMAPHORE *semid,unsigned short *pstatus )

    {

	RXLINE                  *rxp;

	disable_rx_ints(line);
	rxp=&rxlinetab[line];
	rxp->rxstate = RXINIT;
	rxp->rxix = 0;
	rxp->semid = semid;
	rxp->status = pstatus;
	enable_rx_ints(line);
    }


/*
/*      stoprx
/*
/*      The purpose of this routine is to initialize a line table
/*      for receive and to disable the receiver.
/*
*/
static  void stop_rx( unsigned short line )

    {
	RXLINE                  *rxp;

	rxp=&rxlinetab[line];
	rxp->rxstate = RXIGNORE;
    }

/*
/*      rx_int_proc
/*
/*      The purpose of this routine is to process a receive character interrupt
/*      for a given line. (Protocol Specific)
/*
*/
  void rx_int_proc( unsigned short line )

    {
	unsigned char   rxchar;
	unsigned short  rxstat;
	unsigned short  rx_err_mask;
	RXLINE          *rxp;

	read_rx_hw(line,&rxchar,&rxstat,&rx_err_mask);
	rxp=&rxlinetab[line];

	/* Restart Timer after Every character */

	restart_timer ( linetimers[line], RXTIMEOUT);

	if (rxp->rxstate==RXIGNORE) return;

	if ((rxstat & rx_err_mask) != 0) {
	    *(rxp->status) = RXBAD;
	    rxp->rxbuf[299]=rxstat;
	    rxp->rxbuf[300]=rxchar;
	    rxp->rxstate = RXIGNORE;
	    return;
	}


	switch (rxp->rxstate) {
	    case RXINIT:
		if (rxchar == STX) {
		    rxp->rxbuf[0] = rxchar;
		    rxp->rxix = 1;
		    rxp->rxstate = RXGETPARAMNO;
		    return;
		}
		if ((rxchar == ACK) || (rxchar == NAK) || (rxchar == EOT)) {
		    rxp->rxbuf[0] = rxchar;
		    rxp->rxstate = RXIGNORE;
		    *(rxp->status) = RXGOOD;
		    semGive(rxp->semid);
		    return;
		}
	    case RXGETPARAMNO:
		if ((rxchar == ETX) || (rxchar == ETB)) {
		    rxp->rxbuf[rxp->rxix++] = rxchar;
		    rxp->rxstate = RXGETBCC;
		    return;
		}
		if ((rxchar & 0x80) == 0) { /* Parameters must be > 128 */
		    rxp->rxstate = RXINIT;  /* Abort the message */
		    return;
		}
		else {
		    rxp->rxbuf[rxp->rxix++] = rxchar;
		    if (rxp->rxix >= MAXRXMSG) {
			rxp->rxstate = RXINIT;  /* Abort the message */
			return;
		    }
		    rxp->rxstate = RXGETDATA1;
		    return;
		}
	    case RXGETDATA1:
		if ((rxchar & 0x80) == 0) { /* Data must be > 128 */
		    rxp->rxstate = RXINIT;  /* Abort the message */
		    return;
		}
		rxp->rxbuf[rxp->rxix++] = rxchar;
		rxp->rxstate = RXGETDATA2;
		return;
	    case RXGETDATA2:
		if ((rxchar & 0x80) == 0) { /* Data must be > 128 */
		    rxp->rxstate = RXINIT;  /* Abort the message */
		    return;
		}
		rxp->rxbuf[rxp->rxix++] = rxchar;
		rxp->rxstate = RXGETDATA3;
		return;
	    case RXGETDATA3:
		if ((rxchar & 0x80) == 0) { /* Data must be > 128 */
		    rxp->rxstate = RXINIT;  /* Abort the message */
		    return;
		}
		rxp->rxbuf[rxp->rxix++] = rxchar;
		rxp->rxstate = RXGETPARAMNO;
		return;
	    case RXGETBCC:
		rxp->rxbuf[rxp->rxix++] = rxchar;
		rxp->rxstate = RXIGNORE;
		*(rxp->status) = RXGOOD;
		semGive(rxp->semid);
		return;
	    default:  /* Should Never Get Here */
		rxp->rxstate = RXIGNORE;
		return;
	}
    }


/*
/*      starttx
/*
/*      The purpose of this routine is to initialize a line table
/*      for send and to enable the transmitter.
/*
*/
static  void start_tx( unsigned short line,SEMAPHORE *semid,unsigned short *pstatus )

    {

	TXLINE                  *txp;

	disable_tx_ints(line);
	txp=&txlinetab[line];
	txp->txix = 0;
	txp->semid = semid;
	txp->status = pstatus;

	/* Might have to output the first character here */


	enable_tx_ints(line);

    }


/*
/*      stoptx
/*
/*      The purpose of this routine is to initialize a line table
/*      for receive and to disable the receiver.
/*
*/
static  void stop_tx( unsigned short line )

    {
	disable_tx_ints(line);
    }

/*
/*      tx_int_proc
/*
/*      The purpose of this routine is to process a receive character interrupt
/*      for a given line.
/*
*/
  void tx_int_proc( unsigned short line )

    {

	unsigned char           txchar;
	unsigned short          txstat;
	unsigned short          tx_err_mask;
	TXLINE                  *txp;

	txp=&txlinetab[line];
	txchar = txp->txbuf[txp->txix++];
	write_tx_hw ( line,txchar,&txstat,&tx_err_mask );
	if ( (txstat & tx_err_mask) != 0 ) {
	    disable_tx_ints (line);
	    *(txp->status) = TXBAD;
	    semGive(txp->semid);
	    return;
	}

	if (txp->txix == txp->txmsglen) {
	    disable_tx_ints (line);
	    *(txp->status) = TXGOOD;
	    semGive(txp->semid);
	    return;
	}

	if (txp->txix >= MAXTXMSG) {
	    disable_tx_ints (line);
	    *(txp->status) = TXBAD;
	    semGive(txp->semid);
	    return;
	}

    }


/*
/*      msgxfer
/*
/*      The purpose of this routine is to transmit a buffer to the
/*      Allen Bradley PLCs and to await a response. The transmit and receive
/*      buffers are in their respective line tables. This routine will retry
/*      the message up to a maximum number of times or until a data response
/*      is received with a proper checksum.
/*              returns         XFERGOOD if response received
/*                              XFERBAD  if no good repsonse is received
/*
*/

  unsigned short  msgxfer(unsigned short fline,unsigned short resptype, TCSCNTRL *pmod)

    {

	unsigned short  retry,txstatus,rxstatus,rxlen,line;
	RXLINE          *rxp;
	TXLINE          *txp;
	unsigned char   csums[2];

	line = (fline & 0x0f);          /* strip off the no retry flag*/

	rxp = &rxlinetab[line];


	for (retry=((fline == line)?0:MAXRETRYS-1); (retry<MAXRETRYS); retry++ ) {
	    stop_timer(linetimers[line]);
	    txstatus = IO_TIMEOUT;

	    rxstatus = IO_TIMEOUT;
	    start_rx(line, semids[line], &rxstatus);

	    start_tx(line, semids[line], &txstatus);
	    start_timer (linetimers[line],TXTIMEOUT,semids[line], &txstatus);
	    semTake(semids[line]);
	    stop_timer(linetimers[line]);
	    stop_tx(line);

	    if (txstatus == TXGOOD) {
		if (resptype == NOREPLY) return (XFERGOOD);
		start_timer (linetimers[line],RXTIMEOUT,semids[line], &rxstatus);
#ifdef TEST
		enable_rx_ints(line);
#endif
		semTake(semids[line]);
		stop_timer(linetimers[line]);

		stop_rx(line);

		rxlen = rxp->rxix;
		if (rxstatus == RXGOOD) {
		    if ((resptype == ACK) && (rxp->rxbuf[0] == ACK))  return (XFERGOOD);
		    if ((resptype == STX) && (rxp->rxbuf[0] == EOT))  return (XFERGOOD);
		    if (resptype == STX) {
			makecrc(&rxp->rxbuf [1], rxlen-2, csums);
			if (csums[0] == rxp->rxbuf[rxp->rxix-1]) {
			    return (XFERGOOD);
			} else {
			    pmod->no_crcerrs++;
			    logMsg ("%x%x%x%x Bad BCC\n",
					rxp->rxbuf[rxp->rxix-1],
					csums [0]
			    );
			}
		    }
		    logMsg("Command Rejected, response type = %x \n", rxp->rxbuf[0]);
		    return (XFERBADRESP); /* Must have been a NAK */
		} else if (rxstatus == IO_TIMEOUT) {
		    logMsg ("Receive Timeout\n");
			pmod->no_rxnoresp++;
		} else {
		    logMsg ("Receive Error\n");

		    logMsg ( "RXHWERR: %2x, RXCHR: %2x, RXCNT: %2x\n",
			 rxp->rxbuf[299],
			 rxp->rxbuf[300],
			 rxp->rxix);
			 pmod->no_rxhwerrs++;
		}
	    } else if (txstatus == IO_TIMEOUT) {
		logMsg ("Transmit Timeout\n");
		pmod->no_txnoresp++;
	    } else {
		logMsg ("Transmit Error\n");
		pmod->no_txhwerrs++;
	    }
	if (retry != MAXRETRYS) pmod->no_retries++;
	}
	logMsg ("Xfer Bad\n");
	return (XFERBAD);

    }

/*
/*      init_line_tabs
/*
/*      The purpose of this routine is to initialize the line tables for the
/*      turnbull I/O
/*
*/
  void init_line_tabs(unsigned short line)

    {

	unsigned short  mod;
	TCSCNTRL  *pmod;


	for (mod=0; (mod<NUMTCSCNTRLS); mod++) {

	    pmod = plcs[line]+mod;

	    pmod->is_responding = FALSE;
	    pmod->was_requested = FALSE;
	    pmod->cmd_force_offline = FALSE;
	    pmod->prog_load_stat = PLC_NOT_LOADED;
	    pmod->txtsn = 1;
	    pmod->rxtsn = 0xFFF; /* impossible value */
	    pmod->rxcmd = 0xFFF; /* impossible value */

	    initstats(pmod);

	}
	linetimers[line] = gettimer();
	if (linetimers[line] == -1) {
		/*  put an error message in here */
	}

	/* Initialize this tasks semaphore*/
	semInit(&linesems[line]);
	semids[line]=&linesems[line];

    }

/*
/*
/* initialize the statistics for a TCS Controller
/*
*/
  void initstats(TCSCNTRL *pmod)

    {
	pmod->no_dataout_msgs = 0;      /* 5  Count Output Msgs */
	pmod->no_datain_msgs = 0;       /* 6  Count Data Requests */
	pmod->no_rxnoresp = 0;          /* 7  Count No Responses */
	pmod->no_txnoresp = 0;          /* 8  Count No Responses */
	pmod->no_dataerr = 0;           /* 9  Count Message Echo Errors */
	pmod->no_retries = 0;           /* 10 Count Retries */
	pmod->no_rxhwerrs = 0;          /* 11 Count Comm Hardware Errors */
	pmod->no_txhwerrs = 0;          /* 12 Count Comm Hardware Errors */
	pmod->no_crcerrs = 0;           /* 13 Count CRC Errors */
	pmod->no_offlines = 0;          /* 14 Count Off Line Transitions */
	pmod->badoutcnt = 0;		/* 17 Count failed output commands */
    }


/*
/* Readparams
/*    Reads the TCS Controller Parameters
*/
  unsigned short readparams (unsigned short line, unsigned short mod,
			  TCSCNTRL *pmod,unsigned short param,
			  unsigned short numparams)
    {

	unsigned short  wval, count, i, j, lastgroup, ix, mstatus;
	unsigned char	data[MAXRXMSG];
	TXLINE  *txp;
	RXLINE  *rxp;

	txp = &txlinetab[line & (~ONERETRY)];
	rxp = &rxlinetab[line & (~ONERETRY)];

/*	logMsg ( "Read Parameters for Mod = %d, Pmod = %x, Txp = %x, Rxp = %x \n",
			mod,
			pmod,
			txp,
			rxp );
*/
	/* Count number of data request messages */
	pmod->no_datain_msgs++;

	i = 0;

	while (i < numparams) {

	    ix = 0;
	    txp->txbuf[ix++] = EOT;	/* Attention Character	*/
	    txp->txbuf[ix++] = (mod | 0x80);	/* Instrument Number	*/
	    txp->txbuf[ix++] = ((param+i) | 0x80);	/* Parameter Number	*/

	    count = (i+TCSMSGPARAMS <= numparams)?TCSMSGPARAMS:numparams-i;

	    txp->txbuf[ix++]  = (count | 0x80);	/* Put in Count		*/

					/* Add in the CCC 	*/
	    makecrc(&txp->txbuf [1], ix-1, &txp->txbuf[ix]);

	    ix++;			/* Bump Past the CRC	*/

	    txp->txbuf[ix++] = ENQ;		/* Enquiry Poll Character */

	    /* Time to send the message and get an ACK */

	    txp->txmsglen = ix;
	    mstatus = msgxfer( line, STX, pmod);
	    if (mstatus == XFERBAD) {
		pmod->no_offlines++;
		pmod->is_responding = FALSE;
		return(FALSE);
	    } 
	    if (mstatus == XFERBADRESP) return (FALSE);
	    if (rxp->rxbuf[0] == EOT) return (TRUE);
	    processdata(rxp,pmod);
	    i = i+count;
	}

	return (TRUE);
    }

/*
/* Readenqparams
/*    Reads the TCS Controller Enquiry Parameters
*/
 unsigned short readenqparams (unsigned short line, unsigned short mod,
			  TCSCNTRL *pmod )
    {

	unsigned short  wval, count, i, j, lastgroup, ix, mstatus;
	unsigned char	data[MAXRXMSG];
	TXLINE  *txp;
	RXLINE  *rxp;

	txp = &txlinetab[line];
	rxp = &rxlinetab[line];

	/* Count number of data request messages */
	pmod->no_datain_msgs++;

	ix = 0;
	txp->txbuf[ix++] = EOT;	/* Attention Character	*/
	txp->txbuf[ix++] = (mod | 0x80);	/* Instrument Number	*/

	makecrc(&txp->txbuf [1], ix-1, &txp->txbuf[ix]);

	ix++;				/* Bump Past the CRC	*/

	txp->txbuf[ix++] = ENQ;		/* Enquiry Poll Character */

	/* Time to send the message and get an ACK */

	txp->txmsglen = ix;

	mstatus = msgxfer( line, STX, pmod);
	if (mstatus == XFERBAD) {
		pmod->no_offlines++;
		pmod->is_responding = FALSE;
		return(FALSE);
	}
	if (mstatus == XFERBADRESP) return (FALSE);

	if (rxp->rxbuf[0] == ETX) return (TRUE);

	processdata(rxp,pmod);

	return (TRUE);
    }

/*
/*	process data
/*		store the parameters which were returned
*/
  void processdata(RXLINE *rxp,TCSCNTRL *pmod)
    {
	unsigned short ix,tchar,val,paramno;

	ix = 1;			/* skip the STX */

	tchar = rxp->rxbuf[ix];

	while ( (tchar != ETX) && (tchar != ETX) && (ix < rxp->rxix) ) {
	    paramno = (rxp->rxbuf[ix++] & 0x7f);
	    pmod->params[paramno].format = ((rxp->rxbuf[ix] >> 2) & 0x1f);
	    val = (((unsigned short)rxp->rxbuf[ix++] & 0x3) << 14);
	    val = val + (((unsigned short) rxp->rxbuf[ix++] & 0x7F) << 7);
	    val = val + (rxp->rxbuf[ix++] & 0x7F);
	    pmod->params[paramno].value=val;
	    pmod->params[paramno].data_valid = TRUE;
	    tchar = rxp->rxbuf[ix];
	}
    }


/*
/* Setaouts
/*	Set analog outputs in Allen Bradley PLC files.
/*
*/
  unsigned short setaouts(unsigned short line, unsigned short mod,
			 TCSCNTRL *pmod, unsigned short card,
			 unsigned short signal, unsigned short val)

    {

	TXLINE  *txp;
	unsigned short  ix, six, mstatus;
	unsigned char	cval;

	/* Set up pointers for comm buffers */

	txp = &txlinetab[line & (~ONERETRY)];

	if (signal != signal) return (FALSE); /* No Warnings */

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	/* Format the output message */
	ix = 0;

	txp->txbuf[ix++] = EOT;	/* Attention Character	*/
	txp->txbuf[ix++] = (mod | 0x80);	/* Instrument Number	*/

	makecrc(&txp->txbuf [1], ix-1, &txp->txbuf[ix]);

	ix++;				/* Bump Past the CRC	*/

	txp->txbuf[ix++] = STX;		/* Start of the output cmd */

	six = ix;			/* Save the start of the crc */

	txp->txbuf[ix++] = (card | 0x80);	/* Parameter Number	*/

	cval = ((pmod->params[card].format << 2) & 0x7c);
	txp->txbuf[ix++] = ((cval + ((val >> 14) & 0x3)) | 0x80); /* Format */
	txp->txbuf[ix++] = (((val >> 7) & 0x7f) | 0x80);
	txp->txbuf[ix++] = ((val & 0x7f) | 0x80);

	txp->txbuf[ix++] = ETX;

	makecrc(&txp->txbuf [six], (ix-six), &txp->txbuf[ix]);
	ix++;
	/* Time to send the message and get an ACK */

	txp->txmsglen = ix;

	mstatus = msgxfer( line, ACK, pmod);
	if (mstatus == XFERBAD) {
		pmod->no_offlines++;
		pmod->badoutcnt++;
		pmod->is_responding = FALSE;
		return(FALSE);
	}
	if (mstatus == XFERBADRESP) return (FALSE);
	return(TRUE);

    }

/*
/* Setbouts
/*	Set binary outputs 
/*
*/
  unsigned short setbouts(unsigned short line, unsigned short mod,
			 TCSCNTRL *pmod, unsigned short card,
			 unsigned short signal,
			 unsigned long mask, unsigned long val)

    {

	TXLINE  *txp;
	unsigned short  six, ix, baseelement, element, subelement;
	unsigned short	tmask, tvalue, andmask, ormask, mstatus;
	unsigned char 	cval;


	/* Set up pointers for comm buffers */

	txp = &txlinetab[line & (~ONERETRY)];

	if (signal != signal) return (FALSE); /* No Warnings */

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	/* Format the output message */
	ix = 0;

	txp->txbuf[ix++] = EOT;	/* Attention Character	*/
	txp->txbuf[ix++] = (mod | 0x80);	/* Instrument Number	*/

	makecrc(&txp->txbuf [1], ix-1, &txp->txbuf[ix]);

	ix++;				/* Bump Past the CRC	*/

	txp->txbuf[ix++] = STX;		/* Start of the output cmd */

	six = ix;			/* Save the start of the crc */

	txp->txbuf[ix++] = (card | 0x80);	/* Parameter Number	*/

	cval = ((pmod->params[card].format << 2) | 0x80);

	tvalue = (val & mask);

	txp->txbuf[ix++] = (cval + ((tvalue >> 14) & 0x3)); /* Format */
	txp->txbuf[ix++] = (((tvalue >> 7) & 0x7f) | 0x80);
	txp->txbuf[ix++] = ((tvalue & 0x7f) | 0x80);

	txp->txbuf[ix++] = ETX;

	makecrc(&txp->txbuf [six], (ix-six), &txp->txbuf[ix]);
	ix++;
	/* Time to send the message and get an ACK */

	txp->txmsglen = ix;

	mstatus = msgxfer( line, ACK, pmod);

	if (mstatus == XFERBAD) {
		pmod->no_offlines++;
		pmod->badoutcnt++;
		pmod->is_responding = FALSE;
		return(FALSE);
	}
	if (mstatus == XFERBADRESP) return (FALSE);

	return(TRUE);

    }


/* makecrc
/*
*/
static  void makecrc( unsigned char *chkbuf, int len,char unsigned *retbuf)

    {
	unsigned short checksum;
	unsigned short     j;

	checksum = 0;
	for (j = 0; j < len; j++) {
	    checksum ^= chkbuf[j];
	}
	retbuf[0] = ( (checksum & 0xff) | 0x80);
    }

/*
/*      hexascii
/*
/*      The purpose of this routine is to convert the hex byte to 2 ascii
/*      characters and to place them into the buffer.
/*
*/
static  void hexascii(unsigned short val,unsigned char *buf)

    {
	unsigned char    hi,lo;

	hi = ((val & 0x0f0) >> 4);
	lo = (val & 0x0f);

	if (hi<=9)  hi+='0'; else hi+=('A'-10);
	if (lo<=9)  lo+='0'; else lo+=('A'-10);

	buf[0] = hi;
	buf[1] = lo;

    }

/*
/*      asciihex
/*
/*      The purpose of this routine is to convert the next 2 ascii
/*      characters in the buffer to hex and return them in val. It
/*      will return TRUE if good converdion of FALSE if bad.
*/
static  short asciihex(unsigned short *val,unsigned char *buf)

    {

	if ((buf[0] < '0') || (buf[0] > 'F')) return (FALSE);
	if ( buf[0] <= '9') *val = ((buf[0] & 0x0f) << 4);
	else {
	    if (buf[0] < 'A') return (FALSE);
	    else *val = ((buf[0] - 'A' + 10) << 4);
	}
	if ((buf[1] < '0') || (buf[1] > 'F')) return (FALSE);
	if ( buf[1] <= '9') *val += (buf[1] & 0x0f);
	else {
	    if (buf[1] < 'A') return (FALSE);
	    else *val += (buf[1] - 'A' + 10);
	}
	return (TRUE);
    }


/*
/*      dump_mod_table
/*
/*      The purpose of this routine is to dump the module description
/*      portion of the analog devices driver tables for the selected line.
/*      It will print out two lines for each module which is responding.
/*      The first line will give the general module header, the second line
/*      will give module type specific info.
*/
static  void dump_mod_table(unsigned short line)

    {
	TCSCNTRL  *pmod;
	short           ix;



	if (line != line) return;  /* No Warnings */
	for (ix=0; ix<NUMTCSCNTRLS; ix++) {
	    pmod = plcs[line]+ix;
	    if (pmod->is_responding) {


	    }
	}
    }
