/*drvSv85.h,v 1.1.1.1 1999/08/03 14:44:49 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
*	File:	drvSv85.h
*	Auth:	John L. Dalesio
*	Edit:	02/25/93	JLD
*		12/06/93	WEP: comments only
*		01/05/94	WEP: added timeout value for mutual exclusion logic
*		07/18/94	WEP: added expanded results definitions for improved
*                                 : Modbus command NACK handling.                      
*              01/16/95        WEP: change TXTIMEOUT to one second.                   
*              01/17/95        WEP: add more trace bit definitions                     
*
*	This file contains the data structures used by the Modicon MOD
*	I/O driver.                                                            	       	
*                                                                             	      
*/

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d


/**********
	Define the TCP and UDP port numbers for communicating with the
	SV85 and between redundant partner drivers.
**********/

#define MOD_PORT	5800

/*	number of lines of modbus 	*/

#define		MODLINES	4

/*	number of modbus plcs per line */

#define		NUMMODPLCS	20

/*	The IOC will assume that the data to be exchanged with each PLC will           	
*	come from a fixed number of coils, digital inputs, analog inputs               
*	or registers. It will read these in groups to minimize the amount of           	
*	communications overhead. It will also try to build as large a fetch            
*	message as possible & required by remembering which consecutive groups         	
*	have been requested by the SCAN task.
*/

#define MCOILS		0		/* Card number for coils 		*/
#define MDINS		1		/* Card number for digital inputs 	*/
#define MAINS		3		/* Card number for analog inputs 	*/
#define MREGS		4		/* Card number for registers 		*/
#define MPSEUDO		33		/* Card number for pseudo i/o 		*/
#define	MPSEUDO1	34		/* Card number for CPU status words	*/

/* Number of Groups of items */

#define NUMMODCOILS	128		/* 128 groups of 32 coils		*/
#define NUMMODDIS	64		/* 64 groups of 32 digitals		*/
#define NUMMODAIS	32		/* 32 groups of 8 analogs		*/
#define NUMREGISTERS	128		/* 128 groups of 8 registers		*/

#define BITBLOCK        32		/* size of coil/digital groups	        */
#define REGBLOCK	8		/* size of analog/register groups	*/

/* Number of items */

#define MINCOILNO       1		/* min number of individual coil 	*/
#define MINDINO         1		/* max number of individual digital 	*/
#define MINAINO         1		/* max number of individual analog      */
#define MINREGNO        1		/* max number of individual register	*/

/* Number of items */

#define MAXCOILNO	(NUMMODCOILS * BITBLOCK)	/* max number of individual coils 	*/
#define MAXDINO		(NUMMODDIS * BITBLOCK)		/* max number of individual digitals 	*/
#define MAXAINO		(NUMMODAIS * REGBLOCK)		/* max number of analogs 		*/
#define MAXREGNO	(NUMREGISTERS * REGBLOCK)	/* max number of registers 		*/

#define MAXDIGBLKSIZE	60		/* Maximum number of digital groups 	*/
					/*  to request in one packet        	*/
#define MAXREGBLKSIZE	14		/* Maximum number of analog groups 	*/
					/*  to request in one packet        	*/

/*
*	Conversion types for analog inputs
*
*/
#define		AICNVINTEGER	10
#define		AICNVDOUBLEINT	11
#define		AICNVFLOAT	12
#define		AICNVDOUBLEFLT	13
#define		AICNVASCIIFLT	14
#define		AICNVSTATUS	15
#define		AICNVTIME	16
#define		AICNVASCII_2	17
#define		AICNVASCII_4	18
#define		AICNVASCII_12	19
#define		AICNVASCII_20	20
#define		MAXCONV		20

/* The following must match the definition in recChoice.ascii */

#define NOCONV 0
#define LINEAR 1
#define ASCFLT 2


#define    WATCHDOGTIMEOUT       (10 * 60)	/* watchdog monitor timeout.  */


/*
*	Communications I/O Structures
*
*/

#define		RXSEM		0
#define		TXSEM		1
#define		MAXRXMSG	256
#define		MAXTXMSG	256


typedef	struct
{
	unsigned short		rxstate;
	unsigned short		fixedlen;
	unsigned short		rxmsglenexpected;
        unsigned short          rxmsglen;
       	unsigned short		rxix;
	SEMAPHORE		*semid;
	unsigned short		*status;
        unsigned char           txaddress;
        unsigned char           txfunction;
       	unsigned char		rxbuf [MAXRXMSG];
} RXLINE;


typedef	struct
{
	unsigned short		protocol;
	unsigned short		txdisable;
	unsigned short		txtocnt;
	unsigned short		txhwerrcnt;
	unsigned short		txmsglen;
	unsigned short		txix;
	SEMAPHORE		*semid;
	unsigned short		*status;
	unsigned char		txbuf [MAXTXMSG];
} TXLINE;

/*
*	Communications I/O Constants
*
*/

#define		RXGOOD			1
#define		RXBAD			2
#define		RXTIMEOUT		60	/* 16 ms ticks 				*/
#define		ACTIVITYTIME		300	/* must see activity in 5 seconds 	*/

#define		VARRESP			FALSE
#define		FIXEDRESP		TRUE
#define		ONERETRY		0x8000	/* OR'ed into line number for xfermsg 	*/
#define		RXIGNORE		0
#define		RXADRST			1
#define		RXFNCST			2
#define		RXLENST			3
#define		RXDATAST		4

#define		TXGOOD			1
#define		TXBAD			2
#define		IO_TIMEOUT		3	/* Code returned by timer 		*/


#define		TXTIMEOUT		60	/* 16 ms ticks: one second              */

#define		MAXRETRYS		3	/* retransmission trys 			*/

#define		MAXCFGCK		8	/* Try to Bring online each cycle 	*/
#define		BRINGBACKCNT		4	/* Poll loops per cycle 		*/

#define		XFERGOOD		1
#define		XFERBAD			2

#define		RESNAMELEN		16	/* length of a resource name 		*/
#define		SLWWINDOW		900	/* # of ticks to allow slave to try 	*/

/*
*	PLC PROGRAM LOAD STATES
*/

#define		PLC_NOT_LOADING		0
#define		PLC_LOAD_REQUESTED	1
#define		PLC_LOAD_IN_PROGRESS	2
#define		PLC_LOAD_FAILED		3
#define		PLC_LOAD_SUCCESSFUL	4

/*	Define the MODBUS Read Status masks	*/

#define		BATTERY_OK		0x10
#define		MEM_PROTECT		0x20
#define		RUN_LIGHT		0x40
#define		POWER_OK		0x80


/*
* Output Queue Management Structures for Modbus Protocol
*
*/

#define         AOUTTYPE        1
#define         BOUTTYPE        2
#define         SOUTTYPE        3

#define         OUTQSIZE        128
#define		STRINGMSGLEN	80

typedef struct
{
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	short           aovalue;
	double		dblaovalue;
	unsigned short	conversion;
} AOUT, *AOUTPTR;

typedef struct
{
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	long            bomask;
	long            bovalue;
} BOUT, *BOUTPTR;

typedef struct
{
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  sosignal;
	long            solen;
	unsigned char   sovalue [STRINGMSGLEN];
} SOUT, *SOUTPTR;

typedef struct
{
	unsigned short  otype;
	union
	{
		AOUT    aonode;
		BOUT    bonode;
		SOUT	sonode;
	} out;
} OUTQENT;

typedef struct
{
	unsigned short	ooutix;
	unsigned short	oinix;
	OUTQENT         outque [OUTQSIZE];
} OUTPUTQUE;

/*
*	Internal IOC Structure for MODICON PLC
*
*/
typedef	struct
{
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		last_read;
	unsigned char		red_update;
} MODCOILS;

typedef	struct
{
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		last_read;
} MODDIS;

typedef struct
{
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		ioregs [REGBLOCK];
	unsigned short		data_togo [REGBLOCK];
	unsigned short		regs_togo [REGBLOCK];
	unsigned char		red_update;
} REGISTERS;

typedef struct
{
	unsigned short		was_requested;
	unsigned short		data_valid;
	short			aivals [REGBLOCK];
} MODAIS;

typedef	struct
{
	unsigned short  was_requested;		/* 0  The scan task has accesses this RTU 	*/
	unsigned short  is_responding;		/* 1  The RTU is responding 			*/
	unsigned short  holdflag;		/*    The RTU data is in hold 			*/
	unsigned short  prog_load_stat;		/* 2  Program load Status 			*/
	unsigned short  prog_load_page;		/* 3  Program Load Block Number 		*/
	unsigned short  prog_load_number;	/* 4  Program Number to Load 			*/
	unsigned short  no_dataout_msgs;	/* 5  Count Output Msgs 			*/
	unsigned short  no_datain_msgs;		/* 6  Count Data Requests 			*/
	unsigned short  no_rxnoresp;		/* 7  Count No Responses 			*/
	unsigned short  no_txnoresp;		/* 8  Count No Responses 			*/
	unsigned short  no_dataerr;		/* 9  Count Message Echo Errors 		*/
	unsigned short  no_retries;		/* 10 Count Retries 				*/
	unsigned short  no_rxhwerrs;		/* 11 Count Comm Hardware Errors 		*/
	unsigned short  no_txhwerrs;		/* 12 Count Comm Hardware Errors 		*/
	unsigned short  no_crcerrs;		/* 13 Count CRC Errors 				*/
	unsigned short  no_offlines;		/* 14 Count Off Line Transitions		*/
	unsigned short  cmd_reset_stat;		/* 15 Comand to reset stats			*/
	unsigned short  cmd_force_offline;	/* 16 Command to Force Offline 			*/
	unsigned short  badoutcnt;		/* 17 Count Failed output commands 		*/
						/* 18 Back up Mode 1 = Auto 0 = Man		*/
						/* 19 Active Channel 1= Priamry 0 = Backup 	*/
	unsigned short	prog_load_code;		/* 20 Program Load Error Code 			*/
	unsigned short	plc_start;		/* 21 Command the PLC to Start 			*/
	unsigned short	plc_stop;		/* 22 Command the PLC to Stop 			*/
	/* The following 4 points are BI's of type MPSEUDO1	*/

	unsigned long	stat_fetch_time;	/* time in ticks to fetch status */
	unsigned long	last_good_msg_time;		/* time in ticks to fetch status */

	unsigned short	battery_okay;		/* 04 Battery OK, 0 OK, 1 NOT */
	unsigned short	memory_protect;		/* 05 Memory protect, 0 ON, 1 OFF */
	unsigned short	run_light;		/* 06 Run Light, 0 ON, 1 OFF */
	unsigned short	power_okay;		/* 07 Power Okay, 1 ON, 0 OFF */

	unsigned short	red_update;		/* set when outputs change */
	unsigned short	partnerresp;
	unsigned short	outflags[10];		/* flags for output buffer changed */
	MODCOILS	coils [NUMMODCOILS];
	MODDIS		dis [NUMMODDIS];
	MODAIS		ais [NUMMODAIS];
	REGISTERS	regs [NUMREGISTERS];
} MODPLC;

/*
* MODBUS Protocol function codes
*/

#define MODCOILREAD		0x01  /* READ COIL STATUS: (ON/OFF) read a group of logic coils.     */
#define MODDIREAD		0x02  /* READ INPUT STATUS:(ON/OFF) of a group of discrete inputs.   */
#define MODREGREAD		0x03  /* READ HOLDING REGISTERS:    get current binary value in one  */
                                      /*                            or more holding registers.       */
#define MODAIREAD		0x04  /* READ INPUT REGISTERS:      get current binary value in one  */
                                      /*                            or more input registers.         */

#define MODCOILWRITE		0x05  /* FORCE SINGLE COIL:         force logic coil to ON/OFF state.*/
#define MODREGWRITE		0x06  /* PRESET SINGLE REGISTER:    place a specific binary value    */
                                      /*                            into a holding register.         */

#define MODLOOPBACK		0x08  /* LOOPBACK DIAGNOSTIC TEST                                    */

#define MODREPSLVID		0x11  /* MODBUS Report Slave ID */
#define MODMCOILWRITE		0x0F  /* FORCE MULTIPLE COILS:      force series of consecutive      */
                                      /*                            logic coils to ON/OFF states.    */
#define MODMREGWRITE		0x10  /* PRESET MULTIPLE REGISTERS: put specific binary values into  */
                                      /*                            consecutive series of holding    */
                                      /*                            registers.                       */

#define FNCERRBIT		0x80  /* Modbus error function code */


/* Various message lengths used in message transmission and response validation */

#define MODWRITEHDRLEN		0x06	/* wep: not currently used */
#define MODREADMSGLEN		0x06
#define MODSETCOILLEN		0x06
#define MODSETREGLEN		0x06
#define MODSET1REGLEN		0x09
#define MODSET2REGLEN		0x0b
#define MODSET3REGLEN		0x0d
#define MODSET4REGLEN		0x0f
#define MODMSETCOILLEN		0x07	/* wep: not currently used */
#define MODMSETCOILRESPLEN	0x06
#define MODCRCLEN		0x02
#define MODLOOPBACKLEN		0x06
#define MODLOOPBACKRESPLEN	0x08	/* Including CRC */
#define MODWRITERESPLEN		0x08	/* Including CRC */
#define MODSETMREGRESPLEN	0x06
#define MODMSETREGRESPLEN	0x06
#define NAKMSGLEN		0x03	/* bytes left get */
#define MODREPSLVIDMSGLEN	0x02
#define MODREPSLVIDRESPLEN	0x09

/*
* MODBUS Exception Codes
*/

#define MODEXCEPTFNC		0x01	/* wep: not used */
#define MODEXCEPTADR		0x02	/* "    "   "    */
#define MODEXCEPTDATA		0x03	/* "    "   "    */
#define MODEXCEPSLAVE		0x04	/* "    "   "    */
#define MODEXCEPTSACK		0x05	/* "    "   "    */
#define MODEXCEPTBUSY		0x06	/* "    "   "    */

/* Command Results */

#define CMDDONE                 0x00     /* the command was sent and the response was error free.   */
#define CMDFAIL                 0x01     /* there were errors in sending or receiving the command.  */
#define CMDNACK                 0x02     /* response was error free, but command was not completed. */
#define CMDREJECT               0x03     /* driver rejected command due to unsupported data type.   */
#define CMDBUSY			0x05	 /* The command was sent, but the device is busy */

/* task definitions */

#define MOD_STACK		8000   /* Stack size              */
#define	MOD_PRIORITY		152    /* default priority        */
#define	MOD_TASK_OPTIONS	8      /* Floating point          */


/* Loader Header Information */

/*	Constants	*/

/*	File Offsets and PLC Memory Offsets	*/

#define PAGE0			71
#define PAGEF			80
#define ASCII			144
#define FUTURE			160
#define DXCFG			98

#define NUMDUMPAREAS		4	/* Number of Dump Blocks 				*/
#define DUMPAREASIZE		18	/* Number of Bytes in a dump block 			*/
#define AREASPERBLK		4	/* Number of areas to dump per block 			*/

#define CONFIGAREA		0	/* Identifier of Configuration Area 			*/

#define LOGICAREATYPE		0x08	/* Bit mask in specifier to indicate logic		*/
#define ASCIIAREATYPE		0x80	/* Bit mask in specifier to indicate ascii msgs 	*/
#define MEMORYAREATYPE		0x00	/* Bit mask in specifier to indicate memory 		*/
#define AREATYPEMASK		0x88	/* Or of Above 						*/
#define MACHINESTATEADDR	101	/* PLC Memory Address of Running and Memory Protect 	*/
#define RUNNINGSTATE		0x40	/* Value of Running controller 				*/
#define MEMPROTECT		0x20	/* Value of Memory Protected Controller 		*/
#define RUNLOADSWITCHADDR	110	/* PLC Memory Addres of the Run Load Switch Value 	*/
#define LOADSWITCH		0x02	/* Switch Value for LOAD MODE 				*/
#define RUNSWITCH		0x01	/* Switch Value for RUN MODE 				*/

/*	File Related Constants */

#define EOLNODE			5	/* End of logic node type 				*/
#define BOCNODE			4	/* Begining of column codes are less this 		*/ 
#define NODELEN			3	/* Length of a logic node 				*/
#define WORDLEN			2	/* Length of a PLC memory word 				*/

/* Message Related Constants */

#define MAXMESSAGELEN		50	/* Greater than any message we will send 		*/
#define NOSTARTNODEMSGLEN	15	/* Maximum size of logic load msg without start found 	*/
#define MAXMSGNODES		11	/* Maximum size of logic load msg 			*/
#define MAXWORDMSGLEN		32	/* Maximum size of memory load message 			*/

#define MODCMDSTARTED		0x05	/* Code for Command Started (must poll for done) 	*/
#define MODCMDDONE		0x06	/* Code for Command Completed 				*/
#define MAXPOLLSFORDONE		0x04	/* Maximum polls to get command done 			*/
#define MODLOAD			0x0D	/* LOAD Function Code 					*/
#define MODPOLL			0x0E	/* POLL Function Code 					*/
#define MODSFSTOP		0x01	/* STOP Subfunction Code 				*/
#define MODSFSTART		0x02	/* START Subfunction Code 				*/
#define MODSFREADMEM		0x03	/* READ MEMORY Subfunction Code 			*/
#define MODSFWRITEMEM		0x04	/* WRITE MEMORY Subfunction Code 			*/
#define MODSFWRITENODE		0x07	/* WRITE NODE Subfunction Code 				*/
#define MODSFLOGIN		0x19	/* LOGIN Subfunction Code 				*/
#define MODSFLOGOUT		0x1A	/* LOGOUT Subfunction Code 				*/
#define MODSFINITIALIZE		0x2B	/* INITIALIZE Subfunction Code 				*/
#define MODSFMYSTERY1		0x31	/* MYSTERY1 Subfunction Code 				*/
#define MODSFMYSTERY2		0x34	/* MYSTERY2 Subfunction Code 				*/
#define MODSFENTERDIM		0x35	/* INVOKE DIM Subfunction Code 				*/
#define MODSFCHKDIM		0x36	/* CONFIRM DIM Subfunction Code 			*/
#define MODSFWRITEASCII		0x39	/* WRITE ASCII MSG Subfunction Code 			*/


/* Termination Error Codes */

#define	LOADOK			0x00
#define	CANNOTOPENFILE		0x01
#define	CANNOTSEEKTO		0x02
#define	CANNOTREAD		0x03
#define	CANNOTLOGOUT		0x04
#define	CANNOTLOGIN		0x05
#define	MEMORYPROTECT		0x06
#define	PLCWONTSTOP		0x07
#define	PLCWONTGETDIM		0x08
#define	PLCWONTSETDIM		0x09
#define	PLCWONTBEDIM		0x0A
#define	PLCWONTSTART		0x0B
#define	PLCWONTINIT		0x0C
#define	CANTGETMODE		0x0D
#define	CANTSETLOADMODE		0x0E
#define	UNEXPECTDEOF		0x0F
#define	LOGICWRITEERROR		0x10
#define	MEMORYWRITEERROR	0x11
#define	CANTSETRUNMODE		0x12
#define	CANTSETLOADEDMODE	0x13


/*	Structure Definitions */

typedef struct
{
	unsigned short		length;
	unsigned short		strtaddr;
} AREADESC;

typedef struct
{
	unsigned char	areaspec;
	unsigned char	page;
	AREADESC	areas [AREASPERBLK];
} DUMPAREA;

typedef struct
{
	unsigned short	line;
	unsigned short	loading;
	MODPLC		*pmod;
	unsigned short	mod;
	FILE		*loadfile;
	long		fileaddr;
	short		rlmode;
	short		machstate;
	unsigned char	filename [30];
	unsigned short	ldblock;
	unsigned short	ldarea;
	DUMPAREA	dadesc [NUMDUMPAREAS];
} LOADINFO;


typedef struct
{
        unsigned long   TXintcount;
        unsigned long   RXintcount;
        unsigned long   msgxfercount;
        unsigned long   startingtick;
} LINESTATS;

/* Trace Bit Definitions */

#define ALLTRAFFICTRACE      0x0001
#define MODBUSNACKTRACE      0x0002        /* MODBUS NACK errors.                               */
#define CONFIGURATIONTRACE   0x0010        /* trace configuration errors in driver calls.       */
#define RUNTIMEERRORTRACE    0x0020        /* trace run time errors in driver calls.            */
#define CALLREJECTTRACE      0x0040        /* trace non-error based driver call rejections.     */

typedef	struct	{
	unsigned short	function;		/* type of processing to do for msg on this path */
	MODPLC		*pmod;			/* pointer to module data */
	unsigned long	timeout;		/* time the output was sent */
	unsigned short	line;			/* line number */
	unsigned short	mod;			/* module number */
	unsigned long	lval;			/* binary value */
	unsigned long	val;			/* mcoil value */
	unsigned long	mask;			/* mbbo mask */
	unsigned short	lsignal;		/* bo signal number */
	unsigned short	sval;			/* analog value */
	unsigned short	regaddr;		/* register address */
	unsigned short	resplen;		/* number of characters in response */
	unsigned short  reqlen;			/* Number of requested bytes */
	unsigned short	updateflag;		/* is this message a redundant update */
	unsigned char	respbuf[50];		/* response expected */
		}	PATHINFO;

typedef struct  {
	PATHINFO	paths[16];		/* Array entry for each path */
		}	LPATHS;

/* Path Processing Functions */

#define SCOIL		0
#define	SMCOIL		1
#define	WMCOIL		2
#define	SREG		3
#define	WMREG		4
#define RCOIL		5
#define RREG		6
#define RDI		7
#define RAI		8

#define	SV85SRRESET	1
#define	SV85SRLOCK	2
#define	SV85SRDUART1L	4
#define SV85INTENB	4
#define	SV85SRDUART2L	8
#define	SV85SRBATOK	0x10
#define	SV85SRBDPRES	0x20
#define	SV85SRHOSTRAM	0x40
#define	SV85SRPEERRAM	0x80

/* SV85 board dual port RAM base addresses */

#define	SV85L1BASE	0xfffff800
#define	SV85L2BASE	0xfffff000
#define	SV85L3BASE	0xffffe800
#define SV85L4BASE	0xffffe000

#define	SV85L1VEC	0xE0
#define	SV85L2VEC	0xE4
#define	SV85L3VEC	0xE8
#define SV85L4VEC	0xEC

#define SV85INTLEV	2

/* Dual port memory offsets */

#define	STATUSREG	0
#define CONTROLREG	0
#define VECTORREG	0x100
#define	DUART1BASE	0x200
#define	DUART2BASE	0x300
#define	RAMBASE		0x400
#define MODBUSBASE	0xf
#define	HOSTMBX		0x7fe
#define	PEERMBX		0x7ff

/* Trace Structure */
typedef	struct {
	unsigned short		path;
	unsigned short		cmd;
	unsigned long		data;
		}	VTRACE;
#define	VTRACELEN	64

typedef struct {
	unsigned short		nextptr;
	VTRACE			tdata[VTRACELEN];
		} VTRACEQUE;


/* hw68681.h - motorola 68681 dual dusart definitions - mvme335 card */

/* base address register offsets */
/* Duart 1 */
/* channel A */

#define	D1MODEA1	0x200	/* rw- Mode reg 1A and 2A */
#define	D1MODEA2	0x200	/* Mode reg 1B and 2B	  */
#define	D1STATA		0x202	/* r-Status w-Clock reg A */
#define D1BAUDA		0X202	/* r-Status w-Clock reg A */
#define	D1CMDA		0x204	/* w- CoMmanD reg A	  */
#define	D1RXBUFA	0x206	/* r-Rx w-Tx Buf A	  */
#define D1TXBUFA	0x206	/* r-Rx w-Tx Buf A        */

/* channel B */

#define	D1MODEB1	0x210	/* rw- Mode reg 1A and 2A */
#define	D1MODEB2	0x210	/* Mode reg 1B and 2B	  */
#define	D1STATB		0x212	/* r-Status w-Clock reg B */
#define D1BAUDB		0x212	/* r-Status w-Clock reg B */
#define	D1CMDB		0x214	/* Command reg B	  */
#define	D1RXBUFB	0x216	/* r- Rx w-Tx  Buf B	  */
#define D1TXBUFB	0x216	/* r- Rx w-Tx  Buf B      */

/* common (A&B) */

#define	D1INPCR		0x208	/* Input port Change reg  */
#define D1AUX		0x208	/* Auxilary control reg   */
#define	D1INTSTA	0x20a	/* r-Interrupt Status	  */
#define D1INTMSK	0x20a	/* w-Interupt Mask	  */
#define	D1CNTMSB	0x20c	/* Current Cntr MSB	  */
#define	D1CNTLSB	0x20e	/* Current Cntr LSB	  */
#define	D1IVR		0x218	/* Interrupt Vector reg	  */
#define	D1INP		0x21a	/* Input Port unlatched	  */
#define	D1OPCFG		0x21a	/* Output Port Config	  */
#define	D1STRCNT	0x21c	/* Start Cntr Command	  */
#define	D1BITSET	0x21c	/* Output Port Bit Set Cmd*/
#define	D1STPCNT	0x21e	/* Stop Cntr Command	  */
#define	D1BITRST	0x21e	/* Output Port Bit Reset  */

/* Duart 2 */
/* channal A */

#define	D2MODEA1	0x300	/* rw- Mode reg 1A and 2A */
#define	D2MODEA2	0x300	/* Mode reg 1B and 2B	  */
#define	D2STATA		0x302	/* r-Status w-Clock reg A */
#define D2BAUDA		0X302	/* r-Status w-Clock reg A */
#define	D2CMDA		0x304	/* w- CoMmanD reg A	  */
#define	D2RXBUFA	0x306	/* r-Rx w-Tx Buf A	  */
#define D2TXBUFA	0x306	/* r-Rx w-Tx Buf A        */

/* channel B */

#define	D2MODEB1	0x310	/* rw- Mode reg 1A and 2A */
#define	D2MODEB2	0x310	/* Mode reg 1B and 2B	  */
#define	D2STATB		0x312	/* r-Status w-Clock reg B */
#define D2BAUDB		0x312	/* r-Status w-Clock reg B */
#define	D2CMDB		0x314	/* Command reg B	  */
#define	D2RXBUFB	0x316	/* r- Rx w-Tx  Buf B	  */
#define D2TXBUFB	0x316	/* r- Rx w-Tx  Buf B      */

/* common (A&B) */

#define	D2INPCR		0x308	/* Input port Change reg  */
#define D2AUX		0x308	/* Auxilary control reg   */
#define	D2INTSTA	0x30a	/* r-Interrupt Status	  */
#define D2INTMSK	0x30a	/* w-Interupt Mask	  */
#define	D2CNTMSB	0x30c	/* Current Cntr MSB	  */
#define	D2CNTLSB	0x30e	/* Current Cntr LSB	  */
#define	D2IVR		0x318	/* Interrupt Vector reg	  */
#define	D2INP		0x31a	/* Input Port unlatched	  */
#define	D2OPCFG		0x31a	/* Output Port Config	  */
#define	D2STRCNT	0x31c	/* Start Cntr Command	  */
#define	D2BITSET	0x31c	/* Output Port Bit Set Cmd*/
#define	D2STPCNT	0x31e	/* Stop Cntr Command	  */
#define	D2BITRST	0x31e	/* Output Port Bit Reset  */

/* mode1 defines */

#define RTS_OUT	       0x00		/* Receiver RTS ctrl by out reg   */
#define RTS_FIFO       0x80            /* Receiver RTS ctrl by FIFO      */
#define RX_INT_RR      0x00            /* Rx Interrupt on receiver ready */
#define RX_INT_FIFO    0x40            /* Rx Interrupt on FIFO full      */
#define ERR_CHAR       0x00            /* Error mode on character        */
#define ERR_BLOCK      0x20            /* Error mode on block            */
#define EVEN_PARITY    0x00           /* Even Parity                     */
#define ODD_PARITY     0x04           /* Odd Parity                      */
#define FORCELO	       0x08           /* Force low Parity                */
#define FORCEHI	       0x0a           /* Force high Parity               */
#define NO_PARITY      0x10           /* NO Parity                       */
#define MULTI_DATA     0x18           /* Mutildrop data tag rep. parity  */
#define MUTLI_ADD      0x1a           /* Mutildrop add. tag rep. parity  */
#define _8BITS	       0x03           /* Eight bits                      */
#define _7BITS	       0x02           /* Seven bits                      */
#define _6BITS	       0x01           /* Six Bits                        */
#define _5BITS	       0x00           /* Five Bits                       */

/* mode 2 defines */

#define NORMAL		0x00		/*Normal rx independant of tx    */
#define ECHO		0x40		/*Automatic Echo                 */
#define LOCAL_LOOP	0x80		/*Local Loopback                 */
#define REMOTE_LOOP	0x0c		/*Remote Loopback                */
#define TXRTSOUT	0x00		/*RTS controlled by output reg   */
#define TXRTSRDY	0x20		/*RTS controlled by tx RDY/BUSY  */
#define TXCTSOUT	0x00		/*CTS controlled by output reg   */
#define TXCTS		0x01		/*tx controlled by CTS status    */
#define _2STOP		0x0f		/* 2 stop bits		         */
#define _1STOP          0x07		/* 1 stop bit                    */

/* Aux control register */

#define BAUDSET_1      0x00		/*Baud set 1 50-38400             */
#define BAUDSET_2      0x80		/*Baud set 2 75-19200             */
#define CNTRMODEA      0x10		/*Counter Mode, clock of A tx     */
#define CNTRMODEB      0x20		/*Counter Mode, clock of B tx     */
#define CTRMODE_16     0x30		/*Counter Mode, DUART clock/16    */
#define TMRMODE_3      0x60		/*Timer Mode, DUART clock 3.6864  */
#define TMRMODE_16     0x70		/*Timer Mode, DUART clock/16      */
#define EN_DSRINTB     0x08		/*Enable DSR int channel B        */
#define DI_DSRINTB     0x00		/*Disable DSR int channel B       */
#define EN_DSRINTA     0x04		/*Enable DSR int channel A        */
#define DI_DSRINTA     0x00		/*Disable DSR int channel A       */
#define EN_CTSINTB     0x02		/*Enable CTS int channel B        */
#define DI_CTSINTB     0x00		/*Disable CTS int channel B       */
#define EN_CTSINTA     0x01		/*Enable CTS int channel A        */
#define DI_CTSINTA     0x00		/*Disable CTS int channel A       */

/* command register */

#define RXENABLE       0x01		/*Enable receiver                 */
#define RXDISABLE      0x02		/*Disable receiver                */
#define TXENABLE       0x04		/*Enable transmitter              */
#define TXDISABLE      0X08		/*Disable transmitter             */
#define MDPRESET       0x10		/*Reset mode reg ptr to reg 1	  */
#define RXRESET	       0x20		/*Reset receiver                  */
#define TXRESET        0x30             /*Reset transmitter               */
#define ERSRESET       0x40		/*Reset error status bits	  */
#define BRDRESET       0x50		/*Reset Break Detect int bit	  */

/* clock select register */

#define RXTX_300   0x44
#define RXTX_1200  0x66       /*ACR(7)=1*/
#define RXTX_2400  0x88
#define RXTX_4800  0x99
#define RXTX_9600  0xbb
#define RXTX_38400 0xcc
#define RXTX_19200 RXTX_38400 /*ACR(7)=1*/

/* input port unlatched */

#define CTSA		0x01
#define	CTSB		0x02
#define	DSRA		0x04
#define	DSRB		0x08
#define DCDA		0x10
#define	DCDB		0x20

/* output port (set) register */

#define RTSA		0x01
#define RTSB		0x02
#define DTRA		0x04
#define DTRB		0x08

/* interrupt status  */

#define ATXRDY		0x01
#define ARXRDY		0x02
#define ABRK		0x04
#define TMRRDY		0x08
#define BTXRDY		0x10
#define BRXRDY		0x20
#define BBRK		0x40
#define PORTCH		0x80
#define TXRXINTRDY	(ARXRDY | BRXRDY | ATXRDY | BTXRDY)

#define OVERRUN		0x10
#define PARITYERR	0x20
#define FRAMERR		0x40
#define	RX_ERR_MASK	(FRAMERR | PARITYERR | OVERRUN)

#define	TX_ERR_MASK		0	/* No Error Indication */
#define TXMODE		1
#define RXMODE		2
#define	NULLMODE	3

/* Message definitions for redundant updates */

#define	MOD_ACK		1
#define MOD_REDUPDT	2

typedef struct
    {
    unsigned int	ackSeqNo;
    } MOD_ACK_MSG;

typedef struct
    {
    unsigned int	ackSeqNo;
    short		blockNo;
    unsigned short	value;
    } MOD_UPDT_MSG;
/*
*	Redundant Communications I/O Structures
*
*/

typedef	struct
{
       	unsigned char		rxbuf;
	unsigned char		rxnew;
} REDRXLINE;


typedef	struct
{
	unsigned short		txmsglen;
	unsigned short		txix;
	unsigned char		txbuf;
} REDTXLINE;


/*	End of File:	drvSv85.h */
