/*drvTankway.h,v 1.1.1.1 1999/08/03 14:44:58 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
**  File:     drvTankway.h                                                          
**  Author:   Walter E. Palmer                                                      
**  Created:  07/21/97                                                              
**  Modified: 12/18/97    WEP: modified and expanded TANKLEVELS structure to handle 
**                           : both forms of the level value.                       
**                                                                                  
**  This file contains the data structures used by the Tankway I/O driver.          
**                                                                                  
*/

#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/*	number of lines of Tankway 	*/
#define         TANKLINES	32
/*	number of Tankway units per line */
#define         NUMUNITS       128

#define LEVEL           0               /* Card number for tank level inputs  */
#define TEMP            1               /* Card number for temperature inputs */
#define DISCRETE        1               /* Card number for discrete inputs    */
#define MPSEUDO		33              /* Card number for pseudo i/o         */

/* Number of items */
#define MAXLEVEL       11               /* max signal value of tank level      */
#define MAXTEMP         0               /* max signal value of temperature     */
#define MAXDISCRETE     1               /* max signal value of discrete inputs */

/*
**	Conversion types for analog inputs	*/
/*
*/
#define		AICNVINTEGER	10
#define		AICNVDOUBLEINT	11
#define		AICNVFLOAT	12
#define		AICNVDOUBLEFLT	13
#define		AICNVASCIIFLT	14
#define		AICNVSTATUS	15
#define		AICNVTIME	16
#define		AICNVASCII_2	17
#define		AICNVASCII_4	18
#define		AICNVASCII_12	19
#define		AICNVASCII_20	20
#define		MAXCONV		20

/* The following must match the definition in recChoice.ascii */
#define NOCONV 0
#define LINEAR 1
#define ASCFLT 2

/*
**	Communications I/O Structures
**
*/

#define		RXSEM		0
#define		TXSEM		1
#define		MAXRXMSG	256
#define		MAXTXMSG	256


typedef	struct {
	unsigned short		rxstate;
	unsigned short		fixedlen;
	unsigned short		rxmsglenexpected;
        unsigned short          rxmsglen;
       	unsigned short		rxix;
	SEMAPHORE		*semid;
	unsigned short		*status;
        unsigned char           txaddress;
        unsigned char           txfunction;
       	unsigned char		rxbuf[MAXRXMSG];
		} 	RXLINE;


typedef	struct {
	unsigned short		protocol;
	unsigned short		txdisable;
	unsigned short		txtocnt;
	unsigned short		txhwerrcnt;
	unsigned short		txmsglen;
	unsigned short		txix;
	SEMAPHORE		*semid;
	unsigned short		*status;
	unsigned char		txbuf[MAXTXMSG];
		} 	TXLINE;
/*
**	Communications I/O Constants
**
*/
#define		RXGOOD			1
#define		RXBAD			2
#define		RXTIMEOUT		60	/* 16 ms ticks 				*/
#define		ACTIVITYTIME		300	/* must see activity in 5 seconds 	*/

/*
#define		VARRESP			FALSE
#define		FIXEDRESP		TRUE
*/
#define		ONERETRY		0x8000	/* OR'ed into line number for xfermsg 	*/
#define		RXIGNORE		0
#define		RXBYTE1			1
#define		RXBYTE2			2

#define		TXGOOD			1
#define		TXBAD			2
#define		IO_TIMEOUT		3	/* Code returned by timer 		*/


#define		TXTIMEOUT		60	/* 16 ms ticks: one second              */

#define		MAXRETRYS		3	/* retransmission trys 			*/

#define		MAXCFGCK		8	/* Try to Bring online each cycle 	*/
#define		BRINGBACKCNT		4	/* Poll loops per cycle 		*/

#define		XFERGOOD		1
#define		XFERBAD			2

#define		RESNAMELEN		16	/* length of a resource name 		*/
#define		SLWWINDOW		900	/* # of ticks to allow slave to try 	*/


/* msgxfer testing data types */
#define         LOOPBACK                0
#define         LEVELWORD               1
#define         TEMPWORD                2





/*
**	Internal IOC Structure for Tankway Unit
**
*/
typedef	struct {
	unsigned short	        was_requested;
	unsigned short	        data_valid;
	unsigned short          raw1_read;
	unsigned short          raw2_read;

	unsigned short	        data_valid16;
	float                   last_read16;
	float                   gray1_read16;
	float                   gray2_read16;
	float                   tmp1_read16;
	float                   tmp2_read16;

	unsigned short	        data_valid32;
	unsigned short          raw_read32;
	float                   last_read32;

		}	TANKLEVELS;

typedef	struct {
	unsigned short	        was_requested;
	unsigned short	        data_valid;
	unsigned short	        discretes;
	unsigned short	        overrange;
	unsigned short          raw1_read;
	unsigned short          raw2_read;
	short                   last_read;
		}	TANKTEMPS;


typedef	struct	{
	unsigned short  was_requested;		/* 0  The scan task has accesses this RTU 	*/
	unsigned short  is_responding;		/* 1  The RTU is responding 			*/
	unsigned short  holdflag;		/*    The RTU data is in hold 			*/
	unsigned short  no_dataout_msgs;	/* 5  Count Output Msgs 			*/
	unsigned short  no_datain_msgs;		/* 6  Count Data Requests 			*/
	unsigned short  no_rxnoresp;		/* 7  Count No Responses 			*/
	unsigned short  no_txnoresp;		/* 8  Count No Responses 			*/
	unsigned short  no_dataerr;		/* 9  Count Message Echo Errors 		*/
	unsigned short  no_retries;		/* 10 Count Retries 				*/
	unsigned short  no_rxhwerrs;		/* 11 Count Comm Hardware Errors 		*/
	unsigned short  no_txhwerrs;		/* 12 Count Comm Hardware Errors 		*/
	unsigned short  no_crcerrs;		/* 13 Count CRC Errors 				*/
	unsigned short  no_offlines;		/* 14 Count Off Line Transitions		*/
	unsigned short  cmd_reset_stat;		/* 15 Comand to reset stats			*/
	unsigned short  cmd_force_offline;	/* 16 Command to Force Offline 			*/
	unsigned short  badoutcnt;		/* 17 Count Failed output commands 		*/
						/* 18 Back up Mode 1 = Auto 0 = Man		*/
						/* 19 Active Channel 1= Primary 0 = Backup 	*/

        unsigned short  tscanpass;		/* Total number of scanpasses. incremented at   */
                                                /*  the start of each unit scan.                */
        unsigned short  msgs;			/* Count of messages which got good responses.  */
                                                /*  incremented on each exit from 'good' msgxfer*/
        unsigned long   responsetime;		/* Total time(ticks) waiting for good responses.*/
                                                /* accumu (exit-entry) time for 'good' msgxfers */

	TANKLEVELS      levels;
	TANKTEMPS       temps[2];
	}  TANKWAY;



/*
** Tankway Protocol function codes
*/

#define TANKLOOPBACK       0x00  /* READ COIL STATUS: (ON/OFF) read a group of logic coils.     */
#define READTANKLEVEL      0x01  /* READ INPUT STATUS:(ON/OFF) of a group of discrete inputs.   */
#define READTANKTEMP1      0x02  /* READ HOLDING REGISTERS:    get current binary value in one  */
#define READTANKTEMP2      0x04  /* READ HOLDING REGISTERS:    get current binary value in one  */


/*
** Tankway Protocol function codes
*/

/* #define MODCOILREAD		0x01  ** READ COIL STATUS: (ON/OFF) read a group of logic coils.     */
/* #define MODDIREAD		0x02  ** READ INPUT STATUS:(ON/OFF) of a group of discrete inputs.   */
/* #define MODREGREAD		0x03  ** READ HOLDING REGISTERS:    get current binary value in one  */
/*                                       **                            or more holding registers.       */
/* #define MODAIREAD		0x04  ** READ INPUT REGISTERS:      get current binary value in one  */
/*                                       **                            or more input registers.         */
/*
** #define MODCOILWRITE		0x05  ** FORCE SINGLE COIL:         force logic coil to ON/OFF state.*/
/* #define MODREGWRITE		0x06  ** PRESET SINGLE REGISTER:    place a specific binary value    */
/*                                       **                            into a holding register.         */
/*
** #define MODLOOPBACK		0x08  ** LOOPBACK DIAGNOSTIC TEST                                    */
/*
** #define MODMCOILWRITE		0x0F  ** FORCE MULTIPLE COILS:      force series of consecutive      */
/*                                       **                            logic coils to ON/OFF states.    */
/* #define MODMREGWRITE		0x10  ** PRESET MULTIPLE REGISTERS: put specific binary values into  */
/*                                       **                            consecutive series of holding    */
/*                                       **                            registers.                       */
/*
** #define FNCERRBIT		0x80  ** Modbus error function code */


/* Various message lengths used in message transmission and response validation */

/* #define MODWRITEHDRLEN		0x06	** wep: not currently used */
/* #define MODREADMSGLEN		0x06
** #define MODSETCOILLEN		0x06
** #define MODSETREGLEN		0x06
** #define MODSET1REGLEN		0x09
** #define MODSET2REGLEN		0x0b
** #define MODSET3REGLEN		0x0d
** #define MODSET4REGLEN		0x0f
** #define MODMSETCOILLEN		0x07	** wep: not currently used */
/* #define MODMSETCOILRESPLEN	0x06
** #define MODCRCLEN		0x02
** #define MODLOOPBACKLEN		0x06
** #define MODLOOPBACKRESPLEN	0x08	** Including CRC */
/* #define MODWRITERESPLEN		0x08	** Including CRC */
/* #define MODSETMREGRESPLEN	0x06
** #define NAKMSGLEN		0x03	** bytes left get */

/*
** MODBUS Exception Codes
*/

/* #define MODEXCEPTFNC		0x01	** wep: not used */
/* #define MODEXCEPTADR		0x02	** "    "   "    */
/* #define MODEXCEPTDATA		0x03	** "    "   "    */
/* #define MODEXCEPSLAVE		0x04	** "    "   "    */
/* #define MODEXCEPTSACK		0x05	** "    "   "    */
/* #define MODEXCEPTBUSY		0x06	** "    "   "    */

/* Command Results */

#define CMDDONE                 0x00     /* the command was sent and the response was error free.   */
#define CMDFAIL                 0x01     /* there were errors in sending or receiving the command.  */
/* #define CMDNACK                 0x02     ** response was error free, but command was not completed. */
/* #define CMDREJECT               0x03     ** driver rejected command due to unsupported data type.   */
/* #define CMDBUSY			0x05	 ** The command was sent, but the device is busy */

/* task definitions */

#define TANK_STACK		8000   /* Stack size              */
#define	TANK_PRIORITY		152    /* default priority        */
#define	TANK_TASK_OPTIONS	8      /* Floating point          */




/*
** Mutual-Exclusion Control for diagnostic/data radio channel sharing */
/*
*/

#define		MUXTIMEOUT	60*30	/* 16.67 ms ticks (60Hz): 30 seconds 	*/

/* Loader Header Information */

/*	Constants	*/

/*	File Offsets and PLC Memory Offsets	*/
/* #define PAGE0			71
** #define PAGEF			80
** #define ASCII			144
** #define FUTURE			160
** #define DXCFG			98
**
** #define NUMDUMPAREAS		4	** Number of Dump Blocks 				*/
** #define DUMPAREASIZE		18	** Number of Bytes in a dump block 			*/
/* #define AREASPERBLK		4	** Number of areas to dump per block 			*/
/*
** #define CONFIGAREA		0	** Identifier of Configuration Area 			*/
/*
** #define LOGICAREATYPE		0x08	** Bit mask in specifier to indicate logic		*/
/* #define ASCIIAREATYPE		0x80	** Bit mask in specifier to indicate ascii msgs 	*/
/* #define MEMORYAREATYPE		0x00	** Bit mask in specifier to indicate memory 		*/
/* #define AREATYPEMASK		0x88	** Or of Above 						*/
/* #define MACHINESTATEADDR	101	** PLC Memory Address of Running and Memory Protect 	*/
/* #define RUNNINGSTATE		0x40	** Value of Running controller 				*/
/* #define MEMPROTECT		0x20	** Value of Memory Protected Controller 		*/
/* #define RUNLOADSWITCHADDR	110	** PLC Memory Addres of the Run Load Switch Value 	*/
/* #define LOADSWITCH		0x02	** Switch Value for LOAD MODE 				*/
/* #define RUNSWITCH		0x01	** Switch Value for RUN MODE 				*/
/*
** **	File Related Constants */
/* #define EOLNODE			5	** End of logic node type 				*/
/* #define BOCNODE			4	** Begining of column codes are less this 		*/
/* #define NODELEN			3	** Length of a logic node 				*/
/* #define WORDLEN			2	** Length of a PLC memory word 				*/
/*
** ** Message Related Constants */
/* #define MAXMESSAGELEN		50	** Greater than any message we will send 		*/
/* #define NOSTARTNODEMSGLEN	15	** Maximum size of logic load msg without start found 	*/
/* #define MAXMSGNODES		11	** Maximum size of logic load msg 			*/
/* #define MAXWORDMSGLEN		32	** Maximum size of memory load message 			*/
/*
** #define MODCMDSTARTED		0x05	** Code for Command Started (must poll for done) 	*/
/* #define MODCMDDONE		0x06	** Code for Command Completed 				*/
/* #define MAXPOLLSFORDONE		0x04	** Maximum polls to get command done 			*/
/* #define MODLOAD			0x0D	** LOAD Function Code 					*/
/* #define MODPOLL			0x0E	** POLL Function Code 					*/
/* #define MODSFSTOP		0x01	** STOP Subfunction Code 				*/
/* #define MODSFSTART		0x02	** START Subfunction Code 				*/
/* #define MODSFREADMEM		0x03	** READ MEMORY Subfunction Code 			*/
/* #define MODSFWRITEMEM		0x04	** WRITE MEMORY Subfunction Code 			*/
/* #define MODSFWRITENODE		0x07	** WRITE NODE Subfunction Code 				*/
/* #define MODSFLOGIN		0x19	** LOGIN Subfunction Code 				*/
/* #define MODSFLOGOUT		0x1A	** LOGOUT Subfunction Code 				*/
/* #define MODSFINITIALIZE		0x2B	** INITIALIZE Subfunction Code 				*/
/* #define MODSFMYSTERY1		0x31	** MYSTERY1 Subfunction Code 				*/
/* #define MODSFMYSTERY2		0x34	** MYSTERY2 Subfunction Code 				*/
/* #define MODSFENTERDIM		0x35	** INVOKE DIM Subfunction Code 				*/
/* #define MODSFCHKDIM		0x36	** CONFIRM DIM Subfunction Code 			*/
/* #define MODSFWRITEASCII		0x39	** WRITE ASCII MSG Subfunction Code 			*/


/* Termination Error Codes */
/*
** #define	LOADOK			0x00
** #define	CANNOTOPENFILE		0x01
** #define	CANNOTSEEKTO		0x02
** #define	CANNOTREAD		0x03
** #define	CANNOTLOGOUT		0x04
** #define	CANNOTLOGIN		0x05
** #define	MEMORYPROTECT		0x06
** #define	PLCWONTSTOP		0x07
** #define	PLCWONTGETDIM		0x08
** #define	PLCWONTSETDIM		0x09
** #define	PLCWONTBEDIM		0x0A
** #define	PLCWONTSTART		0x0B
** #define	PLCWONTINIT		0x0C
** #define	CANTGETMODE		0x0D
** #define	CANTSETLOADMODE		0x0E
** #define	UNEXPECTDEOF		0x0F
** #define	LOGICWRITEERROR		0x10
** #define	MEMORYWRITEERROR	0x11
** #define	CANTSETRUNMODE		0x12
** #define	CANTSETLOADEDMODE	0x13


**	Structure Definitions */
/* typedef struct {
** 	unsigned short		length;
** 	unsigned short		strtaddr;
** 		}	AREADESC;
**
** typedef struct {
** 	unsigned char	areaspec;
** 	unsigned char	page;
** 	AREADESC	areas[AREASPERBLK];
** 		}	DUMPAREA;
**
** typedef struct {
** 	unsigned short	line;
** 	unsigned short	loading;
** 	MODPLC		*pmod;
** 	unsigned short	mod;
** 	FILE		*loadfile;
** 	long		fileaddr;
** 	short		rlmode;
** 	short		machstate;
** 	unsigned char	filename[30];
** 	unsigned short	ldblock;
** 	unsigned short	ldarea;
** 	DUMPAREA	dadesc[NUMDUMPAREAS];
** 		} 	LOADINFO;
**
**
*/
typedef struct {
        unsigned long   TXintcount;
        unsigned long   RXintcount;
        unsigned long   RXbadcount;
        unsigned long   msgxfercount;
        unsigned long   startingtick;
               }        LINESTATS;


/* Trace Bit Definitions */

#define ALLTRAFFICTRACE      0x0001
#define MODBUSNACKTRACE      0x0002        /* MODBUS NACK errors.                               */
#define CONFIGURATIONTRACE   0x0010        /* trace configuration errors in driver calls.       */
#define RUNTIMEERRORTRACE    0x0020        /* trace run time errors in driver calls.            */
#define CALLREJECTTRACE      0x0040        /* trace non-error based driver call rejections.     */


/*	End of File:	drvTankway.h */
