/*drvTiway.h,v 1.1.1.1 1999/08/03 14:45:06 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/**********
*
*	File:	drvTiway.h
*	Auth:	William H. Szostak
*	Edit:	11/11/92	WHS
*
*	This file contains the data structures used by the Tiway
*	I/O driver.
*
**********/

#define	TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/*	number of lines supported by the Tiway driver */

#define		TILINES			20

/*	number of Tiway devices per line */

#define		NUMTIPLCS		100

/*
/*	The IOC will assume the data to exchanged with each PLC will
/*	come from the a fixed number of file elements It will read
/*	these groups to minimize the amount of
/*	communications overhead. It will also try to build as large a fetch
/*	message as possible & required by remembering which consecutive groups
/*	have been requested by the SCAN task.
*/

#define	TIVMEM		1		/* VMEM - Binary, Integer and Float */
#define	TICMEM		2		/* CMEM - Binary, Integer and Float */
#define	TIXMEM		3		/* X memory (input only) */
#define	TITXMEM		3		/* Tank X memory (input only) */
#define	TIYMEM		4		/* Y memory */
#define	TITYMEM		4		/* Tank Y memory */
#define	TICRMEM		5		/* CR memory */
#define	TIWXIN		9		/* WX word input memory (input only) */
#define	TIWYOUT		10		/* WY word output memory */
#define	TISTATUS	26		/* Status words (input only) */
#define TILPSTAT	45		/* Loop Status (input only) */
#define	TILPCNTL	46		/* Loop Control Flags */
#define	TILPMD		53		/* Loop Mode */
#define	TIAVFLAG	69		/* Analog Variable Flags (input only) */
#define	TIACFLAG	70		/* Analog Control Flags */
#define TILPGAIN	96		/* Loop Gain */
#define	TILPRESET	97		/* Loop Reset */
#define TILPRATE	98		/* Loop Rate */
#define	TILPHIALRM	99		/* Loop High Alarm */
#define	TILPLOALRM	100		/* Loop Low Alarm */
#define	TILPPROCVAR	101		/* Loop Process Variable */
#define	TILPPVHILIM	102		/* Loop PV High Limit */
#define	TILPPVLOLIM	103		/* Loop PV Low Limit */
#define	TILPORGDEVALRM	104		/* Loop Orange Dev Alarm */
#define	TILPYELDEVALRM	105		/* Loop Yellow Dev Alarm */
#define TILPSMPRATE	106		/* Loop Sample Rate */
#define TILPREMSP	107		/* Loop Remote Set Point */
#define TILPOUTPUT	108		/* Loop Output */
#define	TILPERROR	112		/* Loop Error (Input Only) */
#define TILPBIAS	113		/* Loop Bias */
#define TILPPVHIHILIM	114		/* Loop PV High High Limit */
#define TILPPVLOLOLIM	115		/* Loop PV Low Low Limit */
#define	TILPROCALRM	116		/* Loop Rate of Change Alarm */
#define TIANPVHIALRM	124		/* Analog Alarm PV High Alarm */
#define TIANPVLOALRM	125		/* Analog Alarm PV Low Alarm */
#define	TIANPROCVAR	126		/* Analog Alarm Process Variable */
#define TIANPVHILIM	127		/* Analog Alarm PV High Limit */
#define	TIANPVLOLIM	128		/* Analog Alarm PV Low Limit */
#define TIANORGDEVLIM	129		/* Analog Alarm Orange Deviation Limit */
#define TIANYELDEVLIM	130		/* Analog Alarm Yelow Deviation Limit */
#define TIANSMPRATE	131		/* Analog Alarm Sample Rate */
#define TIANSETPOINT	132		/* Analog Alarm Set Point */
#define TIANERROR	135		/* Analog Alarm Error (Input Only) */
#define TIANPVHIHIALRM	136		/* Analog Alarm PV High High Alarm */
#define TIANPVLOLOALRM	137		/* Analog Alarm PV Low Low Alarm */
#define TIANROC		138		/* Analog Alarm Rate of Change */
#define	SCANCNTL	997		/* Scan frequency for loops and alarms */
#define PSEUDO1		998		/* PLC State of Health */
#define	PSEUDO2		999		/* Tiway driver State of Health */

#define TINOGRPELEMS    4		/* Number of Elements in Each Group */

#define	NUMTIVMEM		256	/* 256 groups of 4 VMEM locations */
#define	NUMTICMEM		256	/* 256 groups of 4 CMEM locations */
#define	NUMTIXMEM		1024	/* 256 groups of 4 XMEM locations */
#define	NUMTITXMEM		16	/* 256 groups of 4 XMEM locations */
#define	NUMTIYMEM		1024	/* 256 groups of 4 XMEM locations */
#define	NUMTITYMEM		16	/* 256 groups of 4 YMEM locations */
#define	NUMTICRMEM		1024	/* 256 groups of CRMEM locations */
#define	NUMTIWXIN		256	/* 256 groups of WXIN locations */
#define	NUMTIWYOUT		256	/* 256 groups of WYOUT locations */
#define	NUMTISTATUS		256	/* 256 groups of TISTATUS locations */
#define	NUMTILPSTAT		16	/* 16 groups of TILPSTAT locations */
#define NUMTILPCNTL		16	/* 16 groups of TILPCNTL locations */
#define	NUMTILPMD		16	/* 16 groups of TILPMD locations */
#define	NUMTIAVFLAG		16	/* 16 groups of TIAVFLAG locations */
#define	NUMTIACFLAG		16	/* 16 groups of TIACFLAG locations */
#define NUMTILPGAIN		16	/* 16 groups of TILPGAIN locations */
#define	NUMTILPRESET		16	/* 16 groups of TILPRESET locations */
#define NUMTILPRATE		16	/* 16 groups of TILPRATE locations */
#define	NUMTILPHIALRM		16	/* 16 groups of TILPHIALRM locations */
#define	NUMTILPLOALRM		16	/* 16 groups of TILPLOALRM locations */
#define	NUMTILPPROCVAR		16	/* 16 groups of TILPPROCVAR locations */
#define	NUMTILPPVHILIM		16	/* 16 groups of TILPPVHILIM locations */
#define	NUMTILPPVLOLIM		16	/* 16 groups of TILPPVLOLIM locations */
#define	NUMTILPORGDEVALRM	16	/* 16 groups of TILPORGDEVALRM locations */
#define	NUMTILPYELDEVALRM	16	/* 16 groups of TILPORGDEVALRM locations */
#define NUMTILPSMPRATE		16	/* 16 groups of TILPSMPRATE locations */
#define NUMTILPREMSP		16	/* 16 groups of TILPREMSP locations */
#define NUMTILPOUTPUT		16	/* 16 groups of TILPOUTPUT locations */
#define	NUMTILPERROR		16	/* 16 groups of TILPERROR locations */
#define NUMTILPBIAS		16	/* 16 groups of TILPBIAS locations */
#define NUMTILPPVHIHILIM	16	/* 16 groups of TILPPVHIHILIM locations */
#define NUMTILPPVLOLOLIM	16	/* 16 groups of TILPPVLOLOLIM locations */
#define	NUMTILPROCALRM		16	/* 16 groups of TILPROCALRM locations */
#define NUMTIANPVHIALRM		16	/* 16 groups of TILPROCALRM locations */
#define NUMTIANPVLOALRM		16	/* 16 groups of TILPROCALRM locations */
#define	NUMTIANPROCVAR		16	/* 16 groups of TIANPVLOALRM locations */
#define NUMTIANPVHILIM		16	/* 16 groups of TIANPVHILIM locations */
#define	NUMTIANPVLOLIM		16	/* 16 groups of TIANPVLOLIM locations */
#define NUMTIANORGDEVLIM	16	/* 16 groups of TIANORGDEVLIM locations */
#define NUMTIANYELDEVLIM	16	/* 16 groups of TIANYELDEVLIM locations */
#define NUMTIANSMPRATE		16	/* 16 groups of TIANSMPRATE locations */
#define NUMTIANSETPOINT		16	/* 16 groups of TIANSETPOINT locations */
#define NUMTIANERROR		16	/* 16 groups of TIANERROR locations */
#define NUMTIANPVHIHIALRM	16	/* 16 groups of TIANPVHIHIALRM locations */
#define NUMTIANPVLOLOALRM	16	/* 16 groups of TIANPVLOLOALRM locations */
#define NUMTIANROC		16	/* 16 groups of TIANROC locations */

#define SLOWPASS	8		/* Number of passes to skip slow points */

/* MAXIMUM ELEMENTS PER MEMORY TYPE */

#define	MAXTIVMEM		(NUMTIVMEM * TINOGRPELEMS)
#define	MAXTICMEM		(NUMTICMEM * TINOGRPELEMS)
#define	MAXTIXMEM		NUMTIXMEM
#define	MAXTITXMEM		NUMTITXMEM
#define	MAXTIYMEM		NUMTIYMEM
#define	MAXTITYMEM		NUMTITYMEM
#define	MAXTICRMEM		NUMTICRMEM
#define	MAXTIWXIN		(NUMTIWXIN * TINOGRPELEMS)
#define	MAXTIWYOUT		(NUMTIWYOUT * TINOGRPELEMS)
#define	MAXTISTATUS		(NUMTISTATUS * TINOGRPELEMS)
#define	MAXTILPSTAT		(NUMTILPSTAT * TINOGRPELEMS)
#define	MAXTILPCNTL		(NUMTILPCNTL * TINOGRPELEMS)
#define	MAXTILPMD		(NUMTILPMD * TINOGRPELEMS)
#define	MAXTIAVFLAG		(NUMTIAVFLAG * TINOGRPELEMS)
#define	MAXTIACFLAG		(NUMTIACFLAG * TINOGRPELEMS)
#define MAXTILPGAIN     	(NUMTILPGAIN * TINOGRPELEMS)
#define	MAXTILPRESET    	(NUMTILPRESET * TINOGRPELEMS)
#define MAXTILPRATE     	(NUMTILPRATE * TINOGRPELEMS)
#define	MAXTILPHIALRM           (NUMTILPHIALRM * TINOGRPELEMS)
#define	MAXTILPLOALRM           (NUMTILPLOALRM * TINOGRPELEMS)
#define	MAXTILPPROCVAR          (NUMTILPPROCVAR * TINOGRPELEMS)
#define	MAXTILPPVHILIM          (NUMTILPPVHILIM * TINOGRPELEMS)
#define	MAXTILPPVLOLIM          (NUMTILPPVLOLIM * TINOGRPELEMS)
#define	MAXTILPORGDEVALRM       (NUMTILPORGDEVALRM * TINOGRPELEMS)
#define	MAXTILPYELDEVALRM       (NUMTILPYELDEVALRM * TINOGRPELEMS)
#define MAXTILPSMPRATE          (NUMTILPSMPRATE * TINOGRPELEMS)
#define MAXTILPREMSP            (NUMTILPREMSP * TINOGRPELEMS)
#define MAXTILPOUTPUT           (NUMTILPOUTPUT * TINOGRPELEMS)
#define	MAXTILPERROR            (NUMTILPERROR * TINOGRPELEMS)
#define MAXTILPBIAS             (NUMTILPBIAS * TINOGRPELEMS)
#define MAXTILPPVHIHILIM        (NUMTILPPVHIHILIM * TINOGRPELEMS)
#define MAXTILPPVLOLOLIM        (NUMTILPPVLOLOLIM * TINOGRPELEMS)
#define	MAXTILPROCALRM          (NUMTILPROCALRM * TINOGRPELEMS)
#define MAXTIANPVHIALRM         (NUMTIANPVHIALRM * TINOGRPELEMS)
#define MAXTIANPVLOALRM         (NUMTIANPVLOALRM * TINOGRPELEMS)
#define	MAXTIANPROCVAR          (NUMTIANPROCVAR * TINOGRPELEMS)
#define MAXTIANPVHILIM          (NUMTIANPVHILIM * TINOGRPELEMS)
#define	MAXTIANPVLOLIM          (NUMTIANPVLOLIM * TINOGRPELEMS)
#define MAXTIANORGDEVLIM        (NUMTIANORGDEVLIM * TINOGRPELEMS)
#define MAXTIANYELDEVLIM        (NUMTIANYELDEVLIM * TINOGRPELEMS)
#define MAXTIANSMPRATE          (NUMTIANSMPRATE * TINOGRPELEMS)
#define MAXTIANSETPOINT         (NUMTIANSETPOINT * TINOGRPELEMS)
#define MAXTIANERROR            (NUMTIANERROR * TINOGRPELEMS)
#define MAXTIANPVHIHIALRM       (NUMTIANPVHIHIALRM * TINOGRPELEMS)
#define MAXTIANPVLOLOALRM       (NUMTIANPVLOLOALRM * TINOGRPELEMS)
#define MAXTIANROC              (NUMTIANROC * TINOGRPELEMS)

/* MAXIMUM ELEMENTS TO ASK FOR IN A COMM REQUEST */

#define	MAXVMEM			8	/*?  */
#define	MAXCMEM			8	/*?  */
#define	MAXXMEM			8	/*?  */
#define	MAXTXMEM		4	/*?  */
#define	MAXYMEM			8	/*? */
#define	MAXTYMEM		4	/*? */
#define	MAXCRMEM		8	/*? */
#define	MAXWXIN			8	/*? */
#define	MAXWYOUT		8	/*? */
#define	MAXSTATUS		8	/*? */
#define	MAXLPSTAT		8	/*? */
#define	MAXLPCNTL		8	/*? */
#define	MAXLPMD			8	/*? */
#define	MAXAVFLAG		8	/*? */
#define	MAXACFLAG		8	/*? */
#define MAXLPGAIN               8	/*? */
#define	MAXLPRESET              8	/*? */
#define MAXLPRATE               8	/*? */
#define	MAXLPHIALRM             8	/*? */
#define	MAXLPLOALRM             8	/*? */
#define	MAXLPPROCVAR            8	/*? */
#define	MAXLPPVHILIM            8	/*? */
#define	MAXLPPVLOLIM            8	/*? */
#define	MAXLPORGDEVALRM         8	/*? */
#define	MAXLPYELDEVALRM         8	/*? */
#define MAXLPSMPRATE            8	/*? */
#define MAXLPREMSP              8	/*? */
#define MAXLPOUTPUT             8	/*? */
#define	MAXLPERROR              8	/*? */
#define MAXLPBIAS               8	/*? */
#define MAXLPPVHIHILIM          8	/*? */
#define MAXLPPVLOLOLIM          8	/*? */
#define	MAXLPROCALRM            8	/*? */
#define MAXANPVHIALRM           8	/*? */
#define MAXANPVLOALRM           8	/*? */
#define	MAXANPROCVAR            8	/*? */
#define MAXANPVHILIM            8	/*? */
#define	MAXANPVLOLIM            8	/*? */
#define MAXANORGDEVLIM          8	/*? */
#define MAXANYELDEVLIM          8	/*? */
#define MAXANSMPRATE            8	/*? */
#define MAXANSETPOINT           8	/*? */
#define MAXANERROR              8	/*? */
#define MAXANPVHIHIALRM         8	/*? */
#define MAXANPVLOLOALRM         8	/*? */
#define MAXANROC                8	/*? */

/* TI device types supported on TIWAY */

#define PLC_5TI		0x0
#define	PLC_510		0x10
#define	PLC_520		0x20
#define	PLC_520C	0x2C
#define	PLC_525		0x25
#define	PLC_530		0x30
#define	PLC_530C	0x3C
#define	PLC_540		0x40
#define	PLC_550		0x50
#define	PLC_560		0x60
#define	PLC_565		0x65
#define	PLC_570		0x70
#define	UHA		0x7E
#define	PM550		0x80
#define	PM551		0x81
#define	UA		0x100
#define	IT_111_121	0x200
#define	IT_150		0x204
#define	IT_160		0x208

/**********
*
*	Scan masks for slow scan memory types
*
**********/

#define SCNLPGAIN	0x00000001	/* Loop Gain */
#define	SCNLPRESET	0x00000002	/* Loop Reset */
#define SCNLPRATE	0x00000004	/* Loop Rate */
#define	SCNLPHIALRM	0x00000008	/* Loop High Alarm */
#define	SCNLPLOALRM	0x00000010	/* Loop Low Alarm */
#define	SCNLPPROCVAR	0x00000020	/* Loop Process Variable */
#define	SCNLPPVHILIM	0x00000040	/* Loop PV High Limit */
#define	SCNLPPVLOLIM	0x00000080	/* Loop PV Low Limit */
#define	SCNLPORGDEVALRM	0x00000100	/* Loop Orange Dev Alarm */
#define	SCNLPYELDEVALRM	0x00000200	/* Loop Yellow Dev Alarm */
#define SCNLPSMPRATE	0x00000400	/* Loop Sample Rate */
#define SCNLPREMSP	0x00000800	/* Loop Remote Set Point */
#define SCNLPOUTPUT	0x00001000	/* Loop Output */
#define	SCNLPERROR	0x00002000	/* Loop Error (Input Only) */
#define SCNLPBIAS	0x00004000	/* Loop Bias */
#define SCNLPPVHIHILIM	0x00008000	/* Loop PV High High Limit */
#define SCNLPPVLOLOLIM	0x00010000	/* Loop PV Low Low Limit */
#define	SCNLPROCALRM	0x00020000	/* Loop Rate of Change Alarm */
#define SCNANPVHIALRM	0x00040000	/* Analog Alarm PV High Alarm */
#define SCNANPVLOALRM	0x00080000	/* Analog Alarm PV Low Alarm */
#define	SCNANPROCVAR	0x00100000	/* Analog Alarm Process Variable */
#define SCNANPVHILIM	0x00200000	/* Analog Alarm PV High Limit */
#define	SCNANPVLOLIM	0x00400000	/* Analog Alarm PV Low Limit */
#define SCNANORGDEVLIM	0x00800000	/* Analog Alarm Orange Deviation Limit */
#define SCNANYELDEVLIM	0x01000000	/* Analog Alarm Yelow Deviation Limit */
#define SCNANSMPRATE	0x02000000	/* Analog Alarm Sample Rate */
#define	SCNANSETPOINT	0x04000000	/* Analog Alarm Set Point */
#define SCNANERROR	0x08000000	/* Analog Alarm Error (Input Only) */
#define SCNANPVHIHIALRM	0x10000000	/* Analog Alarm PV High High Alarm */
#define SCNANPVLOLOALRM	0x20000000	/* Analog Alarm PV Low Low Alarm */
#define SCNANROC	0x40000000	/* Analog Alarm Rate of Change */

/**********
*
*	Basic IO types
*
**********/

#define	TI_AI		0x01
#define	TI_AO		0x02
#define	TI_BI		0x03
#define	TI_BO		0x04

/**********
*
*	Communications I/O Structures
*
**********/

#define		RXSEM		0
#define		TXSEM		1

/* Consecutive No responses to data to force offline */

#define		MAXDATASTRIKE	3	

#define		MAXRXMSG	300

/* Allen Bradley Receive Message tokens */

typedef	struct {
	unsigned short		protocol;
	unsigned short		rxstate;
	unsigned short		rxtocnt;
	unsigned short		rxhwerrcnt;
	unsigned short		token;
	unsigned short		rxmsglen;
	unsigned short		rxix;
	SEM_ID			*semid;
	unsigned short		*status;
	unsigned char		rxbuf [MAXRXMSG + 2];
		} 	RXLINE;

#define		MAXTXMSG	300

typedef	struct {
	unsigned short		protocol;
	unsigned short		txtocnt;
	unsigned short		txhwerrcnt;
	unsigned short		txmsglen;
	unsigned short		txix;
	SEM_ID			*semid;
	unsigned short		*status;
	unsigned short		txdisable;
	unsigned char		txbuf [MAXTXMSG + 2];
		} 	TXLINE;

/**********
*
*	Communications I/O Constants
*
**********/

#define		RXGOOD			1
#define		RXBAD			2
#define		RXTIMEOUT		20 	/* 16 ms ticks */
#define		ACTIVITYTIME		300	/* 5 seconds activity time */

#ifdef TEST
#define		RX_ERR_MASK		(TIMEOUT | FRAMERR | PARITYERR | OVERRUN)
#define		TX_ERR_MASK		0	/* No Error Indication */
#endif

#define		VARRESP			FALSE
#define		FIXEDRESP		TRUE

/* ored into line number for xfermsg */

#define		ONERETRY		0x8000


#define		TXGOOD			1
#define		TXBAD			2

#define		TXTIMEOUT		1255	/* 16 ms ticks */

#define		IO_TIMEOUT		3	/* Code returned by timer */

#define		MAXRETRYS		3	/* retransmission trys */

#define		MAXCFGCK		8	/* Bring online each cycle */
#define		BRINGBACKCNT		4	/* Poll loops per cycle */

#define		XFERGOOD		1
#define		XFERBAD			2

#define		RESNAMELEN		16	/* length of a resource name */

/* Receive Interrupt States */

#define		R_COLON			0
#define		R_COUNT1		1
#define		R_COUNT2		2
#define		R_COUNT3		3
#define		R_COUNT4		4
#define		R_DATA			5
#define		R_ECC1			6
#define		R_ECC2			7
#define		R_ECC3			8
#define		R_ECC4			9
#define		R_SEMICOLON		10
#define		R_CR			11
#define		R_LF			12
#define		R_IGNORE		255

/* # of ticks to allow slave to try 	*/

#define		SLWWINDOW		900

/*	Tiway Primitives	*/

#define	RD_BLOCK	0x20
#define	WR_BLOCK	0x30

/**********
*
* Output Que Management Structures for Allen Bradley Protocol
*

**********/

typedef union {
	char			cval [4];
	short			sval [2];
	long			lval;
	unsigned char		uchar [4];
	unsigned short		usval [2];
	unsigned long		ulval;
	float			fval;
	} TI_RET_VAL;

#define         AOUTTYPE        1
#define         BOUTTYPE        2
#define         OUTQSIZE        128

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	unsigned short	conversion;
	TI_RET_VAL      aovalue;
		} AOUT, *AOUTPTR;

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	long            bomask;
	unsigned short	boshift;
	long            bovalue;
		} BOUT, *BOUTPTR;

typedef struct {
		unsigned short  otype;
		union   {
			AOUT    aonode;
			BOUT    bonode;
			}out;
	       } OUTQENT;

typedef struct {
		unsigned short	ooutix;
		unsigned short	oinix;
		OUTQENT         outque [OUTQSIZE];
		} OUTPUTQUE;

/**********
*
*	Internal IOC Structure for TIWAY devices
*
**********/
typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		vmem [TINOGRPELEMS];
		} VMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		cmem [TINOGRPELEMS];
		} CMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		xmem;
		} XMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned char		txmem;
		} TXMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		ymem;
		} YMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned char		tymem;
		} TYMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		crmem;
		} CRMEMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		wxin [TINOGRPELEMS];
		} WXINIMG;

typedef	struct {
	unsigned short	      	was_requested;
	unsigned short		data_valid;
	unsigned short		wyout [TINOGRPELEMS];
		} WYOUTIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		status [TINOGRPELEMS];
		} STATUSIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		lpstat [TINOGRPELEMS];
		} LPSTATIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		lpcntl [TINOGRPELEMS];
		} LPCNTLIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		lpmd [TINOGRPELEMS];
		} LPMDIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned short		avflag [TINOGRPELEMS];
		} AVFLAGIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		acflag [TINOGRPELEMS];
		} ACFLAGIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned long		lpgain [TINOGRPELEMS];
		} LPGAINIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpreset [TINOGRPELEMS];
		} LPRESETIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lprate [TINOGRPELEMS];
		} LPRATEIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lphialrm [TINOGRPELEMS];
		} LPHIALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lploalrm [TINOGRPELEMS];
		} LPLOALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpprocvar [TINOGRPELEMS];
		} LPPROCVARIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lppvhilim [TINOGRPELEMS];
		} LPPVHILIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lppvlolim [TINOGRPELEMS];
		} LPPVLOLIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lporgdevalrm [TINOGRPELEMS];
		} LPORGDEVALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpyeldevalrm [TINOGRPELEMS];
		} LPYELDEVALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpsmprate [TINOGRPELEMS];
		} LPSMPRATEIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpremsp [TINOGRPELEMS];
		} LPREMSPIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpoutput [TINOGRPELEMS];
		} LPOUTPUTIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lperror [TINOGRPELEMS];
		} LPERRORIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lpbias [TINOGRPELEMS];
		} LPBIASIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lppvhihilim [TINOGRPELEMS];
		} LPPVHIHILIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lppvlololim [TINOGRPELEMS];
		} LPPVLOLOLIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			lprocalrm [TINOGRPELEMS];
		} LPROCALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anpvhialrm [TINOGRPELEMS];
		} ANPVHIALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anpvloalrm [TINOGRPELEMS];
		} ANPVLOALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anprocvar [TINOGRPELEMS];
		} ANPROCVARIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anpvhilim [TINOGRPELEMS];
		} ANPVHILIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anpvlolim [TINOGRPELEMS];
		} ANPVLOLIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anorgdevlim [TINOGRPELEMS];
		} ANORGDEVLIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anyeldevlim [TINOGRPELEMS];
		} ANYELDEVLIMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			ansmprate [TINOGRPELEMS];
		} ANSMPRATEIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			ansetpoint [TINOGRPELEMS];
		} ANSETPOINTIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anerror [TINOGRPELEMS];
		} ANERRORIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anpvhihialrm [TINOGRPELEMS];
		} ANPVHIHIALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anpvloloalrm [TINOGRPELEMS];
		} ANPVLOLOALRMIMG;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	long			anroc [TINOGRPELEMS];
		} ANROCIMG;

/* This structure defines the reponse to the Request primitive (3) */

typedef struct {
	unsigned short		nitp_sz;
	unsigned char		dev_adr;
        unsigned char		cmd;
	unsigned short		prim_sz;
	unsigned char		prim;
	unsigned char		status;
	unsigned short		dev_type;
	unsigned short		vmem_sz;
	unsigned short		cmem_sz;
	unsigned short		local_io_sz;
	unsigned short		global_io_sz;
	unsigned long		total_user_sz;
		} TI_DEV;

typedef	struct {
	unsigned short	was_requested;		/* 1  The scan task accessed this RTU */
	unsigned short	is_responding;		/* 2  The RTU is responding */
	unsigned short	holdflag;		/*    The RTU data is in hold */
	unsigned short  no_dataout_msgs;	/* 5  Count Output Msgs */
	unsigned short  no_datain_msgs;		/* 6  Count Data Requests */
	unsigned short  no_rxnoresp;		/* 7  Count No Responses */
	unsigned short  no_txnoresp;		/* 8  Count No Responses */
	unsigned short  no_dataerr;		/* 9  Count Message Echo Errors */
	unsigned short  no_retries;		/* 10 Count Retries */
	unsigned short  no_rxhwerrs;		/* 11 Count Comm Hardware Errors */
	unsigned short  no_txhwerrs;		/* 12 Count Comm Hardware Errors */
	unsigned short  no_crcerrs;		/* 13 Count CRC Errors */
	unsigned short  no_offlines;		/* 14 Count Off Line Transitions */
	unsigned short  cmd_reset_stat;		/* 15 Comand to reset stats*/
	unsigned short  cmd_force_offline;	/* 16 Command to Force Offline */
	unsigned short  badoutcnt;		/* 17 Count Failed output commands */
	unsigned short	scanpass;               /* Scan Pass Counter */
	unsigned short	nodatastrike;		/* Count consequtive data noresponses */
	unsigned long	scanflag;		/* bit mask to activate scan by mem type */
	unsigned short	dev_type;               /* Tiway device type */
	unsigned short	dev_type_idx;           /* Tiway device type index */
	unsigned short	imem_sz;
	unsigned short	vmem_sz;
	unsigned short	cmem_sz;
	unsigned short	local_io_sz;
	unsigned short	global_io_sz;
	unsigned long	total_user_sz;		/* excludes IO mem */
	VMEMIMG		*vmem;
	CMEMIMG		*cmem;
	XMEMIMG		*xmem;
	TXMEMIMG	*txmem;
	YMEMIMG		*ymem;
	TYMEMIMG	*tymem;
	CRMEMIMG	*crmem;
	WXINIMG		*wxin;
	WYOUTIMG	*wyout;
	STATUSIMG	*status;
	LPSTATIMG	*lpstat;
	LPCNTLIMG	*lpcntl;
	LPMDIMG		*lpmd;
	AVFLAGIMG	*avflag;
	ACFLAGIMG	*acflag;
	LPGAINIMG	*lpgain;
	LPRESETIMG	*lpreset;
	LPRATEIMG	*lprate;
	LPHIALRMIMG	*lphialrm;
	LPLOALRMIMG	*lploalrm;
	LPPROCVARIMG	*lpprocvar;
	LPPVHILIMIMG	*lppvhilim;
	LPPVLOLIMIMG	*lppvlolim;
	LPORGDEVALRMIMG	*lporgdevalrm;
	LPYELDEVALRMIMG	*lpyeldevalrm;
	LPSMPRATEIMG	*lpsmprate;
	LPREMSPIMG	*lpremsp;
	LPOUTPUTIMG	*lpoutput;
	LPERRORIMG	*lperror;
	LPBIASIMG	*lpbias;
	LPPVHIHILIMIMG	*lppvhihilim;
	LPPVLOLOLIMIMG	*lppvlololim;
	LPROCALRMIMG	*lprocalrm;
	ANPVHIALRMIMG	*anpvhialrm;
	ANPVLOALRMIMG	*anpvloalrm;
	ANPROCVARIMG	*anprocvar;
	ANPVHILIMIMG	*anpvhilim;
	ANPVLOLIMIMG	*anpvlolim;
	ANORGDEVLIMIMG	*anorgdevlim;
	ANYELDEVLIMIMG	*anyeldevlim;
	ANSMPRATEIMG	*ansmprate;
	ANSETPOINTIMG	*ansetpoint;
	ANERRORIMG	*anerror;
	ANPVHIHIALRMIMG	*anpvhihialrm;
	ANPVLOLOALRMIMG	*anpvloloalrm;
	ANROCIMG	*anroc;
	} TIPLC;

/* task definitions */

#define 	TI_STACK		2000   /* Stack size              */
#define		TI_PRIORITY		200    /* default proirity        */
#define		TI_TASK_OPTIONS		8      /* Floating point 	  */

