/*drvTrane.h,v 1.1.1.1 1999/08/03 14:45:09 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
**	File:	drvBcu.h							       
**	Auth:	Walter E. Palmer                                        	       
**                                                                                     
**	Edit:	06/28/94	WEP                                                    
**                                                                                     
**	This file contains the data structures used by the Trane BCU I/O driver.       
**                                                                             	       
*/

/********************/
/* Simple Constants */
/********************/
#define TRUE	1
#define FALSE	0
#define NULLP	0
#define CR	0x0d

/**************************************************************/
/* Driver, Resource Management, and Polling Related Constants */
/**************************************************************/

#define    BCULINES     20      /* number of lines of Trane BCU BACnet Protocol         */
#define    NUMBCUS      1       /* number of BCUs per line.                             */

#define	   RESNAMELEN	16	/* length of a resource name                            */
#define	   SLWWINDOW	900	/* # of ticks to allow slave to try                     */

/*#define   BRINGBACKCNT  4	** Poll loops per cycle                                 */

/*                                                                           		*/
/*	The IOC will assume that the data to be exchanged with each BCU will           	*/
/*	come from a fixed number of binary outputs, binary inputs, analog inputs       	*/
/*	or analog outputs. It will read these in groups to minimize the amount of       */
/*	communications overhead. It will also try to build as large a fetch            	*/
/*	message as possible & required by remembering which consecutive groups         	*/
/*	have been requested by the SCAN task.                                          	*/


#define AICARD          0			/* Card number for analog inputs 	*/
#define AOCARD		1			/* Card number for analog outputs	*/
#define BICARD		3			/* Card number for binary inputs 	*/
#define BOCARD    	4			/* Card number for binary outputs	*/
#define PSEUDOCARD     33			/* Card number for pseudo i/o 		*/


/* note: redundant definitions below allow for easier conversion from */
/*       individually referenced data values to grouped data values,  */
/*       should the need occur.                                       */

/* Number of items */
  #define NUMBCUBOS    100 /* 50		** max number of binary outputs         */
  #define NUMBCUBIS    100 /*100		** max number of binary inputs          */
  #define NUMBCUAOS    100 /* 50		** max number of analog outputs         */
  #define NUMBCUAIS    100 /*100		** max number of analog inputs		*/
  #define REALBLOCK	4		        /* size of analog real values           */

/* Number of items */
  #define MINBONO       1                       /* min ID number of individual binary outputs 	*/
  #define MINBINO       1                       /* max ID number of individual binary inputs 	*/
  #define MINAINO       1                       /* max ID number of individual analog inputs	*/
  #define MINAONO       1                       /* max ID number of individual analog outputs	*/

  #define MAXBONO       NUMBCUBOS		/* max ID number of individual binary outputs 	*/
  #define MAXBINO       NUMBCUBIS		/* max ID number of individual binary inputs 	*/
  #define MAXAINO       NUMBCUAIS               /* max ID number of individual analog inputs	*/
  #define MAXAONO       NUMBCUAOS               /* max ID number of individual analog outputs	*/




/************************************************/
/*	Conversion types for analog inputs	*/
/************************************************/

#define		AICNVINTEGER	10
#define		AICNVDOUBLEINT	11
#define		AICNVFLOAT	12
#define		AICNVDOUBLEFLT	13
#define		AICNVASCIIFLT	14
#define		AICNVSTATUS	15
#define		AICNVTIME	16
#define		AICNVASCII_2	17
#define		AICNVASCII_4	18
#define		AICNVASCII_12	19
#define		AICNVASCII_20	20
#define		MAXCONV		20

/* The following must match the definition in recChoice.ascii */
#define NOCONV 0
#define LINEAR 1
#define ASCFLT 2

/******************************************/
/*	Communications I/O Constants      */
/******************************************/

/* semaphore array index assignment */
#define		RXSEM		0
#define		TXSEM		1

/* communication buffer size definitions */
#define		MAXRXMSG	510
#define		MAXTXMSG        510
#define		MAXRXRAWMSG	600

/* fixed header offsets */
#define         NPDUOFFSET       0
#define         APDUOFFSET       9

/* general status flags */
#define		RXGOOD			1
#define		RXBAD			2
#define		TXGOOD			1
#define		TXBAD			2

#define		IO_TIMEOUT		3	/* Code returned by timer 		*/
#define	        XFERGOOD	        1
#define	        XFERBAD		        2

/* communications timing constants */
#define    T_CONNECT_REQUEST     1*60       /* (15) max time from "BACnet" to Connect Request.  */
                                            /* John Hartman: Trane implements 'immediate'       */
#define    T_CONNECT_RESPONSE    1*60       /* (15) max time from Connect Request to Response.  */
                                            /* John Hartman: Trane implements 'immediate'       */
#define    T_FRAME_ABORT         1*60       /* (2) max time between inter fram characters.      */
                                            /* John Hartman: Trane implements 'immediate'       */
#define    T_HEARTBEAT          15*60       /* (15) rate at which we output heartbeats.         */
                                            /*                                                  */
#define    T_INACTIVITY         30*60       /* (60) max time we can go not detecting RX before  */
                                            /*      we declare the connection DISCONNECTED.     */
                                            /* John Hartman: Trane requires 100-200 msec to     */
                                            /*               process each object-property in a  */
                                            /*               data request. if we limit each     */
                                            /*               request to 10, then we have a max  */
                                            /*               process time of approx 2 seconds.  */
                                            /*               Use an inactivity time of 5 sec.   */ 
#define    T_RESPONSE            1*60       /* (5) max time to wait for a data response frame.  */
                                            /* John Hartman: Trane implements 'immediate'       */
#define    T_APDU_TIMEOUT        5*60       /* (20) max response time for a complex_ack.        */
                                            /* John Hartman: Trane requires 100-200 msec to     */
                                            /*               process each object-property in a  */
                                            /*               data request. if we limit each     */
                                            /*               request to 10, then we have a max  */
                                            /*               process time of approx 2 seconds.  */
                                            /*               Use an inactivity time of 5 sec.   */
#define    T_TRANSMIT_BLOCKED    1*60       /* (not stated) time to wait for first HeartbeatXON */ 
                                            /* John Hartman: Trane implements 'immediate'       */

#define    N_RETRIES             3 /* BACnet standard. */
#define    N_CONNECT_RETRIES     2 /* Trane Driver local definition. */
#define    N_DISCONNECT_RETRIES  1 /* Trane Driver local definition. */

#define    TXTIMEOUT             1*60 /* 1 second timeout. (at 9600 and 500 char, .5 sec is max) */

#define    ACTIVITYTIME         10*60 /* time that initslave waits to see if partner is master.  */
                                      /* in check_line_activity, used to age out old status.     */
#define    SLAVETRYTIME          6*60 /* time SLV tries to go online when partner is SLVWANT.    */
#define    SLAVEWAITTIME        14*60 /* time SLV waits after SLAVETRYTIME when pstate=SLVWANT.  */

#define    WATCHDOGTIMEOUT      10*60 /* watchdog monitor timeout.  */


/*****************************************/
/*	Communications I/O Structures    */
/*****************************************/

typedef	struct {
	unsigned char	        rxstate;
        unsigned char           DLE_Mask;
        unsigned char           FrameType;
        unsigned short          Index;
        unsigned char           HeaderCRC;
        unsigned short          DataLength;
        unsigned short          DataCRC;
       	unsigned char		InputBuffer[MAXRXMSG];
        unsigned char           ReceivedValidFrame;
        unsigned char           ReceivedInvalidFrame;
        unsigned char           ReceiveError;
	SEMAPHORE		*semid;
	unsigned short		*status;

        unsigned short          SilenceTimer;
		} 	RX_FRAME;

    /* RX Frame State Definitions */
    #define    RXIGNORE                    0
    #define    RXIDLE                      1        /* waiting for beginning of a frame */
    #define    RXPREAMBLE                  2        /* beginning of new frame preamble */
    #define    RXHEADER                    3        /* preamble was good. ready for header */
    #define    RXHEADER_CRC                4        /* header done. ready for it header CRC */
    #define    RXDATA                      5        /* ready for datalink data */
    #define    RXDATA_CRC                  6        /* datalink data CRC */
    #define    RXENDFRAME                  7        /* end of old frame. */
                                                    /* must set to RXIDLE to start new frame. */

typedef	struct {
	unsigned char		txstate;
        unsigned char           DLE_Mask;
        unsigned char           FrameType;
        unsigned short          Index;
        unsigned char           HeaderCRC;
        unsigned short          DataLength;
        unsigned short          DataCRC;
       	unsigned char		OutputBuffer[MAXTXMSG];
        unsigned char           TransmittedValidFrame;
        unsigned char           TransmittedInvalidFrame;
        unsigned char           TransmitError;
	SEMAPHORE		*semid;
	unsigned short		*status;
		} 	TX_FRAME;


    /* TX Frame State Definitions */
    #define    TXIGNORE                    0
    #define    TXSTART                     1        /* starting transmission of a frame */
    #define    TXPREAMBLE                  2        /* sending preamble */
    #define    TXFRAME                     3        /* sending frame type */
    #define    TXLENGTH                    4        /* sending data length */
    #define    TXHEADER_CRC                5        /* sending calculated header CRC */
    #define    TXDATA                      6        /* sending data */
    #define    TXDATA_CRC                  7        /* sending data CRC */
    #define    TXINITCONNECT               8        /* special: starting Initalize Connect Seq */
    #define    TXCONNECT                   9        /* special: sending Initalize Connect Seq  */
    #define    TXUNKNOWN                0xFF        /* unknown state - debug only */


/* Circular Raw RX Data Buffer */

typedef	struct {
        unsigned char           rxchar;
        unsigned char           rxstatus;
		} 	RXRAW;

#define         RXRAWQSIZE      2000

typedef struct {
         unsigned short         rxinix;
         unsigned short         rxoutix;
         RXRAW                  rxque[RXRAWQSIZE];
               }      RXRAWQUE;

/****************************************************************/
/* Output Queue Management Structures for BCU BACnet Protocol   */
/****************************************************************/

#define         AOUTTYPE        1
#define         BOUTTYPE        2
#define         OUTQSIZE        128

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  aosignal;
	short           aovalue;
	double		dblaovalue;
	unsigned short	conversion;
		} AOUT, *AOUTPTR;

typedef struct {
	unsigned short  link;
	unsigned short  adapter;
	unsigned short  card;
	unsigned short  bosignal;
	long            bomask;
	long            bovalue;
		} BOUT, *BOUTPTR;

typedef struct {
		unsigned short  otype;
		union   {
			AOUT    aonode;
			BOUT    bonode;
			}out;
	       } OUTQENT;

typedef struct {
		unsigned short	ooutix;
		unsigned short	oinix;
		OUTQENT         outque[OUTQSIZE];
		} OUTPUTQUE;

/*
**	Internal IOC Structure for Trane BCU
**
*/
typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned char		last_read;
		}	BINARYOUTPUTS;

typedef	struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned char		last_read;
		}	BINARYINPUTS;

typedef struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned char		aovals[REALBLOCK];
		}	ANALOGOUTPUTS;

typedef struct {
	unsigned short		was_requested;
	unsigned short		data_valid;
	unsigned char		aivals[REALBLOCK];
		}	ANALOGINPUTS;



/*********************************************/
/* BACnet State Machine Structure Defintions */

/* Transaction SM structure */
typedef struct
{
    unsigned char  state;                   /* holds current value of transaction state machine  */
    unsigned char  status;
    unsigned char  confirmation;
    unsigned long  starttime;
    unsigned char  source_address[3];
    unsigned char  destination_address[3];
    unsigned short DataLength;
    unsigned short Index;
    unsigned char  OutputValid;
    unsigned char  OutputBuffer[MAXTXMSG];
    unsigned char  invokeID;
  } TSM;
 
    /* Transaction State Definitions */
    #define    TSM_IDLE                    0
    #define    TSM_SEGMENTING_REQUEST      1
    #define    TSM_AWAIT_CONFIRMATION      2
    #define    TSM_SEGMENTED_CONF_TO_AE    3
    #define    TSM_SEGMENTED_CONF_TO_AP    4
    #define    TSM_SEGMENTED_REQUEST       5
    #define    TSM_AWAIT_RESPONSE          6
    #define    TSM_SEGMENTED_RESPONSE      7

    /* Transaction 'confirmation' Definitions */
    #define    TSM_NOTCONFIRMED            0
    #define    TSM_CONFIRMED               1

    /* Transaction Status Definitions */
    #define    TSM_TIMEOUT            0
    #define    TSM_SIMPLE_ACK         1
    #define    TSM_COMPLEX_ACK        2
    #define    TSM_ERROR              3
    #define    TSM_ABORT              4
    #define    TSM_REJECT             5


/* Application Input structure */
typedef struct
{
    unsigned char  InputValid;
    unsigned short InputLength;
    unsigned char  InputBuffer[MAXRXMSG];
    unsigned char  InputDNET[2];
    unsigned char  InputDLEN[1];
    unsigned char  InputDMAC[2];
    unsigned char  InputSNET[2];
    unsigned char  InputSLEN[1];
    unsigned char  InputSMAC[2];
    unsigned char  InputHopCount;
    unsigned char  MessageType;
} APPINPUT;

/* APDU Complex ACK response parsing: state values */

#define LOOKING_FOR_OBJECT_ID                  0
#define BUILDING_OBJECT_ID                     1
#define LOOKING_FOR_OPENING_RESULTS            2
    #define LOOKING_FOR_PROPERTY_ID            3
    #define GETTING_PROPERTY_ID                4
    #define LOOKING_FOR_OPENING_VALUE          5 /* looking for opening property value or property access error */ 
       #define LOOKING_FOR_DATATYPE            6
       #define GETTING_ENUMERATEDVALUE         7
       #define GETTING_REALVALUE               8
    #define LOOKING_FOR_CLOSING_VALUE          9
       #define LOOKING_FOR_ERROR_CLASS_TAG    10
       #define GETTING_ERROR_CLASS_VALUE      11
       #define LOOKING_FOR_ERROR_CODE_TAG     12
       #define GETTING_ERROR_CODE_VALUE       13
    #define LOOKING_FOR_CLOSING_ACCESS_ERROR  14
    #define LOOKING_FOR_TRANE_BUG             15
        #define POST_CLOSING_VALUE            16  /* either looking for property id, or looking for closing results */
#define LOOKING_FOR_CLOSING_RESULTS           17
#define POST_CLOSING_RESULTS                  18  /* either another object to input or at the end of the list */

/* Network SM structure */
typedef struct
{
    unsigned char  state;                   /* holds current value of the state machine.         */
    unsigned char  version;
    unsigned char  OutputValid;
    unsigned short OutputLength;
    unsigned char  OutputBuffer[MAXTXMSG];
    unsigned char  InputValid;
    unsigned short InputLength;
    unsigned char  InputBuffer[MAXRXMSG];
    unsigned char  NetMessage;
    unsigned char  InputBroadcast;
    unsigned char  InputDNET[2];
    unsigned char  InputDLEN[1];
    unsigned char  InputDMAC[2];
    unsigned char  InputSNET[2];
    unsigned char  InputSLEN[1];
    unsigned char  InputSMAC[2];
    unsigned char  InputHopCount;
    unsigned char  MessageType;
} NSM;
 
#define BACnetNetworkVersion 1


/* Connection SM structure */
typedef struct
{
    unsigned char  state;                   /* holds current value of the state machine.         */
    unsigned char  retry;
    unsigned long  responsetimer;
} CSM;
    /* Connection State Definitions */
    #define    DISCONNECTED                0
    #define    OUTBOUND                    1
    #define    INBOUND                     2
    #define    CONNECTED                   3
    #define    DISCONNECTING               4


/* Input SM structure */
typedef struct
{
    unsigned char  state;                   /* holds current value of the state machine.         */
    unsigned char  RxSequenceNumber;
/*    unsigned char  LastSequenceNumber;      ** stores the sequence number of the                 */
/*                                            ** last segment received in order.                   */
/*    unsigned char  InitialSequenceNumber;   ** stores the sequence number of the first segment   */
/*                                            ** of a sequence of segment which fill a window.     */
    unsigned int   LastFrameTime;           /* used to determine inactivity time */
    unsigned char  ReceivedInvalidFrame;
    unsigned char  ReceivedValidFrame;

    unsigned char  ReceptionBlocked;
    unsigned char  FullBuffers;

    unsigned char  Ack0Received;
    unsigned char  Ack1Received;
    unsigned char  Reject0Received;
    unsigned char  Reject1Received;
} ISM;
    /* Input State Definitions */
    #define    RECEIVE_IDLE                0
    #define    RECEIVE_NOT_READY           1
    #define    RECEIVE_READY               2
    #define    DATA_REQUEST                3
    #define    DATA_RESPONSE               4
    #define    DATA_REJECT                 5


/* Output SM structure */
typedef struct
{
    unsigned char  state;              /* holds current value of the state machine            */
    unsigned char  TransmissionBlocked;
    unsigned char  TxSequenceNumber;
    unsigned char  RetryCount;
    unsigned long  LastFrameTime;
    unsigned long  starttime;          /* used for timing response time                       */
} OSM;
    /* Output State Definitions */
    #define    TRANSMIT_IDLE               0
    #define    TRANSMIT_BLOCKED            1
    #define    TRANSMIT_READY              2
    #define    TRANSMIT_PENDING            3



typedef	struct	{
    unsigned short was_requested;      /* 0  The scan task has accesses this RTU              */
    unsigned short is_responding;      /* 1  The RTU is responding                            */
    unsigned short holdflag;           /*    The RTU data is in hold                          */
    unsigned short no_dataout_msgs;    /* 5  Count Output Msgs                                */
    unsigned short no_datain_msgs;     /* 6  Count Data Requests                              */
    unsigned short no_rxnoresp;	       /* 7  Count No Responses                               */
    unsigned short no_txnoresp;        /* 8  Count No Responses                               */
    unsigned short no_dataerr;         /* 9  Count Message Echo Errors                        */
    unsigned short no_retries;         /* 10 Count Retries                                    */
    unsigned short no_rxhwerrs;        /* 11 Count Comm Hardware Errors                       */
    unsigned short no_txhwerrs;        /* 12 Count Comm Hardware Errors                       */
    unsigned short no_crcerrs;         /* 13 Count CRC Errors                                 */
    unsigned short no_offlines;        /* 14 Count Off Line Transitions                       */
    unsigned short cmd_reset_stat;     /* 15 Comand to reset stats                            */
    unsigned short cmd_force_offline;  /* 16 Command to Force Offline                         */
    unsigned short badoutcnt;          /* 17 Count Failed output commands                     */
    TSM            serverTSM;
    TSM            clientTSM;
    NSM            networkSM;
    CSM            connectionSM;
    ISM            inputSM;
    OSM            outputSM;
    APPINPUT       application_input;
    BINARYOUTPUTS  binaryoutputs[NUMBCUBOS];
    BINARYINPUTS   binaryinputs[NUMBCUBIS];
    ANALOGINPUTS   analoginputs[NUMBCUAIS];
    ANALOGOUTPUTS  analogoutputs[NUMBCUAOS];
}       BCU;


/***********************************/
/* BCU BACnet function codes       */
/***********************************/

/* Data Link/Physical Layers: Point-To-Point MAC Frame Types */

#define HeartbeatXOFF       0       /* */
#define HeartbeatXON        1       /* */
#define DataRequest0        2       /* */
#define DataRequest1        3       /* */
#define DataResponse0XOFF   4       /* */
#define DataResponse1XOFF   5       /* */
#define DataResponse0XON    6       /* */
#define DataResponse1XON    7       /* */
/* these definitions may one day be changed. see Trane guy - John Hartman. */
#define DataReject0XOFF     80      /* */
#define DataReject0XON      8       /* */
#define DataReject1XOFF     90      /* */
#define DataReject1XON      9       /* */
#define ConnectRequest     10       /* */
#define ConnectResponse    11       /* */
#define DisconnectRequest  12       /* */
#define DisconnectResponse 13       /* */
#define TestRequest        14       /* */
#define TestResponse       15       /* */
#define BACnetString      128       /* used to cause output of BACnet string.  */
                                    /*  this is NOT a true frame type.         */


/* Network Layer: Message Types */

#define Who_Is_Router_To_Network                0       /* */
#define I_Am_Router_To_Network                  1       /* */
#define I_Could_Be_Router_To_Network            2       /* */
#define Reject_Message_To_Network               3       /* */
#define Router_Busy_To_Network                  4       /* */
#define Router_Available_To_Network             5       /* */
#define Initialize_Routing_Table                6       /* */
#define Initialize_Routing_Table_Ack            7       /* */
#define Establishing_Connection_To_Network      8       /* */
#define Disconnect_Connection_To_Network        9       /* */

/* define Control Octet bits */
#define Non_Network_Message                     0x00
#define Network_Message                         0x80
#define DefaultDestinationFormat                0x40
#define DefaultSourceFormat                     0x10
#define DefaultPriority                         0x00 /* 0x03 */


/*********************************************************************/
/* ADPU: Fixed Part:  Encoding for Service Request Primitives (SRPs) */
/*********************************************************************/

#define ConfirmedRequest_PDU     0 << 4  /* conveys info for confirmed SRPs                */
#define UnconfirmedRequest_PDU   1 << 4  /* conveys info for unconfirmed SRPs              */
#define SimpleACK_PDU            2 << 4  /* conveys simple ACK for SRPs                    */
#define ComplexACK_PDU           3 << 4  /* conveys simple ACK and other info for SRPs     */ 
#define SegmentACK_PDU           4 << 4  /* requests continuation of segmented message     */
#define Error_PDU                5 << 4  /* conveys SPR failure and the failure reason     */
#define Reject_PDU               6 << 4  /* conveys SRP rejection and the rejection reason */
#define Abort_PDU                7 << 4  /* conveys SRP abort and the abort reason         */


#define SegmentedMessage                 1 << 3
#define NotSegmentedMessage              0 & SegmentedMessage
#define MoreFollows                      1 << 2
#define NoMoreFollows                    0 & MoreFollows
#define SegmentedResponseAccepted        1 << 1
#define SegmentedResponseNotAccepted     0 & SegmentedResponseAccepted

#define MaxADPUResponseSize              3        /* 480 octets */



/************************************************************************/
/* ADPU: Variable Part:  Encoding for Service Request Primitives (SRPs) */
/************************************************************************/

/****************************************************/
/* Confirmed Request (0): 'service-request' Choices */

/* Alarm and Event Services */
#define AcknowledgeAlarm_Request             0
#define ConfirmedCOVNotification_Request     1
#define ConfirmedEventNotification_Request   2
#define GetAlarmSummary_Request              3
#define GetEnrollmentSummary_Request         4
#define subscribeCOV_Request                 5

/* Object Access Services */
#define ReadProperty_Request                12
#define ReadPropertyMultiple_Request        14
#define WriteProperty_Request               15
#define WritePropertyMultiple_Request       16

/* Object Access Services */
#define ReadPropertyMultiple_ACK            14
#define WritePropertyMultiple_ACK           16

/* Tag Class (Context vs Application) */

#define ContextClassMask                    8
#define LengthValueTypeMask                 7
#define TagNumberShift                      4

/* Details: Context Tag Numbers for ReadPropertyMultiple_Request*/

#define RAS_objectIdentifier                0
#define RAS_listOfPropertyReferences        1
#define BACnetPR_propertyIdentifier         0
#define BACnetPI_present_value             85

/* Details: Context Tag Numbers for ReadPropertyMultiple_ACK*/

#define RAR_objectIdentifier                0
#define RAR_listOfResults                   1
/*#define RAR_listOfProperties                1 */
#define RAR_PropertyIdentifier              2
#define RAR_propertyValue                   4
#define RAR_propertyAccessError             5

#define ErrorClass_Object                   1
#define ErrorCode_UnknownObject            31


/* Details: Context Tag Numbers for WritePropertyMultiple_Request*/

#define WAS_objectIdentifier                0
#define WAS_listOfProperties                1
#define BACnetPV_propertyIdentifier         0
#define BACnetPV_value                      2
#define BACnetPV_priority                   3

#define DefaultWritePriority                1

/* BACnet Tags */

#define ApplicationTag                   0x00
#define ContextTag                       0x08
#define OpeningTag                       0x06
#define ClosingTag                       0x07

/* value: Application Primitive Types */

#define ApplicationNull                  0    /* L/V field == 0 */
#define ApplicationBoolean               1
#define ApplicationUnSignedInt           2
#define ApplicationSignedInt             3
#define ApplicationReal                  4
#define ApplicationDouble                5
#define ApplicationOctetString           6
#define ApplicationCharacterString       7
#define ApplicationBitString             8
#define ApplicationEnumerated            9
#define ApplicationDate                 10
#define ApplicationTime                 11
#define ApplicationObjectID             12
#define ApplicationASHRAE1              13
#define ApplicationASHRAE2              14
#define ApplicationASHRAE3              15


/* standard object types: middle octet of object identifiers */
#define BAC_AnalogInput                 0
#define BAC_AnalogOutput                1
#define BAC_BinaryInput                 3
#define BAC_BinaryOutput                4

/******************************************************/
/* Unconfirmed Request (1): 'service_request' Choices */

#define UnconfirmedEventNotification_Request 0
#define Who_Is_Request                       1
#define I_Am_Request                         2
#define TimeSynchronization_Request          3
#define UnconfirmedTextMessage_Request       4


/*************************************/
/* Abort : Definitions               */

#define Abort_Client_True                    0
#define Abort_Server_True                    1
#define Abort_Server_Bit                     1

/* Abort Reasons */
#define Other_Abort_Reason                   0
#define Buffer_Overflow                      1
#define Invalid_APDU_In_This_State           2
#define Preempted_By_Higher_Priority_Task    3
#define Segmentation_Not_Supported           4



/********************/
/* task definitions */
/********************/

#define BCU_STACK		8000   /* Stack size              */
#define	BCU_PRIORITY		152    /* default priority        */
#define	BCU_TASK_OPTIONS	8      /* Floating point          */

#define BCUTIMER_STACK		8000   /* Stack size              */
#define	BCUTIMER_PRIORITY	152    /* default priority        */
#define	BCUTIMER_TASK_OPTIONS	8      /* Floating point          */


/**********************************************************************/
/* Mutual_Exclusion Control for diagnostic/data radio channel sharing */
/**********************************************************************/

#define		MUXTIMEOUT	2*60	/* 16.67 ms ticks (60Hz): 2 seconds 	*/


/* Trace Bit Definitions */

#define ALLTRAFFICTRACE      0x0001
#define DATALINKTRACE        0x0002
#define NETWORKTRACE         0x0004
#define APPLICATIONTRACE     0x0008
#define CONFIGURATIONTRACE   0x0010        /* trace configuration errors in driver calls */
#define RUNTIMEERRORTRACE    0x0020        /* trace run time errors in driver calls */
#define CALLREJECTTRACE      0x0040        /* trace non-error based driver call rejections */
/* skip bit definitions */
#define SKIPAO          0x0001
#define SKIPAI          0x0002
#define SKIPBO          0x0004
#define SKIPBI          0x0008
#define SKIPOUT         0x0010


/*	End of File:	smod_driver.h */
