/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#include	<stdio.h>
#include	<sys/time.h>
#include	<string.h>
#include	<memory.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<d4all.h>
#include	<alarm.h>
#include	<alarmString.h>
#include	<cadef.h>	/* channel access */
#include	<db_access.h>
#include	<dbDefs.h>
#include	<shareMem.h>	/* src/alarm */
#include	<AlrmGrpRpt.h>	/* all codebase fields */

extern PvData	*FindPV ();
extern char	**GetPVNames ();
extern int 	InitAlarmAccess ();

static void close_dbase_files ();
static int  compare ();
static void connect_handler ();
static void data_handler ();
static void free_mem ();
static int get_rest_of_fields ();
static void print_header ();
static void print_report ();
static void print_usage ();
static void remove_spaces ();

static int	cacalls = 0;
static	char	outfile[128];
static char	RequestedAlrmGrp[18];


/********************************************************************************/

main (argc, argv)
    int		argc;
    char	*argv[];
{
    T4TAG		*nametag;
    F4FIELD		*name; 
    F4FIELD		*type; 
    F4FIELD		*desc; 
    F4FIELD		*onscan; 
    F4FIELD		*onalarm; 
    char		RecField[64];
    char		pvmappath[128];
    char		pvname[PVNAME_SZ + 1];
    int			status;
    int			numpoints = 0;
    float		tot_time = 0.0;
    chid		pCh;
    struct AlrmgrpPoint	*NewPoint;
    struct AlrmgrpPoint	*LastPoint;
    char		**PvNameList, **p;

    if (argc != 3)
    {
	print_usage();
    }
    strncpy (RequestedAlrmGrp, argv[1], sizeof (RequestedAlrmGrp));
    strncpy (outfile, argv[2], sizeof (outfile));

    if (InitAlarmAccess() == 0)
    {
	fprintf (stderr, 
		"AlrmgrpRpt: exiting since alarm mgr not running\n");
	exit (1);
    }

    while (ca_task_initialize () != ECA_NORMAL)
	sleep (1);


    d4init (&cb);
    cb.go_error = 0;
    cb.open_error = 0;
    cb.create_error = 0;
    cb.skip_error = 0;
    cb.field_name_error = 0;

    sprintf (pvmappath, "%spvmap", DEFAULT_PATH);
    if ((pvmap = d4open (&cb, pvmappath)) == 0)
    {
	fprintf (stderr, "AlrmGrpRpt: failed to open %s\n", pvmappath);
	exit (1);
    }
    nametag = d4tag (pvmap, "NAME");
    d4tag_select (pvmap, nametag);
    name = d4field (pvmap, "NAME");
    type = d4field (pvmap, "TYPE");
    desc = d4field (pvmap, "DESC");
    onscan = d4field (pvmap, "ONSCAN");
    onalarm = d4field (pvmap, "ONALARM");

    if (PvNameList = p = GetPVNames (RequestedAlrmGrp))
    {
	while (*p)
	{
	    numpoints++;
	    p++;
	}
	p = PvNameList;
	qsort (p, numpoints, sizeof (char *), compare);
	d4top (pvmap);
	while (*p)
	{
	    sprintf (pvname, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, *p);
	    if ((d4seek (pvmap, pvname)) == 0)
	    {
		if ( (NewPoint = 
		   (struct AlrmgrpPoint *)calloc (1, sizeof (struct AlrmgrpPoint)))
		   == NULL)
		{
		    fprintf (stderr, "AlrmGrpRpt: calloc failed for %s28.28\n", 
				      RecField);
		    close_dbase_files ();
		    free_mem();
		    exit (1);
		}
		NewPoint -> NextPoint = (struct AlrmgrpPoint *)0;
		strncpy (NewPoint -> name, pvname, sizeof (NewPoint -> name));
		strncpy (RecField, f4str(type), sizeof (RecField));
		remove_spaces (RecField);
		strncpy (NewPoint -> type, RecField, sizeof (NewPoint -> type));
		strncpy (NewPoint -> desc, f4str(desc), sizeof (NewPoint -> desc));
		if (f4true (onalarm))
		    NewPoint -> onalarm = 'T';
		else
		    NewPoint -> onalarm = 'F';
		if (f4true (onscan))
		    NewPoint -> onscan = 'T';
		else
		    NewPoint -> onscan = 'F';
		if (get_rest_of_fields (NewPoint))
		{
		    close_dbase_files ();
		    free_mem ();
		    exit (1);
		}
		strncpy (RecField, f4str(name), sizeof (RecField));
		remove_spaces (RecField);
		if ( (status = ca_build_and_connect (RecField,
						     AGNT_NOCONN,
						     0,
						     &pCh,
						     0,
						     connect_handler,
						     (void*)NewPoint))
						     == ECA_NORMAL)
		{
		    NewPoint -> pCh = pCh;
		    cacalls++;
		    if (strncmp (NewPoint -> type, "bi", 2) == 0)
		    {
			strcat (RecField, ".TCNT");
			if ( (status = ca_build_and_connect (RecField,
							     AGNT_NOCONN,
							     0,
							     &pCh,
							     0,
							     connect_handler,
							     (void *)NewPoint))
							     == ECA_NORMAL)
			{
			    NewPoint -> tcntCh = pCh;
			    cacalls++;
			}
		    }
		}
		if (FirstPoint == (struct AlrmgrpPoint *)0)
		    FirstPoint = NewPoint;
		else
		    LastPoint -> NextPoint = NewPoint;
		LastPoint = NewPoint;
	    }
	    else
	    {
		fprintf (stderr, "AlrmGrpRpt, %s is not in pvmappath\n",pvname);
	    }
	    p++;
	}
    }

    close_dbase_files ();
    while(cacalls > 0)
    {
	ca_pend_event (0.2);
	tot_time += 0.2;
	if (tot_time > MAX_TIME_TO_WAIT)
	    break;
    }
    ca_flush_io ();
    print_report ();
    free_mem ();
    ca_task_exit ();

    exit (0);
}


/********************************************************************************/

static int compare (e1, e2)
    void	*e1;
    void	*e2;
{
    char	*el1 = *((char **)e1);
    char	*el2 = *((char **)e2);

    return (strcmp (el1, el2));
}


/********************************************************************************/

static void connect_handler (connArgs)
    struct connection_handler_args    connArgs;
{
    long		status;
    long		RequestType;
    char		*p;
    chid		pCh;
    struct AlrmgrpPoint	*pPt;

    pCh = connArgs.chid;
    pPt = (struct AlrmgrpPoint *)ca_puser (pCh);

    RequestType = DBR_STS_STRING;
    if (connArgs.op == CA_OP_CONN_UP)
    {
	if ((strncmp (pPt -> type, "bi", 2)) == 0)
	{
	    if ((p = strchr (ca_name(pCh), '.')) != NULL)
	    {
		p++;
		if ((strncmp (p, "TCNT", 4)) == 0)
		    RequestType = DBR_FLOAT;
	    }
	}
	ca_get_callback (RequestType, pCh, data_handler, (void*)pPt);
    }

    return;

}


/********************************************************************************/

static void close_dbase_files ()
{
    d4close (pvmap);
    if (accopen)
	d4close (accum);
    if (aiopen)
	d4close (ai);
    if (aoopen)
	d4close (ao);
    if (biopen)
	d4close (bi);
    if (boopen)
	d4close (bo);
    if (calcopen)
	d4close (calc);
    if (dmopen)
	d4close (datamove);
    if (dtopen)
	d4close (deadtime);
    if (fanoutopen)
	d4close (fanout);
    if (llopen)
	d4close (leadlag);
    if (mbbiopen)
	d4close (mbbi);
    if (mbboopen)
	d4close (mbbo);
    if (pidopen)
	d4close (pid);
    if (pipeopen)
	d4close (pipeData);
    if (potopen)
	d4close (pot);
    if (ptoopen)
	d4close (pto);
    if (pumpopen)
	d4close (pump);
    if (rampopen)
	d4close (ramp);
    if (selopen)
	d4close (sel);
    if (sigopen)
	d4close (sigchar);
    if (stateopen)
	d4close (state);
    if (strinopen)
	d4close (strngin);
    if (stroutopen)
	d4close (strngout);
    if (tank4open)
	d4close (tank4);
    if (tdopen)
	d4close (td);
/*    if (udbopen)
	d4close (udb); */
    if (valveopen)
	d4close (valve);
    if (wfopen)
	d4close (waveform);
    return;
}


/********************************************************************************/

static void data_handler (eventArgs)
    struct event_handler_args	eventArgs;
{
    int			got_it = 0;
    chid		pCh = eventArgs.chid;
    char		*p;
    union db_access_val	*pData;
    struct AlrmgrpPoint	*pPoint;

    pData = (union db_access_val *)eventArgs.dbr;
    pPoint = (struct AlrmgrpPoint *)eventArgs.usr;

    if ((strncmp (pPoint -> type, "bi", 2)) == 0)
    {
	if (p = strchr (ca_name(pCh), '.'))
	{
	    p++;
	    if ((strncmp (p, "TCNT", 4)) == 0)
	    {
		sprintf (pPoint -> tcnt, "%-15.0f", pData -> fltval);
		pPoint -> got_tcnt = 1;
		got_it = 1;
		ca_clear_channel (pCh);
		cacalls--;
	    }
	}
    }
    if (!got_it)
    {
	if (pData -> sstrval.status)
	    strncpy (pPoint -> stat, alarmStatusString[pData -> sstrval.status],
							sizeof (pPoint -> stat));
	strncpy (pPoint -> value, pData -> sstrval.value, sizeof (pPoint -> value));
	ca_clear_channel (pCh);
	pPoint -> got_data = 1;
	cacalls--;
    }

    return;
}


/********************************************************************************/

static void free_mem ()
{
    struct AlrmgrpPoint	*curr_pt;
    struct AlrmgrpPoint	*prev_pt;

    if (FirstPoint == NULL)
	return;

    curr_pt = FirstPoint;
    while(curr_pt)
    {
	prev_pt = curr_pt;
	curr_pt = curr_pt -> NextPoint;
	free ((void*)prev_pt);
    }

    return;
}


/********************************************************************************/

static int get_rest_of_fields (pt)
    struct AlrmgrpPoint		*pt;
{
    char	RecField[64];
    char	dbasefile[64];

    if ((strncmp (pt -> type, "accum", 5)) == 0)
    {
	if (!accopen)
	{
	    sprintf (dbasefile, "%saccum", DEFAULT_PATH);
	    if ((accum = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    acctag = d4tag (accum, "NAME");
	    d4tag_select (accum, acctag);
	    accegu = d4field (accum, AL_EGUTAG);
	    accauto = d4field (accum, AL_AUTOMATIC);
	    accopen = 1;
	}
	if ( (d4seek (accum, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(accegu), sizeof (pt -> egu_tag));
	    if (f4true (accauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (accum);
    }
    else if ((strncmp (pt -> type, "ai", 2)) == 0)
    {
	if (!aiopen)
	{
	    sprintf (dbasefile, "%sai", DEFAULT_PATH);
	    if ((ai = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    aitag = d4tag (ai, "NAME");
	    d4tag_select (ai, aitag);
	    aiegu = d4field (ai, AL_EGUTAG);
	    aiauto = d4field (ai, AL_AUTOMATIC);
	    aiopen = 1;
	}
	if ( (d4seek (ai, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(aiegu), sizeof (pt -> egu_tag));
	    if (f4true (aiauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (ai);
    }
    else if ((strncmp (pt -> type, "ao", 2)) == 0)
    {
	if (!aoopen)
	{
	    sprintf (dbasefile, "%sao", DEFAULT_PATH);
	    if ((ao = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    aotag = d4tag (ao, "NAME");
	    d4tag_select (ao, aotag);
	    aoegu = d4field (ao, AL_EGUTAG);
	    aoclosed = d4field (ao, AL_CLOSED);
	    aoopen = 1;
	}
	if ( (d4seek (ao, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(aoegu), sizeof (pt -> egu_tag));
	    if (f4true (aoclosed))
		strncpy (pt -> mode, AL_CLOSED_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_SUPER_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (ao);
    }
    else if ((strncmp (pt -> type, "bi", 2)) == 0)
    {
	if (!biopen)
	{
	    sprintf (dbasefile, "%sdi", DEFAULT_PATH);
	    if ((bi = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    bitag = d4tag (bi, "NAME");
	    d4tag_select (bi, bitag);
	    biauto = d4field (bi, AL_AUTOMATIC);
	    biopen = 1;
	}
	if ( (d4seek (bi, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (biauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (bi);
    }
    else if ((strncmp (pt -> type, "bo", 2)) == 0)
    {
	if (!boopen)
	{
	    sprintf (dbasefile, "%sdo", DEFAULT_PATH);
	    if ((bo = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    botag = d4tag (bo, "NAME");
	    d4tag_select (bo, botag);
	    boclosed = d4field (bo, AL_CLOSED);
	    boopen = 1;
	}
	if ( (d4seek (bo, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (boclosed))
		strncpy (pt -> mode, AL_CLOSED_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_SUPER_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (bo);
    }
    else if ((strncmp (pt -> type, "calc", 4)) == 0)
    {
	if (!calcopen)
	{
	    sprintf (dbasefile, "%scalc", DEFAULT_PATH);
	    if ((calc = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    calctag = d4tag (calc, "NAME");
	    d4tag_select (calc, calctag);
	    calcegu = d4field (calc, AL_EGUTAG);
	    calcauto = d4field (calc, AL_AUTOMATIC);
	    calcopen = 1;
	}
	if ( (d4seek (calc, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(calcegu), sizeof (pt -> egu_tag));
	    if (f4true (calcauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (calc);
    }
    else if ((strncmp (pt -> type, "datamove", 8)) == 0)
    {
	if (!dmopen)
	{
	    sprintf (dbasefile, "%sdatamove", DEFAULT_PATH);
	    if ((datamove = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    dmtag = d4tag (datamove, "NAME");
	    d4tag_select (datamove, dmtag);
	    dmauto = d4field (datamove, AL_AUTOMATIC);
	    dmopen = 1;
	}
	if ( (d4seek (datamove, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (dmauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (datamove);
    }
    else if ((strncmp (pt -> type, "deadtime", 8)) == 0)
    {
	if (!dtopen)
	{
	    sprintf (dbasefile, "%sdeadtime", DEFAULT_PATH);
	    if ((deadtime = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    dttag = d4tag (deadtime, "NAME");
	    d4tag_select (deadtime, dttag);
	    dtegu = d4field (deadtime, AL_EGUTAG);
	    dtauto = d4field (deadtime, AL_AUTOMATIC);
	    dtopen = 1;
	}
	if ( (d4seek (deadtime, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(dtegu), sizeof (pt -> egu_tag));
	    if (f4true (dtauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (deadtime);
    }
    else if ((strncmp (pt -> type, "fanout", 6)) == 0)
    {
	strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	strncpy (pt -> mode, AL_NOT_USED, sizeof (pt -> mode));
    }
    else if ((strncmp (pt -> type, "leadlag", 7)) == 0)
    {
	if (!llopen)
	{
	    sprintf (dbasefile, "%sleadlag", DEFAULT_PATH);
	    if ((leadlag = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    lltag = d4tag (leadlag, "NAME");
	    d4tag_select (leadlag, lltag);
	    llegu = d4field (leadlag, AL_EGUTAG);
	    llauto = d4field (leadlag, AL_AUTOMATIC);
	    llopen = 1;
	}
	if ( (d4seek (leadlag, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(llegu), sizeof (pt -> egu_tag));
	    if (f4true (llauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (leadlag);
    }
    else if ((strncmp (pt -> type, "mbbi", 4)) == 0)
    {
	if (!mbbiopen)
	{
	    sprintf (dbasefile, "%smbbi", DEFAULT_PATH);
	    if ((mbbi = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    mbbitag = d4tag (mbbi, "NAME");
	    d4tag_select (mbbi, mbbitag);
	    mbbiauto = d4field (mbbi, AL_AUTOMATIC);
	    mbbiopen = 1;
	}
	if ( (d4seek (mbbi, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (mbbiauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (mbbi);
    }
    else if ((strncmp (pt -> type, "mbbo", 4)) == 0)
    {
	if (!mbboopen)
	{
	    sprintf (dbasefile, "%smbbo", DEFAULT_PATH);
	    if ((mbbo = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    mbbotag = d4tag (mbbo, "NAME");
	    d4tag_select (mbbo, mbbotag);
	    mbboclosed = d4field (mbbo, AL_CLOSED);
	    mbboopen = 1;
	}
	if ( (d4seek (mbbo, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (mbboclosed))
		strncpy (pt -> mode, AL_CLOSED_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_SUPER_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (mbbo);
    }
    else if ((strncmp (pt -> type, "pid", 3)) == 0)
    {
	if (!pidopen)
	{
	    sprintf (dbasefile, "%spid", DEFAULT_PATH);
	    if ((pid = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    pidtag = d4tag (pid, "NAME");
	    d4tag_select (pid, pidtag);
	    pidegu = d4field (pid, AL_EGUTAG);
	    pidauto = d4field (pid, AL_AUTOMATIC);
	    pidopen = 1;
	}
	if ( (d4seek (pid, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(pidegu), sizeof (pt -> egu_tag));
	    if (f4true (pidauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (pid);
    }
    else if ((strncmp (pt -> type, "pipe", 4)) == 0)
    {
	if (!pipeopen)
	{
	    sprintf (dbasefile, "%spipe", DEFAULT_PATH);
	    if ((pipeData = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    pipetag = d4tag (pipeData, "NAME");
	    d4tag_select (pipeData, pipetag);
	    pipemode = d4field (pipeData, AL_MODE);
	    pipeopen = 1;
	}
	if ( (d4seek (pipeData, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, f4str(pipemode), sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (pipeData);
    }
    else if ((strncmp (pt -> type, "pot", 3)) == 0)
    {
	if (!potopen)
	{
	    sprintf (dbasefile, "%spot", DEFAULT_PATH);
	    if ((pot = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    pottag = d4tag (pot, "NAME");
	    d4tag_select (pot, pottag);
	    potmode = d4field (pot, AL_MODE);
	    potopen = 1;
	}
	if ( (d4seek (pot, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, f4str(potmode), sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (pot);
    }
    else if ((strncmp (pt -> type, "pto", 3)) == 0)
    {
	if (!ptoopen)
	{
	    sprintf (dbasefile, "%spto", DEFAULT_PATH);
	    if ((pto = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    ptotag = d4tag (pto, "NAME");
	    d4tag_select (pto, ptotag);
	    ptoegu = d4field (pto, AL_EGUTAG);
	    ptoclosed = d4field (pto, AL_CLOSED);
	    ptoopen = 1;
	}
	if ( (d4seek (pto, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(ptoegu), sizeof (pt -> egu_tag));
	    if (f4true (ptoclosed))
		strncpy (pt -> mode, AL_CLOSED_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_SUPER_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (pto);
    }
    else if ((strncmp (pt -> type, "pump", 4)) == 0)
    {
	if (!pumpopen)
	{
	    sprintf (dbasefile, "%spump", DEFAULT_PATH);
	    if ((pump = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    pumptag = d4tag (pump, "NAME");
	    d4tag_select (pump, pumptag);
	    pumpmode = d4field (pump, AL_MODE);
	    pumpopen = 1;
	}
	if ( (d4seek (pump, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, f4str(pumpmode), sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (pump);
    }
    else if ((strncmp (pt -> type, "ramp", 4)) == 0)
    {
	if (!rampopen)
	{
	    sprintf (dbasefile, "%sramp", DEFAULT_PATH);
	    if ((ramp = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    ramptag = d4tag (ramp, "NAME");
	    d4tag_select (ramp, ramptag);
	    rampegu = d4field (ramp, AL_EGUTAG);
	    rampauto = d4field (ramp, AL_AUTOMATIC);
	    rampopen = 1;
	}
	if ( (d4seek (ramp, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(rampegu), sizeof (pt -> egu_tag));
	    if (f4true (rampauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (ramp);
    }
    else if ((strncmp (pt -> type, "sel", 3)) == 0)
    {
	if (!selopen)
	{
	    sprintf (dbasefile, "%ssel", DEFAULT_PATH);
	    if ((sel = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    seltag = d4tag (sel, "NAME");
	    d4tag_select (sel, seltag);
	    selegu = d4field (sel, AL_EGUTAG);
	    selauto = d4field (sel, AL_AUTOMATIC);
	    selopen = 1;
	}
	if ( (d4seek (sel, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(selegu), sizeof (pt -> egu_tag));
	    if (f4true (selauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (sel);
    }
    else if ((strncmp (pt -> type, "sigchar", 6)) == 0)
    {
	if (!sigopen)
	{
	    sprintf (dbasefile, "%ssigchar", DEFAULT_PATH);
	    if ((sigchar = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    sigtag = d4tag (sigchar, "NAME");
	    d4tag_select (sigchar, sigtag);
	    sigegu = d4field (sigchar, AL_EGUTAG);
	    sigauto = d4field (sigchar, AL_AUTOMATIC);
	    sigopen = 1;
	}
	if ( (d4seek (sigchar, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(sigegu), sizeof (pt -> egu_tag));
	    if (f4true (sigauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (sigchar);
    }
    else if ((strncmp (pt -> type, "state", 5)) == 0)
    {
	strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	strncpy (pt -> mode, AL_NOT_USED, sizeof (pt -> mode));
    }
    else if ((strncmp (pt -> type, "stringin", 8)) == 0)
    {
	if (!strinopen)
	{
	    sprintf (dbasefile, "%sstrngin", DEFAULT_PATH);
	    if ((strngin = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    strintag = d4tag (strngin, "NAME");
	    d4tag_select (strngin, strintag);
	    strinauto = d4field (strngin, AL_AUTOMATIC);
	    strinopen = 1;
	}
	if ( (d4seek (strngin, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (strinauto))
		strncpy (pt -> mode, AL_AUTO_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_MANUAL_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (strngin);
    }
    else if ((strncmp (pt -> type, "stringout", 9)) == 0)
    {
	if (!stroutopen)
	{
	    sprintf (dbasefile, "%sstrngout", DEFAULT_PATH);
	    if ((strngout = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    strouttag = d4tag (strngout, "NAME");
	    d4tag_select (strngout, strouttag);
	    stroutclosed = d4field (strngout, AL_CLOSED);
	    stroutopen = 1;
	}
	if ( (d4seek (strngout, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (stroutclosed))
		strncpy (pt -> mode, AL_CLOSED_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_SUPER_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (strngout);
    }
    else if ((strncmp (pt -> type, "tank", 4)) == 0)
    {
	if (!tank4open)
	{
	    sprintf (dbasefile, "%stank4", DEFAULT_PATH);
	    if ((tank4 = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    tank4tag = d4tag (tank4, "NAME");
	    d4tag_select (tank4, tank4tag);
	    tank4mode = d4field (tank4, AL_MODE);
	    tank4open = 1;
	}
	if ( (d4seek (tank4, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, f4str(tank4mode), sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (tank4);
    }
    else if ((strncmp (pt -> type, "td", 2)) == 0)
    {
	if (!tdopen)
	{
	    sprintf (dbasefile, "%std", DEFAULT_PATH);
	    if ((td = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    tdtag = d4tag (td, "NAME");
	    d4tag_select (td, tdtag);
	    tdclosed = d4field (td, AL_CLOSED);
	    tdopen = 1;
	}
	if ( (d4seek (td, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    if (f4true (tdclosed))
		strncpy (pt -> mode, AL_CLOSED_MODE, sizeof (pt-> mode));
	    else
		strncpy (pt -> mode, AL_SUPER_MODE, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (td);
    }
/*    else if ((strncmp (pt -> type, "udb", 3)) == 0)
    {
	strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	strncpy (pt -> mode, AL_NOT_USED, sizeof (pt -> mode));
    }*/
    else if ((strncmp (pt -> type, "valve", 5)) == 0)
    {
	if (!valveopen)
	{
	    sprintf (dbasefile, "%svalve", DEFAULT_PATH);
	    if ((valve = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    valvetag = d4tag (valve, "NAME");
	    d4tag_select (valve, valvetag);
	    valvemode = d4field (valve, AL_MODE);
	    valveopen = 1;
	}
	if ( (d4seek (valve, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, f4str(valvemode), sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NOT_USED, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NO_DATA, sizeof (pt -> mode));
	}
	d4unlock_all (valve);
    }
    else if ((strncmp (pt -> type, "waveform", 8)) == 0)
    {
	if (!wfopen)
	{
	    sprintf (dbasefile, "%swaveform", DEFAULT_PATH);
	    if ((waveform = d4open (&cb, dbasefile)) == 0)
	    {
		fprintf (stderr, "AlrmGrpRpt failed to open %s\n", dbasefile);
		return (1);
	    }
	    wftag = d4tag (waveform, "NAME");
	    d4tag_select (waveform, wftag);
	    wfegu = d4field (waveform, AL_EGUTAG);
	    wfopen = 1;
	}
	if ( (d4seek (waveform, pt -> name)) == 0)
	{
	    strncpy (pt -> egu_tag, f4str(wfegu), sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NOT_USED, sizeof (pt -> mode));
	}
	else
	{
	    strncpy (pt -> egu_tag, AL_NO_DATA, sizeof (pt -> egu_tag));
	    strncpy (pt -> mode, AL_NOT_USED, sizeof (pt -> mode));
	}
	d4unlock_all (waveform);
    }

    return (0);
}


/********************************************************************************/

static void print_header (fp, lcnt, pgnum)
    FILE	*fp;
    int		*lcnt;		/* line count */
    int		*pgnum;		/* page number */
{
    time_t 	now;
    char	timebuff[62];
    char	*p;
    extern	char       *ctime ();

    now = time(0);
    strcpy (timebuff, ctime(&now));
    if (p = strchr (timebuff, '\n'))
	*p = NULL;
    (*pgnum)++;
    fprintf (fp, "\fPAGE %d	%s	ALARM GROUP %s REPORT\n\n", *pgnum, timebuff, RequestedAlrmGrp);
    *lcnt = 2;
    fprintf (fp, "NAME                         DESCRIPTION                  VALUE            EGU TAG    ALARM STATUS      T/C COUNTER  AUTO/MAN     ALARM    SCAN\n");
    fprintf (fp, "---------------------------------------------------------------------------------------------------------------------------------------------------\n");
    (*lcnt) += 2;

    return;
}



/********************************************************************************/

static void print_report ()
{
    FILE		*fp;
    int			lcnt = 0;	/* line count	*/
    int			numpts = 0;
    int			pgnum = 0;
    int			fpg = 1;
    struct AlrmgrpPoint	*CurrPt;

    if ((fp = fopen (outfile, "w+")) == NULL)
    {
	fprintf (stderr, "failed to open %s\n", outfile);
	return;
    }
    if (FirstPoint == NULL)
    {
	print_header (fp, &lcnt, &pgnum);
	fprintf (fp, "\n	NO POINTS FOUND WITH ALARM GROUP %s\n",
		     RequestedAlrmGrp);
	fclose (fp);
	return;
    }
    CurrPt = FirstPoint;
    while (CurrPt)
    {
	if (fpg)
	{
	    print_header (fp, &lcnt, &pgnum);
	    fpg = 0;
	}
	else
	{
	    if ((lcnt + 1) > MAX_LINES_PER_PAGE)
		print_header (fp, &lcnt, &pgnum);
	}
	if (!CurrPt -> got_data)
	{
	    strncpy (CurrPt -> value, AL_NO_DATA, sizeof (CurrPt -> value));
	    strncpy (CurrPt -> stat, AL_NO_DATA, sizeof (CurrPt -> value));
	}
	if ((strncmp (CurrPt -> type, "bi", 2)) == 0)
	{
	    if (!CurrPt -> got_tcnt)
		strncpy (CurrPt -> tcnt, AL_NO_DATA, sizeof (CurrPt -> tcnt));
	}
	fprintf (fp, "%-28.28s %-28.28s %-16.16s %-10.10s %-16.16s  %-6.6s       %-12.12s %-8.8s %-8.8s\n",
		    CurrPt -> name,
		    CurrPt -> desc,
		    CurrPt -> value,
		    CurrPt -> egu_tag,
		    CurrPt -> stat,
		    CurrPt -> tcnt,
		    CurrPt -> mode,
		    (CurrPt -> onalarm == 'T') ? AL_ENABLED : AL_DISABLED,
		    (CurrPt -> onscan == 'T') ? AL_ENABLED : AL_DISABLED);
	lcnt++;
	numpts++;
	CurrPt = CurrPt -> NextPoint;
    }
    fprintf (fp, "\n\nTOTAL POINTS BELONGING TO ALARM GROUP %s is %d\n",
		 RequestedAlrmGrp, numpts);
    fclose (fp);

    return;
}


/********************************************************************************/

static void print_usage()
{
    fprintf (stderr, "AlrmGrpRpt:Usage, AlrmGrpRpt alarmgroup outputfile\n");
    exit (1);
}



/********************************************************************************/

static void remove_spaces (buf)
    char	*buf;
{
    char	*p;

    p = strrchr (buf, '\0');
    if (p == NULL)
	return;
    p--;
    while ((*p) == ' ')
	p--;
    p++;
    *p = NULL;
    return;
}


