/****************************************************************/
/*								*/
/*	COMMON INTIALIZATION FOR CHANNEL ACCESS SERVERS		*/
/*								*/
/*	Many of the things that we do for the three servers	*/
/*	that make up the agent architecture for channel access	*/
/*	are common. This routine avoids code duplication.	*/
/*								*/
/*	Author:	Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <fcntl.h>
#ifdef WIN32
#include <direct.h>
#include <process.h>
#include <stdio.h>
#include <io.h>
#include <sys/locking.h>
#else
#include <unistd.h>
#include <sys/file.h>
#include <termios.h>
#include <sys/ioctl.h>
#endif WIN32
#include <signal.h>
#include "msgMon.h"
#include <string.h>

#include <alarmDefs.h>

void serverInit(appName, dirPath, lockFile)
char	*appName;
char	*dirPath;
char	*lockFile;
{
    int		fd;
    char	buff[10];

/******
	Change our controlling TTY so that we don't pick up any
	stray signals.
******/

#if !(defined (SOLARIS) || defined (AXP) || defined(WIN32))
    fd = open("/dev/tty", O_RDWR);
    ioctl(fd, TIOCNOTTY, 0);
    close(fd);
#endif

/******
	Move to the defined directory so that the core file
	can be found in case we die an unnatural death.
******/

#ifdef WIN32
    if (_chdir(dirPath) != 0) {
	tisSysLog (0, 2, "%s(%d): Unable to find trend lock directory (%s)",
			appName, _getpid (), dirPath);
	if (_mkdir (dirPath) == 0) {
	    tisSysLog (0, 2, "%s(%d): Lock directory %s created.",
			appName, _getpid (), dirPath);
	} else {
	    tisSysLog (0, 2, "%s(%d): Lock directory %s could not be created.",
			appName, _getpid (), dirPath);
	    exit (-1);
	}
#else
    if (chdir(dirPath) != 0) {
	tisSysLog (0, 2, "%s(%d): Unable to find trend lock directory (%s)",
			appName, getpid (), dirPath);
	if (mkdir (dirPath, 0775) == 0) {
	    tisSysLog (0, 2, "%s(%d): Lock directory %s created.",
			appName, getpid (), dirPath);
	} else {
	    tisSysLog (0, 2, "%s(%d): Lock directory %s could not be created.",
			appName, getpid (), dirPath);
	    exit (-1);
	}
#endif WIN32
    }

/******
	Insure that we are the only copy of the server running
******/

#ifdef WIN32
    fd = _open(lockFile, _O_RDWR | _O_CREAT, 0640);
    if (fd < 0)
	exit(1);
    if (_locking (fd, _LK_LOCK, 0) == -1)
	exit (0);

/******
	Record our PID in the lock file to make it easy to find
******/

    sprintf(buff, "%d\n", _getpid());
    _write(fd, buff, strlen(buff));
    _close(fd);		/* I think that we keep the lock even if
			   the file is closed. */
#else
/******
	Make a process group just for ourselves
******/

#ifdef SOLARIS
    setpgrp ();
#else
#ifdef AXP
    setpgrp ();
    setpgid (0,0);
#else
    setpgrp(0, 0);
#endif
#endif

    fd = open(lockFile, O_RDWR | O_CREAT, 0640);
    if (fd < 0) {
	tisSysLog (0, 2, "%s(%d): Unable to access lock file in %s\n",
		appName, getpid (), dirPath); 
	exit(-1);
    }
#ifdef SOLARIS
    if (lockf (fd, F_TLOCK, 0) == -1) {
	tisSysLog (0, 2, "%s(%d): Already running.\n", appName, getpid ());
	exit (-1);
    }
#else
    if (flock(fd, LOCK_EX | LOCK_NB)) {
	tisSysLog (0, 2, "%s(%d): Already running.\n", appName, getpid ());
	exit(-1);
    }
#endif

/******
	Record our PID in the lock file to make it easy to find
******/

    sprintf(buff, "%d\n", getpid());
    write(fd, buff, strlen(buff));

#endif WIN32
}

