#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vtextitemHEADER
#include vlistitemHEADER
#include vbuttonHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "alarmDefs.h"
}

#include "alcfg.h"
#include "alCfgTreeListClass.hxx"
#include "alarmConfig.hxx"
#include "alCfgDatabaseClass.hxx"
#include "alCfgMainDialogClass.hxx"
#include "alCfgMainListClass.hxx"
#include "alCfgListDialogClass.hxx"
#include "alCfgUtils.hxx"



vkindTYPED_FULL_DEFINITION(alCfgListDialogClass, vdialog, 
						"alCfgListDialogClass");

/*******************************************************************
*
*    alCfgListDialogClass override for constructors init ()
*
*******************************************************************/

void alCfgListDialogClass::LoadInit (vresource res)
{
    title = NULL;
    listView = alCfgListClass::RequestSubLoad (this,
					alcfg_alCfgListView);
    okButton = alCfgListOkButtonClass::RequestSubLoad (this,
					alcfg_alCfgListOkButton);
    cancelButton = alCfgListCancelButtonClass::RequestSubLoad (this,
					alcfg_alCfgListCancelButton);
    // init inherited data
    vdialog::LoadInit (res);

    // store ptrs
    listView -> storeListDialog (this);
    okButton -> storeListDialog (this);
    cancelButton -> storeListDialog (this);

}


/*******************************************************************
*
*    alCfgListDialogClass Destroy function
*
*******************************************************************/

void alCfgListDialogClass::Destroy (void)
{
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
    if (title != NULL)
    {
	vstrDestroy (title);
    }
    // this list was created in listOpen
    if ( (listType == EDIT_PRINTER_FLAG) || (listType == REMOVE_PRINTER_FLAG) )
    {
	vlist *list = listView -> GetList ();
	delete list;
    }
    listView -> SetList (NULL);
    vdialog::Destroy ();
}


/*******************************************************************
*
*    alCfgListDialogClass notify function
*
*******************************************************************/

void alCfgListDialogClass::ObserveWindow (vevent *event)
{
    if ( event -> GetType () == veventWINDOW_STATE && event -> IsClose () )
    {
	vevent::StopProcessing ();
    }
}


/*******************************************************************
*
*    alCfgListDialogClass Open function
*
*******************************************************************/

void alCfgListDialogClass::listOpen ( alCfgMainDialogClass *mnDialog, int flag)
{
    mainDialog = mnDialog;
    listType = flag;

    alCfgApplication *app = mainDialog -> getApplication ();
    vresource titlesRes = mainDialog -> getTitlesResource ();

    vlist *list;

    if (listType == REMOVE_GROUP_FLAG)
    {
	title = vresourceGetString (titlesRes, alcfg_alCfgRemoveGrpTitle);
	list = app -> getGroupList ();
    }
    else if (listType == INSERT_GROUP_FLAG)
    {
	title = vresourceGetString (titlesRes, alcfg_alCfgInsertGrpTitle);
	list = app -> getGroupList ();
    }
    else if (listType == EDIT_PRINTER_FLAG)
    {
	title = vresourceGetString (titlesRes, alcfg_alCfgEditPrtTitle);
	list = new vlist;
	app -> getPrinterNamesList (list);
    }
    else 
    {
	title = vresourceGetString (titlesRes, alcfg_alCfgRemovePrtTitle);
	list = new vlist;
	app -> getPrinterNamesList (list);
    }

    listView -> SetList (list);
    vlistSelection *selection = listView -> GetSelection ();
    list -> SetSelectMethod (selection, vlistONE_ONLY_SELECTION);
    list -> SetAtLeastOneSelection (selection, vTRUE);

    // start as nothing
    vmemSet ((void *)selectedName, 0, sizeof (selectedName));

    SetTitle (title);
    Place (GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);

    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    vevent::Process ();
}


/*******************************************************************
*
*    alCfgListDialogClass delete function
*
*******************************************************************/

int alCfgListDialogClass::deleteListItem (vchar *listItem)
{
    int ret = ALCFG_ERROR;
    int result;
    alCfgApplication *app = mainDialog -> getApplication ();

    if (listType == REMOVE_GROUP_FLAG)
    {
	char listName[ALCFG_SELECTEDNAME_SIZE + 1];

	vcharExportBounded (vcharScribeSystemExport (listItem),
			    listName,
			    ALCFG_SELECTEDNAME_SIZE);
	listName[ALCFG_SELECTEDNAME_SIZE] = 0;
	vstr *mainGroup = vstrCloneScribed (vcharScribeSystem (ALCFG_MAIN_GROUP));
	// dont allow them to delete MAIN
	if ((vcharCompare (listItem, mainGroup)) == 0)
	{
	    alCfgMessageDialogCenter (this, ALCFG_NO_DELETE_MAIN_ERR);
	    ret = ALCFG_ERROR;
	}
	else
	{
	    alarmTreeClass *tree = app -> getAlarmTree ();
	    TREENODE_PTR root = tree -> getTreeRoot ();
	    // cant delete a group if it is in the tree
	    if ((tree -> findNode (root, listName)) != NULL)
	    {
		alCfgMessageDialogCenter (this, ALCFG_GRP_INTREE_ERR, listItem);
		ret = ALCFG_ERROR;
	    }
	    else
	    {
		result = alCfgMessageDialogCenter (this, ALCFG_DELETE_GRP_REQ, listItem);
		if (result == ALCFG_TRUE)
		{
		    app -> deleteFromGroupList (listItem);
		    ret = ALCFG_SUCCESS;
		}
		else
		{
		    ret = ALCFG_ERROR;
		}
	    }
	}
	vstrDestroy (mainGroup);
    }
    // delete a printer group
    else
    {
	result = alCfgMessageDialogCenter (this, ALCFG_DELETE_PRT_REQ, listItem);
	if (result == ALCFG_TRUE)
	{
	    app -> deleteFromPrinterList (listItem);
	    ret = ALCFG_SUCCESS;
	}
	else
	{
	    ret = ALCFG_ERROR;
	}
    }

    return (ret);
}


/*******************************************************************
*
*    alCfgListDialogClass insert alarm group into alarm tree
*
*******************************************************************/

int alCfgListDialogClass::insertGroup(vlist *list, vchar *listItem)
{
    int ret = ALCFG_ERROR;
    char listName[ALCFG_SELECTEDNAME_SIZE + 1];

    vcharExportBounded (vcharScribeSystemExport (listItem),
			listName,
			ALCFG_SELECTEDNAME_SIZE);
    listName[ALCFG_SELECTEDNAME_SIZE] = 0;

    alCfgApplication *app = mainDialog -> getApplication ();
    alarmTreeClass *tree = app -> getAlarmTree ();
    TREENODE_PTR root = tree -> getTreeRoot ();
    // cant insert a group into the tree if it is already in the tree
    if ((tree -> findNode (root, listName)) != NULL)
    {
	alCfgMessageDialogCenter (this, ALCFG_GRP_INTREE_ERR, listItem);
	ret = ALCFG_ERROR;
    }
    else
    {
	alCfgMainListClass *mainListView = mainDialog -> getListView ();
	vlist *list = mainListView -> GetList ();
	vlistSelection *selection = mainListView -> GetSelection ();
	int row, column;

	// has a node in the tree (displayed on main list view) been selected
	list -> GetSelectedCell (selection, &row, &column);
	if (row == vlistNO_CELLS_SELECTED)
	{
	    alCfgMessageDialogCenter (this, ALCFG_NO_NODE_SEL_ERR);
	    ret = ALCFG_ERROR;
	}
	else
	{
	    listEntry *entry = (listEntry *)(list -> GetCellValue (row, 0));
	    TREENODE_PTR node = entry -> getTreeNode ();
	    vstr *group = vstrCloneScribed (vcharScribeSystem (node -> groupName));
	    int result = alCfgMessageDialogCenter (this, 
						   ALCFG_INSERT_GRP_TREE_REQ, 
						   listItem,
						   group);
	    if (result == ALCFG_TRUE)
	    {
		// copy for later use by tree list class
		vcharCopyBounded (listItem, selectedName, ALCFG_SELECTEDNAME_SIZE);
		selectedName[ALCFG_SELECTEDNAME_SIZE] = 0;
		ret = ALCFG_SUCCESS;
	    }
	    else
	    {
		ret = ALCFG_ERROR;
	    }
	    vstrDestroy (group);
	}
    }

    return (ret);
}


vkindTYPED_FULL_DEFINITION(alCfgListClass, vlistview, "alCfgListClass");

/*******************************************************************
*
*    alCfgListClass 
*	Observer function
*		a double click does basically the same as the 
*		OK button
*
*******************************************************************/

void alCfgListClass::ObserveSelect (int state)
{
    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	int clickCount = GetLastPointerClickCount ();
	if (clickCount == 2)
	{
	    vlist *list = GetList ();
	    vlistSelection *listSelection = GetSelection ();
	    int row, column;

	    list -> GetSelectedCell (listSelection, &row, &column);
	    vchar *value = (vchar *)(list -> GetCellValue (row, 0));
	    int listType = listDialog -> getListType ();
	    if ( (listType == REMOVE_GROUP_FLAG) || (listType == REMOVE_PRINTER_FLAG) )
	    {
		if ((listDialog -> deleteListItem (value)) == ALCFG_ERROR)
		{
		    return;
		}
	    }
	    else if (listType == INSERT_GROUP_FLAG)
	    {
		if ((listDialog -> insertGroup (list, value)) == ALCFG_ERROR)
		{
		    return;
		}
	    }
	    // edit printer group
	    else
	    {
		vchar *selectedName = listDialog -> getSelectedName ();
		vcharCopyBounded (value, selectedName, ALCFG_SELECTEDNAME_SIZE);
		selectedName[ALCFG_SELECTEDNAME_SIZE] = 0;
	    }
	    // close dialog and stop event loop started in listOpen ()
	    listDialog -> Close();
	    vevent::StopProcessing ();
	}
    }
}


vkindTYPED_FULL_DEFINITION(alCfgListOkButtonClass, vbutton, 
				"alCfgListOkButtonClass");

/*******************************************************************
*
*    alCfgListOkButtonClass 
*	Observer function
*
*******************************************************************/

void alCfgListOkButtonClass::ObserveDialogItem (vevent *event)
{
    vlist *list= listDialog -> listView -> GetList ();
    vlistSelection *selection = listDialog -> listView -> GetSelection ();
    int row, column;

    list -> GetSelectedCell (selection, &row, &column);
    if (row == vlistNO_CELLS_SELECTED)
    {
	alCfgMessageDialogCenter (listDialog, ALCFG_NOTHING_SELECTED_ERR);
	return;
    }
    vchar *value = (vchar *)(list -> GetCellValue (row, 0));
    int listType = listDialog -> getListType ();
    if ( (listType == REMOVE_GROUP_FLAG) || (listType == REMOVE_PRINTER_FLAG) )
    {
	if ((listDialog -> deleteListItem (value)) == ALCFG_ERROR)
	{
	    return;
	}
    }
    else if (listType == INSERT_GROUP_FLAG)
    {
	if ((listDialog -> insertGroup (list, value)) == ALCFG_ERROR)
	{
	    return;
	}
    }
    // edit printer group
    else
    {
	vchar *selectedName = listDialog -> getSelectedName ();
	vcharCopyBounded (value, selectedName, ALCFG_SELECTEDNAME_SIZE);
	selectedName[ALCFG_SELECTEDNAME_SIZE] = 0;
    }
    // close dialog and stop event loop started in listOpen ()
    listDialog -> Close();
    vevent::StopProcessing ();
}


vkindTYPED_FULL_DEFINITION(alCfgListCancelButtonClass, vbutton, 
				"alCfgListCancelButtonClass");

/*******************************************************************
*
*    alCfgListCancelButtonClass 
*	Observer function
*
*******************************************************************/

void alCfgListCancelButtonClass::ObserveDialogItem (vevent *event)
{
    // zap so caller has nothing to get
    vchar *selectedName = listDialog -> getSelectedName ();
    vmemSet ((void *)selectedName, 0, ALCFG_SELECTEDNAME_SIZE);

    // close dialog and stop event loop started in listOpen ()
    listDialog -> Close();
    vevent::StopProcessing ();
}


