#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vlistitemHEADER
#include vimageHEADER
#include vfontHEADER
#include vbuttonHEADER

extern "C"
{
#include "alarmDefs.h"
}

#include "alcfg.h"
#include "alCfgTreeListClass.hxx"
#include "alarmConfig.hxx"
#include "alCfgMainDialogClass.hxx"
#include "alCfgMainListClass.hxx"
#include "alCfgUtils.hxx"


/*******************************************************************
*
*    alCfgMainListClass 
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(alCfgMainListClass, vlistitem, "alCfgMainListClass");

/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

void alCfgMainListClass::LoadInit (vresource resource)
{
    vlistitem::LoadInit (resource);

    // load in the images from the image dictionary

    vresource appRes = vapplication::GetCurrent () -> GetResources ();
    vresource imageRes = vresourceGet (appRes, alcfg_alCfgImageDict);

    mainsClosedImage = new vimage (vresourceGet (imageRes, 
						alcfg_alCfgMainClsIm));
    mainsOpenImage = new vimage (vresourceGet (imageRes, 
						alcfg_alCfgMainOpIm));
    closedWithSubGroups = new vimage (vresourceGet (imageRes, 
						alcfg_alCfgGrpStClsIm));
    closedWithNoSubGroups = new vimage (vresourceGet (imageRes,
						alcfg_alCfgGrpEndClsIm));
    openWithGroups = new vimage (vresourceGet (imageRes,
						alcfg_alCfgGrpStOpIm));
    openWithNoGroups = new vimage (vresourceGet (imageRes,
						alcfg_alCfgGrpEndOpIm));
    continueLine = new vimage (vresourceGet (imageRes,
						alcfg_alCfgContLineIm));
    continueLineWithGroups = new vimage (vresourceGet (imageRes,
						alcfg_alCfgContGrpIm));
    endOfGroup = new vimage (vresourceGet (imageRes,
						alcfg_alCfgContGrpEndIm));
    vlist *list = GetList ();
    vlistSelection *select = GetSelection ();
    list -> SetSelectMethod (select, vlistONE_ONLY_SELECTION);
    list -> SetRowCount (0);
}


/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

void alCfgMainListClass::Destroy (void)
{
    delete mainsClosedImage;
    delete mainsOpenImage;
    delete closedWithSubGroups;
    delete closedWithNoSubGroups;
    delete openWithGroups;
    delete openWithNoGroups;
    delete continueLine;
    delete continueLineWithGroups;
    delete endOfGroup;

    vlistitem::Destroy ();
}


/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

int alCfgMainListClass::HandleButtonUp (vevent *event)
{
    buttonUpFlag = vFALSE;
    return (vlistitem::HandleButtonUp (event));
}

/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

void alCfgMainListClass::ObserveSelect (int state)
{
    int row = GetSelectedItem ();
    if (row == vlistitemNO_CELLS_SELECTED)
    {
	return;
    }
    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	listEntry *entry = (listEntry *)(GetItemValue (row));
	if (entry == NULL)
	{
	    return;
	}
	TREENODE_PTR node = entry -> getTreeNode ();
	if (node == NULL)
	{
	    return;
	}
	if (node -> boxType == ALCFG_NO_GROUP_BOX)
	{
	    return;
	}
	int currx, curry;
	GetLastPointerPosition (&currx, &curry);
	vrect tempRect;
	GetCellRect (row, 0, &tempRect);
	vimage *image = entry -> getImage (this);
	tempRect.SetX (tempRect.GetX () + (node -> level * ENTRY_LEVEL_INSET));
	tempRect.SetWidth (image -> GetWidth ());
	if ( ((entry -> IsLocked ()) == vFALSE) &&
	     (buttonUpFlag == vFALSE) &&
	     (tempRect.ContainsPoint (currx, curry)) )
	{
	     entry -> SetLocked (vTRUE);
	     buttonUpFlag = vTRUE;
	     if ( (entry -> IsOpen ()) == ALCFG_BOX_OPENED)
	     {
		CloseEntry (row, entry);
	     }
	     else
	     {
		OpenEntry (row, entry);
	     }
	     entry -> SetLocked (vFALSE);
//	     InvalViewCell (row, 0);
	     InvalRect ((const vrect *)(&tempRect), vwindowINVAL_UPDATE);    
	}
    }
}


/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

void alCfgMainListClass::DrawCellBackground (int row, int col, 
					     int selected, vrect *cellRect)
{
    vlist *list = GetList ();
    listEntry *tempEntry = (listEntry *)list -> GetCellValue (row, col);
    if (tempEntry == NULL)
    {
	return;
    }
    TREENODE_PTR node = tempEntry -> getTreeNode ();
    if ( (selected == vFALSE) || (node == NULL) )
    {
	vlistview::DrawCellBackground (row, col, selected, cellRect);
    }
    else
    {
	vimage *image = tempEntry -> getImage (this);
	vrect tempRect = *cellRect;
	int offset = node -> level * ENTRY_LEVEL_INSET + image -> GetWidth ();
	vfont *font = vfont::FindScribed (vcharScribeLiteral (""));
	int stringWidth = font -> ScribedStringWidthX (vcharScribeSystem (node -> groupName));
	tempRect.SetX (tempRect.GetX() + offset);
	tempRect.SetWidth (stringWidth + TEXT_WIDTH_ADD);
	vlistview::DrawCellBackground (row, col, selected, &tempRect);
    }
}


/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

void alCfgMainListClass::DrawCellFocus (int row, int col, vrect *cellRect)
{
}


/*******************************************************************
*
*    alCfgMainListClass override 
*
*******************************************************************/

void alCfgMainListClass::DrawCellContents (int row, int col, 
					   int selected, vrect *cellRect)
{
    vlist *list = GetList();
    listEntry *entry = (listEntry *)(list -> GetCellValue(row, col));
    if (entry == NULL)
	return;
    TREENODE_PTR node = entry -> getTreeNode ();
    if (node == NULL)
	return;

    alCfgApplication *app = mainDialog -> getApplication ();
    alarmTreeClass *alarmTree = app -> getAlarmTree ();

    vdrawGSave();
    vdrawRectsClip(cellRect, 1);

    vrect temp = *cellRect;
    vdrawRectClip(temp.GetX(), temp.GetY(), temp.GetWidth(), temp.GetHeight());

    vimage *image;
    int imageOffset;

    if (row > 0)
    {
	image = continueLine;
	listEntry *tempEntry;
	int tempRow;
	int lastLevel = -1;
	int lastImage = -1;
	TREENODE_PTR tempNode;
	for (tempRow = row - 1; tempRow >= 1; tempRow--)
	{
	    tempEntry = (listEntry *)(list -> GetCellValue (tempRow, 0));
	    tempNode = tempEntry -> getTreeNode ();
	    if (tempNode -> level < node -> level)
	    {
		if (tempNode -> level >= 1)
		{
		    if ((lastLevel == tempNode -> level) && 
			((lastImage == ALCFG_PARENT) || (lastImage == ALCFG_LAST_PARENT)))
		    {
			continue;
		    }
		    if (tempNode -> imageType == ALCFG_PARENT)
		    {
			imageOffset = tempNode -> level * ENTRY_LEVEL_INSET;
			vdrawMoveTo( temp.GetX () + imageOffset,
			             temp.GetY() + 
				     (temp.GetHeight() - image->GetHeight())/2);
			vdrawImageCompositeIdent(image);
		    }
		    lastLevel = tempNode -> level;
		    lastImage = tempNode->imageType;
		}
		if (tempNode -> level == 1)
		{
		    break;
		}
	    }
	}
    }

    image = entry -> getImage (this);

    imageOffset = node -> level * ENTRY_LEVEL_INSET;

    vdrawMoveTo( temp.GetX () + imageOffset,
		temp.GetY() + (temp.GetHeight() - image->GetHeight())/2);
    vdrawImageCompositeIdent(image);

    int offset = node -> level * ENTRY_LEVEL_INSET + image -> GetWidth();
    temp.SetX(temp.GetX() + offset);
    temp.SetWidth(temp.GetWidth() - offset);

    vstr *alarmGroup;
    alarmGroup = vstrCloneScribed (vcharScribeSystem (node -> groupName));
    vdrawRectShowCenterY(alarmGroup, &temp);

    vdrawGRestore();

    vstrDestroy (alarmGroup);
}



//** START EXTENSIONS TO THE CLASS ********************************
//**
//
/*******************************************************************
*
*    alCfgMainListClass close an entry
*
*******************************************************************/

void alCfgMainListClass::CloseEntry (int row, listEntry *entry)
{
    vlist *list = GetList ();
    TREENODE_PTR parent = entry -> getTreeNode ();
    row++;
    while (list -> GetRowCount () && row < list -> GetRowCount ())
    {
	listEntry *currEntry = (listEntry *)(list -> GetCellValue (row, 0));
	TREENODE_PTR node = currEntry -> getTreeNode ();
	if ( (currEntry == NULL) || (node -> parent != parent) )
	{
	    break;
	}
	if ( (currEntry -> IsOpen ()) == ALCFG_BOX_OPENED)
	{
	    CloseEntry (row, currEntry);
	}
	list -> RemoveRow (row);
	delete currEntry;
    }
    entry -> SetOpen (ALCFG_BOX_CLOSED);
}


/*******************************************************************
*
*    alCfgMainListClass cut a group from the tree list
*
*******************************************************************/

void alCfgMainListClass::CutGroup (void)
{
    vlist *list = GetList ();
    vlistSelection *selected = GetSelection ();
    int row, column;
    list -> GetSelectedCell (selected, &row, &column);
    if (row == vlistNO_CELLS_SELECTED)
    {
	alCfgMessageDialogCenter (mainDialog, ALCFG_NO_NODE_SEL_ERR);
	return;
    }
    listEntry *entry = (listEntry *)(list -> GetCellValue (row, 0));
    TREENODE_PTR node = entry -> getTreeNode ();
    alCfgApplication *app = mainDialog -> getApplication ();
    alarmTreeClass *alarmTree = app -> getAlarmTree ();
    int result;
    if ((alarmTree -> clipBoardActive ()) == vTRUE)
    {
	TREENODE_PTR clipNode = alarmTree -> getClipBoard ();
	vstr *group = vstrCloneScribed (vcharScribeSystem (clipNode -> groupName));
	result = alCfgMessageDialogCenter (mainDialog, ALCFG_CLIPBOARD_ACTIVE_MSG, group);
	vstrDestroy (group);
	if (result == ALCFG_FALSE)
	{
	    return;
	}
	alarmTree -> deleteSubTreeNodesTrav (clipNode);
    }
    if ((alarmTree -> isRoot (node)) == vTRUE)
    {
	alCfgMessageDialogCenter (mainDialog, ALCFG_NOCUT_MAIN_ERR);
	return;
    }
    if ((entry -> IsOpen ()) == ALCFG_BOX_OPENED)
    {
	entry -> SetLocked (vTRUE);
	CloseEntry (row, entry);
	entry -> SetLocked (vFALSE);
    }
    alarmTree -> cutSubTree (node);
    list -> RemoveRow (row);
    delete entry;
    InvalView ();
    mainDialog -> pasteMenuItem -> Enable ();
}


/*******************************************************************
*
*    alCfgMainListClass delete group from list and tree
*
*******************************************************************/

void alCfgMainListClass::DeleteGroup (void)
{
    vlist *list = GetList ();
    vlistSelection *selected = GetSelection ();
    int row, column;
    list -> GetSelectedCell (selected, &row, &column);
    if (row == vlistNO_CELLS_SELECTED)
    {
	alCfgMessageDialogCenter (mainDialog, ALCFG_NO_NODE_SEL_ERR);
	return;
    }
    listEntry *entry = (listEntry *)(list -> GetCellValue (row, 0));
    TREENODE_PTR node = entry -> getTreeNode ();
    alCfgApplication *app = mainDialog -> getApplication ();
    alarmTreeClass *alarmTree = app -> getAlarmTree ();
    if ((alarmTree -> isRoot (node)) == vTRUE)
    {
	alCfgMessageDialogCenter (mainDialog, ALCFG_NO_DELETE_MAIN_ERR);
	return;
    }
    if ((entry -> IsOpen ()) == ALCFG_BOX_OPENED)
    {
	entry ->SetLocked (vTRUE);
	CloseEntry (row, entry);
	entry -> SetLocked (vFALSE);
    }
    alarmTree -> deleteSubTree (node);
    list -> RemoveRow (row);
    delete entry;
    InvalView ();

}


/*******************************************************************
*
*    alCfgMainListClass insert group into tree
*
*******************************************************************/

void alCfgMainListClass::InsertGroup (vchar *grpName)
{
    char groupName[AL_GROUPNAME_SIZE + 1];
    vcharExportBounded (vcharScribeSystemExport (grpName),
			groupName, AL_GROUPNAME_SIZE);
    groupName[AL_GROUPNAME_SIZE] = 0;

    alCfgApplication *app = mainDialog -> getApplication ();
    alarmTreeClass *alarmTree = app -> getAlarmTree ();
    vlist *list = GetList ();
    vlistSelection *selected = GetSelection ();
    int selectedRow, column;
    list -> GetSelectedCell (selected, &selectedRow, &column);

    // get ptr to parent for group being added to tree
    listEntry *entry = (listEntry *)(list -> GetCellValue (selectedRow, 0));
    TREENODE_PTR node = entry -> getTreeNode ();
    alarmTree -> addChild (node, groupName);
    if ((entry -> IsOpen ()) == ALCFG_BOX_OPENED)
    {
	// find row toplace new entry after
	int row = TraverseView (selectedRow, entry);
	// find node just added to tree
	TREENODE_PTR newNode = alarmTree -> findNode (node, groupName);
	entry -> SetLocked (vTRUE);
	list -> InsertRow (row);
	listEntry *newEntry = new listEntry ();
	newEntry -> setTreeNode (newNode);
	list -> SetCellValue (row, 0, (const void *)newEntry);
	entry -> SetLocked (vFALSE);
    }
    InvalViewCell (selectedRow, 0);
}


/*******************************************************************
*
*    alCfgMainListClass insert MAIN group 
*
*******************************************************************/

void alCfgMainListClass::InsertMain (void)
{
    vlist *list = GetList();

    listEntry *entry = new listEntry ();

    alCfgApplication *alCfgApp = mainDialog -> getApplication ();
    alarmTreeClass *alarmTree = alCfgApp -> getAlarmTree ();
    TREENODE_PTR treeRoot = alarmTree -> getTreeRoot ();
    entry -> setTreeNode (treeRoot);
    if (treeRoot -> sublink == NULL)
    {
	entry -> SetOpen (ALCFG_BOX_OPENED);
    }
    else
    {
	entry -> SetOpen (ALCFG_BOX_CLOSED);
    }
    list -> SetCellValue (0, 0, (const void *)entry);
 //   InvalViewCell (0, 0);

}


/*******************************************************************
*
*    alCfgMainListClass open an entry
*
*******************************************************************/

void alCfgMainListClass::OpenEntry (int row, listEntry *entry)
{
    TREENODE_PTR node = entry -> getTreeNode ();
    if (node -> sublink != NULL)
    {
	vlist *list = GetList ();
	TREENODE_PTR currNode = node -> sublink;
	while (currNode)
	{
	    listEntry *newEntry = new listEntry ();
	    newEntry -> setTreeNode (currNode);
	    row++;
	    list -> InsertRow (row);
	    list -> SetCellValue (row, 0, (const void *)newEntry);
	    currNode = currNode -> fwdlink;
	}
	entry -> SetOpen (ALCFG_BOX_OPENED);
	InvalViewCell (row, 0);
    }
}


/*******************************************************************
*
*    alCfgMainListClass paste a group from the clipboard to the 
*	tree
*
*******************************************************************/

void alCfgMainListClass::PasteGroup (void)
{
    vlist *list = GetList ();
    vlistSelection *selected = GetSelection ();
    int row, column;
    list -> GetSelectedCell (selected, &row, &column);
    if (row == vlistNO_CELLS_SELECTED)
    {
	alCfgMessageDialogCenter (mainDialog, ALCFG_NO_NODE_SEL_ERR);
	return;
    }
    listEntry *entry = (listEntry *)(list -> GetCellValue (row, 0));
    TREENODE_PTR node = entry -> getTreeNode ();
    alCfgApplication *app = mainDialog -> getApplication ();
    alarmTreeClass *alarmTree = app -> getAlarmTree ();

    alarmTree -> pasteSubTree (node);
    if ((entry -> IsOpen ()) == ALCFG_BOX_OPENED)
    {
	listEntry *newEntry = new listEntry ();
	newEntry -> setTreeNode (node -> sublink);
	entry -> SetLocked (vTRUE);
	row++;
	list -> InsertRow (row);
	list -> SetCellValue (row, 0, (const void *)newEntry);
	entry -> SetLocked (vFALSE);
    }
    InvalView ();
    mainDialog -> pasteMenuItem -> Disable ();
}


/*******************************************************************
*
*    alCfgMainListClass Traverse the view to find a row to insert
*	a new row after
*
*******************************************************************/

int alCfgMainListClass::TraverseView (int row, listEntry *entry)
{
    vlist *list = GetList ();
    TREENODE_PTR parent = entry -> getTreeNode ();
    row++;
    while (list -> GetRowCount () && row < list -> GetRowCount ())
    {
	listEntry *currEntry = (listEntry *)(list -> GetCellValue (row, 0));
	TREENODE_PTR node = currEntry -> getTreeNode ();
	if ( (currEntry == NULL) || (node -> parent != parent) )
	{
	    break;
	}
	if ( (currEntry -> IsOpen ()) == ALCFG_BOX_OPENED)
	{
	    row = TraverseView (row, currEntry);
	}
	else
	{
	    row++;
	}
    }
    return (row);
}


/*******************************************************************
*
*    listEntry Class
*
*******************************************************************/

listEntry::listEntry ()
{
    isOpen = ALCFG_BOX_CLOSED;
    isLocked = vFALSE;
    treeNode = NULL;
}

listEntry::~listEntry ()
{
}


/*******************************************************************
*
*    listEntry Class get image for a row
*
*******************************************************************/

vimage *listEntry::getImage (alCfgMainListClass *mainList)
{
    vimage *image = NULL;
    switch (treeNode -> boxType)
    {
	case ALCFG_NO_GROUP_BOX:
	    if (treeNode -> imageType == ALCFG_CHILD)
	    {
		image = mainList -> continueLineWithGroups;
	    }
	    else
	    {
		image = mainList -> endOfGroup;
	    }
	    break;

	case ALCFG_GROUP_BOX:
	    if (isOpen == ALCFG_BOX_CLOSED)
	    {
		switch (treeNode -> imageType)
		{
		    case ALCFG_MAIN_PARENT:
			image = mainList -> mainsClosedImage;
			break;

		    case ALCFG_PARENT:
			image = mainList -> closedWithSubGroups;
			break;

		    case ALCFG_LAST_PARENT:
			image = mainList -> closedWithNoSubGroups;
			break;
		}
	    }
	    // need an open box
	    else
	    {
		switch (treeNode -> imageType)
		{
		    case ALCFG_MAIN_PARENT:
			image = mainList -> mainsOpenImage;
			break;

		    case ALCFG_PARENT:
			image = mainList -> openWithGroups;
			break;

		    case ALCFG_LAST_PARENT:
			image = mainList -> openWithNoGroups;
			break;
		}
	    }
	    break;
    }
    return (image);
}
