#ifndef alcfgmainlistclassHXX
#define	alcfgmainlistclassHXX

/*
    mainsClosedImage		mainsOpenImage
	+ -MAIN				- -MAIN
	|				|

    closedWithSubGroups		closedWithNoSubGroups
	|				|
	+ -GROUP			+ -GROUP
	|

    openWithGroups		openWithNoGroups
	|				|
	- -GROUP			- -GROUP
	|

    continueLine
	|

    continueLineWithGroups	endOfGroup
	|				|
	|- -GROUP			- -GROUP
	|
*/

// define inset value
#define	ENTRY_LEVEL_INSET	16
// define a few pixels to place after the text string for highlighting
#define	TEXT_WIDTH_ADD		2

// define isOpen
typedef enum
{
    ALCFG_BOX_CLOSED = 0,
    ALCFG_BOX_OPENED
} ALCFG_BOX_STATUS;

class alCfgMainListClass;

class listEntry
{
    ALCFG_BOX_STATUS	isOpen;
    int			isLocked;
    TREENODE_PTR	treeNode;

public:

    listEntry (void);
    ~listEntry (void);

    // functions to handle isOpen flag
    void SetOpen (ALCFG_BOX_STATUS flag)
	{ isOpen = flag; }
    ALCFG_BOX_STATUS IsOpen (void)
	{ return (isOpen); }

    // functions to handle isLocked flag
    void SetLocked (int flag)
	{ isLocked = flag; }
    int IsLocked (void)
	{ return (isLocked); }

    // functions to handle treeNode
    void setTreeNode (TREENODE_PTR node)
	{ treeNode = node; }
    TREENODE_PTR getTreeNode (void)
	{ return (treeNode); }

    // function to return image ptr
    vimage *getImage (alCfgMainListClass *mainListClass);
};


//** CLASS alCfgMainListClass *************************************************

class	alCfgMainListClass : public vlistitem
{
    alCfgMainDialogClass	*mainDialog;
    int				buttonUpFlag;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(alCfgMainListClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(alCfgMainListClass, vlistitem);

    // override
    void LoadInit (vresource resource);

    // override
    void Destroy (void);

    // virtual member function override
    void ObserveSelect (int state);

    // override
    void DrawCellContents (int row, int col, int selected, vrect *rect);

    // override
    void DrawCellBackground (int row, int column, int selected, vrect *rect);

    // override
    void DrawCellFocus (int row, int column, vrect *rect);

    // override 
    int HandleButtonUp (vevent *event);

    // extensions to class

    // functions to handle mainDialog
    void storeMainDialog (alCfgMainDialogClass *cd)
	{ mainDialog = cd; }

    // close entry in list
    void CloseEntry (int row, listEntry *entry);

    // cut a group from the list
    void CutGroup (void);

    // delete a group from the list
    void DeleteGroup (void);

    // add a new group to the list
    void InsertGroup (vchar *groupName);

    // put MAIN into list
    void InsertMain (void);

    // open entry in list
    void OpenEntry (int row, listEntry *entry);

    // paste group from clipboard
    void PasteGroup (void);

    // traverse the view
    int TraverseView (int row, listEntry *entry);

    // function for buttonUpFlag
    void setButtonUpFlag (int val)
	{ buttonUpFlag = val; }

    // public so dont need funcs to get at
    vimage			*mainsClosedImage;
    vimage			*mainsOpenImage;
    vimage			*closedWithSubGroups;
    vimage			*closedWithNoSubGroups;
    vimage			*openWithGroups;
    vimage			*openWithNoGroups;
    vimage			*continueLine;
    vimage			*continueLineWithGroups;
    vimage			*endOfGroup;

};

#endif	alcfgmainlistclassHXX

