#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vlistHEADER
#include vlistitemHEADER
#include vbuttonHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "alarmDefs.h"
}

#include "alcfg.h"
#include "alCfgTreeListClass.hxx"
#include "alarmConfig.hxx"
#include "alCfgMainDialogClass.hxx"
#include "alCfgDatabaseClass.hxx"
#include "alCfgMainListClass.hxx"
#include "alCfgUtils.hxx"

// use by the dblibrary functions and is declared extern there
// therefore I declare it global here
DB_DATABASE	DB_Database;	


/*******************************************************************
*
*    Main ()
*
*******************************************************************/

int main (int argc, char **argv)
{
    char dbPath[ALCFG_MAX_DATABASE_PATH];

    // Initialize Galaxy
    vstartup (argc, argv);


    vapplication *app = vapplication::GetCurrent ();
#ifndef WIN32
    strncpy (dbPath, "/tis4000db", ALCFG_MAX_DATABASE_PATH);
    app -> SetFont (vfont::FindScribed (vcharScribeLiteral ("family:courier-face:medium-size:12")));
#else
    strncpy (dbPath, "c:\\tis4000db", ALCFG_MAX_DATABASE_PATH);
    app -> SetFont (vfont::FindScribed (vcharScribeLiteral ("family:courier-face:medium-size:10")));
#endif WIN32

    alCfgApplication *alCfgApp = new alCfgApplication (argv[0], dbPath);

    if ( (alCfgApp -> getInitErrors ()) != ALCFG_SUCCESS)
    {
	delete alCfgApp;
	exit (EXIT_FAILURE);
    }

    // enter main event process loop
    veventProcess ();

    // have exited process loop, start clean up
    delete alCfgApp;

    exit (EXIT_SUCCESS);
    return EXIT_FAILURE;
}


/*******************************************************************
*
*    Alarm Config Application constructor
*
*******************************************************************/

alCfgApplication::alCfgApplication (char *programName, char *dbPath)
{
    initErrors = ALCFG_SUCCESS;
    initedTisGalaxy = vFALSE;
    alCfgMainDialog = NULL;
    alarmTree = NULL;
    groupList = NULL;
    printerList = NULL;
    groupListModified = vFALSE;
    printerListModified = vFALSE;

    alCfgvapplication = vapplication::GetCurrent ();
    alCfgvresource = alCfgvapplication -> GetResources ();

    // create array of defines for tag names in app
    alcfgInternNames ();

    // init the database engine
    DB_InitDatabase (programName);

    alCfgDataBase = new alCfgDatabaseClass (dbPath, alCfgvresource);

    if ( (alCfgDataBase -> getInitErrors ()) != ALCFG_SUCCESS)
    {
	initErrors = ALCFG_ERROR;
	return;
    }
    groupList = new vlist;
    alCfgDataBase -> loadGroupList (groupList);

    printerList = new vlist;
    alCfgDataBase -> loadPrinterList (printerList);

    // init the alarm group tree list
    alarmTree = new alarmTreeClass (this);

    // store the default database path
    strncpy (dbasePath, dbPath, ALCFG_MAX_DATABASE_PATH);
    dbasePath [ALCFG_MAX_DATABASE_PATH] = 0;

    // init tisGalaxy
    tisGalaxyResource = startTisGalaxy ();
    initedTisGalaxy = vTRUE;

    // load the main dialog
    vresource mainDialogRes = vresourceGet (alCfgvresource, 
					alcfg_alCfgMainDialog);
    alCfgMainDialogClass *mainDialog = new alCfgMainDialogClass (mainDialogRes);

    // store main dialog in application
    alCfgMainDialog = mainDialog;

    if ((mainDialog -> getLoadInitErrors ()) == vTRUE)
    {
	initErrors = ALCFG_ERROR;
	return;
    }

    // store main dialogs resource
    mainDialog -> storeMainsResource (mainDialogRes);

    // get and store titles dictionary resources in main dialog
    vresource titleRes = vresourceGet (alCfgvresource,
					alcfg_alCfgTitlesDict);

    mainDialog -> storeTitlesResource (titleRes);

    // get and store messages dictionary resources in main dialog
    vresource messagesRes = vresourceGet (alCfgvresource,
					alcfg_alCfgMessagesDict);

    mainDialog -> storeMessagesResource (messagesRes);

    // store ptr to main dialog in the database class
    alCfgDataBase -> storeMainDialog (mainDialog);

    // store ptr to application in the main dialog
    mainDialog -> storeAlCfgApp (this);

    mainDialog -> Open ();
}


/*******************************************************************
*
*    Alarm Config Application destructor
*
*******************************************************************/

alCfgApplication::~alCfgApplication (void)
{
    vlistIterator iterator;

    if (initedTisGalaxy == vTRUE)
    {
	stopTisGalaxy (tisGalaxyResource);
    }
    // delete some lists
    if (groupList != NULL)
    {
	if (groupListModified == vTRUE)
	{
	    alCfgDataBase -> writeGroupList (groupList);
	}
	// free up the memory for the group names in the list
	vstr *groupName;
	iterator.Start (groupList);
	while (iterator.Next ())
	{
	    groupName = (vstr *)(iterator.GetValue ());
	    vstrDestroy (groupName);
	}
	iterator.Finish ();
	delete groupList;
    }
    if (printerList != NULL)
    {
	if (printerListModified == vTRUE)
	{
		alCfgDataBase -> writePrinterList (printerList);
	}
	iterator.Start (printerList);
	while (iterator.Next ())
	{
	    PRINTER_ENTRY_PTR entry = (PRINTER_ENTRY_PTR)(iterator.GetValue ());
	    vmemFree ((void *)entry);
	}
	iterator.Finish ();
	delete printerList;
    }

    if (alarmTree != NULL)
    {
	delete alarmTree;
    }

    // delete the database stuff
    delete alCfgDataBase;

    // delete the main dialog
    if (alCfgMainDialog != NULL)
    {
	delete alCfgMainDialog;
    }
}



/*******************************************************************
*
*    Alarm Config add to the group list and resort list
*
*******************************************************************/

int alCfgApplication::addToGroupList (vstr *groupName)
{
    int rowCount = groupList -> GetRowCount ();
    if (rowCount > 0)
    {
	int found = vFALSE;
	vlistIterator iterator;
	iterator.Start (groupList);
	while (iterator.Next ())
	{
	    vchar *value = (vchar *)(iterator.GetValue ());
	    if ((vcharCompare (value, groupName)) == 0)
	    {
		found = vTRUE;
		break;
	    }
	}
	iterator.Finish ();
	if (found == vTRUE)
	{
	    return (1);
	}
    }
    groupList -> SetCellValue (rowCount, 0, (const void *)groupName);
    groupList -> Sort (vlistROW, 0, NULL);
    groupListModified = vTRUE;
    return (0);
}


/*******************************************************************
*
*    Alarm Config delete group from list 
*
*******************************************************************/

void alCfgApplication::deleteFromGroupList (vchar *groupName)
{
    int row;
    int foundIt = vFALSE;
    vlistIterator iterator;
    vstr *value;

    iterator.Start (groupList);
    while (iterator.Next())
    {
	row = iterator.GetRow ();
	value = (vstr *)(iterator.GetValue ());
	if ((vcharCompare (groupName, value)) == 0)
	{
	    foundIt = vTRUE;
	    break;
	}
    }
    iterator.Finish ();

    if (foundIt == vTRUE)
    {
	vstrDestroy (value);
	groupList -> RemoveRow (row);
	groupListModified = vTRUE;
    }
}


/*******************************************************************
*
*    Alarm Config add to printer list function
*
*******************************************************************/

int alCfgApplication::addToPrinterList (vchar *printerName)
{
    PRINTER_ENTRY_PTR entry;

    int rowCount = printerList -> GetRowCount ();
    if (rowCount > 0)
    {
	int found = vFALSE;
	vlistIterator iterator;
	iterator.Start (printerList);
	while (iterator.Next ())
	{
	    entry = (PRINTER_ENTRY_PTR)(iterator.GetValue ());
	    if ((vcharCompare (entry -> printerGroupName, printerName)) == 0)
	    {
		found = vTRUE;
		break;
	    }
	}
	iterator.Finish ();
	if (found == vTRUE)
	{
	    return (1);
	}
    }

    entry = (PRINTER_ENTRY_PTR)(vmemAllocAndClear (sizeof (struct printerEntry)));
    vcharCopyBounded (printerName, entry -> printerGroupName, AL_GROUPNAME_SIZE);
    entry -> printerGroupName[AL_GROUPNAME_SIZE] = 0;
    printerList -> SetCellValue (rowCount, 0, (const void *)entry);
    printerList -> Sort (vlistROW, 0, alCfgComparePrinters);
    printerListModified = vTRUE;
    return (0);
}


/*******************************************************************
*
*    Alarm Config delete printer from list and resort list
*
*******************************************************************/

void alCfgApplication::deleteFromPrinterList (vchar *printerName)
{
    int row;
    int foundIt = vFALSE;
    vlistIterator iterator;
    PRINTER_ENTRY_PTR entry;

    iterator.Start (printerList);
    while (iterator.Next())
    {
	row = iterator.GetRow ();
	entry = (PRINTER_ENTRY_PTR)(iterator.GetValue ());
	if ((vcharCompare (entry -> printerGroupName, printerName)) == 0)
	{
	    foundIt = vTRUE;
	    break;
	}
    }
    iterator.Finish ();

    if (foundIt == vTRUE)
    {
	vmemFree ((void *)entry);
	printerList -> RemoveRow (row);
	printerListModified = vTRUE;
    }
}


/*******************************************************************
*
*    Alarm Config return a list of print group names
*	the list passed has already been allocated, caller
*	will delete
*
*******************************************************************/

void alCfgApplication::getPrinterNamesList (vlist *list)
{
    int row = 0;
    vlistIterator iterator;
    PRINTER_ENTRY_PTR entry;

    iterator.Start (printerList);
    while (iterator.Next())
    {
	entry = (PRINTER_ENTRY_PTR)(iterator.GetValue ());
	list -> SetCellValue (row, 0, (const void *)(&entry -> printerGroupName[0]));
	row++;
    }
    iterator.Finish ();
}

