#ifndef	alarmconfigHXX
#define	alarmconfigHXX

#include "textFilters.hxx"
#include "tisgalaxylib.hxx"

//** SOME DEFINES *************************************************************

// define the max database path size
#ifndef WIN32
#define	ALCFG_MAX_DATABASE_PATH	MAXPATHLEN
#else
#define ALCFG_MAX_DATABASE_PATH	_MAX_PATH
#endif WIN32


//** ENUM DEFINITIONS *********************************************************
enum {
    ALCFG_ERROR = -1,
    ALCFG_SUCCESS,
    ALCFG_TRUE,
    ALCFG_FALSE,
    ALCFG_NUMMISCENUMS
};


//** ERROR and MESSAGES TO GO TO NOTICE DIALOGS *****************************
//** return values and also used to produce info message notices,
//** see alCfgUtils.cxx/alCfgMessageCenter ()
enum {
    ALCFG_QUIT_MSG = ALCFG_NUMMISCENUMS,
    ALCFG_TREEFILE_ERR,
    ALCFG_INFOFILE_ERR,
    ALCFG_PRINTERFILE_ERR,
    ALCFG_GROUPFILE_ERR,
    ALCFG_GROUP_EXISTS_ERR,
    ALCFG_DELETE_GRP_REQ,
    ALCFG_GRP_INTREE_ERR,
    ALCFG_NO_DELETE_MAIN_ERR,
    ALCFG_NOTHING_SELECTED_ERR,
    ALCFG_NO_NODE_SEL_ERR,
    ALCFG_INSERT_GRP_TREE_REQ,
    ALCFG_PRINTER_EXISTS_ERR,
    ALCFG_DATAMOD_MSG,
    ALCFG_DELETE_PRT_REQ,
    ALCFG_CLIPBOARD_ACTIVE_MSG,
    ALCFG_NOCUT_MAIN_ERR,
    ALCFG_DELETE_PARAMS_REQ,
    ALCFG_CLIP_QUIT_MSG,
    ALCFG_INIT_OPERPRIV_ERR,
    ALCFG_NUMERR_MSGS
};

//** TYPEDEFS ****************************************************************

typedef struct printerEntry
{
    vchar	printerGroupName [AL_GROUPNAME_SIZE + 1];
    vchar	groupPrinters [MAX_PRINTERS_PER_GROUP][AL_PATHNAME_SIZE + 1];
} PRINTER_ENTRY, *PRINTER_ENTRY_PTR;


class alCfgMainDialogClass;
class alCfgDatabaseClass;
class alarmTreeClass;

//** APPLICATION CLASS DEFINITION *********************************************

class	alCfgApplication
{
    vapplication		*alCfgvapplication;
    vresource			alCfgvresource;
    vresource			tisGalaxyResource;
    alCfgMainDialogClass	*alCfgMainDialog;
    alCfgDatabaseClass		*alCfgDataBase;

    // the alarm group tree
    alarmTreeClass		*alarmTree;

    // database path to use
    char dbasePath [ALCFG_MAX_DATABASE_PATH + 1];

    // flags
    int	initErrors;
    int	initedTisGalaxy;

    // group list
    vlist		*groupList;
    int			groupListModified;

    // printer list
    vlist		*printerList;
    int			printerListModified;

public:

    // class constructor
    alCfgApplication (char *programName, char *dbpath);
    // class destructor
    ~alCfgApplication (void);

    // function to return the value of initError
    int getInitErrors (void) 
	{ return (initErrors); }

    // function to return alcfg's's application
    vapplication *getAlCfgvapplication (void) 
	{ return (alCfgvapplication); }
    // function to return the applications resource
    vresource getAlCfgvresource (void) 
	{ return (alCfgvresource); }
    // return the tis galaxy resource
    vresource getTisGalaxyResource (void)
	{ return (tisGalaxyResource); }

    // function to return the database class
    alCfgDatabaseClass *getDatabaseClass (void) 
	{ return (alCfgDataBase); }

    // function to handle alCfgMainDialog
    alCfgMainDialogClass *getMainDialog (void)
	{ return (alCfgMainDialog); }

    // function to return alarmTree
    alarmTreeClass *getAlarmTree (void)
	{ return (alarmTree); }

    // function to handle groupList
    vlist *getGroupList (void)
	{ return (groupList); }
    int addToGroupList (vstr *groupName);
    void deleteFromGroupList (vchar *groupName);

    // function to handle printerList
    vlist *getPrinterList (void)
	{ return (printerList); }
    void getPrinterNamesList (vlist *list);
    int addToPrinterList (vchar *printerName);
    void deleteFromPrinterList (vchar *printerName);
    void setPrinterListModified (int mod)
	{ printerListModified = mod; }

};

#endif	alarmconfigHXX
