/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <malloc.h>
#include <memory.h>
#include <dbDefs.h>
#include <tsDefs.h>
#include <tisAlarm.h>
#include <alarmDefs.h>
#include "alarmq.h"

/* Functions defined in this module */

#if defined(__STDC__) || defined(WIN32)
void	aqInit (AlarmQueue *, long);
int	aqCurrentSize (AlarmQueue *);
int	aqInsertAlarm (AlarmQueue *, tisAlarm *);
int	aqRemoveHead (AlarmQueue *, tisAlarm *);
int	aqEmpty (AlarmQueue *);
#else
void	aqInit ();
int	aqCurrentSize ();
int	aqInsertAlarm ();
int	aqRemoveHead ();
int	aqEmpty ();
#endif

/* Local data */

static	int		aqInitialized = 0;
static	AlarmEntry	*freeList;

/*----------*/
void aqInit (aq, size)	/* Initialize a alarm queue to empty */

    AlarmQueue	*aq;
    long	size;

    {

	AlarmEntry	*p;
	int		i;

	/* If the alarmq mechanisn has not been initialized, create a big list of alarmq nodes */

	if (!aqInitialized) {
	    freeList = NULL;
	    for (i=0 ; i<size ; i++) {
		p = (AlarmEntry *) malloc (sizeof (AlarmEntry));
		p->next = freeList;
		freeList = p;
	    }
	    aqInitialized = 1;
	}
 
	aq->head = aq->tail = NULL;
	aq->currentSize = 0;
	aq->maxSize = size;

    }

/*----------*/
int aqCurrentSize (aq)

    AlarmQueue	*aq;

    {

	return aq->currentSize;

    }

/*----------*/
int aqInsertAlarm (aq, alarmMsg)

    AlarmQueue	*aq;
    tisAlarm	*alarmMsg;

    {

	AlarmEntry	*p;

	if (freeList == NULL)
	    return 0;
	p = freeList;
	freeList = freeList->next;

	memcpy (&(p->alarmMessage), alarmMsg, sizeof (tisAlarm));

	p->next = NULL;
	if (aq->head == NULL)
	    aq->head = p;
	else
	    aq->tail->next = p;
	aq->tail = p;

	aq->currentSize++;

	return 1;

    }

/*----------*/
int aqRemoveHead (aq, alarmMsg)

    AlarmQueue	*aq;
    tisAlarm	*alarmMsg;

    {

	AlarmEntry	*p;

	if (aq->head == NULL)
	    return 0;

	p = aq->head;
	aq->head = aq->head->next;

	memcpy (alarmMsg, &(p->alarmMessage), sizeof (tisAlarm));

	p->next = freeList;
	freeList = p;

	aq->currentSize--;

	return 1;

    }

/*----------*/
int aqEmpty (aq)

    AlarmQueue	*aq;

    {

	return aq->head == NULL;

    }


