/* Send an alarm to the printer */

#include <stdio.h>
#if !(defined (SOLARIS) || defined (AXP) || defined(WIN32))
#include <sys/stdtypes.h>
#endif
#include <sys/types.h>
#include <time.h>
#ifndef WIN32
#include <sys/time.h>
#include <signal.h>
#endif WIN32
#include <string.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include <alarmDefs.h>
#include <shareMem.h>
#include <groupInfo.h>

#include "msgMon.h"
#include <cadef.h>
#include <dbDefs.h>
#include <db_access.h>
#include <paramStructs.h>
#include <tsDefs.h>
#include <alarm.h>
#include <tisAlarm.h>
#include "../common/dbase.h"
#include "alarmq.h"

#include "globals.h"
#include "shareMem.h"
#include "sharedMem.h"
#include "tree.h"
#include "points.h"
#include <messages.h>

#define	CONN_ALARM			ALARM_NSEV

/* Functions defined in this module */

#if defined(__STDC__) || defined(WIN32)
	int	printAlarm (PvData *, LogEntry *);
	int	printAck (PvData *, LogEntry *);
	int	printGrpAck (TreeNode *, LogEntry *);
static	int	printMessage (PvData *, LogEntry *, int);
#else
	int	printAlarm ();
	int	printAck ();
	int	printGrpAck ();
static	int	printMessage ();
#endif

/* External functions */

extern	int		sendMsgToPrinter ();
extern	void		stringCopy ();

/*----------*/
int printAlarm (p, logEntry)
     PvData *p;
     LogEntry *logEntry;

    {

	return printMessage (p, logEntry, 0);

    }

/*----------*/
int printAck (p, logEntry)
     PvData *p;
     LogEntry *logEntry;

    {

	return printMessage (p, logEntry, 1);

    }


/*----------*/
static int printMessage (p, logEntry, isAck)
     PvData *p;
     LogEntry *logEntry;
     int isAck;

    {

	char		msg [300], statusStr [50];
	char		dateStr [20], timeStr [20], printer [50];
	time_t		t;
	struct tm	*tim;

	/* Get the appropriate printer */

	TreeNode *group = p->parentGroup;
	strcpy (printer, "");
	if (group->auxValue) {	/* In aux mode */
	    stringCopy (printer, group->groupParms.auxPrinters [p->severity], sizeof (printer));
	    if ((*printer == '\0') || (strcmp (printer, "*") == 0))
		return 0;
	} else {	/* In normal mode */
	    stringCopy (printer, group->groupParms.printers [p->severity], sizeof (printer));
	    if ((*printer == '\0') || (strcmp (printer, "*") == 0))
		return 0;
	}

	/* Put the alarm time in a nice human readable format */

	t = logEntry->logTime.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
	tim = localtime (&t);
	sprintf (dateStr, "%02d/%02d/%02d", tim->tm_mon + 1, tim->tm_mday, tim->tm_year % 100);
	sprintf (timeStr, "%02d:%02d:%02d", tim->tm_hour, tim->tm_min, tim->tm_sec);

	/* Determine the status string */

	if (logEntry->severity == CONN_ALARM)
	    strcpy (statusStr, "CONNECT ERR");
	else if ((logEntry->severity == EVENT_ALARM) && (logEntry->status == 0))
	    strcpy (statusStr, "");
	else
	    stringCopy (statusStr, logEntry->status, sizeof (statusStr));

	/* Format the alarm message */

	sprintf (msg, "%-*.*s  %-*.*s  %-*.*s  %-*.*s  %-*.*s  %-*.*s %-*.*s %-*.*s\n",
			AL_DATE_SIZE, AL_DATE_SIZE, dateStr,
			AL_TIME_SIZE, AL_TIME_SIZE, timeStr,
			AL_POINTNAME_SIZE, AL_POINTNAME_SIZE, logEntry->pointName,
			AL_DESCRIPTOR_SIZE, AL_DESCRIPTOR_SIZE, logEntry->descriptor,
			AL_STATUS_SIZE, AL_STATUS_SIZE, statusStr,
			PRINT_VALUE_SIZE, PRINT_VALUE_SIZE, logEntry->value,
			AL_EGUTAG_SIZE, AL_EGUTAG_SIZE, logEntry->eguTag,
			PRINT_OP_SIZE, PRINT_OP_SIZE, isAck ? logEntry->oper : "");

	return sendMsgToPrinter (printer, msg);

}

/*----------*/
int printGrpAck (tnode, logEntry)
     TreeNode *tnode;
     LogEntry *logEntry;

    {

	char		msg [300], statusStr [50];
	char		dateStr [20], timeStr [20], printer [50];
	time_t		t;
	struct tm	*tim;

	/* Get the appropriate printer */

	strcpy (printer, "");
	if (tnode->auxValue) {	/* In aux mode */
	    stringCopy (printer, tnode->groupParms.auxPrinters [NO_ALARM], sizeof (printer));
	    if ((*printer == '\0') || (strcmp (printer, "*") == 0))
		return 0;
	} else {	/* In normal mode */
	    stringCopy (printer, tnode->groupParms.printers [NO_ALARM], sizeof (printer));
	    if ((*printer == '\0') || (strcmp (printer, "*") == 0))
		return 0;
	}

	/* Put the alarm time in a nice human readable format */

	t = logEntry->logTime.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
	tim = localtime (&t);
	sprintf (dateStr, "%02d/%02d/%02d", tim->tm_mon + 1, tim->tm_mday, tim->tm_year % 100);
	sprintf (timeStr, "%02d:%02d:%02d", tim->tm_hour, tim->tm_min, tim->tm_sec);

	/* Determine the status string */

	if (logEntry->severity == CONN_ALARM)
	    strcpy (statusStr, "CONNECT ERR");
	else if ((logEntry->severity == EVENT_ALARM) && (logEntry->status == 0))
	    strcpy (statusStr, "");
	else
	    stringCopy (statusStr, logEntry->status, sizeof (statusStr));

	/* Format the alarm message */

	sprintf (msg, "%-*.*s  %-*.*s  %-*.*s  %-*.*s  %-*.*s  %-*.*s %-*.*s %-*.*s\n",
			AL_DATE_SIZE, AL_DATE_SIZE, dateStr,
			AL_TIME_SIZE, AL_TIME_SIZE, timeStr,
			AL_POINTNAME_SIZE, AL_POINTNAME_SIZE, logEntry->pointName,
			AL_DESCRIPTOR_SIZE, AL_DESCRIPTOR_SIZE, logEntry->descriptor,
			AL_STATUS_SIZE, AL_STATUS_SIZE, statusStr,
			PRINT_VALUE_SIZE, PRINT_VALUE_SIZE, logEntry->value,
			AL_EGUTAG_SIZE, AL_EGUTAG_SIZE, logEntry->eguTag,
			PRINT_OP_SIZE, PRINT_OP_SIZE, logEntry->oper);

	return sendMsgToPrinter (printer, msg);

}
