/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include <alarmDefs.h>
#include <groupInfo.h>
/*#include <groupList.h> */
#include "printerGroupList.h"

#include <tsDefs.h>
#define	CONFIG
#include <dbase.h>

/* Functions defined in this module */

#if defined(__STDC__) || defined(WIN32)
#define	VOID	void
	void			PrinterGroupListInit (void);
	void			writePrinterGroups (void);
	int			addPrinterGroup (PrinterGroupEntry *);
static	int			compare (const void *, const void *);
	void			removePrinterGroup (char *);
	int			inPrinterGroupList (char *);
static	int			locatePrinterGroup (char *);
	char			*getPrinterGroupList (int *);
	PrinterGroupEntry	*getPrinterGroup (char *);
	void			setPrinterGroupModified (void);
#else
#define	VOID
	void			PrinterGroupListInit ();
	void			writePrinterGroups ();
	int			addPrinterGroup ();
static	int			compare ();
	void			removePrinterGroup ();
	int			inPrinterGroupList ();
static	int			locatePrinterGroup ();
	char			*getPrinterGroupList ();
	PrinterGroupEntry	*getPrinterGroup ();
	void			setPrinterGroupModified ();
#endif

static	int			numGroups;
static	PrinterGroupEntry	groups [MAX_PRINTER_GROUPS];
static	int			modified;

/*----------*/
void printerGroupListInit (VOID)	/* Load the group list from the dBASE file */

   {

	/* Load groups from the dBASE file. Note that the list was sorted when it */
	/* was written to the database so it will end up sorted in the list. */

	numGroups = 0;
	if (!openPrinterFile ())
	    createPrinterFile ();

	while (getNextPrinterGroup (&groups [numGroups]))
	    numGroups++;
	modified = 0;

	/* The group file is left open so that it's modification date is not changed */
	/* until the program terminates. This keep alarmMgr from reconfiguring when */
	/* alarmConfig starts up. */

    }

/*----------*/
void writePrinterGroups (VOID)

    {

	int	i;

	closePrinterFile ();
	if (modified) {
 	    createPrinterFile ();    
	    for (i=0 ; i<numGroups ; i++)
		writePrinterGroup (&groups [i]);
	    closePrinterFile ();
	}

    }

/*----------*/
int addPrinterGroup (p)	/* Add a new group to a group list */

     PrinterGroupEntry *p;

    {

	if (numGroups == MAX_PRINTER_GROUPS)
	    return 0;

	/* Place the new group in at the end of the list */

	memcpy ((char *)&groups [numGroups++], (char *)p, sizeof (PrinterGroupEntry));
	modified = 1;

	qsort ((void *)&groups, numGroups, sizeof (PrinterGroupEntry), compare);
	return 1;

    }

/*----------*/
static int compare (e1, e2)

    const void	 *e1;
    const void	*e2;

    {

	PrinterGroupEntry *g1 = (PrinterGroupEntry *)e1, *g2 = (PrinterGroupEntry *)e2;
	return strcmp (g1->name, g2->name);

    }

/*----------*/
void removePrinterGroup (groupName)	/* Remove a group from a group list */

    char *groupName;

    {

	int	i;

	/* Find the index of the group */

	i = locatePrinterGroup (groupName);
	if (i == -1)
	    return;

	/* Slide all entries below this up */

	numGroups--;
	while (i < numGroups) {
	    groups [i] = groups [i+1];
	    i++;
	}
	modified = 1;

    }

/*----------*/
int inPrinterGroupList (groupName)		/* Find a group */

    char	*groupName;

    {

	return locatePrinterGroup (groupName) != -1;

    }

/*----------*/
static int locatePrinterGroup (groupName)	/* Find the index of an entry in a group list */

    char	*groupName;

    {

	int	i;

	for (i=0 ; i<numGroups ; i++)
	    if (strcmp (groupName, groups [i].name) == 0)
		return i;
	return -1;

    }

/*----------*/
char *getPrinterGroupList (num)

    int		*num;

    {

	int	i, stringBytes;
	char	*p, *q;

	*num = 0;
	stringBytes = 0;
	for (i=0 ; i<numGroups ; i++)
	    stringBytes += strlen (groups [i].name) + 1;

	p = q = malloc (stringBytes + 1);
	for (i=0 ; i<numGroups ; i++) {
	    strcpy (q, groups [i].name);
	    q += strlen (groups [i].name) + 1;
	    (*num)++;
	}
	*q = '\0';
	return p;

    }

/*----------*/
PrinterGroupEntry *getPrinterGroup (groupName)

    char	*groupName;

    {

	int	i;

	i = locatePrinterGroup (groupName);
	if (i == -1)
	    return NULL;
	return &groups [i];

    }


/*----------*/
void setPrinterGroupModified (VOID)

    {

	modified = 1;

    }

