/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <sys/stat.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include <alarm.h>
#include <alarmDefs.h>
#include <shareMem.h>
#include <groupInfo.h>
#include "points.h"
#include "globals.h"

#include <tsDefs.h>
#include <dbase.h>
#include <cadef.h>

#include <ChannelList.h>

/* Functions defined in this module */

#define	VOID	void
	void		initializeTree (void);
	void		terminateTree (void);
	TreeNode	*allocateNode (void);
static	int		sortCompare (const void *, const void *);
	TreeNode	*findNode (char *);
static	int		findCompare (const void *, const void *);
	void		addChild (TreeNode *, TreeNode *);
	int		loadTree (void);
static	void		installParms (TreeNode *, GroupInfo *);
static	void		propogateParameters (TreeNode *);
	void		tallyCounts (VOID);
static	struct GroupData tallyTraverse (TreeNode *);
static	void		addGroupData (struct GroupData *, struct GroupData *);
	void		updateAuxSelect (void);
static	void		updateAuxSelectTrav (TreeNode *);
	void		dumpTree (void);
static	void		dumpTrav (TreeNode *);

/* Local data */

static	TreeNode	*treeRoot;

	long		treeFileTime;
static	char		*treeSegmentAllocationLimit;
static	TreeNode	**nodeList = NULL;	/* List of pointers to node that is sorted so that */
						/* a binary search can be used to find a node. */
static	int		numNodes;		/* Number of nodes in tree. */
static	TreeNode	**nextNode;

/* externals */
extern void addPrinter (char *printerName);
/*----------*/
void initializeTree (VOID)		/* Initialize tree segment */

    {

	treeSegmentAllocationLimit = (char *)shareHeader->treeRoot; /* Initialized by sharedMem.C */
	treeRoot = shareHeader->treeRoot;
	numNodes = 0;
	if (nodeList)
	    free ((char *)nodeList);
	nodeList = nextNode = (TreeNode **)malloc (10000 * sizeof (TreeNode));

    }

/*----------*/
void terminateTree (VOID)		/* Terminate the tree mechanism */

    {

	free ((char *)nodeList);

    }

/*----------*/
TreeNode *allocateNode (VOID)		/* Allocate a new node from the node pool */

    {

	TreeNode	*p;

	p = (TreeNode *)treeSegmentAllocationLimit;
	treeSegmentAllocationLimit += sizeof (TreeNode);
	memset ((char *)p, 0, sizeof (TreeNode));
	*nextNode++ = p;
	numNodes++;
	return p;

    }

/*----------*/
static int sortCompare (p1, p2)

    const void *p1;
    const void *p2;

    {

	TreeNode *t1 = *(TreeNode **)p1;
	TreeNode *t2 = *(TreeNode **)p2;
	return strcmp (t1->groupName, t2->groupName);

    }

/*----------*/
TreeNode *findNode (nodeName)
     char *nodeName;	/* Locate a node in the tree */

    {
	
	TreeNode **p = (TreeNode **)bsearch (nodeName, (char *)nodeList, numNodes, sizeof (TreeNode *), findCompare);
	return p ? *p : NULL;

    }

/*----------*/
int findCompare (p1, p2)

    const void	*p1;
    const void	*p2;

    {

	TreeNode *t = *(TreeNode **)p2;
	return strcmp ((char *)p1, t->groupName);

    }

/*----------*/
void addChild (parent, child)
     TreeNode *parent;
     TreeNode *child;

    {

	child->rightSibling = parent->leftChild;
	parent->leftChild = child;
	child->parent = parent;
	
    }
	    
/*----------*/
int loadTree (VOID)

    {

	GroupName parentName, childName;
	GroupInfo groupParms;
	struct stat status;

	/* Read through the tree file and allocate all the nodes. */

	openTreeFile ();
	openInfoFile ();
	while (getNextTreeEdge (parentName, childName)) {
	    TreeNode *p = allocateNode ();
	    p->groupName = addNewString (childName);
	    getInfo (childName, &groupParms);
	    installParms (p, &groupParms);
	}
	closeInfoFile ();
	closeTreeFile ();

	/* Sort the array of node pointers */

	qsort ((char *)nodeList, numNodes, sizeof (TreeNode *), sortCompare);

	/* Read through the tree file and build the tree. */

	openTreeFile ();
	while (getNextTreeEdge (parentName, childName)) {
	    TreeNode *p = findNode (childName);
	    if (strlen (parentName) == 0) {		/* Root node */
		shareHeader->treeRoot = p;
	    } else {
	    	TreeNode *parent = findNode (parentName);
		addChild (parent, p); 
	    }
	}
	closeTreeFile ();

	/* Propogate parameters down the tree */

	propogateParameters (treeRoot);

	/* Get the file date/time of the tree file */

	stat (AL_TREE_FILE_NAME, &status);
	treeFileTime = status.st_mtime;

	return 1;

    }

/*----------*/
static void installParms (p, info)

    TreeNode	*p;
    GroupInfo	*info;

    {

	int	i;

	p->groupParms.autoScript = addNewString (info->autoScript);
	p->groupParms.auxAutoScript = addNewString (info->auxAutoScript);
	p->groupParms.manualScript = addNewString (info->manualScript);
	p->groupParms.auxManualScript = addNewString (info->auxManualScript);
	p->groupParms.displayName = addNewString (info->displayName);
	p->groupParms.helpTopic = addNewString (info->helpTopic);
	p->groupParms.auxSelectPV = addNewString (info->auxSelectPV);
	p->groupParms.guidance = addNewString (info->guidance);

	for (i=0 ; i<AL_NUM_SEVR ; i++) {

	    p->groupParms.printers [i] = addNewString (info->printers [i]);
	    if (strlen (info->printers [i])) {
		if (strcmp (info->printers [i], "*") != 0)
		    addPrinter (info->printers [i]);
	    }

	    p->groupParms.auxPrinters [i] = addNewString (info->auxPrinters [i]);
	    if (strlen (info->auxPrinters [i])) {
		if (strcmp (info->auxPrinters [i], "*") != 0)
		    addPrinter (info->auxPrinters [i]);
	    }

	    p->groupParms.sounds [i] = addNewString (info->sounds [i]);
	    p->groupParms.soundRates [i] = atoi (info->rates [i]);
	    p->groupParms.auxSounds [i] = addNewString (info->auxSounds [i]);
	    p->groupParms.auxSoundRates [i] = atoi (info->auxRates [i]);

	}

    }

/*----------*/
static void propogateParameters (p)

    TreeNode	*p;

    {

	int	i;

	/* Perform a depth-first traverse of the tree. At each node other than the */
	/* root check check to see if each parameter is defined.  If a parameter is not */
	/* defined, grab it from the parent. */

	if (p == NULL)
	    return;

	if (p->parent) {	/* Not the root node */

	    /* Check each parameter to see if it is defined. If it is not defined then grab */
	    /* it from our parent. */

	    if (*(p->groupParms.autoScript) == '\0')
		p->groupParms.autoScript = p->parent->groupParms.autoScript;

	    if (*(p->groupParms.auxAutoScript) == '\0')
		p->groupParms.auxAutoScript = p->parent->groupParms.auxAutoScript;

	    if (*(p->groupParms.manualScript) == '\0')
		p->groupParms.manualScript = p->parent->groupParms.manualScript;

	    if (*(p->groupParms.auxManualScript) == '\0')
		p->groupParms.auxManualScript = p->parent->groupParms.auxManualScript;

	    if (*(p->groupParms.displayName) == '\0')
		p->groupParms.displayName = p->parent->groupParms.displayName;

	    if (*(p->groupParms.helpTopic) == '\0')
		p->groupParms.helpTopic = p->parent->groupParms.helpTopic;

	    if (*(p->groupParms.guidance) == '\0')
		p->groupParms.guidance = p->parent->groupParms.guidance;

	    if (*(p->groupParms.auxSelectPV) == '\0')
		p->groupParms.auxSelectPV = p->parent->groupParms.auxSelectPV;
	    if (*(p->groupParms.auxSelectPV) != '\0')
		p->chanNum = clAdd (p->groupParms.auxSelectPV);

	    for (i=0 ; i<AL_NUM_SEVR ; i++) {

		if (*(p->groupParms.printers [i]) == '\0')
		    p->groupParms.printers [i] = p->parent->groupParms.printers [i];

		if (*(p->groupParms.auxPrinters [i]) == '\0')
		    p->groupParms.auxPrinters [i] = p->parent->groupParms.auxPrinters [i];

		if (*(p->groupParms.sounds [i]) == '\0')
		    p->groupParms.sounds [i] = p->parent->groupParms.sounds [i];

		if (p->groupParms.soundRates [i] == 0)
		    p->groupParms.soundRates [i] = p->parent->groupParms.soundRates [i];

		if (*(p->groupParms.auxSounds [i]) == '\0')
		    p->groupParms.auxSounds [i] = p->parent->groupParms.auxSounds [i];

		if (p->groupParms.auxSoundRates [i] == 0)
		    p->groupParms.auxSoundRates [i] = p->parent->groupParms.auxSoundRates [i];

	    }

	} else {	/* Root node gets special attention for auxSelectPV */
	    if (*(p->groupParms.auxSelectPV) != '\0')
		p->chanNum = clAdd (p->groupParms.auxSelectPV);
	}

	propogateParameters (p->leftChild);
	propogateParameters (p->rightSibling);

    }

/*----------*/
void tallyCounts (VOID)	/* Determine the counts for each group */

    {

	tallyTraverse (treeRoot);
/*	dumpTree (); */

    }

/*----------*/
static struct GroupData tallyTraverse (p)

    TreeNode	*p;

    {

	struct GroupData	groupData, subTreeData;
	TS_STAMP	timeNow;
	PvData *q;

	/* Initialize all counts to zero. */

	memset ((char *)&groupData, 0, sizeof (struct GroupData));

	/* Copy the last change date so that we can compare the current status */
	/* to the new one without a conflict on the timestamp. */

	groupData.lastStatusChange = p->groupData.lastStatusChange;

	/* Each group contains information about the number of outstanding alarms. */
	/* Recursively summarize each subtree. */

	if (p) {

	    /* Gather the statistics for the subtrees rooted by this node */

	    TreeNode *t;
	    t = p->leftChild;
	    while (t) {
		subTreeData = tallyTraverse (t);
		addGroupData (&groupData, &subTreeData);
		t = t->rightSibling;
	    }
	    
	}

	/* Add into the counts received from the subtree the count for the points */
	/* associated with this group. Traverse the list of points for this node that */
	/* are on the alarm summary list. */

	q = p->pvList;	
	while (q) {

	    /* If this pv is not on the alarm summary list then it is not in alarm */

	    if (!inSummaryList (q)) {
		q = q->nextPv;
		continue;
	    }

	    /* Point is in alarm */

	    if (q->RTN) {	/* RTN but not acked */
		groupData.RTNnotACK [q->unAckedSeverity]++;
		groupData.numUnackedBySeverity [q->unAckedSeverity]++;
		groupData.numAlarmsBySeverity [q->unAckedSeverity]++;
	    } else {
		if (q->Ack) {	/* Not RTN but ACKed */
		    groupData.notRTNandACK [q->unAckedSeverity]++;
		    groupData.numAlarmsBySeverity [q->unAckedSeverity]++;
		} else {	/* Not RTN and not ACKed */
		    groupData.notRTNnotACK [q->unAckedSeverity]++;
		    groupData.numUnackedBySeverity [q->unAckedSeverity]++;
		    groupData.numAlarmsBySeverity [q->unAckedSeverity]++;
		}
	    }

	    /* Tally up the status */

	    groupData.curStat [q->unAckedStatus] ++;

	    q = q->nextPv;

	}

	/* Set the group status and severity */

	groupData.groupSeverity = 0;
	groupData.groupStatus = 0;
	if (groupData.notRTNnotACK [MAJOR_ALARM]) {
	    groupData.groupSeverity = 9;
	    groupData.groupStatus = 3;
	} else if (groupData.notRTNnotACK [MINOR_ALARM]) {
	    groupData.groupSeverity = 8;
	    groupData.groupStatus = 3;
	} else if (groupData.notRTNnotACK [INFO_ALARM]) {
	    groupData.groupSeverity = 7;
	    groupData.groupStatus = 3;
	} else if (groupData.RTNnotACK [MAJOR_ALARM]) {
	    groupData.groupSeverity = 6;
	    groupData.groupStatus = 2;
	} else if (groupData.RTNnotACK [MINOR_ALARM]) {
	    groupData.groupSeverity = 5;
	    groupData.groupStatus = 2;
	} else if (groupData.RTNnotACK [INFO_ALARM]) {
	    groupData.groupSeverity = 4;
	    groupData.groupStatus = 2;
	} else if (groupData.notRTNandACK [MAJOR_ALARM]) {
 	    groupData.groupSeverity = 3;
	    groupData.groupStatus = 1;
	} else if (groupData.notRTNandACK [MINOR_ALARM]) {
	    groupData.groupSeverity = 2;
	    groupData.groupStatus = 1;
	} else if (groupData.notRTNandACK [INFO_ALARM]) {
	    groupData.groupSeverity = 1;
	    groupData.groupStatus = 1;
	}

	/* Update the counts for this group */

	if (memcmp (&(p->groupData), &groupData, sizeof (groupData)) != 0) {
	    tsLocalTime (&timeNow);
	    memcpy ((char *)&groupData.lastStatusChange, (char *)&timeNow, sizeof (TS_STAMP));
	    p->groupData = groupData;
	}

	return groupData;

    }

/*----------*/
static void addGroupData (a, b)  /* Add b into a */

    struct GroupData	*a;
    struct GroupData	*b;

    {

	int	i;

	for (i=0 ; i<AL_NUM_SEVR ; i++) {
	    a->numAlarmsBySeverity [i] += b->numAlarmsBySeverity [i];
	    a->numUnackedBySeverity [i] += b->numUnackedBySeverity [i];
	    a->notRTNnotACK [i] += b->notRTNnotACK [i];
	    a->RTNnotACK [i] += b->RTNnotACK [i];
	    a->notRTNandACK [i] += b->notRTNandACK [i];
	}

	for (i=0 ; i<ALARM_NSTATUS ; i++)
	    a->curStat [i] += b->curStat [i];

    }

/*----------*/
void updateAuxSelect (VOID)	/* Traverse the tree and update aux select PV values */

    {

	updateAuxSelectTrav (shareHeader->treeRoot);

    }

/*----------*/
static void updateAuxSelectTrav (p)

    TreeNode	*p;

    {

	if (p) {
	    p->auxValue = clGetValue (p->chanNum);
	    updateAuxSelectTrav (p->leftChild);
	    updateAuxSelectTrav (p->rightSibling);
	}

    }

/*----------*/
void dumpTree (VOID)	/* Traverse the tree and print node names */

    {

	dumpTrav (shareHeader->treeRoot);

    }

/*----------*/
static void dumpTrav (p)
     TreeNode *p;

    {

	if (p) {
	    printf ("%s chan=%d, aux=%d\n", p->groupName, p->chanNum, p->auxValue);
/*	    printf ("    al=%hd un=%hd unrtn=%hd \n",
			p->groupData.numAlarmsBySeverity [MAJOR_ALARM],
			p->groupData.numUnackedBySeverity [MAJOR_ALARM],
	    printf ("    al=%hd un=%hd unrtn=%hd\n",
			p->groupData.numAlarmsBySeverity [MINOR_ALARM],
			p->groupData.numUnackedBySeverity [MINOR_ALARM],
			p->groupData.numUnackRTNBySeverity [MINOR_ALARM]);
 	    printf ("    al=%hd un=%hd unrtn=%hd\n",
			p->groupData.numAlarmsBySeverity [INFO_ALARM],
			p->groupData.numUnackedBySeverity [INFO_ALARM],
			p->groupData.numUnackRTNBySeverity [INFO_ALARM]);
   	    printf ("   gstat=%hd, gsevr=%hd, aux=%hd\n", p->groupData.groupStatus,
			p->groupData.groupSeverity, p->auxValue);

	    pvData *q = p->pvList;
	    while (q) {
		printf ("   %s\n", q->pvName);
		printf ("       ack=%hd, RTN=%hd, stat=%hd, sevr=%hd\n",
			q->Ack, q->RTN, q->status, q->severity);
		q = q->nextPv;
	    }
*/
	    dumpTrav (p->leftChild);
	    dumpTrav (p->rightSibling);
	}

    }


