/*
**	alarmfmt.c - alarm object display formats
*/
#ifdef USEX11
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#endif USEX11

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#ifdef WIN32
#include <stdlib.h>
#endif WIN32
#ifdef AXP
#include <stdlib.h>
#endif AXP
#include "shareMem.h"
#include "time.h"
#include "alarmfil.h"
#include "alpvmap.h"
#include "alarmfmt.h"

extern char	*alarmStatusString[];

FORMAT	displayvalue =
    { NULL, "DISPLAYVALUE:", TYPEQUAL, NAMEQUAL,
      { { "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 0, MEMDATE },
	{ " TIME", 0, 0, MEMTIME	},
	{ " POINTNAME", 0, 28, MEMPOINT },
	{ " DESCRIPT", 48, 28, MEMDESCR },
	{ " STATUS", 76, 12, CASTATUS },
	{ " VALUE", 88, 12, CAVALUE },
	{ " EGUTAG", 100, 12, MEMTAG },
	{ " STATION", 0, 0, MEMSTATION }, 
	{ " ONSCAN", 0, 0, MEMOSCN },
	{ " AENABLE", 0, 0, MEMAENB },
	{ " GROUP", 0, 0, MEMGROUP },
	{ " ACKSTATUS", 0, 0, MEMACKSTATUS },
	{ " ACKDATE", 0, 0, MEMACKDATE },
	{ " ACKTIME", 0, 0, MEMACKTIME	},
	{ " OPERATOR", 0, 0, MEMACKOP },
	{ " GUIDANCE", 0, 0, MEMGUIDE },
	{ " POINTTYPE", 0, 0, PVMAPTYPE },
	{ " SCANTYPE", 0, 0, PVMAPSCANTYPE },
	{ " SCANRATE", 0, 0, PVMAPSCANRATE },
	{ " PHASE", 0, 0, PVMAPPHASE },
	{ " EVENTNUM", 0, 0, PVMAPEVENTNUM },
	{ " SYSTAGS", 0, 0, PVMAPSYSTAG },
	{ " INFOTAGS", 0, 0, PVMAPINFTAG },
	{ " PRIVGROUP", 0, 0, PVMAPPRIVGROUP },
	{ " DEVTYPE", 0, 0, PVMAPDEVTYPE },
	{ " TREND", 0, 0, PVMAPTREND },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandvalue =
    { NULL, "EXPANDVALUE:", TYPEQUAL, NAMEQUAL,
      { { "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, MEMDATE },
	{ " TIME", 10, 8, MEMTIME	},
	{ " POINTNAME", 20, 28, MEMPOINT },
	{ " DESCRIPT", 49, 28, MEMDESCR },
	{ " STATUS", 78, 12, CASTATUS },
	{ " VALUE", 100, 12, CAVALUE },
	{ " EGUTAG", 113, 12, MEMTAG },
	{ " STATION", 126, 12, MEMSTATION }, 
	{ " ONSCAN", 150, 4, MEMOSCN },
	{ " AENABLE", 155, 10, MEMAENB },
	{ " GROUP", 167, 28, MEMGROUP },
	{ " ACKSTATUS", 195, 7, MEMACKSTATUS },
	{ " ACKDATE", 205, 8, MEMACKDATE },
	{ " ACKTIME", 214, 8, MEMACKTIME	},
	{ " OPERATOR", 224, 30, MEMACKOP },
	{ " GUIDANCE", 0, 0, MEMGUIDE },
	{ " POINTTYPE", 0, 0, PVMAPTYPE },
	{ " SCANTYPE", 0, 0, PVMAPSCANTYPE },
	{ " SCANRATE", 0, 0, PVMAPSCANRATE },
	{ " PHASE", 0, 0, PVMAPPHASE },
	{ " EVENTNUM", 0, 0, PVMAPEVENTNUM },
	{ " SYSTAGS", 0, 0, PVMAPSYSTAG },
	{ " INFOTAGS", 0, 0, PVMAPINFTAG },
	{ " PRIVGROUP", 0, 0, PVMAPPRIVGROUP },
	{ " DEVTYPE", 0, 0, PVMAPDEVTYPE },
	{ " TREND", 0, 0, PVMAPTREND },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	displaygroup =
    { NULL, "DISPLAYGROUP:", GROUPQUAL, NOQUAL,
      { { "MAJ=", 29, 4, FLDTEXT },
	{ ",  MIN=", 35, 7, FLDTEXT },
	{ ",  EVENT=", 44, 9, FLDTEXT },
	{ ",  INFO=", 55, 8, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " GROUPNAME", 0, 28, GRPGROUP },
	{ " MAJOR", 33, 2, GRPMAJOR },
	{ " MINOR", 42, 2, GRPMINOR },
	{ " EVENT", 53, 2, GRPEVENT },
	{ " INFO", 63, 2, GRPINFO },
	{ " GUIDANCE", 0, 0, GRPGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandgroup =
    { NULL, "EXPANDGROUP:", GROUPQUAL, NOQUAL,
      { { "MAJOR ALARMS =", 30, 14, FLDTEXT },
	{ ",  MINOR ALARMS =", 48, 17, FLDTEXT },
	{ ",  EVENTS =", 67, 11, FLDTEXT },
	{ ",  INFORMATION ALARMS =", 80, 21, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " GROUPNAME", 0, 28, GRPGROUP },
	{ " MAJOR", 45, 2, GRPMAJOR },
	{ " MINOR", 65, 2, GRPMINOR },
	{ " EVENT", 78, 2, GRPEVENT },
	{ " INFO", 101, 2, GRPINFO },
	{ " GUIDANCE", 0, 0, GRPGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	displayack =
    { NULL, "DISPLAYACK:", FILETYPEQUAL, FILENAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
 	{ " DATE", 0, 8, FILEDATE },
	{ " TIME", 9, 8, FILETIME },
	{ " NOTEIND", 18, 1, FILENOTE },
	{ " POINTNAME", 19, 28, FILEPOINT },
	{ " DESCRIPT", 48, 28, FILEDESCR },
	{ " STATUS", 76, 12, FILESTATUS },
	{ " SEVERITY", 0, 0, FILESEVR },
	{ " VALUE", 88, 12, FILEVALUE },
	{ " EGUTAG", 100, 12, FILETAG },
	{ " ACKED", 0, 0, FILEACKED },
	{ " ACKDATE", 0, 0, FILEACKDATE },
	{ " ACKTIME", 0, 0, FILEACKTIME },
	{ " STATION", 112, 0, FILESTATION },
	{ " OPERATOR", 88, 12, FILEOPERATOR },
	{ " COMMENT", 0, 0, FILECOMMENT },
	{ " INFO1", 0, 0, FILEINFO1 },
	{ " INFO2", 0, 0, FILEINFO2 },
	{ " INFO3", 0, 0, FILEINFO3 },
	{ " INFO4", 0, 0, FILEINFO4 },
	{ " INFO5", 0, 0, FILEINFO5 },
	{ " GUIDANCE", 0, 0, FILEGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandack =
    { NULL, "EXPANDACK:", FILETYPEQUAL, FILENAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
 	{ " DATE", 0, 8, FILEDATE },
	{ " TIME", 9, 8, FILETIME	},
	{ " NOTEIND", 18, 1, FILENOTE },
	{ " POINTNAME", 19, 28, FILEPOINT },
	{ " DESCRIPT", 48, 28, FILEDESCR },
	{ " STATUS", 76, 12, FILESTATUS },
	{ " SEVERITY", 0, 0, FILESEVR },
	{ " VALUE", 88, 12, FILEVALUE },
	{ " EGUTAG", 100, 12, FILETAG },
	{ " ACKED", 120, 14, FILEACKED },
	{ " ACKDATE", 135, 8, FILEACKDATE },
	{ " ACKTIME", 144, 8, FILEACKTIME },
	{ " STATION", 154, 28, FILESTATION },
	{ " OPERATOR", 188, 12, FILEOPERATOR },
	{ " COMMENT", 205, 200, FILECOMMENT },
	{ " INFO1", 0, 0, FILEINFO1 },
	{ " INFO2", 0, 0, FILEINFO2 },
	{ " INFO3", 0, 0, FILEINFO3 },
	{ " INFO4", 0, 0, FILEINFO4 },
	{ " INFO5", 0, 0, FILEINFO5 },
	{ " GUIDANCE", 0, 0, FILEGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	printack =
    { NULL, "PRINTACK:", FILETYPEQUAL, FILENAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
 	{ " DATE", 0, 8, FILEDATE },
	{ " TIME", 9, 8, FILETIME	},
	{ " NOTEIND", 18, 1, FILENOTE },
	{ " POINTNAME", 19, 28, FILEPOINT },
	{ " DESCRIPT", 48, 28, FILEDESCR },
	{ " STATUS", 89, 12, FILESTATUS },
	{ " SEVERITY", 0, 0, FILESEVR },
	{ " VALUE", 88, 12, FILEVALUE },
	{ " EGUTAG", 100, 12, FILETAG },
	{ " ACKED", 0, 0, FILEACKED },
	{ " ACKDATE", 0, 0, FILEACKDATE },
	{ " ACKTIME", 0, 0, FILEACKTIME },
	{ " STATION", 112, 0, FILESTATION },
	{ " OPERATOR", 102, 12, FILEOPERATOR },
	{ " COMMENT", 0, 0, FILECOMMENT },
	{ " INFO1", 0, 0, FILEINFO1 },
	{ " INFO2", 0, 0, FILEINFO2 },
	{ " INFO3", 0, 0, FILEINFO3 },
	{ " INFO4", 0, 0, FILEINFO4 },
	{ " INFO5", 0, 0, FILEINFO5 },
	{ " GUIDANCE", 0, 0, FILEGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	displayalarm =
    { NULL, "DISPLAYALARM:", FILETYPEQUAL, FILENAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, FILEDATE },
	{ " TIME", 9, 8, FILETIME	},
	{ " NOTEIND", 18, 1, FILENOTE },
	{ " POINTNAME", 19, 28, FILEPOINT },
	{ " DESCRIPT", 48, 28, FILEDESCR },
	{ " STATUS", 76, 12, FILESTATUS },
	{ " SEVERITY", 0, 0, FILESEVR },
	{ " VALUE", 88, 12, FILEVALUE },
	{ " EGUTAG", 100, 12, FILETAG },
	{ " ACKED", 0, 0, FILEACKED },
	{ " ACKDATE", 0, 0, FILEACKDATE },
	{ " ACKTIME", 0, 0, FILEACKTIME },
	{ " STATION", 112, 0, FILESTATION },
	{ " OPERATOR", 102, 12, FILEOPERATOR },
	{ " COMMENT", 0, 0, FILECOMMENT },
	{ " INFO1", 0, 0, FILEINFO1 },
	{ " INFO2", 0, 0, FILEINFO2 },
	{ " INFO3", 0, 0, FILEINFO3 },
	{ " INFO4", 0, 0, FILEINFO4 },
	{ " INFO5", 0, 0, FILEINFO5 },
	{ " GUIDANCE", 0, 0, FILEGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandalarm =
    { NULL, "EXPANDALARM:", FILETYPEQUAL, FILENAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, FILEDATE },
	{ " TIME", 9, 8, FILETIME	},
	{ " NOTEIND", 18, 1, FILENOTE },
	{ " POINTNAME", 19, 28, FILEPOINT },
	{ " DESCRIPT", 48, 28, FILEDESCR },
	{ " STATUS", 76, 12, FILESTATUS },
	{ " SEVERITY", 0, 0, FILESEVR },
	{ " VALUE", 88, 12, FILEVALUE },
	{ " EGUTAG", 100, 12, FILETAG },
	{ " ACKED", 115, 14, FILEACKED },
	{ " ACKDATE", 130, 8, FILEACKDATE },
	{ " ACKTIME", 140, 8, FILEACKTIME },
	{ " OPERATOR", 150, 24, FILEOPERATOR },
	{ " STATION", 180, 28, FILESTATION },
	{ " COMMENT", 210, 200, FILECOMMENT },
	{ " INFO1", 0, 0, FILEINFO1 },
	{ " INFO2", 0, 0, FILEINFO2 },
	{ " INFO3", 0, 0, FILEINFO3 },
	{ " INFO4", 0, 0, FILEINFO4 },
	{ " INFO5", 0, 0, FILEINFO5 },
	{ " GUIDANCE", 0, 0, FILEGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };


FORMAT	printalarm =
    { NULL, "PRINTALARM:", FILETYPEQUAL, FILENAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, FILEDATE },
	{ " TIME", 9, 8, FILETIME	},
	{ " NOTEIND", 18, 1, FILENOTE },
	{ " POINTNAME", 19, 28, FILEPOINT },
	{ " DESCRIPT", 48, 28, FILEDESCR },
	{ " STATUS", 89, 12, FILESTATUS },
	{ " SEVERITY", 0, 0, FILESEVR },
	{ " VALUE", 102, 12, FILEVALUE },
	{ " EGUTAG", 115, 12, FILETAG },
	{ " ACKED", 0, 0, FILEACKED },
	{ " ACKDATE", 0, 0, FILEACKDATE },
	{ " ACKTIME", 0, 0, FILEACKTIME },
	{ " OPERATOR", 102, 12, FILEOPERATOR },
	{ " STATION", 127, 0, FILESTATION },
	{ " COMMENT", 0, 0, FILECOMMENT },
	{ " INFO1", 0, 0, FILEINFO1 },
	{ " INFO2", 0, 0, FILEINFO2 },
	{ " INFO3", 0, 0, FILEINFO3 },
	{ " INFO4", 0, 0, FILEINFO4 },
	{ " INFO5", 0, 0, FILEINFO5 },
	{ " GUIDANCE", 0, 0, FILEGUIDE },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	displaysummary =
    { NULL, "DISPLAYSUMMARY:", TYPEQUAL, NAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, MEMDATE },
	{ " TIME", 9, 8, MEMTIME	},
	{ " POINTNAME", 19, 28, MEMPOINT },
	{ " DESCRIPT", 48, 28, MEMDESCR },
	{ " STATUS", 76, 12, MEMSTATUS },
	{ " VALUE", 88, 12, MEMVALUE },
	{ " EGUTAG", 100, 12, MEMTAG },
	{ " STATION", 0, 0, MEMSTATION }, 
	{ " ONSCAN", 0, 0, MEMOSCN },
	{ " AENABLE", 0, 0, MEMAENB },
	{ " GROUP", 0, 0, MEMGROUP },
	{ " ACKSTATUS", 0, 0, MEMACKSTATUS },
	{ " ACKDATE", 0, 0, MEMACKDATE },
	{ " ACKTIME", 0, 0, MEMACKTIME	},
	{ " OPERATOR", 0, 0, MEMACKOP },
	{ " GUIDANCE", 0, 0, MEMGUIDE },
	{ " POINTTYPE", 0, 0, PVMAPTYPE },
	{ " SCANTYPE", 0, 0, PVMAPSCANTYPE },
	{ " SCANRATE", 0, 0, PVMAPSCANRATE },
	{ " PHASE", 0, 0, PVMAPPHASE },
	{ " EVENTNUM", 0, 0, PVMAPEVENTNUM },
	{ " SYSTAGS", 0, 0, PVMAPSYSTAG },
	{ " INFOTAGS", 0, 0, PVMAPINFTAG },
	{ " PRIVGROUP", 0, 0, PVMAPPRIVGROUP },
	{ " DEVTYPE", 0, 0, PVMAPDEVTYPE },
	{ " TREND", 0, 0, PVMAPTREND },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandsummary =
    { NULL, "EXPANDSUMMARY:", TYPEQUAL, NAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, MEMDATE },
	{ " TIME", 9, 8, MEMTIME	},
	{ " POINTNAME", 19, 28, MEMPOINT },
	{ " DESCRIPT", 48, 28, MEMDESCR },
	{ " STATUS", 76, 12, MEMSTATUS },
	{ " VALUE", 88, 12, MEMVALUE },
	{ " EGUTAG", 100, 12, MEMTAG },
	{ " STATION", 0, 0, MEMSTATION }, 
	{ " ONSCAN", 0, 0, MEMOSCN },
	{ " AENABLE", 0, 0, MEMAENB },
	{ " GROUP", 0, 0, MEMGROUP },
	{ " ACKSTATUS", 0, 0, MEMACKSTATUS },
	{ " ACKDATE", 0, 0, MEMACKDATE },
	{ " ACKTIME", 0, 0, MEMACKTIME	},
	{ " OPERATOR", 0, 0, MEMACKOP },
	{ " GUIDANCE", 0, 0, MEMGUIDE },
	{ " POINTTYPE", 120, 10, PVMAPTYPE },
	{ " SCANTYPE", 130, 10, PVMAPSCANTYPE },
	{ " SCANRATE", 141, 10, PVMAPSCANRATE },
	{ " PHASE", 150, 10, PVMAPPHASE },
	{ " EVENTNUM", 160, 10, PVMAPEVENTNUM },
	{ " SYSTAGS", 170, 10, PVMAPSYSTAG },
	{ " INFOTAGS", 180, 10, PVMAPINFTAG },
	{ " PRIVGROUP", 190, 10, PVMAPPRIVGROUP },
	{ " DEVTYPE", 200, 10, PVMAPDEVTYPE },
	{ " TREND", 210, 10, PVMAPTREND },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	displaytrack =
    { NULL, "DISPLAYTRACK:", TRKTYPEQUAL, TRKOPQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, TRACKDATE },
	{ " TIME", 9, 8, TRACKTIME	},
	{ " POINTNAME", 19, 28, TRACKPOINT },
	{ " FIELDNAME", 48, 6, TRACKFIELD },
	{ " OLDVALUE", 54, 12, TRACKOLDVALUE },
	{ " NEWVALUE", 66, 12, TRACKNEWVALUE },
	{ " OPERATOR", 78, 12, TRACKOPERATOR },
	{ " DESCRIPT", 0, 0, TRACKDESC },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandtrack =
    { NULL, "EXPANDTRACK:", TRKTYPEQUAL, TRKOPQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, TRACKDATE },
	{ " TIME", 9, 8, TRACKTIME	},
	{ " POINTNAME", 19, 28, TRACKPOINT },
	{ " FIELDNAME", 48, 6, TRACKFIELD },
	{ " OLDVALUE", 54, 12, TRACKOLDVALUE },
	{ " NEWVALUE", 66, 12, TRACKNEWVALUE },
	{ " OPERATOR", 78, 30, TRACKOPERATOR },
	{ " DESCRIPT", 110, 40, TRACKDESC },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	printtrack =
    { NULL, "PRINTTRACK:", TRKTYPEQUAL, TRKOPQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " DATE", 0, 8, TRACKDATE },
	{ " TIME", 9, 8, TRACKTIME	},
	{ " POINTNAME", 19, 28, TRACKPOINT },
	{ " FIELDNAME", 48, 6, TRACKFIELD },
	{ " OLDVALUE", 54, 12, TRACKOLDVALUE },
	{ " NEWVALUE", 66, 12, TRACKNEWVALUE },
	{ " OPERATOR", 78, 12, TRACKOPERATOR },
	{ " DESCRIPT", 0, 0, TRACKDESC },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	displaypoint =
    { NULL, "DISPLAYPOINT:", TYPEQUAL, NAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " POINTNAME", 0, 28, MEMPOINT },
	{ " STATION", 29, 8, MEMSTATION }, 
	{ " DESCRIPT", 37, 28, MEMDESCR },
	{ " STATUS", 78, 12, MEMSTATUS },
	{ " DATE", 0, 0, MEMDATE },
	{ " TIME", 0, 0, MEMTIME	},
	{ " VALUE", 0, 0, MEMVALUE },
	{ " ONSCAN", 0, 0, MEMOSCN },
	{ " AENABLE", 0, 0, MEMAENB },
	{ " GROUP", 0, 0, MEMGROUP },
	{ " ACKSTATUS", 0, 0, MEMACKSTATUS },
	{ " RESOURCE", 0, 0, MEMRESRC },
	{ " ACKDATE", 0, 0, MEMACKDATE },
	{ " ACKTIME", 0, 0, MEMACKTIME	},
	{ " OPERATOR", 0, 0, MEMACKOP },
	{ " GUIDANCE", 0, 0, MEMGUIDE },
	{ " POINTTYPE", 0, 0, PVMAPTYPE },
	{ " SCANTYPE", 0, 0, PVMAPSCANTYPE },
	{ " SCANRATE", 0, 0, PVMAPSCANRATE },
	{ " PHASE", 0, 0, PVMAPPHASE },
	{ " EVENTNUM", 0, 0, PVMAPEVENTNUM },
	{ " SYSTAGS", 0, 0, PVMAPSYSTAG },
	{ " INFOTAGS", 0, 0, PVMAPINFTAG },
	{ " PRIVGROUP", 0, 0, PVMAPPRIVGROUP },
	{ " DEVTYPE", 0, 0, PVMAPDEVTYPE },
	{ " TREND", 0, 0, PVMAPTREND },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT	expandpoint =
    { NULL, "EXPANDPOINT:", TYPEQUAL, NAMEQUAL,
      {	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ "", 0, 0, FLDTEXT },
	{ " POINTNAME", 0, 28, MEMPOINT },
	{ " STATION", 29, 8, MEMSTATION }, 
	{ " DESCRIPT", 37, 28, MEMDESCR },
	{ " STATUS", 78, 12, MEMSTATUS },
	{ " DATE", 90, 8, MEMDATE },
	{ " TIME", 100, 8, MEMTIME	},
	{ " VALUE", 110, 10, MEMVALUE },
	{ " ONSCAN", 120, 10, MEMOSCN },
	{ " AENABLE", 130, 10, MEMAENB },
	{ " GROUP", 140, 20, MEMGROUP },
	{ " ACKSTATUS", 160, 10, MEMACKSTATUS },
	{ " RESOURCE", 170, 10, MEMRESRC },
	{ " ACKDATE", 180, 10, MEMACKDATE },
	{ " ACKTIME", 190, 10, MEMACKTIME	},
	{ " OPERATOR", 200, 30, MEMACKOP },
	{ " GUIDANCE", 0, 0, MEMGUIDE },
	{ " POINTTYPE", 230, 10, PVMAPTYPE },
	{ " SCANTYPE", 240, 10, PVMAPSCANTYPE },
	{ " SCANRATE", 250, 10, PVMAPSCANRATE },
	{ " PHASE", 260, 10, PVMAPPHASE },
	{ " EVENTNUM", 270, 10, PVMAPEVENTNUM },
	{ " SYSTAGS", 280, 10, PVMAPSYSTAG },
	{ " INFOTAGS", 290, 10, PVMAPINFTAG },
	{ " PRIVGROUP", 300, 10, PVMAPPRIVGROUP },
	{ " DEVTYPE", 310, 10, PVMAPDEVTYPE },
	{ " TREND", 320, 10, PVMAPTREND },
	{ NULL, 0, 0, 0 }
      }
    };

FORMAT *AlarmFormats [] = {
	&displayvalue, &expandvalue,
	&displaygroup, &expandgroup,
	&displayack, &expandack, &printack, 
	&displayalarm, &expandalarm, &printalarm,
	&displaysummary, &expandsummary,
	&displaytrack, &expandtrack, &printtrack,
	&displaypoint, &expandpoint,
	NULL
};

DictEntry dictTable [MAXDICTITEMS];
int	nextDictIx = 0;

char	textoutput [3000];	/* reusable output buffer */
char	stagebuffer [1000];	/* staging for conversion of data */

char	formatSelect [100];	/* built up format selector */


/*
**	formatSelect is base_name:pointtype:pointname
**	memSource is treenode, pvdata, or groupitem
**	fileSource is pvdata, AlarmLogRecord, or char**
*/
char *AlarmFormat (FORMAT *formatBase, void *memSource, void *fileSource)
{
    TreeNode *nodep = memSource;
    PvData *pvdatap = memSource;
    char *guidptr;
    char *dspptr;
    CaData *itemp = fileSource;
    AlarmLogRecord *logp = fileSource;
    FMT_FIELD *fldp;
    char *srcp;
    time_t unixtime;
    struct tm *curTimePtr;
    FORMAT *format;
    int ix;
    char *q1;
    char *q2;

    ResetPVPoint (); /* force load of PV data if PVMAP item encountered */
    /* if extended formatting... */
    if (formatBase->link)
    {
	switch (formatBase->QualType1)
	{
	    case NOQUAL:
		q1 = "";
		break;
	    case TYPEQUAL:
		q1 = GetPVPointInfo (pvdatap->pvName, "TYPE");
		break;
	    case GROUPQUAL:
		q1 = nodep->groupName;
		break;
	    case TRKTYPEQUAL:
		q1 = GetPVPointInfo (TrkPointname (), "TYPE");
		break;
	    case FILETYPEQUAL:
		q1 = GetPVPointInfo (logp->pointName, "TYPE");
		break;
	}
	switch (formatBase->QualType2)
	{
	    case NOQUAL:
		q2 = "";
		break;
	    case NAMEQUAL:
		q2 = pvdatap->pvName;
		break;
	    case TRKOPQUAL:
		q2 = TrkOperator ();
		break;
	    case FILENAMEQUAL:
		q2 = logp->pointName;
		break;
	}
	sprintf (formatSelect, "%s%s", formatBase->FormatName, q1);
	trimblanks (formatSelect);
	strcat (formatSelect, ":");
	strcat (formatSelect, q2);
	trimblanks (formatSelect);
	/* if first part of formatSelect matches first part of spec */
	if (strncmp (formatBase->FormatName, formatSelect, strcspn (formatBase->FormatName, ":")) == 0)
	{
	    /* find exact match or return default */
	    for (format = formatBase->link; format; format = format->link)
	    {
		if (strncmp (formatSelect, format->FormatName, strlen (format->FormatName)) == 0)
		    break;
	    }
	    if (format == NULL)
		format = formatBase;
	}
    }
    else
    {
	format = formatBase;
    }


    /* blank and null terminate the output buffer */
    memset (textoutput, ' ', sizeof(textoutput)-1);
    textoutput [sizeof(textoutput)-1] = 0;

    fldp = format->fields;
    while (fldp->FieldName)
    {
	if (fldp->Length != 0)
	{
	    switch (fldp->Source)
	    {
	    case FLDTEXT:
		srcp = fldp->FieldName;
		break;
	    case MEMDATE:
		unixtime = pvdatap->lastAlarm.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
		curTimePtr = localtime (&unixtime);
		sprintf (stagebuffer, "%02d/%02d/%02d",
			curTimePtr->tm_mon+1, curTimePtr->tm_mday, curTimePtr->tm_year%100);
		srcp = stagebuffer;
		break;
	    case MEMTIME:
		unixtime = pvdatap->lastAlarm.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
		curTimePtr = localtime (&unixtime);
		sprintf (stagebuffer, "%02d:%02d:%02d",
			curTimePtr->tm_hour, curTimePtr->tm_min, curTimePtr->tm_sec);
		srcp = stagebuffer;
		break;
	    case MEMPOINT:
		srcp = pvdatap->pvName;
		break;
	    case MEMDESCR:
		srcp = pvdatap->description;
		break;
    	    case MEMSTATUS:
		srcp = alarmStatusString[pvdatap->unAckedStatus];
		break;
	    case MEMVALUE:
		srcp = pvdatap->unAckedValue;
		break;
	    case MEMTAG:
		srcp = pvdatap->eguTag;
		if (srcp  == NULL)
		    srcp = "                ";
		break;
	    case MEMSTATION:
		srcp = pvdatap->stationName;
		break;
	    case MEMOSCN:
		if (pvdatap->oscn)
		    srcp = "ONSCAN  ";
		else
		    srcp = "OFFSCAN ";
		break;
	    case MEMAENB:
		if (pvdatap->aenb)
		    srcp = "ENABLED ";
		else
		    srcp = "DISABLED";
		break;
	    case MEMGROUP:
		srcp = pvdatap->parentGroup->groupName;
		break;
	    case MEMRESRC:
		srcp = pvdatap->resource;
		break;
	    case MEMACKDATE:
		if (pvdatap->Ack)
		{
		    unixtime = pvdatap->lastAck.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
		    curTimePtr = localtime (&unixtime);
		    sprintf (stagebuffer, "%02d/%02d/%02d",
				curTimePtr->tm_mon+1, curTimePtr->tm_mday, curTimePtr->tm_year%100);
		    srcp = stagebuffer;
		}
		else
		{
		    srcp = "        ";
		}
		break;
	    case MEMACKTIME:
		if (pvdatap->Ack)
		{
		    unixtime = pvdatap->lastAck.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
		    curTimePtr = localtime (&unixtime);
		    sprintf (stagebuffer, "%02d:%02d:%02d",
				curTimePtr->tm_hour, curTimePtr->tm_min, curTimePtr->tm_sec);
		    srcp = stagebuffer;
		}
		else
		{
		    srcp = "        ";
		}
		break;
    	    case MEMACKSTATUS:
		if (pvdatap->Ack)
		    srcp = "ACKNOWLEDGED  ";
		else
		    srcp = "UNACKNOWLEDGED";
		break;
	    case MEMACKOP:
		if (pvdatap->Ack)
		    srcp = pvdatap->op;
		else
		    srcp = "                        ";
		break;
	    case MEMGUIDE:
		GetAlarmInfo (pvdatap->pvName, &guidptr, &dspptr);
		if (guidptr)
		    srcp = guidptr;
		else
		    srcp = "                        ";
		break;
	    case FILEDATE:
		srcp = logp->date;
		break;
	    case FILETIME:
		srcp = logp->time;
		break;
	    case FILEPOINT:
		srcp = logp->pointName;
		break;
	    case FILEDESCR:
		srcp = logp->descriptor;
		break;
	    case FILESTATUS:
		srcp = logp->status;
		break;
	    case FILESEVR:
		srcp = logp->severity;
		break;
	    case FILEVALUE:
		srcp = logp->value;
		break;
	    case FILETAG:
		srcp = logp->egutag;
		break;
	    case FILEACKED:
		if (logp->acked != 0)
		    srcp = "ACKNOWLEDGED  ";
		else
		    srcp = "UNACKNOWLEDGED";
		break;
	    case FILEACKDATE:
		if (logp->acked != 0)
		{
		    unixtime = logp->acktime.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
		    curTimePtr = localtime (&unixtime);
		    sprintf (stagebuffer, "%02d/%02d/%02d",
			curTimePtr->tm_mon+1, curTimePtr->tm_mday, curTimePtr->tm_year%100);
		    srcp = stagebuffer;
		}
		else
		    srcp = "        ";
		break;
	    case FILEACKTIME:
		if (logp->acked != 0)
		{
		    unixtime = logp->acktime.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
		    curTimePtr = localtime (&unixtime);
		    sprintf (stagebuffer, "%02d:%02d:%02d",
			curTimePtr->tm_hour, curTimePtr->tm_min, curTimePtr->tm_sec);
		    srcp = stagebuffer;
		}
		else
		    srcp = "        ";
		break;
	    case FILENOTE:
		if (logp->comment[0] != 0)
		    srcp = "*";
		else
		    srcp = " ";
		break;
	    case FILEOPERATOR:
		srcp = logp->operatorName;
		break;
	    case FILECOMMENT:
		srcp = logp->comment;
		break;
	    case FILESTATION:
		srcp = logp->stationName;
		break;
#ifdef HASINFOFLDS
	    case FILEINFO1:
		srcp = logp->info1;
		break;
	    case FILEINFO2:
		srcp = logp->info2;
		break;
	    case FILEINFO3:
		srcp = logp->info3;
		break;
	    case FILEINFO4:
		srcp = logp->info4;
		break;
	    case FILEINFO5:
		srcp = logp->info5;
		break;
#endif HASINFOFLDS
	    case FILEGUIDE:
		GetAlarmInfo (logp->pointName, &guidptr, &dspptr);
		if (guidptr)
		    srcp = guidptr;
		else
		    srcp = "                        ";
		break;
	    case GRPGROUP:
		srcp = nodep->groupName;
		break;
	    case GRPMAJOR:
		sprintf (stagebuffer, "%02d", nodep->groupData.numAlarmsBySeverity[MAJOR_ALARM]);
		srcp = stagebuffer;
		break;
	    case GRPMINOR:
		sprintf (stagebuffer, "%02d", nodep->groupData.numAlarmsBySeverity[MINOR_ALARM]);
		srcp = stagebuffer;
		break;
	    case GRPEVENT:
		sprintf (stagebuffer, "%02d", nodep->groupData.numAlarmsBySeverity[EVENT_ALARM]);
		srcp = stagebuffer;
		break;
	    case GRPINFO:
		sprintf (stagebuffer, "%02d", nodep->groupData.numAlarmsBySeverity[INFO_ALARM]);
		srcp = stagebuffer;
		break;
	    case GRPGUIDE:
		srcp = nodep->groupParms.guidance;
		break;
	    case CASTATUS:
		if (itemp->CurStatus)
		    srcp = alarmStatusString[itemp->CurStatus];
		else
		    srcp = "                ";
		break;
	    case CAVALUE:
		srcp = itemp->Value;
		break;
	    case TRACKDATE:
		srcp = TrkDate ();
		break;
	    case TRACKTIME:
		srcp = TrkTime ();
		break;
	    case TRACKPOINT:
		srcp = TrkPointname ();
		break;
	    case TRACKFIELD:
		srcp = TrkFieldname ();
		break;
	    case TRACKOLDVALUE:
		srcp = TrkOldvalue ();
		break;
	    case TRACKNEWVALUE:
		srcp = TrkNewvalue ();
		break;
	    case TRACKOPERATOR:
		srcp = TrkOperator ();
		break;
	    case TRACKDESC:
		srcp = TrkDescriptor ();
		break;
	    case PVMAPTYPE:
		srcp = GetPVPointInfo (pvdatap->pvName, "TYPE");
		break;
	    case PVMAPSCANTYPE:
		srcp = GetPVPointInfo (pvdatap->pvName, "STYPE");
		break;
	    case PVMAPSCANRATE:
		sprintf (stagebuffer, "%4.1f", GetPVPointValue (pvdatap->pvName, "SRATE"));
		srcp = stagebuffer;
		break;
	    case PVMAPPHASE:
		sprintf (stagebuffer, "%3.0f", GetPVPointValue (pvdatap->pvName, "PHASE"));
		srcp = stagebuffer;
		break;
	    case PVMAPEVENTNUM:
		sprintf (stagebuffer, "%4.0f", GetPVPointValue (pvdatap->pvName, "EVENT"));
		srcp = stagebuffer;
		break;
	    case PVMAPSYSTAG:
		sprintf (stagebuffer, "%2.0f",  GetPVPointValue (pvdatap->pvName, "TAGDEPTH"));
		srcp = stagebuffer;
		break;
	    case PVMAPINFTAG:
		if (GetInfoTag (pvdatap->pvName))
		    srcp = "TAGGED    ";
		else
		    srcp = "NOT TAGGED";
		break;
		break;
	    case PVMAPPRIVGROUP:
		srcp = GetPVPointInfo (pvdatap->pvName, "PRIVGROUP");
		break;
	    case PVMAPDEVTYPE:
		srcp = GetPVPointInfo (pvdatap->pvName, "DEVTYPE");
		break;
	    case PVMAPTREND:
		if (GetPVPointValue (pvdatap->pvName, "ONTREND"))
		    srcp = "ON TREND   ";
		else
		    srcp = "NOT TRENDED";
		break;
	    default:
		srcp = "Unidentified source type";
		break;
	    }
	    if (strlen(srcp) != 0)
	    {
		/* use look-aside table (for fields only) if fieldname contains leading '*' char */
		if  ((fldp->Source != FLDTEXT) && (fldp->FieldName [0] == '*'))
		{
		    for (ix = 0; ix < nextDictIx; ix++)
		    {
			if (strncmp (srcp, dictTable [ix].oldText, strlen(dictTable [ix].oldText)) == 0)
			{
			    srcp = dictTable [ix].newText;
			    break;
			}
		    }
		}

		if (strlen (srcp) > fldp->Length)
		    strncpy (&textoutput[fldp->Start], srcp, fldp->Length);
		else
		    strncpy (&textoutput[fldp->Start], srcp, strlen (srcp));
	    }
	}
	fldp++;
    }
    /* clip to last non-blank field */
    trimblanks (textoutput);
    return (textoutput);
}

/*
**	ReadFormats () - open format file, consisting of text in the format:
**		<group>:
**		   <field> <start> <length>
**	where group is the format group being modified,
**	      field is the field being configured,
**	      start is the beginning column,
**	      length is the length of the field (0 means don't display).
**	If a group appears in the format file, all fields in the group
**	must be defined in order to be displayed. Text can be formatted by
**	entering a quoted field name in the format, for example
**		DISPLAYVALUE:
**			"Value is" 10, 8
**	(each format can take up to five text fields in the definition file)
**
**	The available groups and fields and their standard values):
** DISPLAYVALUE - Point values in the group displays
**	 DATE 0, 0
**	 TIME 0, 0
**	 POINTNAME 0, 28
**	 DESCRIPT 48, 28
**	 STATUS 76, 12
**	 VALUE 88, 12
**	 EGUTAG 100, 12
**	 STATION 0, 0
**	 ONSCAN 0, 0
**	 AENABLE 0, 0
**	 GROUP 0, 0
**	 ACKSTATUS 0, 0
**	 ACKDATE 0, 0
**	 ACKTIME 0, 0
**	 OPERATOR 0, 0
** DISPLAYGROUP - Group values in the group displays (standard display has 4 text fields)
**	 "MAJ=" 29, 4
**	 ",  MIN=" 35, 7
**	 ",  EVENT=" 44, 9
**	 ",  INFO=" 55, 8
**	 POINTNAME 0, 28
**	 MAJOR 33, 2
**	 MINOR 42, 2
**	 EVENT 53, 2
**	 INFO 63, 2
** DISPLAYACK - Acknowledgements in the history, event and alarm window displays
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 DESCRIPT 48, 28
**	 STATUS 76, 12
**	 OPERATOR 88, 12
** DISPLAYALARM - Alarms in the history, event and alarm window displays
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 DESCRIPT 48, 28
**	 STATUS 76, 12
**	 VALUE 88, 12
**	 EGUTAG 100, 12
**	 STATION 0, 0
** PRINTACK - Acknowledgements in the history, event and alarm window printouts
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 DESCRIPT 48, 28
**	 STATUS 89, 12
**	 OPERATOR 102, 12

** PRINTALARM - Alarms in the history, event and alarm window printouts
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 DESCRIPT 48, 28
**	 STATUS 89, 12
**	 VALUE 102, 12
**	 EGUTAG 115, 12
**	 STATION 0, 0
** DISPLAYSUMMARY - Alarms in the summary window
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 DESCRIPT 48, 28
**	 STATUS 76, 12
**	 VALUE 88, 12
**	 EGUTAG 100, 12
**	 STATION 0, 0
**	 ONSCAN 0, 0
**	 AENABLE 0, 0
**	 GROUP 0, 0
**	 ACKSTATUS 0, 0
**	 ACKDATE 0, 0
**	 ACKTIME 0, 0
**	 OPERATOR 0, 0
** DISPLAYTRACK - Items in the operator tracking file display
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 FIELDNAME 48, 6
**	 OLDVALUE 54, 12
**	 NEWVALUE 66, 12
**	 OPERATOR 78, 12
** PRINTTRACK - Items in the operator tracking file printout
**	 DATE 0, 8
**	 TIME 9, 8
**	 POINTNAME 19, 28
**	 FIELDNAME 48, 6
**	 OLDVALUE 54, 12
**	 NEWVALUE 66, 12
**	 OPERATOR 78, 12
** DISPLAYPOINT - Point items in the point status displays
**	 POINTNAME 0, 28
**	 STATION 29, 8
**	 DESCRIPT 37, 28
**	 STATUS 78, 12
**	 DATE 0, 0
**	 TIME 0, 0
**	 VALUE 0, 0
**	 ONSCAN 0, 0
**	 AENABLE 0, 0
**	 GROUP 0, 0
**	 ACKSTATUS 0, 0
**	 RESOURCE 0, 0
**	 ACKDATE 0, 0
**	 ACKTIME 0, 0
**	 OPERATOR 0, 0
**	 POINTTYPE 0, 0
**	 SCANTYPE 0, 0
**	 SCANRATE 0, 0
**	 PHASE 0, 0
**	 EVENTNUM 0, 0
**	 SYSTAGS 0, 0
**	 INFOTAGS 0, 0
**	 PRIVGROUP 0, 0
**	 DEVTYPE 0, 0
**	 TREND 0, 0
*/

static char inpstring [132];

void ReadFormats ()
{
    int ix, ix2;
    char *sp;
    FILE *fp;
    char *src;
    char *dst;
    char *cp;
    char *newstring;
    FORMAT *curFormat = NULL;
    char *field;
    int lookAside = 0;
    char    filebuff[100];

#ifdef WIN32
    strcpy (filebuff, "c:\\tis4000db\\alformat.def");
#else
    strcpy (filebuff, "/tis4000db/alformat.def");
#endif WIN32

    fp = fopen (filebuff, "rt");
    if (fp)
    {
        while (!feof (fp))
        {
            ix = 0;
            while (!feof (fp))
            {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
                }
		ix++;
                if (ix == 80)
                    return; /* bad string in file. */
            }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && (*sp == ' ' || *sp == 0x09))
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */

	    /* if not a text field line, and contains one or more ':' chars, it is a group name */
	    if (((strchr(sp, '\"') == NULL) && (cp = strchr (sp, ':')) != NULL))
	    {
		/* start of a set of group fields */
		/* *cp = 0; */
		/* find group and field */
		curFormat = NULL;
		for (ix = 0; AlarmFormats[ix]; ix++)
		{
		    if (strncmp (AlarmFormats[ix]->FormatName, sp,
				 strcspn (AlarmFormats[ix]->FormatName, ":")) == 0)
		    {
			/* find exact match or create new (clone base struct) */
			for (curFormat = AlarmFormats[ix]; curFormat; curFormat = curFormat->link)
			{
			    if (strcmp (sp, AlarmFormats[ix]->FormatName) == 0)
				break;
			}
			if (curFormat == NULL)
			{
			    curFormat = malloc (sizeof(*AlarmFormats[ix]));
			    if (curFormat)
			    {
				*curFormat = *AlarmFormats[ix];
				curFormat->FormatName = malloc (strlen(sp)+1);
				if (curFormat->FormatName)
				    strcpy (curFormat->FormatName, sp);
				/* link at head of add-on list */
				curFormat->link = AlarmFormats[ix]->link;
				AlarmFormats[ix]->link = curFormat;
			    }
			    else
			    {
				printf ("No memory for group name %s\n", inpstring);
				continue;
			    }
			}
			/* clear out old format info (set lengths to 0) */
			for (ix2 = 0; curFormat->fields[ix2].FieldName; ix2++)
			    curFormat->fields[ix2].Length = 0;
			break;
		    }
		}
		if (curFormat == NULL)
		{
		    printf ("Bad bad group name %s\n", inpstring);
		}
	    }
	    else if (curFormat != NULL) /* must be a field def */
	    {
		
		field = sp;
		/* update fixed text field */
		if (*field == '"')
		{
		    field++;
		    sp = strchr (field, '"');
		    if (sp)
		    {
			*sp = 0;
			sp++;
			/* find empty text field, and adjust pointer to new text */
			for (ix2 = 0; curFormat->fields[ix2].FieldName; ix2++)
			{
			    if ((curFormat->fields[ix2].Source == FLDTEXT)
			      && (curFormat->fields[ix2].Length == 0))
			    {
				if (curFormat->fields[ix2].FieldName = malloc (strlen(field)+1))
				{
				    /* copy manually, to convert "\n" to nl character */
				    /* strcpy (curFormat->fields[ix2].FieldName, field); */
				    src = field;
				    dst = curFormat->fields[ix2].FieldName;
				    while (*src)
				    {
					*dst = *src;
					if (*dst == '\\')
					{
					    src++;
					    if (*src == 'n')
						*dst = 0x0a;
					    else
						*dst = *src;
					}
					dst++;
					src++;
				    }
				    curFormat->fields[ix2].Start = atoi (sp);
				    cp = strchr (sp, ',');
				    if (cp)
					curFormat->fields[ix2].Length = atoi (cp+1);
				    if ((cp == NULL) || (curFormat->fields[ix2].Length == 0))
				    {
					printf ("Bad field specification %s\n", inpstring);
				    }
				}
				break;
			    }
			}
			/* if at end of format group, we have a problem */
			if (curFormat->fields[ix2].FieldName == NULL)
			{
			    printf ("Too many text fields for group %s\n", curFormat->FormatName);
			}
		    }
		    else
		    {	
			/* badly formed text string */
			printf ("Bad text string %s\n", inpstring);
		    }
		}
		else /* find field */
		{
		    while (*sp && (*sp != ' ' && *sp != 0x09))
			sp++;
	            *sp = 0;
		    sp++;
		    if (*field == '*')
		    {
			lookAside = 1;
			field++;
		    }
		    else
			lookAside = 0;
		    for (ix2 = 0; curFormat->fields[ix2].FieldName; ix2++)
		    {
			if (strcmp (&curFormat->fields[ix2].FieldName[1], field) == 0)
			{
			    if (newstring = malloc (strlen (curFormat->fields[ix2].FieldName)))
			    {
				strcpy (newstring, curFormat->fields[ix2].FieldName);
				curFormat->fields[ix2].FieldName = newstring;
				if (lookAside)
				    curFormat->fields[ix2].FieldName[0] = '*';
				else
				    curFormat->fields[ix2].FieldName[0] = ' ';
			    }
			    curFormat->fields[ix2].Start = atoi (sp);
			    cp = strchr (sp, ',') + 1;
			    if (cp)
				curFormat->fields[ix2].Length = atoi (cp);
			    if ((cp == NULL) || (curFormat->fields[ix2].Length == 0))
			    {
				printf ("Bad field specification %s\n", inpstring);
			    }
			    break;
			}
		    }
		    /* if at end of format group, we have a problem */
		    if (curFormat->fields[ix2].FieldName == NULL)
		    {
			printf ("Unknown field name %s in %s\n", inpstring, curFormat->FormatName);
		    }
		}
	    }
	    else /* field outside of format definition */
	    {
		printf ("No group selected for %s\n", inpstring);
	    }

	}
        fclose (fp);
    } /* end of opened file code */

    /* read in lookAside dictionary */

#ifdef WIN32
    strcpy (filebuff, "c:\\tis4000db\\aldict.def");
#else
    strcpy (filebuff, "/tis4000db/aldict.def");
#endif WIN32

    fp = fopen (filebuff, "rt");
    if (fp)
    {
        while (!feof (fp))
        {
            ix = 0;
            while (!feof (fp))
            {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
                }
		ix++;
                if (ix == 80)
                    return; /* bad string in file. */
            }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && (*sp == ' ' || *sp == 0x09))
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */

	    /* Add an item to dictionary */
	    if ((cp = strchr (sp, '~')) != NULL )
	    {
		/* found separator, terminate first text, get second */
		*cp = 0;
		cp++;
		if (dictTable [nextDictIx].oldText = calloc (strlen (sp)+1, 1))
		    strcpy (dictTable [nextDictIx].oldText, sp);
		if (dictTable [nextDictIx].newText = calloc (strlen (cp)+1, 1))
		    strcpy (dictTable [nextDictIx].newText, cp);
		nextDictIx++;
		if (nextDictIx == MAXDICTITEMS)
		{
		    printf ("Dictionary overflow at %d items\n", nextDictIx);
		    break;
		}
	    }
	    else
	    {
		printf ("Bad line in dictionary, a '~' character is required to separate the old\nand new strings. Input is %s\n" ,inpstring);
	    }
	}
        fclose (fp);
    }
}
