/*
**
**	alarmfmt.h - alarm object display formats
**
*/

#define MAXDICTITEMS	1000

#define		FLDTEXT		0	/* Use field name in output */
#define 	MEMDATE		1	/* Use pvdata date in output */
#define 	MEMTIME		2	/* Use pvdata time in output */	
#define 	MEMPOINT	3	/* Use pvdata name in output */
#define 	MEMDESCR	4	/* Use pvdata descr in output */
#define 	MEMOSCN		5	/* Use pvdata ON-SCAN in output */
#define 	MEMAENB		6	/* Use pvdata alarm enable in output */
#define 	MEMGROUP	7	/* Use pvdata parent group name in output */
#define 	MEMRESRC	8	/* Use pvdata resource in output */
#define 	MEMACKDATE	9	/* Use pvdata ack date in output */
#define 	MEMACKTIME	10	/* Use pvdata ack time in output */
#define 	MEMACKOP	11	/* Use pvdata ack operator name in output */
#define 	MEMSTATUS	12	/* Use pvdata status in output */
#define 	MEMVALUE	13	/* Use pvdata value in output */
#define 	MEMSTATION	14	/* Use pvdata station in output */
#define 	MEMTAG		15	/* Use pvdata egutag in output */
#define 	MEMACKSTATUS	16	/* Use pvdata acknowledged status in output */
#define 	MEMGUIDE	17	/* Use guidance from alarminfo file */
/* skip to 20 for additional mem fields */
#define 	CASTATUS	20	/* Use pvdata status in output */
#define 	CAVALUE		21	/* Use pvdata value in output */
/* skip to 25 for additional channel access fields */
#define		GRPGROUP	25	/* Use group name in output */
#define		GRPMAJOR	26	/* Use group major count in output */
#define		GRPMINOR	27	/* Use group minor count in output */
#define		GRPEVENT	28	/* Use group event count in output */
#define		GRPINFO		29	/* Use group info count in output */
#define		GRPGUIDE	30	/* Use group guidance in output */
/* skip to 35 for additional group fields */
#define 	FILEDATE	35	/* Use log file date in output */
#define 	FILETIME	36	/* Use log file time in output */
#define 	FILEPOINT	37	/* Use log file name in output */
#define 	FILEDESCR	38	/* Use log file descr in output */
#define 	FILESTATUS	39	/* Use log file status in output */
#define		FILESEVR 	40	/* Use log file severity in output */
#define 	FILEVALUE	41	/* Use log file value in output */
#define 	FILETAG		42	/* Use log file tag in output */
#define		FILEACKED	43	/* Use log file acked status in output */
#define		FILEACKDATE	44	/* Use log file ack time in output */
#define		FILEACKTIME	45	/* Use log file ack time in output */
#define 	FILENOTE	46	/* Use log file note indicator */
#define 	FILEOPERATOR	47	/* Use log file operator in output */
#define		FILECOMMENT	48	/* Use log file comment in output */
#define 	FILESTATION	49	/* Use log file station in output */
#define 	FILEINFO1	50	/* Use log file info1 in output */
#define 	FILEINFO2	51	/* Use log file info2 in output */
#define 	FILEINFO3	52	/* Use log file info3 in output */
#define 	FILEINFO4	53	/* Use log file info4 in output */
#define 	FILEINFO5	54	/* Use log file info5 in output */
#define 	FILEGUIDE	55	/* Use guidance from alarminfo file */
/* skip to 60 for additional file fields */
#define 	TRACKDATE	60	/* Use track file date in output */
#define 	TRACKTIME	61	/* Use track file time in output */
#define 	TRACKPOINT	62	/* Use track file name in output */
#define 	TRACKFIELD	63	/* Use track file fieldname in output */
#define 	TRACKOLDVALUE	64	/* Use track file old value in output */
#define 	TRACKNEWVALUE	65	/* Use track file new value in output */
#define 	TRACKOPERATOR	66	/* Use track file operator in output */
#define 	TRACKDESC	67	/* Use track file descriptor in output */
/* skip to 80 for additional tracking fields */
#define 	PVMAPTYPE	80	/* Use type from pvmap */
#define 	PVMAPSCANTYPE	81	/* Use scan type from pvmap */
#define 	PVMAPSCANRATE	82	/* Use scan rate from pvmap */
#define 	PVMAPPHASE	83	/* Use phase from pvmap */
#define 	PVMAPEVENTNUM	84	/* Use event num from pvmap */
#define 	PVMAPSYSTAG	85	/* Use systag count from pvmap */
#define 	PVMAPINFTAG	86	/* Use infotag count from tagvu db */
#define 	PVMAPPRIVGROUP	87	/* Use priv group from pvmap */
#define 	PVMAPDEVTYPE	88	/* Use device type from pvmap */
#define 	PVMAPTREND	89	/* Use trend enable from pvmap */

/* format selection qualifier types */
#define		NOQUAL		0	/* no qualifier */
#define		TYPEQUAL	1	/* point type qualifier */
#define		NAMEQUAL	2	/* name qualifier */
#define		GROUPQUAL	3	/* group name qualifier */
#define		TRKOPQUAL	4	/* tracking operator name qualifier */
#define		TRKTYPEQUAL	5	/* tracking point type qualifier */
#define		FILETYPEQUAL	6	/* log point type qualifier */
#define		FILENAMEQUAL	7	/* log name qualifier */

typedef struct {
	char*	FieldName;	/* the identifier for the field */
	int	Start;		/* start in display line */
	unsigned int Length;	/* number of characters */
	int	Source;		/* case index of data source */
} FMT_FIELD;

typedef struct Format {
	struct Format *link;	/* format link field */
	char*	FormatName;	/* identifier of format */
	int	QualType1;	/* first format select qualifier */
	int	QualType2;	/* first format select qualifier */
	FMT_FIELD fields [32];	/* up to 31 fields can be formatted on a line (+ a terminator) */
} FORMAT;

typedef struct {
	int	CurStatus;	/* ca status value */
	char*	Value;		/* ca value string */
} CaData;

typedef struct {
	char*	oldText;		/* the original value */
	char*	newText;		/* the replacement value */
} DictEntry;


extern FORMAT	displayvalue;
extern FORMAT	expandvalue;

extern FORMAT	displaygroup;
extern FORMAT	expandgroup;

extern FORMAT	displayack;
extern FORMAT	expandack;
extern FORMAT	printack;

extern FORMAT	displayalarm;
extern FORMAT	expandalarm;
extern FORMAT	printalarm;

extern FORMAT	displaysummary;
extern FORMAT	expandsummary;

extern FORMAT	displaytrack;
extern FORMAT	expandtrack;
extern FORMAT	printtrack;

extern FORMAT	displaypoint;
extern FORMAT	expandpoint;
extern FORMAT	printpoint;

char *AlarmFormat (FORMAT *format, void *memSource, void *fileSource);
void ReadFormats ();
