#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcontainerHEADER
#include vdictHEADER
#include vexHEADER
#include vtimerHEADER
#include vcolorHEADER
#include vmenubarHEADER
#include vlistviewHEADER
#include vprintHEADER


extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
#include <time.h>
#include "NTUnixSubs.h"
#else
#include <sys/ipc.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/time.h>
#include <errno.h>
#endif WIN32

#include "cadef.h"
#include "shareMem.h"
#include "alarmDefs.h"
#include "alarmfil.h"
#include "alarmfmt.h"
#include "alarmvu.h"
#include "bell.h"

extern char *optarg;	// cmd line arg arg
extern int  optopt;	// value of illegal cmd

ShareHeader	*shareHeader;
TreeNode	*treeRoot;

} // end extern "C"

#ifdef WIN32
#include <windows.h>
// undocumented GALAXY call
extern void _setupMSWPrinter (vprint *printer, HDC dc);
#endif WIN32

#include "alvu.h"
#include "alarmvu.hxx"
#include "alvuMainDialog.hxx"
#include "aldlgListView.hxx"
#include "alarmDialogs.hxx"
#include "commentDialog.hxx"
#include "popupDialog.hxx"
#include "textDialog.hxx"
#include "alvuUtils.hxx"
#include "utils.hxx"

// global stuff
int		alarmBlinkLevel;
int		alarmSoundEnabled;
int		alarmSummaryDisabled = vFALSE;
int		CAInit = 0;
char		**channelNames;
PvData		**channelPtrs;
int		channelCount;
char		**groupNames;
TreeNode	**groupPtrs;
int		groupCount;
int		groupdisplayHeight;
char		linePrinter [LINEPRINTER_SIZE + 1];
char		defFilterName [AL_FILTERNAME_SIZE + 1];
int		MaxSendsPerSecond;
unsigned int	popupAlrmEnable = vFALSE;
unsigned int	winEnable = vFALSE;

// global ptrs to possible Dialogs
AlarmEventDialog	*alarmEventDialogPtr = NULL;
AlarmHistoryDialog	*alarmHistoryDialogPtr = NULL;
AlarmPointsDialog	*reportsDialogPtr = NULL;
AlarmSummaryDialog	*alarmSummaryDialogPtr = NULL;
AlarmTrackDialog	*alarmTrackDialogPtr = NULL;
AlarmWindowDialog	*alarmWindowDialogPtr = NULL;
AlarmSystemDialog	*systemDialogPtr = NULL;
CommentDialog		*commentDialogPtr = NULL;
DynamicGroupDialog	*dynamicGroupDialogPtr = NULL;
PopupDialog		*popAlarmPtr = NULL;
TextDialog		*guidanceDialogPtr = NULL;
TextDialog		*expandDialogPtr = NULL;

// color stuff
long		CurrUnackSevColor [NUM_SEV_COLORS] = 
			{ 0x00a000, 0x00c8ff, 0x000000, 0xffc800, 0xff0000, 0xffffff };
long		defaultAlarmColors [NUM_SEV_COLORS] = 
			{ 0x00a000, 0x00c8ff, 0x000000, 0xffc800, 0xff0000, 0xffffff };
long		SevColor [NUM_SEV_COLORS] = 
			{ 0x00a000, 0x00c8ff, 0x000000, 0xffc800, 0xff0000, 0xffffff };
long		UnackSevColor [NUM_SEV_COLORS] = 
			{ 0x00a000, 0x00c8ff, 0x000000, 0xffc800, 0xff0000, 0xffffff };
long		backgroundColor;
vcolor		*backgroundVcolor;
vcolor		*blackVcolor;



/*******************************************************************
*
*    Main ()
*
*******************************************************************/

int main (int argc, char **argv)
{
    char	dbpath[ALVU_MAX_DATABASE_PATH];

    // Initialize Galaxy
    vstartup (argc, argv);

    vapplication *app = vapplication::GetCurrent();
#ifndef WIN32
    strncpy (dbpath, "/tis4000db", ALVU_MAX_DATABASE_PATH);
#else
    strncpy (dbpath, "c:\\tis4000db", ALVU_MAX_DATABASE_PATH);
#endif WIN32

    alvuApplication *alvuApp = new alvuApplication (argc, argv, dbpath);

    if ((alvuApp -> GetInitErrors ()) == ALVU_ERROR)
    {
	delete alvuApp;
	exit (EXIT_FAILURE);
    }

    // enter main event process loop
    veventProcess ();

    delete alvuApp;
    
    exit (EXIT_SUCCESS);
    return EXIT_FAILURE;
}


/********************************************************************************
*
*    AlarmVu application constuctor
*
********************************************************************************/

alvuApplication::alvuApplication (int argv, char **argc, char *dbpath)
{
    initErrors = ALVU_SUCCESS;
    initedOperPriv = vFALSE;
    initedOperTrack = vFALSE;
    initedTisGalaxy = vFALSE;
    mainDialog = NULL;
    shrMemIds = NULL;
    groupNames = NULL;
    groupPtrs = NULL;
    groupCount = 0;
    channelNames = NULL;
    channelPtrs = NULL;
    channelCount = 0;
    printer = NULL;

    alvuApp = vapplication::GetCurrent ();
    alvuResource = alvuApp -> GetResources ();

    // intern tag names
    alvuInternNames ();

    // get and store the titles and messages dictionary

    titlesResource = vresourceGet (alvuResource, alvu_TitlesDictionary);
    messagesResource = vresourceGet (alvuResource, alvu_MessagesDictionary);
    menutagsResource = vresourceGet (alvuResource, alvu_MenuTagsDictionary);

    // initialize channel access
    ca_task_initialize ();

    // set default height for group display window
    groupdisplayHeight = ALVU_DEFAULT_GRP_DISPLAY_HGT;

    // init shared memory region with alarmMgr
    if ((MapSharedMemory ()) == ALVU_ERROR)
    {
	initErrors = ALVU_ERROR;
	return;
    }

    if (!OpenCodeBase ())
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_DatabaseOpenErr);
	initErrors = ALVU_ERROR;
	return;
    }

    // read in setup information
    strcpy (defFilterName, "");
    InitSetup ();

    // read in display format information
    ReadFormats ();

    if (*defFilterName)
    {
	strcpy (shellFilter.name, defFilterName);
	trimblanks (shellFilter.name);
	if (shellFilter.name[0])
	{
	    GetFilter (shellFilter.name, &shellFilter);
	}
    }
    /*
	PROCESS CMD LINE ARGS HERE
    */
    if ((ReadCmdLineArgs ()) == ALVU_ERROR)
    {
	initErrors = ALVU_ERROR;
	return;
    }

    // set up socket connection to alarm manager
    if ((ContactAlarmMgr ()) == ALVU_ERROR)
    {
	initErrors = ALVU_ERROR;
	return;
    }

    // init operator privileges and tracking
    if ((InitPrivTrack ()) != PRIVOK)
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_InitPrivErr);
	initErrors = ALVU_ERROR;
	return;
    }
    else
    {
	initedOperPriv = vTRUE;
    }

    if (TrkOpenFile () != NO_ERR)
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_OpTrackOpenErr);
	initErrors = ALVU_ERROR;
	return;
    }
    else
    {
	initedOperTrack = vTRUE;
    }


    groupNames = GetGroups (&groupPtrs);
    channelNames = GetChannels (&channelPtrs);

    // load the mainDialog

    mainDlgResource = vresourceGet (alvuResource, alvu_MainDialog);
    mainDialog = new alvuMainDialog (mainDlgResource);

    // test if loadinit errors for main
    if ((mainDialog -> GetLoadInitError ()) != ALVU_SUCCESS)
    {
	initErrors = ALVU_ERROR;
	return;
    }

    // init the printer
    InitPrinter ();

    // init tisGalaxy, see tisgalaxylib.<hxx,cxx>
    tisGalaxyResource = startTisGalaxy ();
    initedTisGalaxy = vTRUE;

    mainDialog -> SetColors ();

    UpdateBlinkColors ();	// in alvuUtils.cxx

    mainDialog -> OpenMain (this);

    if (alarmSoundEnabled == vTRUE)
    {
	// init the sound system
	Bell (0);

	// start the sound
	if ( (HighestSeverity (treeRoot -> groupData.numUnackedBySeverity)) ||
	     treeRoot -> groupData.numUnackedBySeverity [NO_ALARM]);
	{
	    BellTurnOn ();
	}
    }

    // colorize mains buttons
    mainDialog -> RefreshDisplay ();

    // start main timer
    mainDialog -> mainTimer -> StartMainTimer ();

    // start most recent alarm window
    if ( (winEnable == vTRUE) || (shellFilter.startupDisplay == ALWINSTART) )
    {
	if (alarmWindowDialogPtr == NULL)
	{
	    vresource alarmDlgRes = vresourceGet (alvuResource, alvu_AlarmDialog);
	    alarmWindowDialogPtr = new AlarmWindowDialog (alarmDlgRes);
	}
    }

    // startup display
    StartupDisplay ();
}


/********************************************************************************
*
*    AlarmVu application destructor
*
********************************************************************************/

alvuApplication::~alvuApplication (void)
{
    if (shrMemIds != NULL)
    {
	UnMapSharedMemory ();
    }

    // close socket to alarmMgr
#ifdef WIN32
    closesocket (alarmhndSocket);
#else
    close (alarmhndSocket);
#endif WIN32

    // close down privilege tracking, utils.cxx
    ClosePrivTrack (initedOperPriv, initedOperTrack);

    // clean up some allocated memory
    if (groupNames != NULL)
    {
	free ((void *)groupNames);
    }
    if (groupPtrs != NULL)
    {
	free ((void *)groupPtrs);
    }
    if (channelNames != NULL)
    {
	free ((void *)channelNames);
    }
    if (channelPtrs != NULL)
    {
	free ((void *)channelPtrs);
    }

    // delete the printer
    if (printer != NULL)
    {
	delete printer;
    }

    // close down tisGalaxy if used
    if (initedTisGalaxy == vTRUE)
    {
	stopTisGalaxy (tisGalaxyResource);
    }

    // delete some dialogs
    if (systemDialogPtr != NULL)
    {
	delete systemDialogPtr;
    }
    if (alarmEventDialogPtr != NULL)
    {
	delete alarmEventDialogPtr;
    }
    if (alarmHistoryDialogPtr != NULL)
    {
	delete alarmHistoryDialogPtr;
    }
    if (alarmSummaryDialogPtr != NULL)
    {
	delete alarmSummaryDialogPtr;
    }
    if (alarmTrackDialogPtr != NULL)
    {
	delete alarmTrackDialogPtr;
    }
    if (alarmWindowDialogPtr != NULL)
    {
	delete alarmWindowDialogPtr;
    }
    if (commentDialogPtr != NULL)
    {
	delete commentDialogPtr;
    }
    if (dynamicGroupDialogPtr != NULL)
    {
	delete dynamicGroupDialogPtr;
    }
    if (expandDialogPtr != NULL)
    {
	delete expandDialogPtr;
    }
    if(guidanceDialogPtr != NULL)
    {
	delete guidanceDialogPtr;
    }
    if (popAlarmPtr != NULL)
    {
	delete popAlarmPtr;
    }
    if (reportsDialogPtr != NULL)
    {
	delete reportsDialogPtr;
    }
    // delete the main dialog
    if (mainDialog != NULL)
    {
	delete mainDialog;
    }
}


/********************************************************************************
*
*    AlarmVu application connect to alarmMgr function
*
********************************************************************************/

int alvuApplication::ContactAlarmMgr (void)
{
    struct hostent	*hostval;
    struct sockaddr_in	setPort;
    short		msgPort = MSGPORT;
    int			optionValue = 1;
    int			error;

    if ((alarmhndSocket = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {
#ifdef WIN32
	error = WSAGetLastError ();
#else
	error = errno;
#endif WIN32
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_MgrSocketErr, error);
	return (ALVU_ERROR);
    }

    memset ((char *) &setPort, 0, sizeof (setPort));
    setPort.sin_family = AF_INET;
    hostval = gethostbyname ("localhost");
    if (hostval == NULL) 
    {
#ifdef WIN32
	error = WSAGetLastError ();
#else
	error = errno;
#endif WIN32
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_LocalHostErr, error);
	return (ALVU_ERROR);
    }
    setPort.sin_addr.s_addr =**((long **)(hostval -> h_addr_list));
    setPort.sin_port = htons(msgPort);
    if (connect (alarmhndSocket, (struct sockaddr *) &setPort, sizeof (setPort)) == -1)
    {
#ifdef WIN32
	error = WSAGetLastError ();
#else
	error = errno;
#endif WIN32
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_MgrConnectErr, error);
	return (ALVU_ERROR);
    }
    /*
     * should be:
     *
     *     if (ioctl (listenSocket, FIONBIO, &optionValue) == -1)
     *
     * but doesn't work with C++ compiler
     */
#ifdef WIN32
    if (ioctlsocket (alarmhndSocket, FIONBIO, (unsigned long *) &optionValue) == -1)
#else
    if (ioctl (alarmhndSocket, (0x80000000|((sizeof(int)&0xff)<<16)|('f'<<8)|126), 
		(caddr_t) &optionValue) == -1)
#endif WIN32
    {
#ifdef WIN32
	error = WSAGetLastError ();
#else
	error = errno;
#endif WIN32
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_SocketBlkErr, errno);
	return (ALVU_ERROR);
    }

    return (ALVU_SUCCESS);
}


/*******************************************************************************
*
*   AlarmVu application init printer function
*   At present this function is only good on a windows machine
*
*******************************************************************************/

void alvuApplication::InitPrinter (void)
{
#ifndef WIN32
    return;
#else
    // get default printer context
    printer = vprint::CreatePreferred ();
    if (printer == NULL)
    {
 	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_NoPrinterMsg);
	return;
    }
   
    // set up the printer for landscape print
    PRINTDLG *pd = (PRINTDLG *) printer -> GetDevice ();
    if (pd == NULL)
    {
 	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_NoPrinterMsg);
	delete printer;
	printer = NULL;
	return;
    }
    if (pd -> hDC != NULL)
    {
	DeleteDC (pd -> hDC);
    }
    DEVMODE *dm = (DEVMODE *)(GlobalLock (pd -> hDevMode));
    if (dm == NULL)
    {
	GlobalUnlock (pd -> hDevMode);
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_NoPrinterMsg);
	delete printer;
	printer = NULL;
	return;
    }
    dm -> dmOrientation = DMORIENT_LANDSCAPE;
    dm -> dmFields |= DM_ORIENTATION;
    pd -> hDC = CreateDC ("WINSPOOL", (char *)dm -> dmDeviceName, NULL, dm);
    GlobalUnlock (pd -> hDevMode);
    if (pd -> hDC != NULL)
    {
	// undocumented Galaxy func
	_setupMSWPrinter (printer, pd -> hDC);
    }
    // get screen display, logicall pixel value/screen inch
    HDC dc = CreateDC ("DISPLAY", NULL, NULL, NULL);
    logicalPixelsY = GetDeviceCaps (dc, LOGPIXELSY);
    DeleteDC (dc);
#endif	WIN32
}


/********************************************************************************
*
*    AlarmVu application init setup function
*
********************************************************************************/

void alvuApplication::InitSetup (void)
{
#ifndef WIN32
    sprintf (setupFileName, "%s/%s", ALVU_DBPATH, ALVU_SETUP_FILE);
#else
    sprintf (setupFileName, "%s\\%s", ALVU_DBPATH, ALVU_SETUP_FILE);
#endif WIN32

    FILE		*fileDesc = (FILE *)0;
    int			numItems;
    unsigned int	magicNumber;
    unsigned int	version;
    unsigned char	alarmColors [NUM_SEV_COLORS][3];

    if ( (fileDesc = fopen (setupFileName, "r")) &&
	 ((numItems = fread ((char *)&magicNumber, sizeof (magicNumber), 1, fileDesc)) == 1) &&
	 (magicNumber == ALARMVU_MAGIC_NUMBER) &&
	 ((numItems = fread ((char *)&version, sizeof (version), 1, fileDesc)) == 1) &&
	 ((version == ALVU_NEW_VERSION) || (version == ALARMVU_VERSION)) &&
	 ((numItems = fread (linePrinter, sizeof (linePrinter), 1, fileDesc)) == 1) &&
	 ((numItems = fread ((char *)alarmColors, sizeof (alarmColors), 1, fileDesc)) == 1) &&
	 ((numItems = fread (defFilterName, sizeof (defFilterName), 1, fileDesc)) == 1) &&
	 ((numItems = fread ((char *)&winEnable, sizeof (winEnable), 1, fileDesc)) == 1) &&
	 ((numItems = fread ((char *)&popupAlrmEnable, sizeof (popupAlrmEnable),1, fileDesc)) == 1))
    {
	fclose (fileDesc);
	int i;
	for (i = 0; i < NUM_SEV_COLORS; i++)
	{
	    SevColor[i] = Color (alarmColors[i][0], alarmColors[i][1], alarmColors[i][2]);
	    UnackSevColor[i] = Color (alarmColors[i][0], alarmColors[i][1], alarmColors[i][2]);
	}
    }
    else
    {
	if (fileDesc)
	{
	    fclose (fileDesc);
	    if (numItems != 1)
	    {
		alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, 
					 alvu_SetupFileReadErr, setupFileName);
	    }
	    else if (magicNumber != ALARMVU_MAGIC_NUMBER)
	    {
		alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE,
					 alvu_NotSetupFileErr, setupFileName);
	    }
	    else if ((version != ALVU_NEW_VERSION) || (version != ALARMVU_VERSION))
	    {
		alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE,
					 alvu_SetupVersionErr, setupFileName, version,
					 ALVU_NEW_VERSION, ALARMVU_VERSION);
	    }
	}
	strcpy (linePrinter, "");
	popupAlrmEnable = vTRUE;
    }
    // setup default shell filter
    strcpy (shellFilter.name, ALVU_DEFAULT_FILTER_NAME);
    strcpy (shellFilter.group, "");
    shellFilter.ack = 0;
    shellFilter.sev = 0;
    shellFilter.alarmOnly = ALLFILTER;
    strcpy (shellFilter.start, "");
    strcpy (shellFilter.end, "");
    strcpy (shellFilter.op, "");
    shellFilter.startupDisplay = 0;
    shellFilter.modified = vFALSE;

    // some other inits
    alarmSoundEnabled = vTRUE;
    alarmBlinkLevel = ALVU_BLINK_LEVEL_DEFAULT;
    MaxSendsPerSecond = ALVU_MAX_SENDS_DEFAULT;
}



/********************************************************************************
*
*    AlarmVu application mapper to shared memory with alarmMgr
*
********************************************************************************/

int alvuApplication::MapSharedMemory (void)
{
    int		shrMemId;
    char	*shrMemSegPtr;
    int		i;
    int		error = vFALSE;
#ifdef WIN32
    int		shmkey = AL_SHR_MEM_KEY;
#else
    key_t	shmkey = AL_SHR_MEM_KEY;
#endif WIN32

    if ((shrMemId = shmget (shmkey, AL_SEGMENT_SIZE, 0666)) == -1)
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmgetFail);
	return (ALVU_ERROR);
    }
    else if ((shrMemSegPtr = (char *)(shmat (shrMemId, (char *) AL_SHARED_BASE, 0))) == (char *) -1)
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmatFail);
	return (ALVU_ERROR);
    }
    shareHeader = (ShareHeader *)shrMemSegPtr;
    if (shareHeader->shareSignature != AL_SIGNATURE)
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmemFormatErr);
	return (ALVU_ERROR);
    }

    numShrMemSegs = (int)(shareHeader -> shareSegs);
    treeRoot = shareHeader -> treeRoot;
    vexWITH_HANDLING
    {
 	shrMemIds = (int *)(vmemAllocAndClear (sizeof (int) * numShrMemSegs));
    }
    vexON_EXCEPTION
    {
	alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmemAllocFail);
	error = vTRUE;
    }
    vexEND_HANDLING
    if (error == vTRUE)
    {
	return (ALVU_ERROR);
    }

    shrMemIds[0] = shrMemId;
    for (i = 1; i < numShrMemSegs; i++)
    {
	if ((shrMemIds[i] = shmget (shmkey + i, AL_SEGMENT_SIZE, 0666)) == -1)
	{
	    alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmgetFail);
	    return (ALVU_ERROR);
	}
	else
	{
	    shrMemSegPtr = (char *)(shmat (shrMemIds[i], (char *)shareHeader+(i*AL_SEGMENT_SIZE), 0));
	    if  (shrMemSegPtr == (char *) -1)
	    {
		alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmatFail);
		return (ALVU_ERROR);
	    }
	}
    }
    return (ALVU_SUCCESS);
}


/********************************************************************************
*
*    AlarmVu application read command line args that are not 
*	galaxy args
*
********************************************************************************/

int alvuApplication::ReadCmdLineArgs (void)
{
    int newArgc;
    const vchar **newArgv;

    vpref::GetArgs (&newArgc, &newArgv);

    int cmd;
    char argerr = 0;

    while ((cmd = getopt (newArgc, (char **)newArgv, "f:g:s:a:o:b:d:e:r:k:t:w:h")) != EOF)
    {
	switch (cmd)
	{
	    case 'f':	// use this filter template
		if (*optarg != '-')
		{
		    strncpy (shellFilter.name, optarg, sizeof (shellFilter.name));
		    shellFilter.name [AL_FILTERNAME_SIZE] = 0;
		    GetFilter (shellFilter.name, &shellFilter);
		}
		else
		{
		    argerr = 'f';
		}
		break;

	    case 'g':	// display alarm starting at group
		if (*optarg != '-')
		{
		    strncpy (shellFilter.group, optarg, sizeof (shellFilter.group));
		    shellFilter.group [AL_GROUPNAME_SIZE] = 0;
		}
		else
		{
		    argerr = 'g';
		}
		break;

	    case 's':	// hide alarms with selected status
		if (*optarg != '-')
		{
		    if (*optarg == 'M')
			shellFilter.sev |= MAJORFILTER;
		    else if (*optarg == 'm')
			shellFilter.sev |= MINORFILTER;
		    else if (*optarg == 'I')
			shellFilter.sev |= INFOFILTER;
		    else if (*optarg == 'E')
			shellFilter.sev |= EVENTFILTER;
		    else
		    {
			argerr = 's';
			break;
		    }
		    shellFilter.modified = vTRUE;
		}
		else
		{
		    argerr = 's';
		}
		break;

	    case 'a':	// show all or only unacknowledged alarms
		if (*optarg != '-')
		{
		    if (*optarg == 'A')
			shellFilter.ack = ACKFILTER;
		    else if (*optarg == 'u')
			shellFilter.ack = UNACKFILTER;
		    else
		    {
			argerr = 'a';
			break;
		    }
		    shellFilter.modified = vTRUE;
		}
		else
		{
		    argerr = 'a';
		}
		break;

	    case 'o':	// restrict display to operator
		if (*optarg != '-')
		{
		    strncpy (shellFilter.op, optarg, sizeof (shellFilter.op));
		    shellFilter.op [AL_OPERATOR_SIZE] = 0;
		}
		else
		{
		    argerr = 'o';
		}
		break;

	    case 'b':	// show entries after time
		if (*optarg != '-')
		{
		    if ((ValidTimeSpec (optarg)) == ALVU_SUCCESS)
		    {
			strncpy (shellFilter.start, optarg, sizeof (shellFilter.start));
			shellFilter.start [sizeof (shellFilter.start) - 1] = 0;
		    }
		    else
		    {
			argerr = 'b';
		    }
		}
		else
		{
		    argerr = 'b';
		}
		break;

	    case 'd':	// show entries - hours before alarmvu starts
		if (*optarg != '-')
		{
		    time_t now = time (NULL);
		    int timeInSecs = atoi (optarg) * 3600;
		    now -= timeInSecs;
		    struct tm *tmnow = localtime (&now);
		    sprintf (shellFilter.start, "%2.2d/%2.2d/%2.2d %2.2d:%2.2d:%2.2d",
						tmnow -> tm_mon + 1,
						tmnow -> tm_mday,
						tmnow -> tm_year % 100,
						tmnow -> tm_hour,
						tmnow -> tm_min,
						tmnow -> tm_sec);
		    shellFilter.modified = vTRUE;
			     
		}
		else
		{
		    argerr = 'd';
		}
		break;

	    case 'e':	// show entries before time
		if (*optarg != '-')
		{
		    if ((ValidTimeSpec (optarg)) == ALVU_SUCCESS)
		    {
			strncpy (shellFilter.end, optarg, sizeof (shellFilter.end));
			shellFilter.end [sizeof (shellFilter.end) - 1] = 0;
		    }
		    else
		    {
			argerr = 'e';
		    }
		}
		else
		{
		    argerr = 'e';
		}
		break;

	    case 'r':	// disable/enable alarm sounds
		if (*optarg != '-')
		{
		    int soundState = atoi (optarg);
		    if ((soundState == 0) || (soundState == 1))
		    {
			alarmSoundEnabled = (soundState == 0) ? vFALSE : vTRUE;
		    }
		    else
		    {
			argerr = 'r';
		    }
		}
		else
		{
		    argerr = 'r';
		}
		break;

	    case 'k':
		if (*optarg != '-')
		{
		    int blinkLevel = atoi (optarg);

		    if ((blinkLevel < ALVU_MIN_BLINK_LEVEL) ||
			(blinkLevel > ALVU_MAX_BLINK_LEVEL) )
		    {
			argerr = 'k';
		    }
		    else
		    {
			alarmBlinkLevel = blinkLevel;
		    }
		}
		else
		{
		    argerr = 'k';
		}
		break;

	    case 't':
		if (*optarg != '-')
		{
		    MaxSendsPerSecond = atoi (optarg);
		}
		else
		{
		    argerr = 't';
		}
		break;

	    case 'w':
		if (*optarg != '-')
		{
		    groupdisplayHeight = atoi (optarg);
		}
		else
		{
		    argerr = 'w';
		}
		break;

	    case 'h':
		alvuUsage ();
		return (ALVU_ERROR);

	    case '?':
	    {
		char badArg = optopt & 0xff;
		alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, 
					 alvu_IllegalCmdLineArg, badArg);
		alvuUsage ();
		return (ALVU_ERROR);
	    }

	}
	if (argerr != 0)
	{
	    alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_IllegalCmdLineArg, argerr);
	    alvuUsage ();
	    return (ALVU_ERROR);
	}
    }

    return (ALVU_SUCCESS);
}


/********************************************************************************
*
*    AlarmVu application startup display function
*
********************************************************************************/

void alvuApplication::StartupDisplay (void)
{
    Filter  tempFilter;

    if (shellFilter.startupDisplay)
    {
	vresource alarmDlgRes = vresourceGet (alvuResource, alvu_AlarmDialog);

	switch (shellFilter.startupDisplay)
	{
	    case ALSUMSTART:
		alarmSummaryDialogPtr = new AlarmSummaryDialog (alarmDlgRes);
		alarmSummaryDialogPtr -> AlarmDialogOpen (this, &this -> shellFilter);
		break;

	    case GRPSTART:
		if (FindGroup (treeRoot, shellFilter.group))
		{
		    dynamicGroupDialogPtr = new DynamicGroupDialog (alarmDlgRes);
		    dynamicGroupDialogPtr -> AlarmDialogOpen (this, &this -> shellFilter);
		    break;
		}
		else
		{
		    vstr *groupPtr = vstrCloneScribed (vcharScribeSystem (shellFilter.group));
		    alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE,
					     alvu_StartupGroupNotExist, groupPtr);
		    vstrDestroy (groupPtr);
		}
		break;

	    case ALHISTSTART:
		tempFilter = shellFilter;
		tempFilter.sev |= EVENTFILTER;
		tempFilter.modified = True;
		alarmHistoryDialogPtr = new AlarmHistoryDialog (alarmDlgRes);
		alarmHistoryDialogPtr -> AlarmDialogOpen (this, &tempFilter);
		break;
	    
	    case EVHISTSTART:
		tempFilter = shellFilter;
		tempFilter.sev = NOALARMFILTER | INFOFILTER | MINORFILTER | MAJORFILTER;
		tempFilter.modified = True;
		alarmEventDialogPtr = new AlarmEventDialog (alarmDlgRes);
		alarmEventDialogPtr -> AlarmDialogOpen (this, &tempFilter);
		break;

	    case OPTRKSTART:
		alarmTrackDialogPtr = new AlarmTrackDialog (alarmDlgRes);
		alarmTrackDialogPtr -> AlarmDialogOpen (this, &this -> shellFilter);
		break;

	    case REPORTSTART:
		reportsDialogPtr = new AlarmPointsDialog (alarmDlgRes);
		reportsDialogPtr -> AlarmDialogOpen (this, &this -> shellFilter);
		break;

	    case SYSTEMSTART:
		systemDialogPtr = new AlarmSystemDialog ();
		systemDialogPtr -> AlarmDialogOpen (this, &this -> shellFilter);
		break;
	}
    }
}


/********************************************************************************
*
*    AlarmVu application unmpa shared memory with alarmMgr
*
********************************************************************************/

void alvuApplication::UnMapSharedMemory (void)
{
    int		i;


    numShrMemSegs = (int)(shareHeader -> shareSegs);
    for (i = 0; i < numShrMemSegs; i++)
    {
	if (shmdt ((char *)shareHeader + (i * AL_SEGMENT_SIZE)) == -1)
	{
	    alvuMessageDialogCenter (NULL, ALVU_INFO_NOTICE, alvu_ShmdtFail);
	    break;
	}
    }
    shareHeader = NULL;
    if (shrMemIds != NULL)
    {
	vmemFree ((void *)shrMemIds);
    }
}



/********************************************************************************
*
*    AlarmVu application unmpa shared memory with alarmMgr
*
********************************************************************************/

void alvuApplication::alvuUsage (void)
{
    vstr *str = vresourceGetString (messagesResource, alvu_LegalCmdLineArgs);
    char *cStr = (char *)(vcharExportAlloc (vcharScribeSystemExport ((vchar *)str)));
    printf ("%s\n", cStr);
    vstrDestroy (str);
    vcharExportFree (cStr);
}

