#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vrectHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vdialogHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vlistviewHEADER
#include vfontHEADER
#include vmenubarHEADER
#include vlistHEADER
#include vprintHEADER

extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
// this bit of sillyness is needed because windows #defines AppendMenu
// as #define AppendMenu AppendMenuA, which interfers with galaxy class
// vmenubar::AppendMenu
#undef AppendMenu
#endif WIN32
#include "cadef.h"
#include "shareMem.h"
#include "tsDefs.h"
#include "alarmDefs.h"
#include "alarmfil.h"
#include "alarmfmt.h"
#include "alarmvu.h"
}

#include "alvu.h"
#include "alarmvu.hxx"
#include "aldlgListView.hxx"
#include "alarmDialogs.hxx"
#include "alvuUtils.hxx"

extern long	CurrUnackSevColor [];	// see alarmvu.cxx
extern long	SevColor [];		// see alarmvu.cxx

// *** AldlgItemList CLASS **********************************************

// implementation of the vkindTyped overrides
vkindTYPED_FULL_DEFINITION(AldlgItemList, vdialogItemList, "AldlgItemList");

/**********************************************************************
*
*    AldlgItemList::DetermineBounds
*	return the bounding box of the owning object in the 
*	coordinate system of its window.
*
***********************************************************************/
 
void AldlgItemList::DetermineBounds (register vrect *rect)
{
    ((AldlgListView *)GetOwner()) -> GetItemListIn() -> DetermineBounds (rect);
} 
    
/**********************************************************************
*
*    AldlgItemList::DetermineColorsFrom
*	given an owner, figure out a set of colors to use for drawing.
*	note: phlags is spelled to avoid hiding vdialogItemList::flags
*
***********************************************************************/
 
void AldlgItemList::DetermineColorsFrom (long phlags, vcolor **known,
				       int *ids, int count, vcolor **colors)
{
    ((AldlgListView *)GetOwner()) -> DetermineColorsFrom (phlags, known,
							  ids, count,
							  colors);
}


/**********************************************************************
*
*    AldlgItemList::DetermineColorSchemeFrom
*	given an owner, figure out the color scheme
*	-> known   -- a set of colors that we know already
*	-> phlags  -- the elements that are set in known
*	-> scheme  -- the color scheme to modify to fill in the 
*		      unknown slots
*	NOTE: 'phlags' is spelled so to avoid hiding vdialogItemList::flags
*
***********************************************************************/
 
void AldlgItemList::DetermineColorSchemeFrom (vcolorScheme *known, 
					      unsigned long phlags, 
					      vcolorScheme *scheme)
{
    ((AldlgListView *)GetOwner ()) -> DetermineColorSchemeFrom (known,
								phlags,
								scheme);
}

/**********************************************************************
*
*    AldlgItemList::DetermineContent
*	return the bounding box of the visible portion of the owning 
*	object in the coordinate system of its window.
*
***********************************************************************/
 
void AldlgItemList::DetermineContent (vrect *rect)
{
    ((AldlgListView *)GetOwner()) -> GetItemListIn() -> DetermineContent (rect);
}


/**********************************************************************
*
*    AldlgItemList::DetermineCursor
*	given an owner, figure the cursor that should be used when 
*	the mouse pointer is over this area.
*	Just use the cursor that's set for the owning dialog item.
*
***********************************************************************/
 
vcursor *AldlgItemList::DetermineCursor ()
{
    return ((AldlgListView *)GetOwner ()) -> DetermineCursor ();
}

/**********************************************************************
*
*    AldlgItemList::DetermineDialog
*	given an owner, figure out what dialog it's in.
*
***********************************************************************/
 
vdialog *AldlgItemList::DetermineDialog ()
{
    return ((AldlgListView *)GetOwner ()) -> DetermineDialog ();
}

/**********************************************************************
*
*    AldlgItemList::DetermineEnabled
*	given an owner, figure out if it's currently enabled.
*	This allows for a container-like object to disable its contents 
*	when it gets disabled.
*
***********************************************************************/
 
int AldlgItemList::DetermineEnabled ()
{
    return ((AldlgListView *)GetOwner()) -> DetermineEnabled ();
}


/**********************************************************************
*
*    AldlgItemList::DetermineFont
*	given an owner, figure out the font to use for text drawing.
*	Just use the font that'd be used for text drawing in owner.
*
***********************************************************************/
 
vfont *AldlgItemList::DetermineFont ()
{
    return ((AldlgListView *)GetOwner())->DetermineFont();
}

/**********************************************************************
*
*    AldlgItemList::DetermineLineWidth
*	given an owner, figure out the line width to use for vdraw 
*	Just use the line width that'd be used for path stroking 
*	for the owner.
*
***********************************************************************/
 
int AldlgItemList::DetermineLineWidth ()
{
    return ((AldlgListView *)GetOwner ()) -> DetermineLineWidth ();
}

/**********************************************************************
*
*    AldlgItemList::DetermineShown
*	given an owner, figure out if it's currently visible
*	This allows for a container-like object to hide its contents 
*	when it gets hidden.
*
***********************************************************************/
 
int AldlgItemList::DetermineShown ()
{
    return ((AldlgListView *)GetOwner ()) -> DetermineShown ();
} 


/**********************************************************************
*
*    AldlgItemList::DrawDevice
*	the drawing device is being set for an item in
*	the item list.
*
***********************************************************************/
 
void AldlgItemList::DrawDevice()
{
    AldlgListView *view = (AldlgListView *)(GetOwner());

    view -> DrawDevice ();

    vdraw::RectsClip (view -> GetListRect(), 1);

}

/**********************************************************************
*
*    AldlgItemList::GetKeyDispatch
*	given an owner, figure out the key dispatch dict to use.
*	Just use the key dispatch dict	that'd be used for our owner.
*
***********************************************************************/
 
vdict *AldlgItemList::GetKeyDispatch ()
{
    return ((AldlgListView *)GetOwner ()) -> GetKeyDispatch ();
}

/**********************************************************************
*
*    AldlgItemList::GetKeyMap ()
*	given an owner, figure out the key map to use.
*	Just use the key map that'd be used for our owner.
*
***********************************************************************/
 
vdict *AldlgItemList::GetKeyMap ()
{
    return ((AldlgListView *)GetOwner ()) -> GetKeyMap( );
}


// *** AldlgListView CLASS *********************************************

vkindTYPED_FULL_DEFINITION(AldlgListView, vlistview, "AldlgListView");

// the functions which follow are the method overrides for our listview
// subclass.

/**********************************************************************
*
*    AldlgListView::LoadInit ()
*
***********************************************************************/
 
void AldlgListView::LoadInit (vresource resource)
{
    vlistview::LoadInit(resource);
    entry    = NULL;
    itemList = new AldlgItemList;
    lastList = NULL;
    firstList = vTRUE;

    itemList -> SetOwner(this);

}

/**********************************************************************
*
*    AldlgListView::Destroy ()
*	Clean up any memory we may have consumed.
*
***********************************************************************/
 
void AldlgListView::Destroy ()
{
    // nuke our item list
    if (itemList != NULL) 
    {
	// this destroys the dialog items contined inside
	delete itemList;
    }

    // sign our death warrant
    vlistview::Destroy ();

}


/**********************************************************************
*
*    AldlgListView::ChangedtAttribute ()
*	An attribute's value changed in our parent
*	-> attr     -- the ID of the attribute being changed
*	-> context  -- the context of this attribute
*	Right now we're just sniffing out changes in our "Bounds" 
*	attribute in order to propagate the change on to our item list
*
***********************************************************************/
 
void AldlgListView::ChangedAttribute (const vname *attr, vdict *context)
{
    vlistview::ChangedAttribute (attr, context);

    if (attr == vname_Bounds) 
    {
	itemList -> ChangedAttribute (attr, context);
    }

}

/**********************************************************************
*
*    AldlgListView::ChangedParentAttribute ()
*	An attribute's value changed in our parent
*	-> attr     -- the ID of the attribute being changed
*	-> context  -- the context of this attribute
*	Right now we're just sniffing out changes in our "Bounds" 
*	attribute in order to propagate the change on to our item list
*
***********************************************************************/
 
void AldlgListView::ChangedParentAttribute (const vname *attr, vdict *context)
{
    vlistview::ChangedParentAttribute (attr, context);

    if (attr == vname_Bounds) 
    {
	itemList -> ChangedParentAttribute (attr, context);
    }

}


/**********************************************************************
*
*    AldlgListView::Close ()
*	Do what's necessary when the listview gets closed.
*
***********************************************************************/
 
void AldlgListView::Close ()
{
    // close our dialog items.
    itemList -> Close ();

    //  close ourselves.
    vlistview::Close();
}


/**********************************************************************
*
*    AldlgListView::DrawCellBackgroud ()
*	by sending selected as FALSE, background color does not chg
*
***********************************************************************/
 
void AldlgListView::DrawCellBackground (int row, int col, int selected, 
					vrect *cellRect)
{
    vlistview::DrawCellBackground (row, col, vFALSE, cellRect);
}


/**********************************************************************
*
*    AldlgListView::DrawCellContents ()
*
***********************************************************************/
 
void AldlgListView::DrawCellContents (int row, int col, int selected, 
					vrect *cellRect)
{
    vlist *list = GetList();
    ListEntry *tempEntry = (ListEntry *)(list -> GetCellValue (row, col));
    if (!tempEntry)
	return;

    // save current graphics state
    vdrawGSave();
    vdrawRectsClip(cellRect, 1);

    int visibleRow = row - GetTopRow ();
    double red, green, blue;
    int colorIdx = tempEntry -> GetColor ();
    long color;
    if (colorIdx <= NUM_SEV_COLORS)
    {
	color = SevColor [colorIdx];
    }
    else
    {
	color = CurrUnackSevColor [colorIdx - NUM_SEV_COLORS];
    }
    GetRGBColor (color, &red, &green, &blue);
    vcolorSpec *spec = new vcolorSpec;
    spec -> FSetRGB (red, green, blue);
    vcolor *fg = vcolor::Intern (spec);

    vrect temp = *cellRect;
    vdrawRectClip(temp.GetX(), temp.GetY(), temp.GetWidth(), temp.GetHeight());

    // draw the  button
    vrect buttonRect = temp;
    tempEntry -> buttonItem = ackButtons [visibleRow];
    ackButtons [visibleRow] -> StoreListEntry (tempEntry);
    buttonRect.SetX (buttonRect.GetX () + 3);
    buttonRect.SetY (buttonRect.GetY () + 1);
    buttonRect.SetWidth (temp.GetHeight () -2);
    buttonRect.SetHeight (temp.GetHeight () -2);
    ackButtons [visibleRow] -> SetRect (&buttonRect);
    vdrawSetFont (ackButtons [visibleRow]-> DetermineFont());
    vdrawSetColor (fg);
    ackButtons [visibleRow] -> SetForeground (fg);
    ackButtons [visibleRow] -> DrawDevice ();
    ackButtons [visibleRow] -> SetDraw ();
    ackButtons [visibleRow] -> Draw();

    // draw the text
    vrect textRect = temp;
    textRect.SetX (textRect.GetX () + 3 + buttonRect.GetWidth () + 3);
    textRect.SetY (temp.GetY ());
    textRect.SetHeight (temp.GetHeight ());
    textRect.SetWidth ((temp.GetWidth () - buttonRect.GetWidth ()) - 6);
    vdrawScribedRectShowCenterY (vcharScribeSystem(tempEntry -> GetDesc ()), &textRect);

    // restore current graphics state
    vdrawGRestore();
}


/**********************************************************************
*
*    AldlgListView::Focus ()
*	Do what's necessary when a listview gets focus.
*
***********************************************************************/
 
void AldlgListView::Focus ()
{
    vlistview::Focus ();

    itemList -> Focus ();

}

/**********************************************************************
*
*    AldlgListView::FindItem ()
*
***********************************************************************/
 
vdialogItem *AldlgListView::FindItem (const vname *itemTag)
{
    vdialogItem *ret;

    ret = (vdialogItem *)(itemList -> Find (itemTag));

    return (ret);
}


/*******************************************************************
*
*    AldlgListView::GetCellHeight
*	determine cell height
*
*******************************************************************/

int AldlgListView::GetCellHeight(void)
{
    vfont *font = DetermineFont ();
    int height = font -> Height ();
    int inset = GetDefaultSelectionInset ();
    return (height + (inset * 2));
}

/*******************************************************************
*
*    AldlgListView::GetListNumElements
*	determine listview height
*
*******************************************************************/

int AldlgListView::GetListNumElements(void)
{
    // get listview drawing area
    vrect *listRect = (vrect *)(GetListRect ());
    int height = listRect -> GetHeight ();
    int cellHeight = GetCellHeight ();
    return (height / cellHeight);
}


/**********************************************************************
*
*    AldlgListView::HandleButtonDown ()
*	Handle a button down in the listview.
*	-> event -- the button down event.
*	We look at the item immediately underneath the mouse pointer.
*	If it is one of our dialog items, forward the event to that item.
*
***********************************************************************/
 
int AldlgListView::HandleButtonDown (vevent *event)
{
    int ret = vFALSE;

    if (GetListRect() -> ContainsPoint (event -> GetX (), event -> GetY ())) 
    {
	// let the item list handle the work of figuring out who gets it
	ret = itemList -> HandleButtonDown (event);
    }
    
    // the dialog item didn't handle the event.
    // Let the listview have a chance at it.
    if (!ret) 
    {
	ret = vlistview::HandleButtonDown (event);
    }
    
    return (ret);

}


/**********************************************************************
*
*    AldlgListView::HandleButtonUp ()
*	Handle a button up in the listview
*	-> event  -- the button up event.
*
***********************************************************************/
 
int AldlgListView::HandleButtonUp (vevent *event)
{
    int ret = vFALSE;
    // see if one of our dialogItemInAListview is tracking.
    vdialogItem *item = itemList -> GetDragItem ();
    if (item != NULL) 
    {
	// see if the item wants to handle it
	ret = item -> HandleButtonUp (event);

	// clear out the value so we don't get confused on a subsequent
        // drag or mouse up event
	itemList -> SetDragItem (NULL);
    }

    // either there wasn't a tracking item, or it didn't want to handle
    // this.  We've had our fun, let the listview have a crack at it.
    
    if (!ret) 
    {
	ret = vlistview::HandleButtonUp(event);
    }

    return (ret);
}


/**********************************************************************
*
*    AldlgListView::HandleDrag ()
*	Handle mouse drag event.
*	-> event  -- the drag event.
*
***********************************************************************/
 
int AldlgListView::HandleDrag (vevent *event)
{
    int ret = vFALSE;
    // see if one of our dialogItemInAListview is tracking.
    vdialogItem *item = itemList -> GetDragItem ();

    if (item != NULL) 
    {
	// see if the item wants to handle it.
	ret = item -> HandleDrag (event);

    }

    // either there wasn't a tracking item, or it didn't want to handle
    // this.  We've had our fun, let the listview have a crack at it.
    
    if (!ret) 
    {
        ret = vlistview::HandleDrag (event);
    }
	
    return (ret);

}


/**********************************************************************
*
*    AldlgListView::HandleKeyDown ()
	handle a key press in the listview.
*	-> event  -- the key event.
*
***********************************************************************/
 
int AldlgListView::HandleKeyDown (vevent *event)
{
    int ret = vFALSE;

    // try to bind the key to our key map
    vkeyStroke key = event -> GetKeyStroke ();
    
    const vname *binding = LoadKeyMap (key);
    if (binding != NULL) 
    {
	event -> SetBinding (binding);
    }

    // try to dispatch the event to an item in the key dispatcher

    vdialogItem *item = (vdialogItem *)LoadKeyDispatch (key);
    if ((item != NULL) && item -> IsEnabled () && item -> HandleKeyDown (event)) 
    {
	ret = vTRUE;
    }

    if (!ret) 
    {
	item = itemList -> GetFocusItem ();
	if ((item != NULL) && item -> IsEnabled () &&
	    item -> HandleKeyDown (event))
	{
	    ret = vTRUE;
	} else 
	{
	    ret = vlistview::HandleKeyDown (event);
	}
    }

    return (ret);
}


/**********************************************************************
*
*    AldlgListView::IssueSelector ()
*	handle selector commands issued against the listview
*	-> selector -- the ID of the command being issued
*	-> context -- the context this command is issuing in.
*	Right now we're just sniffing the focus moving so we can move 
*	the focus in our owned itemlist.
*
***********************************************************************/
 
int AldlgListView::IssueSelector (int selector, vdict *context)
{
    int ret;

    switch (selector) 
    {
	case vdialogITEM_SELECT_MOVE_FOCUS: 
	{
	    const int *direction =
	      (const int *)context -> GetElem (vname_Direction);
	    if (direction != NULL) 
	    {
	        itemList -> MoveFocus (*direction,
                                       context -> GetElem (vname_Text)
                                       != NULL);

		// since we're effectively a "flat" container, see if we
		// need to pop ourselves out
		
		if (itemList -> HasFocusWrapped ()) 
		{
		    // let superclass handle it
		    ret = vFALSE;
		} else 
		{
		    ret = vTRUE;
		}
	    }
	    break;
	}
    }

    if (!ret) 
    {
        vlistview::IssueSelector(selector, context);
    }

    return (ret);

}



/*******************************************************************
*
*    AlarmListView Class 
*	Scrolls the contents in the listview
*
*******************************************************************/

void AldlgListView::ObserveScroll (int horzDistance, int vertDistance)
{
    vlistview::ObserveScroll (horzDistance, vertDistance);
}


/*******************************************************************
*
*    AlarmListView Class 
*	listview observer
*
*******************************************************************/

void AldlgListView::ObserveSelect(int state)
{
    vlist *list = GetList ();
    vlistSelection *selected = GetSelection ();
    int row, column;
    list -> GetSelectedCell (selected, &row, & column);
    if (row == vlistNO_CELLS_SELECTED)
    {
	entry = NULL;
	return;
    }

    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	ListEntry *tempEntry = (ListEntry *)(list -> GetCellValue(row, 0));
	if (tempEntry == NULL)
	    return;

	// update listview current entry pointer to that from the
	// selected cell
	entry = tempEntry;
	if (!alarmDialog -> FileBased())
	{
	    alvuApplication *app = alarmDialog -> GetApp ();
 	    vstr *format = vresourceGetString (app -> messagesResource,
					       alvu_ItemCountMsg);
	    vstr *msg = vstrCloneScribed (vcharScribeFormat (format, 
							     row + 1, 
							     alarmDialog -> NumItems()));
	    vstrDestroy (format);
	    alarmDialog -> elementCount -> SetText (msg);
	    vstrDestroy (msg);
	}
	else
	{
	    alarmDialog -> elementCount -> SetTextScribed (vcharScribeSystem (""));
	}


	// if we clicked in the textitem, need to do something in the
	// control function
	int currx, curry;
	GetLastPointerPosition (&currx, &curry);
	vrect tempRect;
	GetCellRect (row, 0, &tempRect);
	AckButton *button = tempEntry -> GetButtonItem ();
	const vrect *buttonRect = button -> GetRect ();
	tempRect.SetX (tempRect.GetX () + 3 + buttonRect -> GetWidth () + 3);
	if (tempRect.ContainsPoint (currx, curry))
	{
	    AlarmDialog *alarmDlg = tempEntry -> GetAlarmDialog ();
	    alarmDlg -> alarmListView -> SetListEntry (tempEntry);
	    alarmDlg -> CtrlFunc (EXPANDCLICK, row);
	}
   }
}


/**********************************************************************
*
*    AldlgListView::Open ()
*	Do what's necessary when the listview gets opened
*	(i.e. mapped on the screen and ready to accept user input.
*
***********************************************************************/
 
void AldlgListView::Open ()
{
    // open ourselves
    vlistview::Open ();

    itemList -> Open ();
}

/**********************************************************************
*
*    AldlgListView::ScrollAbsolute ()
*
***********************************************************************/
 
void AldlgListView::ScrollAbsolute (int x, int y)
{
    vlistview::ScrollAbsolute (x, y);
}

/**********************************************************************
*
*    AldlgListView::SetColumnWidth ()
*
***********************************************************************/
 
void AldlgListView::SetColumnWidth (int column, int width)
{
    vlistview::SetColumnWidth (column, width);
}

/**********************************************************************
*
*    AldlgListView::SetList ()
*
***********************************************************************/

void AldlgListView::SetList (vlist *list)
{
    vlistview::SetList(list);
    int i;

    if (list == NULL)
    {
	return;
    }

    if (((itemList != NULL) || (list != lastList)) && (firstList != vTRUE))
    {
	delete itemList;
	itemList = new AldlgItemList;

	itemList -> SetOwner(this);
	itemList -> Open ();

	for (i = 0; i < ALVU_MAXITEMS; i++)
	{
	    ackButtons [i] = new AckButton;
	    itemList -> Append (ackButtons[i]);
	}
    }
    else if (firstList == vTRUE)
    {
 	for (i = 0; i < ALVU_MAXITEMS; i++)
	{
	    ackButtons [i] = new AckButton;
	    itemList -> Append (ackButtons[i]);
	}
   }
    lastList = list;
    firstList = vFALSE;
}

/**********************************************************************
*
*    AldlgListView::SetRowHeight ()
*
***********************************************************************/

void AldlgListView::SetRowHeight (int row, int height)
{
    vlistview::SetRowHeight(row, height);
}

/**********************************************************************
*
*    AldlgListView::Unfocus ()
*	Do what's necessary when a listview loses focus.
*
***********************************************************************/
 
void AldlgListView::Unfocus ()
{
    itemList -> Unfocus();

    vlistview::Unfocus();
}


/*******************************************************************
*
*    ListEntry Class 
*
*******************************************************************/

/*******************************************************************
*
*    ListEntry constructor 
*
*******************************************************************/

ListEntry::ListEntry (unsigned int id, int ColorIdx, char *Date, char *Name,
	   char *Desc, TS_STAMP *AlmTime, AlarmDialog *dlg)
{
    ListEntry (id, ColorIdx, False, Date, Name, Desc, AlmTime, dlg);
}

/*******************************************************************
*
*    ListEntry constructor 
*
*******************************************************************/

ListEntry::ListEntry (unsigned int id, int ColorIdx, int AckM, char *Date,
		      char *Name, char *Desc, TS_STAMP *AlmTime, AlarmDialog *dlg)
{
    alarmDialog = dlg;
    buttonItem = NULL;
    row = id;
    AlmColor = ColorIdx;
    ackMany = AckM;
    strncpy (date, Date, sizeof (date));
    date [sizeof (date) - 1] = 0;
    strncpy (name, Name, sizeof (name));
    name [sizeof (name) - 1];
    strncpy (desc, Desc, sizeof (desc));
    desc [sizeof (desc) - 1] = 0;
    if (AlmTime != 0)
    {
	evttime = *AlmTime;
    }
    else
    {
	evttime.secPastEpoch = 0;
	evttime.nsec = 0;
    }
    caChid = 0;
    evId = 0;
    monitored = FALSE;
    curstat = 0;
    curseverity = 0;
    ackstate = 0;
    strcpy (value, "");
}

/*******************************************************************
*
*    ListEntry destructor 
*
*******************************************************************/

ListEntry::~ListEntry (void)
{
}


// some needed functions, outside of the class

void caConnGateCB (connection_handler_args args)
{
//    (((int (*)(void*, connection_handler_args))&ListEntry::ConnCB))(ca_puser (args.chanid), args);
    ((ListEntry *)ca_puser (args.chanid))->ConnCB (args);
}

void caMonGateCB (event_handler_args args)
{
//    (((int (*)(void *, event_handler_args))&ListEntry::MonCB))(args.usr, args);
    ((ListEntry *)args.usr)->MonCB (args);
}

/*******************************************************************
*
*    ListEntry  
*
*******************************************************************/

int ListEntry::Monitor (void)
{
  if (monitored)
    {
	// error, already monitored
	return (False);
    }
    else
    {
	monitored = TRUE;
	// open channel, and start monitors
	ca_build_and_connect (name, TYPENOTCONN, 0, &caChid, NULL, caConnGateCB, this);
	return (True);
    }
}

/*******************************************************************
*
*    ListEntry  
*
*******************************************************************/

int ListEntry::ConnCB (connection_handler_args args)
{
    if (!monitored)
    {
	// spurious callback, monitor was cancelled
	return (False);
    }
    else
    {
	// establish monitor
	ca_add_masked_array_event (OLD_DBR_STS_STRING, 1, caChid, caMonGateCB, this, (float)0., 
				   (float)0., (float)0., &evId, DBE_VALUE | DBE_ALARM);
	return (True);
    }
}


/*******************************************************************
*
*    ListEntry  
*
*******************************************************************/

int ListEntry::MonCB (event_handler_args args)
{
    if (!monitored)
    {
	// spurious callback, monitor was cancelled
	return (False);
    }
    else
    {
	// update value fields, and call action routine.
	strcpy( value, ((struct dbr_sts_string *)args.dbr) -> value);
	value [sizeof (value) - 1] = 0;
        curstat = ((struct dbr_sts_string *)args.dbr) -> status;
        curseverity = ((struct dbr_sts_string *)args.dbr) -> severity;
	if ((curseverity == VALID_ALARM) && (curstat == SBAD_ALARM))
	    curseverity = MAJOR_ALARM;

	ackstate = (*((struct dbr_sts_string *)args.dbr) -> ack == 'Y');
	alarmDialog -> MonitorCB (row);
	return (True);
    }
}


/*******************************************************************
*
*    ListEntry  
*
*******************************************************************/

void ListEntry::StopMonitor ()
{
    if (!monitored)
    {
	// error, not being monitored
    }
    else
    {
	// cancel chid, monitors and all
	if (evId)
	{
	    ca_clear_event (evId);
	}
        evId = 0;
	if (caChid)
	{
	    ca_clear_channel (caChid);
	}
        caChid = 0;
	monitored = FALSE;
    }
}


/*******************************************************************
*
*    ListEntry AckButton class 
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(AckButton, vbutton, "AckButton");

int AckButton::HandleButtonDown (vevent *event)
{
    int whichButton = event -> GetButton ();
    AlarmDialog *alarmDlg = entry -> GetAlarmDialog ();
    alarmDlg -> alarmListView -> SetListEntry (entry);

    if (whichButton == 0)
    {
	if (alarmDlg -> FileBased())
	{
	    alarmDlg -> elementCount -> SetTextScribed (vcharScribeSystem(""));
	}
	else
	{
	    alvuApplication *app = alarmDlg -> GetApp ();
	    vstr *format = vresourceGetString (app -> messagesResource, alvu_ItemCountMsg);
	    vstr *msg = vstrCloneScribed (vcharScribeFormat (format, 
							     entry -> GetRow() + 1, 
							     alarmDlg -> NumItems()));
	    vstrDestroy (format);
	    alarmDlg -> elementCount -> SetText (msg);
	    vstrDestroy (msg);
	}
	alarmDlg -> CtrlFunc (ACKCLICK, entry -> GetRow ());
    }
    else if (whichButton == 2)
    {
	if (alarmDlg -> FileBased())
	{
	    alarmDlg -> elementCount -> SetTextScribed (vcharScribeSystem(""));
	}
	else
	{
	    alvuApplication *app = alarmDlg -> GetApp ();
	    vstr *format = vresourceGetString (app -> messagesResource, alvu_ItemCountMsg);
	    vstr *msg = vstrCloneScribed (vcharScribeFormat (format, 
							     entry -> GetRow() + 1, 
							     alarmDlg -> NumItems()));
	    vstrDestroy (format);
	    alarmDlg -> elementCount -> SetText (msg);
	    vstrDestroy (msg);
	}
	alarmDlg -> DoPopup (event);
    }
    return (vdialogItem::HandleButtonDown (event));
}


int AckButton::HandleButtonUp (vevent *event)
{
    return (vdialogItem::HandleButtonUp (event));
}

void AckButton::DrawContent (void)
{   
    vstr *buttonTitle = 
		vstrCloneScribed (vcharScribeLiteral (entry -> AckMany () ? "+" : " "));

    vdrawGSave ();
    vdrawRectsClip (GetRect (), 1);
    //show the text
    vdrawRectShowCenter (buttonTitle, GetRect ());
    vdrawGRestore ();
    vstrDestroy (buttonTitle);
}

