#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vbuttonHEADER
#include vcomboboxHEADER
#include vprintHEADER

extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
// this bit of sillyness is needed because windows #defines AppendMenu
// as #define AppendMenu AppendMenuA, which interfers with galaxy class
// vmenubar::AppendMenu
#undef AppendMenu
#endif WIN32
#include "cadef.h"
#include "shareMem.h"
#include "alarmDefs.h"
#include "alarmfil.h"
#include "alarmfmt.h"
#include "alarmvu.h"
    
extern ShareHeader	*shareHeader;
extern TreeNode		*treeRoot;
}


#include "alvu.h"
#include "alarmvu.hxx"
#include "comboDialog.hxx"


/*******************************************************************
*
*    ComboDialog Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ComboDialog, vdialog, "ComboDialog");

/*******************************************************************
*
*    ComboDialog constructor
*
*******************************************************************/

void ComboDialog::LoadInit (vresource res)
{
    title = NULL;
    vmemSet ((void *)selected, 0, sizeof (selected));
    comboBox     = ComboBox::RequestSubLoad (this, alvu_ComboBox);
    okButton     = ComboOkButton::RequestSubLoad (this, alvu_OkButton);
    cancelButton = ComboCancelButton::RequestSubLoad (this, alvu_CancelButton);

    vdialog::LoadInit (res);

    // store ptr to dialog in buttons
    comboBox     -> StoreComboDialog (this);
    okButton     -> StoreComboDialog (this);
    cancelButton -> StoreComboDialog (this);
}


/*******************************************************************
*
*    ComboDialog window observer
*
*******************************************************************/

void ComboDialog::ObserveWindow (vevent *event)
{
    if (event -> GetType () == veventWINDOW_STATE && event -> IsClose ())
    {
	vmemSet ((void *)selected, 0, sizeof (selected));
	Close ();
	vevent::StopProcessing ();
    }
}

/*******************************************************************
*
*    ComboDialog destructor
*
*******************************************************************/

void ComboDialog::Destroy ()
{
    if (IsOpen ())
    {
	Close ();
	vevent::StopProcessing ();
    }
    if (title != NULL)
    {
	vstrDestroy (title);
    }
    vdialog::Destroy ();
}


/*******************************************************************
*
*    ComboDialog open function
*
*******************************************************************/

void ComboDialog::OpenCombo (alvuApplication *alvuapp, int listtype, 
			     char **selections, char *selection)
{
    app = alvuapp;
    listType = listtype;

    SetDialogTitle ();
    comboBox -> BuildComboBoxList (selections);
    if (selection != NULL)
    {
	comboBox -> SelectItem (selection);
    }

    Place (vwindow::GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    vevent::Process ();
}


/*******************************************************************
*
*    ComboDialog title function
*
*******************************************************************/

void ComboDialog::SetDialogTitle (void)
{
    vresource titlesRes = app -> titlesResource;
    switch (listType)
    {
	case BROWSE_GROUP_LIST:
	    title = vresourceGetString (titlesRes, alvu_BrowseGroupTitle);
	    break;

	case ENABLE_POINT_LIST:
	    title = vresourceGetString (titlesRes, alvu_EnablePtTitle);
	    break;

	case DISABLE_POINT_LIST:
	    title = vresourceGetString (titlesRes, alvu_DisablePtTitle);
	    break;

	case ENABLE_GROUP_LIST:
	    title = vresourceGetString (titlesRes, alvu_EnableGrpTitle);
	    break;

	case DISABLE_GROUP_LIST:
	    title = vresourceGetString (titlesRes, alvu_DisableGrpTitle);
	    break;

	case BASE_FILTER_LIST:
	    title = vresourceGetString (titlesRes, alvu_SelectBaseFilterTitle);
	    break;

	case EDIT_FILTER_LIST:
	    title = vresourceGetString (titlesRes, alvu_SelectEditFilterTitle);
	    break;

	case DELETE_FILTER_LIST:
	    title = vresourceGetString (titlesRes, alvu_SelectDeleteFilterTitle);
	    break;

	case STARTUP_FILTER_LIST:
	    title = vresourceGetString (titlesRes, alvu_SelectStartupFilterTitle);
	    break;

	case BROWSE_REPORTS_LIST:
	    title = vresourceGetString (titlesRes, alvu_BrowseReportsTitle);
	    break;

    }
    if (title != NULL)
    {
	SetTitle (title);
    }
}


/*******************************************************************
*
*    CombBox Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ComboBox, vcombobox, "ComboBox");

void ComboBox::ObserveComboBox (vevent *event, int message)
{
    if (message != vcomboboxNOTIFY_SELECT && message != vcomboboxNOTIFY_ACCEPT)
    {
	return;
    }
    vchar *selected = comboDialog -> GetSelected ();
    vcomboboxValue *value = GetValue ();
    vcharCopyBounded (value -> GetTitle (), selected, COMBO_SELECTED_SIZE);
    selected [COMBO_SELECTED_SIZE] = 0;
}


/*******************************************************************
*
*    ComboBox build list function
*
*******************************************************************/

void ComboBox::BuildComboBoxList (char **selections)
{
    vcomboboxValue	*value;
    int			i = 0;

    while (selections [i])
    {
	value = new vcomboboxValue;
	value -> SetTitleScribed (vcharScribeSystem (selections[i]));
	AppendValue (value);
	i++;
    }
    if (GetValueCount () > 0)
    {
	SortValues ();
    }
}


/*******************************************************************
*
*    ComboBox select designated item
*
*******************************************************************/

void ComboBox::SelectItem (char *selectedItem)
{
    vstr *selStr = vstrCloneScribed (vcharScribeSystem (selectedItem));
    int numValues = GetValueCount ();
    int i;

    for (i = 0; i < numValues; i++)
    {
	vchar *value = (vchar *)(GetValueAt (i));
	if (vcharCompare (selStr, value) == 0)
	{
	    SelectValueAt (i);
	    break;
	}
    }
    vstrDestroy (selStr);
}


/*******************************************************************
*
*    ComboOkButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ComboOkButton, vbutton, "ComboOkButton");

void ComboOkButton::ObserveDialogItem (vevent *event)
{
    vdialogItem *entryBox = comboDialog -> comboBox -> GetEntrybox ();
    vcomboboxValue *value = comboDialog -> comboBox -> GetEntryboxValue (entryBox);
    vchar *selected = comboDialog -> GetSelected ();
    vcharCopyBounded (value -> GetTitle (), selected, COMBO_SELECTED_SIZE);
    selected [COMBO_SELECTED_SIZE] = 0;
    comboDialog -> Close ();
    vevent::StopProcessing ();
}


/*******************************************************************
*
*    ComboCancelButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ComboCancelButton, vbutton, "ComboCancelButton");

void ComboCancelButton::ObserveDialogItem (vevent *event)
{
    vchar *selected = comboDialog -> GetSelected ();
    vmemSet ((void *)selected, 0, COMBO_SELECTED_SIZE);
    comboDialog -> Close ();
    vevent::StopProcessing ();
}

