#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vbuttonHEADER
#include vcomboboxHEADER
#include vprintHEADER

extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
// this bit of sillyness is needed because windows #defines AppendMenu
// as #define AppendMenu AppendMenuA, which interfers with galaxy class
// vmenubar::AppendMenu
#undef AppendMenu
#endif WIN32
#include "shareMem.h"
#include "tsDefs.h"
#include "alarmDefs.h"
#include "alarmfil.h"
#include "alarmvu.h"
extern TreeNode	*treeRoot;	// see alarmvu.cxx
}

#include "alvu.h"
#include "alarmvu.hxx"
#include "filterDialog.hxx"
#include "comboDialog.hxx"
#include "alvuUtils.hxx"

// static filteredTextitem filter functions

static int groupFilterFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len);
static int nameFilterFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len);
static int operatorFilterFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len);
extern char **groupNames;	// see alarmvu.cxx


/*******************************************************************
*
*    FilterDialog Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(FilterDialog, vdialog, "FilterDialog");

/*******************************************************************
*
*    FilterDialog constructor
*
*******************************************************************/

void FilterDialog::LoadInit (vresource res)
{
    title = NULL;

    newFilter = vFALSE;

    filterGroup = filteredTextitem::RequestSubLoad (this,
						alvu_FilterGroup);
    browseButton = FilterBrowseButton::RequestSubLoad (this,
						alvu_BrowseButton);
    groupContainer = vcontainerExclusiveGroup::RequestSubLoad (this,
						alvu_GroupContainer);
    showAll = vcontrolToggle::RequestSubLoad (groupContainer,
						alvu_ShowAll);
    showAlarm = vcontrolToggle::RequestSubLoad (groupContainer,
						alvu_ShowAlarm);
    ackContainer = vcontainerGroup::RequestSubLoad (this,
						alvu_AckContainer);
    hideAck = vcontrolToggle::RequestSubLoad (ackContainer,
						alvu_HideAck);
    hideUnAck = vcontrolToggle::RequestSubLoad (ackContainer,
						alvu_HideUnAck);
    severityContainer = vcontainerGroup::RequestSubLoad (this,
						alvu_SeverityContainer);
    hideMajor = vcontrolToggle::RequestSubLoad (severityContainer,
						alvu_HideMajor);
    hideMinor = vcontrolToggle::RequestSubLoad (severityContainer,
						alvu_HideMinor);
    hideInfo = vcontrolToggle::RequestSubLoad (severityContainer,
						alvu_HideInfo);
    hideEvent = vcontrolToggle::RequestSubLoad (severityContainer,
						alvu_HideEvent);
    alarmsAfter = filteredTextitem::RequestSubLoad (this,
						alvu_AlarmsAfter);
    alarmsBefore = filteredTextitem::RequestSubLoad (this,
						alvu_AlarmsBefore);
    operatorName = filteredTextitem::RequestSubLoad (this,
						alvu_OperatorName);
    acceptButton = FilterAcceptButton::RequestSubLoad (this,
						alvu_AcceptButton);
    cancelButton = FilterCancelButton::RequestSubLoad (this,
						alvu_CancelButton);

    vdialog::LoadInit (res);

    // store ptr to dialog in buttons
    acceptButton -> StoreFilterDialog (this);
    browseButton -> StoreFilterDialog (this);
    cancelButton -> StoreFilterDialog (this);

    // set filter functions
    filterGroup -> SetItemFilterFunc (groupFilterFunc);
    alarmsAfter -> SetItemFilterFunc (datetimeFilter);
    alarmsBefore -> SetItemFilterFunc (datetimeFilter);
    operatorName -> SetItemFilterFunc (operatorFilterFunc);
}


/*******************************************************************
*
*    FilterDialog window observer
*
*******************************************************************/

void FilterDialog::ObserveWindow (vevent *event)
{
    if (event -> GetType () == veventWINDOW_STATE && event -> IsClose ())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

/*******************************************************************
*
*    FilterDialog destructor
*
*******************************************************************/

void FilterDialog::Destroy ()
{
    if (IsOpen ())
    {
	Close ();
    }
    if (title != NULL)
    {
	vstrDestroy (title);
    }
    vdialog::Destroy ();
}


/*******************************************************************
*
*    FilterDialog open function
*
*******************************************************************/

void FilterDialog::OpenFilter (alvuApplication *alvuapp, 
			       char *filtername,
			       Filter *filter,     
			       FilterType typeflag)
{
    app = alvuapp;
    filterName = filtername;
    if (filter != NULL)
    {
	filterDef = *filter;
    }
    typeFlag = typeflag;

    SetDialogTitle ();

    switch (typeFlag)
    {
	case NEW_FILTER:
	    GetFilter (filterName, &filterDef);
	    break;

	case EDIT_FILTER:
	    GetFilter (filterName, &filterDef);
	    break;

	case MODIFY_FILTER:
	    break;
    }
    LoadFilter ();

    Place (vwindow::GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    vevent::Process ();
}


/*******************************************************************
*
*    FilterDialog load filter to modify
*
*******************************************************************/

void FilterDialog::LoadFilter (void)
{
    if (typeFlag == EDIT_FILTER)
    {
	filterNameFld -> SetTextScribed (vcharScribeSystem (filterDef.name));
    }
    filterGroup -> SetTextScribed (vcharScribeSystem(filterDef.group));
    if (filterDef.alarmOnly & ALLFILTER)
    {
	groupContainer -> SetSelection (showAll);
    }
    else if (filterDef.alarmOnly & ALMONLYFILTER)
    {
	groupContainer -> SetSelection (showAlarm);
    }
    if (filterDef.ack & ACKFILTER)
    {
	hideAck -> SetValue (1);
    }
    else
    {
	hideAck -> SetValue (0);
    }
    if (filterDef.ack & UNACKFILTER)
    {
	hideUnAck -> SetValue (1);
    }
    else
    {
	hideUnAck -> SetValue (0);
    }
    if (filterDef.sev & MAJORFILTER)
    {
	hideMajor -> SetValue (1);
    }
    else
    {
	hideMajor -> SetValue (0);
    }
    if (filterDef.sev & MINORFILTER)
    {
	hideMinor -> SetValue (1);
    }
    else
    {
	hideMinor -> SetValue (0);
    }
    if (filterDef.sev & INFOFILTER)
    {
	hideInfo -> SetValue (1);
    }
    else
    {
	hideInfo -> SetValue (0);
    }
    if (filterDef.sev & EVENTFILTER)
    {
	hideEvent -> SetValue (1);
    }
    else
    {
	hideEvent -> SetValue (0);
    }
    alarmsAfter -> SetTextScribed (vcharScribeSystem (filterDef.start));
    alarmsBefore -> SetTextScribed (vcharScribeSystem (filterDef.end));
    operatorName -> SetTextScribed (vcharScribeSystem (filterDef.op));
    if (typeFlag == EDIT_FILTER)
    {
	int i;
	int index = 0;
	int min = startupDisplay -> GetMin ();
	int max = startupDisplay -> GetMax ();

	for (i = min; i <= max; i++)
	{
	    if (filterDef.startupDisplay & (1 << i))
	    {
		index = i;
		break;
	    }
	}
	startupDisplay -> SetValue (index);
    }
}


/*******************************************************************
*
*    FilterDialog new filter function
*
*******************************************************************/

int FilterDialog::NewFilter (Filter *filter)
{
    if (newFilter == vTRUE)
    {
	*filter = filterDef;
    }
    return (newFilter);
}


/*******************************************************************
*
*    FilterDialog set title function
*
*******************************************************************/

void FilterDialog::SetDialogTitle (void)
{
    vresource titlesRes = app -> titlesResource;

    switch (typeFlag)
    {
	case NEW_FILTER:
	    title = vresourceGetString (titlesRes, alvu_DefineNewFilterTitle);
	    break;

	case MODIFY_FILTER:
	    title = vresourceGetString (titlesRes, alvu_ModifyFilterTitle);
	    break;

	case EDIT_FILTER:
	    title = vresourceGetString (titlesRes, alvu_EditExistFilterTitle);
	    break;

    }
    if (title != NULL)
    {
	SetTitle (title);
    }
}


/*******************************************************************
*
*    FilterNameContainer Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(FilterNameContainer, vcontainer, "FilterNameContainer");

void FilterNameContainer::LoadInit (vresource res)
{
    vapplication *app = vapplication::GetCurrent ();
    vresource contDictRes = vresourceGet (app -> GetResources (),
					  alvu_ContainerDictionary);
    vresource realRes = vresourceGet (contDictRes, alvu_FilterNameContainer);

    filterName = filteredTextitem::RequestSubLoad (this, alvu_FilterName);

    vcontainer::LoadInit (realRes);

    // set a filter on the filter name
    filterName -> SetItemFilterFunc (nameFilterFunc);
}


/*******************************************************************
*
*    StartupContainer Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(StartupContainer, vcontainer, "StartupContainer");

void StartupContainer::LoadInit (vresource res)
{
    vapplication *app = vapplication::GetCurrent ();
    vresource contDictRes = vresourceGet (app -> GetResources (),
					alvu_ContainerDictionary);
    vresource realRes = vresourceGet (contDictRes, 
					alvu_FilterStartupContainer);

    startupDisplay = vcontrolOptionMenu::RequestSubLoad (this, 
					alvu_FilterStartup);

    vcontainer::LoadInit (realRes);
}


/*******************************************************************
*
*    FilterAcceptButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(FilterAcceptButton, vbutton, "FilterAcceptButton");

void FilterAcceptButton::ObserveDialogItem (vevent *event)
{
    vstr	*value;
    char	timeStr[36];

    FilterType filterType = filterDialog -> GetFilterType ();
    Filter *filterDef = filterDialog -> GetDefFilter ();
    if ((filterType == NEW_FILTER) || (filterType == EDIT_FILTER))
    {
	value = filterDialog -> filterNameFld -> GetTextAsString ();
	vcharExportBounded (vcharScribeSystemExport (value),
			    filterDef -> name,
			    sizeof (filterDef -> name));
	filterDef -> name [sizeof (filterDef -> name) - 1] = 0;
	if (filterDef -> name[0] == 0)
	{
	    alvuMessageDialogCenter (filterDialog, ALVU_INFO_NOTICE,
				     alvu_FilterNameNeeded);
	    vstrDestroy (value);
	    return;
	}
	vstrDestroy (value);
    }
    value = filterDialog -> filterGroup -> GetTextAsString ();
    vcharExportBounded (vcharScribeSystemExport (value),
			filterDef -> group,
			sizeof (filterDef -> group));
    filterDef -> group [sizeof (filterDef -> group) - 1] = 0;
    vstrDestroy (value);

    filterDef -> alarmOnly = 0;
    if ((filterDialog -> showAll -> GetValue ()) == 1)
    {
	filterDef -> alarmOnly |= ALLFILTER;
    }
    if ((filterDialog -> showAlarm -> GetValue ()) == 1)
    {
	filterDef -> alarmOnly |= ALMONLYFILTER;
    }

    filterDef -> ack = 0;
    if ((filterDialog -> hideAck -> GetValue ()) == 1)
    {
	filterDef -> ack |= ACKFILTER;
    }
    if ((filterDialog -> hideUnAck -> GetValue ()) == 1)
    {
	filterDef -> ack |= UNACKFILTER;
    }

    filterDef -> sev = 0;
    if ((filterDialog -> hideMajor -> GetValue ()) == 1)
    {
	filterDef -> sev |= MAJORFILTER;
    }
    if ((filterDialog -> hideMinor -> GetValue ()) == 1)
    {
	filterDef -> sev |= MINORFILTER;
    }
    if ((filterDialog -> hideInfo -> GetValue ()) == 1)
    {
	filterDef -> sev |= INFOFILTER;
    }
    if ((filterDialog -> hideEvent -> GetValue ()) == 1)
    {
	filterDef -> sev |= EVENTFILTER;
    }

    value = filterDialog -> filterGroup -> GetTextAsString ();
    vcharExportBounded (vcharScribeSystemExport (value),
			filterDef -> group,
			sizeof (filterDef -> group));
    filterDef -> group [sizeof (filterDef -> group) - 1] = 0;
    vstrDestroy (value);

    value = filterDialog -> alarmsAfter -> GetTextAsString ();
    vcharExportBounded (vcharScribeSystemExport (value),
			timeStr,
			sizeof (timeStr));
    timeStr[sizeof(timeStr) - 1] = 0;
    if (strlen (timeStr) > 0)
    {
	if (ValidTimeSpec (timeStr) == ALVU_ERROR)
	{	
	    alvuMessageDialogCenter (filterDialog, ALVU_INFO_NOTICE, 
				     alvu_BadDateTimeStr, value);
            vstrDestroy (value);
	    return;
	}	
    }
    vstrDestroy (value);
    strcpy (filterDef -> start, timeStr);
    filterDef -> start[sizeof (filterDef -> start) - 1] = 0;

    value = filterDialog -> alarmsBefore -> GetTextAsString ();
    vcharExportBounded (vcharScribeSystemExport (value),
			timeStr,
			sizeof (timeStr));
    timeStr[sizeof(timeStr) - 1] = 0;
    if (strlen (timeStr) > 0)
    {
	if (ValidTimeSpec (timeStr) == ALVU_ERROR)
	{
	    alvuMessageDialogCenter (filterDialog, ALVU_INFO_NOTICE, 
				     alvu_BadDateTimeStr, value);
            vstrDestroy (value);
	    return;
	}
    }
    vstrDestroy (value);
    strcpy (filterDef -> end, timeStr);
    filterDef -> end [sizeof (filterDef -> end) - 1] = 0;

    value = filterDialog -> operatorName -> GetTextAsString ();
    vcharExportBounded (vcharScribeSystemExport (value),
			filterDef -> op,
			sizeof (filterDef -> op));
    filterDef -> op [sizeof (filterDef -> op) - 1] = 0;
    vstrDestroy (value);

    if ((filterType == NEW_FILTER) || (filterType == EDIT_FILTER))
    {
	filterDef -> startupDisplay = 0;
	int val = filterDialog -> startupDisplay -> GetValue ();
	if (val > 0)
	{
	    filterDef -> startupDisplay = 1 << val;
	}
    }
    filterDef -> modified = True;
    filterDialog -> SetNewFilter (vTRUE);
    filterDialog -> Close ();
    vevent::StopProcessing ();
}


/*******************************************************************
*
*    FilterBrowseButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(FilterBrowseButton, vbutton, "FilterBrowseButton");

void FilterBrowseButton::ObserveDialogItem (vevent *event)
{
    alvuApplication *app = filterDialog -> GetApp ();
    vresource alvuRes = app -> alvuResource;
    vresource comboRes = vresourceGet (alvuRes, alvu_ComboDialog);
    ComboDialog *comboDialog = new ComboDialog (comboRes);
    comboDialog -> OpenCombo (app, BROWSE_GROUP_LIST, groupNames, NULL);
    vchar selectedGroup[AL_GROUPNAME_SIZE + 1];
    vcharCopyBounded (comboDialog -> GetSelected (), selectedGroup, AL_GROUPNAME_SIZE);
    selectedGroup [AL_GROUPNAME_SIZE] = 0;
    delete comboDialog;
    char groupname [AL_GROUPNAME_SIZE + 1];
    if (selectedGroup[0])
    {
	vcharExportBounded (vcharScribeSystemExport (selectedGroup),
			    groupname,
			    AL_GROUPNAME_SIZE);
	groupname [AL_GROUPNAME_SIZE] = 0;
	if (FindGroup (treeRoot, groupname))
	{
	    filterDialog -> filterGroup -> SetText (selectedGroup);
	}
	else
	{
	    alvuMessageDialogCenter (filterDialog, ALVU_INFO_NOTICE,
				     alvu_GroupNotExistMsg, selectedGroup);
	}
    }
    return;
}


/*******************************************************************
*
*    FilterCancelButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(FilterCancelButton, vbutton, "FilterCancelButton");

void FilterCancelButton::ObserveDialogItem (vevent *event)
{
    filterDialog -> SetNewFilter (vFALSE);
    filterDialog -> Close ();
    vevent::StopProcessing ();
}


/*******************************************************************
*
*    Filter Edit Dialog Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(FilterEditDialog, FilterDialog, "FilterEditDialog");

void FilterEditDialog::LoadInit (vresource res)
{
    filterNameContainer = FilterNameContainer::RequestSubLoad (this,
					    alvu_NameContainer);
    startupContainer = StartupContainer::RequestSubLoad (this,
					    alvu_StartupContainer);
    FilterDialog::LoadInit (res);
}


/*******************************************************************
*
*    Filter Edit Dialog Open
*******************************************************************/

void FilterEditDialog::OpenFilter (alvuApplication *alvuApp, 
				   char *filtername,
				   Filter *filter, 
				   FilterType typeflag)
{
    filterNameFld = filterNameContainer -> filterName;
    startupDisplay = startupContainer -> startupDisplay;
    FilterDialog::OpenFilter (alvuApp, filtername, filter, typeflag);
}


/*******************************************************************
*
*    some filters
*
*******************************************************************/

static int groupFilterFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, AL_GROUPNAME_SIZE));
}
static int nameFilterFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, AL_FILTERNAME_SIZE));
}
static int operatorFilterFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, AL_OPERATOR_SIZE));
}

