#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vbuttonHEADER
#include vtextHEADER
#include vtextviewHEADER
#include vprintHEADER

extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
// this bit of sillyness is needed because windows #defines AppendMenu
// as #define AppendMenu AppendMenuA, which interfers with galaxy class
// vmenubar::AppendMenu
#undef AppendMenu
#endif WIN32
#include "shareMem.h"
#include "tsDefs.h"
#include "alarmDefs.h"
#include "alarmfil.h"
#include "alarmvu.h"
}

#include "alvu.h"
#include "alarmvu.hxx"
#include "oneLineDialog.hxx"
#include "alvuUtils.hxx"


/*******************************************************************
*
*    OneLineDialog Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(OneLineDialog, vdialog, "OneLineDialog");

/*******************************************************************
*
*    OneLineDialog constructor
*
*******************************************************************/

void OneLineDialog::LoadInit (vresource res)
{
    title         = NULL;
    textEntry     = vtextitem::RequestSubLoad (this, alvu_TextItem);
    okButton      = OneLineOkButton::RequestSubLoad (this, alvu_OkButton);
    cancelButton  = OneLineCancelButton::RequestSubLoad (this, alvu_CancelButton);

    vdialog::LoadInit (res);

    // store ptr to dialog in close button
    okButton  -> StoreOneLineDialog (this);
    cancelButton   -> StoreOneLineDialog (this);
}


/*******************************************************************
*
*    OneLineDialog window observer
*
*******************************************************************/

void OneLineDialog::ObserveWindow (vevent *event)
{
    if (event -> GetType () == veventWINDOW_STATE && event -> IsClose ())
    {
	// zap so caller has nothing
	vmemSet ((void *)oneLineText, 0, sizeof (oneLineText));
	Close ();
	vevent::StopProcessing ();
    }
}

/*******************************************************************
*
*    OneLineDialog destructor
*
*******************************************************************/

void OneLineDialog::Destroy ()
{
    if (IsOpen ())
    {
	Close ();
    }
    if (title != NULL)
    {
	vstrDestroy (title);
    }
    vdialog::Destroy ();
}


/*******************************************************************
*
*    OneLineDialog open function
*
*******************************************************************/

void OneLineDialog::OpenOneLine (alvuApplication *alvuapp, int lineType)
{
    app = alvuapp;
    oneLineType = lineType;
    vmemSet ((void *)oneLineText, 0, sizeof (oneLineText));
    SetDialogTitle ();


    Place (vwindow::GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    vevent::Process ();
}



/*******************************************************************
*
*    OneLineDialog set title function
*
*******************************************************************/

void OneLineDialog::SetDialogTitle (void)
{
    vresource titlesRes = app -> titlesResource;

    switch (oneLineType)
    {
	case ONELINE_PRINT_DEVICE:
	    title = vresourceGetString (titlesRes, alvu_PrintNameTitle);
	    break;
	
	case ONELINE_FILENAME:
	    title = vresourceGetString (titlesRes, alvu_FileNameTitle);
	    break;
    }

    SetTitle (title);
}


/*******************************************************************
*
*    OneLineOkButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(OneLineOkButton, vbutton, "OneLineOkButton");

void OneLineOkButton::ObserveDialogItem (vevent *event)
{
    vchar *oneLineText = oneLineDialog -> GetOneLineText ();
    oneLineDialog -> textEntry -> GetText (oneLineText, ONELINE_TEXTSIZE);
    if (vcharLength (oneLineText) == 0)
    {
	alvuMessageDialogCenter (oneLineDialog, ALVU_INFO_NOTICE,
				     alvu_NothingEntered);
	return;
    }
    oneLineDialog -> Close ();
    vevent::StopProcessing ();
}


/*******************************************************************
*
*    OneLineCancelButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(OneLineCancelButton, vbutton, "OneLineCancelButton");

void OneLineCancelButton::ObserveDialogItem (vevent *event)
{
    vchar *oneLineText = oneLineDialog -> GetOneLineText ();
    vmemSet ((void *)oneLineText, 0, ONELINE_TEXTSIZE);
    oneLineDialog -> Close ();
    vevent::StopProcessing ();
}

