#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vbuttonHEADER
#include vtextitemHEADER
#include vcontrolHEADER
#include vcomboboxHEADER
#include vprintHEADER

extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
// this bit of sillyness is needed because windows #defines AppendMenu
// as #define AppendMenu AppendMenuA, which interfers with galaxy class
// vmenubar::AppendMenu
#undef AppendMenu
#endif WIN32
#include "shareMem.h"
#include "tsDefs.h"
#include "alarmDefs.h"
#include "alarmfil.h"
}

#include "alvu.h"
#include "alarmvu.hxx"
#include "reportDialog.hxx"
#include "comboDialog.hxx"
#include "alvuUtils.hxx"

// static filteredTextitem filter function

static int reportNameReportFunc(filteredText *text, vtextSelection *sel, 
				const vchar **s, long *len);

/*******************************************************************
*
*    ReportDialog Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ReportDialog, vdialog, "ReportDialog");

/*******************************************************************
*
*    ReportDialog constructor
*
*******************************************************************/

void ReportDialog::LoadInit (vresource res)
{
    reportName = filteredTextitem::RequestSubLoad (this, alvu_ReportName);
    openButton = ReportOpenButton::RequestSubLoad (this, alvu_OpenButton);
    browseButton = ReportBrowseButton::RequestSubLoad (this, alvu_BrowseButton);

    vcontainer *container;
    container = vcontainer::RequestSubLoad (this, alvu_OffscanContainer);
    offScan = vcontrolToggle::RequestSubLoad (container, alvu_OffScan);

    container = vcontainer::RequestSubLoad (this, alvu_DisabledContainer);
    disabled = vcontrolToggle::RequestSubLoad (container, alvu_Disabled);

    container = vcontainer::RequestSubLoad (this, alvu_InitFailContainer);
    initFail = vcontrolToggle::RequestSubLoad (container, alvu_InitFail);

    container = vcontainer::RequestSubLoad (this, alvu_IOFailContainer);
    ioFail = vcontrolToggle::RequestSubLoad (container, alvu_IOFail);

    container = vcontainer::RequestSubLoad (this, alvu_BadSensorContainer);
    badSensor = vcontrolToggle::RequestSubLoad (container, alvu_BadSensor);

    container = vcontainer::RequestSubLoad (this, alvu_SystemTagContainer);
    systemTag = vcontrolToggle::RequestSubLoad (container, alvu_SystemTag);

    container = vcontainer::RequestSubLoad (this, alvu_InfoTagContainer);
    infoTag = vcontrolToggle::RequestSubLoad (container, alvu_InfoTag);

    container = vcontainer::RequestSubLoad (this, alvu_ManualContainer);
    manual = vcontrolToggle::RequestSubLoad (container, alvu_Manual);

    container = vcontainer::RequestSubLoad (this, alvu_PumpFaultContainer);
    pumpFault = vcontrolToggle::RequestSubLoad (container, alvu_PumpFault);

    container = vcontainer::RequestSubLoad (this, alvu_ValveFaultContainer);
    valveFault = vcontrolToggle::RequestSubLoad (container, alvu_ValveFault);

    saveButton = ReportSaveButton::RequestSubLoad (this, alvu_SaveButton);
    okButton = ReportOKButton::RequestSubLoad (this, alvu_OKButton);
    cancelButton = ReportCancelButton::RequestSubLoad (this, alvu_CancelButton);

    vdialog::LoadInit (res);

    // store ptr to dialog in buttons
    openButton   -> StoreReportDialog (this);
    browseButton -> StoreReportDialog (this);
    saveButton   -> StoreReportDialog (this);
    okButton     -> StoreReportDialog (this);
    cancelButton -> StoreReportDialog (this);

    // set filter functions
    reportName -> SetItemFilterFunc (reportNameReportFunc);
}


/*******************************************************************
*
*    ReportDialog window observer
*
*******************************************************************/

void ReportDialog::ObserveWindow (vevent *event)
{
    if (event -> GetType () == veventWINDOW_STATE && event -> IsClose ())
    {
	result = vFALSE;
	Close ();
	vevent::StopProcessing ();
    }
}

/*******************************************************************
*
*    ReportDialog destructor
*
*******************************************************************/

void ReportDialog::Destroy ()
{
    if (IsOpen ())
    {
	Close ();
    }
    vdialog::Destroy ();
}


/*******************************************************************
*
*    ReportDialog open function
*
*******************************************************************/

void ReportDialog::OpenReport (alvuApplication *alvuApp, RepCriteria *repCriteria)
{
    app = alvuApp;
    criteria = *repCriteria;
    reportName -> SetTextScribed (vcharScribeSystem (criteria.name));
    UpdateReportSelections ();

    Place (vwindow::GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    vevent::Process ();
}


/*******************************************************************
*
*    ReportDialog Update Criteria function
*	read the dialog and update the criteria variable
*
*******************************************************************/

void ReportDialog::UpdateCriteria (void)
{
    criteria.reportitems = 0;
    if (offScan -> GetValue () == 1)
    {
	criteria.reportitems |= RPTOFFSCAN;
    }
    if (disabled -> GetValue () == 1)
    {
	criteria.reportitems |= RPTDISABLE;
    }
    if (initFail -> GetValue () == 1)
    {
	criteria.reportitems |= RPTINITFAIL;
    }
    if (ioFail -> GetValue () == 1)
    {
	criteria.reportitems |= RPTIOFAIL;
    }
    if (badSensor -> GetValue () == 1)
    {
	criteria.reportitems |= RPTBADSENS;
    }
    if (systemTag -> GetValue () == 1)
    {
	criteria.reportitems |= RPTSYSTAG;
    }
    if (infoTag -> GetValue () == 1)
    {
	criteria.reportitems |= RPTINFOTAG;
    }
    if (manual -> GetValue () == 1)
    {
	criteria.reportitems |= RPTMANUAL;
    }
    if (pumpFault -> GetValue () == 1)
    {
	criteria.reportitems |= RPTPUMPFAULT;
    }
    if (valveFault -> GetValue () == 1)
    {
	criteria.reportitems |= RPTVALVFAULT;
    }
}



/*******************************************************************
*
*    ReportDialog Update Report Selections function
*	read the criteria variable, and update the dialog
*
*******************************************************************/

void ReportDialog::UpdateReportSelections (void)
{
    if (criteria.reportitems & RPTOFFSCAN)
    {
	offScan -> SetValue (1);
    }
    if (criteria.reportitems & RPTDISABLE)
    {
	disabled -> SetValue (1);
    }
    if (criteria.reportitems & RPTINITFAIL)
    {
	initFail -> SetValue (1);
    }
    if (criteria.reportitems & RPTIOFAIL)
    {
	ioFail -> SetValue (1);
    }
    if (criteria.reportitems & RPTBADSENS)
    {
	badSensor -> SetValue (1);
    }
    if (criteria.reportitems & RPTSYSTAG)
    {
	systemTag -> SetValue (1);
    }
    if (criteria.reportitems & RPTINFOTAG)
    {
	infoTag -> SetValue (1);
    }
    if (criteria.reportitems & RPTMANUAL)
    {
	manual -> SetValue (1);
    }
    if (criteria.reportitems & RPTPUMPFAULT)
    {
	pumpFault -> SetValue (1);
    }
    if (criteria.reportitems & RPTVALVFAULT)
    {
	valveFault -> SetValue (1);
    }
}


/*******************************************************************
*
*    ReportBrowseButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ReportBrowseButton, vbutton, "ReportBrowseButton");

void ReportBrowseButton::ObserveDialogItem (vevent *event)
{
    alvuApplication *app = reportDialog -> GetApp ();
    vresource alvuRes = app -> alvuResource;
    vresource comboRes = vresourceGet (alvuRes, alvu_ComboDialog);
    ComboDialog *comboDialog = new ComboDialog (comboRes);

    // GetReportNames found in alarmfil.c returns char **
    comboDialog -> OpenCombo (app, BROWSE_REPORTS_LIST, GetReportNames (), NULL);
    vchar selectedReport[REPORTNAME_SIZE + 1];
    vcharCopyBounded (comboDialog -> GetSelected (), selectedReport, REPORTNAME_SIZE);
    selectedReport [REPORTNAME_SIZE] = 0;
    delete comboDialog;
    if (selectedReport[0])
    {
	char reportname [REPORTNAME_SIZE + 1];
	vcharExportBounded (vcharScribeSystemExport (selectedReport),
			    reportname,
			    REPORTNAME_SIZE);
	reportname [REPORTNAME_SIZE] = 0;
	RepCriteria *criteria = reportDialog -> GetCriteria ();
	GetReport (reportname, criteria);
	if (strlen (reportname))
	{
	    reportDialog -> reportName -> SetText (selectedReport);
	    reportDialog -> UpdateReportSelections ();
	}
	else
	{
	    alvuMessageDialogCenter (reportDialog, ALVU_INFO_NOTICE,
				     alvu_OpenReportError, selectedReport);
	}
    }
    return;
}


/*******************************************************************
*
*    ReportCancelButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ReportCancelButton, vbutton, "ReportCancelButton");

void ReportCancelButton::ObserveDialogItem (vevent *event)
{
    reportDialog -> SetResult (vFALSE);
    reportDialog -> Close ();
    vevent::StopProcessing ();
}


/*******************************************************************
*
*    ReportOKButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ReportOKButton, vbutton, "ReportOKButton");

void ReportOKButton::ObserveDialogItem (vevent *event)
{
    vstr *repName = reportDialog -> reportName -> GetTextAsString ();
    RepCriteria *criteria = reportDialog -> GetCriteria ();
    char reportname [REPORTNAME_SIZE + 1];
    if (vcharLength (repName) == 0)
    {
	reportname[0] = 0;
    }
    else
    {
	vcharExportBounded (vcharScribeSystemExport (repName),
			    reportname,
			    REPORTNAME_SIZE);
	reportname [REPORTNAME_SIZE] = 0;
    }
    vstrDestroy (repName);
    strncpy (criteria -> name, reportname, REPORTNAME_SIZE);
    criteria -> name [REPORTNAME_SIZE - 1] = 0;

    // update from dialog 
    reportDialog -> UpdateCriteria ();

    reportDialog -> SetResult (vTRUE);
    reportDialog -> Close ();
    vevent::StopProcessing ();
}


/*******************************************************************
*
*    ReportOpenButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ReportOpenButton, vbutton, "ReportOpenButton");

void ReportOpenButton::ObserveDialogItem (vevent *event)
{
    vstr *repName = reportDialog -> reportName -> GetTextAsString ();
    RepCriteria *criteria = reportDialog -> GetCriteria ();
    char reportname [REPORTNAME_SIZE + 1];
    vcharExportBounded (vcharScribeSystemExport (repName),
			    reportname,
			    REPORTNAME_SIZE);
    reportname [REPORTNAME_SIZE] = 0;
    vstrDestroy (repName);
    strncpy (criteria -> name, reportname, REPORTNAME_SIZE);
    criteria -> name [REPORTNAME_SIZE - 1] = 0;
    // update from dialog 
    reportDialog -> UpdateCriteria ();
    GetReport (reportname, criteria);
    reportDialog -> reportName -> SetTextScribed (vcharScribeSystem (criteria -> name));
    // update dialog with current report criteria
    reportDialog -> UpdateReportSelections ();
}


/*******************************************************************
*
*    ReportSaveButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(ReportSaveButton, vbutton, "ReportSaveButton");

void ReportSaveButton::ObserveDialogItem (vevent *event)
{
    vstr *repName = reportDialog -> reportName -> GetTextAsString ();
    RepCriteria *criteria = reportDialog -> GetCriteria ();
    char reportname [REPORTNAME_SIZE + 1];
    vcharExportBounded (vcharScribeSystemExport (repName),
			    reportname,
			    REPORTNAME_SIZE);
    reportname [REPORTNAME_SIZE] = 0;
    vstrDestroy (repName);
    strncpy (criteria -> name, reportname, REPORTNAME_SIZE);
    criteria -> name [REPORTNAME_SIZE - 1] = 0;

    // update from dialog 
    reportDialog -> UpdateCriteria ();
    SaveReport (criteria -> name, criteria);
    reportDialog -> reportName -> SetTextScribed (vcharScribeSystem (criteria -> name));
    // update dialog with current report criteria
    reportDialog -> UpdateReportSelections ();
}


/*******************************************************************
*
*    some filters
*
*******************************************************************/

static int reportNameReportFunc(filteredText *text, vtextSelection *sel, 
					       const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, REPORTNAME_SIZE - 1));
}

