#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vbuttonHEADER
#include vtextHEADER
#include vtextviewHEADER
#include vprintHEADER

extern "C"
{
#ifdef WIN32
#include <stdio.h>
#include <winsock.h>
// this bit of sillyness is needed because windows #defines AppendMenu
// as #define AppendMenu AppendMenuA, which interfers with galaxy class
// vmenubar::AppendMenu
#undef AppendMenu
#endif WIN32
#include "shareMem.h"
#include "tsDefs.h"
#include "alarmDefs.h"
#include "alarmfil.h"
#include "alarmvu.h"
}

#include "alvu.h"
#include "alarmvu.hxx"
#include "textDialog.hxx"


/*******************************************************************
*
*    TextDialog Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(TextDialog, vdialog, "TextDialog");

/*******************************************************************
*
*    TextDialog constructor
*
*******************************************************************/

void TextDialog::LoadInit (vresource res)
{
    title       = NULL;
    textView    = vtextitem::RequestSubLoad (this, alvu_TextView);
    closeButton = TextCloseButton::RequestSubLoad (this, alvu_CloseButton);

    vdialog::LoadInit (res);

    // store ptr to dialog in close button
    closeButton -> StoreTextDialog (this);
}


/*******************************************************************
*
*    TextDialog window observer
*
*******************************************************************/

void TextDialog::ObserveWindow (vevent *event)
{
    if (event -> GetType () == veventWINDOW_STATE && event -> IsClose ())
    {
	Close ();
    }
}

/*******************************************************************
*
*    TextDialog destructor
*
*******************************************************************/

void TextDialog::Destroy ()
{
    if (IsOpen ())
    {
	Close ();
    }
    if (title != NULL)
    {
	vstrDestroy (title);
    }
    vdialog::Destroy ();
}


/*******************************************************************
*
*    TextDialog open function
*
*******************************************************************/

void TextDialog::OpenText (alvuApplication *alvuapp, int type, 
			   char *txt, char *misc)
{
    app = alvuapp;
    textType = type;
    miscStuff = misc;

    SetDialogTitle ();

    BuildText (txt);

    vdialog::Open();
}


/*******************************************************************
*
*    TextDialog build list function
*
*******************************************************************/

void TextDialog::BuildText (char *txt)
{
    switch (textType)
    {
	case EXPAND_TEXT:
	case GUIDANCE_TEXT:
	    textView -> SetTextScribed (vcharScribeSystem (txt));
	    break;
    }
}


/*******************************************************************
*
*    TextDialog set title function
*
*******************************************************************/

void TextDialog::SetDialogTitle (void)
{
    vresource titlesRes = app -> titlesResource;
    vstr	*formattedMsg;
    vchar	pvname [AL_POINTNAME_SIZE + 1];

    switch (textType)
    {
	case EXPAND_TEXT:
	    title = vresourceGetString (titlesRes, alvu_ExpandViewTitle);
	    break;
	
	case GUIDANCE_TEXT:
	    formattedMsg = vresourceGetString (titlesRes, alvu_GuidanceTitle);
	    vcharCopyScribedBounded (vcharScribeSystem (miscStuff),
				 pvname, AL_POINTNAME_SIZE);
	    pvname [AL_POINTNAME_SIZE] = 0;
	    title = vstrCloneScribed (vcharScribeFormat (formattedMsg, pvname));
	    vstrDestroy (formattedMsg);
	    break;
    }
    if (title != NULL)
    {
	SetTitle (title);
    }
}


/*******************************************************************
*
*    TextOkButton Class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(TextCloseButton, vbutton, "TextCloseButton");

void TextCloseButton::ObserveDialogItem (vevent *event)
{
    textDialog -> Close ();
}

