
/* Fix the dbase file header and remove funny control-z's */

#include <stdio.h>
#include <sys/stat.h>
#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32
#include <sys/time.h>

static	void	dumpIndex (),printRec(),reNdx(),showRecDate(),touchDate();
static time_t	synchTime();

/* dBase file header record */

typedef struct {
    char	Id;		/* Identification */
    char	Year;		/* Last update year */
    char	Month;		/* Last update month */
    char	Day;		/* Last update day */
    unsigned int	NumRecs;	/* Number of records */
    unsigned short HeaderLength;/* Length (in bytes) of header info */
    unsigned short RecordLen;	/* Length (in bytes) of each record */
    char	res [16];
    short	Mdx;		/* A .MDX file exists */
    char	Reserved2 [2];	/* Reserved area */
} HEADER;
   
#ifdef BYTEORDER_0123
#define	swapShort(x)	(x)
#define	swapLong(x)	(x)
#else
#define	swapShort(x)	(((x) >> 8) | ((x) << 8))
#define	swapLong(x)	( (((x) & 0xff000000) >> 24) | (((x) & 0xff0000) >> 8) | \
			(((x) & 0xff00) << 8) | (((x) & 0xff) << 24))
#endif

/*----------*/
main (argc, argv)

    int		argc;
    char	*argv [];

    {

	extern	char	*optarg;
	extern	int	optind; 
		int	noWrite = 0, dumpIx = 0, recPrint = 0,reIndex = 0,showDate = 0,touchPvmap = 0;
		FILE	*fil;
		HEADER	header;
		struct stat statBuf;
		int	numRecs,rest,offset;
		char	c,*fptr, tagName [20];
		short	recLen, headLen;
		unsigned long	nRecs;

	/* Process command line options */

	while ((c = getopt (argc, argv, "ni:rxd:t")) != -1)
	    switch (c)
	      {
	      case 'n':	/* Do not write fixed file */
		noWrite = 1;
		break;
	      case 'r':	/* print records to stdout */
		recPrint = 1;
		break;
	      case 'i':	/* Dump the index */
		strcpy (tagName, optarg);
		dumpIx = 1;		   
		break;
	      case 'x':
		reIndex = 1;
		break;
	      case 'd':
		showDate = 1;
		break;
	      case 't':
		touchPvmap = 1;
		break;
	      case '?':
		fprintf(stderr,"Usage is:\n");
		fprintf(stderr,"dbFix [-x -n -r [-i tagname] [-d name]] filename.dbf\n");
		fprintf(stderr,"\t-x -- reindex named file\n");
		fprintf(stderr,"\t-n -- look but do not fix\n");
		fprintf(stderr,"\t-i -- dump contents of index(MDX) file\n");
		fprintf(stderr,"\t-r -- dump contents of file 1 record/line\n");
		fprintf(stderr,"\t-d -- display date/time for named record\n");
		fprintf(stderr,"\t-t -- touch the pvmap.dbf file to force current date/time\n");
		fprintf(stderr,"\tNo switch means report AND fix record count & non-printable chars\n");
		exit(1);
	      }

	/* If we are just dumping the index, do it. */

	if (dumpIx)
	  {
	    dumpIndex (argv [optind], tagName);
	    exit (0);
	  }
	if(reIndex)
	  {
	    reNdx(argv[optind]);
	    exit(0);
	  }
	if(showDate)
	  {
	    showRecDate(argv[optind-1]);
	    exit(0);
	  }
	if(touchPvmap)
	  {
	    touchDate();
	    exit(0);
	  }

	/* Get the actual size of the file */

	stat (argv [optind], &statBuf);

	/* Open the file for read/write */

	fil = fopen (argv [optind], "r+");
	if (fil == NULL)
	    exit (-1);

	/* Read in the file header */

	if (fread ((char *)&header, sizeof (header), 1, fil) != 1)
	    exit (-1);

	/* Do byte swapping */

	headLen = swapShort (header.HeaderLength);
	recLen = swapShort (header.RecordLen);
	nRecs = swapLong(header.NumRecs);

	/* Calculate actual number of records */

	numRecs = (statBuf.st_size - headLen) / recLen;

	/* If we are just printing each record, do it. */

	if (recPrint) {
	    printRec (fil, recLen, numRecs,headLen);
	    exit (0);
	}
	/* Print out some interesting things */

	printf ("File ID: %2x\n", (int)header.Id);
	printf ("File Date: %d/%d/%d\n", (int)header.Month, (int)header.Day,
			(int)header.Year);
	printf ("# Records (from header): %d\n", nRecs);
	printf ("# Records (calculated): %d\n", numRecs);
	printf ("Header Length: %d\n", (int)headLen);
	printf ("Record Length: %d\n", (int)recLen);

	/* Update the record count in the header */

	header.NumRecs = swapLong (numRecs);
/*
	get some space and read file in.
*/
	rest = statBuf.st_size - headLen;	/* amount of rest of record */
	printf("Checking for non-printable characters in %s\n",argv[optind]);
	if((fptr = (char *)malloc(rest)) == 0)
		exit(-1);
	fseek(fil,headLen,0);				/* skip past field info...start from begining of file */
	if (fread ((char *)fptr, rest, 1, fil) != 1)
	    exit (-1);
	for(offset = 0;offset < rest;offset++)
		{
		if(!isprint(fptr[offset]))
			{
			if(offset != (rest - 1))
				{
				printf("At %08o data was %02x\n",offset+headLen,fptr[offset]);
				fptr[offset] = ' ';				/* convert to space */
				}
			}
		}
	
	if (!noWrite) {
		printf("Applying fixes... ");
	    rewind (fil);
	    fwrite ((char *)&header, sizeof (header), 1, fil);
		fseek(fil,headLen,0);				/* skip past field info */
	    fwrite ((char *)fptr, rest, 1, fil);
		printf("Done.\n");
	}

	fclose (fil);

    }

/*----------*/
static void dumpIndex (name, tagName)

    char	*name, *tagName;

    {

	C4CODE	cb;
	D4DATA	*data;
	T4TAG	*tag;

	d4init (&cb);

	data = d4open (&cb, name);
	if (data == NULL) {
	    printf ("Unable to open %s\n", name);
	    return;
	}

	tag = d4tag (data, tagName);
	if (tag == NULL) {
	    printf ("Unable to find tag %s\n", tagName);
	    return;
	}
	d4tag_select (data, tag);

	t4dump (tag, 1, 1);

	d4close (data);
	
    }
/*------------*/
static void	printRec(f,rs,nr,hl)
FILE	*f;					/* file pointer */
int		rs;					/* record size */
int		nr;					/* number of records in file */
int		hl;					/* length (in bytes) of header */
{
char	rec[4096];
fseek(f,hl,0);				/* skip past field info header...start from begining of file */
while(nr--)
	{
	if (fread (rec, rs, 1, f) != 1)
		exit(1);
	rec[rs] = 0;			/* null terminate */
	printf("%s\n",rec);	/* print to stdout...can be redirected */
	}
}
static void	reNdx(name)
char	*name;
{
C4CODE	cb;
D4DATA	*data;

d4init(&cb);

data = d4open(&cb, name);
if (data == NULL)
    printf ("Unable to open %s\n", name);
else
	{
	printf("Re-indexing file %s, please wait...",name);
	d4reindex(data);
	printf("Done.\n");
	}
}

static void showRecDate(char *name)
{
  C4CODE	cb;
  D4DATA	*data;
  F4FIELD *timestamp,*importdate;
  char timeStamp[11],importDate[11],TS[27],ID[27];
  time_t ts,id;

  d4init(&cb);
#ifdef WIN32
  data = d4open(&cb,"\\tis4000db\\pvmap.dbf");
#else
  data = d4open(&cb,"/tis4000db/pvmap.dbf");
#endif WIN32
  timestamp = d4field(data,"TIMESTAMP");
  importdate = d4field(data,"IMPORTDATE");

  if (data == NULL)
    printf ("Unable to open pvmap.dbf");
  else
    {
      if(d4seek(data,name) == 0)
	{
	  ts = (time_t)atoi(f4str(timestamp));
	  id = (time_t)atoi(f4str(importdate));
	  strcpy(TS,(char *)ctime(&ts));
	  strcpy(ID,(char *)ctime(&id));
	  TS[strlen(TS)-1] = 0;
	  ID[strlen(ID)-1] = 0;
	  printf("PV \"%s\" last modified locally on %s(GMT),remotely on %s(GMT)\n",name,TS,ID);
	}
      else
	printf("Can't locate record %s\n",name);
      d4close(data);
    }
}
static void touchDate()
{
  C4CODE	cb;
  D4DATA	*data;
  F4FIELD *timestamp;
  char timeStamp[11],TS[27];
  time_t ts,currentTime;

  d4init(&cb);
#ifdef WIN32
  data = d4open(&cb,"\\tis4000db\\pvmap.dbf");
#else
  data = d4open(&cb,"/tis4000db/pvmap.dbf");
#endif WIN32
  timestamp = d4field(data,"TIMESTAMP");

  currentTime = synchTime(0);
  if (data == NULL)
    printf ("Unable to open pvmap.dbf");
  else
    {
      strcpy(TS,(char *)ctime(&currentTime));
      TS[strlen(TS)-1] = 0;
      printf("Setting all records to %s(GMT)\n",TS);
      d4top(data);
      do
	{
	  sprintf(TS,"%d\0",currentTime);
	  f4assign(timestamp,TS);      	/* set time in record */
	  d4flush_all(data);
	}
      while(d4skip(data,1) == 0);
      d4unlock_all(data);
      d4close(data);
    }
}
/*
	This function provides the synchronization time for the inter-host comm functionality.
	this is in GMT. Naturally ULTRIX does NOT support the same functions for time conversion
	as the SUN workstations do hense the #ifdef's

	12-Sep-1994 GCT
*/
time_t	synchTime()
{
  time_t _tt_; 
  _tt_ = time(0);
#if !(defined(MIPSEL) || defined(SOLARIS) || defined(AXP))
  return(timelocal(gmtime(&_tt_)));
#else
  return(mktime(gmtime(&_tt_)));
#endif
}
