/*bldDrvSup.c,v 1.1.1.1 1999/08/03 14:41:55 tis4000 Exp */

/*  @(#)bldDrvSup.c	1.5	6/7/91
 *	Author: Robert Zieman (ANL)
 *	Date:	6/03/91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	mm-dd-yy	iii	Comment
 * .02	mm-dd-yy	iii	Comment
 * 	...
 */

#include <stdio.h>
#include <ctype.h>
#include <sys/file.h>		/* SUN support for disk io */
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <drvSup.h>
#include <sdrHeader.h>
#include <getSdrLine.h>

#define SAME 0
#define BYTES50		50
#define BYTES20		20
#define SCAN_DIM		1
#define SCAN_FUNCTION		2

extern char    *calloc();
int             errno;

struct inLineInfo {		/* temp storage for input fields */
    int             dim_size;
    int             function_entries;
    int             function_num;
    char            dim_keywd[BYTES20];
    char            drvetName[BYTES50];
}               inLineInfo;

char            inFile[BYTES50];
char            outFile[BYTES50];

struct sdrHeader sdrHeader;
struct drvSup  *drvSup;
/*
 * BLDDRVSUP
 *
 * INPUT:	drvSup.ascii
 * OUTPUT:	drvSup.sdr
 */

main(argc, argv)
    int             argc;
    char          **argv;
{

    FILE           *inFile_fp;
    FILE           *outFile_fp;
    int             len;
    short           i,
                    Icnt;
    long            Size;
    long            SSize;
    char           *addr1,
                   *addrS1;
    struct drvSup  *pTdrvSup;
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo = &inLineInfo;
    Size = SSize = 0;
    i = len = 0;

    /* check cmd line */
    if (!cmd_DrvSup(argc, argv))
	return -1;

    /* allocate space for the getSdrLine input structure */
    if ((plineIn = (struct lineIn *)
	 calloc(1, (unsigned) sizeof(struct lineIn))) == NULL) {
	printf("bldDrvSup: calloc error\n");
	return (-1);
    }
    plineIn->fname = inFile;

    if (chk_align() < 0)
	return (-1);

    if ((inFile_fp = fopen((char *) inFile, "r")) == NULL) {
	printf("bldDrvSup: fopen error errno=%i\nfile=%s\n", errno, inFile);
	return (-1);
    }
    /* get next Sdr line */
    if ((getSdrLine(inFile_fp, plineIn, NO_QUOTES)) < 0)
	return (-1);

    if (plineIn->eof == EOF) {
	printf("bldDrvSup: Error - Unexpected EOF\n");
	printf("line=%d	%s\n", plineIn->line_no, inFile);
	return (-1);
    }
    /* decode as dim line */
    if (inScan(SCAN_DIM, plineIn, pInfo) < 0)
	return (-1);

    /* allocate space for structures and ptr arrays as needed */
    if ((pTdrvSup = (struct drvSup *)
	 calloc(1, (unsigned) sizeof(struct drvSup))) == NULL) {
	printf("bldDrvSup: calloc error (struct drvSup)\n");
	return (-1);
    }
    Size = sizeof(struct drvSup);
    /* allocate drvetName ptr array (Note: papDrvet reserved later ) */
    if ((pTdrvSup->drvetName = (char **)
	 calloc(1, (unsigned)
		pInfo->dim_size * sizeof(long))) == NULL) {
	printf("bldDrvSup: calloc error (drvetName ptr array)\n");
	return (-1);
    }
    /* include size for papDrvet also */
    pTdrvSup->number = pInfo->dim_size;
    Size += 2 * (pInfo->dim_size * sizeof(long));
    Icnt = 0;
    FOREVER {
	if ((getSdrLine(inFile_fp, plineIn, TWO_QUOTES)) < 0)
	    return (-1);
	if (plineIn->eof == EOF) {
	    printf("bldDrvSup: Error - Unexpected EOF\n");
	    printf("line=%d	%s\n", plineIn->line_no, inFile);
	    return (-1);
	}
	if (inScan(SCAN_FUNCTION, plineIn, pInfo) < 0)
	    return (-1);
	len = strlen(pInfo->drvetName);
	if ((pTdrvSup->drvetName[pInfo->function_num] = (char *)
	     calloc(1, (unsigned) len + 1)) == NULL) {
	    printf("bldDrvSup: calloc error (Name array)\n");
	    return (-1);
	}
	strcpy(pTdrvSup->drvetName[pInfo->function_num], pInfo->drvetName);
	SSize += len + 1;
	Icnt++;
	if (Icnt >= pInfo->function_entries)
	    break;
    }
    if ((getc(inFile_fp)) != EOF) {
	printf("bldDrvSup: Error - extra character(s) in file %s\n", inFile);
	return (-1);
    }
    if (Icnt != pInfo->function_entries) {
	printf("bldDrvSup: Error - Number of functions != Dimension Entries\n");
	return (-1);
    }
    /* allocate the contiguous areas */
    SSize = (SSize + sizeof(long) - 1) / sizeof(long) * sizeof(long);
    Size += SSize;

    if ((drvSup = (struct drvSup *) calloc(1, (unsigned) Size)) == NULL) {
	printf("bldDrvSup: calloc error (struct drvSup)\n");
	return (-1);
    }
    addr1 = (char *) drvSup + sizeof(struct drvSup);
    drvSup->number = pTdrvSup->number;
    drvSup->drvetName = (char **) addr1;
    addr1 += drvSup->number * sizeof(long);
    drvSup->papDrvet = (struct drvet **) addr1;

    addr1 += drvSup->number * sizeof(long);

    addrS1 = (char *) drvSup + Size - SSize;

    for (i = 0; i < drvSup->number; i++) {
	if (pTdrvSup->drvetName[i] != NULL) {
	    drvSup->drvetName[i] = (char *) addrS1;
	    len = strlen(pTdrvSup->drvetName[i]);
	    strcpy((char *) addrS1, pTdrvSup->drvetName[i]);
	    addrS1 += len + 1;
	}
    }

    /* write output file */
    if ((outFile_fp = fopen((char *) outFile, "w")) == NULL) {
	printf("bldDrvSup: fopen errno=%i \nfile=%s\n",
	       errno, outFile);
	return (-1);
    }

    fprintf(outFile_fp, "drvSup\n");
    fprintf(outFile_fp, "%d\n", drvSup -> number);
    for (i = 0; i < drvSup -> number; i++)
	{
	if (!(drvSup -> drvetName[i]))
	    fprintf(outFile_fp, "\n");
	else
	    fprintf(outFile_fp, "%s\n", drvSup -> drvetName[i]);
	}

    if ((fclose(outFile_fp)) == EOF) {
	printf("bldDrvSup: fclose error. errno=%i\nfile=%s\n",
	       errno, outFile);
	return (-1);
    }
    return (0);
}
/*
 *
 * CHK_ALIGN
 *
 * checks alignment of input header structure
 *
 * return (0,-1) success, fail
 */
chk_align()
{
    if ((sizeof(struct drvSup) % 4)) {
	printf("bldDrvSup: struct %s size alignment error\n", "drvSup ");
	return (-1);
    }
    return (0);
}
/*
 * CMD_DRVSUP
 *
 * checks command line arg usage
 *
 * return (1,0) success, fail
 */
cmd_DrvSup(argc, argv)
    int             argc;
    char          **argv;
{
    extern char    *optarg;
    struct stat    *buf;
    int             c,
                    err;
    short           aflg,
                    oflg;
    aflg = oflg = err = 0;

    if ((buf = (struct stat *)
	 calloc(1, (unsigned) sizeof(struct stat))) == NULL) {
	printf("bldDrvSup: calloc error\n");
	return 0;
    }
    while ((c = getopt(argc, argv, "a:o:")) != -1)
	switch (c) {
	case 'a':
	    aflg++;
	    strcpy(inFile, optarg);
	    if (stat(inFile, buf) == -1) {
		printf("bldDrvSup: Can't stat file=%s errno=%i\n",
		       inFile, errno);
		err++;
	    }
	    break;
	case 'o':
	    oflg++;
	    strcpy(outFile, optarg);
	    break;
	}
    if (!aflg || !oflg || err) {
	printf("Usage: \n");
	printf(" <bldDrvSup -a drvSup.ascii -o drvSup.sdr>\n");
	return 0;
    }
    return 1;
}
/*
 * INSCAN
 *
 * return (0,-1) success, fail
 */
int
inScan(scan_type, plineIn, pInfo)
    int             scan_type;
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
{
    int             err = 0;
    switch (scan_type) {
    case SCAN_DIM:
	if ((sscanf(plineIn->line, "%3s%i%i",
		    pInfo->dim_keywd,
		    &pInfo->dim_size,
		    &pInfo->function_entries)) != 3) {
	    err = 1;
	    break;
	}
	if ((strcmp("DIM", pInfo->dim_keywd)) != SAME)
	    err = 2;
	break;
    case SCAN_FUNCTION:
	if ((sscanf(plineIn->line, "%i \"%50[^\"]\"",
		    &pInfo->function_num,
		    pInfo->drvetName)) != 2)
	    err = 1;
	break;
    default:
	printf("bldDrvSup: inScan(scan_type) not defined internally\n");
	printf("bldDrvSup: line=%i\n", plineIn->line_no);
	return (-1);
	break;
    }
    if (err == 1) {
	printf("bldDrvSup: Illegal field format file=%s\n", inFile);
	printf("bldDrvSup: line=%i\n", plineIn->line_no);
	return (-1);
    }
    if (err == 2) {
	printf("bldDrvSup: Error on DIM control line - line=%i\n", plineIn->line_no);
	printf("bldDrvSup: Illegal field format file=%s\n", inFile);
	return (-1);
    }
    return (0);
}
