/*bldRecChoice.c,v 1.1.1.1 1999/08/03 14:41:56 tis4000 Exp */

/*  @(#)bldRecChoice.c	1.8	6/7/91
 *	Author: Robert Zieman (ANL)
 *	Date:	6/03/91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	mm-dd-yy	iii	Comment
 * .02	mm-dd-yy	iii	Comment
 * 	...
 */

#include <stdio.h>
#include <ctype.h>
#include <sys/file.h>		/* SUN support for disk io */
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <choice.h>
#include <dbRecType.h>
#include <sdrHeader.h>
#include <getSdrLine.h>

#define SAME 0

#define BYTES50		50
#define BYTES20		20
#define SCAN_DIM		1
#define SCAN_RECTYPE		2
#define SCAN_INDEX		3
#define SCAN_STRING		4

extern char    *calloc();
int             errno;
char            inFile1[BYTES50];
char            inFile2[BYTES50];
char            outFile[BYTES50];

struct sdrHeader sdrHeader;

static struct inLineInfo {	/* temp storage for some sdrLine fields */
    char            dim_keywd[BYTES20];
    int             dim_size;
    int             dim_entries;
    char            RecTyp_keywd[BYTES20];
    char            RecTyp_name[BYTES50];
    int             index_size;
    int             index_entries;
    char            index_keywd[BYTES20];
    int             index_num;
    int             choice_size;
    int             choice_entries;
    int             choice_num;
    char            choice_str[BYTES50];
}               inLineInfo;


/*
 * BLDRECCHOICE
 *
 * INPUT:	choiceRec.ascii
 *       	dbRecType.sdr
 *
 * OUTPUT:	choiceRec.sdr
 */

main(argc, argv)
    int             argc;
    char          **argv;
{

    FILE           *inFile1_fp;
    FILE           *outFile_fp;
    int             found = 0;
    int             i = 0;
    int             j = 0;
    int             k = 0;
    int             Icnt;
    int             Jcnt;
    int             Kcnt;
    int             rec_no = 0;
    int             len = 0;
    long            size1 = 0;
    long            Ssize = 0;
    char           *addr1;
    char           *addrS1;
    struct choiceRec *pRec = NULL;
    struct arrChoiceSet **pArInd;
    struct choiceSet **pInd;
    char          **pStr;
    struct inLineInfo *pInfo = &inLineInfo;
    struct lineIn  *plineIn = NULL;
    /* check cmd line */
    if (!cmd_choiceRec(argc, argv))
	return -1;

    /* allocate space for the getSdrLine input structure */
    if ((plineIn = (struct lineIn *)
	 calloc(1, (unsigned) sizeof(struct lineIn))) == NULL) {
	printf("bldRecChoice: calloc error\n");
	return (-1);
    }
    plineIn->fname = inFile1;

    if (chk_align() < 0)
	return (-1);

    /* load ascii record type names */
    if (sdrLoad(inFile2))
	return (-1);

    if ((inFile1_fp = fopen((char *) inFile1, "r")) == NULL) {
	printf("bldRecChoice: fopen error errno=%i\nfile=%s\n", errno, inFile1);
	return (-1);
    }
    if ((getSdrLine(inFile1_fp, plineIn, NO_QUOTES)) < 0)
	return (-1);
    if (plineIn->eof == EOF) {
	printf("bldRecChoice: Error - Unexpected EOF");
	return (-1);
    }
    /* expect DIM sdrLine */
    if (inScan(SCAN_DIM, plineIn, pInfo) < 0)
	return (-1);

    /* allocate temp  space for structures and pointer arrays as needed */
    size1 = sizeof(struct choiceRec);
    /* allocate structure choiceRec */
    if ((pRec = (struct choiceRec *)
	 calloc(1, (unsigned) size1)) == NULL) {
	printf("bldRecChoice: calloc error\n");
	return (-1);
    }
    /* allocate papArrChoiceSet pointer array */
    if ((pRec->papArrChoiceSet = (struct arrChoiceSet **)
	 calloc(1, (unsigned) (dbRecType->number * sizeof(long)))) == NULL) {
	printf("bldRecChoice: calloc error\n");
	return (-1);
    }
    pRec->number = dbRecType->number;
    size1 += dbRecType->number * sizeof(long);
    pArInd = pRec->papArrChoiceSet;

    Icnt = 0;
    FOREVER {
	if ((getSdrLine(inFile1_fp, plineIn, TWO_QUOTES)) < 0)
	    return (-1);
	if (plineIn->eof == EOF) {
	    break;
	}
	/* expect rectype sdrLine */
	if (inScan(SCAN_RECTYPE, plineIn, pInfo) < 0)
	    return (-1);
	/* Verify record type name */
	found = 0;
	for (i = 0; i < dbRecType->number; i++) {
	    if (dbRecType->papName[i] != NULL) {
		if (strcmp(dbRecType->papName[i], pInfo->RecTyp_name) == SAME) {
		    found = 1;
		    rec_no = i;
		    break;
		}
	    }
	}
	if (!found) {
	    printf("bldRecChoice: Error - Record type '%s' not defined\n",
		   pInfo->RecTyp_name);
	    return (-1);
	}
	/* allocate structure arrChoiceSet */
	if ((pArInd[rec_no] = (struct arrChoiceSet *)
	     calloc(1, (unsigned) sizeof(struct arrChoiceSet))) == NULL) {

	    printf("bldRecChoice: calloc error\n");
	    return (-1);
	}
	size1 += sizeof(struct arrChoiceSet);
	/* allocate papChoiceSet pointer array */
	if ((pArInd[rec_no]->papChoiceSet
	     = (struct choiceSet **)
	  calloc(1, (unsigned) pInfo->index_size * sizeof(long))) == NULL) {
	    printf("bldRecChoice: calloc error\n");
	    return (-1);
	}
	pArInd[rec_no]->number
	    = pInfo->index_size;
	size1 += pInfo->index_size * sizeof(long);

	pInd = pArInd[rec_no]->papChoiceSet;
	Jcnt = 0;
	FOREVER {
	    if ((getSdrLine(inFile1_fp, plineIn, NO_QUOTES)) < 0)
		return (-1);
	    if (plineIn->eof == EOF) {
		break;
	    }
	    /* expect index sdrLine */
	    if (inScan(SCAN_INDEX, plineIn, pInfo) < 0)
		return (-1);

	    /* allocate structure choiceSet */
	    if ((pInd[pInfo->index_num]
		 = (struct choiceSet *)
		 calloc(1, (unsigned) sizeof(struct choiceSet))) == NULL) {
		printf("bldRecChoice: calloc error\n");
		return (-1);
	    }
	    size1 += sizeof(struct choiceSet);

	    /* allocate papChoice pointer array */
	    if ((pInd[pInfo->index_num]->papChoice
		 = (char **)
		 calloc(1, (unsigned) pInfo->choice_size * sizeof(long))) == NULL) {
		printf("bldRecChoice: calloc error\n");
		return (-1);
	    }
	    pInd[pInfo->index_num]->number
		= pInfo->choice_size;
	    size1 += pInfo->choice_size * sizeof(long);
	    pStr = pInd[pInfo->index_num]->papChoice;
	    Kcnt = 0;
	    FOREVER {
		if ((getSdrLine(inFile1_fp, plineIn, TWO_QUOTES)) < 0)
		    return (-1);
		if (plineIn->eof == EOF) {
		    break;
		}
		/* expect choice sdrLine */
		if (inScan(SCAN_STRING, plineIn, pInfo) < 0)
		    return (-1);
		len = strlen(pInfo->choice_str);
		/* allocate Choice string */
		if ((pStr[pInfo->choice_num] = (char *)
		     calloc(1, (unsigned) len + 1)) == NULL) {
		    printf("bldRecChoice: calloc error\n");
		    return (-1);
		}
		strcpy(pStr[pInfo->choice_num], pInfo->choice_str);
		Ssize += len + 1;
		Kcnt++;
		if (Kcnt >= pInfo->choice_entries)
		    break;
	    }
	    /* check for match choice_entries */
	    if (Kcnt != pInfo->choice_entries) {
		printf("bldRecChoice: Error - choice_entries did not match\n");
		return (-1);
	    }
	    Jcnt++;
	    if (Jcnt >= pInfo->index_entries)
		break;
	}
	if (Jcnt != pInfo->index_entries) {
	    printf("bldRecChoice: Error - index_entries did not match\n");
	    return (-1);
	}
	Icnt++;
	if (Icnt >= pInfo->dim_entries)
	    break;
    }

    /* check for match dim_entries */
    if (Icnt != pInfo->dim_entries) {
	printf("bldRecChoice: Error - dim_entries did not match\n");
	return (-1);
    }
    if ((getc(inFile1_fp)) != EOF) {
	printf("bldRecChoice: Error - extra character(s) in file %s\n", inFile1);
	return (-1);
    }
    /* allocate the contiguous area */
    Ssize = (Ssize + sizeof(long) - 1) / sizeof(long) * sizeof(long);
    size1 += Ssize;

    if ((choiceRec = (struct choiceRec *) calloc(1, (unsigned) size1)) == NULL) {
	printf("bldRecChoice: calloc error\n");
	return (-1);
    }
#ifdef DEBUG
    for (i = 0; i < dbRecType->number; i++) {
	if (pArInd[i]) {
	    printf("%d\n", pArInd[i]->number);
	    for (j = 0; j < pArInd[i]->number; j++) {

		if (pArInd[i]->papChoiceSet[j]) {

		    printf("\t%d\n", pArInd[i]->papChoiceSet[j]->number);

		    for (k = 0; k < pArInd[i]->papChoiceSet[j]->number; k++) {
			if (pArInd[i]->papChoiceSet[j]->papChoice[k]) {
			    printf("%s\n", pArInd[i]->papChoiceSet[j]->papChoice[k]);
			}
		    }
		}
	    }
	}
    }
#endif DEBUG
    /* address calculations for pointers */
    addr1 = (char *) choiceRec + sizeof(struct choiceRec);
    choiceRec->papArrChoiceSet = (struct arrChoiceSet **) addr1;
    choiceRec->number = dbRecType->number;
    addr1 += dbRecType->number * sizeof(char *);
    addrS1 = (char *) choiceRec + size1 - Ssize;
    for (i = 0; i < dbRecType->number; i++) {
	if (pRec->papArrChoiceSet[i]) {
	    choiceRec->papArrChoiceSet[i] = (struct arrChoiceSet *) addr1;
	    choiceRec->papArrChoiceSet[i]->number = pRec->papArrChoiceSet[i]->number;
	    addr1 += sizeof(struct arrChoiceSet);

	    choiceRec->papArrChoiceSet[i]->papChoiceSet = (struct choiceSet **) addr1;

	    addr1 += choiceRec->papArrChoiceSet[i]->number * sizeof(char *);

	    for (j = 0; j < choiceRec->papArrChoiceSet[i]->number; j++) {
		if (pRec->papArrChoiceSet[i]->papChoiceSet[j]) {

		    choiceRec->papArrChoiceSet[i]->papChoiceSet[j] = (struct choiceSet *) addr1;

		    addr1 += sizeof(struct choiceSet);

		    choiceRec->papArrChoiceSet[i]->papChoiceSet[j]->number = pRec->papArrChoiceSet[i]->papChoiceSet[j]->number;

		    choiceRec->papArrChoiceSet[i]->papChoiceSet[j]->papChoice = (char **) addr1;
		    addr1 += choiceRec->papArrChoiceSet[i]->papChoiceSet[j]->number * sizeof(char *);

		    for (k = 0; k < choiceRec->papArrChoiceSet[i]->papChoiceSet[j]->number; k++) {
			if (pRec->papArrChoiceSet[i]->papChoiceSet[j]->papChoice[k]) {


			    choiceRec->papArrChoiceSet[i]->papChoiceSet[j]->papChoice[k] = (char *) addrS1;


			    len = strlen(pRec->papArrChoiceSet[i]->papChoiceSet[j]->papChoice[k]);

			    strcpy((char *) addrS1, pRec->papArrChoiceSet[i]->papChoiceSet[j]->papChoice[k]);
			    addrS1 += len + 1;
			}
		    }
		}
	    }
	}
    }

    /* write output file */

    if ((outFile_fp = fopen((char *) outFile, "w")) == NULL) {
	printf("bldRecChoice: fopen errno=%i \nfile=%s\n",
	       errno, outFile);
	return (-1);
    }

    fprintf(outFile_fp, "choiceRec\n");
    fprintf(outFile_fp, "%d\n", choiceRec -> number);
    for (i = 0; i < choiceRec -> number; i++)
	{
	if (!(choiceRec -> papArrChoiceSet[i]))
	    {
	    fprintf(outFile_fp, "0\n");
	    continue;
	    }
	else
	    fprintf(outFile_fp, "%d\n", choiceRec -> papArrChoiceSet[i] -> number);
	for (j = 0; j < choiceRec -> papArrChoiceSet[i] -> number; j++)
	    {
	    if (!(choiceRec -> papArrChoiceSet[i] -> papChoiceSet[j] -> number))
		{
		fprintf(outFile_fp, "0\n");
		continue;
		}
	    else
	    	fprintf(outFile_fp, "%d\n", choiceRec -> papArrChoiceSet[i] -> papChoiceSet[j] -> number);
	    for (k = 0; k < choiceRec -> papArrChoiceSet[i] -> papChoiceSet[j] -> number; k++)
		fprintf(outFile_fp, "%s\n", choiceRec -> papArrChoiceSet[i] -> papChoiceSet[j] -> papChoice[k]);
	    }
	}
    if ((fclose(outFile_fp)) == EOF) {
	printf("bldRecChoice: fclose error. errno=%i\nfile=%s\n",
	       errno, outFile);
	return (-1);
    }
    return (0);
}				/* end of bldRecChoice.c */
/*
 *
 * CHK_ALIGN
 *
 * checks alignment of input header structures
 *
 */
chk_align()
{
    if ((sizeof(struct recType) % 4)) {
	printf("bldRecChoice: struct %s size alignment error\n", "recType");
	return (-1);
    }
    if ((sizeof(struct choiceRec) % 4)) {
	printf("bldRecChoice: struct %s size alignment error\n", "choiceRec");
	return (-1);
    }
    if ((sizeof(struct arrChoiceSet) % 4)) {
	printf("bldRecChoice: struct %s size alignment error\n", "arrChoiceSet");
	return (-1);
    }
    if ((sizeof(struct choiceSet) % 4)) {
	printf("bldRecChoice: struct %s size alignment error\n", "choiceSet");
	return (-1);
    }
    return (0);
}
/*
 * CMD_CHOICEREC
 *
 * checks command line arg usage
 *
 */
cmd_choiceRec(argc, argv)
    int             argc;
    char          **argv;
{
    extern char    *optarg;
    struct stat    *buf;
    int             c,
                    err;
    short           aflg,
                    bflg,
                    oflg;
    aflg = bflg = oflg = err = 0;

    if ((buf = (struct stat *)
	 calloc(1, (unsigned) sizeof(struct stat))) == NULL) {
	printf("bldRecChoice: calloc error\n");
	return 0;
    }
    while ((c = getopt(argc, argv, "a:b:o:")) != -1)
	switch (c) {
	case 'a':
	    aflg++;
	    strcpy(inFile1, optarg);
	    if (stat(inFile1, buf) == -1) {
		printf("bldRecChoice: Can't stat file=%s errno=%i\n",
		       inFile1, errno);
		err++;
	    }
	    break;
	case 'b':
	    bflg++;
	    strcpy(inFile2, optarg);
	    if (stat(inFile2, buf) == -1) {
		printf("bldRecChoice: Can't stat file=%s errno=%i\n",
		       inFile2, errno);
		err++;
	    }
	    break;
	case 'o':
	    oflg++;
	    strcpy(outFile, optarg);
	    break;
	}
    if (!aflg || !bflg || !oflg || err) {
	printf("Usage: \n");
	printf(" <bldRecChoice -a choiceRec.ascii -b dbRecType.sdr -o choiceRec.sdr >\n");
	return 0;
    }
    return 1;
}
/*
 * INSCAN
 *
 */
inScan(scan_type, plineIn, pInfo)
    int             scan_type;
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
{
    int             no_byts;
    int             err = 0;
    switch (scan_type) {
    case SCAN_DIM:
	if ((sscanf(plineIn->line, "%3s%i%i",
		    pInfo->dim_keywd,
		    &pInfo->dim_size,
		    &pInfo->dim_entries)) != 3) {
	    err = 1;
	    break;
	}
	if ((strcmp("DIM", pInfo->dim_keywd)) != SAME)
	    err = 4;
	break;
    case SCAN_RECTYPE:
	if ((sscanf(plineIn->line, "%7s \"%50[^\"]\"%i%i",
		    pInfo->RecTyp_keywd,
		    pInfo->RecTyp_name,
		    &pInfo->index_size,
		    &pInfo->index_entries)) != 4) {
	    err = 1;
	    break;
	}
	if ((strcmp("RECTYPE", pInfo->RecTyp_keywd)) != SAME)
	    err = 2;
	break;
    case SCAN_INDEX:
	if ((sscanf(plineIn->line, "%5s%i%i%i",
		    pInfo->index_keywd,
		    &pInfo->index_num,
		    &pInfo->choice_size,
		    &pInfo->choice_entries)) != 4) {
	    err = 1;
	    break;
	}
	if ((strcmp("INDEX", pInfo->index_keywd)) != SAME)
	    err = 3;
	break;
    case SCAN_STRING:
	if ((sscanf(plineIn->line, "%i \"%n",
		    &pInfo->choice_num, &no_byts)) != 1) {
	    err = 1;
	    break;
	}
	nextInputFld(plineIn, &no_byts);

	if ((plineIn->pnext[0]) == '"') {
	    pInfo->choice_str[0] = '\0';
	    no_byts = 1;
	} else if ((sscanf(plineIn->pnext, "%50[^\"]%n",
			   pInfo->choice_str, &no_byts)) != 1) {
	    err = 1;
	    break;
	}
	nextInputFld(plineIn, &no_byts);
	break;
    default:
	printf("bldRecChoice: inScan(scan_type) not defined internally\n");
	printf("bldRecChoice: line=%i\n", plineIn->line_no);
	return (-1);
	break;
    }
    if (err == 1) {
	printf("bldRecChoice: Illegal field format file=%s\n", plineIn->fname);
	printf("bldRecChoice: line=%i\n", plineIn->line_no);
	return (-1);
    }
    if (err == 2) {
	printf("bldRecChoice: Error on RECTYP control line - line=%i\n", plineIn->line_no);
	printf("bldRecChoice: Illegal field format file=%s\n", plineIn->fname);
	return (-1);
    }
    if (err == 3) {
	printf("bldRecChoice: Error on INDEX control line - line=%i\n", plineIn->line_no);
	printf("bldRecChoice: Illegal field format file=%s\n", plineIn->fname);
	return (-1);
    }
    if (err == 4) {
	printf("bldRecChoice: Error on DIM control line - line=%i\n", plineIn->line_no);
	printf("bldRecChoice: Illegal field format file=%s\n", plineIn->fname);
	return (-1);
    }
    return (0);
}

