/*bldRecDef.c,v 1.1.1.1 1999/08/03 14:41:56 tis4000 Exp */

/*  @(#)bldRecDef.c	1.19	6/7/91
 *	Author: Robert Zieman (ANL)
 *	Date:	6/03/91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	mm-dd-yy	iii	Comment
 * .02	mm-dd-yy	iii	Comment
 * 	...
 */

#include <stdio.h>
#include <ctype.h>
#include <sys/file.h>		/* SUN support for disk io */
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include "dbDefs.h"
#include "dbRecDes.h"
#include "dbFldTypes.h"
#include "dbRecType.h"
#include "sdrHeader.h"
#include "getSdrLine.h"
#include "link.h"
#include "DBDefines.h"
#include "DBProto.h"
#include "vucreateall.h"
#define BYTES10		10
#define BYTES20		20
#define BYTES50		50
#define SAME 0
#define SCAN_RECTYP_A	1
#define SCAN_RECTYP_B	2
#define SCAN_COM_FLDS	3
#define SCAN_STRING	4
#define SCAN_NOACCESS	5
#define SCAN_CHOICE_SET	6
#define SCAN_INIT_VAL	7
#define SCAN_PMT_FLAG	8
#define SCAN_RANGE	9
#define SCAN_CVT_TYPE	10
#define LOWFL	1
#define HIGHFL	2
/* I/O filenames */
static char     in1[BYTES50];
static char     in2[BYTES50];
static char     out1[BYTES50];
static char     out2[BYTES50];
static char     out3[BYTES50];
static char     out4[BYTES50];

static char     rectyp[] = "%6s \"%50[^\"]\"%hd%n";

/* top common part of all header files */
static char    *text_str[] = {
    "#ifndef	INCLlstLibh",
    "#include <lstLib.h>	/* list structure definition for the monitor list */",
    "#endif",
    "#ifndef	INCLsemLibh",
    "#include <semLib.h>	/* semaphore structure definition for the monitor lock*/",
    "#endif",
    "#ifndef INCLfast_lockh",
    "#include <fast_lock.h>	/* fast lock data structures and macros */",
    "#endif",
    "#ifndef INCLlinkh",
    "#include <link.h>	/* fast lock data structures and macros */",
    "#endif",
    "#ifndef INC_tsDefs_h",
    "#include <tsDefs.h>	/* time-stamp related definitions */",
    "#endif"
};
/* bottom common part of all header files */
static char     strend[] = "\n};\n#endif\n\n";
#define TEXT_COUNT (sizeof(text_str) / sizeof(char*))

/* input line work area */
static struct inLineInfo {
    char            prompt[MAXLINE];
    char            fldname[BYTES10];
    short           size;
    short           special;
    short           field_type;
    short           dbr_field_type;
    short           interest;
    short           process_passive;
    short           hilo;	/* highfl or lowfl */
    char            hl_Var[BYTES20];	/* hilo value string */
    short           promptflag;
    short           choice_set;
    short           cvt_type;
    char            initial[BYTES20];
    short           dbCommon_dim;
    short           ascii_dim;
    char            text_code[MAXLINE];	/* if field_type is dbf_noaccess */
    char            rectyp_keywd[BYTES20];
    char            dbCommon_name[BYTES50];
    char          **papdbCommon;
    char            ascii_name[BYTES50];
}               inLineInfo;

struct lineIn   lineIn;		/* buffer for getSdrLine */

struct tmp_flds {
    char            L_Var[BYTES20];	/* lowfl value string */
    char            H_Var[BYTES20];	/* highfl value string */
};

extern char    *calloc();
int             errno;
struct sdrHeader sdrHeader;
struct dbdatabase	DB_Database;
DB_TABLE_PTR		DBC_XrefTable;
DB_FIELD_PTR		xrefNameField;
DB_FIELD_PTR		xrefTableField;
DB_RECORD_PTR		xrefRecord;
char			key[40];
char			xrefFieldName[11];
char			xrefTableName[25];
F4FIELD_INFO		*info;

/*
 *
 * BLDRECDEF.C
 *
 * MAINLINE
 *
 * INPUT:
 *       File     dbRecType.sdr
 *
 *       File     dbRecDes.ascii
 *
 *                Built from dbCommon.ascii and <record type>.ascii files
 *                which have been concatenated and processed by cpp.
 *                Note: dbCommon.ascii must be the first file in
 *                      dbRecDes.ascii.
 *
 * OUTPUT:
 *       File     dbRecDes.sdr
 *
 *       Files    <dbCommon, aiRecord, etc>.h
 *
 *
 *	The SDR field lines from dbCommon.ascii are prepended to
 *      each of the record type description files.
 *	Data is collected in temporary areas as each record type
 *      field line is processed. At the end of each record several
 *      sorted arrays are initialized. When all records have been
 *	processed a contiguous block of data (structure dbRecDes)
 *      is composed from all the stored data. All pointers are
 *      changed to offsets prior to writing the output SDR file.
 *      An sdrHeader is placed at the front of the output file.
 *
 *      As each record is processed, an appropriate header file
 *      is built. Note: The dbCommon.h header file is built while
 *      the first record type is processed.
 *
 */

main(argc, argv)
    int             argc;
    char          **argv;
{
    FILE           *in1_fp,
                   *fopen();
    FILE           *out1_fp;
    FILE           *out2_fp;
    FILE           *out3_fp;
    FILE           *out4_fp;
    struct lineIn  *plineIn = &lineIn;	/* ptr to buffer */

    struct inLineInfo *pInfo = &inLineInfo;	/* input line work area */
    struct recTypDes *pRecTypDes;
    struct fldDes  *pfield;
    struct recDes  *pTdbRecDes;
    struct tmp_flds **ptmp_flds;

    long            rsize;
    char           *pSrc;
    char           *pDest;
    char           *addr;
    char            buffer[MAXBUF];
    char            sNameBuf[BYTES50];
    short           no_fields;
    short           field_type;
    short           dflt_size;
    short           CurOffSet = 0;
    short           delta = 0;
    int             SizeDefined;/* signals lines with defined field size */
    int             rec_no;
    int             last_common = FALSE;
    int             first_common = TRUE;	/* until dbCommon is
						 * processed */
    int             tmpPmt;
    int             ind;	/* used to determine dbCommon fields  */
    int             len;
    int             no_prompt = 0;
    int             no_links = 0;
    int             no_records = 0;
    int             eof_flg = FALSE;
    int             eor_flg = FALSE;
    int             db_com_end_flg;
    int             found;
    int             i = 0;
    int             j = 0;
    int			k;
    int			l;
    struct recTypDes	*recTypDes;
    char	*typeString;
/* allocate space for the getSdrLine input structure */
    if ((plineIn = (struct lineIn *)
	 calloc(1, (unsigned) sizeof(struct lineIn))) == NULL) {
	printf("bldRecDef: calloc error\n");
	return (-1);
    }
    if (chk_align() < 0)
	return (-1);

    if (!cmd_bldRecDef(argc, argv)) {
	return (-1);
    }
    plineIn->fname = in1;
    if ((in1_fp = fopen((caddr_t) in1, "r")) == NULL) {
	printf("bldRecDef: fopen error errno=%i\nfile=%s\n", errno, in1);
	return (-1);
    }
    /* load ascii record type definitions - dbRecType.sdr */
    if (sdrLoad(in2))
	return (-1);

    /* allocate space for temp struct recDes */
    if ((pTdbRecDes = (struct recDes *)
	 calloc(1, (unsigned) sizeof(struct recDes))) == NULL) {
	printf("bldRecDef: calloc error\n");
	return (-1);
    }
    rsize = sizeof(struct recDes);

    /* allocate space for unnamed ptr array */
    if ((pTdbRecDes->papRecTypDes = (struct recTypDes **)
       calloc(1, (unsigned) dbRecType->number * sizeof(caddr_t))) == NULL) {
	printf("bldRecDef: calloc error\n");
	return (-1);
    }
    rsize += dbRecType->number * sizeof(caddr_t);

    /* store a working  copy of dbCommon field definition lines */
    if ((store_dbCommon(in1_fp, plineIn, pInfo)) < 0)
	return (-1);

    while (!eof_flg) {
	/* Main loop - scan file for the next control line */
	while (!eof_flg) {
	    {
		if ((getSdrLine(in1_fp, plineIn, TWO_QUOTES)) < 0)
		    return (-1);
		if (plineIn->eof == EOF) {
		    eof_flg = TRUE;
		    break;
		}
		if ((inScan(SCAN_RECTYP_B, plineIn, pInfo)) < 0)
		    return (-1);

		/* Verify record type name */
		found = FALSE;
		for (i = 0; i < dbRecType->number; i++) {
		    if ((dbRecType->papName[i]) && ((strcmp(dbRecType->papName[i],
					     pInfo->ascii_name)) == SAME)) {
			found = TRUE;
			rec_no = i;
			no_records++;
			break;
		    }
		}
		if (!found) {
		    printf("bldRecDef: Error - Record type '%s' not defined\n",
			   pInfo->ascii_name);
		    return (-1);
		}
		if (first_common) {
		    /* open dbCommon header file for writing */
		    strcpy(out2, "dbCommon.h");
		    if ((out2_fp = fopen((caddr_t) out2, "w")) == NULL) {
			printf("bldRecDef: fopen error errno=%i\nfile=%s\n",
			       errno, out2);
			return (-1);
		    }
		    strcpy(sNameBuf, "dbCommon");
		    if ((wrt_com(out2_fp, buffer, out2, sNameBuf)) < 0)
			return (-1);
		    /* open special file for tispermissions */
		    strcpy (out4, "fieldsInfo.txt");
		    if ((out4_fp = fopen (out4, "w")) == NULL)
		    {
			printf ("bldRecDef, fopen error errno=%i, file=%s\n",
				 errno, out4);
			return (-1);
		    }
		}
		/* open current xxRecord header file for writing */
		strcpy(out3, dbRecType->papName[rec_no]);
		strcat(out3, "Record.h");
		if ((out3_fp = fopen((caddr_t) out3, "w")) == NULL) {
		    printf("bldRecDef: fopen error errno=%i\nfile=%s\n",
			   errno, out3);
		    return (-1);
		}
		strcpy(sNameBuf, dbRecType->papName[rec_no]);
		strcat(sNameBuf, "Record");
		if ((wrt_com(out3_fp, buffer, out3, sNameBuf)) < 0)
		    return (-1);
		/*
		 * allocate space for struct recTypDes and plant pointer
		 */
		if ((pTdbRecDes->papRecTypDes[rec_no] = (struct recTypDes *)
		  calloc(1, (unsigned) sizeof(struct recTypDes))) == NULL) {
		    printf("bldRecDef: calloc error\n");
		    return (-1);
		}
		pRecTypDes = pTdbRecDes->papRecTypDes[rec_no];
		rsize += sizeof(struct recTypDes);
		no_fields = pInfo->dbCommon_dim + pInfo->ascii_dim;
		pRecTypDes->no_fields = no_fields;

		/* allocate a pointer array */
		if ((pRecTypDes->papFldDes = (struct fldDes **)
		calloc(1, (unsigned) no_fields * sizeof(caddr_t))) == NULL) {
		    printf("bldRecDef: calloc error\n");
		    return (-1);
		}
		rsize += no_fields * sizeof(caddr_t);
		/* allocate a tmp pointer array to hold VAR if necessary */
		if ((ptmp_flds = (struct tmp_flds **)
		calloc(1, (unsigned) no_fields * sizeof(caddr_t))) == NULL) {
		    printf("bldRecDef: calloc error\n");
		    return (-1);
		}
	    }
	    if (eof_flg)
		break;
	    else
		eor_flg = FALSE;

	    /* process this record type */
	    ind = 0;
	    db_com_end_flg = FALSE;

	    while (!eof_flg) {	/* FLD PROCESSING */
		if (ind >= pRecTypDes->no_fields) {
		    eor_flg = TRUE;
		    break;	/* end of record type definition */
		}
		if (!db_com_end_flg) {

		    /* copy a line from the stored dbcommon array */
		    pSrc = pInfo->papdbCommon[ind];
		    pDest = plineIn->line;
		    plineIn->pnext = pDest;
		    while ((*pDest++ = *pSrc++) != '\0');
		    if (ind >= pInfo->dbCommon_dim - 1)
			db_com_end_flg = TRUE;
		} else {
		    /* stop dbCommon.h processing */
		    first_common = FALSE;
		    /* obtain next line from the dbRecDes.ascii input file */
		    if ((getSdrLine(in1_fp, plineIn, TWO_QUOTES)) < 0)
			return (-1);
		    if (plineIn->eof == EOF) {
			eof_flg = TRUE;
			break;
		    }
		}
		/* allocate a structure for temp field storage */
		if ((ptmp_flds[ind] = (struct tmp_flds *)
		   calloc(1, (unsigned) sizeof(struct tmp_flds))) == NULL) {
		    printf("bldRecDef: calloc error\n");
		    return (-1);
		}
		/* allocate a field structure */
		if ((pRecTypDes->papFldDes[ind] = (struct fldDes *)
		     calloc(1, (unsigned) sizeof(struct fldDes))) == NULL) {
		    printf("bldRecDef: calloc error\n");
		    return (-1);
		}
		pfield = pRecTypDes->papFldDes[ind];
		rsize += sizeof(struct fldDes);
		SizeDefined = FALSE;
		dflt_size = 0;
		bzero((caddr_t) pInfo->fldname, (int) FLDNAME_SZ);

		/* Note: the first N fields (7)  must be present */
		if ((inScan(SCAN_COM_FLDS, plineIn, pInfo)) < 0)
		    return (-1);

		field_type = pInfo->field_type;
		pfield->field_type = field_type;

		/* check for legal field_type */
		if (field_type >= DBF_NTYPES || field_type < 0) {
		    printf("bldRecDef: illegal field_type\n");
		    printf("file %s line number %d\n%s - RecType=%s\n",
		    plineIn->fname, plineIn->line_no, plineIn->line,
		    pInfo->ascii_name);
		    return (-1);
		}
		/* prompt is always null terminated */
		len = strlen(pInfo->prompt);
		if (len >= PROMPT_SZ) {
		    printf("bldRecDef: prompt string too long\n");
		    printf("file %s line number %d\n%s - RecType=%s\n",
		    plineIn->fname, plineIn->line_no, plineIn->line,
		    pInfo->ascii_name);
		    return (-1);
		}
#if 0
		bcopy(pInfo->prompt, pfield->prompt, PROMPT_SZ - 1);
#endif

		len = strlen(pInfo->fldname);
		if (len > FLDNAME_SZ) {
		    printf("bldRecDef: fldname string too long lineno=%d\n",
			   plineIn->line_no);
		    return (-1);
		}
		/* fldname is space filled if less than FLDNAME_SZ */
		add_sp(pInfo->fldname, FLDNAME_SZ);
		bcopy(pInfo->fldname, pfield->fldname, FLDNAME_SZ);
		pfield->special = pInfo->special;
		pfield->dbr_field_type = pInfo->dbr_field_type;
#if 0
		pfield->interest = pInfo->interest;
#endif
		pfield->process_passive = pInfo->process_passive;

		if (field_type == DBF_STRING) {
		    /* size followed by promptflag */
		    if ((inScan(SCAN_STRING, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->size = pInfo->size;
		    if ((inScan(SCAN_PMT_FLAG, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->promptflag = pInfo->promptflag;
		    SizeDefined = TRUE;
		} else if (field_type == DBF_CHAR
			   || field_type == DBF_UCHAR
			   || field_type == DBF_SHORT
			   || field_type == DBF_USHORT
			   || field_type == DBF_LONG
			   || field_type == DBF_ULONG
			   || field_type == DBF_FLOAT
			   || field_type == DBF_DOUBLE
			   || field_type == DBF_ENUM) {
		    /* initial value followed by promptflag */
		    if ((inScan(SCAN_INIT_VAL, plineIn, pInfo)) < 0)
			return (-1);
#if 0
		    if ((conv_strng(plineIn, pInfo, pInfo->initial, &pfield->initial)) < 0)
			return (-1);
#endif
		    if ((inScan(SCAN_PMT_FLAG, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->promptflag = pInfo->promptflag;

		    /* ranges possibly followed by cvt_type */
		    if (pfield->promptflag > 0)
			if ((scan_ranges(plineIn, pInfo, pfield, ptmp_flds[ind])) < 0)
			    return (-1);

		} else if (field_type == DBF_GBLCHOICE || field_type == DBF_RECCHOICE) {
		    /* initial followed by  choice_set index */
		    if ((inScan(SCAN_INIT_VAL, plineIn, pInfo)) < 0)
			return (-1);
#if 0
		    if ((conv_strng(plineIn, pInfo, pInfo->initial, &pfield->initial)) < 0)
			return (-1);
#endif
		    if ((inScan(SCAN_CHOICE_SET, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->choice_set = pInfo->choice_set;
		    if ((inScan(SCAN_PMT_FLAG, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->promptflag = pInfo->promptflag;
		} else if (field_type == DBF_CVTCHOICE || field_type == DBF_DEVCHOICE) {
		    /* initial value followed by promptflag */
		    if ((inScan(SCAN_INIT_VAL, plineIn, pInfo)) < 0)
			return (-1);
#if 0
		    if ((conv_strng(plineIn, pInfo, pInfo->initial, &pfield->initial)) < 0)
			return (-1);
#endif
		    if ((inScan(SCAN_PMT_FLAG, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->promptflag = pInfo->promptflag;
		} else if (field_type == DBF_INLINK
			   || field_type == DBF_OUTLINK
			   || field_type == DBF_FWDLINK) {
		    no_links++;
		    if ((inScan(SCAN_PMT_FLAG, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->promptflag = pInfo->promptflag;
		} else if (field_type == DBF_NOACCESS) {
		    /* size followed by code to be inserted in .h file */
		    if ((inScan(SCAN_NOACCESS, plineIn, pInfo)) < 0)
			return (-1);
		    pfield->size = pInfo->size;
		    SizeDefined = TRUE;
		} else {
		    printf("bldRecDef: illegal field_type\n");
		    printf("file %s line number %d\n%s - RecType=%s\n",
		    plineIn->fname, plineIn->line_no, plineIn->line,
		    pInfo->ascii_name);
		    return (-1);

		}
		if (pfield->promptflag > 0) {
		    no_prompt++;
		    /*
		     * add dbcommon offset to promptflags > 0 when not
		     * processing dbCommon lines
		     */

		    if (db_com_end_flg) {
			pfield->promptflag += last_common;
		    } else {
			tmpPmt = pfield->promptflag;
			last_common = (last_common < tmpPmt ? tmpPmt : last_common);
		    }

		}
		/* if field size not required in line - set to default */
		if (!SizeDefined) {
		    set_dflt(field_type, &dflt_size);
		    pfield->size = dflt_size;
		}
		pfield->offset = CurOffSet;
		if (chkFieldAlignment(field_type, CurOffSet, pfield, plineIn, pInfo))
		    return (-1);
		CurOffSet += pfield->size;
		if (first_common) {
		    if ((add_member(out2_fp,
				    buffer,
				    out2,
				    pfield,
				    pInfo,
				    plineIn)) < 0)
			return (-1);
		}
		/* add comment */
		if (ind == pInfo->dbCommon_dim) {
		    strcpy(buffer, "\n\t/* start of ");
		    strcat(buffer, dbRecType->papName[rec_no]);
		    strcat(buffer, " specific fields */ ");
		    if ((wrt_buf(out3_fp, buffer, out3)) < 0)
			return (-1);
		}
		if ((add_member(out3_fp,
				buffer,
				out3,
				pfield,
				pInfo,
				plineIn)) < 0)
		    return (-1);
		write_tisPermFieldsInfo (out4_fp,
					 dbRecType -> papName [rec_no],
					 pfield,
					 pInfo);
		ind++;
	    }			/* FLD PROCESSING */
	    /* write the end part of the header file */
	    strcpy(buffer, strend);
	    if ((wrt_buf(out3_fp, buffer, out3)) < 0)
		return (-1);
	    if ((fclose(out3_fp)) == EOF) {
		printf("bldRecDef: fclose error. errno=%i\nfile=%s\n",
		       errno, out1);
		return (-1);
	    }
	    /* convert any VAR ranges */
	    if ((conv_var_ranges(pRecTypDes, pInfo, plineIn, ptmp_flds)) < 0)
		return (-1);
	    /* check ( 	lowfl >= initial <= highfl   ) is true */
	    if ((check_range(pInfo, pRecTypDes, plineIn)) < 0)
		return (-1);
	    /* build the *link_ind array */
	    if ((pRecTypDes->link_ind = (short *)
		 calloc(1, (unsigned) no_links * sizeof(short))) == NULL) {
		printf("bldRecDef: calloc error\n");
		return (-1);
	    }
	    rsize += no_links * sizeof(short);
	    rsize = (rsize + sizeof(long) - 1) / sizeof(long) * sizeof(long);
	    i = 0;
	    for (j = 0; j < pRecTypDes->no_fields; j++) {
		switch (pRecTypDes->papFldDes[j]->field_type) {
		case DBF_INLINK:
		case DBF_OUTLINK:
		case DBF_FWDLINK:
		    pRecTypDes->link_ind[i] = (short) j;
		    i++;
		    break;
		default:
		    break;
		}
	    }

	    /* install sorted fields and corresponding offsets */
	    /* allocate space for the two arrays */
	    if ((pRecTypDes->sortFldName = (unsigned long *)
		 calloc(1, (unsigned) no_fields * sizeof(long))) == NULL) {
		printf("bldRecDef: calloc error\n");
		return (-1);
	    }
	    rsize += no_fields * sizeof(long);
	    if ((pRecTypDes->sortFldInd = (short *)
		 calloc(1, (unsigned) no_fields * sizeof(short))) == NULL) {
		printf("bldRecDef: calloc error\n");
		return (-1);
	    }
	    rsize += no_fields * sizeof(short);
	    rsize = (rsize + sizeof(long) - 1) / sizeof(long) * sizeof(long);
	    order_fields(pRecTypDes);
	    pRecTypDes->no_links = no_links;
	    pRecTypDes->no_prompt = no_prompt;
	    /* round up rec_size for sparc */
	    if ( delta = (CurOffSet % 8))
		CurOffSet += delta;
	    pRecTypDes->rec_size = CurOffSet;
	    no_links = 0;
	    no_prompt = 0;
	    CurOffSet = 0;
	}
	if (eof_flg && !eor_flg) {
	    printf("bldRecDef: Error - Unexpected EOF - RecType=%s\n",
		   pInfo->ascii_name);
	    printf("	%s\n", in1);
	    return (-1);
	}
	if (eof_flg && eor_flg)
	    break;
    }
    strcpy(buffer, strend);
    if ((wrt_buf(out2_fp, buffer, out2)) < 0)
	return (-1);
    if ((fclose(out2_fp)) == EOF) {
	printf("bldRecDef: fclose error. errno=%i\nfile=%s\n",
	       errno, out1);
	return (-1);
    }
    /* allocate the contigous area */
    if ((dbRecDes = (struct recDes *) calloc(1, (unsigned) rsize)) == NULL) {
	printf("bldRecDef: calloc error (struct recDes)\n");
	return (-1);
    }
    /* initialize all of the pointers and structures in the */
    /* contiguous area (dbRecDes) */
    addr = (caddr_t) dbRecDes + sizeof(struct recDes);

    /* insert ptr for the recDes structure */
    dbRecDes->papRecTypDes = (struct recTypDes **) addr;
    dbRecDes->number = dbRecType->number;
    /* set addr to just after the ptr array */
    addr += dbRecType->number * sizeof(caddr_t);
    for (i = 0; i < dbRecType->number; i++) {
	pRecTypDes = pTdbRecDes->papRecTypDes[i];
	if (pRecTypDes) {
	    /* insert ptr for the recTypDes structure */
	    dbRecDes->papRecTypDes[i] = (struct recTypDes *) addr;
	    /* copy the structure (recTypDes) */
	    bcopy((caddr_t) pRecTypDes, addr, sizeof(struct recTypDes));
	    addr += sizeof(struct recTypDes);
	    /* insert ptr to array of fldDes ptrs  */
	    dbRecDes->papRecTypDes[i]->papFldDes = (struct fldDes **) addr;
	    /* set addr to just after the ptr array */
	    addr += pRecTypDes->no_fields * sizeof(caddr_t);

	    for (j = 0; j < pRecTypDes->no_fields; j++) {
		/* insert each fldDes structure ptr */
		dbRecDes->papRecTypDes[i]->papFldDes[j]
		    = (struct fldDes *) addr;
		/* copy the structure (fldDes) */
		bcopy((caddr_t) pRecTypDes->papFldDes[j],
		      addr, sizeof(struct fldDes));
		addr += sizeof(struct fldDes);
	    }
	    /* insert ptr for the sortFldName array */
	    dbRecDes->papRecTypDes[i]->sortFldName = (unsigned long *) addr;
	    /* copy the array (sortFldName) */
	    bcopy((caddr_t) pRecTypDes->sortFldName,
		  addr, pRecTypDes->no_fields * sizeof(long));
	    addr += pRecTypDes->no_fields * sizeof(long);
	    /* insert ptr for the sortFldInd array */
	    dbRecDes->papRecTypDes[i]->sortFldInd = (short *) addr;
	    /* copy the array (sortFldInd) */
	    bcopy((caddr_t) pRecTypDes->sortFldInd,
		  addr, pRecTypDes->no_fields * sizeof(short));
	    addr += pRecTypDes->no_fields * sizeof(short);
	    /* make sure we have padding to long words */
	    addr = (char *)(((long) addr + sizeof(long) - 1)
		/ sizeof(long) * sizeof(long));

	    /* insert ptr for the link_ind array */
	    dbRecDes->papRecTypDes[i]->link_ind = (short *) addr;
	    /* copy the array (sortFldInd) */
	    bcopy((caddr_t) pRecTypDes->link_ind,
		  addr, pRecTypDes->no_links * sizeof(short));
	    addr += pRecTypDes->no_links * sizeof(short);

	    /* make sure we have padding to long words */
	    addr = (char *)(((long) addr + sizeof(long) - 1)
		/ sizeof(long) * sizeof(long));
	}
    }

    /* write output file */
    if ((out1_fp = fopen((caddr_t) out1, "w")) == NULL) {
	printf("bldRecDef: fopen errno=%i \nfile=%s\n",
	       errno, out1);
	return (-1);
    }

    /* Init access to dbLibrary */
    DB_InitDatabase(argv[0]);
    DB_MapDatabase("../../Unix/sun2.4/dbClient");
    if ((DBC_XrefTable = DB_MapTable("pdbxref")) == 0)
	{
	printf("Can't open PDBXREF.DBF. Bummer.\n");
	exit(1);
	}
    xrefNameField = DB_MapField(DBC_XrefTable, "FIELD_NAME");
    xrefTableField = DB_MapField(DBC_XrefTable, "TABLE");

    fprintf(out1_fp, "dbRecDes\n");
    fprintf(out1_fp, "%d\n", dbRecDes -> number);
    for (i = 0; i < dbRecDes -> number; i++)
	{
	if (!(dbRecDes -> papRecTypDes[i]))
	    {
	    fprintf(out1_fp, "0\n");
	    continue;
	    }
        recTypDes = GET_PRECTYPDES(i);	/* point to type descriptor in SDR */
	typeString = GET_PRECTYPE(i);
	fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> rec_size);
	fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> no_fields);
	fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> no_prompt);
	fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> no_links);
	for (j = 0; j < dbRecDes -> papRecTypDes[i] -> no_links; j++)
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> link_ind[j]);
	for (j = 0; j < dbRecDes -> papRecTypDes[i] -> no_fields; j++)
	    fprintf(out1_fp, "%4.4s\n", &(dbRecDes -> papRecTypDes[i] -> sortFldName[j]));
	for (j = 0; j < dbRecDes -> papRecTypDes[i] -> no_fields; j++)
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> sortFldInd[j]);
	for (j = 0; j < dbRecDes -> papRecTypDes[i] -> no_fields; j++)
	    {
	    fprintf(out1_fp, "%4.4s\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> fldname);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> offset);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> size);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> special);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> field_type);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> dbr_field_type);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> process_passive);
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> choice_set);
/*
	Do float or double fields only.
*/
		if((dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> promptflag) &&
		   ((dbRecDes -> papRecTypDes[i] -> papFldDes[j]->field_type == DBF_FLOAT) || (dbRecDes -> papRecTypDes[i] -> papFldDes[j] ->field_type == DBF_DOUBLE))) /* position at XREF record */
			{
			sprintf(key,"%-15.15s%-4.4s\0",typeString,dbRecDes -> papRecTypDes[i] -> papFldDes[j]->fldname);	/* make XREF key */
			if((xrefRecord = DB_Seek(DBC_XrefTable,key)) != 0)
				{
				DB_GetFieldValue(xrefRecord,xrefNameField,xrefFieldName,(int *)(&field_type)); /* get OV field name */
				DB_GetFieldValue(xrefRecord,xrefTableField,xrefTableName,(int *)(&field_type)); /* point type table name */
				if(!strlen(xrefFieldName))		/* skip non-existant point types */
					break;
				if(!strlen(xrefTableName))		/* skip to next field if no table */
					{
					printf("ERROR! Can't find field %s\n", xrefTableName);
					continue;
					}
/*
	New for '95 GCT. Use 'vucreateall.h" to supply the database creation structures. Use the decimal field widths
	in the F4FIELD_INFO structures to modify the prompt field.
*/
				for(k = 0;info_array[k].file_name;k++)
					{
					if(!strcmp(xrefTableName,(char *)info_array[k].file_name))
						{
						info = (F4FIELD_INFO *)info_array[k].field_array;
						for(l = 0;info[l].name;l++)
							{
							if(!strcmp(xrefFieldName,(char *)info[l].name))
								{
								dbRecDes -> papRecTypDes[i] -> papFldDes[j]->promptflag = info[l].dec;
								}
							}
						}
					}
				}
			else
				{
				fprintf(stderr,"No PDBXREF entry for %s\n",key);
				}
			}
	    fprintf(out1_fp, "%d\n", dbRecDes -> papRecTypDes[i] -> papFldDes[j] -> promptflag);
	    }
	}

    if ((fclose(out1_fp)) == EOF) {
	printf("bldRecDef: fclose error. errno=%i\nfile=%s\n",
	       errno, out1);
	return (-1);
    }
    if (eof_flg && eor_flg) {
	if (no_records != dbRecType->number - 1) {
	    printf("bldRecDef: WARNING - MAJOR DIFFERENCES DETECTED:\n");
	    printf("bldRecDef: Only (%d) record types input. There are (%d) defined\n",
		   no_records, dbRecType->number);
	}
    }
    fclose (out4_fp);
    return (0);
}				/* end of main bldRecDef.c */

/*
 * CHKFIELDALIGNMENT
 * return	(0,-1)	SUCCESS,FAIL
 */
chkFieldAlignment(type, offset, papFldDes, plineIn, pInfo)
    short           type;
    short           offset;
    struct fldDes  *papFldDes;
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
{

    switch (type) {
	/* don't care */
    case DBF_STRING:
    case DBF_CHAR:
    case DBF_UCHAR:
	return (0);

	/* must be 2 */
    case DBF_SHORT:
    case DBF_USHORT:
    case DBF_ENUM:
    case DBF_GBLCHOICE:
    case DBF_CVTCHOICE:
    case DBF_RECCHOICE:
    case DBF_DEVCHOICE:
	if (offset % 2)
	    break;
	return (0);
	/* must be 4 */
    case DBF_LONG:
    case DBF_ULONG:
    case DBF_FLOAT:
    case DBF_INLINK:
    case DBF_OUTLINK:
    case DBF_FWDLINK:
	if (offset % 4)
	    break;
	return (0);

	/* must be 8 */
    case DBF_DOUBLE:
	if (offset % 8)
	    break;
	return (0);
    case DBF_NOACCESS:
	/* assume noAccess  definitions >= 4 must be %4 */
	if (papFldDes->size >= 4) {
	    if ((offset % 4) != 0)
		break;
	} else if (papFldDes->size == 0) {
	    break;

	    /* assume noAccess  definitions == 2 must be %2 */
	} else if (papFldDes->size == 2) {
	    if (offset % 2 != 0)
		break;
	}
	return (0);

    default:
	printf("bldRecDef: chkFieldAlignment(default reached)\n");
	break;
    }
    printf("bldRecDef: field alignment fail\n");
    printf("file %s line number %d\n%s - RecType=%s current offset=%d\n",
    plineIn->fname, plineIn->line_no, plineIn->line,
    pInfo->ascii_name, offset);
    return (-1);
}

/*
 * INSCAN
 *
 * Scans the input fields according to format
 *
 * return	-1	error
 *		0	success
 *
 */

int
inScan(scan_type, plineIn, pInfo)
    int             scan_type;
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
{
    int             no_byts;
    int             err = 0;
    if (plineIn->pnext == NULL) {
        printf("bldRecDef: Error: No more fields left in line\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
    }
    switch (scan_type) {
    case SCAN_RECTYP_A:
	if ((sscanf(plineIn->pnext, rectyp,
		    pInfo->rectyp_keywd,
		    pInfo->dbCommon_name,
		    &pInfo->dbCommon_dim,
		    &no_byts)) != 3) {
	    err = 1;
	    break;
	}
	if ((strcmp("RECTYP", pInfo->rectyp_keywd)) != SAME)
	    err = 1;
	break;
    case SCAN_RECTYP_B:
	if ((sscanf(plineIn->pnext, rectyp,
		    pInfo->rectyp_keywd,
		    pInfo->ascii_name,
		    &pInfo->ascii_dim,
		    &no_byts)) != 3) {
	    err = 1;
	    break;
	}
	if ((strcmp("RECTYP", pInfo->rectyp_keywd)) != SAME)
	    err = 1;
	break;
    case SCAN_COM_FLDS:
	if ((sscanf(plineIn->pnext, "\"%[^\"]\"%s%hd%hd%hd%hd%hd%n",
		    pInfo->prompt,
		    pInfo->fldname,
		    &pInfo->special,
		    &pInfo->field_type,
		    &pInfo->dbr_field_type,
		    &pInfo->process_passive,
		    &pInfo->interest,
		    &no_byts)) != 7)
	    err = 1;
	break;
    case SCAN_INIT_VAL:
	if ((sscanf(plineIn->pnext, "%s%n",
		    pInfo->initial,
		    &no_byts)) != 1)
	    err = 1;
	break;
    case SCAN_STRING:
	if ((sscanf(plineIn->pnext, "%hd%n",
		    &pInfo->size,
		    &no_byts)) != 1)
	    err = 1;
	break;
    case SCAN_NOACCESS:
	if ((sscanf(plineIn->pnext, "%hd %[^\n]%n",
		    &pInfo->size,
		    pInfo->text_code,
		    &no_byts)) != 2)
	    err = 1;
	break;
    case SCAN_CHOICE_SET:
	if ((sscanf(plineIn->pnext, "%hd%n",
		    &pInfo->choice_set,
		    &no_byts)) != 1)
	    err = 1;
	break;
    case SCAN_PMT_FLAG:
	if ((sscanf(plineIn->pnext, "%hd%n",
		    &pInfo->promptflag,
		    &no_byts)) != 1)
	    err = 1;
	break;
    case SCAN_RANGE:
	if ((sscanf(plineIn->pnext, "%hd%s%n",
		    &pInfo->hilo,
		    pInfo->hl_Var,
		    &no_byts)) != 2)
	    err = 1;
	break;
    case SCAN_CVT_TYPE:
	if ((sscanf(plineIn->pnext, "%hd%n",
		    &pInfo->cvt_type,
		    &no_byts)) != 1)
	    err = 1;
	break;
    default:
	printf("bldRecDef: inScan(default reached)\n");
	err = 1;
	break;
    }
    if (err == 1) {
        printf("bldRecDef: line format Error\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
    }
    nextInputFld(plineIn, &no_byts);
    return (0);
}

/*
 * CMD_BLDRECDEF
 *
 * checks command line args
 *
 */
int
cmd_bldRecDef(argc, argv)
    int             argc;
    char          **argv;
{
    extern char    *optarg;
    struct stat    *buf;
    int             c,
                    err;
    short           aflg,
                    bflg,
                    oflg;
    aflg = bflg = oflg = err = 0;

    if ((buf = (struct stat *)
	 calloc(1, (unsigned) sizeof(struct stat))) == NULL) {
	printf("bldRecDef: calloc error\n");
	return FALSE;
    }
    while ((c = getopt(argc, argv, "a:b:o:")) != -1)
	switch (c) {
	case 'a':
	    aflg++;
	    strcpy(in1, optarg);
	    if (stat(in1, buf) == -1) {
		printf("bldRecDef: Can't stat file=%s errno=%i\n",
		       in1, errno);
		err++;
	    }
	    break;
	case 'b':
	    bflg++;
	    strcpy(in2, optarg);
	    break;
	case 'o':
	    oflg++;
	    strcpy(out1, optarg);
	    break;
	}
    if (!bflg || !aflg || !oflg || err) {
	printf("Usage:\n");
	printf(" bldRecDef  -a dbRecDes.ascii -b dbRecType.sdr -o dbRecDes.sdr\n");
	return FALSE;
    }
    return TRUE;
}

/*
 * STORE_DBCOMMON
 *
 * Save dbCommon field definition lines
 *
 */
int
store_dbCommon(fp, plineIn, pInfo)
    FILE           *fp;
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
{
    int             i;
    int             len;
    if ((getSdrLine(fp, plineIn, TWO_QUOTES)) < 0)
	return (-1);
    if (plineIn->eof == EOF) {
	printf("bldRecDef: Error - Unexpected EOF\n");
	printf("while processing dbCommon\n");
	return (-1);
    }
    /* check format of dbCommon control line */
    if ((inScan(SCAN_RECTYP_A, plineIn, pInfo)) < 0)
	return (-1);
    len = strlen(plineIn->fname);
    if ((strncmp("dbCommon", pInfo->dbCommon_name, len)) != SAME) {
        printf("bldRecDef: DIM control line incorrect (dbCommon)\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
    }
    /* allocate an array of pointers to store dbCommon fld lines */
    if ((pInfo->papdbCommon = (caddr_t *)
     calloc(1, (unsigned) pInfo->dbCommon_dim * sizeof(caddr_t))) == NULL) {
	printf("bldRecDef: calloc error\n");
	return (-1);
    }
    /* copy the dbCommon field lines for later use */
    for (i = 0; i < pInfo->dbCommon_dim; i++) {
	if ((getSdrLine(fp, plineIn, TWO_QUOTES)) < 0)
	    return (-1);
	if (plineIn->eof == EOF) {
	    printf("bldRecDef: Error - Unexpected EOF\n");
	    printf("while processing dbCommon\n");
	    return (-1);
	}
	len = strlen(plineIn->line);
	/* allocate space for each dbCommon fld line */
	if ((pInfo->papdbCommon[i] = (caddr_t) calloc(1, (unsigned) len + 1)) == NULL) {
	    printf("bldRecDef: calloc error\n");
	    return (-1);
	}
	strcpy(pInfo->papdbCommon[i], plineIn->line);
    }
    return (0);
}

/*
 *	ADD_SP
 *
 *	pads spaces to strings < len
 *
 */
int
add_sp(pin, len)
    register char  *pin;
    register int    len;
{
    register int    i;
    for (i = 0; i < len; i++)
	if (*(pin + i) == 0)
	    *(pin + i) = ' ';
}
/*
 * ADD_MEMBER
 *
 * Add a member to the header file
 *
 */
int
add_member(fp, pbuf, pfilename, papFldDes, pInfo, plineIn)
    FILE           *fp;
    char           *pbuf;
    char           *pfilename;
    struct fldDes  *papFldDes;
    struct inLineInfo *pInfo;
    struct lineIn  *plineIn;
{
    short           fldsize = papFldDes->size;
    char            comment[BYTES50];
    char           *pcomment = comment;
    char            fldname[BYTES50];
    char           *pfldname = fldname;
    short           i = 0;
    /* change field name to lower case */
    while (*(papFldDes->fldname + i) && *(papFldDes->fldname + i) != ' '
	    && i < FLDNAME_SZ) {
	if (isupper(*(papFldDes->fldname + i))) {
	    *(pfldname + i) = tolower(*(papFldDes->fldname + i));
	} else {
	    *(pfldname + i) = *(papFldDes->fldname + i);
	}
	i++;
    }
    *(pfldname + i) = NULL;
#if 0
    strcpy(pcomment, papFldDes->prompt);
#else 
    comment[0] = NULL;
#endif

    switch (papFldDes->field_type) {
    case DBF_NOACCESS:
	strcpy(pbuf, "\n	");
	strcat(pbuf, pInfo->text_code);
	strcat(pbuf, "		/* ");
	strcat(pbuf, pcomment);
	strcat(pbuf, " */");
	break;
    case DBF_STRING:
	sprintf(pbuf, "\n	char		%s[%d];		/* %s */",
		pfldname, fldsize, pcomment);
	break;
    case DBF_ENUM:
    case DBF_CVTCHOICE:
    case DBF_DEVCHOICE:
    case DBF_GBLCHOICE:
    case DBF_RECCHOICE:
	sprintf(pbuf, "\n	unsigned short	%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_SHORT:
	sprintf(pbuf, "\n	short		%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_USHORT:
	sprintf(pbuf, "\n	unsigned short	%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_CHAR:
	sprintf(pbuf, "\n	char		%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_UCHAR:
	sprintf(pbuf, "\n	unsigned char	%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_LONG:
	sprintf(pbuf, "\n	long		%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_ULONG:
	sprintf(pbuf, "\n	unsigned long	%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_FLOAT:
	sprintf(pbuf, "\n	float		%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_DOUBLE:
	sprintf(pbuf, "\n	double		%s;		/* %s */",
		pfldname, pcomment);
	break;
    case DBF_INLINK:
    case DBF_OUTLINK:
    case DBF_FWDLINK:
	sprintf(pbuf, "\n	struct link	%s;		/* %s */",
		pfldname, pcomment);
	break;
    default:
        printf("bldRecDef: add_member (default reached)\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
	break;
    }
    if (wrt_buf(fp, pbuf, pfilename) < 0)
	return (-1);
    return (0);
}

/*
 * ORDER_FIELDS
 *
 * orders the field name and the associated field offset
 * in the sortFldName and sortFldInd arrays
 *
 */

int
order_fields(precTypDes)
    struct recTypDes *precTypDes;
{
    register short  orderinx;
    short           entry,
                   *pentry;
    char           *pFldName;
    unsigned long  *pFldNameArr;
    short          *pFldIndArr;
    unsigned long  *pLconv;
    short          *pSconv;
    short           no_ordered;
    no_ordered = 0;
    pFldNameArr = precTypDes->sortFldName;
    pFldIndArr = precTypDes->sortFldInd;

    for (entry = 0; entry < precTypDes->no_fields; entry++) {
	pFldName = &precTypDes->papFldDes[entry]->fldname[0];
	pentry = &entry;
	pLconv = pFldNameArr;
	pSconv = pFldIndArr;
	/* find entry place in the order so far */
	orderinx = 0;
	while ((orderinx < no_ordered)
		&& no_ordered > 0
	   && (strncmp((caddr_t) (pLconv - 1), pFldName, FLDNAME_SZ)) > 0) {
	    /* open up the space */
	    bcopy((caddr_t) (pLconv - 1), (caddr_t) pLconv, FLDNAME_SZ);
	    bcopy((caddr_t) (pSconv - 1), (caddr_t) pSconv, sizeof(short));
	    pLconv--;
	    pSconv--;
	    orderinx++;
	}
	/* enter at proper position */
	bcopy(pFldName, (caddr_t) pLconv, FLDNAME_SZ);
	bcopy((caddr_t) pentry, (caddr_t) pSconv, sizeof(short));
	pFldNameArr++;
	pFldIndArr++;
	/* increment number ordered */
	no_ordered++;
    }
    return;
}

/*
 *
 * CHK_ALIGN
 *
 * checks alignment of structures
 *
 */
int
chk_align()
{
#if 0
    if ((sizeof(struct fldDes) % 8)) {
	printf("bldRecDef: struct %s size alignment error\n", "fldDes");
	return (-1);
    }
    if ((sizeof(struct recTypDes) % 8)) {
	printf("bldRecDef: struct %s size alignment error\n", "recTypDes");
	return (-1);
    }
    if ((sizeof(struct recDes) % 8)) {
	printf("bldRecDef: struct %s size alignment error\n", "recDes");
	return (-1);
    }
    return (0);
#endif
}

/*
 * WRT_COM
 *
 * writes common part of a header file
 *
 */
int
wrt_com(fp, buf, pfname, psname)
    FILE           *fp;
    char           *pfname;
    char           *psname;
    char           *buf;
{
    char          **ptr = text_str;
    short           i;
    /* 1st 3 lines of header */
    sprintf(buf, "\n/* %s */", pfname);
    if ((wrt_buf(fp, buf, pfname)) < 0)
	return (-1);
    sprintf(buf, "\n#ifndef INC%sh", psname);
    if ((wrt_buf(fp, buf, pfname)) < 0)
	return (-1);
    sprintf(buf, "\n#define INC%sh", psname);
    if ((wrt_buf(fp, buf, pfname)) < 0)
	return (-1);

    /* common text of header */
    for (i = 0; i < TEXT_COUNT; i++, ptr++) {
	strcpy(buf, "\n");
	strcat(buf, *ptr);
	if ((wrt_buf(fp, buf, pfname)) < 0)
	    return (-1);
    }
    /* variable start of struct name */
    strcpy(buf, "\nstruct ");
    strcat(buf, psname);
    strcat(buf, "	{");
    if ((wrt_buf(fp, buf, pfname)) < 0)
	return (-1);
    return (0);
}

/*
 * WRT_BUF
 *
 * writes buffer to file
 *
 */
int
wrt_buf(fp, buf, pfname)
    FILE           *fp;
    char           *pfname;
    char           *buf;
{
    int             rtnval;
    int             len;
    len = strlen(buf);
    if ((rtnval = fwrite((caddr_t) buf, len, 1, fp)) != 1) {
	printf("bldRecDef: fwrite error. errno=%i rtnval=%i\nfile=%s.h\n",
	       errno, rtnval, pfname);
	return (-1);
    }
    return (0);
}

/*
 * proc_var_range
 *
 * return -1  error
 * return  0  success
 */

proc_var_range(pInfo, plineIn, hl_type, ptmp_flds, pRecTypDes, ind)
    struct inLineInfo *pInfo;
    struct lineIn  *plineIn;
    int             hl_type;
    struct tmp_flds *ptmp_flds;
    struct recTypDes *pRecTypDes;
    int             ind;
{
    struct fldDes **pfldDes;
    int             found;
    int             j;
    pfldDes = (struct fldDes **) pRecTypDes->papFldDes;
    found = FALSE;
    /* search for fldname and set fldnum */
    switch (hl_type) {
    case LOWFL:
	add_sp(ptmp_flds->L_Var, FLDNAME_SZ);
	for (j = 0; j < pRecTypDes->no_fields; j++) {
	    if ((strncmp(pfldDes[j]->fldname,
			 ptmp_flds->L_Var, FLDNAME_SZ)) == SAME) {
		found = TRUE;
		/* insert the offset found */
#if 0
		pfldDes[ind]->range1.fldnum = (long) j;
#endif
		break;
	    }
	}
	if (!found) {
	    printf("bldRecDef: Error - Can't match lowfl field name=%4.4s RecType=%s\n",
		   ptmp_flds->L_Var, pInfo->ascii_name);
	    return (-1);
	}
	break;
    case HIGHFL:
	add_sp(ptmp_flds->H_Var, FLDNAME_SZ);
	/* set range for highfl VAR */
	for (j = 0; j < pRecTypDes->no_fields; j++) {
	    if ((strncmp(pfldDes[j]->fldname,
			 ptmp_flds->H_Var, FLDNAME_SZ)) == SAME) {
		found = TRUE;
		/* insert the offset found */
#if 0
		pfldDes[ind]->range2.fldnum = (long) j;
#endif
		break;
	    }
	}
	if (!found) {
	    printf("bldRecDef: Error - Can't match highfl field name=%4.4s RecType=%s\n",
		   ptmp_flds->H_Var, pInfo->ascii_name);
	    return (-1);
	}
	break;
    default:
        printf("bldRecDef: default reached in proc_var_range\n");
	return (-1);
	break;
    }
    return (0);
}

/*
 * get_num_from_string
 *
 *	return 0,-1	success,fail
 */
get_num_from_string(pstring, pnum,plineIn,pInfo)
    char           *pstring;	/* input string */
    double         *pnum;	/* place to put number */
    struct inLineInfo *pInfo;
    struct lineIn  *plineIn;
{
    double          number;
    unsigned long   hexnum;
    if ((strncmp(pstring, "0x", 2)) == SAME) {	/* HEX */
	if (sscanf(pstring + 2, "%lx", &hexnum) != 1) {
        printf("bldRecDef: get_num_from_string hex CONVERT FAILED\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	    return (-1);
	}
	number = (double) hexnum;
    } else if ((sscanf(pstring, "%lg", &number)) != 1) {
        printf("bldRecDef: get_num_from_string float CONVERT FAILED\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);

    }
    *pnum = (double) number;
    return (0);
}

scan_ranges(plineIn, pInfo, pfield, ptmp_flds)
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
    struct fldDes  *pfield;
    struct tmp_flds *ptmp_flds;
{
    if ((inScan(SCAN_RANGE, plineIn, pInfo)) < 0)
	return (-1);
#if 0
    pfield->lowfl = pInfo->hilo;
    switch (pfield->lowfl) {
    case (CON):
	if ((conv_strng(plineIn, pInfo, pInfo->hl_Var, &pfield->range1.value)) < 0)
	    return (-1);
	break;
    case VAR:
	/* save VAR string for later processing */
	strcpy(ptmp_flds->L_Var, pInfo->hl_Var);
	break;
    default:
        printf("bldRecDef: scan_ranges lowfl (default reached)\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
	break;
    }
#endif
    if ((inScan(SCAN_RANGE, plineIn, pInfo)) < 0)
	return (-1);
#if 0
    pfield->highfl = pInfo->hilo;
    switch (pfield->highfl) {
    case (CON):
	if ((conv_strng(plineIn, pInfo, pInfo->hl_Var, &pfield->range2.value)) < 0)
	    return (-1);
	break;
    case VAR:
	/* save VAR string for later processing */
	strcpy(ptmp_flds->H_Var, pInfo->hl_Var);
	break;
    default:
        printf("bldRecDef: scan_ranges highfl (default reached)\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
	break;
    }
#endif

    switch (pfield->field_type) {
    case DBF_STRING:
    case DBF_GBLCHOICE:
    case DBF_CVTCHOICE:
    case DBF_RECCHOICE:
    case DBF_DEVCHOICE:
    case DBF_INLINK:
    case DBF_OUTLINK:
    case DBF_FWDLINK:
    case DBF_NOACCESS:
    case DBF_FLOAT:
    case DBF_DOUBLE:
    case DBF_ENUM:
	break;
    case DBF_CHAR:
    case DBF_UCHAR:
    case DBF_SHORT:
    case DBF_USHORT:
    case DBF_LONG:
    case DBF_ULONG:
	/* cvt_type is CT_DECIMAL or CT_HEX */
	if ((inScan(SCAN_CVT_TYPE, plineIn, pInfo)) < 0)
	    return (-1);
#if 0
	pfield->cvt_type = pInfo->cvt_type;
#endif
	break;
    default:
        printf("bldRecDef: scan_ranges - illegal field_type\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
	break;
    }
    return (0);
}

set_dflt(field_type, pdflt_size)
    short           field_type;
    short          *pdflt_size;
{

    /* define default sizes */

/****************************************************************/
/*								*/
/*		WARNING! BRAIN DAMAGE ALERT! WARNING! 		*/
/*								*/
/*	In a nice world, we could use the C sizeof operator	*/
/*	to make these things self adapting. However, this is	*/
/*	the real world. When 64 bit machines like the DEC	*/
/*	AXP and the someday ultraSPARC come along, it becomes	*/
/*	necessary to put in the size of the structures in	*/
/*	the IOC which will no longer be identical to the	*/
/*	same sizes on the workstation. On the DEC AXP, the	*/
/*	long and all pointers are 8 bytes. These things are	*/
/*	4 on the Sun. Therefore, we have to hard code some	*/
/*	of these things.					*/
/*								*/
/****************************************************************/
    switch (field_type) {
    case DBF_CHAR:
    case DBF_UCHAR:
	*pdflt_size = sizeof(char);
	break;
    case DBF_SHORT:
    case DBF_USHORT:
	*pdflt_size = sizeof(short);
	break;
    case DBF_LONG:
    case DBF_ULONG:
#if 0
	*pdflt_size = sizeof(long);
#endif
	*pdflt_size = 4;	/* IOC size prevails! */
	break;
    case DBF_FLOAT:
	*pdflt_size = sizeof(float);
	break;
    case DBF_DOUBLE:
	*pdflt_size = sizeof(double);
	break;
    case DBF_ENUM:
    case DBF_GBLCHOICE:
    case DBF_CVTCHOICE:
    case DBF_RECCHOICE:
    case DBF_DEVCHOICE:
	*pdflt_size = sizeof(short);
	break;
    case DBF_INLINK:
    case DBF_OUTLINK:
    case DBF_FWDLINK:
#if 0
	*pdflt_size = sizeof(struct link);
#endif
	*pdflt_size = 44;	/* IOC size prevails! */
	break;
    default:
	break;
    }
    return;
}

conv_var_ranges(pRecTypDes, pInfo, plineIn, ptmp_flds)
    struct recTypDes *pRecTypDes;
    struct inLineInfo *pInfo;
    struct lineIn  *plineIn;
    struct tmp_flds **ptmp_flds;
{
#if 0
    struct fldDes  *pfield;
    int             j = 0;
    for (j = 0; j < pRecTypDes->no_fields; j++) {
	pfield = pRecTypDes->papFldDes[j];
	if (pfield->lowfl == VAR)
	    if ((proc_var_range(pInfo, plineIn, LOWFL, ptmp_flds[j],
				pRecTypDes,
				j)) < 0) {
        printf("RecType=%s\nprompt=\"%s\" fldname=%4.4s\n",
		   pInfo->ascii_name,
		   pfield->prompt,
		   pfield->fldname);
		return (-1);
	    }
	if (pfield->highfl == VAR)
	    if ((proc_var_range(pInfo, plineIn, HIGHFL, ptmp_flds[j],
				pRecTypDes,
				j)) < 0) {
        printf("RecType=%s\nprompt=\"%s\" fldname=%4.4s\n",
		   pInfo->ascii_name,
		   pfield->prompt,
		   pfield->fldname);
		return (-1);
	    }
    }
#endif
    return (0);
}

 /* ensure that lowfl >= initial <= highfl */
 /* for each element with promptflag >0! */

check_range(pInfo, precTypDes, plineIn)
    struct inLineInfo *pInfo;
    struct recTypDes *precTypDes;
    struct lineIn  *plineIn;
{
#if 0

    struct fldDes  *pfield;
    struct fldDes  *pXfield;
    double          initial;
    double          range1;
    double          range2;
    int             j;


    for (j = 0; j < precTypDes->no_fields; j++) {
	pfield = precTypDes->papFldDes[j];
	if (pfield->promptflag < 1)
	    continue;

	/* ignore these types */
	switch (pfield->field_type) {
	case DBF_STRING:
	case DBF_GBLCHOICE:
	case DBF_CVTCHOICE:
	case DBF_RECCHOICE:
	case DBF_DEVCHOICE:
	case DBF_INLINK:
	case DBF_OUTLINK:
	case DBF_FWDLINK:
	case DBF_NOACCESS:
	    continue;
	    break;
	default:
	    break;
	}
	if ((get_value(pfield, &pfield->initial, &initial, pInfo, plineIn)) < 0)
	    return (-1);
	if (pfield->lowfl == CON) {
	    if ((get_value(pfield, &pfield->range1.value, &range1, pInfo, plineIn)) < 0)
		return (-1);
	} else if (pfield->lowfl == VAR) {
	    pXfield = precTypDes->papFldDes[pfield->range1.fldnum];
	    if ((get_value(pXfield, &pXfield->initial, &range1, pInfo, plineIn)) < 0)
		return (-1);
	} else if ((pfield->lowfl != CON) && (pfield->lowfl != VAR)) {
	    printf("\nlowfl error RecType=%s\nprompt=\"%s\" fldname=%4.4s\n",
		   pInfo->ascii_name, pfield->prompt, pfield->fldname);
	    return (-1);
	}
	if (pfield->highfl == CON) {
	    if ((get_value(pfield, &pfield->range2.value, &range2, pInfo, plineIn)) < 0)
		return (-1);
	} else if (pfield->highfl == VAR) {
	    pXfield = precTypDes->papFldDes[pfield->range2.fldnum];
	    if ((get_value(pXfield, &pXfield->initial, &range2, pInfo, plineIn)) < 0)
		return (-1);
	} else if ((pfield->highfl != CON) && (pfield->highfl != VAR)) {
	    printf("\nhighfl error RecType=%s\nprompt=\"%s\" fldname=%4.4s\n",
		   pInfo->ascii_name, pfield->prompt, pfield->fldname);
	    return (-1);
	}
	if (initial < range1 || initial > range2) {
	    printf("\nrange error RecType=%s\nprompt=\"%s\" fldname=%4.4s\n",
		   pInfo->ascii_name,
		   pfield->prompt,
		   pfield->fldname);
	    printf("range1=%-12.4g\ninitial=%-12.4g\nrange2=%-12.4g\n",
		   range1,
		   initial,
		   range2);
	    return (-1);

	}
    }
#endif
    return (0);
}

get_value(pfield, pnumber, pvalue, pInfo, plineIn)
    struct fldDes  *pfield;	/* field to be processed */
    union fld_types *pnumber;	/* decode range or initial value */
    double         *pvalue;	/* place to put value obtained */
    struct inLineInfo *pInfo;
    struct lineIn  *plineIn;
{
    switch (pfield->field_type) {
    case (DBF_DOUBLE):
	*(double *) pvalue = pnumber->double_value;
	return (0);
	break;
    case (DBF_FLOAT):
	*(double *) pvalue = pnumber->float_value;
	return (0);
	break;
    case (DBF_CHAR):
	*(double *) pvalue = pnumber->char_value;
	return (0);
	break;
    case (DBF_UCHAR):
	*(double *) pvalue = pnumber->uchar_value;
	return (0);
	break;
    case (DBF_ENUM):
	*(double *) pvalue = pnumber->enum_value;
	return (0);
	break;
    case (DBF_SHORT):
	*(double *) pvalue = pnumber->short_value;
	return (0);
	break;
    case (DBF_USHORT):
	*(double *) pvalue = pnumber->ushort_value;
	return (0);
	break;
    case (DBF_ULONG):
	*(double *) pvalue = pnumber->ulong_value;
	return (0);
	break;
    case (DBF_LONG):
	*(double *) pvalue = pnumber->long_value;
	return (0);
	break;
    default:
        printf("bldRecDef: get_value default reached\n");
        printf("RecType=%s\nfldname=%4.4s\n",
		   pInfo->ascii_name,
		   pfield->fldname);
	return (-1);
    }
    return (-1);
}

conv_strng(plineIn,pInfo,pstring, pvalue)
    struct lineIn  *plineIn;
    struct inLineInfo *pInfo;
    char           *pstring; /* pointer to string */
    union fld_types *pvalue; /* ptr to place to put number */
{
    double          number;
    int             err = 0;
    union fld_types tmpnum;

    if ((get_num_from_string(pstring, &number,plineIn,pInfo)) < 0)
	return (-1);

    switch (pInfo->field_type) {
    case (DBF_DOUBLE):
	*(double*)pvalue = (double) number;
	break;
    case (DBF_FLOAT):
	*(float *) pvalue = (float ) number;
	break;
    case (DBF_CHAR):
	  * (char *) pvalue = (char) number;
	if (number > 127 || number < -128)
	    err = 1;
	break;
    case (DBF_UCHAR):
	*(unsigned char *) pvalue = (unsigned char ) number;
	if (number < 0 || number > 255)
	    err = 1;
	break;
    case (DBF_ENUM):
	*(unsigned short *) pvalue = (unsigned short) number;
	if (number < 0 || number > 65535)
	    err = 1;
	break;
    case (DBF_SHORT):
	 * (short *) pvalue = (short) number;
	if (number > 32767 || number < -32768)
	    err = 1;
	break;
    case (DBF_USHORT):
	 * (unsigned short *) pvalue = (unsigned short) number;
	if (number < 0 || number > 65535)
	    err = 1;
	break;
    case (DBF_ULONG):
        tmpnum.ulong_value = (unsigned long) number;
        * (unsigned long *) pvalue = tmpnum.ulong_value;
	break;
    case (DBF_LONG):
	  * (long *) pvalue = (long ) number;
	break;
    case (DBF_GBLCHOICE):
    case (DBF_CVTCHOICE):
    case (DBF_RECCHOICE):
    case (DBF_DEVCHOICE):
	*(unsigned short *) pvalue = (unsigned short) number;
	if (number < 0 || number > 65535)
	    err = 1;
	break;
    default:
        printf("bldRecDef: conv_strng default reached\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
    }
    if (err == 1) {
        printf("bldRecDef: conv_strng - field out of range\n");
        printf("file %s line number %d\n%s - RecType=%s\n",
        plineIn->fname, plineIn->line_no, plineIn->line,
        pInfo->ascii_name);
	return (-1);
    }
    return (0);
}

int write_tisPermFieldsInfo (FILE *out4_fp, char *ptName, 
			     struct fldDes *pfield, 
			     struct inLineInfo *pInfo)
{
    fprintf (out4_fp, "%-*.*s,%-*.*s,%d,%d,%-*.*s\n",
			POINTTYPE_SZ, POINTTYPE_SZ, ptName,
			FLDNAME_SZ, FLDNAME_SZ, pInfo -> fldname,
			pfield -> field_type,
			pfield -> special,
			PROMPT_SZ, PROMPT_SZ, pInfo -> prompt);
    return (0);
}
