/*caAddEvent.c,v 1.1.1.1 1999/08/03 14:41:58 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		ADD A MASKED ARRAY EVENT TO A CHANNEL		*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifdef WIN32
#include <sys/types.h>
#include	<winsock.h>
#else
#ifdef vxWorks
#include <vxWorks.h>
#endif vxWorks
#include <types.h>
#include <in.h>
#endif WIN32

#include "agentErrs.h"
#include "agentMsgs.h"
#include "cadef.h"
#include "dataTypes.h"
#include "db_access.h"		/* Original EPICS header */


extern int sendCAQ(short, char *, int);

int ca_add_masked_array_event(int type, int count,
			      chid chan,
			      void (*pFunc)(struct event_handler_args), void *pArg,
			      float f1, float f2, float f3,
			      evid *pEvid, int mask)
{
    short	msgType;
    CH_MONADD	monAdd;

    /*
    First, validate the chid
    */

    if (!chan)
	return (AGNT_BADCHID);

    /*
    Validate the data type and count
    */

    if (!dbr_type_is_valid(type))
	return (AGNT_BADTYPE);
    if ((chan -> buildType != AGNT_NOCONN) && (count > chan -> nativeCount))
	return (AGNT_BADCOUNT);
    if (chan -> buildType == AGNT_NOCONN)
	chan -> buildType = type;

    /*
    Get an EVID and fill it out.
    */

    *pEvid = newEVID();
    if (!(*pEvid))
	return (AGNT_ALLOCMEM);
    (*pEvid) -> userFunc = pFunc;
    (*pEvid) -> userArg = pArg;
    (*pEvid) -> pChan = chan;
    (*pEvid) -> type = type;
    (*pEvid) -> count = count;
    (*pEvid) -> float1 = (float) f1;
    (*pEvid) -> float2 = (float) f2;
    (*pEvid) -> float3 = (float) f3;
    (*pEvid) -> mask = mask;

    /*
    Chain it onto the chid.
    */

    (*pEvid) -> pNextEvnt = chan -> pFirstEvnt;
    if ((*pEvid) -> pNextEvnt)
	(*pEvid) -> pNextEvnt -> pPrevEvnt = *pEvid;
    chan -> pFirstEvnt = *pEvid;

    /*
    Form a message and send it to channel Acquisiton
    */

    msgType = htons(CHAN_MONADD);
#ifndef POINTER64
    monAdd.chHandle = (int) chan;
#else
    monAdd.chHandle = chan -> chidToken;
#endif
    monAdd.chType = htons(dbrXlate[type]);
    monAdd.chCount = htons((short)count);
    monAdd.mask = htons((short)mask);
    sendCAQ(msgType, (char *) &monAdd, sizeof(monAdd));

    return (AGNT_NORMAL);
}
