/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		FLUSH ALL BUFFERED OUTPUT			*/
/*								*/
/*	If there is any stuff in the transmit buffer, ship it	*/
/*	to channel acquisition and control. Our method of flow	*/
/*	control is to expand the transmit buffer whenever	*/
/*	possible to keep our application from hanging. When we	*/
/*	max out, then we assume (for now) that chanAcq is hung	*/
/*	and we should abandon ship with all hands on board.	*/
/*								*/
/*	Author:	Gordon Uchenick					*/
/*								*/
/****************************************************************/

#define FALSE 0
#ifdef vms
#include <selectvms.h>
#endif vms

#ifdef vxWorks
#include <vxWorks.h>
#include <string.h>
#include <logLib.h>
#else
#include "bcopy.h"
#endif vxWorks


#include <errno.h>
#ifdef WIN32
#include <sys/types.h>
#include	<winsock.h>
#else
#include <types.h>
#include <socket.h>
extern int errno;
#endif WIN32

#ifdef vxWorks
#include <sockLib.h>
#include <ioLib.h>
#endif vxWorks

#include "agentErrs.h"
#include "cadef.h"
#include "caContext.h"
extern int errno;

int ca_flush_io()
{
    int		bytesSent;
    CATX	*ctx;

    /*
    Get our context
    */

    ctx = getContext();
#ifdef vxWorks
    if (ctx -> importedContext)
    {
	ctx = ctx -> importedContext;
    }
#endif vxWorks

    /*
    See if there is a socket at all
    */

    if (ctx -> caSocketFd == 0)
    {
	ctx -> caOutputQueued = 0;
	return (AGNT_NORMAL);
    }
    while (ctx -> caOutputQueued)
    {
#ifdef vxWorks
	semTake(ctx -> sockMeSem, WAIT_FOREVER);
#endif vxWorks
	bytesSent = send(ctx -> caSocketFd, ctx -> caOutputBuff, ctx -> caOutputQueued, 0);
	if (bytesSent == -1)
	{
#ifdef WIN32
	    errno = WSAGetLastError();
	    if (errno == WSAEWOULDBLOCK)
	    {
#else
	    if (errno == EWOULDBLOCK)
	    {
#endif WIN32
		if (ctx -> caTxTopped)
		{
#ifdef WIN32
		    closesocket(ctx -> caSocketFd);
#else
#ifdef vxWorks
		    logMsg ("Closing Socket %d... EWOULDBLOCK = TRUE\n", ctx -> caSocketFd, 0, 0, 0, 0, 0);
#endif
		    close(ctx -> caSocketFd);
#endif WIN32
		    if (ctx -> regFunc)
		    {
			(*(ctx -> regFunc))(ctx -> regFuncArg, ctx -> caSocketFd, FALSE);
		    }
		    ctx -> caSocketFd = 0;
		    ctx -> caOutputQueued = 0;
#ifdef vxWorks
		    semGive(ctx -> sockMeSem);
#endif
		    return (AGNT_NORMAL);
		}
		ctx -> caTxBuff += ctx -> caOutputQueued;
		if (setsockopt(ctx -> caSocketFd, SOL_SOCKET, SO_SNDBUF, (char *)&(ctx -> caTxBuff), 
			sizeof(ctx -> caTxBuff)) == -1)
		{
		    ctx -> caTxTopped = 1;
		}
#ifdef vxWorks
		semGive(ctx -> sockMeSem);
#endif
		continue;
	    }
	    else
	    {
#ifdef WIN32
		closesocket(ctx -> caSocketFd);
#else
#ifdef vxWorks

		logMsg ("Closing Socket %d... EWOULDBLOCK = FALSE\n", ctx -> caSocketFd, 0, 0, 0, 0, 0);
#endif
		close(ctx -> caSocketFd);
#endif WIN32
		if (ctx -> regFunc)
		{
		    (*(ctx -> regFunc))(ctx -> regFuncArg, ctx -> caSocketFd, FALSE);
		}
		ctx -> caSocketFd = 0;
		ctx -> caOutputQueued = 0;
#ifdef vxWorks
		semGive(ctx -> sockMeSem);
#endif
		return (AGNT_NORMAL);
	    }
	}
	else if ((bytesSent != ctx -> caOutputQueued) && (ctx -> caOutputQueued))
	{
	    bcopy(ctx -> caOutputBuff + bytesSent, ctx -> caOutputBuff, (ctx -> caOutputQueued - bytesSent));
	}
        ctx -> caOutputQueued -= bytesSent;

	ctx -> caOutputQueued = 0;
#ifdef vxWorks
        semGive(ctx -> sockMeSem);
#endif
    }
    return (AGNT_NORMAL);
}
