/*caInit.c,v 1.1.1.1 1999/08/03 14:41:59 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		CHANNEL ACCESS INTIALIZATION			*/
/*								*/
/*	The user must call this routine before calling any	*/
/*	other channel access functions. All functions will	*/
/*	check that and call this routine if necessary.		*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifdef WIN32
#    include <sys/types.h>
#    include <winsock.h>
#    define caddr_t	void *
     extern int printf(char *, ...);
#else WIN32
#    ifdef vxWorks
#	include <string.h>
#	include <ioLib.h>
#	include <sockLib.h>
#	include <hostLib.h>
#	include <taskLib.h>
#	include <taskHookLib.h>
#	include <netinet/tcp.h>
#    else vxWorks
#	include <sys/types.h>
#	include <netdb.h>
#	include <sys/socket.h>
#	include <netinet/tcp.h>
#	include <sys/filio.h>
#	include <netinet/in.h>
#	if (defined(ultrix) || defined(AXP))
#	    include <ioctl.h>
#	else ultrix or AXP
#	    ifdef vms
#		include <ucx$inetdef.h>
#		include <selectvms.h>
#	    else vms
#	    endif vms
#	endif ultrix or AXP
#    endif vxWorks
#endif WIN32

#include <stdlib.h>

#include "cadef.h"
#include "agentPorts.h"
#include "agentErrs.h"
#include "caContext.h"

extern CATX	*firstContext;
static char	*defaultHost = "localhost";

#define TRUE	1
#ifdef vxWorks
static int	caHookAdded = 0;
#endif vxWorks

int ca_task_initialize()
{
    char		*caHostName;
#ifndef vxWorks
    struct hostent	*hostent;
#endif vxWorks
    struct sockaddr_in	portAddr;
    int			true = 1;
    int			intsize;
    CATX		*ctx;

#ifdef WIN32
	WSADATA wsaData; 
	WORD    wVersionRequested = MAKEWORD(1, 1);
	int	    err;

	err = WSAStartup(wVersionRequested, &wsaData); 
 
	if (err != 0) 
	    return 0; 
#endif WIN32

    /*
    Establish our context.
    */

    ctx = getContext();

#ifdef vxWorks
    if (!ctx)
	{

	/*
	Try to reuse a free context before we allocate
	a new one.
	*/

	taskLock();
	ctx = firstContext;
	while (ctx)
	    {
	    if (!(ctx -> taskId))
		{
		ctx -> taskId = taskIdSelf();
		break;
		}
	    ctx = ctx -> nextTask;
	    }
	taskUnlock();
	}
#endif vxWorks

    if (!ctx)
	{
        ctx = (CATX *) calloc(1, sizeof(CATX));
#ifdef vxWorks
	taskLock();
        ctx -> nextTask = firstContext;
        ctx -> taskId = taskIdSelf();
#endif
        firstContext = ctx;
#ifdef vxWorks
	taskUnlock();
	ctx -> sockMeSem = semMCreate(SEM_Q_FIFO | SEM_DELETE_SAFE);
#endif vxWorks
	}
    else
	{
#ifdef vxWorks
	ctx -> importedContext = 0;
	ctx -> sockMeSem = semMCreate(SEM_Q_FIFO | SEM_DELETE_SAFE);
#endif vxWorks
	ctx -> caOutputQueued = 0;
	ctx -> pFirstChid = 0;
	ctx -> regFunc = 0;
	ctx -> nchib = 0;
	ctx -> bufOutPos = 0;
	}



    /*
    The user can specify that we are to used Channel Acquisition and
    Control on a different host. That is done by defining the name
    CAHOST in the environment. If that name isn't defined, then just
    use "localhost".
    */

    caHostName = getenv("CAHOST");
    if (caHostName == NULL)
	caHostName = defaultHost;

    /*
    Establish a socket to Channel Acquisition and Control on the
    specified host.
    */

    ctx -> caSocketFd = socket(PF_INET, SOCK_STREAM, 0);
#ifdef WIN32
    if (ctx -> caSocketFd == INVALID_SOCKET)
    {
	printf ("Socket creation error %d\n", WSAGetLastError ());
	exit (1);
    }
#endif WIN32
    portAddr.sin_family = AF_INET;
#ifdef vxWorks
    portAddr.sin_addr.s_addr = hostGetByName(caHostName);
#else
    hostent = gethostbyname(caHostName);
#ifdef WIN32
    if (hostent == NULL)
    {
	printf ("Cannot find host %s\n", caHostName);
	exit (1);
    }
#endif WIN32
    portAddr.sin_addr.s_addr = ((struct in_addr *)hostent -> h_addr_list[0])->s_addr;
#endif
    portAddr.sin_port = htons(CAM_PORT);

    if (connect(ctx -> caSocketFd, (struct sockaddr *) &portAddr, sizeof(portAddr)) == -1)
	{
#ifdef WIN32
	closesocket(ctx -> caSocketFd);
#else
	close(ctx -> caSocketFd);
#endif WIN32
	return (AGNT_ALLOCMEM);
	}

    /*
    Set up the parameters that we need to implement some kind of flow control.
    */
    
    setsockopt(ctx -> caSocketFd, IPPROTO_TCP, TCP_NODELAY, (char *)&true, sizeof(true));
    setsockopt(ctx -> caSocketFd, SOL_SOCKET, SO_LINGER, (caddr_t) &true, sizeof(true));
    intsize = sizeof(ctx -> caRxBuff);
    getsockopt(ctx -> caSocketFd, SOL_SOCKET, SO_RCVBUF, (char *)&(ctx -> caRxBuff), &intsize);
    intsize = sizeof(ctx -> caTxBuff);
    getsockopt(ctx -> caSocketFd, SOL_SOCKET, SO_SNDBUF, (char *)&(ctx -> caTxBuff), &intsize);
    ctx -> caRxTopped = 0;
    ctx -> caTxTopped = 0;
#ifdef WIN32
    ioctlsocket (ctx -> caSocketFd, FIONBIO, &true);
#else
    ioctl(ctx -> caSocketFd, FIONBIO, (int)&true);
#endif WIN32
    FD_ZERO(&(ctx -> caSocketMask));
    FD_SET(ctx -> caSocketFd, &(ctx -> caSocketMask));

    /*
    Put in our exit hook. This is operating system dependent.
    */

#if (defined (AXP) || defined(ultrix) || defined(SOLARIS) || defined(WIN32))

    /*
    Ultrix, on the DECStation 5000 series. And SysV!
    */

    atexit(ca_task_exit);
#ifdef SOLARIS
    true = 4096;
    while (setsockopt(ctx -> caSocketFd, SOL_SOCKET, SO_RCVBUF, 
		      (const char *)&true, sizeof(true)) != -1)
	{
	intsize = true;
	true += 1024;
	}
    ctx -> caRxBuff = intsize;
    ctx -> caRxTopped = 1;
#endif

#else ultrix
#ifdef vxWorks

    /*
    vxWorks, on the Motorola 680x0 family
    */

    taskLock();
    if (!caHookAdded)
	{
        taskDeleteHookAdd((FUNCPTR)ca_task_exit);
	caHookAdded = 1;
	}
    taskUnlock();

#else vxWorks
#ifndef vms

    /*
    Sun OS.
    */

    on_exit(ca_task_exit, 0);

#else vms

    /*
    Compiler brain damage on the Vax requires an extra init for
    a data translation array. See dataTypes.c
    */

    initXlate();

#endif vms
#endif vxWorks
#endif ultrix

    ctx -> pFirstChid = 0;
    ctx -> caInitialized = 1;

    /*
    Call the user's registration function if he has one.
    */

    if (ctx -> regFunc)
	(*(ctx -> regFunc))(ctx -> regFuncArg, ctx -> caSocketFd, TRUE);

    return (AGNT_NORMAL);
}
