/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		REGISTER AS TREND OR ALARM HANDLERS		*/
/*								*/
/*	These two routines register the caller as a trend	*/
/*	or an alarm handler. Although unlikely, one program	*/
/*	can have both functions. There may also be more than	*/
/*	one handler of either type on any workstation. Once	*/
/*	you have registered as one of these types of handlers,	*/
/*	you can't "unregister." The only way out is to termi-	*/
/*	nate your program.					*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifdef WIN32
#include <sys/types.h>
#include	<winsock.h>
extern int sendCAQ(short, char *, int);
#else
#include <types.h>
#include <in.h>
#endif WIN32

#include "agentErrs.h"
#include "agentMsgs.h"
#ifdef vms
#include <selectvms.h>
#endif
#include "caContext.h"
#include "tsDefs.h"


/****************************************************************/
/*								*/
/*		REGISTER AS A TREND HANDLER			*/
/*								*/
/****************************************************************/

ca_register_as_archiver(void (*pDataFunc)(), void (*pConnFunc)())
{
    short		msgType;
    CATX		*ctx;
    CH_HANDLERS		handlerMsg;

    ctx = getContext();

    /*
    Post the address of the data function and the connect function.
    */

    ctx -> trendEventFunc = pDataFunc;
    ctx -> trendConnFunc = pConnFunc;

    /*
    Send a message to chanAcq. These messages are "additive" so
    set the status of the alarm handler to zero even if the user
    has already registered as an alarm handler.
    */

    handlerMsg.chTrendHandler = ntohs(1);
    handlerMsg.chAlarmHandler = 0;
    msgType = htons(CHAN_HANDLERS);
    sendCAQ(msgType, (char *) &handlerMsg, sizeof(handlerMsg));

    return (AGNT_NORMAL);
}


/****************************************************************/
/*								*/
/*		REGISTER AS A ALARM HANDLER			*/
/*								*/
/****************************************************************/

ca_register_as_alarm_handler(void (*pDataFunc)(tisAlarm *), 
			     void (*pConnFunc)(char *, short))
{
    short		msgType;
    CATX		*ctx;
    CH_HANDLERS		handlerMsg;

    ctx = getContext();

    /*
    Post the address of the data function and the connect function.
    */

    ctx -> alarmEventFunc = pDataFunc;
    ctx -> alarmConnFunc = (void (*)())pConnFunc;

    /*
    Send a message to chanAcq. These messages are "additive" so
    set the status of the alarm handler to zero even if the user
    has already registered as an alarm handler.
    */

    handlerMsg.chTrendHandler = 0;
    handlerMsg.chAlarmHandler = ntohs(1);
    msgType = htons(CHAN_HANDLERS);
    sendCAQ(msgType, (char *) &handlerMsg, sizeof(handlerMsg));

    return (AGNT_NORMAL);
}
