/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	CONVERT AGENT DATA STREAM TO PARAMETER STRUCTURES	*/
/*								*/
/*	This routine converts a free format data stream	into	*/
/*	the channel parameter structures that are defined in	*/
/*	paramStructs.h.						*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifndef AXP
#include <bcopy.h>
#endif

#ifdef WIN32
#include <sys/types.h>
#include <winsock.h>
#include <stdio.h>
#else
#include <types.h>
#include <in.h>
#endif WIN32

#ifdef AXP
#include <string.h>
#endif

#include "dbDefs.h"
#include "paramStructs.h"
#include "tisByteOrder.h"

static char* convTCP();
static void  convCmn();

caConvParamData(agentBuff, type, dbrBuff)
char	*agentBuff;	/* Channel Access stream */
short	type;		/* DBR_xxx_PARAM */
char	*dbrBuff;	/* Caller's buffer */
{
    short	dbrTempShort;

    if (type == DBR_SHORT_PARAM)
	{
	struct dbr_short_param	*shParam = (struct dbr_short_param *) dbrBuff;

	bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
	agentBuff += sizeof(short);
	shParam -> hopr = ntohs(dbrTempShort);
	bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
	agentBuff += sizeof(short);
	shParam -> lopr = ntohs(dbrTempShort);
	bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
	agentBuff += sizeof(short);
	shParam -> drvh = ntohs(dbrTempShort);
	bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
	agentBuff += sizeof(short);
	shParam -> drvl = ntohs(dbrTempShort);
	bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
	agentBuff += sizeof(short);
	agentBuff = convTCP(agentBuff, &(shParam -> nativeType), &(shParam -> nativeCount),
		&(shParam -> prec));
	convCmn(agentBuff, shParam -> chanName, shParam -> units, shParam -> resName,
		shParam -> desc);
	}

    else if (type == DBR_ENUM_PARAM)
	{
	struct dbr_enum_param	*enParam = (struct dbr_enum_param *) dbrBuff;
	short			i;
	short			j;

	bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
	agentBuff += sizeof(short);
	enParam -> no_str = ntohs(dbrTempShort);
	for (i = 0; i < enParam -> no_str; i++)
	    {
	    j = 0;
	    while (enParam -> strs[i][j++] = *agentBuff++);
	    }
	convCmn(agentBuff, enParam -> chanName, enParam -> units, enParam -> resName,
		enParam -> desc);
	}

    else if (type == DBR_STRING_PARAM)
	{
	struct dbr_string_param	*stParam = (struct dbr_string_param *) dbrBuff;

	/*
	Nothing to do but the common stuff.
	*/

	convCmn(agentBuff, stParam -> chanName, stParam -> units, stParam -> resName,
		stParam -> desc);
	}

    else if (type == DBR_FLOAT_PARAM)
	{
	struct dbr_float_param	*flParam = (struct dbr_float_param *) dbrBuff;
	float	dbrTempFloat;

	bcopy(agentBuff, (char *) &dbrTempFloat, sizeof(dbrTempFloat));
	agentBuff += sizeof(float);
	ntohf(dbrTempFloat, (flParam -> hopr));
	bcopy(agentBuff, (char *) &dbrTempFloat, sizeof(dbrTempFloat));
	agentBuff += sizeof(float);
	ntohf(dbrTempFloat, (flParam -> lopr));
	bcopy(agentBuff, (char *) &dbrTempFloat, sizeof(dbrTempFloat));
	agentBuff += sizeof(float);
	ntohf(dbrTempFloat, (flParam -> drvh));
	bcopy(agentBuff, (char *) &dbrTempFloat, sizeof(dbrTempFloat));
	agentBuff += sizeof(float);
	ntohf(dbrTempFloat, (flParam -> drvl));
	agentBuff = convTCP(agentBuff, &(flParam -> nativeType), &(flParam -> nativeCount),
		&(flParam -> prec));
	convCmn(agentBuff, flParam -> chanName, flParam -> units, flParam -> resName,
		flParam -> desc);
	}

    else if (type == DBR_LONG_PARAM)
	{
	struct dbr_long_param	*loParam = (struct dbr_long_param *) dbrBuff;
	int	dbrTempLong;

	bcopy(agentBuff, (char *) &dbrTempLong, sizeof(dbrTempLong));
	agentBuff += sizeof(int);
	loParam -> hopr = ntohl(dbrTempLong);
	bcopy(agentBuff, (char *) &dbrTempLong, sizeof(dbrTempLong));
	agentBuff += sizeof(int);
	loParam -> lopr = ntohl(dbrTempLong);
	bcopy(agentBuff, (char *) &dbrTempLong, sizeof(dbrTempLong));
	agentBuff += sizeof(int);
	loParam -> drvh = ntohl(dbrTempLong);
	bcopy(agentBuff, (char *) &dbrTempLong, sizeof(dbrTempLong));
	agentBuff += sizeof(int);
	agentBuff = convTCP(agentBuff, &(loParam -> nativeType), &(loParam -> nativeCount),
		&(loParam -> prec));
	convCmn(agentBuff, loParam -> chanName, loParam -> units, loParam -> resName,
		loParam -> desc);
	}

    else if (type == DBR_DOUBLE_PARAM)
	{
	struct dbr_double_param	*dbParam = (struct dbr_double_param *) dbrBuff;
	double	dbrTempDouble;

	bcopy(agentBuff, (char *) &dbrTempDouble, sizeof(dbrTempDouble));
	agentBuff += sizeof(double);
	ntohd(dbrTempDouble, (dbParam -> hopr));
	bcopy(agentBuff, (char *) &dbrTempDouble, sizeof(dbrTempDouble));
	agentBuff += sizeof(double);
	ntohd(dbrTempDouble, (dbParam -> lopr));
	bcopy(agentBuff, (char *) &dbrTempDouble, sizeof(dbrTempDouble));
	agentBuff += sizeof(double);
	ntohd(dbrTempDouble, (dbParam -> drvh));
	bcopy(agentBuff, (char *) &dbrTempDouble, sizeof(dbrTempDouble));
	agentBuff += sizeof(double);
	ntohd(dbrTempDouble, (dbParam -> drvl));
	agentBuff = convTCP(agentBuff, &(dbParam -> nativeType), &(dbParam -> nativeCount),
		&(dbParam -> prec));
	convCmn(agentBuff, dbParam -> chanName, dbParam -> units, dbParam -> resName,
		dbParam -> desc);
	}

    else
	{
	printf("Invalid type given to caConvParamData\n");
	return 0;
	}
    return 0;
}

/****************************************************************/
/*								*/
/*	CONVERT NATIVE TYPE, COUNT AND PRECISION		*/
/*								*/
/*	Almost all of the structures have the channel's native	*/
/*	type, count, and precision as shorts. This is just	*/
/*	some code that can be used in converting a number of	*/
/*	types.							*/
/*								*/
/****************************************************************/

static char* convTCP(agentBuff, nativeType, nativeCount, prec)
char	*agentBuff;
short	*nativeType;
short	*nativeCount;
short	*prec;
{
    short	dbrTempShort;

    bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
    agentBuff += sizeof(short);
    *nativeType = ntohs(dbrTempShort);
    bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
    agentBuff += sizeof(short);
    *nativeCount = ntohs(dbrTempShort);
    bcopy(agentBuff, (char *) &dbrTempShort, sizeof(dbrTempShort));
    agentBuff += sizeof(short);
    *prec = ntohs(dbrTempShort);
    return (agentBuff);
}

/****************************************************************/
/*								*/
/*		CONVERT FIELDS COMMON TO ALL TYPES		*/
/*								*/
/*	Every parameter type has the channel name, the units,	*/
/*	the resource name, and the descriptor. This function	*/
/*	just avoids duplicating a lot of code.			*/
/*								*/
/*	Note that we don't check the length of the strings.	*/
/*	This was done for the units parameter when the data	*/
/*	stream was created, in rsrvTask.c. The other fields	*/
/*	are constrained in size by the definitions in the	*/
/*	sdr and then in dbCommon.h.				*/
/*								*/
/****************************************************************/

static void  convCmn(agentBuff, chanName, units, resName, desc)
char	*agentBuff;
char	*chanName;
char	*units;
char	*resName;
char	*desc;
{
    short	strLen;

    /*
    Get the length of the channel name, adding 1 for the null
    */

    strLen = strlen(agentBuff) + 1;
    bcopy(agentBuff, chanName, strLen);
    agentBuff += strLen;

    /*
    Get the length of the units, also adding 1 for the null
    */

    strLen = strlen(agentBuff) + 1;
    bcopy(agentBuff, units, strLen);
    agentBuff += strLen;

    /*
    Get the length of the resource name, adding 1 ....
    */

    strLen = strlen(agentBuff) + 1;
    bcopy(agentBuff, resName, strLen);
    agentBuff += strLen;

    /*
    Get the length of the descriptor, adding 1 ...
    */

    strLen = strlen(agentBuff) + 1;
    bcopy(agentBuff, desc, strLen);
}

