/*dumpBlocks.c,v 1.1.1.1 1999/08/03 14:42:02 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		DUMP THE CHANNEL CONTROL BLOCKS			*/
/*								*/
/*	This is a set of routines to dump out the contents	*/
/*	of the basic blocks that are internal to Channel	*/
/*	Acquisition and Control.				*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifdef WIN32
#include <winsock.h>
#endif WIN32
#ifdef vxWorks
#include <vxWorks.h>
#include <sockLib.h>
#include <inetLib.h>
#endif vxWorks

#include <stdio.h>
#if !defined(vxWorks) && !defined(WIN32)
#include <sys/types.h>
#include <netinet/in.h>
#endif !defined vxWorks

#include "inet_ntoa.h"
#include "cadef.h"
#include "channelControl.h"
#include "dumpBlocks.h"


/******
	Dump an IOC Status Block
******/

void dumpISB(fd, pISB)
FILE	*fd;
ISB	*pISB;
{
#ifndef POINTER64
    fprintf(fd, "ISB at %X:\n", (int) pISB);
    fprintf(fd, "\tInet: %s\tpFirstCHB: %X\tnextISB: %X\tconnected: %d splicable: %d\n",
	INET_NTOA(pISB -> Inet), (int) (pISB -> pFirstCHB), (int) (pISB -> nextISB), pISB -> connected,
		pISB -> splicable);
#else
#endif
}


/******
	Dump a Channel Header Block
******/

void dumpCHB(fd, pCHB)
FILE	*fd;
CHB	*pCHB;
{
#ifndef POINTER64
    fprintf(fd, "CHB at %X:\n", (int) pCHB);
    fprintf(fd, "\tchanName: %s\tpISB1: %X\tpISB2: %X\tpCurrISB: %X\n",
	pCHB -> chanName, (int) (pCHB -> pISB1), (int) (pCHB -> pISB2), (int)(pCHB -> pCurrISB));
    fprintf(fd, "\thandleIOC1: %X\thandleIOC2: %X\tcurrType: %X\tcurrMask: %X\n",
	pCHB -> handleIOC1, pCHB -> handleIOC2, pCHB -> currType, pCHB-> currMask);
    fprintf(fd, "\tnativeType: %d\tnativeCount: %d\tcurrCount: %d\n",
	pCHB -> nativeType, pCHB -> nativeCount, pCHB -> currCount);
    fprintf(fd, "\tpFirstCAB: %X\tpNextIOC1: %X\tpPrevIOC1: %X\tpNextIOC2: %X\tpPrevIOC2: %X\n",
	(int) (pCHB -> pFirstCAB), (int) (pCHB -> pNextIOC1), (int) (pCHB -> pPrevIOC1),
	(int) (pCHB -> pNextIOC2), (int) (pCHB -> pPrevIOC2));
    fprintf(fd, "\tpNextRes: %X\tpPrevRes: %X\tpNextHash: %X\tpHash: %X\n",
	(int) (pCHB -> pNextRes), (int) (pCHB -> pPrevRes), (int) (pCHB -> pNextHash), (int) (pCHB -> pHash));
#else
#endif
}


/******
	Dump a Channel Application Block
******/

void dumpCAB(fd, pCAB)
FILE	*fd;
CAB	*pCAB;
{
#ifndef POINTER64
    fprintf(fd, "CAB at %X:\n", (int) pCAB);
    fprintf(fd, "\tpCHB: %X\tpACB: %X\thandle: %X\tmask: %X\tdataType: %X\tcount: %d\n",
	(int) (pCAB -> pCHB), (int) (pCAB -> pACB), pCAB -> handle, pCAB -> mask, pCAB -> dataType, pCAB -> count);
    fprintf(fd, "\treadType: %X\tawaitAck: %d\tpNextChan: %X\tpPrevApp: %X\tpNextApp: %X\n",
	pCAB -> readType, pCAB -> awaitAck, (int) (pCAB -> pNextChan), (int) (pCAB -> pPrevApp),
	(int) (pCAB -> pNextApp));
#else
    fprintf(fd, "CAB at %lX:\n", pCAB);
    fprintf(fd, "\tpCHB: %lX\tpACB: %lX\thandle: %X\tmask: %X\tdataType: %X\tcount: %d\n",
	(pCAB -> pCHB), (pCAB -> pACB), pCAB -> handle, pCAB -> mask, pCAB -> dataType, pCAB -> count);
    fprintf(fd, "\treadType: %X\tawaitAck: %d\tpNextChan: %lX\tpPrevApp: %X\tpNextApp: %lX\n",
	pCAB -> readType, pCAB -> awaitAck, (pCAB -> pNextChan), (pCAB -> pPrevApp),
	(pCAB -> pNextApp));
#endif
}


/******
	Dump a Resource Control Block
******/

void dumpRCB(fd, pRCB)
FILE	*fd;
RCB	*pRCB;
{
#ifndef POINTER64
    fprintf(fd, "RCB at %X:\n", (int) pRCB);
    fprintf(fd, "\tresName: %s\tpFirstCHB: %X\tpNextRCB: %X\n",
	pRCB -> resName, (int) (pRCB -> pFirstCHB), (int) (pRCB -> pNextRCB));
#else
    fprintf(fd, "RCB at %lX:\n", pRCB);
    fprintf(fd, "\tresName: %s\tpFirstCHB: %lX\tpNextRCB: %lX\n",
	pRCB -> resName, (pRCB -> pFirstCHB), (pRCB -> pNextRCB));
#endif
}


/******
	Dump an Application Control Block
******/

void dumpACB(fd, pACB)
FILE	*fd;
ACB	*pACB;
{
#ifndef POINTER64
    fprintf(fd, "ACB at %X\n", (int) pACB);
    fprintf(fd, "\tsockFd: %d\tpFirstCAB: %X\tpNextACB: %X\n",
	pACB -> sockFd, (int) (pACB -> pFirstCAB), (int) (pACB -> pNextACB));
#else
    fprintf(fd, "ACB at %lX\n", pACB);
    fprintf(fd, "\tsockFd: %d\tpFirstCAB: %lX\tpNextACB: %lX\n",
	pACB -> sockFd, (pACB -> pFirstCAB), (pACB -> pNextACB));
#endif
}


/******
	Dump a Hash Table block
******/

void dumpHTB(fd, pHTB)
FILE	*fd;
HTB	*pHTB;
{
#ifndef POINTER64
    fprintf(fd, "HTB at %X:\n", (int) pHTB);
    fprintf(fd, "\tpFirstCHB: %X\n", (int) (pHTB -> pFirstCHB));
#else
    fprintf(fd, "HTB at %lX:\n", pHTB);
    fprintf(fd, "\tpFirstCHB: %lX\n", (pHTB -> pFirstCHB));
#endif
}
