/*memMgt.c,v 1.1.1.1 1999/08/03 14:42:03 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	MEMORY MANAGEMENT FOR CHANNEL ACCESS LIBRARY		*/
/*								*/
/*	This routine is very similar to buffPoolMgt.c which	*/
/*	is part of Channel Acquisition and Maintenance. These	*/
/*	routines manage tables of chid and evid structures	*/
/*	as defined in <cadef.h>. We manage the memory by	*/
/*	ourselves to minimize the number of trips that the	*/
/*	the user takes through the operating system.		*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/


#ifdef vxWorks
#include <vxWorks.h>
#include <taskLib.h>
#include <memLib.h>
#include <string.h>
#endif vxWorks

#if defined(AXP) || defined(WIN32)
#include <stdlib.h>
#endif AXP

#if defined(WIN32)
#include <memory.h>
#endif WIN32

#include "cadef.h"

/****************************************************************/
/*								*/
/*			TUNING PARAMETERS			*/
/*								*/
/*	Whenever we allocate structures from the operating	*/
/*	system, we allocate a bunch of them. These variables	*/
/*	control the size of the chunks.				*/
/*								*/
/****************************************************************/

#define CHID_ALLOC	20
#define	EVID_ALLOC	20

/******
	Pointers to the internal memory pools. Make sure that this
	data does not NOT NOT! get shared between users if you
	every make a shared library version of channel access,
	especially under VMS!
******/

static chid	chidPool = 0;
static evid	evidPool = 0;

/****************************************************************/
/*								*/
/*		ALLOCATE A CHANNEL BLOCK			*/
/*								*/
/****************************************************************/

chid newCHID(void)
{
    chid	currChid;
    int	index;
#ifdef POINTER64
    int		tokenVal;
#endif

    /*
    If the pool is empty, fill it up again.
    */

#ifdef vxWorks
    taskLock();
#endif

    if (!chidPool)
	{
	chidPool = (chid) malloc(CHID_ALLOC * sizeof(struct chanBlock));
	if (!chidPool)
	    return (0);
	currChid = chidPool;
	index = CHID_ALLOC;
	while (index--)
	    {
	    if (index)
		currChid -> pNextChan = (struct chanBlock *)(currChid + 1);
	    else
		currChid -> pNextChan = 0;
	    currChid++;
	    }
	}

    /*
    Take the one off the front of the pool and clear it.
    */

#ifdef POINTER64
    if ((tokenVal = TokenFromPointer64 (chidPool)) == 0)
	return 0;
#endif
    currChid = chidPool;
    chidPool = (chid) (currChid -> pNextChan);

#ifdef vxWorks
    taskUnlock();
#endif

    memset((void *) currChid, 0, sizeof(struct chanBlock));
#ifdef POINTER64
    currChid->chidToken = tokenVal;
#endif
    return (currChid);
}


/****************************************************************/
/*								*/
/*		FREE A CHANNEL BLOCK				*/
/*								*/
/****************************************************************/

void deleteCHID(oldChid)
chid	oldChid;
{

    /*
    If this is a local channel, get rid of the struct dbAddr
    that is referenced through the pDbAddr member.
    */

#ifdef vxWorks
    if (oldChid -> pDbAddr)
	free(oldChid -> pDbAddr);
#endif vxWorks

    /*
    Push this one onto the front of the pool
    */

#ifdef vxWorks
    taskLock();
#endif

    oldChid -> pNextChan = (struct chanBlock *) chidPool;
    chidPool = oldChid;

#ifdef vxWorks
    taskUnlock();
#endif

}


/****************************************************************/
/*								*/
/*		ALLOCATE AN EVENT BLOCK				*/
/*								*/
/****************************************************************/

evid newEVID(void)
{
    evid	currEvid;
    int	index;

    /*
    If the pool is empty, fill it up again.
    */

#ifdef vxWorks
    taskLock();
#endif

    if (!evidPool)
	{
	evidPool = (evid) malloc(CHID_ALLOC * sizeof(struct evntBlock));
	if (!evidPool)
	    return (0);
	currEvid = evidPool;
	index = CHID_ALLOC;
	while (index--)
	    {
	    if (index)
		currEvid -> pNextEvnt = (struct evntBlock *)(currEvid + 1);
	    else
		currEvid -> pNextEvnt = 0;
	    currEvid++;
	    }
	}

    /*
    Take the one off the front of the pool and clear it.
    */

    currEvid = evidPool;
    evidPool = (evid) (currEvid -> pNextEvnt);
#ifdef vxWorks
    taskUnlock();
#endif

    memset((char *) currEvid, 0, sizeof(struct evntBlock));
    return (currEvid);
}


/****************************************************************/
/*								*/
/*		FREE A CHANNEL BLOCK				*/
/*								*/
/****************************************************************/

void deleteEVID(oldEvid)
evid	oldEvid;
{

    /*
    Push this one onto the front of the pool
    */

#ifdef vxWorks
    taskLock();
#endif

    oldEvid -> pNextEvnt = (struct evntBlock *) evidPool;
    evidPool = oldEvid;

#ifdef vxWorks
    taskUnlock();
#endif
}
