#include	<stdio.h>
#ifndef AXP
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32
#include	"ao.h"
#include	"do.h"
#include	"mbbo.h"

static CODE4		cb;
static char		pathbuff[128];
static char		sysbuff[256];
static char		*old_rec, *new_rec;
static struct stat	statbuff;

static int create_subdir (void);
static int convert_ao (void);
static int convert_bo (void);
static int convert_mbbo (void);

#define	TIS4000DB	"/tis4000db"
#define	DBCNVRT		"rbcnvrt"


int main()
{
    fprintf (stdout, "starting RB conversion\n\n");
    d4init (&cb);

#if defined(WIN32) || defined(USECB63)
    cb.errGo = 0;
    cb.errOpen = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
#else
    cb.go_error = 0;
    cb.open_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
#endif WIN32 or USECB63

    if (create_subdir ())
	return (1);
    if (convert_ao ())
	return (1);
    if (convert_bo ())
	return (1);
    if (convert_mbbo ())
	return (1);
    d4close_all (&cb);
    fprintf (stdout, "\nRB conversion finished\n");
    return (0);
}


static int create_subdir (void)
{
    sprintf (pathbuff, "%s/%s", TIS4000DB, DBCNVRT);
    if (stat (pathbuff, &statbuff))
    {
	if (errno == ENOENT)
	{
	    sprintf (sysbuff, "mkdir %s", pathbuff);
	    system (sysbuff);
	    if (stat (pathbuff, &statbuff))
	    {
		fprintf (stdout, "rbconv err: failed to create %s\n", pathbuff);
		return (1);
	    }
	    else
	    {
		fprintf (stdout, "rbconv: %s created\n", pathbuff);
		return (0);
	    }
	}
	else
	{
	    fprintf (stdout, "rbconv err: stat problem (%d) on %s\n", errno, pathbuff);
	    return (1);
	}
    }
    else
    {
	fprintf (stdout, "rbconv err: %s already exists\n", pathbuff);
    }
    return (0);
}


static int convert_ao (void)
{
    DATA4	*old_ao, *new_ao;
    FIELD4	*readback;

    fprintf (stdout, "...converting ao.dbf\n");

    sprintf (pathbuff, "%s/ao", TIS4000DB);
    if (!(old_ao = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "rbconv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((readback = d4field (old_ao, "READBACK")) != 0)
    {
	fprintf (stdout, "rbconv err: %s already converted\n", pathbuff);
	d4close (old_ao);
	return (1);
    }
    d4alias_set (old_ao, "old_ao");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/ao.dbf %s/%s/ao.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/ao.mdx %s/%s/ao.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ao", TIS4000DB,DBCNVRT);
    if (!(new_ao = d4create (&cb, pathbuff, aoFieldInfo, aoTagInfo)))
    {
	fprintf (stdout, "rbconv err: failed to create %s\n", pathbuff);
	d4close (old_ao);
	return (1);
    }

    old_rec = d4record (old_ao);
    new_rec = d4record (new_ao);

    if ((readback = d4field (new_ao, "READBACK")) == 0)
    {
	fprintf (stdout, "rbconv err: READBACK not exist in ao.dbf\n");
	d4close (old_ao);
	d4close (new_ao);
	return (1);
    }

    for (d4top (old_ao); !d4eof (old_ao); d4skip (old_ao, 1))
    {
	if (d4deleted (old_ao))
	    continue;
	d4append_start (new_ao, 0);
	d4blank (new_ao);
	memcpy (new_rec, old_rec, d4record_width (old_ao));
	f4assign_int (readback, 0);
	d4append (new_ao);
	d4flush (new_ao);
    }

    d4close (old_ao);
    d4close (new_ao);

    fprintf (stdout, "...finished ao.dbf\n");

    return (0);
}


static int convert_bo (void)
{
    DATA4	*old_bo, *new_bo;
    FIELD4	*readback;

    fprintf (stdout, "...converting do.dbf\n");

    sprintf (pathbuff, "%s/do", TIS4000DB);
    if (!(old_bo = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "rbconv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((readback = d4field (old_bo, "READBACK")) != 0)
    {
	fprintf (stdout, "rbconv err: %s already converted\n", pathbuff);
	d4close (old_bo);
	return (1);
    }
    d4alias_set (old_bo, "old_bo");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/do.dbf %s/%s/do.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/do.mdx %s/%s/do.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/do", TIS4000DB,DBCNVRT);
    if (!(new_bo = d4create (&cb, pathbuff, doFieldInfo, doTagInfo)))
    {
	fprintf (stdout, "rbconv err: failed to create %s\n", pathbuff);
	d4close (old_bo);
	return (1);
    }

    old_rec = d4record (old_bo);
    new_rec = d4record (new_bo);

    if ((readback = d4field (new_bo, "READBACK")) == 0)
    {
	fprintf (stdout, "rbconv err: READBACK not exist in do.dbf\n");
	d4close (old_bo);
	d4close (new_bo);
	return (1);
    }

    for (d4top (old_bo); !d4eof (old_bo); d4skip (old_bo, 1))
    {
	if (d4deleted (old_bo))
	    continue;
	d4append_start (new_bo, 0);
	d4blank (new_bo);
	memcpy (new_rec, old_rec, d4record_width (old_bo));
	f4assign_int (readback, 0);
	d4append (new_bo);
	d4flush (new_bo);
    }

    d4close (old_bo);
    d4close (new_bo);

    fprintf (stdout, "...finished do.dbf\n");

    return (0);
}


static int convert_mbbo (void)
{
    DATA4	*old_mbbo, *new_mbbo;
    FIELD4	*readback;

    fprintf (stdout, "...converting mbbo.dbf\n");

    sprintf (pathbuff, "%s/mbbo", TIS4000DB);
    if (!(old_mbbo = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "rbconv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((readback = d4field (old_mbbo, "READBACK")) != 0)
    {
	fprintf (stdout, "rbconv err: %s already converted\n", pathbuff);
	d4close (old_mbbo);
	return (1);
    }
    d4alias_set (old_mbbo, "old_mbbo");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/mbbo.dbf %s/%s/mbbo.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/mbbo.mdx %s/%s/mbbo.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/mbbo", TIS4000DB,DBCNVRT);
    if (!(new_mbbo = d4create (&cb, pathbuff, mbboFieldInfo, mbboTagInfo)))
    {
	fprintf (stdout, "rbconv err: failed to create %s\n", pathbuff);
	d4close (old_mbbo);
	return (1);
    }

    old_rec = d4record (old_mbbo);
    new_rec = d4record (new_mbbo);

    if ((readback = d4field (new_mbbo, "READBACK")) == 0)
    {
	fprintf (stdout, "rbconv err: READBACK not exist in mbbo.dbf\n");
	d4close (old_mbbo);
	d4close (new_mbbo);
	return (1);
    }

    for (d4top (old_mbbo); !d4eof (old_mbbo); d4skip (old_mbbo, 1))
    {
	if (d4deleted (old_mbbo))
	    continue;
	d4append_start (new_mbbo, 0);
	d4blank (new_mbbo);
	memcpy (new_rec, old_rec, d4record_width (old_mbbo));
	f4assign_int (readback, 0);
	d4append (new_mbbo);
	d4flush (new_mbbo);
    }

    d4close (old_mbbo);
    d4close (new_mbbo);

    fprintf (stdout, "...finished mbbo.dbf\n");

    return (0);
}

