/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include	<stdio.h>
#ifndef AXP
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include	"strngin.h"
#include	"strngout.h"

static CODE4		cb;
static char		pathbuff[128];
static char		sysbuff[256];
static char		*old_rec, *new_rec;
static struct stat	statbuff;

static int create_subdir (void);
static int convert_strngin (void);
static int convert_strngout (void);

#define	TIS4000DB	"/tis4000db"
#define	DBCNVRT		"strngcnvrt"


int main()
{
    fprintf (stdout, "starting string conversion\n\n");
    d4init (&cb);

#if defined(WIN32) || defined(USECB63)
    cb.errGo = 0;
    cb.errOpen = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
#else
    cb.go_error = 0;
    cb.open_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
#endif WIN32 or USECB63

    if (create_subdir ())
	return (1);
    if (convert_strngin ())
	return (1);
    if (convert_strngout ())
	return (1);
    d4close_all (&cb);
    fprintf (stdout, "\nstring conversion finished\n");
    return (0);
}


static int create_subdir (void)
{
    sprintf (pathbuff, "%s/%s", TIS4000DB, DBCNVRT);
    if (stat (pathbuff, &statbuff))
    {
	if (errno == ENOENT)
	{
	    sprintf (sysbuff, "mkdir %s", pathbuff);
	    system (sysbuff);
	    if (stat (pathbuff, &statbuff))
	    {
		fprintf (stdout, "strngconv err: failed to create %s\n", pathbuff);
		return (1);
	    }
	    else
	    {
		fprintf (stdout, "strngconv: %s created\n", pathbuff);
		return (0);
	    }
	}
	else
	{
	    fprintf (stdout, "strngconv err: stat problem (%d) on %s\n", errno, pathbuff);
	    return (1);
	}
    }
    else
    {
	fprintf (stdout, "strngconv err: %s already exists\n", pathbuff);
    }
    return (0);
}


static int convert_strngin (void)
{
    DATA4	*old_strngin, *new_strngin;
    FIELD4	*oldname, *newname;
    FIELD4	*oldstring, *newstring;
    FIELD4	*oldbdtype, *newbdtype;
    FIELD4	*oldpi, *newpi;
    FIELD4	*oldref, *newref;
    FIELD4	*oldms, *newms;
    FIELD4	*oldconst, *newconst;
    FIELD4	*oldauto, *newauto;
    FIELD4	*oldrdsz, *newrdsz;
    FIELD4	*oldchgsv, *newchgsv;
    int		fieldLen;

    fprintf (stdout, "...converting strngin.dbf\n");

    sprintf (pathbuff, "%s/strngin", TIS4000DB);
    if (!(old_strngin = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "strnginconv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((oldstring = d4field (old_strngin, "CUR_STRING")) == 0)
    {
	fprintf (stdout, "strnginconv err: CUR_STRING not found\n", pathbuff);
	d4close (old_strngin);
	return (1);
    }
    fieldLen  = f4len (oldstring);
    if (fieldLen == 80)
    {
	fprintf (stdout, "strnginconv err: stringin already converted\n", pathbuff);
	d4close (old_strngin);
	return (1);
    }
    d4alias_set (old_strngin, "old_strngin");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/strngin.dbf %s/%s/strngin.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/strngin.mdx %s/%s/strngin.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/strngin", TIS4000DB,DBCNVRT);
    if (!(new_strngin = d4create (&cb, pathbuff, strnginFieldInfo, strnginTagInfo)))
    {
	fprintf (stdout, "strnginconv err: failed to create %s\n", pathbuff);
	d4close (old_strngin);
	return (1);
    }

    old_rec = d4record (old_strngin);
    new_rec = d4record (new_strngin);

    if ((oldname = d4field (old_strngin, "NAME")) == 0)
    {
	fprintf (stdout, "strnginconv err: NAME not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldstring = d4field (old_strngin, "CUR_STRING")) == 0)
    {
	fprintf (stdout, "strnginconv err: CUR_STRING not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldbdtype = d4field (old_strngin, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "strnginconv err: BOARDTYPE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldpi = d4field (old_strngin, "PROC_INPUT")) == 0)
    {
	fprintf (stdout, "strnginconv err: PROC_INPUT not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldref = d4field (old_strngin, "REFERENCE")) == 0)
    {
	fprintf (stdout, "strnginconv err: REFERENCE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldms = d4field (old_strngin, "MAXSEVERTY")) == 0)
    {
	fprintf (stdout, "strnginconv err: MAXSEVERTY not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldconst = d4field (old_strngin, "CONSTANT")) == 0)
    {
	fprintf (stdout, "strnginconv err: CONSTANT not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldauto = d4field (old_strngin, "AUTOMATIC")) == 0)
    {
	fprintf (stdout, "strnginconv err: AUTOMATIC not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldrdsz = d4field (old_strngin, "READSIZE")) == 0)
    {
	fprintf (stdout, "strnginconv err: BOARDTYPE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((oldchgsv = d4field (old_strngin, "CHGVALSEVR")) == 0)
    {
	fprintf (stdout, "strnginconv err: CHGVALSEVR not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newname = d4field (new_strngin, "NAME")) == 0)
    {
	fprintf (stdout, "strnginconv err: NAME not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newstring = d4field (new_strngin, "CUR_STRING")) == 0)
    {
	fprintf (stdout, "strnginconv err: CUR_STRING not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newbdtype = d4field (new_strngin, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "strnginconv err: BOARDTYPE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newpi = d4field (new_strngin, "PROC_INPUT")) == 0)
    {
	fprintf (stdout, "strnginconv err: PROC_INPUT not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newref = d4field (new_strngin, "REFERENCE")) == 0)
    {
	fprintf (stdout, "strnginconv err: REFERENCE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newms = d4field (new_strngin, "MAXSEVERTY")) == 0)
    {
	fprintf (stdout, "strnginconv err: MAXSEVERTY not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newconst = d4field (new_strngin, "CONSTANT")) == 0)
    {
	fprintf (stdout, "strnginconv err: CONSTANT not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newauto = d4field (new_strngin, "AUTOMATIC")) == 0)
    {
	fprintf (stdout, "strnginconv err: AUTOMATIC not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newrdsz = d4field (new_strngin, "READSIZE")) == 0)
    {
	fprintf (stdout, "strnginconv err: BOARDTYPE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((newchgsv = d4field (new_strngin, "CHGVALSEVR")) == 0)
    {
	fprintf (stdout, "strnginconv err: CHGVALSEVR not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    for (d4top (old_strngin); !d4eof (old_strngin); d4skip (old_strngin, 1))
    {
	if (d4deleted (old_strngin))
	    continue;
	d4append_start (new_strngin, 0);
	d4blank (new_strngin);
	f4assign_field (newname, oldname);
	f4assign_field (newstring, oldstring);
	f4assign_field (newbdtype, oldbdtype);
	f4assign_field (newpi, oldpi);
	f4assign_field (newref, oldref);
	f4assign_field (newms, oldms);
	f4assign_field (newconst, oldconst);
	f4assign_field (newauto, oldauto);
	f4assign_field (newrdsz, oldrdsz);
	f4assign_field (newchgsv, oldchgsv);
	d4append (new_strngin);
	d4flush (new_strngin);
    }

    d4close (old_strngin);
    d4close (new_strngin);

    fprintf (stdout, "...finished strngin.dbf\n");
    return (0);
}


static int convert_strngout (void)
{
    DATA4	*old_strngout, *new_strngout;
    FIELD4	*oldname, *newname;
    FIELD4	*oldstring, *newstring;
    FIELD4	*oldbdtype, *newbdtype;
    FIELD4	*oldref, *newref;
    FIELD4	*oldconst, *newconst;
    FIELD4	*oldout, *newout;
    FIELD4	*oldomsl, *newomsl;
    FIELD4	*oldochg, *newochg;
    int		fieldLen;

    fprintf (stdout, "...converting strngout.dbf\n");

    sprintf (pathbuff, "%s/strngout", TIS4000DB);
    if (!(old_strngout = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "strngoutconv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((oldstring = d4field (old_strngout, "CUR_STRING")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CUR_STRING not exist\n", pathbuff);
	d4close (old_strngout);
	return (1);
    }
    fieldLen  = f4len (oldstring);
    if (fieldLen == 80)
    {
	fprintf (stdout, "strngoutconv err: stringout already converted\n", pathbuff);
	d4close (old_strngout);
	return (1);
    }
    d4alias_set (old_strngout, "old_strngout");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/strngout.dbf %s/%s/strngout.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/strngout.mdx %s/%s/strngout.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/strngout", TIS4000DB,DBCNVRT);
    if (!(new_strngout = d4create (&cb, pathbuff, strngoutFieldInfo, strngoutTagInfo)))
    {
	fprintf (stdout, "strngconv err: failed to create %s\n", pathbuff);
	d4close (old_strngout);
	return (1);
    }

    old_rec = d4record (old_strngout);
    new_rec = d4record (new_strngout);

    if ((oldname = d4field (old_strngout, "NAME")) == 0)
    {
	fprintf (stdout, "strngoutconv err: NAME not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldstring = d4field (old_strngout, "CUR_STRING")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CUR_STRING not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldbdtype = d4field (old_strngout, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "strngoutconv err: BOARDTYPE not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldref = d4field (old_strngout, "REFERENCE")) == 0)
    {
	fprintf (stdout, "strngoutconv err: REFERENCE not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldconst = d4field (old_strngout, "CONSTANT")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CONSTANT not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldout = d4field (old_strngout, "OUTPUTREF")) == 0)
    {
	fprintf (stdout, "strngoutconv err: OUTPUTREF not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldomsl = d4field (old_strngout, "CLOSEDLOOP")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CLOSEDLOOP not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((oldochg = d4field (old_strngout, "OUTCHG")) == 0)
    {
	fprintf (stdout, "strngoutconv err: OUTCHG not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newname = d4field (new_strngout, "NAME")) == 0)
    {
	fprintf (stdout, "strngoutconv err: NAME not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newstring = d4field (new_strngout, "CUR_STRING")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CUR_STRING not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newbdtype = d4field (new_strngout, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "strngoutconv err: BOARDTYPE not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newref = d4field (new_strngout, "REFERENCE")) == 0)
    {
	fprintf (stdout, "strngoutconv err: REFERENCE not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newconst = d4field (new_strngout, "CONSTANT")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CONSTANT not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newout = d4field (new_strngout, "OUTPUTREF")) == 0)
    {
	fprintf (stdout, "strngoutconv err: OUTPUTREF not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newomsl = d4field (new_strngout, "CLOSEDLOOP")) == 0)
    {
	fprintf (stdout, "strngoutconv err: CLOSEDLOOP not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    if ((newochg = d4field (new_strngout, "OUTCHG")) == 0)
    {
	fprintf (stdout, "strngoutconv err: OUTCHG not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    for (d4top (old_strngout); !d4eof (old_strngout); d4skip (old_strngout, 1))
    {
	if (d4deleted (old_strngout))
	    continue;
	d4append_start (new_strngout, 0);
	d4blank (new_strngout);
	f4assign_field (newname, oldname);
	f4assign_field (newstring, oldstring);
	f4assign_field (newbdtype, oldbdtype);
	f4assign_field (newref, oldref);
	f4assign_field (newconst, oldconst);
	f4assign_field (newout, oldout);
	f4assign_field (newomsl, oldomsl);
	f4assign_field (newochg, oldochg);
	d4append (new_strngout);
	d4flush (new_strngout);
    }

    d4close (old_strngout);
    d4close (new_strngout);

    fprintf (stdout, "...finished strngout.dbf\n");

    return (0);
}

