/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include	<stdio.h>
#ifndef AXP
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include	"ai.h"
#include	"ao.h"
#include	"ar_reduc.h"
#include	"calc.h"
#include	"ctrend.h"
#include	"ddc.h"
#include	"do.h"
#include	"ldfw1.h"
#include	"ldfw2.h"
#include	"mbbo.h"
#include	"pump.h"
#include	"pat.h"
#include	"pvmap.h"
#include	"reslist.h"
#include	"sel.h"
#include	"sigchar.h"
#include	"strngin.h"
#include	"strngout.h"
#include	"tagvudb.h"
#include	"td.h"

static CODE4		cb;
static char		pathbuff[128];
static char		sysbuff[256];
static char		*old_rec, *new_rec;
static struct stat	statbuff;

static int create_subdir (void);
static int convert_ai (void);
static int convert_ao (void);
static int convert_ar_reduc (void);
static int convert_bo (void);
static int convert_calc (void);
static int create_ctrend (void);
static int create_ddc (void);
static int create_ldfw1 (void);
static int create_ldfw2 (void);
static int convert_mbbo (void);
static int convert_pat (void);
static int convert_pump (void);
static int convert_pvmap (void);
static int convert_reslist (void);
static int convert_sel (void);
static int convert_sigchar (void);
static int convert_strngin (void);
static int convert_strngout (void);
static int convert_tagvu (void);
static int convert_td (void);

#define	TIS4000DB	"/tis4000db"
#define	DBCNVRT		"t50cnvrt"


int main()
{
    fprintf (stdout, "starting T50 conversion\n\n");
    d4init (&cb);

#if defined(WIN32) || defined(USECB63)
    cb.errGo = 0;
    cb.errOpen = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
#else
    cb.go_error = 0;
    cb.open_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
#endif WIN32 or USECB63

    if (create_subdir ())
	return (1);
    if (convert_ai ())
	return (1);
    if (convert_ao ())
	return (1);
    if (convert_ar_reduc ())
	return (1);
    if (convert_bo ())
	return (1);
    if (convert_calc ())
	return (1);
    if (create_ctrend ())
	return (1);
    if (create_ddc ())
	return (1);
    if (create_ldfw2 ())
	return (1);
    if (create_ldfw1 ())
	return (1);
    if (convert_mbbo ())
	return (1);
    if (convert_pat ())
	return (1);
    if (convert_pump ())
	return (1);
    if (convert_pvmap ())
	return (1);
    if (convert_reslist ())
	return (1);
    if (convert_sel ())
	return (1);
    if (convert_sigchar ())
	return (1);
    if (convert_strngin ())
	return (1);
    if (convert_strngout ())
	return (1);
    if (convert_tagvu ())
	return (1);
    if (convert_td ())
	return (1);
    d4close_all (&cb);
    fprintf (stdout, "\nT50 conversion finished\n");
    return (0);
}


static int create_subdir (void)
{
    sprintf (pathbuff, "%s/%s", TIS4000DB, DBCNVRT);
    if (stat (pathbuff, &statbuff))
    {
	if (errno == ENOENT)
	{
	    sprintf (sysbuff, "mkdir %s", pathbuff);
	    system (sysbuff);
	    if (stat (pathbuff, &statbuff))
	    {
		fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
		return (1);
	    }
	    else
	    {
		fprintf (stdout, "t50conv: %s created\n", pathbuff);
		return (0);
	    }
	}
	else
	{
	    fprintf (stdout, "t50conv err: stat problem (%d) on %s\n", errno, pathbuff);
	    return (1);
	}
    }
    else
    {
	fprintf (stdout, "t50conv err: %s already exists\n", pathbuff);
    }
    return (0);
}


static int convert_ai (void)
{
    DATA4	*old_ai, *new_ai;
    FIELD4	*devsetpt, *devlimit, *devsever;
    FIELD4	*gain, *sigtype;

    fprintf (stdout, "...converting ai.dbf\n");

    sprintf (pathbuff, "%s/ai", TIS4000DB);
    if (!(old_ai = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((devsetpt = d4field (old_ai, "DEVSETPT")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_ai);
	return (1);
    }
    d4alias_set (old_ai, "old_ai");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/ai.dbf %s/%s/ai.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/ai.mdx %s/%s/ai.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ai", TIS4000DB,DBCNVRT);
    if (!(new_ai = d4create (&cb, pathbuff, aiFieldInfo, aiTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_ai);
	return (1);
    }

    old_rec = d4record (old_ai);
    new_rec = d4record (new_ai);

    if ((devsetpt = d4field (new_ai, "DEVSETPT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSETPT not exist\n");
	d4close (old_ai);
	d4close (new_ai);
	return (1);
    }

    if ((devlimit = d4field (new_ai, "DEVLIMIT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVLIMIT not exist\n");
	d4close (old_ai);
	d4close (new_ai);
	return (1);
    }

    if ((devsever = d4field (new_ai, "DEVSEVER")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSEVER not exist\n");
	d4close (old_ai);
	d4close (new_ai);
	return (1);
    }

    if ((gain = d4field (new_ai, "GAIN")) == 0)
    {
	fprintf (stdout, "t50conv err: GAIN not exist\n");
	d4close (old_ai);
	d4close (new_ai);
	return (1);
    }

    if ((sigtype = d4field (new_ai, "SIGTYPE")) == 0)
    {
	fprintf (stdout, "t50conv err: SIGTYPE not exist\n");
	d4close (old_ai);
	d4close (new_ai);
	return (1);
    }

    for (d4top (old_ai); !d4eof (old_ai); d4skip (old_ai, 1))
    {
	if (d4deleted (old_ai))
	    continue;
	d4append_start (new_ai, 0);
	d4blank (new_ai);
	memcpy (new_rec, old_rec, d4record_width (old_ai));
	f4assign_int (devsetpt, 0);
	f4assign_int (devlimit, 0);
	f4assign (devsever, "NO ALARM");
	f4assign_int (gain, 0);
	f4assign (sigtype, "N/A");
	d4append (new_ai);
	d4flush (new_ai);
    }

    d4close (old_ai);
    d4close (new_ai);

    fprintf (stdout, "...finished ai.dbf\n");
    return (0);
}


static int convert_ao (void)
{
    DATA4	*old_ao, *new_ao;
    FIELD4	*outgap;

    fprintf (stdout, "...converting ao.dbf\n");

    sprintf (pathbuff, "%s/ao", TIS4000DB);
    if (!(old_ao = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((outgap = d4field (old_ao, "OUTGAP")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_ao);
	return (1);
    }
    d4alias_set (old_ao, "old_ao");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/ao.dbf %s/%s/ao.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/ao.mdx %s/%s/ao.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ao", TIS4000DB,DBCNVRT);
    if (!(new_ao = d4create (&cb, pathbuff, aoFieldInfo, aoTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_ao);
	return (1);
    }

    old_rec = d4record (old_ao);
    new_rec = d4record (new_ao);

    if ((outgap = d4field (new_ao, "OUTGAP")) == 0)
    {
	fprintf (stdout, "t50conv err: OUTGAP not exist\n");
	d4close (old_ao);
	d4close (new_ao);
	return (1);
    }

    for (d4top (old_ao); !d4eof (old_ao); d4skip (old_ao, 1))
    {
	if (d4deleted (old_ao))
	    continue;
	d4append_start (new_ao, 0);
	d4blank (new_ao);
	memcpy (new_rec, old_rec, d4record_width (old_ao));
	f4assign_int (outgap, 0);
	d4append (new_ao);
	d4flush (new_ao);
    }

    d4close (old_ao);
    d4close (new_ao);

    fprintf (stdout, "...finished ao.dbf\n");

    return (0);
}


static int convert_ar_reduc (void)
{
    DATA4	*old_ar_reduc, *new_ar_reduc;
    FIELD4	*old_pointname, *old_rate, *old_method;
    FIELD4	*new_pointname, *new_rate, *new_method, *new_archive;
    int		rate;
    char	rate_buff[6];

    fprintf (stdout, "...converting ar_reduc.dbf\n");

    sprintf (pathbuff, "%s/ar_reduc", TIS4000DB);
    if (!(old_ar_reduc = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((old_rate = d4field (old_ar_reduc, "RATE")) == 0)
    {
	fprintf (stdout, "t50conv err: RATE not exist in old\n");
	d4close (old_ar_reduc);
	return (1);
    }
    if ((f4len (old_rate)) == 5)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_ar_reduc);
	return (1);
    }
    d4alias_set (old_ar_reduc, "old_ar_reduc");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/ar_reduc.dbf %s/%s/ar_reduc.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/ar_reduc.mdx %s/%s/ar_reduc.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ar_reduc", TIS4000DB,DBCNVRT);
    if (!(new_ar_reduc = d4create (&cb, pathbuff, ar_reducFieldInfo, ar_reducTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_ar_reduc);
	return (1);
    }

    if ((old_pointname = d4field (old_ar_reduc, "POINTNAME")) == 0)
    {
	fprintf (stdout, "t50conv err: POINTNAME not exist in old\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }
    if ((new_pointname = d4field (new_ar_reduc, "POINTNAME")) == 0)
    {
	fprintf (stdout, "t50conv err: POINTNAME not exist\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }

    if ((old_rate = d4field (old_ar_reduc, "RATE")) == 0)
    {
	fprintf (stdout, "t50conv err: RATE not exist in old\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }
    if ((new_rate = d4field (new_ar_reduc, "RATE")) == 0)
    {
	fprintf (stdout, "t50conv err: RATE not exist\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }

    if ((old_method = d4field (old_ar_reduc, "METHOD")) == 0)
    {
	fprintf (stdout, "t50conv err: METHOD not exist in old\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }
    if ((new_method = d4field (new_ar_reduc, "METHOD")) == 0)
    {
	fprintf (stdout, "t50conv err: METHOD not exist\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }
    if ((new_archive = d4field (new_ar_reduc, "ARCHIVE")) == 0)
    {
	fprintf (stdout, "t50conv err: ARCHIVE not exist\n");
	d4close (old_ar_reduc);
	d4close (new_ar_reduc);
	return (1);
    }

    for (d4top (old_ar_reduc); !d4eof (old_ar_reduc); d4skip (old_ar_reduc, 1))
    {
	if (d4deleted (old_ar_reduc))
	    continue;
	d4append_start (new_ar_reduc, 0);
	d4blank (new_ar_reduc);
	f4assign_field (new_pointname, old_pointname);
	rate = (atof(f4str(old_rate))) * 60.0;
	memset (rate_buff, 0, sizeof(rate_buff));
	sprintf (rate_buff, "%-5d\0", rate);
	f4assign (new_rate, rate_buff);
	f4assign_field (new_method, old_method);
	f4assign_char (new_archive, 'T');
	d4append (new_ar_reduc);
	d4flush (new_ar_reduc);
    }

    d4close (old_ar_reduc);
    d4close (new_ar_reduc);

    fprintf (stdout, "...finished ar_reduc.dbf\n");
    return (0);
}


static int convert_bo (void)
{
    DATA4	*old_bo, *new_bo;
    FIELD4	*outchg;

    fprintf (stdout, "...converting do.dbf\n");

    sprintf (pathbuff, "%s/do", TIS4000DB);
    if (!(old_bo = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((outchg = d4field (old_bo, "OUTCHG")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_bo);
	return (1);
    }
    d4alias_set (old_bo, "old_bo");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/do.dbf %s/%s/do.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/do.mdx %s/%s/do.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/do", TIS4000DB,DBCNVRT);
    if (!(new_bo = d4create (&cb, pathbuff, doFieldInfo, doTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_bo);
	return (1);
    }

    old_rec = d4record (old_bo);
    new_rec = d4record (new_bo);

    if ((outchg = d4field (new_bo, "OUTCHG")) == 0)
    {
	fprintf (stdout, "t50conv err: OUTCHG not exist\n");
	d4close (old_bo);
	d4close (new_bo);
	return (1);
    }

    for (d4top (old_bo); !d4eof (old_bo); d4skip (old_bo, 1))
    {
	if (d4deleted (old_bo))
	    continue;
	d4append_start (new_bo, 0);
	d4blank (new_bo);
	memcpy (new_rec, old_rec, d4record_width (old_bo));
	f4assign_char (outchg, 'F');
	d4append (new_bo);
	d4flush (new_bo);
    }

    d4close (old_bo);
    d4close (new_bo);

    fprintf (stdout, "...finished do.dbf\n");

    return (0);
}


static int convert_calc (void)
{
    DATA4	*old_calc, *new_calc;
    FIELD4	*devsetpt, *devlimit, *devsever;

    fprintf (stdout, "...converting calc.dbf\n");

    sprintf (pathbuff, "%s/calc", TIS4000DB);
    if (!(old_calc = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((devsetpt = d4field (old_calc, "DEVSETPT")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_calc);
	return (1);
    }
    d4alias_set (old_calc, "old_calc");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/calc.dbf %s/%s/calc.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/calc.mdx %s/%s/calc.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/calc", TIS4000DB,DBCNVRT);
    if (!(new_calc = d4create (&cb, pathbuff, calcFieldInfo, calcTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_calc);
	return (1);
    }

    old_rec = d4record (old_calc);
    new_rec = d4record (new_calc);

    if ((devsetpt = d4field (new_calc, "DEVSETPT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSETPT not exist\n");
	d4close (old_calc);
	d4close (new_calc);
	return (1);
    }

    if ((devlimit = d4field (new_calc, "DEVLIMIT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVLIMIT not exist\n");
	d4close (old_calc);
	d4close (new_calc);
	return (1);
    }

    if ((devsever = d4field (new_calc, "DEVSEVER")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSEVER not exist\n");
	d4close (old_calc);
	d4close (new_calc);
	return (1);
    }

    for (d4top (old_calc); !d4eof (old_calc); d4skip (old_calc, 1))
    {
	if (d4deleted (old_calc))
	    continue;
	d4append_start (new_calc, 0);
	d4blank (new_calc);
	memcpy (new_rec, old_rec, d4record_width (old_calc));
	f4assign_int (devsetpt, 0);
	f4assign_int (devlimit, 0);
	f4assign (devsever, "NO ALARM");
	d4append (new_calc);
	d4flush (new_calc);
    }

    d4close (old_calc);
    d4close (new_calc);

    fprintf (stdout, "...finished calc.dbf\n");
    return (0);
}


static int create_ctrend (void)
{
    DATA4	*old_ctrend, *new_ctrend;

    fprintf (stdout, "...creating ctrend.dbf\n");

    sprintf (pathbuff, "%s/ctrend", TIS4000DB);
    if (old_ctrend = d4open (&cb, pathbuff))
    {
	fprintf (stdout, "t50conv err: %s already exists\n", pathbuff);
	return (0);
    }

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ctrend", TIS4000DB,DBCNVRT);
    if (!(new_ctrend = d4create (&cb, pathbuff, ctrendFieldInfo, ctrendTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	return (1);
    }

    d4close (new_ctrend);

    fprintf (stdout, "...finished ctrend.dbf\n");
    return (0);
}


static int create_ddc (void)
{
    DATA4	*old_ddc, *new_ddc;

    fprintf (stdout, "...creating ddc.dbf\n");

    sprintf (pathbuff, "%s/ddc", TIS4000DB);
    if (old_ddc = d4open (&cb, pathbuff))
    {
	fprintf (stdout, "t50conv err: %s already exists\n", pathbuff);
	return (0);
    }

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ddc", TIS4000DB,DBCNVRT);
    if (!(new_ddc = d4create (&cb, pathbuff, ddcFieldInfo, ddcTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	return (1);
    }

    d4close (new_ddc);

    fprintf (stdout, "...finished ddc.dbf\n");
    return (0);
}


static int create_ldfw1 (void)
{
    DATA4	*old_ldfw1, *new_ldfw1;

    fprintf (stdout, "...creating ldfw1.dbf\n");

    sprintf (pathbuff, "%s/ldfw1", TIS4000DB);
    if (old_ldfw1 = d4open (&cb, pathbuff))
    {
	fprintf (stdout, "t50conv err: %s already exists\n", pathbuff);
	return (0);
    }

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ldfw1", TIS4000DB,DBCNVRT);
    if (!(new_ldfw1 = d4create (&cb, pathbuff, ldfw1FieldInfo, ldfw1TagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	return (1);
    }

    d4close (new_ldfw1);

    fprintf (stdout, "...finished ldfw1.dbf\n");
    return (0);
}

static int create_ldfw2 (void)
{
    DATA4	*old_ldfw2, *new_ldfw2;

    fprintf (stdout, "...creating ldfw2.dbf\n");

    sprintf (pathbuff, "%s/ldfw2", TIS4000DB);
    if (old_ldfw2 = d4open (&cb, pathbuff))
    {
	fprintf (stdout, "t50conv err: %s already exists\n", pathbuff);
	return (0);
    }

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/ldfw2", TIS4000DB,DBCNVRT);
    if (!(new_ldfw2 = d4create (&cb, pathbuff, ldfw2FieldInfo, ldfw2TagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	return (1);
    }

    d4close (new_ldfw2);

    fprintf (stdout, "...finished ldfw2.dbf\n");
    return (0);
}

static int convert_mbbo (void)
{
    DATA4	*old_mbbo, *new_mbbo;
    FIELD4	*outchg;

    fprintf (stdout, "...converting mbbo.dbf\n");

    sprintf (pathbuff, "%s/mbbo", TIS4000DB);
    if (!(old_mbbo = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((outchg = d4field (old_mbbo, "OUTCHG")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_mbbo);
	return (1);
    }
    d4alias_set (old_mbbo, "old_mbbo");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/mbbo.dbf %s/%s/mbbo.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/mbbo.mdx %s/%s/mbbo.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/mbbo", TIS4000DB,DBCNVRT);
    if (!(new_mbbo = d4create (&cb, pathbuff, mbboFieldInfo, mbboTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_mbbo);
	return (1);
    }

    old_rec = d4record (old_mbbo);
    new_rec = d4record (new_mbbo);

    if ((outchg = d4field (new_mbbo, "OUTCHG")) == 0)
    {
	fprintf (stdout, "t50conv err: OUTCHG not exist\n");
	d4close (old_mbbo);
	d4close (new_mbbo);
	return (1);
    }

    for (d4top (old_mbbo); !d4eof (old_mbbo); d4skip (old_mbbo, 1))
    {
	if (d4deleted (old_mbbo))
	    continue;
	d4append_start (new_mbbo, 0);
	d4blank (new_mbbo);
	memcpy (new_rec, old_rec, d4record_width (old_mbbo));
	f4assign_char (outchg, 'F');
	d4append (new_mbbo);
	d4flush (new_mbbo);
    }

    d4close (old_mbbo);
    d4close (new_mbbo);

    fprintf (stdout, "...finished mbbo.dbf\n");

    return (0);
}


static int convert_pat (void)
{
    DATA4	*old_pat, *new_pat;
    FIELD4	*boardtype;
    FIELD4	*rlboard;
    FIELD4	*fbboard;
    char	*old_rec;
    char	*new_rec;

    fprintf (stdout, "...converting pat.pdbf\n");

    sprintf (pathbuff, "%s/pat", TIS4000DB);
    if (!(old_pat = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    d4alias_set (old_pat, "old_pat");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/pat.dbf %s/%s/pat.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/pat.mdx %s/%s/pat.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/pat", TIS4000DB,DBCNVRT);
    if (!(new_pat = d4create (&cb, pathbuff, patFieldInfo, patTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_pat);
	return (1);
    }

    old_rec = d4record (old_pat);
    new_rec = d4record (new_pat);

    if ((boardtype = d4field (new_pat, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50conv err: BOARDTYPE (new) not exist\n");
	d4close (old_pat);
	d4close (new_pat);
	return (1);
    }

    if ((rlboard = d4field (new_pat, "RLBOARD")) == 0)
    {
	fprintf (stdout, "t50conv err: RLBOARD (new) not exist\n");
	d4close (old_pat);
	d4close (new_pat);
	return (1);
    }

    if ((fbboard = d4field (new_pat, "FBBOARD")) == 0)
    {
	fprintf (stdout, "t50conv err: FBBOARD (new) not exist\n");
	d4close (old_pat);
	d4close (new_pat);
	return (1);
    }

    for (d4top (old_pat); !d4eof (old_pat); d4skip (old_pat, 1))
    {
	if (d4deleted (old_pat))
	    continue;
	d4append_start (new_pat, 0);
	d4blank (new_pat);
	/* copy old record into new record, they are same size */
	memcpy (new_rec, old_rec, d4record_width (old_pat));
	if (strncmp (f4str (boardtype), "REFERENCE", 9) == 0)
	{
	    f4assign (boardtype, "PAT BO REFERENCE");
	}
	if (strncmp (f4str (rlboard), "REFERENCE", 9) == 0)
	{
	    f4assign (rlboard, "PAT BI REFERENCE");
	}
	if (strncmp (f4str (fbboard), "REFERENCE", 9) == 0)
	{
	    f4assign (fbboard, "PAT AI REFERENCE");
	}
	d4append (new_pat);
	d4flush (new_pat);
    }

    d4close (old_pat);
    d4close (new_pat);

    fprintf (stdout, "...finished pat.dbf\n");
    return (0);
}


static int convert_pump (void)
{
    DATA4	*old_pump, *new_pump;
    FIELD4	*saveruntm, *runtimprec, *minofftime, *minoffsts;

    fprintf (stdout, "...converting pump.dbf\n");

    sprintf (pathbuff, "%s/pump", TIS4000DB);
    if (!(old_pump = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((saveruntm = d4field (old_pump, "SAVERUNTM")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_pump);
	return (1);
    }
    d4alias_set (old_pump, "old_pump");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/pump.dbf %s/%s/pump.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/pump.mdx %s/%s/pump.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/pump", TIS4000DB,DBCNVRT);
    if (!(new_pump = d4create (&cb, pathbuff, pumpFieldInfo, pumpTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_pump);
	return (1);
    }

    old_rec = d4record (old_pump);
    new_rec = d4record (new_pump);

    if ((saveruntm = d4field (new_pump, "SAVERUNTM")) == 0)
    {
	fprintf (stdout, "t50conv err: SAVERUNTM not exist\n");
	d4close (old_pump);
	d4close (new_pump);
	return (1);
    }

    if ((runtimprec = d4field (new_pump, "RUNTIMPREC")) == 0)
    {
	fprintf (stdout, "t50conv err: RUNTIMPREC not exist\n");
	d4close (old_pump);
	d4close (new_pump);
	return (1);
    }

    if ((minofftime = d4field (new_pump, "MINOFFTIME")) == 0)
    {
	fprintf (stdout, "t50conv err: MINOFFTIME not exist\n");
	d4close (old_pump);
	d4close (new_pump);
	return (1);
    }

    if ((minoffsts = d4field (new_pump, "MINOFFSTS")) == 0)
    {
	fprintf (stdout, "t50conv err: MINOFFSTS not exist\n");
	d4close (old_pump);
	d4close (new_pump);
	return (1);
    }

    for (d4top (old_pump); !d4eof (old_pump); d4skip (old_pump, 1))
    {
	if (d4deleted (old_pump))
	    continue;
	d4append_start (new_pump, 0);
	d4blank (new_pump);
	memcpy (new_rec, old_rec, d4record_width (old_pump));
	f4assign_int (saveruntm, 0);
	f4assign_int (runtimprec, 0);
	f4assign_int (minofftime, 0);
	f4assign (minoffsts, "");
	d4append (new_pump);
	d4flush (new_pump);
    }

    d4close (old_pump);
    d4close (new_pump);

    fprintf (stdout, "...finished pump.dbf\n");
    return (0);
}


static int convert_pvmap (void)
{
    DATA4	*old_pvmap, *new_pvmap;
    FIELD4	*loc_old, *loc_new;
    FIELD4	*ioc1_old, *ioc1_new; 
    FIELD4	*res_old, *res_new, *alarmdelay;
    FIELD4	*devtype;
    FIELD4	*infotag;
    FIELD4	*type;
    FIELD4	*deleteflag;
    char	*ploc_old, *ploc_new;
    char	*pres_old;
    char	*pioc1_old, *pioc1_new;

    fprintf (stdout, "...converting pvmap.dbf\n");

    sprintf (pathbuff, "%s/pvmap", TIS4000DB);
    if (!(old_pvmap = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((alarmdelay = d4field (old_pvmap, "ALARMDELAY")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_pvmap);
	return (1);
    }
    d4alias_set (old_pvmap, "old_pvmap");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/pvmap.dbf %s/%s/pvmap.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/pvmap.mdx %s/%s/pvmap.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/pvmap", TIS4000DB,DBCNVRT);
    if (!(new_pvmap = d4create (&cb, pathbuff, pvmapFieldInfo, pvmapTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_pvmap);
	return (1);
    }

    old_rec = d4record (old_pvmap);
    new_rec = d4record (new_pvmap);

    if ((alarmdelay = d4field (new_pvmap, "ALARMDELAY")) == 0)
    {
	fprintf (stdout, "t50conv err: ALARMDELAY not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }

    if ((loc_old = d4field (old_pvmap, "LOCATION")) == 0)
    {
	fprintf (stdout, "t50conv err: LOCATION (old) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }
    /* ptr to start of LOCATION in old record */
    ploc_old = f4ptr (loc_old);

    if ((loc_new = d4field (new_pvmap, "LOCATION")) == 0)
    {
	fprintf (stdout, "t50conv err: LOCATION (new) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }
    /* ptr to start of LOCATION in new record */
    ploc_new = f4ptr (loc_new);

    if ((res_old = d4field (old_pvmap, "RESRCNAME")) == 0)
    {
	fprintf (stdout, "t50conv err: RESRCNAME (old) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }
    /* ptr to start of RESRNAME in old record */
    pres_old = f4ptr (res_old);

    if ((res_new = d4field (new_pvmap, "RESRCNAME")) == 0)
    {
	fprintf (stdout, "t50conv err: RESRCNAME (new) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }

    if ((ioc1_old = d4field (old_pvmap, "IOC1DNLD")) == 0)
    {
	fprintf (stdout, "t50conv err: IOC1DNLD (old) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }
    /* ptr to start of IOC1DNLD in old record */
    pioc1_old = f4ptr (ioc1_old);

    if ((ioc1_new = d4field (new_pvmap, "IOC1DNLD")) == 0)
    {
	fprintf (stdout, "t50conv err: IOC1DNLD (new) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }
    /* ptr to start of IOC1DNLD in new record */
    pioc1_new = f4ptr (ioc1_new);

    if ((devtype = d4field (new_pvmap, "DEVTYPE")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVTYPE (new) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }

    if ((infotag = d4field (new_pvmap, "INFOTAG")) == 0)
    {
	fprintf (stdout, "t50conv err: INFOTAG (new) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }

    if ((type = d4field (new_pvmap, "TYPE")) == 0)
    {
	fprintf (stdout, "t50conv err: TYPE (new) not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }

    if ((deleteflag = d4field (old_pvmap, "DELETEFLAG")) == 0)
    {
	fprintf (stdout, "t50conv err: DELETEFLAG not exist\n");
	d4close (old_pvmap);
	d4close (new_pvmap);
	return (1);
    }

    for (d4top (old_pvmap); !d4eof (old_pvmap); d4skip (old_pvmap, 1))
    {
	/* do not copy if the delete flag is set */
	if (f4true(deleteflag))
	    continue;
	d4append_start (new_pvmap, 0);
	d4blank (new_pvmap);
	/* copy old record up to the LOCATION field to the new record */
	memcpy (new_rec, old_rec, ploc_old - old_rec);
	/* copy old record from LOCATION to the RESRCNAME field to the new record */
	memcpy (ploc_new, ploc_old, pres_old - ploc_old);
	/* set RESRCNAME in new record */
	f4assign_field (res_new, res_old);
	/* set ALARMDELAY field in new record */
	f4assign_int (alarmdelay, 0);
	/* copy from IOC1DNLD in old record to new record */
	memcpy (pioc1_new, pioc1_old, d4record_width (old_pvmap) -
				      (pioc1_old - old_rec));
	if (strncmp (f4str (devtype), "TIME/DATE", 9) == 0)
	{
	    f4assign (devtype, "SYSINFO");
	}
	f4assign_char (infotag, 'F');
	if (strncmp (f4str (type), "pat", 3) == 0)
	{
	    if (strncmp (f4str (devtype), "REFERENCE", 9) == 0)
	    {
		f4assign (devtype, "PAT BO REFERENCE");
	    }
	}
	d4append (new_pvmap);
	d4flush (new_pvmap);
    }

    d4close (old_pvmap);
    d4close (new_pvmap);

    fprintf (stdout, "...finished pvmap.dbf\n");
    return (0);
}


static int convert_reslist (void)
{
    DATA4	*old_reslist, *new_reslist;
    FIELD4	*dpa;

    fprintf (stdout, "...converting reslist.dbf\n");

    sprintf (pathbuff, "%s/reslist", TIS4000DB);
    if (!(old_reslist = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((dpa = d4field (old_reslist, "DPA")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_reslist);
	return (1);
    }
    d4alias_set (old_reslist, "old_reslist");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/reslist.dbf %s/%s/reslist.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/reslist.mdx %s/%s/reslist.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/reslist", TIS4000DB,DBCNVRT);
    if (!(new_reslist = d4create (&cb, pathbuff, reslistFieldInfo, reslistTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_reslist);
	return (1);
    }

    old_rec = d4record (old_reslist);
    new_rec = d4record (new_reslist);

    if ((dpa = d4field (new_reslist, "DPA")) == 0)
    {
	fprintf (stdout, "t50conv err: DPA not exist\n");
	d4close (old_reslist);
	d4close (new_reslist);
	return (1);
    }

    for (d4top (old_reslist); !d4eof (old_reslist); d4skip (old_reslist, 1))
    {
	if (d4deleted (old_reslist))
	    continue;
	d4append_start (new_reslist, 0);
	d4blank (new_reslist);
	memcpy (new_rec, old_rec, d4record_width (old_reslist));
	f4assign_char (dpa, 'F');
	d4append (new_reslist);
	d4flush (new_reslist);
    }

    d4close (old_reslist);
    d4close (new_reslist);

    fprintf (stdout, "...finished reslist.dbf\n");

    return (0);
}


static int convert_sel (void)
{
    DATA4	*old_sel, *new_sel;
    FIELD4	*devsetpt, *devlimit, *devsever;

    fprintf (stdout, "...converting sel.dbf\n");

    sprintf (pathbuff, "%s/sel", TIS4000DB);
    if (!(old_sel = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((devsetpt = d4field (old_sel, "DEVSETPT")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_sel);
	return (1);
    }
    d4alias_set (old_sel, "old_sel");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/sel.dbf %s/%s/sel.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/sel.mdx %s/%s/sel.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/sel", TIS4000DB,DBCNVRT);
    if (!(new_sel = d4create (&cb, pathbuff, selFieldInfo, selTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_sel);
	return (1);
    }

    old_rec = d4record (old_sel);
    new_rec = d4record (new_sel);

    if ((devsetpt = d4field (new_sel, "DEVSETPT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSETPT not exist\n");
	d4close (old_sel);
	d4close (new_sel);
	return (1);
    }

    if ((devlimit = d4field (new_sel, "DEVLIMIT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVLIMIT not exist\n");
	d4close (old_sel);
	d4close (new_sel);
	return (1);
    }

    if ((devsever = d4field (new_sel, "DEVSEVER")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSEVER not exist\n");
	d4close (old_sel);
	d4close (new_sel);
	return (1);
    }

    for (d4top (old_sel); !d4eof (old_sel); d4skip (old_sel, 1))
    {
	if (d4deleted (old_sel))
	    continue;
	d4append_start (new_sel, 0);
	d4blank (new_sel);
	memcpy (new_rec, old_rec, d4record_width (old_sel));
	f4assign_int (devsetpt, 0);
	f4assign_int (devlimit, 0);
	f4assign (devsever, "NO ALARM");
	d4append (new_sel);
	d4flush (new_sel);
    }

    d4close (old_sel);
    d4close (new_sel);

    fprintf (stdout, "...finished sel.dbf\n");
    return (0);
}


static int convert_sigchar (void)
{
    DATA4	*old_sigchar, *new_sigchar;
    FIELD4	*devsetpt, *devlimit, *devsever;

    fprintf (stdout, "...converting sigchar.dbf\n");

    sprintf (pathbuff, "%s/sigchar", TIS4000DB);
    if (!(old_sigchar = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((devsetpt = d4field (old_sigchar, "DEVSETPT")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_sigchar);
	return (1);
    }
    d4alias_set (old_sigchar, "old_sigchar");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/sigchar.dbf %s/%s/sigchar.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/sigchar.mdx %s/%s/sigchar.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/sigchar", TIS4000DB,DBCNVRT);
    if (!(new_sigchar = d4create (&cb, pathbuff, sigcharFieldInfo, sigcharTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_sigchar);
	return (1);
    }

    old_rec = d4record (old_sigchar);
    new_rec = d4record (new_sigchar);

    if ((devsetpt = d4field (new_sigchar, "DEVSETPT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSETPT not exist\n");
	d4close (old_sigchar);
	d4close (new_sigchar);
	return (1);
    }

    if ((devlimit = d4field (new_sigchar, "DEVLIMIT")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVLIMIT not exist\n");
	d4close (old_sigchar);
	d4close (new_sigchar);
	return (1);
    }

    if ((devsever = d4field (new_sigchar, "DEVSEVER")) == 0)
    {
	fprintf (stdout, "t50conv err: DEVSEVER not exist\n");
	d4close (old_sigchar);
	d4close (new_sigchar);
	return (1);
    }

    for (d4top (old_sigchar); !d4eof (old_sigchar); d4skip (old_sigchar, 1))
    {
	if (d4deleted (old_sigchar))
	    continue;
	d4append_start (new_sigchar, 0);
	d4blank (new_sigchar);
	memcpy (new_rec, old_rec, d4record_width (old_sigchar));
	f4assign_int (devsetpt, 0);
	f4assign_int (devlimit, 0);
	f4assign (devsever, "NO ALARM");
	d4append (new_sigchar);
	d4flush (new_sigchar);
    }

    d4close (old_sigchar);
    d4close (new_sigchar);

    fprintf (stdout, "...finished sigchar.dbf\n");
    return (0);
}


static int convert_strngin (void)
{
    DATA4	*old_strngin, *new_strngin;
    FIELD4	*cvsv;
    FIELD4	*bdtype;

    fprintf (stdout, "...converting strngin.dbf\n");

    sprintf (pathbuff, "%s/strngin", TIS4000DB);
    if (!(old_strngin = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((cvsv = d4field (old_strngin, "CHGVALSEVR")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_strngin);
	return (1);
    }
    d4alias_set (old_strngin, "old_strngin");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/strngin.dbf %s/%s/strngin.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/strngin.mdx %s/%s/strngin.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/strngin", TIS4000DB,DBCNVRT);
    if (!(new_strngin = d4create (&cb, pathbuff, strnginFieldInfo, strnginTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_strngin);
	return (1);
    }

    old_rec = d4record (old_strngin);
    new_rec = d4record (new_strngin);

    if ((cvsv = d4field (new_strngin, "CHGVALSEVR")) == 0)
    {
	fprintf (stdout, "t50conv err: CHGVALSEVR not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    if ((bdtype = d4field (new_strngin, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50conv err: BOARDTYPE not exist\n");
	d4close (old_strngin);
	d4close (new_strngin);
	return (1);
    }

    for (d4top (old_strngin); !d4eof (old_strngin); d4skip (old_strngin, 1))
    {
	if (d4deleted (old_strngin))
	    continue;
	d4append_start (new_strngin, 0);
	d4blank (new_strngin);
	memcpy (new_rec, old_rec, d4record_width (old_strngin));
	f4assign (cvsv, "NO ALARM");
	if (strncmp (f4str (bdtype), "TIME/DATE", 9) == 0)
	{
	    f4assign (bdtype, "SYSINFO");
	}
	d4append (new_strngin);
	d4flush (new_strngin);
    }

    d4close (old_strngin);
    d4close (new_strngin);

    fprintf (stdout, "...finished strngin.dbf\n");
    return (0);
}


static int convert_strngout (void)
{
    DATA4	*old_strngout, *new_strngout;
    FIELD4	*outchg;

    fprintf (stdout, "...converting strngout.dbf\n");

    sprintf (pathbuff, "%s/strngout", TIS4000DB);
    if (!(old_strngout = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((outchg = d4field (old_strngout, "OUTCHG")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_strngout);
	return (1);
    }
    d4alias_set (old_strngout, "old_strngout");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/strngout.dbf %s/%s/strngout.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/strngout.mdx %s/%s/strngout.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/strngout", TIS4000DB,DBCNVRT);
    if (!(new_strngout = d4create (&cb, pathbuff, strngoutFieldInfo, strngoutTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_strngout);
	return (1);
    }

    old_rec = d4record (old_strngout);
    new_rec = d4record (new_strngout);

    if ((outchg = d4field (new_strngout, "OUTCHG")) == 0)
    {
	fprintf (stdout, "t50conv err: OUTCHG not exist\n");
	d4close (old_strngout);
	d4close (new_strngout);
	return (1);
    }

    for (d4top (old_strngout); !d4eof (old_strngout); d4skip (old_strngout, 1))
    {
	if (d4deleted (old_strngout))
	    continue;
	d4append_start (new_strngout, 0);
	d4blank (new_strngout);
	memcpy (new_rec, old_rec, d4record_width (old_strngout));
	f4assign_char (outchg, 'F');
	d4append (new_strngout);
	d4flush (new_strngout);
    }

    d4close (old_strngout);
    d4close (new_strngout);

    fprintf (stdout, "...finished strngout.dbf\n");

    return (0);
}



static int convert_tagvu (void)
{
    DATA4	*old_tagvu, *new_tagvu;
    FIELD4	*oldptname, *newptname;
    FIELD4	*oldloc, *newloc;
    FIELD4	*olddesc, *newdesc;
    FIELD4	*olddate_apl, *newdate_apl;
    FIELD4	*oldtime_apl, *newtime_apl;
    FIELD4	*oldoper_apl, *newoper_apl;
    FIELD4	*oldreason_apl, *newreason_apl;
    FIELD4	*olddate_rm, *newdate_rm;
    FIELD4	*oldtime_rm, *newtime_rm;
    FIELD4	*oldoper_rm, *newoper_rm;
    FIELD4	*oldreason_rm, *newreason_rm;
    FIELD4	*tagtype;
    int		create_tagvu = 0;

    fprintf (stdout, "...converting tagvu.dbf\n");

    sprintf (pathbuff, "%s/tagvu.dbf", TIS4000DB);
    /* since the odl tagvu created tagvu.dbf, there may not be one
       so all we have to do is create it
    */
    if (stat (pathbuff, &statbuff) != 0)
    {
	if (errno == ENOENT)
	    create_tagvu = 1;
	else
	{
	    fprintf (stdout, "t50conv err: stat problem (%d) on %s\n", errno, pathbuff);
	    return (1);
	}
    }
    sprintf (pathbuff, "%s/tagvu", TIS4000DB);
    if (create_tagvu == 0)
    {
	if (!(old_tagvu = d4open (&cb, pathbuff)))
	{
	    fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	    return (1);
	}
	if ((tagtype = d4field (old_tagvu, "TAGTYPE")) != 0)
	{
	    fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	    d4close (old_tagvu);
	    return (1);
	}
	d4alias_set (old_tagvu, "old_tagvu");

	/* save the old dbase files */
	sprintf (sysbuff, "cp -p %s/tagvu.dbf %s/%s/tagvu.dbf.sav", 
			   TIS4000DB, TIS4000DB, DBCNVRT);
	system (sysbuff);
	sprintf (sysbuff, "cp -p %s/tagvu.mdx %s/%s/tagvu.mdx.sav", 
			   TIS4000DB, TIS4000DB, DBCNVRT);
	system (sysbuff);
    }

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/tagvu", TIS4000DB,DBCNVRT);
    if (!(new_tagvu = d4create (&cb, pathbuff, tagvuFieldInfo, tagvuTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	if (create_tagvu == 0)
	    d4close (old_tagvu);
	return (1);
    }
    if (create_tagvu)
    {
	d4close (new_tagvu);
	fprintf (stdout, "...finished tagvu.dbf\n");
	return (0);
    }

    old_rec = d4record (old_tagvu);
    new_rec = d4record (new_tagvu);

    if ((oldptname = d4field (old_tagvu, "POINTNAME")) == 0)
    {
	fprintf (stdout, "t50conv err: POINTNAME not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newptname = d4field (new_tagvu, "POINTNAME")) == 0)
    {
	fprintf (stdout, "t50conv err: POINTNAME not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldloc = d4field (old_tagvu, "LOCATION")) == 0)
    {
	fprintf (stdout, "t50conv err: LOCATION not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newloc = d4field (new_tagvu, "LOCATION")) == 0)
    {
	fprintf (stdout, "t50conv err: LOCATION not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((olddesc = d4field (old_tagvu, "DESCRIPTOR")) == 0)
    {
	fprintf (stdout, "t50conv err: DESCRIPTOR not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newdesc = d4field (new_tagvu, "DESCRIPTOR")) == 0)
    {
	fprintf (stdout, "t50conv err: DESCRIPTOR not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((olddate_apl = d4field (old_tagvu, "DATE_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: DATE_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newdate_apl = d4field (new_tagvu, "DATE_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: DATE_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldtime_apl = d4field (old_tagvu, "TIME_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: TIME_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newtime_apl = d4field (new_tagvu, "TIME_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: TIME_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldoper_apl = d4field (old_tagvu, "OPER_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: OPER_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newoper_apl = d4field (new_tagvu, "OPER_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: OPER_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldreason_apl = d4field (old_tagvu, "REASON_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: REASON_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newreason_apl = d4field (new_tagvu, "REASON_APL")) == 0)
    {
	fprintf (stdout, "t50conv err: REASON_APL not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((olddate_rm = d4field (old_tagvu, "DATE_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: DATE_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newdate_rm = d4field (new_tagvu, "DATE_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: DATE_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldtime_rm = d4field (old_tagvu, "TIME_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: TIME_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newtime_rm = d4field (new_tagvu, "TIME_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: TIME_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldoper_rm = d4field (old_tagvu, "OPER_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: OPER_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newoper_rm = d4field (new_tagvu, "OPER_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: OPER_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((oldreason_rm = d4field (old_tagvu, "REASON_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: REASON_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((newreason_rm = d4field (new_tagvu, "REASON_RMV")) == 0)
    {
	fprintf (stdout, "t50conv err: REASON_RMV not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    if ((tagtype = d4field (new_tagvu, "TAGTYPE")) == 0)
    {
	fprintf (stdout, "t50conv err: TAGTYPE not exist\n");
	d4close (old_tagvu);
	d4close (new_tagvu);
	return (1);
    }

    for (d4top (old_tagvu); !d4eof (old_tagvu); d4skip (old_tagvu, 1))
    {
	if (d4deleted (old_tagvu))
	    continue;
	d4append_start (new_tagvu, 0);
	d4blank (new_tagvu);
	f4assign_field (newptname, oldptname);
	f4assign_field (newloc, oldloc);
	f4assign_field (newdesc, olddesc);
	f4assign_field (newdate_apl, olddate_apl);
	f4assign_field (newtime_apl, oldtime_apl);
	f4assign_field (newoper_apl, oldoper_apl);
	f4assign_field (newreason_apl, oldreason_apl);
	f4assign_field (newdate_rm, olddate_rm);
	f4assign_field (newtime_rm, oldtime_rm);
	f4assign_field (newoper_rm, oldoper_rm);
	f4assign_field (newreason_rm, oldreason_rm);
	f4assign_char (tagtype, 'O');
	d4append (new_tagvu);
	d4flush (new_tagvu);
    }

    d4close (old_tagvu);
    d4close (new_tagvu);

    fprintf (stdout, "...finished tagvu.dbf\n");
    return (0);
}


static int convert_td (void)
{
    DATA4	*old_td, *new_td;
    FIELD4	*oldname, *oldtimedelay1, *oldtimedelay2;
    FIELD4	*oldclosedloop;
    FIELD4	*newname, *newtimedelay1, *newtimedelay2; 
    FIELD4	*newclosedloop, *newdwelltime, *newfirststate;
    char	*oldcloop, *newcloop;

    fprintf (stdout, "...converting td.dbf\n");

    sprintf (pathbuff, "%s/td", TIS4000DB);
    if (!(old_td = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50conv err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((newdwelltime = d4field (old_td, "DWELLTIME")) != 0)
    {
	fprintf (stdout, "t50conv err: %s already converted\n", pathbuff);
	d4close (old_td);
	return (1);
    }
    d4alias_set (old_td, "old_td");

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/td.dbf %s/%s/td.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);
    sprintf (sysbuff, "cp -p %s/td.mdx %s/%s/td.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* create the new dbase file */
    sprintf (pathbuff, "%s/%s/td", TIS4000DB,DBCNVRT);
    if (!(new_td = d4create (&cb, pathbuff, tdFieldInfo, tdTagInfo)))
    {
	fprintf (stdout, "t50conv err: failed to create %s\n", pathbuff);
	d4close (old_td);
	return (1);
    }

    old_rec = d4record (old_td);
    new_rec = d4record (new_td);

    if ((oldname = d4field (old_td, "NAME")) == 0)
    {
	fprintf (stdout, "t50conv err: old NAME not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((oldtimedelay1 = d4field (old_td, "TIMEDELAY1")) == 0)
    {
	fprintf (stdout, "t50conv err: old TIMEDELAY1 not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((oldtimedelay2 = d4field (old_td, "TIMEDELAY2")) == 0)
    {
	fprintf (stdout, "t50conv err: old TIMEDELAY2 not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((oldclosedloop = d4field (old_td, "CLOSEDLOOP")) == 0)
    {
	fprintf (stdout, "t50conv err: old CLOSEDLOOP not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((newname = d4field (new_td, "NAME")) == 0)
    {
	fprintf (stdout, "t50conv err: new NAME not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((newtimedelay1 = d4field (new_td, "TIMEDELAY1")) == 0)
    {
	fprintf (stdout, "t50conv err: new TIMEDELAY1 not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((newtimedelay2 = d4field (new_td, "TIMEDELAY2")) == 0)
    {
	fprintf (stdout, "t50conv err: new TIMEDELAY2 not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((newclosedloop = d4field (new_td, "CLOSEDLOOP")) == 0)
    {
	fprintf (stdout, "t50conv err: new CLOSEDLOOP not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((newdwelltime = d4field (new_td, "DWELLTIME")) == 0)
    {
	fprintf (stdout, "t50conv err: new DWELLTIME not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    if ((newfirststate = d4field (new_td, "FIRSTSTATE")) == 0)
    {
	fprintf (stdout, "t50conv err: new FIRSTSTATE not exist\n");
	d4close (old_td);
	d4close (new_td);
	return (1);
    }

    /* get ptr to these fields in the record buffers */
    oldcloop = f4ptr (oldclosedloop);
    newcloop = f4ptr (newclosedloop);

    for (d4top (old_td); !d4eof (old_td); d4skip (old_td, 1))
    {
	if (d4deleted (old_td))
	    continue;
	d4append_start (new_td, 0);
	d4blank (new_td);
	f4assign_field (newname, oldname);
	f4assign_field (newtimedelay1, oldtimedelay1);
	f4assign_field (newtimedelay2, oldtimedelay2);
	memcpy (newcloop, oldcloop, d4record_width (old_td) - 
				    (oldcloop - old_rec));
	f4assign_int (newdwelltime, 0);
	f4assign (newfirststate, "On First");
	d4append (new_td);
	d4flush (new_td);
    }

    d4close (old_td);
    d4close (new_td);

    fprintf (stdout, "...finished td.dbf\n");

    return (0);
}

