/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include	<stdio.h>
#ifndef AXP
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<errno.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include	"ai.h"
#include	"ao.h"
#include	"di.h"
#include	"do.h"
#include	"mbbi.h"
#include	"mbbo.h"
#include	"pvmap.h"
#include	"td.h"

static CODE4	cb;
static DATA4	*new_ai;
static DATA4	*new_ao;
static DATA4	*new_di;
static DATA4	*new_do;
static DATA4	*new_mbbi;
static DATA4	*new_mbbo;
static DATA4	*new_pvmap;
static DATA4	*new_td;

static FIELD4	*ai_ref, *ai_bd_type;
static FIELD4	*ao_ref, *ao_bd_type;
static FIELD4	*bi_ref, *bi_bd_type;
static FIELD4	*bo_ref, *bo_bd_type;
static FIELD4	*mbbi_ref, *mbbi_bd_type;
static FIELD4	*mbbo_ref, *mbbo_bd_type;
static FIELD4	*td_ref, *td_bd_type;
static FIELD4	*pvmap_name, *pvmap_devtype, *pvmap_type;


static char		pathbuff[128];
static char		sysbuff[256];
static struct stat	statbuff;

static int  createSubdir (void);
static int  t50IOOpenFiles (void);
static void t50IOSaveFiles (void);
static int  t50IOConvert (void);

#define	TIS4000DB	"/tis4000db"
#define	DBCNVRT		"t50iocnvrt"
#define	IOCNVRTFILE	"t50iocnvrt.lst"

FILE			*iocnvrtFd;
int			outputList = 0;


int main(int argc, char *argv[])
{
    fprintf (stdout, "starting T50 IO conversion\n\n");
    d4init (&cb);

#if defined(WIN32) || defined(USECB63)
    cb.errGo = 0;
    cb.errOpen = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
#else
    cb.go_error = 0;
    cb.open_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
#endif WIN32 or USECB63

    if (createSubdir ())
	return (1);
    if (argc > 1)
    {
	sprintf (pathbuff, "%s/%s/%s", TIS4000DB, DBCNVRT, IOCNVRTFILE);
	if ( ( iocnvrtFd = fopen (pathbuff, "w+")) == NULL)
	{
	    fprintf (stdout, "t50iocnvrt err: failed to open %s\n", pathbuff);
	    return (0);
	}
	outputList = 1;
    }
    t50IOSaveFiles ();
    if (t50IOOpenFiles ())
    {
	d4close_all (&cb);
	if (outputList)
	    fclose (iocnvrtFd);
	return (0);
    }
    if (t50IOConvert ())
    {
	d4close_all (&cb);
	if (outputList)
	    fclose (iocnvrtFd);
	return (0);
    }
    d4close_all (&cb);
    if (outputList)
	fclose (iocnvrtFd);
    fprintf (stdout, "\nT50 IO conversion finished\n");
    return (0);
}


static int createSubdir (void)
{
    sprintf (pathbuff, "%s/%s", TIS4000DB, DBCNVRT);
    if (stat (pathbuff, &statbuff))
    {
	if (errno == ENOENT)
	{
	    sprintf (sysbuff, "mkdir %s", pathbuff);
	    system (sysbuff);
	    if (stat (pathbuff, &statbuff))
	    {
		fprintf (stdout, "t50iocnvrt err: failed to create %s\n", pathbuff);
		return (1);
	    }
	    else
	    {
		fprintf (stdout, "t50iocnvrt: %s created\n", pathbuff);
		return (0);
	    }
	}
	else
	{
	    fprintf (stdout, "t50iocnvrt err: stat problem (%d) on %s\n", errno, pathbuff);
	    return (1);
	}
    }
    else
    {
	fprintf (stdout, "t50iocnvrt err: %s already exists\n", pathbuff);
    }
    return (0);
}


static int  t50IOConvert (void)
{
    char	name[32];
    char	ptType[32];
    char	devType[32];
    char	bdType[32];
    char	ref[32];

    for (d4top (new_pvmap); !d4eof (new_pvmap); d4skip (new_pvmap, 1))
    {
	if (d4deleted (new_pvmap))
	    continue;
	strncpy (name, f4str (pvmap_name), sizeof (ptType));
	name[sizeof(name)-1] = 0;
	printf ("pt %-28.28s...\r", name);
	strncpy (devType, f4str (pvmap_devtype), sizeof (devType));
	devType[sizeof(devType)-1] = 0;
	strncpy (ptType, f4str (pvmap_type), sizeof (ptType));
	ptType[sizeof(ptType)-1] = 0;
	if ( (strncmp (ptType, "ai", 2)) == 0)
	{
	    if ( (d4seek (new_ai, name)) != 0)
	    {
		fprintf (stdout, "%s not found in ai\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (ai_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (ai_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (ai_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_ai);
		    }
		}
	    }
	}
	else if ( (strncmp (ptType, "ao", 2)) == 0)
	{
	    if ( (d4seek (new_ao, name)) != 0)
	    {
		fprintf (stdout, "%s not found in ao\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (ao_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (ao_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (ao_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_ao);
		    }
		}
	    }
	}
	else if ( (strncmp (ptType, "bi", 2)) == 0)
	{
	    if ( (d4seek (new_di, name)) != 0)
	    {
		fprintf (stdout, "%s not found in di\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (bi_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (bi_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (bi_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_di);
		    }
		}
	    }
	}
	else if ( (strncmp (ptType, "bo", 2)) == 0)
	{
	    if ( (d4seek (new_do, name)) != 0)
	    {
		fprintf (stdout, "%s not found in do\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (bo_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (bo_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (bo_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_do);
		    }
		}
	    }
	}
	else if ( (strncmp (ptType, "mbbi", 4)) == 0)
	{
	    if ( (d4seek (new_mbbi, name)) != 0)
	    {
		fprintf (stdout, "%s not found in mbbi\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (mbbi_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (mbbi_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (mbbi_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_mbbi);
		    }
		}
	    }
	}
	else if ( (strncmp (ptType, "mbbo", 4)) == 0)
	{
	    if ( (d4seek (new_mbbo, name)) != 0)
	    {
		fprintf (stdout, "%s not found in mbbo\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (mbbo_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (mbbo_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (mbbo_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_mbbo);
		    }
		}
	    }
	}
	else if ( (strncmp (ptType, "td", 2)) == 0)
	{
	    if ( (d4seek (new_td, name)) != 0)
	    {
		fprintf (stdout, "%s not found in td\n", name);
		continue;
	    }
	    else
	    {
		strncpy (ref, f4str (td_ref), sizeof (ref));
		ref[sizeof(ref)-1] = 0;
		if (ref[0] == ' ')
		    continue;
		strncpy (bdType, f4str (td_bd_type), sizeof (bdType));
		bdType[sizeof(bdType)-1] = 0;
		if ( ((strncmp (bdType, "REFERENCE", 8)) != 0) ||
		     ((strncmp (devType, "REFERENCE", 8)) != 0) )
		{
		    if (outputList)
		    {
			fprintf (iocnvrtFd, 
			"pt %-28.28s %-15.15s dev %-23.23s ref %-28.28s bd %-25.25s\n",
					     name, ptType, devType, ref, bdType);
		    }
		    else
		    {
			f4assign (pvmap_devtype, "REFERENCE");
			f4assign (td_bd_type, "REFERENCE");
			d4flush (new_pvmap);
			d4flush (new_td);
		    }
		}
	    }
	}
    }
    return (0);
}


static int t50IOOpenFiles (void)
{
    TAG4 *pvmap_tag;

    sprintf (pathbuff, "%s/%s/ai", TIS4000DB, DBCNVRT);
    if (!(new_ai = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((ai_ref = d4field (new_ai, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: ai REFERENCE not exist\n");
	return (1);
    }
    if ((ai_bd_type = d4field (new_ai, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: ai BOARDTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/ao", TIS4000DB, DBCNVRT);
    if (!(new_ao = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((ao_ref = d4field (new_ao, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: ao REFERENCE not exist\n");
	return (1);
    }
    if ((ao_bd_type = d4field (new_ao, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: ao BOARDTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/di", TIS4000DB, DBCNVRT);
    if (!(new_di = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((bi_ref = d4field (new_di, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: di REFERENCE not exist\n");
	return (1);
    }
    if ((bi_bd_type = d4field (new_di, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: di BOARDTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/do", TIS4000DB, DBCNVRT);
    if (!(new_do = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((bo_ref = d4field (new_do, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: do REFERENCE not exist\n");
	return (1);
    }
    if ((bo_bd_type = d4field (new_do, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: do BOARDTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/mbbi", TIS4000DB, DBCNVRT);
    if (!(new_mbbi = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((mbbi_ref = d4field (new_mbbi, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: mbbi REFERENCE not exist\n");
	return (1);
    }
    if ((mbbi_bd_type = d4field (new_mbbi, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: mbbi BOARDTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/mbbo", TIS4000DB, DBCNVRT);
    if (!(new_mbbo = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((mbbo_ref = d4field (new_mbbo, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: mbbo REFERENCE not exist\n");
	return (1);
    }
    if ((mbbo_bd_type = d4field (new_mbbo, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: mbbo BOARDTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/pvmap", TIS4000DB, DBCNVRT);
    if (!(new_pvmap = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    pvmap_tag = d4tag (new_pvmap, "NAME");
    d4tag_select (new_pvmap, pvmap_tag);
    if ((pvmap_name = d4field (new_pvmap, "NAME")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: pvmap NAME not exist\n");
	return (1);
    }
    if ((pvmap_type = d4field (new_pvmap, "TYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: pvmap TYPE not exist\n");
	return (1);
    }
    if ((pvmap_devtype = d4field (new_pvmap, "DEVTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: pvmap DEVTYPE not exist\n");
	return (1);
    }

    sprintf (pathbuff, "%s/%s/td", TIS4000DB, DBCNVRT);
    if (!(new_td = d4open (&cb, pathbuff)))
    {
	fprintf (stdout, "t50iocnvrt err: open error on %s\n", pathbuff);
	return (1);
    }
    if ((td_ref = d4field (new_td, "REFERENCE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: td REFERENCE not exist\n");
	return (1);
    }
    if ((td_bd_type = d4field (new_td, "BOARDTYPE")) == 0)
    {
	fprintf (stdout, "t50iocnvrt err: td BOARDTYPE not exist\n");
	return (1);
    }

    return (0);
}


static void t50IOSaveFiles (void)
{

    /* save the old dbase files */
    sprintf (sysbuff, "cp -p %s/ai.dbf %s/%s/ai.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/ai.mdx %s/%s/ai.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/ao.dbf %s/%s/ao.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/ao.mdx %s/%s/ao.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/di.dbf %s/%s/di.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/di.mdx %s/%s/di.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/do.dbf %s/%s/do.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/do.mdx %s/%s/do.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbi.dbf %s/%s/mbbi.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbi.mdx %s/%s/mbbi.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbo.dbf %s/%s/mbbo.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbo.mdx %s/%s/mbbo.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/td.dbf %s/%s/td.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/td.mdx %s/%s/td.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/pvmap.dbf %s/%s/pvmap.dbf.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/pvmap.mdx %s/%s/pvmap.mdx.sav", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    /* copy files to convert */
 
    sprintf (sysbuff, "cp -p %s/ai.dbf %s/%s/ai.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/ai.mdx %s/%s/ai.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/ao.dbf %s/%s/ao.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/ao.mdx %s/%s/ao.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/di.dbf %s/%s/di.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/di.mdx %s/%s/di.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/do.dbf %s/%s/do.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/do.mdx %s/%s/do.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbi.dbf %s/%s/mbbi.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbi.mdx %s/%s/mbbi.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbo.dbf %s/%s/mbbo.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/mbbo.mdx %s/%s/mbbo.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/td.dbf %s/%s/td.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/td.mdx %s/%s/td.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/pvmap.dbf %s/%s/pvmap.dbf", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    sprintf (sysbuff, "cp -p %s/pvmap.mdx %s/%s/pvmap.mdx", 
		       TIS4000DB, TIS4000DB, DBCNVRT);
    system (sysbuff);

    return;
}

