/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/*
  Main control structure for the Current Trend DPA (ctr:) along with some
  other stuff.

  GCT 15-Mar-1996

*/
#include <stdio.h>
#include <dbDefs.h>
#include <dataTypes.h>
#include <signal.h>
#include <DBDefines.h>
#include <DBProto.h>
#include <string.h>
#include <sys/time.h>
#include <cadef.h>
#include <tsDefs.h>
#include <paramStructs.h>
#include <syslog.h>
#include <avl.h>
/*
  This typedef describes the data node for each point on current
  trend. Each point that is destined for current trend will have a flag
  set in the AR_REDUC file along with the collection rate. The
  TREND_ENTRY structure will contain the time of the last value as well
  as a pointer to the DPA handle. Memory will be allocated for the 500
  value sliding buffer only as needed.
*/

typedef struct
{
  char	     ctName[PVNAME_SZ+FLDNAME_SZ+1];	/* name of trended point */
  TS_STAMP   ctTime;				/* time of most recent sample */
  int	     ctCounter;				/* update counter */
  int	     ctCounterReset;			/* counter reset value */
  int        ctValid;				/* set to zero if point taken off trend */
  void	     *ctCurrentHandle;			/* send data to this DPA handle...one value*/
  void       *ctHistoryHandle;			/* send history...500 values */
  float	     *ctDataPtr;			/* pointer to stored trend data */
  float	     ctRecentSample;			/* places to hold most recent sample from CA */
  float      ctDisplayHigh;			/* from appropriate file */
  float      ctDisplayLow;
} TREND_ENTRY,*TREND_ENTRY_PTR;
/*
  The following is the anchor point for an AVL tree. This tree is used to keep track
  of the TREND_ENTRY structures as they are created. By using the tree in this manner
  CTR will be able to quickly access those point it is interested in among all the
  one comming from CA.
*/
extern  AVL_TREE     *ctTree;
extern  int caFd,dpaFd,maxFd,doConnect;
extern  int debugLevel;
extern  fd_set myFds,saveFds;
extern  char appName[];
extern  DB_TABLE_PTR groupTable;
/*
  prototypes for ctr functions
*/
void  ctrOpen(char *,void *);
void  ctrWrite(void *,short,void*,int);
void  ctrRead(void *handle);
void  ctrClose(void *);
void  sigHandler(int);
void  ctrInitCTR();
int   ctrUpdateSamples(char *,TS_STAMP *,double);
void  ctrDataCallback(char *,short,short,void*);
void  ctrConnectCallback(char *,short);
void  ctrReconnectCallback(void *,int,int);
int   ctrEntryCompare(TREND_ENTRY_PTR,TREND_ENTRY_PTR);
void  ctrCheckAndSend(void *,void *);
void  ctrFreeData(void *,void *);
/*
  defines
*/
#define  TREND_ENTRY_SIZE sizeof(TREND_ENTRY)
#define  LOG_OPTIONS (LOG_ALERT|LOG_LOCAL1)
#define  ALWAYS_MASTER 1
#define  TREND_FILE "ctrend"
#define  TREND_GROUP_FILE "trends"
#define  DB_PATH "/tis4000db"
#define  ERR_LOG_NAME "ctr"
