/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/*
  This is the Current Trend DPA program. In the main function, CTR
  will search the AR_REDUC file for points on current trend. An entry will
  be made in an AVL tree. when a channel requests a connection from CA,
  the tree will be searched and updated with the DPA handle. The point
  name is from the trend group file (trends.dbf). The trend group name will
  follow the same format as for the TRN program. 
*/
#include "ctr.h"

AVL_TREE  *ctTree;
char	appName[100];
char	currentGroup[PVNAME_SZ+FLDNAME_SZ+1];
void	ctrOpen(),ctrRead(),ctrWrite(),sigHandler(),ctrClose();
void	*memberHandle;
void	*updateHandle;
int	signals[] = {SIGPIPE,SIGINT,SIGHUP,0};
struct	dbdatabase DB_Database;
DB_TABLE_PTR	groupTable,arReduc;
short		currentMember = 0;
int dpaFd,caFd,maxFd = -1;
fd_set  myFds,saveFds;
int  doConnect,debugLevel;

main(int argc,char *argv[])
{
char	*slash;
int		status,stat;
int  caInitialized;
int  running;
int  bailingOut;
struct timeval timeout;

slash = (char *)strrchr(argv[0],'/');
if(slash)
  {
    slash++;
    strcpy(appName,slash);
  }
else
  strcpy(appName,argv[0]);
openlog(ERR_LOG_NAME,LOG_PID,LOG_LOCAL1);
ctTree = avl_init(ctrEntryCompare);		/* init the AVL tree library */
/*
	Init and map to database file for trend group
	info (trends).
*/
DB_InitDatabase(ERR_LOG_NAME);
DB_MapDatabase(DB_PATH);
groupTable = DB_MapTable(TREND_GROUP_FILE);
/*
	Init DPA sub-system
*/
if((dpaFd = dpaInit(appName,ctrOpen,ctrRead,ctrWrite,0,sigHandler,signals)) < 0)
	exit();
FD_SET(dpaFd,&myFds);
if(dpaFd > maxFd) maxFd = dpaFd;
dpaSetResMode(ALWAYS_MASTER);
/*
  Initialize the point tree with points on Current trend.
*/
ctrInitCTR();
/*
  Wait for an incoming message to process or until a timeout occurs
*/
caInitialized = 0;
doConnect = 1;
running = 1;
bailingOut = 0;
timeout.tv_usec = 0;
timeout.tv_sec = 1;
bcopy((char *)&myFds,(char *)&saveFds,sizeof(fd_set));
while (running)
  {
/* Check if a reconnect is necessary */
  if (doConnect)
    {
      if (caInitialized)
	ca_task_exit ();
      if (ca_task_initialize () != ECA_NORMAL)
	{
	  syslog (LOG_OPTIONS, "Can't init CA");
	  exit (-1);
	}
       caInitialized = 1;
       ca_add_fd_registration (ctrReconnectCallback, NULL);
       if (ca_register_as_archiver (ctrDataCallback, ctrConnectCallback) != ECA_NORMAL)
	 {
	 syslog (LOG_OPTIONS, "Can't register as trend client");
	 exit (-1);
	 }
       doConnect = 0;
    }
/*
  Wait for any incoming messages from either CA or hsrvTask
*/
  ca_pend_io(0);		/* get data from incomming trend message */
  status = select(maxFd+1,&myFds,NULL,NULL,&timeout);
  if(status == 0)
    {
      avl_traverse(ctTree,INORDER,ctrCheckAndSend,(void *)0);
    }
  else if(status < 0)
    {
      syslog (LOG_OPTIONS, "Error from select %m");
    }
  else
    {
      if(FD_ISSET(dpaFd,&myFds))
	stat = dpaEvent(0);
      if(FD_ISSET(caFd,&myFds))
	ca_pend_io(0);		/* get data from incomming trend message */
      avl_traverse(ctTree,INORDER,ctrCheckAndSend,(void *)0);
    }
  bcopy((char *)&saveFds,(char *)&myFds,sizeof(fd_set));
  }
}
void	sigHandler(int whichone)
{
switch(whichone)
	{
	case SIGPIPE:
		printf("\nPipes're broke, Hon\n");
		break;
	case SIGINT:
	case SIGHUP:
		printf("\nBye-bye...\n");
		break;
	default:
		printf("\nUnknown signal %d\n",whichone);
	}
avl_traverse(ctTree,INORDER,ctrFreeData,(void *)0);
avl_empty(ctTree,avl_freenode);
DB_UnMapDatabase(0);
exit(1);
}
int  ctrEntryCompare(TREND_ENTRY_PTR e1,TREND_ENTRY_PTR e2)
{
return(strcmp(e1->ctName,e2->ctName));
}
/*
  function to read the CTREND file and scan for Current trend
  points. Put the points in an AVL tree.
*/
void  ctrInitCTR()
{
  DB_TABLE_PTR ar;
  DB_RECORD_PTR rec;
  DB_FIELD_PTR  name,rate;
  TREND_ENTRY  entry,*pEntry,*p1;
  int type;
  char  entryName[PVNAME_SZ+FLDNAME_SZ+1],entryRate[6];

  ar = DB_MapTable(TREND_FILE);
  name = DB_MapField(ar,"NAME");
  rate = DB_MapField(ar,"RATE");
  rec = DB_Top(ar);
  while(rec)
    {
      DB_GetFieldValue(rec,name,entryName,&type);
      if(!(pEntry = avl_newnode(TREND_ENTRY_SIZE)))
	syslog(LOG_OPTIONS,"No memory for tree insert...%s",entryName);
      else
	{
	  strcpy(&pEntry->ctName[0],entryName);
	  if((p1 = avl_insert(ctTree,pEntry)) != NULL)
	    {
	      avl_freenode(pEntry);
	    }
	  else
	    {
	      pEntry->ctDataPtr = (float *)calloc(500,sizeof(float));
	      DB_GetFieldValue(rec,rate,entryRate,&type);
	      pEntry->ctCounter = atoi(entryRate);     	/* 10 seconds for now */
	      pEntry->ctCounterReset = atoi(entryRate);	/* should get this from file */
	      pEntry->ctDisplayHigh = 100.0;		/* and these too... */
	      pEntry->ctDisplayLow = 0.0;
	    }
	}
      rec = DB_SeekNext(ar);
    }
DB_UnMapTable(ar);
}
/*
  This functions is called once per second. It checks to see if any data
  should be sent to DM for this node.
*/
void ctrCheckAndSend(void *params,void *node)
{
TREND_ENTRY_PTR  tp = (TREND_ENTRY_PTR)node;
if(--tp->ctCounter <= 0)
  {
    tp->ctCounter = tp->ctCounterReset;		/* reset counter for next time */
    if(tp->ctHistoryHandle)			/* write only to valid (non-zero) handle */
      {
	bcopy((char *)(tp->ctDataPtr+1),(char *)tp->ctDataPtr,499*sizeof(float));  /* Shift to make room for recent value */
	tp->ctDataPtr[499] = tp->ctRecentSample;
	dpaWrite(tp->ctHistoryHandle,(float *)&tp->ctDataPtr[0],sizeof(float),AGNT_VALUE,0,0);
      }
    if(tp->ctCurrentHandle)			/* write only to valid (non-zero) handle */
      {
	dpaWrite(tp->ctCurrentHandle,(float *)&tp->ctRecentSample,sizeof(float),AGNT_VALUE,0,0);
      }

  }
}

/*
  This function is called when the program exits. It is used to free
  the memory allocated during startup
*/
void ctrFreeData(void *params,void *node)
{
TREND_ENTRY_PTR  tp = (TREND_ENTRY_PTR)node;
free(tp->ctDataPtr);
}
