#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vdictHEADER
#include vlimitsHEADER
#include vmemHEADER
#include vnumHEADER
#include vlistitemHEADER
#include vdateHEADER
#include vspinnerHEADER
#include vlistHEADER
#include vsliderHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbFldTypes.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "dbRecDes.h"
#include "dbRecType.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuUtils.hxx"
#include "vuApiCfgDialogClass.hxx"

static int apiNameFilter (filteredText *text, vtextSelection *sel,
			  const vchar **s, long *len);


/***********************************************************************
*
*	API Config Dialog Class
*
***********************************************************************/

vkindTYPED_FULL_DEFINITION (vuApiCfgDialogClass, vdialog, 
					"vuApiCfgDialogClass");

/**********************************************************************
*
*	vuApiCfgDialogClass override for constructor
*
**********************************************************************/

void vuApiCfgDialogClass::LoadInit (vresource res)
{
    dialogModified = vFALSE;

    tablesList = vuApiCfgTablesListClass::RequestSubLoad (this,
					vunames_vuApiCfgTablesList);
    valuesList = vuApiCfgValuesListClass::RequestSubLoad (this,
					vunames_vuApiCfgValuesList);
    vcontainer *namecontainer = vcontainer::RequestSubLoad (this,
					vunames_vuApiCfgNameContainer);
    tableName = filteredTextitem::RequestSubLoad (namecontainer,
					vunames_vuApiCfgName);
    vcontainer *tempcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuApiCfgTempContainer);
    temperature = filteredTextitem::RequestSubLoad (tempcontainer,
					vunames_vuApiCfgTemperature);
    vcontainer *gravcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuApiCfgGravityContainer);
    gravity = filteredTextitem::RequestSubLoad (gravcontainer,
					vunames_vuApiCfgGravity);
    vcontainer *factcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuApiCfgFactorContainer);
    factor = filteredTextitem::RequestSubLoad (factcontainer,
					vunames_vuApiCfgFactor);

    vcontainer *buttoncontainer = vcontainer::RequestSubLoad (this,
					vunames_vuApiCfgButtonContainer);
    applyButton = vuApiCfgApplyButtonClass::RequestSubLoad (buttoncontainer,
					vunames_vuApiCfgApplyButton);
    dismissButton = vuApiCfgDismissButtonClass::RequestSubLoad (buttoncontainer,
					vunames_vuApiCfgDismissButton);
    deleteButton = vuApiCfgDeleteButtonClass::RequestSubLoad (buttoncontainer,
					vunames_vuApiCfgDeleteButton);
    compileButton = vuApiCfgCompileButtonClass::RequestSubLoad (buttoncontainer,
					vunames_vuApiCfgCompileButton);
    addButton = vuApiCfgAddButtonClass::RequestSubLoad (buttoncontainer,
					vunames_vuApiCfgAddButton);
    removeButton = vuApiCfgRemoveButtonClass::RequestSubLoad (buttoncontainer,
					vunames_vuApiCfgRemoveButton);

    vdialog::LoadInit (res);

    tablesList -> storeApiCfgDialog (this);
    valuesList -> storeApiCfgDialog (this);
    applyButton -> storeApiCfgDialog (this);
    dismissButton -> storeApiCfgDialog (this);
    deleteButton -> storeApiCfgDialog (this);
    compileButton -> storeApiCfgDialog (this);
    addButton -> storeApiCfgDialog (this);
    removeButton -> storeApiCfgDialog (this);

    // set ptr to modified flag
    tableName -> SetTextModifiedPtr (&this -> dialogModified);
    temperature -> SetTextModifiedPtr (&this -> dialogModified);
    gravity -> SetTextModifiedPtr (&this -> dialogModified);
    factor -> SetTextModifiedPtr (&this -> dialogModified);

    // set ptrs to filter functions
    tableName -> SetItemFilterFunc (apiNameFilter);
    temperature -> SetItemFilterFunc (floatFilter);
    gravity -> SetItemFilterFunc (floatFilter);
    factor -> SetItemFilterFunc (floatFilter);

    valuesList -> setCalcColumnWidths (vTRUE);
}


/*********************************************************************
*
*	vuApiCfgDialogClass window notify function
*
*********************************************************************/

void vuApiCfgDialogClass::ObserveWindow (vevent *event)
{
    if ( event -> GetType () == veventWINDOW_STATE && event -> IsClose () )
    {
	int result = VU_TRUE;
	if ( dialogModified == vTRUE)
	{
	    result = vuMessageDialogCenter (this, VU_MOD_DISMISS_MSG); 
	}
	if (result == VU_TRUE)
	{
	    Close ();
	    vevent::StopProcessing ();
	}
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}


/*********************************************************************
*
*	vuApiCfgDialogClass destroy function
*
*********************************************************************/

void vuApiCfgDialogClass::Destroy (void)
{
    if (IsOpen ())
    {
	Close ();
	vevent::StopProcessing ();
    }
    // destroy tables list
    vliststr *tables = (vliststr *)tablesList -> GetList ();
    if (tables != NULL)
    {
	tables -> Destroy ();
    }
    // destroy values list
    vliststr *values = (vliststr *)valuesList -> GetList ();
    if (values != NULL)
    {
	values -> Destroy ();
    }

    vdialog::Destroy ();
}


/********************************************************************
*
*	vuApiCfgDialogClass open function
*
********************************************************************/

void vuApiCfgDialogClass::vuApiCfgOpen ( vuMainDialogClass *mnDialog)
{
    // store for later use
    mainDialog = mnDialog;
    dataVuApplication *vuApp = mainDialog -> getApplication ();
    vuDatabaseClass *db = vuApp -> getDatabaseClass ();

    // map to these database fields for later use
    nameField    = DB_MapField (db -> vuApiTable, FLD_NAME);
    tempField    = DB_MapField (db -> vuApiTable, "TEMP");
    gravityField = DB_MapField (db -> vuApiTable, "GRAVITY");
    factorField  = DB_MapField (db -> vuApiTable, "FACTOR");
    //open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog after dialog is closed
    Place (vwindow::GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
    vdialog::Open ();
    MakeTopMostDialog (this, NULL);

    Inval (vwindowINVAL_UPDATE);

    // build the tables name list
    tablesList -> buildTablesList ();

    //done here to get the tablesList::ObserveSelect to fire and
    // cause the valuesList,tableName,temperature, gravity,factor to update
    vliststr *tblList = (vliststr *)tablesList -> GetList ();
    tablesList -> setLastRowSelected (-1);
    if (tblList != NULL)
    {
	int rowCount = tblList -> GetRowCount ();
	if (rowCount > 0)
	{
	    vlistSelection *listSelection = tablesList -> GetSelection ();
	    tblList -> SelectRow (listSelection, 0);
	}
    }
    vevent::Process ();
}


/******************************************************************
*
*	api cfg tables list class
*
******************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgTablesListClass, vlistview,
					"vuApiCfgTablesListClass")

/******************************************************************
*
*	api cfg tables list class observe select
*	check if data has been mod'ed, do they realy want to chg
*	tables ?
*
******************************************************************/

void vuApiCfgTablesListClass::ObserveSelect (int state)
{
    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	vlistSelection *listSelection = GetSelection ();
	vliststr *tblList = (vliststr *)GetList ();
	int row, column;
	tblList -> GetSelectedCell (listSelection, &row, &column);
	if (row != vlistNO_CELLS_SELECTED)
	{
	    if ( (lastRowSelected >= 0) &&
		 ( (apiCfgDialog -> getDialogModify ()) == vTRUE) )
	    {
		int result = vuMessageDialogCenter (apiCfgDialog, VU_MOD_DISMISS_MSG); 
		if (result == VU_FALSE)
		{
		    tblList -> SelectRow (listSelection, lastRowSelected);
		    return;
		}
	    }
	    vchar *table;
	    table = (vchar *)(tblList -> GetCellValue (row, 0));
	    apiCfgDialog -> tableName -> SetText (table);
	    apiCfgDialog -> valuesList -> buildValuesList ();
	    apiCfgDialog -> setDialogModify (vFALSE);
	    setLastRowSelected (row);
	}
    }
}


/******************************************************************
*
*	api cfg tables list class, build list function
*
******************************************************************/

void vuApiCfgTablesListClass::buildTablesList (void)
{
    vliststr *list = (vliststr *)(GetList ());

    // destroy old list if there is one
    if (list != NULL)
    {
	SetList (NULL);
	list -> Destroy ();
    }
    // new list with 3 columns
    list = new vliststr;

    vuMainDialogClass *mainDialog = apiCfgDialog -> getMainDialog ();
    dataVuApplication *vuApp = mainDialog -> getApplication ();
    vuDatabaseClass *db = vuApp -> getDatabaseClass ();

    int row = 0;
    int dataType;
    char tableName [API_STRAP_NAME_SZ + 1];
    char lastTableName [API_STRAP_NAME_SZ + 1];
    memset ((void *)tableName, 0, sizeof (tableName));
    memset ((void *)lastTableName, 0, sizeof (tableName));

    // change cursor to block/wait
    apiCfgDialog -> Block ();

    DB_RECORD_PTR rec = DB_Top (db -> vuApiTable);
    while (rec)
    {
	DB_GetFieldValue (rec, apiCfgDialog -> nameField, tableName, &dataType);
	if ( (strncmp (tableName, lastTableName, API_STRAP_NAME_SZ)) != 0)
	{
	    strncpy (lastTableName, tableName, API_STRAP_NAME_SZ);
	    lastTableName[API_STRAP_NAME_SZ] = 0;
	    list -> SetCellValueScribed (row, 0, vcharScribeSystem (tableName));
	    row++;
	}
	rec = DB_SeekNext (db -> vuApiTable);
    }
    if (row > 0)
    {
	// tell listview about new list
	SetList (list);
	vlistSelection *listSelection = GetSelection ();
	list -> SetSelectMethod (listSelection, vlistONE_ONLY_SELECTION);
    }
    else
    {
	list -> Destroy ();
    }

    // return cursor to normal
    apiCfgDialog -> Unblock ();
}


/*******************************************************************
*
*	api cfg values list class
*
*******************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgValuesListClass, vlistview,
					"vuApiCfgValuesListClass");

/******************************************************************
*
*	api cfg values list class observe select
*
******************************************************************/

void vuApiCfgValuesListClass::ObserveSelect (int state)
{
    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	vlistSelection *listSelection = GetSelection ();
	vliststr *valList = (vliststr *)GetList ();
	int row, column;
	valList -> GetSelectedCell (listSelection, &row, &column);
	if (row != vlistNO_CELLS_SELECTED)
	{
	    vchar *temp, *grav, *fact;
	    temp = (vchar *)(valList -> GetCellValue (row, 0));
	    grav = (vchar *)(valList -> GetCellValue (row, 1));
	    fact = (vchar *)(valList -> GetCellValue (row, 2));
	    
	    apiCfgDialog -> temperature -> SetText (temp);
	    apiCfgDialog -> gravity -> SetText (grav);
	    apiCfgDialog -> factor -> SetText (fact);

	    apiCfgDialog -> setDialogModify (vFALSE);
	}
    }
}


/*******************************************************************
*
*	vuApiCfgValuesList build function
*
*******************************************************************/

void vuApiCfgValuesListClass::buildValuesList (void)
{
    vliststr *list = (vliststr *)(GetList ());

    // destroy old list if there is one
    if (list != NULL)
    {
	SetList (NULL);
	list -> Destroy ();
    }
    // new list with 3 columns
    list = new vliststr;

    vuMainDialogClass *mainDialog = apiCfgDialog -> getMainDialog ();
    dataVuApplication *vuApp = mainDialog -> getApplication ();
    vuDatabaseClass *db = vuApp -> getDatabaseClass ();
    char name[API_STRAP_NAME_SZ + 1];
    char lastName[API_STRAP_NAME_SZ + 1];
    char apiName[API_STRAP_NAME_SZ + 1];
    double temp, grav, fact;
    int dataType;

    // get table name fro the table name field
    vstr *tableName = apiCfgDialog -> tableName -> GetTextAsString ();
    vcharExportBounded (vcharScribeSystemExport (tableName),
			name,
			API_STRAP_NAME_SZ);
    vstrDestroy (tableName);
    name[API_STRAP_NAME_SZ] = 0;
    strcpy (lastName, name);
    sprintf (name, "%-*.*s\0", API_STRAP_NAME_SZ, API_STRAP_NAME_SZ, lastName);

    // change cursor to block/wait
    apiCfgDialog -> Block ();
    DB_RECORD_PTR rec = DB_Seek (db -> vuApiTable, name);
    int row = 0;
    // look for all records in apitbl.dbf with this name
    while (rec)
    {
	DB_GetFieldValue (rec, apiCfgDialog -> tempField, &temp, &dataType);
	DB_GetFieldValue (rec, apiCfgDialog -> gravityField, &grav, &dataType);
	DB_GetFieldValue (rec, apiCfgDialog -> factorField, &fact, &dataType);
	list -> SetCellValueScribed (row, 0, vnum::ScribeFloat (temp));
	list -> SetCellValueScribed (row, 1, vnum::ScribeFloat (grav));
	list -> SetCellValueScribed (row, 2, vnum::ScribeFloat (fact));
	row++;
	rec = DB_SeekNext (db -> vuApiTable);
	DB_GetFieldValue (rec, apiCfgDialog -> nameField, apiName, &dataType);
	if ( (strcmp (lastName,apiName)) != 0)
	{
	    break;
	}
	else
	{
	    strcpy (lastName, apiName);
	}
    }
    if (row > 0)
    {
	// tell listview about new list
	SetList (list);
	if (calcColumnWidths == vTRUE)
	{
	    calcColumnWidths = vFALSE;
	    vlistsize *size = GetSizes ();
	    SetColumnWidth (vlistviewALL_COLUMNS, size -> GetColumnWidth (0) / 3);
	}
	vlistSelection *listSelection = GetSelection ();
	list -> SetSelectMethod (listSelection, vlistONE_ONLY_SELECTION);
	list -> SetExtendRowSelection (listSelection, vTRUE);
	list -> SelectRow (listSelection, 0);
    }
    else
    {
	list -> Destroy ();
    }

    // return cursor to normal
    apiCfgDialog -> Unblock ();
}


/*****************************************************************
*
*	api cfg apply button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgApplyButtonClass, vbutton,
					"vuApiCfgApplyButtonClass")

/*****************************************************************
*
*	vuApiCfgApplyButtonClass observer
*
*****************************************************************/

void vuApiCfgApplyButtonClass::ObserveDialogItem (vevent *event)
{
    printf ("apply button\n");
}


/*****************************************************************
*
*	api cfg dismiss button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgDismissButtonClass, vbutton,
					"vuApiCfgDismissButtonClass")

/*****************************************************************
*
*	vuApiCfgDismissButtonClass observer
*
*****************************************************************/

void vuApiCfgDismissButtonClass::ObserveDialogItem (vevent *event)
{
    int result = VU_TRUE;
    if ( (apiCfgDialog -> getDialogModify ()) == vTRUE)
    {
	result = vuMessageDialogCenter (apiCfgDialog, VU_MOD_DISMISS_MSG); 
    }
    if (result == VU_TRUE)
    {
	apiCfgDialog -> Close ();
	vevent::StopProcessing ();
    }
}


/*****************************************************************
*
*	api cfg delete button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgDeleteButtonClass, vbutton,
					"vuApiCfgDeleteButtonClass")

/*****************************************************************
*
*	vuApiCfgDeleteButtonClass observer
*
*****************************************************************/

void vuApiCfgDeleteButtonClass::ObserveDialogItem (vevent *event)
{
	printf ("delete button\n");
}


/*****************************************************************
*
*	api cfg compile button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgCompileButtonClass, vbutton,
					"vuApiCfgCompileButtonClass")

/*****************************************************************
*
*	vuApiCfgCompileButtonClass observer
*
*****************************************************************/

void vuApiCfgCompileButtonClass::ObserveDialogItem (vevent *event)
{
	printf ("compile button\n");
}


/*****************************************************************
*
*	api cfg add button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgAddButtonClass, vbutton,
					"vuApiCfgAddButtonClass")

/*****************************************************************
*
*	vuApiCfgAddButtonClass observer
*
*****************************************************************/

void vuApiCfgAddButtonClass::ObserveDialogItem (vevent *event)
{
	printf ("add button\n");
}


/*****************************************************************
*
*	api cfg remove button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuApiCfgRemoveButtonClass, vbutton,
					"vuApiCfgRemoveButtonClass")

/*****************************************************************
*
*	vuApiCfgRemoveButtonClass observer
*
*****************************************************************/

void vuApiCfgRemoveButtonClass::ObserveDialogItem (vevent *event)
{
	printf ("remove button\n");
}

/*****************************************************************
*
*	filter function for the API table name field
*
*****************************************************************/

static int apiNameFilter (filteredText *text, vtextSelection *sel,
			  const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, API_STRAP_NAME_SZ));
}
