#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vdictHEADER
#include vlimitsHEADER
#include vmemHEADER
#include vdateHEADER
#include vlistHEADER
#include vlistitemHEADER
#include vspinnerHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbFldTypes.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuUtils.hxx"
#include "vuArchInfoDialogClass.hxx"

static vchar *archTypes [] = 
	{
	(vchar *)("Trend"),
	(vchar *)("Alarm"),
	(vchar *)("Operator Tracking"),
	0
	};

#define	VU_ARCH_ENABLED		"enabled"
#define	VU_ARCH_DISABLED	"disabled"


/***********************************************************************
*
*	Archive Info Dialog Class
*
***********************************************************************/

vkindTYPED_FULL_DEFINITION (vuArchInfoDialogClass, vdialog, 
					"vuArchInfoDialogClass");

/**********************************************************************
*
*	vuArchInfoDialogClass override for constructor
*
**********************************************************************/

void vuArchInfoDialogClass::LoadInit (vresource res)
{
    modified = vFALSE;

    vcontainer *atcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuArchInfoArchiveContainer);
    archiveTime = filteredTextitem::RequestSubLoad (atcontainer,
					vunames_vuArchInfoArchive);

    enableArchive = vuCheckBoxClass::RequestSubLoad (this,
					vunames_vuArchInfoEnableArchive);

    vcontainer *ctcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuArchInfoCleanContainer);
    cleanTime = filteredTextitem::RequestSubLoad (ctcontainer,
					vunames_vuArchInfoClean);

    enableClean = vuCheckBoxClass::RequestSubLoad (this,
					vunames_vuArchInfoEnableClean);

    vcontainer *fcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuArchInfoArchFileContainer);
    archiveFile = filteredTextitem::RequestSubLoad (fcontainer,
					vunames_vuArchInfoArchFile);

    vcontainer *tcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuArchInfoTimeContainer);
    dailyTime = spinnerClass::RequestSubLoad (tcontainer,
					vunames_vuArchInfoTime);

    vcontainer *bcontainer = vcontainer::RequestSubLoad (this,
					vunames_vuArchInfoButtonContainer);
    applyButton = vuArchInfoApplyButtonClass::RequestSubLoad (bcontainer,
					vunames_vuArchInfoApplyButton);
    dismissButton = vuArchInfoDismissButtonClass::RequestSubLoad (bcontainer,
					vunames_vuArchInfoDismissButton);
		
    vdialog::LoadInit (res);

    applyButton -> storeArchInfoDialog (this);
    dismissButton -> storeArchInfoDialog (this);

    // set filters 
    archiveTime -> SetItemFilterFunc (numericFilter);
    cleanTime -> SetItemFilterFunc (numericFilter);

    // set pointer to the modified flag
    archiveTime -> SetTextModifiedPtr (&this -> modified);
    enableArchive -> vuSetModifiedPtr (&this -> modified);
    cleanTime -> SetTextModifiedPtr (&this-> modified);
    enableClean -> vuSetModifiedPtr (&this -> modified);
    archiveFile -> SetTextModifiedPtr (&this -> modified);
    dailyTime -> SetModifiedPtr (&this -> modified);
    if(vuReadOnly == vTRUE)
    {
	applyButton->Disable();
    }
}


/*********************************************************************
*
*	vuArchInfoDialogClass window notify function
*
*********************************************************************/

void vuArchInfoDialogClass::ObserveWindow (vevent *event)
{
    if ( event -> GetType () == veventWINDOW_STATE && event -> IsClose () )
    {
	int result = VU_TRUE;

	if (modified)
	{
	    result = vuMessageDialogCenter (this, VU_MOD_DISMISS_MSG);
	}
	if (result == VU_TRUE)
	{
	    Close ();
	    vevent::StopProcessing ();
	}
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}


/*********************************************************************
*
*	vuArchInfoDialogClass destroy function
*
*********************************************************************/

void vuArchInfoDialogClass::Destroy (void)
{
    if (IsOpen ())
    {
	Close ();
	vevent::StopProcessing ();
    }

    if (archInfoTitle != NULL)
    {
	vstrDestroy (archInfoTitle);
    }
    vdialog::Destroy ();
}


/********************************************************************
*
*	vuArchInfoDialogClass open function
*
********************************************************************/

void vuArchInfoDialogClass::vuArchInfoOpen (int archiveType,
					    vuMainDialogClass *mainDlg)
{
    // store for later use
    mainDialog = mainDlg;
    archType = archiveType;

    vresource titlesRes = mainDialog -> getTitlesResource ();

    switch (archType)
    {
	case VU_EDIT_ARCH_TREND_INFO:
	    if (getenv ("ORACLE_HOME"))
	    {
		system ("/tis4000/tasks/archConfig");
		return;
	    }
	    archInfoTitle = vresourceGetString (titlesRes, 
					vunames_vuArchInfoTrendTitle);
	    break;

	case VU_EDIT_ARCH_ALARM_INFO:
	    archInfoTitle = vresourceGetString (titlesRes, 
					vunames_vuArchInfoAlarmTitle);
	    break;

	case VU_EDIT_ARCH_OPER_INFO:
	    archInfoTitle = vresourceGetString (titlesRes, 
					vunames_vuArchInfoOperTitle);
	    break;
    }
    SetTitle (archInfoTitle);


    // get the data for all 3

    dataVuApplication *vuApp = mainDialog -> getApplication ();
    vuDatabaseClass *db = vuApp -> getDatabaseClass ();
    char *dbPath = db -> getDbasePath ();
    char archInfoFile[VU_MAX_PATH];
    FILE *fp;

    vmemSet ((void *)(&archInfo[0]), 0, (sizeof (struct archinfo)) * VU_MAX_ARCH_EDIT);

    sprintf (archInfoFile, "%s/archiveInfo\0", dbPath);

    // read the entire file regardless of archive type
    if ( (fp = fopen (archInfoFile, "r")) != NULL)
    {
	int i;
	char *readBack;
	char *cPtr;
	for (i = 0; i < VU_MAX_ARCH_EDIT; i++)
	{
	    readBack = fgets (archInfo[i].archFile, sizeof (archInfo[i].archFile), fp);
	    if (readBack != NULL)
	    {
		if ( (cPtr = strrchr (archInfo[i].archFile, '\n')) != NULL)
		{
		    *cPtr = 0;
		}
	    }
	    else
	    {
		break;
	    }	
	    readBack = fgets (archInfo[i].archTime, sizeof (archInfo[i].archTime), fp);
	    if (readBack != NULL)
	    {
		if ( (cPtr = strrchr (archInfo[i].archTime, '\n')) != NULL)
		{
		    *cPtr = 0;
		}
	    }
	    else
	    {
		break;
	    }	
	    readBack = fgets (archInfo[i].cleanTime, sizeof (archInfo[i].cleanTime), fp);
	    if (readBack != NULL)
	    {
		if ( (cPtr = strrchr (archInfo[i].cleanTime, '\n')) != NULL)
		{
		    *cPtr = 0;
		}
	    }
	    else
	    {
		break;
	    }	
	    readBack = fgets (archInfo[i].dailyTime, sizeof (archInfo[i].dailyTime), fp);
	    if (readBack != NULL)
	    {
		if ( (cPtr = strrchr (archInfo[i].dailyTime, '\n')) != NULL)
		{
		    *cPtr = 0;
		}
	    }
	    else
	    {
		break;
	    }	
	    readBack = fgets (archInfo[i].archEnable, sizeof (archInfo[i].archEnable), fp);
	    if (readBack != NULL)
	    {
		if ( (cPtr = strrchr (archInfo[i].archEnable, '\n')) != NULL)
		{
		    *cPtr = 0;
		}
	    }
	    else
	    {
		break;
	    }	
	    readBack = fgets (archInfo[i].cleanEnable, sizeof (archInfo[i].cleanEnable), fp);
	    if (readBack != NULL)
	    {
		if ( (cPtr = strrchr (archInfo[i].cleanEnable, '\n')) != NULL)
		{
		    *cPtr = 0;
		}
	    }
	    else
	    {
		break;
	    }	
	}
	fclose (fp);
	vuLoadArchInfo ();
    }
    else
	vuClearArchInfo ();

    //open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog after dialgo is closed
    Place (vwindow::GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
    vdialog::Open ();
    MakeTopMostDialog (this, NULL);
    vevent::Process ();
}


/********************************************************************
*
*	vuArchInfoDialogClass clear dialog function
*
********************************************************************/

void vuArchInfoDialogClass::vuClearArchInfo (void)
{
    archiveTime -> SetTextScribed (vcharScribeSystem (""));
    enableArchive -> SetValue (0);
    cleanTime -> SetTextScribed (vcharScribeSystem (""));
    enableClean -> SetValue (0);
    archiveFile -> SetTextScribed (vcharScribeSystem (""));
    dailyTime -> SetValue (dailyTime ->GetMin());
    modified = vFALSE;
}


/********************************************************************
*
*	vuArchInfoDialogClass load dialog function
*
********************************************************************/

void vuArchInfoDialogClass::vuLoadArchInfo (void)
{
    vstr *text;

    text = vstrCloneScribed (vcharScribeSystem (archInfo[archType].archTime));
    archiveTime -> SetText (text);
    vstrDestroy (text);

    text = vstrCloneScribed (vcharScribeSystem (archInfo[archType].archEnable));
    if (vcharLength (text) == 0)
    {
	enableArchive -> SetValue (0);
    }
    else
    {
	if (text[0] == 'd')
	    enableArchive -> SetValue (0);
	else
	    enableArchive -> SetValue (1);
    }
    vstrDestroy (text);

    text = vstrCloneScribed (vcharScribeSystem (archInfo[archType].cleanTime));
    cleanTime -> SetText (text);
    vstrDestroy (text);

    text = vstrCloneScribed (vcharScribeSystem (archInfo[archType].cleanEnable));
    if (vcharLength (text) == 0)
    {
	enableClean -> SetValue (0);
    }
    else
    {
	if (text[0] == 'd')
	    enableClean -> SetValue (0);
	else
	    enableClean -> SetValue (1);
    }
    vstrDestroy (text);

    text = vstrCloneScribed (vcharScribeSystem (archInfo[archType].archFile));
    archiveFile -> SetText (text);
    vstrDestroy (text);

    text = vstrCloneScribed (vcharScribeSystem (archInfo[archType].dailyTime));
    if (vcharLength (text) == 0)
    {
	dailyTime -> SetValue (dailyTime -> GetMin());
    }
    else
    {
	dailyTime -> SetValue (vnum::ParseInt (text, NULL));
    }
    vstrDestroy (text);

    modified = vFALSE;
}


/*****************************************************************
*
*	field protection apply button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuArchInfoApplyButtonClass, vbutton,
					"vuArchInfoApplyButtonClass")

/*****************************************************************
*
*	vuArchInfoApplyButtonClass observer
*
*****************************************************************/

void vuArchInfoApplyButtonClass::ObserveDialogItem (vevent *event)
{
    int archType = archInfoDialog -> getArchiveType ();
    int result = vuMessageDialogCenter (archInfoDialog,
					VU_ARCHINFO_UPDATE_REQ_MSG,
					archTypes[archType]);
    if (result == VU_FALSE)
    {
	return;
    }
    // validate clean and archive times
    vstr *text = archInfoDialog -> archiveTime -> GetTextAsString ();
    int arch;
    if (vcharLength (text) == 0)
	arch = 0;
    else
	arch = vnum::ParseInt (text, NULL);
    char archTime[10];
    vcharExportBounded (vcharScribeSystemExport (text),
			archTime,
			sizeof (archTime) - 1);
    archTime[sizeof(archTime)-1] = 0;
    vstrDestroy (text);

    text = archInfoDialog -> cleanTime -> GetTextAsString ();
    int clean;
    if (vcharLength (text) == 0)
	clean = 0;
    else
	clean = vnum::ParseInt (text, NULL);
    char cleanTime[10];
    vcharExportBounded (vcharScribeSystemExport (text),
			cleanTime,
			sizeof (cleanTime) - 1);
    cleanTime[sizeof(cleanTime)-1] = 0;
    vstrDestroy (text);

    if (clean >= arch)
    {
	text = archInfoDialog -> archiveFile -> GetTextAsString ();
	if ( (vcharLength (text)) == 0)
	{
	    vuMessageDialogCenter (archInfoDialog,
				   VU_ARCHINFO_NO_FILENAME_ERR);
	    return;
	}
	vchar fullFileName[VU_MAX_PATH + 1];
	char archFile[VU_MAX_PATH + 1];
	vcharCopyScribed (vcharScribeToLowerCase (text), fullFileName);
	fullFileName[VU_MAX_PATH] = 0;
	vcharExportBounded (vcharScribeSystemExport (fullFileName),
			    archFile,
			    sizeof (archFile) - 1);
	archFile[VU_MAX_PATH] = 0;
	vstrDestroy (text);
	vchar *filename = vcharGetFirstToken (fullFileName, ".");
	vchar *fileext = vcharGetNextToken ("\0");
	if ( ((vcharLength (filename)) == 0) ||
	     ((vcharLength (filename)) > VU_MSDOS_FILENAME_SZ) )
	{
	    vuMessageDialogCenter (archInfoDialog,
				   VU_ARCHINFO_FILENAME_ERR,
				   VU_MSDOS_FILENAME_SZ);
	    return;
	}
	if ( (fileext == NULL) ||
	     (vcharLength (fileext) > VU_MSDOS_EXTENSION_SZ) ||
	     (vcharCompare (fileext, (vchar *)(VU_DBASE_EXTENSION)) != 0) )
	{
	    vuMessageDialogCenter (archInfoDialog,
				   VU_ARCHINFO_FILEEXT_ERR);
	    return;
	}
	vuMainDialogClass *mainDialog = archInfoDialog -> getMainDialog ();
	dataVuApplication *vuApp = mainDialog -> getApplication ();
	vuDatabaseClass *db = vuApp -> getDatabaseClass ();
	char *dbPath = db -> getDbasePath ();
	char filepath[VU_MAX_PATH];
	FILE *fp;

	sprintf (filepath, "%s/archiveInfo", dbPath);
	if ( (fp = fopen (filepath, "w")) != NULL)
	{
	    strcpy (archInfoDialog -> archInfo[archType].archFile, archFile);
	    strcpy (archInfoDialog -> archInfo[archType].archTime, archTime);
	    strcpy (archInfoDialog -> archInfo[archType].cleanTime, cleanTime);
	    // bug fix, spinner may not have proper value
	    archInfoDialog -> dailyTime -> Unfocus ();
	    text = archInfoDialog -> dailyTime -> GetTextAsString ();
	    vcharExportBounded (vcharScribeSystemExport (text),
			        archInfoDialog -> archInfo[archType].dailyTime,
			        sizeof (archInfoDialog -> archInfo[archType].dailyTime) - 1);
	    vstrDestroy (text);
	    strcpy (archInfoDialog -> archInfo[archType].archEnable,
		    archInfoDialog -> enableArchive -> GetValue () ? 
						   VU_ARCH_ENABLED : 
						   VU_ARCH_DISABLED);
	    strcpy (archInfoDialog -> archInfo[archType].cleanEnable,
		    archInfoDialog -> enableClean -> GetValue () ? 
						 VU_ARCH_ENABLED : 
						 VU_ARCH_DISABLED);

	    int i;
	    for (i = 0; i < VU_MAX_ARCH_EDIT; i++)
	    {
		fprintf (fp, "%s\n", archInfoDialog -> archInfo[i].archFile);
		fprintf (fp, "%s\n", archInfoDialog -> archInfo[i].archTime);
		fprintf (fp, "%s\n", archInfoDialog -> archInfo[i].cleanTime);
		fprintf (fp, "%s\n", archInfoDialog -> archInfo[i].dailyTime);
		fprintf (fp, "%s\n", archInfoDialog -> archInfo[i].archEnable);
		fprintf (fp, "%s\n", archInfoDialog -> archInfo[i].cleanEnable);
	    }
	    fclose (fp);
	    mainDialog -> vuMainMessageCenter (vunames_vuArchInfoUpdateMsg,
					       archTypes[archType]);
	    archInfoDialog -> setDialogModify (vFALSE);
	}
	else
	{
	    vuMessageDialogCenter (archInfoDialog,
				   VU_ARCHINFO_FILEOPEN_ERR,
				   filepath);
	    return;
	}
    }
    else
    {
	vuMessageDialogCenter (archInfoDialog,
			       VU_ARCHINFO_ARCH_CLEAN_ERR);
	return;
    }
}


/*****************************************************************
*
*	field protection dismiss button class
*
*****************************************************************/

vkindTYPED_FULL_DEFINITION(vuArchInfoDismissButtonClass, vbutton,
					"vuArchInfoDismissButtonClass")

/*****************************************************************
*
*	vuArchInfoDismissButtonClass observer
*
*****************************************************************/

void vuArchInfoDismissButtonClass::ObserveDialogItem (vevent *event)
{
    int modified = archInfoDialog -> getDialogModify ();
    int result = VU_TRUE;

    if (modified)
    {
	result = vuMessageDialogCenter (archInfoDialog, VU_MOD_DISMISS_MSG);
    }
    if (result == VU_TRUE)
    {
	archInfoDialog -> Close ();
	vevent::StopProcessing ();
    }
}

