#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vdictHEADER
#include vlimitsHEADER
#include vmemHEADER
#include vnumHEADER
#include vlistitemHEADER
#include vdateHEADER
#include vspinnerHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbFldTypes.h"
#include "choiceGbl.h"
#include "choiceLdfw1.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "dbRecDes.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuCommonDialogClass.hxx"
#include "vuLdfwDialogClass.hxx"
#include "vuNavigatorDialogClass.hxx"
#include "vuUtils.hxx"


vkindTYPED_FULL_DEFINITION(vuLdfw1DialogClass, vuCommonDialogClass, "vuLdfw1DialogClass");

/*******************************************************************
*
*    vuLdfw1DialogClass override for constructors init ()
*
*******************************************************************/

void vuLdfw1DialogClass::LoadInit (vresource res)
{
    int	i;

    // load in the ldfw1 specific fields
    ldfw1Container = vuLdfw1ContainerClass::RequestSubLoad (this,
				vunames_vuCommonContainer);
    vuCommonDialogClass::LoadInit (res);

    // store dialog modified flag
    ldfw1Container -> suspendMode -> SetTextModifiedPtr (&this -> dialogModified);
    ldfw1Container -> suspendValue -> SetTextModifiedPtr (&this -> dialogModified);
    ldfw1Container -> onScan -> vuSetModifiedPtr (&this -> dialogModified);
    ldfw1Container -> onAlarm -> vuSetModifiedPtr (&this -> dialogModified);
    ldfw1Container -> manual -> vuSetModifiedPtr (&this -> dialogModified);
    for (i = 0; i < VU_NUM_LDFW_PUMPS; i++)
    {
	ldfw1Container -> pumps[i] -> SetTextModifiedPtr (&this -> dialogModified);
	ldfw1Container -> pumpSeqNums[i] -> SetModifiedPtr (&this -> dialogModified);
	// store dialog ptr for vuPvNameButtonClass
	ldfw1Container -> pumpButtons[i] -> storeCommonDialog ((vuCommonDialogClass *)this);
	// store dialog ptr for vuPvNameFilter
	ldfw1Container -> pumps[i] -> storeCommonDialog ((vuCommonDialogClass *)this);
    }
    ldfw1Container -> offTimeSetpt -> SetTextModifiedPtr (&this -> dialogModified);
    ldfw1Container -> secBetweenStarts -> SetTextModifiedPtr (&this -> dialogModified);
    ldfw1Container -> upperLimSeverity -> vuSetModifiedPtr (&this -> dialogModified);
    ldfw1Container -> reservePumps -> SetModifiedPtr (&this -> dialogModified);
    ldfw1Container -> secBetweenStops -> SetTextModifiedPtr (&this -> dialogModified);
    ldfw1Container -> lowerLimSeverity -> vuSetModifiedPtr (&this -> dialogModified);
    ldfw1Container -> pumpSelMode -> vuSetModifiedPtr (&this -> dialogModified);
    ldfw1Container -> alarmGroup -> SetTextModifiedPtr (&this -> dialogModified);
    ldfw1Container -> alarmDisable -> SetTextModifiedPtr (&this -> dialogModified);


    // store dialog ptr for vuPvNameButtonClass
    ldfw1Container -> suspendModeButton -> storeCommonDialog ((vuCommonDialogClass *)this);
    ldfw1Container -> alarmDisableButton -> storeCommonDialog ((vuCommonDialogClass *)this);

    // store dialog ptr for vuPvNameFilter
    ldfw1Container -> suspendMode -> storeCommonDialog ((vuCommonDialogClass *)this);
    ldfw1Container -> alarmDisable -> storeCommonDialog ((vuCommonDialogClass *)this);

    // store dialog ptr for vuAlarmGroupFilter
    ldfw1Container -> alarmGroup -> storeCommonDialog ((vuCommonDialogClass *)this);

    // set the point type
    setPointType (VU_LDFW1);

    // get the title from the resource file, aiTitle must be
    // freed by the class destuctor
    vapplication *app = vapplication::GetCurrent ();
    vresource dialogTitles = vresourceGet (app -> GetResources (),
				vunames_vuDialogTitles);
    ldfw1Title = vresourceGetString (dialogTitles, 
				vunames_vuLdfw1DialogTitle);
    SetTitle (ldfw1Title);
}


/*******************************************************************
*
*    vuLdfw1DialogClass Destroy function
*
*******************************************************************/

void vuLdfw1DialogClass::Destroy (void)
{
    vstrDestroy (ldfw1Title);
    vuCommonDialogClass::Destroy ();
}


/*******************************************************************
*
*    vuLdfw1DialogClass, reset fields, overrides base class 
*	vuCommonDialogClass
*
*******************************************************************/

void vuLdfw1DialogClass::resetDialog (void)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    vspinner		*spinner;
    vchar		epics[FLDNAME_SZ + 1];

    vresource defRes = mainDialog -> getDefaultsResource ();

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0; 
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	// if not a common dialog field
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    dialogItem = vuFindContainerItem (ldfw1Container, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    if ( (dialogItem -> IsKindOf (&vtextitem::Kind)) == vTRUE)
	    {
		textItem = (vtextitem *)dialogItem;

		// set the default value for the suspend value field
		if ( (vcharCompareBounded (epics, "SUSV", FLDNAME_SZ)) == 0)
		{
		    vstr *susv = vresourceGetString (defRes, 
						vunames_vuOneDefault);
		    textItem -> SetText (susv);
		    vstrDestroy (susv);
		}
		else
		{
		    textItem -> SetTextScribed (vcharScribeSystem (""));
		}
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolToggle::Kind)) == vTRUE)
	    {
		toggleItem = (vcontrolToggle *)dialogItem;
		toggleItem -> SetValue (0);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolOptionMenu::Kind)) == vTRUE)
	    {
		optionMenu = (vcontrolOptionMenu *)dialogItem;
		optionMenu -> SetValue (0);
	    }
	    else if ( (dialogItem -> IsKindOf (&vspinner::Kind)) == vTRUE)
	    {
		spinner = (vspinner *)dialogItem;
		spinner -> SetValue (spinner -> GetMin());
	    }
	}  // end if not in common dialog
    }
    iterator.Finish ();

    // call base class reset function with proper point index
    vuCommonDialogClass::resetDialog ();

    dialogModified = vFALSE;
}


/*******************************************************************
*
*    vuLoadDialog 
*
*******************************************************************/

void vuLdfw1DialogClass::vuLoadDialog (vchar *pointName)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    vspinner		*spinner;
    vchar		epics[FLDNAME_SZ + 1];
    vstr		*finalValue;

    char		pvName [PVNAME_SZ + 1];
    DB_TABLE_PTR	lastTable;	// pointer to last table used
    DB_RECORD_PTR	pntRecord;	// codebase record ptr
    int			dbaseType;	// codebase field type
    int			intValue;
    unsigned int	uIntValue;
    long		longValue;
    unsigned long	uLongValue;
    double		doubleValue;

    vcharExportBounded (vcharScribeSystemExport (pointName),
			pvName,
			PVNAME_SZ);

    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
    pntRecord = DB_Seek (db -> vuPvmapTable, seekBuff);
    if (pntRecord == (DB_RECORD_PTR)0)
    {
	return;
    }
    else
    {
	int ptType;
	DB_GetFieldValue (pntRecord, db -> pvmapTypeField, dbValue, &dbaseType);
	ptType = vuGetPointTypeIndex ((char *)dbValue);
	// the point name in pvname field is NOT an LDFW1
	if (ptType != VU_LDFW1)
	{
	    vuMessageDialogCenter ((vdialog *)this, VU_PTTYPEDIALOGERR,
				   pointName, ptType, VU_LDFW1);
	    vuClearPvNameField ();
	    return;
	}
	// the point name entered in pvname field has been deleted
	if (DB_IsDeleted (db -> vuPvmapTable))
	{
	    vuMessageDialogCenter ((vdialog *)this, VU_PTISDELETEDERR, pointName);
	    vuClearPvNameField ();
	    return;
	}
    }
    // call base class vuLoadDialog first
    vuCommonDialogClass::vuLoadDialog (pointName);

    lastTable = NULL;

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0;
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    // only seek if table has changed
	    if (lastTable != xrefNode -> xrefTable)
		{
		    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
			pntRecord = DB_Seek (xrefNode -> xrefTable, seekBuff);
		}
	    lastTable = xrefNode -> xrefTable;
	    dialogItem = vuFindContainerItem (ldfw1Container, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    DB_GetFieldValue (pntRecord, xrefNode -> xrefField, dbValue, &dbaseType);
	    doubleValue = 0.0;
	    // GetField actually returns a number in the char array
	    if ( (dbaseType == 'F') || (dbaseType == 'N') )
	    {
		doubleValue = *((double *)dbValue);
	    }
	    switch (xrefNode -> xrefEPICSFieldType)
	    {
		case DBF_SHORT:
		    intValue = (int)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeInt (intValue));
		    break;

		case DBF_LONG:
		    longValue = (long)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeLong (longValue));
		    break;

		case DBF_ENUM:
		case DBF_USHORT:
		    uIntValue = (unsigned int)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeUInt (uIntValue));
		    break;

		case DBF_ULONG:
		    uLongValue = (unsigned long)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeULong (uLongValue));
		    break;

		case DBF_FLOAT:
		case DBF_DOUBLE:
		    finalValue = vstrCloneScribed (vnumScribeFloat (doubleValue));
		    break;

		case DBF_STRING:
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		case DBF_GBLCHOICE:
		case DBF_RECCHOICE:
		case DBF_CVTCHOICE:
		case DBF_DEVCHOICE:
		    if (dbaseType == 'L')
		    {
			int	logical;

			logical = (*((char*)dbValue) ? 
					xrefNode -> xrefTrue : 
					(xrefNode -> xrefTrue ^1));
			finalValue = vstrCloneScribed (vnumScribeInt (logical));
			break;
		    }
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		case DBF_INLINK:
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		default:
		    continue;
	    }
	    if ( (dialogItem -> IsKindOf (&vtextitem::Kind)) == vTRUE)
	    {
		textItem = (vtextitem *)dialogItem;
		textItem -> SetTextScribed (vcharScribeSystem (""));
		textItem -> SetText (finalValue);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolToggle::Kind)) == vTRUE)
	    {
		toggleItem = (vcontrolToggle *)dialogItem;
		if (finalValue[0] == '0')
		    toggleItem -> SetValue (0);
		else
		    toggleItem -> SetValue (1);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolOptionMenu::Kind)) == vTRUE)
	    {
		optionMenu = (vcontrolOptionMenu *)dialogItem;
		vuSetOptionMenuValue (optionMenu, finalValue);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vspinner::Kind)) == vTRUE)
	    {
		spinner = (vspinner *)dialogItem;
		spinner -> SetValue (vnum::ParseInt (finalValue, NULL));
		vstrDestroy (finalValue);
	    }
	}  // end if not in commondialog
    }
    iterator.Finish ();

    // call base class vuLoadDialog
    vuCommonDialogClass::vuLoadDialog (pointName);
    setDialogModify (vFALSE);
}


/*******************************************************************
*
*    vuSaveDialog 
*
*******************************************************************/

void vuLdfw1DialogClass::vuSaveDialog (vchar *pointName)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    const vchar 	*optionMenuValue;
    vspinner		*spinner;
    vchar		epics[FLDNAME_SZ + 1];
    vstr		*stringValue;

    char		pvName [PVNAME_SZ + 1];
    DB_FIELD_PTR	putField;
    DB_TABLE_PTR	lastTable;	// pointer to last table used
    DB_RECORD_PTR	pntRecord;	// codebase record ptr
    double		doubleValue;
    int			doSave;

    vcharExportBounded (vcharScribeSystemExport (pointName),
			pvName,
			PVNAME_SZ);

    if (validatePumpSequence () == vFALSE)
	return;

    lastTable = NULL;

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0;
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    doSave = vTRUE;
	    // only seek if table has changed
	    if (lastTable != xrefNode -> xrefTable)
		{
		    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
			pntRecord = DB_Seek (xrefNode -> xrefTable, seekBuff);
		}
	    lastTable = xrefNode -> xrefTable;

	    putField = xrefNode -> xrefField;

	    dialogItem = vuFindContainerItem (ldfw1Container, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    switch (xrefNode -> xrefEPICSFieldType)
	    {
		case DBF_SHORT:
		case DBF_LONG:
		case DBF_ENUM:
		case DBF_USHORT:
		case DBF_ULONG:
		case DBF_FLOAT:
		case DBF_DOUBLE:
		    if ( (dialogItem -> IsKindOf (&vspinner::Kind)) == vTRUE)
		    {
			spinner = (vspinner *)dialogItem;
			// bug fix, spinner may not have proper value
			spinner -> Unfocus ();
			stringValue = spinner -> GetTextAsString ();
		    }
		    else
		    {
			textItem = (vtextitem *)dialogItem;
			stringValue = textItem -> GetTextAsString ();
		    }
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    doubleValue = atof ((const char *)dbValue);
		    DB_PutFieldValue (pntRecord, putField, (void *)&doubleValue);
		    doSave = vFALSE;
		    break;

		case DBF_STRING:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    break;

		case DBF_GBLCHOICE:
		case DBF_RECCHOICE:
		case DBF_CVTCHOICE:
		case DBF_DEVCHOICE:
		{
		    if (xrefNode -> xrefField -> field_type == 'L')
		    {
			int	logical;
			toggleItem = (vcontrolToggle *)dialogItem;
			logical = toggleItem -> GetValue ();
			*((char *)dbValue) = (logical ? xrefNode -> xrefTrue : 
								xrefNode -> xrefTrue ^1);
			break;
		    }
		    optionMenu = (vcontrolOptionMenu *)dialogItem;
		    optionMenuValue = vuGetOptionMenuValue (optionMenu);
		    vcharExport (vcharScribeSystemExport (optionMenuValue), dbValue);
		    break;
		}

		case DBF_INLINK:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    break;

		default:
		    continue;

	    }  // end xrefNode -> xrefEPICSFieldType

	    if (doSave == vTRUE)
	    {
		DB_PutFieldValue (pntRecord, putField, dbValue);
	    }

	}  // end if not in commondialog

    }  // end while iterator

    iterator.Finish ();

    // call base class vuSaveDialog
    vuCommonDialogClass::vuSaveDialog (pointName);
}


/*******************************************************************
*
*    vuLdfw1DialogClass function to validate pumps and 
*	sequence numbers
*	validate that defined pumps are in order, no holes in list
*	validate that a defined sequece number is found within 
*		the range of defined pumps.
*
*******************************************************************/

int vuLdfw1DialogClass::validatePumpSequence (void)
{
    int		numPumpsDefined;
    int		lastPumpDefined = 0;
    int		seqNums[VU_NUM_LDFW_PUMPS];
    int		numSeqEntries = 0;
    int		seqNumValue;
    vchar	pumpName[PVNAME_SZ + 1];
    vchar	seqNumStr[VU_LDFW_SEQ_NUM_SZ + 2];
    int		first = vTRUE;
    int		i, j;

    for (i = 0; i < VU_NUM_LDFW_PUMPS; i++)
    {
	ldfw1Container -> pumps[i] -> GetText (pumpName, PVNAME_SZ);
	pumpName[PVNAME_SZ] = 0;
	if (pumpName[0] != 0)
	{
	    numPumpsDefined = i + 1;
	    if (first == vTRUE)
	    {
		first = vFALSE;
	    }
	    else
	    {
		if (lastPumpDefined < (numPumpsDefined - 1))
		{
		    // hole in pump definitions
		    vuMessageDialogCenter (this, VU_LDFW_PUMPS_DEF_ERR);
		    return (vFALSE);
		}
	    }
	    lastPumpDefined = numPumpsDefined;
	    ldfw1Container -> pumpSeqNums[i] -> Unfocus ();
	    ldfw1Container -> pumpSeqNums[i] -> GetText (seqNumStr, VU_LDFW_SEQ_NUM_SZ + 1);
	    seqNumStr[VU_LDFW_SEQ_NUM_SZ + 1] = 0;
	    seqNumValue = vnum::ParseInt (seqNumStr, NULL);
	    if (seqNumValue > 0)
	    {
		seqNums[numSeqEntries] = seqNumValue;
		numSeqEntries++;
	    }
	}
    }
    for (i = 1; i <= numSeqEntries; i++)
    {
	for (j = 0; j < numPumpsDefined; j++)
	{
	    if (seqNums[j] == i)
	    {
		break;
	    }
	}
	if (j >= numPumpsDefined)
	{
	    // didn't find sequence number in the defined pumps
	    vuMessageDialogCenter (this, VU_LDFW_SEQ_ERR);
	    return (vFALSE);
	}
    }
    return (vTRUE);
}


vkindTYPED_FULL_DEFINITION(vuLdfw1ContainerClass, vcontainer, "vuLdfw1ContainerClass");

/*******************************************************************
*
*    vuLdfw1ContainerClass override for constructor
*
*******************************************************************/

void vuLdfw1ContainerClass::LoadInit (vresource res)
{
    int	i;

    vapplication *app = vapplication::GetCurrent ();
    vresource pointDictRes = vresourceGet (app -> GetResources (), 
				vunames_vuPointDictionary);
    vresource realRes = vresourceGet (pointDictRes, vunames_vuLdfw1Container);

    vcontainer *suspcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1SUSPContainer);
    suspendModeButton = vuPvNameButtonClass::RequestSubLoad (suspcontainer,
				vunames_vuLdfw1SUSPButton);
    suspendMode = vuFilteredTextitem::RequestSubLoad (suspcontainer,
				vunames_SUSP);

    vcontainer *susvcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1SUSVContainer);
    suspendValue = filteredTextitem::RequestSubLoad (susvcontainer,
				vunames_SUSV);

    onScan = vuCheckBoxClass::RequestSubLoad (this,
				vunames_OSCN);

    onAlarm = vuCheckBoxClass::RequestSubLoad (this,
				vunames_AENB);

    manual = vuCheckBoxClass::RequestSubLoad (this,
				vunames_MODE);

    vcontainer *pm01container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM01Container);
    pumpButtons[0] = vuPvNameButtonClass::RequestSubLoad (pm01container,
				vunames_vuLdfw1PM01Button);
    pumps[0] = vuFilteredTextitem::RequestSubLoad (pm01container,
				vunames_PM01);
    vcontainer *pm1seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS01Container);
    pumpSeqNums[0] = spinnerClass::RequestSubLoad(pm1seqcontainer,
				vunames_DS01);

    vcontainer *pm02container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM02Container);
    pumpButtons[1] = vuPvNameButtonClass::RequestSubLoad (pm02container,
				vunames_vuLdfw1PM02Button);
    pumps[1] = vuFilteredTextitem::RequestSubLoad (pm02container,
				vunames_PM02);
    vcontainer *pm2seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS02Container);
    pumpSeqNums[1] = spinnerClass::RequestSubLoad(pm2seqcontainer,
				vunames_DS02);

    vcontainer *pm03container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM03Container);
    pumpButtons[2] = vuPvNameButtonClass::RequestSubLoad (pm03container,
				vunames_vuLdfw1PM03Button);
    pumps[2] = vuFilteredTextitem::RequestSubLoad (pm03container,
				vunames_PM03);
    vcontainer *pm3seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS03Container);
    pumpSeqNums[2] = spinnerClass::RequestSubLoad (pm3seqcontainer,
				vunames_DS03);

    vcontainer *pm04container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM04Container);
    pumpButtons[3] = vuPvNameButtonClass::RequestSubLoad (pm04container,
				vunames_vuLdfw1PM04Button);
    pumps[3] = vuFilteredTextitem::RequestSubLoad (pm04container,
				vunames_PM04);
    vcontainer *pm4seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS04Container);
    pumpSeqNums[3] = spinnerClass::RequestSubLoad (pm4seqcontainer,
				vunames_DS04);

    vcontainer *pm05container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM05Container);
    pumpButtons[4] = vuPvNameButtonClass::RequestSubLoad (pm05container,
				vunames_vuLdfw1PM05Button);
    pumps[4] = vuFilteredTextitem::RequestSubLoad (pm05container,
				vunames_PM05);
    vcontainer *pm5seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS05Container);
    pumpSeqNums[4] = spinnerClass::RequestSubLoad (pm5seqcontainer,
				vunames_DS05);

    vcontainer *pm06container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM06Container);
    pumpButtons[5] = vuPvNameButtonClass::RequestSubLoad (pm06container,
				vunames_vuLdfw1PM06Button);
    pumps[5] = vuFilteredTextitem::RequestSubLoad (pm06container,
				vunames_PM06);
    vcontainer *pm6seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS06Container);
    pumpSeqNums[5] = spinnerClass::RequestSubLoad (pm6seqcontainer,
				vunames_DS06);

    vcontainer *pm07container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM07Container);
    pumpButtons[6] = vuPvNameButtonClass::RequestSubLoad (pm07container,
				vunames_vuLdfw1PM07Button);
    pumps[6] = vuFilteredTextitem::RequestSubLoad (pm07container,
				vunames_PM07);
    vcontainer *pm7seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS07Container);
    pumpSeqNums[6] = spinnerClass::RequestSubLoad (pm7seqcontainer,
				vunames_DS07);

    vcontainer *pm08container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM08Container);
    pumpButtons[7] = vuPvNameButtonClass::RequestSubLoad (pm08container,
				vunames_vuLdfw1PM08Button);
    pumps[7] = vuFilteredTextitem::RequestSubLoad (pm08container,
				vunames_PM08);
    vcontainer *pm8seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS08Container);
    pumpSeqNums[7] = spinnerClass::RequestSubLoad (pm8seqcontainer,
				vunames_DS08);

    vcontainer *pm09container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM09Container);
    pumpButtons[8] = vuPvNameButtonClass::RequestSubLoad (pm09container,
				vunames_vuLdfw1PM09Button);
    pumps[8] = vuFilteredTextitem::RequestSubLoad (pm09container,
				vunames_PM09);
    vcontainer *pm9seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS09Container);
    pumpSeqNums[8] = spinnerClass::RequestSubLoad (pm9seqcontainer,
				vunames_DS09);

    vcontainer *pm10container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM10Container);
    pumpButtons[9] = vuPvNameButtonClass::RequestSubLoad (pm10container,
				vunames_vuLdfw1PM10Button);
    pumps[9] = vuFilteredTextitem::RequestSubLoad (pm10container,
				vunames_PM10);
    vcontainer *pm10seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS10Container);
    pumpSeqNums[9] = spinnerClass::RequestSubLoad (pm10seqcontainer,
				vunames_DS10);

    vcontainer *pm11container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM11Container);
    pumpButtons[10] = vuPvNameButtonClass::RequestSubLoad (pm11container,
				vunames_vuLdfw1PM11Button);
    pumps[10] = vuFilteredTextitem::RequestSubLoad (pm11container,
				vunames_PM11);
    vcontainer *pm11seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS11Container);
    pumpSeqNums[10] = spinnerClass::RequestSubLoad (pm11seqcontainer,
				vunames_DS11);

    vcontainer *pm12container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM12Container);
    pumpButtons[11] = vuPvNameButtonClass::RequestSubLoad (pm12container,
				vunames_vuLdfw1PM12Button);
    pumps[11] = vuFilteredTextitem::RequestSubLoad (pm12container,
				vunames_PM12);
    vcontainer *pm12seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS12Container);
    pumpSeqNums[11] = spinnerClass::RequestSubLoad (pm12seqcontainer,
				vunames_DS12);

    vcontainer *pm13container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM13Container);
    pumpButtons[12] = vuPvNameButtonClass::RequestSubLoad (pm13container,
				vunames_vuLdfw1PM13Button);
    pumps[12] = vuFilteredTextitem::RequestSubLoad (pm13container,
				vunames_PM13);
    vcontainer *pm13seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS13Container);
    pumpSeqNums[12] = spinnerClass::RequestSubLoad (pm13seqcontainer,
				vunames_DS13);

    vcontainer *pm14container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM14Container);
    pumpButtons[13] = vuPvNameButtonClass::RequestSubLoad (pm14container,
				vunames_vuLdfw1PM14Button);
    pumps[13] = vuFilteredTextitem::RequestSubLoad (pm14container,
				vunames_PM14);
    vcontainer *pm14seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS14Container);
    pumpSeqNums[13] = spinnerClass::RequestSubLoad (pm14seqcontainer,
				vunames_DS14);

    vcontainer *pm15container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM15Container);
    pumpButtons[14] = vuPvNameButtonClass::RequestSubLoad (pm15container,
				vunames_vuLdfw1PM15Button);
    pumps[14] = vuFilteredTextitem::RequestSubLoad (pm15container,
				vunames_PM15);
    vcontainer *pm15seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS15Container);
    pumpSeqNums[14] = spinnerClass::RequestSubLoad (pm15seqcontainer,
				vunames_DS15);

    vcontainer *pm16container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM16Container);
    pumpButtons[15] = vuPvNameButtonClass::RequestSubLoad (pm16container,
				vunames_vuLdfw1PM16Button);
    pumps[15] = vuFilteredTextitem::RequestSubLoad (pm16container,
				vunames_PM16);
    vcontainer *pm16seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS16Container);
    pumpSeqNums[15] = spinnerClass::RequestSubLoad (pm16seqcontainer,
				vunames_DS16);

    vcontainer *pm17container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM17Container);
    pumpButtons[16] = vuPvNameButtonClass::RequestSubLoad (pm17container,
				vunames_vuLdfw1PM17Button);
    pumps[16] = vuFilteredTextitem::RequestSubLoad (pm17container,
				vunames_PM17);
    vcontainer *pm17seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS17Container);
    pumpSeqNums[16] = spinnerClass::RequestSubLoad (pm17seqcontainer,
				vunames_DS17);

    vcontainer *pm18container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM18Container);
    pumpButtons[17] = vuPvNameButtonClass::RequestSubLoad (pm18container,
				vunames_vuLdfw1PM18Button);
    pumps[17] = vuFilteredTextitem::RequestSubLoad (pm18container,
				vunames_PM18);
    vcontainer *pm18seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS18Container);
    pumpSeqNums[17] = spinnerClass::RequestSubLoad (pm18seqcontainer,
				vunames_DS18);

    vcontainer *pm19container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM19Container);
    pumpButtons[18] = vuPvNameButtonClass::RequestSubLoad (pm19container,
				vunames_vuLdfw1PM19Button);
    pumps[18] = vuFilteredTextitem::RequestSubLoad (pm19container,
				vunames_PM19);
    vcontainer *pm19seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS19Container);
    pumpSeqNums[18] = spinnerClass::RequestSubLoad (pm19seqcontainer,
				vunames_DS19);

    vcontainer *pm20container = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PM20Container);
    pumpButtons[19] = vuPvNameButtonClass::RequestSubLoad (pm20container,
				vunames_vuLdfw1PM20Button);
    pumps[19] = vuFilteredTextitem::RequestSubLoad (pm20container,
				vunames_PM20);
    vcontainer *pm20seqcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1DS20Container);
    pumpSeqNums[19] = spinnerClass::RequestSubLoad (pm20seqcontainer,
				vunames_DS20);

    vcontainer *osptcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1OSPTContainer);
    offTimeSetpt = filteredTextitem::RequestSubLoad (osptcontainer,
				vunames_OSPT);

    vcontainer *strscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1STRSContainer);
    secBetweenStarts = filteredTextitem::RequestSubLoad (strscontainer,
				vunames_STRS);

    vcontainer *usvrcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1USVRContainer);
    upperLimSeverity = vuChoiceGblMenuClass::RequestSubLoad (usvrcontainer,
				vunames_USVR);

    vcontainer *nrpmcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1NRPMContainer);
    reservePumps = spinnerClass::RequestSubLoad (nrpmcontainer,
				vunames_NRPM);

    vcontainer *stpscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1STPSContainer);
    secBetweenStops = filteredTextitem::RequestSubLoad (stpscontainer,
				vunames_STPS);

    vcontainer *lsvrcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1LSVRContainer);
    lowerLimSeverity = vuChoiceGblMenuClass::RequestSubLoad (lsvrcontainer,
				vunames_LSVR);

    vcontainer *psmdcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1PSMDContainer);
    pumpSelMode = vuChoiceRecMenuClass::RequestSubLoad (psmdcontainer,
				vunames_PSMD);


    vcontainer *alrmgrpcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1ALGPContainer);
    alarmGroup = vuFilteredTextitem::RequestSubLoad (alrmgrpcontainer,
				vunames_ALGP);

    vcontainer *alarmdiscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuLdfw1ADISContainer);
    alarmDisableButton = vuPvNameButtonClass::RequestSubLoad (alarmdiscontainer,
				vunames_vuLdfw1ADISButton);
    alarmDisable = vuFilteredTextitem::RequestSubLoad (alarmdiscontainer,
				vunames_ADIS);

    vcontainer::LoadInit (realRes);

    // store ptr to text field for vuPvNameButtonClass
    suspendModeButton -> storePvName (suspendMode);
    for (i = 0; i < VU_NUM_LDFW_PUMPS; i++)
    {	
	pumpButtons[i] -> storePvName (pumps[i]);
    }
    alarmDisableButton -> storePvName (alarmDisable);

    // set filters
    suspendValue -> SetItemFilterFunc (numericFilter);
    offTimeSetpt -> SetItemFilterFunc (floatFilter);
    secBetweenStarts -> SetItemFilterFunc (floatFilter);
    secBetweenStops -> SetItemFilterFunc (floatFilter);

    suspendMode -> SetItemFilterFunc (vuPvNameFilter);
    filteredText *suspText = (filteredText *)suspendMode -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    suspText -> SetUserPtr ((void *)suspendMode);

    filteredText *pumpText;

    for (i = 0; i < VU_NUM_LDFW_PUMPS; i++)
    {
	pumps[i] -> SetItemFilterFunc (vuPvNameFilter);
	pumpText = (filteredText *)(pumps[i] -> GetTextData ());
	// remember ptr to textitem for later use by the filter
	pumpText -> SetUserPtr ((void *)pumps[i]);
    }


    alarmGroup -> SetItemFilterFunc (vuAlarmGroupFilter);
    filteredText *alarmText = (filteredText *)alarmGroup -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    alarmText -> SetUserPtr ((void *)alarmGroup);

    alarmDisable -> SetItemFilterFunc (vuPvNameFilter);
    filteredText *alrmDisText = (filteredText *)alarmDisable -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    alrmDisText -> SetUserPtr ((void *)alarmDisable);

    // build popup menus
    upperLimSeverity -> vuBuildOptionMenu (GBL_ALARM_SEV);
    lowerLimSeverity -> vuBuildOptionMenu (GBL_ALARM_SEV);
    pumpSelMode -> vuBuildOptionMenu (VU_LDFW1, REC_LDFW1_PSMD);

}

/*******************************************************************
*
*    vuLdfw1ContainerClass override for destructor
*
*******************************************************************/

void vuLdfw1ContainerClass::Destroy (void)
{
    vcontainer::Destroy ();
}

