#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vlistviewHEADER
#include vliststrHEADER
#include vdictHEADER
#include vlimitsHEADER
#include vmemHEADER
#include vnumHEADER
#include vlistitemHEADER
#include vdateHEADER
#include vspinnerHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbDefs.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"

#include "vuNavigatorDialogClass.hxx"
#include "vuUtils.hxx"


vkindTYPED_FULL_DEFINITION(vuNavigatorDialogClass, vuFindDialogClass, 
						"vuNavigatorDialogClass");

/*******************************************************************
*
*    vuNavigatorDialogClass 
*	LoadInit, constructor
*
*******************************************************************/

void vuNavigatorDialogClass::LoadInit(vresource res)
{
    // init return list to NULL
    pvnameList = NULL;

    // set to NULL
    dialogTitle = NULL;

    navContainer = vuNavContainerClass::RequestSubLoad (this,
				vunames_vuFindCommonContainer);

    vcontainer *buttonContainer = vcontainer::RequestSubLoad (this,
				vunames_vuFindButtonContainer);
    
    dismissButton = vuNavDismissButtonClass::RequestSubLoad (buttonContainer,
				vunames_vuFindDismissButton);
    findButton = vuNavFindButtonClass::RequestSubLoad (buttonContainer,
				vunames_vuFindFindButton);

    // call base class loadInit to finish the job
    vuFindDialogClass::LoadInit(res);

    // set pointer to navigator dialog in these items

    navContainer -> pointList -> setNavigatorDialog (this);
    navContainer -> okButton -> setNavigatorDialog (this);
    findButton -> setNavigatorDialog (this);
    dismissButton -> setNavigatorDialog (this);

}


/*******************************************************************
*
*    vuNavigatorDialogClass notify function
*
*******************************************************************/

void vuNavigatorDialogClass::ObserveWindow(vevent *event)
{
    if ( event -> GetType() == veventWINDOW_STATE && event -> IsClose())
    {
	// zap stuff caller could have used
	if (pvnameList != NULL)
	{
	    int i = 0;
	    while (pvnameList[i])
	    {
		vmemFree ( (void *)pvnameList[i]);
		i++;
	    }
	    vmemFree ((void *)pvnameList);
	}
	pvnameList = NULL;
	Close ();
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}


/*******************************************************************
*
*    vuNavigatorDialogClass Destroy function
*	if dialog is open, close it and stop its event loop
*
*******************************************************************/

void vuNavigatorDialogClass::Destroy(void)
{
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
    // get the list attached to the listview and destroy it, also set the 
    // foundList in the find class to NULL because the base class Destroy will
    // also try to destroy it if it is not NULL
    vliststr *listdata = (vliststr *)(navContainer -> pointList -> GetList ());
    if (listdata != NULL)
    {
	navContainer -> pointList -> SetList (NULL);
	listdata -> Destroy ();
	foundList = NULL;
    }
    // free up any memory for the char **list
    if (pvnameList != NULL)
    {
	int i = 0;
	while(pvnameList[i])
	{
	    vmemFree ((void *)(pvnameList[i]));
	    i++;
	}
	vmemFree ((void *)pvnameList);
	pvnameList = NULL;
    }

    vstrDestroy (dialogTitle);

    vuFindDialogClass::Destroy();
}


/*******************************************************************
*
*    vuNavigatorDialogClass Open function
*
*******************************************************************/

void vuNavigatorDialogClass::vuFindOpen(vchar *label,vuMainDialogClass *mainDialg,int multiSelect)
{

    navContainer -> okButton -> SetTitleScribed (vcharScribeSystem(label));
	navContainer -> okButton->Disable();

    mainDialog = mainDialg;

    // set flag for pointList as to the type of select method to use
    vresource titleRes  = mainDialog -> getTitlesResource ();

    // these 3 allow multiple point selections
    singleSelect = multiSelect;

    // destroy pvnameList if it exists
    if (pvnameList != NULL)
    {
	int i = 0;
	while(pvnameList[i])
	{
	    vmemFree ((void *)(pvnameList[i]));
	    i++;
	}
	vmemFree ((void *)pvnameList);
	pvnameList = NULL;
    }

    // set points found label
    navContainer -> pointsFoundLabel -> SetTitleScribed (vcharScribeSystem (" "));
    //
    // update selection method even if FIND not performed
    //
    vliststr *listdata = (vliststr *)(navContainer -> pointList -> GetList ());
    if (listdata != NULL)
    {
	vlistSelection *selection = navContainer -> pointList -> GetSelection ();
	if(listdata->IsSelectionEmpty(selection) == vFALSE)
	{
	    navContainer -> okButton->Enable();
	}
	if (singleSelect == vTRUE)
	{
	    listdata -> SetSelectMethod (selection, vlistONE_ONLY_SELECTION);
	}
	else
	{
	    listdata -> SetSelectMethod (selection, vlistFREE_SELECTION);
	}
    }

    // call base class open
    vuFindDialogClass::vuFindOpen (label, mainDialg, multiSelect);

    if (dialogTitle == NULL)
    {
	dialogTitle = vresourceGetString (titleRes, vunames_vuNavDialogTitle);
	SetTitle (dialogTitle);
    }


    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    MakeTopMostDialog (this, NULL);
    vevent::Process ();
}


/*******************************************************************
*
*    vuNavigatorDialogClass Remove function
*
*******************************************************************/

void vuNavigatorDialogClass::vuRemovePvName(vchar *pvName)
{
    vliststr *list = (vliststr *)(navContainer -> pointList -> GetList ());
    if (list != NULL)
    {
	int row;
	vchar *pv;
	vlistIterator iterator;

	iterator.Start (list);
	while (iterator.Next ())
	{
	    row = iterator.GetRow ();
	    pv = (vchar *)(iterator.GetValue ());
	    if ((vcharCompareBounded (pvName, pv, PVNAME_SZ)) == 0)
	    {
		list -> RemoveRow (row);
		break;
	    }
	}
	iterator.Finish ();
    }
}


vkindTYPED_FULL_DEFINITION(vuNavPtListClass, vlistview, "vuNavPtListClass");

/*******************************************************************
*
*    vuNavPtListClass selection notify function
*	SELECTION_HAS_CHANGED = user finally release the mouse
*	button and commits the new selection, if we have a double
*	mouse click, then take same action as OK BUTTON
*
*******************************************************************/

void vuNavPtListClass::ObserveSelect(int state)
{
    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	navigatorDialog ->getNavContainer()-> okButton->Enable();
	int clickCount = GetLastPointerClickCount ();
	if (clickCount == 2)
	{
	    int	singleSelect = navigatorDialog -> getSingleSelect ();

	    // only allow for certain navigator options
	    if (singleSelect == vFALSE)
	    {
		return;
	    }

	    char		**pvNameList = navigatorDialog -> getPvNameList ();
	    char		*pvname;
	    vlist		*ptList;
	    vlistSelection	*listSelection;
	    vchar		*cellValue;
	    int			row, column;

	    ptList = GetList ();
	    listSelection = GetSelection ();
	    ptList -> GetSelectedCell (listSelection, &row, &column);
	    cellValue = (vchar *)(ptList -> GetCellValue (row, column));
	    // copy value to dialogs pvname list for use by dialog caller
	    if (pvNameList)
	    {
		int i = 0;
		while(pvNameList[i])
		{
		    vmemFree ((void *)(pvNameList[i]));
		    i++;
		}
		vmemFree ((void *)pvNameList);
	    }
	    navigatorDialog -> setPvNameList (NULL);
	    if ( (pvNameList = (char **)(vmemAllocAndClear (2 * sizeof (char *) ))) == NULL)
	    {
		vuMessageDialogCenter (navigatorDialog, VU_NAVPTALLOCERR_MSG);
		return;
	    }
	    if ( (pvname = (char *)(vmemAllocAndClear (PVNAME_SZ + 1))) == NULL)
	    {
		vmemFree ((void *)pvNameList);
		navigatorDialog -> setPvNameList (NULL);
		vuMessageDialogCenter (navigatorDialog, VU_NAVPTALLOCERR_MSG);
		return;
	    }
	    vcharExportBounded (vcharScribeSystemExport (cellValue),
				pvname, PVNAME_SZ);
	    pvname [PVNAME_SZ] = 0;
	    pvNameList[0] = pvname;
	    navigatorDialog -> setPvNameList (pvNameList);
				
	    // close the dialog and stop the sub event loop
	    // started in vuFindOpen ()
	    navigatorDialog -> Close ();
	    vevent::StopProcessing ();
	}
    }
}


vkindTYPED_FULL_DEFINITION(vuNavOkButtonClass, vbutton, "vuNavOkButtonClass");

/*******************************************************************
*
*    vuNavOkButtonClass 
*	Observer function
*
*******************************************************************/

void vuNavOkButtonClass::ObserveDialogItem (vevent *event)
{
    char **pvNameList = navigatorDialog -> getPvNameList ();
    vuNavContainerClass *navContainer = navigatorDialog -> getNavContainer ();
	navigatorDialog -> Status = vTRUE;			// just in case there are error returns;
    if (pvNameList != NULL)
    {
	int i = 0;
	while (pvNameList[i])
	{
	    vmemFree ((void *)pvNameList[i]);
	    i++;
	}
	vmemFree ((void *)pvNameList);
	navigatorDialog -> setPvNameList (NULL);
    }
    vlist *nameList = navContainer -> pointList -> GetList (); 
    if (nameList == NULL)
    {
	vuMessageDialogCenter (navigatorDialog, VU_NO_LIST_ERR);
	return;
    }
    vlistSelection *selection = navContainer -> pointList -> GetSelection ();
    vlistIterator listIterator;
    int row, rowCount;
    int ptIdx = 0;
    char *newPv;

    // loop through the current selection with the iterator
    // looking for items that are selected
    rowCount = nameList -> GetRowCount ();
    listIterator.StartWithSelection (selection);
    while (listIterator.Next ())
    {
	row = listIterator.GetRow ();
	if ( (nameList -> IsCellSelected (selection, row, 0)) == vTRUE)
	{
	    if (pvNameList == NULL)
	    {
		if ( (pvNameList = (char **)vmemAllocAndClear ((rowCount + 1) * sizeof (char *))) == NULL)
		{
		    vuMessageDialogCenter (navigatorDialog, VU_NAVPTALLOCERR_MSG);
		    return;
		}
	    }
	    if ( (newPv = (char *)vmemAllocAndClear (PVNAME_SZ + 1)) == NULL)
	    {
		int i = 0;
		while (pvNameList[i])
		{
		    vmemFree ((void *)pvNameList[i]);
		    i++;
		}
		vmemFree ((void *)newPv);
		navigatorDialog -> setPvNameList (NULL);
		vuMessageDialogCenter (navigatorDialog, VU_NAVPTALLOCERR_MSG);
		return;
            }
	    vcharExportBounded (vcharScribeSystemExport (
					(vchar *)(nameList -> GetCellValue (row, 0))),
					newPv, PVNAME_SZ);
	    newPv[PVNAME_SZ] = 0;
	    pvNameList[ptIdx] = newPv;
	    ptIdx++;
	}
    }  // end while ()

    listIterator.Finish ();

    if (ptIdx == 0)
    {
	vuMessageDialogCenter (navigatorDialog, VU_NO_PTNAMES_SELECTED_MSG);
	return;
    }
    else
	navigatorDialog -> setPvNameList (pvNameList);
	navigatorDialog -> Status = vFALSE;

    // close dialog and stop event loop started in vuNavOpen ()
    navigatorDialog -> Close();
    vevent::StopProcessing ();
}


vkindTYPED_FULL_DEFINITION(vuNavDismissButtonClass, vbutton,
						"vuNavDismissButtonClass");

/*******************************************************************
*
*    vuNavDismissButtonClass 
*	Observer function
*
*******************************************************************/

void vuNavDismissButtonClass::ObserveDialogItem (vevent *event)
{
    char	**pvNameList = navigatorDialog -> getPvNameList ();

    // zap stuff caller could have used
    if (pvNameList)
    {
	int i = 0;
	while (pvNameList[i])
	{
	    vmemFree ( (void *)pvNameList[i]);
	    i++;
	}
	vmemFree ((void *)pvNameList);
    }
    navigatorDialog -> setPvNameList (NULL);
	navigatorDialog -> Status = vTRUE;	// this will be queried later

    // close dialog and stop event loop started in vuFindOpen ()
    navigatorDialog -> Close ();
    vevent::StopProcessing ();
}


vkindTYPED_FULL_DEFINITION(vuNavFindButtonClass, vbutton, 
						"vuNavFindButtonClass");

/*******************************************************************
*
*    vuNavFindButtonClass 
*	Observer function
*
*******************************************************************/

void vuNavFindButtonClass::ObserveDialogItem (vevent *event)
{
    vuNavContainerClass	*navContainer = navigatorDialog -> getNavContainer ();
    vuMainDialogClass   *mainDialog = navigatorDialog -> getMainDialog ();

	navContainer-> okButton->Disable();	// no list or nothing selected
    // destroy the old point name list since a new one has 
    // been requested
    vliststr *listdata = (vliststr *)(navContainer -> pointList -> GetList ());
    if (listdata)
    {
	navContainer -> pointList -> SetList (NULL);
	listdata -> Destroy ();
	// make sure that the list in the find class is set to NULL
	// after we destroy it
	navigatorDialog -> setFoundList (NULL);
    }

    // set Point Found label to nothing
    vresource msgRes = mainDialog -> getMessagesResource ();
    navContainer -> pointsFoundLabel -> SetTitleScribed (vcharScribeSystem (" "));

    // ask base class to get a list of points
    navigatorDialog -> vuFindList ();

    // get the list base class found and attach it to our listview
    listdata = navigatorDialog -> getFoundList ();
    int numInList = navigatorDialog -> getNumInList ();
    if (listdata != NULL)
    {
	navContainer -> pointList -> SetList (listdata);
	vlistSelection *selection = navContainer -> pointList -> GetSelection ();
	int singleSelect = navigatorDialog -> getSingleSelect ();
	if (singleSelect == vTRUE)
	{
	    listdata -> SetSelectMethod (selection, vlistONE_ONLY_SELECTION);
	}
	else
	{
	    listdata -> SetSelectMethod (selection, vlistFREE_SELECTION);
	}
    }
    vstr *foundMsg = vresourceGetString (msgRes, vunames_vuPtsFoundMsg);
    navContainer -> pointsFoundLabel -> SetTitleScribed (vcharScribeFormat (foundMsg, 
									    numInList));
    vstrDestroy (foundMsg);
}



vkindTYPED_FULL_DEFINITION(vuNavContainerClass, vcontainer, 
						"vuNavContainerClass");

/*******************************************************************
*
*    vuNavContainerClass 
*	override for constructor
*	We get the real resource since what comes in is the resource
*	for the find dialog and we need the resource for the
*	container
*
*******************************************************************/

void vuNavContainerClass::LoadInit (vresource res)
{
    vapplication *app = vapplication::GetCurrent ();
    vresource findDictRes = vresourceGet (app -> GetResources (),
					vunames_vuFindDictionary);
    vresource realRes = vresourceGet (findDictRes, 
					vunames_vuNavContainer);

    pointList = vuNavPtListClass::RequestSubLoad (this,
					vunames_vuNavPointList);
    pointsFoundLabel = vdialogLabelItem::RequestSubLoad (this,
					vunames_vuNavFoundLabel);

    vcontainer *buttonContainer = vcontainer::RequestSubLoad (this,
				vunames_vuNavButtonContainer);
    
    okButton = vuNavOkButtonClass::RequestSubLoad (buttonContainer,
				vunames_vuNavOKButton);

    vcontainer::LoadInit (realRes);
}


/*******************************************************************
*
*    vuNavContainerClass 
*	override 
*
*******************************************************************/

void vuNavContainerClass::Destroy (void)
{
    vcontainer::Destroy ();
}

