#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vdictHEADER
#include vlimitsHEADER
#include vmemHEADER
#include vnumHEADER
#include vlistitemHEADER
#include vdateHEADER
#include vspinnerHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbFldTypes.h"
#include "choiceGbl.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "dbRecDes.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuCommonDialogClass.hxx"
#include "vuPipeDialogClass.hxx"
#include "vuNavigatorDialogClass.hxx"
#include "vuUtils.hxx"


vkindTYPED_FULL_DEFINITION(vuPipeDialogClass, vuCommonDialogClass, "vuPipeDialogClass");

/*******************************************************************
*
*    vuPipeDialogClass override for constructors init ()
*
*******************************************************************/

void vuPipeDialogClass::LoadInit (vresource res)
{
    // load in the ai specific fields
    pipeContainer = vuPipeContainerClass::RequestSubLoad (this,
				vunames_vuCommonContainer);
    vuCommonDialogClass::LoadInit (res);

    // store dialog modified flag
    pipeContainer -> tempPV -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> processTemp -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> msTemp -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> tempUnits -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> pressPV -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> processPress -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> msPress -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> volUnits -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> runCommand -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> stopCommand -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> onScan -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> onAlarm -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> runningStatus -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> stoppedStatus -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> mode -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> volChg -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> volChgSeverity -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> initEMS -> vuSetModifiedPtr (&this -> dialogModified);
    pipeContainer -> volChgDeadband -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> monitorDeadband -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> archiveDeadband -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> product -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> settleTime -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> strappingTable -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> apiTable -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> apiSpGravity -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> alarmGroup -> SetTextModifiedPtr (&this -> dialogModified);
    pipeContainer -> alarmDisable -> SetTextModifiedPtr (&this -> dialogModified);

    // store dialog ptr for vuPvNameButtonClass
    pipeContainer -> tempPVButton -> storeCommonDialog ((vuCommonDialogClass *)this);
    pipeContainer -> pressPVButton -> storeCommonDialog ((vuCommonDialogClass *)this);
    pipeContainer -> alarmDisableButton -> storeCommonDialog ((vuCommonDialogClass *)this);

    // store dialog ptr for vuPvNameFilter
    pipeContainer -> tempPV -> storeCommonDialog ((vuCommonDialogClass *)this);
    pipeContainer -> pressPV -> storeCommonDialog ((vuCommonDialogClass *)this);
    pipeContainer -> alarmDisable -> storeCommonDialog ((vuCommonDialogClass *)this);

    // store dialog ptr for vuAlarmGroupFilter
    pipeContainer -> alarmGroup -> storeCommonDialog ((vuCommonDialogClass *)this);

    // set the point type
    setPointType (VU_PIPE);

    // get the title from the resource file, biTitle must be
    // freed by the class destuctor
    vapplication *app = vapplication::GetCurrent ();
    vresource dialogTitles = vresourceGet (app -> GetResources (),
				vunames_vuDialogTitles);
    pipeTitle = vresourceGetString (dialogTitles, 
				vunames_vuPipeDialogTitle);
    SetTitle (pipeTitle);
}


/*******************************************************************
*
*    vuPipeDialogClass Destroy function
*
*******************************************************************/

void vuPipeDialogClass::Destroy (void)
{
    vstrDestroy (pipeTitle);
    vuCommonDialogClass::Destroy ();
}


/*******************************************************************
*
*    vuPipeDialogClass, reset fields, overrides base class 
*	vuCommonDialogClass
*
*******************************************************************/

void vuPipeDialogClass::resetDialog (void)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    vchar		epics[FLDNAME_SZ + 1];

    vresource defRes = mainDialog -> getDefaultsResource ();

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0; 
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	// if not a common dialog field
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    dialogItem = vuFindContainerItem (pipeContainer, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    if ( (dialogItem -> IsKindOf (&vtextitem::Kind)) == vTRUE)
	    {
		textItem = (vtextitem *)dialogItem;

		// set the default value for the archive deadband field
		if ( (vcharCompareBounded (epics, "ADEL", FLDNAME_SZ)) == 0)
		{
		    vstr *adel = vresourceGetString (defRes, 
						vunames_vuADELdefault);
		    textItem -> SetText (adel);
		    vstrDestroy (adel);
		}
		// set the default value for the settle time
		else if ( (vcharCompareBounded (epics, "TSET", FLDNAME_SZ)) == 0)
		{
		    vstr *zero = vresourceGetString (defRes, 
						vunames_vuZeroDefault);
		    textItem -> SetText (zero);
		    vstrDestroy (zero);
		}
		else
		{
		    textItem -> SetTextScribed (vcharScribeSystem (""));
		}
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolToggle::Kind)) == vTRUE)
	    {
		toggleItem = (vcontrolToggle *)dialogItem;
		toggleItem -> SetValue (0);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolOptionMenu::Kind)) == vTRUE)
	    {
		optionMenu = (vcontrolOptionMenu *)dialogItem;
		optionMenu -> SetValue (0);
	    }
	}  // end if not in common dialog
    }
    iterator.Finish ();

    // call base class reset function with proper point index
    vuCommonDialogClass::resetDialog ();

    dialogModified = vFALSE;
}


/*******************************************************************
*
*    vuLoadDialog 
*
*******************************************************************/

void vuPipeDialogClass::vuLoadDialog (vchar *pointName)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    vchar		epics[FLDNAME_SZ + 1];
    vstr		*finalValue;

    char		pvName [PVNAME_SZ + 1];
    char		key [VU_KEY_SIZE + 1];
    DB_TABLE_PTR	lastTable;	// pointer to last table used
    DB_RECORD_PTR	pntRecord;	// codebase record ptr
    int			dbaseType;	// codebase field type
    int			intValue;
    unsigned int	uIntValue;
    long		longValue;
    unsigned long	uLongValue;
    double		doubleValue;

    vcharExportBounded (vcharScribeSystemExport (pointName),
			pvName,
			PVNAME_SZ);

    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
    pntRecord = DB_Seek (db -> vuPvmapTable, seekBuff);
    if (pntRecord == (DB_RECORD_PTR)0)
    {
	return;
    }
    else
    {
	int ptType;
	DB_GetFieldValue (pntRecord, db -> pvmapTypeField, dbValue, &dbaseType);
	ptType = vuGetPointTypeIndex ((char *)dbValue);
	// the point name in pvname field is NOT a PIPE
	if (ptType != VU_PIPE)
	{
	    vuMessageDialogCenter ((vdialog *)this, VU_PTTYPEDIALOGERR,
				   pointName, ptType, VU_PIPE);
	    vuClearPvNameField ();
	    return;
	}
	// the point name entered in pvname field has been deleted
	if (DB_IsDeleted (db -> vuPvmapTable))
	{
	    vuMessageDialogCenter ((vdialog *)this, VU_PTISDELETEDERR, pointName);
	    vuClearPvNameField ();
	    return;
	}
    }
    // call base class vuLoadDialog first
    vuCommonDialogClass::vuLoadDialog (pointName);

    lastTable = NULL;

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0;
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    // only seek if table has changed or new xrefOpId
	    if ( (lastTable != xrefNode -> xrefTable) ||
		 (strlen (xrefNode -> xrefOpId) ) )
	    {
		if (strlen (xrefNode -> xrefOpId))
		{
		    sprintf (key, "%-*.*s%-s\0", 
				   PVNAME_SZ, PVNAME_SZ, pvName,
				   xrefNode -> xrefOpId);
		    pntRecord=  DB_Seek (xrefNode -> xrefTable, key);
		}
		else
		{
		    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
		    pntRecord = DB_Seek (xrefNode -> xrefTable, seekBuff);
		}
	    }
	    lastTable = xrefNode -> xrefTable;
	    dialogItem = vuFindContainerItem (pipeContainer, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    DB_GetFieldValue (pntRecord, xrefNode -> xrefField, dbValue, &dbaseType);
	    doubleValue = 0.0;
	    // GetField actually returns a number in the char array
	    if ( (dbaseType == 'F') || (dbaseType == 'N') )
	    {
		doubleValue = *((double *)dbValue);
	    }
	    switch (xrefNode -> xrefEPICSFieldType)
	    {
		case DBF_SHORT:
		    intValue = (int)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeInt (intValue));
		    break;

		case DBF_LONG:
		    longValue = (long)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeLong (longValue));
		    break;

		case DBF_ENUM:
		case DBF_USHORT:
		    uIntValue = (unsigned int)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeUInt (uIntValue));
		    break;

		case DBF_ULONG:
		    uLongValue = (unsigned long)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeULong (uLongValue));
		    break;

		case DBF_FLOAT:
		case DBF_DOUBLE:
		    finalValue = vstrCloneScribed (vnumScribeFloat (doubleValue));
		    break;

		case DBF_STRING:
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		case DBF_GBLCHOICE:
		case DBF_RECCHOICE:
		case DBF_CVTCHOICE:
		case DBF_DEVCHOICE:
		    if (dbaseType == 'L')
		    {
			int	logical;

			logical = (*((char*)dbValue) ? 
					xrefNode -> xrefTrue : 
					(xrefNode -> xrefTrue ^1));
			finalValue = vstrCloneScribed (vnumScribeInt (logical));
			break;
		    }
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		case DBF_INLINK:
		case DBF_OUTLINK:
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		default:
		    continue;
	    }
	    if ( (dialogItem -> IsKindOf (&vtextitem::Kind)) == vTRUE)
	    {
		textItem = (vtextitem *)dialogItem;
		textItem -> SetTextScribed (vcharScribeSystem (""));
		textItem -> SetText (finalValue);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolToggle::Kind)) == vTRUE)
	    {
		toggleItem = (vcontrolToggle *)dialogItem;
		if (finalValue[0] == '0')
		    toggleItem -> SetValue (0);
		else
		    toggleItem -> SetValue (1);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolOptionMenu::Kind)) == vTRUE)
	    {
		optionMenu = (vcontrolOptionMenu *)dialogItem;
		vuSetOptionMenuValue (optionMenu, finalValue);
		vstrDestroy (finalValue);
	    }
	}  // end if not in commondialog
    }
    iterator.Finish ();

    setDialogModify (vFALSE);
}


/*******************************************************************
*
*    vuSaveDialog 
*
*******************************************************************/

void vuPipeDialogClass::vuSaveDialog (vchar *pointName)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    const vchar 	*optionMenuValue;
    vchar		epics[FLDNAME_SZ + 1];
    vstr		*stringValue;

    char		pvName [PVNAME_SZ + 1];
    char		key [VU_KEY_SIZE + 1];
    DB_FIELD_PTR	putField;
    DB_TABLE_PTR	lastTable;	// pointer to last table used
    DB_RECORD_PTR	pntRecord;	// codebase record ptr
    double		doubleValue;
    int			doSave;

    vcharExportBounded (vcharScribeSystemExport (pointName),
			pvName,
			PVNAME_SZ);

    lastTable = NULL;

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0;
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    doSave = vTRUE;
	    // only seek if table has changed
	    if  ( (lastTable != xrefNode -> xrefTable) ||
		  (strlen (xrefNode -> xrefOpId)) )
	    {
		if (strlen (xrefNode -> xrefOpId))
		{
		    sprintf (key, "%-*.*s%-s\0", 
				   PVNAME_SZ, PVNAME_SZ, pvName,
				   xrefNode -> xrefOpId);
		    pntRecord=  DB_Seek (xrefNode -> xrefTable, key);
		}
		else
		{
		    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
		    pntRecord = DB_Seek (xrefNode -> xrefTable, seekBuff);
		}
	    }
	    lastTable = xrefNode -> xrefTable;

	    putField = xrefNode -> xrefField;

	    dialogItem = vuFindContainerItem (pipeContainer, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    switch (xrefNode -> xrefEPICSFieldType)
	    {
		case DBF_SHORT:
		case DBF_LONG:
		case DBF_ENUM:
		case DBF_USHORT:
		case DBF_ULONG:
		case DBF_FLOAT:
		case DBF_DOUBLE:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    doubleValue = atof ((const char *)dbValue);
		    DB_PutFieldValue (pntRecord, putField, (void *)&doubleValue);
		    doSave = vFALSE;
		    break;

		case DBF_STRING:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    break;

		case DBF_GBLCHOICE:
		case DBF_RECCHOICE:
		case DBF_CVTCHOICE:
		case DBF_DEVCHOICE:
		{
		    if (xrefNode -> xrefField -> field_type == 'L')
		    {
			int	logical;
			toggleItem = (vcontrolToggle *)dialogItem;
			logical = toggleItem -> GetValue ();
			*((char *)dbValue) = (logical ? xrefNode -> xrefTrue : 
								xrefNode -> xrefTrue ^1);
			break;
		    }
		    optionMenu = (vcontrolOptionMenu *)dialogItem;
		    optionMenuValue = vuGetOptionMenuValue (optionMenu);
		    vcharExport (vcharScribeSystemExport (optionMenuValue), dbValue);
		    break;
		}

		case DBF_INLINK:
		case DBF_OUTLINK:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    break;

		default:
		    continue;

	    }  // end xrefNode -> xrefEPICSFieldType

	    if (doSave == vTRUE)
	    {
		DB_PutFieldValue (pntRecord, putField, dbValue);
	    }

	}  // end if not in commondialog

    }  // end while iterator

    iterator.Finish ();

    // call base class vuSaveDialog
    vuCommonDialogClass::vuSaveDialog (pointName);
}


vkindTYPED_FULL_DEFINITION(vuPipeContainerClass, vcontainer, "vuPipeContainerClass");

/*******************************************************************
*
*    vuPipeContainerClass override for constructor
*
*******************************************************************/

void vuPipeContainerClass::LoadInit (vresource res)
{
    vapplication *app = vapplication::GetCurrent ();
    vresource pointDictRes = vresourceGet (app -> GetResources (), 
				vunames_vuPointDictionary);
    vresource realRes = vresourceGet (pointDictRes, vunames_vuPipeContainer);

    vcontainer *tepvcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeTEPVContainer);
    tempPVButton = vuPvNameButtonClass::RequestSubLoad (tepvcontainer,
				vunames_vuPipeTEPVButton);
    tempPV = vuFilteredTextitem::RequestSubLoad (tepvcontainer,
				vunames_TEPV);
    processTemp = vuCheckBoxClass::RequestSubLoad (this,
				vunames_PRC2);
    msTemp = vuCheckBoxClass::RequestSubLoad (this,
				vunames_MXS2);

    vcontainer *tmpucontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeTMPUContainer);
    tempUnits = vuChoiceGblMenuClass::RequestSubLoad (tmpucontainer,
				vunames_TMPU);

    vcontainer *prpvcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipePRPVContainer);
    pressPVButton = vuPvNameButtonClass::RequestSubLoad (prpvcontainer,
				vunames_vuPipePRPVButton);
    pressPV = vuFilteredTextitem::RequestSubLoad (prpvcontainer,
				vunames_PRPV);
    processPress = vuCheckBoxClass::RequestSubLoad (this,
				vunames_PRC1);
    msPress = vuCheckBoxClass::RequestSubLoad (this,
				vunames_MXS1);

    vcontainer *volucontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeVOLUContainer);
    volUnits = vuChoiceGblMenuClass::RequestSubLoad (volucontainer,
				vunames_VOLU);

    vcontainer *runccontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeRUNCContainer);
    runCommand = filteredTextitem::RequestSubLoad (runccontainer,
				vunames_RUNC);

    vcontainer *stpcmdcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeSTPCContainer);
    stopCommand = filteredTextitem::RequestSubLoad (stpcmdcontainer,
				vunames_STPC);

    onScan = vuCheckBoxClass::RequestSubLoad (this,
				vunames_OSCN);

    onAlarm = vuCheckBoxClass::RequestSubLoad (this,
				vunames_AENB);

    vcontainer *runscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeRUNSContainer);
    runningStatus = filteredTextitem::RequestSubLoad (runscontainer,
				vunames_RUNS);

    vcontainer *stpscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeSTPSContainer);
    stoppedStatus = filteredTextitem::RequestSubLoad (stpscontainer,
				vunames_STPS);

    vcontainer *modecontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeMODEContainer);
    mode = vuChoiceGblMenuClass::RequestSubLoad (modecontainer,
				vunames_MODE);

    vcontainer *voldcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeVOLDContainer);
    volChg = filteredTextitem::RequestSubLoad (voldcontainer,
				vunames_VOLD);

    vcontainer *alsvcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeALSVContainer);
    volChgSeverity = vuChoiceGblMenuClass::RequestSubLoad (alsvcontainer,
				vunames_ALSV);

    initEMS = vuCheckBoxClass::RequestSubLoad (this,
				vunames_IEMS);

    vcontainer *hystcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeHYSTContainer);
    volChgDeadband = filteredTextitem::RequestSubLoad (hystcontainer,
				vunames_HYST);

    vcontainer *mdelcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeMDELContainer);
    monitorDeadband = filteredTextitem::RequestSubLoad (mdelcontainer,
				vunames_MDEL);

    vcontainer *adelcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeADELContainer);
    archiveDeadband = filteredTextitem::RequestSubLoad (adelcontainer,
				vunames_ADEL);

    vcontainer *prodcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipePRODContainer);
    product = filteredTextitem::RequestSubLoad (prodcontainer,
				vunames_PROD);

    vcontainer *tsetcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeTSETContainer);
    settleTime = filteredTextitem::RequestSubLoad (tsetcontainer,
				vunames_TSET);

    vcontainer *sttncontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeSTTNContainer);
    strappingTable = filteredTextitem::RequestSubLoad (sttncontainer,
				vunames_STTN);

    vcontainer *apitcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeAPITContainer);
    apiTable = filteredTextitem::RequestSubLoad (apitcontainer,
				vunames_APIT);

    vcontainer *sgrvcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeSGRVContainer);
    apiSpGravity = filteredTextitem::RequestSubLoad (sgrvcontainer,
				vunames_SGRV);

    vcontainer *alrmgrpcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeALGPContainer);
    alarmGroup = vuFilteredTextitem::RequestSubLoad (alrmgrpcontainer,
				vunames_ALGP);

    vcontainer *alarmdiscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuPipeADISContainer);
    alarmDisableButton = vuPvNameButtonClass::RequestSubLoad (alarmdiscontainer,
				vunames_vuPipeADISButton);
    alarmDisable = vuFilteredTextitem::RequestSubLoad (alarmdiscontainer,
				vunames_ADIS);

    vcontainer::LoadInit (realRes);

    // store ptr to text field for vuPvNameButtonClass
    tempPVButton -> storePvName (tempPV);
    pressPVButton -> storePvName (pressPV);
    alarmDisableButton -> storePvName (alarmDisable);

    // set filters
    runCommand -> SetItemFilterFunc (tankfarmCmdStsFilter);
    stopCommand -> SetItemFilterFunc (tankfarmCmdStsFilter);
    runningStatus -> SetItemFilterFunc (tankfarmCmdStsFilter);
    stoppedStatus -> SetItemFilterFunc (tankfarmCmdStsFilter);
    volChg -> SetItemFilterFunc (floatFilter);
    volChgDeadband -> SetItemFilterFunc (floatFilter);
    monitorDeadband -> SetItemFilterFunc (floatFilter);
    archiveDeadband -> SetItemFilterFunc (floatFilter);
    product -> SetItemFilterFunc (tankfarmCmdStsFilter);
    settleTime -> SetItemFilterFunc (floatFilter);
    strappingTable -> SetItemFilterFunc (tankfarmCmdStsFilter);
    apiTable -> SetItemFilterFunc (tankfarmCmdStsFilter);
    apiSpGravity -> SetItemFilterFunc (floatFilter);


    tempPV -> SetItemFilterFunc (vuPvNameFilter);
    filteredText *tempText = (filteredText *)tempPV -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    tempText -> SetUserPtr ((void *)tempPV);

    pressPV -> SetItemFilterFunc (vuPvNameFilter);
    filteredText *pressText = (filteredText *)pressPV -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    pressText -> SetUserPtr ((void *)pressPV);

    alarmDisable -> SetItemFilterFunc (vuPvNameFilter);
    filteredText *alrmDisText = (filteredText *)alarmDisable -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    alrmDisText -> SetUserPtr ((void *)alarmDisable);

    alarmGroup -> SetItemFilterFunc (vuAlarmGroupFilter);
    filteredText *alarmText = (filteredText *)alarmGroup -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    alarmText -> SetUserPtr ((void *)alarmGroup);

    // build popup menus
    tempUnits -> vuBuildOptionMenu (GBL_TF_TEMP);
    volUnits -> vuBuildOptionMenu (GBL_TF_VOLUME);
    mode -> vuBuildOptionMenu (GBL_TF_AUTO_MAN_LOCK);
    volChgSeverity -> vuBuildOptionMenu (GBL_ALARM_SEV);
}

/*******************************************************************
*
*    vuPipeContainerClass override for destructor
*
*******************************************************************/

void vuPipeContainerClass::Destroy (void)
{
    vcontainer::Destroy ();
}
