#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vdictHEADER
#include vlimitsHEADER
#include vmemHEADER
#include vnumHEADER
#include vlistitemHEADER
#include vdateHEADER
#include vspinnerHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbFldTypes.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "dbRecDes.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuCommonDialogClass.hxx"
#include "vuRampDialogClass.hxx"
#include "vuNavigatorDialogClass.hxx"
#include "vuUtils.hxx"


vkindTYPED_FULL_DEFINITION(vuRampDialogClass, vuCommonDialogClass, "vuRampDialogClass");

/*******************************************************************
*
*    vuRampDialogClass override for constructors init ()
*
*******************************************************************/

void vuRampDialogClass::LoadInit (vresource res)
{
    // load in the ai specific fields
    rampContainer = vuRampContainerClass::RequestSubLoad (this,
				vunames_vuCommonContainer);
    vuCommonDialogClass::LoadInit (res);

    // store dialog modified flag
    rampContainer -> reference -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> eguTag -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> manual -> vuSetModifiedPtr (&this -> dialogModified);
    rampContainer -> onScan -> vuSetModifiedPtr (&this -> dialogModified);
    rampContainer -> processInput -> vuSetModifiedPtr (&this -> dialogModified);
    rampContainer -> maximizeSeverity -> vuSetModifiedPtr (&this -> dialogModified);
    rampContainer -> increaseRate -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> decreaseRate -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> noiseFilter -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> precision -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> monitorDeadband -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> archiveDeadband -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> displayLow -> SetTextModifiedPtr (&this -> dialogModified);
    rampContainer -> displayHigh -> SetTextModifiedPtr (&this -> dialogModified);

    // store dialog ptr for vuPvNameButtonClass
    rampContainer -> referenceButton -> storeCommonDialog ((vuCommonDialogClass *)this);

    // store dialog ptr for vuPvNameButtonClass
    rampContainer -> reference -> storeCommonDialog ((vuCommonDialogClass *)this);

    // set the point type
    setPointType (VU_RAMP);

    // get the title from the resource file, aiTitle must be
    // freed by the class destuctor
    vapplication *app = vapplication::GetCurrent ();
    vresource dialogTitles = vresourceGet (app -> GetResources (),
				vunames_vuDialogTitles);
    rampTitle = vresourceGetString (dialogTitles, 
				vunames_vuRampDialogTitle);
    SetTitle (rampTitle);
}


/*******************************************************************
*
*    vuRampDialogClass Destroy function
*
*******************************************************************/

void vuRampDialogClass::Destroy (void)
{
    vstrDestroy (rampTitle);
    vuCommonDialogClass::Destroy ();
}


/*******************************************************************
*
*    vuRampDialogClass, reset fields, overrides base class 
*	vuCommonDialogClass
*
*******************************************************************/

void vuRampDialogClass::resetDialog (void)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    vchar		epics[FLDNAME_SZ + 1];

    vresource defRes = mainDialog -> getDefaultsResource ();

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0; 
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	// if not a common dialog field
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    dialogItem = vuFindContainerItem (rampContainer, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    if ( (dialogItem -> IsKindOf (&vtextitem::Kind)) == vTRUE)
	    {
		textItem = (vtextitem *)dialogItem;

		// set the default value for the archive deadband field
		if ( (vcharCompareBounded (epics, "ADEL", FLDNAME_SZ)) == 0)
		{
		    vstr *adel = vresourceGetString (defRes, 
						vunames_vuADELdefault);
		    textItem -> SetText (adel);
		    vstrDestroy (adel);
		}
		// set the default value for the ramp's precision field
		else if ( (vcharCompareBounded (epics, "PREC", FLDNAME_SZ)) == 0)
		{
		    vstr *precision = vresourceGetString (defRes, 
						vunames_vuPRECdefault);
		    textItem -> SetText (precision);
		    vstrDestroy (precision);
		}
		else
		{
		    textItem -> SetTextScribed (vcharScribeSystem (""));
		}
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolToggle::Kind)) == vTRUE)
	    {
		toggleItem = (vcontrolToggle *)dialogItem;
		toggleItem -> SetValue (0);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolOptionMenu::Kind)) == vTRUE)
	    {
		optionMenu = (vcontrolOptionMenu *)dialogItem;
		optionMenu -> SetValue (0);
	    }
	}  // end if not in common dialog
    }
    iterator.Finish ();

    // call base class reset function with proper point index
    vuCommonDialogClass::resetDialog ();

    dialogModified = vFALSE;
}


/*******************************************************************
*
*    vuLoadDialog 
*
*******************************************************************/

void vuRampDialogClass::vuLoadDialog (vchar *pointName)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    vchar		epics[FLDNAME_SZ + 1];
    vstr		*finalValue;

    char		pvName [PVNAME_SZ + 1];
    DB_TABLE_PTR	lastTable;	// pointer to last table used
    DB_RECORD_PTR	pntRecord;	// codebase record ptr
    int			dbaseType;	// codebase field type
    int			intValue;
    unsigned int	uIntValue;
    long		longValue;
    unsigned long	uLongValue;
    double		doubleValue;

    vcharExportBounded (vcharScribeSystemExport (pointName),
			pvName,
			PVNAME_SZ);

    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
    pntRecord = DB_Seek (db -> vuPvmapTable, seekBuff);
    if (pntRecord == (DB_RECORD_PTR)0)
    {
	return;
    }
    else
    {
	int ptType;
	DB_GetFieldValue (pntRecord, db -> pvmapTypeField, dbValue, &dbaseType);
	ptType = vuGetPointTypeIndex ((char *)dbValue);
	// the point name in pvname field is NOT an RAMP
	if (ptType != VU_RAMP)
	{
	    vuMessageDialogCenter ((vdialog *)this, VU_PTTYPEDIALOGERR,
				   pointName, ptType, VU_RAMP);
	    vuClearPvNameField ();
	    return;
	}
	// the point name entered in pvname field has been deleted
	if (DB_IsDeleted (db -> vuPvmapTable))
	{
	    vuMessageDialogCenter ((vdialog *)this, VU_PTISDELETEDERR, pointName);
	    vuClearPvNameField ();
	    return;
	}
    }
    // call base class vuLoadDialog first
    vuCommonDialogClass::vuLoadDialog (pointName);

    lastTable = NULL;

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0;
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    // only seek if table has changed
	    if (lastTable != xrefNode -> xrefTable)
		{
		    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
			pntRecord = DB_Seek (xrefNode -> xrefTable, seekBuff);
		}
	    lastTable = xrefNode -> xrefTable;
	    dialogItem = vuFindContainerItem (rampContainer, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    DB_GetFieldValue (pntRecord, xrefNode -> xrefField, dbValue, &dbaseType);
	    doubleValue = 0.0;
	    // GetField actually returns a number in the char array
	    if ( (dbaseType == 'F') || (dbaseType == 'N') )
	    {
		doubleValue = *((double *)dbValue);
	    }
	    switch (xrefNode -> xrefEPICSFieldType)
	    {
		case DBF_SHORT:
		    intValue = (int)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeInt (intValue));
		    break;

		case DBF_LONG:
		    longValue = (long)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeLong (longValue));
		    break;

		case DBF_ENUM:
		case DBF_USHORT:
		    uIntValue = (unsigned int)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeUInt (uIntValue));
		    break;

		case DBF_ULONG:
		    uLongValue = (unsigned long)doubleValue;
		    finalValue = vstrCloneScribed (vnumScribeULong (uLongValue));
		    break;

		case DBF_FLOAT:
		case DBF_DOUBLE:
		    finalValue = vstrCloneScribed (vnumScribeFloat (doubleValue));
		    break;

		case DBF_STRING:
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		case DBF_GBLCHOICE:
		case DBF_RECCHOICE:
		case DBF_CVTCHOICE:
		case DBF_DEVCHOICE:
		    if (dbaseType == 'L')
		    {
			int	logical;

			logical = (*((char*)dbValue) ? 
					xrefNode -> xrefTrue : 
					(xrefNode -> xrefTrue ^1));
			finalValue = vstrCloneScribed (vnumScribeInt (logical));
			break;
		    }
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		case DBF_INLINK:
		    finalValue = vstrCloneScribed (vcharScribeSystem (dbValue));
		    break;

		default:
		    continue;
	    }
	    if ( (dialogItem -> IsKindOf (&vtextitem::Kind)) == vTRUE)
	    {
		textItem = (vtextitem *)dialogItem;
		textItem -> SetTextScribed (vcharScribeSystem (""));
		textItem -> SetText (finalValue);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolToggle::Kind)) == vTRUE)
	    {
		toggleItem = (vcontrolToggle *)dialogItem;
		if (finalValue[0] == '0')
		    toggleItem -> SetValue (0);
		else
		    toggleItem -> SetValue (1);
		vstrDestroy (finalValue);
	    }
	    else if ( (dialogItem -> IsKindOf (&vcontrolOptionMenu::Kind)) == vTRUE)
	    {
		optionMenu = (vcontrolOptionMenu *)dialogItem;
		vuSetOptionMenuValue (optionMenu, finalValue);
		vstrDestroy (finalValue);
	    }
	}  // end if not in commondialog
    }
    iterator.Finish ();

    setDialogModify (vFALSE);
}


/*******************************************************************
*
*    vuSaveDialog 
*
*******************************************************************/

void vuRampDialogClass::vuSaveDialog (vchar *pointName)
{
    vuMainDialogClass	*mainDialog = getMainDialog ();
    dataVuApplication	*app  = mainDialog -> getApplication ();
    vuDatabaseClass	*db   = app -> getDatabaseClass ();
    vdict		*dict = db -> getXrefDict (pointType);
    vdictIterator	iterator;
    vname		*epicsName;
    xrefNodeClass	*xrefNode;
    vdialogItem		*dialogItem;
    vtextitem		*textItem;
    vcontrolToggle	*toggleItem;
    vcontrolOptionMenu	*optionMenu;
    const vchar 	*optionMenuValue;
    vchar		epics[FLDNAME_SZ + 1];
    vstr		*stringValue;

    char		pvName [PVNAME_SZ + 1];
    DB_FIELD_PTR	putField;
    DB_TABLE_PTR	lastTable;	// pointer to last table used
    DB_RECORD_PTR	pntRecord;	// codebase record ptr
    double		doubleValue;
    int			doSave;

    vcharExportBounded (vcharScribeSystemExport (pointName),
			pvName,
			PVNAME_SZ);

    lastTable = NULL;

    iterator.Start (dict);
    while ( iterator.Next ())
    {
	epicsName = (vname *)iterator.GetKey ();
	vmemSet (epics, ' ', sizeof (epics));
	vcharCopyBounded (epicsName, epics, FLDNAME_SZ);
	epics[sizeof (epics) - 1] = 0;
	xrefNode  = (xrefNodeClass *)iterator.GetValue ();
	if ( (xrefNode -> getInCommonDialog ()) == vFALSE)
	{
	    doSave = vTRUE;
	    // only seek if table has changed
	    if (lastTable != xrefNode -> xrefTable)
		{
		    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
			pntRecord = DB_Seek (xrefNode -> xrefTable, seekBuff);
		}
	    lastTable = xrefNode -> xrefTable;

	    putField = xrefNode -> xrefField;

	    dialogItem = vuFindContainerItem (rampContainer, epics);
	    if (dialogItem == NULL)
		continue;
	    if ( (dialogItem -> IsEnabled ()) == vFALSE)
		continue;
	    switch (xrefNode -> xrefEPICSFieldType)
	    {
		case DBF_SHORT:
		case DBF_LONG:
		case DBF_ENUM:
		case DBF_USHORT:
		case DBF_ULONG:
		case DBF_FLOAT:
		case DBF_DOUBLE:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    doubleValue = atof ((const char *)dbValue);
		    DB_PutFieldValue (pntRecord, putField, (void *)&doubleValue);
		    doSave = vFALSE;
		    break;

		case DBF_STRING:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    break;

		case DBF_GBLCHOICE:
		case DBF_RECCHOICE:
		case DBF_CVTCHOICE:
		case DBF_DEVCHOICE:
		{
		    if (xrefNode -> xrefField -> field_type == 'L')
		    {
			int	logical;
			toggleItem = (vcontrolToggle *)dialogItem;
			logical = toggleItem -> GetValue ();
			*((char *)dbValue) = (logical ? xrefNode -> xrefTrue : 
								xrefNode -> xrefTrue ^1);
			break;
		    }
		    optionMenu = (vcontrolOptionMenu *)dialogItem;
		    optionMenuValue = vuGetOptionMenuValue (optionMenu);
		    vcharExport (vcharScribeSystemExport (optionMenuValue), dbValue);
		    break;
		}

		case DBF_INLINK:
		    textItem = (vtextitem *)dialogItem;
		    stringValue = textItem -> GetTextAsString ();
		    vcharExport (vcharScribeSystemExport (stringValue), dbValue);
		    vstrDestroy (stringValue);
		    break;

		default:
		    continue;

	    }  // end xrefNode -> xrefEPICSFieldType

	    if (doSave == vTRUE)
	    {
		DB_PutFieldValue (pntRecord, putField, dbValue);
	    }

	}  // end if not in commondialog

    }  // end while iterator

    iterator.Finish ();

    // call base class vuSaveDialog
    vuCommonDialogClass::vuSaveDialog (pointName);
}


vkindTYPED_FULL_DEFINITION(vuRampContainerClass, vcontainer, "vuRampContainerClass");

/*******************************************************************
*
*    vuRampContainerClass override for constructor
*
*******************************************************************/

void vuRampContainerClass::LoadInit (vresource res)
{
    vapplication *app = vapplication::GetCurrent ();
    vresource pointDictRes = vresourceGet (app -> GetResources (), 
				vunames_vuPointDictionary);
    vresource realRes = vresourceGet (pointDictRes, vunames_vuRampContainer);

    vcontainer *refcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampINPContainer);
    referenceButton = vuPvNameButtonClass::RequestSubLoad (refcontainer,
				vunames_vuRampINPButton);
    reference = vuFilteredTextitem::RequestSubLoad (refcontainer,
				vunames_INP);

    vcontainer *egutagcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampEGUContainer);
    eguTag = filteredTextitem::RequestSubLoad (egutagcontainer,
				vunames_EGU);

    manual = vuCheckBoxClass::RequestSubLoad (this,
				vunames_MODE);

    onScan = vuCheckBoxClass::RequestSubLoad (this,
				vunames_OSCN);

    processInput = vuCheckBoxClass::RequestSubLoad (this,
				vunames_PRCI);

    maximizeSeverity = vuCheckBoxClass::RequestSubLoad (this,
				vunames_MAXS);

    vcontainer *ircontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampINCRContainer);
    increaseRate = filteredTextitem::RequestSubLoad (ircontainer,
				vunames_INCR);

    vcontainer *drcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampDECRContainer);
    decreaseRate = filteredTextitem::RequestSubLoad (drcontainer,
				vunames_DECR);

    vcontainer *noiscontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampNOISContainer);
    noiseFilter = filteredTextitem::RequestSubLoad (noiscontainer,
				vunames_NOIS);

    vcontainer *preccontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampPRECContainer);
    precision = filteredTextitem::RequestSubLoad (preccontainer,
				vunames_PREC);

    vcontainer *mdbcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampMDELContainer);
    monitorDeadband = filteredTextitem::RequestSubLoad (mdbcontainer,
				vunames_MDEL);

    vcontainer *archcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampADELContainer);
    archiveDeadband = filteredTextitem::RequestSubLoad (archcontainer,
				vunames_ADEL);

    vcontainer *dsplowcontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampLOPRContainer);
    displayLow = filteredTextitem::RequestSubLoad (dsplowcontainer,
				vunames_LOPR);

    vcontainer *dsphicontainer = vcontainer::RequestSubLoad (this,
				vunames_vuRampHOPRContainer);
    displayHigh = filteredTextitem::RequestSubLoad (dsphicontainer,
				vunames_HOPR);

    vcontainer::LoadInit (realRes);

    // store ptr to text field for vuPvNameButtonClass
    referenceButton -> storePvName (reference);

    // set filters
    reference -> SetItemFilterFunc (vuPvNameFilter);
    filteredText *refText = (filteredText *)reference -> GetTextData ();
    // remember ptr to textitem for later use by the filter
    refText -> SetUserPtr ((void *)reference);

    eguTag -> SetItemFilterFunc (egutagFilter);
    increaseRate -> SetItemFilterFunc (floatFilter);
    decreaseRate -> SetItemFilterFunc (floatFilter);
    noiseFilter -> SetItemFilterFunc (floatFilter);
    precision -> SetItemFilterFunc (numericFilter);
    monitorDeadband -> SetItemFilterFunc (floatFilter);
    archiveDeadband -> SetItemFilterFunc (floatFilter);
    displayLow -> SetItemFilterFunc (floatFilter);
    displayHigh -> SetItemFilterFunc (floatFilter);

}

/*******************************************************************
*
*    vuRampContainerClass override for destructor
*
*******************************************************************/

void vuRampContainerClass::Destroy (void)
{
    vcontainer::Destroy ();
}

