#ifndef vutankdialogclassHXX
#define	vutankdialogclassHXX

// define values for the xrefNode inDialog flag

enum {
    VU_TANK_NOT_IN_DIALOG = 1,
    VU_TANK_IN_MAIN_DIALOG,
    VU_TANK_IN_ALARM_DIALOG,
    VU_TANK_IN_VALVE_DIALOG,
    VU_TANK_IN_TEMP_DIALOG,
    VU_TANK_IN_PV_DIALOG
};

class vuTankAlarmButtonClass;
class vuTankValveButtonClass;
class vuTankTempButtonClass;
class vuTankPvButtonClass;

class vuTankAlarmDialogClass;
class vuTankAlarmDismissButtonClass;
class vuTankValveDialogClass;
class vuTankValveDismissButtonClass;
class vuTankTempDialogClass;
class vuTankTempDismissButtonClass;
class vuTankPvDialogClass;
class vuTankPvDismissButtonClass;


//*** TANK CONTAINER CLASS *******************************************

class	vuTankContainerClass : public vcontainer
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuTankContainerClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuTankContainerClass, vcontainer);

    // vloadable override
    void LoadInit (vresource resource);

    // override 
    void Destroy ();

    // pointers to fields in container, public so don't need funcs to
    // get at them from vuTankDialogClass

    // tank characteristics
    vuChoiceRecMenuClass	*tankShape;
    filteredTextitem		*height;
    filteredTextitem		*diameter;
    filteredTextitem		*elevation;
    filteredTextitem		*product;

    //tank commands
    filteredTextitem		*issueCmd;
    filteredTextitem		*receiveCmd;
    filteredTextitem		*runCmd;
    filteredTextitem		*stopCmd;
    filteredTextitem		*issueStatus;
    filteredTextitem		*receiveStatus;
    filteredTextitem		*runStatus;
    filteredTextitem		*stopStatus;

    filteredTextitem		*settleTime;
    vuChoiceGblMenuClass	*mode;
    vuCheckBoxClass		*onScan;
    filteredTextitem		*strappingTable;
    filteredTextitem		*apiTable;
    filteredTextitem		*apiSpGravity;
    filteredTextitem		*monitorDeadband;
    filteredTextitem		*archiveDeadband;

    // tank roof characteristics
    vuChoiceRecMenuClass	*roofType;
    filteredTextitem		*roofWgt;
    filteredTextitem		*landedLevel;
    filteredTextitem		*floatLevel;
    filteredTextitem		*notInstalled;
    filteredTextitem		*critical;
    filteredTextitem		*landed;
    filteredTextitem		*floating;

    // tank sensors
    vuChoiceRecMenuClass	*productMeasMeth;
    vuChoiceRecMenuClass	*bswMeasMeth;
    filteredTextitem		*productSensorLevel;
    filteredTextitem		*bswSensorLevel;
    filteredTextitem		*hiLevelTrip;
    filteredTextitem		*lowLevelTrip;

    // some buttons
    vuTankAlarmButtonClass	*alarmButton;
    vuTankValveButtonClass	*valveButton;
    vuTankTempButtonClass	*tempButton;
    vuTankPvButtonClass		*pvButton;

};


//*** TANK ALARM BUTTON CLASS ***************************************

class	vuTankAlarmButtonClass : public vbutton
{
    vuTankAlarmDialogClass	*alarmDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankAlarmButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankAlarmButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle alarmDialog
    void storeTankAlarmDialog (vuTankAlarmDialogClass *cd)
	{ alarmDialog = cd; }
};


//*** TANK VALVE BUTTON CLASS ***************************************

class	vuTankValveButtonClass : public vbutton
{
    vuTankValveDialogClass	*valveDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankValveButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankValveButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle valveDialog
    void storeTankValveDialog (vuTankValveDialogClass *cd)
	{ valveDialog = cd; }
};


//*** TANK TEMP BUTTON CLASS ***************************************

class	vuTankTempButtonClass : public vbutton
{
    vuTankTempDialogClass	*tempDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankTempButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankTempButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle tempDialog
    void storeTankTempDialog (vuTankTempDialogClass *cd)
	{ tempDialog = cd; }
};


//*** TANK PV BUTTON CLASS ***************************************

class	vuTankPvButtonClass : public vbutton
{
    vuTankPvDialogClass	*pvDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankPvButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankPvButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle pvDialog
    void storeTankPvDialog (vuTankPvDialogClass *cd)
	{ pvDialog = cd; }
};


//*** TANK DIALOG CLASS *********************************************

class	vuTankDialogClass : public vuCommonDialogClass
{
    vuTankContainerClass	*tankContainer;
    vuTankAlarmDialogClass	*alarmDialog;
    vuTankValveDialogClass	*valveDialog;
    vuTankTempDialogClass	*tempDialog;
    vuTankPvDialogClass		*pvDialog;
    vstr			*tankTitle;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuTankDialogClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuTankDialogClass, vuCommonDialogClass);

    // vloadable override
    void LoadInit (vresource resource);

    // override
    void Destroy ();

    // overide base vuCommonDialogClass
    void resetDialog (void);

    // override base class load and save functions
    void vuLoadDialog (vchar *pointName);
    void vuSaveDialog (vchar *pointName);
};


//*** TANK ALARM DIALOG CLASS ************************************

class	vuTankAlarmDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuTankAlarmDialogClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuTankAlarmDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // public so no funcs needed to access
    filteredTextitem		*hihiLevel;
    vuChoiceGblMenuClass	*hihiSeverity;
    vuCheckBoxClass		*hihiES;
    filteredTextitem		*highLevel;
    vuChoiceGblMenuClass	*highSeverity;
    vuCheckBoxClass		*highES;
    filteredTextitem		*lowLevel;
    vuChoiceGblMenuClass	*lowSeverity;
    vuCheckBoxClass		*lowES;
    filteredTextitem		*loloLevel;
    vuChoiceGblMenuClass	*loloSeverity;
    vuCheckBoxClass		*loloES;
    filteredTextitem		*volChg;
    vuChoiceGblMenuClass	*volChgSeverity;
    vuCheckBoxClass		*volChgES;
    filteredTextitem		*hihiTemp;
    vuChoiceGblMenuClass	*hihiTempSeverity;
    vuCheckBoxClass		*hihiTempES;
    filteredTextitem		*highTemp;
    vuChoiceGblMenuClass	*highTempSeverity;
    vuCheckBoxClass		*highTempES;
    filteredTextitem		*lowTemp;
    vuChoiceGblMenuClass	*lowTempSeverity;
    vuCheckBoxClass		*lowTempES;
    filteredTextitem		*loloTemp;
    vuChoiceGblMenuClass	*loloTempSeverity;
    vuCheckBoxClass		*loloTempES;
    filteredTextitem		*highBsw;
    vuChoiceGblMenuClass	*highBswSeverity;
    vuCheckBoxClass		*highBswES;
    vuChoiceGblMenuClass	*lowLevelTripSeverity;
    vuCheckBoxClass		*lowLevelTripES;
    vuChoiceGblMenuClass	*highLevelTripSeverity;
    vuCheckBoxClass		*highLevelTripES;
    vuChoiceGblMenuClass	*valveStateSeverity;
    vuCheckBoxClass		*valveStateES;
    vuChoiceGblMenuClass	*lowPressSeverity;
    vuCheckBoxClass		*lowPressES;
    vuChoiceGblMenuClass	*highPressSeverity;
    vuCheckBoxClass		*highPressES;
    filteredTextitem		*levelDeadband;
    filteredTextitem		*volumeDeadband;
    filteredTextitem		*tempDeadband;
    filteredTextitem		*bswDeadband;
    vuCheckBoxClass		*onAlarm;
    vuFilteredTextitem		*alarmGroup;
    vuPvNameButtonClass		*alarmDisableButton;
    vuFilteredTextitem		*alarmDisable;

    vuTankAlarmDismissButtonClass	*dismissButton;
};


//*** TANK ALARM DISMISS BUTTON CLASS ********************************

class	vuTankAlarmDismissButtonClass : public vbutton
{
    vuTankAlarmDialogClass	*alarmDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankAlarmDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankAlarmDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle alarmDialog
    void storeTankAlarmDialog (vuTankAlarmDialogClass *cd)
	{ alarmDialog = cd; }
};


//*** TANK VALVE DIALOG CLASS ************************************

class	vuTankValveDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuTankValveDialogClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuTankValveDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // public so no funcs needed to access
    vuPvNameButtonClass		*valve01Button;
    vuFilteredTextitem		*valve01;
    vuChoiceRecMenuClass	*stopped01;
    vuChoiceRecMenuClass	*issuing01;
    vuChoiceRecMenuClass	*receiving01;
    vuChoiceRecMenuClass	*running01;

    vuPvNameButtonClass		*valve02Button;
    vuFilteredTextitem		*valve02;
    vuChoiceRecMenuClass	*stopped02;
    vuChoiceRecMenuClass	*issuing02;
    vuChoiceRecMenuClass	*receiving02;
    vuChoiceRecMenuClass	*running02;

    vuPvNameButtonClass		*valve03Button;
    vuFilteredTextitem		*valve03;
    vuChoiceRecMenuClass	*stopped03;
    vuChoiceRecMenuClass	*issuing03;
    vuChoiceRecMenuClass	*receiving03;
    vuChoiceRecMenuClass	*running03;

    vuPvNameButtonClass		*valve04Button;
    vuFilteredTextitem		*valve04;
    vuChoiceRecMenuClass	*stopped04;
    vuChoiceRecMenuClass	*issuing04;
    vuChoiceRecMenuClass	*receiving04;
    vuChoiceRecMenuClass	*running04;

    vuPvNameButtonClass		*valve05Button;
    vuFilteredTextitem		*valve05;
    vuChoiceRecMenuClass	*stopped05;
    vuChoiceRecMenuClass	*issuing05;
    vuChoiceRecMenuClass	*receiving05;
    vuChoiceRecMenuClass	*running05;

    vuPvNameButtonClass		*valve06Button;
    vuFilteredTextitem		*valve06;
    vuChoiceRecMenuClass	*stopped06;
    vuChoiceRecMenuClass	*issuing06;
    vuChoiceRecMenuClass	*receiving06;
    vuChoiceRecMenuClass	*running06;

    vuPvNameButtonClass		*valve07Button;
    vuFilteredTextitem		*valve07;
    vuChoiceRecMenuClass	*stopped07;
    vuChoiceRecMenuClass	*issuing07;
    vuChoiceRecMenuClass	*receiving07;
    vuChoiceRecMenuClass	*running07;

    vuPvNameButtonClass		*valve08Button;
    vuFilteredTextitem		*valve08;
    vuChoiceRecMenuClass	*stopped08;
    vuChoiceRecMenuClass	*issuing08;
    vuChoiceRecMenuClass	*receiving08;
    vuChoiceRecMenuClass	*running08;

    vuPvNameButtonClass		*valve09Button;
    vuFilteredTextitem		*valve09;
    vuChoiceRecMenuClass	*stopped09;
    vuChoiceRecMenuClass	*issuing09;
    vuChoiceRecMenuClass	*receiving09;
    vuChoiceRecMenuClass	*running09;

    vuPvNameButtonClass		*valve10Button;
    vuFilteredTextitem		*valve10;
    vuChoiceRecMenuClass	*stopped10;
    vuChoiceRecMenuClass	*issuing10;
    vuChoiceRecMenuClass	*receiving10;
    vuChoiceRecMenuClass	*running10;

    filteredTextitem		*stoppedTimeout;
    filteredTextitem		*issuingTimeout;
    filteredTextitem		*receivingTimeout;
    filteredTextitem		*runningTimeout;

    vuTankValveDismissButtonClass	*dismissButton;
};


//*** TANK VALVE DISMISS BUTTON CLASS ********************************

class	vuTankValveDismissButtonClass : public vbutton
{
    vuTankValveDialogClass	*valveDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankValveDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankValveDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle valveDialog
    void storeTankValveDialog (vuTankValveDialogClass *cd)
	{ valveDialog = cd; }
};


//*** TANK TEMP DIALOG CLASS ************************************

class	vuTankTempDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuTankTempDialogClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuTankTempDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // public so no funcs needed to access
    vuPvNameButtonClass		*probe01Button;
    vuFilteredTextitem		*probe01;
    filteredTextitem		*level01;
    vuCheckBoxClass		*proc01;
    vuCheckBoxClass		*ms01;

    vuPvNameButtonClass		*probe02Button;
    vuFilteredTextitem		*probe02;
    filteredTextitem		*level02;
    vuCheckBoxClass		*proc02;
    vuCheckBoxClass		*ms02;

    vuPvNameButtonClass		*probe03Button;
    vuFilteredTextitem		*probe03;
    filteredTextitem		*level03;
    vuCheckBoxClass		*proc03;
    vuCheckBoxClass		*ms03;

    vuPvNameButtonClass		*probe04Button;
    vuFilteredTextitem		*probe04;
    filteredTextitem		*level04;
    vuCheckBoxClass		*proc04;
    vuCheckBoxClass		*ms04;

    vuPvNameButtonClass		*probe05Button;
    vuFilteredTextitem		*probe05;
    filteredTextitem		*level05;
    vuCheckBoxClass		*proc05;
    vuCheckBoxClass		*ms05;

    vuPvNameButtonClass		*probe06Button;
    vuFilteredTextitem		*probe06;
    filteredTextitem		*level06;
    vuCheckBoxClass		*proc06;
    vuCheckBoxClass		*ms06;

    vuPvNameButtonClass		*probe07Button;
    vuFilteredTextitem		*probe07;
    filteredTextitem		*level07;
    vuCheckBoxClass		*proc07;
    vuCheckBoxClass		*ms07;

    vuPvNameButtonClass		*probe08Button;
    vuFilteredTextitem		*probe08;
    filteredTextitem		*level08;
    vuCheckBoxClass		*proc08;
    vuCheckBoxClass		*ms08;

    vuPvNameButtonClass		*probe09Button;
    vuFilteredTextitem		*probe09;
    filteredTextitem		*level09;
    vuCheckBoxClass		*proc09;
    vuCheckBoxClass		*ms09;

    vuPvNameButtonClass		*probe10Button;
    vuFilteredTextitem		*probe10;
    filteredTextitem		*level10;
    vuCheckBoxClass		*proc10;
    vuCheckBoxClass		*ms10;

    vuTankTempDismissButtonClass	*dismissButton;
};


//*** TANK TEMP DISMISS BUTTON CLASS ********************************

class	vuTankTempDismissButtonClass : public vbutton
{
    vuTankTempDialogClass	*tempDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankTempDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankTempDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle tempDialog
    void storeTankTempDialog (vuTankTempDialogClass *cd)
	{ tempDialog = cd; }
};


//*** TANK PV DIALOG CLASS ************************************

class	vuTankPvDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuTankPvDialogClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuTankPvDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // public so no funcs needed to access
    vuPvNameButtonClass		*levelPvButton;
    vuFilteredTextitem		*levelPv;
    vuCheckBoxClass		*procLevelPv;
    vuCheckBoxClass		*msLevelPv;

    vuPvNameButtonClass		*densityPvButton;
    vuFilteredTextitem		*densityPv;
    vuCheckBoxClass		*procDensityPv;
    vuCheckBoxClass		*msDensityPv;

    vuPvNameButtonClass		*massPvButton;
    vuFilteredTextitem		*massPv;
    vuCheckBoxClass		*procMassPv;
    vuCheckBoxClass		*msMassPv;

    vuPvNameButtonClass		*avgTempPvButton;
    vuFilteredTextitem		*avgTempPv;
    vuCheckBoxClass		*procAvgTempPv;
    vuCheckBoxClass		*msAvgTempPv;

    vuPvNameButtonClass		*volumePvButton;
    vuFilteredTextitem		*volumePv;
    vuCheckBoxClass		*procVolumePv;
    vuCheckBoxClass		*msVolumePv;

    vuPvNameButtonClass		*highLevelPvButton;
    vuFilteredTextitem		*highLevelPv;
    vuCheckBoxClass		*procHighLevelPv;
    vuCheckBoxClass		*msHighLevelPv;

    vuPvNameButtonClass		*lowLevelPvButton;
    vuFilteredTextitem		*lowLevelPv;
    vuCheckBoxClass		*procLowLevelPv;
    vuCheckBoxClass		*msLowLevelPv;

    vuPvNameButtonClass		*highPressPvButton;
    vuFilteredTextitem		*highPressPv;
    vuCheckBoxClass		*procHighPressPv;
    vuCheckBoxClass		*msHighPressPv;

    vuPvNameButtonClass		*lowPressPvButton;
    vuFilteredTextitem		*lowPressPv;
    vuCheckBoxClass		*procLowPressPv;
    vuCheckBoxClass		*msLowPressPv;

    vuPvNameButtonClass		*bswPvButton;
    vuFilteredTextitem		*bswPv;
    vuCheckBoxClass		*procBswPv;
    vuCheckBoxClass		*msBswPv;

    vuChoiceRecMenuClass	*levelUnits;
    vuChoiceRecMenuClass	*densityUnits;
    vuChoiceRecMenuClass	*massUnits;
    vuChoiceGblMenuClass	*tempUnits;
    vuChoiceGblMenuClass	*volumeUnits;
    vuChoiceGblMenuClass	*flowRateUnits;

    vuTankPvDismissButtonClass	*dismissButton;
};


//*** TANK PV DISMISS BUTTON CLASS ********************************

class	vuTankPvDismissButtonClass : public vbutton
{
    vuTankPvDialogClass	*pvDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuTankPvDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuTankPvDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle pvDialog
    void storeTankPvDialog (vuTankPvDialogClass *cd)
	{ pvDialog = cd; }
};

#endif	vutankdialogclassHXX

