#ifndef vuudbdialogclassHXX
#define	vuudbdialogclassHXX

// define values for the xrefNode inDialog flag

enum {
    VU_UDB_NOT_IN_DIALOG = 1,
    VU_UDB_IN_MAIN_DIALOG,
    VU_UDB_IN_STATUS_DIALOG,
    VU_UDB_IN_IO_DIALOG,
    VU_UDB_IN_CONSTANTS_DIALOG,
    VU_UDB_IN_LIMITS_DIALOG,
    VU_UDB_IN_MISC_DIALOG
};

// define a structure for the labels
typedef struct udbLabels
{
    vdialogItem	*dialogItem;
    char	widgetId[UDBLABEL_SZ + 1];
    vchar	defaultLabel[UDBLABEL_SZ + 1];
    vchar	newLabel[UDBLABEL_SZ + 1];
} VU_UDBLABEL, *VU_UDBLABEL_PTR;

class	vuUdbDialogClass;
class	vuUdbProgramNameButtonClass;
class	vuUdbStatusButtonClass;
class	vuUdbIOButtonClass;
class	vuUdbConstantsButtonClass;
class	vuUdbLimitsButtonClass;
class	vuUdbMiscButtonClass;

// some classes for the UDB dialogs
class	vuUdbStatusDialogClass;
class	vuUdbStatusDismissButtonClass;
class	vuUdbIODialogClass;
class	vuUdbIODismissButtonClass;
class	vuUdbConstantsDialogClass;
class	vuUdbConstantsDismissButtonClass;
class	vuUdbLimitsDialogClass;
class	vuUdbLimitsDismissButtonClass;
class	vuUdbMiscDialogClass;
class	vuUdbMiscDismissButtonClass;


//*** UDB CONTAINER CLASS ***********************************************

class	vuUdbContainerClass : public vcontainer
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbContainerClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuUdbContainerClass, vcontainer);

    // vloadable override
    void LoadInit (vresource resource);

    // override 
    void Destroy ();

    // pointers to fields in container, public so don't need funcs to
    // get at them from vuUdbDialogClass

    vuUdbProgramNameButtonClass	*udbProgramNameButton;
    vuFilteredTextitem		*udbProgramName;
    vuCheckBoxClass		*onScan;
    vuCheckBoxClass		*onAlarm;
    vuUdbStatusButtonClass	*statusButton;
    vuUdbIOButtonClass		*ioButton;
    vuUdbConstantsButtonClass	*constantsButton;
    vuUdbLimitsButtonClass	*limitsButton;
    vuUdbMiscButtonClass	*miscButton;

};


//*** UDB PROGRAM NAME BUTTON CLASS ******************************

class	vuUdbProgramNameButtonClass : public vbutton
{

    vuUdbDialogClass	*udbDialog;
    vuFilteredTextitem	*programNameText;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbProgramNameButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbProgramNameButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle commonDialog
    void storeUdbDialog (vuUdbDialogClass *cd)
	{ udbDialog = cd; }
    vuUdbDialogClass *getUdbDialog (void)
	{ return (udbDialog); }

    // function to handle programNameText
    void storeProgramName (vuFilteredTextitem *pn)
	{ programNameText = pn; }
    vuFilteredTextitem *getProgramName (void)
	{ return (programNameText); }
};


//*** UDB DIALOG CLASS **************************************************

class	vuUdbDialogClass : public vuCommonDialogClass
{
    vuUdbContainerClass		*udbContainer;
    vstr			*udbTitle;
    vuUdbStatusDialogClass	*statusDialog;
    vuUdbIODialogClass		*ioDialog;
    vuUdbConstantsDialogClass	*constantsDialog;
    vuUdbLimitsDialogClass	*limitsDialog;
    vuUdbMiscDialogClass	*miscDialog;
    vlist			*dialogLabels;
    int				udbLabelsModified;
    void			vuUdbAddLabel (vdialogItem *dialogItem, char *widgetId);

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbDialogClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(vuUdbDialogClass, vuCommonDialogClass);

    // vloadable override
    void LoadInit (vresource resource);

    // override
    void Destroy ();

    // overide base vuCommonDialogClass
    void resetDialog (void);

    // override base class load and save functions
    void vuLoadDialog (vchar *pointName);
    void vuSaveDialog (vchar *pointName);

    // function to handle dialogLists
    vlist *vuUdbGetDialogLabelsList (void) { return (dialogLabels); }

    // functions to handle udbLabelsModified flag
    void vuUdbSetLabelsFlag (int value) { udbLabelsModified = value; }
    int vuUdbGetLabelsFlag (void) { return (udbLabelsModified); }

    // function to modify dialog labels
    void vuUdbModLabels (void);
};


//*** UDB STATUS BUTTON CLASS **********************************************

class	vuUdbStatusButtonClass : public vbutton
{

    vuUdbStatusDialogClass	*statusDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbStatusButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbStatusButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // handle the pointer to the status dialog
    void storeStatusDialog (vuUdbStatusDialogClass *cd) 
	{ statusDialogClass = cd; }
};


//*** UDB IO BUTTON CLASS **********************************************

class	vuUdbIOButtonClass : public vbutton
{

    vuUdbDialogClass	*udbDialogClass;
    vuUdbIODialogClass	*ioDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbIOButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbIOButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // handle the pointer to the udb dialog
    void storeUdbDialog (vuUdbDialogClass *cd) 
	{ udbDialogClass = cd; }

    // handle the pointer to the io dialog
    void storeIODialog (vuUdbIODialogClass *cd) 
	{ ioDialogClass = cd; }
};


//*** UDB CONSTANTS BUTTON CLASS **********************************************

class	vuUdbConstantsButtonClass : public vbutton
{

    vuUdbDialogClass		*udbDialogClass;
    vuUdbConstantsDialogClass	*constantsDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbConstantsButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbConstantsButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // handle the pointer to the udb dialog
    void storeUdbDialog (vuUdbDialogClass *cd) 
	{ udbDialogClass = cd; }

    // handle the pointer to the status dialog
    void storeConstantsDialog (vuUdbConstantsDialogClass *cd) 
	{ constantsDialogClass = cd; }
};


//*** UDB LIMITS BUTTON CLASS **********************************************

class	vuUdbLimitsButtonClass : public vbutton
{

    vuUdbDialogClass		*udbDialogClass;
    vuUdbLimitsDialogClass	*limitsDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbLimitsButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbLimitsButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // handle the pointer to the udb dialog
    void storeUdbDialog (vuUdbDialogClass *cd) 
	{ udbDialogClass = cd; }

    // handle the pointer to the limits dialog
    void storeLimitsDialog (vuUdbLimitsDialogClass *cd) 
	{ limitsDialogClass = cd; }
};


//*** UDB MISCELLANEOUS BUTTON CLASS ***************************************

class	vuUdbMiscButtonClass : public vbutton
{

    vuUdbMiscDialogClass	*miscDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbMiscButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbMiscButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // handle the pointer to the misc dialog
    void storeMiscDialog (vuUdbMiscDialogClass *cd) 
	{ miscDialogClass = cd; }
};


//*** UDB STATUS DIALOG CLASS **********************************************

class	vuUdbStatusDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbStatusDialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbStatusDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // define fields public so dont need func to access
    filteredTextitem	*status1;
    filteredTextitem	*status2;
    filteredTextitem	*status3;
    filteredTextitem	*status4;
    filteredTextitem	*status5;
    filteredTextitem	*status6;
    filteredTextitem	*status7;
    filteredTextitem	*status8;
    filteredTextitem	*status9;
    filteredTextitem	*status10;
    filteredTextitem	*status11;
    filteredTextitem	*status12;
    vuChoiceGblMenuClass	*status1Severity;
    vuChoiceGblMenuClass	*status2Severity;
    vuChoiceGblMenuClass	*status3Severity;
    vuChoiceGblMenuClass	*status4Severity;
    vuChoiceGblMenuClass	*status5Severity;
    vuChoiceGblMenuClass	*status6Severity;
    vuChoiceGblMenuClass	*status7Severity;
    vuChoiceGblMenuClass	*status8Severity;
    vuChoiceGblMenuClass	*status9Severity;
    vuChoiceGblMenuClass	*status10Severity;
    vuChoiceGblMenuClass	*status11Severity;
    vuChoiceGblMenuClass	*status12Severity;
    filteredTextitem	*cmd1;
    filteredTextitem	*cmd2;
    filteredTextitem	*cmd3;
    filteredTextitem	*cmd4;
    filteredTextitem	*cmd5;
    filteredTextitem	*cmd6;
    filteredTextitem	*cmd7;
    filteredTextitem	*cmd8;
    vuChoiceGblMenuClass	*cosvSeverity;
    vuChoiceGblMenuClass	*undefinedSeverity;
    vuFilteredTextitem	*alarmGroup;
    filteredTextitem	*alarmDelay;
    vuPvNameButtonClass	*alarmDisableButton;
    vuFilteredTextitem	*alarmDisable;
    vuUdbStatusDismissButtonClass	*dismissButton;
};


//*** UDB STATUS DIALOG DISMISS BUTTON CLASS *******************************

class vuUdbStatusDismissButtonClass : public vbutton
{

    vuUdbStatusDialogClass	*statusDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbStatusDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbStatusDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle statusDialogClass
    void storeStatusDialog (vuUdbStatusDialogClass *sd)
	{ statusDialogClass = sd; }
};


//*** UDB IO DIALOG CLASS **************************************************

class	vuUdbIODialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbIODialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbIODialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // ptrs to fields are public so dont need func to get at them
    vuPvNameButtonClass	*input1Button;
    vuFilteredTextitem	*input1;
    vuCheckBoxClass	*prc1;
    vuCheckBoxClass	*mxs1;
    vuPvNameButtonClass	*input2Button;
    vuFilteredTextitem	*input2;
    vuCheckBoxClass	*prc2;
    vuCheckBoxClass	*mxs2;
    vuPvNameButtonClass	*input3Button;
    vuFilteredTextitem	*input3;
    vuCheckBoxClass	*prc3;
    vuCheckBoxClass	*mxs3;
    vuPvNameButtonClass	*input4Button;
    vuFilteredTextitem	*input4;
    vuCheckBoxClass	*prc4;
    vuCheckBoxClass	*mxs4;
    vuPvNameButtonClass	*input5Button;
    vuFilteredTextitem	*input5;
    vuCheckBoxClass	*prc5;
    vuCheckBoxClass	*mxs5;
    vuPvNameButtonClass	*input6Button;
    vuFilteredTextitem	*input6;
    vuCheckBoxClass	*prc6;
    vuCheckBoxClass	*mxs6;
    vuPvNameButtonClass	*input7Button;
    vuFilteredTextitem	*input7;
    vuCheckBoxClass	*prc7;
    vuCheckBoxClass	*mxs7;
    vuPvNameButtonClass	*input8Button;
    vuFilteredTextitem	*input8;
    vuCheckBoxClass	*prc8;
    vuCheckBoxClass	*mxs8;
    vuPvNameButtonClass	*output1Button;
    vuFilteredTextitem	*output1;
    vuCheckBoxClass	*pin1;
    vuPvNameButtonClass	*output2Button;
    vuFilteredTextitem	*output2;
    vuCheckBoxClass	*pin2;
    vuPvNameButtonClass	*output3Button;
    vuFilteredTextitem	*output3;
    vuCheckBoxClass	*pin3;
    vuPvNameButtonClass	*output4Button;
    vuFilteredTextitem	*output4;
    vuCheckBoxClass	*pin4;
    vuPvNameButtonClass	*output5Button;
    vuFilteredTextitem	*output5;
    vuCheckBoxClass	*pin5;
    vuPvNameButtonClass	*output6Button;
    vuFilteredTextitem	*output6;
    vuCheckBoxClass	*pin6;
    vuPvNameButtonClass	*output7Button;
    vuFilteredTextitem	*output7;
    vuCheckBoxClass	*pin7;
    vuPvNameButtonClass	*output8Button;
    vuFilteredTextitem	*output8;
    vuCheckBoxClass	*pin8;
    vuUdbIODismissButtonClass	*dismissButton;
};


//*** UDB IO DIALOG CLASS DISMISS BUTTON ***********************************

class	vuUdbIODismissButtonClass : public vbutton
{

    vuUdbIODialogClass	*ioDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbIODismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbIODismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle ioDialogClass
    void storeIODialog (vuUdbIODialogClass *id)
	{ ioDialogClass = id; }
};


//*** UDB CONSTANTS DIALOG CLASS *******************************************

class	vuUdbConstantsDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbConstantsDialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbConstantsDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // fields are public so dont need funcs to get at them
    filteredTextitem		*float1;
    vdialogLabelItem		*float1Label;
    vuChoiceRecMenuClass	*float1Limit;
    filteredTextitem		*float2;
    vdialogLabelItem		*float2Label;
    vuChoiceRecMenuClass	*float2Limit;
    filteredTextitem		*float3;
    vdialogLabelItem		*float3Label;
    vuChoiceRecMenuClass	*float3Limit;
    filteredTextitem		*float4;
    vdialogLabelItem		*float4Label;
    vuChoiceRecMenuClass	*float4Limit;
    filteredTextitem		*float5;
    vdialogLabelItem		*float5Label;
    vuChoiceRecMenuClass	*float5Limit;
    filteredTextitem		*float6;
    vdialogLabelItem		*float6Label;
    vuChoiceRecMenuClass	*float6Limit;
    filteredTextitem		*float7;
    vdialogLabelItem		*float7Label;
    vuChoiceRecMenuClass	*float7Limit;
    filteredTextitem		*float8;
    vdialogLabelItem		*float8Label;
    vuChoiceRecMenuClass	*float8Limit;
    filteredTextitem		*float9;
    vdialogLabelItem		*float9Label;
    vuChoiceRecMenuClass	*float9Limit;
    filteredTextitem		*float10;
    vdialogLabelItem		*float10Label;
    vuChoiceRecMenuClass	*float10Limit;
    filteredTextitem		*float11;
    vdialogLabelItem		*float11Label;
    vuChoiceRecMenuClass	*float11Limit;
    filteredTextitem		*float12;
    vdialogLabelItem		*float12Label;
    vuChoiceRecMenuClass	*float12Limit;
    filteredTextitem		*double1;
    vdialogLabelItem		*double1Label;
    vuChoiceRecMenuClass	*double1Limit;
    filteredTextitem		*double2;
    vdialogLabelItem		*double2Label;
    vuChoiceRecMenuClass	*double2Limit;
    filteredTextitem		*double3;
    vdialogLabelItem		*double3Label;
    vuChoiceRecMenuClass	*double3Limit;
    filteredTextitem		*double4;
    vdialogLabelItem		*double4Label;
    vuChoiceRecMenuClass	*double4Limit;
    filteredTextitem		*double5;
    vdialogLabelItem		*double5Label;
    vuChoiceRecMenuClass	*double5Limit;
    filteredTextitem		*double6;
    vdialogLabelItem		*double6Label;
    vuChoiceRecMenuClass	*double6Limit;
    filteredTextitem		*ulong1;
    vdialogLabelItem		*ulong1Label;
    vuChoiceRecMenuClass	*ulong1Limit;
    filteredTextitem		*ulong2;
    vdialogLabelItem		*ulong2Label;
    vuChoiceRecMenuClass	*ulong2Limit;
    vuChoiceGblMenuClass	*chkbx1;
    vdialogLabelItem		*chkbx1Label;
    vuChoiceGblMenuClass	*chkbx2;
    vdialogLabelItem		*chkbx2Label;
    vuChoiceGblMenuClass	*chkbx3;
    vdialogLabelItem		*chkbx3Label;
    vuChoiceGblMenuClass	*chkbx4;
    vdialogLabelItem		*chkbx4Label;
    vuChoiceGblMenuClass	*chkbx5;
    vdialogLabelItem		*chkbx5Label;
    vuChoiceGblMenuClass	*chkbx6;
    vdialogLabelItem		*chkbx6Label;
    vuChoiceGblMenuClass	*chkbx7;
    vdialogLabelItem		*chkbx7Label;
    vuChoiceGblMenuClass	*chkbx8;
    vdialogLabelItem		*chkbx8Label;
    vuUdbConstantsDismissButtonClass	*dismissButton;
};


//*** UDB CONSTANTS DIALOG DISMISS BUTTON **********************************

class	vuUdbConstantsDismissButtonClass : public vbutton
{

    vuUdbConstantsDialogClass	*constantsDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbConstantsDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbConstantsDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle constantsDialogClass
    void storeConstantsDialog (vuUdbConstantsDialogClass *sd)
	{ constantsDialogClass = sd; }
};


//*** UDB LIMITS DIALOG CLASS **********************************************

class	vuUdbLimitsDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbLimitsDialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbLimitsDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // are public so dont have to use funcs to get at

    vuChoiceRecMenuClass	*tempFloat1Limit;
    vdialogLabelItem		*tempFloat1Label;
    vuChoiceRecMenuClass	*tempFloat2Limit;
    vdialogLabelItem		*tempFloat2Label;
    vuChoiceRecMenuClass	*tempFloat3Limit;
    vdialogLabelItem		*tempFloat3Label;
    vuChoiceRecMenuClass	*tempFloat4Limit;
    vdialogLabelItem		*tempFloat4Label;
    vuChoiceRecMenuClass	*tempFloat5Limit;
    vdialogLabelItem		*tempFloat5Label;
    vuChoiceRecMenuClass	*tempFloat6Limit;
    vdialogLabelItem		*tempFloat6Label;
    vuChoiceRecMenuClass	*tempFloat7Limit;
    vdialogLabelItem		*tempFloat7Label;
    vuChoiceRecMenuClass	*tempFloat8Limit;
    vdialogLabelItem		*tempFloat8Label;
    vuChoiceRecMenuClass	*tempFloat9Limit;
    vdialogLabelItem		*tempFloat9Label;
    vuChoiceRecMenuClass	*tempFloat10Limit;
    vdialogLabelItem		*tempFloat10Label;
    vuChoiceRecMenuClass	*tempFloat11Limit;
    vdialogLabelItem		*tempFloat11Label;
    vuChoiceRecMenuClass	*tempFloat12Limit;
    vdialogLabelItem		*tempFloat12Label;
    vuChoiceRecMenuClass	*tempFloat13Limit;
    vdialogLabelItem		*tempFloat13Label;
    vuChoiceRecMenuClass	*tempFloat14Limit;
    vdialogLabelItem		*tempFloat14Label;
    vuChoiceRecMenuClass	*tempFloat15Limit;
    vdialogLabelItem		*tempFloat15Label;
    vuChoiceRecMenuClass	*tempFloat16Limit;
    vdialogLabelItem		*tempFloat16Label;
    vuChoiceRecMenuClass	*tempDouble1Limit;
    vdialogLabelItem		*tempDouble1Label;
    vuChoiceRecMenuClass	*tempDouble2Limit;
    vdialogLabelItem		*tempDouble2Label;
    vuChoiceRecMenuClass	*tempDouble3Limit;
    vdialogLabelItem		*tempDouble3Label;
    vuChoiceRecMenuClass	*tempDouble4Limit;
    vdialogLabelItem		*tempDouble4Label;
    vuChoiceRecMenuClass	*tempUlong1Limit;
    vdialogLabelItem		*tempUlong1Label;
    vuChoiceRecMenuClass	*tempUlong2Limit;
    vdialogLabelItem		*tempUlong2Label;
    vuChoiceRecMenuClass	*tempUlong3Limit;
    vdialogLabelItem		*tempUlong3Label;
    vuChoiceRecMenuClass	*tempUlong4Limit;
    vdialogLabelItem		*tempUlong4Label;
    vuChoiceRecMenuClass	*tempUlong5Limit;
    vdialogLabelItem		*tempUlong5Label;
    vuChoiceRecMenuClass	*tempUlong6Limit;
    vdialogLabelItem		*tempUlong6Label;
    vuChoiceRecMenuClass	*tempUlong7Limit;
    vdialogLabelItem		*tempUlong7Label;
    vuChoiceRecMenuClass	*tempUlong8Limit;
    vdialogLabelItem		*tempUlong8Label;
    vuUdbLimitsDismissButtonClass	*dismissButton;
};


//*** UDB LIMITS DIALOG DISMISS BUTTON *************************************

class	vuUdbLimitsDismissButtonClass : public vbutton
{

    vuUdbLimitsDialogClass	*limitsDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbLimitsDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbLimitsDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle limitsDialogClass
    void storeLimitsDialog (vuUdbLimitsDialogClass *sd)
	{ limitsDialogClass = sd; }
};


//*** UDB MISCELLANEOUS DIALOG CLASS ***************************************

class	vuUdbMiscDialogClass : public vdialog
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbMiscDialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbMiscDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // public so dont need funcs to get at

    vuChoiceRecMenuClass	*initialMode;
    filteredTextitem		*mode01;
    filteredTextitem		*mode02;
    filteredTextitem		*mode03;
    filteredTextitem		*mode04;
    filteredTextitem		*mode05;
    filteredTextitem		*mode06;
    filteredTextitem		*mode07;
    filteredTextitem		*mode08;
    vuChoiceRecMenuClass	*initialFunction;
    filteredTextitem		*func01;
    filteredTextitem		*func02;
    filteredTextitem		*func03;
    filteredTextitem		*func04;
    filteredTextitem		*limit1Low;
    filteredTextitem		*limit2Low;
    filteredTextitem		*limit3Low;
    filteredTextitem		*limit1High;
    filteredTextitem		*limit2High;
    filteredTextitem		*limit3High;
    filteredTextitem		*limit1Precision;
    filteredTextitem		*limit2Precision;
    filteredTextitem		*limit3Precision;
    vuUdbMiscDismissButtonClass	*dismissButton;
};


//*** UDB MISCELLANEOUS DIALOG DISMISS BUTTON ******************************

class	vuUdbMiscDismissButtonClass : public vbutton
{

    vuUdbMiscDialogClass	*miscDialogClass;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbMiscDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbMiscDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle miscDialogClass
    void storeMiscDialog (vuUdbMiscDialogClass *sd)
	{ miscDialogClass = sd; }
};


//*** UDB PROGRAM NAMES DIALOG CLASS **************************************
class	vuUdbProgramNamesListClass;
class	vuUdbProgramNamesOKButtonClass;
class	vuUdbProgramNamesDismissButtonClass;

class	vuUdbProgramNamesDialogClass : public vdialog
{
    vchar	udbProgramName [UDBPROGRAMNAME_SZ + 1];

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbProgramNamesDialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbProgramNamesDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // override
    void Destroy ();

    // vwindow overide
    void ObserveWindow (vevent *event);

    // public so dont need funcs to access
    vuUdbProgramNamesListClass		*namesList;
    vuUdbProgramNamesOKButtonClass	*okButton;
    vuUdbProgramNamesDismissButtonClass	*dismissButton;

    // open function
    void vuUdbNamesDialogOpen (dataVuApplication *vuApp);

    // function to handle udbProgramName
    vchar *getUdbProgramName (void) { return (udbProgramName); }
};


//*** UDB PROGRAM NAMES LIST CLASS ***********************************

class	vuUdbProgramNamesListClass : public vlistitem
{

    vuUdbProgramNamesDialogClass	*namesDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbProgramNamesListClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbProgramNamesListClass, vlistitem);

    // virtual member function override
    void ObserveSelect (int state);

    // functions to handle namesDialog
    void storeNamesDialog (vuUdbProgramNamesDialogClass *cd)
	{ namesDialog = cd; }
};


//*** UDB PROGRAM NAMES OK BUTTON CLASS ***********************************

class	vuUdbProgramNamesOKButtonClass : public vbutton
{

    vuUdbProgramNamesDialogClass	*namesDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbProgramNamesOKButtonClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbProgramNamesOKButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle namesDialog
    void storeNamesDialog (vuUdbProgramNamesDialogClass *cd)
	{ namesDialog = cd; }
};

//*** UDB PROGRAM NAMES DISMISS BUTTON CLASS ******************************

class	vuUdbProgramNamesDismissButtonClass : public vbutton
{

    vuUdbProgramNamesDialogClass	*namesDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbProgramNamesDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbProgramNamesDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // functions to handle namesDialog
    void storeNamesDialog (vuUdbProgramNamesDialogClass *cd)
	{ namesDialog = cd; }
};


//*** UDB CONFIGURATION DIALOG CLASS **************************************

class	vuUdbCfgPgNamesListClass;
class	vuUdbCfgLabelsListClass;
class	vuUdbCfgApplyButtonClass;
class	vuUdbCfgDismissButtonClass;
class	vuUdbCfgDeleteButtonClass;
class	vuUdbCfgDefaultButtonClass;
class	vuUdbCfgReplaceButtonClass;

class	vuUdbConfigDialogClass : public vdialog
{

    vuDatabaseClass	*database;
    vuUdbDialogClass	*udbDialog;
    int			cfgModified;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbConfigDialogClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbConfigDialogClass, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // vwindow overide
    void ObserveWindow (vevent *event);

    // destroy override
    void Destroy (void);

    // open function
    void vuUdbConfigDialogOpen (dataVuApplication *vuApp, 
				vuFilteredTextitem *programName,
				vuUdbDialogClass *udbDialogClass);

    // public so dont need funcs to access
    vuUdbCfgPgNamesListClass	*pgnamesList;
    vuUdbCfgLabelsListClass	*labelsList;
    filteredTextitem		*programName;
    vtextitem			*defaultLabel;
    filteredTextitem		*newLabel;
    vuUdbCfgApplyButtonClass	*applyButton;
    vuUdbCfgDismissButtonClass	*dismissButton;
    vuUdbCfgDeleteButtonClass	*deleteButton;
    vuUdbCfgDefaultButtonClass	*defaultButton;
    vuUdbCfgReplaceButtonClass	*replaceButton;

    // functions to handle database
    vuDatabaseClass *getDatabase (void) { return (database); }

    // function to handle udb dialog
    vuUdbDialogClass *getUdbDialog (void) { return (udbDialog); }

    // functions to handle cfgChanged flag
    void vuUdbSetCfgModFlag (int val) { cfgModified = val; }
    int  vuUdbGetCfgModFlag (void) { return (cfgModified); }

    // function to reload udb's dialogList newLabel with defaults
    void vuUdbLoadDefaultConfig (void);

    // function to load udb's dialogList newLabel with values
    // based on the current selected udbprogram name
    void vuUdbLoadCurrentConfig (void);
};


//*** UDB CONFIG PROGRAM NAMES LIST CLASS ***********************************

class	vuUdbCfgPgNamesListClass : public vlistitem
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbCfgPgNamesListClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbCfgPgNamesListClass, vlistitem);

    // virtual member function override
    void ObserveSelect (int state);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }

    // function to build the names list
    void buildCfgNamesList (void);

};


//*** UDB CONFIG LABELS LIST CLASS ******************************************

class	vuUdbCfgLabelsListClass : public vlistitem
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(vuUdbCfgLabelsListClass);

    // macros for standard versions of the constructors and destructors
    vloadableINLINE_CONSTRUCTORS(vuUdbCfgLabelsListClass, vlistitem);

    // virtual member function override
    void ObserveSelect (int state);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }

    // function to build the labels list
    void buildCfgLabelsList (int useCurrentConfig);
};


//*** UDB CONFIG APPLY BUTTON CLASS ***************************************

class	vuUdbCfgApplyButtonClass : public vbutton
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbCfgApplyButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbCfgApplyButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }
};


//*** UDB CONFIG DISMISS BUTTON CLASS *************************************

class	vuUdbCfgDismissButtonClass : public vbutton
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbCfgDismissButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbCfgDismissButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }
};


//*** UDB CONFIG DELETE BUTTON CLASS *************************************

class	vuUdbCfgDeleteButtonClass : public vbutton
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbCfgDeleteButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbCfgDeleteButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }
};


//*** UDB CONFIG DEFAULT BUTTON CLASS *************************************

class	vuUdbCfgDefaultButtonClass : public vbutton
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbCfgDefaultButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbCfgDefaultButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }
};


//*** UDB CONFIG REPLACE BUTTON CLASS *************************************

class	vuUdbCfgReplaceButtonClass : public vbutton
{

    vuUdbConfigDialogClass	*configDialog;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION (vuUdbCfgReplaceButtonClass);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS (vuUdbCfgReplaceButtonClass, vbutton);

    // virtual member function override
    void ObserveDialogItem (vevent *event);

    // function to handle configDialogClass
    void storeConfigDialog (vuUdbConfigDialogClass *cd)
	{ configDialog = cd; }
};

#endif	vuudbdialogclassHXX

