#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vtextHEADER
#include vtextviewHEADER
#include vtextitemHEADER
#include vstartupHEADER
#include vcontainerHEADER
#include vcontrolHEADER
#include vbuttonHEADER
#include vlistviewHEADER
#include vliststrHEADER
#include vlistitemHEADER
#include vlimitsHEADER
#include vtimeHEADER
#include vstdargHEADER
#include vdateHEADER
#include vspinnerHEADER
#include vregexpHEADER
#include vfsHEADER
#include vfilechsrHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "cadef.h"
#include "choice.h"
#include "choiceGbl.h"
#include "choiceAo.h"
#include "dbRecType.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
#include "dbDefs.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuCommonDialogClass.hxx"

#include "vuAccDialogClass.hxx"
#include "vuAiDialogClass.hxx"
#include "vuAoDialogClass.hxx"
#include "vuBiDialogClass.hxx"
#include "vuBoDialogClass.hxx"
#include "vuCalcDialogClass.hxx"
#include "vuDatamoveDialogClass.hxx"
#include "vuDdcDialogClass.hxx"
#include "vuDeadtimeDialogClass.hxx"
#include "vuFanoutDialogClass.hxx"
#include "vuLdfwDialogClass.hxx"
#include "vuLeadLagDialogClass.hxx"
#include "vuMbbiDialogClass.hxx"
#include "vuMbboDialogClass.hxx"
#include "vuPatDialogClass.hxx"
#include "vuPidDialogClass.hxx"
#include "vuPipeDialogClass.hxx"
#include "vuPotDialogClass.hxx"
#include "vuPtoDialogClass.hxx"
#include "vuPumpDialogClass.hxx"
#include "vuRampDialogClass.hxx"
#include "vuSelDialogClass.hxx"
#include "vuSigDialogClass.hxx"
#include "vuStateDialogClass.hxx"
#include "vuStringinDialogClass.hxx"
#include "vuStringoutDialogClass.hxx"
#include "vuTankDialogClass.hxx"
#include "vuTdDialogClass.hxx"
#include "vuUdbDialogClass.hxx"
#include "vuValveDialogClass.hxx"
#include "vuWaveformDialogClass.hxx"

#include "vuNavigatorDialogClass.hxx"
#include "vuAlmGrpSelDialogClass.hxx"
#include "vugrplistdialogclass.hxx"
#include "vuUtils.hxx"
#include "vunetwork.hxx"

enum
{
    VU_NO_MSG = 0,
    VU_INFO_MSG,
    VU_CAUTION_MSG
};

static vdialogItem *traverseContainer (vcontainer *container, vchar *tagName);


/*******************************************************************
*
*    vuAlarmGroupFilter ()
*	special filter function for the alarm group field
*	if a space is typed as the first character, the
*	Alarm Group Selector Dialog is brought up so the user can 
*	make a selection
*
*******************************************************************/

int vuAlarmGroupFilter (filteredText *text, vtextSelection *sel,
			const vchar **string, long *len)
{
    int		ret;
    vchar	aSpace;

    if (*len > 0)
    {
	aSpace = vcharFromLiteral (' ');
	if (**string == aSpace)
	{
	    vuFilteredTextitem *textItem = (vuFilteredTextitem *)text -> GetUserPtr ();
	    if (textItem != NULL)
	    {
		int	ok = vFALSE;
		vuCommonDialogClass *commonDialog = textItem -> getCommonDialog ();
		if (commonDialog != NULL)
		{
		    vuMainDialogClass *mainDialog = commonDialog -> getMainDialog ();
		    dataVuApplication *vuApp = mainDialog -> getApplication ();
		    vresource vuRes = vuApp -> getDataVuvresource ();
		    vresource almGrpRes = vresourceGet (vuRes, 
						vunames_vuAlarmGrpSelDialog);
		    vuAlmGrpSelDialogClass *almGrpDialog = 
						new vuAlmGrpSelDialogClass (almGrpRes);
		    almGrpDialog -> vuAlmGrpSelOpen (mainDialog);
		    const vchar *almGrp = (const vchar *)(almGrpDialog -> vuGetAlarmGroup ());
		    vchar *newString = ((vchar *)(*string));
		    if (almGrp[0] != 0)
		    {
			textItem ->SetTextScribed (vcharScribeSystem (""));
			*len = vcharLength (almGrp);
			vcharCopyBounded (almGrp, newString, ALARMGRP_SZ);
			ret = NCharFilter (text, sel, string, len, ALARMGRP_SZ - 1);
			ok = vTRUE;
		    }
		    else
		    {
			ok = vFALSE;
		    }
		    delete almGrpDialog;
		}
		if (ok == vFALSE)
		{
		    vwindow::Beep ();
		    ret = vFALSE;
		}
	    }
	    else
	    {
		vwindow::Beep ();
		ret = vFALSE;
	    }
	}
	else
	{
	    ret = NCharFilter (text, sel, string, len, ALARMGRP_SZ - 1);
	}
    }
    else
    {
	vwindow::Beep ();
	ret = vFALSE;
    }

    return (ret);
}


/**********************************************************************
*
*    vuChanConnect
*	channel access connenction handler
*
**********************************************************************/

void	vuChanConnect(struct connection_handler_args conn)
{
    chid	chanID;
    int		stat;

    chanID = conn.chanid;
    int value = (int)(chanID -> puser);

    if((stat = ca_put(ca_field_type(chanID), chanID, (void *)&value)) != ECA_NORMAL)
    {
	vstr *caMsg = vstrCloneScribed (vcharScribeSystem (ca_message ((char)stat)));
	vuMessageDialogCenter (NULL, VU_CA_PUT_ERR, caMsg);
	vstrDestroy (caMsg);
    }
    else
    {
	ca_pend_io ((float)0.0);
    }
    ca_clear_channel(chanID);
}



/**********************************************************************
*
*    vuCaPutValue
*	use channel access to place a field value to an IOC
*
**********************************************************************/

void vuCaPutValue (dataVuApplication *app, char *pvname, char *fldname, int value)
{
    chid	channelID;
    char	fullName[PVNAME_SZ+FLDNAME_SZ+2];
    int		stat;
    vstr	*caMsg;

    sprintf(fullName,"%s.%-*.*s",pvname, FLDNAME_SZ, FLDNAME_SZ, fldname);
    // if channel access has not been init'd yet
    if(app -> getCAInit () == vFALSE)
    {
	if((stat = ca_task_initialize()) != ECA_NORMAL)
	{
	    caMsg = vstrCloneScribed (vcharScribeSystem (ca_message ((char)stat)));
	    vuMessageDialogCenter (NULL, VU_CA_INIT_ERR, caMsg);
	    vstrDestroy (caMsg);
	    return;
	}
	else
	{
	    // set flag that chan acc init'd, see app destructor
	    app -> setCAInit (vTRUE);
	    // start timer to allow chan acc to process events
	    // stays running from now until end of dataVu
	    app -> StartChanAccTimer ();
	}
    }
    if((stat = ca_build_and_connect(fullName, OLD_DBR_ENUM, 0, &channelID, 
				    0, vuChanConnect,(void *)value)) != ECA_NORMAL)
    {
	caMsg = vstrCloneScribed (vcharScribeSystem (ca_message ((char)stat)));
	vuMessageDialogCenter (NULL, VU_CA_CONN_ERR, caMsg);
	vstrDestroy (caMsg);
    }
    else
    {
	ca_pend_io((float)0.0);
	vstr *trendMsg;
	vresource vuRes = app -> getDataVuvresource ();
	vresource msgRes = vresourceGet (vuRes, vunames_vuMessagesDictionary);
	if (value)
	{
	    trendMsg = vresourceGetString (msgRes, vunames_EnabledMsg);
	}
	else
	{
	    trendMsg = vresourceGetString (msgRes, vunames_DisabledMsg);
	}
	vuMainDialogClass *mainDialog = app -> getMainDialogClass ();
	vstr *vstrName = vstrCloneScribed (vcharScribeSystem (pvname));
	mainDialog -> vuMainMessageCenter (vunames_TrendOnLineMsg, vstrName, trendMsg);
	vstrDestroy (trendMsg);
	vstrDestroy (vstrName);
    }
}


/**********************************************************************
*
*    vuDeletePoint
*	this is the generic delete function for calls to the delete
*	menu function and the delete button on the common dialog
*
**********************************************************************/

int vuDeletePoint (vuMainDialogClass *mainDialog, vchar *pvname)
{
    char	pvName[PVNAME_SZ + 1];
    char	seekBuff[PVNAME_SZ + 1];

    dataVuApplication *vuApp = mainDialog -> getApplication ();
    vuDatabaseClass *db = vuApp -> getDatabaseClass ();

    vcharExportBounded (vcharScribeSystemExport (pvname),
			pvName, PVNAME_SZ);
    pvName[PVNAME_SZ] = 0;
    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvName);
    DB_RECORD_PTR rec;
    vstr *fileName;
    
    if ((rec = DB_Seek (db -> vuPvmapTable, seekBuff)) == 0)
    {
	fileName = vstrCloneScribed (vcharScribeSystem (VU_PVMAPFILE));
	vuMessageDialogCenter (NULL, VU_PTNOTINPVMAPERR, pvname);
	vstrDestroy (fileName);
	return (vFALSE);
    }
    if (!dbLockTest (VU_LOCK_TRIES))
    {
	vuMessageDialogCenter (NULL, VU_NO_LOCK_DELETE_MSG, pvname);
	return (vFALSE);
    }

    // get the point type and varify info block from pvmap
    // set the delete flag and the time stamp
    VERIFY_INFO	verifyInfo;
    DB_GetVerifyInfo (db -> vuPvmapTable, seekBuff, &verifyInfo, 0);
    verifyInfo.viPVStatus |= VI_DELETED;
    verifyInfo.viTimeStamp = vuSynchTime ();
    DB_PutVerifyInfo (db -> vuPvmapTable, seekBuff, &verifyInfo, 0);

    int		dataType;
    char	pointType[POINTTYPE_SZ + 1];
    vstr	*ptTypeName = vstrCloneScribed (vcharScribeSystem (pointType));
    DB_GetFieldValue (rec, db -> pvmapTypeField, pointType, &dataType);
    int pointTypeIndex = vuGetPointTypeIndex (pointType); 
    int deleteFromioAddr = vFALSE;
    int result = vTRUE;

    switch (pointTypeIndex)
    {
	case VU_AI:
	{
	    DB_UseKey (db -> vuAiTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuAiTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_AIFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuAiTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_AO:
	{
	    DB_UseKey (db -> vuAoTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuAoTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_AOFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuAoTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_BI:
	{
	    DB_UseKey (db -> vuBiTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuBiTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_BIFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuBiTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_BO:
	{
	    DB_UseKey (db -> vuBoTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuBoTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_BOFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuBoTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_CALC:
	{
	    DB_UseKey (db -> vuCalcTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuCalcTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_CALCFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuCalcTable, rec);
	    }
	    // delete from calcops.dbf
	    DB_UseKey (db -> vuCalcOpsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuCalcOpsTable, seekBuff);
	    break;
	}

	case VU_FANOUT:
	{
	    DB_UseKey (db -> vuFanoutTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuFanoutTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_FANOUTFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuFanoutTable, rec);
	    }
	    // delete from fanlink.dbf
	    DB_UseKey (db -> vuFanLinkTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuFanLinkTable, seekBuff);
	    break;
	}

	case VU_MBBI:
	{
	    DB_UseKey (db -> vuMbbiTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuMbbiTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_MBBIFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuMbbiTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_MBBO:
	{
	    DB_UseKey (db -> vuMbboTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuMbboTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_MBBOFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuMbboTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_PID:
	{
	    DB_UseKey (db -> vuPidTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuPidTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_PIDFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuPidTable, rec);
	    break;
	}

	case VU_SEL:
	{
	    DB_UseKey (db -> vuSelTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuSelTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_SELFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuSelTable, rec);
	    }
	    // delete from sellink.dbf
	    DB_UseKey (db -> vuSelLinkTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuSelLinkTable, seekBuff);
	    break;
	}

	case VU_STATE:
	{
	    DB_UseKey (db -> vuStateTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuStateTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_STATEFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuStateTable, rec);
	    break;
	}

	case VU_STRINGIN:
	{
	    DB_UseKey (db -> vuStringinTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuStringinTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_STRINGINFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuStringinTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_STRINGOUT:
	{
	    DB_UseKey (db -> vuStringoutTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuStringoutTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_STRINGOUTFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuStringoutTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_WAVEFORM:
	{
	    DB_UseKey (db -> vuWaveformTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuWaveformTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_WAVEFORMFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuWaveformTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_LEADLAG:
	{
	    DB_UseKey (db -> vuLeadLagTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuLeadLagTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_LEADLAGFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuLeadLagTable, rec);
	    break;
	}

	case VU_RAMP:
	{
	    DB_UseKey (db -> vuRampTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuRampTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_RAMPFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuRampTable, rec);
	    break;
	}

	case VU_PIPE:
	{
	    DB_UseKey (db -> vuPipeTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuPipeTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_PIPEFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuPipeTable, rec);
	    }
	    // delete from pipecmds.dbf and pipevars.dbf
	    DB_UseKey (db -> vuPipeCmdsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuPipeCmdsTable, seekBuff);
	    DB_UseKey (db -> vuPipeVarsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuPipeVarsTable, seekBuff);
	    break;
	}

	case VU_TANK:
	{
	    DB_UseKey (db -> vuTank1Table, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuTank1Table, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_TANK1FILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuTank1Table, rec);
	    }
	    DB_UseKey (db -> vuTank2Table, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuTank2Table, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_TANK2FILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuTank2Table, rec);
	    }
	    DB_UseKey (db -> vuTank3Table, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuTank3Table, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_TANK3FILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuTank3Table, rec);
	    }
	    DB_UseKey (db -> vuTank4Table, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuTank4Table, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_TANK4FILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuTank4Table, rec);
	    }
	    // delete from tankcmds.dbf, tankprbs.dbf, tankvars.dbf and tankvlvs.dbf
	    DB_UseKey (db -> vuTankCmdsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuTankCmdsTable, seekBuff);
	    DB_UseKey (db -> vuTankPrbsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuTankPrbsTable, seekBuff);
	    DB_UseKey (db -> vuTankVarsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuTankVarsTable, seekBuff);
	    DB_UseKey (db -> vuTankVlvsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuTankVlvsTable, seekBuff);
	    break;
	}

	case VU_VALVE:
	{
	    DB_UseKey (db -> vuValveTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuValveTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_VALVEFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuValveTable, rec);
	    }
	    // delete from vlvvars.dbf
	    DB_UseKey (db -> vuValveVarsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuValveVarsTable, seekBuff);
	    break;
	}

	case VU_PUMP:
	{
	    DB_UseKey (db -> vuPumpTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuPumpTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_PUMPFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuPumpTable, rec);
	    }
	    // delete from vlvvars.dbf
	    DB_UseKey (db -> vuPumpVarsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuPumpVarsTable, seekBuff);
	    break;
	}

	case VU_POT:
	{
	    DB_UseKey (db -> vuPotTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuPotTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_POTFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuPotTable, rec);
	    }
	    // delete from vlvvars.dbf
	    DB_UseKey (db -> vuPotVarsTable, FLD_NAME);
	    DB_DeleteMulti (db -> vuPotVarsTable, seekBuff);
	    break;
	}

	case VU_ACCUM:
	{
	    DB_UseKey (db -> vuAccumTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuAccumTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_ACCUMFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuAccumTable, rec);
	    break;
	}

	case VU_PTO:
	{
	    DB_UseKey (db -> vuPtoTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuPtoTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_PTOFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuPtoTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_DEADTIME:
	{
	    DB_UseKey (db -> vuDeadtimeTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuDeadtimeTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_DEADTIMEFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuDeadtimeTable, rec);
	    break;
	}

	case VU_DATAMOVE:
	{
	    DB_UseKey (db -> vuDatamoveTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuDatamoveTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_DATAMOVEFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuDatamoveTable, rec);
	    break;
	}

	case VU_TD:
	{
	    DB_UseKey (db -> vuTdTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuTdTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_TDFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuTdTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_SIGCHAR:
	{
	    DB_UseKey (db -> vuSigCharTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuSigCharTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_SIGFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuSigCharTable, rec);
	    break;
	}

	case VU_UDB:
	{
	    DB_UseKey (db -> vuUdbTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuUdbTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_UDBFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuUdbTable, rec);

		// delete from udb config file also, it it exists
		DB_UseKey (db -> vuUdbCfgTable, FLD_UDBPGNAME);
		DB_FIELD_PTR udbNameField = DB_MapField (db -> vuUdbTable, FLD_UDBPGNAME);
		char udbName[UDBPROGRAMNAME_SZ + 1];
		DB_GetFieldValue (rec, udbNameField, udbName, &dataType);
		char udbSeekBuff[UDBPROGRAMNAME_SZ + 1];
		sprintf (udbSeekBuff, "%-*.*s\0", UDBPROGRAMNAME_SZ,
						  UDBPROGRAMNAME_SZ,
						  udbName);
		if ((rec = DB_Seek (db -> vuUdbCfgTable, udbSeekBuff)) != 0)
		{
		    DB_DeleteRecord (db -> vuUdbCfgTable, rec);
		}
	    }
	    break;
	}

	case VU_PAT:
	{
	    DB_UseKey (db -> vuPatTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuPatTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_PATFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuPatTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

	case VU_LDFW1:
	{
	    DB_UseKey (db -> vuLdfw1Table, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuLdfw1Table, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_LDFW1FILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuLdfw1Table, rec);
	    break;
	}

	case VU_LDFW2:
	{
	    DB_UseKey (db -> vuLdfw2Table, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuLdfw2Table, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_LDFW2FILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
		break;
	    }
	    DB_DeleteRecord (db -> vuLdfw2Table, rec);
	    break;
	}

	case VU_DDC:
	{
	    DB_UseKey (db -> vuDdcTable, FLD_NAME);
	    if ((rec = DB_Seek (db -> vuDdcTable, seekBuff)) == 0)
	    {
		fileName = vstrCloneScribed (vcharScribeSystem (VU_DDCFILE));
		vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		vstrDestroy (fileName);
		result = vFALSE;
	    }
	    else
	    {
		DB_DeleteRecord (db -> vuDdcTable, rec);
	    }
	    deleteFromioAddr = vTRUE;
	    break;
	}

    } // end switch pointTypeIndex

    if (deleteFromioAddr)
    {
	DB_UseKey (db -> vuIOAddrTable, FLD_NAME);
	switch (pointTypeIndex)
	{
	    case VU_PTO:
	    {
		sprintf (seekBuff, "%-*.*s1\0", PVNAME_SZ, PVNAME_SZ, pvName);
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
		sprintf (seekBuff, "%-*.*s2\0", PVNAME_SZ, PVNAME_SZ, pvName);
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
		break;
	    }

	    case VU_PAT:
	    {
		sprintf (seekBuff, "%-*.*s1\0", PVNAME_SZ, PVNAME_SZ, pvName);
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
		sprintf (seekBuff, "%-*.*s2\0", PVNAME_SZ, PVNAME_SZ, pvName);
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
		sprintf (seekBuff, "%-*.*s3\0", PVNAME_SZ, PVNAME_SZ, pvName);
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
		sprintf (seekBuff, "%-*.*s4\0", PVNAME_SZ, PVNAME_SZ, pvName);
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
		break;
	    }

	    default:
		if ((rec = DB_Seek (db -> vuIOAddrTable, seekBuff)) == 0)
		{
		    fileName = vstrCloneScribed (vcharScribeSystem (VU_IOADDRFILE));
		    vuMessageDialogCenter (NULL, VU_NOT_IN_DELETE_MSG, pvname, fileName);
		    vstrDestroy (fileName);
		    result = vFALSE;
		}
		else
		{
		    DB_DeleteRecord (db -> vuIOAddrTable, rec);
		}
	}
    }
    // use the new arreduc function to get list from arreduc to delete
    // NOTE: func uses epics pt type index where AI = 1 not 0
    //       which is why 1 is added to pointTypeIndex since dataVu indexes
    //	     point type from 0
    int numStrings = 0;
    char **arReduc = dbFindInArreduc (db -> vuArReducTable,
				      pvName, 
				      pointTypeIndex + 1, 
				      &numStrings);
    if (arReduc != NULL)
    {
	DB_UseKey (db -> vuArReducTable, FLD_ARCH_PTNAME_KEY);
	int index = 0;
	char *pvname;
	while (arReduc[index])
	{
	    pvname = strtok (arReduc[index], ",");
	    sprintf (seekBuff, "%-*.*s\0", PVNAME_SZ, PVNAME_SZ, pvname);
	    if ((rec = DB_Seek (db -> vuArReducTable, seekBuff)) != 0)
	    {
		DB_DeleteRecord (db -> vuArReducTable, rec);
	    }
	    free ((void *)arReduc[index]);
	    index++;
	}
	free ((void *)arReduc);
    }
    vstrDestroy (ptTypeName);
    dbUnLock ();
		//
		// tell dbClient about this modification
		//
	if(result == vTRUE)
		vuApp->dbClientNotify->sendNotify((char *)&pvname[0]);
    return (result);
}


/**********************************************************************
*
*    vuFindContainerItem
*	given a container ptr and a tag, find the associated dialogItem
*	this is used by each point type, since there stuff sites in
*	a container
*
**********************************************************************/

vdialogItem *vuFindContainerItem (vcontainer *container, vchar *tagname)
{
    int		i;
    int		containerItemCount;
    vchar	tagName[FLDNAME_SZ + 1];
    vdialogItem	*dialogItem;

    containerItemCount = container -> GetItemCount ();

    // some epics field names are 2/3 chars and a space, ex. "INP " or "KI  "
    // but the tags from the .vr file do not have the trailing spaces
    // so here we get rid of them so we can find the tag
    vcharCopyBounded (tagname, tagName, FLDNAME_SZ);
    tagName[FLDNAME_SZ] = 0;
    if (tagName[3] == ' ')
	tagName[3] = 0;
    if (tagName[2] == ' ')
	tagName[2] = 0;
    if (tagName[1] == ' ')
	tagName[1] = 0;

    for (i = 0; i < containerItemCount; i++)
    {
	dialogItem = container -> GetItemAt (i);
	// if item is a container, we have to look inside
	if( (dialogItem -> IsKindOf (&vcontainer::Kind)) == vTRUE)
	{
	    dialogItem = traverseContainer ((vcontainer *)dialogItem, tagName);
	    if (dialogItem == NULL)
		continue;
	    else
	    {
		return (dialogItem);
	    }
	}
	// not dealing with a container item, simple compare tags
	else
	{
	    const vname *tag = dialogItem -> GetTag ();	
	    if (tag == NULL)
		continue;
	    if ( (vcharCompare (tagName, tag)) == 0)
		return (dialogItem);
	}
    }

    return (NULL);
}


/**********************************************************************
*
*    vuFindDialogItem
*	given a dialog ptr and a tag, find the associated dialogItem
*
**********************************************************************/

vdialogItem *vuFindDialogItem (vdialog *dialog, vchar *tagname)
{
    int		i;
    int		dialogItemCount;
    vchar	tagName[FLDNAME_SZ +1];
    vdialogItem	*dialogItem;
    const vname	*tag;

    dialogItemCount = dialog -> GetItemCount ();

    // some epics field names are 2/3 chars and a space, ex. "INP " or "KI  "
    // but the tags from the .vr file do not have the trailing spaces
    // so here we get rid of them so we can find the tag
    vcharCopyBounded (tagname, tagName, FLDNAME_SZ);
    tagName[FLDNAME_SZ] = 0;
    if (tagName[3] == ' ')
	tagName[3] = 0;
    if (tagName[2] == ' ')
	tagName[2] = 0;
    if (tagName[1] == ' ')
	tagName[1] = 0;

    for (i = 0; i < dialogItemCount; i++)
    {
	dialogItem = dialog -> GetItemAt (i);
	// if item is a container, we have to look inside
	if( (dialogItem -> IsKindOf (&vcontainer::Kind)) == vTRUE)
	{
	    dialogItem = traverseContainer ((vcontainer *)dialogItem, tagName);
	    if (dialogItem == NULL)
		continue;
	    else
	    {
		return (dialogItem);
	    }
	}
	// not dealing with a container item, simple compare tags
	else
	{
	    tag = dialogItem -> GetTag ();	
	    if (tag == NULL)
		continue;
	    if ( (vcharCompare (tagName, tag)) == 0)
		return (dialogItem);
	}
    }

    return (NULL);
}


/*******************************************************************
*
*    vuGetAlarmGroupList ()
*	passed a ptr to the database class
*
*******************************************************************/

vliststr *vuGetAlarmGroupList (vuDatabaseClass *db)
{
    DB_RECORD_PTR	rec;
    DB_FIELD_PTR	nameField;
    char		alarmGroupName [ALARMGRP_SZ + 1];
    int			row = 0;
    int			type;
    vliststr		*newlist = NULL;

    if (db -> vuAlgrListTable == NULL)
    {
	if((db -> vuAlgrListTable = DB_MapTable(VU_ALGRLISTFILE)) == 0)
	{
	    vuMessageDialogCenter (NULL, VU_ALGRFILEERR);
	    return(newlist);
	}
    }
    newlist = new vliststr ();
    nameField = DB_MapField (db -> vuAlgrListTable, FLD_GROUPNAME);
    rec = DB_Top (db -> vuAlgrListTable);
    while (rec)
    {
	DB_GetFieldValue (rec, nameField, alarmGroupName, &type);
	alarmGroupName [ALARMGRP_SZ] = 0;
	newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (alarmGroupName));
	row++;
	rec = DB_SeekNext (db -> vuAlgrListTable);
    }
    DB_UnMapTable (db -> vuAlgrListTable, 0);
    db -> vuAlgrListTable = NULL;
    return (newlist);
}


/*******************************************************************
*
*    vuGetGroupList ()
*	passed a ptr to the database class, return a list of
*	protection groups 
*
*******************************************************************/

vliststr *vuGetGroupList (vuDatabaseClass *db)
{
    DB_RECORD_PTR	rec;
    DB_FIELD_PTR	privGroupFld;
    char		privGroup [PROTECTIONGRP_SZ + 1];
    int			foundIt, type;
    vlistIterator	iterator;
    vstr		*strPrivGroup;
    vchar		*value;
    int			row;

    vliststr *list = new vliststr ();
    privGroupFld = DB_MapField (db -> vuPrivTabTable, FLD_PROT_GRP);
    rec = DB_Top (db -> vuPrivTabTable);
    row = 0;

    while (rec)
    {
	DB_GetFieldValue (rec, privGroupFld, privGroup, &type);
	strPrivGroup = vstrCloneScribed (vcharScribeSystem (privGroup));
	foundIt = vFALSE;
	iterator.Start (list);
	while (iterator.Next ())
	{
	    value = (vchar *)(iterator.GetValue());
	    if ((vcharCompare (strPrivGroup, value)) == 0)
	    {
		foundIt = vTRUE;
		break;
	    }
	}
	iterator.Finish ();
	if (foundIt == vFALSE)
	{
	    list -> SetCellValueScribed (row, 0, vcharScribeSystem(privGroup));
	    row++;
	}
	vstrDestroy (strPrivGroup);
	rec = DB_SeekNext (db -> vuPrivTabTable);
    }

    return (list);
}


/*******************************************************************
*
*    vuGetGroupOpList ()
*	passed a ptr to the database class and an operators name
*	there can be multiple entries for each operator
*
*******************************************************************/

vliststr *vuGetGroupOpList (vuDatabaseClass *db, char *operatorName)
{
    DB_RECORD_PTR	rec;
    DB_FIELD_PTR	privOperNameFld, privGroupFld, privLevelFld;
    char		seekString [OPERATOR_SZ + 1];
    char		privOperName [OPERATOR_SZ + 1];
    char		privGroup [PROTECTIONGRP_SZ + 1];
    char		privLevel [PRIV_LEVEL_SZ + 1];
    char		groupLevel [PROTECTIONGRP_SZ + 1 + PRIV_LEVEL_SZ + 1];
    int			type;

    sprintf (seekString, "%-*.*s\0", OPERATOR_SZ, OPERATOR_SZ, operatorName);
    DB_UseKey (db -> vuPrivTabTable, FLD_OPERID);
    privOperNameFld = DB_MapField (db -> vuPrivTabTable, FLD_OPERID);
    privGroupFld = DB_MapField (db -> vuPrivTabTable, FLD_PROT_GRP);
    privLevelFld = DB_MapField (db -> vuPrivTabTable, FLD_PRIV_LVL);

    rec = DB_Top (db -> vuPrivTabTable);
    rec = DB_Seek (db -> vuPrivTabTable, seekString);

    int row = 0;
    vliststr *newlist = new vliststr ();
    while (rec)
    {
	DB_GetFieldValue (rec, privOperNameFld, privOperName, &type);
	if ( (strncmp (operatorName, privOperName, strlen (operatorName))) == 0)
	{
	    DB_GetFieldValue (rec, privGroupFld, privGroup, &type);
	    DB_GetFieldValue (rec, privLevelFld, privLevel, &type);
	    
	    sprintf (groupLevel, "%-*.*s %-*.*s\0",
				 PROTECTIONGRP_SZ, PROTECTIONGRP_SZ, privGroup,
				 PRIV_LEVEL_SZ, PRIV_LEVEL_SZ, privLevel);
	    newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (groupLevel));
	    row++;
	}
	rec = DB_SeekNext (db -> vuPrivTabTable);
    }

    return (newlist);
}


/*******************************************************************
*
*    vuGetIocList ()
*	passed a ptr to the database class
*
*******************************************************************/

vliststr *vuGetIocList (vuDatabaseClass *db)
{
    DB_RECORD_PTR	rec;
    char		iocName [IOCNAME_SZ + 1];
    int			type;

    rec = DB_Top (db -> vuIocListTable);
    int row = 0;
    vliststr *newlist = new vliststr ();
    while (rec)
    {
	DB_GetFieldValue (rec, db -> iocNameField, iocName, &type);
	iocName [IOCNAME_SZ] = 0;
	newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (iocName));
	row++;
	rec = DB_SeekNext (db -> vuIocListTable);
    }
    return (newlist);
}


/*******************************************************************
*
*    vuGetOperatorList ()
*	passed a ptr to the database class
*	there can be multiple entries for each operator, we only 
*	want 1 each
*
*******************************************************************/

vliststr *vuGetOperatorList (vuDatabaseClass *db)
{
    DB_RECORD_PTR	rec;
    DB_FIELD_PTR	privOperName;
    char		currOpName [OPERATOR_SZ + 1];
    char		lastOpName [OPERATOR_SZ + 1];
    int			type;

    vmemSet ((void *)lastOpName, 0, sizeof (lastOpName));
    DB_UseKey (db -> vuPrivTabTable, FLD_OPERID);
    privOperName = DB_MapField (db -> vuPrivTabTable, FLD_OPERID);
    int row = 0;
    vliststr *newlist = new vliststr ();
    rec = DB_Top (db -> vuPrivTabTable);
    while (rec)
    {
	DB_GetFieldValue (rec, privOperName, currOpName, &type);
	currOpName[OPERATOR_SZ] = 0;
	if ( (strncmp (lastOpName, currOpName, OPERATOR_SZ)) != 0)
	{
	    strncpy (lastOpName, currOpName, OPERATOR_SZ);
	    lastOpName[OPERATOR_SZ] = 0;
	    newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (currOpName));
	    row++;
	}
	rec = DB_SeekNext (db -> vuPrivTabTable);
    }
    return (newlist);
}


/*******************************************************************
*
*    vuGetPointTypeIndex ()
*	given a pointName string, find its index value from the
*	SDR list of point names
*	NOTE: return index value -1, dbRecType ->papname[0] is 
*		NULL, so ai (first) is [1], but everthing in
*		program indexes point type from 0
*
*******************************************************************/

int vuGetPointTypeIndex (char *pointTypeName)
{
    int index;

    for (index = 0; index < dbRecType -> number; index++)
    {
	if (dbRecType -> papName[index] == NULL)
	    continue;
	if ( (strcmp (dbRecType -> papName[index], pointTypeName)) == 0)
	    break;
    }

    return (index - 1);
}


/*******************************************************************
*
*    vuGetPointTypeList ()
*
*******************************************************************/

char **vuGetPointTypeList ( void )
{
    char	**ptList = NULL;
    int		i;
    int		listIndex = 0;
    char	*pointType;

    if ( (ptList = (char **)(vmemAllocAndClear ( (dbRecType -> number + 1) *
						 sizeof (char *) )))== NULL)
    {
	vuMessageDialogCenter (NULL, VU_PTTYPEALLOCERR);
	return (ptList);
    }

    for (i = 0; i < dbRecType -> number; i++)
    {
	if (dbRecType -> papName[i] == NULL)
	    continue;
	if ( (pointType = (char *)(vmemAllocAndClear ( PVNAME_SZ + 1))) == NULL)
	{
	    i = 0;
	    while (ptList [i])
	    {
		vmemFree ((void *)ptList[i]);
		i++;
	    }
	    vmemFree (ptList);
	    vuMessageDialogCenter (NULL, VU_PTTYPEALLOCERR);
	    ptList = NULL;
	    break;
	}
	strncpy (pointType, dbRecType -> papName [i], PVNAME_SZ);
	pointType [PVNAME_SZ] = 0;
	ptList [listIndex++] = pointType;
    }
    return (ptList);
}


/*******************************************************************
*
*    vuGetResList ()
*	passed a ptr to the database class
*
*******************************************************************/

vliststr *vuGetResList (vuDatabaseClass *db, int getDpa)
{
    DB_RECORD_PTR	rec;
    char		resName [RESOURCE_SZ + 1];
    char		isDpa;
    int			type;

    rec = DB_Top (db -> vuResListTable);
    int row = 0;
    vliststr *newlist = new vliststr ();
    while (rec)
    {

	DB_GetFieldValue (rec, db -> resDPAField, &isDpa, &type);
	DB_GetFieldValue (rec, db -> resNameField, resName, &type);
	resName [RESOURCE_SZ] = 0;
	switch (getDpa)
	{
	    case VU_NO_DPAS:
		if (isDpa)
		{
		    break;
		}
		newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (resName));
		row++;
		break;

	    case VU_ALL_RESOURCES:
		newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (resName));
		row++;
		break;

	    case VU_ONLY_DPAS:
		if (isDpa)
		{
		    newlist -> SetCellValueScribed (row, 0, vcharScribeSystem (resName));
		    row++;
		}
		break;

	    default:
		break;
	}

	rec = DB_SeekNext (db -> vuResListTable);
    }
    return (newlist);
}

/*******************************************************************
*
*    vuGetOptionMenuValue ()
*	given a control option menu find the string
*	in the attached menu at set the controls value
*
*******************************************************************/

const vchar *vuGetOptionMenuValue (vcontrolOptionMenu *optionMenu)
{
    const vchar	*value;
    int		controlValue;
    vmenu	*menu;
    vmenuItem	*menuItem;

    controlValue = optionMenu -> GetValue ();
    menu = optionMenu -> GetMenu ();
    menuItem = menu -> GetItemAt (controlValue);
    value = menuItem -> GetTitle ();

    return (value);
}


/*******************************************************************
*
*    vuGoTo ()
*	bring up the Navigator and go to the point selected
*	called from vuMainDialog Go To menu option and
*	vuCommonDialog Go To button
*
*******************************************************************/

void vuGoTo (vuMainDialogClass *mainDialog)
{
    dataVuApplication *dataVuApp = mainDialog -> getApplication ();
    vuDatabaseClass *db = dataVuApp -> getDatabaseClass ();
    vresource vuRes = dataVuApp -> getDataVuvresource ();
    vresource commonResource = vresourceGet (vuRes, vunames_vuCommonDialog);
    vresource titlesRes = mainDialog -> getTitlesResource ();
    vresource findRes = vresourceGet (vuRes, vunames_vuFindDialog);

    vuNavigatorDialogClass *navigatorDialog = mainDialog -> getNavigator ();
    if (navigatorDialog == NULL)
    {
	navigatorDialog = new vuNavigatorDialogClass(findRes);
	mainDialog -> storeNavigator (navigatorDialog);
    }
    else
	navigatorDialog = mainDialog -> getNavigator ();
    vstr *GoTo = vresourceGetString (titlesRes, vunames_vuGoToButtonTitle);
    navigatorDialog -> vuFindOpen(GoTo, mainDialog,vTRUE);
    char **ptList = navigatorDialog -> getPvNameList ();
    if (ptList != NULL)
    {
	int	dbaseType;
	char	pointTypeName[VU_KEY_SIZE];

	// find the record in pvmap and get the point type
	DB_RECORD_PTR pntRecord = DB_Seek (db -> vuPvmapTable, ptList[0]);
	DB_FIELD_PTR pType = DB_MapField(db -> vuPvmapTable, FLD_TYPE);
	DB_GetFieldValue (pntRecord, pType, (void *)pointTypeName, &dbaseType);

	// convert point type string to an index
	int pointType = vuGetPointTypeIndex (pointTypeName);
	vstr *pvname = vstrCloneScribed (vcharScribeSystem (ptList[0]));
	if(db->vuVerifyDb(ptList[0],pointType) != 0)
	    vuNewLoadAndGo (mainDialog, pvname, commonResource, pointType);

	vstrDestroy (pvname);
	vmemFree ((void *)ptList[0]);
	vmemFree ((void *)ptList);
	navigatorDialog ->setPvNameList (NULL);
    }
    vstrDestroy (GoTo);
}


/**********************************************************************
*
*    vuLoadAndGo
*	called by various functions, given a dialog load it with the
*	given pointname, since it is of the same point type
*	
*
**********************************************************************/

void vuLoadAndGo (vdialog *dialog, vchar *pvname, int pointTypeIndex)
{
    switch (pointTypeIndex)
    {
	case VU_AI:
	{
	    vuAiDialogClass *aiDialog = (vuAiDialogClass *)dialog;
	    aiDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_AO:
	{
	    vuAoDialogClass *aoDialog = (vuAoDialogClass *)dialog;
	    aoDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_BI:
	{
	    vuBiDialogClass *biDialog = (vuBiDialogClass *)dialog;
	    biDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_BO:
	{
	    vuBoDialogClass *boDialog = (vuBoDialogClass *)dialog;
	    boDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_CALC:
	{
	    vuCalcDialogClass *calcDialog = (vuCalcDialogClass *)dialog;
	    calcDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_FANOUT:
	{
	    vuFanoutDialogClass *fanoutDialog = (vuFanoutDialogClass *)dialog;
	    fanoutDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_MBBI:
	{
	    vuMbbiDialogClass *mbbiDialog = (vuMbbiDialogClass *)dialog;
	    mbbiDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_MBBO:
	{
	    vuMbboDialogClass *mbboDialog = (vuMbboDialogClass *)dialog;
	    mbboDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_PID:
	{
	    vuPidDialogClass *pidDialog = (vuPidDialogClass *)dialog;
	    pidDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_SEL:
	{
	    vuSelDialogClass *selDialog = (vuSelDialogClass *)dialog;
	    selDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_STATE:
	{
	    vuStateDialogClass *stateDialog = (vuStateDialogClass *)dialog;
	    stateDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_STRINGIN:
	{
	    vuStringinDialogClass *stringinDialog = (vuStringinDialogClass *)dialog;
	    stringinDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_STRINGOUT:
	{
	    vuStringoutDialogClass *stringoutDialog = (vuStringoutDialogClass *)dialog;
	    stringoutDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_WAVEFORM:
	{
	    vuWaveformDialogClass *waveformDialog = (vuWaveformDialogClass *)dialog;
	    waveformDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_LEADLAG:
	{
	    vuLeadLagDialogClass *leadlagDialog = (vuLeadLagDialogClass *)dialog;
	    leadlagDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_RAMP:
	{
	    vuRampDialogClass *rampDialog = (vuRampDialogClass *)dialog;
	    rampDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_PIPE:
	{
	    vuPipeDialogClass *pipeDialog = (vuPipeDialogClass *)dialog;
	    pipeDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_TANK:
	{
	    vuTankDialogClass *tankDialog = (vuTankDialogClass *)dialog;
	    tankDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_VALVE:
	{
	    vuValveDialogClass *valveDialog = (vuValveDialogClass *)dialog;
	    valveDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_PUMP:
	{
	    vuPumpDialogClass *pumpDialog = (vuPumpDialogClass *)dialog;
	    pumpDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_POT:
	{
	    vuPotDialogClass *potDialog = (vuPotDialogClass *)dialog;
	    potDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_ACCUM:
	{
	    vuAccDialogClass *accDialog = (vuAccDialogClass *)dialog;
	    accDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_PTO:
	{
	    vuPtoDialogClass *ptoDialog = (vuPtoDialogClass *)dialog;
	    ptoDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_DEADTIME:
	{
	    vuDeadtimeDialogClass *deadtimeDialog = (vuDeadtimeDialogClass *)dialog;
	    deadtimeDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_DATAMOVE:
	{
	    vuDatamoveDialogClass *datamoveDialog = (vuDatamoveDialogClass *)dialog;
	    datamoveDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_TD:
	{
	    vuTdDialogClass *tdDialog = (vuTdDialogClass *)dialog;
	    tdDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_SIGCHAR:
	{
	    vuSigDialogClass *sigDialog = (vuSigDialogClass *)dialog;
	    sigDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_UDB:
	{
	    vuUdbDialogClass *udbDialog = (vuUdbDialogClass *)dialog;
	    int modified = udbDialog -> vuUdbGetLabelsFlag ();
	    // if labels have been modifed reset for new point
	    if (modified == vTRUE)
	    {
		udbDialog -> resetDialog ();
	    }
	    udbDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_PAT:
	{
	    vuPatDialogClass *patDialog = (vuPatDialogClass *)dialog;
	    patDialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_LDFW1:
	{
	    vuLdfw1DialogClass *ldfw1Dialog = (vuLdfw1DialogClass *)dialog;
	    ldfw1Dialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_LDFW2:
	{
	    vuLdfw2DialogClass *ldfw2Dialog = (vuLdfw2DialogClass *)dialog;
	    ldfw2Dialog -> vuLoadDialog (pvname);
	    break;
	}

	case VU_DDC:
	{
	    vuDdcDialogClass *ddcDialog = (vuDdcDialogClass *)dialog;
	    ddcDialog -> vuLoadDialog (pvname);
	    break;
	}

    } // end switch pointTypeIndex
}


/*******************************************************************
*
*    vuMatchPattern ()
*	match a search pattern
*
*******************************************************************/

int vuMatchPattern (DB_RECORD_PTR rec, DB_FIELD_PTR field, vregexp *regExp)
{
    vstr	*strText;
    vchar	*result;
    char	charText[128];
    int		type;
    int		resultLength;

    DB_GetFieldValue (rec, field, charText, &type);
    strText = vstrCloneScribed (vcharScribeSystem (charText));
    if ( ( result = regExp -> Search (strText, vcharLength (strText), &resultLength)) == NULL)
	type = 0;
    else
	type = 1;
    vstrDestroy (strText);
    return (type);
}


/*******************************************************************
*
*    vuMessageDialogCenter ()
*	passed a message number as defined in vuUtils.hxx
*
*******************************************************************/

int vuMessageDialogCenter (vdialog *dialog, int msgNumber, ...)
{
    int		result = VU_SUCCESS;
    
    vapplication *dataVuApp = vapplication::GetCurrent ();
    vresource    dataVuRes  = dataVuApp -> GetResources ();
    vresource    titleRes   = vresourceGet (dataVuRes,
					vunames_vuDialogTitles);
    vresource    msgRes     = vresourceGet (dataVuRes,
					vunames_vuMessagesDictionary);
    vstr         *infoTitle = vresourceGetString (titleRes,
					vunames_vuInfoNoticeTitle);
    vstr         *cautionTitle = vresourceGetString (titleRes,
					vunames_vuCautionNoticeTitle);
    vstr         *msg = NULL;
    vstr	 *formattedMsg = NULL;
    int          msgType = VU_NO_MSG;
    int		 freeMsg = 1;
    int		 pointType;
    int		 errorValue;
    int		 intValue;
    char	 *cPtr1, *cPtr2;
    vchar	 *vPtr1, *vPtr2;
    vchar	 pointTypeName1[VU_POINTTYPE_NAME_SZ + 1];
    vchar	 pointTypeName2[VU_POINTTYPE_NAME_SZ + 1];
    va_list	 argptr;

    va_start (argptr, msgNumber);

    switch (msgNumber)
    {
	case VU_QUIT_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuQuitMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_XREFERR:
	    msg = vresourceGetString (msgRes, vunames_vuXrefErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PVMAPERR:
	    msg = vresourceGetString (msgRes, vunames_vuPvmapErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_IOCLISTERR:
	    msg = vresourceGetString (msgRes, vunames_vuIocListErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_RESLISTERR:
	    msg = vresourceGetString (msgRes, vunames_vuResListErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PRIVTABERR:
	    msg = vresourceGetString (msgRes, vunames_vuPrivTabErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FIELDTABERR:
	    msg = vresourceGetString (msgRes, vunames_vuFieldProtectErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARREDUCERR:
	    msg = vresourceGetString (msgRes, vunames_vuArReducErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_IOCTABLEERR:
	    msg = vresourceGetString (msgRes, vunames_vuIocBuildErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SDRERR:
	    msg = vresourceGetString (msgRes, vunames_vuSDRLoadMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_IOADDRERR:
	    msg = vresourceGetString (msgRes, vunames_vuIOAddrErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_CALCOPERR:
	    msg = vresourceGetString (msgRes, vunames_vuCalcOpsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SELLINKERR:
	    msg = vresourceGetString (msgRes, vunames_vuSelLinkErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FANLINKERR:
	    msg = vresourceGetString (msgRes, vunames_vuFanLinkErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PUMPVARSERR:
	    msg = vresourceGetString (msgRes, vunames_vuPumpVarsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_VALVEVARSERR:
	    msg = vresourceGetString (msgRes, vunames_vuValveVarsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_TANKVARSERR:
	    msg = vresourceGetString (msgRes, vunames_vuTankVarsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_TANKCMDSERR:
	    msg = vresourceGetString (msgRes, vunames_vuTankCmdsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_TANKPRBSERR:
	    msg = vresourceGetString (msgRes, vunames_vuTankPrbsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_TANKVLVSERR:
	    msg = vresourceGetString (msgRes, vunames_vuTankVlvsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PIPEVARSERR:
	    msg = vresourceGetString (msgRes, vunames_vuPipeVarsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PIPECMDSERR:
	    msg = vresourceGetString (msgRes, vunames_vuPipeCmdsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_POTVARSERR:
	    msg = vresourceGetString (msgRes, vunames_vuPotVarsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ALGRFILEERR:
	    msg = vresourceGetString (msgRes, vunames_vuAlGrpErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ALARMINFOERR:
	    msg = vresourceGetString (msgRes, vunames_vuAlInfoErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_TOMANYSCREENS:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuToManyDialogsMsg);
	    pointType = va_arg (argptr, int);
	    // grab popinter to point type names from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert it to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, pointTypeName1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_MOD_DISMISS_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuModDismissMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_CREATE_NEWPT_ERR:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuCreateNewRecordsErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PTFILEERR:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuPtFileErrMsg);
	    pointType = va_arg (argptr, int);
	    // grab point type name from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, pointTypeName1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break; 

	case VU_PTTYPEALLOCERR:
	    msg = vresourceGetString (msgRes,
				vunames_vuPtTypeAllocErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PTTYPEDIALOGERR:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuPtTypeDialogErrMsg);
	    // grab the point name
	    vPtr1 = va_arg (argptr, vchar *);
	    // grab incorrect point type
	    pointType = va_arg (argptr, int);
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    // grab correct point type
	    pointType = va_arg (argptr, int);
	    cPtr2 = dbRecType -> papName[pointType + 1];
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr2),
				 pointTypeName2, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, 
						       vPtr1,
						       pointTypeName1,
						       pointTypeName2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PTISDELETEDERR:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuPtDeletedErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_POINTTYPE_SELECTED_MSG:
	    msg = vresourceGetString (msgRes,
				vunames_vuNoPtTypeSelErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_RESOURCE_SELECTED_MSG:
	    msg = vresourceGetString (msgRes,
				vunames_vuNoResSelErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_IOCNAME_SELECTED_MSG:
	    msg = vresourceGetString (msgRes,
				vunames_vuNoIocNameSelErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NAVPTALLOCERR_MSG:
	    msg = vresourceGetString (msgRes,
				vunames_vuNavPtAllocErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_PTNAMES_SELECTED_MSG:
	    msg = vresourceGetString (msgRes,
				vunames_vuNoPtNameSelErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_PTNAME_ENTERED_MSG:
	    msg = vresourceGetString (msgRes,
				vunames_vuNoPtNameEnteredErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_CALCEXPRESSIONERR:
	    errorValue = va_arg (argptr, int);
	    switch (errorValue)
	    {
		case CALC_ERR_OPNDNEEDED:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcOpndNeededErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_OPEREXP:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcOperExpectedErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_UNPAREN:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcParenErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_EXPTOBIG:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcExpSizeErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_ILLELEM:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcElementErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_ONEIF:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcQuestionErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_ONEELSE:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcColonErr);
		    msgType = VU_INFO_MSG;
		    break;

		case CALC_ERR_NEEDIF:
		    msg = vresourceGetString (msgRes,
				vunames_vuCalcIfElseErr);
		    msgType = VU_INFO_MSG;
		    break;

		default:
		    formattedMsg = vresourceGetString (msgRes,
				vunames_vuCalcUnknownErr);
		    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, 
							       errorValue));
		    vstrDestroy (formattedMsg);
		    msgType = VU_INFO_MSG;
		    break;
	    }
	    break;

	case VU_PTNOTINPVMAPERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuNotInPvmapErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ENDOFLIST_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuEndOfListMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_STARTOFLIST_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuStartOfListMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_LIST_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuNoListErr);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FANOUT_SELN_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuFanoutSELNErr);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FANOUT_SELNPV_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFanoutSELNPvErr);
	    errorValue = va_arg (argptr, int);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, errorValue));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_UDBCFGERR:
	    msg = vresourceGetString (msgRes, vunames_vuUdbCfgErr);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_UDB_CFG_NO_PGNAME_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuUdbCfgNoPgnameErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_UDB_CFG_UPDATE_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuUdbCfgUpdateMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_UDB_CFG_LABEL_SELECT_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuUdbCfgNoLabelSelectedErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_UDB_CFG_NO_NEWLABEL_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuUdbCfgNoNewLabelErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_UDB_CFG_DELETE_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuUdbCfgDeleteMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_UDB_CFG_NO_RECORD_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuUdbCfgNoRecordErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FLDPROT_CLEAR_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFldPrtClearReqMsg);
	    pointType = va_arg (argptr, int);
	    // grab pointer to point type names from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert it to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    vPtr1=  va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, 
						       pointTypeName1, 
						       vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_FLDPROT_CLEAR_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFldPrtClearErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    pointType = va_arg (argptr, int);
	    // grab pointer to point type names from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert it to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg,
						       vPtr1,
						       pointTypeName1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FLDPROT_APPLY_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFldPrtApplyErrMsg);
	    pointType = va_arg (argptr, int);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    // grab pointer to point type names from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert it to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg,
						       pointTypeName1,
						       vPtr1,
						       vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FLDPROT_APPLY_UPDATE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFldPrtApplyUpdateReqMsg);
	    pointType = va_arg (argptr, int);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    // grab pointer to point type names from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert it to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg,
						       pointTypeName1,
						       vPtr1,
						       vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_FLDPROT_APPLY_NEW_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFldPrtApplyNewReqMsg);
	    pointType = va_arg (argptr, int);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    // grab pointer to point type names from SDR
	    cPtr1 = dbRecType -> papName[pointType + 1];
	    // convert it to vchar
	    vcharCopyScribedBounded (vcharScribeSystem (cPtr1),
				 pointTypeName1, VU_POINTTYPE_NAME_SZ);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg,
						       pointTypeName1,
						       vPtr1,
						       vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_SYSUSERSERR:
	    msg = vresourceGetString (msgRes, vunames_vuSysUsersErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SYSUSERS_APPLY_MOD_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuSysUsersApplyModReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_SYSUSERS_APPLY_ADD_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuSysUsersApplyAddReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_SYSUSERS_DELETE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuSysUsersDeleteReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_SCRIPTFILEERR:
	    msg = vresourceGetString (msgRes, vunames_vuScriptFileErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SCRIPT_NO_SELECTED_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuNoScriptSelErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SCRIPT_NO_ID_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuScriptNoIDErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SCRIPT_NOT_EXIST_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuScriptNotExistErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SCRIPT_NEW_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuScriptCreateReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_SCRIPT_MOD_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuScriptModReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_SCRIPT_DEL_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuScriptDelReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_HOUR_ERR_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuHourErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_MINUTE_ERR_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuMinuteErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_SECONDS_ERR_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuSecondsErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LDFW_PUMPS_DEF_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuLdfwPumpsDefErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LDFW_SEQ_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuLdfwSeqErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LDFW2_NO_LDFW1_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuLdfw2NoLdfw1ErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LDFW2_LDFW1_EXIST_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuLdfw2Ldfw1ExistErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LAUNCHCFGERR:
	    msg = vresourceGetString (msgRes, vunames_vuLaunchCfgErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LAUNCHCFG_NO_KEY_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuLaunchCfgNoKeyErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LAUNCHCFG_NEW_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuLaunchCfgNewReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_LAUNCHCFG_MOD_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuLaunchCfgModReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_LAUNCHCFG_DEL_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuLaunchCfgDelReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_LAUNCHCFG_NOT_EXIST_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuLaunchCfgNotExistErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_LAUNCHCFG_NO_SELECTED_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuLaunchCfgNoSelectedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARCHINFO_UPDATE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuArchInfoUpdateReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_ARCHINFO_ARCH_CLEAN_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuArchInfoCleanErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARCHINFO_NO_FILENAME_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuArchInfoNoFilenameErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARCHINFO_FILENAME_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuArchInfoFilenameErrMsg);
	    intValue = va_arg (argptr, int);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, intValue));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARCHINFO_FILEEXT_ERR:
	    msg = vresourceGetString (msgRes, vunames_vuArchInfoFileExtErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARCHINFO_FILEOPEN_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuArchInfoFileOpenErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    msgType = VU_INFO_MSG;
	    break;

	case VU_FILTER_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuFilterErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NOT_TREND_PT_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuNotTrendPtErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_ARCH_NEW_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuArReducNewReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1, vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_ARCH_UPDATE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuArReducUpdateReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1, vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_ARCH_DELETE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuArReducDeleteReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_APIFILEERR:
	    msg = vresourceGetString (msgRes, vunames_vuApiErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_STRAPFILEERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuStrapErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_DPAFILEERR:
	    msg = vresourceGetString (msgRes, vunames_vuDpaErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_DPA_NODPA_SEL_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuDpaNoSelectErrMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_DPA_UPDATE_REQ_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuDpaUpdateReqMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_DPA_DEL_REQ_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuDpaDelReqMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_DPA_NO_PT_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuDpaNoPtMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NOT_IN_DELETE_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuDeleteNotInErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1, vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NO_LOCK_DELETE_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuDeleteNoLockErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_DELETE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuDeleteReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_DELETE_SELECTED_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuDeleteSelectedReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_ALARMINFO_UPDATE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuAlarmInfoUpdateReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_ALARMINFO_DELETE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuAlarmInfoDeleteReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_ALARMINFO_DELETE_ERR_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuAlarmInfoDeleteErrMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_NOTHING_SELECTED_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuNothingSelectedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_OPPRIV_NO_OP_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuOpPrivNoOpMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_OPPRIV_NO_GRP_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuOpPrivNoGrpMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_OPPRIV_NO_LVL_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuOpPrivNoLvlMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_OPPRIV_NEW_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuOpPrivNewReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1, vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_OPPRIV_UPDATE_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuOpPrivUpdateReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1, vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_OPPRIV_NOREC_MSG:
	    msg = vresourceGetString (msgRes, vunames_vuOpPrivNoRecMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_OPPRIV_DEL_REQ_MSG:
	    formattedMsg = vresourceGetString (msgRes,
				vunames_vuOpPrivDelReqMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1, vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_RESEDIT_RESNAME_ERR:
	    msg = vresourceGetString (msgRes, vunames_ReseditResnameErr);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_RESEDIT_NEW_REQ:
	    formattedMsg = vresourceGetString (msgRes, vunames_ReseditNewReq);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_RESEDIT_UPDATE_REQ:
	    formattedMsg = vresourceGetString (msgRes, vunames_ReseditUpdateReq);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;
	    
	case VU_RESEDIT_DEL_REQ:
	    formattedMsg = vresourceGetString (msgRes, vunames_ReseditDelReq);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_CAUTION_MSG;
	    break;

	case VU_RESEDIT_DEL_ERR:
	    msg = vresourceGetString (msgRes, vunames_ReseditDelErr);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_CA_DBC_RESTART_MSG:
	    msg = vresourceGetString (msgRes, vunames_CaDbcRestartMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_PARTIALPT_MSG:
	    formattedMsg = vresourceGetString (msgRes, vunames_vuPartialPointMsg);
	    vPtr1 = va_arg (argptr, vchar *);
	    vPtr2 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1,vPtr2));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_CA_INIT_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_CaInitErr);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_CA_CONN_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_CaConnErr);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_CA_PUT_ERR:
	    formattedMsg = vresourceGetString (msgRes, vunames_CaPutErr);
	    vPtr1 = va_arg (argptr, vchar *);
	    msg = vstrCloneScribed (vcharScribeFormat (formattedMsg, vPtr1));
	    vstrDestroy (formattedMsg);
	    msgType = VU_INFO_MSG;
	    break;

	case VU_INIT_OPERPRIV_ERR:
	    msg = vresourceGetString (msgRes, vunames_InitOperPrivErr);
	    msgType = VU_INFO_MSG;
	    break;

	default:
	    break;
    }

    va_end (argptr);

    if (vuOutput == VU_CONSOLE_MSG)
    {
    }
    else
    {
	switch (msgType)
	{
	    case VU_NO_MSG:
		break;

	    case VU_INFO_MSG:
		infoNotice (dialog, infoTitle, msg);
		break;

	    case VU_CAUTION_MSG:
		result = cautionNotice (dialog, cautionTitle, msg);
		if (result == vTRUE)
		    result = VU_TRUE;
		else
		    result = VU_FALSE;
		break;

	}
    }
    vstrDestroy (infoTitle);
    vstrDestroy (cautionTitle);
    if ( (msg != NULL) && (freeMsg) )
	vstrDestroy (msg);

    return (result);
}


/**********************************************************************
*
*    vuNewLoadAndGo
*	called by various functions, get a new dialog and load it with
*	with the pvname given
*	
*
**********************************************************************/

void vuNewLoadAndGo (vuMainDialogClass	*mainDialog,
		  vchar			*pvname,
		  vresource		commonResource, 
		  int			ptTypeIndex)
{
    switch (ptTypeIndex)
    {
	case VU_AI:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_AI);
	    vuAiDialogClass *aiDialog = (vuAiDialogClass *)(mainDialog -> newPoint(
							    dialogs,
							    commonResource, 
							    VU_AI));
	    if (aiDialog != NULL)
	    {
		aiDialog -> resetDialog ();
		aiDialog -> vuLoadDialog (pvname);
		aiDialog -> Open ();
	    }
	    break;
	}

	case VU_AO:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_AO);
	    vuAoDialogClass *aoDialog = (vuAoDialogClass *)(mainDialog -> newPoint(
							    dialogs,
							    commonResource, 
							    VU_AO));
	    if (aoDialog != NULL)
	    {
		aoDialog -> resetDialog ();
		aoDialog -> vuLoadDialog (pvname);
		aoDialog -> Open ();
	    }
	    break;
	}

	case VU_BI:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_BI);
	    vuBiDialogClass *biDialog = (vuBiDialogClass *)(mainDialog -> newPoint(
							    dialogs,
							    commonResource, 
							    VU_BI));
	    if (biDialog != NULL)
	    {
		biDialog -> resetDialog ();
		biDialog -> vuLoadDialog (pvname);
		biDialog -> Open ();
	    }
	    break;
	}

	case VU_BO:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_BO);
	    vuBoDialogClass *boDialog = (vuBoDialogClass *)(mainDialog -> newPoint(
							    dialogs,
							    commonResource, 
							    VU_BO));
	    if (boDialog != NULL)
	    {
		boDialog -> resetDialog ();
		boDialog -> vuLoadDialog (pvname);
		boDialog -> Open ();
	    }
	    break;
	}

	case VU_CALC:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_CALC);
	    vuCalcDialogClass *calcDialog = 
				(vuCalcDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_CALC));
	    if (calcDialog != NULL)
	    {
		calcDialog -> resetDialog ();
		calcDialog -> vuLoadDialog (pvname);
		calcDialog -> Open ();
	    }
	    break;
	}

	case VU_FANOUT:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_FANOUT);
	    vuFanoutDialogClass *fanoutDialog = 
				(vuFanoutDialogClass *)(mainDialog -> newPoint(
						        dialogs,
						        commonResource, 
						        VU_FANOUT));
	    if (fanoutDialog != NULL)
	    {
		fanoutDialog -> resetDialog ();
		fanoutDialog -> vuLoadDialog (pvname);
		fanoutDialog -> Open ();
	    }
	    break;
	}

	case VU_MBBI:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_MBBI);
	    vuMbbiDialogClass *mbbiDialog = 
				(vuMbbiDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_MBBI));
	    if (mbbiDialog != NULL)
	    {
		mbbiDialog -> resetDialog ();
		mbbiDialog -> vuLoadDialog (pvname);
		mbbiDialog -> Open ();
	    }
	    break;
	}

	case VU_MBBO:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_MBBO);
	    vuMbboDialogClass *mbboDialog = 
				(vuMbboDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_MBBO));
	    if (mbboDialog != NULL)
	    {
		mbboDialog -> resetDialog ();
		mbboDialog -> vuLoadDialog (pvname);
		mbboDialog -> Open ();
	    }
	    break;
	}

	case VU_PID:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_PID);
	    vuPidDialogClass *pidDialog = 
				(vuPidDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_PID));
	    if (pidDialog != NULL)
	    {
		pidDialog -> resetDialog ();
		pidDialog -> vuLoadDialog (pvname);
		pidDialog -> Open ();
	    }
	    break;
	}

	case VU_SEL:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_SEL);
	    vuSelDialogClass *selDialog = 
				(vuSelDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_SEL));
	    if (selDialog != NULL)
	    {
		selDialog -> resetDialog ();
		selDialog -> vuLoadDialog (pvname);
		selDialog -> Open ();
	    }
	    break;
	}

	case VU_STATE:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_STATE);
	    vuStateDialogClass *stateDialog = 
				(vuStateDialogClass *)(mainDialog -> newPoint(
						       dialogs,
						       commonResource, 
						       VU_STATE));
	    if (stateDialog != NULL)
	    {
		stateDialog -> resetDialog ();
		stateDialog -> vuLoadDialog (pvname);
		stateDialog -> Open ();
	    }
	    break;
	}

	case VU_STRINGIN:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_STRINGIN);
	    vuStringinDialogClass *stringinDialog = 
				(vuStringinDialogClass *)(mainDialog -> newPoint(
						          dialogs,
						          commonResource, 
						          VU_STRINGIN));
	    if (stringinDialog != NULL)
	    {
		stringinDialog -> resetDialog ();
		stringinDialog -> vuLoadDialog (pvname);
		stringinDialog -> Open ();
	    }
	    break;
	}

	case VU_STRINGOUT:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_STRINGOUT);
	    vuStringoutDialogClass *stringoutDialog = 
				(vuStringoutDialogClass *)(mainDialog -> newPoint(
						           dialogs,
						           commonResource, 
						           VU_STRINGOUT));
	    if (stringoutDialog != NULL)
	    {
		stringoutDialog -> resetDialog ();
		stringoutDialog -> vuLoadDialog (pvname);
		stringoutDialog -> Open ();
	    }
	    break;
	}

	case VU_WAVEFORM:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_WAVEFORM);
	    vuWaveformDialogClass *waveformDialog = 
				(vuWaveformDialogClass *)(mainDialog -> newPoint(
						          dialogs,
						          commonResource, 
						          VU_WAVEFORM));
	    if (waveformDialog != NULL)
	    {
		waveformDialog -> resetDialog ();
		waveformDialog -> vuLoadDialog (pvname);
		waveformDialog -> Open ();
	    }
	    break;
	}

	case VU_LEADLAG:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_LEADLAG);
	    vuLeadLagDialogClass *leadlagDialog = 
				(vuLeadLagDialogClass *)(mainDialog -> newPoint(
						         dialogs,
						         commonResource, 
						         VU_LEADLAG));
	    if (leadlagDialog != NULL)
	    {
		leadlagDialog -> resetDialog ();
		leadlagDialog -> vuLoadDialog (pvname);
		leadlagDialog -> Open ();
	    }
	    break;
	}

	case VU_RAMP:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_RAMP);
	    vuRampDialogClass *rampDialog = 
				(vuRampDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_RAMP));
	    if (rampDialog != NULL)
	    {
		rampDialog -> resetDialog ();
		rampDialog -> vuLoadDialog (pvname);
		rampDialog -> Open ();
	    }
	    break;
	}

	case VU_PIPE:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_PIPE);
	    vuPipeDialogClass *pipeDialog = 
				(vuPipeDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_PIPE));
	    if (pipeDialog != NULL)
	    {
		pipeDialog -> resetDialog ();
		pipeDialog -> vuLoadDialog (pvname);
		pipeDialog -> Open ();
	    }
	    break;
	}

	case VU_TANK:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_TANK);
	    vuTankDialogClass *tankDialog = 
				(vuTankDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_TANK));
	    if (tankDialog != NULL)
	    {
		tankDialog -> resetDialog ();
		tankDialog -> vuLoadDialog (pvname);
		tankDialog -> Open ();
	    }
	    break;
	}

	case VU_VALVE:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_VALVE);
	    vuValveDialogClass *valveDialog = 
				(vuValveDialogClass *)(mainDialog -> newPoint(
						       dialogs,
						       commonResource, 
						       VU_VALVE));
	    if (valveDialog != NULL)
	    {
		valveDialog -> resetDialog ();
		valveDialog -> vuLoadDialog (pvname);
		valveDialog -> Open ();
	    }
	    break;
	}

	case VU_PUMP:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_PUMP);
	    vuPumpDialogClass *pumpDialog = 
				(vuPumpDialogClass *)(mainDialog -> newPoint(
						      dialogs,
						      commonResource, 
						      VU_PUMP));
	    if (pumpDialog != NULL)
	    {
		pumpDialog -> resetDialog ();
		pumpDialog -> vuLoadDialog (pvname);
		pumpDialog -> Open ();
	    }
	    break;
	}

	case VU_POT:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_POT);
	    vuPotDialogClass *potDialog = 
				(vuPotDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_POT));
	    if (potDialog != NULL)
	    {
		potDialog -> resetDialog ();
		potDialog -> vuLoadDialog (pvname);
		potDialog -> Open ();
	    }
	    break;
	}

	case VU_ACCUM:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_ACCUM);
	    vuAccDialogClass *accDialog = (vuAccDialogClass *)(mainDialog -> newPoint(
							       dialogs,
							       commonResource, 
							       VU_ACCUM));
	    if (accDialog != NULL)
	    {
		accDialog -> resetDialog ();
		accDialog -> vuLoadDialog (pvname);
		accDialog -> Open ();
	    }
	    break;
	}

	case VU_PTO:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_PTO);
	    vuPtoDialogClass *ptoDialog = 
				(vuPtoDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_PTO));
	    if (ptoDialog != NULL)
	    {
		ptoDialog -> resetDialog ();
		ptoDialog -> vuLoadDialog (pvname);
		ptoDialog -> Open ();
	    }
	    break;
	}

	case VU_DEADTIME:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_DEADTIME);
	    vuDeadtimeDialogClass *deadtimeDialog = 
				(vuDeadtimeDialogClass *)(mainDialog -> newPoint(
							  dialogs,
							  commonResource, 
							  VU_DEADTIME));
	    if (deadtimeDialog != NULL)
	    {
		deadtimeDialog -> resetDialog ();
		deadtimeDialog -> vuLoadDialog (pvname);
		deadtimeDialog -> Open ();
	    }
	    break;
	}

	case VU_DATAMOVE:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_DATAMOVE);
	    vuDatamoveDialogClass *datamoveDialog = 
				(vuDatamoveDialogClass *)(mainDialog -> newPoint(
							  dialogs,
							  commonResource, 
							  VU_DATAMOVE));
	    if (datamoveDialog != NULL)
	    {
		datamoveDialog -> resetDialog ();
		datamoveDialog -> vuLoadDialog (pvname);
		datamoveDialog -> Open ();
	    }
	    break;
	}

	case VU_TD:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_TD);
	    vuTdDialogClass *tdDialog = 
				(vuTdDialogClass *)(mainDialog -> newPoint(
						    dialogs,
						    commonResource, 
						    VU_TD));
	    if (tdDialog != NULL)
	    {
		tdDialog -> resetDialog ();
		tdDialog -> vuLoadDialog (pvname);
		tdDialog -> Open ();
	    }
	    break;
	}

	case VU_SIGCHAR:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_SIGCHAR);
	    vuSigDialogClass *sigDialog = 
				(vuSigDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_SIGCHAR));
	    if (sigDialog != NULL)
	    {
		sigDialog -> resetDialog ();
		sigDialog -> vuLoadDialog (pvname);
		sigDialog -> Open ();
	    }
	    break;
	}

	case VU_UDB:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_UDB);
	    vuUdbDialogClass *udbDialog = 
				(vuUdbDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_UDB));
	    if (udbDialog != NULL)
	    {
		udbDialog -> resetDialog ();
		udbDialog -> vuLoadDialog (pvname);
		udbDialog -> Open ();
	    }
	    break;
	}

	case VU_PAT:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_PAT);
	    vuPatDialogClass *patDialog = 
				(vuPatDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_PAT));
	    if (patDialog != NULL)
	    {
		patDialog -> resetDialog ();
		patDialog -> vuLoadDialog (pvname);
		patDialog -> Open ();
	    }
	    break;
	}

	case VU_LDFW1:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_LDFW1);
	    vuLdfw1DialogClass *ldfw1Dialog = 
				(vuLdfw1DialogClass *)(mainDialog -> newPoint(
						       dialogs,
						       commonResource, 
						       VU_LDFW1));
	    if (ldfw1Dialog != NULL)
	    {
		ldfw1Dialog -> resetDialog ();
		ldfw1Dialog -> vuLoadDialog (pvname);
		ldfw1Dialog -> Open ();
	    }
	    break;
	}

	case VU_LDFW2:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_LDFW2);
	    vuLdfw2DialogClass *ldfw2Dialog = 
				(vuLdfw2DialogClass *)(mainDialog -> newPoint(
						       dialogs,
						       commonResource, 
						       VU_LDFW2));
	    if (ldfw2Dialog != NULL)
	    {
		ldfw2Dialog -> resetDialog ();
		ldfw2Dialog -> vuLoadDialog (pvname);
		ldfw2Dialog -> Open ();
	    }
	    break;
	}

	case VU_DDC:
	{
	    vdialog **dialogs = mainDialog -> getPointUsedPointDialogs(VU_DDC);
	    vuDdcDialogClass *ddcDialog = 
				(vuDdcDialogClass *)(mainDialog -> newPoint(
						     dialogs,
						     commonResource, 
						     VU_DDC));
	    if (ddcDialog != NULL)
	    {
		ddcDialog -> resetDialog ();
		ddcDialog -> vuLoadDialog (pvname);
		ddcDialog -> Open ();
	    }
	    break;
	}

	default:
	    break;
    }  // end switch pointType
}


/*******************************************************************
*
*    vuProtectionGroupFilter ()
*	special filter function for the protection  group field
*	if a space is typed as the first character, the
*	Operator Privilege Dialog is brought up so the user can 
*	make a selection
*
*******************************************************************/

int vuProtectionGroupFilter (filteredText *text, vtextSelection *sel,
			     const vchar **string, long *len)
{
    int		ret;
    vchar	aSpace;

    if (*len > 0)
    {
	aSpace = vcharFromLiteral (' ');
	if (**string == aSpace)
	{
	    filteredTextitem *textItem = (filteredTextitem *)text -> GetUserPtr ();
	    if (textItem != NULL)
	    {
		vdialogItemList *list = textItem -> GetItemListIn ();
		int	ok = vFALSE;
		if (list != NULL)
		{
		    vdialog *dialog = list -> DetermineDialog ();
		    if (dialog != NULL)
		    {
			vuCommonDialogClass *commonDialog = (vuCommonDialogClass *)dialog;
		        vuMainDialogClass *mainDialog = commonDialog -> getMainDialog ();
			dataVuApplication *vuApp = mainDialog -> getApplication ();
			vresource vuRes = vuApp -> getDataVuvresource ();
			vresource grpRes = vresourceGet (vuRes, vunames_vuGrpListDialog);
			vuGrpListDialogClass *grpDialog = new vuGrpListDialogClass (grpRes);
			grpDialog -> vuGrpListOpen (mainDialog);
			vchar *grpName = grpDialog -> getGroupName ();
			vchar *newString = ((vchar *)(*string));
			if (grpName[0] != 0)
			{
			    textItem -> SetTextScribed (vcharScribeSystem (""));
			    *len = vcharLength (grpName);
			    vcharCopyBounded (grpName, newString, PROTECTIONGRP_SZ);
			    ret = NCharFilter (text, sel, string, len, PROTECTIONGRP_SZ - 1);
			    ok = vTRUE;
			}
			else
			{
			    ok = vFALSE;
			}
			delete grpDialog;
		    }
		}
		if (ok == vFALSE)
		{
		    vwindow::Beep ();
		    ret = vFALSE;
		}
	    }
	    else
	    {
		vwindow::Beep ();
		ret = vFALSE;
	    }
	}
	else
	{
	    ret = NCharFilter (text, sel, string, len, PROTECTIONGRP_SZ - 1);
	}
    }
    else
    {
	vwindow::Beep ();
	ret = vFALSE;
    }

    return (ret);
}


/*******************************************************************
*
*    vuPvNameFilter ()
*	special filter function for pvname fields
*	if a space is typed as the first character, the
*	Navigator is brought up so the user can make a selection
*
*******************************************************************/

int vuPvNameFilter (filteredText *text, vtextSelection *sel,
		    const vchar **string, long *len)
{
    int		ret;
    vchar	aSpace;

    if (*len > 0)
    {
	aSpace = vcharFromLiteral (' ');
	if (**string == aSpace)
	{
	    vuFilteredTextitem *textItem = (vuFilteredTextitem *)text -> GetUserPtr ();
	    if (textItem != NULL)
	    {
		vuCommonDialogClass *commonDialog = textItem -> getCommonDialog ();
		int	ok = vFALSE;
		if (commonDialog != NULL)
		{
		    vuMainDialogClass *mainDialog = commonDialog -> getMainDialog ();
		    dataVuApplication *vuApp = mainDialog -> getApplication ();
		    vresource vuRes = vuApp -> getDataVuvresource ();
		    vresource findRes = vresourceGet (vuRes, vunames_vuFindDialog);
		    vuNavigatorDialogClass *navigatorDialog = mainDialog -> getNavigator ();
		    if (navigatorDialog == NULL)
		    {
			navigatorDialog = new vuNavigatorDialogClass(findRes);
			mainDialog -> storeNavigator (navigatorDialog);
		    }
		    else
		    {
			navigatorDialog = mainDialog -> getNavigator ();
		    }
		    vresource titlesRes = mainDialog -> getTitlesResource ();
		    vstr *Ok = vresourceGetString (titlesRes, vunames_vuOKButtonTitle);
		    navigatorDialog -> vuFindOpen (Ok, mainDialog,vTRUE);
		    vstrDestroy (Ok);
		    char **ptList = navigatorDialog -> getPvNameList ();
		    vchar *newString = ((vchar *)(*string));
		    if (ptList != NULL)
		    {
			vstr *pvname = vstrCloneScribed (vcharScribeSystem (ptList[0]));
			*len = vcharLength (pvname);
			vcharCopyBounded (pvname, newString, PVNAME_SZ);
			textItem -> SetTextScribed (vcharScribeSystem (""));
			ret = NCharFilter (text, sel, string, len, PVNAME_SZ - 1);
			vstrDestroy (pvname);
			vmemFree ((void *)ptList[0]);
			vmemFree ((void *)ptList);
			navigatorDialog -> setPvNameList (NULL);
			ok = vTRUE;
		    }
		    else
		    {
			ok = vFALSE;
		    }
		}
		if (ok == vFALSE)
		{
		    vwindow::Beep ();
		    ret = vFALSE;
		}
	    }
	    else
	    {
		vwindow::Beep ();
		ret = vFALSE;
	    }
	}
	else
	{
	    ret = NCharFilter (text, sel, string, len, PVNAME_SZ - 1);
	}
    }
    else
    {
	vwindow::Beep ();
	ret = vFALSE;
    }

    return (ret);
}


/*******************************************************************
*
*    vuSetOptionMenuValue ()
*	given a control option menu and a string, find the string
*	in the attached menu at set the controls value
*
*******************************************************************/

void vuSetOptionMenuValue (vcontrolOptionMenu *optionMenu, vstr *ptr)
{
    int   i;
    vmenu *menu = optionMenu -> GetMenu ();
    int   max   = optionMenu -> GetMax ();
    int   min   = optionMenu -> GetMin ();

    for (i = min; i <= max; i++)
    {
	vmenuItem *menuItem = menu -> GetItemAt (i);
	if ( (vcharCompare ( (const)ptr, menuItem -> GetTitle ())) == 0)
	{
	    optionMenu -> SetValue (i);
	    return;
	}
    }
}


/*******************************************************************
*
*    vuSynchTime ()
*	return the time in GMT and is used for the inter-client
*	synch functions
*
*******************************************************************/

time_t vuSynchTime (void)
{
    time_t	_tt_;

    _tt_ = time(0);
    return (mktime (gmtime (&_tt_)));

}


/*************************************************************************
*
*    traverseContainer ()
*	called recursively by vuFindDialogItem () and vuFindContainerItem
*
*************************************************************************/
static vdialogItem *traverseContainer (vcontainer *container, vchar *tagName)
{
    int		i;
    int		containerItemCount;
    const vname	*tag;
    vdialogItem	*dialogItem;

    containerItemCount = container -> GetItemCount ();

    for (i = 0; i < containerItemCount; i++)
    {
	dialogItem = container -> GetItemAt (i);
	if( (dialogItem -> IsKindOf (&vcontainer::Kind)) == vTRUE)
	    dialogItem = traverseContainer ((vcontainer *)dialogItem, tagName);
	if (dialogItem == NULL)
	    continue;
	tag = dialogItem -> GetTag ();
	if (tag == NULL)
	    continue;
	if ( (vcharCompare (tagName, tag)) == 0)
	    return (dialogItem);
    }

    return (NULL);
}
/******************************************************************
*	File chooser support functions. One open function and a hook
*	function for Ok/Apply button clicks
*
*	GCT 17-Feb-1998
******************************************************************/
vkindTYPED_FULL_DEFINITION(vuFileChooser, vfilechsr, "vuFileChooser");
/*
 * Notification procedure for filechooser for Apply/OK.
 *
 */
int vuFileChooser::HookApplyFile(vfsPath *path)
{

	selectedPath = path->CloneString();	// get the selected path
	return(vTRUE);				// always return true to close dialog
}
vstr *vuFileChooser::getSelectedPath(void)
{
	return(selectedPath);
}
int vuFileChooser::fileChooserOpen(vstr *chooserTitle)
{
	SetSelectMethod(vfilechsrSINGLE);	// select only one item
	// try to place in center of screen
	Place(GetRoot(), vrectPLACE_CENTER, vrectPLACE_CENTER);
	SetTitle(chooserTitle);			// set title from parameter
	selectedPath = NULL;			// nothing selected yet
	Process();												// Open & wait for OK/Cancel
	return(vTRUE);
}
