#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vcharHEADER
#include vnameHEADER
#include vresourceHEADER
#include vdialogHEADER
#include vtextitemHEADER
#include vcontainerHEADER
#include vbuttonHEADER
#include vlistviewHEADER
#include vliststrHEADER
#include vlistitemHEADER
#include vdateHEADER
#include vspinnerHEADER

extern "C"
{
#undef INT_MAX
#undef UINT_MAX
#undef USHRT_MAX
#include "dbDefs.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"
#include "DBProto.h"
}

#include "vunames.h"
#include "dataVu.hxx"
#include "vuMainDialogClass.hxx"
#include "vuDatabaseClass.hxx"
#include "vuiocdialogclass.hxx"
#include "vuUtils.hxx"
#include <memory.h>


vkindTYPED_FULL_DEFINITION(vuIocDialogClass, vdialog, "vuIocDialogClass");

/*******************************************************************
*
*    vuIocDialogClass 
*	LoadInit, constructor
*
*******************************************************************/

void vuIocDialogClass::LoadInit(vresource res)
{
    // and the list
    iocList = vuIocDialogListClass::RequestSubLoad(this,vunames_vuIocList);
    iocList->vuSetIocDialog(this);
    // finally the buttons
    okButton = vuIocOkClass::RequestSubLoad(this,vunames_vuIocOK);
    dismissButton = vuIocDismissClass::RequestSubLoad(this,vunames_vuIocDismiss);
    okButton -> vuSetIocDialog(this);
    dismissButton -> vuSetIocDialog(this);
    // call base class loadInit to finish the job
    vdialog::LoadInit(res);
}


/*******************************************************************
*
*    vuIocEditDialogClass 
*	open function
*
*******************************************************************/

void vuIocDialogClass::vuIocOpen(vuMainDialogClass *mDialog,vstr *buttonTitle)
{
    mainDialog = mDialog;
    char **iocListStr = NULL;
    vliststr *iocListData;
    vlistSelection *selection;
    dataVuApplication *app = mainDialog->getApplication();
    vuDatabaseClass *db = app->getDatabaseClass();

    // destroy any existing listdata
    iocListData = (vliststr *)iocList -> GetList();	
    if(iocListData != NULL) iocListData -> Destroy();

    // then create a new IOC list.
    iocListData = vuGetIocList(db);
	//  tell list view what to look at
    iocList -> SetList(iocListData);
	// set so only one iocource name can be selected
    selection = iocList -> GetSelection();
    iocListData -> SetSelectMethod(selection, vlistONE_ONLY_SELECTION);
    // set proper button title
    okButton->SetTitleScribed(vstrScribe(buttonTitle));
    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    MakeTopMostDialog (this, NULL);
    vevent::Process();
}


/*******************************************************************
*
*    vuIocDialogClass Destroy function
*	if dialog is open, close it and stop its event loop
*
*******************************************************************/

void vuIocDialogClass::Destroy(void)
{
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
    vliststr *listdata;
    listdata = (vliststr *)iocList -> GetList();
    if(listdata) 
	listdata->Destroy();
    vdialog::Destroy();
}


/*******************************************************************
*
*    vuIocDialogClass Observe Window function
*   handle a close event, else handle a cover event
*
*******************************************************************/

void vuIocDialogClass::ObserveWindow (vevent *event)
{
    if ( event -> GetType() == veventWINDOW_STATE && event -> IsClose())
    {
	// zap buffer so user has nothing to use
	clearIocName();
	Close();
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}


vkindTYPED_FULL_DEFINITION(vuIocOkClass, vbutton, "vuIocOkClass");
/*******************************************************************
*
*    vuIocOkButtonClass 
*	Observer function
*
*******************************************************************/
void vuIocOkClass::ObserveDialogItem (vevent *event)
{
    IocDialog -> Close();
    vevent::StopProcessing();
}

vkindTYPED_FULL_DEFINITION(vuIocDismissClass, vbutton, "vuIocDismissClass");
/*******************************************************************
*
*    vuIocDismissButtonClass 
*	Observer function
*
*******************************************************************/

void vuIocDismissClass::ObserveDialogItem (vevent *event)
{
    IocDialog->clearIocName();
    IocDialog -> Close();
    vevent::StopProcessing();
}


vkindTYPED_FULL_DEFINITION(vuIocDialogListClass, vlistview, "vuIocDialogListClass");
/*******************************************************************
*
*    vuIocDialogListClass selection notify function
*	SELECTION_HAS_CHANGED = user finally released the mouse
*	button to establish the new selection.
*
*******************************************************************/

void vuIocDialogListClass::ObserveSelect(int state)
{
    int row,column;
    const vchar *iocname;

    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	vliststr *ptListData = (vliststr *)GetList();
	vlistSelection *listSelection = GetSelection();
	ptListData -> GetSelectedCell (listSelection, &row, &column);
	iocname = (const vchar *) (ptListData -> GetCellValue(row, 0));
	IocDialog->setIocName(iocname);
    }
}
