/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			04-Jan-1993				      */
/*									      */
/*	Module Name:		dbClient				      */
/*	Module Description:						      */
/*									      */
/*		This program will perform the following database operations:  */
/*									      */
/*			1.	Online database modifications		      */
/*			2.	IOC downloading				      */
/*			3.	Database integrity checking		      */
/*									      */
/*	Modification History:	First try		04-Jan-1993	      */
/*				Use lstLib functions	12-Feb-1993	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
#include	<time.h>
#include	<stdio.h>
#include	<signal.h>
#include	<lstLib.h>
#ifndef WIN32
#include	<sys/time.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<sys/ioctl.h>
#include	<sys/fcntl.h>
#include	<netinet/in.h>
#include	<netinet/tcp.h>
#include	<netdb.h>
#include	<error.h>
#else
#include	<winsock.h>
#include	"NTUnixSubs.h"
typedef char * caddr_t;
#endif WIN32
#include	<dbDefs.h>
#include	<dbRecType.h>
#include	<dbRecDes.h>
#include	<dbRecords.h>
#include	<dbFldTypes.h>
#include	<redmsgs.h>
#include	<DBDefines.h>
#include	<DBCVerifyDefs.h>
#include	<DBCDefines.h>
#include	<DBProto.h>
#include	<errno.h>
#include	<msgMon.h>
#include	<sys/stat.h>
#ifdef SOLARIS
#include	<fcntl.h>
#endif

#if defined(SOLARIS) || defined(AXP) || defined(WIN32)
#define bzero(b,n)	memset (b, 0, n)
#define bcopy(s,d,n)	memcpy (d, s, n)
#endif
#define DONT_USE_UDP
#ifdef PRIV_AUDIT
/* for the auditing and priviledge functions */
#include "sslShared.h"
#include "iocPriv.h"
#include "auditSystem.h"
extern AUDIT_ID	auditLog;	/* the system log */

static char	X509_dn [X509_MAX_DN]; /* distinguished name from certificate */

#endif PRIV_AUDIT


#define	MAX_BUFFS	1
#define	FLD_HOSTFLAG	"IS_HOST"
#define	FLD_IOCNAME    	"IOCNAME"
#define DBC_THROTTLE_PARAMS "/tis4000db/dbThrottle.txt"
/*
	Net name list node
*/
typedef	struct
{
  NODE	node;
  unsigned int	net_addr;
  char	net_name[32];
}DBC_NETNAME,*DBC_NETNAME_PTR;
extern	int	errno;
extern	FILE	*dbLog;		/* from DBInitDatabase */
DB_RECORD_PTR	DB_NextVerifyInfo(),DB_FirstVerifyInfo(),DB_FindVerifyInfo();
/*
	Database pointers.
*/
DB_TABLE_PTR	DBC_PvmapTable,DBC_IOAddrTable,DBC_IOCListTable,DBC_ArReducTable;
DB_TABLE_PTR	calcOpsTable,pipeVarsTable,pipeCmdsTable,pidTable,accumTable;
DB_TABLE_PTR	tankVarsTable,tankCmdsTable,tankPrbsTable,tankVlvsTable,valveVarsTable;
DB_TABLE_PTR	selLinkTable,fanLinkTable,pumpVarsTable,potVarsTable;
DB_FIELD_PTR	pvmapTypeField,pvmapCRCField,pvmapIOCField[2],DBC_IOCSyncField,DBC_IOCIsHost,DBC_IOCName;
DB_FIELD_PTR	ioaddrName,ioaddrLine,ioaddrAddress,ioaddrCard,ioaddrSignal,pvmapNameField,pvmapDeleteFlag;
DB_FIELD_PTR	pipeCmdField;
#ifndef WIN32
ERRSYMTAB_ID	errSymTbl;				/* defined in error.h */
#endif
LIST		DBC_RESHeader;				/* doubly linked list stuff */
LIST		DBC_IOCHeader;
DBC_XREFHDR_PTR	DBC_XREFTables;				/* indexed by point type */
char		DBC_DefaultDatabase[MAXPATHLEN];	/* default database path */
char		DBC_DefaultSDR[MAXPATHLEN];		/* default SDR file name */
DBC_RESINFO_PTR	dbcResSearch();
DBC_IOCINFO_PTR	dbcIocSearch();
DBC_IOCINFO_PTR	DBC_LastIOC;
void	dbcSendVerifyInfo(DBC_IOCINFO_PTR hostinfo);
void	dbcCheckVerifyInfo(DBC_IOCINFO_PTR hostinfo);
void	dbcHandleHostMsgs(DBC_IOCINFO_PTR iocinfo);
void   	dbcHandleBCastResp();
void   	dbcHandleIOCConnect(int *newfd,int addr);
void   	dbcHandleHostConnect(int *newfd,int addr);
void  	dbcHandleIOCMsgs();
void	DBC_ProcessAwake();
void	dbcHandleUtilMsgs();
void   	DBC_CheckTimeStamps();
void   	sendBoomerang(),refreshFds();
void	dbSetNoBlock(DBC_IOCINFO_PTR info);
void	dbcConnectToHost(DBC_IOCINFO_PTR node);
void	dbcInitializeHosts(DBC_IOCINFO_PTR hostinfo,int num);
void	dbcDoAccept(int addr,DBC_IOCINFO_PTR node,int *newfd);
int	dbcFindHost(int addr);
void	maxSocket(int fd);
void	dbCtrace (TRACE_TYPE type, DBC_IOCINFO_PTR iocinfo);
short	haveCompleteMsg(DBC_IOCINFO_PTR);
DBC_ERROR_CODES	DBC_ProcVerifyRequest(),DBC_ProcVerifyReply(),DBC_ProcRecordRequest(),DBC_ProcRecordReply(),DBC_ProcNewRecRequest();
DBC_ERROR_CODES	DBC_InvalidMessage(),DBC_CRCResponse(),DBC_DownloadRequest(),DBC_SDRRequest();
DBC_ERROR_CODES	DBC_ProcessDbMod(),DBC_OnlineDnldRequest(),DBC_DatabaseInit();
DBC_ERROR_CODES	DBC_ProcessSymResp(),DBC_ProcessNewSymResp(),DBC_ProcessSymPurge(),dbcNetworkInit(),dbcBuildIOCTable();
DBC_ERROR_CODES	DBC_OnlineDelResponse(),DBC_InitCheckSum(),DBC_OkToUnmark(),dbcForwardDbMod(DBC_IOCINFO_PTR);
DBC_ERROR_CODES dbcDownloadRequest(DBC_IOCINFO_PTR iocinfo);
DBC_ERROR_CODES dbcOnlineDownloadRequest(DBC_IOCINFO_PTR iocinfo);
DBC_ERROR_CODES dbcResetAck(DBC_IOCINFO_PTR iocinfo);/* GCT 4-May-1999 new for UDP removal */
DBC_ERROR_CODES dbcDeleteStatus(DBC_IOCINFO_PTR iocinfo);/* GCT 6-May-1999 new for UDP removal */
void		dbcPollIOCs();
void		dbSignalHandler(),DBC_DropFD();	/* signal handler */
void processIocInhibitList(void);
void allowAllIocs(void);
struct	dbdatabase	DB_Database;			/* for DB functions */
struct	timeval	DBC_PollTimer;
struct	timeval	maxWait,exitTime;
struct	timeval	currTime;
char		DBC_BCastMsgBuff[1024];
char		dbVerbose,*searchNetName(),inetString[20],*addrToString();
LIST		DBC_NetNameList;
int		DBC_Cardiac;
DBC_RECDESC	downloadRecDesc[DBC_WINDOW_SIZE];	/* one for each record downloaded */
int    	maxRecSize;
int    	myAddr;			       	/* my INET address */
char  dbThrottleParams[100];
typedef enum {INVALID,VALID} THROTTLE_STATE;
typedef struct
{
  THROTTLE_STATE throttleStatus;	/* INVALID means no info parsed...VALID means data is usable */
  int  pointsInSyncMsg;			/* points in sync message to other hosts. default 20 */
  int  timeToSync;			/* time between host sync messages. default 15 */
  int  numPointsToWatch;		/* number of points to watch during sync cycles */
  char pointsToWatch[8][PVNAME_SZ+1];	/* points worth looking at during sync cycles */
  int  numPointsToSync;			/* number of points to sync NOW */
  char pointsToSync[8][PVNAME_SZ+1];	/* points to sync NOW */
} THROTTLE_INFO;
THROTTLE_INFO  dbThrottleInfo;
int parseThrottleParams(void);		/* parse throttle params from file 0 return means failure 1=success */
static void kick_TCP (DBC_IOCINFO_PTR iocinfo /* in net order */,int port /* in host order */);
/*
	Dispatch Table for incomming network messages
*/
DBC_ERROR_CODES	(*DBC_Dispatch[DBC_MAX_DISPATCH])();
/*
	State strings for VERBOSE mode
*/
char	*state_string[] =
{
  "IOC_OK",
  "IOC_DOWNLOADING",
  "IOC_SDRDWNLD",
  "IOC_ERROR",
  "IOC_CRCCHECK",
  "IOC_SYMREQ",
  "IOC_NONET",
  "IOC_INHIBITED"
};
/*
	dbClient <-> dbServer messages for VERBOSE mode
*/
char	*DBC_WhatMessage[] =	/* actual index will be type - 64 */
{
  "IOC Wakeup call from dbClient C->S.",       	/* 64 */
  "Symbol table request, C->S",		       	/* 65 */
  "Symbol table response, S->C",	       	/* 66 */
  "IOC response to Wakeup call S->C",	       	/* 67 */
  "Symbol table purge order, S->C",	       	/* 68 */
  "Checksum list from IOC S->C",	       	/* 69 */
  "Request for database records S->C",	       	/* 70 */
  "Response for database download C->S",       	/* 71 */
  "",
  "",
  "Record modification request S->C",	       	/* 74 */
  "Record upload request, C->S",	       	/* 75 */
  "Online download notify C->S",	       	/* 76 */
  "Online download request S->C",	       	/* 77 */
  "Online download response C->S",	       	/* 78 */
  "New record symbol S-> C",		       	/* 79 */
  "Heartbeat between client and server",       	/* 80 */
  "On-line point delete util. -> S",	       	/* 81 */
  "On-line point delete S -> C",	       	/* 82 */
  "Server point deletion status",	  	/* 83 */
  "",
  "IOC Reset Ack S -> C",			/* 85 */
  "",
  "",
  "OK to un-mark points",		       	/* 88 */
  "Forwarded DBMODREQ message",			/* 89 */
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "",
  "Inter-host verify request message",		/* 100 */
  "Inter-host verify reply message",	       	/* 101 */
  "Inter-host record request message",		/* 102 */
  "Inter-host record response message",		/* 103 */
  "Inter-host immediate record request"		/* 104 */
};
/*
	Message level strings
*/
char	*msgLevel[] =
{
  "Silent",
  "Informational",
  "Status",
  "DEBUG!"
};
/*
	Network sockets
*/
int	DBC_UDPSocket,listenSocket;
int	maxFd = -1;
fd_set	myFds,allFds;
/*
  New stuff for new host connection management
*/
ACCEPT_STATE	acceptIOC;
int maxConnected;
int numConnected;
int numToConnect;
DBC_IOCINFO  hostInfo[DBC_MAX_HOSTS];		/* host info for inter-host sync */;
char partnerName[DBC_IOCNAMESIZE+1];		/* other guy with -r switch */
DBC_IOCINFO	utilIoc[DBC_MAX_HOSTS];		/* GCT 11-Jul-1997 handle more dataVus */
int creatingdbTree = 0;

void main(argc,argv)
int	argc;					/* get database location */
char	*argv[];
{
int	DBC_Status,i,select_stat,cmd;
DBC_IOCINFO_PTR	iocinfo;
struct hostent	*ent;
char	defaultSDR[MAXPATHLEN],hostName[MAXHOSTNAMELEN];
extern	char	*optarg;
extern	int	optind;
time_t	start_time;
struct sockaddr_in	sockAddr;
int  	len;
int  addr;
int  acceptFd;
struct tm *checkTime;
int today = -1;			/* set to -1 to force the first check */
#ifdef WIN32
    WSADATA wsaData; 
    WORD    wVersionRequested = MAKEWORD(1, 1);
	int err;
#endif WIN32

serverInit(DBCPATH,DBCLOCK);	/* set up stuff like channel access does */

#ifdef PRIV_AUDIT
    if (sslSharedInit () == 0)
    {
	VERBOSE(V_INFO)(LOG_ALERT|LOG_LOCAL1,"dbClient can't prepare SSL context!");
	exit(1);
    }

    /* open default audit trail file */
    if (InitSystemLog (NULL, 0) == NULL)
    {
	VERBOSE(V_INFO)(LOG_ALERT|LOG_LOCAL1,"dbClient can't open default system audit file %s!\nNo auditing will occur\n", systemLogFile);
    }
#endif PRIV_AUDIT


/*
	Setup dispatch table
*/
for(i = 0;i < DBC_MAX_DISPATCH; i++)
	DBC_Dispatch[i] = 0;
DBC_Dispatch[DBC_MSG_DWNLDREQ]		= DBC_DownloadRequest;
DBC_Dispatch[DBC_MSG_OLDREQ]		= DBC_OnlineDnldRequest;
DBC_Dispatch[DBC_MSG_DBMODREQ]		= DBC_ProcessDbMod;
DBC_Dispatch[DBC_MSG_SYMRESPONSE]	= DBC_ProcessSymResp;
DBC_Dispatch[DBC_MSG_NEWSYM]		= DBC_ProcessNewSymResp;		/* New symbol from online load */
DBC_Dispatch[DBC_MSG_SYMPURGE]		= DBC_ProcessSymPurge;
DBC_Dispatch[DBC_MSG_OLDELRESP]		= DBC_OnlineDelResponse;
DBC_Dispatch[DBC_MSG_OKTOUNMARK]	= DBC_OkToUnmark;
DBC_Dispatch[DBC_MSG_FWDDBMOD]		= dbcForwardDbMod;
DBC_Dispatch[DBC_MSG_RESETACK]		= dbcResetAck;
DBC_Dispatch[DBC_MSG_DELSTATUS]		= dbcDeleteStatus;
/*
  30-Apr-1999 GCT
  
  The CRC response message is now recieved from the TCP socket so we must
  include it in the dispatch table.
*/
DBC_Dispatch[DBC_MSG_CRCRESPONSE]      	= DBC_CRCResponse;
/*
	Inter-host database sync
*/
DBC_Dispatch[VI_MSG_VERIFYREQUEST]	= DBC_ProcVerifyRequest;
DBC_Dispatch[VI_MSG_RECORDREQUEST]	= DBC_ProcRecordRequest;
DBC_Dispatch[VI_MSG_RECORDREPLY]	= DBC_ProcRecordReply;
DBC_Dispatch[VI_MSG_IHAVENEWREC]	= DBC_ProcNewRecRequest;
#ifndef WIN32
#ifdef SOLARIS
sigset(SIGINT,dbSignalHandler);			/* handle ^C from keyboard */
sigset(SIGPIPE,dbSignalHandler);       	       	/* handle broken pipes on writes */
sigset(SIGUSR1,dbSignalHandler);		/* increment verbose flag */
sigset(SIGUSR2,dbSignalHandler);		/* decrement verbose flag */
#else
signal(SIGINT,dbSignalHandler);			/* handle ^C from keyboard */
signal(SIGPIPE,dbSignalHandler);       	       	/* handle broken pipes on writes */
/*signal(SIGUSR1,dbSignalHandler);		increment verbose flag */
/*signal(SIGUSR2,dbSignalHandler);		decrement verbose flag */
#endif
#endif WIN32

DBC_UDPSocket = -1;
listenSocket = -1;
tisOpenLog("dbClient",V_DEBUG,0,0,0);
strcpy(DBC_DefaultDatabase,DBASEPATH);		/* default /tis4000db */
strcpy(defaultSDR,DBC_SDRFILE);			/* default default.dctsdr */
strcpy(dbThrottleParams,DBC_THROTTLE_PARAMS);	/* default /tis4000db/dbThrottle.txt */
acceptIOC = NO_ACCEPT_IOCS;    			/* default is to NOT accept connections from IOCs/RTUs */
maxConnected = 0;				/* maximun number of connections to this host */
numConnected = 0;				/* number of actual host connections */
dbVerbose = V_INFO;
DBC_Cardiac = DBC_CARDIAC_ARREST;
while((cmd = getopt(argc,argv,"v:p:h:t:rk")) != -1)
  {
    switch(cmd)
      {
      case 'v':
	dbVerbose = V_DEBUG;
	if(*optarg != '-')
	  {
	    dbVerbose = abs(atoi(optarg));	/* set initial verbosity level */
	    tisBumpLogLevel(dbVerbose);
	    break;
	  }
	else
	  fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],cmd);
	break;
      case 'p':
	if(*optarg != '-')
	  {
	    strcpy(DBC_DefaultDatabase,optarg);
	    break;
	  }
	else
	  fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],cmd);
	break;
      case 'h':
	if(*optarg != '-')
	  {
	    DBC_Cardiac = abs(atoi(optarg));	/* get time since last I heard from IOC/HOST */
	    break;
	  }
	else
	  fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],cmd);
	break;
      case 't':
	if(*optarg != '-')
	  {
	    strcpy(dbThrottleParams,optarg);	/* get the throttle params file name */
	    parseThrottleParams();
	    break;
	  }
	else
	  fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],cmd);
	break;
      case 'r':
	acceptIOC = ACCEPT_IOCS;
	break;
      case 'k':
	DBC_Dispatch[DBC_MSG_DWNLDREQ] = dbcDownloadRequest;
	DBC_Dispatch[DBC_MSG_OLDREQ] = dbcOnlineDownloadRequest;
	break;
      case '?':
	fprintf(stderr,"Usage is:\n");
	fprintf(stderr,"\tdbClient - p pathname -h timeout -v level -t file -r [host1] [host2]\n");
	fprintf(stderr,"\tpathname - location of database files (%s).\n",DBASEPATH);
	fprintf(stderr,"\ttimeout -- time since last recv (%d).\n",DBC_CARDIAC_ARREST);
	fprintf(stderr,"\tlevel ---- debug level 1,2,3(%d)\n",V_INFO);
	fprintf(stderr,"\tfile ----- throttle params file(%s)\n",DBC_THROTTLE_PARAMS);
	fprintf(stderr,"\t-r ------- if present then accept connections from RTUs/IOCs\n");
	exit(1);
      }
  }
/*
  get the rest of the command line args. These will be the hosts designated for
  connection. Max=2(for now!).
*/
for(i=0;i < DBC_MAX_HOSTS;i++)
  {
    hostInfo[i].ioc_name[0] = 0;
    hostInfo[i].ioc_state = IOC_NONET;		       	/* No network connection for now */
    hostInfo[i].ioc_socket = -1;		       	/* Not connected yet */
    hostInfo[i].ioc_host = 1;			       	/* IOC is a host */
    hostInfo[i].ioc_rxCount = 0;		       	/* nothing in buffer yet */
    hostInfo[i].ioc_rxNextChar = 0;			/* start of buffer */
    hostInfo[i].ioc_addr = 0;				/* used to talk to other hosts */
    /*
      GCT 11-Jul-1997 Init all the possible utilIocs here. There could
      be many dataVus connected.
    */
    sprintf(&utilIoc[i].ioc_name[0],"Utility%d\0",i);
    utilIoc[i].ioc_state = IOC_NONET;		       	/* No network connection for now */
    utilIoc[i].ioc_socket = -1;			       	/* Not connected yet */
    utilIoc[i].ioc_host = 0;			       	/* IOC is a host */
    utilIoc[i].ioc_rxCount = 0;				/* nothing in buffer yet */
    utilIoc[i].ioc_rxNextChar = 0;			/* start of buffer */
    utilIoc[i].ioc_addr = 0;				/* used to talk to other hosts */
  }
for(numToConnect = 0,i = optind;i < argc;i++)
  strcpy(&hostInfo[numToConnect++].ioc_name[0],argv[i]);
/*
   GCT 22-Mar-1997 from Vegas 
   Save name of partner host if there is one. This is to prevent
   THIS host from forwarding field update messages from IOCs/RTUs
   to the partner
*/
partnerName[0] = 0;				/* in case there is no partner (dbClient -r) */
if(numToConnect > 0)
  strcpy(partnerName,&hostInfo[0].ioc_name[0]);
tisSysLog(V_INFO,0,"Database path %s,Receive timeout %d",DBC_DefaultDatabase,DBC_Cardiac);
#ifndef WIN32
sprintf(DBC_DefaultSDR,"%s/%s\0",DBC_DefaultDatabase,defaultSDR);
#else
sprintf(DBC_DefaultSDR,"%s\\%s\0",DBC_DefaultDatabase,defaultSDR);
#endif WIN32
lstInit(&DBC_IOCHeader);			/* init the list headers */
lstInit(&DBC_RESHeader);
/*
	Setup database and network
*/
#ifdef WIN32
err = WSAStartup(wVersionRequested, &wsaData); 
if (err != 0) 
/* Tell the user that we couldn't find a useable */ 
/* winsock.dll.     */ 
	exit(0); 
#endif WIN32
myAddr = 0;
gethostname(hostName,MAXHOSTNAMELEN);						/* get my network name */
if(ent = gethostbyname(hostName))							/* get my INET address also */
  myAddr = ntohl(*(long *)*ent -> h_addr_list);
if(acceptIOC == ACCEPT_IOCS)
  {
    if((DBC_Status = dbcNetworkInit()) != DBC_SUCCESS)
      {
	perror(DBC_ErrorMsg[DBC_Status]);
	exit(2);
      }
  }
if((DBC_Status = DBC_DatabaseInit(argv[0])) != DBC_SUCCESS)
  {
    perror(DBC_ErrorMsg[DBC_Status]);
    exit(1);
  }
/*
	Say hello.
*/
start_time = time(0);
tisSysLog(V_INFO,0,"Starting at: %s",ctime(&start_time));
if(acceptIOC == ACCEPT_IOCS)
  dbcPollIOCs();					/* start off by polling IOCS */
dbcInitializeHosts(hostInfo,numToConnect);      		/* init hostInfo array; */
for(i = 0;i < numToConnect;i++)
  dbcConnectToHost(&hostInfo[i]);			/* connect to Hosts */
/*
  Main message wait loop. The select inside this loop wait for messages from hosts & IOCs
*/
while(1)
  {
    maxWait.tv_sec = DBC_POLL_TIMER_VALUE;
    maxWait.tv_usec = 0;
    while((maxWait.tv_sec >= 0) && (maxWait.tv_usec >= 0))
      {
#ifdef WIN32
	currTime.tv_sec = time(0);
	currTime.tv_usec = 0;
#else
	gettimeofday(&currTime,0);
#endif WIN32
	refreshFds(&myFds);
	bcopy(&myFds,&allFds,sizeof(fd_set));
	select_stat = select(maxFd+1,&allFds,0,0,&maxWait);
	if(select_stat == 0)
	  break;
	else if(select_stat < 0)
	{
	    tisSysLog(V_DEBUG,0,"Error from select...%m");
	}
	/*
	  A socket has something for us to read. Find out which one and
	  read it.
	  */
	else
	  {
	    if((DBC_UDPSocket != -1) && FD_ISSET(DBC_UDPSocket,&allFds))		/* broadcast from IOC */
	      dbcHandleBCastResp();
	    if((listenSocket != -1) && FD_ISSET(listenSocket,&allFds) && (acceptIOC == ACCEPT_IOCS))   	/* listening for connects */
	      {
#ifdef PRIV_AUDIT
		/* promote the new socket to SSL */
		SSL  *newConn;
		char *hostName;

		if (do_accept (listenSocket, &acceptFd, &hostName))
		    {
		    if ((newConn = setup_ssl_accept (acceptFd)))
			{
			if (accept_ssl_connection_timed (newConn, 15/*seconds*/))
			    {
			    /* validate user privilege */
			    GetPeerCN (newConn, X509_dn, sizeof(X509_dn));
			    if (GetGeneralPrivHost (X509_dn, "DB_") < 1)
				{
				char errMsg [80];

				GetPeerDN (newConn, X509_dn, sizeof(X509_dn));
				sprintf (errMsg, "db Connect privilege error from %s for %s.",
					hostName, X509_dn);
				SecurityAlarm (X509_dn, errMsg, 0);
				if (auditLog)
				    {
				    AuditAddEvent (auditLog, AUDIT_LOGON, AUDIT_FAIL, X509_dn,
						    "dbClient", errMsg);
				    }
				sslSharedCloseConn (newConn);
				}
			    else
				{
				int nameLen = sizeof(sockAddr);

				GetPeerDN (newConn, X509_dn, sizeof(X509_dn));
				if (auditLog)
				    {
				    char msg [80];

				    sprintf (msg, "db connect from %s.", hostName);
				    AuditAddEvent (auditLog, AUDIT_LOGON, AUDIT_SUCCESS, X509_dn,
						   "dbClient", msg);
				    }
				getpeername (acceptFd, (struct sockaddr *)&sockAddr, &nameLen);
				addr = ntohl(sockAddr.sin_addr.s_addr);
				dbcHandleIOCConnect(&acceptFd,addr);
				if(acceptFd != -1)
				  dbcHandleHostConnect(&acceptFd,addr);
				}
			    }
			else
			    {
			    VERBOSE (V_INFO)(LOG_ALERT|LOG_LOCAL1, "dbClient: error engaging SSL with %s\n", hostName);
			    sslSharedCloseConn (newConn);
			    free (hostName);
			    }
			}
		    else
			{
			VERBOSE (V_INFO)(LOG_ALERT|LOG_LOCAL1,"dbClient: SSL build failed for %s - status = 0x%x\n",
				  hostName, errno);
			close (acceptFd);
			free (hostName);
			}
		    }
#else
		len = sizeof(sockAddr);
		if((acceptFd = accept(listenSocket,(struct sockaddr *)&sockAddr,&len)) >= 0)
		  {
		    addr = ntohl(sockAddr.sin_addr.s_addr);
		    dbcHandleIOCConnect(&acceptFd,addr);
		    if(acceptFd != -1)
		      dbcHandleHostConnect(&acceptFd,htonl(addr));
		  }
#endif PRIV_AUDIT
	      }
	    /*
	      GCT 11-Jul-1997 Check ALL possible utilIoc connections for incomming messages
	    */
	    for(i = 0;i < DBC_MAX_HOSTS;i++)
	      {
		if((utilIoc[i].ioc_socket != -1) && FD_ISSET(utilIoc[i].ioc_socket,&allFds))
		  dbcHandleUtilMsgs(&utilIoc[i]);
	      }
	    /*
	      Must have been one from an IOC/HOST that I am connected
	      to already. Find out which one and call the appropriate
	      function with the iocinfo parameter.

	      7-Jan-97 check for IOC messages only if we are communicating with
	      IOCS (-r).
	    */
	    if(acceptIOC == ACCEPT_IOCS)
	      {
		iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
		while(iocinfo)
		  {
		    if((iocinfo->ioc_socket != -1) && FD_ISSET(iocinfo->ioc_socket,&allFds))
		      dbcHandleIOCMsgs(iocinfo);
		    iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
		  }
	      }
	    /*
	      7-Jan-97 check for any host messages. These would be messages for the
	      database verification feature.
	    */
	    for(i = 0;i < DBC_MAX_HOSTS;i++)
	      {
		if((hostInfo[i].ioc_socket != -1) && FD_ISSET(hostInfo[i].ioc_socket,&allFds))
		  dbcHandleHostMsgs(&hostInfo[i]);
	      }
	  }
	/*
	  adjust timer
	  */
#ifdef WIN32
	exitTime.tv_sec = time(0);
	exitTime.tv_usec = 0;
#else
	gettimeofday(&exitTime,0);
#endif WIN32
	exitTime.tv_sec -= currTime.tv_sec;
	exitTime.tv_usec -= currTime.tv_usec;
	if(exitTime.tv_usec < 0)
	  {
	    exitTime.tv_usec += 1000000;
	    exitTime.tv_sec--;
	  }
	maxWait.tv_sec -= exitTime.tv_sec;
	maxWait.tv_usec -= exitTime.tv_usec;
	if(maxWait.tv_usec < 0)
	  {
	    maxWait.tv_usec += 1000000;
	    maxWait.tv_sec--;
	  }
      }/* inner while */
    /*
      Get the current time. if the time is 2:00:00 then
      see if it is a different day if it is then create
      a new dbTree.txt
    */
    start_time = time(0);
    checkTime = (struct tm *)localtime(&start_time);
    if((checkTime->tm_hour == 2) && (checkTime->tm_mday != today))
      {
	today = checkTime->tm_mday;
	tisSysLog(V_STATUS,0,"Preparing to re-build dbTree.txt");
#ifndef WIN32
	unlink("/tis4000db/dbTree.txt");
	system("/tis4000/tasks/dbTree &");
#else
	unlink("c:\\tis4000db\\dbTree.txt");
	system("c:\\tis4000\\tasks\\dbTree &");
#endif WIN32
      }
    if(acceptIOC == ACCEPT_IOCS)
      dbcPollIOCs();
    for(i = 0;i < numToConnect;i++)
      dbcConnectToHost(&hostInfo[i]);			/* connect to Hosts */
    if(acceptIOC == ACCEPT_IOCS)
      {
	for(i = 0;i < DBC_MAX_HOSTS;i++)
	  {
	    if(hostInfo[i].ioc_socket != -1)
	      dbcSendVerifyInfo(&hostInfo[i]);			/* send point verification information */
	  }
      }
  }/* forever loop */
}
void	refreshFds(fd_set *fds)
{
  DBC_IOCINFO_PTR	iocinfo;
  int i;
  bzero((char *)fds,sizeof(fd_set));
  if(DBC_UDPSocket != -1)
    FD_SET(DBC_UDPSocket,fds);
  if(listenSocket != -1)
    FD_SET(listenSocket,fds);
  /* refresh IOC fds */
  iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
  while(iocinfo)
    {
      if(iocinfo->ioc_socket != -1)
	FD_SET(iocinfo->ioc_socket,fds);
      iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
    }
  /* refresh HOST fds too */
  for(i = 0;i < DBC_MAX_HOSTS; i++)
    {
      if(hostInfo[i].ioc_socket != -1)
	FD_SET(hostInfo[i].ioc_socket,fds);
      /*
	GCT 11-JUl-1997 Refresh all the utilIocs here too
      */
      if(utilIoc[i].ioc_socket != -1)
	FD_SET(utilIoc[i].ioc_socket,fds);
    }
}
/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			11-Jan-1993				      */
/*									      */
/*	Function Name:		DBC_DatabaseInit			      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:						      */
/*									      */
/*	Output Parameters:						      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
DBC_ERROR_CODES	DBC_DatabaseInit(name)
char	*name;
{
  int	i;
  DB_InitDatabase(name);						/* init database system */
  DB_MapDatabase(DBC_DefaultDatabase);				/* map to database path */
  if((DBC_PvmapTable = DB_MapTable(DBC_PVMAPFILE)) == 0)		/* PVMAP file */
    return(DBC_PVMAPERR);
  if((DBC_IOAddrTable = DB_MapTable(DBC_IOADDRFILE)) == 0)	/* I/O address file */
    return(DBC_IOADDRERR);
  if((DBC_IOCListTable = DB_MapTable(DBC_IOCLISTFILE)) == 0)	/* I/O list file */
    return(DBC_IOCLISTERR);
  if((calcOpsTable = DB_MapTable(DBC_CALCOPFILE)) == 0)		/* Calc block operands file */
    return(DBC_CALCOPERR);
  if((pipeVarsTable = DB_MapTable(DBC_PIPEVARSFILE)) == 0)	/* Pipe vars file */
    return(DBC_PIPEVARSERR);
  if((pipeCmdsTable = DB_MapTable(DBC_PIPECMDSFILE)) == 0)	/* Pipe commands file */
    return(DBC_PIPECMDSERR);
  if((tankVarsTable = DB_MapTable(DBC_TANKVARSFILE)) == 0)	/* Tank vars file */
    return(DBC_TANKVARSERR);
  if((tankCmdsTable = DB_MapTable(DBC_TANKCMDSFILE)) == 0)	/* Tank commands file */
    return(DBC_TANKCMDSERR);
  if((tankPrbsTable = DB_MapTable(DBC_TANKPRBSFILE)) == 0)	/* Tank probes file */
    return(DBC_TANKPRBSERR);
  if((tankVlvsTable = DB_MapTable(DBC_TANKVLVSFILE)) == 0)	/* Tank valves file */
    return(DBC_TANKVLVSERR);
  if((valveVarsTable = DB_MapTable(DBC_VLVVARSFILE)) == 0)		/* valve vars file */
    return(DBC_VLVVARSERR);
  if((pumpVarsTable = DB_MapTable(DBC_PUMPVARSFILE)) == 0)	/* Pump vars file */
    return(DBC_PUMPVARSERR);
  if((potVarsTable = DB_MapTable(DBC_POTVARSFILE)) == 0)		/* point-of-transfer vars file */
    return(DBC_POTVARSERR);
  if((selLinkTable = DB_MapTable(DBC_SELLINKFILE)) == 0)		/* Selector links file */
    return(DBC_SELLINKERR);
  if((fanLinkTable = DB_MapTable(DBC_FANLINKFILE)) == 0)		/* Fanout links file */
    return(DBC_FANLINKERR);
  if((pidTable = DB_MapTable(DBC_PIDFILE)) == 0)
    return(DBC_NOTYPE);
  if((accumTable = DB_MapTable(DBC_ACCUMFILE)) == 0)
    return(DBC_NOTYPE);
  DB_UseKey(calcOpsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(pipeVarsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(pipeCmdsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(tankVarsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(tankCmdsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(tankPrbsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(tankVlvsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(valveVarsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(pumpVarsTable,FLD_CALCOP_KEY);			/* to index on NAME+OPERANDID */
  DB_UseKey(potVarsTable,FLD_CALCOP_KEY);				/* to index on NAME+OPERANDID */
  DB_UseKey(selLinkTable,FLD_CALCOP_KEY);				/* to index on NAME+OPERANDID */
  DB_UseKey(fanLinkTable,FLD_CALCOP_KEY);				/* to index on NAME+OPERANDID */
  pvmapTypeField	= DB_MapField(DBC_PvmapTable,FLD_TYPE);		/* Map point type field within PVMAP */
  pvmapNameField	= DB_MapField(DBC_PvmapTable,FLD_NAME);		/* Map point name field within PVMAP */
  pvmapDeleteFlag	= DB_MapField(DBC_PvmapTable,VI_FLD_DELETEFLAG);	/* and delete flag */
  pvmapCRCField	= DB_MapField(DBC_PvmapTable,FLD_CRC);		/* field for CRC result */
  pvmapIOCField[0] = DB_MapField(DBC_PvmapTable,FLD_IOC1DNLD);	/* for download checking */
  pvmapIOCField[1] = DB_MapField(DBC_PvmapTable,FLD_IOC2DNLD);	/* for download checking */
  DBC_IOCSyncField = DB_MapField(DBC_IOCListTable,FLD_SYNCFLAG);	/* global sync flag */
  DBC_IOCIsHost	 = DB_MapField(DBC_IOCListTable,FLD_HOSTFLAG);	/* host bit 1 = HOST */
  DBC_IOCName  	 = DB_MapField(DBC_IOCListTable,FLD_IOCNAME);	/* Name of this IOC */
  /*
    Map to IOADDR fields now!
    */
  ioaddrName	= DB_MapField(DBC_IOAddrTable,FLD_NAME);	/* used to get line,address,card & signal */
  ioaddrLine	= DB_MapField(DBC_IOAddrTable,FLD_LINE);
  ioaddrAddress	= DB_MapField(DBC_IOAddrTable,FLD_ADDRESS);
  ioaddrCard	= DB_MapField(DBC_IOAddrTable,FLD_CARD);
  ioaddrSignal	= DB_MapField(DBC_IOAddrTable,FLD_SIGNAL);
  if(dbcBuildIOCTable() != DBC_SUCCESS)				/* Build IOC list in memory */
    return(DBC_IOCTABLEERR);
  /*
    Read the ioc inihibit list file here too. This will allow/dis-allow
    communications from startup
  */
  processIocInhibitList();
  if(sdrLoad(DBC_DefaultSDR,0)<0)
    return(DBC_SDRERR);
  if(DBC_BuildXREFTable() != DBC_SUCCESS)
    return(DBC_XREFERR);
  /*
    Allocate all download record space here. This will avoid the alloc/free
    cycles in previous versions.
    */
  for(i = 0;i < DBC_WINDOW_SIZE;i++)
    downloadRecDesc[i].recPtr = (char *)malloc(maxRecSize);
  return(DBC_SUCCESS);
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			18-Jan-1993				      */
/*									      */
/*	Function Name:		DBC_NetworkInit				      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:	None.					      */
/*									      */
/*	Output Parameters:	None.					      */
/*									      */
/*	Return Values or error codes:	DBC_SUCCESS or DBC_ERROR	      */
/*									      */
/*	Modification History:	18-Jan-1993	GCT	First try	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
DBC_ERROR_CODES	dbcNetworkInit()
{
  int	optval;
  struct sockaddr_in setAddr;
  optval = 1;
  /*
    Create socket for broadcasts
    8-Jan-97 Do this only if I am interested in IOCs
    */
  if((DBC_UDPSocket = socket(AF_INET,SOCK_DGRAM,0)) < 0)
    return(DBC_BCASTERR);
  if((setsockopt(DBC_UDPSocket,SOL_SOCKET,SO_BROADCAST,(caddr_t)&optval,sizeof(optval))) < 0)
    return(DBC_BCASTOPTERR);
  memset(&setAddr,0,sizeof(setAddr));
  setAddr.sin_family	= AF_INET;
  setAddr.sin_addr.s_addr	= htonl(INADDR_ANY);
  setAddr.sin_port	= htons(DBSERVER_UDP_PORT);
  if(bind(DBC_UDPSocket, (struct sockaddr *) &setAddr, sizeof(setAddr)) < 0)
    return(DBC_BCASTBNDERR);
  if(DBC_UDPSocket > maxFd) maxFd = DBC_UDPSocket;
  FD_SET(DBC_UDPSocket,&myFds);
  /*
    Create TCP socket on which to listen for IOC messages
    */
  if((listenSocket = socket(AF_INET,SOCK_STREAM,0)) < 0)
    return(DBC_LISTENERR);
  
  if((setsockopt(listenSocket,SOL_SOCKET,SO_REUSEADDR,(caddr_t)&optval,sizeof(optval))) < 0)
    return(DBC_BCASTOPTERR);
  setAddr.sin_family	= AF_INET;
  setAddr.sin_addr.s_addr	= htonl(INADDR_ANY);
  setAddr.sin_port	= htons(DBSERVER_UDP_PORT);
#ifndef WIN32
  while(bind(listenSocket, (struct sockaddr *) &setAddr, sizeof(setAddr)) < 0)
    sleep(5);
#else
  bind(listenSocket, (struct sockaddr *) &setAddr, sizeof(setAddr));
#endif WIN32
  if(listen(listenSocket,10) < 0)
    return(DBC_NOLISTENERR);
  if(listenSocket > maxFd) maxFd = listenSocket;
  FD_SET(listenSocket,&myFds);
  return(DBC_SUCCESS);
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			30-Jan-1993				      */
/*									      */
/*	Function Name:		DBC_BuildIOCTable			      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:	None.					      */
/*									      */
/*	Output Parameters:	None.					      */
/*									      */
/*	Return Values or error codes:	DBC_SUCCESS or DBC_ERROR	      */
/*									      */
/*	Modification History:	30-Jan-1993	GCT	First try	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
DBC_ERROR_CODES	dbcBuildIOCTable()
{
  DB_TABLE_PTR	DBC_ResListTable;
  DB_FIELD_PTR	resResField,resIOC1Field,resIOC2Field,resDPAField;
  DB_RECORD_PTR	resRecord;
  char 	resResource[DBC_RESNAMESIZE+1],resIOC[2][DBC_RESNAMESIZE+1];
  int   type,i;
  struct hostent	*ent;
  int IOCAddr;
  DBC_RESINFO_PTR	temp;
  char isDPA;

  if((DBC_ResListTable = DB_MapTable(DBC_RESLISTFILE)) == 0)	/* Resource list file */
    return(DBC_RESLISTERR);

  resResField	= DB_MapField(DBC_ResListTable,FLD_RESOURCE);
  resIOC1Field	= DB_MapField(DBC_ResListTable,FLD_IOC1);
  resIOC2Field	= DB_MapField(DBC_ResListTable,FLD_IOC2);
  resDPAField   = DB_MapField(DBC_ResListTable,"DPA");
  /*
    GCT 24-Jul-1997 Use the resource name as the key. This will make the list
    in alphabetical order.
  */
  DB_UseKey(DBC_ResListTable,FLD_RESOURCE);  resRecord	= DB_Top(DBC_ResListTable);
  /*
    Sequentialy read all records from RESLIST file.
    */
  while(resRecord)
    {
      DB_GetFieldValue(resRecord,resResField,resResource,&type);
      DB_GetFieldValue(resRecord,resIOC1Field,&resIOC[0][0],&type);
      DB_GetFieldValue(resRecord,resIOC2Field,&resIOC[1][0],&type);
      DB_GetFieldValue(resRecord,resDPAField,&isDPA,&type);
      if(isDPA == 1)
	{
	  resRecord = DB_SeekNext(DBC_ResListTable);
	  continue;
	}
      /*
	Add RES names to RES linked list. Resolve references to IOC
	names and put them into IOC name array.
	*/
      if((temp = dbcResSearch(resResource)) == 0)
	{
	  temp = (DBC_RESINFO_PTR)malloc(RESINFOSIZE);
	  memset(temp,0,RESINFOSIZE);
	  lstAdd((LIST *)&DBC_RESHeader,(NODE *)temp);
	  strcpy(temp->res_name,resResource);				/* store name here */
	}
      /*
	temp is now pointing to the new/found resource entry.
	*/
      for(i = 0;i < DBC_MAXIOCPERRES;i++)					/* loop through all IOCs */
	{
	  if(strlen(&resIOC[i][0]) == 0)					/* skip blank IOC names */
	    continue;
	  if(ent = gethostbyname(&resIOC[i][0]))			/* get INET address for this IOC */
	    IOCAddr = ntohl(*(long *)*ent -> h_addr_list);
	  else
	    IOCAddr = 0;						/* GCT 8-Feb-1995 IOC NOT in host file */
	  if(myAddr == IOCAddr)						/* skip if it's me ! */
	    continue;
	  if((temp -> res_iocinfo[i] = dbcIocSearch(&resIOC[i][0])) == 0)
	    {
	      temp->res_iocinfo[i] = (DBC_IOCINFO_PTR)malloc(IOCINFOSIZE);
	      memset(temp->res_iocinfo[i],0,IOCINFOSIZE);
	      lstAdd((LIST *)&DBC_IOCHeader,(NODE *)(temp->res_iocinfo[i]));
	      strcpy(&temp -> res_iocinfo[i]->ioc_name[0],&resIOC[i][0]);
	      temp -> res_iocinfo[i]->ioc_state = IOC_NONET;	/* No network connection for now */
	      temp -> res_iocinfo[i]->ioc_socket = -1;		/* Not connected yet */
	      temp -> res_iocinfo[i]->ioc_index = 0;
	      temp -> res_iocinfo[i]->ioc_host = 0;	       	/* IOC is NOT a host */
	      temp -> res_iocinfo[i]->ioc_rxCount = 0;		/* Nothing in buffer */
	      temp -> res_iocinfo[i]->ioc_rxNextChar = 0;      	/* Start of buffer */
	      temp -> res_iocinfo[i]->ioc_addr = IOCAddr;
	      temp -> res_iocinfo[i]->ioc_ackPending = 0;      	/* pointer to ACK_PENDING list */
	      temp -> res_iocinfo[i]->ioc_lastResource = 0;	/* pointer to resource for this IOC */
	      temp -> res_iocinfo[i]->ioc_newResource = -1;	/* New resource flag */
	    }
	}
      resRecord = DB_SeekNext(DBC_ResListTable);
    }
  DB_UnMapTable(DBC_ResListTable,0);
  return(DBC_SUCCESS);
}


DBC_RESINFO_PTR	dbcResSearch(char *resname)
{
  DBC_RESINFO_PTR	temp;
  temp = (DBC_RESINFO_PTR)lstFirst((LIST *)&DBC_RESHeader);
  while(temp)
    {
      if(!strcmp(temp -> res_name,resname))
	break;
      temp = (DBC_RESINFO_PTR)lstNext((NODE *)temp);
    }
  return((DBC_RESINFO_PTR)temp);
}

DBC_IOCINFO_PTR	dbcIocSearch(char *iocname)
{
  DBC_IOCINFO_PTR	temp;
  temp = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
  while(temp)
    {
      if(!strcmp(temp -> ioc_name,iocname))
	break;
      temp = (DBC_IOCINFO_PTR)lstNext((NODE *)temp);
    }
  return((DBC_IOCINFO_PTR)temp);
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			11-Feb-1993				      */
/*									      */
/*	Function Name:		DBC_HandleIOCConnect			      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:	none.					      */
/*									      */
/*	Output Parameters:	none.					      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
void	dbcHandleIOCConnect(int *newfd,int addr)
{
  int true=1;
  DBC_IOCINFO_PTR      	iocinfo;
  int index;

  iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
  while(iocinfo)
    {
      /*
	6-Jan-1997 if acceptIOC is one then accept a connection from this IOC.
	27-May-1997 Do this only if inhibitState is NOT IOC_INHIBIT_COMM
      */
      if((iocinfo -> ioc_addr == addr) && (iocinfo->inhibitState != IOC_INHIBIT_COMM))  	/* This IOC is one of mine and we are interested in IOCs */
	{
	tisSysLog(V_DEBUG,0,
			   "Accepting new connection on socket <%d> from <%s> with addr <%s>...had socket <%d>.\n",
			   *newfd,searchNetName(htonl(addr)),addrToString(addr),iocinfo->ioc_socket);
	  if(iocinfo->ioc_socket != -1)
	    {
#ifdef PRIV_AUDIT
		SSL *conn = sslSharedGetConn (iocinfo->ioc_socket);

		if (auditLog)
		    {
		    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
		    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
				   "dbClient", "Prior connection closed");
		    }
		sslSharedCloseConn (conn);
#else
	      close(iocinfo->ioc_socket);			/* close this socket...might get a new one */
#endif PRIV_AUDIT
	      FD_CLR(iocinfo->ioc_socket,&myFds);
	      iocinfo->ioc_socket	= -1;			/* clear socket */
	      iocinfo ->ioc_state	= IOC_NONET;
	      ackCleanList(iocinfo->ioc_ackPending);
	      iocinfo->ioc_ackPending = 0;
	    }
	  iocinfo -> ioc_socket = *newfd;		/* this is the socket to use */
	  setsockopt(iocinfo -> ioc_socket,IPPROTO_TCP,TCP_NODELAY,(char *)&true,sizeof(true));
	  dbSetNoBlock(iocinfo);
	  iocinfo -> ioc_state = IOC_OK;		/* OK to proceed */
	  iocinfo -> ioc_rxCount = 0;			/* declare buffer empty */
	  iocinfo -> ioc_rxNextChar = 0;
	  iocinfo -> ioc_lastRxTime = time(0);
	  iocinfo -> ioc_lastTxTime = time(0);
	  ackCleanList(iocinfo->ioc_ackPending);
	  iocinfo -> ioc_ackPending = 0;
	  iocinfo -> ioc_lastResource = 0;
	  iocinfo -> ioc_newResource = -1;
	  iocinfo->inhibitState = IOC_ALLOW_COMM;	/* allow communications GCT 27-May-1997 */
	  if(iocinfo->ioc_socket > maxFd) maxFd = iocinfo->ioc_socket;
	  FD_SET(iocinfo->ioc_socket,&myFds);
	  /*maxSocket(iocinfo->ioc_socket);*/		/* max-out this sockets buffers */
	  *newfd = -1;					/* done with this fd */
	  return;
	}
      iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
    }
/*
	see if this is connection from a utility program or dataVu. It would be if
	the address in the 'accept' function is equal to mine.
*/
  if(addr == myAddr)
    {
      /*
	GCT 11-Jul-1997 Get an empty slot in the utilIoc array for this new fd. A free
	slot contains a -1 in the socket.
      */
      for(index = 0;index < DBC_MAX_HOSTS;index++)
	{
	  if(utilIoc[index].ioc_socket == -1)
	    break;
	}
      if(index < DBC_MAX_HOSTS)
	{
	  utilIoc[index].ioc_socket = *newfd;
	  tisSysLog(V_DEBUG,0,"Accepting connection from Utility program...addr %s\n",addrToString(htonl(addr)));
	  setsockopt(utilIoc[index].ioc_socket,IPPROTO_TCP,TCP_NODELAY,(char *)&true,sizeof(true));
	  if(utilIoc[index].ioc_socket > maxFd) maxFd = utilIoc[index].ioc_socket;
	  FD_SET(utilIoc[index].ioc_socket,&myFds);
	  utilIoc[index].ioc_rxCount = 0;
	  utilIoc[index].ioc_rxNextChar = 0;
	  *newfd = -1;
	}
      else
	{
#ifdef PRIV_AUDIT
	    SSL *conn = sslSharedGetConn (*newfd);

	    if (auditLog)
		{
		GetPeerDN (conn, X509_dn, sizeof(X509_dn));
		AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			       "dbClient", "Connection closed, no space in host table");
		}
	    sslSharedCloseConn (conn);
#else
	  close(*newfd);
#endif PRIV_AUDIT
	  *newfd = -1;
	}
      return;
    }
  return;
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			18-Jan-1993				      */
/*									      */
/*	Function Name:		DBC_HandleBcastResp			      */
/*	Function Description:						      */
/*									      */
/*	Input Parameters:	none.					      */
/*									      */
/*	Output Parameters:	none.					      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	18-Jan-1993	GCT 	First try.	      */
/*				11-Mar-1993	GCT	Use dispatch table.   */	
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
void	dbcHandleBCastResp()
{
  DBC_IOCINFO_PTR		iocinfo;
  struct sockaddr_in	sockAddr;
  int		len;
  int		addr;
  unsigned short	*pmsg,msg;
  
  pmsg = (unsigned short *)&DBC_BCastMsgBuff[0];
  len = sizeof(sockAddr);
  recvfrom(DBC_UDPSocket,DBC_BCastMsgBuff,sizeof(DBC_BCastMsgBuff),0,(struct sockaddr *)&sockAddr,&len);
  msg = ntohs(*pmsg++);
  addr = ntohl(sockAddr.sin_addr.s_addr);
  /*
    30-Apr-1999 GCT

    Remove the DBC_MSG_CRCRESPONSE message type check from this code. This message
    will be delivered via TCP from now on.
  */
  if((msg != DBC_MSG_AWAKE) && (msg != DBC_MSG_WAKEUP))
    {
      if(msg != 1)
	  {
		tisSysLog(V_INFO,0,"Got unknown UDP message <%04X> from <%s>\n",msg,searchNetName(htonl(addr)));
	  }
      return;
    }
  iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
  while(iocinfo)
    {
    /*
      6-Jan-1997 if acceptIOC is zero then do not accept connection from this
      IOC.

      27-May-1997 if inhibitState is IOC_INHIBIT_COMM then ignore this broadcast
    */
    if((iocinfo -> ioc_addr == addr) && (iocinfo->inhibitState != IOC_INHIBIT_COMM))	/* This IOC is one of mine */
      {
       	tisSysLog(V_DEBUG,0,"Got UDP message <%s> from IOC <%s>\n",DBC_WhatMessage[msg-64],iocinfo->ioc_name);
       	switch(msg)
	  {
	  case DBC_MSG_WAKEUP:			/* wakeup call from other host */
	    break;
	  case DBC_MSG_AWAKE:
	    DBC_ProcessAwake(iocinfo,&sockAddr);
	    break;
	    /*
	      30-Apr-1999 GCT
	      
	      Remove the DBC_MSG_CRCRESPONSE message type check from this code. This message
	      will be delivered via TCP from now on.
	    */
	  }
	break;
      }
    iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
  }
return;
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			11-Mar-1993				      */
/*									      */
/*	Function Name:		DBC_ProcessAwake			      */
/*	Function Description:						      */
/*		When an IOC responds to his/her wake-up call from	      */
/*	dbClient(me),I must translate the address given in the message	      */
/*	into an IOC name and see if the name is in the IOC table. If it	      */
/*	is then I will 'accept' the message and enter the new socket fd	      */
/*	into the IOCINFO structure.					      */
/*									      */
/*									      */
/*	Input Parameters:	iocinfo - pointer to DBC_IOCINFO structure    */
/*					for the IOC on the FD which recieved  */
/*					the message.			      */
/*									      */
/*	Output Parameters:	none.					      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	11-Mar-1993	GCT	First try.	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
void	DBC_ProcessAwake(iocinfo,psockAddr)
DBC_IOCINFO_PTR	iocinfo;
struct sockaddr_in	*psockAddr;
{
unsigned short	msg;
if(iocinfo->ioc_addr == 0)
	return;
psockAddr->sin_family		= AF_INET;
psockAddr->sin_addr.s_addr	= htonl(iocinfo->ioc_addr);
msg = htons(DBC_MSG_WAKEUP);
sendto(DBC_UDPSocket,(char *)&msg,2,0,(struct sockaddr *)psockAddr,sizeof(struct sockaddr_in));
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			12-Feb-1993				      */
/*									      */
/*	Function Name:		DBC_HandleIOCMessage			      */
/*	Function Description:						      */
/*									      */
/*	This function will retrieve the message from the network stream	      */
/*	and pass it off to the appropriate function for further processing.   */
/*	This will be done via a dispatch table to avoid the possiblity of     */
/*	a large 'switch' statement.					      */
/*									      */
/*	Input Parameters:	iocinfo - pointer to DBC_IOCINFO structure    */
/*					for the IOC on the FD which recieved  */
/*					the message.			      */
/*									      */
/*	Output Parameters:	none.					      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	15-Feb-1993	GCT	First try.	      */
/*			       	14-Sep-1994 GCT	Read bytes into ioc's buffer */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
void	dbcHandleIOCMsgs(iocinfo)
DBC_IOCINFO_PTR	iocinfo;
{
unsigned short	msgType,type;				/* only read message type here */
int status;
short mSize;
int i;
DBC_FWDHDR fwdHdr;

status = readFD(iocinfo);					/* drain the FD */
if(status > 0)
  {
    while(mSize = haveCompleteMsg(iocinfo))
      {						       			/* if complete message is in buffer */
	/*
	  If this message is a field update (or whole record update) message then
	  forward to all hosts I am connected to (maxConnected)
	*/
	readTcp(iocinfo,(char *)&msgType,2);					/* get the message type */
	type = ntohs(msgType);
	if(type == DBC_MSG_DBMODREQ)
	  {
	    msgType = htons(DBC_MSG_FWDDBMOD);
	    strcpy(&fwdHdr.fwdName[0],iocinfo->ioc_name);
	    fwdHdr.fwdSize = htons(mSize);
	    for(i = 0;i < DBC_MAX_HOSTS;i++)
	      {
/*
   GCT 22-Mar-1997 from Vegas

   If this host can talk to IOCs/RTUs (has -r ) AND he has a partner, check the partner
   name with all the hosts this host is connected to. If there is a match
   then do NOT forward the message.
*/
			if((acceptIOC == ACCEPT_IOCS) && !strcmp(partnerName,&hostInfo[i].ioc_name[0]))
			  continue;
			if((hostInfo[i].ioc_socket != -1) && (mSize > 0))
			  {
				writeTcp(&hostInfo[i],(char *)&msgType,sizeof(msgType));
				writeTcp(&hostInfo[i],(char *)&fwdHdr,DBC_FWDHDRSIZE);
				writeTcp(&hostInfo[i],&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar],mSize);
				tisSysLog(V_DEBUG,0,
						 "Forwarding <%s> from IOC <%s> to <%s>\n",
						 DBC_WhatMessage[type - 64],iocinfo->ioc_name,hostInfo[i].ioc_name);
			  }
	      }
	  }
		if(type == DBC_MSG_BOOMERANG)
		  {
			tisSysLog(V_DEBUG,0,"Got     BOOMERANG from from IOC <%s>\n",iocinfo->ioc_name);
		  }
		else
		  {
			tisSysLog(V_STATUS,0,"Received <%s> from IOC <%s>\n",DBC_WhatMessage[type - 64],iocinfo->ioc_name);
			iocinfo -> ioc_msgtype = type;
			if(DBC_Dispatch[type])
			  status = (*DBC_Dispatch[type])(iocinfo);	/* call processing function */
		  }
      }
  }
return;
}
void	dbcHandleHostMsgs(DBC_IOCINFO_PTR iocinfo)
{
unsigned short	msgType,type;				/* only read message type here */
int    	status;
short mSize;
status = readFD(iocinfo);					/* drain the FD */
if(status > 0)
  {
    while(mSize = haveCompleteMsg(iocinfo))
      {						       			/* if complete message is in buffer */
	readTcp(iocinfo,(char *)&msgType,2);					/* get the message type */
	type = ntohs(msgType);
	if(type == DBC_MSG_BOOMERANG)
	  {
	    tisSysLog(V_DEBUG,0,"Got     BOOMERANG from from HOST <%s>\n",iocinfo->ioc_name);
	  }
	else
	  {
	    tisSysLog(V_STATUS,0,"Received <%s> from HOST <%s>\n",DBC_WhatMessage[type - 64],iocinfo->ioc_name);
	    iocinfo -> ioc_msgtype = type;
	    status = (*DBC_Dispatch[type])(iocinfo);	/* call processing function */
	  }
      }
  }
return;
}

void	dbcHandleUtilMsgs(DBC_IOCINFO_PTR info)
{
unsigned short	msgType,type;				/* only read message type here */
int status;
short mSize;
time_t journal_time;
VERIFY_INFO verify_info;
char name[PVNAME_SZ+1];
int iocAddr;
DBC_IOCINFO_PTR iocinfo;
DBC_RESETREQ reset;
DBC_OLDELETE del;

status = readFD(info);						/* read the whole message */
if(status > 0)
  {
    while(mSize = haveCompleteMsg(info))
      {
	readTcp(info,(char *)&msgType,2);			/* get type from buffer */
	type = ntohs(msgType);
	tisSysLog(V_STATUS,0,"Recieved message from Utility program...%d on socket %d.\n",type,info->ioc_socket);
/*
	Handle messages/commands from an external utility. This could be a small
	program for toggling the verbose flag or dataVu with a point modification
	notice. The message type should be defined in the 'redmsgs.h' include file
	and a cooresponding 'case' entry should be made here and in 'haveCompleteMsg'.
*/
	switch(type)
	  {
	  case UTIL_MSG_VERIFYNOW:			/* get the point name for verification */
	    readTcp(info,name,PVNAME_SZ);
	    DB_GetVerifyInfo(DBC_PvmapTable,name,&verify_info,0);
	    DBC_SendToAllHosts(&verify_info,VI_MSG_IHAVENEWREC);
	    break;
	  case UTIL_MSG_JOURNAL_ON:
	    if(dbLog == 0)
	      {
#ifndef WIN32
		dbLog = fopen("/tis4000db/dbaseLog.log","w");
#else
		dbLog = fopen("c:\\tis4000db\\dbaseLog.log","w");
#endif WIN32
		journal_time = time(0);
		tisSysLog(V_INFO,0,"Journaling begun at: %s, to file /tis4000db/dbaseLog.log",ctime(&journal_time));
	      }
	    break;
	  case UTIL_MSG_JOURNAL_OFF:
	    journal_time = time(0);
	    tisSysLog(V_INFO,0,"Journaling ended at: %s",ctime(&journal_time));
	    fclose(dbLog);
	    break;
	  case UTIL_MSG_BYEBYE:
#ifdef PRIV_AUDIT
	    {
	    SSL *conn = sslSharedGetConn (info->ioc_socket);

	    if (auditLog)
		{
		GetPeerDN (conn, X509_dn, sizeof(X509_dn));
		AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			       "dbClient", "Journal connection closed");
		}
	    sslSharedCloseConn (conn);
	    }
#else
	    close(info->ioc_socket);
#endif PRIV_AUDIT
	    FD_CLR(info->ioc_socket,&myFds);
	    for(status=0;status < DBC_MAX_HOSTS;status++)
	      if(utilIoc[status].ioc_socket == info->ioc_socket) break;
	    info->ioc_socket = -1;
	    info->ioc_rxCount = 0;
	    info->ioc_rxNextChar = 0;
	    break;
	    /*
	      4-May-1999 GCT

	      The following code is here so that dataVu can send these messages
	      here via TCP instead of directly to the IOC/RTU via UDP.
	    */
	  case UTIL_MSG_RESETNOTIFY:
	    /*
	      get IOC/RTU inet address from message and loop through the IOCLIST looking
	      for a match. If one is found,send reset message to that IOC/RTU. 
	    */
	    readTcp(info,(char *)&iocAddr,sizeof(iocAddr));
	    iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
	    while(iocinfo)
	      {
		if((iocinfo->ioc_addr == iocAddr) && (iocinfo->inhibitState != IOC_INHIBIT_COMM))	/* This IOC is one of mine */
		  {
		    msgType = htons(DBC_MSG_RESETNOTIFY);
		    writeTcp(iocinfo,(char *)&msgType,sizeof(msgType));		/* send message type */
		    reset.originatorFd = htonl(info->ioc_socket);
		    writeTcp(iocinfo,(char *)&reset,sizeof(DBC_RESETREQ));	/* and dataVu FD */
		    break;
		  }
		iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
	      }
	    break;
	  case UTIL_MSG_OLDNOTIFY:
	    /*
	      get IOC/RTU inet address from message and loop through the IOCLIST looking
	      for a match. If one is found,send Online addition message to that IOC/RTU. 
	    */
	    readTcp(info,(char *)&iocAddr,sizeof(iocAddr));
	    iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
	    while(iocinfo)
	      {
		if((iocinfo->ioc_addr == iocAddr) && (iocinfo->inhibitState != IOC_INHIBIT_COMM))	/* This IOC is one of mine */
		  {
		    msgType = htons(DBC_MSG_OLDNOTIFY);
		    writeTcp(iocinfo,(char *)&msgType,sizeof(msgType));		/* send message type */
		    break;
		  }
		iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
	      }
	    break;
	  case UTIL_MSG_OLDELETE:
	    /*
	      get IOC/RTU inet address from message and loop through the IOCLIST looking
	      for a match. If one is found,send Online deletion message to that IOC/RTU. 
	    */
	    readTcp(info,name,PVNAME_SZ);		/* get the point name to delete */
	    readTcp(info,(char *)&iocAddr,sizeof(iocAddr));
	    strncpy(del.nameToDelete,name,PVNAME_SZ);	/* put name in message */
	    del.originatorFd = htonl(info->ioc_socket);	/* and my dataVu FD */
	    iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
	    while(iocinfo)
	      {
		if((iocinfo->ioc_addr == iocAddr) && (iocinfo->inhibitState != IOC_INHIBIT_COMM))	/* This IOC is one of mine */
		  {
		    msgType = htons(DBC_MSG_OLDELETE);
		    writeTcp(iocinfo,(char *)&msgType,sizeof(msgType));
		    writeTcp(iocinfo,(char *)&del,sizeof(DBC_OLDELETE));
		    break;
		  }
		iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
	      }
	    break;
	  }
      }
  }
  else
  {
    /* error or zero length read from port */
    tcpError(info);
  }
return;
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			12-Feb-1993				      */
/*									      */
/*	Function Name:		DBC_InvalidMessage			      */
/*	Function Description:						      */
/*									      */
/*	Print error message if called. This is to handle messages we should   */
/*	NOT be getting from the IOC.					      */
/*									      */
/*	Input Parameters:	msg	pointer to message.		      */
/*				iocinfo	pointer to DBC_IOCINFO structure for  */
/*					this IOC.			      */
/*									      */
/*	Output Parameters:	none.					      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	15-Feb-1993	GCT	First try.	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
DBC_ERROR_CODES	DBC_InvalidMessage(iocinfo)
DBC_IOCINFO_PTR	iocinfo;
{
tisSysLog(V_INFO,0,"DBC - Invalid message <%d> from IOC <%s>\n",iocinfo->ioc_msgtype,iocinfo->ioc_name);
return(DBC_SUCCESS);
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			12-Feb-1993				      */
/*									      */
/*	Function Name:		DBC_IOCPolling				      */
/*	Function Description:						      */
/*									      */
/*	Handle timer events from 'fdmgr_pend_event'			      */
/*									      */
/*	Input Parameters:	param -	pointer to parameter.		      */
/*									      */
/*	Output Parameters:	none.					      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	16-Feb-1993	GCT	First try.	      */
/*				05-May-1993	GCT	Call tcpError     */
/*							when timeouts occur   */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
void	dbcPollIOCs()
{
DBC_IOCINFO_PTR	iocinfo;
#ifndef DONT_USE_UDP
struct	sockaddr_in	iocaddr;
#endif DONT_USE_UDP
short	wakeup;
static DBC_IOCINFO_PTR	last_kick;
DBC_IOCINFO_PTR this_kick = 0;

wakeup = htons(DBC_MSG_WAKEUP);

/*
	Traverse the IOC list. If any IOC has not been connected then
	attempt to do so. Also, check for last message timeout. If any timeout
	then drop socket and attempt to re-connect.
*/
iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
#ifndef WIN32
tisSysLog(V_DEBUG,0,"\n");
#endif WIN32
while(iocinfo)
  {
    if(iocinfo->ioc_addr == 0)				/* IOC was not found in hosts file */
      {
	iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
	continue;
      }
    tisSysLog(V_DEBUG,0,"Current state for IOC <%-*.*s> on socket %d is <%s>.\n",
		     DBC_IOCNAMESIZE,DBC_IOCNAMESIZE,iocinfo->ioc_name,iocinfo->ioc_socket,state_string[iocinfo->ioc_state]);

    /*
      GCT 27-May-1997 Also check inhibit flag. Attempt to communicate only if flag NOT IOC_INHIBIT_COMM
    */
    if(iocinfo->ioc_socket == -1)
      {
	if (iocinfo->inhibitState == IOC_ALLOW_COMM)
	  {
	    if ((last_kick == NULL) && (this_kick == NULL))
	      {
		this_kick = iocinfo;
		last_kick = iocinfo;
		tisSysLog (V_INFO, 0,
			     "Attempting to communicate with IOC <%s>.\n",
			     searchNetName (htonl(iocinfo->ioc_addr)));
#ifdef DONT_USE_UDP
		kick_TCP(iocinfo,8592);
#else
		iocaddr.sin_family	= AF_INET;
		iocaddr.sin_addr.s_addr	= htonl(iocinfo->ioc_addr);
		iocaddr.sin_port	= htons(DBSERVER_UDP_PORT);
		sendto(DBC_UDPSocket,(char *)&wakeup,2,0,(struct sockaddr *)&iocaddr,sizeof(struct sockaddr_in));
#endif DONT_USE_UDP
	      }
	  }
      }
    else if((time(0) - iocinfo->ioc_lastRxTime) > DBC_Cardiac )
      {
	tisSysLog(V_INFO,0,
			"Lost (Timed out) communication link with <%-*.*s>...closing socket <%d>.\n",
			DBC_IOCNAMESIZE,DBC_IOCNAMESIZE,iocinfo->ioc_name,iocinfo->ioc_socket);
#ifdef PRIV_AUDIT
	{
	SSL *conn = sslSharedGetConn (iocinfo->ioc_socket);

	if (auditLog)
	    {
	    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
	    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			   "dbClient", "Connection closed due to activity timeout");
	    }
	sslSharedCloseConn (conn);
	}
#else
	close(iocinfo->ioc_socket);			/* close this socket...might get a new one */
#endif PRIV_AUDIT
	FD_CLR(iocinfo->ioc_socket,&myFds);
	iocinfo->ioc_socket	= -1;			/* clear socket */
	iocinfo->ioc_state	= IOC_NONET;
	iocinfo -> ioc_rxCount = 0;
	iocinfo -> ioc_rxNextChar = 0;
        iocinfo->dbt.dbtIx = 0;			/* Initialize dbtIx to 0 when iocinfo is initialized */
	ackCleanList(iocinfo->ioc_ackPending);
	iocinfo->ioc_ackPending = 0;
      }
    else if((iocinfo->ioc_state == IOC_OK) && (iocinfo -> ioc_socket != -1))
      {
	if((time(0) - iocinfo->ioc_lastTxTime) > DBC_HEART_BEAT)
	  {
	    sendBoomerang(iocinfo);
	  }
      }
    if ((last_kick == iocinfo) && (this_kick == NULL))
      {
	last_kick = NULL;
      }

  iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
  }
}
/*
  17-Jan-97 New & Improved polling scheme

  If this host was invoked with a "-r" command line parameter then it will
  send the VERIFY_INFO information to all the hosts it is connected to.

  If this host did NOT have the "-r" command line parameter then it
  will only listen for VERIFY_INFO information.

  If the timestamp on this host is less than the one received from the
  other host,the host without the "-r" will request the newest point
  from the host which had the latest & greatest point.
*/
void dbcSendVerifyInfo(DBC_IOCINFO_PTR hostinfo)
{
  static	VERIFY_INFO_PTR	info_msg,verify_info = 0;
  VERIFY_HDR   	verify_hdr;
  char	       	viName[PVNAME_SZ+1];
  DB_RECORD_PTR   rec;
  DB_FIELD_PTR    viNameField;
  int  ftype;
  short	firstVerify;

  firstVerify = 0;
  viNameField = DB_MapField(DBC_PvmapTable,FLD_NAME);
  if((acceptIOC == ACCEPT_IOCS))
    {
      if(hostinfo->ioc_socket != -1)
	{
	  if(!firstVerify)
	    {
	      firstVerify = 1;
	      if(verify_info == 0)
		{
		  verify_info = (VERIFY_INFO_PTR)calloc(VERIFY_COUNT,VERIFY_INFO_SIZE);
		  info_msg = verify_info;
		}
	      else
		verify_info = info_msg;				/* reset pointer to begining */
	      if(strlen(&hostinfo->ioc_lastpt[0]))
		{
		  rec = DB_FindVerifyInfo(DBC_PvmapTable,&hostinfo->ioc_lastpt[0]);
		  rec = DB_NextVerifyInfo(DBC_PvmapTable);
		}
	      else
		rec = DB_FirstVerifyInfo(DBC_PvmapTable);      	/* start at begining */
	      hostinfo->ioc_numToSend = 0;
	      while(rec && hostinfo->ioc_numToSend < VERIFY_COUNT)
		{
		  DB_GetFieldValue(rec,viNameField,viName,&ftype);
		  if(DB_GetVerifyInfo(DBC_PvmapTable,0,verify_info,0) == 0)
		    break;
/* 		  VERBOSE(V_DEBUG)(LOG_ALERT|LOG_LOCAL1, */
/* 				   "Adding <%-*.*s> to VI_MSG_VERIFYREQUEST message %d\n", */
/* 				   PVNAME_SZ,PVNAME_SZ,viName,hostinfo->ioc_numToSend); */
		  verify_info++;
		  hostinfo->ioc_numToSend++;
		  strcpy(&hostinfo->ioc_lastpt[0],viName);
		  rec = DB_NextVerifyInfo(DBC_PvmapTable);
		}
	    }
	  /*
	    Send header and message to all HOSTS I know about. Don't send anything if
	    numToSend is zero. This signifies the start of the second pass over Pvmap.
	    */
	  if(hostinfo->ioc_numToSend > 0)
	    {
		  short temp;
	      verify_hdr.viMsgType = htons(VI_MSG_VERIFYREQUEST);
		  temp = (short)hostinfo->ioc_numToSend;
	      verify_hdr.viCount = htons(temp);
	      writeTcp(hostinfo,(char *)&verify_hdr,VERIFY_HDR_SIZE);
	      writeTcp(hostinfo,(char *)info_msg,(hostinfo->ioc_numToSend * VERIFY_INFO_SIZE));
	      tisSysLog(V_DEBUG,0,"Sending inter-host information to %s",hostinfo->ioc_name);
	    }
	  else
	    {
	      hostinfo->ioc_lastpt[0] = 0;
	    }
	}
    }
}
/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			15-Mar-1993				      */
/*									      */
/*	Function Name:		DBC_BuildXREFTable			      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:						      */
/*									      */
/*	Output Parameters:						      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	15-Mar-1993	GCT	First try	      */
/*				05-May-1993	GCT	Add code to support   */
/*							new fields in PDBXREF */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
int	DBC_BuildXREFTable()
{
  struct recTypDes	*DBC_RecTypDes;
  struct fldDes		*DBC_FldDes;
  int			type_index,field_index,field_type,errCount = 0;
  DBC_XREFTABLE		*temp;
  DB_TABLE_PTR		DBC_XrefTable;
  DB_FIELD_PTR		xrefNameField,xrefTableField,xrefTrueField,xrefOpIDField;
  DB_FIELD_PTR		xrefReferenceField,xrefConstantField,xrefHardwareField;
  DB_RECORD_PTR		xrefRecord;
  char			key[40],xrefFieldName[11],xrefTableName[25];
  char			xrefRefName[11],xrefConstName[11];
  char			*typeString,i;
  DBC_XREFHDR_PTR		tempList;
  
  if((DBC_XrefTable = DB_MapTable(DBC_XREFFILE)) == 0)		/* Cross-reference */
    return(DBC_XREFERR);
  xrefNameField	= DB_MapField(DBC_XrefTable,FLD_FIELD_NAME);	/* Map FIELD_NAME field within XREF table */
  xrefTableField	= DB_MapField(DBC_XrefTable,FLD_TABLE);		/* Map TABLE field within XREF table */
  xrefTrueField	= DB_MapField(DBC_XrefTable,FLD_TRUE_STATE);	/* Map the default TRUE state field */
  xrefOpIDField	= DB_MapField(DBC_XrefTable,FLD_CALCOP);	/* Map the calc block operand field */
  xrefReferenceField = DB_MapField(DBC_XrefTable,FLD_REFFIELD);	/* for new reference scheme */
  xrefConstantField = DB_MapField(DBC_XrefTable,FLD_CONSTFIELD);	/* 30-Apr-1993 */
  xrefHardwareField = DB_MapField(DBC_XrefTable,FLD_HARDFIELD);
  DBC_XREFTables = (DBC_XREFHDR_PTR)malloc(XREFHDRSIZE * dbRecType->number);
  tempList = DBC_XREFTables;
  tempList++;				/* start with the one'th element */
  maxRecSize = -1;
  for(type_index = 1;type_index < dbRecType->number;type_index++)
    {
      lstInit(&tempList->xrefHdrList);					/* initialize list for this point type */
      typeString = GET_PRECTYPE(type_index);
      for(i=0;i<DBC_MAX_TABLES;i++)
	{
	  tempList->xrefHdrTables[i] = 0;
	  tempList->xrefRecCount[i] = DBC_PVMAP_CNT;		/* init min recs per entry */
	  tempList->xrefSeqNumber[i] = type_index * (i + 1);	/* init seq numbers for inter-host record transfers */
	}
      if( (type_index == VU_AI) ||
	  (type_index == VU_AO) ||
	  (type_index == VU_BI) ||
	  (type_index == VU_BO) ||
	  (type_index == VU_MBBI) ||
	  (type_index == VU_MBBO) ||
	  (type_index == VU_STRINGIN) ||
	  (type_index == VU_STRINGOUT) ||
	  (type_index == VU_WAVEFORM) ||
	  (type_index == VU_PTO) ||
	  (type_index == VU_TD) ||
	  (type_index == VU_PAT) ||
	  (type_index == VU_DDC) )
	{
	  tempList->xrefHdrTables[0] = DBC_IOAddrTable;
	  if(!strcmp(typeString,"pto"))
	    tempList->xrefRecCount[0] = DBC_PVMAP_CNT * 2;	/* two entries per PTO point */
	  if(!strcmp(typeString,"pat"))
	    tempList->xrefRecCount[0] = DBC_PVMAP_CNT * 4;	/* four entries per PAT point */
	}
      DBC_RecTypDes = GET_PRECTYPDES(type_index);		/* point to type descriptor in SDR */
      field_index = 0;
      /*
	Find the maximum record size. This will be used for a one-time allocation of buffer space
	for download/field-mod record space.
	
	GCT 10-Feb-1995
	*/
      if(DBC_RecTypDes -> rec_size > maxRecSize)
	maxRecSize = DBC_RecTypDes -> rec_size;
#if (defined(MIPSEL) || defined(AXP) || defined(SUN86))
      temp = (DBC_XREFTABLE_PTR)malloc(XREFSIZE);
      lstAdd(&tempList->xrefHdrList,temp);
#endif
      while(DBC_FldDes = GET_PFLDDES(DBC_RecTypDes,field_index))	/* loop to get all desc's */
	{
	  /*
	    Do prompted fields only unless its the point name.
	    */
	  if((DBC_FldDes -> promptflag) || !strcmp(DBC_FldDes->fldname,"NAME")) /* position at XREF record */
	    {
	      sprintf(key,"%-15.15s%-4.4s\0",typeString,DBC_FldDes->fldname);	/* make XREF key */
	      if((xrefRecord = DB_Seek(DBC_XrefTable,key)) != 0)
		{
		  DB_GetFieldValue(xrefRecord,xrefNameField,xrefFieldName,&field_type); /* get OV field name */
		  DB_GetFieldValue(xrefRecord,xrefTableField,xrefTableName,&field_type); /* point type table name */
		  if(!strlen(xrefFieldName))		/* skip non-existant point types */
		    break;
		  if(!strlen(xrefTableName))		/* skip to next field if no table */
		    {
		      field_index++;
		      continue;
		    }
		  DB_GetFieldValue(xrefRecord,xrefReferenceField,xrefRefName,&field_type);
		  DB_GetFieldValue(xrefRecord,xrefConstantField,xrefConstName,&field_type);
		  temp = (DBC_XREFTABLE_PTR)malloc(XREFSIZE);
		  lstAdd(&tempList->xrefHdrList,(struct node *)temp);
		  temp -> xrefFieldDesc = DBC_FldDes;
		  DB_GetFieldValue(xrefRecord,xrefTrueField,&temp->xrefTrue,&field_type);
		  DB_GetFieldValue(xrefRecord,xrefOpIDField,temp->xrefOpId,&field_type);
		  if((temp->xrefTable = DB_MapTable(xrefTableName)) != 0)	/* map to type table (e.g. AI.DBF) */
		    {
		      if((temp->xrefField = DB_MapField(temp->xrefTable,xrefFieldName)) != 0)
			{
			  temp->xrefRefField = DB_MapField(temp->xrefTable,xrefRefName);
			  temp->xrefConstField = DB_MapField(temp->xrefTable,xrefConstName);
			  DB_GetFieldValue(xrefRecord,xrefHardwareField,&temp->xrefHardware,&field_type);
			  /*
			    Use the prompt flag to hold the decimal point location for the CRC function.
			    */
			  if((DBC_FldDes->field_type == DBF_FLOAT) || (DBC_FldDes->field_type == DBF_DOUBLE))
			    DBC_FldDes->promptflag = temp->xrefField->field_scale;
			  if((i = xrefFindIndex(tempList,temp->xrefTable)) >= 0)
			    {
			      tempList->xrefHdrTables[i] = temp -> xrefTable;
			      /*
				initialize min rec count/entry array. This is done based on which file is currently
				being processed. 15-Jul-1994 GCT.
				*/
			      if(temp->xrefTable == calcOpsTable)
				tempList->xrefRecCount[i] = DBC_CALCOPS_CNT;
			      else if(temp->xrefTable == pipeVarsTable)
				tempList->xrefRecCount[i] = DBC_PIPEVARS_CNT;
			      else if(temp->xrefTable == pipeCmdsTable)
				tempList->xrefRecCount[i] = DBC_PIPECMDS_CNT;
			      else if(temp->xrefTable == tankVarsTable)
				tempList->xrefRecCount[i] = DBC_TANKVARS_CNT;
			      else if(temp->xrefTable == tankCmdsTable)
				tempList->xrefRecCount[i] = DBC_TANKCMDS_CNT;
			      else if(temp->xrefTable == tankPrbsTable)
				tempList->xrefRecCount[i] = DBC_TANKPRBS_CNT;
			      else if(temp->xrefTable == tankVlvsTable)
				tempList->xrefRecCount[i] = DBC_TANKVLVS_CNT;
			      else if(temp->xrefTable == valveVarsTable)
				tempList->xrefRecCount[i] = DBC_VLVVARS_CNT;
			      else if(temp->xrefTable == pumpVarsTable)
				tempList->xrefRecCount[i] = DBC_PUMPVARS_CNT;
			      else if(temp->xrefTable == potVarsTable)
				tempList->xrefRecCount[i] = DBC_POTVARS_CNT;
			      else if(temp->xrefTable == selLinkTable)
				tempList->xrefRecCount[i] = DBC_SELLINK_CNT;
			      else if(temp->xrefTable == fanLinkTable)
				tempList->xrefRecCount[i] = DBC_FANLINK_CNT;
			    }
			}
		      else
				{
				  errCount++;
				  tisSysLog(V_INFO,0,"Can't map to field %s in %s\n",xrefFieldName,xrefTableName);
				}
		    }
		  else
		    {
		      errCount++;
		      tisSysLog(V_INFO,0,"Can't map to database file %s\n",xrefTableName);
		    }
		}
	      else
			{
			  errCount++;
			  tisSysLog(V_INFO,0,"No PDBXREF entry for %s\n",key);
			}
	    }
	  field_index++;
	}
      tempList++;		/* point to next LIST */
    }
  DB_UnMapTable(DBC_XrefTable,0);	/* table is no longer needed */
  return(errCount ? DBC_XREFERR : DBC_SUCCESS);
}
int	xrefFindIndex(DBC_XREFHDR_PTR xref,DB_TABLE_PTR tbl)	/* return 0-3 or -1 on error */
{
  char	i;

  for(i=0;i<DBC_MAX_TABLES;i++)
    {
      if((xref->xrefHdrTables[i] == tbl) || (xref->xrefHdrTables[i] == 0))
	return(i);
    }
  return(-1);
}
/*
  Handle error on TCP socket by closing the socket. This is OK because we will
  get another chance to connect. If the socket belongs to a HOST then decrement
  the numConnected counter.
*/
int	tcpError(DBC_IOCINFO_PTR iocinfo)
{
  if(iocinfo->ioc_socket != -1)
    {
      tisSysLog(V_INFO,0,"Closing socket <%d> to IOC <%s>\n",iocinfo->ioc_socket,iocinfo->ioc_name);
#ifdef PRIV_AUDIT
      {
	SSL *conn = sslSharedGetConn (iocinfo->ioc_socket);

	if (auditLog)
	    {
	    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
	    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			   "dbClient", "Connection closed due to TCP error");
	    }
        FD_CLR(iocinfo->ioc_socket,&myFds);
        iocinfo->ioc_socket	= -1;	/* clear socket now since... */
	sslSharedCloseConn (conn);	/* this can cause callbacks! */
      }
#else
      close(iocinfo->ioc_socket);			/* close this socket...might get a new one */
      FD_CLR((unsigned)iocinfo->ioc_socket,&myFds);
      iocinfo->ioc_socket	= -1;			/* clear socket */
#endif PRIV_AUDIT
      iocinfo ->ioc_state	= IOC_NONET;
      iocinfo->ioc_rxCount = 0;
      iocinfo->ioc_rxNextChar = 0;
      iocinfo->dbt.dbtIx = 0;			/* Initialize dbtIx to 0 when iocinfo is initialized */
      ackCleanList(iocinfo->ioc_ackPending);
      iocinfo->ioc_ackPending = 0;
      if(iocinfo->ioc_host)
	numConnected--;
    }
  return(-1);
}

/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			25-Mar-1993				      */
/*									      */
/*	Function Name:		DBC_SayByeBye				      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:						      */
/*									      */
/*	Output Parameters:						      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	25-Mar-1993	GCT	First time.	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
void	dbSignalHandler(int whichone)
{
#ifndef WIN32
time_t	end_time;
DBC_IOCINFO_PTR	iocinfo;
int i;
switch(whichone)
  {
  case SIGINT:
    close(DBC_UDPSocket);
    close(listenSocket);
    for(i = 0;i < DBC_MAX_HOSTS;i++)
    {
      if(hostInfo[i].ioc_socket != -1)
      {
#ifdef PRIV_AUDIT
	SSL *conn = sslSharedGetConn (hostInfo[i].ioc_socket);

	if (auditLog)
	    {
	    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
	    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			   "dbClient", "Connection closed at signal shutdown");
	    }
	sslSharedCloseConn (conn);
#else
        close(hostInfo[i].ioc_socket);          /* close host connections */
#endif PRIV_AUDIT
      }
    }
    iocinfo = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
    while(iocinfo)
      {
#ifdef PRIV_AUDIT
	SSL *conn = sslSharedGetConn (iocinfo->ioc_socket);

	if (auditLog)
	    {
	    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
	    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			   "dbClient", "Connection closed at signal shutdown");
	    }
	sslSharedCloseConn (conn);
#else
	close(iocinfo->ioc_socket);
#endif PRIV_AUDIT
	iocinfo = (DBC_IOCINFO_PTR)lstNext((NODE *)iocinfo);
      }
    end_time = time(0);
    if(dbLog) fclose(dbLog);		/* close log file if present */
    DB_UnMapDatabase(0);			/* close all dBase files from here */
    tisSysLog(V_INFO,0,"Exiting due to ^C: %s",ctime(&end_time));
    exit(1);
  case SIGUSR1:
    if(++dbVerbose > V_DEBUG) dbVerbose = V_DEBUG;
    tisSysLog(V_INFO,0,"Message level now at %s\n",msgLevel[dbVerbose]);
    if((dbVerbose == V_DEBUG) && (dbThrottleInfo.throttleStatus == INVALID))
      parseThrottleParams();		/* try some throttle parameters */
    if(acceptIOC == ACCEPT_IOCS)
      processIocInhibitList();		/* process the IOC Inhibit list...GCT 2-Sep-1997 but only if we are interested in them */
    break;
  case SIGUSR2:
    if(--dbVerbose < 0) dbVerbose = 0;
#ifndef WIN32
    syslog(LOG_ALERT|LOG_LOCAL1,"Message level now at %s\n",msgLevel[dbVerbose]);
#else
	sprintf(logmsg,"Message level now at %s\n",msgLevel[dbVerbose]);
	syslog(LOG_OPTIONSI,logmsg);
#endif WIN32
    if((dbVerbose != V_DEBUG) && (dbThrottleInfo.throttleStatus == VALID))
      dbThrottleInfo.throttleStatus = INVALID;
    break;
  case SIGPIPE:
    tisSysLog(V_STATUS,0,"DBC - Broken pipe for HOST/IOC <%s> FD <%d>\n",DBC_LastIOC->ioc_name,DBC_LastIOC->ioc_socket);
#ifndef PRIV_AUDIT
    (void)tcpError(DBC_LastIOC);
#endif PRIV_AUDIT
    break;
  }
#endif WIN32
}


/*============================================================================*/
/*									      */
/*									      */
/*	Tate Integrated Systems, LP					      */
/*									      */
/*									      */
/*	Author:			Greg Tomalesky				      */
/*	Date:			29-Mar-1993				      */
/*									      */
/*	Function Name:		writeTcp				      */
/*	Function Description:						      */
/*									      */
/*									      */
/*									      */
/*									      */
/*	Input Parameters:						      */
/*									      */
/*	Output Parameters:						      */
/*									      */
/*	Return Values or error codes:					      */
/*									      */
/*	Modification History:	29-Mar-1993	GCT	First time.	      */
/*									      */
/*									      */
/*									      */
/*									      */
/*============================================================================*/
int	writeTcp(iocinfo,buff,nbytes)
DBC_IOCINFO_PTR	iocinfo;
char			*buff;
int				nbytes;
{
  int		numSent;
  int		intSize = sizeof(int);
  int       buffSize;
#ifdef WIN32
  int localError;
#endif WIN32
  DBC_LastIOC = iocinfo;				/* save in case of broken pipe */
  if(iocinfo->ioc_socket == -1)
    return(1);					/* not really an error */
#ifdef PRIV_AUDIT
  numSent = SSL_write (sslSharedGetConn (iocinfo->ioc_socket),buff,nbytes);
#else
  numSent = send(iocinfo->ioc_socket,buff,nbytes,0);
#endif PRIV_AUDIT
  iocinfo->ioc_lastTxTime = time(0);
  if (numSent == -1)
    {
#ifdef WIN32
	  localError = WSAGetLastError();
      if (localError == WSAEWOULDBLOCK)
#else
	  if(errno == EWOULDBLOCK)
#endif
	{
#ifdef PRIV_AUDIT
	  sleep (1);
#else PRIV_AUDIT
	  getsockopt(iocinfo->ioc_socket, SOL_SOCKET, SO_SNDBUF, (char *)&buffSize, &intSize);
	  buffSize += nbytes;
	  if (setsockopt(iocinfo->ioc_socket, SOL_SOCKET, SO_SNDBUF, (char *)&buffSize, sizeof(buffSize)) == -1)
	    {
	      tisSysLog(V_INFO,0,"Setsockopt error in \"writeTcp\"  on socket %d %m",iocinfo->ioc_socket);
	      return(tcpError(iocinfo));
	    }
	  tisSysLog(V_DEBUG,0,"Socket %d tx buffer bumped by %d bytes to %d\n",iocinfo->ioc_socket,nbytes,buffSize);
#endif PRIV_AUDIT
	}
      else
	{
	  tisSysLog(V_INFO,0,"Send error in \"writeTcp\" on socket %d IOC %s %m",iocinfo->ioc_socket,iocinfo->ioc_name);
	  return(tcpError(iocinfo));
	}
      return (writeTcp(iocinfo, buff, nbytes));
    }
  else if (numSent != nbytes)
    {
      buff += numSent;
      nbytes -= numSent;
      return (writeTcp(iocinfo, buff, nbytes));
    }
  return(1);
}

char	*searchNetName(addr)
unsigned int	addr;
{
  DBC_NETNAME_PTR	net,temp;
  struct	hostent	*host;

  net = (DBC_NETNAME_PTR)lstFirst((LIST *)&DBC_NetNameList);
  while(net)
    {
      if(addr == net->net_addr)
	return(&net->net_name[0]);
      net = (DBC_NETNAME_PTR)lstNext((NODE *)net);
    }
/*
	not found then add
*/
  if((host = gethostbyaddr((char *)&addr,4,AF_INET)))
    {
      temp = (DBC_NETNAME_PTR)malloc(sizeof(DBC_NETNAME));
      temp->net_addr = addr;
      strcpy(&temp->net_name[0],host->h_name);
      lstAdd(&DBC_NetNameList,(struct node *)temp);
      return(&temp->net_name[0]);
    }
  else
    return("UNKNOWN");
}

/****************************************************************/
/*								*/
/*	MAX OUT A SOCKET'S SEND AND RECEIVE BUFFERS		*/
/*								*/
/*	The sockets that connect the Socket Creation and	*/
/*	Maintenance program with Channel Acquisition and	*/
/*	Control require a lot of buffer space, especially	*/
/*	for large systems. The maximum transmit and receive	*/
/*	buffers are implementation specific, so we will		*/
/*	bump up each buffer until it can't be bumped any	*/
/*	more. This will guarantee that we always have as	*/
/*	much buffering space as the system will let us have.	*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

void maxSocket(int fd)
{
    int		size;

	size = 32768;
	setsockopt(fd, SOL_SOCKET, SO_SNDBUF, (char *)&size, sizeof(size));

	size = 32768;
	setsockopt(fd, SOL_SOCKET, SO_RCVBUF, (char *)&size, sizeof(size));
}

/*
	Search for IOC names in the IOCLIST file which have the HOST bit set. If any are
	found, add them to the array for later processing.
*/
void	dbcInitializeHosts(DBC_IOCINFO hostinfo[],int num)
{
  struct hostent	*ent;
  int  	IOCAddr;
  int i;

  for(i = 0;i < num;i++)
    {
      if(ent = gethostbyname(hostinfo[i].ioc_name))    		/* get INET address for this IOC */
	IOCAddr = ntohl(*(long *)*ent -> h_addr_list);
      else
	IOCAddr = 0;
      if(myAddr != IOCAddr)
	{							/* skip if it's me ! */
	  tisSysLog(V_DEBUG,0,"Found HOST %s with addr %s\n",hostinfo[i].ioc_name,addrToString(IOCAddr));
	  hostinfo[i].ioc_state = IOC_NONET;				/* No network connection for now */
	  hostinfo[i].ioc_socket = -1;				/* Not connected yet */
	  hostinfo[i].ioc_host = 1;					/* IOC is a host */
	  hostinfo[i].ioc_rxCount = 0;				/* nothing in buffer yet */
	  hostinfo[i].ioc_rxNextChar = 0;			/* start of buffer */
	  hostinfo[i].ioc_addr = IOCAddr;				/* used to talk to other hosts */
  	  hostinfo[i].dbt.dbtIx = 0;			/* Initialize dbtIx to 0 when iocinfo is initialized */
	}
    }
}
/*
	This function provides the synchronization time for the inter-host comm functionality.
	this is in GMT. Naturally ULTRIX does NOT support the same functions for time conversion
	as the SUN workstations do hense the #ifdef's

	12-Sep-1994 GCT
*/
time_t	synchTime()
{
  time_t _tt_; 
  _tt_ = time(0);
#if !(defined(MIPSEL) || defined(SOLARIS) || defined(AXP) || defined(WIN32))
  return(timelocal(gmtime(&_tt_)));
#else
  return(mktime(gmtime(&_tt_)));
#endif
}
void	sendBoomerang(iocinfo)
DBC_IOCINFO_PTR	iocinfo;
{
  short boomer = htons(DBC_MSG_BOOMERANG);
  tisSysLog(V_DEBUG,0,"Sending BOOMERANG to <%s> on socket <%d>\n",iocinfo->ioc_name,iocinfo->ioc_socket);
  writeTcp(iocinfo,(char *)&boomer,2);
}

/*============================================================================*/
/*								             */
/*								             */
/*	Tate Integrated Systems, LP				            */
/*								       	      */
/*								             */
/*	Author:			Greg Tomalesky			            */
/*	Date:			14-Sep-1994			       	      */
/*								             */
/*	Function Name:		readFD				             */
/*	Function Description:					             */
/*								      	             */
/*								       	             */
/*								       	      */
/*								       	  */
/*	Input Parameters:					       	      */
/*								            */
/*	Output Parameters:					             */
/*								       	      */
/*	Return Values or error codes:				             */
/*								             */
/*	Modification History:	14-Sep-1994	GCT	First time.    	      */
/*								             */
/*								       		  */
/*	This is the only read FD function. It purpose is to fill the ioc buffer	  */
/*	with as much data as it can hold (currently DBC_NETMSG_SIZE = 2048 bytes).*/
/*	The actual amount of data this function will attempt to read is equal to  */
/*	the size of the buffer minus the amount already in there.      		  */
/*								       		  */
/*		e.g.	readsize = DBC_NETMSG_SIZE - ioc_rxCount;      		  */
/*								       		      */
/*								       	      */
/*								       	      */
/*============================================================================*/
int		readFD(iocinfo)
DBC_IOCINFO_PTR	iocinfo;
{
  int		readSize,stat;
#ifdef PRIV_AUDIT
  SSL		*conn = sslSharedGetConn (iocinfo->ioc_socket);
#endif PRIV_AUDIT

  if(iocinfo->ioc_socket == -1)
    return(ERROR);
  errno = 0;
/*
	If we are already some bytes into the buffer then slide them down to the
	begining and zero the next char pointer.
*/
  dbCtrace (READFD_IN, iocinfo);
  if(iocinfo->ioc_rxNextChar)
    {
      if(iocinfo->ioc_rxCount)
         {
           bcopy(&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar],&iocinfo->ioc_rxBuffer[0],iocinfo->ioc_rxCount);
         }
      iocinfo->ioc_rxNextChar = 0;
    }
  readSize = DBC_NETMSG_SIZE - iocinfo->ioc_rxCount;
#ifdef PRIV_AUDIT
  if ((stat = SSL_read (conn, &iocinfo->ioc_rxBuffer[iocinfo->ioc_rxCount+iocinfo->ioc_rxNextChar], readSize)) < 0)
    {
    int status = SSL_get_error (conn, stat);
    long err = ERR_get_error ();
    if (err != 0)
	VERBOSE(V_INFO)(LOG_ALERT|LOG_LOCAL1,"SSL error in \"readFD\" on socket %d IOC %s %s",
			iocinfo->ioc_socket,iocinfo->ioc_name, ERR_error_string (err, NULL));
    switch (status)
	{
	    case SSL_ERROR_NONE:
	    case SSL_ERROR_WANT_READ:
	    case SSL_ERROR_WANT_WRITE:
	    case SSL_ERROR_WANT_X509_LOOKUP:
	    case SSL_ERROR_WANT_CONNECT:
		/* benign, just want to write/read for state machine */
		return 0;
	    case SSL_ERROR_SSL:
		if (errno == EWOULDBLOCK)
		    return 0;
	    case SSL_ERROR_SYSCALL:
	    case SSL_ERROR_ZERO_RETURN:
	    default:
		VERBOSE(V_INFO)(LOG_ALERT|LOG_LOCAL1,"Read error in \"readFD\" on socket %d IOC %s %m",
				iocinfo->ioc_socket,iocinfo->ioc_name);
		return(tcpError(iocinfo));		/* GCT 2-Jul-1996 drop the whole thing on error */
	}
     }
#else
  if ((stat = recv(iocinfo->ioc_socket, &iocinfo->ioc_rxBuffer[iocinfo->ioc_rxCount+iocinfo->ioc_rxNextChar], readSize, 0)) < 0)
    {
      tisSysLog(V_INFO,0,"Read error in \"readFD\" on socket %d IOC %s %m",iocinfo->ioc_socket,iocinfo->ioc_name);
      return(tcpError(iocinfo));		/* GCT 2-Jul-1996 drop the whole thing on error */
    }
  if(stat == 0)					/* zero length read */
    {
      return(tcpError(iocinfo));
    }
#endif PRIV_AUDIT
  iocinfo->ioc_rxCount += stat;			/* number actually read into buffer */
  iocinfo->ioc_lastRxTime = time(0);		/* time of this recieve */
  dbCtrace (READFD_OUT, iocinfo);
  return(stat);
}

/*============================================================================*/
/*								              */
/*								       	      */
/*	Tate Integrated Systems, LP				       	      */
/*								       	      */
/*								       	      */
/*	Author:			Greg Tomalesky			              */
/*	Date:			25-Mar-1993			       	      */
/*								       	      */
/*	Function Name:		readTcp				              */
/*	Function Description:					       	      */
/*								       	      */
/*								       	      */
/*								       	      */
/*									      */
/*	Input Parameters:					       	      */
/*								       	      */
/*	Output Parameters:					       	      */
/*								       	      */
/*	Return Values or error codes:				              */
/*								       	      */
/*	Modification History:	25-Mar-1993	GCT	First time.           */
/*			       	14-Sep-1994 GCT new buffered I/O              */
/*								       	      */
/*								       	      */
/*								       	      */
/*								       	      */
/*============================================================================*/
int	readTcp(iocinfo,buff,nbytes)
DBC_IOCINFO_PTR	iocinfo;
char		*buff;
int		nbytes;
{
  dbCtrace (READTCP_IN, iocinfo);
  if(nbytes > iocinfo->ioc_rxCount)
    {
      tisSysLog(V_INFO,0,"Attempting to read %d bytes past buffer on %s",nbytes,iocinfo->ioc_socket);
      return(tcpError(iocinfo));		/* GCT 2-Jul-1996 drop the whole thing on error */
    }
  bcopy(&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar],buff,nbytes);
  iocinfo->ioc_rxNextChar += nbytes;
  iocinfo->ioc_rxCount -= nbytes;
  dbCtrace (READTCP_OUT, iocinfo);
  return (nbytes);
}

/*============================================================================*/
/*								       	      */
/*								       	      */
/*	Tate Integrated Systems, LP				              */
/*								       	      */
/*								       	      */
/*	Author:			Greg Tomalesky			              */
/*	Date:			14-Sep-1994			       	      */
/*								       	      */
/*	Function Name:		haveCompleteMsg			              */
/*	Function Description:					       	      */
/*								       	      */
/*								              */
/*								       	      */
/*								       	      */
/*	Input Parameters:					              */
/*								              */
/*	Output Parameters:					       	      */
/*								       	      */
/*	Return Values or error codes:				       	      */
/*								       	      */
/*	Modification History:	25-Mar-1993	GCT	First time.    	      */
/*		       		14-Sep-1994 GCT new buffered I/O       	      */
/*				13-Jan-1997 GCT now return size of complete   */
/*						message.		      */
/*								       	      */
/*								       	      */
/*								       	      */
/*								       	      */
/*============================================================================*/
short	haveCompleteMsg(DBC_IOCINFO_PTR iocinfo)
{
  short	mType,temp;
  int		mSize;
  VI_RECREPLY		recReply;
  DBC_DBMODHDR	mod;
  VERIFY_HDR		vhdr;
  DBC_SYMRESPHDR	sym;
  DBC_FWDHDR  fwd;
  DBC_DELSTATUS del;

  if(iocinfo->ioc_rxCount < sizeof(mType))
    return(0);
  bcopy(&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar],&mType,sizeof(mType));
  mSize = DBC_NETMSG_SIZE+1;
  /*
    switch on the message type. All message are considered complete if the number of bytes
    in the buffer (ioc_rxCount) is greater that or equal to the total message size PLUS
    the size of the message type.
    */
  switch(ntohs(mType))
    {
    case DBC_MSG_BOOMERANG:			/* These messages types do not have a body */
    case DBC_MSG_SYMPURGE:			/* DBC_ProcessSymPurge */
      mSize = sizeof(short);
      break;
    case DBC_MSG_OKTOUNMARK:
      mSize = sizeof(DBC_DWNLD_OK) + sizeof(mType);
      break;
    case DBC_MSG_DWNLDREQ:			/* DBC_DownloadRequest */
    case DBC_MSG_OLDREQ:			/* DBC_OnlineDnldRequest */
      mSize = sizeof(DBC_DWNLD_REQUEST) + sizeof(mType);
      break;
    case DBC_MSG_DBMODREQ:			/* DBC_ProcessDbMod */
      if(iocinfo->ioc_rxCount > (sizeof(DBC_DBMODHDR) + sizeof(mType)))
	{
	  bcopy((DBC_DBMODHDR *)&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+2],&mod,sizeof(DBC_DBMODHDR));
	  temp = mod.dbmodSize;
	  mSize = sizeof(DBC_DBMODHDR) + ntohs(temp) + sizeof(mType);
	}
      break;
    case DBC_MSG_FWDDBMOD:
      if(iocinfo->ioc_rxCount > (DBC_FWDHDRSIZE + sizeof(mType)))
	{
	  bcopy((DBC_FWDHDR *)&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+2],&fwd,sizeof(DBC_FWDHDR));
	  temp = fwd.fwdSize;
	  mSize = sizeof(DBC_FWDHDR) + ntohs(temp) + sizeof(mType);
	}
      break;
    case DBC_MSG_SYMRESPONSE:		/* DBC_ProcessSymResp */
    case DBC_MSG_NEWSYM:			/* DBC_ProcessNewSymResp */
      if(iocinfo->ioc_rxCount >= (sizeof(DBC_SYMRESPHDR) + sizeof(mType)))
	{
	  bcopy((DBC_SYMRESPHDR *)&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+2],&sym,sizeof(DBC_SYMRESPHDR));
	  temp = sym.numSymbols;
	  mSize = sizeof(DBC_SYMRESPHDR) + sizeof(mType) + (ntohs(temp) * sizeof(DBC_SYMRESPNAME));
	}
      break;
    case DBC_MSG_OLDELRESP:			/* DBC_OnlineDelResponse */
      mSize = sizeof(DBC_DELRESP) + sizeof(mType);
      break;
      /*
	30-Apr-1999 GCT

	The CRC response message is now recieved on the TCP socket so we must check
	for a complete message BEFORE we process the data stream.
      */
    case DBC_MSG_CRCRESPONSE:		/* Now get message from TCP */
      bcopy(&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+2],&temp,sizeof(short));
      mSize = sizeof(mType) + sizeof(short) + (ntohs(temp) * sizeof(DBC_CRCINFO));
      break;
      /*
	4-May-1999 GCT

	The IOC reset acknowledge message is now recieved on the TCP socket so we must check
	for a complete message BEFORE we process the data stream.
      */
    case DBC_MSG_RESETACK:
      mSize = sizeof(mType) + sizeof(DBC_RESETREQ);	/* IOC reset from dataVu via TCP */
      break;
      /*
	6-May-1999 GCT

	The delete status message is now recieved on the TCP socket so we must check
	for a complete message BEFORE we process the data stream.
      */
    case DBC_MSG_DELSTATUS:
      bcopy(&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+2],(char *)&del,sizeof(DBC_DELSTATUS));
      mSize = sizeof(mType) + sizeof(DBC_DELSTATUS) + (ntohs(del.numRefs) * PVNAME_SZ);
      break;
      /*
	Inter-host database sync
	*/
    case VI_MSG_VERIFYREQUEST:		/* DBC_ProcVerifyRequest */
    case VI_MSG_VERIFYREPLY:		/* DBC_ProcVerifyReply...viCount == 1 */
    case VI_MSG_IHAVENEWREC:		/* DBC_ProcNewRecRequest...viCount == 1 */
      if(iocinfo->ioc_rxCount > VERIFY_HDR_SIZE)
	{
	  bcopy((VERIFY_HDR_PTR)&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar],&vhdr,VERIFY_HDR_SIZE);
	  mSize = VERIFY_HDR_SIZE + ntohs(vhdr.viCount) * VERIFY_INFO_SIZE;
	}
      break;
    case VI_MSG_RECORDREQUEST:		/* DBC_ProcRecordRequest */
      mSize = RECREQUEST_SIZE;
      break;
    case VI_MSG_RECORDREPLY:		/* DBC_ProcRecordReply */
      if(iocinfo->ioc_rxCount > RECREPLY_SIZE)
	{
	  bcopy((VI_RECREPLY_PTR)&iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+2],&recReply,RECREPLY_SIZE);
	  mSize = RECREPLY_SIZE + ntohs(recReply.recRecSize);
	}
      break;
      /*
	Utility messages
	*/
    case UTIL_MSG_TOGGLEVERBOSE:			/* These messages types do not have a body */
    case UTIL_MSG_BYEBYE:
    case UTIL_MSG_JOURNAL_ON:			/* Journal enable/disable commands from */
    case UTIL_MSG_JOURNAL_OFF:			/* dataVu Import function GCT 30-Mar-1996 */
      mSize = sizeof(short);
      break;
    case UTIL_MSG_VERIFYNOW:
      mSize = PVNAME_SZ + sizeof(mType);
      break;
    case UTIL_MSG_RESETNOTIFY:
      mSize = sizeof(mType) + sizeof(DBC_RESETREQ);	/* IOC reset from dataVu via TCP */
      break;
    case UTIL_MSG_OLDNOTIFY:
      mSize = sizeof(mType) + sizeof(int);		/* Online addition from dataVu via TCP */
      break;
    case UTIL_MSG_OLDELETE:
      mSize = sizeof(mType) + sizeof(DBC_OLDELETE);
      break;
    default:						/* all bets are off if the message type is invalid...Close socket! */
      tisSysLog(V_INFO,0,"Invalid message type...%d\n",mType);
      iocinfo->ioc_rxCount = 0;
      iocinfo->ioc_rxNextChar = 0;
      tcpError(iocinfo);
      return(0);
    }
  if(mSize > iocinfo->ioc_rxCount)
    return(0);
  else
    return(mSize);	/* 10-Jan-97 return message size. this includes the message type */
}
char	*addrToString(addr)
unsigned int	addr;
{
  sprintf(inetString,"%d.%d.%d.%d\0",
	  ((addr >> 24) & 0xff),			/* hi-order nibble first 31 - 24 */
	  ((addr >> 16) & 0xff),			/* 23 - 16 next */
	  ((addr >>  8) & 0xff),			/* 15 - 8 next */
	  (addr & 0xff));					/*  7 - 0 at last */
  return(inetString);
}
void dbSetNoBlock(DBC_IOCINFO_PTR info)
{
#ifndef WIN32
  int  stat;
#endif WIN32
  int  true=1;
  int  txSize = 4096;
#ifndef WIN32
#ifdef SOLARIS
  stat = fcntl(info->ioc_socket,F_GETFL,true);
  if(stat >= 0)
    {
      do
	{
	  if (setsockopt(info->ioc_socket, SOL_SOCKET, SO_SNDBUF, (char *)&txSize, sizeof(txSize)) == -1)
	    break;
	  txSize += 4096;
	} while (txSize < 65536);
      txSize = 4096;
      setsockopt(info->ioc_socket, SOL_SOCKET, SO_RCVBUF, (char *)&txSize, sizeof(txSize));
      stat |= O_NDELAY;
      stat = fcntl(info->ioc_socket,F_SETFL,stat);
    }
#else
  stat = ioctl(info->ioc_socket,FIONBIO,&true);	/* GCT 2-Feb-1995 set to non-blocking */
#endif
  if(stat < 0)
    {
#ifndef WIN32
      tisSysLog(V_INFO,0,"Error setting Non-blocking I/O for %s...%m",info->ioc_name);
#endif WIN32
    }
#endif WIN32
}
int parseThrottleParams(void)
{
  FILE *fp;
  char line[81];

  if((fp = fopen(dbThrottleParams,"r")) == 0)
    dbThrottleInfo.throttleStatus = INVALID;
  else
    {
      while(fgets(line,80,fp) != NULL)
	{
	}
    }
  return(1);
}
void dbcConnectToHost(DBC_IOCINFO_PTR node)
{
  struct  sockaddr_in	hostaddr;
  int len,trueFlag = 1;
  tisSysLog(V_DEBUG,0,"Current state for HOST <%-*.*s> on socket %d is <%s>.\n",
		   DBC_IOCNAMESIZE,DBC_IOCNAMESIZE,node->ioc_name,node->ioc_socket,state_string[node->ioc_state]);
  if(node->ioc_socket == -1)
    {
      tisSysLog(V_DEBUG,0,"Attempting to connect to %s",node->ioc_name);
#ifdef PRIV_AUDIT
      {
      SSL *newSSL = connect_ssl_connection_timed (htonl(node->ioc_addr),
			    DBSERVER_UDP_PORT, 15/*seconds*/);
      if (newSSL)
	{
	  GetPeerCN (newSSL, X509_dn, sizeof(X509_dn));
	  if (GetGeneralPrivHost (X509_dn, "DB_") < 1)					
	    {
	      /* No priviledge to connect! */
	      GetPeerDN (newSSL, X509_dn, sizeof(X509_dn));
	      VERBOSE(V_DEBUG)(LOG_ALERT|LOG_LOCAL1,"dbClient priviledge error, %s not authorized\n",
		    X509_dn);
	      SecurityAlarm (X509_dn, "dbClient priviledge error, Not authorized", NULL);
	      if (auditLog)
		{
		  char msg[80];

		  sprintf (msg, "Client connect failed from %s", node->ioc_name);
		  AuditAddEvent (auditLog, AUDIT_LOGON, AUDIT_FAIL, X509_dn,
			"dbClient", "Client connect failed");
		}
	      sslSharedCloseConn (newSSL);
	      newSSL = NULL;
	    }
	else
	    {
	      GetPeerDN (newSSL, X509_dn, sizeof(X509_dn));
	      if (auditLog)
		{
		  char msg[80];

		  sprintf (msg, "Client connect from %s", node->ioc_name);
		  AuditAddEvent (auditLog, AUDIT_LOGON, AUDIT_SUCCESS, X509_dn,
			"dbClient", "Client connect from %s");
		}
	      node->ioc_socket = SSL_get_fd (newSSL);
	      node->ioc_state = IOC_OK;		/* OK to proceed */
	      node->ioc_lastRxTime = time (0);
	      if (node->ioc_socket > maxFd)
		  maxFd = node->ioc_socket;
	      FD_SET (node->ioc_socket, &myFds);
	      node->ioc_rxCount = 0;
	      node->ioc_rxNextChar = 0;
	      node->dbt.dbtIx = 0;			/* Initialize dbtIx to 0 when iocinfo is initialized */
	      numConnected++;
	      if (numConnected > maxConnected) maxConnected = numConnected;
	      maxSocket (node->ioc_socket);		/* max-out this sockets buffers */
 	      sendBoomerang (node);
	      setsockopt (SSL_get_fd (newSSL), IPPROTO_TCP, TCP_NODELAY, (char *)&trueFlag,sizeof(trueFlag));
	    }
	}
      if (newSSL == NULL)
	{
	  VERBOSE(V_INFO)(LOG_ALERT|LOG_LOCAL1,"Error from HOST %s connect...%m",node->ioc_name);
	}
      }
#else
      hostaddr.sin_family		= AF_INET;
      hostaddr.sin_addr.s_addr	= htonl(node->ioc_addr);
      hostaddr.sin_port		= htons(DBSERVER_UDP_PORT);
      node->ioc_socket = socket(AF_INET,SOCK_STREAM,0);
      len = sizeof(struct sockaddr);
      if(connect(node->ioc_socket,(struct sockaddr *)&hostaddr,len) != -1)
	{
	  setsockopt(node ->ioc_socket,IPPROTO_TCP,TCP_NODELAY,(char *)&trueFlag,sizeof(trueFlag));
	  node->ioc_state = IOC_OK;		/* OK to proceed */
	  node->ioc_lastRxTime = time(0);
	  if(node->ioc_socket > maxFd) maxFd = node->ioc_socket;
	  FD_SET(node->ioc_socket,&myFds);
	  node->ioc_rxCount = 0;
	  node->ioc_rxNextChar = 0;
	  node->dbt.dbtIx = 0;			/* Initialize dbtIx to 0 when iocinfo is initialized */
	  numConnected++;
	  if(numConnected > maxConnected) maxConnected = numConnected;
	  /*maxSocket(node->ioc_socket);*/		/* max-out this sockets buffers */
 	  sendBoomerang(node);
	}
      else
	{
	  tisSysLog(V_INFO,0,"Error from HOST %s connect...%m",node->ioc_name);
	  close(node->ioc_socket);
	  FD_CLR((unsigned)node->ioc_socket,&myFds);
	  node->ioc_socket = -1;
	  node->ioc_state = IOC_NONET;
	}
#endif PRIV_AUDIT
    }
  else
    /*
      GCT 18-Jul-1997 We must boomerang the host(s) I am connected to so I can tell
      when my socket to them is gone. Otherwise, since I never send anything to those
      hosts,(because I do not have the -r command line arg) I will never know that
      the socket has gone away(via SIGPIPE signal).
    */
    {
      if((time(0) - node->ioc_lastTxTime) > DBC_Cardiac)
	{
	  sendBoomerang(node);
	}
      /*
	GCT 26-Aug-1997 If we haven't heard from the "-r" host in a while then
	close the socket and re-aquire later.
      */
      if((time(0) - node->ioc_lastRxTime) > DBC_Cardiac )
	{
	  tisSysLog(V_INFO,0,
			   "Lost (Timed out) communication link with HOST <%-*.*s>...closing socket <%d>.\n",
			   DBC_IOCNAMESIZE,DBC_IOCNAMESIZE,node->ioc_name,node->ioc_socket);
#ifdef PRIV_AUDIT
	  {
	  SSL *conn = sslSharedGetConn (node->ioc_socket);

	  if (auditLog)
	    {
	    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
	    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			   "dbClient", "Connection closed due to inactivity");
	    }
	  sslSharedCloseConn (conn);
	  }
#else
	  close(node->ioc_socket);			/* close this socket... get a new one later */
#endif PRIV_AUDIT
	  FD_CLR(node->ioc_socket,&myFds);
	  node->ioc_socket = -1;			/* clear socket */
	  node->ioc_state = IOC_NONET;
	  node -> ioc_rxCount = 0;
	  node -> ioc_rxNextChar = 0;
	  node->dbt.dbtIx = 0;				/* Initialize dbtIx to 0 when node is initialized */
	  ackCleanList(node->ioc_ackPending);
	  node->ioc_ackPending = 0;
	}
    }
}
/*
  handle connection requests from other hosts
*/
void dbcHandleHostConnect(int *newfd,int addr)		/* other host has connected */
{
struct hostent *ent;
char hname[100];
int index;
DBC_IOCINFO_PTR node;

index = dbcFindHost(addr);				/* get the index and the address of the entry with this address */
node = (DBC_IOCINFO_PTR)&hostInfo[index];
if((ent = gethostbyaddr((char *)&addr,sizeof(addr),AF_INET)) != 0)
  {
    strcpy(hname,ent->h_name);
    strtok(hname,".");
    strcpy(node->ioc_name,hname);			/* strip off other stuff from name e.g. "tis4000.com" */
  }
dbcDoAccept(addr,node,newfd);
}
void dbcDoAccept(int addr,DBC_IOCINFO_PTR node,int *newfd)
{
  char trueFlag = 1;

  if(node->ioc_socket != -1)
    {
#ifdef PRIV_AUDIT
	SSL *conn = sslSharedGetConn (node->ioc_socket);

	if (auditLog)
	    {
	    GetPeerDN (conn, X509_dn, sizeof(X509_dn));
	    AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
			   "dbClient", "Prior connection closed");
	    }
	sslSharedCloseConn (conn);
#else
      close(node->ioc_socket);			/* close this socket...might get a new one */
#endif PRIV_AUDIT
      FD_CLR(node->ioc_socket,&myFds);
      node->ioc_socket	= -1;			/* clear socket */
      node->ioc_state	= IOC_NONET;
    }
  node->ioc_socket = *newfd;			/* this is the socket to use */
  node->ioc_addr = addr;
  setsockopt(node->ioc_socket,IPPROTO_TCP,TCP_NODELAY,(char *)&trueFlag,sizeof(trueFlag));
  dbSetNoBlock(node);
  node->ioc_state = IOC_OK;		       	/* connected now! */
  node->ioc_lastRxTime = time(0);			/* time of this recieve */
  node->ioc_rxCount = 0;			/* nothing in buffer yet */
  node->ioc_rxNextChar = 0;			/* start of buffer */
  node->dbt.dbtIx = 0;				/* Initialize dbtIx to 0 when iocinfo is initialized */
  if(node->ioc_socket > maxFd) maxFd = node->ioc_socket;
  FD_SET(node->ioc_socket,&myFds);
  /*maxSocket(node->ioc_socket);*/			/* max-out this sockets buffers */
  tisSysLog(V_DEBUG,0,"Accepted connection to %s on socket %d",node->ioc_name,node->ioc_socket);
  numConnected++;
  if(numConnected > maxConnected) maxConnected = numConnected;
  *newfd = -1;					/* done with this fd */
}
/*
  Find a host given its inet addr. Return the index in the hostInfo array of a found
  host or the next available slot.
*/
int dbcFindHost(int addr)
{
  int i;
  for(i = 0;i < DBC_MAX_HOSTS;i++)
    {
      if(addr == hostInfo[i].ioc_addr)
	return(i);
      else if(hostInfo[i].ioc_addr == 0)
	return(i);
    }
return(-1);		/* this should NOT happen but return -1 anyway */
}
void	dbCtrace (TRACE_TYPE type, DBC_IOCINFO_PTR iocinfo)
{
	iocinfo->dbt.dbtList [iocinfo->dbt.dbtIx].id = type;
	iocinfo->dbt.dbtList [iocinfo->dbt.dbtIx].ioc_msgtype = iocinfo->ioc_msgtype;
	iocinfo->dbt.dbtList [iocinfo->dbt.dbtIx].rxCount = iocinfo->ioc_rxCount;
	iocinfo->dbt.dbtList [iocinfo->dbt.dbtIx].rxNextChar = iocinfo->ioc_rxNextChar;
	iocinfo->dbt.dbtList [iocinfo->dbt.dbtIx].rxbuffer0 = iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar];
	iocinfo->dbt.dbtList [iocinfo->dbt.dbtIx].rxbuffer1 = iocinfo->ioc_rxBuffer[iocinfo->ioc_rxNextChar+1];

	iocinfo->dbt.dbtIx = (iocinfo->dbt.dbtIx + 1) % 128;
}
/*
  This list is used to allow/dis-allow communication between the IOCs in the list
  and the dbClient who reads it. If the list is empty or does not exist then ALL
  IOC's are eligable for communication (same as now). If an IOC is in the list,
  broadcasts from it will be ignored. If the IOC is currently communicating, the 
  socket will be closed. When the IOC is removed from the list, broadcasts will
  once again be responded to.

  The "kill -USR1" command line signal mechanism will be used to force dbClient
  to re-read the list and make the appropriate changes in the IOC comm status.

  GCT 27-May-1997
*/
void processIocInhibitList(void)
{
  char line[80];
  FILE *fp;
  DBC_IOCINFO_PTR	info;
  /*
    On error or no file proceed as in the past
  */
  allowAllIocs();			/* start by assuming all can communicate */
  if((fp = fopen("/tis4000db/iocControl.txt","r")) != 0)
    {
      while(fgets(line,80,fp) != 0)	/* then turn off the ones in the list */
	{
	  line[strlen(line) -1] = 0;	/* drop trailing line-feed */
	  if((info = dbcIocSearch(line)) != 0)
	    {
	      info->inhibitState = IOC_INHIBIT_COMM;
	      info->ioc_state = IOC_INHIBITED;
	      /*
		if already communicating then close socket and remove from socket mask
	      */
	      if(info->ioc_socket != -1)
		{
		  tisSysLog(V_DEBUG,0,"Stopping communications with IOC <%s>\n",info->ioc_name);
#ifdef PRIV_AUDIT
		  {
		    SSL *conn = sslSharedGetConn (info->ioc_socket);

		    if (auditLog)
			{
			GetPeerDN (conn, X509_dn, sizeof(X509_dn));
			AuditAddEvent (auditLog, AUDIT_LOGOFF, AUDIT_SUCCESS, X509_dn,
				       "dbClient", "Prior connection closed");
			}
		    sslSharedCloseConn (conn);
		  }
#else
		  close(info->ioc_socket);			/* close this socket...might get a new one */
#endif PRIV_AUDIT
		  FD_CLR(info->ioc_socket,&myFds);
		  info->ioc_socket	= -1;			/* clear socket */
		  info->ioc_rxCount = 0;
		  info->ioc_rxNextChar = 0;
		  info->dbt.dbtIx = 0;			/* Initialize dbtIx to 0 when info is initialized */
		  ackCleanList(info->ioc_ackPending);
		  info->ioc_ackPending = 0;
		}
	      else
		tisSysLog (V_STATUS,0,"Inhibiting communications with IOC <%s>\n",info->ioc_name);
	    }
	}
      fclose(fp);
    }
}
/*
  This function turns off the inhibitState flag to allow
  all IOCs to communicate.

  GCT 27-May-1997
*/
void allowAllIocs(void)
{
  DBC_IOCINFO_PTR	info;
  info = (DBC_IOCINFO_PTR)lstFirst((LIST *)&DBC_IOCHeader);
  while(info)
    {
      if(info->inhibitState == IOC_INHIBIT_COMM)
	{
	  tisSysLog(V_STATUS,0,"Enabling communications with IOC <%s>\n",info->ioc_name);
	  info->ioc_state = IOC_NONET;
	}
      info->inhibitState = IOC_ALLOW_COMM;
      info = (DBC_IOCINFO_PTR)lstNext((NODE *)info);
    }
  return;
}

#ifdef DONT_USE_UDP

#ifndef	WIN32
#define	SOCKET	int
#endif	!WIN32

/****************************************************************************/
/*									    */
/*	 Attempt to connect (timed), then close to indicate connect-back    */
/****************************************************************************/
static void kick_TCP (DBC_IOCINFO_PTR iocinfo /* in net order */,
	       int port) /* in host order */
{
    struct sockaddr_in taddr;
    SOCKET socket_fd;
    fd_set	write_fds;
    struct timeval tvtimeout;
    int true = 1;
#ifdef SOLARIS
    int stat;
#endif SOLARIS

   /* create the TCP socket */
    socket_fd = socket (PF_INET, SOCK_STREAM, 0);
    /* set socket non-blocking! */
#ifdef SOLARIS
    stat = fcntl(socket_fd,F_GETFL,true);
    if(stat >= 0)
      {
	stat |= O_NDELAY;
	stat = fcntl(socket_fd,F_SETFL,stat);
      }
    else
    {
      VERBOSE(V_INFO)(LOG_ALERT|LOG_LOCAL1,"Can't set non-block on kick_TCP connection for %s... %m",iocinfo->ioc_name);
    }
#else
#ifdef WIN32
    if (ioctlsocket (socket_fd, FIONBIO, &true) == SOCKET_ERROR)
#else
    if (ioctl (socket_fd, FIONBIO, (int)&true) == ERROR)
#endif WIN32
    {
	tisSysLog (0, 0,  "Can't set non-block on kick_TCP connection %d.\n",errno);
    }
#endif
    taddr.sin_family = AF_INET;
    taddr.sin_port = htons (port);
    taddr.sin_addr.s_addr = htonl (iocinfo->ioc_addr);

    if (connect (socket_fd, (struct sockaddr*)&taddr, sizeof(taddr)) != 0)
    {
#ifdef WIN32
	int localError = WSAGetLastError();
	if ((localError == WSAEINPROGRESS)
	  || (localError == WSAEWOULDBLOCK))
#else WIN32
	if (errno == EINPROGRESS)
#endif WIN32
	{
	    /* wait for read file descriptor */
	    FD_ZERO(&write_fds);
	    FD_SET(socket_fd, &write_fds);

	    /* Set timeout to be 0 */
	    tvtimeout.tv_sec = 1;
	    tvtimeout.tv_usec = 0;

	    /* now call select to block until connect, timeout, or error */
	    select (FD_SETSIZE, NULL, &write_fds, NULL, &tvtimeout);
	}
    }
    /* close up and return */
    close (socket_fd);
}
#endif DONT_USE_UDP
