/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include	<stdio.h>
#ifndef WIN32
#include	<time.h>
#include	<sys/types.h>
#include	<error.h>
#include	<syslog.h>
#endif WIN32
#include	"DBDefines.h"
#include	"DBCVerifyDefs.h"
#include	"DBCDefines.h"
#include	"DBProto.h"
#include	"dbRecType.h"
#include	"redmsgs.h"
#include	<errno.h>
#include	"msgMon.h"

void	DBC_SendVerifyReply();
void	DBC_RequestRecord();
void	DBC_DeleteAll();
int    	DB_GetFullRec(char *,int,char **,int *);
int	DB_PutFullRec(char *,int,char *,int);
int     DB_CrFullRec(char *,int,char *,int);
DBC_ERROR_CODES	DBC_ProcVerifyReply();
DBC_IOCINFO_PTR	dbcIocSearch();
extern	DB_TABLE_PTR	DBC_PvmapTable,DBC_IOAddrTable;
extern	DBC_XREFHDR_PTR	DBC_XREFTables;
extern	LIST	DBC_IOCHeader;

#define REQ_TIME	0
#define REQ_CRC		1
#define REQ_NOLOCAL	2
#define REQ_NEW		3
char *reqReason[] =
{
  "Time stamp mis-match",
  "CRC mis-match",
  "No local record",
  "New record"
};
/*
  17-Jan-97 This message is received from a HOST with the "-r" command line
  parameter. If a newer record is detected it is requested from the HOSt I
  just got this message from.
*/
DBC_ERROR_CODES	DBC_ProcVerifyRequest(DBC_IOCINFO_PTR iocinfo)
{
  VERIFY_INFO	local_info,remote_info;
  IMPORT_INFO	import_info;
  short		info_count;

  readTcp(iocinfo,(char *)&info_count,2);
  info_count = ntohs(info_count);
  while(info_count--)
    {
      readTcp(iocinfo,(char *)&remote_info,VERIFY_INFO_SIZE);
/*
	use PV from message to access local info.
*/
/*       VERBOSE(V_DEBUG)(LOG_ALERT|LOG_LOCAL1, */
/* 		       "Processing Verify request for <%-*.*s> from <%-16.16s>.\n", */
/* 		       PVNAME_SZ,PVNAME_SZ,&remote_info.viPVName[0],iocinfo->ioc_name); */
      if(DB_GetVerifyInfo(DBC_PvmapTable,&remote_info.viPVName[0],&local_info,0) != 0)
	{
	  if(remote_info.viTypeIndex == 0)
	    remote_info.viTypeIndex = local_info.viTypeIndex;
/*
  Do the comparisons begining with the time stamps. If remote timestamp
  is greater then request record unless remote point was deleted. If
  it was deleted then delete it here too.
*/
	  if(remote_info.viTimeStamp > local_info.viTimeStamp)
	    {
	      if(VI_IS_DELETED(remote_info.viPVStatus))
		{
		  local_info.viPVStatus |= VI_DELETED;
		  local_info.viTimeStamp =  remote_info.viTimeStamp;
		  DBC_DeleteAll(&local_info);
		  DB_PutVerifyInfo(DBC_PvmapTable,&local_info.viPVName[0],&local_info,0);
		}
	      else
		{
		  strncpy(import_info.viImportNode,remote_info.viHostName,DBC_IOCNAMESIZE);
		  DBC_RequestRecord(&remote_info,&import_info,REQ_TIME);
		}
	    }
	  else if(remote_info.viTimeStamp == local_info.viTimeStamp)
	    {
	      if(remote_info.viCRC != local_info.viCRC)
		{
		  strncpy(import_info.viImportNode,remote_info.viHostName,DBC_IOCNAMESIZE);
		  DBC_RequestRecord(&remote_info,&import_info,REQ_CRC);
		}
	    }
	}
      else
	{
	  if(remote_info.viTypeIndex > 0)
	    {
	      strncpy(import_info.viImportNode,remote_info.viHostName,DBC_IOCNAMESIZE);
	      DBC_RequestRecord(&remote_info,&import_info,REQ_NOLOCAL);
	    }
	}
    }
  return(DBC_SUCCESS);
}

void	DBC_RequestRecord(VERIFY_INFO_PTR info,IMPORT_INFO_PTR import,int reason)
{
  short			msg = htons(VI_MSG_RECORDREQUEST);
  char			nodeName[DBC_IOCNAMESIZE+1],i;
  short			tndx;
  for(i = 0;i < DBC_IOCNAMESIZE; i++)				/* trim trailing spaces */
    nodeName[i] = (import->viImportNode[i] == ' ' ? 0 : import->viImportNode[i]);
  for(i = 0;i < maxConnected; i++)
    {
      if(!strcmp(nodeName,hostInfo[i].ioc_name))
	{
	  tisSysLog(V_STATUS,0,"Requesting more recent record for <%-*.*s> from <%-*.*s>...%s\n",
	    PVNAME_SZ,PVNAME_SZ,&info->viPVName[0],DBC_IOCNAMESIZE,DBC_IOCNAMESIZE,nodeName,reqReason[reason]);
	  writeTcp(&hostInfo[i],(char *)&msg,2);
	  writeTcp(&hostInfo[i],(char *)&nodeName[0],DBC_IOCNAMESIZE);
	  writeTcp(&hostInfo[i],&info->viPVName[0],PVNAME_SZ+4);
	  tndx = htons(info->viTypeIndex);
	  writeTcp(&hostInfo[i],(char *)&tndx,sizeof(tndx));
	  return;
	}
    }
  tisSysLog(V_INFO,0,"IOC <%-16.16s> not on this host!\n",nodeName);
}

DBC_ERROR_CODES	DBC_ProcRecordRequest(DBC_IOCINFO_PTR iocinfo)
{
  char	name[PVNAME_SZ + 4],node[DBC_IOCNAMESIZE+1],*buff;
  int		recSize,index;
  short	msg = htons(VI_MSG_RECORDREPLY);
  short	isDeleted,sendIndex,rSize;
  VERIFY_INFO	local_info;
  
  readTcp(iocinfo,node,DBC_IOCNAMESIZE);
  readTcp(iocinfo,name,PVNAME_SZ+4);
  readTcp(iocinfo,(char *)&sendIndex,sizeof(sendIndex));
  index = ntohs(sendIndex);
  sprintf(name,"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,name);
  if(!DB_GetVerifyInfo(DBC_PvmapTable,name,&local_info,0))	/* see if record deleted here. */
    {
      tisSysLog(V_INFO,0,"Error getting verify info for <%s>\n",name);
      return(DBC_SUCCESS);
    }
/*
	GCT 6-Feb-1995

	If type index is zero then we got this record request from a HOST that did not have it. Use
	the type index returned in the VERIFY_INFO structure to assemble the record.
*/
  if(index != local_info.viTypeIndex)
    index = local_info.viTypeIndex;
  tisSysLog(V_DEBUG,0,"Received inter-host record request for <%s> from <%-*.*s>\n",
		   name,DBC_IOCNAMESIZE,DBC_IOCNAMESIZE,iocinfo->ioc_name);
  isDeleted = VI_IS_DELETED(local_info.viPVStatus);    	/* Get deleted flag */
  if(isDeleted == 0)
    {
      if(DB_GetFullRec(name,index,&buff,&recSize))		/* allocate space to hold entire record */
	{
	  writeTcp(iocinfo,(char *)&msg,sizeof(msg));		       	/* send message type */
	  writeTcp(iocinfo,name,PVNAME_SZ);		       	/* then PV name */
	  sendIndex = index;
	  sendIndex = htons(sendIndex);
	  writeTcp(iocinfo,(char *)&sendIndex,sizeof(sendIndex));      	/* then point type index */
	  sendIndex = htons(isDeleted);
	  writeTcp(iocinfo,(char *)&sendIndex,sizeof(sendIndex));      	/* deletion status */
	  rSize = recSize;
	  rSize = htons(rSize);
	  writeTcp(iocinfo,(char *)&rSize,sizeof(rSize));		/* send size of following record */
	  writeTcp(iocinfo,buff,recSize);			/* and finally the whole record */
	  free(buff);						/* free space allocated by DB_GetFullRec.c */
	}
      else
	tisSysLog(V_DEBUG,0,"Could not assemble full record for <%s>",name);
      free(buff);						/* free space allocated by DB_GetFullRec.c */
    }
  else
    {
      writeTcp(iocinfo,(char *)&msg,sizeof(msg));		       	/* send message type */
      writeTcp(iocinfo,name,PVNAME_SZ);			       	/* then PV name */
	  sendIndex = index;
      sendIndex = htons(sendIndex);
      writeTcp(iocinfo,(char *)&sendIndex,sizeof(sendIndex));	        /* then point type index */
      sendIndex = htons(isDeleted);
      writeTcp(iocinfo,(char *)&sendIndex,sizeof(sendIndex));	       	/* deletion status */
      rSize = htons(0);						/* size of remaining record...zero if deleted */
      writeTcp(iocinfo,(char *)&rSize,sizeof(rSize));			/* then size of following record */
    }
  return(DBC_SUCCESS);
}

DBC_ERROR_CODES	DBC_ProcRecordReply(DBC_IOCINFO_PTR iocinfo)
{
  char	name[PVNAME_SZ + 1],*buff;
  short	typeIndex;
  int		index,rSize;
  short	isDeleted,recSize;
  VERIFY_INFO	local_info;
  DB_RECORD_PTR	rp;

  readTcp(iocinfo,name,PVNAME_SZ);
  sprintf(name,"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,name);
  tisSysLog(V_DEBUG,0,"Received inter-host record reply for <%s>\n",name);
  readTcp(iocinfo,(char *)&typeIndex,sizeof(typeIndex));
  index = ntohs(typeIndex);
  readTcp(iocinfo,(char *)&isDeleted,sizeof(isDeleted));
  isDeleted = ntohs(isDeleted);
  readTcp(iocinfo,(char *)&recSize,sizeof(recSize));
  recSize = ntohs(recSize);
  rSize = recSize;
  if(isDeleted == 0)
    {
      if((buff = calloc(1,recSize)) != 0)
	{
	  readTcp(iocinfo,buff,rSize);
	  /*
	    At this point the record must be placed in the database files. If the record does not exist
	    on this host then all the files must have the record added. Then the record from the message
	    can replace the newly added (and blank) record.
	  */
	  /*
	    GCT 5-Jul-1996

	    Check point types between the to instances of the same NAME. If the types
	    are different, then delete the older record and create the new record
	    as though it never existed.
	  */
	  if(DB_GetVerifyInfo(DBC_PvmapTable,name,&local_info,0) != 0)
	    {
	      if(local_info.viTypeIndex != index)	/* if types do not match then delete local point and create new from recieved on */
		{
		  local_info.viPVStatus |= VI_DELETED; 	/* set deleted flag */
		  local_info.viTimeStamp = synchTime();	/* and set time in PVMAP */
		  DBC_DeleteAll(&local_info);
		  DB_PutVerifyInfo(DBC_PvmapTable,name,&local_info,0);
		  if(DB_CrFullRec(name,index,buff,rSize))
		  tisSysLog(V_DEBUG,0,"Creating new record entry for <%s> as type %s\n",name,GET_PRECTYPE(index));
		  free(buff);
		  return(DBC_SUCCESS);		       	/* ALL done! */
		}
	    }
	  if(DB_Seek(DBC_PvmapTable,name) == 0)
	    {
	      /*
		Create a new record to be over-written by record from message.
	      */
	      if(DB_CrFullRec(name,index,buff,rSize))
	      {
		  tisSysLog(V_INFO,0,"Creating a new record entry for <%s>\n",name);
	      }
	    }
	  else
	    {
	      if(DB_PutFullRec(name,index,buff,rSize))
	      {
		  tisSysLog(V_DEBUG,0,"Modifying existing record entry for <%s>\n",name);
	      }
	    }
	  free(buff);
	}
    }
  else
    {
      if(DB_GetVerifyInfo(DBC_PvmapTable,name,&local_info,0) != 0)
	{
	  if(index == local_info.viTypeIndex)
	    {
	      tisSysLog(V_DEBUG,0,"Record <%s> was deleted on other host...delete it here too.\n",name);
	      local_info.viPVStatus |= VI_DELETED;		/* set deleted flag */
	      local_info.viTimeStamp = synchTime();			/* and set time in PVMAP */
	      DBC_DeleteAll(&local_info);
	      DB_PutVerifyInfo(DBC_PvmapTable,name,&local_info,0);
	    }
	}
      else
	/*
	  Point not found...maybe never her in the first place. Create dummy entry anyway with info
	  from other host.
	  */
	{
	  rp = (DB_RECORD_PTR)DB_NewRecord(DBC_PvmapTable,name);       	/* create a new record (place holder) */
	  DB_PutFieldValue(rp,pvmapNameField,name);				/* save name */
	  DB_PutFieldValue(rp,pvmapTypeField,GET_PRECTYPE(index));	/* and point type */
	  DB_PutFieldValue(rp,pvmapDeleteFlag,&isDeleted);			/* and deleted flag */
	  tisSysLog(V_DEBUG,0,"Place-holder created for <%s>,type = %d\n",name,index);
	}
    }
}
/*
	Delete all records here if the most recent record among all hosts
	has the deleted flag set.
*/
void	DBC_DeleteAll(VERIFY_INFO_PTR info)
{
  DBC_XREFHDR_PTR	xhdr;
  int	i;
  char	key[PVNAME_SZ+4],*cp,dname[PVNAME_SZ+4];

  strcpy(dname,&info->viPVName[0]);
  if((cp = (char *)strchr(dname,' ')) != 0)
    *cp = 0;			/* drop space padding */
/* Attempt to get a lock on the dbLock.lock file. If this fails then do not */
/* create any new records. */
  if(!dbLockTest(120))
    {
      tisSysLog(V_INFO,0,"Can't delete record(s) for...%s\n",info->viPVName);
      return;
    }
  xhdr = DBC_XREFTables+info->viTypeIndex;
  for(i=0;i<DBC_MAX_TABLES;i++)
    {
      if(xhdr->xrefHdrTables[i] == 0) continue;
      if(xhdr->xrefHdrTables[i] == DBC_PvmapTable) continue;
      if((info->viTypeIndex == VU_PTO) && (xhdr->xrefHdrTables[i] == DBC_IOAddrTable))
	{
	  sprintf(key,"%s1\0",dname);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],key);
	  sprintf(key,"%s2\0",dname);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],key);
	}
      else if((info->viTypeIndex == VU_PAT) && (xhdr->xrefHdrTables[i] == DBC_IOAddrTable))
	{
	  sprintf(key,"%s1\0",dname);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],key);
	  sprintf(key,"%s2\0",dname);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],key);
	  sprintf(key,"%s3\0",dname);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],key);
	  sprintf(key,"%s4\0",dname);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],key);
	}
      else
	{
	  DB_UseKey(xhdr->xrefHdrTables[i],FLD_NAME);
	  DB_DeleteMulti(xhdr->xrefHdrTables[i],info->viPVName);
	  DB_UseKey(xhdr->xrefHdrTables[i],FLD_CALCOP_KEY);
	}
    }
  dbUnLock();
}

void	DBC_SendToAllHosts(VERIFY_INFO_PTR info,short msgType)
{
  VERIFY_HDR   	verify_hdr;
  int i;

  verify_hdr.viMsgType = htons(msgType);
  verify_hdr.viCount = htons(1);
  for(i = 0;i < DBC_MAX_HOSTS;i++)
    {
      if(hostInfo[i].ioc_socket != -1)
	{
	  writeTcp(&hostInfo[i],(char *)&verify_hdr,VERIFY_HDR_SIZE);
	  writeTcp(&hostInfo[i],(char *)info,VERIFY_INFO_SIZE);
	}
    }
}

DBC_ERROR_CODES	DBC_ProcNewRecRequest(DBC_IOCINFO_PTR iocinfo)
{
  VERIFY_INFO	remote_info;
  IMPORT_INFO	import_info;
  short		info_count;
  
  readTcp(iocinfo,(char *)&info_count,2);
  readTcp(iocinfo,(char *)&remote_info,VERIFY_INFO_SIZE);
  strncpy(import_info.viImportNode,remote_info.viHostName,DBC_IOCNAMESIZE);
  DBC_RequestRecord(&remote_info,&import_info,REQ_NEW);
  return(DBC_SUCCESS);
}
