/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#include <stdio.h>
#include <sys/stat.h>
#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#ifdef WIN32
#include <time.h>
#include "NTUnixSubs.h"
typedef char* caddr_t;
#else
#include <sys/time.h>
#endif WIN32

#include "dbDefs.h"
#include "dbFldTypes.h"
#include "dbRecType.h"
#include "dbRecDes.h"
#include "dbRecords.h"
#include "DBDefines.h"
#include "DBCVerifyDefs.h"	/* ems 07-31-97, needs to come before DBProto.h */
#include "DBProto.h"
#include "sdrLoad.h"
#include "resolveLinks.h"

static	void	dumpIndex (),showRecDate(),touchDate();
static time_t	synchTime();
static int reNdx(char *name);
void		scanArch(void);
static void	fixType(char *name,char *type);
static void	printRec(FILE *f,int rs,int nr,int hl);
static void	deleteRec(char *name);
D4DATA		*CheckAndOpen(C4CODE *db,char *name);
void		crossCheck(void);
static void	showStruct (char *dbase);
struct	dbdatabase	DB_Database;			/* for DB functions */

/* dBase file header record */

typedef struct {
    char	Id;		/* Identification */
    char	Year;		/* Last update year */
    char	Month;		/* Last update month */
    char	Day;		/* Last update day */
    unsigned int	NumRecs;	/* Number of records */
    unsigned short HeaderLength;/* Length (in bytes) of header info */
    unsigned short RecordLen;	/* Length (in bytes) of each record */
    char	res [16];
    short	Mdx;		/* A .MDX file exists */
    char	Reserved2 [2];	/* Reserved area */
} HEADER;
typedef	enum
{
  VU_AI=1,VU_AO,VU_BI,VU_BO,VU_CALC,VU_FANOUT,VU_MBBI,VU_MBBO,VU_PID,VU_SEL,VU_STATE,VU_STRINGIN,VU_STRINGOUT,
  VU_WAVEFORM,VU_LEADLAG,VU_RAMP,VU_PIPE,VU_TANK,VU_VALVE,VU_PUMP,VU_POT,VU_ACCUM,VU_PTO,VU_DEADTIME,VU_DATAMOVE,
  VU_TD,VU_SIGCHAR,VU_UDB,VU_PAT,VU_LDFW1,VU_LDFW2,VU_DDC,VU_MAX_TYPES /* ALL VALID TYPES MUST COME BEFORE THIS VALUE !!! */
} VU_VALID_TYPES;
   
#ifdef BYTEORDER_0123
#define	swapShort(x)	(x)
#define	swapLong(x)	(x)
#else
#define	swapShort(x)	(((x) >> 8) | ((x) << 8))
#define	swapLong(x)	( (((x) & 0xff000000) >> 24) | (((x) & 0xff0000) >> 8) | \
			(((x) & 0xff00) << 8) | (((x) & 0xff) << 24))
#endif

/*----------*/
void main (argc, argv)

    int		argc;
    char	*argv [];

    {

	extern	char	*optarg;
	extern	int	optind; 
		int	noWrite = 0,recPrint=0,cbstat = 0;
		FILE	*fil;
		HEADER	header;
#ifdef WIN32
		struct _stat statBuf;
#else
		struct stat statBuf;
#endif WIN32
		int	numRecs,rest,offset;
		char	c,*fptr;
		short	recLen, headLen;
		unsigned long	nRecs;

	/* Process command line options */

		while ((c = getopt (argc, argv, "nrx:d:t:p:ak:c:s")) != -1)
	    switch (c)
	      {
	      case 'n':	/* Do not write fixed file */
		noWrite = 1;
		break;
	      case 'r':	/* print records to stdout */
		recPrint = 1;
		break;
	      case 'i':	/* Dump the index */
		dumpIndex (optarg,argv[optind]);
		exit (0);
		break;
	      case 'x':
		cbstat = reNdx(optarg);
		exit(cbstat);
		break;
	      case 'd':
		showRecDate(argv[optind-1]);
		exit(0);
		break;
	      case 't':
		touchDate(argv[optind-1]);
		exit(0);
		break;
	      case 'p':
		fixType(argv[optind-1],argv[optind]);
		exit(0);
		break;
	      case 'a':
		scanArch();
		exit(0);
		break;
	      case 'k':
		deleteRec(argv[optind-1]);
		exit(0);
		break;
	      case 'c':
		crossCheck();
		exit(0);
	      case 's':
		showStruct (argv[optind]);
		exit(0);
		break;
	      case '?':
		fprintf(stderr,"Usage is:\n");
		fprintf(stderr,"dbFix [-x -n -r [-i tagname] [-d name] [-t ALL|pointnanme]] filename.dbf\n");
		fprintf(stderr,"\t-x -- reindex named file\n");
		fprintf(stderr,"\t-n -- look but do not fix\n");
		/*fprintf(stderr,"\t-i -- dump contents of index(MDX) file\n");*/
		fprintf(stderr,"\t-r -- dump contents of file 1 record/line\n");
		fprintf(stderr,"\t-d -- display date/time for named record\n");
		fprintf(stderr,"\t-t -- touch the pvmap.dbf file to force current date/time\n");
		fprintf(stderr,"\t-a -- scan ar_reduc & pvmap...report differences as cau command file\n");
		fprintf(stderr,"\t-c -- scan pvmap...check point type files for name match\n");
		fprintf(stderr,"\t-s -- print the database structure\n");
		fprintf(stderr,"\tNo switch means report AND fix record count & non-printable chars\n");
		exit(1);
	      }
	/* Get the actual size of the file */

#ifdef WIN32
	_stat (argv [optind], &statBuf);
#else
	stat (argv [optind], &statBuf);
#endif WIN32

	/* Open the file for read/write */

	fil = fopen (argv [optind], "r+");
	if (fil == NULL)
	  {
	    perror("dbFix");
	    exit (-1);
	  }

	/* Read in the file header */

	if (fread ((char *)&header, sizeof (header), 1, fil) != 1)
	  {
	    perror("dbFix");
	    exit (-1);
	  }

	/* Do byte swapping */

	headLen = swapShort (header.HeaderLength);
	recLen = swapShort (header.RecordLen);
	nRecs = swapLong(header.NumRecs);

	/* Calculate actual number of records */

	numRecs = (statBuf.st_size - headLen) / recLen;

	/* If we are just printing each record, do it. */

	if(recPrint)
	  {
	    printRec (fil, recLen, numRecs,headLen);
	    exit (0);
	  }
	/* Print out some interesting things */

	printf ("File ID: %2x\n", (int)header.Id);
	printf ("File Date: %d/%d/%d\n", (int)header.Month, (int)header.Day,
			(int)header.Year);
	printf ("# Records (from header): %d\n", nRecs);
	printf ("# Records (calculated): %d\n", numRecs);
	printf ("Header Length: %d\n", (int)headLen);
	printf ("Record Length: %d\n", (int)recLen);

	/* Update the record count in the header */

	header.NumRecs = swapLong (numRecs);
/*
	get some space and read file in.
*/
	rest = statBuf.st_size - headLen;	/* amount of rest of record */
	printf("Checking for non-printable characters in %s\n",argv[optind]);
	if((fptr = (char *)malloc(rest)) == 0)
	  {
	    perror("dbFix");
	    exit(-1);
	  }
	fseek(fil,headLen,0);				/* skip past field info...start from begining of file */
	if (fread ((char *)fptr, rest, 1, fil) != 1)
	  {
	    perror("dbFix");
	    exit (-1);
	  }
	for(offset = 0;offset < rest;offset++)
	  {
	    if(!isprint(fptr[offset]))
	      {
		if(offset != (rest - 1))
		  {
		    printf("At %08o data was %02x\n",offset+headLen,fptr[offset]);
		    fptr[offset] = ' ';				/* convert to space */
		  }
	      }
	  }
	
	if (!noWrite)
	  {
	    printf("Applying fixes... ");
	    rewind (fil);
	    fwrite ((char *)&header, sizeof (header), 1, fil);
	    fseek(fil,headLen,0);				/* skip past field info */
	    fwrite ((char *)fptr, rest, 1, fil);
	    printf("Done.\n");
	  }
	fclose (fil);
    }

/*----------*/
static void dumpIndex (char *name,char *tagName)
{
  C4CODE	cb;
  D4DATA	*data;
  T4TAG	*tag;

  d4init (&cb);
  data = CheckAndOpen (&cb, name);
  if (data == NULL)
    return;
  tag = d4tag (data, tagName);
  if (tag == NULL)
    {
      printf ("Unable to find tag %s\n", tagName);
      return;
    }
  d4tag_select (data, tag);
#if (defined(WIN32) || defined(SOLARIS))
	printf ("t4dump function is not available in codebase 6! \n");
#else
	t4dump (tag, 1, 1);
#endif WIN32
  d4close (data);
}
/*------------*/
static void	printRec(FILE *f,int rs,int nr,int hl)
{
  char	rec[4096];
  fseek(f,hl,0);				/* skip past field info header...start from begining of file */
  while(nr--)
    {
      if (fread (rec, rs, 1, f) != 1)
	exit(1);
      rec[rs] = 0;			/* null terminate */
      printf("%s\n",rec);	/* print to stdout...can be redirected */
    }
}
static int	reNdx(char *name)
{
  C4CODE	cb;
  D4DATA	*data;
  int stat;

  d4init(&cb);

  data = CheckAndOpen(&cb, name);
  if (data == NULL)
    {
      printf ("Unable to open %s\n", name);
      stat = -1;
    }
  else
    {
      printf("dbFix: Re-indexing file %s, please wait...",name);
      stat = d4reindex(data);
      printf("dbFix: Done.\n");
    }
  return(stat);
}

static void showRecDate(char *name)
{
  C4CODE	cb;
  D4DATA	*data;
  F4FIELD *timestamp,*deleteflag;
  char TS[27];
  time_t ts;

  d4init(&cb);
#ifdef WIN32
  data = d4open(&cb,"c:\\tis4000db\\pvmap.dbf");
#else
  data = d4open(&cb,"/tis4000db/pvmap.dbf");
#endif WIN32

  if (data == NULL)
    return;
  else
    {
      timestamp = d4field(data,"TIMESTAMP");
      deleteflag = d4field(data,"DELETEFLAG");
      if(d4seek(data,name) == 0)
	{
	  ts = (time_t)atoi(f4str(timestamp));
	  strcpy(TS,(char *)ctime(&ts));
	  TS[strlen(TS)-1] = 0;
	  printf("dbFix: PV \"%s\" last modified locally on %s(GMT) %s\n",name,TS,(f4true(deleteflag) ? "DELETED" : ""));
	}
      else
	printf("dbFix: Can't locate record %s\n",name);
      d4close(data);
    }
}
/*
  stuff given point type into file for specified record name
*/
static void fixType(char *name,char *type)
{
  C4CODE	cb;
  D4DATA	*data;
  F4FIELD *pointType;

  d4init(&cb);
#ifndef WIN32
  data = CheckAndOpen(&cb,"/tis4000db/pvmap.dbf");
#else
  data = CheckAndOpen(&cb,"c:\\tis4000db\\pvmap.dbf");
#endif WIN32

  if (data == NULL)
    return;
  else
    {
      pointType = d4field(data,"TYPE");
      if(d4seek(data,name) == 0)
	{
 	  f4assign(pointType,type);
	  printf("dbFix: Point <%s> is now type <%s>\n",name,type);
	}
      else
	printf("dbFix: Can't locate record %s\n",name);
      d4close(data);
    }
}
static void touchDate(char *oneName)
{
  C4CODE	cb;
  D4DATA	*data;
  F4FIELD *timestamp;
  char TS[27];
  time_t currentTime;

  d4init(&cb);
#ifdef WIN32
  data = d4open(&cb,"c:\\tis4000db\\pvmap.dbf");
#else
  data = d4open(&cb,"/tis4000db/pvmap.dbf");
#endif WIN32

  currentTime = synchTime(0);
  if (data == NULL)
    return;
  else
    {
      timestamp = d4field(data,"TIMESTAMP");
      strcpy(TS,(char *)ctime(&currentTime));
      TS[strlen(TS)-1] = 0;
      if(!strcmp(oneName,"ALL"))
	{
	  printf("dbFix: Setting all records to %s(GMT)\n",TS);
	  d4top(data);
	  do
	    {
	      sprintf(TS,"%d\0",currentTime);
	      f4assign(timestamp,TS);      	/* set time in record */
	      d4flush_all(data);
	    }
	  while(d4skip(data,1) == 0);
	  d4unlock_all(data);
	}
      else
	{
	  if(d4seek(data,oneName) == 0)
	    {
	      printf("dbFix: Setting %s to %s(GMT)\n",oneName,TS);
	      sprintf(TS,"%d\0",currentTime);
	      f4assign(timestamp,TS);
	    }
	  else
	    printf("dbFix: Can't locate record %s\n",oneName);
	}
      d4close(data);
    }
}
/*
  Scan pvmap looking for points with the trend enable flag set. For any that
  are set, look for the point in ar_reduc. If there are no entries for this
  point in ar_reduc, generate a text file contining a CAU command line to 
  disable trending for the point.
*/
void scanArch(void)
{
  DB_TABLE_PTR pvmap,ar_reduc;
  DB_FIELD_PTR trendField,nameField,typeField;
  DB_RECORD_PTR rec;
  char name[PVNAME_SZ+1],pType[20],onTrend;
  int type;
  char **arr;
  int num;
#ifndef WIN32
  if(sdrLoad("/tis4000db/default.dctsdr") < 0)
#else
  if(sdrLoad("c:\\tis4000db\\default.dctsdr") < 0)
#endif
    {
      printf ("dbFix: Unable to load SDR\n");
      return;
    }
  DB_InitDatabase("dbFix");    	/* init database system */
#ifndef WIN32
  DB_MapDatabase("/tis4000db");	/* map to database path */
#else
  DB_MapDatabase("c:\\tis4000db");
#endif WIN32
  pvmap = DB_MapTable("pvmap");
  ar_reduc = DB_MapTable("ar_reduc");
  if (pvmap == NULL)
    {
      printf ("dbFix: Unable to open pvmap.dbf\n");
      return;
    }
  if (ar_reduc == NULL)
    {
      printf ("dbFix: Unable to open ar_reduc.dbf\n");
      return;
    }
  trendField = DB_MapField(pvmap,"ONTREND");
  nameField = DB_MapField(pvmap,"NAME");
  typeField = DB_MapField(pvmap,"TYPE");
  rec = DB_Top(pvmap);
  while(rec)
    {
      DB_GetFieldValue(rec,nameField,name,&type);
      DB_GetFieldValue(rec,typeField,pType,&type);
      DB_GetFieldValue(rec,trendField,&onTrend,&type);
      if(strlen(pType))
	{
	  for(type = 1;type < dbRecType->number;type++)
	    {
	      if(!strcmp(pType,GET_PRECTYPE(type)))
		break;
	    }
	  if(type >= dbRecType->number)
		{
		DB_SeekNext(pvmap);
		continue;
		}
/*       printf("Name=<%s>,type=<%d>,trend=<%s>\n",name,type,(onTrend ? "T" : "F")); */
	  if(onTrend)
	    {
	      if((arr = (char **)dbFindInArreduc(ar_reduc,name,type,&num)) != 0)
		{
		  int i = 0;
		  while(arr[i])
		    free((void *)arr[i++]);
		  free((void *)arr);
		}
	      else
		{
		  printf("put %s.TENB NO\n",name);
		}
	    }
	}
      rec = DB_SeekNext(pvmap);
    }
  printf("quit\n");
  DB_UnMapTable(pvmap,0);
  DB_UnMapTable(ar_reduc,0);
  DB_UnMapDatabase(0);
}
/*
	This function provides the synchronization time for the inter-host comm functionality.
	this is in GMT. Naturally ULTRIX does NOT support the same functions for time conversion
	as the SUN workstations do hense the #ifdef's

	12-Sep-1994 GCT
*/
time_t	synchTime()
{
  time_t _tt_; 
  _tt_ = time(0);
#if !(defined(MIPSEL) || defined(SOLARIS) || defined(AXP) || defined(WIN32))
  return(timelocal(gmtime(&_tt_)));
#else
  return(mktime(gmtime(&_tt_)));
#endif
}
static void deleteRec(char *name)
{
  C4CODE	cb;
  D4DATA	*data;
  F4FIELD *deleteFlag,*timestamp;
  char TS[27];
  time_t currentTime;

  d4init(&cb);
#ifndef WIN32
  data = CheckAndOpen(&cb,"/tis4000db/pvmap.dbf");
#else
  data = CheckAndOpen(&cb,"c:\\tis4000db\\pvmap.dbf");
#endif WIN32

  currentTime = synchTime(0);

  if (data == NULL)
    return;
  else
    {
      timestamp = d4field(data,"TIMESTAMP");
      deleteFlag = d4field(data,"DELETEFLAG");
      if(d4seek(data,name) == 0)
	{
	  f4assign_char(deleteFlag,'T');
	  sprintf(TS,"%d\0",currentTime);
	  f4assign(timestamp,TS);      	/* set time in record */
	}
      else
	printf("Can't locate record %s\n",name);
      d4close(data);
    }
}
/*
  Cross check PV names in pvmap with coorespondin records in point
  type specific files
*/
void crossCheck(void)
{
  DB_TABLE_PTR pvmap,ai=0,bi=0,ao=0,bo=0,calc=0,calcops=0,fanout=0,fanlink=0,mbbi=0,mbbo=0,pid=0,sel=0,sellink=0,state=0,stringin=0,stringout=0;
  DB_TABLE_PTR waveform=0,leadlag=0,ramp=0,pto=0,pat=0,ddc=0,deadtime=0;
  DB_TABLE_PTR datamove=0,ldfw1=0,ldfw2=0,udb=0,sigchar=0,td=0,accum=0;
  DB_TABLE_PTR pump=0,pumpvars=0;
  DB_TABLE_PTR pot=0,potvars=0;
  DB_TABLE_PTR valve=0,vlvvars=0;
  DB_TABLE_PTR pipe=0,pipevars=0,pipecmds=0;
  DB_TABLE_PTR tank1=0,tank2=0,tank3=0,tank4=0,tankvars=0,tankcmds=0,tankvlvs=0,tankprbs=0;
  DB_TABLE_PTR ioaddr=0;
  DB_FIELD_PTR nameField,typeField,delField;
  DB_RECORD_PTR rec;
  char name[PVNAME_SZ+1],pType[20],key[40],del;
  int type,typeIndex;

#ifndef WIN32
  if(sdrLoad("/tis4000db/default.dctsdr") < 0)
#else
      if(sdrLoad("c:\\tis4000db\\default.dctsdr") < 0)
#endif WIN32
    {
      printf ("dbFix: Unable to load SDR\n");
      return;
    }
  DB_InitDatabase("dbFix");    	/* init database system */
#ifndef WIN32
  DB_MapDatabase("/tis4000db");	/* map to database path */
#else
  DB_MapDatabase("c:\\tis4000db");	/* map to database path */
#endif WIN32
  pvmap = DB_MapTable("pvmap");
  if (pvmap == NULL)
    {
      printf ("dbFix: Unable to open pvmap.dbf\n");
      return;
    }
  ioaddr = DB_MapTable("ioaddr");
  if (ioaddr == NULL)
    {
      printf ("dbFix: Unable to open ioaddr.dbf\n");
      return;
    }
  nameField = DB_MapField(pvmap,"NAME");
  typeField = DB_MapField(pvmap,"TYPE");
  delField = DB_MapField(pvmap,"DELETEFLAG");
  rec = DB_Top(pvmap);
  while(rec)
    {
      DB_GetFieldValue(rec,nameField,name,&type);
      DB_GetFieldValue(rec,typeField,pType,&type);
      DB_GetFieldValue(rec,delField,&del,&type);
      if(del)
	{
	  rec = DB_SeekNext(pvmap);
	  continue;
	}
      for(typeIndex = 1;typeIndex < dbRecType->number;typeIndex++)
	{
	  if(!strcmp(pType,GET_PRECTYPE(typeIndex)))
	    break;
	}
      if(typeIndex >= dbRecType->number)
	break;
      switch(typeIndex)
	{
	case VU_AI:
	  if(ai == 0)
	    ai = DB_MapTable("ai");
	  if(DB_Seek(ai,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ai.dbf\n",name);
	  if(DB_Seek(ioaddr,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_BI:
	  if(bi == 0)
	    bi = DB_MapTable("di");
	  if(DB_Seek(bi,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in di.dbf\n",name);
	  if(DB_Seek(ioaddr,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_AO:
	  if(ao == 0)
	    ao = DB_MapTable("ao");
	  if(DB_Seek(ao,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ao.dbf\n",name);
	  if(DB_Seek(ioaddr,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_BO:
	  if(bo == 0)
	    bo = DB_MapTable("do");
	  if(DB_Seek(bo,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in do.dbf\n",name);
	  if(DB_Seek(ioaddr,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_MBBI:
	  if(mbbi == 0)
	    mbbi = DB_MapTable("mbbi");
	  if(DB_Seek(mbbi,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in mbbi.dbf\n",name);
	  if(DB_Seek(ioaddr,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_MBBO:
	  if(mbbo == 0)
	    mbbo = DB_MapTable("mbbo");
	  if(DB_Seek(mbbo,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in mbbo.dbf\n",name);
	  if(DB_Seek(ioaddr,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_PID:
	  if(pid == 0)
	    pid = DB_MapTable("pid");
	  if(DB_Seek(pid,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pid.dbf\n",name);
	  break;
	case VU_PTO:
	  if(pto == 0)
	    pto = DB_MapTable("pto");
	  if(DB_Seek(pto,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pto.dbf\n",name);
	  sprintf(key,"%s1\0",name);
	  if(DB_Seek(ioaddr,key) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_STRINGIN:
	  if(stringin == 0)
	    stringin = DB_MapTable("strngin");
	  if(DB_Seek(stringin,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in strngin.dbf\n",name);
	  break;
	case VU_STRINGOUT:
	  if(stringout == 0)
	    stringout = DB_MapTable("strngout");
	  if(DB_Seek(stringout,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in strngout.dbf\n",name);
	  break;
	case VU_TD:
	  if(td == 0)
	    td = DB_MapTable("td");
	  if(DB_Seek(td,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in td.dbf\n",name);
	  break;
	case VU_SIGCHAR:
	  if(sigchar == 0)
	    sigchar = DB_MapTable("sigchar");
	  if(DB_Seek(sigchar,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in sigchar.dbf\n",name);
	  break;
	case VU_ACCUM:
	  if(accum == 0)
	    accum = DB_MapTable("accum");
	  if(DB_Seek(accum,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in accum.dbf\n",name);
	  break;
	case VU_WAVEFORM:
	  if(waveform == 0)
	    waveform = DB_MapTable("waveform");
	  if(DB_Seek(waveform,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in waveform.dbf\n",name);
	  break;
	case VU_LEADLAG:
	  if(leadlag == 0)
	    leadlag = DB_MapTable("leadlag");
	  if(DB_Seek(leadlag,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in leadlag.dbf\n",name);
	  break;
	case VU_RAMP:
	  if(ramp == 0)
	    ramp = DB_MapTable("ramp");
	  if(DB_Seek(ramp,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ramp.dbf\n",name);
	  break;
	case VU_PAT:
	  if(pat == 0)
	    pat = DB_MapTable("pat");
	  if(DB_Seek(pat,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pat.dbf\n",name);
	  sprintf(key,"%s1\0",name);
	  if(DB_Seek(ioaddr,key) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ioaddr.dbf\n",name);
	  break;
	case VU_DDC:
	  if(ddc == 0)
	    ddc = DB_MapTable("ddc");
	  if(DB_Seek(ddc,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ddc.dbf\n",name);
	  break;
	case VU_DEADTIME:
	  if(deadtime == 0)
	    deadtime = DB_MapTable("deadtime");
	  if(DB_Seek(deadtime,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in deadtime.dbf\n",name);
	  break;
	case VU_CALC:
	  if(calc == 0)
	    {
	      calc = DB_MapTable("calc");
	      calcops = DB_MapTable("calcops");
	    }
	  if(DB_Seek(calc,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in calc.dbf\n",name);
	  if(DB_Seek(calcops,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in calcops.dbf\n",name);
	  break;
	case VU_SEL:
	  if(sel == 0)
	    {
	      sel = DB_MapTable("sel");
	      sellink = DB_MapTable("sellink");
	    }
	  if(DB_Seek(sel,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in sel.dbf\n",name);
	  if(DB_Seek(sellink,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in sellink.dbf\n",name);
	  break;
	case VU_FANOUT:
	  if(fanout == 0)
	    {
	      fanout = DB_MapTable("fanout");
	      fanlink = DB_MapTable("fanlink");
	    }
	  if(DB_Seek(fanout,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in fanout.dbf\n",name);
	  if(DB_Seek(fanlink,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in fanlink.dbf\n",name);
	  break;
	case VU_STATE:
	  if(state == 0)
	    state = DB_MapTable("state");
	  if(DB_Seek(state,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in state.dbf\n",name);
	  break;
	case VU_DATAMOVE:
	  if(datamove == 0)
	    datamove = DB_MapTable("datamove");
	  if(DB_Seek(datamove,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in datamove.dbf\n",name);
	  break;
	case VU_LDFW1:
	  if(ldfw1 == 0)
	    ldfw1 = DB_MapTable("ldfw1");
	  if(DB_Seek(ldfw1,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ldfw1.dbf\n",name);
	  break;
	case VU_LDFW2:
	  if(ldfw2 == 0)
	    ldfw2 = DB_MapTable("ldfw2");
	  if(DB_Seek(ldfw2,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in ldfw2.dbf\n",name);
	  break;
	case VU_UDB:
	  if(udb == 0)
	    udb = DB_MapTable("udb");
	  if(DB_Seek(udb,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in udb.dbf\n",name);
	  break;
	case VU_PUMP:
	  if(pump == 0)
	    {
	      pump = DB_MapTable("pump");
	      pumpvars = DB_MapTable("pumpvars");
	    }
	  if(DB_Seek(pump,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pump.dbf\n",name);
	  if(DB_Seek(pumpvars,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pumpvars.dbf\n",name);
	  break;
	case VU_PIPE:
	  if(pipe == 0)
	    {
	      pipe = DB_MapTable("pipe");
	      pipevars = DB_MapTable("pipevars");
	      pipecmds = DB_MapTable("pipecmds");
	    }
	  if(DB_Seek(pipe,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pipe.dbf\n",name);
	  if(DB_Seek(pipevars,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pipevars.dbf\n",name);
	  if(DB_Seek(pipecmds,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pipecmds.dbf\n",name);
	  break;
	case VU_TANK:
	  if(tank1 == 0)
	    {
	      tank1 = DB_MapTable("tank1");
	      tank2 = DB_MapTable("tank2");
	      tank3 = DB_MapTable("tank3");
	      tank4 = DB_MapTable("tank4");
	      tankvars = DB_MapTable("tankvars");
	      tankcmds = DB_MapTable("tankcmds");
	      tankvlvs = DB_MapTable("tankvlvs");
	      tankprbs = DB_MapTable("tankprbs");
	    }
	  if(DB_Seek(tank1,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tank1.dbf\n",name);
	  if(DB_Seek(tank2,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tank2.dbf\n",name);
	  if(DB_Seek(tank3,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tank3.dbf\n",name);
	  if(DB_Seek(tank4,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tank4.dbf\n",name);
	  if(DB_Seek(tankvars,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tankvars.dbf\n",name);
	  if(DB_Seek(tankcmds,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tankcmds.dbf\n",name);
	  if(DB_Seek(tankvlvs,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tankvlvs.dbf\n",name);
	  if(DB_Seek(tankprbs,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in tankprbs.dbf\n",name);
	  break;
	case VU_POT:
	  if(pot == 0)
	    {
	      pot = DB_MapTable("pot");
	      potvars = DB_MapTable("potvars");
	    }
	  if(DB_Seek(pot,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in pot.dbf\n",name);
	  if(DB_Seek(potvars,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in potvars.dbf\n",name);
	  break;
	case VU_VALVE:
	  if(valve == 0)
	    {
	      valve = DB_MapTable("valve");
	      vlvvars = DB_MapTable("vlvvars");
	    }
	  if(DB_Seek(valve,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in valve.dbf\n",name);
	  if(DB_Seek(vlvvars,name) != (DB_RECORD_PTR)1)
	    printf("Point <%-27.27s> not in vlvvars.dbf\n",name);
	  break;
	default:
	  printf("Unsupported point type <%s>\n",GET_PRECTYPE(typeIndex));
	}
      rec = DB_SeekNext(pvmap);
    }
  DB_UnMapTable(pvmap,0);
  if(ai) DB_UnMapTable(ai,0);
  if(ao) DB_UnMapTable(ao,0);
  if(bi) DB_UnMapTable(bi,0);
  if(bo) DB_UnMapTable(bo,0);
  if(mbbi) DB_UnMapTable(mbbi,0);
  if(mbbo) DB_UnMapTable(mbbo,0);
  if(calc) DB_UnMapTable(calc,0);
  if(calcops) DB_UnMapTable(calcops,0);
  if(sel) DB_UnMapTable(sel,0);
  if(sellink) DB_UnMapTable(sellink,0);
  if(fanout) DB_UnMapTable(fanout,0);
  if(fanlink) DB_UnMapTable(fanlink,0);
  if(state) DB_UnMapTable(state,0);
  if(datamove) DB_UnMapTable(datamove,0);
  if(ldfw1) DB_UnMapTable(ldfw1,0);
  if(ldfw2) DB_UnMapTable(ldfw2,0);
  if(udb) DB_UnMapTable(udb,0);
  if(pump) DB_UnMapTable(pump,0);
  if(pumpvars) DB_UnMapTable(pumpvars,0);
  if(pipe) DB_UnMapTable(pipe,0);
  if(pipevars) DB_UnMapTable(pipevars,0);
  if(pipecmds) DB_UnMapTable(pipecmds,0);
  if(pot) DB_UnMapTable(pot,0);
  if(potvars) DB_UnMapTable(potvars,0);
  if(valve) DB_UnMapTable(valve,0);
  if(vlvvars) DB_UnMapTable(vlvvars,0);
  if(pid) DB_UnMapTable(pid,0);
  if(pto) DB_UnMapTable(pto,0);
  if(td) DB_UnMapTable(td,0);
  if(sigchar) DB_UnMapTable(sigchar,0);
  if(accum) DB_UnMapTable(accum,0);
  if(waveform) DB_UnMapTable(waveform,0);
  if(leadlag) DB_UnMapTable(leadlag,0);
  if(ramp) DB_UnMapTable(ramp,0);
  if(pat) DB_UnMapTable(pat,0);
  if(ddc) DB_UnMapTable(ddc,0);
  if(deadtime) DB_UnMapTable(deadtime,0);
  if(ioaddr) DB_UnMapTable(ioaddr,0);
}
D4DATA *CheckAndOpen(C4CODE *db,char *name)
{
  struct stat stBuff;

#ifdef	USECB63
  db->errFieldName = 0;
  db->errOpen = 0;
  db->errTagName = 0;
  db->readLock = 0;
  db->errCreate = 0;
  db->errGo = 0;	/* 18-apr-1996 */
  db->errOff = 1;	/* 19-Apr-1996 turn off error display */
#else
  db->field_name_error = 0;
  db->open_error = 0;
  db->tag_name_error = 0;
  db->read_lock = 0;
  db->create_error = 0;
  db->go_error = 0;	/* 18-apr-1996 */
  db->off_error = 1;	/* 19-Apr-1996 turn off error display */
#endif	USECB63
#ifndef CB51
  db->wait = 0;
#endif
  if(stat(name,&stBuff) != -1)
    return(d4open(db,name));
  else
    {
      perror("dbFix");
      return((D4DATA *)0);
    }
}
static void showStruct (char *database)
{
    C4CODE	cb;
    D4DATA	*data;
    FIELD4INFO	*fieldInfo;
    TAG4INFO	*indexInfo;
    INDEX4	*index;
    char	pathBuff [256];
    char	resolvedPath [256];
    int		i;
    char	*typePtr;

#ifdef	WIN32
    sprintf (pathBuff, "\\tis4000db\\%s", database);
#else
    sprintf (pathBuff, "/tis4000db/%s", database);
#endif	WIN32

    resolveLinks (pathBuff, resolvedPath);
    d4init(&cb);
    cb.errOpen = 0;
    data = d4open(&cb, pathBuff);

    if (data == NULL)
    {
	fprintf(stderr," Failed to open %s\n", pathBuff);
	return;
    }
    if ((fieldInfo = d4fieldInfo (data)) == NULL)
    {
	fprintf (stderr, "insufficient memory to get field info\n");
	d4close (data);
	return;
    }
    /* print header */
    fprintf (stdout, "Structure for database:  %s\n", resolvedPath);
    fprintf (stdout, "Number of data records:  %ld\n", d4recCount (data));
    fprintf (stdout, "Database structure:\n");
    fprintf (stdout, "Field  Field Name Type         Width    Dec     Index\n");
    i = 0;
    while (fieldInfo[i].name != NULL)
    {
	switch (fieldInfo[i].type)
	{
	    case 'B':
		typePtr = "Binary";
		break;
	    case 'C':
		typePtr = "Character";
		break;
	    case 'D':
		typePtr = "Date";
		break;
	    case 'F':
		typePtr = "Float";
		break;
	    case 'G':
		typePtr = "General";
		break;
	    case 'L':
		typePtr = "Logical";
		break;
	    case 'M':
		typePtr = "Memo";
		break;
	    case 'N':
		typePtr = "Numeric";
		break;
	    default:
		typePtr = "Unknown";
		break;
	}
	fprintf (stdout,"%5d  %-10.10s  %-10.10s  %5d    %3d %-10.10s\n",
			i+1, 
			fieldInfo[i].name,
			typePtr,
			fieldInfo[i].len,
			fieldInfo[i].dec,
			" ");
	i++;
    }
    fprintf (stdout, "** Total                       %5d\n", d4recWidth (data));
    index = d4index (data, NULL);
    if (index != NULL)
    {
	indexInfo = i4tagInfo (index);
	if (indexInfo != NULL)
	{
	    fprintf (stdout, "\nIndex structure\n");
	    fprintf (stdout, "Tag Name    Expression              Filter          Unique Descending\n");
	    i = 0;
	    while (indexInfo[i].name != NULL)
	    {
		fprintf (stdout,"%-10.10s  %s    %s  %d  %d\n",
				indexInfo[i].name,
				indexInfo[i].expression,
				(strlen (indexInfo[i].filter) == 0) ? "    0" : indexInfo[i].filter,
				indexInfo[i].unique,
				indexInfo[i].descending);
		i++;
	    }
	}
    }
    d4close (data);
    return;
}
