/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include	<stdio.h>
#include	<string.h>
#ifdef WIN32
#include	<time.h>
#else
#include	<sys/time.h>
#include	<sys/param.h>
#include	<types.h>
#endif WIN32
#include	<memory.h>
#include	<errno.h>
#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32
#include	"avl.h"
#include	"dbDefs.h"
#include	"dbRecType.h"
#include	"dbRecDes.h"
#include	"choice.h"
#include	"choiceGbl.h"
#include	"devSup.h"
#include	"drvSup.h"
#include	"cvtTable.h"
#include	"recSup.h"
#include	"special.h"
#include	"dbRecords.h"
#include	"dbFldTypes.h"
#include	"descDataTable.h"
#include	"fieldAttributes.h"

#define LIMIT_CONSTANT  0
#define LIMIT_REFERENCE 1
#define LIMIT_SPECIAL  2
#define BIG_STRING_SIZE 4096
#define UDB_SPECIAL_31  31
#define UDB_SPECIAL_32  32

AVL_TREE  *faTree = 0;
static char bigString[BIG_STRING_SIZE];

extern SPECL_PASS udbpass;
extern UDBPVS_PTR udbpvs;
/*
  Initialize the tree and populate it from the database by reading
  PVMAP.DBF for all points and extracting the information to fill
  the PTDATA structure.
*/
int FA_InitAvlTree (typeDesc **descTable)
{
  int stat;
  char name[PVNAME_SZ+1];
  UDBPVS_PTR uptr;
  PTDATA_PTR pData;

  stat = 0;
  if(faTree == 0)
    {
      faTree = avl_init(FA_NameCompare);
      if(FA_XrefFile != 0)
	{
/*
  Build the tree. Loop sequentially through PVMAP
*/
	  d4top(FA_Pvmap);
	  do
	    {
	      sprintf(&name[0],"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,f4str(FA_PvmapNAME));
	      if(FA_AddLeaf(name,descTable) == 0)
		{
		  stat = 1;
		  break;
		}
	    }
	  while(!d4skip(FA_Pvmap,1));
	  /*
	    Process any points (probably UDB's) that could not be completed in
	    pass #1.
	  */
	  udbpass = PASS_2;
	  uptr = udbpvs;
	  while(uptr)
	    {
	      sprintf(&name[0],"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,uptr->pvname);
	      if((pData = FA_FindPoint(name)) != 0)
		FA_AssignUDBValues(pData,descTable);
	      udbpvs = uptr->nextUdb;			/* un-chain and free this node */
	      free(uptr);
	      uptr = udbpvs;
	    }
	  udbpass = PASS_1;
	}
    }
return(stat);
}
PTDATA_PTR FA_AddLeaf(char *pvname,typeDesc **descTable)
{
  PTDATA data;
  PTDATA_PTR pData;
  char tb[20],ts[20];
  char index;
  int typelen;

  if(f4true(deleteFlag))
    return((PTDATA_PTR)-1);
  strcpy(&data.pvname[0],pvname);
  data.timestamp = (unsigned int)atoi(f4str(FA_PvmapTSTP));
  data.crc = (unsigned short)strtol(f4str(FA_PvmapCRC),NULL,16);
  strcpy(ts,f4str(FA_PvmapTYPE));
  typelen = f4len(FA_PvmapTYPE);
  for(index = 1;GET_PRECTYPE(index);index++)
    {
      sprintf(tb,"%-*.*s\0",typelen,typelen,GET_PRECTYPE(index));
      if(!strncmp(ts,tb,typelen))
	break;
    }
  if(index >= dbRecType->number)
    return((PTDATA_PTR)-1);
  if((pData = FA_AddNode(&data)) == 0)
    return((PTDATA_PTR)-1);
  pData->recType = index;
  pData->enumTable = 0;
  pData->numberTable = 0;
  pData->currValue = 0;			/* No current value pointer yet */
  if(descTable[index] == 0)		/* if no entry for this point type then nothing to do */
    return(pData);
  pData->numberTable = (double *)calloc(descTable[index]->tableSize,sizeof(double));
  pData->numTableSize = descTable[index]->tableSize;
  /* put default and/or exception data into the table of doubles */
  FA_AssignValues(pData,descTable);
  return(pData);
}
/*
  This function is used to fetch the data as required by getFieldAttributes.
*/
int FA_FetchData(typeDesc **descTable,char *pv,short index,
		 unsigned short crc,unsigned int tstamp,
		 double *dl,double *dh,double *cl,double *ch,double *pr,
		 char **eStrings,int *numStrs)
{
  char name[PVNAME_SZ+1];
  char *dot,*fld;
  char nm[PVNAME_SZ+FLDNAME_SZ+1];
  PTDATA_PTR data;
  numericField  *exPtr;
  enumField  *ePtr;
  UDBPVS_PTR uptr;
  /*
    set the default return values. These are here in case a point type has no entries in the
    descriptor table.
  */
  *dl = 0.0;
  *dh = 0.0;
  *cl = 0.0;
  *ch = 0.0;
  *pr = 0.0;
  *eStrings = 0;
  *numStrs = 0;
  /*
    if no entry in descTable then done.
  */
  if(descTable[index] == 0)
    return(1);
  /*
    eles find point name in tree.
  */
  strcpy(nm,pv);
  if((dot = strtok(nm,".")) == 0)	/* no dot then name = nm and fld="VAL" */
    {
      dot = nm;
      fld = "VAL";
    }
  else
    {
      fld = strtok(NULL,"\0");
      if(!fld) fld = "VAL";
    }
  sprintf(&name[0],"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,dot);
  /*
    Attempt to find an entry for the given point name in the tree. If FA_FindPoint
    returns a zero (not in pvmap during initialization), then add it to the tree now
    and return the pointer.
  */
  if((data = FA_FindPoint(name)) == 0)
    {
      if(d4seek(FA_Pvmap,name) == 0)
	{
	  udbpass = PASS_1;
	  data = FA_AddLeaf(name,descTable);
	  if(data == (PTDATA_PTR)-1)			/* point has been deleted */
	    return(1);
	  udbpass = PASS_2;
	  uptr = udbpvs;
	  while(uptr)
	    {
	      sprintf(&name[0],"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,uptr->pvname);
	      if((data = FA_FindPoint(name)) != 0)
		FA_AssignUDBValues(data,descTable);
	      udbpvs = uptr->nextUdb;			/* un-chain and free this node */
	      free(uptr);
	      uptr = udbpvs;
	    }
	}
      else
	return(1);
    }
  /*
    Check the date/time and the CRC as passed to this function. If either are
    different, then we must re-do the data in the tree entry without re-doing the
    tree entry itself.
  */
  if((data->crc != crc) || (data->timestamp != tstamp))
    {
      data->crc = crc;
      data->timestamp = tstamp;
      udbpass = PASS_1;
      FA_AssignValues(data,descTable);
      udbpass = PASS_2;
      uptr = udbpvs;
      while(uptr)
	{
	  sprintf(&name[0],"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,uptr->pvname);
	  if((data = FA_FindPoint(name)) != 0)
	    FA_AssignUDBValues(data,descTable);
	  udbpvs = uptr->nextUdb;			/* un-chain and free this node */
	  free(uptr);
	  uptr = udbpvs;
	}
    }
  if(descTable[index]->defaultDisplayHigh)
    {
      switch(descTable[index]->defaultDisplayHigh->limitType)
	{
	case LIMIT_CONSTANT:
	  *dh = descTable[index]->defaultDisplayHigh->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *dh = data->numberTable[descTable[index]->defaultDisplayHigh->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultDisplayLow)
    {
      switch(descTable[index]->defaultDisplayLow->limitType)
	{
	case LIMIT_CONSTANT:
	  *dl = descTable[index]->defaultDisplayLow->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *dl = data->numberTable[descTable[index]->defaultDisplayLow->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultControlHigh)
    {
      switch(descTable[index]->defaultControlHigh->limitType)
	{
	case LIMIT_CONSTANT:
	  *ch = descTable[index]->defaultControlHigh->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *ch = data->numberTable[descTable[index]->defaultControlHigh->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultControlLow)
    {
      switch(descTable[index]->defaultControlLow->limitType)
	{
	case LIMIT_CONSTANT:
	  *cl = descTable[index]->defaultControlLow->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *cl = data->numberTable[descTable[index]->defaultControlLow->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultPrec)
    {
      switch(descTable[index]->defaultPrec->limitType)
	{
	case LIMIT_CONSTANT:
	  *pr = descTable[index]->defaultPrec->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *pr = data->numberTable[descTable[index]->defaultPrec->tableSlot];
	  break;
	}
    }
  /*
    process exceptions to the default data here. Defaults may be overwritten
    by the exception. Do this only for the requested field (could be VAL).
  */
  exPtr = descTable[index]->exceptions;
  while(exPtr)
    {
      if(!strncmp(fld,exPtr->fieldName,FLDNAME_SZ))
	{
	  if(exPtr->displayHigh)
	    {
	      switch(exPtr->displayHigh->limitType)
		{
		case LIMIT_CONSTANT:
		  *dh = exPtr->displayHigh->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *dh = data->numberTable[exPtr->displayHigh->tableSlot];
		  break;
		}
	    }
	  if(exPtr->displayLow)
	    {
	      switch(exPtr->displayLow->limitType)
		{
		case LIMIT_CONSTANT:
		  *dl = exPtr->displayLow->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *dl = data->numberTable[exPtr->displayLow->tableSlot];
		  break;
		}
	    }
	  if(exPtr->controlHigh)
	    {
	      switch(exPtr->controlHigh->limitType)
		{
		case LIMIT_CONSTANT:
		  *ch = exPtr->controlHigh->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *ch = data->numberTable[exPtr->controlHigh->tableSlot];
		  break;
		}
	    }
	  if(exPtr->controlLow)
	    {
	      switch(exPtr->controlLow->limitType)
		{
		case LIMIT_CONSTANT:
		  *cl = exPtr->controlLow->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *cl = data->numberTable[exPtr->controlLow->tableSlot];
		  break;
		}
	    }
	  if(exPtr->precision)
	    {
	      switch(exPtr->precision->limitType)
		{
		case LIMIT_CONSTANT:
		  *pr = exPtr->precision->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *pr = data->numberTable[exPtr->precision->tableSlot];
		  break;
		}
	    }
	}
      exPtr = exPtr->nextException;
    }
  *eStrings = data->enumTable;
  ePtr = descTable[index]->enumList;
  while(ePtr)
    {
      if(!strncmp(fld,ePtr->fieldName,FLDNAME_SZ))
	{
	  *numStrs = ePtr->numStrings;
	  break;
	}
      *eStrings += ePtr->numStrings * (FA_STRING_SIZE+1);
      ePtr = ePtr -> nextEnum;
    }
return(0);
}
/*
  Search tree for occurance of "name". return pointer to PTDATA
  structure or NULL if not found.
*/
PTDATA_PTR FA_FindPoint(char *name)
{
  PTDATA data;
  strcpy(data.pvname,name);
  return((PTDATA_PTR)avl_findsym(faTree,&data));
}

/*
  Set the currValue pointer in the tree to a value. The name will
  always be in the tree because CA wouldn't be calling us if this
  weren't a "made" channel.
*/

void FA_SetValuePtr(char *name, void *ptr)
{
  PTDATA	data;
  PTDATA_PTR	pLeaf;
  char		*dot;

  strcpy(data.pvname, name);
  if ((dot = (char *) strchr(data.pvname, '.')))
      {
      while (dot < (data.pvname + sizeof(data.pvname) - 1))
          *dot++ = ' ';
      *dot = 0;
      }
  pLeaf = (PTDATA_PTR)avl_findsym(faTree, &data);
  if (!pLeaf)
      return;
  pLeaf -> currValue = ptr;
}

/*
  Get the currValue pointer in the tree for a point.
  Null if the name is not found.
*/

void *FA_GetValuePtr(char *name)
{
  PTDATA	data;
  PTDATA_PTR	pLeaf;
  char		*dot;

  strcpy(data.pvname, name);
  if ((dot = (char *) strchr(data.pvname, '.')))
      {
      while (dot < (data.pvname + sizeof(data.pvname) - 1))
	  *dot++ = ' ';
      *dot = 0;
      }
  pLeaf = (PTDATA_PTR)avl_findsym(faTree, &data);
  if (!pLeaf)
      return ((void *) 0);
  return ((void *)(pLeaf -> currValue));
}

/*
  Add new node to the tree. Check for duplicate entries and disallow. Return
  a pointer to the new node or NULL if node can't be added.
*/
PTDATA_PTR FA_AddNode(PTDATA_PTR ptPtr)
{
  PTDATA_PTR newNode,nn;

  if((newNode = (PTDATA_PTR)avl_newnode(sizeof(PTDATA))) == 0)
    return((PTDATA_PTR)0);
  *newNode = *ptPtr;			/* copy contents to new Node */
  if((nn = avl_insert(faTree,newNode)) != NULL)
    {
      avl_freenode(newNode);
      return((PTDATA_PTR)0);
    }
  return((PTDATA_PTR)newNode);
}

/*
  Compare the "pvname" fields in two nodes. Used to enter or find
  specified node.
*/
int FA_NameCompare(void *p1, void *p2)
{
  return(strcmp(((PTDATA_PTR)p1)->pvname,((PTDATA_PTR)p2)->pvname));
}

void FA_PrintNode(char *name)
{
  PTDATA_PTR pData;
  if((pData = FA_FindPoint(name)) != 0)
    {
      printf("Point name %s\n",pData->pvname);
      printf("  Point type %s\n",GET_PRECTYPE(pData->recType));
      printf("  Time stamp %s",ctime((time_t *)&pData->timestamp));
      printf("  CRC        %04X\n",pData->crc);
    }
}
/*
  Get the value for the appropriate field either from the hardConst field
  or by using the reference field to get the value from the dBase files
*/
double FA_GetLimitValue(char *pv,short index,limitStruct *limit)
{
  double doubleValue;

  doubleValue = 0.0;
  if(limit)
    doubleValue = FA_GetDBValue(pv,index,limit->reference);	/* get value from database */
  return(doubleValue);
}
/*
  Get the required value from the dBase files and return it as a double
*/
double FA_GetDBValue(char *pv,short index,char *sdrName)
{
  char dbaseFieldString[11];
  char dbaseFileString[100];
  char dbaseOperandId[21];
  char key[80];
  DATA4 *dbFile;
  FIELD4 *dbFld;
  FIELD4 *nameFld;
  double value;

  value = 0.0;
  sprintf(key,"%-15.15s%-*.*s\0",GET_PRECTYPE(index),FLDNAME_SZ,FLDNAME_SZ,sdrName);	/* make XREF key */
  if(d4seek(FA_XrefFile,key) == 0)			/* find record in XREF table */
    {
      strcpy(dbaseFieldString,f4str(dbaseField));
      FA_removeTrailingSpaces(dbaseFieldString);

      strcpy(key,f4str(dbaseFile));
      FA_removeTrailingSpaces(key);
      sprintf(dbaseFileString,"%s%s\0",FA_DATABASEPATH,key);

      strcpy(dbaseOperandId,f4str(operandID));
      FA_removeTrailingSpaces(dbaseOperandId);

      if((dbFile = getFile(dbaseFileString,index)) != 0)
	{
	  nameFld = d4field(dbFile,"NAME");
	  dbFld = d4field(dbFile,dbaseFieldString);
	  if(!strlen(dbaseOperandId))
	    sprintf(key,"%*.*s",PVNAME_SZ,PVNAME_SZ,pv);
	  else
	    sprintf(key,"%*.*s%s",PVNAME_SZ,PVNAME_SZ,pv,dbaseOperandId);
	  if(d4seek(dbFile,key) == 0)
	    value = f4double(dbFld);		   
	}
    }
return(value);
}
char *FA_GetEnumValue(char *pv,short index,enumField *eptr,int *tSize)
{
  char dbaseFieldString[11];
  char dbaseFileString[100];
  char dbaseOperandId[21];
  char key[80];
  DATA4 *dbFile;
  FIELD4 *dbFld;
  FIELD4 *nameFld;
  char *value;
  int i;

  dbFile = 0;
  value = 0;

  for(i=0;i < eptr->numStrings;i++)
    {
      sprintf(key,"%-15.15s%-*.*s\0",GET_PRECTYPE(index),FLDNAME_SZ,FLDNAME_SZ,&eptr->refField[i][0]);	/* make XREF key */
      if(d4seek(FA_XrefFile,key) == 0)			/* find record in XREF table */
	{
	  strcpy(dbaseFieldString,f4str(dbaseField));
	  FA_removeTrailingSpaces(dbaseFieldString);

	  strcpy(key,f4str(dbaseFile));
	  FA_removeTrailingSpaces(key);
	  sprintf(dbaseFileString,"%s%s\0",FA_DATABASEPATH,key);

	  strcpy(dbaseOperandId,f4str(operandID));
	  FA_removeTrailingSpaces(dbaseOperandId);

	  if((dbFile = getFile(dbaseFileString,index)) != 0)
	    {
	      nameFld = d4field(dbFile,"NAME");
	      dbFld = d4field(dbFile,dbaseFieldString);
	      sprintf(key,"%*.*s%s",PVNAME_SZ,PVNAME_SZ,pv,dbaseOperandId);
	  /*

	    Seek optimization...Don't seek to the same record more than once!

	    If current record in the dbFile is the same as the one I want to
	    seek to then just get the data from the desired field else do the
	    seek. This saves time! From 10 minutes down to 7 minutes on BUD.

	  */
	      if(d4seek(dbFile,key) == 0)
	      	{
	      	value = f4str(dbFld);		/* copy string to bigString include NULL */
	      	FA_removeTrailingSpaces(value);	/* trim trailing spaces */
	      	memcpy(&bigString[*tSize],value,strlen(value)+1);
	      	*tSize += (FA_STRING_SIZE+1)/* strlen(value)+1 */;
	      	}
	    }
	}
    }
  return(&bigString[0]);
}
/*
  Return codebase file handle or 0 if error. fname is fully qualified file name.
  index is the point type index.
*/
DATA4 *getFile(char *fname,short index)
{
DBFILES_PTR temp;
temp = dbFileArray[index];
while(temp)					/* search for file name in list */
  {
    if(!strcmp(temp->dbName,fname))
      break;					/* found it then return with the file handle */
    temp = (DBFILES_PTR)temp->nextFile;			/* else go to next one */
  }
if(temp == 0)					/* not found then add name to list */
  {
    temp = (DBFILES_PTR)calloc(1,sizeof(DBFILES));
    strcpy(temp->dbName,fname);			/* save name here */
    temp->dbFile = d4open(&FA_CodeBase,fname);	/* open the file */
    temp->nextFile = dbFileArray[index];	/* add to list */
    dbFileArray[index] = temp;
  }
return(temp->dbFile);
}
/*
  Assign values to the proper places in this tree node.
*/
void FA_AssignValues(PTDATA_PTR pData,typeDesc **descTable)
{
  short index;
  numericField  *exPtr;
  enumField  *ePtr;
  int totalSize;
  double retValue;	/* return value from special or reference */

  index = pData->recType;
  if(descTable[index] == 0)
    return;
  if(descTable[index]->defaultDisplayHigh)
    {
      if(descTable[index]->defaultDisplayHigh->limitType != LIMIT_CONSTANT)
	{
	  if(descTable[index]->defaultDisplayHigh->limitType == LIMIT_REFERENCE)
	    retValue = FA_GetLimitValue(&pData->pvname[0],index,descTable[index]->defaultDisplayHigh);
	  else if((descTable[index]->defaultDisplayHigh->limitType == LIMIT_SPECIAL) && (dbSpecials[descTable[index]->defaultDisplayHigh->backDoor] != 0))
	    retValue = (*dbSpecials[descTable[index]->defaultDisplayHigh->backDoor])(&pData->pvname[0],index,"VAL",SPECL_DISPHIGH);
	  pData->numberTable[descTable[index]->defaultDisplayHigh->tableSlot] = retValue;
	}
    }
  if(descTable[index]->defaultDisplayLow)
    {
      if(descTable[index]->defaultDisplayLow->limitType != LIMIT_CONSTANT)
	{
	  if(descTable[index]->defaultDisplayLow->limitType == LIMIT_REFERENCE)
	    retValue = FA_GetLimitValue(&pData->pvname[0],index,descTable[index]->defaultDisplayLow);
	  else if((descTable[index]->defaultDisplayLow->limitType == LIMIT_SPECIAL) && (dbSpecials[descTable[index]->defaultDisplayLow->backDoor] != 0))
	    retValue = (*dbSpecials[descTable[index]->defaultDisplayLow->backDoor])(&pData->pvname[0],index,"VAL",SPECL_DISPLOW);
	  pData->numberTable[descTable[index]->defaultDisplayLow->tableSlot] = retValue;
	}
    }
  if(descTable[index]->defaultControlHigh)
    {
      if(descTable[index]->defaultControlHigh->limitType != LIMIT_CONSTANT)
	{
	  if(descTable[index]->defaultControlHigh->limitType == LIMIT_REFERENCE)
	    retValue = FA_GetLimitValue(&pData->pvname[0],index,descTable[index]->defaultControlHigh);
	  else if((descTable[index]->defaultControlHigh->limitType == LIMIT_SPECIAL) && (dbSpecials[descTable[index]->defaultControlHigh->backDoor] != 0))
	    retValue = (*dbSpecials[descTable[index]->defaultControlHigh->backDoor])(&pData->pvname[0],index,"VAL",SPECL_CTRHIGH);
	  pData->numberTable[descTable[index]->defaultControlHigh->tableSlot] = retValue;
	}
    }
  if(descTable[index]->defaultControlLow)
    {
      if(descTable[index]->defaultControlLow->limitType != LIMIT_CONSTANT)
	{
	  if(descTable[index]->defaultControlLow->limitType == LIMIT_REFERENCE)
	    retValue = FA_GetLimitValue(&pData->pvname[0],index,descTable[index]->defaultControlLow);
	  else if((descTable[index]->defaultControlLow->limitType == LIMIT_SPECIAL) && (dbSpecials[descTable[index]->defaultControlLow->backDoor] != 0))
	    retValue = (*dbSpecials[descTable[index]->defaultControlLow->backDoor])(&pData->pvname[0],index,"VAL",SPECL_CTRLOW);
	  pData->numberTable[descTable[index]->defaultControlLow->tableSlot] = retValue;
	}
    }
  if(descTable[index]->defaultPrec)
    {
      if(descTable[index]->defaultPrec->limitType != LIMIT_CONSTANT)
	{
	  if(descTable[index]->defaultPrec->limitType == LIMIT_REFERENCE)
	    retValue = FA_GetLimitValue(&pData->pvname[0],index,descTable[index]->defaultPrec);
	  else if((descTable[index]->defaultPrec->limitType == LIMIT_SPECIAL) && (dbSpecials[descTable[index]->defaultPrec->backDoor] != 0))
	    retValue = (*dbSpecials[descTable[index]->defaultPrec->backDoor])(&pData->pvname[0],index,"VAL",SPECL_PREC);
	  pData->numberTable[descTable[index]->defaultPrec->tableSlot] = retValue;
	}
    }
  /*
    process exceptions to the default data here. Defaults may be overwritten
    by the exception.
    */
  exPtr = descTable[index]->exceptions;
  while(exPtr)
    {
      if(exPtr->displayHigh)
	{
	  if(exPtr->displayHigh->limitType != LIMIT_CONSTANT)
	    {
	      if(exPtr->displayHigh->limitType == LIMIT_REFERENCE)
		retValue = FA_GetLimitValue(&pData->pvname[0],index,exPtr->displayHigh);
	      else if((exPtr->displayHigh->limitType == LIMIT_SPECIAL) && (dbSpecials[exPtr->displayHigh->backDoor] != 0))
		retValue = (*dbSpecials[exPtr->displayHigh->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_DISPHIGH);
	      pData->numberTable[exPtr->displayHigh->tableSlot] = retValue;
	    }
	}
      if(exPtr->displayLow)
	{
	  if(exPtr->displayLow->limitType != LIMIT_CONSTANT)
	    {
	      if(exPtr->displayLow->limitType == LIMIT_REFERENCE)
		retValue = FA_GetLimitValue(&pData->pvname[0],index,exPtr->displayLow);
	      else if((exPtr->displayLow->limitType == LIMIT_SPECIAL) && (dbSpecials[exPtr->displayLow->backDoor] != 0))
		retValue = (*dbSpecials[exPtr->displayLow->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_DISPLOW);
	      pData->numberTable[exPtr->displayLow->tableSlot] = retValue;
	    }
	}
      if(exPtr->controlHigh)
	{
	  if(exPtr->controlHigh->limitType != LIMIT_CONSTANT)
	    {
	      if(exPtr->controlHigh->limitType == LIMIT_REFERENCE)
		retValue = FA_GetLimitValue(&pData->pvname[0],index,exPtr->controlHigh);
	      else if((exPtr->controlHigh->limitType == LIMIT_SPECIAL) && (dbSpecials[exPtr->controlHigh->backDoor] != 0))
		retValue = (*dbSpecials[exPtr->controlHigh->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_CTRHIGH);
	      pData->numberTable[exPtr->controlHigh->tableSlot] = retValue;
	    }
	}
      if(exPtr->controlLow)
	{
	  if(exPtr->controlLow->limitType != LIMIT_CONSTANT)
	    {
	      if(exPtr->controlLow->limitType == LIMIT_REFERENCE)
		retValue = FA_GetLimitValue(&pData->pvname[0],index,exPtr->controlLow);
	      else if((exPtr->controlLow->limitType == LIMIT_SPECIAL) && (dbSpecials[exPtr->controlLow->backDoor] != 0))
		retValue = (*dbSpecials[exPtr->controlLow->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_CTRLOW);
	      pData->numberTable[exPtr->controlLow->tableSlot] = retValue;
	    }
	}
      if(exPtr->precision)
	{
	  if(exPtr->precision->limitType != LIMIT_CONSTANT)
	    {
	      if(exPtr->precision->limitType == LIMIT_REFERENCE)
		retValue = FA_GetLimitValue(&pData->pvname[0],index,exPtr->precision);
	      else if((exPtr->precision->limitType == LIMIT_SPECIAL) && (dbSpecials[exPtr->precision->backDoor] != 0))
		retValue = (*dbSpecials[exPtr->precision->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_PREC);
	      pData->numberTable[exPtr->precision->tableSlot] = retValue;
	    }
	}
      exPtr = exPtr->nextException;
    }
  /*
    check for enumerated strings that are NOT part of the SDR
    */
  ePtr = descTable[index]->enumList;
  totalSize = 0;
  memset(&bigString,BIG_STRING_SIZE,0);				/* this will ensure 2 NULLs at the end */
  pData->numStrings = 0;
  while(ePtr)
    {
      FA_GetEnumValue(&pData->pvname[0],index,ePtr,&totalSize);	/* load-up bigString with enum strings */
      pData->numStrings += ePtr->numStrings;   			/* count the strings */
      ePtr = ePtr->nextEnum;
    }
  if(totalSize > 0)
    {
      if(pData->enumTable == 0)
	pData->enumTable = (char *)calloc(1,totalSize+1);		/* allocate memory for return string */
      else
	{
	  free(pData->enumTable);				/* if pointer already exists free it... */
	  pData->enumTable = (char *)calloc(1,totalSize+1);	/* re-allocate, possibly for a different size */
	}
      memcpy(pData->enumTable,&bigString[0],totalSize+1);	/* copy totalSize bytes + 1 NULL byte */
    }
}
/*
  Fetch data from the tree without checking for CRC or time changes. Also
  the point we are looking for had better br in the tree by now (PASS_2) or
  we return all zeros.
*/
int FA_FetchSpecialData(typeDesc **descTable,char *pv,short index,double *dl,double *dh,double *cl,double *ch)
{
  char name[PVNAME_SZ+1];
  char *dot,*fld;
  char nm[PVNAME_SZ+FLDNAME_SZ+1];
  PTDATA_PTR data;
  numericField  *exPtr;
  /*
    set the default return values. These are here in case a point type has no entries in the
    descriptor table.
  */
  *dl = 0.0;
  *dh = 0.0;
  *cl = 0.0;
  *ch = 0.0;
  /*
    if no entry in descTable then done.
  */
  if(descTable[index] == 0)
    return(1);
  /*
    eles find point name in tree.
  */
  strcpy(nm,pv);
  if((dot = strtok(nm,".")) == 0)	/* no dot then name = nm and fld="VAL" */
    {
      dot = nm;
      fld = "VAL";
    }
  else
    {
      fld = strtok(NULL,"\0");
      if(!fld) fld = "VAL";
    }
  sprintf(&name[0],"%-*.*s\0",PVNAME_SZ,PVNAME_SZ,dot);
  /*
    Find an entry for the given point name in the tree.
  */
  if((data = FA_FindPoint(name)) == 0)
    return(1);
  if(descTable[index]->defaultDisplayHigh)
    {
      switch(descTable[index]->defaultDisplayHigh->limitType)
	{
	case LIMIT_CONSTANT:
	  *dh = descTable[index]->defaultDisplayHigh->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *dh = data->numberTable[descTable[index]->defaultDisplayHigh->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultDisplayLow)
    {
      switch(descTable[index]->defaultDisplayLow->limitType)
	{
	case LIMIT_CONSTANT:
	  *dl = descTable[index]->defaultDisplayLow->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *dl = data->numberTable[descTable[index]->defaultDisplayLow->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultControlHigh)
    {
      switch(descTable[index]->defaultControlHigh->limitType)
	{
	case LIMIT_CONSTANT:
	  *ch = descTable[index]->defaultControlHigh->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *ch = data->numberTable[descTable[index]->defaultControlHigh->tableSlot];
	  break;
	}
    }
  if(descTable[index]->defaultControlLow)
    {
      switch(descTable[index]->defaultControlLow->limitType)
	{
	case LIMIT_CONSTANT:
	  *cl = descTable[index]->defaultControlLow->hardConst;
	  break;
	case LIMIT_SPECIAL:
	case LIMIT_REFERENCE:
	  *cl = data->numberTable[descTable[index]->defaultControlLow->tableSlot];
	  break;
	}
    }
  /*
    process exceptions to the default data here. Defaults may be overwritten
    by the exception. Do this only for the requested field (could be VAL).
  */
  exPtr = descTable[index]->exceptions;
  while(exPtr)
    {
      if(!strncmp(fld,exPtr->fieldName,FLDNAME_SZ))
	{
	  if(exPtr->displayHigh)
	    {
	      switch(exPtr->displayHigh->limitType)
		{
		case LIMIT_CONSTANT:
		  *dh = exPtr->displayHigh->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *dh = data->numberTable[exPtr->displayHigh->tableSlot];
		  break;
		}
	    }
	  if(exPtr->displayLow)
	    {
	      switch(exPtr->displayLow->limitType)
		{
		case LIMIT_CONSTANT:
		  *dl = exPtr->displayLow->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *dl = data->numberTable[exPtr->displayLow->tableSlot];
		  break;
		}
	    }
	  if(exPtr->controlHigh)
	    {
	      switch(exPtr->controlHigh->limitType)
		{
		case LIMIT_CONSTANT:
		  *ch = exPtr->controlHigh->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *ch = data->numberTable[exPtr->controlHigh->tableSlot];
		  break;
		}
	    }
	  if(exPtr->controlLow)
	    {
	      switch(exPtr->controlLow->limitType)
		{
		case LIMIT_CONSTANT:
		  *cl = exPtr->controlLow->hardConst;
		  break;
		case LIMIT_SPECIAL:
		case LIMIT_REFERENCE:
		  *cl = data->numberTable[exPtr->controlLow->tableSlot];
		  break;
		}
	    }
	}
      exPtr = exPtr->nextException;
    }
return(0);
}
/*
  Assign values to the proper places in this UDB tree node. The UDB specials #31 & #32
  contain only exceptions and no enumerated strings.
*/
void FA_AssignUDBValues(PTDATA_PTR pData,typeDesc **descTable)
{
  short index;
  numericField  *exPtr;
  double retValue;	/* return value from special */

  index = pData->recType;
  if(descTable[index] == 0)
    return;
  /*
    process exceptions to the default data here. Defaults may be overwritten
    by the exception.
  */
  exPtr = descTable[index]->exceptions;
  while(exPtr)
    {
      if(exPtr->displayHigh)
	{
	  if((exPtr->displayHigh->backDoor == UDB_SPECIAL_31) ||
	     (exPtr->displayHigh->backDoor == UDB_SPECIAL_32))
	    {
	      retValue = (*dbSpecials[exPtr->displayHigh->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_DISPHIGH);
	      pData->numberTable[exPtr->displayHigh->tableSlot] = retValue;
	    }
	}
      if(exPtr->displayLow)
	{
	  if((exPtr->displayLow->backDoor == UDB_SPECIAL_31) ||
	     (exPtr->displayLow->backDoor == UDB_SPECIAL_32))
	    {
	      retValue = (*dbSpecials[exPtr->displayLow->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_DISPLOW);
	      pData->numberTable[exPtr->displayLow->tableSlot] = retValue;
	    }
	}
      if(exPtr->controlHigh)
	{
	  if((exPtr->controlHigh->backDoor == UDB_SPECIAL_31) ||
	     (exPtr->controlHigh->backDoor == UDB_SPECIAL_32))
	    {
	      retValue = (*dbSpecials[exPtr->controlHigh->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_CTRHIGH);
	      pData->numberTable[exPtr->controlHigh->tableSlot] = retValue;
	    }
	}
      if(exPtr->controlLow)
	{
	  if((exPtr->controlLow->backDoor == UDB_SPECIAL_31) ||
	     (exPtr->controlLow->backDoor == UDB_SPECIAL_32))
	    {
	      retValue = (*dbSpecials[exPtr->controlLow->backDoor])(&pData->pvname[0],index,exPtr->fieldName,SPECL_CTRLOW);
	      pData->numberTable[exPtr->controlLow->tableSlot] = retValue;
	    }
	}
      exPtr = exPtr->nextException;
    }
      return;
}
