/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include	<stdio.h>
#include	<string.h>
#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(USECB63) || defined(WIN32)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#else
#include	<sys/param.h>
#endif WIN32
#include	"dbDefs.h"
#include	"dbRecType.h"
#include	"dbRecDes.h"
#include	"choice.h"
#include	"choiceGbl.h"
#include	"devSup.h"
#include	"drvSup.h"
#include	"cvtTable.h"
#include	"recSup.h"
#include	"special.h"
#include	"dbRecords.h"
#include	"dbFldTypes.h"
#include	"pvmap.h"		/* .../share/src/dataVu */
#include	"descDataTable.h"
#include	"fieldAttributes.h"
#include	"msgMon.h"

void	FA_InitCodeBase (void);
int	FA_InitPvmap (void);
int	FA_InitPdbxref (void);
DBFILES_PTR *dbFileArray;


/**********************************************
*
*    FA_InitDatabase ()
*	called from FA_InitAttributes ()
*
**********************************************/

int	FA_InitDatabase (void)
{
    FA_InitCodeBase ();
    if (FA_InitPvmap ())
	return (1);
    if (FA_InitPdbxref ())
	return (1);
    dbFileArray = (DBFILES_PTR *)calloc(dbRecType->number,sizeof(DBFILES));

    return (0);
}

/**********************************************
*
*    FA_InitCodeBase ()
*	start up codebase stuff
*
**********************************************/

void	FA_InitCodeBase (void)
{
    static int firstTime = 1;

    if (firstTime)
    {
	firstTime = 0;
	d4init (&FA_CodeBase);
#ifdef CB60
	FA_CodeBase.errFieldName = 0;
	FA_CodeBase.errOpen = 0;
	FA_CodeBase.errTagName = 0;
	FA_CodeBase.readLock = 0;
	FA_CodeBase.errCreate = 0;
	FA_CodeBase.errGo = 0;
	FA_CodeBase.errOff = 1;
#else
	FA_CodeBase.field_name_error = 0;
	FA_CodeBase.open_error = 0;
	FA_CodeBase.tag_name_error = 0;
	FA_CodeBase.read_lock = 0;
	FA_CodeBase.create_error = 0;
	FA_CodeBase.go_error = 0;
	FA_CodeBase.off_error = 1;
#endif CB60
    }

    return;
}


/***************************************************************
*
*    FA_InitPvmap ()
*
***************************************************************/

int	FA_InitPvmap (void)
{
    char	pathbuff[MAXPATHLEN];

    sprintf (pathbuff, "%spvmap.dbf",FA_DATABASEPATH);
    if ( (FA_Pvmap = d4open (&FA_CodeBase, pathbuff)) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: failed to open %s\n", pathbuff);
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (FA_PvmapNAME = d4field (FA_Pvmap, "NAME")) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: could not find \"NAME\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (FA_PvmapTYPE = d4field (FA_Pvmap, "TYPE")) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: could not find \"TYPE\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (FA_PvmapDTYP = d4field (FA_Pvmap, "DEVTYPE")) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: could not find \"DEVTYPE\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (FA_PvmapCRC = d4field (FA_Pvmap, "CHECKSUM")) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: could not find \"CHECKSUM\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (FA_PvmapTSTP = d4field (FA_Pvmap, "TIMESTAMP")) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: could not find \"TIMESTAMP\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if((deleteFlag = d4field(FA_Pvmap,"DELETEFLAG")) == 0)
    {
	tisSysLog (0,0,"FA_InitPvmap: could not find \"DELETEFLAG\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    return (0);
}

/***************************************************************
*
*    FA_InitPdbxref ()
*
***************************************************************/

int	FA_InitPdbxref (void)
{
    char	pathbuff[MAXPATHLEN];

    sprintf (pathbuff, "%spdbxref.dbf",FA_DATABASEPATH);
    if ( (FA_XrefFile = d4open (&FA_CodeBase, pathbuff)) == 0)
    {
	tisSysLog (0,0,"FA_InitPdbxref: failed to open %s\n", pathbuff);
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (dbaseField = d4field (FA_XrefFile, "FIELD_NAME")) == 0)
    {
	tisSysLog (0,0,"FA_InitPdbxref: could not find \"FIELD_NAME\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (dbaseFile = d4field (FA_XrefFile, "TABLE")) == 0)
    {
	tisSysLog (0,0,"FA_InitPdbxref: could not find \"TABLE\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    if ( (operandID = d4field (FA_XrefFile, "OPERANDID")) == 0)
    {
	tisSysLog (0,0,"FA_InitPdbxref: could not find \"OPERANDID\"\n");
	d4close_all (&FA_CodeBase);
	return (1);
    }
    return(0);
}
