/*devSoft.c,v 1.1.1.1 1999/08/03 14:42:44 tis4000 Exp */

/****************************************************************/
/*                                                              */
/*   This software is the proprietary property of Tate          */
/*   Integrated Systems, L.P.  Patents and copyrights are       */
/*   pending.                                                   */
/*                                                              */
/*   This software is furnished under a license and may         */
/*   be used and copied only in accordance with the terms       */
/*   of such license and with the inclusion of the above        */
/*   notice. This software or any other copies thereof          */
/*   may not be provided or otherwise made available to         */
/*   any other person. No title to and ownership of the         */
/*   software is hereby transferred.                            */
/*                                                              */
/*   The information in this software is subject to             */
/*   change without notice and should not be construed          */
/*   as a commitment by Tate Integrated Systems L.P. or         */
/*   its third party suppliers.                                 */
/*                                                              */
/*   Tate Integrated Systems, L.P. (TIS) and its third          */
/*   party suppliers assume no responsibility for the           */
/*   use or inability to use any of its software. TIS           */
/*   software is provided "as is" without warranty of           */
/*   any kind and TIS expressly disclaims all implied           */
/*   warranties, including but not limited to the implied       */
/*   warranties of merchantability and fitness for a            */
/*   particular purpose.                                        */
/*                                                              */
/*   Notice: Notwithstanding any other lease or license         */
/*   that may pertain to or accompany the delivery of this      */
/*   computer software, the rights of the Government            */
/*   regarding its use, reproduction, and disclosure are        */
/*   as set forth in Section 52.227-19 of the FARS              */
/*   Computer Software-Restricted Rights clause.                */
/*                                                              */
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : devSoft.c 
 *  AUTHOR                 : emsteel
 *  DATE                   : 23 Dec 93
 *
 *
 * DESCRIPTION: device support for Soft
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 4 Mar 94
 *  REVISION:  AO, BO, MBBO, init_record, if out type is DB_LINK but input
 *             dol is a constant, init rval and val from dol.value.value
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 16 Mar 94
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 94
 *  REVISION: make forward func defs agree with actual func defs
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Apr 94
 *  REVISION: ai_init_record, if inp.type is DB_LINK, then do the dbGetLink
 *                            if okToUse()
 *            ao_init_record, if out.type is DB_LINK and if dol.type is 
 *                            DB_LINK, do the dbGetLink if okToUse
 *            bi_init_record, if inp.type is DB_LINK and if dol.type is 
 *                            DB_LINK, do the dbGetLink if okToUse
 *            read_bi,        have dbGetLink return rval not val
 *            bo_init_record, if out.type is DB_LINK and if dol.type is 
 *                            DB_LINK, do the dbGetLink if okToUse
 *			      also set pbo->mask to 1 to allow inversion
 *			      in pbo process func
 *	      write_bo,       put rval not val
 *            mbbi_init_record, if inp.type is DB_LINK and if dol.type is 
 *                            DB_LINK, do the dbGetLink if okToUse
 *            mbbo_init_record, if out.type is DB_LINK and if dol.type is 
 *                            DB_LINK, do the dbGetLink if okToUse
 *	      so_init_record, added
 *	      si_init_record, if inp.type is DB_LINK then do the dbGetLink
 *			      if okToUse, remove CONSTANT code, not used
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 24 May 94
 *  REVISION: added pto_init_record
 *		write_pto,	arg bit_no was never defined, also added
 *				break out of internal switches, added
 *				default illegal bit no value
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 27 May 94
 *  REVISION:	td_init_record, set mask to 1, and check for valid output
 *				type
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 10 June 94
 *  REVISION:	wf_init_record, if input is CONSTANT then init buffer bptr
 *		by ramping the CONSTANT value by 1
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 June 94
 *  REVISION:	wf_init_record, added DBF_CHAR and DBF_UCHAR to CONSTANT init
 *		read_wf, if input is DB_LINK, shift array elements done 1
 *		and init the last element with the dbGetLink
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 26 Apr 96
 *  REVISION:	added PAT
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 26 Apr 96
 *  REVISION:	added DDC
 *
****************************************************************************/

/****************************************************************************
 *
 * Programmer's Notes:
 *    This is a compendium of all dev**Soft.c files in .../share/src/dev
 *    Where noted, change appropriately to create a new device support
 *    module
 *
 *
****************************************************************************/


#include    <vxWorks.h>
#include    <types.h>
#include    <stdioLib.h>
#include    <string.h>

#include    "alarm.h"
#include    "cvtTable.h"
#include    "dbDefs.h"
#include    "dbCommon.h"
#include    "link.h"
#include    "db_field_log.h"
#include    "dbAccess.h"
#include    "dbFldTypes.h"
#include    "devSup.h"
#include    "special.h"
#include    "cvnTypes.h"   /* defines for conversion types */
#include    "rec/aiRecord.h"
#include    "rec/aoRecord.h"
#include    "rec/biRecord.h"
#include    "rec/boRecord.h"
#include    "rec/ddcRecord.h"
#include    "rec/mbbiRecord.h"
#include    "rec/mbboRecord.h"
#include    "rec/ptoRecord.h"
#include    "rec/stringinRecord.h"
#include    "rec/stringoutRecord.h"
#include    "rec/tdRecord.h"
#include    "rec/waveformRecord.h"
#include    "rec/patRecord.h"
#include    "error.h"
#include    "okToUse.h"

static char *io_name = "Soft";         /* NOTE: change here */

/* Create the dset for devAiSoft */

static long ai_init_record();
static long read_ai();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_ai;
   DEVSUPFUN   special_linconv;
} devAiSoft = {                        /* NOTE: change here */
   6,
   NULL,
   NULL,
   ai_init_record,
   NULL,
   read_ai,
   NULL};

/* Create the dset for devAoSoft */

static long ao_init_record();
static long write_ao();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_ao;
   DEVSUPFUN   special_linconv;
   DEVSUPFUN   readback_ao;
} devAoSoft = {                        /* NOTE: change here */
   7,
   NULL,
   NULL,
   ao_init_record,
   NULL,
   write_ao,
   NULL,
   NULL};

/* Create the dset for devBiSoft */

static long bi_init_record();
static long read_bi();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_bi;
} devBiSoft = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   bi_init_record,
   NULL,
   read_bi};

/* Create the dset for devBoSoft */

static long bo_init_record();
static long write_bo();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_bo;
   DEVSUPFUN   readback_bo;
} devBoSoft = {                        /* NOTE: change here */
   6,
   NULL,
   NULL,
   bo_init_record,
   NULL,
   write_bo,
   NULL};

/* Create the dset for devDdcSoft */

static long ddc_init_record();
static long write_ddc();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_ddc;
} devDdcSoft = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   ddc_init_record,
   NULL,
   write_ddc};

/* Create the dset for devMbbiSoft */

static long mbbi_init_record();
static long read_mbbi();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_mbbi;
} devMbbiSoft = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   mbbi_init_record,
   NULL,
   read_mbbi};

/* Create the dset for devMbboSoft */

static long mbbo_init_record();
static long write_mbbo();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_mbbo;
   DEVSUPFUN   readback_mbbo;
} devMbboSoft = {                      /* NOTE: change here */
   6,
   NULL,
   NULL,
   mbbo_init_record,
   NULL,
   write_mbbo,
   NULL};

/* Create the dset for devPtoSoft */

static long pto_init_record ();
static long write_pto();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_bo;
} devPtoSoft = {                       /* NOTE: change here */
   5,
   NULL,
   NULL,
   pto_init_record,
   NULL,
   write_pto};

/* Create the dset for devSiSoft */

static long si_init_record();
static long read_stringin();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_stringin;
} devSiSoft = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   si_init_record,
   NULL,
   read_stringin};

/* Create the dset for devSoSoft */

static long so_init_record();
static long write_stringout();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_stringout;
} devSoSoft = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   so_init_record,
   NULL,
   write_stringout};

/* Create the dset for devTdSoft */

static long td_init_record();
static long write_td();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_td;
} devTdSoft = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   td_init_record,
   NULL,
   write_td};

/* Create the dset for devWfSoft */

static long wf_init_record();
static long read_wf();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_wf;
} devWfSoft = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   wf_init_record,
   NULL,
   read_wf};

/* Create the dsets for Pat block */

static long pat_init_record ();
static long write_pat ();
static long read_pat ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_pat;
   DEVSUPFUN   special_lincon;
} devPatAiSoft = {                     /* NOTE: change here */
   6,
   NULL,
   NULL,
   pat_init_record,
   NULL,
   read_pat,
   NULL};

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_pat;
} devPatBiSoft = {                     /* NOTE: change here */
   5,
   NULL,
   NULL,
   pat_init_record,
   NULL,
   read_pat};

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_pat;
} devPatBoSoft = {                     /* NOTE: change here */
   5,
   NULL,
   NULL,
   pat_init_record,
   NULL,
   write_pat};

/* used by waveform functions */
/* NOTE: this array must agree with same in recWaveform.c, the values
	 are the size of the datatypes as defined in dbFldTypes.h
*/

static int    sizeofTypes[] = {0, 1, 1, 2, 2, 4, 4, 4, 8, 2};


/********************************** AI ************************************/

static long ai_init_record (pai)
   struct aiRecord   *pai;
{
   char  message[100];
   long  status = 0;
   long  options = 0;
   long  nRequests = 1;

      /* ai.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pai->inp.type)
   {
      case (CONSTANT) :
         pai->val = pai->inp.value.value;
         pai->udf = FALSE;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :

            /* our input link is not available */

         if (!okToUse (&pai -> inp))
            status = -1;
         else
         {
            status = dbGetLink (&pai -> inp.value.db_link, (struct dbCommon *)pai, 
				DBR_DOUBLE, (caddr_t)&pai -> val, 
				&options, &nRequests);
         }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pai->name);
         strcat (message, ": devAi");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }

   return(status);
}

static long read_ai (pai)
   struct aiRecord   *pai;
{
   char  message[100];
   long  status, options, nRequest;

      /* ai.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pai->inp.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         options=0;
         nRequest=1;
         status = dbGetLink (&(pai->inp.value.db_link), (struct dbCommon *)pai, 
			     DBR_DOUBLE, (caddr_t)(&pai->val), 
			     &options, &nRequest);
	 if(status != 0)
         {
            if(pai->nsev<VALID_ALARM)
            {
               pai->nsev = VALID_ALARM;
               pai->nsta = LINK_ALARM;
            }
         }
         else
            pai->udf = FALSE;
         break;

         case (CA_LINK) :
            break;

         default :
            if(pai->nsev<VALID_ALARM)
            {
               pai->nsev = VALID_ALARM;
               pai->nsta = SOFT_ALARM;
               if(pai->stat != SOFT_ALARM)
               {
                  strcpy (message, pai->name);
                  strcat (message, ": devAi");
                  strcat (message, io_name);
                  strcat (message, " (read_ai) Illegal INP field");
                  errMessage (S_db_badField, message);
               }
            }
	    break;
   }
   return(2); /*don't convert*/
}


/************************************ AO *********************************/

static long ao_init_record (pao)
   struct aoRecord   *pao;
{
   char  message[100];
   long  status = 0;
   long  options = 0;
   long  nRequests = 1;

      /* ao.out must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pao->out.type)
   {
      case (CONSTANT) :
         pao -> rval = pao->val = pao->out.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
	 if (pao -> dol.type == CONSTANT)
	     pao -> rval = pao -> val = pao -> dol.value.value;
         else if ( (pao -> dol.type == DB_LINK) &&
		   (pao -> omsl == CLOSED_LOOP) )
         {
            if (!okToUse (&pao ->dol))
               status = -1;
            else
            {
               status = dbGetLink (&pao -> dol.value.db_link, (struct dbCommon *)pao,
                                   DBR_DOUBLE, (caddr_t)(&pao -> val),
                                   &options, &nRequests);
               if (!status)
                  pao -> rval = pao -> val;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pao->name);
         strcat (message, ": devAo");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   
   return(status);
}

static long write_ao (pao)
   struct aoRecord   *pao;
{
   char  message[100];
   long  status = 0;

      /* ao.out must be a CONSTANT, DB_LINK or a CA_LINK*/

   switch (pao->out.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         status = dbPutLink(&pao->out.value.db_link, (struct dbCommon *)pao,
			    DBR_DOUBLE, (caddr_t)(&pao->oval),1L);
         if(status!=0)
         {
            if(pao->nsev<VALID_ALARM)
            {
               pao->nsev = VALID_ALARM;
               pao->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pao->nsev<VALID_ALARM)
         {
            pao->nsev = VALID_ALARM;
            pao->nsta = SOFT_ALARM;
            if(pao->stat!=SOFT_ALARM)
            {
               strcpy (message, pao->name);
               strcat (message, ": devAo");
               strcat (message, io_name);
               strcat (message, " (write_ao) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
	 break;
   }
   return(0);
}


/********************************** BI ***********************************/

static long bi_init_record (pbi)
   struct biRecord   *pbi;
{
   char  message[100];
   long  status    = 0;
   long  options   = 0;
   long  nRequests = 1;

      /* bi.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pbi->inp.type)
   {
      case (CONSTANT) :
         pbi->val = pbi->inp.value.value;
         pbi->udf = FALSE;
         break;

      case (PV_LINK) :
            break;

      case (DB_LINK) :
	    if (!okToUse (&pbi -> inp))
		status = -1;
	    else 
	    {
		status = dbGetLink (&pbi -> inp.value.db_link, (struct dbCommon *)pbi, 
				    DBR_ULONG, (caddr_t)(&pbi -> rval), 
				    &options, &nRequests);
	    }
            break;

      case (CA_LINK) :
            break;

      default :
         strcpy (message, pbi->name);
         strcat (message, ": devBi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return(status);
}

static long read_bi (pbi)
   struct biRecord   *pbi;
{
   char  message[100];
   long  status = 2; 
   long  options; 
   long  nRequest;

      /* bi.inp must be a CONSTANT, DB_LINK, CA_LINK*/

   switch (pbi->inp.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         options=0;
         nRequest=1;
         status = dbGetLink(&(pbi->inp.value.db_link), (struct dbCommon *)pbi,
			    DBR_ULONG, (caddr_t)(&pbi->rval),
			    &options,&nRequest);
         if(status!=0)
         {
            if(pbi->nsev<VALID_ALARM)
            {
               pbi->nsev = VALID_ALARM;
	       pbi->nsta = LINK_ALARM;
            }
         }
         else
	 {
            pbi->udf = FALSE;
	    status = 0;
	 }
         break;

      case (CA_LINK) :
            break;

      default :
            if(pbi->nsev<VALID_ALARM)
            {
               pbi->nsev = VALID_ALARM;
               pbi->nsta = SOFT_ALARM;
               if(pbi->stat != SOFT_ALARM)
               {
                  strcpy (message, pbi->name);
                  strcat (message, ": devBi");
                  strcat (message, io_name);
                  strcat (message, " (read_bi) Illegal INP field");
                  errMessage (S_db_badField, message);
               }
            }
	    break;
   }
   return(status);
}


/*********************************** BO **********************************/

static long bo_init_record (pbo)
   struct boRecord   *pbo;
{
   char  message[100];
   long  status = 0;
   long  options;
   long  nRequests;

      /* bo.out must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pbo->out.type)
   {
      case (CONSTANT) :
         pbo -> rval = pbo->val = pbo->out.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
	 /* setup so REFS put 1 or 0 from rval */
         pbo -> mask = 1;
	 if (pbo -> dol.type == CONSTANT)
	     pbo -> rval = pbo -> val = pbo -> dol.value.value;
	 else if ( (pbo -> dol.type == DB_LINK) &&
		   (pbo -> omsl == CLOSED_LOOP) )
	 {
	    if (!okToUse (&pbo -> dol))
	       status = -1;
	    else
	    {
	       options = 0;
	       nRequests = 1;
	       status = dbGetLink (&pbo -> dol.value.db_link, (struct dbCommon *)pbo, 
				   DBR_USHORT, (caddr_t)(&pbo -> val), 
				   &options, &nRequests);
	    }
	 }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pbo->name);
         strcat (message, ": devBo");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   
   return(status);
}

static long write_bo (pbo)
    struct boRecord   *pbo;
{
   char  message[100];
   long  status;

      /* bo.out must be a CONSTANT, DB_LINK or a CA_LINK*/

   switch (pbo->out.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         status = dbPutLink(&pbo->out.value.db_link, (struct dbCommon *)pbo, 
			    DBR_ULONG, (caddr_t)(&pbo->rval), 1L);
         if(status!=0)
         {
            if(pbo->nsev<VALID_ALARM)
            {
               pbo->nsev = VALID_ALARM;
               pbo->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pbo->nsev<VALID_ALARM)
         {
            pbo->nsev = VALID_ALARM;
            pbo->nsta = SOFT_ALARM;
            if (pbo->stat != SOFT_ALARM)
            {
               strcpy (message, pbo->name);
               strcat (message, ": devBo");
               strcat (message, io_name);
               strcat (message, " (write_bo) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
	 break;
   }
   return(0);
}


/*********************************** DDC ********************************/

static long ddc_init_record (pddc)
   struct ddcRecord   *pddc;
{
   char  message[100];
   long  status = 0;

      /* ddc.out must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pddc->out.type)
   {
      case (CONSTANT) :
         pddc -> rval = pddc->val = pddc->out.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pddc->name);
         strcat (message, ": devDdc");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
   }
   
   return(status);
}

static long write_ddc(pddc)
   struct ddcRecord   *pddc;
{
   char  message[100];
   long  status;

      /* ddc.out must be a CONSTANT, DB_LINK, CA_LINK*/

   switch (pddc->out.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         status = dbPutLink (&pddc->out.value.db_link, (struct dbCommon *)pddc, 
			     DBR_USHORT, (caddr_t)(&pddc->val),1L);
         if(status != 0)
         {
            if(pddc->nsev<VALID_ALARM)
            {
               pddc->nsev = VALID_ALARM;
               pddc->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pddc->nsev<VALID_ALARM)
         {
            pddc->nsev = VALID_ALARM;
            pddc->nsta = SOFT_ALARM;
            if (pddc->stat != SOFT_ALARM)
            {
               strcpy (message, pddc->name);
               strcat (message, ": devDdc");
               strcat (message, io_name);
               strcat (message, " (write_ddc) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/********************************* MBBI *********************************/

static long mbbi_init_record(pmbbi)
   struct mbbiRecord   *pmbbi;
{
   char  message[100];
   long  status = 0;
   long  options;
   long  nRequests;

      /* mbbi.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pmbbi->inp.type)
   {
      case (CONSTANT) :
         pmbbi->val = pmbbi->inp.value.value;
         pmbbi->udf = FALSE;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
	 if (!okToUse (&pmbbi -> inp) )
	    status = -1;
	 else
	 {
	    options = 0;
	    nRequests = 1;

	    status = dbGetLink (&pmbbi -> inp.value.db_link, (struct dbCommon *)pmbbi, 
				DBR_USHORT, (caddr_t)(&pmbbi -> val), 
				&options, &nRequests);
	    pmbbi -> rval = (unsigned long)pmbbi -> val;
	    pmbbi -> udf = FALSE;
	 }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pmbbi->name);
         strcat (message, ": devMbbi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
   }
   return(status);
}

static long read_mbbi (pmbbi)
   struct mbbiRecord   *pmbbi;
{
   char  message[100];
   long  status, options, nRequest;

      /* mbbi.inp must be a CONSTANT, DB_LINK, CA_LINK*/

   switch (pmbbi->inp.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         options=0;
         nRequest=1;
         status = dbGetLink (&(pmbbi->inp.value.db_link), (struct dbCommon *)pmbbi, 
			     DBR_USHORT, (caddr_t)(&pmbbi->val), 
			     &options, &nRequest);
         pmbbi->rval = pmbbi->val;
         if(status!=0)
         {
            if(pmbbi->nsev<VALID_ALARM)
            {
               pmbbi->nsev = VALID_ALARM;
               pmbbi->nsta = LINK_ALARM;
            }
         }
         else
            pmbbi->udf = FALSE;
         return (status);
         break;

      case (CA_LINK) :
         break;

      default :
         if(pmbbi->nsev<VALID_ALARM)
         {
            pmbbi->nsev = VALID_ALARM;
            pmbbi->nsta = SOFT_ALARM;
            if(pmbbi->stat!=SOFT_ALARM)
            {
               strcpy (message, pmbbi->name);
               strcat (message, ": devMbbi");
               strcat (message, io_name);
               strcat (message, " (read_mbbi) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(2);
}


/*********************************** MBBO ********************************/

static long mbbo_init_record (pmbbo)
   struct mbboRecord   *pmbbo;
{
   char  message[100];
   long  status = 0;
   long  options;
   long  nRequests;

      /* mbbo.out must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pmbbo->out.type)
   {
      case (CONSTANT) :
         pmbbo -> rval = pmbbo->val = pmbbo->out.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
	 if (pmbbo -> dol.type == CONSTANT)
	     pmbbo -> rval = pmbbo -> val = pmbbo -> dol.value.value;
	 else if ( (pmbbo -> dol.type == DB_LINK) &&
		   (pmbbo -> omsl == CLOSED_LOOP) )
	 {
	    if (!okToUse (&pmbbo -> dol) )
		status = -1;
	    else
	    {
		options = 0;
		nRequests = 1;

		status = dbGetLink (&pmbbo -> dol.value.db_link, (struct dbCommon *)pmbbo,
				    DBR_USHORT, (caddr_t)(&pmbbo -> val), 
				    &options, &nRequests);
	    }
	 }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pmbbo->name);
         strcat (message, ": devMbbo");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
   }
   
   return(status);
}

static long write_mbbo(pmbbo)
   struct mbboRecord   *pmbbo;
{
   char  message[100];
   long  status;

      /* mbbo.out must be a CONSTANT, DB_LINK, CA_LINK*/

   switch (pmbbo->out.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         status = dbPutLink (&pmbbo->out.value.db_link, (struct dbCommon *)pmbbo, 
			     DBR_USHORT, (caddr_t)(&pmbbo->val),1L);
         if(status!=0)
         {
            if(pmbbo->nsev<VALID_ALARM)
            {
               pmbbo->nsev = VALID_ALARM;
               pmbbo->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pmbbo->nsev<VALID_ALARM)
         {
            pmbbo->nsev = VALID_ALARM;
            pmbbo->nsta = SOFT_ALARM;
            if (pmbbo->stat != SOFT_ALARM)
            {
               strcpy (message, pmbbo->name);
               strcat (message, ": devMbbo");
               strcat (message, io_name);
               strcat (message, " (write_mbbo) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** PTO **********************************/

static long pto_init_record (ppto, bit_no)
    struct ptoRecord	*ppto;
    unsigned int	bit_no;
{
    long		status = 0;
    char		message[128];
    unsigned long	value;
    long		options = 0;
    long		nRequests = 1;

    switch (bit_no)
    {
	case 0:
	    switch (ppto -> pout.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    if (!okToUse(&ppto -> pout) )
			status = -1;
		    else
		    {
			status = dbGetLink (&ppto -> pout.value.db_link,
					    (struct dbCommon *)ppto, 
					    DBR_ULONG, (caddr_t)&value,
					    &options, &nRequests);
			if (!status)
			    ppto -> rbv = value;
		    }
		    ppto -> pmsk = 1;
		    break;
	
		case CA_LINK:
		    break;

		default:
		    strcpy (message, ppto->name);
		    strcat (message, ": devPto");
		    strcat (message, io_name);
		    strcat (message," (init_record) Illegal POS OUT field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }
	    break;

	case 1:
	    switch (ppto -> nout.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    if (!okToUse (&ppto -> nout) )
			status = -1;
		    else
		    {
			status = dbGetLink (&ppto -> nout.value.db_link,
					    (struct dbCommon *)ppto, 
					    DBR_ULONG, (caddr_t)&value,
					    &options, &nRequests);
			if (!status)
			    ppto -> rbv = value;
		    }
		    ppto -> nmsk = 1;
		    break;
	
		case CA_LINK:
		    break;

		default:
		    strcpy (message, ppto->name);
		    strcat (message, ": devPto");
		    strcat (message, io_name);
		    strcat (message," (init_record) Illegal NEG OUT field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }
	    break;

	default:
	    strcpy (message, ppto->name);
	    strcat (message, ": devPto");
	    strcat (message, io_name);
	    strcat (message," (init_record) Illegal OUT field");
	    errMessage (S_db_badField, message);
	    status = S_db_badField;
	    break;
    }

    return (status);
}

static long write_pto (ppto, bit_no)
   struct ptoRecord   *ppto;
   unsigned int	      bit_no;
{
   char  message[100];
   long  status;

      /* pto.out must be a CONSTANT, DB_LINK or a CA_LINK*/


   switch (bit_no)
   {

      case 0:
         switch (ppto->pout.type)
         {
            case (CONSTANT) :
               break;

            case (DB_LINK) :
               status = dbPutLink (&ppto->pout.value.db_link, 
				   (struct dbCommon *)ppto,
                                   DBR_LONG, (caddr_t)(&ppto -> rbv), 1L);
               if (status != 0)
               {
                  if (ppto->nsev < VALID_ALARM)
                  {
                     ppto->nsev = VALID_ALARM;
                     ppto->nsta = LINK_ALARM;
                  }
               }
               break;

            case (CA_LINK) :
               break;

            default :
               if ( ppto->nsev < VALID_ALARM)
               {
                  ppto->nsev = VALID_ALARM;
                  ppto->nsta = SOFT_ALARM;
                  if (ppto->stat != SOFT_ALARM)
                  {
                     strcpy (message, ppto->name);
                     strcat (message, ": devPto");
                     strcat (message, io_name);
                     strcat (message, " (write_pto) Illegal POS OUT field");
                     errMessage (S_db_badField, message);
                  }
               }
         }
	 break;

      case 1:
         switch (ppto->nout.type)
         {

            case (CONSTANT) :
               break;

            case (DB_LINK) :
               status = dbPutLink (&ppto->nout.value.db_link, 
				   (struct dbCommon *)ppto,
                                   DBR_LONG, (caddr_t)(&ppto -> rbv), 1L);

               if (status != 0)
               {
                  if (ppto->nsev < VALID_ALARM)
                  {
                     ppto->nsev = VALID_ALARM;
                     ppto->nsta = LINK_ALARM;
                  }
               }
               break;

            case (CA_LINK) :
               break;

            default :
               if ( ppto->nsev < VALID_ALARM)
               {
                  ppto->nsev = VALID_ALARM;
                  ppto->nsta = SOFT_ALARM;
                  if (ppto->stat != SOFT_ALARM)
                  {
                     strcpy (message, ppto->name);
                     strcat (message, ": devPto");
                     strcat (message, io_name);
                     strcat (message, " (write_pto) Illegal NEG OUT field");
                     errMessage (S_db_badField, message);
                  }
               }
         }
	 break;

      default:
         if ( ppto->nsev < VALID_ALARM)
         {
            ppto->nsev = VALID_ALARM;
            ppto->nsta = SOFT_ALARM;
            if (ppto->stat != SOFT_ALARM)
            {
               strcpy (message, ppto->name);
               strcat (message, ": devPto");
               strcat (message, io_name);
               strcat (message, " (write_pto) Illegal bit_no");
               errMessage (S_db_badField, message);
            }
         }
   }

   return (0);
}


/*********************************** SI ************************************/

static long si_init_record (pstringin)
   struct stringinRecord   *pstringin;
{
   char  message[100];
   long  status = 0;
   long  options = 0;
   long  nRequests = 1;

      /* stringin.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pstringin->inp.type)
   {
      case (CONSTANT) :
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :

            /* our input link is not available */

         if (!okToUse (&pstringin -> inp))
            status = -1;
         else
         {
             status = dbGetLink (&(pstringin->inp.value.db_link), 
				 (struct dbCommon *)pstringin,
				 DBR_STRING, pstringin->val,
				 &options,&nRequests);
         }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pstringin->name);
         strcat (message, ": devSi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
   }
   return(status);
}

static long read_stringin (pstringin)
   struct stringinRecord   *pstringin;
{
   char  message[100];
   long  status = 0, options, nRequest;

      /* stringin.inp must be a CONSTANT, DB_LINK or a CA_LINK*/

   switch (pstringin->inp.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         options=0;
         nRequest=1;
         status = dbGetLink (&(pstringin->inp.value.db_link), 
			     (struct dbCommon *)pstringin,
                             DBR_STRING, pstringin->val,
			     &options,&nRequest);
         if(status!=0)
         {
            if(pstringin->nsev<VALID_ALARM)
            {
               pstringin->nsev = VALID_ALARM;
               pstringin->nsta = LINK_ALARM;
            }
         }
         else
            pstringin->udf = FALSE;
         break;

      case (CA_LINK) :
         break;

      default :
         if(pstringin->nsev<VALID_ALARM)
         {
            pstringin->nsev = VALID_ALARM;
            pstringin->nsta = SOFT_ALARM;
            if(pstringin->stat!=SOFT_ALARM)
            {
               strcpy (message, pstringin->name);
               strcat (message, ": devSi");
               strcat (message, io_name);
               strcat (message, " (read_stringin) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(status);
}


/*********************************** SO **********************************/

static long so_init_record (pstringout)
   struct stringoutRecord   *pstringout;
{
   char  message[100];
   long  status = 0;
   long  options = 0;
   long  nRequests = 1;

      /* stringout.out must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   switch (pstringout->out.type)
   {
      case (CONSTANT) :
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
         if ( (pstringout -> dol.type == DB_LINK) &&
	      (pstringout -> omsl == CLOSED_LOOP) )
         {
            if (!okToUse (&pstringout ->dol))
               status = -1;
            else
            {
               status = dbGetLink (&pstringout -> dol.value.db_link, 
				   (struct dbCommon *)pstringout,
                                   DBR_STRING, pstringout -> val,
                                   &options, &nRequests);
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pstringout->name);
         strcat (message, ": devSo");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   
   return(status);
}

static long write_stringout(pstringout)
   struct stringoutRecord   *pstringout;
{
   char  message[100];
   long  status;

      /* stringout.out must be a CONSTANT, DB_LINK or a CA_LINK*/

   switch (pstringout->out.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         status = dbPutLink (&pstringout->out.value.db_link, 
			     (struct dbCommon *)pstringout,
                             DBR_STRING, pstringout->val,1L);
         if(status!=0)
         {
            if(pstringout->nsev<VALID_ALARM)
            {
               pstringout->nsev = VALID_ALARM;
               pstringout->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pstringout->nsev<VALID_ALARM)
         {
            pstringout->nsev = VALID_ALARM;
            pstringout->nsta = SOFT_ALARM;
            if(pstringout->stat != SOFT_ALARM)
            {
               strcpy (message, pstringout->name);
               strcat (message, ": devSo");
               strcat (message, io_name);
               strcat (message, " (write_stringout) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** TD ***********************************/

static long td_init_record(ptd)
   struct tdRecord   *ptd;
{
   char  message[100];
   long  status = 0;

      /* td.out must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

   ptd -> mask = 1;
   switch (ptd->out.type)
   {
      case (CONSTANT) :
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, ptd->name);
         strcat (message, ": devTd");
         strcat (message, io_name);
         strcat (message," (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         status = S_db_badField;
	 break;
   }
   return(status);
}

static long write_td(ptd)
   struct tdRecord   *ptd;
{
   char  message[100];
   long  status;

      /* td.out must be a CONSTANT, DB_LINK or a CA_LINK*/

   switch (ptd->out.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         status = dbPutLink (&ptd->out.value.db_link, 
			     (struct dbCommon *)ptd, DBR_USHORT,
                             (caddr_t)(&ptd->val), 1L);
         if(status!=0)
         {
            if(ptd->nsev<VALID_ALARM)
            {
               ptd->nsev = VALID_ALARM;
               ptd->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(ptd->nsev<VALID_ALARM)
         {
            ptd->nsev = VALID_ALARM;
            ptd->nsta = SOFT_ALARM;
            if(ptd->stat != SOFT_ALARM)
            {
               strcpy (message, ptd->name);
               strcat (message, ": devTd");
               strcat (message, io_name);
               strcat (message, " (write_td) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** WF ***********************************/

static long wf_init_record (pwf)
   struct waveformRecord   *pwf;
{
    long	status = 0;
    char	message[100];

    /* NOTE: the array bptr is allocated with calloc, which 
	     will init bptr to zeros
    */

    char	*ptr = (char *)pwf -> bptr;
    float	value = pwf -> inp.value.value;
    int		offset;
    unsigned long	elem;

      /* wf.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK*/

    switch (pwf->inp.type)
    {
	/* for input type of CONSTANT, we are ramp initializing the 
	   waveforms buffer starting at the CONSTANT value
	*/
	case (CONSTANT) :
	    pwf->nord = 0;

	    /* we skip DBF_STRING */

	    if (pwf -> ftvl != DBF_STRING)
	    {
		pwf -> nord = pwf -> nelm;
		for (elem = 0; elem < pwf -> nelm; elem++)
		{
		    offset = elem * sizeofTypes [pwf -> ftvl];
		    switch (pwf -> ftvl)
		    {
			case DBF_CHAR:
			    *(ptr + offset) = (char)value;
			    break;

			case DBF_UCHAR:
			    *(ptr + offset) = (unsigned char)value;
			    break;

			case DBF_SHORT:
			    *((short *)(ptr + offset)) = (short)value;
			    break;

			case DBF_USHORT:
			case DBR_ENUM:
			    *((unsigned short *)(ptr + offset)) = (unsigned short)value;
			    break;

			case DBF_LONG:
			    *((long *)(ptr + offset)) = (long)value;
			    break;

			case DBF_ULONG:
			    *((unsigned long *)(ptr + offset)) = (unsigned long)value;
			    break;

			case DBF_FLOAT:
			    *((float *)(ptr + offset)) = value;
			    break;

			case DBF_DOUBLE:
			    *((double *)(ptr + offset)) = (double)value;
			    break;
		    }
		    value++;
		}
	    }
	    break;

	case (PV_LINK) :
	    break;

	case (DB_LINK) :
	    break;

	case (CA_LINK) :
	    break;

	default :
	    strcpy (message, pwf->name);
	    strcat (message, ": devWf");
	    strcat (message, io_name);
	    strcat (message, " (init_record) Illegal INP field");
	    errMessage (S_db_badField, message);
	    status = S_db_badField;
	    break;
    }
    return(status);
}

static long read_wf (pwf)
   struct waveformRecord   *pwf;
{
    char	message[100];
    long	options, nRequest;
    char	*ptr = (char *)pwf -> bptr;
    int		offset1, offset2;
    int		a_string = 0;
    unsigned long	elem;

      /* wf.inp must be a CONSTANT, DB_LINK or a CA_LINK*/

    switch (pwf->inp.type)
    {
	case (CONSTANT) :
	    break;

	case (DB_LINK) :

	    /* input type DB_LINK, we will shift elements in array 
	       done 1, and init the last element with the dbGetLink()
	    */
	    /* we skip DBF_STRING */

	    if (pwf -> ftvl > DBF_STRING)
	    {
		for (elem = 1; elem < pwf -> nelm; elem++)
		{
		    offset1 = (elem - 1) * sizeofTypes [pwf -> ftvl];
		    offset2 = elem * sizeofTypes [pwf -> ftvl];
		    switch (pwf -> ftvl)
		    {
			case DBF_CHAR:
			case DBF_UCHAR:
			    *(ptr + offset1) = *(ptr + offset2);
			    *(ptr + offset2) = 0;
			    break;

			case DBF_SHORT:
			    *((short *)(ptr + offset1)) = 
						*((short *)(ptr + offset2));
			    *((short *)(ptr + offset2)) = 0;
			    break;

			case DBF_USHORT:
			case DBR_ENUM:
			    *((unsigned short *)(ptr + offset1)) = 
					*((unsigned short *)(ptr + offset2));
			    *((unsigned short *)(ptr + offset2)) = 0;
			    break;

			case DBF_LONG:
			    *((long *)(ptr + offset1)) = 
						*((long *)(ptr + offset2));
			    *((long *)(ptr + offset2)) = 0;
			    break;

			case DBF_ULONG:
			    *((unsigned long *)(ptr + offset1)) = 
					*((unsigned long *)(ptr + offset2));
			    *((unsigned long *)(ptr + offset2)) = 0;
			    break;

			case DBF_FLOAT:
			    *((float *)(ptr + offset1)) = 
						*((float *)(ptr + offset2));
			    *((float *)(ptr + offset2)) = 0;
			    break;

			case DBF_DOUBLE:
			    *((double *)(ptr + offset1)) = 
						*((double *)(ptr + offset2));
			    *((double *)(ptr + offset2)) = 0;
			    break;
		    }
		}
		nRequest = 1;
		offset1 = (pwf -> nelm - 1) * sizeofTypes [pwf -> ftvl];
		ptr = ((char *)pwf -> bptr) + offset1;
	    }
	      /* we are a string, just get entire string */
	    else
	    {
		nRequest = pwf -> nelm;
		ptr = (char *)pwf -> bptr;
		a_string = 1;
	    }
	    options = 0;
	    if (dbGetLink (&(pwf->inp.value.db_link), 
			   (struct dbCommon *)pwf, pwf->ftvl,
			    ptr, &options, &nRequest) != 0)
	    {
		if(pwf->nsev < VALID_ALARM)
		{
		    pwf->nsev = VALID_ALARM;
		    pwf->nsta = LINK_ALARM;
		}
	    }
	    if(a_string)
		pwf -> nord = nRequest;
	    else
		pwf -> nord = pwf -> nelm;
	    break;

	case (CA_LINK) :
	    break;

	default :
	    if(pwf->nsev<VALID_ALARM)
	    {
		pwf->nsev = VALID_ALARM;
		pwf->nsta = SOFT_ALARM;
		if(pwf->stat != SOFT_ALARM)
		{
		    strcpy (message, pwf->name);
		    strcat (message, ": devWf");
		    strcat (message, io_name);
		    strcat (message, " (read_wf) Illegal INP field");
		    errMessage (S_db_badField, message);
		}
	    }
    }
    return(0);
}



/********************************** PAT ************************************/
/*
NOTE:	which_one values
		0 = POS OUTPUT
		1 = NEG OUTPUT
		2 = REMOTE/LOCAL
		3 = INPUT
*/

static long pat_init_record (ppat, which_one)
    struct patRecord	*ppat;
    int			which_one;
{
    long	status = 0;
    long	options = 0;
    long	nRequests = 1;
    char	message[128];

    switch (which_one)
    {
	case 0:
	    switch (ppat -> pout.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    if (!okToUse (&ppat -> pout) )
			status = -1;
		    ppat -> pmsk = 1;
		    break;

		case CA_LINK:
		    break;

		default:
		    strcpy (message, ppat -> name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, "(init_record) Illegal POS OUT field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }
	    break;

	case 1:
	    switch (ppat -> nout.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    if (!okToUse (&ppat -> nout) )
			status = -1;
		    ppat -> nmsk = 1;
		    break;

		case CA_LINK:
		    break;

		default:
		    strcpy (message, ppat -> name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, "(init_record) Illegal NEG OUT field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }
	    break;

	case 2:
	    switch (ppat -> rlin.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    if (!okToUse (&ppat -> rlin) )
			status = -1;
		    else
		    {
			ppat -> rmsk = 1;
			status = dbGetLink (&ppat -> rlin.value.db_link, 
					    (struct dbCommon *)ppat,
					    DBR_ULONG, (caddr_t)(&ppat -> rlrv), 
					    &options, &nRequests);
		    }
		    break;

		case CA_LINK:
		    break;

		default:
		    strcpy (message, ppat -> name);
		    strcat (message, ": devPat");
		    strcat (message, " (pat_init_record) Illegal R/L field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }
	    break;

	case 3:
	    switch (ppat -> inp.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    if (!okToUse (&ppat -> inp) )
			status = -1;
		    else
		    {
			status = dbGetLink (&ppat -> inp.value.db_link, 
					    (struct dbCommon *)ppat,
					    DBR_DOUBLE, (caddr_t)(&ppat -> val), 
					    &options, &nRequests);
		    }
		    break;

		case CA_LINK:
		    break;

		default:
		    strcpy (message, ppat -> name);
		    strcat (message, ": devPat");
		    strcat (message, " (pat_init_record) Illegal R/L field");
		    errMessage (S_db_badField, message);
		    status = S_db_badField;
		    break;
	    }
	    break;

	default:
	    strcpy (message, ppat -> name);
	    strcat (message, ": devPat");
	    strcat (message, io_name);
	    strcat (message, " (pat_init_record) Illegal field");
	    errMessage (S_db_badField, message);
	    status = S_db_badField;
	    break;
    }

    return (status);
}

static long write_pat (ppat, which_one)
    struct patRecord	*ppat;
    int			which_one;
{
    long	status = 0;
    char	message[128];

    switch (which_one)
    {
	case 0:
	    switch (ppat -> pout.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    status = dbPutLink (&ppat -> pout.value.db_link, 
					(struct dbCommon *)ppat,
					DBR_ULONG, (caddr_t)(&ppat -> rbv), 1L);
		    if (status != 0)
		    {
			if (ppat -> nsev < VALID_ALARM)
			{
			    ppat -> nsev = VALID_ALARM;
			    ppat -> nsta = LINK_ALARM;
			}
		    }
		    break;

		case CA_LINK:
		    break;

		default:
		    if ( ppat->nsev < VALID_ALARM)
		    {
			ppat->nsev = VALID_ALARM;
			ppat->nsta = SOFT_ALARM;
			if (ppat->stat != SOFT_ALARM)
			{
			    strcpy (message, ppat->name);
			    strcat (message, ": devPat");
			    strcat (message, io_name);
			    strcat (message, " (write_pat) Illegal POS OUT field");
			    errMessage (S_db_badField, message);
			}
		    }
		    break;
	    }
	    break;

	case 1:
	    switch (ppat -> nout.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    status = dbPutLink (&ppat -> nout.value.db_link, 
					(struct dbCommon *)ppat,
					DBR_ULONG, (caddr_t)(&ppat -> rbv), 1L);
		    if (status != 0)
		    {
			if (ppat -> nsev < VALID_ALARM)
			{
			    ppat -> nsev = VALID_ALARM;
			    ppat -> nsta = LINK_ALARM;
			}
		    }
		    break;

		case CA_LINK:
		    break;

		default:
		    if ( ppat->nsev < VALID_ALARM)
		    {
			ppat->nsev = VALID_ALARM;
			ppat->nsta = SOFT_ALARM;
			if (ppat->stat != SOFT_ALARM)
			{
			    strcpy (message, ppat->name);
			    strcat (message, ": devPat");
			    strcat (message, io_name);
			    strcat (message, " (write_pat) Illegal NEG OUT field");
			    errMessage (S_db_badField, message);
			}
		    }
		    break;
	    }
	    break;

	default:
	    if ( ppat->nsev < VALID_ALARM)
	    {
		ppat->nsev = VALID_ALARM;
		ppat->nsta = SOFT_ALARM;
		if (ppat->stat != SOFT_ALARM)
		{
		    strcpy (message, ppat->name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, " (write_pat) Illegal type");
		    errMessage (S_db_badField, message);
		}
	    }
    }

    return (status);
}

static long read_pat (ppat, which_one)
    struct patRecord	*ppat;
    int			which_one;
{
    long	status = 0;
    long	options = 0;
    long	nRequests = 1;
    char	message[128];

    switch (which_one)
    {
	case 2:		/* REMOTE/LOCAL switch */
	    switch (ppat -> rlin.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    status = dbGetLink (&ppat -> rlin.value.db_link, 
					(struct dbCommon *)ppat,
					DBR_ULONG, (caddr_t)(&ppat -> rlrv), 
					&options, &nRequests);
		    if (status != 0)
		    {
			if (ppat -> nsev < VALID_ALARM)
			{
			    ppat -> nsev = VALID_ALARM;
			    ppat -> nsta = LINK_ALARM;
			}
		    }
		    break;

		case CA_LINK:
		    break;

		default:
		    if ( ppat->nsev < VALID_ALARM)
		    {
			ppat->nsev = VALID_ALARM;
			ppat->nsta = SOFT_ALARM;
			if (ppat->stat != SOFT_ALARM)
			{
			    strcpy (message, ppat->name);
			    strcat (message, ": devPat");
			    strcat (message, io_name);
			    strcat (message, " (read_pat) Illegal REMOTE/LOCAL field");
			    errMessage (S_db_badField, message);
			}
		    }
		    break;
	    }
	    break;

	case 3:		/* INPUT, FEEDBACK */
	    switch (ppat -> inp.type)
	    {
		case CONSTANT:
		    break;

		case DB_LINK:
		    status = dbGetLink (&ppat -> inp.value.db_link, 
					(struct dbCommon *)ppat,
					DBR_DOUBLE, (caddr_t)(&ppat -> val), 
					&options, &nRequests);
		    if (status != 0)
		    {
			if (ppat -> nsev < VALID_ALARM)
			{
			    ppat -> nsev = VALID_ALARM;
			    ppat -> nsta = LINK_ALARM;
			}
		    }
		    break;

		case CA_LINK:
		    break;

		default:
		    if ( ppat->nsev < VALID_ALARM)
		    {
			ppat->nsev = VALID_ALARM;
			ppat->nsta = SOFT_ALARM;
			if (ppat->stat != SOFT_ALARM)
			{
			    strcpy (message, ppat->name);
			    strcat (message, ": devPat");
			    strcat (message, io_name);
			    strcat (message, " (read_pat) Illegal INPUT field");
			    errMessage (S_db_badField, message);
			}
		    }
		    break;
	    }
	    break;

	default:
	    if ( ppat->nsev < VALID_ALARM)
	    {
		ppat->nsev = VALID_ALARM;
		ppat->nsta = SOFT_ALARM;
		if (ppat->stat != SOFT_ALARM)
		{
		    strcpy (message, ppat->name);
		    strcat (message, ": devPat");
		    strcat (message, io_name);
		    strcat (message, " (write_pat) Illegal type");
		    errMessage (S_db_badField, message);
		}
	    }
	    break;
    }

    return (status);
}

