/*devSoftRaw.c,v 1.1.1.1 1999/08/03 14:42:44 tis4000 Exp */

/****************************************************************/
/*                                                              */
/*   This software is the proprietary property of Tate          */
/*   Integrated Systems, L.P.  Patents and copyrights are       */
/*   pending.                                                   */
/*                                                              */
/*   This software is furnished under a license and may         */
/*   be used and copied only in accordance with the terms       */
/*   of such license and with the inclusion of the above        */
/*   notice. This software or any other copies thereof          */
/*   may not be provided or otherwise made available to         */
/*   any other person. No title to and ownership of the         */
/*   software is hereby transferred.                            */
/*                                                              */
/*   The information in this software is subject to             */
/*   change without notice and should not be construed          */
/*   as a commitment by Tate Integrated Systems L.P. or         */
/*   its third party suppliers.                                 */
/*                                                              */
/*   Tate Integrated Systems, L.P. (TIS) and its third          */
/*   party suppliers assume no responsibility for the           */
/*   use or inability to use any of its software. TIS           */
/*   software is provided "as is" without warranty of           */
/*   any kind and TIS expressly disclaims all implied           */
/*   warranties, including but not limited to the implied       */
/*   warranties of merchantability and fitness for a            */
/*   particular purpose.                                        */
/*                                                              */
/*   Notice: Notwithstanding any other lease or license         */
/*   that may pertain to or accompany the delivery of this      */
/*   computer software, the rights of the Government            */
/*   regarding its use, reproduction, and disclosure are        */
/*   as set forth in Section 52.227-19 of the FARS              */
/*   Computer Software-Restricted Rights clause.                */
/*                                                              */
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : devSoftRaw.c
 *  AUTHOR                 : emsteel
 *  DATE                   : 22 Dec 93
 *
 *
 * DESCRIPTION: device support for SoftRaw
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 94
 *  REVISION:  make forward func def agree with actual func def
 *
****************************************************************************/

/****************************************************************************
 *
 * Programmer's Notes:
 *    This is a compendium of all dev**SoftRaw.c files in .../share/src/dev
 *    Where noted, change appropriately to create a new device support
 *    module
 *
 *
****************************************************************************/


#include    <vxWorks.h>
#include    <types.h>
#include    <stdioLib.h>
#include    <string.h>

#include    "alarm.h"
#include    "cvtTable.h"
#include    "dbDefs.h"
#include    "dbCommon.h"
#include    "link.h"
#include    "db_field_log.h"
#include    "dbAccess.h"
#include    "devSup.h"
#include    "special.h"
#include    "rec/aiRecord.h"
#include    "rec/biRecord.h"
#include    "rec/mbbiRecord.h"
#include    "error.h"

static char *io_name = "SoftRaw";         /* NOTE: change here */

/* Create the dset for devAiSoftRaw */

static long ai_init_record();
static long read_ai();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_ai;
   DEVSUPFUN   special_linconv;
} devAiSoftRaw = {                         /* NOTE: change here */
   6,
   NULL,
   NULL,
   ai_init_record,
   NULL,
   read_ai,
   NULL};

/* Create the dset for devBiSoftRaw */

static long bi_init_record();
static long read_bi();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_bi;
} devBiSoftRaw = {                        /* NOTE: change here */
   5,
   NULL,
   NULL,
   bi_init_record,
   NULL,
   read_bi};

/* Create the dset for devMbbiSoftRaw */

static long mbbi_init_record();
static long read_mbbi();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_mbbi;
} devMbbiSoftRaw = {                      /* NOTE: change here */
   5,
   NULL,
   NULL,
   mbbi_init_record,
   NULL,
   read_mbbi};


/*********************************** AI **********************************/

static long ai_init_record(pai)
   struct aiRecord   *pai;
{
   char  message[100];

   /* ai.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK */

   switch (pai->inp.type)
   {
      case (CONSTANT) :
         pai->rval = pai->inp.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pai->name);
         strcat (message, ": devAi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return(S_db_badField);
   }
   return(0);
}

static long read_ai(pai)
   struct aiRecord   *pai;
{
   char  message[100];
   long  status, options, nRequest;

   /* ai.inp must be a CONSTANT, DB_LINK or a CA_LINK */

   switch (pai->inp.type)
   {
   case (CONSTANT) :
      break;

   case (DB_LINK) :
      options=0;
      nRequest=1;
      status = dbGetLink(&(pai->inp.value.db_link), (struct dbCommon *)pai,
			 DBR_LONG, (caddr_t)(&pai->rval),
			 &options,&nRequest);
      if(status!=0)
      {
         if(pai->nsev<VALID_ALARM)
         {
            pai->nsev = VALID_ALARM;
            pai->nsta = LINK_ALARM;
         }
      }

      break;

   case (CA_LINK) :
      break;

   default :
      if(pai->nsev<VALID_ALARM)
      {
         pai->nsev = VALID_ALARM;
         pai->nsta = SOFT_ALARM;
         if (pai->stat!=SOFT_ALARM)
         {
            strcpy (message, pai->name);
            strcat (message, ": devAi");
            strcat (message, io_name);
            strcat (message, " (read_ai) Illegal INP field");
            errMessage (S_db_badField, message);
         }
      }
   }
   return(0);
}


/********************************* BI ************************************/

static long bi_init_record(pbi)
   struct biRecord   *pbi;
{
   char  message[100];

   /* bi.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK */

   switch (pbi->inp.type)
   {
      case (CONSTANT) :
         pbi->rval = pbi->inp.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pbi->name);
         strcat (message, ": devBi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_bi(pbi)
   struct biRecord   *pbi;
{
   char  message[100];
   long  status, options, nRequest;

   /* bi.inp must be a CONSTANT, DB_LINK or a CA_LINK */

   switch (pbi->inp.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         options=0;
         nRequest=1;
         status = dbGetLink(&(pbi->inp.value.db_link),
			    (struct dbCommon *)pbi,
			    DBR_ULONG, (caddr_t)(&pbi->rval),
			    &options,&nRequest);
         if(status!=0)
         {
            if(pbi->nsev<VALID_ALARM)
            {
               pbi->nsev = VALID_ALARM;
               pbi->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pbi->nsev<VALID_ALARM)
         {
            pbi->nsev = VALID_ALARM;
            pbi->nsta = SOFT_ALARM;
            if(pbi->stat!=SOFT_ALARM)
            {
               strcpy (message, pbi->name);
               strcat (message, ": devBi");
               strcat (message, io_name);
               strcat (message, " (read_bi) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/********************************* MBBI ***********************************/

static long mbbi_init_record(pmbbi)
   struct mbbiRecord   *pmbbi;
{
   char  message[100];

   /* mbbi.inp must be a CONSTANT, PV_LINK, DB_LINK or a CA_LINK */

   switch (pmbbi->inp.type)
   {
      case (CONSTANT) :
         pmbbi->rval = pmbbi->inp.value.value;
         break;

      case (PV_LINK) :
         break;

      case (DB_LINK) :
         break;

      case (CA_LINK) :
         break;

      default :
         strcpy (message, pmbbi->name);
         strcat (message, ": devMbbi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_mbbi(pmbbi)
   struct mbbiRecord   *pmbbi;
{
   char  message[100];
   long  status, options, nRequest;

   /* mbbi.inp must be a CONSTANT, DB_LINK or a CA_LINK */

   switch (pmbbi->inp.type)
   {
      case (CONSTANT) :
         break;

      case (DB_LINK) :
         options=0;
         nRequest=1;
         status = dbGetLink(&(pmbbi->inp.value.db_link),
			    (struct dbCommon *)pmbbi,
			    DBR_ULONG, (caddr_t)(&pmbbi->rval),
			    &options,&nRequest);
         if(status!=0)
         {
            if(pmbbi->nsev<VALID_ALARM)
            {
               pmbbi->nsev = VALID_ALARM;
               pmbbi->nsta = LINK_ALARM;
            }
         }
         break;

      case (CA_LINK) :
         break;

      default :
         if(pmbbi->nsev<VALID_ALARM)
         {
            pmbbi->nsev = VALID_ALARM;
            pmbbi->nsta = SOFT_ALARM;
            if(pmbbi->stat!=SOFT_ALARM)
            {
               strcpy (message, pmbbi->name);
               strcat (message, ": devMbbi");
               strcat (message, io_name);
               strcat (message, " (read_mbbi) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}
