/*devTestAsyn.c,v 1.1.1.1 1999/08/03 14:42:44 tis4000 Exp */

/****************************************************************/
/*                                                              */
/*   This software is the proprietary property of Tate          */
/*   Integrated Systems, L.P.  Patents and copyrights are       */
/*   pending.                                                   */
/*                                                              */
/*   This software is furnished under a license and may         */
/*   be used and copied only in accordance with the terms       */
/*   of such license and with the inclusion of the above        */
/*   notice. This software or any other copies thereof          */
/*   may not be provided or otherwise made available to         */
/*   any other person. No title to and ownership of the         */
/*   software is hereby transferred.                            */
/*                                                              */
/*   The information in this software is subject to             */
/*   change without notice and should not be construed          */
/*   as a commitment by Tate Integrated Systems L.P. or         */
/*   its third party suppliers.                                 */
/*                                                              */
/*   Tate Integrated Systems, L.P. (TIS) and its third          */
/*   party suppliers assume no responsibility for the           */
/*   use or inability to use any of its software. TIS           */
/*   software is provided "as is" without warranty of           */
/*   any kind and TIS expressly disclaims all implied           */
/*   warranties, including but not limited to the implied       */
/*   warranties of merchantability and fitness for a            */
/*   particular purpose.                                        */
/*                                                              */
/*   Notice: Notwithstanding any other lease or license         */
/*   that may pertain to or accompany the delivery of this      */
/*   computer software, the rights of the Government            */
/*   regarding its use, reproduction, and disclosure are        */
/*   as set forth in Section 52.227-19 of the FARS              */
/*   Computer Software-Restricted Rights clause.                */
/*                                                              */
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : devTestAsyn.c
 *  AUTHOR                 : emsteel
 *  DATE                   : 03 Jan 94
 *
 *
 * DESCRIPTION: device support for Modicon
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 94
 *  REVISION:  make forward func def agree with actual func def
 *
****************************************************************************/

/****************************************************************************
 *
 * Programmer's Notes:
 *    This is a compendium of all dev**TestAsyn.c files in .../share/src/dev
 *    Where noted, change appropriately to create a new device support
 *    module
 *    callbackRequest () - see .../src/db/dbScan.c
 *
 *
****************************************************************************/


#include   <vxWorks.h>
#include   <types.h>
#include   <stdioLib.h>
#include   <wdLib.h>
#include   <memLib.h>
#include   <taskLib.h>
#include   <logLib.h>
#include   <string.h>

#include    "alarm.h"
#include    "cvtTable.h"
#include    "dbDefs.h"
#include    "dbCommon.h"
#include    "link.h"
#include    "db_field_log.h"
#include    "dbAccess.h"
#include    "devSup.h"
#include    "recSup.h"
#include    "special.h"
#include    "rec/aiRecord.h"
#include    "rec/aoRecord.h"
#include    "rec/biRecord.h"
#include    "rec/boRecord.h"
#include    "rec/mbbiRecord.h"
#include    "rec/mbboRecord.h"
#include    "rec/ptoRecord.h"
#include    "rec/stringinRecord.h"
#include    "rec/stringoutRecord.h"
#include    "rec/waveformRecord.h"
#include    "error.h"
#include    "callbacks.h"

static char *io_name = "TestAsyn";         /* NOTE: change here */

/* control block for processCallback */

struct processCallback {
   void (*callback)();
   int  priority;
   struct dbAddr dbAddr;
   WDOG_ID wd_id;
   void (*process)();
};


/* Create the dset for devAiTestAsyn */

static long  ai_init_record ();
static long  read_ai ();
static void  aiCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_ai;
   DEVSUPFUN   special_linconv;
} devAiTestAsyn = {                       /* NOTE: change here */
   6,
   NULL,
   NULL,
   ai_init_record,
   NULL,
   read_ai,
   NULL};

/* Create the dset for devAoTestAsyn */

static long  ao_init_record ();
static long  write_ao ();
static void  aoCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_ao;
   DEVSUPFUN   special_linconv;
   DEVSUPFUN   readback_ao;
} devAoTestAsyn = {                       /* NOTE: change here */
   7,
   NULL,
   NULL,
   ao_init_record,
   NULL,
   write_ao,
   NULL,
   NULL};


/* Create the dset for devBiTestAsyn */

static long  bi_init_record ();
static long  read_bi ();
static void  biCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_bi;
   DEVSUPFUN   special_linconv;
} devBiTestAsyn = {                       /* NOTE: change here */
   6,
   NULL,
   NULL,
   bi_init_record,
   NULL,
   read_bi,
   NULL};

/* Create the dset for devBoTestAsyn */

static long  bo_init_record ();
static long  write_bo ();
static void  boCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_bo;
   DEVSUPFUN   special_linconv;
   DEVSUPFUN   readback_bo;
} devBoTestAsyn = {                       /* NOTE: change here */
   7,
   NULL,
   NULL,
   bo_init_record,
   NULL,
   write_bo,
   NULL,
   NULL};


/* Create the dset for devMbbiTestAsyn */

static long  mbbi_init_record ();
static long  read_mbbi ();
static void  mbbiCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_mbbi;
   DEVSUPFUN   special_linconv;
} devMbbiTestAsyn = {                     /* NOTE: change here */
   6,
   NULL,
   NULL,
   mbbi_init_record,
   NULL,
   read_mbbi,
   NULL};


/* Create the dset for devMbboTestAsyn */

static long  mbbo_init_record ();
static long  write_mbbo ();
static void  mbboCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_mbbo;
   DEVSUPFUN   special_linconv;
   DEVSUPFUN   readback_mbbo;
}devMbboTestAsyn={                        /* NOTE: change here */
   7,
   NULL,
   NULL,
   mbbo_init_record,
   NULL,
   write_mbbo,
   NULL,
   NULL};

/* Create the dset for devPtoTestAsyn */

static long  pto_init_record();
static long  write_pto();
static void  ptoCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_pto;
   DEVSUPFUN   special_linconv;
} devPtoTestAsyn = {                      /* NOTE: change here */              
   6,
   NULL,
   NULL,
   pto_init_record,
   NULL,
   write_pto,
   NULL};

/* Create the dset for devSiTestAsyn */

static long  si_init_record();
static long  read_stringin();
static void  siCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_stringin;
   DEVSUPFUN   special_linconv;
} devSiTestAsyn = {                       /* NOTE: change here */
   6,
   NULL,
   NULL,
   si_init_record,
   NULL,
   read_stringin,
   NULL};

/* Create the dset for devSoTestAsyn */

static long  so_init_record();
static long  write_stringout();
static void  soCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   write_stringout;
   DEVSUPFUN   special_linconv;
} devSoTestAsyn = {                       /* NOTE: change here */
   6,
   NULL,
   NULL,
   so_init_record,
   NULL,
   write_stringout,
   NULL};


/* Create the dset for devWfTestAsyn */

static long  wf_init_record ();
static long  read_wf ();
static void  wfCallback ();

struct {
   long        number;
   DEVSUPFUN   report;
   DEVSUPFUN   init;
   DEVSUPFUN   init_record;
   DEVSUPFUN   get_ioint_info;
   DEVSUPFUN   read_wf;
   DEVSUPFUN   special_linconv;
} devWfTestAsyn = {                       /* NOTE: change here */
   6,
   NULL,
   NULL,
   wf_init_record,
   NULL,
   read_wf,
   NULL};


/********************************* AI ************************************/

static void aiCallback (pcallback)
   struct processCallback *pcallback;
{
   struct aiRecord *pai = (struct aiRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pai);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pai);
}
    
    

static long ai_init_record (pai, process)
   struct aiRecord   *pai;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* ai.inp must be a CONSTANT */

   switch (pai->inp.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pai->dpvt = (caddr_t)pcallback;
         pcallback->callback = aiCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pai->name,&(pcallback->dbAddr)))
         {
            logMsg("dbNameToAddr failed in init_record for devAiTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         pai->val = pai->inp.value.value;
         pai->udf = FALSE;
         break;

      default :
         strcpy (message, pai->name);
         strcat (message, ": devAi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_ai (pai)
   struct aiRecord   *pai;
{
   char              message[100];
   struct processCallback   *pcallback = (struct processCallback *)(pai->dpvt);
   int               wait_time;

   /* ai.inp must be a CONSTANT */

   switch (pai->inp.type)
   {
      case (CONSTANT) :
         if(pai->pact)
         {
            logMsg("%s Completed\n", (int)pai->name,0,0,0,0,0);
            return(2); /* don`t convert*/
         }
         else
         {
            wait_time = (int)(pai->disv * vxTicksPerSecond);
            if(wait_time<=0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pai->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pai->nsev<VALID_ALARM)
         {
            pai->nsev = VALID_ALARM;
            pai->nsta = SOFT_ALARM;
            if(pai->stat!=SOFT_ALARM)
            {
               strcpy (message, pai->name);
               strcat (message, ": devAi");
               strcat (message, io_name);
               strcat (message, " (read_ai) Illegal INP field");
               errMessage(S_db_badField, message);
            }
         }
   }
   return(0);
}


/********************************** AO ***********************************/

static void aoCallback(pcallback)
   struct processCallback *pcallback;
{
   struct aoRecord *pao = (struct aoRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pao);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pao);
}
    

static long ao_init_record (pao, process)
   struct aoRecord   *pao;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* ao.out must be a CONSTANT */

   switch (pao->out.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pao->dpvt = (caddr_t)pcallback;
         pcallback->callback = aoCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pao->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devAoTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         break;

      default :
         strcpy (message, pao->name);
         strcat (message, ": devAo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(2);
}

static long write_ao(pao)
   struct aoRecord   *pao;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback=(struct processCallback *)(pao->dpvt);

   /* ao.out must be a CONSTANT */

   switch (pao->out.type)
   {
      case (CONSTANT) :
         if(pao->pact)
         {
            logMsg("%s Completed\n", (int)pao->name,0,0,0,0,0);
            return(0);
         }
         else
         {
            wait_time = (int)(pao->disv * vxTicksPerSecond);
            if(wait_time<=0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pao->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }
      default :
         if(pao->nsev<VALID_ALARM)
         {
            pao->nsev = VALID_ALARM;
            pao->nsta = SOFT_ALARM;
            if(pao->stat!=SOFT_ALARM)
            {
               strcpy (message, pao->name);
               strcat (message, ": devAo");
               strcat (message, io_name);
               strcat (message, " (read_ao) Illegal OUT field");
               errMessage (S_db_badField,message);
            }
         }
   }
   return(0);
}


/*********************************** BI **********************************/

static void biCallback(pcallback)
   struct processCallback *pcallback;
{
   struct biRecord *pbi = (struct biRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pbi);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pbi);
}
    

static long bi_init_record (pbi, process)
   struct biRecord   *pbi;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* bi.inp must be a CONSTANT */

   switch (pbi->inp.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pbi->dpvt = (caddr_t)pcallback;
         pcallback->callback = biCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pbi->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devBiTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         pbi->val = pbi->inp.value.value;
         pbi->udf = FALSE;
         break;

      default :
         strcpy (message, pbi->name);
         strcat (message, ": devBi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_bi (pbi)
   struct biRecord   *pbi;
{
   int               wait_time;
   char              message[100];
   struct processCallback   *pcallback = (struct processCallback *)(pbi->dpvt);

   /* bi.inp must be a CONSTANT */

   switch (pbi->inp.type)
   {
      case (CONSTANT) :
         if(pbi->pact)
         {
            logMsg ("%s Completed\n", (int)pbi->name,0,0,0,0,0);
            return (2); /* don't convert */
         }
         else
         {
            wait_time = (int)(pbi->disv * vxTicksPerSecond);
            if (wait_time<=0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pbi->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pbi->nsev<VALID_ALARM)
         {
            pbi->nsev = VALID_ALARM;
            pbi->nsta = SOFT_ALARM;
            if(pbi->stat!=SOFT_ALARM)
            {
               strcpy (message, pbi->name);
               strcat (message, ": devBi");
               strcat (message, io_name);
               strcat (message, " (read_bi) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}

/*********************************** BO **********************************/

static void boCallback(pcallback)
   struct processCallback *pcallback;
{
   struct boRecord *pbo = (struct boRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pbo);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pbo);
}
    
    

static long bo_init_record(pbo,process)
   struct boRecord   *pbo;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* bo.out must be a CONSTANT */

   switch (pbo->out.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pbo->dpvt = (caddr_t)pcallback;
         pcallback->callback = boCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pbo->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devBoTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         break;

      default :
         strcpy (message, pbo->name);
         strcat (message, ": devBo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(2);
}

static long write_bo(pbo)
   struct boRecord   *pbo;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *)(pbo->dpvt);

   /* bo.out must be a CONSTANT */

   switch (pbo->out.type)
   {
      case (CONSTANT) :
         if(pbo->pact)
         {
            logMsg ("%s Completed\n", (int)pbo->name,0,0,0,0,0);
            return(0); /* don`t convert*/
         }
         else
         {
            wait_time = (int)(pbo->disv * vxTicksPerSecond);
            if(wait_time<=0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pbo->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pbo->nsev<VALID_ALARM)
         {
            pbo->nsev = VALID_ALARM;
            pbo->nsta = SOFT_ALARM;
            if(pbo->stat != SOFT_ALARM)
            {
               strcpy (message, pbo->name);
               strcat (message, ": devBo");
               strcat (message, io_name);
               strcat (message, " (read_bo) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** MBBI *********************************/

static void mbbiCallback(pcallback)
   struct processCallback *pcallback;
{
   struct mbbiRecord *pmbbi = (struct mbbiRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pmbbi);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pmbbi);
}
    

static long mbbi_init_record(pmbbi,process)
   struct mbbiRecord   *pmbbi;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* mbbi.inp must be a CONSTANT */

   switch (pmbbi->inp.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pmbbi->dpvt = (caddr_t)pcallback;
         pcallback->callback = mbbiCallback;
         pcallback->priority = priorityLow;
         if (dbNameToAddr(pmbbi->name, &(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devMbbiTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         pmbbi->val = pmbbi->inp.value.value;
         pmbbi->udf = FALSE;
         break;

      default :
         strcpy (message, pmbbi->name);
         strcat (message, ": devMbbi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_mbbi(pmbbi)
   struct mbbiRecord   *pmbbi;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *)(pmbbi->dpvt);

   /* mbbi.inp must be a CONSTANT */

   switch (pmbbi->inp.type)
   {
      case (CONSTANT) :
         if(pmbbi->pact)
         {
            logMsg ("%s Completed\n", (int)pmbbi->name,0,0,0,0,0);
            return(2); /* don't convert */
         }
         else
         {
            wait_time = (int)(pmbbi->disv * vxTicksPerSecond);
            if (wait_time <= 0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pmbbi->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pmbbi->nsev<VALID_ALARM)
         {
            pmbbi->nsev = VALID_ALARM;
            pmbbi->nsta = SOFT_ALARM;
            if (pmbbi->stat != SOFT_ALARM)
            {
               strcpy (message, pmbbi->name);
               strcat (message, ": devMbbi");
               strcat (message, io_name);
               strcat (message, " (read_mbbi) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** MBBO *********************************/

static void mbboCallback(pcallback)
   struct processCallback *pcallback;
{
   struct mbboRecord *pmbbo = (struct mbboRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pmbbo);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pmbbo);
}
    
    

static long mbbo_init_record(pmbbo,process)
   struct mbboRecord   *pmbbo;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* mbbo.out must be a CONSTANT */

   switch (pmbbo->out.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pmbbo->dpvt = (caddr_t)pcallback;
         pcallback->callback = mbboCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pmbbo->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devMbboTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         break;

      default :
         strcpy(message,pmbbo->name);
         strcat(message,": devMbboTestAsyn (init_record) Illegal OUT field");
         errMessage(S_db_badField,message);
         return(S_db_badField);
   }
 return(2);
}

static long write_mbbo(pmbbo)
   struct mbboRecord   *pmbbo;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *)(pmbbo->dpvt);

   /* mbbo.out must be a CONSTANT */

   switch (pmbbo->out.type)
   {
      case (CONSTANT) :
         if(pmbbo->pact)
         {
            logMsg ("%s Completed\n", (int)pmbbo->name,0,0,0,0,0);
            return(0); /* don`t convert*/
         }
         else
         {
            wait_time = (int)(pmbbo->disv * vxTicksPerSecond);
            if (wait_time <= 0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pmbbo->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pmbbo->nsev<VALID_ALARM)
         {
            pmbbo->nsev = VALID_ALARM;
            pmbbo->nsta = SOFT_ALARM;
            if(pmbbo->stat!=SOFT_ALARM)
            {
               strcpy (message, pmbbo->name);
               strcat (message, ": devMbbo");
               strcat (message, io_name);
               strcat (message, " (read_mbbo) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** PTO *********************************/

static void ptoCallback (pcallback)
   struct processCallback *pcallback;
{
   struct ptoRecord *ppto = (struct ptoRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)ppto);
   (pcallback->process) (&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)ppto);
}
    
    

static long pto_init_record (ppto, process, bit_no)
   struct ptoRecord   *ppto;
   void       (*process)();
   unsigned int   bit_no;
{
   char              message[100];
   struct processCallback   *pcallback;

   /* pto.out must be a CONSTANT */

   switch (bit_no)
   {

      case 0:
         switch (ppto->pout.type)
         {

            case (CONSTANT) :
               pcallback = (struct processCallback *) (calloc(1, sizeof (struct processCallback)));
               ppto->dpvt = (caddr_t) pcallback;
               pcallback->callback = ptoCallback;
               pcallback->priority = priorityLow;
               if (dbNameToAddr (ppto->name, &(pcallback->dbAddr)))
               {
		    logMsg ("dbNameToAddr failed in init_record for devPtoTestAsyn\n", 0,0,0,0,0,0);
		    taskSuspend (taskIdSelf());
               }
               pcallback->wd_id = wdCreate ();
               pcallback->process = process;
               break;

            default :
               strcpy (message, ppto->name);
               strcat (message, ": devPto");
               strcat (message, io_name);
               strcat (message, " (init_record) Illegal OUT field");
               errMessage (S_db_badField, message);
               return (S_db_badField);
         }
         break;

      case 1:
         switch (ppto->nout.type)
         {

            case (CONSTANT) :
               pcallback = (struct processCallback *) (calloc(1, sizeof (struct processCallback)));
               ppto->dpvt = (caddr_t) pcallback;
               pcallback->callback = ptoCallback;
               pcallback->priority = priorityLow;
               if (dbNameToAddr (ppto->name, &(pcallback->dbAddr)))
               {
		    logMsg ("dbNameToAddr failed in init_record for devPtoTestAsyn\n", 0,0,0,0,0,0);
		    taskSuspend (taskIdSelf());
               }
               pcallback->wd_id = wdCreate ();
               pcallback->process = process;
               break;

            default :
               strcpy (message, ppto->name);
               strcat (message, ": devPto");
               strcat (message, io_name);
               strcat (message, " (init_record) Illegal OUT field");
               errMessage (S_db_badField, message);
               return (S_db_badField);
         }
   }

   return (2);
}

static long write_pto (ppto, bit_no)
   struct ptoRecord   *ppto;
   unsigned int   bit_no;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *) (ppto->dpvt);

   /* pto.out must be a CONSTANT */

   switch (bit_no)
   {

      case 0:
         switch (ppto->pout.type)
         {
            case (CONSTANT) :
               if (ppto->pact)
               {
                  logMsg ("%s Completed\n", (int)ppto->name,0,0,0,0,0);
                  return(0); /* don`t convert*/
               }
               else
               {
                  wait_time = (int) (ppto->disv * vxTicksPerSecond);
                  if (wait_time <= 0)
                     return (0);
                  logMsg ("%s Starting asynchronous processing\n", (int)ppto->name,0,0,0,0,0);
                  wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
                  return(1);
               }

            default :
               if(ppto->nsev < VALID_ALARM)
               {
                  ppto->nsev = VALID_ALARM;
                  ppto->nsta = SOFT_ALARM;
                  if (ppto->stat != SOFT_ALARM)
                  {
                     strcpy (message, ppto->name);
                     strcat (message, ": devPto");
                     strcat (message, io_name);
                     strcat (message, " (read_bo) Illegal OUT field");
                     errMessage (S_db_badField, message);
                  }
               }
         }
         break;

      case 1:
         switch (ppto->nout.type)
         {
            case (CONSTANT) :
               if (ppto->pact)
               {
                  logMsg ("%s Completed\n", (int)ppto->name,0,0,0,0,0);
                  return(0); /* don`t convert*/
               }
               else
               {
                  wait_time = (int) (ppto->disv * vxTicksPerSecond);
                  if (wait_time <= 0)
                     return (0);
                  logMsg ("%s Starting asynchronous processing\n", (int)ppto->name,0,0,0,0,0);
                  wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
                  return(1);
               }

            default :
               if(ppto->nsev < VALID_ALARM)
               {
                  ppto->nsev = VALID_ALARM;
                  ppto->nsta = SOFT_ALARM;
                  if (ppto->stat != SOFT_ALARM)
                  {
                     strcpy (message, ppto->name);
                     strcat (message, ": devPto");
                     strcat (message, io_name);
                     strcat (message, " (read_bo) Illegal OUT field");
                     errMessage (S_db_badField, message);
                  }
               }
         }

   }
   return (0);
}


/*********************************** SI **********************************/

static void siCallback(pcallback)
   struct processCallback *pcallback;
{
   struct stringinRecord *pstringin = (struct stringinRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pstringin);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pstringin);
}
    
    

static long si_init_record(pstringin,process)
   struct stringinRecord   *pstringin;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* stringin.inp must be a CONSTANT */

   switch (pstringin->inp.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pstringin->dpvt = (caddr_t)pcallback;
         pcallback->callback = siCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pstringin->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devSiTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         if (pstringin->inp.value.value!=0.0)
         {
           sprintf (pstringin->val, "%-14.7g",pstringin->inp.value.value);
           pstringin->udf = FALSE;
         }
         break;

      default :
         strcpy (message, pstringin->name);
         strcat (message, ": devSi");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_stringin(pstringin)
   struct stringinRecord   *pstringin;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *)(pstringin->dpvt);

   /* stringin.inp must be a CONSTANT */

   switch (pstringin->inp.type)
   {
      case (CONSTANT) :
         if(pstringin->pact)
         {
            logMsg ("%s Completed\n", (int)pstringin->name,0,0,0,0,0);
            return(0);
         }
         else
         {
            wait_time = (int)(pstringin->disv * vxTicksPerSecond);
            if(wait_time <= 0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pstringin->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pstringin->nsev<VALID_ALARM)
         {
            pstringin->nsev = VALID_ALARM;
            pstringin->nsta = SOFT_ALARM;
            if(pstringin->stat!=SOFT_ALARM)
            {
               strcpy (message, pstringin->name);
               strcat (message, ": devSi");
               strcat (message, io_name);
               strcat (message, " (read_stringin) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** SO **********************************/

static void soCallback(pcallback)
   struct processCallback *pcallback;
{
   struct stringoutRecord *pstringout = (struct stringoutRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pstringout);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pstringout);
}
    

static long so_init_record(pstringout,process)
   struct stringoutRecord   *pstringout;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* stringout.out must be a CONSTANT */

   switch (pstringout->out.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pstringout->dpvt = (caddr_t)pcallback;
         pcallback->callback = soCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pstringout->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devSoTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         break;

      default :
         strcpy (message, pstringout->name);
         strcat (message, ": devSo");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal OUT field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long write_stringout(pstringout)
   struct stringoutRecord   *pstringout;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *)(pstringout->dpvt);

   /* stringout.out must be a CONSTANT */

   switch (pstringout->out.type)
   {
      case (CONSTANT) :
         if(pstringout->pact)
         {
            logMsg ("%s Completed\n", (int)pstringout->name,0,0,0,0,0);
            return(0); /* don`t convert*/
         }
         else
         {
            wait_time = (int)(pstringout->disv * vxTicksPerSecond);
            if(wait_time <= 0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pstringout->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pstringout->nsev<VALID_ALARM)
         {
            pstringout->nsev = VALID_ALARM;
            pstringout->nsta = SOFT_ALARM;
            if(pstringout->stat!=SOFT_ALARM)
            {
               strcpy (message, pstringout->name);
               strcat (message, ": devSo");
               strcat (message, io_name);
               strcat (message, " (read_stringout) Illegal OUT field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}


/*********************************** WF *********************************/

static void wfCallback(pcallback)
   struct processCallback *pcallback;
{
   struct wfRecord *pwf = (struct wfRecord *)(pcallback->dbAddr.precord);

   dbScanLock ((caddr_t)pwf);
   (pcallback->process)(&pcallback->dbAddr);
   dbScanUnlock ((caddr_t)pwf);
}

static long wf_init_record(pwf,process)
   struct waveformRecord   *pwf;
   void (*process)();
{
   char              message[100];
   struct processCallback   *pcallback;

   /* wf.inp must be a CONSTANT */

   switch (pwf->inp.type)
   {
      case (CONSTANT) :
         pcallback = (struct processCallback *)(calloc(1,sizeof(struct processCallback)));
         pwf->dpvt = (caddr_t)pcallback;
         pcallback->callback = wfCallback;
         pcallback->priority = priorityLow;
         if(dbNameToAddr(pwf->name,&(pcallback->dbAddr)))
         {
            logMsg ("dbNameToAddr failed in init_record for devWfTestAsyn\n", 0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
         }
         pcallback->wd_id = wdCreate();
         pcallback->process = process;
         pwf->nord = 0;
         break;

      default :
         strcpy (message, pwf->name);
         strcat (message, ": devWf");
         strcat (message, io_name);
         strcat (message, " (init_record) Illegal INP field");
         errMessage (S_db_badField, message);
         return (S_db_badField);
   }
   return(0);
}

static long read_wf(pwf)
   struct waveformRecord   *pwf;
{
   char              message[100];
   int               wait_time;
   struct processCallback   *pcallback = (struct processCallback *)(pwf->dpvt);

   /* wf.inp must be a CONSTANT */

   switch (pwf->inp.type)
   {
      case (CONSTANT) :
         if(pwf->pact)
         {
            logMsg ("%s Completed\n", (int)pwf->name,0,0,0,0,0);
            return(0); /* don`t convert*/
         }
         else
         {
            wait_time = (int)(pwf->disv * vxTicksPerSecond);
            if (wait_time <= 0)
               return(0);
            logMsg ("%s Starting asynchronous processing\n", (int)pwf->name,0,0,0,0,0);
            wdStart (pcallback->wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
            return(1);
         }

      default :
         if(pwf->nsev<VALID_ALARM)
         {
            pwf->nsev = VALID_ALARM;
            pwf->nsta = SOFT_ALARM;
            if(pwf->stat!=SOFT_ALARM)
            {
               strcpy (message, pwf->name);
               strcat (message, ": devWf");
               strcat (message, io_name);
               strcat (message, " (read_wf) Illegal INP field");
               errMessage (S_db_badField, message);
            }
         }
   }
   return(0);
}
