/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/****************************************************************/
/*								*/
/*		GET DATA FIELD IN AGENT FORMAT			*/
/*								*/
/*	This function fetches a data field in Agent format.	*/
/*	The record always contains stat, sevr, and time, but	*/
/*	these are passed from the caller in case the value	*/
/*	to be fetched came from the event queue instead of	*/
/*	from the record. This is the third routine in the	*/
/*	rewrite of channel access that deals with translating	*/
/*	types. transformTypes() translates between two agent	*/
/*	types. caConvType() converts between agent types	*/
/*	and the original DBR_ types. This function translates	*/
/*	from data fields in the IOC records to the agent types.	*/
/*								*/
/*	Always return the number of bytes copied to the output	*/
/*	buffer.							*/
/*								*/
/*	WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!	*/
/*								*/
/*	This routine was written assuming that we are running	*/
/*	on a big endian processor (i.e., the native byte order	*/
/*	is the same as network byte order). If this is not	*/
/*	true, then some patchwork must be done to insert the	*/
/*	appropriate data conversion calls.			*/
/*								*/
/*	WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!	*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/*								*/
/* Above are the original comments. They are intact for		*/
/* historical purposes. This function accomplhes the same task */
/* but with different parameters such as the control block	*/
/* as assigned during initialization.		*/
/*								*/
/*	Author: Gregory Tomalesky	*/
/*	Date:	26-Jan-1995			*/
/****************************************************************/
#include <sys/time.h>
#include <in.h>

#include "types.h"
#include "dbDefs.h"
#include "choiceGbl.h"
#include "choice.h"
#include "tsDefs.h"
#include "dataTypes.h"
#include "dpa.h"
#include "alarm.h"
#include "alarmString.h"
#include "msgMon.h"

#if defined(SOLARIS) || defined(AXP)
#define bzero(b,n)	memset (b, 0, n)
#define bcopy(s,d,n)	memcpy (d, s, n)
#endif

static	char	*yesNo[] =
	{
	"NO",
	"YES"
	};
int	dpaAgentField(pOut, agentType, numElem, pTime, info,pAddr)
char				*pOut;		/* Pointer to output buffer */
short				agentType;	/* Desired point type (see dataTypes.h)*/
short				numElem;	/* Number of array elements */
TS_STAMP			*pTime;		/* Time Stamp */
CHANNEL_INFO_PTR	info;		/* PV address */
DPA_ADDR		*pAddr;		/* field pointer within above PV entry */
{
    char		*outPtr = pOut;
    char			aenb[4];
    char			ack[4];
    char			desc[28];
    int				oddAddr;
    char tempValue[DPA_MAX_VALUE_SIZE];
    TS_STAMP		currTime;
    int i,dSize;
    float *floatPtr;
    double *doublePtr;
    int *intPtr;
    short *shortPtr;
    char *charPtr;
    long *longPtr;
/*
    Start off by getting all of the embellishments that are desired.
    We get all embellishments as doubles. Then, we put them into the
    data stream in the correct type when it's time to assemble the
    output buffer.
*/
    strcpy(aenb, yesNo[info->chanAenb]);
    strcpy(ack, yesNo[info->chanAck]);
    strcpy(desc,info->chanDesc);
/*
    Now assemble the embellisments for the output data stream.
*/

    if (agentType & AGNT_STAT)
	{
	*(short *) outPtr = info->chanStat;
	outPtr += sizeof(short);
	*(short *) outPtr = info->chanSevr;
	outPtr += sizeof(short);
	if ((agentType & 0xff) == AGNT_STRING)
	    {
	    strncpy(outPtr, aenb, sizeof(aenb));
	    outPtr += sizeof(aenb);
	    strncpy(outPtr, ack, sizeof(ack));
	    outPtr += sizeof(ack);
	    strncpy(outPtr, desc, sizeof(desc));
	    outPtr[sizeof(desc) - 1] = 0;
	    outPtr += sizeof(desc);
 	    strncpy(outPtr, info->chanEgu, 8);
 	    outPtr[7] = 0;
 	    outPtr += 8;
	    }
	}
    if (agentType & AGNT_TIME)
		{
		if(pTime)
			{
			bcopy((char *) pTime, outPtr, sizeof(TS_STAMP));
			outPtr += sizeof(TS_STAMP);
			}
		else
			{
			gettimeofday((struct timeval *)&currTime,0);
			currTime.nsec = 0;
			currTime.secPastEpoch -= TS_EPOCH_SEC_PAST_1970;
			bcopy((char *)&currTime,outPtr,sizeof(TS_STAMP));
			outPtr += sizeof(TS_STAMP);
			}
		}

    switch (agentType & 0xff)
	{
	case AGNT_STRING:
	    {
	    if (agentType & AGNT_GRAPH)
			{
			tisSysLog(0,0,"Graphic string not supported.");
			return (0);
			}
	    if (agentType & AGNT_CNTRL)
			{
			tisSysLog(0,0,"Control string not supported.");
			return (0);
			}
	    break;
	    }

	case AGNT_INT:
	    {
	    if (agentType & AGNT_GRAPH)
			{
/* 			strncpy(outPtr, info->chanEgu, 8); */
/* 			outPtr[7] = 0; */
/* 			outPtr += 8; */
			*(short *) outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispHi : pAddr->hilimit);
			outPtr += sizeof(short);
			*(short *) outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispLo : pAddr->lolimit);
			outPtr += sizeof(short);
/* 			*(short *) outPtr = info->chanHiHi; */
/* 			outPtr += sizeof(short); */
/* 			*(short *) outPtr = info->chanHi; */
/* 			outPtr += sizeof(short); */
/* 			*(short *) outPtr = info->chanLo; */
/* 			outPtr += sizeof(short); */
/* 			*(short *) outPtr = info->chanLoLo; */
/* 			outPtr += sizeof(short); */
			}
	    if (agentType & AGNT_CNTRL)
			{
			*(short *) outPtr = info->chanDrvHi;
			outPtr += sizeof(short);
			*(short *) outPtr = info->chanDrvLo;
			outPtr += sizeof(short);
			}
	    break;
	    }

	case AGNT_FLOAT:
	    {
	    if (agentType & AGNT_GRAPH)
			{
/* 			strncpy(outPtr, info->chanEgu, 8); */
/* 			outPtr[7] = 0; */
/* 			outPtr += 8; */
			*(float *) outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispHi : pAddr->hilimit);
			outPtr += sizeof(float);
			*(float *) outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispLo : pAddr->lolimit);
			outPtr += sizeof(float);
/* 			*(float *) outPtr = info->chanHiHi; */
/* 			outPtr += sizeof(float); */
/* 			*(float *) outPtr = info->chanHi; */
/* 			outPtr += sizeof(float); */
/* 			*(float *) outPtr = info->chanLo; */
/* 			outPtr += sizeof(float); */
/* 			*(float *) outPtr = info->chanLoLo; */
/* 			outPtr += sizeof(float); */
			*(short *) outPtr = info->chanPrec;
			outPtr += sizeof(short);
			}
	    if (agentType & AGNT_CNTRL)
			{
			float tempfl;
			tempfl = info->chanDrvHi;
			bcopy(&tempfl,outPtr,sizeof(float));
			outPtr += sizeof(float);
			tempfl = info->chanDrvLo;
			bcopy(&tempfl,outPtr,sizeof(float));
			outPtr += sizeof(float);
			}
	    break;
	    }

	case AGNT_ENUM:
	    {
	    if ((agentType & AGNT_GRAPH) || (agentType & AGNT_CNTRL))
			{
			int	index;
			short aSize;

			switch(pAddr->field_index)
				{
				case DPA_VAL:
					*(short *) outPtr = info->chanNumStrs;
					outPtr += sizeof(short);
					for (index = 0; index < info->chanNumStrs; index++)
			    		{
			    		strcpy(outPtr, info->chanEnumList[index]);
			    		outPtr += (strlen(info->chanEnumList[index]) + 1);
			    		}
					break;
				case DPA_STAT:
					aSize = sizeof(alarmStatusString)/sizeof(char *);
					*(short *) outPtr = aSize-1;
					outPtr += sizeof(short);
					for (index = 0; index < aSize-1; index++)
	    				{
	    				strcpy(outPtr, alarmStatusString[index]);
	    				outPtr += (strlen(alarmStatusString[index]) + 1);
	    				}
					break;
				case DPA_SEVR:
				case DPA_HHSV:
				case DPA_HISV:
				case DPA_LOSV:
				case DPA_LLSV:
					aSize = sizeof(alarmSeverityString)/sizeof(char *);
					*(short *) outPtr = aSize-1;
					outPtr += sizeof(short);
					for (index = 0; index < aSize-1; index++)
	    				{
	    				strcpy(outPtr, alarmSeverityString[index]);
	    				outPtr += (strlen(alarmSeverityString[index]) + 1);
	    				}
					break;
				case DPA_AENB:
				case DPA_TENB:
				case DPA_ACK:
					aSize = sizeof(yesNo)/sizeof(char *);
					*(short *) outPtr = aSize;
					outPtr += sizeof(short);
					for (index = 0; index < aSize; index++)
	    				{
	    				strcpy(outPtr, yesNo[index]);
	    				outPtr += (strlen(yesNo[index]) + 1);
	    				}
					break;
				case DPA_USER_DEFINED_FIELD:
					*(short *) outPtr = pAddr->numStrings;
					outPtr += sizeof(short);
					for (index = 0; index < pAddr->numStrings; index++)
			    		{
			    		strcpy(outPtr, pAddr->enums[index]);
			    		outPtr += (strlen(pAddr->enums[index]) + 1);
			    		}
					break;
				}
			}
	    break;
	    }

	case AGNT_CHAR:
	    {
	    if (agentType & AGNT_GRAPH)
			{
/* 			strncpy(outPtr, info->chanEgu, 8); */
/* 			outPtr[7] = 0; */
/* 			outPtr += 8; */
			*outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispHi : pAddr->hilimit);
			outPtr += sizeof(char);
			*outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispLo : pAddr->lolimit);
			outPtr += sizeof(char);
/* 			*outPtr = info->chanHiHi; */
/* 			outPtr += sizeof(char); */
/* 			*outPtr = info->chanHi; */
/* 			outPtr += sizeof(char); */
/* 			*outPtr = info->chanLo; */
/* 			outPtr += sizeof(char); */
/* 			*outPtr = info->chanLoLo; */
/* 			outPtr += sizeof(char); */
			}
	    if (agentType & AGNT_CNTRL)
			{
			*outPtr = info->chanDrvHi;
			outPtr += sizeof(char);
			*outPtr = info->chanDrvLo;
			outPtr += sizeof(char);
			}
	    break;
	    }

	case AGNT_LONG:
	    {
	    if (agentType & AGNT_GRAPH)
			{
/* 			strncpy(outPtr, info->chanEgu, 8); */
/* 			outPtr[7] = 0; */
/* 			outPtr += 8; */
			*(long *) outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispHi : pAddr->hilimit);
			outPtr += sizeof(long);
			*(long *) outPtr = (pAddr->field_index == DPA_VAL ? info->chanDispLo : pAddr->lolimit);
			outPtr += sizeof(long);
/* 			*(long *) outPtr = info->chanHiHi; */
/* 			outPtr += sizeof(long); */
/* 			*(long *) outPtr = info->chanHi; */
/* 			outPtr += sizeof(long); */
/* 			*(long *) outPtr = info->chanLo; */
/* 			outPtr += sizeof(long); */
/* 			*(long *) outPtr = info->chanLoLo; */
/* 			outPtr += sizeof(long); */
			}
	    if (agentType & AGNT_CNTRL)
			{
			*(long *) outPtr = info->chanDrvHi;
			outPtr += sizeof(long);
			*(long *) outPtr = info->chanDrvLo;
			outPtr += sizeof(long);
			}
	    break;
	    }

	case AGNT_DOUBLE:
	    {
	    if (agentType & AGNT_GRAPH)
			{
/* 			strncpy(outPtr, info->chanEgu, 8); */
/* 			outPtr[7] = 0; */
/* 			outPtr += 8; */
			bcopy((pAddr->field_index == DPA_VAL ? &info->chanDispHi : &pAddr->hilimit),outPtr,sizeof(double));
			outPtr += sizeof(double);
			bcopy((pAddr->field_index == DPA_VAL ? &info->chanDispLo : &pAddr->lolimit),outPtr,sizeof(double));
			outPtr += sizeof(double);
/* 			bcopy(&info->chanHiHi,outPtr,sizeof(double)); */
/* 			outPtr += sizeof(double); */
/* 			bcopy(&info->chanHi,outPtr,sizeof(double)); */
/* 			outPtr += sizeof(double); */
/* 			bcopy(&info->chanLo,outPtr,sizeof(double)); */
/* 			outPtr += sizeof(double); */
/* 			bcopy(&info->chanLoLo,outPtr,sizeof(double)); */
/* 			outPtr += sizeof(double); */
			bcopy(&info->chanPrec,outPtr,sizeof(short));
			outPtr += sizeof(short);
			}
	    if (agentType & AGNT_CNTRL)
			{
			bcopy(&info->chanDrvHi,outPtr,sizeof(double));
			outPtr += sizeof(double);
			bcopy(&info->chanDrvLo,outPtr,sizeof(double));
			outPtr += sizeof(double);
			}
	    break;
	    }

	default:
	    {
	    tisSysLog(0,0,"Illegal agent data type %d", agentType);
	    break;
	    }
	}

/*
    Now get the data field itself. The address of the dbAddr structure within
	the main entry for this PV is passed so use it to get the field and size.

	29-Jan-96 the size is the size of an individual element. All static fields have
	only one element. User defined fields may have many more. That value is in
	numElem.

	14-Nov-96 Convert out-going data to the requested type for the given client.

*/
    floatPtr = pAddr->pfield;
    doublePtr = pAddr->pfield;
    intPtr = pAddr->pfield;
    charPtr = pAddr->pfield;
    shortPtr = pAddr->pfield;
    longPtr = pAddr->pfield;
    for(i=0;i<numElem;i++)
      {
      switch(agentType & 0xff)
	{
	case AGNT_FLOAT:
	  *(float *)&tempValue[0] = (float)dpaConvertToFloat(pAddr,floatPtr++,pAddr->nativeType);
	  dSize = sizeof(float);
	  break;
	case AGNT_DOUBLE:
	  *(double *)&tempValue[0] = (double)dpaConvertToFloat(pAddr,doublePtr++,pAddr->nativeType);
	  dSize = sizeof(double);
	  break;
	case AGNT_ENUM:
	case AGNT_SHORT:
	  *(short *)&tempValue[0] = (short)dpaConvertToInt(pAddr,shortPtr++,pAddr->nativeType);
	  dSize = sizeof(short);
	  break;
	case AGNT_STRING:
	  strcpy(&tempValue[0],(char *)dpaConvertToString(pAddr,charPtr,pAddr->nativeType));
	  dSize = strlen(&tempValue[0])+1;	/* include NULL too */
	  break;
	case AGNT_CHAR:
	  *(char *)&tempValue[0] = (char)dpaConvertToChar(pAddr,charPtr++,pAddr->nativeType);
	  dSize = sizeof(char);
	  break;
	case AGNT_LONG:
	 *(long *)&tempValue[0] = (long)dpaConvertToLong(pAddr,longPtr++,pAddr->nativeType);
	  dSize = sizeof(long);
	  break;
	default:
	  break;
	}
      memmove(outPtr,(char *)&tempValue[0],dSize);
      outPtr += dSize;
      }

    oddAddr = outPtr - pOut;
    if(oddAddr & 1) oddAddr++;
    return(oddAddr);
}
