/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/*
	The user application calls this functions to establish him/herself
	as a data provider for CA. The following things occur during this
	function:

		1. 	'fork' to 'execl' the host rsrvTask
		2.	wait for rsrvTask to connect
		3.	fill in DPA_INFO structure with function addresses;

*/
#include	<sys/types.h>
#include	<socket.h>
#include	<in.h>
#include	<tcp.h>
#include	<sys/time.h>

#include	"dbDefs.h"
#include	"dpa.h"

#define		DO_FORK	1
DPA_INFO	dpaInfo;
LIST		chanInfoTable;
char   *chanData;
static	struct	timeval	timeout = {60,0};	/* 60 seconds should be enough time to accept a connection. */
static	char	portNumber[6],appname[32];
int	dpaInit(prefix,openFunc,readFunc,writeFunc,closeFunc,sigFunc,sigArray)
char	*prefix;
void	(*openFunc)();
void	(*readFunc)();
void	(*writeFunc)();
void	(*closeFunc)();
void	(*sigFunc)();
int		*sigArray;				/* array of desired signals to be handled end with 0 */
{
int		fd,len,stat,myPID;
char   true = 1;
struct	sockaddr_in	addr;
fd_set	readFds;
char childName[80];

myPID = getpid();	/* get my pid for child name */

/*
	set to call user defined signal handler if specified
*/
if(sigFunc)
	{
	len = 0;
	while(sigArray[len])
		{
#ifdef SOLARIS
		sigset(sigArray[len],sigFunc);
#else
		signal(sigArray[len],sigFunc);
#endif
		len++;
		}
	}
	
/*
	Obtain a socket for communication with the spawned rsrvTask
*/
fd = socket(AF_INET,SOCK_STREAM,0);
addr.sin_family = AF_INET;
addr.sin_addr.s_addr = htonl(INADDR_ANY);
addr.sin_port = htons(0);
if((stat = bind(fd,(struct sockaddr *)&addr,sizeof(struct sockaddr_in))) < 0)
	{
	close(fd);
	return(DPA_NOCONN);
	}
stat = sizeof(addr);
if(getsockname(fd, (struct sockaddr *) &addr, &stat) < 0)
	{
	close(fd);
	return(DPA_NOCONN);
	}
sprintf(portNumber, "%d", htons(addr.sin_port));
strcpy(appname,prefix);
listen(fd,5);
#ifdef	DO_FORK
if((dpaInfo.hsrvTaskPID = fork()) == 0)
	{
/*
	I am here so the 'fork' must have worked. Start the hsrvTask and
	wait for it to connect. If it can't connect it will exit.
*/
	  sprintf(childName,"%s%d\0",prefix,myPID);
	  execl(DPA_HSRV_TASK,childName,appname,portNumber,(char *)0);
	  exit(DPA_NORSRV);
	}
if(dpaInfo.hsrvTaskPID < 0)
	return(DPA_NOFORK);
#else
printf("port number = %d\n", addr.sin_port);
#endif
FD_ZERO(&readFds);
FD_SET(fd,&readFds);
stat = select(fd+1,&readFds,0,0,&timeout);		/* wait for spawned rsrvTask to connect */
if(stat <= 0)
	return(DPA_NOCONN);							/* If we timeout then tell application (it should exit) */
if((dpaInfo.appFd = accept(fd,(struct sockaddr *)&addr,&len)) < 0)	
	{
	close(fd);
	return(DPA_NOCONN);
	}
close(fd);			      	       			/* no longer needed */
setsockopt(dpaInfo.appFd,IPPROTO_TCP,TCP_NODELAY,(char *)&true,sizeof(true));
chanData = (char *)calloc(1,DPA_CHAN_DATA_SIZE);
dpaInfo.appInet		= ntohl(addr.sin_addr.s_addr);	/* rsrvTask Inet address */
dpaInfo.appOpen		= openFunc;
dpaInfo.appRead		= readFunc;
dpaInfo.appWrite	= writeFunc;
dpaInfo.appClose	= closeFunc;
dpaInfo.rxCount		= 0;
dpaInfo.rxNext		= 0;
dpaInfo.appName		= prefix;
dpaInfo.appMaxHandles = 0;						/* no valid handles yet */
lstInit(&chanInfoTable);						/* Use list library functions */
return(dpaInfo.appFd);							/* return FD so user can include in his/her own "select" */
}
