/*iocsiodvr_bridge.c,v 1.1.1.1 1999/08/03 14:42:54 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/* iocsiodvr_bridge.c - bridge interface file to map TATE default interface to Serial Line interface */

#include <vxWorks.h>
#include <string.h>
#include <semLib.h>
#include <wdLib.h>
#include <intLib.h>
#include <logLib.h>
#include <sioLib.h>
#include <serialFile.h>
#include "serLib.h"

#include "iocsiodvr_bridge.h"

#define NUMSERIALLINES 32

extern	LINESTATUS	serialFileTable[MAXLINES];

LOCAL BOOL in_use[ NUMSERIALLINES ];
LOCAL FUNCPTR rxint_procs[ NUMSERIALLINES ];
LOCAL FUNCPTR txint_procs[ NUMSERIALLINES ];


void print_init_values (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serGETSTATS );
    } else {
	return;
    }
    
}

void enable_rx_ints (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETRXHOOK, rxint_procs[line], line );
    } else {
	return;
    }
}

void disable_rx_ints (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETRXHOOK, 0 );
    } else {
	return;
    }
}

void enable_tx_ints (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETTXHOOK, txint_procs[line], line );
    } else {
	enablefiletx (line-FILELINESTART);
	return;
    }
}
 
void disable_tx_ints (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETTXHOOK, 0 );
    } else {
	disablefiletx (line-FILELINESTART);
	return;
    }
}

void set_rts (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, mdmRTS, mdmRTS );
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_SET_RTS, 0);
	return;
    }
}

void clear_rts (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, 0, mdmRTS );
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_CLR_RTS, 0);
	return;
    }
}

unsigned short read_cts (unsigned short line)
{
  unsigned short state;
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, 0, 0 );
	serIoctl( line, serGETMDMSTAT, &state );
	return (state & mdmCTS);
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_GET_CTS, &state);
	return((state != 0));
    }
}

void set_dtr (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, mdmDTR, mdmDTR );
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_SET_DTR, 0);
	return;
    }
}

void clear_dtr (unsigned short line)
{
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, 0, mdmDTR );
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_CLR_DTR, 0);
	return;
    }
}

unsigned short read_dsr (unsigned short line)
{
  unsigned short state;
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, 0, 0 );
	serIoctl( line, serGETMDMSTAT, &state );
	return (state & mdmDSR);
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_GET_DSR, &state);
	return((state != 0));
    }
}

unsigned short read_dcd (unsigned short line)
{
  unsigned short state;
    if (line < FILELINESTART) {
	serIoctl( line, serSETMDMCTL, 0, 0 );
	serIoctl( line, serGETMDMSTAT, &state );
	return (state & mdmDCD);
    } else {
        sioIoctl(serialFileTable[line-FILELINESTART].pchan, SIO_GET_DCD, &state);
	return((state != 0));
    }
}

/*
  read_rx_hw - read char & status @ hardware level

  some of the code in this routine is cobbled from the
  appropriate standard characteristics modules (i.e. hw*.c).
*/
void read_rx_hw (unsigned short line,
 char *retch, unsigned short *retstat, unsigned short *rx_err_mask)
{
    if (line < FILELINESTART) {
	IMPORT struct _line **linedefs;
	IMPORT struct _hwline **hwlinedefs;
	FAST struct _line *p = linedefs[ line ];
	FAST struct _hwifc *ifc = p-> hwifc;
	FAST int ikey = intLock();
	/*currently supported devs all use the same mask!*/
	*rx_err_mask = 0x70;
	*retstat = ifc-> hwRxStatus( ifc-> hwblk );
	*retch   = ifc-> hwRead( ifc-> hwblk );
	if( *retstat & *retstat )
  	{
	    switch( hwlinedefs[ line ]-> dvrType )
	    {
	 	case dvr8530:  *((struct hw8530*)  ifc-> hwblk)-> cr = 0x30; break;
		case dvr68681: *((struct hw68681*) ifc-> hwblk)-> cr = 0x40; break;
		case dvr2698:  *((struct hw2698*)  ifc-> hwblk)-> cr = 0x40; break;
		default: break;
	    }
	}
	intUnlock( ikey );
    } else {
	readfile_rx_char( line-FILELINESTART,  retch,
		     	  retstat, rx_err_mask);
	return;
    }

}

/*
  write_tx_hw - write char, read status @ hardware level

  the code in this routine is cobbled from the appropriate
  standard characteristics modules (i.e. hw*.c).
*/
void write_tx_hw (unsigned short line,
 char outch, unsigned short *retstat, unsigned short *tx_err_mask)
{
    if (line < FILELINESTART) {
	IMPORT struct _line **linedefs;
	FAST struct _line *p = linedefs[ line ];
	FAST struct _hwifc *ifc = p-> hwifc;
	/*no currently supported devs provide a TX error status*/
	*tx_err_mask = 0;
	while( ! ifc-> hwTxEmpty( ifc-> hwblk ) )
	;
	ifc-> hwWrite( ifc-> hwblk, outch );
    } else {
 	writefile_tx_char( line-FILELINESTART, outch,
		     	   retstat, tx_err_mask);
	return;
    }
}


int comm_init (unsigned int channel, char *mode, FUNCPTR txint_callback, FUNCPTR rxint_callback)
{
    char buf[80];
    char aline[3];

    if (channel < FILELINESTART) {
	if( in_use[ channel ] ||
	    serOpen( channel ) != OK ) return -1;

	in_use[ channel ] = TRUE;
	txint_procs[ channel ] = txint_callback;
	rxint_procs[ channel ] = rxint_callback;
	serIoctl( channel, serSETRX_RAW );
	if( mode != 0 )
	    serIoctl( channel, serSETMODE, mode );
	logMsg ("\nInitialized - serial channel %d\n", channel, 0,0,0,0,0);
	serIoctl( channel, serGETSTATS );
	return 0;
    } else {
	strcpy(buf,"/tyCo/");
	aline[1] = 0;
	aline[0] = '0'+(channel-FILELINESTART);
	strcat(buf,aline);

	initSerialFile (channel-FILELINESTART, buf, mode,
		   	rxint_callback, channel-FILELINESTART,
		    	txint_callback, channel-FILELINESTART,
		    	2);
    }
    return 0;
}

int comm_disconnect (unsigned short channel)
{
    if (channel < FILELINESTART) {
	if( ! in_use[ channel ] )
	{
	    logMsg ("***ERROR*** Channel %d not allocated to any drivers...\n", channel, 0,0,0,0,0);
	    return -1;
	}
	serClose( channel );
	in_use[ channel ] = FALSE;
	logMsg ("Channel %d disconnected from driver\n", channel, 0,0,0,0,0);
	return 1;
    } else {
	return 1;
    }
}

 

