/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		   RESOURCE GROUP LOCK MANAGER			*/
/*								*/
/*	This task watches over groups of resources. When any	*/
/*	resource in a group transits from caAnswer 1 -> 0	*/
/*	AND it's state is slave, then set the resource lock	*/
/*	flag for that group to TRUE.				*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <vxWorks.h>
#include <inetLib.h>
#include <taskLib.h>

#include "restable.h"

/****** THE GLOBAL ARRAY ******/

short  resGroupLock[MAXGROUPS];		/* 0 init */
char  *resGroupTripName[MAXGROUPS];	/* 0 init, again */

typedef struct resLock	RESLOCK;
struct resLock
    {
    RES_TABLE	*resEntry;
    int		lastCaAnswer;		/* Compiler would make it 4 anyway */
    RESLOCK	*nextLock;
    };

static RESLOCK	*resLockList[MAXGROUPS];

static RES_TABLE	*keyResource[MAXGROUPS];

/****************************************************************/
/*								*/
/*		DECLARE A KEY RESOURCE FOR THE GROUP		*/
/*								*/
/****************************************************************/

int
declareKeyResource(resName, groupNum)
char	*resName;
int	groupNum;
{
    RES_TABLE	*resEntry;

    if (resGetByName(resName, &resEntry) != RES_OK)
	return (ERROR);
    if ((groupNum <= 0) || (groupNum > MAXGROUPS))
	return (ERROR);
    groupNum--;
    keyResource[groupNum] = resEntry;
    return (OK);
}

/****************************************************************/
/*								*/
/*		ADD A RESOURCE TO A RESOURCE GROUP		*/
/*								*/
/****************************************************************/

int assignLockGroup(group, resource)
int	group;
char	*resource;
{
    RES_TABLE	*resEntry;
    RESLOCK	*newMember;
    RESLOCK	*currMember;

    /*
    Groups are numbered 1 through MAXGROUPS
    */

    if ((group <= 0) || (group > MAXGROUPS))
	return (ERROR);
    group--;

    /*
    Also have to be able to find the resource list.
    */

    if (resGetByName(resource, &resEntry) != RES_OK)
	return (ERROR);

    /*
    Allocate a structure to hold this new one and fill it out.
    */

    if ((newMember = (RESLOCK *) calloc(1, sizeof(RESLOCK))) == NULL)
	return (ERROR);
    newMember -> resEntry = resEntry;

    /*
    Hang it on the END of the chain for this group. If we put it on
    the end instead of the start of the chain we don't have to bother
    with locking the chain. There is no way to remove a resource
    from a group.
    */

    if ((currMember = resLockList[group]) == NULL)
	{
	resLockList[group] = newMember;
	return (OK);
	}

    while (currMember -> nextLock)
	currMember = currMember -> nextLock;
    currMember -> nextLock = newMember;
    return (OK);
}


/****************************************************************/
/*								*/
/*		THE RESOURCE GROUP SCAN TASK			*/
/*								*/
/****************************************************************/

void resGroupScan()
{
    short	index;
    short	newCaAnswer;
    short	localState;
    short	partnerState;

    for (;;)
	{
	for (index = 0; index < MAXGROUPS; index++)
	    {
	    if (!keyResource[index])
		continue;
	    resGetCaAnswer(keyResource[index], &newCaAnswer);
	    resGetStates(keyResource[index], &localState, &partnerState);
	    if ((!newCaAnswer) && (localState == RES_SLV))
		resGroupLock[index] = 1;
	    else
		resGroupLock[index] = 0;
	    }
	taskDelay(60);
	}

#if 0	

/* Code ifdef'd out is the "old" version */

    RESLOCK	*currResource;

for (;;)
    {
    for (index = 0; index < MAXGROUPS; index++)
	{
	if (!resGroupLock[index])
	    resGroupTripName[index] = NULL;		/* Reset back from previous trip */
	currResource = resLockList[index];
	while (currResource)
	    {
	    resGetCaAnswer(currResource -> resEntry, &newCaAnswer);
	    if (currResource -> lastCaAnswer)
		{
		if (!newCaAnswer)
		    {
		    if (localState == RES_SLV)
			{
			if (!resGroupTripName[index])
			    resGroupTripName[index] = currResource -> resEntry -> resourceName;
			resGroupLock[index] = TRUE;
			}
		    }
		}
	    currResource -> lastCaAnswer = newCaAnswer;
	    currResource = currResource -> nextLock;
	    }
	}
    taskDelay(60);
    }
#endif

}



/****************************************************************/
/*								*/
/*	INITIALIZE THE TASK THAT DOES THE RESOURCE SCANNING	*/
/*								*/
/****************************************************************/

int
initResGroupScan(void)
{
    if (taskSpawn("resGroup", 90, VX_FP_TASK, 2048, (FUNCPTR) resGroupScan,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0) == ERROR)
	return (ERROR);
    return (OK);
}
