/* i8250.h - Intel 8250 UART (Universal Asynchronous Receiver Transmitter) */

/* Copyright 1984-1996 Wind River Systems, Inc. */

/*
modification history
--------------------
01d,06jan95,vin  changed the tyCodev struct fields from USHORT to ULONGs,
		 added new macros.
01c,17feb94,hdn	 added two new members to structure TY_CO_DEV.
01b,16jun93,hdn	 updated to 5.1.
01a,15may92,hdn	 written.
*/

#ifndef	__INCi8250h
#define	__INCi8250h

#ifdef __cplusplus
extern "C" {
#endif


#ifndef _ASMLANGUAGE

#include "tyLib.h"


typedef struct			/* TY_CO_DEV */
    {
    TY_DEV tyDev;
    BOOL created;	/* true if this device has really been created */

#if (CPU_FAMILY == I80X86)	/* Registers I/O mapped, max of 64k of I/O */

    USHORT lcr;		/* UART line control register */
    USHORT lst;		/* UART line status register */
    USHORT mdc;         /* UART modem control register */
    USHORT msr;         /* UART modem status register */
    USHORT ier;		/* UART interrupt enable register */
    USHORT iid;		/* UART interrupt ID register */
    USHORT brdl;	/* UART baud rate register */
    USHORT brdh;	/* UART baud rate register */
    USHORT data;	/* UART data register */

#else				/* Registers are Memory mapped */

    ULONG lcr;		/* UART line control register */
    ULONG lst;		/* UART line status register */
    ULONG mdc;		/* UART modem control register */
    ULONG msr;		/* UART modem status register */
    ULONG ier;		/* UART interrupt enable register */
    ULONG iid;		/* UART interrupt ID register */
    ULONG brdl;		/* UART baud rate register */
    ULONG brdh;		/* UART baud rate register */
    ULONG data;		/* UART data register */
#endif /* CPU_FAMILY */
    } TY_CO_DEV;

typedef struct			/* BAUD */
    {
    int rate;		/* a baud rate */
    int preset;		/* counter preset value to write to DUSCC_CTPR[HL]A */
    } BAUD;

/*
 * The macro UART_REG_ADDR_INTERVAL must be defined
 * when including this header.
 */

/* default definitions */

#define UART_ADRS(base,reg)   (CAST (base+(reg*UART_REG_ADDR_INTERVAL)))

/* register definitions */

#define UART_THR(base)	UART_ADRS(base,0x00)	/* Transmitter holding reg. */
#define UART_RDR(base)	UART_ADRS(base,0x00)	/* Receiver data reg.       */
#define UART_BRDL(base)	UART_ADRS(base,0x00)	/* Baud rate divisor (LSB)  */
#define UART_BRDH(base)	UART_ADRS(base,0x01)	/* Baud rate divisor (MSB)  */
#define UART_IER(base)	UART_ADRS(base,0x01)	/* Interrupt enable reg.    */
#define UART_IID(base)	UART_ADRS(base,0x02)	/* Interrupt ID reg.        */
#define UART_LCR(base)	UART_ADRS(base,0x03)	/* Line control reg.        */
#define UART_MDC(base)	UART_ADRS(base,0x04)	/* Modem control reg.       */
#define UART_LST(base)	UART_ADRS(base,0x05)	/* Line status reg.         */
#define UART_MSR(base)	UART_ADRS(base,0x06)	/* Modem status reg.        */

/* interrupt Enable register */

#define I8250_IER_RDI_ENBL	0x01		/* enable receiver interrupt */
#define I8250_IER_THR_ENBL	0x02		/* enable transmit interrupt */
#define I8250_IER_RSI_ENBL	0x04		/* enable recv status intr */
#define I8250_IER_MSI_ENBL	0x08		/* enable modem starus intr */

/* interrupt identification register */

#define I8250_IIR_PI		0x00		/* pending interrupts */
#define I8250_IIR_NPI		0x01		/* no pending interrupts */
#define I8250_IIR_MS		0x00		/* modem status type */
#define I8250_IIR_THR		0x02		/* Xmit holding reg empty */
#define I8250_IIR_RD		0x04		/* receive data available */
#define I8250_IIR_RLS		0x06		/* receive line status */

#define I8250_IIR_PIMASK	0x01		/* pending interrupt mask */

/* line Control register */

#define I8250_LCR_WRD5		0x00		/* data 5 bits */
#define I8250_LCR_WRD6		0x01		/* data 6 bits */
#define I8250_LCR_WRD7		0x02		/* data 7 bits */
#define I8250_LCR_WRD8		0x03		/* data 8 bits */
#define I8250_LCR_SB1		0x00		/* stop bits 1 */
#define I8250_LCR_SB2		0x04		/* stop bits 2 */
#define I8250_LCR_PEN		0x08		/* parity enable */
#define I8250_LCR_PEV		0x10		/* parity even */
#define I8250_LCR_POD		0x00		/* parity odd */
#define I8250_LCR_SP		0x20		/* stuck parity */
#define I8250_LCR_SBE		0x40		/* set break enable */
#define I8250_LCR_DLAB		0x80		/* Divisor latch acess bit */

/* line status register */

#define I8250_LSR_DRDY		0x01		/* Data ready */
#define I8250_LSR_OERR		0x02		/* Overrun error */
#define I8250_LSR_PERR		0x04		/* Parity error */
#define I8250_LSR_FERR		0x08		/* Framing error */
#define I8250_LSR_BII		0x10		/* Break interrupt indicator */
#define I8250_LSR_THRE		0x20		/* Xmit holding reg empty */
#define I8250_LSR_THRT		0x40		/* Xmit & shift reg empty */

/* modem control register */

#define I8250_MCR_DTR		0x01		/* Data terminal ready */
#define I8250_MCR_RTS		0x02		/* Request to send */
#define I8250_MCR_OUT1		0x04		/* Output 1 spare signal */
#define I8250_MCR_OUT2		0x08		/* output 2 intr enable sig */
#define I8250_MCR_LB		0x10		/* Loop back feature */

/* modem status register */

#define I8250_MSR_DCTS		0x01		/* Data clear to Send */
#define I8250_MSR_DDSR		0x02		/* Delta data set ready */
#define I8250_MSR_TERI		0x04		/* Trailing edge ring indctr */
#define I8250_MSR_DDCD		0x08		/* Delta data carrier detect */
#define I8250_MSR_ICTS		0x10		/* inverted clear to send */
#define I8250_MSR_IDSR		0x20		/* inverted data set ready */
#define I8250_MSR_IRI		0x40		/* inverted ring indicator */
#define I8250_MSR_ICD		0x80		/* inverted carrier detect */

/* baud rate generator */

#define BAUD_9600_LSB		0x0c		/* least significant byte */
#define BAUD_9600_MSB		0x00		/* most significant byte */

#if defined(__STDC__) || defined(__cplusplus)

IMPORT  void    tyCoInt (TY_CO_DEV *pTyCoDv);

#else

IMPORT  void    tyCoInt ();

#endif  /* __STDC__ */

#endif	/* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif	/* __INCi8250h */
