/*snwsAlmake.c,v 1.1.1.1 1999/08/03 14:43:04 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
/*  File: snwsAlmake.c                                                       
/*  Edit:  01/08/97      WEP: initial code                                   
/*                                                                           
*/

#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "alarmDefs.h"
#include "resolveLinks.h"
#include "msgMon.h"

#define TRUE              1
#define FALSE             0
#define PREV_HOURS       24
#define SECS_PER_HOUR  3600
#define SECS_PER_DAY  86400
#define MAXPATH         100
#define NUMFIELDS        60
#define HELLNOTFROZEN  TRUE

#define NAMEFIELD    23
#define TYPEFIELD     0
#define YEARFIELD     4
#define MONTHFIELD    7
#define DAYFIELD     10
#define HOURFIELD    13
#define MINUTEFIELD  16
#define SECONDFIELD  19
#define EGUFIELD     49
#define DATAFIELD    32
#define STATEFIELD   33

#define DAYSPERYEAR 365

/****************************************************************/
/*                                                              */
/****************************************************************/
#define MAXDBPOINTS 1000

#define MAXFILESIZE 85000000
/*#define MAXFILESIZE 10000  */

#define MAXFILETIME 60*60*24*(365-1)      /* one year of seconds, less 1 day */

#define ZEROYEAR   94  /* file naming convention: year zero */

#define NOMSGS      0
#define SYNTAXMSGS  1
#define FILEMSGS    2
#define INPUTDATA   3
#define RECORDMSGS  4
#define BLATHERMSGS 5

/* External data */
extern  int    myPid;
        

/************************************************************************
*			PUBLIC DECLARATIONS				*
************************************************************************/

static short normalyear[13] = {0,  0, 31, 59, 90,120,151,181,212,243,273,304,334};
static short leapyear[13]   = {0,  0, 31, 60, 91,121,152,182,213,244,274,305,335};


/************************************************************/
/* this is the structure of the current parsed input record */
/* which is being processed into the current output record  */
/* for eventual output to the destination database.         */
/************************************************************/
typedef struct
{
    unsigned char    pointname[20];
    unsigned char    state[8];
    unsigned char    type[8];
    unsigned char    eventstate[20];
    double           value;
    unsigned short   count;
    time_t           gmt;
    unsigned short   year;
    unsigned short   month;
    unsigned short   day;
    unsigned short   hour;
    unsigned short   min;
    unsigned short   sec;
}       IMPORTRECORD;

static IMPORTRECORD  importrecord;
static IMPORTRECORD *imptr;

/***********************************************************/
/* this is the structure of the global working parameter   */
/*  list, used for processing all records.                 */
/***********************************************************/

typedef struct
{
    unsigned short   firstrecyear;   /*   used to build filename */
    unsigned short   firstrecday;    /*   used to build filename */
    unsigned short   lastrecday;     /*   used to build filename */

    unsigned short   flushflag;      /*   if true, flush & exit. */
    time_t           firstrecordgmt; /*   seconds since 1/1/70   */
    time_t           currentgmt;     /* * seconds since 1/1/70   */
    unsigned long    daynumber;      /*   days since 1/1/70      */

    unsigned short   yday;
    unsigned short   lastyday;

    unsigned long    secondnumber;   /*   seconds since midnight */
    unsigned long    recordnumber;   /*   records since midnight */

    time_t           daygmt;         /*   today's gmt            */
    time_t           recordgmt;      /* * this record's gmt      */

    unsigned short   numpoints;      /* number of defined database points */
}       GLOBALCONTROLS;

static GLOBALCONTROLS  globalcontrols;
static GLOBALCONTROLS *glptr;

/***********************************************************/
/* this is the structure of the current working record     */
/*  being prepared for output to the destination database. */
/***********************************************************/

typedef struct
{
    time_t           recordgmt;
    time_t           firstgmt;          /* used to create database file name */
    time_t           lastgmt;           /* used to create database file name */

    unsigned short   year;
    unsigned short   month;
    unsigned short   day;
    unsigned short   yday;               /* to be calculated */
    unsigned short   hour;
    unsigned short   min;

    unsigned char    pointname[AL_POINTNAME_SIZE+1];
    unsigned char    isack[2];
    unsigned char    alarmsecs[AL_GMTSECS_SIZE+1];
    unsigned char    alarmmsecs[AL_GMTMSECS_SIZE+1];
    unsigned char    acksecs[AL_GMTSECS_SIZE+1];
    unsigned char    ackmsecs[AL_GMTMSECS_SIZE+1];
    unsigned char    status[AL_STATUS_SIZE+1];
    unsigned char    severity[AL_SEVERITY_SIZE+1];
    unsigned char    acked[2];
    unsigned char    noprint[2];
    unsigned char    operator[AL_OPERATOR_SIZE+1];
    unsigned char    resource[AL_RESOURCE_SIZE+1];
    unsigned char    station[AL_IOCNAME_SIZE+1];
    unsigned char    groupname[AL_GROUPNAME_SIZE+1];
    unsigned char    descriptor[AL_DESCRIPTOR_SIZE+1];
    unsigned char    egutag[AL_EGUTAG_SIZE+1];
    unsigned char    value[AL_VALUE_SIZE+1];
    unsigned char    date[AL_DATE_SIZE+1];
    unsigned char    time[AL_TIME_SIZE+1];
    unsigned char    comment[11];
}       EXPORTRECORD;

static EXPORTRECORD  exportrecord;
static EXPORTRECORD *exptr;


static          long  timezoneoffset;


static int             argvindex;
       unsigned char   record[81];
       unsigned char   outbuffer[81];
       unsigned char   oldbuffer[81];
       unsigned char   savebuffer[81];


       unsigned char   debuglevel;


static FILE     *sourcehandle;


/* alarmlogfile */
CODE4	cb;
DATA4	*alarmDesc;

FIELD4 *isAckFld, *pointNameFld, *alarmSecsFld, *alarmMsecsFld, *ackSecsFld;
FIELD4 *ackMsecsFld, *statusFld, *severityFld, *ackedFld, *noprintFld;
FIELD4 *operatorFld, *resourceFld, *stationFld, *groupnameFld, *descriptorFld;
FIELD4 *egutagFld, *valueFld, *dateFld, *timeFld, *commentFld;


/******************************************************************************/
/* This is the alarm log file. It contains both alarms and acknowledgements.  */
/* An acknowledgement can be identified by non-empty ACKSEC & ACKSECS fields. */
/* The values in these fields will match the ALARMSECS & ALARMMSECS fields of */
/* the matching alarm.  The OPERATOR field indicates the username of the      */
/* operator that ACKed the alarm. The DATE & TIME fields are in local time    */
/* and are for customer convienience only.  The RECVACK indicates that the    */
/* alarm was received from CA already ACKed and no ACK record exists in the   */
/* log.                                                                       */
/******************************************************************************/

FIELD4INFO	alarmLogFields [] =
{
    { "ISACK",        'C',    1,                  0 }, /* Is an ACK entry                        */
    { "POINTNAME",    'C',    AL_POINTNAME_SIZE,  0 }, /* Point that is in alarm                 */
    { "ALARMSECS",    'C',    AL_GMTSECS_SIZE,    0 }, /* Alarm time (GMT seconds since 1/1/70)  */
    { "ALARMMSECS",   'C',    AL_GMTMSECS_SIZE,   0 }, /* Alarm time (millisecond)               */
    { "ACKSECS",      'C',    AL_GMTSECS_SIZE,    0 }, /* ACK (GMT seconds since 1/1/70)         */
    { "ACKMSECS",     'C',    AL_GMTSECS_SIZE,    0 }, /* ACK (milliseconds)                     */
    { "STATUS",       'C',    AL_STATUS_SIZE,     0 }, /* HI, HIHI, LO, LOLO, etc                */
    { "SEVERITY",     'C',    AL_SEVERITY_SIZE,   0 }, /* M, m, I, E, V                          */
    { "ACKED",        'L',    1,                  0 }, /* Alarm has been ACKed                   */
    { "NOPRINT",      'L',    1,                  0 }, /* No alarm message printed               */
    { "OPERATOR",     'C',    AL_OPERATOR_SIZE,   0 }, /* Operator that ACKed the alarm          */
    { "RESOURCE",     'C',    AL_RESOURCE_SIZE,   0 }, /* Resource that PV belongs to            */
    { "STATION",      'C',    AL_IOCNAME_SIZE,    0 }, /* Station name                           */
    { "GROUPNAME",    'C',    AL_GROUPNAME_SIZE,  0 }, /* Group to which this point belongs      */
    { "DESCRIPTOR",   'C',    AL_DESCRIPTOR_SIZE, 0 }, /* Point descriptor from pv_map           */
    { "EGUTAG",       'C',    AL_EGUTAG_SIZE,	  0 }, /* EGU tag for PV                         */
    { "VALUE",        'C',    AL_VALUE_SIZE,      0 }, /* Value of PV at alarm time              */
    { "DATE",         'C',    AL_DATE_SIZE,       0 }, /* Local date mm/dd/yy                    */
    { "TIME",         'C',    AL_TIME_SIZE,       0 }, /* Local time hh:mm:ss                    */
    { "COMMENT",      'M',    10,                 0 },
    { 0,                0,    0,                  0 }
};

TAG4INFO	alarmLogTags [] =
{
	{ "EVENTTIME",	"ALARMSECS+ALARMMSECS+POINTNAME+ISACK+STR(RECNO(),8,0)", "",	0,	0 },
	{ 0, 0, 0, 0, 0 }
};





void dumpimport(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: dump import record :::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf(":::: %s", record);  /* debug printing */
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: Name:[%s] Type = %s, RawData = %s\n",
             imptr->pointname,
             imptr->type,
             imptr->eventstate);
    printf ("::::                                               \tValue = %f\n", imptr->value);
    printf (":::: RecTime = %s", ctime(&imptr->gmt));
    printf (":::: ------------------------------------------------------------------------\n");
    printf (":::: RecGMT  =%d, Year=%d, Month=%d, Day=%d, Hour=%d, Min=%d, Sec=%d\n",
             imptr->gmt,
             imptr->year,
             imptr->month,
             imptr->day,
             imptr->hour,
             imptr->min,
             imptr->sec);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


void dumptime(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: dump global time stuff :::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: DayNum= %d, SecNum = %d, RecNum = %d\n",
             glptr->daynumber,
             glptr->secondnumber,
             glptr->recordnumber);
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: DayGMT\t\t= %d,   %s",
             glptr->daygmt,
             ctime(&glptr->daygmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: RecGMT\t\t= %d,   %s",
             glptr->recordgmt,
             ctime(&glptr->recordgmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: currentgmt\t\t= %d,   %s",
             glptr->currentgmt,
             ctime(&glptr->currentgmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


void dumpglobal(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: RecGMT\t= %d, %s",
             glptr->recordgmt,
             ctime(&glptr->recordgmt));
    printf (":::: Name:[%s] EGU = %s \n",
             imptr->pointname,
             imptr->eventstate);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    return;


    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: global controls - begin ::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: DayNum= %d, SecNum = %d, RecNum = %d\n",
             glptr->daynumber,
             glptr->secondnumber,
             glptr->recordnumber);
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: firstrecordgmt\t= %d,   %s",
             glptr->firstrecordgmt,
             ctime(&glptr->firstrecordgmt));
    printf (":::: DayGMT\t\t= %d,   %s",
             glptr->daygmt,
             ctime(&glptr->daygmt));
    printf (":::: RecGMT\t\t= %d,   %s",
             glptr->recordgmt,
             ctime(&glptr->recordgmt));
    printf (":::: currentgmt\t\t= %d,   %s",
             glptr->currentgmt,
             ctime(&glptr->currentgmt));
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: global controls - end   ::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


void dumpmini(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: Name:[%s] Type = %s, EGU = %s \n",
             imptr->pointname,
             imptr->type,
             imptr->eventstate);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    return;


    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: mini dump - begin ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: Name:[%s] Type = %s, EGU = %s, Value = %f\n",
             imptr->pointname,
             imptr->type,
             imptr->eventstate,
             imptr->value);
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: DayNum = %d, SecNum = %d, RecNum = %d\n",
             glptr->daynumber,
             glptr->secondnumber,
             glptr->recordnumber);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: mini dump - end   ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


/* put debug stuff here */
void dumpexport(int i)
{

    printf ("-------------------------------------------------------------------------\n");
    printf ("---- export record: record number%8d -------------------------------\n",i);
    printf ("---- Name:[%8s]  State: '%s', Alarmsecs: %s, date: '%s', time: '%s'\n",
             exptr->pointname, exptr->value, exptr->alarmsecs, exptr->date, exptr->time);
    printf ("-------------------------------------------------------------------------\n");
    return;



    printf ("---------------------------------------------------------------------------------\n");
    printf ("---- export record: record number%8d ---------------------------------------\n",i);
    printf ("---------------------------------------------------------------------------------\n");
    printf ("---- Name:[%s] EGU = %s\n",
             exptr->pointname,
             exptr->egutag);
    printf ("::::                 Year = %d, DOY = %d,  Hour = %d, OffMin = %d\n",
             exptr->year,
             exptr->yday,
             exptr->hour,
             exptr->min);
    printf ("---------------------------------------------------------------------------------\n");
    printf ("-----export record - end --------------------------------------------------------\n");
    printf ("---------------------------------------------------------------------------------\n");
}


/*****************************/
/* Database initialization.  */
/*****************************/

int dbinit (void)
{
    d4init(&cb);
#if defined(USECB63) || defined(WIN32)
    cb.errOpen = 0;
    cb.errTagName = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
    cb.errGo = 0;
    cb.errSkip = 0;
#else
    cb.open_error = 0;
    cb.tag_name_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
    cb.go_error = 0;
    cb.skip_error = 0;
#endif	WIN32

    return (TRUE);

}

/*****************************************************/
/* Open the output database.  If it does not exist,  */
/* then create it and return false indicating that   */
/* it did not previously exist.                      */
/*****************************************************/

int openzfile (void)
{
    char	pathName [200];
    TAG4	*t;

    resolveLinks ("wurkfile", pathName);
    alarmDesc = d4open (&cb, pathName);
    if (alarmDesc == NULL)
    {
        if (debuglevel >= BLATHERMSGS)
        {
            printf("Calling d4create (0x%x, %s, 0x%x, 0x%x) ....\n", &cb, pathName, alarmLogFields, alarmLogTags);
        }

        alarmDesc = d4create (&cb, pathName, alarmLogFields, alarmLogTags);
        if (alarmDesc == NULL)
        {
            return (FALSE);
        }
    }

    if (debuglevel >= BLATHERMSGS)
    {
        printf("Calling d4tag .........\n");
    }
    t = d4tag (alarmDesc, "EVENTTIME");
    if (debuglevel >= BLATHERMSGS)
    {
        printf("Calling d4tag_select ..........\n");
    }
    d4tag_select (alarmDesc, t);
    if (debuglevel >= BLATHERMSGS)
    {
        printf("Calling d4top .........\n");
    }
    d4top (alarmDesc);
    if (debuglevel >= BLATHERMSGS)
    {
        printf("Calling d4unlock_all .........\n");
    }
    d4unlock_all (alarmDesc);

    /* Open up all the fields */

    isAckFld      = d4field (alarmDesc, "ISACK");
    pointNameFld  = d4field (alarmDesc, "POINTNAME");
    alarmSecsFld  = d4field (alarmDesc, "ALARMSECS");
    alarmMsecsFld = d4field (alarmDesc, "ALARMMSECS");
    ackSecsFld    = d4field (alarmDesc, "ACKSECS");
    ackMsecsFld   = d4field (alarmDesc, "ACKMSECS");

    statusFld     = d4field (alarmDesc, "STATUS");
    severityFld   = d4field (alarmDesc, "SEVERITY");
    ackedFld      = d4field (alarmDesc, "ACKED");
    noprintFld    = d4field (alarmDesc, "NOPRINT");
    operatorFld   = d4field (alarmDesc, "OPERATOR");
    resourceFld   = d4field (alarmDesc, "RESOURCE");
    stationFld    = d4field (alarmDesc, "STATION");
    groupnameFld  = d4field (alarmDesc, "GROUPNAME");
    descriptorFld = d4field (alarmDesc, "DESCRIPTOR");
    egutagFld     = d4field (alarmDesc, "EGUTAG");
    valueFld      = d4field (alarmDesc, "VALUE");

    dateFld       = d4field (alarmDesc, "DATE");
    timeFld       = d4field (alarmDesc, "TIME");
    commentFld    = d4field (alarmDesc, "COMMENT");
    if (debuglevel >= BLATHERMSGS)
    {
        printf("All d4file calls completed.\n");
    }

    return (TRUE);
}

/*****************************************************/
/* Close the output database. After closing the      */
/* active file, rename it using the start and stop   */
/* dates to create the new name. Delete any file     */
/* with the same name.                               */
/*****************************************************/

int closezfile (void)
{
    char	pathName1 [200], pathName2 [200];
    char	buffera [10], bufferb [10], bufferc [20], bufferd [20];

    /**********************/
    /* close the database */
    /**********************/
    if (debuglevel >= FILEMSGS)
    {
        printf("**************************\n");
        printf("Closing wurkfile database.\n");
        printf("**************************\n");
    }
    d4close(alarmDesc);

    /* rename the old database based on the start and stop dates. */
    /* get them from the first and last timestamp dates of the    */
    /* database, which are in the global data structure.          */

    sprintf(buffera, "%1d%1d%1d",
             glptr->firstrecday/100,
            (glptr->firstrecday/10)%10,
             glptr->firstrecday%10);
    sprintf(bufferb, "%1d%1d%d",
             (glptr->lastyday+1)/100,
            ((glptr->lastyday+1)/10)%10,
             (glptr->lastyday+1)%10);
    sprintf(bufferc, "EVT_%1d%d", glptr->firstrecyear/10, glptr->firstrecyear%10);
    strcat (bufferc, buffera);
    strcat (bufferc, bufferb);

    /* build the .dbx file pathnames and rename the file */
    strcpy (bufferd, bufferc);
    strcat (bufferd, ".dbf");
    resolveLinks (bufferd, pathName2);
    resolveLinks ("wurkfile.dbf", pathName1);
    if (debuglevel >= FILEMSGS)
    {
        printf("Renaming wurkfile.dbf to %s\n", bufferd);
    }
    if (rename (pathName1, pathName2))
    {
        return (FALSE);
    }

    /* build the .mdx file pathnames and rename the file */
    strcpy (bufferd, bufferc);
    strcat (bufferd, ".mdx"); 
    resolveLinks (bufferd, pathName2);
    resolveLinks ("wurkfile.mdx", pathName1);
    if (debuglevel >= FILEMSGS)
    {
        printf("Renaming wurkfile.mdx to %s\n", bufferd);
    }
    if (rename (pathName1, pathName2))
    {
        return (FALSE);
    }

    /* build the .dbt file pathnames and rename the file */
    strcpy (bufferd, bufferc);
    strcat (bufferd, ".dbt"); 
    resolveLinks (bufferd, pathName2);
    resolveLinks ("wurkfile.dbt", pathName1);
    if (debuglevel >= FILEMSGS)
    {
        printf("Renaming wurkfile.dbt to %s\n", bufferd);
    }
    if (rename (pathName1, pathName2))
    {
        return (FALSE);
    }
    return (TRUE);
}


/**************************************************/
/* determine the size of the output database file */
/* (*.dbf) and decide if it is time to close it.  */
/**************************************************/

long sizezfile (void)
{
    char          pathName [200];
    struct stat   sbuffer;
    struct stat  *sptr;
    long          size;

    sptr = &sbuffer;

    resolveLinks ("wurkfile.dbf", pathName);
    stat(pathName, sptr);
    size = (long) sbuffer.st_size;
    return (size);
}


/*******************************************************/
/* determine the time span of the output database file */
/* (*.dbf) and decide if it is time to close it.       */
/*******************************************************/

long timezfile (void)
{
    long    timespan;

    timespan = glptr->recordgmt - glptr->firstrecordgmt;
    return (timespan);
}


/*****************************************************/
/* get another raw record from the file and fix it.  */
/* if this file is done, read another file. if no    */
/* more files, then set flag and return nothing.     */
/*****************************************************/
int nextrecord(int argc, char *argv[])
{
    unsigned char *baptr;

    /**********************************/
    /**********************************/
    static struct tm     *tmTime;
    static unsigned long  absolutetime;
    static unsigned long  previoustime;
    static          long  deltatime;
    
    static time_t         tt = 0;
    static unsigned long  dbrecnum;

    /**********************************/
    /**********************************/

    unsigned short year, month, day, hour, min, sec;
    unsigned long  totaldays;
    unsigned long  totalsecs;
             short offsetyear, leapdays;

    /**********************************/
    /**********************************/

    /* Get access to a tm structure */

    tmTime = gmtime (&tt);

    imptr = &importrecord;
    baptr = &record[0];

    while (HELLNOTFROZEN)
    {
        /* if first time through, or previous file has been closed ... */
        if (!sourcehandle)
        {
            /* see if we are out of file names to process */
            if (!(argv[argvindex]))
            {
                if (debuglevel >= FILEMSGS)
                {
                    printf(":::: ++++++++++++++++++++++++++\n"); /* debug printing */
                    printf("********* No more input files to process: flush all output buffers *********\n");
                }
                glptr->flushflag = TRUE;
                return (TRUE);
            }

            if (debuglevel >= FILEMSGS)
            {
                printf("******************* OPENING THE %s FILE *******************\n",
                        argv[argvindex]);
            }

            /* Open File for Read Only */

            sourcehandle = fopen(argv[argvindex],"r");
            if (sourcehandle == NULL)
            {
                /* if file open is unsuccessful, there is no new record to return. */
                if (debuglevel >= FILEMSGS)
                {
                    printf("ERROR: Cannot open source database file (%s)\n", argv[argvindex]);
                }
                return(FALSE);
            }
            dbrecnum = 0;
            argvindex++;
        }

        /* file is currently open. get the next record. */
        if (!fgets(record, 80, sourcehandle))
        {
            /************************************************/
            /* if you cannot get another record, then there */
            /* is nothing left in the file. close the file  */
            /************************************************/
            if (debuglevel >= FILEMSGS)
            {
                printf("******************* CLOSING THE %s FILE *******************\n",
                        argv[argvindex-1]);
            }
            fclose (sourcehandle);

            /****************************/
            /* set the file closed flag */
            /****************************/
            sourcehandle = NULL;
            break;
        }

        /* if it is the right kind of record, its a good one. fix it up and keep it */
        if ((record[0]=='E') &&
            (record[1]=='V') &&
            (record[2]=='T') &&

            /**************************************************************/
            /* TEMPORARILY filter to accept an arbitrary subset of points */
            /**************************************************************/

/*
            (record[NAMEFIELD+0]=='P') &&
            (record[NAMEFIELD+1]=='P') &&
            (record[NAMEFIELD+2]=='1') &&
            (record[NAMEFIELD+3]=='P') &&
            (record[NAMEFIELD+4]=='0') &&
/*
            (record[NAMEFIELD+5]=='7') &&
*/

            (record[3]==' '))
        {

            /******************/
            /* parse the data */
            /******************/

            /* copy the input buffer to the output buffer */
            strcpy(&outbuffer[0], &record[0]);
            strcpy(&savebuffer[0], &outbuffer[0]);

            if (debuglevel >= INPUTDATA)
            {
                printf("#### %s", record);  /* debug printing */
/*                dumpimport(); */
            }

            /* insert space in place of newline character */
            if ((baptr = strchr(&outbuffer[NAMEFIELD], 0x0d  ))) *baptr = ' ';
            if ((baptr = strchr(&outbuffer[NAMEFIELD], 0x0a  ))) *baptr = ' ';
            /* insert nulls at the end of all string fields */
            if ((baptr = strchr(&outbuffer[NAMEFIELD], ' '  ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[TYPEFIELD], ' '  ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[MONTHFIELD], ' ' ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[DAYFIELD], ' '   ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[HOURFIELD], ' '  ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[MINUTEFIELD], ' '))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[SECONDFIELD], ' '))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[STATEFIELD], ' ' ))) *baptr = 0;

            strcpy (imptr->pointname, &outbuffer[NAMEFIELD]);

            /**************************************************************/
            /* Grant's last request: drop records with missing point name */
            /**************************************************************/
            if (strlen(imptr->pointname) == 0) continue;

            /* now that we have decide to keep this record, bump the record counter. */
            dbrecnum++;

            strcpy (imptr->type, &outbuffer[TYPEFIELD]);
            strcpy (imptr->state, &outbuffer[STATEFIELD]);

            imptr->year =  (unsigned char) atoi(&outbuffer[YEARFIELD]);
            imptr->month = (unsigned char) atoi(&outbuffer[MONTHFIELD]);
            imptr->day =   (unsigned char) atoi(&outbuffer[DAYFIELD]);
            imptr->hour =  (unsigned char) atoi(&outbuffer[HOURFIELD]);
            imptr->min =   (unsigned char) atoi(&outbuffer[MINUTEFIELD]);
            imptr->sec =   (unsigned char) atoi(&outbuffer[SECONDFIELD]);

            strcpy (imptr->eventstate, &outbuffer[STATEFIELD]);

            year  = (unsigned char) atoi(&outbuffer[YEARFIELD]);
            month = (unsigned char) atoi(&outbuffer[MONTHFIELD]);
            day   = (unsigned char) atoi(&outbuffer[DAYFIELD]);
            hour  = (unsigned char) atoi(&outbuffer[HOURFIELD]);
            min   = (unsigned char) atoi(&outbuffer[MINUTEFIELD]);
            sec   = (unsigned char) atoi(&outbuffer[SECONDFIELD]);


            /* handle century wrap. time does not exist before Jan 1, 1970. */
            if ((offsetyear = (year - 70)) < 0) offsetyear += 100;

            /* make adjustment for leap days in preceding years */
            leapdays = ((offsetyear + 1)/4);

            if (year%4 == 0)
            {
                totaldays = (offsetyear * DAYSPERYEAR) +
                             leapdays +
                             leapyear[month] +
                             (day - 1);
            }
            else
            {
                totaldays = (offsetyear * DAYSPERYEAR) +
                             leapdays +
                             normalyear[month] +
                             (day - 1);
            }
            totalsecs = (((((totaldays * 24) + hour) * 60 ) + min) * 60) + sec;

            /**********************************************************************/
            /**********************************************************************/
            /**********************************************************************/

            /* Convert the struct tm to seconds since the dawn of time */
            tmTime->tm_year  = imptr->year;
            tmTime->tm_mon   = imptr->month - 1;
            tmTime->tm_mday  = imptr->day;
            tmTime->tm_hour  = imptr->hour;
            tmTime->tm_min   = imptr->min;
            tmTime->tm_sec   = imptr->sec;

/* get gmt using system calls. */

#if (defined (SOLARIS))
            absolutetime = mktime (tmTime);
#else
            absolutetime = timegm (tmTime);
#endif
            /* add in the time zone adjustment factor from the command line. */
            absolutetime += timezoneoffset;



            
            /***************************************************/
            /***************************************************/
            /* Convert absolute seconds back to a tm structure */
            /***************************************************/
            /***************************************************/
            tt = absolutetime;
            tmTime = gmtime (&tt);
            /***************************************************/
            /***************************************************/





            /* store the components of the GMT in the input record */
            glptr->yday  = tmTime->tm_yday;

            /* store the absolute GMT time */
            imptr->gmt = absolutetime;
            glptr->currentgmt = imptr->gmt;           /* seconds since 1/1/70   */
            glptr->recordgmt = imptr->gmt;            /* seconds since 1/1/70   */

            deltatime = absolutetime - previoustime;



/*
            if (debuglevel >= INPUTDATA)
            {
                printf("imptr->hour = %d, imptr->min = %d, imptr->sec = %d.\n",
                imptr->hour, imptr->min, imptr->sec); 
            }
*/




            if (deltatime < 0)
            {
                if (debuglevel >= SYNTAXMSGS)
                {
                    printf("*************************************************************\n");
                    printf("******************** TIME SEQUENCE ERROR ********************\n");
                    printf("*************************************************************\n");
                    printf("%s\n", oldbuffer);
                    printf("%s\n", savebuffer);
                    printf("-------------------------------------------------------------\n");
                    printf("DBrecnum = %d, Lasttime = %d, Thistime = %d, Delta = %d\n",
                            dbrecnum, previoustime, absolutetime, deltatime);
                    printf("*************************************************************\n");
                    printf("*************************************************************\n");
                    printf("*************************************************************\n");
                    printf("\n");
                }
            }
            strcpy(&oldbuffer[0], &savebuffer[0]);
            previoustime = absolutetime;

            return(TRUE);

        }
        /* else circle around and get another record */
    }
}

/********************************************************************
                                                                    *
   create a record and write it into the destination db file.       *
                                                                    *
*********************************************************************/

void AddToDestination(D4DATA *alarmDesc)
{
    d4top(alarmDesc);

    if (d4deleted(alarmDesc))
    {
        if (debuglevel >= RECORDMSGS)
        {
            printf ("**** Codebase Write Old Record: %s ****\n", exptr->pointname);
        }
        f4assign     (pointNameFld, exptr->pointname);
        f4assign     (isAckFld, exptr->isack);
        f4assign     (alarmSecsFld, exptr->alarmsecs);
        f4assign     (alarmMsecsFld, exptr->alarmmsecs);
        f4assign     (ackSecsFld, exptr->acksecs);

        f4assign     (ackMsecsFld, exptr->ackmsecs);
        f4assign     (statusFld, exptr->status);
        f4assign     (severityFld, exptr->severity);
        f4assign     (ackedFld, exptr->acked);
        f4assign     (noprintFld, exptr->noprint);

        f4assign     (operatorFld, exptr->operator);
        f4assign     (resourceFld, exptr->resource);
        f4assign     (stationFld, exptr->station);
        f4assign     (groupnameFld, exptr->groupname);
        f4assign     (descriptorFld, exptr->descriptor);

        f4assign     (egutagFld, exptr->egutag);
        f4assign     (valueFld, exptr->value);
        f4assign     (dateFld, exptr->date);
        f4assign     (timeFld, exptr->time);
        f4assign     (commentFld, exptr->comment);

        d4recall(alarmDesc);
    }
    else
    {
        if (debuglevel >= BLATHERMSGS)
        {
            printf ("**** Codebase Write New Record: %s ****\n", exptr->pointname);
        }
        d4append_start( alarmDesc, 0 );   /* '0' => no memo fields */
        d4blank( alarmDesc );

        f4assign     (pointNameFld, exptr->pointname);
        f4assign     (isAckFld, exptr->isack);
        f4assign     (alarmSecsFld, exptr->alarmsecs);
        f4assign     (alarmMsecsFld, exptr->alarmmsecs);
        f4assign     (ackSecsFld, exptr->acksecs);

        f4assign     (ackMsecsFld, exptr->ackmsecs);
        f4assign     (statusFld, exptr->status);
        f4assign     (severityFld, exptr->severity);
        f4assign     (ackedFld, exptr->acked);
        f4assign     (noprintFld, exptr->noprint);

        f4assign     (operatorFld, exptr->operator);
        f4assign     (resourceFld, exptr->resource);
        f4assign     (stationFld, exptr->station);
        f4assign     (groupnameFld, exptr->groupname);
        f4assign     (descriptorFld, exptr->descriptor);

        f4assign     (egutagFld, exptr->egutag);
        f4assign     (valueFld, exptr->value);
        f4assign     (dateFld, exptr->date);
        f4assign     (timeFld, exptr->time);
        f4assign     (commentFld, exptr->comment);

        d4append( alarmDesc );
        d4flush_all( alarmDesc );
    }
    d4unlock_all(alarmDesc);
    return;
}

/***********************************************************************************/
/***********************************************************************************/
/***********************************************************************************/
/***********************************************************************************/
/***********************************************************************************/

void usage(void)
{
    printf("Usage:\n");
    printf(" snwsAlmake [-dn] [-th] {Source...}       where:");
    printf("\t[-dn]\t= optional debug trace level (n range: 0 - 7, n default: 1)");
    printf("\t[-th]\t= optional time zone adjustment. this value is added");
    printf("                  to the calculated GMT. (h range: -23 + 23 hours)");
    printf("\t{Source}\t= complete path to file(s) to be imported.");
}

/***********************************************************************************/
/*                                                                                 */
/* snwsAlmake infilename [...infilename]                                           */
/*                                                                                 */
/*  This program imports one or more sequential foreign database files into        */
/*  an archived alarm/event database. All of the 'EVT' points in the foreign       */
/*  database will be inserted into the alarm log as events occuring at the time    */
/*  represented in the foreign database. The size of the output trend files will   */
/*  be limited to approximately 85 Mbytes. Multiple files will be created when     */
/*  this file size is reached. Files will be named according to a previously       */
/*  specified date-based naming convention.                                        */
/*                                                                                 */
/***********************************************************************************/

main(int argc, char *argv[])

{
    static struct tm  *tmTime;

    FILE           *tempsource;
    FILE           *tempdest;
    unsigned short  firstpass;
    unsigned long   recordindex;
             long   timezonehours;

    char            work[10];
    unsigned long   i;

    tisOpenLog ("snwsAlmake", 6, 2, NULL, NULL);


    /*****************************/
    /* initalize global pointers */
    /*****************************/
    imptr = &importrecord;
    exptr = &exportrecord;
    glptr = &globalcontrols;


    /***************************/
    /* no source file is open. */
    /***************************/
    sourcehandle = NULL;
    argvindex = 1;
    debuglevel = RECORDMSGS;

    /***************************************************************/
    /* determine that the correct number of parameters was passed. */
    /***************************************************************/
    if (argc < 2 )
    {
        tisSysLog(0,0,"Too few command line arguments:");
        usage();
        exit(1);
    }

    timezoneoffset = 0;

    /**********************************************************/
    /* check to see if debug level specified in command line. */
    /**********************************************************/
    strcpy (&work[0], argv[argvindex]);
    while (work[0] == '-')
    {
        if (work[1] == 'd')
        {
            debuglevel = atoi(&work[2]);
            tisSysLog(0,0,"Debug level set to %d.", debuglevel);
            argvindex++;
        }
        else if (work[1] == 't')
        {
            timezonehours = atoi(&work[2]);
            if ((timezonehours < -23) || (timezonehours > 23))
            {
                tisSysLog(0,0,"Invalid timezone offset value(%d):", timezonehours);
                usage();
                exit(1);
            }

            timezoneoffset = (atoi(&work[2])*60*60); /* expressed in seconds */
            tisSysLog(0,0,"Timezone offset = %d(%d).", 
			timezoneoffset, atoi(&work[2]));
            argvindex++;
        }
        else
        {
            tisSysLog(0,0,"Invalid command line option parameter: %c",work[1]);
            usage();
            exit(1);
        }
        strcpy (&work[0], argv[argvindex]);
    }

    /*******************************************************************************/
    /* open the source file, verifying it's existence: if it does not exist, exit. */
    /*******************************************************************************/
    if ((tempsource = fopen(argv[argvindex],"r")) == NULL) /* Open File for Read Only */
    {
        if (debuglevel >= SYNTAXMSGS)
        {
            printf("ERROR : Cannot open source database file.\n");
        }
        usage();
        exit(1);
    }

    /******************************************************/
    /* this was just diagnostic. for now, close the file  */
    /******************************************************/
    fclose (tempsource);

    /************************************************************************************/
    /* open the destination file, verifying it's NON-existence: if it does exist, exit. */
    /************************************************************************************/
    if ((tempdest = fopen("wurkfile.dbf","r")) != NULL)      /* Open File for Read Only */
    {
        fclose (tempdest);
        if (debuglevel >= SYNTAXMSGS)
        {
            printf("ERROR: Destination database file already exists.\n");
        }
        usage();
        exit(1);
    }

    /************************************************************************************/
    /* open the destination file, verifying it's NON-existence: if it does exist, exit. */
    /************************************************************************************/
    if ((tempdest = fopen("wurkfile.mdx","r")) != NULL)      /* Open File for Read Only */
    {
        fclose (tempdest);
        if (debuglevel >= SYNTAXMSGS)
        {
            printf("ERROR: Destination database file already exists.\n");
        }
        usage();
        exit(1);
    }


    /*****************************************************************/
    /* create the destination file. do codebase initalization magic. */
    /*****************************************************************/
    dbinit();

    /* Open destination file, get and select tags, position to the top of the file */
    if (!openzfile())
    {
        exit(1);
    }

    /******************************/
    /******************************/

    glptr->flushflag = FALSE;
    firstpass = TRUE;
    i = 0;
    recordindex = 0;
    /**************/
    /* lets rock! */
    /**************/

    if (debuglevel >= FILEMSGS)
    {
        printf ("*********************************************************\n");
        printf ("********* Begin Processing Input Records .... ***********\n");
        printf ("*********************************************************\n");
    }

    while (nextrecord (argc, argv))
    {
        recordindex++;

        /**************************************/
        /* process each record sequentially.  */
        /**************************************/

        if ((!firstpass)/* && (glptr->flushflag)*/)
        {
            /* calculate the day since 1/1/70 from the seconds since 1/1/70   */
            glptr->daynumber = glptr->currentgmt / SECS_PER_DAY;

            /* break down record's start time and store for potential use */
            /* at a later time for renamed database filename composition. */
            glptr->lastrecday  = tmTime->tm_yday + 1;
            tmTime = gmtime (&glptr->recordgmt);

/*            glptr->lastrecday  = tmTime->tm_yday + 1; */

            exptr = &exportrecord;

            /* break down this record's start time */
            tmTime = gmtime (&glptr->recordgmt);
            exptr->year  = tmTime->tm_year;
            exptr->month = tmTime->tm_mon;
            exptr->day   = tmTime->tm_mday;
            exptr->yday  = tmTime->tm_yday;
            exptr->hour  = tmTime->tm_hour;
            exptr->min   = tmTime->tm_min;

/*
            if (debuglevel >= RECORDMSGS)
            {
                printf ("***** end:   write %d alarmDesc records *****\n", glptr->numpoints);
            }
*/


            /**********************************************************/
/*            printf ("***** DAY NUMBER *****  (%d)\n", glptr->daynumber);
            /**********************************************************/


/*
            if ((glptr->yday) != (glptr->lastyday))
            {
                printf ("************************** \n");
                printf ("************************** \n");
                printf ("************************** \n");
                printf ("***** DAY TRANSITION ***** (%d -> %d)\n", glptr->lastyday, glptr->yday);
                printf ("************************** \n");
                printf ("************************** \n");
                printf ("************************** \n");
            }
*/
            /*********************************************************************/
            /* Before the record is written, check to see if it is time to close */
            /* this database, rename it, and open a new one. Of course, make     */
            /* sure that we are on an even day boundary (unless eof flushing).   */
            /*********************************************************************/
            if (
                (glptr->flushflag) ||
                (
                 ((glptr->yday) != (glptr->lastyday)) &&
                 (
                  (sizezfile() > MAXFILESIZE) ||
                  (timezfile() > MAXFILETIME)
                 )
                )
               )
            {
                /* close the current database and rename the file database files */
                closezfile();

                if (glptr->flushflag)
                {
                    if (debuglevel >= FILEMSGS)
                    {
                        printf ("*********************************************************\n");
                        printf ("********** Finished Processing Input Records ************\n");
                        printf ("*********************************************************\n");
                    }
                    /***********/
                    /* go home */
                    /***********/
                    exit(0);
                }

                /*****************************************************************/
                /* create the destination file. do codebase initalization magic. */
                /*****************************************************************/
                if (debuglevel >= BLATHERMSGS)
                {
                    printf ("Calling dbinit .....\n");
                }

                dbinit();

                if (debuglevel >= BLATHERMSGS)
                {
                    printf ("...... Call to dbinit completed.\n");
                }


                if (debuglevel >= BLATHERMSGS)
                {
                    printf ("Calling openzfile .....\n");
                }

                /* Open destination file, get and select  */
                /* tags, position to the top of the file. */
                if (!openzfile())
                {
                    exit(1);
                }

                if (debuglevel >= BLATHERMSGS)
                {
                    printf ("...... Call to openzfile completed.\n");
                }

                /******************************************************************/
                /* this value unmodifed during life of this output database file. */
                /******************************************************************/
                glptr->recordnumber = 0;
                glptr->firstrecordgmt = glptr->recordgmt;

                /* break down record's start time and store for later file name composition. */
                tmTime = gmtime (&glptr->recordgmt);
                glptr->firstrecyear = tmTime->tm_year - ZEROYEAR;
                glptr->firstrecday  = tmTime->tm_yday + 1;

                if (debuglevel >= FILEMSGS)
                {
                    printf(":::: This is the first record to be written to this output file.\n");
                    dumpglobal();
/*                    dumpexport(i); */
                }

                recordindex = 1;

                /********************/
                /* lets rock again! */
                /********************/
                if (debuglevel >= FILEMSGS)
                {
                    printf ("*********************************************************\n");
                    printf ("********* Continue Processing Input Records .... ********\n");
                    printf ("*********************************************************\n");
                }

                /* OK, now that the old file is flushed, finish building the current */
                /* record fields and write the current record in the new file.       */
            }

            /***********************************************************/
            /* write the record and re-initialize for the next record. */
            /***********************************************************/

            if (debuglevel >= BLATHERMSGS)
            {
                printf ("***** begin: write %d alarmDesc records *****\n", glptr->numpoints);
            }

            exptr = &exportrecord;
            strcpy (&exptr->pointname[0], &imptr->pointname[0]);
            sprintf(&exptr->alarmsecs[0], "%d", glptr->currentgmt);

            strcpy (&exptr->value[0],    &imptr->state[0]);
            sprintf(&exptr->date[0], "%2d/%1d%1d/%1d%1d",
                                     imptr->month,
                                     (imptr->day)/10,
                                     (imptr->day)%10,
                                     (imptr->year)/10,
                                     (imptr->year)%10);
            sprintf(&exptr->time[0], "%2d:%1d%1d:%1d%1d",
                                     imptr->hour,
                                     (imptr->min)/10,
                                     (imptr->min)%10,
                                     (imptr->sec)/10,
                                     (imptr->sec)%10);

            if (debuglevel >= RECORDMSGS)
            {
                /* diagnostic only: print each of the records */
                dumpexport(recordindex);
/*                printf ("    ====  Year = %d, DOY = %d,  Hour = %d, OffMin = %d\n",
                         exptr->year,
                         exptr->yday,
                         exptr->hour,
                         exptr->min);
                printf ("    ====                 FullYear = %d, Month = %d,  Day = %d\n",
                         exptr->year + 1900,
                         exptr->month,
                         exptr->day);
*/
            }

            /* write the record for this point */
            AddToDestination(alarmDesc);


        }

        if (firstpass)
        {
            firstpass = FALSE; /* clear it */

            /* get the first record's time to do initialization processing. */
            if ((!nextrecord (argc, argv)) || (sourcehandle == NULL))
            {
                /* something is wrong; there should be at least one record in the file. */
                /* before exiting the program, close the codebase database file.        */
                d4close(alarmDesc);
                exit (0);
            }


            /* calculate the day since 1/1/70 from the seconds since 1/1/70   */
            glptr->daynumber = glptr->currentgmt / SECS_PER_DAY;


            /********************************************************************/
            /* these values unmodifed during life of this output database file. */
            /********************************************************************/
            glptr->firstrecordgmt = glptr->recordgmt;

            /* break down the first record time and store for later file name composition. */
            tmTime = gmtime (&glptr->currentgmt);
            glptr->firstrecyear = tmTime->tm_year - ZEROYEAR;
            glptr->firstrecday  = tmTime->tm_yday + 1;

            /* initialize all 'static' database fields. */
            strncpy (exptr->isack, " ", 2);
/*            strncpy (exptr->alarmsecs, " ", AL_GMTSECS_SIZE); */
            strncpy (exptr->alarmmsecs, " ", AL_GMTMSECS_SIZE);
            strncpy (exptr->acksecs, " ", AL_GMTSECS_SIZE);

            strncpy (exptr->ackmsecs, " ", AL_GMTMSECS_SIZE);
/*            strncpy (exptr->status, " ", AL_STATUS_SIZE); */
            strncpy (exptr->severity, "EVENT", AL_SEVERITY_SIZE);
            strncpy (exptr->acked, " ", 2);
            strncpy (exptr->noprint, " ", 2);

            strncpy (exptr->operator, " ", AL_OPERATOR_SIZE);
            strncpy (exptr->resource, " ", AL_RESOURCE_SIZE);
            strncpy (exptr->station, " ", AL_IOCNAME_SIZE);
            strncpy (exptr->groupname, " ", AL_GROUPNAME_SIZE);
            strncpy (exptr->descriptor, " ", AL_DESCRIPTOR_SIZE);

            strncpy (exptr->egutag, " ", AL_EGUTAG_SIZE);
            strncpy (exptr->value, " ", AL_VALUE_SIZE);
            strncpy (exptr->date, " ", AL_DATE_SIZE);
            strncpy (exptr->time, " ", AL_TIME_SIZE);
            strncpy (exptr->comment, "IMPORTED", 11);

            if (debuglevel >= FILEMSGS)
            {
                printf(":::: This is the first record to be processed in this file.\n");
                dumpglobal();
                dumpexport(i);
            }
        }
        /*****************************************************************************************/
        /* either way, we are now aimed at the current export record. begin processing the input */
        /*****************************************************************************************/

        glptr->lastyday = glptr->yday;
    }

    /********************************/
    /* close the CodeBase databases */
    /********************************/
    d4close(alarmDesc);

    /***********/
    /* go home */
    /***********/
    exit(0);

}

