/*snwsDbmake.c,v 1.1.1.1 1999/08/03 14:43:04 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
/*  File: snwsDbmake.c                                                       
/*  Edit:  12/20/96      WEP: initial code                                   
/*                                                                           
*/


#include <stdio.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "mlist.h"
#include "trendDefs.h"
#include "resolveLinks.h"
#include "msgMon.h"


#define TRUE              1
#define FALSE             0
#define PREV_HOURS       24
#define SECS_PER_HOUR  3600
#define SECS_PER_DAY  86400
#define MAXPATH         100
#define NUMFIELDS        60
#define HELLNOTFROZEN  TRUE

#define NAMEFIELD    23
#define TYPEFIELD     0
#define YEARFIELD     4
#define MONTHFIELD    7
#define DAYFIELD     10
#define HOURFIELD    13
#define MINUTEFIELD  16
#define SECONDFIELD  19
#define EGUFIELD     49
#define DATAFIELD    32
#define DATAXFIELD   42
#define STATEFIELD   33

#define DAYSPERYEAR 365

/****************************************************************/
/* there are 575 "DB" points in the exported customer database  */
/* this oversized buffer accommodates almost twice that number. */
/****************************************************************/
#define MAXDBPOINTS 1000

#define MAXFILESIZE 85000000
#define MAXFILETIME 60*60*24*(365-1)      /* one year of seconds, less 1 day */
/* #define MAXFILESIZE 850000 */
/* #define MAXFILESIZE 200000*/

#define ZEROYEAR   94  /* file naming convention: year zero */

#define NOMSGS      0
#define SYNTAXMSGS  1
#define FILEMSGS    2
#define RECORDMSGS  3
#define RECORDMSGSA 4
#define RECORDMSGSB 5
#define BUCKETMSGSA 6
#define INPUTDATAA  7
#define BUCKETMSGSB 8
#define INPUTDATAB  9

/* External data */
extern  int    myPid;
        

/************************************************************************
*			PUBLIC DECLARATIONS				*
************************************************************************/

static short normalyear[13] = {0,  0, 31, 59, 90,120,151,181,212,243,273,304,334};
static short leapyear[13]   = {0,  0, 31, 60, 91,121,152,182,213,244,274,305,335};


/************************************************************/
/* this is the structure of the current parsed input record */
/* which is being processed into the current output record  */
/* for eventual output to the destination database.         */
/************************************************************/
typedef struct
{
    unsigned char    pointname[20];
    unsigned char    state[8];
    unsigned char    egu[10];
    unsigned char    type[8];
    unsigned char    rawdata[20];
    double           value;
    unsigned short   count;
    time_t           gmt;
    unsigned short   year;
    unsigned short   month;
    unsigned short   day;
    unsigned short   yday;
    unsigned short   hour;
    unsigned short   min;
    unsigned short   sec;
}       IMPORTRECORD;

static IMPORTRECORD  importrecord;
static IMPORTRECORD *imptr;

/***********************************************************/
/* this is the structure of the global working parameter   */
/*  list, used for processing all records.                 */
/***********************************************************/

typedef struct
{
    unsigned char    displow[LIMITS_SIZE+1];
    unsigned char    disphigh[LIMITS_SIZE+1];
    unsigned char    method[METHOD_SIZE+1];
    unsigned char    rate[RATE_SIZE+1];

    unsigned long    bucketlen;      /*   bucket interval (sec)  */
    unsigned long    recordlen;      /*   record interval (sec)  */
    unsigned long    maxnumbuckets;  /*   maximum num of buckets */
                                     /*   norm 60, but low as 24 */

    unsigned long    maxnumrecords;  /*   max records per day.   */

    unsigned short   firstrecyear;   /*   used to build filename */
    unsigned short   firstrecday;    /*   used to build filename */
    unsigned short   lastrecday;     /*   used to build filename */

    unsigned short   flushflag;      /*   if true, flush & exit. */
    time_t           firstrecordgmt; /*   seconds since 1/1/70   */
    time_t           currentgmt;     /* * seconds since 1/1/70   */
    unsigned long    daynumber;      /*   days since 1/1/70      */
                                   
    unsigned long    secondnumber;   /*   seconds since midnight */
    unsigned long    bucketnumber;   /*   buckets since midnight */
    unsigned long    bucketoffset;   /*   bucket record offset   */
    unsigned long    recordnumber;   /*   records since midnight */

    time_t           daygmt;         /*   today's gmt            */
    time_t           recordgmt;      /* * this record's gmt      */
    time_t           bucketgmt;      /* * this bucket's gmt      */
    time_t           nextbucketgmt;  /* * next bucket's gmt      */

    unsigned short   numpoints;      /* number of defined database points */
}       GLOBALCONTROLS;

static GLOBALCONTROLS  globalcontrols;
static GLOBALCONTROLS *glptr;

/***********************************************************/
/* this is the structure of the current working record     */
/*  being prepared for output to the destination database. */
/***********************************************************/

typedef struct
{
    unsigned char    pointname[20];
    unsigned char    egu[EGUTAG_SIZE+1];
    unsigned char    displow[LIMITS_SIZE+1];
    unsigned char    disphigh[LIMITS_SIZE+1];
    unsigned char    method[METHOD_SIZE+1];
    unsigned char    rate[RATE_SIZE+1];
    time_t           recordgmt;
    time_t           firstgmt;          /* used to create database file name */
    time_t           lastgmt;           /* used to create database file name */
    unsigned short   year;
    unsigned short   month;
    unsigned short   day;
    unsigned short   yday;               /* to be calculated */
    unsigned short   hour;
    unsigned short   min;
    unsigned short   count;
    unsigned short   fieldnum;
    double           previousvalue;
    double           minvalue;
    double           maxvalue;
    double           value[NUMFIELDS];
    unsigned short   state[NUMFIELDS];
    char             status[NUMFIELDS][STATUS_SIZE+1];
}       EXPORTRECORD;

static EXPORTRECORD  exportrecord[MAXDBPOINTS];
static EXPORTRECORD *exptr;


static          long  timezoneoffset;


static int             argvindex;
       unsigned char   record[81];
       unsigned char   outbuffer[81];
       unsigned char   oldbuffer[81];
       unsigned char   savebuffer[81];

       unsigned char   debuglevel;


/* codeBase related data */
D4DATA          *reduc;
T4TAG           *reduc_tag;

static FILE     *sourcehandle;

/* ar_snap.dbf */
CODE4	cb;
DATA4	*arSnapDesc;
FIELD4	*pointNameFld, *eguTagFld, *dispLowFld, *dispHighFld, *yearFld, *dayOfYearFld, *hourFld;
FIELD4	*offsetMinFld, *sampleCntFld, *redMethodFld, *rateFld;
FIELD4  *valueFld [BUCKETS_PER_RECORD], *statusFld [BUCKETS_PER_RECORD];

static  int    rateFieldSize;

/* ar_snap field and tag definitions */

FIELD4INFO	snapFields[] =	{

	/* Field name,		Type,	Width,		Decimals */
	{ "POINTNAME",		'C',	POINTNAME_SIZE,	        0},
	{ "EGUTAG",		'C',	EGUTAG_SIZE,	        0},
	{ "DISP_LOW",		'C',	LIMITS_SIZE, 	        0},
	{ "DISP_HIGH",		'C',	LIMITS_SIZE,	        0},
	{ "YEAR",		'N',	YEAR_SIZE,	        0},
	{ "DAYOFYEAR",		'N',	DAY_SIZE,	        0},
	{ "HOUR",		'N',	HOUR_SIZE,  	        0},
	{ "OFFSET_MIN",		'N',	MIN_SIZE, 	        0},
	{ "SAMPLE_CNT",		'N',	SAMPLECNT_SIZE,         0},
	{ "RED_METHOD",		'C',	METHOD_SIZE,	        0},
	{ "RATE",		'C',	RATE_SIZE,              0},
	{ "VALUE1",		'C',	VALUE_SIZE,		0},
	{ "STATUS1",		'C',	STATUS_SIZE,		0},
	{ "VALUE2",		'C',	VALUE_SIZE,		0},
	{ "STATUS2",		'C',	STATUS_SIZE,		0},
	{ "VALUE3",		'C',	VALUE_SIZE,		0},
	{ "STATUS3",		'C',	STATUS_SIZE,		0},
	{ "VALUE4",		'C',	VALUE_SIZE,		0},
	{ "STATUS4",		'C',	STATUS_SIZE,		0},
	{ "VALUE5",		'C',	VALUE_SIZE,		0},
	{ "STATUS5",		'C',	STATUS_SIZE,		0},
	{ "VALUE6",		'C',	VALUE_SIZE,		0},
	{ "STATUS6",		'C',	STATUS_SIZE,		0},
	{ "VALUE7",		'C',	VALUE_SIZE,		0},
	{ "STATUS7",		'C',	STATUS_SIZE,		0},
	{ "VALUE8",		'C',	VALUE_SIZE,		0},
	{ "STATUS8",		'C',	STATUS_SIZE,		0},
	{ "VALUE9",		'C',	VALUE_SIZE,		0},
	{ "STATUS9",		'C',	STATUS_SIZE,		0},
	{ "VALUE10",		'C',	VALUE_SIZE,		0},
	{ "STATUS10",		'C',	STATUS_SIZE,		0},
	{ "VALUE11",		'C',	VALUE_SIZE,		0},
	{ "STATUS11",		'C',	STATUS_SIZE,		0},
	{ "VALUE12",		'C',	VALUE_SIZE,		0},
	{ "STATUS12",		'C',	STATUS_SIZE,		0},
	{ "VALUE13",		'C',	VALUE_SIZE,		0},
	{ "STATUS13",		'C',	STATUS_SIZE,		0},
	{ "VALUE14",		'C',	VALUE_SIZE,		0},
	{ "STATUS14",		'C',	STATUS_SIZE,		0},
	{ "VALUE15",		'C',	VALUE_SIZE,		0},
	{ "STATUS15",		'C',	STATUS_SIZE,		0},
	{ "VALUE16",		'C',	VALUE_SIZE,		0},
	{ "STATUS16",		'C',	STATUS_SIZE,		0},
	{ "VALUE17",		'C',	VALUE_SIZE,		0},
	{ "STATUS17",		'C',	STATUS_SIZE,		0},
	{ "VALUE18",		'C',	VALUE_SIZE,		0},
	{ "STATUS18",		'C',	STATUS_SIZE,		0},
	{ "VALUE19",		'C',	VALUE_SIZE,		0},
	{ "STATUS19",		'C',	STATUS_SIZE,		0},
	{ "VALUE20",		'C',	VALUE_SIZE,		0},
	{ "STATUS20",		'C',	STATUS_SIZE,		0},
	{ "VALUE21",		'C',	VALUE_SIZE,		0},
	{ "STATUS21",		'C',	STATUS_SIZE,		0},
	{ "VALUE22",		'C',	VALUE_SIZE,		0},
	{ "STATUS22",		'C',	STATUS_SIZE,		0},
	{ "VALUE23",		'C',	VALUE_SIZE,		0},
	{ "STATUS23",		'C',	STATUS_SIZE,		0},
	{ "VALUE24",		'C',	VALUE_SIZE,		0},
	{ "STATUS24",		'C',	STATUS_SIZE,		0},
	{ "VALUE25",		'C',	VALUE_SIZE,		0},
	{ "STATUS25",		'C',	STATUS_SIZE,		0},
	{ "VALUE26",		'C',	VALUE_SIZE,		0},
	{ "STATUS26",		'C',	STATUS_SIZE,		0},
	{ "VALUE27",		'C',	VALUE_SIZE,		0},
	{ "STATUS27",		'C',	STATUS_SIZE,		0},
	{ "VALUE28",		'C',	VALUE_SIZE,		0},
	{ "STATUS28",		'C',	STATUS_SIZE,		0},
	{ "VALUE29",		'C',	VALUE_SIZE,		0},
	{ "STATUS29",		'C',	STATUS_SIZE,		0},
	{ "VALUE30",		'C',	VALUE_SIZE,		0},
	{ "STATUS30",		'C',	STATUS_SIZE,		0},
	{ "VALUE31",		'C',	VALUE_SIZE,		0},
	{ "STATUS31",		'C',	STATUS_SIZE,		0},
	{ "VALUE32",		'C',	VALUE_SIZE,		0},
	{ "STATUS32",		'C',	STATUS_SIZE,		0},
	{ "VALUE33",		'C',	VALUE_SIZE,		0},
	{ "STATUS33",		'C',	STATUS_SIZE,		0},
	{ "VALUE34",		'C',	VALUE_SIZE,		0},
	{ "STATUS34",		'C',	STATUS_SIZE,		0},
	{ "VALUE35",		'C',	VALUE_SIZE,		0},
	{ "STATUS35",		'C',	STATUS_SIZE,		0},
	{ "VALUE36",		'C',	VALUE_SIZE,		0},
	{ "STATUS36",		'C',	STATUS_SIZE,		0},
	{ "VALUE37",		'C',	VALUE_SIZE,		0},
	{ "STATUS37",		'C',	STATUS_SIZE,		0},
	{ "VALUE38",		'C',	VALUE_SIZE,		0},
	{ "STATUS38",		'C',	STATUS_SIZE,		0},
	{ "VALUE39",		'C',	VALUE_SIZE,		0},
	{ "STATUS39",		'C',	STATUS_SIZE,		0},
	{ "VALUE40",		'C',	VALUE_SIZE,		0},
	{ "STATUS40",		'C',	STATUS_SIZE,		0},
	{ "VALUE41",		'C',	VALUE_SIZE,		0},
	{ "STATUS41",		'C',	STATUS_SIZE,		0},
	{ "VALUE42",		'C',	VALUE_SIZE,		0},
	{ "STATUS42",		'C',	STATUS_SIZE,		0},
	{ "VALUE43",		'C',	VALUE_SIZE,		0},
	{ "STATUS43",		'C',	STATUS_SIZE,		0},
	{ "VALUE44",		'C',	VALUE_SIZE,		0},
	{ "STATUS44",		'C',	STATUS_SIZE,		0},
	{ "VALUE45",		'C',	VALUE_SIZE,		0},
	{ "STATUS45",		'C',	STATUS_SIZE,		0},
	{ "VALUE46",		'C',	VALUE_SIZE,		0},
	{ "STATUS46",		'C',	STATUS_SIZE,		0},
	{ "VALUE47",		'C',	VALUE_SIZE,		0},
	{ "STATUS47",		'C',	STATUS_SIZE,		0},
	{ "VALUE48",		'C',	VALUE_SIZE,		0},
	{ "STATUS48",		'C',	STATUS_SIZE,		0},
	{ "VALUE49",		'C',	VALUE_SIZE,		0},
	{ "STATUS49",		'C',	STATUS_SIZE,		0},
	{ "VALUE50",		'C',	VALUE_SIZE,		0},
	{ "STATUS50",		'C',	STATUS_SIZE,		0},
	{ "VALUE51",		'C',	VALUE_SIZE,		0},
	{ "STATUS51",		'C',	STATUS_SIZE,		0},
	{ "VALUE52",		'C',	VALUE_SIZE,		0},
	{ "STATUS52",		'C',	STATUS_SIZE,		0},
	{ "VALUE53",		'C',	VALUE_SIZE,		0},
	{ "STATUS53",		'C',	STATUS_SIZE,		0},
	{ "VALUE54",		'C',	VALUE_SIZE,		0},
	{ "STATUS54",		'C',	STATUS_SIZE,		0},
	{ "VALUE55",		'C',	VALUE_SIZE,		0},
	{ "STATUS55",		'C',	STATUS_SIZE,		0},
	{ "VALUE56",		'C',	VALUE_SIZE,		0},
	{ "STATUS56",		'C',	STATUS_SIZE,		0},
	{ "VALUE57",		'C',	VALUE_SIZE,		0},
	{ "STATUS57",		'C',	STATUS_SIZE,		0},
	{ "VALUE58",		'C',	VALUE_SIZE,		0},
	{ "STATUS58",		'C',	STATUS_SIZE,		0},
	{ "VALUE59",		'C',	VALUE_SIZE,		0},
	{ "STATUS59",		'C',	STATUS_SIZE,		0},
	{ "VALUE60",		'C',	VALUE_SIZE,      	0},
	{ "STATUS60",		'C',	STATUS_SIZE,		0},
	{0, 0, 0, 0}		/* NULL-record terminator */
};


TAG4INFO	snapTags [] =	{
	{ "METH_RATE",	"STR(YEAR,4,0)+STR(DAYOFYEAR,3,0)+STR(HOUR,2,0)+STR(OFFSET_MIN,2,0)+POINTNAME+RED_METHOD+RATE+STR(RECNO(),8,0)", 0, 0, 0},
	{ "METHOD_REC",	"POINTNAME+RED_METHOD+STR(YEAR,4,0)+STR(DAYOFYEAR,3,0)+STR(HOUR,2,0)+STR(OFFSET_MIN,2,0)+STR(RECNO(),8,0)", 0, 0, 0},
	{0, 0, 0, 0, 0}		/* NULL-record terminator */
};


/* ar_reduc.dbf */
CODE4	cbx;
DATA4	*arReducDesc;
FIELD4	*pointNameFldx, *rateFldx, *methodFldx, *archiveFldx;


/* copy of field definition header file ar_reduc.h */
FIELD4INFO	ar_reducFieldInfo[] = 
	{
	{"POINTNAME",	'C',	28,	0},
	{"RATE",	'C',	5,	0},	/* 1-May-96 large enough for 3600 seconds */
	{"METHOD",	'C',	3,	0},
	{"ARCHIVE",	'L',	1,	0},	/* 3-May-96 Archive bit */
	{0,0,0,0}
	};

T4TAG_INFO	ar_reducTagInfo[] = 
	{
	{"POINTNAME",	"POINTNAME",	0,	0,	0},
	{"POINTMETH",	"POINTNAME+METHOD",	0,	0,	0},
	{"METHOD",	"METHOD",	0,	0,	0},
	{0,0,0,0,0}
	};




void dumpimport(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: dump import record :::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf(":::: %s", record);  /* debug printing */
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: Name:[%s] Type = %s, EGU = %s, \tRawData = %s\n",
             imptr->pointname,
             imptr->type,
             imptr->egu,
             imptr->rawdata);
    printf ("::::                                               \tValue = %f\n", imptr->value);
    printf (":::: RecTime = %s", ctime(&imptr->gmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: RecGMT  =%d, Year=%d, Month=%d, Day=%d, DOY=%d, Hour=%d, Min=%d, Sec=%d\n",
             imptr->gmt,
             imptr->year,
             imptr->month,
             imptr->day,
             imptr->yday,
             imptr->hour,
             imptr->min,
             imptr->sec);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


void dumptime(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: dump global time stuff :::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: DayNum= %d, SecNum = %d, BuckNum = %d BuckOff = %d, RecNum = %d\n",
             glptr->daynumber,
             glptr->secondnumber,
             glptr->bucketnumber,
             glptr->bucketoffset,
             glptr->recordnumber);
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: DayGMT\t\t= %d,   %s",
             glptr->daygmt,
             ctime(&glptr->daygmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: RecGMT\t\t= %d,   %s",
             glptr->recordgmt,
             ctime(&glptr->recordgmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: BuckGMT\t\t= %d,   %s",
             glptr->bucketgmt,
             ctime(&glptr->bucketgmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: currentgmt\t\t= %d,   %s",
             glptr->currentgmt,
             ctime(&glptr->currentgmt));
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: NextBuckGMT\t= %d,   %s",
             glptr->nextbucketgmt,
             ctime(&glptr->nextbucketgmt));
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


void dumpglobal(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: global controls ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: displow = %s, disphigh = %s, method = %s, rate = %s\n",
             glptr->displow,
             glptr->disphigh,
             glptr->method,
             glptr->rate);
    printf (":::: bucketlen = %d, recordlen = %d, maxnumbuckets = %d\n",
             glptr->bucketlen,
             glptr->recordlen,
             glptr->maxnumbuckets);
    printf (":::: DayNum= %d, SecNum = %d, BuckNum = %d BuckOff = %d, RecNum = %d\n",
             glptr->daynumber,
             glptr->secondnumber,
             glptr->bucketnumber,
             glptr->bucketoffset,
             glptr->recordnumber);
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: firstrecordgmt\t= %d,   %s",
             glptr->firstrecordgmt,
             ctime(&glptr->firstrecordgmt));
    printf (":::: DayGMT\t\t= %d,   %s",
             glptr->daygmt,
             ctime(&glptr->daygmt));
    printf (":::: RecGMT\t\t= %d,   %s",
             glptr->recordgmt,
             ctime(&glptr->recordgmt));
    printf (":::: BuckGMT\t\t= %d,   %s",
             glptr->bucketgmt,
             ctime(&glptr->bucketgmt));
    printf (":::: currentgmt\t\t= %d,   %s",
             glptr->currentgmt,
             ctime(&glptr->currentgmt));
    printf (":::: NextBuckGMT\t= %d,   %s",
             glptr->nextbucketgmt,
             ctime(&glptr->nextbucketgmt));
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


void dumpmini(void)
{
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: Name:[%s] Type = %s, EGU = %s, RawData = %s,  Value = %f\n",
             imptr->pointname,
             imptr->type,
             imptr->egu,
             imptr->rawdata,
             imptr->value);
    printf (":::: ----------------------------------------------------------------------------\n");
    printf (":::: DayNum = %d, SecNum = %d, BuckNum = %d BuckOff = %d, RecNum = %d\n",
             glptr->daynumber,
             glptr->secondnumber,
             glptr->bucketnumber,
             glptr->bucketoffset,
             glptr->recordnumber);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


/* put debug stuff here */
void dumpexport(int i)
{
    unsigned short j;

    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: export record: point count [%3d] :::::::::::::::::::::::::::::::::::::::::::\n",i);
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
    printf (":::: Name:[%s] EGU = %s, DispLow = %s, DispHigh = %s, Method = %s, Rate = %s\n",
             exptr->pointname,
             exptr->egu,
             exptr->displow,
             exptr->disphigh,
             exptr->method,
             exptr->rate);
    printf ("::::                 Year = %d, DOY = %d,  Hour = %d, OffMin = %d, SampleCnt = %d\n",
             exptr->year,
             exptr->yday,
             exptr->hour,
             exptr->min,
             exptr->fieldnum);
    printf (":::: ----------------------------------------------------------------------------\n");
    for (j = 0; j < NUMFIELDS; j += 4)
    {
        printf ("::::  [%d%d] %15f, '%s'(%d)  %15f, '%s'(%d)   %15f, '%s'(%d)   %15f, '%s'(%d)\n",
                j/10,
                j%10,
                exptr->value[j],   exptr->status[j],   exptr->state[j],
                exptr->value[j+1], exptr->status[j+1], exptr->state[j+1],
                exptr->value[j+2], exptr->status[j+2], exptr->state[j+2],
                exptr->value[j+3], exptr->status[j+3], exptr->state[j+3]);
    }
    printf (":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n");
}


/*****************************/
/* Database initialization.  */
/*****************************/

int dbinit (void)
{
    d4init(&cb);
#if defined(USECB63) || defined(WIN32)
    cb.errOpen = 0;
    cb.errTagName = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
    cb.errGo = 0;
    cb.errSkip = 0;
#else
    cb.open_error = 0;
    cb.tag_name_error = 0;
    cb.create_error = 0;
    cb.field_name_error = 0;
    cb.go_error = 0;
    cb.skip_error = 0;
#endif	USECB63

    return (TRUE);

}

/*****************************************************/
/* Open the output database.  If it does not exist,  */
/* then create it and return false indicating that   */
/* it did not previously exist.                      */
/*****************************************************/

int openzfile (void)
{
    int	i;
    char	buf [20], pathName [200];
    TAG4	*t;

    resolveLinks ("workfile", pathName);
    arSnapDesc = d4open (&cb, pathName);
    if (arSnapDesc == NULL)
    {
        arSnapDesc = d4create (&cb, pathName, snapFields, snapTags);
        if (arSnapDesc == NULL)
        {
            return (FALSE);
        }
    }

    t = d4tag (arSnapDesc, "METH_RATE");
    d4tag_select (arSnapDesc, t);
    d4top (arSnapDesc);
    d4unlock_all (arSnapDesc);

    /* Open up all the fields */

    pointNameFld  = d4field (arSnapDesc, "POINTNAME");
    eguTagFld     = d4field (arSnapDesc, "EGUTAG");
    dispLowFld    = d4field (arSnapDesc, "DISP_LOW");
    dispHighFld   = d4field (arSnapDesc, "DISP_HIGH");
    yearFld       = d4field (arSnapDesc, "YEAR");
    dayOfYearFld  = d4field (arSnapDesc, "DAYOFYEAR");
    hourFld       = d4field (arSnapDesc, "HOUR");
    offsetMinFld  = d4field (arSnapDesc, "OFFSET_MIN");
    sampleCntFld  = d4field (arSnapDesc, "SAMPLE_CNT");
    redMethodFld  = d4field (arSnapDesc, "RED_METHOD");
    rateFld       = d4field (arSnapDesc, "RATE");
    rateFieldSize = f4len (rateFld);

    for (i=0 ; i<BUCKETS_PER_RECORD ; i++) {
        sprintf (buf, "VALUE%d", i + 1); 
        valueFld [i] = d4field (arSnapDesc, buf);
        sprintf (buf, "STATUS%d", i + 1); 
        statusFld [i] = d4field (arSnapDesc, buf);
    }
    return (TRUE);
}

/*****************************/
/* Database initialization.  */
/*****************************/

int dbxinit (void)
{
    d4init(&cbx);
#if defined(USECB63) || defined(WIN32)
    cb.errOpen = 0;
    cb.errTagName = 0;
    cb.errCreate = 0;
    cb.errFieldName = 0;
    cb.errGo = 0;
    cb.errSkip = 0;
#else
    cbx.open_error = 0;
    cbx.tag_name_error = 0;
    cbx.create_error = 0;
    cbx.field_name_error = 0;
    cbx.go_error = 0;
    cbx.skip_error = 0;
#endif	USECB63

    return (TRUE);

}

/****************************************************/
/* Open the ar_reduc file. If it does not exist,    */
/* then create it and return false indicating that  */
/* it did not previously exist.                     */
/****************************************************/

int open_ar_reduc (void)
{
    char	pathName [200];
    TAG4	*t;

    resolveLinks ("ar_reduc", pathName);
    arReducDesc = d4open (&cbx, pathName);
    if (arReducDesc == NULL)
    {
        arReducDesc = d4create (&cbx, pathName, ar_reducFieldInfo, ar_reducTagInfo);
        if (arReducDesc == NULL)
        {
            return (FALSE);
        }
    }

    t = d4tag (arReducDesc, "POINTNAME");
    d4tag_select (arReducDesc, t);
    d4top (arReducDesc);
    d4unlock_all (arReducDesc);

    /* Open up all the fields */

    pointNameFldx  = d4field (arReducDesc, "POINTNAME");
    rateFldx       = d4field (arReducDesc, "RATE");
    methodFldx     = d4field (arReducDesc, "METHOD");
    archiveFldx    = d4field (arReducDesc, "ARCHIVE");

    return (TRUE);
}

/*****************************************************/
/* Close the output database. After closing the      */
/* active file, rename it using the start and stop   */
/* dates to create the new name. Delete any file     */
/* with the same name.                               */
/*****************************************************/

int closezfile (void)
{
    char	pathName1 [200], pathName2 [200];
    char	buffera [10], bufferb [10], bufferc [20], bufferd [20];

    /**********************/
    /* close the database */
    /**********************/
    if (debuglevel >= FILEMSGS)
    {
        printf("Closing workfile database.\n");
    }
    d4close(arSnapDesc);

    /* rename the old database based on the start and stop dates. */
    /* get them from the first and last timestamp dates of the    */
    /* database, which are in the global data structure.          */

    sprintf(buffera, "%1d%1d%1d",
             glptr->firstrecday/100,
            (glptr->firstrecday/10)%10,
             glptr->firstrecday%10);
    sprintf(bufferb, "%1d%1d%d",
             glptr->lastrecday/100,
            (glptr->lastrecday/10)%10,
             glptr->lastrecday%10);
    sprintf(bufferc, "%1d%d", glptr->firstrecyear/10, glptr->firstrecyear%10);
    strcat (bufferc, buffera);
    strcat (bufferc, bufferb);

    /* build the .dbx file pathnames and rename the file */
    strcpy (bufferd, bufferc);
    strcat (bufferd, ".dbf");
    resolveLinks (bufferd, pathName2);
    resolveLinks ("workfile.dbf", pathName1);
    if (debuglevel >= FILEMSGS)
    {
        printf("Renaming workfile.dbf to %s\n", bufferd);
    }
    if (rename (pathName1, pathName2))
    {
        return (FALSE);
    }

    /* build the .mdx file pathnames and rename the file */
    strcpy (bufferd, bufferc);
    strcat (bufferd, ".mdx"); 
    resolveLinks (bufferd, pathName2);
    resolveLinks ("workfile.mdx", pathName1);
    if (debuglevel >= FILEMSGS)
    {
        printf("Renaming workfile.mdx to %s\n", bufferd);
    }
    if (rename (pathName1, pathName2))
    {
        return (FALSE);
    }
    return (TRUE);
}


/**************************************************/
/* determine the size of the output database file */
/* (*.dbf) and decide if it is time to close it.  */
/**************************************************/

long sizezfile (void)
{
    char          pathName [200];
    struct stat   sbuffer;
    struct stat  *sptr;
    long          size;

    sptr = &sbuffer;

    resolveLinks ("workfile.dbf", pathName);
    stat(pathName, sptr);
    size = (long) sbuffer.st_size;
    return (size);
}

/*******************************************************/
/* determine the time span of the output database file */
/* (*.dbf) and decide if it is time to close it.       */
/*******************************************************/

long timezfile (void)
{
    long    timespan;

    timespan = glptr->recordgmt - glptr->firstrecordgmt;
    return (timespan);
}


/*****************************************************/
/* get another raw record from the file and fix it.  */
/* if this file is done, read another file. if no    */
/* more files, then set flag and return nothing.     */
/*****************************************************/
nextrecord(int argc, char *argv[])
{
    unsigned char *baptr;

    /**********************************/
    /**********************************/
static struct tm     *tmTime;
/* static struct tm     *myTime; */
static unsigned long  absolutetime;
static unsigned long  previoustime;
static          long  deltatime;

static time_t         tt = 0;
static unsigned long  dbrecnum;

    /**********************************/
    /**********************************/

    unsigned short year, month, day, hour, min, sec;
    unsigned long  totaldays;
    unsigned long  totalsecs;
             short i, datafield;
/*    unsigned char  tm_zone[20]; */

    short offsetyear, leapdays;

    /**********************************/
    /**********************************/

    /* Get access to a tm structure */

    tmTime = gmtime (&tt);

    imptr = &importrecord;
    baptr = &record[0];

    while (HELLNOTFROZEN)
    {
        /* if first time through, or previous file has been closed ... */
        if (!sourcehandle)
        {
            /* see if we are out of file names to process */
            if (!(argv[argvindex]))
            {
                if (debuglevel >= FILEMSGS)
                {
                    printf(":::: ++++++++++++++++++++++++++\n"); /* debug printing */
                    printf("********* No more input files to process: flush all output buffers *********\n");
                }
                glptr->flushflag = TRUE;
                return (TRUE);
            }

            if (debuglevel >= FILEMSGS)
            {
                printf("******************* OPENING THE %s FILE *******************\n",
                        argv[argvindex]);
            }

            /* Open File for Read Only */

            sourcehandle = fopen(argv[argvindex],"r");
            if (sourcehandle == NULL)
            {
                /* if file open is unsuccessful, there is no new record to return. */
                if (debuglevel >= FILEMSGS)
                {
                    printf("ERROR: Cannot open source database file (%s)\n", argv[argvindex]);
                }
                return(FALSE);
            }
            dbrecnum = 0;
            argvindex++;
        }

        /* file is currently open. get the next record. */
        if (!fgets(record, 80, sourcehandle))
        {
            /************************************************/
            /* if you cannot get another record, then there */
            /* is nothing left in the file. close the file  */
            /************************************************/
            if (debuglevel >= FILEMSGS)
            {
                printf("******************* CLOSING THE %s FILE *******************\n",
                        argv[argvindex-1]);
            }
            fclose (sourcehandle);

            /****************************/
            /* set the file closed flag */
            /****************************/
            sourcehandle = NULL;
            break;
        }

        /* if it is the right kind of record, its a good one. fix it up and keep it */
        if ((record[0]=='D') &&
            (record[1]=='B') &&
            /**************************************************************/
            /* TEMPORARILY filter to accept an arbitrary subset of points */
            /**************************************************************/
/*
            (record[NAMEFIELD+0]=='R') &&
            (record[NAMEFIELD+1]=='F') &&
            (record[NAMEFIELD+2]=='N') &&
            (record[NAMEFIELD+3]=='C') &&
            (record[NAMEFIELD+4]=='L') &&
            (record[NAMEFIELD+5]=='2') &&
            (record[NAMEFIELD+6]=='T') &&
            (record[NAMEFIELD+7]=='U') &&
*/
            (record[2]==' '))
        {

            /******************/
            /* parse the data */
            /******************/

            /* copy the input buffer to the output buffer */
            strcpy(&outbuffer[0], &record[0]);
            strcpy(&savebuffer[0], &outbuffer[0]);

            if (debuglevel >= INPUTDATAA)
            {
                printf(":::: %s", record);  /* debug printing */
            }

            /* insert space in place of newline character */
            if ((baptr = strchr(&outbuffer[NAMEFIELD], 0x0d  ))) *baptr = ' ';
            if ((baptr = strchr(&outbuffer[NAMEFIELD], 0x0a  ))) *baptr = ' ';
            /* insert nulls at the end of all string fields */
            if ((baptr = strchr(&outbuffer[NAMEFIELD], ' '  ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[TYPEFIELD], ' '  ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[EGUFIELD], ' '   ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[MONTHFIELD], ' ' ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[DAYFIELD], ' '   ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[HOURFIELD], ' '  ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[MINUTEFIELD], ' '))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[SECONDFIELD], ' '))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[DATAXFIELD], ' ' ))) *baptr = 0;
            if ((baptr = strchr(&outbuffer[STATEFIELD], ' ' ))) *baptr = 0;

            /* find first character in floating point value string */
            for (i = DATAXFIELD; i > DATAFIELD; i--)
            {
                if (outbuffer[i] == ' ') break;
            }
            datafield = i+1;

            strcpy (imptr->pointname, &outbuffer[NAMEFIELD]);

            /********************************************************/
            /* Grant's last request: drop records with missing name */
            /********************************************************/
            if (strlen(imptr->pointname) == 0) continue;


            dbrecnum++; /* moved from top of current 'if' statement. */

            strcpy (imptr->egu, &outbuffer[EGUFIELD]);

            strcpy (imptr->type, &outbuffer[TYPEFIELD]);
            strcpy (imptr->state, &outbuffer[STATEFIELD]);

            imptr->year =  (unsigned char) atoi(&outbuffer[YEARFIELD]);
            imptr->month = (unsigned char) atoi(&outbuffer[MONTHFIELD]);
            imptr->day =   (unsigned char) atoi(&outbuffer[DAYFIELD]);
            imptr->hour =  (unsigned char) atoi(&outbuffer[HOURFIELD]);
            imptr->min =   (unsigned char) atoi(&outbuffer[MINUTEFIELD]);
            imptr->sec =   (unsigned char) atoi(&outbuffer[SECONDFIELD]);

            imptr->value = (double) atof(&outbuffer[datafield]);

            strcpy (imptr->rawdata, &outbuffer[datafield]);


            year  = (unsigned char) atoi(&outbuffer[YEARFIELD]);
            month = (unsigned char) atoi(&outbuffer[MONTHFIELD]);
            day   = (unsigned char) atoi(&outbuffer[DAYFIELD]);
            hour  = (unsigned char) atoi(&outbuffer[HOURFIELD]);
            min   = (unsigned char) atoi(&outbuffer[MINUTEFIELD]);
            sec   = (unsigned char) atoi(&outbuffer[SECONDFIELD]);


            /* handle century wrap. time does not exist before Jan 1, 1970. */
            if ((offsetyear = (year - 70)) < 0) offsetyear += 100;

            /* make adjustment for leap days in preceding years */
            leapdays = ((offsetyear + 1)/4);

            if (year%4 == 0)
            {
                totaldays = (offsetyear * DAYSPERYEAR) +
                            leapdays +
                            leapyear[month] +
                            (day - 1);
            }
            else
            {
                totaldays = (offsetyear * DAYSPERYEAR) +
                            leapdays +
                            normalyear[month] +
                            (day - 1);
            }
            totalsecs = (((((totaldays * 24) + hour) * 60 ) + min) * 60) + sec;

            /**********************************************************************/
            /**********************************************************************/
            /**********************************************************************/

            /* Convert the struct tm to seconds since the dawn of time */
            tmTime->tm_year  = imptr->year;
            tmTime->tm_mon   = imptr->month - 1;
            tmTime->tm_mday  = imptr->day;
            tmTime->tm_hour  = imptr->hour;
            tmTime->tm_min   = imptr->min;
            tmTime->tm_sec   = imptr->sec;
/*
            strcpy (&tm_zone[0],"GMT-8");
            tmTime->tm_zone = tm_zone;
*/

/* get gmt using system calls. */

#if (defined (SOLARIS))
            absolutetime = mktime (tmTime);
#else
            absolutetime = timegm (tmTime);
#endif
            /* add in the time zone adjustment factor from the command line. */
            absolutetime += timezoneoffset;

            /* store the components of the GMT in the input record */
            imptr->year  = tmTime->tm_year;
            imptr->yday  = tmTime->tm_yday;
            imptr->hour  = tmTime->tm_hour;
            imptr->min   = tmTime->tm_min;
            imptr->sec   = tmTime->tm_sec;

/* raw time from local calculations */
/*            absolutetime = totalsecs;

/* get gmt using system calls. */
/*
            myTime = localtime (&absolutetime);
            absolutetime = timelocal (myTime);
            myTime = localtime (&absolutetime);
            printf ("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW\n");
            printf ("WWWW: our calc %s\n", ctime(&absolutetime));
            absolutetime = timelocal (myTime);
            printf ("WWWW: sys gmt  %s\n", ctime(&absolutetime));
            printf ("WWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWWW\n");
*/

            /* store the absolute GMT time */
            imptr->gmt = absolutetime;
            glptr->currentgmt = imptr->gmt;           /* seconds since 1/1/70   */

            deltatime = absolutetime - previoustime;
/*
            printf("DBrecnum = %d, Lasttime = %d, Thistime = %d, Delta = %d\n",
                    dbrecnum, previoustime, absolutetime, deltatime);
*/

/*            printf("GMT = %d, Delta = %d\n", absolutetime, deltatime); */
            if (deltatime < 0)
            {
                if (debuglevel >= SYNTAXMSGS)
                {
                    printf("*************************************************************\n");
                    printf("******************** TIME SEQUENCE ERROR ********************\n");
                    printf("*************************************************************\n");
                    printf("%s\n", oldbuffer);
                    printf("%s\n", savebuffer);
                    printf("-------------------------------------------------------------\n");
                    printf("DBrecnum = %d, Lasttime = %d, Thistime = %d, Delta = %d\n",
                            dbrecnum, previoustime, absolutetime, deltatime);
                    printf("*************************************************************\n");
                    printf("*************************************************************\n");
                    printf("*************************************************************\n");
                    printf("\n");
                }
            }
            strcpy(&oldbuffer[0], &savebuffer[0]);
            previoustime = absolutetime;

            return(TRUE);

        }
        /* else circle around and get another record */
    }
}


/********************************************************************
                                                                    *
   create a record and write it into the destination db file.       *
                                                                    *
*********************************************************************/

void AddToDestination(D4DATA *arSnapDesc)
{
    int      i;
    char     xbuffer[80];

    d4top(arSnapDesc);

    if (d4deleted(arSnapDesc))
    {
        if (debuglevel >= RECORDMSGSB)
        {
            printf ("**** Codebase Write Old Record: %s ****\n", exptr->pointname);
        }
        f4assign     (pointNameFld, exptr->pointname);
        f4assign     (eguTagFld, exptr->egu );
        f4assign     (dispLowFld, exptr->displow);
        f4assign     (dispHighFld, exptr->disphigh);
        f4assign_int (yearFld, exptr->year+1900);
        f4assign_int (dayOfYearFld, exptr->yday);
        f4assign_int (hourFld, exptr->hour);
        f4assign_int (offsetMinFld, exptr->min);
        f4assign_int (sampleCntFld, exptr->fieldnum);
        f4assign     (redMethodFld, exptr->method);
        f4assign     (rateFld, exptr->rate);
        for (i=0 ; i<60 ; i++)
        {
            sprintf (xbuffer, "%f", exptr->value [i]);
            f4assign (valueFld [i], &xbuffer[0]);
            sprintf (xbuffer, "%s", exptr->status [i]);
            f4assign (statusFld [i], &xbuffer[0]);
        }
        d4recall(arSnapDesc);
    }
    else
    {
        if (debuglevel >= RECORDMSGSB)
        {
            printf ("**** Codebase Write New Record: %s ****\n", exptr->pointname);
        }
        d4append_start( arSnapDesc, 0 );   /* '0' => no memo fields */
        d4blank( arSnapDesc );

        f4assign     (pointNameFld, exptr->pointname);
        f4assign     (eguTagFld, exptr->egu );
        f4assign     (dispLowFld, exptr->displow);
        f4assign     (dispHighFld, exptr->disphigh);
        f4assign_int (yearFld, exptr->year+1900);
        f4assign_int (dayOfYearFld, exptr->yday);
        f4assign_int (hourFld, exptr->hour);
        f4assign_int (offsetMinFld, exptr->min);
        f4assign_int (sampleCntFld, exptr->fieldnum);
        f4assign     (redMethodFld, exptr->method);
        f4assign     (rateFld, exptr->rate);

        for (i=0 ; i<60 ; i++)
        {
            sprintf (xbuffer, "%f", exptr->value [i]);
            f4assign (valueFld [i], &xbuffer[0]);
            strcpy (xbuffer, &exptr->status [i][0]);
            f4assign (statusFld [i], &xbuffer[0]);
        }
        d4append( arSnapDesc );
        d4flush_all( arSnapDesc );
    }
    d4unlock_all(arSnapDesc);
    return;
}



/*****************************************************************
                                                                 *
   create a record and write it into the ar_reduc db file.       *
                                                                 *
*****************************************************************/

void AddToReduc(D4DATA *arReducDesc)
{
    d4top(arReducDesc);

    if (d4deleted(arReducDesc))
    {
        if (debuglevel >= RECORDMSGSB)
        {
            printf ("**** Codebase Write Old ar_reduc: %s ****\n", exptr->pointname);
        }
        f4assign     (pointNameFldx, exptr->pointname);
        f4assign     (rateFldx, exptr->rate);
        f4assign     (methodFldx, exptr->method);
        f4assign     (archiveFldx, "1");
        d4recall(arReducDesc);
    }
    else
    {
        if (debuglevel >= RECORDMSGSB)
        {
            printf ("**** Codebase Write New ar_reduc: %s ****\n", exptr->pointname);
        }
        d4append_start( arReducDesc, 0 );   /* '0' => no memo fields */
        d4blank( arReducDesc );

        f4assign     (pointNameFldx, exptr->pointname);
        f4assign     (rateFldx, exptr->rate);
        f4assign     (methodFldx, exptr->method);
        f4assign     (archiveFldx, "1");
        d4append( arReducDesc );
        d4flush_all( arReducDesc );
    }
    d4unlock_all(arReducDesc);
    return;
}


void usage(void)
{
    printf("Usage:\n");
    printf(" snwsDbmake [-dn] [-th] (Interval) {Source...}       where:\n");
    printf("\t[-dn]\t= optional debug trace level (n range: 0 - 7, n default: 1)");
    printf("\t[-th]\t= optional time zone adjustment. this value is added");
    printf("\t     \t  to the calculated GMT. (h range: -23 + 23 hours)");
    printf("\t(Interval)\t= trending interval in minutes (valid values: 1 and 60 only)");
    printf("\t{Source}\t= complete path to file(s) to be imported.");
}

/***********************************************************************************/
/*                                                                                 */
/* snwsDbmake trendinterval infilename [...infilename]                             */
/*                                                                                 */
/*  This program imports one or more sequential foreign database files into an     */
/*  archived trend database. All of the 'DB' points in the foreign database will   */
/*  be trended, at the trend interval specified, for the entire time represented   */
/*  in the foreign database. The size of the output trend files will be limited    */
/*  to approximately 85 Mbytes. Multiple files will be created when this file      */
/*  size is reached. Files will be named according to a previously specified       */
/*  date-based naming convention.                                                  */
/*                                                                                 */
/***********************************************************************************/

void main(int argc, char *argv[])

{
    static struct tm  *tmTime;

    FILE           *tempsource;
    FILE           *tempdest;
    unsigned short  point_hit;
    unsigned short  interval;
    unsigned short  firstpass;
    unsigned long   recordindex;
             long   timezonehours;

    char            work[10];
    unsigned long   i, j;

    tisOpenLog ("snwsDbmake", 6, 2, NULL, NULL);

    /*****************************/
    /* initalize global pointers */
    /*****************************/
    imptr = &importrecord;
    exptr = &exportrecord[0];
    glptr = &globalcontrols;


    /***************************/
    /* no source file is open. */
    /***************************/
    sourcehandle = NULL;
    argvindex = 1;
    debuglevel = RECORDMSGS;

    /***************************************************************/
    /* determine that the correct number of parameters was passed. */
    /***************************************************************/
    if (argc < 3 )
    {
        tisSysLog(0,0,"Too few command line arguments:");
        usage();
        exit(1);
    }

    timezoneoffset = 0;

    /**********************************************************/
    /* check to see if debug level specified in command line. */
    /**********************************************************/
    strcpy (&work[0], argv[argvindex]);
    while (work[0] == '-')
    {
        if (work[1] == 'd')
        {
            debuglevel = atoi(&work[2]);
            tisSysLog(0,0,"Debug level set to %d.", debuglevel);
            argvindex++;
        }
        else if (work[1] == 't')
        {
            timezonehours = atoi(&work[2]);
            if ((timezonehours < -23) || (timezonehours > 23))
            {
                tisSysLog(0,0,"Invalid timezone offset value(%d):", timezonehours);
                usage();
                exit(1);
            }

            timezoneoffset = (atoi(&work[2])*60*60); /* expressed in seconds */
            tisSysLog(0,0,"Timezone offset = %d(%d).", timezoneoffset, atoi(&work[2]));
            argvindex++;
        }
        else
        {
            tisSysLog(0,0,"Invalid command line option parameter %c:",work[1]);
            usage();
            exit(1);
        }
        strcpy (&work[0], argv[argvindex]);
    }

    /*************************************************************************************/
    /* verify validity of trend interval. must be one of the recognized trend intervals. */
    /*************************************************************************************/
    interval = atoi(argv[argvindex++]);
    switch (interval)
    {
        case 1:
            glptr->maxnumbuckets = NUMFIELDS;
            glptr->maxnumrecords = 24;
            strcpy (&glptr->rate[0], "60");
            break;

        case 60:
            glptr->maxnumbuckets = 24;
            glptr->maxnumrecords = 1;
            strcpy (&glptr->rate[0], "3600");
            break;

        default:
            if (debuglevel >= SYNTAXMSGS)
            {
                printf("ERROR: Valid Trend intervals are 1 or 60 minutes only.\n");
            }
            exit(1);
            break;
    }
    glptr->bucketlen = interval*60;
    glptr->recordlen = glptr->bucketlen * glptr->maxnumbuckets;

    strcpy (&glptr->displow[0],   "0");
    strcpy (&glptr->disphigh[0],  "100000");
    strcpy (&glptr->method[0],    "AVG");

    /*******************************************************************************/
    /* open the source file, verifying it's existence: if it does not exist, exit. */
    /*******************************************************************************/
    if ((tempsource = fopen(argv[argvindex],"r")) == NULL)      /* Open File for Read Only */
    {
        if (debuglevel >= SYNTAXMSGS)
        {
            printf("ERROR : Cannot open source database file.\n");
        }
        usage();
        exit(1);
    }

    /******************************************************/
    /* this was just diagnostic. for now, close the file  */
    /******************************************************/
    fclose (tempsource);

    /************************************************************************************/
    /* open the destination file, verifying it's NON-existence: if it does exist, exit. */
    /************************************************************************************/
    if ((tempdest = fopen("workfile.dbf","r")) != NULL)      /* Open File for Read Only */
    {
        fclose (tempdest);
        if (debuglevel >= SYNTAXMSGS)
        {
            printf("ERROR: Destination database file already exists.\n");
        }
        usage();
        exit(1);
    }

    /************************************************************************************/
    /* open the destination file, verifying it's NON-existence: if it does exist, exit. */
    /************************************************************************************/
    if ((tempdest = fopen("workfile.mdx","r")) != NULL)      /* Open File for Read Only */
    {
        fclose (tempdest);
        if (debuglevel >= SYNTAXMSGS)
        {
            printf("ERROR: Destination database file already exists.\n");
        }
        usage();
        exit(1);
    }


    /*****************************************************************/
    /* create the destination file. do codebase initalization magic. */
    /*****************************************************************/
    dbinit();

    /* Open destination file, get and select tags, position to the top of the file */
    if (!openzfile())
    {
        exit(1);
    }

    /**************************************************************/
    /* create the ar_reduc file. do codebase initalization magic. */
    /**************************************************************/
    dbxinit();

    /* Open ar_reduc file, get and select tags, position to the top of the file */
    if (!open_ar_reduc())
    {
        d4close(arSnapDesc);
        exit(1);
    }

    /******************************/
    /******************************/

    glptr->flushflag = FALSE;
    firstpass = TRUE;
    recordindex = 0;
    /**************/
    /* lets rock! */
    /**************/

    if (debuglevel >= FILEMSGS)
    {
        printf ("*********************************************************\n");
        printf ("********* Begin Processing Input Records .... ***********\n");
        printf ("*********************************************************\n");
    }

    while (nextrecord (argc, argv))
    {
        recordindex++;
        /***********************************************************/
        /* process all records contained in the current "bucket".  */
        /* if record just obtained is not in the current bucket,   */
        /* complete current bucket processing and proceed to the   */
        /* next bucket.                                            */
        /***********************************************************/

        while ((!firstpass) && ((glptr->flushflag) || (imptr->gmt >= glptr->nextbucketgmt)))
        {
            if (debuglevel >= BUCKETMSGSA)
            {
                printf("---- Bucket Time Transition [%d] \n", glptr->numpoints);
            }

            /********************************************************/
            /* ok. the current bucket filling process is completed. */
            /* now it is time to process ALL of the point records   */
            /* for average calculations and closure.                */
            /********************************************************/

            /* calc averages for each point. see details  */
            /* of averaging algorithm for specifics.      */
            for (i = 0; i < glptr->numpoints; i++)
            {
                exptr = &exportrecord[i];

                /* point to jth bucket of each database point export record. */
                j = glptr->bucketoffset;

                if (debuglevel >= BUCKETMSGSB)
                {
                    printf(":::: +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
                }

                /* for now, calculate simple average */
                if(exptr->count)
                {
                    if (debuglevel >= BUCKETMSGSB)
                    {
                        printf ("preprocess:  [%d%d][%d%d], %f/%d=\t%f   v(j-1)=%f, prevv=%f, status=%d\n",
                                 i/10, i%10,
                                 j/10, j%10,
                                 exptr->value[j],
                                 exptr->count,
                                 exptr->value[j]/exptr->count,
                                 exptr->value[j-1],
                                 exptr->previousvalue,
                                 exptr->state[j]);
                    }

                    exptr->value[j] = exptr->value[j] / exptr->count;
                    exptr->previousvalue = exptr->value[j];  /* save it for fill, if needed    */
                    exptr->state[j] = 2;  /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill)  */
                    strcpy(&exptr->status[j][0], "   "); /* "   " = good data in bucket        */
                    exptr->count = 0;      /* reset counter */
                }
                else
                {
                    if (debuglevel >= BUCKETMSGSB)
                    {
                        printf ("preprocess:  [%d%d][%d%d],      \t\t%f   v(j-1)=%f, prevv=%f, status=%d\n",
                                 i/10, i%10,
                                 j/10, j%10,
                                 exptr->value[j],
                                 exptr->value[j-1],
                                 exptr->previousvalue,
                                 exptr->state[j]);
                    }

                    /* since there were no values for this bucket, propigate previous value.   */
                    exptr->value[j] = exptr->previousvalue;
                    exptr->state[j] = 3;  /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill) */
                    strcpy(&exptr->status[j][0], "F  ");/* "F  " = filled from previous bucket */
                }
                /* common processing */
                if (debuglevel >= BUCKETMSGSB)
                {
                     printf ("postprocess: [%d%d][%d%d],      \t\t%f   v(j-1)=%f, prevv=%f, status=%d\n",
                             i/10, i%10,
                             j/10, j%10,
                             exptr->value[j],
                             exptr->value[j-1],
                             exptr->previousvalue,
                             exptr->state[j]);
                }
            }

            if (debuglevel >= BUCKETMSGSB)
            {
                printf(":::: +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++\n");
            }

            /* point to next bucket */
            glptr->bucketgmt = glptr->nextbucketgmt;
            glptr->nextbucketgmt += glptr->bucketlen;
            glptr->bucketoffset++;


            /******************************************************************/
            /* this section not strictly required, but its easier to see when */
            /* examining the records that processing has been completed.      */
            /******************************************************************/
            if (glptr->flushflag)
            {
                for (i = 0; i < glptr->numpoints; i++)
                {
                    exptr = &exportrecord[i];

                    /* put flushing status and data into the remaining buckets. */
                    for (j = glptr->bucketoffset; j < glptr->maxnumbuckets; j++)
                    {
                        if (debuglevel >= BUCKETMSGSB)
                        {
                            printf ("flushprocess:   [%d%d][%d%d],      \t\t%f   v(j-1)=%f, prevv=%f, status=%d\n",
                                     i/10, i%10,
                                     j/10, j%10,
                                     exptr->value[j],
                                     exptr->value[j-1],
                                     exptr->previousvalue,
                                     exptr->state[j]);
                        }
                        exptr->value[j] = -1;
                        exptr->state[j] = 0;  /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill) */
                        strcpy(&exptr->status[j][0], "C  ");/* "I  " = ignore from previous bucket */
                    }
                }
            }

            /***************************************/
            /* check to see if the last bucket in  */
            /* the record has just been processed, */
            /* (or we are flushing the buffer?)    */
            /***************************************/
            if ((j >= glptr->maxnumbuckets - 1) || (glptr->flushflag))
            {
                if (debuglevel >= RECORDMSGS)
                {
                    printf("++++ Record Time Transition [%3d]   GMT:  %s",
                            glptr->numpoints, ctime(&glptr->recordgmt));
                }

                /***********************************************************/
                /* the last bucket in the record has just been processed.  */
                /* write the record and re-initialize for the next record. */
                /***********************************************************/

                if (debuglevel >= RECORDMSGSA)
                {
                    printf ("***** begin: write %d arSnapDesc records *****\n", glptr->numpoints);
                }

                /* break down record's start time and store for potential use */
                /* at a later time for renamed database filename composition. */
                tmTime = gmtime (&glptr->recordgmt);
                glptr->lastrecday  = tmTime->tm_yday + 1;

                for (i = 0; i < glptr->numpoints; i++)
                {
                    exptr = &exportrecord[i];

                    /* break down this record's start time */
                    tmTime = gmtime (&glptr->recordgmt);
                    exptr->year  = tmTime->tm_year;
                    exptr->month = tmTime->tm_mon;
                    exptr->day   = tmTime->tm_mday;
                    exptr->yday  = tmTime->tm_yday;
                    exptr->hour  = tmTime->tm_hour;
                    exptr->min   = tmTime->tm_min;

                    if (debuglevel >= RECORDMSGSB)
                    {
                        /* diagnostic only: print each of the records */
                        dumpexport(i);
                        printf ("::::  Year = %d, DOY = %d,  Hour = %d, OffMin = %d, SampleCnt = %d\n",
                                 exptr->year,
                                 exptr->yday,
                                 exptr->hour,
                                 exptr->min,
                                 exptr->fieldnum);
                        printf ("::::                 FullYear = %d, Month = %d,  Day = %d\n",
                                 exptr->year + 1900,
                                 exptr->month,
                                 exptr->day);
/*                       dumpmini(); */
                    }

                    /* write the record for this point */

                    AddToDestination(arSnapDesc);

                }

                if (debuglevel >= RECORDMSGSA)
                {
                    printf ("***** end:   write %d arSnapDesc records *****\n", glptr->numpoints);
                }

                /* initialize times to convert this "old" record to a "new" record */
                glptr->bucketgmt = glptr->nextbucketgmt;
                glptr->nextbucketgmt += glptr->bucketlen;
                glptr->bucketoffset = 0;
                glptr->recordnumber += 1;
                if (glptr->recordnumber >= glptr->maxnumrecords)
                {
                    glptr->recordnumber = 0;
                }

                /* set up for the next record */
                glptr->recordgmt = glptr->recordgmt += glptr->recordlen;

                /* convert individual "old" records to a "new" records */
                for (i = 0; i < glptr->numpoints; i++)
                {
                    exptr = &exportrecord[i];
                    /* convert this "old" record to a "new" record */
                    exptr->recordgmt = glptr->recordgmt;
                    exptr->fieldnum = 1;
                    for (j = 0; j < glptr->maxnumbuckets; j++)
                    {
                        exptr->value[j] = 0;
                        exptr->state[j] = 0;   /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill)  */
                        strcpy(&exptr->status[j][0], "q  "); /* "?  " = questionable data (raw)     */
                    }
                    /* propigate last bucket value of previous record to first bucket of this record */
                    exptr->count = 0;      /* no points contributed to value field             */
                    exptr->value[0] = .9999; /*exptr->previousvalue; */
                    exptr->state[0] = 4; /* 3;  /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill) */
                    strcpy(&exptr->status[0][0], "?  "); /* "F  ");/* "F  " = filled from previous bucket */
                }

                /********************************************************************/
                /* The record has been written. Check to see if it is time to close */
                /* this database, rename it, and open a new one. Of course, make    */
                /* sure that we are on an even day boundary (unless eof flushing).  */
                /********************************************************************/
                if (
                    (glptr->flushflag) ||
                    (
                     (glptr->recordnumber == 0) &&
                     (
                      (sizezfile() > MAXFILESIZE) ||
                      (timezfile() > MAXFILETIME)
                     )
                    )
                   )
/*
                if ( ((sizezfile() > MAXFILESIZE) ||
                      (                         )) && ( glptr->recordnumber == 0) ) ||
                     (glptr->flushflag))
*/
                {
                    /* close the current database and rename the file database files */
                    closezfile();


                    if (glptr->flushflag)
                    {
                        if (debuglevel >= FILEMSGS)
                        {
                            printf ("*********************************************************\n");
                            printf ("********** Finished Processing Input Records ************\n");
                            printf ("*********************************************************\n");
                        }
                        /***********/
                        /* go home */
                        /***********/
                        exit(0);
                    }

                    /*****************************************************************/
                    /* create the destination file. do codebase initalization magic. */
                    /*****************************************************************/
                    dbinit();

                    /* Open destination file, get and select  */
                    /* tags, position to the top of the file. */
                    if (!openzfile())
                    {
                        exit(1);
                    }

                    /* initialize new set of buckets. */

                    /******************************************************************/
                    /* this value unmodifed during life of this output database file. */
                    /******************************************************************/
                    glptr->recordnumber = 0;
                    glptr->firstrecordgmt = glptr->recordgmt;

                    /* break down record's start time and store for later file name composition. */
                    tmTime = gmtime (&glptr->recordgmt);
                    glptr->firstrecyear = tmTime->tm_year - ZEROYEAR;
                    glptr->firstrecday  = tmTime->tm_yday + 1;

                    if (debuglevel >= FILEMSGS)
                    {
                        printf(":::: This is the first record to be processed in this file.\n");
                        dumpglobal();
                        dumpmini();
                    }

                    recordindex = 1;

                    /********************/
                    /* lets rock again! */
                    /********************/
                    if (debuglevel >= FILEMSGS)
                    {
                        printf ("*********************************************************\n");
                        printf ("********* Continue Processing Input Records .... ********\n");
                        printf ("*********************************************************\n");
                    }
                }
            }
            else
            {
                if (debuglevel >= BUCKETMSGSB)
                {
                    printf ("***** bump valid bucket counters *****\n");
                }

                for (i = 0; i < glptr->numpoints; i++)
                {
                    exptr = &exportrecord[i];
                    exptr->fieldnum++;

                }
            }
        }

        if (firstpass)
        {
            firstpass = FALSE; /* clear it */

            /* get the first record's time to do initialization processing. */
            if ((!nextrecord (argc, argv)) || (sourcehandle == NULL))
            {
                /* something is wrong; there should be at least one record in the file. */
                /* before exiting the program, close the codebase database file.        */
                d4close(arSnapDesc);
                exit (0);
            }

            /**************************************************/
            /* calculate start time from the first record.    */
            /* At 1 minute interval, each record is one hour. */
            /* At 60 minute interval, each record is one day. */
            /**************************************************/

            glptr->currentgmt    =  imptr->gmt;                                 /* seconds since 1/1/70   */
            glptr->daynumber     =  glptr->currentgmt / SECS_PER_DAY;           /* days since 1/1/70      */

            glptr->secondnumber  =  glptr->currentgmt % SECS_PER_DAY;           /* seconds since midnight */
            glptr->bucketnumber  =  glptr->secondnumber / glptr->bucketlen;     /* buckets since midnight */
            glptr->recordnumber  =  glptr->secondnumber / glptr->recordlen;     /* records since midnight */
            glptr->bucketoffset  =  glptr->bucketnumber % glptr->maxnumbuckets; /* bucket offset in rec.  */


            glptr->daygmt        =  glptr->daynumber * SECS_PER_DAY;            /* today's gmt            */
            glptr->recordgmt     =  glptr->daygmt +                             /* this record's gmt      */
                                   (glptr->recordnumber * glptr->recordlen);
            glptr->bucketgmt     =  glptr->recordgmt +                          /* this bucket's gmt      */
                                   (glptr->bucketoffset * glptr->bucketlen);
            glptr->nextbucketgmt =  glptr->bucketgmt + glptr->bucketlen;        /* next bucket's gmt      */

            /******************************************************************/
            /* this value unmodifed during life of this output database file. */
            /******************************************************************/
            glptr->firstrecordgmt = glptr->recordgmt;

            /* break down record's start time and store for later file name composition. */
            tmTime = gmtime (&glptr->recordgmt);
            glptr->firstrecyear = tmTime->tm_year - ZEROYEAR;
            glptr->firstrecday  = tmTime->tm_yday + 1;

            if (debuglevel >= FILEMSGS)
            {
                printf(":::: This is the first record to be processed in this file.\n");
                dumpglobal();
                dumpmini();
            }
        }

        /*****************************************************************************************/
        /* either way, we are now aimed at the current export record. begin processing the input */
        /*****************************************************************************************/
        point_hit = FALSE;
        for (i = 0; i < glptr->numpoints; i++)
        {
            /* find this point's temp data record in the local buffer. */
            exptr = &exportrecord[i];
            if ((strcmp (&exptr->pointname[0], &imptr->pointname[0])) == 0)
            {
                /* we found it! */
                point_hit = TRUE;
                if (debuglevel >= INPUTDATAB)
                {
                    printf("::    Hit [%d,%d]\n", i, exptr->count);
                }
                break;
            }
        }
        if (i > MAXDBPOINTS)
        {
            /* if no room to hold data, then we had better ignore it. */
        }
        else if (point_hit == FALSE)
        {
            if (debuglevel >= INPUTDATAB)
            {
                printf(":: No Hit [%d]\n", i) ;
            }
            /******************************************************************/
            /* this is a customer database point that we haven't encountered  */
            /* before. after verifying that there is sufficent room in the    */
            /* buffer, add it to the list, initialize the buckets & counters. */
            /******************************************************************/
            exptr = &exportrecord[i];
            strcpy (&exptr->pointname[0], &imptr->pointname[0]);
            strcpy (&exptr->egu[0],       &imptr->egu[0]);
            strcpy (&exptr->displow[0],   &glptr->displow[0]);
            strcpy (&exptr->disphigh[0],  &glptr->disphigh[0]);
            strcpy (&exptr->method[0],    &glptr->method[0]);
            strcpy (&exptr->rate[0],    &glptr->rate[0]);
            exptr->recordgmt = glptr->recordgmt;
            exptr->fieldnum = glptr->bucketoffset+1;

            for (j = 0; j < NUMFIELDS; j++)
            {
                exptr->value[j] = 0;
                exptr->state[j] = 0;   /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill) */
                strcpy(&exptr->status[j][0], "C  "); /* "C  " = connect error (no data)    */
            }
            exptr->count = 0;           /* no points contributed to value field             */

            /* add this new point to the ar_reduc.dbf file */
            AddToReduc(arReducDesc);

            /* increment valid record count value */
            glptr->numpoints++;
        }

        /*************************************************/
        /* now add data to the local buffer record. see  */
        /* details of averaging algorithm for specifics. */
        /*************************************************/
        j = glptr->bucketoffset;
        if (exptr->count == 0)
        {
            exptr->value[j] = imptr->value;
        }
        else
        {
            exptr->value[j] += imptr->value;
        }
        exptr->count++;            /* # of points summed in the value field               */
        exptr->state[j] = 1;       /* 0=undef, 1=data(raw), 2=data(proc), 3=data(fill)    */
        strcpy(&exptr->status[j][0], "q  ");      /* "?  " = questionable data (raw)      */



        /* print at end of processing each input record */
        if (debuglevel >= INPUTDATAB)
        {
            printf(":::: bucketoffset = %d, fieldnum = %d, count = %d, value = %f\n",
                    glptr->bucketoffset, exptr->fieldnum, exptr->count, imptr->value);
            dumpexport(i);
        }
    }

    /********************************/
    /* close the CodeBase databases */
    /********************************/
    d4close(arSnapDesc);
    d4close(arReducDesc);

    /***********/
    /* go home */
    /***********/
    exit(0);

}

