/*snwsSort.c,v 1.1.1.1 1999/08/03 14:43:04 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
/*  File: snwsSort.c                                                            
/*  Edit: 12/20/96      WEP: original code                                      
/*                         :                                                    
*/

#include <stdio.h>
#include <string.h>

#include "msgMon.h"

#define   TRUE   1
#define   FALSE  0

#define NAMEFIELD    23
#define TYPEFIELD     0
#define YEARFIELD     4
#define MONTHFIELD    7
#define DAYFIELD     10
#define HOURFIELD    13
#define MINUTEFIELD  16
#define SECONDFIELD  19
#define EGUFIELD     49
#define DATAFIELD    32
#define DATAXFIELD   42
#define STATEFIELD   33

#define DAYSPERYEAR 365

#define RECSIZE        70
#define MAXDBREC   200000
#define MAXEVTREC  200000

/* five hundred thousand record buffer */
static unsigned char bigarray1[MAXDBREC][RECSIZE];
static unsigned char bigarray2[MAXEVTREC][RECSIZE];

/* sort by time, then pointname (to simplify detection of duplicates) */
static int recordcompare(i, j)
char  *i, *j;
{
    short results;

    /* 1st sort field: Date/Time */
    if ((results = strncmp(&i[4], &j[4], 18)) != 0) return (results);
    /* 2nd sort field: Variable Name */
    if ((results = strncmp(&i[23], &j[23], 9)) != 0) return (results);
    return (0);
}


void usage()
{
    printf("Usage:\n snwsSort {Source} (Destination1) (Destination2)   where:");
    printf("\t{Source}\t= complete path to file to be reordered.");
    printf("\t(Destination1) = base name of the trend data output file to be created.");
    printf("\t(Destination2) = base name of the event data output file to be created.");
}

/******************/
void main(int argc, char *argv[])

{
    FILE           *destination1;
    FILE           *destination2;
    FILE           *source;
    unsigned long   i1, i2, j;

    unsigned char   outbuffer[RECSIZE];
    unsigned char   record[RECSIZE];

    tisOpenLog ("snwsSort", 6, 2, NULL, NULL);

    /***************************************************************/
    /* determine that the correct number of parameters was passed. */
    /***************************************************************/

    if (argc != 4)
    {
        usage();
        exit(1);
    }

    /******************************************************************************/
    /* open the source file, verifying its existence: if it does not exist, exit. */
    /******************************************************************************/
    /* Open File for Read Only */
    if ((source = fopen(argv[1],"r")) == NULL)
    {
        tisSysLog(0,0,"ERROR : Cannot open Source file %s.", argv[1]);
        usage();
        exit(1);
    }

    /*******************************************************************************/
    /* open the destination1 file, verifying its non-existence: if it exist, exit. */
    /*******************************************************************************/
    /* Open File for Read Only */
    if ((destination1 = fopen(argv[2],"r")) != NULL)
    {
        fclose (destination1);
        fclose (source);
        tisSysLog(0,0,"ERROR : Destination1 file %s already exists.", argv[2]);
        usage();
        exit(1);
    }
   
    /*******************************************************************************/
    /* open the destination2 file, verifying its non-existence: if it exist, exit. */
    /*******************************************************************************/
    /* Open File for Read Only */
    if ((destination2 = fopen(argv[3],"r")) != NULL)
    {
        fclose (destination2);
        fclose (source);
        tisSysLog(0,0,"ERROR : Destination2 file %s already exists.",argv[3]);
        usage();
        exit(1);
    }

    /***************************************************************/
    /* create the destination1 file: if it cannot be opened, exit. */
    /***************************************************************/

    /* Open File for Write Only */
    if ((destination1 = fopen(argv[2],"w")) == NULL)
    {
        fclose (destination1);
        fclose (source);
        tisSysLog(0,0,"ERROR : Cannot create Destination1 file %s.", argv[2]);
        usage();
        exit(1);
    }
   
    /***************************************************************/
    /* create the destination2 file: if it cannot be opened, exit. */
    /***************************************************************/

    /* Open File for Write Only */
    if ((destination2 = fopen(argv[3],"w")) == NULL)
    {
        fclose (destination2);
        fclose (source);
        tisSysLog(0,0,"ERROR : Cannot create Destination2 file %s.", argv[3]);
        usage();
        exit(1);
    }
   
    /***********************************/
    /* read the current source record. */
    /***********************************/

    /* start with first record in each output buffer */
    i1 = 0;
    i2 = 0;

    while (fgets(record, RECSIZE, source))
    {
        /* copy the input buffer to the output buffer */
        strcpy(&outbuffer[0], &record[0]);

        /* rearrange the time and date fields, put */
        /* year first in date, and fill holes.     */

        outbuffer[4]  = (record[19]==' ')?'0':record[19];
        outbuffer[5]  = (record[20]==' ')?'0':record[20];
        outbuffer[6]  = ' ';
        outbuffer[7]  = (record[13]==' ')?'0':record[13];
        outbuffer[8]  = (record[14]==' ')?'0':record[14];
        outbuffer[9]  = ' ';
        outbuffer[10] = (record[16]==' ')?'0':record[16];
        outbuffer[11] = (record[17]==' ')?'0':record[17];

        outbuffer[13] = (record[4] ==' ')?'0':record[4];
        outbuffer[14] = (record[5] ==' ')?'0':record[5];
        outbuffer[15] = ' ';
        outbuffer[16] = (record[7] ==' ')?'0':record[7];
        outbuffer[17] = (record[8] ==' ')?'0':record[8];
        outbuffer[18] = ' ';
        outbuffer[19] = (record[10]==' ')?'0':record[10];
        outbuffer[20] = (record[11]==' ')?'0':record[11];

        /* if a DB record, then put it in the sorting buffer1 */
        if ((record[0]=='D') &&
            (record[1]=='B') &&
            (record[2]==' '))
        {
            if (i1 < MAXDBREC)
            {
                strncpy(bigarray1[i1++], outbuffer,  RECSIZE);
            }
            else
            {
                tisSysLog (0,0,"DB sort buffer full. additional input ignored...");
            }
        }
        /* if an EVT record, then put it in the sorting buffer2 */
        else if ((record[0]=='E') &&
                 (record[1]=='V') &&
                 (record[2]=='T') &&
                 (record[3]==' '))
        {
            if (i2 < MAXEVTREC)
            {
                /* printf ("#%4d:%s", i2, outbuffer); */
                strncpy(bigarray2[i2++], outbuffer,  RECSIZE);
            }
            else
            {
                tisSysLog (0,0,"EVT sort buffer full. additional input ignored...");
            }
        }
    }

    /*************************/
    /* close the source file */
    /*************************/
    tisSysLog(0,0,"Closing input file....");

    fclose (source);


    if (i1)
    {
        /******************************/
        /* Sort the destination1 file */
        /******************************/
        tisSysLog(0,0,"Begin qsort1 of %d DB records....", i1);

        qsort (bigarray1, i1, RECSIZE, recordcompare);

        /*******************************/
        /* write the destination1 file */
        /*******************************/
        tisSysLog(0,0,"Writing output1 ....");

        fputs(bigarray1[0], destination1);

        for (j = 1; j < i1; j++)
        {
            if (strcmp (bigarray1[j], bigarray1[j-1]) != 0)
            {
                fputs(bigarray1[j], destination1);
            }
        }
    }
    else
    {
        tisSysLog(0,0,"No DB records were found.");
    }

    /*******************************/
    /* close the destination1 file */
    /*******************************/
    tisSysLog(0,0,"Closing output1 ....");

    fclose(destination1);


    if (i2)
    {
        /******************************/
        /* Sort the destination2 file */
        /******************************/
        tisSysLog(0,0,"Begin qsort2 of %d EVT records....", i2);

        qsort (bigarray2, i2, RECSIZE, recordcompare);

        /*******************************/
        /* write the destination2 file */
        /*******************************/
        tisSysLog(0,0,"Writing output2 ....");

        fputs(bigarray2[0], destination2);

        for (j = 1; j < i2; j++)
        {
            if (strcmp (bigarray2[j], bigarray2[j-1]) != 0)
            {
                fputs(bigarray2[j], destination2);
            }
        }
    }
    else
    {
        tisSysLog(0,0,"No EVT records were found.");
    }

    /*******************************/
    /* close the destination2 file */
    /*******************************/
    tisSysLog(0,0,"Closing output2 ....");

      fclose(destination2);

    /***********/
    /* go home */
    /***********/
      exit(0);
}
