/*X_fdmgr.c,v 1.1.1.1 1999/08/03 14:43:10 tis4000 Exp */

/*
 *
 *	X _ F D M G R . C
 *
 *	Author	Grant Irani
 *
 *	A manager for use with the X windows system
 *
 *
 *
 *
 * 	History
 *
 */

#if defined(WIN32)
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <time.h>
#include <io.h>
#include <X11/Xlib.h>
#include <X11/XlibXtra.h>
#include <X11/intrinsic.h>
#include <time.h>
#include <winsock.h>
#include <stdlib.h>
#include <lstLib.h>
#else
@@@@ Dont compile in this case @@@@
#endif

struct fd_handler{
	void		(*pfunc)();
	void		*param;
};

typedef struct{
	NODE			node;
	void			*fd;
	XtInputId		inputId;
	struct fd_handler	rd;
	struct fd_handler	wt;
	struct fd_handler	excp;
	int			delete_pending;
}fdentry;

typedef struct{
	LIST		fdentry_list;
	LIST		fdentry_in_use_list;
	LIST		fdentry_free_list;
	LIST		alarm_list;
	LIST		expired_alarm_list;
	LIST		free_alarm_list;
	XtAppContext	appContext;
	int		timeId;
}fdctx;

typedef struct{
	NODE		node;
	XtIntervalId	timeId;
	struct timeval	t;
	void		(*func)();
	void		*param;
}alarm;

#if defined(WIN32)
int fdmgr_WIN32_gettimeval(fdctx *pfdctx, struct timeval *pt);
#	define LOCK(PFDCTX)
#	define UNLOCK(PFDCTX)
#	define LOCK_FDMGR_PEND_EVENT(PFDCTX) \
	{ \
		if((PFDCTX)->fdmgr_pend_event_in_use){ \
			printf("Double invocation of fdmgr_pend_event()\n"); \
			abort(); \
		} \
		(PFDCTX)->fdmgr_pend_event_in_use++; \
	}
#	define UNLOCK_FDMGR_PEND_EVENT(PFDCTX) \
		{(PFDCTX)->fdmgr_pend_event_in_use--;}
#	define fdmgr_gettimeval		fdmgr_WIN32_gettimeval
#else
@@@@ dont compile in this case @@@@
#endif

#define USEC_PER_SEC 1000000

static void 	select_alarm();
static void	fdmgr_finish_off_fdentry();
static int	fdmgr_select();
static  void	process_alarm_queue ();


/*
 *
 *	fdmgr_init()
 *
 */
fdctx	
*fdmgr_init()
{
    fdctx   *pfdctx;

    pfdctx = (fdctx *) calloc(1, sizeof(fdctx));

    /*
     * returns NULL if unsuccessfull
     */
    return pfdctx;
}




/*
 *
 * 	fdmgr_add_timeout()
 *
 *
 */
alarm 
*fdmgr_add_timeout(pfdctx, ptimeout, func ,param)
fdctx 		*pfdctx;
struct timeval 	*ptimeout;
void		(*func)(void *, XtIntervalId *);
void		*param;
{
    alarm   *palarm=NULL;
    alarm   *pa;

    LOCK(pfdctx);
    palarm = (alarm *) lstGet(&pfdctx->free_alarm_list);
    UNLOCK(pfdctx);
    if(!palarm){
	palarm = (alarm *) malloc(sizeof(alarm));
	if(!palarm){
	    return NULL;
	}
    }

    if ((ptimeout->tv_sec < 0) || (ptimeout->tv_usec < 0))
	return NULL;

    ptimeout->tv_sec += ptimeout->tv_usec/USEC_PER_SEC;
    ptimeout->tv_usec = ptimeout->tv_usec%USEC_PER_SEC;

    palarm->func = func;
    palarm->param = param;
    
    palarm->t.tv_sec = ptimeout->tv_sec;
    palarm->t.tv_usec = ptimeout->tv_usec;

    LOCK(pfdctx);
    for (pa=(alarm *)pfdctx->alarm_list.node.next; 
	pa; 
	pa=(alarm *)pa->node.next)
    {
	if(pa->t.tv_sec > palarm->t.tv_sec)
	{
	    break;
	}
	else
	{
	    if(pa->t.tv_sec == palarm->t.tv_sec)
	    {
		if(pa->t.tv_usec > palarm->t.tv_usec)
		{
		    break;
		}
	    }
	}
    }
    if(pa)
    {
	lstInsert(&pfdctx->alarm_list, pa->node.previous, (NODE *)palarm);
    }
    else
    {
	lstAdd(&pfdctx->alarm_list, (NODE *)palarm);
    }
#ifdef UseXtApp
    palarm->timeId = XtAppAddTimeOut (pfdctx->appContext,
				      (palarm->t.tv_sec*1000)
					+ (palarm->t.tv_sec/1000),
				      func,
				      param);
#else
    palarm->timeId = XtAddTimeOut ((palarm->t.tv_sec*1000)
					+ (palarm->t.tv_sec/1000),
				    func,
				    param);
#endif UseXtApp
    UNLOCK(pfdctx);

    return (void *) palarm;
}


/*
 *
 *	fdmgr_clear_timeout()
 *
 */
int
fdmgr_clear_timeout(pfdctx, palarm)
fdctx 		*pfdctx;
alarm		*palarm;
{
    int status;

    LOCK(pfdctx);
    status = lstFind(&pfdctx->alarm_list, (NODE *)palarm);
    if(status != ERROR){
	XtRemoveTimeOut (palarm->timeId);
	lstDelete(&pfdctx->alarm_list, (NODE *)palarm);
	lstAdd(&pfdctx->free_alarm_list, (NODE *)palarm);
    }
    UNLOCK(pfdctx);

    return status;
}


/*
 *
 *	fdmgr_add_fd()
 *
 *	this rouitine s only supplied for compatibility	
 *	with earlier versions of this software
 */
int 
fdmgr_add_fd(pfdctx, fd, func, param)
fdctx 	*pfdctx;
int	fd;
void	(*func)(void *, int *, unsigned long *);
void	*param;
{
    int status;

#ifdef UseXtApp
    status = XtAppAddInput(
		    pfdctx->appContext, 
		    fd,
		    (XtInputCallbackProc)XtInputReadWinsock,
		    func, 
		    param);
#else
    status = XtAddInput(
		    fd,
		    (XtInputCallbackProc)XtInputReadWinsock,
		    func, 
		    param);
#endif UseXtApp
    return status != 0;
}


/*
 *
 *	fdmgr_add_fd_callbacks()
 *
 */
int 
fdmgr_add_fd_callback(pfdctx, fd, rd_func, rd_param, 
		wt_func, wt_param, excep_func, excep_param)
fdctx 	*pfdctx;
int	fd;
void	(*rd_func)(void *, int *, unsigned long *);
void	*rd_param;
void	(*wt_func)(void *, int *, unsigned long *);
void	*wt_param;
void	(*excep_func)(void *, int *, unsigned long *);
void	*excep_param;
{
    fdentry		*pfdentry;

    LOCK(pfdctx);
    pfdentry = (fdentry *) lstGet(&pfdctx->fdentry_free_list);
    UNLOCK(pfdctx);

    if(!pfdentry){
	pfdentry = (fdentry *) calloc(1, sizeof(fdentry));
	if(!pfdentry){
	    return ERROR;
	}
    }

    pfdentry->fd = (void *)fd;
    pfdentry->rd.pfunc = rd_func;
    pfdentry->rd.param = rd_param;
    if ((pfdentry->wt.pfunc = wt_func)
	 || (pfdentry->excp.pfunc = excep_func))
    {
	printf ("Write or exception funcs were ignored!\n");
    }
    pfdentry->delete_pending = 0;
#ifdef UseXtApp
    pfdentry->inputId = XtAppAddInput(
			    pfdctx->appContext, 
			    fd,
			    (XtInputCallbackProc)XtInputReadWinsock,
			    rd_func, 
			    rd_param);
#else
    pfdentry->inputId = XtAddInput(
			    fd,
			    (XtInputCallbackProc)XtInputReadWinsock,
			    rd_func, 
			    rd_param);
#endif UseXtApp

    LOCK(pfdctx);
    lstAdd(&pfdctx->fdentry_list, (NODE *)pfdentry);
    UNLOCK(pfdctx);

    return 0;
}


/*
 *
 *	fdmgr_clear_fd()
 *
 */
int
fdmgr_clear_fd(pfdctx, fd)
fdctx 	*pfdctx;
int	fd;
{
    register fdentry	*pfdentry;
    int			status;

    status = ERROR;

    LOCK(pfdctx);
    for (pfdentry = (fdentry *) pfdctx->fdentry_list.node.next;
	pfdentry;
	pfdentry = (fdentry *) pfdentry->node.next)
    {
	if(pfdentry->fd == (void *)fd)
	{
	    lstDelete(&pfdctx->fdentry_list, (NODE *)pfdentry);
	    fdmgr_finish_off_fdentry(pfdctx, pfdentry);
	    status = 0;
	    break;
	}
    }

    /*
     * also check entries which are currently in use
     * and therefore undeletable until after the
     * fdmgr gets thru fdmgr_pend_event
     */
    for (pfdentry = (fdentry *) pfdctx->fdentry_in_use_list.node.next;
         pfdentry;
         pfdentry = (fdentry *) pfdentry->node.next)
    {
        if(pfdentry->fd == (void *)fd)
	{
	    pfdentry->delete_pending = TRUE;
            status = 0;
            break;
        }
    }
    UNLOCK(pfdctx);
    
    /*
     * If it is an ukn fd its a bug worth printing out
     */
    if(status != 0){
	printf("fdmg: delete of ukn fd failed\n");
    }

    return status;
}


/*
 *
 *	fdmgr_finish_off_fdentry()
 *
 *	!! LOCK(pfdctx) must be applied !!
 *
 */
static void
fdmgr_finish_off_fdentry(pfdctx, pfdentry)
fdctx 			*pfdctx;
register fdentry	*pfdentry;
{
    XtRemoveInput (pfdentry->inputId);
    lstAdd(&pfdctx->fdentry_free_list, (NODE *)pfdentry);
}


static void fd_timekickout (fdctx *pfdctx, XtIntervalId *id)
{
#ifdef XYZZY
    XClientMessageEvent event;
    Atom MSG_ATOM;
    char msg_name[] = "SESS_MSGQ";
    Window window = XtUnspecifiedWindow;

    pfdctx->timeId = 0;

    MSG_ATOM = XInternAtom(XtDisplay(window),msg_name,FALSE);
    event.display = XtDisplay(window);
    event.window = window;
    event.type = ClientMessage;
    event.format = 8;
    event.message_type = MSG_ATOM;
    XSendEvent(event.display,window,TRUE,0,&event);
    XFlush(event.display);
#endif XYZZY
}

/*
 *
 *	fdmgr_pend_event()
 *
 */
int
fdmgr_pend_event(pfdctx, ptimeout)
fdctx 				*pfdctx;
struct timeval 			*ptimeout;
{
    XEvent  event;
    /*
     * This routine is declared here since is only 
     * intended for use by this routine. If other
     * uses are needed then the locking issues must be
     * reinvestigated
     */
#ifdef UseXtApp
    pfdctx->timeId = XtAppAddTimeOut (pfdctx->appContext,
				 (ptimeout->tv_sec * 1000)
					+ (ptimeout->tv_usec/1000),
				 fd_timekickout,
				 pfdctx);
#else
    pfdctx->timeId = XtAddTimeOut ((ptimeout->tv_sec * 1000)
					+ (ptimeout->tv_usec/1000),
				    fd_timekickout,
				    pfdctx);
#endif UseXtApp

    while(pfdctx->timeId){
#if UseXtApp
	XtAppNextEvent (pfdctx->appContext, &event);
#else
	XtNextEvent (&event);
#endif UseXtApp
	XtDispatchEvent (&event);
    }

    if(pfdctx->timeId)
	XtRemoveTimeOut (pfdctx->timeId);

    return 0;
}


#ifdef WIN32
fdmgr_WIN32_gettimeval(pfdctx, pt)
fdctx 		*pfdctx;
struct timeval	*pt;
{
    SYSTEMTIME systime;

	GetSystemTime (&systime);
	pt->tv_sec = time(0);
	pt->tv_usec = systime.wMilliseconds * 1000;
	return 0;
}
#endif
