/*errMessage.c,v 1.1.1.1 1999/08/03 14:43:07 tis4000 Exp */

/* errMessage.c */
/* share/src/db  @(#)errMessage.c	1.2     6/7/91 */
/*
 *      Author:          Bob Zieman
 *      Date:            6-1-91
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  mm-dd-yy        iii     Comment
 */

/* errMessage.c - Handle error messages */

/***************************************************************************
 * This must ultimately be replaced by a facility that allows remote
 * nodes access to the error messages. A message handling communication
 * task should be written that allows multiple remote nodes to request
 * notification of all error messages.
 * For now lets just print messages and last errno on local console
 ***************************************************************************/
#ifdef WIN32
#define caddr_t	char *
#endif WIN32


#ifdef vxWorks
#include <vxWorks.h>
#include <stdio.h>
#include <taskLib.h>
#include <types.h>
#include <logLib.h>
#else
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#if defined(AXP) || defined(WIN32)
#include <stdlib.h>
#endif
#endif
#include "error.h"

extern ERRSYMTAB_ID errSymTbl;
/*extern char    *calloc();*/
struct errDes  *dbErrDes = NULL;

int errSymBld(void);

/*
 * ERRMESSAGE
 */
void errMessage(status, pmessage)
    long            status;
    char           *pmessage;
{
#ifdef vxWorks
    int             id;
    static int      saveid = -1;
    char           *pname;
#endif
    if (!dbErrDes)
	if (errSymBld())
#ifdef vxWorks
	    taskSuspend (taskIdSelf());
#else
	    exit (1);
#endif vxWorks

#ifdef vxWorks
    id = taskIdSelf();
    if (saveid != id) {
	saveid = id;
	pname = taskName(id);
	logMsg("taskid=%x taskname=%s\n", id, (int)pname, 0, 0, 0, 0);
    }
#endif
    if (pmessage != NULL)
#ifdef vxWorks	
	logMsg("%s\n", (int)pmessage, 0, 0, 0, 0 ,0);
#else
	printf("%s\n", pmessage);
#endif vxWorks
    if (status != -1) {
	errPrint(status);
    }
}

/*
 * ERRSYMBLD
 *
 * Create and initializes the dbErrDes structure.
 * This allows direct lookup of Module error messages.
 *
 */

int errSymBld(void)
{
    struct errDes  *pTdbErrDes;
    ERRSYMBOL      *errArray = errSymTbl->symbols;
    ERRSYMBOL      *perrArray;

    int             i;
    int             j;
    int             modnum;
    int             modoff;
    int             errnum;
    int             erroff;
    int             entDim;
    int             modDim = 0;
    long            Size = 0;
    char           *addrI;
    perrArray = errArray;

    /* determine the maximum module number */
    for (i = 0; i < errSymTbl->nsymbols; i++, errArray++) {
	modnum = errArray->errNum >> 16;
	modoff = modnum - 501;
	errnum = errArray->errNum & 0xffff;
	erroff = errnum >> 1;
	if (modnum < 501) {
	    printf("errSymBld: ERROR - Module number in errSymTbl < 501\n");
	    return (-1);
	}
	modDim = modDim <= modoff ? modoff + 1 : modDim;
    }
    /* allocate pTdbErrDes structure */
    if ((pTdbErrDes = (struct errDes *)
	 calloc(1, (unsigned) sizeof(struct errDes))) == NULL) {
	printf("errSymBld: calloc error\n");
	return (-1);
    }
    Size = sizeof(struct errDes);

    /* allocate papErrSet pointer array */
    if ((pTdbErrDes->papErrSet = (struct errSet **)
	 calloc(1, (unsigned) modDim * sizeof(caddr_t))) == NULL) {
	printf("errSymBld: calloc error\n");
	return (-1);
    }
    pTdbErrDes->number = modDim;
    Size += pTdbErrDes->number * sizeof(caddr_t);

    /* allocate a temp structure for each module */
    /* and insert the maximum dimension for it's  messages */
    errArray = perrArray ;
    for (i = 0; i < errSymTbl->nsymbols; i++, errArray++) {
	modnum = errArray->errNum >> 16;
	modoff = modnum - 501;
	errnum = errArray->errNum & 0xffff;
	erroff = errnum >> 1;
	/* has module been allocated yet */
	if (!pTdbErrDes->papErrSet[modoff]) {
	    /* assign a structure for each module */
	    if ((pTdbErrDes->papErrSet[modoff] = (struct errSet *)
		 calloc(1, (unsigned) sizeof(struct errSet))) == NULL) {
		printf("errSymBld: calloc error\n");
		return (-1);
	    }
	    Size += sizeof(struct errSet);
	}
	/* determine max message number in each module */
	/* max entries in each module */
	entDim = pTdbErrDes->papErrSet[modoff]->number;
	entDim = entDim <= erroff ? erroff + 1 : entDim;
	pTdbErrDes->papErrSet[modoff]->number = entDim;
    }
    /* allocate a temp pointer array for each module */
    /* and insert pointer to err string */

    errArray = perrArray ;
    for (i = 0; i < errSymTbl->nsymbols; i++, errArray++) {
	modnum = errArray->errNum >> 16;
	modoff = modnum - 501;
	errnum = errArray->errNum & 0xffff;
	erroff = errnum >> 1;
	if (pTdbErrDes->papErrSet[modoff]) {
	    if (!pTdbErrDes->papErrSet[modoff]->papName) {
		/* allocate ptr array */
		if (!pTdbErrDes->papErrSet[modoff]->papName) {
		    if ((pTdbErrDes->papErrSet[modoff]->papName = (caddr_t *)
			 calloc(1, (unsigned) pTdbErrDes->papErrSet[modoff]->number
				* sizeof(caddr_t))) == NULL) {
			printf("errSymBld: calloc error\n");
			return (-1);
		    }
		    Size += pTdbErrDes->papErrSet[modoff]->number * sizeof(caddr_t);
		}
	    }
	    if (!pTdbErrDes->papErrSet[modoff]->papName[erroff]) {
		pTdbErrDes->papErrSet[modoff]->papName[erroff] = errArray->name;
	    }
	}
    }

    /* allocate the contiguous area */
    if ((dbErrDes = (struct errDes *) calloc(1, (unsigned) Size)) == NULL) {
	printf("errSymBld: calloc error\n");
	return (-1);
    }
    addrI = (caddr_t) dbErrDes + sizeof(struct errDes);
    dbErrDes->papErrSet = (struct errSet **) addrI;
    dbErrDes->number = pTdbErrDes->number;
    addrI += dbErrDes->number * sizeof(caddr_t);

    /* insert number nd pointers and strings */
    for (i = 0; i < dbErrDes->number; i++) {
	if (pTdbErrDes->papErrSet[i]) {
	    dbErrDes->papErrSet[i] = (struct errSet *) addrI;
	    addrI += sizeof(struct errSet);
	    dbErrDes->papErrSet[i]->number = pTdbErrDes->papErrSet[i]->number;
	    dbErrDes->papErrSet[i]->papName = (caddr_t *) addrI;
	    addrI += dbErrDes->papErrSet[i]->number * sizeof(caddr_t);
	    for (j = 0; j < dbErrDes->papErrSet[i]->number; j++) {
		if (pTdbErrDes->papErrSet[i]->papName[j]) {
		    dbErrDes->papErrSet[i]->papName[j]
			= pTdbErrDes->papErrSet[i]->papName[j];
		}
	    }
	}
    }
return(0);
}				/* end of  errMessage.c */
