/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/* ar_schedule.h - definitions for ar_schedule handling.
** ar_schedules are specified in place of a numeric rate field
** in the ar_reduc file to define a set of successive rates that
** reduce the storage requirements for data past a certain age.
**
** The required information is maintained in a plain text file
** named /tis4000db/ar_schedule.dat with structure:
** <schedule_name>:
**	<age_value> <interval_value>
**	...
** <schedule_name>
**	...
**
*/

#ifndef _AR_SCHEDULE_H_
#define _AR_SCHEDULE_H_

#define MAX_SCHEDULE	10
#define MAX_NAME	10

typedef struct {
    int		age;		/* age in days, always a 0 entry for basic rate */
    int		interval;	/* rate for period between this age value and next */
} SCHED_ENTRY;

typedef struct {
    char	name [MAX_NAME];
    int		numschedule;	/* number of changes in schedule */
    SCHED_ENTRY schedule [MAX_SCHEDULE];
} REDUC_SCHEDULE;

/* support routines in ar_schedule.c */
char	**GetScheduleNames ();		/* get array of schedule names */
void FreeScheduleNames (char **namePtr);/* free array of schedule names */
int	GetBasicRate (char *schedName);	/* get initial reduction rate entry */
int	GetSchedule (char *schedName, REDUC_SCHEDULE *schedule); /* get complete schedule data */
int	PutSchedule (REDUC_SCHEDULE *schedule); /* write a schedule */
int	RemoveSchedule (char *schedName); /* delete a schedule */

#endif _AR_SCHEDULE_H_
