/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#include <time.h>

#ifdef WIN32
#include <process.h>
#endif WIN32
#include <stdlib.h>


#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#include "cb45to63.h"
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "oracleSnap.h"
#include "trendSizes.h"
#ifdef ORACLESNAP
#include "oracleSnapLib.h"
#else
#include "avmDefs.h"
#include "xbaseSnapLib.h"
#endif ORACLESNAP
#include "archiveLib.h"

#include "utils.h"


static C4CODE	cb;
static int cbInit = 0;
static D4DATA	*redlist;
static D4DATA	*defs;
static F4FIELD	*field_ptr;	/* Handle to field in record */
static int	field_ix;   /* field index of next sample */

static F4FIELD_INFO field_info[] = /* define database field structure */
{
   /* field name,   type, width, decimals */
   { "DESCRIPTOR",'C', TRENDDESCRIPTOR_SIZE,	 0 },
   { "ZOOM",	  'N',   10,	 0 },
   { "TREND0",	  'C',   70,	 0 },
   { "TREND1",	  'C',   70,	 0 },
   { "TREND2",	  'C',   70,	 0 },
   { "TREND3",	  'C',   70,	 0 },
   { "TREND4",	  'C',   70,	 0 },
   { "TREND5",	  'C',   70,	 0 },
   { "TREND6",	  'C',   70,	 0 },
   { "TREND7",	  'C',   70,	 0 },
   { 0, 0, 0, 0 },
} ;

static T4TAG_INFO tag_info[] = /* define multiple tags */
{
   /* name,	    expression,	filter, unique,	     descending */
   { "DESCRIPTOR","DESCRIPTOR",	"",	  0,	     	0	   },
   { 0,  0,  0, 0, 0 },
};

static char	*sev = "VMmIE";
#define NUMSEV 5
static char	*qual = "*QCqF"; /* in precedence order*/
#define NUMQUAL 5
#define NUMALM 22

#ifdef ORACLESNAP
static OracleDesc *oDesc;
#else
static XBaseDesc *oDesc;
#endif ORACLESNAP
static SnapRec	snaprec;
static long rectime;

static char tempstring [132];

/* the following array maps my reduc values onto the trend subsystems values */
#include "trendLib.h"
int	libReducs [] = { TREND_SMP, TREND_AVG, TREND_MIN, TREND_MAX };

int trendAccess;	/* have access to current trend data */
#ifdef WIN32
int	myPid;
#else
pid_t myPid;
#endif WIN32

/* collect a set of ar_snap value/status pairs for the given time units/span,
** reducing data if necessary. Returns number values entered in the arrays. */
int GetSnapValues (char *pointname, char *dbreduc, char *reduc,
		   time_t unitsecs, time_t start, long *dbrate,
		    char *egutag, double *disp_low, double *disp_high,
		   int reqnum, double values[], unsigned char status[][3], double values2[])
{
    int	sampix;		/* index in return value arrays */
    int sampcnt;	/* remaining values to collect */
    int snapix;		/* index in snapRec values */
    int mydbrate;	/* local copy of dbrate */
    double elements;	/* number of elements in reduction of values */
    SnapRec snapRec;	/* record from data base */
    int sawvaliddata;	/* flag indicating db data in this sample */
    long valuetime;	/* time for current db value */
    long lastvaluetime;	/* time for last used db value */
    int red;		/* type of reduction to apply to collected data */
    int dbred;		/* type of reduction used in collected data */
    long samptime = start; /* ending time of current sample */
    double tempvalue;
    char tempstatus [4];
    int lookInArchive;
    int fix;
    long fixtime;
    double qualifiers;	/* GCT 29-Apr-1999 used for average calculation */

    sampix = 0;
    sampcnt = reqnum;
    if (strncmp (reduc, "AVG", 3) == 0)
	red = 1;
    else if (strncmp (reduc, "MIN", 3) == 0)
	red = 2;
    else if (strncmp (reduc, "MAX", 3) == 0)
	red = 3;
    else if (strncmp (reduc, "BAR", 3) == 0)
	red = 4;
    else
	red = 0;

    if (strncmp (dbreduc, "AVG", 3) == 0)
	dbred = 1;
    else if (strncmp (dbreduc, "MIN", 3) == 0)
	dbred = 2;
    else if (strncmp (dbreduc, "MAX", 3) == 0)
	dbred = 3;
    else if (strncmp (dbreduc, "BAR", 3) == 0)
	dbred = 4;
    else
	dbred = 0;

    /* need invalid date indication for fill code */
    valuetime = 0x7fffffffL;

    /* Collect and reduce pointdata, start at best guess, one unitsecs before */
    if (StartPointSet (pointname, dbreduc, (start - unitsecs), start + (reqnum*unitsecs), &snapRec))
    {
	lookInArchive = 1;
	snapix = 0;
	valuetime = snapRec.rec_time;
	*disp_low = snapRec.disp_low;
	*disp_high = snapRec.disp_high;
	if (*disp_low == *disp_high)
	    *disp_high = *disp_low + 1000000.0;
        strncpy (egutag, snapRec.egutag, sizeof(snapRec.egutag));
        if (dbrate)
	    *dbrate = snapRec.rate;
	mydbrate = snapRec.rate;
	tempvalue = snapRec.value[0];
	tempstatus [0] = (snapRec.status[snapix][0] == ' ')?'-':snapRec.status[snapix][0];
	tempstatus [1] = (snapRec.status[snapix][1] == ' ')?'-':snapRec.status[snapix][1];
	tempstatus [2] = (snapRec.status[snapix][2] == ' ')?'-':snapRec.status[snapix][2];
        for (sampcnt = reqnum, sampix = 0; sampcnt > 0; sampix++, sampcnt--)
        {
            values[sampix] = 0;
            elements = 0;
	    qualifiers = 0;		/* GCT 29-Apr-1999 used for average calculation */
            *status[sampix] = 0xff; /* data not present flag */
            sawvaliddata = FALSE;
	    lastvaluetime = 0x7fffffffL;
            do
            {
		/* void samples up to time of this data item */
                while ((valuetime != 0x7fffffff)
		  && (samptime < valuetime))
                {
		    sampcnt--;
		    if (sampcnt == 0)
			break;
		    sampix ++;
		    values[sampix] = 0;
 	            elements = 0;
		    qualifiers = 0;
 	            *status[sampix] = 0xff; /* data not present flag */
 	            sawvaliddata = FALSE;
		    samptime += unitsecs;
		}
                /* accumulate values that overlap the previous unitsecs period only */
                if (sampcnt && (valuetime != 0x7fffffff) && (samptime <= time(0))
		  && ((((samptime - unitsecs) < valuetime) && (valuetime <= samptime))
		    || ((valuetime-mydbrate < samptime) && (samptime <= valuetime))))
                {
		    lastvaluetime = valuetime;
                    switch (red)
                    {
                        case 0: /* sample */
                            elements++;
                            values[sampix] = tempvalue;
                            break;
                        case 1: /* average */
                            elements++;
			    if(tempstatus[0] != 'C')
			      {
				qualifiers++;
				values[sampix] = (values[sampix] * (qualifiers - 1) + tempvalue)/qualifiers;
			      }
                            break;
                        case 2: /* min */
                            elements++;
                            if ((elements == 1) || (tempvalue < values[sampix]))
                                values[sampix] = tempvalue; 
                            break;
                        case 3: /* max */
                            elements++;
                            if ((elements == 1) || (tempvalue > values[sampix]))
                                values[sampix] = tempvalue; 
                            break;
                        case 4: /* bar */
                            if (*status[sampix] == 0xff) /* first value in sample */
                            {
                                values[sampix] = tempvalue;
                                elements = tempvalue;
                            }
                            else
                            {
                                if (tempvalue < values[sampix])
                                    values[sampix] = tempvalue;
                                else if (tempvalue > elements)
                                    elements = tempvalue;
                            }
                            break;
                    }
                    if (*status[sampix] == (unsigned char)0xff)
                        strncpy (status[sampix], tempstatus, 3);
                    else    /* rest of samples are subjected to precedence */
                    {
                        /* if new quality is worse, update the quality */
                        if (strchr(qual, tempstatus[0]) < strchr(qual, status[sampix][0]))
                            *status[sampix] = tempstatus[0];
                        if (tempstatus[0] != 'C')
			{
                            sawvaliddata = TRUE;
			}

                        /* if new error is more severe or we haven't seen an alarm,
                        ** take value from new sample */
                        if ((strchr (sev, tempstatus[1]) < strchr (sev, status[sampix][1]))
                          || (*status[sampix]+2 == ' '))
			{
		            *(status[sampix]+1) = tempstatus[1];
                            *(status[sampix]+2) = tempstatus[2];
			}
                    }
                }
                if (sampcnt && (valuetime != 0x7fffffff))
		{
		    /* move to next value */
		    valuetime += mydbrate;
                    if ((snapix + 1) == snapRec.sample_cnt)
		    {
			if (lookInArchive && NextPointSet (&snapRec, start + (reqnum*unitsecs)))
			{
			    mydbrate = snapRec.rate;
			    snapix = 0;
			    valuetime = snapRec.rec_time;
			    tempvalue = snapRec.value[snapix];
			    tempstatus [0] = (snapRec.status[snapix][0] == ' ')?'-':snapRec.status[snapix][0];
			    tempstatus [1] = (snapRec.status[snapix][1] == ' ')?'-':snapRec.status[snapix][1];
			    tempstatus [2] = (snapRec.status[snapix][2] == ' ')?'-':snapRec.status[snapix][2];
			}
			else
			{
			    lookInArchive = 0;
			    /* no more data in file, look in memory */
			    valuetime = MemValue (valuetime, pointname, dbred, &tempvalue, tempstatus);
			    if (tempstatus[0] == ' ') tempstatus [0] = '-';
			    if (tempstatus[1] == ' ') tempstatus [1] = '-';
			    if (tempstatus[2] == ' ') tempstatus [2] = '-';
			}
		    }
		    else
		    {
    			snapix++;
			tempvalue = snapRec.value[snapix];
			tempstatus [0] = (snapRec.status[snapix][0] == ' ')?'-':snapRec.status[snapix][0];
			tempstatus [1] = (snapRec.status[snapix][1] == ' ')?'-':snapRec.status[snapix][1];
			tempstatus [2] = (snapRec.status[snapix][2] == ' ')?'-':snapRec.status[snapix][2];
		    }
		}
            } while ((valuetime <= samptime) && (valuetime != 0x7fffffffL) && (sampcnt != 0));

            /* if quality level is Connect but we saw valid data, convert to Q status */
            if ((*status[sampix] != 0xff) && ((status[sampix][0] == 'C') && sawvaliddata))
		status[sampix][0] = 'Q';
 	    if (values2)
		values2[sampix] = elements;
	    /* fill values back to beginning of previous DB period */
	    if (unitsecs < mydbrate)
	    {
		fix = sampix - 1;
		fixtime = samptime - unitsecs;
		while ((fixtime > (lastvaluetime - mydbrate)) && (fix >=0))
		{
		    if (fixtime <= time(0))
		    {
			values[fix] = values[sampix];
			strncpy (status[fix], status[sampix], 3);
			if (values2)
			    values2[fix] = 0;
		    }
		    fix--;
		    fixtime -= unitsecs;
		}
	    }
	    samptime += unitsecs;
        }
    }
    /* fill values back to beginning of DB period for trailing cells */
    if ((unitsecs < mydbrate) && (valuetime != 0x7fffffffL))
    {
	fix = sampix - 1;
	fixtime = samptime - unitsecs;
	while ((fixtime > (valuetime - mydbrate)) && (fix >=0))
	{
	    if (fixtime <= time(0))
	    {
		values[fix] = tempvalue;
		strncpy (status[fix], tempstatus, 3);
		if (values2)
		    values2[fix] = 0;
	    }
	    fix--;
	    fixtime -= unitsecs;
	}
    }

    for (; sampcnt > 0; sampix++, sampcnt--)
    {
        values[sampix] = 0;
        if (values2)
	    values2 [sampix] = 0;
        *status[sampix] = (unsigned)0xff;
    }
    return sampix;
}

int TrendMemOpen (void)
{
    /* open trend process access */
#ifdef WIN32
    myPid = _getpid ();
#else
    myPid = getpid ();
#endif WIN32
    trendAccess = trendInit ();
    return trendAccess;
}

void TrendMemClose (void)
{
    if (trendAccess)
	trendTerminate ();
}

void PointOpen (char *epn, int dbred)
{
    if (trendAccess)
	trendInsert (epn, libReducs[dbred]);
}

void PointClose (char *epn, int dbred)
{
    if (trendAccess)
	trendRemove (epn, libReducs[dbred]);
}

long MemValue (long valtime, char *pointname, int dbred, double *value, char *status)
{
    unsigned int dataix;
    CurrentData currentdata;

    if (trendAccess && valtime <= time(NULL))
    {
	if ((pointname[0] == '>'
		?trendGet (&pointname[1], libReducs[dbred], &currentdata)
		:trendGet (pointname, libReducs[dbred], &currentdata)))
	{
	    if (valtime < currentdata.startTime)
		valtime = currentdata.startTime;
	    dataix = (valtime - currentdata.startTime)/currentdata.interval;
	    if (dataix < currentdata.sampleCnt)
	    {
		*value = currentdata.value [dataix];
		status[0] = currentdata.quality [dataix];
		status[1] = severity2letter(currentdata.severity [dataix]);
		status[2] = status2letter(currentdata.status [dataix]);
		status[3] = 0;
	    }
	    else if (currentdata.sampleCnt > 0)
	    {
		*value = currentdata.lastValue;
		if (currentdata.quality[currentdata.sampleCnt-1] != 'C')
		    status[0] = 'F';
		else
		    status[0] = 'C';
		status[1] = severity2letter(currentdata.lastSeverity);
		status[2] = status2letter(currentdata.lastStatus);
		status[3] = 0;
	    }
	    else
		return (0x7fffffff);
	    return valtime;
	}
	else
	    return (0x7fffffff);
    }
    else
	return (0x7fffffff);
}

/* iteration routines for walking the trend data */
/* variables used by iteration interface */
int	lastpointtime;
static long	valuetime;

static long	rectime;
static int	tempsamples;
static int	field_ix;
static double	tempvalue;
static char	tempstatus[4];

long NextItem ()
{
    /* move to next item time */
    if (valuetime != 0x7fffffffL)
	valuetime += snaprec.rate;
    /* stop looking for data when time is current */
    if (valuetime >= time(NULL))
	valuetime = 0x7fffffffL;

    /* go on to next value */
    if (rectime != 0x7fffffffL)
    {
        if (tempsamples <= 0)
	{
	    if (!GetNextRec (snaprec.pointname, snaprec.red_method, 0))
	    {
		valuetime = 0x7fffffffL;
	    }
	}
	else
	{
	    tempvalue = snaprec.value[field_ix];
            strncpy (tempstatus, snaprec.status[field_ix], sizeof(tempstatus));
	    field_ix++;
            tempsamples--;
	}
    }
    else if (valuetime != 0x7fffffffL)
    {
	valuetime = 0x7fffffffL;
    }

    return valuetime;
}

long ItemRec ()
{
    return snaprec.rec_time;
}

int ItemField ()
{
    return (field_ix - 1);
}

char *RecName ()
{
    return snaprec.pointname;
}

char *RecTag ()
{
    return snaprec.egutag;
}

int RecReduc ()
{
    if (strncmp (snaprec.red_method, "AVG", 3) == 0)
	return 1;
    else if (strncmp (snaprec.red_method, "MIN", 3) == 0)
	return 2;
    else if (strncmp (snaprec.red_method, "MAX", 3) == 0)
	return 3;
    else if (strncmp (snaprec.red_method, "BAR", 3) == 0)
	return 4;
    else
	return 0;
}

double RecDispLo ()
{
    return snaprec.disp_low;
}

double RecDispHi ()
{
    return snaprec.disp_high;
}

long RecRate ()
{
    return snaprec.rate;
}

int RecSamples ()
{
    return snaprec.sample_cnt;
}


long ItemTime ()
{
    return valuetime;
}

double ItemValue ()
{
    return tempvalue;
}

char *ItemStatus ()
{
    /* fix blanks in status */
    if (tempstatus[0] == ' ') tempstatus[0] = '-';
    if (tempstatus[1] == ' ') tempstatus[1] = '-';
    if (tempstatus[2] == ' ') tempstatus[2] = '-';
    return tempstatus;
}


/* support routine that loads the temp variables from the current record */
static void LoadTemp ()
{
    rectime = snaprec.rec_time;
    tempsamples = snaprec.sample_cnt - 1; /* decrement since we read the first one */
    tempvalue = snaprec.value[0];
    strncpy (tempstatus, snaprec.status[0], sizeof(tempstatus));
    tempstatus[3] = 0; /* terminate the status string */
    field_ix = 1;
}

/* support routine that loads the given variables from the current record */
void LoadRec (char *parpointname, short *pardbreduc, long *partime,
	    long *parrate, short *parsamples, char *partag, double *pardl, double *pardh,
	    double values[], char status[60][4])
{
    int		ix;

    strncpy (parpointname, snaprec.pointname, POINTNAME_SIZE+1);
    parpointname [POINTNAME_SIZE] = 0;
    strncpy (partag, snaprec.egutag, EGUTAG_SIZE);
    if (strcmp (snaprec.red_method, "AVG") == 0)
	*pardbreduc = 1;
    else if (strcmp (snaprec.red_method, "MIN") == 0)
	*pardbreduc = 2;
    else if (strcmp (snaprec.red_method, "MAX") == 0)
	*pardbreduc = 3;
    else
	*pardbreduc = 0;
    *pardl = snaprec.disp_low;
    *pardh = snaprec.disp_high;
    *partime = snaprec.rec_time;
    *parrate = (int)(snaprec.rate);
    *parsamples = snaprec.sample_cnt;
    for (ix = 0; ix < *parsamples; ix++)
    {
	values[ix] = snaprec.value[ix];
	strncpy (status[ix], snaprec.status[ix], sizeof(snaprec.status [ix]));
	status[ix][3] = 0; /* terminate the status string */
    }
}

int GetRec (char *pointname, char *dbreduc, long when)
{
    char *seekpt;
    char *sp;

    /* mark time for no valid data */
    rectime = 0x7fffffff;

    seekpt = malloc (POINTNAME_SIZE+1+3+1); 

    if (seekpt)
    {
	strncpy (seekpt, pointname, POINTNAME_SIZE+1+3+1);
        /* if reduction type is .VAL, strip it */
        if (sp = strchr (seekpt, '.'))
	{
	    if (strcmp (sp, ".VAL") == 0)
	    {
        	*sp = 0;
	    }
	}
	if (oDesc == NULL)
	{
	    snapinit ();
	}
	if (GetSnapRecord (oDesc, seekpt, snaprec.red_method, when, &snaprec))
	{
	    LoadTemp ();
	    if (lastpointtime < rectime + snaprec.rate * snaprec.sample_cnt) /* remember last time visited */
	    {
		lastpointtime = rectime + snaprec.rate * snaprec.sample_cnt;
	    }
	    valuetime = rectime;
	    free (seekpt);
	    return (TRUE);
	}
	else
	{
	    /* mark time for no valid data */
	    valuetime = rectime = 0x7fffffff;
	    free (seekpt);
	    return (FALSE); /* No data in record! */
	}
    }
    return (FALSE);
}

int GetNextRec (char *pointname, char *dbreduc, long when)
{
    if (NextSnapRecord (oDesc, &snaprec))
    {
	LoadTemp ();
	if (lastpointtime < rectime + snaprec.rate * snaprec.sample_cnt) /* remember last time visited */
	{
	    lastpointtime = rectime + snaprec.rate * snaprec.sample_cnt;
	}
	valuetime = rectime;
	return (TRUE);
    }
    else
    {
	/* mark time for no valid data */
	rectime = 0x7fffffff;
	return (FALSE); /* No data in record! */
    }
}

int UpdateRec (char *pointname, char *dbreduc, long rec, int fld, double newval, char *newstat)
{
    char *sp;
    char tempname [POINTNAME_SIZE+1+3+1];

    strncpy (tempname, pointname, POINTNAME_SIZE+1+3+1);
    /* if reduction type is .VAL, strip it */
    if (sp = strchr (tempname, '.'))
    {
	if (strcmp (sp, ".VAL") == 0)
	{
            *sp = 0;
	}
    }

    if (GetSnapRecord (oDesc, tempname, dbreduc, rec, &snaprec))
    {
	snaprec.value [fld] = newval;
	strncpy (snaprec.status[fld], newstat, 3);
	return UpdateSnapRecord (oDesc, &snaprec);
    }
    else
    {
	/* mark time for no valid data */
	rectime = 0x7fffffff;
	return (FALSE); /* No data in record! */
    }
    return TRUE;
}

int UpdateDBRec (char *parpointname, char *pardbreduc, long partime, long parrate,
		 short parsamples, char *partag, double	pardl, double pardh,
		 double parvalue[60], char parstat[60][4])
{
    int		ix;
    int		i;
    char	valstring [12];

    /* if the current DB record is the same time */
    if ( snaprec.rec_time == partime )
    {
	/* if data has different rate (or invalid start time), don't use it */
	if ( parrate == snaprec.rate )
	{
	    for (ix = 0; ix < parsamples; ix++)
	    {
		if (*snaprec.status[ix] == 'C')
		{
		    strncpy (valstring, parstat[ix], 4);
		    for (i = 0; valstring[i] != 0; i++)
		    {
			if (valstring[i] == '-')
			{
				valstring[i] = ' ';
			}
		    }
		    strncpy (snaprec.status[ix], valstring, 4);
		    snaprec.value[ix] = parvalue[ix];
		}
	    }
	    return UpdateSnapRecord (oDesc, &snaprec);
	}
    }
    else /* Adding a new record to the database */
    {
	strncpy (snaprec.pointname, parpointname, sizeof (snaprec.pointname));
	strncpy (snaprec.red_method, pardbreduc, sizeof (snaprec.red_method));
	snaprec.rec_time = partime;
	snaprec.rate = parrate;
	snaprec.sample_cnt = parsamples;
	strncpy (snaprec.egutag, partag, sizeof (snaprec.egutag));
	snaprec.disp_low = pardl;
	snaprec.disp_high = pardh;
	for (ix = 0; ix < parsamples; ix++)
	{
	    snaprec.value[ix] = parvalue[ix];
	    strncpy (snaprec.status[ix], parstat[ix], sizeof (snaprec.status[ix]));
	}
	return UpdateSnapRecord (oDesc, &snaprec);
    }
}

/* reduction file access routines */
void snapinit ()
{
    /* open codebase */
    if (cbInit == 0)
    {
	code4init (&cb);
	cbInit = 1;
#if defined(USECB63) || defined(WIN32)
	cb.errOpen = 0;
	cb.errGo = 0;
	cb.readLock = 0;
#else
	cb.open_error = 0;
	cb.go_error = 0;
	cb.read_lock = 0;
#endif USECB63
	cb.safety = 1;  /* will not create new database */
    }
#ifdef ORACLESNAP
    OpenOracle ();
#else
    OpenXBase ();
#endif ORACLESNAP
    oDesc = MakeSnapDesc ("TISTREND", "TREND_DATA");
}

int opentrendfile ()
{
    /* open codebase if necessary */
    if (cbInit == 0)
    {
	code4init (&cb);
	cbInit = 1;
	cb.errOpen = 0;
	cb.errGo = 0;
	cb.readLock = 0;

	/* open up to data info area */
	TrendMemOpen ();
    }
    cb.safety = 0;  /* will create new database, and overwrite */
    defs = d4open (&cb, "\\tis4000db\\trends");
    if (defs == 0)
    {
        if ((defs = d4create (&cb, "\\tis4000db\\trends", field_info, tag_info)) == 0)
        {
            return DBTRENDERR;
        }
    }

    return DBOK;
}

long trendPosition;

char *FirstTrend ( )
{
    if (defs == 0)
	if (opentrendfile() != DBOK)
	    return 0;
    trendPosition = 0;
    return NextTrend ();
}

char *NextTrend ( )
{
    int ix;

    for ( trendPosition ++; trendPosition <= d4recCount (defs); trendPosition++)
    {
        d4go (defs, trendPosition);
        if (!d4deleted (defs))
            break;
    }
    if (trendPosition <= d4recCount (defs))
    {
        /* Return trend descriptor field */
        field_ptr = d4field (defs, "DESCRIPTOR");
        f4ncpy (field_ptr, tempstring, TRENDDESCRIPTOR_SIZE);
        /* strip trailing blanks */
        tempstring [TRENDDESCRIPTOR_SIZE-1] = 0;
	for (ix = strlen(tempstring)-1; tempstring[ix] == ' '; ix--)
	    tempstring[ix] = 0;
        return (tempstring);
    }
    else
    {
        return (NULL);
    }
}

void RelTrends ( )
{
    d4unlock_all (defs);
}

int closetrends ()
{
    if (defs)
	d4close (defs);
    defs = 0;
    return 0;
}

long GetTrendCount ()
{
    if (defs == 0)
	if (opentrendfile() != DBOK)
	    return 0;
    return d4recCount (defs);
}

/* load current trend set from database, if not found, nothing has been changed */
int GetTrendInfo ( descr, z, autoupdate, trspec )
char *descr;
long *z;
long *autoupdate;
CHARSPEC trspec [];
{
    char    seekdescr [TRENDDESCRIPTOR_SIZE+1];
    int     i, ix;

    if (defs == 0)
	if (opentrendfile() != DBOK)
	    return FALSE;

    strncpy (seekdescr, descr, TRENDDESCRIPTOR_SIZE);
    for (i = strlen(seekdescr); i < TRENDDESCRIPTOR_SIZE; i++)
	seekdescr[i] = ' ';
    seekdescr [TRENDDESCRIPTOR_SIZE] = 0;
    d4tag (defs, "DESCRIPTOR");
    if (d4seek (defs, seekdescr) == 0)
    {
	char *textptr;
        field_ptr = d4field (defs, "ZOOM");
	textptr = f4str (field_ptr);
	if (*textptr == '*')
	{
	    *autoupdate = TRUE;
	    *z = atol (textptr + 1);
	}
	else
	{
	    *autoupdate = FALSE;
            *z = f4int (field_ptr);
	}
        field_ix = d4fieldNumber (defs, "TREND0");
        for (i = 0; i < TREND_NUMPOINTS; i++)
        {
            field_ptr = d4fieldJ (defs, field_ix++);
            f4ncpy (field_ptr, trspec[i], sizeof (CHARSPEC));
 	    for (ix = strlen(trspec[i])-1; trspec[i][ix] == ' '; ix--)
		trspec[i][ix] = 0;
        }
        d4unlock_all (defs);
        return TRUE;
    }
    d4unlock_all (defs);
    return FALSE;
}

/* save trend set in database */
int PutTrendInfo ( descr, z, autoupdate, trspec, flag )
char *descr;
long z;
long autoupdate;
CHARSPEC trspec [];
int flag;
{
    int i, ix;
    int field_num;
    int stat;
    int appending;
    char tempstr [24];

    if (defs == 0)
	if (opentrendfile() != DBOK)
	    return FALSE;

    for (i = strlen(descr); i < TRENDDESCRIPTOR_SIZE; i++)
	descr[i] = ' ';
    descr [TRENDDESCRIPTOR_SIZE] = 0;
    d4tagSelect (defs, d4tag (defs, "DESCRIPTOR"));
    /* if it exists, check for replace and delete if OK */
    if (d4seek (defs, descr) == 0)
    {
        if (flag)
        {
            d4blank (defs);
            d4delete (defs);
            d4flush_all (defs);
        }
        else
        {
	    d4unlock_all (defs);
            return (FALSE);
        }
    }
    /* if the record does not exist now, save and exit */
    if (d4seek (defs, descr))
    {
	stat = d4top (defs);
	if (((stat == 0) && !d4deleted(defs))
	  || (stat == r4eof))
	{
	    /* Couldn't find a blank record */
	    appending = TRUE;
	    d4appendStart (defs, 0);
	}
	else
	{
	    appending = FALSE;
	}
        d4blank (defs);
        field_ptr = d4field (defs, "DESCRIPTOR");
 	for (ix = strlen(descr)-1; descr[ix] == ' '; ix--)
	    descr[ix] = 0;
        f4assign (field_ptr, descr);
        field_ptr = d4field (defs, "ZOOM");
        if (z < 1)
            z = 1;
	if (autoupdate)
	{
	    sprintf (tempstr, "*%d", z);
	    f4assign (field_ptr, tempstr);
	}
	else
	{
            f4assignInt (field_ptr, z);
	}
        field_num = d4fieldNumber (defs, "TREND0");
        for (i = 0; i < TREND_NUMPOINTS; i++)
        {
            field_ptr = d4fieldJ (defs, field_num++);
	    for (ix = strlen(trspec[i])-1; trspec[i][ix] == ' '; ix--)
		trspec[i][ix] = 0;
            f4assign (field_ptr, trspec[i]);
        }
	if (appending)
	{	/* Restore updated record */
	    d4append (defs);
	}
	else
	{
	    d4recall (defs);
	}
        d4flush_all (defs);
#if !(defined(CB51) || defined (CB60))
        d4update_header (defs, TRUE, TRUE);
#endif
        d4unlock_all (defs);
        return TRUE;
    }
    d4unlock_all (defs);
    return FALSE;
}

/* delete trend set in database */
int DelTrendInfo ( descr )
char *descr;
{
    int i, ix;
    char recstring [TRENDDESCRIPTOR_SIZE];
    
    /* find record and delete it */
    for (i = 1; i <= d4recCount (defs); i++)
    {
	    d4go (defs, i);
        /* Delete if we have this descriptor */
        field_ptr = d4field (defs, "DESCRIPTOR");
        f4ncpy (field_ptr, recstring, 80);
        /* strip trailing blanks &*/
        recstring[TRENDDESCRIPTOR_SIZE - 1] = 0;
 	for (ix = strlen(recstring)-1; recstring[ix] == ' '; ix--)
	    recstring[ix] = 0;
        if (strcmp (recstring, descr) == 0)
        {
            d4blank (defs);
            d4delete (defs);
            d4flush_all (defs);
	    break;
        }
    }
    d4unlock_all (defs);
    return 0;
}

long redPosition;

int openreduc ( )
{
    /* open codebase if necessary */
    if (cbInit == 0)
    {
	code4init (&cb);
	cb.errOpen = 0;
	cb.errGo = 0;
	cb.readLock = 0;
    }
    redlist = d4open (&cb, "\\tis4000db\\ar_reduc");
    if (redlist == 0)
    {
    	return DBREDUCERR;
    }
    return DBOK;
}

int closereduc ()
{
    if (redlist)
	d4close (redlist);
    redlist = 0;
    return DBOK;
}

long GetPointCount ()
{
    if (redlist == 0)
	if (openreduc() != DBOK)
	    return 0;
    return d4recCount (redlist);
}

char **GetReducs (cnt)
int *cnt;
{
    char ** pvnames;
    char *cp;

    int	i = 0;

    *cnt = GetPointCount ();
    pvnames = (char **)calloc (*cnt + 1, sizeof(char *));   
    if (cp = FirstPoint ())
    {
        while (cp && i < *cnt)
        {
	    pvnames[i] = (char *)calloc (1, strlen (cp)+1);
	    if (pvnames[i])
	    {
		strcpy (pvnames [i], cp);
		i++;
	    }
            cp = NextPoint ();
        }
    }
    return pvnames;
}

int CheckReduc (point, red)
char *point;
char *red;
{
    char ** pvnames;
    int	 cnt;
    char pointRed [40];

    if (point[0] == '>')
    {
	sprintf (pointRed, "%s|%s", &point[1], red);
    }
    else
    {
	sprintf (pointRed, "%s|%s", point, red);
    }
    
    pvnames = GetReducs(&cnt);

    for (; cnt; cnt--)
    {
	if (pvnames[cnt-1] && strcmp (pvnames[cnt-1], pointRed) == 0)
	{
	    for (; cnt; cnt--)
		free (pvnames[cnt-1]);
	    free (pvnames);
	    return 1;
	}
	free (pvnames[cnt-1]);
    }
    free (pvnames);
    return 0;
}

char *FirstPoint ( )
{
    redPosition = 0;
    return (NextPoint ());
}

char *NextPoint ( )
{
    int ix;

    for ( redPosition ++; redPosition <= d4recCount (redlist); redPosition++)
    {
        d4go (redlist, redPosition);
        if (!d4deleted (redlist))
            break;
    }
    if (redPosition <= d4recCount (redlist))
    {
        /* Return point name and reduction */
        field_ptr = d4field (redlist, "POINTNAME");
        f4ncpy (field_ptr, tempstring, POINTNAME_SIZE);
        /* strip trailing blanks */
        tempstring[POINTNAME_SIZE-1] = 0;
	for (ix = strlen(tempstring)-1; tempstring[ix] == ' '; ix--)
	    tempstring[ix] = 0;
        strcat (tempstring, "|");
        field_ptr = d4field (redlist, "METHOD");
        f4ncpy (field_ptr, &tempstring[strlen(tempstring)], 4);
        return (tempstring);
    }
    else
    {
    	return (NULL);
    }
}

void RelPoints ()
{
    d4unlock_all (redlist);
}
