/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#ifndef ARCHIVELIB_H_
#define ARCHIVELIB_H_

#include <time.h>

/* DB access result codes */
#define DBOK	0
#define DBREDUCERR 1
#define DBSNAPERR 2
#define DBTRENDERR 3
#define DBNOTESERR 4


/* collect a set of ar_snap value/status pairs for the given time units/span,
** reducing data if necessary. Returns number values entered. */
int GetSnapValues (char *pointname, char *dbred, char *reduc,
		   long unitsecs, time_t start, long *dbrate,
		    char *egutag, double *disp_low, double *disp_high,
		   int reqnum, double values[], unsigned char status[][3], double values2[]);

int TrendMemOpen (void);
void TrendMemClose (void);
void PointOpen (char *epn, int dbreduc);
void PointClose (char *epn, int dbreduc);
long MemValue (long valtime, char *pointname, int dbreduc, double *value, char *status);

/* auxiliary functions */
long NextItem ();
long ItemRec ();
char *RecName ();
char *RecTag ();
int RecReduc ();
double RecDispLo ();
double RecDispHi ();
long RecRate ();
int RecSamples ();
long ItemTime ();
double ItemValue ();
char *ItemStatus ();

void LoadRec     (char *parpointname, short *pardbreduc, long *partime,
		 long *parrate, short *parsamples, char *partag, double *pardl, double *pardh,
		 double values[], char status[60][4]);
int GetRec      (char *pointname, char *dbreduc, long when);
int GetNextRec  (char *pointname, char *dbreduc, long when);
int UpdateRec   (char *pointname, char *dbreduc,
		 long rec, int fld, double newval, char *newstat);
int UpdateDBRec (char *parpointname, char *pardbreduc, long partime, long parrate,
		 short parsamples, char *partag, double	pardl, double pardh,
		 double parvalue[60], char parstat[60][4]);

void snapinit (void);
int  opentrendfile ( void );
char *FirstTrend ( void );
char *NextTrend ( void );
void RelTrends ( void );
int closetrends ( void );
int openreduc ( void );
int closereduc ( void );
long GetPointCount ( void );
char **GetReducs ( int * );
int CheckReduc ( char *, char * );
char *FirstPoint ( void );
char *NextPoint ( void );
void RelPoints ( void );
void GetPreference ( long c[], long *b, char *f );
void PutPreference ( long c[], long b, char *f );
long GetTrendCount ( void );
int GetTrendInfo ( char *descr, long *z, long *autoupdate, CHARSPEC trspec[] );
int PutTrendInfo ( char *descr, long z, long autoupdate, CHARSPEC trspec[], int replace );
int DelTrendInfo ( char *descr );
void MakeTSpec ( CHARSPEC specstring, char *epn, int  dbreduc,
                 int  reduc, char *date, char *off, double dl, double dh,
                 double cl, double ch);
void GetTSpec ( CHARSPEC specstring, char *epn, int  *dbreduc,
                int  *reduc, char *date, char *off, long *time,
                double *dl, double *dh, double *cl, double *ch, long z );

int GetRecParams ( char *pointname, int dbred, long when, double *dispLo, double *dispHi ); 
int GetTrendNote ( char *pointname, time_t *when, char **note );
int NextTrendNote ( time_t *when, char **note );

#endif ARCHIVELIB_H_
