/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/* Write the key for each record to stdout */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <syslog.h>
#include "mlist.h"
#include "trendDefs.h"
#include "oracleSnap.h"
#include "oracleSnapLib.h"

	void	stripBlanks ();

SnapRec	snaprec;

/*----------*/
main (int argc, char *argv[])
{
    struct tm	*tmblock;
    int		dotprint;
    char	pointName [50], method [40];
    char	valuebuf [100];
    char	prevvaluebuf [100];
    double      interval;
    OracleDesc	*od;
    SnapRec	snaprec;
    int		cnt;
    int		dumpvalues = 1;

	if (argc < 3)
	{
	    printf ("Usage: dumpOracleSnap [-h] <pointName> <reductionMethod> [<reductionRate>]\n");
	    exit (-1);
	}

	/* Get point name, method, and interval from command line */
	if (*argv[1] == '-')
	{
	    if (*(argv[1]+1) == 'h')
		dumpvalues = 0;
	    strcpy (pointName, argv [2]);
	    strcpy (method, argv [3]);
	    if (argc == 5)
		interval = atoi (argv [4]);
	    else
		interval = 0;
	}
	else
	{
	    strcpy (pointName, argv [1]);
	    strcpy (method, argv [2]);
	    if (argc == 4)
		interval = atoi (argv [3]);
	    else
		interval = 0;
	}
	for (cnt=0 ; cnt<strlen (method) ; cnt++)
	    method [cnt] = toupper (method [cnt]);

	OpenOracle ();
	od = MakeSnapDesc ("TISTREND", "TREND_DATA");

	if (GetSnapRecord (od, pointName, method, 0, &snaprec))
	{
	    do
	    {
		/* See if this is a record we want */
		if ((interval == 0) || (snaprec.rate == interval))
		{
#ifdef ITEMS
		    /* Get the date and time for the record */
		    tmblock = gmtime ((time_t *)&snaprec.rec_time);

		    /* Dump out all the buckets */
		    for (cnt=0 ; cnt < snaprec.sample_cnt ; cnt++)
		    {
			printf ("%2.2d/%2.2d/%4.4d %2.2d:%2.2d:%2.2d %-2d %15.7f %-*s\n",
				tmblock->tm_mon+1, tmblock->tm_mday, tmblock->tm_year+1900,
				tmblock->tm_hour, tmblock->tm_min, tmblock->tm_sec,
				cnt + 1,
				snaprec.value[cnt],
				3, snaprec.status[cnt]);
		    }
#else
		    tmblock = gmtime ((time_t *)&snaprec.rec_time);
		    /* Dump out the record */
		    printf ("%28.28s %s %2.2d/%2.2d/%4.4d %2.2d:%2.2d:%2.2d %-2d samples @ %d seconds",
 				snaprec.pointname, method,
				tmblock->tm_mon+1, tmblock->tm_mday, tmblock->tm_year+1900,
				tmblock->tm_hour, tmblock->tm_min, tmblock->tm_sec,
				snaprec.sample_cnt, snaprec.rate);
		    if (dumpvalues)
		    {
			dotprint = 0;
			prevvaluebuf[0] = 0;
			for (cnt=0 ; cnt < snaprec.sample_cnt ; cnt++)
			{
			    if ((cnt % 4) == 0)
			    {
				if ((cnt >= 4) && (strcmp (&prevvaluebuf[4], &valuebuf[4]) != 0))
				{
				    printf (valuebuf);
				    strcpy (prevvaluebuf, valuebuf);
				    dotprint = 1;
				}
				else
				{
				    if (dotprint)
				    {
					printf ("\n  ...");
					dotprint = 0;
				    }
				}
				sprintf (valuebuf, "\n%2.2d", cnt);
			    }
			    sprintf (&valuebuf[strlen(valuebuf)], " %15.7f %-*s",
				snaprec.value[cnt],
				3, snaprec.status[cnt]);
			}
			if (strlen (valuebuf) > 4)
			    printf (valuebuf);
		    }
		    printf ("\n");
#endif ITEMS
		}
	    } while (NextSnapRecord (od, &snaprec));
	}
	else
	{
	    printf ("point %s not found in file.\n", pointName);
	}
	CloseOracle ();
}

/*----------*/
void stripBlanks (buf)	/* Strip blanks from a string */

    char	*buf;

    {

	while (*buf) {
	    if (*buf == ' ') {
		*buf = '\0';
		return;
	    }
	    buf++;
	}
	
    }

