/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <malloc.h>
#include "mlist.h"

/* p is a pointer to an array of MListLinks */
#define LINKS(p) ((MListLinks *)(((char *)p) + (list->listNumber*sizeof(void *))))

#ifdef MLIST_DEBUG

typedef struct _test {
    MListLinks  links [3];
    char        value [50];
} ListNode;

void    printListForward (MList *);
void    printListBackward (MList *);

/*----------*/
main (int argc, char *argv[])

    {

        MList           oddList, evenList, allList;
        ListNode        *p;

        MListInit (&oddList, 0);
        MListInit (&evenList, 1);
        MListInit (&allList, 2);

        p = malloc (sizeof (ListNode));
        strcpy (p->value, "node 1");
        MListInsertRear (&oddList, (void *)p);
        MListInsertRear (&allList, (void *)p);

        p = malloc (sizeof (ListNode));
        strcpy (p->value, "node 2");
        MListInsertRear (&evenList, (void *)p);
        MListInsertFront (&allList, (void *)p);

        p = malloc (sizeof (ListNode));
        strcpy (p->value, "node 3");
        MListInsertRear (&oddList, (void *)p);
        MListInsertRear (&allList, (void *)p);

        p = malloc (sizeof (ListNode));
        strcpy (p->value, "node 4");
        MListInsertRear (&evenList, (void *)p);
        MListInsertFront (&allList, (void *)p);

        p = malloc (sizeof (ListNode));
        strcpy (p->value, "node 5");
        MListInsertRear (&oddList, (void *)p);
        MListInsertRear (&allList, (void *)p);

        p = malloc (sizeof (ListNode));
        strcpy (p->value, "node 6");
        MListInsertRear (&evenList, (void *)p);
        MListInsertFront (&allList, (void *)p);

        printListForward (&allList);
        printListForward (&oddList);
        printListForward (&evenList);

        printListBackward (&allList);
        printListBackward (&oddList);
        printListBackward (&evenList);

        printf ("Deleting front of allList\n");

        MListRemove (&allList, allList.front);
        printListForward (&allList);
        printListBackward (&allList);

        printf ("Deleting rear of allList\n");

        MListRemove (&allList, allList.rear);
        printListForward (&allList);
        printListBackward (&allList);

        printf ("Deleting second from left of allList\n");

        MListStartFront (&allList);
        p = MListNext (&allList);
        p = MListNext (&allList);
        MListRemove (&allList, p);
        printListForward (&allList);
        printListBackward (&allList);

        printf ("Deleting middle of allList\n");

        MListStartFront (&allList);
        p = MListNext (&allList);
        p = MListNext (&allList);
        MListRemove (&allList, p);
        printListForward (&allList);
        printListBackward (&allList);

        printf ("Deleting front of allList\n");

        MListRemove (&allList, allList.front);
        printListForward (&allList);
        printListBackward (&allList);

        printf ("Deleting front of allList\n");

        MListRemove (&allList, allList.front);
        printListForward (&allList);
        printListBackward (&allList);

    }

/*----------*/
void printListForward (MList *list)

    {

        ListNode        *p;

        MListStartFront (list);
        while (p = (ListNode *) MListNext (list)) {
            printf ("%s, ", p->value);
        }
        printf ("\n");

    }

/*----------*/
void printListBackward (MList *list)

    {

        ListNode        *p;

        MListStartRear (list);
        while (p = (ListNode *) MListNext (list)) {
            printf ("%s, ", p->value);
        }
        printf ("\n");

    }

#endif MLIST_DEBUG

/*----------*/
void MListInit (MList *list, int listNumber)

    {

        list->front = NULL;
        list->rear = NULL;
        list->listNumber = listNumber;
        list->size = 0;
        list->current = NULL;

    }

/*---------- */
void MListInsertFront (MList *list, void *node)

    {

        if (list->size == 0) {
            LINKS(node)->next = NULL;
            LINKS(node)->prev = NULL;
            list->rear = node;
        } else {
            LINKS(node)->prev = NULL;
            LINKS(node)->next = list->front;
            LINKS(list->front)->prev = node;
        }
	list->front = node;
        list->size++;

    }

/*---------- */
void MListInsertRear (MList *list, void *node)

    {

        if (list->size == 0) {
            LINKS(node)->next = NULL;
            LINKS(node)->prev = NULL;
            list->front = node;
        } else {
            LINKS(node)->next = NULL;
            LINKS(node)->prev = list->rear;
            LINKS(list->rear)->next = node;
        }
	list->rear = node;
        list->size++;

    }

/*----------*/
void MListRemove (MList *list, void *node)

    {

        if (list->size == 1) {
            list->front = NULL;
            list->rear = NULL;
        } else if (node == list->front) {       /* Removing front node */
            LINKS(LINKS(list->front)->next)->prev = NULL;
            list->front = LINKS(list->front)->next;
        } else if (node == list->rear) {        /* Removing rear node */
            LINKS(LINKS(list->rear)->prev)->next = NULL;
            list->rear = LINKS(list->rear)->prev;
        } else {
            LINKS(LINKS(node)->prev)->next = LINKS(node)->next;
            LINKS(LINKS(node)->next)->prev = LINKS(node)->prev;
        }
        list->size--;

    }

/*----------*/
void *MListRemoveFront (MList *list)

    {

	void	*node;

	node = list->front;
	MListRemove (list, node);
	return node;

    }

/*----------*/
void *MListRemoveRear (MList *list)

    {

	void	*node;

	node = list->rear;
	MListRemove (list, node);
	return node;

    }

/*----------*/
void MListStartFront (MList *list)   /* Set the iterator to the front */

    {

        list->current = list->front;
	list->reverse = 0;

    }

/*----------*/
void MListStartRear (MList *list)   /* Set the iterator to the end */

    {

        list->current = list->rear;
	list->reverse = 1;

    }

/*----------*/
void *MListNext (MList *list)     /* Return the current node and move */
                                  /* the interator to the next node. */

    {

        void    *p;

        p = list->current;
        if (p != NULL)
            list->current = list->reverse ? LINKS(list->current)->prev :
				LINKS(list->current)->next;
        return p;

    }

/*----------*/
void *MListPrev (MList *list)     /* Return the current node and move */
                                  /* the interator to the previous node. */

    {

        void    *p;

        p = list->current;
        if (p != NULL)
            list->current = list->reverse ? LINKS(list->current)->next :
				LINKS(list->current)->prev;
        return p;

    }

/*----------*/
int MListSize (MList *list)	/* Return number of items on list */

    {

	return list->size;

    }

/*----------*/
int MListEmpty (MList *list)	/* Return 1 if list is empty */

    {

	return list->front == NULL;

    }

/*----------*/
void *MListGetRear (MList *list)	/* Return pointer to rear item */

    {

	return list->rear;

    }

/*----------*/
void *MListGetFront (MList *list)	/* Return pointer to front item */

    {

	return list->front;

    }


