/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/* Write all records to an oracle table */


#include <stdio.h>
#include <stdlib.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "mlist.h"
#include "oracleSnap.h"
#include "oracleSnapLib.h"
#include "msgMon.h"

#define C_EPOCHSTART	2440588L	/* dif from db4 date to 01/01/1970 */
#define C_SECSPERDAY 86400L	/* number of seconds per day */

C4CODE  cb;
D4DATA	*arSnapDesc;
F4FIELD	*pointNameFld, *eguTagFld, *dispLowFld, *dispHighFld, *yearFld, *dayOfYearFld, *hourFld;
F4FIELD	*offsetMinFld, *sampleCntFld, *redMethodFld, *rateFld, *valueFld [60], *statusFld [60];  

	void	stripBlanks ();
static	void	openSnap ();
static	void	closeSnap ();

extern	C4CODE	cb;
OracleDesc *od;
OracleDesc *catDesc;
SnapRec	snaprec;
char	catTable [40];
char	catTablespace [40];
int	targetSize;
char	*tablepath;

/*----------*/
main (int argc, char *argv[])
{
    int		stat;
    char	key [50];
    char	tempstring [80];
    int		i;
    int		numrecs;
    int		rateIsSeconds = 0;
    time_t	earliest_time;
    time_t	cat_time;
    time_t	latest_time;
    time_t	startDate;
    time_t	endDate;
    int		online;
    struct tm	*tmblock;

	tisOpenLog ("oracelSnap", 6, 2, NULL, NULL);

	if ((argc != 2) && (argc != 4))
	{
	    printf ("Usage: oracleSnap <fileName> [<new_table_path> <new_table_size>]");
	    printf ("  Imports X-base data into oracle tables.\n");
	    printf ("    source fileName (an X-base file)\n");
	    printf ("      '-' for <fileName> means /tis4000db/ar_snap\n");
	    printf ("    path to create tables if needed\n");
	    printf ("    size for tables if needed\n");
	    exit (-1);
	}

	d4init (&cb);
#ifdef USECB63
	cb.errOpen = 0;
	cb.errCreate = 0;
#else
	cb.open_error = 0;
	cb.create_error = 0;
#endif USECB63

#ifdef WIN32
	openSnap ((strcmp (argv [1], "-") == 0) ? "\\tis4000db\\ar_snap" : argv [1]);
#else
	openSnap ((strcmp (argv [1], "-") == 0) ? "/tis4000db/ar_snap" : argv [1]);
#endif WIN32

	if (argc == 4)
	{
	    tablepath = argv[2];
	    targetSize = atoi(argv[3]);
	}

	/* determine time span of data */
        d4tag_select (arSnapDesc, d4tag (arSnapDesc, "METH_RATE"));
	if (d4top (arSnapDesc) == 0)
	{
	    f4ncpy (yearFld, tempstring, 80);
	    strcpy (&tempstring [4], "0101");
	    earliest_time = a4long (tempstring);
	    earliest_time += f4int (dayOfYearFld);
	    earliest_time = (earliest_time - C_EPOCHSTART) * C_SECSPERDAY;
	}

	if (d4bottom (arSnapDesc) == 0)
	{
	    f4ncpy (yearFld, tempstring, 80);
	    strcpy (&tempstring [4], "0101");
	    latest_time = a4long (tempstring);
	    latest_time += f4int (dayOfYearFld) + 1;
	    latest_time = (latest_time - C_EPOCHSTART) * C_SECSPERDAY;
	}
	OpenOracle ();
	od = MakeSnapDesc ("TISTREND", "TREND_DATA");
	if (od == NULL)
	{
	    tisSysLog (0,0,"Can't set up snap descriptor\n");
	    exit (-2);
	}
	OpenSnapTable ( od, NULL );
	/* open catalog, create it if necessary */
	if (catOpen (1, "oracleSnap") == 0)
	{
	    tisSysLog (0,0,"Can't open catalog file!\n");
	    exit (-3);
	}
	cat_time = catGetEarliestStartDate ();
	/* Seek to the first record after blanks */
	sprintf (key, "! ");
	if ((stat = d4seek (arSnapDesc, key)) != r4after)
	{
	    tisSysLog (0,0,"No points in database\n");
	    closeSnap (arSnapDesc);
	    exit (-1);
	}

	rateIsSeconds = (f4len(rateFld) != 2);
 	numrecs = 0;
	while (!d4eof (arSnapDesc))
	{
	    numrecs++;
	    if ((numrecs % 100) == 0)
		printf (".");

	    /* Read in a record */
	    strcpy (snaprec.pointname, f4str (pointNameFld));
	    snaprec.pointname[POINTNAME_SIZE] = 0;
	    stripBlanks (snaprec.pointname);
	    strcpy (snaprec.egutag, f4str (eguTagFld));
	    snaprec.egutag[EGUTAG_SIZE] = 0;
	    stripBlanks (snaprec.egutag);
	    snaprec.disp_low = f4double (dispLowFld);
	    snaprec.disp_high = f4double (dispHighFld);
	    f4ncpy (yearFld, tempstring, 80);
	    strcpy (&tempstring [4], "0101");
	    snaprec.rec_time = a4long (tempstring);
	    snaprec.rec_time += f4int (dayOfYearFld);
	    snaprec.rec_time = (snaprec.rec_time - C_EPOCHSTART) * C_SECSPERDAY;
	    snaprec.rec_time += f4int (hourFld) * 3600L;
	    snaprec.rec_time += f4int (offsetMinFld) * 60L;
	    snaprec.sample_cnt = f4int (sampleCntFld);
	    strcpy (snaprec.red_method, f4str (redMethodFld));
	    snaprec.egutag[METHOD_SIZE] = 0;
	    stripBlanks (snaprec.egutag);
	    if (!rateIsSeconds)
		snaprec.rate = (int)(atof (f4str (rateFld)) * 60.);
	    else
		snaprec.rate = atof (f4str (rateFld)); /* need to handle both rate spec formats! */
	    for (i = 0; i < 60; i++)
	    {
		snaprec.value[i] = atof (f4str (valueFld [i]));
		strcpy (snaprec.status[i], f4str (statusFld [i]));
	    }
	    /* Write record to oracle table */
	    /* determine appropriate table from catalog */
	    /* If no archive catalog yet, and archive values are provided,
	    ** create the first catalog entry. */
	    if ((snaprec.rec_time < cat_time)
	      || ((cat_time == 0) && (argc == 4)))
	    {
		if (argc != 4)
		{
		    tisSysLog (0,0,"Data requires a new table,");
		    tisSysLog (0,0,"please enter table path and size parameters on command line.\n");
		    exit (-5);
		}
		/* make up a new tablename */
		/* create name from start time (eldest value in trend_data, or end of catalog file data) */
		tmblock = gmtime(&earliest_time);
		sprintf (catTable, "ARCHIVE%3.3d%4.4d",
			     tmblock->tm_yday+1,
			     tmblock->tm_year+1900);
		/* create new table for this data */
		sprintf (catTablespace, "%s_SPACE", catTable);
		catDesc = MakeSnapDesc (catTablespace, catTable);
		if (catDesc == NULL)
		{
		    tisSysLog (0,0,"Can't set up catalog table descriptor\n");
		    exit (-2);
		}
		if (CreateArchTable (catDesc, targetSize, tablepath))
		{
		    /* establish catalog entry */
		    catAddEntry (earliest_time, cat_time, catDesc->otable, NULL, tablepath, 1);
		    cat_time = catGetEarliestStartDate ();
		    UpdateSnapRecord (catDesc, &snaprec);
		}
		else
		{
		    tisSysLog (0,0,"Cannot create new table, exiting.\n");
		    exit (-6);
		}
	    }	
	    if (catGetEntry (snaprec.rec_time, &startDate, &endDate, catTable, NULL, NULL, &online))
	    {
		if (online)
		{
		    if ((catDesc == 0) || (strcmp (catDesc->otable, catTable) != 0))
		    {
			if (catDesc)
			    free (catDesc);
			sprintf (catTablespace, "%s_SPACE", catTable);
			catDesc = MakeSnapDesc (catTablespace, catTable);
			if (catDesc == NULL)
			{
			    tisSysLog (0,0,"Can't set up catalog table descriptor\n");
			    exit (-2);
			}
		    }
		    UpdateSnapRecord (catDesc, &snaprec);
		}
		else
		{
		    tisSysLog (0,0,"Record not added, appropriate table is offline.\n");
		}
	    }
	    else
	    {
	 	/* if later than any catalog entry, use Trend_data */
		if (catGetLatestEndDate () <= snaprec.rec_time)
		    UpdateSnapRecord (od, &snaprec);
	    }
	    stat = d4skip (arSnapDesc, 1L);
	}
	closeSnap ();
	CloseOracle ();
	tisSysLog (0,0, "tisSysLognOracleSnap completed, processed %d records\n", numrecs);
	return 1;
}

/*----------*/
static void openSnap (fileName)
char	*fileName;
{
    int		i;
    char	buf [20];
    T4TAG	*t;

	/* Open the ar_snap database.  If it does not exist, quit */
	arSnapDesc = d4open (&cb, fileName);
	if (arSnapDesc == NULL)
	{
	    tisSysLog (0,0,"(%d): Unable to open %s.\n", getpid (), fileName);
	    exit (1);
	}
	t = d4tag (arSnapDesc, "METHOD_REC");
	d4tag_select (arSnapDesc, t);
	d4top (arSnapDesc);
	d4unlock_all (arSnapDesc);

	/* Open up all the fields */
	pointNameFld = d4field (arSnapDesc, "POINTNAME");
	eguTagFld = d4field (arSnapDesc, "EGUTAG");
	dispLowFld = d4field (arSnapDesc, "DISP_LOW");
	dispHighFld = d4field (arSnapDesc, "DISP_HIGH");
	yearFld = d4field (arSnapDesc, "YEAR");
	dayOfYearFld = d4field (arSnapDesc, "DAYOFYEAR");
	hourFld = d4field (arSnapDesc, "HOUR");
	offsetMinFld = d4field (arSnapDesc, "OFFSET_MIN");
	sampleCntFld = d4field (arSnapDesc, "SAMPLE_CNT");
	redMethodFld = d4field (arSnapDesc, "RED_METHOD");
	rateFld = d4field (arSnapDesc, "RATE");
	for (i=0 ; i < 60 ; i++)
	{
	    sprintf (buf, "VALUE%d", i + 1); 
	    valueFld [i] = d4field (arSnapDesc, buf);
	    sprintf (buf, "STATUS%d", i + 1); 
	    statusFld [i] = d4field (arSnapDesc, buf);
	}
}

/*----------*/
static void closeSnap ()
{
	d4close (arSnapDesc);
}

/*----------*/
void stripBlanks (buf)	/* Strip blanks from a string */
char	*buf;
{
	while (*buf)
	{
	    if (*buf == ' ')
	    {
		*buf = '\0';
		return;
	    }
	    buf++;
	}
}
