/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	COMMON INTIALIZATION FOR CHANNEL ACCESS SERVERS		*/
/*								*/
/*	Many of the things that we do for the three servers	*/
/*	that make up the agent architecture for channel access	*/
/*	are common. This routine avoids code duplication.	*/
/*								*/
/*	Author:	Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <fcntl.h>
#ifdef WIN32
#include <direct.h>
#include <process.h>
#include <stdio.h>
#include <io.h>
#include <sys/locking.h>
#else
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/file.h>
#include <signal.h>
#endif WIN32
#include <sys/types.h>
#include <string.h>
#include "msgMon.h"


void serverInit(char *appName, char *dirPath, char *lockFile)
{
    int		fd;
    char	buff[10];

/******
	Change our controlling TTY so that we don't pick up any
	stray signals.
******/

#if !(defined(AXP) || defined(SOLARIS) || defined(WIN32))
    fd = open("/dev/tty", O_RDWR);
    ioctl(fd, TIOCNOTTY, 0);
    close(fd);
#endif

/******
	Move to the defined directory so that the core file
	can be found in case we die an unnatural death.
******/

#ifdef WIN32
    if (_chdir(dirPath) != 0) {
	tisSysLog (0, 2, "%s: Unable to find trend lock directory (%s)",
			appName,  dirPath);
	if (_mkdir (dirPath) == 0) {
	    tisSysLog (0, 2, "%s: Lock directory %s created.",
			appName, dirPath);
	} else {
	    tisSysLog (0, 2, "%s: Lock directory %s could not be created.",
			appName, dirPath);
	    exit (-1);
	}
#else
    if (chdir(dirPath) != 0) {
	tisSysLog (0, 2, "%s: Unable to find trend lock directory (%s)",
			appName, dirPath);
	if (mkdir (dirPath, 0775) == 0) {
	    tisSysLog (0, 2, "%s: Lock directory %s created.",
			appName, dirPath);
	} else {
	    tisSysLog (0, 2, "%s: Lock directory %s could not be created.",
			appName, dirPath);
	    exit (-1);
	}
#endif WIN32
    }

/******
	Make a process group just for ourselves
******/

#ifdef SOLARIS
    setpgrp();
#else
#ifdef AXP
    setpgid (0,0);
#else
#ifndef WIN32
    setpgrp(getpid(), 0);
#endif WIN32
#endif AXP
#endif SOLARIS

/******
	Insure that we are the only copy of the server running
******/

#ifdef WIN32
    fd = _open(lockFile, O_RDWR | O_CREAT, 0640);
    fd = _open(lockFile, _O_RDWR | _O_CREAT, 0640);
    if (fd < 0)
	exit(1);
    if (_locking (fd, _LK_LOCK, 0) == -1)
	exit (0);

/******
	Record our PID in the lock file to make it easy to find
******/

    sprintf(buff, "%d\n", _getpid());
    _write(fd, buff, strlen(buff));
    _close(fd);		/* I think that we keep the lock even if
			   the file is closed. */
#else
    fd = open(lockFile, O_RDWR | O_CREAT, 0640);
    if (fd < 0) {
	tisSysLog (0, 2, "%s: Unable to access lock file in %s\n",
		appName,  dirPath); 
	exit(-1);
    }
#ifdef SOLARIS
    if (lockf (fd, F_TLOCK, 0) == -1) {
	tisSysLog (0, 2, "%s: Already running.\n", appName);
	exit (-1);
    }
#else
    if (flock(fd, LOCK_EX | LOCK_NB)) {
	tisSysLog (0, 2, "%s: Already running.\n", appName);
	exit(-1);
    }
#endif SOLARIS

/******
	Record our PID in the lock file to make it easy to find
******/

    sprintf(buff, "%d\n", getpid());
    write(fd, buff, strlen(buff));
#endif WIN32
}

