/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#ifndef _TRENDDEFS_H_
#define _TRENDDEFS_H_

#include "mlist.h"
#include "trendLib.h"
#include "trendSizes.h"

/* General purpose things */

#ifndef FALSE
#define FALSE  0
#define TRUE   1
#endif

/* Key for shared memory segment */

#define	TREND_SH_MEM_KEY	0x33333333

/* Various fields */

typedef char      PointName [POINTNAME_SIZE+1];

typedef char      Resource [RESNAME_SIZE+1];

typedef char      EguTag [EGUTAG_SIZE+1];

typedef char      Status [STATUS_SIZE];

typedef char      PointMethName [POINTNAME_SIZE+1+METHOD_SIZE+1];

/* All files manipulated by trend */

#ifdef WIN32
#define	AR_REDUC_FILE		"\\tis4000db\\ar_reduc.dbf"
#define	ARSNAP_FILE		"\\tis4000db\\ar_snap.dbf"
#define	ARSNAP_IX_FILE		"\\tis4000db\\ar_snap.mdx"
#define	ARCHIVE_INFO_FILE	"\\tis4000db\\archiveInfo"
#define	ARSNAPN_FILE		"\\tis4000db\\ar_snapn.dbf"
#else
#define	AR_REDUC_FILE		"/tis4000db/ar_reduc.dbf"
#define	ARSNAP_FILE		"/tis4000db/ar_snap.dbf"
#define	ARSNAP_IX_FILE		"/tis4000db/ar_snap.mdx"
#define	ARCHIVE_INFO_FILE	"/tis4000db/archiveInfo"
#define	ARSNAPN_FILE		"/tis4000db/ar_snapn.dbf"
#endif WIN32

/* Available reduction methods */

#define	REDUC_MIN	0
#define	REDUC_MAX	1
#define	REDUC_AVG	2
#define	REDUC_SMP	3

#define	NUM_METHODS	4
#define BUCKETS_PER_RECORD      60
#define	SECONDS_PER_DAY	60*60*24

#define	POINTS_PER_PARM_REQUEST	10
#define	PARMS_REQUEST_TIMEOUT	60

#define	RING_BUFFER_ENTRIES	1000

#ifdef DEBUG
#define LOG_OPTIONS     stdout
#define syslog          fprintf
#else
#define	LOG_OPTIONS	LOG_ALERT|LOG_LOCAL1
#endif

/* This is the trend data structure. One for each point/method on trend */

typedef struct {
    MListLinks          links [1];              /* Point list links */
    struct _pointnode   *pointNode;             /* Point node for this trend */
    char                quality [BUCKETS_PER_RECORD]; /* Bucket data quality */
    char		status [BUCKETS_PER_RECORD]; /* Bucket statuses */
    char                severity [BUCKETS_PER_RECORD]; /* Bucket severities */
    double		value [BUCKETS_PER_RECORD];  /* Bucket values */
    int			count [BUCKETS_PER_RECORD];  /* Number of samples accounted for in each bucket */
    int                 method;                 /* reduction method */
    int			interval;	        /* (in seconds) */
    long		startTime;		/* Starting time of current record */
    short		sampleCnt;		/* Number of buckets in use. */
    double		lastValue;		/* Last value received from CA */
    char                lastSeverity;           /* Last severity from CA */
    char		lastStatus;		/* Last status of last bucket written */
    time_t              lastTime;               /* Last sample time */
    int                 resourceUp;             /* Current status of resource */
    int                 active;                 /* Trending is still active for this trend */
    CurrentData         *current;               /* Up-to-date trend data */
} Trend, *TrendPtr;

/* This is the tree node for the AVL index tree. one for each point on trend */

typedef struct _pointnode {
    MListLinks          links [2];              /* Point and Resource list links */
    PointName		pointName;
    int         	reductionMethod;
    Resource            resName;                /* Resource name */
    EguTag		eguTag;	                /* Engineering Units */
    double		dispLow;		/* Lower display limit */
    double		dispHigh;		/* High display limit */
    int			parmsTimeout;	        /* Time at which request has taken too long */
    int			gotParms;		/* Got the parameters */
    int			connected;		/* We have a connection to the point */
    TrendPtr            trend [NUM_METHODS];    /* One for each reduction method */
} PointNode, *PointNodePtr;

/* This is a list of resources for the points on trend. */

typedef struct {
    MListLinks          links [1];              /* For linking resources */
    Resource            resName;                /* Name of the resource */
    MList               pointList;              /* Points belonging to this resource */
} ResListNode, *ResListNodePtr; 

#endif
