/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <stdio.h>
#include <string.h>
#include <memory.h>
#include <sys/types.h>
#ifdef WIN32
#include <time.h>
#include <process.h>
#include "NTUnixSubs.h"
#else
#include <sys/time.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif WIN32
#include <sys/stat.h>
#include "msgMon.h"
#include <error.h>
#include "trendDefs.h"

static  int       upToDateId;      /* Shared memory id */
static  UpToDateInfo *upToDateInfo;
static  int       mxSem;
extern  int       errno;

/* Functions defined in this module */

        int     trendInit (void);
        void    trendTerminate (void);
        int     trendInsert (char *, int);
        int     trendRemove (char *, int);
        int     trendGet (char *, int, CurrentData *);
static  char    *strError (int);

/*----------*/
int trendInit (void)  /* Initialize access to up-to-date trend data */

    {

	/* Open shared memory segment for the up-to-date data */

	if ((upToDateId = shmget (TR_MEM_KEY, sizeof (UpToDateInfo), 0666)) == -1) {
#ifdef WIN32
	    tisSysLog (0, 2, "trendLib: Unable to open shared memory for up-to-date data -- %s\n",
			strError (errno));
#else
	    tisSysLog (0, 2, "trendLib: Unable to open shared memory for up-to-date data -- %s\n",
			strError (errno));
#endif WIN32
	    return (FALSE);
	}
	upToDateInfo = (UpToDateInfo *) shmat (upToDateId, (char *)0, 0666);
	if (upToDateInfo == (UpToDateInfo *)-1) {
#ifdef WIN32
	    tisSysLog (0, 2, "trendLib: Unable to shmat shared memory for database server-- $s",
			strError (errno));
#else
	    tisSysLog (0, 2, "trendLib: Unable to shmat shared memory for database server-- $s",
			strError (errno));
#endif WIN32
	    return (FALSE);
	}

	/* Open the MX semaphore for access to shared memory */

	mxSem = sem_open (TR_SEM_KEY);
	return(TRUE);
    }

/*----------*/
void trendTerminate (void)

    {

	/* Close the mx semaphore */

	sem_close (mxSem);
 
    }

/*----------*/
int trendInsert (char *pointName, int method)

{

        int    i;

	if (upToDateInfo->numEntries == NUM_TRENDS) {
#ifdef WIN32
	    tisSysLog (0, 2, "trendLib: Too many trends for up-to-date trending.\n");
#else
	    tisSysLog (0, 2, "trendLib: Too many trends for up-to-date trending.\n");
#endif WIN32
	    return FALSE;
	}

        sem_wait (mxSem);

	for (i=0 ; i<upToDateInfo->numEntries ; i++)
            if ((strcmp (pointName, upToDateInfo->data [i].pointName) == 0) &&
		(upToDateInfo->data [i].method == method))
	        break;

	if (i<upToDateInfo->numEntries) {
	    upToDateInfo->data [i].useCount++;
	} else {
	    strcpy (upToDateInfo->data [i].pointName, pointName);
	    upToDateInfo->data [i].method = method;
	    upToDateInfo->data [i].valid = FALSE;
	    upToDateInfo->data [i].useCount = 1;
	    upToDateInfo->data [i].p = NULL;
	    upToDateInfo->numEntries++;
	}

	upToDateInfo->changed = TRUE;

	sem_signal (mxSem);

    }

/*----------*/
int trendRemove (char *pointName, int method)

    {

        int    i;

        sem_wait (mxSem);

	for (i=0 ; i<upToDateInfo->numEntries ; i++)
            if ((strcmp (pointName, upToDateInfo->data [i].pointName) == 0) &&
		(upToDateInfo->data [i].method == method))
	        break;

	if (i == upToDateInfo->numEntries) { /* Not found */
	    sem_signal (mxSem);
	    return FALSE;
	}

	if (upToDateInfo->data [i].useCount > 1) {
	    upToDateInfo->data [i].useCount--;
	} else {
	    while (i<upToDateInfo->numEntries) {
	        upToDateInfo->data [i] = upToDateInfo->data [i+1];
	        i++;
	    }
	    upToDateInfo->numEntries--;
	}

	upToDateInfo->changed = TRUE;

	sem_signal (mxSem);

	return TRUE;

    }

/*----------*/
int trendGet (char *pointName, int method, CurrentData *p)

    {

        int   i, ix;

        sem_wait (mxSem);

	for (i=0 ; i<upToDateInfo->numEntries ; i++)
            if ((strcmp (pointName, upToDateInfo->data [i].pointName) == 0) &&
		(upToDateInfo->data [i].method == method))
	        break;

	if (i == upToDateInfo->numEntries) { /* Not found */
	    sem_signal (mxSem);
	    return FALSE;
	}

	/* Check if valid data is in the record */

	if (!upToDateInfo->data [i].valid) {
	    sem_signal (mxSem);
	    return FALSE;
	}

	*p = upToDateInfo->data [i];

	/* Fill to the present time */

	i = p->sampleCnt;
	ix = (time (0) - p->startTime) / p->interval;
	while (i<=ix) {
	    p->value [i] = p->lastValue;
	    p->quality [i] = 'F';
	    p->severity [i] = p->lastSeverity;
	    p->status [i] = p->lastStatus;
	    if(++i >= 60)
	      {
		ix = 59;
		break;
	      }
	}
	p->sampleCnt = ix + 1;

	sem_signal (mxSem);

	return TRUE;

    }

/*----------*/
char *strError (int errorNum)

    {

#if defined(SOLARIS) || defined (AXP)
	return strerror (errorNum);
#else
	extern char *sys_errlist [];
	return sys_errlist [errorNum];
#endif

    }
