/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#ifdef WIN32
#include <process.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "NTUnixSubs.h"
#else
#include <sys/time.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <error.h>
#endif WIN32
#include <time.h>
#include "cadef.h"
#include "dbDefs.h"
#include "db_access.h"
#include "tsDefs.h"
#include "paramStructs.h"
#include "msgMon.h"
#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "trendDefs.h"
#include "utils.h"
#include "avl.h"
#include "arSnapRecord.h"
#include "dbase.h"
#include "trendProcess.h"
#include "dbRequests.h"
#include "rb.h"

/* Functions defined in this module */

static  void           terminateProcessor (int);
static  void           hupHandler (int);
static  void           incrementDebugLevel (int);
static  void           decrementDebugLevel (int);
static  void           allocateSharedMemory (void);
static  void           removeSharedMemory (void);
static  int            treeCompareFunc (PointNodePtr, PointNodePtr);
static  void           processTrendEvents (void);
static  void           reconnectCallback (void *, int, int);
static  void           processDataEvent (char *, short, short, void *);
static  void           processResourceStatusChange (char *, short);
static  int            buildTrends (void);
static  void           updateTrends (void);
static  void           tickleTENB (char *);
static  void           tickleHandler (struct connection_handler_args);
static  void           loadInitialRecords (void);
        void           initializeTrend (TrendPtr, time_t);
static  void           forceParameterRequest (void);
static  void           requestParameters (void);
static  void           processParameters (short, void *, void *);
static  void           processTrends (char *, time_t, double, short, short);
static  void           writeTrends (int);
        PointNodePtr   findPoint (char *);
static  ResListNodePtr findResource (char *);
static  ResListNodePtr getResource (char *);
        void           resetUpToDatePointers (void);
static  void           printTrendsByTrendList (void);
        void	       resetUpToDatePointers (void);
        void	       updateOnly (void);

/* external data */
#ifdef TRENDUPDATES
extern int GetUpdates (PointNodePtr   (findPoint) (char *));
#endif TRENDUPDATES

/* external data/funcs */
extern void serverInit(char *appName, char *dirPath, char *lockFile);
extern	int	errno;

/* Global data */

int     myPid;
int     currentMin;
int	nextSecond;
int	running = TRUE;
int	debugLevel = 0;
int	doConnect = TRUE;
int     numTrends;

static	time_t    reducFileTime;   /* Time and date of currently loaded ar_reduc.dbf file */
static  MList     resourceList;    /* List of resources */
static  MList     pointList;       /* List of all points */
        MList     trendList;       /* List of all trends */
static  AVL_TREE  *tree;           /* Point tree */
        RingBuf   *requestQueue;   /* Database write request buffer */
static  int       eventsProcessed; /* Number of events processed in the last minute */
static  int       upToDateId;      /* Shared memory id */
        UpToDateInfo *upToDateInfo;
        int       mxSem;
/*
  kludge
  */
char debugName[40];
int debugInterval,flag1,flag2,flag3,flag4,eflag;
/*----------*/
main (int argc, char *argv[])

    {

	int	c;
	extern	char	*optarg;
	static WriteRequest    request;

	tisOpenLog ("trendProcessor", 6, 2, NULL, NULL);

#ifdef WIN32
	myPid = _getpid ();
#else
	myPid = getpid ();
#endif WIN32
	debugName[0] = 0; /* no debug info to start */
	flag1=flag2=flag3=flag4=eflag=0;
	/* Process command line options */

	while ((c = getopt (argc, argv, "l:dvs")) != -1)
	    switch (c) {
		case 'l':	/* Explicitly set debug level */
		    debugLevel = atoi (optarg);
		    break;
		case 'd':	/* Debugging on */
		    debugLevel = 3;
		    break;
		case 'v':	/* Verbose mode on */
		    debugLevel = 2;
		    break;
		case 's':	/* Status mode */
		    debugLevel = 1;
		    break;
	    }

	if (debugLevel > 0)
	    tisSysLog (0,  2, "Started, debug level = %d.\n", debugLevel);
	else
	    tisSysLog (0,  2, "Started.\n");

	/* Make ourselves into a server */

	serverInit ("trendProcessor", "/tis4000/tasks/trendProcessor.lock", "lock");

	/* Set up signal handler for kill signal */

#ifdef SOLARIS
	sigset (SIGQUIT, terminateProcessor);
	sigset (SIGTERM, terminateProcessor);
	sigset (SIGINT, terminateProcessor);
	sigset (SIGHUP, hupHandler);
	sigset (SIGUSR1, incrementDebugLevel);
	sigset (SIGUSR2, decrementDebugLevel);
#else
#ifdef WIN32
#else
	signal (SIGQUIT, terminateProcessor);
	signal (SIGTERM, terminateProcessor);
	signal (SIGINT, terminateProcessor);
	signal (SIGHUP, hupHandler);
	signal (SIGUSR1, incrementDebugLevel);
	signal (SIGUSR2, decrementDebugLevel);
#endif WIN32
#endif SOLARIS
	/* Create the shared memory region. */

	allocateSharedMemory ();

	/* Allocate the MX semaphore for Up-To-Date data access */

	mxSem = sem_create (TR_SEM_KEY, 1);

	/* Initialize the index tree and resource list. */

	tree = avl_init ((int(*)(void*,void*))treeCompareFunc);
        MListInit (&pointList, 0);
	MListInit (&trendList, 0);
        MListInit (&resourceList, 0);
	
	/* Initialize the database system */

	initCodebase ();

	/* Perform the initial read of the ar_reduc.dbf file and build the */
	/* search tree and various lists. */

	buildTrends ();

	/* Load the records corresponding to right now */
	
	loadInitialRecords ();

	/* Perform processor functions */

	processTrendEvents ();

	/* Write all the trends to the trendServer */

        writeTrends (TRUE);

	/* Send a terminate command to the server */

	request.method = -1;  /* -1 indicates terminate to server */
	rbInsert (requestQueue, &request);

	/* Remove the shared memory for the up-to-date data */

	removeSharedMemory ();

	/* Remove the mx semaphore */

	sem_rm (mxSem);

        if (debugLevel >= 1)
	    tisSysLog (0,  2, "Terminated.\n");
	
	return 0;
    }

/*----------*/
static void hupHandler (int sig)

    {

        writeTrends (TRUE);

    }

/*----------*/
static void terminateProcessor (int sig)

    {

	running = 0;

    }

/*----------*/
static void incrementDebugLevel (int sig)

    {

	if (debugLevel < 3)
	    debugLevel++;
	tisSysLog (0,  2, "Debug level is now %d\n",  debugLevel);

    }

/*----------*/
static void decrementDebugLevel (int sig)

    {

	if (debugLevel > 0)
	    debugLevel--;
	tisSysLog (0,  2, "Debug level is now %d\n", debugLevel);

    }

/*----------*/
static void allocateSharedMemory (void)

    {

        int id;

	/* Allocate shared memory for the processor/server queue */

	if ((id = shmget (TREND_SH_MEM_KEY, sizeof (RingBuf), 0666|IPC_CREAT)) == -1) {
	    tisSysLog (0,  2, "Unable to allocate shared memory for database server -- %s\n",
			strError (errno));
	    exit (-1);
	}
	requestQueue = (RingBuf *) shmat (id, (char *)0, 0666);
	if (requestQueue == (RingBuf *)-1) {
	    tisSysLog (0,  2, "Unable to shmat shared memory for database server -- $s",
			strError (errno));
	    exit (-1);
	}
	memset ((void *)requestQueue, 0, sizeof (RingBuf));

	/* Allocate shared memory for the up-to-date data */

	if ((upToDateId = shmget (TR_MEM_KEY, sizeof (UpToDateInfo), 0666|IPC_CREAT)) == -1) {
	    tisSysLog (0, 2, "Unable to allocate shared memory for up-to-date data -- %s\n",
			strError (errno));
	    exit (-1);
	}
	upToDateInfo = (UpToDateInfo *) shmat (upToDateId, (char *)0, 0666);
	if (upToDateInfo == (UpToDateInfo *)-1) {
	    tisSysLog (0,  2, "Unable to shmat shared memory for database server-- $s",
			strError (errno));
	    exit (-1);
	}
	memset ((void *)upToDateInfo, 0, sizeof (UpToDateInfo));

    }

/*----------*/
static void removeSharedMemory (void)

    {

	shmdt ((char *)upToDateInfo);
	shmctl (upToDateId, IPC_RMID, (struct shmid_ds *)0);

    }

/*----------*/
static int treeCompareFunc (PointNodePtr p1, PointNodePtr p2)

    {

        return strcmp (p1->pointName, p2->pointName);

    }

/************************************************************************/
/*             Main Processing loop is in this function.                */
/************************************************************************/

/*----------*/
static void processTrendEvents (void)

    {

	struct tm	*t;
	time_t		clock;
	int		caInitialized = FALSE;

        /* Initialize the current minute. */

	clock = time (0);
	t = localtime (&clock);
	currentMin = t->tm_min;

	/* Wait for an incoming message to process or until a timeout occurs */

	while (running) {

	    /* Check if a reconnect is necessary */

	    if (doConnect) {

		if (caInitialized)
		    ca_task_exit ();
		if (ca_task_initialize () != ECA_NORMAL) {
		    tisSysLog (0,  2, "CA won't initialize\n");
		    return;
		}
		caInitialized = 1;

		ca_add_fd_registration (reconnectCallback, NULL);

		if (ca_register_as_archiver ((void(*)(void))processDataEvent, 
			 (void(*)(void))processResourceStatusChange) != ECA_NORMAL) {
		    tisSysLog (0,  2, "Can't register as archiver with CA\n");
		    return;
		}

		doConnect = 0;

	    }

	    /* Wait for any incoming trend events, parameters, or status changes */

	    ca_pend_io ((float)0.1);
	    resetUpToDatePointers ();	/* see if points have benn added/removed from shared memory */

	    /* Get the current time to determine if it is time to do things. */

	    clock = time (0);
	    t = localtime (&clock);

	    if (clock >= nextSecond) {
		requestParameters ();
		updateOnly();		/* update shared memory once per second */
		nextSecond = clock + 1;
	    }

	    if (t->tm_min != currentMin) {

		currentMin = t->tm_min;
		if(eflag) {time_t tt;tt = time(0);printf("TH %s",ctime(&tt));}

#ifdef TRENDUPDATES
		/* Get any update values into the trend tree records */
		/* pass tree lookup routine to evade link problems in other programs */
		GetUpdates (findPoint);
#endif TRENDUPDATES

		writeTrends (FALSE);
		updateTrends ();

		if (debugLevel >= 1) {
		    tisSysLog (2,  2, "%d events/min, %.2f events/sec\n",
			    eventsProcessed, eventsProcessed/60.0);
		    eventsProcessed = 0;
		}

	    }

	} while (running);

	if (debugLevel >= 1)
	    tisSysLog (0,  2, "Terminating - writing all trends.\n");

    }

/*----------*/
static void reconnectCallback (void *arg, int fd, int cond)	/* Reconnect to CA */

    {

	if (cond == 0) {
	    doConnect = 1;
	    if (debugLevel >= 1)
		tisSysLog (1,  2, "Connection to CA lost.\n");
	}

    }

/************************************************************************/
/*     Functions for building and maintaining trend data structures     */
/************************************************************************/

/*----------*/
static int buildTrends (void)    /* Build the trends */

    {

#ifdef WIN32
	struct _stat	status;
#else
	struct stat	status;
#endif WIN32
	PointName	name;
        int 	        method;
	int		interval, archive;
	TrendPtr        t;
        PointNodePtr    p;

	/* Get the last modification date/time of the file */

#ifdef WIN32
	_stat (AR_REDUC_FILE, &status);
#else
	stat (AR_REDUC_FILE, &status);
#endif WIN32
	reducFileTime = status.st_mtime;

	/* Open the ar_reduc database and read it in. */

	if (!openArReduc ()) {
	    tisSysLog (0,  2, "Unable to open ar_reduc.dbf\n");
	    exit (-1);
	}
    
	/* Read in each trend record, place the point in the tree (if not */
	/* already there), and create a bucket set for the entry. */

        numTrends = 0;
	while (getArReducRecord (name, &method, &interval, &archive)) {

	  /* Look up the point in the tree. If not yet in the tree, */
	  /* create a tree node and insert it into the tree. */

	  p = findPoint (name);
	  if (p == NULL) {  /* Not found, create one */
	      p = (PointNodePtr) avl_newnode (sizeof (PointNode));
	      memset ((void *)p, 0, sizeof (PointNode));
	      strcpy (p->pointName, name);
	      p->reductionMethod = method;
	      avl_insert (tree, p);
	      MListInsertRear (&pointList, p);
	  }

	  /* Create the trend and add to the trendList */

	  t = (TrendPtr) calloc (1, sizeof (Trend));
	  t->method = method;
	  t->interval = interval;
	  t->pointNode = p;
	  p->trend [method] = t;
	  MListInsertRear (&trendList, (void *)t);

	  t->lastStatus = 0;
	  t->lastSeverity = 0;
	  t->resourceUp = FALSE;
	  t->active = TRUE;
	  numTrends++;

	}

	/* Close ar_reduc file */

	closeArReduc ();

	if (debugLevel >= 1)
	    tisSysLog (1,  2, " %d trends.\n", numTrends);
	if (debugLevel >= 2) {
	    printTrendsByTrendList ();
	}

	/* Reset the trendList iterator so that disk writing will start at */
	/* the beginning of the list. */

	MListStartFront (&trendList);

	return 1;	/* Indicates table was read */

    }

/*----------*/
static void updateTrends (void)

    {

        /* If the date/time on ar_reduc.dbf has changed, update the trend */
        /* data structures with the new trends. */

	struct stat	status;
	int		method, interval, archive, ix, i;
	PointName       pointName;
	PointNodePtr    p;
	TrendPtr        t;
	time_t          timeNow, startTime;

	/* Get the last modification date/time of the file.  If the file has not changed since */
	/* the last time we loaded it then don't reload. */

	stat (AR_REDUC_FILE, &status);
	if (reducFileTime == status.st_mtime)
	    return;
	reducFileTime = status.st_mtime;

	/* Maybe be verbose about loading the reduction information */

	if (debugLevel >= 2)
	    tisSysLog (1,  2, "Loading changed %s\n", AR_REDUC_FILE);

	/* Reset all the active flags for all trends. 
	 * They will be reactivated as they are read
	 */
	/* from ar_reduc. */

	MListStartFront (&trendList);
	while (t = MListNext (&trendList)) {
	    t->active = FALSE;
	}


        /* Look up the point in the tree. If not yet in the tree then add it. If in the tree then */
	/* check if method and rate are the same. If they are then mark the point as active. */
	/* If not then write the record to the server and reinitialize it. */ 

	openArReduc ();
        timeNow = time (0);
	while (getArReducRecord (pointName, &method, &interval, &archive)) {

	    p = findPoint (pointName);
	    if (p == NULL) {  /* Not found, create one */
	        p = (PointNodePtr) avl_newnode (sizeof (PointNode));
	        memset ((void *)p, 0, sizeof (PointNode));
	        strcpy (p->pointName, pointName);
	        p->reductionMethod = method;
	        avl_insert (tree, p);
	        MListInsertRear (&pointList, p);
		numTrends++;
	    }

	    t = p->trend [method];
	    if (t) {  /* Already have this trend, check if interval has changed */
	        if (t->interval == interval) {  /* Interval is the same, mark as active. */
		    t->active = TRUE;
		} else {    /* Interval has changed, write to database and reinitialize. */
		    writeTrend (t, time(0));
		    t->interval = interval;

		    startTime = recordStartTime (t, timeNow);
		    initializeTrend (t, startTime);

		    /* Fill up to now with connect error */

	            ix = bucketIndex (t, timeNow);
		    for (i=0 ; i<=ix ; i++)
		        t->quality [i] = 'C';
		    t->sampleCnt = ix - 1;
		    t->active = TRUE;
		    tickleTENB (p->pointName);
		}
	    } else {    /* Trend does not exist, create it. */
	        t = (TrendPtr) calloc (1, sizeof (Trend));
	        t->method = method;
	        t->interval = interval;
	        t->pointNode = p;
	        p->trend [method] = t;
	        MListInsertRear (&trendList, (void *)t);

		startTime = recordStartTime (t, timeNow);
		initializeTrend (t, startTime);

		/* Fill up to now with connect error */

	        ix = bucketIndex (t, timeNow);
		for (i=0 ; i<=ix ; i++)
		    t->quality [i] = 'C';
	        t->sampleCnt = ix - 1;

	        t->lastStatus = 0;
	        t->lastSeverity = 0;
	        t->active = TRUE;

		tickleTENB (p->pointName);

	    }

	}

	closeArReduc ();

	/* Force a request for all parameters */

	forceParameterRequest ();

	/* Reset trend list interator so that trend are written starting */
	/* with the first. */

	MListStartFront (&trendList);

    }

/*----------*/
static void tickleTENB (char *chanName)	/* Set the TENB field to force CA to update us */

    {

	char	name [50], *p;
	chid	myChid;

	/* Create the channel name with the TENB suffix */

	strcpy (name, chanName);
	p = strchr (name, '.');
	if (p)
	    *p = '\0';
	strcat (name, ".TENB");

	ca_build_and_connect (name, TYPENOTCONN, 0, &myChid, NULL, tickleHandler, (void *)0);

    }

/*----------*/
static void tickleHandler (struct connection_handler_args args)

    {

	static	short	i;

	i = 1;
	ca_put (OLD_DBR_SHORT, args.chid, &i);
	ca_clear_channel (args.chid);
	ca_flush_io ();

    }

/*----------*/
static void loadInitialRecords (void)  /* Load in records for current time */

    {

        TrendPtr     t;
	time_t       timeNow, startTime;
        int          i, ix;
	ArSnapRecord rec;

	/* Open or create the snap file */

#ifdef ORACLESNAP
	if (!OpenOracle ()) {
	    tisSysLog (0,  2, "Unable to open ar_snap.dbf\n");
	    exit (-1);
	}
#endif ORACLESNAP
	openArSnap ();

	/* Get the current time so that the record start times can be determined. */

	timeNow = time (0);

	/* Traverse through all trends and find any ar_snap records corresponding */
	/* to the current time. */

	MListStartFront (&trendList);
	while (t = MListNext (&trendList)) {

	    startTime = recordStartTime (t, timeNow);
	    initializeTrend (t, startTime);

	    if (getArSnapRecord (t->pointNode->pointName, t->method, t->interval,
				 startTime, &rec)) {

		/* Found a record, copy the data into the trend node */

		for (i=0 ; i<rec.sampleCnt ; i++) {
		    t->value [i] = rec.value [i];
		    t->quality [i] = rec.quality [i];
		    t->status [i] = rec.status [i];
		    t->severity [i] = rec.severity [i];
		}
		t->startTime = startTime;
		t->sampleCnt = rec.sampleCnt;

		/* Fill from the old last bucket to the current bucket with connection error */

	        ix = bucketIndex (t, timeNow);
	        while (i<=ix) {
		    t->quality [i] = 'C';
		    i++;
		}

	    } else {    /* Did not find record. Make empty record and fill to present with */
	                /* connect error. */

	        ix = bucketIndex (t, timeNow);
		for (i=0 ; i<=ix ; i++)
		     t->quality [i] = 'C';

 	    }

	    t->sampleCnt = ix + 1;
	    t->resourceUp = FALSE;
        }

	closeArSnap ();
#ifdef ORACLESNAP
	CloseOracle ();
#endif ORACLESNAP
  

    }

/*----------*/
void initializeTrend (TrendPtr t, time_t startTime)

    {

        int    i;

	t->startTime = startTime;
	t->sampleCnt = 0;
     	for (i=0 ; i<BUCKETS_PER_RECORD ; i++) {
	    t->count [i] = 0;
	    t->quality [i] = ' ';
	    t->status [i] = -1;
	    t->severity [i] = -1;
	}

    }


/************************************************************************/
/*   Functions for processing data and status change messages from CA   */
/************************************************************************/

/*----------*/
static void processDataEvent (char *pointName, short type, short count, void *buffPtr)

    {

	double	doubleVal;
	float	floatVal;
	short	shortVal;
	char	charVal;
	long	longVal;

	/* Check for an invalid timestamp. */

	if (((struct dbr_time_short *)buffPtr)->stamp.secPastEpoch < 100000)
	    return;
	if(eflag) {time_t tt;tt = time(0);printf("DE %s",ctime(&tt));}
	
	switch (type) {
	    case OLD_DBR_TIME_STRING:
		tisSysLog (0,  2, "Received DBR_TIME_STRING -- ignored."); 
		break;

	    case OLD_DBR_TIME_SHORT:
		memcpy ((char *)&shortVal, (char *)&((struct dbr_time_short *)buffPtr)->value, sizeof (short));
		processTrends (pointName,
			      time (0),
			      (double)shortVal,
			      ((struct dbr_time_short *)buffPtr)->status,
			      ((struct dbr_time_short *)buffPtr)->severity);
		break;

	    case OLD_DBR_TIME_FLOAT:
		memcpy ((char *)&floatVal, (char *)&((struct dbr_time_float *)buffPtr)->value, sizeof (float));
		processTrends (pointName,
			      time (0),
			      (double)floatVal,
			      ((struct dbr_time_float *)buffPtr)->status,
			      ((struct dbr_time_float *)buffPtr)->severity);
		break;

	    case OLD_DBR_TIME_ENUM:
		memcpy ((char *)&shortVal, (char *)&((struct dbr_time_enum *)buffPtr)->value, sizeof (short));
		processTrends (pointName,
			      time (0),
			      (double)shortVal,
			      ((struct dbr_time_enum *)buffPtr)->status,
			      ((struct dbr_time_enum *)buffPtr)->severity);
		break;

	    case OLD_DBR_TIME_CHAR:
		memcpy ((char *)&charVal, (char *)&((struct dbr_time_char *)buffPtr)->value, sizeof (char));
		processTrends (pointName,
			      time (0),
			      (double)charVal,
			      ((struct dbr_time_char *)buffPtr)->status,
			      ((struct dbr_time_char *)buffPtr)->severity);
		break;

	    case OLD_DBR_TIME_LONG:
		memcpy ((char *)&longVal, (char *)&((struct dbr_time_long *)buffPtr)->value, sizeof (long));
		processTrends (pointName,
			      time (0),
			      (double)longVal,
			      ((struct dbr_time_long *)buffPtr)->status,
			      ((struct dbr_time_long *)buffPtr)->severity);
		break;

	    case OLD_DBR_TIME_DOUBLE:
		memcpy ((char *)&doubleVal, (char *)&((struct dbr_time_double *)buffPtr)->value, sizeof (double));
		processTrends (pointName,
			      time (0),
			      doubleVal,
			      ((struct dbr_time_double *)buffPtr)->status,
			      ((struct dbr_time_double *)buffPtr)->severity);
		break;
	}

	eventsProcessed++;

    }

/*----------*/
static void processTrends (char *pointName, time_t ts, double value,
			   short status, short severity)

    {

        PointNodePtr    p;
        int             i;

        /* Lookup the point in the tree. If found process each trend */
        /* associated with the point. */

	p = findPoint (pointName);
	if (p != NULL) {
	    for (i=0 ; i<NUM_METHODS ; i++) {
	        if (p->trend [i] != NULL) {
		    processTrend (p->trend [i], ts, value, (int)status, (int)severity);
	        }
	    }
        } else {
	      if (debugLevel >= 3)
                  tisSysLog (0,  2, "Sample for point not on trend -- %s discarded.\n",
			  pointName);
        }

    }

/*----------*/
static void processResourceStatusChange (char *resName, short status)

    {

        ResListNodePtr    r;
        PointNodePtr      p;
        int               i;

        /* Traverse the list of all points for this resource. Update each */
        /* with the status change. */

        r = findResource (resName);
        if (r == NULL)   /* Not yet any points for this resource */
	    return;

	if(eflag) {time_t tt;tt=time(0);printf("CD %s",ctime(&tt));}

        /* Traverse the list of all points updating each of the trends */
        /* for the point. */

        MListStartFront (&(r->pointList));
        while (p = (PointNodePtr) MListNext (&(r->pointList))) {
	    for (i=0 ; i<NUM_METHODS ; i++) {
	        if (p->trend [i] != NULL) {
	            processStatusChange (p->trend [i], status);
	        }
	    }
        }

	if (debugLevel >= 2)
            tisSysLog (0,  2, "Resource status for %s changed to %d\n",
		    resName, (int)status);

    }


/*----------*/
static void writeTrends (int allTrends)   /* Write some trends to the server */

    {

        /* If allTrends is TRUE then queue all trends to the server. */
        /* Otherwise write a tenth of them.  The trends are counted as the */
        /* traverse proceeds. When the low order digit of the count equals */
        /* the current minute then the record is written. */

        int       count;
        TrendPtr  t;

	/* If we are writing all trends the the count is everything and */
	/* we start at the beginning of the trendList. Otherwise the count */
	/* is one tenth of the trends and we continue in the list from where */
	/* we left off. */

	if (allTrends) {
	    count = numTrends;
	    MListStartFront (&trendList);
	} else {
	    count = numTrends / 10;
        }

	/* Write trends until the count is zero or the end of the list. */

	t = (TrendPtr) MListNext (&trendList);
        for (;;) {
	    if (t == NULL)
	        break;
	    if (t->pointNode->gotParms)
	      {
	        writeTrend (t, time(0));
	      }
	    if (count > 0)
	        t = (TrendPtr) MListNext (&trendList);
	    else
	        break;
	    count--;
        }

	/* If we hit the end of the list, set it back to the beginning. */

	if (t == NULL)
	    MListStartFront (&trendList);

    }

/************************************************************************/
/*               Functions for processing point parameters              */
/************************************************************************/

/*----------*/
static void forceParameterRequest (void)   /* Force all parms to be requested. */

    {

        PointNodePtr  p;
	int           i;

	MListStartFront (&pointList);
	while (p = (PointNodePtr) MListNext (&pointList)) {
	    for (i=0 ; i<NUM_METHODS ; i++) {
	        if (p->trend [i]) {
	            if (p->trend[i]->active) {
	                p->gotParms = FALSE;
	                p->parmsTimeout = 0;
		        break;
		    }
		}
	    }
	}

    }

/*----------*/
static void requestParameters (void)	/* Request parameters for each channel */

    {

        int             count;
	PointNodePtr    p;

        /* Send a parameter request for up to POINTS_PER_PARM_REQUEST points */

	count = 0;
	MListStartFront (&pointList);
	while (p = (PointNodePtr) MListNext (&pointList)) {

	    /* If we already have parms for this point, just skip over it */

	    if (p->gotParms)
	        continue;

	    /* If the timout value is non zero then the parms for this */
	    /* point have already been requested but not yet received. */
	    /* If the timeout has not expired then skip over this point. */

	    if (p->parmsTimeout > time (0))
	        continue;
	    
	    /* Request the parameters. */

	    ca_get_params (p->pointName, DBR_DOUBLE_PARAM, 
			   (void(*)(void))processParameters, (void *)p);
	    ca_pend_io ((float)0.001);
	    p->parmsTimeout = time (0) + PARMS_REQUEST_TIMEOUT;

	    if (debugLevel >= 2)
	        tisSysLog (0,  2, "Requested parameters for %s\n", p->pointName);

	    /* If we have requested enough for now, quit. */

	    if (count++ == POINTS_PER_PARM_REQUEST)
	        return;

	}
    }

/*----------*/
static void processParameters (short type, void *buffPtr, void *pArg)

    {

        PointNodePtr   pp;
        PointName      pointName;
	ResListNodePtr r;

	pp = (PointNodePtr) pArg;
	/* if the pointNode is already on a resource list, remove it! */
	if (strlen(pp->resName))
	{
            r = getResource (pp->resName);
            MListRemove (&(r->pointList), (void *)pp);
	}
	pp->gotParms = TRUE;
	switch (type) {
	    case DBR_SHORT_PARAM:
		{
		    static struct dbr_short_param p;
		    memcpy ((char *)&p, (char *)buffPtr, sizeof (struct dbr_short_param));
		    strcpy (pointName, p.chanName);
		    strcpy (pp->resName, p.resName);
		    strcpy (pp->eguTag, p.units);
		    memcpy ((void *)&(pp->dispHigh), (void *)&(p.hopr), sizeof (double));
		    memcpy ((void *)&(pp->dispLow), (void *)&(p.lopr), sizeof (double));
		}
		break;
	    case DBR_ENUM_PARAM:
		{
		    static struct dbr_enum_param p;
		    memcpy ((char *)&p, (char *)buffPtr, sizeof (struct dbr_enum_param));
		    strcpy (pointName, p.chanName);
		    strcpy (pp->resName, p.resName);
		    strcpy (pp->eguTag, p.units);
		    memset ((void *)&(pp->dispHigh), 0, sizeof (double));
		    memset ((void *)&(pp->dispLow), 0, sizeof (double));
		}
		break;
	    case DBR_FLOAT_PARAM:
		{
		    static struct dbr_float_param p;
		    memcpy ((char *)&p, (char *)buffPtr, sizeof (struct dbr_float_param));
		    strcpy (pointName, p.chanName);
		    strcpy (pp->resName, p.resName);
		    strcpy (pp->eguTag, p.units);
		    memcpy ((void *)&(pp->dispHigh), (void *)&(p.hopr), sizeof (double));
		    memcpy ((void *)&(pp->dispLow), (void *)&(p.lopr), sizeof (double));
		}
		break;
	    case DBR_LONG_PARAM:
		{
		    static struct dbr_long_param p;
		    memcpy ((char *)&p, (char *)buffPtr, sizeof (struct dbr_long_param));
		    strcpy (pointName, p.chanName);
		    strcpy (pp->resName, p.resName);
		    strcpy (pp->eguTag, p.units);
		    memcpy ((void *)&(pp->dispHigh), (void *)&(p.hopr), sizeof (double));
		    memcpy ((void *)&(pp->dispLow), (void *)&(p.lopr), sizeof (double));
		}
		break;
	    case DBR_DOUBLE_PARAM:
		{
		    static struct dbr_double_param p;
		    memcpy ((char *)&p, (char *)buffPtr, sizeof (struct dbr_double_param));
		    strcpy (pointName, p.chanName);
		    strcpy (pp->resName, p.resName);
		    strcpy (pp->eguTag, p.units);
		    memcpy ((void *)&(pp->dispHigh), (void *)&(p.hopr), sizeof (double));
		    memcpy ((void *)&(pp->dispLow), (void *)&(p.lopr), sizeof (double));
		}
		break;
	}

        /* We now have the resource name for a point. Link the point onto */
        /* the appropriate resource list. */

        r = getResource (pp->resName);
        MListInsertRear (&(r->pointList), (void *)pp);

	if (debugLevel >=2)
	    tisSysLog (0,  2, "Parameters received for %s, res=%s egu=%s lopr=%lf hopr=%lf\n", 
		    pp->pointName, pp->resName, pp->eguTag, pp->dispLow, pp->dispHigh);

    }

/************************************************************************/
/*                      Various searching utilities                     */
/************************************************************************/
  
/*----------*/
PointNodePtr findPoint (char *name)

    {

       PointNode  pointNode;

       strcpy (pointNode.pointName, name);
       return (PointNodePtr) avl_findsym (tree, &pointNode);
	  
    }

/*----------*/
ResListNodePtr findResource (char *name)  /* Lookup a resource */

  {

        ResListNodePtr    r;

        MListStartFront (&resourceList);
        while (r = (ResListNodePtr) MListNext (&resourceList)) {
            if (strcmp (r->resName, name) == 0) {
                return r;
	    }
        }

        return NULL;

  }

/*----------*/
ResListNodePtr getResource (char *name)  /* Lookup a resource, create if not found */

    {

        ResListNodePtr    r;

	r = findResource (name);
	if (r != NULL)
	    return r;

        r = (ResListNodePtr) calloc (1, sizeof (ResListNode));
	MListInsertFront (&resourceList, (void *)r);
	strcpy (r->resName, name);
	MListInit (&(r->pointList), 1);
        return r;

    }

/************************************************************************/
/*              Functions to print various data structures              */
/************************************************************************/

/*----------*/
void printTrendsByTrendList (void)

    {

        TrendPtr    t;

	tisSysLog (0,  2, "Trends By Trend List\n");

	MListStartFront (&trendList);
	while (t = (TrendPtr) MListNext (&trendList)) {
	    tisSysLog (0,  2, "     %s %s %d\n", 
		    t->pointNode->pointName, int2methodString (t->method), t->interval);
	}

}

