/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#ifdef WIN32
#include <process.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "NTUnixSubs.h"
#else
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <signal.h>
#include <syslog.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <error.h>
#endif WIN32
#include <time.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "mlist.h"
#include "trendDefs.h"
#include "arSnapRecord.h"
#include "dbase.h"
#include "utils.h"
#include "archive.h"
#include "dbRequests.h"
#include "rb.h"
#include "msgMon.h"

/* Functions defined in this module */

static void   terminateServer (int);
static void   incrementDebugLevel (int);
static void   decrementDebugLevel (int);
static void   openSharedMemory (void);
static void   removeSharedMemory (void);
       int    doWriteRequests (void);

/* Data */

	int	running = TRUE;
        int     debugLevel, myPid;
	int	dbOpen = 0;
	int	archiveInProgress = FALSE;
	RingBuf	*requestQueue;
static  int     requestQueueId;

/* External data/functions */

extern  int     errno;
extern void serverInit(char *appName, char *dirPath, char *lockFile);

/*----------*/
main (int argc, char *argv[])

    {

	long		curHour;
	time_t		clock;
	struct tm	*t;
	int	c;
	extern	char	*optarg;

	tisOpenLog ("trendServer", 6, 2, NULL, NULL);

	myPid = getpid ();

	/* Process command line options */

	while ((c = getopt (argc, argv, "l:dvs")) != -1)
	    switch (c) {
		case 'l':	/* Explicitly set debug level */
		    debugLevel = atoi (optarg);
		    break;
		case 'd':	/* Debugging on */
		    debugLevel = 3;
		    break;
		case 'v':	/* Verbose mode on */
		    debugLevel = 2;
		    break;
		case 's':	/* Status mode */
		    debugLevel = 1;
		    break;
	    }

	if (debugLevel > 0)
	    tisSysLog (0, 2, "Started, debug level = %d.\n", debugLevel);
	else
	    tisSysLog (0, 2, "Started.\n");


	serverInit ("trendServer", "/tis4000/tasks/trendServer.lock", "lock");

	/* Set up signal handler for kill signal */

#if defined(SOLARIS)
	sigset (SIGQUIT, terminateServer);
	sigset (SIGTERM, terminateServer);
	sigset (SIGINT, terminateServer);
	sigset (SIGUSR1, incrementDebugLevel);
	sigset (SIGUSR2, decrementDebugLevel);
#else
#if !(defined(WIN32))
	signal (SIGQUIT, terminateServer);
	signal (SIGTERM, terminateServer);
	signal (SIGINT, terminateServer);
	signal (SIGUSR1, incrementDebugLevel);
	signal (SIGUSR2, decrementDebugLevel);
#endif WIN32
#endif SOLARIS

	/* Open the shared memory containing the request queue. */

	openSharedMemory ();

	/* Initialize the CodeBase system */

	initCodebase ();

	/* Read the archiving parameters */

	readArchiveConfig ();
	archiveInProgress = FALSE;

	/* Get initial hour for hourly processing */

	clock = time (0);
	t = localtime (&clock);
	curHour = t->tm_hour;

#ifdef ORACLESNAP
	if (!OpenOracle ()) {
	    tisSysLog (0, 2, "Unable to open Oracle\n");
	    exit (-1);
	}
#endif ORACLESNAP
	/* Start processing requests */
	
	while (running) {

#ifdef WIN32
	    Sleep (1000);
#else
	    sleep (1);
#endif WIN32

	    if (!rbEmpty (requestQueue)) {
		dbOpen = FALSE;
		running = doWriteRequests ();
		if (dbOpen) {
		    closeArSnap ();
		    dbOpen = FALSE;
		}
		if (!running)
		    break;
	    }

	    /* Get the current time to determine if it is time to do things. */

	    clock = time (0);
	    t = localtime (&clock);

	    if (t->tm_hour != curHour) {
		curHour = t->tm_hour;
		readArchiveConfig ();
		archiveAndClean (curHour);
	    }

	}

	if (dbOpen) {
	    closeArSnap ();
#ifdef ORACLESNAP
	    CloseOracle ();
#endif ORACLESNAP
	    dbOpen = FALSE;
	}
	/* Remove the shared memory segment */

	removeSharedMemory ();

	if (debugLevel >= 1)
	    tisSysLog (0, 2, "Terminated.\n");

	return 0;
    }

/*----------*/
static void terminateServer (int sig)

    {

	running = 0;

    }

/*----------*/
static void incrementDebugLevel (int sig)

    {

	if (debugLevel < 3)
	    debugLevel++;
	tisSysLog (0, 2, "Debug level is now %d\n", debugLevel);

    }

/*----------*/
static void decrementDebugLevel (int sig)

    {

	if (debugLevel > 0)
	    debugLevel--;
	tisSysLog (0, 2, "Debug level is now %d\n", debugLevel);

    }

/*----------*/
static void openSharedMemory (void)

    {

	if ((requestQueueId = shmget (TREND_SH_MEM_KEY, sizeof (RingBuf), 0666)) == -1) {
	    tisSysLog (0, 2, "Unable to open shared memory for database server -- %s\n",
			strError (errno));
	    exit (-1);
	}
	requestQueue = (RingBuf *) shmat (requestQueueId, (char *)0, 0666);
	if (requestQueue == (RingBuf *)-1) {
	    tisSysLog (0, 2, "Unable to shmat shared memory -- $s", strError (errno));
	    exit (-1);
	}

    }

/*----------*/
static void removeSharedMemory (void)

    {

	shmdt ((char *)requestQueue);
	shmctl (requestQueueId, IPC_RMID, (struct shmid_ds *)0);

    }


/*----------*/
int doWriteRequests (void)

    {

	WriteRequest	*p;
        int             iOpenedArSnap, i;
	static ArSnapRecord rec;

	/* Traverse through the list of write requests writing each one that is found. */
	while (!rbEmpty (requestQueue)) {

	    /* Get the request from the queue */

	    p = rbGet (requestQueue);

	    /* See if this is a server terminate request */

	    if (p->method == -1)
	        return FALSE;

	    /* The archiver may already have the file opened.  If not then open it. */

	    iOpenedArSnap = FALSE;
	    if (!dbOpen) {
		if (!openArSnap ()) {
		    tisSysLog (0, 2, "Unable to open ar_snap or ar_last.\n");
		    exit (-1);
		}

		iOpenedArSnap = TRUE;
		dbOpen = TRUE;
	    }

	    if (debugLevel >= 2)
		tisSysLog (0, 2, "Writing %s-%s-%d\n",
			p->pointName, int2methodString (p->method),
			p->interval);

	    /* Transfer the data to the database record and write it */

	    strcpy (rec.pointName, p->pointName);
	    strcpy (rec.eguTag, p->eguTag);
	    rec.dispLow = p->dispLow;
	    rec.dispHigh = p->dispHigh;
     	    ts2arSnapTime (p->startTime, &(rec.year), &(rec.dayOfYear),
			   &(rec.hour), &(rec.offsetMin));
	    rec.sampleCnt = p->sampleCnt;
	    rec.redMethod = p->method;
	    rec.rate = p->interval;
	    for (i=0 ; i<BUCKETS_PER_RECORD ; i++) {
	        rec.value [i] = p->value [i];
	        rec.quality [i] = p->quality [i];
		rec.status [i] = p->status [i];
		rec.severity [i] = p->severity [i];
	    }
	    putArSnapRecord (&rec);

	    /* Remove the item from the queue now */

	    rbRemove (requestQueue);

        }

	if (iOpenedArSnap) {
	    closeArSnap ();
	    dbOpen = FALSE;
	}

	return TRUE;

    }

