
/******************************************************************************\
*       This is a part of the Microsoft Source Code Samples. 
*       Copyright (C) 1993-1995 Microsoft Corporation.
*       All rights reserved. 
*       This source code is only intended as a supplement to 
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the 
*       Microsoft samples programs.
\******************************************************************************/

/****************************************************************************

    PROGRAM: Server.c

    PURPOSE: Server template for Windows applications

    FUNCTIONS:

        WinMain() - calls initialization function, processes message loop
        InitApplication() - initializes window data and registers window
        InitInstance() - saves instance handle and creates main window
        MainWndProc() - processes messages
        About() - processes messages for "About" dialog box

    COMMENTS:

        Windows can have several copies of your application running at the
        same time.  The variable hInst keeps track of which instance this
        application is so that processing will be to the correct window.

****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <time.h>
#include <odss.h>
#include <avl.h>
#include "server.h"                 /* specific to this program              */
#include "NTUnixSubs.h"
#include <winsock.h>
typedef char * caddr_t;
#include <cadef.h>
#include <signal.h>
#include "msgMon.h"

struct timeval	halfSecond = {0, 500000};
int odssFd = -1;
int maxFd = -1;
fd_set		fdMask;
AVL_TREE	*odtTree;
DWORD idInst = 0;
extern	char	*optarg;
extern	int	optind,opterr;
double *hValues,*hCounts;
unsigned char *hStatus;

CONVCONTEXT CCFilter = { sizeof(CONVCONTEXT), 0, 0, 0, 0L, 0L,
    {
        sizeof(SECURITY_QUALITY_OF_SERVICE),
        SecurityImpersonation,
        SECURITY_STATIC_TRACKING,
        TRUE
    }
};
HANDLE hInst;                       /* current instance */
HWND hwndServer;
RECT rcRand;
RECT rcCount;
RECT rcComment;
RECT rcExec;
RECT rcConnCount;
RECT rcRndrDelay;
RECT rcRunaway;
RECT rcAllBlock;
RECT rcNextAction;
RECT rcHugeSize;
RECT rcAppowned;
BOOL fAllBlocked = FALSE;
BOOL fAllEnabled = TRUE;
BOOL fEnableOneCB = FALSE;
BOOL fBlockNextCB = FALSE;
BOOL fTermNextCB = FALSE;
BOOL fAppowned = FALSE;
WORD cRunaway = 0;
WORD RenderDelay = 0;
DWORD count = 0;
WORD seed = 0;
HSZ hszAppName = 0;
CHAR szClass[] = "ServerWClass";
CHAR szTopic[MAX_TOPIC] = "Data Transfer";
CHAR szServer[MAX_TOPIC] = "TisDDEServer";
CHAR szComment[MAX_COMMENT] = "";
CHAR szExec[MAX_EXEC] = "";
CHAR *pszComment = szComment;
WORD cyText;
WORD cServers = 0;
HDDEDATA hDataHelp[CFORMATS] = {0};
HDDEDATA hDataCount[CFORMATS] = {0};
HDDEDATA hDataRand[CFORMATS] = {0};
HDDEDATA hDataHuge[CFORMATS] = {0};
DWORD cbHuge = 0;

char szDdeHelp[] = "DDEML test server help:\r\n\n"\
    "The 'Server'(service) and 'Test'(topic) names may change.\r\n\n"\
    "Items supported under the 'Test' topic are:\r\n"\
    "\tCount:\tThis value increments on each data change.\r\n"\
    "\tRand:\tThis value is randomly generated each data change.\r\n"\
    "\tHuge:\tThis is randomlly generated text data >64k that the\r\n"\
    "\t\tDDEML test client can verify.\r\n"\
    "The above items change after any request if in Runaway mode and \r\n"\
    "can bo POKEed in order to change their values.  POKEed Huge data \r\n"\
    "must be in a special format to verify the correctness of the data \r\n"\
    "or it will not be accepted.\r\n"\
    "If the server is set to use app owned data handles, all data sent \r\n"\
    "uses HDATA_APPOWNED data handles."\
    ;

FORMATINFO aFormats[CFORMATS] = {
    { 0, "CF_TEXT" },       // exception!  predefined format
    { 0, "Dummy1"  },
    { 0, "Dummy2"  },
};


/*
 *          Topic and Item tables supported by this application.
 */

/*   HSZ    PROCEDURE       PSZ        */

ITEMLIST SystemTopicItemList[CSYSTEMITEMS] = {

    { 0, TopicListXfer,  SZDDESYS_ITEM_TOPICS   },
    { 0, ItemListXfer,   SZDDESYS_ITEM_SYSITEMS },
    { 0, sysFormatsXfer, SZDDESYS_ITEM_FORMATS  },
    { 0, HelpXfer,       SZDDESYS_ITEM_HELP},
  };


ITEMLIST ServerTopicItemList[CTESTITEMS] = {

    { 0, ProcessGetValue,	"GETVALUE" },			// 0 index
    { 0, ProcessGetHTrend,	"GETHTREND" },			// 1 index
    { 0, ProcessGetHStatus,	"GETHSTATUS" },			// 2 index
    { 0, ItemListXfer,   SZDDESYS_ITEM_SYSITEMS },	// 3 index
  };


/* The system topic is always assumed to be first. */
/*   HSZ   PROCEDURE            #ofITEMS        PSZ     */
TOPICLIST topicList[CTOPICS] = {

    { 0, SystemTopicItemList,   CSYSTEMITEMS,   SZDDESYS_TOPIC},    // 0 index
    { 0, ServerTopicItemList,     CTESTITEMS,     szTopic},           // 1 index
  };
int clientLastTxTime;
int clientLastRxTime;
int clientBoomTime;
int clientNextChar = 0;
int clientCount = 0;
char clientReadBuff[ODT_MAX_BUFF_SIZE];
struct	timeval	maxWait,exitTime;
struct	timeval	currTime;

/****************************************************************************

    FUNCTION: WinMain(HANDLE, HANDLE, LPSTR, int)

    PURPOSE: calls initialization function, processes message loop

    COMMENTS:

        Windows recognizes this function by name as the initial entry point
        for the program.  This function calls the application initialization
        routine, if no other instance of the program is running, and always
        calls the instance initialization routine.  It then executes a message
        retrieval and dispatch loop that is the top-level control structure
        for the remainder of execution.  The loop is terminated when a WM_QUIT
        message is received, at which time this function exits the application
        instance by returning the value passed by PostQuitMessage().

        If this function must abort before entering the message loop, it
        returns the conventional value NULL.

****************************************************************************/

INT WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,INT nCmdShow)
{
    MSG msg;                                 /* message                      */
	int status,msgStatus;
    WSADATA wsaData; 
    WORD    wVersionRequested;
    int err;
	char odssHost[80];
	short odssPort;
	ODSS_REV_MSG rev;
	int argc;
	char *argv[10];
	int cmd;

    if (!hPrevInstance)                  /* Other instances of app running? */
        if (!InitApplication(hInstance)) /* Initialize shared things */
            return (FALSE);              /* Exits if unable to initialize     */
	/*
		parse the command line args from lpCmdLine if it is not NULL
	*/
	argc = 1;
	if(strlen(lpCmdLine))
	{
		argv[argc] = strtok(lpCmdLine," ");
		while(argv[++argc] = strtok(NULL," "));
	}

    /* Perform initializations that apply to a specific instance */

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);
	wVersionRequested = MAKEWORD(1, 1);
	if ((err = WSAStartup(wVersionRequested, &wsaData)) != 0) // Tell the user that we couldn't find a useable winsock.dll.
	{
		exit(0);
	}
	/*
		Initialize point tree
	*/
	odtTree = avl_init(odtChanCompare);
	/*
		See if user specified host name or port number. If not then use defaults
	*/
	strcpy(odssHost,"localhost");		/* default host name */
	odssPort = ODSS_PORT;				/* default port number */
	while((cmd = getopt(argc,argv,"p:h:")) != -1)
	{
		switch(cmd)
		{
		case 'h':
			if(*optarg != '-')
			{
				strcpy(odssHost,optarg);
				break;
			}
			else
				fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],cmd);
			break;
		case 'p':
			if(*optarg != '-')
			{
				odssPort = abs(atoi(optarg));	/* get time since last I heard from IOC/HOST */
				break;
			}
			else
				fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],cmd);
			break;
		}
	}
	/*
		Connect to ODSS server here. If can't connect then exit.
	*/
	if(odssConnect(odssHost,odssPort) < 0)
	{
		odtNetError();
	}
	sprintf(szComment,"%s Connected to %s",szServer,odssHost);
    InvalidateRect(hwndServer, &rcComment, TRUE);
	/*
		Send client revision code to ODSS. For now this is a one. If ODSS
		does not like it then nothing will be allowed to work on the (ODSS)
		server.
	*/
	rev.ODSSRev = htonl(ODSS_CURRENT_REV);
	odtSendToServer(ODSS_REV,(char *)&rev,sizeof(rev));
	/*
		Check for ODSS messages. If none(timeout) then check Windows message
		queue.
	*/
    while(1)
	{
		maxWait.tv_sec = 0;
		maxWait.tv_usec = 0;//10000;
		while((maxWait.tv_sec >= 0) && (maxWait.tv_usec >= 0))
		{
			currTime.tv_sec = time(0);
			currTime.tv_usec = 0;
			/*
				Check for and dispatch messages until a WM_QUIT message is received.
				Use PeekMessage so we don't block here.
			*/
			while((msgStatus = PeekMessage(&msg,0,0,0,PM_REMOVE)) != FALSE)
			{
			    TranslateMessage(&msg);    /* Translates virtual key codes           */
			    DispatchMessage(&msg);     /* Dispatches message to window           */
			}
			FD_ZERO(&fdMask);
			FD_SET(odssFd, &fdMask);
			status = select(maxFd + 1, &fdMask, NULL, NULL, &maxWait);
			if(status > 0)
			{
				odtReadServer();
			}
			/*
			  adjust timer
			*/
			exitTime.tv_sec = time(0);
			exitTime.tv_usec = 0;
			exitTime.tv_sec -= currTime.tv_sec;
			exitTime.tv_usec -= currTime.tv_usec;
			if(exitTime.tv_usec < 0)
			  {
				exitTime.tv_usec += 1000000;
				exitTime.tv_sec--;
			  }
			maxWait.tv_sec -= exitTime.tv_sec;
			maxWait.tv_usec -= exitTime.tv_usec;
			if(maxWait.tv_usec < 0)
			  {
				maxWait.tv_usec += 1000000;
				maxWait.tv_sec--;
			  }
		}//inner while
		avl_traverse(odtTree,INORDER,odtCheckAck,0);
    }// outer while

    UnregisterClass(szClass, hInstance);
    return (msg.wParam);           /* Returns the value from PostQuitMessage */
}


/****************************************************************************

    FUNCTION: InitApplication(HANDLE)

    PURPOSE: Initializes window data and registers window class

    COMMENTS:

        This function is called at initialization time only if no other
        instances of the application are running.  This function performs
        initialization tasks that can be done once for any number of running
        instances.

        In this case, we initialize a window class by filling out a data
        structure of type WNDCLASS and calling the Windows RegisterClass()
        function.  Since all instances of this application use the same window
        class, we only need to do this when the first instance is initialized.


****************************************************************************/

BOOL InitApplication(hInstance)
HANDLE hInstance;                              /* current instance           */
{
    WNDCLASS  wc;

    /* Fill in window class structure with parameters that describe the       */
    /* main window.                                                           */

    wc.style = 0;                       /* Class style(s).                    */
    wc.lpfnWndProc = MainWndProc;       /* Function to retrieve messages for  */
                                        /* windows of this class.             */
    wc.cbClsExtra = 0;                  /* No per-class extra data.           */
    wc.cbWndExtra = 0;                  /* No per-window extra data.          */
    wc.hInstance = hInstance;           /* Application that owns the class.   */
    wc.hIcon = LoadIcon(hInstance, "server");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HANDLE)(COLOR_APPWORKSPACE+1);
    wc.lpszMenuName =  "ServerMenu";   /* Name of menu resource in .RC file. */
    wc.lpszClassName = "ServerWClass"; /* Name used in call to CreateWindow. */

    /* Register the window class and return success/failure code. */

    return (RegisterClass(&wc));

}


/****************************************************************************

    FUNCTION:  InitInstance(HANDLE, int)

    PURPOSE:  Saves instance handle and creates main window

    COMMENTS:

        This function is called at initialization time for every instance of
        this application.  This function performs initialization tasks that
        cannot be shared by multiple instances.

        In this case, we save the instance handle in a static variable and
        create and display the main program window.

****************************************************************************/

BOOL InitInstance(hInstance, nCmdShow)
    HANDLE          hInstance;          /* Current instance identifier.       */
    INT             nCmdShow;           /* Param for first ShowWindow() call. */
{
    INT i;
    RECT Rect;
    TEXTMETRIC metrics;
    HDC hdc;

    /* Save the instance handle in static variable, which will be used in  */
    /* many subsequence calls from this application to Windows.            */

    hInst = hInstance;


    /* Create a main window for this application instance.  */

    hwndServer = CreateWindow(
        "ServerWClass",                 /* See RegisterClass() call.          */
        szServer,
        WS_OVERLAPPEDWINDOW|WS_ICONIC,  /* Window style.                      */
        CW_USEDEFAULT,                  /* Default horizontal position.       */
        CW_USEDEFAULT,                  /* Default vertical position.         */
        600,
        150,
        NULL,                           /* Overlapped windows have no parent. */
        NULL,                           /* Use the window class menu.         */
        hInstance,                      /* This instance owns this window.    */
        NULL                            /* Pointer not needed.                */
    );

    GetClientRect(hwndServer, (LPRECT) &Rect);

    /* If window could not be created, return "failure" */

    if (!hwndServer)
        return (FALSE);

    hdc = GetDC(hwndServer);
    GetTextMetrics(hdc, &metrics);
    cyText = (WORD)(metrics.tmHeight + metrics.tmExternalLeading);
    ReleaseDC(hwndServer, hdc);

    aFormats[0].atom = CF_TEXT; // exception - predefined.
    for (i = 1; i < CFORMATS; i++) {
        aFormats[i].atom = RegisterClipboardFormat(aFormats[i].sz);
    }

    /* Make the window visible; update its client area; and return "success" */

    ShowWindow(hwndServer, nCmdShow);  /* Show the window                        */
    UpdateWindow(hwndServer);          /* Sends WM_PAINT message                 */
    seed = 1;
    srand(1);
    CCFilter.iCodePage = CP_WINANSI;   // initial default codepage
    if (!DdeInitialize(&idInst, (PFNCALLBACK)MakeProcInstance((FARPROC)DdeCallback,
                hInstance), APPCMD_FILTERINITS, 0)) {
        Hszize();
        DdeNameService(idInst, hszAppName, 0, DNS_REGISTER);
        return(TRUE);
    }
    return (FALSE);

}

/****************************************************************************

    FUNCTION: MainWndProc(HWND, unsigned, WORD, LONG)

    PURPOSE:  Processes messages

    MESSAGES:

        WM_COMMAND    - application menu (About dialog box)
        WM_DESTROY    - destroy window

    COMMENTS:

        To process the IDM_ABOUT message, call MakeProcInstance() to get the
        current instance address of the About() function.  Then call Dialog
        box which will create the box according to the information in your
        server.rc file and turn control over to the About() function.   When
        it returns, free the intance address.

****************************************************************************/

LONG  APIENTRY MainWndProc(hWnd, message, wParam, lParam)
HWND hWnd;                                /* window handle                   */
UINT message;                         /* type of message                 */
WPARAM wParam;                              /* additional information          */
LPARAM lParam;                              /* additional information          */
{
    switch (message) {
    case WM_INITMENU:
        if (GetMenu(hWnd) != (HMENU)wParam)
            break;

        break;

    case WM_COMMAND:           /* message: command from application menu */
        switch (LOWORD(wParam)) {

        case IDM_ABOUT:
            DoDialog(MAKEINTRESOURCE(DID_ABOUT), (DLGPROC)About, 0, TRUE);
            break;

		case IDM_BOOMERANG:
			odtBoomerang();
			break;

		case IDM_EXIT:
			{
				DdeNameService(idInst, 0, 0, DNS_UNREGISTER); // unregister all services
				UnHszize();
				DdeUninitialize(idInst);
				avl_traverse(odtTree,INORDER,odtFreeDDEHandles,0);
				close(odssFd);
				free(hValues);
				free(hCounts);
				free(hStatus);
				exit(0);
				break;
			}

        case IDM_HELP:
           break;

        default:
            return (DefWindowProc(hWnd, message, wParam, lParam));
        }
        break;

    case WM_PAINT:
        PaintServer(hWnd);
        break;

    case WM_DESTROY:                  /* message: window being destroyed */
        if (fAppowned)
            SendMessage(hwndServer, WM_COMMAND, (WPARAM)MAKELONG(IDM_APPOWNED, 0), (LONG)(0));
        DdeNameService(idInst, 0, 0, DNS_UNREGISTER); // unregister all services
        UnHszize();
        DdeUninitialize(idInst);
        PostQuitMessage(0);
        break;

    default:
        return (DefWindowProc(hWnd, message, wParam, lParam));
    }
    return(0);
}





VOID Delay(
DWORD delay,
BOOL fModal)
{
    MSG msg;
    delay = GetCurrentTime() + delay;
    while (GetCurrentTime() < delay) {
        if (fModal && PeekMessage(&msg, 0, 0, 0, PM_REMOVE)) {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    }
}



/*
 * This function not only paints the server client area with current info,
 * it also has the side effect of setting the global RECTs that bound each
 * info area.  This way flashing is reduced.
 */
VOID PaintServer(
HWND hwnd)
{
    PAINTSTRUCT ps;
    RECT rc;
    CHAR szT[MAX_COMMENT];

    BeginPaint(hwnd, &ps);
    SetBkMode(ps.hdc, TRANSPARENT);
    GetClientRect(hwnd, &rc);
    rc.bottom = rc.top + cyText;    // all rects are cyText in height.

    rcComment = rc;
    DrawTextLine(ps.hdc, &ps.rcPaint, &rc, pszComment);

    wsprintf(szT, "Number of DDE client connections: %d", cServers);
    rcConnCount = rc;
    DrawTextLine(ps.hdc, &ps.rcPaint, &rc, szT);


    EndPaint(hwnd, &ps);
}


VOID DrawTextLine(
HDC hdc,
RECT *prcClip,
RECT *prcText,
PSTR psz)
{
    RECT rc;

    if (IntersectRect(&rc, prcText, prcClip)) {
        DrawText(hdc, psz, -1, prcText,
                DT_LEFT | DT_EXTERNALLEADING | DT_SINGLELINE | DT_EXPANDTABS |
                DT_NOCLIP | DT_NOPREFIX);
    }
    OffsetRect(prcText, 0, cyText);
}
/*
	ODSS connection function. If connect fails then return "-1" else return usable socket
*/
int odssConnect(char *host,short port)
{
    int addr;							    // inet address for current socket
    struct hostent	*ent;
    struct  sockaddr_in	destAddr;
	int stat;
	int arg = 1;
	odssFd = -1;
	if(ent = gethostbyname(host))			/* get INET address for me */
	{
		addr = *(long *)*ent -> h_addr_list;
		destAddr.sin_family = AF_INET;
		destAddr.sin_addr.s_addr = addr;
		destAddr.sin_port = htons(port);
		odssFd = socket(AF_INET,SOCK_STREAM,0);
		if((stat = connect(odssFd,(struct sockaddr *)&destAddr,sizeof(struct sockaddr))) < 0)
		{
			close(odssFd);
			odssFd = -1;
		}
	}
	if(odssFd != -1)
		ioctlsocket(odssFd,FIONBIO,&arg);
	return(odssFd);
}
