#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <sys/filio.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <netdb.h>
#include <sys/time.h>

#include "avl.h"
#include "db_access.h"
#include "odss.h"
#include "odtmain.h"
#include "msgMon.h"

#ifndef STDIN_FILENO
#define STDIN_FILENO 0
#endif

AVL_TREE *odtTree;
int odtSocket;
int odtLastTxTime;
int odtLastRxTime;
int odtBoomTime;
int odtNextChar = 0;
int odtCount = 0;
char odtReadBuff[ODT_MAX_BUFF_SIZE];
char odtStdinBuff[ODT_MAX_BUFF_SIZE];
char odtLastCommand[ODT_MAX_BUFF_SIZE];

main(int argc,char **argv)
{
  char odtHostName[20];
  short odtPort;
  int cmd;
  char *eofTest;
  fd_set myFds;
  int maxFds;
  int numSelected;
  ODSS_REV_MSG rev;
  struct	timeval	maxWait,exitTime;
  struct	timeval	currTime;

  extern	char	*optarg;

  tisOpenLog ("odt", 6, 2, NULL, NULL);

#ifndef SOLARIS
  signal(SIGINT,odtSignalHandler);	       	/* handle ^C from keyboard */
  signal(SIGHUP,odtSignalHandler);	       	/* handle task termination */
  signal(SIGPIPE,odtSignalHandler);		/* handle broken pipe signals */
  signal(SIGCHLD,odtSignalHandler);		/* handle child return signals */
#else
  sigset(SIGINT,odtSignalHandler);	       	/* handle ^C from keyboard */
  sigset(SIGHUP,odtSignalHandler);	       	/* handle task termination */
  sigset(SIGPIPE,odtSignalHandler);		/* handle broken pipe signals */
  sigset(SIGCHLD,odtSignalHandler);		/* handle child return signals */
#endif
  strcpy(odtHostName,ODT_DEFAULT_HOST);		/* set default host & port number */
  odtPort = ODT_DEFAULT_PORT;
  odtTree = avl_init(odtChanCompare);
  strcpy(odtLastCommand,"boom");		/* default "last command" */
  while((cmd = getopt(argc,argv,"h:p:")) != -1)
    {
      switch(cmd)
	{
	case 'h':
	  if(*optarg != '-')
	    {
	      strcpy(odtHostName,optarg);      	/* set host name */
	    }
	  else
	    {
	      printf ("Host name must have a value.\n");
	      exit(1);
	    }
	  break;
	case 'p':
	  if(*optarg != '-')
	    {
	      odtPort = abs((short)atoi(optarg));	/* set host port value */
	    }
	  else
	    {
	      printf ("Port number must have a value.\n");
	      exit(1);
	    }
	  break;
	case '?':
	  printf ("Usage is:\n");
	  printf ("\todt [-h hostname] [-p port]\n");
	  printf ("\thostname - Name of server host (%s)\n",ODT_DEFAULT_HOST);
	  printf ("\tport ----- Port number on server (%d)\n",ODT_DEFAULT_PORT);
	  exit(1);
	}
    }
  if(odtNetworkInit(odtHostName,odtPort) == -1)
    {
      /*
	If connection refused and odtHostName is me(localhost) then start ODSS program,
	wait a few seconds and try the connection again.
      */
      if((errno == ECONNREFUSED) && !strcmp(odtHostName,ODT_DEFAULT_HOST))
	{
	  system(ODT_ODSS_NAME);			/* spawn the ODSS task */
	  sleep(10);						/* wait 5 seconds */
	  if(odtNetworkInit(odtHostName,odtPort) == -1)		/* try again to connect */
	    {
	      tisSysLog (0,0,"odt error %m");	/* failed again, then exit */
	      exit(1);
	    }
	}
      else
	{
	  tisSysLog (0,0, "odt error %m"); /* error other than ECONNREFUSED or not localhost */
	  exit(1);			/* so exit....*/
	}
    }
  maxFds = -1;
  if(STDIN_FILENO > maxFds) maxFds = STDIN_FILENO;
  if(odtSocket > maxFds) maxFds = odtSocket;
  rev.ODSSRev = htonl(ODSS_CURRENT_REV);
  odtSendToServer(ODSS_REV,(char *)&rev,sizeof(rev));
  /*
    accept command lines from stdin until "quit" command. Commands are:

    GET channel
    PUT chanel value  (value must be in quotes if it contains embedded spaces
    MON channel
    STOP channel
    CLOSE channel
    HIST channel
    BOOM

    Commands are case insensitive but channel names are case sensitive
  */
  write(STDIN_FILENO,ODT_COMMAND_PROMPT,strlen(ODT_COMMAND_PROMPT));
  while(1)
    {
    maxWait.tv_sec = ODT_TIMER;
    maxWait.tv_usec = 0;
    while((maxWait.tv_sec >= 0) && (maxWait.tv_usec >= 0))
      {
	gettimeofday(&currTime,0);
	FD_ZERO(&myFds);
 	FD_SET(STDIN_FILENO,&myFds);
	FD_SET(odtSocket,&myFds);
	numSelected = select(maxFds+1,&myFds,NULL,NULL,&maxWait);
	if(numSelected == 0)
	  {
 	    break;
	  }
	else if(numSelected < 0)
	  {
	    tisSysLog (0,0, "error on select %m");
	  }
	else
	  {
	    if(FD_ISSET(odtSocket,&myFds))
	      {
		odtReadServer();
	      }
	  }
	/*
	  adjust timer
	*/
	gettimeofday(&exitTime,0);
	exitTime.tv_sec -= currTime.tv_sec;
	exitTime.tv_usec -= currTime.tv_usec;
	if(exitTime.tv_usec < 0)
	  {
	    exitTime.tv_usec += 1000000;
	    exitTime.tv_sec--;
	  }
	maxWait.tv_sec -= exitTime.tv_sec;
	maxWait.tv_usec -= exitTime.tv_usec;
	if(maxWait.tv_usec < 0)
	  {
	    maxWait.tv_usec += 1000000;
	    maxWait.tv_sec--;
	  }
      }/* inner while */
    /*
      check for BUILD_ACK timeouts & any stdin input
    */
    if(FD_ISSET(STDIN_FILENO,&myFds))
      {
	eofTest = gets(odtStdinBuff);
	if(eofTest != NULL)
	  {
	    if(strlen(odtStdinBuff))
	      strcpy(odtLastCommand,odtStdinBuff);
	    else
	      strcpy(odtStdinBuff,odtLastCommand);
	    odtProcessCommands(odtStdinBuff);
	  }
      }
    avl_traverse(odtTree,INORDER,odtCheckAck,0);
    }/* outer while */
}
/*
  Signal handler
*/
void odtSignalHandler(int whichOne)
{
switch(whichOne)
  {
  case SIGINT:
    exit(0);
  default:
    return;
  }
}
/*
  Initialize connection to server
*/
int odtNetworkInit(char *hostname,short port)
{
  struct  sockaddr_in	hostaddr;
  struct hostent	*ent;
  int addr;
  int err;
  int len;
  int trueFlag = 1;

  if(ent = gethostbyname(hostname))    		/* get INET address for desired server */
    addr = ntohl(*(long *)*ent -> h_addr_list);
  else if((addr = inet_addr(hostname)) == 0)
    addr = 0;
  if(addr == 0) return(-1);
  hostaddr.sin_family = AF_INET;
  hostaddr.sin_addr.s_addr = htonl(addr);
  hostaddr.sin_port = htons(port);
  odtSocket = socket(AF_INET,SOCK_STREAM,0);
  len = sizeof(struct sockaddr);
  if((err = connect(odtSocket,(struct sockaddr *)&hostaddr,len)) != -1)
    {
      setsockopt(odtSocket,IPPROTO_TCP,TCP_NODELAY,(char *)&trueFlag,sizeof(trueFlag));
      odtLastTxTime = time(0);
      err = 0;
      odtNextChar = 0;
      odtCount = 0;
    }
  else
    {
      close(odtSocket);
    }
  return(err);
}

/*
  Attempt to get a value from channel
*/
int odtGetChannelValue(char *channel)
{
  int32 handle;
  ODSS_READ_MSG msg;
  ODT_CHAN_INFO_PTR pInfo;

  if(channel)
    {
      handle = odtGetHandle(channel);		/* get the handle for this channel name */
      if(handle == 0)				/* if zero then channel not open...open it */
	{
	  pInfo = (ODT_CHAN_INFO_PTR)odtBuildChannel(channel);
	  pInfo->infoCmd = ODSS_READ;		/* save for when we get a build ACK */
	}
      else
	{
	  msg.ODSShandle = htonl(handle);
	  msg.timeout = htons(ODT_READ_TIMEOUT);
	  odtSendToServer(ODSS_READ,(char *)&msg,sizeof(msg));
	}
    }
}

int odtPutChannelValue(char *channel,char *value)
{
  int32 handle;
  ODT_CHAN_INFO_PTR pInfo;
  struct
  {
    ODSS_WRITE_MSG msg;
    char data[80];
  } writeMsg;

  if(channel)
    {
      handle = odtGetHandle(channel);		/* get the handle for this channel name */
      if(handle == 0)				/* if zero then channel not open...open it */
	{
	  pInfo = (ODT_CHAN_INFO_PTR)odtBuildChannel(channel);
	  strcpy((char *)pInfo->infoData,value);	/* save for later after build ACK */
	  pInfo->infoCmd = ODSS_WRITE;		/* save for when we get a build ACK */
	}
      else
	{
	  writeMsg.msg.ODSShandle = htonl(handle);
	  writeMsg.msg.timeout = htons(ODT_WRITE_TIMEOUT);
	  pInfo = (ODT_CHAN_INFO_PTR)handle;
	  writeMsg.msg.dataType = pInfo->infoNativeType;
	  writeMsg.msg.dataLength = odtFormatData(value,(char *)&writeMsg.data,pInfo->infoNativeType);
	  odtSendToServer(ODSS_WRITE,(char *)&writeMsg,(sizeof(ODSS_WRITE_MSG)+writeMsg.msg.dataLength));
	}
    }
}

int odtMonitorChannel(char *channel,short mask)
{
  int32 handle;
  ODSS_MONITOR_MSG msg;
  ODT_CHAN_INFO_PTR pInfo;

  if(channel)
    {
      handle = odtGetHandle(channel);		/* get the handle for this channel name */
      if(handle == 0)				/* if zero then channel not open...open it */
	{
	  pInfo = (ODT_CHAN_INFO_PTR)odtBuildChannel(channel);
	  pInfo->infoCmd = ODSS_MONITOR;       	/* save for when we get a ODSS_CONN_MSG */
	  pInfo->infoMask = mask;	       	/* save desired mask too 9-Oct-1997 */
	}
      else
	{
	  msg.ODSShandle = htonl(handle);
	  msg.eventMask = htons(mask);
	  odtSendToServer(ODSS_MONITOR,(char *)&msg,sizeof(msg));
	}
    }
}

int odtCloseChannel(char *channel)
{
  ODSS_CHAN_CLEAR_MSG msg;
  int32 handle;

  msg.ODSShandle = 0;
  if((handle = odtGetHandle(channel)) != 0)
    {
      msg.ODSShandle = htonl(handle);
      avl_delete(odtTree,(ODT_CHAN_INFO_PTR)handle);
      avl_freenode((ODT_CHAN_INFO_PTR)handle);
    }
    odtSendToServer(ODSS_CHAN_CLEAR,(char *)&msg,sizeof(msg));
}

int odtBoomerang(void)
{
  ODSS_BOOMERANG_MSG msg;

  odtBoomTime = time(0);
  msg.ODSShandle = htonl(1234);
  odtSendToServer(ODSS_BOOMERANG,(char *)&msg,sizeof(msg));
}

int odtGetHistData(char *hist)
{
  struct
  {
    ODSS_HIST_READ_MSG msg;
    char name[40];
  } histMsg;
  char *time,*span,*num;
  int size;

  strcpy((char *)&histMsg.name[0],strtok(hist,","));
  histMsg.msg.nameLen = htons(strlen(histMsg.name)+1);
  size = strlen(histMsg.name)+1;
  time = strtok(NULL,",");
  histMsg.msg.startTime = odtParseTime(time);
  span = strtok(NULL,",");
  histMsg.msg.secsPerSample = htonl(atoi(span));
  strcpy(histMsg.msg.dbReduc,strtok(NULL,","));
  strcpy(histMsg.msg.dataReduc,strtok(NULL,","));
  num = strtok(NULL,",");
  histMsg.msg.numSamples = htonl(atoi(num));
  histMsg.msg.collRate = 1;
  odtSendToServer(ODSS_HIST_READ,(char *)&histMsg,(sizeof(ODSS_HIST_READ_MSG)+size));
}
int odtSendToServer(short msgtype,char *msg,int size)
{
  short mtype = htons(msgtype);
  odtWrite((char *)&mtype,sizeof(short));
  odtWrite(msg,size);
}
int odtWrite(char *buff,int nbytes)
{
  int		buffSize;
  int		numSent;
  int		intSize = sizeof(int);

  if(odtSocket == -1)
    return(-1);
  numSent = send(odtSocket,buff,nbytes,0);
  odtLastTxTime = time(0);
  if (numSent == -1)
    {
      if (errno == EWOULDBLOCK)
	{
	  getsockopt(odtSocket, SOL_SOCKET, SO_SNDBUF, (char *)&buffSize, &intSize);
	  buffSize += nbytes;
	  if (setsockopt(odtSocket, SOL_SOCKET, SO_SNDBUF, (char *)&buffSize, sizeof(buffSize)) == -1)
	    {
	      tisSysLog(0,0,"Setsockopt error in \"odtWrite\"  on socket %m");
	      return(odtNetError());
	    }
	}
      else
	{
	  tisSysLog(0,0,"Send error in \"odtWrite\" on socket %m");
	  return(odtNetError());
	}
      return (odtWrite(buff, nbytes));
    }
  else if (numSent != nbytes)
    {
      buff += numSent;
      nbytes -= numSent;
      return (odtWrite(buff, nbytes));
    }
  return(1);
}
int odtNetError(void)
{
  close(odtSocket);
  odtSocket = -1;
  exit(-1);
}
int32 odtBuildChannel(char *channel)
{
  int32 handle;
  int32 size;

  struct
  {
    ODSS_BUILD_MSG msg;
    char name[ODT_CHAN_NAME_SIZE];
  } nameMsg;
  ODT_CHAN_INFO_PTR pInfo;

  handle = 0;
  if(channel)
    {
      handle = odtMakeHandle(channel);
      nameMsg.msg.ODSShandle = htonl(handle);
      nameMsg.msg.nameLength = htonl(strlen(channel)+1);
      sprintf((char *)&nameMsg.name,"%s\0",channel);
      size = strlen(channel)+1;
      odtSendToServer(ODSS_BUILD,(char *)&nameMsg,(sizeof(ODSS_BUILD_MSG)+size));
      pInfo = (ODT_CHAN_INFO_PTR)handle;
      pInfo->infoTime = time(0);
    }
  return(handle);
}
/*
  return my handle for this channel name. If no handle exists then return
  zero to force an ODSS_BUILD_MSG to be sent
*/
int32 odtGetHandle(char *channel)
{
  ODT_CHAN_INFO info;

  if(channel)
    {
      strcpy((char *)&info.infoChannelName,channel);
      return((int32)avl_findsym(odtTree,&info));	/* either it's there or its not */
    }
  else
    return(0);
}
/*
  Make handle from channel name. The handle is actually the address
  of the node in the tree for the given point name.
*/
int32 odtMakeHandle(char *channel)
{
  ODT_CHAN_INFO_PTR pInfo;

  pInfo = avl_newnode(CHAN_INFO_SIZE);		/* create new node */
  strcpy(pInfo->infoChannelName,channel);      	/* put in node */
  if(avl_insert(odtTree,pInfo) != NULL)
    {
      avl_freenode(pInfo);
      pInfo = 0;
    }
  else
    {
      pInfo->infoHandle = (int32)pInfo;
      pInfo->infoConnState = ODT_NOCONN;		/* not connected yet */
      pInfo->infoAckState = ODT_BUILDACK_PENDING;	/* waiting for build ACK */
    }
  return((int32)pInfo);
}

int odtChanCompare(void *ci1,void *ci2)
{
  return(strcmp(((ODT_CHAN_INFO_PTR)ci1)->infoChannelName,
		((ODT_CHAN_INFO_PTR)ci2)->infoChannelName));
}
void odtDumpNode(void *p1,void *node)
{
  printf("------\n");
  printf("Channel\t\t%s\n", ((ODT_CHAN_INFO_PTR)node)->infoChannelName);
  printf("Handle\t\t%08X\n", ((ODT_CHAN_INFO_PTR)node)->infoHandle);
  printf("Count\t\t%d\n", ((ODT_CHAN_INFO_PTR)node)->infoNativeCount);
  printf("Data type\t%d\n", ((ODT_CHAN_INFO_PTR)node)->infoNativeType);
}
void odtGetMulti(void *p1,void *node)
{
  odtGetChannelValue(((ODT_CHAN_INFO_PTR)node)->infoChannelName);
}
void odtCheckAck(void *p1, void *node)
{
  if(((time(0) - ((ODT_CHAN_INFO_PTR)node)->infoTime) > ODT_ACK_TIMEOUT) && 
      (((ODT_CHAN_INFO_PTR)node)->infoAckState == ODT_BUILDACK_PENDING))
    {
      avl_delete(odtTree,node);
      printf("odt: Build ack timed-out for \"%s\"\n%s",
		((ODT_CHAN_INFO_PTR)node)->infoChannelName,ODT_COMMAND_PROMPT);
    }
}

void odtCloseMulti(void *p1,void *node)
{
  ODSS_CHAN_CLEAR_MSG msg;

  msg.ODSShandle = htonl((int)node);
  odtSendToServer(ODSS_CHAN_CLEAR,(char *)&msg,sizeof(msg));
}
int16 odtFormatData(char *valueString,char *data,int16 type)
{
  int16 len;
  float tempf;
  double tempd;
  short temps;
  int tempi;

  len = 0;
  switch(type)
    {
    case OLD_DBF_CHAR:
      data[0] = valueString[0];
      len = 1;
      break;
    case OLD_DBF_STRING:
      strcpy(data,valueString);
      len = strlen(valueString);
      break;
    case OLD_DBF_FLOAT:
      tempf = atof(valueString);
      memcpy(data,(char *)&tempf,sizeof(float));
      len = sizeof(float);
      break;
    case OLD_DBF_DOUBLE:
      tempd = atof(valueString);
      memcpy(data,(char *)&tempd,sizeof(double));
      len = sizeof(double);
      break;
    case OLD_DBF_SHORT:
    case OLD_DBF_ENUM:
      temps = (short)atoi(valueString);
      memcpy(data,(char *)&temps,sizeof(short));
      len = sizeof(short);
      break;
    case OLD_DBF_LONG:
      tempi = (int)atoi(valueString);
      memcpy(data,(char *)&tempi,sizeof(int));
      len = sizeof(int);
      break;
    default:
      len = 0;
    }
  return(len);
}
int odtReadServer(void)
{
int status;
unsigned short type,msgType;
short mSize;

status = odtReadSocket();					/* drain the FD */
if(status > 0)
  {
    while(mSize = odtCompleteMsg())
      {					       			/* if complete message is in buffer */
	odtReadBytes((char *)&msgType,2);      			/* get the message type */
	type = ntohs(msgType);
/* 	printf("got message type %02X\n",type); */
	switch(type)
	  {
	  case ODSS_BOOMERANG:
	    {
	      ODSS_BOOMERANG_MSG msg;
	      time_t now;
	      struct tm *tms;

	      now = time(0);
	      tms = localtime(&now);
	      odtReadBytes((char *)&msg,sizeof(msg));
	      if((ntohl(msg.ODSShandle) != 1234) || ((time(0) - odtBoomTime) > ODT_BOOM_RESPONSE_TIME))
		tisSysLog(0,0,"Boomerang error or server not responding\n");
	      else
		tisSysLog(0,0,"Server response: %02d:%02d:%02d\n%s",
			tms->tm_hour,tms->tm_min,tms->tm_sec,ODT_COMMAND_PROMPT);
	      break;
	    }
	  case ODSS_BUILD_ACK:
	    {
	      ODT_CHAN_INFO_PTR pInfo;
	      ODSS_BUILD_ACK_MSG msg;
	      odtReadBytes((char *)&msg,sizeof(msg));
	      pInfo = (ODT_CHAN_INFO_PTR)ntohl(msg.ODSShandle);
	      pInfo->infoNativeType = ntohs(msg.nativeType);
	      pInfo->infoNativeCount = ntohs(msg.nativeCount);
	      pInfo->infoAckState = ODT_BUILD_ACKED;
	      break;
	    }
	  case ODSS_CHAN_CONN:
	    {
	      ODT_CHAN_INFO_PTR pInfo;
	      ODSS_CHAN_CONN_MSG msg;
	      int16 state;
	      odtReadBytes((char *)&msg,sizeof(msg));
	      pInfo = (ODT_CHAN_INFO_PTR)ntohl(msg.ODSShandle);
	      state = ntohs(msg.connState);
	      if(state == 1)
		{
		  pInfo->infoConnState = ODT_CONN;
		  switch(ntohs(pInfo->infoCmd))
		    {
		    case ODSS_READ:
		      {
			ODSS_READ_MSG readMsg;
			readMsg.ODSShandle = msg.ODSShandle;
			readMsg.timeout = htons(ODT_READ_TIMEOUT);
			odtSendToServer(ODSS_READ,(char *)&readMsg,sizeof(readMsg));
			break;
		      }
		    case ODSS_WRITE:
		      {
			struct
			{
			  ODSS_WRITE_MSG msg;
			  char data[80];
			} writeMsg;
			writeMsg.msg.ODSShandle = msg.ODSShandle;
			writeMsg.msg.timeout = htons(ODT_WRITE_TIMEOUT);
			writeMsg.msg.dataType = pInfo->infoNativeType;
			writeMsg.msg.dataLength = odtFormatData((char *)&pInfo->infoData[0],(char *)&writeMsg.data,pInfo->infoNativeType);
			odtSendToServer(ODSS_WRITE,(char *)&writeMsg,(sizeof(ODSS_WRITE_MSG)+writeMsg.msg.dataLength));
			break;
		      }
		    case ODSS_MONITOR:
		      {
			ODSS_MONITOR_MSG monMsg;
			monMsg.ODSShandle = msg.ODSShandle;
			monMsg.eventMask = htons(pInfo->infoMask);
			odtSendToServer(ODSS_MONITOR,(char *)&monMsg,sizeof(monMsg));
		      }
		    }
		}
	      else
		{
		  pInfo->infoConnState = ODT_NOCONN;
		}
	      break;
	    }
	  case ODSS_READ_DATA:
	    {
	      int16 size;
	      ODT_CHAN_INFO_PTR pInfo;
	      ODSS_READ_DATA_MSG rMsg;
	      char data[ODT_MAX_BUFF_SIZE];
	      odtReadBytes((char *)&rMsg,sizeof(ODSS_READ_DATA_MSG));	/* get the message body */
	      size = ntohs(rMsg.dataLength);
	      odtReadBytes((char *)data,size);				/* get the data */
	      pInfo = (ODT_CHAN_INFO_PTR)ntohl(rMsg.ODSShandle);
	      odtFormatOutput(data,pInfo,size);				/* format to output */
	      break;
	    }
	  case ODSS_MONDATA:
	    {
	      int16 size;
	      ODT_CHAN_INFO_PTR pInfo;
	      ODSS_MONDATA_MSG mMsg;
	      char data[ODT_MAX_BUFF_SIZE];
	      odtReadBytes((char *)&mMsg,sizeof(ODSS_MONDATA_MSG));	/* get the message body */
	      size = ntohs(mMsg.dataLength);
	      odtReadBytes((char *)data,size);				/* get the data */
	      pInfo = (ODT_CHAN_INFO_PTR)ntohl(mMsg.ODSShandle);
	      odtFormatOutput(data,pInfo,size);				/* format to output */
	      break;
	    }
	  case ODSS_WRITE_ACK:
	    {
	      int16 writeStat;
	      ODSS_WRITE_ACK_MSG waMsg;
	      odtReadBytes((char *)&waMsg,sizeof(ODSS_WRITE_ACK_MSG));
	      writeStat = ntohs(waMsg.status);
	    }
	  case ODSS_HIST_DATA:
	    {
	      ODSS_HIST_DATA_MSG hdMsg;
	      int32 size,num,i;
	      double *values,*counts;
	      unsigned char *status;
	      unsigned char statBytes[4];
	      odtReadBytes((char *)&hdMsg,sizeof(ODSS_HIST_DATA_MSG));
	      size = ntohl(hdMsg.dataLength);
	      num = ntohl(hdMsg.numReturned);
	      values = (double *)calloc(num,sizeof(double));
	      counts = (double *)calloc(num,sizeof(double));
	      status = (unsigned char *)calloc(num,3);
	      odtReadBytes((char *)values,sizeof(double)*num);
	      odtReadBytes((char *)counts,sizeof(double)*num);
	      odtReadBytes((char *)status,3*num);
	      for(i=0;i<num;i++)
		{
		  if(status[i*3] == 0xff)		/* check for No Data */
		    strcpy(statBytes,"---");
		  else
		    sprintf(statBytes,"%3.3s\0",&status[i*3]);
		  printf("Value = %lf,Count = %lf,Status = %s\nodt> ",values[i],counts[i],statBytes);
		}
	      free(values);
	      free(counts);
	      free(status);
	    }
	  }
      }
  }
}
short odtCompleteMsg(void)
{
  short	mType;
  int mSize;
  ODSS_READ_DATA_MSG rMsg;
  ODSS_MONDATA_MSG mMsg;
  ODSS_HIST_DATA_MSG hdMsg;

  if(odtCount < sizeof(mType))
    return(0);
  bcopy(&odtReadBuff[odtNextChar],&mType,sizeof(mType));
  mSize = ODT_MAX_BUFF_SIZE+1;
  /*
    switch on the message type. All message are considered complete if the number of bytes
    in the buffer (ioc_rxCount) is greater that or equal to the total message size PLUS
    the size of the message type.
  */
  switch(ntohs(mType))
    {
    case ODSS_BOOMERANG:
      mSize = sizeof(ODSS_BOOMERANG_MSG) + sizeof(mType);
      break;
    case ODSS_BUILD_ACK:
      mSize = sizeof(ODSS_BUILD_ACK_MSG) + sizeof(mType);
      break;
    case ODSS_CHAN_CONN:
      mSize = sizeof(ODSS_CHAN_CONN_MSG) + sizeof(mType);
      break;
    case ODSS_WRITE_ACK:
      mSize = sizeof(ODSS_WRITE_ACK_MSG) + sizeof(mType);
      break;
    case ODSS_READ_DATA:
      if(odtCount >= (sizeof(ODSS_READ_DATA_MSG) + sizeof(mType)))
	{
	  short temp;
	  memcpy((char *)&rMsg,(char *)&odtReadBuff[odtNextChar+2],sizeof(ODSS_READ_DATA_MSG));
	  temp = rMsg.dataLength;
	  mSize = sizeof(ODSS_READ_DATA_MSG) + ntohs(temp) + sizeof(mType);
	}
      break;
    case ODSS_MONDATA:
      if(odtCount >= (sizeof(ODSS_MONDATA_MSG) + sizeof(mType)))
	{
	  short temp;
	  memcpy((char *)&mMsg,(char *)&odtReadBuff[odtNextChar+2],sizeof(ODSS_MONDATA_MSG));
	  temp = mMsg.dataLength;
	  mSize = sizeof(ODSS_MONDATA_MSG) + ntohs(temp) + sizeof(mType);
	}
      break;
    case ODSS_HIST_DATA:
      if(odtCount >= (sizeof(ODSS_HIST_DATA_MSG) + sizeof(mType)))
	{
	  int32 temp;
	  memcpy((char *)&hdMsg,(char *)&odtReadBuff[odtNextChar+2],sizeof(ODSS_HIST_DATA_MSG));
	  temp = hdMsg.dataLength;
	  mSize = sizeof(ODSS_HIST_DATA_MSG) + ntohl(temp) + sizeof(mType);
	}
      break;
    default:
      tisSysLog(0,0,"Invalid message %04X from ODSS server...exiting\n",mType);
      odtNetError();
    }
  if(mSize > odtCount)
    return(0);
  else
    return(mSize);
}
int odtReadSocket(void)
{
  int		readSize,stat;

  if(odtSocket == -1)
    return(-1);
  errno = 0;
/*
	If we are already some bytes into the buffer then slide them down to the
	begining and zero the next char pointer.
*/
  if(odtNextChar)
    {
      if(odtCount)
         {
           bcopy(&odtReadBuff[odtNextChar],&odtReadBuff[0],odtCount);
         }
      odtNextChar = 0;
    }
  readSize = ODT_MAX_BUFF_SIZE - odtCount;
  if ((stat = recv(odtSocket, &odtReadBuff[odtCount+odtNextChar], readSize, 0)) < 0)
    {
      tisSysLog(0,0,"Read error in \"odtReadSocket\" %m");
      return(odtNetError());		/* GCT 2-Jul-1996 drop the whole thing on error */
    }
  if(stat == 0)					/* zero length read */
    {
      tisSysLog(0,0,"Zero-length read in \"odtReadSocket\" %m");
      return(odtNetError());
    }
  odtCount += stat;			/* number actually read into buffer */
  odtLastRxTime = time(0);		/* time of this recieve */
  return(stat);
}
void odtReadBytes(char *buff,int size)
{
  memcpy(buff,(char *)&odtReadBuff[odtNextChar],size);
  odtNextChar += size;
  odtCount -= size;
}
int odtFormatOutput(char *data,ODT_CHAN_INFO_PTR pInfo,int size)	/* format to output */
{
  char out[80];
  time_t now;
  struct tm *tms;

  now = time(0);
  tms = localtime(&now);
  switch(pInfo->infoNativeType)
    {
    case OLD_DBF_DOUBLE:
      sprintf(out,"%lf\0",*(double *)data);
      break;
    case OLD_DBF_FLOAT:
      sprintf(out,"%f\0",*(float *)data);
      break;
    case OLD_DBF_STRING:
      strncpy(out,data,size);
      break;
    case OLD_DBF_SHORT:
    case OLD_DBF_ENUM:
      sprintf(out,"%d\0",*(short *)data);
      break;
    case OLD_DBF_LONG:
      sprintf(out,"%d\0",*(int *)data);
      break;
    case OLD_DBF_CHAR:
      out[0] = data[0];
      break;
    }
  printf("%*.*s %02d:%02d:%02d %-s\n%s",ODT_CHAN_NAME_SIZE,ODT_CHAN_NAME_SIZE,pInfo->infoChannelName,tms->tm_hour,tms->tm_min,tms->tm_sec,out,ODT_COMMAND_PROMPT);
}
void odtProcessCommands(char *cmdline)
{
  char *cmdToken;
  char *channelToken;
  char *valueToken;

  cmdToken = strtok(cmdline," ");
  if(cmdToken)
    {
      int i;
      channelToken = NULL;
      valueToken = NULL;
      for(i=0;i < strlen(cmdToken);i++) cmdToken[i] = toupper(cmdToken[i]);
/* 		    printf("Got command <%s>\n",cmdToken); */
      if(!strcmp(cmdToken,ODT_COMMAND_QUIT))
	{
	  avl_traverse(odtTree,INORDER,odtCloseMulti,0);
	  avl_empty(odtTree,avl_freenode);
	  if(odtSocket != -1) close(odtSocket);
	  printf("\n");
	  exit(0);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_GET))
	{
	  channelToken = strtok(NULL," ");
	  if(channelToken != 0)
	    odtGetChannelValue(channelToken);
	  else
	    avl_traverse(odtTree,INORDER,odtGetMulti,0);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_PUT))
	{
	  channelToken = strtok(NULL," ");
	  valueToken = strtok(NULL,"\"");
	  odtPutChannelValue(channelToken,valueToken);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_MON))
	{
	  channelToken = strtok(NULL," ");
	  odtMonitorChannel(channelToken,ODT_VALUE_MASK);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_STOP))
	{
	  channelToken = strtok(NULL," ");
	  odtMonitorChannel(channelToken,ODT_STOP_MASK);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_CLOSE))
	{
	  channelToken = strtok(NULL," ");
	  if(channelToken)
	    odtCloseChannel(channelToken);
	  else
	    {
	      avl_traverse(odtTree,INORDER,odtCloseMulti,0);
	      avl_empty(odtTree,avl_freenode);
	    }
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_DUMP))
	{
	  avl_traverse(odtTree,INORDER,odtDumpNode,0);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_BOOM))
	{
	  odtBoomerang();
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_TREND))
	{
	  channelToken = strtok(NULL," ");
	  odtMonitorChannel(channelToken,ODT_TREND_MASK);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_ALARM))
	{
	  channelToken = strtok(NULL," ");
	  odtMonitorChannel(channelToken,ODT_ALARM_MASK);
	}
      else if(!strcmp(cmdToken,ODT_COMMAND_HIST))
	{
	  char *histParams;
	  histParams = strtok(NULL,"\n");
	  odtGetHistData(histParams);
	}
    }
  write(STDIN_FILENO,ODT_COMMAND_PROMPT,strlen(ODT_COMMAND_PROMPT));
}
/*
  functions to parse date/time as entered in the HIST command. Return seconds
  since 1-Jan-1970 as an integer;
*/
static char daytab[2][12] =
{
  {31,28,31,30,31,30,31,31,30,31,30,31},
  {31,29,31,30,31,30,31,31,30,31,30,31},
};
int32 odtParseTime(char *timeString)
{
  struct tm t;
  int32 rTime;
  int leap,days=0;
  short i;

  if((char *)strptime(timeString,"%D %H:%M:%S",&t) != NULL)
    {
      rTime = t.tm_sec+(SECS_PER_MINUTE*t.tm_min)+(SECS_PER_HOUR*(t.tm_hour-1));
      for(i = 1970;i < t.tm_year+1900;i++)
	{
	  leap = (i%4 == 0) && (i%100 != 0) || (i%400 == 0);
	  days += 365;
	  if(leap) days++;
	}
      leap = (i%4 == 0) && (i%100 != 0) || (i%400 == 0);
      for(i=0;i < t.tm_mon;i++) days += daytab[leap][i];
      days += (t.tm_mday-1);
#ifdef SOLARIS
      rTime += (days * SECS_PER_DAY);
#else
      rTime += (days * SECS_PER_DAY);
#endif
      return(rTime);
    }
  else
    return((int32)time(0));
}
