#include <d4all.h>
#include <time.h>
#include <ctype.h>
#include <fcntl.h>

#if !(defined(MIPSEL) || defined(AXP))
#include <memory.h>
#else
#include <sys/types.h>
#include <sys/stat.h>
#endif

#include "client.h"
#include "opNoteDb.h"
#include "opNoteDbServer.h"

static int	openDbServer (void);
static void	closeDbServer (void);

static CLIENT		*client;	/* the client control structure */
static ReqMsgUnion	reqMsg;
static OpNoteRecord	currentrec;
static int		dbserverOpened;


static int openDbServer (void)
{
    int status = 0;

    if (!client)
    {
	client = OpenServer (":opNoteDbServer", 1, NULL);
	if (client)
	{
	    status = 1;
	    dbserverOpened = 1;
	}
    }
    return (status);
}

static void closeDbServer (void)
{
    CloseSession (client);
}

void RelNotes (void)
{
    NoteMsgHeader	*relMsg = (NoteMsgHeader *)&reqMsg;
    NoteMsgRsp		*relRsp;

    if (dbserverOpened == 0)
	return;

    relMsg -> msgType = htons (OPNOTE_RELNOTES);
    relRsp = (NoteMsgRsp *)SendReq (client,
				    relMsg,
				    sizeof (relMsg -> msgType) );
    free ((void *)relRsp);
}

int CloseNotes (void)
{
    NoteMsgHeader	*closeMsg = (NoteMsgHeader *)&reqMsg;
    NoteMsgRsp		*closeRsp;
    int			result;

    if (dbserverOpened == 0)
	return (1);

    closeMsg -> msgType = htons (OPNOTE_CLOSENOTES);
    closeRsp = (NoteMsgRsp *)SendReq (client,
				      closeMsg,
				      sizeof (closeMsg -> msgType));
    if (closeRsp == NULL)
    {
	result = 0;
    }
    else
    {
	result = ntohs (closeRsp -> value);
	free ((void *)closeRsp);
    }
    closeDbServer ();
    return (result);
}
	

int openNotefile (char *dbfile, int create)
{
    OpenDatabaseMsg *openMsg = (OpenDatabaseMsg *)&reqMsg;
    NoteMsgRsp	*answer;
    int		result;

    if (dbserverOpened == 0)
    {
	/* open the opNote database server */
	if ( (openDbServer ()) == 0)
    	{
	    return (DBSERVERERR);
    	}

    }

    openMsg -> msgType = htons (OPNOTE_OPENDATABASE);
    openMsg -> create = htons (create);
    strncpy (openMsg -> dbfile, OPNOTEDATABASE, MAX_NOTEPATH);

    answer = (NoteMsgRsp *)SendReq (client, 
				    openMsg, 
				    sizeof (openMsg -> msgType) +
				    sizeof (openMsg -> create)  +
				    strlen (openMsg -> dbfile)  + 1);
    if (answer == NULL)
    {
	result = DBNOTESERR;
    }
    else
    {
	result = ntohs (answer -> value);
	free ((void *)answer);
    }
    return (result);
}


OpNoteRecord *GetNote ( Filter *filt, long time )
{
    FilterMsg		*filterMsg = (FilterMsg *)&reqMsg;
    OpNoteRecordMsg	*answer = NULL;
    OpNoteRecord	*opnoteRecord;
    time_t		msgTime;
    char		*pStr;

    memset ((void *)&currentrec, 0, sizeof (OpNoteRecord));
    filterMsg -> msgType = htons (OPNOTE_GETNOTE);
    filterMsg -> start   = htonl (filt -> start);
    filterMsg -> end     = htonl (filt -> end);
    filterMsg -> time    = htonl (time);
    /* concate the strings to save msg space */
    strncpy (filterMsg -> op, filt -> op, OP_SIZE);
    strncpy (filterMsg -> op + strlen (filt -> op) + 1,
	     filt -> keytype, KEY_SIZE);
    strncpy (filterMsg -> op + strlen (filt -> op) + 
	     strlen (filt -> keytype) + 2,
	     filt -> keyvalue, KEY_SIZE);
    strncpy (filterMsg -> op + strlen (filt -> op) + 
	     strlen (filt -> keytype) + strlen (filt -> keytype) + 3,
	     filt -> search_text, MAX_NOTE);

    answer = (OpNoteRecordMsg *)SendReq (client,
				         filterMsg,
				         sizeof (filterMsg -> msgType) +
				         sizeof (filterMsg -> pad)     +
				         sizeof (filterMsg -> start)   +
				         sizeof (filterMsg -> end)     +
				         sizeof (filterMsg -> time)    +
				         strlen (filt -> op)           +
				         strlen (filt -> keytype)      +
				         strlen (filt -> keyvalue)     +
				         strlen (filt -> search_text)  + 4);
    if (answer == NULL)
    {
	return (NULL);
    }
    else
    {
	/* since the time cannot be 0, this indicates no
	   record returned
	*/
	msgTime = ntohl (answer -> time);
	if (msgTime == 0)
	{
	    free ((void *)answer);
	    return (NULL);
	}
	currentrec.time = msgTime;
	/* retrieve strings */
	strncpy (currentrec.op, answer -> op, OP_SIZE);
	pStr =  answer -> op + (strlen (answer -> op) + 1);
	strncpy (currentrec.keytype, pStr, KEY_SIZE);
	pStr += strlen (currentrec.keytype) + 1;
	strncpy (currentrec.keyvalue, pStr, KEY_SIZE);
	pStr += strlen (currentrec.keyvalue) + 1;
	strncpy (currentrec.text, pStr, MAX_NOTE);
	free ((void *)answer);
	opnoteRecord = &currentrec;
	return (opnoteRecord);
    }
}


OpNoteRecord *NextNote ( Filter *filt, long time )
{
    FilterMsg		*filterMsg = (FilterMsg *)&reqMsg;
    OpNoteRecordMsg	*answer = NULL;
    OpNoteRecord	*opnoteRecord;
    char		*pStr;
    time_t		msgTime;

    memset ((void *)&currentrec, 0, sizeof (OpNoteRecord));
    filterMsg -> msgType = htons (OPNOTE_NEXTNOTE);
    filterMsg -> start   = htonl (filt -> start);
    filterMsg -> end     = htonl (filt -> end);
    filterMsg -> time    = htonl (time);
    /* concate the strings to save msg space */
    strncpy (filterMsg -> op, filt -> op, OP_SIZE);
    strncpy (filterMsg -> op + strlen (filt -> op) + 1,
	     filt -> keytype, KEY_SIZE);
    strncpy (filterMsg -> op + strlen (filt -> op) + 
	     strlen (filt -> keytype) + 2,
	     filt -> keyvalue, KEY_SIZE);
    strncpy (filterMsg -> op + strlen (filt -> op) + 
	     strlen (filt -> keytype) + strlen (filt -> keytype) + 3,
	     filt -> search_text, MAX_NOTE);

    answer = (OpNoteRecordMsg *)SendReq (client,
				         filterMsg,
				         sizeof (filterMsg -> msgType) +
				         sizeof (filterMsg -> pad)     +
				         sizeof (filterMsg -> start)   +
				         sizeof (filterMsg -> end)     +
				         sizeof (filterMsg -> time)    +
				         strlen (filt -> op)           +
				         strlen (filt -> keytype)      +
				         strlen (filt -> keyvalue)     +
				         strlen (filt -> search_text)  + 4);
    if (answer == NULL)
    {
	return (NULL);
    }
    else
    {
	/* since the time cannot be 0, this indicates no 
	   record returned
	*/
	msgTime = ntohl (answer -> time);
	if (msgTime == 0)
	{
	    free ((void *)answer);
	    return (NULL);
	}
	currentrec.time = msgTime;
	/* retrieve strings */
	strncpy (currentrec.op, answer -> op, OP_SIZE);
	pStr =  answer -> op + (strlen (answer -> op) + 1);
	strncpy (currentrec.keytype, pStr, KEY_SIZE);
	pStr += strlen (currentrec.keytype) + 1;
	strncpy (currentrec.keyvalue, pStr, KEY_SIZE);
	pStr += strlen (currentrec.keyvalue) + 1;
	strncpy (currentrec.text, pStr, MAX_NOTE);
	free ((void *)answer);
	opnoteRecord = &currentrec;
	return (opnoteRecord);
    }
}


int SaveNote ( OpNoteRecord *rec, char *dbfile )
{
    OpNoteSaveNoteMsg	*saveMsg = (OpNoteSaveNoteMsg *)&reqMsg;
    NoteMsgRsp		*answer;
    int			result;

    saveMsg -> msgType = htons (OPNOTE_SAVENOTE);
    saveMsg -> time    = htonl (rec -> time);
    strncpy (saveMsg -> op, rec -> op, OP_SIZE);
    strncpy (saveMsg -> op + strlen (rec -> op) + 1,
	     rec -> keytype, KEY_SIZE);
    strncpy (saveMsg -> op + strlen (rec -> op) + 
	     strlen (rec -> keytype) + 2,
	     rec -> keyvalue, KEY_SIZE);
    strncpy (saveMsg -> op + strlen (rec -> op) + 
	     strlen (rec -> keytype) + strlen (rec -> keyvalue) + 3,
	     rec -> text, MAX_NOTE);
    strncpy (saveMsg -> op + strlen (rec -> op) + 
	     strlen (rec -> keytype) + strlen (rec -> keyvalue) +
	     strlen (rec -> text) + 4, 
	     dbfile, strlen (dbfile));
    answer = (NoteMsgRsp *)SendReq (client,
				    saveMsg,
				    sizeof (saveMsg -> msgType) +
				    sizeof (saveMsg -> pad)     +
				    sizeof (saveMsg -> time)    +
				    strlen (rec -> op)          +
				    strlen (rec -> keytype)     +
				    strlen (rec -> keyvalue)    +
				    strlen (rec -> text)        +
				    strlen (dbfile)             + 5);
    if (answer == NULL)
    {
	return (0);
    }
    else
    {
	result = ntohs (answer -> value);
	free ((void *)answer);
	return (result);
    }
}


int DeleteNote ( OpNoteRecord *rec )
{
    OpNoteRecordMsg	*deleteMsg = (OpNoteRecordMsg *)&reqMsg;
    NoteMsgRsp		*answer;
    int			result;

    deleteMsg -> msgType = htons (OPNOTE_DELETENOTE);
    deleteMsg -> time    = htonl (rec -> time);
    strncpy (deleteMsg -> op, rec -> op, OP_SIZE);
    strncpy (deleteMsg -> op + strlen (rec -> op) + 1,
	     rec -> keytype, KEY_SIZE);
    strncpy (deleteMsg -> op + strlen (rec -> op) + 
	     strlen (rec -> keytype) + 2,
	     rec -> keyvalue, KEY_SIZE);
    strncpy (deleteMsg -> op + strlen (rec -> op) + 
	     strlen (rec -> keytype) + strlen (rec -> keyvalue) + 3,
	     rec -> text, MAX_NOTE);
    answer = (NoteMsgRsp *)SendReq (client,
				    deleteMsg,
				    sizeof (deleteMsg -> msgType) +
				    sizeof (deleteMsg -> pad)     +
				    sizeof (deleteMsg -> time)    +
				    strlen (rec -> op)            +
				    strlen (rec -> keytype)       +
				    strlen (rec -> keyvalue)      +
				    strlen (rec -> text)          + 4);
    if (answer == NULL)
    {
	return (0);
    }
    else
    {
	result = ntohs (answer -> value);
	free ((void *)answer);
	return (result);
    }
}


char **GetKeytypes (char *errmsg )
{
    KeyMsg		*getKeyMsg = (KeyMsg *)&reqMsg;
    KeyMsgRsp		*keyMsgRsp;
    char		**StringPtr = NULL;
    short		totKeys;
    short		blockKeys;
    short		currentKeyCount;
    int			firstMsg;
    int			StringIdx;
    int			i;
    char		*pos;

    getKeyMsg -> msgType = htons (OPNOTE_GETKEYTYPES);
    getKeyMsg -> start   = htons (1);
    keyMsgRsp = (KeyMsgRsp *)SendReq (client,
				      getKeyMsg,
				      sizeof (getKeyMsg -> msgType) +
				      sizeof (getKeyMsg -> start) );
    if (keyMsgRsp == NULL)
    {
	return (StringPtr);
    }
    totKeys = ntohs (keyMsgRsp -> totKeys);
    if (totKeys == 0)
    {
	if (keyMsgRsp -> errmsg[0] != 0)
	{
	    strncpy (errmsg, keyMsgRsp -> errmsg, MAXERRMSG);
	}
	free ((void *)keyMsgRsp);
	return (StringPtr);
    }
    else
    {
	currentKeyCount = 0;
	StringIdx = 0;
	firstMsg = 1;
	if ( (StringPtr = (char **)calloc (MAXKEYITEMS, sizeof (char *))) == NULL)
	{
	    sprintf (errmsg, "GetKey: Unable to alloc mem for key type\n");
	    free ((void *)keyMsgRsp);
	    return (StringPtr);
	}
	do
	{
	    if (firstMsg == 0)
	    {
		getKeyMsg -> msgType = htons (OPNOTE_GETKEYTYPES);
		getKeyMsg -> start   = 0;
		keyMsgRsp = (KeyMsgRsp *)SendReq (client,
					          getKeyMsg,
					          sizeof (getKeyMsg -> msgType) +
					          sizeof (getKeyMsg -> start) );
		if (keyMsgRsp == NULL)
		{
		    return (StringPtr);
		}
		else if (keyMsgRsp -> errmsg[0] != 0)
		{
		    strncpy (errmsg, keyMsgRsp -> errmsg, MAXERRMSG);
		    free ((void *)keyMsgRsp);
		    return (StringPtr);
		}
	    }
	    else
		firstMsg = 0;
	    blockKeys = ntohs (keyMsgRsp -> blockKeys);

	    for (i = 0, pos = keyMsgRsp -> block; i < blockKeys; i++)
	    {
		StringPtr[StringIdx] = (char *)malloc (strlen (pos) + 1);
		if (StringPtr[StringIdx])
		{
		    strcpy (StringPtr[StringIdx], pos);
		    StringIdx++;
		    pos += strlen (pos) + 1;
		}
		else
		{
		    sprintf (errmsg, "GetKey: Unable to alloc mem for key value %s\n", pos);
		    free ((void *)keyMsgRsp);
		    return (StringPtr);
		}
	    }
	    currentKeyCount += blockKeys;
	    free ((void *)keyMsgRsp);

	} while (currentKeyCount < totKeys);
    }

    return (StringPtr);
}


char **GetKeyValues ( char *keytype, char *errmsg )
{
    KeyMsg		*getKeyMsg = (KeyMsg *)&reqMsg;
    KeyMsgRsp		*keyMsgRsp;
    char		**StringPtr = NULL;
    short		totKeys;
    short		blockKeys;
    short		currentKeyCount;
    int			firstMsg;
    int			StringIdx;
    int			i;
    char		*pos;

    getKeyMsg -> msgType = htons (OPNOTE_GETKEYVALUES);
    getKeyMsg -> start   = htons (1);
    strncpy (getKeyMsg -> keytype, keytype, KEY_SIZE);
    keyMsgRsp = (KeyMsgRsp *)SendReq (client,
				      getKeyMsg,
				      sizeof (getKeyMsg -> msgType) +
				      sizeof (getKeyMsg -> start)   +
				      strlen (keytype)              + 1 );
    if (keyMsgRsp == NULL)
    {
	return (StringPtr);
    }
    totKeys = ntohs (keyMsgRsp -> totKeys);
    if (totKeys == 0)
    {
	if (keyMsgRsp -> errmsg[0] != 0)
	{
	    strncpy (errmsg, keyMsgRsp -> errmsg, MAXERRMSG);
	}
	free ((void *)keyMsgRsp);
	return (StringPtr);
    }
    else
    {
	currentKeyCount = 0;
	StringIdx = 0;
	firstMsg = 1;
	if ( (StringPtr = (char **)calloc (MAXKEYITEMS, sizeof (char *))) == NULL)
	{
	    sprintf (errmsg, "GetValue: Unable to alloc mem for key type\n");
	    free ((void *)keyMsgRsp);
	    return (StringPtr);
	}
	do
	{
	    if (firstMsg == 0)
	    {
		getKeyMsg -> msgType = htons (OPNOTE_GETKEYVALUES);
		getKeyMsg -> start   = 0;
		strncpy (getKeyMsg -> keytype, keytype, KEY_SIZE);	
		keyMsgRsp = (KeyMsgRsp *)SendReq (client,
					          getKeyMsg,
					          sizeof (getKeyMsg -> msgType) +
					          sizeof (getKeyMsg -> start)   +
						  strlen (keytype)              + 1);
		if (keyMsgRsp == NULL)
		{
		    return (StringPtr);
		}
		else if (keyMsgRsp -> errmsg[0] != 0)
		{
		    strncpy (errmsg, keyMsgRsp -> errmsg, MAXERRMSG);
		    free ((void *)keyMsgRsp);
		    return (StringPtr);
		}
	    }
	    else
		firstMsg = 0;
	    blockKeys = ntohs (keyMsgRsp -> blockKeys);

	    for (i = 0, pos = keyMsgRsp -> block; i < blockKeys; i++)
	    {
		StringPtr[StringIdx] = (char *)malloc (strlen (pos) + 1);
		if (StringPtr[StringIdx])
		{
		    strcpy (StringPtr[StringIdx], pos);
		    StringIdx++;
		    pos += strlen (pos) + 1;
		}
		else
		{
		    sprintf (errmsg, "GetValue: Unable to alloc mem for key value %s\n", pos);
		    free ((void *)keyMsgRsp);
		    return (StringPtr);
		}
	    }
	    currentKeyCount += blockKeys;
	    free ((void *)keyMsgRsp);

	} while (currentKeyCount < totKeys);
    }

    return (StringPtr);
}


int WriteKeyValue ( char *keytype, char *keyvalue, char *msg )
{
    KeyIOMsg	*writeKeyMsg = (KeyIOMsg *)&reqMsg;
    KeyIOMsgRsp	*writeMsgRsp;
    int		result;

    writeKeyMsg -> msgType = htons (OPNOTE_WRITEKEY);
    strncpy (writeKeyMsg -> keyType, keytype, KEY_SIZE);
    strncpy (writeKeyMsg -> keyValue, keyvalue, KEY_SIZE);
    writeMsgRsp = (KeyIOMsgRsp *)SendReq (client,
				          writeKeyMsg,
				          sizeof (writeKeyMsg -> msgType)  +
				          sizeof (writeKeyMsg -> keyType)  +
				          sizeof (writeKeyMsg -> keyValue) +
				          sizeof (writeKeyMsg -> errmsg) );
    if (writeMsgRsp == NULL)
    {
	return (0);
    }
    if (writeMsgRsp -> errmsg[0] != 0)
    {
	strncpy (msg, writeMsgRsp -> errmsg, MAXERRMSG);
	free ((void *)writeMsgRsp);
	return (0);
    }
    free ((void *)writeMsgRsp);
    result = ntohs (writeMsgRsp -> value);
    return (result);
}


int DeleteKeyValue ( char *keytype, char *keyvalue, char *msg)
{
    KeyIOMsg	*deleteKeyMsg = (KeyIOMsg *)&reqMsg;
    KeyIOMsgRsp	*deleteMsgRsp;
    int		result;

    deleteKeyMsg -> msgType = htons (OPNOTE_DELETEKEY);
    strncpy (deleteKeyMsg -> keyType, keytype, KEY_SIZE);
    strncpy (deleteKeyMsg -> keyValue, keyvalue, KEY_SIZE);
    deleteMsgRsp = (KeyIOMsgRsp *)SendReq (client,
				           deleteKeyMsg,
				           sizeof (deleteKeyMsg -> msgType)  +
				           sizeof (deleteKeyMsg -> keyType)  +
				           sizeof (deleteKeyMsg -> keyValue) +
				           sizeof (deleteKeyMsg -> errmsg) );
    if (deleteMsgRsp == NULL)
    {
	return (0);
    }
    if (deleteMsgRsp -> errmsg[0] != 0)
    {
	strncpy (msg, deleteMsgRsp -> errmsg, MAXERRMSG);
	free ((void *)deleteMsgRsp);
	return (0);
    }
    result = ntohs (deleteMsgRsp -> value);
    free ((void *)deleteMsgRsp);
    return (result);
}


void trimblanks (char *string) 
{
    int last;

    /* strip trailing blanks */
    for (last = strlen(string)-1; (last >= 0) && (string[last] == ' '); last--)
	string[last] = 0;
}

int	getPrivilege (char *privilege)
{
    /* return (OperPriv (&codeBaseDesc, privilege)); */
    return (1);
}
