#include <d4all.h>
#include <time.h>
#include <ctype.h>
#include <fcntl.h>

#if !(defined(MIPSEL) || defined(AXP))
#    include <memory.h>
#else
#    include <sys/types.h>
#    include <sys/stat.h>
#endif MIPSEL or AXP

#include "server.h"
#include "opNoteDb.h"
#include "opNoteDbServer.h"
#include "msgMon.h"



static SERVER		*server;
static RespMsgUnion	rspMsg;

/* these 3 are used by getKeys () */
static short		totKeys;
static int		listIdx;
static char		**list = NULL;

/* some function prototypes */

static int	openDatabase (ReqMsgUnion *reqMsg);
static int	getDbNote (ReqMsgUnion *reqMsg);
static int	saveDbNote (ReqMsgUnion *regMsg);
static int	deleteDbNote (ReqMsgUnion *reqMsg);
static int	getKeys (ReqMsgUnion *reqMsg);
static int	keyIO (ReqMsgUnion *reqMsg);
static int	relNotes (ReqMsgUnion *reqMsg);
static int	closeNotes (ReqMsgUnion *reqMsg);


int main (int argc, char *argv[])
{
    short	msgType;
    int		msgLen;
    ReqMsgUnion	*reqMsg;

    tisOpenLog ("opNoteDbServer", 6, 2, NULL, NULL);

    /* open server as async IO */
    if ((server = OpenTisService ("OpNote", 1, NULL)) == NULL)
    {
	tisSysLog (0,0, "OpNote dbServer, failed to start\n");
	exit (0);
    }

    tisSysLog (0,0, "opNote dbServer running\n");

#ifdef DEBUG
    SetSocketNum (server, 8002);
#endif DEBUG


    while (reqMsg = GetReq (server))
    {
	msgType = ntohs (((NoteMsgHeader *)reqMsg) -> msgType);
#ifdef DEBUG
	tisSysLog (0,0, "process request %d\n", msgType);
#endif
	switch (msgType)
	{
	    case OPNOTE_OPENDATABASE:
		msgLen = openDatabase (reqMsg);
		break;

	    case OPNOTE_GETNOTE:
		msgLen = getDbNote (reqMsg);
		break;

	    case OPNOTE_NEXTNOTE:
		msgLen = getDbNote (reqMsg);
		break;

	    case OPNOTE_SAVENOTE:
		msgLen = saveDbNote (reqMsg);
		break;

	    case OPNOTE_DELETENOTE:
		msgLen = deleteDbNote (reqMsg);
		break;

	    case OPNOTE_GETKEYTYPES:
		msgLen = getKeys (reqMsg);
		break;

	    case OPNOTE_GETKEYVALUES:
		msgLen = getKeys (reqMsg);
		break;

	    case OPNOTE_WRITEKEY:
		msgLen = keyIO (reqMsg);
		break;

	    case OPNOTE_DELETEKEY:
		msgLen = keyIO (reqMsg);
		break;

	    case OPNOTE_DELETEKEYTYPE:
		msgLen= keyIO (reqMsg);
		break;

	    case OPNOTE_RELNOTES:
		msgLen = relNotes (reqMsg);
		break;

	    case OPNOTE_CLOSENOTES:
		msgLen = closeNotes (reqMsg);
		break;

	    default:
		msgLen = -1;
		break;

	}  /* end switch (msgType) */

	if (msgLen > 0)
	{
	    SendRsp (server, (u_char *)&rspMsg, msgLen);
	    free ((void *)reqMsg);
	}

    } /* end while (GetReq()) */

#ifdef DEBUG
    tisSysLog (0,0, "null msg from GetReq\n");
#endif DEBUG
    /* cleanup */
    CloseSvcSession (server);
    tisSysLog (0,0, "opNote server exiting\n");
    closelog ();
}


static int	openDatabase (ReqMsgUnion *reqMsg)
{
    OpenDatabaseMsg	*openMsg = (OpenDatabaseMsg *)reqMsg;
    NoteMsgRsp		*openRsp = (NoteMsgRsp *)&rspMsg;
    int			create;
    int			result;
    int 		msgLen;
    char		dbfile[MAX_NOTEPATH];

    create = (signed short)(ntohs (openMsg -> create));
    strncpy (dbfile, openMsg -> dbfile, MAX_NOTEPATH);	

    result = openNotefile (dbfile, create);

    openRsp -> msgType = htons (OPNOTE_OPENDATABASE);
    openRsp -> value   = htons (result);
    msgLen             = sizeof (openRsp -> msgType) +
			 sizeof (openRsp -> value);
    return (msgLen);
}


static int	getDbNote (ReqMsgUnion *reqMsg)
{
    FilterMsg		*filterMsg = (FilterMsg *)reqMsg;
    OpNoteRecordMsg	*recMsg    = (OpNoteRecordMsg *)&rspMsg;
    OpNoteRecord	*opnoteRec;
    Filter		filter;
    time_t		time;
    int			msgLen;
    int			whichnote;
    char		*pStr;

    memset ((char *)&filter, 0, sizeof (filter));
    whichnote    = ntohs (filterMsg -> msgType);
    time         = ntohl (filterMsg -> time);
    filter.start = ntohl (filterMsg -> start);
    filter.end   = ntohl (filterMsg -> end);
    pStr = filterMsg -> op;
    strncpy (filter.op, pStr, OP_SIZE);
    pStr += (strlen (filter.op) + 1);
    strncpy (filter.keytype, pStr, KEY_SIZE);
    pStr += (strlen (filter.keytype) + 1);
    strncpy (filter.keyvalue, pStr, KEY_SIZE);
    pStr += (strlen (filter.keyvalue) + 1);
    strncpy (filter.search_text, pStr, MAX_NOTE);

    if (whichnote == OPNOTE_GETNOTE)
    {
	opnoteRec = GetNote (&filter, time);
	recMsg -> msgType = htons (OPNOTE_GETNOTE);
    }
    else 
    {
	opnoteRec = NextNote (&filter, time);
	recMsg -> msgType = htons (OPNOTE_NEXTNOTE);
    }

    if (opnoteRec != NULL)
    {
	recMsg -> time = htonl (opnoteRec -> time);
	strncpy (recMsg -> op, opnoteRec -> op, OP_SIZE);
	strncpy (recMsg -> op + strlen (opnoteRec -> op) + 1,
		 opnoteRec -> keytype, KEY_SIZE);
	strncpy (recMsg -> op + strlen (opnoteRec -> op) +
		 strlen (opnoteRec -> keytype) + 2,
		 opnoteRec -> keyvalue, KEY_SIZE);
	strncpy (recMsg -> op + strlen (opnoteRec -> op) +
		 strlen (opnoteRec -> keytype) + 
		 strlen (opnoteRec -> keyvalue) + 3,
		 opnoteRec -> text, MAX_NOTE);
	msgLen = sizeof (recMsg -> msgType)     +
		 sizeof (recMsg -> pad)         +
		 sizeof (recMsg -> time)        +
		 strlen (opnoteRec -> op)       +
		 strlen (opnoteRec -> keytype)  +
		 strlen (opnoteRec -> keyvalue) +
		 strlen (opnoteRec -> text)     + 4;
    }
    else
    {
	recMsg -> time = 0;
	msgLen = sizeof (recMsg -> msgType) +
		 sizeof (recMsg -> pad)     +
		 sizeof (recMsg -> time);
    }

    return (msgLen);
}


static int	saveDbNote (ReqMsgUnion *reqMsg)
{
    OpNoteSaveNoteMsg	*saveMsg = (OpNoteSaveNoteMsg *)reqMsg;
    NoteMsgRsp		*saveRsp = (NoteMsgRsp *)&rspMsg;
    OpNoteRecord	opnoteRec;
    char		*pStr;
    char		dbfile[MAX_NOTEPATH];
    int			result;
    int			msgLen;

    memset ((char *)&opnoteRec, 0, sizeof (opnoteRec));
    opnoteRec.time = ntohl (saveMsg -> time);
    pStr = saveMsg -> op;
    strncpy (opnoteRec.op, pStr, OP_SIZE);
    pStr += (strlen (opnoteRec.op) + 1);
    strncpy (opnoteRec.keytype, pStr, KEY_SIZE);
    pStr += (strlen (opnoteRec.keytype) + 1);
    strncpy (opnoteRec.keyvalue, pStr, KEY_SIZE);
    pStr += (strlen (opnoteRec.keyvalue) + 1);
    strncpy (opnoteRec.text, pStr, MAX_NOTE);
    pStr += (strlen (opnoteRec.text) + 1);
    strncpy (dbfile, pStr, MAX_NOTEPATH);

    result = SaveNote (&opnoteRec, dbfile);

    saveRsp -> msgType = htons (OPNOTE_SAVENOTE);
    saveRsp -> value   = htons (result);
    msgLen = sizeof (saveRsp -> msgType) +
	     sizeof (saveRsp -> value);
    return (msgLen);
}



static int	deleteDbNote (ReqMsgUnion *reqMsg)
{
    OpNoteRecordMsg	*deleteMsg = (OpNoteRecordMsg *)reqMsg;
    NoteMsgRsp		*deleteRsp = (NoteMsgRsp *)&rspMsg;
    OpNoteRecord	opnoteRec;
    char		*pStr;
    int			result;
    int			msgLen;

    memset ((char *)&opnoteRec, 0, sizeof (opnoteRec));
    opnoteRec.time = ntohl (deleteMsg -> time);
    pStr = deleteMsg -> op;
    strncpy (opnoteRec.op, pStr, OP_SIZE);
    pStr += (strlen (opnoteRec.op) + 1);
    strncpy (opnoteRec.keytype, pStr, KEY_SIZE);
    pStr += (strlen (opnoteRec.keytype) + 1);
    strncpy (opnoteRec.keyvalue, pStr, KEY_SIZE);
    pStr += (strlen (opnoteRec.keyvalue) + 1);
    strncpy (opnoteRec.text, pStr, MAX_NOTE);

    result = DeleteNote (&opnoteRec);

    deleteRsp -> msgType = htons (OPNOTE_DELETENOTE);
    deleteRsp -> value   = htons (result);
    msgLen = sizeof (deleteRsp -> msgType) +
	     sizeof (deleteRsp -> value);
    return (msgLen);
}


static int	getKeys (ReqMsgUnion *reqMsg)
{
    KeyMsg		*keyMsg = (KeyMsg *)reqMsg;
    KeyMsgRsp		*keyRsp = (KeyMsgRsp *)&rspMsg;
    short		start;
    char		errmsg[MAXERRMSG + 1];
    char		keytype [KEY_SIZE + 1];
    char		*pos;
    char		*lastpos;
    int			msgLen;
    int			whichkey;
    short		blockKeys;
    short		lenString;

    memset (errmsg, 0, sizeof (errmsg));
    whichkey = ntohs (keyMsg -> msgType);
    start = ntohs (keyMsg -> start);
    if (start == 1)
    {
	if (list != NULL)
	{
	    for (listIdx = 0; list[listIdx]; listIdx++)
		free ((void *)list[listIdx]);
	    free ((void *)list);
	    list = NULL;
	}
	totKeys = 0;
	if (whichkey == OPNOTE_GETKEYTYPES)
	{
	    list = GetKeytypes (errmsg);
	    keyRsp -> msgType = htons (OPNOTE_GETKEYTYPES);
	}
	else
	{
	    strncpy (keytype, keyMsg -> keytype, KEY_SIZE);
	    list = GetKeyValues (keytype, errmsg);
	    keyRsp -> msgType = htons (OPNOTE_GETKEYVALUES);
	}
	if (list == NULL)
	{
	    keyRsp -> totKeys = 0;
	    keyRsp -> blockKeys = 0;
	    if (errmsg[0] != 0)
	    {
		strncpy (keyRsp -> errmsg, errmsg, MAXERRMSG);
	    }
	    msgLen = sizeof (keyRsp -> msgType) +
		     sizeof (keyRsp -> totKeys) +
		     sizeof (keyRsp -> blockKeys) +
		     strlen (errmsg) + 1;
	    return (msgLen);
	}
	/* NOTE: intentional ; at end of for(), just a simple 
		 counting statement
	*/
	for (totKeys = 0; list[totKeys]; totKeys++);

	keyRsp -> totKeys = htons (totKeys);
	listIdx = 0;
    }
    strncpy (keyRsp -> errmsg, errmsg, MAXERRMSG);
    memset (keyRsp -> block, 0, sizeof (keyRsp -> block));
    lastpos = keyRsp -> block + MAXBLOCK_SIZE;
    pos = keyRsp -> block;
    blockKeys = 0;
    msgLen = sizeof (keyRsp -> msgType)   +
	     sizeof (keyRsp -> totKeys)   +
	     sizeof (keyRsp -> blockKeys) +
	     sizeof (keyRsp -> errmsg);

    for (; listIdx < totKeys; listIdx++)
    {
	lenString = strlen (list[listIdx]);
	if ( (pos + lenString + 1) >= lastpos)
	    break;
	blockKeys++;
	strncpy (pos, list[listIdx], KEY_SIZE);
	pos += lenString + 1;
	msgLen += lenString + 1;
    }
    keyRsp -> blockKeys = htons (blockKeys);
    return (msgLen);
}


static int	keyIO (ReqMsgUnion *reqMsg)
{
    KeyIOMsg	*keyIOMsg = (KeyIOMsg *)reqMsg;
    KeyIOMsgRsp	*keyIORsp = (KeyIOMsgRsp *)&rspMsg;
    char	keytype [KEY_SIZE + 1];
    char	keyvalue [KEY_SIZE + 1];
    char	errmsg [MAXERRMSG + 1];
    int		result;
    int		msgLen;
    int		whichio;

    whichio = ntohs (keyIOMsg -> msgType);
    strncpy (keytype, keyIOMsg -> keyType, KEY_SIZE);
    strncpy (keyvalue, keyIOMsg -> keyValue, KEY_SIZE);
    memset (errmsg, 0, sizeof (errmsg));

    if (whichio == OPNOTE_WRITEKEY)
    {
	result = WriteKeyValue (keytype, keyvalue, errmsg);
	keyIORsp -> msgType = htons (OPNOTE_WRITEKEY);
    }
    else if (whichio == OPNOTE_DELETEKEY)
    {
	result = DeleteKeyValue (keytype, keyvalue, errmsg);
	keyIORsp -> msgType = htons (OPNOTE_DELETEKEY);
    }
    else
    {
	result = DeleteKeyType (keytype, errmsg);
	keyIORsp -> msgType = htons (OPNOTE_DELETEKEYTYPE);
    }
    keyIORsp -> value = htons (result);
    strncpy (keyIORsp -> errmsg, errmsg, MAXERRMSG);
    msgLen = sizeof (keyIORsp -> msgType) +
	     sizeof (keyIORsp -> value)   +
	     strlen (keyIORsp -> errmsg) + 1;
    return (msgLen);
}


static int	relNotes (ReqMsgUnion *reqMsg)
{
    NoteMsgRsp		*relRsp = (NoteMsgRsp *)&rspMsg;
    int			msgLen;

    RelNotes ();
    relRsp -> msgType = htons (OPNOTE_RELNOTES);
    relRsp -> value   = htons (1);
    msgLen = sizeof (relRsp -> msgType) +
	     sizeof (relRsp -> value);
    return (msgLen);
}

static int	closeNotes (ReqMsgUnion *reqMsg)
{
    NoteMsgRsp		*closeRsp = (NoteMsgRsp *)&rspMsg;
    int			msgLen;
    int			result;

    result = CloseNotes ();
    closeRsp -> msgType = htons (OPNOTE_CLOSENOTES);
    closeRsp -> value   = htons (result);
    msgLen = sizeof (closeRsp -> msgType) +
	     sizeof (closeRsp -> value);
    return (msgLen);
}

