#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vmenuHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vmenubarHEADER
#include vbuttonHEADER
#include vtextitemHEADER
#include vstartupHEADER
#include vtimeHEADER
#include vmemHEADER
#include vcontrolHEADER
#include vtimerHEADER
extern "C"
{
#include "opNoteDb.h"
#include "OperPriv.h"
}
#include "opno.h"
#include "opNoteUtils.hxx"
#include "opNote.hxx"


vkindTYPED_FULL_DEFINITION(opNoteMainDialog, vdialog, NULL);

/*******************************************************************
*
*    opNoteMainDialog override for constructors init ()
*
*******************************************************************/

void opNoteMainDialog::LoadInit (vresource res)
{
    int		i;

    // store the main dialog's resource
    mainDialogResource = res;

    // NULL pointers to menu dialogs
    searchDialog = NULL;
    editDialog   = NULL;
    keyDialog    = NULL;
    tisAbout	 = NULL;

    loadInitErrors = OPNO_SUCCESS;

    // RequestSubLoad returns ptr to uninited dialog object
    // gets init'd when dialog gets init'd by LoadInit()
    vmenubar *menubar = vmenubar::RequestSubLoad (this,
					opno_mainMenuBarTag);
    // File menu
    vmenu *fileMenu = vmenu::RequestSubLoad (menubar,
					opno_fileMenuTag);
    vmenuItem *exitItem = vmenuItem::RequestSubLoad (fileMenu,
				    	opno_exitMenuItemTag);

    // Search menu
    vmenu *searchMenu = vmenu::RequestSubLoad (menubar,
					opno_searchMenuTag);
    vmenuItem *searchItem = vmenuItem::RequestSubLoad (searchMenu,
				    	opno_searchMenuItemTag);

    // Edit menu
    vmenu *editMenu = vmenu::RequestSubLoad (menubar,
					opno_editMenuTag);
    vmenuItem *newMenuItem = vmenuItem::RequestSubLoad (editMenu,
				    	opno_newMenuItemTag);
    vmenuItem *modifyMenuItem = vmenuItem::RequestSubLoad (editMenu,
					opno_modifyMenuItemTag);
    vmenuItem *deleteMenuItem = vmenuItem::RequestSubLoad (editMenu,
					opno_deleteMenuItemTag);
    vmenuItem *keyMenuItem = vmenuItem::RequestSubLoad (editMenu,
					opno_keyMenuItemTag);
    mainTextItem = opNoteMainTextItem::RequestSubLoad (this,
					opno_mainDisplayTextTag);

    // Help menu
    helpMenu = vmenu::RequestSubLoad (menubar, opno_helpMenuTag);

    // init inherited data
    vdialog::LoadInit (res);

    // init privilege checking, see .../share/src/libtis/OperPrvtrk.c
    if ((InitOperPriv (NULL)) != PRIVOK)
    {
	opnoMessageCenter (NULL, OPNO_INFO_NOTICE, opno_InitOperPrivErr);
	loadInitErrors = OPNO_ERROR;
	return;
    }

    // do the Search menu
    if ((OperPriv (NULL, "*MMI_OPNOTE_SEARCH_SEARCH")) == 0)
    {
	searchItem -> Disable ();
	// since only 1 item in the menu, disable menu also
	searchMenu -> Disable ();
    }

    // do the Edit menu
    int	allDisabled = 0;
    if ((OperPriv (NULL, "*MMI_OPNOTE_EDIT_NEWNOTE")) == 0)
    {
	newMenuItem -> Disable ();
	allDisabled++;
    }
    if ((OperPriv (NULL, "*MMI_OPNOTE_EDIT_MODIFYNOTE")) == 0)
    {
	modifyMenuItem -> Disable ();
	allDisabled++;
    }
    if ((OperPriv (NULL, "*MMI_OPNOTE_EDIT_DELETENOTE")) == 0)
    {
	deleteMenuItem -> Disable ();
	allDisabled++;
    }
    if ((OperPriv (NULL, "*MMI_OPNOTE_EDIT_KEY")) == 0)
    {
	keyMenuItem -> Disable ();
	allDisabled++;
    }
    if (allDisabled == 4)
    {
	editMenu -> Disable ();
    }
    // close privilege checking, see .../share/src/libtis/OperPrvtrk.c
    CloseOperPriv ();

    // store pointer to main dialog in text item
    mainTextItem -> storeMainDialog (this);

    mainTextItem -> opnoteTimer = new opNoteTimer;
    mainTextItem -> opnoteTimer -> StoreMainText (mainTextItem);

    // store internal names for the main menu tags
    commandNames[EXIT_MENUITEM]   = opno_exitMenuItemTag;
    commandNames[SEARCH_MENUITEM] = opno_searchMenuItemTag;
    commandNames[NEW_MENUITEM]    = opno_newMenuItemTag;
    commandNames[MODIFY_MENUITEM] = opno_modifyMenuItemTag;
    commandNames[DELETE_MENUITEM] = opno_deleteMenuItemTag;
    commandNames[KEY_MENUITEM]    = opno_keyMenuItemTag;

    // create a command space
    vcommandSpace *space = new vcommandSpace;

    // create a dictionary with a default hash size and hash function
    vdict *dict = new vdict (vnameHash);

    // populate our dictionary with menu commands
    for (i = EXIT_MENUITEM; i < MENUITEM_COUNT; i++)
    {
	// get an instance of our subclassed selector command
	opNoteMainCommand *command = new opNoteMainCommand;
	// set the commands selector value
	command -> SetSelector(i);
	// store command in the dictionary using the 
	// intern'd tag name
	dict -> SetElem (commandNames[i], command);
    }
    // add dictionary to the command space and give the command
    // space ownership of the dictionary, vname_Command defines
    // the type attribute of the things in the dictionary
    space -> AddDictOwned (vname_Command, dict);

    SetSpaceOwned (space);
    Place (GetRoot (), vrectPLACE_LEFT, vrectPLACE_TOP);
}


/*******************************************************************
*
*    opNoteMainDialog Destroy function
*	first destroy other dialogs if they have been used
*
*******************************************************************/

void opNoteMainDialog::Destroy (void)
{
    if (tisAbout != NULL)
    {
	delete tisAbout;
    }
    if (searchDialog != NULL)
    {
	delete searchDialog;
	searchDialog = NULL;
    }
    if (editDialog != NULL)
    {
	delete editDialog;
	editDialog = NULL;
    }
    if (keyDialog != NULL)
    {
	delete keyDialog;
	keyDialog = NULL;
    }
    vdialog::Destroy ();
}


/*******************************************************************
*
*    opNoteMainDialog notify function
*
*******************************************************************/

void opNoteMainDialog::ObserveWindow (vevent *event)
{
    if ( event -> GetType () == veventWINDOW_STATE && event -> IsClose () )
	vevent::StopProcessing ();
}


/*******************************************************************
*
*    opNoteMainDialog function to disable menu items based on the
*    current operators privileges
*
*******************************************************************/

void opNoteMainDialog::OpenMain (opNoteApplication *App)
{
    opNoteApp = App;

    BuildAboutMenu ();

    // add so opNote will iconize in toolbar under NT
    AddStyle (vwindowSTYLE_GLOBAL);
    vdialog::Open ();

    // display any opnotes
    mainTextItem -> redisplayData ();
    // start timer to refresh main text item display
    mainTextItem -> opnoteTimer -> StartMainTimer ();
}


/*******************************************************************
*
*    opNoteMainDialog function to build the about menu and attach
*    it to the help menu
*
*******************************************************************/

void opNoteMainDialog::BuildAboutMenu (void)
{
    vstr *verMsg = vresourceGetString (opNoteApp -> getMessagesResource(),
				       opno_VersionMsg);
    tisAbout = new TisAbout (opNoteApp -> getGalaxyRes (), verMsg);
    vstrDestroy (verMsg);
    vstr *aboutTitle = vresourceGetString (opNoteApp -> getTitlesResource (),
				      opno_AboutTitle);
    tisAbout -> SetAboutTitle (aboutTitle);
    vstrDestroy (aboutTitle);
    AboutMenuItem *aboutItem = tisAbout -> GetAboutMenuItem ();
    helpMenu -> AppendItem (aboutItem);
}


vkindTYPED_FULL_DEFINITION(opNoteMainCommand, vcommandSelector, NULL);

/*******************************************************************
*
*    opNoteMainCommand menu selector commands
*
*******************************************************************/

int	opNoteMainCommand::Issue (vdict *dict)
{
    opNoteApplication	*opNoteApp;
    opNoteMainDialog	*mainDialog;
    opNoteMainTextItem	*mainTextItem;
    opNoteSearchDialog	*searchDialog;
    opNoteEditDialog	*editDialog;
    opNoteKeyDialog	*keyDialog;
    OpNoteRecord	*currentRecord;

    mainDialog = (opNoteMainDialog *) dict -> GetElem (vname_Dialog);
    opNoteApp = mainDialog -> getApplication ();
    mainTextItem = mainDialog -> getMainTextItem ();

    switch (GetSelector ())
    {
	case EXIT_MENUITEM:
	    vevent::StopProcessing ();
	    break;

	case SEARCH_MENUITEM:
	    searchDialog = mainDialog -> getSearchDialog ();
	    if (searchDialog == NULL)
	    {
    		vresource searchDialogResource = vresourceGet ( opNoteApp -> getOpNoteResource(), 
								opno_searchDialog);
		searchDialog = new opNoteSearchDialog (searchDialogResource);
		searchDialog -> storeSearchDialogResource (searchDialogResource);
		mainDialog -> storeSearchDialog (searchDialog);
	    }
	    searchDialog -> startOpNoteSearch ();
	    if ((searchDialog -> getSearchDialogStatus ()) == SEARCH_OK)
	    {
		Filter	filter;
		// get new filter specs
		searchDialog -> updateSearchFilter (&filter);
		opNoteApp -> updateCurrentFilter (&filter);
		mainTextItem -> redisplayData ();
		opNoteApp -> setIsSelectedRecord (vFALSE);
	    }
	    break;

	case NEW_MENUITEM:
	    editDialog = mainDialog -> getEditDialog ();
	    if (editDialog == NULL)
	    {
		vresource editDialogResource = vresourceGet (opNoteApp -> getOpNoteResource(), 
							     opno_editDialog);
		editDialog = new opNoteEditDialog (editDialogResource);
		editDialog -> storeEditDialogResource (editDialogResource);
		editDialog -> storeOpNoteApp (opNoteApp);
		mainDialog -> storeEditDialog (editDialog);
	    }
	    currentRecord = opNoteApp -> getCurrentSelectedRecord ();
	    editDialog -> startOpNoteEdit (NULL);
	    if ( (editDialog -> getEditDialogStatus ()) == EDIT_OK)
	    {
		OpNoteRecord    newRecord;

		editDialog -> getEditedRecord (&newRecord);
		SaveNote (&newRecord, OPNOTEDATABASE);
		mainTextItem -> redisplayData ();
		opNoteApp -> setIsSelectedRecord (vFALSE);
	    }
	    break;

	case MODIFY_MENUITEM:
	    editDialog = mainDialog -> getEditDialog ();
	    if ( (opNoteApp -> getIsSelectedRecord ()) == vFALSE)
	    {
		opnoMessageCenter (NULL, OPNO_INFO_NOTICE, opno_NothingSelectedMsg);
	    }
	    else
	    {
		if (editDialog == NULL)
		{
		    vresource editDialogResource = vresourceGet (opNoteApp -> getOpNoteResource(), 
								 opno_editDialog);
		    editDialog = new opNoteEditDialog (editDialogResource);
		    editDialog -> storeEditDialogResource (editDialogResource);
		    editDialog -> storeOpNoteApp (opNoteApp);
		    mainDialog -> storeEditDialog (editDialog);
		}
		currentRecord = opNoteApp -> getCurrentSelectedRecord ();
		editDialog -> startOpNoteEdit (currentRecord);
		if ( (editDialog -> getEditDialogStatus ()) == EDIT_OK)
		{
		    OpNoteRecord    modifiedRecord;

		    editDialog -> getEditedRecord (&modifiedRecord);
		    SaveNote (&modifiedRecord, OPNOTEDATABASE);
		    mainTextItem -> redisplayData ();
		    opNoteApp -> setIsSelectedRecord (vFALSE);
		}
	    }
	    break;

	case DELETE_MENUITEM:
	    if ( (opNoteApp -> getIsSelectedRecord ()) == vFALSE)
	    {
		opnoMessageCenter (NULL, OPNO_INFO_NOTICE, opno_NothingSelectedMsg);
	    }
	    else
	    {
		currentRecord = opNoteApp -> getCurrentSelectedRecord ();
		DeleteNote (currentRecord);
		mainTextItem -> redisplayData ();
		opNoteApp -> setIsSelectedRecord (vFALSE);
	    }
	    break;

	case KEY_MENUITEM:
	    keyDialog = mainDialog -> getKeyDialog ();
	    if (keyDialog == NULL)
	    {
		vresource keyDialogResource = vresourceGet (opNoteApp -> getOpNoteResource(), 
							     opno_keyDialog);
		keyDialog = new opNoteKeyDialog (keyDialogResource);
		keyDialog -> storeKeyDialogResource (keyDialogResource);
		mainDialog -> storeKeyDialog (keyDialog);
	    }
	    keyDialog -> startOpNoteKey ();
	    break;

	default:
	    return (vFALSE);
    }
    return (vTRUE);
}


vkindTYPED_FULL_DEFINITION(opNoteMainTextItem, vtextitem, NULL)

/*******************************************************************
*
*    opNoteMainTextItem observer function
*    the default 'this' should be a pointer to the vtextitem
*
*******************************************************************/

void opNoteMainTextItem::ObserveView(vevent *event, int message)
{
    if (message == vtextviewBUTTON_UP_NOTIFY)
    {
	setCurrentSelection ();
    }
}


/*******************************************************************
*
*    opNoteMainTextItem function 
*	given the currFilter in opNoteApp, display the data 
*	from /tis4000db/opnote.dbf
*	GetNote() and NextNote() are in opNoteDb.c
*
*******************************************************************/

void opNoteMainTextItem::redisplayData (void)
{
    OpNoteRecord	currRec;
    OpNoteRecord	*rp = NULL;
    Filter		currentFilter;
    struct tm		*tm_block;
    char		dspmsg[3000];
    char		msgBreak[128];
    char		*text;
    int			memLeft;
    int			strlenMsgBrk;
    int			strlenMsg;
    opNoteApplication	*app;


    text = (char *)vmemAllocAndClear (32000);
    memLeft = 32000;
    app = mainDialog -> getApplication ();
    app -> getCurrentFilter (&currentFilter);
    if (rp = GetNote (&currentFilter, 0))
    {
	SetTextScribed (vcharScribeSystem (""));
	currRec = *rp;
	tm_block = localtime (&currRec.time);
	sprintf (dspmsg, "%02d/%02d/%4d %02d:%02d:%02d %s %s %s\n%s\n",
		       tm_block -> tm_mon + 1,
		       tm_block -> tm_mday,
		       tm_block -> tm_year + 1900,
		       tm_block -> tm_hour,
		       tm_block -> tm_min,
		       tm_block -> tm_sec,
		       currRec.op,
		       currRec.keytype,
		       currRec.keyvalue,
		       currRec.text);
	strcat (text, dspmsg);
	memLeft -= strlen (dspmsg);
	strcpy (msgBreak, "\n------------------\n\n");
	strlenMsgBrk = strlen (msgBreak);
	while (rp = NextNote (&currentFilter, currRec.time))
	{
	    currRec = *rp;
	    tm_block = localtime (&currRec.time);
	    if (strlenMsgBrk >= memLeft)
	    {
		printf ("no more memory to display messages\n");
		break;
	    }
	    strcat (text, msgBreak);
	    memLeft -= strlenMsgBrk;
	    sprintf (dspmsg, "%02d/%02d/%4d %02d:%02d:%02d %s %s %s\n%s\n",
			     tm_block -> tm_mon + 1,
			     tm_block -> tm_mday,
			     tm_block -> tm_year + 1900,
			     tm_block -> tm_hour,
			     tm_block -> tm_min,
			     tm_block -> tm_sec,
			     currRec.op,
			     currRec.keytype,
			     currRec.keyvalue,
			     currRec.text);
	    strlenMsg = strlen (dspmsg);
	    if (strlenMsg >= memLeft)
	    {
		printf ("no more memory to display messages\n");
		break;
	    }
	    strcat (text, dspmsg);
	    memLeft -= strlenMsg;
	}
	SetTextScribed (vcharScribeSystem (text));
    }
    vmemFree ((void *)text);
}


/*******************************************************************
*
*    opNoteMainTextItem function to get the current selected 
*    text, called from ObserveView, with ptr to the vtextitem pass
*    through as the default 'this'
*
*******************************************************************/

void opNoteMainTextItem::setCurrentSelection (void)
{
    vtext		*text;
    vtextSelection	*currSelection;
    vchar		*startLineText;
    long		selectedLine;
    long		previousLine;
    long		nextLine;
    long		startLinePosition;
    long		startLine;
    long		endLine;
    long		lastLine;
    opNoteApplication	*app;

    app = mainDialog -> getApplication ();
    /* get the current text data object that this textview 
       is displaying
    */
    text = GetTextData ();

    /* get the current selected area */
    currSelection = GetSelection ();

    /* get the line number associated with this selection */
    selectedLine = text -> CurrLineNum (currSelection);

    /* get the last line number in the text object */
    lastLine = text -> GetLineCount ();

    /* lines start numbering at 1 in the text object */
    if (selectedLine == 1)
    {
	startLine = selectedLine;
    }
    else
    {
	previousLine = selectedLine;
	while (1)
	{
	    if (previousLine == 1)
	    {
		startLine = previousLine;
		break;
	    }
	    if (previousLine == lastLine)
	    {
		previousLine--;
		continue;
	    }
	    /* get the position in the text object for the start
	       of this line
	    */
	    startLinePosition = text -> GetLinePosition (previousLine);
	    /* given a position in the text object, get a ptr 
	       to the text
	    */
	    startLineText = text -> GetTextAt (startLinePosition);
	    if ( (vcharCompareBounded (startLineText, "------", 6)) == 0)
	    {
		startLine = previousLine + 2;
		break;
	    }
		
	    previousLine--;
	}
    }
    if (selectedLine == lastLine)
    {
	endLine = selectedLine;
    }
    else
    {
	nextLine = selectedLine;
	while (1)
	{
	    if (nextLine == lastLine)
	    {
		endLine = nextLine;
		break;
	    }
	    startLinePosition = text -> GetLinePosition (nextLine);
	    startLineText = text -> GetTextAt (startLinePosition);
	    if ( (vcharCompareBounded (startLineText, "------", 6)) == 0)
	    {
		endLine = nextLine - 2;
		break;
	    }
	    nextLine++;
	}
    }
    if (updateCurrentSelectedRecord (text, startLine))
    {
	text -> SelectLines (currSelection, startLine, endLine, vtextLINE_ALL);
	app -> setIsSelectedRecord (vTRUE);
    }
}


/*******************************************************************
*
*    opNoteMainTextItem function called from setCurrentSelection()
*    given the arguments, set the currentSelectedRecord in the app
*
*******************************************************************/

int opNoteMainTextItem::updateCurrentSelectedRecord (vtext *text, long startLine)
{
    long		lineNumber;
    long		startLinePosition;
    vchar		*startLineText;
    int			month = 0;
    int			day = 0;
    int			year = 0;
    int			hour = 0;
    int			minutes = 0;
    int			seconds = 0;
    int			status;
    long		time;
    char		oper[OP_SIZE + 1];
    char		keyClass[KEY_SIZE + 1];
    char		keyValue[KEY_SIZE + 1];
    struct tm		tim;
    OpNoteRecord	*rec = NULL;
    Filter		filt;
    opNoteApplication	*app;

    app = mainDialog -> getApplication ();
    vmemSet ((void *)&oper[0], 0, sizeof (oper));
    vmemSet ((void *)&keyClass[0], 0, sizeof (keyClass));
    vmemSet ((void *)&keyValue[0], 0, sizeof (keyValue));
    lineNumber = startLine;
    startLinePosition = text -> GetLinePosition (lineNumber);
    startLineText = text -> GetTextAt (startLinePosition);
    sscanf ((char *)startLineText, "%02d/%02d/%4d %02d:%02d:%02d %s %s %s\n",
				   &month, &day, &year, &hour, &minutes, &seconds,
				   oper, keyClass, keyValue);
    tim.tm_sec	= seconds;
    tim.tm_min	= minutes;
    tim.tm_hour	= hour;
    tim.tm_mday	= day;
    tim.tm_mon	= month - 1;
    tim.tm_year	= year - 1900;
    tim.tm_isdst = 0;

#if (defined(MIPSEL) || defined(SOLARIS) || defined(AXP) || defined(WIN32))
    time = mktime (&tim);
#else
    time = timelocal (&tim);
#endif
    // daylight savings time is not known from text on screen, but mktime does
    // so if it is set, redo mktime/localtime with same params but set isdst
    if (tim.tm_isdst)
    {
	tim.tm_sec  = seconds;
	tim.tm_min  = minutes;
	tim.tm_hour = hour;
	tim.tm_mday = day;
	tim.tm_mon  = month - 1;
	tim.tm_year = year - 1900;
	tim.tm_isdst = 1;
#if (defined(MIPSEL) || defined(SOLARIS) || defined(AXP) || defined(WIN32))
	time = mktime (&tim);
#else
	time = timelocal (&tim);
#endif
    }
    vmemSet ((void *)&filt, 0, sizeof (Filter));
    strcpy (filt.op, oper);
    strcpy (filt.keytype, keyClass);
    strcpy (filt.keyvalue, keyValue);
    rec = GetNote (&filt, time);
    if (rec == NULL)
    {
	status = 0;
    }
    else
    {
	app -> setCurrentSelectedRecord (rec);
	status = 1;
    }
    return (status);
}



/*******************************************************************
*
*    timer class to redisplay mainTextItem at regular intervals
*	especially needed when notes are being update on another
*	terminal
*
*******************************************************************/

void opNoteTimer::ObserveTimer (void)
{
    long topLine = mainText -> GetTopLineNum ();
    mainText -> redisplayData ();
    if (topLine > 0)
    {
	mainText -> Scroll (topLine, 0);
    }
}

void opNoteTimer::StartMainTimer (void)
{
    // timer will restart as soon as observe timer finishes
    SetRecurrent ();
    // set timer to expire every 60 seconds
    SetPeriod (60, 0L);
    Start ();
}

