/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


// userMaintShell.C

#include <Gina/Gina.h>

extern "C"
{
#include "logutils.h"

#ifndef MIPSEL
#include "memory.h"
#endif
}

#include <stdio.h>
#include <stdlib.h>

#include "userMaintShell.h"
#include "operDefs.h"
#include "utilDialogs.h"
#include "textEntryShl.h"
#include "crypt.h"

extern	UserMaintShell	*theUserMaintShell;
extern	OperatorList	operList;
extern	int		numOperators;
extern	UserMaintShell	*theUserMaintShell;
extern	char		displayHost [];

int	addUserInUse = 0;
int	passwordInUse = 0;

extern	int	addUser (char *);
extern	void	deleteUser (char *);
extern	void	changePassword (char *, char *);

//----------
UserMaintShell::UserMaintShell (GnDocument *doc, int width, int height) : MaintDiag (doc, width, height)

    {

	document = doc;

    }

//----------
UserMaintShell::~UserMaintShell (void)

    {

    }

//----------
void UserMaintShell::after_create (void)

    {

	MaintDiag :: after_create ();

	// Log this startup

	makeLogEntry ("n/a", displayHost, MAINT_STARTED);

	// Setup callback for the list

	operatorList.add_browseSelectionCallback (CALLBACK (UserMaintShell, operSelectionCallback, this));

	// Set up callbacks for the buttons

	addButton.add_activateCallback (CALLBACK (UserMaintShell, addButtonCallback, this));
	deleteButton.add_activateCallback (CALLBACK (UserMaintShell, deleteButtonCallback, this));
	deleteButton.SetSensitive (False);
	passButton.add_activateCallback (CALLBACK (UserMaintShell, passButtonCallback, this));
	passButton.SetSensitive (False);
	okButton.add_activateCallback (CALLBACK (UserMaintShell, okButtonCallback, this));
	cancelButton.add_activateCallback (CALLBACK (UserMaintShell, cancelButtonCallback, this));

	// Put the operators in the list

	fillList ();

    }

//----------
void UserMaintShell::operSelectionCallback (caddr_t)

    {

	deleteButton.SetSensitive (True);
	passButton.SetSensitive (True);

    }

//----------
void UserMaintShell::addButtonCallback (caddr_t)

    {

	if (!addUserInUse) {

	    addUserInUse = True;

	    char userName [USERNAME_SIZE+1];
	    TextEntryShl *dialog;
	    for (;;) {
		strcpy (userName, "");
		dialog = new TextEntryShl (document, "Add User", userName, USERNAME_SIZE);
		dialog->create (mainForm, "AddUser");
		DialogResult result = OK;
		dialog->alert (result);

		// Check if the user name has any embedded spaces.

		char *p = userName;
		while (*p) {
		    if (*p == ' ')
			break;
		    p++;
		}
		if (*p != ' ')
		    break;
		error_dialog (mainForm, "Spaces not allowed in a user name!");

	    }

	    // Check to see if user already exists. If not add it.

	   if (dialog->OKWasClicked ()) {
		if (!addUser (userName)) {
		    char msg [50];
		    sprintf (msg, "%s already exists!", userName);
		    error_dialog (mainForm, msg);
		} else
		    makeLogEntry (userName, displayHost, CREATED_USER);
	    }

	    delete dialog;

	    fillList ();

	    addUserInUse = False;

	}

    }

//----------
void UserMaintShell::deleteButtonCallback (caddr_t)

    {

	// Get the selection from the listbox

	GnPresentableList *items = operatorList.SelectedItems ();
	GnPresentable *item = operatorList.SelectedItem ();
	char name [USERNAME_SIZE+1];
	item->PrintOn (name, USERNAME_SIZE+1);

	deleteUser (name);

	makeLogEntry (name, displayHost, DELETED_USER);

	fillList ();

    }

//----------
void UserMaintShell::passButtonCallback (caddr_t)

    {


	if (!passwordInUse) {

	    passwordInUse = True;

	    char password [PASSWORD_SIZE+1];
	    TextEntryShl *dialog;
	    for (;;) {
		strcpy (password, "");
		dialog = new TextEntryShl (document, "Change Password", password, PASSWORD_SIZE);
		dialog->create (mainForm, "Password");
		DialogResult result = OK;
		dialog->alert (result);

		// If the password is blank use an asterisk instead

		if (strlen (password) == 0)
		    strcpy (password, "*");
 
		// Check if the password has any embedded spaces.

		char *p = password;
		while (*p) {
		    if (*p == ' ')
			break;
		    p++;
		}
		if (*p != ' ')
		    break;
		error_dialog (mainForm, "Spaces not allowed in a password!");

	    }

	    // Check to see if user already exists. If not add it.

	   if (dialog->OKWasClicked ()) {

		// Get the user name from the listbox

		GnPresentableList *items = operatorList.SelectedItems ();
		GnPresentable *item = operatorList.SelectedItem ();
		char name [USERNAME_SIZE+1];
		item->PrintOn (name, USERNAME_SIZE+1);

		// If a password was specified, encrypt it. Then set the password.

		if (strcmp (password, "*") == 0) {
		    setPassword (name, password);
		} else {
		    char cryptedPassword [PASSWORD_SIZE*2+1];
		    strcpy (cryptedPassword, password);
		    encryptPassword (cryptedPassword);
		    setPassword (name, cryptedPassword);
		    makeLogEntry (name, displayHost, PASSWD_CHANGE_BYADMIN);
		}
	    }

	    delete dialog;

	    passwordInUse = False;

	}

    }

//----------
void UserMaintShell::okButtonCallback (caddr_t)

    {

	// Write the operators file

	FILE *f;
	f = fopen ("/tis4000/operators/tisOperators", "w");
	for (int i=0 ; i<numOperators ; i++)
	    if (!operList [i].deleted)
		if (strlen (operList [i].password))
		    fprintf (f, "%s %s\n", operList [i].name, operList [i].password);
		else
		    fprintf (f, "%s *\n", operList [i].name);

	fclose (f);

	makeLogEntry ("n/a", displayHost, MAINT_ENDED_OK);

	exit (0);

    }

//----------
void UserMaintShell::cancelButtonCallback (caddr_t)

    {

	makeLogEntry ("n/a", displayHost, MAINT_CANCELLED);
	exit (0);


    }

//----------
void UserMaintShell::fillList (void)

    {

	GnPresentableList *list = new GnPresentableList;
	for (int i=0 ; i<numOperators ; i++) {
	    if (!operList [i].deleted)
		list->Append (new GnStringItem (strdup (operList [i].name)));
	}
        operatorList.SetItemList (list, True);
	delete list;

	deleteButton.SetSensitive (False);
	passButton.SetSensitive (False);

    }

