/*dmMacro_subs.c,v 1.1.1.1 1999/08/03 14:43:48 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
************************************************
* Purpose:  Recursively performs the macro-like substitution of
*	channel names with their substitute name.
*	Example: if input string is
*		chan_$(dog)_001
*			then the output string is
*		chan_cat_001
*			where it was specified that "cat is substituted
*			for every occurrance of "dog""
* Assumptions:
*	1) Input character array must be '\0' terminated.
*	   (And all resulting output character arrays will be `\0` terminated.)
*
* Modification History:
* 10/23/90	ges	creation
* 10/25/90	ges	echo error macro names to output string
*************************************************
*/
#include	<stdio.h>
#ifdef SOLARIS
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<gizmo.h>
#include <memLeak.h>

#if 0
char	*calloc();
#endif
void	gsdl_main_macro();
void	gsdl_process_macro();
char	*gsdl_lookup_sub();
void	gsdl_concat();
int	get_macro_pairs();

extern	struct dm_llist_hdr	dm_llist_hdr;
extern	exitKeyPressed;

/*
 *	gsdl_main_macro(pTo, pFrom, pMacro_pairs,maxlen)
*	where:
*		pTo: ptr to the character array which is to receive
*			the channel name after all substitutions.
*		pFrom: ptr to the input character array which possibly
*			has substitutions indicated by 
*			$(stringtobesubstituted).
*		pMacro_pairs: an array of  character ptr pairs terminate
*			with the NULL ptr.  The first element points to
*			the first string which may be substituted by
*			the string pointed to by the second element.
*			The third element points to the second string 
*			which may be substituted by the string pointed to
*			by the forth element.  And so on, until terminated
*			by a NULL.
*/
void gsdl_main_macro(pTo, pFrom, pMacro_pairs, maxlen)
char	*pTo;
char	*pFrom;
char	*pMacro_pairs[];
int		maxlen;
	{
	char	*tmpbuf		= calloc(maxlen+1, sizeof (char));
	char	*pTo_max	= pTo + (maxlen - 1);
	char	*pToOrg		= pTo;

	strcpy(tmpbuf, pFrom);
	do
		{
		pTo = pToOrg;
		pFrom = tmpbuf;
		while (*pFrom != '\0' && pTo < pTo_max)
			if (*pFrom == '$')
				{
				*pFrom = '\177';	/* overwrite $ to indicate that we've seen this one */
				gsdl_process_macro(&pTo, &pFrom, pTo_max, pMacro_pairs, maxlen);
				}
			else
				*pTo++ = *pFrom++;

		*pTo = '\0';
		strcpy(tmpbuf, pToOrg);
		}
	while (strchr(tmpbuf, '$'));
	free(tmpbuf);
	for (pTo = pToOrg; *pTo; pTo++)
		if (*pTo == '\177')
			*pTo = '$';
	}

/*
 *	gsdl_process_macro(pPto, pPfrom, pTo_max, pMacro_pairs, maxlen
 */
void gsdl_process_macro(pPto, pPfrom, pTo_max, pMacro_pairs,maxlen)
char	**pPto;
char	**pPfrom;
char	*pTo_max;
char	*pMacro_pairs[];
int		maxlen;
	{
	char	*macro_name			= calloc(maxlen, sizeof (char));
	char	*pMacro_name		= &macro_name[0];
	char	*pMacro_name_max	= (maxlen-1) + &macro_name[0];
	char	*pSub_name;
	int		i;

	if (macro_name == NULL)
		{
		printf("~calloc failed in gsdl_process_macro\n");
		return;
		}
	for (i = 0; i < maxlen; i++)
		macro_name[i] = '\0';

	(*pPfrom)++;		/* inc past $ */
	if (**pPfrom == '\0')
		{
		gsdl_concat(pPto, "\177", pTo_max);
		free(macro_name);
		return; /* error: uncompleted macro name */
		}
					/* need $ */
	if (**pPfrom == '(')
		{
		(*pPfrom)++;	/* inc past ( */

		do
			{
			if(**pPfrom == '$')
				gsdl_process_macro(&pMacro_name, pPfrom, pMacro_name_max, pMacro_pairs,maxlen); /* recursive macro */

			if (**pPfrom == ')')
				{
				/* macro complete, so look-up and concat */
				(*pPfrom)++;	/* inc past ) */
				pSub_name = gsdl_lookup_sub(&macro_name[0], pMacro_pairs);
				if (pSub_name == NULL)
					{
					gsdl_concat(pPto, "\177(", pTo_max);
					gsdl_concat(pPto, &macro_name[0], pTo_max);
					gsdl_concat(pPto, ")", pTo_max);
					}
				else
					gsdl_concat(pPto, pSub_name, pTo_max);
				free(macro_name);
				return;
				}
			if (**pPfrom == '\0')
				{
				gsdl_concat(pPto, "\177(", pTo_max);
				gsdl_concat(pPto, &macro_name[0], pTo_max);
				free(macro_name);
				return; /* error: uncompleted */
				}
			/* need $( */
			if (pMacro_name >= pMacro_name_max)
				{
				*pMacro_name = '\0';
				gsdl_concat(pPto, "\177(", pTo_max);
				gsdl_concat(pPto, &macro_name[0], pTo_max);
				free(macro_name);
				return; /* error: macro expand exceeds buffer - need $( concat pMacro_name */
				}
			*pMacro_name = **pPfrom;
			pMacro_name++;
			(*pPfrom)++;
			}
		while(1);
		}
	/* error: uncompleted macro name */
	/* need $ */
	gsdl_concat(pPto, "\177", pTo_max);
	free(macro_name);
	}

/*
 *	gsdl_lookup_sub(pMacro_name, pMacro_sub_pairs)
 */
char	*gsdl_lookup_sub(pMacro_name, pMacro_sub_pairs)
char	*pMacro_name;
char	*pMacro_sub_pairs[];
	{
	while (*pMacro_sub_pairs != NULL)
		{
		if (!strcmp(pMacro_name, *pMacro_sub_pairs))
			return (char *) *(++pMacro_sub_pairs);
		pMacro_sub_pairs++;
		pMacro_sub_pairs++;
		}
	return (char *) NULL;
	}

/*
 *	gsdl_concat(pPto, pSub_name, pPto_max)
 */
void gsdl_concat(pPto, pSub_name, pPto_max)
char **pPto;
char *pSub_name;
char *pPto_max;
	{
	if (pSub_name == NULL)
		return;

	while (*pSub_name != '\0')
		{
		if (*pPto >= pPto_max)
			break;
		**pPto = *pSub_name;
		pSub_name++;
		(*pPto)++;
		}
	}



/************************************************************************************
*
*	get_macro_pairs
*
*	DK 2/14/911
*
*	Get_macro_pairs sets up the array of pairs of strings(0-1, 2-3, etc.) as specified
*	in a list represented by a string, i.e. the string is parsed to produce a set of
*	ptrs to pairs of strings.  This set (array) is used for specifying the substitution
*	of the first string in the pair by the second string (see "~gsdl_macro.c.") This will
*	be done when channel access is set up for a given process variable defined using the
*	display editor. 
*
*	Note :
*	The macro substitution string will look something like :
*		rf = RF1, cav = CAV1
*	(where '=' is used to indicate paired strings and ',' is used to delimit pairs)
*	and will produce the array 
*		pMacro_pairs[0] = "rf"		 	
*		pMacro_pairs[1] = "RF1"		
*		pMacro_pairs[2] = "cav"	
*		pMacro_pairs[3] = "CAV1"		
*		pMacro_pairs[4] = NULL
*
*	If the process variable name defined is "SYSTEM1:$(rf):$(cav)"
*	then the string produced by the gsdl_macro routine will be "SYSTEM1:RF1:CAV1"
*
*
*	Input	: pStr	ptr to the string containing the substitution specification(s)
*		  pMac_pairs 	ptr to the array of substitution pairs
*		  max_pairs maximum number to do (pMac_pairs needs to have (max_pairs*2)+1 elements)
*	Output	: pMac_pairs is filled in with ptrs to strings
*	Returns	: nothing
*
*/
#define	TRUE	1
#define	FALSE	0

void remove_spaces(p)
char	*p;
	{
	for (; *p && isspace(*p); p++)
		{
		strcpy(p, p+1);
		p--;
		}
	}


#define mm_to_xpixel(MM,S) (((MM)*WidthOfScreen(S))/WidthMMOfScreen(S))
#define mm_to_ypixel(MM,S) (((MM)*HeightOfScreen(S))/HeightMMOfScreen(S))

#define	MAX_ARG_LENGTH				50
#ifndef NeXT
#define ARG_PROMPT_GIZMO_WIDTH_MM 	55
#define ARG_PROMPT_GIZMO_HEIGHT_MM 	5
#else
#define ARG_PROMPT_GIZMO_WIDTH_MM 	50
#define ARG_PROMPT_GIZMO_HEIGHT_MM 	4
#endif

enum macro_arg_prompt_column_headers
	{
	name_col,
	value_col,
	arg_prompt_last_col
	};

enum macro_arg_prompt_options
	{
	option_cancel,
	option_ok
	};
	
static char  *macro_arg_prompt_choices[] =
	{
	0,
	0,
	0
	};

struct macro_arg_prompt_element
	{
	NODE	node;
	char	*name;
	char	**pvalue;
	};

static void arg_prompt_list_update();
static void modify_arg_prompt_finish();

static gizmo	*ppopupg;

get_macro_pairs(pMacstr, pMac_pairs, max_pairs, s)
char		*pMacstr;
char		*pMac_pairs[];
int			max_pairs;
Screen		*s;
	{
	char				*p;
	unsigned int			index = 0;
	unsigned int			max_index = (2*max_pairs);
	struct macro_arg_prompt_element	*pelem;
	gizmo				*pformg;
	gizmo				*pmenug;
	XEvent				xevent;
	unsigned			row;
	unsigned			height = mm_to_ypixel(ARG_PROMPT_GIZMO_HEIGHT_MM, s);
	unsigned			width = mm_to_xpixel(ARG_PROMPT_GIZMO_WIDTH_MM, s);
	LIST				macro_arg_prompt_list;
	int				retval;
	int				maxWidth;

	if (ppopupg)
		{
		move_gizmo(ppopupg, ppopupg->obj.x, ppopupg->obj.y);
		return 0;
		}
/*
	21-Nov-1994 GCT
	added code here to initialize string pointer array 'macro_arg_prompt_choices'. This is
	the only function that uses this array.
*/
	macro_arg_prompt_choices[0] = "~cancel";
	macro_arg_prompt_choices[1] = "~ok";
	macro_arg_prompt_choices[2] = 0;
	/*
	 * build the macro arg list
	 */
	p = strcpy(calloc(strlen(pMacstr) + 1, sizeof (char)), pMacstr);
	lstInit(&macro_arg_prompt_list);
	for (; *p && index < max_index;)
		{
		char delimiter = (index & 1) ? ',' : '=';

		pMac_pairs[index] = p;
		if (delimiter == '=')
			remove_spaces(p);		/* remove spaces in front of the name */
		while (*p && (*p != delimiter))
			{
			if (delimiter == ',' && *p == '[')
				{
				
				pelem = (struct macro_arg_prompt_element *)calloc(1, sizeof (struct macro_arg_prompt_element));
				pelem->name = pMac_pairs[index] = ++p;
				pelem->pvalue = &pMac_pairs[index];
				lstAdd(&macro_arg_prompt_list, &pelem->node);
				while (*p && *p != ']')
					p++;
				if (*p && *p == ']')
					*p = 0;
				}
			if (isspace(*p) && delimiter == '=')
				{
				remove_spaces(p);		/* remove spaces behind the name */
				p--;
				}
			p++;
			}
		if (!(*p) && (delimiter == '='))
			break;
		if (!(*p) || (*p == delimiter) )
			{
			if (*p)
				*p++ = 0;
			index++;
			}
		}

	if (index & 1)
		index--;
	pMac_pairs[index] = NULL;

	if (lstCount(&macro_arg_prompt_list) == 0)
		/* nothing to prompt for */
		return 1;
		
	/*
	 * create and fill in the form
	 */
	ppopupg = create_popup_gizmo(
				s, 
				RootWindowOfScreen(s),
				" ",
				100, 
				100,
				height,
				modify_arg_prompt_finish,
				&ppopupg,
				macro_arg_prompt_choices,
				sizeof (short));
	*((short *)(ppopupg->pae)) = 0;
	set_gizmo_color(ppopupg, WhitePixelOfScreen(s), ppopupg->pgg->dialog_bg);
	
	pformg = create_form_gizmo(s, ppopupg->w, 0, 0, arg_prompt_last_col, lstCount(&macro_arg_prompt_list)+1, mm_to_xpixel(1, s), 0);

	row = 1;				/* list the arg name lines */
	maxWidth = 0;
	for (pelem = (void *)lstFirst(&macro_arg_prompt_list); pelem; pelem = (void *)lstNext((NODE*)pelem), row++)
		{
		XRectangle	obj;
		gizmo		*pg;

		obj.x		= 0;
		obj.y		= 0;
		obj.height	= height;
		obj.width	= width;
		pg = create_textedit_gizmo(	
					pformg->pgg->s, 
					pformg->w, 
					&obj,
					ok_to_operate,
					arg_prompt_list_update,
					pelem,
					MAX_ARG_LENGTH,
					0);
		set_textedit_gizmo(pg, "");
		form_gizmo_add_string(pformg, name_col,  row, height, pelem->name);
		form_gizmo_add_gizmo( pformg, value_col, row, pg);
		if (strlen (pelem->name) > maxWidth)
		    maxWidth = strlen (pelem->name);
		}
	resize_form_gizmo(pformg);
	popup_gizmo_add_gizmo(ppopupg, pformg);
	XWarpPointer (XDisplayOfScreen (s), None, DefaultRootWindow (XDisplayOfScreen (s)),
			0, 0, 0, 0, 150 + maxWidth * 12, 100 + height * 4);
	XMapWindow(ppopupg->pgg->d, ppopupg->w);
	exitKeyPressed = 0;
	while ((!*((short *)(ppopupg->pae))) && !exitKeyPressed)  /* set below in modify_arg_prompt_finish() */
		{
			x_evhandler (&dm_llist_hdr);
		}
	free_gizmo(ppopupg);
	retval = (*((short *)(ppopupg->pae)) == -1) ? 0 : 1;
	ppopupg = NULL;
	lstFree(&macro_arg_prompt_list);
	return retval;
	}
	
static void arg_prompt_list_update(pelem, pg, pvalue)
struct macro_arg_prompt_element	*pelem;
gizmo							*pg;
char							*pvalue;
	{
	assert((*pelem->pvalue = calloc(strlen(pvalue) + 1, sizeof (char))) != NULL);
	strcpy(*pelem->pvalue, pvalue);
	}

static void modify_arg_prompt_finish(ppg, pg, value)
gizmo	**ppg;
gizmo	*pg;
int		value;
	{
	assert(*ppg == pg);
	switch ((enum macro_arg_prompt_options)value)
	  {
	  case option_ok:		*((short *)(pg->pae)) = 1;	break;
	  case option_cancel:	*((short *)(pg->pae)) = -1;	break;
	  }
	}
