/*op_comm.c,v 1.1.1.1 1999/08/03 14:43:49 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
#undef DEBUG
/****************************************************************/
/*#define PRINT_CA_EVENTS*/
/*
*************************************************************************
*File:  op_comm.c                                                
*Purpose:  Performs the necessary channel access for the OPI     
* 1) Loop forever:
*	1) If new window list flag is set (indicating to ca_comm
*	   that a new window list has arrived) then
*		1) If any window is newly opened then
*			1) Allocate storage for a communications table
*			   containing:for each dynamic display object in window
*				1) chid
*				2) connection count
*				3) pointer to modification bit array
*					1) the bit array itself
*				4) pointer to a structure which will store the
*				   initial ctrl data and the subsequent sts
*				   monitor data
*			2) For each dynamic display object in the window
*				1) attempt to establish connection to data
*				   sources (ca_search)
*				2) if successfully connected then
*					1) attempt to acquire ctrl data
*					   (get_callback)
*				3) if successfully called back then
*					1) attempt to acquire sts data
*					   (add_event)		
*			3) Inform Display Interpreter of addresses for its
*			   access
*				1) address of window's com table
*				2) address of object`s modification bit array
*				3) number of ints per channel in bit array
*		2) If any window is newly closed then
*			1) for each dynamic display object in the window
*				1) close connection (ca_clear)
*				2) free the storage used in the com table
*		3) clear flag (indicating to the Display Interpreter that
*		   ca_comm has processed the new window list)
*	2) pend_event to allow events to execute their handlers and to 
*	   sleep when no events occur
*
* Compile:  make                                                  
*	   This Makefile links archived Sun 3 executable code. On any   
*	   other machine you must rlogin to a Sun 3 to make and then    
*	   execute the resulting code.                                  
*	      Binkley% rlogin r2d2				        
*            Password?  "your usual password to your machine"          
*	      make                                                      
*            op_comm                                                   
* Execute:  op_comm                                               
*                                                                      
* Note: this module requires the following dl structures that can be:
*	Included from file:  /h/dldef.h:
*	1) struct dl_element_hdr
*	2) struct dl_pc
*	3) struct dl_object_hdr
*	4) struct dl_hdr
*	Included from file:  op_gsdl_def.h:
*	1) struct gsdl_com_llist_controller
*	2) struct gsdl_com_llist_element
*	3) struct gsdl_com_record
*
*      Modification History                                            
* version   date      author       comments                            
*----------------------------------------------------------------------
* 1.0       05/16/90  Gale Slentz  Initial version applying chan acc.   
* 1.2       06/05/90  Gale Slentz  Add cntl and sts mechanisms.
* 1.3       06/13/90  Gale Slentz  Modify for multiple window version.
*	    06/21/90  Gale Slentz  Initial version release to Deb K.
*				   3:30 gsdl_add_chan_event: add ca_flush_io
*				   eliminate gsdl_coerce_to_ctrl/sts
* 1.4       06/27/90  Gale Slentz  Allow for C.A. arrays of data.
* 1.5       07/09/90  Gale Slentz  Add C.A. puts and gets mechanism.
*				   Shorten callback_hander, sts_update_storage
*				   Fix connection down bug in connection_handlr
* 1.6       07/19/90  Gale Slentz  Retract above C.A. puts and gets mechanism.
*				   Insert Jeff's file descriptor code.
* 1.7	    01/22/91  DK	   fix bug in gsdl_llist_main to allow mod of
*					pOld_llsit_start
* 1.8       01/28/91  DK	   Change gsdl_broadcast_search to eliminate
*				   	null string problem coming from dl
* 1.9	    08/08/91  DK	   Modified connection handler to do add_event
*				   as per conversation with Jeff.  Also added delete
*				   disconnect to do clear_event. Added setting
*				   and clearing of valid data flag.
************************************************************************
*/
#include <memLeak.h>
#include <stdio.h>
#include <db_access.h>
#include <cadef.h>
#include <dldef.h>
#include <op_gsdl_def.h>
#include <time.h>
#include <ctype.h>
#ifdef PRINT_CA_EVENTS
#include <sys/time.h>
#endif
#ifndef SOLARIS
#include <types.h>
#endif
#include <op_unix_macros.h>
#include <op_common_macros.h>

#ifdef AXP
#include <string.h>
#endif

/*
*-----------------------------------------------------------
* The following definitions allow conditional execution of various
* print statements to allow tracing of the program.
* 	SHOW_TABLE = 0:  to deselect printing the com table contents.
*	SHOW_LLIST = 0:  to deselect printing the linked window list contents.
*	SHOW_EVENTS = 0:  to deselect printing event messages as the are executed.
*	SHOW_VALUES = 0:  to deselect printing the sts structure values at event.
*----------------------------------------------------------
*/
#define	SHOW_TABLE	0
#define SHOW_LLIST	0
#define SHOW_EVENTS	0
#define SHOW_VALUES	0


#define NORMAL	0		/* int return value for normal termination */
#undef ERROR
#define ERROR	1		/* int return value for abnormal termination */

#define PEND_IO_DELAY	0.01	/* standard 0.01 sec delay for all ca_pend_io */
#define	PEND_EVENT_DELAY	0.0001	/* standard 0.0001 sec delay for event*/
/*
*---------------------------------------------------------
* Subroutine Prototypes 
*---------------------------------------------------------
*/
int gsdl_llist_main();			/* entry point for real operation */
int gsdl_add_llist_element();
int gsdl_max_count();
int gsdl_ints_per_chan();
int gsdl_init_chname_list();
void gsdl_print_com_table();	/* table print for test only!!!!! */
void gsdl_display_event_handler();
int gsdl_add_chan_event();
void gsdl_connection_event_handler();
int gsdl_broadcast_search();
void gsdl_callback_handler();
void gsdl_sts_update_storage();
struct gsdl_com_llist_element *gsdl_malloc_llist_element();
struct gsdl_com_llist_element *gsdl_gen_old_llist();
void gsdl_free_llist_element();
void gsdl_free_llist();
struct gsdl_com_llist_element *gsdl_make_copy_llist();
int gsdl_not_in_llist();
void gsdl_com_table_clearchannel();
void gsdl_print_llist();	/* llist element print for test only!!! */
int gsdl_any_mods_set();
void gsdl_clear_mod_bits();

void gsdl_fd_register();		/* Jeff's file descriptor code */
void gsdl_ca_service();		/* Jeff's fd code */

int	caHasUpdated;

extern int inNormalExit;	/* flag indicating normal rundown */
/*
***********************************************************************
* NAME
*	main()
* DESCRIPTION
*	Purpose:  This is the entry point for stand-alone testing.
*	1) An artificial structure is initialized to allow standalone testing
*	2) gsdl_llist_main is called to process.
***********************************************************************
*/
/*
*---------------------------------------------------------
* Entry point for test only!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
*---------------------------------------------------------
*/


/*
*********************************************************************
* NAME 
*	gsdl_llist_main(pCom_llist_controller)
* DESCRIPTION
*	Purpose:  This is the normal usage entry point.  The new_llist_flag
*	is polled to determine when a new window linked list is available
*	for processing.  This processing involves establishing channel
*	connections and setting up and receiving monitor data for
*	opened windows.  And for closed windows, the freeing of resources.
*	1) Loop forever.
*		1) If (new_llist_flag) then
*			1) For (each window in the new window list that doesn't
*				appear in the old window list) do
*				1) Build table and initiate events by call to 
*				   gsdl_add_llist_element.
*			2) For (each window in the old list that doesn't appear
*				in the new list) do
*				1) Free the old window's resources.
*			3) Make a new version of old list by copying new list.
*			4) Clear new_llist_flag.
*		2) Pend event to allow event handlers to execute in Sun OS.
*
* ARGUMENTS
*	(Inputs)
*	pCom_llist_controller:  the address to a structure that must contain:
*		1) new_llist_flag:  indicating a new linked list to process.
*		2) pNew_llist:  the starting element of the new llinked list.
*	(Outputs)
*	pCom_llist_element->pCom_table:  receives the starting address of
*		the channel name table.
*	pCom_llist_element->ints_per_chan:  receives the count of integers
*		forming the array of modified bits.
*	
********************************************************************
*/
int gsdl_llist_main(pCom_llist_controller,ppOld_llist_start, pMac_pairs)
struct gsdl_com_llist_controller *pCom_llist_controller;
struct gsdl_com_llist_element **ppOld_llist_start ;
char *pMac_pairs[];
{
  struct gsdl_com_llist_element *pNew_llist_start;
  struct gsdl_com_llist_element *pOld_llist_element;
  struct gsdl_com_llist_element *pNew_llist_element;

  int i;

	if (pCom_llist_controller->new_llist_flag)
		{
		pNew_llist_start = pCom_llist_controller->pNew_llist;
		/*
		*--------------------------------------------------------
		* Any elements in the new list that need to be added? 
		*--------------------------------------------------------
		*/
		pNew_llist_element = pNew_llist_start;
		while (pNew_llist_element)
			{
			if (gsdl_not_in_llist(pNew_llist_element, *ppOld_llist_start))
				{
				gsdl_macro_show_llist(
					"~adding a new llist element = %p\n",
					pNew_llist_element->pDl_hdr);
				gsdl_add_llist_element(pNew_llist_element, pMac_pairs);
				}
			pNew_llist_element = pNew_llist_element->pNext_element;
			}
		/*
		*-------------------------------------------------------
		* Any elements in the old list that need to be deleted?
		*-------------------------------------------------------
		*/
		pOld_llist_element = *ppOld_llist_start;
		while (pOld_llist_element)
			{
			if (gsdl_not_in_llist(pOld_llist_element, pNew_llist_start))
				{
				gsdl_macro_show_llist(
					"~freeing an old llist element = %p\n",
					pOld_llist_element->pDl_hdr);
				gsdl_free_llist_element(pOld_llist_element);
				}
			pOld_llist_element = pOld_llist_element->pNext_element;
			}

		/*
		*---------------------------------------------------------
		* Generate new version of old llist.
		*---------------------------------------------------------
		*/
		gsdl_macro_show_llist(
			"~generating new generation of old llist\n", NULL);
		*ppOld_llist_start = gsdl_gen_old_llist(pNew_llist_start,
					*ppOld_llist_start);

		pCom_llist_controller->new_llist_flag = 0;
		gsdl_macro_print_llist(*ppOld_llist_start);
		}

}/* end gsdl_llist_main */

/*
*********************************************************************
* NAME 
*	gsdl_free_llist_element(pCom_llist_element)
* DESCRIPTION
*	Purpose:  To close all chids and free the memeory needed to 
*	process the referenced window list element.
*	1) Call gsdl_com_table_clearchannel to close all channels in the table
*		referenced by pCom_llist_element->pCom_table.
*	2) For each record (i.e. row) of pCom_table
*		1) If pStruct field not equal NULL then
*			1) Memory has been allocated for this structure, 
*			   so free it.
*	3) Free the complete bit array for this table.
*	4) Finally free the table storage space itself.
********************************************************************
*/
void gsdl_free_llist_element(pCom_llist_element)
struct gsdl_com_llist_element *pCom_llist_element;
{
struct gsdl_com_record *pCom_table = pCom_llist_element->pCom_table;
int i;
int npc = (pCom_llist_element->pDl_hdr)->npc;

gsdl_com_table_clearchannel(pCom_llist_element);
#if 0
for (i = 0; i < npc; ++i)
	{
	/*-------------------------------------------------------------
	* If NULL, ie no structure alloc yet, then free does nothing. 
	*-------------------------------------------------------------*/
	free((void *) pCom_table->pData_struct);
	}
if (npc > 0)
	{
	free((void *) pCom_table->pMod_bits);
	free((void *) pCom_table);
	}
#endif
for (i = 0; i < npc; ++i)
	{
	/*-------------------------------------------------------------
	* If NULL, ie no structure alloc yet, then free does nothing. 
	*-------------------------------------------------------------*/
	free((void *) (pCom_table + i)->pData_struct);
	}
if (npc > 0)
	{
	free((void *) pCom_table->pMod_bits);
	free((void *) pCom_table);
	}
return;
}
/*
*********************************************************************
* NAME 
*	gsdl_com_table_clearchannel(pCom_llist_element)
* DESCRIPTION
*	Purpose:  To close all chids that were used by the referenced 
*	window list element.
*	1) For each row of com_table (there are npc rows).
*		1) Clear channel.  Note that any pending gets data will
*		   be discarded.  Any defined events (get_callback or add_event)
*		   will be unassociated from the chid and discarded.
*	2) Flush I/O to force immediate execution of the channel access clears.
********************************************************************
*/
void gsdl_com_table_clearchannel(pCom_llist_element)
struct gsdl_com_llist_element *pCom_llist_element;
{
struct gsdl_com_record *pCom_table = pCom_llist_element->pCom_table;
int npc = (pCom_llist_element->pDl_hdr)->npc;
int i;

for (i = 0; i < npc; ++i)
	{
	if((pCom_table + i)->pChan)
	SEVCHK(ca_clear_channel((pCom_table + i)->pChan),
		"~gsdl_com_table_clearchannel:  table clear channel failed\n");
	}
/*ca_flush_io();*/

return;
}
/*
*********************************************************************
* NAME 
*	gsdl_malloc_llist_element()
* DESCRIPTION
*	Purpose:  To allocate the memory needed to form a com list element.
********************************************************************
*/
struct gsdl_com_llist_element *gsdl_malloc_llist_element()
{
struct gsdl_com_llist_element *pCom_llist_element = 
	(struct gsdl_com_llist_element *) calloc(1, sizeof(struct gsdl_com_llist_element));
if (pCom_llist_element == NULL) {
    printf ("~calloc failed in gsdl_com_llist_element\n");
    exit (-1);
}
return pCom_llist_element;
}
/*
*********************************************************************
* NAME 
*	gsdl_gen_old_llist(pNew_llist_start, pOld_llist_start)
* DESCRIPTION
*	Purpose:  To generate a new version fo the old llist by copying
*	the new linked list.
*	1) free the old llist memory
*	2) make a copy of the new llist
*	3) return the starting address of this new llist
********************************************************************
*/
struct gsdl_com_llist_element *gsdl_gen_old_llist(pNew_llist_start,
	pOld_llist_start)
struct gsdl_com_llist_element *pNew_llist_start;
struct gsdl_com_llist_element *pOld_llist_start;
{
gsdl_free_llist(pOld_llist_start);
return gsdl_make_copy_llist(pNew_llist_start);
}
/*
*********************************************************************
* NAME 
*	gsdl_not_in_llist(pNew_llist_element, pOld_llist_start)
* DESCRIPTION
*	Purpose:  Returns 0:  if the dl_hdr address of the referenced
*			list element is identical with any of the dl_hdr
*			addresses of the linked list of list elements.
*		  Returns 1:  otherwise.
********************************************************************
*/
int gsdl_not_in_llist(pNew_llist_element, pOld_llist_start)
struct gsdl_com_llist_element *pNew_llist_element;
struct gsdl_com_llist_element *pOld_llist_start;
{
while(pOld_llist_start)
	{
	if((pNew_llist_element->pDl_hdr) == (pOld_llist_start->pDl_hdr))
		{
		return 0;
		}
	pOld_llist_start = pOld_llist_start->pNext_element;
	}
return 1;
}
/*
*********************************************************************
* NAME 
*	gsdl_free_llist(pOld_llist_start)
* DESCRIPTION
*	Purpose:  To free the memory used by the linked list of
*	gsdl_com_llist_element structures referenced by pOld_llist_start.
********************************************************************
*/
void gsdl_free_llist(pOld_llist_start)
struct gsdl_com_llist_element *pOld_llist_start;
{
struct gsdl_com_llist_element *pOld_llist_next;
while(pOld_llist_start)
	{
	pOld_llist_next = pOld_llist_start->pNext_element;
	free((void *) pOld_llist_start);
	pOld_llist_start = pOld_llist_next;
	}
return;
}

/*
*********************************************************************
* NAME 
*	gsdl_make_copy_llist(pOriginal_llist_start)
* DESCRIPTION
*	Purpose:  To make a copy of the linked list structure pointed to
*	by pOriginal_llist_start and retun a pointer to the start of this
*	copy.  If not sufficient system memory to make a copy, then the
*	valid partial copy is returned and a failure message is printed.
********************************************************************
*/
struct gsdl_com_llist_element *gsdl_make_copy_llist(pOriginal_llist_start)
struct gsdl_com_llist_element *pOriginal_llist_start;
{
int starting_element_flag = 1;
struct gsdl_com_llist_element *pCopy_llist_start = NULL;
struct gsdl_com_llist_element *pCopy_llist_last = NULL;
struct gsdl_com_llist_element *pCopy_llist_element;

while (pOriginal_llist_start)
	{
	pCopy_llist_element = gsdl_malloc_llist_element();
	if (pCopy_llist_element == NULL)
		{
		printf("~gsdl_make_copy_llist:  memory allocation failed.\n");
		return pCopy_llist_start;
		}
	if (starting_element_flag)
		{
		pCopy_llist_start = pCopy_llist_element;
		starting_element_flag = 0;
		}
	else
		{
		pCopy_llist_last->pNext_element = pCopy_llist_element;
		}
	*pCopy_llist_element = *pOriginal_llist_start;
	pCopy_llist_element->pNext_element = NULL;
	pCopy_llist_last = pCopy_llist_element;
	pOriginal_llist_start = pOriginal_llist_start->pNext_element;
	}
return pCopy_llist_start;
}
/*
*********************************************************************
* NAME 
*	gsdl_print_llist(pCom_llist_start)
* DESCRIPTION
*	Purpose:  To print for testing purposes the pDl_hdr and npc fields
*	of each display window element of the linked list.
********************************************************************
*/
void gsdl_print_llist(pCom_llist_start)
struct gsdl_com_llist_element *pCom_llist_start;
{
printf("~printing llist\n");
while (pCom_llist_start)
	{
	printf("~pDl_hdr = %p", pCom_llist_start->pDl_hdr);
	printf("~\tnpc = %d\n", (pCom_llist_start->pDl_hdr)->npc);
	pCom_llist_start = pCom_llist_start->pNext_element;
	}
return;
}
/*
**********************************************************************
* NAME
*	gsdl_add_llist_element(pCom_llist_element)
* DESCRIPTION
*	1) constructs storage sufficient to hold the array of
*	   gsdl_com_record structures.
*	2) constructs storage sufficient to hold the array of ints
*	   containing the "modified" bits for each channel.
*	3) call gsdl_init_chname_list to make channel access connections
*	   to each channel and then initialize the contents of the
*	   table (i.e. array of structures).
*	4) Call ca_add_connection_event to establish sequence of events
*	   as each channel opens its connection.
*	5) Call gsdl_broadcast_search to attempt to open each channel.
*	6) passes ptrs to com table and Mod array back to calling program, along
*	   with ints_per_chan value.
*
* CONTROL
*	This routine initiates a sequence of events for each channel 
*	in the window.  This event structure allows channels to make 
*	connection at any time during this program's execution.
*	The control sequence is as follows:
*
*	Seaches are initiated for each channel.
*	At the channel's first connection event,
*		 a ctrl type get_callback is initiated for each chan
*	At callback event, the ctrl data is stored, and a sts type
*		event is initiated for each channel.
*	At event, the sts data is stored.  These events continue at changes
*		of value exceeding monitor deadband or changes of alarm state.
*
* ARGUMENTS
*	(Inputs)
*	pCom_llist_element:  must contain the address of the dl_hdr 
*		structure for a particular window.
*	(Outputs)
*	pCom_llist_element->pCom_table:  receives the starting address of
*		the channel name table.
*	pCom_llist_element->ints_per_chan:  receives the count of integers
*		forming the array of modified bits.
*********************************************************************
*/
int gsdl_add_llist_element(pCom_llist_element, pMac_pairs)
struct gsdl_com_llist_element *pCom_llist_element;
char *pMac_pairs[];

{
int i;
int max_count;
int ints_per_chan;
struct gsdl_com_record *pCom_table;
int *pMod_array;
struct dl_pc *pDl_pc = (struct dl_pc *)(((short *)(pCom_llist_element->pDl_hdr))
	+ pCom_llist_element->pDl_hdr->opc);
int npc = (pCom_llist_element->pDl_hdr)->npc;

max_count = gsdl_max_count(pDl_pc, npc);
ints_per_chan = gsdl_ints_per_chan(max_count);

if (npc > 0)
	{
	pCom_table = (struct gsdl_com_record *)	
		 calloc(npc, sizeof(struct gsdl_com_record));
	pMod_array = (int *) calloc( (unsigned)(npc * ints_per_chan),sizeof(int) );
	pCom_table -> pMod_bits = pMod_array;	/* GU: Cure memory leak! */
	if ((NULL == pCom_table) || (NULL == pMod_array))
		{
		printf("~gsdl_init_chname_list:  Memory allocation failed\n");
		return NORMAL;
		}

	gsdl_init_chname_list(pDl_pc, npc, ints_per_chan, pCom_table, pMod_array);
	gsdl_broadcast_search(pCom_table, pDl_pc, npc, pMac_pairs);
	}
pCom_llist_element->pCom_table = npc ? pCom_table : NULL;
pCom_llist_element->ints_per_chan = ints_per_chan;

/*
*------------------------------------------------------
* the following prints are exposed for verifying the Mod bits in testing!!!!!
*------------------------------------------------------
*/
gsdl_macro_show_table(pCom_table, npc, ints_per_chan);

return NORMAL;
}
/*
***********************************************************************
* NAME
*	gsdl_max_count(pDl_pc, npc)
* DESCRIPTION
*	Purpose:  To find and return the largest connection count by
*	inspecting each channel name and connection count
*	pair in the sequence of structures *pDl_pc.
*	1) max_count = 0
*	2) For each structure in the concatenated sequence of dl_pc structures
*	   (these are npc of these)
*		1) If (pDl_pc->count > max_count) then
*			1) max_count = pDl_pc->count
*	3) return max_count
*
***********************************************************************
*/
int gsdl_max_count(pDl_pc, npc)
struct dl_pc *pDl_pc;
int npc;
{
unsigned short max_count = 0;
int i;

for (i = 0; i < npc; ++i)
	{
	char *pChanName;
	pChanName = (char *)(pDl_pc + 1);
	
	if(strlen(pChanName) > 0 && pDl_pc->count > max_count)
		max_count = pDl_pc->count;
	pDl_pc = (struct dl_pc *) (((short *) pDl_pc) + pDl_pc->element.nwords);
	}
return (int)max_count;
}
/*
***********************************************************************
* NAME
*	gsdl_ints_per_chan(max_count)
* DESCRIPTION 
*	Purpose:  To calculate the number of integers necessary to
*	represent a bit for each count in max_count.
*	Assumes that one bit is associated with each count held in
*	max_count, and returns the number of integers necessary
*	to contain max_count bits.  (Example:  if max_count = 33,
*	and an integer can hold 32 bits, then returns 2.)
***********************************************************************
*/
gsdl_ints_per_chan(max_count)
int max_count;
{
int i_result;
float f_result;

f_result = ((float) max_count) / (((float) sizeof(int)) * 8.0);
i_result = max_count / (sizeof(int) * 8);
if (f_result > i_result)
	{
	++i_result;
	}
return i_result;
}

/*
*************************************************************************
* NAME
*	gsdl_init_chname_list(pDl_pc, npc, ints_per_chan,
*		pCom_table, pMod_array)
* DESCRIPTION
*	1) initializes contents of the table (i.e. array of structures).
*		1) all "modified" bits are set to 0.
*************************************************************************
*/
int gsdl_init_chname_list(pDl_pc, npc, ints_per_chan, pCom_table, pMod_array) 
struct dl_pc *pDl_pc;
int npc;
int ints_per_chan;
struct gsdl_com_record *pCom_table;
int *pMod_array;
{
int i;
int j;

for (i = 0; i < npc; ++i)
	{
	char *pChanName;
	pChanName = (char *)(pDl_pc + 1);
	
	if(strlen(pChanName) > 0)
		{
		(pCom_table + i)->count = (pDl_pc)->count;
		(pCom_table + i)->pMod_bits = pMod_array;
		for (j = 0; j < ints_per_chan; ++j)
			*pMod_array++ = 0;
		(pCom_table + i)->pData_struct = NULL;
		}
	pDl_pc = (struct dl_pc *) (((short *) pDl_pc) + pDl_pc->element.nwords);
	}

return NORMAL;
}
/*
***********************************************************************
* NAME
*	gsdl_print_com_table(pCom_table, npc, ints_per_chan)
* DESCRIPTION
*	Used in testing.  This routine prints the contents of the
*	table in order to verify:
*		1) pChan:  the channel id.
*		2) count:  the connection count for the channel.
*		3) *pMod_bits:  the int array of "modified" bits.
***********************************************************************
*/
void gsdl_print_com_table(pCom_table, npc, ints_per_chan)
struct gsdl_com_record *pCom_table;
int npc;
int ints_per_chan;
{
int i;
int j;
int *pMod_bits;
for (i = 0; i < npc; ++i)
	{
	if ((pCom_table + i)->pChan)
		printf("~check of channel id: channel name = %s\n",
			ca_name((pCom_table + i)->pChan));
	printf("~connection count = %d\n", (pCom_table + i)->count);
	pMod_bits = (pCom_table + i)->pMod_bits;
	for (j = 0; j < ints_per_chan; ++j)
		{
		printf("~modification bits %d = %x\n", j,
			*pMod_bits++);
		}
	printf("~data structure ptr = %p\n", (pCom_table + i)->pData_struct);
	}
return;
}

/*
***********************************************************************
* NAME
*	gsdl_display_event_handler(arg)
* DESCRIPTION
*	Routine sets to one each bit of the current contents of the 
*	channel's int array of modified bits.  Any partial
*	integers are filled starting with the least 
*	significant bit.
***********************************************************************
*/
void gsdl_display_event_handler(arg)
struct event_handler_args arg;
{
struct gsdl_com_record *pCom_table_record = arg.usr;
int bytes_per_int = sizeof(int);
int count = (pCom_table_record)->count;
int int_count = count / (bytes_per_int * 8);
int bit_count = count - (8 * bytes_per_int * int_count);
int i;
int new_value;
int *pMod_bits = (pCom_table_record)->pMod_bits;
/*
gsdl_macro_show_events("event handler executed for [%s]\n",
	ca_name(pCom_table_record->pChan));
*/

#ifdef PRINT_CA_EVENTS
	struct timeval time3;
	gettimeofday(&time3, NULL);
	printf("%s: %08X%08X MOND, type = %d\n", ca_name(arg.chid), time3.tv_sec,
		time3.tv_usec,arg.type);
	fflush(stdout);
#endif

gsdl_sts_update_storage(arg);
for (i = 0; i < int_count; ++i)
	{
	new_value = 0;
	new_value = ~new_value;
	(*pMod_bits++) = (*pMod_bits) | new_value;
	}
new_value = 0;
new_value = ~new_value;
*pMod_bits = (*pMod_bits) | (~ (new_value << bit_count));

/*
Change by Gordon at Tate: since we don't remove the monitor and do
a read callback on a reconnect (as opposed to the original connect)
then this event should set the valid data flag.
*/

pCom_table_record -> valid_data = 1;

caHasUpdated = 1;	/* Global flag to indicate that something has changed. */

return ;
}

/*
***********************************************************************
* NAME
*	gsdl_add_chan_event(pCom_table)
* DESCRIPTION
*	This routine adds an event for the channel pointed
*	to by pCom_table. 
***********************************************************************
*/
int gsdl_add_chan_event(pCom_table)
struct gsdl_com_record *pCom_table;
{
gsdl_macro_show_events("~add event  executed for [%s]\n",
	ca_name(pCom_table->pChan));
SEVCHK(ca_add_array_event(dbf_type_to_DBR_STS(ca_field_type(pCom_table->pChan)),
		ca_element_count(pCom_table->pChan),
		pCom_table->pChan, gsdl_display_event_handler, 
		pCom_table, (float) 0, (float) 0, (float) 0, &(pCom_table->pevid)), 
		"~gsdl_add_chan_event:  add event failed\n");
/*ca_flush_io();*/
return NORMAL;
}

/*
**************************************************************
* NAME
*	gsdl_connection_event_handler(handler_args)
* DESCRIPTION
*	1) If connection is up
*		1) allocate the necessary memory for the native type
*		   ctrl structure and store its addr in the com table.
*		2) if allocation failed then
*			1) print failure message.
*			2) return.
*		3) Establish a get with callback event with ctrl version
*		   of native type.
*		4) Flush I/O to immediatly initiate the individual
*		   channel get with callback.
*	   else
*		1) Must wait until channel's Mod bits go to zero, indicating
*		   that the display manager has process the pData_struct data,
*		   since this structure is about to be freed (for VXWORKS).
*		   For the UNIX case the op_comm program forces the Mod bits
*		   to zero, since we are executing single threaded, we
*		   know that the Display Manager is not simultaneously
*		   trying to access this storage that is about to be freed.
*		2) return.
* NOTES:
*	1) This allocation and freeing of the data structure is intended
*	   the gracefully handle the situation of a channel being pwred
*	   down, and then its native type and/or native count being
*	   changed, and then pwredup; while the OPI software is still running.
*	2) Note:!!! This embedded wait is dangerous, can deadlock occur with
*	   the display manager? !!!  May need to find another way to accomplish
*	   this without the inherent wait for the display manager to 
*	   finish its processing.
**************************************************************
*/
void gsdl_connection_event_handler (handler_args)
struct connection_handler_args handler_args;
{
struct gsdl_com_record *pCom_record = ca_puser(handler_args.chid);
static void *ptr = NULL;

#ifdef PRINT_CA_EVENTS
	struct timeval time1;
	gettimeofday(&time1, NULL);
#endif
if (handler_args.op == CA_OP_CONN_UP)  
	{
	int cnt;
#ifdef PRINT_CA_EVENTS
	printf("%s: %08X%08X CONN, type = %d\n", ca_name(handler_args.chid), time1.tv_sec, time1.tv_usec,
			ca_field_type(handler_args.chid));
	fflush(stdout);
#endif
	gsdl_macro_show_events("~connection event handler executed for [%s]\n",
		ca_name(pCom_record->pChan));

	ca_complete_bld_read_mon(handler_args.chid);
	cnt = ca_element_count(handler_args.chid);
	if (cnt == 0)
	    cnt = 1;
	if (pCom_record -> pData_struct)
	    {
	    /*
	    Change by Gordon at Tate:

	    This is a reconnect (as opposed to the original connection). Make cause
	    an event to happen. The event handler has been modified to set the valid
	    data flag. That will get display list processing to redisplay the object.
	    */

	    ca_array_get_callback(dbf_type_to_DBR_STS(ca_field_type(handler_args.chid)),
		cnt, handler_args.chid, gsdl_display_event_handler, pCom_record);
	    return;
	    }
	else
	    {
	    pCom_record->pData_struct = (void *) calloc(1,
	           dbr_size_n(dbf_type_to_DBR_CTRL(ca_field_type(handler_args.chid))
		    , cnt));

	    if (pCom_record->pData_struct == NULL)
		    {
		    printf("~gsdl_connection_event_handler: mem alloc failed.\n");
		    return;
		    }
	    }
	}
else{
#ifdef PRINT_CA_EVENTS
	printf("%s: %08X%08X DISC\n", ca_name(handler_args.chid), time1.tv_sec, time1.tv_usec);
	fflush(stdout);
#endif

	/*
	Change by Gordon at Tate: Don't do this. Let the monitor persist
	within channel access. When we get reconnected, just do a read
	callback that will get processed as an event. Then, that will
	set the modified data flag.

DELETED	if(pCom_record->pevid){
DELETED		SEVCHK(ca_clear_event(pCom_record->pevid),
DELETED		"gsdl_connection_event_handler:  clear event failed\n");
	****** END DELETED CODE */

	pCom_record->valid_data = 0;
	caHasUpdated = 1;

	/*
	This line goes along with the deleted code above.
DELETED	}
	*/
}
return;
}
/*
**************************************************************
* NAME
*	gsdl_broadcast_search(pCom_table, pDl_pc, np, pMac_pairs)
* DESCRIPTION
*	1) For all channel names in the pDl_pc list:
*		1) Broadcast search.
*	2) Flush I/O to initiate requests.
**************************************************************
*/
 
int gsdl_broadcast_search(pCom_table, pDl_pc, npc, pMac_pairs)
struct gsdl_com_record *pCom_table;
struct dl_pc *pDl_pc;
int npc;
char *pMac_pairs[];
{
int i;

for (i = 0; i < npc; ++i)
	{
	char *pChanName;
	pChanName = (char *)(pDl_pc + 1);

	/* set up ca if valid string */
	if(strlen(pChanName) > 0 && *pChanName != '.'){
	    char pNew[db_name_dim];

	    if(pMac_pairs != NULL){
	        gsdl_main_macro( pNew, pChanName, pMac_pairs, db_name_dim);
		pChanName = pNew;
	    }
#if DEBUG || defined(PRINT_CA_EVENTS)
printf("CABRM: channel:(%s)\n",pChanName);
#endif
	    SEVCHK(ca_build_read_and_monitor(pChanName, gsdl_connection_event_handler,
			pCom_table, gsdl_callback_handler, pCom_table,
			gsdl_display_event_handler, pCom_table, &(pCom_table -> pChan),
			&(pCom_table -> pevid)),
		    "~gsdl_broadcast_search:  search queueing failed\n");
	}
	else
	    pCom_table->pChan = NULL;
	++pCom_table;
	pDl_pc = (struct dl_pc *) (((short *) pDl_pc) + pDl_pc->element.nwords);
	}
/*ca_flush_io();*/

return NORMAL;
}
/*
**************************************************************
* NAME
*	gsdl_callback_handler(arg)
* DESCRIPTION
*	1) Store results of the start-up control dbr type.
*	   (initiated from the previous connection event handler)
*	2) Establish event with coerced sts type.
**************************************************************
*/
void gsdl_callback_handler(arg)
struct event_handler_args arg;
{
#ifdef PRINT_CA_EVENTS
	struct timeval time2;
	gettimeofday(&time2, NULL);
	printf("%s: %08X%08X READ, type = %d\n", ca_name(arg.chid), time2.tv_sec, time2.tv_usec, arg.type);
	fflush(stdout);
#endif
	if (dbr_type_is_CTRL(arg.type)) {
		struct gsdl_com_record *pCom_record = arg.usr;
		unsigned buf_size = dbr_size_n(arg.type, 
			ca_element_count(arg.chid));
		char *pTo_buf = (char *) pCom_record->pData_struct;
		char *pFrom_buf = (char *) arg.dbr;
		unsigned i;

		gsdl_macro_show_events("~callback event handler executed [%s]\n",
			ca_name(pCom_record->pChan));

		for (i = 0; i < buf_size; i++) {
			*pTo_buf++ = *pFrom_buf++;
		}
		/* set flag to indicate that access to this chan data is valid */
		pCom_record ->valid_data = 1;
		caHasUpdated = 1;
	}
	return;
}

/*
**************************************************************
* NAME
*	gsdl_sts_update_storage(arg)
* DESCRIPTION
*	1) Store results of the sts dbr type.
*	   (initiated from the previous callback event handler)
**************************************************************
*/
void gsdl_sts_update_storage(arg)
struct event_handler_args arg;
{
	if (dbr_type_is_STS(arg.type)) {
		struct gsdl_com_record *pCom_record = arg.usr;
		unsigned buf_size = dbr_size_n(arg.type, 
			ca_element_count(arg.chid)) -
			dbr_size[arg.type];
		char *pTo_buf = ((char *) pCom_record->pData_struct) + 
			dbr_size[dbf_type_to_DBR_CTRL(ca_field_type(arg.chid))];
		char *pFrom_buf = ((char *) arg.dbr) + dbr_size[arg.type];
		void *pBuf = pCom_record->pData_struct;
		unsigned i;

		for (i = 0; i < buf_size; i++) {
			*pTo_buf++ = *pFrom_buf++;
		}

		switch (arg.type) {
		case DBR_STS_STRING: 
			((struct dbr_sts_string *) pBuf)->status =
				((struct dbr_sts_string *) arg.dbr)->status;
			((struct dbr_sts_string *) pBuf)->severity =
				((struct dbr_sts_string *) arg.dbr)->severity;
			strncpy(((struct dbr_sts_string *) pBuf)->value,
				((struct dbr_sts_string *) arg.dbr)->value,
				MAX_STRING_SIZE);
			break;
		case DBR_STS_SHORT:
			((struct dbr_ctrl_short *) pBuf)->status =
				((struct dbr_sts_short *) arg.dbr)->status;
			((struct dbr_ctrl_short *) pBuf)->severity =
				((struct dbr_sts_short *) arg.dbr)->severity;
			((struct dbr_ctrl_short *) pBuf)->value =
				((struct dbr_sts_short *) arg.dbr)->value;

	/*		printf("value = %d\n", 
				((struct dbr_ctrl_short *) pBuf)->value);
			printf("status = %x\n", 
				((struct dbr_ctrl_short *) pBuf)->status);
			printf("severity = %x\n", 
				((struct dbr_ctrl_short *) pBuf)->severity);
	*/
			break;
		case DBR_STS_FLOAT:
			((struct dbr_ctrl_float *) pBuf)->status =
				((struct dbr_sts_float *) arg.dbr)->status;
			((struct dbr_ctrl_float *) pBuf)->severity =
				((struct dbr_sts_float *) arg.dbr)->severity;
			((struct dbr_ctrl_float *) pBuf)->value =
				((struct dbr_sts_float *) arg.dbr)->value;

	/*		printf("value = %f\n", 
				((struct dbr_ctrl_float *) pBuf)->value);
			printf("status = %x\n", 
				((struct dbr_ctrl_float *) pBuf)->status);
			printf("severity = %x\n", 
				((struct dbr_ctrl_float *) pBuf)->severity);
	*/
			break;
		case DBR_STS_ENUM:
			((struct dbr_ctrl_enum *) pBuf)->status =
				((struct dbr_sts_enum *) arg.dbr)->status;
			((struct dbr_ctrl_enum *) pBuf)->severity =
				((struct dbr_sts_enum *) arg.dbr)->severity;
			((struct dbr_ctrl_enum *) pBuf)->value =
				((struct dbr_sts_enum *) arg.dbr)->value;
			break;
		case DBR_STS_CHAR:
			((struct dbr_ctrl_char *) pBuf)->status =
				((struct dbr_sts_char *) arg.dbr)->status;
			((struct dbr_ctrl_char *) pBuf)->severity =
				((struct dbr_sts_char *) arg.dbr)->severity;
			((struct dbr_ctrl_char *) pBuf)->value =
				((struct dbr_sts_char *) arg.dbr)->value;
			break;
		case DBR_STS_LONG:
			((struct dbr_ctrl_long *) pBuf)->status =
				((struct dbr_sts_long *) arg.dbr)->status;
			((struct dbr_ctrl_long *) pBuf)->severity =
				((struct dbr_sts_long *) arg.dbr)->severity;
			((struct dbr_ctrl_long *) pBuf)->value =
				((struct dbr_sts_long *) arg.dbr)->value;
			break;
		case DBR_STS_DOUBLE:
			((struct dbr_ctrl_double *) pBuf)->status =
				((struct dbr_sts_double *) arg.dbr)->status;
			((struct dbr_ctrl_double *) pBuf)->severity =
				((struct dbr_sts_double *) arg.dbr)->severity;
			((struct dbr_ctrl_double *) pBuf)->value =
				((struct dbr_sts_double *) arg.dbr)->value;
			break;
		default:
			break;
		}

	}
	return;
}

/*
******************************************************
* NAME
*	gsdl_any_mods_set(pCom_record)
* DESCRIPTION
*	1) uses pCom_record->count to calculate the number of bits used to
*	   represent this channel's Mod_bits integer array
*	2) calculates the bitwise "or" of all integer array elements comprising
*	   the Mod_bits array section of this channel.  Note that the last
*	   element may be only partially used, so the unused bits are masked
*	   out.
*	3) returns the result of the above "or".
******************************************************
*/
int gsdl_any_mods_set(pCom_record)
struct gsdl_com_record *pCom_record;
{
	int i;
	int bytes_per_int = sizeof(int);
	int count = pCom_record->count;
	int int_count = count / (bytes_per_int * 8);
	int bit_count = count - (8 * bytes_per_int * int_count);

	int *pMod_bits = pCom_record->pMod_bits;
	int result = 0;
	for (i = 0; i < int_count; ++i) {
		result = (*pMod_bits++) | result;
	}
	if (bit_count > 0) {
		result = (*pMod_bits & (~ ((~0) << bit_count))) | result;
	}
	return result;
}

/*
******************************************************
* NAME
*	gsdl_fd_register(pfdctx, fd,condition)
* DESCRIPTION
*	1) if (condition is true) then
*		1) add file descriptor to fd manager
*	else
*		1) delete file descriptor from fd manager
******************************************************
*/
void gsdl_fd_register(pfdctx, fd, condition)
void	*pfdctx;
int	fd;
int	condition;
{
	if(condition){
		fdmgr_add_fd(pfdctx, fd, gsdl_ca_service, NULL);
	}
	else{
		if (!inNormalExit)
			/*======ABORT!! CA indicates failure!======*/
			abort();
		fdmgr_clear_fd(pfdctx, fd);
	}
}

/*
******************************************************
* NAME
*	gsdl_ca_service()
* DESCRIPTION
*	1) call ca_pend_event to allow event handler execution
******************************************************
*/
void gsdl_ca_service()
{
#ifdef CA_TIME_CHECK
	struct timeval tp,tp1;
 	struct timezone tzp;

  	gettimeofday(&tp1, &tzp);
#endif
	ca_pend_event(PEND_EVENT_DELAY);
#ifdef CA_TIME_CHECK
 	gettimeofday(&tp1, &tzp);
	printf(" %d %d\n", tp.tv_sec, tp.tv_usec);
  	printf(" %d %d\n", tp1.tv_sec, tp1.tv_usec);
#endif
}

/*
******************************************************
* NAME
*	gsdl_clear_mod_bits(pCom_record)
* DESCRIPTION
*	1) calculate the number of intergers needed to hold this
*	   channel's mod bits.
*	2) clear to 0 all of this channel's mod bits.
******************************************************
*/
void gsdl_clear_mod_bits(pCom_record)
struct gsdl_com_record *pCom_record;
{
	float f_count = ((float) pCom_record->count) /
		(((float) sizeof(int)) * 8.0);
	int i_count = (pCom_record->count) / (8 * sizeof(int));
	int *pMod_bits = pCom_record->pMod_bits;
	int i;

	if (f_count > i_count)
		++i_count;

	for (i = 0; i < i_count; ++i) {
		*pMod_bits++ = 0;
	}
	return;
}


