/*edd.c,v 1.1.1.1 1999/08/03 14:43:52 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	E D D . C
 *	Author	Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	OPI editor root module - contains main()
 *
 *
 *
 *
 *	Improvements
 *	01	Turn on backing store on the edit window ?
 *
 *
 *	History
 *	joh 031991	Added command line display name entry
 *
 */

#include	<stdio.h>
#ifdef SOLARIS
#include	<string.h>
#else
#include	<strings.h>
#endif
#include	<fcntl.h>
#include	<Xlib.h>
#include	<Xutil.h>
#include	<Xatom.h>
#include	<sys/time.h>
#ifdef AXP
#include <stdlib.h>
#endif
#ifdef TOKENPTRS
#include <tokenSupport.h>
#endif

#include	<cursorfont.h>
#include	<select_stipple.h>

#define		EDD_GLOBAL_SOURCE
#include	<edd.h>
#include	<dlinfo.h>
#include	<plain_menus.h>

#define		EDD_MENUS_GLOBAL_SOURCE
#include	<edd_menus.h>
#include	<edd_icon.h>

extern void *fdmgr_init();
extern void *tisMalloc();
extern void *tisCalloc();
extern void tisFree();

/*
 * Command Line Options
 *
 *	lifted from /.../lib/Xt/Initialize.c
 *
 *	most of these are not integrated yet
 */
static XrmOptionDescRec opTable[] = {
{"+rv",				"*reverseVideo",	XrmoptionNoArg,  (caddr_t) "off"},
{"+synchronous",		"*synchronous",		XrmoptionNoArg,  (caddr_t) "off"},
{"-background",			"*background",		XrmoptionSepArg, (caddr_t) NULL},
{"-bd",				"*borderColor",		XrmoptionSepArg, (caddr_t) NULL},
{"-bg",				"*background",		XrmoptionSepArg, (caddr_t) NULL},
{"-bordercolor",		"*borderColor",		XrmoptionSepArg, (caddr_t) NULL},
{"-borderwidth",		".borderWidth",		XrmoptionSepArg, (caddr_t) NULL},
{"-bw",				".borderWidth",		XrmoptionSepArg, (caddr_t) NULL},
{"-display",			".display",		XrmoptionSepArg, (caddr_t) NULL},
{"-fg",				"*foreground",		XrmoptionSepArg, (caddr_t) NULL},
{"-fn",				"*font",		XrmoptionSepArg, (caddr_t) NULL},
{"-font",			"*font",		XrmoptionSepArg, (caddr_t) NULL},
{"-foreground",			"*foreground",		XrmoptionSepArg, (caddr_t) NULL},
{"-geometry",			".geometry",		XrmoptionSepArg, (caddr_t) NULL},
{"-iconic",			".iconic",		XrmoptionNoArg,  (caddr_t) "on"},
{"-name",			".name",		XrmoptionSepArg, (caddr_t) NULL},
{"-reverse",			"*reverseVideo",	XrmoptionNoArg,  (caddr_t) "on"},
{"-rv",				"*reverseVideo",	XrmoptionNoArg,  (caddr_t) "on"},
{"-selectionTimeout",		".selectionTimeout",	XrmoptionSepArg, (caddr_t) NULL},
{"-synchronous",		"*synchronous",		XrmoptionNoArg,  (caddr_t) "on"},
{"-title",			".title",		XrmoptionSepArg, (caddr_t) NULL},
{"-xnllanguage",		".xnlLanguage",		XrmoptionSepArg, (caddr_t) NULL},
{"-xrm",			NULL,			XrmoptionResArg, (caddr_t) NULL},
};

static char	*GetHomeDir();
static 		check_all_string_offsets();
char		*strrchr();
int		opi_x_error_handler();

void		*pfdctx;
static struct timeval	timeout	= {30, 0};

int	invisibleColor = DEFAULT_INVISIBLE_COLOR;
int	converting = 0;
char	*opiStringTab[OPI_NUM_STRINGS];

/*
 *	MAIN
 */
main(argc, argv)
int	argc;
char	**argv;
	{
	void			x_evhandler();
	due			*pdue;
	XSizeHints		xsh;
	XGCValues		GC_values;
	XSetWindowAttributes	xswa;
	Status			status;
	XColor			dbclr;
	enum			args{program_name, display_name};
	XrmValue		value;
	char			*ptype;
	char			Xdefaults[BUFSIZ];
	char			*pdisplay_name	= "";
	int			Cmd;

	/*
	Handle TIS Licensing
	*/

#ifndef ultrix
	tisSoftLicense();
#endif
/*
	Load all strings here.
*/
	if (getenv ("OPISTRINGTABLE"))
	    tisLoadStrings (getenv ("OPISTRINGTABLE"), opiStringTab, OPI_NUM_STRINGS);
	else
#if !(defined(AXP) || defined(SOLARIS))
	    tisLoadStrings("/home/tis4000/tasks/opiStringTable",opiStringTab,OPI_NUM_STRINGS);
#else
	    tisLoadStrings("/tis4000/tasks/opiStringTable",opiStringTab,OPI_NUM_STRINGS);
#endif
	einfoStringInit();			/* dialog box extra strings */
	sinfoStringInit();			/* dialog box strings */
/*
	28-Nov-1994 GCT
	Initialize menu strings here. This is done because the compiler doesn't want to
	at compile time.
*/

	modifymc[0] = "~attributes";
	modifymc[1] = "~color table";
	modifymc[2] = "~color rules "; 
	modifymc[3] = 0;

	globalmc[0] = "~load     display";
	globalmc[1] = "~modify   display";
	globalmc[2] = "~save     display";
	globalmc[3] = "~copy     display";
	globalmc[4] = "~report   display";
	globalmc[5] = "~activate display";
	globalmc[6] = "~quit";
	globalmc[7] = 0;

	monitormc[0] = "~indicator";
	monitormc[1] = "~bar";
	monitormc[2] = "~text";
	monitormc[3] = "~meter";
	monitormc[4] = 0;

	controlmc[0] = "~valuator";
	controlmc[1] = "~text entry";
	controlmc[2] = "~choice button";
	controlmc[3] = "~message button";
	controlmc[4] = "~menu";
	controlmc[5] = "~related display callup";
	controlmc[6] = "~execute script";
	controlmc[7] = "~kill display";
	controlmc[8] = 0;

	plotmc[0] = "~cartesian plot";
	plotmc[1] = "~surface plot";
	plotmc[2] = "~strip chart";
	plotmc[3] = "~smith overlay plot";
	plotmc[4] = 0;

	createmc[0] = "~rectangle";
	createmc[1] = "~line";
	createmc[2] = "~oval";
	createmc[3] = "~arc";
	createmc[4] = "~text";
	createmc[5] = "~monitor";
	createmc[6] = "~controller";
	createmc[7] = "~plot";
	createmc[8] = 0;

	nseditmc[0] = "~create";
	nseditmc[1] = "~paste"; 
	nseditmc[2] = "~undo";
	nseditmc[3] = "~select all";
	nseditmc[4] = 0;

	distributemc[0] = "~left edges";
	distributemc[1] = "~right edges";
	distributemc[2] = "~top edges";
	distributemc[3] = "~bottom edges";
	distributemc[4] = "~horizontal centers";
	distributemc[5] = "~vertical centers";
	distributemc[6] = 0;

	alignmc[0] = "~left edges";
	alignmc[1] = "~right edges";
	alignmc[2] = "~top edges";
	alignmc[3] = "~bottom edges";
	alignmc[4] = "~horizontal centers";
	alignmc[5] = "~vertical centers";
	alignmc[6] = "~centers";
	alignmc[7] = 0;

	moremc[0] = "~group";
	moremc[1] = "~ungroup"; 
	moremc[2] = "~front"; 
	moremc[3] = "~back";
	moremc[4] = "~align";
	moremc[5] = "~distribute";
#ifdef UNFINISHED
	moremc[6] = "~open";
	moremc[7] = "~close";
#endif
	moremc[8] = 0;

	seditmc[0] = "~modify";
	seditmc[1] = "~modify channel names";
	seditmc[2] = "~cut"; 
	seditmc[3] = "~copy"; 
	seditmc[4] = "~move";
	seditmc[5] = "~resize";
	seditmc[6] = "~deselect";
	seditmc[7] = "~more";
	seditmc[8] = 0;

	rmtseditmc[0] = "~cut";
	rmtseditmc[1] = "~copy";
	rmtseditmc[2] = "~deselect";
	rmtseditmc[3] = 0;

	/* Check if we are performing a conversion from dl to adl format. */

	if (argc == 4) {
	    if (strcmp (argv [1], "-a") == 0) {

		FILE	*f;

		printf ("~Converting %s dl file to %s adl file.\n",
			argv [2], argv [3]);

		f = fopen (argv [2], "r");
		if (f == NULL) {
		    printf ("~Unable to open %s\n", argv [2]);
		    exit (-1);
		}
		fclose (f);

		f = fopen (argv [3], "w");
		if (f == NULL) {
		    printf ("~Unable to create %s\n", argv [3]);
		    exit (-1);
		}

		dump_display (argv [2], f, argv [2]);

		fclose (f);
		
		exit (0);
	    } else if (strcmp (argv [1], "-d") == 0) {
		int	fd;

		printf ("~Converting %s adl file to %s dl file.\n",
			argv [2], argv [3]);

		fd = open (argv [2], O_RDONLY);
		if (fd == -1) {
		    printf ("~Unable to open %s\n", argv [2]);
		    exit (-1);
		}

		/*pdue = (due *) calloc(1, sizeof(due));*/
		pdue = (due *) tisCalloc(__FILE__, __LINE__, 1, sizeof(due));

		if ((pdue->d = XOpenDisplay(pdisplay_name)) == NULL) {
		    printf( "~edd: can't open display [%s]\n", pdisplay_name);
		    return ERROR;
		}

/*		init_due (pdue); */
		converting = 1;
		load_adl (pdue, fd);
		close (fd);
		save_display (pdue, argv [3]);
		exit (0);
	    }		
	}

	check_all_string_offsets();

	/*
	 * install an error handler that makes a core file when
	 * an X Error occurs. This can be usefull if XSync is on.
	 *
	 */
	XSetErrorHandler(opi_x_error_handler);

	pdue = (due *) tisCalloc(__FILE__, __LINE__, 1, sizeof(due));
	assert(pdue != NULL);

	/*
	 * Extract standard command line options and store them in a X resource manager database.
	 */
	XrmInitialize();
	XrmParseCommand(&pdue->xrmdb, opTable, NELEMENTS(opTable), "edd", &argc, argv);
	XrmMergeDatabases(XrmGetFileDatabase(strcat(GetHomeDir(Xdefaults), "/.Xdefaults")), &pdue->xrmdb);

	/*
	 * get any display name and open it
	 */
	if(XrmGetResource(pdue->xrmdb, "edd.display", "TopLevelShell", &ptype, &value) == True)
		pdisplay_name = value.addr;

	if ((pdue->d = XOpenDisplay(pdisplay_name)) == NULL)
		{
		printf( "~edd: can't open display [%s]\n", pdisplay_name);
		return ERROR;
		}

	/*
	 * initialize to fd manager and give it the X fd for X events
	 */
	pfdctx = fdmgr_init();
	assert(pfdctx != NULL);
	fdmgr_add_fd(pfdctx, ConnectionNumber(pdue->d), x_evhandler, pdue);

	/*
	 * get any sync value
	 */
	if(XrmGetResource(pdue->xrmdb, "edd.synchronous", "TopLevelShell", &ptype, &value) == True)
		if(strcmp(value.addr,"on") == 0)
			{
			printf("~edd: X sync is on\n");
			XSynchronize(pdue->d, (Bool) TRUE);
			}

	pdue->s					= ScreenOfDisplay(pdue->d, DefaultScreen(pdue->d));
	pdue->dlc.d				= pdue->d;
	pdue->dlc.pfont_table	= init_font_table(pdue->d);
	assert(pdue->dlc.pfont_table != NULL);

	pdue->dl_seg_atom		= XInternAtom(pdue->d, DL_SEGMENT,  False);
	pdue->XA_CLIPBOARD		= XInternAtom(pdue->d, "CLIPBOARD", False);
	pdue->XA_DELETE			= XInternAtom(pdue->d, "DELETE",    False);
	assert(pdue->dl_seg_atom  != None);
	assert(pdue->XA_CLIPBOARD != None);
	assert(pdue->XA_DELETE	  != None);

	/* This allocates a color which represents the "invisible" color.  This color is not */
	/* actually ever used but the pixel number represents the invisible color. */

/*	{
	    Status	status;
	    long	plane_mask [1];
	    long	ppixel [1];
	    XColor	color, xcolor;

	    if (invisibleColor == DEFAULT_INVISIBLE_COLOR) {		
		status = XAllocColorCells(	
			pdue->d,
			DefaultColormap(pdue->d, DefaultScreen(pdue->d)),
			False,
			plane_mask,
			0,
			ppixel,
			1);
		assert(status != 0);
		invisibleColor = ppixel [0];
printf ("~Invisible color is %d\n", invisibleColor);


	    }
	}
*/
	/*
	 * queries will slow down startup somewhat
	 * - could use XQueryColors
	 */

	/*
	 * get any foreground and background colors
	 */
	if(XrmGetResource(pdue->xrmdb, "edd.foreground", "TopLevelShell", &ptype, &value) == True)
		{
		char    color_string[50];

		strncpy(color_string, value.addr, (int)value.size);
		status = XAllocNamedColor(pdue->d,
					      DefaultColormapOfScreen(pdue->s),
					      color_string,
					      &pdue->foreground,
					      &dbclr);
		if(!status)
			{
			printf("~edd: bad foreground color (%s)\n", color_string);
			pdue->foreground.pixel = WhitePixelOfScreen(pdue->s);
			XQueryColor(pdue->d, DefaultColormapOfScreen(pdue->s), &pdue->foreground);
			}
		}
	else
		{
		pdue->foreground.pixel = WhitePixelOfScreen(pdue->s);
		XQueryColor(pdue->d, DefaultColormapOfScreen(pdue->s), &pdue->foreground);
		}
	if(XrmGetResource(pdue->xrmdb, "edd.background", "TopLevelShell", &ptype, &value) == True)
		{
		char    color_string[50];

		strncpy(color_string, value.addr, (int)value.size);
		status = XAllocNamedColor(pdue->d,
					      DefaultColormapOfScreen(pdue->s),
					      color_string,
					      &pdue->background,
					      &dbclr);
		if(!status)
			{
			printf("~edd: bad background color (%s)\n", color_string);
			pdue->background.pixel = BlackPixelOfScreen(pdue->s);
			XQueryColor(pdue->d, DefaultColormapOfScreen(pdue->s), &pdue->background);
			}
		}
	else
		{
		status = XAllocNamedColor(pdue->d,
					      DefaultColormapOfScreen(pdue->s),
					      "dark slate grey",
					      &pdue->background,
					      &dbclr);
		if(!status)
			{
			pdue->background.pixel = BlackPixelOfScreen(pdue->s);
			XQueryColor(pdue->d, DefaultColormapOfScreen(pdue->s), &pdue->background);
			}
		}

	/*
	 * I use the default visual since most servers can only load one
	 * color map at a time which leads me to use the default color map
	 * which must be used with the default visual.
	 */
	pdue->dlc.color = color_visual_test(pdue->d, DefaultScreen(pdue->d));

	/*
	 * convert from mm to pixels
	 */
	pdue->pixel_tolerence	= mm_to_xpixel(MM_TOLERENCE,		pdue->s);
	pdue->gizmo_width	= mm_to_xpixel(FORM_GIZSMO_WIDTH_MM,	pdue->s);
	pdue->gizmo_height	= mm_to_ypixel(FORM_GIZSMO_HEIGHT_MM,	pdue->s);
	pdue->gizmo_pad		= mm_to_xpixel(FORM_GIZSMO_PAD_MM,      pdue->s);
	pdue->xselect_border	= mm_to_xpixel(SELECT_BORDER_MM,	pdue->s);
	pdue->yselect_border	= mm_to_ypixel(SELECT_BORDER_MM,	pdue->s);

	/*
	 * read in the default load path from an environment variable
	 */
	pdue->penv_path = getenv("EPICS_DISPLAY_PATH");

	/*
	 * Setup the default display under edit parameters
	 * (must be done after dlc.color is set)
	 */
	init_due(pdue);

	/*
	 * setup the header geometry
	 */
	pdue->header_geom.x		= pdue->gizmo_height;
	pdue->header_geom.y		= pdue->gizmo_height;
	pdue->header_geom.width		= pdue->dl_hdr.object.width;
	pdue->header_geom.height	= pdue->gizmo_height*3;

	/*
	 * let them size down the display under edit no smaller than 3 gizmo widths
	 */
	xsh.min_height	= pdue->header_geom.height + pdue->gizmo_height*6;
	xsh.min_width	= pdue->gizmo_height*5; 
	xsh.width	= pdue->dl_hdr.object.width  + pdue->gizmo_height*2;
	xsh.height	= pdue->dl_hdr.object.height + pdue->header_geom.height + pdue->gizmo_height*3;
	xsh.flags	= PSize|PPosition|PMinSize;

	pdue->edd_geom.x	= xsh.x	= 0;
	pdue->edd_geom.y	= xsh.y	= 0;
	pdue->edd_geom.width	= xsh.width;
	pdue->edd_geom.height	= xsh.height;

	/*
	 * Create the top level window with input from the wm its an IO
	 * window to accomadate its siblings
	 */
	xswa.border_pixel	= pdue->foreground.pixel;
	xswa.bit_gravity	= NorthWestGravity;
	xswa.background_pixel	= pdue->background.pixel;
	xswa.event_mask		= StructureNotifyMask|ExposureMask|ButtonPressMask|PLAIN_MENU_MIN_EVENT_MASK;
	pdue->w = XCreateWindow(pdue->d,
				DefaultRootWindow(pdue->d),
				xsh.x,
				xsh.y,
				xsh.width,
				xsh.height,
				DUE_BORDER_WIDTH,
				CopyFromParent,		/* take depth from parent */
				InputOutput,
				CopyFromParent,		/* use the default visual */
				CWBorderPixel|CWBitGravity|CWEventMask|CWBackPixel,
				&xswa);
	{
		XWMHints	xwmh;
		char		banner[100];

		sprintf(banner, "~TIS4000 Display Editor V%d.%d", DL_MAJOR_VERSION, DL_MINOR_VERSION);
		XSetNormalHints(pdue->d, pdue->w, &xsh);
		XStoreName(pdue->d, pdue->w, banner);
		XSetIconName(pdue->d, pdue->w, "edd");

		xwmh.window_group	= pdue->w;
		xwmh.input		= TRUE;
		xwmh.icon_pixmap	= XCreateBitmapFromData(pdue->d, 
								pdue->w,
								edd_icon_bits,
								edd_icon_width,
								edd_icon_height);
		xwmh.flags		= WindowGroupHint|IconPixmapHint|InputHint;
		XSetWMHints(pdue->d, pdue->w, &xwmh);
	}

	/*
	 * Create the display under edit window
	 */
	xswa.border_pixel	= CLIPCLRIX(pdue, pdue->dl_hdr.clr);
	xswa.background_pixel	= CLIPCLRIX(pdue, pdue->dl_hdr.bclr);
	xswa.bit_gravity	= NorthWestGravity;
	xswa.event_mask		= ExposureMask|ButtonPressMask|PLAIN_MENU_MIN_EVENT_MASK;
	pdue->dlc.w = XCreateWindow(pdue->d,
				pdue->w,
				pdue->gizmo_height,				/* x */
				pdue->gizmo_height*2+pdue->header_geom.height,	/* y */
				pdue->dl_hdr.object.width,
				pdue->dl_hdr.object.height,
				DUE_BORDER_WIDTH,
				CopyFromParent,		/* take depth from parent */
				InputOutput,
				CopyFromParent,		/* use the default visual */
				CWBorderPixel| CWBackPixel| CWBitGravity| CWEventMask,
				&xswa);

	GC_values.foreground	= pdue->background.pixel;
	GC_values.background	= pdue->foreground.pixel;
	pdue->back_GC		= XCreateGC(pdue->d, pdue->w, GCForeground|GCBackground, &GC_values);

	GC_values.foreground	= pdue->foreground.pixel;
	GC_values.background	= pdue->background.pixel;
	pdue->fore_GC		= XCreateGC(pdue->d, pdue->w, GCForeground|GCBackground, &GC_values);

	GC_values.foreground	= pdue->foreground.pixel;
	GC_values.background	= pdue->background.pixel;
	GC_values.plane_mask	= 1; 
	GC_values.line_style	= LineOnOffDash; 
	GC_values.function	= GXinvert; 
	pdue->invert_GC		= XCreateGC(pdue->d, pdue->w,
								GCForeground|GCBackground|GCPlaneMask|GCFunction|GCLineStyle, &GC_values);

	GC_values.line_style	= LineDoubleDash; 
	GC_values.dashes	= pdue->pixel_tolerence; 
	pdue->dyn_GC		= XCreateGC(pdue->d, pdue->w, GCLineStyle|GCDashList, &GC_values);

	GC_values.foreground	= pdue->foreground.pixel;
	GC_values.background	= pdue->background.pixel;
	GC_values.cap_style	= CapProjecting;
	GC_values.join_style	= JoinRound;
	GC_values.fill_rule	= WindingRule;
	pdue->dlc.gc		= XCreateGC(pdue->d, pdue->w, GCForeground|GCBackground|GCCapStyle|GCJoinStyle|GCFillRule, &GC_values);

	GC_values.foreground	= pdue->foreground.pixel;
	GC_values.background	= pdue->background.pixel;
	pdue->select_GC		= XCreateGC(pdue->d, pdue->w, GCForeground|GCBackground|GCCapStyle|GCJoinStyle|GCFillRule, &GC_values);

	pdue->create_cursor	= XCreateFontCursor(pdue->d, XC_dotbox);
	pdue->pick_cursor	= XCreateFontCursor(pdue->d, XC_crosshair);
	XRecolorCursor( pdue->d, pdue->create_cursor, &pdue->foreground, &pdue->background);
	XRecolorCursor( pdue->d, pdue->pick_cursor, &pdue->foreground, &pdue->background);

	pdue->dlc.pstring_table = (char **) &pdue->st;

	init_palette_menus(pdue->d);
	init_menu(pdue->d, &global_mc), update_color_rule_menu(pdue);
	init_menu(pdue->d, &nsedit_mc);
	init_menu(pdue->d, &sedit_mc);
	init_menu(pdue->d, &rmtsedit_mc);

	XMapWindow(pdue->d, pdue->dlc.w);
	XMapWindow(pdue->d, pdue->w);

	XSetSelectionOwner(pdue->dlc.d, XA_PRIMARY, None, CurrentTime);

	/*
	 * rudimentary command line arguments
	 */
	if(argc > display_name)
		{
		char	*pslash;

		pslash = strrchr(argv[display_name], '/');
		if(pslash)
			{
			memcpy(pdue->pldpath, argv[display_name], pslash-argv[display_name]);
			strcpy(pdue->pldname, pslash+1);
			}
		else
			{
			pdue->pldpath[0] = 0;
			strcpy(pdue->pldname, argv[display_name]);
			}
		load_display(pdue);
		}
	else
		/*
		 * resize windows for the default display
		 */
		configure_due(pdue, NULL);

	/*
	 * service any X events that may have already been generated
	 * and wait for new X events or blink timers to expire
	 */
	x_evhandler(pdue);
	while(1)
		fdmgr_pend_event(pfdctx, &timeout);
	}

void	x_evhandler(pdue)
due	*pdue;
	{
	while (XPending(pdue->d))
		{
		XEvent  xevent;

		XNextEvent(pdue->d, &xevent);
		pdue->undo_sequence++;
		if(filter_gizmo_events(&xevent))
			continue;

		switch(xevent.type)
		  {
		  case Expose:
			if(xevent.xany.window == pdue->dlc.w)
				{
				XRectangle	rec;
				
				rec.x		= xevent.xexpose.x;
				rec.y		= xevent.xexpose.y;
				rec.width	= xevent.xexpose.width;
				rec.height	= xevent.xexpose.height;
				redraw_dl(pdue, &rec);
				}
			if(xevent.xany.window == pdue->w)
				{
				int	x2  = pdue->header_geom.x + pdue->header_geom.width;
				int	y2  = pdue->header_geom.y + pdue->header_geom.height;
				int	ex2 = xevent.xexpose.x + xevent.xexpose.width;
				int	ey2 = xevent.xexpose.y + xevent.xexpose.height;

				if(xevent.xexpose.x <= x2 && xevent.xexpose.y <= y2 &&
				   ex2 >= pdue->header_geom.x && ey2 >= pdue->header_geom.y)
				   	{
					redraw_header(pdue);
					}
				}
			break;

		  case ButtonPress:
			{
			XButtonEvent    *xbe    = &xevent.xbutton;

			if (xbe->window == pdue->w)	      /* display info (top) part of frame */
				switch(xbe->button)
				  {
				  case MENU_BUTTON:
				      post_menu(&xevent, &global_mc, pdue);
				      break;
				  }
			else if(xbe->window == pdue->dlc.w)     /* display (bottom) part of frame */
				switch(xbe->button)
				  {
				  case MENU_BUTTON:
					{
					struct menu_configuration	*pmc;
					Window						owner;

					/*
					 * makes a round trip to the server
					 * (seems to not cause unreasonable delays)
					 */
					owner = XGetSelectionOwner(pdue->d, XA_PRIMARY);
					if(owner == None)
						pmc = &nsedit_mc;		/* create... */
					else
						{
						if(owner == pdue->dlc.w && pdue->selections.count)
							pmc = &sedit_mc;	/* modify... */
						else
							pmc = &rmtsedit_mc;	/* cut, copy... */
						}
					post_menu(&xevent, pmc, pdue);
					break;
					}
				  case SELECT_TOGGLE_BUTTON:
					find_selection(&xevent, pdue);
					break;
				  case SELECT_BUTTON:
					if(pdue->selections.count > 0)
						{	      
						if(!adjust_with_handle(pdue, xbe->x, xbe->y))
							{
							/*
							 * could not adjust with handle, try to move if x,y is in
							 * one of the selected objects
							 */
							if(in_a_selected_object(pdue, xbe->x, xbe->y))
								move_selections(0, pdue);
							else
								find_selection(&xevent, pdue);
							}
						}
					else
						find_selection(&xevent, pdue);
					break;
					}
				break;
				}
		  case SelectionClear:
			/* This event does not require window enable */
			if(xevent.xselectionclear.window == pdue->dlc.w)
				sel_clear_action(pdue, &xevent.xselectionclear);
			break;

		  case SelectionRequest:
			/* This event does not require window enable */
			if(xevent.xselectionrequest.owner == pdue->dlc.w)
				send_selections(pdue, &xevent.xselectionrequest);
			break;

		  case SelectionNotify:
			/* This event does not require window enable */
			if(xevent.xselection.requestor == pdue->dlc.w)
				recv_selections(pdue, &xevent.xselection);
			break;

		  case ConfigureNotify:
			if(xevent.xconfigure.window == pdue->w)
				{
				XConfigureEvent	*pxce	= &xevent.xconfigure;

				/*
				 * update their display size
				 */
				pdue->dl_hdr.object.width	= pxce->width - pdue->gizmo_height*2;
				pdue->dl_hdr.object.height	= pxce->height - pdue->header_geom.height - pdue->gizmo_height*3;
				pdue->header_geom.width		= pdue->dl_hdr.object.width;
				XResizeWindow(pdue->d, pdue->dlc.w, pdue->dl_hdr.object.width, pdue->dl_hdr.object.height);
				}
			break;

		  case ButtonRelease:	break;
		  case MotionNotify:	break;
		  case EnterNotify:	break;
		  case LeaveNotify:	break;
		  case MapNotify:		break;
		  case UnmapNotify:	break;
		  }
		}
	}



/*
 * CONFIGURE DUE
 */
#define FDCHK(PFDCMP,PFDNAME) ((PFDNAME)?strcmp((PFDCMP),(PFDNAME))==0:TRUE)

void configure_due(pdue, pfdname)
due		*pdue;
char	*pfdname;
	{
	XWindowChanges	xwc;
	int				redraw = FALSE;

	/*
	 * not an ifelse list since all are executed if pfdname is nill
	 */
	if(FDCHK("cmap",pfdname))
		{
		XRectangle	rec;

		/*
		 * background set and redraw occur in edd_install_cmap()
		 */ 
		edd_load_cmap_from_display_file(pdue);
		if(!pdue->dlc.pcmap)
			{
			add_default_color_map(pdue);
			add_default_color_rules(pdue);
			}
		rec.x		= pdue->dl_hdr.object.x;
		rec.y		= pdue->dl_hdr.object.y;
		rec.width	= pdue->dl_hdr.object.width;
		rec.height	= pdue->dl_hdr.object.height;
		redraw_dl(pdue, &rec);
		}
	if(!pdue->dlc.pcmap)
		{
		add_default_color_map(pdue);
		add_default_color_rules(pdue);
		}
	if(FDCHK("width",pfdname))
		{
		xwc.width = pdue->dl_hdr.object.width + pdue->gizmo_height*2;
		xwc.width = max(xwc.width, pdue->gizmo_height*5);
		XConfigureWindow(pdue->d, pdue->w, CWWidth, &xwc);
		}
	if(FDCHK("height",pfdname))
		{
		xwc.height = pdue->dl_hdr.object.height + pdue->header_geom.height + pdue->gizmo_height;
		xwc.height = max(xwc.height,
				pdue->dl_hdr.object.height + pdue->header_geom.height + pdue->gizmo_height*3);
		XConfigureWindow(pdue->d, pdue->w, CWHeight, &xwc);
		}
	if(FDCHK("bclr",pfdname))
		{
		XSetWindowBackground(pdue->d, pdue->dlc.w, CLIPCLRIX(pdue, pdue->dl_hdr.bclr));
		redraw=TRUE;
		}
	if(FDCHK("clr",pfdname))
		{
		XSetWindowBorder(pdue->d, pdue->dlc.w, CLIPCLRIX(pdue, pdue->dl_hdr.clr));
		}
	if(redraw)
		redraw_dl(pdue, NULL);
	}


/*
 *	REDRAW_HEADER
 *
 *	redraw the editor status window
 *
 *	NOTE: acall to XClearArea for this window is a
 *	good way to force an expose with redraw.
 */
void redraw_header(pdue)
due	*pdue;
	{
	XRectangle	rec;
	
	XClearArea(pdue->d, 
		pdue->w, 
		pdue->header_geom.x,
		pdue->header_geom.y,
		0,				/* clear out to the end of the window */
		pdue->header_geom.height+1,	/* +1 get the rectangle */
		False);

      rec.x			= pdue->header_geom.x;
      rec.y			= pdue->header_geom.y;
      rec.width		= pdue->header_geom.width;
      rec.height	= pdue->gizmo_height;
      draw_string(pdue->d, pdue->w, pdue->fore_GC, &rec, pdue->dlc.pfont_table, pdue->ppath, strlen(pdue->ppath), box_begin);

      rec.y	+= rec.height;
      draw_string(pdue->d, pdue->w, pdue->fore_GC, &rec, pdue->dlc.pfont_table, pdue->pname, strlen(pdue->pname), box_begin);

	redraw_msg(pdue);
	}

/*
 *	REDRAW_MSG
 *
 *	redraw the editor msg line
 */
void redraw_msg(pdue)
due	*pdue;
	{
	XRectangle rec;

	rec.x		= pdue->header_geom.x;
	rec.y		= pdue->header_geom.y + (pdue->gizmo_height<<1);
	rec.width	= pdue->header_geom.width;
	rec.height	= pdue->gizmo_height;
	XClearArea(pdue->d, 
			pdue->w, 
			rec.x,
			rec.y,
			0,		      /* clear out to the end of the window */
			rec.height+1,	      /* +1 get the rectangle */
			False);

	if(pdue->pmsg[0])
		draw_string(pdue->d, 
			pdue->w, 
			pdue->fore_GC, 
			&rec, 
			pdue->dlc.pfont_table, 
			pdue->pmsg,
			strlen(pdue->pmsg),
			box_begin);

	/*
	 * so messages will display while edd is busy
	 */
	XFlush(pdue->d);
	}



/*
 *	INIT_DUE
 *
 */
void init_due(pdue)
due	*pdue;
	{
	stringInitList(&pdue->st);
	stringInitList(&pdue->ct);
	open_structure(dl_hdr_sinfo, &pdue->dl_hdr, pdue);
	pdue->dl_hdr.object			= default_display_geometry;
	mmgo_to_pixelgo(pdue->d, &pdue->dl_hdr.object);
	pdue->dl_hdr.nwords_dspy	= sizeof(pdue->dl_hdr);
	pdue->dl_mdt				= FALSE;
	pdue->dl_hdr.magic			= DL_MAGIC;
	pdue->dl_hdr.majv			= DL_MAJOR_VERSION;
	pdue->dl_hdr.mnrv			= DL_MINOR_VERSION;
	pdue->dl_hdr.clr			= white;
	pdue->dl_hdr.bclr			= black;
	pdue->next_groupid			= NO_GROUP+1;

	/*
	 * set the default display path as the current wd
	 * set the display name to none
	 */
	pdue->pname[0] = 0;
#ifdef SOLARIS
	getcwd (pdue->ppath, sizeof(pdue->ppath));
#else
	getwd(pdue->ppath);
#endif

	/*
	 * create a default color map with only white and black
	 * and an empty color rule table
	 */
	add_default_color_map(pdue);
	add_default_color_rules(pdue);
	}



/*
 *	ADD_DEFAULT_COLORMAP
 *
 * create a default color map with only white and black
 */
void add_default_color_map(pdue)
due	*pdue;
	{
#define LITE	~0
#define OUT	0
	struct dl_color		dlc[number_of_default_colors];

	dlc[white].r			= LITE;
	dlc[white].g			= LITE;
	dlc[white].b			= LITE;
	dlc[white].inten		= LITE;
	dlc[white].blink		= FALSE;

	dlc[black].r			= OUT;
	dlc[black].g			= OUT;
	dlc[black].b			= OUT;
	dlc[black].inten		= OUT;
	dlc[black].blink		= FALSE;

	dlc[red].r			= LITE;
	dlc[red].g			= OUT;
	dlc[red].b			= OUT;
	dlc[red].inten			= LITE;
	dlc[red].blink			= FALSE;

	dlc[blinking_red].r		= LITE;
	dlc[blinking_red].g		= OUT;
	dlc[blinking_red].b		= OUT;
	dlc[blinking_red].inten		= LITE;
	dlc[blinking_red].blink		= TRUE;

	dlc[yellow].r			= LITE;
	dlc[yellow].g			= LITE;
	dlc[yellow].b			= OUT;
	dlc[yellow].inten		= LITE;
	dlc[yellow].blink		= FALSE;

	dlc[blinking_yellow].r		= LITE;
	dlc[blinking_yellow].g		= LITE;
	dlc[blinking_yellow].b		= OUT;
	dlc[blinking_yellow].inten	= LITE;
	dlc[blinking_yellow].blink	= TRUE;

	dlc[blue].r			= OUT;
	dlc[blue].g			= OUT;
	dlc[blue].b			= LITE;
	dlc[blue].inten			= LITE;
	dlc[blue].blink			= FALSE;

	dlc[blinking_blue].r		= OUT;
	dlc[blinking_blue].g		= OUT;
	dlc[blinking_blue].b		= LITE;
	dlc[blinking_blue].inten	= LITE;
	dlc[blinking_blue].blink	= TRUE;

	edd_create_cmap(pdue, dlc, NELEMENTS(dlc), 0, TRUE);

	if(pdue->w)
		{
		sprintf(pdue->pmsg, "~Default color map installed\n");
		redraw_msg(pdue);
		}
	}



/*
 *	ADD_DEFAULT_COLORRULES
 *
 * create a default color rules with only white and black
 */
void add_default_color_rules(pdue)
due	*pdue;
	{
	struct dl_color_rule	default_color_rule;
	int	i;

#ifndef TOKENPTRS

	/* Setup default alarm color rule */

	pdue->dl_hdr.clrrule			= stringInsert(&pdue->st, "~alarm");
	default_color_rule.name			= stringInsert(&pdue->st, "~alarm");

	default_color_rule.info[0].chan		= stringInsert(&pdue->st, "~$(C).SEVR");
	default_color_rule.info[0].value	= stringInsert(&pdue->st, "~MAJOR");
	default_color_rule.info[0].connector	= dl_clrrule_and;
	default_color_rule.info[0].operation	= dl_clrrule_eq;
	default_color_rule.info[0].clr		= black;
	default_color_rule.info[1].chan		= stringInsert(&pdue->st, "~$(C).ACK");
	default_color_rule.info[1].value	= stringInsert(&pdue->st, "~YES");
	default_color_rule.info[1].connector	= dl_clrrule_use;
	default_color_rule.info[1].operation	= dl_clrrule_eq;
	default_color_rule.info[1].clr		= blinking_red;

	default_color_rule.info[2].chan		= stringInsert(&pdue->st, "~$(C).SEVR");
	default_color_rule.info[2].value	= stringInsert(&pdue->st, "~MAJOR");
	default_color_rule.info[2].connector	= dl_clrrule_use;
	default_color_rule.info[2].operation	= dl_clrrule_eq;
	default_color_rule.info[2].clr		= red;

	default_color_rule.info[3].chan		= stringInsert(&pdue->st, "~$(C).SEVR");
	default_color_rule.info[3].value	= stringInsert(&pdue->st, "~MINOR");
	default_color_rule.info[3].connector	= dl_clrrule_and;
	default_color_rule.info[3].operation	= dl_clrrule_eq;
	default_color_rule.info[3].clr		= black;
	default_color_rule.info[4].chan		= stringInsert(&pdue->st, "~$(C).ACK");
	default_color_rule.info[4].value	= stringInsert(&pdue->st, "~YES");
	default_color_rule.info[4].connector	= dl_clrrule_use;
	default_color_rule.info[4].operation	= dl_clrrule_eq;
	default_color_rule.info[4].clr		= blinking_yellow;

	default_color_rule.info[5].chan		= stringInsert(&pdue->st, "~$(C).SEVR");
	default_color_rule.info[5].value	= stringInsert(&pdue->st, "~MINOR");
	default_color_rule.info[5].connector	= dl_clrrule_use;
	default_color_rule.info[5].operation	= dl_clrrule_eq;
	default_color_rule.info[5].clr		= yellow;

	default_color_rule.info[6].chan		= stringInsert(&pdue->st, "~$(C).SEVR");
	default_color_rule.info[6].value	= stringInsert(&pdue->st, "~INFO");
	default_color_rule.info[6].connector	= dl_clrrule_and;
	default_color_rule.info[6].operation	= dl_clrrule_eq;
	default_color_rule.info[6].clr		= black;
	default_color_rule.info[7].chan		= stringInsert(&pdue->st, "~$(C).ACK");
	default_color_rule.info[7].value	= stringInsert(&pdue->st, "~YES");
	default_color_rule.info[7].connector	= dl_clrrule_use;
	default_color_rule.info[7].operation	= dl_clrrule_eq;
	default_color_rule.info[7].clr		= blinking_blue;

	default_color_rule.info[8].chan		= stringInsert(&pdue->st, "~$(C).SEVR");
	default_color_rule.info[8].value	= stringInsert(&pdue->st, "~INFO");
	default_color_rule.info[8].connector	= dl_clrrule_use;
	default_color_rule.info[8].operation	= dl_clrrule_eq;
	default_color_rule.info[8].clr		= blue;

	for (i=9 ; i<MAX_RULE_LINES ; i++) {
	    default_color_rule.info[i].chan		= stringInsert(&pdue->st, "");
	    default_color_rule.info[i].value		= stringInsert(&pdue->st, "");
	    default_color_rule.info[i].connector	= dl_clrrule_use;
	    default_color_rule.info[i].operation	= dl_clrrule_eq;
	    default_color_rule.info[i].clr		= black;
	}

#else
	pdue->dl_hdr.clrrule			= dmTokenFromPointer64 (stringInsert (&pdue->st, "~alarm"));
	default_color_rule.name			= dmTokenFromPointer64 (stringInsert (&pdue->st, "~alarm"));

	default_color_rule.info[0].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).SEVR"));
	default_color_rule.info[0].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~MAJOR"));
	default_color_rule.info[0].connector	= dl_clrrule_and;
	default_color_rule.info[0].operation	= dl_clrrule_eq;
	default_color_rule.info[0].clr		= black;
	default_color_rule.info[1].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).ACK"));
	default_color_rule.info[1].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~YES"));
	default_color_rule.info[1].connector	= dl_clrrule_use;
	default_color_rule.info[1].operation	= dl_clrrule_eq;
	default_color_rule.info[1].clr		= blinking_red;

	default_color_rule.info[2].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).SEVR"));
	default_color_rule.info[2].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~MAJOR"));
	default_color_rule.info[2].connector	= dl_clrrule_use;
	default_color_rule.info[2].operation	= dl_clrrule_eq;
	default_color_rule.info[2].clr		= red;

	default_color_rule.info[3].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).SEVR"));
	default_color_rule.info[3].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~MINOR"));
	default_color_rule.info[3].connector	= dl_clrrule_and;
	default_color_rule.info[3].operation	= dl_clrrule_eq;
	default_color_rule.info[3].clr		= black;
	default_color_rule.info[4].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).ACK"));
	default_color_rule.info[4].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~YES"));
	default_color_rule.info[4].connector	= dl_clrrule_use;
	default_color_rule.info[4].operation	= dl_clrrule_eq;
	default_color_rule.info[4].clr		= blinking_yellow;

	default_color_rule.info[5].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).SEVR"));
	default_color_rule.info[5].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~MINOR"));
	default_color_rule.info[5].connector	= dl_clrrule_use;
	default_color_rule.info[5].operation	= dl_clrrule_eq;
	default_color_rule.info[5].clr		= yellow;

	default_color_rule.info[6].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).SEVR"));
	default_color_rule.info[6].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~INFO"));
	default_color_rule.info[6].connector	= dl_clrrule_and;
	default_color_rule.info[6].operation	= dl_clrrule_eq;
	default_color_rule.info[6].clr		= black;
	default_color_rule.info[7].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).ACK"));
	default_color_rule.info[7].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~YES"));
	default_color_rule.info[7].connector	= dl_clrrule_use;
	default_color_rule.info[7].operation	= dl_clrrule_eq;
	default_color_rule.info[7].clr		= blinking_blue;

	default_color_rule.info[8].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, "~$(C).SEVR"));
	default_color_rule.info[8].value	= dmTokenFromPointer64 (stringInsert (&pdue->st, "~INFO"));
	default_color_rule.info[8].connector	= dl_clrrule_use;
	default_color_rule.info[8].operation	= dl_clrrule_eq;
	default_color_rule.info[8].clr		= blue;

	for (i=9 ; i<MAX_RULE_LINES ; i++) {
	    default_color_rule.info[i].chan		= dmTokenFromPointer64 (stringInsert (&pdue->st, ""));
	    default_color_rule.info[i].value		= dmTokenFromPointer64 (stringInsert (&pdue->st, ""));
	    default_color_rule.info[i].connector	= dl_clrrule_use;
	    default_color_rule.info[i].operation	= dl_clrrule_eq;
	    default_color_rule.info[i].clr		= black;
	}

#endif

	default_color_rule.fg_enable		= 1;
	default_color_rule.default_fg		= white;
	default_color_rule.bg_enable		= 1;
	default_color_rule.default_bg		= black;

	edd_create_crules(pdue, &default_color_rule, 1, 0);
	}


/*
 *	INIT_BLINK
 */
struct timeval	blinkrate	= {1, 0};
due				*saved_pdue;

init_blink(pdue)
due	*pdue;
	{
	void			edd_do_blink();
	static int		first_time	= 1;

	if (first_time && !converting)
		{
		first_time = 0;		/* we only need to do this once */
		fdmgr_add_timeout(pfdctx, &blinkrate, edd_do_blink, pdue);
		saved_pdue = pdue;	/* this better not change! */
		}
	}


/*
 *	EDD_DO_BLINK
 */
void	edd_do_blink(pdue)
due	*pdue;
	{
	static int	toggle = 0;

	toggle ^= 1;
	do_blink(pdue->d, pdue->dlc.pcmap, toggle);
	fdmgr_add_timeout(pfdctx, &blinkrate, edd_do_blink, pdue);
	}

kludgeblinker()
	{
	static int	toggle				= 0;
	static long	last_toggle_time	= 0;
	long		current_time		= time(0);

	if((current_time - last_toggle_time) > 0)
		{
		toggle ^= 1;
		do_blink(saved_pdue->d, saved_pdue->dlc.pcmap, toggle);
		last_toggle_time = current_time;
		}
	}

#include <pwd.h>

static char *
GetHomeDir(dest)
char  *dest;
	{
	extern char				*getenv();
	extern int				getuid();
	extern struct passwd	*getpwuid();
	struct passwd			*pw;
	char					*ptr;

	if((ptr = getenv("HOME")) != NULL)
		return strcpy(dest, ptr);

	if((pw = ((ptr = getenv("USER")) != NULL) ? getpwnam(ptr) : getpwuid(getuid())) != NULL)
		return strcpy(dest, pw->pw_dir);

	*dest = '\0';
	return dest;
	}

static check_all_string_offsets()
	{
	struct ginfo	*pginfo;

	check_string_offsets(dl_hdr_sinfo);	
	check_string_offsets(dl_dynamic_attribute_sinfo);
	check_string_offsets(dl_color_rule_sinfo);
	for (pginfo = &dl_elements_ginfo[0]; pginfo->sinfo; pginfo++)
		check_string_offsets(pginfo->sinfo);	
	}
