/*get_coordinate.c,v 1.1.1.1 1999/08/03 14:43:53 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	G E T _ C O O R D I N A T E . C
 *
 *	Author: Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	input an x,y coordinate from the mouse
 *
 *	History
 *
 */
#include 	<Xlib.h>
#include 	<Xutil.h>
#include 	<edd.h>

#define   posdiff(A,B)    ((A)>(B)?(A)-(B):(B)-(A))

/*
 * clear out the button press here so it does not mess up other user
 * interaction such as menus
 */
#define GET2ND_EVMASK	ButtonPressMask|ButtonReleaseMask|PointerMotionMask

#define GETCOORD_EVMASK	ButtonPressMask|ButtonReleaseMask|PointerMotionMask



/*
 *
 *	GET_COORD
 *
 *	(this needs to remove prior motion with
 *	a grab mode sync)
 */
get_coord(d, w, cursor, x, y, pfunc, param)
Display		*d;
Window		w;
Cursor		cursor;
int		x;
int		y;
void		(*pfunc)();
void		*param;
{
	XEvent		xevent;
	int 		status;


	/*
	 * Grab here enables recv of motion events 
	 * which may not be ordinarily enabled
	 * 
	 * It also makes sure that a passive grab will 
	 * not change button response.
	 * 
	 */
	status = XGrabPointer(	
			d, 
			w,
			TRUE,	/* grab, but send to owner window */
			GETCOORD_EVMASK,
			GrabModeAsync,
			GrabModeAsync,
			w,
			cursor,
			CurrentTime);
	if(status != GrabSuccess){
		return ERROR;
	}


	/*
	 * set the pointer to the initial 2nd coord
	 */
	XWarpPointer(	d, 
			None, 
			w,
			0,0,0,0,
			x,
			y);
#ifdef FIRST_CHANGE_UPDATE
	/*
	 * notify them of the first change
	 */
	(*pfunc)(param, x, y);
#endif
	while(TRUE){	

		XWindowEvent(d, w, GETCOORD_EVMASK, &xevent);

		switch(xevent.type){
		case ButtonRelease:
			/*	
			 * the event's time not used here because the
			 * button could have cycled prior to the
			 * grab above. X would ignore the ungrab in this 
			 * case.
			 */
			XUngrabPointer(d, CurrentTime);
			return OK;
	
		case MotionNotify:
			/*
			 * notify them of the change
			 */
			(*pfunc) (	param, 
					xevent.xmotion.x, 
					xevent.xmotion.y);

			break;
 		}
	}
}



/*
 *
 *	GET_2ND_COORD
 *
 *
 */
get_2nd_coord(d, w, gc, ppt1, ppt2, cursor)
Display		*d;
Window		w;
GC		gc;
XPoint		*ppt1;
XPoint		*ppt2;
Cursor		cursor;
{
	int		x2 = ppt2->x;
	int		y2 = ppt2->y;
	XEvent		xevent;
	int 		status;


	/*
	 * Grab here enables recv of motion events which may not be normally
	 * not enabled.
	 * 
	 * It also makes sure that a passive grab will not change button
	 * response.
	 * 
	 */
	status = XGrabPointer(	
			d, 
			w,
			TRUE,	/* grab, but send to owner window */
			GET2ND_EVMASK,
			GrabModeAsync,
			GrabModeAsync,
			w,
			cursor,
			CurrentTime);
	if(status != GrabSuccess){
		return ERROR;
	}

	/*
	 * set the pointer to the initial 2nd coord
	 */
	XWarpPointer(	d, 
			None, 
			w,
			0,0,0,0,
			x2,
			y2);

	/*
	 * draw the initial rectangle
	 */
	XDrawRectangle(	d, 
			w, 
			gc, 
			min(ppt1->x,x2),
			min(ppt1->y,y2),
			posdiff(ppt1->x,x2),
			posdiff(ppt1->y,y2));


	while(TRUE){	

		XWindowEvent(d, w, GET2ND_EVMASK, &xevent);

		switch(xevent.type){
		case ButtonRelease:
			ppt2->x = x2;
			ppt2->y = y2;

			XDrawRectangle(	d, 
					w, 
					gc, 
					min(ppt1->x,x2),
					min(ppt1->y,y2),
					posdiff(ppt1->x,x2),
					posdiff(ppt1->y,y2));

			/*	
			 * the event's time not used here because the
			 * button could have cycled prior to the
			 * grab above. X would ignore the ungrab in this 
			 * case.
			 */
			XUngrabPointer(d, CurrentTime);
			return OK;
	
		case MotionNotify:
/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

problem here is that they are returning motion events for the window border

Issues:
1)	do we accept a 2nd point out side the bounds of the display

2)	if not it is hard to check for here and make this a generic 
	subroutine

- perhaps pass in the object header structure for bounds checking

- perhaps make this code edd dependent

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/
			if(xevent.xmotion.x < 0)
				xevent.xmotion.x = 0; 
			if(xevent.xmotion.y < 0)
				xevent.xmotion.y = 0; 


			XDrawRectangle(	d, 
					w, 
					gc, 
					min(ppt1->x,x2),
					min(ppt1->y,y2),
					posdiff(ppt1->x,x2),
					posdiff(ppt1->y,y2));
			x2 = xevent.xmotion.x;
			y2 = xevent.xmotion.y;
			XDrawRectangle(	d, 
					w, 
					gc, 
					min(ppt1->x,x2),
					min(ppt1->y,y2),
					posdiff(ppt1->x,x2),
					posdiff(ppt1->y,y2));
			break;
 
		}
	}
}

/*
 *
 *	GET_1st_COORD
 *
 *	really a utility- should be moved from edd.c
 *
 */
int get_1st_coord(d, w, gc, width, height, p, cursor)
Display		*d;
Window		w;
GC		gc;
int		width;		/* width of positioned object */
int		height;		/* height of positioned object */
XPoint		*p;
Cursor		cursor;
{
	XEvent		xevent;
	int 		status;



	{
		int	dummy;
		int	x;
		int	y;
		Window	child;
		Window	root;

		status = XQueryPointer(
			d,
			w,
			&root, 
			&child,
			&dummy,
			&dummy,
			&x,
			&y,
			(unsigned int *)&dummy);
		if(!status){
			return ERROR;
		}

		p->x = x;
		p->y = y;
	}

	/*
	 * Grab here enables recv of motion events which may not be
	 * enabled all the time.
	 * 
	 * It also makes sure that a passive grab will not change button
	 * response.
	 * 
	 */
	status = XGrabPointer(	
			d, 
			w,
			TRUE,	/* grab, but send to owner window */
			GET2ND_EVMASK,
			GrabModeAsync,
			GrabModeAsync,
			w,
			cursor,
			CurrentTime);
	if(status != GrabSuccess)
		return ERROR;

	XDrawRectangle(	d, 
			w, 
			gc, 
			p->x,
			p->y,
			width,
			height);

	while(TRUE){	

		XWindowEvent(d, w, GET2ND_EVMASK, &xevent);

		switch(xevent.type){
		case ButtonRelease:
			XDrawRectangle(	d, 
					w, 
					gc, 
					p->x,
					p->y,
					width,
					height);

			/*	
			 * the event's time not used here because the
			 * button could have cycled prior to the
			 * grab above. X would ignore the ungrab in this 
			 * case.
			 */
			XUngrabPointer(d, CurrentTime);
			return OK;
	
		case MotionNotify:
/*
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

problem here is that they are returning motion events for the window border

Issues:
1)	do we accept a 2nd point out side the bounds of the display

2)	if not it is hard to check for here and make this a generic 
	subroutine

- perhaps pass in the object header structure for bounds checking

- perhaps make this code edd dependent

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
*/
			if(xevent.xmotion.x < 0)
				xevent.xmotion.x = 0; 
			if(xevent.xmotion.y < 0)
				xevent.xmotion.y = 0; 


			XDrawRectangle(	d, 
					w, 
					gc, 
					p->x,
					p->y,
					width,
					height);
			p->x = xevent.xmotion.x;
			p->y = xevent.xmotion.y;
			XDrawRectangle(	d, 
					w, 
					gc, 
					p->x,
					p->y,
					width,
					height);
			break;
 
		}
	}
}
