/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	L O A D . C
 *
 *	Author	Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	load a display file
 *
 *
 *	History
 *	-------
 *
 *	Problems
 *	--------
 *	.01	when a new display is loaded and the window resizes
 *		I get an expose event which redraws the display. I 
 *		also must explicitly force redraw since the display
 *		may not resize. This causes a double redraw after some
 *		display load cycles.
 */
#include <stdio.h>
#ifdef NeXT
#include <types.h>
#endif
#ifdef SOLARIS
#include <fcntl.h>
#endif
#include <file.h>

#ifdef AXP
#include <string.h>
#include <stdlib.h>
#endif

#ifdef TOKENPTRS
#include <tokenSupport.h>
#endif

#include <Xlib.h>
#include <Xutil.h>

#include <edd.h>
#include <gizmo.h>
#include <dlinfo.h>

static void	load_popup_finish();
int			load_display();
static int	load_elements();
static void	clear_due();
void		set_next_group_id();

extern void *tisMalloc();
extern void *tisCalloc();
extern void tisFree();

/*
 *
 * GM_LOAD - load new display
 * (see edd_menus.h)
 */
void gm_load(param, pdue, pbe)
void			*param;
due				*pdue;
XButtonEvent	*pbe;
	{
	XRectangle		obj;


	if (pdue->ploadg)
		{
		move_gizmo(pdue->ploadg, pbe->x_root, pbe->y_root);
		return;
		}

	/*
	 * set the default display path as the current wd
	 * set the default display name
	 */
	if (pdue->pldname[0] == 0)
		strcpy(pdue->pldname, DEFAULT_DISPLAY_NAME);
	if (pdue->pldpath[0] == 0)
#ifdef SOLARIS
		getcwd(pdue->pldpath, sizeof (pdue->pldpath));
#else
		getwd(pdue->pldpath);
#endif
	obj.x = pbe->x_root;
	obj.y = pbe->y_root;
	obj.width = (pdue->gizmo_width<<1);
	obj.height = (pdue->gizmo_height<<1);
	pdue->ploadg = create_file_name_entry_gizmo(
				pdue->s, 
				pdue->w,
				&obj, 
				"~Load File Name and Path Entry",
				pdue->pldpath, 
				sizeof(pdue->pldpath), 
				pdue->pldname, 
				sizeof(pdue->pldname), 
				load_popup_finish,
				pdue,
				pdue->gizmo_height);
	set_gizmo_color(pdue->ploadg, pdue->foreground.pixel, pdue->background.pixel);			
	}


/*
 *	LOAD_POPUP_FINISH()
 */
static void load_popup_finish(pdue, execute, pbe)
due				*pdue;
int				execute;
XButtonEvent	*pbe;
	{
	int status;

	free_gizmo(pdue->ploadg);
	pdue->ploadg = NULL;
	if(execute)
		{
		/*
		 *	Did they save changes ?
		 */
		if(pdue->dl_mdt)
			{
			int 			choice;
			enum			choices	{cancel, load, terminate, nchoices};
			static char 	*pmsgs[4];
			char 			*pchoices[nchoices];

			pmsgs[0] = "~You have not saved since your last edit.";
			pmsgs[1] = "~Loading now will discard changes.";
			pmsgs[2] = "~Do you still want to load?";
			pmsgs[3] = 0;

			pchoices[load]		= "~load";
			pchoices[cancel]	= "~cancel";
			pchoices[terminate]	= 0;

			choice = perform_dialog( pdue->s, pbe->x_root, pbe->y_root, pdue->gizmo_height, pmsgs, pchoices, pdue->gizmo_height);
			if(choice == cancel)
				return;
			}

		if (load_display(pdue) < 0)
			gm_load(NULL, pdue, pbe);
		}
	}


/*
 *	EDD_PATH_CAT
 */
edd_path_cat(pdue, ppath, pname, pdest, dest_size)
due		*pdue;
char 	*ppath;
char	*pname;
char	*pdest;
int		dest_size;
	{
	if(strlen(ppath) + strlen(pname) + 1 >= dest_size)
		{
		sprintf(pdue->pmsg, "~Large or corrupt path or file name ignored\n");
		redraw_msg(pdue);
		return ERROR;
		}
	if(*ppath != 0)
		strcat(strcpy(pdest, ppath), "/");
	else
		*pdest = 0;
	strcat(pdest, pname);
	return OK;
	}


/*
 *	LOAD_DISPLAY()
 */
int load_display(pdue)
due		*pdue;
	{
	int 			fd;
	int 			status;
	void			*pbuff;
	int				nwords;
	char  	 	 	pname[sizeof(pdue->pname)+sizeof(pdue->ppath)];
	eue				*peue;
	struct dl_hdr	dl_hdr;
	int				valid_load = FALSE;

	status = edd_path_cat(pdue, pdue->pldpath, pdue->pldname, pname, sizeof(pname));
	if (status == ERROR)
		return ERROR;

	fd = open_display_list_file( pdue->penv_path, pdue->pldpath, sizeof(pdue->pldpath), pname, O_RDONLY);
	if (fd == ERROR)
		{
		sprintf(pdue->pmsg, "~Load file access problem. Context: [%s]\n", pname);
		redraw_msg(pdue);
		return ERROR;
		}
	clear_due(pdue);

	/*
	 * Read in the display list header
	 */
	status = read(fd, &dl_hdr, sizeof(dl_hdr));
	if (status == ERROR || dl_hdr.magic != DL_MAGIC)
		{
		sprintf(pdue->pmsg, "~Not a binary display file- attempting load from text\n");
		redraw_msg(pdue);
		lseek(fd, 0, L_SET);
		status = load_adl(pdue, fd);
		if ((status >= 0) && (pdue->dl_hdr.majv == DL_MAJOR_VERSION) && (pdue->dl_hdr.mnrv == DL_MINOR_VERSION))
			valid_load = TRUE;
		else
			{
			if ((pdue->dl_hdr.majv != DL_MAJOR_VERSION) || (pdue->dl_hdr.mnrv != DL_MINOR_VERSION))
				{
				sprintf(pdue->pmsg, "~Version %d.%d display file does not match Ver %d.%d editor.",
				  pdue->dl_hdr.majv, pdue->dl_hdr.mnrv, DL_MAJOR_VERSION, DL_MINOR_VERSION);
				redraw_msg(pdue);
				sleep(10);
				}
			clear_due(pdue);
			}
		}
	else if(valid_dl_test(&dl_hdr) >= 0)
		{
		nwords = dl_hdr.nwords_dspy - WDRND(sizeof(dl_hdr));
		/*pbuff = (void *) calloc(1, nwords<<1);*/
		pbuff = (void *) tisCalloc(__FILE__, __LINE__, 1, nwords<<1);
		assert(pbuff != NULL);

		/*
		 * Read in the display list
		 */
		status = read(fd, pbuff, nwords<<1);
		if (status >= 0)
			{
			fixup_string_pointers(&dl_hdr, pbuff);
			if (load_elements(pdue, &dl_hdr, pbuff, nwords) == OK)
				valid_load = TRUE;
		
			/*assert(free(pbuff) >= 0);*/
			tisFree(pbuff);
			}
		if (valid_load)
			{
			strcpy(pdue->pname, pdue->pldname);
			strcpy(pdue->ppath, pdue->pldpath);
			}
		}

	if (valid_load)
		{
		sprintf(pdue->pmsg, "~Loaded from [%s].\n", pname);
		/* no redraw here- see clear area below */
		/*
		 * set the default strings for the next rename
		 */
		strcpy(pdue->psnname, pdue->pname);
		strcpy(pdue->psnpath, pdue->ppath);
		init_text_file_name(pdue);
		pdue->dl_mdt = FALSE;
		set_next_group_id(pdue);
		}
	else
		{
		sprintf(pdue->pmsg, "~Load failed from [%s].\n", pname);
		redraw_msg(pdue);
		init_due(pdue);
		}
	close(fd);

	/*
	 * Force the editor header to redraw
	 * resize windows for the new display and redraw
	 */
	XClearArea(pdue->d, pdue->w, 0, 0, 0, 0, True); /* send expose event */
	configure_due(pdue, NULL);

	return valid_load ? OK : ERROR;
	}



/*
 *	LOAD_ELEMENTS
 */
static load_elements(pdue, pdl_hdr, pgo, nwords)
due					*pdue;
struct dl_hdr		*pdl_hdr;
struct dl_anygo 	*pgo;
int					nwords;
	{
	eue					*peue;
	struct dl_sta_attr	sta_attr;
	struct dl_dyn_attr	dyn_attr;
	char				pname[sizeof(pdue->pname)+sizeof(pdue->ppath)];

#ifdef SOLARIS
	memset(&sta_attr, 0, sizeof(sta_attr));
	memset(&dyn_attr, 0, sizeof(dyn_attr));
#else
	bzero((char *)&sta_attr, sizeof(sta_attr));
	bzero((char *)&dyn_attr, sizeof(dyn_attr));
#endif
	for ( ; nwords > 0; nwords -= pgo->element.nwords, pgo = (struct dl_anygo *)(((short *) pgo) + pgo->element.nwords))
		{
		unsigned 	type	= pgo->element.type;
		unsigned 	size	= WDRND(dl_elements_ginfo[type].size)<<1;

#ifdef DEBUG
		dump_element(pgo, &pdue->ct, &pdue->st);
		printf("~loading (%s)\n", dl_elements_einfo[pgo->element.type]);
#endif
		remap_string_ptrs(pdue, dl_elements_ginfo[type].sinfo, pgo);
		switch (type)
		  {
		  case dl_pc:
			stringInsert(&pdue->ct, (char *)(((struct dl_pc *)pgo)+1));	
			break;
		  case dl_string:
			stringInsert(&pdue->st, (char *)(((struct dl_string *)pgo)+1));	
			break;
		  case dl_color_map:
			edd_create_cmap(pdue, ((struct dl_color_map *)pgo)+1, ((struct dl_color_map *)pgo)->ncolors, 0);
			break;
		  case dl_color_rule_table:
			{	/* these are extended objects */
			struct dl_color_rule_table	*pdlcrules 	= (struct dl_color_rule_table *)pgo;
			struct dl_color_rule 		*pdlcr 		= (struct dl_color_rule *)(pdlcrules+1);
			int							elemno;

			for (elemno = 0; elemno < pdlcrules->nrules; elemno++)
				remap_string_ptrs(pdue, dl_color_rule_sinfo, &pdlcr[elemno]);

			edd_create_crules(pdue, pdlcr, pdlcrules->nrules, 0);
			}
			break;
		  case dl_basic_attribute:
			sta_attr = ((struct dl_basic_attribute *)pgo)->attr;
			break;
		  case dl_dynamic_attribute:
			remap_string_ptrs(pdue, dl_dynamic_attribute_sinfo, pgo);
			dyn_attr = ((struct dl_dynamic_attribute *)pgo)->attr;
			break;
		  default:
			switch (dl_element_class[type])
			  {
			  case dl_class_monitor:
			  case dl_class_plot:
			  case dl_class_control:
			  case dl_class_related_display:
			  case dl_class_execute_script:
			  case dl_class_kill_display:
				peue = create_element(pdue, pgo, size);
				break;
			  case dl_class_graphic:
				peue = create_element(pdue, pgo, size);
				peue->sta = sta_attr;
				peue->dyn = dyn_attr;
				break;
			  default:
				sprintf(pdue->pmsg, "~Corrupt display list?\n");
				redraw_msg(pdue);
				dump_element(pgo, &pdue->ct, &pdue->st);
				return ERROR;
			  }
			break;
		  }
		}
	remap_string_ptrs(pdue, dl_hdr_sinfo, pdl_hdr);
	pdue->dl_hdr = *pdl_hdr;
	
	return OK;
	}


/*
 *	CLEAR_DUE
 *	clear anything preexisting in the display list under edit
 */
static void clear_due(pdue)
due		*pdue;
	{
	eue	*peue;

	purge_undo_list(pdue);
	clear_selections(FALSE, pdue);
	while (peue = (eue *) lstGet(&pdue->dl))
		{
		close_element(pdue, peue, FALSE, FALSE);
		/*assert(free(PEUE_TO_PSEL(peue)) >= 0);*/
		tisFree(PEUE_TO_PSEL(peue));
		}

	if (pdue->phdrgz)
		{
		free_gizmo(pdue->phdrgz);
		pdue->phdrgz = NULL;
		}

	/* delete any previous color map and rules and strings! */
	edd_delete_cmap(pdue);
	edd_delete_crules(pdue);

	stringDeleteList(&pdue->st);
	stringDeleteList(&pdue->ct);

	/* all changes cleaned out */
	pdue->dl_mdt = FALSE;
#ifdef TOKENPTRS
	/* clear out all pointer tokens */
	dmTokenFlush ();
#endif
	}



/*
 *	EDD_LOAD_CMAP_FROM_DISPLAY_FILE
 */
void edd_load_cmap_from_display_file(pdue)
due	*pdue;
	{
	char 				*pname;
	unsigned			length;
	struct color_map 	*pcmap;
	struct color_rules 	*pcrules;

	/*
	 * leave as before if they dont specify a color map
	 */
#ifndef TOKENPTRS
	stringName(&pdue->st, pdue->dl_hdr.cmap, &pname, &length);
#else
	stringName(&pdue->st, dmPointer64fromToken (pdue->dl_hdr.cmap), &pname, &length);
#endif
	if (length == 0)
		return;

	pcmap = load_cmap_from_display_file(pdue->d, pname, pdue->penv_path, TRUE);
	if (pcmap)
		edd_install_cmap(pdue, pcmap);
	else
		add_default_color_map(pdue);

	pcrules = load_crules_from_display_file(pname, pdue->penv_path, &pdue->st); 
	if (pcrules)
		edd_install_crules(pdue, pcrules);
	else
		add_default_color_rules(pdue);
	}

void set_next_group_id(pdue)	/* set the next one to be one passed the max in use */
due	*pdue;
	{
	eue	*peue;

	for(peue = (eue *) pdue->dl.node.next; peue; peue = (eue *) peue->node.next)
		pdue->next_groupid = max(pdue->next_groupid, peue->elem.dl_anygo.groupid);

	pdue->next_groupid++;
	}

