/*modify_color_rules.c,v 1.1.1.1 1999/08/03 14:43:53 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	M O D I F Y _ C O L O R _ R U L E S . C
 *	Author:		Larry Blische (from a story told by Jeffrey O. Hill)
 *
 *	Form for modifying a color rule
 *
 *
 *	History
 *	-------
 *
 */


#include 	<Xlib.h>
#include 	<Xutil.h>
#include 	<Xatom.h>

#include 	<edd.h>
#include 	<dlinfo.h>
#include 	<gizmo.h>
#include	<einfoString.h>	/* this file was created by einfoStringConv */

#ifdef TOKENPTRS
#include <tokenSupport.h>
#endif

extern void *tisMalloc();
extern void *tisCalloc();
extern void tisFree();

enum	rule_column_headers { desc_col, chan_col, equal_col, value_col, connector_col, color_col, last_col };

struct color_rule_name_info{
	short 				ruleix;
	short 				lineix;
	enum rule_column_headers 	column;
};
#define	NO_LINE	-1

static void 	color_rule_update();
static void 	color_rule_connector_update();
static void 	color_rule_operation_update();
static void 	color_rule_color_update();
static void	color_button_update();
static void 	nrules_update();
static void 	modify_crules_finish();
static void 	modify_crules();
static gizmo	*modify_crule_list();
static gizmo	*modify_one_crule();

static char  *modify_color_rule_choices[] =
{       
	0,
	0
};



/*
 *
 *
 * 	WRITE_CRULES
 * 
 *
 */
write_crules(pcrules, pstream)
struct color_rules		*pcrules;
edd_stream			*pstream;
{
	int				status;
	struct dl_color_rule_table	dlcrules;
	int				body_size;

	if(pcrules->nrules == 0)
		return;

	body_size = pcrules->nrules * sizeof(struct dl_color_rule);

	dlcrules.element.type		= dl_color_rule_table;
	dlcrules.element.nwords		= WDRND(sizeof dlcrules + body_size);
	dlcrules.nrules = pcrules->nrules;
	status = edd_write(pstream, &dlcrules, sizeof dlcrules);
	assert(status != ERROR);

	status = edd_write(pstream, pcrules->pdl_crules, (dlcrules.element.nwords<<1) - sizeof dlcrules);
	assert(status != ERROR);

	return OK;
}



/*
 *
 *
 * 	MODIFY_CRULES
 * 
 *
 */
static void modify_crules(pdue, x_root, y_root, rule_name)
due	*pdue;
int	x_root;
int	y_root;
char	*rule_name;
{
	gizmo		*pformg;
	char		*pname;
	unsigned	length;
	int		status;

#ifndef TOKENPTRS
	stringName(&pdue->st, pdue->dl_hdr.cmap, &pname, &length);
#else
	stringName(&pdue->st, dmPointer64fromToken (pdue->dl_hdr.cmap), &pname, &length);
#endif
	if(length > 0){
		enum 		choices{cancel, terminate, nchoices};
		static char 	*pmsgs[8];
		char 		*pchoices[nchoices];
		int		choice;
		int		index;

		pmsgs[0] = "~This display's color rules are imported along";
		pmsgs[0] = "~with it's color table from another display.  Load"; 
		pmsgs[0] = "~that display to make shared color rule changes.",
		pmsgs[0] = "";
		pmsgs[0] = "~You may also receive a local copy of this color";
		pmsgs[0] = "~map and rule table, which will belong to this display,";
		pmsgs[0] = "~by requesting to modify the color table.";
		pmsgs[0] = 0;

		pchoices[cancel]	= "~cancel";
		pchoices[terminate]	= 0;
		choice = perform_dialog(
			pdue->s, 
			x_root, 
			y_root, 
			pdue->gizmo_height, 
			pmsgs, 
			pchoices, 
			pdue->gizmo_height);
		
		return;	/* must be cancel */
	}

	/*
	 *	Dont allow two forms at once
 	 *
 	 */
	if( pdue->dlc.pcrules && pdue->dlc.pcrules->ppopg){
		move_gizmo(pdue->dlc.pcrules->ppopg, x_root, y_root);
		return;
	}
	modify_color_rule_choices[0] = "~quit";
	modify_color_rule_choices[1] = 0;
	pdue->dlc.pcrules->ppopg = create_popup_gizmo(
				pdue->s, 
				pdue->w,
				"~Color Rule Modify",
				x_root, 
				y_root,
				pdue->gizmo_height,
				modify_crules_finish,
				pdue,
				modify_color_rule_choices,
				0);
	set_gizmo_color(pdue->dlc.pcrules->ppopg, pdue->foreground.pixel, pdue->background.pixel);

	if (strcmp(rule_name, COLOR_RULE_LISTEDIT_TAG) == 0)
		pformg = modify_crule_list(pdue);
	else
		pformg = modify_one_crule(pdue, rule_name);
	resize_form_gizmo(pformg);
	popup_gizmo_add_gizmo(pdue->dlc.pcrules->ppopg, pformg);
	XMapWindow(pdue->dlc.pcrules->ppopg->pgg->d, pdue->dlc.pcrules->ppopg->w);
}



/*
 *
 *
 * 	UPDATE_COLOR_RULE_MENU
 * 
 *
 */
void update_color_rule_menu(pdue)
due		*pdue;
{
	extern struct menu_configuration 	colorrule_mc;
	extern struct menu_configuration 	global_mc;
	unsigned int				i;
	struct color_rules			*pcrules = pdue->dlc.pcrules;

	/* fix up color rules menu list */
	if(pcrules->pchoices){
		for(i = 1; i < (pcrules->nrules + 1); i++){
			if(pcrules->pchoices[i])
				/*free(pcrules->pchoices[i]);*/
				tisFree(pcrules->pchoices[i]);
		}
		/*free(pcrules->pchoices);*/
		tisFree(pcrules->pchoices);
	}
	/*pcrules->pchoices = (void *)calloc(pcrules->nrules + 2, sizeof (char *));*/
	pcrules->pchoices = (void *)
			tisCalloc(__FILE__, __LINE__, pcrules->nrules + 2, sizeof (char *));
	if(pcrules->pactions)
		/*free(pcrules->pactions);*/
		tisFree(pcrules->pactions);
	/*pcrules->pactions = (void *)calloc(pcrules->nrules + 2, sizeof (struct jump_table));*/
	pcrules->pactions = (void *)
			tisCalloc(__FILE__, __LINE__, pcrules->nrules + 2, sizeof (struct jump_table));

	pcrules->pchoices[0]      = COLOR_RULE_LISTEDIT_TAG;	/* 1st one is the edit button */
	pcrules->pactions[0].func = gm_modify_color_rules;
	pcrules->pactions[0].arg  = pcrules->pchoices[0];
	for(i = 1; i < (pcrules->nrules + 1); i++){		/* remaining buttons are the rule names */
		char		*pstring;
		unsigned	length;

#ifndef TOKENPTRS
		stringName(&pdue->st, pcrules->pdl_crules[i-1].name, &pstring, &length);
#else
		stringName(&pdue->st, dmPointer64fromToken (pcrules->pdl_crules[i-1].name), &pstring, &length);
#endif
		/*pcrules->pchoices[i] = (void *)calloc(length + 1, sizeof (char *));*/
		pcrules->pchoices[i] = (void *)
				tisCalloc(__FILE__, __LINE__, length + 1, sizeof (char *));
		strncpy(pcrules->pchoices[i], pstring, length);
		pcrules->pchoices[i][length] = 0;
		pcrules->pactions[i].func = gm_modify_color_rules;
		pcrules->pactions[i].arg = pcrules->pchoices[i];
	}
	pcrules->pchoices[pcrules->nrules + 1]      = 0;	/* last one is the terminator */
	pcrules->pactions[pcrules->nrules + 1].func = 0;
	pcrules->pactions[pcrules->nrules + 1].arg  = 0;
	colorrule_mc.pchoices = pcrules->pchoices;
	colorrule_mc.pactions = pcrules->pactions;
	refresh_menu(pdue->d, &global_mc);
}



/*
 *
 *
 * 	MODIFY_CRULES_FINISH
 * 
 *
 */
static void modify_crules_finish(pdue)
due		*pdue;
{
	update_color_rule_menu(pdue);
	free_gizmo(pdue->dlc.pcrules->ppopg);
	pdue->dlc.pcrules->ppopg = NULL;
}



/*
 *
 *
 * 	MODIFY_CRULE_LIST
 * 
 *
 */
static gizmo *modify_crule_list(pdue)
due		*pdue;
{
	enum		list_headers { name_col, number_col, last_col };
	gizmo		*pformg;
	gizmo		*pg;
	unsigned	row = 0;
	XRectangle	obj;
	XRectangle	clrobj;
	unsigned	ruleix;

	pformg = create_form_gizmo(
				pdue->s, 
				pdue->dlc.pcrules->ppopg->w,
				0, 
				0,
				last_col,
				pdue->dlc.pcrules->nrules+2,
				pdue->gizmo_pad,
				0);

	/* size known, but position tbd */
	obj.x		= 0;
	obj.y		= 0;
	obj.height	= pdue->gizmo_height;
	obj.width	= pdue->gizmo_width;
	clrobj.x	= 0;
	clrobj.y	= 0;
	clrobj.height	= pdue->gizmo_height;
	clrobj.width	= pdue->gizmo_width/2; 

	/* input the number of colors */
	form_gizmo_add_string(pformg, name_col, row, obj.height, "~number of color rules");
#define MAXCOLORCHARS 3
	pg = create_textedit_gizmo(
				pdue->s, 
				pformg->w, 
				&obj, 
				ok_to_operate,
				nrules_update,
				pdue,
				MAXCOLORCHARS,
				0);
	assert(pg != NULL);
	{
	char  nrules_string[50];
		
	sprintf(nrules_string, "%d", pdue->dlc.pcrules->nrules);
	set_textedit_gizmo(pg, nrules_string);
	}
	form_gizmo_add_gizmo(pformg, number_col, row, pg);

	/* add column headers */
	row++;
	form_gizmo_add_string(pformg, name_col, row, obj.height, "~name");

	/* list the names */
	row++;
	for(ruleix = 0; ruleix < pdue->dlc.pcrules->nrules; ruleix++){
		char				*pstring;
		unsigned 			length;

		pg = create_textedit_gizmo(
				pdue->s, 
				pformg->w, 
				&obj, 
				ruleix == 0 ? not_ok_to_operate : ok_to_operate,
				color_rule_update,
				pdue,
				MAXRULENAMELEN,
				sizeof (struct color_rule_name_info));
		assert(pg != NULL);
		((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
		((struct color_rule_name_info *) pg->pae)->lineix = NO_LINE;

#ifndef TOKENPTRS
		stringName(&pdue->st, pdue->dlc.pcrules->pdl_crules[ruleix].name, &pstring, &length);
#else
		stringName(&pdue->st, dmPointer64fromToken (pdue->dlc.pcrules->pdl_crules[ruleix].name), &pstring, &length);
#endif
		set_textedit_gizmo(pg, pstring);
		form_gizmo_add_gizmo(pformg, name_col, row, pg);
		row++;
	}

	return pformg;
}



/*
 *
 *
 * 	getRuleIndex
 * 
 *
 */
int	getRuleIndex(pdue, rule_name)
due	*pdue;
char	*rule_name;
{
	struct dl_color_rule	*pdlcr	= &pdue->dlc.pcrules->pdl_crules[0];
	unsigned int		ruleix;

	for(ruleix = 0; ruleix < pdue->dlc.pcrules->nrules; ruleix++, pdlcr++){
		char	*pstring;
		int	length;

#ifndef TOKENPTRS
		stringName(&pdue->st, pdlcr->name, &pstring, &length);
#else
		stringName(&pdue->st, dmPointer64fromToken (pdlcr->name), &pstring, &length);
#endif
		if(strcmp(pstring, rule_name) == 0)
			return ruleix;
	}
	assert(FALSE);	/* we must find it */
}



/*
 *
 *
 * 	MODIFY_ONE_CRULE
 * 
 *
 */
static gizmo *modify_one_crule(pdue, rule_name)
due	*pdue;
char	*rule_name;
{
	static void		add_textedit();
	static void		add_menu();
	static void		add_op_menu();
	static void		add_color();
	static void		add_button();
	gizmo			*pformg;
	gizmo			*pg;
	unsigned		row;
	XRectangle		obj;
	XRectangle		clrobj;
	unsigned		i;
	char			*pstring;
	unsigned 		length;
	int			status;
	int			ruleix	= getRuleIndex(pdue, rule_name);
	struct dl_color_rule	*pdlcr	= &pdue->dlc.pcrules->pdl_crules[ruleix];

	pformg = create_form_gizmo(
				pdue->s, 
				pdue->dlc.pcrules->ppopg->w,
				0, 
				0,
				last_col,
				2+MAX_RULE_LINES+1+2,
				pdue->gizmo_pad,
				0);

	/* size known, but position tbd */
	obj.x		= 0;
	obj.y		= 0;
	obj.height	= pdue->gizmo_height;
	obj.width	= pdue->gizmo_width;
	clrobj.x	= 0;
	clrobj.y	= 0;
	clrobj.height	= pdue->gizmo_height;
	clrobj.width	= pdue->gizmo_width/2; 

	/* input the name of the color rule */
	row = 0;
	form_gizmo_add_string(pformg, desc_col, row, obj.height, "~name");
	pg = create_textedit_gizmo(
				pdue->s, 
				pformg->w, 
				&obj, 
				ruleix == 0 ? not_ok_to_operate : ok_to_operate,
				color_rule_update,
				pdue,
				MAXRULENAMELEN,
				sizeof (struct color_rule_name_info));
	assert(pg != NULL);
	((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
	((struct color_rule_name_info *) pg->pae)->lineix = NO_LINE;
	set_textedit_gizmo(pg, rule_name);
	form_gizmo_add_gizmo(pformg, chan_col, row, pg);

	/* add column headers */
	row++;
	form_gizmo_add_string(pformg, chan_col,      row, obj.height, "~channel");
	form_gizmo_add_string(pformg, equal_col,     row, obj.height, "~operator");
	form_gizmo_add_string(pformg, value_col,     row, obj.height, "~value");
	form_gizmo_add_string(pformg, connector_col, row, obj.height, "~connector");
	form_gizmo_add_string(pformg, color_col,     row, obj.height, "~color");

	/* list the rule lines */
	for(i=0; i < MAX_RULE_LINES; i++){
		struct dl_color_rule_info	*pdlcri	= &pdlcr->info[i];

		row++;
		form_gizmo_add_string(pformg, desc_col,  row, obj.height, "~ if ");
#ifndef TOKENPTRS
		add_textedit(pformg, chan_col,      row, &obj,    pdue, ruleix, i, pdlcri->chan,  &pdue->st);
#else
		add_textedit(pformg, chan_col,      row, &obj,    pdue, ruleix, i, dmPointer64fromToken (pdlcri->chan),  &pdue->st);
#endif
		add_op_menu(    pformg, equal_col, row, &clrobj, pdue, ruleix, i, pdlcri->operation);
#ifndef TOKENPTRS
		add_textedit(pformg, value_col,     row, &obj,    pdue, ruleix, i, pdlcri->value, &pdue->st);
#else
		add_textedit(pformg, value_col,     row, &obj,    pdue, ruleix, i, dmPointer64fromToken (pdlcri->value), &pdue->st);
#endif
		add_menu(    pformg, connector_col, row, &clrobj, pdue, ruleix, i, pdlcri->connector);
		add_color(   pformg, color_col,     row, &clrobj, pdue, ruleix, i, pdlcri->clr);
	}

	/* the default foreground and background controls */
	row++;
	form_gizmo_add_string(pformg, desc_col,  row, obj.height, "");
	row++;
	form_gizmo_add_string(pformg, value_col, row, obj.height, "~default foreground");
	add_button(pformg, connector_col, row, &obj,    pdue, ruleix, MAX_RULE_LINES, pdlcr->fg_enable);
	add_color( pformg, color_col,     row, &clrobj, pdue, ruleix, MAX_RULE_LINES, pdlcr->default_fg);

	row++;
	form_gizmo_add_string(pformg, value_col, row, obj.height, "~default background");
	add_button(pformg, connector_col, row, &obj,    pdue, ruleix, MAX_RULE_LINES+1, pdlcr->bg_enable);
	add_color( pformg, color_col,     row, &clrobj, pdue, ruleix, MAX_RULE_LINES+1, pdlcr->default_bg);

	return pformg;
}


static void add_textedit(pformg, column, row, pobj, pdue, ruleix, lineix, stringix, pst)
gizmo				*pformg;
enum rule_column_headers 	column;
unsigned			row;
XRectangle			*pobj;
due				*pdue;
unsigned			ruleix;
unsigned			lineix;
#ifndef AXP
unsigned			stringix;
#else /* use correct type def for AXP (should be extended to all) */
char				*stringix;
#endif
LIST		*pst;
{
	gizmo		*pg;
	char		*pstring;
	unsigned	length;
	unsigned	status;

	pg = create_textedit_gizmo(
				pdue->s, 
				pformg->w, 
				pobj, 
				ok_to_operate,
				color_rule_update,
				pdue,
				MAXCHANORVALUELEN,
				sizeof (struct color_rule_name_info));
	assert(pg != NULL);
	((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
	((struct color_rule_name_info *) pg->pae)->lineix = lineix;
	((struct color_rule_name_info *) pg->pae)->column = column;

	stringName(pst, stringix, &pstring, &length);
	set_textedit_gizmo(pg, pstring);
	form_gizmo_add_gizmo(pformg, column, row, pg);
}


static void add_menu(pformg, column, row, pobj, pdue, ruleix, lineix, connectorix)
gizmo				*pformg;
enum rule_column_headers 	column;
unsigned			row;
XRectangle			*pobj;
due				*pdue;
unsigned			ruleix;
unsigned			lineix;
unsigned			connectorix;
{
	gizmo		*pg;

	pg = create_menu_gizmo(
				pdue->s, 
				pformg->w, 
				pobj, 
				lineix == (MAX_RULE_LINES-1) ? not_ok_to_operate : ok_to_operate,
				color_rule_connector_update,
				pdue,
				&dl_color_rule_connector_einfo_d[0],
				sizeof (struct color_rule_name_info));
	assert(pg != NULL);
	((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
	((struct color_rule_name_info *) pg->pae)->lineix = lineix;
	((struct color_rule_name_info *) pg->pae)->column = column;

	set_menu_gizmo(pg, connectorix);
	form_gizmo_add_gizmo(pformg, column, row, pg);
}

static void add_op_menu(pformg, column, row, pobj, pdue, ruleix, lineix, connectorix)
gizmo				*pformg;
enum rule_column_headers 	column;
unsigned			row;
XRectangle			*pobj;
due				*pdue;
unsigned			ruleix;
unsigned			lineix;
unsigned			connectorix;
{
	gizmo		*pg;

	pg = create_menu_gizmo(
				pdue->s, 
				pformg->w, 
				pobj, 
				lineix == (MAX_RULE_LINES-1) ? not_ok_to_operate : ok_to_operate,
				color_rule_operation_update,
				pdue,
				&dl_color_rule_operation_einfo_d[0],
				sizeof (struct color_rule_name_info));
	assert(pg != NULL);
	((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
	((struct color_rule_name_info *) pg->pae)->lineix = lineix;
	((struct color_rule_name_info *) pg->pae)->column = column;

	set_menu_gizmo(pg, connectorix);
	form_gizmo_add_gizmo(pformg, column, row, pg);
}



static void add_color(pformg, column, row, pobj, pdue, ruleix, lineix, colorix)
gizmo				*pformg;
enum rule_column_headers 	column;
unsigned			row;
XRectangle			*pobj;
due				*pdue;
unsigned			ruleix;
unsigned			lineix;
unsigned			colorix;
{
	gizmo		*pg;

	pg = create_palette_gizmo(	
				pdue->s, 
				pformg->w, 
				pobj,
				color_rule_color_update,
				pdue,
				&pdue->ppixels,
				&pdue->npixels,
				sizeof(struct color_rule_name_info));
	assert(pg != NULL);
	((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
	((struct color_rule_name_info *) pg->pae)->lineix = lineix;
	((struct color_rule_name_info *) pg->pae)->column = column;

	set_palette_gizmo(pg, colorix);
	form_gizmo_add_gizmo(pformg, column, row, pg);
}


static void add_button(pformg, column, row, pobj, pdue, ruleix, lineix, button_value)
gizmo				*pformg;
enum rule_column_headers 	column;
unsigned			row;
XRectangle			*pobj;
due				*pdue;
unsigned			ruleix;
unsigned			lineix;
unsigned			button_value;
{
	gizmo	*pg;

	pg = create_choice_gizmo(pformg->pgg->s, 
				 pformg->w, 
				 pobj,
				 ok_to_operate,
				 color_button_update,
				 pdue,
				 dl_on_off_einfo_d,
				 gz_stack_row,
				 sizeof(struct color_rule_name_info));

	assert(pg != NULL);
	((struct color_rule_name_info *) pg->pae)->ruleix = ruleix;
	((struct color_rule_name_info *) pg->pae)->lineix = lineix;
	((struct color_rule_name_info *) pg->pae)->column = column;

	set_choice_gizmo(pg, button_value);
	form_gizmo_add_gizmo(pformg, column, row, pg);
}


/*
 *
 *
 * 	COLOR_RULE_UPDATE
 * 
 *
 */
static void color_rule_update(pdue, pg, pstring)
due		*pdue;
gizmo		*pg;
char		*pstring;
{
	struct color_rule_name_info	*pcni	= (struct color_rule_name_info *) pg->pae;
	struct dl_color_rule		*pdlcr	= &pdue->dlc.pcrules->pdl_crules[pcni->ruleix];

	pdue->dl_mdt = TRUE;
	if(pcni->lineix == NO_LINE){
		/*
		 * this is only a name change
		 */
#ifndef TOKENPTRS
		pdlcr->name = stringInsert(&pdue->st, pstring);
#else
		pdlcr->name = dmTokenFromPointer64 (stringInsert(&pdue->st, pstring));
#endif
	}
	else {
		struct dl_color_rule_info	*pdlcri	= &pdlcr->info[pcni->lineix];

		/*
		 * this is a rule element change
		 */
		switch (pcni->column){
#ifndef TOKENPTRS
		case chan_col:		pdlcri->chan  = stringInsert(&pdue->st, pstring); break;
		case value_col:		pdlcri->value = stringInsert(&pdue->st, pstring); break;
#else
		case chan_col:
			pdlcri->chan  = dmTokenFromPointer64 (stringInsert(&pdue->st, pstring));
			break;
		case value_col:
			pdlcri->value = dmTokenFromPointer64 (stringInsert(&pdue->st, pstring));
			break;
#endif
		default:			assert(FALSE);
		}
	}
}


/*
 *
 *
 * 	COLOR_RULE_CONNECTOR_UPDATE
 * 
 *
 */
static void color_rule_connector_update(pdue, pg, index)
due		*pdue;
gizmo		*pg;
int		index;
{
	struct color_rule_name_info	*pcni	= (struct color_rule_name_info *) pg->pae;

	pdue->dl_mdt = TRUE;
	pdue->dlc.pcrules->pdl_crules[pcni->ruleix].info[pcni->lineix].connector = index;
}


/*
 *
 *
 * 	COLOR_RULE_OPERATION_UPDATE
 * 
 *
 */
static void color_rule_operation_update(pdue, pg, index)
due		*pdue;
gizmo		*pg;
int		index;
{
	struct color_rule_name_info	*pcni	= (struct color_rule_name_info *) pg->pae;

	pdue->dl_mdt = TRUE;
	pdue->dlc.pcrules->pdl_crules[pcni->ruleix].info[pcni->lineix].operation = index;
}


/*
 *
 *
 * 	COLOR_RULE_COLOR_UPDATE
 * 
 *
 */
static void color_rule_color_update(pdue, pg, index)
due		*pdue;
gizmo		*pg;
int		index;
{
	struct color_rule_name_info	*pcni	= (struct color_rule_name_info *) pg->pae;

	pdue->dl_mdt = TRUE;
	if(pcni->lineix == MAX_RULE_LINES)
		pdue->dlc.pcrules->pdl_crules[pcni->ruleix].default_fg = index;
	else if(pcni->lineix == MAX_RULE_LINES+1)
		pdue->dlc.pcrules->pdl_crules[pcni->ruleix].default_bg = index;
	else
		pdue->dlc.pcrules->pdl_crules[pcni->ruleix].info[pcni->lineix].clr = index;
}



/*
 *
 *
 * 	COLOR_BUTTON_UPDATE
 * 
 *
 */
static void color_button_update(pdue, pg, value)
due		*pdue;
gizmo		*pg;
int		value;
{
	struct color_rule_name_info	*pcni	= (struct color_rule_name_info *) pg->pae;

	pdue->dl_mdt = TRUE;
	if(pcni->lineix == MAX_RULE_LINES)
		pdue->dlc.pcrules->pdl_crules[pcni->ruleix].fg_enable = value;
	else
		pdue->dlc.pcrules->pdl_crules[pcni->ruleix].bg_enable = value;
}



/*
 *
 *
 * 	NRULES_UPDATE
 * 
 *
 */
static void nrules_update(pdue, pg, pvalue)
due		*pdue;
gizmo		*pg;
char		*pvalue;
{
	Display	*d = pdue->d;
	int	nrules;
	int	nnewrules;
	int	status;
	int	i;
	int	x, y;

	status = sscanf(pvalue, "%d", &nrules);
	if(status != 1)
		return;

	if(nrules < 1)
		return;

	nnewrules = nrules - pdue->dlc.pcrules->nrules;

	if(nnewrules == 0)
		return;
	else if(nnewrules < 0){
		enum 		choices{cancel, decrease, terminate, nchoices};
		static char 	*pmsgs[5];

		char 		*pchoices[nchoices];
		int		choice;
		int		x_root, y_root;
		Window		child;
		int		status;

		pmsgs[0] = "~You are decreasing the size of your rule table.";
		pmsgs[1] = "~Some color rules at the table bottom will be discarded.";
		pmsgs[2] = "~Some elements may change to the static color modifier.";
		pmsgs[3] = "~Do you still want less rules?";
		pmsgs[4] = 0;

		pchoices[decrease]	= "~decrease";
		pchoices[cancel]	= "~cancel";
		pchoices[terminate]	= 0;
		status = XTranslateCoordinates(
			pdue->d,
			pg->w,
			RootWindowOfScreen(pg->pgg->s),
			0,
			0,
			&x_root,
			&y_root,
			&child);
		/*
		 * if they are not on the same screen it 
		 * is a fatal error
		 */
		assert(status);
		choice = perform_dialog(
			pdue->s, 
			x_root, 
			y_root, 
			pdue->gizmo_height, 
			pmsgs, 
			pchoices, 
			pdue->gizmo_height);
		if(choice == cancel)
			return;
	}

	pdue->dl_mdt = TRUE;

	/*
	 * free associated gizmo as needed
	 */
	if(pdue->dlc.pcrules->ppopg){
		x = pdue->dlc.pcrules->ppopg->obj.x;
		y = pdue->dlc.pcrules->ppopg->obj.y;
		free_gizmo(pdue->dlc.pcrules->ppopg);
		pdue->dlc.pcrules->ppopg = NULL;
	}else{
		x = 0;
		y = 0;
	}

	if(nnewrules > 0)
		edd_create_crules(pdue, pdue->dlc.pcrules->pdl_crules, nrules-nnewrules, nnewrules);
	else
		edd_create_crules(pdue, pdue->dlc.pcrules->pdl_crules, nrules, 0);

	modify_crules(pdue, x, y, COLOR_RULE_LISTEDIT_TAG);
}


/*
 *	gm_modify_color_rules()
 *	(see edd_menus.h)
 */
void gm_modify_color_rules(param, pdue, pbe)
void		*param;
due		*pdue;
XButtonEvent	*pbe;
{
	modify_crules(pdue, pbe->x_root - pdue->pixel_tolerence, pbe->y_root - pdue->pixel_tolerence, param);
}


/*
 *	EDD_CREATE_CRULES
 *
 */
void edd_create_crules(pdue, pdlc, noldrules, nnewrules)
due			*pdue;
struct dl_color_rule	*pdlc;
unsigned		noldrules;
unsigned		nnewrules;
{
	struct color_rules 	*pcrules;
	struct dl_color_rule	*pdl_crules;
	int			rule, line;

	pcrules = create_crules(pdlc, noldrules, nnewrules);
	assert(pcrules != NULL);

	pdl_crules = &(pcrules->pdl_crules[noldrules]);
	if(nnewrules > 0){
		/*
		 * initialize the new rules
		 */
		for(rule = 0; rule < nnewrules; rule++, pdl_crules++){
#ifndef TOKENPTRS
			pdl_crules->name	= stringInsert(&pdue->st, "");
#else
			pdl_crules->name	= dmTokenFromPointer64 (stringInsert(&pdue->st, ""));
#endif
			for(line = 0; line < MAX_RULE_LINES; line++){
#ifndef TOKENPTRS
				pdl_crules->info[line].chan	= stringInsert(&pdue->st, "");
				pdl_crules->info[line].value	= stringInsert(&pdue->st, "");
#else
				pdl_crules->info[line].chan	= dmTokenFromPointer64 (stringInsert(&pdue->st, ""));
				pdl_crules->info[line].value	= dmTokenFromPointer64 (stringInsert(&pdue->st, ""));
#endif
				pdl_crules->info[line].connector = dl_clrrule_use;
				pdl_crules->info[line].clr	= black;
			}
			pdl_crules->fg_enable	= 1;
			pdl_crules->default_fg	= white;
			pdl_crules->bg_enable	= 1;
			pdl_crules->default_bg	= black;
		}
	}
	edd_install_crules(pdue, pcrules);
}



/*
 *	EDD_INSTALL_COLOR_RULES
 *
 */
void edd_install_crules(pdue, pcrules)
due			*pdue;
struct color_rules 	*pcrules;
{
	if(pdue->dlc.pcrules)
		edd_delete_crules(pdue);

	pdue->dlc.pcrules = pcrules;
	update_color_rule_menu(pdue);
}



/*
 *	EDD_DELETE_CRULES
 *
 */
void edd_delete_crules(pdue)
due	*pdue;
{
	struct color_rules	*pcrules	= pdue->dlc.pcrules;
	
	if(pcrules == NULL)
		return;

	if(pcrules->ppopg){
		free_gizmo(pcrules->ppopg);
		pcrules->ppopg = NULL;
	}
       delete_crules(pcrules);
	pdue->dlc.pcrules = NULL;
}
