/*pick.c,v 1.1.1.1 1999/08/03 14:43:53 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	P I C K . C
 *
 *	Author: 	Jeffrey O. Hill
 *			hill@luke.lanl.gov
 *			505 665 1831
 *
 *	determine if a mouse click was on top of a display list object
 *
 *	History
 *	.01 joh 01xx91	Double click modify added
 *
 *	Improvements
 *	.01	it would be better if the double click time was 
 *		measured from button up to button up instead of from 
 *		button down to button down as it is done now for 
 *		logistical reasons
 */

#include        <edd.h>
#include        <dlinfo.h>
#include	<math.h>
#ifndef NeXT
#include	<values.h>
#endif

#define abs(A)	( (A)>0 ? (A) : -(A) )
#define pw2(A)	((A)*(A))

extern void *tisMalloc();
extern void *tisCalloc();
extern void tisFree();

static int	Xatan();
static int	find_with_pick();
static int	find_with_box();


/*
 *
 *	FIND_SELECTION()
 * 
 *
 */
int find_selection(pevent, pdue)
XEvent	*pevent;
due   *pdue;
{
	eue		*peue;
	eue_sel		*psel;
	unsigned int	tol	= pdue->pixel_tolerence;
	XRectangle	selrec;
	XPoint		point1;
	XPoint		point2;

#ifdef DEBUG
	assert(pevent->xbutton.type == ButtonPress);
#endif
	point1.x = pevent->xbutton.x;
	point1.y = pevent->xbutton.y;

	/* initial box size of zero */
	point2.x = point1.x;
	point2.y = point1.y;

	get_2nd_coord(	pdue->d, 
			pdue->dlc.w,
			pdue->invert_GC, 
			&point1,
			&point2,
			pdue->pick_cursor);

	/*
	 * wait until after mouse up
	 */
	if(pevent->xbutton.time - pdue->pick_time < DOUBLE_CLICK_MAX){
		modify_selected_items(NULL, pdue, pevent);
		return;
	}
	pdue->pick_time = pevent->xbutton.time;

	selrec.x	= min(point1.x, point2.x);
	selrec.y	= min(point1.y, point2.y);
	selrec.width	= posdiff(point1.x, point2.x);
	selrec.height	= posdiff(point1.y, point2.y);

	if(pevent->xbutton.button == SELECT_BUTTON)
		clear_selections(FALSE, pdue);

	if((selrec.width < tol) && (selrec.height < tol)){
		XPoint pick;

		pick.x = (point1.x + point2.x)>>1;
		pick.y = (point1.y + point2.y)>>1;
		find_with_pick(pdue, &pick, pevent->xbutton.time);
	}
	else{
		find_with_box(pdue, &selrec, pevent->xbutton.time);
		if(pevent->xbutton.button != SELECT_BUTTON)
			redraw_dl(pdue, &selrec);

		else if(pdue->selections.count)
			redraw_selections(pdue, NULL);
	}

	/*
	 * Allways clear the server's selection ownership if pick failed in
	 * case another application is the selection owner. (the user should
	 * be able to clear the current selection by clicking away from any
	 * object)
	 */
	if(pdue->selections.count){
		XSetSelectionOwner(pdue->dlc.d,
					XA_PRIMARY,
					pdue->dlc.w,
					pevent->xbutton.time);
		pdue->sel_time_stamp = pevent->xbutton.time;

	}
	else
		XSetSelectionOwner(pdue->dlc.d,
					XA_PRIMARY,
					None,
					pevent->xbutton.time);
	display_selection_status(pdue);
}



/*
 *
 *	FIND_WITH_PICK()
 * 
 *
 */
static int find_with_pick(pdue, ppick, time)
due		*pdue;
XPoint		*ppick;
Time		time;
{
	eue	*peue;
	int	tol	= pdue->pixel_tolerence;
	int	xplus	= ppick->x + tol;
	int	xminus	= ppick->x - tol;
	int	yplus	= ppick->y + tol;
	int	yminus	= ppick->y - tol;
	int	filled;


	/*
	 * start at the top of the dl and work down
	 */
	for(	peue = (eue *) pdue->dl.node.previous; 
		peue; 
		peue = (eue *) peue->node.previous){

		int			lw;
		int			x1, x2, y1, y2;
		XRectangle		bounds;
		struct dl_object_hdr	*pobj	= &peue->elem.dl_anygo.object;
		struct dl_element_hdr	*phdr	= &peue->elem.dl_anygo.element;
		enum dl_element_class	class	= dl_element_class[phdr->type];

		compute_bounds(peue, &bounds);

		x1 = bounds.x;
		y1 = bounds.y;
		x2 = bounds.x + bounds.width;
		y2 = bounds.y + bounds.height;

		/* the line width and filled attributes */
		if(class == dl_class_graphic){
			lw = peue->sta.width;

			filled = (peue->sta.fill == dl_fill_solid);
		}

		/*
		 *	Is the pick within the bounding box +- tol ?
		 *	(continue down the dl if not)
		 */
		if(xplus < x1)
			continue;
		if(yplus < y1)
			continue;
		if(xminus > x2)
			continue;
		if(yminus > y2)
			continue;

		if(class != dl_class_graphic){
			toggle_selection(pdue, peue, TRUE);
			return TRUE;
		}

		switch(phdr->type){
		case	dl_line:
		{
			/*
			 * Find the axis parallel pick extensions which
			 * intersect the line. Use the minimum of the
			 * distances to these as an approximate
			 * distance between the pick and the line.
			 * Approximating the distance to the line by
			 * the minimum of the x and y distance will
			 * be off by a maximum of 1.4 times the 
 			 * actual distance.
			 *
			 * NOTE: This needs work to support filled polygons better!
			 */
			unsigned int	i;
			struct dl_line  *pline  = &peue->elem.dl_line;

			for ( i = 0; i < pline->npoints-1; i++ )
			{
				struct dl_point	*ppoint = &pline->point[i];
				int		width  = (ppoint + 1)->x - ppoint->x;
				int		height = (ppoint + 1)->y - ppoint->y;
				int		dx, dy;

				if (width == 0)
				{
					dx = ppoint->x - ppick->x;
					dy = MAXINT;
				}
				else if (height == 0)
				{
					dx = MAXINT;
					dy = ppoint->y - ppick->y;
				}
				else
				{
					int	xl, yl;
					int	b;		/* the y intercept */

					b  = ppoint->y - (height*ppoint->x)/width;
					xl = ((ppick->y - b)*width)/height;
					yl = (height*ppick->x)/width + b;
					dx = ppick->x - xl;
					dy = ppick->y - yl;
				}
				if(abs(min(dx,dy)) < tol + lw)
				{
					toggle_selection(pdue, peue, TRUE);
					return TRUE;
				}
			}
			break;
		}

		case	dl_text:
		{	
			struct dl_text		*ptext		= (struct dl_text *)phdr;
			XFontStruct  		*pfont;
			static font_table	*pfont_table	= NULL; 
			unsigned		index;
			int			width;
			int			dx,dy;
			char 			*display_str;
			unsigned 		str_length;
			int			status;

			if (!pfont_table)
                              pfont_table = init_font_table(pdue->d);
			index = min(pobj->height, MAXFONTHEIGHT);
			pfont = pfont_table->height_to_pfinfo[index];

			/* I fill the rectangle if no font can be obtained */
			if(!pfont){
				toggle_selection(pdue, peue, TRUE);
				return TRUE;
			}
/*
 * done this way until vertical text strategy
 * firms up.
 */
#define TEXT_SELECT_WITH_BOX
#ifdef TEXT_SELECT_WITH_BOX
	toggle_selection(pdue, peue, TRUE);
	return TRUE;
#else
                      width = pfont->max_bounds.ascent + pfont->max_bounds.descent;
			dy = ppick->y - (pobj->y + (pobj->height>>1));
			dy = abs(dy);
			if(dy > (width>>1) + tol)
				break;

    			/* 
                       *convert string index to string pointer
                       */
			status = string_index_to_display_str(	
						pdue->dlc.pstring_table,
						ptext->textix, 
						&display_str, 
						&str_length);
                      assert(status >= 0);

			/*
			 * Test for select on top of string
			 */
			dx = ppick->x - pobj->x;
			if(dx > tol + str_length*pfont->max_bounds.width)
				break;

			toggle_selection(pdue, peue, TRUE);
			return TRUE;
#endif
			}
		case	dl_oval:
		case	dl_arc:
		{
			/*
			 * The following uses the equation for an
			 * elipse: x*x/a*a + y*y/b*b = 1. I have
			 * manipulated to the following in order to
			 * remain accurate with integers: b*b*x*x +
			 * a*a*y*y = a*a*b*b.
			 */
			int a;
			int b;
			int aa;
			int bb;
			int x	= ppick->x - pobj->x - (pobj->width>>1);
			int y	= ppick->y - pobj->y - (pobj->height>>1);
			int xx	= pw2(x);
			int yy	= pw2(y);

			a	= (pobj->width>>1) - (lw>>1) - tol;
			b	= (pobj->height>>1) - (lw>>1) - tol;
			aa	= pw2(a);
			bb	= pw2(b);
			if(xx*bb + yy*aa < aa*bb && !filled)
				break;

#ifdef DEBUG
			printf("~outside inside limit\n");
#endif
			a	= (pobj->width>>1) + (lw>>1) + tol;
			b	= (pobj->height>>1) + (lw>>1) + tol;
			aa	= pw2(a);
			bb	= pw2(b);
			if(xx*bb + yy*aa > aa*bb)
				break;
#ifdef DEBUG
			printf("~inside outside limit\n");
#endif
			/* check angle crteria on arcs */
			if(phdr->type == dl_arc){
				int		xdeg;
				struct dl_arc	*parc = (struct dl_arc *)phdr;
				int		begin = parc->begin % XDEG_PER_REV;
				int		diff;

				if(begin < 0)
					begin += XDEG_PER_REV;

				xdeg = Xatan(y, x);
				diff = xdeg-begin;
				if(diff < -XDEGREE_TOLERENCE)
					break;

				if(diff > parc->path + XDEGREE_TOLERENCE)
					break;
#ifdef DEBUG
				printf("~degree %d\n", xdeg/64);
#endif
			}

			toggle_selection(pdue, peue, TRUE);
			return TRUE;
		}
		case	dl_rec:
		case	dl_user:
			/*
			 * from above I know its inside the tol box so now I
			 * just check for close to edges
			 */
			if(filled ||
			  (xminus < x1 + lw) || (yminus < y1 + lw) ||
			  (xplus > x2 - lw) || (yplus > y2 - lw)){
				toggle_selection(pdue, peue, TRUE);
				return TRUE;
			}
			break;
		default:
			assert(FALSE);
		}
	}
	return FALSE;
}


/*
 *
 *	groupid_already_toggled()
 * 
 */
typedef struct {
	NODE	node;
	short	groupid;
} GROUPID_ELEMENT;

static int groupid_already_toggled(pgroupid_list, groupid)
LIST	*pgroupid_list;
short	groupid;
{
	GROUPID_ELEMENT	*p;

	if(groupid == NO_GROUP)
		return FALSE;

	/* look for a match in the list */
	for(p = (void *)lstFirst(pgroupid_list); p; p = (void *)lstNext(p))
		if(p->groupid == groupid)
			return TRUE;

	/* didn't find it, add the id to the list */
	/*p = (GROUPID_ELEMENT *)calloc(1, sizeof (GROUPID_ELEMENT));*/
	p = (GROUPID_ELEMENT *)tisCalloc(__FILE__, __LINE__, 1, sizeof (GROUPID_ELEMENT));
	p->groupid = groupid;
	lstAdd(pgroupid_list, &p->node);

	return FALSE;
}


/*
 *
 *	FIND_WITH_BOX()
 * 
 *	potential problem with arcs since they can be quite a bit
 *	smaller than their bounding box
 */
static int find_with_box(pdue, pselrec, time)
due		*pdue;
XRectangle	*pselrec;
Time		time;
{
	eue     *peue;
	int     tol		= pdue->pixel_tolerence;
	int     xp1_minus_tol	= pselrec->x - tol;
	int     yp1_minus_tol	= pselrec->y - tol;
	int     xp2_plus_tol	= pselrec->x + pselrec->width + tol;
	int     yp2_plus_tol	= pselrec->y + pselrec->height + tol;
	int     count		= 0;
	LIST	groupid_list;

	lstInit(&groupid_list);
	for(	peue = (eue *) pdue->dl.node.next; 
		peue; 
		peue = (eue *) peue->node.next){

		int			lw;
		int			x1, x2, y1, y2;
		int			filled;
		struct dl_element_hdr   *phdr   = &peue->elem.dl_anygo.element;
		struct dl_object_hdr    *pobj   = &peue->elem.dl_anygo.object;

		x1 = pobj->x;
		y1 = pobj->y;
		x2 = pobj->x + pobj->width;
		y2 = pobj->y + pobj->height;

		/* the line width and filled attributes */
		if(dl_element_class[phdr->type] == dl_class_graphic){
			lw = peue->sta.width;

			filled = (peue->sta.fill == dl_fill_solid);
			if(!filled){
				x1 -= (lw>>1);
				y1 -= (lw>>1);
				x2 += (lw>>1);
				y2 += (lw>>1);
			}
		}

		if(xp1_minus_tol > x1)
			continue;
		if(yp1_minus_tol > y1) 
			continue;
		if(xp2_plus_tol < x2)
			continue;
		if(yp2_plus_tol < y2)
			continue;

		if (groupid_already_toggled(&groupid_list, peue->elem.dl_anygo.groupid))
			continue;

		toggle_selection(pdue, peue, FALSE);
		count++;
	}
	lstFree(&groupid_list);
	return (count != 0);
}


/*
 *	approximation for the arc tangent of y/x:
 *
 *	a = y/x
 *	atan(a) = x / (1 + 0.28*x*x) + E(x) ; 0 <= x <= 1, abs(E(x)) <= 5*10E-03
 *	
 *	returns its result in the X standard 1/64th of a degree
 *
 */
#define X_atan(Y,X)	((XDEG_PER_RADIAN*100*(Y)*(X))/(100*pw2(X)+28*pw2(Y)))

/*
 *
 *	Xatan()
 *
 *
 */
static int Xatan(y, x)
int	y;
int	x;
{
	int	ymag = abs(y);
	int	xmag = abs(x);
	int 	result;

	if(x==0 && y==0)
		return 0;

	/* X inverts the y axis */
	y = -y;

	if(ymag<xmag)
		result = X_atan(ymag, xmag);
	else
		result = XDEG_PER_REV/4 - X_atan(xmag, ymag);

	if(x>0){
		if(y<0)
			result = (XDEG_PER_REV) - result;
	}
	else{
		if(y<0)
			result = (XDEG_PER_REV)/2 + result;
		else
			result = (XDEG_PER_REV)/2 - result;
	}

	return result;
}
