#include "opiStringDefs.h"
extern	char	*opiStringTab[];
/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *
 *	S E L E C T I O N S . C
 *
 *	Author	Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	Code segments which operate on the current X selection and
 *	the current editor selection.
 *	
 *	Functions in this file attempt to conform to the X ICCCM
 *
 *
 *
 *
 * 	History
 *
 *
 *
 *	Improvements: 
 *	01	dont call set sel owner in every selection add ?
 *
 *	02	go through all of the cut paste stuff and look at robustness
 *		ie what happens when large transfers occur and what happens
 *		to leftover resources etc.
 *
 */
#include        <edd.h>
#include	<edd_menus.h>
#include	<dlinfo.h>
#include	<dlcvrt.h>

static int 	selection_to_clipboard();
static void 	write_aux();
static void 	paste_clipboard();
static int 	valid_clipboard();
static void 	read_aux();
static int	write_clipboard_header();
static int	element_to_clipboard();
static int	selection_to_clipboard_as_string();

#ifdef TOKENPTRS
#include <tokenSupport.h>
#endif

extern void *tisMalloc();
extern void *tisCalloc();
extern void tisFree();


/*
 *
 *	SELECT_ELEMENT
 *
 */
static void select_element(pdue, peue, update_screen)
due		*pdue;
eue		*peue;
int		update_screen;
{
	eue_sel *psel	= PEUE_TO_PSEL(peue); 

	lstAdd(&pdue->selections, (NODE *)psel);
	peue->sel = TRUE;
	if(update_screen){
		XRectangle	bounds;

		DLOBJ_TO_XREC(peue->elem.dl_anygo.object, bounds);
		redraw_selection(pdue, &bounds);
		if( psel->eue.elem.dl_anygo.element.type == dl_line )
			redraw_line_endpoint_handles(pdue, &psel->eue.elem.dl_line);
	}
}


/*
 *
 *	DESELECT_ELEMENT
 *
 */
static void deselect_element(pdue, peue, update_screen)
due		*pdue;
eue		*peue;
int		update_screen;
{
	eue_sel *psel	= PEUE_TO_PSEL(peue); 

	lstDelete(&pdue->selections, (NODE *)psel);
	peue->sel = FALSE;
	if(update_screen){
		XRectangle	bounds;
		int		xborder	= pdue->xselect_border;
		int		yborder	= pdue->yselect_border;
			
		compute_bounds(peue, &bounds);
		bounds.x	-= xborder;
		bounds.y	-= yborder;
		bounds.width	+= (xborder<<1);
		bounds.height	+= (yborder<<1);
		redraw_dl(pdue, &bounds);
	}
}


/*
 *
 *	TOGGLE_SELECTION
 *
 */
void toggle_selection(pdue, peue, update_screen)
due		*pdue;
eue		*peue;
int		update_screen;
{
	short	groupid		= peue->elem.dl_anygo.groupid;
	void	(*action)()	= (peue->sel) ? deselect_element : select_element;  /* reverse the selection status */

	if(groupid == NO_GROUP)
		/*
		 * it's not in a group, just select or deselect it
		 */
		(*action)(pdue, peue, update_screen);
	else
		/*
		 * select or deselect ALL of the elements that match this ones groupid
		 */
		for(peue = (eue *) pdue->dl.node.next; peue; peue = (eue *)peue->node.next)
			if(peue->elem.dl_anygo.groupid == groupid)
				(*action)(pdue, peue, TRUE);
}


/*
 *
 *	CLEAR_X_SELECTIONS
 *	(called by a menu- see edd_menus.h)
 *
 */
void clear_x_selections(delete, pdue, pbe)
int		delete;
due		*pdue;
XButtonEvent	*pbe;
{
	clear_selections(delete, pdue);
	XSetSelectionOwner(pdue->dlc.d, XA_PRIMARY, None, pbe->time);
}


/*
 *
 *    SELECT_ALL()
 * 
 */
void select_all(dummy, pdue, pbe)
int           dummy;
due           *pdue;
XButtonEvent  *pbe;
{
	eue             *peue;
	XRectangle      maxrec;

	for(peue = (eue *) pdue->dl.node.next; peue; peue = (eue *) peue->node.next)
		select_element(pdue, peue, FALSE);

	if(pdue->selections.count){
		XSetSelectionOwner(pdue->dlc.d, XA_PRIMARY, pdue->dlc.w, pbe->time);
		pdue->sel_time_stamp = pbe->time;
	}
	selection_bounds(pdue, &maxrec, TRUE);
	redraw_dl(pdue, &maxrec);
}





/*
 *
 *	SEL_CLEAR_ACTION
 *
 *
 *
 *
 */
void sel_clear_action(pdue, psc)
due			*pdue;
XSelectionClearEvent	*psc;
{
	if(psc->selection == XA_PRIMARY)
		clear_selections(FALSE, pdue);
	else if(psc->selection == pdue->XA_CLIPBOARD){
		/* clear the clip board */
		XFree(pdue->clipboard);
		pdue->clipboard = NULL;
#ifdef DEBUG
		printf(opiStringTab[STR_clear_on_the_clipboardn     ]);
#endif
	}
}


/*
 *
 *	CLEAR_SELECTIONS
 *
 */
void clear_selections(delete, pdue)
int	delete;		/* delete from dl if TRUE */
due	*pdue;
{
	eue_sel		*psel;
	XRectangle	maxrec;


	if(!pdue->selections.count)
		return;

	selection_bounds(pdue, &maxrec, TRUE);
	add_markers_to_bounds(pdue, &maxrec);

	/*
	 * delete from the display list if asked and mark 
	 * unselected.
	 */
	while(psel = (eue_sel *) pdue->selections.node.next ){
		psel->eue.sel = FALSE;
		lstDelete(&pdue->selections, (NODE *)psel);
		if (delete){
			push_delete_undo(pdue, &psel->eue);
		}
	}

	/*
	 * erase from the screen all selection indicators and selected
	 * objects which have been deleted. Its faster to erase them with one
	 * redraw for all of them.
	 */
	redraw_objects(pdue, pdue->dl.node.next, &maxrec);
}



/*
 *
 *	REQUEST_SELECTIONS
 *
 *	!!!! Bypass for local cut/paste would be an improvement !!!!
 *	(see edd_menus.h)
 *
 */
void request_selections(sel_request, pdue, pbe)
enum sel_request	sel_request;
due			*pdue;
XButtonEvent		*pbe;
{
	Atom		selection;
	Atom		property;

	switch(sel_request){
	case cut_request:
	case copy_request:
		selection = XA_PRIMARY;
		property = pdue->XA_CLIPBOARD;
		break;
	case paste_request:
		selection = pdue->XA_CLIPBOARD;
		property = pdue->dl_seg_atom;
		break;
	}

	XConvertSelection(	pdue->dlc.d,
				selection,
				pdue->dl_seg_atom,
				property,
				pdue->dlc.w,
				pbe->time);

	/*
	 * it would be better if the selection were deleted only after a
	 * successful conversion has been acknowledged. This is not easy to
	 * do for now since info about cut/copy is not easy to reliably send
	 * to the recv routine.
	 */
	if(sel_request == cut_request){
		XConvertSelection(	pdue->dlc.d,
					selection,
					pdue->XA_DELETE,
					pdue->XA_DELETE,
					pdue->dlc.w,
					pbe->time);
	}

}



/*
 *
 *	SEND_SELECTIONS
 *
 *
 */
void send_selections(pdue, pxselreq)
due				*pdue;
XSelectionRequestEvent		*pxselreq;
{
	int			ok = TRUE;
	XSelectionEvent		xselnotify;

#ifdef DEBUG
	assert(pxselreq->type == SelectionRequest);
#endif
	/*
	 * supports obsolete clients (doesnt apply hear but its a good
	 * practice if this code is copied)
	 */
	if(pxselreq->property == None)
		pxselreq->property = pxselreq->target;


	/*
	 * set up the reply event
	 */
      	xselnotify.type = SelectionNotify;
	xselnotify.send_event = TRUE;
	xselnotify.display = pxselreq->display;
	xselnotify.requestor = pxselreq->requestor;
	xselnotify.selection = pxselreq->selection;
	xselnotify.target = pxselreq->target;
	xselnotify.property = pxselreq->property;
	xselnotify.time = pxselreq->time;

	/*
	 * Verify they are fetching from a time range during
	 * which I have owned the selection
	 */
	if(pxselreq->time < pdue->sel_time_stamp)
		ok = FALSE;

	/*
	 *
	 *
	 * Transfer to the property on their window
	 * 
	 * NOTE: transfers between clients machines with different floating
	 * point architectures will fail on X clients which send floating
	 * point data. There is no floating point data in the display list
	 * at this time.
	 * 
	 * transfers between clients machines with different byte ordering
	 * architectures are not allowed by this X client because of the
	 * difficulties in byte swapping the bit fields in a display list
	 * segment and because of the infrequency of this problem occuring.
	 * Nevertheless, a detection scheme is provided below which prevents
	 * transfers from occuring between incompatible environments. joh
	 * 7-25-90
	 * 
	 * currently there are no bitfields in the display list so this 
	 * problem could be addresssed.
	 * 6-26-91
	 *
	 */

	/*
	 * placing a magic number at the start of the display list segment
	 * provides for detection of byte swapping problems when the user
	 * tries to cut and paste between machines of different byte swapping
	 * architectures.
	 * 
	 * 8 specifies no byte swapping
	 */


	/*
	 * Verify that I support the target data type (some targets in ICCCM
	 * are requests for action such as DELETE the selection)
	 */
	if(ok){
		if(pxselreq->target == pdue->dl_seg_atom)
		{
			int		magic = DL_MAGIC;
			int		status;
			edd_stream	stream;

			/*
			 * Setup edd stream (Notification Event)
 			 */
#if defined(SOLARIS) || defined(WIN32)
      			memset(&stream, 0, sizeof(stream));
#else
      			bzero((char *)&stream, sizeof(stream));
#endif
			stream.type = use_xresource;
			stream.nbytes = 0;
      			stream.io_info.xsel = xselnotify;

			if(pxselreq->selection == XA_PRIMARY){
				status = selection_to_clipboard(pdue, &stream);
				if(status == ERROR)
					ok = FALSE;
#ifdef DEBUG
				printf(opiStringTab[STR_primary_fetch_processedn    ]);
#endif
			}
			else if(pxselreq->selection == pdue->XA_CLIPBOARD){

				status = edd_write(	&stream, 
							pdue->clipboard, 
							pdue->nbytes_cb);
				if(status == ERROR)
					ok = FALSE;

#ifdef DEBUG
				printf(opiStringTab[STR_clipboard_fetch_processedn  ]);
#endif
			}

		}
		else if(pxselreq->target == pdue->XA_DELETE){
			if (pxselreq->selection == XA_PRIMARY) {

				/*clear then delete */
				clear_selections(TRUE, pdue);
				XSetSelectionOwner(	pdue->dlc.d,
							XA_PRIMARY,
							None,
							pxselreq->time);

			}
			else if(pxselreq->selection == pdue->XA_CLIPBOARD){
				XChangeProperty(	pxselreq->display,
							pdue->dlc.w,
							pxselreq->selection,
							pxselreq->target,
							8,
							PropModeReplace,
							NULL,
							0);
			}
			else
				ok = FALSE;


			/* 
			 * tell em I did it according to ICCCM 
			 */
			if(ok){
				XChangeProperty(	pxselreq->display,
							pxselreq->requestor,
							pxselreq->property,
							pxselreq->target,
							8,
							PropModeReplace,
							NULL,
							0);
			}
		}
		else if(pxselreq->target == XA_STRING){
			edd_stream	stream;

			/*
			 * Setup edd stream (Notification Event)
 			 */
#if defined(SOLARIS) || defined(WIN32)
      			memset(&stream, 0, sizeof(stream));
#else
      			bzero((char *)&stream, sizeof(stream));
#endif
			stream.type = use_xresource;
			stream.nbytes = 0;
      			stream.io_info.xsel = xselnotify;

			if(pxselreq->selection == XA_PRIMARY){
				selection_to_clipboard_as_string(pdue, &stream);
				ok = FALSE;
			}
			else if(pxselreq->selection == pdue->XA_CLIPBOARD){

				ok = FALSE;

				sprintf(pdue->pmsg,
			opiStringTab[STR_Sorry_clipboard_fetch_of_STR]);
				redraw_msg(pdue);
			}
			
		}
		else{
			sprintf(pdue->pmsg,
		opiStringTab[STR_Sorry_cant_convert_clipboard], 
				pxselreq->target,
				pxselreq->selection);
			redraw_msg(pdue);
			ok = FALSE;
		}
	}

	if(!ok){
		xselnotify.property = None;
	}

	/*
	 *	Once all data is sent, tell em its ready
	 */
        XSendEvent(	xselnotify.display, 
			xselnotify.requestor, 
			True, 
			None, 
			(XEvent *)&xselnotify);
}


/*
 *
 *	RECV_SELECTIONS
 *
 *
 *
 *
 */
void recv_selections(pdue, pxsel)
due			*pdue;
XSelectionEvent		*pxsel;
{
#ifdef DEBUG
	assert(pxsel->type == SelectionNotify);
#endif

	if(pxsel->property == None){
		sprintf(pdue->pmsg,
		opiStringTab[STR_No_valid_selection_to_pasten]);
		redraw_msg(pdue);
		return;
	}


	if(pxsel->target == pdue->dl_seg_atom){
		if (pxsel->selection == XA_PRIMARY) {
			Atom		actual_type;
			int		actual_format;
			unsigned long	actual_nitems;
			unsigned long	bytes_left;
			int		status;
#ifdef DEBUG
			printf(opiStringTab[STR_primary_recvn               ]);
#endif
			if(pdue->clipboard)
				XFree(pdue->clipboard);

			pdue->clipboard = NULL;

			status = XGetWindowProperty(
					pxsel->display,
					pxsel->requestor,
					pxsel->property,
					0, /* offset */
					~0, /* fetch len */
					True,
					pxsel->target,
					&actual_type,
					&actual_format,	
					&actual_nitems,
					&bytes_left,
					(unsigned char **)&pdue->clipboard);
			if(	status == Success && 
				bytes_left == 0 && 
				actual_type == pxsel->target &&
				actual_format == 8 &&
				pdue->clipboard){

				pdue->nbytes_cb = actual_nitems;
				
				/*
				 * its a cut or copy so become the clipboard owner
				 * (the clipboard becomes the CLIPBOARD prop
				 *	on this window)
				 */
				XSetSelectionOwner(pdue->dlc.d,
						   pdue->XA_CLIPBOARD,
						   pdue->dlc.w,
						   pxsel->time);
				/*
				 * In case its a paste clear the selection
				 *
				 */
				clear_selections(FALSE, pdue);
				XSetSelectionOwner(	pdue->dlc.d,
							XA_PRIMARY,
							None,
							pxsel->time);
			}else
				assert(FALSE);

		}
		else if(pxsel->selection == pdue->XA_CLIPBOARD){
			/* its a paste */
			/* move the recv segment into the dl */
			Atom		actual_type;
			int		actual_format;
			unsigned long	actual_nitems;
			unsigned long	bytes_left;
			int		status;
			void		*psegment;
#ifdef DEBUG
			printf(opiStringTab[STR_clipboard_recv_pasten       ]);
#endif
			status = XGetWindowProperty(
						pxsel->display,
						pxsel->requestor,
						pxsel->property,
						0, /* offset */
						~0, /* fetch len */
						True,
						pxsel->target,
						&actual_type,
						&actual_format,	
						&actual_nitems,
						&bytes_left,
						(unsigned char **)&psegment);
			if(	status == Success && 
				bytes_left == 0 && 
				actual_type == pxsel->target &&
				actual_format == 8 &&
				psegment){

				paste_clipboard(	pdue,
							psegment,
							actual_nitems);
			}
		}
		else{
			sprintf(pdue->pmsg, 
				opiStringTab[STR_selection_cutpaste_protocol_]);
			redraw_msg(pdue);
			return;
		}
	}
	else if(pxsel->target == pdue->XA_DELETE){
#ifdef DEBUG
		printf(opiStringTab[STR_delete_confirmedn           ]);
#endif
	}
	else{

		char *pname;

		if(pxsel->target != None){
			pname = XGetAtomName(pdue->d, pxsel->target);
			if(pname){
				sprintf(pdue->pmsg,
	opiStringTab[STR_Selection_owner_converted_to],
				pname);
				XFree(pname);
				redraw_msg(pdue);
			}
		}
	}



	/* clear the resources in the server */
	XChangeProperty(	pxsel->display,
				pxsel->requestor,
				pxsel->property,
				pxsel->target,
				8,
				PropModeReplace,
				NULL,
				0);
}



/*
 *
 *	SELECTION_TO_CLIPBOARD
 *
 */
static int 
selection_to_clipboard(pdue, pstream)
due			*pdue;
edd_stream		*pstream;
{
	int		status;
	eue_sel		*psel;
	XRectangle	maxrec;

	/*
	 * find the bounds of the selection
	 */
	selection_bounds(pdue, &maxrec, TRUE);
#if 0
printf(opiStringTab[STR_nselection_to_clipboard_send], maxrec.width, maxrec.height);
#endif
	/*
	 * add a clipboard header
	 */
	status = write_clipboard_header(
			pstream, 
			maxrec.width, 
			maxrec.height);
	if(status<0)
		return ERROR;

	for(	psel = (eue_sel *) pdue->selections.node.next;
		psel;
		psel = (eue_sel *) psel->node.next){

		element_to_clipboard(pdue, &psel->eue, pstream, &maxrec);
	}

	return OK;
}


/*
 *
 *	WRITE_CLIPBOARD_HEADER
 *
 */
static int 
write_clipboard_header(pstream, width, height)
edd_stream	*pstream;
unsigned	width;
unsigned	height;
{
	clip_hdr	hdr;
	int		status;

	hdr.magic = DL_MAGIC;
	hdr.majv = DL_MAJOR_VERSION;
	hdr.mnrv = DL_MINOR_VERSION;
	hdr.width = width;
	hdr.height = height;

#if 0
printf(opiStringTab[STR_write_clipboard_header_writi], width, height);
#endif
	status = edd_write(pstream, &hdr, sizeof(hdr));
	if(status == ERROR)
		return ERROR;

	return OK;
}




/*
 *
 *	ELEMENT_TO_CLIPBOARD
 *
 */
static int 
element_to_clipboard(pdue, peue, pstream, pmaxrec)
due			*pdue;
eue			*peue;
edd_stream		*pstream;
XRectangle		*pmaxrec;
{
	int		status;
	unsigned 	type;
	struct sinfo	*psinfo;
	eue		tmp_eue;

	type = peue->elem.dl_anygo.element.type;
	psinfo = dl_elements_ginfo[type].sinfo;

	/*
	 * convert to a relocatable display segment
	 */
	tmp_eue = *peue;
	tmp_eue.elem.dl_anygo.object.x -= pmaxrec->x;
	tmp_eue.elem.dl_anygo.object.y -= pmaxrec->y;

	/*
	 *	write out the element
	  *
	 */
	status = edd_write(
			pstream, 
			&tmp_eue.elem, 
			tmp_eue.elem.dl_anygo.element.nwords<<1);
	if(status == ERROR)
		return ERROR;

	/*
	 *	write out any PCs, strings, or colors referenced
	  *
	 */
	write_aux(pdue, psinfo, &tmp_eue.elem, pstream);

	/*
	 *	write out attributes if required
	  *
	 */
	if(dl_element_class[type]==dl_class_graphic){
		status = edd_write(
			pstream, 
			&tmp_eue.sta, 
			sizeof(tmp_eue.sta));
		if(status == ERROR)
			return ERROR;

		write_aux(	pdue, 
				dl_sta_attr_sinfo, 
				&tmp_eue.sta, 
				pstream);

		status = edd_write(
					pstream, 
					&tmp_eue.dyn, 
					sizeof(tmp_eue.dyn));

		if(dynamic_graphical_element(&tmp_eue)){
			write_aux(	pdue, 
					dl_dyn_attr_sinfo, 
					&tmp_eue.dyn, 
					pstream);
		}
	}


	return OK;
}


/*
 *
 *	SELECTION_TO_CLIPBOARD_AS_STRING
 *
 */
static int 
selection_to_clipboard_as_string(pdue, pstream)
due			*pdue;
edd_stream		*pstream;
{
	eue_sel		*psel;

	for(	psel = (eue_sel *) pdue->selections.node.next;
		psel;
		psel = (eue_sel *) psel->node.next){

		dump_element(&psel->eue, &pdue->ct, &pdue->st);
	}

	return OK;
}


/*
 *
 *	WRITE_AUX
 *
 */
static void write_aux(pdue, psinfo, pstructure, pstream)
due		*pdue;
struct sinfo	*psinfo;
char		*pstructure;
edd_stream	*pstream;
{
	unsigned	count;
	unsigned	i;
        char	        *psubstruct;

	void		write_string();
	void		write_pc();
	extern int	counting_strings;

	for(	;
		psinfo->name;
		psinfo++){

		count = PSINFO_NELEMENTS(psinfo);

                /*
                 * incremental offset will be wrong if the last element
                 * was a bit field
                 */
                psubstruct = psinfo->offset + pstructure;

		for(i=0; i<count; i++){
			switch(psinfo->type){
			case dl_string_index:
				counting_strings = 1;
#ifndef TOKENPTRS
				stringSearchAndExecute(&pdue->st, *(char **)psubstruct, write_string, pstream);
#else
				stringSearchAndExecute(&pdue->st, dmPointer64fromToken(*(int *)psubstruct), write_string, pstream);
#endif
				break;

			case dl_pc_index:
				counting_strings = 1;
#ifndef TOKENPTRS
				stringSearchAndExecute(&pdue->ct, *(char **)psubstruct, write_pc, pstream);
#else
				stringSearchAndExecute(&pdue->ct, dmPointer64fromToken(*(int *)psubstruct), write_pc, pstream);
#endif
				break;

			case dl_struct:
				write_aux(pdue, psinfo->pinfo, psubstruct, pstream);
				break;
			}
			psubstruct += psinfo->size;
		}
	}

}



/*
 *
 *	PASTE_CLIPBOARD
 *
 */
static void paste_clipboard(pdue, psegment, nbytes)
due			*pdue;
char			*psegment;
unsigned		nbytes;
{
	clip_hdr	*phdr = (clip_hdr *) psegment;
	int		status;
	XPoint		p;
	XRectangle	maxrec;

	if(valid_clipboard(pdue, psegment, nbytes) == ERROR)
		return;

#if 0
printf(opiStringTab[STR_paste_clipboard_got_wHd_hHd_], phdr->width, phdr->height);
#endif
	status = get_1st_coord(	pdue->d, 
				pdue->dlc.w, 
				pdue->invert_GC, 
				phdr->width, 
				phdr->height, 
				&p,
				pdue->pick_cursor);
	if(status == ERROR){
		sprintf(pdue->pmsg,
			opiStringTab[STR_Paste_failed_during_position]);
		redraw_msg(pdue);
		return;
	}

	insert_clipboard(pdue, psegment, nbytes, &p, TRUE);

	maxrec.x = p.x;
	maxrec.y = p.y;
	maxrec.width = phdr->width;
	maxrec.height = phdr->height;

	redraw_dl(pdue, &maxrec);
}


/*
 *
 *	VALID_CLIPBOARD
 *
 */
static int valid_clipboard(pdue, psegment, nbytes)
due			*pdue;
char			*psegment;
unsigned		nbytes;
{
	clip_hdr	*phdr = (clip_hdr *) psegment;

	if(phdr->magic != DL_MAGIC){
		sprintf(pdue->pmsg,
			opiStringTab[STR_Paste_from_incompatible_CPU_]);
		redraw_msg(pdue);
		return ERROR;
	}

	if( phdr->majv != DL_MAJOR_VERSION || phdr->mnrv != DL_MINOR_VERSION ){
		sprintf(pdue->pmsg,
			opiStringTab[STR_cutcopypaste_version_mismatc]);
		redraw_msg(pdue);
		return ERROR;
	}
}


/*
 *
 *	remapped_groupid
 *
 */
typedef struct {
	NODE	node;
	short	old_groupid;
	short	new_groupid;
} GROUPID_MAPPING;

static short remapped_groupid(pdue, pgroupid_list, current_groupid)
due	*pdue;
LIST	*pgroupid_list;
short	current_groupid;
{
	GROUPID_MAPPING	*p;

	if(current_groupid == NO_GROUP)
		/* do not remap ones that are not in groups */
		return NO_GROUP;

	/* look for a match in the list */
	for(p = (void *)lstFirst(pgroupid_list); p; p = (void *)lstNext((NODE*)p))
		if(p->old_groupid == current_groupid)
			return p->new_groupid;

	/* didn't find it, add an id to the list */
	p = (GROUPID_MAPPING *)tisCalloc(__FILE__, __LINE__, 1, sizeof (GROUPID_MAPPING));
	p->old_groupid = current_groupid;
	p->new_groupid = pdue->next_groupid++;
	lstAdd(pgroupid_list, &p->node);

	return p->new_groupid;
}


/*
 *
 *	INSERT_CLIPBOARD
 *
 */
eue *insert_clipboard(pdue, psegment, nbytes, ppt, undoable)
due			*pdue;
char			*psegment;
unsigned		nbytes;
XPoint			*ppt;
int			undoable;
{
	clip_hdr	*phdr = (clip_hdr *) psegment;
	eue		*pdl_eue = NULL;
	XRectangle	maxrec;
	char		*pend = psegment + nbytes;
	LIST		groupid_list;

	if(valid_clipboard(pdue, psegment, nbytes) == ERROR)
		return NULL;

	lstInit(&groupid_list);

	maxrec.x = ppt->x;
	maxrec.y = ppt->y;
	maxrec.width = phdr->width;
	maxrec.height = phdr->height;

	for(	psegment = (char *)(phdr+1); 
		psegment < pend;){
		union dl_elements_union		*pelem;
		enum dl_elements 		type;
		struct sinfo			*psinfo;

		pelem = (union dl_elements_union *) psegment;
		psegment += (pelem->dl_anygo.element.nwords<<1);
		type = pelem->dl_anygo.element.type;
		psinfo = dl_elements_ginfo[type].sinfo;

		/*
		 * offset the paste to the user provided position
		 */
		pelem->dl_anygo.object.x += maxrec.x;
		pelem->dl_anygo.object.y += maxrec.y;
		if ( type == dl_line )
			adjust_line_endpoints(&(pelem->dl_line), 0, 0);

		pelem->dl_anygo.groupid = remapped_groupid(pdue, &groupid_list, pelem->dl_anygo.groupid);

		/*
		 * create element from info in psegment
		 * (read aux gets strings and PC's as required)
		 */
		pdl_eue = create_element(
				pdue, 
				pelem, 
				pelem->dl_anygo.element.nwords<<1);
		read_aux(pdue, psinfo, &pdl_eue->elem, &psegment);

		/*
		 * fetch element attributes from info in psegment
		 * (read aux gets strings and PC's as required)
		 */
		if(dl_element_class[type]==dl_class_graphic){
			pdl_eue->sta = *((struct dl_sta_attr *)psegment);
			psegment += sizeof(pdl_eue->sta);
			read_aux(	
				pdue, 
				dl_sta_attr_sinfo, 
				&pdl_eue->sta, 
				&psegment);
			pdl_eue->dyn = *((struct dl_dyn_attr *)psegment);
			psegment += sizeof(pdl_eue->dyn);
			if(dynamic_graphical_element(pdl_eue)){
				read_aux(	
					pdue, 
					dl_dyn_attr_sinfo, 
					&pdl_eue->dyn, 
					&psegment);
			}
		}

		if(undoable){
			push_create_undo(pdue, pdl_eue);
		}
	}
	lstFree(&groupid_list);

	return pdl_eue;
}


/*
 *
 *	READ_AUX
 *
 */
static void read_aux(pdue, psinfo, pstructure, psegment)
due		*pdue;
struct sinfo	*psinfo;
char		*pstructure;
short		**psegment;
{
	unsigned	count;
	unsigned	i;
        char	        *psubstruct;



	for(	;
		psinfo->name;
		psinfo++){

		count = PSINFO_NELEMENTS(psinfo);

                /*
                 * incremental offset will be wrong if the last element
                 * was a bit field
                 */
                psubstruct = psinfo->offset + pstructure;

		for(i=0; i<count; i++){
			switch(psinfo->type){
			case dl_string_index:
			{
				struct dl_string	*pstr;

				pstr = (struct dl_string *) *psegment;
				assert(pstr->element.type == dl_string);
#ifndef TOKENPTRS
				*(char **)psubstruct = stringInsert(&pdue->st, (char *)(pstr+1));
#else
				*(int *)psubstruct = dmTokenFromPointer64 (stringInsert(&pdue->st, (char *)(pstr+1)));
#endif
				*psegment += pstr->element.nwords;
				break;
			}
			case dl_pc_index:
			{
				struct dl_pc	*ppc;

				ppc = (struct dl_pc *) *psegment;
				assert(ppc->element.type == dl_pc);
#ifndef TOKENPTRS
				*(char **)psubstruct = stringInsert(&pdue->ct, (char *)(ppc+1));
#else
				*(int *)psubstruct = dmTokenFromPointer64 (stringInsert(&pdue->ct, (char *)(ppc+1)));
#endif
				*psegment += ppc->element.nwords;
				break;
			}
			case dl_struct:
				read_aux(
					pdue, 
					psinfo->pinfo, 
					psubstruct, 
					psegment);
				break;
			}
                        psubstruct += psinfo->size;
		}
	}

}

