 /*message_gz.c,v 1.1.1.1 1999/08/03 14:43:57 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	M E S S A G E _ G Z . C
 *
 *	Author: Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	push button gizmo.
 *	sends message at button down and button up.
 *	(Allways pops out when they release the mouse)
 *
 *	History
 *	.01 120291 joh	no longer includes vxWorks.h
 */

#include 	<stdio.h>
#include 	<Xlib.h>
#include 	<Xutil.h>
#include	<keysym.h>
#ifdef AXP
#include	<string.h>
#endif
#include	<font_table.h>
#include 	<gizmo.h>
#include	<color_map.h>

typedef	char	enumValue [40];
typedef struct
	{
	char		**plabels;
	unsigned	button_down:1;
	unsigned	pointer_inside:1;
	unsigned	RISC_pad:14;
	int		(*permissive_func)();
	int		permissive_result;
	int		behavior;
	int		toggledOn;
	enumValue	enumStrings [2];
	} message_gizmo_ext;

static void draw_button();
static void redraw_button();

#define NOSELECTION ((unsigned)0xffff)
#define BUTTONDOWN (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)


/*
 *	MESSAGE_INIT
 */
void message_init(pgg)
gizmo_global *pgg;
	{
	}


/*
 *	CREATE_CHOICE_GIZMO
 */
gizmo *create_message_gizmo(s, w, pobj, perm_func, func, arg, plabels, behavior, zeroStr, oneStr, initValue)
Screen		*s;		/* parent's screen */
Window		w;		/* parent window */
XRectangle	*pobj;		
int		(*perm_func)();
void		(*func)();
void		*arg;
char		**plabels;	/* need two pointers */
int		behavior;
char		*zeroStr, *oneStr;
int		initValue;
	{
	gizmo			*pg;
	message_gizmo_ext	*pmge;
	unsigned		ext_size = 2 * sizeof (char *);	/* one for press, one for release */

	pg = create_gizmo(gz_message, s, w, pobj, func, arg, sizeof(message_gizmo_ext) + ext_size,
	  ButtonPressMask|ButtonReleaseMask|EnterWindowMask|LeaveWindowMask|ExposureMask);
	if(!pg)
		return NULL;

	pmge = (message_gizmo_ext *) (pg+1);
	pg->pae = (void *) (pmge+1);

	if (behavior == 0) {
	    ((char **)(pg->pae))[0] = *plabels[0] ? plabels[0] : plabels[1];	/* press label: always put something */
 	    ((char **)(pg->pae))[1] = *plabels[1] ? plabels[1] : plabels[0];	/* release label: always put somthing */
	} else {
	    strcpy (pmge->enumStrings [0], *plabels [0] ? plabels[1] : zeroStr);
	    strcpy (pmge->enumStrings [1], *plabels [0] ? plabels[0] : oneStr);
 	    ((char **)(pg->pae))[0] = pmge->enumStrings [0];
	    ((char **)(pg->pae))[1] = pmge->enumStrings [1];
	}
	pmge->plabels = (char **)(pg->pae);
	pmge->permissive_func = perm_func;
	pmge->permissive_result = FALSE;
	pmge->button_down = FALSE;
	pmge->behavior = behavior;
	pmge->toggledOn = initValue;

	assert(XSaveContext(pg->pgg->d, pg->w,  gizmo_ctx, (XPointer)pg) == 0);
	assert(XSaveContext(pg->pgg->d, pg->tw, gizmo_ctx, (XPointer)pg) == 0);

	XMapWindow(pg->pgg->d, pg->w);
	return pg;
	}



/*
 * 	MESSAGE_FREE
 */
void message_free(pg)
gizmo *pg;
	{
	}



/*
 * 	MESSAGE_ACTION
 */
void message_action(pevent, pg)
XEvent		*pevent;
gizmo		*pg;
	{
	message_gizmo_ext	*pbge = (message_gizmo_ext *)(pg+1);

	switch (pevent->type)
	  {
	  case EnterNotify:
		if ((pbge->permissive_result = (*pbge->permissive_func)(pg->arg, pg, FALSE)) == FALSE)
			break;
		pbge->pointer_inside = TRUE;
		XSetWindowBorder(pg->pgg->d, pg->w, pg->fg);
		XDefineCursor(pg->pgg->d, pg->tw, pg->pgg->cursor);
		break;
	  case LeaveNotify:
		if (pbge->permissive_result == FALSE)
			break;
		pbge->pointer_inside = FALSE;
		XSetWindowBorder(pg->pgg->d, pg->w, pg->bg);
		XUndefineCursor(pg->pgg->d, pg->tw);
		break;
	  case ButtonPress:
		if ((pbge->permissive_result = (*pbge->permissive_func)(pg->arg, pg, TRUE)) == FALSE)
		    break;
		pbge->button_down = TRUE;
		if (pbge->behavior == 0) {	/* Momentary */
		    if (pg->func)
			(*pg->func)(pg->arg, pg, TRUE, pevent);
		} else {	/* Latched */
		    pbge->toggledOn = !pbge->toggledOn;
		    if (pg->func)
			(*pg->func)(pg->arg, pg, pbge->toggledOn, pevent);
		}
		redraw_button(pg);
		break;
	  case ButtonRelease:
		if (pbge->permissive_result == FALSE)
		    break;
		pbge->button_down = FALSE;
		redraw_button(pg);
		if (pbge->behavior == 0)
		    if (pg->func)
			(*pg->func)(pg->arg, pg, FALSE, pevent);
		break;
	  case Expose:
		redraw_button(pg);
		break;
	  }
	}


/*
 *	REDRAW_BUTTON
 */

static void redraw_button(pg)

    gizmo	*pg;

    {

	message_gizmo_ext	*pmge = (message_gizmo_ext *)(pg+1);
	XRectangle		box;

	if (!pg->mapped)
		return;

	XSetForeground(pg->pgg->d, pg->pgg->bright_GC, pg->bg);
	XSetForeground(pg->pgg->d, pg->pgg->shadow_GC, pg->bg);
	XSetForeground(pg->pgg->d, pg->pgg->normal_GC, pg->bg);
	XSetForeground(pg->pgg->d, pg->pgg->fg_GC, pg->fg);
	XSetForeground(pg->pgg->d, pg->pgg->grey_fg_GC, pg->fg);
	XSetBackground(pg->pgg->d, pg->pgg->grey_fg_GC, pg->bg);
	
	box.x = box.y = 0;
	box.width = pg->obj.width;
	box.height = pg->obj.height;
	if (pmge->behavior == 0)	/* Momentrary */
	    draw_button(pg, &box, pmge->button_down, pmge->plabels [pmge->button_down ? 0 : 1]);
	else
	    draw_button(pg, &box, pmge->button_down, pmge->enumStrings [pmge->toggledOn ? 0 : 1]);

}


/*
 *	DRAW_BUTTON
 */
static void draw_button(pg, prec, in, plabel)
gizmo		*pg;
XRectangle	*prec;
int			in;
char		*plabel;
	{
	Display 	*d = pg->pgg->d;
	Window 		w = pg->w;
	XRectangle 	box;
	int			lw;
	int			lwt2;
	int			lwd2;
	int			x2,y2;
	GC			ulgc;
	GC			brgc;
	GC			centergc;
	GC			textgc;

	if (in)
		{
		ulgc = pg->pgg->black_GC;
		brgc = pg->pgg->bright_GC;
		centergc = pg->pgg->shadow_GC;
		textgc = pg->pgg->fg_GC;
		}
	else
		{
		ulgc = pg->pgg->bright_GC;
		brgc = pg->pgg->black_GC;
		centergc = pg->pgg->normal_GC;
		textgc = pg->pgg->grey_fg_GC;
		}

	lw = max(ulgc->values.line_width, brgc->values.line_width);
	lwt2 = (lw<<1);
	lwd2 = (lw>>1);

	if((unsigned int)(prec->width) < lwt2 || (unsigned int)(prec->height) < lwt2 )
		/*
		 * Draw nothing if to small
		 */
		return;

	box = *prec;
	XFillRectangle(d, w, centergc, box.x, box.y, box.width, box.height);

	/*
	 * 3d reasonable only on 4 or more planes
	 */
	if (DefaultDepth(d, DefaultScreen(d)) >= 4)
		{
		/*
		 * filled poly used here to get miter corner with diffrent color lines
		 */
		XPoint	points[7];
		int		inside_width = box.width - lwt2;
		int		inside_height = box.height - lwt2;

		points[0].x = box.x;
		points[0].y = box.y;
		points[1].x = box.width;
		points[1].y = 0;
		points[2].x = -lw;
		points[2].y = lw;
		points[3].x = -inside_width;
		points[3].y = 0;
		points[4].x = 0;
		points[4].y = inside_height;
		points[5].x = -lw;
		points[5].y = lw;
		points[6].x = 0;
		points[6].y = -box.height;
		XFillPolygon(d, w, ulgc, points, NELEMENTS(points), Nonconvex, CoordModePrevious);

		points[0].x = box.x+box.width;
		points[0].y = box.y+box.height;
		points[1].x = -box.width;
		points[1].y = 0;
		points[2].x = lw;
		points[2].y = -lw;
		points[3].x = inside_width;
		points[3].y = 0;
		points[4].x = 0;
		points[4].y = -inside_height;
		points[5].x = lw;
		points[5].y = -lw;
		points[6].x = 0;
		points[6].y = box.height;
		XFillPolygon(d, w, brgc, points, NELEMENTS(points), Nonconvex, CoordModePrevious);
		}
	else
		{	
		XRectangle outline;

		outline = box;
		outline.x += lwd2;
		outline.y += lwd2;
		outline.width -= lw;
		outline.height -= lw;
		XDrawRectangle(d, w, textgc, outline.x, outline.y, outline.width, outline.height);
		}

	box.x += lwd2;
	box.y += lwd2;
	box.width -= lw;
	box.height -= lw;
	draw_string(d, w, textgc, &box, pg->pgg->pft, plabel, strlen(plabel), box_center);
	}
