/*palette_gz.c,v 1.1.1.1 1999/08/03 14:43:57 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *
 *	P A L E T T E _ G Z . C
 *
 *	Author	Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	a gizmo envelope for a color palette menu
 *
 * 	History
 *	.01 120291 joh	No longer includes vxWorks.h
 *	.02 121191 joh	removed use of NONE
 */
#include 	<stdio.h>
#include 	<Xlib.h>
#include 	<Xutil.h>
#include	<keysym.h>
#include	<font_table.h>
#include 	<gizmo.h>
#include	<color_map.h>
#ifndef INCpalette_menush
#include	<palette_menus.h>
#endif

static void		resize_palette_gizmo();

/*
 *	Extra indirections on pixel pointer tracks colormap changes
 */
typedef struct
	{
	Font			font;
	unsigned short	ycentered;
	unsigned short	char_width;
	XPoint			menu_icon_points[4];
	long			**pp_pixels;
	unsigned		*pnpixels;
	unsigned char	selection;
	} palette_gizmo_ext;

static void redraw_palette_gz();


/*
 * 	PALETTE_INIT
 * 
 * 	NOOP for now
 */
void palette_init(pgg)
gizmo_global *pgg;
	{
	}


/*
 *	CREATE_PALLETTE_GIZMO
 *
 *	extra indirection in pppixels alows the colormap to change while this gizmo is running
 */
gizmo *create_palette_gizmo(s, w, pobj, func, arg, pppixels, pnpixels, ext_size)
Screen		*s;		/* parent's screen */
Window		w;		/* parent window */
XRectangle	*pobj;		
void		(*func)();
void		*arg;
long		**pppixels;
unsigned	*pnpixels;
unsigned	ext_size;	/* size of the application extension */
	{
	gizmo				*pg;
	palette_gizmo_ext	*ppge;
	unsigned			index;
	XFontStruct			*pfi;

	pg = create_gizmo(gz_palette, s, w, pobj, func, arg, sizeof(palette_gizmo_ext) + ext_size,
	  PALETTE_MENU_MIN_EVENT_MASK|EnterWindowMask|LeaveWindowMask|ButtonPressMask|ExposureMask);
	if (!pg)
		return NULL;

	ppge = (palette_gizmo_ext *) (pg+1);
	if (ext_size)
		pg->pae = (void *) (ppge+1);

	ppge->selection = 0; 		/* to detect changes 	*/
	ppge->pp_pixels = pppixels;
	ppge->pnpixels = pnpixels;
	resize_palette_gizmo(pg);

	assert(XSaveContext(pg->pgg->d, pg->w,  gizmo_ctx, (XPointer)pg) == 0);
	assert(XSaveContext(pg->pgg->d, pg->tw, gizmo_ctx, (XPointer)pg) == 0);
	return pg;
	}


/*
 * 	PALETTE_FREE
 */
static void resize_palette_gizmo(pg)
gizmo *pg;
	{
	palette_gizmo_ext	*ppge = (palette_gizmo_ext *)(pg+1);
	unsigned			index;
	XFontStruct			*pfi;

	index = min(NELEMENTS(pg->pgg->pft->height_to_pfinfo)-1, pg->obj.height);
	pfi = pg->pgg->pft->height_to_pfinfo[index];
	if (pfi)
		{
		unsigned char_height = pfi->max_bounds.ascent + pfi->max_bounds.descent;

		ppge->font = pfi->fid;
		ppge->ycentered = ((pg->obj.height-char_height)>>1) + pfi->max_bounds.ascent;
		ppge->char_width = pfi->max_bounds.width;

		/* first x point set prior to each redraw_palette_gz */
		ppge->menu_icon_points[0].x = 0;
		ppge->menu_icon_points[0].y = ppge->ycentered + pfi->max_bounds.descent;
		/* the rest are offsets */
		ppge->menu_icon_points[1].x = 0;
		ppge->menu_icon_points[1].y = - char_height;
		ppge->menu_icon_points[2].x = ppge->char_width;
		ppge->menu_icon_points[2].y = char_height>>1;
		ppge->menu_icon_points[3].x = -ppge->char_width;
		ppge->menu_icon_points[3].y = char_height>>1;
		}
	}


/*
 * 	PALETTE_FREE
 */
void palette_free(pg)
gizmo *pg;
	{
	}


/*
 * SET_PALETTE_GIZMO
 */
void set_palette_gizmo(pg, init)
gizmo 		*pg;
unsigned	init;
	{
	palette_gizmo_ext	*ppge = (palette_gizmo_ext *)(pg+1);

	if (ppge->selection != init)
		{
		ppge->selection = init;
		redraw_palette_gz(pg);
		}
	}


/*
 * PALETTE_ACTION
 */
void 	palette_action(pevent, pg)
XEvent	*pevent;
gizmo	*pg;
	{
	palette_gizmo_ext	*ppge = (palette_gizmo_ext *)(pg+1);

	switch (pevent->type)
	  {
	  case LeaveNotify:
		XSetWindowBorder(pg->pgg->d, pg->w, pg->bg);
		break;
	  case EnterNotify:
		XSetWindowBorder(pg->pgg->d, pg->w, pg->fg);
		break;
	  case ButtonPress:
		if (pevent->xbutton.button == MENU_BUTTON)
			{
			unsigned int	selection	= post_palette_menu(pevent, *ppge->pp_pixels, *ppge->pnpixels);
		
			if (selection < 0)
				return;

			if (ppge->selection != selection)
				{
				ppge->selection = selection;
				redraw_palette_gz(pg);
				(*pg->func)(pg->arg, pg, selection);

				/*
				 * complete all labor for the gizmo before calling their routine since they may delete this gizmo!
				 */
				return;
				}
			}
		break;
	  case Expose:
		redraw_palette_gz(pg);
		break;

	  case ConfigureNotify:
		resize_palette_gizmo(pg);
		break;
	  }
	}


/*
 *	REDRAW_PALETTE_GIZMO
 */
static void redraw_palette_gz(pg)
gizmo *pg;
	{
	palette_gizmo_ext	*ppge	= (palette_gizmo_ext *)(pg+1);
	Display				*d		= pg->pgg->d;
	Window				w		= pg->w;

	if(!pg->mapped)
		return;

	XSetForeground(d, pg->pgg->fg_GC, pg->fg);
	if (ppge->selection == invisibleColor)
		{
		XSetWindowBackground(d, w, pg->pgg->dialog_bg);
		XClearWindow(d, w);
		XSetFont(d, pg->pgg->fg_GC, ppge->font);
		tisDrawString(d, w, pg->pgg->fg_GC, 0, ppge->ycentered, "~Invisible", 9);
		}
	else
		{
		XSetWindowBackground(d, w, (*ppge->pp_pixels)[ppge->selection]);
		XClearWindow(d, w);
		XFillPolygon(d, w, pg->pgg->fg_GC, ppge->menu_icon_points, NELEMENTS(ppge->menu_icon_points), Convex,
		  CoordModePrevious);
		}
	}
