#include "opiStringDefs.h"
extern	char	*opiStringTab[];
/*t_choice_gz.c,v 1.1.1.1 1999/08/03 14:43:57 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *	C H O I C E _ G Z . C
 *
 *	Author: Jeffrey O. Hill
 *		hill@luke.lanl.gov
 *		505 665 1831
 *
 *	car radio button (choice) gizmo
 *	(only one button engaged at a time)
 *
 *	History
 *	.00 050891 joh	recoded for less flicker and less 
 *			roundoff error when determining
 *			the button's position
 *	.01 091991 joh	Dont let them make invalid button choice
 *	.02 120291 joh	No longer includes vxWorks.h
 */

#include 	<stdio.h>
#include 	<X11/Xlib.h>
#ifdef WIN32
#include <X11/XlibXtra.h>
#endif WIN32
#include 	<X11/Xutil.h>
#include	<X11/keysym.h>

#ifdef AXP
#include	<string.h>
#endif

#include	"font_table.h"

#include 	"gizmo.h"
#include	"draw_strings.h"

typedef struct
	{
	char			**pchoices;
	unsigned short	nchoices;
	unsigned short	selection;
	unsigned short	motion_selection;
	unsigned short	stacking;
	unsigned short	nrow;	/* almost the square root of nchoices */
	unsigned short	ncol;
	unsigned		button_down:1;
	unsigned		pointer_inside:1;
	unsigned		RISC_pad:14;
	int				(*permissive_func)();
	int				permissive_result;
	} choice_gizmo_ext;

static void draw_button();
static void redraw_numbered_button();
static what_button();

#define NOSELECTION ((unsigned)0xffff)
#define BUTTONDOWN (Button1Mask|Button2Mask|Button3Mask|Button4Mask|Button5Mask)


/*
 *	CHOICE_INIT
 *
 */
void choice_init(pgg)
gizmo_global *pgg;
	{
	}


/*
 *	CREATE_CHOICE_GIZMO
 *
 *
 *
 */
gizmo *create_choice_gizmo(s, w, pobj, perm_func, func, arg, pchoices, stacking, ext_size)
Screen						*s;		/* parent's screen */
Window						w;		/* parent window */
XRectangle					*pobj;		
int							(*perm_func)();
void						(*func)();
void						*arg;
char						**pchoices;
enum choice_gizmo_stacking	stacking;
unsigned					ext_size;
	{
	gizmo				*pg;
	choice_gizmo_ext	*pbge;
	int					nchoices;
	int 				tmp;

	pg = create_gizmo(gz_choice, s, w, pobj, func, arg, sizeof(choice_gizmo_ext) + ext_size,
		ButtonPressMask|ButtonReleaseMask|EnterWindowMask|
		LeaveWindowMask|ExposureMask|ButtonMotionMask);
	if (!pg)
		return NULL;

	pbge = (choice_gizmo_ext *) (pg+1);
	if (ext_size)
		pg->pae = (void *) (pbge+1);

	pbge->pchoices = pchoices;
	for (nchoices = 0; pchoices[nchoices]; nchoices++)
		;
	pbge->nchoices			= nchoices;
	pbge->permissive_func	= perm_func;
	pbge->permissive_result	= FALSE;
	pbge->stacking			= stacking;
	switch (pbge->stacking)
	  {
	  case gz_stack_row:
		pbge->nrow = 1;
		pbge->ncol = nchoices;
		break;
	  case gz_stack_column:
		pbge->nrow = nchoices;
		pbge->ncol = 1;
		break;
	  case gz_stack_row_column:
		for (tmp = nchoices; tmp*tmp > nchoices; tmp = (tmp>>1))	/* Fast approx to a square root */
			;
		pbge->nrow = (nchoices-1)/tmp + 1;
		pbge->ncol = tmp;
		break;
	  default:
		assert(FALSE);
	  }
	assert(XSaveContext(pg->pgg->d, pg->w,  gizmo_ctx, (XPointer)pg) == 0);
	assert(XSaveContext(pg->pgg->d, pg->tw, gizmo_ctx, (XPointer)pg) == 0);

	XMapWindow(pg->pgg->d, pg->w);
	return pg;
	}



/*
 * 	CHOICE_FREE
 */
void choice_free(pg)
gizmo *pg;
	{
	}




/*
 * 	SET_CHOICE_GIZMO
 */
int set_choice_gizmo(pg, init)
gizmo 		*pg;
unsigned	init;
	{
	choice_gizmo_ext	*pbge = (choice_gizmo_ext *)(pg+1);
	unsigned			oldsel;

	if (pbge->selection != init)
		{
		if (init >= pbge->nchoices)
			return ERROR;

		oldsel = pbge->selection;
		pbge->selection = init;
		if (pbge->button_down)
			return ERROR;
		else
			{
			redraw_numbered_button(pg, oldsel);
			redraw_numbered_button(pg, init);
			}
		}
	return OK;
	}


/*
 * 	CHOICE_GIZMO_VALUE
 */
int choice_gizmo_value(pg)
gizmo	*pg;
	{
	return ((choice_gizmo_ext *)(pg+1))->selection;
	}


/*
 * 	CHOICE_ACTION
 */
void 	choice_action(pevent, pg)
XEvent		*pevent;
gizmo		*pg;
	{
	choice_gizmo_ext	*pbge = (choice_gizmo_ext *)(pg+1);
	unsigned 			button_no;
	unsigned 			oldsel;
	unsigned 			newsel;

	switch (pevent->type)
	  {
	  case EnterNotify:
		if ( (pbge->permissive_result = (*pbge->permissive_func)(pg->arg, pg, FALSE)) == FALSE )
			break;
		pbge->pointer_inside = TRUE;
		XSetWindowBorder(pg->pgg->d, pg->w, pg->fg);
		XDefineCursor(pg->pgg->d, pg->tw, pg->pgg->cursor);
		if (pbge->button_down)
			{
			pbge->motion_selection = what_button(pg, pevent->xcrossing.x, pevent->xcrossing.y);
			if(pbge->motion_selection != pbge->selection)
				{
				redraw_numbered_button(pg, pbge->motion_selection);
				redraw_numbered_button(pg, pbge->selection);
				}
			}
		break;
	  case LeaveNotify:
		if ( pbge->permissive_result == FALSE )
			break;
		pbge->pointer_inside = FALSE;
		XSetWindowBorder(pg->pgg->d, pg->w, pg->bg);
		XUndefineCursor(pg->pgg->d, pg->tw);
		if (pbge->button_down)
			if (pbge->motion_selection != pbge->selection)
				{
				redraw_numbered_button(pg, pbge->motion_selection);
				redraw_numbered_button(pg, pbge->selection);
				}
		break;
	  case MotionNotify:
		if (pbge->permissive_result == FALSE)
			break;
		if (pevent->xmotion.state && BUTTONDOWN)
			{
			unsigned	old_motion_sel	= pbge->motion_selection;
			
			pbge->motion_selection = what_button(pg, pevent->xmotion.x, pevent->xmotion.y);
			if (old_motion_sel != pbge->motion_selection)
				{
				/*
				 * leaving the window
				 */
				if (pbge->motion_selection == NOSELECTION)
					{
					/*
					 * This kludge fixes a bug in the XNEWS server (it does not tell me about enter/leave events
					 * when a grab is on) Other servers do not have this problem.
					 *
					 * note that on a row/column choice the pointer could be inside but motion_selection might still
					 * be NOSELECTION
					 */
					if (pevent->xmotion.x < 0 || pevent->xmotion.y < 0 || pevent->xmotion.x > (short)(pg->obj.width) ||
					  pevent->xmotion.y > (short)(pg->obj.height))
						pbge->pointer_inside = FALSE;
					oldsel = old_motion_sel;
					newsel = pbge->selection;
					}
				else if (old_motion_sel == NOSELECTION)
					{
					/*
					 * entering the window
					 */
					pbge->pointer_inside = TRUE;
					oldsel = pbge->selection;
					newsel = pbge->motion_selection;
					}
				else
					{
					/*
					 * crossing buttons inside the window
					 */
					oldsel = old_motion_sel;
					newsel = pbge->motion_selection;
					}
				if (oldsel != newsel)
					{
					redraw_numbered_button(pg, newsel);
					redraw_numbered_button(pg, oldsel);
					}
			}
		}
		break;
	  case ButtonPress:
		if ((pbge->permissive_result = (*pbge->permissive_func)(pg->arg, pg, TRUE)) == FALSE)
			break;
		pbge->button_down = TRUE;
		pbge->motion_selection = what_button(pg, pevent->xbutton.x, pevent->xbutton.y);
		if (pbge->motion_selection == NOSELECTION)
			/*
			 * this fixes a XNEWS server problem
			 */
			break;
	
		if (pbge->selection != pbge->motion_selection)
			{
			redraw_numbered_button(pg, pbge->motion_selection);
			redraw_numbered_button(pg, pbge->selection);
			}
		break;

	  case ButtonRelease:
		if (pbge->permissive_result == FALSE)
			break;
		pbge->button_down = FALSE;
		if (pbge->pointer_inside && pbge->motion_selection != NOSELECTION)
			{
			pbge->selection = pbge->motion_selection;
			if (pg->func)
				(*pg->func)(pg->arg, pg, pbge->selection, pevent);
			}
		break;
	  case Expose:
		for (button_no = 0; button_no < pbge->nchoices; button_no++)
			redraw_numbered_button(pg, button_no);
		break;
	  }
	}


/*
 *	WHAT_BUTTON()
 */
static what_button(pg, x, y)
gizmo	*pg;
int 	x;
int	y;
	{
	int 				button;
	choice_gizmo_ext	*pbge = (choice_gizmo_ext *)(pg+1);
	unsigned			width;
	unsigned			height;

	if (x >= (short)(pg->obj.width) || x < 0)
		return NOSELECTION;
	if (y >= (short)(pg->obj.height) || y < 0)
		return NOSELECTION;

	width = pg->obj.width/pbge->ncol;
	height = pg->obj.height/pbge->nrow;
	switch (pbge->stacking)
	  {
	  case gz_stack_row:
		button = x/width;
		break;
	  case gz_stack_column:
		button = y/height;
		break;
	  case gz_stack_row_column:
		button = x/width + (y/height)*pbge->ncol;
		break;
	  default:
		assert(FALSE);
	  }

	if(button >= (int)(pbge->nchoices) || button < 0)
		return NOSELECTION;

	return button;
	}


/*
 *	REDRAW_NUMBERED_BUTTON
 *
 *
 */
static void redraw_numbered_button(pg, button_no)
gizmo 		*pg;
unsigned	button_no;
	{
	choice_gizmo_ext	*pbge = (choice_gizmo_ext *)(pg+1);
	XRectangle			box;
	unsigned			row;
	unsigned			col;
	int					in;

	if (!pg->mapped)
		return;

	if (button_no == NOSELECTION)
		{
		printf(opiStringTab[STR_bad_button_n                ]);
		return;
		}

	XSetForeground(pg->pgg->d, pg->pgg->bright_GC, pg->bg);
	XSetForeground(pg->pgg->d, pg->pgg->shadow_GC, pg->bg);
	XSetForeground(pg->pgg->d, pg->pgg->normal_GC, pg->bg);
	XSetForeground(pg->pgg->d, pg->pgg->fg_GC, pg->fg);
	XSetForeground(pg->pgg->d, pg->pgg->grey_fg_GC, pg->fg);
	XSetBackground(pg->pgg->d, pg->pgg->grey_fg_GC, pg->bg);
	
	box.width = pg->obj.width/pbge->ncol;
	box.height = pg->obj.height/pbge->nrow;

	/*
	 * performing the multiply/divide each time is slower but we get a more precise button positioning.
	 * Besides, this is integer arithmetic inbetween graphics operations- delays are lost in the noise.
	 * parenthesis enforce order of operations to avoid round off error
	 */
	switch (pbge->stacking)
	  {
	  case gz_stack_row_column:
		row = button_no/pbge->ncol;
		col = button_no%pbge->ncol;
		box.x = (pg->obj.width*col)/pbge->ncol;
		box.y = (pg->obj.height*row)/pbge->nrow;
		break;
	  case gz_stack_row:
		box.x = (pg->obj.width*button_no)/pbge->ncol;
		box.y = 0;
		break;
	  case gz_stack_column:
		box.x = 0;
		box.y = (pg->obj.height*button_no)/pbge->nrow;
		break;
	  default:
		assert(FALSE);
	  }

	if (pbge->button_down && pbge->pointer_inside && pbge->motion_selection != NOSELECTION)
		in = (button_no == pbge->motion_selection);
	else
		in = (button_no == pbge->selection);

	draw_button(pg, &box, in, pbge->pchoices[button_no]);
	}





/*
 *	DRAW_BUTTON
 */
static void draw_button(pg, prec, in, plabel)
gizmo		*pg;
XRectangle	*prec;
int			in;
char		*plabel;
	{
	Display 	*d = pg->pgg->d;
	Window 		w = pg->w;
	XRectangle 	box;
	int			lw;
	int			lwt2;
	int			lwd2;
	GC			ulgc;
	GC			brgc;
	GC			centergc;
	GC			textgc;

	if (in)
		{
		ulgc = pg->pgg->black_GC;
		brgc = pg->pgg->bright_GC;
		centergc = pg->pgg->shadow_GC;
		textgc = pg->pgg->fg_GC;
		}
	else
		{
		ulgc = pg->pgg->bright_GC;
		brgc = pg->pgg->black_GC;
		centergc = pg->pgg->normal_GC;
		textgc = pg->pgg->grey_fg_GC;
		}

#ifdef ILLEGAL_X_USAGE
	lw = max(ulgc->values.line_width, brgc->values.line_width);
#else
	{
	    XGCValues ulvalues;
	    XGCValues brvalues;

	    if (XGetGCValues (d, ulgc, GCLineWidth, &ulvalues)
	      && XGetGCValues (d, brgc, GCLineWidth, &brvalues))
		lw = max(ulvalues.line_width, brvalues.line_width);
	    else
		lw = 1;
	}
#endif ILLEGAL_X_USAGE
	lwt2 = (lw<<1);
	lwd2 = (lw>>1);

	if ((int)(prec->width) < lwt2 || (int)(prec->height) < lwt2 )
		/*
		 * Draw nothing if to small
		 */
		return;

	box = *prec;
	XFillRectangle(d, w, centergc, box.x, box.y, box.width, box.height);

	/*
	 * 3d reasonable only on 4 or more planes
	 */
	if (DefaultDepth(d, DefaultScreen(d)) >= 4)
		{
		/*
		 * filled poly used here to get miter corner with diffrent color lines
		 */
		XPoint		points[7];
		int			inside_width = box.width - lwt2;
		int			inside_height = box.height - lwt2;

		points[0].x = box.x;
		points[0].y = box.y;
		points[1].x = box.width;
		points[1].y = 0;
		points[2].x = -lw;
		points[2].y = lw;
		points[3].x = -inside_width;
		points[3].y = 0;
		points[4].x = 0;
		points[4].y = inside_height;
		points[5].x = -lw;
		points[5].y = lw;
		points[6].x = 0;
		points[6].y = -box.height;
		XFillPolygon(d, w, ulgc, points, NELEMENTS(points), Nonconvex, CoordModePrevious);

		points[0].x = box.x+box.width;
		points[0].y = box.y+box.height;
		points[1].x = -box.width;
		points[1].y = 0;
		points[2].x = lw;
		points[2].y = -lw;
		points[3].x = inside_width;
		points[3].y = 0;
		points[4].x = 0;
		points[4].y = -inside_height;
		points[5].x = lw;
		points[5].y = -lw;
		points[6].x = 0;
		points[6].y = box.height;
		XFillPolygon(d, w, brgc, points, NELEMENTS(points), Nonconvex, CoordModePrevious);
		}
	else
		{	
		XRectangle outline;

		outline = box;
		outline.x += lwd2;
		outline.y += lwd2;
		outline.width -= lw;
		outline.height -= lw;
		XDrawRectangle(d, w, textgc, outline.x, outline.y, outline.width, outline.height);
		}

	box.x += lwd2;
	box.y += lwd2;
	box.width -= lw;
	box.height -= lw;
	draw_string(d, w, textgc, &box, pg->pgg->pft, plabel, strlen(plabel), box_center);
	}



/*
 *	GET_CHOICE_GZ_STRING
 */
char	*get_choice_gz_string(pg, index)
gizmo	*pg;
int	index;
	{
	return ((choice_gizmo_ext *)(pg+1))->pchoices[index];
	}
