/*edd.h,v 1.1.1.1 1999/08/03 14:44:03 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
#ifndef INCeddh
#define INCeddh

#include <stdio.h>
#include <assert.h>

#ifndef _XLIB_H_
#include <X11/Xlib.h>
#endif
#ifndef _XUTIL_H_
#include <X11/Xutil.h>
#endif
#ifndef _XATOM_H_
#include <X11/Xatom.h>
#endif
#ifndef _Xresource_h
#include <X11/Xresource.h>
#endif
#ifndef INClstLibh
#include <lstLib.h>
#endif
#ifndef INCdldefh
#include <dldef.h>
#endif
#ifndef INCfont_tableh
#include <font_table.h>
#endif
#ifndef INCdl_contexth
#include <dl_context.h>
#endif
#ifndef INCstring_tableh
#include <string_table.h>
#endif
#ifndef INCcolor_maph
#include <color_map.h>
#endif
#ifndef INCcolor_rulesh
#include <color_rules.h>
#endif
#ifndef INCgizmoh
#include <gizmo.h>
#endif

/*
 * edd internal varient io stream
 */
struct memory_stream
	{
	char		*pbuf;
	};

enum	edd_stream_type
	{
	use_stdio, use_xresource, use_memory
	};
	
typedef struct
	{
	enum edd_stream_type 		type;
	unsigned					nbytes;
	union{
		void					*pfile;
		XSelectionEvent			xsel;
		struct memory_stream	mem;
	}							io_info;
	} edd_stream;



/*
 * A wrapper for all display list elements hile in edd
 */
typedef struct element_under_edit
	{
	NODE					node;	/* display list node 		*/
	gizmo					*pg;	/* form gizmo attached else NULL*/
	union dl_elements_union	elem;
	struct dl_sta_attr		sta;	/* for graphical elements only	*/
	struct dl_dyn_attr		dyn;	/* for graphical elements only	*/
	char					sel;	/* in the selection list	*/
	} eue;

typedef struct selected_element
	{
	NODE					node;
	eue						eue;
	gizmo					*pg;	/* form gizmo for list mods attached else NULL*/
	} eue_sel;


/*
 *	Routines called between edd modules
 *
 */
char		*getenv();

/* modify.c */
void		modify_element();
void 		modify_edd_defaults();
void 		modify_selected_items();
void 		modify_selected_channel_names();
void 		modify_structure();

/* selections.c */
void 		clear_selections();
void		add_selection();
void 		sel_clear_action();
void 		send_selections();
void		recv_selections();
int 		find_selection();
void 		request_selections();
void 		clear_x_selections();
eue			*insert_clipboard();
void		select_all();

/* element.c */
void 		close_element();
eue 		*create_element();
void 		transform_element();
void 		close_structure();
void 		assign_final_index();
void		open_structure();
int 		string_index_to_display_str();
void 		change_element();

/* redraw_dl.c */
void 		redraw_dl();
void 		redraw_objects();
void 		redraw_selections();
void 		redraw_selection();
void		redraw_line_endpoint_handles();
void		display_selection_status();

/* edd.c */
int			main();
void 		resize_due();
void 		redraw_header();
void 		redraw_msg();
void		init_due();
void		configure_due();	

/* get_coordinate.c */
int 		get_2nd_coord();
int 		get_1st_coord();
void 		add_default_color_map();
void 		add_default_color_rules();

/* menu_actions.c */
void 		gm_quit();	
void 		gm_activate();	
void 		em_create();

/* edd_stream_io.c */
int 		edd_write();

/* modify_color.c */
int			write_cmap();
void 		gm_modify_color_map();
void 		gm_modify_color_rule();
void		edd_create_cmap();
void		edd_delete_cmap();
void		edd_install_cmap();

/* modify_color_rule.c */
int			write_crules();
void 		gm_modify_color_rules();
void		edd_create_crules();
void		edd_delete_crules();
void		edd_install_crules();
void		update_color_rule_menu();

/* select_util.c */
void		selection_bounds();
void 		offset_selections();
void 		scale_selections();
void 		move_selections();
void 		resize_selections();
void 		sink_selections();
void 		float_selections();
void		adjust_line_endpoints();
void		adjust_original_line_endpoints();
void		push_line_endpoints();
void		push_original_line_endpoints();
void 		group_selections();
void 		ungroup_selections();

/* align_selections.c		*/
void 		align_selections();

/* distribute_selections.c	*/
void 		distribute_selections();

/* load.c */
void 		gm_load();	
void 		edd_load_cmap_from_display_file();
int			load_display();

/* save.c */
void 		gm_save();
void		gm_save_to_new_file();
void 		write_pc();
void 		write_string();
void		gm_save_to_text_file();
void 		init_text_file_name();
long		save_display();

/* undo.c */
void		pop_undo_from_menu();
void		push_field_undo();
void		push_delete_undo();
void		push_create_undo();
void		push_front_back_undo();
void		purge_undo_list();
void 		revert_current_sequence();



/*
 *
 *	One of these for each display under edit
 *
 *	NOTE: 
 *		(key to acronyms): 
 *		dl 	<=> 	display list
 *		due	<=>	display under edit
 *		mdt	<=>	modified data tag
 *		XA	<=>	X atom
 *		pXXXX	<=>	pointer to XXXX
 */
typedef struct display_under_edit
	{
	Display 				*d;					/* display ptr		*/
	Screen					*s;					/* screen ptr		*/
	Window 					w;					/* edit window		*/
	struct dl_object_hdr	edd_geom;			/* 			*/
	struct dl_object_hdr	header_geom;		/*			*/
	XColor					foreground;			/*			*/
	XColor					background;			/*			*/
	GC 						back_GC;			/*			*/
	GC 						fore_GC;			/*			*/
	GC 						invert_GC;			/*			*/
	GC						select_GC;			/*			*/
	GC						dyn_GC;				/* for drawing dynamics	*/
	short					pixel_tolerence;	/* what is close enough	*/
	short					gizmo_width;		/* pixels 		*/
	short					gizmo_height;		/* pixels 		*/
	short					gizmo_pad;			/* pixels 		*/
	short					xselect_border;		/* pixels		*/
	short					yselect_border;		/* pixels		*/
	gizmo					*phdrgz;			/* ptr to defaults giz	*/
	struct dl_hdr			dl_hdr;				/* display list header 	*/
	LIST					dl;					/* dl as linked list 	*/
	LIST					undo;				/* undo linked list 	*/
	unsigned				undo_sequence;		/* current edd MACRO op */
	Atom					dl_seg_atom;		/* dl segment dtype	*/
	Atom					XA_CLIPBOARD;		/* CLIPBOARD sel atom	*/
	Atom					XA_DELETE;			/* DELETE target atom	*/
	char					dl_mdt;				/* dl was modified ? 	*/
	void					*clipboard;			/* cut/copy/paste buf	*/
	unsigned				nbytes_cb;			/* n bytes on clipboard */
	LIST					selections;			/* dl items selected	*/
	LIST					channel_list;		/* channel name list for list mods */
	Time					sel_time_stamp;		/* time of last select	*/
	struct dl_context		dlc;				/* current draw mode	*/
	LIST					ct;					/* process var string list	*/
	LIST					st;					/* string list		*/
	eue						**ppselsort;		/* srt tbl fr distr cmd */
	unsigned short			nselsort;			/* nelelem above tbl  	*/
	Cursor					pick_cursor;		/* for get 2nd coord	*/
	Cursor					create_cursor;		/* for get 2nd coord	*/
	unsigned long			*ppixels;			/* safe indir pix ptr 	*/
	unsigned				npixels;			/* safe pix count 	*/
	gizmo					*ploadg;			/* load file  gizmo 	*/
	gizmo					*psavenewg;			/* save new file gizmo 	*/
	gizmo					*psavetextg;		/* save new file gizmo 	*/
	Time					pick_time;			/* time of last pick	*/
	char					pname[32];			/* due file name	*/
	char					ppath[128];			/* path to the due	*/
	char					psnname[32];		/* save new file name	*/
	char					psnpath[128];		/* save new path	*/
	char					pldname[32];		/* load file name	*/
	char					pldpath[128];		/* load path		*/
	char					ptname[36];			/* txt file name	*/
	char					ptpath[128];		/* txt file path	*/
	char					pmsg[256];			/* message to operator	*/
	char					*penv_path;			/* from environ var	*/
	XrmDatabase				xrmdb;				/* X rm db for cmd line	*/
	int						next_groupid;		/* next available group id */
	} due;

enum default_colors
	{
	white,
	black,
	red,				/* the following 6 are the default alarm colors */
	blinking_red,
	yellow,
	blinking_yellow,
	blue,
	blinking_blue,
	number_of_default_colors
	};

#define DEFAULT_DISPLAY_NAME	"test.dl"
#define	TEXT_DL_NAME_EXTENSION	".adl"
#ifdef WIN32
#define TMP_EXTENSION 			"_tmp_edd.dl"
#else
#define TMP_EXTENSION 			".$@#.dl"
#endif WIN32
#define	COLOR_RULE_LISTEDIT_TAG	"edit list"
#define	ELEMENT_FORMAT_STRING	"%-25s x:%4d y:%4d wd:%4d ht:%4d"
#define BELLVOLUME 				0
#define DOUBLE_CLICK_MAX		300 /* milisec */
#define PI						(3.14159)
#define XDEG_PER_REV			(360*64)
#define XDEG_PER_RADIAN			((unsigned)(XDEG_PER_REV/(2*PI)))
#define MM_TOLERENCE 			2
#define XDEGREE_TOLERENCE 		(XDEG_PER_REV/25)
#define	NO_GROUP				0
#ifndef NeXT
#define FORM_GIZSMO_WIDTH_MM 	55
#define FORM_GIZSMO_HEIGHT_MM 	5
#else
#define FORM_GIZSMO_WIDTH_MM 	50
#define FORM_GIZSMO_HEIGHT_MM 	4
#endif
#define FORM_GIZSMO_PAD_MM		1
#define SELECT_BORDER_MM		2
#define CREATE_DEFAULT_SIZE_MM	0

/*
 *	the number of invertable ops saved
 *	(note that an align of MAXUNDO elements would use all of the available entries)
 */
#define MAXUNDO			5000 

#define DL_SEGMENT		"EPICS_BINARY_DISPLAY_LIST_SEGMENT"

typedef unsigned char EDDLOCK;

#define LOCKTEST(PLOCK)  (*(EDDLOCK *)(PLOCK) ? TRUE : FALSE)
#define LOCKAPPLY(PLOCK) (*(EDDLOCK *)(PLOCK) ? FALSE : (*(EDDLOCK *)(PLOCK)=TRUE) )


typedef struct edd_clipboard_header
	{
	unsigned short	majv;
	unsigned short	mnrv;
	short			width;
	short			height;
	unsigned long	magic;
	} clip_hdr;


/*
 * there will be many cases when I have a pointer to an eue pulled from the
 * display list and must convert it to a pointer to the selection block
 * containing the eue. required since I have two linked list headers in each
 * element under edit one for selections and one for the display list
 */
#define PEUE_TO_PSEL(PEUE)	((eue_sel *) (- OFFSET(eue_sel, eue) + (char *) (PEUE))) 


#ifdef EDD_GLOBAL_SOURCE
struct dl_object_hdr 			default_display_geometry =	{0,0,100,100};	/*	Note that these are in mm */
#else
extern struct dl_object_hdr 	default_display_geometry;
#endif

#define DUE_BORDER_WIDTH		1
#define DUE_BORDER_PAD			(DUE_BORDER_WIDTH*2)
#define SELECT_BUTTON			1
#define SELECT_TOGGLE_BUTTON	2
#define MENU_BUTTON				3

#define	posdiff(A,B)		((A)>(B)?(A)-(B):(B)-(A))

/*
 * INDEX_TO_PIXEL is in color_map.h where it can be shared with the runtime
 */
#define CLIPCLRIX(PDUE, INDEX)	INDEX_TO_PIXEL((INDEX), (PDUE)->npixels, (PDUE)->ppixels, (PDUE)->dl_hdr.clr, (PDUE)->d)

/*
 *
 *	ADD_TO_MAXBOUNDS
 *
 */
#define ADD_TO_MAXBOUNDS(MAXREC, REC)\
	{\
	int x2,y2;\
	x2 = max((MAXREC).x+(int)(MAXREC).width, (REC).x+(int)(REC).width);\
	y2 = max((MAXREC).y+(int)(MAXREC).height, (REC).y+(int)(REC).height);\
	(MAXREC).x = min((MAXREC).x, (REC).x);\
	(MAXREC).y = min((MAXREC).y, (REC).y);\
	(MAXREC).width = x2-(MAXREC).x;\
	(MAXREC).height = y2-(MAXREC).y;\
	}


/*
 *	IN_RECTANGLE
 */
#define IN_RECTANGLE(xx, yy, xrec) ((xrec)->x <= (xx) &&\
				    (xx) <= ((xrec)->x+(unsigned int)((xrec)->width)) &&\
				    (xrec)->y <= (yy) &&\
				    (yy) <= ((xrec)->y+(unsigned int)((xrec)->height)))

enum edd_handle
	{
	outside,
	top_left,
	top,
	top_right,
	right,
	bottom_right,
	bottom,
	bottom_left,
	left
	};


/*
 *	MMGO_TO_PIXELGO
 */
#define mmgo_to_pixelgo(D, PGO)\
/*
Display							*D;
register struct dl_object_hdr	*PGO;
*/\
	{\
	register Screen	*s = DefaultScreenOfDisplay(D);\
	(PGO)->x		= ((PGO)->x * WidthOfScreen(s))/WidthMMOfScreen(s);\
	(PGO)->y		= ((PGO)->y * HeightOfScreen(s))/HeightMMOfScreen(s);\
	(PGO)->width	= ((PGO)->width * WidthOfScreen(s))/WidthMMOfScreen(s);\
	(PGO)->height	= ((PGO)->height * HeightOfScreen(s))/HeightMMOfScreen(s);\
	}

/*
 *	PIXELGO_TO_MMGO
 */
#define pixelgo_to_mmgo(D, PGO)\
/*
Display							*D;
register struct dl_object_hdr	*PGO;
*/\
	{\
	register Screen	*s = DefaultScreenOfDisplay(D);\
	(PGO)->x		= ((PGO)->x * WidthMMOfScreen(s)/WidthOfScreen(s));\
	(PGO)->y		= ((PGO)->y * HeightMMOfScreen(s)/HeightOfScreen(s));\
	(PGO)->width	= ((PGO)->width * WidthMMOfScreen(s)/WidthOfScreen(s));\
	(PGO)->height	= ((PGO)->height * HeightMMOfScreen(s)/HeightOfScreen(s));\
	}

/*
 *
 *	MM_TO_PIXEL
 *
 *
 */
#define mm_to_xpixel(MM,S) (((MM)*WidthOfScreen(S))/WidthMMOfScreen(S))
#define mm_to_ypixel(MM,S) (((MM)*HeightOfScreen(S))/HeightMMOfScreen(S))

#endif
