/*gizmo.h,v 1.1.1.1 1999/08/03 14:44:04 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 *
 *	history
 *	.01 102391 joh	added GIZMO_BORDER_WIDTH
 *
 */


#ifndef INCgizmoh
#define INCgizmoh

#include <stdio.h>

#ifndef _XLIB_H_
#include <X11/Xlib.h>
#endif
#ifndef _XUTIL_H_
#include <X11/Xutil.h>
#endif
#ifndef INCfont_tableh
#include <font_table.h>
#endif
#ifndef INClstLibh
#include <lstLib.h>
#undef assert
#endif

#include <assert.h>

#define SELECT_BUTTON 			1
#define SELECT_EXTEND_BUTTON	2
#define MENU_BUTTON 			3
#define BELLVOLUME				0
#define GIZMO_BORDER_WIDTH		1

#if !defined(FALSE) || (FALSE!=0)
#define FALSE				0
#endif  

#if !defined(TRUE) || (TRUE!=1)
#define TRUE				1
#endif  

#if !defined(OK) || (OK!=0)
#define OK					0
#endif  

#if !defined(ERROR) || (ERROR!=(-1))
#define ERROR				(-1)
#endif  

#ifndef NELEMENTS
#define NELEMENTS(array)    (sizeof(array)/sizeof((array)[0]))
#endif

#if !defined(max)
#define max(x, y)			(((x) < (y)) ? (y) : (x))
#endif

#if !defined(min)
#define min(x, y)			(((x) < (y)) ? (x) : (y))
#endif

/*
 * performs alignment for RISC- assuming no 8 byte quantity (doubles)
 * in gizmo structures
 */
#ifndef AXP
#define ALIGN(A) (((A)|3) + 1)
#else
/* quad align for Alpha */
#define ALIGN(A) (((A)|7) + 1)
#endif

enum gizmo_types
	{
	gz_none,
	gz_valuator,
	gz_textedit,
	gz_menu,
	gz_form,
	gz_palette,
	gz_popup,
	gz_choice, 
	gz_message,
	gz_last
	};

/*
 * how much to pad gizmos which are children of root from the screen border (5 mm)
 */
#define VERT_GIZMO_ROOT_PAD(S)	((5*HeightOfScreen(S))/HeightMMOfScreen(S))
#define HORIZ_GIZMO_ROOT_PAD(S)	((5*WidthOfScreen(S))/WidthMMOfScreen(S))

enum choice_gizmo_stacking 
	{
	gz_stack_row,
	gz_stack_column,
	gz_stack_row_column
	};
	
enum valuator_gizmo_pt_dir 
	{
	gz_point_down,
	gz_point_up,
	gz_point_right,
	gz_point_left
	};

/*
 * array of functions returning void
 * 
 * run for each change to a gizmo
 */
void 	valuator_action();
void 	textedit_action();
void 	menu_action();
void 	form_action();
void 	palette_action();
void 	popup_action();
void 	choice_action();
void 	message_action();
#ifdef GIZMOGLOBALDEF
void 	(*gizmo_actions[gz_last])() =
	{
	(void (*)())abort,
	valuator_action, 
	textedit_action, 
	menu_action,
	form_action,
	palette_action,
	popup_action,
	choice_action,
	message_action,
	};
#else
extern void 	(*gizmo_actions[gz_last])();
#endif

/*
 * array of functions returning void
 * 
 * run only once at startup for each gizmo type
 */
void 	valuator_init();
void 	textedit_init();
void 	menu_init();
void 	form_init();
void 	palette_init();
void 	popup_init();
void 	choice_init();
void 	message_init();
#ifdef GIZMOGLOBALDEF
void 	(*gizmo_init_func[gz_last])() =
	{
	(void (*)())abort,
	valuator_init, 
	textedit_init, 
	menu_init,
	form_init,
	palette_init,
	popup_init,
	choice_init,
	message_init,
	};
#else
extern void 	(*gizmo_init_func[gz_last])();
#endif

/*
 * array of functions returning void
 * 
 * run each time a gizmo deletes if present
 */
void 	valuator_free();
void 	textedit_free();
void 	menu_free();
void 	form_free();
void 	palette_free();
void 	popup_free();
void 	choice_free();
void 	message_free();
#ifdef GIZMOGLOBALDEF
void 	(*gizmo_free_func[gz_last])() =
	{
	(void (*)())abort,
	valuator_free, 
	textedit_free, 
	menu_free,
	form_free,
	palette_free,
	popup_free,
	choice_free,
	message_free
	};
#else
extern void 	(*gizmo_free_func[gz_last])();
#endif


/*
 * array of functions returning void
 * 
 * run each time a gizmo changes color
 */
void 	form_gizmo_color_change();
void 	popup_gizmo_color_change();
#ifdef GIZMOGLOBALDEF
void 	(*gizmo_color_change_func[gz_last])() =
	{
	(void (*)())abort,
	0, 
	0, 
	0,
	form_gizmo_color_change,
	0,
	popup_gizmo_color_change,
	0,
	0,
	};
#else
extern void 	(*gizmo_color_change_func[gz_last])();
#endif


/*
 * stuff that applies to all gizmos on a display
 */
typedef struct
	{
	Window		seln_owner;	/* sel owner or None */
	Atom		seln_atom;	/* sel atom or None */
	} gizmo_display_global;

/*
 * stuff that applies to all gizmos on a screen
 */
typedef struct
	{
	LIST		free_gizmo_list;
	Display		*d;
	Screen		*s;

	/* for normal drawing */
	GC			fg_GC;				/* only fg,bg,font changes */
	GC			grey_fg_GC;			/* only fg,bg,font changes*/

	/* for 3d button shading */
	GC			normal_GC;			/* only fg,font changes */
	GC			bright_GC;			/* only fg,font changes */
	GC			shadow_GC;			/* only fg,font changes */
	GC			black_GC; 			/* only font changes */
	GC			white_GC;			/* only font changes */
	GC			invert_GC;			/* only font changes */

	long		dialog_bg;			/* constant dialog background */

	font_table 	*pft;

	Window		mother;				/* parent of lost gizmos */
	char		sprintf_buf[256];	/* common sprintf buffer */
	int			nbytes_cb;			/* bytes on clipboard */
	void		*pclipboard;		/* ptr to clipboard */
	Atom		XA_WM_TAKE_FOCUS;
	Atom		XA_CLIPBOARD;
	Atom		XA_DELETE;
	Atom		XA_LENGTH;
	Cursor		cursor;
	} gizmo_global;

typedef struct
	{
	NODE			node;
	unsigned short	size;		/* n bytes alloc */
	unsigned short	type;		/* the type of this gizmo */
	Window			w;			/* this gizmos's window */
	Window			tw;			/* this gizmos's transparent window */
	Window			parent;		/* this gizmos's parent window */
	XRectangle		obj;		/* x,y,ht,wd (real estate param) */
	gizmo_global	*pgg;		/* ptr to info common to all gizmos */
	void			(*func)();	/* pointer to application action func */
	void			*arg;		/* argument for application func */
	void			*pae;		/* pointer to application extension */
	unsigned long	fg;			/* foreground color (pixels are long!) */
	unsigned long	bg;			/* background color (pixels are long!) */
	char			mapped;		/* boolean for mapped/unmapped state */
	char			free;		/* on the free gz list */
	short			RISC_pad2;
	} gizmo;

int		ok_to_operate();
int		not_ok_to_operate();
gizmo 	*create_gizmo();
void	free_gizmo();
void	move_gizmo();
void	move_resize_gizmo();
int		filter_gizmo_events();
void	set_gizmo_color();

int 	set_valuator_gizmo(); 				/* returns OK/ERROR */
void 	valuator_action();
gizmo 	*create_valuator_gizmo();
int 	set_valuator_gizmo_precision(); 	/* returns OK/ERROR */
int 	set_valuator_gizmo_scale_enable(); /* returns OK/ ERROR */

gizmo 	*create_textedit_gizmo();
int		set_textedit_gizmo();
char 	*textedit_gizmo_return_pbuf();

gizmo 	*create_menu_gizmo();
void 	set_menu_gizmo();

gizmo 	*create_form_gizmo();
void 	form_gizmo_add_color();
void 	form_gizmo_add_gizmo();
void 	form_gizmo_add_string();
void 	form_gizmo_item_geom();
void 	*form_gizmo_return_pitem();
void 	resize_form_gizmo();

gizmo 	*create_palette_gizmo();
void 	set_palette_gizmo();

gizmo 	*create_popup_gizmo();
gizmo 	*popup_gizmo_return_pgizmo();
gizmo 	*popup_gizmo_return_pmenugizmo();
void 	popup_gizmo_add_gizmo();
gizmo	*create_file_name_entry_gizmo(); 

gizmo	*create_choice_gizmo();
int		set_choice_gizmo();
int		choice_gizmo_value();

gizmo	*create_message_gizmo();

#ifdef GIZMOGLOBALDEF
XContext		gizmo_ctx;
XContext		gizmo_parent_ctx;
XContext		gizmo_display_ctx;
#else
extern XContext	gizmo_ctx;
extern XContext	gizmo_parent_ctx;
extern XContext	gizmo_display_ctx;
#endif


#endif /* ifndef INCgizmoh */
