/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
#ifndef INC_pfoPlotDefs_h
#define INC_pfoPlotDefs_h
/*	@(#)pfoPlotDefs.h	1.2 7/18/91
 *	Author:	Roger A. Cole
 *	Date:	03-04-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01	03-04-91	rac	initial version
 * .02	mm-dd-yy	rac	version 2.0, installed in SCCS
 *
 * make options
 *	For user programs, there are no interesting options.  For private
 *	options relevent only to pfoPlot, see pfoPlot.c .
 */
/*+/mod***********************************************************************
* TITLE	pfoPlotDefs.h - plotting definitions for "plotting for OPI" routines
*
* DESCRIPTION
*
*-***************************************************************************/

typedef enum {
    PFO_ATTR_FG1,
    PFO_ATTR_FG2,
} PFO_ATTR;

/*/subhead PFO_CHAN------------------------------------------------------------
* PFO_CHAN
*----------------------------------------------------------------------------*/
#if !defined PFO_PRIVATE && !defined PFO_CHAN_DEF
  typedef struct pfoChan PFO_CHAN;
#else
  typedef struct pfoChan {
    struct gsd_sync_data *pGsd;	/* pointer to sync data buf */
    SYD_CHAN	*pSchan;	/* pointer to sync data channel struct */
    union db_access_val *pDbrGr;/* pointer to graphics info */
    chtype	dbrGrType;	/* type of graphics info buffer */
  } PFO_CHAN;
#endif

/*/subhead PFO_AXIS------------------------------------------------------------
* PFO_AXIS
*----------------------------------------------------------------------------*/
#ifndef PFO_PRIVATE
  typedef struct pfoAxis PFO_AXIS;
#else
  typedef struct pfoAxis {
    struct pfoAxis *pNext;	/* pointer to next axis structure */
    struct pfoAxis *pPrev;	/* pointer to previous axis structure */
    struct pfoWin *pPfo;	/* pointer to pfo win struct */
    PFO_CHAN	pfoChan;	/* pfo channel struct for data */
    SYD_PL_SLAVE *pSlave;	/* pointer to sydPlot slave struct */
    short	isAnX;		/* 1 if this is an x axis */
  } PFO_AXIS;
#endif

/*/subhead PFO_WIN-------------------------------------------------------------
* PFO_WIN
*----------------------------------------------------------------------------*/
#ifndef PFO_PRIVATE
  typedef struct pfoAxis PFO_WIN;
#else
  typedef struct pfoWin {
    SYD_SPEC	*pSspec;	/* pointer to sync data spec struct */
    SYD_PL_MSTR	*pMstr;		/* pointer to master plot struct */
    SYD_PL_MSTR	master;		/* master plot struct */
    GC		gc;		/* gc to use in drawing */
    unsigned long fg;		/* foreground pixel value */
    unsigned long bg;		/* background pixel value */
    Display	*pDisp;		/* X display */
    Window	window;		/* X window */
    short	nSamples;	/* dimension of circular buffers */
    PFO_AXIS	*pXAxisHead;	/* ptr to first axis struct for horiz axis */
    PFO_AXIS	*pXAxisTail;	/* ptr to last axis struct for horiz axis */
    PFO_AXIS	*pYAxisHead;	/* ptr to first axis struct for vert axis */
    PFO_AXIS	*pYAxisTail;	/* ptr to last axis struct for vert axis */
    short	binNum;		/* bin number, for plotting scalars */
    char	title[120];	/* top title for plot */
    char	yTitle[120];	/* left title for plot */
    char	xTitle[120];	/* bottom title for plot */
    enum dl_plot_style style;	/* plot style */
    int		axis;		/* axis type */
  } PFO_WIN;
#endif

/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
void pfoAxisAutoRange();
static void pfoAxisClose();
PFO_AXIS *pfoAxisOpen();
void pfoWinAutoRange();
void pfoWinClose();
PFO_WIN *pfoWinOpen();
void pfoWinPlotSample();
void pfoWinRepaint();
void pfoWinRestart();
void pfoWinSetAttr();
void pfoWinWritePSFile();

#endif INC_pfoPlotDefs_h
