/*************************************************************************************************
*    Name :     schedule.c
*
*    Purpose:   Determine the pin of the person scheduled to be paged. 
*
*    K.GRIMES  4/1994
**************************************************************************************************/ 
#include  <stdio.h>
#include  <db_access.h>
#include  <dbDefs.h>
#include  <time.h>
#include  <pagerdefs.h>
#include  <string.h>
#include  <sys/types.h>
#include  <sys/time.h>
#include   <alarmlist.h>

#define   NUM_DAYS_IN_WEEK       7

C4CODE    cb;

/*********************************************************************************************/
int   get_active_shift(shiftname, error_message)
char    shiftname[MAX_SHIFT+1];
char    error_message[MAX_ERROR+1];

/*  find the active shift name for the current time */
{

  int       day_found = FALSE, hour, minute, day, prev_rec, shift_found = FALSE, target, 
	    shift_available = TRUE, i, error_set = FALSE;
  D4DATA    *shift;
  T4TAG     *shift_tag;
  F4FIELD   *shift_fld, *day_fld, *hour_fld, *min_fld;
  time_t    epoch;
  char      fileToOpen[1000];

  struct tm *current = (struct tm *)malloc(sizeof(struct tm));

  translate_db_link("shift.dbf", fileToOpen);

  if (! (shift = d4open(&cb, fileToOpen)) ) {
     sprintf(error_message,"CAN NOT PAGE!     The shift table does not exist.\n");
     error_set = TRUE;
     return shift_found;
  }

  shift_tag = d4tag(shift, "DAYTIME");
  d4tag_select(shift, shift_tag);
  
  if (d4reccount(shift) > 0) {

      /* make sure that all deleted records have been removed from table. */

      d4pack(shift);

      /* Find the last shift of the week */

      d4bottom(shift);
      prev_rec = d4recno(shift);

      /* Find the first shift of the week */

      d4top(shift);

      shift_fld = d4field(shift, "SHIFTNAME");
      day_fld = d4field(shift, "WEEKDAY");
      hour_fld = d4field(shift, "STARTHOUR");
      min_fld = d4field(shift, "STARTMIN");

      /* Get the current day of the week / time */

      epoch = time(NULL);
      current = localtime(&epoch);
      target = current->tm_wday; 

      /* find the current days shift schedule, this is the starting point for locating the 
	 active shift.  If there is no data for the current day get the previous days schedule. */

      for (i = 1; !day_found && i <= NUM_DAYS_IN_WEEK; ++i){

         while ( !day_found && !d4eof(shift) ){

	   /* Find the shift for the current day in the table */

	   day = f4int(day_fld); 
           if ( day == target ) 
	       day_found = TRUE;
           else {
	       prev_rec = d4recno(shift);
	       d4skip(shift,1);
           }
         } 
	 if (!day_found) {

	     /* There is no entry for the current day set the day to the most recent day */

	     target = target - 1;
	     if (target < 0)
		 target = 6;
             d4top(shift);
         } 
      }

      if (i == NUM_DAYS_IN_WEEK) {

	  shift_available = FALSE;
	  sprintf(error_message, "CAN NOT PAGE!     The days in the shift table are invalid, day of the week must be between Sunday = 0 and Saturday = 6.\n");
	  error_set = TRUE;
      }
 
      /* find the current shift with respect to time for the current day.  If there is no data
	 for the current day the last shift for the most recent day is considered to be the
	 current shift. */

     while (!shift_found && shift_available) {

      if (day == current->tm_wday ) {

	   /* find the appropriate shift for the current day. */
	   hour = f4int(hour_fld);
	   minute = f4int(min_fld);

           if (hour <= 23 && hour >= 0 && minute <= 59 && minute >=0) {
      
              if (hour < current->tm_hour) {

		 /* The current record is a candidate for the current shift get the next record to 
		    determine if it is also a candidate. */

	         prev_rec = d4recno(shift);
		 d4skip(shift, 1);
	         
		 if ( d4eof(shift)) 
		    d4top(shift);
                 
		 if (day != f4int(d4field(shift, "WEEKDAY")) ) {
		    d4go(shift, prev_rec); 
		    sprintf(shiftname,"%s", f4str(shift_fld));
		    shift_found = TRUE;
                 }
              }
              else if (hour == current->tm_hour)  {
	       
		 /* The current record is a candidate for the current shift determine if the 
		    minute field satisfies the current time. */

                if ( minute <= current->tm_min) {
             	    sprintf(shiftname, "%s",f4str(shift_fld));
	            shift_found = TRUE;
	         }
	         else {
		    d4go(shift, prev_rec); 
		    sprintf(shiftname, "%s",f4str(shift_fld));
		    shift_found = TRUE;
                 }
              }
              else { /* hour > current->tm_hour */ 

		 /* This record starts after the current time the previous record is the current
		    shift. */

	         d4go(shift, prev_rec);
	         shift_found = TRUE;
	         sprintf(shiftname, "%s", f4str(shift_fld));
              }
           }  
	   else {
	      /* if an ivalid time is encountered in the database, look at the next record.  if the
		 end of file or a new day is reached the last valid shift will be notified. */

	      d4skip(shift, 1);
	      if ( d4eof(shift) || day != f4int(d4field(shift, "WEEKDAY")) ) {
		 d4go(shift, prev_rec);
		 sprintf(shiftname,"%s", f4str(shift_fld));
		 shift_found = TRUE;
              }
           }
       }	       
       else {
	   /* the last shift of the most recent day found above is the current shift */

           while ( f4int(d4field(shift,"WEEKDAY")) == day && !d4eof(shift) ) {
	      prev_rec = d4recno(shift);
     	      d4skip(shift, 1);
           }
           d4go(shift, prev_rec);
	   shift_found = TRUE;
	   sprintf(shiftname,"%s", f4str(shift_fld));
       }
     }
  }
  else {
     sprintf(error_message,"CAN NOT PAGE!     The shift table is empty.\n");
     error_set = TRUE;
  }

  free(current);
  d4close(shift);

  if (!shift_found && !error_set)
      sprintf(error_message, "CAN NOT PAGE!     There is no shift scheduled to page.\n");

  return shift_found;
}

/*******************************************************************************************/
int    get_current_staff(area, pin, error_message)

int    area;
char   pin[MAX_PIN+1];
char   error_message[MAX_ERROR+1];

{
int      found = FALSE, needs_packing = FALSE, error_set = 0;
char     activeshift[MAX_SHIFT+1], fileToOpen[1000], thearea[4];
D4DATA   *schedule;
T4TAG    *schedule_tag;

      /*  Using the area get the operator name from the schedule table and use this 
          to find the actual pin number in the directory table. */ 
   
      d4init(&cb);

      cb.open_error = 0;
      cb.go_error = 0;

      if ( get_active_shift(activeshift, error_message)) {

	  translate_db_link("schedule.dbf", fileToOpen);

	  if (! (schedule = d4open(&cb, fileToOpen)) ) {
	      sprintf(error_message,"CAN NOT PAGE!     The schedule table does not exist.\n");  
	      return found;
          }

	  schedule_tag = d4tag(schedule, "AREA");
	  d4tag_select(schedule, schedule_tag);
	  d4top(schedule);

	  if (d4reccount(schedule) > 0) {

	     sprintf(thearea, "%3d", area);

	     if ( d4seek(schedule, thearea) == 0) {

	        while ( f4int(d4field(schedule, "AREA")) == area
		        && !found && !d4eof(schedule) ){ 
	          if (d4deleted(schedule))
	              needs_packing = TRUE;
                  else if (strcmp( activeshift, f4str(d4field(schedule, "SHIFT"))) == 0 ){
                      sprintf(pin, "%s", f4str(d4field(schedule, "PIN")));
		      found = TRUE;

		      /* only digits should be contained in the pin */

		      for (; *pin != '\0'; ++pin)
			 if (!isdigit(*pin))  {
                            error_set = 1;
			    found = FALSE;
                      }
		      if (error_set)
			     sprintf(error_message, "CAN NOT PAGE!     The PIN is not valid for %s shift, area %d\n", activeshift, area);
                  } 
                  d4skip(schedule,1);
                  }

		  if (!error_set)
		      sprintf(error_message,"CAN NOT PAGE!     There is no one scheduled during the %s shift to be\n     paged for area %d\n", activeshift, area);
	     }
	     else
		sprintf(error_message,"CAN NOT PAGE!     There is no one scheduled to be paged for area %d, during the %s shift.\n", area, activeshift);
         }
	 else
	 sprintf(error_message,"CAN NOT PAGE!     The schedule table is empty. \n");

        if (needs_packing)
    	    d4pack(schedule);
	 
         d4close(schedule);
     }
     return found;

}
