/*
**	Get time from CMOS clock chip on pc486 platforms
**
*/

#include <vxWorks.h>
#include <intLib.h>
#include <time.h>
#include <dosFsLib.h>
#include <stdio.h>
#include <symLib.h>
#include <timers.h>
#include <taskLib.h>
#include <logLib.h>

#if ((CPU == I80386) || (CPU == I80486))
#include <..\config\pc486\pc.h>
#endif

#include "tsDefs.h"
#include "GetPCTime.h"

extern SYMTAB_ID sysSymTbl;
long	(*GPSTime) ();	/* allow late bind to this symbol */

static void SetFileTime (DOS_DATE_TIME *pDateTime);

int PCTimeDebug = 0;

/* system timer variables */
extern TS_STAMP	time_stamp;	/* defined in time/ts_driv.c */
extern long	tsMinWest;	/* defined in libCom/tsSubr.c */

long GetPCTime (int MinWest)
{
#if ((CPU == I80386) || (CPU == I80486))
    long retTime;
    struct timespec posixTime;
    int	key;
    struct tm tm_block;
    unsigned char BCDval;
    struct tm *new_tm;
    SYM_TYPE symtype;

    /* If we have a GPS image loaded, take time from GPS */
    if (!GPSTime) /* look up symbol if loaded */
	symFindByName (sysSymTbl, "_GetGPSTime", (char **)&GPSTime, &symtype);
    /* if GPStime is available use it to update the system time stamp */
    if (!GPSTime || !GPSTime (&time_stamp))
    {
	int noTime = 1;
	while (noTime)
	{
	    /* use time from CMOS */
	    do /* wait until we won't see a transition in the subsequent reads */
	    {
		key = intLock ();

		sysOutByte (RTC_INDEX, 0x00);
		tm_block.tm_sec = sysInByte (RTC_DATA);

		intUnlock (key);
	    } while (tm_block.tm_sec == 0x59 /* Note: BCD */);

	    key = intLock ();

	    sysOutByte (RTC_INDEX, 0x00);
	    BCDval = sysInByte (RTC_DATA);
	    if ((BCDval > 0x99) || ((BCDval & 0x0f) > 0x9))
	    {
		intUnlock (key);
		logMsg ("Bad BCD seconds value %x\n", BCDval,0,0,0,0,0);
		continue;
	    }
	    tm_block.tm_sec = (BCDval / 16) * 10 + (BCDval & 0x0f);
	    sysOutByte (RTC_INDEX, 0x02);
	    BCDval = sysInByte (RTC_DATA);
	    if ((BCDval > 0x99) || ((BCDval & 0x0f) > 0x9))
	    {
		intUnlock (key);
		logMsg ("Bad BCD minutes value %x\n", BCDval,0,0,0,0,0);
		continue;
	    }
	    tm_block.tm_min = (BCDval / 16) * 10 + (BCDval & 0x0f);
	    sysOutByte (RTC_INDEX, 0x04);
	    BCDval = sysInByte (RTC_DATA);
	    if ((BCDval > 0x99) || ((BCDval & 0x0f) > 0x9))
	    {
		intUnlock (key);
		logMsg ("Bad BCD hours value %x\n", BCDval,0,0,0,0,0);
		continue;
	    }
	    tm_block.tm_hour = (BCDval / 16) * 10 + (BCDval & 0x0f);
	    sysOutByte (RTC_INDEX, 0x07);
	    BCDval = sysInByte (RTC_DATA);
	    if ((BCDval > 0x99) || ((BCDval & 0x0f) > 0x9))
	    {
		intUnlock (key);
		logMsg ("Bad BCD day of mon value %x\n", BCDval,0,0,0,0,0);
		continue;
	    }
	    tm_block.tm_mday = (BCDval / 16) * 10 + (BCDval & 0x0f);
	    sysOutByte (RTC_INDEX, 0x08);
	    BCDval = sysInByte (RTC_DATA);
	    if ((BCDval > 0x99) || ((BCDval & 0x0f) > 0x9))
	    {
		intUnlock (key);
		logMsg ("Bad BCD month value %x\n", BCDval,0,0,0,0,0);
		continue;
	    }
	    tm_block.tm_mon = ((BCDval / 16) * 10 + (BCDval & 0x0f)) - 1;
	    sysOutByte (RTC_INDEX, 0x09);
	    BCDval = sysInByte (RTC_DATA);
	    if ((BCDval > 0x99) || ((BCDval & 0x0f) > 0x9))
	    {
		intUnlock (key);
		logMsg ("Bad BCD year value %x\n", BCDval,0,0,0,0,0);
		continue;
	    }
	    tm_block.tm_year = (BCDval / 16) * 10 + (BCDval & 0x0f);
	    if (tm_block.tm_year < 80)
		tm_block.tm_year += 100;

	    intUnlock (key);
	    noTime = 0;
	}

	/* one last try for a GPS time, if not, install new time */
	if (!GPSTime || !GPSTime (&time_stamp))
	{
	    /* convert to a UTC time value (RTC registers are in BCD) */
	    tm_block.tm_isdst = -1;

	    retTime = mktime (&tm_block) + MinWest * 60;

	    /* set the system time stamp */
	    time_stamp.secPastEpoch = retTime - TS_EPOCH_SEC_PAST_1970;
	    time_stamp.nsec = 0;

	    /*
		Call to clock_settime added. This makes the vxWorks time functions
		behave much better.
	    */
	    posixTime.tv_sec = retTime;
	    posixTime.tv_nsec = 0;
	    clock_settime (CLOCK_REALTIME, &posixTime);
	}
    }

    tsMinWest = MinWest;

    if (PCTimeDebug)
    {
	/* Time from GPS/RTC clock value */
	new_tm = localtime (&retTime);
 	printf ("RTC time is %d:%2.2d:%2.2d %d/%d/%d\n",
		 new_tm->tm_hour, new_tm->tm_min, new_tm->tm_sec,
		 new_tm->tm_mon+1, new_tm->tm_mday, new_tm->tm_year+1900);
    }

    /* install time update for DOS system */
    dosFsDateTimeInstall ((FUNCPTR)SetFileTime);

    return retTime; 
#else
    return 0;
#endif
}

/* set PC time from epics time_stamp */
void SetPCTime (void)
{
#if ((CPU == I80386) || (CPU == I80486))
    long dosTime;
    int	key;
    unsigned char BCDval;
    struct tm *new_tm;
    struct timespec posixTime;

    /* wait until the start of a second */
    while (time_stamp.nsec > 250000000)
	taskDelay (1);

    /*
     Call to clock_settime added. This makes the vxWorks time functions
     behave much better.
    */
    posixTime.tv_sec = time_stamp.secPastEpoch + TS_EPOCH_SEC_PAST_1970;
    posixTime.tv_nsec = 0;
    clock_settime (CLOCK_REALTIME, &posixTime);

    dosTime = time_stamp.secPastEpoch + TS_EPOCH_SEC_PAST_1970 - tsMinWest * 60;
    new_tm = localtime (&dosTime);

    key = intLock ();

    BCDval = ((new_tm->tm_sec / 10) * 16) + (new_tm->tm_sec % 10);
    sysOutByte (RTC_INDEX, 0x00);
    sysOutByte (RTC_DATA, BCDval);
    BCDval = ((new_tm->tm_min / 10) * 16) + (new_tm->tm_min % 10);
    sysOutByte (RTC_INDEX, 0x02);
    sysOutByte (RTC_DATA, BCDval);
    BCDval = ((new_tm->tm_hour / 10) * 16) + (new_tm->tm_hour % 10);
    sysOutByte (RTC_INDEX, 0x04);
    sysOutByte (RTC_DATA, BCDval);
    BCDval = ((new_tm->tm_mday / 10) * 16) + (new_tm->tm_mday % 10);
    sysOutByte (RTC_INDEX, 0x07);
    sysOutByte (RTC_DATA, BCDval);
    new_tm->tm_mon++;
    BCDval = ((new_tm->tm_mon / 10) * 16) + (new_tm->tm_mon % 10);
    sysOutByte (RTC_INDEX, 0x08);
    sysOutByte (RTC_DATA, BCDval);
    if (new_tm->tm_year > 100)
	new_tm->tm_year -= 100;
    BCDval = ((new_tm->tm_year / 10) * 16) + (new_tm->tm_year % 10);
    sysOutByte (RTC_INDEX, 0x09);
    sysOutByte (RTC_DATA, BCDval);

    intUnlock (key);
#endif
}

static void SetFileTime (DOS_DATE_TIME *pDateTime)
{
    struct tm *new_tm;
    long dosTime;

    /* adjust, dos file times are in local units */
    dosTime = time_stamp.secPastEpoch + TS_EPOCH_SEC_PAST_1970 - tsMinWest * 60;

    new_tm = localtime (&dosTime);
    pDateTime->dosdt_year = new_tm->tm_year+1900;	/* current year */
    pDateTime->dosdt_month = new_tm->tm_mon+1;		/* current month */
    pDateTime->dosdt_day = new_tm->tm_mday;		/* current day */
    pDateTime->dosdt_hour = new_tm->tm_hour;		/* current hour */
    pDateTime->dosdt_minute = new_tm->tm_min;		/* current minute */
    pDateTime->dosdt_second = new_tm->tm_sec;		/* current second */

}


