/*mv167.c,v 1.1.1.1 1999/08/03 14:44:12 tis4000 Exp */
/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/****************************************************************/
/*								*/
/*	    PLATFORM SPECIFIC STARTUP FOR MOTOROLA MV167	*/
/*								*/
/****************************************************************/

/*  File: mv167.c                                                                       
**  Edit:  11/28/94     GMU: created                                                    
**                         :                                                            
* .01	01/03/95	WEP	modified for integration with common 
*				watchdog logic        
* .02	06-28-95	ems	added conditional compilation
*                         :                                                            
*/
/* task definitions */

#include <taskLib.h>
#include <vxWorks.h>
#include <logLib.h>

#include "platform.h"

#if (CPU_FAMILY == MC680X0)

/* prototypes: original routines */
int mv167DisableWatchDog(void);

/* prototypes: new routines      */
int mv167EnableWatchDog(int rate);
void mv167PetWatchDog(void);


/*****************************************
**      spawn the watchdog task         **
*****************************************/

int start_mv167(int rate)
{
    int	rateMask, delay;
/*
    See page 2-62 of the MV167 SBC Programmer's Reference
*/
    if (rate == 32)
	{
	rateMask = 0x600;	/* Rate = 32 ms. */
	delay = 1;		/* delay 16 ms between strobes */
	}
    else if (rate == 64)
	{
	rateMask = 0x700;	/* Rate = 64 ms. */
	delay = 2;		/* Delay 32 ms between strobes */
	}
    else if (rate == 128)
	{
	rateMask = 0x800;	/* Rate = 128 ms. */
	delay = 4;		/* Delay 64 ms between strobes */
	}
    else if (rate == 256)
	{
	rateMask = 0x900;	/* Rate = 256 ms. */
	delay = 8;		/* Delay 128 ms between strobes */
	}
    else if (rate == 512)
	{
	rateMask = 0xA00;	/* Rate = 512 ms. */
	delay = 16;		/* Delay 256 ms between strobes */
	}
    else if (rate == 1000)
	{
	rateMask = 0xB00;	/* Rate = 1000 ms. */
	delay = 30;		/* Delay 500 ms between strobes */
	}
    else if (rate == 4000)
	{
	rateMask = 0xC00;	/* Rate = 4000 ms. */
	delay = 120;		/* Delay 2000 ms between strobes */
	}
    else if (rate == 16000)
	{
	rateMask = 0xD00;	/* Rate = 16000 ms. */
	delay = 480;		/* Delay 8000 ms between strobes */
	}
    else if (rate == 32000)
	{
	rateMask = 0xE00;	/* Rate = 32000 ms. */
	delay = 960;		/* Delay 16000 ms between strobes */
	}
    else if (rate == 64000)
	{
	rateMask = 0xF00;	/* Rate = 64000 ms. */
	delay = 1920;		/* Delay 32000 ms between strobes */
	}
    else
	{
	logMsg("MV167 Watchdog rate of %d is not supported.\n",0,0,0,0,0,0);
	logMsg("Avail rates are 32, 64, 128, 256, 512, 1000, 4000, 16000, 32000, and 64000 ms\n",
		0,0,0,0,0,0);
	return (ERROR);
	}

    /* initialize the VxWorks watchdog */
/*    tisWatchDog ((int(*)(int))mv167EnableWatchDog,
                 (int(*))mv167DisableWatchDog,
                 (void(*))mv167PetWatchDog,
                 rateMask,
                 delay);*/
    tisWatchDog (mv167EnableWatchDog,
                 mv167DisableWatchDog,
                 mv167PetWatchDog,
                 rateMask,
                 delay);
    return (OK);
}
/*********************************************
**      Enable the 167 WatchDog Hardware    **
*********************************************/

int mv167EnableWatchDog(int rateMask)
{
    int lint;

    lint = *(int *) 0xfff4004c; /* Grab control register        */
    lint &= 0xfff0ff;           /* Wipe wdt rate select bits    */
    lint |= rateMask;           /* Select desired timeout       */
    *(int *) 0xfff4004c = lint; /* Put back control register    */
	
    return (OK);
}



/*********************************************
**     Disable the 167 WatchDog Hardware    **
*********************************************/

int mv167DisableWatchDog()
{
    int gmuDog;

    gmuDog = *(int *) 0xfff40060;    /* Grab control register                           */
    gmuDog &= 0xff00ffff;            /* Wipe all wdt bits                               */
    *(int *) 0xfff40060 = gmuDog;    /* Put back control register                       */
    return(OK);
}



/*********************************************
**       Pet the 167 Watch Dog Hardware     **
*********************************************/

void mv167PetWatchDog()
{
    int gmuDog;
/*
    See page 2-67 of the MV167 SBC Programmer's Reference
*/
    gmuDog = *(int *) 0xfff40060;    /* Grab control register                           */
    gmuDog &= 0xff00ffff;	     /* Wipe all wdt bits                               */
    gmuDog |= 0x006f0000;	     /* Set bits that we want:                          */
                                     /*   0x400000 = Clear WDTO bit                     */
                                     /*   0x200000 = Reset WDT counter                  */
                                     /*   0x080000 = Assert BRDFAIL (light FAIL led)    */
                                     /*   0x040000 = SYSRESET on VME bus                */
                                     /*   0x020000 = cause SYSRESET on timeout          */
                                     /*   0x010000 = Enable WDT                         */
    *(int *) 0xfff40060 = gmuDog;    /* Put back control register                       */
}


/*	End of File:	mv167.c */

#endif
