/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*	    PLATFORM SPECIFIC STARTUP FOR INTEL 80486		*/
/*								*/
/****************************************************************/

/*  File: x486.c
  .01	mm-dd-yy	xxx	comment
*/

#include <vxWorks.h>
#include <logLib.h>
#include <sysLib.h>

#include "platform.h"

#if (CPU_FAMILY == I80X86)

/* FPGA I/O */
#define	FPGA_BASE    0x50
#define	FPGA_WDSTROBEPORT  (FPGA_BASE+0x0B)
#define	FPGA_INDEXPORT  (FPGA_BASE+0x0D)
#define FPGA_REGPORT	(FPGA_BASE+0x0F)

/* FPGA registers */
#define	FPGA_WDTREGISTER 3

/* Watchdog control definitions */
#define	WDT_CLEAR   0x80 /* clear WD signal (strobe) */
#define	WDT_DISABLE 0x00 /* mode values */
#define WDT_REBOOT  0x18
#define WDT_17_8MS  0x00 /* strobeRate timeouts */
#define WDT_71_1MS  0x01
#define WDT_284MS   0x02
#define WDT_1_14S   0x03
#define WDT_4_55S   0x04
#define WDT_18_22S  0x05
#define WDT_72_8S   0x06
#define WDT_291S    0x07

int pv5000EnableWatchDog (int wdTimeout);
int pv5000DisableWatchDog (void);
void pv5000PetWatchDog (void);

int start_x486 (int priority, int rate)
{
    int	wdTimeout, strobeRate;
/*
    See the pv5000 SBC Programmer's Reference
    (Note, we can't strobe fast enough to support 17.1 msec)
*/
    if (rate == 71)
	{
	wdTimeout = WDT_71_1MS;	/* Rate = 71.1 ms. */
	strobeRate = 2;		/* strobeRate 32 ms between strobes */
	}
    else if (rate == 284)
	{
	wdTimeout = WDT_284MS;	/* Rate = 284 ms. */
	strobeRate = 8;		/* strobeRate 128 ms between strobes */
	}
    else if (rate == 1140)
	{
	wdTimeout = WDT_1_14S;	/* Rate = 1.14 s. */
	strobeRate = 30;		/* strobeRate 500 ms between strobes */
	}
    else if (rate == 4550)
	{
	wdTimeout = WDT_4_55S;	/* Rate = 4.55 s. */
	strobeRate = 135;		/* strobeRate 2250 ms between strobes */
	}
    else if (rate == 18220)
	{
	wdTimeout = WDT_18_22S;	/* Rate = 18.2 s. */
	strobeRate = 540;		/* strobeRate 9000 ms between strobes */
	}
    else if (rate == 72800)
	{
	wdTimeout = WDT_72_8S;	/* Rate = 72.8 s. */
	strobeRate = 2160;		/* strobeRate 36000 ms between strobes */
	}
    else if (rate == 291000)
	{
	wdTimeout = WDT_291S;	/* Rate = 291 s. */
	strobeRate = 8700;		/* strobeRate 145000 ms between strobes */
	}
    else
	{
	logMsg("pv5000 Watchdog rate of %d is not supported.\n",0,0,0,0,0,0);
	logMsg("Avail rates are 18, 71, 284, 1140, 4550, 18220, 72800, 291000 ms\n",
		0,0,0,0,0,0);
	return (ERROR);
	}

    /* initialize the VxWorks watchdog */
    tisWatchDog (pv5000EnableWatchDog,
                 pv5000DisableWatchDog,
                 pv5000PetWatchDog,
                 wdTimeout,
                 strobeRate);
    return (OK);
}
/*********************************************
**      Enable the pv5000 WatchDog Hardware    **
*********************************************/

int pv5000EnableWatchDog (int wdTimeout)
{
    /* clear old WD state */
    sysOutByte (FPGA_INDEXPORT, FPGA_WDTREGISTER);
    sysOutByte (FPGA_REGPORT, WDT_CLEAR+WDT_DISABLE);
    /* enable WD at desired rate */
    sysOutByte (FPGA_INDEXPORT, FPGA_WDTREGISTER);
    sysOutByte (FPGA_REGPORT, WDT_REBOOT+wdTimeout);
    return (OK);
}



/*********************************************
**     Disable the pv5000 WatchDog Hardware    **
*********************************************/

int pv5000DisableWatchDog (void)
{
    sysOutByte (FPGA_INDEXPORT, FPGA_WDTREGISTER);
    sysOutByte (FPGA_REGPORT, WDT_CLEAR+WDT_DISABLE);
    return(OK);
}



/*********************************************
**       Pet the pv5000 Watch Dog Hardware     **
*********************************************/

void pv5000PetWatchDog (void)
{
    sysOutByte (FPGA_WDSTROBEPORT, 0);
}


/*	End of File:	x486.c */
#endif
