/*ProdInv.h,v 1.1.1.1 1999/08/03 14:44:13 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 * widgets
 */
#define PROD_WINDOW 			1
#define PROD_COLUMNS1 			2
#define PROD_COLUMNS2 			3
#define PROD_LIST 			4
#define TANK_BUTTON 			5
#define PIPE_BUTTON 			6
#define EXIT_BUTTON 			7
#define DETAIL_WINDOW 			8
#define DETAIL_COLUMNS1 		9
#define DETAIL_COLUMNS2 		10
#define DETAIL_LIST 			11
#define DETAIL_TOTALS 			12
#define DETAIL_EXIT 			13
#define DETAIL_EDIT 			14
#define DETAIL_DIALOG 			15
#define DIALOG_OK 			16
#define DIALOG_CANCEL 			17
#define PRINT_INV 			18
#define PRINT_DET 			19
#define SAVE_BUTTON 			20
#define REFRESH_BUTTON 			21



/*
 * miscellaneous
 */
#define	MAX_ARGS			20
#define	CLICK				1
#define	RECORD_BLOCK_SIZE		1000
#define	min(x,y)			(((x)<(y))?(x):(y))

#define AUTO				0	/* enum value of "Automatic" mode string */
#define NO				0	/* enum value of "No" GblChoice */
#define YES				1	/* enum value of "Yes" GblChoice */
#define	INCHES				0
#define	FEET				1
#define	METERS				2

#define	OTHER_LIST			1	/* Current list being detailed is pipe */
#define TANK_LIST			2	/* Current list being detailed is tank */


/*
 * data structure sizes
 */
#define	ON_OFF_ENTRY_SIZE		5
#define	PRODUCT_ENTRY_SIZE		16
#define	CAPACITY_ENTRY_SIZE		10
#define	GROSS_VOLUME_ENTRY_SIZE		4
#define	BSW_VOLUME_ENTRY_SIZE		10
#define	ULLAGE_ENTRY_SIZE		8
#define	TEMP_ENTRY_SIZE			5
#define	GRAVITY_ENTRY_SIZE		5
#define	FACTOR_ENTRY_SIZE		5
#define	NON_ISSUE_ENTRY_SIZE		8
#define ISSUABLE_ENTRY_SIZE		10
#define NET_VOLUME_ENTRY_SIZE		10
#define POINTNAME_SIZE			28
#define DESCRIPTOR_SIZE			28
#define APIDATE_SIZE			12


typedef struct PipeProdStruct
{
	XmString		pipeStr;
	char			pointName[POINTNAME_SIZE+1];
	char			pointDescr[DESCRIPTOR_SIZE+1];
	char			onOff[ON_OFF_ENTRY_SIZE+1];
	double			totalBBLS;
	double			ullage;
	double			temp;
	double			apiGravity;
	double			factor;
	double			netVolume;
	struct PipeProdStruct	*NextPipe;
} PipeDetails;

typedef struct TankProdStruct
{
	XmString		tankStr;
	char			pointName[POINTNAME_SIZE+1];
	char			pointDescr[DESCRIPTOR_SIZE+1];
	char			onOff[ON_OFF_ENTRY_SIZE+1];
	long			levelFt;
	long			levelIn;
	double			level8ths;
	double			totalBBLS;
	long			BSWInches;
	double			BSW8ths;
	double			ullage;
	double			temp;
	double			apiGravity;
	char			apiDate[APIDATE_SIZE];
	double			factor;
	double			netVolume;
	struct TankProdStruct	*NextTank;
} TankDetails;

typedef struct ProdStruct
{
	XmString		prodStr;
	char			product[PRODUCT_ENTRY_SIZE+1];
	char			onOff[ON_OFF_ENTRY_SIZE+1];
	double			capacity;
	double			grossVolume;
	double			bswVolume;
	double			ullage;
	double			nonIssuable;
	double			issuable;
	double			netVolume;
	int			numPipes;
	int			numTanks;
	PipeDetails		*pipeList;
	TankDetails		*tankList;
	struct ProdStruct	*NextProd;
} ProdRecord;

struct  {
	int		numProds;
	ProdRecord	*prodList;
} ProdInv;

typedef struct	chanInfo	{
	char			chanName[POINTNAME_SIZE+6];	/* Allow for 4-byte field name */
	chid			ChanId;
	struct 	chanInfo	*nextChan;
}  ChanDetails;

#define	CHAN_DET_SIZE		sizeof(ChanDetails)


static struct	{
	char	UnitsString[7];		/* Allow for null */
	double	CnvFactor;		/* Factor required to convert to barrels */
} CnvTable[] =	{
	{"brls",	1.0},
	{"gals",	0.023809523},
	{"liters",	(0.2642 * 0.023809523)},	/* gals then brls */
	{"f3",		(7.48052 * 0.023809523)},	/* gals then brls */
	{"m3",		(264.2 * 0.023809523)},		/* gals then brls */
	{0,	0}
};


/*
 *	CodeBase definitions
 */
F4FIELD_INFO	ProdInvFlds[] =	{

	/* Field name,		Type,	Width,			Decimals */
	{ "PRODUCT",		'C',	PRODUCT_ENTRY_SIZE,	    0},
	{ "DATE",		'C',	10,			    0},
	{ "TIME",		'C',	8,			    0},
	{ "ON_OFF",		'C',	ON_OFF_ENTRY_SIZE,	    0},
	{ "CAPACITY",		'N',	8,			    1},
	{ "GROSS_VOL",		'N',	8,			    1},
	{ "BSW_VOLUME",		'N',	8,			    1},
	{ "ULLAGE",		'N',	8,			    1},
	{ "NOT_ISSUE",		'N',	8,			    1},
	{ "ISSUABLE",		'N',	8,			    1},
	{ "NETVOLUME",		'N',	10,			    3},
	{ "NUM_OTHER",		'N',	6,			    0},
	{ "NUM_TANKS",		'N',	6,			    0},
	{0, 0, 0, 0}		/* NULL-record terminator */
};


T4TAG_INFO	ProdInvTag[] =	{
	/* Tag Name,	Key Expression,		Filter Expression,	Unique,	Descending */
	{ "PRODUCT",	"PRODUCT",			0,		  0,	    0	},
	{ "PROD_DATE",	"PRODUCT+DATE+TIME",		0,		  0,	    0	},
	{0, 0, 0, 0, 0 }	/* NULL-record terminator */
};

F4FIELD_INFO	OtherInvFlds[] =	{

	/* Field name,		Type,	Width,			Decimals */
	{ "PRODUCT",		'C',	PRODUCT_ENTRY_SIZE,	    0},
	{ "POINTNAME",		'C',	POINTNAME_SIZE,		    0},
	{ "POINTDESC",		'C',	DESCRIPTOR_SIZE,	    0},
	{ "DATE",		'C',	10,			    0},
	{ "TIME",		'C',	8,			    0},
	{ "ON_OFF",		'C',	ON_OFF_ENTRY_SIZE,	    0},
	{ "TOTAL_BBLS",		'N',	8,			    1},
	{ "ULLAGE",		'N',	8,			    1},
	{ "TEMP",		'N',	8,			    1},
	{ "APIGRAVITY",		'N',	11,			    3},
	{ "FACTOR",		'N',	10,			    4},
	{ "NETVOLUME",		'N',	10,			    3},
	{0, 0, 0, 0}		/* NULL-record terminator */
};


T4TAG_INFO	OtherInvTag[] =	{
	/* Tag Name,	Key Expression,		Filter Expression,	Unique,	Descending */
	{ "PRODUCT",	"PRODUCT",			0,		  0,	    0	},
	{ "POINTNAME",	"POINTNAME",			0,		  0,	    0	},
	{ "POINT_DATE",	"POINTNAME+DATE+TIME",		0,		  0,	    0	},
	{ "PROD_PT_DT",	"PRODUCT+POINTNAME+DATE+TIME",	0,		  0,	    0	},
	{0, 0, 0, 0, 0 }	/* NULL-record terminator */
};

F4FIELD_INFO	TankInvFlds[] =	{

	/* Field name,		Type,	Width,			Decimals */
	{ "PRODUCT",		'C',	PRODUCT_ENTRY_SIZE,	    0},
	{ "POINTNAME",		'C',	POINTNAME_SIZE,		    0},
	{ "POINTDESC",		'C',	DESCRIPTOR_SIZE,	    0},
	{ "DATE",		'C',	10,			    0},
	{ "TIME",		'C',	8,			    0},
	{ "ON_OFF",		'C',	ON_OFF_ENTRY_SIZE,	    0},
	{ "LEVEL_FT",		'N',	6,			    0},
	{ "LEVEL_IN",		'N',	2,			    0},
	{ "LEVEL_8THS",		'N',	3,			    1},
	{ "TOTAL_BBLS",		'N',	8,			    1},
	{ "BSW_INCHES",		'N',	6,			    0},
	{ "BSW_8THS",		'N',	3,			    1},
	{ "ULLAGE",		'N',	8,			    1},
	{ "TEMP",		'N',	8,			    1},
	{ "APIGRAVITY",		'N',	11,			    3},
	{ "API_DATE",		'C',	APIDATE_SIZE,		    0},
	{ "FACTOR",		'N',	10,			    4},
	{ "NETVOLUME",		'N',	10,			    3},
	{0, 0, 0, 0}		/* NULL-record terminator */
};


T4TAG_INFO	TankInvTag[] =	{
	/* Tag Name,	Key Expression,		Filter Expression,	Unique,	Descending */
	{ "PRODUCT",	"PRODUCT",			0,		  0,	    0	},
	{ "POINTNAME",	"POINTNAME",			0,		  0,	    0	},
	{ "POINT_DATE",	"POINTNAME+DATE+TIME",		0,		  0,	    0	},
	{ "PROD_PT_DT",	"PRODUCT+POINTNAME+DATE+TIME",	0,		  0,	    0	},
	{0, 0, 0, 0, 0 }	/* NULL-record terminator */
};


