/*recAi.c,v 1.1.1.1 1999/08/03 14:44:18 tis4000 Exp */

/* recAi.c - Record Support Routines for Analog Input records */
/*
 *      Original Author: Bob Dalesio
 *      Current Author:  Marty Kraimer
 *      Date:            7-14-89
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 *
 * Modification Log:
 * -----------------
 * .01	8-12-88		lrd	Put in breakpoint conversions
 *				Fix conversions for bipolar xy566 cards
 * .02	12-12-88	lrd	Lock record on entry and unlock on exit
 * .03	12-15-88	lrd	Process the forward scan link
 * .04	12-23-88	lrd	Alarm on locked MAX_LOCKED times
 * .05	01-13-89	lrd	deleted db_read_ai
 * .06	01-18-89	lrd	modify adjustment offset to be independent of
 *				adjustment slope
 *				clamp xy566 breakpoint table negative values to
 *				zero
 * .07	03-27-89	lrd	make hardware errors MAJOR alarms
 *				remove hardware severity from the database
 * .08	04-03-89	lrd	add monitor code
 *				removed signal units conversion stuff
 * .09	04-05-89	lrd	modified so that negative adjustment slope ASLO
 *				would work
 * .10	05-03-89	lrd	removed process mask from arg list
 * .11	08-01-89	lrd	only set value for constant when val != 0
 * .12	10-11-89	lrd	fix smoothing initial condition
 * .13	01-31-90	lrd	add AB plc flag to the ab_aidriver call
 * .14	03-21-90	lrd	add db_post_event for RVAL
 * .15	04-11-90	lrd	make locals static
 * .16  04-18-90	mrk	extensible record and device support
 * .17  03-12-93	ems	deleted ack from redundancy
 * .17  11-23-93	ems	added cvnTypes.h
 *                              use conversion type as defined in cvnTypes.h
 * .18	12-01-93	ems	added slope_and_offset() to calc eslo and roff
 *				do here instead of in devAi***.c, also do here
 *				for SPC_LINCONV
 *				modified LINEAR conversion formula
 *				added math.h for sqrt()
 *				added square root conversion
 * .19	01-03-94	ems	ack = 0 in init_record, in process if not init'd
 *				do nothing 
 * .20  01-05-94	ems	added SPC_CNST in special func
 * .21  03-01-94	ems	added support for process stats
 * .22  03-09-94	ems	added scan_type.h to define E_IO_INTERRUPT 
 * .23  03-10-94	ems	alarm chgs, if val is past alarm setting, set that
 *				alarm condition, else if val was in that alarm and
 *				it is still in hyst range of alarm value, then stay
 *				in alarm, remove return from high alarm, let user
 *				determine which is more important.
 * .24	03-15-94	ems	call convert from init_record to process val, 
 *				call monitor in init to post VAL
 * .25	03-29-94	ems	make forward func def agree with actual func def
 * .26  04-04-94	ems	init roci to 0, redund cant handle -1, rework bumping of
 *				roci in alarm, make sure does not exceed MAX_ROC_VALEUS
 * .27	04-20-94	ems	major chg to way block inits, added ini2 as a secondary
 *				init flag to pass to slave when master inits, the
 *				combination of init and ini2 flags allows the slave to
 *				fully init so it can take when fail over occurs
 * .28  04-25-94	ems	get_control_double, if field is HYST, set low to 0 and 
 *				high to hopr-lopr
 * .29  05-09-94	ems	don't apply smoothing algorithm unless we are 
 *				fully init'd
 * .30	06-10-94	ems	move call to tsLocalTime inside if on scan
 * .31	06-13-94	ems	added support for maximize severity msev
 * .32	07-08-94	ems	get_control_double, if field is HOPR or LOPR, set low to
 *				EGUL and high to EGUF
 * .33	07-20-94	ems	monitor(), set sevr = MAJOR if SBAD not ==, typo
 * .34  08-05-94	ems	added many field types to get_graphic_double and
 *				get_control_double
 * .35	08-16-94	ems	monitor(), if nsta==SBAD set nsev to MAJOR, fix problem
 *				where block would go into SBAD and continue to fire 
 *				alarm monitors even if stayed in SBAD
 * .36	08-17-94	ems	rm DBE_LOG from ack, only archive val and update alst if
 *				tenb (trending) is enabled, only set DBE_VALUE for rval
 * .37	10-12-94	ems	add TIME to redundancy, fpas chg'd to frps, added fpps
 *				(1st process pass), do time stamp when stat or sevr chg,
 *				see monitor(),and not in process(),set lalm to val when 
 *				stat or sevr chg,in process() if fpps (first process)
 *				init lalm to val and if time stamp is 0, init it also
 * .38	10-20-94	ems	chg time to latm (last alarm time) added 
 *				added lptm (last process time)
 * .39	01-13-95	ems	add mlst to redundancy, init_record(), init lptm and latm
 * .40	03-15-95	ems	monitor(), do not set ACK if sevr is EVENT_ALARM
 * .41	09-11-95	ems	monitor(), archive when stat/sevr chg's also
 * .42	09-12-95	ems	monitor(), see mod for handling VALID_ALARM
 * .43	10-09-95	ems	convert(), can not take square root of a negative number
 *				causes NaN value, generate a CALC alarm
 * .44	11-08-95        wep     init_record() was modified to handle a returned status 
 *				value of 3. response is valid, but no conversion is done, 
 *				since the dev level code already converted and stored 
 *				value in VAL.
 * .45  11-09-95        wep     (implemented in support of the Netpac driver)
 *                              monitor(), convert OPENTC_ALARM to MAJOR_ALARM 
 *				severity (nsev) 
 * .46	02-06-96	ems	added code for deviation alarm
 *				always update rate of chg buffer as soon as alarms are 
 *				    enabled so will always have latest data, also check 
 *				    if rate of chg value is > 0.0 before process roc 
 *				    alarm
 *				remove return from HIHI and LOLO alarms, so will always 
 *				    check all alarm sevrs
 * .47	04-02-96	ems	added code to support alarm delay
 * .48	06-17-96	ems	added GAIN
 * .49	06-19-96	ems	added code for alarm disable ADIS
 * .50	07-02-96	ems	alarm(), update rocb if roc > 0.0 or frps is set
*/

#include	<vxWorks.h>
#include	<inetLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<string.h>
#include	<lstLib.h>
#include	<memLib.h>
#include	<math.h>
#include	<stdlib.h>

#include	"alarm.h"
#include	"cvtTable.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/aiRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"airedfldidx.h"
#include	"restable.h"
#include	"cvnTypes.h"	/* defines for conversion types */
#include	"scan_types.h"	/* define for E_IO_INTERRUPT */
#include	"error.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/

#define report NULL
#define initialize NULL
static long init_record();
static long process();
static long special();
static long get_value();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
static long get_units();
static long get_precision();
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
static long get_graphic_double();
static long get_control_double();
static long get_alarm_double();

struct rset aiRSET = {

	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
	};

struct aidset {				/* analog input dset */
	long		number;
	DEVSUPFUN	dev_report;
	DEVSUPFUN	init;
	DEVSUPFUN	init_record;	/*returns: (-1,0)=>(failure,success)*/
	DEVSUPFUN	get_ioint_info;
	DEVSUPFUN	read_ai;	/*(0,1,2) => success and */
			/*(convert,don't continue, don't convert)*/
			/* if convert then raw value stored in rval */
	DEVSUPFUN	special_linconv;
	};

/* The folowing definitions are for redundancy */
 
static struct fldDes	**mylist = 0;

static	char *namelist [] = {
		"STAT",
		"SEVR",
		"VAL",
		"ROCI",
		"ROCC",
		"LALM",
		"INI2",
		"LATM",
		"MLST",
		"HHLL",
		"HILO",
		"RTCT",
		"LARF",
		"DVCT",
		"SBCT",
		0
		};

static	unsigned	short	type;

/*Following from timing system		*/

extern unsigned int     gts_trigger_counter;

static void alarm ();
static void convert ();
static void monitor ();
static void slope_and_offset ();


/*------------------------------------------------------------------------*/

static long init_record (pai)

    struct aiRecord	*pai;

{
    struct	aidset	*pdset;
    long		status;
    unsigned	short	i;
    char		*pstr;
    char		redbuf [256];
    int			ini2_flag = 0;
    static int		do_convert;

	if (!(mylist)) 
	{
	    for (type = 1; type < 100; type++) 
	    {
		pstr = GET_PRECTYPE (type);
		if (strcmp (pstr, "ai") == 0) 
		    break;
	    }
	    mylist = redFldAlloc (type, NUMAIFLDS);

	    if (!mylist)
	    {
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pai,
				   "ai, init_record, mylist failed");
		return (S_rec_Alloc);
	    }

	    for (i = 0; i < NUMAIFLDS; i++) 
		redFldAdd (type, i, namelist [i]);
	}

	if (!(pdset = (struct aidset *) (pai->dset))) 
	{
	    recGblRecordError(S_dev_noDSET, (struct dbCommon *)pai, "ai: init_record");
	    return (S_dev_noDSET);
	}

	/* must have read_ai function defined */

	if ( (pdset->number < 6) || (pdset->read_ai == NULL) ) 
	{
	    recGblRecordError (S_dev_missingSup, (struct dbCommon *)pai, "ai: init_record");
	    return (S_dev_missingSup);
	}

	/* Allocate rate of change value buffer */

	if (!(pai->rocb)) 
	{
	    pai->rocb = (char *) calloc (1, (MAX_ROC_VALUES * sizeof (float)));
	    if (!pai -> rocb)
	    {
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pai,
				   "ai, init_record, rocb failed");
		return (S_rec_Alloc);
	    }
	    pai->rocc = 0;
	    pai->roci = 0;
	}

	/*must find breakpoint table*/

	if ((pai->linr >= START_BRKPT) && (pai -> linr <= MAXBRKPTTBLS )) 
	{
	    if (!cvtTable || (cvtTable->number < pai->linr) ||  
	       (!(cvtTable->papBrkTable[pai->linr])) ) 
	    {
		errMessage (S_db_badField, "Breakpoint Table not Found");
		return (S_db_badField);
	    }
	    pai->pbrk = (char *) (cvtTable->papBrkTable [pai->linr]);
	    pai->lbrk = 0;
        }

	if (pai -> gain > 999)
	{
	    errMessage (S_db_badField, "GAIN range 0-999");
	    return (S_db_badField);
	}

        pai -> init = 1;
	pai -> frps = 1;
	pai -> fpps = 1;
	pai -> ack  = 0;

	/* init alarm delay variables */
	pai -> larf = 0;
	pai -> hhll = 0;
	pai -> hilo = 0;
	pai -> dvct = 0;
	pai -> sbct = 0;
	pai -> rtct = 0;

	/* init last process time and last alarm time */
	if (pai -> lptm.secPastEpoch == 0)
	    tsLocalTime (&pai -> lptm);
	if (pai -> latm.secPastEpoch == 0)
	    tsLocalTime (&pai -> latm);

	/* init so do not carry to process when init succeeds, if set by previous
	   failure of init
	*/
	pai -> nsta = NO_ALARM;
	pai -> nsev = NO_ALARM;
	pai -> msev = NO_ALARM;


	do_convert = 1;

	/* do not allow conversion if the board type is CONSTANT and input
	   is CONSTANT or REFERENCE, this was done in devSoft.c
	*/

        if (pai -> dtyp <= 1)
	{
	    switch (pai -> inp.type)
	    {
		case (CONSTANT):
		case (PV_LINK):
		case (DB_LINK):
		case (CA_LINK):
		    if (pai -> linr != NOCONV)
			pai -> linr = NOCONV;

		    /* make sure for this situation we do not do any conversion
		       when we return from device init
		    */
		    do_convert = 0;
		    break;
	    }
        }

	/* calculate conversion slope eslo and offset roff 
	   do before pdset->init_record in case there is a dev that
	   has to recalc eslo and roff for some reason
	*/

	slope_and_offset (pai);

	if (pdset->init_record) 
	{
	    status = (*pdset->init_record) (pai, process);
	    if ((status) && (status != 3))
	    {
		/* if we are tagged, off scan or in manual, then we will
		   allow the point to init
		*/

	        if ( (!pai -> tagd) && (pai -> oscn != OFF_SCAN) &&
		     (pai -> mode == NORMAL) )
		{
		    pai -> nsta = NO_INIT_ALARM;
		    pai -> nsev = VALID_ALARM;
		}
		else
		    ini2_flag = 1;
	    }
	    else
	    {
		ini2_flag = 1;

                if (status == 3)
                {
                    do_convert = 0;
                }

		if ( do_convert && ((pai -> linr <= MAXBRKPTTBLS) ||
				   (pai -> linr == CNVSQRROOT)) )
		    convert (pai);
	    }
	}
	else
        {
	    ini2_flag = 1;
        }

	/* if we are MASTER post VAL and send it to redundant partner */

	if (resGetCaAnswer (pai -> radr, &i) == RES_OK)
	{
	    if (i)
	    {
		/* post STAT, SEVR and VAL (monitor), if we will finally init, 
		   because we have read a value from the input device, set INI2,
		   send all redund fields to slave, when slave receives INI2, 
		   it will be fully init'd also
		*/

	    	/* check for maximize severity */

	    	if (pai -> msev > pai -> nsev)
	    	{
		    pai -> nsev = pai -> msev;
		    pai -> nsta = LINK_ALARM;
	    	}

		monitor (pai);

		if (ini2_flag)
		{
		    pai -> ini2 = 1;
		    redQInit ((struct dbCommon *)pai, redbuf, sizeof (redbuf), type);

		    redQField (redbuf, mylist [STAT]);
		    redQField (redbuf, mylist [SEVR]);
		    redQField (redbuf, mylist [VAL]);
		    redQField (redbuf, mylist [ROCI]);
		    redQField (redbuf, mylist [ROCC]);
		    redQField (redbuf, mylist [INI2]);

		    redQSend (redbuf);
		}
	    }
	}

        return (0);
}

/*------------------------------------------------------------------------*/

static long process (paddr)
    struct dbAddr	*paddr;
{
    struct aiRecord	*pai   = (struct aiRecord *) (paddr->precord);
    struct aidset	*pdset = (struct aidset *) (pai->dset);
    long		status = 0;
    unsigned short	stat   = pai -> stat;
    unsigned short	sevr   = pai -> sevr;
    double		val    = pai -> val;
    short		roci   = pai -> roci;
    short		rocc   = pai -> rocc;
    double		lalm   = pai -> lalm;
    double		mlst   = pai -> mlst;
    TS_STAMP		latm   = pai -> latm;
    short		hhll   = pai -> hhll;
    short		hilo   = pai -> hilo;
    short		rtct   = pai -> rtct;
    unsigned long	larf   = pai -> larf;
    short		dvct   = pai -> dvct;
    short		sbct   = pai -> sbct;
    short		val_sent = 0;
    char		redbuf [256];

    /* if not init'd, do nothing */

    if ( (!pai -> init) || (!pai -> ini2) )
	return (status);

    if ((pdset == NULL) || (pdset->read_ai == NULL)) 
    {
	pai->pact = TRUE;
	recGblRecordError (S_dev_missingSup, (struct dbCommon *)pai,"read_ai");
	return (S_dev_missingSup);
    }

    /* event throttling */

    if (pai->scan == E_IO_INTERRUPT)
    {
	if ((pai->evnt != 0)  && (gts_trigger_counter != 0)) 
	{
	    if ((gts_trigger_counter % pai->evnt) != 0) 
		return (status);
	}
    }

    /* get time stamp for last time record was processed */

    tsLocalTime (&pai -> lptm);

    pai->msev = NO_ALARM;

    /* off normal operations (use VAL and use RVAL) */

    if (pai -> tagd)
    {
	pai->nsta = TAGGED_ALARM;
	pai->nsev = VALID_ALARM;
    }
    else if (pai->oscn == OFF_SCAN)
    {
	pai->nsta = SCAN_ALARM;
	pai->nsev = VALID_ALARM;	
    }
    else
    {
	pai -> nsta = NO_ALARM;
	pai -> nsev = NO_ALARM;

	/* update number of process times */

	if (process_stats)
	    process_cnts[type]++;	

	if (pai -> mode == NORMAL)
	{
	    status = (*pdset->read_ai) (pai); /* read the new value */

	    /* Driver returns -2 if it does not own the resource */

	    if (status == -2)
		return (0);
	        
	    /* status is one if an asynchronous record is being processed*/

	    if (status == 1)
		return (0);
	        
	    if ( (status == 0) && 
		 ((pai->linr <= MAXBRKPTTBLS) || (pai -> linr == CNVSQRROOT)) )
	    {
		convert (pai);
	    }

	    /* set by read_ai, conversion already done */

	    else if (status == 2)
		status = 0;

	    /* pact must not be set true until read_ai is called */
	}

	pai->pact = TRUE;

	/* check for alarms */

	alarm (pai);

	/* check for maximize severity */

	if (pai -> msev > pai -> nsev)
	{
	    pai -> nsev = pai -> msev;
	    pai -> nsta = LINK_ALARM;
	}
    }

    /* if the time stamp is 0, init and post val with
       alarm bit set, this makes sure that proper alarm
       messages are set if IOC was down and alarm status
       may have chg'd
    */
    if (pai -> latm.secPastEpoch == 0)
    {
	tsLocalTime (&pai -> latm);
	db_post_events ((struct dbCommon *)pai, (caddr_t)&pai -> val, DBE_VALUE | DBE_ALARM);
    }

    /* if it is the 1st process pass 
       init last value when alarm chg'd (lalm)
    */

    if (pai -> fpps)
    {
	pai -> fpps = 0;
	pai -> lalm = pai -> val;
    }

    /* check event list */

    monitor (pai);

    pai->pact = FALSE;

    redQInit ((struct dbCommon *)pai, redbuf, sizeof (redbuf), type);
    if (stat != pai->stat) 
	redQField (redbuf, mylist [STAT]);
	
    if (sevr != pai->sevr) 
	redQField (redbuf, mylist [SEVR]);
	
    if (val != pai->val) 
    {
	val_sent = 1;
	redQField (redbuf, mylist [VAL]);
    }
	
    if (roci != pai->roci) 
    {
	if (!val_sent)
	    redQField (redbuf, mylist [VAL]);
	redQField (redbuf, mylist [ROCI]);
    }

    if (rocc != pai->rocc) 
	redQField (redbuf, mylist [ROCC]);
	
    if (lalm != pai->lalm) 
	redQField (redbuf, mylist [LALM]);

    if (TsCmpStampsNE (&latm, &pai -> latm) )
	redQField (redbuf, mylist [LATM]);

    if (mlst != pai -> mlst)
	redQField (redbuf, mylist [MLST]);

    if (hhll != pai -> hhll)
	redQField (redbuf, mylist [HHLL]);

    if (hilo != pai -> hilo)
	redQField (redbuf, mylist [HILO]);

    if (rtct != pai -> rtct)
	redQField (redbuf, mylist [RTCT]);

    if (larf != pai -> larf)
	redQField (redbuf, mylist [LARF]);

    if (dvct != pai -> dvct)
	redQField (redbuf, mylist [DVCT]);

    if (sbct != pai -> sbct)
	redQField (redbuf, mylist [SBCT]);
	
    redQSend (redbuf);

    /* process the forward scan link record */

    if (pai->flnk.type == DB_LINK) 
	dbScanPassive ((struct dbAddr *)pai->flnk.value.db_link.pdbAddr);
	
    return (status);
}

/*------------------------------------------------------------------------*/

static long special (paddr,after)

    struct	dbAddr	*paddr;
    int			after;
{
    struct aiRecord  	*pai = (struct aiRecord *)(paddr->precord);
    int           	special_type = paddr->special;
    static double       save_cnst;

    /* save cnst, if inp ect not CONSTANT, dont let them 
       update CNST
    */

    if (!after)
    {
       save_cnst = pai -> cnst;
       return (0);
    }

    switch (special_type) 
    {

	case (SPC_LINCONV):
	    slope_and_offset (pai);
	    return (0);

	case (SPC_CNST):
	    if ((pai -> dtyp <= 1) && (pai -> inp.type == CONSTANT))
            {
	        pai -> inp.value.value = pai -> cnst;
                pai -> val = pai -> cnst;
            }
	    else
	        pai -> cnst = save_cnst;
	    return (0);

	default:
	recGblDbaddrError (S_db_badChoice, paddr, "ai: special");
	return (S_db_badChoice);
    }
}

/*------------------------------------------------------------------------*/

static long get_value(pai,pvdes)

    struct	aiRecord	*pai;
    struct	valueDes	*pvdes;
{
	pvdes->field_type = DBF_DOUBLE;
	pvdes->no_elements = 1;
	(double *) (pvdes->pvalue) = &pai->val;
	return (0);
}

/*------------------------------------------------------------------------*/

static long get_units(paddr,units)

    struct dbAddr *paddr;
    char	  *units;
{
	struct aiRecord    *pai = (struct aiRecord *) paddr->precord;

	strncpy (units, pai->egu, sizeof (pai->egu));
	return (0);
}

/*------------------------------------------------------------------------*/

static long get_precision (paddr, precision)

    struct	dbAddr	*paddr;
    long		*precision;
{
	struct aiRecord    *pai = (struct aiRecord *) paddr->precord;

	*precision = pai->prec;
	return (0);
}

/*------------------------------------------------------------------------*/

static long get_alarm_double (paddr, pad)

    struct	dbAddr		*paddr;
    struct	dbr_alDouble	*pad;
{
	struct aiRecord    *pai = (struct aiRecord *) paddr->precord;

	pad->upper_alarm_limit = pai->hihi;
	pad->upper_warning_limit = pai->high;
	pad->lower_warning_limit = pai->low;
	pad->lower_alarm_limit = pai->lolo;
	return (0);
}


/*------------------------------------------------------------------------*/

static long get_graphic_double(paddr,pgd)

    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    struct aiRecord    *pai = (struct aiRecord *) paddr->precord;
    float    span;


    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct aiRecord *)0) -> irhi) ):
	case ( (int)&(((struct aiRecord *)0) -> irlo) ):
	    pgd -> upper_disp_limit = 65535;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> eguf) ):
	case ( (int)&(((struct aiRecord *)0) -> egul) ):
	    pgd -> upper_disp_limit = pai -> eguf;
	    pgd -> lower_disp_limit = pai -> egul;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> aoff) ):
	case ( (int)&(((struct aiRecord *)0) -> aslo) ):
	    pgd -> upper_disp_limit = 32767;
	    pgd -> lower_disp_limit = -32768;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> smoo) ):
	    pgd -> upper_disp_limit = 1;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> hihi) ):
	case ( (int)&(((struct aiRecord *)0) -> high) ):
	case ( (int)&(((struct aiRecord *)0) -> low) ):
	case ( (int)&(((struct aiRecord *)0) -> lolo) ):
	case ( (int)&(((struct aiRecord *)0) -> devs) ):
	    pgd -> upper_disp_limit = pai -> eguf;
	    pgd -> lower_disp_limit = pai -> egul;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> roc) ):
	case ( (int)&(((struct aiRecord *)0) -> adel) ):
	case ( (int)&(((struct aiRecord *)0) -> mdel) ):
	case ( (int)&(((struct aiRecord *)0) -> devl) ):
	    span = pai -> eguf - pai -> egul;
	    if (span < 0.0)
		span = -span;
	    pgd -> upper_disp_limit = span;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> rocn) ):
	    pgd -> upper_disp_limit = 60;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> hyst) ):
	    span = pai -> hopr - pai -> lopr;
	    if (span < 0.0)
		span = -span;
	    pgd -> upper_disp_limit = span;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> rval) ):

	    /* if software point */
	    if ( (pai -> dtyp <=1 ) &&
		 ( (pai -> inp.type == CONSTANT) ||
		   (pai -> inp.type == PV_LINK) ||
		   (pai -> inp.type == DB_LINK) ||
		   (pai -> inp.type == CA_LINK) ) )
	    {
		pgd -> upper_disp_limit = pai -> hopr;
		pgd -> lower_disp_limit = pai -> lopr;
	    }
	    /* are hardware */
	    else
	    {
		pgd -> upper_disp_limit = 65535;
		pgd -> lower_disp_limit = 0;
	    }
	    break;

	case ( (int)&(((struct aiRecord *)0) -> idev) ):
	    span = pai -> irhi - pai -> irlo;
	    if (span < 0.0)
		span = -span;
	    pgd -> upper_disp_limit = span;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> gain) ):
	    pgd -> upper_disp_limit = 999;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    pgd->upper_disp_limit = pai->hopr;
	    pgd->lower_disp_limit = pai->lopr;
	    break;
    }
    return (0);
}


/*------------------------------------------------------------------------*/

static long get_control_double(paddr,pcd)

    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    struct   aiRecord    *pai = (struct aiRecord *) paddr->precord;
    float    span;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct aiRecord *)0) -> irhi) ):
	case ( (int)&(((struct aiRecord *)0) -> irlo) ):
	    pcd -> upper_ctrl_limit = 65535;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> eguf) ):
	case ( (int)&(((struct aiRecord *)0) -> egul) ):
	    pcd -> upper_ctrl_limit = pai -> eguf;
	    pcd -> lower_ctrl_limit = pai -> egul;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> aoff) ):
	case ( (int)&(((struct aiRecord *)0) -> aslo) ):
	    pcd -> upper_ctrl_limit = 32767;
	    pcd -> lower_ctrl_limit = -32768;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> smoo) ):
	    pcd -> upper_ctrl_limit = 1;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> hihi) ):
	case ( (int)&(((struct aiRecord *)0) -> high) ):
	case ( (int)&(((struct aiRecord *)0) -> low) ):
	case ( (int)&(((struct aiRecord *)0) -> lolo) ):
	case ( (int)&(((struct aiRecord *)0) -> devs) ):
	    pcd -> upper_ctrl_limit = pai -> eguf;
	    pcd -> lower_ctrl_limit = pai -> egul;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> roc) ):
	case ( (int)&(((struct aiRecord *)0) -> adel) ):
	case ( (int)&(((struct aiRecord *)0) -> mdel) ):
	case ( (int)&(((struct aiRecord *)0) -> devl) ):
	    span = pai -> eguf - pai -> egul;
	    if (span < 0.0)
		span = -span;
	    pcd -> upper_ctrl_limit = span;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> rocn) ):
	    pcd -> upper_ctrl_limit = 60;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> hyst) ):
	{

	    span = pai -> hopr - pai -> lopr;
	    if (span < 0.0)
	        span = -span;
	    pcd -> upper_ctrl_limit = span;
	    pcd -> lower_ctrl_limit = 0;
	    break;
	}

	case ( (int)&(((struct aiRecord *)0) -> hopr) ):
	case ( (int)&(((struct aiRecord *)0) -> lopr) ):
	    pcd -> upper_ctrl_limit = pai -> eguf;
	    pcd -> lower_ctrl_limit = pai -> egul;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> idev) ):
	    span = pai -> irhi - pai -> irlo;
	    if (span < 0.0)
		span = -span;
	    pcd -> upper_ctrl_limit = span;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct aiRecord *)0) -> gain) ):
	    pcd -> upper_ctrl_limit = 999;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	default:
	    pcd -> upper_ctrl_limit = pai -> hopr;
	    pcd -> lower_ctrl_limit = pai -> lopr;
	    break;
    }
    return (0);
}


/*------------------------------------------------------------------------*/

static void alarm (pai)
    struct aiRecord	*pai;
{
    double		val = pai -> val;
    register float	hval, lval;
    register short	i, inx;
    int			adis = 0;
    ALARM_LIMITS	alarm_limits;

    /* add value to the rate of change buffer, 
       if there is a rate of chg limit or 1st pass is set
    */

    if ( (pai -> roc > 0.0) || (pai -> frps == FALSE) )
    {
	pai -> roci++;
	if (pai->roci >= MAX_ROC_VALUES) 
	    pai->roci = 0;
	    
	if (pai->rocc < MAX_ROC_VALUES) 
	    pai->rocc++;
	    
	if (pai -> frps)
	{
	    pai -> frps = FALSE;
	    *(float *) (pai -> rocb) = val;
	}
	*(float *) (pai->rocb + (pai->roci * sizeof (float))) = val;
    }

    /* check for alarm inhibit */

    adis = AlarmDisableLink ((void *)pai);

    if ( (pai->aenb == ALARM_DISABLED) || (adis != 0) )
    {
	if (pai -> nsta != NO_INIT_ALARM)
	{
	    pai->nsta = INHIBIT_ALARM;
	    pai->nsev = NO_ALARM;
	}
	return;
    }
    if(pai->udf == TRUE ) 
    {
	if (pai->nsev < VALID_ALARM) 
	{
	    pai->nsta = UDF_ALARM;
	    pai->nsev = VALID_ALARM;
	}
	return;
    }

    /* rate of change alarms */

    if ( (pai->rcsv != NO_ALARM) && (pai->rocb) && (pai -> roc > 0.0) ) 
    {
	if (pai->rocc >= 2) 
	{
	    inx = pai->roci;
	    hval = lval =  *(float *) (pai->rocb + (inx * sizeof (float)));
	    for (i = 0; 
		 i < ((pai -> rocc < pai->rocn) ? pai -> rocc : pai -> rocn); 
		 i++) 
	    {
		if (--inx < 0) 
		    inx = MAX_ROC_VALUES - 1;
		    
		if ((*((float *) (pai->rocb + (inx * sizeof (float))))) > hval) 
		    hval = *((float *) (pai->rocb + (inx * sizeof (float))));
		    
		if ((*((float *) (pai->rocb + (inx * sizeof (float))))) < lval) 
		    lval = *((float *) (pai->rocb + (inx * sizeof (float))));
		    
	    }
	    if ((hval - lval) > pai->roc) 
	    {
		pai->nsev = pai->rcsv;
		pai->nsta = ROC_ALARM;
	    }
	}
    }

    /* SBAD etc alarms, check for alarm delay ALDL */

    if ( (pai -> nsta == SBAD_ALARM) || (pai -> nsta == CALC_ALARM) ||
	 (pai -> nsta == OPENTC_ALARM) )
    {
	if (pai -> aldl > 0)
	{
	    if (pai -> sbct < ALDL_MAX)
	        pai -> sbct++;
	    if (pai -> sbct >= pai -> aldl)
		pai -> nsev = MAJOR_ALARM;
	    else
		pai -> nsev = pai -> nsta = NO_ALARM;
	}
    }
    else
	pai -> sbct = 0;

    /* set up for limit alarms */

    alarm_limits.hihi = pai -> hihi;
    alarm_limits.high = pai -> high;
    alarm_limits.low  = pai -> low;
    alarm_limits.lolo = pai -> lolo;
    alarm_limits.devl = pai -> devl;
    alarm_limits.devs = pai -> devs;
    alarm_limits.hyst = pai -> hyst;
    alarm_limits.hhsv = pai -> hhsv;
    alarm_limits.hsv  = pai -> hsv;
    alarm_limits.lsv  = pai -> lsv;
    alarm_limits.llsv = pai -> llsv;
    alarm_limits.dvsv = pai -> dvsv;

    AlarmLimits (&alarm_limits, val, (void *)pai);

    /* do a return to normal, check for alarm delay ALDL only
       on the listed alarms
    */

    if ( ((pai -> nsta == NO_ALARM) && (pai -> nsev == NO_ALARM)) &&
	 ((pai -> stat == HIHI_ALARM) || (pai -> stat == HIGH_ALARM) ||
	  (pai -> stat == LOW_ALARM)  || (pai -> stat == LOLO_ALARM) ||
	  (pai -> stat == DEVIATION_ALARM) || (pai -> stat == SBAD_ALARM) ||
	  (pai -> stat == CALC_ALARM) || (pai -> stat == OPENTC_ALARM)) )
    {
	if (pai -> aldl > 0)
	{
	    if (pai -> rtct < ALDL_MAX)
		pai -> rtct++;

	    /* delay return to normal so, continue last alarm */
	    if (pai -> rtct < pai -> aldl)
	    {
		pai -> nsta = pai -> stat;
		pai -> nsev = pai -> sevr;
	    }
	}
    }
    else
	pai -> rtct = 0;

    return;
}


/*------------------------------------------------------------------------*/

static void convert (pai)

    struct	aiRecord	*pai;
{
	double		val;
	float		aslo = pai->aslo;
	float		aoff = pai->aoff;

        /* check value in instrument range */

	val = pai -> rval;
        if ((pai->irhi != 0) || (pai->irlo != 0)) 
	{
            if ((val > (pai->irhi + pai->idev)) || 
		(val < (pai->irlo - pai->idev)) ) 
	    {
		if (pai -> nsev < VALID_ALARM)
		{
                    pai->nsta = SBAD_ALARM;
                    pai->nsev = VALID_ALARM;
		}
                return;
            }
        }


	/* adjust slope and offset */

	if (aslo != 0.0) 
	{
	    val *= aslo;
	}
	if (aoff != 0.0) 
	{
	    val += aoff;
	}

	/* convert raw to engineering units and signal units */

	if(pai->linr == NOCONV) 
	{
		; /* do nothing*/
	}
	else if (pai->linr == LINEAR) 
	{

	   /* y = mx + b, m = slope = eslo, b = offset = roff 
	      calc'd in slope_and_offset()
	   */

	   val = (pai -> eslo * val) + pai -> roff;
	}
		  /* use breakpoint table */
	else if ( (pai -> linr >= START_BRKPT) &&
		  (pai -> linr <= MAXBRKPTTBLS) )
	{
	    struct brkTable *pbrkTable;
	    struct brkInt   *pInt;	
	    struct brkInt   *pnxtInt;
	    short	    lbrk;
	    int		    number;

	    pbrkTable = (struct brkTable *) pai->pbrk;
	    number = pbrkTable->number;
	    if (pai->init) 
		lbrk = number / 2;	/* Just start in the middle */
	    else 
	    {
		lbrk = (pai->lbrk);

		/*make sure we dont go off end of table*/

		if ((lbrk+1) >= number) 
		    lbrk--;
	    }
	    pInt = pbrkTable->papBrkInt [lbrk];
	    pnxtInt = pbrkTable->papBrkInt [lbrk + 1];

	    /* find entry for increased value */

 	    while ((pnxtInt->raw) <= val) 
	    {
		lbrk++;
		pInt = pbrkTable->papBrkInt [lbrk];
		if (lbrk >= number - 1) 
		{
		    if(pai->nsev < VALID_ALARM) 
		    {
			pai->nsta = SOFT_ALARM;
			pai->nsev = VALID_ALARM;
		    }
		    break;		/* out of while */
		}
		pnxtInt = pbrkTable->papBrkInt [lbrk + 1];
	    }
	    while ((pInt->raw) > val) 
	    {
		if (lbrk == 0) 
		{
		    if (pai->nsev < VALID_ALARM) 
		    {
			pai->nsta = SOFT_ALARM;
			pai->nsev = VALID_ALARM;
		    }
		    break;		/* out of while */
		}
		lbrk--;
		pInt = pbrkTable->papBrkInt [lbrk];
	    }
	    pai->lbrk = lbrk;
	    val = pInt->eng + (val - pInt->raw) * pInt->slope;
	} /* end use break pt */

	else if (pai -> linr == CNVSQRROOT)
	{
	    double temp_dbl;
	    double sqrt_val;

	    /* val = eng units full - eng units low * square root of
		    raw aval - instrument range low divided by
		    instrument range high - instrument range low
		    plus eng units low
	    */

	    if (pai -> irhi == pai -> irlo)
		sqrt_val = 0.0;
	    else
	    {
		temp_dbl = ( (val - pai -> irlo) / (pai -> irhi - pai ->irlo) );
		if (temp_dbl < 0.0)
		{
		    if (pai -> nsev < MAJOR_ALARM)
		    {
			pai -> nsta = CALC_ALARM;
			pai -> nsev = VALID_ALARM;
		    }
		    return;
		}
		sqrt_val = sqrt (temp_dbl);
	    }
	    val = ((pai -> eguf - pai -> egul) * sqrt_val) + pai -> egul;
	}

	/* apply smoothing algorithm only if fully init'd since we do not
	   have a valid value in pai -> val until then
	*/

	if ( (pai->smoo != 0.0) && (pai -> ini2) )
	{
	    pai->val = val * (1.00 - pai->smoo) + (pai->val * pai->smoo);
	}
	else
	    pai->val = val;
	    
	pai->udf = FALSE;
	return;
}


/*------------------------------------------------------------------------*/

static void monitor (pai)

    struct	aiRecord	*pai;
{
	unsigned short	monitor_mask;
	double		delta;
	short		stat, sevr, nsta, nsev;
	int		set_ack = 1;
	
	/* get previous stat and sevr  and new stat and sevr*/

	stat = pai->stat;
	sevr = pai->sevr;
	nsta = pai->nsta;
	nsev = pai->nsev;

	/*set current stat and sevr*/

	pai->stat = nsta;
	pai->sevr = nsev;
	pai->nsta = 0;
	pai->nsev = 0;

	/* if we are coming out of VALID_ALARM and our new stat/sevr are
	   the same as when we went into VALID_ALARM, do not set ack
	*/
	if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	     (nsta == pai -> psta) && (nsev == pai -> psev) )
	{
	    set_ack = 0;
	}
	/* if we are going into VALID_ALARM, and out previous stat was not
	   VALID_ALARM, remember previous non VALID_ALARM stat/sevr
	   and we do not want to set ACK,
	*/
	if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
	{
	    set_ack = 0;
	    pai -> psta = stat;
	    pai -> psev = sevr;
	    db_post_events ((struct dbCommon *)pai, (caddr_t)&pai -> psta, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pai, (caddr_t)&pai -> psev, DBE_VALUE);
	}

	monitor_mask = 0;


	/* alarm condition changed this scan */

	if ((stat != nsta) || (sevr != nsev)) 
	{
	    /* post events for alarm condition change*/

	    monitor_mask = DBE_ALARM;

	    /* when alarm status chg's, get current time and
	       save VAL in lalm (last value when alarm chg'd)
	    */
	    tsLocalTime (&pai -> latm);
	    pai -> lalm = pai -> val;

	    /* post stat and nsev fields */

	    db_post_events ((struct dbCommon *)pai, (caddr_t)&pai->stat, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pai, (caddr_t)&pai->sevr, DBE_VALUE);
            if ( (pai->sevr != NO_ALARM) && (set_ack) &&
		 (pai->sevr != EVENT_ALARM) ) 
	    {
                if (pai->ack == 0) 
		{
                    pai->ack = 1;           /* new unacknowledged alarm */
                    db_post_events((struct dbCommon *)pai, (caddr_t)&pai->ack, DBE_VALUE);
                }
	    }
	}
	/* check for value change */

	delta = pai->mlst - pai->val;
	if (delta < 0.0) 
	    delta = -delta;
	
	if (delta > pai->mdel) 
	{
	    /* post events for value change */

	    monitor_mask |= DBE_VALUE;

	    /* update last value monitored */

	    pai->mlst = pai->val;
	}

	/* only archive if trending is enabled */

	if (pai -> tenb)
	{
	    /* check for archive change */

	    delta = pai->alst - pai->val;
	    if (delta < 0.0) 
	    delta = -delta;
       
	    /* if exceed archive deadband or there has been a change in alarm status */
	    if ( (delta > pai->adel) || (monitor_mask & DBE_ALARM) )
	    {
	    	/* post events on value field for archive change */
	    
		monitor_mask |= DBE_LOG;

	    	/* update last archive value monitored */

	    	pai->alst = pai->val;
	    }
	}

	/* send out monitors connected to the value field */

	if (monitor_mask) 
	{
	    db_post_events ((struct dbCommon *)pai, (caddr_t)&pai->val, monitor_mask);
	    if (pai->oraw != pai->rval) 
	    {
		db_post_events ((struct dbCommon *)pai, (caddr_t)&pai->rval, DBE_VALUE);
		pai->oraw = pai->rval;
	    }
	}
	return;
}


/*------------------------------------------------------------------------*/

static void slope_and_offset (pai)
   struct aiRecord	*pai;
{
      /* slope = eng units full - eng units low / 
                 instrument range high - instrument range low
         first check for divide by 0
      */

   if (pai -> irhi == pai -> irlo)
      pai -> eslo = 0.0;
   else
      pai -> eslo = (pai -> eguf - pai -> egul) /
                    (pai -> irhi - pai -> irlo);

      /* offset is eng units low - (slope * instrument range low */

   pai -> roff = pai -> egul - (pai -> eslo * pai -> irlo);

   return;
}
