/*recDeadtime.c,v 1.1.1.1 1999/08/03 14:44:19 tis4000 Exp */

/****************************************************************/
/*                                                              */
/*    This software is the proprietary property of Tate         */
/*    Integrated Systems, L.P.  Patents and copyrights are      */
/*    pending.                                                  */
/*                                                              */
/*    This software is furnished under a license and may        */
/*    be used and copied only in accordance with the term       */
/*    of such license and with the inclusion of the above       */
/*    notice. This software or any other copies thereof         */
/*    may not be provided or otherwise made available to        */
/*    any other person. No title to and ownership of the        */
/*    software is hereby transferred.                           */
/*                                                              */
/*    The information in this software is subject to            */
/*    change without notice and should not be construed         */
/*    as a commitment by Tate Integrated Systems L.P. or        */
/*    its third party suppliers.                                */
/*                                                              */
/*    Tate Integrated Systems, L.P. (TIS) and its third         */
/*    party suppliers assume no responsibility for the          */
/*    use or inability to use any of its software. TIS          */
/*    software is provided "as is" without warranty of          */
/*    any kind and TIS expressly disclaims all implied          */
/*    warranties, including but not limited to the implied      */
/*    warranties of merchantability and fitness for a           */
/*    particular purpose.                                       */
/*                                                              */
/*    Notice: Notwithstanding any other lease or license        */
/*    that may pertain to or accompany the delivery of this     */
/*    computer software, the rights of the Government           */
/*    regarding its use, reproduction, and disclosure are       */
/*    as set forth in Section 52.227-19 of the FARS             */
/*    Computer Software-Restricted Rights clause.               */
/*                                                              */
/****************************************************************/

  
/****************************************************************************
 *
 *  MODULE NAME            : recDeadtime.c 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *
 *  DESCRIPTION: record support routines for the deadtime record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 28 Feb 94
 *  REVISION:  removed ALST, MLST from redundancy, not needed
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 02 Mar 94
 *  REVISION: added process stats code
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 94
 *  REVISION: make forward func defsf agree with actual func defs
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 May 94
 *  REVISION:	rewrite init_record, dont send DETI as redund, let slave keep
 *		it's own DETI, only send IVAL,ISTA,ISEV if NUMP is > 0
 *		to keep down traffic on network, nothing gets put in buffers
 *		unless NUMP > 0 anyway
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 08 Aug 94
 *  REVISION:	added get_control_double and added field types to 
 *		get_graphic_double
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Oct 94
 *  REVISION:	add TIME to redundancy, added fpps
 *		(1st process pass), do time stamp when stat or sevr chg,
 *		see monitor(),and not in process(),
 *		added lalm (last value when alarm chg'd)
 *		set lalm to val when 
 *		stat or sevr chg,in process() if fpps (first process)
 *		init lalm to val and if time stamp is 0, init it also
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Oct 94
 *  REVISION:	chg time to latm (last alarm time)
 *		added lptm (last process time)
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Dec 94
 *  REVISION:	removed fpps and lalm, not needed
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 10 Apr 95
 *  REVISION:	moved check of okToUse on inp from resolve_pv to init_record
 *		we need to set dbAddr for STAT and SEVR for inp regardless
 *		of whether it is okToUse
 *
****************************************************************************/

/****************************************************************************
 *
 * Programmer's Notes:
 *    TFTicksDiffInSeconds () - defined in .../src/rec/tankfarm.c
 *
 *
****************************************************************************/


#include	<vxWorks.h>
#include	<inetLib.h>
#include        <tickLib.h>    /* for tickGet () */
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include        <math.h>
#include	<string.h>
#include	<memLib.h>
#include	<stdlib.h>

#include	"alarm.h"
#include	"cvtTable.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/deadtimeRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"deadtimeredfldidx.h"
#include	"restable.h"
#include	"error.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"okToUse.h"
#include	"dbEvent.h"
#include	"tankfarm.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/

#define report NULL
#define initialize NULL
static long init_record();
static long process();
static long special();
static long get_value();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
static long get_units();
static long get_precision();
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
static long get_graphic_double();
static long get_control_double();
#define get_alarm_double NULL

struct rset deadtimeRSET =  {
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
   };


/* The folowing definitions are for redundancy */
 
static struct fldDes	**mylist = 0;

static	char *namelist [] = {
        "STAT",
        "SEVR",
        "IVAL",
        "ISTA",
        "ISEV",
        "DETI",
        "NUMP",
        "SDET",
        "DTRC",
	"INI2",
	"LATM",
	"MLST",
	"LMIN",
	"LMAV",
	"LMCM",
	"LMCX",
        0
        };

static	unsigned	short	type;

#define  MAX_DELAY_VALUES  64
#define  MIN(x,y)    ((x < y) ? x : y)
#define  MAX(x,y)    ((x > y) ? x : y)

static void compute_pushes ();
static void monitor ();
static void output_deadtime_values ();
static long read_input_values ();
static long resolve_pv ();
static void update_buffers ();


/****************************************************************************
 *
 *  MODULE NAME            : init_record 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION: initialize a deadtime record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 03 Jan 94
 *  REVISION:  init_record, set ack to 0
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 03 Jan 94
 *  REVISION:  init_record, removed ack = 0
 *
****************************************************************************/

static long init_record (pdeadtime)
    struct deadtimeRecord    *pdeadtime;
{
    unsigned short	i;
    char		*pstr;
    int 		ini2_flag = 0;
    char		redbuf[256];

    if (!(mylist))
    {
	for (type = 1, i = 0; type < 100; type++)
	{
	    pstr = GET_PRECTYPE (type);
	    if (strncmp (pstr, "deadtime", 8) == 0)
	    {
		i = 1;
		break;
	    }
	}
	if (!i)
	{
	    recGblRecordError (S_db_noRecType, (struct dbCommon *)pdeadtime, "deadtime,init_record");
	    return (S_db_noRecType);
	}

	mylist = redFldAlloc (type, NUMDEADTIME_FLDS);

	if (!mylist)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pdeadtime,
			       "deadtime, init_record, mylist failed");
	    return (S_rec_Alloc);
	}

	for (i = 0; i < NUMDEADTIME_FLDS; i++)
	    redFldAdd (type, i, namelist [i]);
    }

      /* allocate buffer space */

    if (!(pdeadtime -> detb))
    {
	if ((pdeadtime -> detb =
	    (double *) calloc (1, (MAX_DELAY_VALUES * sizeof (double)))) == NULL)
	{
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pdeadtime, 
				   "deadtime: init_record, detb calloc fail");
		return (S_rec_Alloc);
	}
    }
    if (!(pdeadtime -> stab))
    {
	if ((pdeadtime -> stab =
	    (unsigned short *) calloc (1, (MAX_DELAY_VALUES * sizeof (unsigned short)))) == NULL)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pdeadtime, 
			       "deadtime: init_record, stab calloc fail");
	    return (S_rec_Alloc);
	}
    }
    if (!(pdeadtime -> sevb))
    {
	if ((pdeadtime -> sevb =
	    (unsigned short *) calloc (1, (MAX_DELAY_VALUES * sizeof (unsigned short)))) == NULL)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pdeadtime, 
			       "deadtime: init_record, sevb calloc fail");
	    return (S_rec_Alloc);
	}
    }

    pdeadtime -> deti = 0;
    pdeadtime -> cmin = 0;
    pdeadtime -> cmax = 0;
    pdeadtime -> ctot = 0;
    pdeadtime -> cave = 0;
    pdeadtime -> tcks = tickGet();
    pdeadtime -> nump = 0;
    pdeadtime -> sdet = pdeadtime -> dets;
    pdeadtime -> ival = 0;
    pdeadtime -> ista = NO_ALARM;
    pdeadtime -> isev = NO_ALARM;
    pdeadtime -> dtrc = 0;

    pdeadtime -> init = TRUE;
    pdeadtime -> fpas = TRUE;
    pdeadtime -> nsta = NO_ALARM;
    pdeadtime -> nsev = NO_ALARM;

    /* init last process time and last alarm time */

    if (pdeadtime->lptm.secPastEpoch == 0)
    {
	tsLocalTime (&pdeadtime->lptm);
    }
    if (pdeadtime->latm.secPastEpoch == 0)
    {
	tsLocalTime (&pdeadtime->latm);
    }
      /* check this range */

    if ( (pdeadtime -> dets >= 0) && (pdeadtime -> dets < MAX_DELAY_VALUES) )
    {
	/* is there an input pv, resolve INST and INSV also */
	if (resolve_pv (pdeadtime) == NULL)
	{
	    /* only check under these circumstances */
	    if ( (!pdeadtime -> tagd) && (pdeadtime -> oscn != OFF_SCAN) &&
		 (pdeadtime -> mode == NORMAL) )
	    {
		if (!okToUse (&pdeadtime -> inp))
		{
		    pdeadtime -> nsta = NO_INIT_ALARM;
		    pdeadtime -> nsev = VALID_ALARM;
		}
	    }
	    /* init delay rate if DETR is a constant */
	    if (pdeadtime -> detr.type == CONSTANT)
		pdeadtime -> dtrc = (float)pdeadtime -> detr.value.value;
	    else if (pdeadtime -> detr.type == DB_LINK)
	    {
		/* only check under these circumstances */
		if ( (!pdeadtime -> tagd) && (pdeadtime -> oscn != OFF_SCAN) &&
		     (pdeadtime -> mode == NORMAL) )
		{
		    if (!okToUse (&pdeadtime -> detr))
		    {
		    	pdeadtime -> nsta = NO_INIT_ALARM;
		    	pdeadtime -> nsev = VALID_ALARM;
		    }
		}
	    }
	    if (pdeadtime -> nsta != NO_INIT_ALARM)
		ini2_flag =1;

	}
	else
	{
	    pdeadtime -> nsta = NO_INIT_ALARM;
	    pdeadtime -> nsev = VALID_ALARM;
	}
    }
    else
    {
	recGblRecordError (S_db_badField, (struct dbCommon *)pdeadtime, 
					"deadtime: init_record, range");
	pdeadtime -> nsta = NO_INIT_ALARM;
	pdeadtime -> nsev = VALID_ALARM;
    }

      /* do redundancy if not in slave mode */

    if (resGetCaAnswer (pdeadtime -> radr, &i) == RES_OK)
    {
	if (i)
	{
	    monitor (pdeadtime);

	    if (ini2_flag)
	    {
		pdeadtime -> ini2 = 1;

		redQInit ((struct dbCommon *)pdeadtime, redbuf, sizeof (redbuf), type);

		redQField (redbuf, mylist [STAT]);
		redQField (redbuf, mylist [SEVR]);
		redQField (redbuf, mylist [INI2]);

		redQSend (redbuf);
	    }
	}
    }

    return (0);

} /* end func init_record () */


/****************************************************************************
 *
 *  MODULE NAME            : compute_pushes 
 *  AUTHOR                 : emsteel
 *  DATE                   : 17 Sep 93
 *
 *  CALLING MODULE(S)      : process
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION: how many times to push (nump) IVAL, ISTA and ISEV onto 
 *                  DETB, STAB, SEVB buffers
 *               convert delay rate to clock ticks (rtcks),  use
 *                  vxTicksPerSecond asd defined in .../share/epicsH/dbDefs.h
 *               get the current clock ticks (ctcks)
 *               compute number of times to push current IVAL etc onto que
 *               new TCKS is old TCKS + (nump * rate) in tcks and not ctks + rate
 *                 this handles any flutter in the time between scans
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : ems
 *  REVISION DATE          : 11 nov 93
 *  REVISION:  deleted 60.0, will use vxTicksPerSecond as 
 *             defined in .../share/epicsH/dbDefs.h
****************************************************************************/

static void compute_pushes (pdeadtime)
   struct deadtimeRecord   *pdeadtime;
{
   unsigned long ctcks = 0;       /* current clock ticks */
   unsigned long rtcks = 0;       /* current rate in clock ticks */
   float         delay_rate = 0;
   long          status     = 0;
   long          options    = 0;
   long          nRequests  = 1;

   if (pdeadtime -> detr.type != CONSTANT)
   {
      if (pdeadtime -> detr.type == DB_LINK)
      {
         status = dbGetLink (&pdeadtime -> detr.value.db_link,
                             (struct dbCommon *)pdeadtime, DBR_FLOAT,
                             (caddr_t)&delay_rate, &options, &nRequests);
         if (status)
         {
            pdeadtime -> ista = LINK_ALARM;
            pdeadtime -> isev = VALID_ALARM;
            rtcks = 0;
         }
      }
   }
   else
      delay_rate = pdeadtime -> dtrc;
   if (delay_rate < 0)
      delay_rate = 0;

   rtcks = (unsigned long)(delay_rate * vxTicksPerSecond);

   ctcks = tickGet();
   if (rtcks == 0)
   {
      pdeadtime -> nump = 1;
      pdeadtime -> tcks = ctcks;
   }
   else
   {
      pdeadtime -> nump = 0;
      if (ctcks > pdeadtime -> tcks)
      {
         pdeadtime -> nump = (unsigned short)((ctcks - pdeadtime -> tcks) / rtcks);
         pdeadtime -> tcks += pdeadtime -> nump * rtcks;
      }
   }
   if (pdeadtime -> nump)
      update_buffers (pdeadtime);

   return;

} /* end func compute_pushes () */


/****************************************************************************
 *
 *  MODULE NAME            : get_graphic_double 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
****************************************************************************/

static long get_graphic_double(paddr, pgd)
    struct    dbAddr          *paddr;
    struct    dbr_grDouble    *pgd;
{
    struct deadtimeRecord *pdeadtime =
                                 (struct deadtimeRecord *) paddr -> precord;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct deadtimeRecord *)0) -> adel) ):
	case ( (int)&(((struct deadtimeRecord *)0) -> mdel) ):
	{
	    float    span;

	    span = pdeadtime -> hopr - pdeadtime -> lopr;
	    if (span < 0.0)
	        span = -span;
	    pgd -> upper_disp_limit = span;
	    pgd -> lower_disp_limit = 0;
	    break;
	}

	case ( (int)&(((struct deadtimeRecord *)0) -> ctot) ):
	    pgd -> upper_disp_limit = pdeadtime -> hopr * pdeadtime -> dets;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct deadtimeRecord *)0) -> dtrc) ):
	    pgd -> upper_disp_limit = 86400;	/* seconds/day */
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct deadtimeRecord *)0) -> dets) ):
	    pgd -> upper_disp_limit = 63;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    pgd -> upper_disp_limit = pdeadtime -> hopr;
	    pgd -> lower_disp_limit = pdeadtime -> lopr;
	    break;
    }

    return (0);

} /* end func get_graphic_double () */


/****************************************************************************
 *
 *  MODULE NAME            : get_control_double
 *  AUTHOR                 : emsteel
 *  DATE                   : 08 Aug 94
 *
****************************************************************************/

static long get_control_double(paddr,pcd)
    struct    dbAddr          *paddr;
    struct    dbr_ctrlDouble  *pcd;
{
    struct deadtimeRecord *pdeadtime =
                                 (struct deadtimeRecord *) paddr -> precord;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct deadtimeRecord *)0) -> adel) ):
	case ( (int)&(((struct deadtimeRecord *)0) -> mdel) ):
	{
	    float    span;

	    span = pdeadtime -> hopr - pdeadtime -> lopr;
	    if (span < 0.0)
	        span = -span;
	    pcd -> upper_ctrl_limit = span;
	    pcd -> lower_ctrl_limit = 0;
	    break;
	}

	case ( (int)&(((struct deadtimeRecord *)0) -> dtrc) ):
	    pcd -> upper_ctrl_limit = 86400;	/* seconds/day */
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct deadtimeRecord *)0) -> dets) ):
	    pcd -> upper_ctrl_limit = 63;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	default:
	    pcd -> upper_ctrl_limit = pdeadtime -> hopr;
	    pcd -> lower_ctrl_limit = pdeadtime -> lopr;
	    break;
    }

    return (0);
 
} /* end func get_control_double () */


/****************************************************************************
 *
 *  MODULE NAME            : get_precision
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
****************************************************************************/

static long get_precision (paddr, precision)
   struct   dbAddr    *paddr;
   long               *precision;
{
   struct deadtimeRecord *pdeadtime =
                                 (struct deadtimeRecord *) paddr -> precord;

   *precision = pdeadtime -> prec;

   return (0);

} /* end func get_precision () */

/****************************************************************************
 *
 *  MODULE NAME            : get_units 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
****************************************************************************/

static long get_units(paddr,units)
   struct   dbAddr   *paddr;
   char              *units;
{
    struct deadtimeRecord *pdeadtime =
                                  (struct deadtimeRecord *) paddr -> precord;

    strncpy (units, pdeadtime -> egu, sizeof (pdeadtime -> egu));

    return (0);

} /* end func get_units () */

/****************************************************************************
 *
 *  MODULE NAME            : get_value 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
****************************************************************************/

static long get_value(pdeadtime,pvdes)
   struct    deadtimeRecord    *pdeadtime;
   struct    valueDes          *pvdes;
{
    pvdes -> field_type = DBF_DOUBLE;
    pvdes -> no_elements = 1;
    (double *) (pvdes -> pvalue) = &pdeadtime -> val;

    return (0);

} /* end func get_value () */


/****************************************************************************
 *
 *  MODULE NAME            : monitor 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *  CALLING MODULE(S)      : process
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION: update event list
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 05 Jan 94
 *  REVISION:  check VALID_ALARM before set ACK
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Aug 94
 *  REVISION:	rm DBE_LOG from ack, only archive if tenb (trending) is
 *		enabled
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 07 Feb 95
 *  REVISION:	if post VAL, post CMIN,CMAX,CAVE,CTOT, else if IVAL (input)
 *		has chg'd do the same, else see if CMIN,CMAX,CAVE have chg'd
 *		do the same
 *		for archive, if NUMP(num times IVAL was placed in buffer) > 0
 *		then if above have chg'd archive
 *		LMIN = last input used for monitor
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 15 Mar 95
 *  REVISION:	do not set ack if sevr is EVENT_ALARM
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Sept 95
 *  REVISION:	archive is stat/sevr also chg
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Sept 95
 *  REVISION:	mod to way handle VALID_ALARM
 *
****************************************************************************/

static void monitor (pdeadtime)
   struct deadtimeRecord	*pdeadtime;
{
    unsigned short	monitor_mask;
    double		delta;
    short		stat, sevr, nsta, nsev;
    int			set_ack = 1;

      /* get previous stat and sevr  and new stat and sevr */

    stat = pdeadtime -> stat;
    sevr = pdeadtime -> sevr;
    nsta = pdeadtime -> nsta;
    nsev = pdeadtime -> nsev;

      /* set current stat and sevr */

    pdeadtime -> stat = nsta;
    pdeadtime -> sevr = nsev;
    pdeadtime -> nsta = 0;
    pdeadtime -> nsev = 0;

    /* if we are coming out of VALID_ALARM and our new stat/sevr are
       the same as when we went into VALID_ALARM, do not set ack
    */
    if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	 (nsta == pdeadtime -> psta) && (nsev == pdeadtime -> psev) )
    {
	set_ack = 0;
    }
    /* if we are going into VALID_ALARM, and out previous stat was not
       VALID_ALARM, remember previous non VALID_ALARM stat/sevr
       and we do not want to set ACK,
    */
    if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
    {
	set_ack = 0;
	pdeadtime -> psta = stat;
	pdeadtime -> psev = sevr;
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> psta, DBE_VALUE);
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> psev, DBE_VALUE);
    }

    monitor_mask = 0;

      /* alarm condition changed this scan */

    if ((stat != nsta) || (sevr != nsev))
    {
         /* post events for alarm condition change */

	monitor_mask = DBE_ALARM;

	/* when alarm status chg's, get current time 
	*/
	tsLocalTime (&pdeadtime -> latm);

         /* post stat and nsev fields */

	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> stat, DBE_VALUE);
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> sevr, DBE_VALUE);
	if ((pdeadtime -> sevr != NO_ALARM) && (set_ack) &&
	    (pdeadtime -> sevr != EVENT_ALARM) )
	{
	    if (pdeadtime -> ack == 0)
	    {
		pdeadtime -> ack = 1;           /* new unacknowledged alarm */
		db_post_events((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> ack, DBE_VALUE);
	    }
	}
    }

      /* check for value change */
    delta = pdeadtime -> mlst - pdeadtime -> val;
    if (delta < 0.0)
    {
	delta = -delta;
    }
    if (delta > pdeadtime -> mdel)
    {
         /* post events for value change */

	monitor_mask |= DBE_VALUE;

         /* update last value monitored */

	pdeadtime -> mlst = pdeadtime -> val;
    }

      /* check for archive change if tenb (trending) is enabled */

    if (pdeadtime -> tenb)
    {
	delta = pdeadtime -> alst - pdeadtime -> val;
	if (delta < 0.0)
	{
	    delta = -delta;
	}
	if ( (delta > pdeadtime -> adel) || (monitor_mask & DBE_ALARM) )
	{
	    /* post events on value field for archive change */

	    monitor_mask |= DBE_LOG;

	    /* update last archive value monitored */

	    pdeadtime -> alst = pdeadtime -> val;
	}
    }

      /* send out monitors connected to the value field */

    if (monitor_mask)
    {
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> val, monitor_mask);
    }

   return;

} /* end func monitor () */


/****************************************************************************
 *
 *  MODULE NAME            : output_deadtime_values 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *  CALLING MODULE(S)      : process
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION:
 *              if in AUTOMATIC (NORMAL) mode then update VAL, NSTA and
 *              NSEV from buffers
 *
****************************************************************************/

static void output_deadtime_values (pdeadtime)
   struct deadtimeRecord   *pdeadtime;
{
   short    fifo;
   short    i;
   short    idx;
   double   ival;

   if (pdeadtime -> mode == NORMAL) 
   {
      if (pdeadtime -> sdet == 0)
         fifo = pdeadtime -> deti;
      else
      {
         fifo = pdeadtime -> deti - pdeadtime -> sdet;
         if (fifo < 0)
         {
            fifo = -fifo;
            fifo = MAX_DELAY_VALUES - fifo;
         }
      }
      pdeadtime -> val  = pdeadtime -> detb[fifo];
      if (pdeadtime -> aenb == ALARM_ENABLED)
      {
         pdeadtime -> nsta = pdeadtime -> stab[fifo];
         pdeadtime -> nsev = pdeadtime -> sevb[fifo];
      }
   }

      /* get some more values based on new value of deti */

   pdeadtime -> cmin = pdeadtime -> cmax = pdeadtime -> ival;
   pdeadtime -> ctot = pdeadtime -> cave = pdeadtime -> ival;

   if (pdeadtime -> sdet > 0)
   {
      idx = pdeadtime -> deti;
      for (i = 1; i < pdeadtime -> sdet; i++)
      {
         idx--;
         if (idx < 0)
            idx = MAX_DELAY_VALUES - 1;
         ival = pdeadtime -> detb[idx];
         pdeadtime -> cmin = MIN (pdeadtime -> cmin, ival);
         pdeadtime -> cmax = MAX (pdeadtime -> cmax, ival);
         pdeadtime -> ctot += ival;
      }
      pdeadtime -> cave = pdeadtime -> ctot / (double)pdeadtime -> sdet;
   }

   return;

} /* end func output_deadtime_values () */


/****************************************************************************
 *
 *  MODULE NAME            : process 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *
 *  DESCRIPTION: called when time to process a deadtime record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 28 Feb 94
 *  REVISION:  dont have to check for MASTER for redunadncy, if process is
 *	       called we are master
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 7 Feb 95
 *  REVISION:	added LMIN to redundant update, only post IVAL if chg > MDEL
 *
****************************************************************************/

static long process (paddr)
   struct dbAddr    *paddr;
{
    struct deadtimeRecord   *pdeadtime =
                                (struct deadtimeRecord *) (paddr -> precord);
    long		status = 0;
    unsigned short	stat   = pdeadtime -> stat;
    unsigned short	sevr   = pdeadtime -> sevr;
    unsigned short	ista   = pdeadtime -> ista;
    unsigned short	isev   = pdeadtime -> isev;
    short		sdet   = pdeadtime -> sdet;
    float		dtrc   = pdeadtime -> dtrc;
    double		mlst   = pdeadtime -> mlst;
    double		alst   = pdeadtime -> alst;
    TS_STAMP		latm   = pdeadtime -> latm;
    double		lmin   = pdeadtime -> lmin;
    double		lmav   = pdeadtime -> lmav;
    double		lmcm   = pdeadtime -> lmcm;
    double		lmcx   = pdeadtime -> lmcx;
    double		delta;
    int			post_total = 0;
    char		redbuf [256];

      /* if not initialized, do nothing */

    if ((!pdeadtime -> init) || (!pdeadtime -> ini2))
	return (status);

    /* get time stamp for last time record was processed */

    tsLocalTime (&pdeadtime -> lptm);

      /* Initialize the new status to the last */

    pdeadtime -> nsta = pdeadtime -> stat;
    pdeadtime -> nsev = pdeadtime -> sevr;

       /* off normal operations (use VAL) */

    if (pdeadtime -> tagd)
    {
	pdeadtime -> nsta = TAGGED_ALARM;
	pdeadtime -> nsev = VALID_ALARM;
    }
    else if (pdeadtime -> oscn == OFF_SCAN)
    {
	pdeadtime -> nsta = SCAN_ALARM;
	pdeadtime -> nsev = VALID_ALARM;
    }
    else
    {
	pdeadtime -> nsta = NO_ALARM;
	pdeadtime -> nsev = NO_ALARM;

      /* update number of process times */
	if (process_stats)
	    process_cnts[type]++;	

         /* check this range */

	if ((pdeadtime -> sdet < 0) || (pdeadtime -> sdet > (MAX_DELAY_VALUES - 1)))
	{
	    recGblRecordError (S_db_badField, (struct dbCommon *)pdeadtime, 
						"deadtime: process, range");
	    return (S_db_badField);
	}

	pdeadtime -> pact = TRUE;

	status = read_input_values (pdeadtime);

	compute_pushes (pdeadtime);

	output_deadtime_values (pdeadtime);
    }

    /* if the time stamp is 0, init and post val with
       alarm bit set, this makes sure that proper alarm
       messages are set if IOC was down and alarm status
       may have chg'd
    */
    if (pdeadtime -> latm.secPastEpoch == 0)
    {
	tsLocalTime (&pdeadtime -> latm);
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> val, 
							DBE_VALUE | DBE_ALARM);
    }

      /* check event list */

    monitor (pdeadtime);

    pdeadtime -> pact = FALSE;

    /* post events for monitors  and archive */

    /* if VAL was posted then post CTOT */

    if (mlst != pdeadtime -> mlst)
	post_total = 1;

    /* if IVAL was actually pushed into the buffer */
    if (pdeadtime -> nump > 0)
    {
	delta = pdeadtime -> lmin - pdeadtime -> ival;
	if (delta < 0.0)
	    delta = -delta;
	if (delta > pdeadtime -> mdel)
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> ival, DBE_VALUE);
	    pdeadtime -> lmin = pdeadtime -> ival;
	    /* if IVAL has chg'd then post CTOT */
	    post_total = 1;
	}
	delta = pdeadtime -> lmav - pdeadtime -> cave;
	if (delta < 0.0)
	    delta = -delta;
	if (delta > pdeadtime -> mdel)
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> cave, DBE_VALUE);
	    pdeadtime -> lmav = pdeadtime -> cave;
	    /* if CAVE has chg'd then post CTOT */
	    post_total = 1;
	}
	delta = pdeadtime -> lmcm - pdeadtime -> cmin;
	if (delta < 0.0)
	    delta = -delta;
	if (delta > pdeadtime -> mdel)
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> cmin, DBE_VALUE);
	    pdeadtime -> lmcm = pdeadtime -> cmin;
	    /* if CMIN has chg'd then post CTOT */
	    post_total = 1;
	}
	delta = pdeadtime -> lmcx - pdeadtime -> cmax;
	if (delta < 0.0)
	    delta = -delta;
	if (delta > pdeadtime -> mdel)
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> cmax, DBE_VALUE);
	    pdeadtime -> lmcx = pdeadtime -> cmax;
	    /* if CMAX has chg'd then post CTOT */
	    post_total = 1;
	}
    }

    if (post_total)
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> ctot, DBE_VALUE);

    if (ista != pdeadtime -> ista)
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> ista, DBE_VALUE);
    if (isev != pdeadtime -> isev)
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> isev, DBE_VALUE);

    post_total = 0;
    if ((pdeadtime -> tenb) && (alst != pdeadtime -> alst))
	post_total = 1;
    /* see if we need to archive the tots */
    if ( (pdeadtime -> nump > 0) && (pdeadtime -> tenb) )
    {
	delta = pdeadtime -> laav - pdeadtime -> cave;
	if (delta < 0.0)
	    delta = -delta;
	/* if the average has chg'd, the total should have chg'd also */
	if ( (delta > pdeadtime -> adel) || (stat != pdeadtime ->stat) ||
	     (sevr != pdeadtime -> sevr) )
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> cave, DBE_LOG);
	    pdeadtime -> laav = pdeadtime -> cave;
	    post_total = 1;
	}
	delta = pdeadtime -> lamn - pdeadtime -> cmin;
	if (delta < 0.0)
	    delta = -delta;
	if ( (delta > pdeadtime -> adel) || (stat != pdeadtime ->stat) ||
	     (sevr != pdeadtime -> sevr) )
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> cmin, DBE_LOG);
	    pdeadtime -> lamn = pdeadtime -> cmin;
	    post_total = 1;
	}
	delta = pdeadtime -> lamx - pdeadtime -> cmax;
	if (delta < 0.0)
	    delta = -delta;
	if ( (delta > pdeadtime -> adel) || (stat != pdeadtime ->stat) ||
	     (sevr != pdeadtime -> sevr) )
	{
	    db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> cmax, DBE_LOG);
	    pdeadtime -> lamx = pdeadtime -> cmax;
	    post_total = 1;
	}
    }
    if (post_total)
	db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> ctot, DBE_LOG);

      /* NOTE: the order of redQField calls must be as is, so that
               NUMP is the last in the buffer, therefore the last
               to be processed by redundancy. This is neccessary
               since a special is attached to the NUMP field that
               will update the buffers on the slave, IVAL, ISTA, ISEV
               must already have been processed
      */

    redQInit ((struct dbCommon *)pdeadtime, redbuf, sizeof (redbuf), type);

    if (stat != pdeadtime -> stat)
	redQField (redbuf, mylist [STAT]);

    if (sevr != pdeadtime -> sevr)
	redQField (redbuf, mylist [SEVR]);

    if (sdet != pdeadtime -> sdet)
	redQField (redbuf, mylist [SDET]);

    if (dtrc != pdeadtime -> dtrc)
	redQField (redbuf, mylist [DTRC]);

    if (TsCmpStampsNE (&latm, &pdeadtime -> latm) )
	redQField (redbuf, mylist [LATM]);
	
    if (pdeadtime -> nump > 0)
    {
	redQField (redbuf, mylist [ISTA]);
	redQField (redbuf, mylist [ISEV]);
	redQField (redbuf, mylist [IVAL]);
	redQField (redbuf, mylist [NUMP]);
    }

    if (mlst != pdeadtime -> mlst)
	redQField (redbuf, mylist [MLST]);

    if (lmin != pdeadtime -> lmin)
	redQField (redbuf, mylist [LMIN]);

    if (lmav != pdeadtime -> lmav)
	redQField (redbuf, mylist [LMAV]);

    if (lmcm != pdeadtime -> lmcm)
	redQField (redbuf, mylist [LMCM]);

    if (lmcx != pdeadtime -> lmcx)
	redQField (redbuf, mylist [LMCX]);

    redQSend (redbuf);

      /* process the forward scan link record */

    if (pdeadtime -> flnk.type == DB_LINK)
	dbScanPassive ((struct dbAddr *)pdeadtime -> flnk.value.db_link.pdbAddr);

    return (status);

} /* end func process () */


/****************************************************************************
 *
 *  MODULE NAME            : read_input_values 
 *  AUTHOR                 : emsteel
 *  DATE                   : 14 Sep 93
 *
 *  CALLING MODULE(S)      : process
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION: get VAL, STAT, SEVR from the input PV
 *
****************************************************************************/

static long read_input_values (pdeadtime)
   struct deadtimeRecord   *pdeadtime;
{
   long status    = 0;
   long nRequests = 1;
   long options   = 0;

   if ( (status = dbGetLink (&pdeadtime -> inp.value.db_link,
                           (struct dbCommon *)pdeadtime, DBR_DOUBLE,
                           (caddr_t)&pdeadtime -> ival,
                           &options, &nRequests) ) )
   {
      pdeadtime -> ival = 0;
      pdeadtime -> ista = LINK_ALARM;
      pdeadtime -> isev = VALID_ALARM;
      return (status);
   }
   if ((status = dbGetField (&pdeadtime -> instat_dbAddr, DBR_ENUM,
                            (caddr_t)&pdeadtime -> ista,
                            &options, &nRequests, NULL)) )
   {
      pdeadtime -> ista = LINK_ALARM;
      pdeadtime -> isev = VALID_ALARM;
      return (status);
   }
   if ((status = dbGetField (&pdeadtime -> insevr_dbAddr, DBR_ENUM,
                            (caddr_t)&pdeadtime -> isev,
                            &options, &nRequests, NULL)) )
   {
      pdeadtime -> ista = LINK_ALARM;
      pdeadtime -> isev = VALID_ALARM;
      return (status);
   }

   return (status);

} /* end func read_input_values () */



/****************************************************************************
 *
 *  MODULE NAME            : resolve_pv 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *  CALLING MODULE(S)      : init_record
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION: make sure there is an input pv and call dbNameToAddr to
 *               resolve inp.STAT and inp.SEVR
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 May 94
 *  REVISION:  check okToUse on inp before can init
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 10 Apr 95
 *  REVISION:	moved check on ok ToUse for inp to init_record()
 *
****************************************************************************/

static long resolve_pv (pdeadtime)
   struct deadtimeRecord   *pdeadtime;
{
    long status = 0;
    char pvname_fld[PVNAME_SZ + FLDNAME_SZ + 2];
    char errmsg[128];

    if (pdeadtime -> inp.type != DB_LINK)
    {
	recGblRecordError (S_db_badField, (struct dbCommon *)pdeadtime, 
                         "deadtime: resolve_pv, no input");
	return (S_db_badField);
    }

    strncpy (pvname_fld, pdeadtime -> inp.value.db_link.pvname, PVNAME_SZ);
    strcat (pvname_fld, ".STAT");
    if ((status = dbNameToAddr (pvname_fld, &pdeadtime -> instat_dbAddr)))
    {
	sprintf (errmsg, "deadtime, dbNameToAddr for %s", pvname_fld);
	errMessage (status, errmsg);
	return (status);
    }
    strncpy (pvname_fld, pdeadtime -> inp.value.db_link.pvname, PVNAME_SZ);
    strcat (pvname_fld, ".SEVR");
    if ((status = dbNameToAddr (pvname_fld, &pdeadtime -> insevr_dbAddr)))
    {
	sprintf (errmsg, "deadtime, dbNameToAddr for %s", pvname_fld);
	errMessage (status, errmsg);
	return (status);
    }

    return (status);

} /* end func resolve_pv () */



/****************************************************************************
 *
 *  MODULE NAME            : special 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Sep 93
 *
 *
 *  DESCRIPTION: special processing when the deadtime VAL field is manually
 *               chg'd
 *                   SPC_DT_VAL, .VAL has been manually updated, therefore
 *                   update current stat .STAT and current severity .SEVR
 *               special when redundant update
 *                   SPC_DT_NUMP, update DETB, STAB, SEVB with current
 *                   IVAL, ISTA, ISEV for NUMP times
 *               special when scan value (dets) changes, update sdet which 
 *                   is the working copy
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 Jan 94
 *  REVISION: check for !after
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Oct 94
 *  REVISION:	SPC_DT_VAL, since alarm status is chg'd, get current time
 *		and post val with alarm bit set, also update lalm (value
 *		at last alarm chg, since these fields are redundant if
 *		we are MASTER send to slave
 *
****************************************************************************/

static long special (paddr,after)
   struct    dbAddr    *paddr;
   int                 after;
{
    struct deadtimeRecord *pdeadtime =
                                 (struct deadtimeRecord *)(paddr -> precord);
    int			special_type = paddr -> special;
    char		redbuf[256];
    short		i;

    if (!after)
	return (0);

    switch (special_type)
    {
	case SPC_DT_VAL:
	    pdeadtime -> stat = MANUAL_OVERRIDE;
	    pdeadtime -> sevr = NO_ALARM;
	    tsLocalTime (&pdeadtime -> latm);
	    db_post_events ((struct dbCommon *)pdeadtime, 
				(caddr_t)&pdeadtime -> stat, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pdeadtime, 
				(caddr_t)&pdeadtime -> sevr, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pdeadtime, 
				(caddr_t)&pdeadtime -> val, DBE_VALUE | DBE_ALARM);
	    if (resGetCaAnswer (pdeadtime -> radr, &i) == RES_OK)
	    {
		if (i)
		{
		    redQInit ((struct dbCommon *)pdeadtime, redbuf, sizeof (redbuf), type);
		    redQField (redbuf, mylist [SEVR]);
		    redQField (redbuf, mylist [STAT]);
		    redQField (redbuf, mylist [LATM]);
		    redQSend (redbuf);
		}
	    }
	    break;

	case SPC_DT_NUMP:
	    pdeadtime -> tcks = tickGet();
	    update_buffers (pdeadtime);
	    break;

	case SPC_DT_DETS:
	    pdeadtime -> sdet = pdeadtime -> dets;
	    break;

	default:
	    recGblDbaddrError (S_db_badChoice, paddr, "deadtime: special");
	    return (S_db_badChoice);
    }

    return (0);

} /* end func special () */



/****************************************************************************
 *
 *  MODULE NAME            : update_buffers 
 *  AUTHOR                 : emsteel
 *  DATE                   : 14 Sep 93
 *
 *  CALLING MODULE(S)      : process, special
 *          ARGUMENT(S)    : pdeadtime - ptr to deadtime record
 *
 *  DESCRIPTION: update DETB, STAB, SAVB with input value IVAL, ISTA, ISEV
 *               if 1st scan fill entire buffer
 *
****************************************************************************/

static void update_buffers (pdeadtime)
   struct deadtimeRecord   *pdeadtime;
{
   unsigned short i;

   if (pdeadtime -> fpas == TRUE)
   {
      pdeadtime -> fpas = FALSE;
      for (i = 0; i < MAX_DELAY_VALUES; i++)
      {
         pdeadtime -> detb[i] = pdeadtime -> ival;
         pdeadtime -> stab[i] = pdeadtime -> ista;
         pdeadtime -> sevb[i] = pdeadtime -> isev;
      }
   }
   else
   {
      for (i = 0; i < pdeadtime -> nump; i++)
      {
         pdeadtime -> deti++;
         if (pdeadtime -> deti >= MAX_DELAY_VALUES)
            pdeadtime -> deti = 0;
         pdeadtime -> detb[pdeadtime -> deti] = pdeadtime -> ival;
         pdeadtime -> stab[pdeadtime -> deti] = pdeadtime -> ista;
         pdeadtime -> sevb[pdeadtime -> deti] = pdeadtime -> isev;
      }
      db_post_events ((struct dbCommon *)pdeadtime, (caddr_t)&pdeadtime -> deti, DBE_VALUE);
   }

   return;

} /* end func update_buffers () */

