/*recFanout.c,v 1.1.1.1 1999/08/03 14:44:19 tis4000 Exp */

/* recFanout.c - Record Support Routines for Fanout records */
/*
 *      Original Author: Bob Dalesio
 *      Current Author:  Marty Kraimer
 *      Date:            12-20-88
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  12-23-88        lrd     Alarm on locked MAX_LOCKED times
 * .02  05-03-89        lrd     removed process mask from arg list
 * .03  09-25-89        lrd     add conditional scanning
 * .04  01-21-90        lrd     unlock on scan disable exit
 * .05  04-19-90        lrd     user select disable on 0 or 1
 * .06  10-31-90	mrk	no user select disable on 0 or 1
 * .07  10-31-90	mrk	extensible record and device support
 * .08  01-04-94	ems	added init_record to set INIT = 1 and
 *				set ACK to 0
 *				do nothing in process() if not init'd
 * .21  03-01-94	ems	added support for process stats
 * .22	03-29-94	ems	make forward func defs agree with actual def
 * .23	08-29-94	ems	added get_control_double and get_graphic_double
 *				added dbRecDes.h for fldDes def
 * .24	10-12-94	ems	process(), set tsLocalTime when alarm chgs
 * .11	10-20-94	ems	chg time to latm (last alarm time) added
 *				added lptm (last process time)
 * .26	01-13-95	wep	init lptm and latm
 * .27	02-06-95	ems	set VAL to SELN and add posting and trending of VAL
 * .28	09-11-95	emds	archive val if alarm stat/sevr chg's, added selenct number
 *				test for DB_LINK
 */

#include	<vxWorks.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<string.h>

#include	"alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"dbRecDes.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"rec/fanoutRecord.h"
#include	"db_addr.h"
#include	"dbEvent.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/
#define report NULL
#define initialize NULL
static long init_record ();
static long process();
#define special NULL
#define get_value NULL
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
#define get_precision NULL
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
static long get_graphic_double ();
static long get_control_double ();
#define get_alarm_double NULL

struct rset fanoutRSET={
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double };

#define SELECT_ALL  0
#define SELECTED 1

static unsigned short type;	/* index to point type */


static long init_record (pfanout)
    struct fanoutRecord    *pfanout;
{
    char *pstr;

    for (type = 1; type < 100; type++)
    {
	pstr = GET_PRECTYPE (type);
	if (strncmp (pstr, "fanout", 6) == 0)
	   break;
    }
    pfanout -> init = 1;
    pfanout -> ini2 = 1;
    pfanout -> ack = 0;

    /* init last process time and last alarm time */
    if (pfanout -> lptm.secPastEpoch == 0)
        tsLocalTime (&pfanout -> lptm);
    if (pfanout -> latm.secPastEpoch == 0)
        tsLocalTime (&pfanout -> latm);

    pfanout -> val = (long)pfanout -> seln;

    return (0);
}


static long process(paddr)
    struct dbAddr	*paddr;
{
    struct fanoutRecord	*pfanout=(struct fanoutRecord *)(paddr->precord);
    short		stat,sevr,nsta,nsev;
    unsigned short	monitor_mask;


       /* if not init'd, do nothing */

    if ((!pfanout -> init) || (!pfanout -> ini2))
	return (0);

    /* get time stamp for last time record was processed */

    tsLocalTime (&pfanout -> lptm);

    /* update number of process times */
    if (process_stats)
       process_cnts[type]++;	

    pfanout->pact = TRUE;
    if(pfanout->selm==SELECT_ALL) 
    {
	if (pfanout->lnk1.type==DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pfanout->lnk1.value.db_link.pdbAddr);
	if (pfanout->lnk2.type==DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pfanout->lnk2.value.db_link.pdbAddr);
	if (pfanout->lnk3.type==DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pfanout->lnk3.value.db_link.pdbAddr);
	if (pfanout->lnk4.type==DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pfanout->lnk4.value.db_link.pdbAddr);
	if (pfanout->lnk5.type==DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pfanout->lnk5.value.db_link.pdbAddr);
	if (pfanout->lnk6.type==DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pfanout->lnk6.value.db_link.pdbAddr);
    } 
    else if(pfanout->selm==SELECTED) 
    {
	if( (pfanout->seln<1) || (pfanout->seln>6) )
	{
	    if(pfanout->nsev<VALID_ALARM) 
	    {
		pfanout->nsev = VALID_ALARM;
		pfanout->nsta = SOFT_ALARM;
	    }
	} 
	else 
	{
	    struct link *plink;

	    plink=&(pfanout->lnk1);
	    plink += (pfanout->seln-1);
	    if (plink -> type == DB_LINK)
		dbScanPassive ((struct dbAddr *)plink->value.db_link.pdbAddr);
	    else
	    {
	    	if(pfanout->nsev<VALID_ALARM) 
	    	{
		    pfanout->nsev = VALID_ALARM;
		    pfanout->nsta = SOFT_ALARM;
	    	}
	    }
	}
    } 
    else 
    {
	if(pfanout->nsev<VALID_ALARM) 
	{
	    pfanout->nsev = VALID_ALARM;
	    pfanout->nsta = SOFT_ALARM;
	}
    }

    pfanout->udf=FALSE;

    /* check monitors*/
    /* get previous stat and sevr  and new stat and sevr*/

    stat=pfanout->stat;
    sevr=pfanout->sevr;
    nsta=pfanout->nsta;
    nsev=pfanout->nsev;

    /*set current stat and sevr*/
    pfanout->stat = nsta;
    pfanout->sevr = nsev;
    pfanout->nsta = 0;
    pfanout->nsev = 0;

    if((stat!=nsta || sevr!=nsev))
    {
    	tsLocalTime(&pfanout->latm);
	db_post_events((struct dbCommon *)pfanout, (caddr_t)&pfanout->stat,DBE_VALUE);
	db_post_events((struct dbCommon *)pfanout, (caddr_t)&pfanout->sevr,DBE_VALUE);
    }

    monitor_mask = 0;
    if (pfanout -> val != pfanout -> seln)
    {
	monitor_mask = DBE_VALUE;
    }

    if (pfanout -> tenb)
    {
	if ( (pfanout -> val != pfanout ->seln) || (stat != nsta) ||
	     (sevr != nsev) )
	    monitor_mask |= DBE_LOG;
    }


    if (monitor_mask & (DBE_VALUE | DBE_LOG))
	pfanout -> val = pfanout ->seln;

    if (monitor_mask)
	db_post_events ((struct dbCommon *)pfanout, (caddr_t)&pfanout -> val, monitor_mask);

    pfanout->pact=FALSE;

    return(0);
}

static long get_graphic_double(paddr,pgd)

    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct fanoutRecord *)0) -> seln) ):
	case ( (int)&(((struct fanoutRecord *)0) -> val) ):
	    pgd -> upper_disp_limit = 6;
	    pgd -> lower_disp_limit = 1;
	    break;

	default:
	    pgd -> upper_disp_limit = 0;
	    pgd -> lower_disp_limit = 0;
	    break;
	   
    }
    return (0);
}

static long get_control_double(paddr,pcd)

    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct fanoutRecord *)0) -> seln) ):
	case ( (int)&(((struct fanoutRecord *)0) -> val) ):
	    pcd -> upper_ctrl_limit = 6;
	    pcd -> lower_ctrl_limit = 1;
	    break;

	default:
	    pcd -> upper_ctrl_limit = 0;
	    pcd -> lower_ctrl_limit = 0;
	    break;
    }
    return (0);
}
