/*recMbbi.c,v 1.1.1.1 1999/08/03 14:44:19 tis4000 Exp */

/* recMbbi.c - Record Support Routines for multi bit binary Input records */
/*
 *      Original Author: Bob Dalesio
 *      Current Author:  Marty Kraimer
 *      Date:            5-9-88
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  12-12-88        lrd     lock the record while processing
 * .02  12-15-88        lrd     Process the forward scan link
 * .03  12-23-88        lrd     Alarm on locked MAX_LOCKED times
 * .04  01-13-89        lrd     delete db_read_mbbi
 * .05  03-29-89        lrd     make hardware errors MAJOR
 *                              remove hw severity spec from database
 * .06  04-07-89        lrd     add monitor detection
 * .07  05-03-89        lrd     removed process mask from arg list
 * .08  05-29-89        lrd     support 16 states
 * .09  05-30-89        lrd     fixed masks for allen-bradley IO
 * .10  06-06-89        lrd     fixed AB mbbi conversion - signal wrong
 *                              added ability to enter raw numbers if no
 *                              states are defined - like the mbbo
 * .11  12-06-89        lrd     add database fetch support
 * .12  02-08-90        lrd     add Allen-Bradley PLC support
 * .13  10-31-90	mrk	changes for new record and device support
 * .14  03-12-93	ems	deleted ack from redundancy
 * .15  01-03-94	ems	set ack = 0 in init_record
 * .16  01-05-94	ems	added SPC_CNST to special func
 * .17  03-01-94	ems	added support for process stats
 * .18  03-29-94	ems	make forward func def agree with actual func def
 * .19	04-22-94	ems	added INI2 to namelist, chg NUMFLDS to NUMMBBIFLDS
 *				rewrite of init_record
 * .20	06-10-94	ems	move call to tsLocalTime before call to device
 *				if on scan, used to be in if NORMAL also
 * .21	06-13-94	ems	added support for maxiimize severity (msev)
 * .22	08-30-94	ems	monitor(), rm DBE_LOG from ack, only set DBE_VALUE
 *				on rval, only set DBE_LOG on val if tenb set
 * .23	10-13-94	ems	add TIME to redundancy, added fpps
 *				(1st process pass), do time stamp when stat or sevr chg,
 *				see monitor(),and not in process(),set lalm to val when 
 *				stat or sevr chg,in process() if fpps (first process)
 *				init lalm to val and if time stamp is 0, init it also
 *				added lval to replace lalm for COS alarm, since lalm
 *				has new meaning
 * .24	10-20-94	ems	chg time to latm (last alarm time) added 
 *				added lptm (last process time)
 * .25	01-13-95	wep	add mlst to redundancy, init_record(), init lptm and latm
 * .26	03-15-95	ems	monitor(), do not set ACK if sevr is EVENT_ALARM
 * .27	03-24-95	ems	arg to resGetCaAnswer needs to be a short not int
 * .28	03-29-95	ems	alarm(), COS_ALARM did return before setting lval
 * .29	09-11-95	ems	monitor(), archive when stat/sevr chg's also
 * .30	09-12-95	ems	monitor(), see mod for handling VALID_ALARM
 * .31	03-06-96	ems	added get_graphic_double and get_control_double
 * .32	04-11-96	ems	added code to support alarm delay
 * .33	06-19-96	ems	added code for alarm disable ADIS
 */

#include	<vxWorks.h>
#include	<inetLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<strLib.h>

#include	"alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/mbbiRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"mbbiredfldidx.h"
#include	"restable.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/

#define report NULL
#define initialize NULL
static long init_record ();
static long process ();
static long  special ();
static long get_value ();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
#define get_precision NULL
static long get_enum_str ();
static long get_enum_strs ();
static long put_enum_str ();
static long get_graphic_double ();
static long get_control_double ();
#define get_alarm_double NULL

struct rset mbbiRSET = {
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
	};

struct mbbidset {			/* multi bit binary input dset */
	long		number;
	DEVSUPFUN	dev_report;
	DEVSUPFUN	init;
	DEVSUPFUN	init_record;	/*returns: (-1,0)=>(failure,success)*/
	DEVSUPFUN	get_ioint_info;
	DEVSUPFUN	read_mbbi;	/*(0,1,2)=>(success, asyn, success no convert)*/
	};

/* The following definitions are required for redundancy */

static struct fldDes	**mylist = 0;

static char	*namelist [] = {
		"STAT",
		"SEVR",
		"VAL",
		"LALM",
		"INI2",
		"LATM",
		"LVAL",
		"MLST",
		"HHLL",
		"RTCT",
		0
		};

static	unsigned	short	type;

static void alarm ();
static void init_common ();
static void monitor ();


/*------------------------------------------------------------------------*/

static long init_record (pmbbi)

    struct mbbiRecord		*pmbbi;

{
    struct	mbbidset	*pdset;
    long			status;
    int				i;
    short			master;
    char			*pstr;
    char			redbuf [256];
    int				ini2_flag = 0;
    int				do_convert;

	if (!(mylist)) 
	{
	    for (type = 1; type < 100; type++) 
	    {
		pstr = GET_PRECTYPE (type);
		if (strcmp (pstr, "mbbi") == 0) 
		    break;
	    }
	    mylist = redFldAlloc (type, NUMMBBIFLDS);

	    if (!mylist)
	    {
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pmbbi,
				   "mbbi, init_record, mylist failed");
		return (S_rec_Alloc);
	    }

	    for (i = 0; i < NUMMBBIFLDS; i++) 
		redFldAdd (type, i, namelist [i]);
	}

        if (!(pdset = (struct mbbidset *) (pmbbi->dset))) 
	{
	    recGblRecordError (S_dev_noDSET, (struct dbCommon *)pmbbi, 
				"mbbi: init_record");
	    return (S_dev_noDSET);
        }

        /* must have read_mbbi function defined */

        if ( (pdset->number < 5) || (pdset->read_mbbi == NULL) ) 
	{
	    recGblRecordError (S_dev_missingSup, (struct dbCommon *)pmbbi, 
				"mbbi: init_record");
	    return (S_dev_missingSup);
        }

        /* initialize mask*/

        pmbbi->mask = 0;
        for (i = 0; i < pmbbi->nobt; i++) 
	{
	    pmbbi->mask <<= 1;		/* shift left 1 bit*/
	    pmbbi->mask |= 1;		/* set low order bit*/
        }
	init_common(pmbbi);

	pmbbi -> init = TRUE;
	pmbbi -> fpps = 1;
	pmbbi -> nsta = NO_ALARM;
	pmbbi -> nsev = NO_ALARM;
	pmbbi -> msev = NO_ALARM;

	/* init alarm delay variables */
	pmbbi -> larf = 0;
	pmbbi -> hhll = 0;
	pmbbi -> hilo = 0;
	pmbbi -> dvct = 0;
	pmbbi -> rtct = 0;

	/* init last process time and last alarm time */
	if (pmbbi -> lptm.secPastEpoch == 0)
	    tsLocalTime (&pmbbi -> lptm);
	if (pmbbi -> latm.secPastEpoch == 0)
	    tsLocalTime (&pmbbi -> latm);

	do_convert = 1;

	/* do not allow conversion if the board type is CONSTANT and input
	   is CONSTANT or REFERENCE, this was done in devSoft.c
	*/

        if (pmbbi -> dtyp <= 1)
	{
	    switch (pmbbi -> inp.type)
	    {
		case (CONSTANT):
		case (PV_LINK):
		case (DB_LINK):
		case (CA_LINK):
		    do_convert = 0;
		    break;
	    }
        }

        if ( pdset->init_record ) 
	{
	    status = (*pdset->init_record) (pmbbi);
	    if (status)
	    {
	        if ((!pmbbi -> tagd) && (pmbbi -> oscn != OFF_SCAN) &&
		    (pmbbi -> mode == NORMAL))
		{
		    pmbbi -> nsta = NO_INIT_ALARM;
		    pmbbi -> nsev = VALID_ALARM;
		}
		else
		    ini2_flag = 1;
	    }
	    else
	    {
		ini2_flag = 1;
		if (do_convert)
		{
		    unsigned long 	*pstate_values;
        	    short  		i;
		    unsigned long rval = pmbbi->rval;

		    if(pmbbi->shft > 0) 
		    {
		        rval >>= pmbbi->shft;
		    }
		    if (pmbbi->sdef) 
		    {
		        pstate_values = &(pmbbi->zrvl);
		        pmbbi->val = 65535;         /* initalize to unknown state*/
		        pmbbi->udf = TRUE;
		        for (i = 0; i < 16; i++) 
			{
			    if (*pstate_values == rval) 
			    {
                                pmbbi->val = i;
                                pmbbi->udf = FALSE;
                                break;
			    }
			    pstate_values++;
		        }
		    } 
		    else 
		    {
		        /* the raw value is the desired value */
		        pmbbi->val =  (unsigned short) rval;
		        pmbbi->udf =  FALSE;
		    }
		}
		else
		{
		    /* if our input is a constant and it has a value
		       > 15 then we are not going to let this block
		       init
		    */
		    if (pmbbi -> inp.type == CONSTANT)
		    {
			if (pmbbi -> val > 15)
			{
			    pmbbi -> nsta = NO_INIT_ALARM;
			    pmbbi -> nsev = VALID_ALARM;
			    ini2_flag = 0;
			}
		    }
		}
	    }
        }
	else
	    ini2_flag = 1;

	/* init last value, use by alarm() to test COS alarm */

	pmbbi -> lval = pmbbi -> val;

	if (resGetCaAnswer(pmbbi -> radr, &master) == RES_OK)
	{
	    if (master)
	    {
		/* post STAT, SEVR and VAL (monitor), if we will finally init, 
		   because we have read a value from the input device, set INI2,
		   send all redund fields to slave, when slave receives INI2, 
		   it will be fully init'd also
		*/

	    	/* check for maximize severity */

	    	if (pmbbi -> msev > pmbbi -> nsev)
	    	{
		    pmbbi -> nsev = pmbbi -> msev;
		    pmbbi -> nsta = LINK_ALARM;
	    	}

		monitor (pmbbi);

		if (ini2_flag)
		{
		    pmbbi -> ini2 = 1;

		    redQInit ((struct dbCommon *)pmbbi, redbuf, sizeof (redbuf), type);
		    redQField (redbuf, mylist [STAT]);
		    redQField (redbuf, mylist [SEVR]);
		    redQField (redbuf, mylist [VAL]);
		    redQField (redbuf, mylist [LALM]);
		    redQField (redbuf, mylist [INI2]);
		    redQField (redbuf, mylist [LVAL]);
		    redQSend (redbuf);
		}
	    }
	}
        return(0);
}

/*------------------------------------------------------------------------*/

static void init_common (pmbbi)

    struct mbbiRecord	*pmbbi;

{
    unsigned 	long 	*pstate_values;
    short  		i;

    /* determine if any states are defined */

    pstate_values = &(pmbbi->zrvl);
    pmbbi->sdef = FALSE;
    for (i = 0; i < 16; i++) 
    {
        if (*(pstate_values + i) != 0) 
	{
	    pmbbi->sdef = TRUE;
	    return;
	}
    }
    return;
}


/*------------------------------------------------------------------------*/

static long process (paddr)

    struct dbAddr	*paddr;
{
    struct mbbiRecord	*pmbbi = (struct mbbiRecord *) (paddr->precord);
    struct mbbidset	*pdset = (struct mbbidset *) (pmbbi->dset);
    char		redbuf [256];
    long		status = 0;
    unsigned short	stat   = pmbbi -> stat;
    unsigned short	sevr   = pmbbi -> sevr;
    unsigned short	val    = pmbbi -> val;
    unsigned short	lalm   = pmbbi -> lalm;
    unsigned short	lval   = pmbbi -> lval;
    TS_STAMP		latm   = pmbbi -> latm;
    double		mlst   = pmbbi -> mlst;
    short		hhll   = pmbbi -> hhll;
    short		rtct   = pmbbi -> rtct;

	if( (pdset==NULL) || (pdset->read_mbbi==NULL) ) 
	{
	    pmbbi->pact=TRUE;
	    recGblRecordError(S_dev_missingSup, (struct dbCommon *)pmbbi,"read_mbbi");
	    return(S_dev_missingSup);
	}

	if ((!pmbbi -> init) || (!pmbbi -> ini2))
		return (0);

	/* get time stamp for last time record was processed */

	tsLocalTime (&pmbbi -> lptm);

	if (pmbbi -> tagd)
	{
	    pmbbi -> nsta = TAGGED_ALARM;
	    pmbbi -> nsev = VALID_ALARM;
	}
        else if (pmbbi->oscn == OFF_SCAN) 
	{
            pmbbi->nsta = SCAN_ALARM;
	    pmbbi->nsev = VALID_ALARM;
        } 
	else 
	{
 	    pmbbi -> nsta = NO_ALARM;
	    pmbbi -> nsev = NO_ALARM;

	    /* update number of process times */
	    if (process_stats)
	       process_cnts[type]++;	

	    if (pmbbi -> mode == NORMAL)
	    {
	        status = (*pdset->read_mbbi) (pmbbi); /* read the new value */
	        pmbbi->pact = TRUE;
	        if (status == 1) 
		    return(0);
	        
	        if (status == -2) 
		{
		    pmbbi->pact = FALSE;
		    return (0);
	        }
	        if (status == 0) 
		{ /* convert the value */
		    
		    /* if boardtype and input are not CONSTANT */

		    if ((pmbbi -> dtyp > 1) && (pmbbi -> inp.type != CONSTANT))
		    {
    
        	        unsigned long 	*pstate_values;
        	        short  		i;
		        unsigned long rval = pmbbi->rval;

		        if(pmbbi->shft > 0) 
		        {
		            rval >>= pmbbi->shft;
		        }
		        if (pmbbi->sdef) 
		        {
		            pstate_values = &(pmbbi->zrvl);
		            pmbbi->val = 65535;         /* initalize to unknown state*/
		            pmbbi->udf = TRUE;
		            for (i = 0; i < 16; i++) 
			    {
			        if (*pstate_values == rval) 
				{
                                    pmbbi->val = i;
                                    pmbbi->udf = FALSE;
                                    break;
			        }
			        pstate_values++;
		            }
		        } 
		        else 
		        {
		            /* the raw value is the desired value */
		            pmbbi->val =  (unsigned short) rval;
		            pmbbi->udf =  FALSE;
		        }
		    }
	        } 
		else if (status == 2) 
		{
		    status = 0;
	        }

	        /* check for alarms */

	    }
	    alarm (pmbbi);

	    /* check for maximize severity */

	    if (pmbbi -> msev > pmbbi -> nsev)
	    {
		pmbbi -> nsev = pmbbi -> msev;
		pmbbi -> nsta = LINK_ALARM;
	    }
	}

	/* if the time stamp is 0, init and post val with
	   alarm bit set, this makes sure that proper alarm
	   messages are set if IOC was down and alarm status
	   may have chg'd
	*/
	if (pmbbi -> latm.secPastEpoch == 0)
	{
	    tsLocalTime (&pmbbi -> latm);
	    db_post_events ((struct dbCommon *)pmbbi, (caddr_t)&pmbbi -> val, DBE_VALUE | DBE_ALARM);
	}

	/* if it is the 1st process pass 
	   init last value when alarm chg'd (lalm)
	*/

	if (pmbbi -> fpps)
	{
	    pmbbi -> fpps = 0;
	    pmbbi -> lalm = pmbbi -> val;
	}

	/* check event list */

	monitor(pmbbi);

	redQInit ((struct dbCommon *)pmbbi, redbuf, sizeof (redbuf), type);

	if (stat != pmbbi->stat) 
	    redQField (redbuf, mylist [STAT]);
	
	if (sevr != pmbbi->sevr) 
	    redQField (redbuf, mylist [SEVR]);
	
	if (val != pmbbi->val) 
	    redQField (redbuf, mylist [VAL]);
	
	if (lalm != pmbbi->lalm) 
	    redQField (redbuf, mylist [LALM]);
	
	if (TsCmpStampsNE (&latm, &pmbbi -> latm) )
	    redQField (redbuf, mylist [LATM]);

	if (lval != pmbbi -> lval)
	    redQField (redbuf, mylist [LVAL]);
	
	if (mlst != pmbbi -> mlst)
	    redQField (redbuf, mylist [MLST]);
	
	if (hhll != pmbbi -> hhll)
	    redQField (redbuf, mylist [HHLL]);
	
	if (rtct != pmbbi -> rtct)
	    redQField (redbuf, mylist [RTCT]);
	
	redQSend (redbuf);

	/* process the forward scan link record */

	if (pmbbi->flnk.type == DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pmbbi->flnk.value.db_link.pdbAddr);
	
	pmbbi->pact = FALSE;
	return (status);
}


/*------------------------------------------------------------------------*/

static long special(paddr,after)
    struct dbAddr *paddr;
    int           after;
{
    struct mbbiRecord     *pmbbi = (struct mbbiRecord *)(paddr->precord);
    int                 special_type = paddr->special;
    static unsigned short save_cnst;

    /* if dtyp and inp are not CONSTANT, do not let them
       update CNST
    */

    if(!after) 
    {  
	save_cnst = pmbbi -> cnst;
	return(0);
    }

    switch(special_type) 
    {

	case(SPC_MOD):
	    init_common(pmbbi);
	    return(0);

	case(SPC_CNST):
	    if ((pmbbi -> dtyp <= 1) && (pmbbi -> inp.type == CONSTANT))
            {
	        pmbbi -> inp.value.value = pmbbi -> cnst;
                pmbbi -> val = pmbbi -> cnst;
            }
	    else
	        pmbbi -> cnst = save_cnst;
            return(0);

	default:
            recGblDbaddrError(S_db_badChoice,paddr,"mbbi: special");
            return(S_db_badChoice);
    }
}


/*------------------------------------------------------------------------*/

static long get_value(pmbbi,pvdes)
    struct mbbiRecord		*pmbbi;
    struct valueDes	*pvdes;
{
    pvdes->field_type = DBF_ENUM;
    pvdes->no_elements=1;
    (unsigned short *)(pvdes->pvalue) = &pmbbi->val;
    return(0);
}

/*------------------------------------------------------------------------*/

static long get_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char	  *pstring;
{
    struct mbbiRecord	*pmbbi=(struct mbbiRecord *)paddr->precord;
    char		*psource;
    unsigned short      *field = (unsigned short *)paddr -> pfield;

    if((*field) <= 15) 
    {
	psource = (pmbbi->zrst);
	psource += ((*field) * sizeof(pmbbi->zrst));
	strncpy(pstring,psource,sizeof(pmbbi->zrst));
    } 
    else 
    {
	strcpy(pstring,"Illegal Value");
    }
    return(0);
}


/*------------------------------------------------------------------------*/

static long get_enum_strs(paddr,pes)
    struct dbAddr *paddr;
    struct dbr_enumStrs *pes;
{
    struct mbbiRecord	*pmbbi=(struct mbbiRecord *)paddr->precord;
    char		*psource;
    int			i;
    short		no_str;

    no_str = 0;
    bzero((char *)(&pes->strs[0]),sizeof(pes->strs));
    for(i=0,psource=(pmbbi->zrst); i<16; i++, psource += sizeof(pmbbi->zrst) ) 
    {
	strncpy(pes->strs[i],psource,sizeof(pmbbi->zrst));
	if(*psource!=0) no_str=i+1;
    }
    pes->no_str=no_str;
    return(0);
}

/*------------------------------------------------------------------------*/

static long put_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char          *pstring;
{
    struct mbbiRecord     *pmbbi=(struct mbbiRecord *)paddr->precord;
    char                  *pstate_name;
    short                 i;
    unsigned short        *field = (unsigned short *)paddr -> pfield;

    if (pmbbi->sdef)
    {
        pstate_name = pmbbi->zrst;
        for (i = 0; i < 16; i++)
	{
            if(strncmp(pstate_name,pstring,sizeof(pmbbi->zrst))==0)
	    {
                *field = (unsigned short)i;
                return(0);
            }
            pstate_name += sizeof(pmbbi->zrst);
        }
    }
    return(S_db_badChoice);
}


/*------------------------------------------------------------------------*/

static long get_graphic_double (paddr, pgd)

    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {

	case ( (int)&(((struct mbbiRecord *)0) -> aldl) ):
	    pgd -> upper_disp_limit = 9999;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    pgd -> upper_disp_limit = 15;
	    pgd -> lower_disp_limit = 0;
	    break;
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static long get_control_double (paddr, pcd)

    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct mbbiRecord *)0) -> aldl) ):
	    pcd -> upper_ctrl_limit = 9999;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	default:
	    pcd -> upper_ctrl_limit = 15;
	    pcd -> lower_ctrl_limit = 0;
	    break;
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static void alarm (pmbbi)
    struct mbbiRecord	*pmbbi;
{
	unsigned short *severities;
	unsigned short	val = pmbbi->val;
	int		adis = 0;

        /* check for alarm inhibit */

	adis = AlarmDisableLink ((void *)pmbbi);
        if ( (pmbbi->aenb == ALARM_DISABLED) || (adis != 0) )
	{
	    if (pmbbi -> nsta != NO_INIT_ALARM)
	    {
                pmbbi->nsta = INHIBIT_ALARM;
                pmbbi->nsev = NO_ALARM;
	    }
            return;
        }

        /* check for udf alarm */

        if (pmbbi->udf == TRUE )
	{
            if (pmbbi->nsev < VALID_ALARM) 
	    {
                pmbbi->nsta = UDF_ALARM;
                pmbbi->nsev = VALID_ALARM;
            }
        }

	if (pmbbi -> nsev == VALID_ALARM)
		return;

        /* check for  state alarm */
        /* unknown state */

        if (val > 15)
	{
            if (pmbbi->nsev < pmbbi->unsv) 
	    {
                pmbbi->nsta = STATE_ALARM;
                pmbbi->nsev = pmbbi->unsv;
            }
        } 
	else 
	{
	    /* in a state which is an error */

            severities = (unsigned short *) &(pmbbi->zrsv);
            if (pmbbi->nsev < severities [val]) 
	    {
                pmbbi->nsta = STATE_ALARM;
                pmbbi->nsev = severities [val];
            }
	}

	/* check for time to output STATE alarm */

	AlarmStates ((void *)pmbbi, 0);

        /* check for cos alarm */

	if (val != pmbbi -> lval)
	{
	    if (pmbbi -> nsev < pmbbi -> cosv) 
	    {
		pmbbi -> nsta = COS_ALARM;
		pmbbi -> nsev = pmbbi -> cosv;
	    }
	}
	pmbbi -> lval = val;

	/* check for return to normal */

	AlarmStates ((void *)pmbbi, 1);

	return;
}


/*------------------------------------------------------------------------*/

static void monitor (pmbbi)
    struct mbbiRecord	*pmbbi;
{
	unsigned short	monitor_mask;
	unsigned short	*severities;
        short           stat,sevr,nsta,nsev;
	int		set_ack = 1;

        /* get previous stat and sevr  and new stat and sevr*/

        stat = pmbbi->stat;
        sevr = pmbbi->sevr;
        nsta = pmbbi->nsta;
        nsev = pmbbi->nsev;

        /*set current stat and sevr*/

        pmbbi->stat = nsta;
        pmbbi->sevr = nsev;
        pmbbi->nsta = 0;
        pmbbi->nsev = 0;

	/* if we are coming out of VALID_ALARM and our new stat/sevr are
	   the same as when we went into VALID_ALARM, do not set ack
	*/
	if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	     (nsta == pmbbi -> psta) && (nsev == pmbbi -> psev) )
	{
	    set_ack = 0;
	}
	/* if we are going into VALID_ALARM, and out previous stat was not
	   VALID_ALARM, remember previous non VALID_ALARM stat/sevr
	   and we do not want to set ACK,
	*/
	if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
	{
	    set_ack = 0;
	    pmbbi -> psta = stat;
	    pmbbi -> psev = sevr;
	    db_post_events ((struct dbCommon *)pmbbi, (caddr_t)&pmbbi -> psta, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pmbbi, (caddr_t)&pmbbi -> psev, DBE_VALUE);
	}

	monitor_mask = 0;

        /* alarm condition changed this scan */
	/*
	EXTRA PATCH: If the user defines two states with the same alarm
	severity and then monitors .VAL with mask DBE_ALARM, then he'll
	never see a transition. This patch makes sure that he (i.e., the
	alarm manager) will see the transition.
	*/

	severities = (unsigned short *) &(pmbbi -> zrsv);
	if (((nsta == STATE_ALARM) || (nsta == COS_ALARM)) && 
	    (pmbbi -> mlst != pmbbi -> val) )
		stat = ~nsta;

        if ((stat != nsta) || (sevr != nsev)) 
	{
            /* post events for alarm condition change*/

            monitor_mask = DBE_ALARM;

	    /* when alarm status chg's, get current time and
	       save VAL in lalm (last value when alarm chg'd)
	    */
	    tsLocalTime (&pmbbi -> latm);
	    pmbbi -> lalm = pmbbi -> val;

            /* post stat and sevr fields */

            db_post_events ((struct dbCommon *)pmbbi, (caddr_t)&pmbbi->stat, DBE_VALUE);
            db_post_events ((struct dbCommon *)pmbbi, (caddr_t)&pmbbi->sevr, DBE_VALUE);
            if ((pmbbi->sevr != NO_ALARM) && (set_ack) &&
		(pmbbi->sevr != EVENT_ALARM) ) 
	    {
                if (pmbbi->ack == 0)
		{
                    pmbbi->ack = 1;           /* new unacknowledged alarm */
                    db_post_events ((struct dbCommon *)pmbbi, (caddr_t)&pmbbi->ack, DBE_VALUE);
                }
            }
        }
        /* check for value change */
        if (pmbbi->mlst != pmbbi->val)
	{
            /* post events for value change and archive change */

            monitor_mask |= DBE_VALUE;
        }

	if (pmbbi -> tenb)
	{
	    /* if VAL has changed or there has been a change in alarm status */
	    if ( (pmbbi -> mlst != pmbbi -> val) || (monitor_mask & DBE_ALARM) )
		monitor_mask |= DBE_LOG;
	}

	if (monitor_mask & (DBE_VALUE | DBE_LOG))
	    pmbbi -> mlst = pmbbi -> val;

        /* send out monitors connected to the value field */
        if (monitor_mask)
            db_post_events((struct dbCommon *)pmbbi, (caddr_t)&pmbbi->val,monitor_mask);
	
        if(pmbbi->oraw!=pmbbi->rval) 
	{
            db_post_events((struct dbCommon *)pmbbi, (caddr_t)&pmbbi->rval,DBE_VALUE);
            pmbbi->oraw = pmbbi->rval;
        }
        return;
}
