/*recMbbo.c,v 1.1.1.1 1999/08/03 14:44:19 tis4000 Exp */

/* recMbbo.c - Record Support Routines for multi bit binary Output records */
/*
 *      Original Author: Bob Dalesio
 *      Current Author:  Marty Kraimer
 *      Date:            7-17-87
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  11-28-88        lrd     add desired value fetched
 * .02  12-12-88        lrd     lock the record while processing
 * .03  12-15-88        lrd     Process the forward scan link
 * .04  12-23-88        lrd     Alarm on locked MAX_LOCKED times
 * .05  01-09-89        lrd     Fix direct value outputs so that they shift the
 *                              value into the correct bit position
 * .06  01-13-89        lrd     delete db_write_mbbo and db_write_mbbo
 * .07  01-20-89        lrd     fixed vx inlcudes
 * .08  03-03-89        lrd     add supervisory/closed loop control
 * .09  03-29-89        lrd     make hardware errors MAJOR
 *                              remove hw severity spec from database
 * .10  04-07-89        lrd     add monitor service
 * .11  05-03-89        lrd     removed process mask from arg list
 * .12  05-30-89        lrd     fixed mask for allen-bradley IO
 * .13  01-05-90        joh,lrd fixed write_mbbo() to set rval
 * .14  02-08-90        lrd/cr  fixed the mbbo to read at initialization for
 *                              Allen-Bradley and added PLC support
 * .15  04-11-90        lrd     make locals static
 * .16  10-11-90	mrk	make changes for new record and device support
 * .17  03-12-93	ems	deleted ack from redundancy
 * .18  01-03-94	ems	set ack to 0 in init_record
 *				in process, if not init'd do nothing
 *				  commented out check for init before write, 
 *				  not needed
 * .19  01-05-94	ems	added SPC_CNST to special func
 * .20  03-01-94	ems	added support for process stats
 * .21	02-29-94	ems	make forward func def agree with actual func def
 * .22	04-22-94	ems	added INI2 to namelist, chg NUMFLDS to NUMMBBOFLDS
 *				rewrite init_record
 * .23	06-10-94	ems	move call to tsLocalTime before call to device if
 *				on scan and !pact
 * .24	06-13-94	ems	added support for maximize severity (msev)
 * .25	08-30-94	ems	monitor(), rm DBE_LOG from ack, only set DBE_VALUE
 *				for rval and rbv, only set DBE_LOG on val if tenb set
 * .26	10-13-94	ems	add TIME to redundancy, added fpps
 *				(1st process pass), do time stamp when stat or sevr chg,
 *				see monitor(),and not in process(),set lalm to val when 
 *				stat or sevr chg,in process() if fpps (first process)
 *				init lalm to val and if time stamp is 0, init it also
 *				added lval to replace lalm for COS alarm, since lalm
 *				has new meaning
 * .27	10-20-94	ems	chg time to latm (last alarm time) added 
 *				added lptm (last process time)
 * .28	01-13-95	wep	add mlst to redundancy, init_record(), init lptm and latm
 * .29	03-15-95	ems	monitor(), do not set ACK if sevr is EVENT_ALARM
 * .30	03-24-95	ems	arg to resGetCaAnswer needs to be a short not int
 * .31	03-29-95	ems	alarm(), COS_ALARM did return before setting lval
 * .32	09-11-95	ems	monitor(), archive when stat/sevr chg's also
 * .33	09-12-95	ems	monitor(), see mod for handling VALID_ALARM
 * .34	01-31-96	ems	added support for output on chg only OCHG
 * .35	03-06-96	ems	added get_control_double() and get_graphic_double()
 * .36	05-23-96	ems	monitor(), chg && to || when checking nsta for STATE and
 *				COS
 * .37	06-19-96	ems	added code for alarm disable ADIS
 */

#include	<vxWorks.h>
#include	<inetLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<strLib.h>

#include	"alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/mbboRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"mbboredfldidx.h"
#include	"restable.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */ 
extern int process_stats;

/* Create RSET - Record Support Entry Table*/

#define report NULL
#define initialize NULL
static long init_record ();
static long process ();
static long special ();
static long get_value ();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
#define get_precision NULL
static long get_enum_str ();
static long get_enum_strs ();
static long put_enum_str ();
static long get_graphic_double ();
static long get_control_double ();
#define get_alarm_double NULL

struct rset mbboRSET = {
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
	};

struct mbbodset {			/* multi bit binary output dset */
	long		number;
	DEVSUPFUN	dev_report;
	DEVSUPFUN	init;
	DEVSUPFUN	init_record; 	/*returns: (0,2)=>(success,success no convert)*/
	DEVSUPFUN	get_ioint_info;
	DEVSUPFUN	write_mbbo;	/*(0,1)=>(success,don't Continue*/
	DEVSUPFUN	readback_mbbo;
	};

/* The following definitions are required for redundancy */

static struct	fldDes	**mylist = 0;

static char	*namelist [] = {
		"STAT",
		"SEVR",
		"VAL",
		"LALM",
		"INI2",
		"LATM",
		"LVAL",
		"MLST",
		"LOVL",
		"HHLL",
		"RTCT",
		0
		};

static	unsigned	short	type;

static void alarm ();
static void init_common ();
static void monitor ();
static void test_ochg ();
static void mbboReadback ();


/*------------------------------------------------------------------------*/

static long init_record (pmbbo)

    struct	mbboRecord	*pmbbo;
{
    struct	mbbodset	*pdset;
    long			status = 0;
    int				i;
    short			master;
    char			*pstr;
    char			redbuf [256];
    int				ini2_flag = 0;
    int				do_convert;

	if (!(mylist)) 
	{
	    for (type = 1; type < 100; type++) 
	    {
		pstr = GET_PRECTYPE (type);
		if (strcmp (pstr, "mbbo") == 0) 
		    break;
	    }
	    mylist = redFldAlloc (type, NUMMBBOFLDS);

	    if (!mylist)
	    {
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pmbbo,
				   "mbbo, init_record, mylist failed");
		return (S_rec_Alloc);
	    }

	    for (i = 0; i < NUMMBBOFLDS; i++) 
		redFldAdd (type, i, namelist [i]);
	}

	if (!(pdset = (struct mbbodset *) (pmbbo->dset))) 
	{
	    recGblRecordError (S_dev_noDSET, (struct dbCommon *)pmbbo, 
						"mbbo: init_record");
	    return (S_dev_noDSET);
        }

	/* must have write_mbbo function defined */

	if ( (pdset->number < 5) || (pdset->write_mbbo == NULL) ) 
	{
	    recGblRecordError (S_dev_missingSup, (struct dbCommon *)pmbbo, "mbbo: init_record");
	    return (S_dev_missingSup);
        }

        /* initialize mask*/

        pmbbo->mask = 0;
        for (i = 0; i < pmbbo->nobt; i++) 
	{
            pmbbo->mask <<= 1;		/* shift left 1 bit*/
            pmbbo->mask |= 1;		/* set low order bit*/
        }
        init_common (pmbbo);
	pmbbo -> init = TRUE;
	pmbbo -> fpps = 1;
	pmbbo -> nsta = NO_ALARM;
	pmbbo -> nsev = NO_ALARM;
	pmbbo -> msev = NO_ALARM;

	/* init alarm delay variables */
	pmbbo -> larf = 0;
	pmbbo -> hhll = 0;
	pmbbo -> hilo = 0;
	pmbbo -> dvct = 0;
	pmbbo -> rtct = 0;

	/* init readback variables */
	pmbbo -> rbfn = mbboReadback;

	/* init last process time and last alarm time */
	if (pmbbo -> lptm.secPastEpoch == 0)
	    tsLocalTime (&pmbbo -> lptm);
	if (pmbbo -> latm.secPastEpoch == 0)
	    tsLocalTime (&pmbbo -> latm);

	do_convert = 1;

	/* do not allow conversion if the board type is CONSTANT and output
	   is CONSTANT or REFERENCE, this was done in devSoft.c
	*/

        if (pmbbo -> dtyp <= 1)
	{
	    switch (pmbbo -> out.type)
	    {
		case (CONSTANT):
		case (PV_LINK):
		case (DB_LINK):
		case (CA_LINK):
		    do_convert = 0;
		    break;
	    }
        }

        
        if ( pdset->init_record ) 
	{
	    unsigned long rval;

	    status = (*pdset->init_record) (pmbbo, process);

	    if (status)
	    {
		pmbbo -> nsta = NO_INIT_ALARM;
		pmbbo -> nsev = VALID_ALARM;
	    }
	    else
	    {
		ini2_flag = 1;
		pmbbo->udf = FALSE;
		if (do_convert)
		{
		    rval = pmbbo -> rval;
		    if (pmbbo->shft > 0) 
		    {
			rval >>= pmbbo->shft;
		    }
		    if (pmbbo->sdef) 
		    {
			unsigned long   *pstate_values = &(pmbbo -> zrvl);
			short           i;
		    
		        pmbbo->val = 65535;        /* initalize to unknown state*/
		        for (i = 0; i < 16; i++) 
			{
		    	    if (*pstate_values == rval) 
			    {
			        pmbbo->val = i;
			        break;
			    }
		    	    pstate_values++;
		        }
		    }
		    else 
		    {
			/* the raw  is the desired val */

			pmbbo->val =  (unsigned short)rval;
		    }
		}
		else
		{
		    /* do not init if our output is a CONSTANT or
		       our output is a REFERNECE and our input is a
		       CONSTANT, and the value of the CONSTANT > 15
		    */
		    if ( (pmbbo -> out.type == CONSTANT) ||
			 ( (pmbbo -> out.type == DB_LINK) &&
			   (pmbbo -> dol.type == CONSTANT) ) )
		    {
			if (pmbbo -> val > 15)
			{
			    pmbbo -> nsta = NO_INIT_ALARM;
			    pmbbo -> nsev = VALID_ALARM;
			    ini2_flag = 0;
			}
		    }
		}
	    }
        }
	else
	    ini2_flag = 1;

	/* init last value, use by alarm() to test COS alarm */

	pmbbo -> lovl = pmbbo -> lval = pmbbo -> val;

	if (resGetCaAnswer(pmbbo -> radr, &master) == RES_OK)
	{
	    if (master)
	    {
		/* post STAT, SEVR and VAL (monitor), if we will finally init, 
		   because we have read a value from the input device, set INI2,
		   send all redund fields to slave, when slave receives INI2, 
		   it will be fully init'd also
		*/

		alarm (pmbbo);

	    	/* check for maximize severity */

	    	if (pmbbo -> msev > pmbbo -> nsev)
	    	{
		    pmbbo -> nsev = pmbbo -> msev;
		    pmbbo -> nsta = LINK_ALARM;
	    	}

		monitor (pmbbo);

		if (ini2_flag)
		{
		    pmbbo -> ini2 = 1;

		    redQInit ((struct dbCommon *)pmbbo, redbuf, sizeof (redbuf), type);
		    redQField (redbuf, mylist [STAT]);
		    redQField (redbuf, mylist [SEVR]);
		    redQField (redbuf, mylist [VAL]);
		    redQField (redbuf, mylist [LALM]);
		    redQField (redbuf, mylist [INI2]);
		    redQField (redbuf, mylist [LVAL]);
		    redQField (redbuf, mylist [LOVL]);
		    redQSend (redbuf);
		}
	    }
	}
        return (status);
}


/*------------------------------------------------------------------------*/

static void init_common (pmbbo)

    struct	mbboRecord	*pmbbo;
{
    unsigned	long	*pstate_values;
    short           	i;

        /* determine if any states are defined */

        pstate_values = &(pmbbo->zrvl);
        pmbbo->sdef = FALSE;
        for (i = 0; i < 16; i++) 
	{
            if (*(pstate_values + i) != 0) 
	    {
                pmbbo->sdef = TRUE;
                return;
            }
        }
}


/*------------------------------------------------------------------------*/

static long process (paddr)
    struct dbAddr	*paddr;
{
    struct mbboRecord	*pmbbo = (struct mbboRecord *) (paddr->precord);
    struct mbbodset	*pdset = (struct mbbodset *) (pmbbo->dset);
    char		redbuf [256];
    long		status = 0;
    unsigned short	stat   = pmbbo -> stat;
    unsigned short	sevr   = pmbbo -> sevr;
    unsigned short	val    = pmbbo -> val;
    unsigned short	lalm   = pmbbo -> lalm;
    unsigned short	lval   = pmbbo -> lval;
    TS_STAMP		latm   = pmbbo -> latm;
    double		mlst   = pmbbo -> mlst;
    unsigned short	lovl   = pmbbo -> lovl;
    short		hhll   = pmbbo -> hhll;
    short		rtct   = pmbbo -> rtct;
    int			write_output = 1;

	   /* if not init'd do nothing */

	if ((!pmbbo -> init) || (!pmbbo -> ini2))
	    return (status);

	if((pdset == NULL) || (pdset->write_mbbo == NULL)) 
	{
	    pmbbo->pact = TRUE;
	    recGblRecordError (S_dev_missingSup, (struct dbCommon *)pmbbo, "write_mbbo");
	    return (S_dev_missingSup);
	}
	/* get time stamp for last time record was processed */

	tsLocalTime (&pmbbo -> lptm);

	pmbbo -> msev = NO_ALARM;

	if (pmbbo -> tagd)
	{
	    pmbbo -> nsta = TAGGED_ALARM;
	    pmbbo -> nsev = VALID_ALARM;
	}
        else if (pmbbo->oscn == OFF_SCAN) 
	{
            pmbbo->nsta = SCAN_ALARM;
	    pmbbo->nsev = VALID_ALARM;
	} 
	else 
	{
	    pmbbo -> nsta = NO_ALARM;
	    pmbbo -> nsev = NO_ALARM;

	    if (!pmbbo->pact) 
	    {
	        /* update number of process times */
	        if (process_stats)
	           process_cnts[type]++;	

		if ((pmbbo->dol.type == DB_LINK) && (pmbbo->omsl == CLOSED_LOOP)) 
		{
	            long options = 0;
	            long nRequest = 1;
	            long status;
	            unsigned short val;

	            pmbbo->pact = TRUE;
	            status = dbGetLink (&pmbbo->dol.value.db_link, 
				        (struct dbCommon *)pmbbo,
					DBR_USHORT, (caddr_t)&val, 
					&options, &nRequest);
	            pmbbo->pact = FALSE;
	            if (status == 0) 
		    {
		        pmbbo->val= val;
		        pmbbo->udf= FALSE;
	            } 
		    else 
		    {
		        if (pmbbo->nsev < VALID_ALARM) 
			{
		            pmbbo->nsev = VALID_ALARM;
		            pmbbo->nsta = LINK_ALARM;
		        }
		        goto DONT_WRITE;
	            }
	        }
	        if(pmbbo->udf == TRUE) 
		{
		    if(pmbbo->nsev < VALID_ALARM) 
		    {
		        pmbbo->nsev = VALID_ALARM;
		        pmbbo->nsta = UDF_ALARM;
		    }
		    goto DONT_WRITE;
	        }
	        if(pmbbo->sdef) 
		{
	            unsigned long *pvalues = &(pmbbo->zrvl);

	            if (pmbbo->val > 15) 
		    {
		        if(pmbbo->nsev < VALID_ALARM ) 
			{
		            pmbbo->nsta = SOFT_ALARM;
		            pmbbo->nsev = VALID_ALARM;
		        }
		        goto DONT_WRITE;
	            }
	            pmbbo->rval = pvalues[pmbbo->val];
	        } 
		else 
		{
		    pmbbo->rval = (unsigned long) (pmbbo->val);
	        }
	        if(pmbbo->shft > 0) 
		{
		    pmbbo->rval <<= pmbbo->shft;
	        }
	    }

	    test_ochg (pmbbo, &write_output);
	    if (write_output)
	    {
		status = (*pdset->write_mbbo) (pmbbo); /* write the new value */
		pmbbo -> lovl = pmbbo -> val;
	    }
DONT_WRITE:
            pmbbo->pact = TRUE;

            /* status is one if an asynchronous record is being processed*/

            if(status == 1) 
	        return (0);
	    
	    if (status == -2) 
	    {
		pmbbo->pact = FALSE;
		return (0);
	    }

            /* check for alarms */

            alarm ( pmbbo);

	    /* check for maximize severity */

	    if (pmbbo -> msev > pmbbo -> nsev)
	    {
		pmbbo -> nsev = pmbbo -> msev;
		pmbbo -> nsta = LINK_ALARM;
	    }
	}

	/* if the time stamp is 0, init and post val with
	   alarm bit set, this makes sure that proper alarm
	   messages are set if IOC was down and alarm status
	   may have chg'd
	*/
	if (pmbbo -> latm.secPastEpoch == 0)
	{
	    tsLocalTime (&pmbbo -> latm);
	    db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo -> val, DBE_VALUE | DBE_ALARM);
	}

	/* if it is the 1st process pass 
	   init last value when alarm chg'd (lalm)
	*/

	if (pmbbo -> fpps)
	{
	    pmbbo -> fpps = 0;
	    pmbbo -> lalm = pmbbo -> val;
	}

        /* check event list */

        monitor (pmbbo);

	redQInit ((struct dbCommon *)pmbbo, redbuf, sizeof (redbuf), type);

	if (stat != pmbbo->stat) 
	    redQField (redbuf, mylist [STAT]);
	
	if (sevr != pmbbo->sevr) 
	    redQField (redbuf, mylist [SEVR]);
	
	if (val != pmbbo->val) 
	    redQField (redbuf, mylist [VAL]);
	
	if (lalm != pmbbo->lalm) 
	    redQField (redbuf, mylist [LALM]);
	
	if (TsCmpStampsNE (&latm, &pmbbo -> latm) )
	    redQField (redbuf, mylist [LATM]);

	if (lval != pmbbo -> lval)
	    redQField (redbuf, mylist [LVAL]);
	
	if (mlst != pmbbo -> mlst)
	    redQField (redbuf, mylist [MLST]);
	
	if (lovl != pmbbo -> lovl)
	    redQField (redbuf, mylist [LOVL]);
	
	if (hhll != pmbbo -> hhll)
	    redQField (redbuf, mylist [HHLL]);
	
	if (rtct != pmbbo -> rtct)
	    redQField (redbuf, mylist [RTCT]);
	
	redQSend (redbuf);

        /* process the forward scan link record */

        if(pmbbo->flnk.type == DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pmbbo->flnk.value.db_link.pdbAddr);
	
        pmbbo->pact = FALSE;
        return (status);
}


/*------------------------------------------------------------------------*/

static long special (paddr,after)
    struct dbAddr *paddr;
    int           after;
{
    struct mbboRecord		*pmbbo = (struct mbboRecord *)(paddr->precord);
    int				special_type = paddr->special;
    short			master;
    static unsigned short	save_cnst;

    /* if dtyp and out are not CONSTANT, do not let them
       update CNST
    */

    if(!after) 
    {
	save_cnst = pmbbo -> cnst;
	return(0);
    }

    switch(special_type) 
    {

	case(SPC_MOD):
            init_common(pmbbo);
            break;

	case(SPC_CNST):
	    if ((pmbbo -> dtyp <= 1) && (pmbbo -> out.type == CONSTANT))
            {
	        pmbbo -> out.value.value = pmbbo -> cnst;
                pmbbo -> rval = pmbbo -> val = pmbbo -> cnst;
            }
	    else
	        pmbbo -> cnst = save_cnst;
            break;

	case SPC_MBBO_VAL:
	    /* if output chg is set and I am in SUPERVISORY mode 
	       (NOTE: dol!=link and omsl=closed_loop acts like supervisory), and 
	       my output channel is not a reference, and my resource is MASTER , 
	       set LOVL to this value, this will ensure that the VAL the user 
	       input will be output
	    */
	    if ( (pmbbo -> ochg == 0) || 
		 ( (pmbbo -> dol.type == DB_LINK) && (pmbbo -> omsl == CLOSED_LOOP) ) ||
		 ( (pmbbo -> dtyp <= 1) && (pmbbo -> out.type == DB_LINK) ) )
		break;
	    if (resGetCaAnswer (pmbbo -> radr, &master) == RES_OK)
	    {
		if (master)
		{
		    pmbbo -> lovl = 20;
		}
	    }
	    break;

	default:
            recGblDbaddrError(S_db_badChoice,paddr,"mbbo: special");
            return(S_db_badChoice);
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static long get_value(pmbbo,pvdes)
    struct mbboRecord		*pmbbo;
    struct valueDes	*pvdes;
{
    pvdes->field_type = DBF_ENUM;
    pvdes->no_elements=1;
    (unsigned short *)(pvdes->pvalue) = &pmbbo->val;
    return(0);
}

/*------------------------------------------------------------------------*/

static long get_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char	  *pstring;
{
    struct mbboRecord	*pmbbo=(struct mbboRecord *)paddr->precord;
    char		*psource;
    unsigned short      *field = (unsigned short *)paddr->pfield;

    if((*field) <= 15)
    {
	psource = (pmbbo->zrst);
	psource += ((*field) * sizeof(pmbbo->zrst));
	strncpy(pstring,psource,sizeof(pmbbo->zrst));
    } 
    else 
    {
	strcpy(pstring,"Illegal Value");
    }
    return(0);
}


/*------------------------------------------------------------------------*/

static long get_enum_strs(paddr,pes)
    struct dbAddr *paddr;
    struct dbr_enumStrs *pes;
{
    struct mbboRecord	*pmbbo=(struct mbboRecord *)paddr->precord;
    char		*psource;
    int			i;
    short		no_str;

    no_str=0;
    bzero((char *)pes->strs,sizeof(pes->strs));
    for(i=0,psource=(pmbbo->zrst); i<16; i++, psource += sizeof(pmbbo->zrst) ) 
    {
	strncpy(pes->strs[i],psource,sizeof(pmbbo->zrst));
  	if(*psource!=0)no_str=i+1;
    }
    pes->no_str = no_str;

    return(0);
}

/*------------------------------------------------------------------------*/

static long put_enum_str(paddr,pstring)
    struct dbAddr *paddr;
    char          *pstring;
{
    struct mbboRecord     *pmbbo=(struct mbboRecord *)paddr->precord;
    char                  *pstate_name;
    short                 i;
    unsigned short        *field = (unsigned short *)paddr->pfield;

    if (pmbbo->sdef)
    {
        pstate_name = pmbbo->zrst;
        for (i = 0; i < 16; i++)
	{
            if(strncmp(pstate_name,pstring,sizeof(pmbbo->zrst))==0)
	    {
                *field = (unsigned short)i;
                return(0);
            }
            pstate_name += sizeof(pmbbo->zrst);
        }
    }
    return(S_db_badChoice);
}


/*------------------------------------------------------------------------*/

static long get_graphic_double (paddr, pgd)

    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {

	case ( (int)&(((struct mbboRecord *)0) -> aldl) ):
	case ( (int)&(((struct mbboRecord *)0) -> rbdt) ):
	    pgd -> upper_disp_limit = 9999;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    pgd -> upper_disp_limit = 15;
	    pgd -> lower_disp_limit = 0;
	    break;
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static long get_control_double (paddr, pcd)

    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct mbboRecord *)0) -> aldl) ):
	case ( (int)&(((struct mbboRecord *)0) -> rbdt) ):
	    pcd -> upper_ctrl_limit = 9999;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	default:
	    pcd -> upper_ctrl_limit = 15;
	    pcd -> lower_ctrl_limit = 0;
	    break;
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static void alarm (pmbbo)
    struct mbboRecord	*pmbbo;
{
	unsigned short *severities;
	unsigned short	val=pmbbo->val;
	int		adis = 0;

        /* check for alarm inhibit */

        if (pmbbo->stat == MANUAL_ALARM) 
	    return;
	
	adis = AlarmDisableLink ((void *)pmbbo);
        if ( (pmbbo->aenb == ALARM_DISABLED) || (adis != 0) )
	{
	    if (pmbbo -> nsta != NO_INIT_ALARM)
	    {
                pmbbo->nsta = INHIBIT_ALARM;
                pmbbo->nsev = NO_ALARM;
	    }
            return;
        }

	if (pmbbo -> nsev == VALID_ALARM)
	    return;

        /* check for  state alarm */
        /* unknown state */

        if (val > 15)
	{
            if (pmbbo->nsev < pmbbo->unsv) 
	    {
                pmbbo->nsta = STATE_ALARM;
                pmbbo->nsev = pmbbo->unsv;
            }
        } 
	else 
	{
     	    /* in a state which is an error */

      	    severities = (unsigned short *) &(pmbbo->zrsv);
            if (pmbbo->nsev < severities [val])
	    {
                pmbbo->nsta = STATE_ALARM;
                pmbbo->nsev = severities [val];
            }
	}

	/* check for output of STATE alarm */

	AlarmStates ((void *)pmbbo, 0);

        /* check for cos alarm */

	if (val != pmbbo -> lval)
	{
	    if (pmbbo -> nsev < pmbbo -> cosv) 
	    {
		pmbbo -> nsta = COS_ALARM;
		pmbbo -> nsev = pmbbo -> cosv;
	    }
	}

	pmbbo -> lval = val;

	/* check for return to normal */

	AlarmStates ((void *)pmbbo, 1);

	return;
}


/*------------------------------------------------------------------------*/

static void monitor (pmbbo)
    struct mbboRecord	*pmbbo;
{
	unsigned short	monitor_mask;
	unsigned short	*severities;
        short           stat,sevr,nsta,nsev;
	int		set_ack = 1;

        /* get previous stat and sevr  and new stat and sevr*/

        stat = pmbbo->stat;
        sevr = pmbbo->sevr;
        nsta = pmbbo->nsta;
        nsev = pmbbo->nsev;

        /*set current stat and sevr*/

        pmbbo->stat = nsta;
        pmbbo->sevr = nsev;
        pmbbo->nsta = 0;
        pmbbo->nsev = 0;

	monitor_mask = 0;

	/* if we are coming out of VALID_ALARM and our new stat/sevr are
	   the same as when we went into VALID_ALARM, do not set ack
	*/
	if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	     (nsta == pmbbo -> psta) && (nsev == pmbbo -> psev) )
	{
	    set_ack = 0;
	}
	/* if we are going into VALID_ALARM, and out previous stat was not
	   VALID_ALARM, remember previous non VALID_ALARM stat/sevr
	   and we do not want to set ACK,
	*/
	if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
	{
	    set_ack = 0;
	    pmbbo -> psta = stat;
	    pmbbo -> psev = sevr;
	    db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo -> psta, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo -> psev, DBE_VALUE);
	}

        /* alarm condition changed this scan */

	/*
	Extra Patch! If the user makes two of his states have
	the same severity and then monitors VAL with DBE_ALARM,
	he'll never see the state change from one to the other.
	This is the way that the alarm handler works. Make sure
	that such transitions get seen.

	Extra Extra Patch! If the user only enables COS alarming
	on passibe points, we will get the first transition and
	then never get events posted on any others because stat
	and nsta will always be COS and sevr will also never
	change. This fixes that problem.
	*/

	severities = (unsigned short *) &(pmbbo -> zrsv);
	if (((nsta == STATE_ALARM) || (nsta == COS_ALARM)) &&
	    (pmbbo -> mlst != pmbbo -> val))
		stat = ~nsta;

        if ((stat != nsta) || (sevr != nsev)) 
	{
            /* post events for alarm condition change*/

            monitor_mask = DBE_ALARM;

	    /* when alarm status chg's, get current time and
	       save VAL in lalm (last value when alarm chg'd)
	    */
	    tsLocalTime (&pmbbo -> latm);
	    pmbbo -> lalm = pmbbo -> val;

            /* post stat and sevr fields */

            db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo->stat, DBE_VALUE);
            db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo->sevr, DBE_VALUE);
            if ((pmbbo->sevr != NO_ALARM) && (set_ack) &&
		(pmbbo->sevr != EVENT_ALARM) ) 
	    {
                if (pmbbo->ack == 0) 
		{
                    pmbbo->ack = 1;           /* new unacknowledged alarm */
                    db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo->ack, DBE_VALUE);
                }
            }
        }
        /* check for value change */

        if (pmbbo->mlst != pmbbo->val) 
	{
            /* post events for value change and archive change */

            monitor_mask |= DBE_VALUE;
        }

	if (pmbbo -> tenb)
	{
	    /* if VAL has changed or there has been a change in alarm status */
	    if ( (pmbbo -> mlst != pmbbo -> val) || (monitor_mask & DBE_ALARM) )
		monitor_mask |= DBE_LOG;
	}

	if (monitor_mask & (DBE_VALUE | DBE_LOG))
	    pmbbo -> mlst = pmbbo -> val;

        /* send out monitors connected to the value field */

        if (monitor_mask) 
            db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo->val, monitor_mask);
	
        if (pmbbo->oraw != pmbbo->rval) 
	{
            db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo->rval, DBE_VALUE);
            pmbbo->oraw = pmbbo->rval;
        }
        if(pmbbo->orbv != pmbbo->rbv) 
	{
            db_post_events ((struct dbCommon *)pmbbo, (caddr_t)&pmbbo->rbv, DBE_VALUE);
            pmbbo->orbv = pmbbo->rbv;
        }
        return;
}


/*------------------------------------------------------------------------*/

static void test_ochg (pmbbo, write_output)
    struct mbboRecord	*pmbbo;
    int			*write_output;
{
    long		status;
    long		options = 0;
    long		nRequests = 1;
    unsigned short	lovl;

    if (pmbbo -> ochg == 0)
	return;

    /* if my output is a reference, then get its value, and set lovl
       (last val output) to the refs value. this protects us from the
       case where the output may be a field that has been changed
       from a graphic
    */
    if ( (pmbbo -> dtyp <= 1) && (pmbbo -> out.type == DB_LINK) )
    {
	status = dbGetLink (&pmbbo -> out.value.db_link, 
			    (struct dbCommon *)pmbbo, DBR_ENUM,
			    (caddr_t)&lovl, 
			    &options, &nRequests);
	if (status)
	{
	    *write_output = 0;
	    pmbbo -> nsev = VALID_ALARM;
	    pmbbo -> nsta = LINK_ALARM;
	    return;
	}
	pmbbo -> lovl = lovl;
    }
    if (pmbbo -> lovl == pmbbo -> val)
	*write_output = 0;
    return;
}


/*------------------------------------------------------------------------*/

static void	mbboReadback (struct mbboRecord *pmbbo)
{
    struct mbbodset	*pdset;
    unsigned long	rval = pmbbo -> rval;
    long		status;

    pdset = (struct mbbodset *)pmbbo -> dset;

    /* no use if we don't have the ability to readback the output */
    if (pdset -> readback_mbbo == NULL)
	return;

    status = (*pdset -> readback_mbbo)(pmbbo);
    if (status == 0)
    {
	/* if there was a chg */
	if (rval != pmbbo -> rval)
	{
	    unsigned short	stat = pmbbo -> stat;
	    unsigned short	sevr = pmbbo -> sevr;
	    unsigned short	val  = pmbbo -> val;
	    unsigned short	lalm = pmbbo -> lalm;
	    unsigned short	lval = pmbbo -> lval;
	    TS_STAMP		latm = pmbbo -> latm;
	    double		mlst = pmbbo -> mlst;
	    short		hhll = pmbbo -> hhll;
	    short		rtct = pmbbo -> rtct;
	    short		i;
	    unsigned long	*pstate_values = &(pmbbo -> zrvl);
	    char		redbuf [256];

	    /* update last process time */
	    tsLocalTime (&pmbbo -> lptm);

	    rval = pmbbo -> rval;
	    if (pmbbo -> shft)
		rval >>= pmbbo -> shft;
	    if (pmbbo -> sdef)
	    {
		/* init to unknown value */
		pmbbo -> val = 65535;
		for (i = 0; i < 16; i++)
		{
		    if (*pstate_values == rval)
		    {
			pmbbo -> val = i;
			break;
		    }
		    pstate_values++;
		}
	    }
	    else
	    {
		/* use raw value */
		pmbbo -> val = (unsigned short)rval;
	    }

	    alarm (pmbbo);
	    monitor (pmbbo);

	    /* update redundant fields */
	    redQInit ((struct dbCommon *)pmbbo, redbuf, sizeof (redbuf), type);

	    if (stat != pmbbo->stat) 
		redQField (redbuf, mylist [STAT]);
	
	    if (sevr != pmbbo->sevr) 
		redQField (redbuf, mylist [SEVR]);
	
	    if (val != pmbbo->val) 
		redQField (redbuf, mylist [VAL]);
	
	    if (lalm != pmbbo->lalm) 
		redQField (redbuf, mylist [LALM]);
	
	    if (TsCmpStampsNE (&latm, &pmbbo -> latm) )
		redQField (redbuf, mylist [LATM]);

	    if (lval != pmbbo -> lval)
		redQField (redbuf, mylist [LVAL]);
	
	    if (mlst != pmbbo -> mlst)
		redQField (redbuf, mylist [MLST]);
	
	    if (hhll != pmbbo -> hhll)
		redQField (redbuf, mylist [HHLL]);
	
	    if (rtct != pmbbo -> rtct)
		redQField (redbuf, mylist [RTCT]);
	
	    redQSend (redbuf);
	}
    }

    return;
}

