/*recPipe.c,v 1.1.1.1 1999/08/03 14:44:20 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : recPipe.c 
 *  AUTHOR                 : emsteel
 *  DATE                   : 16 Mar 93
 *
 *
 *  DESCRIPTION: record support routines for Pipe records
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 23 Feb 94
 *  REVISION:  removed INIT, MLST, ALST from namelist and pipered, 
 *             not needed, also removed time from pipered, not needed
 *             only db_post_events if MASTER, in some routines the init flag
 *             is checked before posting, if set we are called from process
 *             and we are MASTER.
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 02 Mar 94
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 94
 *  REVISION:  make forward func def agree with actual func def
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 May 94
 *  REVISION:	rewrite init_record, move all db_post_events to new func
 *		post_events(), pass cmd fields to redundant update
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 13 June 94
 *  REVISION:	added support for maximize severity (msev), move call to 
 *		tsLocalTime inside on scan
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 31 Aug 94
 *  REVISION:	added get_graphic_double() and get_control_double() and
 *		get_link_value()
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 13 Oct 94
 *  REVISION:	add TIME to redundancy, added fpps
 *		(1st process pass), do time stamp when stat or sevr chg,
 *		see monitor(),and not in process(),set lalm to val when 
 *		stat or sevr chg,in process() if fpps (first process)
 *		init lalm to val and if time stamp is 0, init it also
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Oct 94
 *  REVISION:	chg time to latm (last alarm time)
 *		added lptm (last process time)
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 13 Jan 95
 *  REVISION:	add mlst to redundancy, init latm and lptm in init_record
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 21 Mar 95
 *  REVISION:	update QVOL with NETV not VAL
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 Jun 96
 *  REVISION:	added code for alarm disable ADIS
 *
****************************************************************************/

/****************************************************************************
*
* Programmer's Notes:
*
* ComputeTFVolume() - see tankfarm.c
* FindStrappingTable () - see tankfarm.c
* FindAPITable () - see tankfarm.c
*
****************************************************************************/


#include <vxWorks.h>
#include <inetLib.h>
#include <tickLib.h>          /* for tickGet () */
#include <types.h>
#include <stdioLib.h>
#include <lstLib.h>
#include <string.h>

#include "alarm.h"
#include "cvtTable.h"
#include "dbDefs.h"
#include "dbCommon.h"
#include "link.h"
#include "db_field_log.h"
#include "dbAccess.h"
#include "dbFldTypes.h"
#include "dbRecType.h"
#include "devSup.h"
#include "errMdef.h"
#include "recSup.h"
#include "special.h"
#include "rec/pipeRecord.h"
#include "redflds.h"
#include "redque.h"
#include "piperedfldidx.h"
#include "tsDefs.h"          
#include "tankfarm.h"
#include "restable.h"
#include "db_addr.h"
#include "recGbl.h"
#include "dbEvent.h"
#include "okToUse.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table */

#define report NULL
#define initialize NULL
static long init_record();
static long process();
static long special();
static long get_value();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
static long get_precision();
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
static long get_graphic_double ();
static long get_control_double ();
#define get_alarm_double NULL

struct rset pipeRSET = {

    RSETNUMBER,
    report,
    initialize,
    init_record,
    process,
    special,
    get_value,
    cvt_dbaddr,
    get_array_info,
    put_array_info,
    get_units,
    get_precision,
    get_enum_str,
    get_enum_strs,
    put_enum_str,
    get_graphic_double,
    get_control_double,
    get_alarm_double
    };


/* The folowing definitions are for redundancy */
 
static struct fldDes **mylist = 0;

   /* NOTE: the order and spelling in namelist must be the
            same as in piperedfldidx.h
   */

static char *namelist [] = {
               "STAT",
               "SEVR",
               "VAL",
               "PSTS",
               "PISV",
               "TTIM",
               "QVOL",
               "LALM",
               "ESOU",
               "TVAL",
	       "INI2",
	       "CMDS",
	       "CRUN",
	       "CSTP",
	       "SRUN",
	       "SSTP",
	       "PCMD",
	       "LATM",
	       "MLST",
               0
               };

/* Following are strictly local */

static   unsigned short type;       /* index to pipe type */

/* structure to save vals for redundancy check, except
   for time, used to store time value before updated
   in process ()
*/

struct pipered 
{
    unsigned short	stat;
    unsigned short	sevr;
    double		val;
    char		psts[18];
    short		pisv;
    double		ttim;
    double		qvol;
    double		lalm;
    unsigned long	tcks;
    short		esou;
    short		tval;
    double		cpre;
    double		ctem;
    double		gcap;
    double		ullg;
    double		netv;
    char		pcmd[18];
    char		cmds[18];
    short		crun;
    short		cstp;
    short		srun;
    short		sstp;
    short		ini2;
    TS_STAMP		latm;
    double		mlst;
};


/* pipe commands
   NOTE: these must have these values to
         coincide with pipeRecord.ascii
*/

#define  PIPE_RUN_CMD      1
#define  PIPE_STOP_CMD     2

/* pipe operatinal state */

#define  PIPE_RUNNING        0
#define  PIPE_STOPPED        1


static void alarm ();
static void compute_some_values ();
static void get_link_value ();
static void monitor ();
static void post_events ();
static long read_pressure_pv ();
static long read_temperature_pv ();
static void run_commands ();
static void save_pipe ();
static void update_redun ();
static void update_qvol ();


/****************************************************************************
 *
 *  MODULE NAME            : init_record 
 *  AUTHOR                 : emsteel
 *  DATE                   : 18 Mar 93
 *
 *  ARGUMENT(S)            : ppipe - ptr to pipe record  
 *
 *  DESCRIPTION: initialize a pipe record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 14 dec 93
 *  REVISION:  add init of PCMD, TF_INTI_CMD is defined in
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 03 Jan 94
 *  REVISION:  set ack to 0
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 23 Feb 94
 *  REVISION: put db_post_events inside redundancy check, only post if 
 *            MASTER, remove ack = 0
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 May 94
 *  REVISION: init ttim to -1 not 0, 0 will cause pipe to start timing atfter
 *		init, init qvol to val
 *
****************************************************************************/

static long init_record (ppipe)
    struct pipeRecord	*ppipe;

{
    long		status = 0;
    unsigned short	i;
    char		*pstr;
    int			ini2_flag = 0;
    struct pipered	pipe_old;

    if (!(mylist))
    {
         /* find record type for pipe */
        for (type = 1, i = 0; type < 100; type++)
        {
            pstr = GET_PRECTYPE (type);     
            if (strncmp (pstr, "pipe", 4) == 0)
            {
                i = 1;
                break;
            }
        }

        if (i == 0)
        {
            recGblRecordError (S_db_noRecType, (struct dbCommon *)ppipe, "pipe:init_record");
            return (S_db_noRecType);
         }

         /* allocate memory for mylist */
        mylist = redFldAlloc (type, NUMPIPEFLDS);

	if (!mylist)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)ppipe,
			   "pipe, init_record, mylist failed");
  	  return (S_rec_Alloc);
 
	}

         /* add fields in namelist to mylist */
        for (i = 0; i < NUMPIPEFLDS; i++)
            redFldAdd (type, i, namelist [i]);
    }

      /* now find the strapping table for this record 
         but 1st make sure ptr to strapping table is NULL */

    ppipe -> pstr = NULL;
    status = FindStrappingTable (ppipe -> sttn, &ppipe -> pstr, (int *)&ppipe -> snum); 
    if (status)
	return (status);

      /* now find the API table for this record
         but 1st make sure ptr to API tables is NULL */

    ppipe -> papi = NULL;
    status =FindAPITable (ppipe -> apit, &ppipe -> papi, (int *)&ppipe -> anum);
    if (status)
	return (status);

	/* save old pipe status */
    save_pipe (ppipe, &pipe_old);

      /* init some stuff */

    bzero (ppipe -> cmds, sizeof (ppipe -> cmds)); /* command string */
    ppipe -> crun = 0;		/* run digital cmd */
    ppipe -> cstp = 0;		/* stop digital cmd */
    ppipe -> srun = 0;		/* special run cmd */
    ppipe -> sstp = 0;		/* special stop cmd */
    ppipe -> cpre = 0;		/* current pressure in pipe */
    ppipe -> ctem = 0;		/* current temperature in pipe */
    ppipe -> qvol = 0;		/* vol in pipe when stopped */
    ppipe -> gcap = 0;		/* gross pipe capacity */
    ppipe -> netv = 0;		/* net pipe volume */
    ppipe -> ullg = 0;		/* ullage */
    ppipe -> ttim = -1;		/* timer */
    ppipe -> tcks = 0;		/* current system ticks */
    ppipe -> esou = 0;		/* emergency stop init'd */
    ppipe -> prec = 2;		/* display precision */
    ppipe -> tval = 0;		/* pipe is in volume alarm condition */
				/* init previous cmd */
    strncpy (ppipe -> pcmd, TF_INIT_CMD, sizeof (ppipe -> pcmd));

    ppipe -> init = TRUE;
    ppipe -> fpps = 1;
    ppipe -> nsta = NO_ALARM;
    ppipe -> nsev = NO_ALARM;
    ppipe -> msev = NO_ALARM;
    /* init last alarm time and last process time */
    if (ppipe -> latm.secPastEpoch == 0)
	tsLocalTime (&ppipe -> latm);
    if (ppipe -> lptm.secPastEpoch == 0)
	tsLocalTime (&ppipe -> lptm);

      /* set pipe ops state to stopped */

    strncpy ( ppipe -> psts, ppipe -> stps, sizeof (ppipe -> psts));
    ppipe -> pisv = PIPE_STOPPED;

      /* read the pressure sensor */

    status = read_pressure_pv (ppipe);
    if (!status)
    {
	/* read the temperature sensor */

	status = read_temperature_pv (ppipe);
	if (!status)
	{
	    compute_some_values (ppipe);

	    /* init quiescent volume to pipe's current net volume
	       since the pipe is stopped
	    */
	    ppipe -> qvol = ppipe -> netv;

	    ini2_flag = 1;
	}
    }
    if (status)
    {
	ppipe -> nsta = NO_INIT_ALARM;
	ppipe -> nsev = VALID_ALARM;
    }

      /* do redundancy if not in slave mode */

    if (resGetCaAnswer (ppipe -> radr, &i) == RES_OK)
    {
        if (i)
        {
	    /* check for maximize severity */

	    if (ppipe -> msev > ppipe -> nsev)
	    {
		ppipe -> nsev = ppipe -> msev;
		ppipe -> nsta = LINK_ALARM;
	    }

	    monitor (ppipe);

	    if (ini2_flag)
	    {
		ppipe -> ini2 = 1;
		post_events (ppipe, &pipe_old);
		update_redun (&pipe_old, ppipe);
	    }
        }
    }

    return (0);

} /* end func init_records () */


/****************************************************************************
 *
 *  MODULE NAME            : alarm 
 *  AUTHOR                 : emsteel
 *  DATE                   : 25 Mar 93
 *
 *  CALLING MODULE(S)      : process ()
 *  ARGUMENT(S)            : ppipe
 *
 *  DESCRIPTION: set alarm status
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Mar 94
 *  REVISION: alarm chgs, if val is past alarm setting, set that
 *            alarm condition, else if val was in that alarm and
 *            it is still in hyst range of alarm value, then stay
 *            in alarm
 *       
****************************************************************************/

static void alarm (ppipe)
     struct pipeRecord  *ppipe;
{
   double	delta;
   int		emergency_stop = 0;
   int		adis = 0;

      /* check for inhibited alarms */

   adis = AlarmDisableLink ((void *)ppipe);
   if ( (ppipe -> aenb == ALARM_DISABLED) || (adis != 0) )
   {
      ppipe -> nsta = INHIBIT_ALARM;
      ppipe -> nsev = NO_ALARM;
      return;
   }

   if (ppipe -> udf == TRUE)
   {
      if (ppipe -> nsev < VALID_ALARM)
      {
         ppipe -> nsta = UDF_ALARM;
         ppipe -> nsev = VALID_ALARM;
      }
      return;
   }

   if (ppipe -> nsev == VALID_ALARM)
      return;

      /* if the pipe is stopped and we are not timing to set
         the quiescent volume, check for alarm condition
      */

   if (ppipe -> pisv == PIPE_STOPPED)
   {
      if (ppipe -> ttim < 0.0)
      {
            /* stopped volume - current net volume */
      
         delta = ppipe -> qvol - ppipe -> netv;
         if (delta < 0.0)
            delta = -delta;
      
            /* alarm if volume change is > than alarm limit or if
               was in VOLUME alarm and still within hyst of alarm
               value, stay in alarm 
            */
      
         if ( (delta > ppipe -> vold) ||
              ( (ppipe -> stat == VOLUME) &&
                (delta > (ppipe -> vold - ppipe -> hyst)) ) )
         {
            if (ppipe -> nsev < ppipe -> alsv)
            {
               ppipe -> nsta = VOLUME;
               ppipe -> nsev = ppipe -> alsv;
            }
               /* set emergency shutdown indicator if allowed */
            if (ppipe -> iems)
            {
               emergency_stop = 1;
            }

               /* reset quiescent volume and start timing again
                  to see if the pipe is still changing volume
                  by more than vold, also set tval to indicate
                  the pipe is in a volume alarm condition
               */

            ppipe -> qvol = ppipe -> netv;
            ppipe -> ttim = 0;
            ppipe -> tval = 1;
         }
         else
         {
            ppipe -> tval = 0;
         }
      }
         /* if the pipe was in a volume alarm, continue the alarm
            while we are timing to settle time
         */

      else
      {
         if (ppipe -> tval)
         {
            if (ppipe -> nsev < ppipe -> alsv)
            {
               ppipe -> nsta = VOLUME;
               ppipe -> nsev = ppipe -> alsv;
            }
            if (ppipe -> iems)
            {
               emergency_stop = 1;
            }
         }
      }
   }

   if (emergency_stop)
      ppipe -> esou = 1;
   else
      ppipe -> esou = 0;

   return;

} /* end func alarm () */



/****************************************************************************
 *
 *  MODULE NAME            : compute_some_values 
 *  AUTHOR                 : emsteel
 *  DATE                   : 14 May 93
 *
 *  CALLING MODULE(S)      : init_record, process
 *          ARGUMENT(S)    : ppipe - ptr to pipe record
 *
 *  DESCRIPTION:
 *     1 - set gross capacity (gcap) to top volume in strapping table
 *     2 - uillage (ullg) = gross capacity (gcap) - current volume (val)
 *     3 - look up thermal expansion value (texp) in API table
 *     4 - net volume (netv) = current volume (val) * thermal expansion (texp)
 *
****************************************************************************/

static void compute_some_values (ppipe)
       struct pipeRecord   *ppipe;
{
   struct strappingValue   *stbl;
   long                    status = 0;

      /* if there is a strapping table */

   if (ppipe -> pstr)
   {
      stbl = (struct strappingValue *)ppipe -> pstr;
      ppipe -> gcap = (stbl + (ppipe -> snum - 1)) -> volume;

      ppipe -> ullg = ppipe -> gcap - ppipe -> val;
      if (ppipe -> ullg < 0.0)
	 ppipe -> ullg = 0.0;
   }

      /* if there is an API table */

   if (ppipe -> papi)
   {
      if ((status = FindAPIcf (ppipe -> papi, ppipe -> anum, ppipe -> ctem,
                               ppipe -> sgrv, &ppipe -> texp, ppipe -> name,
                               "TMPU", ppipe -> tmpu)) == 0)
      {
         ppipe -> netv = ppipe -> val * ppipe -> texp;
      }
   }

   return;

} /* end func compute_some_values () */


/****************************************************************************
 *
 *  MODULE NAME            : get_control_double 
 *  AUTHOR                 : emsteel
 *  DATE                   : 31 Aug 94
 *
 *
 *  DESCRIPTION: return control limits
 *
****************************************************************************/

static long get_control_double(paddr,pcd)
    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    struct   pipeRecord     *ppipe = (struct pipeRecord *) paddr->precord;
    struct   strappingValue *stbl;
    double                  top_strap_vol = 0;
    double		    bottom_strap_vol = 0;
    float		    hopr;
    float		    lopr;

    if (ppipe -> pstr)
    {
	stbl = (struct strappingValue *)ppipe -> pstr;
	top_strap_vol = (stbl + (ppipe -> snum - 1)) -> volume;
	bottom_strap_vol = stbl -> volume;
    }

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct pipeRecord *)0) -> val) ):
	case ( (int)&(((struct pipeRecord *)0) -> gcap) ):
	case ( (int)&(((struct pipeRecord *)0) -> netv) ):

	    /* if there is a strapping table, limit is bottom and top entry */
	    if (ppipe -> pstr)
	    {
		pcd -> upper_ctrl_limit = top_strap_vol;
		pcd -> lower_ctrl_limit = bottom_strap_vol;
	    }
	    else
	    {
		pcd -> upper_ctrl_limit = 1000000;
		pcd -> lower_ctrl_limit = 0;
	    }
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> vold) ):
	case ( (int)&(((struct pipeRecord *)0) -> adel) ):
	case ( (int)&(((struct pipeRecord *)0) -> mdel) ):
	case ( (int)&(((struct pipeRecord *)0) -> ullg) ):
	case ( (int)&(((struct pipeRecord *)0) -> hyst) ):

	    if (ppipe -> pstr)
	    {
		pcd -> upper_ctrl_limit = top_strap_vol;
		pcd -> lower_ctrl_limit = 0;
	    }
	    else
	    {
		if (ppipe -> gcap > 0)
		{
		    pcd -> upper_ctrl_limit = ppipe -> gcap;
		    pcd -> lower_ctrl_limit = 0;
		}
		else
		{
		    pcd -> upper_ctrl_limit = 1000000;
		    pcd -> lower_ctrl_limit = 0;
		}
	    }
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> tset) ):
	    pcd -> upper_ctrl_limit = 86400;	/* seconds/day */
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> sgrv) ):
	    pcd -> upper_ctrl_limit = 100;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> pisv) ):
	case ( (int)&(((struct pipeRecord *)0) -> esou) ):
	    pcd -> upper_ctrl_limit = 1;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> ttim) ):
	    if (ppipe -> tset)
	    	pcd -> upper_ctrl_limit = ppipe -> tset;
	    else
		pcd -> upper_ctrl_limit = 86400;
	    pcd -> lower_ctrl_limit = -1;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> texp) ):
	    pcd -> upper_ctrl_limit = 2;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> cpre) ):
	    hopr = lopr = 0;
	    if (ppipe -> prpv.type == DB_LINK)
	    {
		get_link_value (ppipe -> prpv.value.db_link.pvname, ".HOPR", &hopr);
		get_link_value (ppipe -> prpv.value.db_link.pvname, ".LOPR", &lopr);
	    }
	    if ( (hopr == 0) && (lopr == 0) )
	    {
	    	pcd -> upper_ctrl_limit = 1000000;
	    	pcd -> lower_ctrl_limit = 0;
	    }
	    else
	    {
	    	pcd -> upper_ctrl_limit = hopr;
	    	pcd -> lower_ctrl_limit = lopr;
	    }
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> ctem) ):
	    hopr = lopr = 0;
	    if (ppipe -> tepv.type == DB_LINK)
	    {
		get_link_value (ppipe -> tepv.value.db_link.pvname, ".HOPR", &hopr);
		get_link_value (ppipe -> tepv.value.db_link.pvname, ".LOPR", &lopr);
	    }
	    if ( (hopr == 0) && (lopr == 0) )
	    {
	    	pcd -> upper_ctrl_limit = 300;
	    	pcd -> lower_ctrl_limit = 0;
	    }
	    else
	    {
	    	pcd -> upper_ctrl_limit = hopr;
	    	pcd -> lower_ctrl_limit = lopr;
	    }
	    break;
    }
    return (0);

} /* end func get_control_double () */


/****************************************************************************
 *
 *  MODULE NAME            : get_graphic_double 
 *  AUTHOR                 : emsteel
 *  DATE                   : 31 Aug 94
 *
 *
 *  DESCRIPTION: return graphic limits
 *
****************************************************************************/

static long get_graphic_double(paddr,pgd)
    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    struct pipeRecord    *ppipe = (struct pipeRecord *) paddr->precord;
    struct strappingValue   *stbl;
    double                  top_strap_vol = 0;
    double		    bottom_strap_vol = 0;
    float		    hopr;
    float		    lopr;

    if (ppipe -> pstr)
    {
	stbl = (struct strappingValue *)ppipe -> pstr;
	top_strap_vol = (stbl + (ppipe -> snum - 1)) -> volume;
	bottom_strap_vol = stbl -> volume;
    }

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct pipeRecord *)0) -> val) ):
	case ( (int)&(((struct pipeRecord *)0) -> qvol) ):
	case ( (int)&(((struct pipeRecord *)0) -> gcap) ):
	case ( (int)&(((struct pipeRecord *)0) -> netv) ):

	    /* if there is a strapping table, limit is bottom and top entry */
	    if (ppipe -> pstr)
	    {
		pgd -> upper_disp_limit = top_strap_vol;
		pgd -> lower_disp_limit = bottom_strap_vol;
	    }
	    else
	    {
		pgd -> upper_disp_limit = 1000000;
		pgd -> lower_disp_limit = 0;
	    }
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> vold) ):
	case ( (int)&(((struct pipeRecord *)0) -> adel) ):
	case ( (int)&(((struct pipeRecord *)0) -> mdel) ):
	case ( (int)&(((struct pipeRecord *)0) -> ullg) ):
	case ( (int)&(((struct pipeRecord *)0) -> hyst) ):

	    if (ppipe -> pstr)
	    {
		pgd -> upper_disp_limit = top_strap_vol;
		pgd -> lower_disp_limit = 0;
	    }
	    else
	    {
		if (ppipe -> gcap > 0)
		{
		    pgd -> upper_disp_limit = ppipe -> gcap;
		    pgd -> lower_disp_limit = 0;
		}
		else
		{
		    pgd -> upper_disp_limit = 1000000;
		    pgd -> lower_disp_limit = 0;
		}
	    }
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> tset) ):
	    pgd -> upper_disp_limit = 86400;	/* seconds/day */
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> sgrv) ):
	    pgd -> upper_disp_limit = 100;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> pisv) ):
	case ( (int)&(((struct pipeRecord *)0) -> esou) ):
	    pgd -> upper_disp_limit = 1;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> ttim) ):
	    if (ppipe -> tset)
	    	pgd -> upper_disp_limit = ppipe -> tset;
	    else
		pgd -> upper_disp_limit = 86400;
	    pgd -> lower_disp_limit = -1;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> texp) ):
	    pgd -> upper_disp_limit = 2;
	    pgd -> lower_disp_limit = 0;
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> cpre) ):
	    hopr = lopr = 0;
	    if (ppipe -> prpv.type == DB_LINK)
	    {
		get_link_value (ppipe -> prpv.value.db_link.pvname, ".HOPR", &hopr);
		get_link_value (ppipe -> prpv.value.db_link.pvname, ".LOPR", &lopr);
	    }
	    if ( (hopr == 0) && (lopr == 0) )
	    {
	    	pgd -> upper_disp_limit = 1000000;
	    	pgd -> lower_disp_limit = 0;
	    }
	    else
	    {
	    	pgd -> upper_disp_limit = hopr;
	    	pgd -> lower_disp_limit = lopr;
	    }
	    break;

	case ( (int)&(((struct pipeRecord *)0) -> ctem) ):
	    hopr = lopr = 0;
	    if (ppipe -> tepv.type == DB_LINK)
	    {
		get_link_value (ppipe -> tepv.value.db_link.pvname, ".HOPR", &hopr);
		get_link_value (ppipe -> tepv.value.db_link.pvname, ".LOPR", &lopr);
	    }
	    if ( (hopr == 0) && (lopr == 0) )
	    {
	    	pgd -> upper_disp_limit = 300;
	    	pgd -> lower_disp_limit = 0;
	    }
	    else
	    {
	    	pgd -> upper_disp_limit = hopr;
	    	pgd -> lower_disp_limit = lopr;
	    }
	    break;
    }
    return (0);

} /* end func get_graphic_double () */


/****************************************************************************
 *
 *  MODULE NAME            : get_link_value
 *  AUTHOR                 : emsteel
 *  DATE                   : 31 Aug 94
 *
 *
 *  DESCRIPTION: do dbNameToAddr on a record field and return its value
 *
****************************************************************************/

static void get_link_value (recname, recfld, value)
    char	*recname;
    char	*recfld;
    float	*value;
{
    long     status    = 0;
    long     nRequests = 1;
    long     options   = 0;
    char     pvname_fld[36];
    float    val = 0;
    struct   dbAddr   dbAddr;

    strncpy (pvname_fld, recname, 28);
    strcat (pvname_fld, recfld);
    status = dbNameToAddr (pvname_fld, &dbAddr);
    if (status)
	return;
    status = dbGetField (&dbAddr, DBR_FLOAT, (caddr_t)&val,
			 &options, &nRequests, NULL);
    if (status)
	return;
    *value = val;
    return;

} /* end func get_link_value () */


/****************************************************************************
 *
 *  MODULE NAME            : get_precision 
 *  AUTHOR                 : emsteel
 *  DATE                   : 29 June 93
 *
 *
 *  DESCRIPTION: return precision value
 *
****************************************************************************/

static long get_precision (paddr, precision)
   struct dbAddr     *paddr;
   long              *precision;
{
   struct pipeRecord *ppipe = (struct pipeRecord *)paddr -> precord;

   *precision = ppipe -> prec;

   return (0);

} /* end func get_precision () */



/****************************************************************************
 *
 *  MODULE NAME            : get_value 
 *  AUTHOR                 : emsteel
 *  DATE                   : 26 Mar 93
 *
 *
 *  DESCRIPTION: return the value field (current pipe volume)
 *
****************************************************************************/

static long get_value (ppipe, pvdes)
       struct pipeRecord   *ppipe;
       struct valueDes     *pvdes;
{
   pvdes -> field_type = DBF_DOUBLE;
   pvdes -> no_elements = 1;
   (double *)(pvdes -> pvalue) = &ppipe -> val;

   return (0);

} /* end func get_value () */



/****************************************************************************
 *
 *  MODULE NAME            : monitor 
 *  AUTHOR                 : emsteel
 *  DATE                   : 25 Mar 93
 *
 *  CALLING MODULE(S)      : process ()
 *                           run_commands ()
 *
 *  DESCRIPTION: check event list
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 05 Jan 94
 *  REVISION:  check VALID_ALARM before set ACK
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 30 Aug 94
 *  REVISION:	rm DBE_LOG from ack, only archive if tenb (trending) is
 *		enabled
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 15 Mar 95
 *  REVISION:	do not set ack if sevr is EVENT_ALARM
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Sept 95
 *  REVISION:	archive if alarm stat/sevr chg's
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Sept 95
 *  REVISION:	chg to way handle VALID_ALARM
 *
****************************************************************************/

static void monitor (ppipe)
       struct pipeRecord *ppipe;
{
    unsigned short monitor_mask;
    double         delta;
    short          stat, sevr, nsta, nsev;
    int            set_ack = 1;

      /* get previous stat and sevr and new stat and sevr */

    stat = ppipe -> stat;
    sevr = ppipe -> sevr;
    nsta = ppipe -> nsta;
    nsev = ppipe -> nsev;

      /* set current stat and sevr */

    ppipe -> stat = nsta;
    ppipe -> sevr = nsev;
    ppipe -> nsta = 0;
    ppipe -> nsev = 0;

    /* if we are coming out of VALID_ALARM and our new stat/sevr are
       the same as when we went into VALID_ALARM, do not set ack
    */
    if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	 (nsta == ppipe -> psta) && (nsev == ppipe -> psev) )
    {
	set_ack = 0;
    }
    /* if we are going into VALID_ALARM, and out previous stat was not
       VALID_ALARM, remember previous non VALID_ALARM stat/sevr
       and we do not want to set ACK,
    */
    if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
    {
	set_ack = 0;
	ppipe -> psta = stat;
	ppipe -> psev = sevr;
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> psta, DBE_VALUE);
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> psev, DBE_VALUE);
    }

    monitor_mask = 0;

      /* did alarm condition change this scan */

    if ((stat != nsta) || (sevr != nsev))
    {
	/* post events for alarm condition change */

	monitor_mask |= DBE_ALARM;

	/* when alarm status chg's, get current time and
	   save VAL in lalm (last value when alarm chg'd)
	*/
	tsLocalTime (&ppipe -> latm);
	ppipe -> lalm = ppipe -> val;

	/* post stat and nsev fields */

	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> stat, DBE_VALUE);
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> sevr, DBE_VALUE);
	if ((ppipe -> sevr != NO_ALARM) && (set_ack) &&
	    (ppipe -> sevr != EVENT_ALARM) )
	{
	    if (ppipe -> ack == 0)
	    {
		ppipe -> ack = 1;
		db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> ack, DBE_VALUE);
	    }
	}
    }

      /* check for value changes */

    delta = ppipe -> mlst - ppipe -> val;  /* last monitored val - val */
    if (delta < 0.0)
	delta = -delta;

    if ( delta > ppipe -> mdel)   /* monitor deadband */
    {
	/* post events for value change */

	monitor_mask |= DBE_VALUE;

	/* update last value monitored */

	ppipe -> mlst = ppipe -> val ;
    }

      /* check for archive change if tenb (trending) is enabled */

    if (ppipe -> tenb)
    {
	delta = ppipe -> alst - ppipe -> val;
	if (delta < 0.0)
	    delta = -delta;

	/* if exceed archive deadband or there has been a change in alarm status */
	if ( (delta > ppipe -> adel) || (monitor_mask & DBE_ALARM) )
	{
	    /* post events for value change */

	    monitor_mask |= DBE_LOG;

	    /* update last archive value */

	    ppipe -> alst = ppipe -> val;
	}
    }

      /* send out monitors connected to the value field */

    if (monitor_mask)
    {
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> val, monitor_mask);
    }

   return;


} /* end func monitor () */



/****************************************************************************
 *
 *  MODULE NAME            : post_events 
 *  AUTHOR                 : emsteel
 *  DATE                   : 19 May 94
 *
 *  PARAMETERS             : ppipe - ptr to current record
 *			     ppipe_old - ptr to struct pipered 
 * 
 *  DESCRIPTION: called from init_record and process, if chg between old and
 *		 new then post the change
 *
****************************************************************************/

static void  post_events (ppipe, ppipe_old)
    struct pipeRecord		*ppipe;
    struct pipered		*ppipe_old;
{
    if ( strncmp (ppipe -> psts, ppipe_old -> psts, sizeof (ppipe -> psts)) != 0)
	db_post_events ((struct dbCommon *)ppipe, ppipe -> psts, DBE_VALUE);

    if ( strncmp (ppipe -> pcmd, ppipe_old -> pcmd, sizeof (ppipe -> pcmd)) != 0)
	db_post_events ((struct dbCommon *)ppipe, ppipe -> pcmd, DBE_VALUE);

    if (ppipe -> pisv != ppipe_old -> pisv)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> pisv, DBE_VALUE);

    if (ppipe -> cpre != ppipe_old -> cpre)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> cpre, DBE_VALUE);

    if (ppipe -> ctem != ppipe_old -> ctem)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> ctem, DBE_VALUE);

    if (ppipe -> gcap != ppipe_old -> gcap)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> gcap, DBE_VALUE);

    if (ppipe -> ullg != ppipe_old -> ullg)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> ullg, DBE_VALUE);

    if (ppipe -> netv != ppipe_old -> netv)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> netv, DBE_VALUE);

    if (ppipe -> qvol != ppipe_old -> qvol)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> qvol, DBE_VALUE);

    if (ppipe -> ttim != ppipe_old -> ttim)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> ttim, DBE_VALUE);

    if (ppipe -> tval != ppipe_old -> tval)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> tval, DBE_VALUE);

    if (ppipe -> esou != ppipe_old -> esou)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> esou, DBE_VALUE);

    if (ppipe -> tcks != ppipe_old -> tcks)
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> tcks, DBE_VALUE);

    return;
}


/****************************************************************************
 *
 *  MODULE NAME            : process 
 *  AUTHOR                 : emsteel
 *  DATE                   : 18 Mar 93
 *
 *  PARAMETERS             : paddr - ptr to record struct packet            
 * 
 *  DESCRIPTION: called when time to process a pipe record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 17 Apr 95
 *  REVISION:	added code to process the forward link flnk
 *
****************************************************************************/

static long process (paddr)
       struct dbAddr *paddr;
{
   struct pipeRecord *ppipe = (struct pipeRecord *) (paddr -> precord);
   long   status      = 0;                /* return value */
   struct pipered pipe_old;               /* save initial values for redundancy
                                             check */


      /* do nothing if we did not init */

   if ((!ppipe -> init) || (!ppipe -> ini2))
      return (status);

   /* get time stamp for last time record was processed */

   tsLocalTime (&ppipe -> lptm);

   save_pipe (ppipe, &pipe_old);

      /* get system tick counter to use for time elapse */

   ppipe -> tcks = tickGet (); 

      /* init new status to previous */

   ppipe -> nsta = ppipe -> stat;
   ppipe -> nsev = ppipe -> sevr;
   ppipe -> msev = NO_ALARM;

   /* off normal operations */

   if (ppipe -> tagd)
   {
      ppipe -> nsta = TAGGED_ALARM;
      ppipe -> nsev = VALID_ALARM;
   }
   else if (ppipe -> oscn == OFF_SCAN)
   {
      ppipe -> nsta = SCAN_ALARM;
      ppipe -> nsev = VALID_ALARM;
   }
   else
   {
      ppipe -> nsta = NO_ALARM;
      ppipe -> nsev = NO_ALARM;

      /* update number of process times */
      if (process_stats)
         process_cnts[type]++;	

         /* read the pressure sensor */
   
      status = read_pressure_pv (ppipe);
   
         /* read the temperature sensor */
   
      if (!status)
      {
         status = read_temperature_pv (ppipe);
         if (!status)
            compute_some_values (ppipe);

      }
         /* set pipe to active */
   
      ppipe -> pact = TRUE;

         /* if pipe is stopped see if we are timing to
            set quiescent volume
         */

      if (ppipe -> pisv == PIPE_STOPPED)
         update_qvol (ppipe, &pipe_old);

         /* get current command if any and process */

      run_commands (ppipe);
   
      alarm (ppipe);

      /* check for maximize severity */

      if (ppipe -> msev > ppipe -> nsev)
      {
         ppipe -> nsev = ppipe -> msev;
         ppipe -> nsta = LINK_ALARM;
      }

   } /* end else (oscn != OFF_SCAN) */

   /* if the time stamp is 0, init and post val with
      alarm bit set, this makes sure that proper alarm
      messages are set if IOC was down and alarm status
      may have chg'd
   */
   if (ppipe -> latm.secPastEpoch == 0)
   {
	tsLocalTime (&ppipe -> latm);
	db_post_events ((struct dbCommon *)ppipe, (caddr_t)&ppipe -> val, DBE_VALUE | DBE_ALARM);
   }

   /* if it is the 1st process pass 
      init last value when alarm chg'd (lalm)
   */

   if (ppipe -> fpps)
   {
	ppipe -> fpps = 0;
	ppipe -> lalm = ppipe -> val;
   }

      /* check event list */

   monitor (ppipe);

      /* the process is no longer active */

   ppipe -> pact = FALSE;

      /* post events for monitors */

   post_events (ppipe, &pipe_old);

      /* now check for redundancy, see programmers notes */

   update_redun (&pipe_old, ppipe);

   /* process the forward scan link record */

   if (ppipe -> flnk.type == DB_LINK) 
      dbScanPassive ((struct dbAddr *)ppipe -> flnk.value.db_link.pdbAddr);

   return (status);

} /* end func process () */



/****************************************************************************
 *
 *  MODULE NAME            : read_pressure_pv 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 May 93
 *
 *  CALLING MODULE(S)      : init_record, process
 *          ARGUMENT(S)    : ppipe - ptr to pipe record
 *
 *  DESCRIPTION: read the pressure sensor if it exists
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 May 94
 *  REVISION: if ini2 is not set, checkif PV is init'd
 *
****************************************************************************/

static long read_pressure_pv (ppipe)
       struct pipeRecord      *ppipe;
{
    long     status      = 0;
    long     nRequests   = 1;
    long     options     = 0;
    double   curr_pval   = 0;    /* current pressure value returned */
    double   curr_volume = 0;

            /* is there a record for the device */

    if (ppipe -> prpv.type != CONSTANT)
    {
        if (ppipe -> prpv.type == DB_LINK)
        {
            /* this arg list will return the val field from the
               process variable in curr_pval 
	    */

            status = dbGetLink (&ppipe -> prpv.value.db_link, 
                             (struct dbCommon *)ppipe, DBR_DOUBLE, 
                             (caddr_t)&curr_pval, &options, &nRequests);
            if (status)
	    {
		TankFarmPostAlarm ((struct dbCommon *)ppipe, LINK_ALARM, 
							       VALID_ALARM);
		return (status);
	    }
	    else if (!ppipe -> ini2)
	    {
		if (!okToUse (&ppipe -> prpv) )
		    return (-1);
	    }

            /* now compute the volume in the pipe by interpolation using
               the strapping table pointed to by pstr in the pipeRecord 
	    */

	    if (curr_pval)
               ComputeTFVolume (ppipe -> pstr, ppipe -> snum, curr_pval, 
							       &curr_volume);

               /* store current pipe stats */

	    ppipe -> cpre = curr_pval;
	    ppipe -> val = curr_volume;
	}
    }

   return (status);
   
} /* end func read_pressure_pv () */



/****************************************************************************
 *
 *  MODULE NAME            : read_temperature_pv 
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 May 93
 *
 *  CALLING MODULE(S)      : init_record, process
 *          ARGUMENT(S)    : ppipe - ptr to pipe record
 *
 *  DESCRIPTION: read the pipe temperature if sensor exists
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 May 94
 *  REVISION: if ini2 is not set, check if PV is init'd
 *
****************************************************************************/

static long read_temperature_pv (ppipe)
       struct pipeRecord      *ppipe;
{
    long     status    = 0;
    long     nRequests = 1;
    long     options   = 0;
    double   curr_temp = 0;        /* current temperature returned */

            /* is there a record for the device */

    if (ppipe -> tepv.type != CONSTANT)
    {
        if (ppipe -> tepv.type == DB_LINK)
        {
            /* this arg list will return the val field from the
               process variable in curr_temp */

            status = dbGetLink (&ppipe -> tepv.value.db_link, 
                                (struct dbCommon *)ppipe, DBR_DOUBLE, 
                                (caddr_t)&curr_temp, &options, &nRequests);
            if (status)
	    {
                TankFarmPostAlarm ((struct dbCommon *)ppipe, LINK_ALARM, 
                                                               VALID_ALARM);
		return (status);
	    }
	    else if (!ppipe -> ini2)
	    {
		if (!okToUse (&ppipe -> tepv) )
		    return (-1);
	    }
            ppipe -> ctem = curr_temp;
         }
     }

   return (status);

} /* end func read_temperature_pv () */



/****************************************************************************
 *
 *  MODULE NAME            : run_commands 
 *  AUTHOR                 : emsteel
 *  DATE                   : 01 Apr 93
 *
 *  CALLING MODULE(S)      : process ()
 *  ARGUMENT(S)            : ppipe = ptr to pipe record
 *
 *  DESCRIPTION: run a command to the pipe
 *  NOTE:
 *       see tankfarm.c for GetTFCommand() and ResetTFCommand()
 *       see tankfarm.h for struct tfcmd
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 14 dec 93
 *  REVISION:  deleted db_post_events for command fields, not needed
 *             added code to process TF_SPECIAL_DIGITAL_CMDs
 *             added code to update PCMD, last command
 *
****************************************************************************/

static void run_commands (ppipe)
       struct pipeRecord   *ppipe;
{
   struct tfcmd   tfcmd;
   int            cmd;
   int            cmd_type;

   tfcmd.NumCmds = 2;
   tfcmd.FirstDigitalCmd = &ppipe -> crun;
   tfcmd.FirstSpecialDigitalCmd = &ppipe -> srun;
   tfcmd.FirstCmdString = ppipe -> runc;
   tfcmd.CmdString = ppipe -> cmds;

   cmd = GetTFCommand (&tfcmd, &cmd_type);

   if (cmd != NO_TF_COMMAND)
   {
      if ((ppipe -> mode == TF_AUTOMATIC) || (ppipe -> mode == TF_MANUAL))
      {
         if (cmd == ILLEGAL_TF_STRING_CMD)
         {
            TankFarmPostAlarm ((struct dbCommon *)ppipe, CMD_NOT_ALLOWED,
                                                              INFO_ALARM);
         }
         else if ( ((ppipe -> mode == TF_AUTOMATIC) &&
                    ((cmd_type == TF_STRING_CMD) ||
                     (cmd_type == TF_SPECIAL_DIGITAL_CMD))) ||
                   ((ppipe -> mode == TF_MANUAL) &&
                    (cmd_type == TF_DIGITAL_CMD)) )
         {
            switch (cmd)
            {
               case PIPE_RUN_CMD:
                  strncpy (ppipe -> psts, ppipe -> runs, 
                                                   sizeof (ppipe -> psts));
                  ppipe -> pisv = PIPE_RUNNING;
                  ppipe -> ttim = -1.0;
                  ppipe -> tval = 0;

                     /* update last command */

                  if (ppipe -> runc[0])
                     strncpy (ppipe -> pcmd, ppipe -> runc, sizeof (ppipe -> pcmd));
                  else
                     strncpy (ppipe -> pcmd, TF_UNDEF_CMD, sizeof (ppipe -> pcmd));

                  break;
         
                case PIPE_STOP_CMD:
                  strncpy (ppipe -> psts, ppipe -> stps,
                                                    sizeof (ppipe -> psts));
                  ppipe -> pisv = PIPE_STOPPED;
         
                     /* reset quiescent volume timer */
         
                  ppipe -> ttim = 0;
                  ppipe -> tval = 0;

                  if (ppipe -> stpc[0])
                     strncpy (ppipe -> pcmd, ppipe -> stpc, sizeof (ppipe -> pcmd));
                  else
                     strncpy (ppipe -> pcmd, TF_UNDEF_CMD, sizeof (ppipe -> pcmd));
         
                  break;
         
                default:
                  TankFarmPostAlarm ((struct dbCommon *)ppipe,
                                              CMD_NOT_ALLOWED, INFO_ALARM);
            }
         }
         else
         {
            TankFarmPostAlarm ((struct dbCommon *)ppipe, CMD_NOT_ALLOWED, 
                                                               INFO_ALARM);
         }
      }
      else if (ppipe -> mode == TF_LOCKED)
      {
            /* no commands allowed when locked */
   
         TankFarmPostAlarm ((struct dbCommon *)ppipe, CMD_NOT_ALLOWED, 
                                                               INFO_ALARM);
      }
   }

   ResetTFCommand (&tfcmd);
   
   return;

} /* end func run_command () */



/****************************************************************************
 *
 *  MODULE NAME            : save_pipe 
 *  AUTHOR                 : emsteel
 *  DATE                   : 14 May 93
 *
 *  CALLING MODULE(S)      : process
 *  ARGUMENT(S)            : ppipe - ptr to current pipe record
 *                           pipe_old - ptr to redundancy struct
 *
 *  DESCRIPTION: save values from from pipe record for reduandancy check
 *               at end of process
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 23 Feb 94
 *  REVISION: removed init, mlst, alst, time, not needed
 *
****************************************************************************/

static void save_pipe (ppipe, pipe_old)
       struct pipeRecord   *ppipe;
       struct pipered      *pipe_old;
{

   pipe_old -> stat = ppipe -> stat;
   pipe_old -> sevr = ppipe -> sevr;
   pipe_old -> val  = ppipe -> val;
   strncpy (pipe_old -> psts, ppipe -> psts, sizeof (pipe_old -> psts));
   strncpy (pipe_old -> pcmd, ppipe -> pcmd, sizeof (pipe_old -> pcmd));
   strncpy (pipe_old -> cmds, ppipe -> cmds, sizeof (pipe_old -> cmds));
   pipe_old -> pisv = ppipe -> pisv;
   pipe_old -> ttim = ppipe -> ttim;
   pipe_old -> qvol = ppipe -> qvol;
   pipe_old -> lalm = ppipe -> lalm;
   pipe_old -> tcks = ppipe -> tcks;
   pipe_old -> esou = ppipe -> esou;
   pipe_old -> tval = ppipe -> tval;
   pipe_old -> crun = ppipe -> crun;
   pipe_old -> cstp = ppipe -> cstp;
   pipe_old -> srun = ppipe -> srun;
   pipe_old -> sstp = ppipe -> sstp;
   pipe_old -> ini2 = ppipe -> ini2;
   pipe_old -> esou = ppipe -> esou;
   pipe_old -> cpre = ppipe -> cpre;
   pipe_old -> ctem = ppipe -> ctem;
   pipe_old -> gcap = ppipe -> gcap;
   pipe_old -> ullg = ppipe -> ullg;
   pipe_old -> netv = ppipe -> netv;
   pipe_old -> latm = ppipe -> latm;
   pipe_old -> mlst = ppipe -> mlst;

   return;

} /* end func save_pipe () */



/****************************************************************************
 *
 *  MODULE NAME            : special
 *  AUTHOR                 : emsteel
 *  DATE                   : 06 Apr 93
 *
 *  CALLING MODULE(S)      : 
 *  ARGUMENT(S)            : paddr = ptr to dbAddr as defined in dbAccess.h
 *                         : after = not used here
 *
 *
 *  DESCRIPTION: special handling when a field changes etc
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 Jan 94
 *  REVISION: check for !after
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 May 94
 *  REVISION: FindStrappingTable and FindAPITable now return a status
 *
****************************************************************************/

static long special (paddr, after)
       struct dbAddr	*paddr;
       int		after;
{
   struct pipeRecord	*ppipe = (struct pipeRecord *)(paddr -> precord);
   int	  special_type         = paddr -> special;
   long			status = 0;

   if (!after)
      return (status);

   switch (special_type)
   {
      case SPC_STRAP_TBL:    /* new strapping table */
         status = FindStrappingTable (ppipe -> sttn, &ppipe -> pstr, (int *)&ppipe -> snum);
         break;

      case SPC_API_TBL:       /* new API table number */
         status = FindAPITable (ppipe -> apit, &ppipe -> papi, (int *)&ppipe -> anum);
         break;

      default:
         recGblDbaddrError (S_db_badChoice, paddr, "pipe: special");
         status = S_db_badChoice;
   }

   return (status);

} /* end func special () */



/****************************************************************************
 *
 *  MODULE NAME            : update_redun 
 *  AUTHOR                 : emsteel
 *  DATE                   : 01 Apr 93
 *
 *  CALLING MODULE(S)      : process ()
 *  ARGUMENT(S)            : ppipe_old = ptr t pipe state at start of process
 *                         : ppipe = ptr to pipe start at end of process
 *
 *
 *  DESCRIPTION: have any of the fields deemed as being redundant changed
 *               it is not neccessary to check for MASTER since we are
 *               called from process and are guaranteed to be MASTER at 
 *               this point
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 23 Feb 94
 *  REVISION: removed init, mlst, alst, not needed
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 May 94
 *  REVISION: added PCMD, and the command fields
 *
****************************************************************************/

static void update_redun (ppipe_old, ppipe)
     struct pipered    *ppipe_old;
     struct pipeRecord *ppipe;
{
   char     redbuf [REDQ_MAX_MSGLEN];
      
   redQInit ((struct dbCommon *)ppipe, redbuf, sizeof (redbuf), type);

   if (ppipe_old -> stat != ppipe -> stat)
      redQField (redbuf, mylist[STAT]);

   if (ppipe_old -> sevr != ppipe -> sevr)
      redQField (redbuf, mylist[SEVR]);

   if (ppipe_old -> val != ppipe -> val)
      redQField (redbuf, mylist[VAL]);

   if (strncmp (ppipe_old -> psts, ppipe -> psts, 
                                   sizeof (ppipe_old -> psts)) != 0)
      redQField (redbuf, mylist[PSTS]);

   if (strncmp (ppipe_old -> pcmd, ppipe -> pcmd, 
                                   sizeof (ppipe_old -> pcmd)) != 0)
      redQField (redbuf, mylist[PSTS]);

   if (strncmp (ppipe_old -> cmds, ppipe -> cmds, 
                                   sizeof (ppipe_old -> cmds)) != 0)
      redQField (redbuf, mylist[CMDS]);

   if (ppipe_old -> pisv != ppipe -> pisv)
      redQField (redbuf, mylist[PISV]);

   if (ppipe_old -> ttim != ppipe -> ttim)
      redQField (redbuf, mylist[TTIM]);

   if (ppipe_old -> qvol != ppipe -> qvol)
      redQField (redbuf, mylist[QVOL]);

   if (ppipe_old -> lalm != ppipe -> lalm)
      redQField (redbuf, mylist[LALM]);

   if (ppipe_old -> esou != ppipe -> esou)
      redQField (redbuf, mylist[ESOU]);

   if (ppipe_old -> tval != ppipe -> tval)
      redQField (redbuf, mylist[TVAL]);

   if (ppipe_old -> ini2 != ppipe -> ini2)
      redQField (redbuf, mylist[INI2]);

   if (ppipe_old -> crun != ppipe -> crun)
      redQField (redbuf, mylist[CRUN]);

   if (ppipe_old -> cstp != ppipe -> cstp)
      redQField (redbuf, mylist[CSTP]);

   if (ppipe_old -> srun != ppipe -> srun)
      redQField (redbuf, mylist[SRUN]);

   if (ppipe_old -> sstp != ppipe -> sstp)
      redQField (redbuf, mylist[SSTP]);

   if (TsCmpStampsNE (&ppipe_old -> latm, &ppipe -> latm) )
      redQField (redbuf, mylist [LATM]);

   if (ppipe_old -> mlst != ppipe -> mlst)
      redQField (redbuf, mylist[MLST]);
	
   redQSend (redbuf);

   return;

} /* end func update_redun () */



/****************************************************************************
 *
 *  MODULE NAME            : update_qvol 
 *  AUTHOR                 : emsteel
 *  DATE                   : 14 May 93
 *
 *  CALLING MODULE(S)      : process
 *          ARGUMENT(S)    : ppipe - ptr to pipe record
 *                           ppipe_old - ptr to redundancy struct
 *
 *  DESCRIPTION: ppipe -> ttim is a timer that counts time from a
 *               stop command until ppipe -> tset which is a settle
 *               time, the time for the product in the pipe to settle before
 *               the quiescent volume (ppipe -> qvol) is updated with
 *               the current product net volume (ppipe -> netv)
 *
****************************************************************************/

static void update_qvol (ppipe, ppipe_old)
       struct pipeRecord   *ppipe;
       struct pipered      *ppipe_old;
{
   double   delta_time;

      /* if not timing, do nothing */

   if (ppipe -> ttim < 0.0)
      return;

      /* diff in secs between last scan and current */

   delta_time = TFTicksDiffInSeconds (&ppipe -> tcks, &ppipe_old -> tcks);

   ppipe -> ttim += delta_time;

      /* if = or exceeded the settling time, update qvol */

   if (ppipe -> ttim >= ppipe -> tset)
   {
      ppipe -> ttim = -1.0;
      
         /* if not already alarming volume */

      if (!ppipe -> tval)
      {
         ppipe -> qvol = ppipe -> netv;
      }
   }

   return;

} /* end func update_qvol () */
