/*recRamp.c,v 1.1.1.1 1999/08/03 14:44:20 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
  
/* recRamp.c - Record Support Routines for Ramp records */
/*
**	Author:		John L. Dalesio
**	Date:		5/6/93
*/
/*
 *
 * Modification Log
 * ----------------
 * .01  01-03-94	ems	set ack to 0 in init_record
 *				in process, do nothing if not init'
 * .02  01-05-94	ems	in monitor,check VALID_ALARM before
 *				set ACK
 * .03  03-01-94	ems	added support for process stats
 *				removed ack = 0
 * .04	03-29-94	ems	make forward func def agree with actual func def
 * .05	05-10-94	ems	rewrite init_record, add INI2 to redundancy
 *				deleted RAMP_AUTO use NORMAL as defined in alarm.h
 * .06	06-05-94	ems	added special() to handle ltim, in ramp(), make times as
 *				floats so dont get truncation on scan times in fractions
 *				of a second
 * .07	06-14-94	ems	added support for maximize severity (msev), added call to
 *				tsLocalTime
 * .08	08-08-94	ems	added field types to get_graphic_double and
 *				get_control_double
 * .09	08-18-94	ems	monitor(), rm DBE_LOG from ack, only archive val if
 *				tenb (trending) is enabled
 * .10	10-12-94	ems	add TIME to redundancy, added fpps (1st process pass)
 *				and lalm, do time stamp when stat or sevr chg,
 *				see monitor(),and not in process(),set lalm to val when 
 *				stat or sevr chg,in process() if fpps (first process)
 *				init lalm to val and if time stamp is 0, init it also
 * .11	10-20-94	ems	chg time to latm (last alarm time) added 
 *				added lptm (last process time)
 * .12	12-20-94	ems	removed fpps and lalm, not needed
 * .13	03-15-95	ems	monitor(), do not set ACK if sevr is EVENT_ALARM
 * .14	09-11-95	ems	monitor(), archive when stat/sevr chg's also
 * .15	09-12-95	ems	monitor(), see mod for handling VALID_ALARM
 */

#include	<vxWorks.h>
#include	<inetLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<math.h>
#include	<string.h>
#include	<tickLib.h>

#include	"alarm.h"
#include	"cvtTable.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/rampRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"rampredfldidx.h"
#include	"restable.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"
#include	"okToUse.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/

#define report NULL
#define initialize NULL
static long init_record();
static long process();
static long special();
static long get_value();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
static long get_units();
static long get_precision();
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
static long get_graphic_double();
static long get_control_double();
#define get_alarm_double NULL

struct rset rampRSET = {

	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
	};


/* The folowing definitions are for redundancy */
 
static struct fldDes	**mylist = 0;

static	char *namelist [] = {
		"STAT",
		"SEVR",
		"VAL",
		"LTIM",
		"INI2",
		"LATM",
		"MLST",
		0
		};

static	unsigned	short	type;

static void monitor ();
static long ramp ();


/*------------------------------------------------------------------------*/

static long init_record (pramp)

    struct rampRecord	*pramp;

{
    double		value;
    long		options, nRequest;
    unsigned	short	i;
    char		*pstr;
    char		redbuf [256];
    int			ini2_flag = 0;

    if (!(mylist)) 
    {
	for (type = 1; type < 100; type++) 
	{
	    pstr = GET_PRECTYPE (type);
	    if (strncmp (pstr, "ramp", 4) == 0) 
		break;
	}

	mylist = redFldAlloc (type, NUMRAMPFLDS);

	if (!mylist)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pramp,
				   "ramp, init_record, mylist failed");
	    return (S_rec_Alloc);
	}

	for (i = 0; i < NUMRAMPFLDS; i++) 
	    redFldAdd (type, i, namelist [i]);
    }

    pramp -> init = TRUE;
    pramp -> nsta = NO_ALARM;
    pramp -> nsev = NO_ALARM;
    pramp -> msev = NO_ALARM;

    /* init last process time and last alarm time */

    if (pramp->lptm.secPastEpoch == 0)
    {
	tsLocalTime (&pramp->lptm);
    }
    if (pramp->latm.secPastEpoch == 0)
    {
	tsLocalTime (&pramp->latm);
    }
    if (pramp -> inp.type == DB_LINK)
    {
	/* if we are tagged, off scan or in manual mode we will let
	   the block init without getting VAL
	*/
	if ( (!pramp -> tagd) && (pramp -> oscn != OFF_SCAN) &&
	     (pramp -> mode == NORMAL) )
	{
	    options = 0;
	    nRequest = 1;

	    if ( (dbGetLink (&pramp->inp.value.db_link, 
			     (struct dbCommon *)pramp, 
			     DBR_DOUBLE, (caddr_t)&value, 
			     &options, &nRequest) == NULL) &&
		 (okToUse (&pramp -> inp)) )
	    {
		pramp->val = value;

		/* fetch the start time */
		pramp->ltim = tickGet();

		ini2_flag = 1;
	    }
	    else
	    {
		pramp -> nsta = NO_INIT_ALARM;
		pramp -> nsev = VALID_ALARM;
	    }
	}
	else
	    ini2_flag = 1;
    }
    else
    {
	pramp -> nsta = NO_INIT_ALARM;
	pramp -> nsev = VALID_ALARM;
    }

    if (resGetCaAnswer(pramp -> radr, &i) == RES_OK)
    {
	if (i)
	{

	    /* check for maximize severity */

	    if (pramp -> msev > pramp -> nsev)
	    {
		pramp -> nsev = pramp -> msev;
		pramp -> nsta = LINK_ALARM;
	    }

	    monitor (pramp);

	    if (ini2_flag)
	    {
		pramp -> ini2 = 1;

		redQInit ((struct dbCommon *)pramp, redbuf, sizeof (redbuf), type);
		redQField (redbuf, mylist [STAT]);
		redQField (redbuf, mylist [SEVR]);
		redQField (redbuf, mylist [VAL]);
		redQField (redbuf, mylist [LTIM]);
		redQField (redbuf, mylist [INI2]);

		redQSend (redbuf);
	    }
	}
    }

    return (0);

}


/*------------------------------------------------------------------------*/

static long process (paddr)

    struct dbAddr	*paddr;

    {
	struct rampRecord *pramp = (struct rampRecord *) (paddr->precord);
	char		redbuf [256];

	long		status = 0;
	unsigned short	stat   = pramp->stat;
	unsigned short	sevr   = pramp->sevr;
	double		val    = pramp->val;
	unsigned long	ltim   = pramp->ltim;
	float		mlst   = pramp->mlst;
	TS_STAMP	latm   = pramp->latm;

	   /* if not init'd, do nothing */

	if ((!pramp -> init) || (!pramp -> ini2))
	    return (status);

	/* get time stamp for last time record was processed */

	tsLocalTime (&pramp -> lptm);

	/* Initialize the new status to the last */
	pramp->nsta = pramp->stat;
	pramp->nsev = pramp->sevr;
	pramp->msev = NO_ALARM;

        /* off normal operations (use VAL and use RVAL) */

	if (pramp -> tagd)
	{
	    pramp->nsta = TAGGED_ALARM;
	    pramp->nsev = VALID_ALARM;
	}
        else if (pramp->oscn == OFF_SCAN)
	{
            pramp->nsta = SCAN_ALARM;
	    pramp->nsev = VALID_ALARM;
	}
	else
	{
	    pramp -> nsta = NO_ALARM;
	    pramp -> nsev = NO_ALARM;

	    /* update number of process times */
	    if (process_stats)
	       process_cnts[type]++;	

	    status = ramp (pramp);

	    /* check for maximize severity */

	    if (pramp -> msev > pramp -> nsev)
	    {
		pramp -> nsev = pramp -> msev;
		pramp -> nsta = LINK_ALARM;
	    }
	}

	/* check event list */

	/* if the time stamp is 0, init and post val with
	   alarm bit set, this makes sure that proper alarm
	   messages are set if IOC was down and alarm status
	   may have chg'd
	*/
	if (pramp -> latm.secPastEpoch == 0)
	{
	    tsLocalTime (&pramp -> latm);
	    db_post_events ((struct dbCommon *)pramp, (caddr_t)&pramp -> val, 
				DBE_VALUE | DBE_ALARM);
	}

	monitor (pramp);

	pramp->pact = FALSE;

	redQInit ((struct dbCommon *)pramp, redbuf, sizeof (redbuf), type);

	if (stat != pramp->stat) 
	    redQField (redbuf, mylist [STAT]);
	
	if (sevr != pramp->sevr) 
	    redQField (redbuf, mylist [SEVR]);
	
	if (val != pramp->val) 
	    redQField (redbuf, mylist [VAL]);
	
	if (ltim != pramp->ltim) 
	    redQField (redbuf, mylist [LTIM]);
	
	if (TsCmpStampsNE (&latm, &pramp -> latm) )
	    redQField (redbuf, mylist [LATM]);
	
	if (mlst != pramp->mlst)
	    redQField (redbuf, mylist [MLST]);

	redQSend (redbuf);


	/* process the forward scan link record */

	if (pramp->flnk.type == DB_LINK) 
	    dbScanPassive ((struct dbAddr *)pramp->flnk.value.db_link.pdbAddr);
	
	return (status);
}

/*------------------------------------------------------------------------*/

/*
**	Ramp Algotithm
*/
static long ramp (pramp)
    struct rampRecord *pramp;

    {
	long		status;
	unsigned long   thistime;
	float           timedif;
	double		tval,ivalue,dif,ioabsdif;
	long		options, nRequest;


	options = 0;
	nRequest = 1;
	status = 0;

	if (pramp->mode == NORMAL) 
	{
	    status = dbGetLink (&pramp->inp.value.db_link, 
				(struct dbCommon *)pramp, 
				DBR_DOUBLE, (caddr_t)&pramp->trgt, 
				&options, &nRequest);
	} 
	if (status != 0) 
	{
	    pramp->nsev  = VALID_ALARM;
	    pramp->nsta  = LINK_ALARM;
	    return(status);
	}

	ivalue = pramp->trgt;

	dif = ioabsdif =(ivalue-pramp->val);
	if (ioabsdif<0)
	    ioabsdif = -ioabsdif;

	/* Get the seconds of time difference since started moving*/
	thistime = tickGet();
	timedif = (((float)thistime) - ((float)pramp->ltim)) / 60.0;
	pramp->ltim = thistime;

	/* If close enough just set it */
	if (ioabsdif <= pramp->nois) 
	{
	    pramp->val = ivalue;
	    return(0);
	}

	if (dif==0) 
	    return(0); 

	if (dif>0) 
	{
	    tval = pramp->val + (pramp->incr * timedif);
	    if (tval > ivalue) 
		pramp->val = ivalue;
	    else 
		pramp->val = tval;
	}
	else 
	{
	    tval = pramp->val - (pramp->decr * timedif);
	    if (tval < ivalue) 
		pramp->val = ivalue;
	    else 
		pramp->val = tval;
	}
	return(0);
    }



/*------------------------------------------------------------------------*/

static long get_value(pramp,pvdes)

    struct	rampRecord	*pramp;
    struct	valueDes	*pvdes;
{
	pvdes->field_type = DBF_DOUBLE;
	pvdes->no_elements = 1;
	(double *) (pvdes->pvalue) = &pramp->val;
	return (0);
}

/*------------------------------------------------------------------------*/

static long get_units(paddr,units)

    struct dbAddr *paddr;
    char	  *units;
{
    struct	rampRecord	*pramp = (struct rampRecord *) paddr->precord;

	strncpy (units, pramp->egu, sizeof (pramp->egu));
	return (0);
}

/*------------------------------------------------------------------------*/

static long get_precision (paddr, precision)

    struct	dbAddr	*paddr;
    long		*precision;
{
	struct rampRecord	*pramp = (struct rampRecord *) paddr->precord;

	*precision = pramp->prec;
	return (0);
}


/*------------------------------------------------------------------------*/

static long get_graphic_double(paddr,pgd)

    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    struct rampRecord	*pramp = (struct rampRecord *) paddr->precord;


    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct rampRecord *)0) -> adel) ):
	case ( (int)&(((struct rampRecord *)0) -> mdel) ):
	case ( (int)&(((struct rampRecord *)0) -> incr) ):
	case ( (int)&(((struct rampRecord *)0) -> decr) ):
	case ( (int)&(((struct rampRecord *)0) -> nois) ):
	{
	    float    span;

	    span = pramp -> hopr - pramp -> lopr;
	    if (span < 0.0)
	        span = -span;
	    pgd -> upper_disp_limit = span;
	    pgd -> lower_disp_limit = 0;
	    break;
	}

	default:
	    pgd -> upper_disp_limit = pramp -> hopr;
	    pgd -> lower_disp_limit = pramp -> lopr;
	    break;
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static long get_control_double(paddr,pcd)

    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    struct rampRecord *pramp = (struct rampRecord *) paddr->precord;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct rampRecord *)0) -> adel) ):
	case ( (int)&(((struct rampRecord *)0) -> mdel) ):
	case ( (int)&(((struct rampRecord *)0) -> incr) ):
	case ( (int)&(((struct rampRecord *)0) -> decr) ):
	case ( (int)&(((struct rampRecord *)0) -> nois) ):
	{
	    float    span;

	    span = pramp -> hopr - pramp -> lopr;
	    if (span < 0.0)
	        span = -span;
	    pcd -> upper_ctrl_limit = span;
	    pcd -> lower_ctrl_limit = 0;
	    break;
	}

	default:
	    pcd -> upper_ctrl_limit = pramp -> hopr;
	    pcd -> lower_ctrl_limit = pramp -> lopr;
	    break;
    }
 
    return (0);
}


/*------------------------------------------------------------------------*/

static void monitor (pramp)

    struct	rampRecord	*pramp;
{
	unsigned short	monitor_mask;
	double		delta;
	short		stat, sevr, nsta, nsev;
	int		set_ack = 1;

	/* get previous stat and sevr  and new stat and sevr*/

	stat = pramp->stat;
	sevr = pramp->sevr;
	nsta = pramp->nsta;
	nsev = pramp->nsev;

	/*set current stat and sevr*/

	pramp->stat = nsta;
	pramp->sevr = nsev;
	pramp->nsta = 0;
	pramp->nsev = 0;

	/* if we are coming out of VALID_ALARM and our new stat/sevr are
	   the same as when we went into VALID_ALARM, do not set ack
	*/
	if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	     (nsta == pramp -> psta) && (nsev == pramp -> psev) )
	{
	    set_ack = 0;
	}
	/* if we are going into VALID_ALARM, and out previous stat was not
	   VALID_ALARM, remember previous non VALID_ALARM stat/sevr
	   and we do not want to set ACK,
	*/
	if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
	{
	    set_ack = 0;
	    pramp -> psta = stat;
	    pramp -> psev = sevr;
	    db_post_events ((struct dbCommon *)pramp, (caddr_t)&pramp -> psta, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pramp, (caddr_t)&pramp -> psev, DBE_VALUE);
	}

	monitor_mask = 0;

	/* alarm condition changed this scan */

	if ((stat != nsta) || (sevr != nsev)) 
	{
	    /* when alarm status chg's, get current time 
	    */
	    tsLocalTime (&pramp -> latm);

	    /* post events for alarm condition change*/

	    monitor_mask = DBE_ALARM;

	    /* post stat and nsev fields */

	    db_post_events ((struct dbCommon *)pramp, (caddr_t)&pramp->stat, DBE_VALUE);
	    db_post_events ((struct dbCommon *)pramp, (caddr_t)&pramp->sevr, DBE_VALUE);
            if ((pramp->sevr != NO_ALARM) && (set_ack) &&
		(pramp->sevr != EVENT_ALARM) )
	    {
                if (pramp->ack == 0) 
		{
                    pramp->ack = 1;           /* new unacknowledged alarm */
                    db_post_events((struct dbCommon *)pramp, (caddr_t)&pramp->ack, DBE_VALUE);
                }
	    }
	}
	/* check for value change */

	delta = pramp->mlst - pramp->val;
	if (delta < 0.0) 
	    delta = -delta;
	
	if (delta > pramp->mdel) 
	{
	    /* post events for value change */

	    monitor_mask |= DBE_VALUE;

	    /* update last value monitored */

	    pramp->mlst = pramp->val;
	}

	/* check for archive change if tenb (trending) is enabled */
	
	if (pramp -> tenb)
	{
	    delta = pramp->alst - pramp->val;
	    if (delta < 0.0) 
	    	delta = -delta;
	
	    if ( (delta > pramp->adel) || (monitor_mask & DBE_ALARM) )
	    {
	    	/* post events on value field for archive change */

	    	monitor_mask |= DBE_LOG;

	    	/* update last archive value monitored */

	    	pramp->alst = pramp->val;
	    }
	}

	/* send out monitors connected to the value field */

	if (monitor_mask) 
	    db_post_events ((struct dbCommon *)pramp, (caddr_t)&pramp->val, monitor_mask);
	
	return;
}


/*------------------------------------------------------------------------*/

static long special (paddr, after)
    struct dbAddr    *paddr;
    int              after;
{
    struct rampRecord	*pramp = (struct rampRecord *) (paddr -> precord);
    int			special_type = paddr -> special;

    if (!after)
	return (0);

    switch (special_type)
    {
	case SPC_RAMP_LTIM:
	    pramp -> ltim = tickGet();
	    break;

	default:
	    recGblDbaddrError (S_db_badChoice, paddr, "ramp: special");
	    return (S_db_badChoice);
    }

    return (0);
}  

