/*recStringout.c,v 1.1.1.1 1999/08/03 14:44:22 tis4000 Exp */

/* recStringout.c - Record Support Routines for Stringout records */
/*
 * Author: 	Janet Anderson
 * Date:	4/23/91
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  mm-dd-yy        iii     Comment
 * .02  01-03-94	ems	set ack to 0 in init_record
 * .03  03-01-94	ems	added support for process stats
 *				removed ack = 0
 * .04	03-29-94	ems	make forward func def agree with actual func def
 * .05	05-16-94	ems	rewrite init_record, add redundancy, add ack to
 *				monitor
 * .06	06-10-94	ems	move call to tsLocalTime inside on scan and !pact
 *				but before call to device
 * .07	06-14-94	ems	added support for maximize severity (msev)
 * .08	08-18-94	ems	monitor(), rm DBE_LOG from ack
 * .09	10-12-94	ems	add TIME to redundancy, added fpps (1st process pass)
 *				and lalm, do time stamp when stat or sevr chg,
 *				see monitor(),and not in process(),set lalm to val when 
 *				stat or sevr chg,in process() if fpps (first process)
 *				init lalm to val and if time stamp is 0, init it also
 * .10	10-20-94	ems	chg time to latm (last alarm time) added 
 *				added lptm (last process time)
 * .11	12-19-94	ems	removed lalm and fpps (first process pass) not needed
 * .12  01-13-95        wep     init lptm and latm
 * .13	03-15-95	ems	monitor(), do not set ACK if sevr is EVENT_ALARM
 * .14	09-12-95	ems	monitor(), see mod for handling VALID_ALARM
 * .15	02-02-96	ems	added support for output on chg only (OCHG)
 */


#include	<vxWorks.h>
#include	<inetLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<strLib.h>

#include        "alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"special.h"
#include	"recSup.h"
#include	"rec/stringoutRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"soredfldidx.h"
#include	"restable.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"


/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/
#define report NULL
#define initialize NULL
static long init_record();
static long process();
static long special ();
static long get_value();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
#define get_precision NULL
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
#define get_graphic_double NULL
#define get_control_double NULL
#define get_alarm_double NULL

struct rset stringoutRSET={
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double };

struct stringoutdset { /* stringout input dset */
	long		number;
	DEVSUPFUN	dev_report;
	DEVSUPFUN	init;
	DEVSUPFUN	init_record; /*returns: (-1,0)=>(failure,success)*/
	DEVSUPFUN	get_ioint_info;
	DEVSUPFUN	write_stringout;/*(-1,0,1)=>(failure,success,don't Continue*/
};

/* The folowing definitions are for redundancy */
 
static struct fldDes	**mylist = 0;

static	char *namelist [] = {
		"STAT",
		"SEVR",
		"VAL",
		"INI2",
		"LATM",
		"LOVL",
		0
		};

static unsigned short	type;

static void monitor ();
static void test_ochg ();


/*------------------------------------------------------------------------*/

static long init_record (pstringout)
    struct stringoutRecord	*pstringout;
{
    struct stringoutdset *pdset;
    long		status=0;
    char		*pstr;
    unsigned short	i;
    char		redbuf[256];
    int			ini2_flag = 0;

    if (!(mylist)) 
    {
	for (type = 1; type < 100; type++) 
	{
	    pstr = GET_PRECTYPE (type);
	    if (strncmp (pstr, "stringout", 9) == 0) 
		break;
	}
	mylist = redFldAlloc (type, NUMSOFLDS);

	if (!mylist)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pstringout,
				"stringout, init_record, mylist failed");
	    return (S_rec_Alloc);
	}

	for (i = 0; i < NUMSOFLDS; i++) 
	    redFldAdd (type, i, namelist [i]);
    }

    if(!(pdset = (struct stringoutdset *)(pstringout->dset))) 
    {
	recGblRecordError (S_dev_noDSET, (struct dbCommon *)pstringout,
							"stringout: init_record");
	return(S_dev_noDSET);
    }

    /* must have  write_stringout functions defined */
    if( (pdset->number < 5) || (pdset->write_stringout == NULL) ) 
    {
	recGblRecordError (S_dev_missingSup, (struct dbCommon *)pstringout,
							"stringout: init_record");
	return(S_dev_missingSup);
    }

    pstringout -> init = 1;
    pstringout -> nsta = NO_ALARM;
    pstringout -> nsev = NO_ALARM;
    pstringout -> msev = NO_ALARM;
    pstringout -> udf = FALSE;

    /* init last process time and last alarm time */
    if (pstringout -> lptm.secPastEpoch == 0)
        tsLocalTime (&pstringout -> lptm);
    if (pstringout -> latm.secPastEpoch == 0)
        tsLocalTime (&pstringout -> latm);

    if( pdset->init_record )
    {
	status = (*pdset->init_record) (pstringout,process);
	if (status)
	{
	    pstringout -> nsta = NO_INIT_ALARM;
	    pstringout -> nsev = VALID_ALARM;
	}
	else
	    ini2_flag = 1;
    }
    else
	ini2_flag = 1;

    strncpy (pstringout -> oval, pstringout -> val, sizeof (pstringout -> oval));
    strncpy (pstringout -> lovl, pstringout -> val, sizeof (pstringout -> lovl));

    /* if we are MASTER post VAL and send it to redundant partner */

    if (resGetCaAnswer (pstringout -> radr, &i) == RES_OK)
    {
	if (i)
	{
	    /* post STAT, SEVR and VAL (monitor), if we will finally init, 
	       because we have read a value from the input device, set INI2,
	       send all redund fields to slave, when slave receives INI2, 
	       it will be fully init'd also
	    */
	
	    /* check for maximize severity */

	    if (pstringout -> msev > pstringout -> nsev)
	    {
		pstringout -> nsev = pstringout -> msev;
		pstringout -> nsta = LINK_ALARM;
	    }


	    monitor (pstringout);

	    if (ini2_flag)
	    {
		pstringout -> ini2 = 1;
		redQInit ((struct dbCommon *)pstringout, redbuf, sizeof (redbuf), type);
		redQField (redbuf, mylist [STAT]);
		redQField (redbuf, mylist [SEVR]);
		redQField (redbuf, mylist [VAL]);
		redQField (redbuf, mylist [INI2]);
		redQField (redbuf, mylist [LOVL]);
		
		redQSend (redbuf);
	    }
	}
    }

    return(0);
}


/*------------------------------------------------------------------------*/

static long process(paddr)
    struct dbAddr	*paddr;
{
    struct stringoutRecord	*pstringout=(struct stringoutRecord *)(paddr->precord);
    struct stringoutdset	*pdset = (struct stringoutdset *)(pstringout->dset);
    long			status=0;
    unsigned short		stat = pstringout -> stat;
    unsigned short		sevr = pstringout -> sevr;
    char			val[80];
    char			lovl[80];
    int				write_output = 1;
    TS_STAMP			latm = pstringout -> latm;
    char			redbuf[256];

	if ((!pstringout -> init) || (!pstringout -> ini2))
		return (0);

	if( (pdset==NULL) || (pdset->write_stringout==NULL) ) 
	{
	    pstringout->pact=TRUE;
	    recGblRecordError (S_dev_missingSup, (struct dbCommon *)pstringout,
							"write_stringout");
    	    return(S_dev_missingSup);
	}

	/* get time stamp for last time record was processed */

	tsLocalTime (&pstringout -> lptm);

	pstringout -> nsta = pstringout -> stat;
	pstringout -> nsev = pstringout -> sevr;
	pstringout -> msev = NO_ALARM;
	strncpy (val, pstringout -> val, sizeof (val));
	strncpy (lovl, pstringout -> lovl, sizeof (lovl));

	if (pstringout -> tagd)
	{
	    pstringout -> nsta = TAGGED_ALARM;
	    pstringout -> nsev = VALID_ALARM;
	}
	else if (pstringout -> oscn == OFF_SCAN)
	{
	    pstringout -> nsta = SCAN_ALARM;
	    pstringout -> nsev = VALID_ALARM;
	}
	else
	{
	    pstringout -> nsev = NO_ALARM;
	    pstringout -> nsta = NO_ALARM;
		
	    if (!pstringout->pact) 
	    {

		/* update number of process times */
		if (process_stats)
		    process_cnts[type]++;	

		if ( (pstringout->dol.type == DB_LINK) && 
		     (pstringout->omsl == CLOSED_LOOP) )
		{
		    long options=0;
		    long nRequest=1;

		    pstringout->pact = TRUE;
		    status = dbGetLink(&pstringout->dol.value.db_link,
				       (struct dbCommon *)pstringout,
				       DBR_STRING, pstringout->val,
				       &options,&nRequest);
		    pstringout->pact = FALSE;
		    if(!status==0)
		    {
		    	if(pstringout->nsev < VALID_ALARM) 
			{
			    pstringout->nsev = VALID_ALARM;
			    pstringout->nsta = LINK_ALARM;
			}
		    } 
		    else 
			pstringout->udf=FALSE;
		}
	    }

	    if(status==0) 
	    {
		test_ochg (pstringout, &write_output);
		if (write_output)
		{
		    /* write the new value */
		    status=(*pdset->write_stringout)(pstringout); 
		    strncpy (pstringout -> lovl, pstringout -> val, 
			     sizeof (pstringout -> lovl));
		}
	    }
	    pstringout->pact = TRUE;

	    /* status is one if an asynchronous record is being processed*/
	    if (status==1) 
		return(0);
	
	    /* check for maximize severity */

	    if (pstringout -> msev > pstringout -> nsev)
	    {
		pstringout -> nsev = pstringout -> msev;
		pstringout -> nsta = LINK_ALARM;
	    }

	}


	/* if the time stamp is 0, init and post val with
	   alarm bit set, this makes sure that proper alarm
	   messages are set if IOC was down and alarm status
	   may have chg'd
	*/
	if (pstringout -> latm.secPastEpoch == 0)
	{
	    tsLocalTime (&pstringout -> latm);
	    db_post_events ((struct dbCommon *)pstringout, 
					(caddr_t)pstringout -> val, DBE_VALUE | DBE_ALARM);
	}

	/* check event list */
	monitor(pstringout);

	/* process the forward scan link record */
	if (pstringout->flnk.type==DB_LINK) 
	    dbScanPassive((struct dbAddr *)pstringout->flnk.value.db_link.pdbAddr);

	/* update redundant fields to slave */

	redQInit ((struct dbCommon *)pstringout, redbuf, sizeof (redbuf), type);

	if (stat != pstringout -> stat)
	    redQField (redbuf, mylist [STAT]);

	if (sevr != pstringout -> sevr)
	    redQField (redbuf, mylist [SEVR]);

	if (strncmp (val, pstringout -> val, sizeof (val)))
	    redQField (redbuf, mylist [VAL]);

	if (strncmp (lovl, pstringout -> lovl, sizeof (lovl)))
	    redQField (redbuf, mylist [LOVL]);

	if (TsCmpStampsNE (&latm, &pstringout -> latm) )
	    redQField (redbuf, mylist [LATM]);

	redQSend (redbuf);

	pstringout->pact=FALSE;

	return(status);
}


/*------------------------------------------------------------------------*/

static long get_value(pstringout,pvdes)
    struct stringoutRecord             *pstringout;
    struct valueDes     *pvdes;
{
    pvdes->field_type = DBF_STRING;
    pvdes->no_elements=1;
    pvdes->pvalue = (caddr_t)(&pstringout->val[0]);
    return(0);
}


/*------------------------------------------------------------------------*/

static void monitor (pstringout)
    struct stringoutRecord             *pstringout;
{
    unsigned short  monitor_mask;
    short           stat,sevr,nsta,nsev;
    int	            set_ack = 1;

    /* get previous stat and sevr  and new stat and sevr*/
    stat=pstringout->stat;
    sevr=pstringout->sevr;
    nsta=pstringout->nsta;
    nsev=pstringout->nsev;

    /*set current stat and sevr*/
    pstringout->stat = nsta;
    pstringout->sevr = nsev;
    pstringout->nsta = 0;
    pstringout->nsev = 0;

    /* if we are coming out of VALID_ALARM and our new stat/sevr are
       the same as when we went into VALID_ALARM, do not set ack
    */
    if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	 (nsta == pstringout -> psta) && (nsev == pstringout -> psev) )
    {
	set_ack = 0;
    }
    /* if we are going into VALID_ALARM, and out previous stat was not
       VALID_ALARM, remember previous non VALID_ALARM stat/sevr
       and we do not want to set ACK,
    */
    if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
    {
	set_ack = 0;
	pstringout -> psta = stat;
	pstringout -> psev = sevr;
	db_post_events ((struct dbCommon *)pstringout, 
					(caddr_t)&pstringout -> psta, DBE_VALUE);
	db_post_events ((struct dbCommon *)pstringout, 
					(caddr_t)&pstringout -> psev, DBE_VALUE);
    }

    /* Flags which events to fire on the value field */
    monitor_mask = 0;

    /* alarm condition changed this scan */
    if (stat!=nsta || sevr!=nsev) 
    {
	/* post events for alarm condition change*/
	monitor_mask = DBE_ALARM;

	/* when alarm status chg's, get current time 
	*/
	tsLocalTime (&pstringout -> latm);

	/* post stat and nsev fields */
	db_post_events ((struct dbCommon *)pstringout, 
					(caddr_t)&pstringout->stat,DBE_VALUE);
	db_post_events ((struct dbCommon *)pstringout,
					(caddr_t)&pstringout->sevr,DBE_VALUE);

	if ( (pstringout -> sevr != NO_ALARM) && 
	     (set_ack) &&
	     (pstringout -> sevr != EVENT_ALARM) )
	{
	    if (pstringout -> ack == 0)
	    {
		pstringout -> ack = 1;
		db_post_events ((struct dbCommon *)pstringout, 
					(caddr_t)&pstringout -> ack, DBE_VALUE);
	    }
	}
    }

    if(strncmp(pstringout->oval,pstringout->val,sizeof(pstringout->val))) 
    {
        db_post_events ((struct dbCommon *)pstringout,
				(caddr_t)&(pstringout->val[0]),monitor_mask|DBE_VALUE);
	strncpy(pstringout->oval,pstringout->val,sizeof(pstringout->val));
    }

    return;
}


/*------------------------------------------------------------------------*/

static long special (paddr,after)
    struct dbAddr *paddr;
    int           after;
{
    struct stringoutRecord     *pstringout = (struct stringoutRecord *)(paddr->precord);
    int                 special_type = paddr->special;
    short		master;

    switch(special_type) 
    {

	case (SPC_SO_VAL):
	    /* if output gap is set and I am in SUPERVISORY mode 
	       (NOTE: dol!=link and omsl=closed_loop acts like supervisory), and 
	       my output channel is not a reference, and my resource is MASTER , 
	       set LOVL to this value, this will ensure that the VAL the user 
	       input will be output
	    */
	    if ( (pstringout -> ochg == 0) || 
		 ( (pstringout -> dol.type == DB_LINK) && 
		   (pstringout -> omsl == CLOSED_LOOP) ) ||
		 ( (pstringout -> dtyp <= 1) && (pstringout -> out.type == DB_LINK) ) )
		break;

	    if (resGetCaAnswer (pstringout -> radr, &master) == RES_OK)
	    {
		if (master)
		{
		    strcpy (pstringout -> lovl, "leetsmdrawde");
		}
	    }
	    break;

	default:
            recGblDbaddrError(S_db_badChoice,paddr,"so: special");
            return(S_db_badChoice);
    }

    return (0);
}


/*------------------------------------------------------------------------*/

static void test_ochg (pstringout, write_output)
    struct stringoutRecord	*pstringout;
    int				*write_output;
{
    long	status;
    long	options = 0;
    long	nRequests = 1;
    char	lovl[80];

    if (pstringout -> ochg == 0)
	return;

    /* if my output is a reference, then get its value, and set lovl
       (last val output) to the refs value. this protects us from the
       case where the output may be a field that has been changed
       from a graphic
    */
    if ( (pstringout -> dtyp <= 1) && (pstringout -> out.type == DB_LINK) )
    {
	status = dbGetLink (&pstringout -> out.value.db_link, 
			    (struct dbCommon *)pstringout, 
			    DBR_STRING, lovl, 
			    &options, &nRequests);
	if (status)
	{
	    *write_output = 0;
	    pstringout -> nsev = VALID_ALARM;
	    pstringout -> nsta = LINK_ALARM;
	    return;
	}
	strncpy (pstringout -> lovl, lovl, sizeof (pstringout -> lovl));
    }
    if (strncmp (pstringout -> val, pstringout -> lovl, 
	sizeof (pstringout -> val)) == 0)
	*write_output = 0;
    return;
}
