/*recTd.c,v 1.1.1.1 1999/08/03 14:44:23 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : recTd.c 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *
 *  DESCRIPTION: functions to process a timedelay (td) record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 02 Mar 94
 *  REVISION: added process stats code
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 94
 *  REVISION:  make forward func def agree with actual func def
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 31 May 94
 *  REVISION:	write init_record, take iact code and place in init_action
 *		a new func. new define MAX_TD_TYPE, if ptd->tdch is >= to
 *		this value, then block will not init
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 June 94
 *  REVISION:	add special to handle CBMD and CTCK fields
 *		new routine slave_callback() to handle callbacks when point
 *			is in slave mode, uses flag CBMD set in process_ival
 *		add NEXT, TVAL, CTCK, NVAL, CBMD and PVAL to redundancy list
 *		in tdCallback() do not do write_td if not MASTER
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 14 June 94
 *  REVISION:	added support for maximize severity (msev)
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 14 Oct 94
 *  REVISION:	add TIME to redundancy,added fpps(1st process pass), 
 *		added lval to test COS alarm because lalm has different job,
 *		do time stamp when stat or sevr chg,
 *		see monitor(),and not in process(),set lalm to val when 
 *		stat or sevr chg,in process() if fpps (first process)
 *		init lalm to val and if time stamp is 0, init it also
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Oct 94
 *  REVISION:	chg time to latm (last alarm time)
 *		added lptm (last process time)
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 13 Jan 95
 *  REVISION:	added mlst to redundancy, init latm and lptm in init_record
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 13 Mar 96
 *  REVISION:	added code to support:
 *			time remaining in callback
 *			cycle callback wit no reset when input signal drops
 *			cycle, start ON/OFF
 *			dwell time
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Apr 96
 *  REVISION:	added code to support alarm delay
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Jun 96
 *  REVISION:	added code for alarm disable ADIS
 *
****************************************************************************/


/****************************************************************************
 *
 * Programmer's Notes:
 *
 * Types of timedelay
 *
 *    PULSE_OS - pulse one shot, output goes high when input goes high
 *               if leading edge or when input goes low if trailing edge
 *               output stays high for ptd -> tdv1 even if get change
 *               in input
 *
 *    Leading     **************           ****   ***
 *    edge        *            *           *  *   * *
 *                *            *           *  *   * *
 *    input       *            *           *  *   * *
 *    *************            *************  ***** ************
 *
 *    Leading     ***********              ***********  
 *    edge        *         *              *         *  
 *                *         *              *         *  
 *    outout      *         *              *         *  
 *    *************         ****************         ******
 *
 *    Trailing    **************           ****   ***
 *    edge        *            *           *  *   * *
 *                *            *           *  *   * *
 *    input       *            *           *  *   * *
 *    *************            *************  ***** ************
 *
 *    Trailing                 ***********    ***********  
 *    edge                     *         *    *         *  
 *                             *         *    *         *  
 *    outout                   *         *    *         *  
 *    **************************         ******         ********
 *
 *
 *-------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 *
 *    PULSE_RST - pulse reset, output goes high when input goes high if
 *                leading edge or when input goes low if trailing edge,
 *                output stays high for ptd -> tdv1 or if input goes low
 *                if leading edge or input goes high if trailing edge,
 *                which ever comes first
 *
 *    Leading     **************           ****   ***
 *    edge        *            *           *  *   * *
 *                *            *           *  *   * *
 *    input       *            *           *  *   * *
 *    *************            *************  ***** ************
 *
 *    Leading     ***********              ****   ***
 *    edge        *         *              *  *   * *
 *                *         *              *  *   * *
 *    outout      *         *              *  *   * *
 *    *************         ****************  ***** ************
 *
 *    Trailing    **************           ****   ***
 *    edge        *            *           *  *   * *
 *                *            *           *  *   * *
 *    input       *            *           *  *   * *
 *    *************            *************  ***** ************
 *
 *    Trailing                 ***********    ***** ************ 
 *    edge                     *         *    *   * *   
 *                             *         *    *   * *   
 *    outout                   *         *    *   * *   
 *    **************************         ******   ***
 *
 *
 *--------------------------------------------------------------------------
 *
 *    TD_ENERGIZE - time delay on energize, output goes high ptd -> tdv1
 *                  time after input goes high, output drops when input
 *                  drops, if input drops before tdv1 time, output does not
 *                  go high, there is no TRAILING_EDGE
 *
 *
 *                **************           ***    *********
 *                *            *           * *    *       *
 *                *            *           * *    *       *
 *    input       *            *           * *    *       *
 *    *************            ************* ******       *******
 *                                          V
 *                    **********          less than   *****
 *                    *        *          tdv1,no     *   *
 *                    *        *          output      *   *
 *    output          *        *                      *   *
 *    *****************        ************************   *******
 *
 *
 *--------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 *
 *    TD_DEENERGIZE - time delay on deenergize, output goes high when input
 *                    does, output goes low tdv1 time after input goes low
 *                    there is no TRAILING_EDGE
 *
 *
 *                **************           ***  *********
 *                *            *           * *  *       *
 *                *            *           * *  *       *
 *    input       *            *           * *  *       *
 *    *************            ************* ****       *******
 *                                            V
 *                                          less than tdv1
 *                                          no output
 *
 *                                          
 *                *****************        *****************
 *                *               *        *               *
 *                *               *        *               *
 *    output      *               *        *               *
 *    *************               **********               *******
 *
 *
 *--------------------------------------------------------------------------
 *
 *    TD_CYCLE - time delay cycle, start to cycle output when input goes
 *               high if leading edge, or input goes low if traling edge
 *               continue to cycle as long as input remains high/low
 *
 *    Leading     ***************************************
 *    edge        *                                     *
 *                *                                     *
 *    input       *                                     *
 *    *************                                     **************
 *
 *    Leading     ********    ********    ********    ***
 *    edge        *      *    *      *    *      *    * *
 *                * tdv1 *tdv2*      *    *      *    * *
 *    output      *      *    *      *    *      *    * *
 *    *************      ******      ******      ****** **************
 *                             
 *
 *    *************                                     **************
 *    Trailing    *                                     *
 *    edge        *                                     *
 *    input       *                                     *
 *                ***************************************
 *
 *    Trailing    ********    ********    ********    ***
 *    edge        *      *    *      *    *      *    * *
 *                * tdv1 *tdv2*      *    *      *    * *
 *    output      *      *    *      *    *      *    * *
 *    *************      ******      ******      ****** **************
 *                             
 *
 *--------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 *
 *    TD_NO_RST_CYCLE - time delay cycle, start to cycle output when input goes
 *               	high if leading edge, or input goes low if traling edge
 *               	if input drops, finish the output high signal
 *
 *    Leading     ***************************************
 *    edge        *                                     *
 *                *                                     *
 *    input       *                                     *
 *    *************                                     **************
 *
 *    Leading     ********    ********    ********    ********
 *    edge        *      *    *      *    *      *    *      *
 *                * tdv1 *tdv2*      *    *      *    *      *
 *    output      *      *    *      *    *      *    *      *
 *    *************      ******      ******      ******      **************
 *                             
 *
 *    *************                                     **************
 *    Trailing    *                                     *
 *    edge        *                                     *
 *    input       *                                     *
 *                ***************************************
 *
 *    Trailing    ********    ********    ********    ********
 *    edge        *      *    *      *    *      *    *      *
 *                * tdv1 *tdv2*      *    *      *    *      *
 *    output      *      *    *      *    *      *    *      *
 *    *************      ******      ******      ******      **************
 *                             
 *--------------------------------------------------------------------------
 *	CYCLE START ON/OFF and DWELL TIME
 *
 *	START ON
 *
 *                ********         ********         ********
 *                *      *         *      *         *      *
 *                * tdv1 *tdv2 dwel* tdv1 *tdv2 dwel* tdv1 *
 *    output      *      *         *      *         *      *
 *    *************      *****I*****      *****I*****      **************
 *
 *
 *	START OFF
 *
 *                ********         ********         ********
 *                *      *         *      *         *      *
 *           tdv2 * tdv1 *dwel tdv2* tdv1 *dwel tdv2* tdv1 *
 *    output      *      *         *      *         *      *
 *    *************      *****I*****      *****I*****      **************
 *
 *
 *--------------------------------------------------------------------------
 *--------------------------------------------------------------------------
 *    callbackRequest () - see .../src/db/dbScan.c
 *    struct callback - see dbScan.c
 *    vxTicksPerSecond - see .../share/epicsH/dbDefs.h
 *
****************************************************************************/


#include	<vxWorks.h>
#include	<inetLib.h>
#include        <tickLib.h>        /* for tickGet () */
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<strLib.h>
#include	<wdLib.h>
#include	<stdlib.h>
#include	<logLib.h>
#include	<taskLib.h>

#include	"alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/tdRecord.h"
#include	"redflds.h"
#include	"redque.h"
#include	"tdredfldidx.h"
#include	"restable.h"
#include	"callbacks.h"
#include	"error.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"
#include	"okToUse.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/

#define report NULL
#define initialize NULL
static long init_record ();
static long process ();
static long special ();
static long get_value ();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
static long get_precision ();
static long get_enum_str ();
static long get_enum_strs ();
static long put_enum_str ();
static long get_graphic_double ();
static long get_control_double ();
#define get_alarm_double NULL

struct rset tdRSET = {
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double
	};

struct tddset {        /* timedelay output dset */
        long        number;
        DEVSUPFUN   dev_report;
        DEVSUPFUN   init;
        DEVSUPFUN   init_record;
        DEVSUPFUN   get_ioint_info;
        DEVSUPFUN   write_td;
        };

/* following definitions are for redundancy 
   NOTE: spelling and order in namelist MUST match that in
         .../share/epicsH/tdredfldidx.h and the
         spelling in .../share/ascii/tdRecord.ascii
*/

static struct fldDes  **mylist = 0;

static char *namelist [] = {
            "STAT",
            "SEVR",
            "VAL",
            "LALM",
            "PVAL",
            "IVAL",
	    "INI2",
	    "TVAL",
	    "CTCK",
	    "NVAL",
	    "LVAL",
	    "LATM",
	    "MLST",
	    "TMRM",
	    "NEXT",
	    "LCBM",
	    "CBMD",
	    "HHLL",
	    "RTCT",
            0
            };

static unsigned short type;    /* index for td point type */

/* defines for ptd -> tdch, type of time delay 
   NOTE: must match values in .../share/ascii/choiceRec.ascii
*/

#define PULSE_OS        0	/* pulse one shot */
#define PULSE_RST       1	/* pulse reset     */
#define TD_ENERGIZE     2	/* time delay on energize, input 0 -> 1 
				   state change */
#define TD_DEENERGIZE   3	/* time delay on deenergize, input 1 -> 0 
				   state change */
#define TD_CYCLE        4	/* cycle timer as long as input is 1 */
#define	TD_CYCLE_NO_RST	5	/* cycle timer with no reset */
#define MAX_TD_TYPE	(TD_CYCLE_NO_RST + 1)

/* defines for ptd -> lted, type of edge to time on
   NOTE: must match values in .../share/ascii/choiceRec.ascii
*/

#define LEADING_EDGE    0
#define TRAILING_EDGE   1

/* defines for callback active flag CBAC */

#define	CB_NOT_ACTIVE	0
#define	CB_ACTIVE	1

/* defines for trouble flag TFLG */

#define	NO_TROUBLE      0
#define TROUBLE         1

/* defines for the locking flag LFLG */

#define TD_NO_LOCK      0
#define TD_LOCK         1

/* defines for CBMD (callback mode field) */

#define	NO_CALLBACK_ACTIVE	0
#define PULSE_TDE_TDD_CB	1
#define CB_CYCLE_ON		2
#define	CB_CYCLE_OFF		3
#define	CB_CYCLE_DWELL		4

/* defines for the FRST (cycle first state) field 
   NOTE: must agree with values in .../share/ascii/choiceRec.ascii
*/

#define	CYCLE_ON_FIRST		0
#define	CYCLE_OFF_FIRST		1

static void alarm ();
static void decrement_tmrm ();
static long init_action ();
static void monitor ();
static long process_ival ();
static void slave_callback ();
static void tdCallback ();



/****************************************************************************
 *
 *  MODULE NAME            : init_record 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *          ARGUMENT(S)    : ptd - ptr to record
 *
 *  DESCRIPTION: 
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 03 Jan 94
 *  REVISION:  set ack to 0
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 28 Feb 94
 *  REVISION:  removed ack = 0, only call monitor if are MASTER
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 June 94
 *  REVISION:	handle new redundant fields, only call init_action() if we 
 *		are MASTER
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 25 July 94
 *  REVISION:	if dol == DB_LINK, must also be in closed loop before read
 *		input
 *
****************************************************************************/

static long init_record (ptd)
    struct tdRecord  *ptd;
{
    CALLBACK_PTR	pcallback;
    struct   tddset	*pdset;
    long		status = 0;
    char		*pstr;
    char		errmsg[128];
    char		redbuf[256];
    short		i;
    int			ini2_flag = 0;
    int			dont_init = 0;

    if (!mylist)
    {
	for (type = 1, i = 0; type < 100; type++)
	{
	    pstr = GET_PRECTYPE (type);
	    if (strncmp (pstr, "td", 2) == 0)
	    {
		i = 1;
		break;
	    }
	}
	if (!i)
	{
	    recGblRecordError (S_db_noRecType, (struct dbCommon *)ptd, "td: init_record");
	    return (S_db_noRecType);
	}
	mylist = redFldAlloc (type, NUMTDFLDS);

        if (!mylist)
        {
            recGblRecordError (S_rec_Alloc, (struct dbCommon *)ptd,
			   "td, init_record, mylist failed");
	    return (S_rec_Alloc);
        }

	for (i = 0; i < NUMTDFLDS; i++)
	    redFldAdd (type, i, namelist [i]);
    }

    if (!(pdset = (struct tddset *) (ptd -> dset))) 
    {
	recGblRecordError (S_dev_noDSET, (struct dbCommon *)ptd, "td: init_record");
	return (S_dev_noDSET);
    }
      /* must have the write_td function */

    if ((pdset -> number < 5) || (pdset -> write_td == NULL))
    {
	recGblRecordError (S_dev_missingSup, (struct dbCommon *)ptd, "td: init_record");
	return (S_dev_missingSup);
    }

    if (ptd -> tdch >= MAX_TD_TYPE)
    {
	recGblRecordError (S_db_badField, (struct dbCommon *)ptd, "td: init_record");
	return (S_db_badField);
    }

    if (!ptd -> tdcb)
    {
	if ((pcallback = (CALLBACK_PTR) (calloc (1, sizeof (CALLBACK)))) == NULL)
	{
	    sprintf (errmsg, "td: %s init_record calloc failed", ptd -> name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
	ptd -> tdcb = (caddr_t) pcallback;
	pcallback -> callback = tdCallback;
	pcallback -> priority = priorityMedium;
	if (dbNameToAddr (ptd->name, &(pcallback->dbAddr))) 
	{
	    logMsg("dbNameToAddr failed in init_record for recTd\n",0,0,0,0,0,0);
	    taskSuspend (taskIdSelf());
	}
    }

    ptd -> dtck = 0;			/* time delay tick count */
    ptd -> tmrm = 0;			/* time remaining */
    ptd -> tmct = 0;
    ptd -> tflg = NO_TROUBLE;		/* trouble flag */
    ptd -> lflg = TD_NO_LOCK;		/* locking flag */
    ptd -> cbmd = NO_CALLBACK_ACTIVE;	/* callback mode flag */
    ptd -> cbac = CB_NOT_ACTIVE;	/* callback active flag */
    ptd -> next = 0;
    ptd -> lcbm = NO_CALLBACK_ACTIVE;

    ptd -> init = TRUE;
    ptd -> fpps = 1;
    ptd -> udf = FALSE;
    ptd -> nsta = NO_ALARM;
    ptd -> nsev = NO_ALARM;
    ptd -> msev = NO_ALARM;

    /* init alarm delay variables */
    ptd -> larf = 0;
    ptd -> hhll = 0;
    ptd -> hilo = 0;
    ptd -> dvct = 0;
    ptd -> rtct = 0;

    /* init last alarm time and last proecss time */
    if (ptd -> latm.secPastEpoch == 0)
	tsLocalTime (&ptd -> latm);
    if (ptd -> lptm.secPastEpoch == 0)
	tsLocalTime (&ptd -> lptm);

    if (pdset -> init_record)
    {
	status = (*pdset -> init_record) (ptd, process);
	if (status)
	{
	    ptd -> nsta = NO_INIT_ALARM;
	    ptd -> nsev = VALID_ALARM;
	}
	else
	{
		/* get input value */
	    if ( (ptd -> dtyp <= 1) && (ptd -> dol.type == CONSTANT))
	    {
		if (ptd -> dol.value.value)
		    ptd -> ival = 1;
		else
		    ptd -> ival = 0;
	    }
	    else if (ptd -> dol.type == DB_LINK)
	    {
		if (ptd -> omsl == CLOSED_LOOP)
		{
		    long    options = 0;
		    long    nRequests = 1;

		    /* if dol has not been init'd yet, be cool and wait, try
		       again next time we are called
		    */

		    if (!(okToUse (&ptd -> dol)))
		    	dont_init = 1;
		    else
		    {
		    	status = dbGetLink (&(ptd -> dol.value.db_link),
					    (struct dbCommon *)ptd, 
					    DBR_USHORT, (caddr_t)&ptd -> ival, 
					    &options, &nRequests);
		    	if (status)
			    dont_init = 1;
		    }
		}
		else
		    ptd -> ival = 0;
	    }
	       /* invalid input type, do not let block init */
	    else
		dont_init = 1;
	    if (dont_init)
	    {
		ptd -> nsta = NO_INIT_ALARM;
		ptd -> nsev = VALID_ALARM;
	    }
	    else
	    {
		/* we only want to do this if we are MASTER, slave
		   will be told what to do 
		*/
    		if (resGetCaAnswer (ptd -> radr, &i) == RES_OK)
    		{
		    if (i)
		    {
		        /* init val and timers */
			status = init_action (ptd);
			if (status)
			    dont_init = 1;
			else
			    ini2_flag = 1;
		    }
		    else
			dont_init = 1;
		}
		else
		    dont_init = 1;
		if (dont_init)
		{
		    ptd -> nsta = NO_INIT_ALARM;
		    ptd -> nsev = VALID_ALARM;
		}
	    }
	}
    }
    else
	ini2_flag = 1;

    /* init last value, use by alarm() to test COS alarm */

    ptd -> lval = ptd -> val;

    if (resGetCaAnswer (ptd -> radr, &i) == RES_OK)
    {
	if (i)
	{
	    /* check for maximize severity */

	    if (ptd -> msev > ptd -> nsev)
	    {
		ptd -> nsev = ptd -> msev;
		ptd -> nsta = LINK_ALARM;
	    }

	    monitor (ptd);

	    if (ini2_flag)
	    {
		ptd -> ini2 = 1;

		redQInit ((struct dbCommon *)ptd, redbuf, sizeof (redbuf), type);
		redQField (redbuf, mylist [STAT]);
		redQField (redbuf, mylist [SEVR]);
		redQField (redbuf, mylist [VAL]);
		redQField (redbuf, mylist [LALM]);
		redQField (redbuf, mylist [PVAL]);
		redQField (redbuf, mylist [IVAL]);
		redQField (redbuf, mylist [INI2]);
		redQField (redbuf, mylist [TVAL]);
		redQField (redbuf, mylist [CTCK]);
		redQField (redbuf, mylist [NVAL]);
		redQField (redbuf, mylist [LVAL]);
		redQField (redbuf, mylist [TMRM]);
		redQField (redbuf, mylist [NEXT]);
		redQField (redbuf, mylist [LCBM]);

		/* NOTE: CBMD must be last, a special is attached to it,
			 that may cause a callback to start on the slave
			 and the above values must already be there
		*/
		redQField (redbuf, mylist [CBMD]);

		redQSend (redbuf);
	    }
	}
    }

   return (status);

} /* end func init_record () */



/****************************************************************************
 *
 *  MODULE NAME            : alarm 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *          ARGUMENT(S)    : ptd - ptr to record
 *
 *  DESCRIPTION: update alarm status for record
 *
****************************************************************************/

static void alarm (ptd)
    struct tdRecord   *ptd;
{
    unsigned short	val = ptd -> val;
    int			adis = 0;

    adis = AlarmDisableLink ((void *)ptd);
    if ( (ptd->aenb == ALARM_DISABLED) || (adis != 0) )
    {
	ptd -> nsta = INHIBIT_ALARM;
	ptd -> nsev = NO_ALARM;
	return;
    }
    /* check for udf alarm for input value */

    if(ptd -> udf == TRUE )
    {
	if (ptd -> nsev < VALID_ALARM) 
	{
	    ptd -> nsta = UDF_ALARM;
	    ptd -> nsev = VALID_ALARM;
	}
    }
	
    if (ptd -> nsev == VALID_ALARM)
	return;

    /* check for  state alarm */

    if (val == 0) 
    {
	if (ptd -> nsev < ptd -> zsv) 
	{
	    ptd -> nsta = STATE_ALARM;
	    ptd -> nsev = ptd -> zsv;
	}
    }
    else 
    {
	if (ptd -> nsev < ptd -> osv)
	{
	    ptd -> nsta = STATE_ALARM;
	    ptd -> nsev = ptd -> osv;
	}
    }

    /* check for output of STATE alarm */

    AlarmStates ((void *)ptd, 0);

    /* check for cos alarm */

    if (val != ptd -> lval)
    {
	if (ptd -> nsev < ptd -> cosv)
	{
	    ptd -> nsta = COS_ALARM;
	    ptd -> nsev = ptd -> cosv;
	}
    }

    ptd -> lval = val;

    /* check for return to normal */

    AlarmStates ((void *)ptd, 1);

    return;

} /* end func alarm () */


/****************************************************************************
 *
 *  MODULE NAME            : decrement_tmrm
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Mar 96
 *
****************************************************************************/

static void decrement_tmrm (ptd)
    struct tdRecord	*ptd;
{
    float	tempflt;

    if ( (ptd -> tmrm > 0) && (ptd -> ctck > ptd -> tmct) )
    {
	tempflt = (ptd -> ctck - ptd -> tmct) / (float)vxTicksPerSecond;
	ptd -> tmrm -= tempflt;
	if (ptd -> tmrm < 0)
	    ptd -> tmrm = 0;
	ptd -> tmct = ptd -> ctck;
    }
    return;
}


/****************************************************************************
 *
 *  MODULE NAME            : get_value 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
****************************************************************************/

static long get_value (ptd, pvdes)
   struct tdRecord    *ptd;
   struct valueDes           *pvdes;
{
   pvdes -> field_type = DBF_ENUM;
   pvdes -> no_elements = 1;
   (unsigned short *)(pvdes -> pvalue) = &ptd -> val;
   return (0);

} /* end func get_value () */


/****************************************************************************
 *
 *  MODULE NAME            : get_enum_str 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
****************************************************************************/

static long get_enum_str (paddr, pstring)
   struct dbAddr    *paddr;
   char             *pstring;
{
   struct tdRecord *ptd = (struct tdRecord *)paddr -> precord;
   unsigned short  *field = (unsigned short *)paddr -> pfield;

   if ((*field) == 0)
   {
      strncpy (pstring, ptd -> znam, sizeof (ptd -> znam));
      pstring[sizeof (ptd -> znam)] = 0;
   } 
   else if ((*field) == 1)
   {
      strncpy (pstring, ptd -> onam, sizeof (ptd -> onam));
      pstring[sizeof (ptd -> onam)] = 0;
   }
   else
      strcpy (pstring, "Illegal_Value");
   return (0);

} /* end func get_enum_str () */

      

/****************************************************************************
 *
 *  MODULE NAME            : get_enum_strs 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *
****************************************************************************/

static long get_enum_strs (paddr, pes)
   struct dbAddr        *paddr;
   struct dbr_enumStrs  *pes;
{
   struct tdRecord *ptd = (struct tdRecord *)paddr -> precord;

   pes -> no_str = 2;
   bzero ((char *)(&pes -> strs[0]), sizeof (pes -> strs));
   strncpy (pes -> strs[0], ptd -> znam, sizeof (ptd -> znam));
   strncpy (pes -> strs[1], ptd -> onam, sizeof (ptd -> onam));
   return (0);

} /* end func get_enum_strs () */


/****************************************************************************
 *
 *  MODULE NAME            : put_enum_str
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *
****************************************************************************/

static long put_enum_str (paddr, pstring)
   struct dbAddr    *paddr;
   char             *pstring; 
{
   struct tdRecord *ptd = (struct tdRecord *)paddr -> precord;
   unsigned short  *field = (unsigned short *)paddr -> pfield;

   if (strncmp (pstring, ptd -> znam, sizeof (ptd -> znam)) == 0)
      *field = 0;
   else if (strncmp (pstring, ptd -> onam, sizeof (ptd -> onam)) == 0)
      *field = 1;
   else
      return (S_db_badChoice);
   return (0);

} /* end func put_enum_str () */


/****************************************************************************
 *
 *  MODULE NAME            : get_control_double
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Mar 96
 *
 *
****************************************************************************/

static long get_control_double(paddr,pcd)
    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct tdRecord *)0) -> tdv1) ):
	case ( (int)&(((struct tdRecord *)0) -> tdv2) ):
	case ( (int)&(((struct tdRecord *)0) -> dwel) ):
	    pcd -> upper_ctrl_limit = 99999.99;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	default:
	    pcd -> upper_ctrl_limit = 100;
	    pcd -> lower_ctrl_limit = 0;
	    break;
    }
    return (0);
}


/****************************************************************************
 *
 *  MODULE NAME            : get_graphic_double
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Mar 96
 *
 *
****************************************************************************/

static long get_graphic_double(paddr,pgd)
    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct tdRecord *)0) -> tdv1) ):
	case ( (int)&(((struct tdRecord *)0) -> tdv2) ):
	case ( (int)&(((struct tdRecord *)0) -> dwel) ):
	    pgd -> upper_disp_limit = 99999.99;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    pgd->upper_disp_limit = 100;
	    pgd->lower_disp_limit = 0;
	    break;
    }
    return (0);
}


/****************************************************************************
 *
 *  MODULE NAME            : get_precision
 *  AUTHOR                 : emsteel
 *  DATE                   : 13 Mar 96
 *
 *
****************************************************************************/

static long get_precision (paddr, precision)
    struct	dbAddr	*paddr;
    long		*precision;
{
    *precision = 2;
    return (0);
}


/****************************************************************************
 *
 *  MODULE NAME            : init_action 
 *  AUTHOR                 : emsteel
 *  DATE                   : 27 May 94
 *
 *          ARGUMENT(S)    : ptd - ptr to record
 *
 *  DESCRIPTION:  called from init_record to update val according to ival
 *		  and the type of IACT (init action) chosen if any
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 25 July 94
 *  REVISION:	set VAL and write to output
 *
****************************************************************************/

static long init_action (ptd)
    struct tdRecord	*ptd;
{
    long	status = 0;
    int		write_output = 0;
    int		do_invert = 1;
    int		update_pval = 1;
    struct tddset   *pdset = (struct tddset *) (ptd -> dset);

      /* set output according to input value tdch (type of timedelay), 
         if iact (init action), set timer in motion
      */

    switch (ptd -> tdch)
    {
	case PULSE_OS:        /* pulse one shot */
	case PULSE_RST:       /* pulse reset    */

	    if (ptd -> lted == LEADING_EDGE)
	    {
		if (ptd -> iact && (ptd -> ival == 1) )
		{
		    ptd -> pval = 0;
		    process_ival (ptd, &update_pval);
		}
		else
		{
		    /* set output, assume pulse is over
		       handle invert here so we get proper 
		       output value
		    */
		    if (ptd -> invt)
		        ptd -> val = 1;
		    else
			ptd -> val = 0;
		    do_invert = 0;
		    write_output++;
		}
	    }
	    else	/* TRAILING_EDGE */
	    {
		if (ptd -> iact && (ptd -> ival == 0) )
		{
		    ptd -> pval = 1;
		    process_ival (ptd, &update_pval);
		}
		else
		{
		    /* set output, assume pulse is over
		       handle invert here so we get proper 
		       output value
		    */
		    if (ptd -> invt)
		        ptd -> val = 1;
		    else
			ptd -> val = 0;
		    do_invert = 0;
		    write_output++;
		}
	    }

	    break;

	case TD_ENERGIZE:     /* time delay on energize */

	    if ( (ptd -> ival == 1) && ptd -> iact)
	    {
		ptd -> pval = 0;
		process_ival (ptd, &update_pval);

		/* set val according to tval as set in process_ival() */

		if (ptd -> tval)
		    ptd -> val = 0;
		else
		    ptd -> val = 1;
		do_invert = 0;
	    }
	    else
		ptd -> val = ptd -> ival;
	    write_output++;

	    break;

	case TD_DEENERGIZE:   /* time delay on de-energize */

	    /* since we do not know what the status was just before init */
	    ptd -> val = ptd -> ival;
	    write_output++;

	    break;

	case TD_CYCLE:		/* cycle timer */
	case TD_CYCLE_NO_RST:	/* cycle timer with no reset */

	    if ( (ptd -> ival == 0) && (ptd -> lted == TRAILING_EDGE) )
	    {
		ptd -> pval = 1;
		process_ival (ptd, &update_pval);
	    }
	    else if ( (ptd -> ival == 1) && (ptd -> lted == LEADING_EDGE) )
	    {
		ptd -> pval = 0;
		process_ival (ptd, &update_pval);
	    }
	    else
	    {
		ptd -> val = ptd -> ival;
		write_output++;
	    }

	    break;

    }  /* end switch (ptd -> tdch) */

    ptd -> pval = ptd -> ival;
    if (write_output)
    {
	if (ptd -> invt && do_invert)
	{
	    if (ptd -> ival)
		ptd -> val = 0;
	    else
		ptd -> val = 1;
	}
	if (ptd -> mask != 0)
	{
	    if (ptd -> val == 0)
		ptd -> rval = 0;
	    else
		ptd -> rval = ptd -> mask;
	}
	ptd -> nval = ptd -> val;
	status = (*pdset -> write_td) (ptd);
    }
    return (status);
}


/****************************************************************************
 *
 *  MODULE NAME            : monitor
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *          ARGUMENT(S)    : ptd - ptr to record
 *
 *  DESCRIPTION: update monitors for status and val
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 05 Jan 94
 *  REVISION:  check VALID_ALARM before set ACK
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 30 Aug 94
 *  REVISION:	rm DBE_LOG from ack, only set DBE_LOG on val if tenb
 *		(trending) is enabled.
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 15 Mar 95
 *  REVISION:	do not set ack if sevr is EVENT_ALARM
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Sept 95
 *  REVISION:	archive if stat/sevr also chg's
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Sept 95
 *  REVISION:	chg way handle VALID_ALARM
 *
****************************************************************************/

static void monitor (ptd)
   struct tdRecord    *ptd;
{
    unsigned short    monitor_mask;
    short             stat, sevr, nsta, nsev;
    int		      set_ack = 1;

      /* get previous stat and sevr  and new stat and sevr */

    stat = ptd -> stat;
    sevr = ptd -> sevr;
    nsta = ptd -> nsta;
    nsev = ptd -> nsev;

      /*set current stat and sevr*/

    ptd -> stat = nsta;
    ptd -> sevr = nsev;
    ptd -> nsta = 0;
    ptd -> nsev = 0;

    /* if we are coming out of VALID_ALARM and our new stat/sevr are
       the same as when we went into VALID_ALARM, do not set ack
    */
    if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	 (nsta == ptd -> psta) && (nsev == ptd -> psev) )
    {
	set_ack = 0;
    }
    /* if we are going into VALID_ALARM, and out previous stat was not
       VALID_ALARM, remember previous non VALID_ALARM stat/sevr
       and we do not want to set ACK,
    */
    if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
    {
	set_ack = 0;
	ptd -> psta = stat;
	ptd -> psev = sevr;
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> psta, DBE_VALUE);
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> psev, DBE_VALUE);
    }

    monitor_mask = 0;

      /* alarm condition changed this scan */

      /*
       Extra Patch: If the user defines his output to have
       the same alarm severity for both states and the value
       is monitored with only the DBE_ALARM mask, then he'll
       never see that state change. This patch will avoid
       that problem.

       Extra Extra Patch: If we have a passive  TIMEDELAY with 
       only COS alarming, then we will get the first alarm on the
       first state change and then no others because stat and
       sevr never change even though the value has.
      */

    if ( ( ((ptd -> nsta == STATE_ALARM) && (ptd -> zsv == ptd -> osv))
         || (ptd -> nsta == COS_ALARM) ) && (ptd -> mlst != ptd -> val) )
	stat = ~nsta;

    if ((stat != nsta) || (sevr != nsev))
    {
	/* post events for alarm condition change */

	monitor_mask = DBE_ALARM;

	/* when alarm status chg's, get current time and
	   save VAL in lalm (last value when alarm chg'd)
	*/
	tsLocalTime (&ptd -> latm);
	ptd -> lalm = ptd -> val;

	/* post stat and sevr fields */

	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> stat, DBE_VALUE);
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> sevr, DBE_VALUE);
	if ((ptd -> sevr != NO_ALARM) && (set_ack) &&
	    (ptd -> sevr != EVENT_ALARM) )
	{
	    if (ptd -> ack == 0)
	    {
		ptd -> ack = 1;           /* new unacknowledged alarm */
		db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> ack, DBE_VALUE);
	    }
	}
    }
      /* check for value change */

    if (ptd -> mlst != ptd -> val)
    {
	/* post events for value change and archive change */

	monitor_mask |= DBE_VALUE;
    }

    if (ptd -> tenb)
    {
	if ( (ptd -> mlst != ptd -> val) || (monitor_mask & DBE_ALARM) )
	    monitor_mask |= DBE_LOG;
    }

    if (monitor_mask & (DBE_VALUE | DBE_LOG))
	ptd -> mlst = ptd -> val;

      /* send out monitors connected to the value field etc */

    if (monitor_mask)
    {
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> val, monitor_mask);
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> nval, DBE_VALUE);
    }

    if (ptd -> linp != ptd -> ival)
    {
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> ival, DBE_VALUE);
	ptd -> linp = ptd -> ival;
    }

    if (ptd -> oraw != ptd -> rval)
    {
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> rval, DBE_VALUE);
	ptd -> oraw = ptd -> rval;
    }
    if (ptd -> orbv!=ptd -> rbv)
    {
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> rbv, DBE_VALUE);
	ptd -> orbv = ptd -> rbv;
    }

    return;

} /* end func monitor () */



/****************************************************************************
 *
 *  MODULE NAME            : process 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *
 *  DESCRIPTION: process a time delay record
 *               after wait_time, callbackRequest is called by the watchdog
 *               task, it in turn places pcallback in a ring buffer and does
 *               a semGive, callbackTask (dbScan.c) grabs pcallback and
 *               executes tdCallback ()
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 28 Feb 94
 *  REVISION:  dont have to check for MASTER for redundancy, if process is
 *             is called, we have to be MASTER
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 June 94
 *  REVISION:	added new fields to redundancy, init cbmd
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 10 June 94
 *  REVISION:	added call to tsLocalTime if on scan
 *
****************************************************************************/

static long process (paddr)
   struct dbAddr  *paddr;
{
    struct tdRecord	*ptd = (struct tdRecord *) (paddr -> precord);
    struct tddset	*pdset = (struct tddset *) (ptd -> dset);
    long		status = 0;
    char		redbuf [256];
    int			update_pval = 1;

    unsigned short	stat = ptd -> stat;     /* save these for redundancy */
    unsigned short	sevr = ptd -> sevr;
    unsigned short	val  = ptd -> val;
    unsigned short	lalm = ptd -> lalm;
    unsigned short	pval = ptd -> pval;
    unsigned short	ival = ptd -> ival;
    unsigned short	tval = ptd -> tval;
    unsigned short	nval = ptd -> nval;
    unsigned long	ctck = ptd -> ctck;
    short		next = ptd -> next;
    unsigned short	lval = ptd -> lval;
    TS_STAMP		latm = ptd -> latm;
    unsigned short	mlst = ptd -> mlst;
    float		tmrm = ptd -> tmrm;
    short		cbmd = ptd -> cbmd;
    short		hhll = ptd -> hhll;
    short		rtct = ptd -> rtct;
    short		send_cbmd = 0;

      /* if not init'd, do nothing */

    if ((!ptd -> init) || (!ptd -> ini2))
	return (status);

    if ((pdset == NULL) || (pdset -> write_td == NULL))
    {
	ptd -> pact = TRUE;
	recGblRecordError (S_dev_missingSup, (struct dbCommon *)ptd, "td: process");
	return (S_dev_missingSup);
    }

    /* get time stamp for last time record was processed */

    tsLocalTime (&ptd -> lptm);

    ptd -> msev = NO_ALARM;

    if (ptd -> tagd)
    {
	ptd -> nsta = TAGGED_ALARM;
	ptd -> nsev = VALID_ALARM;
    }
    else if (ptd -> oscn == OFF_SCAN)
    {
	ptd -> nsta = SCAN_ALARM;
	ptd -> nsev = VALID_ALARM;
    }
    else
    {
         /* set active so can't be processed while in here */

	ptd -> pact = TRUE;

	/* update number of process times */
	if (process_stats)
	    process_cnts[type]++;	

         /* setup for new alarm condition */

	ptd -> nsta = NO_ALARM;
	ptd -> nsev = NO_ALARM;

	if ((ptd -> dol.type == DB_LINK) && (ptd -> omsl == CLOSED_LOOP))
	{
	    long  options = 0;
	    long  nRequest = 1;
	    unsigned short ival;

	    status = dbGetLink (&ptd -> dol.value.db_link, 
				(struct dbCommon *)ptd, 
				DBR_USHORT, (caddr_t)&ival, 
				&options, &nRequest);
	    if (status == 0)
	    {
		ptd -> ival = ival;
		ptd -> udf = FALSE;
	    }
	    else
	    {
		if (ptd -> nsev < VALID_ALARM)
		{
		    ptd -> nsta = LINK_ALARM;
		    ptd -> nsev = VALID_ALARM;
		}
	    }
	}


	if (status == 0)
	{
            /* get current clock tick count */

	    ptd -> ctck = tickGet ();
	    status = process_ival (ptd, &update_pval);
	    if (update_pval)
		ptd -> pval = ptd -> ival;
	}

         /* check for alarms */

	alarm (ptd);

	/* check for maximize severity */

	if (ptd -> msev > ptd -> nsev)
	{
	    ptd -> nsev = ptd -> msev;
	    ptd -> nsta = LINK_ALARM;
	}
    }

    /* if the time stamp is 0, init and post val with
       alarm bit set, this makes sure that proper alarm
       messages are set if IOC was down and alarm status
       may have chg'd
    */
    if (ptd -> latm.secPastEpoch == 0)
    {
	tsLocalTime (&ptd -> latm);
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> val, DBE_VALUE | DBE_ALARM);
    }

    /* if it is the 1st process pass 
       init last value when alarm chg'd (lalm)
    */

    if (ptd -> fpps)
    {
	ptd -> fpps = 0;
	ptd -> lalm = ptd -> val;
    }

      /* check event list */

    monitor (ptd);

      /* update redundant fields */

    redQInit ((struct dbCommon *)ptd, redbuf, sizeof (redbuf), type);

    if ( (cbmd != ptd -> cbmd) || (ptd -> cbmd != ptd -> lcbm) )
	send_cbmd = 1;

    if (stat != ptd -> stat)
	redQField (redbuf, mylist [STAT]);

    if (sevr != ptd -> sevr)
	redQField (redbuf, mylist [SEVR]);

    if ( (val != ptd -> val) || send_cbmd)
	redQField (redbuf, mylist [VAL]);

    if (lalm != ptd -> lalm)
	redQField (redbuf, mylist [LALM]);

    if (ival != ptd -> ival)
	redQField (redbuf, mylist [IVAL]);

    if (pval != ptd -> pval)
	redQField (redbuf, mylist [PVAL]);

    if ( (tval != ptd -> tval) || send_cbmd)
	redQField (redbuf, mylist [TVAL]);

    if ( (nval != ptd -> nval) || send_cbmd)
	redQField (redbuf, mylist [NVAL]);

    if (ctck != ptd -> ctck)
	redQField (redbuf, mylist [CTCK]);

    if (TsCmpStampsNE (&latm, &ptd -> latm) )
	redQField (redbuf, mylist [LATM]);
	
    if (lval != ptd -> lval)
	redQField (redbuf, mylist [LVAL]);
	
    if (mlst != ptd -> mlst)
	redQField (redbuf, mylist [MLST]);
	
    if (hhll != ptd -> hhll)
	redQField (redbuf, mylist [HHLL]);
	
    if (rtct != ptd -> rtct)
	redQField (redbuf, mylist [RTCT]);
	
    if ( (tmrm != ptd -> tmrm) || send_cbmd)
    {
	redQField (redbuf, mylist [TMRM]);
	db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> tmrm, DBE_VALUE);
    }

    if ( (next != ptd -> next) || send_cbmd)
	redQField (redbuf, mylist [NEXT]);

    /* NOTE: this must be the last to be sent, it has a
       special attached to it that depends on the others
       already being there
    */
    if (send_cbmd)
    {
	ptd -> lcbm = ptd -> cbmd;
	redQField (redbuf, mylist [LCBM]);
	redQField (redbuf, mylist [CBMD]);
    }

    redQSend (redbuf);

      /* process the forward scan link record */

    if (ptd -> flnk.type == DB_LINK)
	dbScanPassive((struct dbAddr *)ptd -> flnk.value.db_link.pdbAddr);

    ptd -> pact = FALSE;

    return (status);

} /* end func process () */


/****************************************************************************
 *
 *  MODULE NAME            : process_ival 
 *  AUTHOR                 : emsteel
 *  DATE                   : 02 Nov 93
 *
 *  CALLING MODULE(S)      : process
 *          ARGUMENT(S)    : ptd - ptr to timedelay record
 *                           update_pval - int ptr, default is 1
 *
 *  DESCRIPTION: start/stop timedelay
 *  NOTES:
 *    ptd -> lflg, if set to TD_LOCK, tdCallback will do nothing
 *    update_pval - set to 1 by caller, if needed, set to 0 by this func,
 *                  delays ival == pval for certain conditions
 *
 *    PULSE_OS
 *          ival == 0 and LEADING_EDGE, do nothing
 *          ival == 1 and TRAILING_EDGE, do nothing
 *
 *    PULSE_RST
 *          ival == 0
 *             LEADING_EDGE
 *                if timer is active, deactivate and do callback
 *                if timer not active, do nothing, maybe already doing
 *                   callback, set TROUBLE just in case
 *             TRAILING_EDGE
 *                only test for cbmd, could be in between timer expired
 *                and doing callback, should not get here with timer
 *                still active, I HOPE !!!
 *          ival == 1
 *             LEADING_EDGE
 *                same as ival == 0 and TRAILING_EDGE
 *             TRAILING_EDGE
 *                same as ival == 0 and LEADING_EDGE
 *
 *    TD_ENERGIZE
 *          ival == 0
 *             if timer is active, then input has dropped before timer
 *                has expired, deactivate timer thereby not setting output
 *                high, but if cbmd is still set, we are in that twilight
 *                zone between expired timer and output of signal, let it
 *                happen
 *             else drop output
 *          ival == 1
 *             if ptd -> cbmd not set start timer etc
 *
 *    TD_DEENERGIZE
 *          ival == 0
 *             if ptd -> cbmd not set start timer etc
 *          ival == 1
 *             if timer active then input has gone high before the timer
 *             has expiered, deactiveate timer thereby not setting output
 *             to low
 *
 *    TD_CYCLE
 *          if timer is active, deactivate, set tval, do callback, if timer 
 *          not active then callback may or not be in progress, in either 
 *          case do nothing, cycling starts in this function, but continues 
 *          via the callback func tdCallback
 *
 *    TD_CYCLE_NO_RST
 *          if timer is active, set next to 4, so that the output will
 *	    remain high for rest ot TDV1
 *          not active then callback may or not be in progress, in either 
 *          case do nothing, cycling starts in this function, but continues 
 *          via the callback func tdCallback
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 June 94
 *  REVISION:	set cbmd flag for the slave
 *
****************************************************************************/

static long process_ival (ptd, update_pval)
   struct tdRecord   *ptd;
   int               *update_pval;
{
   CALLBACK_PTR		pcallback = (CALLBACK_PTR) (ptd -> tdcb);
   struct tddset	*pdset = (struct tddset *) (ptd -> dset);
   int			wait_time = 0;
   long			status = 0;
   int			write_output = 0;
   int			start_timer = 0;

      /* switch on type of timer */

    switch (ptd -> tdch)
    {
	case PULSE_OS:       /* one shot pulse */

            /* if input has chg'd and the timer is not active,
               and the callback flag is 0
            */

	    if ( (ptd -> ival != ptd -> pval) &&
		 (ptd -> dtck == 0)    &&
		 (ptd -> cbmd == NO_CALLBACK_ACTIVE) )
	    {
		if ( ((ptd -> ival == 0) && (ptd -> lted == TRAILING_EDGE)) ||
		     ((ptd -> ival == 1) && (ptd -> lted == LEADING_EDGE)) )
		{
		    if (ptd -> invt)
		    {
			ptd -> val = 0;
			ptd -> tval = 1;
		    }
		    else
		    {
			ptd -> val = 1;
			ptd -> tval = 0;
		    }
		    ptd -> tflg = NO_TROUBLE;
		    write_output++;
		    wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
		    if (wait_time > 0)
		    {
			ptd -> tmrm = ptd -> tdv1;
			ptd -> tmct = ptd -> ctck;
			ptd -> cbmd = PULSE_TDE_TDD_CB;
			ptd -> next = 5;
			start_timer++;
		    }
		}
	    }
	    else if ( (ptd -> cbmd == PULSE_TDE_TDD_CB) || (ptd -> tmrm > 0) )
	    {
		decrement_tmrm (ptd);
	    }

	    break;

	case PULSE_RST:      /* pulse reset */

	    if (ptd -> ival != ptd -> pval)
	    {
		if ( ((ptd -> ival == 0) && (ptd -> lted == LEADING_EDGE)) ||
		     ((ptd -> ival == 1) && (ptd -> lted == TRAILING_EDGE)) )
		{
		    if (ptd -> cbmd != NO_CALLBACK_ACTIVE)
		    {
			ptd -> lflg = TD_LOCK;
			if ( (ptd -> dtck > ptd -> ctck) || (ptd -> tflg == TROUBLE) )
			{
			    wdCancel (pcallback -> wd_id);
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> next = 0;
			    ptd -> tmrm = 0;
			    if (ptd -> invt)
				ptd -> val = 1;
			    else
				ptd -> val = 0;
			    ptd -> tflg = NO_TROUBLE;
			    ptd -> cbmd = NO_CALLBACK_ACTIVE;
			    write_output++;
			}
			else
			{
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> tflg = TROUBLE;
			    *update_pval = 0;
			}
		    }
		}
		else if ( ((ptd -> ival == 0) && (ptd -> lted == TRAILING_EDGE)) ||
			  ((ptd -> ival == 1) && (ptd -> lted == LEADING_EDGE)) )
		{
		    if (ptd -> cbmd == NO_CALLBACK_ACTIVE)
		    {
			if (ptd -> invt)
			{
			    ptd -> val = 0;
			    ptd -> tval = 1;
			}
			else
			{
			    ptd -> val = 1;
			    ptd -> tval = 0;
			}
			ptd -> tflg = NO_TROUBLE;
			write_output++;
			wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
			if (wait_time > 0)
			{
			    ptd -> tmrm = ptd -> tdv1;
			    ptd -> tmct = ptd -> ctck;
			    ptd -> next = 5;
			    ptd -> cbmd = PULSE_TDE_TDD_CB;
			    start_timer++;
			}
		    }
		}
	    }
	    else if ( (ptd -> cbmd == PULSE_TDE_TDD_CB) || (ptd -> tmrm > 0) )
	    {
		decrement_tmrm (ptd);
	    }

	    break;

	case TD_ENERGIZE:    /* time delay on energize */

	    if (ptd -> ival != ptd -> pval)
	    {
		if (ptd -> ival == 0)
		{
		    if (ptd -> cbmd != NO_CALLBACK_ACTIVE)
		    {
			ptd -> lflg = TD_LOCK;
			if ( (ptd -> dtck > ptd -> ctck) || (ptd -> tflg == TROUBLE) )
			{
			    wdCancel (pcallback -> wd_id);
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> tflg = NO_TROUBLE;
			    ptd -> dtck = 0;
			    ptd -> cbmd = NO_CALLBACK_ACTIVE;
			    ptd -> next = 0;
			    ptd -> tmrm = 0;
			}
			else
			{
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> tflg = TROUBLE;
			    *update_pval = 0;
			}
		    }
		    else
		    {
			ptd -> tflg = NO_TROUBLE;
			ptd -> cbmd = NO_CALLBACK_ACTIVE;
			ptd -> next = 0;
			if (ptd -> invt)
			    ptd -> val = 1;
			else
			    ptd -> val = 0;
			write_output++;
		    }
		}
		else     /* ival == 1 */
		{
		    if (ptd -> cbmd == NO_CALLBACK_ACTIVE)
		    {
			if (ptd -> invt)
			    ptd -> tval = 0;
			else
			    ptd -> tval = 1;
			ptd -> tflg = NO_TROUBLE;
			wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
			if (wait_time > 0)
			{
			    ptd -> tmrm = ptd -> tdv1;
			    ptd -> tmct = ptd -> ctck;
			    ptd -> next = 5;
			    ptd -> cbmd = PULSE_TDE_TDD_CB;
			    start_timer++;
			}
		    }
		}
	    }
	    else if ( (ptd -> cbmd == PULSE_TDE_TDD_CB) || (ptd -> tmrm > 0) )
	    {
		decrement_tmrm (ptd);
	    }

         break;

	case TD_DEENERGIZE:  /* time delay on deenergize */

	    if (ptd -> ival != ptd -> pval)
	    {
		if (ptd -> ival == 0)
		{
		    if (ptd -> cbmd == NO_CALLBACK_ACTIVE)
		    {
			if (ptd -> invt)
			    ptd -> tval = 1;
			else
			    ptd -> tval = 0;
			ptd -> tflg = NO_TROUBLE;
			wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
			if (wait_time > 0)
			{
			    ptd -> tmrm = ptd -> tdv1;
			    ptd -> tmct = ptd -> ctck;
			    ptd -> next = 5;
			    ptd -> cbmd = PULSE_TDE_TDD_CB;
			    start_timer++;
			}
		    }
		}
		else     /* ival == 1 */
		{
		    if (ptd -> cbmd != NO_CALLBACK_ACTIVE)
		    {
			ptd -> lflg = TD_LOCK;
			if ( (ptd -> dtck > ptd -> ctck) || (ptd -> tflg == TROUBLE) )
			{
			    wdCancel (pcallback -> wd_id);
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> tflg = NO_TROUBLE;
			    ptd -> dtck = 0;
			    ptd -> cbmd = NO_CALLBACK_ACTIVE;
			    ptd -> next = 0;
			    ptd -> tmrm = 0;
			}
			else
			{
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> tflg = TROUBLE;
			    *update_pval = 0;
			}
		    }
		    else
		    {
			    if (ptd -> invt)
				ptd -> val = 0;
			    else
				ptd -> val = 1;
			    ptd -> tflg = NO_TROUBLE;
			    ptd -> cbmd = NO_CALLBACK_ACTIVE;
			    ptd -> next = 0;
			    write_output++;
		    }
		}
	    }
	    else if ( (ptd -> cbmd == PULSE_TDE_TDD_CB) || (ptd -> tmrm > 0) )
	    {
		decrement_tmrm (ptd);
	    }

	    break;

	case TD_CYCLE:       /* cycle timer */

	    if (ptd -> ival != ptd -> pval)
	    {
		if ( ((ptd -> ival == 0) && (ptd -> lted == LEADING_EDGE)) ||
		     ((ptd -> ival == 1) && (ptd -> lted == TRAILING_EDGE)) )
		{
		    if (ptd -> cbmd != NO_CALLBACK_ACTIVE)
		    {
			ptd -> lflg = TD_LOCK;
			if ((ptd -> dtck > ptd -> ctck) || (ptd -> tflg == TROUBLE))
			{
			    wdCancel (pcallback -> wd_id);
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> next = 0;
			    ptd -> tmrm = 0;
			    if (ptd -> invt)
				ptd -> val = 1;
			    else
				ptd -> val = 0;
			    ptd -> tflg = NO_TROUBLE;
			    ptd -> cbmd = NO_CALLBACK_ACTIVE;
			    write_output++;
			}
			else
			{
			    ptd -> lflg = TD_NO_LOCK;
			    ptd -> tflg = TROUBLE;
			    *update_pval = 0;
			}
		    }
		}
		else if ( ((ptd -> ival == 0) && (ptd -> lted == TRAILING_EDGE)) ||
			  ((ptd -> ival == 1) && (ptd -> lted == LEADING_EDGE)) )
		{
		    if ( (ptd -> invt) || (ptd -> frst == CYCLE_OFF_FIRST) )
		    {
			ptd -> tval = 1;
			ptd -> val = 0;
		    }
		    else
		    {
			ptd -> tval = 0;
			ptd -> val = 1;
		    }

		    if (ptd -> frst == CYCLE_ON_FIRST)
		    {
			wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
			ptd -> tmrm = ptd -> tdv1;
			ptd -> tmct = ptd -> ctck;
			if (ptd ->tdv2 > 0)
			    ptd -> next = 2;	/* go to off time */
			else
			    ptd -> next = 3;	/* go to dwell time */
			ptd -> cbmd = CB_CYCLE_ON;
		    }
		    else
		    {
			ptd -> next = 1;
			if (ptd -> tdv2 > 0)
			{
			    wait_time = (int)(ptd -> tdv2 * vxTicksPerSecond);
			    ptd -> tmrm = ptd -> tdv2;
			    ptd -> tmct = ptd -> ctck;
			    ptd -> cbmd = CB_CYCLE_OFF;
			}
			else
			{
			    ptd -> tmrm = 0;
			    wait_time = (int)(ptd -> dwel * vxTicksPerSecond);
			    ptd -> cbmd = CB_CYCLE_DWELL;
			}
		    }
		    ptd -> tflg = NO_TROUBLE;
		    write_output++;
		    start_timer++;
		}
	    }
	    else if ( (ptd -> cbmd == CB_CYCLE_ON) || (ptd -> cbmd == CB_CYCLE_OFF)  || 
		      (ptd -> tmrm > 0) )
	    {
		decrement_tmrm (ptd);
	    }

	    break;


/**************************************************************************************
*
*	Value of next in each stage of cycle callback to be used when input drops
*	to determine where we are, we want to let the output stay high for rest of
*	TDV1, unlike the TD_CYLE where we drop the output signal
*
*	START ON
*
*                ********         ********         ********
*                *      *         *      *         *      *
*                * tdv1 *tdv2 dwel* tdv1 *tdv2 dwel* tdv1 *
*    output      *      *         *      *         *      *
*    *************      *****I*****      *****I*****      **************
*     all 3         n2    n3   n1    n2    n3   n1    n2
*    tdv1&dwel      n3         n1    n3         n1    n3
*    tdv1&tdv2      n2    n1         n2    n1         n2
*
*
*	START OFF
*
*                ********         ********         ********
*                *      *         *      *         *      *
*           tdv2 * tdv1 *dwel tdv2* tdv1 *dwel tdv2* tdv1 *
*    output      *      *         *      *         *      *
*    *************      *****I*****      *****I*****      **************
*     all 3    n1   n3    n2   n1    n3
*    tdv1&dwel n1   n3    n1         n3
*    tdv1&tdv2 n1   n2         n1    n2
*
**************************************************************************************/


	case TD_CYCLE_NO_RST:	/* cycle timer with no reset when input drops */

	    if (ptd -> ival != ptd -> pval)
	    {
		if ( ((ptd -> ival == 0) && (ptd -> lted == LEADING_EDGE)) ||
		     ((ptd -> ival == 1) && (ptd -> lted == TRAILING_EDGE)) )
		{
		    if (ptd -> cbmd != NO_CALLBACK_ACTIVE)
		    {
			/* determine if we are in TDV1, to let it finish */
			if (ptd -> cbmd == CB_CYCLE_ON)
			{
			    if (ptd -> cbac == CB_ACTIVE)
			    {
				ptd -> tflg = TROUBLE;
				*update_pval = 0;
			    }
			    else
				ptd -> next = 4;
			}
			else
			{
			    ptd -> lflg = TD_LOCK;
			    if ((ptd -> dtck > ptd -> ctck) || (ptd -> tflg == TROUBLE))
			    {
				wdCancel (pcallback -> wd_id);
				ptd -> lflg = TD_NO_LOCK;
				ptd -> next = 0;
				if (ptd -> invt)
				    ptd -> val = 1;
				else
				    ptd -> val = 0;
				ptd -> tflg = NO_TROUBLE;
				ptd -> cbmd = NO_CALLBACK_ACTIVE;
				ptd -> tmrm = 0;
				write_output++;
			    }
			    else
			    {
				ptd -> lflg = TD_NO_LOCK;
				ptd -> tflg = TROUBLE;
				*update_pval = 0;
			    }
			}
		    }
		}
		else if ( ((ptd -> ival == 0) && (ptd -> lted == TRAILING_EDGE)) ||
			  ((ptd -> ival == 1) && (ptd -> lted == LEADING_EDGE)) )
		{
		    if ( (ptd -> invt) || (ptd -> frst == CYCLE_OFF_FIRST) )
		    {
			ptd -> tval = 1;
			ptd -> val = 0;
		    }
		    else
		    {
			ptd -> tval = 0;
			ptd -> val = 1;
		    }

		    if (ptd -> frst == CYCLE_ON_FIRST)
		    {
			wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
			ptd -> tmrm = ptd -> tdv1;
			ptd -> tmct = ptd -> ctck;
			if (ptd ->tdv2 > 0)
			    ptd -> next = 2;	/* go to off time */
			else
			    ptd -> next = 3;	/* go to dwell time */
			ptd -> cbmd = CB_CYCLE_ON;
		    }
		    else
		    {
			ptd -> next = 1;
			if (ptd -> tdv2 > 0)
			{
			    wait_time = (int)(ptd -> tdv2 * vxTicksPerSecond);
			    ptd -> tmrm = ptd -> tdv2;
			    ptd -> tmct = ptd -> ctck;
			    ptd -> cbmd = CB_CYCLE_OFF;
			}
			else
			{
			    ptd -> tmrm = 0;
			    wait_time = (int)(ptd -> dwel * vxTicksPerSecond);
			    ptd -> cbmd = CB_CYCLE_DWELL;
			}
		    }
		    ptd -> tflg = NO_TROUBLE;
		    write_output++;
		    start_timer++;
		}
	    }
	    else if ( (ptd -> cbmd == CB_CYCLE_ON) || (ptd -> cbmd == CB_CYCLE_OFF) ||
		      (ptd -> tmrm > 0) )
	    {
		decrement_tmrm (ptd);
	    }

	    break;

	default:

	    break;
    }

    if (write_output)
    {
	/* set raw output value */

	if (ptd -> mask != 0)
	{
	    if (ptd -> val == 0)
		ptd -> rval = 0;
	    else
		ptd -> rval = ptd -> mask;
	}

	/* update numeric version of val */

	ptd -> nval = ptd -> val;
	status = (*pdset -> write_td) (ptd);
    }

    if (start_timer)
    {
	if (wait_time > 0)
	{
	    ptd -> dtck = ptd -> ctck + wait_time;
	    pcallback = (CALLBACK_PTR) (ptd -> tdcb);
	    if (pcallback -> wd_id == NULL)
	    {
		pcallback -> wd_id = wdCreate ();
	    }
	    wdStart (pcallback -> wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
	}
    }

    return (status);

} /* end func process_ival () */


/****************************************************************************
 *
 *  MODULE NAME            : slave_callback
 *  AUTHOR                 : emsteel
 *  DATE                   : 06 June 94
 *
 *  DESCRIPTION:
 *	the MASTER has told us (SLAVE) to handle our own callbacks so that
 *		we stay in sync with him. he has already passsed us the 
 *		necessary values such as CBMD, IVAL, TVAL, NEXT etc 
 *		(see redundancy list)
 *	do not do anything if both of our init flags are not set, any values
 *		passed by MASTER may be overwritten by SLAVES init_record
 *			
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             :
 *  REVISION DATE          :
 *  REVISION:
 *
****************************************************************************/

static void slave_callback (ptd, save_cbmd)
    struct tdRecord	*ptd;
    short		save_cbmd;
{
    CALLBACK_PTR	pcallback = (CALLBACK_PTR) (ptd -> tdcb);
    int			start_timer = 0;
    int			wait_time = 0;

    if ( (!ptd -> init) || (!ptd -> ini2) )
    {
	ptd -> lflg = TD_NO_LOCK;
	return;
    }

    if (save_cbmd > NO_CALLBACK_ACTIVE)
	wdCancel (pcallback -> wd_id);

    ptd -> lflg = TD_NO_LOCK;
    ptd -> tflg = NO_TROUBLE;

    switch (ptd -> cbmd)
    {
	case NO_CALLBACK_ACTIVE:
	    break;

	case PULSE_TDE_TDD_CB:
	case CB_CYCLE_ON:
	case CB_CYCLE_OFF:
	    wait_time = (int)(ptd -> tmrm * vxTicksPerSecond);
	    start_timer++;
	    break;

	case CB_CYCLE_DWELL:
	    wait_time = (int)(ptd -> dwel * vxTicksPerSecond);
	    start_timer++;
	    break;

	default:
	    break;

    }

    if (start_timer)
    {
	if (wait_time > 0)
	{
	    ptd -> dtck = ptd -> ctck + wait_time;
	    if (pcallback -> wd_id == NULL)
	    {
		pcallback -> wd_id = wdCreate ();
	    }
	    wdStart (pcallback -> wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
	}
    }

    return;
}


/****************************************************************************
 *
 *  MODULE NAME            : special
 *  AUTHOR                 : emsteel
 *  DATE                   : 06 June 94
 *
 *  DESCRIPTION:
 *	specials are not only acted on if the field value is changed but also 
 *		if the field is redundant, we are the SLAVE and value was i
 *		sent from the MASTER
 *	SPC_TD_CTCK, we got here because we are SLAVE so get our own tick
 *		count
 *	SPC_TD_NEXT, we got here because we are SLAVE and the MASTER has told
 *		use to handle our own callbacks according to his parameters
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             :
 *  REVISION DATE          :
 *  REVISION:
 *
****************************************************************************/

static long special (paddr, after)
    struct dbAddr    *paddr;
    int              after;
{
    struct tdRecord	*ptd = (struct tdRecord *) (paddr -> precord);
    int			special_type = paddr -> special;
    static short	save_cbmd;

    if (!after)
    {
	save_cbmd = ptd -> cbmd;
	if (special_type == SPC_TD_CBMD)
	    ptd -> lflg = TD_LOCK;
	return (0);
    }

    switch (special_type)
    {
	case SPC_TD_CTCK:
	    ptd -> ctck = tickGet();
	    /* keep TMCT used for TMRM in sync with CTCK */
	    ptd -> tmct = ptd -> ctck;
	    break;

	case SPC_TD_CBMD:
	    slave_callback (ptd, save_cbmd);
	    break;

	default:
	    recGblDbaddrError (S_db_badChoice, paddr, "td: special");
	    return (S_db_badChoice);
    }

    return (0);
}  


/****************************************************************************
 *
 *  MODULE NAME            : tdCallback 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Oct 93
 *
 *
 *  DESCRIPTION: 
 *               called after some time delay to finish the processing of
 *                a timedelay cycle
 *               1 - set rval (raw value), used bu write_td ()
 *               2 - ptd -> next will == 0 in all but the cycle timer
 *                   for cycle timer, if leading edge, then next starts
 *                      as 2, for trailng edge next starts as 1
 *  NOTES:
 *	if locking flag is set do nothing, that means that process_ival ()
 *		has the ball
 *	if callback flag is not set, do nothing, that means that 
 *		process_ival wants the ball
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 03 Jan 94
 *  REVISION:  comment out check for init flag, not needed
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 06 June 94
 *  REVISION:	only do the write if we are MASTER
 *
****************************************************************************/

static void tdCallback (pcallback)
    CALLBACK_PTR	pcallback;
{
    struct tdRecord	*ptd = (struct tdRecord *) (pcallback->dbAddr.precord);
    struct tddset	*pdset = (struct tddset *) (ptd -> dset);
    int			wait_time = 0;
    short		master = 0;
    unsigned short	mask = 0;
    float		tmrm = ptd -> tmrm;

    if (ptd -> lflg == TD_LOCK)
	return;

    if (ptd -> cbmd == NO_CALLBACK_ACTIVE)
	return;

    ptd -> cbac = CB_ACTIVE;

      /* output ptd -> tval */

    ptd -> val = ptd -> tval;

    /* set raw output value */

    if (ptd -> mask != 0)
    {
	if (ptd -> val == 0)
            ptd -> rval = 0;
	else
	    ptd -> rval = ptd -> mask;
    }

         /* update numeric version of output value */

    ptd -> nval = ptd -> val;

    /* only write if our resource is MASTER */

    if (resGetCaAnswer (ptd -> radr, &master) == RES_OK)
    {
	if (master)
	    (*pdset -> write_td) (ptd);
    }

    /* next = 0, means we are at the end of a callback */
    if ( (ptd -> next == 0) || (ptd -> next == 5) )
    {
	ptd -> cbmd = NO_CALLBACK_ACTIVE;
	ptd -> dtck = 0;
	ptd -> tmrm = 0;
	ptd -> next = 0;
    }
    /* we just turned output on, set to turn it off after TDV1 time */
    else if (ptd -> next == 1)
    {
	if (ptd -> invt)
	    ptd -> tval = 1;
	else
	    ptd -> tval = 0;
	ptd -> cbmd = CB_CYCLE_ON;
	wait_time = (int)(ptd -> tdv1 * vxTicksPerSecond);
	ptd -> tmrm = ptd -> tdv1;
	ptd -> tmct = tickGet ();
	if (ptd -> frst == CYCLE_ON_FIRST)
	{
	    if (ptd -> tdv2 > 0)
		ptd -> next = 2;
	    else
		ptd -> next = 3;
	}
	else	/* CYCLE_OFF_FIRST */
	{
	    if (ptd -> dwel > 0)
		ptd -> next = 3;
	    else
		ptd -> next = 2;
	}
    }
    /* we just turn output off, set to turn back on or enter 
       dwell time after TDV2 seconds
    */
    else if (ptd -> next == 2)
    {
	ptd -> cbmd = CB_CYCLE_OFF;
	wait_time = (int)(ptd -> tdv2 * vxTicksPerSecond);
	ptd -> tmrm = ptd -> tdv2;
	ptd -> tmct = tickGet ();
	if (ptd -> frst == CYCLE_ON_FIRST)
	{
	    if (ptd -> dwel > 0)
	    {
		ptd -> next = 3;
		if (ptd -> invt)
		    ptd -> tval = 1;
		else
		    ptd -> tval = 0;
	    }
	    else
	    {
		ptd -> next = 1;
		if (ptd -> invt)
		    ptd -> tval = 0;
		else
		    ptd -> tval = 1;
	    }
	}
	else	/* CYCLE_OFF_FIRST */
	{
	    ptd -> next = 1;
	    if (ptd -> invt)
		ptd -> tval = 0;
	    else
		ptd -> tval = 1;
	}
    }
    /* we just turned output off, set for to turn back on or
       enter off time time TDV2 at end of DWEL seconds
    */
    else if (ptd -> next == 3)
    {
	ptd -> cbmd = CB_CYCLE_DWELL;
	wait_time = (int)(ptd -> dwel * vxTicksPerSecond);
	ptd -> tmrm = 0;
	if (ptd -> frst == CYCLE_OFF_FIRST)
	{
	    if (ptd -> tdv2 > 0)
	    {
		ptd -> next = 2;
		if (ptd -> invt)
		    ptd -> tval = 1;
		else
		    ptd -> tval = 0;
	    }
	    else
	    {
		ptd -> next = 1;
		if (ptd -> invt)
		    ptd -> tval = 0;
		else
		    ptd -> tval = 1;
	    }
	}
	else	/* CYCLE_ON_FIRST */
	{
	    ptd -> next = 1;
	    if (ptd -> invt)
		ptd -> tval = 0;
	    else
		ptd -> tval = 1;
	}
    }
    /* we have just turned off output for TD_CYCLE_NO_RST, and the input
       signal has dropped,we do not want to continue the output cycle
    */
    else if (ptd -> next == 4)
    {
	ptd -> cbmd = NO_CALLBACK_ACTIVE;
	ptd -> dtck = 0;
	ptd -> tmrm = 0;
	ptd -> next = 0;
    }

    if (wait_time > 0)
    {
	unsigned long curr_ticks;

	curr_ticks = tickGet ();
	ptd -> dtck = curr_ticks + wait_time;
	if (pcallback -> wd_id == NULL)
	{
	    pcallback -> wd_id = wdCreate ();
	}
	wdStart (pcallback -> wd_id, wait_time, (FUNCPTR)callbackRequest, (int)pcallback);
    }

    if (master)
    {
	if (tmrm != ptd -> tmrm)
	    db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> tmrm, DBE_VALUE);
	if (ptd -> mlst != ptd -> val)
	{
	    mask = DBE_VALUE;
	    if (ptd -> tenb)
		mask |= DBE_LOG;
	    db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> val, mask);
	    db_post_events ((struct dbCommon *)ptd, (caddr_t)&ptd -> nval, mask);
	    ptd -> mlst = ptd -> val;
	}
    }

    ptd -> cbac = CB_NOT_ACTIVE;

    return;

} /* end func tdCallback () */

