/*recUdb.c,v 1.1.1.1 1999/08/03 14:44:23 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************************
 *
 *  MODULE NAME            : recUdb.c 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: record support routines for Udb records
 *
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Jan 96
 *  REVISION:	added support for ZCMF, zero commands flag, to zero cmds 
 *		on the slave, see special()
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 15 Jan 96
 *  REVISION:	added FPAS, 1st pass flag as redundant
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Mar 96
 *  REVISION:	process(), increase wait time on udb code from 
 *			3 to 5 ticks
 *		udb_process_task(), increease watchdog timer from
 *			2 to 4 ticks
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Apr 96
 *  REVISION:	added code to support alarm delay
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 20 Jun 96
 *  REVISION:	added code for alarm disable ADIS
 *
****************************************************************************/

/****************************************************************************
*
* Programmer's Notes:
*
****************************************************************************/


#include	<vxWorks.h>
#include	<inetLib.h>
#include	<semLib.h>
#include	<sysLib.h>
#include	<symLib.h>
#include	<sysSymTbl.h>	/* for sysSymTbl*/
#include	<a_out.h>	/* for N_TEXT */
#include	<tickLib.h>	/* for tickGet () */
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<wdLib.h>
#include	<string.h>
#include	<logLib.h>

#include	"alarm.h"
#include	"cvtTable.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"special.h"
#include	"rec/udbRecord.h"
#include	"tsDefs.h"
#include	"udbDefs.h"
#include	"redflds.h"
#include	"redque.h"
#include	"udbredfldidx.h"
#include	"restable.h"
#include	"task_params.h"
#include	"db_addr.h"
#include	"dbEvent.h"
#include	"recGbl.h"
#include	"error.h"
#include	"dbcSub.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table */

#define report NULL
#define initialize NULL
static long init_record();
static long process();
static long special ();
static long get_value();
#define cvt_dbaddr NULL
#define get_array_info NULL
#define put_array_info NULL
#define get_units NULL
static long get_precision();
static long get_enum_str ();
static long get_enum_strs ();
static long put_enum_str ();
static long get_graphic_double ();
static long get_control_double ();
#define get_alarm_double NULL

struct rset udbRSET = {

    RSETNUMBER,
    report,
    initialize,
    init_record,
    process,
    special,
    get_value,
    cvt_dbaddr,
    get_array_info,
    put_array_info,
    get_units,
    get_precision,
    get_enum_str,
    get_enum_strs,
    put_enum_str,
    get_graphic_double,
    get_control_double,
    get_alarm_double
    };


/* The folowing definitions are for redundancy */

static struct fldDes **mylist = 0;

/* NOTE: the order and spelling in namelist must be the
         same as in udbredfldidx.h
*/
static char *namelist [] = {
               "STAT",
               "SEVR",
	       "INI2",
               "VAL",
	       "LVAL",
	       "MLST",
	       "MODE",
	       "LCMD",
	       "FUNC",
	       "NCMD",
	       "STTS",
	       "CK1",
	       "CK2",
	       "CK3",
	       "CK4",
	       "CK5",
	       "CK6",
	       "CK7",
	       "CK8",
	       "N01",
	       "N02",
	       "N03",
	       "N04",
	       "N05",
	       "N06",
	       "N07",
	       "N08",
	       "N09",
	       "N10",
	       "N11",
	       "N12",
	       "D01",
	       "D02",
	       "D03",
	       "D04",
	       "D05",
	       "D06",
	       "U01",
	       "U02",
	       "MSGF",
	       "LALM",
	       "CTCK",
	       "TM01",
	       "TM02",
	       "TM03",
	       "TM04",
	       "TM05",
	       "TM06",
	       "TM07",
	       "TM08",
	       "TM09",
	       "TM10",
	       "TM11",
	       "TM12",
	       "TF01",
	       "TF02",
	       "TF03",
	       "TF04",
	       "TF05",
	       "TF06",
	       "TF07",
	       "TF08",
	       "TF09",
	       "TF10",
	       "TF11",
	       "TF12",
	       "FP01",
	       "FP02",
	       "FP03",
	       "FP04",
	       "FP05",
	       "FP06",
	       "FP07",
	       "FP08",
	       "FP09",
	       "FP10",
	       "FP11",
	       "FP12",
	       "FP13",
	       "FP14",
	       "FP15",
	       "FP16",
	       "LU01",
	       "LU02",
	       "LU03",
	       "LU04",
	       "LU05",
	       "LU06",
	       "LU07",
	       "LU08",
	       "DB01",
	       "DB02",
	       "DB03",
	       "DB04",
	       "BL01",
	       "BL02",
	       "BL03",
	       "BL04",
	       "BL05",
	       "BL06",
	       "BL07",
	       "BL08",
	       "BL09",
	       "BL10",
	       "BL11",
	       "BL12",
	       "BL13",
	       "BL14",
	       "BL15",
	       "BL16",
	       "ZCMF",
	       "FPAS",
	       "HHLL",
	       "RTCT",
               0
               };

static   unsigned short type;       /* index to udb type */

/* will be set to 1 in init_record if user has loaded udbLib */

static int udblib_loaded = 0;

static void alarm ();
static void delete_udb_task ();
static void init_fields ();
static void monitor ();
static void post_and_redund ();
static long send_outputs ();
void   udb_process_task();
static void update_timers ();
void   udb_watchdog ();

static SEM_ID	semmutex = 0;
static struct udb_task	*first_udb_task = (struct udb_task *)0;

struct udb_task
{
    int 		udb_task_id;
    SEM_ID	  	sem_activate;
    SEM_ID		sem_done;
    char		udb_task_name[32];
    UDBPTR		udb_process_func;
    struct udbRecord	*pudb;
    struct udb_task	*next_udb_task;
};



/****************************************************************************
 *
 *  MODULE NAME            : init_record 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *  ARGUMENT(S)            : pudb - ptr to udb record  
 *
 *  DESCRIPTION: initialize a udb record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 15 Jan 96
 *  REVISION:	do not set FPAS to 1, let user do that in his INI2 code
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 Jan 96
 *  REVISION:	wait 5 ticks not 2, when call init func and ini2 func
 *		also give back semmutex before do printf's etc
 *
****************************************************************************/

static long init_record (pudb)
    struct udbRecord	*pudb;

{
    long		rtnval;
    int			*initialize_flag;
    int			found_task;
    int			new_task;
    SYM_TYPE		symType;
    unsigned short	i;
    char		*pstr;
    char		symname[64];
    char		errmsg[256];
    TS_STAMP		timenow;
    UDBPTR		udb_func, udb_init_func, udb_ini2_func, udb_process_func;
    struct udbRecord	old_udb;
    char		udb_task_name[32];
    struct udb_task	*new_udb_task = NULL;
    struct udb_task	*curr_udb_task = NULL;

    if (!(mylist))
    {
         /* find record type for udb */
        for (type = 1, i = 0; type < 100; type++)
        {
            pstr = GET_PRECTYPE (type);     
            if (strncmp (pstr, "udb", 4) == 0)
            {
                i = 1;
                break;
            }
        }

        if (i == 0)
        {
            recGblRecordError (S_db_noRecType, (struct dbCommon *)pudb, "udb:init_record");
            return (S_db_noRecType);
         }

        mylist = redFldAlloc (type, NUMUDBFLDS);

	if (!mylist)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pudb,
			   "udb, init_record, mylist failed");
  	  return (S_rec_Alloc);
 
	}

        for (i = 0; i < NUMUDBFLDS; i++)
            redFldAdd (type, i, namelist [i]);

    }

    /* get a mutual exclusion semaphore to protect the linked list
       of structs udb_task
    */
    if (!semmutex)
    {
	if ( (semmutex = semMCreate (SEM_Q_FIFO)) == NULL)
	{
	    sprintf (errmsg, "udb:init_record, %s, semMCreate failed", pudb -> name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
    }
	    

    tsLocalTime (&timenow);
    if (timenow.secPastEpoch < 110000)
    {
	logMsg ("UDB for %s can not initialize until Time is set in IOC\n",
		 (int)pudb -> name,0,0,0,0,0);
	return (0);
    }

    /* if user has not loaded the udbLib, then do not allow any UDBs to init */

    if (!udblib_loaded)
    { 
	sprintf (symname, "%s", "_UDB_data_transfer");
	rtnval = symFindByName (sysSymTbl, symname, (char **)&udb_func, &symType);
	if ( (rtnval != OK) || ( (symType & N_TEXT) == 0) )
	{
	    recGblRecordError (S_udb_NoUdbLib, (struct dbCommon *)pudb, "udb:init_record");
	    return (S_udb_NoUdbLib);
	}
	udblib_loaded = 1;
    }

    /* find the global initialization flag for the UDB */

    sprintf (symname, "_UDB_%s_initialize_flag", pudb -> dnam);
    rtnval = symFindByName (sysSymTbl, symname, (char **)&initialize_flag, &symType);
    if ( (rtnval != OK) || ( (symType & N_TEXT) == 0) )
    {
	recGblRecordError (S_udb_NoInitFlag, (struct dbCommon *)pudb, "udb:init_record");
	return (S_udb_NoInitFlag);
    }

    /* find the global init function for this udb */

    if (!(*initialize_flag))
    {
	sprintf (symname, "_UDB_%s_initialize", pudb -> dnam);
	rtnval = symFindByName (sysSymTbl, symname, (char **)&udb_init_func, &symType);
	if ( (rtnval != OK) || ( (symType & N_TEXT) == 0) )
	{
	    recGblRecordError (S_udb_NoInit, (struct dbCommon *)pudb, "udb:init_record");
	    return (S_udb_NoInit);
	}
    }

    /* find the ini2 function for this udb */

    sprintf (symname, "_UDB_%s_ini2", pudb -> dnam);
    rtnval = symFindByName (sysSymTbl, symname, (char **)&udb_ini2_func, &symType);
    if ( (rtnval != OK) || ( (symType & N_TEXT) == 0) )
    {
	recGblRecordError (S_udb_NoInit, (struct dbCommon *)pudb, "udb:init_record");
	return (S_udb_NoInit);
    }

    /* find the process function for thsi udb */
    sprintf (symname, "_UDB_%s_process", pudb -> dnam);
    rtnval = symFindByName (sysSymTbl, symname, (char **)&udb_process_func, &symType);
    if ( (rtnval != OK) || ( (symType & N_TEXT) == 0) )
    {
	recGblRecordError (S_udb_NoProcess, (struct dbCommon *)pudb, "udb:init_record");
	return (S_udb_NoProcess);
    }

    sprintf (udb_task_name, "udb_%s_task", pudb -> dnam);

    new_task = 0;
    /* if the task does not exist */
    if ( (taskNameToId (udb_task_name)) == ERROR)
    {
	if ( (new_udb_task = (struct udb_task *)calloc (1, sizeof (struct udb_task))) == NULL)
	{
	    sprintf (errmsg, "udb:init_record, %s", pudb -> name);
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pudb, errmsg);
	    return (S_rec_Alloc);
	}
	if ( (new_udb_task -> sem_activate = semBCreate (SEM_Q_FIFO, SEM_EMPTY)) == NULL)
	{
	    free ((void*)new_udb_task);
	    sprintf (errmsg, "udb:init_record, %s, semBCreate failed", pudb -> name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
	if ( (new_udb_task -> sem_done = semBCreate (SEM_Q_FIFO, SEM_EMPTY)) == NULL)
	{
	    semDelete (new_udb_task -> sem_activate);
	    free ((void*)new_udb_task);
	    sprintf (errmsg, "udb:init_record, %s, semBCreate failed", pudb -> name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
	strncpy (new_udb_task -> udb_task_name, udb_task_name, sizeof (new_udb_task -> udb_task_name));
	new_udb_task -> next_udb_task = (struct udb_task *)0;

	if ( (new_udb_task -> udb_task_id = taskSpawn(udb_task_name,
						      98,
						      VX_FP_TASK,
						      4096,
						      (FUNCPTR)udb_process_task,
						      (int)new_udb_task,
						      0,0,0,0,0,0,0,0,0)) == ERROR)
	{
	    semDelete (new_udb_task -> sem_activate);
	    semDelete (new_udb_task -> sem_done);
	    free((void*)new_udb_task);
	    sprintf (errmsg, "udb:init_record, %s, taskSpawn failed", pudb -> name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
	new_task = 1;
    }

    /* save current udb record to test later for redundancy update */
    old_udb = *pudb;

    init_fields (pudb);

    pudb -> init = 1;

    pudb -> lppt = (void *)udb_process_func;	/* remember pointer to the udb's process func */

    /* init last process time and last alarm time */

    if (pudb -> lptm.secPastEpoch == 0)
	tsLocalTime (&pudb -> lptm);
    if (pudb -> latm.secPastEpoch == 0)
	tsLocalTime (&pudb -> latm);
	
    /* protect my link list resource */
    semTake (semmutex, WAIT_FOREVER);
    if (new_task)
    {
	if (!first_udb_task)
	    first_udb_task = new_udb_task;
	else
	{
	    curr_udb_task = first_udb_task;
	    while(1)
	    {
		 if (curr_udb_task -> next_udb_task == 0)
		     break;
		else
		    curr_udb_task = curr_udb_task -> next_udb_task;
	    }
	    curr_udb_task -> next_udb_task = new_udb_task;
	}
	curr_udb_task = new_udb_task;
    }
    /* then find task for this udb */
    else
    {
	curr_udb_task = first_udb_task;
	found_task = 0;
	while (1)
	{
	    if (strncmp (udb_task_name, curr_udb_task -> udb_task_name,
				sizeof (udb_task_name) ) == 0)
	    {
		found_task = 1;
		break;
	    }
	    curr_udb_task = curr_udb_task -> next_udb_task;
	    if (curr_udb_task)
		continue;
	    break;
	}
	if (!found_task)
	{
	    semGive (semmutex);
	    sprintf (errmsg, "udb:init_record, task %s, not found", udb_task_name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
    }

    /* call the global init routine for this udb if it has not already been
       called with a successful return
    */
    if (!(*initialize_flag))
    {
	curr_udb_task -> udb_process_func = udb_init_func;
	curr_udb_task -> pudb = pudb;
	/* activate the udb's task */
	semGive (curr_udb_task -> sem_activate);
	if (semTake (curr_udb_task -> sem_done, 5) == ERROR)
	{
	    delete_udb_task (curr_udb_task, 1);
	    semGive (semmutex);
	    sprintf (errmsg, "udb:init_record, %s, failed initialize return", pudb -> dnam);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
	/* we won't init yet */
	if (!(*initialize_flag))
	{
	    semGive (semmutex);
	    sprintf (errmsg, "udb:init_record, %s", pudb -> dnam);
	    recGblRecordError (S_udb_InitFailed, (struct dbCommon *)pudb, errmsg);
	    return (S_udb_InitFailed);
	}
    }

    /* call the ubd's ini2 function */

    if (!pudb -> ini2)
    {
	curr_udb_task -> udb_process_func = udb_ini2_func;
	curr_udb_task -> pudb = pudb;
	/* activate the udb's task */
	semGive (curr_udb_task -> sem_activate);
	if (semTake (curr_udb_task -> sem_done, 5) == ERROR)
	{
	    delete_udb_task (curr_udb_task, 1);
	    semGive (semmutex);
	    sprintf (errmsg, "udb:init_record, %s, failed ini2 return", pudb -> name);
	    errMessage (-1L, errmsg);
	    return (-1L);
	}
	if (!pudb -> ini2)
	{
	    pudb -> nsta = NO_INIT_ALARM;
	    pudb -> nsev = VALID_ALARM;
	}
    }
    semGive (semmutex);

    /* remember ptr to udb_task struct */

    pudb -> udbt = (void *)curr_udb_task;


    /* if we are MASTER, post monitors and send to redundant partner */

    if (resGetCaAnswer (pudb -> radr, &i) == RES_OK)
    {
	if (i)
	{
	    /* init val to the current status, init lval to this value also
	       so will not generate an alarm due to value change
	    */
	    pudb -> lval = pudb -> val = pudb -> stts;
	    monitor (pudb);
	    if (pudb -> ini2)
		post_and_redund (pudb, &old_udb);
	}
    }

    return (0);

} /* end func init_records () */


/****************************************************************************
 *
 *  MODULE NAME            : init_fields
 *  AUTHOR                 : emsteel
 *  DATE                   : 27 Feb 95
 *
 *  CALLING MODULE(S)      : init_record
 *  ARGUMENT(S)            : pudb
 *
 *  DESCRIPTION: init fields in record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             :
 *  REVISION DATE          :
 *  REVISION:
 *       
****************************************************************************/

static void init_fields (pudb)
    struct udbRecord	*pudb;
{
    double		*dblptr;
    unsigned short	*usptr;
    short		*shptr1;
    short		*shptr2;
    unsigned long	*ulptr;
    float		*flptr;
    int			i;

    pudb -> pact = FALSE;

    /* zero input vals and ok flags */

    for (i = 0,
	 dblptr = &pudb -> inv1,
	 usptr  = &pudb -> iok1;
	 i < UDB_NUM_INPUTS;
	 i++)
    {
	*dblptr = 0;
	*usptr  = NO_ALARM;
	usptr++;
	dblptr++;
    }

    /* zero output vals and ok flags */

    for (i = 0,
	 dblptr = &pudb -> ovl1,
	 usptr  = &pudb -> ook1;
	 i < UDB_NUM_OUTPUTS;
	 i++)
    {
	*dblptr = 0;
	*usptr  = NO_ALARM;
	dblptr++;
	usptr++;
    }

    /* zero the command flags */

    for (i = 0,
	 shptr1 = &pudb -> cm1m,
	 shptr2 = &pudb -> cm1a;
	 i < UDB_NUM_CMDS;
	 i++)
    {
	*shptr1 = 0;
	*shptr2 = 0;
	shptr1++;
	shptr2++;
    }

    /* zero the function flags */

    for (i = 0,
	 shptr1 = &pudb -> ffl1;
	 i < UDB_NUM_FUNCS;
	 i++)
    {
	*shptr1 = 0;
	shptr1++;
    }

    /* zero date fields */

    pudb -> year = 0;
    pudb -> leap = 0;
    pudb -> doy = 0;
    pudb -> mon = 0;
    pudb -> dom = 0;
    pudb -> dow = 0;
    pudb -> hod = 0;
    pudb -> min = 0;
    pudb -> sec = 0;

    /* zero bit flags */

    pudb -> npfl = 0;		/* numeric promted fields post flag */
    pudb -> nrfl = 0;		/* numeric promted fields redund flag */
    pudb -> inpf = 0;		/* post an input value flag */
    pudb -> outf = 0;		/* send output bit flag */
    pudb -> tmfl = 0;		/* temp/msg bit map flag	*/
    pudb -> trfl = 0;		/* temp float trend flag */
    pudb -> trbl = 0;		/* bool flag trend flag */
    pudb -> trnc = 0;		/* NCMD trend flag */

    pudb -> nsta = NO_ALARM;
    pudb -> nsev = NO_ALARM;
    pudb -> msev = NO_ALARM;
    pudb -> dctk = 0;		/* initial delta clock ticks	*/
    pudb -> fpps = 1;		/* 1st pass for LALM to be updated in process() */
    pudb -> zcmf = 0;		/* the zero cmds flag */

    /* only do these once, especially if you are the SLAVE since these
       are redundant fields and we do not want their values wiped out
       on the SLAVE
    */
    if (pudb -> init == 0)
    {
	pudb -> ctck = tickGet ();	/* initial clock ticks	*/
	pudb -> lalm = 0;		/* last val alarmed */
	pudb -> fpas = 0;		/* 1st pass flag */
	pudb -> lval = 0;		/* last val */
	pudb -> mlst = UDB_NUM_STATUS + 1; /* las tval monitored */
	pudb -> ncmd = UDB_NUM_CMDS;	/* new command field */
	pudb -> lcmd = UDB_NUM_CMDS;	/* las tcommand field */
	pudb -> val = UDB_NUM_STATUS;
	pudb -> stts = UDB_NUM_STATUS;
	bzero (pudb -> msgf, sizeof (pudb -> msgf)); /* null message string	*/

	/* init alarm delay variables */
	pudb -> larf = 0;
	pudb -> hhll = 0;
	pudb -> hilo = 0;
	pudb -> dvct = 0;
	pudb -> rtct = 0;

	/* set mode to initial mode, imde = 0 - UDB_NUM_MODES where
	   0 is last mode and 1 - UDB_NUM_MODES are the actual modes
	   mode has value 0 - (UDB_NUM_MODES -1)
	*/
	if (pudb -> imde != 0)
	    pudb -> mode = pudb -> imde - 1;

	/* set current function to initial function, ifun has
	   value = 0 - UDB_NUM_FUNCS, 0 is last func and
	   1 - UDB_NUM_FUNCS are the actual functions, func
	   has value 0 - (UDB_NUM_FUNCS - 1) 
	*/
	if (pudb -> ifun != 0)
	    pudb -> func = pudb -> ifun - 1;

	/* zero the timers and timer flags */

	for (i = 0,
	     ulptr = &pudb -> tm01,
	     usptr = &pudb -> tf01;
	     i < UDB_NUM_TIMERS;
	     i++)
	{
	    *ulptr = 0;
	    *usptr = UDB_TIMER_NOT_ACTIVE;
	    ulptr++;
	    usptr++;
	}

	/* zero the temp floats */

	for (i = 0,
	     flptr = &pudb -> fp01;
	     i < UDB_NUM_TEMP_FLOAT;
	     i++)
	{
	    *flptr = 0;
	    flptr++;
	}

	/* zero the temp unsigned longs */
	for (i = 0,
	     ulptr = &pudb -> lu01;
	     i < UDB_NUM_TEMP_ULONG;
	     i++)
	{
	    *ulptr = 0;
	    ulptr++;
	}

	/* zero the temp doubles */

	for (i = 0,
	     dblptr = &pudb -> db01;
	     i < UDB_NUM_TEMP_DOUBLE;
	     i++)
	{
	    *dblptr = 0;
	    dblptr++;
	}

	/* zero the boolean flags */
	for (i = 0,
	     usptr = &pudb -> bl01;
	     i < UDB_NUM_FLAGS;
	     i++)
	{
	    *usptr = 0;
	    usptr++;
	}

    } /* end if init == 0 */
	 
    return;
}


/****************************************************************************
 *
 *  MODULE NAME            : alarm 
 *  AUTHOR                 : emsteel
 *  DATE                   : 25 Mar 93
 *
 *  CALLING MODULE(S)      : process ()
 *  ARGUMENT(S)            : pudb
 *
 *  DESCRIPTION: set alarm status
 *  NOTES:
 *	if the user has set his own alarm condition in his code, then honor
 *		that and do not check for state alarm
 *	if VAL is 0, then consider this an undefined state alarm and set this
 *		only if user has defined a severity for undefined
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 29 Mar 95
 *  REVISION:	COS_ALARM did return before setting lval
 *       
****************************************************************************/

static void alarm (pudb)
    struct udbRecord  *pudb;
{
    unsigned short	val = pudb -> val;
    int			adis = 0;
    unsigned short	*sevrs = (unsigned short *)&pudb -> sv01;

    /* check for inhibited alarms */

    adis = AlarmDisableLink ((void *)pudb);
    if ( (pudb -> aenb == ALARM_DISABLED) || (adis != 0) )
    {
	pudb -> nsta = INHIBIT_ALARM;
	pudb -> nsev = NO_ALARM;
	return;
    }

    if (pudb -> udf == TRUE)
    {
	if (pudb -> nsev < VALID_ALARM)
	{
	    pudb -> nsta = UDF_ALARM;
	    pudb -> nsev = VALID_ALARM;
	}
	return;
    }

    if (pudb -> nsev == VALID_ALARM)
	return;

    /* if the user has set an alarm condition then 
       honor it
    */
    if ( (pudb -> nsev != NO_ALARM) || (pudb -> nsta != NO_ALARM) )
	return;

    if (val >= UDB_NUM_STATUS)
    {
	if (pudb -> nsev < pudb -> unsv)
	{
	    pudb -> nsta = STATE_ALARM;
	    pudb -> nsev = pudb -> unsv;
	}
    }
    else
    {
	if (pudb -> nsev < sevrs[val])
	{
	    pudb -> nsta = STATE_ALARM;
	    pudb -> nsev = sevrs[val];
	}
    }

    /* check for output of STATE alarm */

    AlarmStates ((void *)pudb, 0);

    /* check for change of state alarm */

    if (val != pudb -> lval)
    {
	if (pudb -> nsev < pudb -> cosv)
	{
	    pudb -> nsta = COS_ALARM;
	    pudb -> nsev = pudb -> cosv;
	}
    }

    pudb -> lval = val;

    /* check for return to normal */

    AlarmStates ((void *)pudb, 1);

    return;

} /* end func alarm () */


/****************************************************************************
 *
 *  MODULE NAME            : delete_udb_task
 *  AUTHOR                 : emsteel
 *  DATE                   : 09 Nov 95
 *
 *  DESCRIPTION: delete a udb intermediate task
 *
****************************************************************************/

static void delete_udb_task (curr_udb_task, delete_task)
    struct udb_task	*curr_udb_task;
    int			delete_task;
{
    struct udb_task	*prev_udb_task, *next_udb_task, *save_udb_task;

    next_udb_task = prev_udb_task = first_udb_task;
    while(1)
    {
	save_udb_task = next_udb_task;
	if (curr_udb_task == next_udb_task)
	{
	    if (next_udb_task == first_udb_task)
	    {
		first_udb_task = first_udb_task -> next_udb_task;
	    }
	    else
	    {
		prev_udb_task -> next_udb_task = next_udb_task -> next_udb_task;
	    }
	    semDelete (save_udb_task -> sem_activate);
	    semDelete (save_udb_task -> sem_done);
	    if (delete_task)
		taskDelete (save_udb_task -> udb_task_id);
	    free ((void*)save_udb_task);
	    break;
	}
	prev_udb_task = next_udb_task;
	next_udb_task = next_udb_task -> next_udb_task;
	if (next_udb_task)
	    continue;
	break;
    }
    return;
}



/****************************************************************************
 *
 *  MODULE NAME            : get_enum_str
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *  DESCRIPTION: return enumeration string
 *
****************************************************************************/

static long get_enum_str (paddr, pstring)
    struct dbAddr *paddr;
    char	  *pstring;
{
    struct udbRecord	*pudb = (struct udbRecord *)paddr->precord;
    char		*psource = NULL;
    unsigned short      *field = (unsigned short *)paddr -> pfield;
    int			bad_value = 0;
    int			num_bytes = 0;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset)
    {
	case ( (int)&(((struct udbRecord *)0) -> mode)):
	    if ((*field) < UDB_NUM_MODES )
	    {
		psource = (char *)pudb -> mst1;
		psource += ((*field) * sizeof (pudb -> mst1));
		num_bytes = sizeof (pudb -> mst1);
	    }
	    else
		bad_value++;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> ncmd)):
	    if ((*field) < UDB_NUM_CMDS )
	    {
		psource = (char *)pudb -> cst1;
		psource += ((*field) * sizeof (pudb -> cst1));
		num_bytes = sizeof (pudb -> cst1);
	    }
	    else
		bad_value++;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> lcmd)):
	    if ((*field) < UDB_NUM_CMDS )
	    {
		psource = (char *)pudb -> cst1;
		psource += ((*field) * sizeof (pudb -> cst1));
		num_bytes = sizeof (pudb -> cst1);
	    }
	    else
		bad_value++;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> func)):
	    if ((*field) < UDB_NUM_FUNCS )
	    {
		psource = (char *)pudb -> fst1;
		psource += ((*field) * sizeof (pudb -> fst1));
		num_bytes = sizeof (pudb -> fst1);
	    }
	    else
		bad_value++;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> stts)):
	case ( (int)&(((struct udbRecord *)0) -> val)):
	    if ((*field) < UDB_NUM_STATUS)
	    {
		psource = (char *)pudb -> ss01;
		psource += ((*field) * sizeof (pudb -> ss01));
		num_bytes = sizeof (pudb -> ss01);
	    }
	    else
		bad_value++;
	    break;

	default:
	    bad_value++;
	    break;
    }

    if (bad_value) 
    {
	strcpy (pstring, "Illegal value");
    }
    else
    {
	strncpy (pstring, psource, num_bytes);
    }

    return(0);

} /* end func get_enum_str () */


/****************************************************************************
 *
 *  MODULE NAME            : get_enum_strs
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: return enumeration strings
 *
****************************************************************************/

static long get_enum_strs(paddr,pes)
    struct dbAddr *paddr;
    struct dbr_enumStrs *pes;
{
    struct udbRecord	*pudb=(struct udbRecord *)paddr->precord;
    char		*psource = NULL;
    int			i;
    short		no_str = 0;
    int			num_bytes = 0;
    int			num_strs = 0;
    int			bad_value = 0;

    bzero ((char *)&pes->strs[0], sizeof(pes->strs));

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset)
    {
	case ( (int)&(((struct udbRecord *)0) -> mode)):
	    psource = (char *)pudb -> mst1;
	    num_bytes = sizeof (pudb -> mst1);
	    num_strs = UDB_NUM_MODES;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> ncmd)):
	case ( (int)&(((struct udbRecord *)0) -> lcmd)):
	    psource = (char *)pudb -> cst1;
	    num_bytes = sizeof (pudb -> cst1);
	    num_strs = UDB_NUM_CMDS;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> func)):
	    psource = (char *)pudb -> fst1;
	    num_bytes = sizeof (pudb -> fst1);
	    num_strs = UDB_NUM_FUNCS;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> stts)):
	case ( (int)&(((struct udbRecord *)0) -> val)):
	    psource = (char *)pudb -> ss01;
	    num_bytes = sizeof (pudb -> ss01);
	    num_strs = UDB_NUM_STATUS;
	    break;

	default:
	    bad_value++;
	    break;
    }

    if (bad_value)
    {
	strcpy (pes -> strs[0], "Illegal value");
	pes -> no_str = 1;
	return (0);
    }

    for (i = 0; i < num_strs; i++, psource += num_bytes)
    {
	strncpy (pes -> strs[i], psource, num_bytes);
	if ((*psource) != 0)
	    no_str = i+1;
    }
    pes -> no_str = no_str;

    return(0);

} /* end func get_enum_strs () */



/****************************************************************************
 *
 *  MODULE NAME            : get_control_double 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: return control limits
 *
****************************************************************************/

static long get_control_double(paddr,pcd)
    struct	dbAddr		*paddr;
    struct	dbr_ctrlDouble	*pcd;
{
    struct udbRecord	*pudb = (struct udbRecord *) paddr->precord;
    struct dbAddr	dbAddr;
    struct link		*pdblink;
    struct rset		*prset;
    struct dbr_ctrlDouble ctrlDouble;
    int			offset_start;
    int			offset_in;
    int			index;
    short		fld_size;
    unsigned short	*float_limits = &pudb -> fl01;
    unsigned short	*dbl_limits = &pudb -> dl01;
    unsigned short	*ulong_limits = &pudb -> ul01;
    unsigned short	*tmp_float_limits = &pudb -> ft01;
    unsigned short	*tmp_dbl_limits = &pudb -> td01;
    unsigned short	*tmp_ulong_limits = &pudb -> ud01;
    double		*inv = &pudb -> inv1;
    double		*outv = &pudb -> ovl1;
    long		status = 0;
    char		pvname[PVNAME_SZ + 8];
    int			have_limits;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct udbRecord *)0) -> n01) ):
	case ( (int)&(((struct udbRecord *)0) -> n02) ):
	case ( (int)&(((struct udbRecord *)0) -> n03) ):
	case ( (int)&(((struct udbRecord *)0) -> n04) ):
	case ( (int)&(((struct udbRecord *)0) -> n05) ):
	case ( (int)&(((struct udbRecord *)0) -> n06) ):
	case ( (int)&(((struct udbRecord *)0) -> n07) ):
	case ( (int)&(((struct udbRecord *)0) -> n08) ):
	case ( (int)&(((struct udbRecord *)0) -> n09) ):
	case ( (int)&(((struct udbRecord *)0) -> n10) ):
	case ( (int)&(((struct udbRecord *)0) -> n11) ):
	case ( (int)&(((struct udbRecord *)0) -> n12) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> n01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    float_limits += index;

	    if ((*float_limits) == 0)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*float_limits) == 1)
	    {
		pcd -> upper_ctrl_limit = pudb -> l1hi;
		pcd -> lower_ctrl_limit = pudb -> l1lo;
	    }
	    else if ((*float_limits) == 2)
	    {
		pcd -> upper_ctrl_limit = pudb -> l2hi; 
		pcd -> lower_ctrl_limit = pudb -> l2lo;
	    }
	    else if ((*float_limits) == 3)
	    {
		pcd -> upper_ctrl_limit = pudb -> l3hi; 
		pcd -> lower_ctrl_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> d01) ):
	case ( (int)&(((struct udbRecord *)0) -> d02) ):
	case ( (int)&(((struct udbRecord *)0) -> d03) ):
	case ( (int)&(((struct udbRecord *)0) -> d04) ):
	case ( (int)&(((struct udbRecord *)0) -> d05) ):
	case ( (int)&(((struct udbRecord *)0) -> d06) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> d01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    dbl_limits += index;

	    if ((*dbl_limits) == 0)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*dbl_limits) == 1)
	    {
		pcd -> upper_ctrl_limit = pudb -> l1hi;
		pcd -> lower_ctrl_limit = pudb -> l1lo;
	    }
	    else if ((*dbl_limits) == 2)
	    {
		pcd -> upper_ctrl_limit = pudb -> l2hi; 
		pcd -> lower_ctrl_limit = pudb -> l2lo;
	    }
	    else if ((*dbl_limits) == 3)
	    {
		pcd -> upper_ctrl_limit = pudb -> l3hi; 
		pcd -> lower_ctrl_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> u01) ):
	case ( (int)&(((struct udbRecord *)0) -> u02) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> u01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    ulong_limits += index;

	    if ((*ulong_limits) == 0)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*ulong_limits) == 1)
	    {
		pcd -> upper_ctrl_limit = pudb -> l1hi;
		pcd -> lower_ctrl_limit = pudb -> l1lo;
	    }
	    else if ((*ulong_limits) == 2)
	    {
		pcd -> upper_ctrl_limit = pudb -> l2hi; 
		pcd -> lower_ctrl_limit = pudb -> l2lo;
	    }
	    else if ((*ulong_limits) == 3)
	    {
		pcd -> upper_ctrl_limit = pudb -> l3hi; 
		pcd -> lower_ctrl_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> fp01) ):
	case ( (int)&(((struct udbRecord *)0) -> fp02) ):
	case ( (int)&(((struct udbRecord *)0) -> fp03) ):
	case ( (int)&(((struct udbRecord *)0) -> fp04) ):
	case ( (int)&(((struct udbRecord *)0) -> fp05) ):
	case ( (int)&(((struct udbRecord *)0) -> fp06) ):
	case ( (int)&(((struct udbRecord *)0) -> fp07) ):
	case ( (int)&(((struct udbRecord *)0) -> fp08) ):
	case ( (int)&(((struct udbRecord *)0) -> fp09) ):
	case ( (int)&(((struct udbRecord *)0) -> fp10) ):
	case ( (int)&(((struct udbRecord *)0) -> fp11) ):
	case ( (int)&(((struct udbRecord *)0) -> fp12) ):
	case ( (int)&(((struct udbRecord *)0) -> fp13) ):
	case ( (int)&(((struct udbRecord *)0) -> fp14) ):
	case ( (int)&(((struct udbRecord *)0) -> fp15) ):
	case ( (int)&(((struct udbRecord *)0) -> fp16) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> fp01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_float_limits += index;

	    if ((*tmp_float_limits) == 0)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*tmp_float_limits) == 1)
	    {
		pcd -> upper_ctrl_limit = pudb -> l1hi;
		pcd -> lower_ctrl_limit = pudb -> l1lo;
	    }
	    else if ((*tmp_float_limits) == 2)
	    {
		pcd -> upper_ctrl_limit = pudb -> l2hi; 
		pcd -> lower_ctrl_limit = pudb -> l2lo;
	    }
	    else if ((*tmp_float_limits) == 3)
	    {
		pcd -> upper_ctrl_limit = pudb -> l3hi; 
		pcd -> lower_ctrl_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> lu01) ):
	case ( (int)&(((struct udbRecord *)0) -> lu02) ):
	case ( (int)&(((struct udbRecord *)0) -> lu03) ):
	case ( (int)&(((struct udbRecord *)0) -> lu04) ):
	case ( (int)&(((struct udbRecord *)0) -> lu05) ):
	case ( (int)&(((struct udbRecord *)0) -> lu06) ):
	case ( (int)&(((struct udbRecord *)0) -> lu07) ):
	case ( (int)&(((struct udbRecord *)0) -> lu08) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> lu01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_ulong_limits += index;

	    if ((*tmp_ulong_limits) == 0)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*tmp_ulong_limits) == 1)
	    {
		pcd -> upper_ctrl_limit = pudb -> l1hi;
		pcd -> lower_ctrl_limit = pudb -> l1lo;
	    }
	    else if ((*tmp_ulong_limits) == 2)
	    {
		pcd -> upper_ctrl_limit = pudb -> l2hi; 
		pcd -> lower_ctrl_limit = pudb -> l2lo;
	    }
	    else if ((*tmp_ulong_limits) == 3)
	    {
		pcd -> upper_ctrl_limit = pudb -> l3hi; 
		pcd -> lower_ctrl_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> db01) ):
	case ( (int)&(((struct udbRecord *)0) -> db02) ):
	case ( (int)&(((struct udbRecord *)0) -> db03) ):
	case ( (int)&(((struct udbRecord *)0) -> db04) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> db01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_dbl_limits += index;

	    if ((*tmp_dbl_limits) == 0)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*tmp_dbl_limits) == 1)
	    {
		pcd -> upper_ctrl_limit = pudb -> l1hi;
		pcd -> lower_ctrl_limit = pudb -> l1lo;
	    }
	    else if ((*tmp_dbl_limits) == 2)
	    {
		pcd -> upper_ctrl_limit = pudb -> l2hi; 
		pcd -> lower_ctrl_limit = pudb -> l2lo;
	    }
	    else if ((*tmp_dbl_limits) == 3)
	    {
		pcd -> upper_ctrl_limit = pudb -> l3hi; 
		pcd -> lower_ctrl_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> inv1) ):
	case ( (int)&(((struct udbRecord *)0) -> inv2) ):
	case ( (int)&(((struct udbRecord *)0) -> inv3) ):
	case ( (int)&(((struct udbRecord *)0) -> inv4) ):
	case ( (int)&(((struct udbRecord *)0) -> inv5) ):
	case ( (int)&(((struct udbRecord *)0) -> inv6) ):
	case ( (int)&(((struct udbRecord *)0) -> inv7) ):
	case ( (int)&(((struct udbRecord *)0) -> inv8) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> inv1);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    pdblink = &pudb -> inp1;
	    pdblink += index;
	    inv += index;

	    have_limits = 0;
	    if (pdblink -> type == DB_LINK)
	    {
		strncpy (pvname, pdblink -> value.db_link.pvname, sizeof (pvname));
		if ((status = dbNameToAddr (pvname, &dbAddr)) == NULL)
		{
		    prset = GET_PRSET(dbAddr.record_type);
		    if (prset)
		    {
			if (prset -> get_control_double)
			{
			    (*(prset -> get_control_double))(&dbAddr, &ctrlDouble);
			    pcd -> upper_ctrl_limit = ctrlDouble.upper_ctrl_limit;
			    pcd -> lower_ctrl_limit = ctrlDouble.lower_ctrl_limit;
			    have_limits++;
			}
		    }
		}
	    }
	    if (!have_limits)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> ovl1) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl2) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl3) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl4) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl5) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl6) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl7) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl8) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> ovl1);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    pdblink = &pudb -> out1;
	    pdblink += index;
	    outv += index;

	    have_limits = 0;
	    if (pdblink -> type == DB_LINK)
	    {
		strncpy (pvname, pdblink -> value.db_link.pvname, sizeof (pvname));
		if ((status = dbNameToAddr (pvname, &dbAddr)) == NULL)
		{
		    prset = GET_PRSET(dbAddr.record_type);
		    if (prset)
		    {
			if (prset -> get_control_double)
			{
			    (*(prset -> get_control_double))(&dbAddr, &ctrlDouble);
			    pcd -> upper_ctrl_limit = ctrlDouble.upper_ctrl_limit;
			    pcd -> lower_ctrl_limit = ctrlDouble.lower_ctrl_limit;
			    have_limits++;
			}
		    }
		}
	    }
	    if (!have_limits)
	    {
		pcd -> upper_ctrl_limit = UDB_UPPER_CTRL_LIMIT;
		pcd -> lower_ctrl_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> l1lo) ):
	case ( (int)&(((struct udbRecord *)0) -> l1hi) ):
	case ( (int)&(((struct udbRecord *)0) -> l2lo) ):
	case ( (int)&(((struct udbRecord *)0) -> l2hi) ):
	case ( (int)&(((struct udbRecord *)0) -> l3lo) ):
	case ( (int)&(((struct udbRecord *)0) -> l3hi) ):
	    pcd -> upper_ctrl_limit = 999999999;
	    pcd -> lower_ctrl_limit = -999999999;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> l1pr) ):
	case ( (int)&(((struct udbRecord *)0) -> l2pr) ):
	case ( (int)&(((struct udbRecord *)0) -> l3pr) ):
	    pcd -> upper_ctrl_limit = 9;
	    pcd -> lower_ctrl_limit = 0;
	    break;

	default:
	    break;

    }
    return (0);

} /* end func get_control_double () */


/****************************************************************************
 *
 *  MODULE NAME            : get_graphic_double 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: return graphic limits
 *
****************************************************************************/

static long get_graphic_double(paddr,pgd)
    struct	dbAddr		*paddr;
    struct	dbr_grDouble	*pgd;
{
    struct udbRecord    *pudb = (struct udbRecord *) paddr->precord;
    struct dbAddr	dbAddr;
    struct link		*pdblink;
    struct rset		*prset;
    struct dbr_grDouble	grDouble;
    int			offset_start;
    int			offset_in;
    int			index;
    short		fld_size;
    unsigned short	*float_limits = &pudb -> fl01;
    unsigned short	*dbl_limits = &pudb -> dl01;
    unsigned short	*ulong_limits = &pudb -> ul01;
    unsigned short	*tmp_float_limits = &pudb -> ft01;
    unsigned short	*tmp_dbl_limits = &pudb -> td01;
    unsigned short	*tmp_ulong_limits = &pudb -> ud01;
    double		*inv = &pudb -> inv1;
    double		*outv = &pudb -> ovl1;
    long		status = 0;
    char		pvname[PVNAME_SZ + 8];
    int			have_limits;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct udbRecord *)0) -> n01) ):
	case ( (int)&(((struct udbRecord *)0) -> n02) ):
	case ( (int)&(((struct udbRecord *)0) -> n03) ):
	case ( (int)&(((struct udbRecord *)0) -> n04) ):
	case ( (int)&(((struct udbRecord *)0) -> n05) ):
	case ( (int)&(((struct udbRecord *)0) -> n06) ):
	case ( (int)&(((struct udbRecord *)0) -> n07) ):
	case ( (int)&(((struct udbRecord *)0) -> n08) ):
	case ( (int)&(((struct udbRecord *)0) -> n09) ):
	case ( (int)&(((struct udbRecord *)0) -> n10) ):
	case ( (int)&(((struct udbRecord *)0) -> n11) ):
	case ( (int)&(((struct udbRecord *)0) -> n12) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> n01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    float_limits += index;

	    if ((*float_limits) == 0)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*float_limits) == 1)
	    {
		pgd -> upper_disp_limit = pudb -> l1hi;
		pgd -> lower_disp_limit = pudb -> l1lo;
	    }
	    else if ((*float_limits) == 2)
	    {
		pgd -> upper_disp_limit = pudb -> l2hi; 
		pgd -> lower_disp_limit = pudb -> l2lo;
	    }
	    else if ((*float_limits) == 3)
	    {
		pgd -> upper_disp_limit = pudb -> l3hi; 
		pgd -> lower_disp_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> d01) ):
	case ( (int)&(((struct udbRecord *)0) -> d02) ):
	case ( (int)&(((struct udbRecord *)0) -> d03) ):
	case ( (int)&(((struct udbRecord *)0) -> d04) ):
	case ( (int)&(((struct udbRecord *)0) -> d05) ):
	case ( (int)&(((struct udbRecord *)0) -> d06) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> d01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    dbl_limits += index;

	    if ((*dbl_limits) == 0)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*dbl_limits) == 1)
	    {
		pgd -> upper_disp_limit = pudb -> l1hi;
		pgd -> lower_disp_limit = pudb -> l1lo;
	    }
	    else if ((*dbl_limits) == 2)
	    {
		pgd -> upper_disp_limit = pudb -> l2hi; 
		pgd -> lower_disp_limit = pudb -> l2lo;
	    }
	    else if ((*dbl_limits) == 3)
	    {
		pgd -> upper_disp_limit = pudb -> l3hi; 
		pgd -> lower_disp_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> u01) ):
	case ( (int)&(((struct udbRecord *)0) -> u02) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> u01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    ulong_limits += index;

	    if ((*ulong_limits) == 0)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*ulong_limits) == 1)
	    {
		pgd -> upper_disp_limit = pudb -> l1hi;
		pgd -> lower_disp_limit = pudb -> l1lo;
	    }
	    else if ((*ulong_limits) == 2)
	    {
		pgd -> upper_disp_limit = pudb -> l2hi; 
		pgd -> lower_disp_limit = pudb -> l2lo;
	    }
	    else if ((*ulong_limits) == 3)
	    {
		pgd -> upper_disp_limit = pudb -> l3hi; 
		pgd -> lower_disp_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> fp01) ):
	case ( (int)&(((struct udbRecord *)0) -> fp02) ):
	case ( (int)&(((struct udbRecord *)0) -> fp03) ):
	case ( (int)&(((struct udbRecord *)0) -> fp04) ):
	case ( (int)&(((struct udbRecord *)0) -> fp05) ):
	case ( (int)&(((struct udbRecord *)0) -> fp06) ):
	case ( (int)&(((struct udbRecord *)0) -> fp07) ):
	case ( (int)&(((struct udbRecord *)0) -> fp08) ):
	case ( (int)&(((struct udbRecord *)0) -> fp09) ):
	case ( (int)&(((struct udbRecord *)0) -> fp10) ):
	case ( (int)&(((struct udbRecord *)0) -> fp11) ):
	case ( (int)&(((struct udbRecord *)0) -> fp12) ):
	case ( (int)&(((struct udbRecord *)0) -> fp13) ):
	case ( (int)&(((struct udbRecord *)0) -> fp14) ):
	case ( (int)&(((struct udbRecord *)0) -> fp15) ):
	case ( (int)&(((struct udbRecord *)0) -> fp16) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> fp01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_float_limits += index;

	    if ((*tmp_float_limits) == 0)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*tmp_float_limits) == 1)
	    {
		pgd -> upper_disp_limit = pudb -> l1hi;
		pgd -> lower_disp_limit = pudb -> l1lo;
	    }
	    else if ((*tmp_float_limits) == 2)
	    {
		pgd -> upper_disp_limit = pudb -> l2hi; 
		pgd -> lower_disp_limit = pudb -> l2lo;
	    }
	    else if ((*tmp_float_limits) == 3)
	    {
		pgd -> upper_disp_limit = pudb -> l3hi; 
		pgd -> lower_disp_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> lu01) ):
	case ( (int)&(((struct udbRecord *)0) -> lu02) ):
	case ( (int)&(((struct udbRecord *)0) -> lu03) ):
	case ( (int)&(((struct udbRecord *)0) -> lu04) ):
	case ( (int)&(((struct udbRecord *)0) -> lu05) ):
	case ( (int)&(((struct udbRecord *)0) -> lu06) ):
	case ( (int)&(((struct udbRecord *)0) -> lu07) ):
	case ( (int)&(((struct udbRecord *)0) -> lu08) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> lu01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_ulong_limits += index;

	    if ((*tmp_ulong_limits) == 0)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*tmp_ulong_limits) == 1)
	    {
		pgd -> upper_disp_limit = pudb -> l1hi;
		pgd -> lower_disp_limit = pudb -> l1lo;
	    }
	    else if ((*tmp_ulong_limits) == 2)
	    {
		pgd -> upper_disp_limit = pudb -> l2hi; 
		pgd -> lower_disp_limit = pudb -> l2lo;
	    }
	    else if ((*tmp_ulong_limits) == 3)
	    {
		pgd -> upper_disp_limit = pudb -> l3hi; 
		pgd -> lower_disp_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> db01) ):
	case ( (int)&(((struct udbRecord *)0) -> db02) ):
	case ( (int)&(((struct udbRecord *)0) -> db03) ):
	case ( (int)&(((struct udbRecord *)0) -> db04) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> db01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_dbl_limits += index;

	    if ((*tmp_dbl_limits) == 0)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    else if ((*tmp_dbl_limits) == 1)
	    {
		pgd -> upper_disp_limit = pudb -> l1hi;
		pgd -> lower_disp_limit = pudb -> l1lo;
	    }
	    else if ((*tmp_dbl_limits) == 2)
	    {
		pgd -> upper_disp_limit = pudb -> l2hi; 
		pgd -> lower_disp_limit = pudb -> l2lo;
	    }
	    else if ((*tmp_dbl_limits) == 3)
	    {
		pgd -> upper_disp_limit = pudb -> l3hi; 
		pgd -> lower_disp_limit = pudb -> l3lo;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> inv1) ):
	case ( (int)&(((struct udbRecord *)0) -> inv2) ):
	case ( (int)&(((struct udbRecord *)0) -> inv3) ):
	case ( (int)&(((struct udbRecord *)0) -> inv4) ):
	case ( (int)&(((struct udbRecord *)0) -> inv5) ):
	case ( (int)&(((struct udbRecord *)0) -> inv6) ):
	case ( (int)&(((struct udbRecord *)0) -> inv7) ):
	case ( (int)&(((struct udbRecord *)0) -> inv8) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> inv1);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    pdblink = &pudb -> inp1;
	    pdblink += index;
	    inv += index;

	    have_limits = 0;
	    if (pdblink -> type == DB_LINK)
	    {
		strncpy (pvname, pdblink -> value.db_link.pvname, sizeof (pvname));
		if ((status = dbNameToAddr (pvname, &dbAddr)) == NULL)
		{
		    prset = GET_PRSET(dbAddr.record_type);
		    if (prset)
		    {
			if (prset -> get_graphic_double)
			{
			    (*(prset -> get_graphic_double))(&dbAddr, &grDouble);
			    pgd -> upper_disp_limit = grDouble.upper_disp_limit;
			    pgd -> lower_disp_limit = grDouble.lower_disp_limit;
			    have_limits++;
			}
		    }
		}
	    }
	    if (!have_limits)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> ovl1) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl2) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl3) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl4) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl5) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl6) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl7) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl8) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> ovl1);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    pdblink = &pudb -> out1;
	    pdblink += index;
	    outv += index;

	    have_limits = 0;
	    if (pdblink -> type == DB_LINK)
	    {
		strncpy (pvname, pdblink -> value.db_link.pvname, sizeof (pvname));
		if ((status = dbNameToAddr (pvname, &dbAddr)) == NULL)
		{
		    prset = GET_PRSET(dbAddr.record_type);
		    if (prset)
		    {
			if (prset -> get_graphic_double)
			{
			    (*(prset -> get_graphic_double))(&dbAddr, &grDouble);
			    pgd -> upper_disp_limit = grDouble.upper_disp_limit;
			    pgd -> lower_disp_limit = grDouble.lower_disp_limit;
			    have_limits++;
			}
		    }
		}
	    }
	    if (!have_limits)
	    {
		pgd -> upper_disp_limit = UDB_UPPER_CTRL_LIMIT;
		pgd -> lower_disp_limit = UDB_LOWER_CTRL_LIMIT;
	    }
	    break;

	case ( (int)&(((struct udbRecord *)0) -> l1lo) ):
	case ( (int)&(((struct udbRecord *)0) -> l1hi) ):
	case ( (int)&(((struct udbRecord *)0) -> l2lo) ):
	case ( (int)&(((struct udbRecord *)0) -> l2hi) ):
	case ( (int)&(((struct udbRecord *)0) -> l3lo) ):
	case ( (int)&(((struct udbRecord *)0) -> l3hi) ):
	    pgd -> upper_disp_limit = 999999999;
	    pgd -> lower_disp_limit = -999999999;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> l1pr) ):
	case ( (int)&(((struct udbRecord *)0) -> l2pr) ):
	case ( (int)&(((struct udbRecord *)0) -> l3pr) ):
	    pgd -> upper_disp_limit = 9;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    break;

    }

    return (0);

} /* end func get_graphic_double () */



/****************************************************************************
 *
 *  MODULE NAME            : get_precision 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: return precision value
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 17 Dec 96
 *  REVISION:	tmp dbl and tmp ul limits were reversed
 *
****************************************************************************/

static long get_precision (paddr, precision)
    struct dbAddr     *paddr;
    long              *precision;
{
    struct udbRecord *pudb = (struct udbRecord *)paddr -> precord;
    struct dbAddr	dbAddr;
    struct link		*pdblink;
    struct rset		*prset;
    int			offset_start;
    int			offset_in;
    int			index;
    short		fld_size;
    unsigned short	*float_limits = &pudb -> fl01;
    unsigned short	*dbl_limits = &pudb -> dl01;
    unsigned short	*ulong_limits = &pudb -> ul01;
    unsigned short	*tmp_float_limits = &pudb -> ft01;
    unsigned short	*tmp_dbl_limits = &pudb -> td01;
    unsigned short	*tmp_ulong_limits = &pudb -> ud01;
    double		*inv = &pudb -> inv1;
    double		*outv = &pudb -> ovl1;
    long		status = 0;
    char		pvname[PVNAME_SZ + 8];
    long		prec;
    int			have_precision;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct udbRecord *)0) -> n01) ):
	case ( (int)&(((struct udbRecord *)0) -> n02) ):
	case ( (int)&(((struct udbRecord *)0) -> n03) ):
	case ( (int)&(((struct udbRecord *)0) -> n04) ):
	case ( (int)&(((struct udbRecord *)0) -> n05) ):
	case ( (int)&(((struct udbRecord *)0) -> n06) ):
	case ( (int)&(((struct udbRecord *)0) -> n07) ):
	case ( (int)&(((struct udbRecord *)0) -> n08) ):
	case ( (int)&(((struct udbRecord *)0) -> n09) ):
	case ( (int)&(((struct udbRecord *)0) -> n10) ):
	case ( (int)&(((struct udbRecord *)0) -> n11) ):
	case ( (int)&(((struct udbRecord *)0) -> n12) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> n01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    float_limits += index;

	    if ((*float_limits) == 0)
		*precision = UDB_DEFAULT_PRECISION;
	    else if ((*float_limits) == 1)
		*precision = pudb -> l1pr;
	    else if ((*float_limits) == 2)
		*precision = pudb -> l2pr;
	    else if ((*float_limits) == 3)
		*precision = pudb -> l3pr;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> d01) ):
	case ( (int)&(((struct udbRecord *)0) -> d02) ):
	case ( (int)&(((struct udbRecord *)0) -> d03) ):
	case ( (int)&(((struct udbRecord *)0) -> d04) ):
	case ( (int)&(((struct udbRecord *)0) -> d05) ):
	case ( (int)&(((struct udbRecord *)0) -> d06) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> d01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    dbl_limits += index;

	    if ((*dbl_limits) == 0)
		*precision = UDB_DEFAULT_PRECISION;
	    else if ((*dbl_limits) == 1)
		*precision = pudb -> l1pr;
	    else if ((*dbl_limits) == 2)
		*precision = pudb -> l2pr;
	    else if ((*dbl_limits) == 3)
		*precision = pudb -> l3pr;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> u01) ):
	case ( (int)&(((struct udbRecord *)0) -> u02) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> u01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    ulong_limits += index;

	    if ((*ulong_limits) == 0)
		*precision = UDB_DEFAULT_PRECISION;
	    else if ((*ulong_limits) == 1)
		*precision = pudb -> l1pr;
	    else if ((*ulong_limits) == 2)
		*precision = pudb -> l2pr;
	    else if ((*ulong_limits) == 3)
		*precision = pudb -> l3pr;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> fp01) ):
	case ( (int)&(((struct udbRecord *)0) -> fp02) ):
	case ( (int)&(((struct udbRecord *)0) -> fp03) ):
	case ( (int)&(((struct udbRecord *)0) -> fp04) ):
	case ( (int)&(((struct udbRecord *)0) -> fp05) ):
	case ( (int)&(((struct udbRecord *)0) -> fp06) ):
	case ( (int)&(((struct udbRecord *)0) -> fp07) ):
	case ( (int)&(((struct udbRecord *)0) -> fp08) ):
	case ( (int)&(((struct udbRecord *)0) -> fp09) ):
	case ( (int)&(((struct udbRecord *)0) -> fp10) ):
	case ( (int)&(((struct udbRecord *)0) -> fp11) ):
	case ( (int)&(((struct udbRecord *)0) -> fp12) ):
	case ( (int)&(((struct udbRecord *)0) -> fp13) ):
	case ( (int)&(((struct udbRecord *)0) -> fp14) ):
	case ( (int)&(((struct udbRecord *)0) -> fp15) ):
	case ( (int)&(((struct udbRecord *)0) -> fp16) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> fp01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_float_limits += index;

	    if ((*tmp_float_limits) == 0)
		*precision = UDB_DEFAULT_PRECISION;
	    else if ((*tmp_float_limits) == 1)
		*precision = pudb -> l1pr;
	    else if ((*tmp_float_limits) == 2)
		*precision = pudb -> l2pr;
	    else if ((*tmp_float_limits) == 3)
		*precision = pudb -> l3pr;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> lu01) ):
	case ( (int)&(((struct udbRecord *)0) -> lu02) ):
	case ( (int)&(((struct udbRecord *)0) -> lu03) ):
	case ( (int)&(((struct udbRecord *)0) -> lu04) ):
	case ( (int)&(((struct udbRecord *)0) -> lu05) ):
	case ( (int)&(((struct udbRecord *)0) -> lu06) ):
	case ( (int)&(((struct udbRecord *)0) -> lu07) ):
	case ( (int)&(((struct udbRecord *)0) -> lu08) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> lu01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_ulong_limits += index;

	    if ((*tmp_ulong_limits) == 0)
		*precision = UDB_DEFAULT_PRECISION;
	    else if ((*tmp_ulong_limits) == 1)
		*precision = pudb -> l1pr;
	    else if ((*tmp_ulong_limits) == 2)
		*precision = pudb -> l2pr;
	    else if ((*tmp_ulong_limits) == 3)
		*precision = pudb -> l3pr;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> db01) ):
	case ( (int)&(((struct udbRecord *)0) -> db02) ):
	case ( (int)&(((struct udbRecord *)0) -> db03) ):
	case ( (int)&(((struct udbRecord *)0) -> db04) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> db01);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    tmp_dbl_limits += index;

	    if ((*tmp_dbl_limits) == 0)
		*precision = UDB_DEFAULT_PRECISION;
	    else if ((*tmp_dbl_limits) == 1)
		*precision = pudb -> l1pr;
	    else if ((*tmp_dbl_limits) == 2)
		*precision = pudb -> l2pr;
	    else if ((*tmp_dbl_limits) == 3)
		*precision = pudb -> l3pr;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> inv1) ):
	case ( (int)&(((struct udbRecord *)0) -> inv2) ):
	case ( (int)&(((struct udbRecord *)0) -> inv3) ):
	case ( (int)&(((struct udbRecord *)0) -> inv4) ):
	case ( (int)&(((struct udbRecord *)0) -> inv5) ):
	case ( (int)&(((struct udbRecord *)0) -> inv6) ):
	case ( (int)&(((struct udbRecord *)0) -> inv7) ):
	case ( (int)&(((struct udbRecord *)0) -> inv8) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> inv1);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    pdblink = &pudb -> inp1;
	    pdblink += index;
	    inv += index;

	    have_precision = 0;
	    if (pdblink -> type == DB_LINK)
	    {
		strncpy (pvname, pdblink -> value.db_link.pvname, sizeof (pvname));
		if ((status = dbNameToAddr (pvname, &dbAddr)) == NULL)
		{
		    prset = GET_PRSET(dbAddr.record_type);
		    if (prset)
		    {
			if (prset -> get_precision)
			{
			    (*(prset -> get_precision))(&dbAddr, &prec);
			    *precision = prec;
			    have_precision++;
			}
		    }
		}
	    }
	    if (!have_precision)
		*precision = UDB_DEFAULT_PRECISION;

	    break;

	case ( (int)&(((struct udbRecord *)0) -> ovl1) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl2) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl3) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl4) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl5) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl6) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl7) ):
	case ( (int)&(((struct udbRecord *)0) -> ovl8) ):
	    offset_start = (int)&(((struct udbRecord *)0) -> ovl1);
	    offset_in = ((struct fldDes *)paddr -> pfldDes) -> offset;
	    fld_size = ((struct fldDes *)paddr -> pfldDes) -> size;
	    index = (offset_in - offset_start) / fld_size;
	    pdblink = &pudb -> out1;
	    pdblink += index;
	    outv += index;

	    have_precision = 0;
	    if (pdblink -> type == DB_LINK)
	    {
		strncpy (pvname, pdblink -> value.db_link.pvname, sizeof (pvname));
		if ((status = dbNameToAddr (pvname, &dbAddr)) == NULL)
		{
		    prset = GET_PRSET(dbAddr.record_type);
		    if (prset)
		    {
			if (prset -> get_precision)
			{
			    (*(prset -> get_precision))(&dbAddr, &prec);
			    *precision = prec;
			    have_precision++;
			}
		    }
		}
	    }
	    if (!have_precision)
		*precision = UDB_DEFAULT_PRECISION;

	    break;

	default:
	    break;

    }

    return (0);

} /* end func get_precision () */


/****************************************************************************
 *
 *  MODULE NAME            : get_value 
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: return the value field 
 *
****************************************************************************/

static long get_value (pudb, pvdes)
       struct udbRecord   *pudb;
       struct valueDes     *pvdes;
{
   pvdes -> field_type = DBF_ENUM;
   pvdes -> no_elements = 1;
   (unsigned short *)(pvdes -> pvalue) = &pudb -> val;

   return (0);

} /* end func get_value () */



/****************************************************************************
 *
 *  MODULE NAME            : monitor 
 *  AUTHOR                 : emsteel
 *  DATE                   : 17 Jan 95
 *
 *  CALLING MODULE(S)      : process ()
 *                           run_commands ()
 *
 *  DESCRIPTION: check event list
 *
 ***************************** REVISION HISTORY *****************************
 *
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 15 Mar 95
 *  REVISION:	do not set ack if sevr is EVENT_ALARM
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 11 Sept 95
 *  REVISION:	archive val if stat/sevr also chg's
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 12 Sept 95
 *  REVISION:	chg way handle VALID_ALARM
 *
****************************************************************************/

static void monitor (pudb)
    struct udbRecord *pudb;
{
    unsigned short	monitor_mask;
    unsigned short	*severities;
    short		stat,sevr,nsta,nsev;
    int			set_ack = 1;

    /* get previous stat and sevr  and new stat and sevr*/

    stat = pudb->stat;
    sevr = pudb->sevr;
    nsta = pudb->nsta;
    nsev = pudb->nsev;

    /*set current stat and sevr*/

    pudb->stat = nsta;
    pudb->sevr = nsev;
    pudb->nsta = 0;
    pudb->nsev = 0;

    /* if we are coming out of VALID_ALARM and our new stat/sevr are
       the same as when we went into VALID_ALARM, do not set ack
    */
    if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	 (nsta == pudb -> psta) && (nsev == pudb -> psev) )
    {
	set_ack = 0;
    }
    /* if we are going into VALID_ALARM, and out previous stat was not
       VALID_ALARM, remember previous non VALID_ALARM stat/sevr
       and we do not want to set ACK,
    */
    if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
    {
	set_ack = 0;
	pudb -> psta = stat;
	pudb -> psev = sevr;
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> psta, DBE_VALUE);
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> psev, DBE_VALUE);
    }

    monitor_mask = 0;

    /* alarm condition changed this scan */
    /*
	EXTRA PATCH: If the user defines two states with the same alarm
	severity and then monitors .VAL with mask DBE_ALARM, then he'll
	never see a transition. This patch makes sure that he (i.e., the
	alarm manager) will see the transition.
    */

    severities = (unsigned short *) &(pudb -> sv01);
    if ( ((nsta == STATE_ALARM) || (nsta == COS_ALARM)) && 
         (pudb -> mlst != pudb -> val) )
	stat = ~nsta;

    if ((stat != nsta) || (sevr != nsev)) 
    {
	/* post events for alarm condition change*/

	monitor_mask = DBE_ALARM;

	/* when alarm status chg's, get current time and
	   save VAL in lalm (last value when alarm chg'd)
	*/
	tsLocalTime (&pudb -> latm);
	pudb -> lalm = pudb -> val;

	/* post stat and sevr fields */

	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb->stat, DBE_VALUE);
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb->sevr, DBE_VALUE);
	if ((pudb->sevr != NO_ALARM) && (set_ack) &&
	    (pudb->sevr != EVENT_ALARM) ) 
	{
	    if (pudb->ack == 0)
	    {
		pudb->ack = 1;           /* new unacknowledged alarm */
		db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb->ack, DBE_VALUE);
	    }
	}
    }

    /* check for value change */
    if (pudb->mlst != pudb->val)
    {
	/* post events for value change and archive change */

	monitor_mask |= DBE_VALUE;
    }

    if (pudb -> tenb)
    {
	if ( (pudb -> mlst != pudb -> val) || (monitor_mask & DBE_ALARM) )
	{
	    monitor_mask |= DBE_LOG;
	}
    }

    if (monitor_mask & (DBE_VALUE | DBE_LOG))
	pudb -> mlst = pudb -> val;

    /* send out monitors connected to the value field */
    if (monitor_mask)
	db_post_events((struct dbCommon *)pudb, (caddr_t)&pudb->val, monitor_mask);

    return;

} /* end func monitor () */


/****************************************************************************
 *
 *  MODULE NAME            : post_and_redund
 *  AUTHOR                 : emsteel
 *  DATE                   : 16 Jan 95
 *
 *  PARAMETERS             : pudb - ptr to udb record
 *			     old_udb - ptr to saved udb data
 * 
 *  DESCRIPTION: post events and send redundancies where needed
 *
****************************************************************************/

static void post_and_redund (pudb, old_udb)
    struct udbRecord		*pudb;
    struct udbRecord		*old_udb;
{
    char		redbuf[REDQ_MAX_MSGLEN];
    unsigned short	monitor_mask;
    float		*old_float;
    float		*new_float;
    double		*old_dbl;
    double		*new_dbl;
    double		*inpv;
    double		*outv;
    unsigned long	*old_ul;
    unsigned long	*new_ul;
    unsigned long	*old_timer;
    unsigned long	*new_timer;
    unsigned short	*old_flag;
    unsigned short	*new_flag;
    int			do_rcrc = 0;
    int			i;

    /* if any prompted fields have changed, we will have to 
       recalcuate the CRC (check sum)
    */
    if (old_udb -> mode != pudb -> mode)
	do_rcrc = 1;
    if (old_udb -> func != pudb -> func)
	do_rcrc = 1;
    if (!do_rcrc)
    {
	for (i = 0, old_float = &old_udb -> n01, new_float = &pudb -> n01;
	     i < UDB_NUM_NUMERIC_FLOAT;
	     i++)
	{
	    if ( (*old_float) != (*new_float) )
	    {
		do_rcrc = 1;
		break;
	    }
	    old_float++;
	    new_float++;
	}
    }
    if (!do_rcrc)
    {
	for (i = 0, old_dbl = &old_udb -> d01, new_dbl = &pudb -> d01;
	     i < UDB_NUM_NUMERIC_DOUBLE;
	     i++)
	{
	    if ( (*old_dbl) != (*new_dbl) )
	    {
		do_rcrc = 1;
		break;
	    }
	    old_dbl++;
	    new_dbl++;
	}
    }
    if (!do_rcrc)
    {
	for (i = 0, old_ul = &old_udb -> u01, new_ul = &pudb -> u01;
	     i < UDB_NUM_NUMERIC_ULONG;
	     i++)
	{
	    if ( (*old_ul) != (*new_ul) )
	    {
		do_rcrc = 1;
		break;
	    }
	    old_ul++;
	    new_ul++;
	}
    }
    if (!do_rcrc)
    {
	for (i = 0, old_flag = &old_udb -> ck1, new_flag = &pudb -> ck1;
	     i < UDB_NUM_CHKBXS;
	     i++)
	{
	    if ( (*old_flag) != (*new_flag) )
	    {
		do_rcrc = 1;
		break;
	    }
	    old_flag++;
	    new_flag++;
	}
    }
    if (do_rcrc)
    {
	pudb -> rcrc = DBC_CRCRecord ((char *)pudb, type);
    }
    
    redQInit ((struct dbCommon *)pudb, redbuf, sizeof (redbuf), type);

    if (old_udb -> stat != pudb -> stat)
	redQField (redbuf, mylist [STAT]);

    if (old_udb -> sevr != pudb -> sevr)
	redQField (redbuf, mylist [SEVR]);

    if (old_udb -> ini2 != pudb -> ini2)
	redQField (redbuf, mylist [INI2]);

    if (old_udb -> val != pudb -> val)
	redQField (redbuf, mylist [VAL]);

    if (old_udb -> lval != pudb -> lval)
	redQField (redbuf, mylist [LVAL]);

    if (old_udb -> mlst != pudb -> mlst)
	redQField (redbuf, mylist [MLST]);

    /* if the user has told use to post an input value
       then do so
    */
    if (pudb -> inpf)
    {
	inpv = &pudb -> inv1;
	for (i = 0; i < UDB_NUM_INPUTS; i++)
	{
	    if (pudb -> inpf & (1 << i))
		db_post_events ((struct dbCommon *)pudb, (caddr_t)inpv, DBE_VALUE);
	    inpv++;
	}
    }

    /* if the user has told use to post an output value
       then do so
    */
    if (pudb -> outf)
    {
	outv = &pudb -> ovl1;
	for (i = 0; i < UDB_NUM_OUTPUTS; i++)
	{
	    if (pudb -> outf & (1 << i))
		db_post_events ((struct dbCommon *)pudb, (caddr_t)outv, DBE_VALUE);
	    outv++;
	}
    }

    if (old_udb -> mode != pudb -> mode)
    {
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> mode, DBE_VALUE);
	redQField (redbuf, mylist [MODE]);
    }

    if (old_udb -> lcmd != pudb -> lcmd)
    {
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> lcmd, DBE_VALUE);
	redQField (redbuf, mylist [LCMD]);
    }

    if (old_udb -> func != pudb -> func)
    {
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> func, DBE_VALUE);
	redQField (redbuf, mylist [FUNC]);
    }

    if (old_udb -> stts != pudb -> stts)
    {
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> stts, DBE_VALUE);
	redQField (redbuf, mylist [STTS]);
    }

    /* do the check boxes */
    for (i = 0, old_flag = &old_udb -> ck1, new_flag = &pudb -> ck1;
	 i < UDB_NUM_CHKBXS;
	 i++)
    {
	if ( (*old_flag) != (*new_flag) )
	{
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_flag, DBE_VALUE);
	    redQField (redbuf, mylist [CK1 + i]);
	}
	old_flag++;
	new_flag++;
    }

    /* do the numeric floats, only post and redund if told 
       to do so by user
    */
    for (i = 0, old_float = &old_udb -> n01, new_float = &pudb -> n01;
	 i < UDB_NUM_NUMERIC_FLOAT;
	 i++)
    {
	if (pudb -> npfl & (1 << i) )
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_float, DBE_VALUE);
	if (pudb -> nrfl & ( 1 << i) )
	    redQField (redbuf, mylist [N01 + i]);
	old_float++;
	new_float++;
    }
 
    /* do the numeric doubles, only post and redund if told
       to do so by user
    */
    for (i = 0, old_dbl = &old_udb -> d01, new_dbl = &pudb -> d01;
	 i < UDB_NUM_NUMERIC_DOUBLE;
	 i++)
    {
	if (pudb -> npfl & (1 << (UDB_NUM_NUMERIC_FLOAT + i) ) )
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_dbl, DBE_VALUE);
	if (pudb -> nrfl & (1 << (UDB_NUM_NUMERIC_FLOAT + i) ) )
	    redQField (redbuf, mylist [D01 + i]);
	old_dbl++;
	new_dbl++;
    }

    /* do the numeric unsigned longs, only post and redund 
       if told to do so by user
    */
    for (i = 0, old_ul = &old_udb -> u01, new_ul = &pudb -> u01;
	 i < UDB_NUM_NUMERIC_ULONG;
	 i++)
    {
	if (pudb -> npfl & 
		(1 << (UDB_NUM_NUMERIC_FLOAT + UDB_NUM_NUMERIC_DOUBLE + i) ) )
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_ul, DBE_VALUE);
	if (pudb -> nrfl & 
		(1 << (UDB_NUM_NUMERIC_FLOAT + UDB_NUM_NUMERIC_DOUBLE + i) ) )
	    redQField (redbuf, mylist [U01 + i]);
	old_ul++;
	new_ul++;
    }

    if (old_udb -> ctck != pudb -> ctck)
	redQField (redbuf, mylist [CTCK]);

    /* do the timers */
    for (i = 0, old_timer = &old_udb -> tm01, new_timer = &pudb -> tm01;
	 i < UDB_NUM_TIMERS;
	 i++)
    {
	if ( (*old_timer) != (*new_timer) )
	    redQField (redbuf, mylist [TM01 + i]);
	old_timer++;
	new_timer++;
    }

    /* do the timer flags */
    for (i = 0, old_flag = &old_udb -> tf01, new_flag = &pudb -> tf01;
	 i < UDB_NUM_TIMERS;
	 i++)
    {
	if ( (*old_flag) != (*new_flag) )
	    redQField (redbuf, mylist [TF01 + i]);
	old_flag++;
	new_flag++;
    }

    /* do the temp floats */
    for (i = 0, old_float = &old_udb -> fp01, new_float = &pudb -> fp01;
	 i < UDB_NUM_TEMP_FLOAT;
	 i++)
    {
	/* have we been told to post this change */
	if (pudb -> tmfl & (1 << i))
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_float, DBE_VALUE);
	/* if trending is enabled, we can only trend the 1st 8 floats and
	   only if we have been told to do so
	*/
	if ( (i < 8) && (pudb -> tenb) && (pudb -> trfl & (1 << i) ) )
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_float, DBE_LOG);
	if ( (*old_float) != (*new_float) )
	    redQField (redbuf, mylist [FP01 + i]);
	old_float++;
	new_float++;
    }

    /* do the temp unsigned long */
    for (i = 0, old_ul = &old_udb -> lu01, new_ul = &pudb -> lu01;
	 i < UDB_NUM_TEMP_ULONG;
	 i++)
    {
	/* have we been told to post this change */
	if (pudb -> tmfl & (1 << (UDB_NUM_TEMP_FLOAT + i)))
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_ul, DBE_VALUE);
	if ( (*old_ul) != (*new_ul) )
	    redQField (redbuf, mylist [LU01 + i]);
	old_ul++;
	new_ul++;
    }

    /* do the temp doubles */
    for (i = 0, old_dbl = &old_udb -> db01, new_dbl = &pudb -> db01;
	 i < UDB_NUM_TEMP_DOUBLE;
	 i++)
    {
	/* have we been told to post this change */
	if (pudb -> tmfl & (1 << (UDB_NUM_TEMP_FLOAT + UDB_NUM_TEMP_ULONG + i)))
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_dbl, DBE_VALUE);
	if ( (*old_dbl) != (*new_dbl) )
	    redQField (redbuf, mylist [DB01 + i]);
	old_dbl++;
	new_dbl++;
    }

    /* do the boolean flags */
    for (i = 0, old_flag = &old_udb -> bl01, new_flag = &pudb -> bl01;
	 i < UDB_NUM_FLAGS;
	 i++)
    {
	if ( (*old_flag) != (*new_flag) )
	{
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_flag, DBE_VALUE);
	    redQField (redbuf, mylist [BL01 + i]);
	}
	/* we can only trend the 1st 8 bool flags and only if we have
	   been told to, whether flag has chg'd or not
	*/
	if ( ( i < 8) && (pudb -> tenb) && (pudb -> trbl & (1 << i)) )
	    db_post_events ((struct dbCommon *)pudb, (caddr_t)new_flag, DBE_LOG);
	old_flag++;
	new_flag++;
    }

    /* have we been told to post this field */
    if (pudb -> tmfl & (1 << 
		(UDB_NUM_TEMP_FLOAT + UDB_NUM_TEMP_ULONG + UDB_NUM_TEMP_DOUBLE)))
	db_post_events ((struct dbCommon *)pudb, pudb -> msgf, DBE_VALUE);

    if ( (strncmp (old_udb -> msgf, pudb -> msgf, sizeof (pudb -> msgf))) != 0)
	redQField (redbuf, mylist [MSGF]);

    if (old_udb -> lalm != pudb -> lalm)
	redQField (redbuf, mylist [LALM]);

    if (old_udb -> ncmd != pudb -> ncmd)
    {
	monitor_mask = DBE_VALUE;
	/* if trending is enabled and we have been told to trend NCMD */
	if (pudb -> tenb && pudb -> trnc)
	    monitor_mask |= DBE_LOG;
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> ncmd, monitor_mask);
	redQField (redbuf, mylist [NCMD]);
    }

    if (old_udb -> zcmf != pudb -> zcmf)
	redQField (redbuf, mylist [ZCMF]);

    if (old_udb -> fpas != pudb -> fpas)
	redQField (redbuf, mylist [FPAS]);

    if (old_udb -> hhll != pudb -> hhll)
	redQField (redbuf, mylist [HHLL]);

    if (old_udb -> rtct != pudb -> rtct)
	redQField (redbuf, mylist [RTCT]);

    /* post date fields */

    if (old_udb -> year != pudb -> year)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> year, DBE_VALUE);

    if (old_udb -> leap != pudb -> leap)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> leap, DBE_VALUE);

    if (old_udb -> doy != pudb -> doy)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> doy, DBE_VALUE);

    if (old_udb -> mon != pudb -> mon)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> mon, DBE_VALUE);

    if (old_udb -> dom != pudb -> dom)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> dom, DBE_VALUE);

    if (old_udb -> dow != pudb -> dow)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> dow, DBE_VALUE);

    if (old_udb -> hod != pudb -> hod)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> hod, DBE_VALUE);

    if (old_udb -> min != pudb -> min)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> min, DBE_VALUE);

    if (old_udb -> sec != pudb -> sec)
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> sec, DBE_VALUE);

    redQSend (redbuf);

    return;
}


/****************************************************************************
 *
 *  MODULE NAME            : process 
 *  AUTHOR                 : emsteel
 *  DATE                   : 16 Jan 95
 *
 *  PARAMETERS             : paddr - ptr to record struct packet            
 * 
 *  DESCRIPTION: called when time to process a udb record
 *
 ***************************** REVISION HISTORY *****************************
 *
 *  REVISED BY             : emsteel
 *  REVISION DATE          : 19 Jan 96
 *  REVISION:	give back semmutex before printf's etc
 *
****************************************************************************/

static long process (paddr)
    struct dbAddr *paddr;
{
    struct udbRecord	*pudb = (struct udbRecord *) (paddr -> precord);
    struct udbRecord	old_udb;
    long		status = 0;                /* return value */
    long		rtnval;
    int			found_task;
    char		func_name[32];
    char		udb_task_name[32];
    char		errmsg[128];
    unsigned long	save_ctck;
    SYM_TYPE		symType;
    UDBPTR		udb_func;
    struct udb_task	*curr_udb_task;

      /* do nothing if we did not init */

    if ((!pudb -> init) || (!pudb -> ini2))
	return (status);

    /* set these flags to 0 (zero) to test if user wants
       us to do anything
    */
    pudb -> inpf = 0;
    pudb -> outf = 0;
    pudb -> tmfl = 0;
    pudb -> npfl = 0;
    pudb -> nrfl = 0;
    pudb -> trfl = 0;
    pudb -> trbl = 0;
    pudb -> trnc = 0;
    pudb -> zcmf = 0;

    /* save the current record to test later for posting events
       end redundancy update
    */
    old_udb = *pudb;

    /* get time stamp for last time record was processed */

    tsLocalTime (&pudb -> lptm);

    /* get current clock ticks, do it here
       so that if we are off scan etc, we can
       maintain the tick count so that process
       is bumpless, also calc the delta ticks
    */
    save_ctck = pudb -> ctck;
    pudb -> ctck = tickGet ();
    pudb -> dctk = pudb -> ctck - save_ctck;

    pudb -> msev = NO_ALARM;

    /* off normal operations */

    if (pudb -> tagd)
    {
	pudb -> nsta = TAGGED_ALARM;
	pudb -> nsev = VALID_ALARM;
    }
    else if (pudb -> oscn == OFF_SCAN)
    {
	pudb -> nsta = SCAN_ALARM;
	pudb -> nsev = VALID_ALARM;
    }
    else
    {
	/* set to this as a test if user set his own alarm */
	pudb -> nsta = 0xff;
	pudb -> nsev = 0xff;

	/* update number of process times */

	if (process_stats)
	    process_cnts[type]++;

         /* set udb to active */
   
	pudb -> pact = TRUE;

	/* find the udb's process function */

	sprintf (func_name, "_UDB_%s_process", pudb -> dnam);
	rtnval = symFindByName (sysSymTbl, func_name, (char **)&udb_func, &symType);
	if ( (rtnval != OK) || ( (symType & N_TEXT) == 0) )
	{
	    pudb -> init = 0;
	    pudb -> ini2 = 0;
	    recGblRecordError (S_udb_NoProcess, (struct dbCommon *)pudb, "udb:process");
	    return (S_udb_NoProcess);
	}
	sprintf (udb_task_name, "udb_%s_task", pudb -> dnam);

	/* take this semaphore to protect this area */
	semTake (semmutex, WAIT_FOREVER);

	curr_udb_task = first_udb_task;
	found_task = 0;
	if (first_udb_task)
	{
	    while(1)
	    {
		if ((struct udb_task *)pudb -> udbt == curr_udb_task)
		{
		    found_task = 1;
		    break;
		}
		curr_udb_task = curr_udb_task -> next_udb_task;
		if (curr_udb_task)
		    continue;
		break;
	    }
	}
	if (found_task)
	{
	    if (taskIsSuspended (curr_udb_task -> udb_task_id))
	    {
		delete_udb_task (curr_udb_task, 1);
		semGive (semmutex);
		pudb -> init = 0;
		pudb -> ini2 = 0;
		sprintf (errmsg, "udb:process, task %s, %s suspended, block will reinit\n",
					udb_task_name, pudb -> dnam);
		errMessage (-1L, errmsg);
		return (status);
	    }
	    else if (taskIdVerify (curr_udb_task -> udb_task_id) == ERROR)
	    {
		delete_udb_task(curr_udb_task, 0);
		semGive (semmutex);
		pudb -> init = 0;
		pudb -> ini2 = 0;
		sprintf (errmsg, "udb:process, task %s, no longer exists, block will reinit\n",
				  udb_task_name);
		errMessage (-1L, errmsg);
		return (status);
	    }
	    /* task is ok */
	    else
	    {
		/* the symbol table pointer does not agree with the address of the
	   	   users process function recorded in init_record, the user has
	   	   probably reloaded his code and there is a new symbol table
	   	   entry, force the block to reinit
		*/
		if (pudb -> lppt != ((void*)udb_func))
		{
		    semGive (semmutex);
		    sprintf (errmsg, "udb:process, %s reloaded, block will reinit\n",
					udb_task_name);
		    errMessage (-1L, errmsg);
		    pudb -> init = 0;
		    pudb -> ini2 = 0;
		    return (status);
		}
		else
		{
		    /* update any timers the user may be using */
		    update_timers (pudb);

		    curr_udb_task -> udb_process_func = udb_func;
		    curr_udb_task -> pudb = pudb;

		    /* active the task */
		    semGive (curr_udb_task -> sem_activate);

		    /* wait 5 ticks for task to return */
		    if ( (semTake (curr_udb_task -> sem_done, 5)) == ERROR)
		    {
			pudb -> nsta = CALC_ALARM;
			pudb -> nsev = VALID_ALARM;
		    }
		    else
		    {
			/* update VAL with current status */
			pudb -> val = pudb -> stts;

			if ((status = send_outputs (pudb, &old_udb)) != 0)
			{
			    ;	/* ?????????? */
			}

		    }
		}
	    }
	}
	/* did not find ptr to udb_task block */
	else
	{
	    semGive (semmutex);
	    pudb -> ini2 = 0;
	    pudb -> init = 0;
	    sprintf (errmsg, "udb:process, task %s not found, block will reinit\n",
			     udb_task_name);
	    errMessage(-1L, errmsg);
	    return (status);
	}

	/* give back the mutual exclusion semaphore */
	semGive (semmutex);


	/* if the user did not set his own alarm, then set these to
	   NO_ALARM so alarm() will work properly
	*/
	if ((pudb -> nsta == 0xff) || (pudb -> nsev == 0xff))
	{
	    pudb -> nsta = NO_ALARM;
	    pudb -> nsev = NO_ALARM;
	}
    
	alarm (pudb);

	/* check for maximize severity */

	if (pudb -> msev > pudb -> nsev)
	{
	    pudb -> nsev = pudb -> msev;
	    pudb -> nsta = LINK_ALARM;
	}

    } /* end else (oscn != OFF_SCAN) */

    /* if the time stamp is 0, init and post val with
       alarm bit set, this makes sure that proper alarm
       messages are set if IOC was down and alarm status
       may have chg'd
    */
    if (pudb -> latm.secPastEpoch == 0)
    {
	tsLocalTime (&pudb -> latm);
	db_post_events ((struct dbCommon *)pudb, (caddr_t)&pudb -> val, DBE_VALUE | DBE_ALARM);
    }

    /* if it is the 1st process pass 
       init last value when alarm chg'd (lalm)
    */

    if (pudb -> fpps)
    {
	pudb -> fpps = 0;
	pudb -> lalm = pudb -> val;
    }

    /* check event list */

    monitor (pudb);

    /* poost events and update redundancy */

    post_and_redund (pudb, &old_udb);

    /* the process is no longer active */

    pudb -> pact = FALSE;

    /* process the forward scan link record */

    if (pudb -> flnk.type == DB_LINK)
	dbScanPassive ((struct dbAddr *)pudb -> flnk.value.db_link.pdbAddr);

    return (status);

} /* end func process () */



/****************************************************************************
 *
 *  MODULE NAME            : put_enum_str
 *  AUTHOR                 : emsteel
 *  DATE                   : 15 Dec 94
 *
 *
 *  DESCRIPTION: modify enumeration string
 *
****************************************************************************/

static long put_enum_str (paddr, pstring)
    struct dbAddr *paddr;
    char          *pstring;
{
    struct udbRecord	*pudb=(struct udbRecord *)paddr->precord;
    char		*psource = NULL;
    short		i;
    unsigned short	*field = (unsigned short *)paddr -> pfield;
    int			num_bytes = 0;
    int			num_strs = 0;
    int			bad_value = 0;
    int			start = 0;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset)
    {
	case ( (int)&(((struct udbRecord *)0) -> mode)):
	    psource = pudb -> mst1;
	    num_bytes = sizeof (pudb -> mst1);
	    num_strs = UDB_NUM_MODES;
	    start = 0;
	    break;

	/* IMDE has value 0 - UDB_NUM_MODES, 0 is special case of LAST MODE
	   1 - UDB_NUM_MODES are user defined
	*/
	case ( (int)&(((struct udbRecord *)0) -> imde)):
	    if (strncmp (pstring, UDB_LAST_MODE, sizeof (pudb -> mst1)) == 0)
	    {
		*field = 0;
		return (0);
	    }
	    psource = pudb -> mst1;
	    num_bytes = sizeof (pudb -> mst1);
	    num_strs = UDB_NUM_MODES + 1;
	    start = 1;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> ncmd)):
	case ( (int)&(((struct udbRecord *)0) -> lcmd)):
	    psource = pudb -> cst1;
	    num_bytes = sizeof (pudb -> cst1);
	    num_strs = UDB_NUM_CMDS;
	    start = 0;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> func)):
	    psource = pudb -> fst1;
	    num_bytes = sizeof (pudb -> fst1);
	    num_strs = UDB_NUM_FUNCS;
	    start = 0;
	    break;

	/* IFUN has value 0 - UDB_NUM_FUNCS, 0 is LAST FUNC 1 - UDB_NUM_FUNCS
	   are the function strings
	*/
	case ( (int)&(((struct udbRecord *)0) -> ifun)):
	    if (strncmp (pstring, UDB_LAST_FUNC, sizeof (pudb -> fst1)) == 0)
	    {
		*field = 0;
		return (0);
	    }
	    psource = pudb -> fst1;
	    num_bytes = sizeof (pudb -> fst1);
	    num_strs = UDB_NUM_FUNCS + 1;
	    start = 1;
	    break;

	case ( (int)&(((struct udbRecord *)0) -> stts)):
	case ( (int)&(((struct udbRecord *)0) -> val)):
	    psource = pudb -> ss01;
	    num_bytes = sizeof (pudb -> ss01);
	    num_strs = UDB_NUM_STATUS;
	    start = 0;
	    break;

	default:
	    bad_value++;
	    break;
    }

    if (bad_value)
	return (S_db_badChoice);

    for (i = start; i < num_strs; i++, psource += num_bytes)
    {
	if (strncmp (psource, pstring, num_bytes) == 0)
	{
	    *field = (unsigned short)i;
	    return (0);
	}
    }

    return(S_db_badChoice);

} /* end func put_enum_str () */


/****************************************************************************
 *
 *  MODULE NAME            : send_outputs
 *  AUTHOR                 : emsteel
 *  DATE                   : 18 Jan 95
 *
 *
 *  DESCRIPTION: send output values if user told us to do so
 *		 	pudb -> outf is a bitmap, bits are set if output
 *		 	is to take place, user can set outf to 0xffff if 
 *			they want all output values to be sent
 *		 if outf is not set then only send those outputs whose value
 *			has changed
 *
****************************************************************************/

static long send_outputs (pudb, old_udb)
    struct udbRecord	*pudb;
    struct udbRecord	*old_udb;
{
    struct link		*outPV;
    double		*outv;
    double		*old_outv;
    long		status = 0;
    long		nRequests = 1;
    int			i;

    if (pudb -> outf)
    {
	for (i = 0, outPV = &pudb -> out1, outv = &pudb -> ovl1;
	     i < UDB_NUM_OUTPUTS;
	     i++)
	{
	    if (outPV -> type == DB_LINK)
	    {
		if (pudb -> outf & (1 << i))
		{
		    if ( (status = dbPutLink (&outPV -> value.db_link,
					      (struct dbCommon *)pudb, DBR_DOUBLE,
					      (caddr_t)outv, nRequests) ) != NULL)
			return (status);
		}
	    }
	    outPV++;
	    outv++;
	}
    }
    else
    {
	for (i = 0, outPV = &pudb -> out1, outv = &pudb -> ovl1, 
		    old_outv = &old_udb -> ovl1;
	     i < UDB_NUM_OUTPUTS;
	     i++)
	{
	    if (outPV -> type == DB_LINK)
	    {
		if ( (*old_outv) != (*outv) )
		{
		    if ( (status = dbPutLink (&outPV -> value.db_link,
					      (struct dbCommon *)pudb, DBR_DOUBLE,
					      (caddr_t)outv, nRequests) ) != NULL)
			return (status);
		}
	    }
	    outPV++;
	    outv++;
	    old_outv++;
	}
    }

    return (status);
}



/****************************************************************************
 *
 *  MODULE NAME            : special
 *  AUTHOR                 : emsteel
 *  DATE                   : 18 Jan 95
 *
 *
 *  DESCRIPTION: handle specials attached to fields
 *		 a) SPC_UDB_CTCK, will be called when the slave is updated
 *		    with the value from the master, however, since the 
 *		    clock tick count on the 2 systems are different, have the
 *		    slave get its own ctck, and compute delta tick just in
 *		    case we need it
 *		 b) SPC_UDB_ZCMF, will be called when slave field ZCMF is
 *		    updated fro the master, this lets slave know to zero
 *		    the manual and automaic command fields
 *
****************************************************************************/

static long special (paddr, after)
     struct dbAddr	*paddr;
     int		after;
{
    struct udbRecord	*pudb = (struct udbRecord *)(paddr -> precord);
    int			special_type = paddr -> special;
    unsigned long	ctck = pudb -> ctck;
    long		status = 0;
    short		*cm_rec;		/* manual cmd ptr */
    short		*ca_rec;		/* automatic cmd ptr */
    int			i;

    if (!after)
	return (status);

    switch (special_type)
    {
	case SPC_UDB_CTCK:
	    pudb -> ctck = tickGet();
	    pudb -> dctk = pudb -> ctck - ctck;
	    break;

	case SPC_UDB_ZCMF:
	    for (i = 0,
		 cm_rec = &pudb -> cm1m,
		 ca_rec = &pudb -> cm1a;
		 i < UDB_NUM_CMDS;
		 i++)
	    {
		*cm_rec = 0;
		*ca_rec = 0;
		cm_rec++;
		ca_rec++;
	    }
	    pudb -> zcmf = 0;
	    break;

	default:
	    recGblDbaddrError (S_db_badChoice, paddr, "udb:special");
	    status = S_db_badChoice;
	    break;
    }

    return (status);
}


/****************************************************************************
 *
 *  MODULE NAME            : update_timers
 *  AUTHOR                 : emsteel
 *  DATE                   : 18 Jan 95
 *
 *
 *  DESCRIPTION: update any timers that the user is currently using
 *		 only if timer is active, timer may also be not active
 *			or in a hold state
 *
****************************************************************************/

static void update_timers (pudb)
    struct udbRecord	*pudb;
{
    unsigned long	*timer = &pudb -> tm01;
    unsigned short	*flag = &pudb -> tf01;
    int			i;

    for (i = 0; i < UDB_NUM_TIMERS; i++)
    {
	if ( (*flag) == UDB_TIMER_ACTIVE)
	    *timer += pudb -> dctk;
	flag++;
	timer++;
    }

    return;
}


/****************************************************************************
 *
 *  MODULE NAME            : udb_process_task
 *  AUTHOR                 : emsteel
 *  DATE                   : 07 Mar 95
 *
 *
 *  DESCRIPTION: intermediate task between the scan task and the user code
 *		 if the user code bombs, it will take out this task and not
 *		 the scan task
 *
****************************************************************************/

void udb_process_task (udb_task)
    struct udb_task	*udb_task;
{
    struct udb_task	*curr_udb_task = udb_task;
    WDOG_ID		wdId;
    char		errmsg[128];
    int			taskId;

    if ( (wdId = wdCreate()) == NULL)
    {
	sprintf (errmsg, "udb task %s wdCreate failed, will exit\n",
			  curr_udb_task -> udb_task_name);
	errMessage (-1L, errmsg);
	taskSuspend (taskIdSelf());
    }

    /* get my own task id */
    taskId = taskIdSelf ();

    for (;;)
    {
	semTake (curr_udb_task -> sem_activate, WAIT_FOREVER);

	/* start a watch dog timer to expire in 4 ticks */
	wdStart (wdId, 4, (FUNCPTR)udb_watchdog, taskId);

	curr_udb_task -> udb_process_func ((void*)curr_udb_task -> pudb);

	/* we are ok so cancel the watchdog timer */
	wdCancel (wdId);

	semGive (curr_udb_task -> sem_done);
    }
}



/****************************************************************************
 *
 *  MODULE NAME            : udb_watchdog
 *  AUTHOR                 : emsteel
 *  DATE                   : 04 Dec 95
 *
 *
 *  DESCRIPTION: called by the watchdog timer if udb_process_task times out
 *		 suspend the udb task, this will cause the udb records to
 *		 eventually reinit
 *
****************************************************************************/

void udb_watchdog (taskId)
    int		taskId;
{
    taskSuspend (taskId);	/* suspends the task */
    return;
}

