/*recWaveform.c,v 1.1.1.1 1999/08/03 14:44:23 tis4000 Exp */

/* recWaveform.c - Record Support Routines for Waveform records */
/*
 *      Original Author: Bob Dalesio
 *      Current Author:  Marty Kraimer
 *      Date:            7-14-89
 *
 *      Experimental Physics and Industrial Control System (EPICS)
 *
 *      Copyright 1991, the Regents of the University of California,
 *      and the University of Chicago Board of Governors.
 *
 *      This software was produced under  U.S. Government contracts:
 *      (W-7405-ENG-36) at the Los Alamos National Laboratory,
 *      and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *      Initial development by:
 *              The Controls and Automation Group (AT-8)
 *              Ground Test Accelerator
 *              Accelerator Technology Division
 *              Los Alamos National Laboratory
 *
 *      Co-developed with
 *              The Controls and Computing Group
 *              Accelerator Systems Division
 *              Advanced Photon Source
 *              Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 * .01  11-27-89        lrd     using one more buffer entry than allocated
 * .02  11-27-89        lrd     throttle on monitor requests
 * .03  02-27-90        lrd     handle character value links for Joerger
 *                              digitizer (convert to short array for access
 * .04  03-05-90        lrd     add averaging of entire waveforms
 * .05  04-02-90        ba/lrd  fix the joerger processing and
 *                              add get_a_byte macro
 *                              fix the determination of buff_size
 * .06  04-11-90        lrd     make locals static
 * .07  05-02-90        lrd     fix mdct so that it would remain 0 on the
 *                              pass where the monitors are sent
 * .08  05-08-90        ba      mdct is never equal to mcnt during record pro-
 *                              cessing, causing some code never to run. Fix
 *                              is to check (mdct==mcnt || mdct==0) to indicate
 *                              first time through an averaging loop.
 * .09  07-26-90        lrd     fixed the N-to-1 character waveformion
 *                              value was not initialized
 * .10  10-11-90	mrk	Made changes for new record support
 * .11  03-01-94	ems	added support for process stats
 *				removed ack = 0
 * .12  03-09-94	ems	added scan_types.h to define E_IO_INTERRUPT
 * .13	03-29-94	ems	make forward func def agree with actual func def
 *				in init_record, cast nelm as long to test for < 0
 *				since it is defined as ulong, also cast ftvl as
 *				unsigned to check against DBF_ENUM, and delete test
 *				for < 0 since it is defined as enum
 * .14	05-17-94	ems	rewrite init_record
 * .15  06-09-94	ems	cvt_dbaddr, check if return from init_record is
 *				S_rec_Alloc, means calloc fro bptr failed
 * .16	06-10-94	ems	mods to process()
 *					check for tagd, off scan
 *					add call to tsLocalTime
 *					removed call to read_wf for E_IO_INTERRUPT
 * .17	06-14-94	ems	added support for maximize severity (msev)
 * .18	08-31-94	ems	rm DBE_LOG from ack and bptr, bptr is array and
 *				we dont trend arrays
 *				added nelm and nord to get_graphic_double()
 *				added dbRecDes.h for def of struct fldDes
 * .19	10-14-94	ems	added redundancy, do time stamp when stat or sevr chg,
 *				see monitor(),and not in process()
 * .20	10-20-94	ems	chg time to latm (last alarm time) added 
 *				added lptm (last process time)
 * .21  01-13-95        wep     init lptm and latm
 * .22	02-14-95	ems	only post bptr if on scan (see monitor())
 * .23	03-15-95	ems	monitor(), do not set ACK if sevr is EVENT_ALARM
 * .24	09-12-95	ems	monitor(), see mod for handling VALID_ALARM
 */

#include	<vxWorks.h>
#include	<inetLib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<string.h>
#include	<stdlib.h>

#include	"alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"dbRecDes.h"
#include	"devSup.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"rec/waveformRecord.h"
#include	"scan_types.h"
#include	"restable.h"
#include	"wfredfldidx.h"
#include	"redflds.h"
#include	"redque.h"
#include	"db_addr.h"
#include	"recGbl.h"
#include	"dbEvent.h"

/* defined as global in .../share/db/iocInit.c */
extern long *process_cnts;
/* defined as global in .../share/src/procstat.c */
extern int process_stats;

/* Create RSET - Record Support Entry Table*/
#define report NULL
#define initialize NULL
static long init_record();
static long process();
#define special NULL
static long get_value();
static long cvt_dbaddr();
static long get_array_info();
static long put_array_info();
static long get_units();
static long get_precision();
#define get_enum_str NULL
#define get_enum_strs NULL
#define put_enum_str NULL
static long get_graphic_double();
static long get_control_double();
#define get_alarm_double NULL

struct rset waveformRSET={
	RSETNUMBER,
	report,
	initialize,
	init_record,
	process,
	special,
	get_value,
	cvt_dbaddr,
	get_array_info,
	put_array_info,
	get_units,
	get_precision,
	get_enum_str,
	get_enum_strs,
	put_enum_str,
	get_graphic_double,
	get_control_double,
	get_alarm_double };

struct wfdset { /* waveform dset */
        long            number;
        DEVSUPFUN       dev_report;
        DEVSUPFUN       init;
        DEVSUPFUN       init_record; /*returns: (-1,0)=>(failure,success)*/
        DEVSUPFUN       get_ioint_info;
        DEVSUPFUN       read_wf;/*(0,1)=> success and */
                        /*(convert,don't continue)*/
};

/* The follwoing definitions are required for redundancy */

static struct fldDes	**mylist = 0;

static char *namelist [] = {
	"STAT",
	"SEVR",
	"INI2",
	"LATM",
	0
	};

static unsigned short type;

/* sizes of field types as defined in dbFldTypes.h
   NOTE: this must agree with same in devCom/devSoft.c
*/
static int sizeofTypes[] = {0,1,1,2,2,4,4,4,8,2};

static void monitor();

/*Following from timing system          */
extern unsigned int     gts_trigger_counter;


/*------------------------------------------------------------------------*/

static long init_record(pwf)
    struct waveformRecord	*pwf;
{
    struct wfdset	*pdset;
    unsigned short	i;
    long	  	status;
    char	  	*pstr;
    int		  	ini2_flag = 0;
    char		redbuf [128];


    if (!mylist)
    {
	for (type = 1; type < 100; type++)
	{
	    pstr = GET_PRECTYPE (type);
	    if (strncmp (pstr, "waveform", 8) == 0)
		break;
	}
	mylist = redFldAlloc (type, NUMWFFLDS);

	if (!mylist)
	{
	    recGblRecordError (S_rec_Alloc, (struct dbCommon *)pwf,
				"wf, init_record, mylist failed");
	    return (S_rec_Alloc);
	}

	for (i = 0; i < NUMWFFLDS; i++)
	    redFldAdd (type, i, namelist [i]);
    }

    /* This routine may get called twice. Once by cvt_dbaddr. Once by iocInit*/
    if(pwf->bptr==NULL) 
    {
	if(((long)pwf->nelm)<=0) pwf->nelm=1;
	if(pwf->ftvl == 0) 
	{
	    pwf->bptr = (char *)calloc(pwf->nelm,MAX_STRING_SIZE);
	    if (!pwf -> bptr)
	    {
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pwf,
				   "wf, init_record, bptr1 failed");
		return (S_rec_Alloc);
	    }
	} 
	else 
	{
	    if( ((unsigned)pwf->ftvl) > DBF_ENUM) 
		pwf->ftvl=2;
	    pwf->bptr = (char *)calloc(pwf->nelm,sizeofTypes[pwf->ftvl]);
	    if (!pwf -> bptr)
	    {
		recGblRecordError (S_rec_Alloc, (struct dbCommon *)pwf,
				   "wf, init_record, bptr2 failed");
		return (S_rec_Alloc);
	    }
	}
    }

    pwf->nord = pwf -> nelm;

    /* must have read_wf function defined */
    if(!(pdset = (struct wfdset *)(pwf->dset))) 
    {
	recGblRecordError(S_dev_noDSET, (struct dbCommon *)pwf, "wf: init_record");
	return(S_dev_noDSET);
    }
    if( (pdset->number < 5) || (pdset->read_wf == NULL) ) 
    {
	recGblRecordError(S_dev_missingSup, (struct dbCommon *)pwf, "wf: init_record");
	return(S_dev_missingSup);
    }

    pwf->init = TRUE;
    pwf -> nsta = NO_ALARM;
    pwf -> nsev = NO_ALARM;
    pwf -> msev = NO_ALARM;

    /* init last process time and last alarm time */
    if (pwf -> lptm.secPastEpoch == 0)
        tsLocalTime (&pwf -> lptm);
    if (pwf -> latm.secPastEpoch == 0)
        tsLocalTime (&pwf -> latm);


    if( pdset->init_record ) 
    {
	status = (*pdset->init_record)(pwf,process);
	if (status)
	{
	    pwf -> nsta = NO_INIT_ALARM;
	    pwf -> nsev = VALID_ALARM;
	}
	else
	    ini2_flag = 1;
    }
    else
	ini2_flag = 1;

    if (resGetCaAnswer (pwf -> radr, &i) == RES_OK)
    {
	if (i)
	{
	    /* post STAT, SEVR and BPTR (monitor), if we will finally init, 
	       because we have read a value from the input device, set INI2,
	    */

	    /* check for maximize severity */

	    if (pwf -> msev > pwf -> nsev)
	    {
		pwf -> nsev = pwf -> msev;
		pwf -> nsta = LINK_ALARM;
	    }

	    monitor (pwf);

	    if (ini2_flag)
	    {
		pwf -> ini2 = 1;
		redQInit ((struct dbCommon *)pwf, redbuf, sizeof (redbuf), type);
		redQField (redbuf, mylist [STAT]);
		redQField (redbuf, mylist [SEVR]);
		redQField (redbuf, mylist [INI2]);
		redQSend (redbuf);
	    }
	}
    }

       
    return(0);
}


/*------------------------------------------------------------------------*/

static long process(paddr)
    struct dbAddr	*paddr;
{
    struct waveformRecord	*pwf=(struct waveformRecord *)(paddr->precord);
    struct wfdset		*pdset = (struct wfdset *)(pwf->dset);
    long			status = 0;
    unsigned short		stat   = pwf -> stat;
    unsigned short		sevr   = pwf -> sevr;
    TS_STAMP			latm   = pwf -> latm;
    char			redbuf [128];

           /* if not init'd, do nothing */

    if ((!pwf -> init) || (!pwf -> ini2))
	return (0);

    if( (pdset==NULL) || (pdset->read_wf==NULL) ) 
    {
	pwf->pact=TRUE;
	recGblRecordError(S_dev_missingSup, (struct dbCommon *)pwf, "read_wf");
	return(S_dev_missingSup);
    }

        /* event throttling */
    if (pwf->scan == E_IO_INTERRUPT)
    {
	if ( (pwf->evnt != 0)  && (gts_trigger_counter != 0) )
	{
	    if ((gts_trigger_counter % pwf->evnt) != 0)
	    {
		return(status);
	    }
	}
    }

    /* get time stamp for last time record was processed */

    tsLocalTime (&pwf -> lptm);

    /* init new status to previous status */
    pwf -> nsta = pwf -> stat;
    pwf -> nsev = pwf -> sevr;
    pwf -> msev = NO_ALARM;

    if (pwf -> tagd)
    {
	pwf -> nsta = TAGGED_ALARM;
	pwf -> nsev = VALID_ALARM;
    }
    else if (pwf -> oscn == OFF_SCAN)
    {
	pwf -> nsta = SCAN_ALARM;
	pwf -> nsev = VALID_ALARM;
    }
    else
    {
	pwf -> nsta = NO_ALARM;
	pwf -> nsev = NO_ALARM;

    	/* update number of process times */
	if (process_stats)
	    process_cnts[type]++;

        /*pact must not be set true until read_wf completes*/

        status=(*pdset->read_wf)(pwf); /* read the new value */
        pwf->pact = TRUE;

        /* status is one if an asynchronous record is being processed*/

        if(status==1) 
	    return(0);
	pwf->udf=FALSE;

	/* check for maximize severity */

	if (pwf -> msev > pwf -> nsev)
	{
	    pwf -> nsev = pwf -> msev;
	    pwf -> nsta = LINK_ALARM;
	}
    }

    /* if the time stamp is 0, init and post val with
       alarm bit set, this makes sure that proper alarm
       messages are set if IOC was down and alarm status
       may have chg'd
    */
    if (pwf -> latm.secPastEpoch == 0)
    {
	tsLocalTime (&pwf -> latm);
	db_post_events ((struct dbCommon *)pwf, pwf -> bptr, DBE_VALUE | DBE_ALARM);
    }

    if (status != 2)
	monitor(pwf);

    /* do redundant update */

    redQInit ((struct dbCommon *)pwf, redbuf, sizeof (redbuf), type);

    if (stat != pwf -> stat)
	redQField (redbuf, mylist [STAT]);

    if (sevr != pwf -> sevr)
	redQField (redbuf, mylist [SEVR]);

    if (TsCmpStampsNE (&latm, &pwf -> latm) )
	redQField (redbuf, mylist [LATM]);

    redQSend (redbuf);

    /* process the forward scan link record */

    if (pwf->flnk.type==DB_LINK) 
	dbScanPassive((struct dbAddr *)pwf->flnk.value.db_link.pdbAddr);

    pwf->pact=FALSE;
    return(0);
}


/*------------------------------------------------------------------------*/

static long get_value(pwf,pvdes)
    struct waveformRecord    *pwf;
    struct valueDes	*pvdes;
{

    pvdes->no_elements=pwf->nord;
    pvdes->pvalue = pwf->bptr;
    pvdes->field_type = pwf->ftvl;
    return(0);
}

/*------------------------------------------------------------------------*/

static long cvt_dbaddr(paddr)
    struct dbAddr *paddr;
{
    struct waveformRecord *pwf=(struct waveformRecord *)paddr->precord;
    long status = 0;

    /* This may get called before init_record. If so just call it*/

    if(pwf->bptr==NULL) 
    {
	if ( (status = init_record(pwf)) == S_rec_Alloc)
	    return (status);
    }
    paddr->pfield = (caddr_t)(pwf->bptr);
    paddr->no_elements = pwf->nelm;
    paddr->field_type = pwf->ftvl;
    if(pwf->ftvl==0)  
	paddr->field_size = MAX_STRING_SIZE;
    else 
	paddr->field_size = sizeofTypes[pwf->ftvl];
    paddr->dbr_field_type = pwf->ftvl;
    return(0);
}

/*------------------------------------------------------------------------*/

static long get_array_info(paddr,no_elements,offset)
    struct dbAddr *paddr;
    long	  *no_elements;
    long	  *offset;
{
    struct waveformRecord	*pwf=(struct waveformRecord *)paddr->precord;

    *no_elements =  pwf->nord;
    *offset = 0;
    return(0);
}


/*------------------------------------------------------------------------*/

static long put_array_info(paddr,nNew)
    struct dbAddr *paddr;
    long	  nNew;
{
    struct waveformRecord	*pwf=(struct waveformRecord *)paddr->precord;

    pwf->nord = nNew;
    if(pwf->nord > pwf->nelm) 
	pwf->nord = pwf->nelm;
    return(0);
}

/*------------------------------------------------------------------------*/

static long get_units(paddr,units)
    struct dbAddr *paddr;
    char	  *units;
{
    struct waveformRecord	*pwf=(struct waveformRecord *)paddr->precord;

    strncpy(units,pwf->egu,sizeof(pwf->egu));
    return(0);
}

/*------------------------------------------------------------------------*/

static long get_precision(paddr,precision)
    struct dbAddr *paddr;
    long	  *precision;
{
    struct waveformRecord	*pwf=(struct waveformRecord *)paddr->precord;

    *precision = pwf->prec;
    return(0);
}


/*------------------------------------------------------------------------*/

static long get_graphic_double(paddr,pgd)
    struct dbAddr *paddr;
    struct dbr_grDouble *pgd;
{
    struct waveformRecord     *pwf=(struct waveformRecord *)paddr->precord;

    switch ( ((struct fldDes *)paddr -> pfldDes) -> offset )
    {
	case ( (int)&(((struct waveformRecord *)0) -> nelm) ):
	case ( (int)&(((struct waveformRecord *)0) -> nord) ):
	    pgd -> upper_disp_limit = 32767;
	    pgd -> lower_disp_limit = 0;
	    break;

	default:
	    pgd->upper_disp_limit = pwf->hopr;
	    pgd->lower_disp_limit = pwf->lopr;
	    break;
    }

    return(0);
}

/*------------------------------------------------------------------------*/

static long get_control_double(paddr,pcd)
    struct dbAddr *paddr;
    struct dbr_ctrlDouble *pcd;
{
    struct waveformRecord     *pwf=(struct waveformRecord *)paddr->precord;

    pcd->upper_ctrl_limit = pwf->hopr;
    pcd->lower_ctrl_limit = pwf->lopr;
    return(0);
}


/*------------------------------------------------------------------------*/

static void monitor(pwf)
    struct waveformRecord	*pwf;
{
    unsigned short	monitor_mask;
    short		stat,sevr,nsta,nsev;
    int			set_ack = 1;

    /* get previous stat and sevr  and new stat and sevr*/
    stat=pwf->stat;
    sevr=pwf->sevr;
    nsta=pwf->nsta;
    nsev=pwf->nsev;

    /*set current stat and sevr*/
    pwf->stat = nsta;
    pwf->sevr = nsev;
    pwf->nsta = 0;
    pwf->nsev = 0;

    /* if we are coming out of VALID_ALARM and our new stat/sevr are
       the same as when we went into VALID_ALARM, do not set ack
    */
    if ( (nsev != VALID_ALARM) && (sevr == VALID_ALARM) &&
	 (nsta == pwf -> psta) && (nsev == pwf -> psev) )
    {
	set_ack = 0;
    }
    /* if we are going into VALID_ALARM, and out previous stat was not
       VALID_ALARM, remember previous non VALID_ALARM stat/sevr
       and we do not want to set ACK,
    */
    if ( (nsev == VALID_ALARM) && (sevr != VALID_ALARM) )
    {
	set_ack = 0;
	pwf -> psta = stat;
	pwf -> psev = sevr;
	db_post_events ((struct dbCommon *)pwf, (caddr_t)&pwf -> psta, DBE_VALUE);
	db_post_events ((struct dbCommon *)pwf, (caddr_t)&pwf -> psev, DBE_VALUE);
    }

    /* Flags which events to fire on the value field */
    monitor_mask = 0;

    /* alarm condition changed this scan */
    if (stat!=nsta || sevr!=nsev) 
    {
	/* post events for alarm condition change*/
	monitor_mask = DBE_ALARM;

	/* when alarm status chg's, get current time */
	tsLocalTime (&pwf -> latm);

	/* post stat and nsev fields */
	db_post_events((struct dbCommon *)pwf, (caddr_t)&pwf->stat,DBE_VALUE);
	db_post_events((struct dbCommon *)pwf, (caddr_t)&pwf->sevr,DBE_VALUE);

	if ( (pwf -> sevr != NO_ALARM) && (set_ack) &&
	     (pwf -> sevr != EVENT_ALARM) )
	{
	    if (pwf -> ack == 0)
	    {
		pwf -> ack = 1;
		db_post_events ((struct dbCommon *)pwf, (caddr_t)&pwf -> ack, DBE_VALUE);
	    }
	}
    }

    /* only post buffer if we are ON SCAN */
    if ( (!pwf -> tagd) && (pwf -> oscn != OFF_SCAN) )
	monitor_mask |= DBE_VALUE;
    if(monitor_mask) 
	db_post_events((struct dbCommon *)pwf,pwf->bptr,monitor_mask);
    return;
}
