/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*
 *    Modfication log
 * .01	mm-dd-yy	iii	comment
 *
*/


#include	<vxWorks.h>
#include	<inetLib.h>
#include	<stdlib.h>
#include	<types.h>
#include	<stdioLib.h>
#include	<lstLib.h>
#include	<ctype.h>
#include	<logLib.h>
#include	<string.h>

#include	"alarm.h"
#include	"dbDefs.h"
#include	"dbCommon.h"
#include	"link.h"
#include	"db_field_log.h"
#include	"dbAccess.h"
#include	"dbFldTypes.h"
#include	"dbRecType.h"
#include	"errMdef.h"
#include	"recSup.h"
#include	"rec/udbRecord.h"
#include	"udbDefs.h"
#include	"restable.h"
#include	"okToUse.h"

static long UDB_get_inputs (void *, double *, unsigned short *);
static long UDB_get_outputs (void *, double *, unsigned short *);


/****
    transfer data from record to user stack or back depending on value
    in update_type
****/

void UDB_data_transfer (prec, puser, update_type)
    void		*prec;		/* void pointer to udb record */
    struct userudb_data	*puser;
    int			update_type;
{
    struct udbRecord	*pudb = (struct udbRecord *)prec;
    struct tsDetail	tsD;
    double		*dbl_rec;
    double		*dbl_user;
    short		*cm_rec;
    short		*cm_user;
    short		*ca_rec;
    short		*ca_user;
    short		*ffl_rec;
    short		*ffl_user;
    unsigned short	*us_rec;
    unsigned short	*us_user;
    float		*fl_rec;
    float		*fl_user;
    unsigned long	*ul_rec;
    unsigned long	*ul_user;
    int			i;
    long		status = 0;
    short		master = 0;

    /* if copying from user data to record, only do so if our resource
       is MASTER, else we are SLAVE and redundant update from MASTER
       will give us our data
    */
    if (update_type == UDB_COPY_TO_REC)
    {
	if (resGetCaAnswer (pudb -> radr, &master) == RES_OK)
	{
	    if (!master)
		return;
	}
	else
	{
	    return;
	}
    }

    if (update_type == UDB_COPY_TO_LOCAL)
    {
	/* copy the UDB block name */
	strncpy (puser -> udb_name, pudb -> name, sizeof (puser -> udb_name));

	/* copy ini2 flag */
	puser -> ini2 = pudb -> ini2;

	/* copy alarm status */
	puser -> stat = pudb -> stat;
	puser -> sevr = pudb -> sevr;
	puser -> nsta = pudb -> nsta;
	puser -> nsev = pudb -> nsev;
	puser -> tenb  = pudb -> tenb;

	/* get inputs values and their stati */
	status = UDB_get_inputs (prec, &puser -> inv[0], &puser -> iok[0]);
	/* get output values and there stati */
	status = UDB_get_outputs (prec, &puser -> ovl[0], &puser -> ook[0]);

	/* copy the input flag */
	puser -> inpf = pudb -> inpf;

	/* copy the output flag */
	puser -> outf = pudb -> outf;

	/* copy temp/message flag */
	puser -> tmfl = pudb -> tmfl;

	/* copy the NCMD trend flag, TEMP FLOAT trend flag
	   and the BOOL FLAG trend flag
	*/
	puser -> trfl = pudb -> trfl;
	puser -> trbl = pudb -> trbl;
	puser -> trnc = pudb -> trnc;

	/* copy mode */
	puser -> mode = pudb -> mode;

	/* copy command fields */
	puser -> ncmd = pudb -> ncmd;
	puser -> lcmd = pudb -> lcmd;

	/* copy function value */
	puser -> func = pudb -> func;

	/* copy the status flag */
	puser -> stts = pudb -> stts;

	/* copy numeric prompt flag (NPFL) and
	   numeric redundant flag (NRFL)
	*/
	puser -> npfl = pudb -> npfl;
	puser -> nrfl = pudb -> nrfl;

	/* copy the time variables
	   see .../epicsH/tsDefs.h for def of TS_STAMP (pudb -> lptm is a TS_STAMP
	   see .../epicsH/tsDefs.h for def of struft tsDetail
	   see .../src/rec/libCom/tsSubr.c for tsStampToLocal()
	*/
	puser -> cetm = pudb -> lptm.secPastEpoch;	/* current epics tiem in secs */
	/* convert current epics time to its parts */
	tsStampToLocal (pudb -> lptm, &tsD);
	pudb -> year = puser -> year = tsD.year;		/* as 1995 etc */
	pudb -> leap = puser -> leap = tsD.leapYear;		/* 0 = isnt, 1 = is */
	pudb -> doy  = puser -> doy  = tsD.dayYear + 1;		/* 1 = jan 1 */
	pudb -> mon  = puser -> mon  = tsD.monthNum + 1;	/* 1 - 12, jan - dec */
	pudb -> dom  = puser -> dom  = tsD.dayMonth + 1;	/* 1 ... */
	pudb -> dow  = puser -> dow  = tsD.dayOfWeek + 1;	/* 1 - 7, sun - sat */
	pudb -> hod  = puser -> hour = tsD.hours;
	pudb -> min  = puser -> min  = tsD.minutes;
	pudb -> sec  = puser -> sec  = tsD.seconds;

	/* copy current clock ticks (CTCK) and delta clock ticks (DCTK) to user */
	puser -> ctck = pudb -> ctck;
	puser -> dctk = pudb -> dctk;

	/* copy the message field (MSGF) */
	strncpy (puser -> msgf, pudb -> msgf, UDB_MSG_SIZE);

	/* copy the first pass flag */
	puser -> fpas = pudb -> fpas;

	/* copy the zero cmds flag */
	puser -> zcmf = pudb -> zcmf;
    }
    /* copy from user's data back to record */
    else
    {
	/* copy ini2 flag */
	pudb -> ini2 = puser -> ini2;

	/* only copy nsta and nsev back to the record, 
	   do not give user ability to chg stat, sevr and tenb
	*/
	pudb -> nsta = puser -> nsta;
	pudb -> nsev = puser -> nsev;

	/* copy the input flag */
	pudb -> inpf = puser -> inpf;

	/* copy the output flag */
	pudb -> outf = puser -> outf;

	/* copy temp/message flag */
	pudb -> tmfl = puser -> tmfl;

	/* copy the NCMD trend flag, TEMP FLOAT trend flag
	   and the BOOL FLAG trend flag
	*/
	pudb -> trfl = puser -> trfl;
	pudb -> trbl = puser -> trbl;
	pudb -> trnc = puser -> trnc;

	/* copy output values back to record */
	for (i = 0, dbl_rec = &pudb -> ovl1, dbl_user = &puser -> ovl[0];
	     i < UDB_NUM_OUTPUTS;
	     i++)
	{
	    *dbl_rec = *dbl_user;
	    dbl_rec++;
	    dbl_user++;
	}

	/* copy mode */
	pudb -> mode = puser -> mode;

	/* copy command fields */
	pudb -> ncmd = puser -> ncmd;
	pudb -> lcmd = puser -> lcmd;

	/* copy function value */
	pudb -> func = puser -> func;

	/* copy the status flag */
	pudb -> stts = puser -> stts;

	/* copy numeric prompt flag (NPFL) and
	   numeric redundant flag (NRFL)
	*/
	pudb -> npfl = puser -> npfl;
	pudb -> nrfl = puser -> nrfl;

	/* copy the message field (MSGF) */
	strncpy (pudb -> msgf, puser -> msgf, UDB_MSG_SIZE);

	/* copy the first pass flag */
	pudb -> fpas = puser -> fpas;

	/* copy the zero cmds flag */
	pudb -> zcmf = puser -> zcmf;
    }

    /* copy manual/automatic command fields */

    for (i = 0,
	 cm_rec = &pudb -> cm1m,	/* rec manual commands */
	 cm_user = &puser -> cmm[0],	/* user manual commands */
	 ca_rec = &pudb -> cm1a,	/* rec automatic commands */
	 ca_user = &puser -> cma[0];	/* user automatic commands */
	 i < UDB_NUM_CMDS;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	{
	    *cm_user = *cm_rec;
	    *ca_user = *ca_rec;
	}
	else
	{
	    *cm_rec = *cm_user;
	    *ca_rec = *ca_user;
	}
	cm_rec++;
	cm_user++;
	ca_rec++;
	ca_user++;
    }

    /* copy function flags */

    for (i = 0, 
	 ffl_rec = &pudb -> ffl1,		/* record function flags */
	 ffl_user = &puser -> ffl[0];		/* user function flags   */
	 i < UDB_NUM_FUNCS;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *ffl_user = *ffl_rec;
	else
	    *ffl_rec = *ffl_user;
	ffl_rec++;
	ffl_user++;
    }

    /* copy the check boxs */

    for (i = 0,
	 us_rec = &pudb -> ck1,			/* record check box */
	 us_user = &puser -> chk[0];		/* user check box */
	 i < UDB_NUM_CHKBXS;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *us_user = *us_rec;
	else
	    *us_rec = *us_user;
	us_rec++;
	us_user++;
    }

    /* copy the numeric floats */

    for (i = 0,
	 fl_rec = &pudb -> n01,			/* record numeric floats */
	 fl_user = &puser -> n[0];		/* user numeric floats   */
	 i < UDB_NUM_NUMERIC_FLOAT;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *fl_user = *fl_rec;
	else
	    *fl_rec = *fl_user;
	fl_rec++;
	fl_user++;
    }

    /* copy numeric doubles */

    for (i = 0,
	 dbl_rec = &pudb -> d01,		/* record numeric doubles */
	 dbl_user = &puser -> d[0];		/* user numeric doubles */
	 i < UDB_NUM_NUMERIC_DOUBLE;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *dbl_user = *dbl_rec;
	else
	    *dbl_rec = *dbl_user;
	dbl_rec++;
	dbl_user++;
    }

    /* copy numeic unsigned longs */

    for (i = 0,
	 ul_rec = &pudb -> u01,			/* record numeric ulongs */
	 ul_user = &puser -> u[0];		/* user numeric ulongs */
	 i < UDB_NUM_NUMERIC_ULONG;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *ul_user = *ul_rec;
	else
	    *ul_rec = *ul_user;
	ul_rec++;
	ul_user++;
    }

    /* copy timers and timer flags */

    for (i = 0,
	 ul_rec = &pudb -> tm01,		/* record timer */
	 ul_user = &puser -> tmr[0],		/* user timer */
	 us_rec = &pudb -> tf01,		/* record timer flag */
	 us_user = &puser -> tf[0];		/* user timer flag */
	 i < UDB_NUM_TIMERS;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	{
	    *ul_user = *ul_rec;
	    *us_user = *us_rec;
	}
	else
	{
	    *ul_rec = *ul_user;
	    *us_rec = *us_user;
	}
	ul_rec++;
	ul_user++;
	us_rec++;
	us_user++;
    }

    /* copy the temp float values */

    for (i = 0,
	 fl_rec = &pudb -> fp01,		/* record float value */
	 fl_user = &puser -> fp[0];		/* user float value */
	 i < UDB_NUM_TEMP_FLOAT;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *fl_user = *fl_rec;
	else
	    *fl_rec = *fl_user;
	fl_rec++;
	fl_user++;
    }

    /* copy the temp ulong values */

    for (i = 0,
	 ul_rec = &pudb -> lu01,		/* record ulong value */
	 ul_user = &puser -> ul[0];		/* user ulong value */
	 i < UDB_NUM_TEMP_ULONG;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *ul_user = *ul_rec;
	else
	    *ul_rec = *ul_user;
	ul_rec++;
	ul_user++;
    }

    /* copy the temp double values */

    for (i = 0,
	 dbl_rec = &pudb -> db01,		/* record double value */
	 dbl_user = &puser -> db[0];		/* user double value */
	 i < UDB_NUM_TEMP_DOUBLE;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *dbl_user = *dbl_rec;
	else
	    *dbl_rec = *dbl_user;
	dbl_rec++;
	dbl_user++;
    }

    /* copy the boolean flags */

    for (i = 0,
	 us_rec = &pudb -> bl01,		/* record boolean flag */
	 us_user = &puser -> bl[0];		/* user boolean flag */
	 i < UDB_NUM_FLAGS;
	 i++)
    {
	if (update_type == UDB_COPY_TO_LOCAL)
	    *us_user = *us_rec;
	else
	    *us_rec = *us_user;
	us_rec++;
	us_user++;
    }

    return;
}


/****
    get a value from the input or output PVs
****/

long UDB_get_field (prec, io, fld_name, value, fld_type, iotype)
    void		*prec;		/* void point to udb record */
    int			io;		/* which input/output */
    char		*fld_name;	/* field to get */
    void		*value;		/* place to stick field value */
    int			fld_type;	/* type of field value is */
    int			iotype;		/* input or output */
{
    struct udbRecord	*pudb = (struct udbRecord *)prec;
    struct link		*inputPV = &pudb -> inp1;
    struct link		*outputPV = &pudb -> out1;
    struct link		*plink;
    struct dbAddr	dbAddr;
    long		options = 0;
    long		nRequests = 1;
    long		status = 0;
    char		pvname[PVNAME_SZ + FLDNAME_SZ + 4];
    char		*p;

    if ( (fld_type < 0) || (fld_type >= UDB_NTYPES) )
	return (S_db_badDbrtype);

    if (iotype == UDB_INPUT_FIELD)
    {
	plink = inputPV;
	if ( (io < 1) || (io > UDB_NUM_INPUTS) )
	    return (S_udb_RangeError);
    }
    else
    {
	plink = outputPV;
	if ( (io < 1) || (io > UDB_NUM_OUTPUTS) )
	    return (S_udb_RangeError);
    }
    plink += (io - 1);
    if (plink -> type != DB_LINK)
	return (S_db_notFound);

    strncpy (pvname, plink -> value.db_link.pvname, sizeof (pvname) );

    /* if name already has .NAME remove it */
    if ( (p = (char*)index (pvname, '.')) != NULL)
	*p = NULL;
    strcat (pvname, ".");
    strcat (pvname, fld_name);
    if ( (status = dbNameToAddr (pvname, &dbAddr)) != NULL)
	return (status);
    if (fld_type == UDB_DBR_STRING)
	nRequests = dbAddr.field_size;
    if ( (status = dbGetField (&dbAddr, (short)fld_type, (caddr_t)value,
			       &options, &nRequests, NULL)) != 0)
	return (status);

    return (0);
}


/****
    for all inputs that are links get their values and their
    	status
****/

static long UDB_get_inputs (prec, inv, iok)
    void		*prec;		/* void ptr to udb record */
    double		*inv;		/* input values	*/
    unsigned short	*iok;		/* input status */
{
    struct udbRecord	*pudb = (struct udbRecord *)prec;
    struct link		*inputPV = &pudb -> inp1;
    double		*rec_input_value = &pudb -> inv1;
    unsigned short	*rec_iok = &pudb -> iok1;
    double		*in = inv;
    unsigned short	*ok = iok;
    struct dbAddr	dbAddr;
    long		options = 0;
    long		nRequest = 1;
    long		status = 0;
    char		pvname[PVNAME_SZ + FLDNAME_SZ + 4];
    char		*p;
    int			i;

    for (i = 0; i < UDB_NUM_INPUTS; i++)
    {
	status = 0;
	if (inputPV -> type == DB_LINK)
	{
	    /* get the value of the field for the input PV, if the user
	       has selected process passive, the dbGetLink will cause the
	       input to process before the field value is gotten
	    */
	    if ( (status = dbGetLink (&inputPV -> value.db_link, 
				      (struct dbCommon *)pudb,
				      DBR_DOUBLE, (caddr_t)rec_input_value,
				      &options, &nRequest)) != 0)
	    {
		*in = 0;
		*rec_input_value = 0;
		*ok = UDB_IO_GET_FAILURE;
		*rec_iok = UDB_IO_GET_FAILURE;
	    }
	    else
	    {
		/* update users copy of the input value */
		*in = *rec_input_value;

		strncpy (pvname, inputPV -> value.db_link.pvname, sizeof (pvname));

		/* if name already includes a field extension, get
		   rid of it so we can tack on our own (.SEVR)
		*/
		if ( (p = (char *)index (pvname, '.')) != NULL)
		    *p = NULL;
		strcat (pvname, ".SEVR");
		if ( (status = dbNameToAddr (pvname, &dbAddr)) != 0)
		{
		    *ok = UDB_IO_GET_FAILURE;
		    *rec_iok = UDB_IO_GET_FAILURE;
		}
		
		/* get the value in pvname.SEVR */
		if (!status)
		{
		    if ( (status = dbGetField (&dbAddr, DBR_ENUM, (caddr_t)rec_iok,
		    			       &options, &nRequest, NULL)) != 0)
		    {
		    	*ok = UDB_IO_GET_FAILURE;
		    	*rec_iok = UDB_IO_GET_FAILURE;
		    }
		    else
		    {
			*ok = *rec_iok;
		    }
		}
		/* not init'd or SEVR is VALID_ALARM */
		if (!(okToUse (inputPV)))
		{
		    *ok = VALID_ALARM;
		    *rec_iok = VALID_ALARM;
		}
	    }
	}
	/* not a link */
	else
	{
	    *in = 0;
	    *rec_input_value = 0;
	    *ok = UDB_NO_IO_POINT;
	    *rec_iok = UDB_NO_IO_POINT;
	}

	in++;
	ok++;
	rec_input_value++;
	rec_iok++;
	inputPV++;

    } /* end for all input PVs */

    return (status);
}


/****
    for all outputs that are links get their values and their
    	status
****/

static long UDB_get_outputs (prec, ovl, ook)
    void		*prec;		/* void ptr to udb record */
    double		*ovl;		/* output value */
    unsigned short	*ook;		/* output status */
{
    struct udbRecord	*pudb = (struct udbRecord *)prec;
    struct link		*outputPV = &pudb -> out1;
    double		*rec_out_value = &pudb -> ovl1;
    double		*ov = ovl;
    unsigned short	*rec_ook = &pudb -> ook1;
    unsigned short	*ok = ook;
    struct dbAddr	dbAddr;
    long		options = 0;
    long		nRequest = 1;
    long		status = 0;
    char		pvname[PVNAME_SZ + FLDNAME_SZ + 4];
    char		*p;
    int			i;

    for (i = 0; i < UDB_NUM_OUTPUTS; i++)
    {
	status = 0;
	if (outputPV -> type == DB_LINK)
	{
	    /* do a dbGetField for the output rather than a dbGetLink, we do not
	       want the output point to process which is what a dbGetLink will cause
	       if the user has selected process passive
	    */
	    if ( (status = dbGetField ((struct dbAddr*)outputPV -> value.db_link.pdbAddr, 
				       DBR_DOUBLE, (caddr_t)rec_out_value,
				       &options, &nRequest, NULL)) != NULL)
	    {
		*ov = 0;
		*rec_out_value = 0;
		*ok = UDB_IO_GET_FAILURE;
		*rec_ook = UDB_IO_GET_FAILURE;
	    }
	    else
	    {
		/* update users local copy of the value */
		*ov = *rec_out_value;

		strncpy (pvname, outputPV -> value.db_link.pvname, sizeof (pvname));

		/* if name already includes a field extension, get
		   rid of it so we can tack on our own (.SEVR)
		*/
		if ( (p = (char *)index (pvname, '.')) != NULL)
		    *p = NULL;
		strcat (pvname, ".SEVR");
		if ( (status = dbNameToAddr (pvname, &dbAddr)) != 0)
		{
		    	*ok = UDB_IO_GET_FAILURE;
		    	*rec_ook = UDB_IO_GET_FAILURE;
		}
		
		/* get the value in pvname.SEVR */
		if (!status)
		{
		    if ( (status = dbGetField (&dbAddr, DBR_ENUM, (caddr_t)rec_ook,
					       &options, &nRequest, NULL)) != NULL)
		    {
		    	*ok = UDB_IO_GET_FAILURE;
		    	*rec_ook = UDB_IO_GET_FAILURE;
		    }
		    /* update the users local value status */
		    else
		    {
			*ok = *rec_ook;
		    }
		}

		/* not init'd or SEVR is VALID_ALARM */
		if (!(okToUse (outputPV)))
		{
		    *ok = VALID_ALARM;
		    *rec_ook = VALID_ALARM;
		}
	    }
	}
	/* not a link */
	else
	{
	    *ov = 0;
	    *rec_out_value = 0;
	    *ok = UDB_NO_IO_POINT;
	    *rec_ook = UDB_NO_IO_POINT;
	}

	ov++;
	ok++;
	rec_out_value++;
	rec_ook++;
	outputPV++;

    } /* end for all output PVs */

    return (status);
}


/****
    return 1 if io's resource is master else return 0
****/

long UDB_master (prec, io, iotype, error)
    void	*prec;		/* void ptr to udb record */
    int		io;		/* which input/output */
    int		iotype;		/* input or output */
    long	*error;		/* error to set */
{
    struct udbRecord	*pudb = (struct udbRecord *)prec;
    struct dbCommon	*recHdr;
    struct link		*inputPV = &pudb -> inp1;
    struct link		*outputPV = &pudb -> out1;
    struct link		*plink;
    short		answer;

    *error = 0;

    if (iotype == UDB_INPUT_FIELD)
    {
	plink = inputPV;
	if ( (io < 1) || (io > UDB_NUM_INPUTS) )
	{
	    *error = S_udb_RangeError;
	    return (0);
	}
    }
    else
    {
	plink = outputPV;
	if ( (io < 1) || (io > UDB_NUM_OUTPUTS) )
	{
	    *error = S_udb_RangeError;
	    return (0);
	}
    }

    plink += (io - 1);

    if (plink -> type != DB_LINK)
    {
	*error = S_db_notFound;
	return (0);
    }

    recHdr = (struct dbCommon *)((struct dbAddr *)(plink -> value.db_link.pdbAddr)) -> precord;

    if (resGetCaAnswer (recHdr -> radr, &answer) == RES_OK)
    {
	if (answer)
	    return (1);
    }

    return (0);
}
    

/****
    user wants to set an alarm condition, but 1st set nsta and nsev
	to NO_ALARM, if are set to 0xff, they are set to 0xff by
	recUdb:process() to let it know if the user has set an alarm
****/

void UDB_new_alarm (new_stat, new_sevr, nsta, nsev, debug, error, udb_name)
    unsigned short	new_stat;
    unsigned short	new_sevr;
    unsigned short	*nsta;		/* current alarm status */
    unsigned short	*nsev;		/* current alarm severity */
    int			debug;
    long		*error;
    char		*udb_name;
{
    /* the alarm they want to set is not currently supported */
    if ( (new_stat >= ALARM_NSTATUS) || (new_sevr >= ALARM_NSEV) )
    {
	if (debug)
	    logMsg ("UDB:%s, SET_ALARM sta range 0 - %d, sev range 0 - %d (%d,%d)\n",
			(int)udb_name, ALARM_NSTATUS - 1, ALARM_NSEV - 1, 
			new_stat, new_sevr,0);
	*error = S_udb_RangeError;
	return;
    }
    if ( ((*nsta) == 0xff) || ((*nsev) == 0xff) )
    {
	*nsta = NO_ALARM;
	*nsev = NO_ALARM;
    }
    if ( (*nsev) < new_sevr)
    {
	*nsta = new_stat;
	*nsev = new_sevr;
    }

    return;
}



/****
    put a value to the input or output PVs
****/

long UDB_put_field (prec, io, fld_name, value, fld_type, iotype)
    void		*prec;		/* void point to udb record */
    int			io;		/* which input/output */
    char		*fld_name;	/* field to put */
    void		*value;		/* value to place in fld_name */
    int			fld_type;	/* type of field value is */
    int			iotype;		/* input or output */
{
    struct udbRecord	*pudb = (struct udbRecord *)prec;
    struct link		*inputPV = &pudb -> inp1;
    struct link		*outputPV = &pudb -> out1;
    struct link		*plink;
    struct db_link	dblink;
    struct dbAddr	dbAddr;
    long		nRequests = 1;
    long		status = 0;
    char		pvname[PVNAME_SZ + FLDNAME_SZ + 4];
    char		*p;

    if ( (fld_type < 0) || (fld_type >= UDB_NTYPES) )
	return (S_db_badDbrtype);

    if (iotype == UDB_INPUT_FIELD)
    {
	plink = inputPV;
	if ( (io < 1) || (io > UDB_NUM_INPUTS) )
	    return (S_udb_RangeError);
    }
    else
    {
	plink = outputPV;
	if ( (io < 1) || (io > UDB_NUM_OUTPUTS) )
	    return (S_udb_RangeError);
    }
    plink += (io - 1);
    if (plink -> type != DB_LINK)
	return (S_db_notFound);

    strncpy (pvname, plink -> value.db_link.pvname, sizeof (pvname) );

    /* if name already has .NAME remove it */
    if ( (p = (char*)index (pvname, '.')) != NULL)
	*p = NULL;
    strncpy (dblink.pvname, pvname, PVNAME_SZ);
    dblink.fldname[0] = 0;
    if (*fld_name)
    {
	strcat (pvname, ".");
	strcat (pvname, fld_name);
	strncpy (dblink.fldname, fld_name, FLDNAME_SZ);
    }
    if ( (status = dbNameToAddr (pvname, &dbAddr)) != NULL)
	return (status);
    if (fld_type == UDB_DBR_STRING)
	nRequests = dbAddr.field_size;

    dblink.process_passive = 0;
    dblink.maximize_sevr = 0;
    dblink.pdbAddr = (caddr_t)&dbAddr;
    if ( (status = dbPutLink (&dblink, (struct dbCommon *)prec, (short)fld_type,
			      (caddr_t)value, nRequests)) != NULL)
	return (status);

    return (0);
}


/****
    convert an epics time into a string of type
	mm/dd/yy hh:mm:ss
    NOTE:
	see .../epicsH/tsDefs.h for def of TS_STAMP 
	tsStampToText() see .../src/libCom/tsSubr.c, converts a timestamp
		which is in GMT to local time as a string
****/

long UDB_stamp_to_text (ptime, buff)
    unsigned long	*ptime;		/* epics time in seconds */
    char		*buff;		/* place to put the string */
{
    TS_STAMP	tStamp;
    char	tbuff[64];
    char	*p;

    /* set up TS_STAMP struct, nano secs not needed */
    tStamp.secPastEpoch = *ptime;
    tStamp.nsec = 0;

    if ( (tsStampToText (&tStamp, TS_TEXT_MMDDYY, tbuff)) == NULL)
	return (S_ts_sysTimeError);

    /* get rid of the fractions of a second */
    if ( (p = (char *)index (tbuff, '.')) != NULL)
	*p = NULL;
    strcpy (buff, tbuff);

    return (0);
}


/****
    given a text string of form
	mm/dd/yy hh:mm:ss
    return the epics time stamp seconds, the nano secs are not needed
    NOTE:
	see .../epicsH/tsDefs.h for def of TS_STAMP 
	tsTextToStamp() see .../src/libCom/tsSubr.c, converts a string
		to an epics TS_STAMP
****/

long UDB_text_to_stamp (buff, ptime)
    char		*buff;		/* buffer with time string */
    unsigned long	*ptime;		/* place to put epics seconds */
{
    long	retval;
    TS_STAMP	tStamp;

    retval = tsTextToStamp (&tStamp, &buff);
    if (retval != S_ts_OK)
	return (retval);
    *ptime = tStamp.secPastEpoch;

    return (0);
}

