/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		CONVERT DATABASE RECORD TO LOCAL ENDIAN		*/
/*								*/
/*	All records are transmitted from dbClient in network	*/
/*	byte order (i.e., big Endian). This routine will	*/
/*	convert that record to the local Endian.		*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <types.h>
#include <in.h>
#include <stdio.h>

#include "dbDefs.h"
#include "dbRecDes.h"
#include "dbFldTypes.h"
#include "link.h"
#include "tisByteOrder.h"

int localEndian(char *pRec, short recType)
{
    short		endian = 0x1234;
    short		i;
    struct recTypDes	*recTypDes;
    struct fldDes 	*fldDes;
    struct link		*link;

    /*
    If we are on a big Endian machine (i.e., MC680X0), we are
    done.
    */

    if (*((char *)&endian) == 0x12)
	return (0);

    /*
    Must convert. Get a pointer to the SDR for this record type.
    */

    recTypDes = GET_PRECTYPDES(recType);

    /*
    Loop along the field definitions, modifying them as necessary
    */

    for (i = 0; i < recTypDes -> no_fields; i++)
	{

	/*
	Get the field descriptor for this field type.
	*/

	fldDes = GET_PFLDDES(recTypDes, i);
	if (!(fldDes -> promptflag))
	    continue;

	/*
	Convert as necessary.
	*/

	switch (fldDes -> field_type)
	    {
	    case DBF_STRING:
	    case DBF_CHAR:
	    case DBF_UCHAR:

		/*
		No conversion is required for these data types.
		*/

		break;

	    case DBF_SHORT:
	    case DBF_USHORT:
	    case DBF_ENUM:
	    case DBF_GBLCHOICE:
	    case DBF_CVTCHOICE:
	    case DBF_RECCHOICE:
	    case DBF_DEVCHOICE:

		/*
		A 16 bit integer. Convert it in place.
		*/

		*(short *)(pRec + fldDes -> offset) = ntohs(*(short *)(pRec + fldDes -> offset));
		break;

	    case DBF_LONG:
	    case DBF_ULONG:

		/*
		A 32 bit integer. Convert it in place, again.
		*/

		*(int *)(pRec + fldDes -> offset) = ntohl(*(int *)(pRec + fldDes -> offset));
		break;


	    case DBF_FLOAT:

		/*
		A 32 bit float. Another conversion in place.
		*/

		ntohf(*(float *)(pRec + fldDes -> offset), *(float *)(pRec + fldDes -> offset));
		break;

	    case DBF_DOUBLE:

		/*
		A 64 bit float. Our last "easy" conversion in place.
		*/

		ntohd(*(pRec + fldDes -> offset), *(pRec + fldDes -> offset));
		break;


	    case DBF_INLINK:
	    case DBF_OUTLINK:
	    case DBF_FWDLINK:

		/*
		A link. Must handle this according to link type.
		*/

		link = (struct link *)(pRec + fldDes -> offset);
		link -> type = ntohl(link -> type);

		/*
		All the "TATE" types start with SAB_IO. Assume
		that anything that number or higher uses the
		Tate standard methods.
		*/

		if (link -> type >= SAB_IO)
		    {
		    link -> value.tateio.link = ntohs(link -> value.tateio.link);
		    link -> value.tateio.adapter = ntohs(link -> value.tateio.adapter);
		    link -> value.tateio.card = ntohs(link -> value.tateio.card);
		    link -> value.tateio.signal = ntohs(link -> value.tateio.signal);
		    }
		else
		    {
		    switch (link -> type)
		        {
		        case CONSTANT:
			    ntohf((link -> value.value), (link -> value.value));
			    break;
		        case PV_LINK:
			    link -> value.pv_link.process_passive = ntohs(link -> value.pv_link.process_passive);
			    link -> value.pv_link.maximize_sevr = ntohs(link -> value.pv_link.maximize_sevr);
			    break;
		        case VME_IO:
			    link -> value.vmeio.card = ntohs(link -> value.vmeio.card);
			    link -> value.vmeio.signal = ntohs(link -> value.vmeio.signal);
			    break;
		        case CAMAC_IO:
			    link -> value.camacio.branch = ntohs(link -> value.camacio.branch);
			    link -> value.camacio.crate = ntohs(link -> value.camacio.crate);
			    link -> value.camacio.slot = ntohs(link -> value.camacio.slot);
			    link -> value.camacio.channel = ntohs(link -> value.camacio.channel);
			    break;
		        case AB_IO:
			    link -> value.abio.link = ntohs(link -> value.abio.link);
			    link -> value.abio.adapter = ntohs(link -> value.abio.adapter);
			    link -> value.abio.card = ntohs(link -> value.abio.card);
			    link -> value.abio.signal = ntohs(link -> value.abio.signal);
			    link -> value.abio.plc_flag = ntohs(link -> value.abio.plc_flag);
			    break;
		        case GPIB_IO:
			    link -> value.gpibio.link = ntohs(link -> value.gpibio.link);
			    link -> value.gpibio.addr = ntohs(link -> value.gpibio.addr);
			    break;
		        case BITBUS_IO:

			    /*
			    No conversion needed
			    */

			    break;
		        case INST_IO:

			    /*
			    No conversion needed
			    */

			    break;
		        case BBGPIB_IO:

			    /*
			    No conversion needed
			    */

			    break;
			default:
			    printErr("Invalid link type in %s\n", pRec);
			    return (-1);
			}
		    }
		break;
	    }
	}
    return (0);
}
