/****************************************************************/
/*								*/
/*		   RESOURCE GROUP LOCK MANAGER			*/
/*								*/
/*	This task watches over groups of resources. When any	*/
/*	resource in a group transits from caAnswer 1 -> 0	*/
/*	AND it's state is slave, then set the resource lock	*/
/*	flag for that group to TRUE.				*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <vxWorks.h>
#include <taskLib.h>

#include <restable.h>

#define MAXGROUPS	16

/****** THE GLOBAL ARRAY ******/

short  resGroupLock[MAXGROUPS];		/* 0 init */
char  *resGroupTripName[MAXGROUPS];	/* 0 init, again */

typedef struct resLock	RESLOCK;
struct resLock
    {
    RES_TABLE	*resEntry;
    int		lastCaAnswer;		/* Compiler would make it 4 anyway */
    RESLOCK	*nextLock;
    };

static RESLOCK	*resLockList[MAXGROUPS];

/****************************************************************/
/*								*/
/*		ADD A RESOURCE TO A RESOURCE GROUP		*/
/*								*/
/****************************************************************/

int assignLockGroup(group, resource)
int	group;
char	*resource;
{
    RES_TABLE	*resEntry;
    RESLOCK	*newMember;
    RESLOCK	*currMember;

    /*
    Groups are numbered 1 through MAXGROUPS
    */

    if ((group <= 0) || (group > MAXGROUPS))
	return (ERROR);
    group--;

    /*
    Also have to be able to find the resource list.
    */

    if (resGetByName(resource, &resEntry) != RES_OK)
	return (ERROR);

    /*
    Allocate a structure to hold this new one and fill it out.
    */

    if ((newMember = (RESLOCK *) calloc(1, sizeof(RESLOCK))) == NULL)
	return (ERROR);
    newMember -> resEntry = resEntry;

    /*
    Hang it on the END of the chain for this group. If we put it on
    the end instead of the start of the chain we don't have to bother
    with locking the chain. There is no way to remove a resource
    from a group.
    */

    if ((currMember = resLockList[group]) == NULL)
	{
	resLockList[group] = newMember;
	return (OK);
	}

    while (currMember -> nextLock)
	currMember = currMember -> nextLock;
    currMember -> nextLock = newMember;
    return (OK);
}


/****************************************************************/
/*								*/
/*		THE RESOURCE GROUP SCAN TASK			*/
/*								*/
/****************************************************************/

void resGroupScan(void)
{
    short	index;
    short	newCaAnswer;
    short	localState;
    short	partnerState;
    RESLOCK	*currResource;

for (;;)
    {
    for (index = 0; index < MAXGROUPS; index++)
	{
	if (!resGroupLock[index])
	    resGroupTripName[index] = NULL;		/* Reset back from previous trip */
	currResource = resLockList[index];
	while (currResource)
	    {
	    resGetCaAnswer(currResource -> resEntry, &newCaAnswer);
	    if (currResource -> lastCaAnswer)
		{
		if (!newCaAnswer)
		    {
		    resGetStates(currResource -> resEntry, &localState, &partnerState);
		    if (localState == RES_SLV)
			{
			if (!resGroupTripName[index])
			    resGroupTripName[index] = currResource -> resEntry -> resourceName;
			resGroupLock[index] = TRUE;
			}
		    }
		}
	    currResource -> lastCaAnswer = newCaAnswer;
	    currResource = currResource -> nextLock;
	    }
	}
    taskDelay(60);
    }
}



/****************************************************************/
/*								*/
/*	INITIALIZE THE TASK THAT DOES THE RESOURCE SCANNING	*/
/*								*/
/****************************************************************/

int initResGroupScan(void)
{
    if (taskSpawn("resGroup", 90, VX_FP_TASK, 2048, (FUNCPTR) resGroupScan,
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0) == ERROR)
	return (ERROR);
    return (OK);
}
