/*reslib.c,v 1.1.1.1 1999/08/03 14:44:28 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/************************************************************************/
/*									*/
/*			RESOURCE TABLE UTILITY ROUTINES			*/
/*									*/
/*	This is a collection of routines that manipulate the resource	*/
/*	table. They perform common functions such as finding resources,	*/
/*	getting and setting structure members, etc.			*/
/*									*/
/************************************************************************/

/****** MOD LOG
01/09/93	GMU	410-581-0422	uchenick@tate.com
		Original Entry
******/


/************************************************************************/
/*									*/
/*			INCLUDE FILES					*/
/*									*/
/************************************************************************/

#include <vxWorks.h>
#include <stdlib.h>
#include <memLib.h>
#include <types.h>
#include <stdio.h>
#include <string.h>
#include <inetLib.h>

#include "fast_lock.h"
#include "scanLock.h"
#include "restable.h"

/* these 4 are needed because rbMgt.h includes serverControl.h
 * which includes dbAccess.h, which needs these 4, ems
*/
#include "dbDefs.h"
#include "dbCommon.h"
#include "link.h"
#include "db_field_log.h"

/* also needed by rbMgt.h */
#include "dbDefs.h"
#include "dbRecDes.h"

#include "inet_ntoa.h"
#include "rbMgt.h"

/************************************************************************/
/*									*/
/*			GET RESOURCE BY NAME				*/
/*									*/
/*	This function looks up an entry in the resource table by	*/
/*	comparing the name passed by the caller. We return RES_OK	*/
/*	if the resource is found, RES_NOTYET if the resource table	*/
/*	hasn't been built yet, or RES_NOTFOUND if we can't find the	*/
/*	name that was passed to us. When we return RES_NOTFOUND, the	*/
/*	pointer provided by the user will always be set to the tail	*/
/*	of the linked list. This is a convenience to the dbServer task.	*/
/*									*/
/************************************************************************/

int resGetByName(name, tableAddr)
char		*name;
RES_TABLE	**tableAddr;
{
	if (!(*tableAddr = resTableHead))
		return (RES_NOTYET);
	for (;;)
		{
		if (!strcmp((*tableAddr) -> resourceName, name))
			return (RES_OK);
		if ((*tableAddr) -> link)
			*tableAddr = (*tableAddr) -> link;
		else
			return (RES_NOTFOUND);
		}
}


/************************************************************************/
/*									*/
/*		DECLARE A RESOURCE TO BE NON-REDUNDANT			*/
/*									*/
/*	The resource indicated by the pointer passed by the caller	*/
/*	is declared to be non-redundant. This means that we are to	*/
/*	set the private and caAnswer members. We will also clear some	*/
/*	of the networking information to keep the table neat. This	*/
/*	allows the driver to change his mind later by registering	*/
/*	himself.							*/
/*									*/
/************************************************************************/

int resDeclarePrivate(resEntry)
RES_TABLE	*resEntry;
{
	if (!resEntry)
		return (RES_BADADDR);
	resEntry -> private = 1;
	resEntry -> caAnswer = 1;
	return (RES_OK);
}


/************************************************************************/
/*									*/
/*			REGISTER A DRIVER				*/
/*									*/
/*	A driver declares his willingness to participate in the		*/
/*	redundancy scheme by registering himself for a resource.	*/
/*	Registration consists of posting the driver's local TCP		*/
/*	port number and fd into the table. The driver also tells	*/
/*	dbServer how frequently to broadcast a request for a		*/
/*	redundant partner by passing the desired time interval in	*/
/*	seconds.							*/
/*									*/
/************************************************************************/

int resRegisterDriver(resEntry, tcpPort, fd, rate)
RES_TABLE	*resEntry;
short		tcpPort;
int		fd;
int		rate;
{
	if (!resEntry)
		return (RES_BADADDR);
	resEntry -> driverPort = tcpPort;
	resEntry -> partnerFd = fd;
	/*
	For now, assume that the dbServer task will run
	ten times a second. The following expression should
	be made more general very shortly.
	*/
	resEntry -> bcastRate = rate;	/* Nearest .1 sec */
	resEntry -> bcastCounter = 0;
	resEntry -> bcastsMade = 0;
	
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*	GET THE NUMBER OF BRODCAST ATTEMPTS THAT WERE MADE	*/
/*								*/
/****************************************************************/

int resGetBcastsMade(resEntry, attempts)
RES_TABLE	*resEntry;
long		*attempts;
{
	if (!resEntry)
		return (RES_BADADDR);
	*attempts = resEntry -> bcastsMade;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		DECLARE DRIVER CONNECTION ESTABLISHED		*/
/*								*/
/*	This routine is called by a driver to declare that	*/
/*	he has successfully connected to his redundant partner	*/
/*	for this resource. This function will not only set	*/
/*	the partnerConnected member of the resource we are	*/
/*	given, but also every other resource in the table	*/
/*	that has equivalent networking information.		*/
/*								*/
/****************************************************************/

int resDriverConnected(resEntry)
RES_TABLE	*resEntry;
{
	RES_TABLE	*currEntry = 0;
	if (!resEntry)
		return (RES_BADADDR);
	while (resFindEquivDriver(resEntry, &currEntry) == RES_OK)
		currEntry -> partnerConnected = 1;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		DECLARE DRIVER CONNECTION BROKEN		*/
/*								*/
/*	This routine is precisely the same as the one that	*/
/*	declares the connection established, except that it	*/
/*	clears the partnerConnected member instead of setting	*/
/*	it.							*/
/*								*/
/****************************************************************/

int resDriverBroken(resEntry)
RES_TABLE	*resEntry;
{
	RES_TABLE	*currEntry = 0;
	if (!resEntry)
		return (RES_BADADDR);
	while (resFindEquivDriver(resEntry, &currEntry) == RES_OK)
		currEntry -> partnerConnected = 0;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		GET FD OF DRIVER <-> DRIVER SOCKET		*/
/*								*/
/****************************************************************/

int resGetDriverFd(resEntry, fd)
RES_TABLE	*resEntry;
int		*fd;
{
	if (!resEntry)
		return (RES_BADADDR);
	if (!resEntry -> partnerConnected)
		return (RES_NOCONN);
	*fd = resEntry -> partnerFd;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*	FIND NEXT ENTRY WITH EQUIVALENT DRIVER CONNECTION	*/
/*								*/
/*	This routine will find the next entry that has the	*/
/*	same partner Internet address and the same partner	*/
/*	port number. The user passes the address of a		*/
/*	table entry, or zero to start at the head of the	*/
/*	table. We won't return the starting pointer (we are	*/
/*	finding the NEXT one!), but we will return the one	*/
/*	that we are given as the search target.			*/
/*								*/
/****************************************************************/

int resFindEquivDriver(resEntry, startEntry)
RES_TABLE	*resEntry;
RES_TABLE	**startEntry;
{
	if (!(*startEntry))
		*startEntry = resTableHead;
	else
		*startEntry = (*startEntry) -> link;
	while (*startEntry)
		{
		if ((*startEntry) -> partnerInet.s_addr == resEntry -> partnerInet.s_addr
		  && (*startEntry) -> partnerPort == resEntry -> partnerPort)
			return (RES_OK);
		*startEntry = (*startEntry) -> link;
		}
	return (RES_NOMORE);
}


/****************************************************************/
/*								*/
/*	FIND NEXT ENTRY WITH EQUIVALENT DBSERVER CONNECTION	*/
/*								*/
/*	This routine is similar to resFindEquivDriver with	*/
/*	the exception that we are looking for an entry with	*/
/*	an equivalent dbServer connection.			*/
/*								*/
/****************************************************************/

int resFindEquivDbServer(resEntry, startEntry)
RES_TABLE	*resEntry;
RES_TABLE	**startEntry;
{
	if (!(*startEntry))
		*startEntry = resTableHead;
	else
		*startEntry = (*startEntry) -> link;
	while (*startEntry)
		{
		if ((*startEntry) -> partnerInet.s_addr == resEntry -> partnerInet.s_addr)
			return (RES_OK);
		*startEntry = (*startEntry) -> link;
		}
	return (RES_NOMORE);
}


/****************************************************************/
/*								*/
/*		SET THE caAnswer STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resSetCaAnswer(resEntry)
RES_TABLE	*resEntry;
{
	LOCK_LIST	*list;
	int		index;
	struct scanLock	*lock;
	if (!resEntry)
		return (RES_BADADDR);
	if (resEntry -> caAnswer)
		return (RES_OK);	/* Benign duplicate call */
	resEntry -> caAnswer = 1;
	reRbPut(resEntry -> resourceName, (short) 1);

	/*
	Find all the lock sets that need this resource and bump
	their count by 1.
	*/


	list = resEntry -> lockSets;
	while (list)
		{
		for (index = 0; index < list -> locksUsed; index++)
			{
			lock = (struct scanLock *)(list -> lockSets[index]);
			if (lock -> resGot >= lock -> resDefined)
				{
				printErr("Lock set overflow\n");
				return (RES_LOCKERR);
				}
			lock -> resGot++;
			}
		list = list -> next;
		}
	
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		CLEAR THE caAnswer STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resClearCaAnswer(resEntry)
RES_TABLE	*resEntry;
{
	LOCK_LIST	*list;
	int		index;
	struct scanLock	*lock;

	if (!resEntry)
		return (RES_BADADDR);
	if (!(resEntry -> caAnswer))
		return (RES_OK);	/* Benign duplicate call */
	resEntry -> caAnswer = 0;
	reRbPut(resEntry -> resourceName, (short) 0);

	/*
	Decrement the resource count in every lock set that uses
	this resource
	*/

	list = resEntry -> lockSets;
	while (list)
		{
		for (index = 0; index < list -> locksUsed; index++)
			{
			lock = (struct scanLock *)(list -> lockSets[index]);
			if (lock -> resGot <= 0)
				{
				printErr("Lock set underflow\n");
				return (RES_LOCKERR);
				}
			lock -> resGot--;
			}
		list = list -> next;
		}

	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		GET THE caAnswer STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resGetCaAnswer(resEntry, answer)
RES_TABLE	*resEntry;
short		*answer;
{
	if (!resEntry)
		return (RES_BADADDR);
	*answer = resEntry -> caAnswer;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*	FIND FIRST ENTRY WITH A GIVEN PARTNER FD		*/
/*								*/
/****************************************************************/

int resFindPartnerFd(fd, found)
int		fd;
RES_TABLE	**found;
{
	if (!(*found = resTableHead))
		return (RES_NOTYET);
	while (*found)
		{
		if ((*found) -> dbServerFd == fd)
			return (RES_OK);
		*found = (*found) -> link;
		}
	return (RES_NOTFOUND);
}


/****************************************************************/
/*								*/
/*	FIND FIRST ENTRY WITH A GIVEN DBSERVER FD		*/
/*								*/
/****************************************************************/

int resFindServerFd(fd, found)
int		fd;
RES_TABLE	**found;
{
	if (!(*found = resTableHead))
		return (RES_NOTYET);
	while (*found)
		{
		if ((*found) -> dbServerFd == fd)
			return (RES_OK);
		*found = (*found) -> link;
		}
	return (RES_NOTFOUND);
}


/****************************************************************/
/*								*/
/*		PUT DRIVER DATA STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resPutDriverData(resEntry, driverAddr)
RES_TABLE	*resEntry;
void		*driverAddr;
{
	if (!resEntry)
		return (RES_BADADDR);
	resEntry -> driverData = driverAddr;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		GET DRIVER DATA STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resGetDriverData(resEntry, callerAddr)
RES_TABLE	*resEntry;
void		**callerAddr;
{
	if (!resEntry)
		return (RES_BADADDR);
	*callerAddr = resEntry -> driverData;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		SET LOCAL STATE STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resPutLocalState(resEntry, newState)
RES_TABLE	*resEntry;
unsigned short	newState;
{
	if (!resEntry)
		return (RES_BADADDR);
	resEntry -> localState = newState | RES_CHANGED;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		SET PARTNER STATE STRUCTURE MEMBER		*/
/*								*/
/****************************************************************/

int resPutPartnerState(resEntry, newState)
RES_TABLE	*resEntry;
unsigned short	newState;
{
	if (!resEntry)
		return (RES_BADADDR);
	resEntry -> partnerState = newState;
	return (RES_OK);
}


/****************************************************************/
/*								*/
/*		GET LOCAL AND/OR PARTNER STATE			*/
/*								*/
/****************************************************************/

int resGetStates(resEntry, local, partner)
RES_TABLE	*resEntry;
unsigned short	*local;
unsigned short	*partner;
{
	if (!resEntry)
		return (RES_BADADDR);
	if (local)
		*local = resEntry -> localState & ~RES_CHANGED;
	if (partner)
		*partner = resEntry -> partnerState & ~RES_CHANGED;
	return (RES_OK);
}

/****************************************************************/
/*								*/
/*		DUMP THE RESOURCE TABLE				*/
/*								*/
/****************************************************************/

int resDump(void)
{
	RES_TABLE	*resEntry = resTableHead;

	while (resEntry)
		{
		printErr("resourceName: %s\n", resEntry -> resourceName);
		printErr("\tdriverPort: %d\t\tlocalDbPort: %d\n",
				resEntry -> driverPort,
				resEntry -> localDbPort);
		printErr("\tpartnerInet: %s\tpartnerPort: %d\n",
				INET_NTOA(resEntry -> partnerInet),
				resEntry -> partnerPort);
		printErr("\tdbServerPort: %d\t\tdbServerConnected: %d\n",
				resEntry -> dbServerPort,
				resEntry -> dbServerConnected);
		printErr("\tdbServerFd: %d\t\tprivate: %d\t partnerFd: %d\n",
				resEntry -> dbServerFd,
				resEntry -> private,
				resEntry -> partnerFd);
		printErr("\tcaAnswer: %d\t\tbcastRate: %d\n",
				resEntry -> caAnswer,
				resEntry -> bcastRate);
		printErr("\tbcastCounter: %d\t\tbcastsMade: %d\n",
				resEntry -> bcastCounter,
				resEntry -> bcastsMade);
		printErr("\tdriverData: 0x%X\t\tpartnerConnected: %d\n",
				resEntry -> driverData,
				resEntry -> partnerConnected);
		printErr("\tlocalState: 0x%X\t\tpartnerState: 0x%X\n",
				resEntry -> localState,
				resEntry -> partnerState);
		resEntry = resEntry -> link;
		if (resEntry)
			printErr("\n");
		}
	return (0);
}

/****************************************************************/
/*								*/
/*		DECLARE ALL RESOURCES TO BE PRIVATE		*/
/*								*/
/****************************************************************/

int resTakeAll(void)
{
	RES_TABLE	*resEntry;

	resEntry = resTableHead;
	while (resEntry)
		{
		resDeclarePrivate(resEntry);
		resEntry = resEntry -> link;
		}
	return (0);
}

/****************************************************************/
/*								*/
/*			PUT THE COMMAND FIELD			*/
/*								*/
/****************************************************************/

int resPutCommand(resEntry, newCommand)
RES_TABLE	*resEntry;
unsigned short	newCommand;
{
	if (!resEntry)
		return (RES_BADADDR);
	resEntry -> command = newCommand;
	return (RES_OK);
}

/****************************************************************/
/*								*/
/*			GET THE COMMAND FIELD			*/
/*								*/
/****************************************************************/

int resGetCommand(resEntry, commandOut)
RES_TABLE	*resEntry;
unsigned short	*commandOut;
{
	if (!resEntry)
		return (RES_BADADDR);
	*commandOut = resEntry -> command;
	return (RES_OK);
}

/****************************************************************/
/*								*/
/*		ADD A LOCK SET TO A RESOURCE			*/
/*								*/
/****************************************************************/

int resAppendLock(resEntry, lockSet)
RES_TABLE	*resEntry;
void		*lockSet;
{
	LOCK_LIST	*list;
	int		index;

	list = resEntry -> lockSets;
	if (!list)
		{
		resEntry -> lockSets = (LOCK_LIST *) calloc(1, sizeof(LOCK_LIST));
		list = resEntry -> lockSets;
		}

	/*
	Don't add this lock set if we have seen it before.
	*/

	for (;;)
		{
		for (index = 0; index < list -> locksUsed; index++)
			{
			if (lockSet == list -> lockSets[index])
				return (0);
			}
		if (list -> next)
			list = list -> next;
		else
			break;
		}

	if (list -> locksUsed == LOCKS_PER_LIST)
		{
		list -> next = (LOCK_LIST *) calloc(1, sizeof(LOCK_LIST));
		list = list -> next;
		}
	list -> lockSets[list -> locksUsed] = lockSet;
	list -> locksUsed++;

	return (0);
}


/****************************************************************/
/*								*/
/*	TRAVERSE RESOURCE TABLE TO SET COUNTS IN LOCK SETS	*/
/*								*/
/****************************************************************/

int resLockInit(void)
{
	RES_TABLE	*currEntry = resTableHead;
	LOCK_LIST	*list;
	int		index;

	while (currEntry)
		{
		list = currEntry -> lockSets;
		while (list)
			{
			for (index = 0; index < list -> locksUsed; index++)
				{
				((struct scanLock *)(list -> lockSets[index])) -> resDefined++;
				if (currEntry -> caAnswer)
					((struct scanLock *)(list -> lockSets[index])) -> resGot++;
				}
			list = list -> next;
			}
		currEntry = currEntry -> link;
		}

	return (0);
}

/****************************************************************/
/*								*/
/*		FREE ALL MEMORY IN LOCK SET CHAINS		*/
/*								*/
/****************************************************************/

void resFreeChains(void)
{
	RES_TABLE	*resEntry;
	LOCK_LIST	*list;
	LOCK_LIST	*next;

	for (resEntry = resTableHead; resEntry; resEntry = resEntry -> link)
		{
		list = resEntry -> lockSets;
		while (list)
			{
			next = list -> next;
			free(list);
			list = next;
			}
		resEntry -> lockSets = 0;
		}
}
