/*getAgentField.c,v 1.1.1.1 1999/08/03 14:44:30 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		GET DATA FIELD IN AGENT FORMAT			*/
/*								*/
/*	This function fetches a data field in Agent format.	*/
/*	The record always contains stat, sevr, and time, but	*/
/*	these are passed from the caller in case the value	*/
/*	to be fetched came from the event queue instead of	*/
/*	from the record. This is the third routine in the	*/
/*	rewrite of channel access that deals with translating	*/
/*	types. transformTypes() translates between two agent	*/
/*	types. caConvType() converts between agent types	*/
/*	and the original DBR_ types. This function translates	*/
/*	from data fields in the IOC records to the agent types.	*/
/*								*/
/*	Always return the number of bytes copied to the output	*/
/*	buffer.							*/
/*								*/
/*	WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!	*/
/*								*/
/*	This routine was written assuming that we are running	*/
/*	on a big endian processor (i.e., the native byte order	*/
/*	is the same as network byte order). If this is not	*/
/*	true, then some patchwork must be done to insert the	*/
/*	appropriate data conversion calls.			*/
/*								*/
/*	WARNING! WARNING! WARNING! WARNING! WARNING! WARNING!	*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

/* bcopt conflicts with actual definition in vxWorks */
#ifndef vxWorks
#include <bcopy.h>
#endif vxWorks
#include <vxWorks.h>
#include <string.h>
#include <stdio.h>
#include <in.h>

#include "dbDefs.h"
#include "choiceGbl.h"
#include "choice.h"
#include "dbCommon.h"
#include "link.h"
#include "db_field_log.h"
#include "dbAccess.h"
#include "tsDefs.h"
#include "recSup.h"
#include "dataTypes.h"
#include "mapTypes.h"
#include "tisByteOrder.h"

int getAgentField(pOut, agentType, numElem, stat, sevr, alarmAck, pTime, pAddr)
char		*pOut;		/* Pointer to output buffer */
short		agentType;	/* Desired point type (see dataTypes.h) */
short		numElem;	/* Number of array elements */
short		stat;		/* Point status */
short		sevr;		/* Point severity */
short		alarmAck;	/* Alarm ack */
TS_STAMP	*pTime;		/* Time Stamp */
struct dbAddr	*pAddr;		/* Field address */
{
    struct rset			*prset;
    struct dbCommon		*recHdr;
    register char		*outPtr = pOut;
    char			aenb[4];
    char			ack[4];
    char			desc[28];
    char			egu[18];
    long			prec;
    double			hopr;
    double			lopr;
    double			hihi;
    double			high;
    double			low;
    double			lolo;
    double			drvh;
    double			drvl;
    struct dbr_enumStrs		enumStrs;
    short			reqType;
    long			options = 0;
    long			numReq;
    int				oddAddr;


    /*
    Start off by getting all of the embellishments that are desired.
    We get all embellishments as doubles. Then, we put them into the
    data stream in the correct type when it's time to assemble the
    output buffer.
    */

    recHdr = (struct dbCommon *) (pAddr -> precord);
    if (agentType & 0xff00)
	{
	prset = GET_PRSET(pAddr -> record_type);

	if (agentType & AGNT_STAT)
	    {

	    /*
	    We have stat and sevr, but strings have extra fields.
	    These were added to reduce the number of channels that have
	    to be opened by the alarm handler. We put everything he
	    needs into just one data type.
	    */

	    if ((agentType & 0xff) == AGNT_STRING)
		{
		struct choiceSet	*pchoiceSet;
		char			*choice;

		agentType &= ~AGNT_GRAPH;
		agentType &= ~AGNT_CNTRL;
		pchoiceSet = GET_PCHOICE_SET(choiceGbl, GBL_YES_NO);
		aenb[0] = 0;
		ack[0] = 0;
		if (pchoiceSet)
		    {
		    if ((choice = GET_CHOICE(pchoiceSet, recHdr -> aenb)))
			strncpy(aenb, choice, sizeof(aenb));
		    if ((choice = GET_CHOICE(pchoiceSet, recHdr -> ack)))
			strncpy(ack, choice, sizeof(ack));
		    }
		strncpy(desc, recHdr -> desc, sizeof(desc));
		desc[sizeof(desc) - 1] = 0;
		egu[0] = 0;
		if (prset)
		    {
		    if (prset -> get_units)
			(*(prset -> get_units))(pAddr, egu);
		    egu[7] = 0;
		    }
		}
	    }
	
	if (agentType & AGNT_GRAPH)
	    {
	    if ((agentType & 0xff) == AGNT_STRING)
		{
		printErr("Non-supported conversion, GRAPHIC STRING\n");
		return (0);
		}
	    else if ((agentType & 0xff) != AGNT_ENUM)	/* i.e., INT, SHORT, FLOAT, CHAR, LONG, DOUBLE */
		{
	    	/*
		i.e., INT, SHORT, FLOAT, CHAR, LONG, DOUBLE
		*/

		egu[0] = 0;
		hopr = 0;
		lopr = 0;
		hihi = 0;
		high = 0;
		low = 0;
		lolo = 0;
		prec = 0;
		if (prset)
		    {
		    if (prset -> get_units)
			{
			(*(prset -> get_units))(pAddr, egu);
			egu[7] = 0;
			}
		    if (prset -> get_graphic_double)
			{
			struct dbr_grDouble	grDouble;

			(*(prset -> get_graphic_double))(pAddr, &grDouble);
			hopr = grDouble.upper_disp_limit;
			lopr = grDouble.lower_disp_limit;
			}
		    if (prset -> get_alarm_double)
			{
			struct dbr_alDouble	alDouble;

			(*(prset -> get_alarm_double))(pAddr, &alDouble);
			hihi = alDouble.upper_alarm_limit;
			high = alDouble.upper_warning_limit;
			low = alDouble.lower_warning_limit;
			lolo = alDouble.lower_alarm_limit;
			}
		    if (((agentType & 0xff) == AGNT_FLOAT) || ((agentType & 0xff) == AGNT_DOUBLE))
			if (prset -> get_precision)
			    (*(prset -> get_precision))(pAddr, &prec);
		    }
		}
	    else if ((agentType & 0xff) == AGNT_ENUM)
		{
		/*
		Also make sure CNTRL not turned on at same time!
		*/

		agentType &= ~AGNT_CNTRL;
		if (pAddr -> field_type != DBF_ENUM)
		    {

		    /*
		    Type is global choice, rec choice, or cvt choice. Let dbGetField worry about it.
		    */

		    long	getOpt = DBR_ENUM_STRS;
		    long	getNum = 0;
		    dbGetField(pAddr, 0, (caddr_t)&enumStrs, &getOpt, &getNum, 0);
		    }
		else if (prset)
		    {
		    bzero((char *) &enumStrs, sizeof(enumStrs));
		    if (prset -> get_enum_strs)
			(*(prset -> get_enum_strs))(pAddr, &enumStrs);
		    }
		}
	    else
		{
		static illegalagent;

		if (illegalagent++ < 10)
		    printErr("Illegal agent type %d.\n", agentType);
		return (0);
		}
	    }

	if (agentType & AGNT_CNTRL)
	    {
	    if ((agentType & 0xff) != AGNT_ENUM)
		{
	    	/*
		i.e., INT, SHORT, FLOAT, CHAR, LONG, DOUBLE
		*/

		if (prset)
		    if (prset -> get_control_double)
			{
			struct dbr_ctrlDouble	ctrlDouble;

			(*(prset -> get_control_double))(pAddr, &ctrlDouble);
			drvh = ctrlDouble.upper_ctrl_limit;
			drvl = ctrlDouble.lower_ctrl_limit;
			}
		}
	    else if ((agentType & 0xff) == AGNT_ENUM)
		{
		agentType &= ~AGNT_CNTRL;
		if (pAddr -> field_type != DBF_ENUM)
		    {

		    /*
		    Type is global choice, rec choice, or cvt choice. Let dbGetField worry about it.
		    */

		    long	getOpt = DBR_ENUM_STRS;
		    long	getNum = 0;
		    dbGetField(pAddr, 0, (caddr_t)&enumStrs, &getOpt, &getNum, 0);
		    }
		else if (prset)
		    {
		    bzero((char *) &enumStrs, sizeof(enumStrs));
		    if (prset -> get_enum_strs)
			(*(prset -> get_enum_strs))(pAddr, &enumStrs);
		    }
		}
	    else
		{
		printErr("Illegal agent type %d.\n", agentType);
		return (0);
		}
	    }
	}

    /*
    Now assemble the embellisments for the output data stream.
    */

    if (agentType & AGNT_STAT)
	{
	*(short *) outPtr = htons(stat);
	outPtr += sizeof(short);
	sevr |= 0x8000;
	if (alarmAck)
	    sevr |= 0x4000;
	*(short *) outPtr = htons(sevr);
	outPtr += sizeof(short);
	if ((agentType & 0xff) == AGNT_STRING)
	    {
	    strncpy(outPtr, aenb, sizeof(aenb));
	    outPtr += sizeof(aenb);
	    strncpy(outPtr, ack, sizeof(ack));
	    outPtr += sizeof(ack);
	    strncpy(outPtr, desc, sizeof(desc));
	    outPtr[sizeof(desc) - 1] = 0;
	    outPtr += sizeof(desc);
	    strncpy(outPtr, egu, 8);
	    outPtr[7] = 0;
	    outPtr += 8;
	    }
	}
    if (agentType & AGNT_TIME)
	{
	TS_STAMP	stamp;
	bcopy((char *) pTime, (char *) &stamp, sizeof(TS_STAMP));
	stamp.secPastEpoch = htonl(stamp.secPastEpoch);
	stamp.nsec = htonl(stamp.nsec);
	bcopy((char *) &stamp, outPtr, sizeof(TS_STAMP));
	outPtr += sizeof(TS_STAMP);
	}

    switch (agentType & 0xff)
	{
	case AGNT_STRING:
	    {
	    if (agentType & AGNT_GRAPH)
		{
		printErr("Graphic string not supported.\n");
		return (0);
		}
	    if (agentType & AGNT_CNTRL)
		{
		printErr("Control string not supported.\n");
		return (0);
		}
	    break;
	    }

	case AGNT_INT:
	    {
	    if (agentType & AGNT_GRAPH)
		{
		*(short *) outPtr = htons((short) hopr);
		outPtr += sizeof(short);
		*(short *) outPtr = htons((short) lopr);
		outPtr += sizeof(short);
		}
	    if (agentType & AGNT_CNTRL)
		{
		*(short *) outPtr = htons((short) drvh);
		outPtr += sizeof(short);
		*(short *) outPtr = htons((short) drvl);
		outPtr += sizeof(short);
		}
	    break;
	    }

	case AGNT_FLOAT:
	    {
	    float	tempFlt;

	    if (agentType & AGNT_GRAPH)
		{
		tempFlt = hopr;
		htonf(tempFlt, *(float *)outPtr);
		outPtr += sizeof(float);
		tempFlt = lopr;
		htonf(tempFlt, *(float *)outPtr);
		outPtr += sizeof(float);
		*(short *) outPtr = htons(prec);
		outPtr += sizeof(short);
		}
	    if (agentType & AGNT_CNTRL)
		{
		tempFlt = drvh;
		htonf(tempFlt, *(float *)outPtr);
		outPtr += sizeof(float);
		tempFlt = drvl;
		htonf(tempFlt, *(float *)outPtr);
		outPtr += sizeof(float);
		}
	    break;
	    }

	case AGNT_ENUM:
	    {
	    if ((agentType & AGNT_GRAPH) || (agentType & AGNT_CNTRL))
		{
		int	index;

		*(short *) outPtr = htons(enumStrs.no_str);
		outPtr += sizeof(short);
		for (index = 0; index < enumStrs.no_str; index++)
		    {
		    strcpy(outPtr, enumStrs.strs[index]);
		    outPtr += (strlen(enumStrs.strs[index]) + 1);
		    }
		}
	    break;
	    }

	case AGNT_CHAR:
	    {
	    if (agentType & AGNT_GRAPH)
		{
		*outPtr = hopr;
		outPtr += sizeof(char);
		*outPtr = lopr;
		outPtr += sizeof(char);
		}
	    if (agentType & AGNT_CNTRL)
		{
		*outPtr = drvh;
		outPtr += sizeof(char);
		*outPtr = drvl;
		outPtr += sizeof(char);
		}
	    break;
	    }

	case AGNT_LONG:
	    {
	    if (agentType & AGNT_GRAPH)
		{
		*(int *) outPtr = htonl((int) hopr);
		outPtr += sizeof(long);
		*(int *) outPtr = htonl((int) lopr);
		outPtr += sizeof(long);
		}
	    if (agentType & AGNT_CNTRL)
		{
		*(long *) outPtr = htonl((int) drvh);
		outPtr += sizeof(long);
		*(long *) outPtr = htonl((int) drvl);
		outPtr += sizeof(long);
		}
	    break;
	    }

	case AGNT_DOUBLE:
	    {
	    if (agentType & AGNT_GRAPH)
		{
		htond(hopr, *(double *)outPtr);
		outPtr += sizeof(double);
		htond(lopr, *(double *)outPtr);
		outPtr += sizeof(double);
		*(short *) outPtr = htons(prec);
		outPtr += sizeof(short);
		}
	    if (agentType & AGNT_CNTRL)
		{
		htond(drvh, *(double *)outPtr);
		outPtr += sizeof(double);
		htond(drvl, *(double *)outPtr);
		outPtr += sizeof(double);
		}
	    break;
	    }

	default:
	    {
	    printErr("Illegal agent data type %d\n", agentType);
	    return 0;
	    }
	}

    /*
    Now get the data field itself. We've been careful to make sure
    that our data types match the original Epics data types, so wipe
    out the embellishments and use the original routines.
    */

    agentType &= 0xff;
    reqType = mapOld2New(agentType);
    numReq = numElem;

    /*
    Take care of odd addresses for numeric data types on MC68000
    */

    if (((long)outPtr & 1) && (agentType != AGNT_STRING))
	{
	oddAddr = 1;
	outPtr++;
	}
    else
	oddAddr = 0;
    dbGetField(pAddr, reqType, outPtr, &options, &numReq, 0);
    switch (agentType)
	{
	case AGNT_SHORT:
	case AGNT_ENUM:
	    *(short *)outPtr = htons(*(short *)outPtr);
	    break;
	case AGNT_FLOAT:
	    htonf(*(float *)outPtr, *(float *)outPtr);
	    break;
	case AGNT_LONG:
	    *(int *)outPtr = htonl(*(int *)outPtr);
	    break;
	case AGNT_DOUBLE:
	    htond(*(double *)outPtr, *(double *)outPtr);
	    break;
	}
    if (oddAddr)
	{
	bcopy(outPtr, outPtr - 1, dbBufferSize(reqType, 0, numElem));
	outPtr--;
	}
    if (agentType != AGNT_STRING)
    	return ((outPtr - pOut) + dbBufferSize(reqType, 0, numElem));
    else
	return ((outPtr - pOut) + strlen(outPtr) + 1);
}
