/****************************************************************/
/*								*/
/*		BUILD STREAM DATA FOR AN ALARM MESSAGE		*/
/*								*/
/*	This routine builds a buffer consisting of the		*/
/*	null terminated point name followed by the null		*/
/*	terminated descriptor followed by the null termi-	*/
/*	nated value followed by the null terminated EGU		*/
/*	tag. If a string is empty, its null is still copied	*/
/*	to the buffer. Return the total number of bytes		*/
/*	in the buffer, including the nulls.			*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <vxWorks.h>
#include <string.h>

#include "dbDefs.h"
#include "dbCommon.h"
#include "link.h"
#include "db_field_log.h"
#include "recSup.h"
#include "dbRecDes.h"
#include "dbAccess.h"
#include "dataTypes.h"
#include "mapTypes.h"

extern struct fldDes **valList;

int getAlarmStream(struct dbCommon *recHdr, short recType, char *valBuff, char *outBuff)
{
    struct rset		*prset;
    char		*saveBuff;
    struct dbAddr	dbAddr;
    struct fldDes	*pfldDes;
    long		options = 0;
    long		numReq = 1;

    /*
    Set up by saving the original buffer pointer and then locating
    the RSET pointer for this point type.
    */

    saveBuff = outBuff;
    prset = GET_PRSET(recType);

    /*
    The name is easy. It's always in the same place.
    */

    strcpy(outBuff, recHdr -> name);
    outBuff += (strlen(outBuff) + 1);

    /*
    Get the descriptor. That's also in the header.
    */

    strcpy(outBuff, recHdr -> desc);
    outBuff += (strlen(outBuff) + 1);

    /*
    We need a struct dbAddr to get the value. All of these fields
    may not be necessary, but play it safe just to be sure.
    */

    pfldDes = valList[recType];
    dbAddr.record_type = recType;
    dbAddr.precord = (caddr_t) recHdr;
    dbAddr.pfield = (caddr_t) (valBuff);
    dbAddr.pfldDes = (caddr_t) pfldDes;
    dbAddr.no_elements = 1;
    dbAddr.field_type = pfldDes -> field_type;
    dbAddr.dbr_field_type = pfldDes -> dbr_field_type;
    dbAddr.field_size = pfldDes -> size;
    dbAddr.choice_set = pfldDes -> choice_set;
    dbAddr.special = pfldDes -> special;

    dbGetField(&dbAddr, mapOld2New(AGNT_STRING), outBuff, &options, &numReq, 0);
    outBuff += (strlen(outBuff) + 1);

    /*
    The last component is the EGU tag. Get that out of the RSET.
    Chop it to 7 bytes if its longer than that.
    */

    *outBuff = 0;
    if (prset)
	if (prset -> get_units)
	    (*(prset -> get_units))(&dbAddr, outBuff);
    *(outBuff + 7) = 0;
    outBuff += (strlen(outBuff) + 1);

    /*
    The number of bytes we created is outBuff minus its original value.
    */

    return (outBuff - saveBuff);
}
