/*rbMgt.c,v 1.1.1.1 1999/08/03 14:44:30 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		RING BUFFER MANAGEMENT FOR CHANNEL ACCESS	*/
/*								*/
/*	There is a ring buffer between all of the tasks that	*/
/*	can call dbPostEvents and the channel access server.	*/
/*	This file contains a set of routines that are used	*/
/*	to create, write into, and read from that ring buffer.	*/
/*	This set of routines is really some wrappers around	*/
/*	the routines in the vxWorks rngLib. We don't need a	*/
/*	wrapper for reading because there is only one reader.	*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#include <vxWorks.h>
#include <rngLib.h>
#include <semLib.h>
#include <string.h>
#include <stdio.h>
#include <inetLib.h>

#include "dbDefs.h"
#include "dbCommon.h"
#include "db_field_log.h"
#include "caRing.h"
#include "fast_lock.h"
#include "dbRecDes.h"
#include "pvd.h"
#include "rbMgt.h"
#include "rsrv.h"

extern SEM_ID		rsrvSem;

FAST_LOCK		ringLock;
static short		ringCount;
RING_ID			ringId = 0;

void gurngBufPut (RING_ID rid, char *buffer, int count);

/****************************************************************/
/*								*/
/*			TUNING PARAMETER			*/
/*								*/
/*	The following parameter defines the size of the ring	*/
/*	buffer. This buffer must be fairly substantial because	*/
/*	it is most commonly filled by the scan tasks and they	*/
/*	run at a higher priority than the Channel Access	*/
/*	server task that empties it.				*/
/*								*/
/****************************************************************/

#define CA_RING_SIZE	32768


/****************************************************************/
/*								*/
/*		INITIALIZE RING BUFFER MANAGEMENT		*/
/*								*/
/****************************************************************/

int caRbInit(void)
{

    /*
    Create the ring buffer.
    */

    if ((ringId = rngCreate(CA_RING_SIZE)) == NULL)
	{
	printErr("Can't allocate CA ring buffer.\n");
	return (0);
	}

    /*
    There are many writers into the ring buffer. Therefore, we have
    to make sure that there is only one writer at a time. The CA
    server is the only reader, so he doesn't have to worry about mutual
    exclusion.
    */

    FASTLOCKINIT(&ringLock);

    return (1);
}

/****************************************************************/
/*								*/
/*	WRITE AN EVENT MONITOR TO THE RING BUFFER		*/
/*								*/
/****************************************************************/

int caRbPut(FCB *pFCB, struct dbCommon *precord, caddr_t pfield, short length, short mask)
{
    eventRBH		ringHdr;
    struct dbCommon	*recHdr;
    short		ringType;

    /*
    If we are called before the ring buffer has been created,
    just ignore the call
    */

    if (!ringId)
	return 0;

    /*
    First, lock the ring buffer so that life will be stable for
    a moment.
    */

    FASTLOCK(&ringLock);

    /*
    Next, make sure that there's enough room in the buffer.
    */

    length = pFCB -> dbAddr.field_size * pFCB -> dbAddr.no_elements;
    if ((length + sizeof(ringHdr) + sizeof(short)) >= rngFreeBytes(ringId))
	{
	FASTUNLOCK(&ringLock);
	return (0);
	}

    ringCount = 0;
    ringType = EVENT_HDR;
    gurngBufPut(ringId, (char *) &ringType, sizeof(ringType));

    /*
    Build the event monitor ring buffer header.
    */

    recHdr = (struct dbCommon *) pFCB -> dbAddr.precord;
    if (mask & DBE_ALARM)
    	ringHdr.stamp = recHdr -> latm;
    else
	ringHdr.stamp = recHdr -> lptm;
    ringHdr.precord = precord;
    ringHdr.pfield = pfield;
    ringHdr.recType = pFCB -> dbAddr.record_type;
    ringHdr.pFldDes = (struct fldDes *) (pFCB -> dbAddr.pfldDes);
    ringHdr.length = length;
    ringHdr.mask = mask;
    ringHdr.stat = recHdr -> stat;
    ringHdr.sevr = recHdr -> sevr;
    ringHdr.ack = recHdr -> ack;

    /*
    Send the header
    */

    gurngBufPut(ringId, (char *) &ringHdr, sizeof(ringHdr));

    /*
    Send the data.
    */

    gurngBufPut(ringId, (char *) (pFCB -> dbAddr.pfield), length);

    /*
    Unlock the ring buffer
    */

    rngMoveAhead(ringId, ringCount);
    FASTUNLOCK(&ringLock);

    /*
    Smack the Channel Access server on the head to wake him up. He's the one
    that will pull stuff out of the ring buffer. Then we're done.
    */

    semGive(rsrvSem);
    return (1);
}


/****************************************************************/
/*								*/
/*	WRITE A RESOURCE STATUS CHANGE TO THE RING BUFFER	*/
/*								*/
/****************************************************************/

int reRbPut(resName, newState)
char	*resName;
short	newState;
{
    resrcRBH		ringHdr;
    short		ringType;

    /*
    If we are called before the ring buffer has been created,
    just ignore the call.
    */

    if (!ringId)
	return 0;

    /*
    First, lock the ring buffer so that life will be stable for
    a moment.
    */

    FASTLOCK(&ringLock);

    /*
    Next, make sure that there's enough room in the buffer.
    */

    if ((sizeof(ringHdr) + sizeof(short)) >= rngFreeBytes(ringId))
	{
	FASTUNLOCK(&ringLock);
	return (0);
	}

    ringCount = 0;
    ringType = RESRC_HDR;
    gurngBufPut(ringId, (char *) &ringType, sizeof(ringType));

    /*
    Build the resource status change header
    */

    ringHdr.newStat = newState;
    strcpy(ringHdr.resName, resName);

    /*
    Send the header and then the data.
    */

    gurngBufPut(ringId, (char *) &ringHdr, sizeof(ringHdr));

    /*
    Unlock the ring buffer
    */

    rngMoveAhead(ringId, ringCount);
    FASTUNLOCK(&ringLock);

    /*
    Smack the Channel Access server on the head to wake him up. He's the one
    that will pull stuff out of the ring buffer. Then we're done.
    */

    semGive(rsrvSem);
    return (1);
}


/****************************************************************/
/*								*/
/*	WRITE A DATA ARCHIVER OR ALARM MSG TO THE RING BUFFER	*/
/*								*/
/****************************************************************/

int caRbHandlerPut(pRec, pField, mask)
struct dbCommon		*pRec;
char			*pField;
short			mask;
{
    short		recType;
    short		offset;
    struct fldDes	**ppfldDes;
    struct recTypDes	*precTypeDes;
    handlerRBH		handlerHdr;

    /*
    If we are called before the ring buffer was created, just ignore it.
    */

    if (!ringId)
	return 0;

    /*
    If this call is for an alarm only and the alarm group is blank,
    then ignore it.
    */

    if ((mask == DBE_ALARM) && (pRec -> algp[0] == 0))
	return 0;

    /*
    We need to know the size of the data field. In order to do that,
    the initial implementation we have to use the pointer to the
    record and the pointer to the field to find the struct fldDes.
    That will give us the length of the field. We'll stick the mask,
    the pointer to the struct fldDes, the time, and the pointer to the
    record into the queue along with the status, severity, and
    the value. That will enable the CA server to send the messages
    on to everyone that needs them.
    */


    pvdGetRec(pRec -> name, &recType, &offset);
    offset = pField - (char *) pRec;
    handlerHdr.recType = recType;
    precTypeDes = GET_PRECTYPDES(recType);
    ppfldDes = precTypeDes -> papFldDes;
    for (recType = 0; recType < precTypeDes -> no_fields; recType++)
	if (ppfldDes[recType] -> offset == offset)
	    break;
    if (recType >= precTypeDes -> no_fields)
	return 0;

    /*
    Fill in the header structure
    */

    if (mask & DBE_ALARM)
    	handlerHdr.stamp = pRec -> latm;
    else
	handlerHdr.stamp = pRec -> lptm;
    handlerHdr.pFldDes = ppfldDes[recType];
    handlerHdr.pRecord = pRec;
    handlerHdr.mask = mask;
    handlerHdr.stat = pRec -> stat;
    handlerHdr.sevr = pRec -> sevr;
    handlerHdr.ack = pRec -> ack;
    handlerHdr.client = 0;	/* This message is for all clients */


    /*
    Now that we've found what we need, lock the queue so we can stick
    stuff into it.
    */

    FASTLOCK(&ringLock);

    /*
    Next, make sure that there's enough room in the buffer.
    */

    if ((ppfldDes[recType] -> size + sizeof(handlerHdr) + sizeof(short)) >= rngFreeBytes(ringId))
	{
	FASTUNLOCK(&ringLock);
	return (0);
	}

    ringCount = 0;
    offset = HNDLR_HDR;
    gurngBufPut(ringId, (char *) &offset, sizeof(offset));

    /*
    Send the header
    */

    gurngBufPut(ringId, (char *) &handlerHdr, sizeof(handlerHdr));

    /*
    Send the data.
    */

    gurngBufPut(ringId, pField, ppfldDes[recType] -> size);

    /*
    Unlock the ring buffer
    */

    rngMoveAhead(ringId, ringCount);
    FASTUNLOCK(&ringLock);

    /*
    Smack the Channel Access server on the head to wake him up. He's the one
    that will pull stuff out of the ring buffer. Then we're done.
    */

    semGive(rsrvSem);
    return (1);
}


/****************************************************************/
/*								*/
/*	INSERT AN ALARM OR A TREND MESSAGE FOR A SINGLE CLIENT	*/
/*								*/
/****************************************************************/

int caRbSinglePut(recType, pRec, pFldDes, mask, client)
short		recType;
struct dbCommon	*pRec;
struct fldDes	*pFldDes;
short		mask;
CCB		*client;
{
    handlerRBH	handlerHdr;
    short	msgType;

    /*
    If we are called before the ring buffer was created, just ignore it.
    */

    if (!ringId)
	return 0;

    /*
    If this call is for an alarm only and the alarm group is blank,
    then ignore it.
    */

    if ((mask == DBE_ALARM) && (pRec -> algp[0] == 0))
	return 0;

    /*
    Fill out the header structure first so that we can lock the ring
    buffer for the minimum amount of time possible.
    */

    if (mask & DBE_ALARM)
	handlerHdr.stamp = pRec -> latm;
    else
	handlerHdr.stamp = pRec -> lptm;
    handlerHdr.recType = recType;
    handlerHdr.pFldDes = pFldDes;
    handlerHdr.pRecord = pRec;
    handlerHdr.mask = mask;
    handlerHdr.stat = pRec -> stat;
    handlerHdr.sevr = pRec -> sevr;
    handlerHdr.ack = pRec -> ack;
    handlerHdr.client = client;		/* Restrict to just one client */

    /*
    Now lock the queue so we can put the stuff into it.
    */

    FASTLOCK(&ringLock);

    /*
    Make sure that there's enough room
    */

    if ((pFldDes -> size + sizeof(handlerHdr) + sizeof(short)) >= rngFreeBytes(ringId))
	{
	FASTUNLOCK(&ringLock);
	return (0);
	}

    /*
    Send the message type
    */

    ringCount = 0;
    msgType = HNDLR_HDR;
    gurngBufPut(ringId, (char *) &msgType, sizeof(msgType));

    /*
    Send the header
    */

    gurngBufPut(ringId, (char *) &handlerHdr, sizeof(handlerHdr));

    /*
    Send the data
    */

    gurngBufPut(ringId, (char *) pRec + pFldDes -> offset, pFldDes -> size);

    /*
    Unlock the ring buffer
    */

    rngMoveAhead(ringId, ringCount);
    FASTUNLOCK(&ringLock);

    /*
    And now, the traditional smack on the head for the CA server task
    */

    semGive(rsrvSem);
    return (1);
}


/****************************************************************/
/*								*/
/*	WRITE BYTES INTO RING BUFFER WITHOUT ADVANCING POINTER	*/
/*								*/
/*	All of the writes into the ring buffer are written	*/
/*	without advancing the output data pointer. This is	*/
/*	done to make sure that rsrvTask doesn't know that	*/
/*	any data is available in the ring buffer until all	*/
/*	of the pieces that we want to write have been written.	*/
/*								*/
/****************************************************************/


void gurngBufPut(rid, buffer, count)
RING_ID	rid;
char	*buffer;
int	count;
{
    while (count--)
	rngPutAhead(rid, *buffer++, ringCount++);
}
