/*mon.c,v 1.1.1.1 1999/08/03 14:44:36 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/************************************************************************/
/*                                                                      */
/*            State Notation Language program monitor                   */
/*                                                                      */
/*  This task is responsible for starting SNL programs, monitoring	*/
/*  their status, and program termination.				*/
/*                                                                      */
/************************************************************************/

#include <vxWorks.h>
#include <symLib.h>
#include <sysSymTbl.h>
#include <socket.h>
#include <in.h>
#include <errno.h>
#include <sys/fcntlcom.h>
#include <task_params.h>        /* For getting task priorities */
#include "mon_defs.h"

/* Tuning Parameters */

#define MAX_FDS		10		/* max number of clients we can handle */

/* Local data */

static int              rmfd;		/* fd for listen socket */
static fd_set           rmReadFds;	/* fd mask for select() */
static int              maxTcpFd = -1;

typedef struct {
	unsigned long	Inet;		/* Connected Inet address */
	int		fd;		/* connection fd */
	long		lastmsg;	/* last message report rx time */
} CONNECTION;

static	CONNECTION	resFds [MAX_FDS];

/* Message buffers */

static	MSG_HEADER	msgHead;
static	char		msgBody [1000];

static	int	DebugOn = FALSE;
static	int	Running = TRUE;

/* Module Lists - Keeps track of all modules loaded. */

typedef	struct {
    MODULE_NAME		Name;
    int			Location;	
} MODULE_LIST_ENTRY;
static	MODULE_LIST_ENTRY	ModuleList [MAX_MODULES];
static	int	NumModules;

/* Program List - Keeps track of all invokations of each module */

typedef	struct {	/* Program list entry */
    PROGRAM_NAME	OrigName;
    PROGRAM_NAME	ActualName;
    PARAMETERS		Parameters;
    RESOURCE		Resource;
    int			TaskId;
} TaskListEntry;
static	ProgListEntry	ProgList [MAX_PROGRAMS];
static	int	NumPrograms;	

/* Functions defined in this module */

	int	snlMonInit ();
static	void	snlMonTask ();
static	void	acceptNewClient ();
static	int	readClientMsg ();
static	void	doDisconnect ();
static	int	sendMessage ();

static	void	processRequest ();
static	void	processPing ();
static	void	processLoad ();
static	void	processStart ();
static	void	processStatus ();
static	void	processKill ();
static	void	processUnload ();
static	void	processTerminate ();

static	BOOL	AddModule ();
static	BOOL	ModTable ();

static	BOOL	AddProgram ();
static	BOOL	ProgTableFull ();

/*----------*/
int snlMonInit(Deb)
    int		Deb;
{
    DebugOn = Deb; /* start with debug off */

    /* get the current userid and password for later filesystem access */
    /* to the host. */

    remCurIdGet (UserId, Password);
  
    /* Spawn the snlMon task. For now, use the same priority as the */
    /* periodic scan task. */

    if (taskSpawn(SNLMON_NAME, SNLMON_PRI, VX_FP_TASK, SNLMON_STACK,
                   snlMonTask) == ERROR) {
        printErr("Can't spawn snlMon.\n");
        return (-1);
    }

    /* Initialize various things */

    NumModules = 0;
    NumPrograms = 0;

    /* Initialization complete */

    return(0);
}

/*----------*/
static void snlMonTask ()
{
    long            nextTime;
    long            delay;
    struct sockaddr_in serverAddr;    /* server's address   */
    short           serveport;
    int             sock;
    register int    status, i;
    fd_set          tempfds;
    int		    addrSize;

    if (rmfd != 0 && rmfd != ERROR)
        if ((status = close(rmfd)) == ERROR)
            printErr("snlMon: Unable to close open snlMon socket\n");

    /* Open a stream socket. */

    if ((rmfd = socket(AF_INET, SOCK_STREAM, 0)) == ERROR) {
        printErr("snlMon: Socket creation error\n");
        taskSuspend(0);
    }
    
    /* Zero the sock_addr structure */

    bzero (&serverAddr, sizeof(serverAddr)); 
    serverAddr.sin_family = AF_INET;
    serverAddr.sin_port = htons(SNLMON_PORT);

    /* get server port address */

    if (bind(rmfd, &serverAddr, sizeof(serverAddr)) == ERROR) {
        printErr("snlMon: Bind error\n");
        close(rmfd);
        taskSuspend(0);
    }
    addrSize = sizeof(serverAddr);
    if (getsockname (rmfd, &serverAddr, &addrSize)) {
        printErr("snlMon: getsockname error\n");
        close(rmfd);
        taskSuspend(0);
    }
    serveport = htons (serverAddr.sin_port);

    /* clear fd mask listen and accept new connections */

    FD_ZERO (&rmReadFds);
    FD_SET (rmfd, &rmReadFds); /* select on incoming connections */
    if (listen(rmfd, 0) == ERROR) {
        printErr("snlMon: Listen error\n");
        close(rmfd);
        taskSuspend(0);
    }

    while (Running) {
	tempfds = rmReadFds;
	status = select (sizeof(tempfds)*8, &tempfds,
                 NULL, NULL, NULL);
	if (DebugOn)
	    printf ("snlMon: select triggered, status = %d\n", status);
        if (status > 0) { /* One of the targets has activity */
            if (FD_ISSET (rmfd, &tempfds)) {
                acceptNewClient (rmfd);
                status--;
            }
            /* check the resource table fds for incoming signals */
	    for (i = 0; i < MAX_FDS && status; i++) {
                if (resFds[i].Inet && FD_ISSET (resFds[i].fd, &tempfds)) {
                    if (readClientMsg (resFds[i].fd))
			processRequest (resFds[i].fd);
		    else
			doDisconnect (resFds[i].fd);
                    status--;
                }
            }
        } else if (status < 0) { /* error processing */
            printErr("snlMon: error on select statement\n");
        }

    }

    /* Server has terminated so close all sockets */

    close (rmfd);
    for (i=0 ; i<MAX_FDS ; i++)
	if (resFds [i].fd != 0)
	    close (resFds [i].fd);

}

/*----------*/
static void acceptNewClient (rmfd)
int	rmfd;
{
    int     tempfd;
    struct  sockaddr_in newaddress;
    long    newInet;
    int	    i;
    int     blankIx, size;

    size = sizeof (newaddress);
    if ((tempfd = accept(rmfd, &newaddress, &size)) != ERROR) {
        /* scan the fd table for the connecting IP */
	newInet = ntohs(newaddress.sin_addr.s_addr);
        blankIx = MAX_FDS;
	for (i = 0; i < MAX_FDS; i++) {
	    if (resFds[i].Inet == newInet)
		return;
	    else if ((resFds[i].Inet == 0) && (blankIx == MAX_FDS)) {
		blankIx = i;
            }
	}
	if (i == MAX_FDS) {
            if (blankIx < MAX_FDS) {
		resFds[blankIx].Inet = newInet;
		resFds[blankIx].fd = tempfd;
		resFds[blankIx].lastmsg = tickGet ();
                FD_SET (tempfd, &rmReadFds);
		if (DebugOn)
		    printErr("snlMon: accepted new client, fd = %d\n", tempfd);
            } else {
  	        printErr("snlMon: accept out of resource fd map space\n");
		taskSuspend(0);
            }
        }
    }
    else {
        printErr("snlMon: Accept error, errno = %d\n", errno);
        taskSuspend(0);
    }
}

/*----------*/
static int readClientMsg (fd)
int	fd;
{
    int		i;
    int		bytesGot, bytesNeeded;
    char	*p;

    /* Read the message header */

    bytesNeeded = sizeof (msgHead);
    p = (char *)&msgHead;
    while (bytesNeeded > 0) {
	bytesGot = read (fd, p, bytesNeeded);
	if (bytesGot <= 0)
	    return 0;
	p += bytesGot;
	bytesNeeded -= bytesGot;
    }

    if (DebugOn)
	printf ("Head received: Type=%d, Length=%d, Info=%d\n",
			msgHead.Type, msgHead.Length, msgHead.Info);

    /* Read the message body */

    bytesNeeded = msgHead.Length;
    p = msgBody;
    while (bytesNeeded > 0) {
	bytesGot = read (fd, p, bytesNeeded);
	if (bytesGot <= 0)
	    return 0;
	p += bytesGot;
	bytesNeeded -= bytesGot;
    }

    if (DebugOn) {
	printf ("Message body: ");
	for (i=0 ; i<msgHead.Length ; i++)
	    printf ("%x ", msgBody [i] & 0xFF);
	printf ("\n");
    }

    return 1;
}

/*----------*/
static void doDisconnect (fd)
int	fd;
{
    int		i;

    if (DebugOn)
	printf ("Disconnecting, fd = %d\n", fd);

    /* find and close the fd struct */ 

    for (i = 0; i < MAX_FDS; i++) {
	if (resFds[i].fd == fd)
	    break;
    }
    resFds[i].Inet = 0;
    resFds[i].fd = 0;
    resFds[i].lastmsg = 0;

    /* close socket */

    close (fd);
    FD_CLR (fd, &rmReadFds);
}

/*----------*/
static int sendMessage (Type, Body, Length, Info, fd)

    int		Type, Length, Info, fd;
    char	*Body;

{
    MSG_HEADER	msgHeader;
    int		i;

    if (DebugOn) {
	printf ("Sending message: Type=%d, Length=%d, Info=%d, Msg = ",
		Type, Length, Info);
	for (i=0 ; i<Length ; i++)
	    printf ("%x ", *(Body + i));
	printf ("\n");
    }

    /* Send the message header */

    msgHeader.Type = Type;
    msgHeader.Length = Length;
    msgHeader.Info = Info;
    write (fd, (char *)&msgHeader, sizeof (msgHeader));

    /* Send the message body */

    if (Length > 0)
	write (fd, msgBody, Length);

    return 1;
}

/*----------*/
static void processRequest (fd)

    int		fd;

{
    switch (msgHead.Type) {
	case PING_MSG_TYPE:
	    processPing (fd);
	    break;
	case LOAD_MSG_TYPE:
	    processLoad (fd);
	    break;
	case START_MSG_TYPE:
	    processStart (fd);
	    break;
	case STATUS_MSG_TYPE:
	    processStatus (fd);
	    break;
	case KILL_MSG_TYPE:
	    processKill (fd);
	    break;
	case UNLOAD_MSG_TYPE:
	    processUnload (fd);
	    break;
	case TERMINATE_MSG_TYPE:
	    Running = 0;
	    break;
	default:
	    break;
    }
}

/*----------*/
static void processPing (fd)

    int		fd;

{
    if (DebugOn)
        printf ("Ping!  Info = %x\n", msgHead.Info);

    /* Echo the Info field of message as a reply */

    sendMessage (PING_REPLY_TYPE, NULL, 0, msgHead.Info, fd);
}

/*----------*/
static void processLoad (fd)

    int		fd;

{
    LOAD_MSG	*loadMsg;
    int		Location, Info;
		
    loadMsg = (LOAD_MSG *)msgBody;

    if (DebugOn)
	printf ("Received Load request: FileName = %s\n", loadMsg->FileName);

    /* Check if there is room in the module table.  If yes, then load the module */

    Info = FALSE;	/* Return status */
    if (!ModTableFull ()) {
	if ((Location = ld (0, TRUE, loadMsg->FileName)) != NULL) {
	    AddModule (loadMsg->FileName, Location);
	    if (DebugOn)
		printf ("%s loaded, Location = %x(%d)\n",
			loadMsg->FileName, Location, Location);
	    Info = TRUE;
	}
    } else {
	if (DebugOn)
	    printf ("Unable to load %s\n", loadMsg->FileName);
    }

    sendMessage (LOAD_REPLY_TYPE, NULL, 0, Info, fd);
}

/*----------*/
static void processStart (fd)

    int		fd;

{

    START_MSG	*startMsg;
    START_REPLY	startReply;
    int		i, Status, tid, Info;
    char	*EntryPoint, EntrySym [50];
    SYM_TYPE	pType;
    TASK_DESC	Desc;

    startMsg = (START_MSG *)msgBody;

    if (DebugOn) {
	printf ("Received Start request: ProgramName=%s, Parameters=%s\n",
		startMsg->ProgramName, startMsg->Parameters);
	printf ("Received Start request: StackSize=%d, Resource=%s\n",
		startMsg->StackSize, startMsg->Resource);
    }

    /* Lookup the address of the entry point and start the task */

    Info = FALSE;
    if (!ProgTableFull ()) {
	sprintf (EntrySym, "_%s", startMsg->ProgramName);
	Status = symFindByName (sysSymTbl, EntrySym, &EntryPoint, &pType);
	if (Status == OK) {
	    tid = seq (EntryPoint, startMsg->Parameters,
			startMsg->StackSize, startMsg->Resource);
	    if (tid > 0) {
		if (taskInfoGet (tid, &Desc) == OK) {
		    strcpy (startReply.ProgramName, Desc.td_name);
		    AddProgram (startMsg->ProgramName, Desc.td_name,
				startMsg->Parameters, Desc.td_id);
		    Info = TRUE;
		}
	    }
	}
    }

    sendMessage (START_REPLY_TYPE, (char *)&startReply, sizeof (startReply), Info, fd);
}

/*----------*/
static void processStatus (fd)

    int		fd;

{

    STATUS_MSG	*statusMsg;
    STATUS_REPLY	statusReply;

    statusMsg = (STATUS_MSG *)msgBody;

    printf ("Received Status request: TaskId = %d\n", statusMsg->TaskId);

    strcpy (statusReply.TaskName, "Fred the Task");
    statusReply.Status = 1234;
    statusReply.Priority = 1;
    strcpy (statusReply.InitState, "Delaware");
    strcpy (statusReply.PrevState, "New York");
    strcpy (statusReply.CurState, "Maryland");
    statusReply.TimeEnt = 2001;
    statusReply.TimeIn = 500;
    sendMessage (STATUS_REPLY_TYPE, (char *)&statusReply, sizeof (statusReply), 0, fd);
}

/*----------*/
static void processKill (fd)

    int		fd;

{

    KILL_MSG	*killMsg;

    killMsg = (KILL_MSG *)msgBody;

    printf ("Received Kill request: TaskId = %s\n", killMsg->TaskName);

    sendMessage (KILL_REPLY_TYPE, NULL, 0, 1, fd);
 }

/*----------*/
static void processUnload (fd)

    int		fd;

{

    UNLOAD_MSG	*unloadMsg;
    int		Info;

    unloadMsg = (UNLOAD_MSG *)msgBody;

    printf ("Received Unload request: ModuleName = %s\n", unloadMsg->ModuleName);

    if (unld (unloadMsg->ModuleName, 0) == OK) {
	if (DebugOn)
	    printf ("%s unloaded.\n", unloadMsg->ModuleName);
	Info = TRUE;
    } else {
	Info = FALSE;
	if (DebugOn)
	    printf ("Unable to unload %s\n", unloadMsg->ModuleName);
    }

    sendMessage (UNLOAD_REPLY_TYPE, NULL, 0, Info, fd);
}

/*----------------------- Utilities ----------------------*/

/*----------*/
static BOOL AddModule (Name, Location)

    char	*Name;
    int		Location;

{

    if (NumModules == MAX_MODULES)
	return FALSE;

    strcpy (ModuleList [NumModules].Name, Name);
    ModuleList [NumModules].Location = Location;
    NumModules++;

    return TRUE;
}

/*----------*/
static BOOL ModTableFull ()
{
    return NumModules == MAX_MODULES;
}

/*----------*/
static BOOL AddProgram (OrigName, ActualName, Parameters, Resource, TaskId)

    char	*OrigName, *ActualName, *Parameters, *Resource;
    int		TaskId;

{

    if (NumPrograms == MAX_PROGRAMS)
	return FALSE;

    strcpy (ProgList [NumPrograms].OrigName, OrigName);
    strcpy (ProgList [NumPrograms].ActualName, ActualName);
    strcpy (ProgList [NumPrograms].Parameters, Parameters);
    strcpy (ProgList [NumPrograms].Resource, Resource);
    ProgList [NumPrograms].TaskId = TaskId;
    NumPrograms++;

    return TRUE;
}

/*----------*/
static BOOL ProgTableFull ()
{
    return NumPrograms == MAX_PROGRAMS;
}

