/*mon_defs.h,v 1.1.1.1 1999/08/03 14:44:36 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#ifndef	INCmon_defsH
#define	INCmon_defsH

#define	SNLMON_PORT		8599

/* Various defines and types */

#define	MODULE_NAME_SIZE	50
#define	PROGRAM_NAME_SIZE	40
#define	TASK_NAME_SIZE		40
#define	PARAMETERS_SIZE		100
#define	RESOURCE_NAME_SIZE	20
#define	STATE_NAME_SIZE		40
#define	SS_NAME_SIZE		40
#define	CHAN_NAME_SIZE		32
#define	VARIABLE_NAME_SIZE	40

#define	MC68000	1
#if CPU != MC68000
#define	MAX_MODULES		100
#define	MAX_PROGRAMS		100
#else
#define MAX_MODULES		5
#define MAX_PROGRAMS		5
#endif

typedef	char	MODULE_NAME [MODULE_NAME_SIZE];
typedef	char	PROGRAM_NAME [PROGRAM_NAME_SIZE];
typedef	char	TASK_NAME [TASK_NAME_SIZE];
typedef	char	PARAMETERS [PARAMETERS_SIZE];
typedef	char	RESOURCE_NAME [RESOURCE_NAME_SIZE];
typedef	char	STATE_NAME [STATE_NAME_SIZE];
typedef	char	SS_NAME [SS_NAME_SIZE];
typedef	char	CHAN_NAME [CHAN_NAME_SIZE];
typedef	char	VARIABLE_NAME [VARIABLE_NAME_SIZE];


/************************************************************************/
/* 			     snlMon messages				*/
/************************************************************************/

typedef struct {
    long	Type;
    long	seq;		/* message sequence number */
    long	Length;
    long	Info;		/* Message type dependent */
} MSG_HEADER;

/************************************************************************/
/* PING: Used to ping the server.  Echos the Info field.		*/
/************************************************************************/

#define	PING_MSG_TYPE	1

#define	PING_REPLY_TYPE	(-PING_MSG_TYPE)
/* No message bodies for this message type */

/************************************************************************/
/* LOAD: Load an SNL program into the IOC				*/
/************************************************************************/

#define	LOAD_MSG_TYPE	2
typedef struct {
    MODULE_NAME	ModuleName;
} LOAD_MSG;

#define	LOAD_REPLY_TYPE	(-LOAD_MSG_TYPE)
/* No reply to this message. Info field contains OK or ERROR. */

/************************************************************************/
/* START: Start an SNL program	(maybe multiple state sets)		*/
/************************************************************************/

#define	START_MSG_TYPE	3
typedef struct {
    long		StackSize;
    PROGRAM_NAME	ProgramName;
    PARAMETERS		Parameters;
    RESOURCE_NAME	Resource;
} START_MSG;

#define	START_REPLY_TYPE (-START_MSG_TYPE)
typedef struct {
    char	ProgramName [PROGRAM_NAME_SIZE];
} START_REPLY;
/* Info field contains OK or ERROR */


/************************************************************************/
/* KILL PROGRAM: Kill a currently executing program's tasks.		*/
/************************************************************************/

#define	KILL_MSG_TYPE	4
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
    PARAMETERS		Parameters;
} KILL_MSG;

#define	KILL_REPLY_TYPE (-KILL_MSG_TYPE)
/* Info field contains OK or ERROR */

/************************************************************************/
/* UNLOAD PROGRAM: Unload a program.					*/
/************************************************************************/

#define	UNLOAD_MSG_TYPE	5
typedef struct {
   MODULE_NAME	ModuleName;
} UNLOAD_MSG;

#define	UNLOAD_REPLY_TYPE (-KILL_MSG_TYPE)
/* Info field contains OK or ERROR */

/************************************************************************/
/* TERMINATE SERVER: Terminate the snlMon server.			*/
/************************************************************************/

#define	TERMINATE_MSG_TYPE	6

#define	TERMINATE_MSG_REPLY	(-TERMINATE_MSG_TYPE)
/* Info field contains OK or ERROR */

/************************************************************************/
/* REPORT MODULES: Report the modules loaded.				*/
/************************************************************************/

#define	REPMOD_MSG_TYPE		7

#define	REPMOD_REPLY_TYPE	(-REPMOD_MSG_TYPE)
typedef struct {
    MODULE_NAME		ModuleName;
} REPMOD_REPLY;


/************************************************************************/
/* REPORT PROGRAMS: Report the programs running.			*/
/************************************************************************/

#define	REPPROG_MSG_TYPE	8

#define	REPPROG_REPLY_TYPE	(-REPPROG_MSG_TYPE)
/* Reply is a sequence of these structures.  The number of structures */
/* is indicated in the Info field. */
typedef struct repProg_Reply {
    long		Stat;
    PROGRAM_NAME	ProgramName;
    long		TaskId;
    PARAMETERS		Parameters;
    RESOURCE_NAME	Resource;
} REPPROG_REPLY;
#define	REPPROG_PENDING		0
#define	REPPROG_STARTED		1
#define	REPPROG_CHWAIT		2

/************************************************************************/
/* REPORT STATE SETS: Report the tasks for a program.			*/
/************************************************************************/

#define	REPSS_MSG_TYPE	9
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
} REPSS_MSG;

#define	REPSS_REPLY_TYPE	(-REPSS_MSG_TYPE)
/* Reply is a sequence of these structures.  The number of structures */
/* is indicated in the Info field. */
typedef struct {
    double	TimeEnt;
    double	TimeIn;
    long	TaskId;
    long	Stat;		/* ef_wait, delay, etc */
    long	Priority;
    STATE_NAME	InitState;
    STATE_NAME	PrevState;
    STATE_NAME	CurState;
    SS_NAME	StateSetName;
    long	Pad;
} REPSS_REPLY;

/************************************************************************/
/* REPORT CHANNELS: Report status of all channels for a program.	*/
/************************************************************************/

#define	REPCHANS_MSG_TYPE	10
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
} REPCHANS_MSG;

#define	REPCHANS_REPLY_TYPE	(-REPCHANS_MSG_TYPE)
/* Reply is a sequence of these structures.  The number of structures */
/* is indicated in the Info field. */
typedef struct {
    CHAN_NAME	ChanName;
    char	Value [16];
    long	Monitored;
    long	Connected;
    long	Stat;
    long	Severity;	
} REPCHANS_REPLY;


/************************************************************************/
/* REPORT VARIABLES: Report all variables for a program.		*/
/************************************************************************/

#define	REPVARS_MSG_TYPE	11
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
} REPVARS_MSG;

#define	REPVARS_REPLY_TYPE	(-REPVARS_MSG_TYPE)
/* Reply is a sequence of these structures.  The number of structures */
/* is indicated in the Info field. */
typedef	struct {
    long	Type;		/* Type of symbol, V_... */
    long	Size;		/* Size of indivual item (long, long, ...) */
    long	NumElements;	/* Number of elements in array */
    long	Offset;		/* Offset from beginning of structure */
    VARIABLE_NAME Name;		/* Symbol name */
				/* value follows Name */
} REPVARS_REPLY;

/************************************************************************/
/* REPORT A VARIABLE: Report the value of a variable.			*/
/************************************************************************/

#define	REPVAR_MSG_TYPE		12
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
    VARIABLE_NAME	VariableName;
} REPVAR_MSG;

#define	REPVAR_REPLY_TYPE	(-REPVAR_MSG_TYPE)
/* Reply is a sequence of bytes depending upon the variable type.  The number */
/* of bytes is indicated in the Info field. */

/************************************************************************/
/* CHANGE A VARIABLE: Change the value of a variable.			*/
/************************************************************************/

#define	CHVAR_MSG_TYPE		13
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
    VARIABLE_NAME	VariableName;
} CHVAR_MSG;
/* This structure is followed by the actual data */

#define	CHVAR_REPLY_TYPE	(-CHVAR_MSG_TYPE)
/* No body */


/************************************************************************/
/* SUSPEND A TASK: Suspend a running state set task.			*/
/************************************************************************/

#define	SUSPEND_MSG_TYPE		14
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
    TASK_NAME		TaskName;
} SUSPEND_MSG;

#define	SUSPEND_REPLY_TYPE	(-SUSPEND_MSG_TYPE)
/* No body */

/************************************************************************/
/* RESUME A TASK: Resume a suspended state set task.			*/
/************************************************************************/

#define	RESUME_MSG_TYPE		15
typedef struct {
    PROGRAM_NAME	ProgramName;
    long		TaskId;
    TASK_NAME		TaskName;
} RESUME_MSG;

#define	RESUME_REPLY_TYPE	(-RESUME_MSG_TYPE)
/* No body */


/************************************************************************/
/* snlmonlib function prototypes					*/
/************************************************************************/

extern	int	snlMonConnect (char *IOCName);
extern	void	snlMonDisconnect (void);
extern	int	snlMon_Kill (char *ProgramMame, long tid, char *Parameters);
extern	int	snlMon_Load (char *ModuleName);
extern	int	snlMon_Ping (int sendValue, int *ReceiveValue);
extern	int	snlMon_Resume (char *ProgramName, long tid, char *TaskName);
extern	int	snlMon_Start (char *ProgramName, char *Parameters, int StackSize,
			char *Resources, char *ActualName);
extern	int	snlMon_Suspend (char *ProgramName, long tid, char *TaskName);
extern	int	snlMonTerminate (void);
extern	int	snlMon_Unload (char *ModuleName);

extern	int	snlMon_ChangeVariable (char *ProgramName, long tid, char *VariableName,
				       char *NewVAlue, int Size);

extern	int	snlMon_ReportChannels (char *ProgramName, long tid, 
				       REPCHANS_REPLY **ChanList, int *NumChans);
extern	int	snlMon_ReportModules (REPMOD_REPLY **ModList, int *NumModules);
extern	int	snlMon_ReportPrograms (REPPROG_REPLY **, int *);
extern	int	snlMon_ReportStateSets (char *, long, REPSS_REPLY **, int *);
extern	int	snlMon_ReportVariables (char *ProgramName, long tid, 
					REPVARS_REPLY **Var_List, int *NumVars);
extern	int	snlMon_ReportVariable (char *ProgramName, long tid, char *VarName,
				       char **VarValue, int *VarLength);

#endif	INCmon_defsH
