/*seq_task.c,v 1.4 1994/04/21 14:15:54 epics Exp */

/**************************************************************************
			GTA PROJECT   AT division
	Copyright, 1990, The Regents of the University of California.
		         Los Alamos National Laboratory

	@(#)seq_task.c	1.8	2/13/92
	DESCRIPTION: Seq_tasks.c: Task creation and control for sequencer
	state sets.

	ENVIRONMENT: VxWorks
	HISTORY
04dec91,ajk	Implemented linked list of state programs, eliminating task
		variables.
11dec91,ajk	Made cosmetic changes and cleaned up comments.
19dec91,ajk	Changed algoritm in get_timeout().
***************************************************************************/


#include <vxWorks.h>
#include <inetLib.h>
#include <ioLib.h>
#include <semLib.h>
#include <taskLib.h>
#include <tickLib.h>
#include <string.h>
#include <logLib.h>
#include <stdio.h>

#include <sysseq.h>
#include <restable.h>
#include <resseq.h>
#include <resmgr.h>

/* Function declarations */
#if defined(ANSI) || defined(vxWorks)
LOCAL	VOID ss_task_init(SPROG *, SSCB *);
LOCAL	long get_timeout(SSCB *);
LOCAL	VOID seq_resChange (SPROG *pSP);
#else
LOCAL	VOID ss_task_init();
LOCAL	long get_timeout();
LOCAL	VOID seq_resChange ();
#endif	ANSI

#define		TASK_NAME_SIZE 10

#define	MAX_TIMEOUT	(1<<30) /* like 2 years */

/* sequence redundancy definitions */
/* redundancy structures */
typedef struct {
	short		current_state;	/* current state index */
	short		next_state;	/* next state index */
	short		prev_state;	/* previous state index */
	short		error_state;	/* error state index (-1 if none defined) */
	short		trans_num;	/* highest priority trans. # that triggered */
/* action complete is always true if an action has completed at the point where the
** state exchange occurs. * /
	BOOL		action_complete;/ * TRUE if action complete
					   FALSE if waiting for an event * /
/ * the events array contains the events that have occurred since the state was entered
**	bitMask		events[NWRDS];	/ * outstanding events * /
/ * the pMask value is set to the state_set[current_state].eventMask when the
** state is received.
**	bitMask		*pMask;		/ * current event mask * /
/ * the timeout values are set by calling the new state's delay function after the
** time is updated from the time in this structure.
**	int		ndelay;		/ * number of delays activated * /
**	ULONG		timeout[MAX_NDELAY]; / * time-out for delay() */
	ULONG		time;		/* time that a state was entered */
} STATE_RPT;

typedef struct {
    unsigned short	num_states;  /* number of states */
    STATE_RPT		states [1];  /* state set data instances (actually
					more than one) */
    unsigned short	data_size;   /* size of sequence program data area */
    char		seq_data [1]; /* sequence program data area */
} STATE_UPDATE; 

int	SEQDEBUG;	/* turn on debug info */
/*
 * sequencer() - Sequencer main task entry point.
 *  */
int sequencer(pSP, stack_size, ptask_name, resName)
SPROG		*pSP;	/* ptr to original (global) state program table */
int		stack_size;	/* stack size */
char		*ptask_name;	/* Parent task name */
char		*resName;
{
	SSCB		*pSS;
	int		nss, task_id;
	char		task_name[TASK_NAME_SIZE+10];
	extern		VOID ss_entry();
	MACRO		*pMac;

	pSP->task_id = taskIdSelf(); /* my task id */
	pSP->prog_time = MAX_TIMEOUT; /* me vs partner tick delta */

	/* Add the program to the state program list */
	seqAddProg(pSP);

	/* Set init state of each additional state set task */
	pSS = pSP->sscb + 1;
	for (nss = 1; nss < pSP->nss; nss++, pSS++)
	{
		pSS->current_state = 0;
	}

	/* register the program with sequence redundancy in resMgr */
	if (resName)
	{
		char *params = malloc (256);
		if(params == 0)
		{
		    logMsg("Unable to malloc params memory\n", 0,0,0,0,0,0);
		    taskSuspend(taskIdSelf());
		}

		*params = 0; /* clear out string */
		for (nss = 0, pMac = pSP->mac_ptr; nss < MAX_MACROS; nss++, pMac++)
		{
			if (pMac->name != NULL)
				strncat (params, pMac->name, 256-strlen(params));
			if (pMac->value != NULL)
				strncat (params, pMac->value, 256-strlen(params));
		}
if (SEQDEBUG == 5)
	logMsg ("Registering: %s %s %s\n", (int)pSP->name, (int)resName, (int)params, 0, 0, 0);
        	pSP->seq_token = resSeqInit (pSP->name, resName, params, seq_resChange, (long)pSP);
		free (params); 
        }
	else
	{
		pSP->seq_token = 0;
	}

	/*
	 * Connect to database channels & initiate monitor requests.
	 * Returns here immediately if "connect" option is not set (-c),
	 * otherwise, waits for all channels to connect (+c).
	 */
	seq_connect(pSP);

	/* Additional state set task names are derived from the first ss */
/*	if (strlen(ptask_name) > TASK_NAME_SIZE)
**		ptask_name[TASK_NAME_SIZE] = 0;
*/
	/* Create each additional state set task */
	pSS = pSP->sscb + 1;
	for (nss = 1; nss < pSP->nss; nss++, pSS++)
	{
		/* Form task name from program name + state set number */
		sprintf(task_name, "%.*s_%d", TASK_NAME_SIZE, ptask_name, nss);

		/* Spawn the task */
		task_id = taskSpawn(
		  task_name,				/* task name */
		  SPAWN_PRIORITY+pSS->task_priority,	/* priority */
		  SPAWN_OPTIONS,			/* task options */
		  stack_size,				/* stack size */
		  (FUNCPTR)ss_entry,				/* entry point */
		  (int)pSP, (int)pSS, 0,0,0,0,0,0,0,0);	/* pass 2 parameters */

		seq_log(pSP, "Spawning task %d: \"%s\"\n", task_id, (int)task_name, 0,0,0,0,0,0);
	}

	/* First state set jumps directly to entry point */
	ss_entry(pSP, pSP->sscb);
	return 0;
}
/*
 * ss_entry() - Task entry point for all state sets.
 * Provides the main loop for state set processing.
 */
VOID ss_entry(pSP, pSS)
SPROG	*pSP;
SSCB	*pSS;
{
	BOOL		ev_trig;
	STATE		*pST, *pStNext;
	long		delay;
	int		i;
	char		*pVar;
	int		seq_token;
	int		resstate;
	int		activity;
	STATE_UPDATE	*statePtr;
	unsigned short	stateLen;
        int		ss;
	STATE_RPT	*srp = 0;
        SSCB		*ssp = 0;
	long		now;
	int		wasSlave = 0;

	pSS->task_id = taskIdSelf();

	/* Initialize CA for all tasks except the main task */
	if (pSS->task_id != pSP->task_id)
		ss_task_init(pSP, pSS);

	/* Initialize state set to enter the first state */
	pST = pSS->states;
	seq_token = pSP->seq_token;

	/* Use the event mask for this state */
	pSS->pMask = (pST->eventMask);

	/* Local ptr to user variables (for reentrant code only) */
	pVar = pSP->user_area;

	/*
	 * ============= Main loop ==============
	 */
	while (1)
	{
		/* Clear event bits */
		semTake(pSP->caSemId, WAIT_FOREVER); /* Lock CA event update */
		for (i = 0; i < NWRDS; i++)
			pSS->events[i] = 0;
		semGive(pSP->caSemId); /* Unlock CA event update */

		pSS->time = tickGet(); /* record time we entered this state */

		while (seq_token && resSeqStatus (seq_token, &activity) != RES_MSTR)
                {
			/* indicate we were slave for channel reconnect test */
			wasSlave = 1;

			/* only the main task installs the new state information */
			if ((activity == READNEWSTATE) && (pSS->task_id == pSP->task_id))
                	{
				now = tickGet (); /* get the time for adjustments */
				/* get size of the state report */
				stateLen = sizeof(unsigned short) + (pSP->nss * sizeof(STATE_RPT))
					   + sizeof(unsigned short) + pSP->user_size;
				statePtr = malloc(stateLen);
				if (statePtr)
				{
					if (resSeqGetState (seq_token, (char *)statePtr, stateLen) != RES_MSTR)
					{
						/* install the new stuff */
						if (statePtr->num_states == pSP->nss)
						{
							for (ss=0, srp = statePtr->states, ssp = pSP->sscb; ss < pSP->nss; ss++, srp++,ssp++)
							{
								ssp->current_state = srp->current_state;
								ssp->next_state = srp->next_state;
								ssp->prev_state = srp->prev_state;
								ssp->error_state = srp->error_state;
								ssp->trans_num = srp->trans_num;
								ssp->time = srp->time + now;
if (SEQDEBUG == 5)
	logMsg ("%ld seq_task: receive new state %d, time %ld\n", tickGet(), srp->current_state, ssp->time, 0, 0, 0);
							}
							memcpy (pSP->user_area, ((unsigned short *)srp)+1, pSP->user_size);	
                                		}
                                	}
					free (statePtr);
				}
				else
				{
				    logMsg("Unable to malloc statePtr\n", 0,0,0,0,0,0);
				    taskSuspend(taskIdSelf());
				}
			}

			/* Wait til syncsem */
			semTake(pSS->syncSemId, MAX_TIMEOUT );
		}

		/* set up the pointers again, since the state may have changed */
		pST = pSS->states + pSS->current_state;
		/* Use the event mask for this state */
		pSS->pMask = (pST->eventMask);

		/* We are running the program, send the current state set to partner */

		if (seq_token)
                {
			/* if we are becoming master, wait for reconnect of all channels */
			if (wasSlave)
			{
				while (pSP->conn_flag && (pSP->conn_count < pSP->nchan))
				{
					/* was "ca_pend_io (0.5);", but this loops for imorted contexts */
					taskDelay (15);
				}
if (SEQDEBUG == 5)
	logMsg ("%ld seq_task: becoming master, all channels connected\n", tickGet(), srp->current_state, ssp->time, 0, 0, 0);
				wasSlave = 0;
			}
			else
			{
				seq_sendState (pSP);
			}
			/* check for return to slave status */
			if ((resstate = resSeqStatus (seq_token, &activity)) != RES_MSTR)
			{
if (SEQDEBUG == 5)
	logMsg ("%ld seq_task: no longer master! %d, time %ld\n", tickGet(), resstate, ssp->time, 0, 0, 0);
				continue;
			}
		}

		/* Call delay function to set up delays */
		pSS->ndelay = 0;
		pST->delay_func(pSP, pSS, pVar);

		/* Generate a phoney event.
		 * This guarantees that a when() is always executed at
		 * least once when a state is entered. */
		semGive(pSS->syncSemId);

		/*
		 * Loop until an event is triggered, i.e. when()
		 * returns TRUE.
		 */
		do {
			ev_trig = FALSE;
			/* Wake up on CA event, event flag, or expired time delay */
			/* Wake up on CA event, event flag, or expired time delay */
			delay = get_timeout(pSS);
			if (delay > 0)
				semTake(pSS->syncSemId, delay);
			delay = get_timeout(pSS);

			/* check redundancy status after wakeup (we were master
			** when we went to sleep ! ) */
			if (seq_token)
			{
				if ((resstate = resSeqStatus (seq_token, &activity)) != RES_MSTR)
				{
					/* get back into waiting mode */
					continue;
				}
				if (activity == SENDSTATEDATA)
				{
					seq_sendState (pSP);
					/* check for return to slave status */
					if ((resstate = resSeqStatus (seq_token, &activity)) != RES_MSTR)
					{
if (SEQDEBUG == 5)
	logMsg ("%ld seq_task: not master! %d, time %ld\n", tickGet(), resstate, ssp->time, 0, 0, 0);
						continue;
					}
				}
                        }
			else
			{
				resstate = RES_MSTR;
			}
			/* Apply resource lock: any new events coming in will
			   be deferred until next state is entered */
			semTake(pSP->caSemId, WAIT_FOREVER);

			/* Call the event function to check for an event trigger.
			 * Everything inside the when() statement is executed.
			 */
			ev_trig = (*(pST->event_func))(pSP, pSS, pVar);

			/* Unlock CA resource */
			semGive(pSP->caSemId);

		} while ((!ev_trig) && (resstate == RES_MSTR));
		if (ev_trig) {
			/*
			 * An event triggered:
			 * execute the action statements and enter the new state.
			 */

			/* Change event mask ptr for next state */
			pStNext = pSS->states + pSS->next_state;
			pSS->pMask = (pStNext->eventMask);

			/* Execute the action for this event */
			pSS->action_complete = FALSE;
			pST->action_func(pSP, pSS, pVar);

			/* Flush any outstanding DB requests */
			ca_flush_io ();

			/* Change to next state */
			pSS->prev_state = pSS->current_state;
			pSS->current_state = pSS->next_state;
			pST = pSS->states + pSS->current_state;
			pSS->action_complete = TRUE;
		}
	}
}
/* Initialize state-set tasks */
LOCAL VOID ss_task_init(pSP, pSS)
SPROG	*pSP;
SSCB	*pSS;
{
	/* Import Channel Access context from the main task */
	ca_import(pSP->task_id);

	return;
}
/* Return time-out for delay() */
LOCAL long get_timeout(pSS)
SSCB	*pSS;
{
	int		ndelay;
	ULONG		timeout, timeoutMin;	/* expiration clock time */
	ULONG		now;

	if (pSS->ndelay == 0)
		return MAX_TIMEOUT;

	now = tickGet();
	timeoutMin = MAX_TIMEOUT; /* start with largest timeout */

	/* Find the minimum abs. timeout (0 means already expired) */
	for (ndelay = 0; ndelay < pSS->ndelay; ndelay++)
	{
		timeout = pSS->timeout[ndelay];
		if (timeout == 0)
			continue;	/* already expired */
		else if (timeout - now < timeoutMin)
		{
			timeoutMin = timeout - now;
		}
	}

	return timeoutMin;
}
/* Set-up for delay() on entering a state.  This routine is called
by the state program for each delay in the "when" statement  */
VOID seq_start_delay(pSP, pSS, delay_id, delay)
SPROG	*pSP;
SSCB	*pSS;
int	delay_id;		/* delay id */
float	delay;
{
	int		td_tics;

	/* Note: the following 2 lines could be combined, but doing so produces
	 * the undefined symbol "Mund", which is not in VxWorks library */
	td_tics = delay*60.0;
	if (td_tics > tickGet() - pSS->time) /* only set up unexpired timers */
		pSS->timeout[delay_id] = pSS->time + td_tics;
	else /* timeout has already expired, mark as expired */
		pSS->timeout[delay_id] = 0;
	delay_id += 1;
	if (delay_id > pSS->ndelay)
		pSS->ndelay = delay_id;
	return;
}

/* Test for time-out expired */
BOOL seq_test_delay(pSP, pSS, delay_id)
SPROG	*pSP;
SSCB	*pSS;
int	delay_id;
{
	if (pSS->timeout[delay_id] == 0)
		return TRUE; /* previously expired t-o */

	if (tickGet() - pSS->time >= pSS->timeout[delay_id] - pSS->time)
	{
		pSS->timeout[delay_id] = 0; /* mark as expired */
		return TRUE;
	}
	return FALSE;
}
/*
 * Delete the state set tasks and do general clean-up.
 * General procedure is:
 * 1.  Suspend all state set tasks except self.
 * 2.  Call the user program's exit routine.
 * 3.  Delete all state set tasks except self.
 * 4.  Delete semaphores, close log file, and free allocated memory.
 * 5.  Return, causing self to be deleted.
 *
 * This task is run whenever ANY task in the system is deleted.
 * Therefore, we have to check the task belongs to a state program.
 * With VxWorks 5.0 we need to change references to the TCBX to handle
 * the Wind kernel.
 */
#ifdef	V5_vxWorks
int sprog_delete(tid)
int		tid;
{
	int		nss, tid_ss;
	SPROG		*pSP, *seqFindProg();
	SSCB		*pSS;
	MACRO		*pMac;

#else
int sprog_delete(pTcbX)
TCBX	*pTcbX;	/* ptr to TCB of task to be deleted */
{
	int		tid, nss, tid_ss;
	SPROG		*pSP, *seqFindProg();
	SSCB		*pSS;

	tid = pTcbX->taskId;
#endif
	pSP = seqFindProg(tid);
	if (pSP == NULL)
		return 0; /* not a state program task */

#ifdef	DEBUG
	logMsg("Delete %s: pSP=%d=0x%x, tid=%d\n",
	 pSP->name, pSP, pSP, tid);
#endif

	/* Is this a real sequencer task? */
	if (pSP->magic != MAGIC)
	{
#ifdef DEBUG
		logMsg("  Not main state program task\n");
#endif
		return -1;
	}

	/* Suspend all state set tasks except self */
	pSS = pSP->sscb;
#ifdef	DEBUG
	logMsg("   Suspending state set tasks:\n");
#endif	DEBUG
	for (nss = 0; nss < pSP->nss; nss++, pSS++)
	{
		tid_ss = pSS->task_id;
		if ( (tid_ss > 0) && (tid != tid_ss) )
		{
#ifdef	DEBUG
			logMsg("    suspend task: tid=%d\n", tid_ss);
#endif	DEBUG
			taskSuspend(tid_ss);
		}
	}

	/* Call user exit routine (only if task has run) */
	if (pSP->sscb->task_id > 0)
	{
#ifdef	DEBUG
		logMsg("   Call exit function\n");
#endif	DEBUG
		pSP->exit_func(pSP, pSP->user_area);
	}

	/* Close the log file */
	if (pSP->logFd > 0 && pSP->logFd != ioGlobalStdGet(1))
	{
#ifdef	DEBUG
		logMsg("Closing log fd=%d\n", pSP->logFd);
#endif	DEBUG
		if (!isatty(pSP->logFd))
		    close(pSP->logFd);
		pSP->logFd = ioGlobalStdGet(1);
	}

	/* Remove the state program from the state program list */
	seqDelProg(pSP);

	/* Remove from resmgr functions */
	if (pSP->seq_token)
	{
		char *params = malloc (256);
		if(params == 0)
		{
		    logMsg("Unable to malloc params memory\n", 0,0,0,0,0,0);
		    taskSuspend(taskIdSelf());
		}

		*params = 0; /* clear out string */
		for (nss = 0, pMac = pSP->mac_ptr; nss < MAX_MACROS; nss++, pMac++)
		{
			if (pMac->name != NULL)
				strncat (params, pMac->name, 256-strlen(params));
			if (pMac->value != NULL)
				strncat (params, pMac->value, 256-strlen(params));
		}
if (SEQDEBUG == 5)
	logMsg ("Deregistering: %s %s\n", (int)pSP->name, (int)params, 0, 0, 0, 0);
 		resSeqDelete (pSP->name, params);
		free (params); 
        }

	/* Delete state set tasks (except self) & delete their semaphores */
	pSS = pSP->sscb;
	for (nss = 0; nss < pSP->nss; nss++, pSS++)
	{
		tid_ss = pSS->task_id;
		if ( tid_ss > 0)
		{
			if ( (tid != tid_ss) && (tid_ss > 0) )
			{
#ifdef	DEBUG
				logMsg("   delete ss task: tid=%d\n", tid_ss);
#endif	DEBUG
				taskDelete(tid_ss);
			}
			semDelete(pSS->syncSemId);
			if (!pSP->async_flag)
				semDelete(pSS->getSemId);
		}
	}

	/* Delete program-wide semaphores */
	semDelete(pSP->caSemId);

	/* Free the memory that was allocated for the task area */
#ifdef	DEBUG
	logMsg("free pSP->dyn_ptr=0x%x\n", pSP->dyn_ptr);
#endif	DEBUG
	taskDelay(5);
	free(pSP->dyn_ptr);
	
	return 0;
}
/* VxWorks Version 4 only */
#ifndef	V5_vxWorks

/* seq_semTake - take semaphore:
 * VxWorks semTake() with timeout added. (emulates VxWorks 5.0)
 */
VOID seq_semTake(semId, timeout)
SEM_ID		semId;		/* semaphore id to take */
long		timeout;	/* timeout in tics */
{
	int		dummy;

	if (timeout == WAIT_FOREVER)
		vrtxPend(&semId->count, 0, &dummy);
	else if (timeout == NO_WAIT)
		semClear(semId);
	else
		vrtxPend(&semId->count, timeout, &dummy);
}
#endif	V5_vxWorks

/*
 * seq_resChange() - Wake up the state sets to check resource change
 */
VOID seq_resChange (pSP)
SPROG		*pSP;
{
	SSCB		*pSS;
	int		nss;

	pSS = pSP->sscb;
	/* For all state sets: */
	for (nss = 0; nss < pSP->nss; nss++, pSS++)
	{
		/* Apply resource lock */
		semTake(pSP->caSemId, WAIT_FOREVER);

		semGive(pSS->syncSemId); /* wake up the ss task */

		/* Unlock resource */
		semGive(pSP->caSemId);
	}
}

int seq_sendState (pSP)
SPROG		*pSP;
{
    STATE_UPDATE *statePtr;
    unsigned short stateLen;
    int		ss;
    STATE_RPT	*srp;
    SSCB	*ssp;

    /* build the state report */
    stateLen = sizeof(unsigned short) + (pSP->nss * sizeof(STATE_RPT))
			   + sizeof(unsigned short) + pSP->user_size;
    statePtr = malloc(stateLen);
    if (statePtr)
    {
	statePtr->num_states = pSP->nss;
	for (ss=0, srp = statePtr->states, ssp = pSP->sscb; ss < pSP->nss; ss++, srp++,ssp++)
	{
	    srp->current_state = ssp->current_state;
	    srp->next_state = ssp->next_state;
	    srp->prev_state = ssp->prev_state;
	    srp->error_state = ssp->error_state;
	    srp->trans_num = ssp->trans_num;
	    srp->time = ssp->time - tickGet ();
	}
if (SEQDEBUG == 5)
	logMsg ("Send new state: state %d, time %ld\n", srp->current_state, srp->time, 0, 0, 0, 0);
	*(unsigned short *)srp = pSP->user_size;
	memcpy (((unsigned short *)srp)+1, pSP->user_area, pSP->user_size);
	resSeqSendState (pSP->seq_token, (char *)statePtr, stateLen);
	free (statePtr);
	return 1;
    }
    else
    {
	logMsg ("Unable to send current state, Malloc failure.\n", 0, 0, 0, 0, 0, 0);
	taskSuspend(taskIdSelf());
    }
    return 0;
}
