/* svcproc.c - service dispatching for client/server programs.
**	This program is for systems without inetd, and waits for
**	a connect attempt on the listen socket, then launches a
**	server program based on the first message on the new socket.
**	The program will read the message and exec the program found in
**	the service.conf file.
*/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <sys/types.h>
#ifdef WIN32
#include <winsock.h>
#include <process.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#endif WIN32

#include "msgMon.h"

#include "inet_ntoa.h"
#include "tisservice.h"

#define TISSERVERPORT 8001

SvcReq	inmsg;		/* input message information */
char *newargv [20];	/* up to 20 args */

char	inpstring [81];

#ifndef WIN32
#define SOCKET	int
#endif WIN32

int launch (SOCKET newsock);
int respond (SOCKET bsock, struct sockaddr_in *client_addr);

int main (argc, argv)
{
    int ix, ix2;
    int	err;
    int client_len;
    struct sockaddr_in client_addr;
#ifdef WIN32
    WSADATA wsaData; 
    WORD    wVersionRequested = MAKEWORD(1, 1);
#endif WIN32
    SOCKET newsock;
    SOCKET lsock;
    SOCKET bsock;
    int stat;
    fd_set  readfds;
    struct sockaddr_in sa;
    int salen;
    struct hostent *ha;
    int true = 1;

    tisOpenLog ("svcproc", 6, 2, NULL, NULL);
    tisSysLog (0, 0, "TIS Service Location and Launch process started");

#ifdef WIN32
    err = WSAStartup(wVersionRequested, &wsaData); 
 
    if (err != 0)
    {
	/* Tell the user that we couldn't find a useable */ 
	/* winsock.dll.     */ 
	tisSysLog (0, 0, "TIS Service can't open winSock");
	exit (1);
    }
#endif WIN32

    /* create the UDP input socket */
    bsock = socket(PF_INET, SOCK_DGRAM, 0);
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = 0;
    sa.sin_port = htons(TISSERVERPORT);
    if (bind (bsock, (struct sockaddr *)&sa, sizeof(sa)) != 0)
    {
#ifdef WIN32
	errno = WSAGetLastError ();
#endif WIN32
	tisSysLog (0, 0, "cannot bind broadcast socket, err=%d\n", errno);
	exit (1);
    }

    /* create the TCP listen socket */
    lsock = socket(PF_INET, SOCK_STREAM, 0);
    ha = gethostbyname("localhost");
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = 0;
    sa.sin_port = htons(TISSERVERPORT);
    if (bind (lsock, (struct sockaddr *)&sa, sizeof(sa)) == 0)
    {
	if (listen (lsock, 4) != 0)
	    exit (0);
    }
    else
    {
#ifdef WIN32
	errno = WSAGetLastError ();
#endif WIN32
	tisSysLog (0, 0, "cannot bind socket, err=%d\n", errno);
	exit (1);
    }

    /* loop - wait for accept, UDP message
    **   if accept,
    **	    get message,
    **	    spawn process for requested service.
    **   if UDP
    **      look up service,
    **      reply if found.
    */
    while (1)
    {
	FD_ZERO (&readfds);
	FD_SET (bsock, &readfds);
	FD_SET (lsock, &readfds);
	stat = select (64, &readfds, NULL, NULL, 0);
	if (stat < 0)
	{
	    perror ("Error on select");
	    exit (0);
	}
	    
	if (FD_ISSET (bsock, &readfds))
	{
	    /* get message, exit if error or insufficient data */
	    client_len = sizeof(client_addr);
	    if ((ix = recvfrom(bsock, (char *)&inmsg, sizeof(inmsg), 0, (struct sockaddr *)&client_addr, &client_len)) < 0)
	    {
#ifdef WIN32
		errno = WSAGetLastError ();
		if (errno != WSAEINTR)
#else
		if (errno != EINTR)
#endif WIN32
		{
		    tisSysLog (0, 0, "Error receiving UDP message size %d, %m", ix);
		    exit(0);
		}
	    }
	    if (ix < sizeof(inmsg))
	    {
		tisSysLog (0, 0, "UDP: short message received\n");
	    }
	    else if ((ntohs(inmsg.type) == SVCREQ) && (ntohl(inmsg.sftlvl) == SOFTWARE_LEVEL))
	    {
		respond (bsock, &client_addr);
	    }
	}
	if (FD_ISSET (lsock, &readfds))
	{
	    salen = sizeof(sa);
	    if ((newsock = accept (lsock, (struct sockaddr *)&sa, &salen)) < 0)
	    {
#ifdef WIN32
		errno = WSAGetLastError ();
		if (errno != WSAEINTR)
#else
		if (errno != EINTR)
#endif WIN32
		    exit (0);
	    }

	    /* get message, exit if error */
	    client_len = sizeof(client_addr);
	    for (ix = 0; ix < sizeof(inmsg); ix += ix2)
	    {
		ix2 = recv (newsock, ((char*)&inmsg)+ix, sizeof(inmsg)-ix, 0);
		if ( ix2 < 0)
		{
		    err = WSAGetLastError ();
		    tisSysLog (0, 0, "Error reading message %m");
		    exit (1);
		}
	    }
	    if ((inmsg.type == SVCREQ) && (ntohl (inmsg.sftlvl) == SOFTWARE_LEVEL))
	    {
		launch (newsock);
	    }
	}
    }
    return (0);
}

int launch (SOCKET newsock)
{
    int ix;
    char *sp;
    FILE *fp;
    char *cp;
    char *execname;
#ifdef WIN32
    STARTUPINFO startUpInfo;
    PROCESS_INFORMATION procInfo;
    SECURITY_ATTRIBUTES sa;
    BOOL success;
    DWORD lastError;
    char commandLine[260];
#endif WIN32

    /* look up requested service, exec it if found */
#ifdef WIN32
    fp = fopen ("\\tis4000db\\tisservice.conf", "rt");
#else
    fp = fopen ("/tis4000db/tisservice.conf", "rt");
#endif WIN32
    if (fp)
    {
	while (!feof (fp))
	{
            ix = 0;
	    while (!feof (fp))
	    {
		inpstring [ix] = fgetc(fp);
		if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
 		}
		ix++;
 		if (ix == 80)
		{
		    inpstring[79] = 0;
		    tisSysLog (0, 0, "bad string in config file %s", inpstring);
		}
	    }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && *sp == ' ')
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */
	    /* process input- line is formatted as
	    ** <service_name> <executable> <args-as in inetd.conf>
	    */
	    cp = strchr (sp, ' ');
	    if (cp)
	    {
		*cp = 0;
		if (strcmp (sp, inmsg.service) == 0)
		{
#ifdef WIN32
		    fclose (fp);
#else
		    close (fp);
#endif WIN32
		    /* exec the image */
		    sp = cp+1; /* scan the executable name */
		    while (*sp && *sp == ' ')
			sp++;
		    if (*sp == 0)
		    {
			/* error, no program image spec */
			tisSysLog (0, 0, "Missing program in %s", inpstring);
			exit (1); /* bad string in file. */
		    }
		    execname = sp;
		    while (*sp && *sp != ' ')
			sp++;
		    if (*sp)
			*sp++ = 0;
#ifdef WIN32
		    _fileinfo = 1;
		    GetStartupInfo (&startUpInfo);
		    sa.nLength = sizeof(SECURITY_ATTRIBUTES);
		    sa.lpSecurityDescriptor = 0;
		    sa.bInheritHandle = TRUE;
		    /* append the handle of the server socket to the command line */
		    sprintf (commandLine, "%s %d", sp, newsock);
		    if ((success = CreateProcess (execname, commandLine, NULL,
						NULL, TRUE, DETACHED_PROCESS,
						NULL, NULL, &startUpInfo, &procInfo)) == FALSE)
		    {
			lastError = GetLastError();
    			tisSysLog (0, 0, "Fail to exec %s with args %s (%d)", execname, commandLine, lastError);
		    }
		    closesocket (newsock);
#else
		    /* build newargv array */
		    for (ix = 0; *sp && ix < 19; ix++)
		    {
			while (*sp && *sp == ' ')
			    sp++;
			if (*sp)
			    newargv [ix] = sp;
			while (*sp && *sp != ' ')
			{
			    if (*sp == '"')
			    {
				while (*sp && *sp != '"')
				{
				    if (*sp == '\\')
					sp++;
				    sp++;
				}
			    }
			    sp++;
			}
		    }
		    newargv[ix] = 0;
		    /* run it */
		    execvp (execname, newargv);
		    close (newsock);
#endif WIN32
		    break;
		}
	    }
	}
	tisSysLog (0, 0, "Service %s not found", inmsg.service);
#ifdef WIN32
	fclose (fp);
#else
	close (fp);
#endif WIN32
    }
    else
    {
	tisSysLog (0, 0, "Can't open config file %m");
    }
#ifdef WIN32
	closesocket (newsock);
#else
	close (0);
#endif WIN32
    return (1); /* not found */
}

int respond (SOCKET bsock, struct sockaddr_in *client_addr)
{
    int ix;
    char *sp;
    FILE *fp;
    char *cp;
    SvcResp	msg;		/* output message information */
    struct hostent *hp;

    /* look up requested service, exit if not found */
#ifdef WIN32
    fp = fopen ("\\tis4000db\\service.conf", "rt");
#else
    fp = fopen ("/tis4000db/service.conf", "rt");
#endif WIN32
    if (fp)
    {
        while (!feof (fp))
        {
            ix = 0;
	    while (!feof (fp))
	    {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
 		}
		ix++;
 		if (ix == 80)
		{
		    inpstring[79] = 0;
		    tisSysLog (0, 0, "bad string in config file %s", inpstring);
		    return FALSE; /* bad string in file. */
		}
	    }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && *sp == ' ')
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */
	    /* process input- line is formatted as
	    ** <service_name> <executable> <args-as in inetd.conf>
	    */
	    cp = strchr (sp, ' ');
	    if (cp)
	    {
		*cp = 0;
		if (strcmp (sp, inmsg.service) == 0)
		{
		    /* return response message */
		    msg.type = htons(SVCRESP);
		    msg.sftlvl = htonl(SOFTWARE_LEVEL);
		    /* send response to other guy */
		    if (sendto (bsock, (char *)&msg, sizeof(msg), 0,
			     (struct sockaddr *)client_addr, sizeof(struct sockaddr)) < 0)
		    {
#ifdef WIN32
			errno = WSAGetLastError ();
			if (errno != WSAEINTR)
#else
			if (errno != EINTR)
#endif WIN32
			{
			    tisSysLog (0, 0, "Error receiving UDP message size %d, %m", ix);
			    exit(0);
			}
#ifdef WIN32
		    }
		    fclose (fp);
#else
		    close (fp);
#endif WIN32
		    if ((hp = gethostbyaddr ((char *)&client_addr->sin_addr,
					    sizeof(struct sockaddr_in), AF_INET)) == NULL)
		    {
			tisSysLog (1, 0, "Service %s sent to %s",
					     inmsg.service, INET_NTOA(client_addr->sin_addr));
		    }
		    else
		    {
			tisSysLog (1, 0, "Service %s sent to %s",
					     inmsg.service, hp->h_name);
		    }
		    return TRUE;
		}
	    }
	}
    }
    if ((hp = gethostbyaddr ((char *)&client_addr->sin_addr,
			    sizeof(struct sockaddr_in), AF_INET)) == NULL)
    {
	tisSysLog (0, 0, "No service %s for %s",
			    inmsg.service, INET_NTOA(client_addr->sin_addr));
    }
    else
    {
	tisSysLog (0, 0, "No service %s for %s",
			   inmsg.service, hp->h_name);
    }
    return FALSE; /* not found */
}