/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/*
   File:   oracleRemArchive.c - trend data program to access trend files
		 on other systems.
   Author: Grant Irani
   Date:   11/08/96 19:52
*/

#include <ctype.h>
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32
#include "defines.h"
#include "trendfil.h"
#ifndef MIPSEL
#include "memory.h"
#endif

#include "client.h"
#include "trenddata.h"
#include "archiveLib.h"

#define MAXSERVERNAME	80
#define MAXPARTNERS	20

typedef struct {
    char	server [MAXSERVERNAME]; /* name of partner server */
    long	filetime;	/* file saved partner valid response time */
    long	responsetime;	/* session partner valid response time */
    long	scantime;	/* current point partner valid response time */
    CLIENT	*client;	/* client structure to access server */
    RecRsp	*recResponse;	/* last query result */
} PARTNER;

int	numpartners;
PARTNER	partner [MAXPARTNERS];

void SyncInfo (char *pointname);
RecRsp *GetDataRec (CLIENT *client, char *pointname, int dbred, long when);
void ImportRecord (RecRsp *recResponse);
int GetPartnerData (PARTNER partner []);
void SavePartnerData (int numpartners, PARTNER partner []);

#ifdef STLOUIS
char *partnerdataname = {"/stlouis/tis4000db/snapsync.def"};
char *tempdataname = {"/stlouis/tis4000db/snapsyncXXXXXX"};
char *databasename = {"/stlouis/tis4000db/ar_snap.dbf"};
#else
char *partnerdataname = {"/tis4000db/snapsync.def"};
char *tempdataname = {"/tis4000db/snapsyncXXXXXX"};
char *databasename = {"/tis4000db/ar_snap.dbf"};
#endif


RecMsg	rqmsg;

RecRsp *GetDataRec (CLIENT *client, char *pointname, int dbred, long when)
{
    RecMsg  *recreq = (RecMsg *)&rqmsg;
    RecRsp  *recResponse;

    recreq->type = htons(DATA);
    strncpy (recreq->pointname, pointname, sizeof(recreq->pointname));
    recreq->dbreduc = htons(dbred);
    recreq->time = htonl(when);
    recResponse = (RecRsp *)SendReq(client, recreq, sizeof(*recreq));
    if (recResponse != NULL)
    {
	ImportRecord (recResponse);
    }
    return recResponse;
}

/* open access to oracle. */
int OpenOracle ( void )
{
	    partner[numpartners].responsetime = partner[numpartners].filetime;
	    partner[numpartners].scantime = partner[numpartners].filetime;
	    partner[numpartners].client = OpenServer (partner[numpartners].server, TRUE, NULL);

	    if (partner[numpartners].client)
	    {
		file.type = htons(SELECT_FILE);
		strncpy (file.filename, databasename, sizeof(file.filename));
		fileRsp = (FileRsp *)SendReq(partner[numpartners].client, &file, sizeof(file));
		if (fileRsp)
		{
		    result = ntohs(fileRsp->value);
		    free (fileRsp);
		}
		if (result != 0)
		{
		    CloseSession (partner[numpartners].client);
		    partner[ix].client = NULL;
		}
	    }
}

/* create a descriptor object to access data */
OracleDesc *MakeSnapDesc ( char *tablespace, char *tablename );

/* close access to oracle*/
int CloseOracle ( void );


int GetSnapValues (char *pointname, char *dbred, char *reduc,
		   long unitsecs, time_t start, long *dbrate,
		    char *egutag, double *disp_low, double *disp_high,
		   int reqnum, double values[], unsigned char status[][3], double values2[])
{
}
/* retrieve list of offline files from previous query, NULL means no offline data required */
char *GetOfflineTables ( );

/* Note operations with archive catalog support */
int AddSetNote (char *pointname, time_t when, char *text);
int DeleteSetNote (char *pointname, time_t when);
int PrevSetNote (char *pointname, time_t *when, char **text);
int StartNoteSet (char *pointname, time_t *when, char **note);
int NextNoteSet (time_t *when, char **note);
int EndNoteSet ( void );

void PointOpen (char *epn, int dbreduc);
void PointClose (char *epn, int dbreduc);

/* old file stuff=======================*/
main ()
{
    int cnt;
    int ix;
    char **pvnames;

    /* open local database file */
    snapinit ();
    if (opensnap (databasename))
    {
	/* Can't open data files */
    }

    /* open sync partners file, read in partners and times */
    numpartners = GetPartnerData (partner);
    /* if no one can help us, quit */
    if (numpartners == 0)
	exit (0);

    pvnames = GetReducs (&cnt);
    RelPoints ();
    for (ix = 0; ix < cnt; ix++)
    {
        SyncInfo (pvnames[ix]);
	free (pvnames[ix]);
    }
    free ((char *)pvnames);
    /* write out sync partners file */
    SavePartnerData (numpartners, partner);
}

void SyncInfo (char *pointname)
{
    char	point [C_PTNMLEN];
    int		reduc;
    long	datatime = 0;	/* time we need data */
    long	nexttime; 	/* time used to recognize gap in data */
    int		ix;
    char	*cp;

    /* Split pointname into parts */
    strncpy (point, pointname, sizeof(point));
    if ((cp = strchr (point, '|')) != NULL)
	*cp = 0;
    if (cp = strchr (pointname, '|'))
    {
	cp++;
	if (strcmp(cp, "SMP") == 0)
	    reduc = 0;
	else if (strcmp(cp, "AVG") == 0)
	    reduc = 1;
	else if (strcmp(cp, "MIN") == 0)
	    reduc = 2;
	else if (strcmp(cp, "MAX") == 0)
	    reduc = 3;
	else
	    reduc = 0;
    }
    else
	reduc = 0;

    /* set scan times to saved times */
    for (ix = 0; ix < numpartners; ix++)
    {
	partner[ix].scantime = partner[ix].filetime;
    }

    /* search for missing data from time up to now (minus 10 minutes) */
    while (datatime < (time(0) - 600))
    {
	/* start at earliest saved partner time */
	datatime = 0x7fffffff;
	for (ix = 0; ix < numpartners; ix++)
	{
	    if (partner[ix].client && (partner[ix].scantime < datatime))
	    {
		datatime = partner[ix].scantime;
	    }
	}
	/* if no valid time found, just return */
	if (datatime == 0x7fffffff)
	    return;

	GetRec (point, reduc, datatime);
	if (ItemTime() <= datatime)
	{
	    nexttime = ItemTime();
	    /* move time forward until C error, gap in data, or time is >= t-10min */
	    while ((ItemTime() < time(0)-600) &&  (*ItemStatus () != 'C'))
	    {
		nexttime = ItemTime() + RecRate ();
		NextItem ();
		if (ItemTime() != nexttime)
		    break; /* gap in info, go fish */
	    }
	    /* adjust time to needed data */
	    if (nexttime > datatime)
		datatime = nexttime;
	}

	if (datatime >= (time(0) - 600))
	    continue; /* and return to caller */

	/* go fishing */
	/* find each partner whose scantime is unknown, or earlier than the desired time */
	for (ix = 0; ix < numpartners; ix++)
	{
	    if (partner[ix].client
	      && ((partner[ix].scantime == 0) || (partner[ix].scantime <= datatime)))
	    {
		/* check current data from server */
		if (partner[ix].recResponse
		  && (partner[ix].recResponse->time <= datatime)
		  && (datatime < (partner[ix].recResponse->time
				  + (partner[ix].recResponse->rate
				   * partner[ix].recResponse->samples))))
		{
		    /* process returned data */
		    UpdateDBRec (partner[ix].recResponse->pointname, partner[ix].recResponse->dbreduc,
				 partner[ix].recResponse->time, partner[ix].recResponse->rate,
				 partner[ix].recResponse->samples, partner[ix].recResponse->tag,
				 partner[ix].recResponse->dl, partner[ix].recResponse->dh,
				 partner[ix].recResponse->value, partner[ix].recResponse->status);
		    /* update partner time */
		    if (partner[ix].recResponse->time != 0x7fffffff)
		    {
			partner[ix].scantime = partner[ix].recResponse->time
						 + (partner[ix].recResponse->rate
							 * partner[ix].recResponse->samples);
			if ((partner[ix].scantime > partner[ix].responsetime))
			{
			    partner[ix].responsetime = partner[ix].scantime;
			}
		    }
		    else
			partner[ix].scantime = partner[ix].recResponse->time;
		    free (partner[ix].recResponse);
		    partner[ix].recResponse = NULL;
		}
		else
		{
		    /* ask them for data */
		    if (partner[ix].recResponse)
		    {
			free (partner[ix].recResponse);
			partner[ix].recResponse = NULL;
		    }
		    partner[ix].recResponse = GetDataRec (partner[ix].client, point, reduc, datatime);
		    if (partner[ix].recResponse)
		    {
			if ((partner[ix].recResponse->time <= datatime)
			  && (datatime < (partner[ix].recResponse->time
				      + (partner[ix].recResponse->rate
				       * partner[ix].recResponse->samples))))
			{
			    if (partner[ix].recResponse->result == TRUE)
			    {
				/* process returned data */
				UpdateDBRec (partner[ix].recResponse->pointname, partner[ix].recResponse->dbreduc,
					partner[ix].recResponse->time, partner[ix].recResponse->rate,
					partner[ix].recResponse->samples, partner[ix].recResponse->tag,
					partner[ix].recResponse->dl, partner[ix].recResponse->dh,
					partner[ix].recResponse->value, partner[ix].recResponse->status);
				/* we have processed data through the end of this record */
				partner[ix].scantime = partner[ix].recResponse->time
						 + (partner[ix].recResponse->rate
							 * partner[ix].recResponse->samples);
			    }
			    free (partner[ix].recResponse);
			    partner[ix].recResponse = NULL;
			}
			else
			    partner[ix].scantime = partner[ix].recResponse->time;

			/* update partner time */
			if (partner[ix].scantime != 0x7fffffff)
			{
			    if ((partner[ix].scantime > partner[ix].responsetime))
			    {
			    partner[ix].responsetime = partner[ix].scantime;
			    }
			}
			else
			    partner[ix].scantime = partner[ix].recResponse->time;
		    }
		    else /* server failed, stop asking this client */
		    {
		        CloseSession (partner[ix].client);
		        partner[ix].client = NULL;
		    }
		}
	    }
	}
	/* advance search time to earliest partner time */
    }
}

RecMsg	rqmsg;

RecRsp *GetDataRec (CLIENT *client, char *pointname, int dbred, long when)
{
    RecMsg  *recreq = (RecMsg *)&rqmsg;
    RecRsp  *recResponse;

    recreq->type = htons(DATA);
    strncpy (recreq->pointname, pointname, sizeof(recreq->pointname));
    recreq->dbreduc = htons(dbred);
    recreq->time = htonl(when);
    recResponse = (RecRsp *)SendReq(client, recreq, sizeof(*recreq));
    if (recResponse != NULL)
    {
	ImportRecord (recResponse);
    }
    return recResponse;
}

void ImportRecord (RecRsp *recResponse)
{
    int	    ix;

    recResponse->result = ntohs(recResponse->result);
    if (recResponse->result == FALSE)
    {
	return;
    }
    recResponse->dbreduc = ntohs (recResponse->dbreduc);
    recResponse->rec = ntohl (recResponse->rec);
    recResponse->time = ntohl (recResponse->time);
    recResponse->rate = ntohl (recResponse->rate);
    recResponse->samples = ntohs (recResponse->samples);
    ntohd (recResponse->dl, recResponse->dl);
    ntohd (recResponse->dh, recResponse->dh);
    for (ix = 0; ix < recResponse->samples; ix++)
    {
	 ntohd (recResponse->value[ix], recResponse->value[ix]);
    }
    return;
}

static char inpstring [132];
static char outstring [132];

int GetPartnerData (PARTNER partner [])
{
    int numpartners;
    int ix;
    int ix2;
    char *sp;
    FILE *fp;
    char *cp;
    char *field;
    int result = DBSNAPERR;
    FileMsg file;
    FileRsp *fileRsp;

    numpartners = 0;
    fp = fopen (partnerdataname, "rt");
    if (fp)
    {
        while (!feof (fp))
        {
            ix = 0;
            while (!feof (fp))
            {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
                }
		ix++;
                if (ix == 80)
                    return; /* bad string in file. */
            }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && *sp == ' ')
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */
	    if (cp = strchr (sp, ','))
	    {
		*cp++ = 0;
		strncpy (partner[numpartners].server, sp, MAXSERVERNAME);
		while (*cp && *cp == ' ')
		    cp++;
		partner[numpartners].filetime = atoi (cp);
	    }
	    else
	    {
		strncpy (partner[numpartners].server, sp, MAXSERVERNAME);
		partner[numpartners].filetime = 0;
	    }
	    partner[numpartners].responsetime = partner[numpartners].filetime;
	    partner[numpartners].scantime = partner[numpartners].filetime;
	    partner[numpartners].client = OpenServer (partner[numpartners].server, TRUE, NULL);

	    if (partner[numpartners].client)
	    {
		file.type = htons(SELECT_FILE);
		strncpy (file.filename, databasename, sizeof(file.filename));
		fileRsp = (FileRsp *)SendReq(partner[numpartners].client, &file, sizeof(file));
		if (fileRsp)
		{
		    result = ntohs(fileRsp->value);
		    free (fileRsp);
		}
		if (result != 0)
		{
		    CloseSession (partner[numpartners].client);
		    partner[ix].client = NULL;
		}
	    }

	    partner[numpartners].recResponse = NULL;
	    /* don't count him if we didn't connect */
	    if (partner[numpartners].client != NULL)
		    numpartners ++;
	}
        fclose (fp);
    } /* end of opened file code */
    return numpartners;
}

void SavePartnerData (int numpartners, PARTNER partner [])
{
    int ix;
    int ix2;
    char *sp;
    FILE *fp;
    FILE *outfp;
    char *cp;
    char *field;
    char tempname[40];

    strncpy (tempname, tempdataname, sizeof(tempname));
    mktemp (tempname);
    outfp = fopen (tempname, "w");
    fp = fopen (partnerdataname, "rt");
    if (outfp && fp)
    {
        while (!feof (fp))
        {
            ix = 0;
            while (!feof (fp))
            {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
                }
		ix++;
                if (ix == 80)
                    return; /* bad string in file. */
            }
	    if ((ix == 0) && (inpstring [ix] == -1))
		continue;

	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && *sp == ' ')
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
	    {
		sprintf (outstring, "%s\n", inpstring);
		fwrite (outstring, strlen(outstring), 1, outfp);
		continue; /* blank line or comment! */
	    }
	    else
	    {
		if (cp = strchr (sp, ','))
		{
		    *cp++ = 0;
		}
		for (ix = 0; ix < numpartners; ix++)
		{
	            if (strncmp (sp, partner [ix].server, strlen (partner [ix].server)) == 0)
		    {
			if (cp = strchr (sp, '#'))
			{
			    sprintf (outstring, "%s, %d, %s\n",
				     partner[ix].server,
				     partner[ix].responsetime,
				     cp);
			}
			else
			{
			    sprintf (outstring, "%s, %d\n",
				     partner[ix].server,
				     partner[ix].responsetime);
			}
			fwrite (outstring, strlen (outstring), 1, outfp);
			break; /* leave loop */
		    }
		}
		if (ix == numpartners)
		{
		    sprintf (outstring, "%s\n", inpstring);
		    fwrite (outstring, strlen(outstring), 1, outfp);
		}
	    }
	}
        fclose (fp);
	fclose (outfp);
	rename (tempname, partnerdataname);
    } /* end of opened file code */
    return;
}
