/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*
   File:   snapsync.c - trend data client program to update the ar_snap files
		 with data from other systems.
   Author: Grant Irani
   Date:   4/30/96 13:21
*/

#include <ctype.h>
#include "trendSizes.h"

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
    #include "cb45to63.h"
#endif WIN32
    #include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "archiveLib.h"
#include "oracleSnap.h"
#include "avmDefs.h"
#include "xbaseSnapLib.h"
#include "trendfil.h"
#ifndef MIPSEL
#include "memory.h"
#endif

#include "client.h"
#include "trenddata.h"

#define TRUE 1
#define FALSE 0

int DEBUG_MODE = FALSE;


#define MAXSERVERNAME	80
#define MAXPARTNERS	20

typedef struct {
    char	server [MAXSERVERNAME]; /* name of partner server */
    long	filetime;	/* file saved partner valid response time */
    long	responsetime;	/* session partner valid response time */
    long	scantime;	/* current point partner valid response time */
    CLIENT	*client;	/* client structure to access server */
    RecRsp	*recResponse;	/* last query result */
} PARTNER;

char *dbreducs [] = { "SMP", "AVG", "MIN", "MAX" };

int	numpartners;
PARTNER	partner [MAXPARTNERS];

void SyncInfo (char *pointname);
RecRsp *GetDataRec (CLIENT *client, char *pointname, int dbred, long when);
void ImportRecord (RecRsp *recResponse);
int GetPartnerData (PARTNER partner []);
void SavePartnerData (int numpartners, PARTNER partner []);

char *partnerdataname = {"/tis4000db/snapsync.def"};
char *tempdataname = {"/tis4000db/snapsyncXXXXXX"};
char *databasename = {"/tis4000db/ar_snap.dbf"};

int statistics;

main (int argc, char *argv[])
{
    int cnt;
    int ix;
    char **pvnames;

    /* Get Debug Mode flag */
    if (argc > 1 && (strcmp(argv[1], "-d")) == 0)
    {
	DEBUG_MODE = TRUE;
    }

    /* Get noise flag */
    statistics = 0;
    if (argc > 1 && *argv[1] == '-')
    {
	if (*(argv[1]+1) == 's')
	    statistics = 1 + atoi(argv[1]+2);
    }

    /* open local database file */
    snapinit ();
    if (opensnap (databasename))
    {
	/* Can't open data files */
	printf ("Can't open data file for synchronization operation!\n");
	return;
    }

    /* open sync partners file, read in partners and times */
    numpartners = GetPartnerData (partner);
    /* if no one can help us, quit */
    if (numpartners == 0)
	exit (0);

    pvnames = GetReducs (&cnt);
    RelPoints ();

    if (statistics >= 1)
	printf ("Syncing %d points with %d partner(s)\n", cnt, numpartners);

    if (DEBUG_MODE)
	printf ("Syncing %d points with %d partner(s)\n", cnt, numpartners);

    for (ix = 0; ix < cnt; ix++)
    {
        SyncInfo (pvnames[ix]);
	free (pvnames[ix]);
    }
    free ((char *)pvnames);
    /* ------ SyncNotes (); ------ */
    /* write out sync partners file */
    for (ix = 0; ix < numpartners; ix++)
    {
	/* set host query time to start of previous hour */
	if (partner[ix].client)
	{
	  if (!DEBUG_MODE)
	  {
	    partner[ix].responsetime = ((long)(time (NULL) - 3599) / 3600) * 3600;
	  }
	    CloseSession (partner[numpartners].client);
	    partner[ix].client = NULL;
	}
    }
    if (!DEBUG_MODE)
    	SavePartnerData (numpartners, partner);
}

void SyncInfo (char *pointname)
{
    char	point [POINTNAME_SIZE];
    char	reduc [4];
    long	datatime = 0;	/* time we need data */
    long	localtime = 0;	/* start time for current local record */
    long	nexttime; 	/* time used to recognize gap in data */
    int		ix;
    char	*cp;
    int		queries = 0;
    struct timeval startsync;
    struct timeval startquery;
    struct timeval endtime;
    struct timeval avgquery = {0,0};
    char nullstr = 0;
    int 	i;

    if (statistics >= 2)
	printf ("Syncing %s\n", pointname);

    if(DEBUG_MODE)
	printf ("Syncing %s\n", pointname);

    /* Split pointname into parts */
    strncpy (point, pointname, sizeof(point));
    if ((cp = strchr (point, '|')) != NULL)
	*cp = 0;
    if (cp = strchr (pointname, '|'))
    {
	cp++;
	strncpy (reduc, cp, sizeof(reduc));
	if (strlen(reduc) == 0)
	    strcpy (reduc, "SMP");
    }
    else
	strcpy (reduc, "SMP");

    if (statistics >= 2)
	gettimeofday (&startsync, &nullstr);

    if (DEBUG_MODE)
	gettimeofday (&startsync, &nullstr);

    /* set scan times to saved times */
    for (ix = 0; ix < numpartners; ix++)
    {
	partner[ix].scantime = partner[ix].filetime;
    }

    /* search for missing data from time up to now (minus 10 minutes) */
    while (datatime < (time(0) - 600))
    {
	/* start at earliest saved partner time */
	datatime = 0x7fffffff;
	for (ix = 0; ix < numpartners; ix++)
	{
	    if (partner[ix].client && (partner[ix].scantime < datatime))
	    {
		datatime = partner[ix].scantime;
	    }
	}
	/* if no valid time found, just return */
	if (datatime == 0x7fffffff)
	    break;

	/* find first rec in local db for the partner's last report time */
	/* only do this once ( kinda expensive on Oracle )*/
 	if (localtime == 0 || localtime < datatime)
	    GetRec (point, reduc, datatime);
	if ((localtime = ItemTime()) <= datatime)
	{
	    nexttime = ItemTime();
	    /* move time forward until C error, gap in data, or time is >= t-10min */
	    while ((ItemTime() < time(0)-600) &&  (*ItemStatus () != 'C'))
	    {
		nexttime = ItemTime() + RecRate ();
		NextItem ();
		if ((localtime = ItemTime()) != nexttime)
		    break; /* gap in info, go fish */
	    }
	    /* adjust time to needed data */
	    if (nexttime > datatime)
		datatime = nexttime;
	}

	if (datatime >= (time(0) - 600))
	    continue; /* and return to caller */

	/* go fishing */
	/* find each partner whose scantime is unknown, or earlier than the desired time */
	for (ix = 0; ix < numpartners; ix++)
	{
	    if (partner[ix].client
	      && ((partner[ix].scantime == 0) || (partner[ix].scantime <= datatime)))
	    {
		if (statistics >= 4)
		    printf ("Query partner %d - time %d\n", ix, datatime);
		if (DEBUG_MODE) 
		    printf ("Query partner %d - time %d\n", ix, datatime);
		/* check current data from server */
		if (partner[ix].recResponse
		  && (partner[ix].recResponse->time <= datatime)
		  && (datatime < (partner[ix].recResponse->time
				  + (partner[ix].recResponse->rate
				   * partner[ix].recResponse->samples))))
		{
		    if(DEBUG_MODE)
		    {
		    	printf( "point number: %d\n"
				"pointname: %s\n"
				"reduction: %d\n"
				"start time: %d\n"
				"rate of samples: %d\n"
				"number of samples: %d\n"
				"tag space: %s\n"
				"display limit (high val): %f\n"
				"display limit (low val): %f\n",
				ix,
				partner[ix].recResponse->pointname, 
				partner[ix].recResponse->dbreduc,
				partner[ix].recResponse->time, 
				partner[ix].recResponse->rate,
				partner[ix].recResponse->samples, 
				partner[ix].recResponse->tag,
				partner[ix].recResponse->dl, 
				partner[ix].recResponse->dh);
			printf("sample values:\n");
			for(i = 0; i < 60; i++)
			  printf("%d ", partner[ix].recResponse->value[i]); 
			printf("sample status:\n");
			for(i = 0; i < 60; i++)
			  printf("%s ", partner[ix].recResponse->status[i]); 
		    }
		    else
		    {
		    	/* process returned data */
		    	UpdateDBRec (partner[ix].recResponse->pointname, dbreducs[partner[ix].recResponse->dbreduc],
				 partner[ix].recResponse->time, partner[ix].recResponse->rate,
				 partner[ix].recResponse->samples, partner[ix].recResponse->tag,
				 partner[ix].recResponse->dl, partner[ix].recResponse->dh,
				 partner[ix].recResponse->value, partner[ix].recResponse->status);
		    }
		    /* update partner time */
		    if (partner[ix].recResponse->time != 0x7fffffff)
		    {
			partner[ix].scantime = partner[ix].recResponse->time
						 + (partner[ix].recResponse->rate
							 * partner[ix].recResponse->samples);
			if ((partner[ix].scantime > partner[ix].responsetime))
			{
			    partner[ix].responsetime = partner[ix].scantime;
			}
		    }
		    else
			partner[ix].scantime = partner[ix].recResponse->time;
		    free (partner[ix].recResponse);
		    partner[ix].recResponse = NULL;
		}
		else
		{
		    /* ask them for data */
		    if (partner[ix].recResponse)
		    {
			free (partner[ix].recResponse);
			partner[ix].recResponse = NULL;
		    }
		    gettimeofday (&startquery, &nullstr);
		    partner[ix].recResponse = GetDataRec (partner[ix].client, point, RecReduc(reduc), datatime);
		    if (partner[ix].recResponse)
		    {
			if (statistics >= 3)
			{
			    gettimeofday (&endtime, &nullstr);
			    startquery.tv_sec = endtime.tv_sec - startquery.tv_sec;
			    startquery.tv_usec = endtime.tv_usec - startquery.tv_usec;
			    if (startquery.tv_usec < 0)
			    {
				startquery.tv_usec += 1000000;
				startquery.tv_sec --;
			    }
			    avgquery.tv_usec = startquery.tv_usec + avgquery.tv_usec;
			    avgquery.tv_sec = startquery.tv_sec + avgquery.tv_sec;
			    if (avgquery.tv_usec >= 1000000)
			    {
				avgquery.tv_usec -= 1000000;
				avgquery.tv_sec++;
			    }
			    queries++;
			    if (statistics >= 4)
			    {
				printf ("Response returned: %s, reduc %d, time %d, rate %d\n",
					partner[ix].recResponse->pointname, partner[ix].recResponse->dbreduc,
					partner[ix].recResponse->time, partner[ix].recResponse->rate);
			    }
			}
			if (DEBUG_MODE)
			{
			    gettimeofday (&endtime, &nullstr);
			    startquery.tv_sec = endtime.tv_sec - startquery.tv_sec;
			    startquery.tv_usec = endtime.tv_usec - startquery.tv_usec;
			    if (startquery.tv_usec < 0)
			    {
				startquery.tv_usec += 1000000;
				startquery.tv_sec --;
			    }
			    avgquery.tv_usec = startquery.tv_usec + avgquery.tv_usec;
			    avgquery.tv_sec = startquery.tv_sec + avgquery.tv_sec;
			    if (avgquery.tv_usec >= 1000000)
			    {
				avgquery.tv_usec -= 1000000;
				avgquery.tv_sec++;
			    }
			    queries++;
		   	    printf ("Response returned: %s, reduc %d, time %d, rate %d\n",
					partner[ix].recResponse->pointname, partner[ix].recResponse->dbreduc,
					partner[ix].recResponse->time, partner[ix].recResponse->rate);
			}
			/* if the returned values satisfy our needs */
			if ((partner[ix].recResponse->time <= datatime)
			  && (datatime < (partner[ix].recResponse->time
				      + (partner[ix].recResponse->rate
				       * partner[ix].recResponse->samples))))
			{
			    if (partner[ix].recResponse->result == TRUE)
			    {
		    		if(DEBUG_MODE)
		    		{
		    			printf( "point number: %d\n"
						"pointname: %s\n"
						"reduction: %d\n"
						"start time: %d\n"
						"rate of samples: %d\n"
						"number of samples: %d\n"
						"tag space: %s\n"
						"display limit (high val): %f\n"
						"display limit (low val): %f\n",
						ix,
						partner[ix].recResponse->pointname, 
						partner[ix].recResponse->dbreduc,
						partner[ix].recResponse->time, 
						partner[ix].recResponse->rate,
						partner[ix].recResponse->samples, 
						partner[ix].recResponse->tag,
						partner[ix].recResponse->dl, 
						partner[ix].recResponse->dh);
					printf("sample values:\n");
					for(i = 0; i < 60; i++)
			  		   printf("%d ", partner[ix].recResponse->value[i]); 
			                printf("sample status:\n");
					for(i = 0; i < 60; i++)
			  		   printf("%s ", partner[ix].recResponse->status[i]); 
		    		}
		    		else
		    		{
					/* process returned data */
					UpdateDBRec (partner[ix].recResponse->pointname, 
						dbreducs [partner[ix].recResponse->dbreduc],
						partner[ix].recResponse->time, partner[ix].recResponse->rate,
						partner[ix].recResponse->samples, partner[ix].recResponse->tag,
						partner[ix].recResponse->dl, partner[ix].recResponse->dh,
						partner[ix].recResponse->value, partner[ix].recResponse->status);
				}
				/* we have processed data through the end of this record */
				partner[ix].scantime = partner[ix].recResponse->time
						 + (partner[ix].recResponse->rate
							 * partner[ix].recResponse->samples);
			    }
			    free (partner[ix].recResponse);
			    partner[ix].recResponse = NULL;
			}
			else
			{
			    /* data may satisfy future needs, adjust scantime */
			    partner[ix].scantime = partner[ix].recResponse->time;
			}
			/* update partner time */
			if (partner[ix].scantime != 0x7fffffff)
			{
			    if ((partner[ix].scantime > partner[ix].responsetime))
			    {
				/* set time to best indication of when he collected data */
				partner[ix].responsetime = partner[ix].scantime;
			    }
			}
			else
			{
			    /* partner has nothing better for us on this point */
			    partner[ix].scantime = partner[ix].recResponse->time;
			}
		    }
		    else /* server failed, stop asking this client */
		    {
		        CloseSession (partner[ix].client);
		        partner[ix].client = NULL;
		    }
		}
	    }
	}
	/* advance search time to earliest partner time */
    }

    if (statistics >= 3)
    {
	gettimeofday (&endtime, &nullstr);
	startsync.tv_sec = endtime.tv_sec - startsync.tv_sec;
	startsync.tv_usec = endtime.tv_usec - startsync.tv_usec;
	if (startsync.tv_usec < 0)
	{
	    startsync.tv_usec += 1000000;
	    startsync.tv_sec --;
	}
	if (queries)
	{
	    avgquery.tv_sec = avgquery.tv_sec/queries;
	    avgquery.tv_usec = (avgquery.tv_usec + ((avgquery.tv_sec % queries) * 1000000))/queries;
	}

	printf ("Sync time %d.%6.6d: requested %d records, average query %d.%6.6d\n",
		startsync.tv_sec, startsync.tv_usec, queries, avgquery.tv_sec, avgquery.tv_usec);
    }
    if (DEBUG_MODE) 
    {
	gettimeofday (&endtime, &nullstr);
	startsync.tv_sec = endtime.tv_sec - startsync.tv_sec;
	startsync.tv_usec = endtime.tv_usec - startsync.tv_usec;
	if (startsync.tv_usec < 0)
	{
	    startsync.tv_usec += 1000000;
	    startsync.tv_sec --;
	}
	if (queries)
	{
	    avgquery.tv_sec = avgquery.tv_sec/queries;
	    avgquery.tv_usec = (avgquery.tv_usec + ((avgquery.tv_sec % queries) * 1000000))/queries;
	}

	printf ("Sync time %d.%6.6d: requested %d records, average query %d.%6.6d\n",
		startsync.tv_sec, startsync.tv_usec, queries, avgquery.tv_sec, avgquery.tv_usec);
    }
}

RecMsg	rqmsg;

RecRsp *GetDataRec (CLIENT *client, char *pointname, int dbred, long when)
{
    RecMsg  *recreq = (RecMsg *)&rqmsg;
    RecRsp  *recResponse;

    recreq->type = htons(DATA);
    strncpy (recreq->pointname, pointname, sizeof(recreq->pointname));
    recreq->dbreduc = htons(dbred);
    recreq->time = htonl(when);
    recResponse = (RecRsp *)SendReq(client, recreq, sizeof(*recreq));
    if (recResponse != NULL)
    {
	ImportRecord (recResponse);
    }
    return recResponse;
}

void ImportRecord (RecRsp *recResponse)
{
    int	    ix;

    recResponse->result = ntohs(recResponse->result);
    if (recResponse->result == FALSE)
    {
	return;
    }
    recResponse->dbreduc = ntohs (recResponse->dbreduc);
    recResponse->rec = ntohl (recResponse->rec);
    recResponse->time = ntohl (recResponse->time);
    recResponse->rate = ntohl (recResponse->rate);
    recResponse->samples = ntohs (recResponse->samples);
    ntohd (recResponse->dl, recResponse->dl);
    ntohd (recResponse->dh, recResponse->dh);
    for (ix = 0; ix < recResponse->samples; ix++)
    {
	 ntohd (recResponse->value[ix], recResponse->value[ix]);
    }
    return;
}

static char inpstring [132];
static char outstring [132];
static struct tm tm_struct;

int GetPartnerData (PARTNER partner [])
{
    int numpartners;
    int ix;
    int ix2;
    char *sp;
    FILE *fp;
    char *cp;
    char *field;
    int result = DBSNAPERR;
    FileMsg file;
    FileRsp *fileRsp;

    numpartners = 0;
    fp = fopen (partnerdataname, "rt");
    if (fp)
    {
        while (!feof (fp))
        {
            ix = 0;
            while (!feof (fp))
            {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
                }
		ix++;
                if (ix == 80)
                    return; /* bad string in file. */
            }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && *sp == ' ')
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */
	    /* clip off any comment part on line */
	    if (strchr (sp, '#'))
		*strchr (sp, '#') = 0;
	    if (cp = strchr (sp, ','))
	    {
		*cp++ = 0;
		strncpy (partner[numpartners].server, sp, MAXSERVERNAME);
		while (*cp && *cp == ' ')
		    cp++;
		if (strstr (cp, "UTC"))
		{
		    /* if UTC is found, interpret time from mm/dd/yyyy hh:mm */
		    partner[numpartners].filetime = 0; /* invalid time will generate 0 */
		    tm_struct.tm_isdst = -1;
		    tm_struct.tm_mon = atoi(cp) - 1;
		    cp = strchr (cp, '/') + 1;
		    if (cp != (char *)1)
		    {
			tm_struct.tm_isdst = 0;
			tm_struct.tm_mday = atoi(cp);
			cp = strchr (cp, '/') + 1;
			if (cp != (char *)1)
			{
			    tm_struct.tm_year = atoi(cp) - 1900;
			    cp = strchr (cp, ' ') + 1;
			    if (cp != (char *)1)
			    {
				tm_struct.tm_hour = atoi(cp);
				cp = strchr (cp, ':') + 1;
				if (cp != (char *)1)
				{
				    tm_struct.tm_min = atoi(cp);
#if (defined(SOLARIS) || defined(AXP) || defined(WIN32))
				    partner[numpartners].filetime = mktime (&tm_struct) - timezone;
#else
				    partner[numpartners].filetime = timegm (&tm_struct);
#endif
				}
			    }
			}
		    }
		}
		else
		{
		    /* else assume Unix time_t */
		    partner[numpartners].filetime = atoi (cp);
		}
	    }
	    else
	    {
		strncpy (partner[numpartners].server, sp, MAXSERVERNAME);
		partner[numpartners].filetime = 0;
	    }
	    partner[numpartners].responsetime = partner[numpartners].filetime;
	    partner[numpartners].scantime = partner[numpartners].filetime;
	    while (partner[numpartners].server[strlen(partner[numpartners].server)-1] == ' ')
		partner[numpartners].server[strlen(partner[numpartners].server)-1] = 0;
	    partner[numpartners].client = OpenServer (partner[numpartners].server, TRUE, NULL);

	    if (partner[numpartners].client)
	    {
		file.type = htons(SELECT_FILE);
		strncpy (file.filename, databasename, sizeof(file.filename));
		fileRsp = (FileRsp *)SendReq(partner[numpartners].client, &file, sizeof(file));
		if (fileRsp)
		{
		    result = ntohs(fileRsp->value);
		    free (fileRsp);
		}
		if (result != 0)
		{
		    CloseSession (partner[numpartners].client);
		    partner[ix].client = NULL;
		}
	    }

	    partner[numpartners].recResponse = NULL;
	    /* don't count him if we didn't connect */
	    if (partner[numpartners].client != NULL)
		    numpartners ++;
	}
        fclose (fp);
    } /* end of opened file code */
    return numpartners;
}

void SavePartnerData (int numpartners, PARTNER partner [])
{
    int ix;
    int ix2;
    char *sp;
    FILE *fp;
    FILE *outfp;
    char *cp;
    char *field;
    char tempname[40];
    struct tm *tm_struct;

    strncpy (tempname, tempdataname, sizeof(tempname));
    mktemp (tempname);
    outfp = fopen (tempname, "w");
    fp = fopen (partnerdataname, "rt");
    if (outfp && fp)
    {
        while (!feof (fp))
        {
            ix = 0;
            while (!feof (fp))
            {
                inpstring [ix] = fgetc(fp);
                if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
                    break;
                }
		ix++;
                if (ix == 80)
                    return; /* bad string in file. */
            }
	    if ((ix == 0) && (inpstring [ix] == -1))
		continue;

	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    while (*sp && *sp == ' ')
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
	    {
		sprintf (outstring, "%s\n", inpstring);
		fwrite (outstring, strlen(outstring), 1, outfp);
		continue; /* blank line or comment! */
	    }
	    else
	    {
		if (cp = strchr (sp, ','))
		{
		    if (cp < strchr (sp, '#'))
			*cp++ = 0;
		}
		for (ix = 0; ix < numpartners; ix++)
		{
	            if (strncmp (sp, partner [ix].server, strlen (partner [ix].server)) == 0)
		    {
			/* Save responsetime as start of day to allow future corrections */
			
			tm_struct = gmtime (&partner[ix].responsetime);
			if (cp = strchr (sp, '#'))
			{
			    sprintf (outstring, "%s, %2.2d/%2.2d/%4.4d 00:00 UTC %s\n",
				     partner[ix].server,
				     tm_struct->tm_mon+1,
				     tm_struct->tm_mday,
				     tm_struct->tm_year+1900,
				     cp);
			}
			else
			{
			    sprintf (outstring, "%s, %2.2d/%2.2d/%4.4d 00:00 UTC\n",
				     partner[ix].server,
				     tm_struct->tm_mon+1,
				     tm_struct->tm_mday,
				     tm_struct->tm_year+1900);
			}
			fwrite (outstring, strlen (outstring), 1, outfp);
			break; /* leave loop */
		    }
		}
		if (ix == numpartners)
		{
		    sprintf (outstring, "%s\n", inpstring);
		    fwrite (outstring, strlen(outstring), 1, outfp);
		}
	    }
	}
        fclose (fp);
	fclose (outfp);
	rename (tempname, partnerdataname);
    } /* end of opened file code */
    return;
}
