/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/*
    trenddata.h - definitions for the trend data client
    Author: Grant Ireni
    Date:   March 21, 1996
*/


#define	    GET_PRIV	    1		/* get file priviledges */
#define	    GET_TRENDNAMES  2		/* get trend set names */
#define	    GET_NOTE	    3		/* get note */
#define	    GET_PREVNOTE    4		/* get prev note */
#define	    SAVE_NOTE	    5		/* save note */
#define	    DEL_NOTE	    6		/* delete note */
#define	    SELECT_FILE	    7		/* select ar_snap file source */
#define	    GET_REDUC	    8		/* get reduc file data */
#define	    GET_TREND	    9		/* get trendset */
#define	    SAVE_TREND	    10		/* save trendset */
#define	    DELETE_TREND    11		/* delete trendset */
#define	    SEARCH_DB	    12		/* search for data */
#define	    RECORD	    13		/* get record */
#define	    UPDATE	    14		/* update record data */
#define	    UPDREC	    15		/* update entire record */
#define	    DATA	    16		/* find next valid data */

typedef struct {
    short   type;	    /* message type */
    char    opname [40];    /* operator name */
    char    pass [40];	    /* password information (encrypted?) */
    char    group [40];	    /* protection group info */
} PrivMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
} PrivRsp;

typedef struct {
    short   type;	    /* message type */
    short   position;	    /* position in list */
} TrNamesMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
    short   position;	    /* position of first item */
    short   count;	    /* number of items */
    char    trendDescriptors[20][TRENDDESCRIPTOR_SIZE]; /* descriptors space */
} TrNamesRsp;

typedef struct {
    short   type;	    /* message type */
    short   pad;
    long    time;	    /* requested time */
    char    pointname[POINTNAME_SIZE]; /* pointname space */
} NoteMsg;
typedef struct {
    short   type;	    /* message type */
    short   pad;
    long    time;	    /* time of next note */
    char    pointname[POINTNAME_SIZE]; /* pointname space */
    char    note[1024];	    /* note text */
} NoteSaveMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
    long    time;	    /* time of next note */
    char    pointname[POINTNAME_SIZE]; /* pointname space */
    char    note[1024];	    /* note text */
} NoteRsp;

typedef struct {
    short   type;	    /* message type */
    char    filename[132];  /* pointname space */
} FileMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
} FileRsp;

typedef struct {
    short   type;	    /* message type */
    short   position;	    /* position in list */
} ReducMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
    short   position;	    /* position of first item */
    short   total;	    /* total number of reducs */
    short   count;	    /* number in msg */
    char    names[1024];    /* descriptors space */
} ReducRsp;

typedef struct {
    short   type;	    /* message type */
    char    descr[POINTNAME_SIZE]; /* pointname space */
} TrendMsg;
typedef struct {
    short   type;	    /* message type */
    short   flag;	    /* create flag */
    long    zoom;	    /* time of next note */
    char    descr[POINTNAME_SIZE]; /* pointname space */
    char    specs[TREND_NUMPOINTS][TRENDDESCRIPTOR_SIZE]; /* specifier */
} TrendSaveMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
    long    zoom;	    /* time of next note */
    char    descr[POINTNAME_SIZE]; /* pointname space */
    char    specs[TREND_NUMPOINTS][TRENDDESCRIPTOR_SIZE]; /* specifier */
} TrendRsp;

typedef struct {
    short   type;	    /* message type */
    short   dbred;	    /* db reduction type */
    long    time;	    /* requested time */
    SEARCH  srch;	    /* search criteria */
    char    pointname[POINTNAME_SIZE]; /* pointname space */
} SearchMsg;
typedef struct {
    short   type;	    /* message type */
    short   value;	    /* response status */
    long    time;	    /* requested time */
} SearchRsp;

typedef struct {
    short   type;	    /* message type */
    short   dbreduc;	    /* reduction used in ar_snap */
    long    time;	    /* start time for this record */
    char    pointname [POINTNAME_SIZE];/* pointname space */
} RecMsg;
typedef struct {
    short   type;	    /* message type */
    short   dbreduc;	    /* reduction used in ar_snap */
    long    rec;	    /* data base record */
    short int ix;	    /* value index */
    short   pad;
    long    dblpad;	    /* pad to quad align */
    double  value;	    /* update value */
    char    status[4];	    /* update status */
    char    pointname [POINTNAME_SIZE];/* pointname space */
} UpdateMsg;
typedef struct {
    short   type;	    /* message type */
    short   pad;
    short   dbreduc;	    /* reduction used in ar_snap */
    short   samples;	    /* number of samples provided */
    long    rec;	    /* data base record */
    long    time;	    /* start time for this record */
    long    rate;	    /* rate of samples */
    long    dblpad;	    /* pad to quad align */
    double  dl;		    /* display limit high value */
    double  dh;		    /* display limit high value */
    double  value[60];	    /* sample values */
    char    status[60][4];  /* sample status */
    char    pointname [POINTNAME_SIZE];/* pointname space */
    char    tag [EGUTAG_SIZE]; /* tag space */
} UpdRecMsg;
typedef struct {
    short   type;	    /* message type */
    short   result;	    /* response status */
    short   dbreduc;	    /* reduction used in ar_snap */
    short   samples;	    /* number of samples provided */
    long    rec;	    /* data base record */
    long    time;	    /* start time for this record */
    long    rate;	    /* rate of samples */
    long    dblpad;	    /* pad to quad align */
    double  dl;		    /* display limit high value */
    double  dh;		    /* display limit high value */
    double  value[60];	    /* sample values */
    char    status[60][4];  /* sample status */
    char    pointname [POINTNAME_SIZE];/* pointname space */
    char    tag [EGUTAG_SIZE]; /* tag space */
} RecRsp;


typedef union {
    PrivMsg	priv;
    TrNamesMsg	trendnames;
    NoteMsg	note;
    NoteSaveMsg	notesave;
    FileMsg	file;
    ReducMsg	reduc;
    TrendMsg	trend;
    TrendSaveMsg trendsave;
    SearchMsg	search;
    RecMsg	record;
    UpdateMsg	update;
    UpdRecMsg	updrec;
} ReqUnion;

typedef union {
    PrivRsp	priv;
    TrNamesRsp	trendnames;
    NoteRsp	note;
    FileRsp	file;
    ReducRsp	reduc;
    TrendRsp	trend;
    SearchRsp	search;
    RecRsp	record;
} RespUnion;

/* function prototypes */
int main ();

int GetPrivResp ();
int GetTrNameResp ();
int GetNoteResp ();
int GetSaveNoteResp ();
int GetFileResp ();
int GetReducResp ();
int GetTrendResp ();
int GetSaveTrendResp ();
int GetDelTrendResp ();
int GetSearchResp ();
int GetRecordResp ();
int GetUpdateResp ();
int GetUpdrecResp ();
int GetDataResp ();
