/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <stdio.h>
#include <Xm/MainW.h>
#include <Xm/Label.h>
#include <Xm/BulletinB.h>
#include <Xm/SelectioB.h>
#include <Xm/Text.h>
#include <Xm/Label.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/ToggleB.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <slmDefs.h>
#include <sys/termios.h>
#include <fcntl.h>
#include <sys/stat.h>
#ifdef SOLARIS
#include <sys/ioctl.h>
#endif
#include <sys/filio.h>
#include <sys/errno.h>

#include "avl.h"
#include "DBDefines.h"
#include "DBProto.h"
#include "db_access.h"                  /* for real DBF_XXXX defs */
#include "cadef.h"                      /* EPICS channel access defs */
#include "tisEncrypt.h"
#include "msgMon.h"

/* database structure */
typedef enum {NON_REDUNDANT=0,REDUNDANT_A,REDUNDANT_B} REDUNDANCY_STATE;
typedef struct _rtu_entry
{
  struct _rtu_entry *next;
  char  slmRTUName[SLM_RTUNAME_SIZE+1];
  char  slmRTUPntName[SLM_RTUNAME_SIZE+9];	/* rtu name + _RSA.VAL */
  char  slmRTUTimeStamp[SLM_RTUNAME_SIZE+9];	/* rtu name + _ADT.VAL used to get time of file in RTU GCT 15-Apr-97 for SNWS */
  chid  slmChannelA;				/* CA channel */
  chid  slmChannelB;				/* CA channel for redundant partner if present */
  chid  slmTSchannelA;				/* CA channel for "ADT" point */
  chid  slmTSchannelB;				/* CA channel for "ADT" point for redundant partner if present */
  REDUNDANCY_STATE   slmIsRedundant;	       	/* 0 == NOT redundant 1 == redundant A 2 == redundant B */
  char  slmTSValueA[50];		       	/* value from CA from the A Time Stamp point */
  char  slmTSValueB[50];		       	/* value from CA from the B Time Stamp point */
} RTU_ENTRY,*RTU_ENTRY_PTR;

typedef struct
{
  FILE  *fp;
  RTU_ENTRY_PTR pRtu;
} FILE_NODE;
typedef struct
{
  char      slmName[SLM_NAME_SIZE+1];
  char      slmKeypadID[SLM_KEYPADID_SIZE*2+1];
  char	    slmClearID[SLM_KEYPADID_SIZE+1];	/* GCT for SNWS 10-Apr-97 clear-text password */
  char      slmCardID[SLM_CARDID_SIZE+1];
  char      slmAutoLogoutTime[SLM_TIME_SIZE+1];
  short     slmIsGlobal;	/* name has global access */
  short     slmIsOPI;		/* name has access to ALL workstations */
  RTU_ENTRY *slmRTUList;	/* tree of RTU names for this user */
}ACCESS_ENTRY, *ACCESS_ENTRY_PTR;


typedef enum
{
  PROGRAM_OK,IDNOT_YET,IOCTL_ERROR,READ_ERROR
} READ_CODE;
#define ETX 0x03
#define STX 0x02
#define ACCESS_ENTRY_SIZE sizeof(ACCESS_ENTRY)
#define RTU_ENTRY_SIZE sizeof(RTU_ENTRY)
#define SLM_KEYPAD_INUSE "Keypad ID already in use"
#define SLM_CARD_INUSE "Card ID already in use"
#define SLM_PORT_BUFF_SIZE 80
/* */
extern Widget w_main,w_toplevel,m_menubar,slmAreYouSure,slmInfo;
extern Widget w_personnel,w_rtu,w_download,w_adm,w_pgm,slmPgmButton;
extern AVL_TREE  *slmNameTree;
extern RTU_ENTRY  *slmRTUNameTree;
extern DB_TABLE_PTR slmIocListTable;
extern int maxFd,motifFd,slmFd,autoMode,slmCaFd;
extern fd_set useFds,saveFds;
extern struct termios	savetty;
extern char *sys_errlist[];
extern int errno;
/* function prototypes */
void slmExit(Widget, XtPointer, XtPointer);
void slmRTU(Widget, XtPointer, XtPointer);
void slmPersonnel(Widget, XtPointer, XtPointer);
void slmDownload(Widget, XtPointer, XtPointer);
void slmOkToQuit(Widget, XtPointer, XtPointer);
void slmWriteMain();	/* write main database file */
void slmWriteRTUTree(FILE *,RTU_ENTRY *);
void slmWriteNameTree(void *, void *);
int  slmReadMain();	/* read main database file */
void slmWriteRTUFile();
void slmBuildRTUTree();
void slmLoadRTUList(Widget);
void slmListBuild(void *,void *);
void slmWriteRTUFiles();
void slmCheckAndWrite(void *,void *);
void slmCheckPassword(Widget, XtPointer, XtPointer);
void slmCAMonitor(struct event_handler_args arg);
void slmCARead(struct	event_handler_args arg);
void slmCAConnect(struct connection_handler_args arg);
void slmReconnectCallback (void *, int , int);
/* */
void slmCreatePersonnel(Widget);
void slmCreateProgram(Widget);
void slmCreateRTU(Widget);
void slmCreateDownload(Widget);
void slmCreateAdmit(Widget);
void slmCreateAreYouSure(char *,char *);
void slmCreateInfo(char *,char *);
int  slmNameCompare(void *,void *);
int  slmRTUCompare(RTU_ENTRY_PTR,RTU_ENTRY_PTR);
void slmTreeDelete(void *);
void slmEmptyList(RTU_ENTRY_PTR);
int  slmInitPort(int, char *);			/* "borrwowed" from tisCard...port specified on cmd line */
READ_CODE  slmReadFd(int ,char *);	       	/* read port for msg from programer */
void	slmLoadDownloadRTUList(Widget list);

void slmPerInit();
void slmPerDismiss(Widget, XtPointer, XtPointer);
void slmPerOkToDismiss(Widget, XtPointer, XtPointer);
void slmPerAddReplace(Widget, XtPointer, XtPointer);
void slmPerEdit(Widget, XtPointer,XmSelectionBoxCallbackStruct *);
void slmPerNoEdit(Widget, XtPointer,XtPointer);
void slmPerOkToEdit(Widget, XtPointer,XtPointer);
void slmPerDelete(Widget, XtPointer, XtPointer);
void slmPerOkToDelete(Widget, XtPointer, XtPointer);
void slmPerAdmit(Widget, XtPointer, XtPointer);
void slmPerUpdate(Widget, XtPointer, XtPointer);
void slmPerRTUSelect(RTU_ENTRY *,Widget);
void slmPerLoadList(void *,void *);
int  slmPerCheckTime(Widget ,char *,XmTextVerifyCallbackStruct *);
int  slmPerCheckCard(Widget ,char *,XmTextVerifyCallbackStruct *);
void slmPerAddGlobal(Widget ,XtPointer, XmToggleButtonCallbackStruct *);
void slmPerToggleOPI(Widget ,XtPointer, XmToggleButtonCallbackStruct *);
void slmPerCheckUnique(void *,void *);
void slmPerCheckUniqueCard(void *,void *);
void slmPerCardChanged(Widget, XtPointer, XtPointer);
void slmPerProgram(Widget, XtPointer, XtPointer);
void slmPerNew(Widget, XtPointer, XtPointer);

void slmRTUInit();
void slmRTUDismiss(Widget, XtPointer, XtPointer);
void slmRTUSelect(Widget, XtPointer, XtPointer);
void slmRTULoadList(void *,void *);

void slmDnlInit();
void slmDnlDismiss(Widget, XtPointer, XtPointer);
void slmDnlSelectAll(Widget, XtPointer, XtPointer);
void slmDnlDownload(Widget, XtPointer, XtPointer);

void slmAdmUpdate(Widget, XtPointer, XtPointer);
RTU_ENTRY_PTR searchGlobalRTUList(char *name);

void slmPgmDismiss(Widget, XtPointer, XtPointer);	/* do nothing */
void slmPgmProgram(Widget, XtPointer, XtPointer);	/* manual program...must click this for each card */
void slmPgmAuto(Widget, XtPointer, XtPointer);		/* initiate auto-programming sequence */
void slmPgmNextCard(char *);				/* slmPortBuff contains name of last card */
void slmPgmInit(void);					/* load list of names each time */
void slmPgmFillList(void *,void *);			/* fill list with name/CardID pairs */
void slmPgmSendProgram(char *);				/* send program string to MT-80 */
void slmPgmDeselectItem(void *p1,void *p2);		/* deselect line with this card ID */
void slmPgmClearItem(char *);				/* deselect item in single program mode */
