#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vmenuHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vmenubarHEADER
#include vbuttonHEADER
#include vstartupHEADER
#include vcontrolHEADER
#include vtimerHEADER
#include vprefHEADER
#include vlistviewHEADER
#include vrulerHEADER
#include vnumHEADER

extern "C"
{
#include "usrseq.h"
}

#include "snlvu.h"
#include "snlvuApp.hxx"
#include "snlvuMain.hxx"
#include "snlvuEnter.hxx"
#include "snlvuStateSets.hxx"
#include "snlvuExpand.hxx"
#include "snlvuStateSets.hxx"
#include "snlvuUtils.hxx"


vkindTYPED_FULL_DEFINITION(StateSetsDialog, vdialog, NULL);

/*******************************************************************
*
*    StateSetsDialog override for constructors init ()
*
*******************************************************************/

void StateSetsDialog::LoadInit (vresource res)
{
    // init stuff
    PrevStateSets = NULL;
    InUse = vFALSE;

    // RequestSubLoad returns ptr to uninited dialog object
    // gets init'd when dialog gets init'd by LoadInit()

    // ruler header
    stateSetsRuler = vruler::RequestSubLoad (this, snlvu_StateSetsRuler);

    stateSetsListView = StateSetsListView::RequestSubLoad (this, snlvu_StateSetsListView);

    pauseButton = StateSetsPauseButton::RequestSubLoad (this, snlvu_PauseButton);

    continueButton = StateSetsContinueButton::RequestSubLoad (this, snlvu_ContinueButton);

    exitButton = StateSetsExitButton::RequestSubLoad (this, snlvu_ExitButton);

    // init inherited data
    vdialog::LoadInit (res);

    // store ptrs to main dialog
    stateSetsListView -> StoreStateSetsDialog (this);
    pauseButton -> StoreStateSetsDialog (this);
    continueButton -> StoreStateSetsDialog (this);
    exitButton -> StoreStateSetsDialog (this);

    Place (GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
}


/*******************************************************************
*
*    StateSetsDialog Destroy function
*
*******************************************************************/

void StateSetsDialog::Destroy (void)
{
    if (PrevStateSets != NULL)
    {
	free ((void *)PrevStateSets);
    }
    if (IsOpen ())
    {
	Close ();
    }
    vdialog::Destroy ();
}

/*******************************************************************
*
*    StateSetsDialog observe window event, looking for exit
*
*******************************************************************/

void StateSetsDialog::ObserveWindow (vevent *event)
{
    if ( event -> GetType () == veventWINDOW_STATE && event -> IsClose () )
	vevent::StopProcessing ();
}


/*******************************************************************
*
*    StateSetsDialog observe dialog reconfigure, looking for a 
*	resize event. we want to resize the ruler and its components
*	also
*
*******************************************************************/

int StateSetsDialog::HandleConfigure (vevent *event)
{
    vwindow::HandleConfigure (event);
    if ( event -> GetType () == veventWINDOW_CONFIGURE && event -> IsSize () )
    {
	int		rulerWidth;
	int		itemCount;
	int		totalItemWidth = 0;
	int		itemWidth;
	int		i;
	vrulerItem	*item;
	const vrect	*rulerRect;
	vlist *list = stateSetsListView -> GetList ();

	rulerRect = stateSetsRuler -> GetRect ();
	rulerWidth = rulerRect -> GetWidth ();
	itemCount = stateSetsRuler -> GetItemCount ();
	for (i = 0; i < itemCount; i++)
	{
	    item = stateSetsRuler -> GetItemAt (i);
	    itemWidth = item -> GetWidth ();
	    totalItemWidth += itemWidth;
	}
	if (totalItemWidth < rulerWidth)
	{
	    float	itemWidth;
	    float	itemWidthPct;
	    int		newItemWidth;
	    int		remainingWidth = rulerWidth;

	    for (i = 0; i < itemCount; i++)
	    {
		item = stateSetsRuler -> GetItemAt (i);
		itemWidth = (float)(item -> GetWidth ());
		itemWidthPct = itemWidth / (float)totalItemWidth;
		newItemWidth = (int)(itemWidthPct * rulerWidth);
		if ((remainingWidth - newItemWidth) <= 0)
		{
		    newItemWidth = remainingWidth;
		}
		item -> SetWidth (newItemWidth);
		remainingWidth -= newItemWidth;
	    }
	    // update the listviews columns widths
	    stateSetsListView -> SetColumnsWidths ();
	}
    }
    return (vTRUE);
}


/*******************************************************************
*
*    StateSetsDialog 
*
*******************************************************************/

void StateSetsDialog::OpenStateSets (MainDialog *dlg, char *progName, long tid)
{
    mainDialog = dlg;
    strncpy (programName, progName, sizeof (programName));
    programName[PROGRAM_NAME_SIZE] = 0;
    TaskId = tid;

    if (mainDialog -> pauseStateSets == vFALSE)
    {
	pauseButton -> Disable ();
    }
    if (mainDialog -> continueStateSets == vFALSE)
    {
	continueButton -> Disable ();
    }

    SetStateSetsTitle ();

    stateSetsListView -> SetDefaultCellContentIndent (0);

    UpdateStateSetsList ();

    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open ();
    vevent::Process ();
}


/*******************************************************************
*
*    StateSetsDialog function to set the dialog title 
*
*******************************************************************/

void StateSetsDialog::SetStateSetsTitle ()
{
    snlvuApplication *app = mainDialog -> GetApplication ();
    vresource titleRes = app -> GetTitlesResource ();

    vstr *titleFormat = vresourceGetString (titleRes, snlvu_StateSetsTitle);
    vstr *progName = vstrCloneScribed (vcharScribeSystem (programName));
    vstr *stateSetsTitle = vstrCloneScribed (vcharScribeFormat (titleFormat, progName));
    SetTitle (stateSetsTitle);
    vstrDestroy (progName);
    vstrDestroy (titleFormat);
    vstrDestroy (stateSetsTitle);
}


/*******************************************************************
*
*    StateSetsDialog function to update the stateSets list in the main
*	view
*
*******************************************************************/

void StateSetsDialog::UpdateStateSetsList (void)
{
    REPSS_REPLY	*stateSets;
    int		numStateSets;

    snlMon_ReportStateSets (programName, TaskId, &stateSets, &numStateSets);

    // check the list against the previous list to see if changes occured
    if (PrevStateSets != NULL)
    {
	if ((memcmp ((void *)stateSets, (void *)PrevStateSets,
		     sizeof (REPSS_REPLY) * numStateSets)) == 0)
	{
	    free ((void *)stateSets);
	    InUse = vFALSE;
	    return;
	}
	free ((void *)PrevStateSets);
    }
    // remember the new stuff
    PrevStateSets = stateSets;

    // build list for listview
    stateSetsListView -> BuildList (stateSets, numStateSets);
}



vkindTYPED_FULL_DEFINITION(StateSetsListView, vlistview, NULL);

/*******************************************************************
*
*    StateSetsListView 
*
*******************************************************************/

void StateSetsListView::Destroy (void)
{
    vlist *list = GetList ();
    if (list != NULL)
    {
	SetList (NULL);
	delete list;
    }
    vlistview::Destroy ();
}


/*******************************************************************
*
*    StateSetsListView, override
*
*******************************************************************/

static char *StatStrs [] = {"Suspend", "Pend", "Delay", "Dead", 0};

void StateSetsListView::DrawCellContents (int row, int col, int selected, vrect *cellRect)
{
    REPSS_REPLY	*p;
    char	*cPtr;
    char	buff[36];
    vlist	*viewList = GetList ();

    vdraw::GSave ();
    vdraw::RectsClip (cellRect, 1);

    p = (REPSS_REPLY *)(viewList -> GetCellValue (row, col));

    cPtr = " ";

    if (col == 0)
    {
	if (strlen (p -> StateSetName))
	{
	    cPtr = p -> StateSetName;
	}
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (cPtr), cellRect);
    }
    else if (col == 1)
    {
	vdraw::ScribedRectShowCenterY (vnum::ScribeLong (p -> TaskId), cellRect);
    }
    else if (col == 2)
    {
	if (p -> Stat == 0)
	{
	    cPtr = "Ready";
	}
	else
	{
	    int i, mask;
	    char stat[20];
	    mask = 1;
	    for (i = 0; i < 4; i++)
	    {
		if (p -> Stat & mask)
		{
		    strcat (stat, StatStrs[i]);
		    if (p -> Stat & (mask << 1))
		    {
			strcat (stat, "+");
		    }
		}
		mask <<= 1;
	    }
	    cPtr = stat;
	}
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (cPtr), cellRect);
    }
    else if (col == 3)
    {
	if (strlen (p -> InitState))
	{
	    cPtr = p -> InitState;
	}
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (cPtr), cellRect);
    }
    else if (col == 4)
    {
	if (strlen (p -> PrevState))
	{
	    cPtr = p -> PrevState;
	}
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (cPtr), cellRect);
    }
    else if (col == 5)
    {
	if (strlen (p -> CurState))
	{
	    cPtr = p -> CurState;
	}
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (cPtr), cellRect);
    }
    else if (col == 6)
    {
	sprintf (buff,"%6.1lf", p -> TimeEnt);
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (buff), cellRect);
    }
    else if (col == 7)
    {
	sprintf (buff,"%6.1lf", p -> TimeIn);
	vdraw::ScribedRectShowCenterY (vcharScribeSystem (buff), cellRect);
    }

    vdraw::GRestore ();
}


/*******************************************************************
*
*    StateSetsListView, override
*
*******************************************************************/

void StateSetsListView::ObserveSelect (int state)
{
    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	stateSetsDialog -> InUse = vTRUE;
    }
}


/*******************************************************************
*
*    StateSetsListView, build the list
*
*******************************************************************/

void StateSetsListView::BuildList (REPSS_REPLY *stateSets, int numStateSets)
{
    REPSS_REPLY	*p = stateSets;
    int		i;
    int		row = 0;

    vlist *newList = new vlist;
    vlist *oldList = GetList ();

    // all cells have same value, let DrawCellContents handle it
    for (i = 0; i < numStateSets; i++, p++)
    {
	newList -> SetCellValue (row, 0, (void *)p);
	newList -> SetCellValue (row, 1, (void *)p);
	newList -> SetCellValue (row, 2, (void *)p);
	newList -> SetCellValue (row, 3, (void *)p);
	newList -> SetCellValue (row, 4, (void *)p);
	newList -> SetCellValue (row, 5, (void *)p);
	newList -> SetCellValue (row, 6, (void *)p);
	newList -> SetCellValue (row, 7, (void *)p);
	row++;
    }
    SetList (newList);
    if (oldList != NULL)
    {
	delete oldList;
    }
    int rowCount = newList -> GetRowCount ();
    if (rowCount > 0)
    {
	SetColumnsWidths ();
	vlistSelection *selected = GetSelection ();
	newList -> SetSelectMethod (selected, vlistONE_ONLY_SELECTION);
	newList -> SetExtendRowSelection (selected, vTRUE);
    }
}


/*******************************************************************
*
*    StateSetsListView, set list column widths based on main dialog's
*	ruler item widths
*
*******************************************************************/

void StateSetsListView::SetColumnsWidths (void)
{
    vruler	*stateSetsRuler = stateSetsDialog -> GetStateSetsRuler ();
    vrulerItem	*item;
    int		itemWidth;
    int		itemCount = stateSetsRuler -> GetItemCount ();
    int		i;

    for (i = 0; i < itemCount; i++)
    {
	item = stateSetsRuler -> GetItemAt (i);
	itemWidth = item -> GetWidth ();
	SetColumnWidth (i, itemWidth);
    }
}


vkindTYPED_FULL_DEFINITION(StateSetsPauseButton, vbutton, NULL);

/*******************************************************************
*
*    StateSetsPauseButton
*
*******************************************************************/

void StateSetsPauseButton::ObserveDialogItem (vevent *event)
{
    REPSS_REPLY		*p;
    StateSetsListView	*stateSetsListView = stateSetsDialog -> GetStateSetsListView ();
    vlistSelection	*selection = stateSetsListView -> GetSelection ();
    vlist		*list = stateSetsListView -> GetList ();

    stateSetsDialog -> InUse = vTRUE;
    if (list == NULL)
    {
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_EmptyListMsg);
	stateSetsDialog -> InUse = vFALSE;
	return;
    }

    int rowCount = list -> GetRowCount ();

    if (rowCount == 0)
    {
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_EmptyListMsg);
	stateSetsDialog -> InUse = vFALSE;
	return;
    }
    if ((list -> IsSelectionEmpty (selection)) == vTRUE)
    {
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_NothingSelectedMsg);
	stateSetsDialog -> InUse = vFALSE;
	return;
    }

    int row, col;
    list -> GetSelectedCell (selection, &row, &col);
    p = (REPSS_REPLY *)(list -> GetCellValue (row, col));
    char *programName = stateSetsDialog -> GetProgramName ();
    long taskId = stateSetsDialog -> GetTaskId ();
    int result = snlMon_Suspend (programName, taskId, p -> StateSetName);
    if (result == 0)
    {
	vstr *setname = vstrCloneScribed (vcharScribeSystem (p -> StateSetName));
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_StatePauseFailMsg, 
			    setname);
	vstrDestroy (setname);
    }
    stateSetsDialog -> InUse = vFALSE;
}


vkindTYPED_FULL_DEFINITION(StateSetsContinueButton, vbutton, NULL);

/*******************************************************************
*
*    StateSetsContinueButton
*
*******************************************************************/

void StateSetsContinueButton::ObserveDialogItem (vevent *event)
{
    REPSS_REPLY		*p;
    StateSetsListView	*stateSetsListView = stateSetsDialog -> GetStateSetsListView ();
    vlistSelection	*selection = stateSetsListView -> GetSelection ();
    vlist		*list = stateSetsListView -> GetList ();

    stateSetsDialog -> InUse = vTRUE;
    if (list == NULL)
    {
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_EmptyListMsg);
	stateSetsDialog -> InUse = vFALSE;
	return;
    }

    int rowCount = list -> GetRowCount ();

    if (rowCount == 0)
    {
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_EmptyListMsg);
	stateSetsDialog -> InUse = vFALSE;
	return;
    }
    if ((list -> IsSelectionEmpty (selection)) == vTRUE)
    {
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_NothingSelectedMsg);
	stateSetsDialog -> InUse = vFALSE;
	return;
    }

    int row, col;
    list -> GetSelectedCell (selection, &row, &col);
    p = (REPSS_REPLY *)(list -> GetCellValue (row, col));
    char *programName = stateSetsDialog -> GetProgramName ();
    long taskId = stateSetsDialog -> GetTaskId ();
    int result = snlMon_Resume (programName, taskId, p -> StateSetName);
    if (result == 0)
    {
	vstr *setname = vstrCloneScribed (vcharScribeSystem (p -> StateSetName));
	snlvuMessageCenter (NULL, SNLVU_INFO_NOTICE, snlvu_StateContinueFailMsg, 
			    setname);
	vstrDestroy (setname);
    }
    stateSetsDialog -> InUse = vFALSE;
}


vkindTYPED_FULL_DEFINITION(StateSetsExitButton, vbutton, NULL);

/*******************************************************************
*
*    StateSetsExitButton
*
*******************************************************************/

void StateSetsExitButton::ObserveDialogItem (vevent *event)
{
    // stop secondary event loop started in open()
    vevent::StopProcessing ();
}

