/****
*
*	File name :		sslib.c
*	Author :		Steve Johnson
*	Creation date :	April 12, 1985
*
*	Copyright (c) 1985,1999 Sigma Six Corporation
*
*	Provides the following entry points
*
*		sslib_init
*		sslib_exit
*		sslib_add
*		sslib_remove
*		sslib_get_inihandle
*		sslib_get_tracehandle
*
*	Top level interface to Sigma Six library functions
*
****/

#include <sscommon.h>
#include <ssnames.h>
#include <ssqueues.h>
#include <sstrace.h>
#include <sslevel1.h>
#include <sstcplib.h>
#include <sslib.h>

typedef struct {
	QUEUE_DEF		link;
	int				task_id;
	INI_HANDLE		ini_handle;
	TRACE_HANDLE	trace_handle;
} TASK_CONTEXT;

static QUEUE_DEF	root_task;

static TASK_CONTEXT	*find_context (int);
static TASK_CONTEXT	*add_context (char *, char *);

int sslib_init (char *ini_file, char *ini_group)
{
	int				status;
	TASK_CONTEXT	*context;

	GetTickInit ();
	queue_init (&root_task);

	context = add_context (ini_file, ini_group);
	if (!context) {
		return -1;
	}
	status = level1_init ();
	if (LEVEL1_NORMAL != status) {
		trace_exit (context -> trace_handle);
		ini_exit (context -> ini_handle);
		queue_remove (&root_task, &context -> link);
		free (context);
		return -4;
	}
	return 0;
}

/****
*
****/

int sslib_add (char *ini_file, char *ini_group)
{
	TASK_CONTEXT	*context;

	context = add_context (ini_file, ini_group);
	if (!context) {
		return -1;
	}
	if (!localtcp_init ()) {
	 	ERROR_TRACE ("ssllib_add, localtcp_setup failed");
		ini_exit (context -> trace_handle);
		ini_exit (context -> ini_handle);
		queue_remove (&root_task, &context -> link);
		free (context);
		return -4;
	}
	return 0;
}

int sslib_remove (void)
{
	TASK_CONTEXT	*context;

	context = find_context (GET_TASK_ID ());
	if (!context) {
		return FALSE;
	}
	localtcp_exit ();
	trace_exit (context -> trace_handle);
	ini_exit (context -> ini_handle);
	queue_remove (&root_task, &context -> link);
	free (context);
	return TRUE;
}

/****
*
****/

int sslib_exit (void)
{
	TASK_CONTEXT	*context;

	level1_exit ();
	context = (TASK_CONTEXT *) queue_get (&root_task);
	while (context) {
		trace_exit (context -> trace_handle);
		ini_exit (context -> ini_handle);
		free (context);
		context = (TASK_CONTEXT *) queue_get (&root_task);
	}
	return TRUE;
}

INI_HANDLE sslib_get_inihandle (int task_id)
{
	TASK_CONTEXT		*context;

	context = find_context (task_id);
	if (!context) {
		return (TRACE_HANDLE) NULL;
	}
	return context -> ini_handle;
}

TRACE_HANDLE sslib_get_tracehandle (int task_id)
{
	TASK_CONTEXT		*context;

	context = find_context (task_id);
	if (!context) {
		return (TRACE_HANDLE) NULL;
	}
	return context -> trace_handle;
}

static TASK_CONTEXT *add_context (char *ini_file, char *ini_group)
{
	TASK_CONTEXT	*context;

	context = find_context (GET_TASK_ID ());
	if (context) {
		return context;
	}
	context = (TASK_CONTEXT *) malloc (sizeof (TASK_CONTEXT));
	if (!context) {
		return (TASK_CONTEXT *) NULL;
	}
	context -> task_id = GET_TASK_ID ();
	context -> ini_handle = (INI_HANDLE) NULL;
	context -> trace_handle = (TRACE_HANDLE) NULL;
	queue_append (&root_task, &context -> link);
	context -> ini_handle = ini_init (ini_file, ini_group);
	if (!context -> ini_handle) {
		free (context);
		return (TASK_CONTEXT *) NULL;
	}
	context -> trace_handle = trace_init (ini_group);
	if (!context -> trace_handle) {
		ini_exit (context -> ini_handle);
		free (context);
		return (TASK_CONTEXT *) NULL;
	}
	return context;
}

static TASK_CONTEXT *find_context (int task_id)
{
	TASK_CONTEXT		*context;

	context = (TASK_CONTEXT *) queue_next (&root_task);
	while (context) {
		if (context -> task_id == task_id) {
			return context;
		}
		context = (TASK_CONTEXT *) queue_next (&context -> link);
	}
	return (TASK_CONTEXT *) NULL;
}

