/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
 * The CSL Interpreter
 *
 * Each CSL script runs as a copy of this executable.
 *
 * the interpreter is invoked with:
 *	argv[1] == the script's id (required)
 *	argv[2] == the script's name (required)
 *	argv[3] == the macro substitution list (as one arg)
 */
#include	<stdio.h>
#include	<syslog.h>
#include	<stdlib.h>
#include	<malloc.h>
#include	<memory.h>
#include	<string.h>
#include	<time.h>
#include	<ctype.h>
#include	<math.h>
#include	<signal.h>
#include	<sys/wait.h>
#include	<cadef.h>
#include	<db_access.h>
#include	<msapp.h>
#include	<CSLScript.h>
#include	<CSLMonitor.h>
#include	"opcodes.h"

/* Functions defined in this module */

static	void	assert ();
static	void	connectChannels ();	/* Connect any channels needed */
static	void	connectHandler ();
static	void	eventHandler ();
static	void	writeChannel ();
static	void	termHandler ();

/* Some definitions */

#define	MAXPATHLENGTH	100
typedef	unsigned int	usint32;
typedef	int		sint32;

/* External functions */

extern	double	atof();

#define	StackMaximum		24	/* maximum number of elements on stack */
#define	SEEK_SET		0
#define	SEEK_CUR		1
#define	SEEK_END		2

/* The interpreter defines the following data types */

typedef double		StackItem;		/* things that the stack holds */
typedef	StackItem	Stack [StackMaximum];

typedef unsigned short	Opcode;		/* a program is made of a list of these */

typedef	struct {
	unsigned char	hour;
	unsigned char	minute;
	unsigned char	second;
	unsigned char	pad;
} TimeValue;			/* a time value */
	
typedef struct {
	sint32	TimerValue;
	usint32	TimerRunning;
} ATimer;			/* a timer variable */

typedef struct Operand {			/* and operates on a collection of these */
	void		*pCopy;			/* the REAL operand if this is a duplicate */
	char		type;			/* the operands type */
	char		*name;			/* pointer to the operands resolved name */
	double		dble;			/* Value of a double */
	TimeValue	time;			/* Value of a time */
	ATimer		timer;			/* Value of a time of day */
	chid		chid;			/* info for channels only */
	evid		eventId;		/* Event id for callback */
	short		connected;		/* Channel currently connected */
	short		updated;		/* Value has been updated */
	ScriptId	scriptId;		/* for lis
/stopping scripts */
	char		*scriptHostName;
} Operand;

#define	ROLE_NOREDUNDANCY	0
#define	ROLE_MASTER		1
#define	ROLE_SLAVE		2
#define	ROLE_MASTER_NOPARTNER	3

/* Redundancy messages. All padded toi be multiple of 8 bytes in length to */
/* ensure portability. */

#define	UPDATE		0
#define	SENDUPDATE	1
#define	TERMINATE	2

typedef struct {
	usint32	msgType;
} RedMsgHeader;

typedef	char	TextDouble [16];
typedef	struct {
	usint32		msgType;		/* Msg type == UPDATE */
	usint32		VarIndex;		/* Index of variable being updated */
	usint32		PC;			/* Current PC at time of update */
	usint32		NewSP;			/* Updated Stack Pointer */
	usint32		ScriptId;		/* Script Id if this is a script entry */
	char		dble [24];		/* New value of variable */
	TimeValue	time;			/* New value of variable */
	ATimer		timer;			/* New value of variable */
	TextDouble	NewStack [StackMaximum];/* Updated stack */
} UpdateMsg;

typedef	struct {
	usint32	msgType;	/* Msg type == SENDUPDATE */
	char	pad [4];
} SendUpdateMsg;

typedef	struct {
	usint32	msgType;	/* Msg type == TERMINATE */
	usint32	exitCode;	/* Exited, Killed, etc */
} TerminateMsg;

typedef union {
	RedMsgHeader	msgHeader;
	UpdateMsg	updateMsg;
	SendUpdateMsg	sendUpdateMsg;
	TerminateMsg	terminateMsg;
} RedundMsg;

/* The interpreter uses the following data */

char		localHostName [MAXHOSTNAMELENGTH+1];/* the name of the machine that we're running on */
char		environmentString [500]	= { 0 };/* string to collect local environment variables in */

Boolean		monitorAttached	= No;		/* outside resources */
ScriptId	scriptId;			/* the script's id givinn from the monitor */
char		CSLId [100];			/* the script's application name */

int		MyHandle;			/* Returned when registered with arbiter */
int		MyRole, OldRole;		/* Role played by this instance of the interpreter */

int		numChannels;			/* Number of channels referenced by the script */
#define		CHANNEL_CONNECT_TIMEOUT	15	/* All channels must connect within 15 seconds */

char		*imageName;			/* the CSL image's name (wo/subdirs) */
char		path [MAXPATHLENGTH];		/* the script's full path name */
char		**ppMacroPairs;			/* macro pair list */

CSLHeader	*Header;			/* the image's header info */
Operand		*CSLOperand;			/* the operand array pointer */
int		numOperands;
Opcode		*CSLTextStart;			/* the start of the image's text section */
Opcode		*CSLTextEnd;			/* the end of the image's text section */

Opcode		*pc;				/* the current program counter */
#define		nextImmediateOp() (*(pc++))	/* immediate operands are in the text section */

Stack		CSLStack;			/* the stack base (this may move when the stack grows) */
StackItem	*tos;				/* the current top  of the stack */

int		lineNumber = 1;			/* the current script line number */
int		TraceOn;			/* Opcode tracing off/on */
int		Debug = 1;			/* Debug output to syslog enabled */

#define		DEGREES_TO_RADIANS	0.01745329251994
#define		SETV_WAIT_SECONDS	10

ScriptStatus	exitCode = Exited;
int		InterpRunning = Yes;

/*
 * pseudo variables:
 *
 * The pseudo variables are added to the Operand table after all of the images operands
 * are resolved.  The following structs hold the info defining these pseudo variables.
 * If the users image contains definitions for the pseudo variables, makeing them real variables,
 * the associated index is reset to the index of the real operand.  The interpreter always
 * references the pseudo variable values through these indexes with the #defines below
 * (so be sure to include one!).
 */

typedef struct {
	char	*name;
	double	initialValue;
	int	index;
} PseudoVariableInfo;

PseudoVariableInfo	PseudoVariable[] = {
	{ "VTimeNow",		0.0, 0 },
	{ "VCompletionCode",	0.0, 0 },
	{ "VScriptId",		0.0, 0 },
	{ "VPauseTime",		1.0, 0 },
	{ "VTimedOut",		0.0, 0 },
	{ "VVerifyStatus",	0.0, 0 },
	{ "VTimerCounter", 	0.0, 0 },
};
int	numPseudos = (sizeof PseudoVariable) / (sizeof PseudoVariable[0]);

#define	pTimeNow		(&CSLOperand[PseudoVariable[0].index])
#define	pCompletionCode		(&CSLOperand[PseudoVariable[1].index])
#define	pScriptId		(&CSLOperand[PseudoVariable[2].index])
#define	pPauseTime		(&CSLOperand[PseudoVariable[3].index])
#define	pTimedOut		(&CSLOperand[PseudoVariable[4].index])
#define	pVerifyStatus		(&CSLOperand[PseudoVariable[5].index])
#define	pTimerCounter		(&CSLOperand[PseudoVariable[6].index])
#define	TIMER_COUNTER_INDEX	PseudoVariable[6].index

/*
 * The following CSLxxx functions are local 'helper' functions used by the opcode
 * implementation functions to help them do their job.
 */

/*----------*/
void CSLsleep (delay)	/* sleep for the given delay seconds */

    long	delay;

    {
	time_t	currentTime = (time_t)time(0);
	time_t	wakeupTime = currentTime + delay;

	while(wakeupTime > (currentTime = (time_t)time(0))) {	/* narrow in on the wakeup time */
	    unsigned sleepTime = (unsigned) ((wakeupTime - currentTime)/2);
	    sleep(sleepTime > 0 ? sleepTime : 1);
	}
    }

/*----------*/
long CSLseconds (timeInterval, unitsIndex)	/* calculate a number of seconds from the given data */

    double	timeInterval;
    int		unitsIndex;

    {

	switch (CSLOperand[unitsIndex].name[0]) {
	    case 'h':
		timeInterval *= 60.0;		/* NO breaks please! */
	    case 'm':
		timeInterval *= 60.0;
	    case 's':
		timeInterval *=  1.0;
	}

	return timeInterval;

    }

/*----------*/
long CSLdeltaSeconds (timeIndex)	/* calculate a number of seconds between now and the given data */

    int	timeIndex;

    {

	Operand	*pOp = &CSLOperand[timeIndex];
	long	delayHours = pOp->time.hour - pTimeNow->time.hour;
	long	delayMinutes = pOp->time.minute - pTimeNow->time.minute;
	long	delaySeconds = pOp->time.second - pTimeNow->time.second;

	if (delaySeconds < 0) {
	    delaySeconds += 60;
	    delayMinutes -= 1;
	}
	if (delayMinutes < 0) {
	    delayMinutes += 60;
	    delayHours   -= 1;
	}
	if (delayHours < 0)
	    delayHours += 24;

	return ((((delayHours * 60L) + delayMinutes) * 60L) + delaySeconds);

    }


/*----------*/
CSLexecute (scriptType, scriptIndex, pHostName, preStartDelay, waitConstraint)

    ScriptType		scriptType;
    int			scriptIndex;
    char		*pHostName;
    long		preStartDelay;
    WaitConstraint	waitConstraint;

    {

	char	errorTextBuffer[100];
	Operand	*pScript;

	pScript = &CSLOperand [scriptIndex];

	pScript->scriptHostName	= pHostName;
	pScript->scriptId = CSLMonitorExecute(pScript->scriptHostName, scriptType,
		pScript->name, preStartDelay, environmentString);
	sprintf(errorTextBuffer, "CSLMonitorExecute(...%s,...) failed!", pScript->name);
	assert(pScript->scriptId != ERROR, errorTextBuffer);
	
	pScriptId->scriptHostName = pHostName;
	pScriptId->scriptId = pScript->scriptId;
	pScriptId->dble = pScript->scriptId;

	if (waitConstraint == Linger) {
	    ScriptStatus scriptStatus;
	    SendUpdate (scriptIndex);	/* Update slave before sleeping */
	    do {
		scriptStatus = CSLMonitorGetScriptStatus(pHostName,
				pScript->scriptId, NULL);
		sleep(1);
	    } while (scriptStatus == Scheduled || scriptStatus == Running);
	}

	return scriptIndex;	/* Update the slave */
    }

/*----------*/
void addEnvironmentVariable (variableName)

    char	*variableName;

    {

	char	*getenv(), *s = getenv(variableName);
	char	buf[100];

	sprintf(buf, "%s=%s;export %s;", variableName, s ? s : "", variableName);
	if ((strlen(environmentString) + strlen(buf)) < sizeof environmentString)
		strcat(environmentString, buf);
    }

/*----------*/
void InitializeInterpreter (argc, argv)

    int		argc;
    char	*argv[];
	
    {

	char	*macroList;	/* macro substitution string, ex: "rf=RF1,cav=CAV1" */
	char	*p;
	int	numReplacements;
	int	Status;

	/*
	 * get our host's name
	 * collect the command line arguments
	 * attach to the local monitor server
	 * initialize the environment variable string
	 */

	gethostname(localHostName, sizeof localHostName - 1);
	localHostName[MAXHOSTNAMELENGTH] = 0;	/* ensure null-termination */

	if (argc < 3 || argc > 4) {
	    printf( "usage:CSLInterpreter scriptId imageName [macroList]\n");
	    exit(1);
	}
	scriptId = atoi(argv[1]);
	imageName = argv[2];
	macroList = (argc > 3) ? argv[3] : "";
	sprintf(CSLId, "CSL%d", scriptId);

	monitorAttached = (CSLMonitorAttach(localHostName, scriptId) == TF_OK) ? Yes : No;
	assert(monitorAttached == Yes, "CSLMonitorAttach() failed in InitializeInterpreter()");

	addEnvironmentVariable("USER");
	addEnvironmentVariable("HOME");
	addEnvironmentVariable("DISPLAY");

	/* Construct the script's full pathname */

	if (*imageName != '/') {
	    char *scriptPath = getenv(CSL_SCRIPT_PATH);

	    /* Use the script path from the environment or use '.'
		then append the images name to this path */

	    strcat(strcat(strcpy(path, scriptPath ? scriptPath : "."), "/"), imageName);
	} else
	    strcpy(path, imageName);
	strcat(path, CSL_SCRIPT_EXTENSION);

	/* Get the macro definitions */

	for (p = macroList, numReplacements = 0; *p; p++)  /* count the number of replacements */
	    if (*p == '=')
		numReplacements++;

	ppMacroPairs = (char **)calloc((numReplacements*2)+1, sizeof(char *));
	get_macro_pairs(macroList, ppMacroPairs, numReplacements);
	
	/* Register with the msArbiter */

	if (appRegister(CSLId, &MyHandle)) {
	    int	Status;
	    if (!appGetStatus (MyHandle, &Status)) {
		syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter has died -- Terminating CSL script.",
				scriptId);
		exit (-1);
	    }
	    switch (Status) {
		case MASTER:
		    MyRole = ROLE_MASTER;
		    break;
		case SLAVE:
		    MyRole = ROLE_SLAVE;
		    break;
		case MASTER_NOPARTNER:
		    MyRole = ROLE_MASTER_NOPARTNER;
		    break;
	    }
	    if (Debug)
		syslog (LOG_OPTIONS, "CSLInt(%d): Registered as %s\n",
				scriptId, CSLId);
	} else {
	    MyRole = ROLE_NOREDUNDANCY;
	    if (Debug)
		syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter not running -- no redundancy.\n",
			scriptId);
	}

    }


/*----------*/
char *fixupShort (p)	/* byte swap to put the short into little-endian form */

    char	*p;

    {

	char	tmp;

	tmp  = p[0];
	p[0] = p[1];
	p[1] = tmp;

	return p + 2;

    }

/*----------*/
char *fixupLong (p)		/* byte swap to put the long into little-endian form */

    char	*p;
	
    {

	char	tmp;

	tmp  = p[0];
	p[0] = p[3];
	p[3] = tmp;
	tmp  = p[1];
	p[1] = p[2];
	p[2] = tmp;

	return p + 4;

    }

/*----------*/
void FixupByteOrdering (image, filelen)

    char	*image;
    long	filelen;

    {

	char	*p = image;
	char	*pend = image + filelen;
	char	tmp;

	/* Order the header */

	p = fixupLong(p);	/* MAGIC */
	p = fixupLong(p);	/* MAJOR */
	p = fixupLong(p);	/* MINOR */

	/* Skip over the list of operand strings */

	while (*p && (p < pend))
	    p++;
	p++;

	/* Order the list of opcodes */
	while (p < pend)
	    p = fixupShort(p);

    }

/*----------*/
char *resolveName (pp, pFormat)

    char	**pp;
    char	*pFormat;	/* includes a %s where caller wants the resolved name */

    {

	char	*pOldName;
	char	*pNewName;
	char	buf1 [100];
	char	buf2 [100];

	/*
	 * find the end of the string (\n) and terminate it
	 * apply the macro definitions
	 * apply the callers format
	 * allocate enough space for it, copy it and return it
	 * (checking for \0 allows this to work for PseudoVariable list too)
	 */

	for (pOldName = *pp; **pp && (**pp != '\n'); (*pp)++);
	if (**pp != 0)
	    **pp = 0;

	gsdl_main_macro (buf1, pOldName, ppMacroPairs, sizeof buf1);
	sprintf (buf2, pFormat, buf1);

	pNewName = (char *) calloc (strlen (buf2) + 1, sizeof (char));
	assert (pNewName != NULL, "out of memory in resolveName()");

	return strcpy(pNewName, buf2);

    }

/*----------*/
char *resolveChannel (p, pOperand)

    char	*p;
    Operand	*pOperand;

    {

	char	Str [50];
	int	result;

	pOperand->type = *p++;
	pOperand->name = resolveName (&p, "%s");

	return p;

    }

/*----------*/
char *resolveVariable (p, pOperand)

    char	*p;
    Operand	*pOperand;

    {

	int	i;

	pOperand->type = *p++;
	pOperand->name = resolveName(&p, "%s");
	pOperand->dble = 0.0;			/* in case atof() fails */
	pOperand->dble = atof(pOperand->name);	/* in case it resolves to a constant */
	pOperand->pCopy = NULL;

	/* Check for duplicate variable names */

	for ( i = 0; pOperand->name != CSLOperand[i].name; i++ )
	   if (pOperand->type == 'V' && strcmp(pOperand->name, CSLOperand[i].name) == 0) {
		pOperand->pCopy = &CSLOperand[i];
		break;
	   }

	return p;

    }

/*----------*/
char *resolveTimer (p, pOperand)

    char	*p;
    Operand	*pOperand;

    {

	static char format [sizeof localHostName + 10 + 1];
	int	status;

	pOperand->type = *p++;
	p++;	/* skip @ */
	pOperand->name	= resolveName(&p, format);
	pOperand->timer.TimerRunning = No;
	pOperand->timer.TimerValue = 0;

	return p;

    }

/*----------*/
char *resolveIntegerConstant (p, pOperand)

    char	*p;
    Operand	*pOperand;

    {

	pOperand->type = *p++;
	pOperand->name = resolveName(&p, "%s");
	pOperand->dble = atof(pOperand->name);

	return p;

    }

/*----------*/
char *resolveRealConstant (p, pOperand)

    char	*p;
    Operand	*pOperand;

    {

	pOperand->type = *p++;
	pOperand->name = resolveName(&p, "%s");
	pOperand->dble = atof(pOperand->name);

	return p;

    }

/*----------*/
char *resolveLiteralConstant (p, pOperand)

    char	*p;
    Operand	*pOperand;

    {

	pOperand->type = *p++;
	pOperand->name = resolveName(&p, "%s");

	return p;

    }

/*----------*/
char *resolveTimeConstant(p, pOperand)

    char	*p;
   Operand	*pOperand;

    {

	char	*s;

	pOperand->type = *p++;
	pOperand->name = resolveName(&p, "%s");

	s = pOperand->name;
	pOperand->time.hour = atoi(s);		/* pick up the hour */
	assert(isdigit(*s), "no hours digit");		/* need at least one digit */
	s++;						/* consume h */
	if (isdigit(*s))
	    s++;					/* consume h */
	assert(*s == ':',   "separator is not a :");	/* need at least h:m */
	s++;						/* consume : */

	pOperand->time.minute = atoi(s);		/* pick up the minute */
	assert(isdigit(*s), "no minutes digit");	/* need at least one digit */
	s++;						/* consume m */
	if (isdigit(*s))
	    s++;					/* consume m */

	if (*s == ':') {
	    s++;					/* consume : */
	    pOperand->time.second = atoi(s);	/* pick up the second */
	    assert(isdigit(*s), "no seconds digit");	/* need at least one digit */
	    s++;					/* consume s */
	    if (isdigit(*s))
		s++;					/* consume s */
	} else
	    pOperand->time.second = 0;

	if (*s) {
	    /* check for am or pm */
	    *s = tolower(*s);
	    assert((*s == 'p') || (*s == 'a'), "time constant ends with something that is not am or pm");
	    if (*s == 'p')
		pOperand->time.hour += 12;	/* handle pm case */
	    s++;					/* consume p */
	    *s = tolower(*s);
	    assert(*s == 'm', "time constant ends with something that is not am or pm");
	}

	return p;

    }

/*----------*/
void GetCSLImage()

    {

	FILE	*CSLFile;
	long	filelen;
	char	*image, *p, *d, *s;
	Operand	*pOperand;
	int	i, j;
	size_t	bytesRead;

	/* Open the image file, get it's length, read it into a malloc'ed buffer and close it */

	CSLFile = fopen (path, "r");
	if (CSLFile == NULL) {
	    printf ("Unable to open CSL script file: %s\n", path);
	    exit (-1);
	}
	
	fseek (CSLFile, 0L, SEEK_END);
	filelen = ftell (CSLFile);
	assert(filelen >= (sizeof (CSLHeader) + 1 + 2), "CSL image is too small");
			/* header + string null + 1 opcode */

	image = malloc (filelen);
	assert(image != NULL, "out of memory in GetCSLImage()");

	fseek (CSLFile, 0L, SEEK_SET);
	bytesRead = fread(image, sizeof (char), filelen, CSLFile);
	assert(bytesRead == filelen, "can't read CSL image");

	fclose (CSLFile);

	/* Fixup the byte-ordering and check the header */

#ifndef BYTEORDER_0123
	FixupByteOrdering (image, filelen);
#endif

	Header = (CSLHeader *)image;
	assert(Header->Magic == CSL_MAGIC, "bad magic");
	assert(Header->Major == CSL_MAJOR_VERSION, "bad major version");
	assert(Header->Minor == CSL_MINOR_VERSION, "bad minor version");

	/*
	 * set up the operand table:
	 *	count the number of operands and allocate memory for the Operand table
	 *	run thru the string table and resolve them
	 *	add in the pseudo variables
	 */

	for (p = image + sizeof(CSLHeader), numOperands = 0; *p; p++, numOperands++)
	    while (*p && *p != '\n')
		p++;
	CSLOperand = (Operand *) calloc (numOperands + numPseudos, sizeof (Operand));
	assert(CSLOperand != NULL, "out of memory in GetCSLImage()");
	
	for (p = image + sizeof(CSLHeader), i = 0, pOperand = &CSLOperand[0];
		i < numOperands; p++, i++, pOperand++) {
	   switch (*p) {
		default:
		    assert(1, "bad operand found");
		case 'C':
		    p = resolveChannel (p, pOperand);
		    break;
		case 'V':
		    p = resolveVariable (p, pOperand);
		    break;
		case 'T':
		    p = resolveTimer (p, pOperand);
		    break;
		case 'I':
		    p = resolveIntegerConstant (p, pOperand);
		    break;
		case 'R':
		    p = resolveRealConstant (p, pOperand);
		    break;
		case 'L':
		    p = resolveLiteralConstant (p, pOperand);
		   break;
		case 't':
		    p = resolveTimeConstant (p, pOperand);
		    break;
	    }
	}
	
	for ( j = 0; j < numPseudos; j++, i++, pOperand++ ) {
	    PseudoVariable[j].index = i;
	    resolveVariable (PseudoVariable[j].name, pOperand);
	    if (pOperand->pCopy != NULL)
		PseudoVariable[j].index = ((char *)(pOperand->pCopy) -
				(char *)&CSLOperand[0])/sizeof CSLOperand[0];
	    CSLOperand[PseudoVariable[j].index].dble = PseudoVariable[j].initialValue;
	}

	/*
	 * setup the text segment:
	 *	copy the text segment over the original strings to recover some space
	 *	and to ensure that the opcodes are aligned on a 2 byte boundary
	 */

	for (d = image + sizeof(CSLHeader), s = ++p; s < (image + filelen); )
	    *d++ = *s++;
	realloc (image, (size_t)(d - image));

	CSLTextStart = (Opcode *)(image + sizeof(CSLHeader));
	CSLTextEnd = (Opcode *)((char *)CSLTextStart + ((size_t)(d - image) - sizeof(CSLHeader)));
	pc = CSLTextStart;

    }

/*----------*/
FreeResources ()

    {

	int	i;

	/* Unhook from the monitor and the database */

	if (monitorAttached == Yes)
	    CSLMonitorDetach (localHostName, scriptId);

	for (i = 0; i < numOperands; i++)
	    if (CSLOperand[i].type == 'C' && CSLOperand[i].chid)
		ca_clear_channel(CSLOperand[i].chid);
	ca_task_exit();

	if (MyRole != ROLE_NOREDUNDANCY)
	    appUnRegister (MyHandle);

    }

/*----------*/
findScriptIndex (scriptId)

    int	scriptId;

    {

	int	i;
	
	for (i = 0; i < numOperands; i++)
	    if (CSLOperand[i].scriptId == scriptId)
		return i;
	return 0;

    }

/*----------*/
StackItem *InitializeStack ()

    {

	tos = &CSLStack[-1];	/* -1 along with pre-inc allows CSLStack [0] to be first elem */

    }

/*
 * the instruction functions:
 *
 * these functions return:
 *	- an operand index (>=0) to cause that operand to be sent to the slave partner
 *	- NO_UPDATE if an update is not required
 *	- PC_UPDATE if only a PC update is required
 */
#define	NO_UPDATE	-1
#define	PC_UPDATE	-2

/*----------*/
SendUpdate (opIndex)

    int	opIndex;

    {

	UpdateMsg	msg;
	int		i, SP;

	/* If no update requested, just return */

	if (opIndex == NO_UPDATE)
	    return;

	/* If we are the master (with a partner), send an update message to the slave */

	if (MyRole == ROLE_MASTER) {

	    msg.msgType	= htonl (UPDATE);

	    /* opIndex tells which variable is being updated. <-2 => PC update only */

	    msg.VarIndex = htonl (opIndex);
	    if (opIndex >= 0) {
		sprintf (msg.dble, "%.15le", CSLOperand[opIndex].dble);
		msg.time = CSLOperand [opIndex].time;
		msg.timer.TimerValue = htonl (CSLOperand [opIndex].timer.TimerValue);
		msg.timer.TimerRunning = htonl (CSLOperand [opIndex].timer.TimerRunning);

		/* Script Id in case this entry is for a script */

		msg.ScriptId = htonl (CSLOperand[opIndex].scriptId);

	    }

	    /* The PC and Stack are also sent */

	    msg.PC = htonl (pc - CSLTextStart);
	    SP = tos - CSLStack;
	    for (i=0 ; i<SP ; i++) {
		sprintf (msg.NewStack [i], "%.15le", CSLStack [i]);
	    }
	    msg.NewSP = htonl (SP);

	    if (Debug) {
		if (opIndex >= 0)
		    syslog (LOG_OPTIONS, "CSLInt(%d): Sending update, ix=%d(%s), Val=%10.2lf, pc=%d",
				scriptId, opIndex, CSLOperand [opIndex].name, CSLOperand[opIndex].dble,
				pc - CSLTextStart);
		else
		    syslog (LOG_OPTIONS, "CSLInt(%d): Sending PC update, pc=%d",
				scriptId, pc - CSLTextStart);
	    }

	    if (!appSendData(MyHandle, (char *)&msg, sizeof (msg))) {
		syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter has died -- Terminating CSL script.",
			scriptId);
		exit (-1);
	    }

	}

    }

/*----------*/
Fnop()

    {

	return NO_UPDATE;	/* that was quick */

    }

/*----------*/
Fjump()		/* the pc is bounds checked in ExecuteNextOpcode() because of these jump ops */

    {

	pc = &CSLTextStart[*(pc)];
	return NO_UPDATE;

    }

/*----------*/
Fjzero()

    {

	pc = (tos[0] == 0.0) ? &CSLTextStart[*(pc)] : pc + 1;
	return NO_UPDATE;

    }

/*----------*/
Fjnzero()

    {

	pc = (tos[0] != 0.0) ? &CSLTextStart[*(pc)] : pc + 1;
	return NO_UPDATE;

    }

/*----------*/
Feq()

    {

	tos[-1] = (tos[-1] == tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fne()

    {

	tos[-1] = (tos[-1] != tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fgt()

    {

	tos[-1] = (tos[-1] > tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fge()

    {

	tos[-1] = (tos[-1] >= tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Flt()

    {

	tos[-1] = (tos[-1] < tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fle()

    {

	tos[-1] = (tos[-1] <= tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fleq()

    {

	char	*s1 = CSLOperand [(int)(tos[-1])].name;
	char	*s2 = CSLOperand [(int)(tos[0])].name;

	tos[-1] = (strcmp(s1, s2) == 0);
	return NO_UPDATE;

    }

/*----------*/
Flne()

    {

	char	*s1 = CSLOperand [(int)(tos[-1])].name;
	char	*s2 = CSLOperand [(int)(tos[0])].name;

	tos[-1] = (strcmp(s1, s2) != 0);
	return NO_UPDATE;

    }

/*----------*/
Flgt()

    {

	char	*s1 = CSLOperand [(int)(tos[-1])].name;
	char	*s2 = CSLOperand [(int)(tos[0])].name;

	tos[-1] = (strcmp(s1, s2) > 0);
	return NO_UPDATE;

    }

/*----------*/
Flge()

    {

	char	*s1 = CSLOperand [(int)(tos[-1])].name;
	char	*s2 = CSLOperand [(int)(tos[0])].name;

	tos[-1] = (strcmp(s1, s2) >= 0);
	return NO_UPDATE;

    }

/*----------*/
Fllt()

    {

	char	*s1 = CSLOperand [(int)(tos[-1])].name;
	char	*s2 = CSLOperand [(int)(tos[0])].name;

	tos[-1] = (strcmp(s1, s2) < 0);
	return NO_UPDATE;

    }

/*----------*/
Flle()

    {

	char	*s1 = CSLOperand [(int)(tos[-1])].name;
	char	*s2 = CSLOperand [(int)(tos[0])].name;

	tos[-1] = (strcmp(s1, s2) <= 0);
	return NO_UPDATE;

    }

/*----------*/
Fand()

    {

	tos[-1] = (tos[-1] && tos[0]);
	return NO_UPDATE;

    }

/*----------*/
For()

    {

	tos[-1] = (tos[-1] || tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fnot()

    {

	tos[0] = !(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fadd()

    {

	tos[-1] = (tos[-1] + tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fsub()

    {

	tos[-1] = (tos[-1] - tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fmult()

    {

	tos[-1] = (tos[-1] * tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fdiv()

    {
	assert(tos[0] != 0.0, "attempt to divide by zero");
	tos[-1] = (tos[-1] / tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fpower()

    {

	tos[-1] = pow(tos[-1], tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fneg()

    {

	tos[0] = -(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fabs()

    {

	tos[0] = fabs(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Facos()

    {

	tos[0] = acos(tos[0]) / DEGREES_TO_RADIANS;
	return NO_UPDATE;

    }

/*----------*/
Fasin()

    {

	tos[0] = asin(tos[0]) / DEGREES_TO_RADIANS;
	return NO_UPDATE;

    }

/*----------*/
Fatan()

    {

	tos[0] = atan(tos[0]) / DEGREES_TO_RADIANS;
	return NO_UPDATE;

    }

/*----------*/
Fceiling()

    {

	tos[0] = ceil(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fcos()

    {

	tos[0] = cos(tos[0] * DEGREES_TO_RADIANS);
	return NO_UPDATE;

    }

/*----------*/
Fexp()

    {

	tos[0] = exp(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Ffloor()

    {

	tos[0] = floor(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fln()

    {

	assert(tos[0] > 0.0, "attempt to take ln of negative number");
	tos[0] = log(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Flog()

    {

	assert(tos[0] > 0.0, "attempt to take log of negative number");
	tos[0] = log10(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Fmax()

    {

	tos[-1] = (tos[0] > tos[-1]) ? tos[0] : tos[-1];
	return NO_UPDATE;

    }

/*----------*/
Fmin()

    {

	tos[-1] = (tos[0] < tos[-1]) ? tos[0] : tos[-1];
	return NO_UPDATE;

    }

/*----------*/
Fmod()

    {

	assert(tos[0] != 0.0, "attempt to take mod 0.0");
	tos[-1] = ((long)(tos[-1]) % (long)(tos[0]));
	return NO_UPDATE;

    }

/*----------*/
Fsin()

    {

	tos[0] = sin(tos[0] * DEGREES_TO_RADIANS);
	return NO_UPDATE;

    }

/*----------*/
Fsqrt()

    {

	assert(tos[0] >= 0.0, "attempt to take sqrt of negative number");
	tos[0] = sqrt(tos[0]);
	return NO_UPDATE;

    }

/*----------*/
Ftan()

    {

	tos[0] = tan(tos[0] * DEGREES_TO_RADIANS);
	return NO_UPDATE;

    }

/*----------*/
Fpush()

    {

	tos[1] = nextImmediateOp();
	return NO_UPDATE;

    }

/*----------*/
Fpop()

    {

	return NO_UPDATE;

    }

/*----------*/
Fxchg()

    {

	StackItem	temp;

	temp	= tos[-1];
	tos[-1]	= tos[0];
	tos[0]	= temp;
	return NO_UPDATE;

    }

/*----------*/
Fload()

    {

	int		opIndex	= nextImmediateOp();
	Operand		*pOp = &CSLOperand[opIndex];
	StackItem	value;

	switch (pOp->type) {
	    case 'C':
		value = pOp->dble;
		break;
	    case 'I':
		value = pOp->dble;
		break;
	    case 'L':
		value = pOp->dble;
		break;
	    case 'R':
		value = pOp->dble;
		break;
	    case 'T':
		value = ((value = pOp->timer.TimerValue -
				  (pOp->timer.TimerRunning ?
				  time(0) : 0)) < 0) ? 0 : value;
		break;
	    case 't':
		value = (((pOp->time.hour * 60.0) +
			pOp->time.minute) * 60.0) +
			pOp->time.second;
		break;
	    case 'V':
		value = (pOp->pCopy ? (Operand *)(pOp->pCopy) : pOp)->dble;
		break;
	}
	tos[1] = value;
	return NO_UPDATE;

    }

/*----------*/
Fstore()

    {

	int		opIndex	= nextImmediateOp();
	Operand		*pOp = &CSLOperand[opIndex];
	StackItem	value = tos[0];

	switch (pOp->type) {
	    case 'C':
		writeChannel(opIndex, pOp->dble = value);
		return PC_UPDATE;
	    case 'T':
		pOp->timer.TimerValue = value;
		pOp->timer.TimerRunning = No;
		return opIndex;
	    case 'V':
		(pOp->pCopy ? (Operand *)(pOp->pCopy) : pOp)->dble = value;
		return opIndex;
	    case 'I':	/* don't change the constants */
		break;
	    case 'L':
		break;
	    case 'R':
		break;
	    case 't':
		break;
	}
	return NO_UPDATE;

    }

/*----------*/
Ftstart()

    {

	int	timerIndex = nextImmediateOp();

	if (CSLOperand[timerIndex].timer.TimerRunning == Yes)
	    return NO_UPDATE;

	CSLOperand[timerIndex].timer.TimerValue += time(0);
	CSLOperand[timerIndex].timer.TimerRunning = Yes;
	return timerIndex;

    }

/*----------*/
Ftstop()

    {

	int	timerIndex = nextImmediateOp();

	if (CSLOperand[timerIndex].timer.TimerRunning == No)
	    return NO_UPDATE;

	CSLOperand[timerIndex].timer.TimerValue	-= time (0);
	CSLOperand[timerIndex].timer.TimerRunning = No;

	if (CSLOperand[timerIndex].timer.TimerValue < 0)
	    CSLOperand[timerIndex].timer.TimerValue = 0;
	return timerIndex;

    }

/*----------*/
Ftexp()

    {
	int	timerIndex = nextImmediateOp();

	tos [1]	= (CSLOperand[timerIndex].timer.TimerValue < time(0));	
	return NO_UPDATE;

    }

/*----------*/
Fcomd()

    {

	int	deviceIndex = nextImmediateOp();
	int	commandIndex = nextImmediateOp();

	ca_put(DBR_STRING, CSLOperand[deviceIndex].chid, CSLOperand[commandIndex].name);
	ca_flush_io();
	return PC_UPDATE;

    }

/*----------*/
Fstoretint()

    {

	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();

	pTimerCounter->dble = CSLseconds(timeInterval, unitsIndex);
	return TIMER_COUNTER_INDEX;

    }

/*----------*/
Fstorettod()

    {

	int	timeIndex = nextImmediateOp();

	pTimerCounter->dble = CSLdeltaSeconds(timeIndex);
	return TIMER_COUNTER_INDEX;

    }

/*----------*/
Fdecrt ()

    {

	tos [1] = pTimerCounter->dble;
	pTimerCounter->dble--;
	return TIMER_COUNTER_INDEX;

    }

/*----------*/	
Fwaitcsl()

    {

	int	scriptIndex = nextImmediateOp();
	Operand	*pScript = &CSLOperand[scriptIndex];
	ExtendedScriptStatus	extendedScriptStatus;

	SendUpdate (scriptIndex);

	while (CSLMonitorGetScriptStatus(pScript->scriptHostName,
			pScript->scriptId, &extendedScriptStatus) == Running)
		sleep(1);
	pCompletionCode->dble = extendedScriptStatus.completionCode;
	return PC_UPDATE;

    }

/*----------*/
Fwaitcslt()

    {

	int	scriptIndex = nextImmediateOp();
	Operand	*pScript = &CSLOperand[scriptIndex];
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	waitTime= CSLseconds(timeInterval, unitsIndex);
	ExtendedScriptStatus	extendedScriptStatus;

	SendUpdate (scriptIndex);

	while ((waitTime-- > 0) &&
	       (CSLMonitorGetScriptStatus(pScript->scriptHostName,
			pScript->scriptId, &extendedScriptStatus) == Running))
		sleep(1);
	pCompletionCode->dble = extendedScriptStatus.completionCode;
	pTimedOut->dble = (waitTime == 0);
	return PC_UPDATE;

    }

/*----------*/
Fpause()

    {

	CSLsleep((long)(pPauseTime->dble));
	return PC_UPDATE;

    }

/*----------*/
Faccess()

    {

	int	channelIndex = nextImmediateOp();

	ca_flush_io();
	tos[1] = (ca_state(CSLOperand[channelIndex].chid) == cs_conn);
	return NO_UPDATE;

    }

/*----------*/
Fset()

   {

	int	channelIndex = nextImmediateOp();
	Operand	*pOp = &CSLOperand[channelIndex];
	pOp->dble = tos[0];
	writeChannel(channelIndex, pOp->dble);
	return PC_UPDATE;

    }

/*----------*/
Fsetv()

    {

	int	channelIndex = nextImmediateOp();
	Operand	*pOp = &CSLOperand[channelIndex];
	double	newValue;
	int	startTime;

	/* If the value we are about to write is the same as the known current */
	/* value, write it and skip the verify procedure. */

	if (pOp->dble == tos [0]) {
	    writeChannel (channelIndex, pOp->dble);
	    pVerifyStatus->dble = 1;
	    return PC_UPDATE;
	}

	/* Write a value to CA and wait for the value to update */

	newValue = tos[0];
	writeChannel (channelIndex, newValue);
	startTime = time (0);
	while ((time (0) - startTime) < SETV_WAIT_SECONDS) {
	    ca_pend_io (1.0);
	    if (pOp->updated) {
		if (pOp->dble == newValue) {
		    pVerifyStatus->dble = 1;
		    return PC_UPDATE;
		} else {
		    pVerifyStatus->dble = 0;
		    return PC_UPDATE;
		}
	    }
	}

	/* Time has expired */

	pVerifyStatus->dble = 0;
	return PC_UPDATE;

    }

/*----------*/
Falarm()

    {

	int	channelIndex = nextImmediateOp();
	int	severityIndex = nextImmediateOp();

	assert(1, "alarm opcode attempted");

    }

/*----------*/
Fscsl()

    {

	int	scriptIndex = nextImmediateOp();

	SendUpdate (scriptIndex);

	return CSLexecute(CSLscript, scriptIndex, localHostName, 0L, Continue);

    }

/*----------*/
Fscslw()

    {

	int	scriptIndex = nextImmediateOp();

	SendUpdate (scriptIndex);

	return CSLexecute(CSLscript, scriptIndex, localHostName, 0L, Linger);

    }

/*----------*/
Fscsltod()

    {

	int	scriptIndex = nextImmediateOp();
	int	timeIndex = nextImmediateOp();
	long	delay = CSLdeltaSeconds(timeIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(CSLscript, scriptIndex, localHostName, delay, Continue);

    }

/*----------*/
Fscsltodw()

    {

	int	scriptIndex = nextImmediateOp();
	int	timeIndex = nextImmediateOp();
	long	delay = CSLdeltaSeconds(timeIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(CSLscript, scriptIndex, localHostName, delay, Linger);

    }

/*----------*/
Fscslint()

    {

	int	scriptIndex = nextImmediateOp();
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	delay = CSLseconds(timeInterval, unitsIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(CSLscript, scriptIndex, localHostName, delay, Continue);

    }

/*----------*/
Fscslintw()

    {

	int	scriptIndex = nextImmediateOp();
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	delay = CSLseconds(timeInterval, unitsIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(CSLscript, scriptIndex, localHostName, delay, Linger);

    }

/*----------*/
Fssh()

    {

	int	scriptIndex = nextImmediateOp();

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, localHostName, 0L, Continue);
    }

/*----------*/
Fsshw()

    {

	int	scriptIndex = nextImmediateOp();

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, localHostName, 0L, Linger);

    }

/*----------*/
Fsshtod()

    {

	int	scriptIndex = nextImmediateOp();
	int	timeIndex = nextImmediateOp();
	long	delay = CSLdeltaSeconds(timeIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, localHostName, delay, Continue);

    }

/*----------*/
Fsshtodw()

    {

	int	scriptIndex = nextImmediateOp();
	int	timeIndex = nextImmediateOp();
	long	delay = CSLdeltaSeconds(timeIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, localHostName, delay, Linger);

    }

/*----------*/
Fsshint()

    {

	int	scriptIndex = nextImmediateOp();
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	delay = CSLseconds(timeInterval, unitsIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, localHostName, delay, Continue);

    }

/*----------*/
Fsshintw()

    {

	int	scriptIndex = nextImmediateOp();
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	delay = CSLseconds(timeInterval, unitsIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, localHostName, delay, Linger);

    }

/*----------*/
Fsshh()

    {

	int	scriptIndex = nextImmediateOp();
	int	hostIndex = nextImmediateOp();

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, CSLOperand[hostIndex].name, 0L, Continue);

    }

/*----------*/
Fsshwh()

    {

	int	scriptIndex = nextImmediateOp();
	int	hostIndex = nextImmediateOp();

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, CSLOperand[hostIndex].name, 0L, Linger);

    }

/*----------*/
Fsshtodh()

    {

	int	scriptIndex = nextImmediateOp();
	int	hostIndex = nextImmediateOp();
	int	timeIndex = nextImmediateOp();
	long	delay = CSLdeltaSeconds(timeIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, CSLOperand[hostIndex].name, delay, Continue);

    }

/*----------*/
Fsshtodwh()

    {

	int	scriptIndex = nextImmediateOp();
	int	hostIndex = nextImmediateOp();
	int	timeIndex = nextImmediateOp();
	long	delay = CSLdeltaSeconds(timeIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, CSLOperand[hostIndex].name, delay, Linger);

    }

/*----------*/
Fsshinth()

    {

	int	scriptIndex = nextImmediateOp();
	int	hostIndex = nextImmediateOp();
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	delay = CSLseconds(timeInterval, unitsIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, CSLOperand[hostIndex].name, delay, Continue);

    }

/*----------*/
Fsshintwh()

    {

	int	scriptIndex = nextImmediateOp();
	int	hostIndex = nextImmediateOp();
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();
	long	delay = CSLseconds(timeInterval, unitsIndex);

	SendUpdate (scriptIndex);

	return CSLexecute(SHELLscript, scriptIndex, CSLOperand[hostIndex].name, delay, Linger);

    }

/*----------*/
Fact()

    {

	int	messageIndex = nextImmediateOp();

	CSLMonitorMessage(localHostName, scriptId, CSLOperand[messageIndex].name);
	return NO_UPDATE;

    }

/*----------*/
Fstop()

    {

	return NO_UPDATE;

    }


Fstopx()

    {

	pCompletionCode->dble = tos[0];
	return NO_UPDATE;

    }

/*----------*/
Fstopcsl()

    {

	int	scriptIndex = nextImmediateOp();
	Operand	*pScript = &CSLOperand[scriptIndex];

	SendUpdate (scriptIndex);

	CSLMonitorKill(pScript->scriptHostName, pScript->scriptId);
	return NO_UPDATE;

    }

/*----------*/
Flinenum()

    {

	lineNumber = nextImmediateOp();
	return NO_UPDATE;

    }

/*----------*/
Fwithin()

    {

	tos[-2] = ((tos[0]-tos[-1]) <= tos[-2]) && (tos[-2] <= (tos[0]+tos[-1]));
	return NO_UPDATE;

    }

/*----------*/
Fnwithin()

    {

	tos[-2] = (tos[-2] < (tos[0]-tos[-1])) || ((tos[0]+tos[-1]) < tos[-2]);
	return NO_UPDATE;

    }

/*----------*/
Frunstatus()

    {

	int	scriptIndex = findScriptIndex((int)tos[0]);
	Operand	*pScript = &CSLOperand[scriptIndex];
	ExtendedScriptStatus	extendedScriptStatus;

	SendUpdate (scriptIndex);

	tos[0] = (CSLMonitorGetScriptStatus(pScript->scriptHostName,
			pScript->scriptId, &extendedScriptStatus) == Running);
	if (!tos[0])
		pCompletionCode->dble = extendedScriptStatus.completionCode;
	return NO_UPDATE;

    }

/*----------*/
Fcompstatus()

    {

	int	scriptIndex = findScriptIndex((int)tos[0]);
	Operand	*pScript = &CSLOperand[scriptIndex];
	ExtendedScriptStatus	extendedScriptStatus;

	SendUpdate (scriptIndex);

	CSLMonitorGetScriptStatus(pScript->scriptHostName, pScript->scriptId, &extendedScriptStatus);
	tos[0] = extendedScriptStatus.completionCode;
	return NO_UPDATE;

    }

/*----------*/
Fsett()

    {

	int	timerIndex = nextImmediateOp();
	Operand	*pTimer = &CSLOperand[timerIndex];
	double	timeInterval = tos[0];
	int	unitsIndex = nextImmediateOp();

	pTimer->timer.TimerValue = CSLseconds(timeInterval, unitsIndex);
	pTimer->timer.TimerRunning = No;
	return timerIndex;

    }


struct instructionSet {
	int	opcode;		/* the opcode value */
	char	*instr;		/* For tracing purposes */
	int	(*func)();	/* the implementation function */
	int	argsOff;	/* number of args popped from the stack */
	int	argsOn;		/* number of args pushed onto the stack */
	int	immediateOps;	/* number of immediate operands needed */
} instructionSet [] = {
/*		opcode		instr,		func			argsOff		argsOn		immediateOps */
/*  0 */	NOP,		"NOP",		Fnop,			0,		0,		0,
	/* Don't count the immedeate ops for jumps since they don't reference the operand list */
/*  1 */	JUMP,		"JUMP",		Fjump,			0,		0,		0,
/*  2 */	JZERO,		"JZERO",	Fjzero,			1,		0,		0,
/*  3 */	JNZERO,		"JNZERO",	Fjnzero,		1,		0,		0,
/*  4 */	EQ,		"EQ",		Feq,			2,		1,		0,
/*  5 */	NE,		"NE",		Fne,			2,		1,		0,
/*  6 */	GT,		"GT",		Fgt,			2,		1,		0,
/*  7 */	GE,		"GE",		Fge,			2,		1,		0,
/*  8 */	LT,		"LT",		Flt,			2,		1,		0,
/*  9 */	LE,		"LE",		Fle,			2,		1,		0,
/* 10 */	LEQ,		"LEQ",		Fleq,			2,		1,		0,
/* 11 */	LNE,		"LNE",		Flne,			2,		1,		0,
/* 12 */	LGT,		"LGT",		Flgt,			2,		1,		0,
/* 13 */	LGE,		"LGE",		Flge,			2,		1,		0,
/* 14 */	LLT,		"LLT",		Fllt,			2,		1,		0,
/* 15 */	LLE,		"LLE",		Flle,			2,		1,		0,
/* 16 */	AND,		"AND",		Fand,			2,		1,		0,
/* 17 */	OR,		"OR",		For,			2,		1,		0,
/* 18 */	NOT,		"NOT",		Fnot,			1,		1,		0,
/* 19 */	ADD,		"ADD",		Fadd,			2,		1,		0,
/* 20 */	SUB,		"SUB",		Fsub,			2,		1,		0,
/* 21 */	MULT,		"MULT",		Fmult,			2,		1,		0,
/* 22 */	DIV,		"DIV",		Fdiv,			2,		1,		0,
/* 23 */	POWER,		"POWER",	Fpower,			2,		1,		0,
/* 24 */	NEG,		"NEG",		Fneg,			1,		1,		0,
/* 25 */	ABS,		"ABS",		Fabs,			1,		1,		0,
/* 26 */	ACOS,		"COS",		Facos,			1,		1,		0,
/* 27 */	ASIN,		"ASIN",		Fasin,			1,		1,		0,
/* 28 */	ATAN,		"ATAN",		Fatan,			1,		1,		0,
/* 29 */	CEILING,	"CEILING",	Fceiling,		1,		1,		0,
/* 30 */	COS,		"COS",		Fcos,			1,		1,		0,
/* 31 */	EXP,		"EXP",		Fexp,			1,		1,		0,
/* 32 */	FLOOR,		"FLOOR",	Ffloor,			1,		1,		0,
/* 33 */	LN,		"LN",		Fln,			1,		1,		0,
/* 34 */	LOG,		"LOG",		Flog,			1,		1,		0,
/* 35 */	MAX,		"MAX",		Fmax,			2,		1,		0,
/* 36 */	MIN,		"MIN",		Fmin,			2,		1,		0,
/* 37 */	MOD,		"MOD",		Fmod,			2,		1,		0,
/* 38 */	SIN,		"SIN",		Fsin,			1,		1,		0,
/* 39 */	SQRT,		"SQRT",		Fsqrt,			1,		1,		0,
/* 40 */	TAN,		"TAN",		Ftan,			1,		1,		0,
/* 41 */	PUSH,		"PUSH",		Fpush,			0,		1,		1,
/* 42 */	POP,		"POP",		Fpop,			1,		0,		0,
/* 43 */	XCHG,		"XCHG",		Fxchg,			2,		2,		0,
/* 44 */	LOAD,		"LOAD",		Fload,			0,		1,		1,
/* 45 */	STORE,		"STORE",	Fstore,			1,		0,		1,
/* 46 */	TSTART,		"TSTART",	Ftstart,		0,		0,		1,
/* 47 */	TSTOP,		"TSTOP",	Ftstop,			0,		0,		1,
/* 48 */	TEXP,		"TEXP",		Ftexp,			0,		1,		1,
/* 49 */	COMD,		"COMD",		Fcomd,			0,		0,		2,
/* 50 */	0,		"",		0,			0,		0,		0,
/* 51 */	0,		"",		0,			0,		0,		0,
/* 52 */	WAITCSL,	"WAITCSL",	Fwaitcsl,		0,		0,		1,
/* 53 */	WAITCSLT,	"WAITCSLT",	Fwaitcslt,		1,		0,		2,
/* 54 */	PAUSE,		"PAUSE",	Fpause,			0,		0,		0,
/* 55 */	ACCESS,		"ACCESS",	Faccess,		0,		1,		1,
/* 56 */	SET,		"SET",		Fset,			1,		0,		1,
/* 57 */	SETV,		"SETV",		Fsetv,			1,		0,		1,
/* 58 */	ALARM,		"ALARM",	Falarm,			0,		0,		2,
/* 59 */	SCSL,		"SCSL",		Fscsl,			0,		0,		1,
/* 60 */	SCSLW,		"SCSLW",	Fscslw,			0,		0,		1,
/* 61 */	SCSLTOD,	"SCSLTOD",	Fscsltod,		0,		0,		2,
/* 62 */	SCSLTODW,	"SCSLTODW",	Fscsltodw,		0,		0,		2,
/* 63 */	SCSLINT,	"SCSLINT",	Fscslint,		1,		0,		2,
/* 64 */	SCSLINTW,	"SCSLINTW",	Fscslintw,		1,		0,		2,
/* 65 */	SSH,		"SSH",		Fssh,			0,		0,		1,
/* 66 */	SSHW,		"SSHW",		Fsshw,			0,		0,		1,
/* 67 */	SSHTOD,		"SSHTOD",	Fsshtod,		0,		0,		2,
/* 68 */	SSHTODW,	"SSHTODW",	Fsshtodw,		0,		0,		2,
/* 69 */	SSHINT,		"SSHINT",	Fsshint,		1,		0,		2,
/* 70 */	SSHINTW,	"SSHINTW",	Fsshintw,		1,		0,		2,
/* 71 */	SSHH,		"SSHH",		Fsshh,			0,		0,		2,
/* 72 */	SSHWH,		"SSHWH",	Fsshwh,			0,		0,		2,
/* 73 */	SSHTODH,	"SSHTODW",	Fsshtodh,		0,		0,		3,
/* 74 */	SSHTODWH,	"SSHTODWH",	Fsshtodwh,		0,		0,		3,
/* 75 */	SSHINTH,	"SSHINTH",	Fsshinth,		1,		0,		3,
/* 76 */	SSHINTWH,	"SSHINTWH",	Fsshintwh,		1,		0,		3,
/* 77 */	ACT,		"ACT",		Fact,			0,		0,		1,
/* 78 */	STOP,		"STOP",		Fstop,			0,		0,		0,
/* 79 */	STOPX,		"STOPX",	Fstopx,			1,		0,		0,
/* 80 */	STOPCSL,	"STOPCSL",	Fstopcsl,		1,		0,		0,
/* 81 */	LINENUM,	"LINENUM",	Flinenum,		0,		0,		1,
/* 82 */	WITHIN,		"WITHIN",	Fwithin,		3,		1,		0,
/* 83 */	NWITHIN,	"NWITHIN",	Fnwithin,		3,		1,		0,
/* 84 */	RUNSTATUS,	"RUNSTATUS",	Frunstatus,		1,		1,		0,
/* 85 */	COMPSTATUS,	"COMPSTATUS",	Fcompstatus,		1,		1,		0,
/* 86 */	SETT,		"SETT",		Fsett,			0,		0,		0,
/* 87 */	STORETINT,	"STOREINT",	Fstoretint,		1,		0,		1,
/* 88 */	STORETTOD,	"STORETOD",	Fstorettod,		0,		0,		1,
/* 89 */	DECRT,		"DECRT",	Fdecrt,			0, 		1, 		0,
};
int	numOpcodes	= (sizeof instructionSet) / (sizeof instructionSet[0]);

/*----------*/
ExecuteNextOpcode()

    {

	struct instructionSet	*p;
	int			opcode;
	time_t			t;		
	struct tm		*ptm;
	int			i;
	int			opIndex;
	Opcode			*pcSave;

	/*
	 * do the Opcode fetch
	 *	is the pc still pointing in the text segment?
	 *	does it point to a valid opcode?
	 */

	pcSave = pc;
	assert((CSLTextStart <= pc) && (pc < CSLTextEnd), "pc went out of range");
	opcode = *(pc++);

	assert((0 <= opcode) && (opcode < numOpcodes), "invalid opcode found");

	if (TraceOn)
	    syslog (LOG_OPTIONS, "CSLInt(%d) %3d: %s\n",
			scriptId, pcSave - CSLTextStart, instructionSet [opcode].instr);

	p = &instructionSet[opcode];
	assert(opcode == p->opcode, "instructionSet table mismatched"); /* in case the list gets out of order */

	/* Check the immediate operands */

	for (i = 0; i < p->immediateOps; i++)
	    if (opcode != LINENUM)
		assert((*(pc+i) < numOperands), "immediate operand is out of range");

	/* Enough args and enough space on the stack? */

	assert(tos >= &CSLStack[p->argsOff - 1], "stack underflowed");
	assert ((tos + (p->argsOn - p->argsOff)) < &CSLStack[StackMaximum], "Stack Overflow");
		
	/*
	 * update TimeNow
	 * execute the opcode
	 * fixup the stack pointer
	 */

	t = (time_t)time(0);
	ptm = localtime(&t);
	pTimeNow->time.hour = ptm->tm_hour;
	pTimeNow->time.minute = ptm->tm_min;
	pTimeNow->time.second = ptm->tm_sec;

	/* Dispatch to the instruction handler function */

	opIndex = (*(p->func))();

	for (i = 0; i < p->argsOff; i++)
	    tos--;
	for (i = 0; i < p->argsOn; i++)
	    tos++;

	/* Send an update to the partner if necessary */

	SendUpdate(opIndex);

	return ((opcode == STOP) || (opcode == STOPX)) ? 0 : 1;

    }

/*----------*/
void GetMyRole()

    {

	if (MyRole != ROLE_NOREDUNDANCY) {
	    int	Status;

	    if (!appGetStatus(MyHandle, &Status)) {
		syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter has died -- Terminating CSL script.",
				scriptId);
		exit (-1);
	    }

	    switch (Status) {
		case MASTER:
		    MyRole = ROLE_MASTER;
		    break;
		case SLAVE:
		    MyRole = ROLE_SLAVE;
		    break;
		case MASTER_NOPARTNER:
		    MyRole = ROLE_MASTER_NOPARTNER;
		    break;
	    }
	}

    }

/*----------*/
void DisplayRole ()

    {

	if (Debug) {
	    char	*text;
	    switch (MyRole) {
		case ROLE_NOREDUNDANCY:
		    text = "Unable to register with arbiter -- NO REDUNDANCY OPERATONAL";
		    break;
		case ROLE_MASTER:
		    text = "I'm Master";
		    break;
		case ROLE_SLAVE:
		    text = "I'm Slave";
		    break;
		case ROLE_MASTER_NOPARTNER:
		    text = "I'm Master with a disconnected partner";
		    break;
	    }
	    syslog (LOG_OPTIONS, "CSLInt(%d): %s: %s", scriptId, CSLId, text);
	}

    }

/*----------*/
void sendFullUpdate ()	/* Send a copy of all variables */

    {

	int	i;

	if (Debug)
	    syslog (LOG_OPTIONS, "CSLInt(%d): Sending a full update to the slave", scriptId);

	for (i = 0; i < numOperands; i++)
	    if ((CSLOperand [i].type == 'T') || (CSLOperand [i].type == 'V'))
		SendUpdate(i);

	if (Debug)
	    syslog (LOG_OPTIONS, "CSLInt(%d): Full update to slave complete.\n", scriptId);

    }

/*----------*/
main (argc, argv)

    int		argc;
    char	*argv[];

    {

	int		i, sp;
	RedundMsg	msgBuffer;
	Boolean		KeepGoing = Yes;

	/* Determine if in verbose mode */

	Debug = getenv ("CSLINTERPRETERVERBOSE") != 0;
	TraceOn = getenv ("CSLINTERPRETERTRACE") != 0;

	if (Debug)
	    syslog (LOG_OPTIONS, "CSLInt(%d): Running.\n", getpid ());

	InitializeInterpreter(argc, argv);

	GetCSLImage();

	InitializeStack();

	/* Connect all the channels */

	connectChannels ();

	/* Get our role in life */

	GetMyRole ();
	DisplayRole ();

	/* If we have come up as a slave, then we need to get a full update of all variables */
	/* from our master */

	if (MyRole == ROLE_SLAVE) {
	    SendUpdateMsg	SendUpdateMessage;
	    SendUpdateMessage.msgType = htonl (SENDUPDATE);
	    if (!appSendData(MyHandle, (char *)&SendUpdateMessage, sizeof (SendUpdateMessage))) {
		syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter has died -- CSL script terminated.",
				scriptId);
		exit (-1);
	    }
	    if (Debug)
		syslog (LOG_OPTIONS, "CSLInt(%d): Sending full update request to master.\n",
				 scriptId);
	}

	/* If we have come up as a master with a slave connected, */
	/* wait for the full update request before continuing. */

	if (MyRole == ROLE_MASTER) {
	    int Count = 10;
	    while (Count--) {
		if (Debug)
		    syslog (LOG_OPTIONS, "CSLInt(%d): (%s) Waiting for slave synchronization",
				 scriptId, imageName);
		if (appRecvData (MyHandle, (char *)&msgBuffer, 1)) {	/* Still a master - Check for any requests from the slave */
		    int i;
		    if (ntohl (msgBuffer.msgHeader.msgType) == SENDUPDATE) {	/* Message type */
			if (Debug)
			    syslog (LOG_OPTIONS,
				"CSLInt(%d): Received a full update request from slave",
					scriptId);
			    sendFullUpdate ();
		    } else {
			syslog (LOG_OPTIONS,
			    "CSLInt(%d): Unexpected message while wait for slave to start", scriptId);
			exit (1);
		    }
		    break;
		}
		sleep (1);
	    }
	    if (Debug && (Count <= 0))
		syslog (LOG_OPTIONS,
			"CSLInt(%d): Slave did not respond - presumed lost", scriptId);
	}

	CSLMonitorSetMyStatus (localHostName, scriptId, Running, pCompletionCode->dble = 0.0);
	signal (SIGTERM, termHandler);

	while (KeepGoing && InterpRunning) {

	    ca_pend_io (0.0);

	    /* Check if my role has changed */

	    OldRole = MyRole;
	    GetMyRole();
	    if (MyRole != OldRole)
		DisplayRole ();

	    if (MyRole == ROLE_NOREDUNDANCY) {
		KeepGoing = ExecuteNextOpcode();
	    } else if ((MyRole == ROLE_MASTER) || (MyRole == ROLE_MASTER_NOPARTNER)) {
		if ((OldRole == ROLE_MASTER) || (OldRole == ROLE_MASTER_NOPARTNER)) {
		    if (appRecvData (MyHandle, (char *)&msgBuffer, 1)) {	/* Still a master - Check for any requests from the slave */
			int i;
			switch (ntohl (msgBuffer.msgHeader.msgType)) {	/* Message type */
			    case SENDUPDATE:	/* Send value of all variables */
				sendFullUpdate ();
				break;
			    default:	/* Maybe left over from when I was a slave */
				break;
			}
		    }
		    KeepGoing = ExecuteNextOpcode();
		} else {
		    if (Debug)
			syslog (LOG_OPTIONS, "CSLInt(%d): (%s) Taking over as master, PC=%d",
				scriptId, imageName, pc - CSLTextStart);
		    OldRole = MyRole;
		}
	    } else {		/* I'm a slave */
		if ((OldRole == ROLE_MASTER) || (OldRole == ROLE_MASTER_NOPARTNER)) {
			/* Just became slave - Ask the master to send a complete update of all varbs and PC */
		    SendUpdateMsg	SendUpdateMessage;
		    SendUpdateMessage.msgType = htonl (SENDUPDATE);

		    if (!appSendData (MyHandle, (char *)&SendUpdateMessage, sizeof (SendUpdateMessage))) {
			syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter has died -- CSL script terminated.",
					scriptId);
			exit (-1);
		    }
		    OldRole = ROLE_SLAVE;
		} else {		/* Still a slave - Check for a message from the master */

		    if (appRecvData (MyHandle, (char *)&msgBuffer, 1)) {
			switch (ntohl (msgBuffer.msgHeader.msgType)) {
			    case UPDATE:	/* Variable Update */
				{
				    int i;
				    int ix = ntohl (msgBuffer.updateMsg.VarIndex);
				    if (ix >= 0) {		/* Variable update */
				    	CSLOperand[ix].dble = atof (msgBuffer.updateMsg.dble);
				    	CSLOperand[ix].time = msgBuffer.updateMsg.time;
				    	CSLOperand[ix].timer.TimerValue = ntohl (msgBuffer.updateMsg.timer.TimerValue);
				    	CSLOperand[ix].timer.TimerRunning = ntohl (msgBuffer.updateMsg.timer.TimerRunning);
					CSLOperand[ix].scriptId = ntohl (msgBuffer.updateMsg.ScriptId);
					CSLOperand[ix].scriptHostName = localHostName; 
				    }
				    pc = CSLTextStart + ntohl (msgBuffer.updateMsg.PC);
				    sp = ntohl (msgBuffer.updateMsg.NewSP);
				    if (sp >= 0) {
					for (i=0 ; i<sp ; i++) {
					    CSLStack [i] = atof (msgBuffer.updateMsg.NewStack [i]);
					}
				    }
				    tos = CSLStack + sp;
				    if (Debug) {
					if (ix >= 0) {
					    syslog (LOG_OPTIONS,
						"CSLInt(%d): Received update, PC=%d, ix=%d(%s), Value=%s, ScriptId=%d",
							scriptId, ntohl (msgBuffer.updateMsg.PC), ix, CSLOperand [ix].name,
							msgBuffer.updateMsg.dble, ntohl (msgBuffer.updateMsg.ScriptId));
					} else {
					    syslog (LOG_OPTIONS, "CSLInt(%d): Received PC update, PC=%d",
							scriptId, ntohl (msgBuffer.updateMsg.PC));
					}
				    }
				}
				break;
			    case TERMINATE:	/* Terminate slave */
				{
				    if (Debug)
					syslog (LOG_OPTIONS, "CSLInt(%d): Termination request received", scriptId);
				    exitCode = ntohl (msgBuffer.terminateMsg.exitCode);
				    KeepGoing = No;
				}
				break;
			}
		    }
		}
	    }
	}

	CSLMonitorSetMyStatus (localHostName, scriptId, exitCode, pCompletionCode->dble);

	if (MyRole == ROLE_MASTER) {
	    /* Interpreter is terminating so terminate the slave */
	    TerminateMsg TerminateMessage;

	    TerminateMessage.msgType = htonl (TERMINATE);
	    TerminateMessage.exitCode = htonl (exitCode);
	    if (!appSendData (MyHandle, (char *)&TerminateMessage, sizeof TerminateMessage)) {
		syslog (LOG_OPTIONS, "CSLInt(%d): Arbiter has died -- CSL script terminated.",
				scriptId);
		exit (-1);
	    }
	    if (Debug)
		syslog (LOG_OPTIONS, "CSLInt(%d): Sent termination message to partner", scriptId);
	}

	FreeResources ();

	if (Debug)
	    syslog (LOG_OPTIONS, "CSLInt(%d): %s Terminating", scriptId, CSLId);
	return 0;

    }

/*----------*/
static void assert (condition, msg)

    int		condition;
    char	*msg;

    {

	if (condition)
	    return;

	syslog (LOG_OPTIONS, "cslInt(%d): %s", scriptId, msg);
	exit (-1);

    }

/*----------*/
static void connectChannels ()	/* Connect any channels needed */

    {

	int	i, startTime;

	/* Initialize interface to channel access. */

	ca_task_initialize();

	/* Initiate connection to all channels */

	numChannels = 0;
	for (i = 0; i < numOperands; i++) {
	    if (CSLOperand [i].type == 'C') {
		ca_build_and_connect (CSLOperand [i].name, TYPENOTCONN, 0, 
				&CSLOperand [i].chid, NULL, connectHandler, (void *)&CSLOperand [i]);
		CSLOperand [i].connected = 0;
		numChannels++;
	    }
	}

	/* We can't let the interpreter start until all the channels connect. We will */
	/* wait here for up to CHANNEL_CONNECT_TIMEOUT seconds or until all connect. */

	startTime = time (0);
	for (;;) {

	    /* Allow connections to happen. */

	    ca_pend_io (1.0);

	    /* Check if all have connected */

	    for (i=0 ; i<numOperands ; i++)
		if (CSLOperand [i].type == 'C')
		    if (!CSLOperand [i].connected)
			break;
	    if (i == numOperands)
		break;

	    /* Check if our timeout has expired */

	    if ((time (0) - startTime) > CHANNEL_CONNECT_TIMEOUT) {
		syslog (LOG_OPTIONS, "CSLInt(%d): The following channels will not connect:", scriptId);
		for (i=0 ; i<numOperands ; i++)
		    if (CSLOperand [i].type == 'C')
			if (!CSLOperand [i].connected)
			    syslog (LOG_OPTIONS, "    %s", CSLOperand [i].name);
		exit (-1);
	    }
	}

	/* We can't let the interpreter start until all the channels report. We will */
	/* wait here for up to CHANNEL_CONNECT_TIMEOUT seconds or until all connect. */

	startTime = time (0);
	for (;;) {

	    /* Allow connections to happen. */

	    ca_pend_io (1.0);

	    /* Check if all have connected */

	    for (i=0 ; i<numOperands ; i++)
		if (CSLOperand [i].type == 'C')
		    if (!CSLOperand [i].updated)
			break;
	    if (i == numOperands)
		return;

	    /* Check if our timeout has expired */

	    if ((time (0) - startTime) > CHANNEL_CONNECT_TIMEOUT) {
		syslog (LOG_OPTIONS, "CSLInt(%d): The following channels did not report an initial value:", scriptId);
		for (i=0 ; i<numOperands ; i++)
		    if (CSLOperand [i].type == 'C')
			if (!CSLOperand [i].updated)
			    syslog (LOG_OPTIONS, "    %s", CSLOperand [i].name);
		exit (-1);
	    }
	}
			
    }

/*----------*/
static void connectHandler (args)

    struct connection_handler_args args;

    {

	Operand	*p;

	/* If channel has connected, set callback for event. */
	/* If disconnected, mark it as disconnected. */

	p = (Operand *) ca_puser (args.chid);
	if (args.op == CA_OP_CONN_UP) {
	    if (strstr (p->name, ".CMDS") == 0) {
		ca_add_event (DBR_DOUBLE, args.chid, eventHandler,
				(void *)p,
				&(p->eventId));
    	        p->connected = 1;
		p->updated = 0;
	    } else {

		/* This is an output only channel. We need not set up an event callback. */
		/* We set connected and updated to true so that we do not wait for them */
		/* to report. */

		p->connected = 1;
		p->updated = 1;
	    }
	} else {
	    p->connected = 0;
	    p->updated = 0;
	}

    }

/*----------*/
static void eventHandler (args)

    struct event_handler_args args;

    {

	((Operand *)args.usr)->dble = *(double *)args.dbr;
	((Operand *)args.usr)->updated = 1;
/*	printf ("CSLInt(%d): CA event for %s, value = %lf\n", getpid (), ((Operand *)args.usr)->name,
				((Operand *)args.usr)->dble);
*/

    }

/*----------*/
static void writeChannel (chanIx, value)

    int		chanIx;
    double	value;

    {

	static	double	v;

	v = value;
	CSLOperand [chanIx].updated = 0;
	ca_put (DBR_DOUBLE, CSLOperand [chanIx].chid, &v);
	ca_flush_io ();

    }

/*----------*/
static void termHandler (sig)

    int 		sig;

    {

	InterpRunning = No;
	exitCode = Killed;

    }

