/*gen.c,v 1.1.1.1 1999/08/03 14:44:54 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software was developed for DLA by Tate Integrated  */
/*	Systems, L.P. via a contract with Vitro Corporation.	*/
/*	It is designed to work with TIS4000 Release 4.2.	*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*	File:	gen.c		CSL Code Generator
/*	Author:	Douglas P. Dotson
/*	Date:	20 Apr 1993
/*	Edit:	20 Jan 1994	1000	DPD
*/
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "symtab.h"
#include "tree.h"
#include "opcodes.h"
#include "csldefs.h"

	void Gen_Program ();
static	void Gen_Statements ();
static	void Gen_Statement ();
static	void Gen_Label_Statement ();
static	void Gen_Command_Statement ();
static	void Gen_Wait_Statement ();
static	void Gen_If_Statement ();
static	void Gen_While_Statement ();
static	void Gen_Set_Statement ();
static	void Gen_Alarm_Statement ();
static	void Gen_Start_Statement ();
static	void Gen_Execute_Statement ();
static	void Gen_Activity_Statement ();
static	void Gen_GoTo_Statement ();
static	void Gen_Stop_Statement ();
static	void Gen_StopCSL_Statement ();
static	void Gen_Expression ();
static	int Gen_Optional_Argument_List ();
static	void Gen_FunctionCall ();
static	void Gen_Operation ();
static	void Gen_Immediate ();
static	void Gen_Arith_Operator ();

extern	TreeNode	*TreeRoot;

short	int	(*Code) [];	
int	pc;

/* Table of functions */

static struct {
	int	FuncNumber;	/* Function Number returned by lex */
	int	Nargs;		/* Proper number of arguments */
	int	OpCode;		/* Opcode for this function */
} FuncTable [] = {
	{FUNC_ABS,	1,	ABS},
	{FUNC_ACOS,	1,	ACOS},
	{FUNC_ASIN,	1,	ASIN},
	{FUNC_ATAN,	1,	ATAN},
	{FUNC_CEILING,	1,	CEILING},
	{FUNC_COS,	1,	COS},
	{FUNC_EXP,	1,	EXP},
	{FUNC_FLOOR,	1,	FLOOR},
	{FUNC_LN,	1,	LN},
	{FUNC_LOG,	1,	LOG},
	{FUNC_MAX,	2,	MAX},
	{FUNC_MIN,	2,	MIN},
	{FUNC_MOD,	2,	MOD},
	{FUNC_SIN,	1,	SIN},
	{FUNC_SQRT,	1,	SQRT},
	{FUNC_TAN,	1,	TAN},
	{FUNC_RUNSTATUS,1,	RUNSTATUS},
	{FUNC_COMPSTATUS,1,	COMPSTATUS},
};

/*----------*/
void Gen_Program ()

    {

	int	i;

	/* Initialize program counter to 0 and allocate code array */

	pc = 0;
	Code = (short int (*)[]) malloc (MAXCODESIZE * sizeof (short));

	/* Put sequence numbers on the symbols */

	SequenceSymTab ();

	/* If a tree was created, generate code for it */

	if (TreeRoot)
	    Gen_Statements (TreeRoot);

	/* Put a STOP at the end of the file in case one wasn't */
	/* put there by the programmer */

	Gen_Operation (TreeRoot->LineNumber, STOP);

	/* Dump the code to stdout */

	for (i=0 ; i<pc ; i++) {
	    putchar ((*Code) [i]);
	    putchar ((*Code) [i] >> 8);
	}

	free (Code);

    }

/* end Gen_Program */

/*----------*/
static void Gen_Statements (p)

    TreeNode	*p;

    {

	if (p != NULL) {
	    Gen_Statement (p->Left);
	    Gen_Statements (p->Middle);
	}
	
    }

/* end Gen_Statements */

/*----------*/
static void Gen_Statement (p)

    TreeNode	*p;

    {

	if (p == NULL)
	    return;

	switch (p->Left->NodeType) {
	    case COMMAND_STATEMENT:
		Gen_Command_Statement (p->Left);
		break;
	    case WAIT_STATEMENT:
		Gen_Wait_Statement (p->Left);
		break;
	    case IF_STATEMENT:
		Gen_If_Statement (p->Left);
		break;
	    case WHILE_STATEMENT:
		Gen_While_Statement (p->Left);
		break;
	    case SET_STATEMENT:
		Gen_Set_Statement (p->Left);
		break;
	    case ALARM_STATEMENT:
		Gen_Alarm_Statement (p->Left);
		break;
	    case START_STATEMENT:
		Gen_Start_Statement (p->Left);
		break;
	    case EXECUTE_STATEMENT:
		Gen_Execute_Statement (p->Left);
		break;
	    case ACTIVITY_STATEMENT:
		Gen_Activity_Statement (p->Left);
		break;
	    case GOTO_STATEMENT:
		Gen_GoTo_Statement (p->Left);
		break;
	    case STOP_STATEMENT:
		Gen_Stop_Statement (p->Left);
		break;
	    case STOPCSL_STATEMENT:
		Gen_StopCSL_Statement (p->Left);
		break;
	    case LABEL_STATEMENT:
		Gen_Label_Statement (p->Left);
		break;
	}
    }

/* end Gen_Statement */

/*----------*/
static void Gen_Label_Statement (p)

    TreeNode	*p;

    {

	int	Address, i, j;

	/* A label has been found.  If the Address field in the */
	/* symbol table has a value in it then it is pointing */
	/* to a the location of the target of a GOTO that was a */
	/* forward reference to this label.  All forward references */
	/* to the label form a linked list. */

	Address = p->Left->Sym->Address = pc;

	/* Traverse the chain and insert addresses */

	i = p->Left->Sym->AddrList;
	while (i) {
	    j = i;
	    i = (*Code) [i];
	    (*Code) [j] = Address;
	}

    }

/* end Gen_Label_Statement */

/*----------*/
static void Gen_Command_Statement (p)

    TreeNode	*p;

    {
	if (p->MoreInfo == COMD_DEVICE) {	/* Device command */
	    Gen_Operation (p->LineNumber, COMD);
	    Gen_Immediate (p->Left->Sym->SymNumber);	/* device */
	    Gen_Immediate (p->Middle->Sym->SymNumber);	/* command */
	} else {	/* Timer command */
	    Gen_Operation (p->LineNumber, (p->MoreInfo == COMD_TIMERSTART) ? TSTART : TSTOP);
	    Gen_Immediate (p->Left->Sym->SymNumber);
	}
	
    }

/* end Gen_Command_Statement */

/*----------*/
static void Gen_Wait_Statement (p)

    TreeNode	*p;

    {

	int	TopAddr, BottomAddr;

	switch (p->MoreInfo) {
	    case WAIT_INTERVAL:
		Gen_Expression (p->Left);
		Gen_Operation (p->LineNumber, STORETINT);
		Gen_Immediate (p->Middle->Left->Sym->SymNumber);
		TopAddr = pc;
		Gen_Operation (p->LineNumber, DECRT);
		Gen_Operation (p->LineNumber, JZERO);
		BottomAddr = pc++;
		Gen_Operation (p->LineNumber, PAUSE);
		Gen_Operation (p->LineNumber, JUMP);
		Gen_Immediate (TopAddr);
		(*Code) [BottomAddr] = pc;
		break;
	    case WAIT_COMP:
		Gen_Operation (p->LineNumber, WAITCSL);
		Gen_Immediate (p->Left->Sym->SymNumber);
		break;
	    case WAIT_COMP_TIME:
		Gen_Expression (p->Middle);
		Gen_Operation (p->LineNumber, WAITCSLT);
		Gen_Immediate (p->Left->Sym->SymNumber);
		Gen_Immediate (p->Right->Left->Sym->SymNumber);
		break;
	    case WAIT_TIME:
		Gen_Operation (p->LineNumber, STORETTOD);
		Gen_Immediate (p->Left->Sym->SymNumber);
		TopAddr = pc;
		Gen_Operation (p->LineNumber, DECRT);
		Gen_Operation (p->LineNumber, JZERO);
		BottomAddr = pc++;
		Gen_Operation (p->LineNumber, PAUSE);
		Gen_Operation (p->LineNumber, JUMP);
		Gen_Immediate (TopAddr);
		(*Code) [BottomAddr] = pc;
		break;
	    case WAIT_EXPR:
		TopAddr = pc;
		Gen_Expression (p->Left);
		Gen_Operation (p->LineNumber, JNZERO);
		BottomAddr = pc++;
		Gen_Operation (p->LineNumber, PAUSE);
		Gen_Operation (p->LineNumber, JUMP);
		Gen_Immediate (TopAddr);
		(*Code) [BottomAddr] = pc;
		break;
	}

    }

/* end Gen_Wait_Statement */

/*----------*/
static void Gen_If_Statement (p)

    TreeNode	*p;

    {
	int	Addr, ElseJumpInst, EndJumpInst;

	switch (p->MoreInfo) {
	    case IF_GOTO:

		/* Generate code for the conditional expression */

		Gen_Expression (p->Left);
	
		/* If an address is defined for the target then generate */
		/* a complete jump.  If target is not yet defined (forward */
		/* reference) add address of the target address to the label's */
		/* list. */

		Gen_Operation (p->LineNumber, JNZERO);

		if (p->Middle->Sym->Address != -1)
		    Gen_Immediate (p->Middle->Sym->Address);
		 else {
		    (*Code) [pc] = p->Middle->Sym->AddrList;
		    p->Middle->Sym->AddrList = pc++;
		}
	
		break;

	    case IF_THEN:

		/* Generate code for the conditional expression */

		Gen_Expression (p->Left);

		/* Generate the jump the the end of the if and save */
		/* location of the target so it can be filled in */
		/* when the address of the end of the if is determined */

		Gen_Operation (p->LineNumber, JZERO);
		Addr = pc++;

		/* Generate the body of the if */

		Gen_Statements (p->Middle);

		/* Fill in the target address for the condition */

		(*Code) [Addr] = pc;

		break;

	    case IF_THEN_ELSE:

		/* Generate the code for the condition */

		Gen_Expression (p->Left);

		/* Generate the jump to the else part and save the */
		/* location of the target of the jump so it can be filled */
		/* in when the target is known. */

		Gen_Operation (p->LineNumber, JZERO);
		ElseJumpInst = pc++;

		/* Generate the code for the then part */

		Gen_Statements (p->Middle);

		/* Generate the jump to the end of the if.   Save the */
		/* location of the target address so it can be filled */
		/* in later */

		Gen_Operation (p->LineNumber, JUMP);
		EndJumpInst = pc++;

		/* Fill in the address of the else part */

		(*Code) [ElseJumpInst] = pc;

		/* Generate code for the else part */

		Gen_Statements (p->Right);

		/* Fill in the address of the end of the if */

		(*Code) [EndJumpInst] = pc;

		break;
	}

    }

/* end Gen_If_Statement */

/*----------*/
static void Gen_While_Statement (p)

    TreeNode	*p;

    {

	int	TopAddr, BottomAddr;

	/* Save the current value of the pc as the target of the */
	/* final jump of the while. */

	TopAddr = pc;

	/* Generate code for the conditional expression */

	Gen_Expression (p->Left);

	/* Generate a conditional jump to the end of the loop. Save the */
	/* location of the target of the jump so that it can be filled */
	/* in when the location is known. */

	Gen_Operation (p->LineNumber, JZERO);
	BottomAddr = pc++;

	/* Generate code for the body of the loop */

	Gen_Statements (p->Middle);

	/* Generate the jump to the beginning of the loop */

	Gen_Operation (p->LineNumber, JUMP);
	Gen_Immediate (TopAddr);

	/* The pc now points to the location following the final jump. */
	/* Fill in the top jump with this address. */

	(*Code) [BottomAddr] = pc;
 
    }

/* end Gen_While_Statement */

/*----------*/
static void Gen_Set_Statement (p)

    TreeNode	*p;

    {

	switch (p->MoreInfo) {
	    case SET_VARIABLE:
		Gen_Expression (p->Middle);
		Gen_Operation (p->LineNumber, STORE);
		Gen_Immediate (p->Left->Sym->SymNumber);
		break;
	    case SET_TIMER:
		Gen_Expression (p->Middle);
		Gen_Operation (p->LineNumber, SETT);
		Gen_Immediate (p->Left->Sym->SymNumber);
		Gen_Immediate (p->Right->Left->Sym->SymNumber);
		break;
	    case SET_CHANNEL:
		Gen_Expression (p->Middle);
		Gen_Operation (p->LineNumber, SET);
		Gen_Immediate (p->Left->Sym->SymNumber);
		break;
	    case SET_CHANNELV:
		Gen_Expression (p->Middle);
		Gen_Operation (p->LineNumber, SETV);
		Gen_Immediate (p->Left->Sym->SymNumber);
		break;
	}
	
    }

/* end Gen_Set_Statement */

/*----------*/
static void Gen_Alarm_Statement (p)

    TreeNode	*p;

    {
	Gen_Operation (p->LineNumber, ALARM);
	Gen_Immediate (p->Left->Sym->SymNumber);
	Gen_Immediate (p->Middle->Sym->SymNumber);
	Gen_Immediate (p->Right->Sym->SymNumber); 
    }

/* end Gen_Alarm_Statement */

/*----------*/
static void Gen_Start_Statement (p)

    TreeNode	*p;

    {
	if (p->Middle && p->Right) {	/* Interval specified */
	    Gen_Expression (p->Middle);
	    Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SCSLINT : SCSLINTW);	
	    Gen_Immediate (p->Left->Sym->SymNumber);
	    Gen_Immediate (p->Right->Left->Sym->SymNumber);
	} else if (p->Middle) {		/* Time of day specified */
	    Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SCSLTOD : SCSLTODW);	
	    Gen_Immediate (p->Left->Sym->SymNumber);
	    Gen_Immediate (p->Middle->Sym->SymNumber);
	} else {
	    Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SCSL : SCSLW);	
	    Gen_Immediate (p->Left->Sym->SymNumber);
	}
    }

/* end Gen_Start_Statement */

/*----------*/
static void Gen_Execute_Statement (p)

    TreeNode	*p;

    {
	if (p->Middle && p->Right) {	/* Interval specified */
	    Gen_Expression (p->Middle);
	    if (p->Extra)
	        Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SSHINTH : SSHINTWH);
	    else
	        Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SSHINT : SSHINTW);	
	    Gen_Immediate (p->Left->Sym->SymNumber);
	    Gen_Immediate (p->Right->Left->Sym->SymNumber);
	} else if (p->Middle) {		/* Time of day specified */
	    if (p->Extra)
		Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SSHTODH : SSHTODWH);
	    else
		Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SSHTOD : SSHTODW);
	    Gen_Immediate (p->Left->Sym->SymNumber);
	    Gen_Immediate (p->Middle->Sym->SymNumber);
	} else {
	    if (p->Extra)
		Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SSHH : SSHWH);
	    else
		Gen_Operation (p->LineNumber, p->MoreInfo == NOWAIT ? SSH : SSHW);	
	    Gen_Immediate (p->Left->Sym->SymNumber);
	}
	if (p->Extra)
	    Gen_Immediate (p->Extra->Sym->SymNumber);
    }

/* end Gen_Execute_Statement */

/*----------*/
static void Gen_Activity_Statement (p)

    TreeNode	*p;

    {

	Gen_Operation (p->LineNumber, ACT);
	Gen_Immediate (p->Left->Sym->SymNumber);

    }

/* end Gen_Activity_Statement */

/*----------*/
static void Gen_GoTo_Statement (p)

    TreeNode	*p;

    {
	/* If an address is defined for the target then generate */
	/* a complete jump.  If target is not yet defined (forward */
	/* reference) add address of the target address to the label's */
	/* list. */

	Gen_Operation (p->LineNumber, JUMP);

	if (p->Left->Sym->Address != -1)
	    Gen_Immediate (p->Left->Sym->Address);
	 else {
	    (*Code) [pc] = p->Left->Sym->AddrList;
	    p->Left->Sym->AddrList = pc++;
	}
	
    }

/* end Gen_GoTo_Statement */

/*----------*/
static void Gen_Stop_Statement (p)

    TreeNode	*p;

    {
	if (p->Left) {
	    Gen_Expression (p->Left);
	    Gen_Operation (p->LineNumber, STOPX);
	} else
	    Gen_Operation (p->LineNumber, STOP);
    }

/* end Gen_Stop_Statement */

/*----------*/
static void Gen_StopCSL_Statement (p)

    TreeNode	*p;

    {
	Gen_Operation (p->LineNumber, STOPCSL);
	Gen_Immediate (p->Left->Sym->SymNumber);
    }

/* end Gen_StopCSL_Statement */

/*----------*/
static void Gen_Expression (p)

    TreeNode	*p;

    {

	if (p) {
	    if (p->NodeType == EXPRESSION) {
		if (p->MoreInfo != FUNCTIONCALL) {
		    Gen_Expression (p->Left);
		    Gen_Expression (p->Middle);
		    Gen_Expression (p->Right);
		}
		if (p->MoreInfo != -1)
		    Gen_Arith_Operator (p);
	    } else if (p->NodeType == TEXP_NODE) {
		Gen_Operation (p->LineNumber, TEXP);
		Gen_Immediate (p->Left->Sym->SymNumber);
	    } else if (p->NodeType == TACCESS_NODE) {
		Gen_Operation (p->LineNumber, ACCESS);
		Gen_Immediate (p->Left->Sym->SymNumber);
		if (p->MoreInfo == NOTREACHABLEOP)
		    Gen_Operation (p->LineNumber, NOT);
	    } else {
		Gen_Operation (p->LineNumber, LOAD);
		Gen_Immediate (p->Sym->SymNumber);
	    }
	}

    }

/* end Gen_Expression */

/*----------*/
static int Gen_Optional_Argument_List (p)

    TreeNode	*p;

    {
	TreeNode	*q;
	int		Nargs;

	if (!p) return;

	q = p->Left;
	Nargs = 0;
	while (q) {
	    Gen_Expression (q->Left);
	    Nargs++;
	    q = q->Middle;
	}

	return Nargs;

    }

/* end Gen_Optional_Argument_List */

/*----------*/
static void Gen_Operation (LineNumber, Operation)	/* Generate and opcode */

    int		LineNumber;
    int		Operation;

    {
    	static int	lastLineNumber	= 0;

	if (pc == MAXCODESIZE)
	    Error ("Maximum program size reached - Write smaller programs.");
	else {
	    if (LineNumber != 0 && LineNumber != lastLineNumber) {
	    	(*Code) [pc++] = LINENUM;
	    	(*Code) [pc++] = lastLineNumber = LineNumber;
	    }
	    (*Code) [pc++] = Operation;
	}
	
    }

/* end Gen_Operation */

/*----------*/
static void Gen_Immediate (Operand)	/* Generate an immediate operand */

    int		Operand;

    {

	if (pc == MAXCODESIZE)
	    Error ("Maximum program size reached - Write smaller programs.");
	else
	    (*Code) [pc++] = Operand;

    }

/* end Gen_Immediate */

/*----------*/
static void Gen_Arith_Operator (p)

    TreeNode	*p;

    {
	int	i, Nargs;

	switch (p->MoreInfo) {
	    case ADDOP:		Gen_Operation (p->LineNumber, ADD);		break;
	    case SUBTRACTOP:	Gen_Operation (p->LineNumber, SUB);		break;
	    case MULTIPLYOP:	Gen_Operation (p->LineNumber, MULT);		break;
	    case DIVIDEOP:	Gen_Operation (p->LineNumber, DIV);		break;
	    case EXPONENTOP:	Gen_Operation (p->LineNumber, POWER);		break;
	    case LTOP:		Gen_Operation (p->LineNumber, p->Type == NUMERIC ? LT : LLT);	break;
	    case LEOP:		Gen_Operation (p->LineNumber, p->Type == NUMERIC ? LE : LLE);	break;
	    case GTOP:		Gen_Operation (p->LineNumber, p->Type == NUMERIC ? GT : LGT);	break;
	    case GEOP:		Gen_Operation (p->LineNumber, p->Type == NUMERIC ? GE : LGE);	break;
	    case EQOP:		Gen_Operation (p->LineNumber, p->Type == NUMERIC ? EQ : LEQ);	break;
	    case NEOP:		Gen_Operation (p->LineNumber, p->Type == NUMERIC ? NE : LNE);	break;
	    case ISWITHINOP:	Gen_Operation (p->LineNumber, WITHIN);		break;
	    case ISNOTWITHINOP:	Gen_Operation (p->LineNumber, NWITHIN);	break;
	    case ANDOP:		Gen_Operation (p->LineNumber, AND);		break;
	    case OROP:		Gen_Operation (p->LineNumber, OR);		break;
	    case NOTOP:		Gen_Operation (p->LineNumber, NOT);		break;
	    case NEGOP:		Gen_Operation (p->LineNumber, NEG);		break;
	    case REACHABLEOP:	Gen_Operation (p->LineNumber, ACCESS);		break;
	    case NOTREACHABLEOP:Gen_Operation (p->LineNumber, ACCESS);
				Gen_Operation (p->LineNumber, NOT);
				break;
	    case FUNCTIONCALL:

		/* Generate code for args */

		Nargs = Gen_Optional_Argument_List (p->Left);

		/* Look up this function in the function table */

		for (i=0 ;; i++) {
		    if (FuncTable [i].FuncNumber == p->FuncType)
			break;
		}

		/* Check for proper number of arguments */

		if (Nargs != FuncTable [i].Nargs)
			Error ("Improper number of arguments on function.");

		/* Generate instruction for function */

		Gen_Operation (p->LineNumber, FuncTable [i].OpCode);

		break;
	}

    }

/* end Gen_Arith_Operator */

