/*symtab.c,v 1.1.1.1 1999/08/03 14:44:56 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software was developed for DLA by Tate Integrated  */
/*	Systems, L.P. via a contract with Vitro Corporation.	*/
/*	It is designed to work with TIS4000 Release 4.2.	*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*	file:	symtab.c	Symbol table support routines
/*	author:	Douglas P. Dotson
/*	date:	08 Mar 1993
/*	edit:	27 Apr 1993	1909	DPD
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "symtab.h"

static	Symbol	*SymTab;
static	int	SymNumber;
static	char	Msg [80];

char SymTypes [] = {"CVTIRtL"};

/*----------*/
void SymTabInit ()	/* Initialize the symbol table to empty */

    {

	SymTab = NULL;
	SymNumber = 0;

    }

/* end SymTabInit */

/*----------*/
Symbol *SymLookUp (Sym)	/* Lookup a symbol */

    char	*Sym;

    {

	Symbol	*p;

	p = SymTab;
	while (p) {
	    if (strcmp (p->String, Sym) == 0)
		return p;
	    p = p->Next;
	}

	return NULL;

    }

/* end SymLookup */
	
/*----------*/
Symbol *SymInsert (Sym, Type)	/* Insert a new symbol */

    char	*Sym;
    int		Type;

    {

	Symbol	*p;

	/* Allocate memory for the node itself */

	p = (Symbol *) malloc (sizeof (Symbol));

	/* Allocate memory for the symbol's string */

	p->String = malloc (strlen (Sym) + 1);
	strcpy (p->String, Sym);

	/* Save the symbol type */

	p->Type = Type;

	/* Initialize Address field in case this is a label */

	p->Address = -1;
	p->AddrList = 0;

	/* Link the new symbol onto the list */

	p->Next = SymTab;
	SymTab = p;

        return p;

    }

/* end SymInsert */

/*----------*/
Symbol *LookUp (Sym, Type)

    char	*Sym;
    int		Type;

    {

	Symbol	*s;

	s = SymLookUp (Sym);

	/* If not in table, insert it. */

	if (s == NULL) {
	    s = SymInsert (Sym,  Type);
	    return s;
	}

	return s;

    }

/* end LookUp */

/*----------*/
void SequenceSymTab ()	/* Put sequence numbers on the symbols */

    {

	Symbol	*p;
	int	Seq;

	p = SymTab;
	Seq = 0;
	while (p) {
	    if ((p->Type != LABEL) && (p->Type != DEFUNCT))
		p->SymNumber = Seq++;	
	    p = p->Next;
	}

    }

/* end SequenceSymTab */

/*----------*/
void WriteSymbols ()	/* Write symbols to stdout */

    {
	Symbol	*p;

	p = SymTab;
	while (p) {
	    if ((p->Type != LABEL) && (p->Type != DEFUNCT))
		printf ("%c%s\n", SymTypes [p->Type], p->String);
	    p = p->Next;
	}

	putchar (0);
    }

/* end WriteSymbols */

/*----------*/
void CheckLabels ()	/* Check for undefined labels */

    {
	Symbol	*p;
	char	Msg [50];

	p = SymTab;
	while (p) {
	    if (p->Type == LABEL) {
		if (p->Address == -1) {
		    sprintf (Msg, "Missing label: %s", p->String);
		    Error (Msg);
		}
	    }
	    p = p->Next;
	}
    }

/* end CheckLabels */

