/*tree.c,v 1.1.1.1 1999/08/03 14:44:57 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software was developed for DLA by Tate Integrated  */
/*	Systems, L.P. via a contract with Vitro Corporation.	*/
/*	It is designed to work with TIS4000 Release 4.2.	*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*	File:	tree.c		CSL parse tree functions
/*	Author:	Douglas P. Dotson
/*	Date:	10 Apr 1993
/*	Edit:	27 Apr 1993	2110	DPD
*/
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include "csldefs.h"
#include "symtab.h"
#include "tree.h"

/* Functions defined in this module */

TreeNode	*NewNode ();
void		DumpTree ();
void		TravTree ();

/* External data */

extern	char	*SymTypes [];
extern	int	yylineno;

/* Node type descriptions */

static	char	*NodeTypes [] = {
	"Program",
	"Statements",
	"Statement",
	"Command_Statement",
	"Wait_Statement",
	"If_Statement",
	"While_Statement",
	"Set_Statement",
	"Alarm_Statement",
	"Start_Statement",
	"Execute_Statement",
	"Activity_Statement",
	"Goto_Statement",
	"Stop_Statement",
	"StopCSL_Statement",
	"Label_Statement",
	"Time_Units",
	"Expression",
	"Optional_Argument_List",
	"Argument_List",
	"Identifier_Node",
	"Constant_Node",
	"Channel_Node",
	"Texp_Node",
	"Time_Node",
	"Literal_Node"
};

static	char	*ExprOps [] = {
	"",
	"+",
	"-",
	"*",
	"/",
	"<",
	"<=",
	">",
	">=",
	"=",
	"!=",
	"is within",
	"is not within",
	"&&",
	"||",
	"!",
	"^",
	"FuncCall",
	"HasExpired",
	"Reachable",
	"NotReachable",
	"Seconds",
	"Minutes",
	"Hours",
	"#",
	"INVALID",
	"INVALID",
	"INVALID",
	"INVALID",
	"INVALID",
	"INVALID",
	"abs",
	"acos",
	"asin",
	"atan",
	"ceiling",
	"cos",
	"exp",
	"floor",
	"ln",
	"log",
	"max",
	"min",
	"mod",
	"scriptstatus",
	"sin",
	"sqrt",
	"tan"
};

/* Misc data */

TreeNode	*TreeRoot = NULL;
FILE	*TreeFile;

/*----------*/
void SetRoot (p)	/* Save a pointer to the root of the parse tree */

    TreeNode	*p;

    {

	TreeRoot = p;

    }

/* end SetRoot */

/*----------*/
TreeNode *NewNode (NodeType, Left, Middle, Right, Extra, MoreInfo)

    int		NodeType, MoreInfo;
    TreeNode	*Left, *Middle, *Right, *Extra;

    {

	TreeNode	*p;

	p = (TreeNode *) malloc (sizeof (TreeNode));
	if (p == NULL) {
	    printf ("Out of memory for parser tree -- write smaller programs!\n");
	    exit (1);
	}

	p->NodeType = NodeType;
	p->Left = Left;
	p->Middle = Middle;
	p->Right = Right;
	p->Extra = Extra;
	p->MoreInfo = MoreInfo;
	p->Sym = NULL;
	p->LineNumber = yylineno;
	return p;

    }

/* end NewNode */

/*----------*/
TreeNode *NewTermNode (NodeType, Sym)

    int		NodeType;
    Symbol	*Sym;

    {

	TreeNode	*p;

	p = (TreeNode *) malloc (sizeof (TreeNode));
	if (p == NULL) {
	    printf ("Out of memory for parser tree -- write smaller programs!\n");
	    exit (1);
	}

	p->NodeType = NodeType;
	p->Left = NULL;
	p->Middle = NULL;
	p->Right = NULL;
	p->Extra = NULL;
	p->Sym = Sym;
	p->MoreInfo = -1;
	p->LineNumber = yylineno;

	return p;


    }

/* end NewTermNode */

