/*ts_driv.c,v 1.1.1.1 1999/08/03 14:44:59 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/****************************************************************/
/*								*/
/*			TIME STAMP GENERATION			*/
/*								*/
/*	EPICS has its own format for time stamps, defined in	*/
/*	tsDefs.h. This task maintains a globally available	*/
/*	TS_STAMP named time_stamp. We will listen to the	*/
/*	broadcasts from the time synchronization master and	*/
/*	unconditionally set our time of day to whatever he	*/
/*	says. Also, every vxWorks tick we will add the		*/
/*	appropriate number of nanoseconds to time_stamp.	*/
/*								*/
/****************************************************************/

#include <tsDefs.h>
#include <task_params.h>
#include <vxWorks.h>
#include <wdLib.h>
#include <socket.h>
#include <sockLib.h>
#include <in.h>
#include <timeSync.h>
#include <stdio.h>

#include "timLib.h"

TS_STAMP		time_stamp = {0, 0};	/* The global time stamp */

int			timeSocket;		/* UDP Socket */
struct tisTimeSync	timeMsg;		/* Message we'll receive */
TS_STAMP		oldStamp;		/* Our stamp prior to msg */
WDOG_ID			watchDog;		/* Watchdog ID */

int		gts_trigger_counter = 0;


#define	ONE_TICK_IN_NSECS	16666667
#define TIME_PORT		8586

void watchDogService(void);
void timerTask(void);

/****************************************************************/
/*								*/
/*			INITIALIZE				*/
/*								*/
/****************************************************************/

int ts_init()
{
#ifndef PRIV_AUDIT
	struct sockaddr_in	setPort;

/*
	Read the current date and time out of the hardware
	TOD clock if there is one.
*/

	readTodClock();

/*
	Create the socket for receiving time from the master
*/

	if ((timeSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
		{
		printErr("Can't create time stamp socket.\n");
		return (-1);
		}

	setPort.sin_family = AF_INET;
	setPort.sin_addr.s_addr = htonl(INADDR_ANY);
	setPort.sin_port = htons(TIME_PORT);
	if (bind(timeSocket, (struct sockaddr *) &setPort, sizeof(setPort)) == -1)
		{
		printErr("Can't bind to time stamp socket.\n");
		return (-1);
		}
#endif PRIV_AUDIT
/*
	Create the watchdog so we can strobe every tick
*/

	if ((watchDog = wdCreate()) == NULL)
		{
		printErr("Can't create time stamp watch dog.\n");
		return (-1);
		}
	wdStart(watchDog, 1, (FUNCPTR)watchDogService, 0);

#ifndef PRIV_AUDIT
/*
	And now, spawn the task that will read the time from the master
*/

	taskSpawn(TIMESTAMP_NAME, TIMESTAMP_PRI, TIMESTAMP_OPT, TIMESTAMP_STACK,
		  (FUNCPTR)timerTask,0,0,0,0,0,0,0,0,0,0);
#endif PRIV_AUDIT
	return 0;
}

/****************************************************************/
/*								*/
/*		WATCHDOG TIMER SERVICE ROUTINE			*/
/*								*/
/****************************************************************/

void watchDogService(void)
{

	wdStart(watchDog, 1, (FUNCPTR)watchDogService, 0);

/*
	Update the time stamp with interrupts locked out
*/

/******
	Try to avoid playing with the interrupt system, even for
	this little amount of time. A watchdog service routine
	runs at interrrupt level, so we have little to fear from
	running without locking the interrupts.

	key = intLock();
******/
	if ((time_stamp.nsec += ONE_TICK_IN_NSECS) > 1000000000)
		{
		time_stamp.nsec -= 1000000000;
		time_stamp.secPastEpoch++;
		}

/******
	Can't unlock that which we didn't lock. See the above note.

	intUnlock(key);
******/
}

#ifndef PRIV_AUDIT
/****************************************************************/
/*								*/
/*		WATCHDOG MESSAGE TASK				*/
/*								*/
/****************************************************************/

extern long tsMinWest;	/* defined in libCom/tsSubr.c */
void timerTask(void)
{
	struct sockaddr		timeMaster;
	int			timeMasterSize;

	for (;;)
		{
		timeMasterSize = sizeof(timeMaster);
		recvfrom(timeSocket, (char *)&timeMsg, sizeof(timeMsg), 0,
				&timeMaster, &timeMasterSize);
		

		/*
		Do the rest of this with interrupts off to prevent
		interference from watchdog service or someone from
		grabbing just part of a stamp.
		*/

		/******
		This makes me nervous. I think we should at least do
		a taskLock() here. We system guys should talk amongst
		ourselves about this.

		key = intLock();
		******/

		timeMsg.time.secPastEpoch = ntohl(timeMsg.time.secPastEpoch);
		timeMsg.time.nsec = ntohl(timeMsg.time.nsec);
		oldStamp = time_stamp;
		time_stamp = timeMsg.time;

		/******
		This is the other half of what we've removed above. I
		vote for a taskLock() which will keep any of the record
		processing routines from grabbing a time stamp that's
		only been half updated.

		intUnlock(key);
		******/

		tsMinWest = ntohl(timeMsg.minWest);

		/*
		Now write this new time into the TOD clock if there
		is one.
		*/

		setTodClock();
		}
}
#endif PRIV_AUDIT

