#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#ifndef	WIN32
#include <unistd.h>
#endif	!WIN32

#include "dbDefs.h"
#include "msgMon.h"
#include "server.h"
#include "client.h"
#include "OperPriv.h"
#include "OpPrivData.h"

CLIENT *client;	/* connection to OperPriv server */

static void SetListenSocket (int port);
static void GetSessionSelectMask (fd_set *fds);
static void ProcessSessionSelect (fd_set *fds);
static void processServerMsg (SERVER *server);

static SOCKET	listenSocket;

#define MAX_APPS 50
static	char *appNames[MAX_APPS];
static SERVER *appServers[MAX_APPS];

RespUnion   rspmsg;

int GetAppOpenResp (AppControlMsg *reqmsg, SERVER *server);
int GetAppExitResp (AppControlMsg *reqmsg, SERVER *server);
int GetAppKillResp (AppControlMsg *reqmsg, SERVER *server);
int GetAppListResp (AppListMsg *reqmsg, SERVER *server);


/* _beginthread and thr_create expect differant function types */
#ifdef	WIN32
void privService (void * param)
#else
void *privService (void * param)
#endif	WIN32
{
    fd_set	fds;
    int		stat;
#ifdef WIN32
    WSADATA	wsaData; 
    WORD	wVersionRequested = MAKEWORD(1, 1);
    int		err;
 
    err = WSAStartup(wVersionRequested, &wsaData); 
 
    if (err != 0) 
	/* Tell the user that we couldn't find a useable */ 
	/* winsock.dll.     */ 
	return; 
#endif WIN32
   /* open the priviledge service port */
    SetListenSocket (8003);
    while (1)
    {
	GetSessionSelectMask (&fds);
	stat = select (FD_SETSIZE, &fds/* read */, NULL/* write */, NULL/* except */, NULL);
	if (stat > 0)
	{
	    /* this function will accept new priv clients and call back
	    ** to processServerMsg () if any priv client has business.
	    */
	    ProcessSessionSelect (&fds);
	}
	else if (stat == SOCKET_ERROR)
	{
#ifdef	WIN32
	    err = WSAGetLastError ();
	    tisSysLog (0, 0, "privService Select error %d", err);
#else
	    tisSysLog (0, 0, "privService Select error %m");
#endif	WIN32
	}
    }
}


void SetListenSocket (int port)
{
    struct sockaddr_in sa;
    int salen;
    struct hostent *ha;
    int true = 1;

    /* create the TCP socket */
    listenSocket = socket(PF_INET, SOCK_STREAM, 0);
    ha = gethostbyname("localhost");
    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = 0;
    sa.sin_port = htons((short)port);
    if (bind (listenSocket, (struct sockaddr *)&sa, sizeof(sa)) == 0)
    {
	if (listen (listenSocket, 2) != 0)
	    exit (0);
	salen = sizeof(sa);
#ifdef WIN32
	if (ioctlsocket (listenSocket, FIONBIO, &true) == SOCKET_ERROR)
#else
	if (ioctl (listenSocket, FIONBIO, &true) == SOCKET_ERROR)
#endif WIN32
	{
	    /* couldn't set non-blocking */
#ifdef	WIN32
	    int ioctlErr = WSAGetLastError ();
	    tisSysLog (0, 0, "Couldn't set socket non-blocking %d", ioctlErr);
#else
	    tisSysLog (0, 0, "Couldn't set socket non-blocking %m");
#endif	WIN32
	}
    }
}

void GetSessionSelectMask (fd_set *fds)
{
    int i;

    FD_ZERO (fds);
    FD_SET (listenSocket, fds);
    for (i = 0; i < MAX_APPS; i++)
    {
	if (appServers[i])
	{
	    /* clean up closed or bad socket */
	    if (appServers[i]->s == INVALID_SOCKET)
	    {
		CloseSvcSession (appServers[i]);
		appServers[i] = NULL;
	    }
	    else
	    {
		FD_SET (appServers[i]->s, fds);
	    }
	}
    }
}


void ProcessSessionSelect (fd_set *fds)
{
    struct sockaddr_in sa;
    int salen;
    SOCKET  newSock;
    int i;

    if (FD_ISSET (listenSocket, fds))
    {
	FD_CLR (listenSocket, fds);
	salen = sizeof(sa);
	if ((newSock = accept (listenSocket, (struct sockaddr *)&sa, &salen)) < 0)
	{
#ifdef WIN32
	    errno = WSAGetLastError ();
	    if (errno != WSAEINTR)
#else
	    if (errno != EINTR)
#endif WIN32
	    exit (0);
	}
	for (i = 0; i < MAX_APPS; i++)
	{
	    if (appServers [i] == NULL)
	        break;
	}
	if (i < MAX_APPS)
	{
	    appServers [i] = OpenMultiService ("tisPrivRedirect", FALSE, "", newSock);
	}

    }
    for (i = 0; i < MAX_APPS; i++)
    {
	if (appServers[i] && FD_ISSET (appServers[i]->s, fds))
	{
	    FD_CLR (appServers[i]->s, fds);
	    processServerMsg (appServers[i]);
	}
    }
}


/* process messages for OpPrivClient functions, filter app control */
static void processServerMsg (SERVER *server)
{
    ReqUnion	*reqmsg;
    RespUnion   *resp;
    short int	msglen;
    short int	msgSize;
    short int	reqtype;

    while (reqmsg = GetReqLen (server, &msgSize))
    {
	reqtype = ntohs (((PrivMsg *)reqmsg)->type);
	switch (reqtype)
	{
	    case APP_OPEN:  /* App name registration */
		msglen = GetAppOpenResp ((AppControlMsg *)reqmsg, server);
		SendRsp (server, (u_char *)&rspmsg, msglen);
		break;
	    case APP_EXIT:  /* App exiting (send done in sub, server now closed!) */
		msglen = GetAppExitResp ((AppControlMsg *)reqmsg, server);
		break;
	    case APP_KILL: /* Terminate an application */
		msglen = GetAppKillResp ((AppControlMsg *)reqmsg, server);
		SendRsp (server, (u_char *)&rspmsg, msglen);
		break;
	    case APP_LIST:  /* Get list of apps */
		msglen = GetAppListResp ((AppListMsg *)reqmsg, server);
		SendRsp (server, (u_char *)&rspmsg, msglen);
		break;
	    case OP_LOGON:
	    case OP_LOGOFF:
	    case OP_LOGCHECK:
	    case OP_CHECK_PRIV:
	    case OP_PRIV:
	    case WRITE_TRACK:
	    case OPEN_TRACK:
	    case CLOSE_TRACK:
	    case CLOSE_PRIV:
		/* open trend data server if necessary*/
		if (!client)
		{
		    client = OpenServer (GetPrivServer (NULL, 0), 1, NULL);
		}
		if (client)
		{
		    resp = SendReqLen(client, reqmsg, msgSize, &msglen);
		    if (resp)
			SendRsp (server, (unsigned char *)resp, msglen);
		}
		break;
	}
	free (reqmsg);
    }
}


int GetAppOpenResp (AppControlMsg *reqmsg, SERVER *server)
{
    int	    i;
    short int	retVal;
    int	    blankIx = -1;

    AppControlRsp *appRsp = (AppControlRsp*)&rspmsg;
    appRsp->type = htons (APP_OPEN);
    /* search apps for app name, record server and name,
    *  return OK or DUP */
    retVal = APP_OK;
    for (i = 0; i < MAX_APPS; i++)
    {
	/* should compare only to \t or 0 to allow PIDs in names */
	if (strcmp (appNames[i], reqmsg->appname) == 0)
	    retVal = APP_DUP;
	else if (appServers[i] == server)
	    retVal = APP_ERROR; /* duplicate server id! */
	else if ((appNames[i] == NULL) && (blankIx == -1))
	    blankIx = i;
    }
    if (retVal != APP_ERROR)
    {
	appNames[i] = malloc (strlen (reqmsg->appname) + 1);
	if (appNames[i])
	{
	    strcpy (appNames[i], reqmsg->appname);
	    appServers[i] = server;
	}
	else /* malloc failed */
	    retVal = APP_ERROR;
    }
    appRsp->value = htons ( retVal );
    return (sizeof(AppControlRsp));
}


int GetAppExitResp (AppControlMsg *reqmsg, SERVER *server)
{
    int	    i;
    short int	retVal;

    AppControlRsp *appRsp = (AppControlRsp*)&rspmsg;
    appRsp->type = htons (APP_EXIT);
    /* search apps for app name, close server and free name,
    *  return OK or ERROR */
    retVal = APP_OK;
    for (i = 0; i < MAX_APPS; i++)
    {
	if (strcmp (appNames[i], reqmsg->appname) == 0)
	    break;
    }
    if (i < MAX_APPS)
    {
	free (appNames[i]);
	appNames[i] = NULL;
	appRsp->value = htons ( retVal );
	SendRsp (server, (unsigned char *)appRsp, sizeof(AppControlRsp));
	/* CloseSvcSession (server); */
    }
    else
    { /* not found */
	retVal = APP_ERROR;
	appRsp->value = htons ( retVal );
	SendRsp (server, (unsigned char *)appRsp, sizeof(AppControlRsp));
    }
    return (0);
}


int GetAppKillResp (AppControlMsg *reqmsg, SERVER *server)
{
    int	    i;
    short int retVal;

    AppControlRsp *appRsp = (AppControlRsp*)&rspmsg;
    appRsp->type = htons (APP_KILL);
    /* search apps for app name, deliver kill msg to associated server, close server */
    retVal = APP_OK;
    for (i = 0; i < MAX_APPS; i++)
    {
	if (strcmp (appNames[i], reqmsg->appname) == 0)
	    break;
    }
    if (i < MAX_APPS)
    {
	free (appNames[i]);
	appNames[i] = NULL;
	CloseSvcSession (appServers[i]);
	appServers[i] = NULL;
    }
    else
    { /* not found */
	retVal = APP_ERROR;
    }
    appRsp->value = htons ( retVal );
    return (sizeof(AppControlRsp));
}


int GetAppListResp (AppListMsg *reqmsg, SERVER *server)
{
    int	    i;
    short int appLen;
    char    *dptr;

    AppListRsp *appRsp = (AppListRsp*)&rspmsg;
    appRsp->type = htons (APP_LIST);
    /* return list of appNames, terminate with empty string */
    appLen = 0;
    dptr = appRsp->appString;
    for (i = 0; i < MAX_APPS; i++)
    {
	if (appNames[i])
	{
	    if ((appLen + strlen (appNames[i]) + 2) < 1024)
	    {
		strcpy (dptr, appNames[i]);
		dptr += strlen (appNames[i]) + 1;
		appLen += strlen (appNames[i]) + 1;
	    }
	    else /* can't deliver app names in 1024 buffer! */
	    {
		break;
	    }
	}
    }
    strcpy (dptr, "");
    appLen ++;
    appRsp->value = htons ( APP_OK );
    return (sizeof(AppListRsp) - 1024 + appLen);
}

