#ifndef tispermaccessHXX
#define	tispermaccessHXX

// define extra width for dialog if ruler title > ruler length
#define	EXTRA_WIDTH	25

class UtilListView;
class UtilItemList;
class ListEntry;
class Toggle;
class PtProtListView;
class PtProtPrivLevel;
class PtProtApplyButton;
class PtProtDismissButton;

class AccessDialog : public vdialog
{
    MainDialog	*mainDialog;
    vchar	groupName [GROUPNAME_SZ + 1];
    int		utilModified;		    // utility access toggled
    int		ptgrpModified;		    // change to protection group permission

    void	InitSelectedFlag (void);
    void	SetDialogTitle (void);
    void	SetPtProtCellWidth (void);
    void	SetPtProtRulerWidth (void);
    void	SetUtilRulerWidth (void);
    void	UpdateToggleModifyPtr (void);

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(AccessDialog);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(AccessDialog, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // overrides 
    void ObserveWindow (vevent *event);
    int HandleConfigure (vevent *event);
    int HandleLose (vevent *event);

    // override
    void Destroy ();

    void OpenAccess (MainDialog *main, vchar *groupname);

    // ptrs to some dialog stuff
    vruler		*utilRuler;
    UtilListView	*utilListView;
    vruler		*ptProtRuler;
    PtProtListView	*ptProtListView;
    PtProtPrivLevel	*privilegeLevel;
    PtProtApplyButton	*applyButton;
    PtProtDismissButton	*dismissButton;


    // some public funcs
    vchar *GetGroupName (void)
	{ return (groupName); }
    MainDialog *GetMainDialog (void)
	{ return (mainDialog); }
    int GetUtilModifiedFlag (void)
	{ return (utilModified); }
    int GetPtGrpModifiedFlag (void)
	{ return (ptgrpModified); }
};


/*
    mainsClosedImage		mainsOpenImage
	+ -MAIN				- -MAIN
					|

    closedWithSubGroups		closedWithNoSubGroups
	|				|
	+ -GROUP			+ -GROUP
	|

    openWithGroups		openWithNoGroups
	|				|
	- -GROUP			- -GROUP
	|

    continueLine
	|

    continueLineWithGroups	endOfGroup
	|				|
	|- -GROUP			- -GROUP
	|
*/

// define inset value
#define	ENTRY_LEVEL_INSET	20
// define a few pixels to place after the text string for highlighting
#define	TEXT_WIDTH_ADD		2
// define space between toggle and togggle option text
#define	TOGGLE_OPTION_SPACE 5
// define max nuber toggles in toggle array
#define	UTIL_MAX_TOGGLES    60

// define isOpen
typedef enum
{
    TISPERM_BOX_CLOSED = 0,
    TISPERM_BOX_OPENED
} TISPERM_BOX_STATUS;


class  UtilListView : public vlistview
{
    vlist	    *list;
    AccessDialog    *accessDialog;
    int		    buttonUpFlag;
    UtilItemList    *itemList;
    ListEntry	    *selectedEntry;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(UtilListView);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(UtilListView, vlistview);

    // vloadable override
    void LoadInit (vresource resource);

    // override
    void Destroy ();

    //  overrides
    void Close();
    void DrawCellBackground (int row, int column, int selected, vrect *rect);
    void DrawCellContents (int row, int col, int selected, vrect *rect);
    void DrawCellFocus (int row, int column, vrect *rect);
    void Focus();
    int  HandleButtonDown(vevent *event);
    int  HandleButtonUp(vevent *event);
    int  HandleDrag(vevent *event);
    int  HandleKeyDown(vevent *event);
    int  HandleKeyUp(vevent *event);
    int  IssueSelector (int selector, vdict *context);
    void ScrollAbsolute(int horzDistance, int vertDistance);
    void ObserveSelect (int state);
    void Open();
    void Unfocus();

    // class extensions

    // close entry in list
    void CloseEntry (int row, ListEntry *entry);

    // put MAIN into list
    void InsertMain (void);

    // get the accessDialog
    AccessDialog *GetAccessDialog (void)
	{ return (accessDialog); }

    // function for selectedEnrty
    ListEntry *GetSelectedEntry (void)
	{ return (selectedEntry); }

    // open entry in list
    void OpenEntry (int row, ListEntry *entry);

    // function for buttonUpFlag
    void SetButtonUpFlag (int val)
	{ buttonUpFlag = val; }

    // store AccessDialog
    void StoreAccessDialog (AccessDialog *dlg)
	{ accessDialog = dlg; }

    // public so dont need funcs to get at
    vimage	*mainsClosedImage;
    vimage	*mainsOpenImage;
    vimage	*closedWithSubGroups;
    vimage	*closedWithNoSubGroups;
    vimage	*openWithGroups;
    vimage	*openWithNoGroups;
    vimage	*continueLine;
    vimage	*continueLineWithGroups;
    vimage	*endOfGroup;

    // function to fill toggles array array
    void	AddToggles (void);
    Toggle	*toggles [UTIL_MAX_TOGGLES];
};


class UtilItemList : public vdialogItemList
{

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(UtilItemList);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(UtilItemList, vdialogItemList);
    
    void    DetermineBounds (vrect *bounds);
    void    DetermineColorsFrom (long flags, vcolor **known, int *ids,
				 int count, vcolor **colors);
    void    DetermineColorSchemeFrom (vcolorScheme *known,
				      unsigned long flags, vcolorScheme *scheme);
    void    DetermineContent (vrect *rect);
    vcursor *DetermineCursor (void);
    vdialog *DetermineDialog (void);
    int	    DetermineEnabled (void);
    vfont   *DetermineFont (void);
    int	    DetermineLineWidth (void);
    int	    DetermineShown (void);
    void    DrawDevice (void);
    vdict   *GetKeyDispatch (void);
    vdict   *GetKeyMap (void);
};


class ListEntry
{
    TISPERM_BOX_STATUS	isOpen;
    int			isLocked;
    TREENODE_PTR	treeNode;
    Toggle		*toggle;

public:

    ListEntry (void);
    ~ListEntry (void);

    // functions to handle isOpen flag
    void SetOpen (TISPERM_BOX_STATUS flag)
	{ isOpen = flag; }
    TISPERM_BOX_STATUS IsOpen (void)
	{ return (isOpen); }

    // functions to handle isLocked flag
    void SetLocked (int flag)
	{ isLocked = flag; }
    int IsLocked (void)
	{ return (isLocked); }

    // functions to handle treeNode
    void SetTreeNode (TREENODE_PTR node)
	{ treeNode = node; }
    TREENODE_PTR GetTreeNode (void)
	{ return (treeNode); }

    // function to return image ptr
    vimage *GetImage (UtilListView *utilListView);

    // function for vdialogItem item
    Toggle *GetToggle (void)
	{ return (toggle); }
    void SetToggle (Toggle *tg)
	{ toggle = tg; }
};  


class Toggle : public vcontrolToggle
{
    UtilListView *utilListView;
    ListEntry	*listEntry;
    int		*utilModified;	    // ptr to this flag in AccessDialog

public:
    // overrides from vloadable
    vkindTYPED_FULL_DECLARATION(Toggle);

    // the standard versions of the necessary constructor/destructor
    vloadableINLINE_CONSTRUCTORS(Toggle, vcontrolToggle);
    
    // virtual member function overrides
    void ObserveDialogItem(vevent *event);

    // class extensions
    void SetEntry (ListEntry *entry)
	{ listEntry = entry; }
    void SetUtilListView (UtilListView *lview)
	{ utilListView = lview; }
    void SetUtilModified (int *mod)
	{ utilModified = mod; }
};  


/* define stuff for the point protection list
*/

class  PtProtListView : public vlistview
{
    AccessDialog    *accessDialog;
    vliststr	    *list;

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(PtProtListView);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(PtProtListView, vlistview);

    // overrides
    void Destroy ();
    void LoadInit (vresource res);
    void ObserveSelect (int state);

    // some public funcs
    void BuildPtGroupsList (void);
    // store AccessDialog
    void StoreAccessDialog (AccessDialog *dlg)
	{ accessDialog = dlg; }
};


class PtProtPrivLevel : public vcontainerExclusiveGroup
{
    AccessDialog    *accessDialog;
    int		    *ptgrpModified;

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(PtProtPrivLevel);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(PtProtPrivLevel, vcontainerExclusiveGroup);

    // vloadable override
    void LoadInit (vresource resource);

    //override virtual issue function
    void ObserveDialogItem(vevent *event);

    // radio button toggles in the container
#ifdef	CALISO
    vcontrolToggle	    *fullAccessToggle;
    vcontrolToggle	    *viewOnlyToggle;
    vcontrolToggle	    *noAccessToggle;
#else
    vcontrolToggle	    *adminToggle;
    vcontrolToggle	    *engineerToggle;
    vcontrolToggle	    *supervisorToggle;
    vcontrolToggle	    *operatorToggle;
    vcontrolToggle	    *traineeToggle;
    vcontrolToggle	    *viewToggle;
#endif	CALISO

    // some public funcs
    char *GetPrivStr (void);

    // store AccessDialog
    void StoreAccessDialog (AccessDialog *dlg)
	{ accessDialog = dlg; }

    void SetPrivilegeButton (DefaultPrivilegeLevels privLevel);
    void SetProtectionButton (DefaultProtectionLevels privLevel);

    void UpdatePtGrpModified (int *mod)
	{ ptgrpModified = mod; }

};


class  PtProtApplyButton : public vbutton
{
    AccessDialog    *accessDialog;

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(PtProtApplyButton);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(PtProtApplyButton, vbutton);

    //override virtual issue function
   void ObserveDialogItem(vevent *event);

    // store AccessDialog
    void StoreAccessDialog (AccessDialog *dlg)
	{ accessDialog = dlg; }
};


class  PtProtDismissButton : public vbutton
{
    AccessDialog    *accessDialog;

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(PtProtDismissButton);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(PtProtDismissButton, vbutton);

    //override virtual issue function
   void ObserveDialogItem(vevent *event);

    // store AccessDialog
    void StoreAccessDialog (AccessDialog *dlg)
	{ accessDialog = dlg; }
};

#endif	tispermaccessHXX

