#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vlistviewHEADER
#include vrulerHEADER

extern "C"
{
#include "msgMon.h"
}

#include "tisperm.h"
#include "tisPermApp.hxx"
#include "tisPermDatabase.hxx"
#include "tisPermOptions.hxx"
#include "tisPermMain.hxx"


/*******************************************************************
*
*    Main ()
*
*******************************************************************/

int main (int argc, char **argv)
{
    tisOpenLog ("tisPermissions", 6, 2, NULL, NULL);

    // Initialize Galaxy
    vstartup (argc, argv);

    vapplication *app = vapplication::GetCurrent();

    TisPermApp *tisPermApp = new TisPermApp (argc, argv);

    if ((tisPermApp -> GetInitErrors ()) == TISPERM_ERROR)
    {
	delete tisPermApp;
	exit (EXIT_FAILURE);
    }

    // enter main event process loop
    veventProcess ();

    delete tisPermApp;

    tisSysLog (0,0, "Normal exit\n");
    
    exit (EXIT_SUCCESS);
    return EXIT_FAILURE;
}


/********************************************************************************
*
*    TisPermApp application constuctor
*
********************************************************************************/

TisPermApp::TisPermApp (int argv, char **argc)
{
    initErrors = TISPERM_SUCCESS;
    databaseInited = vFALSE;
    optionTree = NULL;
    mainDialog = NULL;
    initedTisGalaxy = vFALSE;

    tisPermVapp = vapplication::GetCurrent ();
    tisPermResource = tisPermVapp -> GetResources ();

    // intern tag names
    tispermInternNames ();

    // get and store the titles and messages dictionary

    titlesResource = vresourceGet (tisPermResource, tisperm_TitlesDictionary);
    messagesResource = vresourceGet (tisPermResource, tisperm_MessagesDictionary);

    permDatabase = new TisPermDatabase (this);
    if ((permDatabase -> OpenDbFiles ()) == TISPERM_ERROR)
    {
	initErrors = TISPERM_ERROR;
	return;
    }
    databaseInited = vTRUE;
    optionTree = new OptionTree (this);
    if ((optionTree -> LoadTree ()) == TISPERM_ERROR)
    {
	initErrors = TISPERM_ERROR;
	return;
    }
//    optionTree -> PrintTree ();

    // init tisGalaxy, see tisgalaxylib.<hxx,cxx>
    tisGalaxyResource = startTisGalaxy ();
    initedTisGalaxy = vTRUE;

    // load the mainDialog

    mainDlgResource = vresourceGet (tisPermResource, tisperm_MainDialog);
    mainDialog = new MainDialog (mainDlgResource);
 
    mainDialog -> OpenMain (this);
}


/********************************************************************************
*
*    TisPermApp application destructor
*
********************************************************************************/

TisPermApp::~TisPermApp (void)
{
    // close down tisGalaxy if used
    if (initedTisGalaxy == vTRUE)
    {
	stopTisGalaxy (tisGalaxyResource);
    }

    // delete the options tree
    if (optionTree != NULL)
    {
	delete optionTree;
    }

    // close down codebase
    if (databaseInited == vTRUE)
    {
	delete permDatabase;
    }

    // delete the main dialog
    if (mainDialog != NULL)
    {
	delete mainDialog;
    }
}

