#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstartupHEADER
#include vlistviewHEADER
#include vlistHEADER
#include vliststrHEADER
#include vmemHEADER

#include <errno.h>

extern "C"
{
#include <string.h>
#include "resolveLinks.h"
#include "special.h"
#include "dbFldTypes.h"
}

#include "tisgalaxy.h"
#include "tisperm.h"
#include "tisPermApp.hxx"
#include "tisPermDatabase.hxx"
#include "tisPermOptions.hxx"
#include "tisPermDbDefs.hxx"
#include "tisPermUtils.hxx"

static int  compare_fields (const void *fld1, const void *fld2);

/********************************************************************************
*
*    TisPermDatabase constuctor
*
********************************************************************************/

TisPermDatabase::TisPermDatabase (TisPermApp *app)
{
    tisPermApp = app;

#if defined (WIN32) || defined(USECB63)
    code4init (&this -> cb);
    cb.errFieldName = 0;
    cb.errOpen = 0;
    cb.errTagName = 0;
    cb.readLock = 0;
    cb.errCreate = 0;
    cb.errGo = 0;
    cb.errSkip = 0;
#else
    d4init (&this -> cb);
    cb.field_name_error = 0;
    cb.open_error = 0;
    cb.tag_name_error = 0;
    cb.read_lock = 0;
    cb.create_error = 0;
    cb.go_error = 0;
    cb.skip_error = 0;
#endif WIN32

    sysUsers = NULL;
    tisFldPrott = NULL;
    tisGroups = NULL;
    tisPrivTab = NULL;
    tisProtGroups = NULL;
    tisUsers = NULL;
    utilAcc = NULL;
    fieldsLists = NULL;
    numFieldsLists = 0;
    ptTypeList = NULL;
}


/********************************************************************************
*
*    TisPermDatabase destructor
*
********************************************************************************/

TisPermDatabase::~TisPermDatabase (void)
{
    if (sysUsers != NULL)
    {
	delete sysUsers;
    }
    if (tisFldPrott != NULL)
    {
	delete tisFldPrott;
    }
    if (tisGroups != NULL)
    {
	delete tisGroups;
    }
    if (tisPrivTab != NULL)
    {
	delete tisPrivTab;
    }
    if (tisProtGroups != NULL)
    {
	delete tisProtGroups;
    }
    if (tisUsers != NULL)
    {
	delete tisUsers;
    }
    if (utilAcc != NULL)
    {
	delete utilAcc;
    }
    if (fieldsLists != NULL)
    {
	vlist		*list;
	void		*listValue;
	vlistIterator	iterator;
	int		i;
	for (i = 0; i < numFieldsLists; i++)
	{
	    list = fieldsLists[i];
	    iterator.Start (list);
	    while (iterator.Next ())
	    {
		listValue = (void *)(iterator.GetValue ());
		vmemFree (listValue);
	    }
	    iterator.Finish ();
	    delete list;
	}
	vmemFree ((void *)fieldsLists);
    }
    if (ptTypeList != NULL)
    {
	delete ptTypeList;
    }
#if defined (WIN32) || defined(USECB63)
    code4initUndo (&this -> cb);
#else
    d4init_undo (&this -> cb);
#endif WIN32
}


/********************************************************************************
*
*    TisPermDatabase open database files
*
********************************************************************************/

int TisPermDatabase::OpenDbFiles (void)
{
    if (BuildLists ())
    {
	return (TISPERM_ERROR);
    }
    TraceLists ();

    sysUsers = new Sysusers (&this -> cb);
    sysUsers -> SetDb (this);
    tisFldPrott = new Tisfldprott (&this -> cb);
    tisFldPrott -> SetDb (this);
    tisGroups = new Tisgroups (&this -> cb);
    tisGroups -> SetDb (this);
    tisPrivTab = new Tisprivtab (&this -> cb);
    tisPrivTab -> SetDb (this);
    tisProtGroups = new Tisprotgroups (&this -> cb);
    tisProtGroups -> SetDb (this);
    tisUsers = new Tisusers (&this -> cb);
    tisUsers -> SetDb (this);
    utilAcc = new Tisutilacc (&this -> cb);
    utilAcc -> SetDb (this);

    if ((sysUsers -> OpenSysusers ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((sysUsers -> MapSysuserFlds()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisFldPrott -> OpenFldProttDb ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisFldPrott -> MapFldProttFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisGroups -> OpenGroupsDb (sysUsers)) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisGroups -> MapGroupsFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisPrivTab -> OpenPrivTabDb ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisPrivTab -> MapPrivTabFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    // NOTE: open after tisPrivTab, since need ptr to it
    if ((tisProtGroups -> OpenProtGroupsDb (tisPrivTab)) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisProtGroups -> MapProtGroupsFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisUsers -> OpenUsersDb (sysUsers)) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((tisUsers -> MapUsersFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((utilAcc -> OpenUtilAccDb ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    if ((utilAcc -> MapUtilAccFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*    TisPermDatabase build fieldsLists  and ptFldList function
*	build a pointer to an array of vlists, that defines the
*	field paramters for each point type.
*	Build the point types list
*	These lists is used by tisPermFldPerms (field permissions)
*
********************************************************************************/

int TisPermDatabase::BuildLists (void)
{
    char		unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char		path [TISPERM_MAX_DATABASE_PATH];
    FILE		*fd;
    char		inbuff[256];
    vlist		*list;
    TISPERM_FIELD_DEF	*fldDef;
    int			listsIndx = 0;
    int			row;
    char		currPtType [POINTTYPE_SZ + 1];
    char		*cPtr;
    char		*token;
    char		*fgetsRet;
    int			firstList = vTRUE;
    int			ptRow = 0;
    int			numPtTypes = 0;

#ifdef	WIN32
    sprintf (unresolvedPath, "\\tis4000\\tasks\\%s", FIELDS_INFO_FILE);
#else
    sprintf (unresolvedPath, "/tis4000/tasks/%s", FIELDS_INFO_FILE);
#endif	WIN32

    resolveLinks (unresolvedPath, path);

    if (CountPointTypes (path, &numPtTypes))
    {
	return (1);
    }

    if ((fieldsLists = 
	(vlist **)(vmemAllocAndClear (sizeof (vlist *) *
		     (numPtTypes + 1)))) == NULL)
    {
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_MemAllocErr,
			   "fields list");
	return (1);
    }

    ptTypeList = new vliststr;

    if ((fd = fopen (path, "r")) == NULL)
    {
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_OpenErr, path);
	return (1);
    }

    memset ((void *)currPtType, 0, sizeof (currPtType));
    row = 0;
    while ((fgetsRet = fgets (inbuff, sizeof (inbuff), fd)) != NULL)
    {
	if ((cPtr = strchr (inbuff, '\n')) != NULL)
	{
	    *cPtr = 0;
	}
	token = strtok (inbuff, ",");
	StripWhitespace (token);
	if ((strcmp (currPtType, token)) != 0)
	{
	    if (firstList == vTRUE)
	    {
		firstList = vFALSE;
	    }
	    else
	    {
		list = fieldsLists[listsIndx-1];
		list -> Sort (vlistROW, 0, compare_fields);
	    }
	    list = new vlist;
	    row = 0;
	    fieldsLists[listsIndx] = list;
	    listsIndx++;
	    numFieldsLists++;
	    strcpy (currPtType, token);
	    // update our point type list
	    ptTypeList -> SetCellValueScribed (ptRow, 0, vcharScribeSystem (currPtType));
	    ptRow++;
	}
	fldDef = (TISPERM_FIELD_DEF *)(vmemAllocAndClear (sizeof (TISPERM_FIELD_DEF)));
	strcpy (fldDef -> ptType, token);
	fldDef -> ptType [sizeof (fldDef -> ptType) - 1] = 0;
	token = strtok (NULL, ",");
	StripWhitespace (token);
	strcpy (fldDef -> fldName, token);
	fldDef -> fldName [sizeof (fldDef -> fldName) - 1] = 0;
	token = strtok (NULL, ",");
	StripWhitespace (token);
	fldDef -> fldType = atoi (token);
	token = strtok (NULL, ",");
	StripWhitespace (token);
	fldDef -> special = atoi (token);
	// these are not to be shown to the viewer, so why put them
	// in the list
	if ( (fldDef -> special == SPC_NOMOD) ||
	     (fldDef -> fldType == DBF_NOACCESS) ||
	     ((strncmp (fldDef -> fldName, "UDF", 3)) == 0) )
	{
	    vmemFree ((void *)fldDef);
	    continue;
	}
	token = strtok (NULL, ",");
	StripWhitespace (token);
	strcpy (fldDef -> fldComment, token);
	fldDef -> fldComment [sizeof (fldDef -> fldComment) - 1] = 0;
	list -> SetCellValue (row, 0, (void *)fldDef);
	row++;
    }
    // sort the last list
    list = fieldsLists[listsIndx-1];
    list -> Sort (vlistROW, 0, compare_fields);

    fclose (fd);

    return (0);
}



/********************************************************************************
*
*    TisPermDatabase count point types function
*
********************************************************************************/

int TisPermDatabase::CountPointTypes (char *path, int *numPtTypes)
{
    FILE	*fd;
    char	inbuff[256];
    char	*token;
    char	currPtType [POINTTYPE_SZ + 1];
    char	*cPtr;
    char	*fgetsRet;
    int		numTypes = 0;


    if ((fd = fopen (path, "r")) == NULL)
    {
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_OpenErr, path);
	return (1);
    }
    while ((fgetsRet = fgets (inbuff, sizeof (inbuff), fd)) != NULL)
    {
	if ((cPtr = strchr (inbuff, '\n')) != NULL)
	{
	    *cPtr = 0;
	}
	token = strtok (inbuff, ",");
	StripWhitespace (token);
	if ((strcmp (currPtType, token)) != 0)
	{
	    strcpy (currPtType, token);
	    numTypes++;
	}
    }
    fclose (fd);
    *numPtTypes = numTypes;
    return (0);
}

void TisPermDatabase::TraceLists (void)
{
    FILE		*fd;
    int			i;
    vlist		*list;
    vlistIterator	iterator;
    TISPERM_FIELD_DEF	*fldDef;
    char		ptType[POINTTYPE_SZ + 1];
    vchar		*vptType;


    memset ((void *)ptType, 0, sizeof (ptType));
    fd = fopen ("/tis4000db/listTrace.txt", "w");
    fprintf (fd, "FIELD LISTS\n\n");
    for (i = 0; i < numFieldsLists; i++)
    {
	list = fieldsLists[i];
	iterator.Start (list);
	while (iterator.Next())
	{
	    fldDef = (TISPERM_FIELD_DEF *)(iterator.GetValue ());
	    if ((strcmp (ptType, fldDef -> ptType)) != 0)
	    {
		strcpy (ptType, fldDef -> ptType);
		fprintf (fd, "POINT TYPE\t%s\n",ptType);
	    }
	    fprintf (fd, "FieldName %s\tFieldType %d\tSpecial %d\tFieldComment %s\n",
			fldDef -> fldName, fldDef -> fldType, 
			fldDef -> special, fldDef -> fldComment);
	}
	iterator.Finish ();
    }
    fprintf (fd, "\n\nPOINT TYPES\n\n");
    iterator.Start (ptTypeList);
    while (iterator.Next ())
    {
	vptType = (vchar *)(iterator.GetValue ());
	vcharExportBounded (vcharScribeSystemExport (vptType),
			    ptType,
			    POINTTYPE_SZ);
	ptType [POINTTYPE_SZ] = 0;
	fprintf (fd, "\t%s\n", ptType);
    }
    iterator.Finish ();
    fclose (fd);
}


/********************************************************************************
*
*    TisPermDatabase function to get all groups not in this protection group
*
********************************************************************************/

vliststr *TisPermDatabase::GetGrpsNotInPrGrp (char *protGroup)
{
    vliststr	*groupsList = tisGroups -> GetGroupsList ();
    vliststr	*newGroupsList = NULL;
    char	seekBuff [256];
    DATA4	*tisprivtab = tisPrivTab -> GetPrvTabDb ();
    vchar	*groupNameValue;
    char	groupName [GROUPNAME_SZ + 1];
 
    int		    row = 0;
    int		    col = 0;
    vlistIterator   iterator;

    // select privtab.dbf tag GROUP + PROTECTION GROUP
    DATA4 *tisprivtabDb = tisPrivTab -> GetPrvTabDb ();
    tisPrivTab -> SetOperGroupTag ();

    iterator.Start (groupsList);

    // iterate through groups list and see if group belongs to this 
    // protection group, if not add group to new list
    while (iterator.Next ())
    {
	col = iterator.GetColumn ();
	// column 1 has the group description, not needed
	if (col != 0)
	{
	    continue;
	}
	groupNameValue = (vchar *)(iterator.GetValue());
	vcharExportBounded (vcharScribeSystemExport (groupNameValue),
			    groupName,
			    sizeof (groupName));
	groupName [sizeof (groupName) - 1] = 0;
	sprintf (seekBuff, "%-*.*s%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName,
					   PROTECTIONGRP_SZ, PROTECTIONGRP_SZ, protGroup);
	if ((d4seek (tisprivtabDb, seekBuff)) != r4success)
	{
	    if (newGroupsList == NULL)
	    {
		newGroupsList = new vliststr;
	    }
	    newGroupsList -> SetCellValueScribed (row, 0,
				    vcharScribeSystem (groupName));
	    row++;
	}
    }
    iterator.Finish ();
    delete groupsList;

    return (newGroupsList);
}

/********************************************************************************
*
*    TisPermDatabase function to get all users not in groups
*
********************************************************************************/

vliststr *TisPermDatabase::GetUsersNotInGroups (void)
{
    vlist	*usersList = tisUsers -> GetUsersList ();
    vliststr	*newUsersList = new vliststr;
    char	seekBuff [OPERATOR_SZ + 1];
    DATA4	*sysUsersDb = sysUsers -> GetSysusersDb ();
 
    int		    row = 0;
    int		    col = 0;
    TISUSER	    *tisUser;
    vlistIterator   iterator;
    iterator.Start (usersList);

    // iterate through list and see if user belongs to a group
    // if not add user to new list
    while (iterator.Next ())
    {
	col = iterator.GetColumn ();
	// col 0 and 1 have same struct only use 0
	if (col != 0)
	{
	    continue;
	}
	tisUser = (TISUSER *)(iterator.GetValue ());
	sprintf (seekBuff, "%-*.*s", OPERATOR_SZ, OPERATOR_SZ, tisUser -> username);
	if ((d4seek (sysUsersDb, seekBuff)) != r4success)
	{
	    newUsersList -> SetCellValueScribed (row, 0,
				    vcharScribeSystem (tisUser -> username));
	    row++;
	}
    }
    iterator.Finish ();
    tisUsers -> DeleteUsersList (usersList);

    return (newUsersList);
}


/********************************************************************************
*
*   Sysusers database class, constructor and destructor
*
********************************************************************************/

Sysusers::Sysusers (CODE4 *cB)
{
    cb = cB;
    sysusers = NULL;
    sysnameFld = NULL;
    systypeFld = NULL;
    createdSysusers = vFALSE;
}

Sysusers::~Sysusers (void)
{
    if (sysusers != NULL)
    {
	d4close (sysusers);
    }
}


/********************************************************************************
*
*   Sysusers database class, add a user
*   NOTE: try to reuse deleted records
*
********************************************************************************/

void Sysusers::AddUser (char *userName, char *groupName)
{
    int newRec = vFALSE;
    d4top (sysusers);
    if (d4deleted (sysusers))
    {
	d4recall (sysusers);
    }
    else
    {
	d4append_start (sysusers, 0);
	newRec = vTRUE;
    }
    d4blank (sysusers);
    f4assign (sysnameFld, userName);
    f4assign (systypeFld, groupName);
    if (newRec == vTRUE)
    {
	d4append (sysusers);
    }
    d4flush (sysusers);
    d4unlock (sysusers);
}


/********************************************************************************
*
*    Sysusers class create sysusers.dbf file
*
********************************************************************************/

int Sysusers::CreateSysusersDb (char *dbpath)
{
    if ((sysusers = d4create (cb, dbpath, sysusers_fields, sysusers_tag)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif	USECB63
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }
    createdSysusers = vTRUE;

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Sysusers database class, delete a group
*   NOTE: many users can belong to 1 group so delete them all
*
********************************************************************************/

void Sysusers::DeleteGroup (char *groupName)
{
    char    grpInDb [GROUPNAME_SZ + 1];

    for (d4top (sysusers); !d4eof (sysusers); d4skip (sysusers, 1L))
    {
	if (d4deleted (sysusers))
	{
	    continue;
	}
	strncpy (grpInDb, f4str (systypeFld), sizeof (grpInDb));
	grpInDb [sizeof (grpInDb) - 1] = 0;
	StripWhitespace (grpInDb);
	if ((strncmp (groupName, grpInDb, GROUPNAME_SZ)) == 0)
	{
	    d4blank (sysusers);
	    d4delete (sysusers);
	}
    }
    d4flush (sysusers);
    d4unlock (sysusers);
}


/********************************************************************************
*
*   Sysusers database class, delete a user
*
********************************************************************************/

void Sysusers::DeleteUser (vchar *name)
{
    char    seekBuff [256];
    char    userName [OPERATOR_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), userName, sizeof (userName));
    userName [sizeof (userName) - 1] = 0;
    
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", OPERATOR_SZ, OPERATOR_SZ, userName);
    if ((d4seek (sysusers, seekBuff)) == r4success)
    {
	d4blank (sysusers);
	d4delete (sysusers);
	d4flush (sysusers);
    }
    d4unlock (sysusers);
}


/********************************************************************************
*
*   Sysusers database class, get list of users that belong to groupname
*
********************************************************************************/

vliststr *Sysusers::GetGroupsUsers (vchar *grp)
{
    int		row = 0;
    vliststr	*list = new vliststr;
    char	group [GROUPNAME_SZ + 1];
    char	groupFld [GROUPNAME_SZ + 1];
    char	userName [OPERATOR_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (grp),
		        group,
			sizeof (group));
    group [sizeof (group) - 1] = 0;

    for (d4top (sysusers); !d4eof (sysusers); d4skip (sysusers, 1))
    {
	if (d4deleted (sysusers))
	{
	    continue;
	}
	strncpy (groupFld, f4str (systypeFld), sizeof (groupFld));
	groupFld [sizeof (groupFld) - 1] = 0;
	StripWhitespace (groupFld);
	if ((strncmp (group, groupFld, sizeof (group))) == 0)
	{
	    strncpy (userName, f4str (sysnameFld), sizeof (userName));
	    userName [sizeof (userName) - 1] = 0;
	    StripWhitespace (userName);
	    list -> SetCellValueScribed (row, 0, 
					 vcharScribeSystem (userName));
	    row++;
	}
    }
    return (list);
}

/********************************************************************************
*
*   Sysusers database class, does a group have users in it
*
********************************************************************************/

int Sysusers::GroupHasUsers (char *group)
{
    int		row = 0;
    char	groupFld [GROUPNAME_SZ + 1];

    for (d4top (sysusers); !d4eof (sysusers); d4skip (sysusers, 1))
    {
	if (d4deleted (sysusers))
	{
	    continue;
	}
	strncpy (groupFld, f4str (systypeFld), sizeof (groupFld));
	groupFld [sizeof (groupFld) - 1] = 0;
	StripWhitespace (groupFld);
	if ((strncmp (groupFld, group, sizeof (groupFld))) == 0)
	{
	    return (vTRUE);
	}
    }
    return (vFALSE);
}


/********************************************************************************
*
*   Sysusers database class, Map database fields function
*
********************************************************************************/

int Sysusers::MapSysuserFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    if ((sysnameFld = d4field (sysusers, NAME_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (SYSUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (NAME_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((systypeFld = d4field (sysusers, SYSTYPE_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (SYSUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (SYSTYPE_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Sysusers database class, Open function
*
********************************************************************************/

int Sysusers::OpenSysusers (void)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, SYSUSERS_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreateSysusersDb (dbpath)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((sysusers = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)	
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
       
    nameTag = d4tag (sysusers, NAME_TAG);
    d4tag_select (sysusers, nameTag);

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Sysusers database class, update group with list of users
*
********************************************************************************/

void Sysusers::UpdateSysUsers (vliststr *list, vchar *group)
{
    vlistIterator   iterator;
    vchar	    *userInList;
    char	    seekBuff [OPERATOR_SZ + 1];
    char	    userName [OPERATOR_SZ + 1];
    char	    groupName [GROUPNAME_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (group),
			groupName,
			sizeof (groupName));
    groupName [sizeof (groupName) - 1] = 0;

    // add any new users for group
    iterator.Start (list);
    while (iterator.Next ())
    {
	userInList = (vchar *)(iterator.GetValue ());
	vcharExportBounded (vcharScribeSystemExport (userInList),
			    userName,
			    sizeof (userName));
	userName [sizeof (userName) - 1] = 0;
	sprintf (seekBuff, "%-*.*s", OPERATOR_SZ, OPERATOR_SZ, userName);
	if ((d4seek (sysusers, seekBuff)) != r4success)
	{
	    AddUser (userName, groupName);
	}
    }
    iterator.Finish ();

    // delete any users that are not in the list
    char    groupInDb [GROUPNAME_SZ + 1];
    int	    inDb;
    char    userNameInDb [OPERATOR_SZ + 1];
    for (d4top (sysusers); !d4eof (sysusers); d4skip (sysusers, 1))
    {
	if (d4deleted (sysusers))
	{
	    continue;
	}
	strncpy (groupInDb, f4str (systypeFld), sizeof (groupInDb));
	groupInDb [sizeof (groupInDb) - 1] = 0;
	StripWhitespace (groupInDb);
	// ok found the group, see if the user is in the list
	if ((strncmp (groupName, groupInDb, GROUPNAME_SZ)) == 0)
	{
	    inDb = vFALSE;
	    iterator.Start (list);
	    while (iterator.Next ())
	    {
		userInList = (vchar *)(iterator.GetValue ());
		vcharExportBounded (vcharScribeSystemExport (userInList),
				    userName,
				    sizeof (userName));
		userName [sizeof (userName) - 1] = 0;
		strncpy (userNameInDb, f4str (sysnameFld), sizeof (userNameInDb));
		userNameInDb [sizeof (userNameInDb) - 1] = 0;
		StripWhitespace (userNameInDb);
		if ((strncmp (userName, userNameInDb, OPERATOR_SZ)) == 0)
		{
		    inDb = vTRUE;
		    break;
		}
	    }
	    iterator.Finish ();

	    if (inDb == vFALSE)
	    {
		d4blank (sysusers);
		d4delete (sysusers);
	    }
	}
    }
    d4flush (sysusers);
    d4unlock (sysusers);
}


/********************************************************************************
*
*   Sysusers database class, is a user in a group
*   NOTE: user can only be in 1 group
*
********************************************************************************/

int Sysusers::UserInGroup (char *user, char *group)
{
    char    seekBuff [OPERATOR_SZ + 1];
    char    groupName [GROUPNAME_SZ + 1];

    sprintf (seekBuff, "%-*.*s", OPERATOR_SZ, OPERATOR_SZ, user);
    if ((d4seek (sysusers, seekBuff)) == r4success)
    {
	if ((d4deleted (sysusers)) == 0)
	{
	    strncpy (groupName, f4str (systypeFld), sizeof (groupName));
	    groupName [sizeof (groupName) - 1] = 0;
	    StripWhitespace (groupName);
	    strcpy (group, groupName);
	    return (vTRUE);
	}
    }
    return (vFALSE);
}


/********************************************************************************
*
*   Tisgroups database class
*
********************************************************************************/

Tisgroups::Tisgroups (CODE4 *cB)
{
    cb = cB;
    tisgroups = NULL;
    groupnameFld = NULL;
    groupdescFld = NULL;
}

Tisgroups::~Tisgroups (void)
{
    if (tisgroups != NULL)
    {
	d4close (tisgroups);
    }
}


/********************************************************************************
*
*   Tisgroups database class, add new group to database
*
********************************************************************************/

void Tisgroups::AddGroup (vchar *name, vchar *desc)
{
    char    groupName [GROUPNAME_SZ + 1];
    char    descriptor [DESCRIPTION_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), groupName, sizeof (groupName));
    groupName [sizeof (groupName) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (desc), descriptor, sizeof (descriptor));
    descriptor [sizeof (descriptor) - 1] = 0;

    int newRec = vFALSE;
    d4top (tisgroups);
    if (d4deleted (tisgroups))
    {
	d4recall (tisgroups);
    }
    else
    {
	d4append_start (tisgroups, 0);
	newRec = vTRUE;
    }
    d4blank (tisgroups);
    f4assign (groupnameFld, groupName);
    f4assign (groupdescFld, descriptor);
    if (newRec == vTRUE)
    {
	d4append (tisgroups);
    }
    d4flush (tisgroups);
    d4unlock (tisgroups);
}


/********************************************************************************
*
*   Tisgroups database class, create sysusers.dbf file
*   If did not create sysusers.dbf, create tisgrps.dbf from it. Since multiple
*   users can belong to a group, need to eliminate dupilates
*
********************************************************************************/

int Tisgroups::CreateGroupsDb(char *dbpath, Sysusers *sysUser)
{
    if ((tisgroups = d4create (cb, dbpath, tisgroups_fields, tisgroups_tag)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif	USECB63
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }
    // if didn't create sysusers.dbf then create tisgrps.dbf from it
    if ((sysUser -> GetCreatedSysUsers()) == vFALSE)
    {
	if ((MapGroupsFlds ()) == TISPERM_ERROR)
	{
	    return (TISPERM_ERROR);
	}
	DATA4		*sysusers = sysUser -> GetSysusersDb ();
	FIELD4		*sysuserTypeFld = sysUser -> GetSysTypeFld ();
	char		groupName [GROUPNAME_SZ + 1];
	vstr		*vstrGrpName;
	vchar		*cellValue;
	vlistIterator	iterator;
	vliststr	*grpList = new vliststr;
	int		isDup;
	int		row = 0;

	for (d4top (sysusers); !d4eof (sysusers); d4skip (sysusers, 1))
	{
	    if (d4deleted (sysusers))
	    {
		continue;
	    }
	    isDup = vFALSE;
	    strncpy (groupName, f4str (sysuserTypeFld), sizeof (groupName));
	    groupName [sizeof (groupName) - 1] = 0;
	    StripWhitespace (groupName);
	    vstrGrpName = vstrCloneScribed (vcharScribeSystem (groupName));
	    iterator.Start (grpList);
	    while (iterator.Next ())
	    {
		cellValue = (vchar *)(iterator.GetValue ());
		if ((vcharCompare (cellValue, vstrGrpName)) == 0)
		{
		    isDup = vTRUE;
		    break;
		}
	    }
	    iterator.Finish ();
	    vstrDestroy (vstrGrpName);
	    if (isDup == vTRUE)
	    {
		continue;
	    }
	    else
	    {
		grpList -> SetCellValueScribed (row, 0, vcharScribeSystem (groupName));
		row++;
	    }
	    d4append_start (tisgroups, 0);
	    d4blank (tisgroups);
	    f4assign (groupnameFld, groupName);
	    d4append (tisgroups);
	    d4flush (tisgroups);
	}
	d4unlock (tisgroups);
	delete grpList;
    }
    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisgroups database class, delete group from database and delete all entries
*   in sysusers.dbf
*
********************************************************************************/

void Tisgroups::DeleteGroup (vchar *name)
{
    char    seekBuff [256];
    char    groupName [GROUPNAME_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), groupName, sizeof (groupName));
    groupName [sizeof (groupName) - 1] = 0;
    
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisgroups, seekBuff)) == r4success)
    {
	d4blank (tisgroups);
	d4delete (tisgroups);
	d4flush (tisgroups);
    }
    d4unlock (tisgroups);

    // delete all entries from sysusers.dbf
    Sysusers *sysUsers = db -> GetSysUsers ();
    sysUsers -> DeleteGroup (groupName);
    // delete all entries from privtab.dbf
    Tisprivtab *privTab = db -> GetPrivTab ();
    privTab -> DeleteGroup (groupName);
}


/********************************************************************************
*
*   Tisgroups database class, get a list of groups
*
********************************************************************************/

vliststr *Tisgroups::GetGroupsList (void)
{

    vliststr	*list = new vliststr;
    int		row = 0;
    char	groupName [GROUPNAME_SZ + 1];
    char	groupDesc [DESCRIPTION_SZ + 1];

    for (d4top (tisgroups); !d4eof (tisgroups); d4skip (tisgroups, 1))
    {
	if (d4deleted (tisgroups))
	{
	    continue;
	}
	strncpy (groupName, f4str (groupnameFld), sizeof (groupName));
	groupName [sizeof (groupName) - 1] = 0;
	StripWhitespace (groupName);
	strncpy (groupDesc, f4str (groupdescFld), sizeof (groupName));
	groupDesc [sizeof (groupDesc) - 1] = 0;
	StripWhitespace (groupDesc);
	list -> SetCellValueScribed (row, 0, vcharScribeSystem (groupName));
	list -> SetCellValueScribed (row, 1, vcharScribeSystem (groupDesc));
	row++;
    }
    return (list);
}


/********************************************************************************
*
*   Tisgroups database class, Map database fields function
*
********************************************************************************/

int Tisgroups::MapGroupsFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    if ((groupnameFld = d4field (tisgroups, GROUPNAME_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISGROUPS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (GROUPNAME_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((groupdescFld = d4field (tisgroups, DESCRIPTION_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISGROUPS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (DESCRIPTION_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisgroups database class, modify group in database
*   NOTE: only the descriptor fld need be updated, since it is the only other
*	fields besides the group name
*
********************************************************************************/

void Tisgroups::ModifyGroup (vchar *name, vchar *desc)
{
    char    seekBuff [256];
    char    groupName [GROUPNAME_SZ + 1];
    char    descriptor [DESCRIPTION_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), groupName, sizeof (groupName));
    groupName [sizeof (groupName) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (desc), descriptor, sizeof (descriptor));
    descriptor [sizeof (descriptor) - 1] = 0;
    
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisgroups, seekBuff)) == r4success)
    {
	f4assign (groupdescFld, descriptor);
	d4flush (tisgroups);
    }
    d4unlock (tisgroups);
}



/********************************************************************************
*
*   Tisgroups database class, Open function
*
********************************************************************************/

int Tisgroups::OpenGroupsDb (Sysusers *sysUser)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, TISGROUPS_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreateGroupsDb (dbpath, sysUser)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((tisgroups = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
    
    groupTag = d4tag (tisgroups, GROUPNAME_TAG);
    d4tag_select (tisgroups, groupTag);

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisgroups database class, update database function
*
********************************************************************************/

void Tisgroups::UpdateGroupsDb (DbFuncs todo, vchar *name, vchar *desc)
{
    switch (todo)
    {
	case AddToDb:
	    AddGroup (name, desc);
	    break;

	case ModTheDb:
	    ModifyGroup (name, desc);
	    break;

	case DelFromDb:
	    DeleteGroup (name);
	    break;
    }
}


/********************************************************************************
*
*   Tisusers database class, constructor and destructor
*
********************************************************************************/

Tisusers::Tisusers (CODE4 *cB)
{
    cb = cB;
    tisusers = NULL;
    usernameFld = NULL;
    userdescFld = NULL;
    userpasswdFld = NULL;
}

Tisusers::~Tisusers (void)
{
    if (tisusers != NULL)
    {
	d4close (tisusers);
    }
}


/********************************************************************************
*
*   Tisusers database class, add new user to database
*
********************************************************************************/

void Tisusers::AddUser (vchar *name, vchar *desc, vchar *encrypasswd,
			int allowAuto, int autoLogoutTime)
{
    char    userName [OPERATOR_SZ + 1];
    char    descriptor [DESCRIPTION_SZ + 1];
    char    encrypPasswd [ENCRYPTPASSWD_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), userName, sizeof (userName));
    userName [sizeof (userName) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (desc), descriptor, sizeof (descriptor));
    descriptor [sizeof (descriptor) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (encrypasswd), 
			encrypPasswd, sizeof (encrypPasswd));
    encrypPasswd [sizeof (encrypPasswd) - 1] = 0;

    int newRec = vFALSE;
    d4top (tisusers);
    if (d4deleted (tisusers))
    {
	d4recall (tisusers);
    }
    else
    {
	d4append_start (tisusers, 0);
	newRec = vTRUE;
    }
    d4blank (tisusers);
    f4assign (usernameFld, userName);
    f4assign (userdescFld, descriptor);
    f4assign (userpasswdFld, encrypPasswd);
    f4assign_char (allowautoFld, allowAuto == 1 ? 'T' : 'F');
    f4assign_int (autologoutFld, autoLogoutTime);
    if (newRec == vTRUE)
    {
	d4append (tisusers);
    }
    d4flush (tisusers);
    d4unlock (tisusers);
}


/********************************************************************************
*
*   Tisusers database class, create tisusers.dbf file
*   If did not create sysusers.dbf (existed) create tisusers.dbf from it
*
********************************************************************************/

int Tisusers::CreateTisusersDb (char *dbpath, class Sysusers *sysUser)
{
    if ((tisusers = d4create (cb, dbpath, tisusers_fields, tisusers_tag)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif	USECB63
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }
    // if did not create sysuers, create tisusers.dbf from it
    if ((sysUser -> GetCreatedSysUsers()) == vFALSE)
    {
	if ((MapUsersFlds ()) == TISPERM_ERROR)
	{
	    return (TISPERM_ERROR);
	}
	DATA4 *sysusers = sysUser -> GetSysusersDb ();
	FIELD4	*sysuserNameFld = sysUser -> GetSysNameFld ();
	for (d4top (sysusers); !d4eof (sysusers); d4skip (sysusers, 1))
	{
	    if (d4deleted (sysusers))
	    {
		continue;
	    }
	    d4append_start (tisusers, 0);
	    d4blank (tisusers);
	    f4assign (usernameFld, f4str (sysuserNameFld));
	    f4assign_char (allowautoFld, 'F');
	    f4assign_int (autologoutFld, 0);
	    d4append (tisusers);
	    d4flush (tisusers);
	}
	d4unlock (tisusers);
    }
    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisusers database class, delete existing user from database
*
********************************************************************************/

void Tisusers::DeleteUser (vchar *name)
{
    char    seekBuff [256];
    char    userName [OPERATOR_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), userName, sizeof (userName));
    userName [sizeof (userName) - 1] = 0;
    
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", OPERATOR_SZ, OPERATOR_SZ, userName);
    if ((d4seek (tisusers, seekBuff)) == r4success)
    {
	d4blank (tisusers);
	d4delete (tisusers);
	d4flush (tisusers);
    }
    d4unlock (tisusers);
}


/********************************************************************************
*
*   Tisusers database class, delete a users list
*
********************************************************************************/

void Tisusers::DeleteUsersList (vlist *list)
{
    TISUSER	*tisUser;
    int	row;
    int	rowCount = list -> GetRowCount ();
	
    // free up memory stored in list
    for (row = 0; row < rowCount; row++)
    {
	tisUser = (TISUSER *)(list -> GetCellValue (row, 0));
	vmemFree ((void *)tisUser);
    }
    // finally get rid of list
    delete list;
}


/********************************************************************************
*
*   Tisusers database class, get list of users function
*
********************************************************************************/

vlist *Tisusers::GetUsersList (void)
{
    vlist	*list = new vlist;
    int		row = 0;
    TISUSER	*tisUser;

    for (d4top (tisusers); !d4eof (tisusers); d4skip (tisusers, 1))
    {
	if (d4deleted (tisusers))
	{
	    continue;
	}
	tisUser = (TISUSER *)(vmemAllocAndClear (sizeof (TISUSER)));
	strncpy (tisUser -> username, 
		 f4str (usernameFld),
		 sizeof (tisUser -> username));
	tisUser -> username [sizeof (tisUser -> username) - 1] = 0;
	StripWhitespace (tisUser -> username);
	strncpy (tisUser -> descfld, 
		 f4str (userdescFld), 
		 sizeof (tisUser -> descfld));
	tisUser -> descfld [sizeof (tisUser -> descfld) - 1] = 0;
	StripWhitespace (tisUser -> descfld);
	strncpy (tisUser -> passwdfld, 
		 f4str (userpasswdFld), 
		 sizeof (tisUser -> passwdfld));
	tisUser -> passwdfld [sizeof (tisUser -> passwdfld) - 1] = 0;
	StripWhitespace (tisUser -> passwdfld);
	tisUser -> allowAutologout = f4true (allowautoFld);
	tisUser -> autoLogoutTime = f4int (autologoutFld);
	list -> SetCellValue (row, 0, (const void *)tisUser);
	list -> SetCellValue (row, 1, (const void *)tisUser);
	row++;
    }
    return (list);
}


/********************************************************************************
*
*   Tisusers database class, Map database fields function
*
********************************************************************************/

int Tisusers::MapUsersFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    if ((usernameFld = d4field (tisusers, NAME_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (NAME_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((userdescFld = d4field (tisusers, DESCRIPTION_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (DESCRIPTION_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((userpasswdFld = d4field (tisusers, USERPASSWD_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (USERPASSWD_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((allowautoFld = d4field (tisusers, ALLOW_AUTOLOGOUT_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (ALLOW_AUTOLOGOUT_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((autologoutFld = d4field (tisusers, AUTOLOGOUT_TIME_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISUSERS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (AUTOLOGOUT_TIME_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisusers database class, modify existing user in database
*
********************************************************************************/

void Tisusers::ModifyUser (vchar *name, vchar *desc, vchar *encrypasswd,
			   int allowAuto, int autoLogoutTime)
{
    char    seekBuff [256];
    char    userName [OPERATOR_SZ + 1];
    char    descriptor [DESCRIPTION_SZ + 1];
    char    encrypPasswd [ENCRYPTPASSWD_SZ + 1];
  
    vcharExportBounded (vcharScribeSystemExport (name), userName, sizeof (userName));
    userName [sizeof (userName) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (desc), descriptor, sizeof (descriptor));
    descriptor [sizeof (descriptor) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (encrypasswd), 
			encrypPasswd, sizeof (encrypPasswd));
    encrypPasswd [sizeof (encrypPasswd) - 1] = 0;

    // create seek buffer
    sprintf (seekBuff, "%-*.*s", OPERATOR_SZ, OPERATOR_SZ, userName);
    if ((d4seek (tisusers, seekBuff)) == r4success)
    {
	f4assign (userdescFld, descriptor);
	if ((vcharLength (encrypasswd)) != 0)
	{
	    f4assign (userpasswdFld, encrypPasswd);
	}
	f4assign_char (allowautoFld, allowAuto == 1 ? 'T' : 'F');
	f4assign_int (autologoutFld, autoLogoutTime);
    }
    d4flush (tisusers);
    d4unlock (tisusers);
}


/********************************************************************************
*
*   Tisusers database class, update database function
*
********************************************************************************/

void Tisusers::UpdateUsersDb (DbFuncs todo, vchar *name, vchar *desc,
			       vchar *encrypaswd, int allowAuto, int autoLogoutTime)
{
    switch (todo)
    {
	case AddToDb:
	    AddUser (name, desc, encrypaswd, allowAuto, autoLogoutTime);
	    break;

	case ModTheDb:
	    ModifyUser (name, desc, encrypaswd, allowAuto, autoLogoutTime);
	    break;

	case DelFromDb:
	    DeleteUser (name);
	    break;
    }
}


/********************************************************************************
*
*   Tisusers database class, Open function
*
********************************************************************************/

int Tisusers::OpenUsersDb (class Sysusers *sysUser)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, TISUSERS_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreateTisusersDb (dbpath, sysUser)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((tisusers = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
    
    nameTag = d4tag (tisusers, NAME_TAG);
    d4tag_select (tisusers, nameTag);

    return (TISPERM_SUCCESS);
}



/********************************************************************************
*
*   Tisprotgroups database class
*
********************************************************************************/

Tisprotgroups::Tisprotgroups (CODE4 *cB)
{
    cb = cB;
    tisprotgrps = NULL;
    protgrpFld = NULL;
    protgrpdescFld = NULL;
}

Tisprotgroups::~Tisprotgroups (void)
{
    if (tisprotgrps != NULL)
    {
	d4close (tisprotgrps);
    }
}


/********************************************************************************
*
*   Tisprotgroups database class, add new protection group to database
*
********************************************************************************/

void Tisprotgroups::AddProtGroup (vchar *name, vchar *desc)
{
    char    groupName [PROTECTIONGRP_SZ + 1];
    char    descriptor [DESCRIPTION_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), groupName, sizeof (groupName));
    groupName [sizeof (groupName) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (desc), descriptor, sizeof (descriptor));
    descriptor [sizeof (descriptor) - 1] = 0;

    int newRec = vFALSE;
    d4top (tisprotgrps);
    if (d4deleted (tisprotgrps))
    {
	d4recall (tisprotgrps);
    }
    else
    {
	d4append_start (tisprotgrps, 0);
	newRec = vTRUE;
    }
    d4blank (tisprotgrps);
    f4assign (protgrpFld, groupName);
    f4assign (protgrpdescFld, descriptor);
    if (newRec == vTRUE)
    {
	d4append (tisprotgrps);
    }
    d4flush (tisprotgrps);
    d4unlock (tisprotgrps);
}


/********************************************************************************
*
*   Tisprotgroups database class, create protgrps.dbf file
*   If privtab.dbf was not created (existed) then populate protgrps.dbf
*   from privtab.dbf
*   Since multiple groups can belong to a protection group eliminate duplicate
*   entries
*
********************************************************************************/

int Tisprotgroups::CreateProtGroupsDb(char *dbpath, Tisprivtab *tisPrivTab)
{
    if ((tisprotgrps = d4create (cb, dbpath, protgrps_fields, protgrps_tag)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif	USECB63
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }
    // add this as a default protection group
    if ((MapProtGroupsFlds ()) == TISPERM_ERROR)
    {
	return (TISPERM_ERROR);
    }
    vchar   untitledPrGrp [PROTECTIONGRP_SZ + 1];
    vchar   description [DESCRIPTION_SZ + 1];
    vcharCopyScribed (vcharScribeLiteral (UNTITLED_DEFAULT), untitledPrGrp);
    description[0] = 0;
    AddProtGroup (untitledPrGrp, description);

    // if did not create privtab.dbf, then create protgrps.dbf from it
    if ((tisPrivTab -> GetCreatedPrivTab ()) == vFALSE)
    {
	DATA4		*privtab = tisPrivTab -> GetPrvTabDb ();
	FIELD4		*privtabPrGrpFld = tisPrivTab -> GetProtGrpFld ();
	char		protGrpName [PROTECTIONGRP_SZ + 1];
	vlistIterator	iterator;
	vliststr	*prtGrpList = new vliststr;
	vstr		*vstrProtGrp;
	vchar		*cellValue;
	int		isDup;
	int		row = 0;

	// add default prot group to list
	prtGrpList -> SetCellValueScribed (row, 0, vcharScribe (untitledPrGrp));
	row++;

	for (d4top (privtab); !d4eof (privtab); d4skip (privtab, 1))
	{
	    if (d4deleted (privtab))
	    {
		continue;
	    }
	    isDup = vFALSE;
	    strncpy (protGrpName, f4str (privtabPrGrpFld), sizeof (protGrpName));
	    protGrpName [sizeof (protGrpName) -1] = 0;
	    StripWhitespace (protGrpName);
	    vstrProtGrp = vstrCloneScribed (vcharScribeSystem (protGrpName));
	    iterator.Start (prtGrpList);
	    while (iterator.Next ())
	    {
		cellValue = (vchar *)(iterator.GetValue ());
		if ((vcharCompare (cellValue, vstrProtGrp)) == 0)
		{
		    isDup = vTRUE;
		    break;
		}
	    }
	    iterator.Finish ();
	    vstrDestroy (vstrProtGrp);
	    if (isDup == vTRUE)
	    {
		continue;
	    }
	    else
	    {
		prtGrpList -> SetCellValueScribed (row, 0, vcharScribeSystem (protGrpName));
		row++;
	    }
	    d4append_start (tisprotgrps, 0);
	    d4blank (tisprotgrps);
	    f4assign (protgrpFld, protGrpName);
	    d4append (tisprotgrps);
	    d4flush (tisprotgrps);
	}
	delete prtGrpList;
    }
    d4unlock (tisprotgrps);
    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisprotgroups database class, delete protection group from database
*
********************************************************************************/

void Tisprotgroups::DeleteProtGroup (vchar *prgrpName)
{
    char    seekBuff [256];
    char    prGrpName [PROTECTIONGRP_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (prgrpName), 
		        prGrpName, 
			sizeof (prGrpName));
    prGrpName [sizeof (prGrpName) - 1] = 0;
    
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", PROTECTIONGRP_SZ, PROTECTIONGRP_SZ, prGrpName);
    if ((d4seek (tisprotgrps, seekBuff)) == r4success)
    {
	d4blank (tisprotgrps);
	d4delete (tisprotgrps);
	d4flush (tisprotgrps);
    }
    d4unlock (tisprotgrps);
}


/********************************************************************************
*
*   Tisprotgroups database class, get a list of protection groups
*
********************************************************************************/

vliststr *Tisprotgroups::GetProtGroupsList (void)
{

    vliststr	*list = NULL;
    int		row = 0;
    char	groupName [PROTECTIONGRP_SZ + 1];
    char	groupDesc [DESCRIPTION_SZ + 1];

    for (d4top (tisprotgrps); !d4eof (tisprotgrps); d4skip (tisprotgrps, 1))
    {
	if (d4deleted (tisprotgrps))
	{
	    continue;
	}
	if (list == NULL)
	{
	    list = new vliststr;
	}
	strncpy (groupName, f4str (protgrpFld), sizeof (groupName));
	groupName [sizeof (groupName) - 1] = 0;
	StripWhitespace (groupName);
	strncpy (groupDesc, f4str (protgrpdescFld), sizeof (groupDesc));
	groupDesc [sizeof (groupDesc) - 1] = 0;
	StripWhitespace (groupDesc);
	list -> SetCellValueScribed (row, 0, vcharScribeSystem (groupName));
	list -> SetCellValueScribed (row, 1, vcharScribeSystem (groupDesc));
	row++;
    }
    return (list);
}


/********************************************************************************
*
*   Tisprotgroups database class, Map database fields function
*
********************************************************************************/

int Tisprotgroups::MapProtGroupsFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    if ((protgrpFld = d4field (tisprotgrps, PROT_GROUP_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISPROTGRPS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (PROT_GROUP_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((protgrpdescFld = d4field (tisprotgrps, DESCRIPTION_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (TISPROTGRPS_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (DESCRIPTION_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisprotgroups database class, modify protection group in database
*   NOTE: only the descriptor fld need be updated, since it is the only other
*	field besides the protection group name
*
********************************************************************************/

void Tisprotgroups::ModifyProtGroup (vchar *name, vchar *desc)
{
    char    seekBuff [256];
    char    groupName [PROTECTIONGRP_SZ + 1];
    char    descriptor [DESCRIPTION_SZ + 1];

    vcharExportBounded (vcharScribeSystemExport (name), groupName, sizeof (groupName));
    groupName [sizeof (groupName) - 1] = 0;
    vcharExportBounded (vcharScribeSystemExport (desc), descriptor, sizeof (descriptor));
    descriptor [sizeof (descriptor) - 1] = 0;
    
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", PROTECTIONGRP_SZ, PROTECTIONGRP_SZ, groupName);
    if ((d4seek (tisprotgrps, seekBuff)) == r4success)
    {
	f4assign (protgrpdescFld, descriptor);
	d4flush (tisprotgrps);
    }
    d4unlock (tisprotgrps);
}



/********************************************************************************
*
*   Tisprotgroups database class, Open function
*
********************************************************************************/

int Tisprotgroups::OpenProtGroupsDb (Tisprivtab *tisPrivTab)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, TISPROTGRPS_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreateProtGroupsDb (dbpath, tisPrivTab)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((tisprotgrps = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
    
    protGrpTag = d4tag (tisprotgrps, PROTGRP_TAG);
    d4tag_select (tisprotgrps, protGrpTag);

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisprotgroups database class
*	does the protectin group have points assigned to it
*
********************************************************************************/

int Tisprotgroups::ProtGrpHasPts (char *protGrpName)
{
    int	    result = vFALSE;
    int	    pvmapOpened = vTRUE;
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    DATA4   *pvmap;
    TAG4    *nameTag;
    FIELD4  *pvmapProtGrpFld;
    vstr    *str1;
    vstr    *str2;

    sprintf (unresolvedPath, "%spvmap.dbf", TIS_DATABASE_PATH);
    resolveLinks (unresolvedPath, dbpath);

    if ((pvmap = d4open (cb, dbpath)) == NULL)
    {
	str1 = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr,
			  str1, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr,
			  str1, cb -> error_code);
#endif	USECB63
	vstrDestroy (str1);
	result = TISPERM_ERROR;
	pvmapOpened = vFALSE;
    }
    else if ((pvmapProtGrpFld = d4field (pvmap, "PRIVGROUP")) == NULL)
    {
	str1 = vstrCloneScribed (vcharScribeLiteral ("pvmap.dbf"));
	str2 = vstrCloneScribed (vcharScribeLiteral ("PRIVGROUP"));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr,
			  str1, str2);
	vstrDestroy (str1);
	vstrDestroy (str2);
	result = TISPERM_ERROR;
    }
    else if ((nameTag = d4tag (pvmap, "NAME")) == NULL)
    {
	str1 = vstrCloneScribed (vcharScribeLiteral ("pvmap.dbf"));
	str2 = vstrCloneScribed (vcharScribeLiteral ("NAME"));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr,
			  str1, str2);
	vstrDestroy (str1);
	vstrDestroy (str2);
	result = TISPERM_ERROR;
    }
    else
    {
	char	    protGrpInDb [PROTECTIONGRP_SZ + 1];
	TisPermApp  *app = db -> GetTisPermApp ();
	vresource   msgRes = app -> GetMessagesRes ();
	vresource   tisRes = app -> GetTisGalaxyRes ();
	vresource   waitRes = vresourceGet (tisRes, tisgalaxy_waitDialog);

 	str1 = vresourceGetString (msgRes, tisperm_SearchingDbMsg);
	tisWaitDialog *waitDialog = new tisWaitDialog (waitRes);
	waitDialog -> SetMsgText (str1);
	waitDialog -> OpenWaitDialog ();
	vstrDestroy (str1);

	d4tag_select (pvmap, nameTag);
	for (d4top (pvmap); !d4eof (pvmap); d4skip (pvmap, 1))
	{
	    if (d4deleted (pvmap))
	    {
		continue;
	    }
	    strncpy (protGrpInDb, f4str (pvmapProtGrpFld), sizeof (protGrpInDb));
	    protGrpInDb [sizeof (protGrpInDb) - 1] = 0;
	    StripWhitespace (protGrpInDb);
	    if ((strcmp (protGrpName, protGrpInDb)) == 0)
	    {
		result = vTRUE;
		break;
	    }
	}
	delete waitDialog;
    }

    if (pvmapOpened == vTRUE)
    {
	d4close (pvmap);
    }
    return (result);
}


/********************************************************************************
*
*   Tisprotgroups database class, update database function
*
********************************************************************************/

void Tisprotgroups::UpdateProtGroupsDb (DbFuncs todo, vchar *name, vchar *desc)
{
    switch (todo)
    {
	case AddToDb:
	    AddProtGroup (name, desc);
	    break;

	case ModTheDb:
	    ModifyProtGroup (name, desc);
	    break;

	case DelFromDb:
	    DeleteProtGroup (name);
	    break;
    }
}


/********************************************************************************
*
*   Field protectection  database class, constructor and destructor
*
********************************************************************************/

Tisfldprott::Tisfldprott (CODE4 *cB)
{
    cb = cB;
    tisfldprott = NULL;
    typefieldTag = NULL;
    pointtypeTag = NULL;
    pointtypeFld = NULL;
    fieldnameFld = NULL;
    privlevelFld = NULL;
}

Tisfldprott::~Tisfldprott (void)
{
    if (tisfldprott != NULL)
    {
	d4close (tisfldprott);
    }
}


/********************************************************************************
*
*   Tisfldprott database class, clear privileges function
*
********************************************************************************/

void Tisfldprott::ClearPrivLevel (char *pType, char *fieldName)
{
    char    seekString[POINTTYPE_SZ + FDPROTFLD_SZ + 1];
    
    sprintf (seekString, "%-*.*s%-*.*s", POINTTYPE_SZ, POINTTYPE_SZ, pType,
					 FDPROTFLD_SZ, FDPROTFLD_SZ, fieldName);
    if ((d4seek (tisfldprott, seekString)) == r4success)
    {
	d4blank (tisfldprott);
	d4delete (tisfldprott);
	d4flush (tisfldprott);
	d4unlock (tisfldprott);
    }
}


/********************************************************************************
*
*   Tisfldprott database class, Create function
*
********************************************************************************/

int Tisfldprott::CreateFldProttDb (char *dbpath)
{
    if ((tisfldprott = d4create (cb, dbpath, fldprottFieldInfo, fldprottTagInfo)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif	USECB63
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisfldprott database class, return a protection level
*
********************************************************************************/

int Tisfldprott::GetPrivLevel (char *pType, char *fieldName, int *pLevel)
{
    char    seekString[POINTTYPE_SZ + FDPROTFLD_SZ + 1];
    int	    returnVal = vFALSE;
    
    sprintf (seekString, "%-*.*s%-*.*s", POINTTYPE_SZ, POINTTYPE_SZ, pType,
					 FDPROTFLD_SZ, FDPROTFLD_SZ, fieldName);
    if ((d4seek (tisfldprott, seekString)) == r4success)
    {
	*pLevel = f4int (privlevelFld);
	returnVal = vTRUE;
    }
    return (returnVal);
}


/********************************************************************************
*
*   Tisfldprott database class, map fields function
*
********************************************************************************/

int Tisfldprott::MapFldProttFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    // map fields
    if ((pointtypeFld = d4field (tisfldprott, POINTTYPE_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (FLDPROTT_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (POINTTYPE_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((fieldnameFld = d4field (tisfldprott, FIELDNAME_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (FLDPROTT_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (FIELDNAME_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((privlevelFld = d4field (tisfldprott, PRIVLEVEL_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (FLDPROTT_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (PRIVLEVEL_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    // get the tags
    if ((typefieldTag = d4tag (tisfldprott, TYPEFIELD_TAG)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (FLDPROTT_DB));
 	vstrFldName = vstrCloneScribed (vcharScribeSystem (TYPEFIELD_TAG));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((pointtypeTag = d4tag (tisfldprott, POINTTYPE_TAG)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (FLDPROTT_DB));
 	vstrFldName = vstrCloneScribed (vcharScribeSystem (POINTTYPE_TAG));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisfldprott database class, Open function
*
********************************************************************************/

int Tisfldprott::OpenFldProttDb (void)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, FLDPROTT_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreateFldProttDb (dbpath)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((tisfldprott = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
    
    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisfldprott database class, update function
*
********************************************************************************/

void Tisfldprott::UpdatePrivLevel (char *pointType, char *fieldName, int privLevel)
{
    char    seekString[256];
    
    sprintf (seekString, "%-*.*s%-*.*s", POINTTYPE_SZ, POINTTYPE_SZ, pointType,
					 FDPROTFLD_SZ, FDPROTFLD_SZ, fieldName);

    if ((d4seek (tisfldprott, seekString)) == r4success)
    {
 	f4assign (pointtypeFld, pointType);
	f4assign (fieldnameFld, fieldName);
	f4assign_int (privlevelFld, privLevel);
    }
    else
    {
	int newRec = vFALSE;
	d4top (tisfldprott);
	if (d4deleted (tisfldprott))
	{
	    d4recall (tisfldprott);
	}
	else
	{
	    d4append_start (tisfldprott, 0);
	    newRec = vTRUE;
	}
	d4blank (tisfldprott);
	f4assign (pointtypeFld, pointType);
	f4assign (fieldnameFld, fieldName);
	f4assign_int (privlevelFld, privLevel);
	if (newRec == vTRUE)
	{
	    d4append (tisfldprott);
	}
    }
    d4flush (tisfldprott);
    d4unlock (tisfldprott);
}


/********************************************************************************
*
*   Privilege database class, constructor and destructor
*
********************************************************************************/

Tisprivtab::Tisprivtab (CODE4 *cB)
{
    cb = cB;
    tisprivtab = NULL;
    operatoridTag = NULL;
    opergroupTag = NULL;
    operatoridFld = NULL;
    prot_groupFld = NULL;
    priv_levelFld = NULL;
    createdPrivTab = vFALSE;
}

Tisprivtab::~Tisprivtab (void)
{
    if (tisprivtab != NULL)
    {
	d4close (tisprivtab);
    }
}


/********************************************************************************
*
*   Tisprivtab database class, Create function
*
********************************************************************************/

int Tisprivtab::CreatePrivTabDb (char *dbpath)
{
    if ((tisprivtab = d4create (cb, dbpath, privtabFieldInfo, privtabTagInfo)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif	USECB63
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }
    createdPrivTab = vTRUE;

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisprivtab database class, delete group from privtab.dbf
*   NOTE: a group can belong to more than 1 point protection group
*
********************************************************************************/

void Tisprivtab::DeleteGroup (char *groupName)
{
    char	seekBuff [GROUPNAME_SZ + 1];
    int		status;
  
    // select tag for groupName only
    SetOperatorIdTag ();
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((status = d4seek (tisprivtab, seekBuff)) == r4success)
    {
	do 
	{
	    d4blank (tisprivtab);
	    d4delete (tisprivtab);
	    d4flush (tisprivtab);
	}
	while ((GetNextGroup (seekBuff)) == r4success);
    }
    d4unlock (tisprivtab);
}


/********************************************************************************
*
*   Tisprivtab database class
*	position us at the next group that matches the seekBuff
*
********************************************************************************/

int Tisprivtab::GetNextGroup (char *seekBuff)
{
    int status;
#if defined(USECB63) || defined(WIN32)
    status = d4seekNext (tisprivtab, seekBuff);
#else
    char	groupInTable [GROUPNAME_SZ + 1];

    status = r4success;
    while (1)
    {
	if ((status = d4skip (tisprivtab, 1)) == r4success)
	{
	    if (d4deleted (tisprivtab))
	    {
		continue;
	    }
	    strncpy (groupInTable, f4str (operatoridFld), 
						sizeof (groupInTable));
	    if ((strcmp (seekBuff, groupInTable)) == 0)
	    {
		break;
	    }
	    status = r4after;
	    break;
	}
	else
	{
	    break;
	}
    }
#endif USECB63
    return (status);
}


/********************************************************************************
*
*   Tisprivtab database class, get a list of groups and their access privileges
*   to the protection group protGroup
*
********************************************************************************/

vliststr *Tisprivtab::GetProtGroupList (char *protGroup)
{
    vliststr	*list = NULL;
    int		row = 0;
    char	groupName [GROUPNAME_SZ + 1];
    char	privLevel [PRIV_LEVEL_SZ + 1];
    char	protGroupInDb [PROTECTIONGRP_SZ + 1];

    // select the operator/group as the tag
    d4tag_select (tisprivtab, operatoridTag);
    // loop through entire database
    for (d4top (tisprivtab); !d4eof (tisprivtab); d4skip (tisprivtab, 1))
    {
	if (d4deleted (tisprivtab))
	{
	    continue;
	}
	strncpy (protGroupInDb, f4str (prot_groupFld), sizeof (protGroupInDb));
	protGroupInDb [sizeof (protGroupInDb) - 1] = 0;
	StripWhitespace (protGroupInDb);
	// is this the protection group we are looking for
	if ((strncmp (protGroup, protGroupInDb, PROTECTIONGRP_SZ)) == 0)
	{
	    strncpy (groupName, f4str (operatoridFld), sizeof (groupName));
	    groupName [sizeof (groupName) - 1] = 0;
	    StripWhitespace (groupName);
	    strncpy (privLevel, f4str (priv_levelFld), sizeof (privLevel));
	    privLevel [sizeof (privLevel) - 1] = 0;
	    StripWhitespace (privLevel);
	    if (list == NULL)
	    {
		list = new vliststr;
		list -> SetColumnCount (2);
	    }
	    list -> SetCellValueScribed (row, 0, vcharScribeSystem (groupName));
	    list -> SetCellValueScribed (row, 1, vcharScribeSystem (privLevel));
	    row++;
	}
    }
    return (list);
}


/********************************************************************************
*
*   Tisprivtab database class, get a list of protection group permissions
*
********************************************************************************/

vliststr *Tisprivtab::GetPrGrpPermissions (char *groupName)
{
    char	seekBuff [256];
    vliststr	*list = NULL;

    // select proper tag to seek just on groupName
    SetOperatorIdTag ();
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisprivtab, seekBuff)) == r4success)
    {
	char	protGroup [PROTECTIONGRP_SZ + 1];
	char	privLevel [PRIV_LEVEL_SZ + 1];
	int	row = 0;
	do
	{
	    if (d4deleted (tisprivtab))
	    {
		continue;
	    }
	    strncpy (protGroup, f4str (prot_groupFld), sizeof (protGroup));
	    protGroup [sizeof (protGroup ) - 1] = 0;
	    if (list == NULL)
	    {
		list = new vliststr;
	    }
	    StripWhitespace (protGroup);
	    list -> SetCellValueScribed (row, 0, vcharScribeSystem (protGroup));
	    strncpy (privLevel, f4str (priv_levelFld), sizeof (privLevel));
	    privLevel [sizeof (privLevel) - 1] = 0;
	    StripWhitespace (privLevel);
	    list -> SetCellValueScribed (row, 1, vcharScribeSystem (privLevel));
	    row++;
	}
	while ((GetNextGroup (seekBuff)) == r4success);
    }
    return (list);
}


/********************************************************************************
*
*   Tisprivtab database class, is groupName in a point protection group 
*
********************************************************************************/

int Tisprivtab::GroupInProtectionGroup (char *groupName)
{
    char    seekBuff [256];
    char    groupNameInDb [GROUPNAME_SZ + 1];
    int	    result = vFALSE;
  
    // select tag for groupName only
    SetOperatorIdTag ();
    // create seek buffer
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisprivtab, seekBuff)) == r4success)
    {
	do 
	{
	    strncpy (groupNameInDb, f4str (operatoridFld), sizeof (groupNameInDb));
	    groupNameInDb [sizeof (groupNameInDb) - 1] = 0;
	    StripWhitespace (groupNameInDb);
	    if ((strcmp (groupNameInDb, groupName)) == 0)
	    {
		result = vTRUE;
		break;
	    }
	}
	while ((GetNextGroup (seekBuff)) == r4success);
    }
    return (result);
}


/********************************************************************************
*
*   Tisprivtab database class, map fields function
*
********************************************************************************/

int Tisprivtab::MapPrivTabFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    // map fields
    if ((operatoridFld = d4field (tisprivtab, OPERATORID_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (PRIVTAB_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (OPERATORID_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((prot_groupFld = d4field (tisprivtab, PROT_GROUP_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (PRIVTAB_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (PROT_GROUP_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((priv_levelFld = d4field (tisprivtab, PRIV_LEVEL_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (PRIVTAB_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (PRIV_LEVEL_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    // get the tags
    if ((operatoridTag = d4tag (tisprivtab, OPERATORID_TAG)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (PRIVTAB_DB));
 	vstrFldName = vstrCloneScribed (vcharScribeSystem (OPERATORID_TAG));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((opergroupTag = d4tag (tisprivtab, OPERGROUP_TAG)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (PRIVTAB_DB));
 	vstrFldName = vstrCloneScribed (vcharScribeSystem (OPERGROUP_TAG));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisprivtab database class, Open function
*
********************************************************************************/

int Tisprivtab::OpenPrivTabDb (void)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, PRIVTAB_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreatePrivTabDb (dbpath)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((tisprivtab = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
    
    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisprivtab database class
*	return true if protection group has groups assigned to it
*
********************************************************************************/

int Tisprivtab::PrtGrpHasGroups (char *protGrpName)
{
    int	    result = vFALSE;
    char    prtGrpInDb [PROTECTIONGRP_SZ + 1];

    d4tag_select (tisprivtab, operatoridTag);

    for (d4top (tisprivtab); !d4eof (tisprivtab); d4skip (tisprivtab, 1))
    {
	if (d4deleted (tisprivtab))
	{
	    continue;
	}
	strncpy (prtGrpInDb, f4str (prot_groupFld), sizeof (prtGrpInDb));
	prtGrpInDb [sizeof (prtGrpInDb) - 1] = 0;
	StripWhitespace (prtGrpInDb);
	if ((strncmp (prtGrpInDb, protGrpName, PROTECTIONGRP_SZ)) == 0)
	{
	    result = vTRUE;
	    break;
	}
    }

    return (result);
}


/********************************************************************************
*
*   Tisprivtab database class, Update function, updates point protection group
*   protGroup with groups and privileges in list
*
********************************************************************************/

void Tisprivtab::UpdatePrtGrp (vliststr *list, char *protGroup)
{
    vchar	    *groupNameStr;
    vchar	    *privLevelStr;
    char	    seekBuff [256];
    char	    groupName [GROUPNAME_SZ + 1];
    char	    protFld [PROTECTIONGRP_SZ + 1];
    char	    privLevel [PRIV_LEVEL_SZ + 1];

    // select tag of Groupname + protection group
    SetOperGroupTag ();

    int row;
    int rowCount = list -> GetRowCount ();
    for (row = 0; row < rowCount; row++)
    {
	groupNameStr = (vchar *)(list -> GetCellValue (row, 0));
	privLevelStr = (vchar *)(list -> GetCellValue (row, 1));
	vcharExportBounded (vcharScribeSystemExport (groupNameStr),
			    groupName,
			    sizeof (groupName));
	groupName [sizeof (groupName) - 1] = 0;
	vcharExportBounded (vcharScribeSystemExport (privLevelStr),
			    privLevel,
			    sizeof (privLevel));
	privLevel [sizeof (privLevel) - 1] = 0;
	sprintf (seekBuff, "%-*.*s%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, 
					   groupName,
					   PROTECTIONGRP_SZ, PROTECTIONGRP_SZ, 
					   protGroup);
	if ((d4seek (tisprivtab, seekBuff)) == r4success)
	{
	    f4assign (priv_levelFld, privLevel);
	    d4flush (tisprivtab);
	    d4unlock (tisprivtab);
	}
	else
	{
	    int newRec = vFALSE;
	    d4top (tisprivtab);
	    if (d4deleted (tisprivtab))
	    {
		d4recall (tisprivtab);
	    }
	    else
	    {
		d4append_start (tisprivtab, 0);
		newRec = vTRUE;
	    }
	    d4blank (tisprivtab);
	    f4assign (operatoridFld, groupName);
	    f4assign (prot_groupFld, protGroup);
	    f4assign (priv_levelFld, privLevel);
	    if (newRec == vTRUE)
	    {
		d4append (tisprivtab);
	    }
	    d4flush (tisprivtab);
	    d4unlock (tisprivtab);
	}
    }

    // delete those group/protGroup records not in list
    // set tag to select on just groupname
    vlistIterator   iterator;
    int		    col;
    int		    foundIt;
    char	    groupNameInList [GROUPNAME_SZ + 1];

    SetOperatorIdTag ();

    for (d4top (tisprivtab); !d4eof (tisprivtab); d4skip (tisprivtab, 1))
    {
	if (d4deleted (tisprivtab))
	{
	    continue;
	}
	strncpy (protFld, f4str (prot_groupFld), sizeof (protFld));
	protFld [sizeof (protFld) - 1] = 0;
	StripWhitespace (protFld);
	// wrong protection group
	if ((strcmp (protFld, protGroup)) != 0)
	{
	    continue;
	}
	// found our baby, get the group name
	strncpy (groupName, f4str (operatoridFld), sizeof (groupName));
	groupName [sizeof (groupName) - 1] = 0;
	StripWhitespace (groupName);
	// now find this combo of groupname-protGroup in our list
	// if not there then delete this record
	foundIt = vFALSE;
	iterator.Start (list);
	while (iterator.Next ())
	{
	    col = iterator.GetColumn ();
	    // only interested in col 0, the groupname
	    if (col != 0)
	    {
		continue;
	    }
	    groupNameStr = (vchar *)(iterator.GetValue ());
	    vcharExportBounded (vcharScribeSystemExport (groupNameStr),
				groupNameInList,
				sizeof (groupNameInList));
	    groupNameInList [sizeof (groupNameInList) - 1] = 0;
	    if ((strcmp (groupName, groupNameInList)) == 0)
	    {
		foundIt = vTRUE;
		break;
	    }
	}
	iterator.Finish ();
	if (foundIt == vFALSE)
	{
	    d4blank (tisprivtab);
	    d4delete (tisprivtab);
	    d4flush (tisprivtab);
	    d4unlock (tisprivtab);
	}
    }

    d4flush (tisprivtab);
    d4unlock (tisprivtab);
    delete list;
}


/********************************************************************************
*
*   Tisprivtab database class, Update function, updates point protection group
*   privileges for groupName
*
********************************************************************************/

void Tisprivtab::UpdatePrtGrpPrivs (vliststr *list, char *groupName)
{
    char	    seekBuff [256];
    vlistIterator   iterator;
    vchar	    *protGroupStr;
    vchar	    *privLevelStr;
    char	    protGroup [PROTECTIONGRP_SZ + 1];
    char	    privLevel [PRIV_LEVEL_SZ + 1];
    int		    row, col;

    // select tag of Groupname + protection group
    SetOperGroupTag ();

    iterator.Start (list);
    while (iterator.Next ())
    {
	row = iterator.GetRow ();
	col = iterator.GetColumn ();
	if (col == 1)
	{
	    continue;
	}
	protGroupStr = (vchar *)(list -> GetCellValue (row, 0));
	privLevelStr = (vchar *)(list -> GetCellValue (row, 1));
	vcharExportBounded (vcharScribeSystemExport (protGroupStr),
			    protGroup,
			    sizeof (protGroup));
	protGroup [sizeof (protGroup) - 1] = 0;
	vcharExportBounded (vcharScribeSystemExport (privLevelStr),
			    privLevel,
			    sizeof (privLevel));
	privLevel [sizeof (privLevel) - 1] = 0;
	sprintf (seekBuff, "%-*.*s%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, 
					   groupName,
					   PROTECTIONGRP_SZ, PROTECTIONGRP_SZ, 
					   protGroup);
	if ((d4seek (tisprivtab, seekBuff)) == r4success)
	{
	    f4assign (priv_levelFld, privLevel);
	    d4flush (tisprivtab);
	    d4unlock (tisprivtab);
	}
	else
	{
	    int newRec = vFALSE;
	    d4top (tisprivtab);
	    if (d4deleted (tisprivtab))
	    {
		d4recall (tisprivtab);
	    }
	    else
	    {
		d4append_start (tisprivtab, 0);
		newRec = vTRUE;
	    }
	    d4blank (tisprivtab);
	    f4assign (operatoridFld, groupName);
	    f4assign (prot_groupFld, protGroup);
	    f4assign (priv_levelFld, privLevel);
	    if (newRec == vTRUE)
	    {
		d4append (tisprivtab);
	    }
	    d4flush (tisprivtab);
	    d4unlock (tisprivtab);
	}
    }
    iterator.Finish ();

    // delete those protection group privileges not in list
    // set tag to select on just groupname
    SetOperatorIdTag ();
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisprivtab, seekBuff)) == r4success)
    {
	int	inDb;
	char	protGroupInDb [PROTECTIONGRP_SZ + 1];
	do
	{
	    strncpy (protGroupInDb, f4str (prot_groupFld), sizeof (protGroupInDb));
	    protGroupInDb [sizeof (protGroupInDb) - 1] = 0;
	    StripWhitespace (protGroupInDb);
	    inDb = vFALSE;
	    iterator.Start (list);
	    while (iterator.Next ())
	    {
		col = iterator.GetColumn ();
		if (col == 1)
		{
		    continue;
		}
		protGroupStr = (vchar *)(iterator.GetValue ());
		vcharExportBounded (vcharScribeSystemExport (protGroupStr),
				    protGroup,
				    sizeof (protGroup));
		protGroup [sizeof (protGroup) - 1] = 0;
		if ((strcmp (protGroupInDb, protGroup)) == 0)
		{
		    inDb = vTRUE;
		    break;
		}
	    }
	    iterator.Finish ();
	    if (inDb == vFALSE)
	    {
		d4blank (tisprivtab);
		d4delete (tisprivtab);
	    }
	}
	while ((GetNextGroup (seekBuff)) == r4success);
    }
    d4flush (tisprivtab);
    d4unlock (tisprivtab);
    delete list;
}


/********************************************************************************
*
*   Tisutilacc database class, constructor and destructor
*
********************************************************************************/

Tisutilacc::Tisutilacc (CODE4 *cB)
{
    cb = cB;
    tisutilacc = NULL;
    groupnameTag = NULL;
    groupprivTag = NULL;
    groupnameFld = NULL;
    privilegeFld = NULL;
    priv_levelFld = NULL;
}

Tisutilacc::~Tisutilacc (void)
{
    if (tisutilacc != NULL)
    {
	d4close (tisutilacc);
    }
}


/********************************************************************************
*
*   Tisutilacc database class, Create function
*
********************************************************************************/

int Tisutilacc::CreateUtilAccDb (char *dbpath)
{
    if ((tisutilacc = d4create (cb, dbpath, utilaccFieldInfo, utilaccTagInfo)) == NULL)
    {
	vstr *vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> errorCode);
#else
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbCreateErr,
			   vstrPath, cb -> error_code);
#endif
	vstrDestroy (vstrPath);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisutilacc database class
*	position us at the next group that matches the seekBuff
*
********************************************************************************/

int Tisutilacc::GetNextGroup (char *seekBuff)
{
    int status;
#if defined(USECB63) || defined(WIN32)
    status = d4seekNext (tisutilacc, seekBuff);
#else
    char	groupInTable [GROUPNAME_SZ + 1];

    status = r4success;
    while (1)
    {
	if ((status = d4skip (tisutilacc, 1)) == r4success)
	{
	    if (d4deleted (tisutilacc))
	    {
		continue;
	    }
	    strncpy (groupInTable, f4str (groupnameFld), 
						sizeof (groupInTable));
	    if ((strcmp (seekBuff, groupInTable)) == 0)
	    {
		break;
	    }
	    status = r4after;
	    break;
	}
	else
	{
	    break;
	}
    }
#endif USECB63
    return (status);
}


/********************************************************************************
*
*   Tisutilacc database class, get a list of utility permissions
*
********************************************************************************/

vliststr *Tisutilacc::GetUtilPermissions (char *groupName)
{
    char	seekBuff [256];
    vliststr	*list = NULL;

    // select proper tag to seek just on groupName
    SetGroupNameTag ();
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisutilacc, seekBuff)) == r4success)
    {
	char	utilPerm [UTILACCPRIV_SZ + 1];
	int	row = 0;
	list = new vliststr;
	do
	{
	    strncpy (utilPerm, f4str (privilegeFld), sizeof (utilPerm));
	    utilPerm [sizeof (utilPerm ) - 1] = 0;
	    StripWhitespace (utilPerm);
	    list -> SetCellValueScribed (row, 0, vcharScribeSystem (utilPerm));
	    row++;
	}
	while ((GetNextGroup(seekBuff)) == r4success);
    }
    return (list);
}


/********************************************************************************
*
*   Tisutilacc database class, map fields function
*
********************************************************************************/

int Tisutilacc::MapUtilAccFlds (void)
{
    vstr    *vstrDb;
    vstr    *vstrFldName;

    // map fields
    if ((groupnameFld = d4field (tisutilacc, GROUPNAME_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (UTILACC_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (GROUPNAME_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((privilegeFld = d4field (tisutilacc, PRIVILEGE_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (UTILACC_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (PRIVILEGE_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((priv_levelFld = d4field (tisutilacc, PRIV_LEVEL_FLD)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (UTILACC_DB));
	vstrFldName = vstrCloneScribed (vcharScribeSystem (PRIV_LEVEL_FLD));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbFieldErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    // get the tags
    if ((groupnameTag = d4tag (tisutilacc, GROUPNAME_TAG)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (UTILACC_DB));
 	vstrFldName = vstrCloneScribed (vcharScribeSystem (GROUPNAME_TAG));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }
    if ((groupprivTag = d4tag (tisutilacc, GROUPPRIV_TAG)) == NULL)
    {
	vstrDb = vstrCloneScribed (vcharScribeSystem (UTILACC_DB));
 	vstrFldName = vstrCloneScribed (vcharScribeSystem (GROUPPRIV_TAG));
	tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbTagErr, 
			   vstrDb, vstrFldName);
	vstrDestroy (vstrDb);
	vstrDestroy (vstrFldName);
	return (TISPERM_ERROR);
    }

    return (TISPERM_SUCCESS);
}

/********************************************************************************
*
*   Tisutilacc database class, open database function
********************************************************************************/

int Tisutilacc::OpenUtilAccDb (void)
{
    char    unresolvedPath [TISPERM_MAX_DATABASE_PATH];
    char    dbpath [TISPERM_MAX_DATABASE_PATH];
    vstr    *vstrPath;

    sprintf (unresolvedPath, "%s%s", TIS_DATABASE_PATH, UTILACC_DB);
    resolveLinks (unresolvedPath, dbpath);

#ifdef WIN32
    if ((_access (dbpath, 6)) != 0)
#else
    if ((access (dbpath, R_OK | W_OK | F_OK)) != 0)
#endif WIN32
    {
	if (errno != ENOENT)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_AccessErr, 
			       vstrPath, errno);
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
	else
	{
	    if ( (CreateUtilAccDb (dbpath)) == TISPERM_ERROR)
	    {
		return (TISPERM_ERROR);
	    }
	}
    }
    else
    {
	if ((tisutilacc = d4open (cb, dbpath)) == NULL)
	{
	    vstrPath = vstrCloneScribed (vcharScribeSystem (dbpath));
#if defined(USECB63) || defined(WIN32)
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> errorCode);
#else
	    tisPermMsgCenter (NULL, TISPERM_INFO_NOTICE, tisperm_DbOpenErr, 
			       vstrPath, cb -> error_code);
#endif	USECB63
	    vstrDestroy (vstrPath);
	    return (TISPERM_ERROR);
	}
    }
    
    return (TISPERM_SUCCESS);
}


/********************************************************************************
*
*   Tisutilacc database class, Update function, handles utility privileges
*   for groupName
*
********************************************************************************/

void Tisutilacc::UpdateUtilAccPrivileges (vliststr *selectedList, char *groupName)
{
    char	    seekBuff [256];
    char	    utilOption [UTILACCPRIV_SZ + 1];
    vlistIterator   iterator;
    vchar	    *listValue;

    // select tag of Groupname + privilege
    SetGroupPrivTag ();

    iterator.Start (selectedList);
    while (iterator.Next ())
    {
	listValue = (vchar *)(iterator.GetValue ());
	vcharExportBounded (vcharScribeSystemExport (listValue),
			    utilOption,
			    sizeof (utilOption));
	utilOption [sizeof (utilOption) - 1] = 0;
	sprintf (seekBuff, "%-*.*s%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName,
					  UTILACCPRIV_SZ, UTILACCPRIV_SZ, utilOption);
	if ((d4seek (tisutilacc, seekBuff)) != r4success)
	{
	    int newRec = vFALSE;
	    d4top (tisutilacc);
	    if (d4deleted (tisutilacc))
	    {
		d4recall (tisutilacc);
	    }
	    else
	    {
		d4append_start (tisutilacc, 0);
		newRec = vTRUE;
	    }
	    d4blank (tisutilacc);
	    f4assign (groupnameFld, groupName);
	    f4assign (privilegeFld, utilOption);
	    // you either access or not, set priv to MAX for OperPrivTrak funcs
	    f4assign (priv_levelFld, MAX_PRIVILEGE_LEVEL);
	    if (newRec == vTRUE)
	    {
		d4append (tisutilacc);
	    }
	    d4flush (tisutilacc);
	    d4unlock (tisutilacc);
	}
    }
    iterator.Finish ();

    // delete those utils not in list
    // set tag to select on just groupname
    SetGroupNameTag ();
    sprintf (seekBuff, "%-*.*s", GROUPNAME_SZ, GROUPNAME_SZ, groupName);
    if ((d4seek (tisutilacc, seekBuff)) == r4success)
    {
	int	inDb;
	char	utilOptionInDb [UTILACCPRIV_SZ + 1];
	do
	{
	    strncpy (utilOptionInDb, f4str (privilegeFld), sizeof (utilOptionInDb));
	    utilOptionInDb [sizeof (utilOptionInDb) - 1] = 0;
	    inDb = vFALSE;
	    StripWhitespace (utilOptionInDb);
	    iterator.Start (selectedList);
	    while (iterator.Next ())
	    {
		listValue = (vchar *)(iterator.GetValue ());
		vcharExportBounded (vcharScribeSystemExport (listValue),
				    utilOption,
				    sizeof (utilOption));
		utilOption [sizeof (utilOption) - 1] = 0;
		if ((strcmp (utilOptionInDb, utilOption)) == 0)
		{
		    inDb = vTRUE;
		    break;
		}
	    }
	    iterator.Finish ();
	    if (inDb == vFALSE)
	    {
		d4blank (tisutilacc);
		d4delete (tisutilacc);
	    }
	}
	while ((GetNextGroup(seekBuff)) == r4success);
    }
    d4flush (tisutilacc);
    d4unlock (tisutilacc);
}


static int compare_fields (const void *fld1, const void *fld2)
{
    TISPERM_FIELD_DEF	*field1 = (TISPERM_FIELD_DEF *)fld1;
    TISPERM_FIELD_DEF	*field2 = (TISPERM_FIELD_DEF *)fld2;

    return (strcmp (field1 -> fldName, field2 -> fldName));
}
