#ifndef	tispermoptionsHXX
#define	tispermoptionsHXX

// define some common stuff
#define	MAX_TREENODE_LEVELS	    10
#define	OPTION_DELIMETER_STR	    "_"
#define	OPTION_DELIMETER_CHAR	    '_'
#define	OPTIONFILE_NAME		    "tisUtilAccess.txt"
#define	OPTIONNAME_SZ		    32
#define	TOP_LEVEL_UTIL_PRIVILEGE    "MMI"

// define the class to describe the MMI tree of utility priileges

// define box for display
typedef enum
{
    TISPERM_NO_GROUP_BOX = 0,
    TISPERM_GROUP_BOX
} TISPERM_BOX_TYPE;

// define image type for display
typedef enum
{
    TISPERM_MAIN_PARENT = 0,
    TISPERM_PARENT,
    TISPERM_LAST_PARENT,
    TISPERM_CHILD,
    TISPERM_LAST_CHILD
} TISPERM_IMAGE_TYPE;

// define the tree node structure
typedef struct TreeNode
{
    TreeNode		*fwdlink;	    // links to others at same level, siblings
    TreeNode		*sublink;	    // link to next level, children
    TreeNode		*parent;	    // reverse link to previous level
    char		optionName [OPTIONNAME_SZ + 1];	    // simple option ex. FILE
    char		dbOptionName [UTILACCPRIV_SZ + 1]; // db option ex. MMI_DATAVU_FILE
    int			level;
    int			selected;
    TISPERM_BOX_TYPE	boxType;
    TISPERM_IMAGE_TYPE	imageType;
} TREENODE, *TREENODE_PTR;


class OptionTree
{
    // some private stuff

    TisPermApp	    *app;
    TREENODE_PTR    treeRoot;
    void	    DeleteFwdLink (TREENODE_PTR node);
    void	    DeleteSubLink (TREENODE_PTR node);
    TREENODE_PTR    FindInSubLink (TREENODE_PTR node, char *optionName);
    void	    printTree (TREENODE_PTR node);
    int		    ReturnLevel (char *option);
    void	    UpdateTreeTraverse (TREENODE_PTR node);
    void	    WriteTreeTraverse (TREENODE_PTR node);

    // some stuff needed by WriteTreeTraverse
    vliststr	    *selectedList;  // list of selected utility options
    int		    rowCount;

    int IsRoot (TREENODE_PTR ptr)
    { return ((ptr == treeRoot) ? vTRUE : vFALSE); }

public:
    // class constructor and destructor
    OptionTree (TisPermApp *permApp);
    ~OptionTree (void);

    // public functions
    void	    ClearSelected (TREENODE_PTR node);
    void	    ClearChildrenSelected (TREENODE_PTR node, int *changed);
    void	    ClearParentSelected (TREENODE_PTR node, int *changed);
    TREENODE_PTR    FindNode (TREENODE_PTR node, char *optionName);
    int		    LoadTree (void);
    int		    ParentIsSelected (TREENODE_PTR node);
    void	    SetChildrenSelected (TREENODE_PTR node, int *changed);
    void	    SetParentSelected (TREENODE_PTR node, int *changed);
    void	    SetParentsChildrenSelected (TREENODE_PTR parent);
    void	    WriteTree (TREENODE_PTR node, char *groupName);

    // some public inline functions
    TREENODE_PTR GetTreeRoot (void)
	{ return (treeRoot); }
    void PrintTree (void)
	{ printTree (treeRoot); }
};

#endif	tispermoptionsHXX
