#include <vport.h>
/* ANSI headers */
#include vstdlibHEADER	/* for exit(), EXIT_ codes */

/* Galaxy headers */
#include vkindHEADER
#include vloadableHEADER
#include vobjectHEADER
#include vscribeHEADER
#include vrectHEADER
#include vcharHEADER
#include vnameHEADER
#include veventHEADER
#include vresourceHEADER
#include vwindowHEADER
#include vapplicationHEADER
#include vdialogHEADER
#include vstrHEADER
#include vlistviewHEADER
#include vrulerHEADER

#include "tisperm.h"
#include "tisPermApp.hxx"
#include "tisPermDatabase.hxx"
#include "tisPermMain.hxx"
#include "tisPermUtils.hxx"
#include "tisPermProtGrp.hxx"

static int protGroupNameFilter (filteredText *text, vtextSelection *sel,
			    const vchar **s, long *len);
static int descriptionFilter (filteredText *text, vtextSelection *sel,
			      const vchar **s, long *len);

vkindTYPED_FULL_DEFINITION(ProtGroupDialog, vdialog, NULL);

/*******************************************************************
*
*    ProtGroupDialog override for constructors init ()
*
*******************************************************************/

void ProtGroupDialog::LoadInit (vresource res)
{
    modifiedFlag = vFALSE;
    protGroupName = filteredTextitem::RequestSubLoad (this,
					tisperm_ProtGroup);

    description = filteredTextitem::RequestSubLoad (this,
					tisperm_Description);
   
    okButton = ProtGroupOkButton::RequestSubLoad (this,
					tisperm_OkButton);
   
    cancelButton = ProtGroupCancelButton::RequestSubLoad (this,
					tisperm_CancelButton);
    vdialog::LoadInit (res);

    // give buttons ptr to group dialog
    okButton -> SetProtGroupDialog (this);
    cancelButton -> SetProtGroupDialog (this);

    // set some filters
    protGroupName -> SetItemFilterFunc (protGroupNameFilter);
    protGroupName -> SetTextModifiedPtr (&this -> modifiedFlag);
    description -> SetItemFilterFunc (descriptionFilter);
    description -> SetTextModifiedPtr (&this -> modifiedFlag);
    Place (GetRoot (), vrectPLACE_CENTER, vrectPLACE_CENTER);
}


/*******************************************************************
*
*    ProtGroupDialog Destroy function
*
*******************************************************************/

void ProtGroupDialog::Destroy (void)
{
    if (IsOpen ())
    {
	Close ();
    }

    vdialog::Destroy ();
}



/*******************************************************************
*
*    ProtGroupDialog notify function
*
*******************************************************************/

void ProtGroupDialog::ObserveWindow (vevent *event)
{
    if ( event -> GetType () == veventWINDOW_STATE && event -> IsClose () )
    {
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}


/*******************************************************************
*
*    ProtGroupDialog Open function
*
*******************************************************************/

void ProtGroupDialog::OpenProtGroup (MainDialog *mnDlg, ProtGroupFunc grpFunc)
{
    mainDialog = mnDlg;
    protGroupFunc = grpFunc;

    if (protGroupFunc == ProtGroupModify)
    {
	// cannot modify protection group name
	protGroupName -> Disable ();
	UpdateTextFields ();
    }
    SetDialogTitle ();
    // reset here because UpdateTextFields will set it to true
    modifiedFlag = vFALSE;
    // open the dialog and start a sub event loop, halts the caller
    // at the statement after the call to this function
    // allows caller to access the dialog data after dialog is closed
    vdialog::Open();
    MakeTopMostDialog (this, NULL);
    vevent::Process ();
}


/*******************************************************************
*
*    ProtGroupDialog Set dialog title function, depends on what
*	function is to be done
*
*******************************************************************/

void ProtGroupDialog::SetDialogTitle (void)
{
    vstr	*dialogTitle;
    TisPermApp	*tisPerm = mainDialog -> GetPermApp ();
    vresource	titlesRes = tisPerm -> GetTitlesRes ();

    switch (protGroupFunc)
    {
	case ProtGroupAdd:
	    dialogTitle = vresourceGetString (titlesRes, tisperm_NewProtGrp);
	    break;

	case ProtGroupModify:
	    dialogTitle = vresourceGetString (titlesRes, tisperm_ModProtGrp);
	    break;
    }

    SetTitle (dialogTitle);
    vstrDestroy (dialogTitle);
}


/*******************************************************************
*
*    ProtGroupDialog 
*	update groupname and description fields
*
*******************************************************************/

void ProtGroupDialog::UpdateTextFields (void)
{
    int		    row, col;
    const vchar	    *value;
    vlistSelection  *selection;
    vliststr	    *protGroupList;
    
    selection = mainDialog -> prGrpListView -> GetSelection ();
    protGroupList = (vliststr *)mainDialog -> prGrpListView -> GetList ();
    protGroupList -> GetSelectedCell (selection, &row, &col);
    value = (vchar *)(protGroupList -> GetCellValue (row, 0));
    protGroupName -> SetText (value);
    value = (vchar *)(protGroupList -> GetCellValue (row, 1));
    description -> SetText (value);
}


/*******************************************************************
*
*    Some filter functions,for the ProtGroupDialog
*	NCharFilter (see tisgalaxyLib/textFilters.cxx
*
*******************************************************************/

static int protGroupNameFilter (filteredText *text, vtextSelection *sel,
				const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, PROTECTIONGRP_SZ));
}

static int descriptionFilter (filteredText *text, vtextSelection *sel,
			      const vchar **s, long *len)
{
    return (NCharFilter (text, sel, s, len, DESCRIPTION_SZ));
}


vkindTYPED_FULL_DEFINITION(ProtGroupOkButton, vbutton, NULL);

/*******************************************************************
*
*    ProtGroupOkButton observe function
*
*******************************************************************/

void ProtGroupOkButton::ObserveDialogItem (vevent *event)
{
    vstr    *nameStr = protGroupDialog -> protGroupName -> GetTextAsString ();
    vstr    *descStr = protGroupDialog -> description -> GetTextAsString ();
    ProtGroupFunc func = protGroupDialog -> GetProtGroupFunction ();
    MainDialog	*mainDialog = protGroupDialog -> GetMainDialog ();
    int	    groupExists =  mainDialog -> 
			    prGrpListView -> ProtGroupExists (nameStr);
    int	    todo = TISPERM_ERROR;

    if ((protGroupDialog -> GetModifiedFlag ()) == vFALSE)
    {
	tisPermMsgCenter (protGroupDialog, TISPERM_INFO_NOTICE, tisperm_NothingModMsg);
	vstrDestroy (nameStr);
	vstrDestroy (descStr);
	return;
    }

    if (vcharLength (nameStr) == 0)
    {
	tisPermMsgCenter (protGroupDialog, TISPERM_INFO_NOTICE, tisperm_NoProtGroupMsg);
	vstrDestroy (nameStr);
	vstrDestroy (descStr);
	return;
    }
    if (func == ProtGroupAdd)
    {
	if (groupExists == vTRUE)
	{
	    tisPermMsgCenter (protGroupDialog,TISPERM_INFO_NOTICE,
			       tisperm_ProtGrpAlreadyExistsMsg, nameStr);
	    vstrDestroy (nameStr);
	    vstrDestroy (descStr);
	    return;
	}
	else
	{
	    mainDialog -> prGrpListView -> ModList (AddToList, nameStr, descStr);
	    todo = AddToList;
	}
    }
    // modify existing group
    else if (func == ProtGroupModify)
    {
	if (groupExists == vFALSE)
	{
	    tisPermMsgCenter (protGroupDialog, TISPERM_INFO_NOTICE,
			       tisperm_ProtGrpDoesNotExistMsg, nameStr);
	    vstrDestroy (nameStr);
	    vstrDestroy (descStr);
	    return;
	}
	else
	{
	    mainDialog -> prGrpListView -> ModList (ModTheList, nameStr, descStr);
	    todo = ModTheList;
	}
    }

    vstr	*todoStr;
    TisPermApp	*app = mainDialog -> GetPermApp ();
    vresource	msgsRes = app -> GetMessagesRes ();
    switch (todo)
    {
	case TISPERM_ERROR:
	    vstrDestroy (nameStr);
	    vstrDestroy (descStr);
	    return;

	case AddToList:
	    todoStr = vresourceGetString (msgsRes, tisperm_AddedStr);
	    break;

	case ModTheList:
	    todoStr = vresourceGetString (msgsRes, tisperm_ModStr);
	    break;
    }
    tisPermMsgCenter (protGroupDialog, TISPERM_INFO_NOTICE, 
		       tisperm_PtGrpDbUpdateMsg, nameStr, todoStr);
    vstrDestroy (nameStr);
    vstrDestroy (descStr);
    vstrDestroy (todoStr);
    vevent::StopProcessing ();
    return;
}


vkindTYPED_FULL_DEFINITION(ProtGroupCancelButton, vbutton, NULL);

/*******************************************************************
*
*    ProtGroupCancelButton observe function
*
*******************************************************************/

void ProtGroupCancelButton::ObserveDialogItem (vevent *event)
{
    vevent::StopProcessing ();
    return;
}

