#ifndef tispermuserHXX
#define	tispermuserHXX

class	UserCheckBox;
class	UserSpinner;
class	UserOkButton;
class	UserCancelButton;
class	passwordTextitem;

// define functionality
enum UserFunc
{
    UserAdd = 0,
    UserModify,
    UserDelete
};


class UserDialog : public vdialog
{
    MainDialog	    *mainDialog;
    UserFunc	    userFunc;
    void	    UpdateFields (void);
    void	    SetDialogTitle (void);
    int		    modifiedFlag;

public:

    // overrides from vkindTyped
    vkindTYPED_FULL_DECLARATION(UserDialog);

    // the default versions of necessary constructors/destructor
    vloadableINLINE_CONSTRUCTORS(UserDialog, vdialog);

    // vloadable override
    void LoadInit (vresource resource);

    // override
    void Destroy ();

    // override
    void ObserveWindow (vevent *event);

    void OpenUser (MainDialog *mnDlg, UserFunc usrFunc);

    // ptrs to some dialog stuff
    filteredTextitem	*userName;
    filteredTextitem	*description;
    passwordTextitem	*password;
    UserCheckBox	*allowLogout;
    spinnerClass	*logoutTime; // spinnerClass in tisgalaxylib.
    vtextitem		*memberOfGroup;
    passwordTextitem	*verify;
    UserOkButton	*okButton; 
    UserCancelButton	*cancelButton;
    
    // some public funcs
    UserFunc GetUserFunction (void)
	{ return (userFunc); }
    MainDialog *GetMainDialog (void)
	{ return (mainDialog); }
    int VerifyPassword (vchar *encrybuff);
    int GetModifiedFlag (void)
	{ return (modifiedFlag); }
};


class  UserOkButton : public vbutton
{
    UserDialog	    *userDialog;

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(UserOkButton);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(UserOkButton, vbutton);

    //override virtual issue function
   void ObserveDialogItem(vevent *event);

   // handle userDialog
   void SetUserDialog (UserDialog *dlg)
    { userDialog = dlg; }
};


class  UserCancelButton : public vbutton
{
    UserDialog	    *userDialog;

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(UserCancelButton);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(UserCancelButton, vbutton);

    //override virtual issue function
   void ObserveDialogItem (vevent *event);

   // handle userDialog
   void SetUserDialog (UserDialog *dlg)
    { userDialog = dlg; }
};


// class to handle allowing logout, spinnerClass in tisgalaxylib.cxx
class UserCheckBox : public checkBoxClass
{

public:

    // macros for standard versions of the
    vkindTYPED_FULL_DECLARATION(UserCheckBox);

    // constructors and destructors
    vloadableINLINE_CONSTRUCTORS(UserCheckBox, checkBoxClass);

    void ObserveDialogItem (vevent *event);
};


// special class to handle password text, 
class passwordText : public vtextitemText
{

    // if you need to know whether the text was modified,
    // set this to vTRUE inside of ObserveDelete/ObserverInsert
    // need to init this pointer
    int	*modified;

public:

    vkindTYPED_FULL_DECLARATION(passwordText);
    vloadableINLINE_CONSTRUCTORS(passwordText, vtextitemText);

    passwordText (vtextitemText &from);

    // overrides of class functions
    void Destroy ();
    void DeleteText (vtextSelection *sel);
    void InsertText (vtextSelection *sel, const vchar *text, long len);
    void ObserveDelete (long pos, long delta);
    void ObserveInsert (long pos, long delta);

    int		    firstChar;
    vtext	    *realPassword;
    vtextSelection  *realSelection;

    // handle the modified flag
    void SetModifiedPtr (int *mod) { modified = mod; }
    int *GetModifiedPtr (void) { return (modified); }
    void SetModifiedFlag (void) { *modified = vTRUE; }

protected:
    void Init(void);
    void LoadInit(vresource);
    void CopyInit(vloadable*);
    void Copy(vloadable*);
};

class passwordTextitem : public vtextitem
{

public:

    vkindTYPED_FULL_DECLARATION(passwordTextitem);
    vloadableINLINE_CONSTRUCTORS(passwordTextitem, vtextitem);

    void LoadInit (vresource resource);
    void ObserveView (vevent *event, int message);

    void SetTextModifiedPtr (int *mod)
    {
	passwordText *text = (passwordText *)(GetTextData ());
	text -> SetModifiedPtr (mod);
    }
};


#endif	tispermuserHXX

