// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (C) 1993-1995  Microsoft Corporation.  All Rights Reserved.
//
//  MODULE:   tisSvc.c
//
//  PURPOSE:  Implements the body of the service.
//            This starts channelAccess up as a service.
//
//  FUNCTIONS:
//            ServiceStart(DWORD dwArgc, LPTSTR *lpszArgv);
//            ServiceStop( );
//
//  COMMENTS: The functions implemented in tisSvc.c are
//            prototyped in tisService.h
//              
//
//  AUTHOR: Grant Irani
//	    (modified from original by Craig Link - Microsoft Developer Support)
//


#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <process.h>
#include <tchar.h>
#include <time.h>
#include <direct.h>

#include "tisService.h"
#include "msgMon.h"

static void ResetAppRead ();
static char *GetNextApp (char *command);

// this event is signalled when the
// service should end
//
HANDLE  hServerStopEvent = NULL;

// handles to processes, and their names
#define MAX_PROCESSES 20
int	procIx = 0;
HANDLE	procHandles [MAX_PROCESSES];
CHAR	*procNames [MAX_PROCESSES];

// handles to events (1 per process + event object)
int	monIx = 0;
HANDLE  hEvents [MAX_PROCESSES + 1];

// debug level for spawned processes (incomplete until signalling method is worked out)
int	hServerDebugLevel = 0;

//
//  FUNCTION: ServiceStart
//
//  PURPOSE: Actual code of the service
//           that does the work.
//
//  PARAMETERS:
//    dwArgc   - number of command line arguments
//    lpszArgv - array of command line arguments
//
//  RETURN VALUE:
//    none
//
//  COMMENTS:
//
VOID ServiceStart (DWORD dwArgc, LPTSTR *lpszArgv)
{
    int	    i;
    int	    stopped;
    char    *argv [20];
    char    cmdtext [300];
    time_t  starttime;
    DWORD   dwWait;
    DWORD   exitCode;

    ///////////////////////////////////////////////////
    //
    // Service initialization
    //

    // report the status to the service control manager.
    //
    if (!ReportStatusToSCMgr(
        SERVICE_START_PENDING, // service state
        NO_ERROR,              // exit code
        3000))                 // wait hint
        goto cleanup;

    // create the event object. The control handler function signals
    // this event when it receives the "stop" control code.
    //
    hServerStopEvent = CreateEvent(
        NULL,    // no security attributes
        TRUE,    // manual reset event
        FALSE,   // not-signalled
        NULL);   // no name

    // failed to create event, quit
    if ( hServerStopEvent == NULL)
        goto cleanup;

    // first event for us to wait on
    hEvents[0] = hServerStopEvent;

    // report the status to the service control manager.
    //
    if (!ReportStatusToSCMgr(
        SERVICE_START_PENDING, // service state
        NO_ERROR,              // exit code
        3000))                 // wait hint
        goto cleanup;

    // initialize event logger
    tisOpenLog ("tisservice", 6, 2, NULL, NULL);
    
    // system will auto restart if any process fails, until stopped by operator
    stopped = FALSE;
    while (!stopped)
    {
	// process start_TIS script
	ResetAppRead ();
	while (GetNextApp (cmdtext))
	{
	    // start an app: detached if last character is '&'
	    char *argp;
	    static char execdir [200];
	    static char savedir [200];
	    char *execdirp;
	    int argc;

	    /* break command into pieces */
	    argp = cmdtext;
	    
	    // prepare for optional execution directory spec
	    execdir[0] = 0;
	    if (*argp == '(')
	    {
		// collect exceutiion directory name
		execdirp = execdir;
		argp++;
		while (*argp && *argp != ')')
		{
		    *execdirp++ = *argp++;
		}
		// terminate string
		*execdirp = 0;

		// skip past terminator
		if (*argp)
		    argp++;
		// skipp over white space
		while (*argp && (*argp == ' ' || *argp == '\t'))
		    argp++;
	    }
	    // process command text
	    for (argc = 0; argp && *argp && argc < 19; argc++)
	    {
		// handle quoted argument (present as single arg)
		if (*argp == '\"')
		{
		    argp++;
		    argv [argc] = argp;
		    // go to for terminating '"'
		    argp = strchr (argp, '"');
		    if (argp)
		    {
			*argp = 0;
			argp++;
			while (*argp && (*argp == ' ' || *argp == '\t'))
			    argp++;
		    }
		    else /* missing trailing '"' */
			break;
		}
		// stop processing if end of command line
		if (*argp == '&')
		{
		    break;
		}
		else
		{
		    //set pointer to next arg
		    argv [argc] = argp;
		    // go to next white space - check for blank
		    if (strchr (argp, ' '))
		    {
			argp = strchr (argp, ' ');
			*argp = 0;
			argp++;
			// skip to next arg
			while (*argp && (*argp == ' ' || *argp == '\t'))
			    argp++;
		    }
		    else
		    {
			// white space - check for tab
			argp = strchr (argp, '\t');
			if (argp)
			{
			    *argp = 0;
			    argp++;
			    // skip to next arg
			    while (*argp && (*argp == ' ' || *argp == '\t'))
				argp++;
			}
		    }
		}
	    }
	    // terminate argument list
	    argv [argc] = 0;

	    // launch the process - switch to execution directory if specified
	    if (execdir[0])
	    {
		_getcwd (savedir, 200);
		_chdir (execdir);
	    }
	    // spawn it
	    procHandles[procIx]
		= (HANDLE)_spawnvp (_P_NOWAIT, argv[0], (const char * const *)argv);
	    // check for type of command
	    if (argp == NULL || *argp != '&')
	    {
		// Wait for completion if line didn't end with '&'
		while (((int)procHandles[procIx] != -1)
		  && WaitForSingleObject ( procHandles[procIx], 2500 ) == WAIT_TIMEOUT)
		{
		    // report the status to the service control manager.
		    //
		    if (!ReportStatusToSCMgr(
			SERVICE_START_PENDING, // service state
			NO_ERROR,              // exit code
			3000))                 // wait hint
			goto cleanup;
		}
	    }
	    else
	    {
		// add name to names array
		procNames [procIx] = malloc (strlen(argv[0])+1);
		if (procNames [procIx])
		    strcpy (procNames [procIx], argv[0]);
		else
		    // malloc failed, quit the service
		    goto cleanup;
		// Add to monitored process list.
		if (*(argp+1) != '!') // An unmonitored process is indicated by "... &!"
		{
		    hEvents[monIx+1] = procHandles[procIx]; // bump monitored process index
		    monIx++;
		}
		procIx++; // bump started process index
	    }
	    // restore cwd to startup directory
	    if (execdir[0])
	    {
		_chdir (savedir);
	    }
	    // report the status to the service control manager.
	    //
	    if (!ReportStatusToSCMgr(
		SERVICE_START_PENDING, // service state
		NO_ERROR,              // exit code
		3000))                 // wait hint
		goto cleanup;
	}
	// report the status to the service control manager.
	//
	if (!ReportStatusToSCMgr(
	    SERVICE_RUNNING,       // service state
	    NO_ERROR,              // exit code
	    0))                    // wait hint
	    goto cleanup;

	//
	// End of initialization
	//
	////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////
	//
	// Service is now running, monitor ca processes
	//
	starttime = time(0);

	// Wait for events. any event indicates shutdown!
	dwWait = WaitForMultipleObjects( monIx+1, hEvents, FALSE, INFINITE );

	if (dwWait == WAIT_FAILED)
	{
	    tisSysLog (0,0, "tisService wait failed, error = %ld", GetLastError ());
	    stopped = TRUE;
	}
	else if (dwWait == WAIT_OBJECT_0)
	{
	    // Stop service signaled, stop TIS
	    // report the status to the service control manager.
	    //
	    ReportStatusToSCMgr(
		SERVICE_STOP_PENDING, // service state
		NO_ERROR,             // exit code
		10000);               // wait hint
	    stopped = TRUE;
	}
	else
	{
	    // one of the monitored processes failed - log the event
	    // identify name of treminated process
	    for (i = 0; i < procIx; i++)
		if (procHandles[i] == hEvents[dwWait - WAIT_OBJECT_0])
		    break;
	    if (GetExitCodeProcess (hEvents[dwWait - WAIT_OBJECT_0 - 1], &exitCode))
		tisSysLog (0,0, "tisService process %s failed, error = %lx, Attempting restart.",
			procNames [i],	exitCode);
	    else
		tisSysLog (0,0, "tisService process %s failed, unable to get error, Attempting restart.",
			procNames [i]);
	    // if failed within 5 seconds of start process completion, exit service!
	    if ((time(0) - starttime) < 5)
	    {
		tisSysLog (0,0, "tisService fail within 5 seconds, stopping monitored services.",
			procNames [dwWait - WAIT_OBJECT_0 - 1],	exitCode);
		stopped = TRUE;
	    }
	}

	// terminate the spawned processes! (this is an abrupt termination, should signal)
	for (i = 0; i < procIx; i++)
	{
	    TerminateProcess (procHandles [i], 0);
	    // free up the process name memory
	    free (procNames [i]);
	    procNames [i] = NULL;
	}
	procIx = 0;
	monIx = 0;
    }
  cleanup:
    // All done!
    tisCloseLog ();

    // clean up the signal handle
    if (hServerStopEvent)
        CloseHandle(hServerStopEvent);

}


//
//  FUNCTION: ServiceStop
//
//  PURPOSE: Stops the service
//
//  PARAMETERS:
//    none
//
//  RETURN VALUE:
//    none
//
//  COMMENTS:
//    If a ServiceStop procedure is going to
//    take longer than 3 seconds to execute,
//    it should spawn a thread to execute the
//    stop code, and return.  Otherwise, the
//    ServiceControlManager will believe that
//    the service has stopped responding.
//    
VOID ServiceStop()
{
    if ( hServerStopEvent )
        SetEvent(hServerStopEvent);
}

//
//  FUNCTION: ServiceDebugIncrement
//
//  PURPOSE: Changes the debug level of the service
//
//  PARAMETERS:
//    change value
//
//  RETURN VALUE:
//    none
//
//  COMMENTS:
//    
VOID ServiceDebugIncrement(int increment)
{
    hServerDebugLevel += increment;
    if (hServerDebugLevel < 0)
	hServerDebugLevel = 0;
}


FILE	*fp;

/*********
 *  ResetAppRead()
 *
 *  Resets the position in the ap menu specification file.
 *
 **********/
static void ResetAppRead ()
{
    if (fp)
	fclose (fp);

    /* look up requested service, exit if not found */
    fp = fopen ("\\tis4000\\tasks\\start_TIS", "rt");
}

/*********
 *  GetNextAppw()
 *
 *  Reads the next app item.
 *
 **********/
static char *GetNextApp (char *command)
{
    int ix;
    char *sp;
    char inpstring [256];

    if (fp)
    {
        while (!feof (fp))
        {
	    ix = 0;
	    while (!feof (fp))
	    {
		inpstring [ix] = fgetc(fp);
		if ((inpstring [ix] == 0x0d) || (inpstring [ix] == 0x0a) 
		  || (inpstring [ix] == -1))
		{
		    break;
 		}
 		if (ix < 255)
		{
		    ix++;
		}
	    }
	    inpstring [ix] = 0;

	    /* process input line */
	    sp = inpstring;
	    /* skip blanks */
	    while (*sp && (*sp == ' ' || *sp == 0x09))
		sp++;
	    if ((*sp == 0) || (*sp == '#'))
		continue; /* blank line or comment! */
	    /* process input- line is formatted as
	    ** <executable command>
	    */
	    strncpy (command, sp, 255);
	    return command;
	}
    }
    if (fp)
	fclose(fp);
    fp = NULL;
    return NULL;
}
