#ifndef textFilterH
#define textFilterH 1

#include vtextHEADER
#include vtextviewHEADER
#include vtextitemHEADER

class filteredText;
class filteredTextitem;


typedef int (*filterFunc)(filteredText *text, vtextSelection *selection,
			  const vchar **newText, long *length);

// vtextitemText represents the text within a vtextitem
// it inherits most of it's functionality from vtext class

class filteredText : public vtextitemText 
{
    // a void ptr for what ever you need
    void *userPtr;

    // if you need to know whether the text was modified,
    // set this to vTRUE inside of ObserveDelete/ObserverInsert
    // need to init this pointer
    int	*modified;

    // local modifiy if you are looking at your textfield 1 at a time
    int	localModified;

public:
    vkindTYPED_FULL_DECLARATION(filteredText);
    vloadableINLINE_CONSTRUCTORS(filteredText, vtextitemText);
    filteredText(vtextitemText &from);

    // overrides of class functions
    void InsertText(vtextSelection*, const vchar*, long);
    void ObserveDelete (long pos, long delta);
    void ObserveInsert (long pos, long delta);

    // handle the filter function
    void SetFilterFunc(filterFunc func) { filterfunc = func; }
    filterFunc GetFilterFunc() const { return filterfunc; }

    // handle the modified flag
    void SetModifiedPtr (int *mod) { modified = mod; }
    int *GetModifiedPtr (void) { return (modified); }
    void SetModifiedFlag (void) { *modified = vTRUE; }

    // handle the localModified flag
    void SetLocalModifiedFlag (int mod) { localModified = mod; }
    int	 GetLocalModifiedFlag (void) { return (localModified); }

    // handle the user ptr
    void *GetUserPtr (void) { return (userPtr); }
    void SetUserPtr (void *ptr) { userPtr = ptr; }

protected:
    void Init(void);
    void LoadInit(vresource);
    void CopyInit(vloadable*);
    void Copy(vloadable*);
    void Destroy();

private:
    filterFunc filterfunc;
}; 


class filteredTextitem : public vtextitem 
{
    // a void ptr for what ever you need
    void *userPtr;

public:
    vkindTYPED_FULL_DECLARATION(filteredTextitem);
    vloadableINLINE_CONSTRUCTORS(filteredTextitem, vtextitem);

    vtext *CreateTextData();
    void SetItemFilterFunc(filterFunc func);
    void SetTextModifiedPtr (int *mod)
    {
	filteredText *text = (filteredText *)(GetTextData ());
	text -> SetModifiedPtr (mod);
    }

    // handle the user ptr
    void *GetUserPtr (void) { return (userPtr); }
    void SetUserPtr (void *ptr) { userPtr = ptr; }

protected:
    void Init(void);
    void LoadInit(vresource resource);
};


// *** define time subclasses of filteredTextitem

// *** defines fro the size of the time strings,
// *** MUST agree withstrings in textFilters.cxx
#define	HHMMTIME_SZ	5
#define	HHMMSSTIME_SZ	8
#define	HHMMSSTTIME_SZ	10

// *** HH:MM TIME SUB CLASS OF filteredTextitem

class hhmmTimeClass : public filteredTextitem
{

public:
    vkindTYPED_FULL_DECLARATION(hhmmTimeClass);

    vloadableINLINE_CONSTRUCTORS(hhmmTimeClass, filteredTextitem);

    // virtual member function override
    void ObserveView (vevent *event, int message);
};

// *** HH:MM:SS TIME SUB CLASS OF filteredTextitem

class hhmmssTimeClass : public filteredTextitem
{

public:
    vkindTYPED_FULL_DECLARATION(hhmmssTimeClass);

    vloadableINLINE_CONSTRUCTORS(hhmmssTimeClass, filteredTextitem);

    // virtual member function override
    void ObserveView (vevent *event, int message);
};

// *** HH:MM:SS.T TIME SUB CLASS OF filteredTextitem

class hhmmsstTimeClass : public filteredTextitem
{

public:
    vkindTYPED_FULL_DECLARATION(hhmmsstTimeClass);

    vloadableINLINE_CONSTRUCTORS(hhmmsstTimeClass, filteredTextitem);

    // virtual member function override
    void ObserveView (vevent *event, int message);
};


// *** Filter function prototypes

int alarmGroupFilter (filteredText *text, vtextSelection *sel,
		      const vchar **s, long *len);

int calcExpressionFilter (filteredText *text, vtextSelection *sel,
			  const vchar **s, long *len);

int descriptorFilter (filteredText *text, vtextSelection *sel,
		      const vchar **s, long *len);

int digitalStateFilter (filteredText *text, vtextSelection *sel,
			const vchar **s, long *len);

int egutagFilter (filteredText *text, vtextSelection *sel,
		  const vchar **s, long *len);

int floatFilter (filteredText *text, vtextSelection *sel,
		 const vchar **s, long *len);

int integerFilter (filteredText *text, vtextSelection *sel,
		   const vchar **s, long *len);

int iocnameFilter (filteredText *text, vtextSelection *sel,
		   const vchar **s, long *len);

int locationFilter (filteredText *text, vtextSelection *sel,
		    const vchar **s, long *len);

int multiDigitalStateFilter (filteredText *text, vtextSelection *sel,
			     const vchar **s, long *len);

int NCharFilter (filteredText *text, vtextSelection *sel,
		 const vchar **s, long *len, int n);

int numericFilter (filteredText *text, vtextSelection *sel,
		   const vchar **s, long *len);

int operatorNameFilter (filteredText *text, vtextSelection *sel,
		   const vchar **s, long *len);

int protectionGroupFilter (filteredText *text, vtextSelection *sel,
			   const vchar **s, long *len);

int protectionLevelFilter (filteredText *text, vtextSelection *sel,
		   const vchar **s, long *len);

int pvnameFilter (filteredText *text, vtextSelection *sel,
		  const vchar **s, long *len);

int resourceNameFilter (filteredText *text, vtextSelection *sel,
			const vchar **s, long *len);

int stateValFilter (filteredText *text, vtextSelection *sel,
		    const vchar **s, long *len);

int stringValFilter (filteredText *text, vtextSelection *sel,
		    const vchar **s, long *len);

int tankfarmCmdStsFilter (filteredText *text, vtextSelection *sel,
			  const vchar **s, long *len);

int udbStringsFilter (filteredText *text, vtextSelection *sel,
		      const vchar **s, long *len);


// *** time filter function prototypes

int hhmmTimeFilter (filteredText *text, vtextSelection *sel,
		    const vchar **string, long *len);

int hhmmssTimeFilter (filteredText *text, vtextSelection *sel,
		      const vchar **string, long *len);

int hhmmsstTimeFilter (filteredText *text, vtextSelection *sel,
		       const vchar **string, long *len);

int datetimeFilter (filteredText *text, vtextSelection *sel,
		    const vchar **string, long *len);

int dateFilter (filteredText *text, vtextSelection *sel,
		    const vchar **string, long *len);

#endif textFilterH

