/* trenddetaillist.cxx,v 1.1.1.1 1999/08/03 14:45:09 tis4000 Exp
 *
 */

/*
 * TrendDetailList - trend graph container class
 */

#include <vport.h>		/* for v*HEADER */
#include vstdlibHEADER		/* for random, exit, EXIT_SUCCESS */
#include vlistHEADER
#include vlistviewHEADER
#include vliststrHEADER
#include vstartupHEADER
#include vmemHEADER
#include vprefHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcharHEADER
#include vnumHEADER

#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#endif WIN32

extern "C"
{
    #include "trendutils.h"
    extern char    *RedStrings[];
}
#include "trenddocs.h"
#include "trenddetaillist.h"

// Helper functions
class DetailButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(DetailButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(DetailButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendDetailList *p, int t, int i) { parent = p; type = t; id = i; }
  virtual void DrawContent (void);
private: 
    TrendDetailList *parent;
    int type;
    int id;
};

vkindTYPED_FULL_DEFINITION(DetailButton, vbutton, NULL);
void DetailButton::ObserveDialogItem (vevent *event)
{
    parent->ControlClick (type, id);
}

void DetailButton::DrawContent ()
{
    vdrawGSave();
    vdrawRectsClip(GetRect(), 1);

    vdrawRectShowCenter (GetTitle(), GetRect());// this shows the text

    vdrawGRestore();
}


// macro definitions for vkindtyped overrides

vkindTYPED_FULL_DEFINITION(TrendDetailList, vlistview, NULL);

// vloadable
void TrendDetailList::LoadInit(vresource resource)
{
    vlistview::LoadInit(resource);
    trendSet = NULL;
    for (int ix = 0; ix < C_TRENDPTS; ix++)
    {
	ctlButton[ix] = NULL;
	enbButton[ix] = NULL;
    }
    vgrey.SetRGB ((192 * vcolorCOMPONENT_MAX)/256,
		  (192 * vcolorCOMPONENT_MAX)/256,
		  (192 * vcolorCOMPONENT_MAX)/256);
    greyColor = vcolor::Intern (&vgrey);
}

/*
 * TrendLimitList::Destroy
 *
 * Deletes the list
 */
void TrendDetailList::Destroy()
{
    // Destroy list
    if (GetList())
	delete GetList();  // !!! Same as vdialogItem::itemList ??

    vlistview::Destroy();
}

/*
 * TrendDetailList::ObserveSelect
 *
 */
void TrendDetailList::ObserveSelect(int state)
{
    int row;
    int col;
    vlist *list = GetList();

    list->GetSelectedCell(GetSelection (), &row, &col);
    if (row == vlistNO_CELLS_SELECTED)
    {
	return;
    }

    if (state == vlistviewSELECTION_HAS_CHANGED)
    {
	// Do appropriate action based on where clicked and click count
	int currx, curry;
	GetLastPointerPosition(&currx, &curry);
	vrect tempRect;
	GetCellRect(row, 0, &tempRect);
	tempRect.SetX(tempRect.GetX() + 3);
	tempRect.SetWidth(tempRect.GetHeight());
	if (tempRect.ContainsPoint(currx, curry))
	{
	    // ctlButton hit!
	    ctlButton[row]->ObserveDialogItem (NULL);
	}
	else
	{
	    tempRect.SetX(tempRect.GetX() + tempRect.GetWidth() + 2);
	    if (tempRect.ContainsPoint(currx, curry))
	    {
		enbButton[row]->ObserveDialogItem (NULL);
	    }
	}
	InvalViewCell(row, 0);
    }
}

/*
 * TrendDetailList::DrawCellContents
 *
 * Draw resource entry in list cell
 */
void TrendDetailList::DrawCellContents(int row, int col, 
				      int selected, vrect *cellRect)
{
    char    ptstring [200];

    if ((trendSet == NULL) || (trendSet->numTrends <= row))
	return;

    vdrawGSave();
    vdrawRectsClip(cellRect, 1);

    vrect temp = *cellRect;
    vdrawRectClip(temp.GetX(), temp.GetY(), temp.GetWidth(), temp.GetHeight());
    // create buttons if necessary
    if (ctlButton[row] == NULL)
    {
	ctlButton[row] = new DetailButton;
	ctlButton[row]->SetTitleScribed (vcharScribeLiteral ("C"));
	ctlButton[row]->SetParent (this, CTL_CLICK, row);
	DetermineDialog()->AppendItem (ctlButton[row]);
	enbButton[row] = new DetailButton;
	enbButton[row]->SetTitleScribed (vnumScribeInt (row));
	enbButton[row]->SetParent (this, ENB_CLICK, row);
	DetermineDialog()->AppendItem (enbButton[row]);
    }
    vrect buttonRect = temp;
    buttonRect.SetX (buttonRect.GetX() + 1);
    buttonRect.SetY (buttonRect.GetY() + 1);
    buttonRect.SetWidth (temp.GetHeight()-2);
    buttonRect.SetHeight (temp.GetHeight()-2);
    ctlButton[row]->SetRect (&buttonRect);
    buttonRect.SetX (buttonRect.GetX() + buttonRect.GetWidth() + 2);
    enbButton[row]->SetRect (&buttonRect);

    int offset = ctlButton[row]->GetWidth() + enbButton[row]->GetWidth() + 5;
    temp.SetX(temp.GetX() + offset);
    temp.SetWidth(temp.GetWidth() - offset);

    // show buttons and text in proper colors
    if (trendSet->trends[row].enable)
    {
	if (trendSet->trends[row].ctlEnable)
	{
	    vdrawSetColor (fgColors[row]);
	}
	else
	{
	    vdrawSetColor (greyColor);
	}
	ctlButton[row]->Draw ();
	vdrawSetColor(fgColors[row]);
    }
    else
    {
	vdrawSetColor (greyColor);
	ctlButton[row]->Draw ();
    }
    enbButton[row]->Draw ();
    if (selected)
    {
	vdraw::SetFunction(vdrawFUNC_INVERT);
    }
    trendSet->trends[row].GetDetailString (ptstring, sizeof(ptstring), cursor);
    vdrawRectShowCenterY ((unsigned char *)ptstring, &temp);// this shows the text

    vdrawGRestore();
}

void TrendDetailList::SetPrefs (vcolorSpec fgColorSpec[C_TRENDPTS], vcolorSpec bgColorSpec)
{
    int i;

    // convert to vcolor and store
    for (i = 0; i < 8; i++)
    {
	fgColors[i] = vcolor::Intern (&fgColorSpec [i]);
    }
    bgColor = vcolor::Intern (&bgColorSpec);
    Inval (0);
}

void TrendDetailList::SetTrendSet (TrendSet *set)
{
    vliststr	*newlist;
    vlist	*oldlist;
    int		ix;
    int		w;
    int		h;

    trendSet = set;
    // Destroy old list
    if (oldlist = GetList())
	delete oldlist;

    newlist = new vliststr;
    for (ix = 0; ix < trendSet->numTrends; ix++)
    {
        trimblanks (trendSet->specs[ix]);
        if (trendSet->specs[ix][0])
        {
            newlist->SetCellValueScribed (ix, 0, vcharScribeSystem (trendSet->specs[ix]));
        }
    }
    SetList (newlist);

    DetermineDialog()->CalcMinSize (&w, &h);
    DetermineDialog()->SetMinSize (w, h);
    vrect newrect = *DetermineDialog()->GetRect();
    newrect.SetX (0);
    newrect.SetY (0);
    DetermineDialog()->Arrange (&newrect);
}

void TrendDetailList::ControlClick (int type, int id) // override with change callback!
{
    // User code should adjust document for new value
}

