/* trendlistdlg.cxx,v 1.1.1.1 1999/08/03 14:45:10 tis4000 Exp
 *
 */

/*
 * TrendListDialog - trend graph container class
 */

#include <vport.h>		/* for v*HEADER */
#include vstdlibHEADER		/* for random, exit, EXIT_SUCCESS */
#include vlistHEADER
#include vstartupHEADER
#include vmemHEADER
#include vprefHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcharHEADER
#include vcomboboxHEADER
#include vdialogHEADER

#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#endif WIN32

extern "C"
{
    #include "trendutils.h"
}
#include "trenddocs.h"
#include "trendlistdlg.h"
#include "tisgalaxylib.hxx"

// Helper functions
class tlAcceptButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tlAcceptButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tlAcceptButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendListDialog *p) { parent = p; }
private:
    TrendListDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tlAcceptButton, vbutton, NULL);
void tlAcceptButton::ObserveDialogItem (vevent *event)
{
    parent->AcceptClick();
}

class tlCancelButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tlCancelButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tlCancelButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendListDialog *p) { parent = p; }
private:
    TrendListDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tlCancelButton, vbutton, NULL);
void tlCancelButton::ObserveDialogItem (vevent *event)
{
    parent->CancelClick();
}

// macro definitions for vkindtyped overrides
vkindTYPED_FULL_DEFINITION(TrendListDialog, vdialog, NULL);

// vloadable
void TrendListDialog::LoadInit(vresource resource)
{
    // get handles to interesting elements
    comboList = vcombobox::RequestSubLoad (this, vnameInternGlobalLiteral("ComboTag"));
    accept = tlAcceptButton::RequestSubLoad (this, vnameInternGlobalLiteral("AcceptButtonTag"));
    cancel = tlCancelButton::RequestSubLoad (this, vnameInternGlobalLiteral("CancelButtonTag"));

    vdialog::LoadInit(resource);

    SetPopup (TRUE);
    accept->SetParent (this);
    cancel->SetParent (this);
}

/*********
 *  TrendListDialog::ObserveWindow()
 *
 *  The dialog notification procedure gets called whenever any 
 *  major event applies to the dialog as a whole.  We need to
 *  look for the event which tells us that the dialog has
 *  closed.
 *
 **********/
void TrendListDialog::ObserveWindow (vevent *event)
{
    vcomboboxValue	*value;
    int			valueCnt;
    int			i;

    result = RESULT_CANCEL;
    if (event->GetType() == veventWINDOW_STATE && event->IsClose ())
    {
	vevent::StopProcessing ();
	valueCnt = comboList->GetValueCount ();
	for (i = valueCnt - 1; i >= 0; i--)
	{
	    value = comboList->GetValueAt (i);
	    comboList->DeleteValueAt (i);
	    delete value;
	}
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}

/*********
 *  TrendListDialog::AcceptClick()
 *
 *  The procedure gets called whenever the accept button is clicked 
 *
 **********/
void TrendListDialog::AcceptClick ()
{
    vcomboboxValue	*value;
    int			valueCnt;
    int			i;

    result = RESULT_OK;

    value = comboList->GetValue();
    vcharExportBounded (vcharScribeSystemExport (value->GetTitle()),
		    resultstring, sizeof(resultstring));

    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
    valueCnt = comboList->GetValueCount ();
    for (i = valueCnt - 1; i >= 0; i--)
    {
	value = comboList->GetValueAt (i);
	comboList->DeleteValueAt (i);
	delete value;
    }
}

/*********
 *  TrendListDialog::CancelClick()
 *
 *  The procedure gets called whenever the cancel button is clicked 
 *
 **********/
void TrendListDialog::CancelClick ()
{
    vcomboboxValue	*value;
    int			valueCnt;
    int			i;

    result = RESULT_CANCEL;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
    valueCnt = comboList->GetValueCount ();
    for (i = valueCnt - 1; i >= 0; i--)
    {
	value = comboList->GetValueAt (i);
	comboList->DeleteValueAt (i);
	delete value;
    }
}

int TrendListDialog::Alert (char *dlgtitle, char **listitems)
{
    vcomboboxValue	*value;
    int			valueCnt;
    int			i;

    // if the combo is open,close it to make the build faster
    if (IsOpen ())
    {
	Close ();
    }

    valueCnt = comboList->GetValueCount ();
    for (i = valueCnt - 1; i >= 0; i--)
    {
	value = comboList->GetValueAt (i);
	comboList->DeleteValueAt (i);
	delete value;
    }
    // set title for dialog
    SetTitle ((unsigned char *)dlgtitle);

    // load the list items
    while (*listitems)
    {
	value = new vcomboboxValue;
	value -> SetTitleScribed (vcharScribeSystem (*listitems));
	comboList->AppendValue (value);
	listitems++;
    }

    result = RESULT_CANCEL;
    resultstring [0] = 0;

    // Pop it up and handle events
    Open ();
    comboList->SelectValueAt (0);
    MakeTopMostDialog (this, NULL);
    // process events
    vevent::Process();

    // return answer
    return result;
}
