/* trendmacrodlg.cxx,v 1.1.1.1 1999/08/03 14:45:10 tis4000 Exp
 *
 */

/*
 * TrendMacroDialog - trend graph container class
 */

#include <vport.h>		/* for v*HEADER */
#include vstdlibHEADER		/* for random, exit, EXIT_SUCCESS */
#include vlistHEADER
#include vstartupHEADER
#include vmemHEADER
#include vprefHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcharHEADER
#include vcomboboxHEADER
#include vdialogHEADER

#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#endif WIN32

extern "C"
{
    #include "trendutils.h"
}
#include "trenddocs.h"
#include "trendmacrodlg.h"
#include "tisgalaxylib.hxx"

// Help functions
class tmAcceptButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tmAcceptButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tmAcceptButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendMacroDialog *p) { parent = p; }
private:
    TrendMacroDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tmAcceptButton, vbutton, NULL);
void tmAcceptButton::ObserveDialogItem (vevent *event)
{
    parent->AcceptClick();
}

class tmCancelButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tmCancelButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tmCancelButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendMacroDialog *p) { parent = p; }
private:
    TrendMacroDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tmCancelButton, vbutton, NULL);
void tmCancelButton::ObserveDialogItem (vevent *event)
{
    parent->CancelClick();
}

// macro definitions for vkindtyped overrides
vkindTYPED_FULL_DEFINITION(TrendMacroDialog, vdialog, NULL);

// vloadable
void TrendMacroDialog::LoadInit(vresource resource)
{
    // get handles to interesting elements
    sourceText = vdialogLabelItem::RequestSubLoad (this, vnameInternGlobalLiteral("SourceTextTag"));
    substText = vtextitem::RequestSubLoad (this, vnameInternGlobalLiteral("SubstTextTag"));
    accept = tmAcceptButton::RequestSubLoad (this, vnameInternGlobalLiteral("OKButtonTag"));
    cancel = tmCancelButton::RequestSubLoad (this, vnameInternGlobalLiteral("CancelButtonTag"));

    vdialog::LoadInit(resource);

    accept->SetParent (this);
    cancel->SetParent (this);
}

/*********
 *  TrendMacroDialog::ObserveWindow()
 *
 *  The dialog notification procedure gets called whenever any 
 *  major event applies to the dialog as a whole.  We need to
 *  look for the event which tells us that the dialog has
 *  closed.
 *
 **********/
void TrendMacroDialog::ObserveWindow (vevent *event)
{
    result = RESULT_CANCEL;
    if (event->GetType() == veventWINDOW_STATE && event->IsClose ())
    {
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}

/*********
 *  TrendMacroDialog::AcceptClick()
 *
 *  The procedure gets called whenever the accept button is clicked 
 *
 **********/
void TrendMacroDialog::AcceptClick ()
{
    char    temptext[80];
    char    *macptr;

    substText->GetText ((unsigned char *)temptext, sizeof(temptext));
    AddSym (macrotext, temptext);

    do { /* find the next unresolved macro in the string */
	/* find a macro */
	while (*srcptr && (*srcptr != '$' || *(srcptr+1) != '('))
	    srcptr++;
	if (*srcptr)
	    srcptr = srcptr+2;
	/* copy the macroname into macrotext */
	macptr = macrotext;
	while (*srcptr && (*srcptr != ')'))
	    *macptr++ = *srcptr++;
	*macptr = 0;
    } while (*srcptr && LookupSym (macrotext));

    if (*srcptr)
    {
	srcptr++;
	sprintf (temptext, "~Define substitution for: %s", macrotext);
	sourceText->SetTitle ((unsigned char *)temptext);
	// blank the substText item
	substText->SetText ((unsigned char *)"");
    }
    else
    {
	result = RESULT_OK;
	if (IsOpen())
	{
	    Close ();
	    vevent::StopProcessing ();
	}
    }
}

/*********
 *  TrendMacroDialog::CancelClick()
 *
 *  The procedure gets called whenever the cancel button is clicked 
 *
 **********/
void TrendMacroDialog::CancelClick ()
{
    result = RESULT_CANCEL;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

int TrendMacroDialog::Alert (char *tmdtitle, char *src)
{
    char    temptext[132];
    char    *macptr;

    strncpy (srcstring, src, sizeof(srcstring));
    srcptr = srcstring;
    // load the source text string
    do { /* find the next unresolved macro in the string */
	/* find a macro */
	while (*srcptr && (*srcptr != '$' || *(srcptr+1) != '('))
	    srcptr++;
	if (*srcptr)
	    srcptr = srcptr+2;
	/* copy the macroname into tempstring */
	/* copy the macroname into macrotext */
	macptr = macrotext;
	while (*srcptr && (*srcptr != ')'))
	    *macptr++ = *srcptr++;
	*macptr = 0;
    } while (*srcptr && LookupSym (macrotext));
    if (*srcptr)
    {
	sprintf (temptext, "~Define substitution for: %s", macrotext);
	sourceText->SetTitle ((unsigned char *)temptext);
	// blank the substText item
	substText->SetText ((unsigned char *)"");

	// Pop it up and handle events
	Open ();
	MakeTopMostDialog (this, NULL);
	// process events
	vevent::Process();
    }
    else
    {
	result = RESULT_OK;
    }

    // return answer
    return result;
}
