/* trendprefsdlg.cxx,v 1.1.1.1 1999/08/03 14:45:10 tis4000 Exp
 *
 */

/*
 * TrendPrefsDialog - trend graph container class
 */

#include <vport.h>		/* for v*HEADER */
#include vstdlibHEADER		/* for random, exit, EXIT_SUCCESS */
#include vlistHEADER
#include vstartupHEADER
#include vmemHEADER
#include vprefHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcharHEADER
#include vcomboboxHEADER
#include vdialogHEADER

#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#endif WIN32

extern "C"
{
    #include "trendutils.h"
}
#include "trenddocs.h"
#include "trendprefsdlg.h"
#include "tisgalaxylib.hxx"

// Helper functions
class tpColorButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tpColorButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tpColorButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendPrefsDialog *p, int ix) { parent = p; index = ix;}
private:
    TrendPrefsDialog *parent;
    int index;
};

vkindTYPED_FULL_DEFINITION(tpColorButton, vbutton, NULL);
void tpColorButton::ObserveDialogItem (vevent *event)
{
    parent->AdjustColorClick (index);
}

class tpSlider : public vslider {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tpSlider);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tpSlider, vslider);
  
  // virtual member function overrides
  int HandleDrag(vevent *event);

  void SetParent (TrendPrefsDialog *p, int ix) { parent = p; index = ix;}
private:
    TrendPrefsDialog *parent;
    int index;
};

vkindTYPED_FULL_DEFINITION(tpSlider, vslider, NULL);
int tpSlider::HandleDrag(vevent *event)
{
    vslider::HandleDrag (event);
    parent->SliderChange (index, GetValue ());
    return vTRUE;
}

class tpAcceptButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tpAcceptButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tpAcceptButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendPrefsDialog *p) { parent = p; }
private:
    TrendPrefsDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tpAcceptButton, vbutton, NULL);
void tpAcceptButton::ObserveDialogItem (vevent *event)
{
    parent->AcceptClick();
}

class tpCancelButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tpCancelButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tpCancelButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendPrefsDialog *p) { parent = p; }
private:
    TrendPrefsDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tpCancelButton, vbutton, NULL);
void tpCancelButton::ObserveDialogItem (vevent *event)
{
    parent->CancelClick();
}

// macro definitions for vkindtyped overrides
vkindTYPED_FULL_DEFINITION(TrendPrefsDialog, vdialog, NULL);

// vloadable
void TrendPrefsDialog::LoadInit(vresource resource)
{
    // get handles to interesting elements
    colorButton[0] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend1Tag"));
    colorButton[1] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend2Tag"));
    colorButton[2] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend3Tag"));
    colorButton[3] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend4Tag"));
    colorButton[4] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend5Tag"));
    colorButton[5] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend6Tag"));
    colorButton[6] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend7Tag"));
    colorButton[7] = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("Trend8Tag"));
    backButton = tpColorButton::RequestSubLoad (this, vnameInternGlobalLiteral("BackgroundTag"));
    redSlider = tpSlider::RequestSubLoad (this, vnameInternGlobalLiteral("RedSliderTag"));
    greenSlider = tpSlider::RequestSubLoad (this, vnameInternGlobalLiteral("GreenSliderTag"));
    blueSlider = tpSlider::RequestSubLoad (this, vnameInternGlobalLiteral("BlueSliderTag"));
    accept = tpAcceptButton::RequestSubLoad (this, vnameInternGlobalLiteral("AcceptButtonTag"));
    cancel = tpCancelButton::RequestSubLoad (this, vnameInternGlobalLiteral("CancelButtonTag"));

    vdialog::LoadInit(resource);

    SetPopup (TRUE);
    for (int i = 0; i < 8; i++)
    {
	colorButton[i]->SetParent (this, i);
    }
    backButton->SetParent (this, -1);
    redSlider->SetParent (this, 0);
    greenSlider->SetParent (this, 1);
    blueSlider->SetParent (this, 2);
    accept->SetParent (this);
    cancel->SetParent (this);
}

/*********
 *  TrendPrefsDialog::ObserveWindow()
 *
 *  The dialog notification procedure gets called whenever any 
 *  major event applies to the dialog as a whole.  We need to
 *  look for the event which tells us that the dialog has
 *  closed.
 *
 **********/
void TrendPrefsDialog::ObserveWindow (vevent *event)
{
    result = RESULT_CANCEL;
    if (event->GetType() == veventWINDOW_STATE && event->IsClose ())
    {
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}

/*********
 *  TrendPrefsDialog::AdjustColorClick()
 *
 *  The procedure gets called whenever a color button is clicked 
 *
 **********/
void TrendPrefsDialog::AdjustColorClick (int ix)
{
    double red;
    double green;
    double blue;
    
    if (ix == -1)
	wbackSpec.FGetRGB (&red, &green, &blue);
    else
	wcolorSpec[ix].FGetRGB (&red, &green, &blue);
    
    // load the slider values from the color button.
    redSlider->SetValueScroll ((int)(red * 255));
    greenSlider->SetValueScroll ((int)(green * 255));
    blueSlider->SetValueScroll ((int)(blue * 255));
    // remember active color index
    activeColor = ix;
}

/*********
 *  TrendPrefsDialog::SliderChange()
 *
 *  The procedure gets called whenever a slider is dragged 
 *
 **********/
void TrendPrefsDialog::SliderChange (int type, int value)
{
    double red;
    double green;
    double blue;
    
    if (activeColor == -1)
	wbackSpec.FGetRGB (&red, &green, &blue);
    else
	wcolorSpec[activeColor].FGetRGB (&red, &green, &blue);

    // load the slider values from the color button.
    switch (type)
    {
    case 0:
	red = value / 255.;
	break;
    case 1:
	green = value / 255.;
	break;
    case 2:
	blue = value / 255.;
	break;
    }

    if (activeColor == -1)
    {
	wbackSpec.FSetRGB (red, green, blue);
	wback = vcolor::Intern (&wbackSpec);
	backButton->SetForeground (wback);
    }
    else
    {
	wcolorSpec[activeColor].FSetRGB (red, green, blue);
	wcolors[activeColor] = vcolor::Intern (&wcolorSpec [activeColor]);
	colorButton[activeColor]->SetForeground (wcolors [activeColor]);
    }
}

/*********
 *  TrendPrefsDialog::AcceptClick()
 *
 *  The procedure gets called whenever the accept button is clicked 
 *
 **********/
void TrendPrefsDialog::AcceptClick ()
{
    result = RESULT_OK;
    
    // get trend set values and store them (abort if unaccepted overwrite! 
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

/*********
 *  TrendPrefsDialog::CancelClick()
 *
 *  The procedure gets called whenever the cancel button is clicked 
 *
 **********/
void TrendPrefsDialog::CancelClick ()
{
    result = RESULT_CANCEL;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

int TrendPrefsDialog::Alert (vcolorSpec colorSpec[8], vcolorSpec backSpec)
{
    int			i;

    // if the combo is open,close it make the build faster
    if (IsOpen ())
	Close ();
    for (i = 0; i < 8; i++)
    {
	wcolorSpec[i] = colorSpec[i];
	wcolors[i] = vcolor::Intern (&wcolorSpec [i]);
	colorButton[i]->SetForeground (wcolors[i]);
    }
    wbackSpec = backSpec;
    wback = vcolor::Intern (&wbackSpec);
    backButton->SetForeground (wback);

    result = RESULT_CANCEL;
    AdjustColorClick (0);

    // Pop it up and handle events
    Open ();
    MakeTopMostDialog (this, NULL);

    // process events
    vevent::Process();

    // return answer
    return result;
}
