/* trendselectdlg.cxx,v 1.1.1.1 1999/08/03 14:45:11 tis4000 Exp
 *
 */

/*
 * TrendSelectDialog - trend set editor dialog class
 */

#include <vport.h>		/* for v*HEADER */
#include vstdlibHEADER		/* for random, exit, EXIT_SUCCESS */
#include vlistHEADER
#include vlistviewHEADER
#include vstartupHEADER
#include vmemHEADER
#include vprefHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcharHEADER
#include vdialogHEADER
#include vcomboboxHEADER
#include vnumHEADER

#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#endif WIN32

extern "C"
{
    #include "trendutils.h"
    extern char    * RedStrings[];
}
#include "trenddocs.h"
#include "trendselectdlg.h"
#include "tisgalaxylib.hxx"

// Help functions
class tselAcceptButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tselAcceptButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tselAcceptButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendSelectDialog *p) { parent = p; }
private:
    TrendSelectDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tselAcceptButton, vbutton, NULL);
void tselAcceptButton::ObserveDialogItem (vevent *event)
{
    parent->AcceptClick();
}

class tselCancelButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tselCancelButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tselCancelButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendSelectDialog *p) { parent = p; }
private:
    TrendSelectDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tselCancelButton, vbutton, NULL);
void tselCancelButton::ObserveDialogItem (vevent *event)
{
    parent->CancelClick();
}

// macro definitions for vkindtyped overrides
vkindTYPED_FULL_DEFINITION(TrendSelectDialog, vdialog, NULL);

// vloadable
void TrendSelectDialog::LoadInit(vresource resource)
{
    // get handles to interesting elements
    selectTrend = vcontainerExclusiveGroup::RequestSubLoad (this, vnameInternGlobalLiteral("SelectGroupTag"));
    startType = vcontainerExclusiveGroup::RequestSubLoad (this, vnameInternGlobalLiteral("FromGroupTag"));
    accept = tselAcceptButton::RequestSubLoad (this, vnameInternGlobalLiteral("AcceptButtonTag"));
    cancel = tselCancelButton::RequestSubLoad (this, vnameInternGlobalLiteral("CancelButtonTag"));

    vdialog::LoadInit(resource);

    accept->SetParent (this);
    cancel->SetParent (this);
}

/*********
 *  TrendSelectDialog::ObserveWindow()
 *
 *  The dialog notification procedure gets called whenever any 
 *  major event applies to the dialog as a whole.  We need to
 *  look for the event which tells us that the dialog has
 *  closed.
 *
 **********/
void TrendSelectDialog::ObserveWindow (vevent *event)
{
    result = RESULT_CANCEL;
    if (event->GetType() == veventWINDOW_STATE && event->IsClose ())
    {
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}

/*********
 *  TrendSelectDialog::AcceptClick()
 *
 *  The procedure gets called whenever the accept button is clicked 
 *
 **********/
void TrendSelectDialog::AcceptClick ()
{
    searchStruct.type[0] = '0' + selectTrend->GetItemIndex (selectTrend->GetSelection());
    if (!startType->GetItemIndex (startType->GetSelection()))
	searchStruct.type[1] = 'C';
    else
	searchStruct.type[1] = 'S';
    searchStruct.type[2] = 0;

    // collect the search characters!

    /* determine which ones need to be removed */
    int statix = 0;
    searchStruct.status[statix] = 0;

    // collect the severity characters! 
    int sevix = 0;
    searchStruct.severity[sevix] = 0;

    result = RESULT_OK;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

/*********
 *  TrendSelectDialog::CancelClick()
 *
 *  The procedure gets called whenever the cancel button is clicked 
 *
 **********/
void TrendSelectDialog::CancelClick ()
{
    result = RESULT_CANCEL;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

int TrendSelectDialog::Alert (TrendSet *trendSet)
{
    int itemIx;
    int itemCnt;
    vdialogItemList *items;
    vcontrol *item;

    // place all values into fields
    if (trendSet == NULL)
    {
	// init a blank trendset
	strcpy (workset.setname, "");
	workset.autoupdate = 0;
	workset.zoom = 60;
	workset.numTrends = 0;
    }
    else
    {
	workset = *trendSet;
    }
    if (selectTrend->GetItemIndex (selectTrend->GetSelection()) > workset.numTrends)
	selectTrend->SetSelection (selectTrend->GetItemAt (0));
    /* disable unavailable traces */
    items = selectTrend->GetItemList ();
    for (itemIx = 0, itemCnt = items->GetCount(); itemIx < itemCnt; itemIx++)
    {
	item = (vcontrol *)items->GetAt(itemIx);
	if (itemIx < workset.numTrends)
	    item->Enable();
	else
	    item->Disable();
    }

    result = RESULT_CANCEL;

    // Pop it up and handle events
    Open ();
    MakeTopMostDialog (this, NULL);
    // process events
    vevent::Process();

    // return answer
    return result;
}
