/* trendvaluedlg.cxx,v 1.1.1.1 1999/08/03 14:45:11 tis4000 Exp
 *
 */

/*
 * TrendValueDialog - trend graph container class
 */

#include <vport.h>		/* for v*HEADER */
#include vstdlibHEADER		/* for random, exit, EXIT_SUCCESS */
#include vlistHEADER
#include vstartupHEADER
#include vmemHEADER
#include vprefHEADER
#include vtextitemHEADER
#include vbuttonHEADER
#include vcharHEADER
#include vcomboboxHEADER
#include vdialogHEADER

#ifdef WIN32
#include <windows.h>
#include <winbase.h>
#include <stdio.h>
#endif WIN32

extern "C"
{
    #include "trendutils.h"
}
#include "trenddocs.h"
#include "trendvaluedlg.h"
#include "tisgalaxylib.hxx"

// Help functions
class tvAcceptButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tvAcceptButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tvAcceptButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendValueDialog *p) { parent = p; }
private:
    TrendValueDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tvAcceptButton, vbutton, NULL);
void tvAcceptButton::ObserveDialogItem (vevent *event)
{
    parent->AcceptClick();
}

class tvCancelButton : public vbutton {
public:
  // macro for overrides from vkindTyped
  vkindTYPED_FULL_DECLARATION(tvCancelButton);
  
  // macro for the standard versions of the 
  // necessary constructors and destructors
  vloadableINLINE_CONSTRUCTORS(tvCancelButton, vbutton);
  
  // virtual member function overrides
  void ObserveDialogItem(vevent *event);

  void SetParent (TrendValueDialog *p) { parent = p; }
private:
    TrendValueDialog *parent;
};

vkindTYPED_FULL_DEFINITION(tvCancelButton, vbutton, NULL);
void tvCancelButton::ObserveDialogItem (vevent *event)
{
    parent->CancelClick();
}

// macro definitions for vkindtyped overrides
vkindTYPED_FULL_DEFINITION(TrendValueDialog, vdialog, NULL);

// vloadable
void TrendValueDialog::LoadInit(vresource resource)
{
    // get handles to interesting elements
    promptText = vdialogLabelItem::RequestSubLoad (this, vnameInternGlobalLiteral("PromptTextTag"));
    valueText = vtextitem::RequestSubLoad (this, vnameInternGlobalLiteral("ValueTextTag"));
    accept = tvAcceptButton::RequestSubLoad (this, vnameInternGlobalLiteral("OKButtonTag"));
    cancel = tvCancelButton::RequestSubLoad (this, vnameInternGlobalLiteral("CancelButtonTag"));

    vdialog::LoadInit(resource);

    accept->SetParent (this);
    cancel->SetParent (this);
}

/*********
 *  TrendValueDialog::ObserveWindow()
 *
 *  The dialog notification procedure gets called whenever any 
 *  major event applies to the dialog as a whole.  We need to
 *  look for the event which tells us that the dialog has
 *  closed.
 *
 **********/
void TrendValueDialog::ObserveWindow (vevent *event)
{
    result = RESULT_CANCEL;
    if (event->GetType() == veventWINDOW_STATE && event->IsClose ())
    {
	vevent::StopProcessing ();
    }
    else
    {
	MakeTopMostDialog (this, event);
    }
}

/*********
 *  TrendValueDialog::AcceptClick()
 *
 *  The procedure gets called whenever the accept button is clicked 
 *
 **********/
void TrendValueDialog::AcceptClick ()
{
    valueText->GetText ((unsigned char *)resultString, sizeof(resultString));
    result = RESULT_OK;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

/*********
 *  TrendValueDialog::CancelClick()
 *
 *  The procedure gets called whenever the cancel button is clicked 
 *
 **********/
void TrendValueDialog::CancelClick ()
{
    result = RESULT_CANCEL;
    if (IsOpen())
    {
	Close ();
	vevent::StopProcessing ();
    }
}

int TrendValueDialog::Alert (char *prompt, char *value)
{
    promptText->SetTitle ((unsigned char *)prompt);
    valueText->SetText ((unsigned char *)value);

    // Pop it up and handle events
    Open ();
    MakeTopMostDialog (this, NULL);
    // process events
    vevent::Process();

    // return answer
    return result;
}
