/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <stdio.h>
#include <string.h>

#include "ChannelList.h"

// Static functions in this module 

static void connectHandler (struct connection_handler_args args);
static void eventHandler (struct event_handler_args args);

// External functions

extern "C"
{
#include "cadef.h"
#include "tisAlarm.h"
}
static	ChannelList	channelList;
extern	int		debugLevel;

//----------
void clInitialize (void)

    {

	channelList.numChannels = 0;

    }

//----------
void clTerminate (void)

    {

	for (int i=0 ; i<channelList.numChannels ; i++)
	    ca_clear_channel (channelList.chans [i].chanId);

    }

//----------
int clAdd (char *channelName)

    {

	/* Check to see if the channel is already in list */

	for (int i=0 ; i<channelList.numChannels ; i++)
	    if (strcmp (channelName, channelList.chans [i].chanName) == 0)
		return i;

	/* Place the channel in the list */

	strcpy (channelList.chans [channelList.numChannels].chanName, channelName);
	channelList.chans [channelList.numChannels].connected = 0;
	channelList.chans [channelList.numChannels].gotData = 0;
	return channelList.numChannels++;

    }

//----------
void clConnect (void)	/* Connect any channels that are not already */

    {

	for (int i=0 ; i<channelList.numChannels ; i++) {
	    ca_build_and_connect (channelList.chans [i].chanName, TYPENOTCONN, 0, 
				&channelList.chans [i].chanId, NULL, connectHandler, (void *)i);
	}

    }

//----------
static void connectHandler (struct connection_handler_args args)

    {

	/* Find the channel that has connected/disconnected */

	for (int i=0 ; i<channelList.numChannels ; i++)
	    if (args.chanid == channelList.chans [i].chanId)
		break;

	/* Channel has connected, now set callback for event. */

	if (args.op == CA_OP_CONN_UP) {
	    if (debugLevel >= 2)
		printf ("%s connected.\n",  channelList.chans [i].chanName);
	    ca_add_event (OLD_DBR_SHORT, args.chanid, eventHandler,
			(void *)&channelList.chans [i],
			&channelList.chans [i].eventId);
	    channelList.chans [i].connected = 1;
	} else {
	    if (debugLevel >= 2)
		printf ("%s disconnected.\n",  channelList.chans [i].chanName);
	
	    channelList.chans [i].connected = 0;
	}
    
    }

//----------
static void eventHandler (struct event_handler_args args)

    {

	((Channel *)args.usr)->value = *(short *)args.dbr;
	((Channel *)args.usr)->gotData = 1;

	if (debugLevel >= 2)
	    printf ("%s changed to %d\n", ((Channel *)args.usr)->chanName,
			((Channel *)args.usr)->value);
    }

//----------
int clGetValue (int i)
{

	return channelList.chans [i].value;

}

//----------
void clPutValue (int i, int value)
{
//	static char	offString [] = "OFF", onString [] = "ON";
//	ca_put (OLD_DBR_STRING, channelList.chans [i].chanId, value ? onString : offString);
	short	v = value;
	ca_put (OLD_DBR_SHORT, channelList.chans [i].chanId, &v);
}

//----------
int clValid (int i)
{
    return channelList.chans [i].connected && channelList.chans [i].gotData;
}
