/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include   <stdio.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32

#include "pvmap.h"
#include "accum.h"
#include "msgMon.h"

/* global variables */

C4CODE	cb;
D4DATA	*pvmap, *accum;
F4FIELD	*pvmapName, *pvmapType, *pvmapPhase, *accumName, *accumSvrt;
char 	*accumId = "accum         ";

/************************************************************************************/
/************************************************************************************/

void	usage()
{
	printf ("Can not execute OrderAccumPoints invalid input\n");
	printf ("The SaveRunTime input parameter must be between 0 and 60\n");
	exit(1);
}
/************************************************************************************/
/************************************************************************************/

int   validateInput(int num, char *input)
{
int    svrt = -1;


	if (num == 2)
		svrt = atoi(input);

	if (svrt > 0 && svrt <= 60)
		return svrt;
	else if (svrt  == 0)
		/* insure that actual input was 0 */
		if (*input != '0')
			return -1;
		else
			return svrt;
	else
		return -1;
}
/***********************************************************************************/
/***********************************************************************************/

void initdb()
{
char    dbname[300];

	d4init(&cb);
#if defined(USECB63) || defined(WIN32)
	cb.errOpen = 0;
#else
	cb.open_error = 0;
#endif USECB63

	/* open the database files */

	memset(dbname, 0, sizeof dbname);
#ifdef WIN32
	resolveLinks("\\tis4000db\\pvmap.dbf", dbname);
#else
	resolveLinks("/tis4000db/pvmap.dbf", dbname);
#endif WIN32
	if (!(pvmap = d4open( &cb, dbname)) ) {
		tisSysLog (0,0,"Can not open %s\n", dbname);
		exit(1);
        }

	memset(dbname, 0, sizeof dbname);
#ifdef WIN32
	resolveLinks ("\\tis4000db\\accum.dbf", dbname);
#else
	resolveLinks ("/tis4000db/accum.dbf", dbname);
#endif WIN32
	if (!(accum = d4open( &cb, dbname)) ) {
		tisSysLog (0,0,"Can not open %s\n", dbname);
		closedb();
		exit(1);
        }

	/* select the table tags */
	d4tag_select(pvmap, d4tag(pvmap, "NAME"));
	d4tag_select(accum, d4tag(accum, "NAME"));

	/* open the needed fields */
	pvmapName = d4field(pvmap, "NAME");
	pvmapType = d4field(pvmap, "TYPE");
	pvmapPhase = d4field(pvmap, "PHASE");
	accumName = d4field(accum, "NAME");
	accumSvrt = d4field(accum, "SAVRUNTIME");

	d4top(pvmap);
	d4top(accum);
}

/*************************************************************************************/
/*************************************************************************************/
closedb()
{
	if (pvmap) {
		d4unlock_all(pvmap);
		d4close(pvmap);
	}
	if (accum) {
		d4unlock_all(accum);
		d4close(accum);
	}
}

/*************************************************************************************
	For every accum point in pvmap assign to the phase field a number between
	0 and 59.  This number is assigned in increasing order with the first accum
	point incountered being assigned 0 and so on.  When 59 has been assigned the
	cycle starts a 0 again.  Each accum point in pvmap is found in accum.dbf and
	the input parameter is assigned to the SAVRUNTIME field.
*************************************************************************************/
void updatePoints(int svrt)
{
int order = 0;

	/* examine every point in pvmap for the accum type */

	while (!d4eof(pvmap)) {

		if (order > 59)
			/* cycle from 0 - 59 */
			order = 0;

		if ( !d4deleted(pvmap) &&
		     (strncmp(accumId, f4str(pvmapType), 14) == 0) ){

			/* this point is an accum type */

			f4assign_int(pvmapPhase, order);
			++order;

			if (d4seek(accum, f4str(pvmapName)) == 0)
				/* update SAVRUNTIME value in accum.dbf */
				f4assign_int(accumSvrt, svrt);

			d4top(accum);
		}

		d4unlock_all(pvmap);
		d4unlock_all(accum);
		d4skip(pvmap, 1);
	}
}
/*************************************************************************************/
/*************************************************************************************/
	
void main(int argc, char *argv[])
{
    int	svrt;

    tisOpenLog ("OrderAccumPoints", 6, 2, NULL, NULL);

    if ((svrt = validateInput(argc, argv[1])) < 0)
	usage();

    initdb();

    updatePoints(svrt);

    closedb();
}
