/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#if defined(WIN32) || defined(USECB63)
#include "cb45to63.h"
#endif WIN32
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#endif WIN32
#ifdef	WIN32
#    include <search.h>
#    include "NTUnixSubs.h"
     extern char *optarg;
#endif WIN32
#include "resolveLinks.h"


#define	PATH_NAME_SIZE	100
typedef	char	PathName [PATH_NAME_SIZE+1];

FILE		*outFile;
PathName	dbDir, specFileName;
CODE4		codebase;

/* Array of structures to contain point types and fields */

typedef struct fieldentry {
    struct fieldentry	*next;
    char	fieldName [5];		/* Actual field name */
    int		inOrOutLink;		/* Field is an INLINK ot OUTLINK */
    int		hwFlag;			/* Is a hardware thing */
    char	table [20];		/* name of database file */
    char	dbFieldName [12];	/* Field within the table */
    char	operandId [21];		/* Operand id for array type things */
} FieldNode;

#define	MAX_TYPES	100
typedef struct {
    char	pointType [20];
    FieldNode	*fieldList;
} PointTypeEntry;
static	PointTypeEntry	pointTypeList [MAX_TYPES];
static	int		numTypes = 0;

/* Table of open tables */

#define	MAX_TABLES	100
typedef struct {
	char	tableName [20];
	DATA4	*handle;
} TableListEntry;
static	TableListEntry	tableList [MAX_TABLES];
static	int	numTables = 0;

static	int	dougMode = 0;

/* Functions defined in this module */

static	void	getFieldLists (void);
	int	sortCompare (const void *, const void *);
static	void	getLinks (void);
static	DATA4	*getTable (char *);
static	void	closeAllTables (void);
static	PointTypeEntry *getPointType (char *);
	int	searchCompare (const void *, const void *);
static	void	stripBlanks (char *);
static	void	makeLower (char *);

/*----------*/
void main (int argc, char *argv [])

    {

	int		c;
	extern	char	*optarg;

	/* Set up the defaults */

	strcpy (specFileName, "linkfields");
#ifdef WIN32
	strcpy (dbDir, "\\tis4000db\\");
#else
	strcpy (dbDir, "/tis4000db/");
#endif WIN32
	outFile = stdout;

	/* Process command line options */

	while ((c = getopt (argc, argv, "f:p:o:hD")) != -1)
	    switch (c) {
		case 'f':	/* Specification file name */
		    strcpy (specFileName, optarg);
		    break;
		case 'p':	/* Database directory */
		    strcpy (dbDir, optarg);
		    break;
		case 'o':	/* Output file */
		    outFile = fopen (optarg, "w");
		    if (outFile == NULL) {
			fprintf (stderr, "Unable to create %s\n", optarg);
			exit (-1);
		    }
		    break;
		case 'D':
		    dougMode = 1;
		    break;
		case 'h':
		case '?':
		    fprintf (stderr, "Usage: pvXref [-f <specFile>] [-o <outputfile>] [-p <directory>]\n");
		    fprintf (stderr, "   Default <specFile> is \"linkfields\"\n");
		    exit (0);
	    }

	/* Initialize the codebase system */

	d4init (&codebase);

	/* Get the field lists */

	getFieldLists ();

	/* Get the link points */

	getLinks ();

    }

/*----------*/
static void getFieldLists (void)

    {

	DATA4		*xref;
	FIELD4		*xrefFieldNameFld, *xrefTableNameFld, *xrefOperandIdFld;
	PathName	path, dbPath;
	FILE		*specFile;
	char		line [200], pointType [20], key [50], *p;
	FieldNode	*field;
	int		i, isInOrOut;

	/* Open the spec file */

	specFile = fopen (specFileName, "r");
	if (specFile == NULL) {
	    fprintf (stderr, "Unable to open specification file %s\n", specFileName);
	    exit (-1);
	}

	/* Open up the pdbxref database. This maps point types and their fields into */
	/* the correcponding dBase file and field. */

	d4init (&codebase);
#if defined(USECB63) || defined(WIN32)
	codebase.errTagName = 0;
#else
	codebase.tag_name_error = 0;
#endif	USECB63 or WIN32

	sprintf (path, "%spdbxref", dbDir);
	resolveLinks (path, dbPath);
	xref = d4open (&codebase, dbPath);
	if (xref == NULL) {
	    fprintf (stderr, "Unable to open %s\n", dbPath);
	    exit (-1);
	}
	xrefFieldNameFld = d4field (xref, "FIELD_NAME");
	xrefTableNameFld = d4field (xref, "TABLE");
	xrefOperandIdFld = d4field (xref, "OPERANDID");
	d4tag_select (xref, d4tag (xref, "XREF"));

	/* Process each record type */

	while (fgets (line, sizeof (line), specFile)) {

	   if (strlen (line) < 3)
		continue;

	    /* Strip the newline */

	    line [strlen (line) - 1] = '\0';

	    /* Get the point type and create an entry for it */

	    strcpy (pointType, strtok (line, ":"));
	    strcpy (pointTypeList [numTypes].pointType, pointType);
	    pointTypeList [numTypes].fieldList = NULL;

	    /* Within each point type, go through and find the fields that contain the */
	    /* names of other points. */

	    for (;;) {

		/* Get the next field name */

		p = strtok (NULL, ",");
		if (p == NULL)
		    break;

		/* Check if the field is an INLINK or OUTLINK */

		isInOrOut = 0;
		i = strlen (p) - 1;
		if (*(p + i) == '*') {
		    isInOrOut = 1;
		    *(p + i) = '\0';
		}

		/* Look up the type and field in pdbxref */

		sprintf (key, "%-15s%-4s", pointType, p);
		if (d4seek (xref, key) != 0) {
		fprintf (stderr, "pdbxref does not contain an entry for point type %s and field type %s\n",
					pointType, p);
		    d4close_all (&codebase);
		    exit (-1);
		}

		/* Create a field entry if not a reference to hardware */


		field = (FieldNode *) malloc (sizeof (FieldNode));
		if (field == NULL) {
		    fprintf (stderr, "Out of memory!\n");
		    exit (-1);
		}
		field->inOrOutLink = isInOrOut;
		strcpy (field->fieldName, p);
		strcpy (field->table, f4str (xrefTableNameFld));
		stripBlanks (field->table);
		makeLower (field->table);
		strcpy (field->dbFieldName, f4str (xrefFieldNameFld));
		stripBlanks (field->dbFieldName);
		strcpy (field->operandId, f4str (xrefOperandIdFld));
		stripBlanks (field->operandId);

		/* Link field entry onto point type's field list */

		field->next = pointTypeList [numTypes].fieldList;
		pointTypeList [numTypes].fieldList = field;

	    }

	    numTypes++;

	}

	/* Close the files */

	fclose (specFile);
	d4close (xref);

	/* Sort the pointTypeList */
#ifdef	WIN32
	qsort ((char *)pointTypeList, numTypes, sizeof (PointTypeEntry), 
			    (int (__cdecl *)(const void *, const void *))sortCompare);
#else
	qsort ((char *)pointTypeList, numTypes, sizeof (PointTypeEntry), sortCompare);
#endif	WIN32

/*	for (i=0 ; i<numTypes ; i++) {
	    printf ("%s:\n", pointTypeList [i].pointType);
	    field = pointTypeList [i].fieldList;
	    while (field) {
		printf ("   %s,%s,%s,%s\n", field->fieldName, field->table, field->dbFieldName,
				field->operandId);
		field = field->next;
	    }
	}
*/
    }

/*----------*/
int sortCompare (const void *e1, const void *e2)

    {

	return strcmp (((PointTypeEntry *)e1)->pointType, ((PointTypeEntry *)e2)->pointType);

    }

/*----------*/
static void getLinks (void)

    {

	DATA4		*pvMap, *handle;
	FIELD4		*pointNameFld, *deleteFlagFld, *pointTypeFld, *fld;
	long		i, numRecs;
	PointTypeEntry	*p;
	char		pointName [50], pointType [50], linkPoint [50];
	FieldNode	*field;
	char		key [100];
	FILE		*tmpFile;
	char		line [200], line2 [200], lastName [50];

	/* Create the temporary output file */

	tmpFile = fopen ("/tmp/pvXref.tmp", "w");
	if (tmpFile == NULL) {
	    fprintf (stderr, "Unable to create temporary file \"pvXref.tmp\"\n");
	    exit (-1);
	}

	/* Open up pvmap and set up it's fields */

	pvMap = getTable ("pvmap");
	pointNameFld = d4field (pvMap, "NAME");
	deleteFlagFld = d4field (pvMap, "DELETEFLAG");
	pointTypeFld = d4field (pvMap, "TYPE");

	/* Go through all the pvmap entries and process the non-deleted ones. We have to */
	/* use specific record numbers since pvmap may be repositioned while fetching the */
	/* link values. */

	numRecs = d4reccount (pvMap);
	for (i=1 ; i<=numRecs ; i++) {
	    d4go (pvMap, i);
	    if (!f4true (deleteFlagFld)) {

		/* Get the point name */

		strcpy (pointName, f4str (pointNameFld));

		/* Get the point type and find it's entry in the pointTypeTable */

		strcpy (pointType, f4str (pointTypeFld));
		stripBlanks (pointType);
		p = getPointType (pointType);
		if (p == NULL) {
		    fprintf (stderr, "Unknown point type \"%s\" for \"%s\"\n", pointType, pointName);
		    closeAllTables ();
		    exit (-1);
		}

		/* Sequence through the fields and get values for each */

		field = p->fieldList;
		while (field) {
		    handle = getTable (field->table);
		    fld = d4field (handle, field->dbFieldName);
		    sprintf (key, "%-28s%s", pointName, field->operandId);
		    if (d4seek (handle, key) == 0) {
			strcpy (linkPoint, f4str (fld));
			stripBlanks (linkPoint);
			if (field->inOrOutLink) {
			    if (strcmp (linkPoint, "REFERENCE") == 0) {
				strcpy (linkPoint, f4str (d4field (handle, "REFERENCE")));
				stripBlanks (linkPoint);
			    }
			}

			/* Lookup the point in pvmap to see if it is really a point */

			sprintf (key, "%-28s", linkPoint);
			if (d4seek (pvMap, key) != 0)
			    strcpy (linkPoint, "");

			if (strlen (linkPoint)) {
			    if (dougMode)
				fprintf (tmpFile, "%-28s %-28s %-20s %-20s\n",
					linkPoint, pointName, pointType, field->fieldName);
			    else
				fprintf (tmpFile, "%-28s %-28s %-20s %-20s\n",
					linkPoint, pointName, field->table, field->dbFieldName);
			}
		    }
		    field = field->next; 		
		}

	    }
	}

	/* Close all the database tables we opened. */

	closeAllTables ();

	/* Close the temp file */

	fclose (tmpFile);

	/* Sort the temporary file */

	system ("sort </tmp/pvXref.tmp >/tmp/pvXref.out.tmp");

	/* Open the temp output file, read it in and print it to the output file. */
	/* In the first column, the pointName only appears on the first entry to */
	/* which it applies. */

	tmpFile = fopen ("/tmp/pvXref.out.tmp", "r");
	if (tmpFile == NULL) {
	    fprintf (stderr, "Unable to open /tmp/pvXref.out.tmp\n");
	    exit (-1);
	}

	strcpy (lastName, "");
	while (fgets (line, sizeof (line), tmpFile)) {
	    line [strlen (line) - 1] = '\0';
	    strcpy (line2, line);
	    strcpy (pointName, strtok (line2, " "));
	    if (strcmp (pointName, lastName) == 0)
		memset (line, ' ', 28);
	    strcpy (lastName, pointName);
	    fprintf (outFile, "%s\n", line);
	}

	fclose (outFile);
	unlink ("/tmp/pvXref.tmp");
	unlink ("/tmp/pvXref.out.tmp");
	    	    
    }

/*----------*/
static DATA4 *getTable (char *name)

    /* Get the handle for a table. If not found open the table and add it to the list */

    {

	int		i;
	PathName	path, dbPath;
	TAG4		*tag;

	for (i=0 ; i<numTables ; i++)
	    if (strcmp (name, tableList [i].tableName) == 0)
		return tableList [i].handle;

	strcpy (tableList [numTables].tableName, name);
	sprintf (path, "%s%s", dbDir, name);
	resolveLinks (path, dbPath);
	tableList [numTables].handle = d4open (&codebase, dbPath);
	if (tableList [numTables].handle == NULL) {
	    fprintf (stderr, "Unable to open %s\n", dbPath);
	    exit (-1);
	}

	/* If an index called NAMEID exists, select it. */

	tag = d4tag (tableList [numTables].handle, "NAMEID");
	if (tag)
	    d4tag_select (tableList [numTables].handle, tag);

	return tableList [numTables++].handle;

    }

/*----------*/
static void closeAllTables (void)

    {

	int	i;

	for (i=0 ; i<numTables ; i++)
	    d4close (tableList [i].handle);

    }

/*----------*/
static PointTypeEntry *getPointType (char *type)

    {
#ifdef	WIN32
	return (PointTypeEntry *) bsearch (type, (char *)pointTypeList, numTypes, 
				           sizeof (PointTypeEntry), 
					(int (__cdecl *)(const void *, const void *))searchCompare);
#else
	return (PointTypeEntry *) bsearch (type, (char *)pointTypeList, numTypes, 
				           sizeof (PointTypeEntry), searchCompare);
#endif	WIN32
	
    }

/*----------*/
int searchCompare (const void *key, const void *elt)

    {

	return strcmp ((char *)key, ((PointTypeEntry *)elt)->pointType);

    }

/*----------*/
static void stripBlanks (char *buf)	/* Strip blanks from a string */

    {

	while (*buf) {
	    if (*buf == ' ') {
		*buf = '\0';
		return;
	    }
	    buf++;
	}
	
    }

/*----------*/
static void makeLower (char *s)

    {

	while (*s) {
	    *s = tolower (*s);
	    s++;
	}

    }

