/*synclist.c,v 1.1.1.1 1999/08/03 14:45:16 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/


/****************************************************************/
/*								*/
/*		NAME SERVER FOR CHANNEL ACCESS			*/
/*								*/
/*	This program resolves channel names to a list of	*/
/*	possible Internet addresses and a resource name.	*/
/*	Other services include returning a list of the		*/
/*	Internet address for all the IOCs that we know		*/
/*	and also returning a list of all the resources		*/
/*	that are in the system.					*/
/*								*/
/*	Endian issues: We may be a different endian from any	*/
/*	user.							*/
/*								*/
/*	Author: Gordon Uchenick					*/
/*								*/
/****************************************************************/

#ifndef AXP
#include <bcopy.h>
#endif
#ifndef WIN32
#include <sys/types.h>
#endif WIN32
#include <errno.h>
#ifdef AXP
#include <sys/time.h>
#else
#include <time.h>
#ifndef WIN32
#include <sys/time.h>
#endif WIN32
#endif

#ifdef WIN32
#pragma pack (push, 1)
#endif WIN32
#include "cb45to63.h"
#include "d4all.h"
#ifdef WIN32
#pragma pack (pop)
#include "NTUnixSubs.h"
#endif WIN32
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>


/****************************************************************/
/*								*/
/*		GLOBAL DATA FOR THIS PROCESS			*/
/*								*/
/****************************************************************/

extern int		errno;


C4CODE			codeBase;

D4DATA			*pvmap;
F4FIELD			*pvmapName;
F4FIELD			*pvmapResource;
F4FIELD			*pvmapCreationTime;
F4FIELD			*pvmapChecksum;
F4FIELD			*pvmapPointType;
F4FIELD			*pvmapIoc1Dnld;
F4FIELD			*pvmapIoc2Dnld;
F4FIELD			*pvmapDeleteFlag;
T4TAG			*pvmapTag;



/****************************************************************/
/*								*/
/*			FUNCTION PROTOTYPES			*/
/*								*/
/****************************************************************/

void trimCpy();


/****************************************************************/
/*								*/
/*			THE MAIN PROGRAM			*/
/*								*/
/****************************************************************/

void main(argc, argv)
int	argc;
char	**argv;
{
    char	pvName[41];
    unsigned short	checksum;
    short	printCrc = 1;
    short printDownload = 1;
    char	downloadFlag1;
    char	downloadFlag2;
    long	creation;
    char	pvRes[41];
    char	c;
    extern	char	*optarg;
    extern	int	optind;
	char defaultPath[1024];

/******
	See if the first option is -nocrc
******/
#ifndef WIN32
    strcpy(defaultPath, "/tis4000db/pvmap.dbf");
#else
	strcpy(defaultPath, "\\tis4000db\\pvmap.dbf");
#endif
	while ((c = getopt (argc, argv, "clp:")) != -1)
	    switch (c)
	      {
	      case 'c':	/* Do not print CRCs */
		printCrc = 0;
		break;
	      case 'l':	/* Do not print download status */
		printDownload = 0;
		break;
		  case 'p':
			if(*optarg != '-')
			  {
				strcpy(defaultPath,optarg);
				break;
			  }
			else
			  fprintf(stderr,"%s: option requires an argument -- %c\n",argv[0],c);
			break;
	      case '?':
		fprintf(stderr,"Usage is:\n");
		fprintf(stderr,"synclist -c -l\n");
		fprintf(stderr,"\t-c -- Do not print CRCs\n");
		fprintf(stderr,"\t-l -- Do not print Download Flags\n");
		fprintf(stderr,"\t-p -- Specify FULL path\n");
		break;
	      }

/******
	Establish connections to all the dBase files that we need.
******/

    d4init(&codeBase);
#if defined(WIN32) || defined (USECB63)
    codeBase.errFieldName = 0;
    codeBase.errOpen = 0;
    codeBase.errTagName = 0;
    codeBase.readLock = 0;
#else
    codeBase.field_name_error = 0;
    codeBase.open_error = 0;
    codeBase.tag_name_error = 0;
    codeBase.read_lock = 0;
#endif WIN32
#ifndef CB51
    codeBase.wait = 0;
#endif

    /*
    PVMAP.DBF
    */
	if ((pvmap = d4open(&codeBase, defaultPath)) == 0)
	{
		exit(1);
	}
    if ((pvmapIoc1Dnld = d4field(pvmap, "LOAD1FLAG")) == 0)
	{
		printf("Can't find field LOAD1FLAG in PVMAP.\n");
		exit(1);
	}
    if ((pvmapIoc2Dnld = d4field(pvmap, "LOAD2FLAG")) == 0)
	{
		printf("Can't find field LOAD2FLAG in PVMAP.\n");
		exit(1);
	}
    if ((pvmapDeleteFlag = d4field(pvmap, "DELETEFLAG")) == 0)
	{
		printf("Can't find field DELETEFLAG in PVMAP.\n");
		exit(1);
	}
    if ((pvmapName = d4field(pvmap, "NAME")) == 0)
	{
		exit(1);
	}
    if ((pvmapResource = d4field(pvmap, "RESRCNAME")) == 0)
	{
		exit(1);
	}
    if ((pvmapCreationTime = d4field(pvmap, "TIMESTAMP")) == 0)
	{
		exit(1);
	}
    if ((pvmapChecksum = d4field(pvmap, "CHECKSUM")) == 0)
	{
		exit(1);
	}
    if ((pvmapPointType = d4field(pvmap, "TYPE")) == 0)
	{
		exit(1);
	}
    if ((pvmapTag = d4tag(pvmap, "NAME")) == 0)
	{
		exit(1);
	}
    d4tag_select(pvmap, pvmapTag);
    d4unlock_all(pvmap);
	
    d4top(pvmap);
    do
	{
		if (d4eof(pvmap))
			break;
		d4unlock_all(pvmap);
		if (d4deleted(pvmap))
			continue;
		if (f4true(pvmapDeleteFlag))
			continue;
		trimCpy(pvName, f4str(pvmapName));
		trimCpy(pvRes, f4str(pvmapResource));
		checksum = (unsigned short) strtol(f4str(pvmapChecksum),NULL,16);
		creation = f4int(pvmapCreationTime);
		if(printDownload)
		  {
			if (f4true(pvmapIoc1Dnld))
			  downloadFlag1 = 'Y';
			else
			  downloadFlag1 = 'N';
			if (f4true(pvmapIoc2Dnld))
			  downloadFlag2 = 'Y';
			else
			  downloadFlag2 = 'N';
		  }
		else
		  {
			downloadFlag1 = ' ';
			downloadFlag2 = ' ';
		  }
		if (printCrc)
			printf("%28s, 0X%04X (%05d), %c, %c, %s, (0X%X) %s", pvName, checksum, checksum, downloadFlag1, 
			downloadFlag2, pvRes, creation, ctime(&creation));
		else
			printf("%28s, %c, %c, %s, (0X%X) %s", pvName, downloadFlag1, 
			downloadFlag2, pvRes, creation, ctime(&creation));
	} while(d4skip(pvmap, (long) 1) != r4eof);
    d4unlock_all(pvmap);
    exit(1);
}

/****************************************************************/
/*                                                              */
/*      COPY ONE STRING TO ANOTHER, STOPPING AT FIRST BLANK     */
/*                                                              */
/****************************************************************/
 
 void trimCpy(destination, source)
 char    *destination;
 char    *source;
 {
     while (*destination = *source)
	 {
	 if (*destination == ' ')
		 {
		 *destination = 0;
		 return;
		 }
	 destination++;
	 source++;
	  }
    return;
}
