/*drvModicon.c,v 1.1.1.1 1999/08/03 14:43:20 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/

/*  File: drvModicon.c
**  Edit:  2/25/93 12:33 JLD                                                                         
**      : 12/08/93 10:15 WEP: comments only                                                          
**	: 12/16/93      WEP: half duplex mode with rts/cts delay and data tail squelch avoidance     
**	: 12/17/93      WEP: dynamically allocate data communications buffers on a per line basis    
**	: 12/21/93      WEP: dynamically allocate MODPLC structures on a per PLC basis               
**	: 12/23/93      WEP: dynamically allocate output queue structures on a per line basis        
**	:  1/03/94      WEP: mutual exclusion logic used when a diagnostic driver shares data path.  
**	:  1/06/94      WEP: add pointname pointer to all driver/read functions. for error messages. 
**	:  1/10/94      WEP: '#define' public routine names to make the driver cloning effort easier.
**	:  1/20/94	WEP: Duluth fix: added microsol support to mod_aoread                            
**      :  2/21/94      WEP: take semaphore immediately after its creation.                          
**	:  3/30/94	ems: STARTDRIVER1, STARTDRIVER2, remove test ln >= 0, it is unsigned         
**			     msgxfer(), remove cast (char *), not needed                             
**      :  4/18/94      WEP: tasklocking added to improve IOC terminal messgae viewing.  
**                         : implemented Polled Report By Exception Function.            
**                         : added half duplex RTS control with timing set in call.      
**                         : modified msgxfer RX logic to timeout on inactivity.         
**                         : modified mod_aidriver to mask out MSBit of statistics counts.   
**                         : added string support - mod_sodriver & mod_sidriver (see devArcom.c).
**                         : modified mod_aidriver to support non-block-aligned FLOAT,       
**                           DOUBLE INTEGER, and DOUBLE FLOAT data types.                
**                         : modified mod_aodriver and mod_aoread to support non-block-aligned   
**                           ASCII FLOAT                                                 
**     :  05/05/94      WEP: moved while(TRUE) taskDelay logic to while(master) loop.    
**                         : added taskDelay to while(pstate=slavewant).                 
**                         : change all failed msgxfer holdflag=FALSE to holdflag=TRUE.  
**     :  05/06/94      WEP: modified loopback logic to correct performance statistics   
**     :  06/06/94      WEP: modified msgxfer RX timing logic to handle long responses   
**     :  06/08/94      WEP: modified rx_int_proc to reject pre-response false input.    
**     :  06/14/94      WEP: modified register_resource to print pointname causing error.
**     :  06/22/94      WEP: corrected performance and use of the cmd_force_offline flag.
**     :  06/29/94      WEP: added printing of pstate & activity on initial master state.
**     :  07/07/94      WEP: added wait for pstate to become valid before deciding       
**                         : whether to become master.                                   
**     :  07/13/94      WEP: modified A0DRIVER, BODRIVER, & mod_sodriver to increment the    
**                         : 'badoutcnt' pseudo point counter when the output request is 
**                         : rejected due to the output queue being full.                
**     :  07/14/94      WEP: modified master state logic in mod_driver to detect and recover   
**                         : from the local and partner drivers being in lockstep.       
**     :  07/15/94         : added test of is_responding flag in checkoutputs and, if    
**                         : FALSE, discard the output queue entry.                      
**     :  07/18/94	WEP: modified scan_plc, readcoils, readdis, readais, readregs,   
**                         : setcoil, setreg, & setstring to disable points and continue 
**                         : processing after NACK received in response to Modbus cmd.   
**     :  08/31/94	WEP: modified AODRIVER, BODRIVER, & mod_sodriver to generate a       
**                         : modtraceon enabled diagnostic message whenever the output      
**                         : buffer overflows on an output driver call.                  
**                         : removed mltrace details from the three output drivers while 
**                         : adding diagnostic messages for driver call failures.        
**     :  09/12/94	WEP: modified AODRIVER, BODRIVER, SODRIVER, & checkoutputs to    
**                         : generate, at trace level 3, output queue and dequeue msgs.  
**                         : added line level communication statistics to accumulate RX  
**                         : interrupt counts, TX interrupt counts, and msgxfer counts.  
**                         : added routines to display and clear the same counters.      
**                         : added logic to measure time to scan the entire set of PLCs, 
**                         : and the elapsed time since scanning the entire set of PLCs. 
**                         : modified mod_driver to generate a debug message, when TRACEON,    
**                         : displaying the elapsed time.                                
**     :  09/16/94	WEP: added calculated statistics to line level statistics above. 
**     :  09/15/94	WEP: modified traffic trace logic to trace either all or a       
**                         : single selected plc address, and either all or a single     
**                         : selected modbus command type.                               
**     :  09/20/94	WEP: added modhelp routine to display this driver's utility info.
**     :  09/21/94	WEP: modified drivers to simulate responsive plcs. while in      
**                         : simulation mode, no communications take place with the plcs,
**                         : and if in slave mode, the driver is forced to master mode.  
**                         : added modsimon and modsimoff to enable & disable simulation.
**     :  12/05/94      WEP: modified dumpplcstats to mask out MSBit of stat counts.     
**     :  01/16/95      WEP: implemented calls to watchdog timer logic in platform.      
**     :  01/18/95      WEP: added explict disableable error messages to driver calls.   
**     :  01/27/95      WEP: parameterized and renamed statistics control functions.     
**     :  02/13/95      WEP: clarified and made conditional the illegal lstate message.  
**     :  02/13/95      WEP: corrected pstatewait logic.                                 
**     :  04/28/95      WEP: added private parameter to driver call. if true, causes     
**                         : driver to immediately become master, assuming no partner.   
**     :  05/08/95      WEP: added logic to: count 'good' msgxfers, sum the time it      
**                         : took to transact these transfers, and count the number of   
**                         : plc scans, all on a plc basis. These registers are cleared  
**                         : in initstats. these values, along with derived statistics,  
**                         : are displayed in moddumpplcstats and TABLEINFO.                
**     :  05/22/95      WEP: modified master loop logic to exit when local state is not  
**                         : master. This causes lockstep logic to function correctly.   
**     :  01/21/97      WEP: added a third variation on the modicon driver to handle     
**                         : the Hathaway Modicon interface which doesn't support the    
**                         : Modicon loopback test command (code 8). This version uses   
**                         : a modified loopback routine that always reads a fixed input 
**                         : input status (0) using code 2. invoked with "startmodhia"   
**     :  07/17/97      WEP: added 32 line support. modified state logic to not allow    
**                         : MSTR to SLV transition while partner is SLV or SLVWANTCA.   
**                         : modified modhelp to show startup command format. added      
**                         : diagnostic logic to count and display RX errors. moved      
**                         : comm semaphore init to MAIN. modified rx_init_proc to only  
**                         : set activity flag on good RX characters. added diagnostic   
**                         : print line level parameters. removed watchdog call.         
**                         : expanded private feature to bypass lockstep detection.      
**     :  07/21/97      WEP: remove refernces to linemux structure for line sharing.     
**     :  09/10/97      WEP: added support for Resource Group Lock function
**     :  08/19/98      WEP: corrected Resource Group Lock state tests. Added additional 
**                         : features to Hathaway SR8550 variant of the Modbus interface.
**                         : In addition to needing to use a status read instead of the  
**                         : loopback command (see 1/21/97), the SR8550 does not support 
**                         : Coil reads and Holding Register reads. scan_plc has been    
**                         : modified to not scan these card types. BI_DRIVER has been   
**                         : modified to reject Coil BI points. BO_READ has been modified
**                         : to ignore the data_valid flag and to return zero for Coil   
**                         : points on initialization. AI_DRIVER has been modified to    
**                         : reject Holding Register AI points. AO_READ has been modified
**                         : to ignore the data_valid flag and to return zero for Holding
**                         : Register points on initialization. modified modhelp ().     
**     :  03/12/99      WEP: in private mode, register resource via rmPrivate().           
*/

#include <ctype.h>

#include    <vxWorks.h>
#include    <sysLib.h>
#include    <semLib.h>
#include    <wdLib.h>
#include    <rngLib.h>
#include    <logLib.h>
#include    <tickLib.h>
#include    <remLib.h>
#include    <stdioLib.h>
#include    <string.h>

/*#include    "iocsiodvr_bridge.h"*/
#include    "resmgr.h"
#include    "task_params.h"
#include    "dbDefs.h"
#include    "drvSup.h"
#include    "drvModicon.h"
#include    "drvModicon_proto.h"
#include    "restable.h"
#include    "iocsiodvr_bridge.h"
#include     <serialFile.h>

void modtraceon (unsigned short line);

void modtraceoff (unsigned short line);

void modtrafficon (unsigned short line);

void modtrafficoff (unsigned short line);

void modrestrlv (unsigned short line, unsigned short level);

void modrestraceon (unsigned short line);

void modrestraceoff (unsigned short line);

void modtrlv (unsigned short line, unsigned short level);

void modsimon (unsigned short line, unsigned short delay, int low, int high, int digenable);

void modsimoff (unsigned short line);

void modcontraceon (unsigned short line);

void modcontraceoff (unsigned short line);

void modruntraceon (unsigned short line);

void modruntraceoff (unsigned short line);

void modrejecttraceon (unsigned short line);

void modrejecttraceoff (unsigned short line);

void modnacktraceon (unsigned short line);

void modnacktraceon (unsigned short line);

void moddefaulttrace (unsigned short line);

int stopmodbus (short ln);

int startmodbus (unsigned short line, char *mode, unsigned short delay,
		 unsigned short hdflag, unsigned short delay1,
		 unsigned short delay2, unsigned short private);

int startmicrosol (unsigned short line, char *mode, unsigned short delay,
		   unsigned short hdflag, unsigned short delay1,
		   unsigned short delay2, unsigned short private);

void modcomm (unsigned short line, unsigned short tickcount1,
	      unsigned short hdflag, unsigned short tickcount2,
	      unsigned short tickcount3);

void mod_driver (unsigned short line, unsigned char *mode, unsigned short delay,
		 unsigned short hdflag, unsigned short delay1,unsigned short delay2,
		 unsigned short private);

int checkgroup (unsigned short line, unsigned short *lstate, unsigned short tracenum);



static long	init();
static 	void on_line (unsigned short line,unsigned short mod);
static 	void mark_not_valid (MODPLC  *pmod);
static	void scan_plc (MODPLC *pmod, unsigned short line,unsigned short mod);
static	void checkoutputs (unsigned short line);
static void  register_resource (unsigned short link, MODPLC *pmod,
                                unsigned char *resname, char *pointname);
static	void outqueinit (unsigned short line);
static	OUTQENT *ogetemptyp (unsigned short line);
static	void omarkfull (unsigned short line);
static	OUTQENT *ogetnextp (unsigned short line);
static	void oreleasefull (unsigned short line);
static 	int allocate_plc (unsigned short line, unsigned short plc);
static	void start_rx (unsigned short line,SEMAPHORE *semid,unsigned short *pstatus);
static	void stop_rx (unsigned short line );
static	int rx_int_proc (unsigned short line );
static	int check_line_activity (unsigned short line, unsigned short predelay);
static	void start_tx (unsigned short line,SEMAPHORE *semid,unsigned short *pstatus);
static	void stop_tx (unsigned short line );
static	int tx_int_proc (unsigned short line );
static	unsigned short msgxfer (unsigned short fline,unsigned short resplen,
        MODPLC *pmod, unsigned short rxlength);
static	void init_plc_tab (unsigned short line, unsigned short mod);
static 	void initplc (MODPLC  *pmod);
static	void initstats (MODPLC *pmod);
static	unsigned short hiamodloopback (unsigned short line, unsigned short mod, MODPLC *pmod);
static	unsigned short modloopback (unsigned short line, unsigned short mod, MODPLC *pmod);
static	void prepresponses (unsigned short line);
static	int any_responding (unsigned short line);
static	unsigned short readcoils (unsigned short line, unsigned short mod,
                       MODPLC *pmod, unsigned short bitaddr, unsigned short readlen);
static	unsigned short readdis (unsigned short line, unsigned short mod,
	               MODPLC *pmod, unsigned short bitaddr, unsigned short readlen);
static	unsigned short readais (unsigned short line, unsigned short mod,
	               MODPLC *pmod, unsigned short regaddr, unsigned short reglen);
static	unsigned short readregs (unsigned short line, unsigned short mod,
                       MODPLC *pmod, unsigned short regaddr, unsigned short reglen);
static	unsigned short setcoil (unsigned short line, unsigned short mod,
		       MODPLC *pmod, unsigned short bitaddr, unsigned long mask,
		       unsigned long val);
static	unsigned short setreg (unsigned short line, unsigned short mod,
		       MODPLC *pmod, unsigned short regaddr, unsigned short val,
		       double dblval, unsigned short conv);
static 	int loadinit (unsigned short line,MODPLC *pmod,unsigned short mod);
static 	int loadnextblock (unsigned short line);
static 	int loadlogic (LOADINFO *plinfo);
static 	int loadmemory (LOADINFO *plinfo, unsigned short type);
static 	terminateload (LOADINFO *plinfo,unsigned short line, unsigned short success,
        unsigned short code);
static 	int plc_logout (LOADINFO *plinfo);
static 	int plc_login (LOADINFO *plinfo);
static 	int plc_start (LOADINFO *plinfo);
static 	int plc_stop (LOADINFO *plinfo);
static 	int pollfordone (LOADINFO *plinfo, int subfunction, unsigned short geterrcode);
static 	int plc_init (LOADINFO *plinfo);
static 	int plc_getdim (LOADINFO *plinfo);
static 	int plc_setdim (LOADINFO *plinfo);
static 	int setrunmode (LOADINFO *plinfo, short value);
static 	int getrunmode (LOADINFO *plinfo, short *pvalue);
static 	int get_plc_state (LOADINFO *plinfo, short *pvalue);
static 	int readmem (LOADINFO *plinfo, unsigned short page, unsigned short address,
	    unsigned short datalen, char *databuf);
static  int writemem (LOADINFO *plinfo, unsigned short page, unsigned short address,
	    unsigned short datalen, char *databuf);
static 	int writelogic (LOADINFO *plinfo, unsigned short page, unsigned short address,
	    unsigned short datalen, char *databuf);
static 	int writeascii (LOADINFO *plinfo, unsigned short page, unsigned short address,
	    unsigned short datalen, char *databuf);
static	void makecrc (unsigned char *chkbuf, int len,char unsigned *retbuf);
static 	void printcoilhdr (unsigned char *buffer, unsigned short cnt);
static 	void printdihdr (unsigned char *buffer, unsigned short cnt);
static	void printaihdr (unsigned char *buffer, unsigned short cnt);
static 	void printreghdr (unsigned char *buffer, unsigned short cnt);
static	void sim_plc (MODPLC *pmod, unsigned short line, unsigned short mod);
static int write_aos (unsigned short line);
static	unsigned short setstring (unsigned short line, unsigned short mod,
			          MODPLC *pmod, unsigned short signal,
			          unsigned char *buf, unsigned short olen);


struct	{
		long		number;
		DRVSUPFUN	report;
		DRVSUPFUN	init;
	}
	drvModicon={ 2,	NULL, init};

/*
**      External Line Level Data Structures (declared in drvcommon.c)
**
*/
extern	unsigned char	resnames [MODLINES] [RESNAMELEN]; /* array of resource name strings.		*/

extern	RES_TABLE	*prestable [MODLINES];		/* array of pointers to resource tables 	*/

extern	unsigned short	resresponse [MODLINES];		/* array of resource response flags. 		*/
							/* False=not active resource(slave),		*/
							/* True= active resource(master) 		*/

extern	unsigned short	line_activity [MODLINES];	/* true/false line active flag. calculated	*/
							/* using activity_time below. 			*/
extern	unsigned short	lastactivity [MODLINES];	/* assigned value of line_activity prior to	*/
							/* it's recalculation.   			*/

extern	unsigned short	modtrace [MODLINES];		/* controls output of communication error log	*/
							/* messages (true/false) 			*/
extern	unsigned short	mltrace [MODLINES];		/* controls output of certain resource related	*/
							/* log messages (level) 			*/
extern	unsigned short	modrestrace [MODLINES];		/* controls output of MASTER/SLAVE arbitration	*/
							/* log messages (binary) 			*/

extern	SEM_ID		semids [MODLINES] [2];		/* 'new' semaphore logic. the old SEMAPHORE is	*/
							/* still used. 					*/

extern	unsigned short	intermsgdelay [MODLINES];	/* holds value passed in task spawn. used for	*/
							/* each msgxfer attempt 			*/
extern	unsigned short	pretxdelay [MODLINES];		/* holds post RTS delay before TX. 		*/
extern	unsigned short	posttxdelay [MODLINES];		/* holds post TX delay before dropping RTS. 	*/

extern	long		activity_time [MODLINES];	/* holds timetick of last activity. used to	*/
							/* set line_activity flag  			*/

extern  unsigned short  halfduplex [MODLINES];          /* store half/full duplex line status           */

extern  unsigned short  pstatewait [MODLINES];          /* holds max time in seconds that driver will   */
                                                        /* wait to obtain partner state on first        */
                                                        /* attempting to decide if it should be master. */

extern  short           resGroupLock [];                /* holds line driver group lock flags.          */
extern  char           *resGroupTripName [];            /* holds line driver trip name.                 */


/*
**      Storage for MODBUS Comm Line Structures
**
*/

static RXLINE          *rxlinetab [MODLINES] = 
			{NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL,} ; /* array of pointers to dynamically           */
                                        /* allocated comm controls/buffers            */

static TXLINE          *txlinetab [MODLINES] = 
			{NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL,} ; /* array of pointers to dynamically           */
                                        /* allocated comm controls/buffers            */

/*
**      Storage for MODBUS PLC Structures
**
*/
static unsigned short   pryvate [MODLINES];              /* 0 = redundant mode,   !0=private mode      */
static unsigned short   link [MODLINES];              	/* line Number started with for file I/O */
static unsigned int     modsim [MODLINES];               /* 0 = normal operation, 1 = plc simulation.  */

static unsigned int	simdelay [MODLINES];

static 		int	simlow [MODLINES];

static 		int	simhigh [MODLINES];

static		int	simdigon [MODLINES];

static unsigned int     tick [MODLINES][5];

static unsigned int     lastscan [MODLINES];             /* holds timestamp of beginning of major loop.  */

static unsigned int     scantime [MODLINES];            /* holds actual time taken to scan plcs.        */

static unsigned int     loopoutmsgs [MODLINES];

static unsigned int     loopmsgs [MODLINES];

static LINESTATS	linestats [MODLINES];		/* array of line level statistics.              */

static LOADINFO		linfo [MODLINES];		/* array of control structures for              */
                                                        /* loading PLC headers 			        */

static MODPLC		*plcs [MODLINES] [NUMMODPLCS + 1];/* two dimensional array of pointers to the   */
                                                        /* fixed length dynamically allocated PLC       */
                                                        /* control and data structures. extra pointer   */
                                                        /* is for Query structure used independently    */
							/* of the others. it also allows for the array  */
                                                        /* index to match the mod value.                */

static	unsigned short	trace [MODLINES];		/* controls output of system error messages:	*/
							/* bit mapped controls                          */
static  unsigned char   traceadr [MODLINES];
static  unsigned char   tracetype [MODLINES];

static OUTPUTQUE	*oq [MODLINES];			/* array of ptrs to the line output queues and  */
                                                        /* their control structures                     */

static unsigned short	microsolflag [MODLINES];	/* used in mod_aodriver & mod_aidriver to distinguish   */
							/*  between the microsol & modicon plc devices 	*/

static unsigned short	hathawayflag [MODLINES];	/* used in online routine to call alternative   */
							/*  loopback routine, as Hathaway doesn't do it.*/
static unsigned short   hathaway16bit[] =               /* Used to determine if sign extentions are required */
                                          { TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,
                                            TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,
                                            TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,
                                            TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE };

static unsigned long	nextouttime [MODLINES];

static unsigned long	outagain [MODLINES];

static char		user [40], password [40];	/* holds id and password of the user that       */
                                                        /* spawned the driver task required by host to  */
                                                        /* check file read privileges on PLC load files */
static char modes [MODLINES][16];			/* Room to save the mode strings */

static int		taskId [MODLINES];

static char *tasknames [] = {"MODBUS Ch0", "MODBUS Ch1", "MODBUS Ch2", "MODBUS Ch3",
                             "MODBUS Ch4", "MODBUS Ch5", "MODBUS Ch6", "MODBUS Ch7",
                             "MODBUS Ch8", "MODBUS Ch9", "MODBUS Ch10","MODBUS Ch11",
                             "MODBUS Ch12","MODBUS Ch13","MODBUS Ch14","MODBUS Ch15",
                             "MODBUS Ch16","MODBUS Ch17","MODBUS Ch18","MODBUS Ch19",
                             "MODBUS Ch20","MODBUS Ch21","MODBUS Ch22","MODBUS Ch23",
                             "MODBUS Ch24","MODBUS Ch25","MODBUS Ch26","MODBUS Ch27",
                             "MODBUS Ch28","MODBUS Ch29","MODBUS Ch30","MODBUS Ch31"
                           };

static char *tasknames2 [] = {"MICROSOL Ch0", "MICROSOL Ch1", "MICROSOL Ch2", "MICROSOL Ch3",
                             "MICROSOL Ch4", "MICROSOL Ch5", "MICROSOL Ch6", "MICROSOL Ch7",
                             "MICROSOL Ch8", "MICROSOL Ch9", "MICROSOL Ch10","MICROSOL Ch11",
                             "MICROSOL Ch12","MICROSOL Ch13","MICROSOL Ch14","MICROSOL Ch15",
                             "MICROSOL Ch16","MICROSOL Ch17","MICROSOL Ch18","MICROSOL Ch19",
                             "MICROSOL Ch20","MICROSOL Ch21","MICROSOL Ch22","MICROSOL Ch23",
                             "MICROSOL Ch24","MICROSOL Ch25","MICROSOL Ch26","MICROSOL Ch27",
                             "MICROSOL Ch28","MICROSOL Ch29","MICROSOL Ch30","MICROSOL Ch31"
                           };
static char *tasknames3 [] = {"MODHIA Ch0", "MODHIA Ch1", "MODHIA Ch2", "MODHIA Ch3",
                              "MODHIA Ch4", "MODHIA Ch5", "MODHIA Ch6", "MODHIA Ch7",
                              "MODHIA Ch8", "MODHIA Ch9", "MODHIA Ch10","MODHIA Ch11",
                              "MODHIA Ch12","MODHIA Ch13","MODHIA Ch14","MODHIA Ch15",
                              "MODHIA Ch16","MODHIA Ch17","MODHIA Ch18","MODHIA Ch19",
                              "MODHIA Ch20","MODHIA Ch21","MODHIA Ch22","MODHIA Ch23",
                              "MODHIA Ch24","MODHIA Ch25","MODHIA Ch26","MODHIA Ch27",
                              "MODHIA Ch28","MODHIA Ch29","MODHIA Ch30","MODHIA Ch31"
                           };
static unsigned short modResGroups [MODLINES] =
			{0, 0, 0, 0, 0, 0, 0, 0,
                         0, 0, 0, 0, 0, 0, 0, 0,
                         0, 0, 0, 0, 0, 0, 0, 0,
                         0, 0, 0, 0, 0, 0, 0, 0} ; /* line driver group assignments   */

/**********
*
*	This routine is required by iocinit in the EPICS system.
*
**********/
static long init ()
{
	return (0);
}

/********
**
**	This routine outputs information to help with debugging.
**
********/
void modhelp (void)
{
    printf ("Starting and Stopping the Modicon Driver:\n");
    printf ("\n");
    printf (" startmodbus(N,C,t1,f,t2,t3,P)                                                  \n");
    printf (" startmodhia(N,C,t1,f,t2,t3,P)                                                  \n");
    printf (" startmicrosol(N,C,t1,f,t2,t3,P)                                                \n");
    printf ("                     N:  start the driver on line N.                            \n");
    printf ("                     C:  communications parameter string (comma delimited ascii)\n"); 
    printf ("                     t1: intermessage delay (in ticks)                          \n");
    printf ("                     f:  half duplex flag. 1=half duplex mode, 1!=full duplex   \n");
    printf ("                     t2: post RTS assert, pre TX delay (in ticks)               \n");
    printf ("                     t3: post TX, pre RTS drop delay (in ticks)                 \n");
    printf ("                     P:  Private flag.  0 = Redundant, 1 = Non-Redundant        \n");
    printf ("                                                                                \n");
    printf (" Example: startmodbus(2,\"9600,N,8,1\", 2,1,4,0)                                \n");
    printf ("                                                                                \n");
    printf (" stopmodbus(N)                                                                  \n");
    printf ("                     N = line number                                            \n");
    printf ("\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" Modbus Driver Control & Display Functions (* = default)\n");
    printf ("\n");
    printf (" modtraceon(N)         Enable the following debug messages for line N:             \n");
    printf ("                         1) Major Scan loop Info:                                  \n");
    printf ("                           scan time, total output messages, and total messages.   \n"); 
    printf ("                         2) Output Queue Overflow.                                 \n");
    printf ("                         3) All communication errors encountered in msgxfer.       \n");
    printf ("*modtraceoff(N)        Disable all of the modtraceon debug messages for line N.    \n");
    printf ("\n");
    printf (" modrestrace(N,X)      Resource Trace Level: 0 = off. 1 = state change; 2 = delay. \n");
    printf ("*modrestraceon(N)      Enable display of master/slave/etc state changes in MAIN.   \n");
    printf (" modrestraceoff(N)     Disable display of master/slave/etc state changes in MAIN.  \n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" modcontraceon(N)      Enable display of configuration errors in driver calls.     \n");
    printf ("*modcontraceoff(N)     Disable display of configuration errors in driver calls.    \n");
    printf (" modruntraceon(N)      Enable display of runtime errors (queue overflow).          \n");
    printf ("*modruntraceoff(N)     Disable display of runtime errors (queue overflow).         \n");
    printf (" modrejecttraceon(N)   Enable display of driver call rejections in driver calls.   \n");
    printf ("*modrejecttraceoff(N)  Disable display of driver call rejections in driver calls.  \n");
    printf (" modnacktraceon(N)     Enable display of point deselection on MODBUS command NACK. \n");
    printf ("*modnacktraceoff(N)    Disable display of point deselection on MODBUS command NACK.\n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" modtrlv(N,X)          Enable the following message for line N:                  \n");
    printf ("                         X=0,  display none of the messages below.               \n");
    printf ("                         X=1,  beginning of plc scan loop, bringing plcs online. \n");
    printf ("                         X=2,  beginning of plc scan.                            \n");
    printf ("                         X=3,  AO queue actions (normal: mod_aodriver & checkoutputs)\n");
    printf ("                         X=4,  BO queue actions (normal: mod_bodriver & checkoutputs)\n");
    printf ("                         X=5,  SO queue actions (normal: mod_sodriver & checkoutputs)\n");
    printf ("                         X=6,  calls to checkoutputs.                            \n");
    printf ("                         X=7,  checkoutputs: trace each queue entry read.        \n");
    printf ("                         X=8,  AIDRIVER, AOREAD: normal call return.             \n");
    printf ("                         X=9,  BIDRIVER, BOREAD: normal call return.             \n");
    printf ("                         X=10, SIDRIVER: normal call return.                     \n");
    printf ("                         X=11 to 14,  reserved.                                  \n");
    printf ("                         X=15, 'Microsol Busy', waiting in checkoutputs.         \n");
    printf ("                         X=16 to 254,  reserved.                                 \n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" modtrafficon(N)       Enable traffic tracing for all plc addresses                \n");
    printf ("                         and all modbus functions on line N.                       \n");
    printf ("*modtrafficoff(N)      Disable all traffic tracing on line N.                      \n");
    printf (" modtraffic(N,X,Y)     Modifies line N traffic tracing as follows:                 \n");
    printf ("                         X=0   disables tracing of all addresses.                  \n");
    printf ("                         X=255 enables tracing of all addresses.                   \n");
    printf ("                         0 < X < 255 enables tracing of address X traffic.         \n");
    printf ("                         Y=0   disables tracing of all modbus functions.           \n");
    printf ("                         Y=255 enables tracing of all modbus functions.            \n");
    printf ("                         0 < Y < 255 enables tracing of function Y traffic.        \n");
    printf ("\n");
    printf (" moddefaulttrace(N)    Restores default setting for all traces.                    \n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" modsimon(N,D,L,H,T)   Enables the simulation of responding plcs.                  \n");
    printf ("                        D: simulation interval (in seconds). if D<1 or D>60, D=60. \n");
    printf ("                        L: low value of analog simulation range.                   \n");
    printf ("                        H: high value of analog simulation range.                  \n");
    printf ("                        T: 0 = digital inputs are zero. 1 = digitial inputs toggle.\n");
    printf ("*modsimoff(N)          Disables simulation, thus enabling normal driver function.  \n");
    printf ("\n");

    taskDelay (30); /* Time for operator to see it and pause the output */

    printf (" modinfo(N)            Display startup and current state information.              \n");
    printf (" modcomm(N,t1,f,t2,t3) Set communication timing parameters for line N as follows:  \n");
    printf ("                         t1: intermessage delay (in ticks).                        \n");
    printf ("                          f: 0 = full duplex, 1 = half duplex.                     \n");
    printf ("                         t2: post RTS assert, pre TX delay (in ticks).             \n");
    printf ("                         t3: post TX, pre RTS drop delay (in ticks).               \n");
    printf ("\n");
    printf (" moddumpmsgcnts(N)     Display line N communication statistics.                    \n");
    printf (" modclearmsgcnts(N)    Display and clear line N communication statistics.          \n");
    printf (" moddumpplcstats(N,X)  Display line N, plc X, communication statistics.            \n");
    printf (" modclearplcstats(N,X) Display and clear line N, plc X, xfers, status & errors.    \n");
    printf (" dump_mod_table(N,X)   Display line N, plc X, xfers, status, errors and input data.\n");
    printf (" modhelp               Display this information.                                   \n");
    printf ("\n");
    printf (" modResGroup(N,G)      Assign line N to group G (valid G range is 1 to 16)         \n");
}
/*
*    Routine to toggle between 12 and 16 bit analog input modes
*    16 bit mode is default. 12 bit mode may be set in the startup.cmd
*/

void modhath12bit (unsigned short line)
{
    hathaway16bit[line] = FALSE;
}
void modhath16bit (unsigned short line)
{
    hathaway16bit[line] = TRUE;
}



/********
**
**	This routine holds the task from starting to grap for debug
**
********/
int modtaskhold = FALSE; /* Default to run freely */
void modtasklock (unsigned short line)
{
	modtaskhold  = TRUE;
}

/********
**
**	This routine lets the task start up
**
********/
void modtaskunlock (unsigned short line)
{
	modtaskhold  = FALSE;
}

/********
**
**	This routine starst up the task
**
********/
int	hsMOD( int line)
{
	short 	value;
	int	status;

	printf ("HI There\n");
	trace[line] = CONFIGURATIONTRACE;
	status = mod_aidriver("Test1",line, 1,3,1,&value,AICNVINTEGER,"John");
	printf(" Value = %x \n",value);
	return (status);
}

/********
**
**	This routine enables logging of comm trace messages
**
********/
void modtraceon (unsigned short line)
{
	modtrace [line] = TRUE;
}

/********
**
**	This routine disables logging of comm trace messages
**
********/
void modtraceoff (unsigned short line)
{
	modtrace [line] = FALSE;
}

/********
**
**	This routine enables detailed display of normal communications traffic
**
********/
void modtraffic (unsigned short line, unsigned char add, unsigned type)
{
	trace [line] = trace [line] | ALLTRAFFICTRACE;
        traceadr [line] = add;
        tracetype [line] = type;
        return;
}

/********
**
**	This routine enables detailed display of normal communications traffic
**
********/
void modtrafficon (unsigned short line)
{
	trace [line] = trace [line] | ALLTRAFFICTRACE;
        traceadr [line] = 255;  /* every address */
        tracetype [line] = 255; /* every message type */
}

/********
**
**	This routine disables detailed display of normal communications traffic
**
********/
void modtrafficoff (unsigned short line)
{
	trace [line] = trace [line] & ~ALLTRAFFICTRACE;
}

/********
**
**	This routine enables logging of resource trace messages
**
********/
void modrestrlv (unsigned short line, unsigned short level)
{
	modrestrace [line] = level;
}

/********
**
**	This routine enables logging of resource trace messages
**
********/
void modrestraceon (unsigned short line)
{
	modrestrace [line] = 1;
}

/********
**
**	This routine disables logging of resource trace messages
**
********/
void modrestraceoff (unsigned short line)
{
	modrestrace [line] = 0;
}

/********
**
**	This routine controls logging of trace messages
**
********/
void modtrlv (unsigned short line, unsigned short level)
{
	mltrace [line] = level;
}

/********
**
**	This routine enables the simulation of responding plcs, reading all input values as zero.
**
********/
void modsimon (unsigned short line, unsigned short delay, int low, int high, int digenable)
{
	if ((delay < 1) || (delay > 60))
	{
	    simdelay [line] = 60;
	}
	else
	{
	    simdelay [line] = delay;
	}
	if (low > high)
	{
	    simhigh [line] = low;
	    simlow [line] = high;
	}
	else
	{
	    simlow [line] = low;
	    simhigh [line] = high;
	}
	modsim [line] = TRUE;
	simdigon [line] = digenable;
}

/********
**
**	This routine disables simulation, thus enabling normal driver function.
**
********/
void modsimoff (unsigned short line)
{
	modsim [line] = FALSE;
}

/********
**
**	These routines enable and disable tracing of database configuration errors in driver calls
**
********/
void modcontraceon (unsigned short line)
    {
	trace[line] = trace[line] | CONFIGURATIONTRACE;
    }
void modcontraceoff (unsigned short line)
    {
	trace[line] = trace[line] & ~CONFIGURATIONTRACE;
    }

/********
**
**	These routines enable and disable tracing of runtime errors in driver calls
**
********/
void modruntraceon (unsigned short line)
    {
	trace[line] = trace[line] | RUNTIMEERRORTRACE;
    }
void modruntraceoff (unsigned short line)
    {
	trace[line] = trace[line] & ~RUNTIMEERRORTRACE;
    }

/********
**
**	These routines enable and disable tracing of call rejection errors in driver calls
**
********/
void modrejecttraceon (unsigned short line)
    {
	trace[line] = trace[line] | CALLREJECTTRACE;
    }
void modrejecttraceoff (unsigned short line)
    {
	trace[line] = trace[line] & ~CALLREJECTTRACE;
    }


/********
**
**	These routines enable and disable tracing of call rejection errors in driver calls
**
********/
void modnacktraceon (unsigned short line)
    {
	trace[line] = trace[line] | MODBUSNACKTRACE;
    }
void modnacktraceoff (unsigned short line)
    {
	trace[line] = trace[line] & ~MODBUSNACKTRACE;
    }

/********
**
**	This routine restores default setting for all traces.
**
********/
void moddefaulttrace (unsigned short line)
    {
        /* debug structures */
	modtrace [line] = FALSE;
	mltrace [line] = 0;
	modrestrace [line] = 1;
        trace [line] = 0;
        traceadr [line] = 255;  /* every address      */
        tracetype [line] = 255; /* every message type */
    }
/********
**
**	This routine turns on all error tracing.
**
********/
void moderrortrace (unsigned short line)
    {
        /* debug structures */
	modtrace [line] = TRUE;
	modrestrace [line] = 2;
	trace[line] = MODBUSNACKTRACE ||
                      CONFIGURATIONTRACE ||
                      RUNTIMEERRORTRACE ||
                      CALLREJECTTRACE ||
                      0;
        traceadr [line] = 255;  /* every address      */
        tracetype [line] = 255; /* every message type */
    }


int stopmodbus (short ln)
{
	if (comm_disconnect (link[ln]) != -1)
	{
	    taskDelete (taskId [ln]);
	    return (1);
	}
	else
	{
	    return (-1);
	}
}

int startmodbus (unsigned short tlink,      /* the line number of this driver.                      */
                  char 		*mode,      /* UART initializations string (comma delimited ascii). */
                  unsigned short delay,     /* intermessage delay value, expressed in ticks.        */
                  unsigned short hdflag,    /* half duplex flag. 1=half duplex mode, 1!=full duplex */
                  unsigned short delay1,    /* half duplex post RTS assert, pre TX delay.           */
                  unsigned short delay2,    /* half duplex post TX, pre RTS drop delay.             */
                  unsigned short private)   /* private flag. if 0, normal. if !0, no partner driver.*/

{
    unsigned short	line;

	if (tlink >=FILELINESTART) {
	    line = tlink - FILELINESTART;
	    if (line >= MODLINES)
		return(-1); 
	} else {
	    line = tlink;
	}


	if (line < MODLINES)
	{
	    strncpy (modes[line],mode,sizeof(modes[line]));

	    link[line] = tlink;
	    microsolflag [line] = FALSE;
	    hathawayflag [line] = FALSE;
	    taskId [line] = taskSpawn (tasknames [line],
                                     MOD_PRIORITY,
                                     MOD_TASK_OPTIONS,
                                     MOD_STACK,
                                     (FUNCPTR) mod_driver,
                                     line,
                                     (int) modes[line],
                                     delay,
                                     hdflag,
                                     delay1,
                                     delay2,
                                     private,
				     0, 0, 0);

	    logMsg ("Started the MODBUS driver on line %d\n",line, 0,0,0,0,0);

	    remCurIdGet (user,password);
	    return (1);
	}
	else
	{

	    logMsg ("Invalid line number %d\n", line, 0,0,0,0,0);
	    return (-1);
	}
}

int startmicrosol (unsigned short line,      /* the line number of this driver.                      */
                  char *mode,               /* UART initializations string (comma delimited ascii). */
                  unsigned short delay,     /* intermessage delay value, expressed in ticks.        */
                  unsigned short hdflag,    /* half duplex flag. 1=half duplex mode, 1!=full duplex */
                  unsigned short delay1,    /* half duplex post RTS assert, pre TX delay.           */
                  unsigned short delay2,    /* half duplex post TX, pre RTS drop delay.             */
                  unsigned short private)   /* private flag. if 0, normal. if !0, no partner driver.*/
{
	if (line < MODLINES)
	{
	    microsolflag [line] = TRUE;
	    hathawayflag [line] = FALSE;
	    taskId [line] = taskSpawn (tasknames2 [line],
                                     MOD_PRIORITY,
                                     MOD_TASK_OPTIONS,
                                     MOD_STACK,
                                     (FUNCPTR) mod_driver,
                                     line,
                                     (int) mode,
                                     delay,
                                     hdflag,
                                     delay1,
                                     delay2,
                                     private,
				     0, 0, 0);

	    logMsg ("Started the MICROSOL driver on line %d\n",line, 0,0,0,0,0);

	    remCurIdGet (user, password);
	    return (1);
	}
	else
	{
	    logMsg ("Invalid line number %d\n", line, 0,0,0,0,0);
	    return (-1);
	}
}
int startmodhia (unsigned short tlink,      /* the line number of this driver.                      */
                  char *mode,               /* UART initializations string (comma delimited ascii). */
                  unsigned short delay,     /* intermessage delay value, expressed in ticks.        */
                  unsigned short hdflag,    /* half duplex flag. 1=half duplex mode, 1!=full duplex */
                  unsigned short delay1,    /* half duplex post RTS assert, pre TX delay.           */
                  unsigned short delay2,    /* half duplex post TX, pre RTS drop delay.             */
                  unsigned short private)   /* private flag. if 0, normal. if !0, no partner driver.*/

{
	short	line;

	if (tlink >=FILELINESTART) {
	    line = tlink - FILELINESTART;
	    if (line >= MODLINES)
		return(-1); 
	} else {
	    line = tlink;
	}

	if (line < MODLINES)
	{
	    strncpy (modes[line],mode,sizeof(modes[line]));

	    link[line] = tlink;	    microsolflag [line] = FALSE;
	    hathawayflag [line] = TRUE;
	    taskId [line] = taskSpawn (tasknames3 [line],
                                     MOD_PRIORITY,
                                     MOD_TASK_OPTIONS,
                                     MOD_STACK,
                                     (FUNCPTR) mod_driver,
                                     line,
                                     (int) modes[line],
                                     delay,
                                     hdflag,
                                     delay1,
                                     delay2,
                                     private,
				     0, 0, 0);

	    logMsg ("Started the Modicon-Hathaway Interface MODHIA driver on line %d\n",line, 0,0,0,0,0);

	    remCurIdGet (user,password);
	    return (1);
	}
	else
	{

	    logMsg ("Invalid line number %d\n", line, 0,0,0,0,0);
	    return (-1);
	}
}
/********
**
** Set communication logic time delays to handle half/full duplex, radio delays, etc 
**
********/
void modcomm (unsigned short line,
              unsigned short tickcount1,
              unsigned short hdflag,
              unsigned short tickcount2,
              unsigned short tickcount3)
{
	intermsgdelay [line] = tickcount1;  /* at beginning new message/response sequence.               */
	halfduplex [line]    = hdflag;
	pretxdelay [line]    = tickcount2;  /* after asserting RTS but before beginning transmission.    */
	posttxdelay [line]   = tickcount3;  /* after sending last character but before dropping RTS.     */
}

int modResGroup (unsigned short line, unsigned short group)
{
    if (line < MODLINES)
    {
        if (group <= MAXGROUPNUM)
        {
            modResGroups [line] = group;
            logMsg ("Assigned Line %d to Group %d\n", line, group, 0,0,0,0);
            return (1);
        }
        else
        {
            logMsg ("Invalid Group Assignment: line %d: group number must be less than %d.\n", line, MAXGROUPNUM+1, 0,0,0,0);
            return (-1);
        }
    }
    else
    {
        logMsg ("Invalid Group Assignment: line %d: line number must be less than %d\n", line, MODLINES, 0,0,0,0);
        return (-1);
    }
}

void modgrouptable (void)
{
    unsigned short x, y, i, j, index;

    printf (" Resource Group Lock Manager:  Limitorque Driver Line to Group Map Table\n");
    printf (" -----------------------------------------------------------------------\n");
    printf ("  Ln:Grp   Ln:Grp   Ln:Grp   Ln:Grp   Ln:Grp   Ln:Grp   Ln:Grp   Ln:Grp \n");

    x = 8;
    y = MODLINES/x;

    for (i = 0; i < y; i++)
    {
        for (j = 0; j < x; j++)
        {
            index = i + j*y;
            printf ("  %d%d:%d%d  ",
                    index/10,
                    index%10,
                    modResGroups [index]/10,
                    modResGroups [index]%10);
        }
        printf ("\n");
    }
}

void modlocktable (void)
{
    unsigned short x, y, i, j, index;

    printf (" Resource Group Lock Manager Global Array\n");
    printf (" ----------------------------------------\n");
    printf ("  Grp:Flg   Grp:Flg   Grp:Flg   Grp:Flg\n");

    x = 4;
    y = MAXGROUPNUM/x;

    for (i = 0; i < y; i++)
    {
        for (j = 0; j < x; j++)
        {
            index = i + j*y;
            printf ("  %d%d:%d%d   ",
                    (index+1)/10,
                    (index+1)%10,
                    resGroupLock [index]/10,
                    resGroupLock [index]%10);
        }
        printf ("\n");
    }
}

void modlockall (void)
{
    unsigned short i;

    printf (" Resource Group Lock Manager Global Array\n");
    printf (" ----------------------------------------\n");
    printf ("  Group  Flag  Trip Name\n");
    for (i = 0; i < MAXGROUPNUM; i++)
    {
        printf ("   %d%d     %d%d",
                    (i+1)/10,
                    (i+1)%10,
                    resGroupLock [i]/10,
                    resGroupLock [i]%10);
        if (resGroupTripName[i])
        {
            printf ("   %s\n", resGroupTripName[i]);
        }
        else
        {
            printf ("\n");
        }
    }
}

void modsetlock (unsigned short group)
{
    if (group <= MAXGROUPNUM)
    {
        printf ("Setting Resource Lock Flag for Group %d.\n", group);
        taskDelay(60);
        resGroupLock [group -1] = 1;
    }
    else
    {
        logMsg ("Invalid Group %d: group number must be less than %d.\n", group, MAXGROUPNUM+1, 0,0,0,0);
    }
}

void modclrlock (unsigned short group)
{
    if (group <= MAXGROUPNUM)
    {
        printf ("Clearing Resource Lock Flag for Group %d.\n", group);
        taskDelay(60);
        resGroupLock [group -1] = 0;
    }
    else
    {
        logMsg ("Invalid Group %d: group number must be less than %d.\n", group, MAXGROUPNUM+1, 0,0,0,0);
    }
}

void modgrouphelp (void)
{
    printf (" Modicon Driver Group Related Functions\n");
    printf ("\n");
    printf (" modResGroup(N, G)     Assign line N to group G (valid G range is 1 to 16)         \n");
    printf (" modgrouptable         Display the Limitorque's line to group assignment table.    \n");
    printf (" modlocktable          Display the resource group lock Global Array.               \n");
    printf (" modlockall            Display the resource group lock and trip name Global Array. \n");
    printf (" modsetlock (G)        Set the resource group lock for group G. (1 to 16)          \n");
    printf (" modclrlock (G)        Clear the resource group lock for group G. (1 to 16)        \n");
    printf (" modgrouphelp          Display this help message.                                  \n");
    printf ("\n");
}

/********
**
** Show Driver Info and Current State Information 
**
********/
void modinfo (unsigned short line)
{
    printf ("intermsgdelay [%d]  = %d\n", line, intermsgdelay[line]);
    printf ("halfduplex [%d]     = %d\n", line, halfduplex[line]);
    printf ("pretxdelay [%d]     = %d\n", line, pretxdelay[line]);
    printf ("posttxdelay [%d]    = %d\n", line, posttxdelay[line]);
    printf ("pryvate [%d]        = %d\n", line, pryvate[line]);
    printf ("resresponse [%d]    = %d\n", line, resresponse[line]);
    printf ("activity_time [%d]  = %ld\n", line, activity_time[line]);
    printf ("line_activity [%d]  = %d\n", line, line_activity[line]);
    printf ("lastactivity [%d]   = %d\n", line, lastactivity[line]);
    printf ("modtrace [%d]       = %d\n", line, modtrace[line]);
    printf ("mltrace [%d]        = %d\n", line, mltrace[line]);
    printf ("modrestrace [%d]    = %d\n", line, modrestrace[line]);
}


/*
**      MOD_driver
**
**      The purpose of this task is to scan the modbus plc
**      lines. It will maintain a configuration list supporting online additions
**      and replacement of modules, and maintain a table of most recently
**      scanned values.
**
*/                                                                                     	
/* not called from within this driver 	*/

void mod_driver (unsigned short line,
           unsigned char *mode,
           unsigned short delay,
           unsigned short hdflag,
           unsigned short delay1,
           unsigned short delay2,
           unsigned short private)
{
	unsigned short  mod, cfgchk, cnt, modspolled, onlinefreq;
	unsigned short  activity, pstate, lstate, execute, command,i;
	long	        this_time, last_time;
	MODPLC          *pmod;
	RXLINE          *rxp;
        unsigned int    temptick, tempdiff;
        int             temp;
        unsigned short  group;

	activity = 0;	/* assign a default value */

/* Initialize the data structures */
	intermsgdelay [line] = delay;

	halfduplex [line]  = hdflag;
	pretxdelay [line]  = delay1;
	posttxdelay [line] = delay2;
        pryvate [line]     = private;

	linfo [line].loading = FALSE;

	resresponse [line] = FALSE;
	resnames [line] [0] = 0;
	prestable [line] = NULL;

	activity_time [line] = tickGet ();
	lastactivity [line] = line_activity [line] = FALSE;
	nextouttime [line] = 0;
	outagain [line] = 0;

        pstatewait[line] = 60;      /* will wait up to one minute for partner to become valid */

        /* debug structures */
/*        moddebugtrace (line); */
        moddefaulttrace (line);

        tick [line] [0] = tick [line] [1] = tick [line] [2] = tick [line] [3] = 0;

	linestats [line].TXintcount = 0;
	linestats [line].RXintcount = 0;
	linestats [line].RXbadcount = 0;
	linestats [line].msgxfercount = 0;
        linestats [line].startingtick = tickGet ();

        modsim [line] = FALSE;              /* start with normal operation. */

        /* output queue initialization */

	oq [line] = (OUTPUTQUE *) calloc (1, (sizeof (OUTPUTQUE)));
	if (!oq [line])
	{
	    logMsg ("Not enough memory for line %d data structure (data output queue)\n",line, 0,0,0,0,0);
	    return;
	}
        outqueinit (line);

        /* communication initialization */

	rxlinetab [line] = (RXLINE *) calloc (1, (sizeof (RXLINE)));
	if (!rxlinetab [line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: rx)\n",line, 0,0,0,0,0);
	    return;
	}
	txlinetab [line] = (TXLINE *) calloc (1, (sizeof (TXLINE)));
	if (!txlinetab [line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: tx)\n",line, 0,0,0,0,0);
	    return;
	}

	/* Debug task hold off */
	if (modtaskhold) {
	    logMsg ("Modicon driver for line %d waiting for modtaskunlock\n",line,0,0,0,0,0);
	    while (modtaskhold) {
		taskDelay(10);
	    }
	}


        taskLock ();  /* make terminal display easier to read - avoids interleaving of other task data */
	comm_init (link[line], mode, (FUNCPTR)tx_int_proc, (FUNCPTR)rx_int_proc);
	clear_rts (link[line]);
        taskUnlock ();

	onlinefreq = 1;	  /* causes on_line calls to begin on first pass if master */

	/* Initialize the Rx interrupts to receive once for activity check         */
	/* enable the receiver interrupt after set to ignore                       */

	rxp = rxlinetab [line];
	rxp->rxstate = RXIGNORE;
	enable_rx_ints (link[line]);

        /* Initialize the line communication semaphores */
        semids [line] [0] = semBCreate (SEM_Q_PRIORITY, SEM_EMPTY);
        semids [line] [1] = semBCreate (SEM_Q_PRIORITY, SEM_EMPTY);

	/* Wait for resource to be defined via an initial call to 'register_resource' via driver call.  */
	while ((resnames [line] [0] == 0) || (prestable [line] == NULL))
	{
            if (modrestrace [line] > 1)
	    {
	    	logMsg ( "No Resource Defined for line - %d \n", line, 0,0,0,0,0);
	    }
	    taskDelay (120);
	}
	if (pryvate [line])
        {
            /* mark the restable as private. */
            rmPrivate (resnames [line]);

            /*****************************/
	    /* Immediately become master */
            /*****************************/

	    resSetCaAnswer (prestable [line]);
	    resPutLocalState (prestable [line], RES_MSTR);

            lastscan [line] = tickGet ();
            scantime [line] = 0;

	    lstate = RES_MSTR;
	    if (modrestrace [line])
	    {
	    	logMsg ("0] Private: Initialize as RES_MSTR\n", 0,0,0,0,0,0);
	    }
        }
        else /* this is the 'normal' path */
        {
            /**********************/
            /* Start out as slave */

            resClearCaAnswer (prestable [line]);		/* start quiet */
            resPutLocalState (prestable [line], RES_SLV);

            if (modrestrace [line])
            {
                logMsg ("1] Initialized as RES_SLV\n", 0,0,0,0,0,0);
            }

            /* wait finite time for pstate to be resolved */
            logMsg ("Begin wait for pstate [%d] to become valid.\n", pstatewait [line], 0,0,0,0,0);
            for (i = 0; i <= pstatewait [line]; i++)
            {
                resGetStates (prestable [line], &lstate, &pstate);
                if (pstate != 0)
                {
	            break;
                }
                taskDelay (60);
            }
            if (pstate == 0)
            {
                logMsg ("Timeout of pstate wait.\n", 0,0,0,0,0,0);
            }
            else
            {
                logMsg ("Completion of pstate wait.\n", 0,0,0,0,0,0);
            }

            for (i = 0; i < ACTIVITYTIME/10; i++) ;  /* ten seconds */
            {
                taskDelay(10);
            }

            activity = check_line_activity(line, 0);
            resGetStates (prestable [line], &lstate, &pstate);

            /******************************************/
            /* if the partner is not there, be master */

            if ((pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity))
            {
                resSetCaAnswer (prestable [line]);
                resPutLocalState (prestable [line], RES_MSTR);
                lastscan [line] = tickGet ();
                scantime [line] = 0;

                lstate = RES_MSTR;
                if (modrestrace [line])
                {
                    logMsg ("2] Became RES_MSTR   P= %x A= %d \n", pstate, activity, 0,0,0,0);
                }
            }
        }

	cfgchk = 0;     /* Init the new config pointer */
	last_time = tickGet (); /* Initialize the time  */

	while (TRUE)
	{
	    activity_time [line] = tickGet ();
	    lastactivity [line] = line_activity [line] = FALSE;

            /************************************/
            /* check for line group disablement */
            /************************************/
            group = modResGroups [line];
            if ((group) && (resGroupLock [group-1]))
            {
                if (modrestrace [line])
                {
                    logMsg ("13] Forced to RES_SLV.\n", 0,0,0,0,0,0);
                }
                resClearCaAnswer (prestable [line]);
                resPutLocalState (prestable [line], RES_SLV);
                lstate = RES_SLV;
                resresponse [line] = FALSE; 	/* Make Sure driver is not responding   */
                if (modrestrace [line])
                {
                    logMsg ("14] Resource Group Locked     P= %x A= %d \n", pstate, activity, 0,0,0,0);
                }
                while (resGroupLock [group-1])
                {
                    group = modResGroups [line];
                    if (group)
                    {
                        clear_rts (link[line]); 		/* Make sure line is left for master    */
                        taskDelay (2);
                    }
                    else /* the line has been taken out of the group */
                    {
                        break;
                    }
                }
                if (modrestrace [line])
                {
                    logMsg ("15] Resource Group Unlocked   P= %x A= %d \n", pstate, activity, 0,0,0,0);
                    logMsg ("16] Released to RES_SLV.\n", 0,0,0,0,0,0);
                }
            }



            /********************************************************************************************/
	    /* we arrive here because we are a slave.                                                   */
	    /* we stay until:   asked to be master by partner. if we can reach PLCs, we become master.  */
            /*                  the partner is not master and there is no activity on the line.         */
            /********************************************************************************************/
	    while (lstate == RES_SLV)
	    {
                /* check for plc simulated mode. */
                if (modsim [line])
                {
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;

                    lastscan [line] = tickGet ();
                    scantime [line] = 0;

		    if (modrestrace [line])
		    {
                        logMsg ("Simulation Mode] Became RES_MSTR  P= %x A= %d \n",pstate,activity, 0,0,0,0);
		    }
		    break;
                }

		if (pstate != RES_SLVWANT)
		{
		    clear_rts (link[line]); 				/* make sure line is left for master 	*/
		}
		resresponse [line] = FALSE; 			/* make sure driver is not responding 	*/
		resPutCommand (prestable [line], 0); 		/* clear out commands 			*/
		activity = check_line_activity (line, 10); 	/* wait 10 seconds before checking 	*/
		resGetStates (prestable [line], &lstate, &pstate);

                /************************************/
                /* check for line group disablement */
                if (checkgroup (line, &lstate, 17)) break;

                /****************************************************************************************/
		/* see if needed by partner to be master. if yes, do it and break out of slave state.   */
		if (pstate == RES_SLVWANTCA)
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;

                    lastscan [line] = tickGet ();
                    scantime [line] = 0;

		    if (modrestrace [line])
		    {
			logMsg ("3] Became RES_MSTR  P= %x A= %d \n",pstate,activity, 0,0,0,0);
		    }
		    break;
		}
                /***************************************************************/
		/* If partner is there and wants to give it up, try to take it */

		if (pstate == RES_SLVWANT)
		{
		    execute = FALSE;
		    taskDelay (2); /* Time for switch to occur */

		    /* See if anyone is out there and able to respond to a loopback command */
		    for (mod = 1; (mod <= NUMMODPLCS); mod++)
		    {
                        /************************************/
                        /* check for line group disablement */
                        if (checkgroup (line, &lstate, 18)) break;

			pmod = plcs [line] [mod];
			if ((pmod) &&			/* verify pointer validity prior to use */
			    (pmod->was_requested) &&    /* data points have been requested      */
                            (!pmod->cmd_force_offline)) /* user has NOT forced plc offline      */
			{
			    on_line (line, mod);
                            /* check cmd_force_offline flag before AND after due to asynchronous tasks */
                            if ((pmod->is_responding) && (!pmod->cmd_force_offline))
			    {
				execute = TRUE;
				break;
			    }
			}
		    }
                    /************************************/
                    /* check for line group disablement */
                    if (checkgroup (line, &lstate, 19)) break;

		    /* If someone answered, become master and make sure partner really gave it up */
		    if (execute)
		    {
			resPutLocalState (prestable [line], RES_MSTR);
			lstate = RES_MSTR;

                        lastscan [line] = tickGet ();
                        scantime [line] = 0;

			if (modrestrace [line])
			{
				logMsg ("4] Became RES_MSTR  P= %x A= %d \n",pstate,activity, 0,0,0,0);
			}
			execute = FALSE;
			activity_time [line] = tickGet ();
			lastactivity [line] = line_activity [line] = FALSE;
			for (i = 0; (i < 60); i++)
			{
			    activity = check_line_activity (line, 2); /* pause before testing */
			    resGetStates (prestable [line], &lstate, &pstate);
			    if ((pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity) )
			    {
				execute = TRUE; /* Partner Gave It Up so take CA */
				break;
			    }
			}
			if (!execute)
			{   /* Partner did not give it up go back to slave */
			    resPutLocalState (prestable [line], RES_SLV);
			    lstate = RES_SLV;
			    if (modrestrace [line])
			    {
			    	logMsg ("5] Became RES_SLV  P= %x A= %d \n",pstate,activity, 0,0,0,0);
			    }
			}
			else
			{ /* Assume control by taking the CA responsibilities */ 
			    resSetCaAnswer (prestable [line]);
			    prepresponses (line);
			    break;
			}
		    }
		}

                /********************************************/
		/* if the partner is not talking, be master */

		if ((pstate != RES_MSTR) &&  (pstate != RES_SLVWANT) && (!activity))
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;

                    lastscan [line] = tickGet ();
                    scantime [line] = 0;

		    if (modrestrace [line])
		    {
		    	logMsg ("6] Became RES_MSTR P= %x A= %d \n",pstate,activity, 0,0,0,0);
		    }
		    prepresponses (line);
		}
	    }

            /****************************************************************************************/
            /* to arrive here, we used to be master, and had been told to relinquish master status. */
	    /* stay in this loop until the partner takes over, or the command is canceled */
	    while (lstate == RES_SLVWANTCA)
	    {
	        taskDelay (2);                   /* Give OS a chance...  */
   
                /************************************/
                /* check for line group disablement */
                if (checkgroup (line, &lstate, 20)) break;

		clear_rts (link[line]); 		/* Make sure line is left for master    */
		resresponse [line] = FALSE; 	/* Make Sure driver is not responding   */

		resGetStates (prestable [line], &lstate, &pstate);
		if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT))
		{
		    resClearCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_SLV);
		    lstate = RES_SLV;
		    if (modrestrace [line])
		    {
		    	logMsg ("7] Became RES_SLV  P= %x A= %d \n",pstate,activity, 0,0,0,0);
		    }
		    break;
		}
		resGetCommand (prestable [line], &command);
		resPutCommand (prestable [line], 0); /* clear out commands */
		if (command == RES_CANCELREL)
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;
		    if (modrestrace [line])
		    {
		    	logMsg ("8] Became RES_MSTR  P= %x A= %d \n",pstate,activity, 0,0,0,0);
		    }
		    prepresponses (line);
		}
	    }

            /****************************************************************************************/
	    /* in normal operation this code should never execute. it has, however, been seen to    */
            /* execute when the database has been inadvertently configured with the same resource   */
	    /* assigned to two different drivers in the same IOC. The state does funny things.      */
            /****************************************************************************************/
	    while (lstate == RES_SLVWANT)
	    {
		if (checkgroup (line, &lstate, 21)) break;
                if ((modrestrace [line]) || (trace[line] & CONFIGURATIONTRACE))

		{
                    /* condition should NEVER occur. print error message every 5 seconds. */
                    logMsg ("Illegal Local Resource State.\n", 0,0,0,0,0,0);
                    if (trace[line] & CONFIGURATIONTRACE)
                    {
                        logMsg ("  Possible Resource Configuration Error:\n", 0,0,0,0,0,0);
                        logMsg ("   Check for multiple drivers sharing single Resource.\n", 0,0,0,0,0,0);
                    }
                    taskDelay (600);
                }
                taskDelay (2);
 		resGetStates (prestable [line], &lstate, &pstate);
            }
            /****************************************************************************************/
            /* we come here because we have gone to, or are looping in, the master state.           */
	    /* we stay in this loop until:    we have been told to give master state to the partner.*/
	    /*                                we break out to RES_SLVWANTCA.                        */
	    /*                                we have successfully handed master state to partner.  */
	    /* While master, this task will process all modules sending outputs and getting inputs. */
	    /* It will then try to configure requested non-responding modules.                      */
            /****************************************************************************************/
	    last_time = tickGet ();		/* initialize gap */
	    while (lstate == RES_MSTR)
	    {
		if (mltrace [line] == 1)
		{
			logMsg ( "Top of Master Loop... \n", 0,0,0,0,0,0);
	        }
                /* check for plc simulated mode. */
                while (modsim [line])
                {
		    resresponse [line] = TRUE; /* Make sure driver is responding. driver won't         */
		    for (mod = 1; (mod <= NUMMODPLCS); mod++)
		    {
		        pmod = plcs [line] [mod];	/* shorthand pointer 			*/
		        if (pmod)			/* verify pointer validity prior to use */
		        {
                            /************************************/
                            /* check for line group disablement */
                            if (checkgroup (line, &lstate, 22)) break;

			    if (pmod->cmd_force_offline)
			    { 
				/* if user has turned this plc offline via pseudo commands,     */
                                /* mark it as not responding.                                   */
 				pmod->is_responding = FALSE;
	    		    }
			    else
			    {
				sim_plc (pmod, line, mod);      /* I/O done here! 		*/
		   		taskDelay (simdelay [line]);
                            }
			}
		    }
                    /************************************/
                    /* test for line group disablement. */
                    if (lstate == RES_LOCKED) break;

                }
                /************************************/
                /* test for line group disablement. */
                if (lstate == RES_LOCKED) break;

		/* initial stuff */
		resresponse [line] = TRUE; /* Make sure driver is responding. driver won't           */
                                           /* issue commands if not set.                             */

		resGetCommand (prestable [line], &command);
		resPutCommand (prestable [line], 0);        /* clear out commands */

                /************************************************************************/
		/* if relinquish command sent, change state and break out to outer loop */
		if (command == RES_RELINQUISH)
		{
		    resClearCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_SLVWANTCA);
		    lstate = RES_SLVWANTCA;
		    if (modrestrace [line])
		    {
                       logMsg ("9] Became RES_SLVWANTCA  P= %x A= %d \n",pstate,activity, 0,0,0,0);
		    }
		    break;
		}

                /* measure time between scans of the entire set of plcs. display if requested. */ 
                temptick = tickGet ();
                tempdiff = temptick - lastscan [line];

                if (modtrace [line])
		{
                    logMsg ("ScanTime: =%4d, # OutputMsgs =%5d, # TotalMsgs =%3d\n",
                            scantime[line], loopoutmsgs[line], loopmsgs[line], 0,0,0);
		}

                loopoutmsgs [line] = 0;
                loopmsgs [line] = 0;
                lastscan [line] = temptick;

		/* scan plcs for data i/o, if allowed */

		modspolled = 0;

		/* call 'scan_plc' for each PLC device, unless user has */
                /* taken it off line or it is not responding.           */

		for (mod = 1; (mod <= NUMMODPLCS); mod++)
		{
		    pmod = plcs [line] [mod];		/* shorthand pointer 			*/
		    if (pmod)				/* verify pointer validity prior to use */
		    {
                        /************************************/
                        /* check for line group disablement */
                        if (checkgroup (line, &lstate, 23)) break;

			if (pmod->cmd_force_offline)
			{ 
			    /* if user has turned this plc offline via pseudo commands,     */
                            /* mark it as not responding.                                   */
 			    pmod->is_responding = FALSE;
	    		}
			else if (pmod->is_responding)
			{
                            /* local/partner lockstep recovery */

                            resGetStates (prestable [line], &lstate, &pstate);
                            if ((pstate == RES_MSTR) && (!pryvate[line]))
                            {
                                resClearCaAnswer (prestable [line]);
                                resPutLocalState (prestable [line], RES_SLV);
                                lstate = RES_SLV;
                                clear_rts (link[line]); 		/* Make sure line is left for master */
                                if (modrestrace [line])
				{
                                    logMsg ("13] Became RES_SLV  P= %x A= %d \n", pstate, activity, 0,0,0,0);
				}
                                temp = ((tickGet () % 8) + 5); /* random 5 to 12 second delay */
                                for ( i = 0; i < temp; i++)
                                {
                                   taskDelay(60);
                                }
                                break;
                            }
			    modspolled++;

			    scan_plc (pmod, line, mod);      	/* I/O done here! 		*/
			    if (!pmod->is_responding)
			    {
				mark_not_valid (pmod);
                            }
			}
		    }
		    /* end of single module action */

		    /* if told to relinquish our master state, stop plc scanning immediately. */
		    /* don't trash the command, so that we can still read it on the next loop */
                    /* through this while statement.                                          */
		    resGetCommand (prestable [line], &command);
		    if (command == RES_RELINQUISH)
		    {
			break;
		    }
		} /* end of scan loop */

                /* test for line group disablement. */
                if (lstate == RES_LOCKED) break;
                /* test for lockstep detection. */
                if (lstate == RES_SLV) break;

                /* measure how long the scan took. to be displayed at beginning of next loop. */
                temptick = tickGet ();
                scantime [line] = temptick - lastscan [line];

		/* try and bring all nonresponding plcs on line, but */
                /* don't bring in more than a specified number.      */

		cnt = 0;
		if (--onlinefreq == 0) 		/* do we try to bring any in on this pass? */
		{ 
		    onlinefreq = BRINGBACKCNT;
		    for (mod = 1; (mod <= NUMMODPLCS); mod++)
		    {
                        /************************************/
                        /* check for line group disablement */
                        if (checkgroup (line, &lstate, 24)) break;

			cfgchk++;		/* manage plc index */
			if (cfgchk == NUMMODPLCS + 1)
			{
			    cfgchk=1;	/* wrap plc index */
                        }
			pmod = plcs [line] [cfgchk];
			if ((pmod) &&		/* verify pointer validity prior to use*/
			    (!pmod->is_responding) &&
			    (pmod->was_requested) &&
			    (!pmod->cmd_force_offline))
		    	{
			    on_line (line, cfgchk);	/* loopback testing done here! 	*/

			    modspolled++;
			    cnt++;
                            if (cnt == MAXCFGCK)
			    {
				break;
			    }
			}
		    }
		}

                /* test for line group disablement. */
                if (lstate == RES_LOCKED) break;

		/* see if anyone on the line has been responding. */
                /* if not, try to get partner to become slave.    */
		/* Check for no one responding */

		if (!any_responding (line) && (!pryvate[line]))
		{
		    /* See if it is time to let partner try */

		    this_time = tickGet ();
		    if ((this_time - last_time) > (3 * SLWWINDOW))
		    {   /* #ticks for slave to try */

			/* Switch States to let partner know it's its time to try */
			resPutLocalState (prestable [line], RES_SLVWANT);
			lstate = RES_SLVWANT;
			if (modrestrace [line])
			{
			    logMsg ("10] Became RES_SLVWANT  P= %x A= %d \n",pstate, activity, 0,0,0,0);
			}
			/* Give partner access to the hardware */
			clear_rts (link[line]);

			/* Remember the time of this try */
			last_time = tickGet ();
			lastactivity [line] = line_activity [line] = FALSE;
			for (i = 0; (i < SLWWINDOW); i += 10)
			{       		/* SLWINDOW is the # of ticks for slave to try */
                            /************************************/
                            /* check for line group disablement */
                            if (checkgroup (line, &lstate, 25)) break;

			    taskDelay (10);
		    				/* If partner picked up let it have control */
			    activity = check_line_activity (line, 0);  /* check immediately */
			    resGetStates (prestable [line], &lstate, &pstate);
			    if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT) || (activity))
			    {
				resClearCaAnswer (prestable [line]);
				resPutLocalState (prestable [line], RES_SLV);
				lstate = RES_SLV;
				if (modrestrace [line])
				{
				    logMsg ("11] Became RES_SLV  P= %x A= %d \n",pstate,activity, 0,0,0,0);
				}
                                taskDelay(60);
                                taskDelay(60);
                                taskDelay(60);
				break;
			    }
			}

                	/* test for lockstep detection or line group disablement. */
                	if (lstate == RES_LOCKED) break;

			/* Go Back to being the Master and take control of the hardware */
			if (lstate != RES_SLV)
			{
                            resSetCaAnswer (prestable [line]);
                            resPutLocalState (prestable [line], RES_MSTR);
			    lstate = RES_MSTR;
			    if (modrestrace [line])
			    {
			    	logMsg ("12] Became RES_MSTR  P= %x A= %d \n",pstate,activity, 0,0,0,0);
			    }
			}
		    }
		}

		/* take plcs off hold status. the plc's data are not to be held off since now master. */
		for (mod = 1; (mod <= NUMMODPLCS); mod++ )
		{
		    pmod = plcs [line] [mod];
		    if (pmod)			/* verify pointer validity prior to use */
		    {        
			pmod->holdflag = FALSE;
                    }
		}
                if (modspolled == 0)
		{
                   taskDelay(6);
		}
                else
		{
		    taskDelay(2);
		}
	    }
	} /* end infinite task loop */
}


/************************************/
/* check for line group disablement */
/************************************/
int checkgroup (unsigned short line, unsigned short *lstate, unsigned short tracenum)
{
    unsigned short group;

    group = modResGroups [line];
    if ((group) && (resGroupLock [group-1]))
    {
        *lstate = RES_LOCKED;
        if (modrestrace [line])
        {
            logMsg ("%d] Resource Group Lock Detected.\n", tracenum, 0,0,0,0,0);
        }
        return (TRUE);
    }
    return (FALSE);
}


/*
** Try to bring a PLC on line
**
*/

/* this routine only called from mod_driver */


static void on_line (unsigned short line,unsigned short mod)
   {
	MODPLC		*pmod;

	pmod = plcs [line] [mod];	
	if (pmod)			/* verify pointer prior to use */
	{
		if (mltrace [line] == 1)
		{
			logMsg ( "Bring Online PLC %d\n", mod, 0,0,0,0,0);
	        }
		/* Use the loop back function to see if the unit is there and responding */
                if (hathawayflag [line])
                {
                    if (!hiamodloopback (ONERETRY + line, mod, pmod))
                        return; /* if no response, get out */
                }
                else
                {
                    if (!modloopback (ONERETRY + line, mod, pmod))
                        return; /* if no response, get out */
                }

		mark_not_valid (pmod);	/* a PLC that has just been brought online must have    */
                                        /* it's data marked as being not valid.                 */

		pmod->is_responding = TRUE;
        }
}
/*							
** Mark all plc data elements as not valid     		
**                                             		
*/                                             	
/* called from on_line and prepresponses      		*/    	

static void mark_not_valid (MODPLC  *pmod)
{
	unsigned	short i;

/*
**      COILS ARE INITIALIZED AS READ WRITE VALUES      
**              - READ BACK NOT VALID 			
*/
	for (i = 0; (i < NUMMODCOILS); i++)
	{
	    pmod->coils [i].data_valid = FALSE;
	}

/*
**      DIGITALS ARE INITIALIZED AS NOT VALID		
*/
	for (i = 0; (i < NUMMODDIS); i++)
	{
	    pmod->dis [i].data_valid = FALSE;
	}

/*
**      ANALOG INS ARE INITIALIZED AS NOT VALID		
*/
	for (i = 0; (i < NUMMODAIS); i++)
	{
	    pmod->ais [i].data_valid = FALSE;
	}

/*
**      REGISTERS ARE INITIALIZED AS READ WRITE VALUES		
**      WRITES ARE TREATED BY INDIVIDUAL REGISTERS NOT GROUPS 	
**              - READ BACK NOT VALID      	               	
*/
	for (i = 0; (i < NUMREGISTERS); i++)
	{
	    pmod->regs [i].data_valid = FALSE;
	}
}
/*                                                           	
** Scan a MODBUS PLC -  get inputs and set outputs as required	
**                                                            	
*/
/* this routine only called from mod_driver 		*/


static	void scan_plc (MODPLC *pmod, unsigned short line, unsigned short mod)
    {
	unsigned short	i, j, k, bitaddr, regaddr, readlen;
	LOADINFO	sslinfo;
        unsigned short  temp, cmd;

	bitaddr = 0; /* assign a default value */

	pmod->tscanpass++;

	if (mltrace [line] == 2)
	{
	    logMsg ( "Scanning PLC - %d.\n", mod, 0,0,0,0,0);
	}
	checkoutputs (line);

	/* Test to see if load requested or in progress */

	if (pmod->prog_load_stat == PLC_LOAD_REQUESTED)
	{
	    loadinit (line, pmod, mod);
	    return;
	}
	if (pmod->prog_load_stat == PLC_LOAD_IN_PROGRESS)
	{
	    loadnextblock (line);
	    return;
	}
	/* Test to see if a start or stop is requested */

	if (pmod->plc_stop)
	{
	    sslinfo.line = line;
	    sslinfo.mod = mod;
	    sslinfo.pmod = pmod;
	    if (plc_login (&sslinfo))
	    {
		plc_stop (&sslinfo);
	    }
	    plc_logout (&sslinfo);
	    pmod->plc_stop = FALSE;
	}
	if (pmod->plc_start)
	{
	    sslinfo.line = line;
	    sslinfo.mod = mod;
	    sslinfo.pmod = pmod;
	    if (plc_login (&sslinfo))
	    {
		plc_start (&sslinfo);
	    }
	    plc_logout (&sslinfo);
	    pmod->plc_start = FALSE;
	}

	/* read the coil values to be scanned */

	for (i = 0; (i < NUMMODCOILS); i++)
	{
	    if (hathawayflag [line])
		break;
	    if (pmod->coils [i].was_requested)
	    {
		for (j = 1; ((j < MAXDIGBLKSIZE) && ((i + j) < NUMMODCOILS )); j++)
		{
		    if (!pmod->coils [i + j].was_requested)
		    {
		    	break;
		    }
		}
		bitaddr = (i * 8);
		readlen = j * 8;
		temp = readcoils (line, mod, pmod, bitaddr, readlen);
                if (temp == CMDFAIL)
		{
		    return;
		}
                if ((temp == CMDNACK) || (temp == CMDREJECT))
                {
                    cmd = MCOILS;
                    if (trace[line] & MODBUSNACKTRACE)
                        logMsg (
                          "Modbus Command NACK: Disabling %d Points starting at L=%d,A=%d,C=%d,S=%d\n",
                           readlen, line, mod, cmd, bitaddr+1, 0);
                    for (k = 0 ; k < j ; k++)
                    {
                        pmod->coils [i + k].was_requested = FALSE;
                        pmod->coils[i + k].data_valid = FALSE;
                    }
                }
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the di values to be scanned */

	for (i = 0; (i < NUMMODDIS); i++)
	{
	    if (pmod->dis [i].was_requested)
	    {
		for (j = 1; ((j < MAXDIGBLKSIZE) && ((i + j) < NUMMODDIS )); j++)
		{
		    if (!pmod->dis [i + j].was_requested)
		    {
		    	break;
		    }
		}
		bitaddr = (i * 8);
		readlen = j * 8;
		temp = readdis (line, mod, pmod, bitaddr, readlen);
                if (temp == CMDFAIL)
		{
		    return;
		}
                if ((temp == CMDNACK) || (temp == CMDREJECT))
                {
                    cmd = MDINS;
                    if (trace[line] & MODBUSNACKTRACE)
                        logMsg (
                          "Modbus Command NACK: Disabling %d Points starting at L=%d,A=%d,C=%d,S=%d\n",
                           readlen, line, mod, cmd, bitaddr+1, 0);
                    for (k = 0; k < j; k++)
                    {
                        pmod->dis [i + k].was_requested = FALSE;
                        pmod->dis [i + k].data_valid = FALSE;
                    }
                }
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the ai values to be scanned */

	for (i = 0; (i < NUMMODAIS); i++)
	{
	    if (pmod->ais [i].was_requested)
	    {
		for (j = 1; ((j < MAXREGBLKSIZE) && ((i + j) < NUMMODAIS)); j++)
		{
		    if (!pmod->ais [i + j].was_requested)
		    	break;
		}
		regaddr = i * REGBLOCK;
		readlen = j * REGBLOCK;
		temp = readais (line, mod, pmod, regaddr, readlen);
                if (temp == CMDFAIL)
		{
		    return;
		}
                if ((temp == CMDNACK) || (temp == CMDREJECT))
                {
                    /* disable the entire range of NACKed points */
                    cmd = MAINS;
                    if (trace[line] & MODBUSNACKTRACE)
                        logMsg (
                          "Modbus Command NACK: Disabling %d Points starting at L=%d,A=%d,C=%d,S=%d\n",
                           readlen, line, mod, cmd, bitaddr+1, 0);
                    for (k = 0; k < j; k++)
                    {
                        pmod->ais [i + k].was_requested = FALSE;
                        pmod->ais [i + k].data_valid = FALSE;
                    }
                }
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the reg values to be scanned */

	for (i = 0; (i < NUMREGISTERS); i++)
	{
	    if (hathawayflag [line])
		break;	    if (pmod->regs [i].was_requested)
	    {
		for (j = 1; ((j < MAXREGBLKSIZE) && ((i + j) < NUMREGISTERS)); j++)
		{
		    if (!pmod->regs [i + j].was_requested)
		    {
		    	break;
		    }
		}
		regaddr = i * REGBLOCK;
		readlen = j * REGBLOCK;
		temp = readregs (line, mod, pmod, regaddr, readlen);
                if (temp == CMDFAIL)
		{
		    return;
		}
                if ((temp == CMDNACK) || (temp == CMDREJECT))
                {
                    cmd = MREGS;
                    if (trace[line] & MODBUSNACKTRACE)
                        logMsg (
                          "Modbus Command NACK: Disabling %d Points starting at L=%d,A=%d,C=%d,S=%d\n",
                           readlen, line, mod, cmd, bitaddr+1, 0);
                    for (k = 0; k < j; k++)
                    {
                        pmod->regs [i + k].was_requested = FALSE;
                        pmod->regs [i + k].data_valid = FALSE;
                    }
                }
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

}

static	void sim_plc (MODPLC *pmod, unsigned short line, unsigned short mod)
    {
	unsigned short	i, j;

	pmod->is_responding = pmod->was_requested;

	/* read the coil values to be scanned */

	for (i = 0; (i < NUMMODCOILS); i++)
	{
	    pmod->coils [i].data_valid = pmod->coils [i].was_requested;
	    if (simdigon [line])
	    {
		pmod->coils [i].last_read = ~pmod->coils [i].last_read;
	    }
	}

	/* read the di values to be scanned */

	for (i = 0; (i < NUMMODDIS); i++)
	{
	    pmod->dis [i].data_valid = pmod->coils [i].was_requested;
	    if (simdigon [line])
	    {
	        pmod->dis [i].last_read = ~pmod->dis [i].last_read;
	    }
	}

	/* read the ai values to be scanned */

	for (i = 0; (i < NUMMODAIS); i++)
	{
	    pmod->ais [i].data_valid = pmod->ais [i].was_requested;
	    for (j = 0; j < REGBLOCK; j++)
	    {
		if ((pmod->ais [i].aivals [j] < simlow [line]) ||
		    (pmod->ais [i].aivals [j] >= simhigh [line]))
		{
		    pmod->ais [i].aivals [j] = simlow [line];
		}
		else
		{
		    if (microsolflag [line])
		    {
			pmod->ais [i].aivals [j] += 16;
		    }
		    else
		    {
			pmod->ais [i].aivals [j] += 1;
		    }
		}
	    }
	}

	/* read the reg values to be scanned */

	for (i = 0; (i < NUMREGISTERS); i++)
	{
	    pmod->regs [i].data_valid = pmod->regs [i].was_requested;
	    for (j = 0; j < REGBLOCK; j++)
	    {
		if ((pmod->regs [i].ioregs [j] < simlow [line]) ||
		    (pmod->regs [i].ioregs [j] >= simhigh [line]))
		{
		    pmod->regs [i].ioregs [j] = simlow [line];
		}
		else
		{
		    if (microsolflag [line])
		    {
			pmod->regs [i].ioregs [j] += 16;
		    }
		    else
		    {
			pmod->regs [i].ioregs [j] += 1;
		    }
		}
	    }
	}
}
/*
** checkoutputs
**      checks output que and returns if empty.
**      if there is something on the queue it attempts to process all
**      pending outputs, doing immediate retries. If attempts fail the request is dropped.      
**
*/                                                                                              
/* this routine only called from scan_plc */

static	void checkoutputs (unsigned short line)
{
	OUTQENT         *optr;
	unsigned short  regaddr, regval, link, adapter, coiladdr, conv;
	unsigned long   coilmask, coilval;
	MODPLC          *pmod;
	double 		dblaovalue;
        unsigned short  stringaddr, stringlen;
	unsigned char	*pstringval;
	unsigned short	write_status;

        if (mltrace [line] == 6)
	{
            logMsg ("Checking Output Queue ...\n", 0,0,0,0,0,0); 
	}

	if (microsolflag [line])
	{
	    if (outagain [line] > tickGet ())
	    {
		return;
	    }
	    if (nextouttime [line] < tickGet ())
	    {
		if (write_aos (line))
		{
		    return;
		}
	    }
	}
	while ((optr = ogetnextp (line)) != NULLP)
	{
	    if ((optr->otype) == AOUTTYPE)
	    {
		link = optr->out.aonode.link;
		adapter = optr->out.aonode.adapter;
		pmod = plcs [link] [adapter];
		if (pmod == NULL)
		{
		    if (!allocate_plc (link, adapter))
		    {
			return; /* if not allocated, try. if can't allocate, get out. */
		    }
        	    pmod = plcs [link] [adapter];
		}
		regaddr = optr->out.aonode.aosignal;
		regval = optr->out.aonode.aovalue;
		dblaovalue = optr->out.aonode.dblaovalue;
		conv = optr->out.aonode.conversion;
                if (mltrace [link] == 7)
                {
                    logMsg("AO IN QUEUE: L=%d, A=%d,       S=%d, Value=%d\n",
                           link, adapter, regaddr, regval, 0, 0);
                } 

		if ((!pmod->cmd_force_offline) &&
                    (resresponse [link]) &&
                    (pmod->is_responding))
		{
                    loopoutmsgs [line]++;
		    write_status = setreg (link, adapter, pmod, regaddr, regval, dblaovalue, conv);
		    if (write_status != CMDDONE)
		    {
			if ((microsolflag [line]) && (write_status == CMDNACK))
			{
			    outagain [line] = tickGet () + 6; /* Wait 1/10 sec */
			    if (mltrace [line] == 15)
			    {
				logMsg ("Busy message received...Waiting 100 ms\n", 0,0,0,0,0,0);
			    }
			    return;
			}
		        pmod->badoutcnt++;		    
                        if (mltrace [link] == 3)
			{
                            logMsg("AO DEQUEUE: Output failed: L=%d, A=%d,       S=%d, Value=%d\n",
                                   link, adapter, regaddr, regval, 0, 0);
			} 
		    }
                    else
		    {
                        if (mltrace [link] == 3)
			{
                            logMsg ("AO DEQUEUE: Output succeeded: L=%d, A=%d,       S=%d, Value=%d\n",
                                   link, adapter, regaddr, regval, 0, 0);
			} 
		    }

		}
	    }
	    else if ((optr->otype) == BOUTTYPE)
	    {
		link = optr->out.bonode.link;
		adapter = optr->out.bonode.adapter;
		pmod = plcs [link] [adapter];
		if (pmod == NULL)
		{
		    if (!allocate_plc (link, adapter))
		    {
			return; /* if not allocated, try. if can't allocate, get out. */
		    }
        	    pmod = plcs [link] [adapter];
		}
		coiladdr = optr->out.bonode.bosignal;
		coilval = optr->out.bonode.bovalue;
		coilmask = optr->out.bonode.bomask;
                if (mltrace [link] == 7)
                {
                    logMsg("BO IN QUEUE: L=%d, A=%d,       S=%d, Value=%d\n",
                           link, adapter, coiladdr, coilval, 0, 0);
                }

		if ((!pmod->cmd_force_offline) &&
                    (resresponse [link]) &&
                    (pmod->is_responding))
		{
                    loopoutmsgs [line]++;
		    write_status = setcoil (link, adapter, pmod, coiladdr, coilmask, coilval);
		    if (write_status != CMDDONE)
		    {
			if ((microsolflag [line]) && (write_status == CMDBUSY))
			{
			    outagain [line] = tickGet () + 6; /* Wait 1/10 sec */
			    if (mltrace [line] == 15)
			    {
				logMsg ("Busy message received...Waiting 100 ms\n", 0,0,0,0,0,0);
			    }
			    return;
			}
			pmod->badoutcnt++;
                        if (mltrace [link] == 4)
			{
                            logMsg("BO DEQUEUE: Output failed: L=%d, A=%d,       S=%d, Value=%d\n",
                                   link, adapter, coiladdr, coilval, 0, 0);
			}
		    }
                    else
		    {
                        if (mltrace [link] == 4)
			{
                            logMsg("BO DEQUEUE: Output succeeded: L=%d, A=%d,       S=%d, Value=%d\n",
                                   link, adapter, coiladdr, coilval, 0, 0);
			}
		    }
		}
	    }
	    else if ((optr->otype) == SOUTTYPE)
	    {
		link = optr->out.sonode.link;
		adapter = optr->out.sonode.adapter;
/*		card = optr->out.sonode.card; */
		pmod = plcs [link] [adapter];
		if (pmod == NULL)
		{
		    if (!allocate_plc (link, adapter))
		    {
			return; /* if not allocated, try. if can't allocate, get out. */
		    }
        	    pmod = plcs [link] [adapter];
		}
		stringaddr = optr->out.sonode.sosignal;
		stringlen  = optr->out.sonode.solen;
		pstringval = optr->out.sonode.sovalue;
		if (mltrace [line] == 7)
		{
                    logMsg("SO IN QUEUE: L=%d, A=%d,       S=%d\n",
                           link, adapter, stringaddr, 0, 0, 0);
		}

		if ((!pmod->cmd_force_offline) &&
                    (resresponse [link]) &&
                    (pmod->is_responding))
		{
                    loopoutmsgs [line]++;
		    if ((setstring (link, adapter, pmod, stringaddr, pstringval, stringlen)) != CMDDONE)
		    {
			pmod->badoutcnt++;
                        if (mltrace [link] == 5)
			{
                            logMsg ("SO DEQUEUE: Output failed: L=%d, A=%d,       S=%d\n",
                                   link, adapter, stringaddr, 0, 0, 0);
			}
		    }
                    else
		    {
                        if (mltrace [link] == 5)
			{
                            logMsg ("SO DEQUEUE: Output succeeded: L=%d, A=%d,       S=%d\n",
                                   link, adapter, stringaddr, 0, 0, 0);
			}
		    }
		}
	    }
	    oreleasefull (line);     /* release the output que entry */
	}
}

static int write_aos (unsigned short line)
{
	unsigned short  regval;
	MODPLC          *pmod;
	double 		dblaovalue;
	unsigned short	i, j, mod, write_status;

	dblaovalue = 0.0; /* assign a default value */

	for (mod = 1; (mod <= NUMMODPLCS); mod++ )
	{
	    pmod = plcs [line] [mod];
	    if (pmod)			/* verify pointer validity prior to use */
	    {        
		for (i = 0; (i < NUMREGISTERS); i++)
		{
		    for (j = 0; j < REGBLOCK; j++)
		    {
			if (pmod->regs [i].data_togo [j])
			{
			    /* Set regval to the number of registers to output */

			    for (regval = 0; regval < (REGBLOCK - j); regval++)
			    {
				if (pmod->regs [i].data_togo [regval + j] == 0)
				{
				    break;
				}
			    }
			    
			    write_status = setreg (line, mod, pmod, ((i * 4) + j),
                                                   regval, dblaovalue, j);
			    if (write_status == CMDBUSY)
			    {
				outagain [line] = tickGet () + 6; /* Wait 1/10 sec */
				if (mltrace [line] == 15)
				{
				    logMsg ("Busy message received...Waiting 100 ms\n", 0,0,0,0,0,0);
				}
				return (TRUE);
			    }
			    else
			    {
				for (; regval != 0; regval--, j++)
				{
				    pmod->regs [i].data_togo [j] = FALSE;
				}
				j--;
			    }
			}
		    }
		}

	    }
	}
	nextouttime [line] = tickGet () + 62;
	return (FALSE);
}

/*                                                                                     
**      mod_aidriver                                                                       
**                                                                                     
**      This routines main purpose is to return the analog input value for the         
**      specified module. For a module value                                           
**      a (-1) will be returned if there is no valid data ready.                       
**
*/
/* this routine not called from within this driver */

int mod_aidriver (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, short *pvalue, unsigned short conversion,
		unsigned char *resname)
    {
	MODPLC		*pmod;
	unsigned short  blkix;
	unsigned short	*pdata;

	if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AIDRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if (adapter > NUMMODPLCS)
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AIDRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if ((card != MAINS) && (card != MREGS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
        if (hathawayflag [link])
        {
            if (card == MREGS)
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int)pointname, link, adapter, card, signal, 0);
                return (-1);
            }
        }

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	if (card == MPSEUDO)
	{
	    switch (signal)
	    {
	    case 0: /* Were the PLC values requested */
		*pvalue = pmod->was_requested;
		break;

	    case 1: /* Is the PLC responding */
		*pvalue = pmod->is_responding;
		break;

	    case 2: /* The program Load request */
		*pvalue = pmod->prog_load_stat;
		break;

	    case 3: /* PLC Program Load Block Number */
		*pvalue = pmod->prog_load_page;
		break;

	    case 4: /* Set the program number to Load */
		*pvalue = pmod->prog_load_number;
		break;

	    case 5: /* Output Messages */
		*pvalue = pmod->no_dataout_msgs & 0x7fff;
		break;

	    case 6: /* Input Messages */
		*pvalue = pmod->no_datain_msgs & 0x7fff;
		break;

	    case 7: /* No Receiver Responses */
		*pvalue = pmod->no_rxnoresp & 0x7fff;
		break;

	    case 8: /* No Transmitter Response */
		*pvalue = pmod->no_txnoresp & 0x7fff;
		break;

	    case 9: /* Data Echo Errors */
		*pvalue = pmod->no_dataerr & 0x7fff;
		break;

	    case 10: /* Message Retries */
		*pvalue = pmod->no_retries & 0x7fff;
		break;

	    case 11: /* RX Hardware errors */
		*pvalue = pmod->no_rxhwerrs & 0x7fff;
		break;

	    case 12: /* TX Hardware errors */
		*pvalue = pmod->no_txhwerrs & 0x7fff;
		break;

	    case 13: /* CRC errors */
		*pvalue = pmod->no_crcerrs & 0x7fff;
		break;

	    case 14: /* Offline transitions */
		*pvalue = pmod->no_offlines & 0x7fff;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 17: /* Failed output commands */
		*pvalue = pmod->badoutcnt & 0x7fff;
		break;

	    case 20: /* What is the program load comp code for the plc? */
		*pvalue = pmod->prog_load_code;
		break;

	    default:
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AIDRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
		return (-1);
	    }
	    return (0);
	}

	if (card == MAINS)
	{
            if ((signal > MAXAINO) || (signal < MINAINO))
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AIDRIVER: Illegal signal(0):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
                return (-1);
            }

            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--; /* Signals are defined as 1 relative */
            blkix = (signal / REGBLOCK);

            /* check for trans-REGBLOCK boundary access */
            if (((conversion == AICNVDOUBLEINT)  ||
                 (conversion == AICNVFLOAT    )) &&
                ((signal % REGBLOCK) == 3      ))
            {
	            if (blkix+1 >= NUMMODAIS)   /* if at top of register space, give up */
                    {
                        if (trace[link] & CONFIGURATIONTRACE)
                            logMsg(
                    "AIDRIVER: Point extends beyond legal signal range(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                    /* as soon as the address is confirmed as valid, set requested bits. */
                    pmod->ais [blkix].was_requested = TRUE;
	            pmod->ais [blkix+1].was_requested = TRUE;
	            if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Driver Offline(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: On Hold(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
	            if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Not Responding(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if ((!pmod->ais [blkix].data_valid) || (!pmod->ais [blkix+1].data_valid))
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Data Not Valid(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
		    pvalue[1] = pmod->ais [blkix].aivals [3];
		    pvalue[0] = pmod->ais [blkix+1].aivals [0];
 
                    if (mltrace[link] == 8)
                        logMsg("AIDRIVER: Data Returned(1):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                                (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
                    return(0);
            }
            /* check for trans-REGBLOCK boundary access */
            if ((conversion == AICNVDOUBLEFLT) &&
                ((signal % REGBLOCK) != 0      ))
            {
	            if (blkix+1 >= NUMMODAIS)   /* if at top of register space, give up */
                    {
                        if (trace[link] & CONFIGURATIONTRACE)
                            logMsg(
                    "AIDRIVER: Point extends beyond legal signal range(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                    /* as soon as the address is confirmed as valid, set requested bits. */
	            pmod->ais [blkix].was_requested = TRUE;
	            pmod->ais [blkix+1].was_requested = TRUE;
	            if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Driver Offline(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: On Hold(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
	            if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Not Responding(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if ((!pmod->ais [blkix].data_valid) || (!pmod->ais [blkix+1].data_valid))
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Data Not Valid(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
	            }
		    switch (signal % REGBLOCK)
                    {
                        case (1):
                             pvalue[3] = pmod->ais [blkix].aivals [1];
		             pvalue[2] = pmod->ais [blkix].aivals [2];
		             pvalue[1] = pmod->ais [blkix].aivals [3];
		             pvalue[0] = pmod->ais [blkix+1].aivals [0];
                             break;
                        case (2):
                             pvalue[3] = pmod->ais [blkix].aivals [2];
		             pvalue[2] = pmod->ais [blkix].aivals [3];
		             pvalue[1] = pmod->ais [blkix+1].aivals [0];
		             pvalue[0] = pmod->ais [blkix+1].aivals [1];
                             break;
                        case (3):
                             pvalue[3] = pmod->ais [blkix].aivals [3];
		             pvalue[2] = pmod->ais [blkix+1].aivals [0];
		             pvalue[1] = pmod->ais [blkix+1].aivals [1];
		             pvalue[0] = pmod->ais [blkix+1].aivals [2];
                             break;
                        default:
                             return(-1); /* condition can never happen */
                    }
                    if (mltrace[link] == 8)
                        logMsg("AIDRIVER: Data Returned(2):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                                (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
                    return(0);
            }
            /* as soon as the address is confirmed as valid, set requested bits. */
	    pmod->ais [blkix].was_requested = TRUE;
            if (!resresponse[link])
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: Driver Offline(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
            if (pmod->holdflag)
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: On Hold(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-2);
            }
            if (!pmod->is_responding)
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: Not Responding(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
	    if (!pmod->ais [blkix].data_valid)
            {
               if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: Data Not Valid(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
               return (-1);
	    }
	    if (!microsolflag [link])
	    {
		pdata = ((unsigned short*) &(pmod->ais [blkix].aivals [signal % REGBLOCK]));
	    }
	    else
	    {
		*pvalue = (pmod->ais [blkix].aivals [signal % REGBLOCK]>>4) & 0xFFF;
                if (mltrace[link] == 8)
                    logMsg("AIDRIVER: Data Returned(3):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                            (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
		return (0);
	    }
	    /* Select the Size and Type of Response based on Conversion */
	    switch (conversion)
	    {
	        case (AICNVDOUBLEINT):
		     pvalue[0] = pdata[1];
		     pvalue[1] = pdata[0];
		     break;
	        case (AICNVFLOAT):
		     pvalue[0] = pdata[1];
		     pvalue[1] = pdata[0];
		     break;
	        case (AICNVDOUBLEFLT):
		     pvalue[0] = pdata[3];
		     pvalue[1] = pdata[2];
		     pvalue[2] = pdata[1];
		     pvalue[3] = pdata[0];
		     break;
	        case (AICNVINTEGER):
	        default:
                 if ((hathawayflag [link]) && 
                     (hathaway16bit[link]) &&
                     (signal >= HIAAI12START) &&
                     (signal < HIAAI12END) &&
                     ((*pdata & 0x800) !=0) ) {

                     *pvalue = *pdata | 0xf000;
                 } else {
                     *pvalue = *pdata;
                 }
                 if ((hathawayflag [link]) && 
                     (signal >= HIAAICLRSTART) &&
                     (signal < HIAAICLREND) ) {

		        *pdata = 0;
                 }

                 /* Binarys are 0, 1, or 0xFFFF */ 
                 if ((hathawayflag [link]) && 
                     (signal >= HIAUASSBOSTART) &&
                     (signal < HIAUASSBOEND) ) {
                     if (*pdata == 0xFF00)
		        *pvalue = 1;
                     else if (*pdata == 0x00)
                        *pvalue = 0;
                     else *pvalue = 0xFFFF;

                     *pdata = 0xFFFF;

                 }

		 break;
            }
            if (mltrace[link] == 8)
                logMsg("AIDRIVER: Data Returned(3):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                        (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
            return(0);
	}

	if (card == MREGS)
	{
            if ((signal > MAXREGNO) || (signal < MINREGNO))
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AIDRIVER: Illegal signal(0):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
                return (-1);
            }

            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--; /* Signals are defined as 1 relative */
            blkix = (signal / REGBLOCK);

            /* check for trans-REGBLOCK boundary access */
            if (((conversion == AICNVDOUBLEINT)  ||
                 (conversion == AICNVFLOAT    )) &&
                ((signal % REGBLOCK) == 3      ))
            {
	            if (blkix+1 >= NUMREGISTERS)   /* if at top of register space, give up */
                    {
                        if (trace[link] & CONFIGURATIONTRACE)
                            logMsg(
                    "AIDRIVER: Point extends beyond legal signal range(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
		        return (-1);
	            }
                    /* as soon as the address is confirmed as valid, set requested bits. */
                    pmod->regs [blkix].was_requested = TRUE;
	            pmod->regs [blkix+1].was_requested = TRUE;
 	            if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Driver Offline(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: On Hold(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
	            if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Not Responding(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if ((!pmod->regs [blkix].data_valid) || (!pmod->regs [blkix+1].data_valid))
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Data Not Valid(1):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
		        return (-1);
	            }
		    pvalue[1] = pmod->regs [blkix].ioregs [3];
		    pvalue[0] = pmod->regs [blkix+1].ioregs [0];
                    if (mltrace[link] == 8)
                        logMsg("AIDRIVER: Data Returned(2):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                                (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
                    return(0);
            }
            /* check for trans-REGBLOCK boundary access */
            if ((conversion == AICNVDOUBLEFLT) &&
                ((signal % REGBLOCK) != 0      ))
            {
	            if (blkix+1 >= NUMREGISTERS)   /* if at top of register space, give up */
                    {
                        if (trace[link] & CONFIGURATIONTRACE)
                            logMsg(
                    "AIDRIVER: Point extends beyond legal signal range(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
		        return (-1);
	            }
                    /* as soon as the address is confirmed as valid, set requested bits. */
	            pmod->regs [blkix].was_requested = TRUE;
	            pmod->regs [blkix+1].was_requested = TRUE;
	            if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Driver Offline(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: On Hold(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
	            if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Not Responding(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
	            if ((!pmod->regs [blkix].data_valid) || (!pmod->regs [blkix+1].data_valid))
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AIDRIVER: Data Not Valid(2):[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
		        return (-1);
	            }
		    switch (signal % REGBLOCK)
                    {
                        case (1):
                             pvalue[3] = pmod->regs [blkix].ioregs [1];
		             pvalue[2] = pmod->regs [blkix].ioregs [2];
		             pvalue[1] = pmod->regs [blkix].ioregs [3];
		             pvalue[0] = pmod->regs [blkix+1].ioregs [0];
                             break;
                        case (2):
                             pvalue[3] = pmod->regs [blkix].ioregs [2];
		             pvalue[2] = pmod->regs [blkix].ioregs [3];
		             pvalue[1] = pmod->regs [blkix+1].ioregs [0];
		             pvalue[0] = pmod->regs [blkix+1].ioregs [1];
                             break;
                        case (3):
                             pvalue[3] = pmod->regs [blkix].ioregs [3];
		             pvalue[2] = pmod->regs [blkix+1].ioregs [0];
		             pvalue[1] = pmod->regs [blkix+1].ioregs [1];
		             pvalue[0] = pmod->regs [blkix+1].ioregs [2];
                             break;
                        default:
                             return(-1); /* condition can never happen */
                    }
                    if (mltrace[link] == 8)
                        logMsg("AIDRIVER: Data Returned(2):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                                (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
                    return(0);
            }
            /* as soon as the address is confirmed as valid, set requested bits. */
	    pmod->regs [blkix].was_requested = TRUE;
            if (!resresponse[link])
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: Driver Offline(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
            if (pmod->holdflag)
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: On Hold(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-2);
            }
            if (!pmod->is_responding)
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: Not Responding(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
	    if (!pmod->regs [blkix].data_valid)
            {
               if (trace[link] & CALLREJECTTRACE)
                    logMsg("AIDRIVER: Data Not Valid(3):[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
	       return (-1);
	    }
	    if (!microsolflag [link])
	    {
		pdata = &(pmod->regs [blkix].ioregs [signal % REGBLOCK]);
	    }
	    else
	    {
		*pvalue = (pmod->regs [blkix].ioregs [signal % REGBLOCK] >> 4) & 0xFFF;
                if (mltrace[link] == 8)
                    logMsg("AIDRIVER: Data Returned(3):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                            (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
		return(0);
	    }
	    /* Select the Size and Type of Response based on Conversion */
	    switch (conversion)
	    {
	        case (AICNVDOUBLEINT):
		     pvalue[0] = pdata[1];
		     pvalue[1] = pdata[0];
		     break;
	        case (AICNVFLOAT):
		     pvalue[0] = pdata[1];
		     pvalue[1] = pdata[0];
		     break;
	        case (AICNVDOUBLEFLT):
		     pvalue[0] = pdata[3];
		     pvalue[1] = pdata[2];
		     pvalue[2] = pdata[1];
		     pvalue[3] = pdata[0];
		     break;
	        case (AICNVINTEGER):
	        default:
		     *pvalue = *pdata;
		     break;
            }
            if (mltrace[link] == 8)
                logMsg("AIDRIVER: Data Returned(3):[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                        (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
            return(0);
	}
	return(-1);
    }

/*  										
**      mod_aodriver                                                                
**                                                                            	
**      This routines main purpose is to set requested analog outputs to the  	
**      specified module.                                                     	
*/                                                                            	
/* this routine not called from within this driver */


int mod_aodriver (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
	    unsigned short signal,double dvalue, unsigned short value,
	    unsigned short conversion, unsigned char *resname)

{
	MODPLC		*pmod;
	unsigned	short  tvalue,blkix;
	OUTQENT		*oqp;

	if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AODRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
            return (-1);
        }
	if (adapter > NUMMODPLCS)
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AODRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
            return (-1);
        }
	if ((card != MREGS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AODRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
            return (-1);
        }

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}
	if (card == MPSEUDO)
	{
	    switch (signal)
	    {
	    case 2: /* Set the program Load request */
		if (value != 0)
		{
		    pmod->prog_load_stat = PLC_LOAD_REQUESTED;
		}
		break;

	    case 4: /* Set the program number to Load */
		pmod->prog_load_number = value;
		break;

	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
		{
		    initstats(pmod);
		}
		break;

	    case 16: /* Force the PLC online/offline */
		pmod->cmd_force_offline = (value != 0);
                if (value != 0)
		{
                    pmod->is_responding = FALSE;/* only required when asserting cmd_force_offline */
		}
		break;

	    default:
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AODRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                            (int) pointname, link, adapter, card, signal, value);
		return (-1);
	    }
	    return (0);
	}

        if ( (hathawayflag [link]) && 
	     ((signal > MAXREGNO) || (signal < MINREGNO)) )
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AODRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
	    return (-1);
	}
	if (resname != 0)
	{ 
	    register_resource (link, pmod, resname, pointname);
	}

	pmod->was_requested = TRUE;

	signal--; /* Signals are defined as 1 relative */
	blkix = (signal / REGBLOCK);

	pmod->regs [blkix].was_requested = TRUE;

        /* check for trans-REGBLOCK boundary access */
        if ((conversion == ASCFLT) && ((signal % REGBLOCK) == 3))
        {
            if (blkix+1 >= NUMREGISTERS)   /* if at top of register space, give up */
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg(
                "AODRIVER: Point extends beyond legal signal range:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                            (int) pointname, link, adapter, card, signal+1, value);
                return (-1);
            }
            /* as soon as the address is confirmed as valid, set next block's requested bits. */
            pmod->regs [blkix+1].was_requested = TRUE;
        }
	else if (microsolflag [link])
	{
	    pmod->regs [blkix].regs_togo [signal % 4] = (value << 4);
	    pmod->regs [blkix].data_togo [signal % 4] = TRUE;
            if (mltrace [link] == 3)
	    {
                logMsg("AODRIVER: RegsToGo:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    }
	    return (0);
	}

	if (!resresponse[link])
        {
            if (trace[link] & CALLREJECTTRACE)
                logMsg("AODRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
            return (-1);
        }

        /* check for plc simulated mode. */
        if (modsim [link])
        {
            if (mltrace [link] == 3)
	    {
                logMsg("AODRIVER: SimReturn:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    }
            return (0);
        }

	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
            if (trace[link] & CALLREJECTTRACE)
                logMsg("AODRIVER: Not Responding or Forced offline:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    return (-1);
	}

        if ( (hathawayflag [link]) && 
	     (signal >= MINAGCREGNO) &&
	     (signal <= MAXAGCREGNO) &&
	     ((value == 0 ) || (dvalue == 0)) )
	{
	    return (0);
	}


	oqp = ogetemptyp (link);
	if (oqp == NULLP)
	{
            if (trace[link] & RUNTIMEERRORTRACE)
            {
                logMsg("AODRIVER: Queue Overflow:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    }
	    pmod->badoutcnt++;
	    return (-1);		/* output queue is full */
	}
	oqp->otype = AOUTTYPE;
	oqp->out.aonode.link = link;
	oqp->out.aonode.adapter = adapter;
	oqp->out.aonode.card = card;
	oqp->out.aonode.aosignal = signal;
	tvalue = value;

  
#ifdef  OLD8850PROM
        if ((hathawayflag [link]) && (signal > HIAAGCWRAP))
        {
 	    if (dvalue < 0) {
		tvalue = -dvalue;
		oqp->out.aonode.aosignal = signal - HIAAGCWRAP + 1;
	    } else {
		oqp->out.aonode.aosignal = signal - HIAAGCWRAP;
	    }

        }
#endif


	if (!microsolflag [link])
	{
	    oqp->out.aonode.aovalue = tvalue;
	}
	else
	{
	    oqp->out.aonode.aovalue = (tvalue << 4);
	}

	oqp->out.aonode.dblaovalue = dvalue;
	oqp->out.aonode.conversion = conversion;

	omarkfull (link);		/* Mark this entry as ready for output */

        if (mltrace [link] == 3)
	{
            logMsg("AODRIVER: Queued:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                    (int) pointname, link, adapter, card, signal+1, value);
	}
	return (0);
}

/*                                                                             
**      mod_aoread                                                                  
**                                                                          	
**      This routines main purpose is to return the value last read back     	
**      from the analog output hardware. This is used to initialize analog   	
**      outputs.                                                             	
*/
/* this routine not called from within this driver */


int mod_aoread (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
	  unsigned short signal, unsigned short *pvalue,
	  unsigned short conversion, unsigned char *resname)

    {
	MODPLC		*pmod;
	unsigned short  blkix, *pdata;
	
	if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AOREAD: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if (adapter > NUMMODPLCS)
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AOREAD: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if ((card != MREGS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("AOREAD: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
                return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	if (card == MPSEUDO)
	{
	    switch (signal)
	    {
	    case 2: /* read the most recent program Load request */
		*pvalue = pmod->prog_load_stat;
		break;

	    case 4: /* read the most recent program number to Load */
		*pvalue = pmod->prog_load_number;
		break;

	    case 15: /* read statistics command value */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Read the current PLC offline command status */
		*pvalue = pmod->cmd_force_offline;
		break;

	    default:
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AOREAD: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
		return(-1);
	    }
	    return (0);
	}

	if (card == MREGS)
	{
            /**********************/
            if (hathawayflag [link])
            {
                *pvalue = 0;
                if (mltrace[link] == 8)
                    logMsg("AOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int)pointname, link, adapter, card, signal+1, 0);
                return (0);
            }
            /**********************/
            if ((signal > MAXREGNO) || (signal < MINREGNO))
	    {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("AOREAD: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
		return (-1);
	    }

            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--; /* Signals are defined as 1 relative */
            blkix = (signal / REGBLOCK);

            /* check for trans-REGBLOCK boundary access */
            if (  (conversion == ASCFLT)  &&
                  ((signal % REGBLOCK) == 3) )
            {
	            if (blkix+1 >= NUMREGISTERS)   /* if at top of register space, give up */
                    {
                        if (trace[link] & CONFIGURATIONTRACE)
                            logMsg(
                    "AOREAD: Point extends beyond legal signal range:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		        return (-1);
	            }
                    /* as soon as the address is confirmed as valid, set requested bits. */
                    pmod->regs [blkix].was_requested = TRUE;
	            pmod->regs [blkix+1].was_requested = TRUE;
                    if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                    if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
                    if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("AOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                    /**********************/
                    if (hathawayflag [link])
                    {
                        *pvalue = 0;
                        if (mltrace[link] == 8)
                            logMsg("AOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int)pointname, link, adapter, card, signal+1, 0);
                        return (0);
                    }
                    /**********************/
	            if ((!pmod->regs [blkix].data_valid) || (!pmod->regs [blkix+1].data_valid))
                    {
                       if (trace[link] & CALLREJECTTRACE)
                            logMsg("AOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
		        return (-1);
	            }
		    pvalue[1] = pmod->regs [blkix].ioregs [3];
		    pvalue[0] = pmod->regs [blkix+1].ioregs [0];
                    if (mltrace[link] == 8)
                        logMsg("AOREAD: Data Returned:[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                                (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
                    return(0);
            }
            /* as soon as the address is confirmed as valid, set requested bits. */
	    pmod->regs [blkix].was_requested = TRUE;
            if (!resresponse[link])
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
            if (pmod->holdflag)
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-2);
            }
            if (!pmod->is_responding)
            {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("AOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
            /**********************/
            if (hathawayflag [link])
            {
                *pvalue = 0;
                if (mltrace[link] == 8)
                    logMsg("AOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int)pointname, link, adapter, card, signal+1, 0);
                return (0);
            }
            /**********************/
	    if (!pmod->regs [blkix].data_valid)
               {
                   if (trace[link] & CALLREJECTTRACE)
                        logMsg("AOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		   return (-1);
	       }
	    if (!microsolflag [link])
	    {
		pdata = &(pmod->regs [blkix].ioregs [signal % REGBLOCK]);
	    }
	    else
	    {
		*pvalue = (pmod->regs [blkix].ioregs [signal % REGBLOCK] >> 4) & 0xFFF;
                if (mltrace[link] == 8)
                    logMsg("AOREAD: Data Returned:[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                            (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
		return (0);
	    }
	    /* Select the Size and Type of Response based on Conversion */
	    switch (conversion)
	    {
	        case (ASCFLT):
		     pvalue[0] = pdata[1];
		     pvalue[1] = pdata[0];
		     break;
	       default:
		     *pvalue = *pdata;
		     break;
            }
            if (mltrace[link] == 8)
                logMsg("AOREAD: Data Returned:[%s] L=%d, A=%d, C=%d, V=0x%x 0x%x\n",
                        (int) pointname, link, adapter, card, pvalue[0], pvalue[1]);
	    return(0);
	}
	return(-1);
    }

/*                                                                                     
**      mod_bidriver                                                                        
**                                                                               	
**      The main purpose of this routine is to return the request digital input  	
**      value(s) from the requested card. This routine will return a (-1) if the  	
**      requested module is the wrong type or the data is not yet valid.          	
**                                                                                	
*/
/* this routine not called from within this driver */

int mod_bidriver (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long mask, unsigned long *pvalue,
		unsigned char *resname)
    {
	MODPLC 		*pmod;
	unsigned short	blkix;
	unsigned long	tvalue;

	if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BIDRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if (adapter > NUMMODPLCS)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BIDRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if ((card != MDINS) && (card != MCOILS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
        if (hathawayflag [link])
        {
            if (card == MCOILS)
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("BIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int)pointname, link, adapter, card, signal, 0);
                return (-1);
            }
        }

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == MPSEUDO)
	{
	    switch (signal)
	    {
                case 0: /* Were the PLC values requested */
                    *pvalue = pmod->was_requested;
                    break;

                case 1: /* Is the PLC responding */
                    *pvalue = pmod->is_responding;
                    break;

                case 15: /* reset statistics */
                    *pvalue = pmod->cmd_reset_stat;
                    break;

                case 16: /* Force the PLC offline */
                    *pvalue = pmod->cmd_force_offline;
                    break;

                case 20: /* What is the selected channel for the line */
                    *pvalue = pmod->prog_load_code;
                    break;

                default:
                    if (trace[link] & CONFIGURATIONTRACE)
                        logMsg("BIDRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal, 0);
                    return (-1);
            }
            return (0);
        }

	tvalue = 0;

	if (card == MDINS)
	{
            if ((signal > MAXDINO) || (signal < MINDINO))
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("BIDRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
                return (-1);
            }
            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--; /* Signals are defined as 1 relative */
            blkix = ((signal / 8) & 0xFFFC);	/* 8 bits per digital BLOCK */

	    if ( (mask & 0x000000FF) != 0)
	    {
	        pmod->dis [blkix].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->dis [blkix].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue = pmod->dis [blkix].last_read & mask;
	    }
	    if ( (mask & 0x0000FF00) != 0)
	    {
	        pmod->dis [blkix+1].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->dis [blkix+1].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue += (unsigned long) (pmod->dis [blkix+1].last_read) << 8;
	    }
	    if ( (mask & 0x00FF0000) != 0)
	    {
	        pmod->dis [blkix+2].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->dis [blkix+2].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue += (unsigned long) (pmod->dis [blkix+2].last_read) << 16;
	    }
	    if ( (mask & 0xFF000000) != 0)
	    {
	        pmod->dis [blkix+3].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->dis [blkix+3].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue += (unsigned long) (pmod->dis [blkix+3].last_read) << 24;
	    }
	    *pvalue = tvalue & mask;
            if (mltrace[link] == 8)
                logMsg("BIDRIVER: Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                        (int) pointname, link, adapter, card, signal+1, *pvalue);
	    return (0);
	}

	tvalue = 0;
	if (card == MCOILS)
	{
            if ((signal > MAXCOILNO) || (signal < MINCOILNO))
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("BIDRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
                return (-1);
            }
            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--; /* Signals are defined as 1 relative */
            blkix = ((signal / 8) & 0xFFFC);	/* 8 bits per digital BLOCK */

	    if ( (mask & 0x000000FF) != 0)
	    {
	        pmod->coils [blkix].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->coils [blkix].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue = pmod->coils [blkix].last_read & mask;
	    }
	    if ( (mask & 0x0000FF00) != 0)
	    {
	        pmod->coils [blkix+1].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->coils [blkix+1].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue += (unsigned long) (pmod->coils [blkix+1].last_read) << 8;
	    }
	    if ( (mask & 0x00FF0000) != 0)
	    {
	        pmod->coils [blkix+2].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->coils [blkix+2].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue += (unsigned long) (pmod->coils [blkix+2].last_read) << 16;
	    }
	    if ( (mask & 0xFF000000) != 0)
	    {
	        pmod->coils [blkix+3].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
		if (!pmod->coils [blkix+3].data_valid)
		{
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
		    return (-1);
		}
	        tvalue += (unsigned long) (pmod->coils [blkix+3].last_read) << 24;
	    }
	    *pvalue = tvalue & mask;
            if (mltrace[link] == 9)
                logMsg("BIDRIVER: Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                        (int) pointname, link, adapter, card, signal+1, *pvalue);
	    return (0);
	}
	return (-1);
    }
/*                                                                                     
**      mod_bodriver                                                                        
**                                                                                 	
**      The main purpose of this routine is to set the request digital output      	
**      value(s) on the requested card. This routine will return a (-1) if the     	
**      requested module is the wrong type or not configured yet.                  	
**                                                                                 	
*/
/* this routine not called from within this driver */

int mod_bodriver (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long value, unsigned long mask,
		unsigned char *resname)
    {
	MODPLC 		*pmod;
	OUTQENT		*oqp;
	unsigned short  blkix;

        if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BODRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
            return (-1);
        }
        if (adapter > NUMMODPLCS)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BODRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
            return (-1);
        }
        if ((card != MCOILS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BODRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
            return (-1);
        }

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return(-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	if (card == MPSEUDO)
	{
	    switch (signal & 0x1f)
	    {

	    case 2: /* Set the program Load request */
		if (value != 0)
		{
		    pmod->prog_load_stat = PLC_LOAD_REQUESTED;
		}
		break;

	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
		{
		    initstats (pmod);
		}
		break;

	    case 16: /* Force the PLC offline */
		pmod->cmd_force_offline = (value != 0);
                if (value != 0)
		{
                    pmod->is_responding = FALSE;/* only required when asserting cmd_force_offline */
		}
		break;

	    case 21: /* Start the PLC */
		pmod -> plc_start = (value != 0);
		break;

	    case 22: /* What is the selected channel for the line */
		pmod-> plc_stop = (value != 0);
		break;

	    default:
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("BODRIVER: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                            (int) pointname, link, adapter, card, signal, value);
		return (-1);
	    }
	    return (0);
	}

        if ( (!hathawayflag [link]) && 
	     ((signal > MAXCOILNO) || (signal < MINCOILNO)) )
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BODRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal, value);
	    return (-1);
	}

	if (resname != 0)
	{ 
	    register_resource(link,pmod,resname,pointname);
	}

        pmod->was_requested = TRUE;

	signal--; /* Signals are defined as 1 relative */
	blkix = ((signal / 8) & 0xFFFC);	/* 8 bits per digital BLOCK */
						/* masks aligned on 32 bits */

	if ((mask & 0x000000FF) != 0)
	{
	    pmod->coils [blkix].was_requested = TRUE;
	}
	if ((mask & 0x0000FF00) != 0)
	{
	    pmod->coils [blkix+1].was_requested = TRUE;
	}
	if ((mask & 0x00FF0000) != 0)
	{
	    pmod->coils [blkix+2].was_requested = TRUE;
	}
	if ((mask & 0xFF000000) != 0)
	{
	    pmod->coils [blkix+3].was_requested = TRUE;
	}

	if (!resresponse[link])
        {
            if (trace[link] & CALLREJECTTRACE)
                logMsg("BODRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
            return (-1);
        }

        if (modsim [link])
        {
            if (mltrace [link] == 4)
	    {
                logMsg("BODRIVER: SimReturn:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    }
            return (0);
        }
	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
            if (trace[link] & CALLREJECTTRACE)
                logMsg("BODRIVER: Not Responding or Forced offline:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    return (-1);
	}

	oqp = ogetemptyp (link);
	if (oqp == NULLP)
	{
            if (trace[link] & RUNTIMEERRORTRACE)
                logMsg("BODRIVER: Queue Overflow:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                        (int) pointname, link, adapter, card, signal+1, value);
	    pmod->badoutcnt++;
	    return (-1);		/* output queue is full */
	}
	oqp->otype = BOUTTYPE;
	oqp->out.bonode.link = link;
	oqp->out.bonode.adapter = adapter;
	oqp->out.bonode.card = card;
	oqp->out.bonode.bosignal = signal;
	oqp->out.bonode.bomask = mask;
	oqp->out.bonode.bovalue = value;

	omarkfull (link);		/* Mark this entry as ready for output */

        if (mltrace[link] == 4)
            logMsg("BODRIVER: Queued:[%s] L=%d, A=%d, C=%d, S=%d, V=%d\n",
                    (int) pointname, link, adapter, card, signal+1, value);
	return (0);
    }
/*
**      mod_boread                                                                  
**                                                                              
**      This routines main purpose is to return the value last read back      	
**      from the digital output hardware. This is used to initialize digital  	
**      outputs. This will return a (-1) if the module is the wrong type or    	
**      the data is not yet valid                                              	
*/
/* this routine not called from within this driver */


int mod_boread (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, unsigned long *pvalue, unsigned long mask,
		unsigned char *resname)

    {
	MODPLC		*pmod;
	unsigned short  blkix;
	unsigned long	tvalue;

        if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BOREAD: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
        if (adapter > NUMMODPLCS)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BOREAD: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
        if ((card != MCOILS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BOREAD: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return(-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	if (card == MPSEUDO)
	{
	    switch (signal)
	    {
	    case 2: /* Set the program Load request */
		*pvalue = 0;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;


	    case 21: /* Start the PLC */
		*pvalue = pmod -> plc_start;
		break;

	    case 22: /* What is the selected channel for the line */
		*pvalue = pmod-> plc_stop;
		break;

	    default:
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("BOREAD: Illegal Pseudo Point:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
		return (-1);
	    }
	    return (0);
	}

        /**********************/
        if (hathawayflag [link])
        {
            *pvalue = 0;
            if (mltrace[link] == 9)
                logMsg("BOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                       (int)pointname, link, adapter, card, signal+1, (int)pvalue);
	    return (0);
        }
        /**********************/

        if ((signal > MAXCOILNO) || (signal < MINCOILNO))
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("BOREAD: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal+1, 0);
	    return (-1);
	}

	if (resname != 0)
	{
             register_resource (link, pmod, resname, pointname);
	}

        pmod->was_requested = TRUE;

	signal--; /* Signals are defined as 1 relative */
	blkix = ((signal / 8) & 0xFFFC);	/* 8 bits per digital BLOCK */
						/* masks aligned on 32 bits */
	tvalue = 0;

	if ((mask & 0x000000FF) != 0)
	{
	    pmod->coils [blkix].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                /**********************/
                if (hathawayflag [link])
                {
                    *pvalue = 0;
                    if (mltrace[link] == 9)
                        logMsg("BOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                                (int)pointname, link, adapter, card, signal+1, (int)pvalue);
		    return (0);
                }
                /**********************/
	    if (!pmod->coils [blkix].data_valid)
	    {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("BOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
		return (-1);
	    }
	    tvalue = pmod->coils [blkix].last_read & mask;
	}
	if ( (mask & 0x0000FF00) != 0)
	{
	    pmod->coils [blkix+1].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                /**********************/
                if (hathawayflag [link])
                {
                    *pvalue = 0;
                    if (mltrace[link] == 9)
                        logMsg("BOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                                (int)pointname, link, adapter, card, signal+1, (int)pvalue);
		    return (0);
                }
                /**********************/
	    if (!pmod->coils [blkix+1].data_valid)
	    {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("BOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
	    }
	    tvalue += (unsigned long) (pmod->coils [blkix+1].last_read) << 8;
	}
	if ( (mask & 0x00FF0000) != 0)
	{
	    pmod->coils [blkix+2].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                /**********************/
                if (hathawayflag [link])
                {
                    *pvalue = 0;
                    if (mltrace[link] == 9)
                        logMsg("BOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                                (int)pointname, link, adapter, card, signal+1, (int)pvalue);
		    return (0);
                }
                /**********************/
	    if (!pmod->coils [blkix+2].data_valid)
	    {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("BOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
	    }
	    tvalue += (unsigned long) (pmod->coils [blkix+2].last_read) << 16;
	}
	if ( (mask & 0xFF000000) != 0)
	{
	    pmod->coils [blkix+3].was_requested = TRUE;
                if (!resresponse[link])
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                if (pmod->holdflag)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-2);
                }
                if (!pmod->is_responding)
                {
                    if (trace[link] & CALLREJECTTRACE)
                        logMsg("BOREAD: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                (int) pointname, link, adapter, card, signal+1, 0);
                    return (-1);
                }
                /**********************/
                if (hathawayflag [link])
                {
                    *pvalue = 0;
                    if (mltrace[link] == 9)
                        logMsg("BOREAD: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                                (int)pointname, link, adapter, card, signal+1, (int)pvalue);
		    return (0);
                }
                /**********************/
	    if (!pmod->coils [blkix+3].data_valid)
	    {
                if (trace[link] & CALLREJECTTRACE)
                    logMsg("BOREAD: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
	    }
	    tvalue += (unsigned long) (pmod->coils [blkix+3].last_read) << 24;
	}
	*pvalue = tvalue & mask;
        if (mltrace[link] == 9)
            logMsg("BOREAD: Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                    (int) pointname, link, adapter, card, signal+1, *pvalue);

	return (0);
    }
/*                                                                                     
**      mod_sidriver                                                                        
**                                                                              	
**      This routines main purpose is to return the analog input value for the         	
**      specified module. For a module value                                           	
**      a (-1) will be returned if there is no valid data ready.                       	
**                                                                                     	
*/
/* this routine not called from within this driver */

int mod_sidriver (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
		unsigned short signal, short *pvalue, unsigned short len,
		unsigned char *resname)
    {
	MODPLC          *pmod;
	short           oldblkix, blkix, regix;
        unsigned short  reglen;
        int             i;

        /**********************/
        if (hathawayflag [link])
        {
             if (mltrace[link] == 9)
                logMsg("sidriver: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d, V=0x%x\n",
                        (int)pointname, link, adapter, card, signal+1, (int)pvalue);
	     return (-1);
        }
        /**********************/

	reglen = 0; /* assign a default value */

	if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SIDRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if (adapter > NUMMODPLCS)
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SIDRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if ((card != MAINS) && (card != MREGS) && (card != MPSEUDO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SIDRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }

	pmod = plcs[link][adapter];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, adapter))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
	        pmod = plcs[link][adapter];
	}

/********************/
/* no pseudo points */
/*                  */
/*                  */
/*                  */
/*                  */
/********************/

        if (card == MAINS)
	{
            /* ensure string contained entirely within the address space of the allocated region */
            if ((signal > MAXAINO) || (signal < MINAINO))
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("SIDRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal, 0);
                return (-1);
            }
            if ((signal+reglen) > MAXAINO)
            {
                if (trace[link] & CONFIGURATIONTRACE)
                  logMsg(
                  "SIDRIVER: String extends beyond legal signal range:[%s] L=%d, A=%d, C=%d, S=%d\n",
                         (int) pointname, link, adapter, card, signal, 0);
                return (-1);
            }
            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--;               /* Signals are defined as 1 relative */
            reglen = (len+1)/2;     /* round up odd length character count string lengths */
            blkix = -1;

            for ( i = 0; i < reglen ; i++)
            {
                oldblkix = blkix;
                blkix = ((signal+i) / REGBLOCK);
                regix = ((signal+i) % REGBLOCK);
                if (oldblkix != blkix)
                {
                    pmod->ais [blkix].was_requested = TRUE;
                    if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                                    return (-1);
                    }
                    if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
                    /* check for plc simulated mode. */
                    if (modsim [link])
                    {
                        *pvalue = 0;
                        if (mltrace [link] == 10)
                        {
                            logMsg("BODRIVER: SimReturn:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        }
                        return (0);
                    }
                    if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                     (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                    if (!pmod->ais [blkix].data_valid)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                }
                pvalue[i] = pmod->ais [blkix].aivals [regix];
            }
            pvalue[i] = 0; /* null terminate the string */
            return(0);
        }
        if (card == MREGS)
	{
            /* ensure string contained entirely within the address space of the allocated region */
            if ((signal > MAXREGNO) || (signal < MINREGNO))
            {
                if (trace[link] & CONFIGURATIONTRACE)
                    logMsg("SIDRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                            (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
            if ((signal+reglen) > MAXREGNO)
            {
                if (trace[link] & CONFIGURATIONTRACE)
                  logMsg(
                  "SIDRIVER: String extends beyond legal signal range:[%s] L=%d, A=%d, C=%d, S=%d\n",
                         (int) pointname, link, adapter, card, signal+1, 0);
                return (-1);
            }
            if (resname != 0)
            { 
                register_resource(link,pmod,resname,pointname);
            }

            pmod->was_requested = TRUE;

            signal--;               /* Signals are defined as 1 relative */
            reglen = (len+1)/2;     /* round up odd length character count string lengths */
            blkix = -1;

            for ( i = 0; i < reglen ; i++)
            {
                oldblkix = blkix;
                blkix = ((signal+i) / REGBLOCK);
                regix = ((signal+i) % REGBLOCK);
                if (oldblkix != blkix)
                {
                    if (!resresponse[link])
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                                    return (-1);
                    }
                    if (pmod->holdflag)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: On Hold:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-2);
                    }
                    /* check for plc simulated mode. */
                    if (modsim [link])
                    {
                        *pvalue = 0;
                        if (mltrace [link] == 10)
                        {
                            logMsg("SIDRIVER: SimReturn:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        }
                        return (0);
                    }
                    if (!pmod->is_responding)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: Not Responding:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                     (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                    if (!pmod->regs [blkix].data_valid)
                    {
                        if (trace[link] & CALLREJECTTRACE)
                            logMsg("SIDRIVER: Data Not Valid:[%s] L=%d, A=%d, C=%d, S=%d\n",
                                    (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
                    }
                            return (-1);
                }
                pvalue[i] = pmod->regs [blkix].ioregs [regix];
            }
            pvalue[i] = 0; /* null terminate the string */
            return(0);
        }
	return(-1);
    }


/*  										
**      mod_sodriver                                                                
**                                                                            	
**      This routines main purpose is to set requested string outputs to the  	
**      specified module via the output buffer.                                 
*/                                                                            	
/* this routine not called from within this driver */

int mod_sodriver (char *pointname, unsigned short link, unsigned short adapter, unsigned short card,
	    unsigned short signal, unsigned char *value, unsigned short len, unsigned char *resname)
    {
	MODPLC  *pmod;
	unsigned short  blkix;
	OUTQENT *oqp;
        unsigned short reglen;
        int            i;

        /**********************/
        if (hathawayflag [link])
        {
            if (mltrace[link] == 9)
                logMsg("Sodriver: Zero Data Returned:[%s] L=%d, A=%d, C=%d, S=%d \n",
                        (int)pointname, link, adapter, card, signal+1,0);
	    return (-1);
        }
        /**********************/

	reglen = 0; /* assign a default value */

	if (link > MODLINES)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SODRIVER: Illegal line:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if (adapter > NUMMODPLCS)
	{
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SODRIVER: Illegal adapter:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }
	if (card != MREGS)
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SODRIVER: Illegal card:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }

	pmod = plcs[link][adapter];
	if (pmod == NULL)
	{
		if (!allocate_plc(link, adapter))
			return(-1); /* if not allocated, try. if can't allocate, get out. */
	        pmod = plcs[link][adapter];
	}

/********************/
/* no pseudo points */
/*                  */
/*                  */
/*                  */
/*                  */
/********************/

        /* ensure string contained entirely within the address space of the allocated region */
        if ((signal > MAXREGNO) || (signal < MINREGNO))
        {
            if (trace[link] & CONFIGURATIONTRACE)
                logMsg("SODRIVER: Illegal signal:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }

        if ((signal+reglen) > MAXREGNO)
        {
            if (trace[link] & CONFIGURATIONTRACE)
              logMsg(
              "SODRIVER: String extends beyond legal signal range:[%s] L=%d, A=%d, C=%d, S=%d\n",
                     (int) pointname, link, adapter, card, signal, 0);
            return (-1);
        }

        if (resname != 0)
        { 
            register_resource(link,pmod,resname,pointname);
        }

        pmod->was_requested = TRUE;

	signal--;                                       /* Signals are defined as 1 relative */
        reglen = (len+1)/2;            /* round up odd length character count string lengths */
	blkix = (signal / REGBLOCK);

	pmod->regs [blkix].was_requested = TRUE; /* notice we only set one here */

        if (!resresponse[link])
        {
            if (trace[link] & CALLREJECTTRACE)
                logMsg("SODRIVER: Driver Offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal+1, 0);
                        return (-1);
        }

        /* check for plc simulated mode. */
        if (modsim [link])
        {
            if (mltrace [link] == 5)
            {
                logMsg("SODRIVER: SimReturn:[%s] L=%d, A=%d, C=%d, S=%d\n",
                       (int) pointname, link, adapter, card, signal+1, 0);
            }
            return (0);
        }
        if ((!pmod->is_responding) || (pmod->cmd_force_offline))
        {
            if (trace[link] & CALLREJECTTRACE)
                logMsg("SODRIVER: Not Responding or Forced offline:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal+1, 0);
            return (-1);
        }

	oqp = ogetemptyp (link);
	if (oqp == NULLP)
	{
            if (trace[link] & RUNTIMEERRORTRACE)
                logMsg("SODRIVER: Queue Overflow:[%s] L=%d, A=%d, C=%d, S=%d\n",
                        (int) pointname, link, adapter, card, signal+1, 0);
	    pmod->badoutcnt++;
	    return (-1);		/* output queue is full */
	}
	oqp->otype = SOUTTYPE;
	oqp->out.sonode.link = link;
	oqp->out.sonode.adapter = adapter;
	oqp->out.sonode.card = card;
	oqp->out.sonode.sosignal = signal;
	oqp->out.sonode.solen = len;

	/* Move in the output string (strcpy not used because */
        /*  string content could contain terminators)         */
	for (i = 0 ;(i < STRINGMSGLEN) ; i++)
        {
	        oqp->out.sonode.sovalue[i] = value[i];
	}

	omarkfull (link);		/* Mark this entry as ready for output */

        if (mltrace[link] == 5)
            logMsg("SODRIVER: Queued:[%s] L=%d, A=%d, C=%d, S=%d, V=%s\n",
                    (int) pointname, link, adapter, card, signal+1, (int) value);
	return (0);
    }

/*
** register_resource
**
**	This routine is used to initialize the resource pointers
**  If this is the first time the resource for this line is being
**  referenced the pointer to the resource table entry will be set
**  and the resource name will be saved. If the resource name has
**  already been set, this routine will verify that all other points
**  for this driver are using the same resource name.
**
*/                                                                                    	
/* this routine called from AIDRIVER, AODRIVER, AOREAD, BIDRIVER, BODRIVER,		
			    BOREAD, SIDRIVER, and SODRIVER.                             
*/                               	

static void  register_resource (unsigned short link, MODPLC *pmod,
                                unsigned char *resname, char *pointname)
{
    int group;

	if (resnames [link] [0] == 0) /* This is the first reference */
	{
	    if (resGetByName (resname, &prestable [link]) != RES_OK)
	    {
		logMsg ("Resource not found in table.(Point Name = %s, Resource Name = %s)\n",
                                (int) pointname,
				(int) resname, 0,0,0,0 );
	    }
	    else
	    {
		strcpy (resnames [link], resname);
                group = modResGroups [link];
                if (group)
                {
                    if (assignLockGroup (group, resname) == ERROR)
                    {
                        logMsg ("assignLockGroup error: Line = %d, Group = %d, Point Name = %s, Resource Name = %s\n",
                                 link, group, (int) pointname, (int) resname, 0,0);
                    }
                }
	    }
	}
	else
	{
	    if (strcmp (resnames [link], resname) != 0)
	    {
		logMsg ("Resource name[%s] conflict.(Point Name = %s, Resource Name = %s)\n",
				(int) resnames[link],
                                (int) pointname,
				(int) resname, 0,0,0 );
	    }
	}
}

/*
**      Initialize the Queue                                        		
*/                                                                        	
/* not called from within this driver */

static	void outqueinit (unsigned short line)
{
	oq [line]->ooutix = oq [line]->oinix = 0;
}


/*                                                                     		
**  ogetempty - get the next available slot on the output list         		
*/
/*    called from mod_aodriver and mod_bodriver   */

static	OUTQENT *ogetemptyp (unsigned short line)
{
	if (((oq [line]->oinix + 1) % OUTQSIZE) != oq [line]->ooutix)
	{
	    return (&oq [line]->outque[oq [line]->oinix]);
	}
        if (mltrace [line] == 11)
	{
	    logMsg ( "Output Queue Full\n", 0,0,0,0,0,0);
	}
	return (NULLP);
}


/*                                                                    		
** omarkfull                                                          		
**      mark the next block as full                                     	
*/
/*  called from mod_aodriver and mod_bodriver   */

static	void omarkfull (unsigned short line)
{
	oq [line]->oinix = ((oq [line]->oinix + 1) % OUTQSIZE);
}


/*                                                                      	
**  ogetnextp - get the next full slot on the output list               	
*/                                                                      	
/* only called from checkoutputs */

static	OUTQENT *ogetnextp (unsigned short line)
{
	if (oq [line]->ooutix != oq [line]->oinix)
	{
	    return (&oq [line]->outque[oq [line]->ooutix]);
	}
	else
	{
	    return (NULLP);
	}
}


/*                                                       	               	
** oreleasefull - release the current full output block    	              	
*/                                                                            	
/* only called from checkoutputs */

static	void oreleasefull (unsigned short line)
{
	if (oq [line]->ooutix == oq [line]->oinix)
	{
	    return;
	}
	oq [line]->ooutix = ((oq [line]->ooutix + 1) % OUTQSIZE);
}

/*                                                                             	
**                                                                              
**	allocate_plc                                                            
**                                                                              
**	The purpose of this routine is to allocate the memory for a single      
**	plc data structure. If not successful, a console error message is  	
**	printed and an error flag is returned to the caller.                    
**	If successful, the newly allocated data structures are initialized.     
**                                                                             	
*/

static int allocate_plc (unsigned short line, unsigned short plc )
{
	plcs [line] [plc] = (MODPLC *) calloc (1, (sizeof (MODPLC)));
	if (!plcs [line] [plc])
	{
	    logMsg ("Unable to allocate memory", 0,0,0,0,0,0);
            printf (" for MODPLC data structure on line %d, plc %d\n",line ,plc);
	    return(FALSE);
	}
	else
	{
	    logMsg ("Allocated memory for MODPLC data structure on line %d, plc %d\n",line, plc, 0,0,0,0);
	    init_plc_tab (line, plc);	     /*	initialize the data structures. */
	}

	return (TRUE);
}


/*                                                                   		
**      start_rx                                                        	
**                                                                     	 	
**      The purpose of this routine is to initialize a line table        	
**      for receive and to enable the receiver.                           	
**                                                                        	
*/
/* only called from msgxfer */


static	void start_rx (unsigned short line,SEMAPHORE *semid,unsigned short *pstatus )

{
	RXLINE		*rxp;

	rxp = rxlinetab [line];
	rxp->rxix = 0;
	rxp->semid = semid;
	rxp->status = pstatus;
	rxp->rxstate = RXADRST;
        rxp->rxmsglen = rxp->rxmsglenexpected;
}

/*                                                                           	
**      stop_rx                                                              	
**                                                                           	
**      The purpose of this routine is to initialize a line table            	
**      for receive and to disable the receiver.                             	
**                                                                           	
*/
/* only called from msgxfer */


static	void stop_rx (unsigned short line )

    {
	RXLINE		*rxp;

	rxp = rxlinetab [line];
	rxp->rxstate = RXIGNORE;
    }

/*                                                                        	
**      NEW rx_int_proc                                                        	
**                                                                         	
**      The purpose of this routine is to process a receive character interrupt	
**      for a given line.                                                     	
*/
/* referenced in system call in MAIN. indirectly called through actions in msgxfer. */

static	int rx_int_proc (unsigned short line )
    {
	unsigned char   rxchar;
	unsigned short  rxstat;
	unsigned short  rx_err_mask;
	RXLINE          *rxp;

	read_rx_hw (link[line], &rxchar, &rxstat, &rx_err_mask);

        /* only execute if no comm error */
        if ((rxstat & rx_err_mask) == 0)
        {
            /* Something is sending in data. */
            line_activity [line] = TRUE;
            linestats [line].RXintcount++;
        }
        else
        {
            linestats [line].RXbadcount++;
        }

	rxp = rxlinetab [line];

	if (rxp->rxstate == RXIGNORE)
	{
	    return 0;
	}
        /* no matter what the state, errors cause a logical reset of the message processing */
        if ((rxstat & rx_err_mask) != 0)
	{
	    rxp->rxstate = RXADRST;    /* if comm goes bad, assume it was garbage,  */
                                     /* and go wait for the good stuff.           */ 
            rxp->rxix = 0;           /* reset the buffer index/counter            */
            rxp->rxmsglen = rxp->rxmsglenexpected; /* reload decrementing counter.*/
	    return 0;
	}

	rxp->rxbuf [rxp->rxix++] = rxchar; /* store the character in the buffer and bump the index.*/
	rxp->rxmsglen--;                  /* decrement the message length counter. although        */
                                          /* superfluous at the beginning of the fixed length      */
                                          /* messages, it does no harm.                            */
	switch (rxp->rxstate)
	{
	case RXADRST:    /* Address Field State */
             /* test for address validity */
             if (rxchar == rxp->txaddress)
             {
        	rxp->rxstate = RXFNCST;
                return 0;
             }
             else
             {
                rxp->rxstate = RXADRST;                  /* presume it was a false input.  */
                                                               /* go wait for the real one.      */ 
                rxp->rxix = 0;                         /* reset the buffer index/counter.*/
                rxp->rxmsglen = rxp->rxmsglenexpected; /* reload decrementing counter.   */
                return 0;
             }

	case RXFNCST:    /* Function Field State */
            /* if a NAK response is being returned, convert to fixed length processing */
            if ((rxchar & FNCERRBIT) != 0)
	    {
		rxp->fixedlen = FIXEDRESP;
		rxp->rxmsglen = NAKMSGLEN;
                rxp->rxstate = RXDATAST;
                return 0;
	    }
            if (rxp->fixedlen) /* fixed length, rxmsglen is already setup. just go get the data. */
            {
                rxp->rxstate = RXDATAST;
            }
            else               /* for variable length, go read the length byte.                  */
            {
                rxp->rxstate = RXLENST;
            }
	    return 0;

	case RXLENST:    /* Length Field State */
	    rxp->fixedlen = FIXEDRESP;
	    rxp->rxmsglen = rxchar + 2;       /* Add 2 for crc */
	    rxp->rxstate = RXDATAST;
	    return 0;

        case RXDATAST:  /* Data Section State */
	default:
	    if (rxp->rxmsglen == 0)
	    {
		*(rxp->status) = RXGOOD;
		rxp->rxstate = RXIGNORE;
		semGive (rxp->semid);
		return 0;
	    }
	}
	return 0;
}

/*                                                                             
**      check_line_activity                                                     
**                                                                             	
*/
/* only called from mod_driver  */


static	int check_line_activity (unsigned short line, unsigned short predelay )

    {
	RXLINE          *rxp;
	long		thistime;
	short		status;

	if (predelay != 0)
	{
	    taskDelay (predelay);
	}
	thistime = tickGet ();
	if ((thistime - activity_time [line]) < ACTIVITYTIME)
	{
	    return (lastactivity [line] || (line_activity [line]));
	}
	lastactivity [line] = status = line_activity [line];
	line_activity [line] = FALSE;
	activity_time [line] = thistime;
	rxp = rxlinetab [line];
	rxp->rxstate = RXIGNORE;

	return (status);
}

/*                                                                         	
**      start_tx                                                            	
**                                                                          	
**      The purpose of this routine is to initialize a line table           	
**      for send and to enable the transmitter.                              	
**                                                                            	
*/
/* only called from msgxfer  */


static	void start_tx (unsigned short line,SEMAPHORE *semid,unsigned short *pstatus )

{

	TXLINE		*txp;

	disable_tx_ints (link[line]);
	txp = txlinetab [line];
	txp->txix = 0;
	txp->semid = semid;
	txp->status = pstatus;
	txp->txdisable = FALSE;

	/* Might have to output the first character here */


	enable_tx_ints (link[line]);

}


/*
**      stop_tx                                                                	
**                                                                             	
**      The purpose of this routine is to initialize a line table              	
**      for receive and to disable the receiver.                               	
**                                                                             	
*/
/* only called from msgxfer  */


static	void stop_tx (unsigned short line )

    {
	TXLINE		*txp;

	disable_tx_ints (link[line]);
	txp = txlinetab [line];
	txp->txdisable = TRUE;
    }

/*                                                                           	
**      tx_int_proc                                                           	
**                                                                            	
**      The purpose of this routine is to process a transmit character interrupt
**      for a given line.                                                  	
**                                                                         	
*/
/* only called from mod_driver  */


static	int tx_int_proc (unsigned short line )

{
	char                    txchar;
	unsigned short          txstat;
	unsigned short          tx_err_mask;
	TXLINE                  *txp;

        linestats [line].TXintcount++;

	txp = txlinetab [line];
	if (txp->txdisable)
	{
	    return 0;
	}
	if (txp->txix == txp->txmsglen)
	{
	    txp->txdisable = TRUE;
	    disable_tx_ints (link[line]);
	    *(txp->status) = TXGOOD;
	    semGive (txp->semid);
	    return 0;
	}
	else
	{
	    txchar = txp->txbuf [txp->txix++];
	    write_tx_hw (link[line], txchar, &txstat, &tx_err_mask);
	    if ((txstat & tx_err_mask) != 0 )
	    {
	 	txp->txdisable = TRUE;
	        disable_tx_ints (link[line]);
	        *(txp->status) = TXBAD;
	        semGive (txp->semid);
	        return 0;
	    }
	}

	if (txp->txix >= MAXTXMSG)
	{
	    txp->txdisable = TRUE;
	    disable_tx_ints (link[line]);
	    *(txp->status) = TXBAD;
	    semGive (txp->semid);
	    return 0;
	}
	return 0;
}


/*                                                                         	
**      msgxfer                                                              	
**                                                                           	
**      The purpose of this routine is to transmit a buffer to the         	
**      analog devices I/O and to await a response. The transmit and receive	
**      buffers are in their respective line tables. This routine will retry 	
**      the message up to a maximum number of times or until a response      	
**      is received with a proper checksum.                                  	
**              returns         XFERGOOD if response received                	
**                              XFERBAD  if no good response is received     	
**                                                                           	
*/
/* called from modloopback, readcoils, readdis, readais, readregs, setcoil, setreg, */
/*             plc_start, plc_stop, pollfordone, plc_init, plc_getdim, plc_setdim,  */
/*             readmem, writemem, writelogic, and writeascii.                       */

static	unsigned short  msgxfer (unsigned short fline,
                                 unsigned short resplen,
                                 MODPLC *pmod,
                                 unsigned short rxlength)
{
	unsigned short  retry, txstatus, rxstatus, txlen, rxlen, line, ix;
	RXLINE          *rxp;
	TXLINE          *txp;
	unsigned char   csums [2];
        unsigned short  oldrxix;
        unsigned int    delta40, delta01, delta12, delta23, delta34;
        unsigned long   msgstart;

	msgstart = tickGet();

	line = (fline & (~ONERETRY));          /* strip off the no retry flag*/

        delta40 = delta01 = delta12 = delta23 = delta34 = 0;

	rxp = rxlinetab [line];
	txp = txlinetab [line];

	for (retry = ((fline == line) ? 0 : MAXRETRYS - 1); (retry < MAXRETRYS); retry++)
	{
            linestats [line].msgxfercount++;
            loopmsgs [line]++;

            /* time since end of last msgxfer */

            tick [line] [0] = tickGet ();
            delta40 = (tick [line] [0] - tick [line] [4]);

	    if (intermsgdelay [line] != 0)
	    {
		taskDelay (intermsgdelay [line]);
	    }
	    set_rts (link[line]);
	    set_dtr (link[line]);

	    if (pretxdelay [line] != 0)
	    {
		taskDelay (pretxdelay [line]);
	    }

	    txstatus = IO_TIMEOUT;
	    rxstatus = IO_TIMEOUT;
	    rxp->fixedlen = resplen;
	    rxp->rxmsglenexpected = rxlength;
	    rxp->txaddress = txp->txbuf [0];       /* rx_int_proc uses info to parse the response */
	    rxp->txfunction = txp->txbuf [1];

	    semTake (semids [line] [TXSEM], NO_WAIT);
	    semTake (semids [line] [RXSEM], NO_WAIT);

	    txlen = txp->txmsglen;

            if ((trace [line] & ALLTRAFFICTRACE) &&
                ((traceadr [line] == txp->txbuf [0]) || (traceadr [line] == 255)) &&
                ((tracetype [line] == txp->txbuf [1]) || (tracetype [line] == 255)))
            {
               printf ("TX:");
               for (ix = 0; ix < 12 ; ix++) /* 12 is maximum TX message length */
               {
                    if (ix < txlen)
		    {
                       printf("_%2x", txp->txbuf [ix]);
		    }
                    else
		    {
                       printf ("...");
		    }
               } 
               printf ("\n");
            }

            /* time from entering msgxfer to starting transmission. (including delays & diags) */ 

            tick [line] [1] = tickGet ();
            delta01 = (tick [line] [1] - tick [line] [0]);

            start_rx (line, semids [line] [RXSEM], &rxstatus);
	    start_tx (line, semids [line] [TXSEM], &txstatus); /* this fires off the transmission. */

	    if ((semTake (semids [line] [TXSEM], TXTIMEOUT)) == ERROR)
	    {
		txstatus = IO_TIMEOUT;
	    }

            /* time to transmit message. */

            tick [line] [2] = tickGet ();
            delta12 = (tick [line] [2] - tick [line] [1]);

	    stop_tx (line);

	    if (posttxdelay [line] != 0)
	    {
		taskDelay (posttxdelay [line]);
	    }

	    if (halfduplex [line] == TRUE)
	    {
	       clear_rts (link[line]);
	    }
            /* intermessage wait time. */

            tick [line] [3] = tickGet ();
            delta23 = (tick [line] [3] - tick [line] [2]);

	    if (txstatus == TXGOOD)
	    {
                oldrxix = -1;    /* make it pass through timeout loop at least once */
                while ((semTake (semids [line] [RXSEM], (int) (RXTIMEOUT))) == ERROR)
                {
                    if (oldrxix == rxp->rxix)
                    {
                        if (modtrace [line])
			{
                            logMsg("RX timeout\n", 0,0,0,0,0,0);
			}
                        rxstatus = IO_TIMEOUT;
                        break;
                    }
                    else
                    {
                        if ((modtrace [line]) || (mltrace[line] == 12))
			{
                             logMsg("waiting...", 0,0,0,0,0,0);
			}
                    }
                    oldrxix = rxp->rxix;
                }
		stop_rx (line);

                /* time to receive message. */

                tick [line] [4] = tickGet();
                delta34 = (tick [line] [4] - tick [line] [3]);

		rxlen = rxp->rxix;
		if (rxstatus == RXGOOD)
		{
                    if ((trace [line] & ALLTRAFFICTRACE) &&
                        ((traceadr [line] == txp->txbuf [0]) || (traceadr [line] == 255)) &&
                        ((tracetype [line] == txp->txbuf [1]) || (tracetype [line] == 255)))
                    {
                        printf("RX:");
                        for (ix = 0; ix < 26; ix++) /* 26 is maximum RX message length */
                        {
                            if (ix < rxlen)
			    {
                                printf("_%2x", rxp->rxbuf [ix]);
			    }
                            else
			    {
                                printf("...");
			    }
                        } 
	                printf("\n");
                        printf("DELTA: %3d %3d %3d %3d %3d\n", delta40, delta01, delta12, delta23, delta34);
                    }
		    if (rxlen < 3)
		    {
			if (modtrace [line])
			{
			    logMsg("Invalid rxlen... %d\n", rxlen, 0,0,0,0,0);
			}
		    }
		    else
		    {
		        makecrc (&rxp->rxbuf [0], rxlen-2, csums);
		    }
		    if ((csums [0] == rxp->rxbuf [rxp->rxix - 2]) &&
			 (csums [1] == rxp->rxbuf [rxp->rxix - 1]))
		    {
			if ((modtrace [line]) && ((rxp->rxbuf[ 1] & FNCERRBIT) != 0))
			{
			    switch (rxp->rxbuf [2])
			    {
			    case (1):
				logMsg("Modbus Error Illegal Function \n", 0,0,0,0,0,0);
				break;

			    case (2):
				logMsg("Modbus Error Illegal Data Address \n", 0,0,0,0,0,0);
				break;

			    case (3):
				logMsg("Modbus Error Illegal Data Value \n", 0,0,0,0,0,0);
				break;

			    case (4):
				logMsg("Modbus Error Device Failure \n", 0,0,0,0,0,0);
				break;

			    case (5):
				logMsg("Modbus Error Acknowledgement \n", 0,0,0,0,0,0);
				break;

			    case (6):
				logMsg("Modbus Error Busy/Rejected Message \n", 0,0,0,0,0,0);
				break;

			    case (7):
				logMsg("Modbus Error Negative Ack \n", 0,0,0,0,0,0);
				break;
			    }
			}

                        pmod->msgs++;
                        pmod->responsetime += (tickGet() - msgstart);
			return (XFERGOOD);
		    }
		    else
		    {
			pmod->no_crcerrs++;
			if (modtrace [line])
			{
			    logMsg ("CRC Message Length = %d\n", rxlen, 0,0,0,0,0);
			}
			if (modtrace [line])
			{
			    logMsg ("%x%x%x%x Bad CRC\n",
					rxp->rxbuf [rxp->rxix - 2], rxp->rxbuf [rxp->rxix - 1],
					csums [0], csums [1],
					0,0);
			}
			for (ix = 0; ix < rxlen; ix++)
			{
			    if (modtrace [line])
			    {
			    	logMsg ("%x\n", rxp->rxbuf [ix], 0,0,0,0,0);
			    }
			}
		    }
		}
		else if (rxstatus == IO_TIMEOUT)
		{
                    if (modtrace [line])
                    {
		    	logMsg ("Receive Timeout... Length: %d\n", rxlen, 0,0,0,0,0);
                    }
                    if ((trace [line] & ALLTRAFFICTRACE) &&
                        ((traceadr [line] == txp->txbuf [0]) || (traceadr [line] == 255)) &&
                        ((tracetype [line] == txp->txbuf [1]) || (tracetype [line] == 255)))
                    {
                        printf("DELTA: %3d %3d %3d %3d %3d\n", delta40, delta01, delta12, delta23, delta34);
                    }
		    pmod->no_rxnoresp++;
		}
		else
		{
		    if (modtrace [line])
		    {
		    	logMsg ("Receive Error\n", 0,0,0,0,0,0);
		    }
		    if (modtrace [line])
		    {
		    	logMsg ( "RXHWERR: %2x, RXCHR: %2x, RXCNT: %2x\n",
				 rxp->rxbuf[100],
				 rxp->rxbuf[101],
				 rxp->rxix, 0,0,0);
		    }
		    pmod->no_rxhwerrs++;
		}
	    }
	    else if (txstatus == IO_TIMEOUT)
	    {
		if (modtrace [line])
                {
			logMsg ("Transmit Timeout\n", 0,0,0,0,0,0);
                }
                if ((trace [line] & ALLTRAFFICTRACE) &&
                    ((traceadr [line] == txp->txbuf [0]) || (traceadr [line] == 255)) &&
                    ((tracetype [line] == txp->txbuf [1]) || (tracetype [line] == 255)))
                {
                    printf("DELTA: %3d %3d %3d %3d %3d\n", delta40, delta01, delta12, delta23, delta34);
                }
		pmod->no_txnoresp++;
	    }
	    else
	    {
		if (modtrace [line])
		{
		    logMsg ("Transmit Error\n", 0,0,0,0,0,0);
		}
		pmod->no_txhwerrs++;
	    }
	if (retry != MAXRETRYS)
		pmod->no_retries++;
	}
	if (modtrace [line])
	{
	    logMsg ("Xfer Bad\n", 0,0,0,0,0,0);
	}
	return (XFERBAD);

}

/*
**      init_plc_tab
**
**      The purpose of this routine is to initialize the line tables for the
**      analog devices I/O
**
*/
/* only called from allocate_plc  */


static	void init_plc_tab (unsigned short line, unsigned short mod)

{
	MODPLC  *pmod;

	pmod = plcs [line] [mod];
	if (!pmod)
	{
		return;
	}
	pmod->is_responding = FALSE;
	pmod->holdflag = FALSE;
	pmod->was_requested = FALSE;
	pmod->cmd_force_offline = FALSE;
	pmod->prog_load_stat = PLC_NOT_LOADING;
	pmod->prog_load_number = 0;
	pmod->prog_load_page = 0;
	pmod->plc_start = FALSE;
	pmod->plc_stop = FALSE;

	initplc (pmod);
	initstats (pmod);
}
/*
** Cold Init a PLC
**
*/
/* only called from init_line_tabs  */


static void initplc (MODPLC  *pmod)

{

	unsigned short	i,j;

                                                                	
/*                                                              	
**      COILS ARE INITIALIZED AS READ WRITE VALUES              	
**      WRITES ARE TREATED BY INDIVIDUAL BITS NOT GROUPS       		
**              - NOT REQUESTED                                		
**              - READ BACK NOT VALID                             	
**              - NO NEW VALUES TO GO                                	
*/

	for (i = 0; (i < NUMMODCOILS); i++)
	{
	    pmod->coils [i].was_requested = FALSE;
	    pmod->coils [i].data_valid = FALSE;
	    pmod->coils [i].last_read = 0;
	}

/*                                                                 	
**      DIGITALS ARE INITIALIZED AS NOT REQUESTED AND NOT VALID		
*/
	for (i = 0; (i < NUMMODDIS); i++)
	{
	    pmod->dis [i].was_requested = FALSE;
	    pmod->dis [i].data_valid = FALSE;
	}

/*                                                                 	
**      ANALOG INS ARE INITIALIZED AS NOT REQUESTED AND NOT VALID    	
*/
	for (i = 0; (i < NUMMODAIS); i++)
	{
	    pmod->ais [i].was_requested = FALSE;
	    pmod->ais [i].data_valid = FALSE;
	}

/*                                                             		
**      REGISTERS ARE INITIALIZED AS READ WRITE VALUES             	
**      WRITES ARE TREATED BY INDIVIDUAL REGISTERS NOT GROUPS      	
**              - NOT REQUESTED                                    	
**              - READ BACK NOT VALID                              	
**              - NO NEW VALUES TO GO                               	
*/                                                                  	
	for (i = 0; (i < NUMREGISTERS); i++)
	{
	    pmod->regs [i].was_requested = FALSE;
	    pmod->regs [i].data_valid = FALSE;
	    for (j = 0; j < REGBLOCK; j++)
	    {
		pmod->regs [i].data_togo [j] = FALSE;
	    }
	}

}

/*                                                            		
**                                                              	
** initialize the statistics for a modicon PLC                  	
**                                                              	
*/                                                               	
/* called from AODRIVER, BODRIVER, and init_line_tabs  */


static	void initstats (MODPLC *pmod)

{
	pmod->no_dataout_msgs = 0;  /* 5  Count Output Msgs 		*/
	pmod->no_datain_msgs = 0;   /* 6  Count Data Requests 		*/
	pmod->no_rxnoresp = 0;      /* 7  Count No Responses 		*/
	pmod->no_txnoresp = 0;      /* 8  Count No Responses 		*/
	pmod->no_dataerr = 0;       /* 9  Count Message Echo Errors 	*/
	pmod->no_retries = 0;       /* 10 Count Retries 		*/          	
	pmod->no_rxhwerrs = 0;      /* 11 Count Comm Hardware Errors	*/
	pmod->no_txhwerrs = 0;      /* 12 Count Comm Hardware Errors 	*/
	pmod->no_crcerrs = 0;       /* 13 Count CRC Errors 		*/
	pmod->no_offlines = 0;      /* 14 Count Off Line Transitions 	*/
	pmod->badoutcnt = 0;	    /* 17 Count failed output commands 	*/

        pmod->tscanpass = 0;        /* plc scan pass counter            */
        pmod->msgs = 0;             /* good response message counts     */
        pmod->responsetime = 0;     /* total wait time for good messages*/
}



/*
** hiamodloopback                                                            
**                                                                        
*/                                                                          	
/* only called from on_line */

static	unsigned short hiamodloopback (unsigned short line, unsigned short mod, MODPLC *pmod)
{
    TXLINE         *txp;
    RXLINE         *rxp;
    unsigned short  bitaddr;
    unsigned short  readlen;

    bitaddr = 0;
    readlen = 1;

    /* Set up pointers for comm buffers */
    txp = txlinetab [line & (~ONERETRY)];
    rxp = rxlinetab [line & (~ONERETRY)];

    pmod->no_datain_msgs++;

    /* Format the output message */

    txp->txbuf [0] =  mod;
    txp->txbuf [1] =  MODDIREAD;
    txp->txbuf [2] =  (bitaddr >> 8);
    txp->txbuf [3] =  (bitaddr & 0xFF);
    txp->txbuf [4] =  (readlen >> 8);
    txp->txbuf [5] =  (readlen & 0xFF);

    makecrc (&(txp->txbuf [0]), MODREADMSGLEN, &(txp->txbuf [MODREADMSGLEN]));

    txp->txmsglen = MODREADMSGLEN + MODCRCLEN;

    /* Send out the message */
    if (msgxfer (line, VARRESP, pmod, 0) == XFERBAD)
    {
        pmod->is_responding = FALSE;
        return (FALSE);
    }
    return (TRUE);
}


/*                                                                       
** modloopback                                                            
**                                                                        
*/                                                                          	
/* only called from on_line */

static	unsigned short modloopback (unsigned short line, unsigned short mod,
			 MODPLC *pmod)
{
	TXLINE 		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Format the output message */

	txp->txbuf [0] =  mod;
	txp->txbuf [1] =  MODLOOPBACK;
	txp->txbuf [2] =  0;
	txp->txbuf [3] =  0;
	txp->txbuf [4] =  0xA5;
	txp->txbuf [5] =  0x37;

	makecrc (&(txp->txbuf [0]), MODLOOPBACKLEN, &(txp->txbuf [MODLOOPBACKLEN]));

	txp->txmsglen = MODLOOPBACKLEN+MODCRCLEN;


	pmod->no_datain_msgs++;

	/* Send out the message */

	if (msgxfer (line, FIXEDRESP, pmod, MODLOOPBACKRESPLEN) == XFERBAD)
	{
	    pmod->is_responding = FALSE;
	    return (FALSE);
	}

	return (TRUE);
}

/*                                                                           	
**     prepresponses
**                                                                             	
**      The purpose of this routine is to mark the PLCs as on hold and the    	
**	data as not valid until read again from the PLC                       	
**                                                                           	
*/
/* only called from mod_driver */

static	void prepresponses (unsigned short line)

{
	unsigned short	mod;
	MODPLC		*pmod;

	for (mod = 1; (mod <= NUMMODPLCS); mod++)
	{
            pmod = plcs [line] [mod];
	    if (pmod)				/* verify pointer validity prior to use */
	    {
	    	pmod->holdflag = TRUE;
		mark_not_valid (pmod);
            }
	}
}




/*                                                                  	
**      any_responding                                               	
**                                                                   	
**      The purpose of this routine is to see if any units are       	
**	responding on this line                                      	
**                                                                   	
*/
/* only called from mod_driver */

static	int any_responding (unsigned short line)

{
	unsigned short  mod;
	MODPLC		*pmod;

	for (mod = 1; (mod <= NUMMODPLCS); mod++)
	{
	    pmod = plcs [line] [mod];
	    if (pmod)			/* verify pointer validity prior to use */
	    {
                /* check cmd_force_offline flag because it is set in an asynchronous task */
                if ((pmod->is_responding) && (!pmod->cmd_force_offline))
		{
		    return (TRUE);
		}
	    }
	}
	return (FALSE);
}




/*                                                                	
** Readcoils                                                       	
**                                                                  	
*/
static	unsigned short readcoils (unsigned short line, unsigned short mod,
			 MODPLC *pmod, unsigned short bitaddr,
			 unsigned short readlen)

/* only called from scan_plc */

{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned short  i;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Format the output message */

	txp->txbuf [0] =  mod;
	txp->txbuf [1] =  MODCOILREAD;
	txp->txbuf [2] =  (bitaddr >> 8);
	txp->txbuf [3] =  (bitaddr & 0xFF);
	txp->txbuf [4] =  (readlen >> 8);
	txp->txbuf [5] =  (readlen & 0xFF);

	makecrc (&(txp->txbuf [0]), MODREADMSGLEN, &(txp->txbuf [MODREADMSGLEN]));

	txp->txmsglen = MODREADMSGLEN + MODCRCLEN;

	/* Send out the message */

	if (msgxfer (line, VARRESP, pmod, 0) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (CMDFAIL);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != mod) ||
	    (rxp->rxbuf [1] != MODCOILREAD) ||
	    (rxp->rxbuf [2] != ((readlen+7)/8)))
	{
	    pmod->no_dataerr++;
	    return (CMDNACK);
	}

	/* Store the returned data                      */
	/*  data is aligned by 8 pseudo register        */
	/*  and a multiple of 8 bits long               */

	for (i = 0; (i < rxp->rxbuf [2]); i++)
	{
	    pmod->coils [(bitaddr / 8) + i].last_read = rxp->rxbuf [3 + i];
	    pmod->coils [(bitaddr / 8) + i].data_valid = TRUE;
	}

	return (CMDDONE);
}

/*                                                             	
** Readdis                                                      	
**                                                              	
*/
static	unsigned short readdis (unsigned short line, unsigned short mod,
			 MODPLC *pmod, unsigned short bitaddr,
			 unsigned short readlen)

/* only called from scan_plc */

{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned short  i;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Format the output message */

	txp->txbuf [0] =  mod;
	txp->txbuf [1] =  MODDIREAD;
	txp->txbuf [2] =  (bitaddr >> 8);
	txp->txbuf [3] =  (bitaddr & 0xFF);
	txp->txbuf [4] =  (readlen >> 8);
	txp->txbuf [5] =  (readlen & 0xFF);

	makecrc (&(txp->txbuf [0]), MODREADMSGLEN, &(txp->txbuf [MODREADMSGLEN]));

	txp->txmsglen = MODREADMSGLEN + MODCRCLEN;

	/* Send out the message */

	if (msgxfer (line, VARRESP, pmod, 0) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (CMDFAIL);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != mod) ||
	    (rxp->rxbuf [1] != MODDIREAD) ||
	    (rxp->rxbuf [2] != ((readlen + 7) / 8)))
	{
	    pmod->no_dataerr++;
	    return (CMDNACK);
	}

	/* Store the returned data                      */
	/*  data is aligned by 8 pseudo register        */
	/*  and a multiple of 8 bits long               */

	for (i = 0; (i < rxp->rxbuf [2]); i++)
	{
	    pmod->dis [(bitaddr / 8) + i].last_read = rxp->rxbuf[3+i];
	    pmod->dis [(bitaddr / 8) + i].data_valid = TRUE;
	}

	return (CMDDONE);
}

/*                                                               	
**                                                         	
**                                                                	
*/
static	unsigned short readais (unsigned short line, unsigned short mod,
			 MODPLC *pmod, unsigned short regaddr,
			 unsigned short reglen)

/* only called from scan_plc */

    {

	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned short  buildreg,signal;
	unsigned short  i, j;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Format the output message */

	txp->txbuf [0] =  mod;
	txp->txbuf [1] =  MODAIREAD;
	txp->txbuf [2] =  (regaddr >> 8);
	txp->txbuf [3] =  (regaddr & 0xFF);
	txp->txbuf [4] =  (reglen >> 8);
	txp->txbuf [5] =  (reglen & 0xFF);

	makecrc (&(txp->txbuf [0]), MODREADMSGLEN, &(txp->txbuf [MODREADMSGLEN]));

	txp->txmsglen = MODREADMSGLEN + MODCRCLEN;

	/* Send out the message */

	if (msgxfer (line, VARRESP, pmod, 0) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (CMDFAIL);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != mod) ||
	    (rxp->rxbuf [1] != MODAIREAD) ||
	    (rxp->rxbuf [2] != (reglen * 2)))
	{
	    pmod->no_dataerr++;
	    return (CMDNACK);
	}

	/* Store the returned data                      */
	/*  data is aligned by 4 register sets          */
	/*  and a multiple of 4 registers long          */

	for (i = 0; (i < rxp->rxbuf [2]); i += 8)
	{
	    for (j = 0; (j < 4); j++)
	    {
                buildreg = rxp->rxbuf [3 + i + (j * 2)];
                buildreg = (buildreg << 8) + rxp->rxbuf [3 + i + (j * 2) + 1];

                signal = (regaddr+(i/2)+j);
                pmod->ais [(regaddr / 4) + (i / 8)].data_valid = TRUE;
 
                /*    If the data has a 0xffff in it do not store the value */
                if ((hathawayflag [line]) && 
                    (signal >= HIAUASAOSTART) &&
                    (signal < HIAUASAOEND) &&
                    (buildreg == 0xffff) )
                {
                    continue;
                } else {
                    pmod->ais [(regaddr / 4) + (i / 8)].aivals [j] = buildreg;
                }
            }
	}
	return (CMDDONE);
    }


/*                                                                      	
** Readregs                                                                	
**                                                                            	
*/
static	unsigned short readregs (unsigned short line, unsigned short mod,
			 MODPLC *pmod, unsigned short regaddr,
			 unsigned short reglen)

/* only called from scan_plc */

{

	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned short  buildreg;
	unsigned short  i, j;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Format the output message */

	txp->txbuf [0] =  mod;
	txp->txbuf [1] =  MODREGREAD;
	txp->txbuf [2] =  (regaddr >> 8);
	txp->txbuf [3] =  (regaddr & 0xFF);
	txp->txbuf [4] =  (reglen >> 8);
	txp->txbuf [5] =  (reglen & 0xFF);

	makecrc (&(txp->txbuf [0]), MODREADMSGLEN, &(txp->txbuf [MODREADMSGLEN]));

	txp->txmsglen = MODREADMSGLEN + MODCRCLEN;

	/* Send out the message */

	if (msgxfer (line, VARRESP, pmod, 0) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (CMDFAIL);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != mod) ||
	    (rxp->rxbuf [1] != MODREGREAD) ||
	    (rxp->rxbuf [2] != (reglen * 2)) )
	{
	    pmod->no_dataerr++;
	    return (CMDNACK);
	}

	/* Store the returned data                      */
	/*  data is aligned by 4 register sets          */
	/*  and a multiple of 4 registers long          */

	for (i = 0; (i < rxp->rxbuf [2]); i += 8)
	{
	    for (j = 0; (j < 4); j++)
	    {
		buildreg = rxp->rxbuf [3 + i + (j * 2)];
		buildreg = (buildreg << 8) + rxp->rxbuf [3 + i + (j * 2) + 1];
		pmod->regs [(regaddr / 4) + (i / 8)].ioregs [j] = buildreg;
	    }
	    pmod->regs [(regaddr / 4) + (i / 8)].data_valid = TRUE;
	}
	return (CMDDONE);
}


/*                                                           		
** Setcoil                                                        	
**                                                                 	
*/
static	unsigned short setcoil (unsigned short line, unsigned short mod,
			 MODPLC *pmod, unsigned short bitaddr,
			 unsigned long mask, unsigned long val)

/* only called from checkoutputs */

{
	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned short  i, ix, lsignal;
	unsigned long	lmask, lval, tmask ,tval;


	if (mask == 0)
	{
	    return (CMDREJECT);
	}
	tmask = lmask = mask;
	tval = lval = val;
	lsignal = bitaddr;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	 /* adjust the bits to the low order bit */

	while ((tmask & 1) == 0)
	{
	    tval = tval >> 1;          /* adjust the value */
	    tmask = tmask >> 1;         /* adjust the mask */
	}

	/* Is it a single coil output */

	if ((tmask & 2) == 0)
	{
	    /* Format the output message */

	    txp->txbuf [0] =  mod;
	    txp->txbuf [1] =  MODCOILWRITE;
	    txp->txbuf [2] =  (lsignal >> 8);
	    txp->txbuf [3] =  (lsignal & 0xFF);
	    txp->txbuf [4] =  (lval==0)?0:0xFF;  /* 0 => off  0xFF => on */
	    txp->txbuf [5] =  0;

	    makecrc (&(txp->txbuf [0]), MODSETCOILLEN, &(txp->txbuf [MODSETCOILLEN]));

	    txp->txmsglen = MODSETCOILLEN + MODCRCLEN;


	    /* Send out the message */

	    if (msgxfer (line, FIXEDRESP, pmod, MODWRITERESPLEN) == XFERBAD)
	    {
		pmod->no_offlines++;
	        pmod->holdflag = TRUE;
		pmod->is_responding = FALSE;
		return (CMDFAIL);
	    }

	    /* Verify the response - the message is repeated back*/
	    for (i = 0; (i < MODSETCOILLEN); i++)
	    {
		if (txp->txbuf [i] != rxp->rxbuf [i])
		{
		    if (rxp->rxbuf [1] & FNCERRBIT)
		    {
			if (rxp->rxbuf [2] == MODEXCEPTBUSY)
			{
			    return (CMDBUSY);
			}
		    }
		    pmod->no_dataerr++;
		    return (CMDNACK);
		}
	    }
	    return (CMDDONE);
	}


	/* Must be a multiple coil output */

	else
	{
	    lmask = tmask;
	    lval = tval;

	    /* count the number of coils to output */

	    for (i = 0,tmask = 1; ((tmask & lmask) != 0); i++)
	    	tmask = tmask << 1;

	    /* Format the output message */

	    txp->txbuf [0] =  mod;
	    txp->txbuf [1] =  MODMCOILWRITE;
	    txp->txbuf [2] =  (lsignal >> 8);
	    txp->txbuf [3] =  (lsignal & 0xFF);
	    txp->txbuf [4] =  (i >> 8);          /* number of coils */
	    txp->txbuf [5] =  (i & 0xFF );
	    txp->txbuf [6] =  (((i + 7) / 8) & 0xFF); /* number of data bytes */
	    txp->txbuf [7] =  (lval & 0xFF );
	    ix = 8;
	    if (i > 8)
	    {
		txp->txbuf [ix++] =  ((lval >> 8) & 0xFF );
		if (i > 16)
		{
		    txp->txbuf [ix++] =  ((lval >> 16) & 0xFF );
		    if (i > 24)
		    {
			txp->txbuf [ix++] =  ((lval >> 24) & 0xFF );
		    }
		}
	    }

	    makecrc (&(txp->txbuf [0]), ix, &(txp->txbuf [ix]));

	    txp->txmsglen = ix+MODCRCLEN;

	    /* Send out the message */

	    if (msgxfer (line, FIXEDRESP, pmod, MODWRITERESPLEN) == XFERBAD)
	    {
		pmod->no_offlines++;
	        pmod->holdflag = TRUE;
		pmod->is_responding = FALSE;
		return (CMDFAIL);
	    }

	    /* Verify the response - first part of the message is repeated back*/

	    for (i = 0; (i < MODMSETCOILRESPLEN); i++)
	    {
		if (txp->txbuf [i] != rxp->rxbuf [i])
		{
		    if (rxp->rxbuf [1] & FNCERRBIT)
		    {
			if (rxp->rxbuf [2] == MODEXCEPTBUSY)
			{
			    return (CMDBUSY);
			}
		    }
		    pmod->no_dataerr++;
		    return (CMDNACK);
		}
	    }
	    return (CMDDONE);
	}
}

/*
** Setreg                                                       	
**                                                                  	
*/
static	unsigned short setreg (unsigned short line, unsigned short mod,
			 MODPLC *pmod, unsigned short regaddr,
			 unsigned short val,double dblval, unsigned short conv)

/* only called from checkoutputs */

{
	TXLINE 		*txp;
	RXLINE 		*rxp;
	unsigned short	i;
   	union 	u_value {
		    short		ivalue;
		    long		lvalue;
		    float		fvalue;
		    double		dvalue;
		    char		afvalue[100];
		}  uval;

	unsigned short	msglen [] =
			{
					0,
					MODSET1REGLEN,
					MODSET2REGLEN,
					MODSET3REGLEN,
					MODSET4REGLEN,
			};

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	uval.fvalue = dblval;

	/* Format the output message */

	txp->txbuf [0] =  mod;
	txp->txbuf [2] =  (regaddr >> 8);
	txp->txbuf [3] =  (regaddr & 0xFF);

	if (microsolflag [line])
	{
	    /* In this case val is the number of registers to write and
		conv is the offset into the 4 register array of the first one */

	    txp->txbuf [1] =  MODMREGWRITE;
	    txp->txbuf [4] = 0x00;
	    txp->txbuf [5] = val;	/* Number of registers */
	    txp->txbuf [6] = (val * 2);	/* Number of bytes */

	    txp->txbuf [7] = (pmod->regs [regaddr / 4].regs_togo [conv] >> 8) & 0xFF;
	    txp->txbuf [8] = pmod->regs [regaddr / 4].regs_togo [conv] & 0xFF;
	    if (val > 1)
	    {
		txp->txbuf [9] = (pmod->regs [regaddr / 4].regs_togo [conv + 1] >> 8) & 0xFF;
		txp->txbuf [10] = pmod->regs [regaddr / 4].regs_togo [conv + 1] & 0xFF;
	    }
	    if (val > 2)
	    {
		txp->txbuf [11] = (pmod->regs [regaddr / 4].regs_togo [conv + 2] >> 8) & 0xFF;
		txp->txbuf [12] = pmod->regs [regaddr / 4].regs_togo [conv + 2] & 0xFF;
	    }
	    if (val > 3)
	    {
		txp->txbuf [13] = (pmod->regs [regaddr / 4].regs_togo [conv + 3] >> 8) & 0xFF;
		txp->txbuf [14] = pmod->regs [regaddr / 4].regs_togo [conv + 3] & 0xFF;
	    }

	    makecrc (&(txp->txbuf [0]), msglen [val], &(txp->txbuf [msglen [val]]));

	    txp->txmsglen = msglen [val] + MODCRCLEN;

	    /* Send out the message */

	    if (msgxfer (line, FIXEDRESP, pmod, MODWRITERESPLEN) == XFERBAD)
	    {
	        pmod->no_offlines++;
	        pmod->holdflag = TRUE;
	        pmod->is_responding = FALSE;
	        return (CMDFAIL);
	    }

	    /* Verify the response - the message is repeated back*/

	    for (i = 0; (i < MODSETMREGRESPLEN); i++)
	    {
		if (txp->txbuf [i] != rxp->rxbuf [i])
		{
		    if (rxp->rxbuf [1] & FNCERRBIT)
		    {
			if (rxp->rxbuf [2] == MODEXCEPTBUSY)
			{
			    return (CMDBUSY);
			}
		    }
		    pmod->no_dataerr++;
		    return (CMDNACK);
	        }
	    }
	    return (CMDDONE);
	}
	if (conv != ASCFLT)
	{
	    txp->txbuf [1] =  MODREGWRITE;
	    txp->txbuf [4] =  (val >> 8);
	    txp->txbuf [5] =  (val & 0xFF);

	    makecrc (&(txp->txbuf [0]), MODSETREGLEN, &(txp->txbuf [MODSETREGLEN]));

	    txp->txmsglen = MODSETREGLEN + MODCRCLEN;

	    /* Send out the message */

	    if (msgxfer (line, FIXEDRESP, pmod, MODWRITERESPLEN) == XFERBAD)
	    {
	        pmod->no_offlines++;
	        pmod->holdflag = TRUE;
	        pmod->is_responding = FALSE;
	        return (CMDFAIL);
	    }

	    /* Verify the response - the message is repeated back*/

	    for (i = 0; (i < MODSETREGLEN); i++)
	    {
		if (txp->txbuf [i] != rxp->rxbuf [i])
		{
		    pmod->no_dataerr++;
		    return (CMDNACK);
	        }
	    }
	    return (CMDDONE);
	}
	else
	{ /* Send out float as multiple register write */

	    txp->txbuf [1] =  MODMREGWRITE;
	    txp->txbuf [4] = 0x00;
	    txp->txbuf [5] = 0x02;	/* Number of registers */
	    txp->txbuf [6] = 0x04;	/* Number of bytes */

	    txp->txbuf [7] = (uval.lvalue >> 8) & 0xFF;
	    txp->txbuf [8] =  uval.lvalue & 0xFF;
	    txp->txbuf [9] = (uval.lvalue >> 24) & 0xFF;
	    txp->txbuf [10] = (uval.lvalue >> 16) & 0xFF;


	    makecrc (&(txp->txbuf [0]), MODSET2REGLEN, &(txp->txbuf [MODSET2REGLEN]));

	    txp->txmsglen = MODSET2REGLEN + MODCRCLEN;

	    /* Send out the message */

	    if (msgxfer (line, FIXEDRESP, pmod, MODWRITERESPLEN) == XFERBAD)
	    {
	        pmod->no_offlines++;
	        pmod->holdflag = TRUE;
	        pmod->is_responding = FALSE;
	        return (CMDFAIL);
	    }

	    /* Verify the response - the message is repeated back*/

	    for (i = 0; (i < MODSETMREGRESPLEN); i++)
	    {
		if (txp->txbuf [i] != rxp->rxbuf [i])
		{
		    pmod->no_dataerr++;
		    return (CMDNACK);
	        }
	    }
	    return (CMDDONE);
	}
}


/*
** Setstring                                                       	
**                                                                  	
*/
static	unsigned short setstring (unsigned short line, unsigned short mod,
			          MODPLC *pmod, unsigned short signal,
			          unsigned char *buf, unsigned short olen)
/*  if (!(setstring (link, adapter, pmod, stringaddr, pstringval, stringlen))) */
/* only called from checkoutputs */
{
	TXLINE 		*txp;
	RXLINE 		*rxp;
	unsigned short  i, slen, tlen, base;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	/* Format the output message */

	txp->txbuf [0] =  mod;
        txp->txbuf [1] =  MODMREGWRITE;
        txp->txbuf [2] =  (signal >> 8);
        txp->txbuf [3] =  (signal & 0xFF);

	/* Store the transmit data                      */

	base = 7;
        i = 0;
	slen = olen;
	while  (i < STRINGMSGLEN)
        {
	    if ((slen != 0) && (i >= slen))
	    {
               break;
	    }
	    txp->txbuf [base+i] = buf [i];

	    if ((slen == 0) && ((buf [i] == CR) || (buf [i] == 0)))
            {
                i++;
                break;
            }
	    i++;
	}
        if ((i % 2) == 1) /* convert all odd length strings to even length strings */
        {
            txp->txbuf [base + i] = buf [i];
            i++;
        }
 	slen = i;
        tlen = base + slen;

	txp->txbuf [4] = ((slen / 2) / 256);
	txp->txbuf [5] = ((slen / 2) & 0xFF);
        txp->txbuf [6] = slen;

	makecrc (&(txp->txbuf [0]), tlen, &(txp->txbuf [tlen]));
        txp->txmsglen = tlen + MODCRCLEN;

        /* Send out the message */

        if (msgxfer (line, FIXEDRESP, pmod, MODWRITERESPLEN) == XFERBAD)
        {
            pmod->no_offlines++;
            pmod->holdflag = TRUE;
            pmod->is_responding = FALSE;
	    return (CMDFAIL);
        }

        /* Verify the response - the message is repeated back*/

        for (i = 0; (i < MODSETMREGRESPLEN); i++)
        {
	    if (txp->txbuf [i] != rxp->rxbuf [i])
            {
                pmod->no_dataerr++;
                return (CMDNACK);
            }
        }
        return (CMDDONE);
}

/*
**	Read Area Descriptors                                		
*/                                                                 	
/* only called from scan_plc */

static int loadinit (unsigned short line,MODPLC *pmod,unsigned short mod)
    {

	LOADINFO	*plinfo;
	char		addr [8];
        int 		status;
	long		pos;
	unsigned short	prognum;

	plinfo = &linfo [line];		/* establish the pointer */

	/* If already one being loaded wait else claim it */

	if (plinfo->loading)
	{
	    return(0);
	}
	plinfo->line = line;	/* easy way to find it */
	plinfo->loading = TRUE;
	plinfo->pmod = pmod;
	plinfo->mod = mod;
	prognum = pmod->prog_load_number;

	/* Make File Name	*/

	strcpy (plinfo->filename, "P");
	addr [0] = 'L';			/* Line Number */
	addr [1] = '0' + (line / 10);
	addr [2] = '0' + (line % 10);
	addr [3] = 'A';			/* Address */
	addr [4] = '0' + (mod / 10);
	addr [5] = '0' + (mod % 10);
	addr [6] = '0' + (prognum % 10);
	addr [7] = 0;
	strcat (plinfo->filename, addr);
	strcat (plinfo->filename, ".prg");

	/* Open File	*/

	iam (user, password);
	if ((plinfo->loadfile = fopen (&plinfo->filename [0], "r")) == NULL)
	{
	    terminateload (plinfo, line, FALSE, CANNOTOPENFILE);
	    return (-1);
	}

	/* Seek to First Area */

	plinfo->fileaddr = (PAGE0 * WORDLEN);
	if ((pos = fseek (plinfo->loadfile, plinfo->fileaddr, 0)) != OK)
	{
	    terminateload (plinfo, line, FALSE, CANNOTSEEKTO);
	    return (-1);
	}

	/* Read First Dump Block */

	if (fread (&plinfo->dadesc [0], 1, DUMPAREASIZE, plinfo->loadfile) != DUMPAREASIZE)
	{
	    terminateload (plinfo, line, FALSE, CANNOTREAD);
	    return (-1);
	}

	/* Seek to Next Area */

	plinfo->fileaddr = (PAGEF * WORDLEN);
	if ((pos = fseek (plinfo->loadfile, plinfo->fileaddr, 0)) != OK)
	{
	    terminateload (plinfo, line, FALSE, CANNOTSEEKTO);
	    return (-1);
	}

	/* Read Next Dump Block */

	if (fread (&plinfo->dadesc [1], 1, DUMPAREASIZE, plinfo->loadfile) != DUMPAREASIZE)
	{
	    terminateload (plinfo, line, FALSE, CANNOTREAD);
	    return (-1);
	}

	/* Seek to Next Area */

	plinfo->fileaddr = (ASCII * WORDLEN);
	if ((pos = fseek (plinfo->loadfile, plinfo->fileaddr, 0)) != OK)
	{
	    terminateload (plinfo, line, FALSE, CANNOTSEEKTO);
	    return (-1);
	}

	/* Read Next Dump Block */

	if (fread (&plinfo->dadesc [2], 1, DUMPAREASIZE, plinfo->loadfile) != DUMPAREASIZE)
	{
	    terminateload (plinfo, line, FALSE, CANNOTREAD);
	    return (-1);
	}

	/* Seek to Last Area */

	plinfo->fileaddr = (FUTURE * WORDLEN);
	if ((pos = fseek (plinfo->loadfile, plinfo->fileaddr, 0)) != OK)
	{
	    terminateload (plinfo, line, FALSE, CANNOTSEEKTO);
	    return (-1);
	}

	/* Read Last Dump Block */

	if (fread (&plinfo->dadesc [3], 1, DUMPAREASIZE, plinfo->loadfile) != DUMPAREASIZE)
	{
	    terminateload (plinfo, line, FALSE, CANNOTREAD);
	    return (-1);
	}

	/* Seek to The First Area in the File */

	plinfo->fileaddr = ((plinfo->dadesc [0].areas [0].strtaddr) * WORDLEN);
	if ((pos = fseek (plinfo->loadfile, plinfo->fileaddr, 0)) != OK)
	{
	    terminateload (plinfo, line, FALSE, CANNOTSEEKTO);
	    return (-1);
	}

	/* LogOut on PLC	*/

	if (!plc_logout (plinfo))
	{
	    terminateload (plinfo, line, FALSE, CANNOTLOGOUT);
	    return (-1);
	}

	/* LogIn on PLC	*/

	if (!plc_login (plinfo))
	{
	    terminateload (plinfo, line, FALSE, CANNOTLOGIN);
	    return (-1);
	}

	/* Read The Machine State	*/

	if (!get_plc_state (plinfo, &plinfo->machstate))
	{
	    terminateload (plinfo, line, FALSE, CANNOTREAD);
	    return (-1);
	}

	/* If Memory Protect Abort */

	if ((plinfo->machstate & MEMPROTECT) == 0)
	{
	    terminateload (plinfo, line, FALSE, MEMORYPROTECT);
	    return (-1);
	}

	/* If Running Stop The Controller */

	if ((plinfo->machstate & RUNNINGSTATE) == 0)
	{
	    if(!plc_stop (plinfo))
	    {
		terminateload (plinfo, line, FALSE, PLCWONTSTOP);
		return (-1);
	    }
	}

	/* Check for dim Awareness */

	status = plc_getdim (plinfo);
	if (status < 0)
	{
	    terminateload (plinfo, line, FALSE, PLCWONTGETDIM);
	    return (-1);
	}

	/* If not in dim awareness */

	if (status == 0)
	{
	    /* Send to dim awareness */

	    if (!plc_setdim (plinfo))
	    {
		terminateload (plinfo, line, FALSE, PLCWONTSETDIM);
		return (-1);
	    }
	    /* Verify dim awareness */

	    status = plc_getdim (plinfo);
	    if (status != 1)
	    {
		terminateload (plinfo, line, FALSE, PLCWONTBEDIM);
		return (-1);
	    }
	}

	/* Initialize the Load Pointer to the first area */

	plinfo->ldblock = 0;
	plinfo->ldarea = 0;

	/* Set The Load State Variables */

	plinfo->pmod->prog_load_stat = PLC_LOAD_IN_PROGRESS;
	plinfo->pmod->prog_load_page = 0;

	return (0);
}


/*                                                              	
**	Load The Next Dump Block                               		
*/
/* only called from scan_plc */

static int loadnextblock (unsigned short line)
{
	LOADINFO	*plinfo;
	unsigned short	type_mask, currarea;
	int		status;

	plinfo = &linfo [line];	/* establish the pointer */

	type_mask = ((plinfo->dadesc [plinfo->ldblock].areaspec << plinfo->ldarea) & AREATYPEMASK);
	switch (type_mask)
	{
	case LOGICAREATYPE:
	    status = loadlogic (plinfo);
	    break;

	case ASCIIAREATYPE:
	    status = loadmemory( plinfo, ASCIIAREATYPE);
	    break;

	case MEMORYAREATYPE:
	default:
	    status = loadmemory (plinfo, MEMORYAREATYPE);
	    break;
	}

	if (status != 0)
	{
	    return(-1);
	}
	currarea = ((plinfo->ldblock * AREASPERBLK) + plinfo->ldarea);
	if (currarea == CONFIGAREA)
	{
	    /* Exit Dim Arareness with a start command*/

	    if (!plc_start (plinfo))
	    {
		terminateload (plinfo, line, FALSE, PLCWONTSTART);
		return (-1);
	    }

	    /* !!Three messages were issued here: 	*/
	    /*	 sf=0x31				*/
	    /*	 sf=0x34				*/
	    /*	confirm dim awareness			*/

	    /* Init PLC (Special Message Issued Here) initialize msg*/

	    status = plc_init (plinfo);
	    if (!status)
	    {
		terminateload (plinfo, line, FALSE, PLCWONTINIT);
		return (-1);
	    }

	    /* Read and Save the Run Load Switch from the PLC */

	    if (!getrunmode (plinfo, &plinfo->rlmode))
	    {
		terminateload (plinfo, line, FALSE, CANTGETMODE);
		return (-1);
	    }

	    /* Set the Run Load Switch to Load */

	    if (!setrunmode (plinfo, LOADSWITCH))
	    {
		terminateload (plinfo, line, FALSE, CANTSETLOADMODE);
		return (-1);
	    }

	}

	/* Advance the block and area pointers for the next load operation */

	plinfo->ldarea = ((plinfo->ldarea + 1) % AREASPERBLK);
	if (plinfo->ldarea == 0)
	{
	    plinfo->ldblock++;
	}
	plinfo->pmod->prog_load_page = currarea + 1;

	/* When we reach the end clean up and enable the next load */

	if (plinfo->ldblock >= NUMDUMPAREAS)
	{
	    terminateload (plinfo, line, TRUE, LOADOK);
	}

	return (0);

}

/*                                                                      	
**	Load a Logic Area                                                	
*/                                                                        	
/* only called from loadnextblock */

static int loadlogic (LOADINFO *plinfo)
    {
	long		pos;
	short		length, page, startaddr, startnode, endnode, dbix, status;
	short		dowrite, endwritten, thisnodecount;
	unsigned char	databuf [MAXTXMSG];

	page = plinfo->dadesc [plinfo->ldblock].page;
	startaddr = plinfo->dadesc [plinfo->ldblock].areas [plinfo->ldarea].strtaddr;
	length = plinfo->dadesc [plinfo->ldblock].areas [plinfo->ldarea].length * NODELEN;
	startnode = FALSE;
	endnode = FALSE;
	endwritten = FALSE;
	dowrite = FALSE;
	dbix = 0;
	thisnodecount = 0;

	while (length)
	{
	    /* While there are nodes to read */
	    /* Read a node (3 Bytes) if error abort */

	    status = fread (&databuf [dbix], 1, NODELEN, plinfo->loadfile);
	    if (status == 0)
	    {
		terminateload (plinfo, plinfo->line, FALSE, UNEXPECTDEOF);
		return (-1);
	    }
	    thisnodecount++;
	    length -= NODELEN;

	    /* If 5X0 set Endnode found */

	    if ((databuf [dbix] == EOLNODE) && (databuf [dbix + 2] == 0))
	    {
		endnode = TRUE;
	    }
	    /* If (<5)XX (this is a start) and I already have one */
	    /*   backup and write what We have so far and resume */

	    if ( (databuf [dbix] <= BOCNODE) && (startnode) )
	    {
		/* Back up over start node from current position */

		pos = fseek (plinfo->loadfile, (long) -NODELEN, 1);
		thisnodecount--;
		length += NODELEN;
		dowrite = TRUE;
		dbix -= NODELEN;
	    }

	    /* If this is the first start node set the flag */

	    if ((databuf [dbix] < BOCNODE) && (!startnode))
	    {
	        startnode = TRUE;
	    }

	    dbix += NODELEN;

	    if ((length == 0) ||
		 (dowrite) || 
		 ((!startnode) && (thisnodecount > 4) ) ||
		 (thisnodecount >= MAXMSGNODES))
	    {
		if (!endwritten)
		{   /* Continue to read but don't write */
		    /* Write logic load message out */
		    status = writelogic (
		    plinfo,
		    page,
		    startaddr,
		    dbix,
		    (char*) databuf);  /* databuf); ###*/
                                      /* cast because the local array is unsigned char */

		    /* Abort on error */
		    if (!status)
		    {
			terminateload (plinfo, plinfo->line, FALSE, LOGICWRITEERROR);
			return (-1);
		    }
		}
		dowrite = FALSE;
		startaddr += thisnodecount; /* Set Start for next block */
		thisnodecount = 0;
		startnode = FALSE;
		endwritten = endnode; 
		dbix = 0;		   /* Set index for next block */
 	    }
	}
	return (0);
}

/*                                                       		
**	Load A Memory or ASCII Area                                  	
*/
/* only called from loadnextblock */

static int loadmemory (LOADINFO *plinfo, unsigned short type)

{
	short		length, page, startaddr, status, thislength;
	char		databuf [MAXTXMSG];

	page = plinfo->dadesc [plinfo->ldblock].page;
	startaddr = plinfo->dadesc [plinfo->ldblock].areas [plinfo->ldarea].strtaddr;
	length = plinfo->dadesc [plinfo->ldblock].areas [plinfo->ldarea].length * WORDLEN;

	while (length)
	{
	    /* While there are nodes to read */
	    /* Read a block of data if error abort */

	    thislength = (length > MAXWORDMSGLEN) ? MAXWORDMSGLEN : length;
	    status = fread (databuf, 1, thislength, plinfo->loadfile);
	    if (status == 0)
	    {
		terminateload (plinfo, plinfo->line, FALSE, UNEXPECTDEOF);
		return (-1);
	    }
	    length -= thislength;
	    if (type == MEMORYAREATYPE)
	    {
		status = writemem (plinfo, page, startaddr, thislength, databuf);
	    }
	    else
	    {
		status = writeascii (plinfo, page, startaddr, thislength, databuf);
	    }
	    /* Abort on error */
	    if (!status)
	    {
		terminateload (plinfo, plinfo->line, FALSE, MEMORYWRITEERROR);
		return (-1);
	    }
	    startaddr += thislength / 2;
	}
	return (0);
}


/*                                                             	
**	End The Load Sequence                                      	
*/
/* called from loadinit, loadnextblock, loadlogic, loadmemory */

static int terminateload (LOADINFO *plinfo,unsigned short line,
	       unsigned short success,unsigned short code)
{
	/* Close The File		*/

	fclose (plinfo->loadfile);

	/* Release for another PLC */

	plinfo->loading = FALSE;

	if (!success)
	{
	    plinfo->pmod->prog_load_stat = PLC_LOAD_FAILED;
	    plinfo->pmod->prog_load_code = code;
	    return 0;
	}

	/* Switch The Controller from Load to Run */

	if (!setrunmode (plinfo, RUNSWITCH))
	{
	    terminateload (plinfo, line, FALSE, CANTSETRUNMODE);
	    return (-1);
	}

	/* Set The Controller to the mode stored in the file */
	/* Which was read back from the controller after config area loaded */

	if (!setrunmode (plinfo, plinfo->rlmode))
	{
	    terminateload (plinfo, line, FALSE, CANTSETLOADEDMODE);
	    return (-1);
	}

	/* !!Two messages were issued here: 		*/
	/*	 sf=0x31				*/
	/*	 sf=0x34				*/

	if (!plc_logout (plinfo))
	{
	    terminateload (plinfo, line, FALSE, CANNOTLOGOUT);
	    return (-1);
	}

	/* Set Pmod status back to no loading */

	plinfo->pmod->prog_load_stat = PLC_NOT_LOADING;
	plinfo->pmod->prog_load_code = code;
	return 0;
}


/*                                                            		
**	PLC Logout                                                	
*/

/* called from scan_plc, loadinit, and terminateload  */


static int plc_logout (LOADINFO *plinfo)

    {
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address		*/
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  1;		/* Length		*/
	txp->txbuf [3] =  MODSFLOGOUT;	/* Subfunction Code 	*/

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [2] != 2 ) ||
	    (rxp->rxbuf [3] != MODSFLOGOUT) ||
	    (rxp->rxbuf [4] != MODCMDDONE) )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}
	return (TRUE);
}

/*                                                               	
**	PLC Login                                                 	
*/

/* called from scan_plc and loadinit  */


static int plc_login (LOADINFO *plinfo)
{
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  1;		/* Length	        */
	txp->txbuf [3] =  MODSFLOGIN;	/* Subfunction Code     */

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [2] != 2 ) ||
	    (rxp->rxbuf [3] != MODSFLOGIN) ||
	    ((rxp->rxbuf [4] & 0x80) != 0) )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}

	return (TRUE);
}

/*                                                          		
**	PLC Start                                                 	
*/
/* called from scan_plc and loadnextblock  */


static int plc_start (LOADINFO *plinfo)
    {
  	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  1;		/* Length	        */
	txp->txbuf [3] =  MODSFSTART;	/* Subfunction Code     */

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    ((rxp->rxbuf [1] & 0x7F) != MODLOAD) )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}

	/* Is it a command done */

	if ((rxp->rxbuf [1] == MODLOAD) &&
	    (rxp->rxbuf [3] == MODSFSTART) &&
	    (rxp->rxbuf [4] == MODCMDDONE) )
	{
	    return (TRUE);
	}

	/* Is it an ACK done */

	if ((rxp->rxbuf [1] == (0x80 + MODLOAD) )&&
	    (rxp->rxbuf [2] == MODCMDSTARTED) )
	{
	    return (pollfordone (plinfo, MODSFSTART, FALSE));
	}
	return (FALSE);
}


/*                                                                	
**	PLC Stop                                         		
*/

/* called from loadinit and scan_plc */


static int plc_stop (LOADINFO *plinfo)
    {
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  1;		/* Length	        */
	txp->txbuf [3] =  MODSFSTOP;	/* Subfunction Code     */

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    ((rxp->rxbuf [1] & 0x7F) != MODLOAD) )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}

	/* Is it a command done */

	if ((rxp->rxbuf [1] == MODLOAD) &&
	    (rxp->rxbuf [3] == MODSFSTOP) &&
	    (rxp->rxbuf [4] == MODCMDDONE) )
	{
	    return (TRUE);
	}

	/* Is it an ACK done */

	if ( (rxp->rxbuf [1] == (0x80 + MODLOAD)) /*&&
	     (rxp->rxbuf [2] == MODCMDSTARTED) */ )
	{
	    return (pollfordone (plinfo, MODSFSTOP, FALSE));
	}
	return (FALSE);
}

/*    									
** pollfordone                                                     	
**	subfunction                                                 	
**                                                                  	
*/
/* called from plc_start, plc_stop, and writelogic  */


static int pollfordone (LOADINFO *plinfo, int subfunction, unsigned short geterrcode)
    {
	unsigned short	i;
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	for (i = 0;i < MAXPOLLSFORDONE; i++)
	{

	    /* Format the output message */

	    txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	    txp->txbuf [1] =  MODPOLL;		/* Function Code	*/

	    makecrc (&(txp->txbuf [0]), 2, &(txp->txbuf [2]));

	    txp->txmsglen = 4;

	    /* Send out the message */

	    if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	    {
		plinfo->pmod->no_offlines++;
	    	plinfo->pmod->holdflag = TRUE;
		plinfo->pmod->is_responding = FALSE;
		return (FALSE);
	    }

	    /* May be used to fetch error code only */

	    if (geterrcode)
	    {
	    	return (TRUE);
	    }
	    /* Verify the response */

	    if ((rxp->rxbuf [0] == plinfo->mod) &&
		(rxp->rxbuf [1] == MODPOLL) &&
		(rxp->rxbuf [2] == 2 ) &&
		(rxp->rxbuf [3] == subfunction) &&
		(rxp->rxbuf [4] == MODCMDDONE) )
	    {
		return (TRUE);
	    }
	}
	return (FALSE);
}

/* 										
**	PLC Init                                                     		
*/                                                                      	
/* only called from loadnextblock  */

static int plc_init (LOADINFO *plinfo)
{
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;		/* Address		*/
	txp->txbuf [1] =  MODLOAD;		/* Function Code	*/
	txp->txbuf [2] =  1;			/* Length		*/
	txp->txbuf [3] =  MODSFINITIALIZE;	/* Subfunction Code 	*/

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [2] != 2 ) ||
	    (rxp->rxbuf [3] != MODSFINITIALIZE) ||
	    (rxp->rxbuf [4] != MODCMDDONE) )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}
	return (TRUE);
}

/*                    								
**	Get Dim Awareness
**		-1 => error    0 => not dim   1=> dim                       	
*/
/* only called from loadinit  */

static int plc_getdim (LOADINFO *plinfo)
    {
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  1;		/* Length	        */
	txp->txbuf [3] =  MODSFCHKDIM;	/* Subfunction Code     */

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (-1);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [3] != MODSFCHKDIM) )
	{
	    plinfo->pmod->no_dataerr++;
	    return (-1);
	}

	if  (rxp->rxbuf [4] == 0)
	{
	    return(0); /* Not in Dim Awareness */
	}
	return (1); /* Am in Dim Awareness */
}


/*          									
**	Set Dim Awareness                                                   	
*/
/* only called from loadinit  */

static int plc_setdim (LOADINFO *plinfo)
    {
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  1;		/* Length	        */
	txp->txbuf [3] =  MODSFENTERDIM;	/* Subfunction Code     */

	makecrc (&(txp->txbuf [0]), 4, &(txp->txbuf [4]));

	txp->txmsglen = 6;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [3] != MODSFENTERDIM) ||
	    (rxp->rxbuf [4] != MODCMDDONE))
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}
	return (TRUE);
}

/*                                                			
**	Set Run Mode                                                	
*/
/* called from loadnextblock and terminateload  */

static int setrunmode (LOADINFO *plinfo, short value)
    {
	return (writemem (
		plinfo,
		0,
	 	RUNLOADSWITCHADDR,
	 	2,
		(char*) &value));  /* &value)); ### */
			   /* (wep: cast required when prototyping was performed. */
}

/*                                            				
**	Get Run Mode                                               	
*/
/* only called from loadnextblock  */

static int getrunmode (LOADINFO *plinfo, short *pvalue)
{
	return (readmem(
		plinfo,
		0,
		RUNLOADSWITCHADDR,
		2,
		(char*) pvalue)); /* pvalue)); ###*/
}

/*                                                                 	
**	Get Machine State                                          	
*/
/* only called from loadinit  */

static int get_plc_state (LOADINFO *plinfo, short *pvalue)
{
	return (readmem (
		plinfo,
		0,
		MACHINESTATEADDR,
		2,
		(char*) pvalue));  /* pvalue)); ###*/
}


/*                                                                	
**	Read Memory (Page,Address,datalength,buffer)               	
*/
/* called from getrunmode and get_plc_state */

static int readmem (LOADINFO *plinfo, unsigned short page, unsigned short address,
	     unsigned short datalen, char *databuf)
{

	TXLINE		*txp;
	RXLINE		*rxp;
	unsigned short	i;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;		/* Address		*/
	txp->txbuf [1] =  MODLOAD;		/* Function Code	*/
	txp->txbuf [2] =  5;			/* Length		*/
	txp->txbuf [3] =  MODSFREADMEM;		/* Subfunction Code 	*/
	txp->txbuf [4] =  (datalen / 2);	/* Count of elements 	*/
	txp->txbuf [5] =  page;			/* Memory Page 		*/
	txp->txbuf [6] =  (address >> 8);	/* Address High 	*/
	txp->txbuf [7] =  (address & 0xFF);	/* Address Low 		*/

	makecrc (&(txp->txbuf [0]), 8, &(txp->txbuf [8]));

	txp->txmsglen = 10;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response */

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [2] != datalen + 5) ||
	    (rxp->rxbuf [3] != MODSFREADMEM) ||
	    (rxp->rxbuf [4] != (datalen / 2)) ||
	    (rxp->rxbuf [5] != page) ||
	    (rxp->rxbuf [6] != (unsigned char) (address >> 8)) ||
	    (rxp->rxbuf [7] != (unsigned char) (address & 0xFF))  )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}
	/* move in the data */
	for (i = 0;(i < datalen); i++)
	{
	    databuf [i] = rxp->rxbuf [8 + i];
	}
	return (TRUE);
}
/*                                                            		
**	Write Memory (Page,Address,datalength,buffer)              	
*/
/* called from loadmemory and setrunmode */

static int writemem (LOADINFO *plinfo, unsigned short page, unsigned short address,
	     unsigned short datalen, char *databuf)
    {
	unsigned short	i;
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;	/* Address	        */
	txp->txbuf [1] =  MODLOAD;	/* Function Code	*/
	txp->txbuf [2] =  5 + datalen;	/* Length	        */
	txp->txbuf [3] =  MODSFWRITEMEM;	/* Subfunction Code     */
	txp->txbuf [4] =  (datalen  /2);	/* Count of elements    */
	txp->txbuf [5] =  page;		/* Memory Page */
	txp->txbuf [6] =  (address >> 8);	/* Address High */
	txp->txbuf [7] =  (address & 0xFF);	/* Address Low  */

	/* move in the data */
	for (i = 0;(i < datalen); i++)
	{
	    txp->txbuf [8 + i] = databuf [i];
	}
	makecrc (&(txp->txbuf [0]), 8 + datalen, &(txp->txbuf [8 + datalen]));

	txp->txmsglen = 10 + datalen;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response is command done*/

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [2] != 2) ||
	    (rxp->rxbuf [3] != MODSFWRITEMEM) ||
	    (rxp->rxbuf [4] != MODCMDDONE)  )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}
	return (TRUE);
}

/*                                                                	
**	Write Logic (Page,Address,datalength,buffer)               	
*/
/* only called from loadlogic */

static int writelogic (LOADINFO *plinfo, unsigned short page, unsigned short address,
	     unsigned short datalen, char *databuf)
    {
	unsigned short	i;
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;		/* Address		*/
	txp->txbuf [1] =  MODLOAD;		/* Function Code	*/
	txp->txbuf [2] =  5 + datalen;		/* Length		*/
	txp->txbuf [3] =  MODSFWRITENODE;	/* Subfunction Code 	*/
	txp->txbuf [4] =  (datalen / 3);		/* Count of elements 	*/
	txp->txbuf [5] =  page;			/* Memory Page 		*/
	txp->txbuf [6] =  (address >> 8);	/* Address High 	*/
	txp->txbuf [7] =  (address & 0xFF);	/* Address Low 		*/

	/* move in the data */
	for (i = 0; (i < datalen); i++)
	{
	    txp->txbuf [8 + i] = databuf [i];
	}
	makecrc (&(txp->txbuf [0]), 8 + datalen, &(txp->txbuf [8 + datalen]));

	txp->txmsglen = 10 + datalen;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}


	/* Verify the response is command done*/

	if ((rxp->rxbuf [0] == plinfo->mod) &&
	    (rxp->rxbuf [1] == MODLOAD) &&
	    (rxp->rxbuf [2] == 2) &&
	    (rxp->rxbuf [3] == MODSFWRITENODE) &&
	    (rxp->rxbuf [4] == MODCMDDONE)  )
	{
	    return (TRUE);
	}

	/* Is it an ACK done */

	if ((rxp->rxbuf [1] == (0x80 + MODLOAD)) &&
	    (rxp->rxbuf [2] == MODCMDSTARTED))
	{
	    return (pollfordone (plinfo, MODSFWRITENODE, FALSE));
	}
	pollfordone (plinfo, MODSFWRITENODE, TRUE);
	return (FALSE);
}

/*                                                            		
**	Write Ascii (Page,Address,datalength,buffer)              	
*/
/* only called from loadmemory */

static int writeascii (LOADINFO *plinfo, unsigned short page, unsigned short address,
	     unsigned short datalen, char *databuf)
    {
	unsigned short	i;
	TXLINE		*txp;
	RXLINE		*rxp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [plinfo->line];
	rxp = rxlinetab [plinfo->line];

	/* Format the output message */

	txp->txbuf [0] =  plinfo->mod;		/* Address		*/
	txp->txbuf [1] =  MODLOAD;		/* Function Code	*/
	txp->txbuf [2] =  5 + datalen;		/* Length		*/
	txp->txbuf [3] =  MODSFWRITEASCII;	/* Subfunction Code 	*/
	txp->txbuf [4] =  (datalen / 2);		/* Count of elements 	*/
	txp->txbuf [5] =  page;			/* Memory Page 		*/
	txp->txbuf [6] =  (address >> 8);	/* Address High 	*/
	txp->txbuf [7] =  (address & 0xFF);	/* Address Low 		*/

	/* move in the data */
	for (i = 0; (i < datalen); i++)
	{
	    txp->txbuf [8 + i] = databuf [i];
	}
	makecrc (&(txp->txbuf [0]), 8 + datalen, &(txp->txbuf [8 + datalen]));

	txp->txmsglen = 10 + datalen;

	/* Send out the message */

	if (msgxfer (plinfo->line, VARRESP, plinfo->pmod, 0) == XFERBAD)
	{
	    plinfo->pmod->no_offlines++;
	    plinfo->pmod->holdflag = TRUE;
	    plinfo->pmod->is_responding = FALSE;
	    return (FALSE);
	}

	/* Verify the response is command done*/

	if ((rxp->rxbuf [0] != plinfo->mod) ||
	    (rxp->rxbuf [1] != MODLOAD) ||
	    (rxp->rxbuf [2] != 2) ||
	    (rxp->rxbuf [3] != MODSFWRITEASCII) ||
	    (rxp->rxbuf [4] != MODCMDDONE)  )
	{
	    plinfo->pmod->no_dataerr++;
	    return (FALSE);
	}
	return (TRUE);
}
/*                                                                           	
** makecrc 									
**                                                                        	
*/
/* called from msgxfer, modloopback, readcoils, readdis, readais, readregs, setcoil, setreg,  */
/*             plc_logout, plc_login, plc_start, plc_stop, pollfordone, plc_init, plc_getdim, */
/*             plc_setdim, readmem, writemem, writelogic, and writeascii.                     */

static	void makecrc (unsigned char *chkbuf, int len,char unsigned *retbuf)

    {
	unsigned short	checksum;
	unsigned int    i, j;
	unsigned short  flag;
	unsigned char   tempch;

	checksum = 0x0ffff;
	for (j = 0; j < len; j++)
	{
	    tempch = chkbuf [j];
	    checksum ^= tempch;
	    for (i = 0; i < 8; i++)
	    {
		flag = checksum & 1;
		checksum >>= 1;
		if (flag)
		{
		    checksum ^= (unsigned short) 0x0a001;
		}
	    }
	}
	retbuf [1] = ((checksum >> 8) & 0xff);
	retbuf [0] = (checksum & 0xff);
}

/*
**      moddumpmsgcnts
**
**      The purpose of this routine is to dump the line level statistics for this driver. 
**
*/
/* called from dump_mod_table */

void moddumpmsgcnts (unsigned short line)

    {
        unsigned long	temp, t, x, y, e, z;

        temp = tickGet ();
        t = (temp - linestats [line].startingtick);

 	x = linestats [line].TXintcount;
	y = linestats [line].RXintcount;
	e = linestats [line].RXbadcount;
	z = linestats [line].msgxfercount;

        if (z == 0)
        {
            printf ("TX Interrupt Count    = %10ld\n", x);
            printf ("RX Interrupt Count    = %10ld\n", y);
            printf ("BadRX Intrpt Count    = %10ld\n", e);
            printf (" ******  No Messages Transferred ****** \n");
            return;
        }

 	printf ("\n");
	printf ("Measurement Interval: clock ticks = %ld, seconds = %ld.%1ld \n", t, t/60, (t%60)/6);
 	printf ("\n");
 	printf ("TX Interrupt Count    = %10ld   Avg TX cnts/sec        = %ld \n", x, (60*x)/t);
 	printf ("RX Interrupt Count    = %10ld   Avg RX cnts/sec        = %ld \n", y, (60*y)/t);
 	printf ("                                     Avg RX cnts/Msgxfer    = %ld.\n", x/z);
 	printf ("                                     Avg TX cnts/Msgxfer    = %ld.\n", y/z);
        printf ("BadRX Intrpt Count    = %10ld\n", e);
 	printf ("\n");
 	printf ("RX+TX Interrupt Count = %10ld   Avg RX+TX cnts/sec     = %ld \n",(x+y),(60*(x+y))/t);
 	printf ("                                     Avg RX+TX cnts/Msgxfer = %ld \n",(x+y)/z);
 	printf ("\n");
	printf ("Msgxfer Count         = %10ld   Avg Msgxfers/sec       = %ld \n", z,(60*z)/t);
	printf ("                                     Avg msec/Msgxfer       = %ld \n",(t*1000)/(z*60));
 	printf ("\n");
        return;
}

/*
**      modclearmsgcnts
**
**      The purpose of this routine is to dump and clear the line level statistics
**      for this driver.
**
*/
/* called from dump_mod_table */

void modclearmsgcnts (unsigned short line)

{
        moddumpmsgcnts (line);

	linestats [line].TXintcount = 0;
	linestats [line].RXintcount = 0;
	linestats [line].RXbadcount = 0;
	linestats [line].msgxfercount = 0;

        linestats [line].startingtick = tickGet ();
	printf (" ******  All Counts Cleared  ****** \n");
 	printf ("\n");
        return;
}


/*
**      moddumpplcstats
**
**      The purpose of this routine is to dump the plc level statistics for this driver.  
**
*/
/* called from dump_mod_table */

void moddumpplcstats (unsigned short line, unsigned short maddr)
{
    MODPLC		*pmod;
    unsigned long   microsecs;

    pmod = plcs [line] [maddr];
    if (!pmod)
    {
        logMsg ("Line/ModbusAddress Not Defined \n", 0,0,0,0,0,0);
        return;
    }

    printf (" Was requested = %d \n", pmod->was_requested);
    printf (" Is responding = %d \n", pmod->is_responding);
    printf (" Prog Load State = %d \n", pmod->prog_load_stat);
    printf (" Prog Load Page = %d \n",  pmod->prog_load_page);
    printf (" Prog Number = %d \n", pmod->prog_load_number);
    printf (" # Output messages = %d \n", pmod->no_dataout_msgs & 0x7fff);
    printf (" # Data in messages = %d \n", pmod->no_datain_msgs & 0x7fff);
    printf (" #No Rx Response = %d \n", pmod->no_rxnoresp & 0x7fff);
    printf (" #No Tx Responses = %d \n", pmod->no_txnoresp & 0x7fff);
    printf (" #Data Errors = %d \n", pmod->no_dataerr & 0x7fff);
    printf (" #Retries = %d \n", pmod->no_retries & 0x7fff);
    printf (" #Rx Hw Errors = %d \n", pmod->no_rxhwerrs & 0x7fff);
    printf (" #Tx Hw Errors = %d \n", pmod->no_txhwerrs & 0x7fff);
    printf (" #Crc Errors = %d \n", pmod->no_crcerrs & 0x7fff);
    printf (" #Off Lines = %d \n", pmod->no_offlines & 0x7fff);
    printf (" Reset Statistics = %d \n", pmod->cmd_reset_stat);
    printf (" Force Offline = %d \n", pmod->cmd_force_offline);
    printf (" #Failed Outputs = %d \n", pmod->badoutcnt & 0x7fff);

    printf ("\n");
    printf (" # of Messages %d \n", pmod->msgs);
    printf (" # Total response time (ticks) = %lu \n", pmod->responsetime);
    if (pmod->msgs)
    {
        printf ("   Average response time (ticks) = %ld \n", (pmod->responsetime/pmod->msgs));
        microsecs = (16667*pmod->responsetime)/pmod->msgs;
        printf ("   Average response time (msec) = %ld.%1ld \n", microsecs/1000, (microsecs%1000)/100);
    }
    printf (" # of scan passes = %d \n", pmod->tscanpass);
    printf ("\n");

    printf (" Resource Responding = %d \n", resresponse[line]);
    printf (" Hold flag  = %d \n", pmod->holdflag );
    return;
}

/*
**      modclearplcstats
**
**      The purpose of this routine is to clear the plc level statistics for this driver. 
**
*/
/* called from dump_mod_table */

void modclearplcstats (unsigned short line, unsigned short maddr)

    {
	MODPLC		*pmod;

	pmod = plcs [line] [maddr];
	if (!pmod)
	{
	    logMsg ("Line/ModbusAddress Not Defined \n", 0,0,0,0,0,0);
	    return;
	}

	initstats (pmod); /* use existing procedure */

        return;
    }


/*
**      dump_mod_table                                                       	
**                                                                           	
**      The purpose of this routine is twofold:                                 
**         1) dump the plc level statistics for the modbus driver.              
**         2) dump the data for the specified modbus plc.                       
**                                                                              
*/
/* not called from within this driver */

void dump_mod_table (unsigned short line, unsigned short maddr)

    {
	MODPLC		*pmod;
	unsigned	short	i, n, j; 

	MODCOILS	*coilptr;
	MODDIS		*diptr;
	REGISTERS	*regptr;
	MODAIS		*aiptr;
	unsigned	short addr, linecnt;
	unsigned	char	buffer [200];
	unsigned short	skip,s;

	pmod = plcs [line] [maddr];
	if (!pmod)
	{
	    logMsg("Line/ModbusAddress Not Defined \n", 0,0,0,0,0,0);
	    return;
	}

        /* previously integral code moved to seperate procedure for debugging convenience */
        moddumpplcstats (line, maddr);


	linecnt = 1;
	printcoilhdr (buffer, linecnt);
	addr = 1;
	for (i = 0; i < (MAXCOILNO / 8); i += 4)
	{
	    if (hathawayflag [line] == TRUE) 
			break;

	    skip = TRUE;
	    for (s = 0; s < 4; s++) {
		coilptr = &pmod->coils[i+s];
		if (coilptr->was_requested) {
			skip = FALSE;
			break;
		}
	    }
	    if (skip) {
	    	addr += 32;		
		continue;
	    }
			
	    printf("  %4d    ", addr);

	    for (n = 0; n < 4; n++)
	    {		
		coilptr = &pmod->coils[i+n];
		printf("%1x,  %1x,         %lx,\t",
                          coilptr->was_requested,
                          coilptr->data_valid,
                          coilptr->last_read);
	    }
	    printf ("\n");
	    linecnt += 1;
	    if ( (linecnt >= 17) && ((i + 4)  < (MAXCOILNO /8)) )
	    {
		linecnt = 1;
		printcoilhdr(buffer, linecnt);
	    }
	    addr += 32;		
	}

	linecnt = 1;
	printdihdr (buffer, linecnt);
	addr = 1;

	for (i = 0; i < (MAXDINO / 8); i += 4)
	{

	    skip = TRUE;
	    for (s = 0; s < 4; s++) {
		diptr = &pmod->dis[i+s];
		if (diptr->was_requested) {
			skip = FALSE;
			break;
		}
	    }
	    if (skip) {
		addr += 32;		
		continue;			
	    }

	    printf ("  %4d    ", addr);

	    for (n = 0; n < 4; n++)
	    {		
 		diptr = &pmod->dis[i+n];
		printf("%1x,  %1x,         %lx,\t",
                           diptr->was_requested,
                           diptr->data_valid,
                           diptr->last_read);
	    }
	    printf ("\n");
	    linecnt += 1;
	    if ((linecnt >= 17) && ((i + 4)  < (MAXDINO / 8)) )
	    {
		linecnt = 1;
		printdihdr (buffer, linecnt);
	    }
	    addr += 32;		
	}


	linecnt = 1;
	printaihdr(buffer, linecnt);
	addr = 1;
	for (i = 0; i < MAXAINO; i += 8)
	{

	    skip = TRUE;
	    for (s = 0; s < 2; s++) {
		aiptr = &pmod->ais[(i/4)+s];
		if (aiptr->was_requested) {
			skip = FALSE;
			break;
		}
	    }
	    if (skip) {
	       addr += 8;		
		 continue;
	    }

			

	    printf ("  %4d\t", addr);
				   
	    for (n = 0; n < 2; n++ )
	    {		
		aiptr = &pmod->ais [(i / 4) + n];
		printf ("\t%1x, %1x,\t", aiptr->was_requested, aiptr->data_valid);
		for (j = 0; j < 4; j++)
		{
		    printf("%6d,\t",  aiptr->aivals[j]);
		}
	    }
	    printf("\n");
	    linecnt += 1;
	    if ((linecnt >= 17) && ((i + 8)  < NUMMODAIS) )
	    {
		linecnt = 1;
		printaihdr (buffer, linecnt);
	    }
	    addr += 8;		
	}

	linecnt = 1;
	printreghdr (buffer, linecnt);
	addr = 1;
	for (i = 0; i < MAXREGNO; i += 8)
	{
	    if (hathawayflag [line] == TRUE) 
			break;

	    skip = TRUE;
	    for (s = 0; s < 2; s++) {
		regptr = &pmod->regs[(i/4)+s];
		if (regptr->was_requested) {
			skip = FALSE;
			break;
		}
	    }
	    if (skip) {
	       addr += 8;		
		 continue;
	    }
	    printf("  %4d\t", addr);

	    for (n = 0; n < 2; n++)
	    {		
		regptr = &pmod->regs[(i/4)+n];
		printf("\t%1x, %1x,\t", regptr->was_requested, regptr->data_valid);
		for (j = 0; j < 4; j++)
		{
		    printf("%6d,\t",regptr->ioregs[j]);
		}
	    }
	    printf ("\n");
	    linecnt += 1;
	    if ((linecnt >= 17) && ((i + 8)  < NUMREGISTERS) )
	    {
		linecnt = 1;
		printreghdr (buffer, linecnt);
	    }
	    addr += 8;		
	}
}

/*
** only called from dump_mod_table 
*/


static void printcoilhdr (unsigned char *buffer, unsigned short cnt)
{	
	taskDelay (120);
	printf ("\n");
	sprintf (buffer,"                                                    PLC REPORT         ");
	printf ( "%s\n", buffer);
	cnt += 1;
	printf ("\n");
	cnt += 1;
        sprintf (buffer,"                                                   MODBUS COILS        ");
	printf ( "%s\n", buffer);
	cnt += 1;
        sprintf (buffer,
               "          R   V      Value\tR   V      Value\tR   V      Value\tR   V      Value");
 	printf ( "%s\n", buffer);
 	cnt += 1;
}
/*
** only called from dump_mod_table
*/


static void printdihdr (unsigned char *buffer, unsigned short cnt)
{
	taskDelay (120);
	printf ("\n");
	sprintf (buffer,"                                                    PLC REPORT         ");
	printf ( "%s\n", buffer);
	cnt += 1;
	printf ("\n");
	cnt += 1;
        sprintf (buffer,"                                              MODBUS DIGITAL INPUTS        ");
	printf ( "%s\n", buffer);
	cnt += 1;
        sprintf (buffer,
                "          R   V      Value\tR   V      Value\tR   V      Value\tR   V      Value");
	printf ( "%s\n", buffer);
 	cnt += 1;
}
/*
** only called from dump_mod_table
*/


static void printaihdr(unsigned char *buffer, unsigned short cnt)
{
	taskDelay (120);
	printf ("\n");
	sprintf (buffer,"                                               PLC REPORT         ");
	printf ( "%s\n", buffer);
	cnt += 1;
	printf ("\n");
	cnt += 1;
       sprintf (buffer,"                                            MODBUS ANALOG INPUTS        ");
	printf ( "%s\n", buffer);
	cnt += 1;
       sprintf (buffer,"\t\tR  V\tValue\tValue\tValue\tValue\t\tR  V\tValue\tValue\tValue\tValue");
 	printf ( "%s\n", buffer);
 	cnt += 1;
}

/*
** only called from dump_mod_table
*/


static void printreghdr (unsigned char *buffer, unsigned short cnt)
{
	taskDelay (120);
	printf ("\n");
	sprintf (buffer,"                                               PLC REPORT         ");
	printf ( "%s\n", buffer);
	cnt += 1;
	printf ("\n");
	cnt += 1;
       sprintf (buffer,"                                             MODBUS REGISTERS        ");
	printf ( "%s\n", buffer);
	cnt += 1;
       sprintf (buffer,"\t\tR  V\tValue\tValue\tValue\tValue\t\tR  V\tValue\tValue\tValue\tValue");
 	printf ( "%s\n", buffer);
 	cnt += 1;

}
/*                                                               		
**                                                                	 	
**	The purpose of this routine is to display the contents            	
**	of the static variables contained within this driver.             	
**                                                                        	
*/

/* this routine not called from within this driver */


int showlineinfo ()
{
	short	i;
	char	buffer [132];
	char	*resrsp;
	char	*lineact;


        printf ("                                 Modbus Line Information Table          \n");
        printf ("\n");
        printf (" Line        Pointer to        Resource      Resource");
        printf ("       Line        Delay Times:       Activity\n");
        printf ("Number      MODPLC Table         Name        Response");
        printf ("     Activity      IM  .TX  TX.         Time\n");
	printf ("\n");

	for (i = 0; i < MODLINES; i++)
	{
	    if(plcs [i] == NULL)
	    {
		continue;
	    }
	    if (resresponse [i] == 0)
	    {
		resrsp = "FALSE";
	    }
	    else
	    {
		resrsp = "TRUE";
	    }
	    if (line_activity [i] == 0)
	    {
		lineact = "FALSE";
	    }
	    else
	    {
		lineact = "TRUE";
	    }
	    sprintf (buffer,
                       "  %2d         %4x   %16s        %5s        %5s        %2d   %2d   %2d        %4ld\n",
			i, (int) plcs[i], resnames[i],
			&resrsp[0],
			&lineact[0],
			intermsgdelay[i],
			pretxdelay[i],
			posttxdelay[i],
			activity_time[i]);
	    printf ( "%s\n", buffer);
	}
	return 0;
}

/*	End of File:	drvmodbus.c */
