/*drvTiway.c,v 1.1.1.1 1999/08/03 14:45:05 tis4000 Exp */

/****************************************************************/
/*								*/
/*	This software is the proprietary property of Tate	*/
/*	Integrated Systems, L.P.  Patents and copyrights are	*/
/*	pending.						*/
/*								*/
/*	This software is furnished under a license and may	*/
/*	be used and copied only in accordance with the terms	*/
/*	of such license and with the inclusion of the above	*/
/*	notice. This software or any other copies thereof	*/
/*	may not be provided or otherwise made available to	*/
/*	any other person. No title to and ownership of the	*/
/*	software is hereby transferred.				*/
/*								*/
/*	The information in this software is subject to		*/
/*	change without notice and should not be construed 	*/
/*	as a commitment by Tate Integrated Systems L.P. or	*/
/*	its third party suppliers.				*/
/*								*/
/*	Tate Integrated Systems, L.P. (TIS) and its third	*/
/*	party suppliers assume no responsibility for the	*/
/*	use or inability to use any of its software. TIS	*/
/*	software is provided "as is" without warranty of	*/
/*	any kind and TIS expressly disclaims all implied	*/
/*	warranties, including but not limited to the implied	*/
/*	warranties of merchantability and fitness for a		*/
/*	particular purpose.					*/
/*								*/
/*	Notice: Notwithstanding any other lease or license	*/
/*	that may pertain to or accompany the delivery of this	*/
/*	computer software, the rights of the Government		*/
/*	regarding its use, reproduction, and disclosure are	*/
/*	as set forth in Section 52.227-19 of the FARS		*/
/*	Computer Software-Restricted Rights clause.		*/
/*								*/
/****************************************************************/
/*
/*  File:	drvTiway.c
/*  Edit:	12/18/92 	12:33 JLD
/*        	01/18/94:	WEP	name changes only
/*		03-30-94	ems	starttiway(), remove test ln >= 0
/*					it is unsigned
/*		04/13/94	WHS	upgrading driver to EPICS
/*		05/09/94	WHS	Creating Tiway driver from Allen Bradley driver
/*              05/22/95        WEP     modified master loop logic to exit when local state is not
/*                                      master. This causes lockstep logic to function correctly.
/*                                      test for and conditionally display an illegal lstate msg.
/*
*/

#define	MAIN		tiway_driver		/* NOTE: change here */
#define	AIDRIVER	tiway_aidriver		/* NOTE: change here */
#define	AODRIVER	tiway_aodriver		/* NOTE: change here */
#define	AOREAD		tiway_aoread		/* NOTE: change here */
#define	BIDRIVER	tiway_bidriver		/* NOTE: change here */
#define	BODRIVER	tiway_bodriver		/* NOTE: change here */
#define	BOREAD		tiway_boread		/* NOTE: change here */

#define	TRACEON		tiwaytraceon		/* NOTE: change here */
#define	TRACEOFF	tiwaytraceoff		/* NOTE: change here */
#define	RTRACEON	tiwayrestraceon		/* NOTE: change here */
#define	RTRACEOFF	tiwayrestraceoff	/* NOTE: change here */
#define	TRACELEVEL	tiwaytrlv		/* NOTE: change here */

#define	STOPDRIVER	stoptiway		/* NOTE: change here */
#define	STARTDRIVER1	starttiway		/* NOTE: change here */
#define	STARTDRIVER2	starttiway		/* NOTE: change here */
#define	SETDELAYS	tiwaysetdelays		/* NOTE: change here */
#define	TABLEINFO	dump_tiway_table	/* NOTE: change here */
#define	LINEINFO	showlineinfo		/* NOTE: change here */

/*#define TEST*/

#ifdef TEST

/* #include "testmode.h" */

#include	"bioscomm.h"
#include	"dummyos.h"
#include 	"drvtiway.h"
#include 	"link.h"
#include	"tiproto.h"
#define		logMsg printf

#endif

#ifndef TEST

#include	<ctype.h>
#include	<vxWorks.h>
#include	<sysLib.h>
#include	<semLib.h>
#include	<wdLib.h>
#include	<rngLib.h>
#include	<task_params.h>
#include	<stdioLib.h>
#include	<tsDefs.h>

#include	<dbDefs.h>
#include	<drvSup.h>
#include	<module_types.h>
#include	<drvTiway.h>
#include	<drvTiway_proto.h>
#include	<restable.h>
#include   	<cvnTypes.h>   /* defines for conversion types */
#endif

struct {
	long		number;
	DRVSUPFUN	report;
	DRVSUPFUN	init;
} drvtiway = {2, NULL, init};

/**********
*
*      External Line Level Data Structures (declared in drvcommon.c)
*
**********/

extern	unsigned	char	resnames [TILINES] [RESNAMELEN];	/* array of resource name strings.		*/

extern	RES_TABLE	*prestable [TILINES];				/* array of pointers to resource tables 	*/

extern	unsigned	short	resresponse [TILINES];			/* array of resource response flags. 		*/
									/* False=not active resource(slave),		*/
									/* True= active resource(master) 		*/

extern	unsigned	short	line_activity [TILINES];		/* true/false line active flag. calculated	*/
									/* using activity_time below. 			*/
extern	unsigned	short	lastactivity [TILINES];			/* assigned value of line_activity prior to	*/
									/* it's recalculation.   			*/

extern	unsigned	short	modtrace [TILINES];			/* controls output of communication error log	*/
									/* messages (true/false) 			*/
extern	unsigned	short	mltrace [TILINES];			/* controls output of certain resource related	*/
									/* log messages (level) 			*/
extern	unsigned	short	modrestrace [TILINES];			/* controls output of MASTER/SLAVE arbitration	*/
									/* log messages (binary) 			*/

extern	SEM_ID			semids [TILINES] [2];			/* 'new' semaphore logic. the old SEMAPHORE is	*/
									/* still used. 					*/

extern	unsigned	short	intermsgdelay [TILINES];		/* holds value passed in task spawn. used for	*/
									/* each msgxfer attempt 			*/
extern	unsigned	short	pretxdelay [TILINES];			/* holds post RTS delay before TX. 		*/
extern	unsigned	short	posttxdelay [TILINES];			/* holds post TX delay before dropping RTS. 	*/

extern	long			activity_time [TILINES];		/* holds timetick of last activity. used to	*/
									/* set line_activity flag  			*/
extern	unsigned	long	masks [];									/* shared resource of the data link 		*/
									/* controls access to common path shared by	*/
									/* data and diagnostic channels. 		*/
									/* originally designed for use with MDS remote	*/									/* radio diagnostics. 				*/
 
/**********
*
*      Storage for Allen Bradley Comm Line Structures
*
**********/

/* array of pointers to dynamically allocated comm controls/buffers */


static	RXLINE		*rxlinetab [TILINES] =
 
			{NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,} ;

static TXLINE          *txlinetab [TILINES] = 

			{NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,
			 NULL, NULL, NULL, NULL, NULL,} ;

/**********
*
*      Storage for Tiway PLC Structures
*
**********/

static	TIPLC		*plcs [TILINES] [NUMTIPLCS + 1];
static	OUTPUTQUE	*oq [TILINES];

static	char		user [40],password [40];	/* holds id and password of the user that       */
							/* spawned the driver task required by host to  */
							/* check file read privileges on PLC load files */
static	short		backup_mode [TILINES];
static	short		channel_select [TILINES];
static	short		uha_state [TILINES];

static	short		lpgainscn [TILINES];		/* Loop Gain */
static	short		lpresetscn [TILINES];		/* Loop Reset */
static	short		lpratescn [TILINES];		/* Loop Rate */
static	short		lphialrmscn [TILINES];		/* Loop High Alarm */
static	short		lploalrmscn [TILINES];		/* Loop Low Alarm */
static	short		lpprocvarscn [TILINES];		/* Loop Process Variable */
static	short		lppvhilimscn [TILINES];		/* Loop PV High Limit */
static	short		lppvlolimscn [TILINES];		/* Loop PV Low Limit */
static	short		lporgdevalrmscn [TILINES];	/* Loop Orange Dev Alarm */
static	short		lpyeldevalrmscn [TILINES];	/* Loop Yellow Dev Alarm */
static	short		lpsmpratescn [TILINES];		/* Loop Sample Rate */
static	short		lpremspscn [TILINES];		/* Loop Remote Set Point */
static	short		lpoutputscn [TILINES];		/* Loop Output */
static	short		lperrorscn [TILINES];		/* Loop Error (Input Only) */
static	short		lpbiasscn [TILINES];		/* Loop Bias */
static	short		lppvhihilimscn [TILINES];	/* Loop PV High High Limit */
static	short		lppvlololimscn [TILINES];	/* Loop PV Low Low Limit */
static	short		lprocalrmscn [TILINES];		/* Loop Rate of Change Alarm */
static	short		anpvhialrmscn [TILINES];	/* Analog Alarm PV High Alarm */
static	short		anpvloalrmscn [TILINES];	/* Analog Alarm PV Low Alarm */
static	short		anprocvarscn [TILINES];		/* Analog Alarm Process Variable */
static	short		anpvhilimscn [TILINES];		/* Analog Alarm PV High Limit */
static	short		anpvlolimscn [TILINES];		/* Analog Alarm PV Low Limit */
static	short		anorgdevlimscn [TILINES];	/* Analog Alarm Orange Deviation Limit */
static	short		anyeldevlimscn [TILINES];	/* Analog Alarm Yelow Deviation Limit */
static	short		ansmpratescn [TILINES];		/* Analog Alarm Sample Rate */
static	short		ansetpointscn [TILINES];	/* Analog Alarm Set Point */
static	short		anerrorscn [TILINES];		/* Analog Alarm Error (Input Only) */
static	short		anpvhihialrmscn [TILINES];	/* Analog Alarm PV High High Alarm */
static	short		anpvloloalrmscn [TILINES];	/* Analog Alarm PV Low Low Alarm */
static	short		anrocscn [TILINES];		/* Analog Alarm Rate of Change */

static	unsigned long	lpgainlst [TILINES];		/* Loop Gain */
static	unsigned long	lpresetlst [TILINES];		/* Loop Reset */
static	unsigned long	lpratelst [TILINES];		/* Loop Rate */
static	unsigned long	lphialrmlst [TILINES];		/* Loop High Alarm */
static	unsigned long	lploalrmlst [TILINES];		/* Loop Low Alarm */
static	unsigned long	lpprocvarlst [TILINES];		/* Loop Process Variable */
static	unsigned long	lppvhilimlst [TILINES];		/* Loop PV High Limit */
static	unsigned long	lppvlolimlst [TILINES];		/* Loop PV Low Limit */
static	unsigned long	lporgdevalrmlst [TILINES];	/* Loop Orange Dev Alarm */
static	unsigned long	lpyeldevalrmlst [TILINES];	/* Loop Yellow Dev Alarm */
static	unsigned long	lpsmpratelst [TILINES];		/* Loop Sample Rate */
static	unsigned long	lpremsplst [TILINES];		/* Loop Remote Set Point */
static	unsigned long	lpoutputlst [TILINES];		/* Loop Output */
static	unsigned long	lperrorlst [TILINES];		/* Loop Error (Input Only) */
static	unsigned long	lpbiaslst [TILINES];		/* Loop Bias */
static	unsigned long	lppvhihilimlst [TILINES];	/* Loop PV High High Limit */
static	unsigned long	lppvlololimlst [TILINES];	/* Loop PV Low Low Limit */
static	unsigned long	lprocalrmlst [TILINES];		/* Loop Rate of Change Alarm */
static	unsigned long	anpvhialrmlst [TILINES];	/* Analog Alarm PV High Alarm */
static	unsigned long	anpvloalrmlst [TILINES];	/* Analog Alarm PV Low Alarm */
static	unsigned long	anprocvarlst [TILINES];		/* Analog Alarm Process Variable */
static	unsigned long	anpvhilimlst [TILINES];		/* Analog Alarm PV High Limit */
static	unsigned long	anpvlolimlst [TILINES];		/* Analog Alarm PV Low Limit */
static	unsigned long	anorgdevlimlst [TILINES];	/* Analog Alarm Orange Deviation Limit */
static	unsigned long	anyeldevlimlst [TILINES];	/* Analog Alarm Yelow Deviation Limit */
static	unsigned long	ansmpratelst [TILINES];		/* Analog Alarm Sample Rate */
static	unsigned long	ansetpointlst [TILINES];	/* Analog Alarm Set Point */
static	unsigned long	anerrorlst [TILINES];		/* Analog Alarm Error (Input Only) */
static	unsigned long	anpvhihialrmlst [TILINES];	/* Analog Alarm PV High High Alarm */
static	unsigned long	anpvloloalrmlst [TILINES];	/* Analog Alarm PV Low Low Alarm */
static	unsigned long	anroclst [TILINES];		/* Analog Alarm Rate of Change */

static	TS_STAMP	tmnow [TILINES];

static	UCHAR		linemon [TILINES];

static	char    	*bools [] = {
				"TRUE ",
				"FALSE"};

static	int		taskId [TILINES];

static	char		*tasknames [] =

			{"TIWAY Ch0", "TIWAY Ch1", "TIWAY Ch2", "TIWAY Ch3",
			 "TIWAY Ch4", "TIWAY Ch5", "TIWAY Ch6", "TIWAY Ch7",
			 "TIWAY Ch8", "TIWAY Ch9", "TIWAY Ch10", "TIWAY Ch11",
			 "TIWAY Ch12", "TIWAY Ch13", "TIWAY Ch14", "TIWAY Ch15",
			 "TIWAY Ch16", "TIWAY Ch17", "TIWAY Ch18", "TIWAY Ch19"};

static	UCHAR		report_adapter_config [] = { 0x21 };
static	UCHAR		soft_reset []            = { 0xFE };
static	UCHAR		config_adapter []        = { 0x20, 1,0,4, 3,0,100, 4,0,2 };
static	UCHAR		config_HIU []            = { 0x11, 3,0,0 };
static	UCHAR		config_NM []             = { 0x30, /*1,0x2B,0x2B,*/ 0x08,0,0,
							0x09,0,15, 0x0A,0,50,
							0x0B,0,50, 0x0E,0,5 };
static	UCHAR		connect_secondary []     = { 0x04, 0 };

static	UCHAR		*ti_dev_type []		=
	{
	"PLC 5TI",
	"PLC 510",
	"PLC 520",
	"PLC 520C",
	"PLC 525",
	"PLC 530",
	"PLC 530C",
	"PLC 545",
	"PLC 550",
	"PLC 560",
	"PLC 565",
	"PLC 570",
	"Unilink Host Adaptor",
	"PM550",
	"PM551",
	"Unilink Adaptor",
	"IT-111/121 Tank Transmitter",
	"IT-150 Hydrostatic Tank Transmitter",
	"IT-160 Micro Remote Control Unit",
	"Unknown Device"
	};

static UCHAR		*prim_err_code []	=
	{
	"Primitive not implemented",
	"Data type not defined in device",
	"Data element location out of range",
	"Primitive has excess data unit bytes",
	"Primitive has insufficient data unit bytes",
	"Primitive length field does not match actual length",
	"Device in wrong mode for primitive execution",
	"User program has disabled communication to this device",
	"Written data type location did not verify",
	"Data type location is write protected",
	"Device fails to respond",
	"Primitive aborted due to fatal error in device",
	"Data type has invalid value due to primitive execution",
	"Error encountered while executing primitive",
	"Primitive not valid for data type",
	"Data pattern requested not found",
	"Number of locations requested exceeds maximum",
	"Number assigned to data acqusition block invalid",
	"Block or record number not defined",
	"Data bytes requested exceeds maximum",
	"Request number report by condition exceeds maximum",
	"Primitive not allowed when device is in local mode",
	"Data type not programmed in device",
	"Device did not respond properly",
	"Data type not implemented in NIM, but is in the device",
	"Data element plus number requested exceeds range",
	"Device communications not established",
	"Store and forward buffer full, message discarded",
	"Data element field improperly formatted",
	"Number of locations to access is zero",
	"Write conflict detected with another device",
	"Unexpected primitive received while temp storage reserved",
	"Incorrect command for primitive 33",
	"Non-contiguous data location request while temp storage reserved",
	"Command modifier not supported by device",
	"Number of data blocks defined exceeds maximum",
	"Illegal IEEE floating point value in write request",
	"Address value exceeds structure format in request",
	"Host identifier invalid",
	"Communication buffer overflow - information may be lost",
	"Broadcast transactions are not supported",
	"Store and forward no supported with this primitive",
	"Address value exceeds structure format in request",
	"Write request refused - device is in download or upload mode",
	"Upload or download command field no supported",
	"Upload or download request refused",
	"Upload request segment not supported or not defined",
	"Data count value within value does not match actual count",
	"Unidentified exception generated in device"
	};

/**********
*
*	This routine is required by iocinit in the EPICS system.
*
**********/

static long init ()
{
	return (0);
}
/********
*
*	This routine enables logging of comm trace messages
*
********/

void TRACEON (unsigned short line)
{
	modtrace [line] = TRUE;
}

/********
*
*	This routine disables logging of comm trace messages
*
********/

void TRACEOFF (unsigned short line)
{
	modtrace [line] = FALSE;
}

/********
*
*	This routine enables logging of resource trace messages
*
********/

void lnmonon (unsigned short line)
{
	linemon [line] = TRUE;
}

/********
*
*	This routine disables logging of comm trace messages
*
********/

void lnmonoff (unsigned short line)
{
	linemon [line] = FALSE;
}

/********
*
*	This routine enables logging of resource trace messages
*
********/

void RTRACEON (unsigned short line)
{
	modrestrace [line] = TRUE;
}

/********
*
*	This routine disables logging of resource trace messages
*
********/

void RTRACEOFF (unsigned short line)
{
	modrestrace [line] = FALSE;
}

/********
*
*	This routine controls logging of trace messages
*
********/

void TRACELEVEL (unsigned short line, unsigned short level)
{
	mltrace [line] = level;
}

#ifndef TEST
int STOPDRIVER (short ln)
{
	if (comm_disconnect (ln) != -1)
	{
	    taskDelete (taskId [ln]);
	    return (1);
	}
	else
	{
	    return (-1);
	}
}
int STARTDRIVER1 (unsigned short ln, char *mode, unsigned short delay,
			unsigned short adapter_addr)
{
	if ( ln < TILINES )
	{
	    taskId [ln] = taskSpawn (tasknames [ln],
				     TI_PRIORITY,
				     TI_TASK_OPTIONS,
				     TI_STACK, MAIN,
				     ln,
                                     mode,
                                     delay);

	    logMsg ("Started the TIWAY driver on line %d\n", ln);

	    remCurIdGet (user, password);
	    return (1);
	}
	else
	{
	    logMsg ("Invalid line number %d\n", ln);
	    return (-1);
	}
}

#endif

/********
*
* Set communication logic time delays to handle half/full duplex,
* radio delays, etc 
*
********/

void SETDELAYS (unsigned short line,
		unsigned short tickcount1,
		unsigned short tickcount2,
		unsigned short tickcount3)
{
	intermsgdelay [line] = tickcount1;  /* at beginning new message/response sequence.               */
	pretxdelay [line]    = tickcount2;  /* after asserting RTS but before beginning transmission.    */
	posttxdelay [line]   = tickcount3;  /* after sending last character, but before dropping RTS     */
}

/**********
*
*      tiway_driver
*
*      The purpose of this task is to scan the TI plc
*      lines. It will maintain a configuration list supporting online additions
*      and repplacement of modules, and  maintain a table of most recently
*      scanned values.
*
**********/
void MAIN (unsigned short line, unsigned char *mode, unsigned short delay)
{
	unsigned short  mod, cfgchk, cnt, modspolled, onlinefreq;
	unsigned short  activity, pstate, lstate, execute, command,i;
	long	        this_time, last_time;
	TIPLC           *pmod;
	RXLINE          *rxp;

	/* Initialize the data structures */

	intermsgdelay [line] = delay;
	pretxdelay [line] = 0;
	posttxdelay [line] = 0;

	resresponse [line] = FALSE;
	resnames [line] [0] = 0;
	prestable [line] = NULL;

	modtrace [line] = FALSE;
	mltrace [line] = 0;
	modrestrace [line] = TRUE;

	activity_time [line] = tickGet ();
	lastactivity [line] = line_activity [line] = FALSE;

/* wep: output queue initialization */

	oq [line] = (OUTPUTQUE *) calloc (1, (sizeof (OUTPUTQUE)));
	if (!oq [line])
	{
	    logMsg ("Not enough memory for line %d data structure (data output queue)\n", line);
	    return;
	}

/* wep: communication initialization */

	rxlinetab [line] = (RXLINE *) calloc (1, (sizeof (RXLINE)));
	if (!rxlinetab [line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: rx)\n",line);
	    return;
	}
	txlinetab [line] = (TXLINE *) calloc (1, (sizeof (TXLINE)));
	if (!txlinetab [line])
	{
	    logMsg ("Not enough memory for line %d data structure (comm: tx)\n", line);
	    return;
	}
        taskLock ();  /* make terminal display easier to read - avoids interleaving of other task data */
	comm_init (line, mode, &tx_int_proc, &rx_int_proc);
	clear_rts (line);
        taskUnlock ();
	onlinefreq = 1;		/* causes on_line calls to begin on first pass if master */

	/* Initialize the Rx interrupts to receive once for activity check */
	/* enable the receiver interrupt after set to ignore */

	rxp = rxlinetab [line];
	rxp->rxstate = R_IGNORE;
	enable_rx_ints (line);

#ifdef  TEST
	scansome (0);
#endif
	/* Wait for resource to be defined via an initial call to 'register_resource'.*/
	while ((resnames [line] [0] == 0) || (prestable [line] == NULL))
	{
	    if (mltrace [line] >= 1)
	    {
	    	logMsg ( "No Resource Defined for line - %d \n", line);
	    }
	    taskDelay (120);
	}

	/* Start out as slave */

	resClearCaAnswer (prestable [line]);		/* start quiet */
	resPutLocalState (prestable [line], RES_SLV);
	if (modrestrace [line])
	{
		logMsg ("1] Became RES_SLV\n");
	}
	taskDelay (300);
	resGetStates (prestable [line], &lstate, &pstate);
	activity = check_line_activity (line, ACTIVITYTIME);

	/* if the partner is not there, be master */

	if ((pstate != RES_MSTR) && (pstate != RES_SLVWANT) && (!activity))
	{
	    resSetCaAnswer (prestable [line]);
	    resPutLocalState (prestable [line], RES_MSTR);
	    lstate = RES_MSTR;
	    set_rts (line);
	    if (modrestrace [line])
	    {
		logMsg ("2] Became RES_MSTR\n");
	    }
	}

	/* Task will process all modules sending outputs and getting inputs */
	/* It will then try to configure requested non responding modules   */

	cfgchk = 0;     	/* Init the new config pointer */
	last_time = tickGet (); /* Initialize the time  */

	while (TRUE)
	{
	    activity_time [line] = tickGet ();
	    lastactivity [line] = line_activity [line] = FALSE;

	    /* we arrive here because we are a slave. 							*/
	    /* we stay here until:    asked to be master by partner. if we reach PLCs, become master.   */
	    /*                        the partner is not master and there is no activity on the line. 	*/

	    while (lstate == RES_SLV)
	    {
		if (pstate != RES_SLVWANT)
		{
		    clear_rts (line); 				/* make sure line is left for master 	*/
		}

		resresponse [line] = FALSE; 			/* make sure driver is not responding 	*/
		resPutCommand (prestable [line], 0); 		/* clear out commands 			*/
		activity = check_line_activity (line, 10); 	/* wait 10 seconds before checking 	*/
		resGetStates (prestable [line], &lstate, &pstate);


		/* see if needed by partner to be master. if yes, do it and break out of slave state */

		if (pstate == RES_SLVWANTCA)
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;
		    set_rts (line);
		    if (modrestrace [line])
		    {
			logMsg ("3] Became RES_MSTR  P= %x A= %d \n", pstate, activity);
		    }
		    break;
		}

		/* If partner is there and wants to give it up, try to take it */

		if (pstate == RES_SLVWANT)
		{
		    execute = FALSE;

		    taskDelay (2); /* Time for switch to occur */

		    /* See if anyone is out there and able to respond to a loopback command */

		    for (mod = 1; (mod <= NUMTIPLCS); mod++)
		    {
			pmod = plcs [line] [mod];
			if (pmod)    /* verify pointer validity prior to use */
			{
			    if (pmod->was_requested)
			    {
			        on_line (line, mod);	
			        if (pmod->is_responding)
			        {   
				    execute = TRUE;
				    break;
			        }
			    }
			}
		    }
		    /* If someone answered, become master and make sure partner really gave it up */

		    if (execute)
		    {
			resPutLocalState (prestable [line], RES_MSTR);
			lstate = RES_MSTR;
	    		set_rts (line);
			if (modrestrace [line])
			{
			    logMsg ("4] Became RES_MSTR  P= %x A= %d \n",
					pstate, activity);
			}
			execute = FALSE;
			activity_time [line] = tickGet ();
			lastactivity [line] = line_activity [line] = FALSE;
			for (i = 0; (i < 60); i++)
			{
			    activity = check_line_activity (line, 2);    /* pause before testing */
			    resGetStates (prestable [line], &lstate, &pstate);
			    if ((pstate != RES_MSTR) &&
				(pstate != RES_SLVWANT) && (!activity))
			    {
				execute = TRUE; /* Partner Gave It Up so take CA */
				break;
			    }
			}
			if (!execute)
			{   /* Partner did not give it up go back to slave */

			    resPutLocalState (prestable [line], RES_SLV);
			    lstate = RES_SLV;
			    if (modrestrace [line])
			    {
			    	logMsg ("5] Became RES_SLV  P= %x A= %d \n",
					pstate, activity);
			    }
			}
			else
			{ /* Assume control by taking the CA responsibilities */ 
			    resSetCaAnswer (prestable [line]);
			    prepresponses (line);
			    break;
			}
		    }
		}

		/* if the partner is not talking, be master */

		if ((pstate != RES_MSTR) &&
			(pstate != RES_SLVWANT) && (!activity))
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;
		    set_rts (line);
		    if (modrestrace [line])
		    {
		    	logMsg ("6] Became RES_MSTR P= %x A= %d \n", 
				pstate, activity);
		    }
		    prepresponses (line);
		}
	    }

	    /* to arrive here, we used to be master, and had been told to relinquish master status. */
	    /* stay in this loop until the partner takes over, or the command is canceled */

	    while (lstate == RES_SLVWANTCA)
	    {
	        taskDelay(2);                   /* Give OS a chance...                  */
		clear_rts (line); 		/* Make sure line is left for master */
		resresponse [line] = FALSE; 	/* Make Sure driver is not responding */
 		resGetStates (prestable [line], &lstate, &pstate);
		if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT))
		{
		    resClearCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_SLV);
		    lstate = RES_SLV;
		    if (modrestrace [line])
		    {
		    	logMsg ("7] Became RES_SLV  P= %x A= %d \n",
				pstate, activity);
		    }
		    break;
		}
		resGetCommand (prestable [line], &command);
		resPutCommand (prestable [line], 0); /* clear out commands */
		if (command == RES_CANCELREL)
		{
		    resSetCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_MSTR);
		    lstate = RES_MSTR;
	    	    set_rts (line);
		    if (modrestrace [line])
		    {
		    	logMsg ("8] Became RES_MSTR  P= %x A= %d \n",
				pstate, activity);
		    }
		    prepresponses (line);
		}
 	    }

            /****************************************************************************************/
	    /* in normal operation this code should never execute. it has, however, been seen to    */
            /* execute when the database has been inadvertently configured with the same resource   */
	    /* assigned to two different drivers in the same IOC. The state does funny things.      */
            /****************************************************************************************/
	    while (lstate == RES_SLVWANT)
	    {
                if (modrestrace [line])
		{
                    /* condition should NEVER occur. print error message every 5 seconds. */
                    logMsg ("Illegal Local Resource State.\n");
                    taskDelay (600);
                }
                taskDelay (2);
            }

             /* we come here because we have gone to, or are looping in, the master state.           */
	    /* we stay in this loop until:    we have been told to give master state to the partner.*/
	    /*                                we break out to RES_SLVWANTCA.                        */
	    /*                                we have successfully handed master state to partner.  */
	    /* While master, this task will process all modules sending outputs and getting inputs. */
	    /* It will then try to configure requested non-responding modules.                      */

	    /* it doesn't appear that we ever come here in a RES_SLVWANT lstate mode. it happens    */
	    /*  within this loop, but is never seen by the while loop conditional evaluation.       */

	    last_time = tickGet ();		/* initialize gap */
	    while ((lstate == RES_MSTR) || (lstate == RES_SLVWANT))
	    {
		/* initial stuff */

		resresponse [line] = TRUE; /* Make sure driver is responding. */

		resGetCommand (prestable [line], &command);
		resPutCommand (prestable [line], 0);    /* clear out commands */

		/* if relinquish command sent, change state and break out to outer loop */
		if (command == RES_RELINQUISH)
		{
		    resClearCaAnswer (prestable [line]);
		    resPutLocalState (prestable [line], RES_SLVWANTCA);
		    lstate = RES_SLVWANTCA;
		    if (modrestrace [line])
		    {
                       logMsg ("Became RES_SLVWANTCA  P= %x A= %d \n",
				pstate, activity);
		    }
		    break;
		}

		modspolled = 0;

		/* call 'scan_plc' for each PLC device, unless user has */
                /* taken it off line or it is not responding.           */

		for (mod = 1; (mod <= NUMTIPLCS); mod++)
		{
		    pmod = plcs [line] [mod];		/* shorthand pointer 			*/
		    if (pmod)				/* verify pointer validity prior to use */
		    {
			if (pmod->cmd_force_offline)
			{ 
			    /* if user has turned this plc offline via pseudo commands,     */
                            /* mark it as not responding.                                   */
 			    pmod->is_responding = FALSE;
	    		}
			else if (pmod->is_responding)
			{
			    modspolled++;

			     /* local/partner lockstep recovery */

			    resGetStates (prestable [line], &lstate, &pstate);
			    if (pstate == RES_MSTR)
			    {
				resClearCaAnswer (prestable [line]);
				resPutLocalState (prestable [line], RES_SLV);
				lstate = RES_SLV;
				clear_rts(line); 		/* Make sure line is left for master */
				if (modrestrace [line])
				{
				    logMsg ("13] Became RES_SLV  P= %x A= %d \n", pstate, activity);
				}
				taskDelay (((tickGet () % 8) + 5) * 60); /* random 5 to 12 second delay */
				break;
			    }

			    scan_plc (pmod, line, mod);

			    if (!pmod->is_responding)
			    {
				mark_not_valid (pmod);
                            }
			}
		    }
		    /* end of single module action */

		    /* if told to relinquish our master state, stop plc scanning immediately. */
		    /* don't trash the command, so that we can still read it on the next loop */
                    /* through this while statement.                                          */

		    resGetCommand (prestable [line], &command);
		    if (command == RES_RELINQUISH)
		    {
			break;
		    }
		}					/* end of scan loop */
                /* test for lockstep detection. */
                if (lstate != RES_MSTR) break;


		/* try and bring all nonresponding plcs on line, but */
                /* don't bring in more than a specified number.      */

	        cnt = 0;  /* count the number to bring on in one pass */

	        if (--onlinefreq == 0)
		{
		    onlinefreq = BRINGBACKCNT;

		    for ( mod = 1; (mod < NUMTIPLCS); mod++)
		    {
		       cfgchk++;
		        if (cfgchk == NUMTIPLCS)
			{
			    cfgchk = 1;
			}
		        pmod = plcs [line] [cfgchk];
			if (pmod)
			{
			    if ((!pmod->is_responding) &&
				(pmod->was_requested) &&
				(!pmod->cmd_force_offline))
			    {
				on_line (line, cfgchk);
				modspolled++;
				cnt++;
				if (cnt == MAXCFGCK)
				{
				    break;
				}
			    }
			}
		    }
	        }


		/* see if anyone on the line has been responding. */
                /* if not, try to get partner to become slave.    */
		/* Check for no one responding */

		if (!any_responding (line))
		{
		    /* See if it is time to let partner try */

		    this_time = tickGet ();
		    if ((this_time - last_time) > (3 * SLWWINDOW))
		    {   /* # ticks for slave to try */

			/* Switch States to let partner know it's its time to try */

			resPutLocalState (prestable [line], RES_SLVWANT);
			lstate = RES_SLVWANT;
			if (modrestrace [line])
			{
			    logMsg ("10] Became RES_SLVWANT  P= %x A= %d \n",
				pstate,activity);
			}

			/* Give partner access to the hardware */

			clear_rts (line);

			/* Remember the time of this try */

			last_time = tickGet ();
			lastactivity [line] = line_activity [line] = FALSE;
			for (i = 0; (i < SLWWINDOW); i += 10)
			{       		/* SLWINDOW is the # of ticks for slave to try */
			    taskDelay (10);
		    				/* If partner picked up let it have control */
			    activity = check_line_activity (line, 0);  /* check immediately */
			    resGetStates (prestable [line], &lstate, &pstate);
 			    if ((pstate == RES_MSTR) || (pstate == RES_SLVWANT) || (activity))
			    {
				resClearCaAnswer (prestable [line]);
				resPutLocalState (prestable [line], RES_SLV);
				lstate = RES_SLV;
				if (modrestrace [line])
				{
				    logMsg ("11] Became RES_SLV  P= %x A= %d \n",
					 pstate, activity);
				}
				taskDelay (240);
				break;
			    }
			}

			/* Go Back to being the Master and take control of the hardware */

			if (lstate != RES_SLV)
			{
                            resSetCaAnswer (prestable [line]);
                            resPutLocalState (prestable [line], RES_MSTR);
			    lstate = RES_MSTR;
	    		    set_rts (line);
			    if (modrestrace [line])
			    {
			    	logMsg ("12] Became RES_MSTR  P= %x A= %d \n",
					pstate,activity);
			    }
			}
		    }
		}

		/* take plcs off hold status. the plc's data are not to be held off since now master. */

		for (mod = 1; (mod <= NUMTIPLCS); mod++ )
		{
		    pmod = plcs [line] [mod];
		    if (pmod)			/* verify pointer validity prior to use */
		    {        
			pmod->holdflag = FALSE;
                    }
		}
	    }

#ifndef TEST
	    if (modspolled == 0)
	    {
		taskDelay (60);
	    }
	    else 
	    {
		taskDelay (10);
	    }
#endif
#ifdef  TEST
	    scansome();
#endif

	} /* end task loop */
}

/*********
*
* Try to bring a PLC on line
*
**********/

static	void    on_line (unsigned short line, unsigned short mod)
{
	TIPLC			*pmod;
	unsigned	short	i, j, offlines;

	pmod = plcs [line] [mod];

	if (pmod)
	{
	    pmod->nodatastrike = 0;
	    offlines = pmod->no_offlines;	/* avoid extra counts */
	    if (! uha_state [line])
	    {
		if (setupUHA (line, pmod) == -1)
		{
		    return;
		}
		else
		{
		    uha_state [line] = TRUE;
		}
	    }
	    if (connect_sec (line, (UCHAR) mod, pmod) == -1)
	    {
		return;
	    }
	    mark_not_valid (pmod);	/* PLC is online - Init data structures */

	    pmod->scanpass = 0;
	    pmod->is_responding = TRUE;
	    initscan (line, pmod);
	}
}

/**********
*
*	setupUHA (int line, TIPLC *pmod)
*
*	This routine performs a soft reset and initializes the UHA
*
**********/
static int setupUHA (int line, TIPLC *pmod)
{
	TXLINE			*txp;
	RXLINE			*rxp;
	unsigned	short	length;
	unsigned	short	ecc, i;

	txp = txlinetab [line];
	rxp = rxlinetab [line];

	length = (10 + (sizeof (report_adapter_config) * 2));
	txp->txbuf [0] = ':';
	nitp_bin2ASCII ((UCHAR *) &length, sizeof (length), &txp->txbuf [1]);
	nitp_bin2ASCII (report_adapter_config, sizeof (report_adapter_config),
			&txp->txbuf [strlen (txp->txbuf)]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer (line, pmod) != XFERGOOD)
	{
	    return (-1);
	}

	length = (10 + (sizeof (soft_reset) * 2));
	txp->txbuf [0] = ':';
	nitp_bin2ASCII ((UCHAR *) &length, sizeof (length), &txp->txbuf [1]);
	nitp_bin2ASCII (soft_reset, sizeof (soft_reset),
			&txp->txbuf [strlen (txp->txbuf)]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer (line, pmod) != XFERGOOD)
	{   
	    return (-1);
	}

	taskDelay (60);

	length = (10 + (sizeof (config_adapter) * 2));
	txp->txbuf [0] = ':';
	nitp_bin2ASCII ((UCHAR *) &length, sizeof (length), &txp->txbuf [1]);
	nitp_bin2ASCII (config_adapter, sizeof (config_adapter),
			&txp->txbuf [strlen (txp->txbuf)]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer (line, pmod) != XFERGOOD)
	{
	    return (-1);
	}

	length = (10 + (sizeof (config_HIU) * 2));
	txp->txbuf [0] = ':';
	nitp_bin2ASCII ((UCHAR *) &length, sizeof (length), &txp->txbuf [1]);
	nitp_bin2ASCII (config_HIU, sizeof (config_HIU),
			&txp->txbuf [strlen (txp->txbuf)]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer (line, pmod) != XFERGOOD)
	{
	    return (-1);
	}

	length = (10 + (sizeof (config_NM) * 2));
	txp->txbuf [0] = ':';
	nitp_bin2ASCII ((UCHAR *) &length, sizeof (length), &txp->txbuf [1]);
	nitp_bin2ASCII (config_NM, sizeof (config_NM),
			&txp->txbuf [strlen (txp->txbuf)]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *)&ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer (line, pmod) != XFERGOOD)
	{
	    return (-1);
	}
	logMsg ("UHA Initialized on line %d\n", line);
	return (0);
}

/**********
*
*	connect_secondary (int line, UCHAR addr, TIPLC *pmod)
*
*	This routine brings a PLC on line through the UHA
*
**********/
static int connect_sec (int line, UCHAR addr, TIPLC *pmod)
{
	TXLINE			*txp;
	RXLINE			*rxp;
        TI_DEV			*dev;
	unsigned	short	length;
	unsigned	short	ecc, err_code;
	unsigned	char	rbuf [100];

	txp = txlinetab [line];
	rxp = rxlinetab [line];

	connect_secondary [1] = addr;
	length = (10 + (sizeof (connect_secondary) * 2));
	txp->txbuf [0] = ':';
	nitp_bin2ASCII ((UCHAR *) &length, sizeof (length), &txp->txbuf [1]);
	nitp_bin2ASCII (connect_secondary, sizeof (connect_secondary),
			&txp->txbuf [strlen (txp->txbuf)]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer (line, pmod) != XFERGOOD)
	{
	    return (-1);
	}
	if (strncmp (&txp->txbuf [1], &rxp->rxbuf [0], strlen (rxp->rxbuf)) != 0)
	{
	    return (-1);
	}

	txp->txbuf [0] = ':';		/* NITP Delimiter */
	txp->txbuf [1] = '0';		/* NITP Msg Length */
	txp->txbuf [2] = '0';
	txp->txbuf [3] = '1';
	txp->txbuf [4] = '4';
	txp->txbuf [5] = '0';		/* Send Net Data Function Code */
	txp->txbuf [6] = '1';

	nitp_bin2ASCII ((UCHAR *) &addr, sizeof (addr), &txp->txbuf [7]);

	txp->txbuf [9] = '0';		/* Tiway Primitive Msg Length */
	txp->txbuf [10] = '0';
	txp->txbuf [11] = '0';
	txp->txbuf [12] = '1';
	txp->txbuf [13] = '0';		/* Tiway Primitive Reqquest */
	txp->txbuf [14] = '3';
        txp->txbuf [15] = 0;
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;
	if (msgxfer (line, pmod) != XFERGOOD)
	{
	    return (-1);
	}
	nitp_ASCII2bin (rxp->rxbuf, rbuf, strlen (rxp->rxbuf));
	dev = (TI_DEV *) rbuf;
	if (rbuf [6] != 0)
	{
	    pmod->dev_type = dev->dev_type;
	    switch (dev->dev_type)
	    {
            case PLC_5TI:
		pmod->dev_type_idx = 0;
		break;

	    case PLC_510:
		pmod->dev_type_idx = 1;
		break;

	    case PLC_520:
		pmod->dev_type_idx = 2;
		break;

	    case PLC_520C:
		pmod->dev_type_idx = 3;
		break;

	    case PLC_525:
		pmod->dev_type_idx = 4;
		break;

	    case PLC_530:
		pmod->dev_type_idx = 5;
		break;

	    case PLC_530C:
		pmod->dev_type_idx = 6;
		break;

	    case PLC_540:
		pmod->dev_type_idx = 7;
		break;

	    case PLC_550:
		pmod->dev_type_idx = 8;
		break;

	    case PLC_560:
		pmod->dev_type_idx = 9;
		break;

	    case PLC_565:
		pmod->dev_type_idx = 10;
		break;

	    case PLC_570:
		pmod->dev_type_idx = 11;
		break;

	    case UHA:
		pmod->dev_type_idx = 12;
		break;

	    case PM550:
		pmod->dev_type_idx = 13;
		break;

	    case PM551:
		pmod->dev_type_idx = 14;
		break;

	    case UA:
		pmod->dev_type_idx = 15;
		break;

	    case IT_111_121:
		pmod->dev_type_idx = 16;
		break;

	    case IT_150:
		pmod->dev_type_idx = 17;
		break;

	    case IT_160:
		pmod->dev_type_idx = 18;
		break;

	    default:
		pmod->dev_type_idx = 19;
		break;
	    }
	    pmod->vmem_sz = dev->vmem_sz;
	    pmod->cmem_sz = dev->cmem_sz;
	    pmod->local_io_sz = dev->local_io_sz;
	    pmod->global_io_sz = dev->global_io_sz;
	    pmod->total_user_sz = dev->total_user_sz;

	    logMsg ("Connection to %s, Addr: %d established\n",
	    		ti_dev_type [pmod->dev_type_idx], addr);
	    return (0);
	}
	else
	{
	    err_code = (rbuf [8] << 8);
	    err_code += rbuf [9];
	    logMsg ("READ Error - %d %s\n", rbuf [5], prim_err_code );
	    return (FALSE);
	    return (-1);
	}
}

/**********
*							
* Mark all plc data elements as not valid     		
*                                             		
**********/                                             	

/* called from on_line and prepresponses      		*/    	

static	void	mark_not_valid (TIPLC *pmod)
{
	unsigned	short	i;
/*
/*      INITIALIZE ALL VALUES AS
/*              - DATA NOT VALID
/*
*/

	for (i = 0; ((i < NUMTIVMEM) && (pmod->vmem)); i++)
	{
	    pmod->vmem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTICMEM) && (pmod->cmem)); i++)
	{
	    pmod->cmem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIXMEM) && (pmod->xmem)); i++)
	{
	    pmod->xmem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTITXMEM) && (pmod->txmem)); i++)
	{
	    pmod->txmem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIYMEM) && (pmod->ymem)); i++)
	{
	    pmod->ymem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTITYMEM) && (pmod->tymem)); i++)
	{
	    pmod->tymem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTICRMEM) && (pmod->crmem)); i++)
	{
	    pmod->crmem [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIWXIN) && (pmod->wxin)); i++)
	{
	    pmod->wxin [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIWYOUT) && (pmod->wyout)); i++)
	{
	    pmod->wyout [i].data_valid = FALSE;
	}
	for (i=0; ((i < NUMTISTATUS) && (pmod->status)); i++)
	{
	    pmod->status [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPSTAT) && (pmod->lpstat)); i++)
	{
	    pmod->lpstat [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPCNTL) && (pmod->lpcntl)); i++)
	{
	    pmod->lpcntl [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPMD) && (pmod->lpmd)); i++)
	{
	    pmod->lpmd [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIAVFLAG) && (pmod->avflag)); i++)
	{
	    pmod->avflag [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIACFLAG) && (pmod->acflag)); i++)
	{
	    pmod->acflag [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPGAIN) && (pmod->lpgain)); i++)
	{
	    pmod->lpgain [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPRESET) && (pmod->lpreset)); i++)
	{
	    pmod->lpreset [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPRATE) && (pmod->lprate)); i++)
	{
	    pmod->lprate [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPHIALRM) && (pmod->lphialrm)); i++)
	{
	    pmod->lphialrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPLOALRM) && (pmod->lploalrm)); i++)
	{
	    pmod->lploalrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPPROCVAR) && (pmod->lpprocvar)); i++)
	{
	    pmod->lpprocvar [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPPVHILIM) && (pmod->lppvhilim)); i++)
	{
	    pmod->lppvhilim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPPVLOLIM) && (pmod->lppvlolim)); i++)
	{
	    pmod->lppvlolim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPORGDEVALRM) && (pmod->lporgdevalrm)); i++)
	{
	    pmod->lporgdevalrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPYELDEVALRM) && (pmod->lpyeldevalrm)); i++)
	{
	    pmod->lpyeldevalrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPSMPRATE) && (pmod->lpsmprate)); i++)
	{
	    pmod->lpsmprate [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPREMSP) && (pmod->lpremsp)); i++)
	{
	    pmod->lpremsp [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPOUTPUT) && (pmod->lpoutput)); i++)
	{
	    pmod->lpoutput [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPERROR) && (pmod->lperror)); i++)
	{
	    pmod->lperror [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPBIAS) && (pmod->lpbias)); i++)
	{
	    pmod->lpbias [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPPVHIHILIM) && (pmod->lppvhihilim)); i++)
	{
	    pmod->lppvhihilim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPPVLOLOLIM) && (pmod->lppvlololim)); i++)
	{
	    pmod->lppvlololim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTILPROCALRM) && (pmod->lprocalrm)); i++)
	{
	    pmod->lprocalrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPVHIALRM) && (pmod->anpvhialrm)); i++)
	{
	    pmod->anpvhialrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPVLOALRM) && (pmod->anpvloalrm)); i++)
	{
	    pmod->anpvloalrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPROCVAR) && (pmod->anprocvar)); i++)
	{
	    pmod->anprocvar [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPVHILIM) && (pmod->anpvhilim)); i++)
	{
	    pmod->anpvhilim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPVLOLIM) && (pmod->anpvlolim)); i++)
	{
	    pmod->anpvlolim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANORGDEVLIM) && (pmod->anorgdevlim)); i++)
	{
	    pmod->anorgdevlim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANYELDEVLIM) && (pmod->anyeldevlim)); i++)
	{
	    pmod->anyeldevlim [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANSMPRATE) && (pmod->ansmprate)); i++)
	{
	    pmod->ansmprate [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANSETPOINT) && (pmod->ansetpoint)); i++)
	{
	    pmod->ansetpoint [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANERROR) && (pmod->anerror)); i++)
	{
	    pmod->anerror [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPVHIHIALRM) && (pmod->anpvhihialrm)); i++)
	{
	    pmod->anpvhihialrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANPVLOLOALRM) && (pmod->anpvloloalrm)); i++)
	{
	    pmod->anpvloloalrm [i].data_valid = FALSE;
	}
	for (i = 0; ((i < NUMTIANROC) && (pmod->anroc)); i++)
	{
	    pmod->anroc [i].data_valid = FALSE;
	}
}

/**********
*
* Scan a Seimens PLC -  get inputs and set outputs as required
*
**********/
static	void    scan_plc (TIPLC *pmod, unsigned short line, unsigned short mod)
{
	unsigned	short	i, j, val, maxints, numgroups;
	unsigned	long	bitmask;
	unsigned	char	mem_allocated = 0;

	pmod->scanpass++;
	checkoutputs (line);
	prepscan (line, pmod);

	/* read the VMEM values to be scanned */

	for (i = 0; ((i < NUMTIVMEM) && (pmod->vmem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->vmem [i].was_requested)
	    {
		for (j = 1; ((j < MAXVMEM) && ((i + j) < NUMTIVMEM)); j++)
		{
		    if (!pmod->vmem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtivmem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the CMEM values to be scanned */

	for (i = 0; ((i < NUMTICMEM) && (pmod->cmem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->cmem [i].was_requested)
	    {
		for (j = 1; ((j < MAXCMEM) && ((i + j) < NUMTICMEM)); j++)
		{
		    if (!pmod->cmem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readticmem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the XMEM values to be scanned */

	for (i = 0; ((i < NUMTIXMEM) && (pmod->xmem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->xmem [i].was_requested)
	    {
		for (j = 1; ((j < MAXXMEM) && ((i + j) < NUMTIXMEM )); j++)
		{
		    if (!pmod->xmem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtixmem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the TXMEM values to be scanned */

	for (i = 0; ((i < NUMTITXMEM) && (pmod->txmem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->txmem [i].was_requested)
	    {
		for (j = 1; ((j < MAXTXMEM) && ((i + j) < NUMTITXMEM )); j++)
		{
		    if (!pmod->txmem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtixmem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the YMEM values to be scanned */

	for (i = 0; ((i < NUMTIYMEM) && (pmod->ymem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->ymem [i].was_requested)
	    {
		for (j = 1; ((j < MAXYMEM) && ((i + j) < NUMTIYMEM)); j++)
		{
		    if (!pmod->ymem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtiymem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the TYMEM values to be scanned */

	for (i = 0; ((i < NUMTITYMEM) && (pmod->tymem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->tymem [i].was_requested)
	    {
		for (j = 1; ((j < MAXTYMEM) && ((i + j) < NUMTITYMEM)); j++)
		{
		    if (!pmod->tymem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtiymem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the CRMEM values to be scanned */

	for (i = 0; ((i < NUMTICRMEM) && (pmod->crmem)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->crmem [i].was_requested)
	    {
		for (j = 1; ((j < MAXCRMEM) && ((i + j) < NUMTICRMEM)); j++)
		{
		    if (!pmod->crmem [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readticrmem (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the WXIN values to be scanned */

	for (i = 0; ((i < NUMTIWXIN) && (pmod->wxin)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->wxin [i].was_requested)
	    {
		for (j = 1; ((j < MAXWXIN) && ( (i + j) < NUMTIWXIN)); j++)
		{
		    if (!pmod->wxin [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtiwxin (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the WYOUT values to be scanned */

	for (i = 0; ((i < NUMTIWYOUT) && (pmod->wyout)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->wyout [i].was_requested)
	    {
		for (j = 1; ((j < MAXWYOUT) && ((i + j) < NUMTIWYOUT)); j++)
		{
		    if (!pmod->wyout [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtiwyout (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the STATUS values to be scanned */

	for (i = 0; ((i < NUMTISTATUS) && (pmod->status)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->status [i].was_requested)
	    {
		for (j = 1; ((j < MAXSTATUS) && ((i + j) < NUMTISTATUS)); j++)
		{
		    if (!pmod->status [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtistatus (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the LPSTAT values to be scanned */

	for (i = 0; ((i < NUMTILPSTAT) && (pmod->lpstat)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->lpstat [i].was_requested)
	    {
		for (j = 1; ((j < MAXLPSTAT) && ((i + j) < NUMTILPSTAT)); j++)
		{
		    if (!pmod->lpstat [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtilpstat (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the LPCNTL values to be scanned */

	for (i = 0; ((i < NUMTILPCNTL) && (pmod->lpcntl)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->lpcntl [i].was_requested)
	    {
		for (j = 1; ((j < MAXLPCNTL) && ((i + j) < NUMTILPCNTL)); j++)
		{
		    if (!pmod->lpcntl [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtilpcntl (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the LPMD values to be scanned */

	for (i = 0; ((i < NUMTILPMD) && (pmod->lpmd)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->lpmd [i].was_requested)
	    {
		for (j = 1; ((j < MAXLPMD) && ((i + j) < NUMTILPMD)); j++)
		{
		    if (!pmod->lpmd [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtilpmd (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the AVFLAG values to be scanned */

	for (i = 0; ((i < NUMTIAVFLAG) && (pmod->avflag)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->avflag [i].was_requested)
	    {
		for (j = 1; ((j < MAXAVFLAG) && ((i + j) < NUMTIAVFLAG)); j++)
		{
		    if (!pmod->avflag [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtiavflag (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the ACFLAG values to be scanned */

	for (i = 0; ((i < NUMTIACFLAG) && (pmod->acflag)); i++)
	{
	    mem_allocated = 1;
	    if (pmod->acflag [i].was_requested)
	    {
		for (j = 1; ((j < MAXACFLAG) && ((i + j) < NUMTIACFLAG)); j++)
		{
		    if (!pmod->acflag [i + j].was_requested)
		    {
			break;
		    }
		}
		numgroups = j;
		if (!readtiacflag (line, mod, pmod, i, numgroups))
		{
		    return;
		}
		checkoutputs (line);
		i = (i + j) - 1;
	    }
	}

	/* read the LPGAIN values to be scanned */

	if (pmod->scanflag && SCNLPGAIN)
	{
	    for (i = 0; ((i < NUMTILPGAIN) && (pmod->lpgain)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpgain [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPGAIN) && ((i + j) < NUMTILPGAIN)); j++)
		    {
			if (!pmod->lpgain [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpgain (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	pmod->scanflag &= ~SCNLPGAIN;
	}

	/* read the LPRESET values to be scanned */

	if (pmod->scanflag && SCNLPRESET)
	{
	    for (i = 0; ((i < NUMTILPRESET) && (pmod->lpreset)); i++)
	    {
		if (pmod->lpreset [i].was_requested)
		{
		mem_allocated = 1;
		    for (j = 1; ((j < MAXLPRESET) && ((i + j) < NUMTILPRESET)); j++)
		    {
			if (!pmod->lpreset [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpreset (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	pmod->scanflag &= ~SCNLPRESET;
	}

	/* read the LPRATE values to be scanned */

	if (pmod->scanflag && SCNLPRATE)
	{
	    for (i = 0; ((i < NUMTILPRATE) && (pmod->lprate)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lprate [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPRATE) && ((i + j) < NUMTILPRATE)); j++)
		    {
			if (!pmod->lprate [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilprate (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPRATE;
	}

	/* read the LPHIALRM values to be scanned */

	if (pmod->scanflag && SCNLPHIALRM)
	{
	    for (i = 0; ((i < NUMTILPHIALRM) && (pmod->lphialrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lphialrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPHIALRM) && ((i + j) < NUMTILPHIALRM)); j++)
		    {
			if (!pmod->lphialrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilphialrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPHIALRM;
	}

	/* read the LPLOALRM values to be scanned */

	if (pmod->scanflag && SCNLPLOALRM)
	{
	    for (i = 0; ((i < NUMTILPLOALRM) && (pmod->lploalrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lploalrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPLOALRM) && ((i + j) < NUMTILPLOALRM)); j++)
		    {
			if (!pmod->lploalrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilploalrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPLOALRM;
	}

	/* read the LPPROCVAR values to be scanned */

	if (pmod->scanflag && SCNLPPROCVAR)
	{
	    for (i = 0; ((i < NUMTILPPROCVAR) && (pmod->lpprocvar)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpprocvar [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPPROCVAR) && ((i + j) < NUMTILPPROCVAR)); j++)
		    {
			if (!pmod->lpprocvar [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpprocvar (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPPROCVAR;
	}

	/* read the LPPVHILIM values to be scanned */

	if (pmod->scanflag && SCNLPPVHILIM)
	{
	    for (i = 0; ((i < NUMTILPPVHILIM) && (pmod->lppvhilim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lppvhilim [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPPVHILIM) && ((i + j) < NUMTILPPVHILIM)); j++)
		    {
			if (!pmod->lppvhilim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilppvhilim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPPVHILIM;
	}

	/* read the LPPVLOLIM values to be scanned */

	if (pmod->scanflag && SCNLPPVLOLIM)
	{
	    for (i = 0; ((i < NUMTILPPVLOLIM) && (pmod->lppvlolim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lppvlolim [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPPVLOLIM) && ((i + j) < NUMTILPPVLOLIM)); j++)
		    {
			if (!pmod->lppvlolim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilppvlolim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPPVLOLIM;
	}

	/* read the LPORGDEVALRM values to be scanned */

	if (pmod->scanflag && SCNLPORGDEVALRM)
	{
	    for (i = 0; ((i < NUMTILPORGDEVALRM) && (pmod->lporgdevalrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lporgdevalrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPORGDEVALRM) && ((i + j) < NUMTILPORGDEVALRM)); j++)
		    {
			if (!pmod->lporgdevalrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilporgdevalrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPORGDEVALRM;
	}

	/* read the LPYELDEVALRM values to be scanned */

	if (pmod->scanflag && SCNLPYELDEVALRM)
	{
	    for (i = 0; ((i < NUMTILPYELDEVALRM) && (pmod->lpyeldevalrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpyeldevalrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPYELDEVALRM) && ((i + j) < NUMTILPYELDEVALRM)); j++)
		    {
			if (!pmod->lpyeldevalrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpyeldevalrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPYELDEVALRM;
	}

	/* read the LPSMPRATE values to be scanned */

	if (pmod->scanflag && SCNLPSMPRATE)
	{
	    for (i = 0; ((i < NUMTILPSMPRATE) && (pmod->lpsmprate)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpsmprate [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPSMPRATE) && ((i + j) < NUMTILPSMPRATE)); j++)
		    {
			if (!pmod->lpsmprate [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpsmprate (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPSMPRATE;
	}

	/* read the LPREMSP values to be scanned */

	if (pmod->scanflag && SCNLPREMSP)
	{
	    for (i = 0; ((i < NUMTILPREMSP) && (pmod->lpremsp)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpremsp [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPREMSP) && ((i + j) < NUMTILPREMSP)); j++)
		    {
			if (!pmod->lpremsp [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpremsp (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPREMSP;
	}

	/* read the LPOUTPUT values to be scanned */

	if (pmod->scanflag && SCNLPOUTPUT)
	{
	    for (i = 0; ((i < NUMTILPOUTPUT) && (pmod->lpoutput)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpoutput [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPOUTPUT) && ((i + j) < NUMTILPOUTPUT)); j++)
		    {
			if (!pmod->lpoutput [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpoutput (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPOUTPUT;
	}

	/* read the LPERROR values to be scanned */

	if (pmod->scanflag && SCNLPERROR)
	{
	    for (i = 0; ((i < NUMTILPERROR) && (pmod->lperror)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lperror [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPERROR) && ((i + j) < NUMTILPERROR)); j++)
		    {
			if (!pmod->lperror [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilperror (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPERROR;
	}

	/* read the LPBIAS values to be scanned */

	if (pmod->scanflag && SCNLPBIAS)
	{
	    for (i = 0; ((i < NUMTILPBIAS) && (pmod->lpbias)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lpbias [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPBIAS) && ((i + j) < NUMTILPBIAS)); j++)
		    {
			if (!pmod->lpbias [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilpbias (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPBIAS;
	}

	/* read the LPPVHIHILIM values to be scanned */

	if (pmod->scanflag && SCNLPPVHIHILIM)
	{
	    for (i = 0; ((i < NUMTILPPVHIHILIM) && (pmod->lppvhihilim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lppvhihilim [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPPVHIHILIM) && ((i + j) < NUMTILPPVHIHILIM)); j++)
		    {
			if (!pmod->lppvhihilim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilppvhihilim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPPVHIHILIM;
	}

	/* read the LPPVLOLOLIM values to be scanned */

	if (pmod->scanflag && SCNLPPVLOLOLIM)
	{
	    for (i = 0; ((i < NUMTILPPVLOLOLIM) && (pmod->lppvlololim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lppvlololim [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPPVLOLOLIM) && ((i + j) < NUMTILPPVLOLOLIM)); j++)
		    {
			if (!pmod->lppvlololim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilppvlololim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPPVLOLOLIM;
	}

	/* read the LPROCALRM values to be scanned */

	if (pmod->scanflag && SCNLPROCALRM)
	{
	    for (i = 0; ((i < NUMTILPROCALRM) && (pmod->lprocalrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->lprocalrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXLPROCALRM) && ((i + j) < NUMTILPROCALRM)); j++)
		    {
			if (!pmod->lprocalrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtilprocalrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNLPROCALRM;
	}

	/* read the ANPVHIALRM values to be scanned */

	if (pmod->scanflag && SCNANPVHIALRM)
	{
	    for (i = 0; ((i < NUMTIANPVHIALRM) && (pmod->anpvhialrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anpvhialrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPVHIALRM) && ((i + j) < NUMTIANPVHIALRM)); j++)
		    {
			if (!pmod->anpvhialrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianpvhialrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPVHIALRM;
	}

	/* read the ANPVLOALRM values to be scanned */

	if (pmod->scanflag && SCNANPVLOALRM)
	{
	    for (i = 0; ((i < NUMTIANPVLOALRM) && (pmod->anpvloalrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anpvloalrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPVLOALRM) && ((i + j) < NUMTIANPVLOALRM)); j++)
		    {
			if (!pmod->anpvloalrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianpvloalrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPVLOALRM;
	}

	/* read the ANPROCVAR values to be scanned */

	if (pmod->scanflag && SCNANPROCVAR)
	{
	    for (i = 0; ((i < NUMTIANPROCVAR) && (pmod->anprocvar)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anprocvar [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPROCVAR) && ((i + j) < NUMTIANPROCVAR)); j++)
		    {
			if (!pmod->anprocvar [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianprocvar (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPROCVAR;
	}

	/* read the ANPVHILIM values to be scanned */

	if (pmod->scanflag && SCNANPVHILIM)
	{
	    for (i = 0; ((i < NUMTIANPVHILIM) && (pmod->anpvhilim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anpvhilim [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPVHILIM) && ((i + j) < NUMTIANPVHILIM)); j++)
		    {
			if (!pmod->anpvhilim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianpvhilim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPVHILIM;
	}

	/* read the ANPVLOLIM values to be scanned */

	if (pmod->scanflag && SCNANPVLOLIM)
	{
	    for (i = 0; ((i < NUMTIANPVLOLIM) && (pmod->anpvlolim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anpvlolim [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPVLOLIM) && ((i + j) < NUMTIANPVLOLIM)); j++)
		    {
			if (!pmod->anpvlolim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianpvlolim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPVLOLIM;
	}

	/* read the ANORGDEVLIM values to be scanned */

	if (pmod->scanflag && SCNANORGDEVLIM)
	{
	    for (i = 0; ((i < NUMTIANORGDEVLIM) && (pmod->anorgdevlim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anorgdevlim [i].was_requested)
		{
		    for (j = 1; ((j < MAXANORGDEVLIM) && ((i + j) < NUMTIANORGDEVLIM)); j++)
		    {
			if (!pmod->anorgdevlim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianorgdevlim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANORGDEVLIM;
	}

	/* read the ANYELDEVLIM values to be scanned */

	if (pmod->scanflag && SCNANYELDEVLIM)
	{
	    for (i = 0; ((i < NUMTIANYELDEVLIM) && (pmod->anyeldevlim)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anyeldevlim [i].was_requested)
		{
		    for (j = 1; ((j < MAXANYELDEVLIM) && ((i + j) < NUMTIANYELDEVLIM)); j++)
		    {
			if (!pmod->anyeldevlim [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianyeldevlim (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANYELDEVLIM;
	}

	/* read the ANSMPRATE values to be scanned */

	if (pmod->scanflag && SCNANSMPRATE)
	{
	    for (i = 0; ((i < NUMTIANSMPRATE) && (pmod->ansmprate)); i++)
	    {
		mem_allocated = 1;
		if (pmod->ansmprate [i].was_requested)
		{
		    for (j = 1; ((j < MAXANSMPRATE) && ((i + j) < NUMTIANSMPRATE)); j++)
		    {
			if (!pmod->ansmprate [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtiansmprate (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANSMPRATE;
	}

	/* read the ANSETPOINT values to be scanned */

	if (pmod->scanflag && SCNANSETPOINT)
	{
	    for (i = 0; ((i < NUMTIANSETPOINT) && (pmod->ansetpoint)); i++)
	    {
		mem_allocated = 1;
		if (pmod->ansetpoint [i].was_requested)
		{
		    for (j = 1; ((j < MAXANSETPOINT) && ((i + j) < NUMTIANSETPOINT)); j++)
		    {
			if (!pmod->ansetpoint [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtiansetpoint (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANSETPOINT;
	}

	/* read the ANERROR values to be scanned */

	if (pmod->scanflag && SCNANERROR)
	{
	    for (i = 0; ((i < NUMTIANERROR) && (pmod->anerror)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anerror [i].was_requested)
		{
		    for (j = 1; ((j < MAXANERROR) && ((i + j) < NUMTIANERROR)); j++)
		    {
			if (!pmod->anerror [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianerror (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANERROR;
	}

	/* read the ANPVHIHIALRM values to be scanned */

	if (pmod->scanflag && SCNANPVHIHIALRM)
	{
	    for (i = 0; ((i < NUMTIANPVHIHIALRM) && (pmod->anpvhihialrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anpvhihialrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPVHIHIALRM) && ((i + j) < NUMTIANPVHIHIALRM)); j++)
		    {
			if (!pmod->anpvhihialrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianpvhihialrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPVHIHIALRM;
	}

	/* read the ANPVLOLOALRM values to be scanned */

	if (pmod->scanflag && SCNANPVLOLOALRM)
	{
	    for (i = 0; ((i < NUMTIANPVLOLOALRM) && (pmod->anpvloloalrm)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anpvloloalrm [i].was_requested)
		{
		    for (j = 1; ((j < MAXANPVLOLOALRM) && ((i + j) < NUMTIANPVLOLOALRM)); j++)
		    {
			if (!pmod->anpvloloalrm [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianpvloloalrm (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANPVLOLOALRM;
	}

	/* read the ANROC values to be scanned */

	if (pmod->scanflag && SCNANROC)
	{
	    for (i = 0; ((i < NUMTIANROC) && (pmod->anroc)); i++)
	    {
		mem_allocated = 1;
		if (pmod->anroc [i].was_requested)
		{
		    for (j = 1; ((j < MAXANROC) && ((i + j) < NUMTIANROC)); j++)
		    {
			if (!pmod->anroc [i + j].was_requested)
			{
			    break;
			}
		    }
		    numgroups = j;
		    if (!readtianroc (line, mod, pmod, i, numgroups))
		    {
			return;
		    }
		    checkoutputs (line);
		    i = (i + j) - 1;
		}
	    }
	    pmod->scanflag &= ~SCNANROC;
	}
	if (mem_allocated == 0)
	{
	    taskDelay (250);
	}
}

/***********
*
*	checkoutputs
*
*	checks output que and returns if empty.
*	if there is something on the que it attempts to process all
*	pending outputs doing imediate retries. If attempts fail the
*	request is dropped
*
**********/
static	void	checkoutputs (unsigned short line)
{
	OUTQENT		*optr;
	unsigned short  signal, link, adapter, card, conversion;
	TI_RET_VAL	value;
	unsigned long   bomask, boval;
	unsigned short	boshift;
	TIPLC  		*pmod;

	while ((optr = ogetnextp (line)) != NULLP)
	{
	    if ((optr->otype) == AOUTTYPE)
	    {
		link = optr->out.aonode.link;
		adapter = optr->out.aonode.adapter;

		pmod = plcs [link] [adapter];
		if (pmod == NULL)
		{
		    if (!allocate_plc (link, adapter))
		    {
			return; /* if not allocated, try. if can't allocate, get out. */
		    }
		    pmod = plcs [link] [adapter];
		}
		card = optr->out.aonode.card;
		signal = optr->out.aonode.aosignal;
		conversion = optr->out.aonode.conversion;
		if (conversion == ASCFLT)
		{
		    value.fval = optr->out.aonode.aovalue.fval;
		    if (mltrace [line] >= 2)
		    {
			logMsg ("Analog Output to line %d, address %d, card %d signal %d,m val %f\n",
				link, adapter, card, (signal + 1), value.fval);
		    }
		}
		else
		{
		    value.usval [0] = optr->out.aonode.aovalue.usval [0];
		    if (mltrace [line] >= 2)
		    {
			logMsg ("Analog Output to line %d, address %d, card %d signal %d,m val %d\n",
				link, adapter, card, (signal + 1), value.usval);
		    }
		}

		if ((!pmod->cmd_force_offline)  && (resresponse [link]) && (pmod->is_responding))
		{
		    if (!setaouts (link, adapter, pmod, card, signal, conversion, value))
		    {
			pmod->badoutcnt++;
		    }
		}
	    }
	    else if ((optr->otype) == BOUTTYPE)
	    {
		link = optr->out.bonode.link;
		adapter = optr->out.bonode.adapter;

		pmod = plcs [link] [adapter];
		if (pmod == NULL)
		{
		    if (!allocate_plc (link, adapter))
		    {
			return; /* if not allocated, try. if can't allocate, get out. */
		    }
		    pmod = plcs [link] [adapter];
		}

		card = optr->out.bonode.card;
		signal = optr->out.bonode.bosignal;
		boval = optr->out.bonode.bovalue;
		bomask = optr->out.bonode.bomask;
		boshift = optr->out.bonode.boshift;

		if (mltrace [line] >= 2)
		{ 
		    logMsg ( "Binary Output to line %d, address %d, card %d signal %d\n",
				link, adapter, card, signal);
		}
		if ((!pmod->cmd_force_offline) && (resresponse [link]) && (pmod->is_responding))
		{
		    if (!setbouts (line, adapter, pmod, card, signal,
			bomask, boshift, boval))
		    {
			pmod->badoutcnt++;
		    }
		}
	    }
	    oreleasefull (line);     /* release the output que entry */
	}
}

/**********
*
*	prepscan (line)
*
**********/
void	prepscan (unsigned short line, TIPLC *pmod)
{
	tsLocalTime (&tmnow);

	if (lpgainscn [line])
	{
	    if ((lpgainscn [line] + lpgainlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpgainlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPGAIN;
	    }
	}
	if (lpresetscn [line])
	{
	    if ((lpresetscn [line] + lpresetlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpresetlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPRESET;
	    }
	}
	if (lpratescn [line])
	{
	    if ((lpratescn [line] + lpratelst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpratelst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPRATE;
	    }
	}
	if (lphialrmscn [line])
	{
	    if ((lphialrmscn [line] + lphialrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lphialrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPHIALRM;
	    }
	}
	if (lploalrmscn [line])
	{
	    if ((lploalrmscn [line] + lploalrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lploalrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPLOALRM;
	    }
	}
	if (lpprocvarscn [line])
	{
	    if ((lpprocvarscn [line] + lpprocvarlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpprocvarlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPPROCVAR;
	    }
	}
	if (lppvhilimscn [line])
	{
	    if ((lppvhilimscn [line] + lppvhilimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lppvhilimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPPVHILIM;
	    }
	}
	if (lppvlolimscn [line])
	{
	    if ((lppvlolimscn [line] + lppvlolimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lppvlolimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPPVLOLIM;
	    }
	}
	if (lporgdevalrmscn [line])
	{
	    if ((lporgdevalrmscn [line] + lporgdevalrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lporgdevalrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPORGDEVALRM;
	    }
	}
	if (lpyeldevalrmscn [line])
	{
	    if ((lpyeldevalrmscn [line] + lpyeldevalrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpyeldevalrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPYELDEVALRM;
	    }
	}
	if (lpsmpratescn [line])
	{
	    if ((lpsmpratescn [line] + lpsmpratelst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpsmpratelst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPSMPRATE;
	    }
	}
	if (lpremspscn [line])
	{
	    if ((lpremspscn [line] + lpremsplst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpremsplst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPREMSP;
	    }
	}
	if (lpoutputscn [line])
	{
	    if ((lpoutputscn [line] + lpoutputlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpoutputlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPOUTPUT;
	    }
	}
	if (lperrorscn [line])
	{
	    if ((lperrorscn [line] + lperrorlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lperrorlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPERROR;
	    }
	}
	if (lpbiasscn [line])
	{
	    if ((lpbiasscn [line] + lpbiaslst [line]) >= tmnow [line].secPastEpoch)
	    {
		lpbiaslst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPBIAS;
	    }
	}
	if (lppvhihilimscn [line])
	{
	    if ((lppvhihilimscn [line] + lppvhihilimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lppvhihilimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPPVHIHILIM;
	    }
	}
	if (lppvlololimscn [line])
	{
	    if ((lppvlololimscn [line] + lppvlololimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lppvlololimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPPVLOLOLIM;
	    }
	}
	if (lprocalrmscn [line])
	{
	    if ((lprocalrmscn [line] + lprocalrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		lprocalrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNLPROCALRM;
	    }
	}
	if (anpvhialrmscn [line])
	{
	    if ((anpvhialrmscn [line] + anpvhialrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anpvhialrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPVHIALRM;
	    }
	}
	if (anpvloalrmscn [line])
	{
	    if ((anpvloalrmscn [line] + anpvloalrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anpvloalrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPVLOALRM;
	    }
	}
	if (anprocvarscn [line])
	{
	    if ((anprocvarscn [line] + anprocvarlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anprocvarlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPROCVAR;
	    }
	}
	if (anpvhilimscn [line])
	{
	    if ((anpvhilimscn [line] + anpvhilimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anpvhilimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPVHILIM;
	    }
	}
	if (anpvlolimscn [line])
	{
	    if ((anpvlolimscn [line] + anpvlolimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anpvlolimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPVHILIM;
	    }
	}
	if (anorgdevlimscn [line])
	{
	    if ((anorgdevlimscn [line] + anorgdevlimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anorgdevlimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANORGDEVLIM;
	    }
	}
	if (anyeldevlimscn [line])
	{
	    if ((anyeldevlimscn [line] + anyeldevlimlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anyeldevlimlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANYELDEVLIM;
	    }
	}
	if (ansmpratescn [line])
	{
	    if ((ansmpratescn [line] + ansmpratelst [line]) >= tmnow [line].secPastEpoch)
	    {
		ansmpratelst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANSMPRATE;
	    }
	}
	if (ansetpointscn [line])
	{
	    if ((ansetpointscn [line] + ansetpointlst [line]) >= tmnow [line].secPastEpoch)
	    {
		ansetpointlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANSETPOINT;
	    }
	}
	if (anerrorscn [line])
	{
	    if ((anerrorscn [line] + anerrorlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anerrorlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANERROR;
	    }
	}
	if (anpvhihialrmscn [line])
	{
	    if ((anpvhihialrmscn [line] + anpvhihialrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anpvhihialrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPVHIHIALRM;
	    }
	}
	if (anpvloloalrmscn [line])
	{
	    if ((anpvloloalrmscn [line] + anpvloloalrmlst [line]) >= tmnow [line].secPastEpoch)
	    {
		anpvloloalrmlst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANPVLOLOALRM;
	    }
	}
	if (anrocscn [line])
	{
	    if ((anrocscn [line] + anroclst [line]) >= tmnow [line].secPastEpoch)
	    {
		anroclst [line] = tmnow [line].secPastEpoch;
		pmod->scanflag |= SCNANROC;
	    }
	}
}

/**********
*
*	initscan (line)
*
**********/
void	initscan (unsigned short line, TIPLC *pmod)
{
	tsLocalTime (&tmnow);

	lpgainscn [line] = 60;
	lpgainlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPGAIN;

	lpresetscn [line] = 60;
	lpresetlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPRESET;

	lpratescn [line] = 60;
	lpratelst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPRATE;

	lphialrmscn [line] = 60;
	lphialrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPHIALRM;

	lploalrmscn [line] = 60;
	lploalrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPLOALRM;

	lpprocvarscn [line] = 60;
	lpprocvarlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPPROCVAR;

	lppvhilimscn [line] = 60;
	lppvhilimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPPVHILIM;

	lppvlolimscn [line] = 60;
	lppvlolimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPPVLOLIM;

	lporgdevalrmscn [line] = 60;
	lporgdevalrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPORGDEVALRM;

	lpyeldevalrmscn [line] = 60;
	lpyeldevalrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPYELDEVALRM;

	lpsmpratescn [line] = 60;
	lpsmpratelst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPSMPRATE;

	lpremspscn [line] = 60;
	lpremsplst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPREMSP;

	lpoutputscn [line] = 60;
	lpoutputlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPOUTPUT;

	lperrorscn [line] = 60;
	lperrorlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPERROR;

	lpbiasscn [line] = 60;
	lpbiaslst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPBIAS;

	lppvhihilimscn [line] = 60;
	lppvhihilimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPPVHIHILIM;

	lppvlololimscn [line] = 60;
	lppvlololimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPPVLOLOLIM;

	lprocalrmscn [line] = 60;
	lprocalrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNLPROCALRM;

	anpvhialrmscn [line] = 60;
	anpvhialrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPVHIALRM;

	anpvloalrmscn [line] = 60;
	anpvloalrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPVLOALRM;

	anprocvarscn [line] = 60;
	anprocvarlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPROCVAR;

	anpvhilimscn [line] = 60;
	anpvhilimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPVHILIM;

	anpvlolimscn [line] = 60;
	anpvlolimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPVLOLIM;

	anorgdevlimscn [line] = 60;
	anorgdevlimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANORGDEVLIM;

	anyeldevlimscn [line] = 60;
	anyeldevlimlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANYELDEVLIM;

	ansmpratescn [line] = 60;
	ansmpratelst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANSMPRATE;

	ansetpointscn [line] = 60;
	ansetpointlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANSETPOINT;

	anerrorscn [line] = 60;
	anerrorlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANERROR;

	anpvhihialrmscn [line] = 60;
	anpvhihialrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPVHIHIALRM;

	anpvloloalrmscn [line] = 60;
	anpvloloalrmlst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANPVLOLOALRM;

	anrocscn [line] = 60;
	anroclst [line] = tmnow [line].secPastEpoch;
	pmod->scanflag |= SCNANROC;
}

/**********
*
*      aidriver
*
*      This routines main purpose is to return the analog input value for the
*      specified module. For a module value
*      a (-1) will be returned if there is no valid data ready.
*
**********/

int	AIDRIVER (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal,
			unsigned short *pvalue, unsigned short conversion,
			unsigned char *resname)
{
	TIPLC 			*pmod;
	TI_RET_VAL		uval;
	unsigned	short 	blkix, elemix, subelement;

	if (!rangeok (link, adapter, card, signal, TI_AI))
	{
	    logMsg ("Invalid Address, Card, or Signal in point %s\n", pointname);
	    return (-1);
	}
	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == PSEUDO2)
	{
	    switch (signal)
	    {

	    case 1: /* Were the PLC values requested */
		*pvalue = pmod->was_requested;
		break;

	    case 2: /* Is the PLC responding */
		*pvalue = pmod->is_responding;
		break;

	    case 5: /* Output Messages */
		*pvalue = pmod->no_dataout_msgs;
		break;

	    case 6: /* Input Messages */
		*pvalue = pmod->no_datain_msgs;
		break;

	    case 7: /* No Receiver Responses */
		*pvalue = pmod->no_rxnoresp;
		break;

	    case 8: /* No Transmitter Response */
		*pvalue = pmod->no_txnoresp;;
		break;

	    case 9: /* Data Echo Errors */
		*pvalue = pmod->no_dataerr;
		break;

	    case 10: /* Message Retries */
		*pvalue = pmod->no_retries;
		break;

	    case 11: /* RX Hardware errors */
		*pvalue = pmod->no_rxhwerrs;
		break;

	    case 12: /* TX Hardware errors */
		*pvalue = pmod->no_txhwerrs;
		break;

	    case 13: /* CRC errors */
		*pvalue = pmod->no_crcerrs;
		break;

	    case 14: /* Offline transitions */
		*pvalue = pmod->no_offlines;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 17: /* Failed output commands */
		*pvalue = pmod->badoutcnt;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode [link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select [link];
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}

	if (card == SCANCNTL)
	{
	    switch (signal)
	    {
	    case 1:
		*pvalue = lpgainscn [link];
		break;

	    case 2:
		*pvalue = lpresetscn [link];
		break;

	    case 3:
		*pvalue = lpratescn [link];
		break;

	    case 4:
		*pvalue = lphialrmscn [link];
		break;

	    case 5:
		*pvalue = lploalrmscn [link];
		break;

	    case 6:
		*pvalue = lpprocvarscn [link];
		break;

	    case 7:
		*pvalue = lppvhilimscn [link];
		break;

	    case 8:
		*pvalue = lppvlolimscn [link];
		break;

	    case 9:
		*pvalue = lporgdevalrmscn [link];
		break;

	    case 10:
		*pvalue = lpyeldevalrmscn [link];
		break;

	    case 11:
		*pvalue = lpsmpratescn [link];
		break;

	    case 12:
		*pvalue = lpremspscn [link];
		break;

	    case 13:
		*pvalue = lpoutputscn [link];
		break;

	    case 14:
		*pvalue = lperrorscn [link];
		break;

	    case 15:
		*pvalue = lpbiasscn [link];
		break;

	    case 16:
		*pvalue = lppvhihilimscn [link];
		break;

	    case 17:
		*pvalue = lppvlololimscn [link];
		break;

	    case 18:
		*pvalue = lprocalrmscn [link];
		break;

	    case 19:
		*pvalue = anpvhialrmscn [link];
		break;

	    case 20:
		*pvalue = anpvloalrmscn [link];
		break;

	    case 21:
		*pvalue = anprocvarscn [link];
		break;

	    case 22:
		*pvalue = anpvhilimscn [link];
		break;

	    case 23:
		*pvalue = anpvlolimscn [link];
		break;

	    case 24:
		*pvalue = anorgdevlimscn [link];
		break;

	    case 25:
		*pvalue = anyeldevlimscn [link];
		break;

	    case 26:
		*pvalue = ansmpratescn [link];
		break;

	    case 27:
		*pvalue = ansetpointscn [link];
		break;

	    case 28:
		*pvalue = anerrorscn [link];
		break;

	    case 29:
		*pvalue = anpvhihialrmscn [link];
		break;

	    case 30:
		*pvalue = anpvloloalrmscn [link];
		break;

	    case 31:
		*pvalue = anrocscn [link];
		break;

	    default:
		logMsg ("%s Signal out of range\n", pointname);
		return (-1);
	    }
	    return (0);
	}
	if (resname != 0)
	{
	    register_resource (link, pmod, resname, pointname);
	}

	if ((card == TIVMEM) || (card == TICMEM) || (card == TIWXIN)
		|| (card == TIWYOUT) || (card == TISTATUS)
		|| ((card >= TILPGAIN) && (card <= TILPOUTPUT))
		|| ((card >= TILPERROR) && (card <= TILPROCALRM))
		|| ((card >= TIANPVHIALRM) && (card <= TIANSETPOINT))
		|| ((card >= TIANERROR) && (card <= TIANROC)))
	{
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		signal -= 1;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		break;

	    default:
		return (-1);
	    }
	    blkix = (signal / TINOGRPELEMS);
	    elemix = (signal % TINOGRPELEMS);
	}
	else
	{
	    if (mltrace [link] >= 2)
	    {
		logMsg ("%s Invalid card type for Anolog Input\n", pointname);
	    }
	    return (-1);
	}

	if (!setrequested (pmod, card, blkix))
	{
	    return (-1);
	}

	if ((elemix == 3) && ((conversion == CNVDOUBLEINT)
		|| (conversion == CNVFLOAT))
		&& ((card == TIVMEM) || (card == TICMEM)))
	{
	    switch (card)
	    {
	    case TIVMEM:
		if ((blkix + 1) >= NUMTIVMEM)
		{
		    logMsg ("%s Signal exceeds maximum\n", pointname);
		    return (-1);
		}

	    case TICMEM:
		if ((blkix + 1) >= NUMTICMEM)
		{
		    logMsg ("%s Signal exceeds maximum\n", pointname);
		    return (-1);
		}
	    }
	    if (!setrequested (pmod, card, (blkix + 1)))
	    {
		return (-1);
	    }
	}
	if (!resresponse [link])
	{
	    return (-1);
	}
	if (pmod->holdflag)
	{
	    return (-2);
	}
	if (!pmod->is_responding)
	{
	    return (-1);
	}
	if (fetchvalue (pmod, card, blkix, elemix, subelement, &uval, conversion) == -1)
	{
	    return (-1);
	}
	if ((conversion == CNVDOUBLEINT) || (conversion == CNVFLOAT))
	{
	    pvalue [0] = uval.usval [0];
	    pvalue [1] = uval.usval [1];
	}
	else
	{
	    *pvalue = uval.usval [0];
	}
	return (0);
}

/**********
*
*	tiway_aodriver
*
*	This routines main purpose is to set requested analog outputs to the
*	specified module.
*
**********/

int	AODRIVER (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal,
			double dval, unsigned short value,
			unsigned short conversion, unsigned char *resname)
{
	TIPLC 			*pmod;
	unsigned	short 	blkix, elemix;
	OUTQENT			*oqp;

	if (!rangeok (link, adapter, card, signal, TI_AO))
	{
	    logMsg ("Invalid Address, Card, or Signal in point %s\n", pointname);
	    return (-1);
	}
	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == PSEUDO2)
	{
	    switch (signal)
	    {

	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
		{
		    initstats (pmod);
		}
		break;

	    case 16: /* Force the PLC offline */
		pmod->cmd_force_offline = (value != 0);
		pmod->is_responding = (value != 0);
		break;

	    case 18: /* What is the backup mode for the line*/
		backup_mode [link] = (value != 0);
		break;

	    case 19: /* What is the selected channel for the line */
		channel_select [link] = (value != 0);
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}
	if (card == SCANCNTL)
	{
	    tsLocalTime (&tmnow);
	    switch (signal)
	    {
	    case 1:
		lpgainscn [link] = value;
		lpgainlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPGAIN;
		break;

	    case 2:
		lpresetscn [link] = value;
		lpresetlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPRESET;
		break;

	    case 3:
		lpratescn [link] = value;
		lpratelst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPRATE;
		break;

	    case 4:
		lphialrmscn [link] = value;
		lphialrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPHIALRM;
		break;

	    case 5:
		lploalrmscn [link] = value;
		lploalrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPLOALRM;
		break;

	    case 6:
		lpprocvarscn [link] = value;
		lpprocvarlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPPROCVAR;
		break;

	    case 7:
		lppvhilimscn [link] = value;
		lppvhilimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPPVHILIM;
		break;

	    case 8:
		lppvlolimscn [link] = value;
		lppvlolimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPPVLOLIM;
		break;

	    case 9:
		lporgdevalrmscn [link] = value;
		lporgdevalrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPORGDEVALRM;
		break;

	    case 10:
		lpyeldevalrmscn [link] = value;
		lpyeldevalrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPYELDEVALRM;
		break;

	    case 11:
		lpsmpratescn [link] = value;
		lpsmpratelst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPSMPRATE;
		break;

	    case 12:
		lpremspscn [link] = value;
		lpremsplst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPREMSP;
		break;

	    case 13:
		lpoutputscn [link] = value;
		lpoutputlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPOUTPUT;
		break;

	    case 14:
		lperrorscn [link] = value;
		lperrorlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPERROR;
		break;

	    case 15:
		lpbiasscn [link] = value;
		lpbiaslst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPBIAS;
		break;

	    case 16:
		lppvhihilimscn [link] = value;
		lppvhihilimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPPVHIHILIM;
		break;

	    case 17:
		lppvlololimscn [link] = value;
		lppvlololimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPPVLOLOLIM;
		break;

	    case 18:
		lprocalrmscn [link] = value;
		lprocalrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNLPROCALRM;
		break;

	    case 19:
		anpvhialrmscn [link] = value;
		anpvhialrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPVHIALRM;
		break;

	    case 20:
		anpvloalrmscn [link] = value;
		anpvloalrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPVLOALRM;
		break;

	    case 21:
		anprocvarscn [link] = value;
		anprocvarlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPROCVAR;
		break;

	    case 22:
		anpvhilimscn [link] = value;
		anpvhilimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPVHILIM;
		break;

	    case 23:
		anpvlolimscn [link] = value;
		anpvlolimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPVLOLIM;
		break;

	    case 24:
		anorgdevlimscn [link] = value;
		anorgdevlimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANORGDEVLIM;
		break;

	    case 25:
		anyeldevlimscn [link] = value;
		anyeldevlimlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANYELDEVLIM;
		break;

	    case 26:
		ansmpratescn [link] = value;
		ansmpratelst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANSMPRATE;
		break;

	    case 27:
		ansetpointscn [link] = value;
		ansetpointlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANSETPOINT;
		break;

	    case 28:
		anerrorscn [link] = value;
		anerrorlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANERROR;
		break;

	    case 29:
		anpvhihialrmscn [link] = value;
		anpvhihialrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPVHIHIALRM;
		break;

	    case 30:
		anpvloloalrmscn [link] = value;
		anpvloloalrmlst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANPVLOLOALRM;
		break;

	    case 31:
		anrocscn [link] = value;
		anroclst [link] = tmnow [link].secPastEpoch;
		pmod->scanflag |= SCNANROC;
		break;

	    default:
		logMsg ("%s Signal out of range\n", pointname);
		return (-1);
	    }
	    return (0);
	}
	if (resname != 0)
	{ 
	    register_resource (link, pmod, resname, pointname);
	}
	if ((card == TIVMEM)
		|| (card == TICMEM) || (card == TIWYOUT)
		|| ((card >= TILPGAIN) && (card <= TILPOUTPUT))
		|| ((card >= TILPBIAS) && (card <= TILPROCALRM))
		|| ((card >= TIANPVHIALRM) && (card <= TIANSETPOINT))
		|| ((card >= TIANPVHIHIALRM) && (card <= TIANROC)))
	{
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		signal -= 1;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		break;

	    default:
		return (-1);
	    }
	    blkix = (signal / TINOGRPELEMS);
	    elemix = (signal % TINOGRPELEMS);
	}
	else
	{
	    if (mltrace [link] >= 2)
	    {
		logMsg ("%s Invalid card type for Anolog Output\n", pointname);
	    }
	    return (-1);
	}
	if (!setrequested (pmod, card, blkix))
	{
	    return (-1);
	}
	if ((elemix == 3) && ((conversion == CNVDOUBLEINT)
		|| (conversion == CNVFLOAT)) && (card == TIVMEM))
	{
	    if ((blkix + 1) >= NUMTIVMEM)
	    {
		logMsg ("%s Signal exceeds maximum\n", pointname);
		return (-1);
	    }
	    if (!setrequested (pmod, card, (blkix + 1)))
	    {
		return (-1);
	    }
	}
	if (!resresponse [link])
	{
	    return (-2);
	}
	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}

	oqp = ogetemptyp (link);
	if (oqp == NULLP)
	{
	    return (-1); /* output que is full */
	}
	oqp->otype = AOUTTYPE;
	oqp->out.aonode.link = link;
	oqp->out.aonode.adapter = adapter;
	oqp->out.aonode.card = card;
	oqp->out.aonode.aosignal = signal;
	oqp->out.aonode.conversion = conversion;
	if (conversion == ASCFLT)
	{
	    oqp->out.aonode.aovalue.fval = dval;
	}
	else
	{
	    oqp->out.aonode.aovalue.sval [0] = value;
	}
	omarkfull (link); /* Mark this entry as ready for output */

	return (0);
}

/**********
*
*      tiway_aoread
*
*      This routines main purpose is to return the value last read back
*      from the analog output hardware. This is used to initialize analog
*      outputs.
*
**********/

int	AOREAD (char *pointname, unsigned short link, unsigned short adapter,
		unsigned short card, unsigned short signal,
		unsigned short *pvalue, unsigned short conversion, 
		unsigned char *resname)

{
	TIPLC			*pmod;
	TI_RET_VAL		uval;
	unsigned	short 	blkix, elemix, subelement;

	if (!rangeok (link, adapter, card, signal, TI_AO))
	{
	    logMsg ("Invalid Address, Card, or Signal in point %s\n", pointname);
	    return (-1);
	}
	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1);   /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == PSEUDO2)
	{
	    switch (signal)
	    {

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode [link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select [link];
		break;

	    default:
		return(-1);
	    }
	    return (0);
	}
	if (resname != 0)
	{ 
	    register_resource (link, pmod, resname, pointname);
	}
	if ((card == TIVMEM)
		|| (card == TIVMEM) || (card == TIWYOUT)
		|| ((card >= TILPGAIN) && (card <= TILPOUTPUT))
		|| ((card >= TILPBIAS) && (card <= TILPROCALRM))
		|| ((card >= TIANPVHIALRM) && (card <= TIANSETPOINT))
		|| ((card >= TIANPVHIHIALRM) && (card <= TIANROC)))
	{
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		signal -= 1;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		break;

	    default:
		return (-1);
	    }

	    blkix = (signal / TINOGRPELEMS);
	    elemix = (signal % TINOGRPELEMS);
	}
	else
	{
	    if (mltrace [link] >= 2)
	    {
		logMsg ("%s Invalid card type for Anolog Output\n", pointname);
	    }
	    return (-1);
	}
	if (!setrequested (pmod, card, blkix))
	{
	    return (-1);
	}
	if (!resresponse [link])
	{
	    return (-1);
	}
	if (pmod->holdflag)
	{
	    return (-2);
	}
	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}

	if (fetchvalue (pmod, card, blkix, elemix, subelement, &uval, conversion) == -1)
	{
	    return (-1);
	}
	if (conversion == ASCFLT)
	{
	    pvalue [0] = uval.usval [0];
	    pvalue [1] = uval.usval [1];
	}
	else
	{
	    *pvalue = uval.usval [0];
	}
	return (0);
}

/***********
*
*      tiway_bidriver
*
*      The main purpose of this routine is to return the request digital input
*      value(s) from the requested card. This routine will return a (-1) if the
*      requested module is the wrong type or the data is not yet valid.
*
***********/
int	BIDRIVER (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal,
			unsigned long *mask, unsigned short *shift,
			unsigned long *pvalue, unsigned char *resname)
{
	TIPLC			*pmod;
	TI_RET_VAL		uval;
	unsigned	short	blkix, elemix, subelement, tvallow, tvalhigh;
	unsigned	short	tsignal;
	long			retval;

	if (!rangeok (link, adapter, card, signal, TI_BI))
	{
	    logMsg ("Invalid Address, Card, or Signal in point %s\n", pointname);
	    return (-1);
	}
	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == PSEUDO2)
	{
	    if (*mask == 0)
	    {
		*mask = masks [(signal % 32)];
	    }
	    switch (signal)
	    {

	    case 1: /* Were the PLC values requested */
		*pvalue = pmod->was_requested;
		break;

	    case 2: /* Is the PLC responding */
		*pvalue = pmod->is_responding;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode [link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select [link];
		break;

	    default:
		return (-1);
	    }

	    return (0);
	}
	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWXIN:
	    case TIWYOUT:
	    case TISTATUS:
	    case TILPSTAT:
	    case TILPMD:
	    case TIAVFLAG:

		if (shift)
		{
		    *shift = (signal % 16);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 16)];
		    }
		}
		blkix = (((signal / 16) - 1) / TINOGRPELEMS);
		elemix = (((signal / 16) - 1) % TINOGRPELEMS);
		subelement = 0;
		break;

	    case TIXMEM:
	    case TIYMEM:
	    case TICRMEM:

		if (shift)
		{
		    *shift = ((signal - 1) % 32);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [((signal - 1) % 32)];
		    }
		}
		blkix = ((signal - 1) / 32);
		elemix = 0;
		subelement = 0;
		break;


	    case TILPCNTL:
	    case TIACFLAG:
		if (shift)
		{
		    *shift = (signal % 32);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 32)];
		    }
		}
		blkix = (((signal / 32) - 1) / TINOGRPELEMS);
		elemix = (((signal / 32) - 1) % TINOGRPELEMS);
		subelement = 0;
		break;

	    default:
		if (mltrace [link] >= 2)
		{
		    logMsg ("%s Invalid card type for Binary Input\n", pointname);
		}
		return (-1);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWXIN:
	    case TIWYOUT:
	    case TISTATUS:
	    case TILPSTAT:
	    case TILPMD:
	    case TIAVFLAG:

		if (shift)
		{
		    *shift = (signal % 16);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 16)];
		    }
		}
		blkix = ((signal / 16) / TINOGRPELEMS);
		elemix = ((signal / 16) % TINOGRPELEMS);
		subelement = 0;
		break;

	    case TIXMEM:
	    case TIYMEM:
		if (shift)
		{
		    *shift = signal % 32;
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 32)];
		    }
		}
		blkix = signal;
		elemix = 0;
		subelement = 0;
		break;

	    case TICRMEM:

		if (shift)
		{
		    *shift = signal % 32;
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 32)];
		    }
		}
		blkix = (signal / 32);
		elemix = 0;
		subelement = 0;
		break;

	    default:
		if (mltrace [link] >= 2)
		{
		    logMsg ("%s Invalid card type for Binary Input\n", pointname);
		}
		return (-1);
	    }
	    break;

	default:
	    return (-1);
	}

	if (resname != 0)
	{
	    register_resource (link, pmod, resname, pointname);
	}
	if (!setrequested (pmod, card, blkix))
	{
	    return (-1);
	}
	if (!resresponse [link])
	{
	    return (-1);
	}
	if (pmod->holdflag)
	{
	    return (-2);
	}
	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}
	switch (card)
	{
	case TIVMEM:
	case TICMEM:
	case TIWXIN:
	case TIWYOUT:
	case TISTATUS:
	case TILPSTAT:
	case TILPMD:
	case TIAVFLAG:

	    if ((fetchvalue (pmod, card, blkix, elemix, subelement, &uval, 0)) == -1)
	    {
		return (-1);
	    }
	    *pvalue = (((long) uval.usval [0]) & *mask);
	    break;

	case TIXMEM:
	case TIYMEM:
	case TICRMEM:
	case TILPCNTL:
	case TIACFLAG:

	    if ((fetchvalue (pmod, card, blkix, elemix, subelement, &uval, 0)) == -1)
	    {
		return (-1);
	    }
	    *pvalue = (uval.ulval & *mask);
	    break;
	}
	return (0);
}

/**********
*
*      tiway_bodriver
*
*      The main purpose of this routine is to set the request digital output
*      value(s) on the requested card. This routine will return a (-1) if the
*      requested module is the wrong type or not configured yet.
*
***********/
int	BODRIVER (char *pointname, unsigned short link, unsigned short adapter,
			unsigned short card, unsigned short signal,
			unsigned long value, unsigned long mask,
			unsigned short shift, unsigned char *resname)
{
	TIPLC			*pmod;
	OUTQENT			*oqp;
	unsigned	short	tsignal;
	unsigned	short	blkix, elemix;

	if (!rangeok (link, adapter, card, signal, TI_BO))
	{
	    logMsg ("Invalid Address, Card, or Signal in point %s\n", pointname);
	    return (-1);
	}
	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == PSEUDO2)
	{
	    switch (signal & 0x1f)
	    {
	    case 15: /* Reset the accumulated statistics */
		if (value != 0)
		{
		    initstats (pmod);
		}
		break;

	    case 16: /* Force the PLC offline */
		pmod->cmd_force_offline = (value != 0);
		pmod->is_responding = (value != 0);
		break;

	    case 18: /* What is the backup mode for the line*/
		backup_mode [link] = (value != 0);
		break;

	    case 19: /* What is the selected channel for the line */
		channel_select [link] = (value != 0);
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}
	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWYOUT:
	    case TILPSTAT:
	    case TILPMD:

		blkix = (((signal / 16) - 1) / TINOGRPELEMS);
		break;

	    case TIYMEM:
	    case TICRMEM:

		blkix = ((signal - 1) / 32);
		break;

	    case TILPCNTL:
	    case TIACFLAG:

		blkix = (((signal / 32) - 1) / TINOGRPELEMS);
		break;

	    default:

		if (mltrace [link] >= 2)
		{
		    logMsg ("%s Invalid card type for Binary Output\n", pointname);
		}
		return (-1);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWYOUT:
	    case TILPSTAT:
	    case TILPMD:

		blkix = ((signal / 16) / TINOGRPELEMS);
		break;

	    case TITYMEM:
		blkix = signal;
		break;

	    case TICRMEM:

		blkix = (signal / 32);
		break;

	    default:

		if (mltrace [link] >= 2)
		{
		    logMsg ("%s Invalid card type for Binary Output\n", pointname);
		}
		return (-1);
	    }
	    break;

	default:
	    return (-1);
	}
	if (resname != 0)
	{ 
	    register_resource (link, pmod, resname, pointname);
	}
	if (!setrequested (pmod, card, blkix))
	{
	    return (-1);
	}
	if (!resresponse [link])
	{
	    return (-1);
	}
	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}

	oqp = ogetemptyp (link);

	if (oqp == NULLP)
	{
	    return (-1); /* output que is full */
	}
	oqp->otype = BOUTTYPE;
	oqp->out.bonode.link = link;
	oqp->out.bonode.adapter = adapter;
	oqp->out.bonode.card = card;
	oqp->out.bonode.bosignal = signal;
	oqp->out.bonode.bomask = mask;
	oqp->out.bonode.boshift = shift;
	oqp->out.bonode.bovalue = value;

	omarkfull (link); /* Mark this entry as ready for output */

	return (0);
}

/**********
*
*      tiway_boread
*
*      This routines main purpose is to return the value last read back
*      from the digital output hardware. This is used to initialize digital
*      outputs. This will return a (-1) if the module is the wrong type or
*      the data is not yet valid
*
**********/

int	BOREAD (char *pointname, unsigned short link, unsigned short adapter,
		unsigned short card, unsigned short signal,
		unsigned long *pvalue, unsigned long *mask,
		unsigned short *shift, unsigned char *resname)
{
	TIPLC			*pmod;
	TI_RET_VAL		uval;
	unsigned	short	blkix, elemix, subelement, tvallow, tvalhigh;
	unsigned	short	tsignal;
	long			retval;

	if (!rangeok (link, adapter, card, signal, TI_BO))
	{
	    logMsg ("Invalid Address, Card, or Signal in point %s\n", pointname);
	    return (-1);
	}
	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}

	pmod->was_requested = TRUE;

	if (card == PSEUDO2)
	{
	    if (*mask == 0)
	    {
		*mask = masks [(signal % 32)];
	    }
	    switch (signal)
	    {

	    case 2: /* Set the program Load request */
		*pvalue = 0;
		break;

	    case 15: /* reset statistics */
		*pvalue = pmod->cmd_reset_stat;
		break;

	    case 16: /* Force the PLC offline */
		*pvalue = pmod->cmd_force_offline;
		break;

	    case 18: /* What is the backup mode for the line*/
		*pvalue = backup_mode [link];
		break;

	    case 19: /* What is the selected channel for the line */
		*pvalue = channel_select [link];
		break;

	    default:
		return (-1);
	    }
	    return (0);
	}
	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWYOUT:
	    case TILPSTAT:
	    case TILPMD:
		if (shift)
		{
		    *shift = (signal % 16);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 16)];
		    }
		}
		blkix = ((signal / 16) / TINOGRPELEMS);
		elemix = ((signal / 16) % TINOGRPELEMS);
		subelement = 0;
		break;

	    case TIYMEM:
	    case TICRMEM:

		if (shift)
		{
		    *shift = ((signal - 1) % 32);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [((signal - 1) % 32)];
		    }
		}
		blkix = ((signal - 1) / 32);
		elemix = 0;
		subelement = 0;
		break;

	    case TILPCNTL:
	    case TIACFLAG:

		if (shift)
		{
	 	    *shift = (signal % 32);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 32)];
		    }
		}
		blkix = (((signal / 32) - 1) / TINOGRPELEMS);
		elemix = (((signal / 32) - 1) % TINOGRPELEMS);
		subelement = 0;
		break;

	    default:

		if (mltrace [link] >= 2)
		{
		    logMsg ("%s Invalid card type for Binary Output\n", pointname);
		}
		return (-1);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWYOUT:
		if (shift)
		{
		    *shift = (signal % 16);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 16)];
		    }
		}
		blkix = ((signal / 16) / TINOGRPELEMS);
		elemix = ((signal / 16) % TINOGRPELEMS);
		subelement = 0;
		break;

	    case TIYMEM:

		if (shift)
		{
		    *shift = (signal % 32);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 32)];
		    }
		}
		blkix = signal;
		elemix = 0;
		subelement = 0;
		break;

	    case TICRMEM:

		if (shift)
		{
		    *shift = (signal % 32);
		    *mask <<= *shift;
		}
		else
		{
		    if (*mask == 0)
		    {
			*mask = masks [(signal % 32)];
		    }
		}
		blkix = (signal / 32);
		elemix = 0;
		subelement = 0;
		break;


	    default:

		if (mltrace [link] >= 2)
		{
		    logMsg ("%s Invalid card type for Binary Output\n", pointname);
		}
		return (-1);
	    }
	    break;

	default:
	    return (-1);
	}
	if (resname != 0)
	{
	    register_resource (link, pmod, resname, pointname); 
	}
	if (!resresponse [link])
	{
	    return (-1);
	}
	if (pmod->holdflag)
	{
	    return (-2);
	}
	if ((!pmod->is_responding) || (pmod->cmd_force_offline))
	{
	    return (-1);
	}
	if (!setrequested (pmod, card, blkix))
	{
	    return (-1);
	}
	if ((fetchvalue (pmod, card, blkix, elemix, subelement, &uval, 0)) == -1)
	{
	    return (-1);
	}
	*pvalue = (uval.ulval & *mask);
	return (0);
}

/***********
*
*	register_resource
*
*	This routine is used to initialize the resource pointers
*	If this is the first time the resource for this line is being
*	referenced the pointer to the resource table entry will be set
*	and the resource name will be saved. If the resouce name has
*	already been set, this routine will verify that all other points
*	for this driver are using the same resource name.
*
**********/                                                                                    	
/* this routine called from AIDRIVER, AODRIVER, AOREAD, BOREAD,				
			    BIDRIVER, and BODRIVER */                               	
static	void 	register_resource (unsigned short link, TIPLC *pmod,
				unsigned char *resname,
				unsigned char *pointname)
{
	if (resnames [link] [0] == 0) /* This is the first reference */
	{
	    if (resGetByName (resname, &prestable [link]) != RES_OK)
	    {
		logMsg ("Resource not found in table - %s, %s \n",
				pointname,
				resname );
	    }
	    else
	    {
		strcpy (resnames [link], resname);
	    }
	}
	else
	{
	    if (strcmp (resnames [link], resname) != 0)
	    {
		logMsg ("Resource name conflict - %s,  %s, %s \n",
				pointname,
				resnames [link],
				resname );
	    }
	}
}

/**********
*
*	rangeok
*
*	routine to check the boundaries for link adapter card and signal
*
**********/
static	int	rangeok (unsigned short link, unsigned short adapter,
		unsigned short card, unsigned short signal,
		unsigned short iotype)
{
	TIPLC  *pmod;

	pmod = plcs [link] [adapter];
	if (pmod == NULL)
	{
	    if (!allocate_plc (link, adapter))
	    {
		return (-1); /* if not allocated, try. if can't allocate, get out. */
	    }
	    pmod = plcs [link] [adapter];
	}
	if (link >= TILINES)
	{
	    return (FALSE);
	}
	if (adapter >= NUMTIPLCS)
	{
	    return (FALSE);
	}
	switch (card)
        {
	case TIVMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if ((signal > MAXTIVMEM) || (signal == 0))
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if ((signal > ((MAXTIVMEM * 16) + 15)) || (signal < 16))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if (signal > MAXTIVMEM)
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if (signal > (((MAXTIVMEM - 1) * 16) + 15))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;
	    }
	    break;

	case TICMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if ((signal > MAXTICMEM) || (signal == 0))
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if ((signal > ((MAXTICMEM * 16) + 16)) || (signal < 16))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if (signal > MAXTICMEM)
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if (signal > (((MAXTICMEM - 1) * 16) + 15))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;
	    }
	    break;

	case TIXMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if ((signal > MAXTIXMEM) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (signal >= MAXTITXMEM)
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TIYMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if ((signal > MAXTIYMEM) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (signal >= MAXTITYMEM)
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TICRMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if ((signal > MAXTICRMEM) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (signal >= MAXTICRMEM)
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TIWXIN:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if ((signal > MAXTIWXIN) || (signal == 0))
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if ((signal > (((MAXTIWXIN - 1) * 16) + 15)) || (signal < 16))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if (signal >= MAXTIWXIN)
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if (signal > (((MAXTIWXIN - 1) * 16) + 15))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;
	    }
	    break;

	case TIWYOUT:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if ((signal > MAXTIWYOUT) || (signal == 0))
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if ((signal > ((MAXTIWYOUT * 16) + 15)) || (signal < 16))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if (signal >= MAXTIWYOUT)
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if (signal > (((MAXTIWYOUT - 1) * 16) + 15))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;
	    }
	    break;

	case TISTATUS:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if ((signal > MAXTISTATUS) || (signal == 0))
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if ((signal > ((MAXTISTATUS * 16) + 15)) || (signal < 16))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		switch (iotype)
		{
		case TI_AI:
		case TI_AO:
		    if (signal >= MAXTISTATUS)
		    {
			return (FALSE);
		    }
		    break;

		case TI_BI:
		case TI_BO:
		    if (signal > (((MAXTISTATUS - 1) * 16) + 15))
		    {
			return (FALSE);
		    }
		    break;
		}
		break;
	    }
	    break;

	case TILPSTAT:
	    switch (iotype)
	    {
	    case TI_AI:
	    case TI_AO:
		if ((signal > MAXTILPSTAT) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case TI_BI:
	    case TI_BO:
		if ((signal > ((MAXTILPSTAT * 16) + 15)) || (signal < 16))
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TILPCNTL:
	    switch (iotype)
	    {
	    case TI_AI:
	    case TI_AO:
		if ((signal > MAXTILPCNTL) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case TI_BI:
	    case TI_BO:
		if ((signal > ((MAXTILPCNTL * 32) + 31)) || (signal < 32))
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TILPMD:
	    switch (iotype)
	    {
	    case TI_AI:
	    case TI_AO:
		if ((signal > MAXTILPMD) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case TI_BI:
	    case TI_BO:
		if ((signal > ((MAXTILPMD * 16) + 15)) || (signal < 16))
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TIAVFLAG:
	    switch (iotype)
	    {
	    case TI_AI:
	    case TI_AO:
		if ((signal > MAXTIAVFLAG) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case TI_BI:
	    case TI_BO:
		if ((signal > ((MAXTIAVFLAG * 16) + 15)) || (signal < 16))
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TIACFLAG:
	    switch (iotype)
	    {
	    case TI_AI:
	    case TI_AO:
		if ((signal > MAXTIACFLAG) || (signal == 0))
		{
		    return (FALSE);
		}
		break;

	    case TI_BI:
	    case TI_BO:
		if ((signal > ((MAXTIACFLAG * 32) + 31)) || (signal < 32))
		{
		    return (FALSE);
		}
		break;
	    }
	    break;

	case TILPGAIN:
	    if ((signal > MAXTILPGAIN) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPRESET:
	    if ((signal > MAXTILPRESET) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPRATE:
	    if ((signal > MAXTILPRATE) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPHIALRM:
	    if ((signal > MAXTILPHIALRM) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPLOALRM:
	    if ((signal > MAXTILPLOALRM) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPPROCVAR:
	    if ((signal > MAXTILPPROCVAR) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPPVHILIM:
	    if ((signal > MAXTILPPVHILIM) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPPVLOLIM:
	    if ((signal > MAXTILPPVLOLIM) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPORGDEVALRM:
	    if ((signal > MAXTILPORGDEVALRM) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPYELDEVALRM:
	    if ((signal > MAXTILPYELDEVALRM) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPSMPRATE:
	    if ((signal > MAXTILPSMPRATE) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPREMSP:
	    if ((signal > MAXTILPREMSP) || (signal == 0))
	    {
	        return (FALSE);
	    }
	    break;

	case TILPOUTPUT:
	    if ((signal > MAXTILPOUTPUT) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TILPERROR:
	    if ((signal > MAXTILPERROR) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TILPBIAS:
	    if ((signal > MAXTILPBIAS) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TILPPVHIHILIM:
	    if ((signal > MAXTILPPVHIHILIM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TILPPVLOLOLIM:
	    if ((signal > MAXTILPPVLOLOLIM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TILPROCALRM:
	    if ((signal > MAXTILPROCALRM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPVHIALRM:
	    if ((signal > MAXTIANPVHIALRM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPVLOALRM:
	    if ((signal > MAXTIANPVLOALRM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPROCVAR:
	    if ((signal > MAXTIANPROCVAR) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPVHILIM:
	    if ((signal > MAXTIANPVHILIM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPVLOLIM:
	    if ((signal > MAXTIANPVLOLIM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANORGDEVLIM:
	    if ((signal > MAXTIANORGDEVLIM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANYELDEVLIM:
	    if ((signal > MAXTIANYELDEVLIM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANSMPRATE:
	    if ((signal > MAXTIANSMPRATE) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANSETPOINT:
	    if ((signal > MAXTIANSETPOINT) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANERROR:
	    if ((signal > MAXTIANERROR) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPVHIHIALRM:
	    if ((signal > MAXTIANPVHIHIALRM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANPVLOLOALRM:
	    if ((signal > MAXTIANPVLOLOALRM) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case TIANROC:
	    if ((signal > MAXTIANROC) || (signal == 0))
	    {
		return (FALSE);
	    }
	    break;

	case SCANCNTL:
	case PSEUDO1:
	case PSEUDO2:
	    break;

	default:
	    return (FALSE);
	}
	return (TRUE);
}

/**********
*
*	setrequested
*	routine to set the requested flag for the selected block
*
**********/
static	int	setrequested (TIPLC *pmod, unsigned short card, unsigned short blkix)
{
	unsigned	int	i;

	switch (card)
	{
	case TIVMEM:
	    if (!pmod->vmem)
	    {
		pmod->vmem = (VMEMIMG *) calloc (1, ((sizeof (VMEMIMG) * NUMTIVMEM)));
		if (!pmod->vmem)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for VMEM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for VMEM data structure\n");
		    for (i = 0; (i < NUMTIVMEM); i++)
		    {
			pmod->vmem [i].was_requested = FALSE;
			pmod->vmem [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->vmem [blkix].was_requested = TRUE;
	    break;

	case TICMEM:
	    if (!pmod->cmem)
	    {
		pmod->cmem = (CMEMIMG *) calloc (1, ((sizeof (CMEMIMG) * NUMTICMEM)));
		if (!pmod->cmem)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for CMEM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for CMEM data structure\n");
		    for (i = 0; (i < NUMTICMEM); i++)
		    {
			pmod->cmem [i].was_requested = FALSE;
			pmod->cmem [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->cmem [blkix].was_requested = TRUE;
	    break;

	case TIXMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if (!pmod->xmem)
		{
		    pmod->xmem = (XMEMIMG *) calloc (1, ((sizeof (XMEMIMG) * NUMTIXMEM)));
		    if (!pmod->xmem)
		    {
			logMsg ("Unable to allocate memory");
		   	printf (" for XMEM data structure\n");
		   	return (FALSE);
		    }
		    else
		    {
			logMsg ("Allocated memory for XMEM data structure\n");
			for (i = 0; (i < NUMTIXMEM); i++)
			{
			    pmod->xmem [i].was_requested = FALSE;
			    pmod->xmem [i].data_valid = FALSE;
			}
		    }
		}
		pmod->xmem [blkix].was_requested = TRUE;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (!pmod->txmem)
		{
		    pmod->txmem = (TXMEMIMG *) calloc (1, ((sizeof (TXMEMIMG) * 
NUMTITXMEM)));
		    if (!pmod->txmem)
		    {
			logMsg ("Unable to allocate memory");
		   	printf (" for TXMEM data structure\n");
		   	return (FALSE);
		    }
		    else
		    {
			logMsg ("Allocated memory for TXMEM data structure\n");
			for (i = 0; (i < NUMTITXMEM); i++)
			{
			    pmod->txmem [i].was_requested = FALSE;
			    pmod->txmem [i].data_valid = FALSE;
			}
		    }
		}
		pmod->txmem [blkix].was_requested = TRUE;
		break;
	    }
	    break;

	case TIYMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if (!pmod->ymem)
		{
		    pmod->ymem = (YMEMIMG *) calloc (1, ((sizeof (YMEMIMG) * NUMTIYMEM)));
		    if (!pmod->ymem)
		    {
			logMsg ("Unable to allocate memory");
			printf (" for YMEM data structure\n");
		   	return (FALSE);
		    }
		    else
		    {
			logMsg ("Allocated memory for YMEM data structure\n");
			for (i = 0; (i < NUMTIYMEM); i++)
			{
			    pmod->ymem [i].was_requested = FALSE;
			    pmod->ymem [i].data_valid = FALSE;
			}
		    }
		}
		pmod->ymem [blkix].was_requested = TRUE;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (!pmod->tymem)
		{
		    pmod->tymem = (TYMEMIMG *) calloc (1, ((sizeof (TYMEMIMG) * 
NUMTITYMEM)));
		    if (!pmod->tymem)
		    {
			logMsg ("Unable to allocate memory");
			printf (" for TYMEM data structure\n");
		   	return (FALSE);
		    }
		    else
		    {
			logMsg ("Allocated memory for TYMEM data structure\n");
			for (i = 0; (i < NUMTITYMEM); i++)
			{
			    pmod->tymem [i].was_requested = FALSE;
			    pmod->tymem [i].data_valid = FALSE;
			}
		    }
		}
		pmod->tymem [blkix].was_requested = TRUE;
		break;
	    }
	    break;

	case TICRMEM:
	    if (!pmod->crmem)
	    {
		pmod->crmem = (CRMEMIMG *) calloc (1, ((sizeof (CRMEMIMG) * NUMTICRMEM)));
		if (!pmod->crmem)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for CRMEM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for CRMEM data structure\n");
		    for (i = 0; (i < NUMTICRMEM); i++)
		    {
			pmod->crmem [i].was_requested = FALSE;
			pmod->crmem [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->crmem [blkix].was_requested = TRUE;
	    break;

	case TIWXIN:
	    if (!pmod->wxin)
	    {
		pmod->wxin = (WXINIMG *) calloc (1, ((sizeof (WXINIMG) * NUMTIWXIN)));
		if (!pmod->wxin)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for WXIN data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for WXIN data structure\n");
		    for (i = 0; (i < NUMTIWXIN); i++)
		    {
			pmod->wxin [i].was_requested = FALSE;
			pmod->wxin [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->wxin [blkix].was_requested = TRUE;
	    break;

	case TIWYOUT:
	    if (!pmod->wyout)
	    {
		pmod->wyout = (WYOUTIMG *) calloc (1, ((sizeof (WYOUTIMG) * NUMTIWYOUT)));
		if (!pmod->wyout)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for WYOUT data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for WYOUT data structure\n");
		    for (i = 0; (i < NUMTIWYOUT); i++)
		    {
			pmod->wyout [i].was_requested = FALSE;
			pmod->wyout [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->wyout [blkix].was_requested = TRUE;
	    break;

	case TISTATUS:
	    if (!pmod->status)
	    {
		pmod->status = (STATUSIMG *) calloc (1, ((sizeof (STATUSIMG) * NUMTISTATUS)));
		if (!pmod->status)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for STATUS data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for STATUS data structure\n");
		    for (i = 0; (i < NUMTISTATUS); i++)
		    {
			pmod->status [i].was_requested = FALSE;
			pmod->status [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->status [blkix].was_requested = TRUE;
	    break;

	case TILPSTAT:
	    if (!pmod->lpstat)
	    {
		pmod->lpstat = (LPSTATIMG *) calloc (1, ((sizeof (LPSTATIMG) * NUMTILPSTAT)));
		if (!pmod->lpstat)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPSTAT data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPSTAT data structure\n");
		    for (i = 0; (i < NUMTILPSTAT); i++)
		    {
			pmod->lpstat [i].was_requested = FALSE;
			pmod->lpstat [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpstat [blkix].was_requested = TRUE;
	    break;

	case TILPCNTL:
	    if (!pmod->lpcntl)
	    {
		pmod->lpcntl = (LPCNTLIMG *) calloc (1, ((sizeof (LPCNTLIMG) * NUMTILPCNTL)));
		if (!pmod->lpcntl)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPCNTL data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPCNTL data structure\n");
		    for (i = 0; (i < NUMTILPCNTL); i++)
		    {
			pmod->lpcntl [i].was_requested = FALSE;
			pmod->lpcntl [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpcntl [blkix].was_requested = TRUE;
	    break;

	case TILPMD:
	    if (!pmod->lpmd)
	    {
		pmod->lpmd = (LPMDIMG *) calloc (1, ((sizeof (LPMDIMG) * NUMTILPMD)));
		if (!pmod->lpmd)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPMD data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPMD data structure\n");
		    for (i = 0; (i < NUMTILPMD); i++)
		    {
			pmod->lpmd [i].was_requested = FALSE;
			pmod->lpmd [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpmd [blkix].was_requested = TRUE;
	    break;

	case TIAVFLAG:
	    if (!pmod->avflag)
	    {
		pmod->avflag = (AVFLAGIMG *) calloc (1, ((sizeof (AVFLAGIMG) * NUMTIAVFLAG)));
		if (!pmod->avflag)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for AVFLAG data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for AVFLAG data structure\n");
		    for (i = 0; (i < NUMTIAVFLAG); i++)
		    {
			pmod->avflag [i].was_requested = FALSE;
			pmod->avflag [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->avflag [blkix].was_requested = TRUE;
	    break;

	case TIACFLAG:
	    if (!pmod->acflag)
	    {
		pmod->acflag = (ACFLAGIMG *) calloc (1, ((sizeof (ACFLAGIMG) * NUMTIACFLAG)));
		if (!pmod->acflag)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ACFLAG data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ACFLAG data structure\n");
		    for (i = 0; (i < NUMTIACFLAG); i++)
		    {
			pmod->acflag [i].was_requested = FALSE;
			pmod->acflag [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->acflag [blkix].was_requested = TRUE;
	    break;

	case TILPGAIN:
	    if (!pmod->lpgain)
	    {
		pmod->lpgain = (LPGAINIMG *) calloc (1, ((sizeof (LPGAINIMG) * NUMTILPGAIN)));
		if (!pmod->lpgain)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPGAIN data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPGAIN data structure\n");
		    for (i = 0; (i < NUMTILPGAIN); i++)
		    {
			pmod->lpgain [i].was_requested = FALSE;
			pmod->lpgain [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpgain [blkix].was_requested = TRUE;
	    break;

	case TILPRESET:
	    if (!pmod->lpreset)
	    {
		pmod->lpreset = (LPRESETIMG *) calloc (1, ((sizeof (LPRESETIMG) * NUMTILPRESET)));
		if (!pmod->lpreset)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPRESET data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPRESET data structure\n");
		    for (i = 0; (i < NUMTILPRESET); i++)
		    {
			pmod->lpreset [i].was_requested = FALSE;
			pmod->lpreset [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpreset [blkix].was_requested = TRUE;
	    break;

	case TILPRATE:
	    if (!pmod->lprate)
	    {
		pmod->lprate = (LPRATEIMG *) calloc (1, ((sizeof (LPRATEIMG) * NUMTILPRATE)));
		if (!pmod->lprate)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPRATE data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPRATE data structure\n");
		    for (i = 0; (i < NUMTILPRATE); i++)
		    {
			pmod->lprate [i].was_requested = FALSE;
			pmod->lprate [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lprate [blkix].was_requested = TRUE;
	    break;

	case TILPHIALRM:
	    if (!pmod->lphialrm)
	    {
		pmod->lphialrm = (LPHIALRMIMG *) calloc (1, ((sizeof (LPHIALRMIMG) * NUMTILPHIALRM)));
		if (!pmod->lphialrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPHIALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPHIALRM data structure\n");
		    for (i = 0; (i < NUMTILPHIALRM); i++)
		    {
			pmod->lphialrm [i].was_requested = FALSE;
			pmod->lphialrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lphialrm [blkix].was_requested = TRUE;
	    break;

	case TILPLOALRM:
	    if (!pmod->lploalrm)
	    {
		pmod->lploalrm = (LPLOALRMIMG *) calloc (1, ((sizeof (LPLOALRMIMG) * NUMTILPLOALRM)));
		if (!pmod->lploalrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPLOALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPLOALRM data structure\n");
		    for (i = 0; (i < NUMTILPLOALRM); i++)
		    {
			pmod->lploalrm [i].was_requested = FALSE;
			pmod->lploalrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lploalrm [blkix].was_requested = TRUE;
	    break;

	case TILPPROCVAR:
	    if (!pmod->lpprocvar)
	    {
		pmod->lpprocvar = (LPPROCVARIMG *) calloc (1, ((sizeof (LPPROCVARIMG) * NUMTILPPROCVAR)));
		if (!pmod->lpprocvar)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPPROCVAR data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPPROCVAR data structure\n");
		    for (i = 0; (i < NUMTILPPROCVAR); i++)
		    {
			pmod->lpprocvar [i].was_requested = FALSE;
			pmod->lpprocvar [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpprocvar [blkix].was_requested = TRUE;
	    break;

	case TILPPVHILIM:
	    if (!pmod->lppvhilim)
	    {
		pmod->lppvhilim = (LPPVHILIMIMG *) calloc (1, ((sizeof (LPPVHILIMIMG) * NUMTILPPVHILIM)));
		if (!pmod->lppvhilim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPPVHILIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPPVHILIM data structure\n");
		    for (i = 0; (i < NUMTILPPVHILIM); i++)
		    {
			pmod->lppvhilim [i].was_requested = FALSE;
			pmod->lppvhilim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lppvhilim [blkix].was_requested = TRUE;
	    break;

	case TILPPVLOLIM:
	    if (!pmod->lppvlolim)
	    {
		pmod->lppvlolim = (LPPVLOLIMIMG *) calloc (1, ((sizeof (LPPVLOLIMIMG) * NUMTILPPVLOLIM)));
		if (!pmod->lppvlolim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPPVLOLIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPPVLOLIM data structure\n");
		    for (i = 0; (i < NUMTILPPVLOLIM); i++)
		    {
			pmod->lppvlolim [i].was_requested = FALSE;
			pmod->lppvlolim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lppvlolim [blkix].was_requested = TRUE;
	    break;

	case TILPORGDEVALRM:
	    if (!pmod->lporgdevalrm)
	    {
		pmod->lporgdevalrm = (LPORGDEVALRMIMG *) calloc (1, ((sizeof (LPORGDEVALRMIMG) * NUMTILPORGDEVALRM)));
		if (!pmod->lporgdevalrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPORGDEVALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPORGDEVALRM data structure\n");
		    for (i = 0; (i < NUMTILPORGDEVALRM); i++)
		    {
			pmod->lporgdevalrm [i].was_requested = FALSE;
			pmod->lporgdevalrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lporgdevalrm [blkix].was_requested = TRUE;
	    break;

	case TILPYELDEVALRM:
	    if (!pmod->lpyeldevalrm)
	    {
		pmod->lpyeldevalrm = (LPYELDEVALRMIMG *) calloc (1, ((sizeof (LPYELDEVALRMIMG) * NUMTILPYELDEVALRM)));
		if (!pmod->lpyeldevalrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPYELDEVALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPYELDEVALRM data structure\n");
		    for (i = 0; (i < NUMTILPYELDEVALRM); i++)
		    {
			pmod->lpyeldevalrm [i].was_requested = FALSE;
			pmod->lpyeldevalrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpyeldevalrm [blkix].was_requested = TRUE;
	    break;

	case TILPSMPRATE:
	    if (!pmod->lpsmprate)
	    {
		pmod->lpsmprate = (LPSMPRATEIMG *) calloc (1, ((sizeof (LPSMPRATEIMG) * NUMTILPSMPRATE)));
		if (!pmod->lpsmprate)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPSMPRATE data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPSMPRATE data structure\n");
		    for (i = 0; (i < NUMTILPSMPRATE); i++)
		    {
			pmod->lpsmprate [i].was_requested = FALSE;
			pmod->lpsmprate [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpsmprate [blkix].was_requested = TRUE;
	    break;

	case TILPREMSP:
	    if (!pmod->lpremsp)
	    {
		pmod->lpremsp = (LPREMSPIMG *) calloc (1, ((sizeof (LPREMSPIMG) * NUMTILPREMSP)));
		if (!pmod->lpremsp)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPREMSP data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPREMSP data structure\n");
		    for (i = 0; (i < NUMTILPREMSP); i++)
		    {
			pmod->lpremsp [i].was_requested = FALSE;
			pmod->lpremsp [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpremsp [blkix].was_requested = TRUE;
	    break;

	case TILPOUTPUT:
	    if (!pmod->lpoutput)
	    {
		pmod->lpoutput = (LPOUTPUTIMG *) calloc (1, ((sizeof (LPOUTPUTIMG) * NUMTILPOUTPUT)));
		if (!pmod->lpoutput)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPOUTPUT data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPOUTPUT data structure\n");
		    for (i = 0; (i < NUMTILPOUTPUT); i++)
		    {
			pmod->lpoutput [i].was_requested = FALSE;
			pmod->lpoutput [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpoutput [blkix].was_requested = TRUE;
	    break;

	case TILPERROR:
	    if (!pmod->lperror)
	    {
		pmod->lperror = (LPERRORIMG *) calloc (1, ((sizeof (LPERRORIMG) * NUMTILPERROR)));
		if (!pmod->lperror)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPERROR data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPERROR data structure\n");
		    for (i = 0; (i < NUMTILPERROR); i++)
		    {
			pmod->lperror [i].was_requested = FALSE;
			pmod->lperror [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lperror [blkix].was_requested = TRUE;
	    break;

	case TILPBIAS:
	    if (!pmod->lpbias)
	    {
		pmod->lpbias = (LPBIASIMG *) calloc (1, ((sizeof (LPBIASIMG) * NUMTILPBIAS)));
		if (!pmod->lpbias)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPBIAS data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPBIAS data structure\n");
		    for (i = 0; (i < NUMTILPBIAS); i++)
		    {
			pmod->lpbias [i].was_requested = FALSE;
			pmod->lpbias [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lpbias [blkix].was_requested = TRUE;
	    break;

	case TILPPVHIHILIM:
	    if (!pmod->lppvhihilim)
	    {
		pmod->lppvhihilim = (LPPVHIHILIMIMG *) calloc (1, ((sizeof (LPPVHIHILIMIMG) * NUMTILPPVHIHILIM)));
		if (!pmod->lppvhihilim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPPVHIHILIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPPVHIHILIM data structure\n");
		    for (i = 0; (i < NUMTILPPVHIHILIM); i++)
		    {
			pmod->lppvhihilim [i].was_requested = FALSE;
			pmod->lppvhihilim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lppvhihilim [blkix].was_requested = TRUE;
	    break;

	case TILPPVLOLOLIM:
	    if (!pmod->lppvlololim)
	    {
		pmod->lppvlololim = (LPPVLOLOLIMIMG *) calloc (1, ((sizeof (LPPVLOLOLIMIMG) * NUMTILPPVLOLOLIM)));
		if (!pmod->lppvlololim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPPVLOLOLIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPPVLOLOLIM data structure\n");
		    for (i = 0; (i < NUMTILPPVLOLOLIM); i++)
		    {
			pmod->lppvlololim [i].was_requested = FALSE;
			pmod->lppvlololim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lppvlololim [blkix].was_requested = TRUE;
	    break;

	case TILPROCALRM:
	    if (!pmod->lprocalrm)
	    {
		pmod->lprocalrm = (LPROCALRMIMG *) calloc (1, ((sizeof (LPROCALRMIMG) * NUMTILPROCALRM)));
		if (!pmod->lprocalrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for LPROCALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for LPROCALRM data structure\n");
		    for (i = 0; (i < NUMTILPROCALRM); i++)
		    {
			pmod->lprocalrm [i].was_requested = FALSE;
			pmod->lprocalrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->lprocalrm [blkix].was_requested = TRUE;
	    break;

	case TIANPVHIALRM:
	    if (!pmod->anpvhialrm)
	    {
		pmod->anpvhialrm = (ANPVHIALRMIMG *) calloc (1, ((sizeof (ANPVHIALRMIMG) * NUMTIANPVHIALRM)));
		if (!pmod->anpvhialrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANPVHIALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPVHIALRM data structure\n");
		    for (i = 0; (i < NUMTIANPVHIALRM); i++)
		    {
			pmod->anpvhialrm [i].was_requested = FALSE;
			pmod->anpvhialrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anpvhialrm [blkix].was_requested = TRUE;
	    break;

	case TIANPVLOALRM:
	    if (!pmod->anpvloalrm)
	    {
		pmod->anpvloalrm = (ANPVLOALRMIMG *) calloc (1, ((sizeof (ANPVLOALRMIMG) * NUMTIANPVLOALRM)));
		if (!pmod->anpvloalrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for AVPVLOALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPVLOALRM data structure\n");
		    for (i = 0; (i < NUMTIANPVLOALRM); i++)
		    {
			pmod->anpvloalrm [i].was_requested = FALSE;
			pmod->anpvloalrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anpvloalrm [blkix].was_requested = TRUE;
	    break;

	case TIANPROCVAR:
	    if (!pmod->anprocvar)
	    {
		pmod->anprocvar = (ANPROCVARIMG *) calloc (1, ((sizeof (ANPROCVARIMG) * NUMTIANPROCVAR)));
		if (!pmod->anprocvar)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANPROCVAR data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPROCVAR data structure\n");
		    for (i = 0; (i < NUMTIANPROCVAR); i++)
		    {
			pmod->anprocvar [i].was_requested = FALSE;
			pmod->anprocvar [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anprocvar [blkix].was_requested = TRUE;
	    break;

	case TIANPVHILIM:
	    if (!pmod->anpvhilim)
	    {
		pmod->anpvhilim = (ANPVHILIMIMG *) calloc (1, ((sizeof (ANPVHILIMIMG) * NUMTIANPVHILIM)));
		if (!pmod->anpvhilim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANPVHILIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPVHILIM data structure\n");
		    for (i = 0; (i < NUMTIANPVHILIM); i++)
		    {
			pmod->anpvhilim [i].was_requested = FALSE;
			pmod->anpvhilim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anpvhilim [blkix].was_requested = TRUE;
	    break;

	case TIANPVLOLIM:
	    if (!pmod->anpvlolim)
	    {
		pmod->anpvlolim = (ANPVLOLIMIMG *) calloc (1, ((sizeof (ANPVLOLIMIMG) * NUMTIANPVLOLIM)));
		if (!pmod->anpvlolim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANPVLOLIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPVLOLIM data structure\n");
		    for (i = 0; (i < NUMTIANPVLOLIM); i++)
		    {
			pmod->anpvlolim [i].was_requested = FALSE;
			pmod->anpvlolim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anpvlolim [blkix].was_requested = TRUE;
	    break;

	case TIANORGDEVLIM:
	    if (!pmod->anorgdevlim)
	    {
		pmod->anorgdevlim = (ANORGDEVLIMIMG *) calloc (1, ((sizeof (ANORGDEVLIMIMG) * NUMTIANORGDEVLIM)));
		if (!pmod->anorgdevlim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANORGDEVLIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANORGDEVLIM data structure\n");
		    for (i = 0; (i < NUMTIANORGDEVLIM); i++)
		    {
			pmod->anorgdevlim [i].was_requested = FALSE;
			pmod->anorgdevlim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anorgdevlim [blkix].was_requested = TRUE;
	    break;

	case TIANYELDEVLIM:
	    if (!pmod->anyeldevlim)
	    {
		pmod->anyeldevlim = (ANYELDEVLIMIMG *) calloc (1, ((sizeof (ANYELDEVLIMIMG) * NUMTIANYELDEVLIM)));
		if (!pmod->anyeldevlim)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANYELDEVLIM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANYELDEVLIM data structure\n");
		    for (i = 0; (i < NUMTIANYELDEVLIM); i++)
		    {
			pmod->anyeldevlim [i].was_requested = FALSE;
			pmod->anyeldevlim [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anyeldevlim [blkix].was_requested = TRUE;
	    break;

	case TIANSMPRATE:
	    if (!pmod->ansmprate)
	    {
		pmod->ansmprate = (ANSMPRATEIMG *) calloc (1, ((sizeof (ANSMPRATEIMG) * NUMTIANSMPRATE)));
		if (!pmod->ansmprate)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANSMPRATE data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANSMPRATE data structure\n");
		    for (i = 0; (i < NUMTIANSMPRATE); i++)
		    {
			pmod->ansmprate [i].was_requested = FALSE;
			pmod->ansmprate [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->ansmprate [blkix].was_requested = TRUE;
	    break;

	case TIANSETPOINT:
	    if (!pmod->ansetpoint)
	    {
		pmod->ansetpoint = (ANSETPOINTIMG *) calloc (1, ((sizeof (ANSETPOINTIMG) * NUMTIANSETPOINT)));
		if (!pmod->ansetpoint)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANSETPOINT data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANSETPOINT data structure\n");
		    for (i = 0; (i < NUMTIANSETPOINT); i++)
		    {
			pmod->ansetpoint [i].was_requested = FALSE;
			pmod->ansetpoint [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->ansetpoint [blkix].was_requested = TRUE;
	    break;

	case TIANERROR:
	    if (!pmod->anerror)
	    {
		pmod->anerror = (ANERRORIMG *) calloc (1, ((sizeof (ANERRORIMG) * NUMTIANERROR)));
		if (!pmod->anerror)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANERROR data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANERROR data structure\n");
		    for (i = 0; (i < NUMTIANERROR); i++)
		    {
			pmod->anerror [i].was_requested = FALSE;
			pmod->anerror [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anerror [blkix].was_requested = TRUE;
	    break;

	case TIANPVHIHIALRM:
	    if (!pmod->anpvhihialrm)
	    {
		pmod->anpvhihialrm = (ANPVHIHIALRMIMG *) calloc (1, ((sizeof (ANPVHIHIALRMIMG) * NUMTIANPVHIHIALRM)));
		if (!pmod->anpvhihialrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANPVHIHIALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPVHIHIALRM data structure\n");
		    for (i = 0; (i < NUMTIANPVHIHIALRM); i++)
		    {
			pmod->anpvhihialrm [i].was_requested = FALSE;
			pmod->anpvhihialrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anpvhihialrm [blkix].was_requested = TRUE;
	    break;

	case TIANPVLOLOALRM:
	    if (!pmod->anpvloloalrm)
	    {
		pmod->anpvloloalrm = (ANPVLOLOALRMIMG *) calloc (1, ((sizeof (ANPVLOLOALRMIMG) * NUMTIANPVLOLOALRM)));
		if (!pmod->anpvloloalrm)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANPVLOLOALRM data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANPVLOLOALRM data structure\n");
		    for (i = 0; (i < NUMTIANPVLOLOALRM); i++)
		    {
			pmod->anpvloloalrm [i].was_requested = FALSE;
			pmod->anpvloloalrm [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anpvloloalrm [blkix].was_requested = TRUE;
	    break;

	case TIANROC:
	    if (!pmod->anroc)
	    {
		pmod->anroc = (ANROCIMG *) calloc (1, ((sizeof (ANROCIMG) * NUMTIANROC)));
		if (!pmod->anroc)
		{
		    logMsg ("Unable to allocate memory");
		    printf (" for ANROC data structure\n");
		    return (FALSE);
		}
		else
		{
		    logMsg ("Allocated memory for ANROC data structure\n");
		    for (i = 0; (i < NUMTIANROC); i++)
		    {
			pmod->anroc [i].was_requested = FALSE;
			pmod->anroc [i].data_valid = FALSE;
		    }
		}
	    }
	    pmod->anroc [blkix].was_requested = TRUE;
	    break;

	}
	return (TRUE);
}

/**********
*
*	fetchvalue
*	routine to get a value from the TI data structures
*
**********/
static	int  fetchvalue (TIPLC *pmod, unsigned short card,
		 unsigned short blkix, unsigned short elemix,
		 unsigned short subelement, TI_RET_VAL *pvalue,
		 unsigned short conversion)
{
	unsigned	char	i;

	switch (card)
	{
	case TIVMEM:
	    if (!pmod->vmem [blkix].data_valid)
	    {
		return (-1);
	    }
	    if ((conversion == CNVDOUBLEINT) || (conversion == CNVFLOAT))
	    {
		pvalue->ulval = (pmod->vmem [blkix].vmem [elemix] << 16);
		if (elemix == 3)
		{
		    if (pmod->vmem [blkix + 1].data_valid)
		    {
			pvalue->ulval += pmod->vmem [blkix + 1].vmem [0];
		    }
		    else
		    {
			return (-1);
		    }
		}
		else
		{
		    pvalue->ulval += pmod->vmem [blkix].vmem [elemix + 1];
		}
	    }
	    else
	    {
		pvalue->sval [0] = pmod->vmem [blkix].vmem [elemix];
	    }
	    break;

	case TICMEM:
	    if (!pmod->cmem [blkix].data_valid)
	    {
		return (-1);
	    }
	    if ((conversion == CNVDOUBLEINT) || (conversion == CNVFLOAT))
	    {
		pvalue->ulval = (pmod->cmem [blkix].cmem [elemix] << 16);
		if (elemix == 3)
		{
		    if (pmod->cmem [blkix + 1].data_valid)
		    {
			pvalue->ulval += pmod->cmem [blkix + 1].cmem [0];
		    }
		    else
		    {
			return (-1);
		    }
		}
		else
		{
		    pvalue->ulval += pmod->cmem [blkix].cmem [elemix + 1];
		}
	    }
	    else
	    {
		pvalue->sval [0] = pmod->cmem [blkix].cmem [elemix];
	    }
	    break;

	case TIXMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if (!pmod->xmem [blkix].data_valid)
		{
		    return (-1);
		}
		pvalue->ulval = pmod->xmem [blkix].xmem;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (!pmod->txmem [blkix].data_valid)
		{
		    return (-1);
		}
		pvalue->ulval = 0;
		for (i = 0; i < NUMTITXMEM; i++) {
		    pvalue->ulval |= (pmod->txmem [i].txmem << i);
		}
		break;
	    }
	    break;

	case TIYMEM:
	    switch (pmod->dev_type)
	    {
	    case PLC_5TI:
	    case PLC_510:
	    case PLC_520:
	    case PLC_520C:
	    case PLC_525:
	    case PLC_530:
	    case PLC_530C:
	    case PLC_540:
	    case PLC_550:
	    case PLC_560:
	    case PLC_565:
	    case PLC_570:
	    case UHA:
	    case PM550:
	    case PM551:
	    case UA:
		if (!pmod->ymem [blkix].data_valid)
		{
		    return (-1);
		}
		pvalue->ulval = pmod->ymem [blkix].ymem;
		break;

	    case IT_111_121:
	    case IT_150:
	    case IT_160:
		if (!pmod->tymem [blkix].data_valid)
		{
		    return (-1);
		}
		pvalue->ulval = 0;
		for (i = 0; i < NUMTITXMEM; i++) {
		    pvalue->ulval |= (pmod->tymem [i].tymem << i);
		}
	    }
	    break;

	case TICRMEM:
	    if (!pmod->crmem [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->crmem [blkix].crmem;
	    break;

	case TIWXIN:
	    if (!pmod->wxin [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->usval [0] = pmod->wxin [blkix].wxin [elemix];
	    break;

	case TIWYOUT:
	    if (!pmod->wyout [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->usval [0] = pmod->wyout [blkix].wyout [elemix];
	    break;

	case TISTATUS:
	    if (!pmod->status [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->usval [0] = pmod->status [blkix].status [elemix];
	    break;

	case TILPSTAT:
	    if (!pmod->lpstat [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->usval [0] = pmod->lpstat [blkix].lpstat [elemix];
	    break;

	case TILPCNTL:
	    if (!pmod->lpcntl [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpcntl [blkix].lpcntl [elemix];
	    break;

	case TILPMD:
	    if (!pmod->lpmd [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->usval [0] = pmod->lpmd [blkix].lpmd [elemix];
	    break;

	case TIAVFLAG:
	    if (!pmod->avflag [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->usval [0] = pmod->avflag [blkix].avflag [elemix];
	    break;

	case TIACFLAG:
	    if (!pmod->acflag [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->acflag [blkix].acflag [elemix];
	    break;

	case TILPGAIN:
	    if (!pmod->lpgain [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpgain [blkix].lpgain [elemix];
	    break;

	case TILPRESET:
	    if (!pmod->lpreset [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpreset [blkix].lpreset [elemix];
	    break;

	case TILPRATE:
	    if (!pmod->lprate [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lprate [blkix].lprate [elemix];
	    break;

	case TILPHIALRM:
	    if (!pmod->lphialrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lphialrm [blkix].lphialrm [elemix];
	    break;

	case TILPLOALRM:
	    if (!pmod->lploalrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lploalrm [blkix].lploalrm [elemix];
	    break;

	case TILPPROCVAR:
	    if (!pmod->lpprocvar [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpprocvar [blkix].lpprocvar [elemix];
	    break;

	case TILPPVHILIM:
	    if (!pmod->lppvhilim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lppvhilim [blkix].lppvhilim [elemix];
	    break;

	case TILPPVLOLIM:
	    if (!pmod->lppvlolim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lppvlolim [blkix].lppvlolim [elemix];
	    break;

	case TILPORGDEVALRM:
	    if (!pmod->lporgdevalrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lporgdevalrm [blkix].lporgdevalrm [elemix];
	    break;

	case TILPYELDEVALRM:
	    if (!pmod->lpyeldevalrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpyeldevalrm [blkix].lpyeldevalrm [elemix];
	    break;

	case TILPSMPRATE:
	    if (!pmod->lpsmprate [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpsmprate [blkix].lpsmprate [elemix];
	    break;

	case TILPREMSP:
	    if (!pmod->lpremsp [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpremsp [blkix].lpremsp [elemix];
	    break;

	case TILPOUTPUT:
	    if (!pmod->lpoutput [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpoutput [blkix].lpoutput [elemix];
	    break;

	case TILPERROR:
	    if (!pmod->lperror [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lperror [blkix].lperror [elemix];
	    break;

	case TILPBIAS:
	    if (!pmod->lpbias [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lpbias [blkix].lpbias [elemix];
	    break;

	case TILPPVHIHILIM:
	    if (!pmod->lppvhihilim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lppvhihilim [blkix].lppvhihilim [elemix];
	    break;

	case TILPPVLOLOLIM:
	    if (!pmod->lppvlololim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lppvlololim [blkix].lppvlololim [elemix];
	    break;

	case TILPROCALRM:
	    if (!pmod->lprocalrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->lprocalrm [blkix].lprocalrm [elemix];
	    break;

	case TIANPVHIALRM:
	    if (!pmod->anpvhialrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anpvhialrm [blkix].anpvhialrm [elemix];
	    break;

	case TIANPVLOALRM:
	    if (!pmod->anpvloalrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anpvloalrm [blkix].anpvloalrm [elemix];
	    break;

	case TIANPROCVAR:
	    if (!pmod->anprocvar [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anprocvar [blkix].anprocvar [elemix];
	    break;

	case TIANPVHILIM:
	    if (!pmod->anpvhilim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anpvhilim [blkix].anpvhilim [elemix];
	    break;

	case TIANPVLOLIM:
	    if (!pmod->anpvlolim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anpvlolim [blkix].anpvlolim [elemix];
	    break;

	case TIANORGDEVLIM:
	    if (!pmod->anorgdevlim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anorgdevlim [blkix].anorgdevlim [elemix];
	    break;

	case TIANYELDEVLIM:
	    if (!pmod->anyeldevlim [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anyeldevlim [blkix].anyeldevlim [elemix];
	    break;

	case TIANSMPRATE:
	    if (!pmod->ansmprate [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->ansmprate [blkix].ansmprate [elemix];
	    break;

	case TIANSETPOINT:
	    if (!pmod->ansetpoint [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->ansetpoint [blkix].ansetpoint [elemix];
	    break;

	case TIANERROR:
	    if (!pmod->anerror [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anerror [blkix].anerror [elemix];
	    break;

	case TIANPVHIHIALRM:
	    if (!pmod->anpvhihialrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anpvhihialrm [blkix].anpvhihialrm [elemix];
	    break;

	case TIANPVLOLOALRM:
	    if (!pmod->anpvloloalrm [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anpvloloalrm [blkix].anpvloloalrm [elemix];
	    break;

	case TIANROC:
	    if (!pmod->anroc [blkix].data_valid)
	    {
		return (-1);
	    }
	    pvalue->ulval = pmod->anroc [blkix].anroc [elemix];
	    break;;

	default:
	    return (-1);
	}
	return (0);
}

/**********
*
*      Initialize the Queue
*
**********/

static	void	outqueinit (unsigned short line)
{
	oq [line]->ooutix = oq [line]->oinix = 0;
}

/**********
*
*  ogetempty - get the next available slot on the output list
*
**********/

static	OUTQENT	*ogetemptyp (unsigned short line)
{
	if (((oq [line]->oinix + 1) % OUTQSIZE) != oq [line]->ooutix)
	{
	    return (&oq [line]->outque [oq [line]->oinix]);
	}
	else
	{
	    return (NULLP);
	}
}

/**********
*
* omarkfull - mark the next block as full
*
**********/

static	void	omarkfull (unsigned short line)
{
	oq [line]->oinix = ((oq [line]->oinix + 1) % OUTQSIZE);
}

/**********
*
*  ogetfullp - get the next full slot on the output list
*
**********/

static	OUTQENT	*ogetnextp (unsigned short line)
{
	if (oq [line]->ooutix != oq [line]->oinix)
	{
	    return (&oq [line]->outque [oq [line]->ooutix]);
	}
	else
	{
	    return (NULLP);
	}
}

/**********
*
* oreleasefull - release the current full output block
*
**********/

static	void	oreleasefull (unsigned short line)
{
	if (oq [line]->ooutix == oq [line]->oinix)
	{
	    return;
	}
	oq [line]->ooutix = ((oq [line]->ooutix + 1) % OUTQSIZE);
}

/***********
*
*      link_status
*
*      The purpose of this routine is to print out the state of each of the
*      TIWAY lines and the statistics gathered about them.
*
**********/

static	void	link_status (unsigned short line)
{
	if (line > TILINES - 1) {
	    return;
	}
}

/**********
*                                                                              
*	allocate_plc                                                            
*                                                                              
*	The purpose of this routine is to allocate the memory for a single      
*	plc data structure. If not successful, a console error message is  	
*	printed and an error flag is returned to the caller.                    
*	If successful, the newly allocated data structures are initialized.     
*                                                                             	
**********/

static	int	allocate_plc (unsigned short line, unsigned short plc )
{
	plcs [line] [plc] = (TIPLC *) calloc (1, (sizeof (TIPLC)));
	if (!plcs [line] [plc])
	{
	    logMsg ("Unable to allocate memory");
	    printf (" for TIPLC data structure on line %d, plc %d\n",line ,plc);
	    return (FALSE);
	}
	else
	{
	    logMsg ("Allocated memory for TIPLC data structure on line %d, plc %d\n",line ,plc);
	    init_plc_tab (line, plc);	     /*	initialize the data structures. */
	}
	return (TRUE);
}

/**********
*
*      startrx
*
*      The purpose of this routine is to initialize a line table
*      for receive and to enable the receiver./*
*
**********/

static	void	start_rx (unsigned short line, SEM_ID *semid,
			unsigned short *pstatus )
{

	RXLINE                  *rxp;

	disable_rx_ints (line);
	rxp = rxlinetab [line];
	rxp->rxstate = R_COLON;
	rxp->rxix = 0;
	rxp->semid = semid;
	rxp->status = pstatus;
	enable_rx_ints (line);

}

/**********
*
*      stoprx
*
*      The purpose of this routine is to initialize a line table
*      for receive and to disable the receiver.
*
**********/

static	void	stop_rx (unsigned short line)
{
	RXLINE                  *rxp;

	rxp = rxlinetab [line];
	rxp->rxstate = R_IGNORE;

}

/**********
*
*      rx_int_proc
*
*      The purpose of this routine is to process a receive character interrupt
*      for a given line. (Protocol Specific)
*
**********/

#ifndef TEST

static	void	rx_int_proc (unsigned short line)

#endif

#ifdef TEST

	void	rx_int_proc (unsigned short line)

#endif

{
	unsigned	char	rxchar;
	unsigned 	short	rxstat;
	unsigned 	short	rx_err_mask;
	RXLINE          	*rxp;

	read_rx_hw (line, &rxchar, &rxstat, &rx_err_mask);
	rxp = rxlinetab [line];

	if (rxp->rxstate == R_IGNORE)
	{
	    return;
	}
	if ((rxstat & rx_err_mask) != 0)
	{
	    *(rxp->status) = RXBAD;
	    rxp->rxbuf [299] = rxstat;
	    rxp->rxbuf [300] = rxchar;
	    rxp->rxstate = R_IGNORE;
	    return;
	}

	switch (rxp->rxstate)
	{

	    case R_COLON:
		if (rxchar == ':')
		{
		    rxp->rxstate = R_COUNT1;
		}
		break;

	    case R_COUNT1:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_COUNT2;
		break;

	    case R_COUNT2:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_COUNT3;
		break;

	    case R_COUNT3:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_COUNT4;
		break;

	    case R_COUNT4:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		nitp_ASCII2bin (&rxp->rxbuf [rxp->rxix - 4],
				(UCHAR *) &rxp->rxmsglen,
				4);
		rxp->rxmsglen -= 10;			/* already rcvd ; C1 C2 C3 C4 */
		rxp->rxstate = R_DATA;
		break;

	    case R_DATA:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		if (--rxp->rxmsglen == 0)
		{
		    rxp->rxstate = R_ECC1;
		}
		break;

	    case R_ECC1:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_ECC2;
		break;

	    case R_ECC2:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_ECC3;
		break;

	    case R_ECC3:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_ECC4;
		break;

	    case R_ECC4:
		rxp->rxbuf [rxp->rxix++] = rxchar;
		rxp->rxstate = R_SEMICOLON;
		break;

	    case R_SEMICOLON:
		rxp->rxbuf [rxp->rxix++] = 0;
		rxp->rxstate = R_CR;
		break;

	    case R_CR:
		rxp->rxstate = R_LF;
		break;

	    case R_LF:
		rxp->rxstate = R_IGNORE;
		*(rxp->status) = RXGOOD;
		semGive (rxp->semid);
		break;

	    default:  /* Should Never Get Here */
		rxp->rxstate = R_IGNORE;
		return;
	}
}

/**********
*
*      check_line_activity
*                                                                             	
*      The purpose of this routine is to process a receive character interrupt	
*      for a given line.                                                      	
*                                                                             	
**********/

/* only called from MAIN  */

static	int	check_line_activity (unsigned short line, unsigned short predelay)
{
	RXLINE          *rxp;
	long		thistime;
	short		status;

	if (predelay != 0)
	{
	    taskDelay (predelay);
	}
	thistime = tickGet ();
	if ((thistime - activity_time [line]) < ACTIVITYTIME)
	{
	    return (lastactivity [line] || (line_activity [line]));
	}
	lastactivity [line] = status = line_activity [line];
	line_activity [line] = FALSE;
	activity_time [line] = thistime;
	rxp = rxlinetab [line];
	rxp->rxstate = R_IGNORE;
	return (status);
}

/**********
*
*      starttx
*
*      The purpose of this routine is to initialize a line table
*      for send and to enable the transmitter.
*
**********/
static	void	start_tx (unsigned short line, SEM_ID *semid,
			unsigned short *pstatus )
{
	TXLINE                  *txp;
	disable_tx_ints (line);
	txp = txlinetab [line];
	txp->txix = 0;
	txp->semid = semid;
	txp->status = pstatus;
	txp->txdisable = FALSE;

	/* Might have to output the first character here */

	enable_tx_ints (line);
}

/**********
*
*      stoptx
*
*      The purpose of this routine is to initialize a line table
*      for receive and to disable the receiver.
*
**********/
static	void	stop_tx (unsigned short line)
{
	TXLINE	*txp;
	disable_tx_ints (line);
	txp = txlinetab [line];
	txp->txdisable = TRUE;
}

/**********
*
*      tx_int_proc
*
*      The purpose of this routine is to process a receive character interrupt
*      for a given line.
*
**********/
static	void	tx_int_proc (unsigned short line)


#ifdef TEST


	void	tx_int_proc (unsigned short line)
#endif
{
	unsigned char           txchar;
	unsigned short          txstat;
	unsigned short          tx_err_mask;
	TXLINE                  *txp;

	txp = txlinetab [line];
	if (txp->txdisable)
	{
	    return;
	}
	txchar = txp->txbuf [txp->txix++];
	write_tx_hw (line, txchar, &txstat, &tx_err_mask);

	if ((txstat & tx_err_mask) != 0)
	{
	    disable_tx_ints (line);
	    txp->txdisable = TRUE;
	    *(txp->status) = TXBAD;
	    semGive (txp->semid);
	    return;
	}

	if (txp->txix == txp->txmsglen)
	{
	    disable_tx_ints (line);
	    txp->txdisable = TRUE;
	    *(txp->status) = TXGOOD;
	    semGive (txp->semid);
	    return;
	}

	if (txp->txix >= MAXTXMSG)
	{
	    disable_tx_ints (line);
	    txp->txdisable = TRUE;
	    *(txp->status) = TXBAD;
	    semGive (txp->semid);
	    return;
	}
}

/**********
*
*      msgxfer
*
*      The purpose of this routine is to transmit a buffer to the
*      Allen Bradley PLCs and to await a response. The transmit and receive
*      buffers are in their respective line tables. This routine will retry
*      the message up to a maximum number of times or until a data response
*      is received with a proper checksum.
*              returns         XFERGOOD if response received
*                              XFERBAD  if no good repsonse is received
*
**********/

static	unsigned	short	msgxfer (unsigned short fline, TIPLC *pmod)
{
	unsigned short  retry, txstatus, rxstatus, rxlen;
	unsigned short	line;
	RXLINE          *rxp;
	TXLINE          *txp;
	USHORT		R_ecc;
	USHORT		C_ecc;
	line = (fline & 0x0f);          /* strip off the no retry flag*/

	rxp = rxlinetab [line];
	txp = txlinetab [line];

	for (retry = ((fline == line) ? 0 : MAXRETRYS - 1); (retry < MAXRETRYS); retry++ )
	{
	    if (intermsgdelay [line] != 0)
	    {
		taskDelay (intermsgdelay [line]);
	    }
	    if (pretxdelay [line] != 0)
	    {
		taskDelay (pretxdelay [line]);
	    }
	    if (linemon [line])
	    {
		logMsg ("TX: %s\n", txp->txbuf);
	    }
	    txstatus = IO_TIMEOUT;
	    rxstatus = IO_TIMEOUT;
	    semTake (semids [line] [TXSEM], NO_WAIT);
	    semTake (semids [line] [RXSEM], NO_WAIT);

	    start_rx (line, (SEM_ID *) semids [line] [RXSEM], &rxstatus);
	    start_tx (line, (SEM_ID *) semids [line] [TXSEM], &txstatus); /* this fires off the transmission. */
	    if ((semTake (semids [line] [TXSEM], TXTIMEOUT)) == ERROR)
	    {
		txstatus = IO_TIMEOUT;
	    }
	    stop_tx (line);

	    if (posttxdelay [line] != 0)
	    {
		taskDelay (posttxdelay [line]);
	    }
	    if (txstatus == TXGOOD)
	    {
		if ((semTake (semids [line] [RXSEM], (int) (RXTIMEOUT * 2))) == ERROR)
		{
		    rxstatus = IO_TIMEOUT;
		}

		stop_rx (line);

		if (linemon [line])
		{
		    logMsg ("RX: %s\n", rxp->rxbuf);
		}
		rxlen = rxp->rxix;
		if (rxstatus == RXGOOD)
		{
		    C_ecc = nitp_ECC (C_ecc = 0, rxp->rxbuf, rxlen - 5);
		    nitp_ASCII2bin (&rxp->rxbuf [rxlen - 5], (UCHAR *) &R_ecc, 4);
		    if (C_ecc == R_ecc)
		    {
			if ((txp->txbuf [5] == rxp->rxbuf [4]) &&
				(txp->txbuf [6] == rxp->rxbuf [5]))
			{
			    return (XFERGOOD);
			}
			else
			{
			    if (modtrace [line])
			    {
				logMsg ("ERROR Response from UHA\n");
			    }
			}
		    }
		    else
		    {
			pmod->no_crcerrs++;
			if (modtrace [line])
			{
			    logMsg ("R-%x C-%x Bad Ecc\n",
				R_ecc, C_ecc);
			}
		    }
		}
		else if (rxstatus == IO_TIMEOUT)
		{
		    if (modtrace [line])
		    {
		        logMsg ("Receive Timeout... Length: %d\n", rxlen);
				pmod->no_rxnoresp++;
		    }
		}
		else
		{
		    if (modtrace [line])
		    {
			logMsg ("Receive Error\n");

			logMsg ( "RXHWERR: %2x, RXCHR: %2x, RXCNT: %2x\n",
				rxp->rxbuf [299],
				rxp->rxbuf [300],
				rxp->rxix);
		    }
		    pmod->no_rxhwerrs++;
		}
	    }
	    else if (txstatus == IO_TIMEOUT)
	    {
		if (modtrace [line])
		{
		    logMsg ("Transmit Timeout\n");
		}
		pmod->no_txnoresp++;
	    }
	    else
	    {
		if (modtrace [line])
		{
		    logMsg ("Transmit Error\n");
		}
		pmod->no_txhwerrs++;
	    }
	    if (retry != MAXRETRYS)
	    {
		pmod->no_retries++;
	    }
	}
	if (modtrace [line])
	{
	    logMsg ("Xfer Bad\n");
	}
	return (XFERBAD);
}

/**********
*
*      init_plc_tab
*
*      The purpose of this routine is to initialize the line tables for the
*      Allen Bradley I/O
*
**********/

/* only called from allocate_plc  */

static	void	init_plc_tab (unsigned short line, unsigned short mod)
{
	TIPLC  *pmod;

	pmod = plcs [line] [mod];
	if (!pmod)
	{
		return;
	}
	pmod->is_responding = FALSE;
	pmod->holdflag = FALSE;
	pmod->was_requested = FALSE;
	pmod->cmd_force_offline = FALSE;
	pmod->dev_type = 0xffff;
	pmod->vmem = 0;
	pmod->cmem = 0;
	pmod->xmem = 0;
	pmod->txmem = 0;
	pmod->ymem = 0;
	pmod->tymem = 0;
	pmod->crmem = 0;
	pmod->wxin = 0;
	pmod->wyout = 0;
	pmod->status = 0;
	pmod->lpstat = 0;
	pmod->lpcntl = 0;
	pmod->lpmd = 0;
	pmod->avflag = 0;
	pmod->acflag = 0;
	pmod->lpgain = 0;
	pmod->lpreset = 0;
	pmod->lprate = 0;
	pmod->lphialrm = 0;
	pmod->lploalrm = 0;
	pmod->lpprocvar = 0;
	pmod->lppvhilim = 0;
	pmod->lppvlolim = 0;
	pmod->lporgdevalrm = 0;
	pmod->lpyeldevalrm = 0;
	pmod->lpsmprate = 0;
	pmod->lpremsp = 0;
	pmod->lpoutput = 0;
	pmod->lperror = 0;
	pmod->lpbias = 0;
	pmod->lppvhihilim = 0;
	pmod->lppvlololim = 0;
	pmod->lprocalrm = 0;
	pmod->anpvhialrm = 0;
	pmod->anpvloalrm = 0;
	pmod->anprocvar = 0;
	pmod->anpvhilim = 0;
	pmod->anpvlolim = 0;
	pmod->anorgdevlim = 0;
	pmod->anyeldevlim = 0;
	pmod->ansmprate = 0;
	pmod->ansetpoint = 0;
	pmod->anerror = 0;
	pmod->anpvhihialrm = 0;
	pmod->anpvloloalrm = 0;
	pmod->anroc = 0;

	initstats (pmod);

	/* Initialize this tasks semaphore*/

	semids [line] [0] = semBCreate (SEM_Q_PRIORITY, SEM_EMPTY);
	semids [line] [1] = semBCreate (SEM_Q_PRIORITY, SEM_EMPTY);
}

/**********
*
* initialize the statistics for an ALLEN BRADLEY PLC
*
**********/

static	void	initstats (TIPLC *pmod)
{
	pmod->no_dataout_msgs = 0;      /* 5  Count Output Msgs */
	pmod->no_datain_msgs = 0;       /* 6  Count Data Requests */
	pmod->no_rxnoresp = 0;          /* 7  Count No Responses */
	pmod->no_txnoresp = 0;          /* 8  Count No Responses */
	pmod->no_dataerr = 0;           /* 9  Count Message Echo Errors */
	pmod->no_retries = 0;           /* 10 Count Retries */
	pmod->no_rxhwerrs = 0;          /* 11 Count Comm Hardware Errors */
	pmod->no_txhwerrs = 0;          /* 12 Count Comm Hardware Errors */
	pmod->no_crcerrs = 0;           /* 13 Count CRC Errors */
	pmod->no_offlines = 0;          /* 14 Count Off Line Transitions */
	pmod->badoutcnt = 0;		/* 17 Count failed output commands */
}

/**********
*	prepresponses                                                            
*                                                                             	
*       The purpose of this routine is to mark the PLCs as on hold and the    	
*	data as not valid until read again from the PLC                       	
*                                                                           	
**********/

/* only called from MAIN */

static	void	prepresponses (unsigned short line)
{
	unsigned	short 	mod;
	TIPLC			*pmod;

	for (mod = 1; (mod <= NUMTIPLCS); mod++)
	{
            pmod = plcs [line] [mod];
	    if (pmod)			/* verify pointer validity prior to use */
	    {
	    	pmod->holdflag = TRUE;
		mark_not_valid (pmod);
            }
	}
}

/**********
*
*      any_responding                                               	
*                                                                   	
*      The purpose of this routine is to see if any units are       	
*	responding on this line                                      	
*                                                                   	
**********/

/* only called from MAIN */

static	int	any_responding (unsigned short line)
{
	unsigned short  mod;
	TIPLC  *pmod;

	for (mod = 1; (mod <= NUMTIPLCS); mod++)
	{
	    pmod = plcs [line] [mod];
	    if (pmod)			/* verify pointer validity prior to use */
	    {
		if (pmod->is_responding)
		{
		    return(TRUE);
		}
	    }
	}
	return (FALSE);
}

/**********
*
*	readtivmem
*
*	Reads the TI PLC VMEM
*
**********/

static	unsigned	short	readtivmem (unsigned short line,
				unsigned short mod, TIPLC *pmod,
				unsigned short group, unsigned short numgroups)
{
	unsigned	short 	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements = TINOGRPELEMS * numgroups;

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    if (!(tigetdata ( line, mod, pmod, TIVMEM,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    if (!(tigetdata ( line, mod, pmod, TIVMEM,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	default:
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->vmem [i].vmem [j] = wval;
	    }
	    pmod->vmem [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readticmem
*
*	Reads the TI PLC CMEM
*
**********/

static	unsigned	short	readticmem (unsigned short line,
				unsigned short mod, TIPLC *pmod,
				unsigned short group, unsigned short numgroups)
{
	unsigned	short 	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements = TINOGRPELEMS * numgroups;

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    if (!(tigetdata ( line, mod, pmod, TICMEM,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    if (!(tigetdata ( line, mod, pmod, TICMEM,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	default:
	    return (FALSE);

	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->cmem [i].cmem [j] = wval;
	    }
	    pmod->cmem [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtixmem
*	Reads the TI PLC xmem
*
**********/

static	unsigned	short	readtixmem (unsigned short line,
				unsigned short mod, TIPLC *pmod,
				unsigned short group, unsigned short numgroups)
{
	unsigned	short	elementno, numelements;
	unsigned	long	wval;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    elementno = group * 32;
	    numelements =  numgroups * 32;

	    if (!(tigetdata (line, mod, pmod, TIXMEM,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    lastgroup = group + numgroups - 1;
	    dataix = 8;
	    for (i = group; i <= lastgroup; i++)
	    {
		for (j = wval = 0; j < 32; j++)
		{
		    wval += (unsigned long) data [dataix++] << j;
		}
		pmod->xmem [i].xmem = wval;
		pmod->xmem [i].data_valid = TRUE;
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:

	/* We aonly read 1 element for Tank Transmitters */

	    elementno = group;
	    numelements =  numgroups;

	    if (!(tigetdata ( line, mod, pmod, TIXMEM,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    dataix = 8;
	    pmod->txmem [group].txmem = data [dataix];
	    pmod->txmem [group].data_valid = TRUE;
	    break;

	default:
	    return (FALSE);
	}
	return (TRUE);
}

/**********
*
*	readtiymem
*	Reads the TI PLC ymem
*
**********/

static	unsigned	short	readtiymem (unsigned short line,
				unsigned short mod, TIPLC *pmod,
				unsigned short group, unsigned short numgroups)
{
	unsigned	short	elementno, numelements;
	unsigned	long	lval;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    elementno = group * 32;
	    numelements =  numgroups * 32;

	    if (!(tigetdata ( line, mod, pmod, TIYMEM,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    lastgroup = group + numgroups - 1;
	    dataix = 8;
	    for (i = group; i <= lastgroup; i++)
	    {
		for (j = lval = 0; j < 32; j++)
		{
		    lval += (unsigned long) data [dataix++] << j;
		}
		pmod->ymem [i].ymem = lval;
		pmod->ymem [i].data_valid = TRUE;
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    elementno = group;
	    numelements =  numgroups;

	    if (!(tigetdata ( line, mod, pmod, TIYMEM,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    dataix = 8;
	    pmod->tymem [group].tymem = data [dataix++];
	    pmod->tymem [group].data_valid = TRUE;
	    break;

	default:
	    return (FALSE);
	}
	return (TRUE);
}

/**********
*
*	readticrmem
*	Reads the TI PLC crmem
*
**********/

static	unsigned	short	readticrmem (unsigned short line,
				unsigned short mod, TIPLC *pmod,
				unsigned short group, unsigned short numgroups)
{
	unsigned	short	elementno, numelements;
	unsigned	long	lval;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = group * 32;
	numelements =  numgroups * 32;

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    if (!(tigetdata ( line, mod, pmod, TICRMEM,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    if (!(tigetdata ( line, mod, pmod, TICRMEM,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	default:
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = lval = 0; j < 32; j++)
	    {
		lval += (unsigned long) data [dataix++] << j;
	    }
	    pmod->crmem [i].crmem = lval;
	    pmod->crmem [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtiwxin
*	Reads the TI PLC wxin
*
**********/
static	unsigned short readtiwxin (unsigned short line, unsigned short mod,
				TIPLC *pmod, unsigned short group,
				unsigned short numgroups)
{
	unsigned	short	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    if (!(tigetdata (line, mod, pmod, TIWXIN,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    if (!(tigetdata ( line, mod, pmod, TIWXIN,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	default:
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->wxin [i].wxin [j] = wval;
	    }
	    pmod->wxin [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtiwyout
*	Reads the TI PLC wyout
*
**********/
static	unsigned short readtiwyout (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    if (!(tigetdata (line, mod, pmod, TIWYOUT,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    if (!(tigetdata ( line, mod, pmod, TIWYOUT,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	default:
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->wyout [i].wyout [j] = wval;
	    }
	    pmod->wyout [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtistatus
*	Reads the TI PLC status
*
**********/
static	unsigned short readtistatus (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    if (!(tigetdata (line, mod, pmod, TISTATUS,
			 (elementno + 1), numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    if (!(tigetdata ( line, mod, pmod, TISTATUS,
			 elementno, numelements, data)))
	    {
		return (FALSE);
	    }
	    break;

	default:
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->status [i].status [j] = wval;
	    }
	    pmod->status [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpstat
*   	Reads the TI PLC lpstat
*
**********/
static	unsigned short readtilpstat (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short 	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPSTAT,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->lpstat [i].lpstat [j] = wval;
	    }
	    pmod->lpstat [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpcntl
*   	Reads the TI PLC lpcntl
*
**********/
static	unsigned short readtilpcntl (unsigned short line, unsigned short mod,
			  TIPLC *pmod,unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	long	lval;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPCNTL,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += ((unsigned long) data [dataix++]);
		pmod->lpcntl [i].lpcntl [j] = lval;
	    }
	    pmod->lpcntl [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpmd
*   	Reads the TI PLC lpmd
*
**********/
static	unsigned short readtilpmd (unsigned short line, unsigned short mod,
			  TIPLC *pmod,unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short 	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPMD,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->lpmd [i].lpmd [j] = wval;
	    }
	    pmod->lpmd [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtiavflag
*   	Reads the TI PLC avflag
*
**********/
static	unsigned short readtiavflag (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short 	wval, elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIAVFLAG,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		wval = ((unsigned short) data [dataix++] << 8);
		wval += data [dataix++];
		pmod->avflag [i].avflag [j] = wval;
	    }
	    pmod->avflag [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtiacflag
*   	Reads the TI PLC acflag
*
**********/
static	unsigned short readtiacflag (unsigned short line, unsigned short mod,
			  TIPLC *pmod,unsigned short group,
			  unsigned short numgroups)
{
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	long	lval;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIACFLAG,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += ((unsigned long) data [dataix++]);
		pmod->acflag [i].acflag [j] = lval;
	    }
	    pmod->acflag [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpgain
*   	Reads the TI PLC lpgain
*
**********/
static	unsigned short readtilpgain (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPGAIN,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpgain [i].lpgain [j] = lval;
	    }
	    pmod->lpgain [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpreset
*   	Reads the TI PLC lpreset
*
**********/
static	unsigned short readtilpreset (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPRESET,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpreset [i].lpreset [j] = (float) lval;
	    }
	    pmod->lpreset [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilprate
*   	Reads the TI PLC lprate
*
**********/
static	unsigned short readtilprate (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPRATE,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lprate [i].lprate [j] = (float) lval;
	    }
	    pmod->lprate [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilphialrm
*   	Reads the TI PLC lphialrm
*
**********/
static	unsigned short readtilphialrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPHIALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lphialrm [i].lphialrm [j] = (float) lval;
	    }
	    pmod->lphialrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilploalrm
*   	Reads the TI PLC lploalrm
*
**********/
static	unsigned short readtilploalrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPLOALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lploalrm [i].lploalrm [j] = (float) lval;
	    }
	    pmod->lploalrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpprocvar
*   	Reads the TI PLC lpprocvar
*
**********/
static	unsigned short readtilpprocvar (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPPROCVAR,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpprocvar [i].lpprocvar [j] = (float) lval;
	    }
	    pmod->lpprocvar [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilppvhilim
*   	Reads the TI PLC lppvhilim
*
**********/
static	unsigned short readtilppvhilim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPPVHILIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lppvhilim [i].lppvhilim [j] = (float) lval;
	    }
	    pmod->lppvhilim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilppvlolim
*   	Reads the TI PLC lppvlolim
*
**********/
static	unsigned short readtilppvlolim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPPVLOLIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lppvlolim [i].lppvlolim [j] = (float) lval;
	    }
	    pmod->lppvlolim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilporgdevalrm
*   	Reads the TI PLC lporgdevalrm
*
**********/
static	unsigned short readtilporgdevalrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPORGDEVALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lporgdevalrm [i].lporgdevalrm [j] = (float) lval;
	    }
	    pmod->lporgdevalrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpyeldevalrm
*   	Reads the TI PLC lpyeldevalrm
*
**********/
static	unsigned short readtilpyeldevalrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPYELDEVALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpyeldevalrm [i].lpyeldevalrm [j] = (float) lval;
	    }
	    pmod->lpyeldevalrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpsmprate
*   	Reads the TI PLC lpsmprate
*
**********/
static	unsigned short readtilpsmprate (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPSMPRATE,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpsmprate [i].lpsmprate [j] = (float) lval;
	    }
	    pmod->lpsmprate [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpremsp
*   	Reads the TI PLC lpremsp
*
**********/
static	unsigned short readtilpremsp (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPREMSP,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpremsp [i].lpremsp [j] = (float) lval;
	    }
	    pmod->lpremsp [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpoutput
*   	Reads the TI PLC lpoutput
*
**********/
static	unsigned short readtilpoutput (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPOUTPUT,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpoutput [i].lpoutput [j] = (float) lval;
	    }
	    pmod->lpoutput [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilperror
*   	Reads the TI PLC lperror
*
**********/
static	unsigned short readtilperror (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPERROR,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lperror [i].lperror [j] = (float) lval;
	    }
	    pmod->lperror [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilpbias
*   	Reads the TI PLC lpbias
*
**********/
static	unsigned short readtilpbias (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPBIAS,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lpbias [i].lpbias [j] = (float) lval;
	    }
	    pmod->lpbias [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilppvhihilim
*   	Reads the TI PLC lppvhihilim
*
**********/
static	unsigned short readtilppvhihilim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPPVHIHILIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lppvhihilim [i].lppvhihilim [j] = (float) lval;
	    }
	    pmod->lppvhihilim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilppvlololim
*   	Reads the TI PLC lppvlololim
*
**********/
static	unsigned short readtilppvlololim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPPVLOLOLIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lppvlololim [i].lppvlololim [j] = (float) lval;
	    }
	    pmod->lppvlololim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtilprocalrm
*   	Reads the TI PLC lprocalrm
*
**********/
static	unsigned short readtilprocalrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TILPROCALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->lprocalrm [i].lprocalrm [j] = (float) lval;
	    }
	    pmod->lprocalrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianpvhialrm
*   	Reads the TI PLC anpvhialrm
*
**********/
static	unsigned short readtianpvhialrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPVHIALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anpvhialrm [i].anpvhialrm [j] = (float) lval;
	    }
	    pmod->anpvhialrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianpvloalrm
*   	Reads the TI PLC anpvloalrm
*
**********/
static	unsigned short readtianpvloalrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPVLOALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anpvloalrm [i].anpvloalrm [j] = (float) lval;
	    }
	    pmod->anpvloalrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianprocvar
*   	Reads the TI PLC anprocvar
*
**********/
static	unsigned short readtianprocvar (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPROCVAR,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anprocvar [i].anprocvar [j] = (float) lval;
	    }
	    pmod->anprocvar [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianpvhilim
*   	Reads the TI PLC anpvhilim
*
**********/
static	unsigned short readtianpvhilim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPVHILIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anpvhilim [i].anpvhilim [j] = (float) lval;
	    }
	    pmod->anpvhilim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianpvlolim
*   	Reads the TI PLC anpvlolim
*
**********/
static	unsigned short readtianpvlolim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPVLOLIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anpvlolim [i].anpvlolim [j] = (float) lval;
	    }
	    pmod->anpvlolim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianorgdevlim
*   	Reads the TI PLC anorgdevlim
*
**********/
static	unsigned short readtianorgdevlim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANORGDEVLIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anorgdevlim [i].anorgdevlim [j] = (float) lval;
	    }
	    pmod->anorgdevlim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianyeldevlim
*   	Reads the TI PLC anyeldevlim
*
**********/
static	unsigned short readtianyeldevlim (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANYELDEVLIM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anyeldevlim [i].anyeldevlim [j] = (float) lval;
	    }
	    pmod->anyeldevlim [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtiansmprate
*   	Reads the TI PLC ansmprate
*
**********/
static	unsigned short readtiansmprate (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANSMPRATE,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->ansmprate [i].ansmprate [j] = (float) lval;
	    }
	    pmod->ansmprate [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtiansetpoint
*   	Reads the TI PLC ansetpoint
*
**********/
static	unsigned short readtiansetpoint (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANSETPOINT,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->ansetpoint [i].ansetpoint [j] = (float) lval;
	    }
	    pmod->ansetpoint [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianerror
*   	Reads the TI PLC anerror
*
**********/
static	unsigned short readtianerror (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANERROR,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anerror [i].anerror [j] = (float) lval;
	    }
	    pmod->anerror [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianpvhihialrm
*   	Reads the TI PLC anpvhihialrm
*
**********/
static	unsigned short readtianpvhihialrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{

	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPVHIHIALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anpvhihialrm [i].anpvhihialrm [j] = (float) lval;
	    }
	    pmod->anpvhihialrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianpvloloalrm
*   	Reads the TI PLC anpvloloalrm
*
**********/
static	unsigned short readtianpvloloalrm (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANPVLOLOALRM,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anpvloloalrm [i].anpvloloalrm [j] = (float) lval;
	    }
	    pmod->anpvloloalrm [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	readtianroc
*   	Reads the TI PLC anroc
*
**********/
static	unsigned short readtianroc (unsigned short line, unsigned short mod,
			  TIPLC *pmod, unsigned short group,
			  unsigned short numgroups)
{
	unsigned	long	lval;
	unsigned	short 	elementno, numelements;
	unsigned	short	dataix, i, j, lastgroup;
	unsigned	char	data [MAXRXMSG];

	/* Count number of data request messages */

	pmod->no_datain_msgs++;

	/* Get the data */

	elementno = TINOGRPELEMS * group;
	numelements =  TINOGRPELEMS * numgroups;

	if (!(tigetdata (line, mod, pmod, TIANROC,
			 (elementno + 1), numelements, data)))
	{
	    return (FALSE);
	}
	lastgroup = group + numgroups - 1;
	dataix = 8;
	for (i = group; i <= lastgroup; i++)
	{
	    for (j = 0; j < TINOGRPELEMS; j++)
	    {
		lval = ((unsigned long) data [dataix++] << 24);
		lval += ((unsigned long) data [dataix++] << 16);
		lval += ((unsigned long) data [dataix++] << 8);
		lval += (unsigned long) data [dataix++];
		pmod->anroc [i].anroc [j] = (float) lval;
	    }
	    pmod->anroc [i].data_valid = TRUE;
	}
	return (TRUE);
}

/**********
*
*	Setaouts
*	Set analog outputs in the TI PLC
*
**********/
static	unsigned short setaouts (unsigned short line, unsigned short mod,
			 TIPLC *pmod, unsigned short card,
			 unsigned short signal, unsigned short conversion,
			 TI_RET_VAL val)
{
	TXLINE  		*txp;
	RXLINE  		*rxp;
	unsigned	short	ix, element, subelement, ecc, err_code;
	unsigned	char	cval, retbuf [50];
	int			retcnt;
	unsigned	char	sec_adr, mtyp;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	sec_adr = mod & 0xff;
	mtyp = card & 0xff;
	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:
	    signal += 1;
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    break;
	}

	txp->txbuf [0] = ':';		/* NITP Delimiter */
	txp->txbuf [1] = '0';		/* NITP Msg Length */
	txp->txbuf [2] = '0';
	if (conversion == ASCFLT)
	{
	    txp->txbuf [3] = '2';
	    txp->txbuf [4] = '2';
	}
	else
	{
	    txp->txbuf [3] = '1';
	    txp->txbuf [4] = 'E';
	}
	txp->txbuf [5] = '0';		/* Send Net Data Function Code */
	txp->txbuf [6] = '1';

	nitp_bin2ASCII ((UCHAR *) &sec_adr, sizeof (sec_adr), &txp->txbuf [7]);

	txp->txbuf [9] = '0';		/* Tiway Primitive Msg Length */
	txp->txbuf [10] = '0';
	txp->txbuf [11] = '0';
	if (conversion == ASCFLT)
	{
	    txp->txbuf [12] = '8';
	}
	else
	{
	    txp->txbuf [12] = '6';
	}
	txp->txbuf [13] = '3';		/* Tiway Primitive Write Block */
	txp->txbuf [14] = '0';
	nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	nitp_bin2ASCII ((UCHAR *) &signal, sizeof (signal), &txp->txbuf [17]);
	if (conversion == ASCFLT)
	{
	    nitp_bin2ASCII ((UCHAR *) &val.fval, sizeof (val.fval), &txp->txbuf [21]);
	}
	else
	{
	    nitp_bin2ASCII ((UCHAR *) &val.usval [0], sizeof (val.usval [0]), &txp->txbuf [21]);
	}
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer ( line, pmod) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->badoutcnt++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (FALSE);
	}
	nitp_ASCII2bin (rxp->rxbuf, retbuf, strlen (rxp->rxbuf));
	if (retbuf [6] == 0)
	{
	    if (mltrace [line] >= 3)
	    {
	        err_code = (retbuf [8] << 8);
	        err_code += retbuf [9];
	        logMsg ("AOUT Error - 0x%0x %s\n", err_code, prim_err_code [err_code]);
	    }
	    return (FALSE);
	} 
	return (TRUE);
}

/**********
*
* Setbouts
*	Set binary outputs in a Tiway device PLC files.
*
**********/
static	unsigned short setbouts (unsigned short line, unsigned short mod,
			 TIPLC *pmod, unsigned short card,
			 unsigned short signal,
			 long mask, unsigned short shift, long val)
{
	TXLINE			*txp;
	RXLINE			*rxp;
	unsigned	short	ix, iy, ecc, baseelement, element, subelement;
	unsigned	short	tmask, tvalue, andmask, ormask, err_code;
	unsigned	char 	cval [32], retbuf [50];
	int			retcnt;
	unsigned	short	wval;
	unsigned	long	lval;
	unsigned	char	sec_adr, mtyp, tlen, nlen;

	/* Set up pointers for comm buffers */

	txp = txlinetab [line & (~ONERETRY)];
	rxp = rxlinetab [line & (~ONERETRY)];

	/* Count number of data request messages */

	pmod->no_dataout_msgs++;

	sec_adr = mod & 0xff;
	mtyp = card & 0xff;

	txp->txbuf [0] = ':';		/* NITP Delimiter */
	txp->txbuf [1] = '0';		/* NITP Msg Length */
	txp->txbuf [2] = '0';
	switch (card)
	{
	case TIYMEM:
	case TICRMEM:
	    mask >>= shift;
	    val >>= shift;
	    for (ix = 0; (mask & 0x01); ix++)
	    {
		cval [ix] = val & 0x01;
		mask >>= 1;
		val >>= 1;
	    }
	    if (ix == 0)
	    {
		cval [ix] = (val && mask);
		ix++;
	    }
	    tlen = (0x1a + (ix * 2));
	    nlen = (0x04 + ix);
	    nitp_bin2ASCII ((UCHAR *) &tlen, sizeof (tlen), &txp->txbuf [3]);
	    break;

	case TIVMEM:
	case TICMEM:
	case TIWYOUT:
	case TILPMD:
	case TIAVFLAG:
	    txp->txbuf [3] = '1';
	    txp->txbuf [4] = 'E';
	    break;

	case TILPCNTL:
	case TIACFLAG:
	    txp->txbuf [3] = '2';
	    txp->txbuf [4] = '2';
	    break;

	default:
	    logMsg ("Invalid memory type for Binary Output\n");
	    return (FALSE);
	}
	switch (pmod->dev_type)
	{
	case PLC_5TI:
	case PLC_510:
	case PLC_520:
	case PLC_520C:
	case PLC_525:
	case PLC_530:
	case PLC_530C:
	case PLC_540:
	case PLC_550:
	case PLC_560:
	case PLC_565:
	case PLC_570:
	case UHA:
	case PM550:
	case PM551:
	case UA:

	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWYOUT:
	    case TILPSTAT:
	    case TILPMD:

		baseelement = (((signal / 16) - 1) / TINOGRPELEMS);
		subelement = (((signal / 16) - 1) % TINOGRPELEMS);
	 	element = ((signal + 1) / 16);
		break;

	    case TIYMEM:
	    case TICRMEM:

		break;

	    case TILPCNTL:
	    case TIACFLAG:

		baseelement = (((signal / 32) - 1) / TINOGRPELEMS);
		subelement = (((signal / 32) - 1) % TINOGRPELEMS);
		element = ((signal + 1) / 32);
		break;

	    default:

		return (-1);
	    }
	    break;

	case IT_111_121:
	case IT_150:
	case IT_160:
	    switch (card)
	    {
	    case TIVMEM:
	    case TICMEM:
	    case TIWYOUT:
	    case TILPSTAT:
	    case TILPMD:

		baseelement = ((signal / 16) / TINOGRPELEMS);
		subelement = ((signal / 16) % TINOGRPELEMS);
	 	element = (signal / 16);
		break;

	    case TIYMEM:
	    case TICRMEM:

		break;

	    case TILPCNTL:
	    case TIACFLAG:

		baseelement = ((signal / 32) / TINOGRPELEMS);
		subelement = ((signal / 32) % TINOGRPELEMS);
		element = (signal / 32);
		break;

	    default:

		return (-1);
	    }
	    break;
	}
	txp->txbuf [5] = '0';		/* Send Net Data Function Code */
	txp->txbuf [6] = '1';

	nitp_bin2ASCII ((UCHAR *) &sec_adr, sizeof (sec_adr), &txp->txbuf [7]);

	txp->txbuf [9] = '0';		/* Tiway Primitive Msg Length */
	txp->txbuf [10] = '0';
	switch (card)
	{
	case TIYMEM:
	case TICRMEM:

	    nitp_bin2ASCII (&nlen, sizeof (nlen), &txp->txbuf [11]);
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &signal, sizeof (signal), &txp->txbuf [17]);
	    for (iy = 0; iy < ix; iy++)
	    {
		nitp_bin2ASCII ((UCHAR *) &cval [iy], sizeof (cval [iy]), &txp->txbuf [(21 + (iy * 2))]);
	    }
	    break;

	case TIVMEM:
	    txp->txbuf [11] = '0';
	    wval = pmod->vmem [baseelement].vmem [subelement];
	    wval &= ~mask;
	    wval |= (val & mask);
	    pmod->vmem [baseelement].vmem [subelement] = wval;
	    txp->txbuf [12] = '6';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &wval, sizeof (wval), &txp->txbuf [21]);
	    break;

	case TICMEM:
	    txp->txbuf [11] = '0';
	    wval = pmod->cmem [baseelement].cmem [subelement];
	    wval &= ~mask;
	    wval |= (val & mask);
	    pmod->cmem [baseelement].cmem [subelement] = wval;
	    txp->txbuf [12] = '6';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &wval, sizeof (wval), &txp->txbuf [21]);
	    break;

	case TIWYOUT:
	    txp->txbuf [11] = '0';
	    wval = pmod->wyout [baseelement].wyout [subelement];
	    wval &= ~mask;
	    wval |= (val & mask);
	    pmod->wyout [baseelement].wyout [subelement] = wval;
	    txp->txbuf [12] = '6';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &wval, sizeof (wval), &txp->txbuf [21]);
	    break;

	case TILPMD:
	    txp->txbuf [11] = '0';
	    wval = pmod->lpmd [baseelement].lpmd [subelement];
	    wval &= ~mask;
	    wval |= (val & mask);
	    pmod->lpmd [baseelement].lpmd [subelement] = wval;
	    txp->txbuf [12] = '6';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &wval, sizeof (wval), &txp->txbuf [21]);
	    break;

	case TIAVFLAG:
	    txp->txbuf [11] = '0';
	    wval = pmod->avflag [baseelement].avflag [subelement];
	    wval &= ~mask;
	    wval |= (val & mask);
	    pmod->avflag [baseelement].avflag [subelement] = wval;
	    txp->txbuf [12] = '6';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &wval, sizeof (wval), &txp->txbuf [21]);
	    break;

	case TILPCNTL:
	    txp->txbuf [11] = '0';
	    lval = pmod->lpcntl [baseelement].lpcntl [subelement];
	    lval &= ~mask;
	    lval |= (val & mask);
	    pmod->lpcntl [baseelement].lpcntl [subelement] = lval;
	    txp->txbuf [12] = '8';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &lval, sizeof (lval), &txp->txbuf [21]);
	    break;

	case TIACFLAG:
	    txp->txbuf [11] = '0';
	    lval = pmod->acflag [baseelement].acflag [subelement];
	    lval &= ~mask;
	    lval |= (val & mask) ;
	    pmod->avflag [baseelement].avflag [subelement] = lval;
	    txp->txbuf [12] = '8';
	    nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	    nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [17]);
	    nitp_bin2ASCII ((UCHAR *) &lval, sizeof (lval), &txp->txbuf [21]);
	    break;

	default:
	    logMsg ("Invalid memory type for Binary Output\n");
	    return (FALSE);
	}

	txp->txbuf [13] = '3';		/* Tiway Primitive Write Block */
	txp->txbuf [14] = '0';
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;

	if (msgxfer ( line, pmod) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->badoutcnt++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (FALSE);
	}
	nitp_ASCII2bin (rxp->rxbuf, retbuf, strlen (rxp->rxbuf));
	if (retbuf [6] == 0)
	{
	    if (mltrace [line] >= 3)
	    {
		err_code = (retbuf [8] << 8);
		err_code += retbuf [9];
		logMsg ("BOUT Error - 0x%0x %s\n", err_code, prim_err_code [err_code]);
	    }
	    return (FALSE);
	} 
	return (TRUE);
}

/*
/* Sendtestmsg
/*
*/
static	unsigned	short	sendtestmsg (unsigned short line)

{
	TIPLC			*pmod;
	TXLINE			*txp;
	unsigned	short	mod, ix, checksum, baseelement;
	unsigned	short	element, subelement;
	unsigned	short	tmask, tvalue, andmask, ormask;
	unsigned	char 	cval;

	/* Set up pointers for comm buffers */
	return (TRUE);
}

/**********
*
*	Routine to fetch data from an Seimens PLC
*
**********/
static	int tigetdata ( unsigned short fline, unsigned short mod,
		TIPLC *pmod, unsigned short mtype,
		unsigned short element, unsigned short cnt,
		unsigned char *retbuf)
{
	int			bytecnt;
	unsigned	short	ix, ecc, line, err_code;
	TXLINE			*txp;
	RXLINE			*rxp;
	unsigned	char	sec_adr, mtyp;

	line = fline & (~ONERETRY);
	sec_adr = mod & 0xff;
	mtyp = mtype & 0xff;

	txp = txlinetab [line];
	rxp = rxlinetab [line];

	txp->txbuf [0] = ':';		/* NITP Delimiter */
	txp->txbuf [1] = '0';		/* NITP Msg Length */
	txp->txbuf [2] = '0';
	txp->txbuf [3] = '1';
	txp->txbuf [4] = 'E';
	txp->txbuf [5] = '0';		/* Send Net Data Function Code */
	txp->txbuf [6] = '1';

	nitp_bin2ASCII ((UCHAR *) &sec_adr, sizeof (sec_adr), &txp->txbuf [7]);

	txp->txbuf [9] = '0';		/* Tiway Primitive Msg Length */
	txp->txbuf [10] = '0';
	txp->txbuf [11] = '0';
	txp->txbuf [12] = '6';
	txp->txbuf [13] = '2';		/* Tiway Primitive Read Block */
	txp->txbuf [14] = '0';
	nitp_bin2ASCII (&mtyp, sizeof (mtyp), &txp->txbuf [15]);
	nitp_bin2ASCII ((UCHAR *) &cnt, sizeof (cnt), &txp->txbuf [17]);
	nitp_bin2ASCII ((UCHAR *) &element, sizeof (element), &txp->txbuf [21]);
	ecc = nitp_ECC (ecc = 0, &txp->txbuf [1], (strlen (txp->txbuf) - 1));
	nitp_bin2ASCII ((UCHAR *) &ecc, sizeof (ecc),
			&txp->txbuf [strlen (txp->txbuf)]);
	txp->txbuf [txp->txmsglen = strlen (txp->txbuf)] = ';';
	txp->txmsglen++;
	txp->txbuf [txp->txmsglen] = 0;
	if (msgxfer (fline, pmod) == XFERBAD)
	{
	    pmod->no_offlines++;
	    pmod->holdflag = TRUE;
	    pmod->is_responding = FALSE;
	    return (FALSE);
	}
	nitp_ASCII2bin (rxp->rxbuf, retbuf, strlen (rxp->rxbuf));
	if (retbuf [6] == 0)
	{
	    if (mltrace [line] >= 3)
	    {
	        err_code = (retbuf [8] << 8);
	        err_code += retbuf [9];

	        logMsg ("READ Error - 0x%0x %s\n", err_code, prim_err_code [err_code]);
		logMsg ("Line %d, Addr %d, Card %d, Signal %d, Count %d\n",
			line, sec_adr, mtyp, element, cnt);
	    }
	    return (FALSE);
	} 
	return (TRUE);
}

USHORT nitp_ECC (USHORT seed, UCHAR *p, int len)
{

	UCHAR           binbuf [256];
	FAST BOOL	odd;

	nitp_ASCII2bin (p, binbuf, len);

	p = binbuf;

	len >>= 1;			/*process buff as 16-bit values*/
	odd = len & 1;
	len >>= 1;			/*process buff as 16-bit values*/

	while (len--)
	{
	    seed += *(USHORT *) p;	/*big-endian*/
	    p += sizeof (USHORT); /*had best == 2bytes per*/
	}
	if( odd )
	{
	    seed += (USHORT) *p << 8;
	}
	return -seed;
}

void	nitp_bin2ASCII (UCHAR *p, int len, UCHAR *d)
{
	FAST UCHAR hi, lo;
	while (len--)
	{
	    hi = *p >> 4;
	    lo = *p & 0xf;
	    if ((hi += '0') > '9')
	    {
		hi += 7;
	    }
	    if ((lo += '0') > '9')
	    {
		lo += 7;
	    }
	    ++p;
	    *d++ = hi;
	    *d++ = lo;
	}
	*d = 0;
}

void	nitp_ASCII2bin (UCHAR *p, UCHAR *d, int len)
{
	FAST UCHAR hi, lo;

	len >>= 1;

	while( len-- )
	{
	    if ((hi = *p++) > '@')
	    {
		hi -= 7;
	    }
	    if ((lo = *p++) > '@')
	    {
		lo -= 7;
	    }
	    *d++ = (hi << 4) | (lo & 0xf);
	}
}

/*
/*      tiway_dump_mod_table
/*
/*      The purpose of this routine is to dump the module description
/*      portion of the analog devices driver tables for the selected line.
/*      It will print out two lines for each module which is responding.
/*      The first line will give the general module header, the second line
/*      will give module type specific info.
*/
void	tiway_dump_mod_table (unsigned short line)

{
	TIPLC		*pmod;
	short           ix;


	if (!plcs [line])
	{
	    return;
	}
	if (line != line)
	{
	    return;			/* No Warnings */
	}
	for (ix = 0; ix < NUMTIPLCS; ix++)
	{
	    if ((pmod = plcs [line] [ix]) != 0)
            {
	        if (pmod->is_responding)
	        {
                    logMsg ("TI %s, Addr: 0x%0x is Online\n",
				ti_dev_type [pmod->dev_type_idx],
				ix);
		    logMsg ("IMEM SZ: 0x%0x VMEM SZ: 0x%0x CMEM SZ: 0x%0x\n",
				pmod->imem_sz,
				pmod->vmem_sz,
				pmod->cmem_sz);
		    logMsg ("LOCAL IO SZ: 0x%0x GLOBAL IO SZ: 0x%0x TOTAL USER SZ: 0x%0x\n",
				pmod->local_io_sz,
				pmod->global_io_sz,
				pmod->total_user_sz);
		    logMsg ("IOC Memory used: 0x%0x\n", sizeof (TIPLC));
		}
            }
	}
}
